"use strict";

const EncryptionTest = require("./EncryptionTest");

const objTestsConfig = {
	randomEncryptionRuns: 2000
};

const objEncryptionConfig = {
	active_index: "k1",
	keys: {
		"k0": {
			aes_key: "H9Tgfjmo91Jaj12kjbkPE89Rj89fasjq",
			salt: "SOMErondomSTUFF",
			created: "2018-12-31T23:51:55Z"
		},
		"k1": {
			aes_key: "RXTgfjmo91Jaj12kjbkPE89Rj89fceva",
			salt: "SOMErondomSTUFF",
			created: "2018-12-31T23:51:55Z"
		},
		"k2": {
			aes_key: "CEVAjmo91Jaj12kjbkPE89Rj89fceva1",
			salt: "SOMErondomSTUFF",
			created: "2018-12-31T23:51:55Z"
		},
		"default": {
			aes_key: "HUEHUEo91Jaj12kjbkPE89Rj89fceva1",
			salt: "SOMErondomSTUFF",
			created: "2018-12-31T23:51:55Z"
		}
	}
};

process.on(
	"unhandledRejection",
	(reason, promise) =>
	{
		console.log("[" + process.pid + "] Unhandled Rejection at: Promise", promise, "reason", reason.stack || reason.message);

		process.exit(1);
	}
);

process.on(
	"uncaughtException",
	(error) => {
		console.log("[" + process.pid + "] Unhandled exception.");
		console.error(error.stack || error.message);

		process.exit(1);
	}
);

(
	async() =>
	{
		const nStartTimeMs = (new Date()).getTime();

		let mainTest = new EncryptionTest(
			Object.assign(
				{},
				objEncryptionConfig,
				objTestsConfig
			)
		);
		await mainTest.runAllTests();

		const nFinishTimeMs = (new Date()).getTime();

		console.log("");
		console.log("\x1b[42m\x1b[30mAll tests done. We're still alive.\x1b[0m");
		console.log(`Done in ${nFinishTimeMs - nStartTimeMs} ms.`);

		process.exit(0);
	}
)();
