"use strict";

const BaseValidatorException = require("../Base/BaseValidatorException");

module.exports =
class DomainNameValidatorException extends BaseValidatorException
{
	constructor(strMessage, nCode = 0)
	{
		super(strMessage, nCode);
	}

	/**
	* http://data.iana.org/TLD/tlds-alpha-by-domain.txt
	*/
	static get TLD_NOT_ALLOCATED_BY_IANA() { return 1; }

	/**
	* RFC 1035
	* 2.3.4. Size limits
	* [...]
	* names           255 octets or less (including separators) (-1 byte for null terminator, -1 byte for root label => 253 ASCII readable maximum length)
	*
	* Additional references:
	* http://blogs.msdn.com/b/oldnewthing/archive/2012/04/12/10292868.aspx
	*/
	static get NAME_LONGER_THAN_253_CHARACTERS() { return 2 }
		
	/**
	* RFC 1035
	* 2.3.1. Preferred name syntax
	* [...]
	* The labels must follow the rules for ARPANET host names.  They must
	* start with a letter, end with a letter or digit, and have as interior
	* characters only letters, digits, and hyphen.  There are also some
	* restrictions on the length. Labels must be 63 characters or less.
	*/
	static get LABEL_LONGER_THAN_63_CHARACTERS() { return 3; }
	static get LABEL_MUST_HAVE_AT_LEAST_1_CHARACTER() { return 4; }
	static get LABEL_CONTAINS_DISALLOWED_CHARACTERS() { return 5; }
};
