"use strict";

const BaseValidatorException = require("../Base/BaseValidatorException");

module.exports =
class EmailAddressValidatorException extends BaseValidatorException
{
	constructor(strMessage, nCode = 0)
	{
		super(strMessage, nCode);
	}

	/**
	* The maximum length of 254 characters of an email address exceeded.
	* RFC 5321 4.5.3.1.3 corrected (overriden by) by
	* RFC 2821 4.5.3.1 Size limits and minimums
	* The maximum total length of a reverse-path or forward-path is 256
	* octets (including the punctuation and element separators).
	* => 254 ASCII readable maximum length (after removing punctuation and separators).
	*/
	static get MAXIMUM_254_CHARS_EXCEEDED() { return 1;}

	/**
	* The local part maximum length of 64 characters exceeded.
	* RFC 5321 4.5.3.1.1
	* The local part of an email address has a maximum length of 64 characters.
	*/
	static get LOCAL_PART_MAXIMUM_64_CHARS_EXCEEDED() { return 2;}
	static get LOCAL_PART_IS_EMPTY() { return 3;}

	/**
	* Local part contains disallowed characters.
	* RFC 5322, 3.2.3
	* Allowed characters in the local part are letters from A to Z, a to z, digits 0 to 9 and .!#$%&'*+/=?^`{|}~_-
	*/
	static get DISALLOWED_CHARACTERS() { return 4;}

	/**
	* The . (dot) character is not sorrounded by other allowed characters.
	* RFC 5322, 3.4.1 
	* [...] "." surrounded by atext characters [...]
	*/
	static get DOT_CHARACTER_APPEARS_TWICE_CONSECUTIVELY() { return 5;}
	static get LOCAL_PART_STARTS_OR_ENDS_WITH_DOT_CHARACTER() { return 6;}


	/**
	* Special characters and comments defined by RFC are not supported by this application.
	* RFC 5322, 3.2.3
	*/
	static get SPECIAL_CHARACTERS_AND_COMMENTS_ARE_UNSUPPORTED() { return 7;}

};
