"use strict";

const BaseValidatorException = require("../Base/BaseValidatorException");

module.exports =
class ISCSINameValidatorException extends BaseValidatorException
{
	constructor(strMessage, nCode = 0)
	{
		super(strMessage, nCode);
	}


	/*	RFC 3720  ; 3.2.6.3.  iSCSI Name Structure
	*
	*	 iqn.       - iSCSI Qualified name
	*	 eui.      - Remainder of the string is an IEEE EUI-64
	*	             identifier, in ASCII-encoded hexadecimal.
	*/
	static get INVALID_TYPE_DESIGNATOR() { return 1; }

	/*	RFC 3721 ; 1.1
	*/
	static get NO_IQN_MANDATORY_PART() { return 2; }

	/*	RFC 3721
	*	A date code specifying the year and month in which the
	*	organization registered the domain or sub-domain name used as the
	*	naming authority string.
	*	yyyy-mm , jan == 01, Dec == 12
	*/
	static get WRONG_DATE_FORMAT() { return 3; }

	/*
	* EUI name must only contain hexadecimal characters.
	*/
	static get NOT_HEXADECIMAL() { return 10; }

	/*
	* EUI name must be exactly 64 bits long.
	*/
	static get NOT_64_BIT() { return 11; }
};
