"use strict";

const BaseValidatorException = require("../Base/BaseValidatorException");

module.exports =
class URLValidatorException extends BaseValidatorException
{
	constructor(strMessage, nCode = 0)
	{
		super(strMessage, nCode);
	}

	/**
	* The 3986 RFC does not specify a maximum length, however this validation class does.
	*/
	static get URL_LONGER_THAN_MAXIMUM_ALLOWED() { return 1; }

	/**
	* RFC 3986.
	* Missing authority path-abempty.
	* Missing '//' right after URI scheme and ':'.
	*/
	static get MISSING_AUTHORITY_PATH_ABEMPTY() { return 2; }

	/**
	* It is only allowed to set an empty password or no password
	* as RFC 3986 deprecates (does not recommend) storing user passwords on URL.
	*/
	static get USER_INFO_PASSWORD_NOT_ALLOWED() { return 3; }

	/**
	* RFC 3986.
	* Empty, invalid characters or invalid hex encoding in username.
	*/
	static get USERNAME_INVALID() { return 4; }

	/**
	* The port is out of bounds. Value must be between 1 and 65535 inclusive.
	*/
	static get INVALID_PORT() { return 5; }

	/**
	* The path contains invalid characters.
	*/
	static get PATH_INVALID() { return 6; }

	/**
	* The query contains invalid characters.
	*/
	static get QUERY_INVALID() { return 7; }

	/**
	* The fragment contains invalid characters.
	*/
	static get FRAGMENT_INVALID() { return 8; }

	/**
	* Only http and https URL schemes are allowed.
	*/
	static get SCHEME_NOT_ALLOWED() { return 9; }

	/**
	* Missing scheme separator ':'.
	*/
	static get MISSING_SCHEME_SEPARATOR() { return 10; }
};
