"use strict";

module.exports =
class CacheResult
{
    /**
     * Constructor for CacheResult.
     * 
     * @param {Boolean} bCacheServed 
     * @param {*} mxValue 
     */
    constructor(bCacheServed = false, mxValue = null)
    {
        this._bCacheServed = Boolean(bCacheServed);
        this._mxValue = mxValue;
    }

    /**
     * Getter for value.
     * Checks if it hasValue().
     * 
     * @returns {*}
     */
    get value() 
    { 
        if(!this.hasValue())
        {
            throw new Error("Must check if this.hasValue() to be true before getting the value.");
        }

        return this._mxValue; 
    }

    /**
     * Setter for value.
     * Checks if it hasValue().
     * 
     * @param {*}
     */
    set value(value) 
    { 
        if(!this.hasValue())
        {
            throw new Error("Must check if this.hasValue() to be true before setting the value.");
        }

        this._mxValue = value; 
    }

    /**
     * Returns true if the cache was served.
     * 
     * @returns {Boolean}
     */
    hasValue() { return this._bCacheServed === true; }


    /**
     * Reusable cache miss CacheResult class instance.
     */
    static get staticMissSingleton() 
    {
        if(typeof this._staticMissSingleton == "undefined" || this._staticMissSingleton == null)
        {
            this._staticMissSingleton = new CacheResult(/*bCacheServed*/ false, /*mxValue*/ null);
        }

        return this._staticMissSingleton;
    }
}