"use strict";

const CacheFactory = require("./CacheFactory");

module.exports = 
class CacheSingleton
{
	constructor()
	{
		throw new Error("CacheSingleton is a static class and cannot be instantiated.");
	}

	/**
	 * @type {Object}
	 */
	static get config() { return this._objConfig; }

	/**
	 * @param {Object} objConfig Object with the format {engine, [ENGINE_NAME]: {...}}. engine {String} 
	 * 
	 */
	static set config({engine, ...objEngineConfigs}) 
	{
		if(!CacheFactory.supportedEngines.includes(engine))
		{
			throw new Error(`Unsuported type for cache engine ${JSON.stringify(engine)}. Supported ones are: ${JSON.stringify(CacheFactory.supportedEngines)}. Please configure another cache engine or add support for the required one.`);
		}

		if(!objEngineConfigs.hasOwnProperty(engine))
		{
			throw new Error(`Cache config does not contain config object for engine type "${engine}".`);
		}

		this._objConfig = {engine, ...objEngineConfigs};
	}

	/**
	 * Returns a CacheEngine singleton based on the static config set.
	 * @param {Boolean} bForceRecreate 
	 * 
	 * @returns {CacheEngine}
	 */
	static getInstance(bForceRecreate = false)
	{
		if(bForceRecreate === true || typeof this._cacheEngine == "undefined" || this._cacheEngine == null)
		{
			if(typeof this.config !== "object" || this.config === null)
			{
				throw new Error(`Cache static config not set on ${this.name}.`);
			}

			this._cacheEngine = CacheFactory.makeEngine(this.config.engine, this.config[this.config.engine]);
		}
	
		return this._cacheEngine;
	}

};
