"use strict";

const CacheInterface = require("../CacheInterface");

module.exports =
class CacheEngine
{
	constructor(objConfig)
	{
		this.config = objConfig;
		if(this.config.sandboxCacheKey)
		{
			this.sandboxCacheKey = this.config.sandboxCacheKey;
		}
	}

	get config() { return this._objConfig; }
	set config(value)
	{
		this.constructor._validateConfig(value);
		this._objConfig = value;
	}

	get sandboxCacheKey() { return this._strSandboxCacheKeyValue}
	set sandboxCacheKey(value) { this._strSandboxCacheKeyValue = String(value).trim(); }
   
	/**
	 * 
	 * @param {*} mxValue 
	 * 
	 * @returns {String}
	 */
	static serialize(mxValue)
	{
		return JSON.stringify(mxValue);
	}

	/**
	 * 
	 * @param {String} mxSerializedValue 
	 * 
	 * @returns {*}
	 */
	static deserialize(mxSerializedValue)
	{
		return JSON.parse(mxSerializedValue);
	}

	/**
	 * Sandboxes the cache key to not colide with other keys from another application.
	 * @param {String} strCacheKey 
	 */
	_sandboxTheCacheKey(strCacheKey)
	{
		if(
			typeof this.sandboxCacheKey == "string" 
			&& this.sandboxCacheKey != null
			&& this.sandboxCacheKey.trim().length > 0
		)
		{
			return `${this.sandboxCacheKey}_${strCacheKey}`;
		}
		
		return strCacheKey;
	}
	
	/**
	 * 
	 * @param {function} fnErrorHandler 
	 */
	setErrorHandler(fnErrorHandler)
	{
		throw new Error(`Method setErrorHandler must be implemented in class ${this.name}.`);
	}


	/**
	 * Contains a list of required properties for the config object.
	 * 
	 * @type {Array}
	 */
	static get _requiredConfigProperties() { return []; }
	
	/**
	 * Validates the config object and throws an Error if it is invalid.
	 * 
	 * @param {Object} objConfig
	 * 
	 * @throws {Error}
	 */
	static _validateConfig(objConfig)
	{
		let arrNotPresentProperties = [];

		for(const strRequiredKey of this._requiredConfigProperties)
		{
			if(!objConfig.hasOwnProperty(strRequiredKey))
			{
				arrNotPresentProperties.push(strRequiredKey);
			}
		}

		if(arrNotPresentProperties.length > 0)
		{
			throw new Error(`Invalid cache engine config. Required properties not present: ${JSON.stringify(arrNotPresentProperties)}.`);
		}
	}

};