# Node JS Cache Library

Cache support for Node JS applications. It supports multiple cache engines.

All functions from the API are asynchronous.

## Supported Cache Engines

* **Redis**
* **InMemory** `TODO`
* **File** `TODO`
* **Couchbase** `TODO`

# Requirements

* `node js` >= 8.0
* Used cache provider/engine. Example: `redis` - ``npm install redis``

# Configure

```js
//Redis config example
const CacheSingleton = require("./CacheSingleton");
CacheSingleton.config = {
    engine: "redis",
    redis: {
        //https://www.npmjs.com/package/redis#rediscreateclient
        host: "localhost",
        port: 6379,
        sandboxCacheKey: "SOME_CACHE_KEY"
    },
    //...
};
```

# Usage

```js
/* OPTIONAL
    A global wrapper can be used for easy acces to cache singleton
*/
global.cache = (bForceRecreate) => {
    return CacheSingleton.getInstance(bForceRecreate);
};
```

## Fetch value from a key 
```js
let result = await cache().fetch(strKey);

if(result.hasValue())
{
    console.log(result.value);
}
else
{
    console.log("Cache Miss");
}
```
The result is a ``CacheResult`` instance. 

## Store value for a key
```js
await cache().store(strKey, mxValue, nExpireSeconds);
```
or directly, without waiting for the response
```js
cache().store(strKey, mxValue, nExpireSeconds);
```

## Expire a key
```js
await cache().expire(strKey);
```

## Destroy all keys
```js
await cache().destroy();
```


## CacheResult

```js
class CacheResult
{
   /**
     * Constructor for CacheResult.
     * 
     * @param {Boolean} bCacheServed 
     * @param {*} mxValue 
     */
    constructor(bCacheServed = false, mxValue = null) {...}

     /**
     * Getter for value.
     * Checks if it hasValue().
     * 
     * @returns {*}
     */
    get value() {...}

    /**
     * Setter for value.
     * Checks if it hasValue().
     * 
     * @param {*}
     */
    set value(value) {...}
    
    /**
     * Returns true if the cache was served.
     * 
     * @returns {Boolean}
     */
    hasValue() {...}

    /**
     * Reusable cache miss CacheResult class instance.
     */
    static get staticMissSingleton() 
}
```