"use strict";
// https://nodemailer.com
const Nodemailer = require("nodemailer");

module.exports =
class Mail
{
	constructor(objConfig)
	{
		this.config = objConfig;
	}

	/**
	 * @type {Object}
	 */
	get config() { return this._objConfig; }

	/**
	 * @param {Object} value
	 */
	set config(value) { this._objConfig = value; } //TODO validate

	/**
	 * Sends a mail.
	 * https://nodemailer.com/message/
	 * 
	 * @param {Object} objMessage 
	 * 
	 * @returns {Object}
	 */
	async sendMail(objMessage)
	{
		let transporter = Nodemailer.createTransport({
			host: this.config.hostname,
			port: this.config.port,
			secure: Boolean(this.config.secure),
			auth: {
				user: this.config.username,
				pass: this.config.password
			},
			tls: this.config.tls,
			debug: this.config.debug
		});
	
		let bIsTransportVerified = await transporter.verify();
		if(!bIsTransportVerified)
		{
			throw new Error(`Cannot create transport to Mail server. Please check configuration and server connection.`);
		}
	
		objMessage.from = `${this.config.from.displayName} <${this.config.from.email}>`;
	
		let objSendMailResponse = transporter.sendMail(objMessage);

		//TODO parse the response and return a standard result.

		return objSendMailResponse;
	}

	/**
	 * @type {Object}
	 */
	static get configStatic() { return this._objStaticConfig; }

	/**
	 * @param {Object} value
	 */
	static set configStatic(value) { this._objStaticConfig = value; } //TODO validate

	/**
	 * Sends a mail using the singleton.
	 * https://nodemailer.com/message/
	 * 
	 * @param {Object} objMessage 
	 * 
	 * @returns {Object}
	 */
	static async sendMailStatic(objMessage)
	{
		return this.singleton.sendMail(objMessage);
	}

	/**
	 * Singleton for this class.
	 * 
	 * @type {Mail}
	 */
	static get singleton() 
	{
		if(typeof this._mailSingletonInstance === "undefined" || this._mailSingletonInstance === null)
		{
			if(typeof this.configStatic === "undefined" || this.configStatic === null)
			{
				throw new Error(`Before using the singleton for ${this.name}, please set configStatic property.`);
			}

			this._mailSingletonInstance = new Mail(this.configStatic);
		}

		return this._mailSingletonInstance;
	}
};
