
const os = require("os");
const Commands = require("./Commands");

class UtilityOSInfo
{
	constructor()
	{

	}

	/**
	 * @returns {Object} objResults
	 * 
	 * Run the batch of available commands and returns the parsed and filtered output grouped by command name
	 */
	static async runCommandBatch()
	{
		const strOSPlatform = os.platform();

		const arrCommandsWithErrors = [];
		const objResults = {};
		
		try
		{
			const arrPromises = [];

			if(strOSPlatform === "win32")
			{
				//pass
			}
			
			if(strOSPlatform === "linux")
			{
				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.Lshw.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.HBA.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.CCMcfg.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.NVMe.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.NVMeDiskDetection.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.DiskDetection.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.EFIbootmgr.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.NetworkInterfaces.execCommand(/*objConfig*/)));

				arrPromises.push(UtilityOSInfo._fnHandledPromise(Commands.Lscpu.execCommand(/*objConfig*/)));
			}
				
			// Must wait for all commands to finish to centralize the results
			await Promise.all(arrPromises);

			// Check if all commands produced output
			for(let strCommandName in Commands)
			{
				if(Commands[strCommandName].output)
				{
					objResults[strCommandName] = Commands[strCommandName].output;
				}
				else
				{
					arrCommandsWithErrors.push(strCommandName);
				}
			}

			if(arrCommandsWithErrors.length !== 0)
			{
				console.error(`Empty result on commands: [ ${arrCommandsWithErrors.join(", ")} ]`);
			}
		}
		catch(error)
		{
			console.error(error.stack);
		}

		return objResults;
	}

	/**
	 * @param {Array<string>} arrCommandsToRun
	 * @returns {Object} objResults
	 * 
	 * Run the selected commands and returns the parsed and filtered output grouped by command name
	 */
	static async runCommandSelection(arrCommandsToRun)
	{
		const strOSPlatform = os.platform();

		const arrCommandsWithErrors = [];
		const arrExecutedCommands = [];
		const objResults = {};

		try
		{
			const arrPromises = [];
	
			if(strOSPlatform === "win32")
			{
				//pass
			}
	
			if(strOSPlatform === "linux")
			{
				for(let strCommandName of arrCommandsToRun)
				{
					if(Commands[strCommandName])
					{
						arrPromises.push(UtilityOSInfo.fnHandledPromise(Commands[strCommandName].execCommand(/*objConfig*/)));
						arrExecutedCommands.push(strCommandName);
					}
				}
			}

			// Must wait for all commands to finish to centralize the results
			await Promise.all(arrPromises);

			// Check if all commands produced output
			for(let strCommandName in arrExecutedCommands)
			{
				if(Commands[strCommandName].output)
				{
					objResults[strCommandName] = Commands[strCommandName].output;
				}
				else
				{
					arrCommandsWithErrors.push(strCommandName);
				}
			}
				
			if(arrCommandsWithErrors.length !== 0)
			{
				console.error(`Empty result on commands: [ ${arrCommandsWithErrors.join(", ")} ]`);
			}
		}
		catch(error)
		{
			console.error(error.stack);
		}

		return objResults;
	}

	/**
	 * List of available shell commands
	 * 
	 * @returns {Object} Commands
	 */
	static get Commands()
	{
		return Commands;
	}

	static async _fnHandledPromise(promise) 
	{
		try
		{
			await promise;
		}
		catch(error)
		{
			console.error(error);
		}
	};
}


module.exports = UtilityOSInfo;
