// This prevents pollution of the global namespace
var SQL = (function () {
// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_WEB = typeof window === 'object';
// Three configurations we can be running in:
// 1) We could be the application main() thread running in the main JS UI thread. (ENVIRONMENT_IS_WORKER == false and ENVIRONMENT_IS_PTHREAD == false)
// 2) We could be the application main() thread proxied to worker. (with Emscripten -s PROXY_TO_WORKER=1) (ENVIRONMENT_IS_WORKER == true, ENVIRONMENT_IS_PTHREAD == false)
// 3) We could be an application pthread running in a worker. (ENVIRONMENT_IS_WORKER == true and ENVIRONMENT_IS_PTHREAD == true)
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function' && !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_WORKER;
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) {
    var ret = Module['read'](filename, true);
    if (!ret.buffer) {
      ret = new Uint8Array(ret);
    }
    assert(ret.buffer);
    return ret;
  };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  if (!Module['thisProgram']) {
    if (process['argv'].length > 1) {
      Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
    } else {
      Module['thisProgram'] = 'unknown-program';
    }
  }

  Module['arguments'] = process['argv'].slice(2);

  if (typeof module !== 'undefined') {
    module['exports'] = Module;
  }

  process['on']('uncaughtException', function(ex) {
    // suppress ExitStatus exceptions from showing an error
    if (!(ex instanceof ExitStatus)) {
      throw ex;
    }
  });

  Module['inspect'] = function () { return '[Emscripten Module object]'; };
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    if (typeof readbuffer === 'function') {
      return new Uint8Array(readbuffer(f));
    }
    var data = read(f, 'binary');
    assert(typeof data === 'object');
    return data;
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WORKER) {
    Module['load'] = importScripts;
  }

  if (typeof Module['setWindowTitle'] === 'undefined') {
    Module['setWindowTitle'] = function(title) { document.title = title };
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
if (!Module['thisProgram']) {
  Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  STACK_ALIGN: 16,
  prepVararg: function (ptr, type) {
    if (type === 'double' || type === 'i64') {
      // move so the load is aligned
      if (ptr & 7) {
        assert((ptr & 7) === 4);
        ptr += 4;
      }
    } else {
      assert((ptr & 3) === 0);
    }
    return ptr;
  },
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[sig]) {
      Runtime.funcWrappers[sig] = {};
    }
    var sigCache = Runtime.funcWrappers[sig];
    if (!sigCache[func]) {
      sigCache[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return sigCache[func];
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + size)|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + size)|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) { var success = enlargeMemory(); if (!success) { DYNAMICTOP = ret;  return 0; } }; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}



Module["Runtime"] = Runtime;



//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var JSfuncs = {
    // Helpers for cwrap -- it can't refer to Runtime directly because it might
    // be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
    // out what the minified function name is.
    'stackSave': function() {
      Runtime.stackSave()
    },
    'stackRestore': function() {
      Runtime.stackRestore()
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        // at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
        ret = Runtime.stackAlloc((str.length << 2) + 1);
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. 
  ccall = function ccallFunc(ident, returnType, argTypes, args, opts) {
    var func = getCFunc(ident);
    var cArgs = [];
    var stack = 0;
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) {
      if (opts && opts.async) {
        EmterpreterAsync.asyncFinalizers.push(function() {
          Runtime.stackRestore(stack);
        });
        return;
      }
      Runtime.stackRestore(stack);
    }
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }

  
  cwrap = function cwrap(ident, returnType, argTypes) {
    argTypes = argTypes || [];
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["ccall"] = ccall;
Module["cwrap"] = cwrap;

function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module["setValue"] = setValue;


function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module["getValue"] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module["ALLOC_NORMAL"] = ALLOC_NORMAL;
Module["ALLOC_STACK"] = ALLOC_STACK;
Module["ALLOC_STATIC"] = ALLOC_STATIC;
Module["ALLOC_DYNAMIC"] = ALLOC_DYNAMIC;
Module["ALLOC_NONE"] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module["allocate"] = allocate;

// Allocate memory during any stage of startup - static memory early on, dynamic memory later, malloc when ready
function getMemory(size) {
  if (!staticSealed) return Runtime.staticAlloc(size);
  if ((typeof _sbrk !== 'undefined' && !_sbrk.called) || !runtimeInitialized) return Runtime.dynamicAlloc(size);
  return _malloc(size);
}
Module["getMemory"] = getMemory;

function Pointer_stringify(ptr, /* optional */ length) {
  if (length === 0 || !ptr) return '';
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = 0;
  var t;
  var i = 0;
  while (1) {
    t = HEAPU8[(((ptr)+(i))>>0)];
    hasUtf |= t;
    if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (hasUtf < 128) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }
  return Module['UTF8ToString'](ptr);
}
Module["Pointer_stringify"] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated ASCII-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function AsciiToString(ptr) {
  var str = '';
  while (1) {
    var ch = HEAP8[((ptr++)>>0)];
    if (!ch) return str;
    str += String.fromCharCode(ch);
  }
}
Module["AsciiToString"] = AsciiToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in ASCII form. The copy will require at most str.length+1 bytes of space in the HEAP.

function stringToAscii(str, outPtr) {
  return writeAsciiToMemory(str, outPtr, false);
}
Module["stringToAscii"] = stringToAscii;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the given array that contains uint8 values, returns
// a copy of that string as a Javascript String object.

function UTF8ArrayToString(u8Array, idx) {
  var u0, u1, u2, u3, u4, u5;

  var str = '';
  while (1) {
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    u0 = u8Array[idx++];
    if (!u0) return str;
    if (!(u0 & 0x80)) { str += String.fromCharCode(u0); continue; }
    u1 = u8Array[idx++] & 63;
    if ((u0 & 0xE0) == 0xC0) { str += String.fromCharCode(((u0 & 31) << 6) | u1); continue; }
    u2 = u8Array[idx++] & 63;
    if ((u0 & 0xF0) == 0xE0) {
      u0 = ((u0 & 15) << 12) | (u1 << 6) | u2;
    } else {
      u3 = u8Array[idx++] & 63;
      if ((u0 & 0xF8) == 0xF0) {
        u0 = ((u0 & 7) << 18) | (u1 << 12) | (u2 << 6) | u3;
      } else {
        u4 = u8Array[idx++] & 63;
        if ((u0 & 0xFC) == 0xF8) {
          u0 = ((u0 & 3) << 24) | (u1 << 18) | (u2 << 12) | (u3 << 6) | u4;
        } else {
          u5 = u8Array[idx++] & 63;
          u0 = ((u0 & 1) << 30) | (u1 << 24) | (u2 << 18) | (u3 << 12) | (u4 << 6) | u5;
        }
      }
    }
    if (u0 < 0x10000) {
      str += String.fromCharCode(u0);
    } else {
      var ch = u0 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    }
  }
}
Module["UTF8ArrayToString"] = UTF8ArrayToString;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF8ToString(ptr) {
  return UTF8ArrayToString(HEAPU8,ptr);
}
Module["UTF8ToString"] = UTF8ToString;

// Copies the given Javascript String object 'str' to the given byte array at address 'outIdx',
// encoded in UTF8 form and null-terminated. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outU8Array: the array to copy to. Each index in this array is assumed to be one 8-byte element.
//   outIdx: The starting offset in the array to begin the copying.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=1, only the null terminator will be written and nothing else.
//                    maxBytesToWrite=0 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8Array(str, outU8Array, outIdx, maxBytesToWrite) {
  if (!(maxBytesToWrite > 0)) // Parameter maxBytesToWrite is not optional. Negative values, 0, null, undefined and false each don't write out any bytes.
    return 0;

  var startIdx = outIdx;
  var endIdx = outIdx + maxBytesToWrite - 1; // -1 for string null terminator.
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      if (outIdx >= endIdx) break;
      outU8Array[outIdx++] = u;
    } else if (u <= 0x7FF) {
      if (outIdx + 1 >= endIdx) break;
      outU8Array[outIdx++] = 0xC0 | (u >> 6);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0xFFFF) {
      if (outIdx + 2 >= endIdx) break;
      outU8Array[outIdx++] = 0xE0 | (u >> 12);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x1FFFFF) {
      if (outIdx + 3 >= endIdx) break;
      outU8Array[outIdx++] = 0xF0 | (u >> 18);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x3FFFFFF) {
      if (outIdx + 4 >= endIdx) break;
      outU8Array[outIdx++] = 0xF8 | (u >> 24);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else {
      if (outIdx + 5 >= endIdx) break;
      outU8Array[outIdx++] = 0xFC | (u >> 30);
      outU8Array[outIdx++] = 0x80 | ((u >> 24) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    }
  }
  // Null-terminate the pointer to the buffer.
  outU8Array[outIdx] = 0;
  return outIdx - startIdx;
}
Module["stringToUTF8Array"] = stringToUTF8Array;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF8 form. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8(str, outPtr, maxBytesToWrite) {
  return stringToUTF8Array(str, HEAPU8,outPtr, maxBytesToWrite);
}
Module["stringToUTF8"] = stringToUTF8;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF8 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF8(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      ++len;
    } else if (u <= 0x7FF) {
      len += 2;
    } else if (u <= 0xFFFF) {
      len += 3;
    } else if (u <= 0x1FFFFF) {
      len += 4;
    } else if (u <= 0x3FFFFFF) {
      len += 5;
    } else {
      len += 6;
    }
  }
  return len;
}
Module["lengthBytesUTF8"] = lengthBytesUTF8;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module["UTF16ToString"] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16 form. The copy will require at most str.length*4+2 bytes of space in the HEAP.
// Use the function lengthBytesUTF16() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=2, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<2 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF16(str, outPtr, maxBytesToWrite) {
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 2) return 0;
  maxBytesToWrite -= 2; // Null terminator.
  var startPtr = outPtr;
  var numCharsToWrite = (maxBytesToWrite < str.length*2) ? (maxBytesToWrite / 2) : str.length;
  for (var i = 0; i < numCharsToWrite; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[((outPtr)>>1)]=codeUnit;
    outPtr += 2;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[((outPtr)>>1)]=0;
  return outPtr - startPtr;
}
Module["stringToUTF16"] = stringToUTF16;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF16(str) {
  return str.length*2;
}
Module["lengthBytesUTF16"] = lengthBytesUTF16;

function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module["UTF32ToString"] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32 form. The copy will require at most str.length*4+4 bytes of space in the HEAP.
// Use the function lengthBytesUTF32() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=4, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<4 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF32(str, outPtr, maxBytesToWrite) {
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 4) return 0;
  var startPtr = outPtr;
  var endPtr = startPtr + maxBytesToWrite - 4;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++i);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[((outPtr)>>2)]=codeUnit;
    outPtr += 4;
    if (outPtr + 4 > endPtr) break;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[((outPtr)>>2)]=0;
  return outPtr - startPtr;
}
Module["stringToUTF32"] = stringToUTF32;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF32(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i);
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) ++i; // possibly a lead surrogate, so skip over the tail surrogate.
    len += 4;
  }

  return len;
}
Module["lengthBytesUTF32"] = lengthBytesUTF32;

function demangle(func) {
  var hasLibcxxabi = !!Module['___cxa_demangle'];
  if (hasLibcxxabi) {
    try {
      var buf = _malloc(func.length);
      writeStringToMemory(func.substr(1), buf);
      var status = _malloc(4);
      var ret = Module['___cxa_demangle'](buf, 0, 0, status);
      if (getValue(status, 'i32') === 0 && ret) {
        return Pointer_stringify(ret);
      }
      // otherwise, libcxxabi failed, we can try ours which may return a partial result
    } catch(e) {
      // failure when using libcxxabi, we can try ours which may return a partial result
    } finally {
      if (buf) _free(buf);
      if (status) _free(status);
      if (ret) _free(ret);
    }
  }
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  var parsed = func;
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    parsed = parse();
  } catch(e) {
    parsed += '?';
  }
  if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
    Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
  }
  return parsed;
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
  var err = new Error();
  if (!err.stack) {
    // IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
    // so try that as a special-case.
    try {
      throw new Error(0);
    } catch(e) {
      err = e;
    }
    if (!err.stack) {
      return '(no stack trace available)';
    }
  }
  return err.stack.toString();
}

function stackTrace() {
  return demangleAll(jsStackTrace());
}
Module["stackTrace"] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;

function alignMemoryPage(x) {
  if (x % 4096 > 0) {
    x += (4096 - (x % 4096));
  }
  return x;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk


function abortOnCannotGrowMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with  -s TOTAL_MEMORY=X  with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with  -s ALLOW_MEMORY_GROWTH=1  which adjusts the size at runtime but prevents some optimizations, (3) set Module.TOTAL_MEMORY to a higher value before the program runs, or if you want malloc to return NULL (0) instead of this abort, compile with  -s ABORTING_MALLOC=0 ');
}

function enlargeMemory() {
  abortOnCannotGrowMemory();
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer;



buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);


// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;
var runtimeExited = false;


function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
  runtimeExited = true;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module["addOnPreRun"] = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module["addOnInit"] = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module["addOnPreMain"] = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module["addOnExit"] = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module["addOnPostRun"] = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var len = length > 0 ? length : lengthBytesUTF8(stringy)+1;
  var u8array = new Array(len);
  var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
  if (dontAddNull) u8array.length = numBytesWritten;
  return u8array;
}
Module["intArrayFromString"] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module["intArrayToString"] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module["writeStringToMemory"] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[((buffer++)>>0)]=array[i];
  }
}
Module["writeArrayToMemory"] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; ++i) {
    HEAP8[((buffer++)>>0)]=str.charCodeAt(i);
  }
  // Null-terminate the pointer to the HEAP.
  if (!dontAddNull) HEAP8[((buffer)>>0)]=0;
}
Module["writeAsciiToMemory"] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}


// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


if (!Math['clz32']) Math['clz32'] = function(x) {
  x = x >>> 0;
  for (var i = 0; i < 32; i++) {
    if (x & (1 << (31 - i))) return i;
  }
  return 32;
};
Math.clz32 = Math['clz32']

var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;
var Math_clz32 = Math.clz32;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled

function getUniqueRunDependency(id) {
  return id;
}

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
}
Module["addRunDependency"] = addRunDependency;

function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module["removeRunDependency"] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data



var memoryInitializer = null;



// === Body ===

var ASM_CONSTS = [];




STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 48352;
  /* global initializers */  __ATINIT__.push();
  

/* memory initializer */ allocate([1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,254,255,255,127,0,0,0,0,128,0,0,0,244,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,250,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,233,69,0,0,245,69,0,0,1,70,0,0,25,70,0,0,36,70,0,0,56,70,0,0,70,70,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,78,67,0,0,52,3,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,83,67,0,0,56,3,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,93,67,0,0,60,3,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,106,67,0,0,52,3,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,135,0,0,0,136,0,0,0,137,0,0,0,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,202,154,59,0,202,154,59,208,7,0,0,232,3,0,0,244,1,0,0,168,97,0,0,127,0,0,0,10,0,0,0,80,195,0,0,231,3,0,0,232,3,0,0,0,0,0,0,3,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,134,0,0,0,138,0,0,0,139,0,0,0,140,0,0,0,141,0,0,0,142,0,0,0,135,0,0,0,131,0,0,0,132,0,0,0,133,0,0,0,132,0,0,0,130,0,0,0,143,0,0,0,134,0,0,0,133,0,0,0,146,71,0,0,136,0,0,0,0,0,0,0,151,71,0,0,133,0,0,0,0,0,0,0,157,71,0,0,144,0,0,0,0,0,0,0,164,71,0,0,145,0,0,0,0,0,0,0,171,71,0,0,146,0,0,0,0,0,0,0,176,71,0,0,147,0,0,0,0,0,0,0,182,71,0,0,148,0,0,0,0,0,0,0,192,71,0,0,137,0,0,0,0,0,0,0,198,71,0,0,138,0,0,0,0,0,0,0,203,71,0,0,0,0,0,0,0,0,0,0,209,71,0,0,0,0,0,0,0,0,0,0,217,71,0,0,139,0,0,0,0,0,0,0,223,71,0,0,0,0,0,0,0,0,0,0,230,71,0,0,0,0,0,0,0,0,0,0,239,71,0,0,149,0,0,0,0,0,0,0,246,71,0,0,0,0,0,0,0,0,0,0,0,72,0,0,134,0,0,0,0,0,0,0,7,72,0,0,150,0,0,0,0,0,0,0,21,72,0,0,151,0,0,0,0,0,0,0,27,72,0,0,135,0,0,0,0,0,0,0,33,72,0,0,140,0,0,0,0,0,0,0,40,72,0,0,130,0,0,0,0,0,0,0,48,72,0,0,130,0,0,0,0,0,0,0,53,72,0,0,152,0,0,0,0,0,0,0,60,72,0,0,0,0,0,0,0,0,0,0,67,72,0,0,131,0,0,0,0,0,0,0,79,72,0,0,141,0,0,0,0,0,0,0,88,72,0,0,153,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,72,0,0,254,72,0,0,7,73,0,0,36,71,0,0,0,0,0,0,3,0,0,0,136,0,0,0,131,0,0,0,132,0,0,0,134,0,0,0,138,0,0,0,139,0,0,0,154,0,0,0,155,0,0,0,156,0,0,0,135,0,0,0,131,0,0,0,132,0,0,0,0,0,0,0,132,0,0,0,130,0,0,0,143,0,0,0,134,0,0,0,133,0,0,0,1,0,0,0,137,0,0,0,131,0,0,0,132,0,0,0,134,0,0,0,138,0,0,0,139,0,0,0,157,0,0,0,158,0,0,0,159,0,0,0,135,0,0,0,131,0,0,0,132,0,0,0,0,0,0,0,132,0,0,0,130,0,0,0,143,0,0,0,134,0,0,0,133,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,138,0,0,0,135,0,0,0,136,0,0,0,142,0,0,0,160,0,0,0,161,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,161,75,0,0,171,75,0,0,182,75,0,0,194,75,0,0,205,75,0,0,216,75,0,0,227,75,0,0,232,75,0,0,237,75,0,0,248,75,0,0,4,76,0,0,11,76,0,0,19,76,0,0,27,76,0,0,32,76,0,0,38,76,0,0,45,76,0,0,59,76,0,0,72,76,0,0,78,76,0,0,82,76,0,0,93,76,0,0,98,76,0,0,106,76,0,0,112,76,0,0,119,76,0,0,124,76,0,0,133,76,0,0,138,76,0,0,147,76,0,0,152,76,0,0,157,76,0,0,163,76,0,0,171,76,0,0,181,76,0,0,189,76,0,0,199,76,0,0,208,76,0,0,215,76,0,0,225,76,0,0,238,76,0,0,243,76,0,0,255,76,0,0,7,77,0,0,12,77,0,0,17,77,0,0,20,77,0,0,26,77,0,0,33,77,0,0,42,77,0,0,53,77,0,0,59,77,0,0,70,77,0,0,80,77,0,0,90,77,0,0,99,77,0,0,109,77,0,0,123,77,0,0,137,77,0,0,148,77,0,0,161,77,0,0,172,77,0,0,178,77,0,0,190,77,0,0,197,77,0,0,204,77,0,0,211,77,0,0,218,77,0,0,229,77,0,0,238,77,0,0,244,77,0,0,254,77,0,0,1,78,0,0,5,78,0,0,14,78,0,0,23,78,0,0,30,78,0,0,37,78,0,0,45,78,0,0,48,78,0,0,51,78,0,0,54,78,0,0,57,78,0,0,60,78,0,0,63,78,0,0,73,78,0,0,80,78,0,0,86,78,0,0,96,78,0,0,107,78,0,0,111,78,0,0,120,78,0,0,129,78,0,0,136,78,0,0,146,78,0,0,153,78,0,0,160,78,0,0,167,78,0,0,175,78,0,0,186,78,0,0,200,78,0,0,211,78,0,0,218,78,0,0,226,78,0,0,232,78,0,0,240,78,0,0,245,78,0,0,0,79,0,0,5,79,0,0,12,79,0,0,25,79,0,0,35,79,0,0,45,79,0,0,50,79,0,0,59,79,0,0,65,79,0,0,71,79,0,0,77,79,0,0,83,79,0,0,91,79,0,0,97,79,0,0,109,79,0,0,121,79,0,0,133,79,0,0,145,79,0,0,158,79,0,0,168,79,0,0,178,79,0,0,190,79,0,0,202,79,0,0,212,79,0,0,223,79,0,0,234,79,0,0,242,79,0,0,247,79,0,0,253,79,0,0,7,80,0,0,16,80,0,0,23,80,0,0,29,80,0,0,41,80,0,0,51,80,0,0,64,80,0,0,77,80,0,0,86,80,0,0,94,80,0,0,103,80,0,0,114,80,0,0,121,80,0,0,131,80,0,0,138,80,0,0,146,80,0,0,155,80,0,0,161,80,0,0,169,80,0,0,175,80,0,0,183,80,0,0,193,80,0,0,202,80,0,0,207,80,0,0,218,80,0,0,223,80,0,0,52,81,0,0,61,81,0,0,68,81,0,0,74,81,0,0,229,87,0,0,236,87,0,0,244,87,0,0,248,87,0,0,1,88,0,0,8,88,0,0,57,100,0,0,62,100,0,0,69,100,0,0,72,100,0,0,75,100,0,0,78,100,0,0,81,100,0,0,84,100,0,0,92,100,0,0,101,100,0,0,107,100,0,0,112,100,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,17,132,0,0,1,0,0,0,19,132,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,197,111,0,0,1,112,0,0,248,111,0,0,124,68,0,0,81,120,0,0,87,120,0,0,92,120,0,0,97,120,0,0,53,127,0,0,64,0,0,0,59,127,0,0,66,0,0,0,64,127,0,0,65,0,0,0,69,127,0,0,67,0,0,0,75,115,0,0,3,0,0,0,71,115,0,0,3,0,0,0,69,136,0,0,0,0,0,0,8,0,0,0,91,133,0,0,1,1,0,0,0,0,0,0,84,136,0,0,2,0,0,0,0,0,16,0,100,136,0,0,3,0,0,0,0,0,0,0,250,132,0,0,4,1,0,0,0,0,0,0,123,133,0,0,5,0,0,0,0,0,0,0,113,136,0,0,6,0,0,0,0,0,0,0,133,136,0,0,2,0,0,0,0,0,0,16,149,136,0,0,2,0,0,0,16,0,0,0,170,136,0,0,7,0,0,0,0,0,0,0,185,136,0,0,8,0,0,0,0,0,0,0,201,136,0,0,2,0,0,0,128,0,0,0,215,136,0,0,0,2,0,0,15,0,0,0,228,136,0,0,10,1,0,0,0,0,0,0,242,136,0,0,11,1,0,0,0,0,0,0,5,137,0,0,2,0,0,0,0,0,0,1,24,137,0,0,2,0,0,0,0,1,0,0,170,135,0,0,12,0,0,0,0,0,0,0,47,137,0,0,13,1,0,0,0,0,0,0,65,137,0,0,14,1,0,0,0,0,0,0,82,137,0,0,2,0,0,0,0,0,8,0,95,137,0,0,0,2,0,0,0,0,0,0,110,137,0,0,2,0,0,0,4,0,0,0,128,137,0,0,2,0,0,0,8,0,0,0,138,137,0,0,2,0,0,0,0,32,0,0,163,137,0,0,15,1,0,0,0,0,0,0,182,137,0,0,16,1,0,0,0,0,0,0,193,137,0,0,17,1,0,0,0,0,0,0,204,137,0,0,16,1,0,0,1,0,0,0,207,134,0,0,18,1,0,0,0,0,0,0,59,133,0,0,19,1,0,0,0,0,0,0,72,133,0,0,20,0,0,0,0,0,0,0,216,137,0,0,2,0,0,0,0,128,0,0,46,133,0,0,22,0,0,0,0,0,0,0,235,137,0,0,23,1,0,0,0,0,0,0,135,133,0,0,24,0,0,0,0,0,0,0,250,137,0,0,23,1,0,0,0,0,0,0,5,133,0,0,25,0,0,0,0,0,0,0,5,138,0,0,2,0,0,0,0,0,0,2,16,138,0,0,18,1,0,0,0,0,0,0,28,138,0,0,2,0,0,0,0,64,0,0,45,138,0,0,2,0,0,0,0,0,4,0,64,138,0,0,2,0,0,0,0,0,2,0,90,138,0,0,0,0,0,0,1,0,0,0,15,133,0,0,26,0,0,0,0,0,0,0,105,138,0,0,2,0,0,0,64,0,0,0,124,138,0,0,27,0,0,0,0,0,0,0,45,136,0,0,28,0,0,0,0,0,0,0,138,138,0,0,29,1,0,0,0,0,0,0,202,133,0,0,30,1,0,0,0,0,0,0,144,138,0,0,31,1,0,0,0,0,0,0,145,133,0,0,32,0,0,0,0,0,0,0,156,133,0,0,33,0,0,0,0,0,0,0,61,136,0,0,34,0,0,0,0,0,0,0,155,138,0,0,0,0,0,0,6,0,0,0,18,136,0,0,35,0,0,0,0,0,0,0,168,138,0,0,36,1,0,0,0,0,0,0,183,138,0,0,2,0,0,0,0,8,1,0,11,134,0,0,15,134,0,0,20,134,0,0,25,134,0,0,33,134,0,0,44,134,0,0,48,113,0,0,38,121,0,0,54,134,0,0,60,134,0,0,76,134,0,0,11,134,0,0,15,134,0,0,82,134,0,0,87,134,0,0,92,134,0,0,104,134,0,0,15,134,0,0,108,134,0,0,115,134,0,0,122,134,0,0,130,134,0,0,132,134,0,0,44,134,0,0,104,134,0,0,15,134,0,0,140,134,0,0,104,134,0,0,15,134,0,0,152,134,0,0,104,134,0,0,48,113,0,0,107,100,0,0,155,134,0,0,158,134,0,0,168,134,0,0,53,127,0,0,48,113,0,0,201,106,0,0,192,134,0,0,199,134,0,0,112,135,0,0,1,0,0,0,117,135,0,0,1,0,0,0,123,135,0,0,2,0,0,0,132,135,0,0,3,0,0,0,141,135,0,0,2,0,0,0,149,135,0,0,3,0,0,0,157,135,0,0,0,0,0,0,164,135,0,0,0,0,0,0,0,0,0,0,0,0,0,0,239,135,0,0,244,135,0,0,248,135,0,0,3,0,1,0,0,0,0,0,0,0,0,0,130,0,0,0,0,0,0,0,49,141,0,0,0,0,0,0,0,0,0,0,95,142,0,0,0,0,2,0,102,142,0,0,0,0,4,0,0,0,0,0,0,0,0,0,115,142,0,0,1,0,0,0,118,142,0,0,2,0,0,0,121,142,0,0,6,0,0,0,1,88,0,0,128,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,131,0,0,0,0,0,0,0,185,142,0,0,0,0,0,0,0,0,0,0,216,84,0,0,100,143,0,0,113,143,0,0,0,0,0,0,126,143,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,132,0,0,0,0,0,0,0,175,143,0,0,0,0,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,133,0,0,0,0,0,0,0,185,143,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,134,0,0,0,0,0,0,0,195,143,0,0,0,0,0,0,0,0,0,0,1,0,1,8,1,0,0,0,0,0,0,0,135,0,0,0,0,0,0,0,32,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,1,0,0,0,0,0,0,0,135,0,0,0,0,0,0,0,32,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,2,0,0,0,0,0,0,0,135,0,0,0,0,0,0,0,38,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,2,0,0,0,0,0,0,0,135,0,0,0,0,0,0,0,38,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,3,0,0,0,0,0,0,0,135,0,0,0,0,0,0,0,44,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,3,0,0,0,0,0,0,0,135,0,0,0,0,0,0,0,44,149,0,0,0,0,0,0,0,0,0,0,255,255,33,8,0,0,0,0,0,0,0,0,136,0,0,0,0,0,0,0,121,129,0,0,0,0,0,0,0,0,0,0,0,0,33,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,121,129,0,0,0,0,0,0,0,0,0,0,1,0,33,16,0,0,0,0,0,0,0,0,137,0,0,0,131,0,0,0,121,129,0,0,0,0,0,0,0,0,0,0,255,255,33,8,1,0,0,0,0,0,0,0,136,0,0,0,0,0,0,0,125,129,0,0,0,0,0,0,0,0,0,0,0,0,33,8,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,125,129,0,0,0,0,0,0,0,0,0,0,1,0,33,16,1,0,0,0,0,0,0,0,137,0,0,0,131,0,0,0,125,129,0,0,0,0,0,0,0,0,0,0,1,0,129,8,0,0,0,0,0,0,0,0,138,0,0,0,0,0,0,0,49,149,0,0,0,0,0,0,0,0,0,0,1,0,65,8,0,0,0,0,0,0,0,0,139,0,0,0,0,0,0,0,56,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,140,0,0,0,0,0,0,0,63,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,141,0,0,0,0,0,0,0,69,149,0,0,0,0,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,141,0,0,0,0,0,0,0,69,149,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,142,0,0,0,0,0,0,0,76,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,143,0,0,0,0,0,0,0,83,149,0,0,0,0,0,0,0,0,0,0,255,255,1,8,0,0,0,0,0,0,0,0,144,0,0,0,0,0,0,0,91,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,145,0,0,0,0,0,0,0,96,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,146,0,0,0,0,0,0,0,100,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,146,0,0,0,0,0,0,0,100,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,147,0,0,0,0,0,0,0,106,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,148,0,0,0,0,0,0,0,112,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,118,149,0,0,0,0,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,118,149,0,0,0,0,0,0,0,0,0,0,255,255,1,10,0,0,0,0,0,0,0,0,149,0,0,0,0,0,0,0,118,149,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,150,0,0,0,0,0,0,0,127,149,0,0,0,0,0,0,0,0,0,0,2,0,1,10,0,0,0,0,0,0,0,0,149,0,0,0,0,0,0,0,131,149,0,0,0,0,0,0,0,0,0,0,1,0,1,12,0,0,0,0,0,0,0,0,149,0,0,0,0,0,0,0,138,149,0,0,0,0,0,0,0,0,0,0,2,0,1,12,0,0,0,0,0,0,0,0,149,0,0,0,0,0,0,0,147,149,0,0,0,0,0,0,0,0,0,0,1,0,1,12,0,0,0,0,0,0,0,0,149,0,0,0,0,0,0,0,158,149,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,151,0,0,0,0,0,0,0,165,149,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,152,0,0,0,0,0,0,0,172,149,0,0,0,0,0,0,0,0,0,0,2,0,33,8,0,0,0,0,0,0,0,0,153,0,0,0,0,0,0,0,183,149,0,0,0,0,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,149,0,0,0,0,0,0,0,190,149,0,0,0,0,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,154,0,0,0,0,0,0,0,205,149,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,155,0,0,0,0,0,0,0,222,149,0,0,0,0,0,0,0,0,0,0,1,0,1,32,0,0,0,0,0,0,0,0,156,0,0,0,0,0,0,0,233,149,0,0,0,0,0,0,0,0,0,0,1,0,1,32,0,0,0,0,0,0,0,0,157,0,0,0,0,0,0,0,3,150,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,158,0,0,0,0,0,0,0,28,150,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,159,0,0,0,0,0,0,0,34,150,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,160,0,0,0,0,0,0,0,52,150,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,161,0,0,0,0,0,0,0,60,150,0,0,0,0,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,162,0,0,0,0,0,0,0,74,150,0,0,0,0,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,163,0,0,0,0,0,0,0,82,150,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,164,0,0,0,132,0,0,0,91,150,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,164,0,0,0,133,0,0,0,95,150,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,164,0,0,0,134,0,0,0,101,150,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,165,0,0,0,135,0,0,0,105,150,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,165,0,0,0,135,0,0,0,105,150,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,166,0,0,0,136,0,0,0,111,150,0,0,0,0,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,166,0,0,0,136,0,0,0,111,150,0,0,0,0,0,0,0,0,0,0,2,0,13,8,116,68,0,0,0,0,0,0,167,0,0,0,0,0,0,0,59,127,0,0,0,0,0,0,0,0,0,0,2,0,5,8,120,68,0,0,0,0,0,0,167,0,0,0,0,0,0,0,64,127,0,0,0,0,0,0,0,0,0,0,3,0,5,8,120,68,0,0,0,0,0,0,167,0,0,0,0,0,0,0,64,127,0,0,0,0,0,0,0,0,0,0,16,130,0,0,255,255,1,32,0,0,0,0,0,0,0,0,168,0,0,0,0,0,0,0,180,150,0,0,0,0,0,0,0,0,0,0,255,255,1,32,0,0,0,0,0,0,0,0,169,0,0,0,0,0,0,0,190,150,0,0,0,0,0,0,0,0,0,0,255,255,1,32,0,0,0,0,0,0,0,0,170,0,0,0,0,0,0,0,195,150,0,0,0,0,0,0,0,0,0,0,255,255,1,32,0,0,0,0,0,0,0,0,171,0,0,0,0,0,0,0,200,150,0,0,0,0,0,0,0,0,0,0,255,255,1,32,0,0,0,0,0,0,0,0,172,0,0,0,0,0,0,0,209,150,0,0,0,0,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,173,0,0,0,0,0,0,0,218,150,0,0,0,0,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,174,0,0,0,0,0,0,0,231,150,0,0,0,0,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,175,0,0,0,0,0,0,0,249,150,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,176,0,0,0,0,0,0,0,224,151,0,0,0,0,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,177,0,0,0,0,0,0,0,244,151,0,0,0,0,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,178,0,0,0,0,0,0,0,10,152,0,0,0,0,0,0,0,0,0,0,139,0,0,0,137,0,0,0,162,0,0,0,140,0,0,0,141,0,0,0,142,0,0,0,138,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,143,0,0,0,139,0,0,0,143,0,0,0,130,0,0,0,144,0,0,0,144,0,0,0,179,0,0,0,130,0,0,0,131,0,0,0,140,0,0,0,141,0,0,0,195,152,0,0,208,152,0,0,0,0,0,0,244,152,0,0,13,153,0,0,44,153,0,0,63,153,0,0,85,73,0,0,88,153,0,0,125,153,0,0,137,153,0,0,152,153,0,0,185,153,0,0,203,153,0,0,228,153,0,0,1,154,0,0,18,154,0,0,240,82,0,0,148,67,0,0,41,154,0,0,59,154,0,0,77,154,0,0,116,154,0,0,147,154,0,0,168,154,0,0,200,154,0,0,234,154,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,131,0,0,0,132,0,0,0,163,0,0,0,145,0,0,0,146,0,0,0,164,0,0,0,147,0,0,0,137,0,0,0,148,0,0,0,149,0,0,0,145,0,0,0,165,0,0,0,134,0,0,0,150,0,0,0,151,0,0,0,152,0,0,0,153,0,0,0,138,0,0,0,166,0,0,0,167,0,0,0,168,0,0,0,169,0,0,0,0,0,0,0,146,0,0,0,154,0,0,0,135,0,0,0,155,0,0,0,133,0,0,0,0,0,0,0,48,0,0,0,7,232,0,0,6,108,1,0,47,236,1,0,7,172,2,0,1,208,2,0,3,216,2,0,1,236,2,0,1,252,2,0,1,92,3,0,1,220,3,0,4,8,11,0,14,72,11,0,7,148,11,0,1,180,11,0,129,188,11,0,1,212,13,0,1,248,13,0,2,16,14,0,1,28,14,0,1,216,15,0,8,8,18,0,6,104,21,0,2,36,22,0,1,60,22,0,55,68,22,0,2,204,23,0,5,0,24,0,22,24,24,0,2,120,24,0,21,44,25,0,4,168,25,0,1,192,25,0,1,80,27,0,15,88,27,0,7,156,27,0,2,244,27,0,14,0,28,0,1,60,28,0,1,68,28,0,27,192,28,0,11,152,30,0,9,172,31,0,4,216,31,0,4,88,32,0,9,108,32,0,3,148,32,0,5,164,32,0,15,192,32,0,3,100,33,0,1,120,33,0,27,144,35,0,4,0,36,0,3,232,36,0,18,248,36,0,7,68,37,0,4,136,37,0,1,192,37,0,3,4,38,0,1,240,38,0,7,248,38,0,2,28,39,0,3,44,39,0,1,92,39,0,2,136,39,0,2,200,39,0,2,232,39,0,3,4,40,0,1,240,40,0,5,248,40,0,2,28,41,0,3,44,41,0,1,68,41,0,2,192,41,0,1,212,41,0,3,4,42,0,1,240,42,0,8,248,42,0,3,28,43,0,3,44,43,0,2,136,43,0,2,192,43,0,3,4,44,0,1,240,44,0,7,248,44,0,2,28,45,0,3,44,45,0,2,88,45,0,2,136,45,0,1,192,45,0,1,8,46,0,5,248,46,0,3,24,47,0,4,40,47,0,1,92,47,0,8,204,47,0,3,4,48,0,7,248,48,0,3,24,49,0,4,40,49,0,2,84,49,0,2,136,49,0,1,252,49,0,2,8,50,0,1,240,50,0,7,248,50,0,3,24,51,0,4,40,51,0,2,84,51,0,2,136,51,0,2,8,52,0,7,248,52,0,3,24,53,0,4,40,53,0,1,92,53,0,2,136,53,0,1,228,53,0,2,8,54,0,1,40,55,0,6,60,55,0,1,88,55,0,8,96,55,0,3,200,55,0,1,196,56,0,7,208,56,0,1,252,56,0,9,28,57,0,2,104,57,0,1,196,58,0,6,208,58,0,2,236,58,0,6,32,59,0,31,4,60,0,12,208,60,0,23,196,61,0,11,52,62,0,36,100,62,0,15,248,62,0,13,56,63,0,20,172,64,0,6,40,65,0,4,88,65,0,3,120,65,0,3,136,65,0,7,156,65,0,4,196,65,0,12,8,66,0,1,60,66,0,6,104,66,0,1,236,67,0,12,116,77,0,10,64,78,0,1,0,80,0,2,180,89,0,1,0,90,0,2,108,90,0,3,172,91,0,3,72,92,0,5,200,92,0,2,72,93,0,2,200,93,0,35,208,94,0,4,96,95,0,1,116,95,0,15,0,96,0,1,164,98,0,12,128,100,0,12,192,100,0,1,0,101,0,2,16,101,0,17,192,102,0,2,32,103,0,34,120,103,0,5,92,104,0,2,120,104,0,10,84,105,0,29,128,105,0,1,252,105,0,7,128,106,0,6,160,106,0,5,0,108,0,17,208,108,0,35,104,109,0,3,0,110,0,13,132,110,0,14,152,111,0,4,240,111,0,20,144,112,0,5,236,112,0,2,248,113,0,8,0,115,0,25,64,115,0,1,180,115,0,3,200,115,0,39,0,119,0,4,240,119,0,1,244,126,0,3,252,126,0,3,52,127,0,3,116,127,0,3,180,127,0,2,244,127,0,101,0,128,0,6,168,129,0,5,232,129,0,5,40,130,0,26,128,130,0,33,64,131,0,2,0,132,0,4,12,132,0,2,32,132,0,1,80,132,0,3,88,132,0,6,120,132,0,1,148,132,0,1,156,132,0,1,164,132,0,1,184,132,0,2,232,132,0,5,0,133,0,4,40,133,0,1,60,133,0,100,66,134,0,39,0,144,0,11,0,145,0,78,112,146,0,0,2,148,0,117,4,156,0,185,83,158,0,10,64,173,0,6,148,179,0,3,188,179,0,4,228,179,0,2,248,179,0,1,192,181,0,1,252,181,0,79,128,183,0,12,192,184,0,26,0,186,0,89,108,186,0,214,0,188,0,12,192,191,0,5,0,192,0,25,32,192,0,7,168,192,0,2,216,192,0,3,244,192,0,4,100,194,0,1,128,194,0,1,236,195,0,2,64,198,0,10,88,198,0,36,0,199,0,31,0,200,0,30,168,200,0,1,64,201,0,32,128,201,0,39,40,202,0,63,0,203,0,0,1,204,0,64,0,55,1,55,64,146,2,2,248,147,2,3,52,152,2,16,188,153,2,1,124,154,2,8,192,155,2,23,0,156,2,2,128,156,2,2,36,158,2,1,8,160,2,1,24,160,2,1,44,160,2,9,140,160,2,4,216,160,2,4,208,161,2,2,0,162,2,17,208,162,2,2,56,163,2,18,128,163,2,3,224,163,2,10,152,164,2,13,28,165,2,1,124,165,2,4,0,166,2,27,204,166,2,2,120,167,2,14,164,168,2,1,12,169,2,2,48,169,2,4,112,169,2,3,220,169,2,1,236,169,2,1,192,170,2,3,200,170,2,2,220,170,2,2,248,170,2,1,4,171,2,2,120,171,2,7,172,171,2,2,212,171,2,11,140,175,2,1,0,96,3,2,252,109,3,2,252,111,3,1,252,127,3,1,120,236,3,1,164,236,3,16,200,238,3,2,248,244,3,2,240,247,3,26,0,248,3,7,128,248,3,35,192,248,3,19,80,249,3,4,160,249,3,1,252,251,3,15,4,252,3,7,104,252,3,6,236,252,3,11,108,253,3,7,128,255,3,7,160,255,3,5,228,255,3,3,0,4,4,9,220,4,4,17,228,5,4,12,64,6,4,46,64,7,4,1,124,14,4,1,64,15,4,1,92,33,4,1,124,36,4,1,252,36,4,3,4,40,4,2,20,40,4,4,48,40,4,3,224,40,4,1,252,40,4,9,64,41,4,1,252,41,4,7,228,44,4,3,0,64,4,22,224,64,4,3,0,66,4,18,192,66,4,3,0,68,4,14,156,68,4,4,0,69,4,3,0,70,4,14,204,70,4,4,20,71,4,13,172,90,4,4,192,145,4,46,68,189,5,4,60,190,5,246,0,64,7,39,0,68,7,181,164,68,7,70,0,72,7,87,0,76,7,1,4,91,7,1,108,91,7,1,236,91,7,1,84,92,7,1,212,92,7,1,60,93,7,1,188,93,7,1,36,94,7,1,164,94,7,1,12,95,7,2,192,187,7,44,0,192,7,100,192,192,7,15,128,194,7,14,196,194,7,15,4,195,7,15,68,195,7,31,64,196,7,60,192,196,7,43,192,197,7,29,152,199,7,43,64,200,7,9,0,201,7,2,64,201,7,33,0,204,7,6,192,204,7,70,220,204,7,20,0,206,7,37,128,206,7,5,24,207,7,17,128,207,7,63,0,208,7,1,0,209,7,182,8,209,7,4,228,211,7,62,0,212,7,4,0,213,7,24,64,213,7,70,236,215,7,11,20,217,7,70,0,218,7,116,0,220,7,1,4,0,56,96,128,0,56,240,0,4,56,255,255,255,255,255,255,0,252,1,0,0,248,1,0,0,248,0,0,0,0,134,0,0,0,134,0,0,0,170,0,0,0,156,0,0,0,156,0,0,0,171,0,0,0,157,0,0,0,139,0,0,0,158,0,0,0,159,0,0,0,147,0,0,0,172,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,86,157,0,0,130,157,0,0,198,157,0,0,226,157,0,0,255,157,0,0,26,158,0,0,54,158,0,0,79,158,0,0,103,158,0,0,168,158,0,0,227,158,0,0,39,159,0,0,87,159,0,0,199,159,0,0,78,160,0,0,130,160,0,0,196,160,0,0,239,160,0,0,42,161,0,0,81,161,0,0,125,161,0,0,166,161,0,0,213,161,0,0,255,161,0,0,124,68,0,0,124,68,0,0,37,162,0,0,92,162,0,0,153,162,0,0,2,163,0,0,105,163,0,0,160,163,0,0,218,163,0,0,69,164,0,0,148,164,0,0,213,164,0,0,17,165,0,0,65,165,0,0,171,165,0,0,244,165,0,0,0,0,0,0,148,0,0,0,160,0,0,0,136,0,0,0,161,0,0,0,135,0,0,0,0,0,0,0,0,0,0,0,149,0,0,0,162,0,0,0,137,0,0,0,163,0,0,0,136,0,0,0,0,0,0,0,30,156,0,0,9,0,0,0,38,168,0,0,6,0,0,0,45,168,0,0,8,0,0,0,54,168,0,0,10,0,0,0,101,100,0,0,5,0,0,0,65,168,0,0,7,0,0,0,73,168,0,0,10,0,0,0,84,168,0,0,10,0,0,0,138,173,0,0,2,0,4,0,141,173,0,0,3,1,3,0,145,173,0,0,3,1,2,0,149,173,0,0,4,0,1,0,14,156,0,0,180,0,0,0,22,156,0,0,181,0,0,0,40,156,0,0,182,0,0,0,30,156,0,0,183,0,0,0,0,0,0,0,137,0,0,0,137,0,0,0,173,0,0,0,164,0,0,0,164,0,0,0,174,0,0,0,165,0,0,0,140,0,0,0,166,0,0,0,167,0,0,0,150,0,0,0,175,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,196,36,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,151,0,0,0,152,0,0,0,216,184,0,0,0,4,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,2,2,16,16,2,16,33,0,32,0,30,0,28,0,26,0,0,0,2,0,3,0,5,0,6,0,7,0,8,0,9,0,0,0,2,0,2,0,8,0,9,0,14,0,16,0,20,0,23,0,25,0,25,0,29,0,33,0,36,0,41,0,46,0,48,0,53,0,54,0,59,0,62,0,65,0,67,0,69,0,78,0,81,0,86,0,91,0,95,0,96,0,101,0,105,0,109,0,117,0,122,0,128,0,136,0,142,0,152,0,159,0,162,0,162,0,165,0,167,0,167,0,171,0,176,0,179,0,184,0,184,0,188,0,192,0,199,0,204,0,209,0,212,0,218,0,221,0,225,0,234,0,240,0,240,0,240,0,243,0,246,0,250,0,251,0,255,0,5,1,9,1,16,1,22,1,34,1,40,1,49,1,51,1,57,1,62,1,64,1,71,1,76,1,81,1,87,1,93,1,98,1,102,1,105,1,111,1,115,1,122,1,124,1,131,1,133,1,135,1,144,1,148,1,154,1], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);
/* memory initializer */ allocate([160,1,168,1,173,1,173,1,189,1,196,1,203,1,204,1,211,1,215,1,219,1,223,1,227,1,230,1,232,1,234,1,240,1,244,1,252,1,1,2,9,2,12,2,17,2,22,2,28,2,32,2,37,2,5,0,33,4,75,5,46,4,180,4,180,4,180,4,90,0,60,0,237,255,58,0,58,0,186,0,180,4,180,4,180,4,180,4,180,4,180,4,180,4,67,0,67,0,182,0,80,1,218,0,38,2,135,0,7,1,84,1,161,1,238,1,59,2,110,2,187,2,8,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,59,3,110,3,59,3,161,3,212,3,212,3,132,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,234,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,180,4,185,255,209,255,209,255,209,255,209,255,209,255,250,255,88,0,190,255,218,0,218,0,162,1,239,1,23,2,23,2,33,0,43,0,10,0,226,255,167,255,167,255,167,255,11,0,169,1,169,1,12,1,199,1,93,2,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,172,2,138,0,10,0,43,0,125,0,125,0,125,0,125,0,125,0,125,0,167,255,167,255,167,255,228,0,85,1,85,1,207,0,20,1,44,1,24,1,96,1,98,1,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,51,2,51,2,51,2,218,0,218,0,179,1,218,0,218,0,218,0,67,2,218,0,218,0,73,2,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,218,0,69,2,0,3,199,2,199,2,199,2,192,2,215,0,41,4,244,2,178,1,197,2,197,2,200,2,178,1,200,2,22,2,90,3,129,2,185,3,197,2,168,255,185,3,185,3,99,3,233,1,191,1,176,4,94,4,94,4,179,4,179,4,94,4,205,4,160,4,102,4,218,4,218,4,218,4,218,4,94,4,226,4,102,4,205,4,160,4,160,4,102,4,94,4,226,4,115,4,213,4,94,4,94,4,226,4,253,4,94,4,226,4,94,4,226,4,253,4,181,4,181,4,181,4,235,4,253,4,181,4,186,4,181,4,235,4,181,4,181,4,171,4,194,4,171,4,194,4,171,4,194,4,171,4,194,4,94,4,94,4,174,4,253,4,230,4,230,4,253,4,199,4,207,4,206,4,212,4,102,4,66,5,68,5,83,5,83,5,93,5,93,5,93,5,93,5,167,255,167,255,167,255,167,255,167,255,167,255,221,1,35,2,130,1,50,3,238,2,253,2,188,2,238,3,219,2,243,3,247,3,248,3,249,3,180,3,68,3,167,3,191,2,255,3,31,4,40,4,53,4,87,3,150,3,63,4,61,4,99,2,112,5,114,5,97,5,236,4,105,5,53,5,108,5,102,5,103,5,7,5,254,4,17,5,9,5,110,5,8,5,118,5,134,5,13,5,6,5,60,5,61,5,32,5,116,5,109,5,24,5,146,5,143,5,127,5,43,5,11,5,98,5,128,5,99,5,94,5,113,5,49,5,135,5,138,5,141,5,50,5,56,5,142,5,115,5,144,5,145,5,140,5,147,5,117,5,148,5,149,5,119,5,125,5,150,5,151,5,152,5,63,5,154,5,157,5,155,5,156,5,59,5,160,5,161,5,158,5,159,5,163,5,64,5,164,5,162,5,165,5,166,5,164,5,169,5,170,5,171,5,173,5,174,5,178,5,176,5,180,5,179,5,172,5,181,5,182,5,184,5,185,5,181,5,187,5,186,5,188,5,189,5,191,5,82,5,92,5,95,5,96,5,192,5,204,5,219,5,214,3,20,5,20,5,20,5,190,4,190,4,190,4,25,5,20,5,85,4,114,4,114,4,250,4,25,5,25,5,25,5,25,5,25,5,25,5,188,4,25,5,25,5,25,5,20,5,25,5,89,4,120,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,249,4,251,4,128,4,127,4,230,4,101,4,125,4,118,4,122,4,191,4,184,4,185,4,183,4,187,4,192,4,25,5,121,4,153,4,168,4,152,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,162,4,167,4,174,4,166,4,163,4,155,4,154,4,156,4,157,4,25,5,25,5,240,3,50,4,25,5,25,5,158,4,25,5,252,3,159,4,171,4,170,4,169,4,247,3,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,214,3,20,5,25,5,25,5,20,5,20,5,20,5,20,5,20,5,20,5,12,5,89,4,79,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,0,5,254,4,25,5,203,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,85,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,220,3,25,5,223,4,85,4,85,4,85,4,87,4,65,4,77,4,222,3,124,4,103,4,103,4,235,4,124,4,235,4,21,4,44,4,18,4,114,4,103,4,186,4,114,4,114,4,86,4,77,4,25,5,5,5,94,4,94,4,253,4,253,4,94,4,133,4,54,4,124,4,61,4,61,4,61,4,61,4,94,4,237,3,124,4,133,4,54,4,54,4,124,4,94,4,237,3,229,4,227,4,94,4,94,4,237,3,196,4,94,4,237,3,94,4,237,3,196,4,52,4,52,4,52,4,36,4,196,4,52,4,21,4,52,4,36,4,52,4,52,4,107,4,102,4,107,4,102,4,107,4,102,4,107,4,102,4,94,4,94,4,25,5,196,4,200,4,200,4,196,4,119,4,108,4,117,4,115,4,124,4,243,3,39,4,230,3,230,3,219,3,219,3,219,3,219,3,17,5,17,5,12,5,23,4,23,4,6,4,25,5,25,5,25,5,25,5,25,5,25,5,254,3,25,5,205,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,140,4,25,5,215,3,7,5,25,5,25,5,4,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,233,4,25,5,25,5,25,5,25,5,25,5,25,5,226,4,225,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,68,4,25,5,25,5,25,5,72,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,116,4,25,5,109,4,25,5,189,4,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,25,5,22,5,25,5,25,5,25,5,21,5,25,5,25,5,25,5,25,5,25,5,142,4,25,5,141,4,145,4,25,5,228,3,25,5,55,1,26,5,145,0,139,2,2,0,192,0,140,2,82,1,12,3,92,0,92,0,92,0,92,0,85,0,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,88,0,88,0,88,0,87,0,79,1,71,1,88,3,88,3,92,0,92,0,92,0,92,0,185,2,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,76,0,39,3,74,0,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,79,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,12,3,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,100,1,40,3,8,3,189,2,177,2,177,2,86,0,83,0,166,0,1,1,41,3,203,2,174,1,86,0,83,0,166,0,68,1,185,2,88,3,88,3,201,0,158,0,20,1,131,1,15,1,130,1,188,0,177,2,177,2,60,3,86,0,83,0,166,0,13,1,65,3,49,0,123,0,87,0,79,1,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,239,0,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,251,2,77,1,76,1,216,0,152,1,138,1,69,0,231,0,137,1,178,2,179,2,140,1,142,3,251,0,98,1,250,0,32,1,59,1,174,1,140,3,174,1,141,3,89,0,89,0,88,0,88,0,88,0,87,0,79,1,135,1,88,3,88,3,178,2,179,2,183,0,95,0,123,0,128,1,125,1,124,1,65,3,31,0,65,3,49,0,144,3,144,3,239,2,240,2,123,1,123,0,55,1,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,114,0,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,174,1,152,1,143,1,179,1,145,2,88,3,88,3,90,1,57,0,232,0,60,3,109,0,192,2,110,1,177,2,177,2,107,1,57,3,248,2,97,0,237,2,240,2,65,3,49,0,196,2,196,2,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,167,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,114,0,22,0,105,1,176,2,58,0,152,1,134,1,251,0,93,1,240,0,213,0,250,2,177,2,177,2,79,3,173,2,115,0,105,1,231,0,137,1,177,2,177,2,140,1,183,0,177,2,177,2,128,1,125,1,124,1,105,1,88,3,88,3,178,2,179,2,160,0,159,0,223,0,123,1,226,2,25,0,38,3,195,2,73,3,143,0,177,2,177,2,67,3,136,1,83,1,254,2,254,2,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,146,3,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,72,3,72,3,72,3,10,1,1,1,178,2,179,2,10,3,194,2,86,0,83,0,166,0,219,0,178,2,179,2,225,2,1,0,178,2,179,2,177,2,177,2,177,2,177,2,174,1,86,0,83,0,166,0,249,0,176,2,169,3,88,3,88,3,171,1,187,2,188,2,60,3,42,1,178,2,179,2,221,0,174,2,115,0,123,0,176,3,27,3,65,3,48,0,86,1,49,1,202,3,79,3,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,114,0,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,172,3,73,3,167,2,201,2,173,1,67,3,174,1,251,0,98,1,250,0,99,1,32,1,178,2,179,2,178,2,179,2,29,1,173,3,84,1,203,3,31,1,210,0,23,0,174,0,25,3,64,3,174,1,97,1,65,3,10,0,88,3,88,3,24,0,174,3,151,0,241,2,72,3,72,3,72,3,26,3,200,3,10,5,65,1,142,1,10,5,100,1,96,1,242,2,65,3,49,0,167,3,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,120,1,114,0,139,3,193,2,174,1,139,3,72,1,122,3,114,0,65,3,10,0,198,3,174,1,89,3,89,3,64,1,189,0,163,0,64,3,165,0,174,1,138,3,88,1,67,1,138,3,136,3,65,3,10,0,197,3,50,1,88,3,88,3,187,0,163,1,65,3,10,0,220,0,101,3,104,3,64,3,222,0,147,1,65,3,49,0,195,4,25,3,68,0,169,3,150,1,245,0,66,0,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,93,3,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,148,1,213,0,250,2,66,3,89,1,114,0,172,3,134,3,112,1,215,2,5,0,60,1,192,0,140,1,4,3,12,3,13,1,230,0,242,0,3,3,244,0,141,1,164,0,173,3,129,1,123,0,91,1,55,0,99,1,73,1,88,3,88,3,216,2,77,1,76,1,176,2,200,3,11,5,212,2,174,3,11,5,157,1,214,0,65,3,9,0,106,1,30,1,187,3,115,0,206,2,55,1,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,144,3,144,3,20,5,20,5,246,2,88,3,88,3,69,1,198,3,12,3,65,3,35,0,235,2,208,2,78,1,187,2,188,2,209,3,140,2,82,1,243,0,233,2,152,3,152,3,113,1,187,0,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,114,0,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,174,1,186,3,174,1,112,0,54,1,174,1,181,2,61,1,186,2,144,1,174,1,25,3,103,1,174,1,249,3,174,1,192,0,174,1,145,1,12,3,174,1,104,1,65,3,36,0,65,3,12,0,174,1,65,3,27,0,60,1,88,3,88,3,65,3,37,0,20,0,65,3,38,0,65,3,39,0,65,3,28,0,72,0,65,3,29,0,151,2,152,2,153,2,8,1,65,3,40,0,234,0,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,55,1,174,1,186,2,174,1,149,3,147,0,174,1,165,0,148,3,19,1,65,3,41,0,174,1,12,3,174,1,21,0,174,1,3,1,174,1,6,1,18,1,174,1,111,1,65,3,42,0,65,3,11,0,174,1,65,3,43,0,235,0,88,3,88,3,25,3,65,3,99,0,65,3,44,0,65,3,45,0,65,3,32,0,75,0,65,3,46,0,49,1,199,3,1,1,1,1,65,3,47,0,55,1,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,174,1,186,0,185,0,184,0,238,0,88,3,88,3,138,2,2,0,40,4,65,3,33,0,227,2,217,0,218,0,1,1,203,3,1,1,170,1,61,1,1,1,6,3,65,3,117,0,1,1,55,1,93,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,174,1,62,1,124,0,212,0,163,0,88,3,88,3,175,3,132,3,130,3,65,3,118,0,247,2,214,2,213,2,1,1,243,2,33,1,33,1,221,2,222,2,193,3,65,3,119,0,170,2,55,1,93,0,82,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,174,1,204,2,246,0,66,1,75,1,88,3,88,3,0,1,114,0,101,1,65,3,53,0,40,3,145,3,145,3,164,3,156,0,160,1,164,1,168,1,162,3,41,3,65,3,34,0,108,1,55,1,253,0,94,0,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,174,1,114,0,114,0,114,0,192,3,88,3,88,3,51,1,2,1,62,3,65,3,100,0,191,0,252,0,121,1,11,1,68,0,197,0,68,0,5,1,204,2,1,3,65,3,50,0,71,0,143,3,143,3,7,1,84,0,100,3,103,3,92,3,92,3,91,0,91,0,92,0,92,0,92,0,92,0,174,1,90,0,90,0,90,0,90,0,89,0,89,0,88,0,88,0,88,0,87,0,79,1,80,0,169,1,34,3,3,0,190,4,191,0,174,1,9,1,80,1,80,1,65,3,101,0,229,2,80,0,169,1,129,3,3,0,211,2,210,2,172,1,209,2,80,1,80,1,174,1,125,3,14,1,174,1,197,0,65,3,102,0,174,1,32,3,172,1,174,1,183,2,174,1,75,3,111,0,158,1,174,1,16,3,153,1,174,1,63,3,174,1,65,3,98,0,123,0,65,3,116,0,79,3,158,1,65,3,49,0,11,3,65,3,113,0,65,3,106,0,226,0,123,0,65,3,105,0,79,3,65,3,103,0,65,3,104,0,23,3,155,1,77,0,78,0,34,1,156,1,174,1,35,1,114,0,79,0,176,1,175,1,133,1,174,1,67,3,77,0,78,0,129,3,71,3,152,1,154,1,174,1,79,0,176,1,175,1,116,1,191,2,67,3,65,3,52,0,174,1,80,0,169,1,174,1,3,0,65,3,54,0,4,3,75,3,80,1,80,1,172,2,3,3,65,3,51,0,72,3,72,3,72,3,74,3,19,0,172,1,160,2,65,3,26,0,159,2,65,3,30,0,161,2,72,3,72,3,72,3,74,3,19,0,207,0,149,2,22,1,48,1,148,0,24,1,158,1,26,1,248,0,102,1,54,3,126,1,6,0,92,1,161,0,17,1,80,0,169,1,79,3,3,0,166,3,127,3,208,2,126,3,80,1,80,1,40,1,157,0,159,1,241,0,28,1,162,2,190,3,194,0,185,3,172,1,183,3,180,3,77,0,78,0,9,3,63,1,56,0,59,0,135,0,79,0,176,1,175,1,121,0,66,0,67,3,146,0,128,0,94,1,158,1,51,3,130,0,95,1,131,0,132,0,133,0,119,1,173,0,107,0,138,0,149,0,79,3,109,1,178,0,62,0,70,0,169,1,168,3,3,0,59,3,121,3,115,1,255,0,80,1,80,1,24,3,72,3,72,3,72,3,74,3,19,0,77,0,78,0,147,3,172,1,208,0,179,0,144,0,79,0,176,1,175,1,117,1,4,1,67,3,180,0,70,1,163,2,181,0,52,1,232,2,132,1,231,2,219,2,158,1,206,2,230,2,218,2,200,2,146,1,53,1,199,2,16,1,20,3,65,0,198,2,79,3,197,2,21,1,193,0,21,3,19,3,23,1,108,3,73,0,72,3,72,3,72,3,74,3,19,0,18,3,25,1,162,1,27,1,166,1,227,0,77,0,78,0,74,1,228,0,229,0,96,0,255,2,79,0,176,1,175,1,151,1,67,0,67,3,215,0,36,1,37,1,149,1,38,1,47,1,46,1,45,1,204,0,43,1,39,1,202,0,164,2,169,2,7,0,177,1,157,2,203,0,205,0,206,0,125,0,110,0,57,1,178,1,155,2,154,2,146,2,168,0,224,0,237,0,72,3,72,3,72,3,74,3,19,0,120,0,144,2,81,1,236,0,155,0,167,0,85,1,233,0,58,1,108,0,137,3,135,3,58,3,127,0,126,0,244,2,170,0,129,0,172,0,247,0,160,3,134,0,136,0,171,0,60,0,61,0,123,0,169,0,137,0,165,3,175,0,176,0,159,3,8,0,13,0,177,0,254,0,150,3,139,0,191,0,156,3,140,0,114,1,166,2,150,0,118,1,182,0,18,1,12,1,141,0,122,0,63,0,14,0,122,1,15,0,127,1,64,0,225,0,78,3,77,3,106,3,16,0,4,0,217,2,253,2,2,3,162,0,139,1,209,0,211,0,142,0,33,3,110,3,28,3,56,1,71,0,68,0,107,3,105,3,171,3,190,0,161,1,170,3,17,0,195,0,196,0,152,0,18,0,207,3,199,0,208,3,153,0,198,0,154,0,165,1,109,3,76,3,184,2,81,0,200,0,41,1,87,1,251,3,250,3,44,1,141,2,113,255,186,3,86,0,21,0,206,255,23,0,79,0,134,0,226,0,136,255,129,255,146,0,161,0,35,1,93,1,110,1,55,1,126,1,118,1,231,0,108,1,111,1,140,1,142,1,236,0,61,1,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,204,1,247,1,55,2,57,2,60,2,65,2,68,2,70,2,72,2,75,2,81,2,119,2,132,2,134,2,137,2,143,2,145,2,147,2,149,2,152,2,158,2,196,2,208,2,247,2,3,3,42,3,54,3,93,3,105,3,144,3,162,3,179,3,182,3,189,3,191,3,195,3,198,3,200,3,230,3,237,3,245,3,254,3,1,4,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,153,255,218,1,212,0,15,0,242,1,222,0,255,1,153,255,97,0,45,2,153,255,153,255,153,255,153,255,176,255,9,0,59,0,19,0,38,1,38,1,203,255,194,255,178,2,179,2,223,2,225,2,228,2,232,2,133,0,54,1,148,0,74,1,160,0,124,1,18,3,20,3,145,1,40,1,21,3,221,2,85,0,210,2,214,255,68,1,252,1,16,3,60,3,61,3,62,3,166,2,201,2,151,1,69,0,150,0,194,0,188,0,33,1,45,1,147,1,205,1,229,1,56,2,105,2,161,2,212,2,11,3,24,3,56,3,63,3,69,3,74,3,78,3,80,3,113,3,124,3,132,3,163,3,168,3,190,1,142,3,143,3,176,3,181,3,133,3,187,3,199,3,210,3,155,3,224,3,225,3,188,3,228,3,231,3,242,3,33,1,250,3,9,4,19,4,22,4,25,4,32,4,166,3,205,3,229,3,232,3,234,3,133,3,244,3,251,3,36,4,246,3,236,3,252,3,207,3,0,4,208,3,16,4,11,4,23,4,21,4,253,3,239,3,27,4,29,4,7,4,10,4,59,4,2,4,58,4,60,4,240,3,241,3,65,4,12,4,44,4,35,4,45,4,47,4,48,4,49,4,81,4,87,4,52,4,26,4,56,4,66,4,55,4,91,4,93,4,34,4,24,4,104,4,114,4,116,4,100,4,121,4,124,4,125,4,127,4,107,4,111,4,113,4,117,4,106,4,118,4,119,4,120,4,123,4,110,4,126,4,128,4,82,4,88,4,89,4,92,4,90,4,101,4,99,4,103,4,147,4,151,4,95,4,140,4,96,4,97,4,142,4,122,4,131,4,133,4,136,4,143,4,187,4,190,4,200,4,201,4,208,4,209,4,210,4,211,4,108,4,129,4,109,4,177,4,184,4,204,4,12,0,14,0,24,0,31,0,59,0,15,39,65,0,14,26,181,0,64,1,192,0,14,23,216,0,14,7,0,1,1,48,50,1,1,6,57,1,1,16,74,1,1,46,120,1,116,1,121,1,1,6,127,1,104,1,129,1,50,1,130,1,1,4,134,1,44,1,135,1,0,1,137,1,42,2,139,1,0,1,142,1,32,1,143,1,38,1,144,1,40,1,145,1,0,1,147,1,42,1,148,1,46,1,150,1,52,1,151,1,48,1,152,1,0,1,156,1,52,1,157,1,54,1,159,1,56,1,160,1,1,6,166,1,60,1,167,1,0,1,169,1,60,1,172,1,0,1,174,1,60,1,175,1,0,1,177,1,58,2,179,1,1,4,183,1,62,1,184,1,0,1,188,1,0,1,196,1,2,1,197,1,0,1,199,1,2,1,200,1,0,1,202,1,2,1,203,1,1,18,222,1,1,18,241,1,2,1,242,1,1,4,246,1,122,1,247,1,134,1,248,1,1,40,32,2,110,1,34,2,1,18,58,2,70,1,59,2,0,1,61,2,108,1,62,2,68,1,65,2,0,1,67,2,106,1,68,2,28,1,69,2,30,1,70,2,1,10,69,3,36,1,112,3,1,4,118,3,0,1,134,3,18,1,136,3,16,3,140,3,26,1,142,3,24,2,145,3,14,17,163,3,14,9,194,3,0,1,207,3,4,1,208,3,140,1,209,3,142,1,213,3,146,1,214,3,144,1,216,3,1,24,240,3,136,1,241,3,138,1,244,3,130,1,245,3,128,1,247,3,0,1,249,3,152,1,250,3,0,1,253,3,110,3,0,4,34,16,16,4,14,32,96,4,1,34,138,4,1,54,192,4,6,1,193,4,1,14,208,4,1,88,49,5,22,38,160,16,66,38,199,16,66,1,205,16,66,1,0,30,1,150,155,30,132,1,158,30,96,1,160,30,1,96,8,31,150,8,24,31,150,6,40,31,150,8,56,31,150,8,72,31,150,6,89,31,151,8,104,31,150,8,136,31,150,8,152,31,150,8,168,31,150,8,184,31,150,2,186,31,126,2,188,31,148,1,190,31,100,1,200,31,124,4,204,31,148,1,216,31,150,2,218,31,120,2,232,31,150,2,234,31,118,2,236,31,152,1,248,31,112,2,250,31,114,2,252,31,148,1,38,33,98,1,42,33,92,1,43,33,94,1,50,33,12,1,96,33,8,16,131,33,0,1,182,36,10,26,0,44,22,47,96,44,0,1,98,44,88,1,99,44,102,1,100,44,90,1,103,44,1,6,109,44,84,1,110,44,86,1,111,44,80,1,112,44,82,1,114,44,0,1,117,44,0,1,126,44,78,2,128,44,1,100,235,44,1,4,242,44,0,1,64,166,1,46,128,166,1,24,34,167,1,14,50,167,1,62,121,167,1,4,125,167,76,1,126,167,1,10,139,167,0,1,141,167,74,1,144,167,1,4,160,167,1,10,170,167,72,1,33,255,14,26,1,0,2,0,8,0,15,0,16,0,26,0,28,0,32,0,37,0,38,0,40,0,48,0,63,0,64,0,69,0,71,0,79,0,80,0,116,0,202,0,203,0,205,0,206,0,207,0,209,0,210,0,211,0,213,0,214,0,217,0,218,0,219,0,7,3,96,28,40,42,43,42,188,90,216,90,252,117,193,213,225,213,226,213,228,213,3,214,9,214,25,214,65,223,186,223,65,226,163,226,251,227,26,241,244,254,61,255,93,255,126,255,128,255,130,255,135,255,144,255,156,255,159,255,170,255,182,255,192,255,196,255,198,255,200,255,202,255,208,255,226,255,231,255,234,255,241,255,247,255,248,255,249,255,0,0,5,7,56,7,67,7,99,7,136,7,148,7,203,7,232,7,248,7,12,8,62,8,120,8,158,8,216,8,238,8,40,9,79,9,168,9,184,9,212,9,36,10,108,10,172,10,222,10,26,11,78,11,138,11,168,11,184,11,212,11,8,13,128,13,112,14,128,14,144,14,160,14,56,15,72,15,88,15,128,15,168,15,200,15,10,16,42,16,74,16,106,16,138,16,170,16,200,16,216,16,248,16,56,17,72,17,120,17,152,17,4,24,54,24,120,24,136,24,216,24,29,25,105,25,129,25,8,240,28,240,94,240,152,240,202,240,248,240,8,241,30,241,88,241,104,241,140,241,184,241,218,241,252,241,46,242,170,242,202,242,248,242,10,243,94,243,156,243,234,243,14,244,72,244,90,244,120,244,140,244,176,244,184,244,192,244,200,244,10,245,204,245,74,246,106,246,42,247,158,247,51,46,49,49,46,48,0,83,81,76,73,84,69,95,0,0,1,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,117,110,105,120,0,117,110,105,120,45,110,111,110,101,0,117,110,105,120,45,100,111,116,102,105,108,101,0,117,110,105,120,45,101,120,99,108,0,4,5,3,5,1,5,1,5,2,5,2,5,1,5,1,5,4,5,3,5,1,5,1,5,2,5,2,5,1,5,1,5,115,116,114,105,110,103,32,111,114,32,98,108,111,98,32,116,111,111,32,98,105,103,0,99,97,110,110,111,116,32,111,112,101,110,32,118,105,114,116,117,97,108,32,116,97,98,108,101,58,32,37,115,0,99,97,110,110,111,116,32,111,112,101,110,32,116,97,98,108,101,32,119,105,116,104,111,117,116,32,114,111,119,105,100,58,32,37,115,0,99,97,110,110,111,116,32,111,112,101,110,32,118,105,101,119,58,32,37,115,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,58,32,34,37,115,34,0,102,111,114,101,105,103,110,32,107,101,121,0,105,110,100,101,120,101,100,0,99,97,110,110,111,116,32,111,112,101,110,32,37,115,32,99,111,108,117,109,110,32,102,111,114,32,119,114,105,116,105,110,103,0,148,0,0,0,54,0,0,0,28,1,1,0,70,0,7,1,47,0,0,1,33,1,0,0,13,0,2,0,61,0,0,0,21,0,0,0,37,115,0,42,63,91,0,37,95,0,1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,64,0,0,0,0,0,0,0,0,0,0,0,12,12,12,12,12,12,12,12,12,12,0,0,0,0,0,0,0,10,10,10,10,10,10,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,0,0,0,64,0,42,42,42,42,42,42,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,0,0,0,0,0,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,1,4,3,2,5,116,114,105,103,103,101,114,0,116,101,109,112,0,50,48,49,54,45,48,50,45,49,53,32,49,55,58,50,57,58,50,52,32,51,100,56,54,50,102,50,48,55,101,51,97,100,99,48,48,102,55,56,48,54,54,55,57,57,97,99,53,97,56,99,50,56,50,52,51,48,97,53,102,0,117,110,107,110,111,119,110,32,100,97,116,97,98,97,115,101,58,32,37,115,0,73,78,84,69,71,69,82,0,68,73,83,65,66,76,69,95,76,70,83,0,69,78,65,66,76,69,95,70,84,83,51,0,69,78,65,66,76,69,95,70,84,83,51,95,80,65,82,69,78,84,72,69,83,73,83,0,73,78,84,54,52,95,84,89,80,69,0,79,77,73,84,95,76,79,65,68,95,69,88,84,69,78,83,73,79,78,0,83,89,83,84,69,77,95,77,65,76,76,79,67,0,84,72,82,69,65,68,83,65,70,69,61,48,0,37,0,100,10,1,1,0,0,115,0,4,6,0,0,103,0,1,4,30,0,122,0,4,7,0,0,113,0,4,10,0,0,81,0,4,11,0,0,119,0,4,15,0,0,99,0,0,9,0,0,111,8,0,1,0,2,117,10,0,1,0,0,120,16,0,1,16,1,88,16,0,1,0,4,102,0,1,2,0,0,101,0,1,3,30,0,69,0,1,3,14,0,71,0,1,4,14,0,105,10,1,1,0,0,110,0,0,5,0,0,37,0,0,8,0,0,112,16,0,14,0,1,84,0,2,12,0,0,83,0,2,13,0,0,114,10,3,16,0,0,116,104,115,116,110,100,114,100,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,48,49,50,51,52,53,54,55,56,57,97,98,99,100,101,102,0,45,120,48,0,88,48,0,78,97,78,0,73,110,102,0,78,85,76,76,0,40,78,85,76,76,41,0,46,0,99,97,110,110,111,116,32,102,115,116,97,116,32,100,98,32,102,105,108,101,32,37,115,0,102,105,108,101,32,117,110,108,105,110,107,101,100,32,119,104,105,108,101,32,111,112,101,110,58,32,37,115,0,109,117,108,116,105,112,108,101,32,108,105,110,107,115,32,116,111,32,102,105,108,101,58,32,37,115,0,102,105,108,101,32,114,101,110,97,109,101,100,32,119,104,105,108,101,32,111,112,101,110,58,32,37,115,0,111,112,101,110,0,99,108,111,115,101,0,97,99,99,101,115,115,0,103,101,116,99,119,100,0,115,116,97,116,0,102,115,116,97,116,0,102,116,114,117,110,99,97,116,101,0,102,99,110,116,108,0,114,101,97,100,0,112,114,101,97,100,0,112,114,101,97,100,54,52,0,119,114,105,116,101,0,112,119,114,105,116,101,0,112,119,114,105,116,101,54,52,0,102,99,104,109,111,100,0,102,97,108,108,111,99,97,116,101,0,117,110,108,105,110,107,0,111,112,101,110,68,105,114,101,99,116,111,114,121,0,109,107,100,105,114,0,114,109,100,105,114,0,102,99,104,111,119,110,0,103,101,116,101,117,105,100,0,109,109,97,112,0,109,117,110,109,97,112,0,109,114,101,109,97,112,0,103,101,116,112,97,103,101,115,105,122,101,0,114,101,97,100,108,105,110,107,0,108,115,116,97,116,0,97,116,116,101,109,112,116,32,116,111,32,111,112,101,110,32,34,37,115,34,32,97,115,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,37,100,0,47,100,101,118,47,110,117,108,108,0,111,115,95,117,110,105,120,46,99,58,37,100,58,32,40,37,100,41,32,37,115,40,37,115,41,32,45,32,37,115,0,99,97,110,110,111,116,32,111,112,101,110,32,102,105,108,101,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,102,117,108,108,95,102,115,121,110,99,0,37,115,47,101,116,105,108,113,115,95,37,108,108,120,37,99,0,47,118,97,114,47,116,109,112,0,47,117,115,114,47,116,109,112,0,47,116,109,112,0,83,81,76,73,84,69,95,84,77,80,68,73,82,0,84,77,80,68,73,82,0,37,115,45,115,104,109,0,114,101,97,100,111,110,108,121,95,115,104,109,0,112,115,111,119,0,37,115,46,108,111,99,107,0,102,115,121,110,99,0,47,100,101,118,47,117,114,97,110,100,111,109,0,111,117,116,32,111,102,32,109,101,109,111,114,121,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,97,32,116,101,109,112,111,114,97,114,121,32,100,97,116,97,98,97,115,101,32,102,105,108,101,32,102,111,114,32,115,116,111,114,105,110,103,32,116,101,109,112,111,114,97,114,121,32,116,97,98,108,101,115,0,58,109,101,109,111,114,121,58,0,45,106,111,117,114,110,97,108,0,0,45,119,97,108,0,0,110,111,108,111,99,107,0,105,109,109,117,116,97,98,108,101,0,114,101,99,111,118,101,114,101,100,32,37,100,32,102,114,97,109,101,115,32,102,114,111,109,32,87,65,76,32,102,105,108,101,32,37,115,0,100,97,116,97,98,97,115,101,32,99,111,114,114,117,112,116,105,111,110,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,99,97,110,110,111,116,32,108,105,109,105,116,32,87,65,76,32,115,105,122,101,58,32,37,115,0,217,213,5,249,32,161,99,215,0,114,101,99,111,118,101,114,101,100,32,37,100,32,112,97,103,101,115,32,102,114,111,109,32,37,115,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,83,81,76,105,116,101,32,102,111,114,109,97,116,32,51,0,64,32,32,0,65,80,73,32,99,97,108,108,101,100,32,119,105,116,104,32,102,105,110,97,108,105,122,101,100,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,0,70,79,82,69,73,71,78,32,75,69,89,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,37,115,45,109,106,88,88,88,88,88,88,57,88,88,122,0,77,74,32,100,101,108,101,116,101,58,32,37,115,0,77,74,32,99,111,108,108,105,100,101,58,32,37,115,0,45,109,106,37,48,54,88,57,37,48,50,88,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,0,1,2,3,0,1,0,0,37,108,108,100,0,37,33,46,49,53,103,0,57,50,50,51,51,55,50,48,51,54,56,53,52,55,55,53,56,48,0,65,80,73,32,99,97,108,108,101,100,32,119,105,116,104,32,78,85,76,76,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,0,37,46,50,120,0,83,97,118,101,112,111,105,110,116,0,65,117,116,111,67,111,109,109,105,116,0,84,114,97,110,115,97,99,116,105,111,110,0,83,111,114,116,101,114,78,101,120,116,0,80,114,101,118,73,102,79,112,101,110,0,78,101,120,116,73,102,79,112,101,110,0,80,114,101,118,0,78,101,120,116,0,67,104,101,99,107,112,111,105,110,116,0,74,111,117,114,110,97,108,77,111,100,101,0,86,97,99,117,117,109,0,86,70,105,108,116,101,114,0,86,85,112,100,97,116,101,0,71,111,116,111,0,71,111,115,117,98,0,82,101,116,117,114,110,0,73,110,105,116,67,111,114,111,117,116,105,110,101,0,69,110,100,67,111,114,111,117,116,105,110,101,0,89,105,101,108,100,0,78,111,116,0,72,97,108,116,73,102,78,117,108,108,0,72,97,108,116,0,73,110,116,101,103,101,114,0,73,110,116,54,52,0,83,116,114,105,110,103,0,78,117,108,108,0,83,111,102,116,78,117,108,108,0,66,108,111,98,0,86,97,114,105,97,98,108,101,0,77,111,118,101,0,67,111,112,121,0,83,67,111,112,121,0,73,110,116,67,111,112,121,0,82,101,115,117,108,116,82,111,119,0,67,111,108,108,83,101,113,0,70,117,110,99,116,105,111,110,48,0,70,117,110,99,116,105,111,110,0,65,100,100,73,109,109,0,77,117,115,116,66,101,73,110,116,0,82,101,97,108,65,102,102,105,110,105,116,121,0,67,97,115,116,0,80,101,114,109,117,116,97,116,105,111,110,0,67,111,109,112,97,114,101,0,74,117,109,112,0,79,110,99,101,0,73,102,0,73,102,78,111,116,0,67,111,108,117,109,110,0,65,102,102,105,110,105,116,121,0,77,97,107,101,82,101,99,111,114,100,0,67,111,117,110,116,0,82,101,97,100,67,111,111,107,105,101,0,83,101,116,67,111,111,107,105,101,0,82,101,111,112,101,110,73,100,120,0,79,112,101,110,82,101,97,100,0,79,112,101,110,87,114,105,116,101,0,79,112,101,110,65,117,116,111,105,110,100,101,120,0,79,112,101,110,69,112,104,101,109,101,114,97,108,0,83,111,114,116,101,114,79,112,101,110,0,83,101,113,117,101,110,99,101,84,101,115,116,0,79,112,101,110,80,115,101,117,100,111,0,67,108,111,115,101,0,67,111,108,117,109,110,115,85,115,101,100,0,83,101,101,107,76,84,0,83,101,101,107,76,69,0,83,101,101,107,71,69,0,83,101,101,107,71,84,0,78,111,67,111,110,102,108,105,99,116,0,78,111,116,70,111,117,110,100,0,70,111,117,110,100,0,78,111,116,69,120,105,115,116,115,0,79,114,0,65,110,100,0,83,101,113,117,101,110,99,101,0,78,101,119,82,111,119,105,100,0,73,110,115,101,114,116,0,73,115,78,117,108,108,0,78,111,116,78,117,108,108,0,78,101,0,69,113,0,71,116,0,76,101,0,76,116,0,71,101,0,73,110,115,101,114,116,73,110,116,0,66,105,116,65,110,100,0,66,105,116,79,114,0,83,104,105,102,116,76,101,102,116,0,83,104,105,102,116,82,105,103,104,116,0,65,100,100,0,83,117,98,116,114,97,99,116,0,77,117,108,116,105,112,108,121,0,68,105,118,105,100,101,0,82,101,109,97,105,110,100,101,114,0,67,111,110,99,97,116,0,68,101,108,101,116,101,0,66,105,116,78,111,116,0,83,116,114,105,110,103,56,0,82,101,115,101,116,67,111,117,110,116,0,83,111,114,116,101,114,67,111,109,112,97,114,101,0,83,111,114,116,101,114,68,97,116,97,0,82,111,119,75,101,121,0,82,111,119,68,97,116,97,0,82,111,119,105,100,0,78,117,108,108,82,111,119,0,76,97,115,116,0,83,111,114,116,101,114,83,111,114,116,0,83,111,114,116,0,82,101,119,105,110,100,0,83,111,114,116,101,114,73,110,115,101,114,116,0,73,100,120,73,110,115,101,114,116,0,73,100,120,68,101,108,101,116,101,0,83,101,101,107,0,73,100,120,82,111,119,105,100,0,73,100,120,76,69,0,73,100,120,71,84,0,73,100,120,76,84,0,73,100,120,71,69,0,68,101,115,116,114,111,121,0,67,108,101,97,114,0,82,101,115,101,116,83,111,114,116,101,114,0,67,114,101,97,116,101,73,110,100,101,120,0,67,114,101,97,116,101,84,97,98,108,101,0,80,97,114,115,101,83,99,104,101,109,97,0,76,111,97,100,65,110,97,108,121,115,105,115,0,68,114,111,112,84,97,98,108,101,0,68,114,111,112,73,110,100,101,120,0,68,114,111,112,84,114,105,103,103,101,114,0,73,110,116,101,103,114,105,116,121,67,107,0,82,111,119,83,101,116,65,100,100,0,82,111,119,83,101,116,82,101,97,100,0,82,111,119,83,101,116,84,101,115,116,0,80,114,111,103,114,97,109,0,82,101,97,108,0,80,97,114,97,109,0,70,107,67,111,117,110,116,101,114,0,70], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+10240);
/* memory initializer */ allocate([107,73,102,90,101,114,111,0,77,101,109,77,97,120,0,73,102,80,111,115,0,79,102,102,115,101,116,76,105,109,105,116,0,73,102,78,111,116,90,101,114,111,0,68,101,99,114,74,117,109,112,90,101,114,111,0,74,117,109,112,90,101,114,111,73,110,99,114,0,65,103,103,83,116,101,112,48,0,65,103,103,83,116,101,112,0,65,103,103,70,105,110,97,108,0,73,110,99,114,86,97,99,117,117,109,0,69,120,112,105,114,101,0,84,97,98,108,101,76,111,99,107,0,86,66,101,103,105,110,0,86,67,114,101,97,116,101,0,86,68,101,115,116,114,111,121,0,86,79,112,101,110,0,86,67,111,108,117,109,110,0,86,78,101,120,116,0,86,82,101,110,97,109,101,0,80,97,103,101,99,111,117,110,116,0,77,97,120,80,103,99,110,116,0,73,110,105,116,0,67,117,114,115,111,114,72,105,110,116,0,78,111,111,112,0,69,120,112,108,97,105,110,0,107,40,37,100,0,66,73,78,65,82,89,0,66,0,44,37,115,37,115,0,45,0,41,0,40,37,46,50,48,115,41,0,37,115,40,37,100,41,0,37,100,0,37,46,49,54,103,0,40,98,108,111,98,41,0,118,116,97,98,58,37,112,0,44,37,100,0,93,0,112,114,111,103,114,97,109,0,78,79,84,32,78,85,76,76,0,85,78,73,81,85,69,0,67,72,69,67,75,0,70,79,82,69,73,71,78,32,75,69,89,0,97,98,111,114,116,32,97,116,32,37,100,32,105,110,32,91,37,115,93,58,32,37,115,0,37,115,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,58,32,37,115,0,37,115,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,0,0,0,0,1,2,0,2,2,0,1,2,1,1,1,2,1,2,0,0,0,0,0,0,0,0,99,97,110,110,111,116,32,111,112,101,110,32,115,97,118,101,112,111,105,110,116,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,110,111,32,115,117,99,104,32,115,97,118,101,112,111,105,110,116,58,32,37,115,0,99,97,110,110,111,116,32,114,101,108,101,97,115,101,32,115,97,118,101,112,111,105,110,116,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,99,97,110,110,111,116,32,99,111,109,109,105,116,32,116,114,97,110,115,97,99,116,105,111,110,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,99,97,110,110,111,116,32,115,116,97,114,116,32,97,32,116,114,97,110,115,97,99,116,105,111,110,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,99,97,110,110,111,116,32,114,111,108,108,98,97,99,107,32,45,32,110,111,32,116,114,97,110,115,97,99,116,105,111,110,32,105,115,32,97,99,116,105,118,101,0,99,97,110,110,111,116,32,99,111,109,109,105,116,32,45,32,110,111,32,116,114,97,110,115,97,99,116,105,111,110,32,105,115,32,97,99,116,105,118,101,0,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,104,97,115,32,99,104,97,110,103,101,100,0,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,0,115,113,108,105,116,101,95,109,97,115,116,101,114,0,83,69,76,69,67,84,32,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,70,82,79,77,32,39,37,113,39,46,37,115,32,87,72,69,82,69,32,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,0,116,111,111,32,109,97,110,121,32,108,101,118,101,108,115,32,111,102,32,116,114,105,103,103,101,114,32,114,101,99,117,114,115,105,111,110,0,99,97,110,110,111,116,32,99,104,97,110,103,101,32,37,115,32,119,97,108,32,109,111,100,101,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,105,110,116,111,0,111,117,116,32,111,102,0,100,97,116,97,98,97,115,101,32,116,97,98,108,101,32,105,115,32,108,111,99,107,101,100,58,32,37,115,0,115,116,97,116,101,109,101,110,116,32,97,98,111,114,116,115,32,97,116,32,37,100,58,32,91,37,115,93,32,37,115,0,0,1,2,3,4,6,8,8,0,0,0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39,40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47,48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57,0,1,2,3,4,6,8,99,114,101,97,116,101,32,0,105,110,118,97,108,105,100,32,114,111,111,116,112,97,103,101,0,63,0,109,97,108,102,111,114,109,101,100,32,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,40,37,115,41,0,37,122,32,45,32,37,115,0,115,113,108,105,116,101,95,115,116,97,116,49,0,83,69,76,69,67,84,32,116,98,108,44,105,100,120,44,115,116,97,116,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,115,116,97,116,49,0,117,110,111,114,100,101,114,101,100,42,0,115,122,61,91,48,45,57,93,42,0,110,111,115,107,105,112,115,99,97,110,42,0,77,97,105,110,32,102,114,101,101,108,105,115,116,58,32,0,80,97,103,101,32,37,100,32,105,115,32,110,101,118,101,114,32,117,115,101,100,0,80,111,105,110,116,101,114,32,109,97,112,32,112,97,103,101,32,37,100,32,105,115,32,114,101,102,101,114,101,110,99,101,100,0,37,100,32,111,102,32,37,100,32,112,97,103,101,115,32,109,105,115,115,105,110,103,32,102,114,111,109,32,111,118,101,114,102,108,111,119,32,108,105,115,116,32,115,116,97,114,116,105,110,103,32,97,116,32,37,100,0,102,97,105,108,101,100,32,116,111,32,103,101,116,32,112,97,103,101,32,37,100,0,102,114,101,101,108,105,115,116,32,108,101,97,102,32,99,111,117,110,116,32,116,111,111,32,98,105,103,32,111,110,32,112,97,103,101,32,37,100,0,102,114,101,101,45,112,97,103,101,32,99,111,117,110,116,32,105,110,32,104,101,97,100,101,114,32,105,115,32,116,111,111,32,115,109,97,108,108,0,105,110,118,97,108,105,100,32,112,97,103,101,32,110,117,109,98,101,114,32,37,100,0,50,110,100,32,114,101,102,101,114,101,110,99,101,32,116,111,32,112,97,103,101,32,37,100,0,70,97,105,108,101,100,32,116,111,32,114,101,97,100,32,112,116,114,109,97,112,32,107,101,121,61,37,100,0,66,97,100,32,112,116,114,32,109,97,112,32,101,110,116,114,121,32,107,101,121,61,37,100,32,101,120,112,101,99,116,101,100,61,40,37,100,44,37,100,41,32,103,111,116,61,40,37,100,44,37,100,41,0,80,97,103,101,32,37,100,58,32,0,117,110,97,98,108,101,32,116,111,32,103,101,116,32,116,104,101,32,112,97,103,101,46,32,101,114,114,111,114,32,99,111,100,101,61,37,100,0,98,116,114,101,101,73,110,105,116,80,97,103,101,40,41,32,114,101,116,117,114,110,115,32,101,114,114,111,114,32,99,111,100,101,32,37,100,0,79,110,32,116,114,101,101,32,112,97,103,101,32,37,100,32,99,101,108,108,32,37,100,58,32,0,79,110,32,112,97,103,101,32,37,100,32,97,116,32,114,105,103,104,116,32,99,104,105,108,100,58,32,0,79,102,102,115,101,116,32,37,100,32,111,117,116,32,111,102,32,114,97,110,103,101,32,37,100,46,46,37,100,0,69,120,116,101,110,100,115,32,111,102,102,32,101,110,100,32,111,102,32,112,97,103,101,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,0,67,104,105,108,100,32,112,97,103,101,32,100,101,112,116,104,32,100,105,102,102,101,114,115,0,77,117,108,116,105,112,108,101,32,117,115,101,115,32,102,111,114,32,98,121,116,101,32,37,117,32,111,102,32,112,97,103,101,32,37,100,0,70,114,97,103,109,101,110,116,97,116,105,111,110,32,111,102,32,37,100,32,98,121,116,101,115,32,114,101,112,111,114,116,101,100,32,97,115,32,37,100,32,111,110,32,112,97,103,101,32,37,100,0,10,0,100,101,108,101,116,101,0,112,101,114,115,105,115,116,0,111,102,102,0,116,114,117,110,99,97,116,101,0,109,101,109,111,114,121,0,119,97,108,0,99,97,110,110,111,116,32,86,65,67,85,85,77,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,99,97,110,110,111,116,32,86,65,67,85,85,77,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,65,84,84,65,67,72,32,39,58,109,101,109,111,114,121,58,39,32,65,83,32,118,97,99,117,117,109,95,100,98,59,0,65,84,84,65,67,72,32,39,39,32,65,83,32,118,97,99,117,117,109,95,100,98,59,0,80,82,65,71,77,65,32,118,97,99,117,117,109,95,100,98,46,115,121,110,99,104,114,111,110,111,117,115,61,79,70,70,0,66,69,71,73,78,59,0,83,69,76,69,67,84,32,39,67,82,69,65,84,69,32,84,65,66,76,69,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,49,52,41,32,32,32,70,82,79,77,32,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,61,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,33,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,32,32,65,78,68,32,99,111,97,108,101,115,99,101,40,114,111,111,116,112,97,103,101,44,49,41,62,48,0,83,69,76,69,67,84,32,39,67,82,69,65,84,69,32,73,78,68,69,88,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,49,52,41,32,32,70,82,79,77,32,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,115,113,108,32,76,73,75,69,32,39,67,82,69,65,84,69,32,73,78,68,69,88,32,37,39,32,0,83,69,76,69,67,84,32,39,67,82,69,65,84,69,32,85,78,73,81,85,69,32,73,78,68,69,88,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,50,49,41,32,32,32,70,82,79,77,32,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,115,113,108,32,76,73,75,69,32,39,67,82,69,65,84,69,32,85,78,73,81,85,69,32,73,78,68,69,88,32,37,39,0,83,69,76,69,67,84,32,39,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,32,83,69,76,69,67,84,32,42,32,70,82,79,77,32,109,97,105,110,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,59,39,70,82,79,77,32,109,97,105,110,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,32,61,32,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,33,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,32,32,65,78,68,32,99,111,97,108,101,115,99,101,40,114,111,111,116,112,97,103,101,44,49,41,62,48,0,83,69,76,69,67,84,32,39,68,69,76,69,84,69,32,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,59,39,32,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,110,97,109,101,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,0,83,69,76,69,67,84,32,39,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,32,83,69,76,69,67,84,32,42,32,70,82,79,77,32,109,97,105,110,46,39,32,124,124,32,113,117,111,116,101,40,110,97,109,101,41,32,124,124,32,39,59,39,32,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,110,97,109,101,61,61,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,59,0,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,32,32,83,69,76,69,67,84,32,116,121,112,101,44,32,110,97,109,101,44,32,116,98,108,95,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,32,32,32,70,82,79,77,32,109,97,105,110,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,32,32,87,72,69,82,69,32,116,121,112,101,61,39,118,105,101,119,39,32,79,82,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,32,32,32,32,32,79,82,32,40,116,121,112,101,61,39,116,97,98,108,101,39,32,65,78,68,32,114,111,111,116,112,97,103,101,61,48,41,0,1,1,3,0,5,0,6,0,8,0,110,111,32,115,117,99,104,32,109,111,100,117,108,101,58,32,37,115,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,99,97,108,108,101,100,32,114,101,99,117,114,115,105,118,101,108,121,58,32,37,115,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,102,97,105,108,101,100,58,32,37,115,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,100,105,100,32,110,111,116,32,100,101,99,108,97,114,101,32,115,99,104,101,109,97,58,32,37,115,0,104,105,100,100,101,110,0,32,104,105,100,100,101,110,0,45,45,32,0,39,37,46,42,113,39,0,122,101,114,111,98,108,111,98,40,37,100,41,0,120,39,0,37,48,50,120,0,39,0,27,27,27,27,27,27,27,27,27,7,7,27,7,7,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,7,15,8,5,4,22,24,8,17,18,21,20,23,11,26,16,3,3,3,3,3,3,3,3,3,3,5,19,12,14,13,6,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,9,27,27,27,1,8,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,27,10,27,25,27,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,82,69,73,78,68,69,88,69,68,69,83,67,65,80,69,65,67,72,69,67,75,69,89,66,69,70,79,82,69,73,71,78,79,82,69,71,69,88,80,76,65,73,78,83,84,69,65,68,68,65,84,65,66,65,83,69,76,69,67,84,65,66,76,69,70,84,72,69,78,68,69,70,69,82,82,65,66,76,69,76,83,69,88,67,69,80,84,82,65,78,83,65,67,84,73,79,78,65,84,85,82,65,76,84,69,82,65,73,83,69,88,67,76,85,83,73,86,69,88,73,83,84,83,65,86,69,80,79,73,78,84,69,82,83,69,67,84,82,73,71,71,69,82,69,70,69,82,69,78,67,69,83,67,79,78,83,84,82,65,73,78,84,79,70,70,83,69,84,69,77,80,79,82,65,82,89,85,78,73,81,85,69,82,89,87,73,84,72,79,85,84,69,82,69,76,69,65,83,69,65,84,84,65,67,72,65,86,73,78,71,82,79,85,80,68,65,84,69,66,69,71,73,78,78,69,82,69,67,85,82,83,73,86,69,66,69,84,87,69,69,78,79,84,78,85,76,76,73,75,69,67,65,83,67,65,68,69,76,69,84,69,67,65,83,69,67,79,76,76,65,84,69,67,82,69,65,84,69,67,85,82,82,69,78,84,95,68,65,84,69,68,69,84,65,67,72,73,77,77,69,68,73,65,84,69,74,79,73,78,83,69,82,84,77,65,84,67,72,80,76,65,78,65,76,89,90,69,80,82,65,71,77,65,66,79,82,84,86,65,76,85,69,83,86,73,82,84,85,65,76,73,77,73,84,87,72,69,78,87,72,69,82,69,78,65,77,69,65,70,84,69,82,69,80,76,65,67,69,65,78,68,69,70,65,85,76,84,65,85,84,79,73,78,67,82,69,77,69,78,84,67,65,83,84,67,79,76,85,77,78,67,79,77,77,73,84,67,79,78,70,76,73,67,84,67,82,79,83,83,67,85,82,82,69,78,84,95,84,73,77,69,83,84,65,77,80,82,73,77,65,82,89,68,69,70,69,82,82,69,68,73,83,84,73,78,67,84,68,82,79,80,70,65,73,76,70,82,79,77,70,85,76,76,71,76,79,66,89,73,70,73,83,78,85,76,76,79,82,68,69,82,69,83,84,82,73,67,84,82,73,71,72,84,82,79,76,76,66,65,67,75,82,79,87,85,78,73,79,78,85,83,73,78,71,86,65,67,85,85,77,86,73,69,87,73,78,73,84,73,65,76,76,89,76,105,117,74,0,45,0,0,82,0,77,0,0,42,12,78,15,0,116,85,54,112,0,19,0,0,121,0,119,115,0,22,93,0,9,0,0,70,71,0,69,6,0,48,90,102,0,118,101,0,0,44,0,103,24,0,17,0,122,53,23,0,5,110,25,96,0,0,124,106,60,123,57,28,55,0,91,0,100,26,0,99,0,0,0,95,92,97,88,109,14,39,108,0,81,0,18,89,111,32,0,120,80,113,62,46,84,0,0,94,40,59,114,0,36,0,0,29,0,86,63,64,0,20,61,0,56,0,0,0,0,4,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,13,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,33,0,21,0,0,0,0,0,50,0,43,3,47,0,0,0,0,30,0,58,0,38,0,0,0,1,66,0,0,67,0,41,0,0,0,0,0,0,49,65,0,0,0,0,31,52,16,34,10,0,0,0,0,0,0,0,11,72,79,0,8,0,104,98,0,107,0,87,0,75,51,0,27,37,73,83,0,35,68,0,0,7,7,5,4,6,4,5,3,6,7,3,6,6,7,7,3,8,2,6,5,4,4,3,10,4,6,11,6,2,7,5,5,9,6,9,9,7,10,10,4,6,2,3,9,4,2,6,5,7,4,5,7,6,6,5,6,5,5,9,7,7,3,2,4,4,7,3,6,4,7,6,12,6,9,4,6,5,4,7,6,5,6,7,5,4,5,6,5,7,3,7,13,2,2,4,6,6,8,5,17,12,7,8,8,2,4,4,4,4,4,2,2,6,5,8,5,8,3,5,5,6,4,9,3,67,28,140,42,84,44,104,53,35,113,46,47,50,2,49,142,41,24,119,16,98,138,11,112,139,117,6,30,107,98,141,57,9,20,13,118,62,105,99,131,55,54,111,21,21,71,103,3,25,66,98,14,34,128,127,110,5,98,58,74,77,19,52,101,50,37,33,109,136,95,17,69,43,8,124,108,51,4,32,56,29,120,65,129,137,130,68,31,59,72,100,106,15,75,38,39,10,40,98,69,69,102,7,121,73,114,45,123,98,50,36,18,76,126,60,98,12,61,115,125,63,64,48,116,98,105,110,100,32,111,110,32,97,32,98,117,115,121,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,58,32,91,37,115,93,0,0,0,0,1,1,1,1,1,0,16,0,1,0,1,1,2,1,2,3,18,8,0,16,16,16,16,0,16,16,0,0,16,16,0,0,0,0,2,3,2,2,0,0,1,1,3,3,0,0,0,16,16,0,0,0,0,0,0,0,0,0,0,0,9,9,9,9,9,9,9,9,38,38,16,16,0,3,3,11,11,11,11,11,11,0,38,38,38,38,38,38,38,38,38,38,0,18,16,0,0,0,0,0,16,0,1,1,1,1,4,4,0,0,16,1,1,1,1,16,0,0,16,16,0,0,0,0,0,0,6,35,11,1,16,16,0,1,4,3,26,3,3,3,0,0,0,1,0,0,0,0,0,0,0,1,0,16,16,1,0,0,0,99,97,110,110,111,116,32,111,112,101,110,32,118,97,108,117,101,32,111,102,32,116,121,112,101,32,37,115,0,110,117,108,108,0,114,101,97,108,0,105,110,116,101,103,101,114,0,110,111,32,115,117,99,104,32,114,111,119,105,100,58,32,37,108,108,100,0,110,111,32,115,117,99,104,32,118,105,101,119,0,110,111,32,115,117,99,104,32,116,97,98,108,101,0,37,115,58,32,37,115,46,37,115,0,37,115,58,32,37,115,0,117,110,111,112,101,110,101,100,0,65,80,73,32,99,97,108,108,32,119,105,116,104,32,37,115,32,100,97,116,97,98,97,115,101,32,99,111,110,110,101,99,116,105,111,110,32,112,111,105,110,116,101,114,0,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,105,115,32,108,111,99,107,101,100,58,32,37,115,0,115,116,97,116,101,109,101,110,116,32,116,111,111,32,108,111,110,103,0,97,100,100,114,0,111,112,99,111,100,101,0,112,49,0,112,50,0,112,51,0,112,52,0,112,53,0,99,111,109,109,101,110,116,0,115,101,108,101,99,116,105,100,0,111,114,100,101,114,0,102,114,111,109,0,100,101,116,97,105,108,0,117,110,114,101,99,111,103,110,105,122,101,100,32,116,111,107,101,110,58,32,34,37,84,34,0,19,144,145,146,147,24,1,2,27,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,91,92,93,94,95,19,50,51,80,81,82,83,27,85,86,87,88,89,90,91,92,93,94,95,137,177,139,71,72,73,74,75,76,77,78,79,80,81,82,83,95,85,86,87,88,89,90,91,92,93,94,95,19,97,85,86,87,88,89,90,91,92,93,94,95,152,33,212,173,27,28,223,224,225,152,42,181,152,223,224,225,95,97,50,51,99,100,101,102,103,104,105,27,28,59,223,224,225,112,174,175,66,94,95,71,72,73,74,75,76,77,78,79,80,81,82,83,195,85,86,87,88,89,90,91,92,93,94,95,19,197,89,90,220,209,210,26,119,120,97,98,208,100,108,109,110,152,157,152,107,152,109,89,90,91,92,93,94,95,163,50,51,97,98,99,55,66,102,103,104,174,175,174,175,132,133,192,193,113,66,19,71,72,73,74,75,76,77,78,79,80,81,82,83,198,85,86,87,88,89,90,91,92,93,94,95,152,209,210,148,149,50,51,100,53,154,59,156,174,229,27,28,232,163,163,22,192,193,174,175,27,28,71,72,73,74,75,76,77,78,79,80,81,82,83,251,85,86,87,88,89,90,91,92,93,94,95,19,198,198,152,152,24,209,210,108,109,110,196,197,27,28,69,164,165,152,119,120,27,28,208,99,27,28,102,103,104,152,50,51,97,98,89,90,185,113,187,22,177,174,97,58,27,28,101,115,245,117,118,71,72,73,74,75,76,77,78,79,80,81,82,83,11,85,86,87,88,89,90,91,92,93,94,95,19,132,133,134,23,152,97,98,91,174,223,224,225,239,97,98,187,22,97,98,27,28,27,28,152,223,224,225,239,152,163,50,51,170,171,172,59,160,97,98,239,164,165,66,242,124,174,175,195,22,23,69,71,72,73,74,75,76,77,78,79,80,81,82,83,198,85,86,87,88,89,90,91,92,93,94,95,19,12,97,21,23,152,101,152,108,109,110,221,152,97,98,97,98,152,29,243,70,226,23,233,26,26,152,152,238,174,175,50,51,22,45,24,47,132,133,134,124,22,23,188,163,26,152,65,59,174,175,163,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,19,19,198,152,23,152,152,209,103,198,174,175,70,152,50,51,219,213,214,152,98,152,171,172,188,171,172,174,175,248,249,50,51,51,251,174,175,220,74,75,152,188,152,174,175,140,124,26,163,188,16,130,71,72,73,74,75,76,77,78,79,80,81,82,83,101,85,86,87,88,89,90,91,92,93,94,95,19,209,196,197,23,231,198,12,231,219,37,22,107,24,208,116,27,112,201,62,121,64,152,152,29,52,66,221,211,221,219,50,51,60,89,90,152,22,23,183,45,26,47,22,174,175,238,152,164,165,106,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,132,133,119,120,163,50,51,111,70,97,174,175,181,182,170,171,172,0,1,2,140,190,108,109,110,51,71,72,73,74,75,76,77,78,79,80,81,82,83,198,85,86,87,88,89,90,91,92,93,94,95,19,152,152,152,22,166,152,168,169,27,19,152,26,19,152,122,152,24,152,28,27,152,28,174,175,174,175,152,174,175,107,50,51,174,175,22,174,175,174,175,174,175,138,174,175,7,8,9,16,174,175,152,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,19,152,97,152,31,24,152,98,35,101,174,175,152,97,152,79,152,62,152,64,112,152,49,174,175,174,175,152,174,175,152,50,51,124,174,175,174,175,174,175,174,175,138,174,175,22,23,152,152,174,175,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,108,109,110,152,50,51,146,147,23,174,175,26,195,195,152,70,152,168,169,152,26,174,175,152,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,246,247,213,214,50,51,195,152,195,174,175,195,100,101,152,195,152,152,7,8,152,174,175,163,19,71,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,27,152,189,189,50,51,195,198,152,174,175,33,132,133,152,123,163,163,163,152,42,174,175,152,19,152,72,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,152,198,198,198,23,50,51,26,152,23,174,175,26,23,23,23,26,26,26,152,97,23,174,175,26,132,133,152,73,74,75,76,77,78,79,80,81,82,83,152,85,86,87,88,89,90,91,92,93,94,95,19,20,23,22,23,26,152,152,27,28,174,175,152,19,20,27,22,183,183,38,152,27,28,152,23,152,152,26,174,175,152,152,38,152,23,152,27,26,57,152,215,163,152,152,152,174,175,66,174,175,69,57,174,175,152,174,175,174,175,212,66,174,175,69,174,175,174,175,152,152,89,90,152,193,152,152,198,96,97,98,91,152,101,89,90,97,152,209,210,152,96,97,98,235,152,101,174,175,152,19,20,152,22,174,175,116,97,27,28,152,121,174,175,132,133,134,135,136,38,152,174,175,152,174,175,152,132,133,134,135,136,234,152,212,150,199,212,57,212,240,240,203,178,200,216,186,177,19,20,69,22,203,177,182,177,27,28,202,200,228,216,216,155,39,122,159,38,159,41,89,90,91,159,241,241,22,96,97,98,71,130,101,222,191,18,57,203,194,159,194,194,194,18,158,244,191,222,69,159,158,137,19,20,203,22,191,203,46,236,27,28,159,132,133,134,135,136,89,90,237,38,159,158,22,96,97,98,179,159,101,158,48,159,158,179,176,107,176,184,57,106,176,184,176,125,179,178,176,218,107,176,69,176,217,159,218,218,217,159,137,132,133,134,135,136,218,217,179,217,179,227,89,90,95,230,230,129,207,96,97,98,126,128,101,5,206,205,127,204,10,11,12,13,14,203,25,17,162,26,161,13,153,153,6,247,180,250,151,151,151,151,32,180,34,132,133,134,135,136,167,4,3,43,22,15,68,142,250,16,23,23,120,111,131,20,56,123,125,16,1,123,131,63,79,79,66,67,111,28,36,122,1,5,22,107,140,54,54,26,61,107,44,20,24,19,105,112,23,22,40,22,22,53,22,53,22,53,23,23,23,22,22,30,116,23,122,26,23,23,22,28,11,124,114,26,26,23,23,23,36,24,23,36,26,22,22,36,23,122,23,22,26,22,24,23,23,23,22,122,23,141,122,122,15,1,0,0,27,27,27,27,0,27,27,27,0,27,27,27,27,0,0,0,27,0,0,27,0,0,0,27,0,0,0,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,112,97,114,115,101,114,32,115,116,97,99,107,32,111,118,101,114,102,108,111,119,0,114,111,119,105,100,0,117,110,107,110,111,119,110,32,116,97,98,108,101,32,111,112,116,105,111,110,58,32,37,46,42,115,0,115,101,116,32,108,105,115,116,0,110,101,97,114,32,34,37,84,34,58,32,115,121,110,116,97,120,32,101,114,114,111,114,0,116,111,111,32,109,97,110,121,32,97,114,103,117,109,101,110,116,115,32,111,110,32,102,117,110,99,116,105,111,110,32,37,84,0,113,117,97,108,105,102,105,101,100,32,116,97,98,108,101,32,110,97,109,101,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,73,78,83,69,82,84,44,32,85,80,68,65,84,69,44,32,97,110,100,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,116,104,101,32,73,78,68,69,88,69,68,32,66,89,32,99,108,97,117,115,101,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,85,80,68,65,84,69,32,111,114,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,116,104,101,32,78,79,84,32,73,78,68,69,88,69,68,32,99,108,97,117,115,101,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,85,80,68,65,84,69,32,111,114,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,144,1,145,2,145,1,146,1,146,3,147,0,147,1,147,3,148,1,149,3,151,0,151,1,151,2,150,0,150,1,150,1,150,1,149,2,149,2,149,2,153,1,153,0,149,2,149,3,149,5,149,2,154,6,156,1,158,0,158,3,157,1,157,0,155,5,155,2,162,0,162,2,160,3,160,1,164,3,165,1,152,1,152,1,152,1,166,0,166,1,168,1,168,4,168,6,169,1,169,2,170,1,170,1,167,2,167,0,173,2,173,2,173,4,173,3,173,3,173,2,173,2,173,3,173,5,173,2,173,4,173,4,173,1,173,2,178,0,178,1,180,0,180,2,182,2,182,3,182,3,182,3,183,2,183,2,183,1,183,1,183,2,181,3,181,2,184,0,184,2,184,2,161,0,161,2,185,3,185,1,186,1,186,0,187,2,187,7,187,5,187,5,187,10,190,0,190,1,176,0,176,3,191,0,191,2,192,1,192,1,192,1,149,4,194,2,194,0,149,9,149,4,149,1,163,2,196,1,196,3,199,1,199,2,199,1,197,9,197,1,208,4,208,5,200,1,200,1,200,0,211,2,211,0,201,3,201,2,201,4,212,2,212,1,212,0,202,0,202,2,214,2,214,0,213,7,213,9,213,7,213,7,159,0,159,2,195,2,215,1,215,2,215,3,215,4,217,2,217,0,216,0,216,3,216,2,218,4,218,0,206,0,206,3,188,4,188,2,177,1,177,1,177,0,204,0,204,3,205,0,205,2,207,0,207,2,207,4,207,4,149,6,203,0,203,2,149,8,220,5,220,3,149,6,149,7,221,2,221,1,222,0,222,3,219,3,219,1,175,1,175,3,174,1,175,1,175,1,175,3,175,5,174,1,174,1,175,1,175,3,175,6,175,5,175,4,174,1,175,3,175,3,175,3,175,3,175,3,175,3,175,3,175,3,223,1,223,2,175,3,175,5,175,2,175,3,175,3,175,4,175,2,175,2,175,2,175,2,224,1,224,2,175,5,225,1,225,2,175,5,175,3,175,5,175,4,175,4,175,5,227,5,227,4,228,2,228,0,226,1,226,0,210,1,210,0,209,3,209,1,149,12,229,1,229,0,179,0,179,3,189,5,189,3,230,0,230,2,149,4,149,1,149,2,149,3,149,5,149,6,149,5,149,6,231,1,231,1,231,1,231,1,231,1,171,2,171,1,172,2,149,5,232,11,234,1,234,1,234,2,234,0,235,1,235,1,235,3,236,0,236,3,237,0,237,2,233,3,233,2,239,1,239,3,240,0,240,3,240,2,238,7,238,5,238,5,238,1,175,4,175,6,193,1,193,1,193,1,149,4,149,6,149,3,242,0,242,2,241,1,241,0,149,1,149,3,149,1,149,3,149,6,149,6,243,1,244,0,244,1,149,1,149,4,245,8,246,1,246,3,247,0,247,2,248,1,248,3,249,1,250,0,250,4,250,2,198,0,198,2,198,3,251,6,251,8,25,0,0,0,108,0,9,0,47,0,0,0,78,0,7,0,103,0,0,0,47,0,1,0,13,0,9,0,7,0,2,0,22,0,0,0,61,0,0,0,110,111,32,115,117,99,104,32,99,111,108,108,97,116,105,111,110,32,115,101,113,117,101,110,99,101,58,32,37,115,0,3,2,1,109,105,115,117,115,101,32,111,102,32,97,103,103,114,101,103,97,116,101,58,32,37,115,40,41,0,117,110,107,110,111,119,110,32,102,117,110,99,116,105,111,110,58,32,37,46,42,115,40,41,0,82,65,73,83,69,40,41,32,109,97,121,32,111,110,108,121,32,98,101,32,117,115,101,100,32,119,105,116,104,105,110,32,97,32,116,114,105,103,103,101,114,45,112,114,111,103,114,97,109,0,37,115,37,115,0,48,120,0,104,101,120,32,108,105,116,101,114,97,108,32,116,111,111,32,98,105,103,58,32,37,115,0,69,88,69,67,85,84,69,32,37,115,37,115,32,83,85,66,81,85,69,82,89,32,37,100,0,67,79,82,82,69,76,65,84,69,68,32,0,76,73,83,84,0,83,67,65,76,65,82,0,66,69,71,73,78,0,110,111,116,32,97,117,116,104,111,114,105,122,101,100,0,97,117,116,104,111,114,105,122,101,114,32,109,97,108,102,117,110,99,116,105,111,110,0,67,79,77,77,73,84,0,82,79,76,76,66,65,67,75,0,82,69,76,69,65,83,69,0,116,101,109,112,111,114,97,114,121,32,116,97,98,108,101,32,110,97,109,101,32,109,117,115,116,32,98,101,32,117,110,113,117,97,108,105,102,105,101,100,0,2,4,8,6,116,97,98,108,101,32,37,84,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,110,32,105,110,100,101,120,32,110,97,109,101,100,32,37,115,0,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,0,6,0,0,0,0,0,99,111,114,114,117,112,116,32,100,97,116,97,98,97,115,101,0,117,110,107,110,111,119,110,32,100,97,116,97,98,97,115,101,32,37,84,0,115,113,108,105,116,101,95,0,111,98,106,101,99,116,32,110,97,109,101,32,114,101,115,101,114,118,101,100,32,102,111,114,32,105,110,116,101,114,110,97,108,32,117,115,101,58,32,37,115,0,65,85,84,79,73,78,67,82,69,77,69,78,84,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,87,73,84,72,79,85,84,32,82,79,87,73,68,32,116,97,98,108,101,115,0,80,82,73,77,65,82,89,32,75,69,89,32,109,105,115,115,105,110,103,32,111,110,32,116,97,98,108,101,32,37,115,0,116,97,98,108,101,0,84,65,66,76,69,0,118,105,101,119,0,86,73,69,87,0,67,82,69,65,84,69,32,37,115,32,37,46,42,115,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,116,121,112,101,61,39,37,115,39,44,32,110,97,109,101,61,37,81,44,32,116,98,108,95,110,97,109,101,61,37,81,44,32,114,111,111,116,112,97,103,101,61,35,37,100,44,32,115,113,108,61,37,81,32,87,72,69,82,69,32,114,111,119,105,100,61,35,37,100,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,40,110,97,109,101,44,115,101,113,41,0,116,98,108,95,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,33,61,39,116,114,105,103,103,101,114,39,0,69,120,112,114,101,115,115,105,111,110,32,116,114,101,101,32,105,115,32,116,111,111,32,108,97,114,103,101,32,40,109,97,120,105,109,117,109,32,100,101,112,116,104,32,37,100,41,0,116,104,101,32,34,46,34,32,111,112,101,114,97,116,111,114,0,102,117,110,99,116,105,111,110,115,0,115,101,99,111,110,100,32,97,114,103,117,109,101,110,116,32,116,111,32,108,105,107,101,108,105,104,111,111,100,40,41,32,109,117,115,116,32,98,101,32,97,32,99,111,110,115,116,97,110,116,32,98,101,116,119,101,101,110,32,48,46,48,32,97,110,100,32,49,46,48,0,110,111,116,32,97,117,116,104,111,114,105,122,101,100,32,116,111,32,117,115,101,32,102,117,110,99,116,105,111,110,58,32,37,115,0,110,111,110,45,100,101,116,101,114,109,105,110,105,115,116,105,99,32,102,117,110,99,116,105,111,110,115,0,109,105,115,117,115,101,32,111,102,32,97,103,103,114,101,103,97,116,101,32,102,117,110,99,116,105,111,110,32,37,46,42,115,40,41,0,110,111,32,115,117,99,104,32,102,117,110,99,116,105,111,110,58,32,37,46,42,115,0,119,114,111,110,103,32,110,117,109,98,101,114,32,111,102,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,117,110,99,116,105,111,110,32,37,46,42,115,40,41,0,115,117,98,113,117,101,114,105,101,115,0,112,97,114,97,109,101,116,101,114,115,0,110,101,119,0,111,108,100,0,109,105,115,117,115,101,32,111,102,32,97,108,105,97,115,101,100,32,97,103,103,114,101,103,97,116,101,32,37,115,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,0,97,109,98,105,103,117,111,117,115,32,99,111,108,117,109,110,32,110,97,109,101,0,37,115,58,32,37,115,46,37,115,46,37,115,0,82,79,87,73,68,0,97,99,99,101,115,115,32,116,111,32,37,115,46,37,115,46,37,115,32,105,115,32,112,114,111,104,105,98,105,116,101,100,0,97,99,99,101,115,115,32,116,111,32,37,115,46,37,115,32,105,115,32,112,114,111,104,105,98,105,116,101,100,0,112,97,114,116,105,97,108,32,105,110,100,101,120,32,87,72,69,82,69,32,99,108,97,117,115,101,115,0,105,110,100,101,120,32,101,120,112,114,101,115,115,105,111,110,115,0,67,72,69,67,75,32,99,111,110,115,116,114,97,105,110,116,115,0,37,115,32,112,114,111,104,105,98,105,116,101,100,32,105,110,32,37,115,0,97,32,71,82,79,85,80,32,66,89,32,99,108,97,117,115,101,32,105,115,32,114,101,113,117,105,114,101,100,32,98,101,102,111,114,101,32,72,65,86,73,78,71,0,79,82,68,69,82,0,71,82,79,85,80,0,97,103,103,114,101,103,97,116,101,32,102,117,110,99,116,105,111,110,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,116,104,101,32,71,82,79,85,80,32,66,89,32,99,108,97,117,115,101,0,115,113,108,105,116,101,95,115,113,95,37,112,0,116,111,111,32,109,97,110,121,32,114,101,102,101,114,101,110,99,101,115,32,116,111,32,34,37,115,34,58,32,109,97,120,32,54,53,53,51,53,0,42,0,37,115,46,37,115,0,37,115,46,37,115,46,37,115,0,110,111,32,115,117,99,104,32,116,97,98,108,101,58,32,37,115,0,110,111,32,116,97,98,108,101,115,32,115,112,101,99,105,102,105,101,100,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,105,110,32,114,101,115,117,108,116,32,115,101,116,0,109,117,108,116,105,112,108,101,32,114,101,102,101,114,101,110,99,101,115,32,116,111,32,114,101,99,117,114,115,105,118,101,32,116,97,98,108,101,58,32,37,115,0,99,105,114,99,117,108,97,114,32,114,101,102,101,114,101,110,99,101,58,32,37,115,0,116,97,98,108,101,32,37,115,32,104,97,115,32,37,100,32,118,97,108,117,101,115,32,102,111,114,32,37,100,32,99,111,108,117,109,110,115,0,109,117,108,116,105,112,108,101,32,114,101,99,117,114,115,105,118,101,32,114,101,102,101,114,101,110,99,101,115,58,32,37,115,0,114,101,99,117,114,115,105,118,101,32,114,101,102,101,114,101,110,99,101,32,105,110,32,97,32,115,117,98,113,117,101,114,121,58,32,37,115,0,37,46,42,122,58,37,117,0,39,37,115,39,32,105,115,32,110,111,116,32,97,32,102,117,110,99,116,105,111,110,0,118,105,101,119,32,37,115,32,105,115,32,99,105,114,99,117,108,97,114,108,121,32,100,101,102,105,110,101,100,0,110,111,32,115,117,99,104,32,105,110,100,101,120,58,32,37,115,0,97,32,78,65,84,85,82,65,76,32,106,111,105,110,32,109,97,121,32,110,111,116,32,104,97,118,101,32,97,110,32,79,78,32,111,114,32,85,83,73,78,71,32,99,108,97,117,115,101,0,99,97,110,110,111,116,32,104,97,118,101,32,98,111,116,104,32,79,78,32,97,110,100,32,85,83,73,78,71,32,99,108,97,117,115,101,115,32,105,110,32,116,104,101,32,115,97,109,101,32,106,111,105,110,0,99,97,110,110,111,116,32,106,111,105,110,32,117,115,105,110,103,32,99,111,108,117,109,110,32,37,115,32,45,32,99,111,108,117,109,110,32,110,111,116,32,112,114,101,115,101,110,116,32,105,110,32,98,111,116,104,32,116,97,98,108,101,115,0,37,114,32,37,115,32,66,89,32,116,101,114,109,32,111,117,116,32,111,102,32,114,97,110,103,101,32,45,32,115,104,111,117,108,100,32,98,101,32,98,101,116,119,101,101,110,32,49,32,97,110,100,32,37,100,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,37,115,32,66,89,32,99,108,97,117,115,101,0,97,108,108,32,86,65,76,85,69,83,32,109,117,115,116,32,104,97,118,101,32,116,104,101,32,115,97,109,101,32,110,117,109,98,101,114,32,111,102,32,116,101,114,109,115,0,83,69,76,69,67,84,115,32,116,111,32,116,104,101,32,108,101,102,116,32,97,110,100,32,114,105,103,104,116,32,111,102,32,37,115,32,100,111,32,110,111,116,32,104,97,118,101,32,116,104,101,32,115,97,109,101,32,110,117,109,98,101,114,32,111,102,32,114,101,115,117,108,116,32,99,111,108,117,109,110,115,0,85,78,73,79,78,32,65,76,76,0,73,78,84,69,82,83,69,67,84,0,69,88,67,69,80,84,0,85,78,73,79,78,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,79,82,68,69,82,32,66,89,32,99,108,97,117,115,101,0,37,114,32,79,82,68,69,82,32,66,89,32,116,101,114,109], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+20480);
/* memory initializer */ allocate([32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,97,110,121,32,99,111,108,117,109,110,32,105,110,32,116,104,101,32,114,101,115,117,108,116,32,115,101,116,0,44,0,10,32,32,0,44,10,32,32,0,10,41,0,67,82,69,65,84,69,32,84,65,66,76,69,32,0,32,84,69,88,84,0,32,78,85,77,0,32,73,78,84,0,32,82,69,65,76,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,111,110,32,37,115,0,100,117,112,108,105,99,97,116,101,32,99,111,108,117,109,110,32,110,97,109,101,58,32,37,115,0,100,101,102,97,117,108,116,32,118,97,108,117,101,32,111,102,32,99,111,108,117,109,110,32,91,37,115,93,32,105,115,32,110,111,116,32,99,111,110,115,116,97,110,116,0,116,97,98,108,101,32,34,37,115,34,32,104,97,115,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,112,114,105,109,97,114,121,32,107,101,121,0,65,85,84,79,73,78,67,82,69,77,69,78,84,32,105,115,32,111,110,108,121,32,97,108,108,111,119,101,100,32,111,110,32,97,110,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,0,105,110,100,101,120,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,97,32,84,69,77,80,32,105,110,100,101,120,32,111,110,32,110,111,110,45,84,69,77,80,32,116,97,98,108,101,32,34,37,115,34,0,97,108,116,101,114,116,97,98,95,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,118,105,101,119,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,118,105,114,116,117,97,108,32,116,97,98,108,101,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,32,116,97,98,108,101,32,110,97,109,101,100,32,37,115,0,105,110,100,101,120,32,37,115,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,95,37,115,95,37,100,0,101,120,112,114,101,115,115,105,111,110,115,32,112,114,111,104,105,98,105,116,101,100,32,105,110,32,80,82,73,77,65,82,89,32,75,69,89,32,97,110,100,32,85,78,73,81,85,69,32,99,111,110,115,116,114,97,105,110,116,115,0,99,111,110,102,108,105,99,116,105,110,103,32,79,78,32,67,79,78,70,76,73,67,84,32,99,108,97,117,115,101,115,32,115,112,101,99,105,102,105,101,100,0,67,82,69,65,84,69,37,115,32,73,78,68,69,88,32,37,46,42,115,0,32,85,78,73,81,85,69,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,37,115,32,86,65,76,85,69,83,40,39,105,110,100,101,120,39,44,37,81,44,37,81,44,35,37,100,44,37,81,41,59,0,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,0,37,115,32,37,84,32,99,97,110,110,111,116,32,114,101,102,101,114,101,110,99,101,32,111,98,106,101,99,116,115,32,105,110,32,100,97,116,97,98,97,115,101,32,37,115,0,37,115,32,99,97,110,110,111,116,32,117,115,101,32,118,97,114,105,97,98,108,101,115,0,105,110,100,101,120,32,39,37,113,39,0,44,32,0,102,111,114,101,105,103,110,32,107,101,121,32,111,110,32,37,115,32,115,104,111,117,108,100,32,114,101,102,101,114,101,110,99,101,32,111,110,108,121,32,111,110,101,32,99,111,108,117,109,110,32,111,102,32,116,97,98,108,101,32,37,84,0,110,117,109,98,101,114,32,111,102,32,99,111,108,117,109,110,115,32,105,110,32,102,111,114,101,105,103,110,32,107,101,121,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,99,111,108,117,109,110,115,32,105,110,32,116,104,101,32,114,101,102,101,114,101,110,99,101,100,32,116,97,98,108,101,0,117,110,107,110,111,119,110,32,99,111,108,117,109,110,32,34,37,115,34,32,105,110,32,102,111,114,101,105,103,110,32,107,101,121,32,100,101,102,105,110,105,116,105,111,110,0,115,113,108,105,116,101,95,115,116,97,116,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,100,114,111,112,112,101,100,0,117,115,101,32,68,82,79,80,32,84,65,66,76,69,32,116,111,32,100,101,108,101,116,101,32,116,97,98,108,101,32,37,115,0,117,115,101,32,68,82,79,80,32,86,73,69,87,32,116,111,32,100,101,108,101,116,101,32,118,105,101,119,32,37,115,0,116,98,108,0,115,113,108,105,116,101,95,115,116,97,116,37,100,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,37,115,61,37,81,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,32,87,72,69,82,69,32,110,97,109,101,61,37,81,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,37,81,32,97,110,100,32,116,121,112,101,33,61,39,116,114,105,103,103,101,114,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,110,97,109,101,61,37,81,32,65,78,68,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,114,111,111,116,112,97,103,101,61,37,100,32,87,72,69,82,69,32,35,37,100,32,65,78,68,32,114,111,111,116,112,97,103,101,61,35,37,100,0,112,97,114,97,109,101,116,101,114,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,118,105,101,119,115,0,101,120,112,101,99,116,101,100,32,37,100,32,99,111,108,117,109,110,115,32,102,111,114,32,39,37,115,39,32,98,117,116,32,103,111,116,32,37,100,0,68,73,83,84,73,78,67,84,0,71,82,79,85,80,32,66,89,0,82,73,71,72,84,32,80,65,82,84,32,79,70,32,79,82,68,69,82,32,66,89,0,79,82,68,69,82,32,66,89,0,111,110,108,121,32,97,32,115,105,110,103,108,101,32,114,101,115,117,108,116,32,97,108,108,111,119,101,100,32,102,111,114,32,97,32,83,69,76,69,67,84,32,116,104,97,116,32,105,115,32,112,97,114,116,32,111,102,32,97,110,32,101,120,112,114,101,115,115,105,111,110,0,79,82,68,69,82,32,66,89,32,99,108,97,117,115,101,32,115,104,111,117,108,100,32,99,111,109,101,32,97,102,116,101,114,32,37,115,32,110,111,116,32,98,101,102,111,114,101,0,76,73,77,73,84,32,99,108,97,117,115,101,32,115,104,111,117,108,100,32,99,111,109,101,32,97,102,116,101,114,32,37,115,32,110,111,116,32,98,101,102,111,114,101,0,114,101,99,117,114,115,105,118,101,32,97,103,103,114,101,103,97,116,101,32,113,117,101,114,105,101,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,67,79,77,80,79,85,78,68,32,83,85,66,81,85,69,82,73,69,83,32,37,100,32,65,78,68,32,37,100,32,37,115,40,37,115,41,0,85,83,73,78,71,32,84,69,77,80,32,66,45,84,82,69,69,32,0,97,116,32,109,111,115,116,32,37,100,32,116,97,98,108,101,115,32,105,110,32,97,32,106,111,105,110,0,116,111,111,32,109,97,110,121,32,97,114,103,117,109,101,110,116,115,32,111,110,32,37,115,40,41,32,45,32,109,97,120,32,37,100,0,83,81,78,79,67,65,83,69,0,109,97,116,99,104,0,103,108,111,98,0,108,105,107,101,0,114,101,103,101,120,112,0,37,115,46,120,66,101,115,116,73,110,100,101,120,40,41,32,109,97,108,102,117,110,99,116,105,111,110,0,116,97,98,108,101,32,37,115,58,32,120,66,101,115,116,73,110,100,101,120,32,114,101,116,117,114,110,101,100,32,97,110,32,105,110,118,97,108,105,100,32,112,108,97,110,0,10,10,9,9,8,8,7,7,7,6,6,6,5,5,5,4,4,4,4,3,3,3,3,3,3,2,2,2,2,2,2,2,110,111,32,113,117,101,114,121,32,115,111,108,117,116,105,111,110,0,97,117,116,111,109,97,116,105,99,32,105,110,100,101,120,32,111,110,32,37,115,40,37,115,41,0,97,117,116,111,45,105,110,100,101,120,0,83,69,65,82,67,72,0,83,67,65,78,0,32,83,85,66,81,85,69,82,89,32,37,100,0,32,84,65,66,76,69,32,37,115,0,32,65,83,32,37,115,0,80,82,73,77,65,82,89,32,75,69,89,0,65,85,84,79,77,65,84,73,67,32,80,65,82,84,73,65,76,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,0,65,85,84,79,77,65,84,73,67,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,0,67,79,86,69,82,73,78,71,32,73,78,68,69,88,32,37,115,0,73,78,68,69,88,32,37,115,0,32,85,83,73,78,71,32,0,61,0,62,63,32,65,78,68,32,114,111,119,105,100,60,0,62,0,60,0,32,85,83,73,78,71,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,32,40,114,111,119,105,100,37,115,63,41,0,32,86,73,82,84,85,65,76,32,84,65,66,76,69,32,73,78,68,69,88,32,37,100,58,37,115,0,32,40,0,32,65,78,68,32,0,37,115,61,63,0,65,78,89,40,37,115,41,0,60,101,120,112,114,62,0,66,64,63,65,0,0,108,105,66,63,65,64,117,115,114,116,7,6,108,105,85,83,69,32,84,69,77,80,32,66,45,84,82,69,69,32,70,79,82,32,37,115,0,68,73,83,84,73,78,67,84,32,97,103,103,114,101,103,97,116,101,115,32,109,117,115,116,32,104,97,118,101,32,101,120,97,99,116,108,121,32,111,110,101,32,97,114,103,117,109,101,110,116,0,83,67,65,78,32,84,65,66,76,69,32,37,115,37,115,37,115,0,32,85,83,73,78,71,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,32,0,109,105,110,0,109,97,120,0,99,111,108,117,109,110,37,100,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,99,111,109,112,111,117,110,100,32,83,69,76,69,67,84,0,97,32,74,79,73,78,32,99,108,97,117,115,101,32,105,115,32,114,101,113,117,105,114,101,100,32,98,101,102,111,114,101,32,37,115,0,79,78,0,85,83,73,78,71,0,110,97,116,117,114,97,108,101,102,116,111,117,116,101,114,105,103,104,116,102,117,108,108,105,110,110,101,114,99,114,111,115,115,0,0,7,4,6,4,40,10,5,32,14,5,48,19,4,56,23,5,1,28,5,3,32,0,117,110,107,110,111,119,110,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,32,106,111,105,110,32,116,121,112,101,58,32,37,84,32,37,84,37,115,37,84,0,82,73,71,72,84,32,97,110,100,32,70,85,76,76,32,79,85,84,69,82,32,74,79,73,78,115,32,97,114,101,32,110,111,116,32,99,117,114,114,101,110,116,108,121,32,115,117,112,112,111,114,116,101,100,0,114,111,119,115,32,100,101,108,101,116,101,100,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,109,111,100,105,102,105,101,100,0,99,97,110,110,111,116,32,109,111,100,105,102,121,32,37,115,32,98,101,99,97,117,115,101,32,105,116,32,105,115,32,97,32,118,105,101,119,0,45,45,32,84,82,73,71,71,69,82,32,37,115,0,102,111,114,101,105,103,110,32,107,101,121,32,109,105,115,109,97,116,99,104,32,45,32,34,37,119,34,32,114,101,102,101,114,101,110,99,105,110,103,32,34,37,119,34,0,77,0,2,0,74,0,0,0,49,0,2,0,75,0,0,0,61,0,0,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,105,110,32,37,115,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,58,32,37,115,0,114,111,119,115,32,117,112,100,97,116,101,100,0,37,115,46,114,111,119,105,100,0,116,97,98,108,101,32,37,83,32,104,97,115,32,110,111,32,99,111,108,117,109,110,32,110,97,109,101,100,32,37,115,0,116,97,98,108,101,32,37,83,32,104,97,115,32,37,100,32,99,111,108,117,109,110,115,32,98,117,116,32,37,100,32,118,97,108,117,101,115,32,119,101,114,101,32,115,117,112,112,108,105,101,100,0,37,100,32,118,97,108,117,101,115,32,102,111,114,32,37,100,32,99,111,108,117,109,110,115,0,114,111,119,115,32,105,110,115,101,114,116,101,100,0,118,97,114,105,97,98,108,101,32,110,117,109,98,101,114,32,109,117,115,116,32,98,101,32,98,101,116,119,101,101,110,32,63,49,32,97,110,100,32,63,37,100,0,116,111,111,32,109,97,110,121,32,83,81,76,32,118,97,114,105,97,98,108,101,115,0,48,0,49,0,115,121,110,116,97,120,32,101,114,114,111,114,32,97,102,116,101,114,32,99,111,108,117,109,110,32,110,97,109,101,32,34,37,46,42,115,34,0,110,111,32,115,117,99,104,32,105,110,100,101,120,58,32,37,83,0,105,110,100,101,120,32,97,115,115,111,99,105,97,116,101,100,32,119,105,116,104,32,85,78,73,81,85,69,32,111,114,32,80,82,73,77,65,82,89,32,75,69,89,32,99,111,110,115,116,114,97,105,110,116,32,99,97,110,110,111,116,32,98,101,32,100,114,111,112,112,101,100,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,110,97,109,101,61,37,81,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,0,105,100,120,0,45,37,84,0,114,101,115,117,108,116,0,2,0,0,0,51,0,1,3,138,1,8,0,22,0,2,0,90,1,2,1,138,1,8,0,22,0,1,0,160,0,0,0,33,1,1,0,99,97,99,104,101,95,115,105,122,101,0,112,97,103,101,95,115,105,122,101,0,115,101,99,117,114,101,95,100,101,108,101,116,101,0,110,111,114,109,97,108,0,101,120,99,108,117,115,105,118,101,0,108,111,99,107,105,110,103,95,109,111,100,101,0,106,111,117,114,110,97,108,95,109,111,100,101,0,106,111,117,114,110,97,108,95,115,105,122,101,95,108,105,109,105,116,0,97,117,116,111,95,118,97,99,117,117,109,0,2,0,1,0,51,0,1,4,45,1,0,0,21,0,2,0,52,0,7,0,99,97,99,104,101,95,115,112,105,108,108,0,109,109,97,112,95,115,105,122,101,0,116,101,109,112,95,115,116,111,114,101,0,116,101,109,112,95,115,116,111,114,101,95,100,105,114,101,99,116,111,114,121,0,110,111,116,32,97,32,119,114,105,116,97,98,108,101,32,100,105,114,101,99,116,111,114,121,0,115,121,110,99,104,114,111,110,111,117,115,0,83,97,102,101,116,121,32,108,101,118,101,108,32,109,97,121,32,110,111,116,32,98,101,32,99,104,97,110,103,101,100,32,105,110,115,105,100,101,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,99,105,100,0,110,97,109,101,0,116,121,112,101,0,110,111,116,110,117,108,108,0,100,102,108,116,95,118,97,108,117,101,0,112,107,0,105,115,115,105,115,105,0,119,105,100,116,104,0,104,101,105,103,104,116,0,115,115,105,105,0,115,105,105,0,115,101,113,110,111,0,100,101,115,99,0,99,111,108,108,0,107,101,121,0,105,105,115,0,105,115,105,0,115,101,113,0,117,110,105,113,117,101,0,111,114,105,103,105,110,0,112,97,114,116,105,97,108,0,99,0,117,0,105,115,105,115,105,0,102,105,108,101,0,105,115,115,0,105,115,0,105,100,0,116,111,0,111,110,95,117,112,100,97,116,101,0,111,110,95,100,101,108,101,116,101,0,105,105,115,115,115,115,115,115,0,78,79,78,69,0,112,97,114,101,110,116,0,102,107,105,100,0,115,105,0,105,110,116,101,103,114,105,116,121,95,99,104,101,99,107,0,42,42,42,32,105,110,32,100,97,116,97,98,97,115,101,32,37,115,32,42,42,42,10,0,78,85,76,76,32,118,97,108,117,101,32,105,110,32,37,115,46,37,115,0,114,111,119,32,0,32,109,105,115,115,105,110,103,32,102,114,111,109,32,105,110,100,101,120,32,0,110,111,110,45,117,110,105,113,117,101,32,101,110,116,114,121,32,105,110,32,105,110,100,101,120,32,0,119,114,111,110,103,32,35,32,111,102,32,101,110,116,114,105,101,115,32,105,110,32,105,110,100,101,120,32,0,37,1,0,0,45,1,4,0,97,0,3,0,33,3,1,0,111,107,0,85,84,70,56,0,85,84,70,45,56,0,85,84,70,45,49,54,108,101,0,85,84,70,45,49,54,98,101,0,85,84,70,49,54,108,101,0,85,84,70,49,54,98,101,0,85,84,70,45,49,54,0,85,84,70,49,54,0,101,110,99,111,100,105,110,103,0,117,110,115,117,112,112,111,114,116,101,100,32,101,110,99,111,100,105,110,103,58,32,37,115,0,2,0,1,0,52,0,0,0,2,0,0,0,51,0,1,0,33,1,1,0,99,111,109,112,105,108,101,95,111,112,116,105,111,110,0,98,117,115,121,0,108,111,103,0,99,104,101,99,107,112,111,105,110,116,101,100,0,102,117,108,108,0,114,101,115,116,97,114,116,0,119,97,108,95,97,117,116,111,99,104,101,99,107,112,111,105,110,116,0,116,105,109,101,111,117,116,0,115,111,102,116,95,104,101,97,112,95,108,105,109,105,116,0,116,104,114,101,97,100,115,0,97,112,112,108,105,99,97,116,105,111,110,95,105,100,0,97,117,116,111,109,97,116,105,99,95,105,110,100,101,120,0,98,117,115,121,95,116,105,109,101,111,117,116,0,99,97,115,101,95,115,101,110,115,105,116,105,118,101,95,108,105,107,101,0,99,101,108,108,95,115,105,122,101,95,99,104,101,99,107,0,99,104,101,99,107,112,111,105,110,116,95,102,117,108,108,102,115,121,110,99,0,99,111,108,108,97,116,105,111,110,95,108,105,115,116,0,99,111,109,112,105,108,101,95,111,112,116,105,111,110,115,0,99,111,117,110,116,95,99,104,97,110,103,101,115,0,100,97,116,97,95,118,101,114,115,105,111,110,0,100,97,116,97,98,97,115,101,95,108,105,115,116,0,100,101,102,97,117,108,116,95,99,97,99,104,101,95,115,105,122,101,0,100,101,102,101,114,95,102,111,114,101,105,103,110,95,107,101,121,115,0,101,109,112,116,121,95,114,101,115,117,108,116,95,99,97,108,108,98,97,99,107,115,0,102,111,114,101,105,103,110,95,107,101,121,95,99,104,101,99,107,0,102,111,114,101,105,103,110,95,107,101,121,95,108,105,115,116,0,102,111,114,101,105,103,110,95,107,101,121,115,0,102,114,101,101,108,105,115,116,95,99,111,117,110,116,0,102,117,108,108,95,99,111,108,117,109,110,95,110,97,109,101,115,0,102,117,108,108,102,115,121,110,99,0,105,103,110,111,114,101,95,99,104,101,99,107,95,99,111,110,115,116,114,97,105,110,116,115,0,105,110,99,114,101,109,101,110,116,97,108,95,118,97,99,117,117,109,0,105,110,100,101,120,95,105,110,102,111,0,105,110,100,101,120,95,108,105,115,116,0,105,110,100,101,120,95,120,105,110,102,111,0,108,101,103,97,99,121,95,102,105,108,101,95,102,111,114,109,97,116,0,109,97,120,95,112,97,103,101,95,99,111,117,110,116,0,112,97,103,101,95,99,111,117,110,116,0,113,117,101,114,121,95,111,110,108,121,0,113,117,105,99,107,95,99,104,101,99,107,0,114,101,97,100,95,117,110,99,111,109,109,105,116,116,101,100,0,114,101,99,117,114,115,105,118,101,95,116,114,105,103,103,101,114,115,0,114,101,118,101,114,115,101,95,117,110,111,114,100,101,114,101,100,95,115,101,108,101,99,116,115,0,115,99,104,101,109,97,95,118,101,114,115,105,111,110,0,115,104,111,114,116,95,99,111,108,117,109,110,95,110,97,109,101,115,0,115,104,114,105,110,107,95,109,101,109,111,114,121,0,115,116,97,116,115,0,116,97,98,108,101,95,105,110,102,111,0,117,115,101,114,95,118,101,114,115,105,111,110,0,119,97,108,95,99,104,101,99,107,112,111,105,110,116,0,119,114,105,116,97,98,108,101,95,115,99,104,101,109,97,0,110,111,110,101,0,105,110,99,114,101,109,101,110,116,97,108,0,116,101,109,112,111,114,97,114,121,32,115,116,111,114,97,103,101,32,99,97,110,110,111,116,32,98,101,32,99,104,97,110,103,101,100,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,111,110,111,102,102,97,108,115,101,121,101,115,116,114,117,101,120,116,114,97,102,117,108,108,0,0,1,2,4,9,12,15,20,2,2,3,5,3,4,5,4,1,0,0,0,1,1,3,2,83,69,84,32,78,85,76,76,0,83,69,84,32,68,69,70,65,85,76,84,0,67,65,83,67,65,68,69,0,82,69,83,84,82,73,67,84,0,78,79,32,65,67,84,73,79,78,0,37,95,0,0,76,73,75,69,32,111,114,32,71,76,79,66,32,112,97,116,116,101,114,110,32,116,111,111,32,99,111,109,112,108,101,120,0,69,83,67,65,80,69,32,101,120,112,114,101,115,115,105,111,110,32,109,117,115,116,32,98,101,32,97,32,115,105,110,103,108,101,32,99,104,97,114,97,99,116,101,114,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,37,115,32,86,65,76,85,69,83,40,39,116,114,105,103,103,101,114,39,44,37,81,44,37,81,44,48,44,39,67,82,69,65,84,69,32,84,82,73,71,71,69,82,32,37,113,39,41,0,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,65,78,68,32,110,97,109,101,61,39,37,113,39,0,116,101,109,112,111,114,97,114,121,32,116,114,105,103,103,101,114,32,109,97,121,32,110,111,116,32,104,97,118,101,32,113,117,97,108,105,102,105,101,100,32,110,97,109,101,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,116,114,105,103,103,101,114,115,32,111,110,32,118,105,114,116,117,97,108,32,116,97,98,108,101,115,0,116,114,105,103,103,101,114,32,37,84,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,116,114,105,103,103,101,114,32,111,110,32,115,121,115,116,101,109,32,116,97,98,108,101,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,37,115,32,116,114,105,103,103,101,114,32,111,110,32,118,105,101,119,58,32,37,83,0,66,69,70,79,82,69,0,65,70,84,69,82,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,73,78,83,84,69,65,68,32,79,70,32,116,114,105,103,103,101,114,32,111,110,32,116,97,98,108,101,58,32,37,83,0,110,111,32,115,117,99,104,32,116,114,105,103,103,101,114,58,32,37,83,0,115,113,108,105,116,101,95,97,116,116,97,99,104,0,116,111,111,32,109,97,110,121,32,97,116,116,97,99,104,101,100,32,100,97,116,97,98,97,115,101,115,32,45,32,109,97,120,32,37,100,0,99,97,110,110,111,116,32,65,84,84,65,67,72,32,100,97,116,97,98,97,115,101,32,119,105,116,104,105,110,32,116,114,97,110,115,97,99,116,105,111,110,0,100,97,116,97,98,97,115,101,32,37,115,32,105,115,32,97,108,114,101,97,100,121,32,105,110,32,117,115,101,0,100,97,116,97,98,97,115,101,32,105,115,32,97,108,114,101,97,100,121,32,97,116,116,97,99,104,101,100,0,97,116,116,97,99,104,101,100,32,100,97,116,97,98,97,115,101,115,32,109,117,115,116,32,117,115,101,32,116,104,101,32,115,97,109,101,32,116,101,120,116,32,101,110,99,111,100,105,110,103,32,97,115,32,109,97,105,110,32,100,97,116,97,98,97,115,101,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,100,97,116,97,98,97,115,101,58,32,37,115,0,102,105,108,101,58,0,108,111,99,97,108,104,111,115,116,0,105,110,118,97,108,105,100,32,117,114,105,32,97,117,116,104,111,114,105,116,121,58,32,37,46,42,115,0,118,102,115,0,99,97,99,104,101,0,115,104,97,114,101,100,0,112,114,105,118,97,116,101,0,109,111,100,101,0,114,111,0,114,119,0,114,119,99,0,110,111,32,115,117,99,104,32,37,115,32,109,111,100,101,58,32,37,115,0,37,115,32,109,111,100,101,32,110,111,116,32,97,108,108,111,119,101,100,58,32,37,115,0,110,111,32,115,117,99,104,32,118,102,115,58,32,37,115,0,115,113,108,105,116,101,95,100,101,116,97,99,104,0,110,111,32,115,117,99,104,32,100,97,116,97,98,97,115,101,58,32,37,115,0,99,97,110,110,111,116,32,100,101,116,97,99,104,32,100,97,116,97,98,97,115,101,32,37,115,0,99,97,110,110,111,116,32,68,69,84,65,67,72,32,100,97,116,97,98,97,115,101,32,119,105,116,104,105,110,32,116,114,97,110,115,97,99,116,105,111,110,0,100,97,116,97,98,97,115,101,32,37,115,32,105,115,32,108,111,99,107,101,100,0,117,110,97,98,108,101,32,116,111,32,105,100,101,110,116,105,102,121,32,116,104,101,32,111,98,106,101,99,116,32,116,111,32,98,101,32,114,101,105,110,100,101,120,101,100,0,116,98,108,44,105,100,120,44,115,116,97,116,0,115,113,108,105,116,101,95,115,116,97,116,51,0,115,113,108,105,116,101,95,115,116,97,116,52,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,37,115,40,37,115,41,0,115,113,108,105,116,101,95,37,0,66,66,66,0,115,116,97,116,95,105,110,105,116,0,115,116,97,116,95,112,117,115,104,0,115,116,97,116,95,103,101,116,0,37,108,108,117,0,32,37,108,108,117,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,110,111,116,104,101,114,32,116,97,98,108,101,32,111,114,32,105,110,100,101,120,32,119,105,116,104,32,116,104,105,115,32,110,97,109,101,58,32,37,115,0,118,105,101,119,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,85,80,68,65,84,69,32,34,37,119,34,46,37,115,32,83,69,84,32,115,113,108,32,61,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,112,97,114,101,110,116,40,115,113,108,44,32,37,81,44,32,37,81,41,32,87,72,69,82,69,32,37,115,59,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,115,113,108,32,61,32,67,65,83,69,32,87,72,69,78,32,116,121,112,101,32,61,32,39,116,114,105,103,103,101,114,39,32,84,72,69,78,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,40,115,113,108,44,32,37,81,41,69,76,83,69,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,97,98,108,101,40,115,113,108,44,32,37,81,41,32,69,78,68,44,32,116,98,108,95,110,97,109,101,32,61,32,37,81,44,32,110,97,109,101,32,61,32,67,65,83,69,32,87,72,69,78,32,116,121,112,101,61,39,116,97,98,108,101,39,32,84,72,69,78,32,37,81,32,87,72,69,78,32,110,97,109,101,32,76,73,75,69,32,39,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,37,37,39,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,32,84,72,69,78,32,39,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,95,39,32,124,124,32,37,81,32,124,124,32,115,117,98,115,116,114,40,110,97,109,101,44,37,100,43,49,56,41,32,69,76,83,69,32,110,97,109,101,32,69,78,68,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,37,81,32,67,79,76,76,65,84,69,32,110,111,99,97,115,101,32,65,78,68,32,40,116,121,112,101,61,39,116,97,98,108,101,39,32,79,82,32,116,121,112,101,61,39,105,110,100,101,120,39,32,79,82,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,41,59,0,85,80,68,65,84,69,32,34,37,119,34,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,32,115,101,116,32,110,97,109,101,32,61,32,37,81,32,87,72,69,82,69,32,110,97,109,101,32,61,32,37,81,0,85,80,68,65,84,69,32,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,32,83,69,84,32,115,113,108,32,61,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,40,115,113,108,44,32,37,81,41,44,32,116,98,108,95,110,97,109,101,32,61,32,37,81,32,87,72,69,82,69,32,37,115,59,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,110,97,109,101,61,37,81,0,37,115,32,79,82,32,110,97,109,101,61,37,81,0,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,65,78,68,32,40,37,115,41,0,116,98,108,95,110,97,109,101,61,37,81,0,67,97,110,110,111,116,32,97,100,100,32,97,32,80,82,73,77,65,82,89,32,75,69,89,32,99,111,108,117,109,110,0,67,97,110,110,111,116,32,97,100,100,32,97,32,85,78,73,81,85,69,32,99,111,108,117,109,110,0,67,97,110,110,111,116,32,97,100,100,32,97,32,82,69,70,69,82,69,78,67,69,83,32,99,111,108,117,109,110,32,119,105,116,104,32,110,111,110,45,78,85,76,76,32,100,101,102,97,117,108,116,32,118,97,108,117,101,0,67,97,110,110,111,116,32,97,100,100,32,97,32,78,79,84,32,78,85,76,76,32,99,111,108,117,109,110,32,119,105,116,104,32,100,101,102,97,117,108,116,32,118,97,108,117,101,32,78,85,76,76,0,67,97,110,110,111,116,32,97,100,100,32,97,32,99,111,108,117,109,110,32,119,105,116,104,32,110,111,110,45,99,111,110,115,116,97,110,116,32,100,101,102,97,117,108,116,0,85,80,68,65,84,69,32,34,37,119,34,46,37,115,32,83,69,84,32,115,113,108,32,61,32,115,117,98,115,116,114,40,115,113,108,44,49,44,37,100,41,32,124,124,32,39,44,32,39,32,124,124,32,37,81,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,37,100,41,32,87,72,69,82,69,32,116,121,112,101,32,61,32,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,32,61,32,37,81,0,118,105,114,116,117,97,108,32,116,97,98,108,101,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,67,97,110,110,111,116,32,97,100,100,32,97,32,99,111,108,117,109,110,32,116,111,32,97,32,118,105,101,119,0,115,113,108,105,116,101,95,97,108,116,101,114,116,97,98,95,37,115,0,67,82,69,65,84,69,32,86,73,82,84,85,65,76,32,84,65,66,76,69,32,37,84,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,116,121,112,101,61,39,116,97,98,108,101,39,44,32,110,97,109,101,61,37,81,44,32,116,98,108,95,110,97,109,101,61,37,81,44,32,114,111,111,116,112,97,103,101,61,48,44,32,115,113,108,61,37,81,32,87,72,69,82,69,32,114,111,119,105,100,61,35,37,100,0,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,61,39,116,97,98,108,101,39,0,100,117,112,108,105,99,97,116,101,32,87,73,84,72,32,116,97,98,108,101,32,110,97,109,101,58,32,37,115,0,108,116,114,105,109,0,114,116,114,105,109,0,116,114,105,109,0,116,121,112,101,111,102,0,108,101,110,103,116,104,0,105,110,115,116,114,0,115,117,98,115,116,114,0,112,114,105,110,116,102,0,117,110,105,99,111,100,101,0,99,104,97,114,0,97,98,115,0,114,111,117,110,100,0,117,112,112,101,114,0,108,111,119,101,114,0,99,111,97,108,101,115,99,101,0,104,101,120,0,105,102,110,117,108,108,0,117,110,108,105,107,101,108,121,0,108,105,107,101,108,105,104,111,111,100,0,108,105,107,101,108,121,0,114,97,110,100,111,109,0,114,97,110,100,111,109,98,108,111,98,0,110,117,108,108,105,102,0,115,113,108,105,116,101,95,118,101,114,115,105,111,110,0,115,113,108,105,116,101,95,115,111,117,114,99,101,95,105,100,0,115,113,108,105,116,101,95,108,111,103,0,115,113,108,105,116,101,95,99,111,109,112,105,108,101,111,112,116,105,111,110,95,117,115,101,100,0,115,113,108,105,116,101,95,99,111,109,112,105,108,101,111,112,116,105,111,110,95,103,101,116,0,113,117,111,116,101,0,108,97,115,116,95,105,110,115,101,114,116,95,114,111,119,105,100,0,99,104,97,110,103,101,115,0,116,111,116,97,108,95,99,104,97,110,103,101,115,0,114,101,112,108,97,99,101,0,122,101,114,111,98,108,111,98,0,115,117,109,0,116,111,116,97,108,0,97,118,103,0,99,111,117,110,116,0,103,114,111,117,112,95,99,111,110,99,97,116,0,1,116,101,120,116,0,98,108,111,98,0,105,110,116,101,103,101,114,32,111,118,101,114,102,108,111,119,0,37,46,42,102,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,37,33,46,50,48,101,0,106,117,108,105,97,110,100,97,121,0,100,97,116,101,0,116,105,109,101,0,100,97,116,101,116,105,109,101,0,115,116,114,102,116,105,109,101,0,99,117,114,114,101,110,116,95,116,105,109,101,0,99,117,114,114,101,110,116,95,116,105,109,101,115,116,97,109,112,0,99,117,114,114,101,110,116,95,100,97,116,101,0,110,111,119,0,52,48,102,45,50,49,97,45,50,49,100,0,50,48,99,58,50,48,101,0,50,48,101,0,50,48,98,58,50,48,101,0,108,111,99,97,108,116,105,109,101,0,117,110,105,120,101,112,111,99,104,0,117,116,99,0,119,101,101,107,100,97,121,32,0,115,116,97,114,116,32,111,102,32,0,109,111,110,116,104,0,121,101,97,114,0,100,97,121,0,104,111,117,114,0,109,105,110,117,116,101,0,115,101,99,111,110,100,0,108,111,99,97,108,32,116,105,109,101,32,117,110,97,118,97,105,108,97,98,108,101,0,37,48,52,100,45,37,48,50,100,45,37,48,50,100,0,37,48,50,100,58,37,48,50,100,58,37,48,50,100,0,37,48,52,100,45,37,48,50,100,45,37,48,50,100,32,37,48,50,100,58,37,48,50,100,58,37,48,50,100,0,37,48,50,100,0,37,48,54,46,51,102,0,37,48,51,100,0,37,48,52,100,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,97,98,108,101,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,112,97,114,101,110,116,0,37,46,42,115,34,37,119,34,37,115,0,37,115,37,46,42,115,34,37,119,34,0,102,97,105,108,101,100,32,116,111,32,97,108,108,111,99,97,116,101,32,37,117,32,98,121,116,101,115,32,111,102,32,109,101,109,111,114,121,0,102,97,105,108,101,100,32,109,101,109,111,114,121,32,114,101,115,105,122,101,32,37,117,32,116,111,32,37,117,32,98,121,116,101,115,0,117,110,97,98,108,101,32,116,111,32,99,108,111,115,101,32,100,117,101,32,116,111,32,117,110,102,105,110,97,108,105,122,101,100,32,115,116,97,116,101,109,101,110,116,115,32,111,114,32,117,110,102,105,110,105,115,104,101,100,32,98,97,99,107,117,112,115,0,110,111,116,32,97,110,32,101,114,114,111,114,0,83,81,76,32,108,111,103,105,99,32,101,114,114,111,114,32,111,114,32,109,105,115,115,105,110,103,32,100,97,116,97,98,97,115,101,0,97,99,99,101,115,115,32,112,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,99,97,108,108,98,97,99,107,32,114,101,113,117,101,115,116,101,100,32,113,117,101,114,121,32,97,98,111,114,116,0,100,97,116,97,98,97,115,101,32,105,115,32,108,111,99,107,101,100,0,100,97,116,97,98,97,115,101,32,116,97,98,108,101,32,105,115,32,108,111,99,107,101,100,0,97,116,116,101,109,112,116,32,116,111,32,119,114,105,116,101,32,97,32,114,101,97,100,111,110,108,121,32,100,97,116,97,98,97,115,101,0,105,110,116,101,114,114,117,112,116,101,100,0,100,105,115,107,32,73,47,79,32,101,114,114,111,114,0,100,97,116,97,98,97,115,101,32,100,105,115,107,32,105,109,97,103,101,32,105,115,32,109,97,108,102,111,114,109,101,100,0,117,110,107,110,111,119,110,32,111,112,101,114,97,116,105,111,110,0,100,97,116,97,98,97,115,101,32,111,114,32,100,105,115,107,32,105,115,32,102,117,108,108,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,100,97,116,97,98,97,115,101,32,102,105,108,101,0,108,111,99,107,105,110,103,32,112,114,111,116,111,99,111,108,0,116,97,98,108,101,32,99,111,110,116,97,105,110,115,32,110,111,32,100,97,116,97,0,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,100,97,116,97,116,121,112,101,32,109,105,115,109,97,116,99,104,0,108,105,98,114,97,114,121,32,114,111,117,116,105,110,101,32,99,97,108,108,101,100,32,111,117,116,32,111,102,32,115,101,113,117,101,110,99,101,0,108,97,114,103,101,32,102,105,108,101,32,115,117,112,112,111,114,116,32,105,115,32,100,105,115,97,98,108,101,100,0,97,117,116,104,111,114,105,122,97,116,105,111,110,32,100,101,110,105,101,100,0,97,117,120,105,108,105,97,114,121,32,100,97,116,97,98,97,115,101,32,102,111,114,109,97,116,32,101,114,114,111,114,0,98,105,110,100,32,111,114,32,99,111,108,117,109,110,32,105,110,100,101,120,32,111,117,116,32,111,102,32,114,97,110,103,101,0,102,105,108,101,32,105,115,32,101,110,99,114,121,112,116,101,100,32,111,114,32,105,115,32,110,111,116,32,97,32,100,97,116,97,98,97,115,101,0,117,110,107,110,111,119,110,32,101,114,114,111,114,0,97,98,111,114,116,32,100,117,101,32,116,111,32,82,79,76,76,66,65,67,75,0,117,110,97,98,108,101,32,116,111,32,100,101,108,101,116,101,47,109,111,100,105,102,121,32,117,115,101,114,45,102,117,110,99,116,105,111,110,32,100,117,101,32,116,111,32,97,99,116,105,118,101,32,115,116,97,116,101,109,101,110,116,115,0,117,110,97,98,108,101,32,116,111,32,117,115,101,32,102,117,110,99,116,105,111,110,32,37,115,32,105,110,32,116,104,101,32,114,101,113,117,101,115,116,101,100,32,99,111,110,116,101,120,116,0,105,110,118,97,108,105,100,0,82,84,82,73,77,0,109,97,105,110,0,77,65,84,67,72,0,97,117,116,111,109,97,116,105,99,32,101,120,116,101,110,115,105,111,110,32,108,111,97,100,105,110,103,32,102,97,105,108,101,100,58,32,37,115,0,115,105,109,112,108,101,0,112,111,114,116,101,114,0,117,110,105,99,111,100,101,54,49,0,102,116,115,51,95,116,111,107,101,110,105,122,101,114,0,115,110,105,112,112,101,116,0,111,102,102,115,101,116,115,0,109,97,116,99,104,105,110,102,111,0,111,112,116,105,109,105,122,101,0,102,116,115,51,0,102,116,115,52,0,114,101,109,111,118,101,95,100,105,97,99,114,105,116,105,99,115,61,49,0,114,101,109,111,118,101,95,100,105,97,99,114,105,116,105,99,115,61,48,0,116,111,107,101,110,99,104,97,114,115,61,0,115,101,112,97,114,97,116,111,114,115,61,0,0,97,99,101,105,110,111,117,121,121,97,99,100,101,101,103,104,105,106,107,108,110,111,114,115,116,117,117,119,121,122,111,117,97,105,111,117,103,107,111,106,103,110,97,101,105,111,114,117,115,116,104,97,101,111,121,0,0,0,0,0,0,0,0,97,98,100,100,101,102,103,104,104,105,107,108,108,109,110,112,114,114,115,116,117,118,119,119,120,121,122,104,116,119,121,97,101,105,111,117,121,102,116,115,52,97,117,120,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,116,101,114,109,44,32,99,111,108,44,32,100,111,99,117,109,101,110,116,115,44,32,111,99,99,117,114,114,101,110,99,101,115,44,32,108,97,110,103,117,97,103,101,105,100,32,72,73,68,68,69,78,41,0,105,110,118,97,108,105,100,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,116,115,52,97,117,120,32,99,111,110,115,116,114,117,99,116,111,114,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,87,72,69,82,69,32,114,111,119,105,100,32,61,32,63,0,83,69,76,69,67,84,32,78,79,84,32,69,88,73,83,84,83,40,83,69,76,69,67,84,32,100,111,99,105,100,32,70,82,79,77,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,87,72,69,82,69,32,114,111,119,105,100,33,61,63,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,116,97,116,39,0,83,69,76,69,67,84,32,37,115,32,87,72,69,82,69,32,114,111,119,105,100,61,63,0,83,69,76,69,67,84,32,40,83,69,76,69,67,84,32,109,97,120,40,105,100,120,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,41,32,43,32,49,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,40,98,108,111,99,107,105,100,44,32,98,108,111,99,107,41,32,86,65,76,85,69,83,40,63,44,32,63,41,0,83,69,76,69,67,84,32,99,111,97,108,101,115,99,101,40,40,83,69,76,69,67,84,32,109,97,120,40,98,108,111,99,107,105,100,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,41,32,43,32,49,44,32,49,41,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,86,65,76,85,69,83,40,63,44,63,44,63,44,63,44,63,44,63,41,0,83,69,76,69,67,84,32,105,100,120,44,32,115,116,97,114,116,95,98,108,111,99,107,44,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,44,32,101,110,100,95,98,108,111,99,107,44,32,114,111,111,116,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,79,82,68,69,82,32,66,89,32,105,100,120,32,65,83,67,0,83,69,76,69,67,84,32,105,100,120,44,32,115,116,97,114,116,95,98,108,111,99,107,44,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,44,32,101,110,100,95,98,108,111,99,107,44,32,114,111,111,116,32,70,82,79,77,32,37], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+30720);
/* memory initializer */ allocate([81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,79,82,68,69,82,32,66,89,32,108,101,118,101,108,32,68,69,83,67,44,32,105,100,120,32,65,83,67,0,83,69,76,69,67,84,32,99,111,117,110,116,40,42,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,0,83,69,76,69,67,84,32,109,97,120,40,108,101,118,101,108,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,32,87,72,69,82,69,32,98,108,111,99,107,105,100,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,86,65,76,85,69,83,40,37,115,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,87,72,69,82,69,32,100,111,99,105,100,32,61,32,63,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,86,65,76,85,69,83,40,63,44,63,41,0,83,69,76,69,67,84,32,115,105,122,101,32,70,82,79,77,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,87,72,69,82,69,32,100,111,99,105,100,61,63,0,83,69,76,69,67,84,32,118,97,108,117,101,32,70,82,79,77,32,37,81,46,39,37,113,95,115,116,97,116,39,32,87,72,69,82,69,32,105,100,61,63,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,115,116,97,116,39,32,86,65,76,85,69,83,40,63,44,63,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,0,83,69,76,69,67,84,32,63,32,85,78,73,79,78,32,83,69,76,69,67,84,32,108,101,118,101,108,32,47,32,40,49,48,50,52,32,42,32,63,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,83,69,76,69,67,84,32,108,101,118,101,108,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,71,82,79,85,80,32,66,89,32,108,101,118,101,108,32,72,65,86,73,78,71,32,99,111,117,110,116,40,42,41,62,61,63,32,32,79,82,68,69,82,32,66,89,32,40,108,101,118,101,108,32,37,37,32,49,48,50,52,41,32,65,83,67,32,76,73,77,73,84,32,49,0,83,69,76,69,67,84,32,50,32,42,32,116,111,116,97,108,40,49,32,43,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,32,45,32,115,116,97,114,116,95,98,108,111,99,107,41,32,32,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,60,32,63,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,61,32,63,0,85,80,68,65,84,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,105,100,120,32,61,32,63,32,87,72,69,82,69,32,108,101,118,101,108,61,63,32,65,78,68,32,105,100,120,61,63,0,83,69,76,69,67,84,32,105,100,120,44,32,115,116,97,114,116,95,98,108,111,99,107,44,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,44,32,101,110,100,95,98,108,111,99,107,44,32,114,111,111,116,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,61,32,63,0,85,80,68,65,84,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,115,116,97,114,116,95,98,108,111,99,107,32,61,32,63,44,32,114,111,111,116,32,61,32,63,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,61,32,63,0,83,69,76,69,67,84,32,49,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,32,87,72,69,82,69,32,98,108,111,99,107,105,100,61,63,32,65,78,68,32,98,108,111,99,107,32,73,83,32,78,85,76,76,0,83,69,76,69,67,84,32,105,100,120,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,61,63,32,79,82,68,69,82,32,66,89,32,49,32,65,83,67,0,83,69,76,69,67,84,32,109,97,120,40,32,108,101,118,101,108,32,37,37,32,49,48,50,52,32,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,83,69,76,69,67,84,32,108,101,118,101,108,44,32,105,100,120,44,32,101,110,100,95,98,108,111,99,107,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,32,79,82,68,69,82,32,66,89,32,108,101,118,101,108,32,68,69,83,67,44,32,105,100,120,32,65,83,67,0,85,80,68,65,84,69,32,79,82,32,70,65,73,76,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,108,101,118,101,108,61,45,49,44,105,100,120,61,63,32,87,72,69,82,69,32,108,101,118,101,108,61,63,32,65,78,68,32,105,100,120,61,63,0,85,80,68,65,84,69,32,79,82,32,70,65,73,76,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,108,101,118,101,108,61,63,32,87,72,69,82,69,32,108,101,118,101,108,61,45,49,0,37,115,95,115,101,103,109,101,110,116,115,0,98,108,111,99,107,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,115,101,115,115,0,115,115,0,115,101,105,0,105,0,100,101,101,0,101,101,0,103,110,105,0,100,101,0,116,97,0,97,116,101,0,108,98,0,98,108,101,0,122,105,0,105,122,101,0,108,97,110,111,105,116,97,0,108,97,110,111,105,116,0,116,105,111,110,0,105,99,110,101,0,101,110,99,101,0,105,99,110,97,0,97,110,99,101,0,114,101,122,105,0,105,103,111,108,0,105,108,98,0,105,108,108,97,0,97,108,0,105,108,116,110,101,0,101,110,116,0,105,108,101,0,101,0,105,108,115,117,111,0,111,117,115,0,110,111,105,116,97,122,105,0,110,111,105,116,97,0,114,111,116,97,0,109,115,105,108,97,0,115,115,101,110,101,118,105,0,105,118,101,0,115,115,101,110,108,117,102,0,102,117,108,0,115,115,101,110,115,117,111,0,105,116,105,108,97,0,105,116,105,118,105,0,105,116,105,108,105,98,0,101,116,97,99,105,0,105,99,0,101,118,105,116,97,0,101,122,105,108,97,0,105,116,105,99,105,0,108,97,99,105,0,108,117,102,0,115,115,101,110,0,116,110,101,109,101,0,116,110,101,109,0,116,110,101,0,110,111,105,0,101,116,97,0,105,116,105,0,0,1,1,1,0,1,1,1,0,1,1,1,1,1,0,1,1,1,1,1,0,1,1,1,2,1,102,116,115,51,116,111,107,101,110,105,122,101,58,32,100,105,115,97,98,108,101,100,32,45,32,114,101,98,117,105,108,100,32,119,105,116,104,32,45,68,83,81,76,73,84,69,95,69,78,65,66,76,69,95,70,84,83,51,95,84,79,75,69,78,73,90,69,82,0,117,110,107,110,111,119,110,32,116,111,107,101,110,105,122,101,114,58,32,37,115,0,116,111,107,101,110,105,122,101,0,112,114,101,102,105,120,0,99,111,109,112,114,101,115,115,0,117,110,99,111,109,112,114,101,115,115,0,99,111,110,116,101,110,116,0,108,97,110,103,117,97,103,101,105,100,0,110,111,116,105,110,100,101,120,101,100,0,117,110,114,101,99,111,103,110,105,122,101,100,32,112,97,114,97,109,101,116,101,114,58,32,37,115,0,117,110,114,101,99,111,103,110,105,122,101,100,32,109,97,116,99,104,105,110,102,111,58,32,37,115,0,97,115,99,0,117,110,114,101,99,111,103,110,105,122,101,100,32,111,114,100,101,114,58,32,37,115,0,101,114,114,111,114,32,112,97,114,115,105,110,103,32,112,114,101,102,105,120,32,112,97,114,97,109,101,116,101,114,58,32,37,115,0,109,105,115,115,105,110,103,32,37,115,32,112,97,114,97,109,101,116,101,114,32,105,110,32,102,116,115,52,32,99,111,110,115,116,114,117,99,116,111,114,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,117,110,107,110,111,119,110,32,116,111,107,101,110,105,122,101,114,0,83,69,76,69,67,84,32,42,32,70,82,79,77,32,37,81,46,37,81,0,100,111,99,105,100,0,44,37,115,40,120,46,39,99,37,100,37,113,39,41,0,44,32,120,46,37,81,0,108,97,110,103,105,100,0,44,32,120,46,39,37,113,39,0,32,70,82,79,77,32,39,37,113,39,46,39,37,113,37,115,39,32,65,83,32,120,0,95,99,111,110,116,101,110,116,0,44,37,115,40,63,41,0,44,32,63,0,100,111,99,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,0,37,122,44,32,39,99,37,100,37,113,39,0,37,122,44,32,108,97,110,103,105,100,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,40,37,115,41,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,40,98,108,111,99,107,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,44,32,98,108,111,99,107,32,66,76,79,66,41,59,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,40,108,101,118,101,108,32,73,78,84,69,71,69,82,44,105,100,120,32,73,78,84,69,71,69,82,44,115,116,97,114,116,95,98,108,111,99,107,32,73,78,84,69,71,69,82,44,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,32,73,78,84,69,71,69,82,44,101,110,100,95,98,108,111,99,107,32,73,78,84,69,71,69,82,44,114,111,111,116,32,66,76,79,66,44,80,82,73,77,65,82,89,32,75,69,89,40,108,101,118,101,108,44,32,105,100,120,41,41,59,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,40,100,111,99,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,44,32,115,105,122,101,32,66,76,79,66,41,59,0,67,82,69,65,84,69,32,84,65,66,76,69,32,73,70,32,78,79,84,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,116,97,116,39,40,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,44,32,118,97,108,117,101,32,66,76,79,66,41,59,0,80,82,65,71,77,65,32,37,81,46,112,97,103,101,95,115,105,122,101,0,95,95,108,97,110,103,105,100,0,37,81,44,32,0,37,122,37,81,44,32,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,37,115,32,37,81,32,72,73,68,68,69,78,44,32,100,111,99,105,100,32,72,73,68,68,69,78,44,32,37,81,32,72,73,68,68,69,78,41,0,68,69,83,67,0,65,83,67,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,116,97,116,39,0,83,69,76,69,67,84,32,37,115,32,87,72,69,82,69,32,114,111,119,105,100,32,66,69,84,87,69,69,78,32,37,108,108,100,32,65,78,68,32,37,108,108,100,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,32,37,115,0,83,69,76,69,67,84,32,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,32,37,115,0,70,84,83,32,101,120,112,114,101,115,115,105,111,110,32,116,114,101,101,32,105,115,32,116,111,111,32,108,97,114,103,101,32,40,109,97,120,105,109,117,109,32,100,101,112,116,104,32,37,100,41,0,109,97,108,102,111,114,109,101,100,32,77,65,84,67,72,32,101,120,112,114,101,115,115,105,111,110,58,32,91,37,115,93,0,79,82,0,65,78,68,0,78,79,84,0,78,69,65,82,0,83,69,76,69,67,84,32,37,115,32,87,72,69,82,69,32,114,111,119,105,100,32,61,32,63,0,114,101,98,117,105,108,100,0,105,110,116,101,103,114,105,116,121,45,99,104,101,99,107,0,109,101,114,103,101,61,0,97,117,116,111,109,101,114,103,101,61,0,37,108,108,100,32,37,108,108,100,0,83,69,76,69,67,84,32,37,115,0,83,69,76,69,67,84,32,49,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,39,37,113,95,115,116,97,116,39,0,60,98,62,0,60,47,98,62,0,60,98,62,46,46,46,60,47,98,62,0,119,114,111,110,103,32,110,117,109,98,101,114,32,111,102,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,117,110,99,116,105,111,110,32,115,110,105,112,112,101,116,40,41,0,105,108,108,101,103,97,108,32,102,105,114,115,116,32,97,114,103,117,109,101,110,116,32,116,111,32,37,115,0,37,100,32,37,100,32,37,100,32,37,100,32,0,73,110,100,101,120,32,111,112,116,105,109,105,122,101,100,0,73,110,100,101,120,32,97,108,114,101,97,100,121,32,111,112,116,105,109,97,108,0,83,65,86,69,80,79,73,78,84,32,102,116,115,51,0,82,69,76,69,65,83,69,32,102,116,115,51,0,82,79,76,76,66,65,67,75,32,84,79,32,102,116,115,51,0,112,99,120,0,117,110,114,101,99,111,103,110,105,122,101,100,32,109,97,116,99,104,105,110,102,111,32,114,101,113,117,101,115,116,58,32,37,99,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,99,111,110,116,101,110,116,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,100,111,99,115,105,122,101,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,116,97,116,39,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,115,116,97,116,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,115,101,103,109,101,110,116,115,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,115,101,103,100,105,114,39,59,0,102,116,115,51,116,111,107,101,110,105,122,101,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,105,110,112,117,116,44,32,116,111,107,101,110,44,32,115,116,97,114,116,44,32,101,110,100,44,32,112,111,115,105,116,105,111,110,41,0,117,110,97,98,108,101,32,116,111,32,100,101,108,101,116,101,47,109,111,100,105,102,121,32,99,111,108,108,97,116,105,111,110,32,115,101,113,117,101,110,99,101,32,100,117,101,32,116,111,32,97,99,116,105,118,101,32,115,116,97,116,101,109,101,110,116,115,0,109,105,115,117,115,101,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,116,121,112,101,32,116,101,120,116,44,110,97,109,101,32,116,101,120,116,44,116,98,108,95,110,97,109,101,32,116,101,120,116,44,114,111,111,116,112,97,103,101,32,105,110,116,101,103,101,114,44,115,113,108,32,116,101,120,116,41,0,117,110,115,117,112,112,111,114,116,101,100,32,102,105,108,101,32,102,111,114,109,97,116,0,83,69,76,69,67,84,32,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,70,82,79,77,32,34,37,119,34,46,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,0,95,82,79,87,73,68,95,0,79,73,68,0,84,33,34,25,13,1,2,3,17,75,28,12,16,4,11,29,18,30,39,104,110,111,112,113,98,32,5,6,15,19,20,21,26,8,22,7,40,36,23,24,9,10,14,27,31,37,35,131,130,125,38,42,43,60,61,62,63,67,71,74,77,88,89,90,91,92,93,94,95,96,97,99,100,101,102,103,105,106,107,108,114,115,116,121,122,123,124,0,73,108,108,101,103,97,108,32,98,121,116,101,32,115,101,113,117,101,110,99,101,0,68,111,109,97,105,110,32,101,114,114,111,114,0,82,101,115,117,108,116,32,110,111,116,32,114,101,112,114,101,115,101,110,116,97,98,108,101,0,78,111,116,32,97,32,116,116,121,0,80,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,79,112,101,114,97,116,105,111,110,32,110,111,116,32,112,101,114,109,105,116,116,101,100,0,78,111,32,115,117,99,104,32,102,105,108,101,32,111,114,32,100,105,114,101,99,116,111,114,121,0,78,111,32,115,117,99,104,32,112,114,111,99,101,115,115,0,70,105,108,101,32,101,120,105,115,116,115,0,86,97,108,117,101,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,100,97,116,97,32,116,121,112,101,0,78,111,32,115,112,97,99,101,32,108,101,102,116,32,111,110,32,100,101,118,105,99,101,0,79,117,116,32,111,102,32,109,101,109,111,114,121,0,82,101,115,111,117,114,99,101,32,98,117,115,121,0,73,110,116,101,114,114,117,112,116,101,100,32,115,121,115,116,101,109,32,99,97,108,108,0,82,101,115,111,117,114,99,101,32,116,101,109,112,111,114,97,114,105,108,121,32,117,110,97,118,97,105,108,97,98,108,101,0,73,110,118,97,108,105,100,32,115,101,101,107,0,67,114,111,115,115,45,100,101,118,105,99,101,32,108,105,110,107,0,82,101,97,100,45,111,110,108,121,32,102,105,108,101,32,115,121,115,116,101,109,0,68,105,114,101,99,116,111,114,121,32,110,111,116,32,101,109,112,116,121,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,112,101,101,114,0,79,112,101,114,97,116,105,111,110,32,116,105,109,101,100,32,111,117,116,0,67,111,110,110,101,99,116,105,111,110,32,114,101,102,117,115,101,100,0,72,111,115,116,32,105,115,32,100,111,119,110,0,72,111,115,116,32,105,115,32,117,110,114,101,97,99,104,97,98,108,101,0,65,100,100,114,101,115,115,32,105,110,32,117,115,101,0,66,114,111,107,101,110,32,112,105,112,101,0,73,47,79,32,101,114,114,111,114,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,32,111,114,32,97,100,100,114,101,115,115,0,66,108,111,99,107,32,100,101,118,105,99,101,32,114,101,113,117,105,114,101,100,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,0,78,111,116,32,97,32,100,105,114,101,99,116,111,114,121,0,73,115,32,97,32,100,105,114,101,99,116,111,114,121,0,84,101,120,116,32,102,105,108,101,32,98,117,115,121,0,69,120,101,99,32,102,111,114,109,97,116,32,101,114,114,111,114,0,73,110,118,97,108,105,100,32,97,114,103,117,109,101,110,116,0,65,114,103,117,109,101,110,116,32,108,105,115,116,32,116,111,111,32,108,111,110,103,0,83,121,109,98,111,108,105,99,32,108,105,110,107,32,108,111,111,112,0,70,105,108,101,110,97,109,101,32,116,111,111,32,108,111,110,103,0,84,111,111,32,109,97,110,121,32,111,112,101,110,32,102,105,108,101,115,32,105,110,32,115,121,115,116,101,109,0,78,111,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,115,32,97,118,97,105,108,97,98,108,101,0,66,97,100,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,0,78,111,32,99,104,105,108,100,32,112,114,111,99,101,115,115,0,66,97,100,32,97,100,100,114,101,115,115,0,70,105,108,101,32,116,111,111,32,108,97,114,103,101,0,84,111,111,32,109,97,110,121,32,108,105,110,107,115,0,78,111,32,108,111,99,107,115,32,97,118,97,105,108,97,98,108,101,0,82,101,115,111,117,114,99,101,32,100,101,97,100,108,111,99,107,32,119,111,117,108,100,32,111,99,99,117,114,0,83,116,97,116,101,32,110,111,116,32,114,101,99,111,118,101,114,97,98,108,101,0,80,114,101,118,105,111,117,115,32,111,119,110,101,114,32,100,105,101,100,0,79,112,101,114,97,116,105,111,110,32,99,97,110,99,101,108,101,100,0,70,117,110,99,116,105,111,110,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,0,78,111,32,109,101,115,115,97,103,101,32,111,102,32,100,101,115,105,114,101,100,32,116,121,112,101,0,73,100,101,110,116,105,102,105,101,114,32,114,101,109,111,118,101,100,0,68,101,118,105,99,101,32,110,111,116,32,97,32,115,116,114,101,97,109,0,78,111,32,100,97,116,97,32,97,118,97,105,108,97,98,108,101,0,68,101,118,105,99,101,32,116,105,109,101,111,117,116,0,79,117,116,32,111,102,32,115,116,114,101,97,109,115,32,114,101,115,111,117,114,99,101,115,0,76,105,110,107,32,104,97,115,32,98,101,101,110,32,115,101,118,101,114,101,100,0,80,114,111,116,111,99,111,108,32,101,114,114,111,114,0,66,97,100,32,109,101,115,115,97,103,101,0,70,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,105,110,32,98,97,100,32,115,116,97,116,101,0,78,111,116,32,97,32,115,111,99,107,101,116,0,68,101,115,116,105,110,97,116,105,111,110,32,97,100,100,114,101,115,115,32,114,101,113,117,105,114,101,100,0,77,101,115,115,97,103,101,32,116,111,111,32,108,97,114,103,101,0,80,114,111,116,111,99,111,108,32,119,114,111,110,103,32,116,121,112,101,32,102,111,114,32,115,111,99,107,101,116,0,80,114,111,116,111,99,111,108,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,80,114,111,116,111,99,111,108,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,83,111,99,107,101,116,32,116,121,112,101,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,78,111,116,32,115,117,112,112,111,114,116,101,100,0,80,114,111,116,111,99,111,108,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,65,100,100,114,101,115,115,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,112,114,111,116,111,99,111,108,0,65,100,100,114,101,115,115,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,78,101,116,119,111,114,107,32,105,115,32,100,111,119,110,0,78,101,116,119,111,114,107,32,117,110,114,101,97,99,104,97,98,108,101,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,110,101,116,119,111,114,107,0,67,111,110,110,101,99,116,105,111,110,32,97,98,111,114,116,101,100,0,78,111,32,98,117,102,102,101,114,32,115,112,97,99,101,32,97,118,97,105,108,97,98,108,101,0,83,111,99,107,101,116,32,105,115,32,99,111,110,110,101,99,116,101,100,0,83,111,99,107,101,116,32,110,111,116,32,99,111,110,110,101,99,116,101,100,0,67,97,110,110,111,116,32,115,101,110,100,32,97,102,116,101,114,32,115,111,99,107,101,116,32,115,104,117,116,100,111,119,110,0,79,112,101,114,97,116,105,111,110,32,97,108,114,101,97,100,121,32,105,110,32,112,114,111,103,114,101,115,115,0,79,112,101,114,97,116,105,111,110,32,105,110,32,112,114,111,103,114,101,115,115,0,83,116,97,108,101,32,102,105,108,101,32,104,97,110,100,108,101,0,82,101,109,111,116,101,32,73,47,79,32,101,114,114,111,114,0,81,117,111,116,97,32,101,120,99,101,101,100,101,100,0,78,111,32,109,101,100,105,117,109,32,102,111,117,110,100,0,87,114,111,110,103,32,109,101,100,105,117,109,32,116,121,112,101,0,78,111,32,101,114,114,111,114,32,105,110,102,111,114,109,97,116,105,111,110,0,0,47,112,114,111,99,47,115,101,108,102,47,102,100,47], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+40960);





/* no memory initializer */
var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}

// {{PRE_LIBRARY}}


   
  Module["_i64Subtract"] = _i64Subtract;

  var _DtoILow=true;

  
  function ___setErrNo(value) {
      if (Module['___errno_location']) HEAP32[((Module['___errno_location']())>>2)]=value;
      return value;
    }
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 85: return totalMemory / PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 79:
          return 0;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  
  var PROCINFO={ppid:1,pid:42,sid:42,pgid:42};
  
  
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            return ''; // an invalid portion invalidates the whole thing
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          stream.tty.ops.flush(stream.tty);
        },flush:function (stream) {
          stream.tty.ops.flush(stream.tty);
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              // we will read data by chunks of BUFSIZE
              var BUFSIZE = 256;
              var buf = new Buffer(BUFSIZE);
              var bytesRead = 0;
  
              var fd = process.stdin.fd;
              // Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
              var usingDevice = false;
              try {
                fd = fs.openSync('/dev/stdin', 'r');
                usingDevice = true;
              } catch (e) {}
  
              bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);
  
              if (usingDevice) { fs.closeSync(fd); }
              if (bytesRead > 0) {
                result = buf.slice(0, bytesRead).toString('utf-8');
              } else {
                result = null;
              }
  
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val);
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap,
                msync: MEMFS.stream_ops.msync
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (!node.contents) return new Uint8Array;
        if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          if (oldContents) {
            node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          }
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
  
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else {
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        },msync:function (stream, buffer, offset, length, mmapFlags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          if (mmapFlags & 2) {
            // MAP_PRIVATE calls need not to be synced back to underlying fs
            return 0;
          }
  
          var bytesWritten = MEMFS.stream_ops.write(stream, buffer, 0, length, offset, false);
          // should we check if bytesWritten and length are the same?
          return 0;
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        if (typeof indexedDB !== 'undefined') return indexedDB;
        var ret = null;
        if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
        assert(ret, 'IDBFS used, but indexedDB not supported');
        return ret;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          if (!fileStore.indexNames.contains('timestamp')) {
            fileStore.createIndex('timestamp', 'timestamp', { unique: false });
          }
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function(e) {
            callback(this.error);
            e.preventDefault();
          };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.chmod(path, entry.mode);
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function(e) {
          done(this.error);
          e.preventDefault();
        };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        flags &= ~0100000 /*O_LARGEFILE*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            path = fs.readlinkSync(path);
            path = NODEJS_PATH.relative(NODEJS_PATH.resolve(node.mount.opts.root), path);
            return path;
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          if (length === 0) return 0; // node errors on 0 length reads
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          return position;
        }}};
  
  var WORKERFS={DIR_MODE:16895,FILE_MODE:33279,reader:null,mount:function (mount) {
        assert(ENVIRONMENT_IS_WORKER);
        if (!WORKERFS.reader) WORKERFS.reader = new FileReaderSync();
        var root = WORKERFS.createNode(null, '/', WORKERFS.DIR_MODE, 0);
        var createdParents = {};
        function ensureParent(path) {
          // return the parent node, creating subdirs as necessary
          var parts = path.split('/');
          var parent = root;
          for (var i = 0; i < parts.length-1; i++) {
            var curr = parts.slice(0, i+1).join('/');
            if (!createdParents[curr]) {
              createdParents[curr] = WORKERFS.createNode(parent, curr, WORKERFS.DIR_MODE, 0);
            }
            parent = createdParents[curr];
          }
          return parent;
        }
        function base(path) {
          var parts = path.split('/');
          return parts[parts.length-1];
        }
        // We also accept FileList here, by using Array.prototype
        Array.prototype.forEach.call(mount.opts["files"] || [], function(file) {
          WORKERFS.createNode(ensureParent(file.name), base(file.name), WORKERFS.FILE_MODE, 0, file, file.lastModifiedDate);
        });
        (mount.opts["blobs"] || []).forEach(function(obj) {
          WORKERFS.createNode(ensureParent(obj["name"]), base(obj["name"]), WORKERFS.FILE_MODE, 0, obj["data"]);
        });
        (mount.opts["packages"] || []).forEach(function(pack) {
          pack['metadata'].files.forEach(function(file) {
            var name = file.filename.substr(1); // remove initial slash
            WORKERFS.createNode(ensureParent(name), base(name), WORKERFS.FILE_MODE, 0, pack['blob'].slice(file.start, file.end));
          });
        });
        return root;
      },createNode:function (parent, name, mode, dev, contents, mtime) {
        var node = FS.createNode(parent, name, mode);
        node.mode = mode;
        node.node_ops = WORKERFS.node_ops;
        node.stream_ops = WORKERFS.stream_ops;
        node.timestamp = (mtime || new Date).getTime();
        assert(WORKERFS.FILE_MODE !== WORKERFS.DIR_MODE);
        if (mode === WORKERFS.FILE_MODE) {
          node.size = contents.size;
          node.contents = contents;
        } else {
          node.size = 4096;
          node.contents = {};
        }
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },node_ops:{getattr:function (node) {
          return {
            dev: 1,
            ino: undefined,
            mode: node.mode,
            nlink: 1,
            uid: 0,
            gid: 0,
            rdev: undefined,
            size: node.size,
            atime: new Date(node.timestamp),
            mtime: new Date(node.timestamp),
            ctime: new Date(node.timestamp),
            blksize: 4096,
            blocks: Math.ceil(node.size / 4096),
          };
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
        },lookup:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        },mknod:function (parent, name, mode, dev) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },rename:function (oldNode, newDir, newName) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },unlink:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },rmdir:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },readdir:function (node) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },symlink:function (parent, newName, oldPath) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },readlink:function (node) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          if (position >= stream.node.size) return 0;
          var chunk = stream.node.contents.slice(position, position + length);
          var ab = WORKERFS.reader.readAsArrayBuffer(chunk);
          buffer.set(new Uint8Array(ab), offset);
          return chunk.size;
        },write:function (stream, buffer, offset, length, position) {
          throw new FS.ErrnoError(ERRNO_CODES.EIO);
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.size;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},filesystems:null,handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        if (!path) return { path: '', node: null };
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
  
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); }
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); }
            }
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var perms = ['r', 'w', 'rw'][flag & 3];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        var err = FS.nodePermissions(dir, 'x');
        if (err) return err;
        if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
        return 0;
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === '.' || name === '..') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        if (!PATH.resolve(oldpath)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return PATH.resolve(FS.getPath(link.parent), link.node_ops.readlink(link));
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        var created = false;
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // if asked only for a directory, then this must be one
        if ((flags & 65536) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        // check permissions, if this is not a file we just created now (it is ok to
        // create and write to a file with read-only permissions; it is read-only
        // for later use)
        if (!created) {
          var err = FS.mayOpen(node, flags);
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        if (stream.getdents) stream.getdents = null; // free readdir state
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },msync:function (stream, buffer, offset, length, mmapFlags) {
        if (!stream || !stream.stream_ops.msync) {
          return 0;
        }
        return stream.stream_ops.msync(stream, buffer, offset, length, mmapFlags);
      },munmap:function (stream) {
        return 0;
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = UTF8ArrayToString(buf, 0);
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var buf = new Uint8Array(lengthBytesUTF8(data)+1);
          var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
          FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
        FS.mkdir('/home');
        FS.mkdir('/home/web_user');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function(stream, buffer, offset, length, pos) { return length; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return (Math.random()*256)|0; };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createSpecialDirectories:function () {
        // create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the name of the stream for fd 6 (see test_unistd_ttyname)
        FS.mkdir('/proc');
        FS.mkdir('/proc/self');
        FS.mkdir('/proc/self/fd');
        FS.mount({
          mount: function() {
            var node = FS.createNode('/proc/self', 'fd', 16384 | 0777, 73);
            node.node_ops = {
              lookup: function(parent, name) {
                var fd = +name;
                var stream = FS.getStream(fd);
                if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
                var ret = {
                  parent: null,
                  mount: { mountpoint: 'fake' },
                  node_ops: { readlink: function() { return stream.path } }
                };
                ret.parent = ret; // make it look like a simple root node
                return ret;
              }
            };
            return node;
          }
        }, {}, '/proc/self/fd');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          //Module.printErr(stackTrace()); // useful for debugging
          this.node = node;
          this.setErrno = function(errno) {
            this.errno = errno;
            for (var key in ERRNO_CODES) {
              if (ERRNO_CODES[key] === errno) {
                this.code = key;
                break;
              }
            }
          };
          this.setErrno(errno);
          this.message = ERRNO_MESSAGES[errno];
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
        FS.createSpecialDirectories();
  
        FS.filesystems = {
          'MEMFS': MEMFS,
          'IDBFS': IDBFS,
          'NODEFS': NODEFS,
          'WORKERFS': WORKERFS,
        };
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        // force-flush all streams, so we get musl std streams printed out
        var fflush = Module['_fflush'];
        if (fflush) fflush(0);
        // close all of our streams
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = (idx / this.chunkSize)|0;
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
          // Find length
          var xhr = new XMLHttpRequest();
          xhr.open('HEAD', url, false);
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          var datalength = Number(xhr.getResponseHeader("Content-length"));
          var header;
          var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
          var chunkSize = 1024*1024; // Chunk size in bytes
  
          if (!hasByteServing) chunkSize = datalength;
  
          // Function to get a range from the remote URL.
          var doXHR = (function(from, to) {
            if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
            if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
            // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
            var xhr = new XMLHttpRequest();
            xhr.open('GET', url, false);
            if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
            // Some hints to the browser that we want binary data.
            if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
            if (xhr.overrideMimeType) {
              xhr.overrideMimeType('text/plain; charset=x-user-defined');
            }
  
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            if (xhr.response !== undefined) {
              return new Uint8Array(xhr.response || []);
            } else {
              return intArrayFromString(xhr.responseText || '', true);
            }
          });
          var lazyArray = this;
          lazyArray.setDataGetter(function(chunkNum) {
            var start = chunkNum * chunkSize;
            var end = (chunkNum+1) * chunkSize - 1; // including this byte
            end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
              lazyArray.chunks[chunkNum] = doXHR(start, end);
            }
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
            return lazyArray.chunks[chunkNum];
          });
  
          this._length = datalength;
          this._chunkSize = chunkSize;
          this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        var dep = getUniqueRunDependency('cp ' + fullname); // might have several active requests for the same fullname
        function processData(byteArray) {
          function finish(byteArray) {
            if (preFinish) preFinish();
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency(dep);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency(dep);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency(dep);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};var SYSCALLS={DEFAULT_POLLMASK:5,mappings:{},umask:511,calculateAt:function (dirfd, path) {
        if (path[0] !== '/') {
          // relative path
          var dir;
          if (dirfd === -100) {
            dir = FS.cwd();
          } else {
            var dirstream = FS.getStream(dirfd);
            if (!dirstream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
            dir = dirstream.path;
          }
          path = PATH.join2(dir, path);
        }
        return path;
      },doStat:function (func, path, buf) {
        try {
          var stat = func(path);
        } catch (e) {
          if (e && e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
            // an error occurred while trying to look up the path; we should just report ENOTDIR
            return -ERRNO_CODES.ENOTDIR;
          }
          throw e;
        }
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode;
        HEAP32[(((buf)+(16))>>2)]=stat.nlink;
        HEAP32[(((buf)+(20))>>2)]=stat.uid;
        HEAP32[(((buf)+(24))>>2)]=stat.gid;
        HEAP32[(((buf)+(28))>>2)]=stat.rdev;
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size;
        HEAP32[(((buf)+(40))>>2)]=4096;
        HEAP32[(((buf)+(44))>>2)]=stat.blocks;
        HEAP32[(((buf)+(48))>>2)]=(stat.atime.getTime() / 1000)|0;
        HEAP32[(((buf)+(52))>>2)]=0;
        HEAP32[(((buf)+(56))>>2)]=(stat.mtime.getTime() / 1000)|0;
        HEAP32[(((buf)+(60))>>2)]=0;
        HEAP32[(((buf)+(64))>>2)]=(stat.ctime.getTime() / 1000)|0;
        HEAP32[(((buf)+(68))>>2)]=0;
        HEAP32[(((buf)+(72))>>2)]=stat.ino;
        return 0;
      },doMsync:function (addr, stream, len, flags) {
        var buffer = new Uint8Array(HEAPU8.subarray(addr, addr + len));
        FS.msync(stream, buffer, 0, len, flags);
      },doMkdir:function (path, mode) {
        // remove a trailing slash, if one - /a/b/ has basename of '', but
        // we want to create b in the context of this function
        path = PATH.normalize(path);
        if (path[path.length-1] === '/') path = path.substr(0, path.length-1);
        FS.mkdir(path, mode, 0);
        return 0;
      },doMknod:function (path, mode, dev) {
        // we don't want this in the JS API as it uses mknod to create all nodes.
        switch (mode & 61440) {
          case 32768:
          case 8192:
          case 24576:
          case 4096:
          case 49152:
            break;
          default: return -ERRNO_CODES.EINVAL;
        }
        FS.mknod(path, mode, dev);
        return 0;
      },doReadlink:function (path, buf, bufsize) {
        if (bufsize <= 0) return -ERRNO_CODES.EINVAL;
        var ret = FS.readlink(path);
        ret = ret.slice(0, Math.max(0, bufsize));
        writeStringToMemory(ret, buf, true);
        return ret.length;
      },doAccess:function (path, amode) {
        if (amode & ~7) {
          // need a valid mode
          return -ERRNO_CODES.EINVAL;
        }
        var node;
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
        var perms = '';
        if (amode & 4) perms += 'r';
        if (amode & 2) perms += 'w';
        if (amode & 1) perms += 'x';
        if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
          return -ERRNO_CODES.EACCES;
        }
        return 0;
      },doDup:function (path, flags, suggestFD) {
        var suggest = FS.getStream(suggestFD);
        if (suggest) FS.close(suggest);
        return FS.open(path, flags, 0, suggestFD, suggestFD).fd;
      },doReadv:function (stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[(((iov)+(i*8))>>2)];
          var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
          var curr = FS.read(stream, HEAP8,ptr, len, offset);
          if (curr < 0) return -1;
          ret += curr;
          if (curr < len) break; // nothing more to read
        }
        return ret;
      },doWritev:function (stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[(((iov)+(i*8))>>2)];
          var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
          var curr = FS.write(stream, HEAP8,ptr, len, offset);
          if (curr < 0) return -1;
          ret += curr;
        }
        return ret;
      },varargs:0,get:function (varargs) {
        SYSCALLS.varargs += 4;
        var ret = HEAP32[(((SYSCALLS.varargs)-(4))>>2)];
        return ret;
      },getStr:function () {
        var ret = Pointer_stringify(SYSCALLS.get());
        return ret;
      },getStreamFromFD:function () {
        var stream = FS.getStream(SYSCALLS.get());
        if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        return stream;
      },getSocketFromFD:function () {
        var socket = SOCKFS.getSocket(SYSCALLS.get());
        if (!socket) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        return socket;
      },getSocketAddress:function (allowNull) {
        var addrp = SYSCALLS.get(), addrlen = SYSCALLS.get();
        if (allowNull && addrp === 0) return null;
        var info = __read_sockaddr(addrp, addrlen);
        if (info.errno) throw new FS.ErrnoError(info.errno);
        info.addr = DNS.lookup_addr(info.addr) || info.addr;
        return info;
      },get64:function () {
        var low = SYSCALLS.get(), high = SYSCALLS.get();
        if (low >= 0) assert(high === 0);
        else assert(high === -1);
        return low;
      },getZero:function () {
        assert(SYSCALLS.get() === 0);
      }};function ___syscall20(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // getpid
      return PROCINFO.pid;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

   
  Module["_memset"] = _memset;

  function ___syscall39(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // mkdir
      var path = SYSCALLS.getStr(), mode = SYSCALLS.get();
      return SYSCALLS.doMkdir(path, mode);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _DtoIHigh=true;

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function ___syscall192(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // mmap2
      var addr = SYSCALLS.get(), len = SYSCALLS.get(), prot = SYSCALLS.get(), flags = SYSCALLS.get(), fd = SYSCALLS.get(), off = SYSCALLS.get()
      off <<= 12; // undo pgoffset
      var ptr;
      var allocated = false;
      if (fd === -1) {
        ptr = _malloc(len);
        if (!ptr) return -ERRNO_CODES.ENOMEM;
        _memset(ptr, 0, len);
        allocated = true;
      } else {
        var info = FS.getStream(fd);
        if (!info) return -ERRNO_CODES.EBADF;
        var res = FS.mmap(info, HEAPU8, addr, len, off, prot, flags);
        ptr = res.ptr;
        allocated = res.allocated;
      }
      SYSCALLS.mappings[ptr] = { malloc: ptr, len: len, allocated: allocated, fd: fd, flags: flags };
      return ptr;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall195(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_stat64
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.stat, path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall194(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // ftruncate64
      var fd = SYSCALLS.get(), zero = SYSCALLS.getZero(), length = SYSCALLS.get64();
      FS.ftruncate(fd, length);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall197(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_fstat64
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.stat, stream.path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall196(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_lstat64
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.lstat, path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___lock() {}

  function ___unlock() {}

  
  var EmterpreterAsync=undefined;function ___syscall118(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fsync
      var stream = SYSCALLS.getStreamFromFD();
      return 0; // we can't do anything synchronously; the in-memory FS is already synced to
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

   
  Module["_i64Add"] = _i64Add;

  
  function _usleep(useconds) {
      // int usleep(useconds_t useconds);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/usleep.html
      // We're single-threaded, so use a busy loop. Super-ugly.
      var msec = useconds / 1000;
      if ((ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) && self['performance'] && self['performance']['now']) {
        var start = self['performance']['now']();
        while (self['performance']['now']() - start < msec) {
          // Do nothing.
        }
      } else {
        var start = Date.now();
        while (Date.now() - start < msec) {
          // Do nothing.
        }
      }
      return 0;
    }
  Module["_usleep"] = _usleep;function _nanosleep(rqtp, rmtp) {
      // int nanosleep(const struct timespec  *rqtp, struct timespec *rmtp);
      var seconds = HEAP32[((rqtp)>>2)];
      var nanoseconds = HEAP32[(((rqtp)+(4))>>2)];
      if (rmtp !== 0) {
        HEAP32[((rmtp)>>2)]=0;
        HEAP32[(((rmtp)+(4))>>2)]=0;
      }
      return _usleep((seconds * 1e6) + (nanoseconds / 1000));
    }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  function ___syscall94(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fchmod
      var fd = SYSCALLS.get(), mode = SYSCALLS.get();
      FS.fchmod(fd, mode);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall91(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // munmap
      var addr = SYSCALLS.get(), len = SYSCALLS.get();
      // TODO: support unmmap'ing parts of allocations
      var info = SYSCALLS.mappings[addr];
      if (!info) return 0;
      if (len === info.len) {
        var stream = FS.getStream(info.fd);
        SYSCALLS.doMsync(addr, stream, len, info.flags)
        FS.munmap(stream);
        SYSCALLS.mappings[addr] = null;
        if (info.allocated) {
          _free(info.malloc);
        }
      }
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall212(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // chown32
      var path = SYSCALLS.getStr(), owner = SYSCALLS.get(), group = SYSCALLS.get();
      FS.chown(path, owner, group);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  
  function _emscripten_set_main_loop_timing(mode, value) {
      Browser.mainLoop.timingMode = mode;
      Browser.mainLoop.timingValue = value;
  
      if (!Browser.mainLoop.func) {
        return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
      }
  
      if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setTimeout() {
          setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
        };
        Browser.mainLoop.method = 'timeout';
      } else if (mode == 1 /*EM_TIMING_RAF*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_rAF() {
          Browser.requestAnimationFrame(Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'rAF';
      } else if (mode == 2 /*EM_TIMING_SETIMMEDIATE*/) {
        if (!window['setImmediate']) {
          // Emulate setImmediate. (note: not a complete polyfill, we don't emulate clearImmediate() to keep code size to minimum, since not needed)
          var setImmediates = [];
          var emscriptenMainLoopMessageId = '__emcc';
          function Browser_setImmediate_messageHandler(event) {
            if (event.source === window && event.data === emscriptenMainLoopMessageId) {
              event.stopPropagation();
              setImmediates.shift()();
            }
          }
          window.addEventListener("message", Browser_setImmediate_messageHandler, true);
          window['setImmediate'] = function Browser_emulated_setImmediate(func) {
            setImmediates.push(func);
            window.postMessage(emscriptenMainLoopMessageId, "*");
          }
        }
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setImmediate() {
          window['setImmediate'](Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'immediate';
      }
      return 0;
    }function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg, noSetTiming) {
      Module['noExitRuntime'] = true;
  
      assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
  
      Browser.mainLoop.func = func;
      Browser.mainLoop.arg = arg;
  
      var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;
  
      Browser.mainLoop.runner = function Browser_mainLoop_runner() {
        if (ABORT) return;
        if (Browser.mainLoop.queue.length > 0) {
          var start = Date.now();
          var blocker = Browser.mainLoop.queue.shift();
          blocker.func(blocker.arg);
          if (Browser.mainLoop.remainingBlockers) {
            var remaining = Browser.mainLoop.remainingBlockers;
            var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
            if (blocker.counted) {
              Browser.mainLoop.remainingBlockers = next;
            } else {
              // not counted, but move the progress along a tiny bit
              next = next + 0.5; // do not steal all the next one's progress
              Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
            }
          }
          console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
          Browser.mainLoop.updateStatus();
          setTimeout(Browser.mainLoop.runner, 0);
          return;
        }
  
        // catch pauses from non-main loop sources
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Implement very basic swap interval control
        Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
        if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
          // Not the scheduled time to render this frame - skip.
          Browser.mainLoop.scheduler();
          return;
        }
  
        // Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
        // VBO double-buffering and reduce GPU stalls.
  
        if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
          Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
          Browser.mainLoop.method = ''; // just warn once per call to set main loop
        }
  
        Browser.mainLoop.runIter(function() {
          if (typeof arg !== 'undefined') {
            Runtime.dynCall('vi', func, [arg]);
          } else {
            Runtime.dynCall('v', func);
          }
        });
  
        // catch pauses from the main loop itself
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
        // to queue the newest produced audio samples.
        // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
        //       do not need to be hardcoded into this function, but can be more generic.
        if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
  
        Browser.mainLoop.scheduler();
      }
  
      if (!noSetTiming) {
        if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
        else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)
  
        Browser.mainLoop.scheduler();
      }
  
      if (simulateInfiniteLoop) {
        throw 'SimulateInfiniteLoop';
      }
    }var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
          Browser.mainLoop.scheduler = null;
          Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
        },resume:function () {
          Browser.mainLoop.currentlyRunningMainloop++;
          var timingMode = Browser.mainLoop.timingMode;
          var timingValue = Browser.mainLoop.timingValue;
          var func = Browser.mainLoop.func;
          Browser.mainLoop.func = null;
          _emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg, true /* do not set timing and call scheduler, we will do it on the next lines */);
          _emscripten_set_main_loop_timing(timingMode, timingValue);
          Browser.mainLoop.scheduler();
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
  
        var ctx;
        var contextHandle;
        if (useWebGL) {
          // For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
          var contextAttributes = {
            antialias: false,
            alpha: false
          };
  
          if (webGLContextAttributes) {
            for (var attribute in webGLContextAttributes) {
              contextAttributes[attribute] = webGLContextAttributes[attribute];
            }
          }
  
          contextHandle = GL.createContext(canvas, contextAttributes);
          if (contextHandle) {
            ctx = GL.getContext(contextHandle).GLctx;
          }
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        } else {
          ctx = canvas.getContext('2d');
        }
  
        if (!ctx) return null;
  
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
  
          Module.ctx = ctx;
          if (useWebGL) GL.makeContextCurrent(contextHandle);
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas, vrDevice) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        Browser.vrDevice = vrDevice;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
        if (typeof Browser.vrDevice === 'undefined') Browser.vrDevice = null;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
  
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
  
        if (vrDevice) {
          canvasContainer.requestFullScreen({ vrDisplay: vrDevice });
        } else {
          canvasContainer.requestFullScreen();
        }
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },allowAsyncCallbacks:true,queuedAsyncCallbacks:[],pauseAsyncCallbacks:function () {
        Browser.allowAsyncCallbacks = false;
      },resumeAsyncCallbacks:function () { // marks future callbacks as ok to execute, and synchronously runs any remaining ones right now
        Browser.allowAsyncCallbacks = true;
        if (Browser.queuedAsyncCallbacks.length > 0) {
          var callbacks = Browser.queuedAsyncCallbacks;
          Browser.queuedAsyncCallbacks = [];
          callbacks.forEach(function(func) {
            func();
          });
        }
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } // drop it on the floor otherwise, next interval will kick in
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = event.wheelDelta;
            break;
          case 'wheel': 
            delta = event['deltaY'];
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return delta;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              var last = Browser.touches[touch.identifier];
              if (!last) last = coords;
              Browser.lastTouches[touch.identifier] = last;
              Browser.touches[touch.identifier] = coords;
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      },wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
        var handle = Browser.nextWgetRequestHandle;
        Browser.nextWgetRequestHandle++;
        return handle;
      }};

  function ___syscall54(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // ioctl
      var stream = SYSCALLS.getStreamFromFD(), op = SYSCALLS.get();
      switch (op) {
        case 21505: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return 0;
        }
        case 21506: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return 0; // no-op, not actually adjusting terminal settings
        }
        case 21519: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          var argp = SYSCALLS.get();
          HEAP32[((argp)>>2)]=0;
          return 0;
        }
        case 21520: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return -ERRNO_CODES.EINVAL; // not supported
        }
        case 21531: {
          var argp = SYSCALLS.get();
          return FS.ioctl(stream, op, argp);
        }
        default: abort('bad ioctl syscall ' + op);
      }
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  var ___tm_current=allocate(44, "i8", ALLOC_STATIC);
  
  
  var ___tm_timezone=allocate(intArrayFromString("GMT"), "i8", ALLOC_STATIC);
  
  
  var _tzname=allocate(8, "i32*", ALLOC_STATIC);
  
  var _daylight=allocate(1, "i32*", ALLOC_STATIC);
  
  var _timezone=allocate(1, "i32*", ALLOC_STATIC);function _tzset() {
      // TODO: Use (malleable) environment variables instead of system settings.
      if (_tzset.called) return;
      _tzset.called = true;
  
      HEAP32[((_timezone)>>2)]=-(new Date()).getTimezoneOffset() * 60;
  
      var winter = new Date(2000, 0, 1);
      var summer = new Date(2000, 6, 1);
      HEAP32[((_daylight)>>2)]=Number(winter.getTimezoneOffset() != summer.getTimezoneOffset());
  
      function extractZone(date) {
        var match = date.toTimeString().match(/\(([A-Za-z ]+)\)$/);
        return match ? match[1] : "GMT";
      };
      var winterName = extractZone(winter);
      var summerName = extractZone(summer);
      var winterNamePtr = allocate(intArrayFromString(winterName), 'i8', ALLOC_NORMAL);
      var summerNamePtr = allocate(intArrayFromString(summerName), 'i8', ALLOC_NORMAL);
      if (summer.getTimezoneOffset() < winter.getTimezoneOffset()) {
        // Northern hemisphere
        HEAP32[((_tzname)>>2)]=winterNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=summerNamePtr;
      } else {
        HEAP32[((_tzname)>>2)]=summerNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=winterNamePtr;
      }
    }function _localtime_r(time, tmPtr) {
      _tzset();
      var date = new Date(HEAP32[((time)>>2)]*1000);
      HEAP32[((tmPtr)>>2)]=date.getSeconds();
      HEAP32[(((tmPtr)+(4))>>2)]=date.getMinutes();
      HEAP32[(((tmPtr)+(8))>>2)]=date.getHours();
      HEAP32[(((tmPtr)+(12))>>2)]=date.getDate();
      HEAP32[(((tmPtr)+(16))>>2)]=date.getMonth();
      HEAP32[(((tmPtr)+(20))>>2)]=date.getFullYear()-1900;
      HEAP32[(((tmPtr)+(24))>>2)]=date.getDay();
  
      var start = new Date(date.getFullYear(), 0, 1);
      var yday = ((date.getTime() - start.getTime()) / (1000 * 60 * 60 * 24))|0;
      HEAP32[(((tmPtr)+(28))>>2)]=yday;
      HEAP32[(((tmPtr)+(36))>>2)]=-(date.getTimezoneOffset() * 60);
  
      // DST is in December in South
      var summerOffset = new Date(2000, 6, 1).getTimezoneOffset();
      var winterOffset = start.getTimezoneOffset();
      var dst = (date.getTimezoneOffset() == Math.min(winterOffset, summerOffset))|0;
      HEAP32[(((tmPtr)+(32))>>2)]=dst;
  
      var zonePtr = HEAP32[(((_tzname)+(dst ? Runtime.QUANTUM_SIZE : 0))>>2)];
      HEAP32[(((tmPtr)+(40))>>2)]=zonePtr;
  
      return tmPtr;
    }function _localtime(time) {
      return _localtime_r(time, ___tm_current);
    }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;

  function ___syscall33(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // access
      var path = SYSCALLS.getStr(), amode = SYSCALLS.get();
      return SYSCALLS.doAccess(path, amode);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _pthread_cleanup_push(routine, arg) {
      __ATEXIT__.push(function() { Runtime.dynCall('vi', routine, [arg]) })
      _pthread_cleanup_push.level = __ATEXIT__.length;
    }

  function ___syscall15(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // chmod
      var path = SYSCALLS.getStr(), mode = SYSCALLS.get();
      FS.chmod(path, mode);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall10(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // unlink
      var path = SYSCALLS.getStr();
      FS.unlink(path);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  
  
  
  var _environ=allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = ENV['LOGNAME'] = 'web_user';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/web_user';
        ENV['LANG'] = 'C';
        ENV['_'] = Module['thisProgram'];
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        writeAsciiToMemory(line, poolPtr);
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }

  function _pthread_cleanup_pop() {
      assert(_pthread_cleanup_push.level == __ATEXIT__.length, 'cannot pop if something else added meanwhile!');
      __ATEXIT__.pop();
      _pthread_cleanup_push.level = __ATEXIT__.length;
    }

  function ___syscall183(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // getcwd
      var buf = SYSCALLS.get(), size = SYSCALLS.get();
      if (size === 0) return -ERRNO_CODES.EINVAL;
      var cwd = FS.cwd();
      if (size < cwd.length + 1) return -ERRNO_CODES.ERANGE;
      writeAsciiToMemory(cwd, buf);
      return buf;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _SItoD=true;

  function ___syscall3(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // read
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get(), count = SYSCALLS.get();
      return FS.read(stream, HEAP8,buf, count);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall5(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // open
      var pathname = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get() // optional TODO
      var stream = FS.open(pathname, flags, mode);
      return stream.fd;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall4(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // write
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get(), count = SYSCALLS.get();
      return FS.write(stream, HEAP8,buf, count);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall6(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // close
      var stream = SYSCALLS.getStreamFromFD();
      FS.close(stream);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) {
        var success = self.alloc(bytes);
        if (!success) return -1 >>> 0; // sbrk failure code
      }
      return ret;  // Previous break location.
    }

   
  Module["_memmove"] = _memmove;

  function _gettimeofday(ptr) {
      var now = Date.now();
      HEAP32[((ptr)>>2)]=(now/1000)|0; // seconds
      HEAP32[(((ptr)+(4))>>2)]=((now % 1000)*1000)|0; // microseconds
      return 0;
    }

  function ___syscall85(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // readlink
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get(), bufsize = SYSCALLS.get();
      return SYSCALLS.doReadlink(path, buf, bufsize);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall207(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fchown32
      var fd = SYSCALLS.get(), owner = SYSCALLS.get(), group = SYSCALLS.get();
      FS.fchown(fd, owner, group);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _abort() {
      Module['abort']();
    }

  
  function ___syscall202(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // getgid32
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }function ___syscall201() {
  return ___syscall202.apply(null, arguments)
  }

  function ___syscall221(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fcntl64
      var stream = SYSCALLS.getStreamFromFD(), cmd = SYSCALLS.get();
      switch (cmd) {
        case 0: {
          var arg = SYSCALLS.get();
          if (arg < 0) {
            return -ERRNO_CODES.EINVAL;
          }
          var newStream;
          newStream = FS.open(stream.path, stream.flags, 0, arg);
          return newStream.fd;
        }
        case 1:
        case 2:
          return 0;  // FD_CLOEXEC makes no sense for a single process.
        case 3:
          return stream.flags;
        case 4: {
          var arg = SYSCALLS.get();
          stream.flags |= arg;
          return 0;
        }
        case 12:
        case 12: {
          var arg = SYSCALLS.get();
          var offset = 0;
          // We're always unlocked.
          HEAP16[(((arg)+(offset))>>1)]=2;
          return 0;
        }
        case 13:
        case 14:
        case 13:
        case 14:
          return 0; // Pretend that the locking is successful.
        case 16:
        case 8:
          return -ERRNO_CODES.EINVAL; // These are for sockets. We don't have them fully implemented yet.
        case 9:
          // musl trusts getown return values, due to a bug where they must be, as they overlap with errors. just return -1 here, so fnctl() returns that, and we set errno ourselves.
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        default: {
          return -ERRNO_CODES.EINVAL;
        }
      }
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _time(ptr) {
      var ret = (Date.now()/1000)|0;
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  function _pthread_self() {
      //FIXME: assumes only a single thread
      return 0;
    }

  function ___syscall140(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // llseek
      var stream = SYSCALLS.getStreamFromFD(), offset_high = SYSCALLS.get(), offset_low = SYSCALLS.get(), result = SYSCALLS.get(), whence = SYSCALLS.get();
      var offset = offset_low;
      assert(offset_high === 0);
      FS.llseek(stream, offset, whence);
      HEAP32[((result)>>2)]=stream.position;
      if (stream.getdents && offset === 0 && whence === 0) stream.getdents = null; // reset readdir state
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _utimes(path, times) {
      var time;
      if (times) {
        var offset = 8 + 0;
        time = HEAP32[(((times)+(offset))>>2)] * 1000;
        offset = 8 + 4;
        time += HEAP32[(((times)+(offset))>>2)] / 1000;
      } else {
        time = Date.now();
      }
      path = Pointer_stringify(path);
      try {
        FS.utime(path, time, time);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function ___syscall146(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // writev
      var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
      return SYSCALLS.doWritev(stream, iov, iovcnt);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall40(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // rmdir
      var path = SYSCALLS.getStr();
      FS.rmdir(path);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }
FS.staticInit();__ATINIT__.unshift(function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() });__ATMAIN__.push(function() { FS.ignorePermissions = false });__ATEXIT__.push(function() { FS.quit() });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;Module["FS_unlink"] = FS.unlink;
__ATINIT__.unshift(function() { TTY.init() });__ATEXIT__.push(function() { TTY.shutdown() });
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); var NODEJS_PATH = require("path"); NODEFS.staticInit(); }
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas, vrDevice) { Browser.requestFullScreen(lockPointer, resizeCanvas, vrDevice) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
  Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) }
___buildEnvironment(ENV);
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);


function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiii(index,a1,a2,a3) {
    return Runtime.functionPointers[index](a1,a2,a3);
}

function invoke_i(index) {
  try {
    return Module["dynCall_i"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_i(index) {
    return Runtime.functionPointers[index]();
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_vi(index,a1) {
    Runtime.functionPointers[index](a1);
}

function invoke_vii(index,a1,a2) {
  try {
    Module["dynCall_vii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_vii(index,a1,a2) {
    Runtime.functionPointers[index](a1,a2);
}

function invoke_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    return Module["dynCall_iiiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
    return Runtime.functionPointers[index](a1,a2,a3,a4,a5,a6);
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_ii(index,a1) {
    return Runtime.functionPointers[index](a1);
}

function invoke_viii(index,a1,a2,a3) {
  try {
    Module["dynCall_viii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_viii(index,a1,a2,a3) {
    Runtime.functionPointers[index](a1,a2,a3);
}

function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_v(index) {
    Runtime.functionPointers[index]();
}

function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiiii(index,a1,a2,a3,a4) {
    return Runtime.functionPointers[index](a1,a2,a3,a4);
}

function invoke_viiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    Module["dynCall_viiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_viiiiii(index,a1,a2,a3,a4,a5,a6) {
    Runtime.functionPointers[index](a1,a2,a3,a4,a5,a6);
}

function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iii(index,a1,a2) {
    return Runtime.functionPointers[index](a1,a2);
}

function invoke_iiiiii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiiiii(index,a1,a2,a3,a4,a5) {
    return Runtime.functionPointers[index](a1,a2,a3,a4,a5);
}

function invoke_viiii(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_viiii(index,a1,a2,a3,a4) {
    Runtime.functionPointers[index](a1,a2,a3,a4);
}

Module.asmGlobalArg = { "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array, "NaN": NaN, "Infinity": Infinity };

Module.asmLibraryArg = { "abort": abort, "assert": assert, "invoke_iiii": invoke_iiii, "jsCall_iiii": jsCall_iiii, "invoke_i": invoke_i, "jsCall_i": jsCall_i, "invoke_vi": invoke_vi, "jsCall_vi": jsCall_vi, "invoke_vii": invoke_vii, "jsCall_vii": jsCall_vii, "invoke_iiiiiii": invoke_iiiiiii, "jsCall_iiiiiii": jsCall_iiiiiii, "invoke_ii": invoke_ii, "jsCall_ii": jsCall_ii, "invoke_viii": invoke_viii, "jsCall_viii": jsCall_viii, "invoke_v": invoke_v, "jsCall_v": jsCall_v, "invoke_iiiii": invoke_iiiii, "jsCall_iiiii": jsCall_iiiii, "invoke_viiiiii": invoke_viiiiii, "jsCall_viiiiii": jsCall_viiiiii, "invoke_iii": invoke_iii, "jsCall_iii": jsCall_iii, "invoke_iiiiii": invoke_iiiiii, "jsCall_iiiiii": jsCall_iiiiii, "invoke_viiii": invoke_viiii, "jsCall_viiii": jsCall_viiii, "_pthread_cleanup_pop": _pthread_cleanup_pop, "___syscall85": ___syscall85, "_utimes": _utimes, "_pthread_cleanup_push": _pthread_cleanup_push, "___syscall6": ___syscall6, "___syscall40": ___syscall40, "___syscall118": ___syscall118, "___setErrNo": ___setErrNo, "___syscall20": ___syscall20, "___syscall183": ___syscall183, "_usleep": _usleep, "___buildEnvironment": ___buildEnvironment, "___syscall91": ___syscall91, "___syscall39": ___syscall39, "_localtime_r": _localtime_r, "_tzset": _tzset, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_sbrk": _sbrk, "___syscall192": ___syscall192, "___syscall197": ___syscall197, "___syscall196": ___syscall196, "_emscripten_memcpy_big": _emscripten_memcpy_big, "___syscall194": ___syscall194, "___syscall212": ___syscall212, "_sysconf": _sysconf, "___syscall94": ___syscall94, "_nanosleep": _nanosleep, "___syscall195": ___syscall195, "___syscall221": ___syscall221, "_pthread_self": _pthread_self, "_getenv": _getenv, "___syscall33": ___syscall33, "___syscall54": ___syscall54, "___unlock": ___unlock, "___syscall140": ___syscall140, "___syscall15": ___syscall15, "_emscripten_set_main_loop": _emscripten_set_main_loop, "___syscall10": ___syscall10, "___syscall3": ___syscall3, "___lock": ___lock, "_abort": _abort, "___syscall5": ___syscall5, "___syscall4": ___syscall4, "_time": _time, "_gettimeofday": _gettimeofday, "___syscall201": ___syscall201, "___syscall207": ___syscall207, "_localtime": _localtime, "___syscall202": ___syscall202, "___syscall146": ___syscall146, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8 };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
  'use asm';
  
  
  var HEAP8 = new global.Int8Array(buffer);
  var HEAP16 = new global.Int16Array(buffer);
  var HEAP32 = new global.Int32Array(buffer);
  var HEAPU8 = new global.Uint8Array(buffer);
  var HEAPU16 = new global.Uint16Array(buffer);
  var HEAPU32 = new global.Uint32Array(buffer);
  var HEAPF32 = new global.Float32Array(buffer);
  var HEAPF64 = new global.Float64Array(buffer);


  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;

  var __THREW__ = 0;
  var threwValue = 0;
  var setjmpId = 0;
  var undef = 0;
  var nan = global.NaN, inf = global.Infinity;
  var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

  var tempRet0 = 0;
  var tempRet1 = 0;
  var tempRet2 = 0;
  var tempRet3 = 0;
  var tempRet4 = 0;
  var tempRet5 = 0;
  var tempRet6 = 0;
  var tempRet7 = 0;
  var tempRet8 = 0;
  var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var Math_min=global.Math.min;
  var Math_clz32=global.Math.clz32;
  var abort=env.abort;
  var assert=env.assert;
  var invoke_iiii=env.invoke_iiii;
  var jsCall_iiii=env.jsCall_iiii;
  var invoke_i=env.invoke_i;
  var jsCall_i=env.jsCall_i;
  var invoke_vi=env.invoke_vi;
  var jsCall_vi=env.jsCall_vi;
  var invoke_vii=env.invoke_vii;
  var jsCall_vii=env.jsCall_vii;
  var invoke_iiiiiii=env.invoke_iiiiiii;
  var jsCall_iiiiiii=env.jsCall_iiiiiii;
  var invoke_ii=env.invoke_ii;
  var jsCall_ii=env.jsCall_ii;
  var invoke_viii=env.invoke_viii;
  var jsCall_viii=env.jsCall_viii;
  var invoke_v=env.invoke_v;
  var jsCall_v=env.jsCall_v;
  var invoke_iiiii=env.invoke_iiiii;
  var jsCall_iiiii=env.jsCall_iiiii;
  var invoke_viiiiii=env.invoke_viiiiii;
  var jsCall_viiiiii=env.jsCall_viiiiii;
  var invoke_iii=env.invoke_iii;
  var jsCall_iii=env.jsCall_iii;
  var invoke_iiiiii=env.invoke_iiiiii;
  var jsCall_iiiiii=env.jsCall_iiiiii;
  var invoke_viiii=env.invoke_viiii;
  var jsCall_viiii=env.jsCall_viiii;
  var _pthread_cleanup_pop=env._pthread_cleanup_pop;
  var ___syscall85=env.___syscall85;
  var _utimes=env._utimes;
  var _pthread_cleanup_push=env._pthread_cleanup_push;
  var ___syscall6=env.___syscall6;
  var ___syscall40=env.___syscall40;
  var ___syscall118=env.___syscall118;
  var ___setErrNo=env.___setErrNo;
  var ___syscall20=env.___syscall20;
  var ___syscall183=env.___syscall183;
  var _usleep=env._usleep;
  var ___buildEnvironment=env.___buildEnvironment;
  var ___syscall91=env.___syscall91;
  var ___syscall39=env.___syscall39;
  var _localtime_r=env._localtime_r;
  var _tzset=env._tzset;
  var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
  var _sbrk=env._sbrk;
  var ___syscall192=env.___syscall192;
  var ___syscall197=env.___syscall197;
  var ___syscall196=env.___syscall196;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var ___syscall194=env.___syscall194;
  var ___syscall212=env.___syscall212;
  var _sysconf=env._sysconf;
  var ___syscall94=env.___syscall94;
  var _nanosleep=env._nanosleep;
  var ___syscall195=env.___syscall195;
  var ___syscall221=env.___syscall221;
  var _pthread_self=env._pthread_self;
  var _getenv=env._getenv;
  var ___syscall33=env.___syscall33;
  var ___syscall54=env.___syscall54;
  var ___unlock=env.___unlock;
  var ___syscall140=env.___syscall140;
  var ___syscall15=env.___syscall15;
  var _emscripten_set_main_loop=env._emscripten_set_main_loop;
  var ___syscall10=env.___syscall10;
  var ___syscall3=env.___syscall3;
  var ___lock=env.___lock;
  var _abort=env._abort;
  var ___syscall5=env.___syscall5;
  var ___syscall4=env.___syscall4;
  var _time=env._time;
  var _gettimeofday=env._gettimeofday;
  var ___syscall201=env.___syscall201;
  var ___syscall207=env.___syscall207;
  var _localtime=env._localtime;
  var ___syscall202=env.___syscall202;
  var ___syscall146=env.___syscall146;
  var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
  size = size|0;
  var ret = 0;
  ret = STACKTOP;
  STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 15)&-16;

  return ret|0;
}
function stackSave() {
  return STACKTOP|0;
}
function stackRestore(top) {
  top = top|0;
  STACKTOP = top;
}
function establishStackSpace(stackBase, stackMax) {
  stackBase = stackBase|0;
  stackMax = stackMax|0;
  STACKTOP = stackBase;
  STACK_MAX = stackMax;
}

function setThrew(threw, value) {
  threw = threw|0;
  value = value|0;
  if ((__THREW__|0) == 0) {
    __THREW__ = threw;
    threwValue = value;
  }
}
function copyTempFloat(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
  HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
  HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
  HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}

function setTempRet0(value) {
  value = value|0;
  tempRet0 = value;
}
function getTempRet0() {
  return tempRet0|0;
}

function _sqlite3_compileoption_used($zOptName) {
 $zOptName = $zOptName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $zOptName;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_strnicmp($2,16697,7)|0);
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 7|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 HEAP32[$n>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(7);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$i>>2]|0;
  $13 = (356 + ($12<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$n>>2]|0;
  $16 = (_sqlite3_strnicmp($11,$14,$15)|0);
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$n>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = (356 + ($19<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18)|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = (_sqlite3IsIdChar($23)|0);
   $25 = ($24|0)==(0);
   if ($25) {
    label = 7;
    break;
   }
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$i>>2] = $27;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _sqlite3_strnicmp($zLeft,$zRight,$N) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $a = sp + 4|0;
 $b = sp;
 HEAP32[$1>>2] = $zLeft;
 HEAP32[$2>>2] = $zRight;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 if ($5) {
  $7 = ($6|0)!=(0|0);
  $8 = $7 ? -1 : 0;
  HEAP32[$0>>2] = $8;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $9 = ($6|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 1;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$a>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$b>>2] = $11;
 while(1) {
  $12 = HEAP32[$3>>2]|0;
  $13 = (($12) + -1)|0;
  HEAP32[$3>>2] = $13;
  $14 = ($12|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$a>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$a>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = (16974 + ($21)|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  $25 = HEAP32[$b>>2]|0;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = (16974 + ($27)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29&255;
  $31 = ($24|0)==($30|0);
  if (!($31)) {
   break;
  }
  $32 = HEAP32[$a>>2]|0;
  $33 = ((($32)) + 1|0);
  HEAP32[$a>>2] = $33;
  $34 = HEAP32[$b>>2]|0;
  $35 = ((($34)) + 1|0);
  HEAP32[$b>>2] = $35;
 }
 $36 = HEAP32[$3>>2]|0;
 $37 = ($36|0)<(0);
 if ($37) {
  $51 = 0;
 } else {
  $38 = HEAP32[$a>>2]|0;
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = (16974 + ($40)|0);
  $42 = HEAP8[$41>>0]|0;
  $43 = $42&255;
  $44 = HEAP32[$b>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = (16974 + ($46)|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = (($43) - ($49))|0;
  $51 = $50;
 }
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3Strlen30($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_strlen($4)|0);
  $6 = 1073741823 & $5;
  HEAP32[$0>>2] = $6;
 }
 $7 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3IsIdChar($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 $3 = (17533 + ($2)|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 70;
 $7 = ($6|0)!=(0);
 $8 = $7&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_compileoption_get($N) {
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(7);
 $or$cond = $3 & $5;
 if ($or$cond) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (356 + ($6<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3_status64($op,$pCurrent,$pHighwater,$resetFlag) {
 $op = $op|0;
 $pCurrent = $pCurrent|0;
 $pHighwater = $pHighwater|0;
 $resetFlag = $resetFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pMutex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pMutex = sp;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pCurrent;
 HEAP32[$3>>2] = $pHighwater;
 HEAP32[$4>>2] = $resetFlag;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<(0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)>=(10);
 $or$cond = $6 | $8;
 if ($or$cond) {
  $9 = (_sqlite3MisuseError(16368)|0);
  HEAP32[$0>>2] = $9;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (16705 + ($10)|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12 << 24 >> 24;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = (_sqlite3Pcache1Mutex()|0);
  $17 = $15;
 } else {
  $16 = (_sqlite3MallocMutex()|0);
  $17 = $16;
 }
 HEAP32[$pMutex>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = (384 + ($18<<2)|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = $21;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((424) + ($26<<2)|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = $29;
 $31 = $30;
 HEAP32[$31>>2] = $28;
 $32 = (($30) + 4)|0;
 $33 = $32;
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$4>>2]|0;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = HEAP32[$1>>2]|0;
  $37 = (384 + ($36<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = ((424) + ($39<<2)|0);
  HEAP32[$40>>2] = $38;
 }
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3MisuseError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_sourceid()|0);
 $3 = ((($2)) + 20|0);
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $3;
 _sqlite3_log(21,45209,$vararg_buffer);
 STACKTOP = sp;return 21;
}
function _sqlite3Pcache1Mutex() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(1668)>>2]|0;
 return ($0|0);
}
function _sqlite3MallocMutex() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[280>>2]|0;
 return ($0|0);
}
function _sqlite3BtreeEnterAll($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$p>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ((($19)) + 4|0);
   HEAP32[$20>>2] = $16;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreePager($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3DeleteTrigger($db,$pTrigger) {
 $db = $db|0;
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTrigger;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3DeleteTriggerStep($4,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 _sqlite3DbFree($8,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3DbFree($11,$14);
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3ExprDelete($15,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3IdListDelete($19,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($23,$24);
 STACKTOP = sp;return;
}
function _sqlite3DeleteTable($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pIndex = 0, $pNext = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pIndex = sp + 8|0;
 $pNext = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 528|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 36|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = (($12) + -1)<<16>>16;
  HEAP16[$11>>1] = $13;
  $14 = $13&65535;
  $15 = ($14|0)>(0);
  if ($15) {
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pIndex>>2] = $18;
 while(1) {
  $19 = HEAP32[$pIndex>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$pIndex>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pNext>>2] = $23;
  $24 = HEAP32[$0>>2]|0;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = HEAP32[$0>>2]|0;
   $27 = ((($26)) + 528|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)==(0|0);
   if ($29) {
    label = 9;
   }
  } else {
   label = 9;
  }
  if ((label|0) == 9) {
   label = 0;
   $30 = HEAP32[$pIndex>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$zName>>2] = $31;
   $32 = HEAP32[$pIndex>>2]|0;
   $33 = ((($32)) + 24|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = ((($34)) + 24|0);
   $36 = HEAP32[$zName>>2]|0;
   (_sqlite3HashInsert($35,$36,0)|0);
  }
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$pIndex>>2]|0;
  _freeIndex($37,$38);
  $39 = HEAP32[$pNext>>2]|0;
  HEAP32[$pIndex>>2] = $39;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 _sqlite3FkDelete($40,$41);
 $42 = HEAP32[$0>>2]|0;
 $43 = HEAP32[$1>>2]|0;
 _sqlite3DeleteColumnNames($42,$43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 _sqlite3DbFree($44,$46);
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 20|0);
 $50 = HEAP32[$49>>2]|0;
 _sqlite3DbFree($47,$50);
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = ((($52)) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 _sqlite3SelectDelete($51,$54);
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$1>>2]|0;
 $57 = ((($56)) + 24|0);
 $58 = HEAP32[$57>>2]|0;
 _sqlite3ExprListDelete($55,$58);
 $59 = HEAP32[$0>>2]|0;
 $60 = HEAP32[$1>>2]|0;
 _sqlite3VtabClear($59,$60);
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($61,$62);
 STACKTOP = sp;return;
}
function _sqlite3VdbeClearObject($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $pNext = 0, $pSub = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pSub = sp + 8|0;
 $pNext = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 68|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7 << 16 >> 16;
 _releaseMemArray($4,$8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 84|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15<<1;
 _releaseMemArray($11,$16);
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 192|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pSub>>2] = $19;
 while(1) {
  $20 = HEAP32[$pSub>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pSub>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pNext>>2] = $24;
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$pSub>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$pSub>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  _vdbeFreeOpArray($25,$27,$30);
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$pSub>>2]|0;
  _sqlite3DbFree($31,$32);
  $33 = HEAP32[$pNext>>2]|0;
  HEAP32[$pSub>>2] = $33;
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 70|0);
 $36 = HEAP16[$35>>1]|0;
 $37 = $36 << 16 >> 16;
 $38 = (($37) - 1)|0;
 HEAP32[$i>>2] = $38;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = ($39|0)>=(0);
  $41 = HEAP32[$0>>2]|0;
  if (!($40)) {
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 64|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  _sqlite3DbFree($41,$47);
  $48 = HEAP32[$i>>2]|0;
  $49 = (($48) + -1)|0;
  HEAP32[$i>>2] = $49;
 }
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 64|0);
 $52 = HEAP32[$51>>2]|0;
 _sqlite3DbFree($41,$52);
 $53 = HEAP32[$0>>2]|0;
 $54 = HEAP32[$1>>2]|0;
 $55 = ((($54)) + 4|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$1>>2]|0;
 $58 = ((($57)) + 32|0);
 $59 = HEAP32[$58>>2]|0;
 _vdbeFreeOpArray($53,$56,$59);
 $60 = HEAP32[$0>>2]|0;
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 16|0);
 $63 = HEAP32[$62>>2]|0;
 _sqlite3DbFree($60,$63);
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 168|0);
 $67 = HEAP32[$66>>2]|0;
 _sqlite3DbFree($64,$67);
 $68 = HEAP32[$0>>2]|0;
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 172|0);
 $71 = HEAP32[$70>>2]|0;
 _sqlite3DbFree($68,$71);
 STACKTOP = sp;return;
}
function _sqlite3DbFree($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBuf = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 528|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  if ($9) {
   _measureAllocationSize($10,$11);
   STACKTOP = sp;return;
  }
  $12 = (_isLookaside($10,$11)|0);
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   HEAP32[$pBuf>>2] = $14;
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 248|0);
   $17 = ((($16)) + 28|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$pBuf>>2]|0;
   HEAP32[$19>>2] = $18;
   $20 = HEAP32[$pBuf>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 248|0);
   $23 = ((($22)) + 28|0);
   HEAP32[$23>>2] = $20;
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 248|0);
   $26 = ((($25)) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + -1)|0;
   HEAP32[$26>>2] = $28;
   STACKTOP = sp;return;
  }
 }
 $29 = HEAP32[$1>>2]|0;
 _sqlite3_free($29);
 STACKTOP = sp;return;
}
function _sqlite3_vfs_find($zVfs) {
 $zVfs = $zVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $or$cond = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVfs = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $zVfs;
 HEAP32[$pVfs>>2] = 0;
 $2 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $5 = HEAP32[464>>2]|0;
 HEAP32[$pVfs>>2] = $5;
 while(1) {
  $6 = HEAP32[$pVfs>>2]|0;
  $7 = ($6|0)==(0|0);
  $8 = HEAP32[$1>>2]|0;
  $9 = ($8|0)==(0|0);
  $or$cond = $7 | $9;
  if ($or$cond) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$pVfs>>2]|0;
  $12 = ((($11)) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_strcmp($10,$13)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   break;
  }
  $16 = HEAP32[$pVfs>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$pVfs>>2] = $18;
 }
 $19 = HEAP32[$pVfs>>2]|0;
 HEAP32[$0>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3_initialize() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, $pHash = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pHash = sp;
 $1 = HEAP32[(236)>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  HEAP32[$0>>2] = 0;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $5;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 HEAP32[(244)>>2] = 1;
 $6 = HEAP32[(248)>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MallocInit()|0);
  HEAP32[$rc>>2] = $8;
 }
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 if ($10) {
  HEAP32[(248)>>2] = 1;
  $11 = HEAP32[(260)>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   HEAP32[(260)>>2] = (8);
   $13 = HEAP32[(12)>>2]|0;
   $14 = ($13|0)==(0);
   $15 = HEAP32[(260)>>2]|0;
   $16 = ($15|0)!=(0|0);
   $or$cond = $14 | $16;
   if (!($or$cond)) {
    HEAP32[$rc>>2] = 7;
   }
  }
 }
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if ($18) {
  $19 = HEAP32[(256)>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[(256)>>2] = $20;
 }
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $24 = HEAP32[(236)>>2]|0;
 $25 = ($24|0)==(0);
 $26 = HEAP32[(240)>>2]|0;
 $27 = ($26|0)==(0);
 $or$cond3 = $25 & $27;
 if ($or$cond3) {
  HEAP32[$pHash>>2] = 848;
  HEAP32[(240)>>2] = 1;
  $28 = HEAP32[$pHash>>2]|0;
  dest=$28; stop=dest+92|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  _sqlite3RegisterGlobalFunctions();
  $29 = HEAP32[(252)>>2]|0;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = (_sqlite3PcacheInitialize()|0);
   HEAP32[$rc>>2] = $31;
  }
  $32 = HEAP32[$rc>>2]|0;
  $33 = ($32|0)==(0);
  if ($33) {
   HEAP32[(252)>>2] = 1;
   $34 = (_sqlite3OsInit()|0);
   HEAP32[$rc>>2] = $34;
  }
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[(212)>>2]|0;
   $38 = HEAP32[(216)>>2]|0;
   $39 = HEAP32[(220)>>2]|0;
   _sqlite3PCacheBufferSetup($37,$38,$39);
   HEAP32[(236)>>2] = 1;
  }
  HEAP32[(240)>>2] = 0;
 }
 $40 = HEAP32[(256)>>2]|0;
 $41 = (($40) + -1)|0;
 HEAP32[(256)>>2] = $41;
 $42 = HEAP32[(256)>>2]|0;
 $43 = ($42|0)<=(0);
 if ($43) {
  HEAP32[(260)>>2] = 0;
 }
 $44 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3_vfs_register($pVfs,$makeDflt) {
 $pVfs = $pVfs|0;
 $makeDflt = $makeDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $makeDflt;
 $3 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $6;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $7 = HEAP32[$1>>2]|0;
 _vfsUnlink($7);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[464>>2]|0;
 $11 = ($10|0)==(0|0);
 $or$cond = $9 | $11;
 $12 = HEAP32[464>>2]|0;
 if ($or$cond) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 12|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  HEAP32[464>>2] = $15;
 } else {
  $16 = ((($12)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 12|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[464>>2]|0;
  $22 = ((($21)) + 12|0);
  HEAP32[$22>>2] = $20;
 }
 HEAP32[$0>>2] = 0;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _vfsUnlink($pVfs) {
 $pVfs = $pVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVfs;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[464>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($3|0)==($4|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[464>>2] = $8;
  STACKTOP = sp;return;
 }
 $9 = HEAP32[464>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[464>>2]|0;
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$p>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ($18|0)!=($19|0);
   $31 = $20;
  } else {
   $31 = 0;
  }
  $21 = HEAP32[$p>>2]|0;
  $22 = ((($21)) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  if (!($31)) {
   break;
  }
  HEAP32[$p>>2] = $23;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ($23|0)==($24|0);
 if (!($25)) {
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$p>>2]|0;
 $30 = ((($29)) + 12|0);
 HEAP32[$30>>2] = $28;
 STACKTOP = sp;return;
}
function _sqlite3_release_memory($n) {
 $n = $n|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 STACKTOP = sp;return 0;
}
function _sqlite3_soft_heap_limit64($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $excess = 0, $nUsed = 0, $priorLimit = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 24|0;
 $priorLimit = sp + 16|0;
 $excess = sp + 8|0;
 $nUsed = sp;
 $rc = sp + 40|0;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $2;
  $12 = $11;
  HEAP32[$12>>2] = -1;
  $13 = (($11) + 4)|0;
  $14 = $13;
  HEAP32[$14>>2] = -1;
  $130 = $2;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  tempRet0 = ($135);
  STACKTOP = sp;return ($132|0);
 }
 $15 = (288);
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $priorLimit;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = $3;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)<(0);
 if ($31) {
  $32 = $priorLimit;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = $2;
  $39 = $38;
  HEAP32[$39>>2] = $34;
  $40 = (($38) + 4)|0;
  $41 = $40;
  HEAP32[$41>>2] = $37;
  $130 = $2;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  tempRet0 = ($135);
  STACKTOP = sp;return ($132|0);
 }
 $42 = $3;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (288);
 $49 = $48;
 HEAP32[$49>>2] = $44;
 $50 = (($48) + 4)|0;
 $51 = $50;
 HEAP32[$51>>2] = $47;
 $52 = (_sqlite3StatusValue(0)|0);
 $53 = tempRet0;
 $54 = $nUsed;
 $55 = $54;
 HEAP32[$55>>2] = $52;
 $56 = (($54) + 4)|0;
 $57 = $56;
 HEAP32[$57>>2] = $53;
 $58 = $3;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)>(0);
 $65 = ($60>>>0)>(0);
 $66 = ($63|0)==(0);
 $67 = $66 & $65;
 $68 = $64 | $67;
 if ($68) {
  $69 = $3;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = $nUsed;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = ($74|0)<($80|0);
  $82 = ($71>>>0)<=($77>>>0);
  $83 = ($74|0)==($80|0);
  $84 = $83 & $82;
  $85 = $81 | $84;
  $87 = $85;
 } else {
  $87 = 0;
 }
 $86 = $87&1;
 HEAP32[(308)>>2] = $86;
 $88 = (_sqlite3_memory_used()|0);
 $89 = tempRet0;
 $90 = $3;
 $91 = $90;
 $92 = HEAP32[$91>>2]|0;
 $93 = (($90) + 4)|0;
 $94 = $93;
 $95 = HEAP32[$94>>2]|0;
 $96 = (_i64Subtract(($88|0),($89|0),($92|0),($95|0))|0);
 $97 = tempRet0;
 $98 = $excess;
 $99 = $98;
 HEAP32[$99>>2] = $96;
 $100 = (($98) + 4)|0;
 $101 = $100;
 HEAP32[$101>>2] = $97;
 $102 = $excess;
 $103 = $102;
 $104 = HEAP32[$103>>2]|0;
 $105 = (($102) + 4)|0;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = ($107|0)>(0);
 $109 = ($104>>>0)>(0);
 $110 = ($107|0)==(0);
 $111 = $110 & $109;
 $112 = $108 | $111;
 if ($112) {
  $113 = $excess;
  $114 = $113;
  $115 = HEAP32[$114>>2]|0;
  $116 = (($113) + 4)|0;
  $117 = $116;
  $118 = HEAP32[$117>>2]|0;
  $119 = $115 & 2147483647;
  (_sqlite3_release_memory($119)|0);
 }
 $120 = $priorLimit;
 $121 = $120;
 $122 = HEAP32[$121>>2]|0;
 $123 = (($120) + 4)|0;
 $124 = $123;
 $125 = HEAP32[$124>>2]|0;
 $126 = $2;
 $127 = $126;
 HEAP32[$127>>2] = $122;
 $128 = (($126) + 4)|0;
 $129 = $128;
 HEAP32[$129>>2] = $125;
 $130 = $2;
 $131 = $130;
 $132 = HEAP32[$131>>2]|0;
 $133 = (($130) + 4)|0;
 $134 = $133;
 $135 = HEAP32[$134>>2]|0;
 tempRet0 = ($135);
 STACKTOP = sp;return ($132|0);
}
function _sqlite3StatusValue($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = (384 + ($1<<2)|0);
 $3 = HEAP32[$2>>2]|0;
 tempRet0 = (0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_memory_used() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $mx = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $res = sp + 8|0;
 $mx = sp;
 (_sqlite3_status64(0,$res,$mx,0)|0);
 $0 = $res;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($0) + 4)|0;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 tempRet0 = ($5);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_malloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $n;
 $2 = (_sqlite3_initialize()|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  $10 = 0;
 } else {
  $6 = HEAP32[$1>>2]|0;
  $7 = ($6|0)<(0);
  $8 = $7 << 31 >> 31;
  $9 = (_sqlite3Malloc($6,$8)|0);
  $10 = $9;
 }
 HEAP32[$0>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3Malloc($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $p = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 $7 = $2;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9|0)==(0);
 $14 = ($12|0)==(0);
 $15 = $13 & $14;
 $16 = $2;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21>>>0)>(0);
 $23 = ($18>>>0)>=(2147483392);
 $24 = ($21|0)==(0);
 $25 = $24 & $23;
 $26 = $22 | $25;
 $or$cond = $15 | $26;
 if ($or$cond) {
  HEAP32[$p>>2] = 0;
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $27 = HEAP32[8>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = $2;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  (_mallocWithAlarm($31,$p)|0);
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 } else {
  $35 = HEAP32[(44)>>2]|0;
  $36 = $2;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (FUNCTION_TABLE_ii[$35 & 255]($38)|0);
  HEAP32[$p>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 return (0)|0;
}
function _sqlite3_malloc64($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_initialize()|0);
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$2>>2] = 0;
  $17 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($17|0);
 } else {
  $10 = $3;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (_sqlite3Malloc($12,$15)|0);
  HEAP32[$2>>2] = $16;
  $17 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3_free($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[8>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_sqlite3MallocSize($5)|0);
  _sqlite3StatusDown(0,$6);
  _sqlite3StatusDown(9,1);
  $7 = HEAP32[(48)>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$7 & 255]($8);
  STACKTOP = sp;return;
 } else {
  $9 = HEAP32[(48)>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$9 & 255]($10);
  STACKTOP = sp;return;
 }
}
function _sqlite3StatusDown($op,$N) {
 $op = $op|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (384 + ($3<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - ($2))|0;
 HEAP32[$4>>2] = $6;
 STACKTOP = sp;return;
}
function _sqlite3MallocSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[(56)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (FUNCTION_TABLE_ii[$1 & 255]($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_realloc($pOld,$n) {
 $pOld = $pOld|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pOld;
 HEAP32[$2>>2] = $n;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  HEAP32[$2>>2] = 0;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)<(0);
 $10 = $9 << 31 >> 31;
 $11 = (_sqlite3Realloc($7,$8,$10)|0);
 HEAP32[$0>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3Realloc($pOld,$0,$1) {
 $pOld = $pOld|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nDiff = 0, $nNew = 0, $nOld = 0, $or$cond = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp;
 $nOld = sp + 20|0;
 $nNew = sp + 16|0;
 $nDiff = sp + 12|0;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $pOld;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 if ($10) {
  $17 = (_sqlite3Malloc($13,$16)|0);
  HEAP32[$2>>2] = $17;
  $116 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $18 = ($13|0)==(0);
 $19 = ($16|0)==(0);
 $20 = $18 & $19;
 if ($20) {
  $21 = HEAP32[$3>>2]|0;
  _sqlite3_free($21);
  HEAP32[$2>>2] = 0;
  $116 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $22 = $4;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27>>>0)>(0);
 $29 = ($24>>>0)>=(2147483392);
 $30 = ($27|0)==(0);
 $31 = $30 & $29;
 $32 = $28 | $31;
 if ($32) {
  HEAP32[$2>>2] = 0;
  $116 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $33 = HEAP32[$3>>2]|0;
 $34 = (_sqlite3MallocSize($33)|0);
 HEAP32[$nOld>>2] = $34;
 $35 = HEAP32[(60)>>2]|0;
 $36 = $4;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (FUNCTION_TABLE_ii[$35 & 255]($38)|0);
 HEAP32[$nNew>>2] = $42;
 $43 = HEAP32[$nOld>>2]|0;
 $44 = HEAP32[$nNew>>2]|0;
 $45 = ($43|0)==($44|0);
 do {
  if ($45) {
   $46 = HEAP32[$3>>2]|0;
   HEAP32[$pNew>>2] = $46;
  } else {
   $47 = HEAP32[8>>2]|0;
   $48 = ($47|0)!=(0);
   if (!($48)) {
    $111 = HEAP32[(52)>>2]|0;
    $112 = HEAP32[$3>>2]|0;
    $113 = HEAP32[$nNew>>2]|0;
    $114 = (FUNCTION_TABLE_iii[$111 & 255]($112,$113)|0);
    HEAP32[$pNew>>2] = $114;
    break;
   }
   $49 = $4;
   $50 = $49;
   $51 = HEAP32[$50>>2]|0;
   $52 = (($49) + 4)|0;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   _sqlite3StatusHighwater(5,$51);
   $55 = HEAP32[$nNew>>2]|0;
   $56 = HEAP32[$nOld>>2]|0;
   $57 = (($55) - ($56))|0;
   HEAP32[$nDiff>>2] = $57;
   $58 = (_sqlite3StatusValue(0)|0);
   $59 = tempRet0;
   $60 = (288);
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($60) + 4)|0;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = HEAP32[$nDiff>>2]|0;
   $67 = ($66|0)<(0);
   $68 = $67 << 31 >> 31;
   $69 = (_i64Subtract(($62|0),($65|0),($66|0),($68|0))|0);
   $70 = tempRet0;
   $71 = ($59|0)>($70|0);
   $72 = ($58>>>0)>=($69>>>0);
   $73 = ($59|0)==($70|0);
   $74 = $73 & $72;
   $75 = $71 | $74;
   if ($75) {
    $76 = HEAP32[$nDiff>>2]|0;
    _sqlite3MallocAlarm($76);
   }
   $77 = HEAP32[(52)>>2]|0;
   $78 = HEAP32[$3>>2]|0;
   $79 = HEAP32[$nNew>>2]|0;
   $80 = (FUNCTION_TABLE_iii[$77 & 255]($78,$79)|0);
   HEAP32[$pNew>>2] = $80;
   $81 = HEAP32[$pNew>>2]|0;
   $82 = ($81|0)==(0|0);
   $83 = (288);
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (($83) + 4)|0;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)>(0);
   $90 = ($85>>>0)>(0);
   $91 = ($88|0)==(0);
   $92 = $91 & $90;
   $93 = $89 | $92;
   $or$cond = $82 & $93;
   if ($or$cond) {
    $94 = $4;
    $95 = $94;
    $96 = HEAP32[$95>>2]|0;
    $97 = (($94) + 4)|0;
    $98 = $97;
    $99 = HEAP32[$98>>2]|0;
    _sqlite3MallocAlarm($96);
    $100 = HEAP32[(52)>>2]|0;
    $101 = HEAP32[$3>>2]|0;
    $102 = HEAP32[$nNew>>2]|0;
    $103 = (FUNCTION_TABLE_iii[$100 & 255]($101,$102)|0);
    HEAP32[$pNew>>2] = $103;
   }
   $104 = HEAP32[$pNew>>2]|0;
   $105 = ($104|0)!=(0|0);
   if ($105) {
    $106 = HEAP32[$pNew>>2]|0;
    $107 = (_sqlite3MallocSize($106)|0);
    HEAP32[$nNew>>2] = $107;
    $108 = HEAP32[$nNew>>2]|0;
    $109 = HEAP32[$nOld>>2]|0;
    $110 = (($108) - ($109))|0;
    _sqlite3StatusUp(0,$110);
   }
  }
 } while(0);
 $115 = HEAP32[$pNew>>2]|0;
 HEAP32[$2>>2] = $115;
 $116 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($116|0);
}
function _sqlite3_realloc64($pOld,$0,$1) {
 $pOld = $pOld|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $pOld;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = (_sqlite3_initialize()|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$2>>2] = 0;
  $19 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $11 = HEAP32[$3>>2]|0;
  $12 = $4;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($12) + 4)|0;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3Realloc($11,$14,$17)|0);
  HEAP32[$2>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3_vmprintf($zFormat,$ap) {
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $z = 0, $zBase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $z = sp + 28|0;
 $zBase = sp + 44|0;
 $acc = sp;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  _sqlite3StrAccumInit($acc,0,$zBase,70,1000000000);
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  _sqlite3VXPrintf($acc,$5,$6);
  $7 = (_sqlite3StrAccumFinish($acc)|0);
  HEAP32[$z>>2] = $7;
  $8 = HEAP32[$z>>2]|0;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3StrAccumInit($p,$db,$zBase,$n,$mx) {
 $p = $p|0;
 $db = $db|0;
 $zBase = $zBase|0;
 $n = $n|0;
 $mx = $mx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zBase;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $mx;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$9>>2] = $5;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 16|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 20|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 24|0);
 HEAP8[$21>>0] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 25|0);
 HEAP8[$23>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VXPrintf($pAccum,$fmt,$ap) {
 $pAccum = $pAccum|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0;
 var $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0;
 var $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0;
 var $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0;
 var $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0;
 var $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0;
 var $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0;
 var $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0;
 var $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0;
 var $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0;
 var $502 = 0, $503 = 0.0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0.0, $512 = 0, $513 = 0, $514 = 0.0, $515 = 0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0, $52 = 0;
 var $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0.0, $536 = 0.0, $537 = 0, $538 = 0;
 var $539 = 0, $54 = 0, $540 = 0.0, $541 = 0.0, $542 = 0.0, $543 = 0.0, $544 = 0, $545 = 0, $546 = 0.0, $547 = 0, $548 = 0.0, $549 = 0.0, $55 = 0, $550 = 0.0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0.0, $556 = 0.0;
 var $557 = 0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0.0, $561 = 0.0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0.0, $567 = 0.0, $568 = 0, $569 = 0, $57 = 0, $570 = 0.0, $571 = 0.0, $572 = 0.0, $573 = 0, $574 = 0;
 var $575 = 0, $576 = 0, $577 = 0.0, $578 = 0.0, $579 = 0, $58 = 0, $580 = 0, $581 = 0.0, $582 = 0.0, $583 = 0.0, $584 = 0, $585 = 0.0, $586 = 0.0, $587 = 0, $588 = 0, $589 = 0.0, $59 = 0, $590 = 0, $591 = 0.0, $592 = 0.0;
 var $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0;
 var $610 = 0, $611 = 0.0, $612 = 0.0, $613 = 0.0, $614 = 0.0, $615 = 0, $616 = 0.0, $617 = 0.0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0;
 var $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0;
 var $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0;
 var $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0;
 var $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0;
 var $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0;
 var $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0;
 var $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0;
 var $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0;
 var $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0;
 var $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0;
 var $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0;
 var $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0;
 var $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0;
 var $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0;
 var $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0;
 var $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0;
 var $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0;
 var $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0;
 var $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0;
 var $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0;
 var $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $arglist_current = 0, $arglist_current12 = 0, $arglist_current15 = 0, $arglist_current18 = 0, $arglist_current21 = 0, $arglist_current24 = 0, $arglist_current27 = 0, $arglist_current30 = 0, $arglist_current33 = 0;
 var $arglist_current36 = 0, $arglist_current39 = 0, $arglist_current42 = 0, $arglist_current45 = 0, $arglist_current48 = 0, $arglist_current51 = 0, $arglist_next = 0, $arglist_next13 = 0, $arglist_next16 = 0, $arglist_next19 = 0, $arglist_next22 = 0, $arglist_next25 = 0, $arglist_next28 = 0, $arglist_next31 = 0, $arglist_next34 = 0, $arglist_next37 = 0, $arglist_next40 = 0, $arglist_next43 = 0, $arglist_next46 = 0, $arglist_next49 = 0;
 var $arglist_next52 = 0, $bArgList = 0, $base = 0, $buf = 0, $bufpt = 0, $c = 0, $ch = 0, $cset = 0, $done = 0, $e2 = 0, $escarg = 0, $exp = 0, $expanded = 0, $expanded101 = 0, $expanded102 = 0, $expanded104 = 0, $expanded105 = 0, $expanded106 = 0, $expanded108 = 0, $expanded109 = 0;
 var $expanded111 = 0, $expanded112 = 0, $expanded113 = 0, $expanded115 = 0, $expanded116 = 0, $expanded118 = 0, $expanded119 = 0, $expanded120 = 0, $expanded122 = 0, $expanded123 = 0, $expanded125 = 0, $expanded126 = 0, $expanded127 = 0, $expanded129 = 0, $expanded130 = 0, $expanded132 = 0, $expanded133 = 0, $expanded134 = 0, $expanded136 = 0, $expanded137 = 0;
 var $expanded139 = 0, $expanded140 = 0, $expanded141 = 0, $expanded143 = 0, $expanded144 = 0, $expanded146 = 0, $expanded147 = 0, $expanded148 = 0, $expanded150 = 0, $expanded151 = 0, $expanded153 = 0, $expanded154 = 0, $expanded155 = 0, $expanded53 = 0, $expanded55 = 0, $expanded56 = 0, $expanded57 = 0, $expanded59 = 0, $expanded60 = 0, $expanded62 = 0;
 var $expanded63 = 0, $expanded64 = 0, $expanded66 = 0, $expanded67 = 0, $expanded69 = 0, $expanded70 = 0, $expanded71 = 0, $expanded73 = 0, $expanded74 = 0, $expanded76 = 0, $expanded77 = 0, $expanded78 = 0, $expanded80 = 0, $expanded81 = 0, $expanded83 = 0, $expanded84 = 0, $expanded85 = 0, $expanded87 = 0, $expanded88 = 0, $expanded90 = 0;
 var $expanded91 = 0, $expanded92 = 0, $expanded94 = 0, $expanded95 = 0, $expanded97 = 0, $expanded98 = 0, $expanded99 = 0, $flag_alternateform = 0, $flag_altform2 = 0, $flag_blanksign = 0, $flag_dp = 0, $flag_leftjustify = 0, $flag_long = 0, $flag_longlong = 0, $flag_plussign = 0, $flag_rtz = 0, $flag_zeropad = 0, $i = 0, $i2 = 0, $idx = 0;
 var $infop = 0, $isnull = 0, $j = 0, $k = 0, $k3 = 0, $length = 0, $longvalue = 0, $n = 0, $nOut = 0, $nPad = 0, $needQuote = 0, $nsd = 0, $or$cond = 0, $or$cond10 = 0, $or$cond4 = 0, $or$cond6 = 0, $pArgList = 0, $pItem = 0, $pSrc = 0, $pToken = 0;
 var $pre = 0, $precision = 0, $prefix = 0, $px = 0, $q = 0, $realvalue = 0, $rounder = 0, $scale = 0, $useIntern = 0, $v = 0, $width = 0, $wx = 0, $x = 0, $x1 = 0, $xtype = 0, $zExtra = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $c = sp + 164|0;
 $bufpt = sp + 160|0;
 $precision = sp + 156|0;
 $length = sp + 152|0;
 $idx = sp + 148|0;
 $width = sp + 144|0;
 $flag_leftjustify = sp + 268|0;
 $flag_plussign = sp + 267|0;
 $flag_blanksign = sp + 266|0;
 $flag_alternateform = sp + 265|0;
 $flag_altform2 = sp + 264|0;
 $flag_zeropad = sp + 263|0;
 $flag_long = sp + 262|0;
 $flag_longlong = sp + 261|0;
 $done = sp + 260|0;
 $xtype = sp + 259|0;
 $bArgList = sp + 258|0;
 $useIntern = sp + 257|0;
 $prefix = sp + 256|0;
 $longvalue = sp + 32|0;
 $realvalue = sp + 24|0;
 $infop = sp + 140|0;
 $zOut = sp + 136|0;
 $nOut = sp + 132|0;
 $zExtra = sp + 128|0;
 $exp = sp + 124|0;
 $e2 = sp + 120|0;
 $nsd = sp + 116|0;
 $rounder = sp + 16|0;
 $flag_dp = sp + 255|0;
 $flag_rtz = sp + 254|0;
 $pArgList = sp + 112|0;
 $buf = sp + 184|0;
 $wx = sp + 108|0;
 $px = sp + 104|0;
 $v = sp + 8|0;
 $x = sp + 100|0;
 $cset = sp + 96|0;
 $base = sp + 183|0;
 $pre = sp + 92|0;
 $x1 = sp + 182|0;
 $scale = sp;
 $i = sp + 88|0;
 $nPad = sp + 84|0;
 $i2 = sp + 80|0;
 $j = sp + 76|0;
 $k = sp + 72|0;
 $n = sp + 68|0;
 $isnull = sp + 64|0;
 $needQuote = sp + 60|0;
 $ch = sp + 181|0;
 $q = sp + 180|0;
 $escarg = sp + 56|0;
 $pToken = sp + 52|0;
 $pSrc = sp + 48|0;
 $k3 = sp + 44|0;
 $pItem = sp + 40|0;
 HEAP32[$0>>2] = $pAccum;
 HEAP32[$1>>2] = $fmt;
 HEAP32[$2>>2] = $ap;
 HEAP8[$xtype>>0] = 0;
 HEAP32[$zExtra>>2] = 0;
 HEAP32[$pArgList>>2] = 0;
 HEAP32[$bufpt>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 25|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 25|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = $10 & 2;
  $12 = $11&255;
  HEAP8[$bArgList>>0] = $12;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $arglist_current = HEAP32[$15>>2]|0;
   $16 = $arglist_current;
   $17 = ((0) + 4|0);
   $expanded53 = $17;
   $expanded = (($expanded53) - 1)|0;
   $18 = (($16) + ($expanded))|0;
   $19 = ((0) + 4|0);
   $expanded57 = $19;
   $expanded56 = (($expanded57) - 1)|0;
   $expanded55 = $expanded56 ^ -1;
   $20 = $18 & $expanded55;
   $21 = $20;
   $22 = HEAP32[$21>>2]|0;
   $arglist_next = ((($21)) + 4|0);
   HEAP32[$15>>2] = $arglist_next;
   HEAP32[$pArgList>>2] = $22;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 25|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $26 & 1;
  $28 = $27&255;
  HEAP8[$useIntern>>0] = $28;
 } else {
  HEAP8[$useIntern>>0] = 0;
  HEAP8[$bArgList>>0] = 0;
 }
 L7: while(1) {
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP8[$29>>0]|0;
  $31 = $30 << 24 >> 24;
  HEAP32[$c>>2] = $31;
  $32 = ($31|0)!=(0);
  if (!($32)) {
   label = 268;
   break;
  }
  $33 = HEAP32[$c>>2]|0;
  $34 = ($33|0)!=(37);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   HEAP32[$bufpt>>2] = $35;
   while(1) {
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 1|0);
    HEAP32[$1>>2] = $37;
    $38 = HEAP32[$1>>2]|0;
    $39 = HEAP8[$38>>0]|0;
    $40 = $39 << 24 >> 24;
    $41 = ($40|0)!=(0);
    if (!($41)) {
     break;
    }
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43 << 24 >> 24;
    $45 = ($44|0)!=(37);
    if (!($45)) {
     break;
    }
   }
   $46 = HEAP32[$0>>2]|0;
   $47 = HEAP32[$bufpt>>2]|0;
   $48 = HEAP32[$1>>2]|0;
   $49 = HEAP32[$bufpt>>2]|0;
   $50 = $48;
   $51 = $49;
   $52 = (($50) - ($51))|0;
   _sqlite3StrAccumAppend($46,$47,$52);
   $53 = HEAP32[$1>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54 << 24 >> 24;
   $56 = ($55|0)==(0);
   if ($56) {
    label = 268;
    break;
   }
  }
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 1|0);
  HEAP32[$1>>2] = $58;
  $59 = HEAP8[$58>>0]|0;
  $60 = $59 << 24 >> 24;
  HEAP32[$c>>2] = $60;
  $61 = ($60|0)==(0);
  if ($61) {
   label = 13;
   break;
  }
  HEAP8[$flag_zeropad>>0] = 0;
  HEAP8[$flag_altform2>>0] = 0;
  HEAP8[$flag_alternateform>>0] = 0;
  HEAP8[$flag_blanksign>>0] = 0;
  HEAP8[$flag_plussign>>0] = 0;
  HEAP8[$flag_leftjustify>>0] = 0;
  HEAP8[$done>>0] = 0;
  while(1) {
   $63 = HEAP32[$c>>2]|0;
   switch ($63|0) {
   case 45:  {
    HEAP8[$flag_leftjustify>>0] = 1;
    break;
   }
   case 43:  {
    HEAP8[$flag_plussign>>0] = 1;
    break;
   }
   case 32:  {
    HEAP8[$flag_blanksign>>0] = 1;
    break;
   }
   case 35:  {
    HEAP8[$flag_alternateform>>0] = 1;
    break;
   }
   case 33:  {
    HEAP8[$flag_altform2>>0] = 1;
    break;
   }
   case 48:  {
    HEAP8[$flag_zeropad>>0] = 1;
    break;
   }
   default: {
    HEAP8[$done>>0] = 1;
   }
   }
   $64 = HEAP8[$done>>0]|0;
   $65 = ($64<<24>>24)!=(0);
   if ($65) {
    break;
   }
   $66 = HEAP32[$1>>2]|0;
   $67 = ((($66)) + 1|0);
   HEAP32[$1>>2] = $67;
   $68 = HEAP8[$67>>0]|0;
   $69 = $68 << 24 >> 24;
   HEAP32[$c>>2] = $69;
   $70 = ($69|0)!=(0);
   if (!($70)) {
    break;
   }
  }
  $71 = HEAP32[$c>>2]|0;
  $72 = ($71|0)==(42);
  if ($72) {
   $73 = HEAP8[$bArgList>>0]|0;
   $74 = ($73<<24>>24)!=(0);
   if ($74) {
    $75 = HEAP32[$pArgList>>2]|0;
    $76 = (_getIntArg($75)|0);
    $77 = tempRet0;
    HEAP32[$width>>2] = $76;
   } else {
    $78 = HEAP32[$2>>2]|0;
    $arglist_current12 = HEAP32[$78>>2]|0;
    $79 = $arglist_current12;
    $80 = ((0) + 4|0);
    $expanded60 = $80;
    $expanded59 = (($expanded60) - 1)|0;
    $81 = (($79) + ($expanded59))|0;
    $82 = ((0) + 4|0);
    $expanded64 = $82;
    $expanded63 = (($expanded64) - 1)|0;
    $expanded62 = $expanded63 ^ -1;
    $83 = $81 & $expanded62;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $arglist_next13 = ((($84)) + 4|0);
    HEAP32[$78>>2] = $arglist_next13;
    HEAP32[$width>>2] = $85;
   }
   $86 = HEAP32[$width>>2]|0;
   $87 = ($86|0)<(0);
   if ($87) {
    HEAP8[$flag_leftjustify>>0] = 1;
    $88 = HEAP32[$width>>2]|0;
    $89 = ($88|0)>=(-2147483647);
    $90 = HEAP32[$width>>2]|0;
    $91 = (0 - ($90))|0;
    $92 = $89 ? $91 : 0;
    HEAP32[$width>>2] = $92;
   }
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 1|0);
   HEAP32[$1>>2] = $94;
   $95 = HEAP8[$94>>0]|0;
   $96 = $95 << 24 >> 24;
   HEAP32[$c>>2] = $96;
  } else {
   HEAP32[$wx>>2] = 0;
   while(1) {
    $97 = HEAP32[$c>>2]|0;
    $98 = ($97|0)>=(48);
    $99 = HEAP32[$c>>2]|0;
    $100 = ($99|0)<=(57);
    $101 = $98 ? $100 : 0;
    $102 = HEAP32[$wx>>2]|0;
    if (!($101)) {
     break;
    }
    $103 = ($102*10)|0;
    $104 = HEAP32[$c>>2]|0;
    $105 = (($103) + ($104))|0;
    $106 = (($105) - 48)|0;
    HEAP32[$wx>>2] = $106;
    $107 = HEAP32[$1>>2]|0;
    $108 = ((($107)) + 1|0);
    HEAP32[$1>>2] = $108;
    $109 = HEAP8[$108>>0]|0;
    $110 = $109 << 24 >> 24;
    HEAP32[$c>>2] = $110;
   }
   $111 = $102 & 2147483647;
   HEAP32[$width>>2] = $111;
  }
  $112 = HEAP32[$c>>2]|0;
  $113 = ($112|0)==(46);
  do {
   if ($113) {
    $114 = HEAP32[$1>>2]|0;
    $115 = ((($114)) + 1|0);
    HEAP32[$1>>2] = $115;
    $116 = HEAP8[$115>>0]|0;
    $117 = $116 << 24 >> 24;
    HEAP32[$c>>2] = $117;
    $118 = HEAP32[$c>>2]|0;
    $119 = ($118|0)==(42);
    if (!($119)) {
     HEAP32[$px>>2] = 0;
     while(1) {
      $144 = HEAP32[$c>>2]|0;
      $145 = ($144|0)>=(48);
      $146 = HEAP32[$c>>2]|0;
      $147 = ($146|0)<=(57);
      $148 = $145 ? $147 : 0;
      $149 = HEAP32[$px>>2]|0;
      if (!($148)) {
       break;
      }
      $150 = ($149*10)|0;
      $151 = HEAP32[$c>>2]|0;
      $152 = (($150) + ($151))|0;
      $153 = (($152) - 48)|0;
      HEAP32[$px>>2] = $153;
      $154 = HEAP32[$1>>2]|0;
      $155 = ((($154)) + 1|0);
      HEAP32[$1>>2] = $155;
      $156 = HEAP8[$155>>0]|0;
      $157 = $156 << 24 >> 24;
      HEAP32[$c>>2] = $157;
     }
     $158 = $149 & 2147483647;
     HEAP32[$precision>>2] = $158;
     break;
    }
    $120 = HEAP8[$bArgList>>0]|0;
    $121 = ($120<<24>>24)!=(0);
    if ($121) {
     $122 = HEAP32[$pArgList>>2]|0;
     $123 = (_getIntArg($122)|0);
     $124 = tempRet0;
     HEAP32[$precision>>2] = $123;
    } else {
     $125 = HEAP32[$2>>2]|0;
     $arglist_current15 = HEAP32[$125>>2]|0;
     $126 = $arglist_current15;
     $127 = ((0) + 4|0);
     $expanded67 = $127;
     $expanded66 = (($expanded67) - 1)|0;
     $128 = (($126) + ($expanded66))|0;
     $129 = ((0) + 4|0);
     $expanded71 = $129;
     $expanded70 = (($expanded71) - 1)|0;
     $expanded69 = $expanded70 ^ -1;
     $130 = $128 & $expanded69;
     $131 = $130;
     $132 = HEAP32[$131>>2]|0;
     $arglist_next16 = ((($131)) + 4|0);
     HEAP32[$125>>2] = $arglist_next16;
     HEAP32[$precision>>2] = $132;
    }
    $133 = HEAP32[$1>>2]|0;
    $134 = ((($133)) + 1|0);
    HEAP32[$1>>2] = $134;
    $135 = HEAP8[$134>>0]|0;
    $136 = $135 << 24 >> 24;
    HEAP32[$c>>2] = $136;
    $137 = HEAP32[$precision>>2]|0;
    $138 = ($137|0)<(0);
    if ($138) {
     $139 = HEAP32[$precision>>2]|0;
     $140 = ($139|0)>=(-2147483647);
     $141 = HEAP32[$precision>>2]|0;
     $142 = (0 - ($141))|0;
     $143 = $140 ? $142 : -1;
     HEAP32[$precision>>2] = $143;
    }
   } else {
    HEAP32[$precision>>2] = -1;
   }
  } while(0);
  $159 = HEAP32[$c>>2]|0;
  $160 = ($159|0)==(108);
  do {
   if ($160) {
    HEAP8[$flag_long>>0] = 1;
    $161 = HEAP32[$1>>2]|0;
    $162 = ((($161)) + 1|0);
    HEAP32[$1>>2] = $162;
    $163 = HEAP8[$162>>0]|0;
    $164 = $163 << 24 >> 24;
    HEAP32[$c>>2] = $164;
    $165 = HEAP32[$c>>2]|0;
    $166 = ($165|0)==(108);
    if ($166) {
     HEAP8[$flag_longlong>>0] = 1;
     $167 = HEAP32[$1>>2]|0;
     $168 = ((($167)) + 1|0);
     HEAP32[$1>>2] = $168;
     $169 = HEAP8[$168>>0]|0;
     $170 = $169 << 24 >> 24;
     HEAP32[$c>>2] = $170;
     break;
    } else {
     HEAP8[$flag_longlong>>0] = 0;
     break;
    }
   } else {
    HEAP8[$flag_longlong>>0] = 0;
    HEAP8[$flag_long>>0] = 0;
   }
  } while(0);
  HEAP32[$infop>>2] = 18005;
  HEAP8[$xtype>>0] = 0;
  HEAP32[$idx>>2] = 0;
  while(1) {
   $171 = HEAP32[$idx>>2]|0;
   $172 = ($171|0)<(23);
   if (!($172)) {
    break;
   }
   $173 = HEAP32[$c>>2]|0;
   $174 = HEAP32[$idx>>2]|0;
   $175 = (18005 + (($174*6)|0)|0);
   $176 = HEAP8[$175>>0]|0;
   $177 = $176 << 24 >> 24;
   $178 = ($173|0)==($177|0);
   $179 = HEAP32[$idx>>2]|0;
   if ($178) {
    label = 56;
    break;
   }
   $193 = (($179) + 1)|0;
   HEAP32[$idx>>2] = $193;
  }
  if ((label|0) == 56) {
   label = 0;
   $180 = (18005 + (($179*6)|0)|0);
   HEAP32[$infop>>2] = $180;
   $181 = HEAP8[$useIntern>>0]|0;
   $182 = $181&255;
   $183 = ($182|0)!=(0);
   if (!($183)) {
    $184 = HEAP32[$infop>>2]|0;
    $185 = ((($184)) + 2|0);
    $186 = HEAP8[$185>>0]|0;
    $187 = $186&255;
    $188 = $187 & 2;
    $189 = ($188|0)==(0);
    if (!($189)) {
     label = 268;
     break;
    }
   }
   $190 = HEAP32[$infop>>2]|0;
   $191 = ((($190)) + 3|0);
   $192 = HEAP8[$191>>0]|0;
   HEAP8[$xtype>>0] = $192;
  }
  $194 = HEAP8[$xtype>>0]|0;
  $195 = $194&255;
  L79: do {
   switch ($195|0) {
   case 14:  {
    HEAP8[$flag_longlong>>0] = 0;
    HEAP8[$flag_long>>0] = 1;
    label = 62;
    break;
   }
   case 1: case 16:  {
    label = 62;
    break;
   }
   case 4: case 3: case 2:  {
    $500 = HEAP8[$bArgList>>0]|0;
    $501 = ($500<<24>>24)!=(0);
    if ($501) {
     $502 = HEAP32[$pArgList>>2]|0;
     $503 = (+_getDoubleArg($502));
     HEAPF64[$realvalue>>3] = $503;
    } else {
     $504 = HEAP32[$2>>2]|0;
     $arglist_current30 = HEAP32[$504>>2]|0;
     $505 = $arglist_current30;
     $506 = ((0) + 8|0);
     $expanded102 = $506;
     $expanded101 = (($expanded102) - 1)|0;
     $507 = (($505) + ($expanded101))|0;
     $508 = ((0) + 8|0);
     $expanded106 = $508;
     $expanded105 = (($expanded106) - 1)|0;
     $expanded104 = $expanded105 ^ -1;
     $509 = $507 & $expanded104;
     $510 = $509;
     $511 = +HEAPF64[$510>>3];
     $arglist_next31 = ((($510)) + 8|0);
     HEAP32[$504>>2] = $arglist_next31;
     HEAPF64[$realvalue>>3] = $511;
    }
    $512 = HEAP32[$precision>>2]|0;
    $513 = ($512|0)<(0);
    if ($513) {
     HEAP32[$precision>>2] = 6;
    }
    $514 = +HEAPF64[$realvalue>>3];
    $515 = $514 < 0.0;
    do {
     if ($515) {
      $516 = +HEAPF64[$realvalue>>3];
      $517 = -$516;
      HEAPF64[$realvalue>>3] = $517;
      HEAP8[$prefix>>0] = 45;
     } else {
      $518 = HEAP8[$flag_plussign>>0]|0;
      $519 = ($518<<24>>24)!=(0);
      if ($519) {
       HEAP8[$prefix>>0] = 43;
       break;
      }
      $520 = HEAP8[$flag_blanksign>>0]|0;
      $521 = ($520<<24>>24)!=(0);
      if ($521) {
       HEAP8[$prefix>>0] = 32;
       break;
      } else {
       HEAP8[$prefix>>0] = 0;
       break;
      }
     }
    } while(0);
    $522 = HEAP8[$xtype>>0]|0;
    $523 = $522&255;
    $524 = ($523|0)==(4);
    $525 = HEAP32[$precision>>2]|0;
    $526 = ($525|0)>(0);
    $or$cond = $524 & $526;
    if ($or$cond) {
     $527 = HEAP32[$precision>>2]|0;
     $528 = (($527) + -1)|0;
     HEAP32[$precision>>2] = $528;
    }
    $529 = HEAP32[$precision>>2]|0;
    $530 = $529 & 4095;
    HEAP32[$idx>>2] = $530;
    HEAPF64[$rounder>>3] = 0.5;
    while(1) {
     $531 = HEAP32[$idx>>2]|0;
     $532 = ($531|0)>(0);
     if (!($532)) {
      break;
     }
     $533 = HEAP32[$idx>>2]|0;
     $534 = (($533) + -1)|0;
     HEAP32[$idx>>2] = $534;
     $535 = +HEAPF64[$rounder>>3];
     $536 = $535 * 0.10000000000000001;
     HEAPF64[$rounder>>3] = $536;
    }
    $537 = HEAP8[$xtype>>0]|0;
    $538 = $537&255;
    $539 = ($538|0)==(2);
    if ($539) {
     $540 = +HEAPF64[$rounder>>3];
     $541 = +HEAPF64[$realvalue>>3];
     $542 = $541 + $540;
     HEAPF64[$realvalue>>3] = $542;
    }
    HEAP32[$exp>>2] = 0;
    $543 = +HEAPF64[$realvalue>>3];
    $544 = (_sqlite3IsNaN($543)|0);
    $545 = ($544|0)!=(0);
    if ($545) {
     HEAP32[$bufpt>>2] = 18192;
     HEAP32[$length>>2] = 3;
     break L79;
    }
    $546 = +HEAPF64[$realvalue>>3];
    $547 = $546 > 0.0;
    if ($547) {
     HEAPF64[$scale>>3] = 1.0;
     while(1) {
      $548 = +HEAPF64[$realvalue>>3];
      $549 = +HEAPF64[$scale>>3];
      $550 = 1.0E+100 * $549;
      $551 = $548 >= $550;
      $552 = HEAP32[$exp>>2]|0;
      $553 = ($552|0)<=(350);
      $554 = $551 ? $553 : 0;
      if (!($554)) {
       break;
      }
      $555 = +HEAPF64[$scale>>3];
      $556 = $555 * 1.0E+100;
      HEAPF64[$scale>>3] = $556;
      $557 = HEAP32[$exp>>2]|0;
      $558 = (($557) + 100)|0;
      HEAP32[$exp>>2] = $558;
     }
     while(1) {
      $559 = +HEAPF64[$realvalue>>3];
      $560 = +HEAPF64[$scale>>3];
      $561 = 1.0E+10 * $560;
      $562 = $559 >= $561;
      $563 = HEAP32[$exp>>2]|0;
      $564 = ($563|0)<=(350);
      $565 = $562 ? $564 : 0;
      if (!($565)) {
       break;
      }
      $566 = +HEAPF64[$scale>>3];
      $567 = $566 * 1.0E+10;
      HEAPF64[$scale>>3] = $567;
      $568 = HEAP32[$exp>>2]|0;
      $569 = (($568) + 10)|0;
      HEAP32[$exp>>2] = $569;
     }
     while(1) {
      $570 = +HEAPF64[$realvalue>>3];
      $571 = +HEAPF64[$scale>>3];
      $572 = 10.0 * $571;
      $573 = $570 >= $572;
      $574 = HEAP32[$exp>>2]|0;
      $575 = ($574|0)<=(350);
      $576 = $573 ? $575 : 0;
      $577 = +HEAPF64[$scale>>3];
      if (!($576)) {
       break;
      }
      $578 = $577 * 10.0;
      HEAPF64[$scale>>3] = $578;
      $579 = HEAP32[$exp>>2]|0;
      $580 = (($579) + 1)|0;
      HEAP32[$exp>>2] = $580;
     }
     $581 = +HEAPF64[$realvalue>>3];
     $582 = $581 / $577;
     HEAPF64[$realvalue>>3] = $582;
     while(1) {
      $583 = +HEAPF64[$realvalue>>3];
      $584 = $583 < 1.0E-8;
      if (!($584)) {
       break;
      }
      $585 = +HEAPF64[$realvalue>>3];
      $586 = $585 * 1.0E+8;
      HEAPF64[$realvalue>>3] = $586;
      $587 = HEAP32[$exp>>2]|0;
      $588 = (($587) - 8)|0;
      HEAP32[$exp>>2] = $588;
     }
     while(1) {
      $589 = +HEAPF64[$realvalue>>3];
      $590 = $589 < 1.0;
      if (!($590)) {
       break;
      }
      $591 = +HEAPF64[$realvalue>>3];
      $592 = $591 * 10.0;
      HEAPF64[$realvalue>>3] = $592;
      $593 = HEAP32[$exp>>2]|0;
      $594 = (($593) + -1)|0;
      HEAP32[$exp>>2] = $594;
     }
     $595 = HEAP32[$exp>>2]|0;
     $596 = ($595|0)>(350);
     if ($596) {
      HEAP32[$bufpt>>2] = $buf;
      $597 = HEAP8[$prefix>>0]|0;
      HEAP8[$buf>>0] = $597;
      $598 = HEAP8[$prefix>>0]|0;
      $599 = $598 << 24 >> 24;
      $600 = ($599|0)!=(0);
      $601 = $600&1;
      $602 = (($buf) + ($601)|0);
      ;HEAP8[$602>>0]=HEAP8[18196>>0]|0;HEAP8[$602+1>>0]=HEAP8[18196+1>>0]|0;HEAP8[$602+2>>0]=HEAP8[18196+2>>0]|0;HEAP8[$602+3>>0]=HEAP8[18196+3>>0]|0;
      $603 = HEAP8[$prefix>>0]|0;
      $604 = $603 << 24 >> 24;
      $605 = ($604|0)!=(0);
      $606 = $605&1;
      $607 = (3 + ($606))|0;
      HEAP32[$length>>2] = $607;
      break L79;
     }
    }
    HEAP32[$bufpt>>2] = $buf;
    $608 = HEAP8[$xtype>>0]|0;
    $609 = $608&255;
    $610 = ($609|0)!=(2);
    if ($610) {
     $611 = +HEAPF64[$rounder>>3];
     $612 = +HEAPF64[$realvalue>>3];
     $613 = $612 + $611;
     HEAPF64[$realvalue>>3] = $613;
     $614 = +HEAPF64[$realvalue>>3];
     $615 = $614 >= 10.0;
     if ($615) {
      $616 = +HEAPF64[$realvalue>>3];
      $617 = $616 * 0.10000000000000001;
      HEAPF64[$realvalue>>3] = $617;
      $618 = HEAP32[$exp>>2]|0;
      $619 = (($618) + 1)|0;
      HEAP32[$exp>>2] = $619;
     }
    }
    $620 = HEAP8[$xtype>>0]|0;
    $621 = $620&255;
    $622 = ($621|0)==(4);
    L137: do {
     if ($622) {
      $623 = HEAP8[$flag_alternateform>>0]|0;
      $624 = ($623<<24>>24)!=(0);
      $625 = $624 ^ 1;
      $626 = $625&1;
      $627 = $626&255;
      HEAP8[$flag_rtz>>0] = $627;
      $628 = HEAP32[$exp>>2]|0;
      $629 = ($628|0)<(-4);
      do {
       if (!($629)) {
        $630 = HEAP32[$exp>>2]|0;
        $631 = HEAP32[$precision>>2]|0;
        $632 = ($630|0)>($631|0);
        if ($632) {
         break;
        }
        $633 = HEAP32[$precision>>2]|0;
        $634 = HEAP32[$exp>>2]|0;
        $635 = (($633) - ($634))|0;
        HEAP32[$precision>>2] = $635;
        HEAP8[$xtype>>0] = 2;
        break L137;
       }
      } while(0);
      HEAP8[$xtype>>0] = 3;
     } else {
      $636 = HEAP8[$flag_altform2>>0]|0;
      HEAP8[$flag_rtz>>0] = $636;
     }
    } while(0);
    $637 = HEAP8[$xtype>>0]|0;
    $638 = $637&255;
    $639 = ($638|0)==(3);
    if ($639) {
     HEAP32[$e2>>2] = 0;
    } else {
     $640 = HEAP32[$exp>>2]|0;
     HEAP32[$e2>>2] = $640;
    }
    $641 = HEAP32[$e2>>2]|0;
    $642 = ($641|0)>(0);
    $643 = HEAP32[$e2>>2]|0;
    $644 = $642 ? $643 : 0;
    $645 = ($644|0)<(0);
    $646 = $645 << 31 >> 31;
    $647 = HEAP32[$precision>>2]|0;
    $648 = ($647|0)<(0);
    $649 = $648 << 31 >> 31;
    $650 = (_i64Add(($644|0),($646|0),($647|0),($649|0))|0);
    $651 = tempRet0;
    $652 = HEAP32[$width>>2]|0;
    $653 = ($652|0)<(0);
    $654 = $653 << 31 >> 31;
    $655 = (_i64Add(($650|0),($651|0),($652|0),($654|0))|0);
    $656 = tempRet0;
    $657 = ($656|0)>(0);
    $658 = ($655>>>0)>(55);
    $659 = ($656|0)==(0);
    $660 = $659 & $658;
    $661 = $657 | $660;
    if ($661) {
     $662 = HEAP32[$e2>>2]|0;
     $663 = ($662|0)>(0);
     $664 = HEAP32[$e2>>2]|0;
     $665 = $663 ? $664 : 0;
     $666 = ($665|0)<(0);
     $667 = $666 << 31 >> 31;
     $668 = HEAP32[$precision>>2]|0;
     $669 = ($668|0)<(0);
     $670 = $669 << 31 >> 31;
     $671 = (_i64Add(($665|0),($667|0),($668|0),($670|0))|0);
     $672 = tempRet0;
     $673 = HEAP32[$width>>2]|0;
     $674 = ($673|0)<(0);
     $675 = $674 << 31 >> 31;
     $676 = (_i64Add(($671|0),($672|0),($673|0),($675|0))|0);
     $677 = tempRet0;
     $678 = (_i64Add(($676|0),($677|0),15,0)|0);
     $679 = tempRet0;
     $680 = (_sqlite3Malloc($678,$679)|0);
     HEAP32[$zExtra>>2] = $680;
     HEAP32[$bufpt>>2] = $680;
     $681 = HEAP32[$bufpt>>2]|0;
     $682 = ($681|0)==(0|0);
     if ($682) {
      label = 161;
      break L7;
     }
    }
    $684 = HEAP32[$bufpt>>2]|0;
    HEAP32[$zOut>>2] = $684;
    $685 = HEAP8[$flag_altform2>>0]|0;
    $686 = $685&255;
    $687 = ($686*10)|0;
    $688 = (16 + ($687))|0;
    HEAP32[$nsd>>2] = $688;
    $689 = HEAP32[$precision>>2]|0;
    $690 = ($689|0)>(0);
    $691 = $690 ? 1 : 0;
    $692 = HEAP8[$flag_alternateform>>0]|0;
    $693 = $692&255;
    $694 = $691 | $693;
    $695 = HEAP8[$flag_altform2>>0]|0;
    $696 = $695&255;
    $697 = $694 | $696;
    $698 = $697&255;
    HEAP8[$flag_dp>>0] = $698;
    $699 = HEAP8[$prefix>>0]|0;
    $700 = ($699<<24>>24)!=(0);
    if ($700) {
     $701 = HEAP8[$prefix>>0]|0;
     $702 = HEAP32[$bufpt>>2]|0;
     $703 = ((($702)) + 1|0);
     HEAP32[$bufpt>>2] = $703;
     HEAP8[$702>>0] = $701;
    }
    $704 = HEAP32[$e2>>2]|0;
    $705 = ($704|0)<(0);
    L155: do {
     if ($705) {
      $706 = HEAP32[$bufpt>>2]|0;
      $707 = ((($706)) + 1|0);
      HEAP32[$bufpt>>2] = $707;
      HEAP8[$706>>0] = 48;
     } else {
      while(1) {
       $708 = HEAP32[$e2>>2]|0;
       $709 = ($708|0)>=(0);
       if (!($709)) {
        break L155;
       }
       $710 = (_et_getdigit($realvalue,$nsd)|0);
       $711 = HEAP32[$bufpt>>2]|0;
       $712 = ((($711)) + 1|0);
       HEAP32[$bufpt>>2] = $712;
       HEAP8[$711>>0] = $710;
       $713 = HEAP32[$e2>>2]|0;
       $714 = (($713) + -1)|0;
       HEAP32[$e2>>2] = $714;
      }
     }
    } while(0);
    $715 = HEAP8[$flag_dp>>0]|0;
    $716 = ($715<<24>>24)!=(0);
    if ($716) {
     $717 = HEAP32[$bufpt>>2]|0;
     $718 = ((($717)) + 1|0);
     HEAP32[$bufpt>>2] = $718;
     HEAP8[$717>>0] = 46;
    }
    $719 = HEAP32[$e2>>2]|0;
    $720 = (($719) + 1)|0;
    HEAP32[$e2>>2] = $720;
    while(1) {
     $721 = HEAP32[$e2>>2]|0;
     $722 = ($721|0)<(0);
     if (!($722)) {
      break;
     }
     $723 = HEAP32[$bufpt>>2]|0;
     $724 = ((($723)) + 1|0);
     HEAP32[$bufpt>>2] = $724;
     HEAP8[$723>>0] = 48;
     $725 = HEAP32[$precision>>2]|0;
     $726 = (($725) + -1)|0;
     HEAP32[$precision>>2] = $726;
     $727 = HEAP32[$e2>>2]|0;
     $728 = (($727) + 1)|0;
     HEAP32[$e2>>2] = $728;
    }
    while(1) {
     $729 = HEAP32[$precision>>2]|0;
     $730 = (($729) + -1)|0;
     HEAP32[$precision>>2] = $730;
     $731 = ($729|0)>(0);
     if (!($731)) {
      break;
     }
     $732 = (_et_getdigit($realvalue,$nsd)|0);
     $733 = HEAP32[$bufpt>>2]|0;
     $734 = ((($733)) + 1|0);
     HEAP32[$bufpt>>2] = $734;
     HEAP8[$733>>0] = $732;
    }
    $735 = HEAP8[$flag_rtz>>0]|0;
    $736 = $735&255;
    $737 = ($736|0)!=(0);
    do {
     if ($737) {
      $738 = HEAP8[$flag_dp>>0]|0;
      $739 = $738&255;
      $740 = ($739|0)!=(0);
      if (!($740)) {
       break;
      }
      while(1) {
       $741 = HEAP32[$bufpt>>2]|0;
       $742 = ((($741)) + -1|0);
       $743 = HEAP8[$742>>0]|0;
       $744 = $743 << 24 >> 24;
       $745 = ($744|0)==(48);
       $746 = HEAP32[$bufpt>>2]|0;
       $747 = ((($746)) + -1|0);
       if (!($745)) {
        break;
       }
       HEAP32[$bufpt>>2] = $747;
       HEAP8[$747>>0] = 0;
      }
      $748 = HEAP8[$747>>0]|0;
      $749 = $748 << 24 >> 24;
      $750 = ($749|0)==(46);
      if (!($750)) {
       break;
      }
      $751 = HEAP8[$flag_altform2>>0]|0;
      $752 = ($751<<24>>24)!=(0);
      $753 = HEAP32[$bufpt>>2]|0;
      if ($752) {
       $754 = ((($753)) + 1|0);
       HEAP32[$bufpt>>2] = $754;
       HEAP8[$753>>0] = 48;
       break;
      } else {
       $755 = ((($753)) + -1|0);
       HEAP32[$bufpt>>2] = $755;
       HEAP8[$755>>0] = 0;
       break;
      }
     }
    } while(0);
    $756 = HEAP8[$xtype>>0]|0;
    $757 = $756&255;
    $758 = ($757|0)==(3);
    if ($758) {
     $759 = HEAP32[$infop>>2]|0;
     $760 = ((($759)) + 4|0);
     $761 = HEAP8[$760>>0]|0;
     $762 = $761&255;
     $763 = (18152 + ($762)|0);
     $764 = HEAP8[$763>>0]|0;
     $765 = HEAP32[$bufpt>>2]|0;
     $766 = ((($765)) + 1|0);
     HEAP32[$bufpt>>2] = $766;
     HEAP8[$765>>0] = $764;
     $767 = HEAP32[$exp>>2]|0;
     $768 = ($767|0)<(0);
     $769 = HEAP32[$bufpt>>2]|0;
     $770 = ((($769)) + 1|0);
     HEAP32[$bufpt>>2] = $770;
     if ($768) {
      HEAP8[$769>>0] = 45;
      $771 = HEAP32[$exp>>2]|0;
      $772 = (0 - ($771))|0;
      HEAP32[$exp>>2] = $772;
     } else {
      HEAP8[$769>>0] = 43;
     }
     $773 = HEAP32[$exp>>2]|0;
     $774 = ($773|0)>=(100);
     if ($774) {
      $775 = HEAP32[$exp>>2]|0;
      $776 = (($775|0) / 100)&-1;
      $777 = (($776) + 48)|0;
      $778 = $777&255;
      $779 = HEAP32[$bufpt>>2]|0;
      $780 = ((($779)) + 1|0);
      HEAP32[$bufpt>>2] = $780;
      HEAP8[$779>>0] = $778;
      $781 = HEAP32[$exp>>2]|0;
      $782 = (($781|0) % 100)&-1;
      HEAP32[$exp>>2] = $782;
     }
     $783 = HEAP32[$exp>>2]|0;
     $784 = (($783|0) / 10)&-1;
     $785 = (($784) + 48)|0;
     $786 = $785&255;
     $787 = HEAP32[$bufpt>>2]|0;
     $788 = ((($787)) + 1|0);
     HEAP32[$bufpt>>2] = $788;
     HEAP8[$787>>0] = $786;
     $789 = HEAP32[$exp>>2]|0;
     $790 = (($789|0) % 10)&-1;
     $791 = (($790) + 48)|0;
     $792 = $791&255;
     $793 = HEAP32[$bufpt>>2]|0;
     $794 = ((($793)) + 1|0);
     HEAP32[$bufpt>>2] = $794;
     HEAP8[$793>>0] = $792;
    }
    $795 = HEAP32[$bufpt>>2]|0;
    HEAP8[$795>>0] = 0;
    $796 = HEAP32[$bufpt>>2]|0;
    $797 = HEAP32[$zOut>>2]|0;
    $798 = $796;
    $799 = $797;
    $800 = (($798) - ($799))|0;
    HEAP32[$length>>2] = $800;
    $801 = HEAP32[$zOut>>2]|0;
    HEAP32[$bufpt>>2] = $801;
    $802 = HEAP8[$flag_zeropad>>0]|0;
    $803 = $802&255;
    $804 = ($803|0)==(0);
    $805 = HEAP8[$flag_leftjustify>>0]|0;
    $806 = ($805<<24>>24)!=(0);
    $or$cond4 = $804 | $806;
    if ($or$cond4) {
     break L79;
    }
    $807 = HEAP32[$length>>2]|0;
    $808 = HEAP32[$width>>2]|0;
    $809 = ($807|0)<($808|0);
    if (!($809)) {
     break L79;
    }
    $810 = HEAP32[$width>>2]|0;
    $811 = HEAP32[$length>>2]|0;
    $812 = (($810) - ($811))|0;
    HEAP32[$nPad>>2] = $812;
    $813 = HEAP32[$width>>2]|0;
    HEAP32[$i>>2] = $813;
    while(1) {
     $814 = HEAP32[$i>>2]|0;
     $815 = HEAP32[$nPad>>2]|0;
     $816 = ($814|0)>=($815|0);
     if (!($816)) {
      break;
     }
     $817 = HEAP32[$i>>2]|0;
     $818 = HEAP32[$nPad>>2]|0;
     $819 = (($817) - ($818))|0;
     $820 = HEAP32[$bufpt>>2]|0;
     $821 = (($820) + ($819)|0);
     $822 = HEAP8[$821>>0]|0;
     $823 = HEAP32[$i>>2]|0;
     $824 = HEAP32[$bufpt>>2]|0;
     $825 = (($824) + ($823)|0);
     HEAP8[$825>>0] = $822;
     $826 = HEAP32[$i>>2]|0;
     $827 = (($826) + -1)|0;
     HEAP32[$i>>2] = $827;
    }
    $828 = HEAP8[$prefix>>0]|0;
    $829 = $828 << 24 >> 24;
    $830 = ($829|0)!=(0);
    $831 = $830&1;
    HEAP32[$i>>2] = $831;
    while(1) {
     $832 = HEAP32[$nPad>>2]|0;
     $833 = (($832) + -1)|0;
     HEAP32[$nPad>>2] = $833;
     $834 = ($832|0)!=(0);
     if (!($834)) {
      break;
     }
     $835 = HEAP32[$i>>2]|0;
     $836 = (($835) + 1)|0;
     HEAP32[$i>>2] = $836;
     $837 = HEAP32[$bufpt>>2]|0;
     $838 = (($837) + ($835)|0);
     HEAP8[$838>>0] = 48;
    }
    $839 = HEAP32[$width>>2]|0;
    HEAP32[$length>>2] = $839;
    break;
   }
   case 5:  {
    $840 = HEAP8[$bArgList>>0]|0;
    $841 = ($840<<24>>24)!=(0);
    if (!($841)) {
     $842 = HEAP32[$0>>2]|0;
     $843 = ((($842)) + 12|0);
     $844 = HEAP32[$843>>2]|0;
     $845 = HEAP32[$2>>2]|0;
     $arglist_current33 = HEAP32[$845>>2]|0;
     $846 = $arglist_current33;
     $847 = ((0) + 4|0);
     $expanded109 = $847;
     $expanded108 = (($expanded109) - 1)|0;
     $848 = (($846) + ($expanded108))|0;
     $849 = ((0) + 4|0);
     $expanded113 = $849;
     $expanded112 = (($expanded113) - 1)|0;
     $expanded111 = $expanded112 ^ -1;
     $850 = $848 & $expanded111;
     $851 = $850;
     $852 = HEAP32[$851>>2]|0;
     $arglist_next34 = ((($851)) + 4|0);
     HEAP32[$845>>2] = $arglist_next34;
     HEAP32[$852>>2] = $844;
    }
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   case 8:  {
    HEAP8[$buf>>0] = 37;
    HEAP32[$bufpt>>2] = $buf;
    HEAP32[$length>>2] = 1;
    break;
   }
   case 9:  {
    $853 = HEAP8[$bArgList>>0]|0;
    $854 = ($853<<24>>24)!=(0);
    if ($854) {
     $855 = HEAP32[$pArgList>>2]|0;
     $856 = (_getTextArg($855)|0);
     HEAP32[$bufpt>>2] = $856;
     $857 = HEAP32[$bufpt>>2]|0;
     $858 = ($857|0)!=(0|0);
     if ($858) {
      $859 = HEAP32[$bufpt>>2]|0;
      $860 = HEAP8[$859>>0]|0;
      $861 = $860 << 24 >> 24;
      $862 = $861;
     } else {
      $862 = 0;
     }
     HEAP32[$c>>2] = $862;
    } else {
     $863 = HEAP32[$2>>2]|0;
     $arglist_current36 = HEAP32[$863>>2]|0;
     $864 = $arglist_current36;
     $865 = ((0) + 4|0);
     $expanded116 = $865;
     $expanded115 = (($expanded116) - 1)|0;
     $866 = (($864) + ($expanded115))|0;
     $867 = ((0) + 4|0);
     $expanded120 = $867;
     $expanded119 = (($expanded120) - 1)|0;
     $expanded118 = $expanded119 ^ -1;
     $868 = $866 & $expanded118;
     $869 = $868;
     $870 = HEAP32[$869>>2]|0;
     $arglist_next37 = ((($869)) + 4|0);
     HEAP32[$863>>2] = $arglist_next37;
     HEAP32[$c>>2] = $870;
    }
    $871 = HEAP32[$precision>>2]|0;
    $872 = ($871|0)>(1);
    if ($872) {
     $873 = HEAP32[$precision>>2]|0;
     $874 = (($873) - 1)|0;
     $875 = HEAP32[$width>>2]|0;
     $876 = (($875) - ($874))|0;
     HEAP32[$width>>2] = $876;
     $877 = HEAP32[$width>>2]|0;
     $878 = ($877|0)<=(1);
     $879 = HEAP8[$flag_leftjustify>>0]|0;
     $880 = ($879<<24>>24)!=(0);
     $or$cond6 = $878 | $880;
     if (!($or$cond6)) {
      $881 = HEAP32[$0>>2]|0;
      $882 = HEAP32[$width>>2]|0;
      $883 = (($882) - 1)|0;
      _sqlite3AppendChar($881,$883,32);
      HEAP32[$width>>2] = 0;
     }
     $884 = HEAP32[$0>>2]|0;
     $885 = HEAP32[$precision>>2]|0;
     $886 = (($885) - 1)|0;
     $887 = HEAP32[$c>>2]|0;
     $888 = $887&255;
     _sqlite3AppendChar($884,$886,$888);
    }
    HEAP32[$length>>2] = 1;
    $889 = HEAP32[$c>>2]|0;
    $890 = $889&255;
    HEAP8[$buf>>0] = $890;
    HEAP32[$bufpt>>2] = $buf;
    break;
   }
   case 7: case 6:  {
    $891 = HEAP8[$bArgList>>0]|0;
    $892 = ($891<<24>>24)!=(0);
    if ($892) {
     $893 = HEAP32[$pArgList>>2]|0;
     $894 = (_getTextArg($893)|0);
     HEAP32[$bufpt>>2] = $894;
     HEAP8[$xtype>>0] = 6;
    } else {
     $895 = HEAP32[$2>>2]|0;
     $arglist_current39 = HEAP32[$895>>2]|0;
     $896 = $arglist_current39;
     $897 = ((0) + 4|0);
     $expanded123 = $897;
     $expanded122 = (($expanded123) - 1)|0;
     $898 = (($896) + ($expanded122))|0;
     $899 = ((0) + 4|0);
     $expanded127 = $899;
     $expanded126 = (($expanded127) - 1)|0;
     $expanded125 = $expanded126 ^ -1;
     $900 = $898 & $expanded125;
     $901 = $900;
     $902 = HEAP32[$901>>2]|0;
     $arglist_next40 = ((($901)) + 4|0);
     HEAP32[$895>>2] = $arglist_next40;
     HEAP32[$bufpt>>2] = $902;
    }
    $903 = HEAP32[$bufpt>>2]|0;
    $904 = ($903|0)==(0|0);
    if ($904) {
     HEAP32[$bufpt>>2] = 17532;
    } else {
     $905 = HEAP8[$xtype>>0]|0;
     $906 = $905&255;
     $907 = ($906|0)==(7);
     if ($907) {
      $908 = HEAP32[$bufpt>>2]|0;
      HEAP32[$zExtra>>2] = $908;
     }
    }
    $909 = HEAP32[$precision>>2]|0;
    $910 = ($909|0)>=(0);
    if (!($910)) {
     $922 = HEAP32[$bufpt>>2]|0;
     $923 = (_sqlite3Strlen30($922)|0);
     HEAP32[$length>>2] = $923;
     break L79;
    }
    HEAP32[$length>>2] = 0;
    while(1) {
     $911 = HEAP32[$length>>2]|0;
     $912 = HEAP32[$precision>>2]|0;
     $913 = ($911|0)<($912|0);
     if (!($913)) {
      break L79;
     }
     $914 = HEAP32[$length>>2]|0;
     $915 = HEAP32[$bufpt>>2]|0;
     $916 = (($915) + ($914)|0);
     $917 = HEAP8[$916>>0]|0;
     $918 = $917 << 24 >> 24;
     $919 = ($918|0)!=(0);
     if (!($919)) {
      break L79;
     }
     $920 = HEAP32[$length>>2]|0;
     $921 = (($920) + 1)|0;
     HEAP32[$length>>2] = $921;
    }
    break;
   }
   case 15: case 11: case 10:  {
    $924 = HEAP8[$xtype>>0]|0;
    $925 = $924&255;
    $926 = ($925|0)==(15);
    $927 = $926 ? 34 : 39;
    $928 = $927&255;
    HEAP8[$q>>0] = $928;
    $929 = HEAP8[$bArgList>>0]|0;
    $930 = ($929<<24>>24)!=(0);
    if ($930) {
     $931 = HEAP32[$pArgList>>2]|0;
     $932 = (_getTextArg($931)|0);
     HEAP32[$escarg>>2] = $932;
    } else {
     $933 = HEAP32[$2>>2]|0;
     $arglist_current42 = HEAP32[$933>>2]|0;
     $934 = $arglist_current42;
     $935 = ((0) + 4|0);
     $expanded130 = $935;
     $expanded129 = (($expanded130) - 1)|0;
     $936 = (($934) + ($expanded129))|0;
     $937 = ((0) + 4|0);
     $expanded134 = $937;
     $expanded133 = (($expanded134) - 1)|0;
     $expanded132 = $expanded133 ^ -1;
     $938 = $936 & $expanded132;
     $939 = $938;
     $940 = HEAP32[$939>>2]|0;
     $arglist_next43 = ((($939)) + 4|0);
     HEAP32[$933>>2] = $arglist_next43;
     HEAP32[$escarg>>2] = $940;
    }
    $941 = HEAP32[$escarg>>2]|0;
    $942 = ($941|0)==(0|0);
    $943 = $942&1;
    HEAP32[$isnull>>2] = $943;
    $944 = HEAP32[$isnull>>2]|0;
    $945 = ($944|0)!=(0);
    if ($945) {
     $946 = HEAP8[$xtype>>0]|0;
     $947 = $946&255;
     $948 = ($947|0)==(11);
     $949 = $948 ? 18200 : 18205;
     HEAP32[$escarg>>2] = $949;
    }
    $950 = HEAP32[$precision>>2]|0;
    HEAP32[$k>>2] = $950;
    HEAP32[$n>>2] = 0;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $951 = HEAP32[$k>>2]|0;
     $952 = ($951|0)!=(0);
     if (!($952)) {
      break;
     }
     $953 = HEAP32[$i2>>2]|0;
     $954 = HEAP32[$escarg>>2]|0;
     $955 = (($954) + ($953)|0);
     $956 = HEAP8[$955>>0]|0;
     HEAP8[$ch>>0] = $956;
     $957 = $956 << 24 >> 24;
     $958 = ($957|0)!=(0);
     if (!($958)) {
      break;
     }
     $959 = HEAP8[$ch>>0]|0;
     $960 = $959 << 24 >> 24;
     $961 = HEAP8[$q>>0]|0;
     $962 = $961 << 24 >> 24;
     $963 = ($960|0)==($962|0);
     if ($963) {
      $964 = HEAP32[$n>>2]|0;
      $965 = (($964) + 1)|0;
      HEAP32[$n>>2] = $965;
     }
     $966 = HEAP32[$i2>>2]|0;
     $967 = (($966) + 1)|0;
     HEAP32[$i2>>2] = $967;
     $968 = HEAP32[$k>>2]|0;
     $969 = (($968) + -1)|0;
     HEAP32[$k>>2] = $969;
    }
    $970 = HEAP32[$isnull>>2]|0;
    $971 = ($970|0)!=(0);
    if ($971) {
     $976 = 0;
    } else {
     $972 = HEAP8[$xtype>>0]|0;
     $973 = $972&255;
     $974 = ($973|0)==(11);
     $976 = $974;
    }
    $975 = $976&1;
    HEAP32[$needQuote>>2] = $975;
    $977 = HEAP32[$i2>>2]|0;
    $978 = (($977) + 3)|0;
    $979 = HEAP32[$n>>2]|0;
    $980 = (($979) + ($978))|0;
    HEAP32[$n>>2] = $980;
    $981 = HEAP32[$n>>2]|0;
    $982 = ($981|0)>(70);
    if ($982) {
     $983 = HEAP32[$n>>2]|0;
     $984 = ($983|0)<(0);
     $985 = $984 << 31 >> 31;
     $986 = (_sqlite3Malloc($983,$985)|0);
     HEAP32[$zExtra>>2] = $986;
     HEAP32[$bufpt>>2] = $986;
     $987 = HEAP32[$bufpt>>2]|0;
     $988 = ($987|0)==(0|0);
     if ($988) {
      label = 241;
      break L7;
     }
    } else {
     HEAP32[$bufpt>>2] = $buf;
    }
    HEAP32[$j>>2] = 0;
    $990 = HEAP32[$needQuote>>2]|0;
    $991 = ($990|0)!=(0);
    if ($991) {
     $992 = HEAP8[$q>>0]|0;
     $993 = HEAP32[$j>>2]|0;
     $994 = (($993) + 1)|0;
     HEAP32[$j>>2] = $994;
     $995 = HEAP32[$bufpt>>2]|0;
     $996 = (($995) + ($993)|0);
     HEAP8[$996>>0] = $992;
    }
    $997 = HEAP32[$i2>>2]|0;
    HEAP32[$k>>2] = $997;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $998 = HEAP32[$i2>>2]|0;
     $999 = HEAP32[$k>>2]|0;
     $1000 = ($998|0)<($999|0);
     if (!($1000)) {
      break;
     }
     $1001 = HEAP32[$i2>>2]|0;
     $1002 = HEAP32[$escarg>>2]|0;
     $1003 = (($1002) + ($1001)|0);
     $1004 = HEAP8[$1003>>0]|0;
     HEAP8[$ch>>0] = $1004;
     $1005 = HEAP32[$j>>2]|0;
     $1006 = (($1005) + 1)|0;
     HEAP32[$j>>2] = $1006;
     $1007 = HEAP32[$bufpt>>2]|0;
     $1008 = (($1007) + ($1005)|0);
     HEAP8[$1008>>0] = $1004;
     $1009 = HEAP8[$ch>>0]|0;
     $1010 = $1009 << 24 >> 24;
     $1011 = HEAP8[$q>>0]|0;
     $1012 = $1011 << 24 >> 24;
     $1013 = ($1010|0)==($1012|0);
     if ($1013) {
      $1014 = HEAP8[$ch>>0]|0;
      $1015 = HEAP32[$j>>2]|0;
      $1016 = (($1015) + 1)|0;
      HEAP32[$j>>2] = $1016;
      $1017 = HEAP32[$bufpt>>2]|0;
      $1018 = (($1017) + ($1015)|0);
      HEAP8[$1018>>0] = $1014;
     }
     $1019 = HEAP32[$i2>>2]|0;
     $1020 = (($1019) + 1)|0;
     HEAP32[$i2>>2] = $1020;
    }
    $1021 = HEAP32[$needQuote>>2]|0;
    $1022 = ($1021|0)!=(0);
    if ($1022) {
     $1023 = HEAP8[$q>>0]|0;
     $1024 = HEAP32[$j>>2]|0;
     $1025 = (($1024) + 1)|0;
     HEAP32[$j>>2] = $1025;
     $1026 = HEAP32[$bufpt>>2]|0;
     $1027 = (($1026) + ($1024)|0);
     HEAP8[$1027>>0] = $1023;
    }
    $1028 = HEAP32[$j>>2]|0;
    $1029 = HEAP32[$bufpt>>2]|0;
    $1030 = (($1029) + ($1028)|0);
    HEAP8[$1030>>0] = 0;
    $1031 = HEAP32[$j>>2]|0;
    HEAP32[$length>>2] = $1031;
    break;
   }
   case 12:  {
    $1032 = HEAP32[$2>>2]|0;
    $arglist_current45 = HEAP32[$1032>>2]|0;
    $1033 = $arglist_current45;
    $1034 = ((0) + 4|0);
    $expanded137 = $1034;
    $expanded136 = (($expanded137) - 1)|0;
    $1035 = (($1033) + ($expanded136))|0;
    $1036 = ((0) + 4|0);
    $expanded141 = $1036;
    $expanded140 = (($expanded141) - 1)|0;
    $expanded139 = $expanded140 ^ -1;
    $1037 = $1035 & $expanded139;
    $1038 = $1037;
    $1039 = HEAP32[$1038>>2]|0;
    $arglist_next46 = ((($1038)) + 4|0);
    HEAP32[$1032>>2] = $arglist_next46;
    HEAP32[$pToken>>2] = $1039;
    $1040 = HEAP32[$pToken>>2]|0;
    $1041 = ($1040|0)!=(0|0);
    if ($1041) {
     $1042 = HEAP32[$pToken>>2]|0;
     $1043 = ((($1042)) + 4|0);
     $1044 = HEAP32[$1043>>2]|0;
     $1045 = ($1044|0)!=(0);
     if ($1045) {
      $1046 = HEAP32[$0>>2]|0;
      $1047 = HEAP32[$pToken>>2]|0;
      $1048 = HEAP32[$1047>>2]|0;
      $1049 = HEAP32[$pToken>>2]|0;
      $1050 = ((($1049)) + 4|0);
      $1051 = HEAP32[$1050>>2]|0;
      _sqlite3StrAccumAppend($1046,$1048,$1051);
     }
    }
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   case 13:  {
    $1052 = HEAP32[$2>>2]|0;
    $arglist_current48 = HEAP32[$1052>>2]|0;
    $1053 = $arglist_current48;
    $1054 = ((0) + 4|0);
    $expanded144 = $1054;
    $expanded143 = (($expanded144) - 1)|0;
    $1055 = (($1053) + ($expanded143))|0;
    $1056 = ((0) + 4|0);
    $expanded148 = $1056;
    $expanded147 = (($expanded148) - 1)|0;
    $expanded146 = $expanded147 ^ -1;
    $1057 = $1055 & $expanded146;
    $1058 = $1057;
    $1059 = HEAP32[$1058>>2]|0;
    $arglist_next49 = ((($1058)) + 4|0);
    HEAP32[$1052>>2] = $arglist_next49;
    HEAP32[$pSrc>>2] = $1059;
    $1060 = HEAP32[$2>>2]|0;
    $arglist_current51 = HEAP32[$1060>>2]|0;
    $1061 = $arglist_current51;
    $1062 = ((0) + 4|0);
    $expanded151 = $1062;
    $expanded150 = (($expanded151) - 1)|0;
    $1063 = (($1061) + ($expanded150))|0;
    $1064 = ((0) + 4|0);
    $expanded155 = $1064;
    $expanded154 = (($expanded155) - 1)|0;
    $expanded153 = $expanded154 ^ -1;
    $1065 = $1063 & $expanded153;
    $1066 = $1065;
    $1067 = HEAP32[$1066>>2]|0;
    $arglist_next52 = ((($1066)) + 4|0);
    HEAP32[$1060>>2] = $arglist_next52;
    HEAP32[$k3>>2] = $1067;
    $1068 = HEAP32[$k3>>2]|0;
    $1069 = HEAP32[$pSrc>>2]|0;
    $1070 = ((($1069)) + 8|0);
    $1071 = (($1070) + (($1068*72)|0)|0);
    HEAP32[$pItem>>2] = $1071;
    $1072 = HEAP32[$pItem>>2]|0;
    $1073 = ((($1072)) + 4|0);
    $1074 = HEAP32[$1073>>2]|0;
    $1075 = ($1074|0)!=(0|0);
    if ($1075) {
     $1076 = HEAP32[$0>>2]|0;
     $1077 = HEAP32[$pItem>>2]|0;
     $1078 = ((($1077)) + 4|0);
     $1079 = HEAP32[$1078>>2]|0;
     _sqlite3StrAccumAppendAll($1076,$1079);
     $1080 = HEAP32[$0>>2]|0;
     _sqlite3StrAccumAppend($1080,18212,1);
    }
    $1081 = HEAP32[$0>>2]|0;
    $1082 = HEAP32[$pItem>>2]|0;
    $1083 = ((($1082)) + 8|0);
    $1084 = HEAP32[$1083>>2]|0;
    _sqlite3StrAccumAppendAll($1081,$1084);
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   default: {
    label = 268;
    break L7;
   }
   }
  } while(0);
  if ((label|0) == 62) {
   label = 0;
   $196 = HEAP32[$infop>>2]|0;
   $197 = ((($196)) + 2|0);
   $198 = HEAP8[$197>>0]|0;
   $199 = $198&255;
   $200 = $199 & 1;
   $201 = ($200|0)!=(0);
   $202 = HEAP8[$bArgList>>0]|0;
   $203 = ($202<<24>>24)!=(0);
   do {
    if ($201) {
     do {
      if ($203) {
       $204 = HEAP32[$pArgList>>2]|0;
       $205 = (_getIntArg($204)|0);
       $206 = tempRet0;
       $207 = $v;
       $208 = $207;
       HEAP32[$208>>2] = $205;
       $209 = (($207) + 4)|0;
       $210 = $209;
       HEAP32[$210>>2] = $206;
      } else {
       $211 = HEAP8[$flag_longlong>>0]|0;
       $212 = ($211<<24>>24)!=(0);
       $213 = HEAP32[$2>>2]|0;
       if ($212) {
        $arglist_current18 = HEAP32[$213>>2]|0;
        $214 = $arglist_current18;
        $215 = ((0) + 8|0);
        $expanded74 = $215;
        $expanded73 = (($expanded74) - 1)|0;
        $216 = (($214) + ($expanded73))|0;
        $217 = ((0) + 8|0);
        $expanded78 = $217;
        $expanded77 = (($expanded78) - 1)|0;
        $expanded76 = $expanded77 ^ -1;
        $218 = $216 & $expanded76;
        $219 = $218;
        $220 = $219;
        $221 = $220;
        $222 = HEAP32[$221>>2]|0;
        $223 = (($220) + 4)|0;
        $224 = $223;
        $225 = HEAP32[$224>>2]|0;
        $arglist_next19 = ((($219)) + 8|0);
        HEAP32[$213>>2] = $arglist_next19;
        $226 = $v;
        $227 = $226;
        HEAP32[$227>>2] = $222;
        $228 = (($226) + 4)|0;
        $229 = $228;
        HEAP32[$229>>2] = $225;
        break;
       } else {
        $arglist_current21 = HEAP32[$213>>2]|0;
        $230 = $arglist_current21;
        $231 = ((0) + 4|0);
        $expanded81 = $231;
        $expanded80 = (($expanded81) - 1)|0;
        $232 = (($230) + ($expanded80))|0;
        $233 = ((0) + 4|0);
        $expanded85 = $233;
        $expanded84 = (($expanded85) - 1)|0;
        $expanded83 = $expanded84 ^ -1;
        $234 = $232 & $expanded83;
        $235 = $234;
        $236 = HEAP32[$235>>2]|0;
        $arglist_next22 = ((($235)) + 4|0);
        HEAP32[$213>>2] = $arglist_next22;
        $237 = ($236|0)<(0);
        $238 = $237 << 31 >> 31;
        $239 = $v;
        $240 = $239;
        HEAP32[$240>>2] = $236;
        $241 = (($239) + 4)|0;
        $242 = $241;
        HEAP32[$242>>2] = $238;
        break;
       }
      }
     } while(0);
     $243 = $v;
     $244 = $243;
     $245 = HEAP32[$244>>2]|0;
     $246 = (($243) + 4)|0;
     $247 = $246;
     $248 = HEAP32[$247>>2]|0;
     $249 = ($248|0)<(0);
     $250 = $v;
     $251 = $250;
     $252 = HEAP32[$251>>2]|0;
     $253 = (($250) + 4)|0;
     $254 = $253;
     $255 = HEAP32[$254>>2]|0;
     if ($249) {
      $256 = ($252|0)==(0);
      $257 = ($255|0)==(-2147483648);
      $258 = $256 & $257;
      if ($258) {
       $259 = $longvalue;
       $260 = $259;
       HEAP32[$260>>2] = 0;
       $261 = (($259) + 4)|0;
       $262 = $261;
       HEAP32[$262>>2] = -2147483648;
      } else {
       $263 = $v;
       $264 = $263;
       $265 = HEAP32[$264>>2]|0;
       $266 = (($263) + 4)|0;
       $267 = $266;
       $268 = HEAP32[$267>>2]|0;
       $269 = (_i64Subtract(0,0,($265|0),($268|0))|0);
       $270 = tempRet0;
       $271 = $longvalue;
       $272 = $271;
       HEAP32[$272>>2] = $269;
       $273 = (($271) + 4)|0;
       $274 = $273;
       HEAP32[$274>>2] = $270;
      }
      HEAP8[$prefix>>0] = 45;
      break;
     }
     $275 = $longvalue;
     $276 = $275;
     HEAP32[$276>>2] = $252;
     $277 = (($275) + 4)|0;
     $278 = $277;
     HEAP32[$278>>2] = $255;
     $279 = HEAP8[$flag_plussign>>0]|0;
     $280 = ($279<<24>>24)!=(0);
     if ($280) {
      HEAP8[$prefix>>0] = 43;
      break;
     }
     $281 = HEAP8[$flag_blanksign>>0]|0;
     $282 = ($281<<24>>24)!=(0);
     if ($282) {
      HEAP8[$prefix>>0] = 32;
      break;
     } else {
      HEAP8[$prefix>>0] = 0;
      break;
     }
    } else {
     do {
      if ($203) {
       $283 = HEAP32[$pArgList>>2]|0;
       $284 = (_getIntArg($283)|0);
       $285 = tempRet0;
       $286 = $longvalue;
       $287 = $286;
       HEAP32[$287>>2] = $284;
       $288 = (($286) + 4)|0;
       $289 = $288;
       HEAP32[$289>>2] = $285;
      } else {
       $290 = HEAP8[$flag_longlong>>0]|0;
       $291 = ($290<<24>>24)!=(0);
       $292 = HEAP32[$2>>2]|0;
       if ($291) {
        $arglist_current24 = HEAP32[$292>>2]|0;
        $293 = $arglist_current24;
        $294 = ((0) + 8|0);
        $expanded88 = $294;
        $expanded87 = (($expanded88) - 1)|0;
        $295 = (($293) + ($expanded87))|0;
        $296 = ((0) + 8|0);
        $expanded92 = $296;
        $expanded91 = (($expanded92) - 1)|0;
        $expanded90 = $expanded91 ^ -1;
        $297 = $295 & $expanded90;
        $298 = $297;
        $299 = $298;
        $300 = $299;
        $301 = HEAP32[$300>>2]|0;
        $302 = (($299) + 4)|0;
        $303 = $302;
        $304 = HEAP32[$303>>2]|0;
        $arglist_next25 = ((($298)) + 8|0);
        HEAP32[$292>>2] = $arglist_next25;
        $305 = $longvalue;
        $306 = $305;
        HEAP32[$306>>2] = $301;
        $307 = (($305) + 4)|0;
        $308 = $307;
        HEAP32[$308>>2] = $304;
        break;
       } else {
        $arglist_current27 = HEAP32[$292>>2]|0;
        $309 = $arglist_current27;
        $310 = ((0) + 4|0);
        $expanded95 = $310;
        $expanded94 = (($expanded95) - 1)|0;
        $311 = (($309) + ($expanded94))|0;
        $312 = ((0) + 4|0);
        $expanded99 = $312;
        $expanded98 = (($expanded99) - 1)|0;
        $expanded97 = $expanded98 ^ -1;
        $313 = $311 & $expanded97;
        $314 = $313;
        $315 = HEAP32[$314>>2]|0;
        $arglist_next28 = ((($314)) + 4|0);
        HEAP32[$292>>2] = $arglist_next28;
        $316 = $longvalue;
        $317 = $316;
        HEAP32[$317>>2] = $315;
        $318 = (($316) + 4)|0;
        $319 = $318;
        HEAP32[$319>>2] = 0;
        break;
       }
      }
     } while(0);
     HEAP8[$prefix>>0] = 0;
    }
   } while(0);
   $320 = $longvalue;
   $321 = $320;
   $322 = HEAP32[$321>>2]|0;
   $323 = (($320) + 4)|0;
   $324 = $323;
   $325 = HEAP32[$324>>2]|0;
   $326 = ($322|0)==(0);
   $327 = ($325|0)==(0);
   $328 = $326 & $327;
   if ($328) {
    HEAP8[$flag_alternateform>>0] = 0;
   }
   $329 = HEAP8[$flag_zeropad>>0]|0;
   $330 = $329&255;
   $331 = ($330|0)!=(0);
   if ($331) {
    $332 = HEAP32[$precision>>2]|0;
    $333 = HEAP32[$width>>2]|0;
    $334 = HEAP8[$prefix>>0]|0;
    $335 = $334 << 24 >> 24;
    $336 = ($335|0)!=(0);
    $337 = $336&1;
    $338 = (($333) - ($337))|0;
    $339 = ($332|0)<($338|0);
    if ($339) {
     $340 = HEAP32[$width>>2]|0;
     $341 = HEAP8[$prefix>>0]|0;
     $342 = $341 << 24 >> 24;
     $343 = ($342|0)!=(0);
     $344 = $343&1;
     $345 = (($340) - ($344))|0;
     HEAP32[$precision>>2] = $345;
    }
   }
   $346 = HEAP32[$precision>>2]|0;
   $347 = ($346|0)<(60);
   if ($347) {
    HEAP32[$nOut>>2] = 70;
    HEAP32[$zOut>>2] = $buf;
   } else {
    $348 = HEAP32[$precision>>2]|0;
    $349 = (($348) + 10)|0;
    HEAP32[$nOut>>2] = $349;
    $350 = HEAP32[$nOut>>2]|0;
    $351 = ($350|0)<(0);
    $352 = $351 << 31 >> 31;
    $353 = (_sqlite3Malloc($350,$352)|0);
    HEAP32[$zExtra>>2] = $353;
    HEAP32[$zOut>>2] = $353;
    $354 = HEAP32[$zOut>>2]|0;
    $355 = ($354|0)==(0|0);
    if ($355) {
     label = 92;
     break;
    }
   }
   $357 = HEAP32[$nOut>>2]|0;
   $358 = (($357) - 1)|0;
   $359 = HEAP32[$zOut>>2]|0;
   $360 = (($359) + ($358)|0);
   HEAP32[$bufpt>>2] = $360;
   $361 = HEAP8[$xtype>>0]|0;
   $362 = $361&255;
   $363 = ($362|0)==(16);
   if ($363) {
    $364 = $longvalue;
    $365 = $364;
    $366 = HEAP32[$365>>2]|0;
    $367 = (($364) + 4)|0;
    $368 = $367;
    $369 = HEAP32[$368>>2]|0;
    $370 = (___uremdi3(($366|0),($369|0),10,0)|0);
    $371 = tempRet0;
    HEAP32[$x>>2] = $370;
    $372 = HEAP32[$x>>2]|0;
    $373 = ($372|0)>=(4);
    if ($373) {
     label = 96;
    } else {
     $374 = $longvalue;
     $375 = $374;
     $376 = HEAP32[$375>>2]|0;
     $377 = (($374) + 4)|0;
     $378 = $377;
     $379 = HEAP32[$378>>2]|0;
     $380 = (___udivdi3(($376|0),($379|0),10,0)|0);
     $381 = tempRet0;
     $382 = (___uremdi3(($380|0),($381|0),10,0)|0);
     $383 = tempRet0;
     $384 = ($382|0)==(1);
     $385 = ($383|0)==(0);
     $386 = $384 & $385;
     if ($386) {
      label = 96;
     }
    }
    if ((label|0) == 96) {
     label = 0;
     HEAP32[$x>>2] = 0;
    }
    $387 = HEAP32[$x>>2]|0;
    $388 = $387<<1;
    $389 = (($388) + 1)|0;
    $390 = (18143 + ($389)|0);
    $391 = HEAP8[$390>>0]|0;
    $392 = HEAP32[$bufpt>>2]|0;
    $393 = ((($392)) + -1|0);
    HEAP32[$bufpt>>2] = $393;
    HEAP8[$393>>0] = $391;
    $394 = HEAP32[$x>>2]|0;
    $395 = $394<<1;
    $396 = (18143 + ($395)|0);
    $397 = HEAP8[$396>>0]|0;
    $398 = HEAP32[$bufpt>>2]|0;
    $399 = ((($398)) + -1|0);
    HEAP32[$bufpt>>2] = $399;
    HEAP8[$399>>0] = $397;
   }
   $400 = HEAP32[$infop>>2]|0;
   $401 = ((($400)) + 4|0);
   $402 = HEAP8[$401>>0]|0;
   $403 = $402&255;
   $404 = (18152 + ($403)|0);
   HEAP32[$cset>>2] = $404;
   $405 = HEAP32[$infop>>2]|0;
   $406 = ((($405)) + 1|0);
   $407 = HEAP8[$406>>0]|0;
   HEAP8[$base>>0] = $407;
   while(1) {
    $408 = $longvalue;
    $409 = $408;
    $410 = HEAP32[$409>>2]|0;
    $411 = (($408) + 4)|0;
    $412 = $411;
    $413 = HEAP32[$412>>2]|0;
    $414 = HEAP8[$base>>0]|0;
    $415 = $414&255;
    $416 = (___uremdi3(($410|0),($413|0),($415|0),0)|0);
    $417 = tempRet0;
    $418 = HEAP32[$cset>>2]|0;
    $419 = (($418) + ($416)|0);
    $420 = HEAP8[$419>>0]|0;
    $421 = HEAP32[$bufpt>>2]|0;
    $422 = ((($421)) + -1|0);
    HEAP32[$bufpt>>2] = $422;
    HEAP8[$422>>0] = $420;
    $423 = $longvalue;
    $424 = $423;
    $425 = HEAP32[$424>>2]|0;
    $426 = (($423) + 4)|0;
    $427 = $426;
    $428 = HEAP32[$427>>2]|0;
    $429 = HEAP8[$base>>0]|0;
    $430 = $429&255;
    $431 = (___udivdi3(($425|0),($428|0),($430|0),0)|0);
    $432 = tempRet0;
    $433 = $longvalue;
    $434 = $433;
    HEAP32[$434>>2] = $431;
    $435 = (($433) + 4)|0;
    $436 = $435;
    HEAP32[$436>>2] = $432;
    $437 = $longvalue;
    $438 = $437;
    $439 = HEAP32[$438>>2]|0;
    $440 = (($437) + 4)|0;
    $441 = $440;
    $442 = HEAP32[$441>>2]|0;
    $443 = ($442>>>0)>(0);
    $444 = ($439>>>0)>(0);
    $445 = ($442|0)==(0);
    $446 = $445 & $444;
    $447 = $443 | $446;
    if (!($447)) {
     break;
    }
   }
   $448 = HEAP32[$nOut>>2]|0;
   $449 = (($448) - 1)|0;
   $450 = HEAP32[$zOut>>2]|0;
   $451 = (($450) + ($449)|0);
   $452 = HEAP32[$bufpt>>2]|0;
   $453 = $451;
   $454 = $452;
   $455 = (($453) - ($454))|0;
   HEAP32[$length>>2] = $455;
   $456 = HEAP32[$precision>>2]|0;
   $457 = HEAP32[$length>>2]|0;
   $458 = (($456) - ($457))|0;
   HEAP32[$idx>>2] = $458;
   while(1) {
    $459 = HEAP32[$idx>>2]|0;
    $460 = ($459|0)>(0);
    if (!($460)) {
     break;
    }
    $461 = HEAP32[$bufpt>>2]|0;
    $462 = ((($461)) + -1|0);
    HEAP32[$bufpt>>2] = $462;
    HEAP8[$462>>0] = 48;
    $463 = HEAP32[$idx>>2]|0;
    $464 = (($463) + -1)|0;
    HEAP32[$idx>>2] = $464;
   }
   $465 = HEAP8[$prefix>>0]|0;
   $466 = ($465<<24>>24)!=(0);
   if ($466) {
    $467 = HEAP8[$prefix>>0]|0;
    $468 = HEAP32[$bufpt>>2]|0;
    $469 = ((($468)) + -1|0);
    HEAP32[$bufpt>>2] = $469;
    HEAP8[$469>>0] = $467;
   }
   $470 = HEAP8[$flag_alternateform>>0]|0;
   $471 = $470&255;
   $472 = ($471|0)!=(0);
   L345: do {
    if ($472) {
     $473 = HEAP32[$infop>>2]|0;
     $474 = ((($473)) + 5|0);
     $475 = HEAP8[$474>>0]|0;
     $476 = $475&255;
     $477 = ($476|0)!=(0);
     if ($477) {
      $478 = HEAP32[$infop>>2]|0;
      $479 = ((($478)) + 5|0);
      $480 = HEAP8[$479>>0]|0;
      $481 = $480&255;
      $482 = (18185 + ($481)|0);
      HEAP32[$pre>>2] = $482;
      while(1) {
       $483 = HEAP32[$pre>>2]|0;
       $484 = HEAP8[$483>>0]|0;
       HEAP8[$x1>>0] = $484;
       $485 = $484 << 24 >> 24;
       $486 = ($485|0)!=(0);
       if (!($486)) {
        break L345;
       }
       $487 = HEAP8[$x1>>0]|0;
       $488 = HEAP32[$bufpt>>2]|0;
       $489 = ((($488)) + -1|0);
       HEAP32[$bufpt>>2] = $489;
       HEAP8[$489>>0] = $487;
       $490 = HEAP32[$pre>>2]|0;
       $491 = ((($490)) + 1|0);
       HEAP32[$pre>>2] = $491;
      }
     }
    }
   } while(0);
   $492 = HEAP32[$nOut>>2]|0;
   $493 = (($492) - 1)|0;
   $494 = HEAP32[$zOut>>2]|0;
   $495 = (($494) + ($493)|0);
   $496 = HEAP32[$bufpt>>2]|0;
   $497 = $495;
   $498 = $496;
   $499 = (($497) - ($498))|0;
   HEAP32[$length>>2] = $499;
  }
  $1085 = HEAP32[$length>>2]|0;
  $1086 = HEAP32[$width>>2]|0;
  $1087 = (($1086) - ($1085))|0;
  HEAP32[$width>>2] = $1087;
  $1088 = HEAP32[$width>>2]|0;
  $1089 = ($1088|0)<=(0);
  $1090 = HEAP8[$flag_leftjustify>>0]|0;
  $1091 = ($1090<<24>>24)!=(0);
  $or$cond10 = $1089 | $1091;
  if (!($or$cond10)) {
   $1092 = HEAP32[$0>>2]|0;
   $1093 = HEAP32[$width>>2]|0;
   _sqlite3AppendChar($1092,$1093,32);
  }
  $1094 = HEAP32[$0>>2]|0;
  $1095 = HEAP32[$bufpt>>2]|0;
  $1096 = HEAP32[$length>>2]|0;
  _sqlite3StrAccumAppend($1094,$1095,$1096);
  $1097 = HEAP32[$width>>2]|0;
  $1098 = ($1097|0)>(0);
  if ($1098) {
   $1099 = HEAP8[$flag_leftjustify>>0]|0;
   $1100 = $1099&255;
   $1101 = ($1100|0)!=(0);
   if ($1101) {
    $1102 = HEAP32[$0>>2]|0;
    $1103 = HEAP32[$width>>2]|0;
    _sqlite3AppendChar($1102,$1103,32);
   }
  }
  $1104 = HEAP32[$zExtra>>2]|0;
  $1105 = ($1104|0)!=(0|0);
  if ($1105) {
   $1106 = HEAP32[$0>>2]|0;
   $1107 = HEAP32[$1106>>2]|0;
   $1108 = HEAP32[$zExtra>>2]|0;
   _sqlite3DbFree($1107,$1108);
   HEAP32[$zExtra>>2] = 0;
  }
  $1109 = HEAP32[$1>>2]|0;
  $1110 = ((($1109)) + 1|0);
  HEAP32[$1>>2] = $1110;
 }
 if ((label|0) == 13) {
  $62 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($62,18003,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 92) {
  $356 = HEAP32[$0>>2]|0;
  _setStrAccumError($356,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 161) {
  $683 = HEAP32[$0>>2]|0;
  _setStrAccumError($683,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 241) {
  $989 = HEAP32[$0>>2]|0;
  _setStrAccumError($989,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 268) {
  STACKTOP = sp;return;
 }
}
function _sqlite3StrAccumFinish($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$0>>2]|0;
   $6 = ((($5)) + 12|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = HEAP32[$0>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (($10) + ($7)|0);
   HEAP8[$11>>0] = 0;
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 20|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14>>>0)>(0);
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = ((($16)) + 25|0);
    $18 = HEAP8[$17>>0]|0;
    $19 = $18&255;
    $20 = $19 & 4;
    $21 = ($20|0)!=(0);
    if (!($21)) {
     $22 = HEAP32[$0>>2]|0;
     $23 = HEAP32[$22>>2]|0;
     $24 = HEAP32[$0>>2]|0;
     $25 = ((($24)) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = (($26) + 1)|0;
     $28 = (_sqlite3DbMallocRaw($23,$27,0)|0);
     $29 = HEAP32[$0>>2]|0;
     $30 = ((($29)) + 8|0);
     HEAP32[$30>>2] = $28;
     $31 = HEAP32[$0>>2]|0;
     $32 = ((($31)) + 8|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)!=(0|0);
     $35 = HEAP32[$0>>2]|0;
     if ($34) {
      $36 = ((($35)) + 8|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$0>>2]|0;
      $39 = ((($38)) + 4|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = HEAP32[$0>>2]|0;
      $42 = ((($41)) + 12|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = (($43) + 1)|0;
      _memcpy(($37|0),($40|0),($44|0))|0;
      $45 = HEAP32[$0>>2]|0;
      $46 = ((($45)) + 25|0);
      $47 = HEAP8[$46>>0]|0;
      $48 = $47&255;
      $49 = $48 | 4;
      $50 = $49&255;
      HEAP8[$46>>0] = $50;
      break;
     } else {
      _setStrAccumError($35,1);
      break;
     }
    }
   }
  }
 } while(0);
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 8|0);
 $53 = HEAP32[$52>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _sqlite3_mprintf($zFormat,$varargs) {
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$1>>2] = $zFormat;
 $2 = (_sqlite3_initialize()|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 } else {
  HEAP32[$ap>>2] = $varargs;
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3_vmprintf($4,$ap)|0);
  HEAP32[$z>>2] = $5;
  $6 = HEAP32[$z>>2]|0;
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 return (0)|0;
}
function _sqlite3_vsnprintf($n,$zBuf,$zFormat,$ap) {
 $n = $n|0;
 $zBuf = $zBuf|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $acc = sp;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $zFormat;
 HEAP32[$4>>2] = $ap;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$2>>2]|0;
 if ($6) {
  HEAP32[$0>>2] = $7;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumInit($acc,0,$7,$8,0);
  $9 = HEAP32[$3>>2]|0;
  $10 = HEAP32[$4>>2]|0;
  _sqlite3VXPrintf($acc,$9,$10);
  $11 = (_sqlite3StrAccumFinish($acc)|0);
  HEAP32[$0>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _sqlite3_snprintf($n,$zBuf,$zFormat,$varargs) {
 $n = $n|0;
 $zBuf = $zBuf|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $z = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $zBuf;
 HEAP32[$2>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3_vsnprintf($3,$4,$5,$ap)|0);
 HEAP32[$z>>2] = $6;
 $7 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_log($iErrCode,$zFormat,$varargs) {
 $iErrCode = $iErrCode|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $iErrCode;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[(264)>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _renderLogMsg($4,$5,$ap);
 STACKTOP = sp;return;
}
function _renderLogMsg($iErrCode,$zFormat,$ap) {
 $iErrCode = $iErrCode|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $acc = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $acc = sp;
 $zMsg = sp + 40|0;
 HEAP32[$0>>2] = $iErrCode;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 _sqlite3StrAccumInit($acc,0,$zMsg,210,0);
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($acc,$3,$4);
 $5 = HEAP32[(264)>>2]|0;
 $6 = HEAP32[(268)>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3StrAccumFinish($acc)|0);
 FUNCTION_TABLE_viii[$5 & 255]($6,$7,$8);
 STACKTOP = sp;return;
}
function _sqlite3_randomness($N,$pBuf) {
 $N = $N|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $i = 0, $k = 0, $or$cond = 0, $t = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $t = sp + 272|0;
 $zBuf = sp + 4|0;
 $i = sp;
 $k = sp + 16|0;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $pBuf;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$zBuf>>2] = $2;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(0|0);
 $or$cond = $6 | $8;
 if ($or$cond) {
  HEAP8[16715>>0] = 0;
  STACKTOP = sp;return;
 }
 $9 = HEAP8[16715>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if (!($10)) {
  HEAP8[(16717)>>0] = 0;
  HEAP8[(16716)>>0] = 0;
  $11 = (_sqlite3_vfs_find(0)|0);
  (_sqlite3OsRandomness($11,256,$k)|0);
  HEAP32[$i>>2] = 0;
  while(1) {
   $12 = HEAP32[$i>>2]|0;
   $13 = ($12|0)<(256);
   if (!($13)) {
    break;
   }
   $14 = HEAP32[$i>>2]|0;
   $15 = $14&255;
   $16 = HEAP32[$i>>2]|0;
   $17 = ((16718) + ($16)|0);
   HEAP8[$17>>0] = $15;
   $18 = HEAP32[$i>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$i>>2] = $19;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $20 = HEAP32[$i>>2]|0;
   $21 = ($20|0)<(256);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$i>>2]|0;
   $23 = ((16718) + ($22)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = HEAP32[$i>>2]|0;
   $27 = (($k) + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28 << 24 >> 24;
   $30 = (($25) + ($29))|0;
   $31 = HEAP8[(16717)>>0]|0;
   $32 = $31&255;
   $33 = (($32) + ($30))|0;
   $34 = $33&255;
   HEAP8[(16717)>>0] = $34;
   $35 = HEAP8[(16717)>>0]|0;
   $36 = $35&255;
   $37 = ((16718) + ($36)|0);
   $38 = HEAP8[$37>>0]|0;
   HEAP8[$t>>0] = $38;
   $39 = HEAP32[$i>>2]|0;
   $40 = ((16718) + ($39)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = HEAP8[(16717)>>0]|0;
   $43 = $42&255;
   $44 = ((16718) + ($43)|0);
   HEAP8[$44>>0] = $41;
   $45 = HEAP8[$t>>0]|0;
   $46 = HEAP32[$i>>2]|0;
   $47 = ((16718) + ($46)|0);
   HEAP8[$47>>0] = $45;
   $48 = HEAP32[$i>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$i>>2] = $49;
  }
  HEAP8[16715>>0] = 1;
 }
 while(1) {
  $50 = HEAP8[(16716)>>0]|0;
  $51 = (($50) + 1)<<24>>24;
  HEAP8[(16716)>>0] = $51;
  $52 = HEAP8[(16716)>>0]|0;
  $53 = $52&255;
  $54 = ((16718) + ($53)|0);
  $55 = HEAP8[$54>>0]|0;
  HEAP8[$t>>0] = $55;
  $56 = HEAP8[$t>>0]|0;
  $57 = $56&255;
  $58 = HEAP8[(16717)>>0]|0;
  $59 = $58&255;
  $60 = (($59) + ($57))|0;
  $61 = $60&255;
  HEAP8[(16717)>>0] = $61;
  $62 = HEAP8[(16717)>>0]|0;
  $63 = $62&255;
  $64 = ((16718) + ($63)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = HEAP8[(16716)>>0]|0;
  $67 = $66&255;
  $68 = ((16718) + ($67)|0);
  HEAP8[$68>>0] = $65;
  $69 = HEAP8[$t>>0]|0;
  $70 = HEAP8[(16717)>>0]|0;
  $71 = $70&255;
  $72 = ((16718) + ($71)|0);
  HEAP8[$72>>0] = $69;
  $73 = HEAP8[(16716)>>0]|0;
  $74 = $73&255;
  $75 = ((16718) + ($74)|0);
  $76 = HEAP8[$75>>0]|0;
  $77 = $76&255;
  $78 = HEAP8[$t>>0]|0;
  $79 = $78&255;
  $80 = (($79) + ($77))|0;
  $81 = $80&255;
  HEAP8[$t>>0] = $81;
  $82 = HEAP8[$t>>0]|0;
  $83 = $82&255;
  $84 = ((16718) + ($83)|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = HEAP32[$zBuf>>2]|0;
  $87 = ((($86)) + 1|0);
  HEAP32[$zBuf>>2] = $87;
  HEAP8[$86>>0] = $85;
  $88 = HEAP32[$0>>2]|0;
  $89 = (($88) + -1)|0;
  HEAP32[$0>>2] = $89;
  $90 = ($89|0)!=(0);
  if (!($90)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3OsRandomness($pVfs,$nByte,$zBufOut) {
 $pVfs = $pVfs|0;
 $nByte = $nByte|0;
 $zBufOut = $zBufOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$2>>2] = $zBufOut;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$5 & 255]($6,$7,$8)|0);
 STACKTOP = sp;return ($9|0);
}
function _sqlite3_stricmp($zLeft,$zRight) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $a = sp + 4|0;
 $b = sp;
 HEAP32[$1>>2] = $zLeft;
 HEAP32[$2>>2] = $zRight;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 if ($4) {
  $6 = ($5|0)!=(0|0);
  $7 = $6 ? -1 : 0;
  HEAP32[$0>>2] = $7;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $8 = ($5|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$a>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$b>>2] = $10;
 while(1) {
  $11 = HEAP32[$a>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$a>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   $18 = (16974 + ($17)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = HEAP32[$b>>2]|0;
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = (16974 + ($23)|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($20|0)==($26|0);
   $45 = $27;
  } else {
   $45 = 0;
  }
  $28 = HEAP32[$a>>2]|0;
  if (!($45)) {
   break;
  }
  $29 = ((($28)) + 1|0);
  HEAP32[$a>>2] = $29;
  $30 = HEAP32[$b>>2]|0;
  $31 = ((($30)) + 1|0);
  HEAP32[$b>>2] = $31;
 }
 $32 = HEAP8[$28>>0]|0;
 $33 = $32&255;
 $34 = (16974 + ($33)|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35&255;
 $37 = HEAP32[$b>>2]|0;
 $38 = HEAP8[$37>>0]|0;
 $39 = $38&255;
 $40 = (16974 + ($39)|0);
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = (($36) - ($42))|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3_os_init() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0>>>0)<(4);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$i>>2]|0;
  $3 = (468 + (($2*88)|0)|0);
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)==(0);
  $6 = $5&1;
  (_sqlite3_vfs_register($3,$6)|0);
  $7 = HEAP32[$i>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$i>>2] = $8;
 }
 STACKTOP = sp;return 0;
}
function _unixOpen($pVfs,$zPath,$pFile,$flags,$pOutFlags) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $pFile = $pFile|0;
 $flags = $flags|0;
 $pOutFlags = $pOutFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctrlFlags = 0, $eType = 0, $fd = 0, $gid = 0, $isCreate = 0, $isDelete = 0, $isExclusive = 0;
 var $isReadWrite = 0, $isReadonly = 0, $noLock = 0, $openFlags = 0, $openMode = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pUnused = 0, $rc = 0, $syncDir = 0, $uid = 0, $zName = 0, $zTmpname = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $4 = sp + 76|0;
 $5 = sp + 72|0;
 $p = sp + 68|0;
 $fd = sp + 64|0;
 $openFlags = sp + 60|0;
 $eType = sp + 56|0;
 $noLock = sp + 52|0;
 $rc = sp + 48|0;
 $ctrlFlags = sp + 44|0;
 $isExclusive = sp + 40|0;
 $isDelete = sp + 36|0;
 $isCreate = sp + 32|0;
 $isReadonly = sp + 28|0;
 $isReadWrite = sp + 24|0;
 $syncDir = sp + 20|0;
 $zTmpname = sp + 96|0;
 $zName = sp + 16|0;
 $pUnused = sp + 12|0;
 $openMode = sp + 8|0;
 $uid = sp + 4|0;
 $gid = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $pFile;
 HEAP32[$4>>2] = $flags;
 HEAP32[$5>>2] = $pOutFlags;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $6;
 HEAP32[$fd>>2] = -1;
 HEAP32[$openFlags>>2] = 0;
 $7 = HEAP32[$4>>2]|0;
 $8 = $7 & -256;
 HEAP32[$eType>>2] = $8;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ctrlFlags>>2] = 0;
 $9 = HEAP32[$4>>2]|0;
 $10 = $9 & 16;
 HEAP32[$isExclusive>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = $11 & 8;
 HEAP32[$isDelete>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 $14 = $13 & 4;
 HEAP32[$isCreate>>2] = $14;
 $15 = HEAP32[$4>>2]|0;
 $16 = $15 & 1;
 HEAP32[$isReadonly>>2] = $16;
 $17 = HEAP32[$4>>2]|0;
 $18 = $17 & 2;
 HEAP32[$isReadWrite>>2] = $18;
 $19 = HEAP32[$isCreate>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$eType>>2]|0;
  $22 = ($21|0)==(16384);
  $23 = HEAP32[$eType>>2]|0;
  $24 = ($23|0)==(2048);
  $or$cond = $22 | $24;
  if ($or$cond) {
   $28 = 1;
  } else {
   $25 = HEAP32[$eType>>2]|0;
   $26 = ($25|0)==(524288);
   $28 = $26;
  }
 } else {
  $28 = 0;
 }
 $27 = $28&1;
 HEAP32[$syncDir>>2] = $27;
 $29 = HEAP32[$2>>2]|0;
 HEAP32[$zName>>2] = $29;
 $30 = HEAP32[1428>>2]|0;
 $31 = (_getpid()|0);
 $32 = ($30|0)!=($31|0);
 if ($32) {
  $33 = (_getpid()|0);
  HEAP32[1428>>2] = $33;
  _sqlite3_randomness(0,0);
 }
 $34 = HEAP32[$p>>2]|0;
 dest=$34; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $35 = HEAP32[$eType>>2]|0;
 $36 = ($35|0)==(256);
 $37 = HEAP32[$zName>>2]|0;
 do {
  if ($36) {
   $38 = HEAP32[$4>>2]|0;
   $39 = (_findReusableFd($37,$38)|0);
   HEAP32[$pUnused>>2] = $39;
   $40 = HEAP32[$pUnused>>2]|0;
   $41 = ($40|0)!=(0|0);
   if ($41) {
    $42 = HEAP32[$pUnused>>2]|0;
    $43 = HEAP32[$42>>2]|0;
    HEAP32[$fd>>2] = $43;
   } else {
    $44 = (_sqlite3_malloc64(12,0)|0);
    HEAP32[$pUnused>>2] = $44;
    $45 = HEAP32[$pUnused>>2]|0;
    $46 = ($45|0)!=(0|0);
    if (!($46)) {
     HEAP32[$0>>2] = 7;
     $172 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($172|0);
    }
   }
   $47 = HEAP32[$pUnused>>2]|0;
   $48 = HEAP32[$p>>2]|0;
   $49 = ((($48)) + 28|0);
   HEAP32[$49>>2] = $47;
  } else {
   $50 = ($37|0)!=(0|0);
   if (!($50)) {
    $51 = HEAP32[$1>>2]|0;
    $52 = ((($51)) + 8|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (_unixGetTempname($53,$zTmpname)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)!=(0);
    if (!($56)) {
     HEAP32[$zName>>2] = $zTmpname;
     break;
    }
    $57 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $57;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
  }
 } while(0);
 $58 = HEAP32[$isReadonly>>2]|0;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$openFlags>>2]|0;
  HEAP32[$openFlags>>2] = $60;
 }
 $61 = HEAP32[$isReadWrite>>2]|0;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = HEAP32[$openFlags>>2]|0;
  $64 = $63 | 2;
  HEAP32[$openFlags>>2] = $64;
 }
 $65 = HEAP32[$isCreate>>2]|0;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$openFlags>>2]|0;
  $68 = $67 | 64;
  HEAP32[$openFlags>>2] = $68;
 }
 $69 = HEAP32[$isExclusive>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$openFlags>>2]|0;
  $72 = $71 | 131200;
  HEAP32[$openFlags>>2] = $72;
 }
 $73 = HEAP32[$openFlags>>2]|0;
 HEAP32[$openFlags>>2] = $73;
 $74 = HEAP32[$fd>>2]|0;
 $75 = ($74|0)<(0);
 do {
  if ($75) {
   $76 = HEAP32[$zName>>2]|0;
   $77 = HEAP32[$4>>2]|0;
   $78 = (_findCreateFileMode($76,$77,$openMode,$uid,$gid)|0);
   HEAP32[$rc>>2] = $78;
   $79 = HEAP32[$rc>>2]|0;
   $80 = ($79|0)!=(0);
   if ($80) {
    $81 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $81;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
   $82 = HEAP32[$zName>>2]|0;
   $83 = HEAP32[$openFlags>>2]|0;
   $84 = HEAP32[$openMode>>2]|0;
   $85 = (_robust_open($82,$83,$84)|0);
   HEAP32[$fd>>2] = $85;
   $86 = HEAP32[$fd>>2]|0;
   $87 = ($86|0)<(0);
   if ($87) {
    $88 = (___errno_location()|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89|0)!=(21);
    $91 = HEAP32[$isReadWrite>>2]|0;
    $92 = ($91|0)!=(0);
    $or$cond3 = $90 & $92;
    if ($or$cond3) {
     $93 = HEAP32[$4>>2]|0;
     $94 = $93 & -7;
     HEAP32[$4>>2] = $94;
     $95 = HEAP32[$openFlags>>2]|0;
     $96 = $95 & -67;
     HEAP32[$openFlags>>2] = $96;
     $97 = HEAP32[$4>>2]|0;
     $98 = $97 | 1;
     HEAP32[$4>>2] = $98;
     $99 = HEAP32[$openFlags>>2]|0;
     HEAP32[$openFlags>>2] = $99;
     HEAP32[$isReadonly>>2] = 1;
     $100 = HEAP32[$zName>>2]|0;
     $101 = HEAP32[$openFlags>>2]|0;
     $102 = HEAP32[$openMode>>2]|0;
     $103 = (_robust_open($100,$101,$102)|0);
     HEAP32[$fd>>2] = $103;
    }
   }
   $104 = HEAP32[$fd>>2]|0;
   $105 = ($104|0)<(0);
   if ($105) {
    $106 = (_sqlite3CantopenError(33050)|0);
    $107 = HEAP32[$zName>>2]|0;
    $108 = (_unixLogErrorAtLine($106,18322,$107,33050)|0);
    HEAP32[$rc>>2] = $108;
    break;
   }
   $109 = HEAP32[$4>>2]|0;
   $110 = $109 & 526336;
   $111 = ($110|0)!=(0);
   if ($111) {
    $112 = HEAP32[$fd>>2]|0;
    $113 = HEAP32[$uid>>2]|0;
    $114 = HEAP32[$gid>>2]|0;
    (_robustFchown($112,$113,$114)|0);
    label = 34;
   } else {
    label = 34;
   }
  } else {
   label = 34;
  }
 } while(0);
 if ((label|0) == 34) {
  $115 = HEAP32[$5>>2]|0;
  $116 = ($115|0)!=(0|0);
  if ($116) {
   $117 = HEAP32[$4>>2]|0;
   $118 = HEAP32[$5>>2]|0;
   HEAP32[$118>>2] = $117;
  }
  $119 = HEAP32[$p>>2]|0;
  $120 = ((($119)) + 28|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($121|0)!=(0|0);
  if ($122) {
   $123 = HEAP32[$fd>>2]|0;
   $124 = HEAP32[$p>>2]|0;
   $125 = ((($124)) + 28|0);
   $126 = HEAP32[$125>>2]|0;
   HEAP32[$126>>2] = $123;
   $127 = HEAP32[$4>>2]|0;
   $128 = HEAP32[$p>>2]|0;
   $129 = ((($128)) + 28|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = ((($130)) + 4|0);
   HEAP32[$131>>2] = $127;
  }
  $132 = HEAP32[$isDelete>>2]|0;
  $133 = ($132|0)!=(0);
  if ($133) {
   $134 = HEAP32[(1260)>>2]|0;
   $135 = HEAP32[$zName>>2]|0;
   (FUNCTION_TABLE_ii[$134 & 255]($135)|0);
  }
  $136 = HEAP32[$eType>>2]|0;
  $137 = ($136|0)!=(256);
  $138 = $137&1;
  HEAP32[$noLock>>2] = $138;
  $139 = HEAP32[$isDelete>>2]|0;
  $140 = ($139|0)!=(0);
  if ($140) {
   $141 = HEAP32[$ctrlFlags>>2]|0;
   $142 = $141 | 32;
   HEAP32[$ctrlFlags>>2] = $142;
  }
  $143 = HEAP32[$isReadonly>>2]|0;
  $144 = ($143|0)!=(0);
  if ($144) {
   $145 = HEAP32[$ctrlFlags>>2]|0;
   $146 = $145 | 2;
   HEAP32[$ctrlFlags>>2] = $146;
  }
  $147 = HEAP32[$noLock>>2]|0;
  $148 = ($147|0)!=(0);
  if ($148) {
   $149 = HEAP32[$ctrlFlags>>2]|0;
   $150 = $149 | 128;
   HEAP32[$ctrlFlags>>2] = $150;
  }
  $151 = HEAP32[$syncDir>>2]|0;
  $152 = ($151|0)!=(0);
  if ($152) {
   $153 = HEAP32[$ctrlFlags>>2]|0;
   $154 = $153 | 8;
   HEAP32[$ctrlFlags>>2] = $154;
  }
  $155 = HEAP32[$4>>2]|0;
  $156 = $155 & 64;
  $157 = ($156|0)!=(0);
  if ($157) {
   $158 = HEAP32[$ctrlFlags>>2]|0;
   $159 = $158 | 64;
   HEAP32[$ctrlFlags>>2] = $159;
  }
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$fd>>2]|0;
  $162 = HEAP32[$3>>2]|0;
  $163 = HEAP32[$2>>2]|0;
  $164 = HEAP32[$ctrlFlags>>2]|0;
  $165 = (_fillInUnixFile($160,$161,$162,$163,$164)|0);
  HEAP32[$rc>>2] = $165;
 }
 $166 = HEAP32[$rc>>2]|0;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = HEAP32[$p>>2]|0;
  $169 = ((($168)) + 28|0);
  $170 = HEAP32[$169>>2]|0;
  _sqlite3_free($170);
 }
 $171 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $171;
 $172 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($172|0);
}
function _unixDelete($NotUsed,$zPath,$dirSync) {
 $NotUsed = $NotUsed|0;
 $zPath = $zPath|0;
 $dirSync = $dirSync|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $fd = sp;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $dirSync;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[(1260)>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 255]($5)|0);
 $7 = ($6|0)==(-1);
 if ($7) {
  $8 = (___errno_location()|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(2);
  if ($10) {
   HEAP32[$rc>>2] = 5898;
  } else {
   $11 = HEAP32[$2>>2]|0;
   $12 = (_unixLogErrorAtLine(2570,18432,$11,33178)|0);
   HEAP32[$rc>>2] = $12;
  }
  $13 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $13;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = $14 & 1;
 $16 = ($15|0)!=(0);
 do {
  if ($16) {
   $17 = HEAP32[(1272)>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = (FUNCTION_TABLE_iii[$17 & 255]($18,$fd)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)==(0);
   if (!($21)) {
    HEAP32[$rc>>2] = 0;
    break;
   }
   $22 = HEAP32[$fd>>2]|0;
   $23 = (_full_fsync($22,0,0)|0);
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$2>>2]|0;
    $26 = (_unixLogErrorAtLine(1290,18754,$25,33188)|0);
    HEAP32[$rc>>2] = $26;
   }
   $27 = HEAP32[$fd>>2]|0;
   _robust_close(0,$27,33190);
  }
 } while(0);
 $28 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _unixAccess($NotUsed,$zPath,$flags,$pResOut) {
 $NotUsed = $NotUsed|0;
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $buf = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pResOut;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[(1116)>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (FUNCTION_TABLE_iii[$6 & 255]($7,$buf)|0);
  $9 = (0)==($8|0);
  $10 = ((($buf)) + 36|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)>(0);
  $13 = $9 ? $12 : 0;
  $14 = $13&1;
  $15 = HEAP32[$3>>2]|0;
  HEAP32[$15>>2] = $14;
  STACKTOP = sp;return 0;
 } else {
  $16 = HEAP32[(1092)>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = (FUNCTION_TABLE_iii[$16 & 255]($17,6)|0);
  $19 = ($18|0)==(0);
  $20 = $19&1;
  $21 = HEAP32[$3>>2]|0;
  HEAP32[$21>>2] = $20;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _unixFullPathname($pVfs,$zPath,$nOut,$zOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $nOut = $nOut|0;
 $zOut = $zOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $bLink = 0, $buf = 0, $n = 0, $nByte = 0, $nLink = 0, $rc = 0, $zDel = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $rc = sp + 100|0;
 $nByte = sp + 96|0;
 $nLink = sp + 92|0;
 $zIn = sp + 88|0;
 $zDel = sp + 84|0;
 $bLink = sp + 80|0;
 $buf = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $nOut;
 HEAP32[$3>>2] = $zOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nLink>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$zIn>>2] = $4;
 HEAP32[$zDel>>2] = 0;
 while(1) {
  HEAP32[$bLink>>2] = 0;
  $5 = HEAP32[(1392)>>2]|0;
  $6 = HEAP32[$zIn>>2]|0;
  $7 = (FUNCTION_TABLE_iii[$5 & 255]($6,$buf)|0);
  $8 = ($7|0)!=(0);
  if ($8) {
   $9 = (___errno_location()|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(2);
   if ($11) {
    $12 = (_sqlite3CantopenError(33302)|0);
    $13 = HEAP32[$zIn>>2]|0;
    $14 = (_unixLogErrorAtLine($12,18520,$13,33302)|0);
    HEAP32[$rc>>2] = $14;
   }
  } else {
   $15 = ((($buf)) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $16 & 61440;
   $18 = ($17|0)==(40960);
   $19 = $18&1;
   HEAP32[$bLink>>2] = $19;
  }
  $20 = HEAP32[$bLink>>2]|0;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$zDel>>2]|0;
   $23 = ($22|0)==(0|0);
   if ($23) {
    $24 = HEAP32[$2>>2]|0;
    $25 = (_sqlite3_malloc($24)|0);
    HEAP32[$zDel>>2] = $25;
    $26 = HEAP32[$zDel>>2]|0;
    $27 = ($26|0)==(0|0);
    if ($27) {
     HEAP32[$rc>>2] = 7;
    }
   } else {
    $28 = HEAP32[$nLink>>2]|0;
    $29 = (($28) + 1)|0;
    HEAP32[$nLink>>2] = $29;
    $30 = ($29|0)>(100);
    if ($30) {
     $31 = (_sqlite3CantopenError(33313)|0);
     HEAP32[$rc>>2] = $31;
    }
   }
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(0);
   do {
    if ($33) {
     $34 = HEAP32[(1380)>>2]|0;
     $35 = HEAP32[$zIn>>2]|0;
     $36 = HEAP32[$zDel>>2]|0;
     $37 = HEAP32[$2>>2]|0;
     $38 = (($37) - 1)|0;
     $39 = (FUNCTION_TABLE_iiii[$34 & 255]($35,$36,$38)|0);
     HEAP32[$nByte>>2] = $39;
     $40 = HEAP32[$nByte>>2]|0;
     $41 = ($40|0)<(0);
     if ($41) {
      $42 = (_sqlite3CantopenError(33319)|0);
      $43 = HEAP32[$zIn>>2]|0;
      $44 = (_unixLogErrorAtLine($42,18511,$43,33319)|0);
      HEAP32[$rc>>2] = $44;
      break;
     }
     $45 = HEAP32[$zDel>>2]|0;
     $46 = HEAP8[$45>>0]|0;
     $47 = $46 << 24 >> 24;
     $48 = ($47|0)!=(47);
     do {
      if ($48) {
       $49 = HEAP32[$zIn>>2]|0;
       $50 = (_sqlite3Strlen30($49)|0);
       HEAP32[$n>>2] = $50;
       while(1) {
        $51 = HEAP32[$n>>2]|0;
        $52 = ($51|0)>(0);
        if (!($52)) {
         break;
        }
        $53 = HEAP32[$n>>2]|0;
        $54 = (($53) - 1)|0;
        $55 = HEAP32[$zIn>>2]|0;
        $56 = (($55) + ($54)|0);
        $57 = HEAP8[$56>>0]|0;
        $58 = $57 << 24 >> 24;
        $59 = ($58|0)!=(47);
        if (!($59)) {
         break;
        }
        $60 = HEAP32[$n>>2]|0;
        $61 = (($60) + -1)|0;
        HEAP32[$n>>2] = $61;
       }
       $62 = HEAP32[$nByte>>2]|0;
       $63 = HEAP32[$n>>2]|0;
       $64 = (($62) + ($63))|0;
       $65 = (($64) + 1)|0;
       $66 = HEAP32[$2>>2]|0;
       $67 = ($65|0)>($66|0);
       if ($67) {
        $68 = (_sqlite3CantopenError(33325)|0);
        HEAP32[$rc>>2] = $68;
        break;
       } else {
        $69 = HEAP32[$n>>2]|0;
        $70 = HEAP32[$zDel>>2]|0;
        $71 = (($70) + ($69)|0);
        $72 = HEAP32[$zDel>>2]|0;
        $73 = HEAP32[$nByte>>2]|0;
        $74 = (($73) + 1)|0;
        _memmove(($71|0),($72|0),($74|0))|0;
        $75 = HEAP32[$zDel>>2]|0;
        $76 = HEAP32[$zIn>>2]|0;
        $77 = HEAP32[$n>>2]|0;
        _memcpy(($75|0),($76|0),($77|0))|0;
        $78 = HEAP32[$n>>2]|0;
        $79 = HEAP32[$nByte>>2]|0;
        $80 = (($79) + ($78))|0;
        HEAP32[$nByte>>2] = $80;
        break;
       }
      }
     } while(0);
     $81 = HEAP32[$nByte>>2]|0;
     $82 = HEAP32[$zDel>>2]|0;
     $83 = (($82) + ($81)|0);
     HEAP8[$83>>0] = 0;
    }
   } while(0);
   $84 = HEAP32[$zDel>>2]|0;
   HEAP32[$zIn>>2] = $84;
  }
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)==(0);
  if ($86) {
   $87 = HEAP32[$zIn>>2]|0;
   $88 = HEAP32[$3>>2]|0;
   $89 = ($87|0)!=($88|0);
   if ($89) {
    $90 = HEAP32[$zIn>>2]|0;
    $91 = HEAP32[$3>>2]|0;
    $92 = HEAP32[$2>>2]|0;
    $93 = (_mkFullPathname($90,$91,$92)|0);
    HEAP32[$rc>>2] = $93;
   }
  }
  $94 = HEAP32[$bLink>>2]|0;
  $95 = ($94|0)==(0);
  if ($95) {
   label = 30;
   break;
  }
  $96 = HEAP32[$3>>2]|0;
  HEAP32[$zIn>>2] = $96;
  $97 = HEAP32[$rc>>2]|0;
  $98 = ($97|0)==(0);
  if (!($98)) {
   label = 30;
   break;
  }
 }
 if ((label|0) == 30) {
  $99 = HEAP32[$zDel>>2]|0;
  _sqlite3_free($99);
  $100 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($100|0);
 }
 return (0)|0;
}
function _unixRandomness($NotUsed,$nBuf,$zBuf) {
 $NotUsed = $NotUsed|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $got = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $fd = sp + 8|0;
 $got = sp + 4|0;
 $t = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $nBuf;
 HEAP32[$2>>2] = $zBuf;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memset(($3|0),0,($4|0))|0;
 $5 = (_getpid()|0);
 HEAP32[1428>>2] = $5;
 $6 = (_robust_open(18760,0,0)|0);
 HEAP32[$fd>>2] = $6;
 $7 = HEAP32[$fd>>2]|0;
 $8 = ($7|0)<(0);
 if ($8) {
  (_time(($t|0))|0);
  $9 = HEAP32[$2>>2]|0;
  ;HEAP8[$9>>0]=HEAP8[$t>>0]|0;HEAP8[$9+1>>0]=HEAP8[$t+1>>0]|0;HEAP8[$9+2>>0]=HEAP8[$t+2>>0]|0;HEAP8[$9+3>>0]=HEAP8[$t+3>>0]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 4|0);
  ;HEAP8[$11>>0]=HEAP8[1428>>0]|0;HEAP8[$11+1>>0]=HEAP8[1428+1>>0]|0;HEAP8[$11+2>>0]=HEAP8[1428+2>>0]|0;HEAP8[$11+3>>0]=HEAP8[1428+3>>0]|0;
  HEAP32[$1>>2] = 8;
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 while(1) {
  $12 = HEAP32[(1164)>>2]|0;
  $13 = HEAP32[$fd>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (FUNCTION_TABLE_iiii[$12 & 255]($13,$14,$15)|0);
  HEAP32[$got>>2] = $16;
  $17 = HEAP32[$got>>2]|0;
  $18 = ($17|0)<(0);
  if (!($18)) {
   break;
  }
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(4);
  if (!($21)) {
   break;
  }
 }
 $22 = HEAP32[$fd>>2]|0;
 _robust_close(0,$22,33449);
 $23 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _unixSleep($NotUsed,$microseconds) {
 $NotUsed = $NotUsed|0;
 $microseconds = $microseconds|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $seconds = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $seconds = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $microseconds;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 999999)|0;
 $4 = (($3|0) / 1000000)&-1;
 HEAP32[$seconds>>2] = $4;
 $5 = HEAP32[$seconds>>2]|0;
 (_sleep($5)|0);
 $6 = HEAP32[$seconds>>2]|0;
 $7 = ($6*1000000)|0;
 STACKTOP = sp;return ($7|0);
}
function _unixCurrentTime($NotUsed,$prNow) {
 $NotUsed = $NotUsed|0;
 $prNow = $prNow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $prNow;
 $2 = $i;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (_unixCurrentTimeInt64(0,$i)|0);
 HEAP32[$rc>>2] = $6;
 $7 = $i;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($9>>>0)) + (4294967296.0*(+($12|0)));
 $14 = $13 / 8.64E+7;
 $15 = HEAP32[$1>>2]|0;
 HEAPF64[$15>>3] = $14;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _unixGetLastError($NotUsed,$NotUsed2,$NotUsed3) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 $NotUsed3 = $NotUsed3|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 HEAP32[$2>>2] = $NotUsed3;
 STACKTOP = sp;return 0;
}
function _unixCurrentTimeInt64($NotUsed,$piNow) {
 $NotUsed = $NotUsed|0;
 $piNow = $piNow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, $sNow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $sNow = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $piNow;
 HEAP32[$rc>>2] = 0;
 (_gettimeofday(($sNow|0),(0|0))|0);
 $2 = HEAP32[$sNow>>2]|0;
 $3 = ($2|0)<(0);
 $4 = $3 << 31 >> 31;
 $5 = (___muldi3(1000,0,($2|0),($4|0))|0);
 $6 = tempRet0;
 $7 = (_i64Add(1045635584,49096,($5|0),($6|0))|0);
 $8 = tempRet0;
 $9 = ((($sNow)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10|0) / 1000)&-1;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = (_i64Add(($7|0),($8|0),($11|0),($13|0))|0);
 $15 = tempRet0;
 $16 = HEAP32[$1>>2]|0;
 $17 = $16;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $15;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _unixSetSystemCall($pNotUsed,$zName,$pNewFunc) {
 $pNotUsed = $pNotUsed|0;
 $zName = $zName|0;
 $pNewFunc = $pNewFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pNotUsed;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pNewFunc;
 HEAP32[$rc>>2] = 12;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$rc>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $5 = HEAP32[$i>>2]|0;
   $6 = ($5>>>0)<(28);
   if (!($6)) {
    break;
   }
   $7 = HEAP32[$i>>2]|0;
   $8 = (1064 + (($7*12)|0)|0);
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$i>>2]|0;
    $13 = (1064 + (($12*12)|0)|0);
    $14 = ((($13)) + 8|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = HEAP32[$i>>2]|0;
    $17 = (1064 + (($16*12)|0)|0);
    $18 = ((($17)) + 4|0);
    HEAP32[$18>>2] = $15;
   }
   $19 = HEAP32[$i>>2]|0;
   $20 = (($19) + 1)|0;
   HEAP32[$i>>2] = $20;
  }
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = ($21>>>0)<(28);
  if (!($22)) {
   label = 16;
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = (1064 + (($24*12)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_strcmp($23,$26)|0);
  $28 = ($27|0)==(0);
  $29 = HEAP32[$i>>2]|0;
  if ($28) {
   break;
  }
  $51 = (($29) + 1)|0;
  HEAP32[$i>>2] = $51;
 }
 if ((label|0) == 16) {
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $30 = (1064 + (($29*12)|0)|0);
 $31 = ((($30)) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  $34 = HEAP32[$i>>2]|0;
  $35 = (1064 + (($34*12)|0)|0);
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$i>>2]|0;
  $39 = (1064 + (($38*12)|0)|0);
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $37;
 }
 HEAP32[$rc>>2] = 0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  $43 = HEAP32[$i>>2]|0;
  $44 = (1064 + (($43*12)|0)|0);
  $45 = ((($44)) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$2>>2] = $46;
 }
 $47 = HEAP32[$2>>2]|0;
 $48 = HEAP32[$i>>2]|0;
 $49 = (1064 + (($48*12)|0)|0);
 $50 = ((($49)) + 4|0);
 HEAP32[$50>>2] = $47;
 $52 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _unixGetSystemCall($pNotUsed,$zName) {
 $pNotUsed = $pNotUsed|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pNotUsed;
 HEAP32[$2>>2] = $zName;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3>>>0)<(28);
  if (!($4)) {
   label = 6;
   break;
  }
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$i>>2]|0;
  $7 = (1064 + (($6*12)|0)|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_strcmp($5,$8)|0);
  $10 = ($9|0)==(0);
  $11 = HEAP32[$i>>2]|0;
  if ($10) {
   label = 4;
   break;
  }
  $15 = (($11) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 if ((label|0) == 4) {
  $12 = (1064 + (($11*12)|0)|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$0>>2] = $14;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _unixNextSystemCall($p,$zName) {
 $p = $p|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zName;
 HEAP32[$i>>2] = -1;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $5 = HEAP32[$i>>2]|0;
    $6 = ($5|0)<(27);
    if (!($6)) {
     break L1;
    }
    $7 = HEAP32[$2>>2]|0;
    $8 = HEAP32[$i>>2]|0;
    $9 = (1064 + (($8*12)|0)|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = (_strcmp($7,$10)|0);
    $12 = ($11|0)==(0);
    if ($12) {
     break L1;
    }
    $13 = HEAP32[$i>>2]|0;
    $14 = (($13) + 1)|0;
    HEAP32[$i>>2] = $14;
   }
  }
 } while(0);
 $15 = HEAP32[$i>>2]|0;
 $16 = (($15) + 1)|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)<(28);
  if (!($18)) {
   label = 11;
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (1064 + (($19*12)|0)|0);
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  $24 = HEAP32[$i>>2]|0;
  if ($23) {
   label = 9;
   break;
  }
  $27 = (($24) + 1)|0;
  HEAP32[$i>>2] = $27;
 }
 if ((label|0) == 9) {
  $25 = (1064 + (($24*12)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _sqlite3ErrorWithMsg($db,$err_code,$zFormat,$varargs) {
 $db = $db|0;
 $err_code = $err_code|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $z = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 52|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  $9 = HEAP32[$1>>2]|0;
  _sqlite3Error($8,$9);
  STACKTOP = sp;return;
 }
 $10 = ((($8)) + 236|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $13 = HEAP32[$0>>2]|0;
  $14 = (_sqlite3ValueNew($13)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 236|0);
  HEAP32[$16>>2] = $14;
  $17 = ($14|0)!=(0|0);
  if (!($17)) {
   STACKTOP = sp;return;
  }
 }
 HEAP32[$ap>>2] = $varargs;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3VMPrintf($18,$19,$ap)|0);
 HEAP32[$z>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 236|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$z>>2]|0;
 _sqlite3ValueSetStr($23,-1,$24,1,169);
 STACKTOP = sp;return;
}
function _sqlite3MallocZero($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $p = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 $7 = $2;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3Malloc($9,$12)|0);
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $23 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = $2;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 _memset(($16|0),0,($19|0))|0;
 $23 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3Error($db,$err_code) {
 $db = $db|0;
 $err_code = $err_code|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 52|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 236|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 236|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3ValueSetNull($11);
 STACKTOP = sp;return;
}
function _sqlite3_backup_step($p,$nPage) {
 $p = $p|0;
 $nPage = $nPage|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bCloseTrans = 0, $destMode = 0, $iEnd = 0, $iOff = 0, $iPg = 0;
 var $iSize = 0, $iSrcPg = 0, $iSrcPg2 = 0, $ii = 0, $nDestTruncate = 0, $nDstPage = 0, $nSrcPage = 0, $or$cond = 0, $or$cond4 = 0, $pDestPager = 0, $pFile = 0, $pPg = 0, $pSrcPager = 0, $pSrcPg = 0, $pSrcPg1 = 0, $pgszDest = 0, $pgszSrc = 0, $ratio = 0, $rc = 0, $zData = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $rc = sp + 100|0;
 $destMode = sp + 96|0;
 $pgszSrc = sp + 92|0;
 $pgszDest = sp + 88|0;
 $pSrcPager = sp + 84|0;
 $pDestPager = sp + 80|0;
 $ii = sp + 76|0;
 $nSrcPage = sp + 72|0;
 $bCloseTrans = sp + 68|0;
 $iSrcPg = sp + 64|0;
 $pSrcPg = sp + 60|0;
 $nDestTruncate = sp + 56|0;
 $ratio = sp + 52|0;
 $iSize = sp + 16|0;
 $pFile = sp + 48|0;
 $iPg = sp + 44|0;
 $nDstPage = sp + 40|0;
 $iOff = sp + 8|0;
 $iEnd = sp;
 $pPg = sp + 36|0;
 $pSrcPg1 = sp + 32|0;
 $iSrcPg2 = sp + 28|0;
 $zData = sp + 24|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nPage;
 HEAP32[$pgszSrc>>2] = 0;
 HEAP32[$pgszDest>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = (_isFatalError($8)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $413 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($413|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3BtreePager($13)|0);
 HEAP32[$pSrcPager>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3BtreePager($17)|0);
 HEAP32[$pDestPager>>2] = $18;
 HEAP32[$nSrcPage>>2] = -1;
 HEAP32[$bCloseTrans>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0|0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(2);
  if ($30) {
   HEAP32[$rc>>2] = 5;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  HEAP32[$rc>>2] = 0;
 }
 $31 = HEAP32[$rc>>2]|0;
 $32 = (0)==($31|0);
 if ($32) {
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 12|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$0>>2]|0;
   $38 = ((($37)) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3BtreeBeginTrans($39,2)|0);
   HEAP32[$rc>>2] = $40;
   $41 = (0)==($40|0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 12|0);
    HEAP32[$43>>2] = 1;
    $44 = HEAP32[$0>>2]|0;
    $45 = ((($44)) + 4|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$0>>2]|0;
    $48 = ((($47)) + 8|0);
    _sqlite3BtreeGetMeta($46,1,$48);
   }
  }
 }
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$0>>2]|0;
  $52 = ((($51)) + 24|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3BtreeIsInReadTrans($53)|0);
  $55 = (0)==($54|0);
  if ($55) {
   $56 = HEAP32[$0>>2]|0;
   $57 = ((($56)) + 24|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = (_sqlite3BtreeBeginTrans($58,0)|0);
   HEAP32[$rc>>2] = $59;
   HEAP32[$bCloseTrans>>2] = 1;
  }
 }
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 24|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3BtreeGetPageSize($62)|0);
 HEAP32[$pgszSrc>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 4|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = (_sqlite3BtreeGetPageSize($66)|0);
 HEAP32[$pgszDest>>2] = $67;
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 4|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (_sqlite3BtreePager($70)|0);
 $72 = (_sqlite3PagerGetJournalMode($71)|0);
 HEAP32[$destMode>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 $74 = (0)==($73|0);
 $75 = HEAP32[$destMode>>2]|0;
 $76 = ($75|0)==(5);
 $or$cond = $74 & $76;
 if ($or$cond) {
  $77 = HEAP32[$pgszSrc>>2]|0;
  $78 = HEAP32[$pgszDest>>2]|0;
  $79 = ($77|0)!=($78|0);
  if ($79) {
   HEAP32[$rc>>2] = 8;
  }
 }
 $80 = HEAP32[$0>>2]|0;
 $81 = ((($80)) + 24|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = (_sqlite3BtreeLastPage($82)|0);
 HEAP32[$nSrcPage>>2] = $83;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $84 = HEAP32[$1>>2]|0;
  $85 = ($84|0)<(0);
  if (!($85)) {
   $86 = HEAP32[$ii>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = ($86|0)<($87|0);
   if (!($88)) {
    break;
   }
  }
  $89 = HEAP32[$0>>2]|0;
  $90 = ((($89)) + 16|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = HEAP32[$nSrcPage>>2]|0;
  $93 = ($91>>>0)<=($92>>>0);
  if (!($93)) {
   break;
  }
  $94 = HEAP32[$rc>>2]|0;
  $95 = ($94|0)!=(0);
  $96 = $95 ^ 1;
  if (!($96)) {
   break;
  }
  $97 = HEAP32[$0>>2]|0;
  $98 = ((($97)) + 16|0);
  $99 = HEAP32[$98>>2]|0;
  HEAP32[$iSrcPg>>2] = $99;
  $100 = HEAP32[$iSrcPg>>2]|0;
  $101 = HEAP32[832>>2]|0;
  $102 = HEAP32[$0>>2]|0;
  $103 = ((($102)) + 24|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = ((($104)) + 4|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = ((($106)) + 32|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = (($101>>>0) / ($108>>>0))&-1;
  $110 = (($109) + 1)|0;
  $111 = ($100|0)!=($110|0);
  if ($111) {
   $112 = HEAP32[$pSrcPager>>2]|0;
   $113 = HEAP32[$iSrcPg>>2]|0;
   $114 = (_sqlite3PagerGet($112,$113,$pSrcPg,2)|0);
   HEAP32[$rc>>2] = $114;
   $115 = HEAP32[$rc>>2]|0;
   $116 = ($115|0)==(0);
   if ($116) {
    $117 = HEAP32[$0>>2]|0;
    $118 = HEAP32[$iSrcPg>>2]|0;
    $119 = HEAP32[$pSrcPg>>2]|0;
    $120 = (_sqlite3PagerGetData($119)|0);
    $121 = (_backupOnePage($117,$118,$120,0)|0);
    HEAP32[$rc>>2] = $121;
    $122 = HEAP32[$pSrcPg>>2]|0;
    _sqlite3PagerUnref($122);
   }
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = ((($123)) + 16|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = (($125) + 1)|0;
  HEAP32[$124>>2] = $126;
  $127 = HEAP32[$ii>>2]|0;
  $128 = (($127) + 1)|0;
  HEAP32[$ii>>2] = $128;
 }
 $129 = HEAP32[$rc>>2]|0;
 $130 = ($129|0)==(0);
 do {
  if ($130) {
   $131 = HEAP32[$nSrcPage>>2]|0;
   $132 = HEAP32[$0>>2]|0;
   $133 = ((($132)) + 36|0);
   HEAP32[$133>>2] = $131;
   $134 = HEAP32[$nSrcPage>>2]|0;
   $135 = (($134) + 1)|0;
   $136 = HEAP32[$0>>2]|0;
   $137 = ((($136)) + 16|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = (($135) - ($138))|0;
   $140 = HEAP32[$0>>2]|0;
   $141 = ((($140)) + 32|0);
   HEAP32[$141>>2] = $139;
   $142 = HEAP32[$0>>2]|0;
   $143 = ((($142)) + 16|0);
   $144 = HEAP32[$143>>2]|0;
   $145 = HEAP32[$nSrcPage>>2]|0;
   $146 = ($144>>>0)>($145>>>0);
   if ($146) {
    HEAP32[$rc>>2] = 101;
    break;
   }
   $147 = HEAP32[$0>>2]|0;
   $148 = ((($147)) + 40|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = ($149|0)!=(0);
   if (!($150)) {
    $151 = HEAP32[$0>>2]|0;
    _attachBackupObject($151);
   }
  }
 } while(0);
 $152 = HEAP32[$rc>>2]|0;
 $153 = ($152|0)==(101);
 if ($153) {
  $154 = HEAP32[$nSrcPage>>2]|0;
  $155 = ($154|0)==(0);
  if ($155) {
   $156 = HEAP32[$0>>2]|0;
   $157 = ((($156)) + 4|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = (_sqlite3BtreeNewDb($158)|0);
   HEAP32[$rc>>2] = $159;
   HEAP32[$nSrcPage>>2] = 1;
  }
  $160 = HEAP32[$rc>>2]|0;
  $161 = ($160|0)==(0);
  $162 = HEAP32[$rc>>2]|0;
  $163 = ($162|0)==(101);
  $or$cond4 = $161 | $163;
  if ($or$cond4) {
   $164 = HEAP32[$0>>2]|0;
   $165 = ((($164)) + 4|0);
   $166 = HEAP32[$165>>2]|0;
   $167 = HEAP32[$0>>2]|0;
   $168 = ((($167)) + 8|0);
   $169 = HEAP32[$168>>2]|0;
   $170 = (($169) + 1)|0;
   $171 = (_sqlite3BtreeUpdateMeta($166,1,$170)|0);
   HEAP32[$rc>>2] = $171;
  }
  $172 = HEAP32[$rc>>2]|0;
  $173 = ($172|0)==(0);
  if ($173) {
   $174 = HEAP32[$0>>2]|0;
   $175 = HEAP32[$174>>2]|0;
   $176 = ($175|0)!=(0|0);
   if ($176) {
    $177 = HEAP32[$0>>2]|0;
    $178 = HEAP32[$177>>2]|0;
    _sqlite3ResetAllSchemasOfConnection($178);
   }
   $179 = HEAP32[$destMode>>2]|0;
   $180 = ($179|0)==(5);
   if ($180) {
    $181 = HEAP32[$0>>2]|0;
    $182 = ((($181)) + 4|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = (_sqlite3BtreeSetVersion($183,2)|0);
    HEAP32[$rc>>2] = $184;
   }
  }
  $185 = HEAP32[$rc>>2]|0;
  $186 = ($185|0)==(0);
  if ($186) {
   $187 = HEAP32[$pgszSrc>>2]|0;
   $188 = HEAP32[$pgszDest>>2]|0;
   $189 = ($187|0)<($188|0);
   if ($189) {
    $190 = HEAP32[$pgszDest>>2]|0;
    $191 = HEAP32[$pgszSrc>>2]|0;
    $192 = (($190|0) / ($191|0))&-1;
    HEAP32[$ratio>>2] = $192;
    $193 = HEAP32[$nSrcPage>>2]|0;
    $194 = HEAP32[$ratio>>2]|0;
    $195 = (($193) + ($194))|0;
    $196 = (($195) - 1)|0;
    $197 = HEAP32[$ratio>>2]|0;
    $198 = (($196|0) / ($197|0))&-1;
    HEAP32[$nDestTruncate>>2] = $198;
    $199 = HEAP32[$nDestTruncate>>2]|0;
    $200 = HEAP32[832>>2]|0;
    $201 = HEAP32[$0>>2]|0;
    $202 = ((($201)) + 4|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = ((($203)) + 4|0);
    $205 = HEAP32[$204>>2]|0;
    $206 = ((($205)) + 32|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = (($200>>>0) / ($207>>>0))&-1;
    $209 = (($208) + 1)|0;
    $210 = ($199|0)==($209|0);
    if ($210) {
     $211 = HEAP32[$nDestTruncate>>2]|0;
     $212 = (($211) + -1)|0;
     HEAP32[$nDestTruncate>>2] = $212;
    }
   } else {
    $213 = HEAP32[$nSrcPage>>2]|0;
    $214 = HEAP32[$pgszSrc>>2]|0;
    $215 = HEAP32[$pgszDest>>2]|0;
    $216 = (($214|0) / ($215|0))&-1;
    $217 = Math_imul($213, $216)|0;
    HEAP32[$nDestTruncate>>2] = $217;
   }
   $218 = HEAP32[$pgszSrc>>2]|0;
   $219 = HEAP32[$pgszDest>>2]|0;
   $220 = ($218|0)<($219|0);
   if ($220) {
    $221 = HEAP32[$pgszSrc>>2]|0;
    $222 = ($221|0)<(0);
    $223 = $222 << 31 >> 31;
    $224 = HEAP32[$nSrcPage>>2]|0;
    $225 = ($224|0)<(0);
    $226 = $225 << 31 >> 31;
    $227 = (___muldi3(($221|0),($223|0),($224|0),($226|0))|0);
    $228 = tempRet0;
    $229 = $iSize;
    $230 = $229;
    HEAP32[$230>>2] = $227;
    $231 = (($229) + 4)|0;
    $232 = $231;
    HEAP32[$232>>2] = $228;
    $233 = HEAP32[$pDestPager>>2]|0;
    $234 = (_sqlite3PagerFile($233)|0);
    HEAP32[$pFile>>2] = $234;
    $235 = HEAP32[$pDestPager>>2]|0;
    _sqlite3PagerPagecount($235,$nDstPage);
    $236 = HEAP32[$nDestTruncate>>2]|0;
    HEAP32[$iPg>>2] = $236;
    while(1) {
     $237 = HEAP32[$rc>>2]|0;
     $238 = ($237|0)==(0);
     if (!($238)) {
      break;
     }
     $239 = HEAP32[$iPg>>2]|0;
     $240 = HEAP32[$nDstPage>>2]|0;
     $241 = ($239>>>0)<=($240>>>0);
     if (!($241)) {
      break;
     }
     $242 = HEAP32[$iPg>>2]|0;
     $243 = HEAP32[832>>2]|0;
     $244 = HEAP32[$0>>2]|0;
     $245 = ((($244)) + 4|0);
     $246 = HEAP32[$245>>2]|0;
     $247 = ((($246)) + 4|0);
     $248 = HEAP32[$247>>2]|0;
     $249 = ((($248)) + 32|0);
     $250 = HEAP32[$249>>2]|0;
     $251 = (($243>>>0) / ($250>>>0))&-1;
     $252 = (($251) + 1)|0;
     $253 = ($242|0)!=($252|0);
     do {
      if ($253) {
       $254 = HEAP32[$pDestPager>>2]|0;
       $255 = HEAP32[$iPg>>2]|0;
       $256 = (_sqlite3PagerGet($254,$255,$pPg,0)|0);
       HEAP32[$rc>>2] = $256;
       $257 = HEAP32[$rc>>2]|0;
       $258 = ($257|0)==(0);
       if (!($258)) {
        break;
       }
       $259 = HEAP32[$pPg>>2]|0;
       $260 = (_sqlite3PagerWrite($259)|0);
       HEAP32[$rc>>2] = $260;
       $261 = HEAP32[$pPg>>2]|0;
       _sqlite3PagerUnref($261);
      }
     } while(0);
     $262 = HEAP32[$iPg>>2]|0;
     $263 = (($262) + 1)|0;
     HEAP32[$iPg>>2] = $263;
    }
    $264 = HEAP32[$rc>>2]|0;
    $265 = ($264|0)==(0);
    if ($265) {
     $266 = HEAP32[$pDestPager>>2]|0;
     $267 = (_sqlite3PagerCommitPhaseOne($266,0,1)|0);
     HEAP32[$rc>>2] = $267;
    }
    $268 = HEAP32[832>>2]|0;
    $269 = HEAP32[$pgszDest>>2]|0;
    $270 = (($268) + ($269))|0;
    $271 = ($270|0)<(0);
    $272 = $271 << 31 >> 31;
    $273 = $iSize;
    $274 = $273;
    $275 = HEAP32[$274>>2]|0;
    $276 = (($273) + 4)|0;
    $277 = $276;
    $278 = HEAP32[$277>>2]|0;
    $279 = ($272|0)<($278|0);
    $280 = ($270>>>0)<($275>>>0);
    $281 = ($272|0)==($278|0);
    $282 = $281 & $280;
    $283 = $279 | $282;
    if ($283) {
     $284 = HEAP32[832>>2]|0;
     $285 = HEAP32[$pgszDest>>2]|0;
     $286 = (($284) + ($285))|0;
     $287 = ($286|0)<(0);
     $288 = $287 << 31 >> 31;
     $297 = $286;$300 = $288;
    } else {
     $289 = $iSize;
     $290 = $289;
     $291 = HEAP32[$290>>2]|0;
     $292 = (($289) + 4)|0;
     $293 = $292;
     $294 = HEAP32[$293>>2]|0;
     $297 = $291;$300 = $294;
    }
    $295 = $iEnd;
    $296 = $295;
    HEAP32[$296>>2] = $297;
    $298 = (($295) + 4)|0;
    $299 = $298;
    HEAP32[$299>>2] = $300;
    $301 = HEAP32[832>>2]|0;
    $302 = HEAP32[$pgszSrc>>2]|0;
    $303 = (($301) + ($302))|0;
    $304 = ($303|0)<(0);
    $305 = $304 << 31 >> 31;
    $306 = $iOff;
    $307 = $306;
    HEAP32[$307>>2] = $303;
    $308 = (($306) + 4)|0;
    $309 = $308;
    HEAP32[$309>>2] = $305;
    while(1) {
     $310 = HEAP32[$rc>>2]|0;
     $311 = ($310|0)==(0);
     if (!($311)) {
      break;
     }
     $312 = $iOff;
     $313 = $312;
     $314 = HEAP32[$313>>2]|0;
     $315 = (($312) + 4)|0;
     $316 = $315;
     $317 = HEAP32[$316>>2]|0;
     $318 = $iEnd;
     $319 = $318;
     $320 = HEAP32[$319>>2]|0;
     $321 = (($318) + 4)|0;
     $322 = $321;
     $323 = HEAP32[$322>>2]|0;
     $324 = ($317|0)<($323|0);
     $325 = ($314>>>0)<($320>>>0);
     $326 = ($317|0)==($323|0);
     $327 = $326 & $325;
     $328 = $324 | $327;
     if (!($328)) {
      break;
     }
     HEAP32[$pSrcPg1>>2] = 0;
     $329 = $iOff;
     $330 = $329;
     $331 = HEAP32[$330>>2]|0;
     $332 = (($329) + 4)|0;
     $333 = $332;
     $334 = HEAP32[$333>>2]|0;
     $335 = HEAP32[$pgszSrc>>2]|0;
     $336 = ($335|0)<(0);
     $337 = $336 << 31 >> 31;
     $338 = (___divdi3(($331|0),($334|0),($335|0),($337|0))|0);
     $339 = tempRet0;
     $340 = (_i64Add(($338|0),($339|0),1,0)|0);
     $341 = tempRet0;
     HEAP32[$iSrcPg2>>2] = $340;
     $342 = HEAP32[$pSrcPager>>2]|0;
     $343 = HEAP32[$iSrcPg2>>2]|0;
     $344 = (_sqlite3PagerGet($342,$343,$pSrcPg1,0)|0);
     HEAP32[$rc>>2] = $344;
     $345 = HEAP32[$rc>>2]|0;
     $346 = ($345|0)==(0);
     if ($346) {
      $347 = HEAP32[$pSrcPg1>>2]|0;
      $348 = (_sqlite3PagerGetData($347)|0);
      HEAP32[$zData>>2] = $348;
      $349 = HEAP32[$pFile>>2]|0;
      $350 = HEAP32[$zData>>2]|0;
      $351 = HEAP32[$pgszSrc>>2]|0;
      $352 = $iOff;
      $353 = $352;
      $354 = HEAP32[$353>>2]|0;
      $355 = (($352) + 4)|0;
      $356 = $355;
      $357 = HEAP32[$356>>2]|0;
      $358 = (_sqlite3OsWrite($349,$350,$351,$354,$357)|0);
      HEAP32[$rc>>2] = $358;
     }
     $359 = HEAP32[$pSrcPg1>>2]|0;
     _sqlite3PagerUnref($359);
     $360 = HEAP32[$pgszSrc>>2]|0;
     $361 = ($360|0)<(0);
     $362 = $361 << 31 >> 31;
     $363 = $iOff;
     $364 = $363;
     $365 = HEAP32[$364>>2]|0;
     $366 = (($363) + 4)|0;
     $367 = $366;
     $368 = HEAP32[$367>>2]|0;
     $369 = (_i64Add(($365|0),($368|0),($360|0),($362|0))|0);
     $370 = tempRet0;
     $371 = $iOff;
     $372 = $371;
     HEAP32[$372>>2] = $369;
     $373 = (($371) + 4)|0;
     $374 = $373;
     HEAP32[$374>>2] = $370;
    }
    $375 = HEAP32[$rc>>2]|0;
    $376 = ($375|0)==(0);
    if ($376) {
     $377 = HEAP32[$pFile>>2]|0;
     $378 = $iSize;
     $379 = $378;
     $380 = HEAP32[$379>>2]|0;
     $381 = (($378) + 4)|0;
     $382 = $381;
     $383 = HEAP32[$382>>2]|0;
     $384 = (_backupTruncateFile($377,$380,$383)|0);
     HEAP32[$rc>>2] = $384;
    }
    $385 = HEAP32[$rc>>2]|0;
    $386 = ($385|0)==(0);
    if ($386) {
     $387 = HEAP32[$pDestPager>>2]|0;
     $388 = (_sqlite3PagerSync($387,0)|0);
     HEAP32[$rc>>2] = $388;
    }
   } else {
    $389 = HEAP32[$pDestPager>>2]|0;
    $390 = HEAP32[$nDestTruncate>>2]|0;
    _sqlite3PagerTruncateImage($389,$390);
    $391 = HEAP32[$pDestPager>>2]|0;
    $392 = (_sqlite3PagerCommitPhaseOne($391,0,0)|0);
    HEAP32[$rc>>2] = $392;
   }
   $393 = HEAP32[$rc>>2]|0;
   $394 = (0)==($393|0);
   if ($394) {
    $395 = HEAP32[$0>>2]|0;
    $396 = ((($395)) + 4|0);
    $397 = HEAP32[$396>>2]|0;
    $398 = (_sqlite3BtreeCommitPhaseTwo($397,0)|0);
    HEAP32[$rc>>2] = $398;
    $399 = (0)==($398|0);
    $$ = $399 ? 101 : $398;
    HEAP32[$rc>>2] = $$;
   }
  }
 }
 $400 = HEAP32[$bCloseTrans>>2]|0;
 $401 = ($400|0)!=(0);
 if ($401) {
  $402 = HEAP32[$0>>2]|0;
  $403 = ((($402)) + 24|0);
  $404 = HEAP32[$403>>2]|0;
  (_sqlite3BtreeCommitPhaseOne($404,0)|0);
  $405 = HEAP32[$0>>2]|0;
  $406 = ((($405)) + 24|0);
  $407 = HEAP32[$406>>2]|0;
  (_sqlite3BtreeCommitPhaseTwo($407,0)|0);
 }
 $408 = HEAP32[$rc>>2]|0;
 $409 = ($408|0)==(3082);
 if ($409) {
  HEAP32[$rc>>2] = 7;
 }
 $410 = HEAP32[$rc>>2]|0;
 $411 = HEAP32[$0>>2]|0;
 $412 = ((($411)) + 28|0);
 HEAP32[$412>>2] = $410;
 $413 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($413|0);
}
function _sqlite3BtreeEnter($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = $2;
 STACKTOP = sp;return;
}
function _isFatalError($rc) {
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $rc;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0);
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(5);
 $or$cond = $2 & $4;
 if (!($or$cond)) {
  $8 = 0;
  $7 = $8&1;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)!=(6);
 $8 = $6;
 $7 = $8&1;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3BtreeBeginTrans($p,$wrflag) {
 $p = $p|0;
 $wrflag = $wrflag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond9 = 0, $pBlock = 0, $pBt = 0, $pIter = 0, $pPage1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pBt = sp + 16|0;
 $rc = sp + 12|0;
 $pBlock = sp + 8|0;
 $pIter = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $wrflag;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(2);
 do {
  if (!($10)) {
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 8|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(1);
   $16 = HEAP32[$1>>2]|0;
   $17 = ($16|0)!=(0);
   $or$cond = $15 | $17;
   if ($or$cond) {
    $18 = HEAP32[$pBt>>2]|0;
    $19 = ((($18)) + 22|0);
    $20 = HEAP16[$19>>1]|0;
    $21 = $20&65535;
    $22 = $21 & 1;
    $23 = ($22|0)!=(0);
    $24 = HEAP32[$1>>2]|0;
    $25 = ($24|0)!=(0);
    $or$cond3 = $23 & $25;
    if ($or$cond3) {
     HEAP32[$rc>>2] = 8;
     break;
    }
    HEAP32[$pBlock>>2] = 0;
    $26 = HEAP32[$1>>2]|0;
    $27 = ($26|0)!=(0);
    if ($27) {
     $28 = HEAP32[$pBt>>2]|0;
     $29 = ((($28)) + 20|0);
     $30 = HEAP8[$29>>0]|0;
     $31 = $30&255;
     $32 = ($31|0)==(2);
     if ($32) {
      label = 8;
     } else {
      label = 7;
     }
    } else {
     label = 7;
    }
    L9: do {
     if ((label|0) == 7) {
      $33 = HEAP32[$pBt>>2]|0;
      $34 = ((($33)) + 22|0);
      $35 = HEAP16[$34>>1]|0;
      $36 = $35&65535;
      $37 = $36 & 64;
      $38 = ($37|0)!=(0);
      if ($38) {
       label = 8;
      } else {
       $43 = HEAP32[$1>>2]|0;
       $44 = ($43|0)>(1);
       if ($44) {
        $45 = HEAP32[$pBt>>2]|0;
        $46 = ((($45)) + 72|0);
        $47 = HEAP32[$46>>2]|0;
        HEAP32[$pIter>>2] = $47;
        while(1) {
         $48 = HEAP32[$pIter>>2]|0;
         $49 = ($48|0)!=(0|0);
         if (!($49)) {
          break L9;
         }
         $50 = HEAP32[$pIter>>2]|0;
         $51 = HEAP32[$50>>2]|0;
         $52 = HEAP32[$0>>2]|0;
         $53 = ($51|0)!=($52|0);
         $54 = HEAP32[$pIter>>2]|0;
         if ($53) {
          break;
         }
         $57 = ((($54)) + 12|0);
         $58 = HEAP32[$57>>2]|0;
         HEAP32[$pIter>>2] = $58;
        }
        $55 = HEAP32[$54>>2]|0;
        $56 = HEAP32[$55>>2]|0;
        HEAP32[$pBlock>>2] = $56;
       }
      }
     }
    } while(0);
    if ((label|0) == 8) {
     $39 = HEAP32[$pBt>>2]|0;
     $40 = ((($39)) + 76|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     HEAP32[$pBlock>>2] = $42;
    }
    $59 = HEAP32[$pBlock>>2]|0;
    $60 = ($59|0)!=(0|0);
    if ($60) {
     HEAP32[$rc>>2] = 262;
     break;
    }
    $61 = HEAP32[$0>>2]|0;
    $62 = (_querySharedCacheTableLock($61,1,1)|0);
    HEAP32[$rc>>2] = $62;
    $63 = HEAP32[$rc>>2]|0;
    $64 = (0)!=($63|0);
    if (!($64)) {
     $65 = HEAP32[$pBt>>2]|0;
     $66 = ((($65)) + 22|0);
     $67 = HEAP16[$66>>1]|0;
     $68 = $67&65535;
     $69 = $68 & -9;
     $70 = $69&65535;
     HEAP16[$66>>1] = $70;
     $71 = HEAP32[$pBt>>2]|0;
     $72 = ((($71)) + 44|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$pBt>>2]|0;
      $76 = ((($75)) + 22|0);
      $77 = HEAP16[$76>>1]|0;
      $78 = $77&65535;
      $79 = $78 | 8;
      $80 = $79&65535;
      HEAP16[$76>>1] = $80;
     }
     while(1) {
      $81 = HEAP32[$pBt>>2]|0;
      $82 = ((($81)) + 12|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = ($83|0)==(0|0);
      if ($84) {
       $85 = HEAP32[$pBt>>2]|0;
       $86 = (_lockBtree($85)|0);
       HEAP32[$rc>>2] = $86;
       $87 = (0)==($86|0);
       if ($87) {
        continue;
       }
      }
      $88 = HEAP32[$rc>>2]|0;
      $89 = ($88|0)==(0);
      $90 = HEAP32[$1>>2]|0;
      $91 = ($90|0)!=(0);
      $or$cond5 = $89 & $91;
      do {
       if ($or$cond5) {
        $92 = HEAP32[$pBt>>2]|0;
        $93 = ((($92)) + 22|0);
        $94 = HEAP16[$93>>1]|0;
        $95 = $94&65535;
        $96 = $95 & 1;
        $97 = ($96|0)!=(0);
        if ($97) {
         HEAP32[$rc>>2] = 8;
         break;
        }
        $98 = HEAP32[$pBt>>2]|0;
        $99 = HEAP32[$98>>2]|0;
        $100 = HEAP32[$1>>2]|0;
        $101 = ($100|0)>(1);
        $102 = $101&1;
        $103 = HEAP32[$0>>2]|0;
        $104 = HEAP32[$103>>2]|0;
        $105 = (_sqlite3TempInMemory($104)|0);
        $106 = (_sqlite3PagerBegin($99,$102,$105)|0);
        HEAP32[$rc>>2] = $106;
        $107 = HEAP32[$rc>>2]|0;
        $108 = ($107|0)==(0);
        if ($108) {
         $109 = HEAP32[$pBt>>2]|0;
         $110 = (_newDatabase($109)|0);
         HEAP32[$rc>>2] = $110;
        }
       }
      } while(0);
      $111 = HEAP32[$rc>>2]|0;
      $112 = ($111|0)!=(0);
      if ($112) {
       $113 = HEAP32[$pBt>>2]|0;
       _unlockBtreeIfUnused($113);
      }
      $114 = HEAP32[$rc>>2]|0;
      $115 = $114 & 255;
      $116 = ($115|0)==(5);
      if (!($116)) {
       break;
      }
      $117 = HEAP32[$pBt>>2]|0;
      $118 = ((($117)) + 20|0);
      $119 = HEAP8[$118>>0]|0;
      $120 = $119&255;
      $121 = ($120|0)==(0);
      if (!($121)) {
       break;
      }
      $122 = HEAP32[$pBt>>2]|0;
      $123 = (_btreeInvokeBusyHandler($122)|0);
      $124 = ($123|0)!=(0);
      if (!($124)) {
       break;
      }
     }
     $125 = HEAP32[$rc>>2]|0;
     $126 = ($125|0)==(0);
     if ($126) {
      $127 = HEAP32[$0>>2]|0;
      $128 = ((($127)) + 8|0);
      $129 = HEAP8[$128>>0]|0;
      $130 = $129&255;
      $131 = ($130|0)==(0);
      if ($131) {
       $132 = HEAP32[$pBt>>2]|0;
       $133 = ((($132)) + 40|0);
       $134 = HEAP32[$133>>2]|0;
       $135 = (($134) + 1)|0;
       HEAP32[$133>>2] = $135;
       $136 = HEAP32[$0>>2]|0;
       $137 = ((($136)) + 9|0);
       $138 = HEAP8[$137>>0]|0;
       $139 = ($138<<24>>24)!=(0);
       if ($139) {
        $140 = HEAP32[$0>>2]|0;
        $141 = ((($140)) + 32|0);
        $142 = ((($141)) + 8|0);
        HEAP8[$142>>0] = 1;
        $143 = HEAP32[$pBt>>2]|0;
        $144 = ((($143)) + 72|0);
        $145 = HEAP32[$144>>2]|0;
        $146 = HEAP32[$0>>2]|0;
        $147 = ((($146)) + 32|0);
        $148 = ((($147)) + 12|0);
        HEAP32[$148>>2] = $145;
        $149 = HEAP32[$0>>2]|0;
        $150 = ((($149)) + 32|0);
        $151 = HEAP32[$pBt>>2]|0;
        $152 = ((($151)) + 72|0);
        HEAP32[$152>>2] = $150;
       }
      }
      $153 = HEAP32[$1>>2]|0;
      $154 = ($153|0)!=(0);
      $155 = $154 ? 2 : 1;
      $156 = $155&255;
      $157 = HEAP32[$0>>2]|0;
      $158 = ((($157)) + 8|0);
      HEAP8[$158>>0] = $156;
      $159 = HEAP32[$0>>2]|0;
      $160 = ((($159)) + 8|0);
      $161 = HEAP8[$160>>0]|0;
      $162 = $161&255;
      $163 = HEAP32[$pBt>>2]|0;
      $164 = ((($163)) + 20|0);
      $165 = HEAP8[$164>>0]|0;
      $166 = $165&255;
      $167 = ($162|0)>($166|0);
      if ($167) {
       $168 = HEAP32[$0>>2]|0;
       $169 = ((($168)) + 8|0);
       $170 = HEAP8[$169>>0]|0;
       $171 = HEAP32[$pBt>>2]|0;
       $172 = ((($171)) + 20|0);
       HEAP8[$172>>0] = $170;
      }
      $173 = HEAP32[$1>>2]|0;
      $174 = ($173|0)!=(0);
      if ($174) {
       $175 = HEAP32[$pBt>>2]|0;
       $176 = ((($175)) + 12|0);
       $177 = HEAP32[$176>>2]|0;
       HEAP32[$pPage1>>2] = $177;
       $178 = HEAP32[$0>>2]|0;
       $179 = HEAP32[$pBt>>2]|0;
       $180 = ((($179)) + 76|0);
       HEAP32[$180>>2] = $178;
       $181 = HEAP32[$pBt>>2]|0;
       $182 = ((($181)) + 22|0);
       $183 = HEAP16[$182>>1]|0;
       $184 = $183&65535;
       $185 = $184 & -33;
       $186 = $185&65535;
       HEAP16[$182>>1] = $186;
       $187 = HEAP32[$1>>2]|0;
       $188 = ($187|0)>(1);
       if ($188) {
        $189 = HEAP32[$pBt>>2]|0;
        $190 = ((($189)) + 22|0);
        $191 = HEAP16[$190>>1]|0;
        $192 = $191&65535;
        $193 = $192 | 32;
        $194 = $193&65535;
        HEAP16[$190>>1] = $194;
       }
       $195 = HEAP32[$pBt>>2]|0;
       $196 = ((($195)) + 44|0);
       $197 = HEAP32[$196>>2]|0;
       $198 = HEAP32[$pPage1>>2]|0;
       $199 = ((($198)) + 56|0);
       $200 = HEAP32[$199>>2]|0;
       $201 = ((($200)) + 28|0);
       $202 = (_sqlite3Get4byte($201)|0);
       $203 = ($197|0)!=($202|0);
       if ($203) {
        $204 = HEAP32[$pPage1>>2]|0;
        $205 = ((($204)) + 72|0);
        $206 = HEAP32[$205>>2]|0;
        $207 = (_sqlite3PagerWrite($206)|0);
        HEAP32[$rc>>2] = $207;
        $208 = HEAP32[$rc>>2]|0;
        $209 = ($208|0)==(0);
        if ($209) {
         $210 = HEAP32[$pPage1>>2]|0;
         $211 = ((($210)) + 56|0);
         $212 = HEAP32[$211>>2]|0;
         $213 = ((($212)) + 28|0);
         $214 = HEAP32[$pBt>>2]|0;
         $215 = ((($214)) + 44|0);
         $216 = HEAP32[$215>>2]|0;
         _sqlite3Put4byte($213,$216);
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $217 = HEAP32[$rc>>2]|0;
 $218 = ($217|0)==(0);
 $219 = HEAP32[$1>>2]|0;
 $220 = ($219|0)!=(0);
 $or$cond9 = $218 & $220;
 if (!($or$cond9)) {
  $228 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 $221 = HEAP32[$pBt>>2]|0;
 $222 = HEAP32[$221>>2]|0;
 $223 = HEAP32[$0>>2]|0;
 $224 = HEAP32[$223>>2]|0;
 $225 = ((($224)) + 500|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = (_sqlite3PagerOpenSavepoint($222,$226)|0);
 HEAP32[$rc>>2] = $227;
 $228 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($228|0);
}
function _sqlite3BtreeGetMeta($p,$idx,$pMeta) {
 $p = $p|0;
 $idx = $idx|0;
 $pMeta = $pMeta|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $pMeta;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(15);
 if ($8) {
  $9 = HEAP32[$pBt>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (_sqlite3PagerDataVersion($10)|0);
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11) + ($14))|0;
  $16 = HEAP32[$2>>2]|0;
  HEAP32[$16>>2] = $15;
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$1>>2]|0;
  $18 = $17<<2;
  $19 = (36 + ($18))|0;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 56|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($19)|0);
  $26 = (_sqlite3Get4byte($25)|0);
  $27 = HEAP32[$2>>2]|0;
  HEAP32[$27>>2] = $26;
  STACKTOP = sp;return;
 }
}
function _sqlite3BtreeIsInReadTrans($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BtreeGetPageSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3PagerGetJournalMode($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 5|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeLastPage($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_btreePagecount($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3PagerGet($pPager,$pgno,$ppPage,$flags) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bMmapOk = 0, $iFrame = 0, $noContent = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pBase = 0, $pData = 0, $pPg = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $rc = sp + 24|0;
 $pPg = sp + 20|0;
 $iFrame = sp + 16|0;
 $noContent = sp + 12|0;
 $bMmapOk = sp + 8|0;
 $pData = sp + 4|0;
 $pBase = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pPg>>2] = 0;
 HEAP32[$iFrame>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 HEAP32[$noContent>>2] = $6;
 $7 = 0;
 HEAP32[$bMmapOk>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8>>>0)<=(1);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0);
 $or$cond = $9 & $11;
 if ($or$cond) {
  $12 = (_sqlite3CorruptError(49076)|0);
  HEAP32[$0>>2] = $12;
  $206 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($206|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0);
 do {
  if ($16) {
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 44|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$rc>>2] = $19;
   label = 23;
  } else {
   $20 = HEAP32[$bMmapOk>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = (_pagerUseWal($22)|0);
    $24 = ($23|0)!=(0);
    if ($24) {
     $25 = HEAP32[$1>>2]|0;
     $26 = ((($25)) + 216|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$2>>2]|0;
     $29 = (_sqlite3WalFindFrame($27,$28,$iFrame)|0);
     HEAP32[$rc>>2] = $29;
     $30 = HEAP32[$rc>>2]|0;
     $31 = ($30|0)!=(0);
     if ($31) {
      break;
     }
    }
   }
   $32 = HEAP32[$bMmapOk>>2]|0;
   $33 = ($32|0)!=(0);
   $34 = HEAP32[$iFrame>>2]|0;
   $35 = ($34|0)==(0);
   $or$cond3 = $33 & $35;
   if ($or$cond3) {
    HEAP32[$pData>>2] = 0;
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 64|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$2>>2]|0;
    $40 = (($39) - 1)|0;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 160|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (___muldi3(($40|0),0,($43|0),($45|0))|0);
    $47 = tempRet0;
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 160|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (_sqlite3OsFetch($38,$46,$47,$50,$pData)|0);
    HEAP32[$rc>>2] = $51;
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)==(0);
    $54 = HEAP32[$pData>>2]|0;
    $55 = ($54|0)!=(0|0);
    $or$cond5 = $53 & $55;
    if ($or$cond5) {
     $56 = HEAP32[$1>>2]|0;
     $57 = ((($56)) + 17|0);
     $58 = HEAP8[$57>>0]|0;
     $59 = $58&255;
     $60 = ($59|0)>(1);
     if ($60) {
      $61 = HEAP32[$1>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = (_sqlite3PagerLookup($61,$62)|0);
      HEAP32[$pPg>>2] = $63;
     }
     $64 = HEAP32[$pPg>>2]|0;
     $65 = ($64|0)==(0|0);
     $66 = HEAP32[$1>>2]|0;
     if ($65) {
      $67 = HEAP32[$2>>2]|0;
      $68 = HEAP32[$pData>>2]|0;
      $69 = (_pagerAcquireMapPage($66,$67,$68,$pPg)|0);
      HEAP32[$rc>>2] = $69;
     } else {
      $70 = ((($66)) + 64|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = HEAP32[$2>>2]|0;
      $73 = (($72) - 1)|0;
      $74 = HEAP32[$1>>2]|0;
      $75 = ((($74)) + 160|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = ($76|0)<(0);
      $78 = $77 << 31 >> 31;
      $79 = (___muldi3(($73|0),0,($76|0),($78|0))|0);
      $80 = tempRet0;
      $81 = HEAP32[$pData>>2]|0;
      (_sqlite3OsUnfetch($71,$79,$80,$81)|0);
     }
     $82 = HEAP32[$pPg>>2]|0;
     $83 = ($82|0)!=(0|0);
     if ($83) {
      $84 = HEAP32[$pPg>>2]|0;
      $85 = HEAP32[$3>>2]|0;
      HEAP32[$85>>2] = $84;
      HEAP32[$0>>2] = 0;
      $206 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($206|0);
     }
    }
    $86 = HEAP32[$rc>>2]|0;
    $87 = ($86|0)!=(0);
    if ($87) {
     break;
    }
   }
   $88 = HEAP32[$1>>2]|0;
   $89 = ((($88)) + 212|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$2>>2]|0;
   $92 = (_sqlite3PcacheFetch($90,$91,3)|0);
   HEAP32[$pBase>>2] = $92;
   $93 = HEAP32[$pBase>>2]|0;
   $94 = ($93|0)==(0|0);
   if ($94) {
    $95 = HEAP32[$1>>2]|0;
    $96 = ((($95)) + 212|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = HEAP32[$2>>2]|0;
    $99 = (_sqlite3PcacheFetchStress($97,$98,$pBase)|0);
    HEAP32[$rc>>2] = $99;
    $100 = HEAP32[$rc>>2]|0;
    $101 = ($100|0)!=(0);
    if ($101) {
     break;
    }
    $102 = HEAP32[$pBase>>2]|0;
    $103 = ($102|0)==(0|0);
    if ($103) {
     $104 = HEAP32[$3>>2]|0;
     HEAP32[$104>>2] = 0;
     HEAP32[$pPg>>2] = 0;
     HEAP32[$rc>>2] = 7;
     break;
    }
   }
   $105 = HEAP32[$1>>2]|0;
   $106 = ((($105)) + 212|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = HEAP32[$2>>2]|0;
   $109 = HEAP32[$pBase>>2]|0;
   $110 = (_sqlite3PcacheFetchFinish($107,$108,$109)|0);
   $111 = HEAP32[$3>>2]|0;
   HEAP32[$111>>2] = $110;
   HEAP32[$pPg>>2] = $110;
   label = 23;
  }
 } while(0);
 do {
  if ((label|0) == 23) {
   $112 = HEAP32[$rc>>2]|0;
   $113 = ($112|0)!=(0);
   if ($113) {
    HEAP32[$pPg>>2] = 0;
    break;
   }
   $114 = HEAP32[$pPg>>2]|0;
   $115 = ((($114)) + 16|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = ($116|0)==(0|0);
   $118 = HEAP32[$noContent>>2]|0;
   $119 = ($118|0)!=(0);
   $or$cond7 = $117 | $119;
   $120 = HEAP32[$1>>2]|0;
   if (!($or$cond7)) {
    $121 = ((($120)) + 192|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = (($122) + 1)|0;
    HEAP32[$121>>2] = $123;
    HEAP32[$0>>2] = 0;
    $206 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($206|0);
   }
   $124 = HEAP32[$pPg>>2]|0;
   $125 = ((($124)) + 16|0);
   HEAP32[$125>>2] = $120;
   $126 = HEAP32[$2>>2]|0;
   $127 = ($126>>>0)>(2147483647);
   if (!($127)) {
    $128 = HEAP32[$2>>2]|0;
    $129 = HEAP32[832>>2]|0;
    $130 = HEAP32[$1>>2]|0;
    $131 = ((($130)) + 160|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = (($129|0) / ($132|0))&-1;
    $134 = (($133) + 1)|0;
    $135 = ($128|0)==($134|0);
    if (!($135)) {
     $137 = HEAP32[$1>>2]|0;
     $138 = ((($137)) + 16|0);
     $139 = HEAP8[$138>>0]|0;
     $140 = $139&255;
     $141 = ($140|0)!=(0);
     if ($141) {
      label = 33;
     } else {
      $142 = HEAP32[$1>>2]|0;
      $143 = ((($142)) + 28|0);
      $144 = HEAP32[$143>>2]|0;
      $145 = HEAP32[$2>>2]|0;
      $146 = ($144>>>0)<($145>>>0);
      $147 = HEAP32[$noContent>>2]|0;
      $148 = ($147|0)!=(0);
      $or$cond9 = $146 | $148;
      if ($or$cond9) {
       label = 33;
      } else {
       $149 = HEAP32[$1>>2]|0;
       $150 = ((($149)) + 64|0);
       $151 = HEAP32[$150>>2]|0;
       $152 = HEAP32[$151>>2]|0;
       $153 = ($152|0)!=(0|0);
       if ($153) {
        $178 = HEAP32[$1>>2]|0;
        $179 = (_pagerUseWal($178)|0);
        $180 = ($179|0)!=(0);
        $181 = HEAP32[$bMmapOk>>2]|0;
        $182 = ($181|0)==(0);
        $or$cond11 = $180 & $182;
        if ($or$cond11) {
         $183 = HEAP32[$1>>2]|0;
         $184 = ((($183)) + 216|0);
         $185 = HEAP32[$184>>2]|0;
         $186 = HEAP32[$2>>2]|0;
         $187 = (_sqlite3WalFindFrame($185,$186,$iFrame)|0);
         HEAP32[$rc>>2] = $187;
         $188 = HEAP32[$rc>>2]|0;
         $189 = ($188|0)!=(0);
         if ($189) {
          break;
         }
        }
        $190 = HEAP32[$1>>2]|0;
        $191 = ((($190)) + 192|0);
        $192 = ((($191)) + 4|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = (($193) + 1)|0;
        HEAP32[$192>>2] = $194;
        $195 = HEAP32[$pPg>>2]|0;
        $196 = HEAP32[$iFrame>>2]|0;
        $197 = (_readDbPage($195,$196)|0);
        HEAP32[$rc>>2] = $197;
        $198 = HEAP32[$rc>>2]|0;
        $199 = ($198|0)!=(0);
        if ($199) {
         break;
        }
       } else {
        label = 33;
       }
      }
     }
     if ((label|0) == 33) {
      $154 = HEAP32[$2>>2]|0;
      $155 = HEAP32[$1>>2]|0;
      $156 = ((($155)) + 164|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = ($154>>>0)>($157>>>0);
      if ($158) {
       HEAP32[$rc>>2] = 13;
       break;
      }
      $159 = HEAP32[$noContent>>2]|0;
      $160 = ($159|0)!=(0);
      if ($160) {
       _sqlite3BeginBenignMalloc();
       $161 = HEAP32[$2>>2]|0;
       $162 = HEAP32[$1>>2]|0;
       $163 = ((($162)) + 32|0);
       $164 = HEAP32[$163>>2]|0;
       $165 = ($161>>>0)<=($164>>>0);
       if ($165) {
        $166 = HEAP32[$1>>2]|0;
        $167 = ((($166)) + 60|0);
        $168 = HEAP32[$167>>2]|0;
        $169 = HEAP32[$2>>2]|0;
        (_sqlite3BitvecSet($168,$169)|0);
       }
       $170 = HEAP32[$1>>2]|0;
       $171 = HEAP32[$2>>2]|0;
       (_addToSavepointBitvecs($170,$171)|0);
       _sqlite3EndBenignMalloc();
      }
      $172 = HEAP32[$pPg>>2]|0;
      $173 = ((($172)) + 4|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = HEAP32[$1>>2]|0;
      $176 = ((($175)) + 160|0);
      $177 = HEAP32[$176>>2]|0;
      _memset(($174|0),0,($177|0))|0;
     }
     HEAP32[$0>>2] = 0;
     $206 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($206|0);
    }
   }
   $136 = (_sqlite3CorruptError(49165)|0);
   HEAP32[$rc>>2] = $136;
  }
 } while(0);
 $200 = HEAP32[$pPg>>2]|0;
 $201 = ($200|0)!=(0|0);
 if ($201) {
  $202 = HEAP32[$pPg>>2]|0;
  _sqlite3PcacheDrop($202);
 }
 $203 = HEAP32[$1>>2]|0;
 _pagerUnlockIfUnused($203);
 $204 = HEAP32[$3>>2]|0;
 HEAP32[$204>>2] = 0;
 $205 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $205;
 $206 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($206|0);
}
function _backupOnePage($p,$iSrcPg,$zSrcData,$bUpdate) {
 $p = $p|0;
 $iSrcPg = $iSrcPg|0;
 $zSrcData = $zSrcData|0;
 $bUpdate = $bUpdate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iDest = 0, $iEnd = 0, $iOff = 0, $nCopy = 0, $nDestPgsz = 0, $nSrcPgsz = 0, $or$cond = 0, $pDestPager = 0, $pDestPg = 0, $rc = 0;
 var $zDestData = 0, $zIn = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $pDestPager = sp + 52|0;
 $nSrcPgsz = sp + 48|0;
 $nDestPgsz = sp + 44|0;
 $nCopy = sp + 40|0;
 $iEnd = sp + 8|0;
 $rc = sp + 36|0;
 $iOff = sp;
 $pDestPg = sp + 32|0;
 $iDest = sp + 28|0;
 $zIn = sp + 24|0;
 $zDestData = sp + 20|0;
 $zOut = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iSrcPg;
 HEAP32[$2>>2] = $zSrcData;
 HEAP32[$3>>2] = $bUpdate;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3BtreePager($6)|0);
 HEAP32[$pDestPager>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3BtreeGetPageSize($10)|0);
 HEAP32[$nSrcPgsz>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3BtreeGetPageSize($14)|0);
 HEAP32[$nDestPgsz>>2] = $15;
 $16 = HEAP32[$nSrcPgsz>>2]|0;
 $17 = HEAP32[$nDestPgsz>>2]|0;
 $18 = ($16|0)<($17|0);
 $19 = HEAP32[$nSrcPgsz>>2]|0;
 $20 = HEAP32[$nDestPgsz>>2]|0;
 $21 = $18 ? $19 : $20;
 HEAP32[$nCopy>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$nSrcPgsz>>2]|0;
 $24 = ($23|0)<(0);
 $25 = $24 << 31 >> 31;
 $26 = (___muldi3(($22|0),0,($23|0),($25|0))|0);
 $27 = tempRet0;
 $28 = $iEnd;
 $29 = $28;
 HEAP32[$29>>2] = $26;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 HEAP32[$rc>>2] = 0;
 $32 = HEAP32[$nSrcPgsz>>2]|0;
 $33 = HEAP32[$nDestPgsz>>2]|0;
 $34 = ($32|0)!=($33|0);
 if ($34) {
  $35 = HEAP32[$pDestPager>>2]|0;
  $36 = (_sqlite3PagerIsMemdb($35)|0);
  $37 = ($36|0)!=(0);
  if ($37) {
   HEAP32[$rc>>2] = 8;
  }
 }
 $38 = $iEnd;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$nSrcPgsz>>2]|0;
 $45 = ($44|0)<(0);
 $46 = $45 << 31 >> 31;
 $47 = (_i64Subtract(($40|0),($43|0),($44|0),($46|0))|0);
 $48 = tempRet0;
 $49 = $iOff;
 $50 = $49;
 HEAP32[$50>>2] = $47;
 $51 = (($49) + 4)|0;
 $52 = $51;
 HEAP32[$52>>2] = $48;
 while(1) {
  $53 = HEAP32[$rc>>2]|0;
  $54 = ($53|0)==(0);
  if (!($54)) {
   label = 14;
   break;
  }
  $55 = $iOff;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($55) + 4)|0;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = $iEnd;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($61) + 4)|0;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = ($60|0)<($66|0);
  $68 = ($57>>>0)<($63>>>0);
  $69 = ($60|0)==($66|0);
  $70 = $69 & $68;
  $71 = $67 | $70;
  if (!($71)) {
   label = 14;
   break;
  }
  HEAP32[$pDestPg>>2] = 0;
  $72 = $iOff;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$nDestPgsz>>2]|0;
  $79 = ($78|0)<(0);
  $80 = $79 << 31 >> 31;
  $81 = (___divdi3(($74|0),($77|0),($78|0),($80|0))|0);
  $82 = tempRet0;
  $83 = (($81) + 1)|0;
  HEAP32[$iDest>>2] = $83;
  $84 = HEAP32[$iDest>>2]|0;
  $85 = HEAP32[832>>2]|0;
  $86 = HEAP32[$0>>2]|0;
  $87 = ((($86)) + 4|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = ((($88)) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = ((($90)) + 32|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($85>>>0) / ($92>>>0))&-1;
  $94 = (($93) + 1)|0;
  $95 = ($84|0)==($94|0);
  if (!($95)) {
   $96 = HEAP32[$pDestPager>>2]|0;
   $97 = HEAP32[$iDest>>2]|0;
   $98 = (_sqlite3PagerGet($96,$97,$pDestPg,0)|0);
   HEAP32[$rc>>2] = $98;
   $99 = (0)==($98|0);
   if ($99) {
    $100 = HEAP32[$pDestPg>>2]|0;
    $101 = (_sqlite3PagerWrite($100)|0);
    HEAP32[$rc>>2] = $101;
    $102 = (0)==($101|0);
    if ($102) {
     $103 = $iOff;
     $104 = $103;
     $105 = HEAP32[$104>>2]|0;
     $106 = (($103) + 4)|0;
     $107 = $106;
     $108 = HEAP32[$107>>2]|0;
     $109 = HEAP32[$nSrcPgsz>>2]|0;
     $110 = ($109|0)<(0);
     $111 = $110 << 31 >> 31;
     $112 = (___remdi3(($105|0),($108|0),($109|0),($111|0))|0);
     $113 = tempRet0;
     $114 = HEAP32[$2>>2]|0;
     $115 = (($114) + ($112)|0);
     HEAP32[$zIn>>2] = $115;
     $116 = HEAP32[$pDestPg>>2]|0;
     $117 = (_sqlite3PagerGetData($116)|0);
     HEAP32[$zDestData>>2] = $117;
     $118 = $iOff;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = (($118) + 4)|0;
     $122 = $121;
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nDestPgsz>>2]|0;
     $125 = ($124|0)<(0);
     $126 = $125 << 31 >> 31;
     $127 = (___remdi3(($120|0),($123|0),($124|0),($126|0))|0);
     $128 = tempRet0;
     $129 = HEAP32[$zDestData>>2]|0;
     $130 = (($129) + ($127)|0);
     HEAP32[$zOut>>2] = $130;
     $131 = HEAP32[$zOut>>2]|0;
     $132 = HEAP32[$zIn>>2]|0;
     $133 = HEAP32[$nCopy>>2]|0;
     _memcpy(($131|0),($132|0),($133|0))|0;
     $134 = HEAP32[$pDestPg>>2]|0;
     $135 = (_sqlite3PagerGetExtra($134)|0);
     HEAP8[$135>>0] = 0;
     $136 = $iOff;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = ($138|0)==(0);
     $143 = ($141|0)==(0);
     $144 = $142 & $143;
     $145 = HEAP32[$3>>2]|0;
     $146 = ($145|0)==(0);
     $or$cond = $144 & $146;
     if ($or$cond) {
      $147 = HEAP32[$zOut>>2]|0;
      $148 = ((($147)) + 28|0);
      $149 = HEAP32[$0>>2]|0;
      $150 = ((($149)) + 24|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = (_sqlite3BtreeLastPage($151)|0);
      _sqlite3Put4byte($148,$152);
     }
    }
   }
   $153 = HEAP32[$pDestPg>>2]|0;
   _sqlite3PagerUnref($153);
  }
  $154 = HEAP32[$nDestPgsz>>2]|0;
  $155 = ($154|0)<(0);
  $156 = $155 << 31 >> 31;
  $157 = $iOff;
  $158 = $157;
  $159 = HEAP32[$158>>2]|0;
  $160 = (($157) + 4)|0;
  $161 = $160;
  $162 = HEAP32[$161>>2]|0;
  $163 = (_i64Add(($159|0),($162|0),($154|0),($156|0))|0);
  $164 = tempRet0;
  $165 = $iOff;
  $166 = $165;
  HEAP32[$166>>2] = $163;
  $167 = (($165) + 4)|0;
  $168 = $167;
  HEAP32[$168>>2] = $164;
 }
 if ((label|0) == 14) {
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 return (0)|0;
}
function _sqlite3PagerGetData($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerUnref($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _sqlite3PagerUnrefNotNull($3);
 STACKTOP = sp;return;
}
function _attachBackupObject($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3BtreePager($3)|0);
 $5 = (_sqlite3PagerBackupPtr($4)|0);
 HEAP32[$pp>>2] = $5;
 $6 = HEAP32[$pp>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 44|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$pp>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 40|0);
 HEAP32[$13>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3BtreeNewDb($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 44|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_newDatabase($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3BtreeUpdateMeta($p,$idx,$iMeta) {
 $p = $p|0;
 $idx = $idx|0;
 $iMeta = $iMeta|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pP1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pBt = sp + 8|0;
 $pP1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $iMeta;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pP1>>2] = $11;
 $12 = HEAP32[$pBt>>2]|0;
 $13 = ((($12)) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 72|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3PagerWrite($16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = $20<<2;
 $22 = (36 + ($21))|0;
 $23 = HEAP32[$pP1>>2]|0;
 $24 = (($23) + ($22)|0);
 $25 = HEAP32[$2>>2]|0;
 _sqlite3Put4byte($24,$25);
 $26 = HEAP32[$1>>2]|0;
 $27 = ($26|0)==(7);
 if (!($27)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = $28&255;
 $30 = HEAP32[$pBt>>2]|0;
 $31 = ((($30)) + 18|0);
 HEAP8[$31>>0] = $29;
 $32 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3ResetAllSchemasOfConnection($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  HEAP32[$pDb>>2] = $11;
  $12 = HEAP32[$pDb>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pDb>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   _sqlite3SchemaClear($18);
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 24|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 & -3;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($25);
 $26 = HEAP32[$0>>2]|0;
 _sqlite3CollapseDatabaseArray($26);
 STACKTOP = sp;return;
}
function _sqlite3BtreeSetVersion($pBtree,$iVersion) {
 $pBtree = $pBtree|0;
 $iVersion = $iVersion|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $aData = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pBt = sp + 8|0;
 $rc = sp + 4|0;
 $aData = sp;
 HEAP32[$0>>2] = $pBtree;
 HEAP32[$1>>2] = $iVersion;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$pBt>>2]|0;
 $6 = ((($5)) + 22|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & -17;
 $10 = $9&65535;
 HEAP16[$6>>1] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)==(1);
 if ($12) {
  $13 = HEAP32[$pBt>>2]|0;
  $14 = ((($13)) + 22|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15&65535;
  $17 = $16 | 16;
  $18 = $17&65535;
  HEAP16[$14>>1] = $18;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3BtreeBeginTrans($19,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 do {
  if ($22) {
   $23 = HEAP32[$pBt>>2]|0;
   $24 = ((($23)) + 12|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ((($25)) + 56|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$aData>>2] = $27;
   $28 = HEAP32[$aData>>2]|0;
   $29 = ((($28)) + 18|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = HEAP32[$1>>2]|0;
   $33 = $32&255;
   $34 = $33&255;
   $35 = ($31|0)!=($34|0);
   if (!($35)) {
    $36 = HEAP32[$aData>>2]|0;
    $37 = ((($36)) + 19|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = HEAP32[$1>>2]|0;
    $41 = $40&255;
    $42 = $41&255;
    $43 = ($39|0)!=($42|0);
    if (!($43)) {
     break;
    }
   }
   $44 = HEAP32[$0>>2]|0;
   $45 = (_sqlite3BtreeBeginTrans($44,2)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$pBt>>2]|0;
    $49 = ((($48)) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = ((($50)) + 72|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (_sqlite3PagerWrite($52)|0);
    HEAP32[$rc>>2] = $53;
    $54 = HEAP32[$rc>>2]|0;
    $55 = ($54|0)==(0);
    if ($55) {
     $56 = HEAP32[$1>>2]|0;
     $57 = $56&255;
     $58 = HEAP32[$aData>>2]|0;
     $59 = ((($58)) + 18|0);
     HEAP8[$59>>0] = $57;
     $60 = HEAP32[$1>>2]|0;
     $61 = $60&255;
     $62 = HEAP32[$aData>>2]|0;
     $63 = ((($62)) + 19|0);
     HEAP8[$63>>0] = $61;
    }
   }
  }
 } while(0);
 $64 = HEAP32[$pBt>>2]|0;
 $65 = ((($64)) + 22|0);
 $66 = HEAP16[$65>>1]|0;
 $67 = $66&65535;
 $68 = $67 & -17;
 $69 = $68&65535;
 HEAP16[$65>>1] = $69;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3PagerFile($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerPagecount($pPager,$pnPage) {
 $pPager = $pPager|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pnPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$5>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3PagerWrite($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pPager = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$pPager>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$0>>2] = $11;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 24|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 4;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$pPager>>2]|0;
  $19 = ((($18)) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20>>>0)>=($23>>>0);
  if ($24) {
   $25 = HEAP32[$pPager>>2]|0;
   $26 = ((($25)) + 104|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$1>>2]|0;
    $30 = (_subjournalPageIfRequired($29)|0);
    HEAP32[$0>>2] = $30;
    $41 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($41|0);
   } else {
    HEAP32[$0>>2] = 0;
    $41 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($41|0);
   }
  }
 }
 $31 = HEAP32[$pPager>>2]|0;
 $32 = ((($31)) + 156|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$pPager>>2]|0;
 $35 = ((($34)) + 160|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($33>>>0)>($36>>>0);
 $38 = HEAP32[$1>>2]|0;
 if ($37) {
  $39 = (_pagerWriteLargeSector($38)|0);
  HEAP32[$0>>2] = $39;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $40 = (_pager_write($38)|0);
  HEAP32[$0>>2] = $40;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _sqlite3PagerCommitPhaseOne($pPager,$zMaster,$noSync) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 $noSync = $noSync|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nNew = 0, $or$cond = 0, $pList = 0, $pPageOne = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pList = sp + 8|0;
 $pPageOne = sp + 4|0;
 $nNew = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$3>>2] = $noSync;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $11 = ((($8)) + 17|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)<(3);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 $19 = HEAP32[$1>>2]|0;
 do {
  if ($18) {
   $20 = ((($19)) + 96|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3BackupRestart($21);
  } else {
   $22 = (_pagerUseWal($19)|0);
   $23 = ($22|0)!=(0);
   $24 = HEAP32[$1>>2]|0;
   if ($23) {
    $25 = ((($24)) + 212|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = (_sqlite3PcacheDirtyList($26)|0);
    HEAP32[$pList>>2] = $27;
    HEAP32[$pPageOne>>2] = 0;
    $28 = HEAP32[$pList>>2]|0;
    $29 = ($28|0)==(0|0);
    if ($29) {
     $30 = HEAP32[$1>>2]|0;
     $31 = (_sqlite3PagerGet($30,1,$pPageOne,0)|0);
     HEAP32[$rc>>2] = $31;
     $32 = HEAP32[$pPageOne>>2]|0;
     HEAP32[$pList>>2] = $32;
     $33 = HEAP32[$pList>>2]|0;
     $34 = ((($33)) + 12|0);
     HEAP32[$34>>2] = 0;
    }
    $35 = HEAP32[$pList>>2]|0;
    $36 = ($35|0)!=(0|0);
    if ($36) {
     $37 = HEAP32[$1>>2]|0;
     $38 = HEAP32[$pList>>2]|0;
     $39 = HEAP32[$1>>2]|0;
     $40 = ((($39)) + 28|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (_pagerWalFrames($37,$38,$41,1)|0);
     HEAP32[$rc>>2] = $42;
    }
    $43 = HEAP32[$pPageOne>>2]|0;
    _sqlite3PagerUnref($43);
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)==(0);
    if (!($45)) {
     break;
    }
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 212|0);
    $48 = HEAP32[$47>>2]|0;
    _sqlite3PcacheCleanAll($48);
    break;
   }
   $49 = (_pager_incr_changecounter($24,0)|0);
   HEAP32[$rc>>2] = $49;
   $50 = HEAP32[$rc>>2]|0;
   $51 = ($50|0)!=(0);
   if (!($51)) {
    $52 = HEAP32[$1>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (_writeMasterJournal($52,$53)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)!=(0);
    if (!($56)) {
     $57 = HEAP32[$1>>2]|0;
     $58 = (_syncJournal($57,0)|0);
     HEAP32[$rc>>2] = $58;
     $59 = HEAP32[$rc>>2]|0;
     $60 = ($59|0)!=(0);
     if (!($60)) {
      $61 = HEAP32[$1>>2]|0;
      $62 = HEAP32[$1>>2]|0;
      $63 = ((($62)) + 212|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (_sqlite3PcacheDirtyList($64)|0);
      $66 = (_pager_write_pagelist($61,$65)|0);
      HEAP32[$rc>>2] = $66;
      $67 = HEAP32[$rc>>2]|0;
      $68 = ($67|0)!=(0);
      if (!($68)) {
       $69 = HEAP32[$1>>2]|0;
       $70 = ((($69)) + 212|0);
       $71 = HEAP32[$70>>2]|0;
       _sqlite3PcacheCleanAll($71);
       $72 = HEAP32[$1>>2]|0;
       $73 = ((($72)) + 28|0);
       $74 = HEAP32[$73>>2]|0;
       $75 = HEAP32[$1>>2]|0;
       $76 = ((($75)) + 36|0);
       $77 = HEAP32[$76>>2]|0;
       $78 = ($74>>>0)>($77>>>0);
       if ($78) {
        $79 = HEAP32[$1>>2]|0;
        $80 = ((($79)) + 28|0);
        $81 = HEAP32[$80>>2]|0;
        $82 = HEAP32[$1>>2]|0;
        $83 = ((($82)) + 28|0);
        $84 = HEAP32[$83>>2]|0;
        $85 = HEAP32[832>>2]|0;
        $86 = HEAP32[$1>>2]|0;
        $87 = ((($86)) + 160|0);
        $88 = HEAP32[$87>>2]|0;
        $89 = (($85|0) / ($88|0))&-1;
        $90 = (($89) + 1)|0;
        $91 = ($84|0)==($90|0);
        $92 = $91&1;
        $93 = (($81) - ($92))|0;
        HEAP32[$nNew>>2] = $93;
        $94 = HEAP32[$1>>2]|0;
        $95 = HEAP32[$nNew>>2]|0;
        $96 = (_pager_truncate($94,$95)|0);
        HEAP32[$rc>>2] = $96;
        $97 = HEAP32[$rc>>2]|0;
        $98 = ($97|0)!=(0);
        $99 = HEAP32[$3>>2]|0;
        $100 = ($99|0)!=(0);
        $or$cond = $98 | $100;
        if ($or$cond) {
         break;
        }
       } else {
        $$old = HEAP32[$3>>2]|0;
        $$old1 = ($$old|0)!=(0);
        if ($$old1) {
         break;
        }
       }
       $101 = HEAP32[$1>>2]|0;
       $102 = HEAP32[$2>>2]|0;
       $103 = (_sqlite3PagerSync($101,$102)|0);
       HEAP32[$rc>>2] = $103;
      }
     }
    }
   }
  }
 } while(0);
 $104 = HEAP32[$rc>>2]|0;
 $105 = ($104|0)==(0);
 if ($105) {
  $106 = HEAP32[$1>>2]|0;
  $107 = (_pagerUseWal($106)|0);
  $108 = ($107|0)!=(0);
  if (!($108)) {
   $109 = HEAP32[$1>>2]|0;
   $110 = ((($109)) + 17|0);
   HEAP8[$110>>0] = 5;
  }
 }
 $111 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $111;
 $112 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($112|0);
}
function _sqlite3OsWrite($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $id;
 HEAP32[$3>>2] = $pBuf;
 HEAP32[$4>>2] = $amt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = $5;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiiii[$13 & 255]($14,$15,$16,$19,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _backupTruncateFile($pFile,$0,$1) {
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iCurrent = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 8|0;
 $iCurrent = sp;
 $rc = sp + 16|0;
 HEAP32[$2>>2] = $pFile;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3OsFileSize($8,$iCurrent)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $12 = $iCurrent;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = $3;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($17|0)>($23|0);
 $25 = ($14>>>0)>($20>>>0);
 $26 = ($17|0)==($23|0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 if (!($28)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = $3;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (_sqlite3OsTruncate($29,$32,$35)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3PagerSync($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $zMaster;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  HEAP32[$pArg>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$pArg>>2]|0;
  $12 = (_sqlite3OsFileControl($10,21,$11)|0);
  HEAP32[$rc>>2] = $12;
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(12);
  $$ = $14 ? 0 : $12;
  HEAP32[$rc>>2] = $$;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 7|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if ($20) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 64|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 12|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = (_sqlite3OsSync($23,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PagerTruncateImage($pPager,$nPage) {
 $pPager = $pPager|0;
 $nPage = $nPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 HEAP32[$4>>2] = $2;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCommitPhaseTwo($p,$bCleanup) {
 $p = $p|0;
 $bCleanup = $bCleanup|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bCleanup;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $33 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $8 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(2);
 do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pBt>>2] = $16;
   $17 = HEAP32[$pBt>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3PagerCommitPhaseTwo($18)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)!=(0);
   $22 = HEAP32[$2>>2]|0;
   $23 = ($22|0)==(0);
   $or$cond = $21 & $23;
   if (!($or$cond)) {
    $25 = HEAP32[$1>>2]|0;
    $26 = ((($25)) + 20|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (($27) + -1)|0;
    HEAP32[$26>>2] = $28;
    $29 = HEAP32[$pBt>>2]|0;
    $30 = ((($29)) + 20|0);
    HEAP8[$30>>0] = 1;
    $31 = HEAP32[$pBt>>2]|0;
    _btreeClearHasContent($31);
    break;
   }
   $24 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $24;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
 } while(0);
 $32 = HEAP32[$1>>2]|0;
 _btreeEndTransaction($32);
 HEAP32[$0>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3BtreeCommitPhaseOne($p,$zMaster) {
 $p = $p|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pBt>>2] = $10;
  $11 = HEAP32[$1>>2]|0;
  _sqlite3BtreeEnter($11);
  $12 = HEAP32[$pBt>>2]|0;
  $13 = ((($12)) + 17|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = ($14<<24>>24)!=(0);
  if ($15) {
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_autoVacuumCommit($16)|0);
   HEAP32[$rc>>2] = $17;
   $18 = HEAP32[$rc>>2]|0;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $20;
    $35 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
  }
  $21 = HEAP32[$pBt>>2]|0;
  $22 = ((($21)) + 19|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = ($23<<24>>24)!=(0);
  if ($24) {
   $25 = HEAP32[$pBt>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pBt>>2]|0;
   $28 = ((($27)) + 44|0);
   $29 = HEAP32[$28>>2]|0;
   _sqlite3PagerTruncateImage($26,$29);
  }
  $30 = HEAP32[$pBt>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (_sqlite3PagerCommitPhaseOne($31,$32,0)|0);
  HEAP32[$rc>>2] = $33;
 }
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3_backup_finish($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pSrcDb = 0, $pp = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pp = sp + 8|0;
 $pSrcDb = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pSrcDb>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + -1)|0;
  HEAP32[$16>>2] = $18;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 40|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 24|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3BtreePager($25)|0);
  $27 = (_sqlite3PagerBackupPtr($26)|0);
  HEAP32[$pp>>2] = $27;
  while(1) {
   $28 = HEAP32[$pp>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = ($29|0)!=($30|0);
   if (!($31)) {
    break;
   }
   $32 = HEAP32[$pp>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = ((($33)) + 44|0);
   HEAP32[$pp>>2] = $34;
  }
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 44|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$pp>>2]|0;
  HEAP32[$38>>2] = $37;
 }
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 (_sqlite3BtreeRollback($41,0,0)|0);
 $42 = HEAP32[$1>>2]|0;
 $43 = ((($42)) + 28|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(101);
 if ($45) {
  $49 = 0;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 28|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $48;
 }
 HEAP32[$rc>>2] = $49;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$rc>>2]|0;
  _sqlite3Error($54,$55);
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  _sqlite3LeaveMutexAndCloseZombie($57);
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0|0);
 if ($60) {
  $61 = HEAP32[$1>>2]|0;
  _sqlite3_free($61);
 }
 $62 = HEAP32[$pSrcDb>>2]|0;
 _sqlite3LeaveMutexAndCloseZombie($62);
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3PagerBackupPtr($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 96|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3BtreeRollback($p,$tripCode,$writeOnly) {
 $p = $p|0;
 $tripCode = $tripCode|0;
 $writeOnly = $writeOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nPage = 0, $pBt = 0, $pPage1 = 0, $rc = 0, $rc2 = 0, $rc21 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rc = sp + 20|0;
 $pBt = sp + 16|0;
 $pPage1 = sp + 12|0;
 $rc2 = sp + 8|0;
 $rc21 = sp + 4|0;
 $nPage = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $tripCode;
 HEAP32[$2>>2] = $writeOnly;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$pBt>>2]|0;
  $10 = (_saveAllCursors($9,0,0)|0);
  HEAP32[$1>>2] = $10;
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   HEAP32[$2>>2] = 0;
  }
 } else {
  HEAP32[$rc>>2] = 0;
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_sqlite3BtreeTripAllCursors($15,$16,$17)|0);
  HEAP32[$rc2>>2] = $18;
  $19 = HEAP32[$rc2>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $21;
  }
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $26 = ($25|0)==(2);
 if (!($26)) {
  $52 = HEAP32[$0>>2]|0;
  _btreeEndTransaction($52);
  $53 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $27 = HEAP32[$pBt>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (_sqlite3PagerRollback($28)|0);
 HEAP32[$rc21>>2] = $29;
 $30 = HEAP32[$rc21>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$rc21>>2]|0;
  HEAP32[$rc>>2] = $32;
 }
 $33 = HEAP32[$pBt>>2]|0;
 $34 = (_btreeGetPage($33,1,$pPage1,0)|0);
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$pPage1>>2]|0;
  $37 = ((($36)) + 56|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ((($38)) + 28|0);
  $40 = (_sqlite3Get4byte($39)|0);
  HEAP32[$nPage>>2] = $40;
  $41 = HEAP32[$nPage>>2]|0;
  $42 = ($41|0)==(0);
  if ($42) {
   $43 = HEAP32[$pBt>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   _sqlite3PagerPagecount($44,$nPage);
  }
  $45 = HEAP32[$nPage>>2]|0;
  $46 = HEAP32[$pBt>>2]|0;
  $47 = ((($46)) + 44|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP32[$pPage1>>2]|0;
  _releasePage($48);
 }
 $49 = HEAP32[$pBt>>2]|0;
 $50 = ((($49)) + 20|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$pBt>>2]|0;
 _btreeClearHasContent($51);
 $52 = HEAP32[$0>>2]|0;
 _btreeEndTransaction($52);
 $53 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _sqlite3LeaveMutexAndCloseZombie($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $j = 0, $p = 0, $pColl = 0, $pDb = 0, $pHash = 0, $pMod = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $pDb = sp + 20|0;
 $pNext = sp + 16|0;
 $pHash = sp + 12|0;
 $p = sp + 8|0;
 $pColl = sp + 4|0;
 $pMod = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 76|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(1691352191);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (_connectionIsBusy($5)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 _sqlite3RollbackAll($8,0);
 $9 = HEAP32[$0>>2]|0;
 _sqlite3CloseSavepoints($9);
 HEAP32[$j>>2] = 0;
 while(1) {
  $10 = HEAP32[$j>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$j>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  HEAP32[$pDb>>2] = $19;
  $20 = HEAP32[$pDb>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$pDb>>2]|0;
   $25 = ((($24)) + 4|0);
   $26 = HEAP32[$25>>2]|0;
   (_sqlite3BtreeClose($26)|0);
   $27 = HEAP32[$pDb>>2]|0;
   $28 = ((($27)) + 4|0);
   HEAP32[$28>>2] = 0;
   $29 = HEAP32[$j>>2]|0;
   $30 = ($29|0)!=(1);
   if ($30) {
    $31 = HEAP32[$pDb>>2]|0;
    $32 = ((($31)) + 12|0);
    HEAP32[$32>>2] = 0;
   }
  }
  $33 = HEAP32[$j>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$j>>2] = $34;
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 16|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ((($37)) + 16|0);
 $39 = ((($38)) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ($40|0)!=(0|0);
 if ($41) {
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ((($44)) + 16|0);
  $46 = ((($45)) + 12|0);
  $47 = HEAP32[$46>>2]|0;
  _sqlite3SchemaClear($47);
 }
 $48 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($48);
 $49 = HEAP32[$0>>2]|0;
 _sqlite3CollapseDatabaseArray($49);
 HEAP32[$j>>2] = 0;
 while(1) {
  $50 = HEAP32[$j>>2]|0;
  $51 = ($50|0)<(23);
  if (!($51)) {
   break;
  }
  $52 = HEAP32[$j>>2]|0;
  $53 = HEAP32[$0>>2]|0;
  $54 = ((($53)) + 340|0);
  $55 = (($54) + ($52<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  HEAP32[$p>>2] = $56;
  while(1) {
   $57 = HEAP32[$p>>2]|0;
   $58 = ($57|0)!=(0|0);
   if (!($58)) {
    break;
   }
   $59 = HEAP32[$p>>2]|0;
   $60 = ((($59)) + 24|0);
   $61 = HEAP32[$60>>2]|0;
   HEAP32[$pHash>>2] = $61;
   while(1) {
    $62 = HEAP32[$p>>2]|0;
    $63 = ($62|0)!=(0|0);
    if (!($63)) {
     break;
    }
    $64 = HEAP32[$0>>2]|0;
    $65 = HEAP32[$p>>2]|0;
    _functionDestroy($64,$65);
    $66 = HEAP32[$p>>2]|0;
    $67 = ((($66)) + 8|0);
    $68 = HEAP32[$67>>2]|0;
    HEAP32[$pNext>>2] = $68;
    $69 = HEAP32[$0>>2]|0;
    $70 = HEAP32[$p>>2]|0;
    _sqlite3DbFree($69,$70);
    $71 = HEAP32[$pNext>>2]|0;
    HEAP32[$p>>2] = $71;
   }
   $72 = HEAP32[$pHash>>2]|0;
   HEAP32[$p>>2] = $72;
  }
  $73 = HEAP32[$j>>2]|0;
  $74 = (($73) + 1)|0;
  HEAP32[$j>>2] = $74;
 }
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 432|0);
 $77 = ((($76)) + 8|0);
 $78 = HEAP32[$77>>2]|0;
 HEAP32[$i>>2] = $78;
 while(1) {
  $79 = HEAP32[$i>>2]|0;
  $80 = ($79|0)!=(0|0);
  if (!($80)) {
   break;
  }
  $81 = HEAP32[$i>>2]|0;
  $82 = ((($81)) + 8|0);
  $83 = HEAP32[$82>>2]|0;
  HEAP32[$pColl>>2] = $83;
  HEAP32[$j>>2] = 0;
  while(1) {
   $84 = HEAP32[$j>>2]|0;
   $85 = ($84|0)<(3);
   if (!($85)) {
    break;
   }
   $86 = HEAP32[$j>>2]|0;
   $87 = HEAP32[$pColl>>2]|0;
   $88 = (($87) + (($86*20)|0)|0);
   $89 = ((($88)) + 16|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = ($90|0)!=(0|0);
   if ($91) {
    $92 = HEAP32[$j>>2]|0;
    $93 = HEAP32[$pColl>>2]|0;
    $94 = (($93) + (($92*20)|0)|0);
    $95 = ((($94)) + 16|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$j>>2]|0;
    $98 = HEAP32[$pColl>>2]|0;
    $99 = (($98) + (($97*20)|0)|0);
    $100 = ((($99)) + 8|0);
    $101 = HEAP32[$100>>2]|0;
    FUNCTION_TABLE_vi[$96 & 255]($101);
   }
   $102 = HEAP32[$j>>2]|0;
   $103 = (($102) + 1)|0;
   HEAP32[$j>>2] = $103;
  }
  $104 = HEAP32[$0>>2]|0;
  $105 = HEAP32[$pColl>>2]|0;
  _sqlite3DbFree($104,$105);
  $106 = HEAP32[$i>>2]|0;
  $107 = HEAP32[$106>>2]|0;
  HEAP32[$i>>2] = $107;
 }
 $108 = HEAP32[$0>>2]|0;
 $109 = ((($108)) + 432|0);
 _sqlite3HashClear($109);
 $110 = HEAP32[$0>>2]|0;
 $111 = ((($110)) + 312|0);
 $112 = ((($111)) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 HEAP32[$i>>2] = $113;
 while(1) {
  $114 = HEAP32[$i>>2]|0;
  $115 = ($114|0)!=(0|0);
  if (!($115)) {
   break;
  }
  $116 = HEAP32[$i>>2]|0;
  $117 = ((($116)) + 8|0);
  $118 = HEAP32[$117>>2]|0;
  HEAP32[$pMod>>2] = $118;
  $119 = HEAP32[$pMod>>2]|0;
  $120 = ((($119)) + 12|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($121|0)!=(0|0);
  if ($122) {
   $123 = HEAP32[$pMod>>2]|0;
   $124 = ((($123)) + 12|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = HEAP32[$pMod>>2]|0;
   $127 = ((($126)) + 8|0);
   $128 = HEAP32[$127>>2]|0;
   FUNCTION_TABLE_vi[$125 & 255]($128);
  }
  $129 = HEAP32[$0>>2]|0;
  $130 = HEAP32[$pMod>>2]|0;
  _sqlite3VtabEponymousTableClear($129,$130);
  $131 = HEAP32[$0>>2]|0;
  $132 = HEAP32[$pMod>>2]|0;
  _sqlite3DbFree($131,$132);
  $133 = HEAP32[$i>>2]|0;
  $134 = HEAP32[$133>>2]|0;
  HEAP32[$i>>2] = $134;
 }
 $135 = HEAP32[$0>>2]|0;
 $136 = ((($135)) + 312|0);
 _sqlite3HashClear($136);
 $137 = HEAP32[$0>>2]|0;
 _sqlite3Error($137,0);
 $138 = HEAP32[$0>>2]|0;
 $139 = ((($138)) + 236|0);
 $140 = HEAP32[$139>>2]|0;
 _sqlite3ValueFree($140);
 $141 = HEAP32[$0>>2]|0;
 $142 = ((($141)) + 76|0);
 HEAP32[$142>>2] = -1254786768;
 $143 = HEAP32[$0>>2]|0;
 $144 = HEAP32[$0>>2]|0;
 $145 = ((($144)) + 16|0);
 $146 = HEAP32[$145>>2]|0;
 $147 = ((($146)) + 16|0);
 $148 = ((($147)) + 12|0);
 $149 = HEAP32[$148>>2]|0;
 _sqlite3DbFree($143,$149);
 $150 = HEAP32[$0>>2]|0;
 $151 = ((($150)) + 76|0);
 HEAP32[$151>>2] = -1623446221;
 $152 = HEAP32[$0>>2]|0;
 $153 = ((($152)) + 248|0);
 $154 = ((($153)) + 6|0);
 $155 = HEAP8[$154>>0]|0;
 $156 = ($155<<24>>24)!=(0);
 if ($156) {
  $157 = HEAP32[$0>>2]|0;
  $158 = ((($157)) + 248|0);
  $159 = ((($158)) + 32|0);
  $160 = HEAP32[$159>>2]|0;
  _sqlite3_free($160);
 }
 $161 = HEAP32[$0>>2]|0;
 _sqlite3_free($161);
 STACKTOP = sp;return;
}
function _sqlite3_sql($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 168|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_finalize($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pStmt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$rc>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  HEAP32[$v>>2] = $4;
  $5 = HEAP32[$v>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$db>>2] = $6;
  $7 = HEAP32[$v>>2]|0;
  $8 = (_vdbeSafety($7)|0);
  $9 = ($8|0)!=(0);
  if ($9) {
   $10 = (_sqlite3MisuseError(72362)|0);
   HEAP32[$0>>2] = $10;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
  $11 = HEAP32[$v>>2]|0;
  $12 = ((($11)) + 128|0);
  $13 = $12;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)>(0);
  $20 = ($15>>>0)>(0);
  $21 = ($18|0)==(0);
  $22 = $21 & $20;
  $23 = $19 | $22;
  if ($23) {
   $24 = HEAP32[$db>>2]|0;
   $25 = HEAP32[$v>>2]|0;
   _invokeProfileCallback($24,$25);
  }
  $26 = HEAP32[$v>>2]|0;
  $27 = (_sqlite3VdbeFinalize($26)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$db>>2]|0;
  $29 = HEAP32[$rc>>2]|0;
  $30 = (_sqlite3ApiExit($28,$29)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$db>>2]|0;
  _sqlite3LeaveMutexAndCloseZombie($31);
 }
 $32 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $32;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeSafety($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  _sqlite3_log(21,19088,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $5 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($5|0);
 } else {
  HEAP32[$0>>2] = 0;
  $5 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($5|0);
 }
 return (0)|0;
}
function _invokeProfileCallback($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iNow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $iNow = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 (_sqlite3OsCurrentTimeInt64($3,$iNow)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 184|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 188|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 168|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $iNow;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 128|0);
 $21 = $20;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (_i64Subtract(($15|0),($18|0),($23|0),($26|0))|0);
 $28 = tempRet0;
 $29 = (___muldi3(($27|0),($28|0),1000000,0)|0);
 $30 = tempRet0;
 FUNCTION_TABLE_viiii[$6 & 255]($9,$12,$29,$30);
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 128|0);
 $33 = $32;
 $34 = $33;
 HEAP32[$34>>2] = 0;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeFinalize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(-1108210269);
 if ($4) {
  label = 3;
 } else {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 40|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(1369188723);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (_sqlite3VdbeReset($9)|0);
  HEAP32[$rc>>2] = $10;
 }
 $11 = HEAP32[$0>>2]|0;
 _sqlite3VdbeDelete($11);
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3ApiExit($db,$rc) {
 $db = $db|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $rc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 65|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(3082);
 $or$cond = $7 | $9;
 if ($or$cond) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_apiOomError($10)|0);
  HEAP32[$0>>2] = $11;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 } else {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 56|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $12 & $15;
  HEAP32[$0>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3_reset($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  HEAP32[$rc>>2] = 0;
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ((($6)) + 128|0);
 $8 = $7;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)>(0);
 $15 = ($10>>>0)>(0);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if ($18) {
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP32[$v>>2]|0;
  _invokeProfileCallback($19,$20);
 }
 $21 = HEAP32[$v>>2]|0;
 $22 = (_sqlite3VdbeReset($21)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$v>>2]|0;
 _sqlite3VdbeRewind($23);
 $24 = HEAP32[$db>>2]|0;
 $25 = HEAP32[$rc>>2]|0;
 $26 = (_sqlite3ApiExit($24,$25)|0);
 HEAP32[$rc>>2] = $26;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3VdbeReset($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeHalt($3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>=(0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  (_sqlite3VdbeTransferError($8)|0);
  $9 = HEAP32[$db>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3DbFree($9,$12);
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 44|0);
  HEAP32[$14>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 88|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = ($17&65535) >>> 4;
  $19 = $18 & 1;
  $20 = $19&65535;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 88|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24 & -9;
   $26 = $25 | 8;
   HEAP16[$23>>1] = $26;
  }
 } else {
  $27 = ((($8)) + 80|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = ((($30)) + 88|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = ($32&65535) >>> 3;
   $34 = $33 & 1;
   $35 = $34&65535;
   $36 = ($35|0)!=(0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    $38 = HEAP32[$0>>2]|0;
    $39 = ((($38)) + 80|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$0>>2]|0;
    $42 = ((($41)) + 44|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)!=(0|0);
    $45 = $44 ? 17521 : 0;
    $46 = HEAP32[$0>>2]|0;
    $47 = ((($46)) + 44|0);
    $48 = HEAP32[$47>>2]|0;
    HEAP32[$vararg_buffer>>2] = $48;
    _sqlite3ErrorWithMsg($37,$40,$45,$vararg_buffer);
    $49 = HEAP32[$db>>2]|0;
    $50 = HEAP32[$0>>2]|0;
    $51 = ((($50)) + 44|0);
    $52 = HEAP32[$51>>2]|0;
    _sqlite3DbFree($49,$52);
    $53 = HEAP32[$0>>2]|0;
    $54 = ((($53)) + 44|0);
    HEAP32[$54>>2] = 0;
   }
  }
 }
 $55 = HEAP32[$0>>2]|0;
 _Cleanup($55);
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 136|0);
 $58 = $57;
 $59 = $58;
 HEAP32[$59>>2] = 0;
 $60 = (($58) + 4)|0;
 $61 = $60;
 HEAP32[$61>>2] = 0;
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 40|0);
 HEAP32[$63>>2] = 649915045;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 80|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$db>>2]|0;
 $68 = ((($67)) + 56|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $66 & $69;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3VdbeRewind($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 HEAP32[$2>>2] = -1108210269;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 76|0);
 HEAP32[$4>>2] = -1;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 80|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 86|0);
 HEAP8[$8>>0] = 2;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 92|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 72|0);
 HEAP32[$12>>2] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 87|0);
 HEAP8[$14>>0] = -1;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 104|0);
 HEAP32[$16>>2] = 0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 144|0);
 $19 = $18;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3_clear_bindings($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$p>>2]|0;
  $4 = ((($3)) + 68|0);
  $5 = HEAP16[$4>>1]|0;
  $6 = $5 << 16 >> 16;
  $7 = ($2|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 60|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*40)|0)|0);
  _sqlite3VdbeMemRelease($12);
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 60|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*40)|0)|0);
  $18 = ((($17)) + 8|0);
  HEAP16[$18>>1] = 1;
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 88|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = ($23&65535) >>> 8;
 $25 = $24 & 1;
 $26 = $25&65535;
 $27 = ($26|0)!=(0);
 if (!($27)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $28 = HEAP32[$p>>2]|0;
 $29 = ((($28)) + 188|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0);
 if (!($31)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 88|0);
 $34 = HEAP16[$33>>1]|0;
 $35 = $34 & -9;
 $36 = $35 | 8;
 HEAP16[$33>>1] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3VdbeMemRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 24|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   STACKTOP = sp;return;
  }
 }
 $11 = HEAP32[$0>>2]|0;
 _vdbeMemClear($11);
 STACKTOP = sp;return;
}
function _sqlite3_value_blob($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pVal;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 18;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (_sqlite3_value_text($26)|0);
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = (_sqlite3VdbeMemExpandBlob($9)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 | 16;
 $17 = $16&65535;
 HEAP16[$13>>1] = $17;
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24;
 } else {
  $25 = 0;
 }
 HEAP32[$0>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3VdbeMemExpandBlob($pMem) {
 $pMem = $pMem|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16384;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 12|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = (($10) + ($12))|0;
   HEAP32[$nByte>>2] = $13;
   $14 = HEAP32[$nByte>>2]|0;
   $15 = ($14|0)<=(0);
   $$ = $15 ? 1 : $13;
   HEAP32[$nByte>>2] = $$;
   $16 = HEAP32[$1>>2]|0;
   $17 = HEAP32[$nByte>>2]|0;
   $18 = (_sqlite3VdbeMemGrow($16,$17,1)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = ((($20)) + 12|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + ($22)|0);
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP32[$27>>2]|0;
    _memset(($26|0),0,($28|0))|0;
    $29 = HEAP32[$1>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + ($30))|0;
    HEAP32[$32>>2] = $34;
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 8|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = $37&65535;
    $39 = $38 & -16897;
    $40 = $39&65535;
    HEAP16[$36>>1] = $40;
    break;
   }
   HEAP32[$0>>2] = 7;
   $41 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3_value_text($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ValueText($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_value_bytes($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ValueBytes($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ValueBytes($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $p = sp;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 2;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 10|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = HEAP8[$2>>0]|0;
  $15 = $14&255;
  $16 = ($13|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$0>>2] = $19;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 16;
 $25 = ($24|0)!=(0);
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 8|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 if ($25) {
  $30 = $29 & 16384;
  $31 = ($30|0)!=(0);
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  if ($31) {
   $35 = HEAP32[$p>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = (($34) + ($36))|0;
   HEAP32[$0>>2] = $37;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  } else {
   HEAP32[$0>>2] = $34;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
 } else {
  $38 = $29 & 1;
  $39 = ($38|0)!=(0);
  if ($39) {
   HEAP32[$0>>2] = 0;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  } else {
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP8[$2>>0]|0;
   $42 = (_valueBytes($40,$41)|0);
   HEAP32[$0>>2] = $42;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
 }
 return (0)|0;
}
function _sqlite3_value_double($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (+_sqlite3VdbeRealValue($1));
 STACKTOP = sp;return (+$2);
}
function _sqlite3VdbeRealValue($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0.0, $val = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $val = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 8;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = +HEAPF64[$8>>3];
  HEAPF64[$0>>3] = $9;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 $10 = ((($8)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 4;
 $14 = ($13|0)!=(0);
 $15 = HEAP32[$1>>2]|0;
 if ($14) {
  $16 = $15;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (+($18>>>0)) + (4294967296.0*(+($21|0)));
  HEAPF64[$0>>3] = $22;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 $23 = ((($15)) + 8|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 & 18;
 $27 = ($26|0)!=(0);
 if ($27) {
  HEAPF64[$val>>3] = 0.0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 10|0);
  $36 = HEAP8[$35>>0]|0;
  (_sqlite3AtoF($30,$val,$33,$36)|0);
  $37 = +HEAPF64[$val>>3];
  HEAPF64[$0>>3] = $37;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 } else {
  HEAPF64[$0>>3] = 0.0;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 return +(0.0);
}
function _sqlite3_value_int($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3VdbeIntValue($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 20|0;
 $flags = sp + 16|0;
 $value = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 HEAP32[$flags>>2] = $5;
 $6 = HEAP32[$flags>>2]|0;
 $7 = $6 & 4;
 $8 = ($7|0)!=(0);
 do {
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = $9;
   $11 = $10;
   $12 = HEAP32[$11>>2]|0;
   $13 = (($10) + 4)|0;
   $14 = $13;
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $16;
   HEAP32[$17>>2] = $12;
   $18 = (($16) + 4)|0;
   $19 = $18;
   HEAP32[$19>>2] = $15;
  } else {
   $20 = HEAP32[$flags>>2]|0;
   $21 = $20 & 8;
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = +HEAPF64[$23>>3];
    $25 = (_doubleToInt64($24)|0);
    $26 = tempRet0;
    $27 = $0;
    $28 = $27;
    HEAP32[$28>>2] = $25;
    $29 = (($27) + 4)|0;
    $30 = $29;
    HEAP32[$30>>2] = $26;
    break;
   }
   $31 = HEAP32[$flags>>2]|0;
   $32 = $31 & 18;
   $33 = ($32|0)!=(0);
   if ($33) {
    $34 = $value;
    $35 = $34;
    HEAP32[$35>>2] = 0;
    $36 = (($34) + 4)|0;
    $37 = $36;
    HEAP32[$37>>2] = 0;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 16|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$1>>2]|0;
    $45 = ((($44)) + 10|0);
    $46 = HEAP8[$45>>0]|0;
    (_sqlite3Atoi64($40,$value,$43,$46)|0);
    $47 = $value;
    $48 = $47;
    $49 = HEAP32[$48>>2]|0;
    $50 = (($47) + 4)|0;
    $51 = $50;
    $52 = HEAP32[$51>>2]|0;
    $53 = $0;
    $54 = $53;
    HEAP32[$54>>2] = $49;
    $55 = (($53) + 4)|0;
    $56 = $55;
    HEAP32[$56>>2] = $52;
    break;
   } else {
    $57 = $0;
    $58 = $57;
    HEAP32[$58>>2] = 0;
    $59 = (($57) + 4)|0;
    $60 = $59;
    HEAP32[$60>>2] = 0;
    break;
   }
  }
 } while(0);
 $61 = $0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 tempRet0 = ($66);
 STACKTOP = sp;return ($63|0);
}
function _sqlite3_value_int64($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 tempRet0 = ($3);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ValueText($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$1>>2]|0;
   $6 = ((($5)) + 8|0);
   $7 = HEAP16[$6>>1]|0;
   $8 = $7&65535;
   $9 = $8 & 514;
   $10 = ($9|0)==(514);
   if ($10) {
    $11 = HEAP32[$1>>2]|0;
    $12 = ((($11)) + 10|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13&255;
    $15 = HEAP8[$2>>0]|0;
    $16 = $15&255;
    $17 = ($14|0)==($16|0);
    if ($17) {
     $18 = HEAP32[$1>>2]|0;
     $19 = ((($18)) + 16|0);
     $20 = HEAP32[$19>>2]|0;
     HEAP32[$0>>2] = $20;
     break;
    }
   }
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 8|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   $25 = $24 & 1;
   $26 = ($25|0)!=(0);
   if ($26) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP8[$2>>0]|0;
    $29 = (_valueToText($27,$28)|0);
    HEAP32[$0>>2] = $29;
    break;
   }
  } else {
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3_value_type($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 31;
 $6 = (17268 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeMemMakeWriteable($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $f = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $f = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16384;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemExpandBlob($8)|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 HEAP32[$f>>2] = $12;
 $13 = HEAP32[$f>>2]|0;
 $14 = $13 & 18;
 $15 = ($14|0)!=(0);
 do {
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 24|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)==(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = ((($20)) + 16|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 20|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)!=($25|0);
    if (!($26)) {
     break;
    }
   }
   $27 = HEAP32[$1>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + 2)|0;
   $32 = (_sqlite3VdbeMemGrow($27,$31,1)|0);
   $33 = ($32|0)!=(0);
   if (!($33)) {
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 12|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$1>>2]|0;
    $38 = ((($37)) + 16|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + ($36)|0);
    HEAP8[$40>>0] = 0;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = (($43) + 1)|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = ((($45)) + 16|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = (($47) + ($44)|0);
    HEAP8[$48>>0] = 0;
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 8|0);
    $51 = HEAP16[$50>>1]|0;
    $52 = $51&65535;
    $53 = $52 | 512;
    $54 = $53&65535;
    HEAP16[$50>>1] = $54;
    break;
   }
   HEAP32[$0>>2] = 7;
   $61 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($61|0);
  }
 } while(0);
 $55 = HEAP32[$1>>2]|0;
 $56 = ((($55)) + 8|0);
 $57 = HEAP16[$56>>1]|0;
 $58 = $57&65535;
 $59 = $58 & -4097;
 $60 = $59&65535;
 HEAP16[$56>>1] = $60;
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3ValueFree($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($6,$7);
 STACKTOP = sp;return;
}
function _sqlite3_result_blob($pCtx,$z,$n,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $xDel;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 _setResultStrOrError($4,$5,$6,0,$7);
 STACKTOP = sp;return;
}
function _setResultStrOrError($pCtx,$z,$n,$enc,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $4 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP8[$3>>0]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_sqlite3VdbeMemSetStr($6,$7,$8,$9,$10)|0);
 $12 = ($11|0)==(18);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 _sqlite3_result_error_toobig($13);
 STACKTOP = sp;return;
}
function _sqlite3_result_blob64($pCtx,$z,$0,$1,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 $xDel = $xDel|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $z;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $xDel;
 $10 = $4;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15>>>0)>(0);
 $17 = ($12>>>0)>(2147483647);
 $18 = ($15|0)==(0);
 $19 = $18 & $17;
 $20 = $16 | $19;
 if ($20) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$5>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  (_invokeValueDestructor($21,$22,$23)|0);
  STACKTOP = sp;return;
 } else {
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = $4;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  _setResultStrOrError($24,$25,$28,0,$32);
  STACKTOP = sp;return;
 }
}
function _invokeValueDestructor($p,$xDel,$pCtx) {
 $p = $p|0;
 $xDel = $xDel|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $xDel;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==((-1)|0);
 $or$cond = $4 | $6;
 if (!($or$cond)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$7 & 255]($8);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return 18;
 }
 $11 = HEAP32[$2>>2]|0;
 _sqlite3_result_error_toobig($11);
 STACKTOP = sp;return 18;
}
function _sqlite3_result_double($pCtx,$rVal) {
 $pCtx = $pCtx|0;
 $rVal = +$rVal;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAPF64[$1>>3] = $rVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = +HEAPF64[$1>>3];
 _sqlite3VdbeMemSetDouble($3,$4);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetDouble($pMem,$val) {
 $pMem = $pMem|0;
 $val = +$val;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0.0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMem;
 HEAPF64[$1>>3] = $val;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 $3 = +HEAPF64[$1>>3];
 $4 = (_sqlite3IsNaN($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = +HEAPF64[$1>>3];
 $7 = HEAP32[$0>>2]|0;
 HEAPF64[$7>>3] = $6;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP16[$9>>1] = 8;
 STACKTOP = sp;return;
}
function _sqlite3_result_error($pCtx,$z,$n) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = 1;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeMemSetStr($8,$9,$10,1,(-1))|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetStr($pMem,$z,$n,$enc,$xDel) {
 $pMem = $pMem|0;
 $z = $z|0;
 $n = $n|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flags = 0;
 var $iLimit = 0, $nAlloc = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 34|0;
 $5 = sp + 12|0;
 $nByte = sp + 8|0;
 $iLimit = sp + 4|0;
 $flags = sp + 32|0;
 $nAlloc = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP8[$4>>0] = $enc;
 HEAP32[$5>>2] = $xDel;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$nByte>>2] = $6;
 HEAP16[$flags>>1] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 if (!($8)) {
  _sqlite3VdbeMemSetNull($9);
  HEAP32[$0>>2] = 0;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 }
 $10 = ((($9)) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 88|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$iLimit>>2] = $17;
 } else {
  HEAP32[$iLimit>>2] = 1000000000;
 }
 $18 = HEAP8[$4>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(0);
 $21 = $20 ? 16 : 2;
 $22 = $21&65535;
 HEAP16[$flags>>1] = $22;
 $23 = HEAP32[$nByte>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP8[$4>>0]|0;
  $26 = $25&255;
  $27 = ($26|0)==(1);
  L11: do {
   if ($27) {
    $28 = HEAP32[$2>>2]|0;
    $29 = (_sqlite3Strlen30($28)|0);
    HEAP32[$nByte>>2] = $29;
    $30 = HEAP32[$nByte>>2]|0;
    $31 = HEAP32[$iLimit>>2]|0;
    $32 = ($30|0)>($31|0);
    if ($32) {
     $33 = HEAP32[$iLimit>>2]|0;
     $34 = (($33) + 1)|0;
     HEAP32[$nByte>>2] = $34;
    }
   } else {
    HEAP32[$nByte>>2] = 0;
    while(1) {
     $35 = HEAP32[$nByte>>2]|0;
     $36 = HEAP32[$iLimit>>2]|0;
     $37 = ($35|0)<=($36|0);
     if (!($37)) {
      break L11;
     }
     $38 = HEAP32[$nByte>>2]|0;
     $39 = HEAP32[$2>>2]|0;
     $40 = (($39) + ($38)|0);
     $41 = HEAP8[$40>>0]|0;
     $42 = $41 << 24 >> 24;
     $43 = HEAP32[$nByte>>2]|0;
     $44 = (($43) + 1)|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = (($45) + ($44)|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = $47 << 24 >> 24;
     $49 = $42 | $48;
     $50 = ($49|0)!=(0);
     if (!($50)) {
      break L11;
     }
     $51 = HEAP32[$nByte>>2]|0;
     $52 = (($51) + 2)|0;
     HEAP32[$nByte>>2] = $52;
    }
   }
  } while(0);
  $53 = HEAP16[$flags>>1]|0;
  $54 = $53&65535;
  $55 = $54 | 512;
  $56 = $55&65535;
  HEAP16[$flags>>1] = $56;
 }
 $57 = HEAP32[$5>>2]|0;
 $58 = ($57|0)==((-1)|0);
 do {
  if ($58) {
   $59 = HEAP32[$nByte>>2]|0;
   HEAP32[$nAlloc>>2] = $59;
   $60 = HEAP16[$flags>>1]|0;
   $61 = $60&65535;
   $62 = $61 & 512;
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = HEAP8[$4>>0]|0;
    $65 = $64&255;
    $66 = ($65|0)==(1);
    $67 = $66 ? 1 : 2;
    $68 = HEAP32[$nAlloc>>2]|0;
    $69 = (($68) + ($67))|0;
    HEAP32[$nAlloc>>2] = $69;
   }
   $70 = HEAP32[$nByte>>2]|0;
   $71 = HEAP32[$iLimit>>2]|0;
   $72 = ($70|0)>($71|0);
   if ($72) {
    HEAP32[$0>>2] = 18;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
   $73 = HEAP32[$1>>2]|0;
   $74 = HEAP32[$nAlloc>>2]|0;
   $75 = ($74|0)>(32);
   $76 = HEAP32[$nAlloc>>2]|0;
   $77 = $75 ? $76 : 32;
   $78 = (_sqlite3VdbeMemClearAndResize($73,$77)|0);
   $79 = ($78|0)!=(0);
   if (!($79)) {
    $80 = HEAP32[$1>>2]|0;
    $81 = ((($80)) + 16|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$2>>2]|0;
    $84 = HEAP32[$nAlloc>>2]|0;
    _memcpy(($82|0),($83|0),($84|0))|0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  } else {
   $85 = HEAP32[$5>>2]|0;
   $86 = ($85|0)==(169|0);
   $87 = HEAP32[$1>>2]|0;
   _sqlite3VdbeMemRelease($87);
   $88 = HEAP32[$2>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = ((($89)) + 16|0);
   HEAP32[$90>>2] = $88;
   if ($86) {
    $91 = HEAP32[$1>>2]|0;
    $92 = ((($91)) + 20|0);
    HEAP32[$92>>2] = $88;
    $93 = HEAP32[$1>>2]|0;
    $94 = ((($93)) + 32|0);
    $95 = HEAP32[$94>>2]|0;
    $96 = HEAP32[$1>>2]|0;
    $97 = ((($96)) + 20|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = (_sqlite3DbMallocSize($95,$98)|0);
    $100 = HEAP32[$1>>2]|0;
    $101 = ((($100)) + 24|0);
    HEAP32[$101>>2] = $99;
    break;
   } else {
    $102 = HEAP32[$5>>2]|0;
    $103 = HEAP32[$1>>2]|0;
    $104 = ((($103)) + 36|0);
    HEAP32[$104>>2] = $102;
    $105 = HEAP32[$5>>2]|0;
    $106 = ($105|0)==(0|0);
    $107 = $106 ? 2048 : 1024;
    $108 = HEAP16[$flags>>1]|0;
    $109 = $108&65535;
    $110 = $109 | $107;
    $111 = $110&65535;
    HEAP16[$flags>>1] = $111;
    break;
   }
  }
 } while(0);
 $112 = HEAP32[$nByte>>2]|0;
 $113 = HEAP32[$1>>2]|0;
 $114 = ((($113)) + 12|0);
 HEAP32[$114>>2] = $112;
 $115 = HEAP16[$flags>>1]|0;
 $116 = HEAP32[$1>>2]|0;
 $117 = ((($116)) + 8|0);
 HEAP16[$117>>1] = $115;
 $118 = HEAP8[$4>>0]|0;
 $119 = $118&255;
 $120 = ($119|0)==(0);
 $121 = HEAP8[$4>>0]|0;
 $122 = $121&255;
 $123 = $120 ? 1 : $122;
 $124 = $123&255;
 $125 = HEAP32[$1>>2]|0;
 $126 = ((($125)) + 10|0);
 HEAP8[$126>>0] = $124;
 $127 = HEAP32[$1>>2]|0;
 $128 = ((($127)) + 10|0);
 $129 = HEAP8[$128>>0]|0;
 $130 = $129&255;
 $131 = ($130|0)!=(1);
 if ($131) {
  $132 = HEAP32[$1>>2]|0;
  $133 = (_sqlite3VdbeMemHandleBom($132)|0);
  $134 = ($133|0)!=(0);
  if ($134) {
   HEAP32[$0>>2] = 7;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
 }
 $135 = HEAP32[$nByte>>2]|0;
 $136 = HEAP32[$iLimit>>2]|0;
 $137 = ($135|0)>($136|0);
 if ($137) {
  HEAP32[$0>>2] = 18;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 } else {
  HEAP32[$0>>2] = 0;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 }
 return (0)|0;
}
function _sqlite3_result_int($pCtx,$iVal) {
 $pCtx = $pCtx|0;
 $iVal = $iVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $iVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $6 = $5 << 31 >> 31;
 _sqlite3VdbeMemSetInt64($3,$4,$6);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetInt64($pMem,$0,$1) {
 $pMem = $pMem|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pMem;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 9312;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = $3;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  _vdbeReleaseAndSetInt64($14,$17,$20);
  STACKTOP = sp;return;
 } else {
  $21 = $3;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = $27;
  $29 = $28;
  HEAP32[$29>>2] = $23;
  $30 = (($28) + 4)|0;
  $31 = $30;
  HEAP32[$31>>2] = $26;
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 8|0);
  HEAP16[$33>>1] = 4;
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_int64($pCtx,$0,$1) {
 $pCtx = $pCtx|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pCtx;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 _sqlite3VdbeMemSetInt64($9,$12,$15);
 STACKTOP = sp;return;
}
function _sqlite3_result_null($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetNull($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  _vdbeMemClearExternAndSetNull($7);
  STACKTOP = sp;return;
 } else {
  $8 = ((($7)) + 8|0);
  HEAP16[$8>>1] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_text($pCtx,$z,$n,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $xDel;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 _setResultStrOrError($4,$5,$6,1,$7);
 STACKTOP = sp;return;
}
function _sqlite3_result_text64($pCtx,$z,$0,$1,$xDel,$enc) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 $xDel = $xDel|0;
 $enc = $enc|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 8|0;
 $6 = sp + 20|0;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $z;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $xDel;
 HEAP8[$6>>0] = $enc;
 $11 = HEAP8[$6>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(4);
 if ($13) {
  $14 = HEAP8[352>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)==(0);
  $17 = $16 ? 3 : 2;
  $18 = $17&255;
  HEAP8[$6>>0] = $18;
 }
 $19 = $4;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24>>>0)>(0);
 $26 = ($21>>>0)>(2147483647);
 $27 = ($24|0)==(0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  (_invokeValueDestructor($30,$31,$32)|0);
  STACKTOP = sp;return;
 } else {
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = $4;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP8[$6>>0]|0;
  $42 = HEAP32[$5>>2]|0;
  _setResultStrOrError($33,$34,$37,$41,$42);
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_value($pCtx,$pValue) {
 $pCtx = $pCtx|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pValue;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeMemCopy($3,$4)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemCopy($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 9312;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($8);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 ;HEAP32[$9>>2]=HEAP32[$10>>2]|0;HEAP32[$9+4>>2]=HEAP32[$10+4>>2]|0;HEAP32[$9+8>>2]=HEAP32[$10+8>>2]|0;HEAP32[$9+12>>2]=HEAP32[$10+12>>2]|0;HEAP32[$9+16>>2]=HEAP32[$10+16>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -1025;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 18;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26 & 2048;
 $28 = (0)==($27|0);
 if (!($28)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 8|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31&65535;
 $33 = $32 | 4096;
 $34 = $33&65535;
 HEAP16[$30>>1] = $34;
 $35 = HEAP32[$0>>2]|0;
 $36 = (_sqlite3VdbeMemMakeWriteable($35)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3VdbeMemSetZeroBlob($pMem,$n) {
 $pMem = $pMem|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $n;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 HEAP16[$4>>1] = 16400;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 12|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)<(0);
 if ($8) {
  HEAP32[$1>>2] = 0;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 10|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 16|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3_result_zeroblob64($pCtx,$0,$1) {
 $pCtx = $pCtx|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pOut = sp + 8|0;
 HEAP32[$3>>2] = $pCtx;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pOut>>2] = $10;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$pOut>>2]|0;
 $18 = ((($17)) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 88|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)<(0);
 $23 = $22 << 31 >> 31;
 $24 = ($16>>>0)>($23>>>0);
 $25 = ($13>>>0)>($21>>>0);
 $26 = ($16|0)==($23|0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 if ($28) {
  HEAP32[$2>>2] = 18;
  $37 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($37|0);
 } else {
  $29 = HEAP32[$3>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $4;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3VdbeMemSetZeroBlob($30,$33);
  HEAP32[$2>>2] = 0;
  $37 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3_result_error_code($pCtx,$errCode) {
 $pCtx = $pCtx|0;
 $errCode = $errCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $errCode;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 1;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_sqlite3ErrStr($16)|0);
 (_sqlite3VdbeMemSetStr($15,$17,-1,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3ErrStr($rc) {
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $or$cond = 0, $zErr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $zErr = sp;
 HEAP32[$0>>2] = $rc;
 HEAP32[$zErr>>2] = 39697;
 $1 = HEAP32[$0>>2]|0;
 $cond = ($1|0)==(516);
 if ($cond) {
  HEAP32[$zErr>>2] = 39711;
 } else {
  $2 = HEAP32[$0>>2]|0;
  $3 = $2 & 255;
  HEAP32[$0>>2] = $3;
  $4 = HEAP32[$0>>2]|0;
  $5 = ($4|0)>=(0);
  $6 = HEAP32[$0>>2]|0;
  $7 = ($6|0)<(27);
  $or$cond = $5 & $7;
  if ($or$cond) {
   $8 = HEAP32[$0>>2]|0;
   $9 = (6112 + ($8<<2)|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = (6112 + ($12<<2)|0);
    $14 = HEAP32[$13>>2]|0;
    HEAP32[$zErr>>2] = $14;
   }
  }
 }
 $15 = HEAP32[$zErr>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3_result_error_toobig($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 HEAP32[$2>>2] = 18;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 25|0);
 HEAP8[$4>>0] = 1;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 (_sqlite3VdbeMemSetStr($6,17300,-1,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3_result_error_nomem($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = 7;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3OomFault($10);
 STACKTOP = sp;return;
}
function _sqlite3OomFault($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 65|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 66|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 65|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 160|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)>(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 240|0);
  HEAP32[$18>>2] = 1;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 248|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$20>>2] = $22;
 STACKTOP = sp;return;
}
function _sqlite3_step($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $db = 0, $rc = 0, $rc2 = 0, $savedPc = 0, $v = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $rc2 = sp + 20|0;
 $v = sp + 16|0;
 $cnt = sp + 12|0;
 $db = sp + 8|0;
 $savedPc = sp + 4|0;
 $zErr = sp;
 HEAP32[$1>>2] = $pStmt;
 HEAP32[$rc>>2] = 0;
 HEAP32[$rc2>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$v>>2] = $2;
 HEAP32[$cnt>>2] = 0;
 $3 = HEAP32[$v>>2]|0;
 $4 = (_vdbeSafetyNotNull($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_sqlite3MisuseError(72913)|0);
  HEAP32[$0>>2] = $6;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ((($9)) + 88|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 & -513;
 HEAP16[$10>>1] = $12;
 while(1) {
  $13 = HEAP32[$v>>2]|0;
  $14 = (_sqlite3Step($13)|0);
  HEAP32[$rc>>2] = $14;
  $15 = ($14|0)==(17);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$cnt>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$cnt>>2] = $17;
  $18 = ($16|0)<(50);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$v>>2]|0;
  $20 = ((($19)) + 76|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$savedPc>>2] = $21;
  $22 = HEAP32[$v>>2]|0;
  $23 = (_sqlite3Reprepare($22)|0);
  HEAP32[$rc>>2] = $23;
  HEAP32[$rc2>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   break;
  }
  $26 = HEAP32[$1>>2]|0;
  (_sqlite3_reset($26)|0);
  $27 = HEAP32[$savedPc>>2]|0;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   continue;
  }
  $29 = HEAP32[$v>>2]|0;
  $30 = ((($29)) + 88|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31 & -513;
  $33 = $32 | 512;
  HEAP16[$30>>1] = $33;
 }
 $34 = HEAP32[$rc2>>2]|0;
 $35 = ($34|0)!=(0);
 do {
  if ($35) {
   $36 = HEAP32[$db>>2]|0;
   $37 = ((($36)) + 236|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3_value_text($38)|0);
   HEAP32[$zErr>>2] = $39;
   $40 = HEAP32[$db>>2]|0;
   $41 = HEAP32[$v>>2]|0;
   $42 = ((($41)) + 44|0);
   $43 = HEAP32[$42>>2]|0;
   _sqlite3DbFree($40,$43);
   $44 = HEAP32[$db>>2]|0;
   $45 = ((($44)) + 65|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = ($46<<24>>24)!=(0);
   if ($47) {
    $56 = HEAP32[$v>>2]|0;
    $57 = ((($56)) + 44|0);
    HEAP32[$57>>2] = 0;
    HEAP32[$rc>>2] = 7;
    $58 = HEAP32[$v>>2]|0;
    $59 = ((($58)) + 80|0);
    HEAP32[$59>>2] = 7;
    break;
   } else {
    $48 = HEAP32[$db>>2]|0;
    $49 = HEAP32[$zErr>>2]|0;
    $50 = (_sqlite3DbStrDup($48,$49)|0);
    $51 = HEAP32[$v>>2]|0;
    $52 = ((($51)) + 44|0);
    HEAP32[$52>>2] = $50;
    $53 = HEAP32[$rc2>>2]|0;
    $54 = HEAP32[$v>>2]|0;
    $55 = ((($54)) + 80|0);
    HEAP32[$55>>2] = $53;
    break;
   }
  }
 } while(0);
 $60 = HEAP32[$db>>2]|0;
 $61 = HEAP32[$rc>>2]|0;
 $62 = (_sqlite3ApiExit($60,$61)|0);
 HEAP32[$rc>>2] = $62;
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _vdbeSafetyNotNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  _sqlite3_log(21,19316,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_vdbeSafety($4)|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 return (0)|0;
}
function _sqlite3Step($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $or$cond = 0;
 var $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(-1108210269);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  (_sqlite3_reset($6)|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 65|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 $13 = HEAP32[$1>>2]|0;
 if ($12) {
  $14 = ((($13)) + 80|0);
  HEAP32[$14>>2] = 7;
  HEAP32[$0>>2] = 7;
  $157 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($157|0);
 }
 $15 = ((($13)) + 76|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)<=(0);
 if ($17) {
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 88|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = ($20&65535) >>> 3;
  $22 = $21 & 1;
  $23 = $22&65535;
  $24 = ($23|0)!=(0);
  if ($24) {
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 80|0);
   HEAP32[$26>>2] = 17;
   HEAP32[$rc>>2] = 1;
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 76|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)<(0);
  if ($30) {
   $31 = HEAP32[$db>>2]|0;
   $32 = ((($31)) + 148|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$db>>2]|0;
    $36 = ((($35)) + 240|0);
    HEAP32[$36>>2] = 0;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = ((($37)) + 184|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)!=(0|0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = ((($41)) + 140|0);
    $43 = ((($42)) + 5|0);
    $44 = HEAP8[$43>>0]|0;
    $45 = ($44<<24>>24)!=(0);
    if (!($45)) {
     $46 = HEAP32[$1>>2]|0;
     $47 = ((($46)) + 168|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)!=(0|0);
     if ($49) {
      $50 = HEAP32[$db>>2]|0;
      $51 = HEAP32[$50>>2]|0;
      $52 = HEAP32[$1>>2]|0;
      $53 = ((($52)) + 128|0);
      (_sqlite3OsCurrentTimeInt64($51,$53)|0);
     }
    }
   }
   $54 = HEAP32[$db>>2]|0;
   $55 = ((($54)) + 148|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($56) + 1)|0;
   HEAP32[$55>>2] = $57;
   $58 = HEAP32[$1>>2]|0;
   $59 = ((($58)) + 88|0);
   $60 = HEAP16[$59>>1]|0;
   $61 = ($60&65535) >>> 6;
   $62 = $61 & 1;
   $63 = $62&65535;
   $64 = ($63|0)==(0);
   if ($64) {
    $65 = HEAP32[$db>>2]|0;
    $66 = ((($65)) + 156|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67) + 1)|0;
    HEAP32[$66>>2] = $68;
   }
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 88|0);
   $71 = HEAP16[$70>>1]|0;
   $72 = ($71&65535) >>> 7;
   $73 = $72 & 1;
   $74 = $73&65535;
   $75 = ($74|0)!=(0);
   if ($75) {
    $76 = HEAP32[$db>>2]|0;
    $77 = ((($76)) + 152|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) + 1)|0;
    HEAP32[$77>>2] = $79;
   }
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 76|0);
   HEAP32[$81>>2] = 0;
  }
  $82 = HEAP32[$1>>2]|0;
  $83 = ((($82)) + 88|0);
  $84 = HEAP16[$83>>1]|0;
  $85 = $84 & 3;
  $86 = $85&65535;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$1>>2]|0;
   $89 = (_sqlite3VdbeList($88)|0);
   HEAP32[$rc>>2] = $89;
  } else {
   $90 = HEAP32[$db>>2]|0;
   $91 = ((($90)) + 160|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = (($92) + 1)|0;
   HEAP32[$91>>2] = $93;
   $94 = HEAP32[$1>>2]|0;
   $95 = (_sqlite3VdbeExec($94)|0);
   HEAP32[$rc>>2] = $95;
   $96 = HEAP32[$db>>2]|0;
   $97 = ((($96)) + 160|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) + -1)|0;
   HEAP32[$97>>2] = $99;
  }
  $100 = HEAP32[$rc>>2]|0;
  $101 = ($100|0)!=(100);
  if ($101) {
   $102 = HEAP32[$1>>2]|0;
   $103 = ((($102)) + 128|0);
   $104 = $103;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (($104) + 4)|0;
   $108 = $107;
   $109 = HEAP32[$108>>2]|0;
   $110 = ($109|0)>(0);
   $111 = ($106>>>0)>(0);
   $112 = ($109|0)==(0);
   $113 = $112 & $111;
   $114 = $110 | $113;
   if ($114) {
    $115 = HEAP32[$db>>2]|0;
    $116 = HEAP32[$1>>2]|0;
    _invokeProfileCallback($115,$116);
   }
  }
  $117 = HEAP32[$rc>>2]|0;
  $118 = ($117|0)==(101);
  if ($118) {
   $119 = HEAP32[$db>>2]|0;
   $120 = (_doWalCallbacks($119)|0);
   $121 = HEAP32[$1>>2]|0;
   $122 = ((($121)) + 80|0);
   HEAP32[$122>>2] = $120;
   $123 = HEAP32[$1>>2]|0;
   $124 = ((($123)) + 80|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = ($125|0)!=(0);
   if ($126) {
    HEAP32[$rc>>2] = 1;
   }
  }
  $127 = HEAP32[$rc>>2]|0;
  $128 = HEAP32[$db>>2]|0;
  $129 = ((($128)) + 52|0);
  HEAP32[$129>>2] = $127;
  $130 = HEAP32[$1>>2]|0;
  $131 = HEAP32[$130>>2]|0;
  $132 = HEAP32[$1>>2]|0;
  $133 = ((($132)) + 80|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = (_sqlite3ApiExit($131,$134)|0);
  $136 = (7)==($135|0);
  if ($136) {
   $137 = HEAP32[$1>>2]|0;
   $138 = ((($137)) + 80|0);
   HEAP32[$138>>2] = 7;
  }
 }
 $139 = HEAP32[$1>>2]|0;
 $140 = ((($139)) + 88|0);
 $141 = HEAP16[$140>>1]|0;
 $142 = ($141&65535) >>> 8;
 $143 = $142 & 1;
 $144 = $143&65535;
 $145 = ($144|0)!=(0);
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)!=(100);
 $or$cond = $145 & $147;
 $148 = HEAP32[$rc>>2]|0;
 $149 = ($148|0)!=(101);
 $or$cond3 = $or$cond & $149;
 if ($or$cond3) {
  $150 = HEAP32[$1>>2]|0;
  $151 = (_sqlite3VdbeTransferError($150)|0);
  HEAP32[$rc>>2] = $151;
 }
 $152 = HEAP32[$rc>>2]|0;
 $153 = HEAP32[$db>>2]|0;
 $154 = ((($153)) + 56|0);
 $155 = HEAP32[$154>>2]|0;
 $156 = $152 & $155;
 HEAP32[$0>>2] = $156;
 $157 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($157|0);
}
function _sqlite3Reprepare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $pNew = 0, $rc = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pNew = sp + 8|0;
 $zSql = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_sql($2)|0);
 HEAP32[$zSql>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3VdbeDb($4)|0);
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$zSql>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3LockAndPrepare($6,$7,-1,0,$8,$pNew,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $16 = HEAP32[$pNew>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  _sqlite3VdbeSwap($16,$17);
  $18 = HEAP32[$pNew>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  (_sqlite3TransferBindings($18,$19)|0);
  $20 = HEAP32[$pNew>>2]|0;
  _sqlite3VdbeResetStepResult($20);
  $21 = HEAP32[$pNew>>2]|0;
  (_sqlite3VdbeFinalize($21)|0);
  HEAP32[$0>>2] = 0;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(7);
 if ($13) {
  $14 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($14);
 }
 $15 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $15;
 $22 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3DbStrDup($db,$z) {
 $db = $db|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $n = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zNew = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $z;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 $7 = (($6) + 1)|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3DbMallocRaw($8,$9,$11)|0);
 HEAP32[$zNew>>2] = $12;
 $13 = HEAP32[$zNew>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$zNew>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$n>>2]|0;
  _memcpy(($15|0),($16|0),($17|0))|0;
 }
 $18 = HEAP32[$zNew>>2]|0;
 HEAP32[$0>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3_user_data($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3_context_db_handle($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3_aggregate_context($p,$nByte) {
 $p = $p|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nByte;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 8192;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = HEAP32[$2>>2]|0;
  $13 = (_createAggContext($11,$12)|0);
  HEAP32[$0>>2] = $13;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $14 = ((($11)) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _createAggContext($p,$nByte) {
 $p = $p|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pMem = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nByte;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pMem>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$pMem>>2]|0;
 if ($6) {
  _sqlite3VdbeMemSetNull($7);
  $8 = HEAP32[$pMem>>2]|0;
  $9 = ((($8)) + 16|0);
  HEAP32[$9>>2] = 0;
 } else {
  $10 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemClearAndResize($7,$10)|0);
  $11 = HEAP32[$pMem>>2]|0;
  $12 = ((($11)) + 8|0);
  HEAP16[$12>>1] = 8192;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$pMem>>2]|0;
  HEAP32[$16>>2] = $15;
  $17 = HEAP32[$pMem>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0|0);
  if ($20) {
   $21 = HEAP32[$pMem>>2]|0;
   $22 = ((($21)) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   _memset(($23|0),0,($24|0))|0;
  }
 }
 $25 = HEAP32[$pMem>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3DbMallocZero($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $db;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3DbMallocRaw($8,$11,$14)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[$p>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $18 = HEAP32[$p>>2]|0;
 $19 = $3;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 _memset(($18|0),0,($21|0))|0;
 $25 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3_column_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pVm = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pVm>>2] = $1;
 $2 = HEAP32[$pVm>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $8 = 0;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$pVm>>2]|0;
 $5 = ((($4)) + 84|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_data_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pVm = sp;
 HEAP32[$1>>2] = $pStmt;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pVm>>2] = $2;
 $3 = HEAP32[$pVm>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = HEAP32[$pVm>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = HEAP32[$pVm>>2]|0;
   $10 = ((($9)) + 84|0);
   $11 = HEAP16[$10>>1]|0;
   $12 = $11&65535;
   HEAP32[$0>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($13|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3_column_blob($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _columnMem($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pOut = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVm = sp + 4|0;
 $pOut = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pVm>>2] = $2;
 $3 = HEAP32[$pVm>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$pVm>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$pVm>>2]|0;
   $11 = ((($10)) + 84|0);
   $12 = HEAP16[$11>>1]|0;
   $13 = $12&65535;
   $14 = ($9|0)<($13|0);
   $15 = HEAP32[$1>>2]|0;
   $16 = ($15|0)>=(0);
   $or$cond = $14 & $16;
   if ($or$cond) {
    $17 = HEAP32[$1>>2]|0;
    $18 = HEAP32[$pVm>>2]|0;
    $19 = ((($18)) + 20|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + (($17*40)|0)|0);
    HEAP32[$pOut>>2] = $21;
    $30 = HEAP32[$pOut>>2]|0;
    STACKTOP = sp;return ($30|0);
   }
  }
 }
 $22 = HEAP32[$pVm>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$pVm>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$pVm>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   _sqlite3Error($28,25);
  }
 }
 $29 = (_columnNullValue()|0);
 HEAP32[$pOut>>2] = $29;
 $30 = HEAP32[$pOut>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _columnMallocFailure($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 80|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3ApiExit($5,$8)|0);
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 80|0);
 HEAP32[$11>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3_column_bytes($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_bytes($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_double($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0, $7 = 0.0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (+_sqlite3_value_double($4));
 HEAPF64[$val>>3] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = +HEAPF64[$val>>3];
 STACKTOP = sp;return (+$7);
}
function _sqlite3_column_int($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_int64($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $val = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_int64($4)|0);
 $6 = tempRet0;
 $7 = $val;
 $8 = $7;
 HEAP32[$8>>2] = $5;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 _columnMallocFailure($11);
 $12 = $val;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 tempRet0 = ($17);
 STACKTOP = sp;return ($14|0);
}
function _sqlite3_column_text($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_value($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pOut = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 HEAP32[$pOut>>2] = $4;
 $5 = HEAP32[$pOut>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 2048;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $23 = HEAP32[$0>>2]|0;
  _columnMallocFailure($23);
  $24 = HEAP32[$pOut>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $11 = HEAP32[$pOut>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -2049;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$pOut>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 | 4096;
 $22 = $21&65535;
 HEAP16[$18>>1] = $22;
 $23 = HEAP32[$0>>2]|0;
 _columnMallocFailure($23);
 $24 = HEAP32[$pOut>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3_column_type($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $iType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iType = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_type($4)|0);
 HEAP32[$iType>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$iType>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_name($pStmt,$N) {
 $pStmt = $pStmt|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnName($2,$3,170,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _columnName($pStmt,$N,$xFunc,$useType) {
 $pStmt = $pStmt|0;
 $N = $N|0;
 $xFunc = $xFunc|0;
 $useType = $useType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $or$cond = 0, $p = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $ret = sp + 12|0;
 $p = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $xFunc;
 HEAP32[$3>>2] = $useType;
 HEAP32[$ret>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_column_count($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$n>>2]|0;
 $11 = ($9|0)<($10|0);
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)>=(0);
 $or$cond = $11 & $13;
 if (!($or$cond)) {
  $31 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$n>>2]|0;
 $16 = Math_imul($14, $15)|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16))|0;
 HEAP32[$1>>2] = $18;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + (($20*40)|0)|0);
 $25 = (FUNCTION_TABLE_ii[$19 & 255]($24)|0);
 HEAP32[$ret>>2] = $25;
 $26 = HEAP32[$db>>2]|0;
 $27 = ((($26)) + 65|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = ($28<<24>>24)!=(0);
 if (!($29)) {
  $31 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $30 = HEAP32[$db>>2]|0;
 _sqlite3OomClear($30);
 HEAP32[$ret>>2] = 0;
 $31 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _sqlite3_bind_blob($pStmt,$i,$zData,$nData,$xDel) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_bindText($5,$6,$7,$8,$9,0)|0);
 STACKTOP = sp;return ($10|0);
}
function _bindText($pStmt,$i,$zData,$nData,$xDel,$encoding) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 $encoding = $encoding|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $pVar = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 32|0;
 $p = sp + 8|0;
 $pVar = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 HEAP8[$5>>0] = $encoding;
 $6 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_vdbeUnbind($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $45 = HEAP32[$4>>2]|0;
  $46 = ($45|0)!=(0|0);
  $47 = HEAP32[$4>>2]|0;
  $48 = ($47|0)!=((-1)|0);
  $or$cond = $46 & $48;
  if (!($or$cond)) {
   $51 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($51|0);
  }
  $49 = HEAP32[$4>>2]|0;
  $50 = HEAP32[$2>>2]|0;
  FUNCTION_TABLE_vi[$49 & 255]($50);
  $51 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $51 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) - 1)|0;
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + (($15*40)|0)|0);
 HEAP32[$pVar>>2] = $19;
 $20 = HEAP32[$pVar>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = HEAP8[$5>>0]|0;
 $24 = HEAP32[$4>>2]|0;
 $25 = (_sqlite3VdbeMemSetStr($20,$21,$22,$23,$24)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP8[$5>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$pVar>>2]|0;
   $32 = HEAP32[$p>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = ((($33)) + 62|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = $35&255;
   $37 = (_sqlite3VdbeChangeEncoding($31,$36)|0);
   HEAP32[$rc>>2] = $37;
  }
 }
 $38 = HEAP32[$p>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$rc>>2]|0;
 _sqlite3Error($39,$40);
 $41 = HEAP32[$p>>2]|0;
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$rc>>2]|0;
 $44 = (_sqlite3ApiExit($42,$43)|0);
 HEAP32[$rc>>2] = $44;
 $51 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _sqlite3_bind_double($pStmt,$i,$rValue) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $rValue = +$rValue;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAPF64[$2>>3] = $rValue;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_vdbeUnbind($4,$5)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 60|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*40)|0)|0);
 $15 = +HEAPF64[$2>>3];
 _sqlite3VdbeMemSetDouble($14,$15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _vdbeUnbind($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $pVar = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pVar = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_vdbeSafetyNotNull($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_sqlite3MisuseError(73485)|0);
  HEAP32[$0>>2] = $6;
  $68 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(-1108210269);
 if (!($10)) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 76|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)>=(0);
  if (!($14)) {
   $21 = HEAP32[$2>>2]|0;
   $22 = ($21|0)<(1);
   if (!($22)) {
    $23 = HEAP32[$2>>2]|0;
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 68|0);
    $26 = HEAP16[$25>>1]|0;
    $27 = $26 << 16 >> 16;
    $28 = ($23|0)>($27|0);
    if (!($28)) {
     $31 = HEAP32[$2>>2]|0;
     $32 = (($31) + -1)|0;
     HEAP32[$2>>2] = $32;
     $33 = HEAP32[$2>>2]|0;
     $34 = HEAP32[$1>>2]|0;
     $35 = ((($34)) + 60|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = (($36) + (($33*40)|0)|0);
     HEAP32[$pVar>>2] = $37;
     $38 = HEAP32[$pVar>>2]|0;
     _sqlite3VdbeMemRelease($38);
     $39 = HEAP32[$pVar>>2]|0;
     $40 = ((($39)) + 8|0);
     HEAP16[$40>>1] = 1;
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     _sqlite3Error($42,0);
     $43 = HEAP32[$1>>2]|0;
     $44 = ((($43)) + 88|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = ($45&65535) >>> 8;
     $47 = $46 & 1;
     $48 = $47&65535;
     $49 = ($48|0)!=(0);
     do {
      if ($49) {
       $50 = HEAP32[$2>>2]|0;
       $51 = ($50|0)<(32);
       if ($51) {
        $52 = HEAP32[$1>>2]|0;
        $53 = ((($52)) + 188|0);
        $54 = HEAP32[$53>>2]|0;
        $55 = HEAP32[$2>>2]|0;
        $56 = 1 << $55;
        $57 = $54 & $56;
        $58 = ($57|0)!=(0);
        if (!($58)) {
         label = 12;
        }
       } else {
        label = 12;
       }
       if ((label|0) == 12) {
        $59 = HEAP32[$1>>2]|0;
        $60 = ((($59)) + 188|0);
        $61 = HEAP32[$60>>2]|0;
        $62 = ($61|0)==(-1);
        if (!($62)) {
         break;
        }
       }
       $63 = HEAP32[$1>>2]|0;
       $64 = ((($63)) + 88|0);
       $65 = HEAP16[$64>>1]|0;
       $66 = $65 & -9;
       $67 = $66 | 8;
       HEAP16[$64>>1] = $67;
      }
     } while(0);
     HEAP32[$0>>2] = 0;
     $68 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($68|0);
    }
   }
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   _sqlite3Error($30,25);
   HEAP32[$0>>2] = 25;
   $68 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($68|0);
  }
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 _sqlite3Error($16,21);
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 168|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer>>2] = $19;
 _sqlite3_log(21,25241,$vararg_buffer);
 $20 = (_sqlite3MisuseError(73493)|0);
 HEAP32[$0>>2] = $20;
 $68 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($68|0);
}
function _sqlite3_bind_int($p,$i,$iValue) {
 $p = $p|0;
 $i = $i|0;
 $iValue = $iValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $iValue;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3_bind_int64($3,$4,$5,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_bind_int64($pStmt,$i,$0,$1) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $pStmt;
 HEAP32[$3>>2] = $i;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (_vdbeUnbind($10,$11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 60|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + (($16*40)|0)|0);
 $21 = $4;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 _sqlite3VdbeMemSetInt64($20,$23,$26);
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3_bind_null($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_vdbeUnbind($3,$4)|0);
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3_bind_text($pStmt,$i,$zData,$nData,$xDel) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_bindText($5,$6,$7,$8,$9,1)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3_bind_value($pStmt,$i,$pValue) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pValue;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3_value_type($3)|0);
 switch ($4|0) {
 case 1:  {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = $7;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($8) + 4)|0;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_bind_int64($5,$6,$10,$13)|0);
  HEAP32[$rc>>2] = $14;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
  break;
 }
 case 2:  {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = +HEAPF64[$17>>3];
  $19 = (_sqlite3_bind_double($15,$16,$18)|0);
  HEAP32[$rc>>2] = $19;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
  break;
 }
 case 4:  {
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22&65535;
  $24 = $23 & 16384;
  $25 = ($24|0)!=(0);
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  if ($25) {
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3_bind_zeroblob($26,$27,$29)|0);
   HEAP32[$rc>>2] = $30;
   $52 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($52|0);
  } else {
   $31 = ((($28)) + 16|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = ((($33)) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3_bind_blob($26,$27,$32,$35,(-1))|0);
   HEAP32[$rc>>2] = $36;
   $52 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
  break;
 }
 case 3:  {
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = ((($39)) + 16|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 12|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 10|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = (_bindText($37,$38,$41,$44,(-1),$47)|0);
  HEAP32[$rc>>2] = $48;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
  break;
 }
 default: {
  $49 = HEAP32[$0>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = (_sqlite3_bind_null($49,$50)|0);
  HEAP32[$rc>>2] = $51;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 }
 return (0)|0;
}
function _sqlite3_bind_zeroblob($pStmt,$i,$n) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_vdbeUnbind($4,$5)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 60|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*40)|0)|0);
 $15 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemSetZeroBlob($14,$15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3_bind_parameter_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $8 = 0;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 68|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6 << 16 >> 16;
 $8 = $7;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_bind_parameter_index($pStmt,$zName) {
 $pStmt = $pStmt|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 $6 = (_sqlite3VdbeParameterIndex($2,$3,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeParameterIndex($p,$zName,$nName) {
 $p = $p|0;
 $zName = $zName|0;
 $nName = $nName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $nName;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0|0);
 L5: do {
  if ($7) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $8 = HEAP32[$i>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = ((($9)) + 70|0);
    $11 = HEAP16[$10>>1]|0;
    $12 = $11 << 16 >> 16;
    $13 = ($8|0)<($12|0);
    if (!($13)) {
     break L5;
    }
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 64|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($17) + ($14<<2)|0);
    $19 = HEAP32[$18>>2]|0;
    HEAP32[$z>>2] = $19;
    $20 = HEAP32[$z>>2]|0;
    $21 = ($20|0)!=(0|0);
    if ($21) {
     $22 = HEAP32[$z>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = HEAP32[$3>>2]|0;
     $25 = (_strncmp($22,$23,$24)|0);
     $26 = ($25|0)==(0);
     if ($26) {
      $27 = HEAP32[$3>>2]|0;
      $28 = HEAP32[$z>>2]|0;
      $29 = (($28) + ($27)|0);
      $30 = HEAP8[$29>>0]|0;
      $31 = $30 << 24 >> 24;
      $32 = ($31|0)==(0);
      if ($32) {
       break;
      }
     }
    }
    $35 = HEAP32[$i>>2]|0;
    $36 = (($35) + 1)|0;
    HEAP32[$i>>2] = $36;
   }
   $33 = HEAP32[$i>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$0>>2] = $34;
   $37 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3TransferBindings($pFromStmt,$pToStmt) {
 $pFromStmt = $pFromStmt|0;
 $pToStmt = $pToStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pFrom = 0, $pTo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pFrom = sp + 8|0;
 $pTo = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pFromStmt;
 HEAP32[$1>>2] = $pToStmt;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFrom>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pTo>>2] = $3;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$pFrom>>2]|0;
  $6 = ((($5)) + 68|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7 << 16 >> 16;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pTo>>2]|0;
  $12 = ((($11)) + 60|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + (($10*40)|0)|0);
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$pFrom>>2]|0;
  $17 = ((($16)) + 60|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + (($15*40)|0)|0);
  _sqlite3VdbeMemMove($14,$19);
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3_value_numeric_type($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $pMem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $eType = sp + 4|0;
 $pMem = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_value_type($1)|0);
 HEAP32[$eType>>2] = $2;
 $3 = HEAP32[$eType>>2]|0;
 $4 = ($3|0)==(3);
 if (!($4)) {
  $9 = HEAP32[$eType>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$pMem>>2] = $5;
 $6 = HEAP32[$pMem>>2]|0;
 _applyNumericAffinity($6,0);
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_value_type($7)|0);
 HEAP32[$eType>>2] = $8;
 $9 = HEAP32[$eType>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _applyNumericAffinity($pRec,$bTryForInt) {
 $pRec = $pRec|0;
 $bTryForInt = $bTryForInt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $enc = 0, $iValue = 0, $rValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rValue = sp + 8|0;
 $iValue = sp;
 $enc = sp + 24|0;
 HEAP32[$0>>2] = $pRec;
 HEAP32[$1>>2] = $bTryForInt;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 10|0);
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$enc>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP8[$enc>>0]|0;
 $12 = (_sqlite3AtoF($7,$rValue,$10,$11)|0);
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP8[$enc>>0]|0;
 $21 = (_sqlite3Atoi64($16,$iValue,$19,$20)|0);
 $22 = (0)==($21|0);
 if ($22) {
  $23 = $iValue;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = $29;
  $31 = $30;
  HEAP32[$31>>2] = $25;
  $32 = (($30) + 4)|0;
  $33 = $32;
  HEAP32[$33>>2] = $28;
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 8|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 | 4;
  $39 = $38&65535;
  HEAP16[$35>>1] = $39;
  STACKTOP = sp;return;
 }
 $40 = +HEAPF64[$rValue>>3];
 $41 = HEAP32[$0>>2]|0;
 HEAPF64[$41>>3] = $40;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 8|0);
 $44 = HEAP16[$43>>1]|0;
 $45 = $44&65535;
 $46 = $45 | 8;
 $47 = $46&65535;
 HEAP16[$43>>1] = $47;
 $48 = HEAP32[$1>>2]|0;
 $49 = ($48|0)!=(0);
 if (!($49)) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$0>>2]|0;
 _sqlite3VdbeIntegerAffinity($50);
 STACKTOP = sp;return;
}
function _sqlite3_blob_open($db,$zDb,$zTable,$zColumn,$0,$1,$flags,$ppBlob) {
 $db = $db|0;
 $zDb = $zDb|0;
 $zTable = $zTable|0;
 $zColumn = $zColumn|0;
 $0 = $0|0;
 $1 = $1|0;
 $flags = $flags|0;
 $ppBlob = $ppBlob|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aOp = 0, $iCol = 0, $iDb = 0, $j = 0, $j1 = 0, $nAttempt = 0, $pBlob = 0, $pFKey = 0, $pIdx = 0, $pParse = 0, $pTab = 0, $rc = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $zErr = 0;
 var $zFault = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 120|0;
 $6 = sp;
 $7 = sp + 116|0;
 $8 = sp + 112|0;
 $nAttempt = sp + 108|0;
 $iCol = sp + 104|0;
 $rc = sp + 100|0;
 $zErr = sp + 96|0;
 $pTab = sp + 92|0;
 $pParse = sp + 88|0;
 $pBlob = sp + 84|0;
 $zFault = sp + 80|0;
 $pIdx = sp + 76|0;
 $pFKey = sp + 72|0;
 $j = sp + 68|0;
 $j1 = sp + 64|0;
 $v = sp + 60|0;
 $iDb = sp + 56|0;
 $aOp = sp + 52|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $zDb;
 HEAP32[$4>>2] = $zTable;
 HEAP32[$5>>2] = $zColumn;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $flags;
 HEAP32[$8>>2] = $ppBlob;
 HEAP32[$nAttempt>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 HEAP32[$pParse>>2] = 0;
 HEAP32[$pBlob>>2] = 0;
 $13 = HEAP32[$8>>2]|0;
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$7>>2]|0;
 $15 = ($14|0)!=(0);
 $16 = $15 ^ 1;
 $17 = $16 ^ 1;
 $18 = $17&1;
 HEAP32[$7>>2] = $18;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3DbMallocZero($19,28,0)|0);
 HEAP32[$pBlob>>2] = $20;
 $21 = HEAP32[$pBlob>>2]|0;
 $22 = ($21|0)!=(0|0);
 L1: do {
  if ($22) {
   $23 = HEAP32[$2>>2]|0;
   $24 = (_sqlite3DbMallocRaw($23,544,0)|0);
   HEAP32[$pParse>>2] = $24;
   $25 = HEAP32[$pParse>>2]|0;
   $26 = ($25|0)!=(0|0);
   if ($26) {
    while(1) {
     $27 = HEAP32[$pParse>>2]|0;
     _memset(($27|0),0,544)|0;
     $28 = HEAP32[$2>>2]|0;
     $29 = HEAP32[$pParse>>2]|0;
     HEAP32[$29>>2] = $28;
     $30 = HEAP32[$2>>2]|0;
     $31 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($30,$31);
     HEAP32[$zErr>>2] = 0;
     $32 = HEAP32[$2>>2]|0;
     _sqlite3BtreeEnterAll($32);
     $33 = HEAP32[$pParse>>2]|0;
     $34 = HEAP32[$4>>2]|0;
     $35 = HEAP32[$3>>2]|0;
     $36 = (_sqlite3LocateTable($33,0,$34,$35)|0);
     HEAP32[$pTab>>2] = $36;
     $37 = HEAP32[$pTab>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$pTab>>2]|0;
      $40 = ((($39)) + 42|0);
      $41 = HEAP8[$40>>0]|0;
      $42 = $41&255;
      $43 = $42 & 16;
      $44 = ($43|0)!=(0);
      if ($44) {
       HEAP32[$pTab>>2] = 0;
       $45 = HEAP32[$pParse>>2]|0;
       $46 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer>>2] = $46;
       _sqlite3ErrorMsg($45,17323,$vararg_buffer);
      }
     }
     $47 = HEAP32[$pTab>>2]|0;
     $48 = ($47|0)!=(0|0);
     if ($48) {
      $49 = HEAP32[$pTab>>2]|0;
      $50 = ((($49)) + 42|0);
      $51 = HEAP8[$50>>0]|0;
      $52 = $51&255;
      $53 = $52 & 32;
      $54 = ($53|0)==(0);
      if (!($54)) {
       HEAP32[$pTab>>2] = 0;
       $55 = HEAP32[$pParse>>2]|0;
       $56 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer1>>2] = $56;
       _sqlite3ErrorMsg($55,17353,$vararg_buffer1);
      }
     }
     $57 = HEAP32[$pTab>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pTab>>2]|0;
      $60 = ((($59)) + 12|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)!=(0|0);
      if ($62) {
       HEAP32[$pTab>>2] = 0;
       $63 = HEAP32[$pParse>>2]|0;
       $64 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $64;
       _sqlite3ErrorMsg($63,17389,$vararg_buffer4);
      }
     }
     $65 = HEAP32[$pTab>>2]|0;
     $66 = ($65|0)!=(0|0);
     if (!($66)) {
      label = 13;
      break;
     }
     HEAP32[$iCol>>2] = 0;
     while(1) {
      $78 = HEAP32[$iCol>>2]|0;
      $79 = HEAP32[$pTab>>2]|0;
      $80 = ((($79)) + 34|0);
      $81 = HEAP16[$80>>1]|0;
      $82 = $81 << 16 >> 16;
      $83 = ($78|0)<($82|0);
      if (!($83)) {
       break;
      }
      $84 = HEAP32[$iCol>>2]|0;
      $85 = HEAP32[$pTab>>2]|0;
      $86 = ((($85)) + 4|0);
      $87 = HEAP32[$86>>2]|0;
      $88 = (($87) + (($84*24)|0)|0);
      $89 = HEAP32[$88>>2]|0;
      $90 = HEAP32[$5>>2]|0;
      $91 = (_sqlite3_stricmp($89,$90)|0);
      $92 = ($91|0)==(0);
      if ($92) {
       break;
      }
      $93 = HEAP32[$iCol>>2]|0;
      $94 = (($93) + 1)|0;
      HEAP32[$iCol>>2] = $94;
     }
     $95 = HEAP32[$iCol>>2]|0;
     $96 = HEAP32[$pTab>>2]|0;
     $97 = ((($96)) + 34|0);
     $98 = HEAP16[$97>>1]|0;
     $99 = $98 << 16 >> 16;
     $100 = ($95|0)==($99|0);
     if ($100) {
      label = 21;
      break;
     }
     $106 = HEAP32[$7>>2]|0;
     $107 = ($106|0)!=(0);
     if ($107) {
      HEAP32[$zFault>>2] = 0;
      $108 = HEAP32[$2>>2]|0;
      $109 = ((($108)) + 24|0);
      $110 = HEAP32[$109>>2]|0;
      $111 = $110 & 524288;
      $112 = ($111|0)!=(0);
      L26: do {
       if ($112) {
        $113 = HEAP32[$pTab>>2]|0;
        $114 = ((($113)) + 16|0);
        $115 = HEAP32[$114>>2]|0;
        HEAP32[$pFKey>>2] = $115;
        while(1) {
         $116 = HEAP32[$pFKey>>2]|0;
         $117 = ($116|0)!=(0|0);
         if (!($117)) {
          break L26;
         }
         HEAP32[$j>>2] = 0;
         while(1) {
          $118 = HEAP32[$j>>2]|0;
          $119 = HEAP32[$pFKey>>2]|0;
          $120 = ((($119)) + 20|0);
          $121 = HEAP32[$120>>2]|0;
          $122 = ($118|0)<($121|0);
          if (!($122)) {
           break;
          }
          $123 = HEAP32[$j>>2]|0;
          $124 = HEAP32[$pFKey>>2]|0;
          $125 = ((($124)) + 36|0);
          $126 = (($125) + ($123<<3)|0);
          $127 = HEAP32[$126>>2]|0;
          $128 = HEAP32[$iCol>>2]|0;
          $129 = ($127|0)==($128|0);
          if ($129) {
           HEAP32[$zFault>>2] = 17431;
          }
          $130 = HEAP32[$j>>2]|0;
          $131 = (($130) + 1)|0;
          HEAP32[$j>>2] = $131;
         }
         $132 = HEAP32[$pFKey>>2]|0;
         $133 = ((($132)) + 4|0);
         $134 = HEAP32[$133>>2]|0;
         HEAP32[$pFKey>>2] = $134;
        }
       }
      } while(0);
      $135 = HEAP32[$pTab>>2]|0;
      $136 = ((($135)) + 8|0);
      $137 = HEAP32[$136>>2]|0;
      HEAP32[$pIdx>>2] = $137;
      while(1) {
       $138 = HEAP32[$pIdx>>2]|0;
       $139 = ($138|0)!=(0|0);
       if (!($139)) {
        break;
       }
       HEAP32[$j1>>2] = 0;
       while(1) {
        $140 = HEAP32[$j1>>2]|0;
        $141 = HEAP32[$pIdx>>2]|0;
        $142 = ((($141)) + 50|0);
        $143 = HEAP16[$142>>1]|0;
        $144 = $143&65535;
        $145 = ($140|0)<($144|0);
        if (!($145)) {
         break;
        }
        $146 = HEAP32[$j1>>2]|0;
        $147 = HEAP32[$pIdx>>2]|0;
        $148 = ((($147)) + 4|0);
        $149 = HEAP32[$148>>2]|0;
        $150 = (($149) + ($146<<1)|0);
        $151 = HEAP16[$150>>1]|0;
        $152 = $151 << 16 >> 16;
        $153 = HEAP32[$iCol>>2]|0;
        $154 = ($152|0)==($153|0);
        if ($154) {
         label = 38;
        } else {
         $155 = HEAP32[$j1>>2]|0;
         $156 = HEAP32[$pIdx>>2]|0;
         $157 = ((($156)) + 4|0);
         $158 = HEAP32[$157>>2]|0;
         $159 = (($158) + ($155<<1)|0);
         $160 = HEAP16[$159>>1]|0;
         $161 = $160 << 16 >> 16;
         $162 = ($161|0)==(-2);
         if ($162) {
          label = 38;
         }
        }
        if ((label|0) == 38) {
         label = 0;
         HEAP32[$zFault>>2] = 17443;
        }
        $163 = HEAP32[$j1>>2]|0;
        $164 = (($163) + 1)|0;
        HEAP32[$j1>>2] = $164;
       }
       $165 = HEAP32[$pIdx>>2]|0;
       $166 = ((($165)) + 20|0);
       $167 = HEAP32[$166>>2]|0;
       HEAP32[$pIdx>>2] = $167;
      }
      $168 = HEAP32[$zFault>>2]|0;
      $169 = ($168|0)!=(0|0);
      if ($169) {
       label = 42;
       break;
      }
     }
     $175 = HEAP32[$pParse>>2]|0;
     $176 = (_sqlite3VdbeCreate($175)|0);
     $177 = HEAP32[$pBlob>>2]|0;
     $178 = ((($177)) + 20|0);
     HEAP32[$178>>2] = $176;
     $179 = HEAP32[$pBlob>>2]|0;
     $180 = ((($179)) + 20|0);
     $181 = HEAP32[$180>>2]|0;
     $182 = ($181|0)!=(0|0);
     if ($182) {
      $183 = HEAP32[$pBlob>>2]|0;
      $184 = ((($183)) + 20|0);
      $185 = HEAP32[$184>>2]|0;
      HEAP32[$v>>2] = $185;
      $186 = HEAP32[$2>>2]|0;
      $187 = HEAP32[$pTab>>2]|0;
      $188 = ((($187)) + 64|0);
      $189 = HEAP32[$188>>2]|0;
      $190 = (_sqlite3SchemaToIndex($186,$189)|0);
      HEAP32[$iDb>>2] = $190;
      $191 = HEAP32[$v>>2]|0;
      $192 = HEAP32[$iDb>>2]|0;
      $193 = HEAP32[$7>>2]|0;
      $194 = HEAP32[$pTab>>2]|0;
      $195 = ((($194)) + 64|0);
      $196 = HEAP32[$195>>2]|0;
      $197 = HEAP32[$196>>2]|0;
      $198 = HEAP32[$pTab>>2]|0;
      $199 = ((($198)) + 64|0);
      $200 = HEAP32[$199>>2]|0;
      $201 = ((($200)) + 4|0);
      $202 = HEAP32[$201>>2]|0;
      (_sqlite3VdbeAddOp4Int($191,2,$192,$193,$197,$202)|0);
      $203 = HEAP32[$v>>2]|0;
      _sqlite3VdbeChangeP5($203,1);
      $204 = HEAP32[$v>>2]|0;
      $205 = (_sqlite3VdbeAddOpList($204,9,17485,0)|0);
      HEAP32[$aOp>>2] = $205;
      $206 = HEAP32[$v>>2]|0;
      $207 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($206,$207);
      $208 = HEAP32[$2>>2]|0;
      $209 = ((($208)) + 65|0);
      $210 = HEAP8[$209>>0]|0;
      $211 = $210&255;
      $212 = ($211|0)==(0);
      if ($212) {
       $213 = HEAP32[$iDb>>2]|0;
       $214 = HEAP32[$aOp>>2]|0;
       $215 = ((($214)) + 4|0);
       HEAP32[$215>>2] = $213;
       $216 = HEAP32[$pTab>>2]|0;
       $217 = ((($216)) + 28|0);
       $218 = HEAP32[$217>>2]|0;
       $219 = HEAP32[$aOp>>2]|0;
       $220 = ((($219)) + 8|0);
       HEAP32[$220>>2] = $218;
       $221 = HEAP32[$7>>2]|0;
       $222 = HEAP32[$aOp>>2]|0;
       $223 = ((($222)) + 12|0);
       HEAP32[$223>>2] = $221;
       $224 = HEAP32[$v>>2]|0;
       $225 = HEAP32[$pTab>>2]|0;
       $226 = HEAP32[$225>>2]|0;
       _sqlite3VdbeChangeP4($224,1,$226,0);
      }
      $227 = HEAP32[$2>>2]|0;
      $228 = ((($227)) + 65|0);
      $229 = HEAP8[$228>>0]|0;
      $230 = $229&255;
      $231 = ($230|0)==(0);
      if ($231) {
       $232 = HEAP32[$7>>2]|0;
       $233 = ($232|0)!=(0);
       if ($233) {
        $234 = HEAP32[$aOp>>2]|0;
        $235 = ((($234)) + 20|0);
        HEAP8[$235>>0] = 55;
       }
       $236 = HEAP32[$pTab>>2]|0;
       $237 = ((($236)) + 28|0);
       $238 = HEAP32[$237>>2]|0;
       $239 = HEAP32[$aOp>>2]|0;
       $240 = ((($239)) + 20|0);
       $241 = ((($240)) + 8|0);
       HEAP32[$241>>2] = $238;
       $242 = HEAP32[$iDb>>2]|0;
       $243 = HEAP32[$aOp>>2]|0;
       $244 = ((($243)) + 20|0);
       $245 = ((($244)) + 12|0);
       HEAP32[$245>>2] = $242;
       $246 = HEAP32[$aOp>>2]|0;
       $247 = ((($246)) + 20|0);
       $248 = ((($247)) + 1|0);
       HEAP8[$248>>0] = -14;
       $249 = HEAP32[$pTab>>2]|0;
       $250 = ((($249)) + 34|0);
       $251 = HEAP16[$250>>1]|0;
       $252 = $251 << 16 >> 16;
       $253 = (($252) + 1)|0;
       $254 = HEAP32[$aOp>>2]|0;
       $255 = ((($254)) + 20|0);
       $256 = ((($255)) + 16|0);
       HEAP32[$256>>2] = $253;
       $257 = HEAP32[$pTab>>2]|0;
       $258 = ((($257)) + 34|0);
       $259 = HEAP16[$258>>1]|0;
       $260 = $259 << 16 >> 16;
       $261 = HEAP32[$aOp>>2]|0;
       $262 = ((($261)) + 80|0);
       $263 = ((($262)) + 8|0);
       HEAP32[$263>>2] = $260;
       $264 = HEAP32[$pParse>>2]|0;
       $265 = ((($264)) + 444|0);
       HEAP16[$265>>1] = 1;
       $266 = HEAP32[$pParse>>2]|0;
       $267 = ((($266)) + 76|0);
       HEAP32[$267>>2] = 1;
       $268 = HEAP32[$pParse>>2]|0;
       $269 = ((($268)) + 72|0);
       HEAP32[$269>>2] = 1;
       $270 = HEAP32[$v>>2]|0;
       $271 = HEAP32[$pParse>>2]|0;
       _sqlite3VdbeMakeReady($270,$271);
      }
     }
     $272 = HEAP32[$7>>2]|0;
     $273 = HEAP32[$pBlob>>2]|0;
     HEAP32[$273>>2] = $272;
     $274 = HEAP32[$iCol>>2]|0;
     $275 = HEAP32[$pBlob>>2]|0;
     $276 = ((($275)) + 12|0);
     HEAP32[$276>>2] = $274;
     $277 = HEAP32[$2>>2]|0;
     $278 = HEAP32[$pBlob>>2]|0;
     $279 = ((($278)) + 24|0);
     HEAP32[$279>>2] = $277;
     $280 = HEAP32[$2>>2]|0;
     $281 = ((($280)) + 65|0);
     $282 = HEAP8[$281>>0]|0;
     $283 = ($282<<24>>24)!=(0);
     if ($283) {
      break L1;
     }
     $284 = HEAP32[$pBlob>>2]|0;
     $285 = ((($284)) + 20|0);
     $286 = HEAP32[$285>>2]|0;
     $287 = $6;
     $288 = $287;
     $289 = HEAP32[$288>>2]|0;
     $290 = (($287) + 4)|0;
     $291 = $290;
     $292 = HEAP32[$291>>2]|0;
     (_sqlite3_bind_int64($286,1,$289,$292)|0);
     $293 = HEAP32[$pBlob>>2]|0;
     $294 = $6;
     $295 = $294;
     $296 = HEAP32[$295>>2]|0;
     $297 = (($294) + 4)|0;
     $298 = $297;
     $299 = HEAP32[$298>>2]|0;
     $300 = (_blobSeekToRow($293,$296,$299,$zErr)|0);
     HEAP32[$rc>>2] = $300;
     $301 = HEAP32[$nAttempt>>2]|0;
     $302 = (($301) + 1)|0;
     HEAP32[$nAttempt>>2] = $302;
     $303 = ($302|0)<(50);
     $304 = HEAP32[$rc>>2]|0;
     $305 = ($304|0)==(17);
     $306 = $303 ? $305 : 0;
     if (!($306)) {
      break L1;
     }
    }
    if ((label|0) == 13) {
     $67 = HEAP32[$pParse>>2]|0;
     $68 = ((($67)) + 4|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ($69|0)!=(0|0);
     if ($70) {
      $71 = HEAP32[$2>>2]|0;
      $72 = HEAP32[$zErr>>2]|0;
      _sqlite3DbFree($71,$72);
      $73 = HEAP32[$pParse>>2]|0;
      $74 = ((($73)) + 4|0);
      $75 = HEAP32[$74>>2]|0;
      HEAP32[$zErr>>2] = $75;
      $76 = HEAP32[$pParse>>2]|0;
      $77 = ((($76)) + 4|0);
      HEAP32[$77>>2] = 0;
     }
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 21) {
     $101 = HEAP32[$2>>2]|0;
     $102 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($101,$102);
     $103 = HEAP32[$2>>2]|0;
     $104 = HEAP32[$5>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $104;
     $105 = (_sqlite3MPrintf($103,17410,$vararg_buffer7)|0);
     HEAP32[$zErr>>2] = $105;
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 42) {
     $170 = HEAP32[$2>>2]|0;
     $171 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($170,$171);
     $172 = HEAP32[$2>>2]|0;
     $173 = HEAP32[$zFault>>2]|0;
     HEAP32[$vararg_buffer10>>2] = $173;
     $174 = (_sqlite3MPrintf($172,17451,$vararg_buffer10)|0);
     HEAP32[$zErr>>2] = $174;
     HEAP32[$rc>>2] = 1;
     break;
    }
   }
  }
 } while(0);
 $307 = HEAP32[$rc>>2]|0;
 $308 = ($307|0)==(0);
 if ($308) {
  $309 = HEAP32[$2>>2]|0;
  $310 = ((($309)) + 65|0);
  $311 = HEAP8[$310>>0]|0;
  $312 = $311&255;
  $313 = ($312|0)==(0);
  if ($313) {
   $314 = HEAP32[$pBlob>>2]|0;
   $315 = HEAP32[$8>>2]|0;
   HEAP32[$315>>2] = $314;
   $327 = HEAP32[$2>>2]|0;
   $328 = HEAP32[$rc>>2]|0;
   $329 = HEAP32[$zErr>>2]|0;
   $330 = ($329|0)!=(0|0);
   $331 = $330 ? 17521 : 0;
   $332 = HEAP32[$zErr>>2]|0;
   HEAP32[$vararg_buffer13>>2] = $332;
   _sqlite3ErrorWithMsg($327,$328,$331,$vararg_buffer13);
   $333 = HEAP32[$2>>2]|0;
   $334 = HEAP32[$zErr>>2]|0;
   _sqlite3DbFree($333,$334);
   $335 = HEAP32[$pParse>>2]|0;
   _sqlite3ParserReset($335);
   $336 = HEAP32[$2>>2]|0;
   $337 = HEAP32[$pParse>>2]|0;
   _sqlite3DbFree($336,$337);
   $338 = HEAP32[$2>>2]|0;
   $339 = HEAP32[$rc>>2]|0;
   $340 = (_sqlite3ApiExit($338,$339)|0);
   HEAP32[$rc>>2] = $340;
   $341 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($341|0);
  }
 }
 $316 = HEAP32[$pBlob>>2]|0;
 $317 = ($316|0)!=(0|0);
 if ($317) {
  $318 = HEAP32[$pBlob>>2]|0;
  $319 = ((($318)) + 20|0);
  $320 = HEAP32[$319>>2]|0;
  $321 = ($320|0)!=(0|0);
  if ($321) {
   $322 = HEAP32[$pBlob>>2]|0;
   $323 = ((($322)) + 20|0);
   $324 = HEAP32[$323>>2]|0;
   (_sqlite3VdbeFinalize($324)|0);
  }
 }
 $325 = HEAP32[$2>>2]|0;
 $326 = HEAP32[$pBlob>>2]|0;
 _sqlite3DbFree($325,$326);
 $327 = HEAP32[$2>>2]|0;
 $328 = HEAP32[$rc>>2]|0;
 $329 = HEAP32[$zErr>>2]|0;
 $330 = ($329|0)!=(0|0);
 $331 = $330 ? 17521 : 0;
 $332 = HEAP32[$zErr>>2]|0;
 HEAP32[$vararg_buffer13>>2] = $332;
 _sqlite3ErrorWithMsg($327,$328,$331,$vararg_buffer13);
 $333 = HEAP32[$2>>2]|0;
 $334 = HEAP32[$zErr>>2]|0;
 _sqlite3DbFree($333,$334);
 $335 = HEAP32[$pParse>>2]|0;
 _sqlite3ParserReset($335);
 $336 = HEAP32[$2>>2]|0;
 $337 = HEAP32[$pParse>>2]|0;
 _sqlite3DbFree($336,$337);
 $338 = HEAP32[$2>>2]|0;
 $339 = HEAP32[$rc>>2]|0;
 $340 = (_sqlite3ApiExit($338,$339)|0);
 HEAP32[$rc>>2] = $340;
 $341 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($341|0);
}
function _sqlite3DbMallocRaw($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $p = sp + 8|0;
 HEAP32[$3>>2] = $db;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = $4;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($12) + 4)|0;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3DbMallocRawNN($11,$14,$17)|0);
  HEAP32[$2>>2] = $18;
  $27 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3Malloc($21,$24)|0);
  HEAP32[$p>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  HEAP32[$2>>2] = $26;
  $27 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3LocateTable($pParse,$isView,$zName,$zDbase) {
 $pParse = $pParse|0;
 $isView = $isView|0;
 $zName = $zName|0;
 $zDbase = $zDbase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pMod = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $p = sp + 32|0;
 $zMsg = sp + 28|0;
 $pMod = sp + 24|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $isView;
 HEAP32[$3>>2] = $zName;
 HEAP32[$4>>2] = $zDbase;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3FindTable($9,$10,$11)|0);
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[$2>>2]|0;
  $16 = ($15|0)!=(0);
  $17 = $16 ? 25510 : 25523;
  HEAP32[$zMsg>>2] = $17;
  $18 = HEAP32[$1>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = (_sqlite3FindDbName($19,$20)|0);
  $22 = ($21|0)<(1);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = ((($24)) + 312|0);
   $26 = HEAP32[$3>>2]|0;
   $27 = (_sqlite3HashFind($25,$26)|0);
   HEAP32[$pMod>>2] = $27;
   $28 = HEAP32[$pMod>>2]|0;
   $29 = ($28|0)!=(0|0);
   if ($29) {
    $30 = HEAP32[$1>>2]|0;
    $31 = HEAP32[$pMod>>2]|0;
    $32 = (_sqlite3VtabEponymousTableInit($30,$31)|0);
    $33 = ($32|0)!=(0);
    if ($33) {
     $34 = HEAP32[$pMod>>2]|0;
     $35 = ((($34)) + 16|0);
     $36 = HEAP32[$35>>2]|0;
     HEAP32[$0>>2] = $36;
     $47 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($47|0);
    }
   }
  }
  $37 = HEAP32[$4>>2]|0;
  $38 = ($37|0)!=(0|0);
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$zMsg>>2]|0;
  if ($38) {
   $41 = HEAP32[$4>>2]|0;
   $42 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer>>2] = $40;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $41;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $42;
   _sqlite3ErrorMsg($39,25537,$vararg_buffer);
  } else {
   $43 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $40;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = $43;
   _sqlite3ErrorMsg($39,25547,$vararg_buffer3);
  }
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 17|0);
  HEAP8[$45>>0] = 1;
 }
 $46 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $46;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3ErrorMsg($pParse,$zFormat,$varargs) {
 $pParse = $pParse|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $db = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $zMsg = sp + 24|0;
 $ap = sp + 8|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$db>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VMPrintf($4,$5,$ap)|0);
 HEAP32[$zMsg>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 69|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$zMsg>>2]|0;
  _sqlite3DbFree($11,$12);
  STACKTOP = sp;return;
 } else {
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 68|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$14>>2] = $16;
  $17 = HEAP32[$db>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  _sqlite3DbFree($17,$20);
  $21 = HEAP32[$zMsg>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 4|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 12|0);
  HEAP32[$25>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3MPrintf($db,$zFormat,$varargs) {
 $db = $db|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VMPrintf($2,$3,$ap)|0);
 HEAP32[$z>>2] = $4;
 $5 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3VdbeCreate($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (_sqlite3DbMallocZero($4,208,0)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$p>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$p>>2]|0;
  $15 = HEAP32[$db>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 48|0);
  HEAP32[$18>>2] = $14;
 }
 $19 = HEAP32[$db>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 52|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 48|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$p>>2]|0;
 $27 = HEAP32[$db>>2]|0;
 $28 = ((($27)) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$p>>2]|0;
 $30 = ((($29)) + 40|0);
 HEAP32[$30>>2] = 649915045;
 $31 = HEAP32[$1>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 24|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3SchemaToIndex($db,$pSchema) {
 $db = $db|0;
 $pSchema = $pSchema|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSchema;
 HEAP32[$i>>2] = -1000000;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $20 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   label = 6;
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 6) {
  $20 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _sqlite3VdbeAddOp4Int($p,$op,$p1,$p2,$p3,$p4) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $p4 = $p4|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $p4;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_sqlite3VdbeAddOp3($6,$7,$8,$9,$10)|0);
 HEAP32[$addr>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$addr>>2]|0;
 $14 = HEAP32[$5>>2]|0;
 $15 = $14;
 _sqlite3VdbeChangeP4($12,$13,$15,-14);
 $16 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3VdbeChangeP5($p,$p5) {
 $p = $p|0;
 $p5 = $p5|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $p5;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 65|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP8[$1>>0]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + (($11*20)|0)|0);
 $16 = ((($15)) + 3|0);
 HEAP8[$16>>0] = $7;
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOpList($p,$nOp,$aOp,$iLineno) {
 $p = $p|0;
 $nOp = $nOp|0;
 $aOp = $aOp|0;
 $iLineno = $iLineno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $i = 0, $pFirst = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $i = sp + 8|0;
 $pOut = sp + 4|0;
 $pFirst = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nOp;
 HEAP32[$3>>2] = $aOp;
 HEAP32[$4>>2] = $iLineno;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 32|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($7) + ($8))|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 88|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($9|0)>($14|0);
 if ($15) {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_growOpArray($16,$17)|0);
  $19 = ($18|0)!=(0);
  if ($19) {
   HEAP32[$0>>2] = 0;
   $89 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($89|0);
  }
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 32|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + (($22*20)|0)|0);
 HEAP32[$pOut>>2] = $26;
 HEAP32[$pFirst>>2] = $26;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP8[$30>>0]|0;
  $32 = HEAP32[$pOut>>2]|0;
  HEAP8[$32>>0] = $31;
  $33 = HEAP32[$3>>2]|0;
  $34 = ((($33)) + 1|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35 << 24 >> 24;
  $37 = HEAP32[$pOut>>2]|0;
  $38 = ((($37)) + 4|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$3>>2]|0;
  $40 = ((($39)) + 2|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = HEAP32[$pOut>>2]|0;
  $44 = ((($43)) + 8|0);
  HEAP32[$44>>2] = $42;
  $45 = HEAP32[$3>>2]|0;
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = (25281 + ($47)|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $50 & 1;
  $52 = ($51|0)!=(0);
  if ($52) {
   $53 = HEAP32[$3>>2]|0;
   $54 = ((($53)) + 2|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = ($56|0)>(0);
   if ($57) {
    $58 = HEAP32[$1>>2]|0;
    $59 = ((($58)) + 32|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$pOut>>2]|0;
    $62 = ((($61)) + 8|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = (($63) + ($60))|0;
    HEAP32[$62>>2] = $64;
   }
  }
  $65 = HEAP32[$3>>2]|0;
  $66 = ((($65)) + 3|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67 << 24 >> 24;
  $69 = HEAP32[$pOut>>2]|0;
  $70 = ((($69)) + 12|0);
  HEAP32[$70>>2] = $68;
  $71 = HEAP32[$pOut>>2]|0;
  $72 = ((($71)) + 1|0);
  HEAP8[$72>>0] = 0;
  $73 = HEAP32[$pOut>>2]|0;
  $74 = ((($73)) + 16|0);
  HEAP32[$74>>2] = 0;
  $75 = HEAP32[$pOut>>2]|0;
  $76 = ((($75)) + 3|0);
  HEAP8[$76>>0] = 0;
  $77 = HEAP32[$i>>2]|0;
  $78 = (($77) + 1)|0;
  HEAP32[$i>>2] = $78;
  $79 = HEAP32[$3>>2]|0;
  $80 = ((($79)) + 4|0);
  HEAP32[$3>>2] = $80;
  $81 = HEAP32[$pOut>>2]|0;
  $82 = ((($81)) + 20|0);
  HEAP32[$pOut>>2] = $82;
 }
 $83 = HEAP32[$2>>2]|0;
 $84 = HEAP32[$1>>2]|0;
 $85 = ((($84)) + 32|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = (($86) + ($83))|0;
 HEAP32[$85>>2] = $87;
 $88 = HEAP32[$pFirst>>2]|0;
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3VdbeUsesBtree($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$1>>2]|0;
 $3 = 1 << $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 96|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 | $3;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(1);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($10<<4)|0);
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3BtreeSharable($17)|0);
 $19 = ($18|0)!=(0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = 1 << $20;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 100|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = $24 | $21;
 HEAP32[$23>>2] = $25;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP4($p,$addr,$zP4,$n) {
 $p = $p|0;
 $addr = $addr|0;
 $zP4 = $zP4|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pOp = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $zP4;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 65|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 if ($9) {
  $10 = HEAP32[$3>>2]|0;
  $11 = ($10|0)!=(-10);
  if (!($11)) {
   STACKTOP = sp;return;
  }
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$3>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _freeP4($12,$13,$14);
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - 1)|0;
  HEAP32[$1>>2] = $20;
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + (($21*20)|0)|0);
 HEAP32[$pOp>>2] = $25;
 $26 = HEAP32[$3>>2]|0;
 $27 = ($26|0)>=(0);
 if (!($27)) {
  $28 = HEAP32[$pOp>>2]|0;
  $29 = ((($28)) + 1|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30 << 24 >> 24;
  $32 = ($31|0)!=(0);
  if (!($32)) {
   $37 = HEAP32[$3>>2]|0;
   $38 = ($37|0)==(-14);
   $39 = HEAP32[$2>>2]|0;
   if ($38) {
    $40 = $39;
    $41 = HEAP32[$pOp>>2]|0;
    $42 = ((($41)) + 16|0);
    HEAP32[$42>>2] = $40;
    $43 = HEAP32[$pOp>>2]|0;
    $44 = ((($43)) + 1|0);
    HEAP8[$44>>0] = -14;
    STACKTOP = sp;return;
   }
   $45 = ($39|0)!=(0|0);
   if (!($45)) {
    STACKTOP = sp;return;
   }
   $46 = HEAP32[$2>>2]|0;
   $47 = HEAP32[$pOp>>2]|0;
   $48 = ((($47)) + 16|0);
   HEAP32[$48>>2] = $46;
   $49 = HEAP32[$3>>2]|0;
   $50 = $49&255;
   $51 = HEAP32[$pOp>>2]|0;
   $52 = ((($51)) + 1|0);
   HEAP8[$52>>0] = $50;
   $53 = HEAP32[$3>>2]|0;
   $54 = ($53|0)==(-10);
   if (!($54)) {
    STACKTOP = sp;return;
   }
   $55 = HEAP32[$2>>2]|0;
   _sqlite3VtabLock($55);
   STACKTOP = sp;return;
  }
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = HEAP32[$pOp>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 _vdbeChangeP4Full($33,$34,$35,$36);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMakeReady($p,$pParse) {
 $p = $p|0;
 $pParse = $pParse|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $n = 0, $nArg = 0, $nCursor = 0, $nMem = 0, $nOnce = 0, $nVar = 0, $or$cond = 0, $x = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $db = sp + 36|0;
 $nVar = sp + 32|0;
 $nMem = sp + 28|0;
 $nCursor = sp + 24|0;
 $nArg = sp + 20|0;
 $nOnce = sp + 16|0;
 $n = sp + 12|0;
 $x = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 444|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6 << 16 >> 16;
 HEAP32[$nVar>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 76|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$nMem>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 72|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$nCursor>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 400|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$nArg>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 84|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$nOnce>>2] = $19;
 $20 = HEAP32[$nOnce>>2]|0;
 $21 = ($20|0)==(0);
 $$ = $21 ? 1 : $19;
 HEAP32[$nOnce>>2] = $$;
 $22 = HEAP32[$nCursor>>2]|0;
 $23 = HEAP32[$nMem>>2]|0;
 $24 = (($23) + ($22))|0;
 HEAP32[$nMem>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27*20)|0;
 $29 = (($28) + 7)|0;
 $30 = $29 & -8;
 HEAP32[$n>>2] = $30;
 $31 = HEAP32[$n>>2]|0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31)|0);
 HEAP32[$x>>2] = $35;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 92|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$n>>2]|0;
 $40 = (($38) - ($39))|0;
 $41 = $40 & -8;
 $42 = ((($x)) + 4|0);
 HEAP32[$42>>2] = $41;
 $43 = ((($x)) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)>(0);
 if ($45) {
  $46 = HEAP32[$x>>2]|0;
  $47 = ((($x)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  _memset(($46|0),0,($48|0))|0;
 }
 $49 = HEAP32[$0>>2]|0;
 _resolveP2Values($49,$nArg);
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 20|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = $52&255;
 $54 = ($53|0)!=(0);
 if ($54) {
  $55 = HEAP32[$1>>2]|0;
  $56 = ((($55)) + 21|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = $57&255;
  $59 = ($58|0)!=(0);
  $61 = $59;
 } else {
  $61 = 0;
 }
 $60 = $61&1;
 $62 = $60&255;
 $63 = $62&255;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 88|0);
 $66 = $63&65535;
 $67 = HEAP16[$65>>1]|0;
 $68 = $66 & 1;
 $69 = ($68 << 5)&65535;
 $70 = $67 & -33;
 $71 = $70 | $69;
 HEAP16[$65>>1] = $71;
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 453|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 $76 = ($75|0)!=(0);
 $77 = HEAP32[$nMem>>2]|0;
 $78 = ($77|0)<(10);
 $or$cond = $76 & $78;
 if ($or$cond) {
  HEAP32[$nMem>>2] = 10;
 }
 $79 = HEAP32[$0>>2]|0;
 $80 = ((($79)) + 88|0);
 $81 = HEAP16[$80>>1]|0;
 $82 = $81 & -9;
 HEAP16[$80>>1] = $82;
 while(1) {
  $83 = ((($x)) + 8|0);
  HEAP32[$83>>2] = 0;
  $84 = HEAP32[$0>>2]|0;
  $85 = ((($84)) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$nMem>>2]|0;
  $88 = ($87*40)|0;
  $89 = (_allocSpace($x,$86,$88)|0);
  $90 = HEAP32[$0>>2]|0;
  $91 = ((($90)) + 8|0);
  HEAP32[$91>>2] = $89;
  $92 = HEAP32[$0>>2]|0;
  $93 = ((($92)) + 60|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = HEAP32[$nVar>>2]|0;
  $96 = ($95*40)|0;
  $97 = (_allocSpace($x,$94,$96)|0);
  $98 = HEAP32[$0>>2]|0;
  $99 = ((($98)) + 60|0);
  HEAP32[$99>>2] = $97;
  $100 = HEAP32[$0>>2]|0;
  $101 = ((($100)) + 12|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = HEAP32[$nArg>>2]|0;
  $104 = $103<<2;
  $105 = (_allocSpace($x,$102,$104)|0);
  $106 = HEAP32[$0>>2]|0;
  $107 = ((($106)) + 12|0);
  HEAP32[$107>>2] = $105;
  $108 = HEAP32[$0>>2]|0;
  $109 = ((($108)) + 56|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$nCursor>>2]|0;
  $112 = $111<<2;
  $113 = (_allocSpace($x,$110,$112)|0);
  $114 = HEAP32[$0>>2]|0;
  $115 = ((($114)) + 56|0);
  HEAP32[$115>>2] = $113;
  $116 = HEAP32[$0>>2]|0;
  $117 = ((($116)) + 200|0);
  $118 = HEAP32[$117>>2]|0;
  $119 = HEAP32[$nOnce>>2]|0;
  $120 = (_allocSpace($x,$118,$119)|0);
  $121 = HEAP32[$0>>2]|0;
  $122 = ((($121)) + 200|0);
  HEAP32[$122>>2] = $120;
  $123 = ((($x)) + 8|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = ($124|0)==(0);
  if ($125) {
   break;
  }
  $126 = HEAP32[$db>>2]|0;
  $127 = ((($x)) + 8|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = ($128|0)<(0);
  $130 = $129 << 31 >> 31;
  $131 = (_sqlite3DbMallocZero($126,$128,$130)|0);
  $132 = HEAP32[$0>>2]|0;
  $133 = ((($132)) + 172|0);
  HEAP32[$133>>2] = $131;
  HEAP32[$x>>2] = $131;
  $134 = ((($x)) + 8|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = ((($x)) + 4|0);
  HEAP32[$136>>2] = $135;
  $137 = HEAP32[$db>>2]|0;
  $138 = ((($137)) + 65|0);
  $139 = HEAP8[$138>>0]|0;
  $140 = ($139<<24>>24)!=(0);
  $141 = $140 ^ 1;
  if (!($141)) {
   break;
  }
 }
 $142 = HEAP32[$nCursor>>2]|0;
 $143 = HEAP32[$0>>2]|0;
 $144 = ((($143)) + 36|0);
 HEAP32[$144>>2] = $142;
 $145 = HEAP32[$nOnce>>2]|0;
 $146 = HEAP32[$0>>2]|0;
 $147 = ((($146)) + 196|0);
 HEAP32[$147>>2] = $145;
 $148 = HEAP32[$0>>2]|0;
 $149 = ((($148)) + 60|0);
 $150 = HEAP32[$149>>2]|0;
 $151 = ($150|0)!=(0|0);
 L14: do {
  if ($151) {
   $152 = HEAP32[$nVar>>2]|0;
   $153 = $152&65535;
   $154 = HEAP32[$0>>2]|0;
   $155 = ((($154)) + 68|0);
   HEAP16[$155>>1] = $153;
   HEAP32[$n>>2] = 0;
   while(1) {
    $156 = HEAP32[$n>>2]|0;
    $157 = HEAP32[$nVar>>2]|0;
    $158 = ($156|0)<($157|0);
    if (!($158)) {
     break L14;
    }
    $159 = HEAP32[$n>>2]|0;
    $160 = HEAP32[$0>>2]|0;
    $161 = ((($160)) + 60|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = (($162) + (($159*40)|0)|0);
    $164 = ((($163)) + 8|0);
    HEAP16[$164>>1] = 1;
    $165 = HEAP32[$db>>2]|0;
    $166 = HEAP32[$n>>2]|0;
    $167 = HEAP32[$0>>2]|0;
    $168 = ((($167)) + 60|0);
    $169 = HEAP32[$168>>2]|0;
    $170 = (($169) + (($166*40)|0)|0);
    $171 = ((($170)) + 32|0);
    HEAP32[$171>>2] = $165;
    $172 = HEAP32[$n>>2]|0;
    $173 = (($172) + 1)|0;
    HEAP32[$n>>2] = $173;
   }
  }
 } while(0);
 $174 = HEAP32[$1>>2]|0;
 $175 = ((($174)) + 448|0);
 $176 = HEAP32[$175>>2]|0;
 $177 = $176&65535;
 $178 = HEAP32[$0>>2]|0;
 $179 = ((($178)) + 70|0);
 HEAP16[$179>>1] = $177;
 $180 = HEAP32[$1>>2]|0;
 $181 = ((($180)) + 476|0);
 $182 = HEAP32[$181>>2]|0;
 $183 = HEAP32[$0>>2]|0;
 $184 = ((($183)) + 64|0);
 HEAP32[$184>>2] = $182;
 $185 = HEAP32[$1>>2]|0;
 $186 = ((($185)) + 448|0);
 HEAP32[$186>>2] = 0;
 $187 = HEAP32[$1>>2]|0;
 $188 = ((($187)) + 476|0);
 HEAP32[$188>>2] = 0;
 $189 = HEAP32[$0>>2]|0;
 $190 = ((($189)) + 8|0);
 $191 = HEAP32[$190>>2]|0;
 $192 = ($191|0)!=(0|0);
 if (!($192)) {
  $218 = HEAP32[$1>>2]|0;
  $219 = ((($218)) + 453|0);
  $220 = HEAP8[$219>>0]|0;
  $221 = $220&255;
  $222 = HEAP32[$0>>2]|0;
  $223 = ((($222)) + 88|0);
  $224 = $221&65535;
  $225 = HEAP16[$223>>1]|0;
  $226 = $224 & 3;
  $227 = $225 & -4;
  $228 = $227 | $226;
  HEAP16[$223>>1] = $228;
  $229 = HEAP32[$0>>2]|0;
  _sqlite3VdbeRewind($229);
  STACKTOP = sp;return;
 }
 $193 = HEAP32[$0>>2]|0;
 $194 = ((($193)) + 8|0);
 $195 = HEAP32[$194>>2]|0;
 $196 = ((($195)) + -40|0);
 HEAP32[$194>>2] = $196;
 $197 = HEAP32[$nMem>>2]|0;
 $198 = HEAP32[$0>>2]|0;
 $199 = ((($198)) + 28|0);
 HEAP32[$199>>2] = $197;
 HEAP32[$n>>2] = 1;
 while(1) {
  $200 = HEAP32[$n>>2]|0;
  $201 = HEAP32[$nMem>>2]|0;
  $202 = ($200|0)<=($201|0);
  if (!($202)) {
   break;
  }
  $203 = HEAP32[$n>>2]|0;
  $204 = HEAP32[$0>>2]|0;
  $205 = ((($204)) + 8|0);
  $206 = HEAP32[$205>>2]|0;
  $207 = (($206) + (($203*40)|0)|0);
  $208 = ((($207)) + 8|0);
  HEAP16[$208>>1] = 128;
  $209 = HEAP32[$db>>2]|0;
  $210 = HEAP32[$n>>2]|0;
  $211 = HEAP32[$0>>2]|0;
  $212 = ((($211)) + 8|0);
  $213 = HEAP32[$212>>2]|0;
  $214 = (($213) + (($210*40)|0)|0);
  $215 = ((($214)) + 32|0);
  HEAP32[$215>>2] = $209;
  $216 = HEAP32[$n>>2]|0;
  $217 = (($216) + 1)|0;
  HEAP32[$n>>2] = $217;
 }
 $218 = HEAP32[$1>>2]|0;
 $219 = ((($218)) + 453|0);
 $220 = HEAP8[$219>>0]|0;
 $221 = $220&255;
 $222 = HEAP32[$0>>2]|0;
 $223 = ((($222)) + 88|0);
 $224 = $221&65535;
 $225 = HEAP16[$223>>1]|0;
 $226 = $224 & 3;
 $227 = $225 & -4;
 $228 = $227 | $226;
 HEAP16[$223>>1] = $228;
 $229 = HEAP32[$0>>2]|0;
 _sqlite3VdbeRewind($229);
 STACKTOP = sp;return;
}
function _blobSeekToRow($p,$0,$1,$pzErr) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pzErr = $pzErr|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pC = 0, $rc = 0;
 var $type = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 52|0;
 $3 = sp;
 $4 = sp + 48|0;
 $rc = sp + 44|0;
 $zErr = sp + 40|0;
 $v = sp + 36|0;
 $pC = sp + 32|0;
 $type = sp + 28|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pzErr;
 HEAP32[$zErr>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = $3;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$v>>2]|0;
 $19 = ((($18)) + 60|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20;
 $22 = $21;
 HEAP32[$22>>2] = $14;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $17;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 20|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (_sqlite3_step($27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)==(100);
 do {
  if ($30) {
   $31 = HEAP32[$v>>2]|0;
   $32 = ((($31)) + 56|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pC>>2] = $34;
   $35 = HEAP32[$2>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$pC>>2]|0;
   $39 = ((($38)) + 80|0);
   $40 = (($39) + ($37<<2)|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$type>>2] = $41;
   $42 = HEAP32[$type>>2]|0;
   $43 = ($42>>>0)<(12);
   $44 = HEAP32[$2>>2]|0;
   if (!($43)) {
    $59 = ((($44)) + 12|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$pC>>2]|0;
    $62 = ((($61)) + 12|0);
    $63 = HEAP16[$62>>1]|0;
    $64 = $63 << 16 >> 16;
    $65 = (($60) + ($64))|0;
    $66 = HEAP32[$pC>>2]|0;
    $67 = ((($66)) + 80|0);
    $68 = (($67) + ($65<<2)|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = ((($70)) + 8|0);
    HEAP32[$71>>2] = $69;
    $72 = HEAP32[$type>>2]|0;
    $73 = (_sqlite3VdbeSerialTypeLen($72)|0);
    $74 = HEAP32[$2>>2]|0;
    $75 = ((($74)) + 4|0);
    HEAP32[$75>>2] = $73;
    $76 = HEAP32[$pC>>2]|0;
    $77 = ((($76)) + 16|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = HEAP32[$2>>2]|0;
    $80 = ((($79)) + 16|0);
    HEAP32[$80>>2] = $78;
    $81 = HEAP32[$2>>2]|0;
    $82 = ((($81)) + 16|0);
    $83 = HEAP32[$82>>2]|0;
    _sqlite3BtreeIncrblobCursor($83);
    break;
   }
   $45 = ((($44)) + 24|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$type>>2]|0;
   $48 = ($47|0)==(0);
   if ($48) {
    $52 = 25472;
   } else {
    $49 = HEAP32[$type>>2]|0;
    $50 = ($49|0)==(7);
    $51 = $50 ? 25477 : 25482;
    $52 = $51;
   }
   HEAP32[$vararg_buffer>>2] = $52;
   $53 = (_sqlite3MPrintf($46,25443,$vararg_buffer)|0);
   HEAP32[$zErr>>2] = $53;
   HEAP32[$rc>>2] = 1;
   $54 = HEAP32[$2>>2]|0;
   $55 = ((($54)) + 20|0);
   $56 = HEAP32[$55>>2]|0;
   (_sqlite3_finalize($56)|0);
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 20|0);
   HEAP32[$58>>2] = 0;
  }
 } while(0);
 $84 = HEAP32[$rc>>2]|0;
 $85 = ($84|0)==(100);
 if ($85) {
  HEAP32[$rc>>2] = 0;
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $86 = HEAP32[$2>>2]|0;
 $87 = ((($86)) + 20|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = ($88|0)!=(0|0);
 if (!($89)) {
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $90 = HEAP32[$2>>2]|0;
 $91 = ((($90)) + 20|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = (_sqlite3_finalize($92)|0);
 HEAP32[$rc>>2] = $93;
 $94 = HEAP32[$2>>2]|0;
 $95 = ((($94)) + 20|0);
 HEAP32[$95>>2] = 0;
 $96 = HEAP32[$rc>>2]|0;
 $97 = ($96|0)==(0);
 $98 = HEAP32[$2>>2]|0;
 $99 = ((($98)) + 24|0);
 $100 = HEAP32[$99>>2]|0;
 if ($97) {
  $101 = $3;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = (($101) + 4)|0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = $vararg_buffer1;
  $108 = $107;
  HEAP32[$108>>2] = $103;
  $109 = (($107) + 4)|0;
  $110 = $109;
  HEAP32[$110>>2] = $106;
  $111 = (_sqlite3MPrintf($100,25490,$vararg_buffer1)|0);
  HEAP32[$zErr>>2] = $111;
  HEAP32[$rc>>2] = 1;
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 } else {
  $112 = HEAP32[$2>>2]|0;
  $113 = ((($112)) + 24|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = (_sqlite3_errmsg($114)|0);
  HEAP32[$vararg_buffer4>>2] = $115;
  $116 = (_sqlite3MPrintf($100,17521,$vararg_buffer4)|0);
  HEAP32[$zErr>>2] = $116;
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 return (0)|0;
}
function _sqlite3ParserReset($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 120|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3DbFree($5,$8);
 $9 = HEAP32[$db>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 324|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3ExprListDelete($9,$12);
 $13 = HEAP32[$db>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 24|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = HEAP32[$db>>2]|0;
  $20 = ((($19)) + 248|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) - ($18))|0;
  HEAP32[$20>>2] = $22;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 24|0);
 HEAP8[$24>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3_blob_close($pBlob) {
 $pBlob = $pBlob|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pBlob;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 24|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$db>>2] = $6;
  $7 = HEAP32[$p>>2]|0;
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3_finalize($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($11,$12);
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  HEAP32[$rc>>2] = 0;
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _sqlite3_blob_read($pBlob,$z,$n,$iOffset) {
 $pBlob = $pBlob|0;
 $z = $z|0;
 $n = $n|0;
 $iOffset = $iOffset|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pBlob;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $iOffset;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_blobReadWrite($4,$5,$6,$7,138)|0);
 STACKTOP = sp;return ($8|0);
}
function _blobReadWrite($pBlob,$z,$n,$iOffset,$xCall) {
 $pBlob = $pBlob|0;
 $z = $z|0;
 $n = $n|0;
 $iOffset = $iOffset|0;
 $xCall = $xCall|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $p = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pBlob;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $iOffset;
 HEAP32[$5>>2] = $xCall;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = (_sqlite3MisuseError(81441)|0);
  HEAP32[$0>>2] = $9;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$v>>2] = $15;
 $16 = HEAP32[$3>>2]|0;
 $17 = ($16|0)<(0);
 $18 = HEAP32[$4>>2]|0;
 $19 = ($18|0)<(0);
 $or$cond = $17 | $19;
 do {
  if ($or$cond) {
   label = 5;
  } else {
   $20 = HEAP32[$4>>2]|0;
   $21 = ($20|0)<(0);
   $22 = $21 << 31 >> 31;
   $23 = HEAP32[$3>>2]|0;
   $24 = ($23|0)<(0);
   $25 = $24 << 31 >> 31;
   $26 = (_i64Add(($20|0),($22|0),($23|0),($25|0))|0);
   $27 = tempRet0;
   $28 = HEAP32[$p>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = ($27|0)>($32|0);
   $34 = ($26>>>0)>($30>>>0);
   $35 = ($27|0)==($32|0);
   $36 = $35 & $34;
   $37 = $33 | $36;
   if ($37) {
    label = 5;
   } else {
    $38 = HEAP32[$v>>2]|0;
    $39 = ($38|0)==(0|0);
    if ($39) {
     HEAP32[$rc>>2] = 4;
     break;
    }
    $40 = HEAP32[$p>>2]|0;
    $41 = ((($40)) + 16|0);
    $42 = HEAP32[$41>>2]|0;
    _sqlite3BtreeEnterCursor($42);
    $43 = HEAP32[$5>>2]|0;
    $44 = HEAP32[$p>>2]|0;
    $45 = ((($44)) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$4>>2]|0;
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 8|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($47) + ($50))|0;
    $52 = HEAP32[$3>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (FUNCTION_TABLE_iiiii[$43 & 255]($46,$51,$52,$53)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)==(4);
    if ($56) {
     $57 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeFinalize($57)|0);
     $58 = HEAP32[$p>>2]|0;
     $59 = ((($58)) + 20|0);
     HEAP32[$59>>2] = 0;
     break;
    } else {
     $60 = HEAP32[$rc>>2]|0;
     $61 = HEAP32[$v>>2]|0;
     $62 = ((($61)) + 80|0);
     HEAP32[$62>>2] = $60;
     break;
    }
   }
  }
 } while(0);
 if ((label|0) == 5) {
  HEAP32[$rc>>2] = 1;
 }
 $63 = HEAP32[$db>>2]|0;
 $64 = HEAP32[$rc>>2]|0;
 _sqlite3Error($63,$64);
 $65 = HEAP32[$db>>2]|0;
 $66 = HEAP32[$rc>>2]|0;
 $67 = (_sqlite3ApiExit($65,$66)|0);
 HEAP32[$rc>>2] = $67;
 $68 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $68;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3BtreeData($pCur,$offset,$amt,$pBuf) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $offset;
 HEAP32[$3>>2] = $amt;
 HEAP32[$4>>2] = $pBuf;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 66|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 4;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 66|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)>=(3);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_btreeRestoreCursorPosition($15)|0);
  $17 = $16;
 } else {
  $17 = 0;
 }
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = HEAP32[$4>>2]|0;
  $24 = (_accessPayload($20,$21,$22,$23,0)|0);
  HEAP32[$rc>>2] = $24;
 }
 $25 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3_blob_bytes($pBlob) {
 $pBlob = $pBlob|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBlob;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $11 = 0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $11 = 0;
  STACKTOP = sp;return ($11|0);
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_blob_reopen($pBlob,$0,$1) {
 $pBlob = $pBlob|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp;
 $rc = sp + 24|0;
 $p = sp + 20|0;
 $db = sp + 16|0;
 $zErr = sp + 12|0;
 HEAP32[$3>>2] = $pBlob;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = (_sqlite3MisuseError(81515)|0);
  HEAP32[$2>>2] = $12;
  $42 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  HEAP32[$rc>>2] = 4;
 } else {
  $20 = HEAP32[$p>>2]|0;
  $21 = $4;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (_blobSeekToRow($20,$23,$26,$zErr)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$rc>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$db>>2]|0;
   $31 = HEAP32[$rc>>2]|0;
   $32 = HEAP32[$zErr>>2]|0;
   $33 = ($32|0)!=(0|0);
   $34 = $33 ? 17521 : 0;
   $35 = HEAP32[$zErr>>2]|0;
   HEAP32[$vararg_buffer>>2] = $35;
   _sqlite3ErrorWithMsg($30,$31,$34,$vararg_buffer);
   $36 = HEAP32[$db>>2]|0;
   $37 = HEAP32[$zErr>>2]|0;
   _sqlite3DbFree($36,$37);
  }
 }
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$rc>>2]|0;
 $40 = (_sqlite3ApiExit($38,$39)|0);
 HEAP32[$rc>>2] = $40;
 $41 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $41;
 $42 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3ExpirePreparedStatements($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 88|0);
  $8 = HEAP16[$7>>1]|0;
  $9 = $8 & -9;
  $10 = $9 | 8;
  HEAP16[$7>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 52|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 STACKTOP = sp;return;
}
function _sqlite3_strglob($zGlobPattern,$zString) {
 $zGlobPattern = $zGlobPattern|0;
 $zString = $zString|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $zGlobPattern;
 HEAP32[$1>>2] = $zString;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_patternCompare($2,$3,17524,91)|0);
 $5 = ($4|0)==(0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _patternCompare($zPattern,$zString,$pInfo,$matchOther) {
 $zPattern = $zPattern|0;
 $zString = $zString|0;
 $pInfo = $pInfo|0;
 $matchOther = $matchOther|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c2 = 0, $cx = 0, $invert = 0, $matchAll = 0, $matchOne = 0, $noCase = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $prior_c = 0, $seen = 0, $zEscaped = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $c = sp + 32|0;
 $c2 = sp + 28|0;
 $matchOne = sp + 24|0;
 $matchAll = sp + 20|0;
 $noCase = sp + 56|0;
 $zEscaped = sp + 16|0;
 $cx = sp + 12|0;
 $prior_c = sp + 8|0;
 $seen = sp + 4|0;
 $invert = sp;
 HEAP32[$1>>2] = $zPattern;
 HEAP32[$2>>2] = $zString;
 HEAP32[$3>>2] = $pInfo;
 HEAP32[$4>>2] = $matchOther;
 $5 = HEAP32[$3>>2]|0;
 $6 = ((($5)) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$matchOne>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$matchAll>>2] = $11;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 3|0);
 $14 = HEAP8[$13>>0]|0;
 HEAP8[$noCase>>0] = $14;
 HEAP32[$zEscaped>>2] = 0;
 L1: while(1) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)<(128);
  if ($18) {
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 1|0);
   HEAP32[$1>>2] = $20;
   $21 = HEAP8[$19>>0]|0;
   $22 = $21&255;
   $24 = $22;
  } else {
   $23 = (_sqlite3Utf8Read($1)|0);
   $24 = $23;
  }
  HEAP32[$c>>2] = $24;
  $25 = ($24|0)!=(0);
  if (!($25)) {
   label = 83;
   break;
  }
  $26 = HEAP32[$c>>2]|0;
  $27 = HEAP32[$matchAll>>2]|0;
  $28 = ($26|0)==($27|0);
  if ($28) {
   label = 7;
   break;
  }
  $145 = HEAP32[$c>>2]|0;
  $146 = HEAP32[$4>>2]|0;
  $147 = ($145|0)==($146|0);
  do {
   if ($147) {
    $148 = HEAP32[$3>>2]|0;
    $149 = ((($148)) + 2|0);
    $150 = HEAP8[$149>>0]|0;
    $151 = $150&255;
    $152 = ($151|0)==(0);
    if ($152) {
     $153 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c>>2] = $153;
     $154 = HEAP32[$c>>2]|0;
     $155 = ($154|0)==(0);
     if ($155) {
      label = 49;
      break L1;
     }
     $156 = HEAP32[$1>>2]|0;
     HEAP32[$zEscaped>>2] = $156;
     break;
    }
    HEAP32[$prior_c>>2] = 0;
    HEAP32[$seen>>2] = 0;
    HEAP32[$invert>>2] = 0;
    $157 = (_sqlite3Utf8Read($2)|0);
    HEAP32[$c>>2] = $157;
    $158 = HEAP32[$c>>2]|0;
    $159 = ($158|0)==(0);
    if ($159) {
     label = 52;
     break L1;
    }
    $160 = (_sqlite3Utf8Read($1)|0);
    HEAP32[$c2>>2] = $160;
    $161 = HEAP32[$c2>>2]|0;
    $162 = ($161|0)==(94);
    if ($162) {
     HEAP32[$invert>>2] = 1;
     $163 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $163;
    }
    $164 = HEAP32[$c2>>2]|0;
    $165 = ($164|0)==(93);
    if ($165) {
     $166 = HEAP32[$c>>2]|0;
     $167 = ($166|0)==(93);
     if ($167) {
      HEAP32[$seen>>2] = 1;
     }
     $168 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $168;
    }
    while(1) {
     $169 = HEAP32[$c2>>2]|0;
     $170 = ($169|0)!=(0);
     $171 = HEAP32[$c2>>2]|0;
     $172 = ($171|0)!=(93);
     $173 = $170 ? $172 : 0;
     $174 = HEAP32[$c2>>2]|0;
     if (!($173)) {
      break;
     }
     $175 = ($174|0)==(45);
     if ($175) {
      $176 = HEAP32[$1>>2]|0;
      $177 = HEAP8[$176>>0]|0;
      $178 = $177&255;
      $179 = ($178|0)!=(93);
      if ($179) {
       $180 = HEAP32[$1>>2]|0;
       $181 = HEAP8[$180>>0]|0;
       $182 = $181&255;
       $183 = ($182|0)!=(0);
       $184 = HEAP32[$prior_c>>2]|0;
       $185 = ($184>>>0)>(0);
       $or$cond = $183 & $185;
       if ($or$cond) {
        $186 = (_sqlite3Utf8Read($1)|0);
        HEAP32[$c2>>2] = $186;
        $187 = HEAP32[$c>>2]|0;
        $188 = HEAP32[$prior_c>>2]|0;
        $189 = ($187>>>0)>=($188>>>0);
        if ($189) {
         $190 = HEAP32[$c>>2]|0;
         $191 = HEAP32[$c2>>2]|0;
         $192 = ($190>>>0)<=($191>>>0);
         if ($192) {
          HEAP32[$seen>>2] = 1;
         }
        }
        HEAP32[$prior_c>>2] = 0;
       } else {
        label = 67;
       }
      } else {
       label = 67;
      }
     } else {
      label = 67;
     }
     if ((label|0) == 67) {
      label = 0;
      $193 = HEAP32[$c>>2]|0;
      $194 = HEAP32[$c2>>2]|0;
      $195 = ($193|0)==($194|0);
      if ($195) {
       HEAP32[$seen>>2] = 1;
      }
      $196 = HEAP32[$c2>>2]|0;
      HEAP32[$prior_c>>2] = $196;
     }
     $197 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $197;
    }
    $198 = ($174|0)==(0);
    if ($198) {
     label = 73;
     break L1;
    }
    $199 = HEAP32[$seen>>2]|0;
    $200 = HEAP32[$invert>>2]|0;
    $201 = $199 ^ $200;
    $202 = ($201|0)==(0);
    if ($202) {
     label = 73;
     break L1;
    } else {
     continue L1;
    }
   }
  } while(0);
  $203 = HEAP32[$2>>2]|0;
  $204 = HEAP8[$203>>0]|0;
  $205 = $204&255;
  $206 = ($205|0)<(128);
  if ($206) {
   $207 = HEAP32[$2>>2]|0;
   $208 = ((($207)) + 1|0);
   HEAP32[$2>>2] = $208;
   $209 = HEAP8[$207>>0]|0;
   $210 = $209&255;
   $212 = $210;
  } else {
   $211 = (_sqlite3Utf8Read($2)|0);
   $212 = $211;
  }
  HEAP32[$c2>>2] = $212;
  $213 = HEAP32[$c>>2]|0;
  $214 = HEAP32[$c2>>2]|0;
  $215 = ($213|0)==($214|0);
  if ($215) {
   continue;
  }
  $216 = HEAP8[$noCase>>0]|0;
  $217 = $216&255;
  $218 = ($217|0)!=(0);
  $219 = HEAP32[$c>>2]|0;
  $220 = ($219>>>0)<(128);
  $or$cond3 = $218 & $220;
  $221 = HEAP32[$c2>>2]|0;
  $222 = ($221>>>0)<(128);
  $or$cond5 = $or$cond3 & $222;
  if ($or$cond5) {
   $223 = HEAP32[$c>>2]|0;
   $224 = $223&255;
   $225 = $224&255;
   $226 = (16974 + ($225)|0);
   $227 = HEAP8[$226>>0]|0;
   $228 = $227&255;
   $229 = HEAP32[$c2>>2]|0;
   $230 = $229&255;
   $231 = $230&255;
   $232 = (16974 + ($231)|0);
   $233 = HEAP8[$232>>0]|0;
   $234 = $233&255;
   $235 = ($228|0)==($234|0);
   if ($235) {
    continue;
   }
  }
  $236 = HEAP32[$c>>2]|0;
  $237 = HEAP32[$matchOne>>2]|0;
  $238 = ($236|0)==($237|0);
  if (!($238)) {
   label = 82;
   break;
  }
  $239 = HEAP32[$1>>2]|0;
  $240 = HEAP32[$zEscaped>>2]|0;
  $241 = ($239|0)!=($240|0);
  $242 = HEAP32[$c2>>2]|0;
  $243 = ($242|0)!=(0);
  $or$cond7 = $241 & $243;
  if (!($or$cond7)) {
   label = 82;
   break;
  }
 }
 if ((label|0) == 7) {
  while(1) {
   label = 0;
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = ($31|0)<(128);
   if ($32) {
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 1|0);
    HEAP32[$1>>2] = $34;
    $35 = HEAP8[$33>>0]|0;
    $36 = $35&255;
    $38 = $36;
   } else {
    $37 = (_sqlite3Utf8Read($1)|0);
    $38 = $37;
   }
   HEAP32[$c>>2] = $38;
   $39 = HEAP32[$matchAll>>2]|0;
   $40 = ($38|0)==($39|0);
   if ($40) {
    $250 = 1;
   } else {
    $41 = HEAP32[$c>>2]|0;
    $42 = HEAP32[$matchOne>>2]|0;
    $43 = ($41|0)==($42|0);
    $250 = $43;
   }
   $44 = HEAP32[$c>>2]|0;
   if (!($250)) {
    break;
   }
   $45 = HEAP32[$matchOne>>2]|0;
   $46 = ($44|0)==($45|0);
   if (!($46)) {
    label = 7;
    continue;
   }
   $47 = (_sqlite3Utf8Read($2)|0);
   $48 = ($47|0)==(0);
   if ($48) {
    label = 15;
    break;
   } else {
    label = 7;
   }
  }
  if ((label|0) == 15) {
   HEAP32[$0>>2] = 0;
   $249 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($249|0);
  }
  $49 = ($44|0)==(0);
  if ($49) {
   HEAP32[$0>>2] = 1;
   $249 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($249|0);
  }
  $50 = HEAP32[$c>>2]|0;
  $51 = HEAP32[$4>>2]|0;
  $52 = ($50|0)==($51|0);
  do {
   if ($52) {
    $53 = HEAP32[$3>>2]|0;
    $54 = ((($53)) + 2|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = ($56|0)==(0);
    if ($57) {
     $58 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c>>2] = $58;
     $59 = HEAP32[$c>>2]|0;
     $60 = ($59|0)==(0);
     if (!($60)) {
      break;
     }
     HEAP32[$0>>2] = 0;
     $249 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($249|0);
    }
    L83: while(1) {
     $61 = HEAP32[$2>>2]|0;
     $62 = HEAP8[$61>>0]|0;
     $63 = $62&255;
     $64 = ($63|0)!=(0);
     if ($64) {
      $65 = HEAP32[$1>>2]|0;
      $66 = ((($65)) + -1|0);
      $67 = HEAP32[$2>>2]|0;
      $68 = HEAP32[$3>>2]|0;
      $69 = HEAP32[$4>>2]|0;
      $70 = (_patternCompare($66,$67,$68,$69)|0);
      $71 = ($70|0)==(0);
      $251 = $71;
     } else {
      $251 = 0;
     }
     $72 = HEAP32[$2>>2]|0;
     if (!($251)) {
      break;
     }
     $73 = ((($72)) + 1|0);
     HEAP32[$2>>2] = $73;
     $74 = HEAP8[$72>>0]|0;
     $75 = $74&255;
     $76 = ($75|0)>=(192);
     if (!($76)) {
      continue;
     }
     while(1) {
      $77 = HEAP32[$2>>2]|0;
      $78 = HEAP8[$77>>0]|0;
      $79 = $78&255;
      $80 = $79 & 192;
      $81 = ($80|0)==(128);
      if (!($81)) {
       continue L83;
      }
      $82 = HEAP32[$2>>2]|0;
      $83 = ((($82)) + 1|0);
      HEAP32[$2>>2] = $83;
     }
    }
    $84 = HEAP8[$72>>0]|0;
    $85 = $84&255;
    $86 = ($85|0)!=(0);
    $87 = $86&1;
    HEAP32[$0>>2] = $87;
    $249 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($249|0);
   }
  } while(0);
  $88 = HEAP32[$c>>2]|0;
  $89 = ($88>>>0)<=(128);
  L95: do {
   if ($89) {
    $90 = HEAP8[$noCase>>0]|0;
    $91 = ($90<<24>>24)!=(0);
    $92 = HEAP32[$c>>2]|0;
    if ($91) {
     $93 = HEAP32[$c>>2]|0;
     $94 = $93&255;
     $95 = $94&255;
     $96 = (17533 + ($95)|0);
     $97 = HEAP8[$96>>0]|0;
     $98 = $97&255;
     $99 = $98 & 32;
     $100 = $99 ^ -1;
     $101 = $92 & $100;
     HEAP32[$cx>>2] = $101;
     $102 = HEAP32[$c>>2]|0;
     $103 = $102&255;
     $104 = $103&255;
     $105 = (16974 + ($104)|0);
     $106 = HEAP8[$105>>0]|0;
     $107 = $106&255;
     HEAP32[$c>>2] = $107;
    } else {
     HEAP32[$cx>>2] = $92;
    }
    while(1) {
     $108 = HEAP32[$2>>2]|0;
     $109 = ((($108)) + 1|0);
     HEAP32[$2>>2] = $109;
     $110 = HEAP8[$108>>0]|0;
     $111 = $110&255;
     HEAP32[$c2>>2] = $111;
     $112 = ($111|0)!=(0);
     if (!($112)) {
      break L95;
     }
     $113 = HEAP32[$c2>>2]|0;
     $114 = HEAP32[$c>>2]|0;
     $115 = ($113|0)!=($114|0);
     if ($115) {
      $116 = HEAP32[$c2>>2]|0;
      $117 = HEAP32[$cx>>2]|0;
      $118 = ($116|0)!=($117|0);
      if ($118) {
       continue;
      }
     }
     $119 = HEAP32[$1>>2]|0;
     $120 = HEAP32[$2>>2]|0;
     $121 = HEAP32[$3>>2]|0;
     $122 = HEAP32[$4>>2]|0;
     $123 = (_patternCompare($119,$120,$121,$122)|0);
     $124 = ($123|0)!=(0);
     if ($124) {
      break;
     }
    }
    HEAP32[$0>>2] = 1;
    $249 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($249|0);
   } else {
    while(1) {
     $125 = HEAP32[$2>>2]|0;
     $126 = HEAP8[$125>>0]|0;
     $127 = $126&255;
     $128 = ($127|0)<(128);
     if ($128) {
      $129 = HEAP32[$2>>2]|0;
      $130 = ((($129)) + 1|0);
      HEAP32[$2>>2] = $130;
      $131 = HEAP8[$129>>0]|0;
      $132 = $131&255;
      $134 = $132;
     } else {
      $133 = (_sqlite3Utf8Read($2)|0);
      $134 = $133;
     }
     HEAP32[$c2>>2] = $134;
     $135 = ($134|0)!=(0);
     if (!($135)) {
      break L95;
     }
     $136 = HEAP32[$c2>>2]|0;
     $137 = HEAP32[$c>>2]|0;
     $138 = ($136|0)!=($137|0);
     if ($138) {
      continue;
     }
     $139 = HEAP32[$1>>2]|0;
     $140 = HEAP32[$2>>2]|0;
     $141 = HEAP32[$3>>2]|0;
     $142 = HEAP32[$4>>2]|0;
     $143 = (_patternCompare($139,$140,$141,$142)|0);
     $144 = ($143|0)!=(0);
     if ($144) {
      break;
     }
    }
    HEAP32[$0>>2] = 1;
    $249 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($249|0);
   }
  } while(0);
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 49) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 52) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 73) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 82) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 83) {
  $244 = HEAP32[$2>>2]|0;
  $245 = HEAP8[$244>>0]|0;
  $246 = $245&255;
  $247 = ($246|0)==(0);
  $248 = $247&1;
  HEAP32[$0>>2] = $248;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 return (0)|0;
}
function _sqlite3_strlike($zPattern,$zStr,$esc) {
 $zPattern = $zPattern|0;
 $zStr = $zStr|0;
 $esc = $esc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $zPattern;
 HEAP32[$1>>2] = $zStr;
 HEAP32[$2>>2] = $esc;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_patternCompare($3,$4,17528,$5)|0);
 $7 = ($6|0)==(0);
 $8 = $7&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_exec($db,$zSql,$xCallback,$pArg,$pzErrMsg) {
 $db = $db|0;
 $zSql = $zSql|0;
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azCols = 0, $azVals = 0, $callbackIsInit = 0, $i = 0, $nCol = 0, $nErrMsg = 0, $or$cond = 0, $or$cond3 = 0;
 var $pStmt = 0, $rc = 0, $zLeftover = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $rc = sp + 32|0;
 $zLeftover = sp + 28|0;
 $pStmt = sp + 24|0;
 $azCols = sp + 20|0;
 $callbackIsInit = sp + 16|0;
 $nCol = sp + 12|0;
 $azVals = sp + 8|0;
 $i = sp + 4|0;
 $nErrMsg = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $xCallback;
 HEAP32[$4>>2] = $pArg;
 HEAP32[$5>>2] = $pzErrMsg;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 HEAP32[$azCols>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3SafetyCheckOk($6)|0);
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = (_sqlite3MisuseError(105510)|0);
  HEAP32[$0>>2] = $9;
  $151 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($151|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$2>>2] = 17532;
 }
 $12 = HEAP32[$1>>2]|0;
 _sqlite3Error($12,0);
 L8: while(1) {
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   break;
  }
  HEAP32[$azVals>>2] = 0;
  HEAP32[$pStmt>>2] = 0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (_sqlite3_prepare_v2($19,$20,-1,$pStmt,$zLeftover)|0);
  HEAP32[$rc>>2] = $21;
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)!=(0);
  if ($23) {
   continue;
  }
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   $26 = HEAP32[$zLeftover>>2]|0;
   HEAP32[$2>>2] = $26;
   continue;
  }
  HEAP32[$callbackIsInit>>2] = 0;
  $27 = HEAP32[$pStmt>>2]|0;
  $28 = (_sqlite3_column_count($27)|0);
  HEAP32[$nCol>>2] = $28;
  while(1) {
   $29 = HEAP32[$pStmt>>2]|0;
   $30 = (_sqlite3_step($29)|0);
   HEAP32[$rc>>2] = $30;
   $31 = HEAP32[$3>>2]|0;
   $32 = ($31|0)!=(0|0);
   do {
    if ($32) {
     $33 = HEAP32[$rc>>2]|0;
     $34 = (100)==($33|0);
     if (!($34)) {
      $35 = HEAP32[$rc>>2]|0;
      $36 = (101)!=($35|0);
      $37 = HEAP32[$callbackIsInit>>2]|0;
      $38 = ($37|0)!=(0);
      $or$cond = $36 | $38;
      if ($or$cond) {
       break;
      }
      $39 = HEAP32[$1>>2]|0;
      $40 = ((($39)) + 24|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = $41 & 256;
      $43 = ($42|0)!=(0);
      if (!($43)) {
       break;
      }
     }
     $44 = HEAP32[$callbackIsInit>>2]|0;
     $45 = ($44|0)!=(0);
     if (!($45)) {
      $46 = HEAP32[$1>>2]|0;
      $47 = HEAP32[$nCol>>2]|0;
      $48 = $47<<1;
      $49 = $48<<2;
      $50 = (($49) + 1)|0;
      $51 = (_sqlite3DbMallocZero($46,$50,0)|0);
      HEAP32[$azCols>>2] = $51;
      $52 = HEAP32[$azCols>>2]|0;
      $53 = ($52|0)==(0|0);
      if ($53) {
       break L8;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $54 = HEAP32[$i>>2]|0;
       $55 = HEAP32[$nCol>>2]|0;
       $56 = ($54|0)<($55|0);
       if (!($56)) {
        break;
       }
       $57 = HEAP32[$pStmt>>2]|0;
       $58 = HEAP32[$i>>2]|0;
       $59 = (_sqlite3_column_name($57,$58)|0);
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$azCols>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       HEAP32[$62>>2] = $59;
       $63 = HEAP32[$i>>2]|0;
       $64 = (($63) + 1)|0;
       HEAP32[$i>>2] = $64;
      }
      HEAP32[$callbackIsInit>>2] = 1;
     }
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)==(100);
     L32: do {
      if ($66) {
       $67 = HEAP32[$nCol>>2]|0;
       $68 = HEAP32[$azCols>>2]|0;
       $69 = (($68) + ($67<<2)|0);
       HEAP32[$azVals>>2] = $69;
       HEAP32[$i>>2] = 0;
       while(1) {
        $70 = HEAP32[$i>>2]|0;
        $71 = HEAP32[$nCol>>2]|0;
        $72 = ($70|0)<($71|0);
        if (!($72)) {
         break L32;
        }
        $73 = HEAP32[$pStmt>>2]|0;
        $74 = HEAP32[$i>>2]|0;
        $75 = (_sqlite3_column_text($73,$74)|0);
        $76 = HEAP32[$i>>2]|0;
        $77 = HEAP32[$azVals>>2]|0;
        $78 = (($77) + ($76<<2)|0);
        HEAP32[$78>>2] = $75;
        $79 = HEAP32[$i>>2]|0;
        $80 = HEAP32[$azVals>>2]|0;
        $81 = (($80) + ($79<<2)|0);
        $82 = HEAP32[$81>>2]|0;
        $83 = ($82|0)!=(0|0);
        if (!($83)) {
         $84 = HEAP32[$pStmt>>2]|0;
         $85 = HEAP32[$i>>2]|0;
         $86 = (_sqlite3_column_type($84,$85)|0);
         $87 = ($86|0)!=(5);
         if ($87) {
          label = 27;
          break L8;
         }
        }
        $89 = HEAP32[$i>>2]|0;
        $90 = (($89) + 1)|0;
        HEAP32[$i>>2] = $90;
       }
      }
     } while(0);
     $91 = HEAP32[$3>>2]|0;
     $92 = HEAP32[$4>>2]|0;
     $93 = HEAP32[$nCol>>2]|0;
     $94 = HEAP32[$azVals>>2]|0;
     $95 = HEAP32[$azCols>>2]|0;
     $96 = (FUNCTION_TABLE_iiiii[$91 & 255]($92,$93,$94,$95)|0);
     $97 = ($96|0)!=(0);
     if ($97) {
      label = 30;
      break L8;
     }
    }
   } while(0);
   $100 = HEAP32[$rc>>2]|0;
   $101 = ($100|0)!=(100);
   if ($101) {
    break;
   }
  }
  $102 = HEAP32[$pStmt>>2]|0;
  $103 = (_sqlite3VdbeFinalize($102)|0);
  HEAP32[$rc>>2] = $103;
  HEAP32[$pStmt>>2] = 0;
  $104 = HEAP32[$zLeftover>>2]|0;
  HEAP32[$2>>2] = $104;
  while(1) {
   $105 = HEAP32[$2>>2]|0;
   $106 = HEAP8[$105>>0]|0;
   $107 = $106&255;
   $108 = (17533 + ($107)|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = $109&255;
   $111 = $110 & 1;
   $112 = ($111|0)!=(0);
   if (!($112)) {
    break;
   }
   $113 = HEAP32[$2>>2]|0;
   $114 = ((($113)) + 1|0);
   HEAP32[$2>>2] = $114;
  }
  $115 = HEAP32[$1>>2]|0;
  $116 = HEAP32[$azCols>>2]|0;
  _sqlite3DbFree($115,$116);
  HEAP32[$azCols>>2] = 0;
 }
 if ((label|0) == 27) {
  $88 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($88);
 }
 else if ((label|0) == 30) {
  HEAP32[$rc>>2] = 4;
  $98 = HEAP32[$pStmt>>2]|0;
  (_sqlite3VdbeFinalize($98)|0);
  HEAP32[$pStmt>>2] = 0;
  $99 = HEAP32[$1>>2]|0;
  _sqlite3Error($99,4);
 }
 $117 = HEAP32[$pStmt>>2]|0;
 $118 = ($117|0)!=(0|0);
 if ($118) {
  $119 = HEAP32[$pStmt>>2]|0;
  (_sqlite3VdbeFinalize($119)|0);
 }
 $120 = HEAP32[$1>>2]|0;
 $121 = HEAP32[$azCols>>2]|0;
 _sqlite3DbFree($120,$121);
 $122 = HEAP32[$1>>2]|0;
 $123 = HEAP32[$rc>>2]|0;
 $124 = (_sqlite3ApiExit($122,$123)|0);
 HEAP32[$rc>>2] = $124;
 $125 = HEAP32[$rc>>2]|0;
 $126 = ($125|0)!=(0);
 $127 = HEAP32[$5>>2]|0;
 $128 = ($127|0)!=(0|0);
 $or$cond3 = $126 & $128;
 do {
  if ($or$cond3) {
   $129 = HEAP32[$1>>2]|0;
   $130 = (_sqlite3_errmsg($129)|0);
   $131 = (_sqlite3Strlen30($130)|0);
   $132 = (1 + ($131))|0;
   HEAP32[$nErrMsg>>2] = $132;
   $133 = HEAP32[$nErrMsg>>2]|0;
   $134 = ($133|0)<(0);
   $135 = $134 << 31 >> 31;
   $136 = (_sqlite3Malloc($133,$135)|0);
   $137 = HEAP32[$5>>2]|0;
   HEAP32[$137>>2] = $136;
   $138 = HEAP32[$5>>2]|0;
   $139 = HEAP32[$138>>2]|0;
   $140 = ($139|0)!=(0|0);
   if ($140) {
    $141 = HEAP32[$5>>2]|0;
    $142 = HEAP32[$141>>2]|0;
    $143 = HEAP32[$1>>2]|0;
    $144 = (_sqlite3_errmsg($143)|0);
    $145 = HEAP32[$nErrMsg>>2]|0;
    _memcpy(($142|0),($144|0),($145|0))|0;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    $146 = HEAP32[$1>>2]|0;
    _sqlite3Error($146,7);
    break;
   }
  } else {
   $147 = HEAP32[$5>>2]|0;
   $148 = ($147|0)!=(0|0);
   if ($148) {
    $149 = HEAP32[$5>>2]|0;
    HEAP32[$149>>2] = 0;
   }
  }
 } while(0);
 $150 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $150;
 $151 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($151|0);
}
function _sqlite3SafetyCheckOk($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $magic = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $magic = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 do {
  if ($3) {
   _logBadConnection(18200);
   HEAP32[$0>>2] = 0;
  } else {
   $4 = HEAP32[$1>>2]|0;
   $5 = ((($4)) + 76|0);
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$magic>>2] = $6;
   $7 = HEAP32[$magic>>2]|0;
   $8 = ($7|0)!=(-1607883113);
   if (!($8)) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $9 = HEAP32[$1>>2]|0;
   $10 = (_sqlite3SafetyCheckSickOrOk($9)|0);
   $11 = ($10|0)!=(0);
   if ($11) {
    _logBadConnection(25554);
   }
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3_prepare_v2($db,$zSql,$nBytes,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $ppStmt;
 HEAP32[$4>>2] = $pzTail;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3LockAndPrepare($5,$6,$7,1,0,$8,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_errmsg($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $4 = (_sqlite3ErrStr(7)|0);
  HEAP32[$0>>2] = $4;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3SafetyCheckSickOrOk($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MisuseError(134947)|0);
  $9 = (_sqlite3ErrStr($8)|0);
  HEAP32[$0>>2] = $9;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 65|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  $14 = (_sqlite3ErrStr(7)|0);
  HEAP32[$z>>2] = $14;
 } else {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 236|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3_value_text($17)|0);
  HEAP32[$z>>2] = $18;
  $19 = HEAP32[$z>>2]|0;
  $20 = ($19|0)==(0|0);
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 52|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3ErrStr($23)|0);
   HEAP32[$z>>2] = $24;
  }
 }
 $25 = HEAP32[$z>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3_prepare($db,$zSql,$nBytes,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $ppStmt;
 HEAP32[$4>>2] = $pzTail;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3LockAndPrepare($5,$6,$7,0,0,$8,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3LockAndPrepare($db,$zSql,$nBytes,$saveSqlFlag,$pOld,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $saveSqlFlag = $saveSqlFlag|0;
 $pOld = $pOld|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 8|0;
 $7 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $nBytes;
 HEAP32[$4>>2] = $saveSqlFlag;
 HEAP32[$5>>2] = $pOld;
 HEAP32[$6>>2] = $ppStmt;
 HEAP32[$7>>2] = $pzTail;
 $8 = HEAP32[$6>>2]|0;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3SafetyCheckOk($9)|0);
 $11 = ($10|0)==(0);
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0|0);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = (_sqlite3MisuseError(110077)|0);
  HEAP32[$0>>2] = $14;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $15 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnterAll($15);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$6>>2]|0;
 $22 = HEAP32[$7>>2]|0;
 $23 = (_sqlite3Prepare($16,$17,$18,$19,$20,$21,$22)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(17);
 if ($25) {
  $26 = HEAP32[$6>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3_finalize($27)|0);
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$4>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  $33 = HEAP32[$6>>2]|0;
  $34 = HEAP32[$7>>2]|0;
  $35 = (_sqlite3Prepare($28,$29,$30,$31,$32,$33,$34)|0);
  HEAP32[$rc>>2] = $35;
 }
 $36 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3_create_module($db,$zName,$pModule,$pAux) {
 $db = $db|0;
 $zName = $zName|0;
 $pModule = $pModule|0;
 $pAux = $pAux|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pModule;
 HEAP32[$3>>2] = $pAux;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_createModule($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _createModule($db,$zName,$pModule,$pAux,$xDestroy) {
 $db = $db|0;
 $zName = $zName|0;
 $pModule = $pModule|0;
 $pAux = $pAux|0;
 $xDestroy = $xDestroy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $or$cond = 0, $pDel = 0, $pMod = 0, $rc = 0, $zCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $rc = sp + 16|0;
 $nName = sp + 12|0;
 $pMod = sp + 8|0;
 $pDel = sp + 4|0;
 $zCopy = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pModule;
 HEAP32[$3>>2] = $pAux;
 HEAP32[$4>>2] = $xDestroy;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 HEAP32[$nName>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 312|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3HashFind($8,$9)|0);
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = (_sqlite3MisuseError(118442)|0);
  HEAP32[$rc>>2] = $12;
 } else {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$nName>>2]|0;
  $15 = (20 + ($14))|0;
  $16 = (($15) + 1)|0;
  $17 = (_sqlite3DbMallocRawNN($13,$16,0)|0);
  HEAP32[$pMod>>2] = $17;
  $18 = HEAP32[$pMod>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pMod>>2]|0;
   $21 = ((($20)) + 20|0);
   HEAP32[$zCopy>>2] = $21;
   $22 = HEAP32[$zCopy>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP32[$nName>>2]|0;
   $25 = (($24) + 1)|0;
   _memcpy(($22|0),($23|0),($25|0))|0;
   $26 = HEAP32[$zCopy>>2]|0;
   $27 = HEAP32[$pMod>>2]|0;
   $28 = ((($27)) + 4|0);
   HEAP32[$28>>2] = $26;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$pMod>>2]|0;
   HEAP32[$30>>2] = $29;
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$pMod>>2]|0;
   $33 = ((($32)) + 8|0);
   HEAP32[$33>>2] = $31;
   $34 = HEAP32[$4>>2]|0;
   $35 = HEAP32[$pMod>>2]|0;
   $36 = ((($35)) + 12|0);
   HEAP32[$36>>2] = $34;
   $37 = HEAP32[$pMod>>2]|0;
   $38 = ((($37)) + 16|0);
   HEAP32[$38>>2] = 0;
   $39 = HEAP32[$0>>2]|0;
   $40 = ((($39)) + 312|0);
   $41 = HEAP32[$zCopy>>2]|0;
   $42 = HEAP32[$pMod>>2]|0;
   $43 = (_sqlite3HashInsert($40,$41,$42)|0);
   HEAP32[$pDel>>2] = $43;
   $44 = HEAP32[$pDel>>2]|0;
   $45 = ($44|0)!=(0|0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    _sqlite3OomFault($46);
    $47 = HEAP32[$0>>2]|0;
    $48 = HEAP32[$pDel>>2]|0;
    _sqlite3DbFree($47,$48);
   }
  }
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$rc>>2]|0;
 $51 = (_sqlite3ApiExit($49,$50)|0);
 HEAP32[$rc>>2] = $51;
 $52 = HEAP32[$rc>>2]|0;
 $53 = ($52|0)!=(0);
 $54 = HEAP32[$4>>2]|0;
 $55 = ($54|0)!=(0|0);
 $or$cond = $53 & $55;
 if (!($or$cond)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $56 = HEAP32[$4>>2]|0;
 $57 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$56 & 255]($57);
 $58 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _sqlite3_create_module_v2($db,$zName,$pModule,$pAux,$xDestroy) {
 $db = $db|0;
 $zName = $zName|0;
 $pModule = $pModule|0;
 $pAux = $pAux|0;
 $xDestroy = $xDestroy|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pModule;
 HEAP32[$3>>2] = $pAux;
 HEAP32[$4>>2] = $xDestroy;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_createModule($5,$6,$7,$8,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3_declare_vtab($db,$zCreateTable) {
 $db = $db|0;
 $zCreateTable = $zCreateTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pCtx = 0, $pParse = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pCtx = sp + 20|0;
 $pParse = sp + 16|0;
 $rc = sp + 12|0;
 $pTab = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zCreateTable;
 HEAP32[$rc>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 328|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pCtx>>2] = $5;
 $6 = HEAP32[$pCtx>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$pCtx>>2]|0;
  $9 = ((($8)) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   $14 = HEAP32[$pCtx>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pTab>>2] = $16;
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3DbMallocZero($17,544,0)|0);
   HEAP32[$pParse>>2] = $18;
   $19 = HEAP32[$pParse>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    HEAP32[$rc>>2] = 7;
   } else {
    $21 = HEAP32[$pParse>>2]|0;
    $22 = ((($21)) + 454|0);
    HEAP8[$22>>0] = 1;
    $23 = HEAP32[$1>>2]|0;
    $24 = HEAP32[$pParse>>2]|0;
    HEAP32[$24>>2] = $23;
    $25 = HEAP32[$pParse>>2]|0;
    $26 = ((($25)) + 428|0);
    HEAP32[$26>>2] = 1;
    $27 = HEAP32[$pParse>>2]|0;
    $28 = HEAP32[$2>>2]|0;
    $29 = (_sqlite3RunParser($27,$28,$zErr)|0);
    $30 = (0)==($29|0);
    if ($30) {
     $31 = HEAP32[$pParse>>2]|0;
     $32 = ((($31)) + 488|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)!=(0|0);
     if ($34) {
      $35 = HEAP32[$1>>2]|0;
      $36 = ((($35)) + 65|0);
      $37 = HEAP8[$36>>0]|0;
      $38 = ($37<<24>>24)!=(0);
      if ($38) {
       label = 14;
      } else {
       $39 = HEAP32[$pParse>>2]|0;
       $40 = ((($39)) + 488|0);
       $41 = HEAP32[$40>>2]|0;
       $42 = ((($41)) + 12|0);
       $43 = HEAP32[$42>>2]|0;
       $44 = ($43|0)!=(0|0);
       if ($44) {
        label = 14;
       } else {
        $45 = HEAP32[$pParse>>2]|0;
        $46 = ((($45)) + 488|0);
        $47 = HEAP32[$46>>2]|0;
        $48 = ((($47)) + 42|0);
        $49 = HEAP8[$48>>0]|0;
        $50 = $49&255;
        $51 = $50 & 16;
        $52 = ($51|0)==(0);
        if ($52) {
         $53 = HEAP32[$pTab>>2]|0;
         $54 = ((($53)) + 4|0);
         $55 = HEAP32[$54>>2]|0;
         $56 = ($55|0)!=(0|0);
         if (!($56)) {
          $57 = HEAP32[$pParse>>2]|0;
          $58 = ((($57)) + 488|0);
          $59 = HEAP32[$58>>2]|0;
          $60 = ((($59)) + 4|0);
          $61 = HEAP32[$60>>2]|0;
          $62 = HEAP32[$pTab>>2]|0;
          $63 = ((($62)) + 4|0);
          HEAP32[$63>>2] = $61;
          $64 = HEAP32[$pParse>>2]|0;
          $65 = ((($64)) + 488|0);
          $66 = HEAP32[$65>>2]|0;
          $67 = ((($66)) + 34|0);
          $68 = HEAP16[$67>>1]|0;
          $69 = HEAP32[$pTab>>2]|0;
          $70 = ((($69)) + 34|0);
          HEAP16[$70>>1] = $68;
          $71 = HEAP32[$pParse>>2]|0;
          $72 = ((($71)) + 488|0);
          $73 = HEAP32[$72>>2]|0;
          $74 = ((($73)) + 34|0);
          HEAP16[$74>>1] = 0;
          $75 = HEAP32[$pParse>>2]|0;
          $76 = ((($75)) + 488|0);
          $77 = HEAP32[$76>>2]|0;
          $78 = ((($77)) + 4|0);
          HEAP32[$78>>2] = 0;
         }
         $79 = HEAP32[$pCtx>>2]|0;
         $80 = ((($79)) + 12|0);
         HEAP32[$80>>2] = 1;
        } else {
         label = 14;
        }
       }
      }
     } else {
      label = 14;
     }
    } else {
     label = 14;
    }
    if ((label|0) == 14) {
     $81 = HEAP32[$1>>2]|0;
     $82 = HEAP32[$zErr>>2]|0;
     $83 = ($82|0)!=(0|0);
     $84 = $83 ? 17521 : 0;
     $85 = HEAP32[$zErr>>2]|0;
     HEAP32[$vararg_buffer>>2] = $85;
     _sqlite3ErrorWithMsg($81,1,$84,$vararg_buffer);
     $86 = HEAP32[$1>>2]|0;
     $87 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($86,$87);
     HEAP32[$rc>>2] = 1;
    }
    $88 = HEAP32[$pParse>>2]|0;
    $89 = ((($88)) + 454|0);
    HEAP8[$89>>0] = 0;
    $90 = HEAP32[$pParse>>2]|0;
    $91 = ((($90)) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)!=(0|0);
    if ($93) {
     $94 = HEAP32[$pParse>>2]|0;
     $95 = ((($94)) + 8|0);
     $96 = HEAP32[$95>>2]|0;
     (_sqlite3VdbeFinalize($96)|0);
    }
    $97 = HEAP32[$1>>2]|0;
    $98 = HEAP32[$pParse>>2]|0;
    $99 = ((($98)) + 488|0);
    $100 = HEAP32[$99>>2]|0;
    _sqlite3DeleteTable($97,$100);
    $101 = HEAP32[$pParse>>2]|0;
    _sqlite3ParserReset($101);
    $102 = HEAP32[$1>>2]|0;
    $103 = HEAP32[$pParse>>2]|0;
    _sqlite3DbFree($102,$103);
   }
   $104 = HEAP32[$1>>2]|0;
   $105 = HEAP32[$rc>>2]|0;
   $106 = (_sqlite3ApiExit($104,$105)|0);
   HEAP32[$rc>>2] = $106;
   $107 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $107;
   $108 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($108|0);
  }
 }
 $12 = HEAP32[$1>>2]|0;
 _sqlite3Error($12,21);
 $13 = (_sqlite3MisuseError(119136)|0);
 HEAP32[$0>>2] = $13;
 $108 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($108|0);
}
function _sqlite3RunParser($pParse,$zSql,$pzErrMsg) {
 $pParse = $pParse|0;
 $zSql = $zSql|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$byval_copy8 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $lastTokenParsed = 0, $mxSqlLen = 0, $nErr = 0, $p = 0, $p1 = 0, $pEngine = 0, $tokenType = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $$byval_copy8 = sp + 72|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $nErr = sp + 52|0;
 $i = sp + 48|0;
 $pEngine = sp + 44|0;
 $tokenType = sp + 40|0;
 $lastTokenParsed = sp + 36|0;
 $db = sp + 32|0;
 $mxSqlLen = sp + 28|0;
 $p = sp + 24|0;
 $p1 = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $pzErrMsg;
 HEAP32[$nErr>>2] = 0;
 HEAP32[$lastTokenParsed>>2] = -1;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 88|0);
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$mxSqlLen>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 148|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$db>>2]|0;
  $15 = ((($14)) + 240|0);
  HEAP32[$15>>2] = 0;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 12|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 484|0);
 HEAP32[$20>>2] = $18;
 HEAP32[$i>>2] = 0;
 $21 = (_sqlite3ParserAlloc(176)|0);
 HEAP32[$pEngine>>2] = $21;
 $22 = HEAP32[$pEngine>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($24);
  HEAP32[$0>>2] = 7;
  $262 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($262|0);
 }
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + ($25)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)!=(0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (($32) + ($31)|0);
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 508|0);
  HEAP32[$35>>2] = $33;
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (($37) + ($36)|0);
  $39 = (_sqlite3GetToken($38,$tokenType)|0);
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 508|0);
  $42 = ((($41)) + 4|0);
  HEAP32[$42>>2] = $39;
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 508|0);
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + ($46))|0;
  HEAP32[$i>>2] = $48;
  $49 = HEAP32[$i>>2]|0;
  $50 = HEAP32[$mxSqlLen>>2]|0;
  $51 = ($49|0)>($50|0);
  if ($51) {
   label = 7;
   break;
  }
  $54 = HEAP32[$tokenType>>2]|0;
  $55 = ($54|0)>=(159);
  if ($55) {
   $56 = HEAP32[$db>>2]|0;
   $57 = ((($56)) + 240|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = ($58|0)!=(0);
   if ($59) {
    label = 10;
    break;
   }
   $62 = HEAP32[$tokenType>>2]|0;
   $63 = ($62|0)==(160);
   if ($63) {
    label = 12;
    break;
   } else {
    continue;
   }
  }
  $67 = HEAP32[$tokenType>>2]|0;
  $68 = ($67|0)==(1);
  if ($68) {
   $69 = HEAP32[$i>>2]|0;
   $70 = HEAP32[$2>>2]|0;
   $71 = (($70) + ($69)|0);
   $72 = HEAP32[$1>>2]|0;
   $73 = ((($72)) + 484|0);
   HEAP32[$73>>2] = $71;
  }
  $74 = HEAP32[$pEngine>>2]|0;
  $75 = HEAP32[$tokenType>>2]|0;
  $76 = HEAP32[$1>>2]|0;
  $77 = ((($76)) + 508|0);
  $78 = HEAP32[$1>>2]|0;
  ;HEAP32[$$byval_copy8>>2]=HEAP32[$77>>2]|0;HEAP32[$$byval_copy8+4>>2]=HEAP32[$77+4>>2]|0;
  _sqlite3Parser($74,$75,$$byval_copy8,$78);
  $79 = HEAP32[$tokenType>>2]|0;
  HEAP32[$lastTokenParsed>>2] = $79;
  $80 = HEAP32[$1>>2]|0;
  $81 = ((($80)) + 12|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($82|0)!=(0);
  if ($83) {
   break;
  }
  $84 = HEAP32[$db>>2]|0;
  $85 = ((($84)) + 65|0);
  $86 = HEAP8[$85>>0]|0;
  $87 = $86&255;
  $88 = ($87|0)!=(0);
  if ($88) {
   break;
  }
 }
 if ((label|0) == 7) {
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 12|0);
  HEAP32[$53>>2] = 18;
 }
 else if ((label|0) == 10) {
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 12|0);
  HEAP32[$61>>2] = 9;
 }
 else if ((label|0) == 12) {
  $64 = HEAP32[$1>>2]|0;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 508|0);
  HEAP32[$vararg_buffer>>2] = $66;
  _sqlite3ErrorMsg($64,25719,$vararg_buffer);
 }
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 12|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = ($91|0)==(0);
 if ($92) {
  $93 = HEAP32[$db>>2]|0;
  $94 = ((($93)) + 65|0);
  $95 = HEAP8[$94>>0]|0;
  $96 = $95&255;
  $97 = ($96|0)==(0);
  if ($97) {
   $98 = HEAP32[$lastTokenParsed>>2]|0;
   $99 = ($98|0)!=(1);
   if ($99) {
    $100 = HEAP32[$pEngine>>2]|0;
    $101 = HEAP32[$1>>2]|0;
    $102 = ((($101)) + 508|0);
    $103 = HEAP32[$1>>2]|0;
    ;HEAP32[$$byval_copy8>>2]=HEAP32[$102>>2]|0;HEAP32[$$byval_copy8+4>>2]=HEAP32[$102+4>>2]|0;
    _sqlite3Parser($100,1,$$byval_copy8,$103);
    $104 = HEAP32[$i>>2]|0;
    $105 = HEAP32[$2>>2]|0;
    $106 = (($105) + ($104)|0);
    $107 = HEAP32[$1>>2]|0;
    $108 = ((($107)) + 484|0);
    HEAP32[$108>>2] = $106;
   }
   $109 = HEAP32[$1>>2]|0;
   $110 = ((($109)) + 12|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = ($111|0)==(0);
   if ($112) {
    $113 = HEAP32[$db>>2]|0;
    $114 = ((($113)) + 65|0);
    $115 = HEAP8[$114>>0]|0;
    $116 = $115&255;
    $117 = ($116|0)==(0);
    if ($117) {
     $118 = HEAP32[$pEngine>>2]|0;
     $119 = HEAP32[$1>>2]|0;
     $120 = ((($119)) + 508|0);
     $121 = HEAP32[$1>>2]|0;
     ;HEAP32[$$byval_copy8>>2]=HEAP32[$120>>2]|0;HEAP32[$$byval_copy8+4>>2]=HEAP32[$120+4>>2]|0;
     _sqlite3Parser($118,0,$$byval_copy8,$121);
    }
   }
  }
 }
 $122 = HEAP32[$pEngine>>2]|0;
 _sqlite3ParserFree($122,142);
 $123 = HEAP32[$db>>2]|0;
 $124 = ((($123)) + 65|0);
 $125 = HEAP8[$124>>0]|0;
 $126 = ($125<<24>>24)!=(0);
 if ($126) {
  $127 = HEAP32[$1>>2]|0;
  $128 = ((($127)) + 12|0);
  HEAP32[$128>>2] = 7;
 }
 $129 = HEAP32[$1>>2]|0;
 $130 = ((($129)) + 12|0);
 $131 = HEAP32[$130>>2]|0;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = HEAP32[$1>>2]|0;
  $134 = ((($133)) + 12|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = ($135|0)!=(101);
  if ($136) {
   $137 = HEAP32[$1>>2]|0;
   $138 = ((($137)) + 4|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = ($139|0)==(0|0);
   if ($140) {
    $141 = HEAP32[$db>>2]|0;
    $142 = HEAP32[$1>>2]|0;
    $143 = ((($142)) + 12|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = (_sqlite3ErrStr($144)|0);
    HEAP32[$vararg_buffer1>>2] = $145;
    $146 = (_sqlite3MPrintf($141,17521,$vararg_buffer1)|0);
    $147 = HEAP32[$1>>2]|0;
    $148 = ((($147)) + 4|0);
    HEAP32[$148>>2] = $146;
   }
  }
 }
 $149 = HEAP32[$1>>2]|0;
 $150 = ((($149)) + 4|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = ($151|0)!=(0|0);
 if ($152) {
  $153 = HEAP32[$1>>2]|0;
  $154 = ((($153)) + 4|0);
  $155 = HEAP32[$154>>2]|0;
  $156 = HEAP32[$3>>2]|0;
  HEAP32[$156>>2] = $155;
  $157 = HEAP32[$1>>2]|0;
  $158 = ((($157)) + 12|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = HEAP32[$3>>2]|0;
  $161 = HEAP32[$160>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $161;
  _sqlite3_log($159,17521,$vararg_buffer4);
  $162 = HEAP32[$1>>2]|0;
  $163 = ((($162)) + 4|0);
  HEAP32[$163>>2] = 0;
  $164 = HEAP32[$nErr>>2]|0;
  $165 = (($164) + 1)|0;
  HEAP32[$nErr>>2] = $165;
 }
 $166 = HEAP32[$1>>2]|0;
 $167 = ((($166)) + 8|0);
 $168 = HEAP32[$167>>2]|0;
 $169 = ($168|0)!=(0|0);
 if ($169) {
  $170 = HEAP32[$1>>2]|0;
  $171 = ((($170)) + 68|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = ($172|0)>(0);
  if ($173) {
   $174 = HEAP32[$1>>2]|0;
   $175 = ((($174)) + 18|0);
   $176 = HEAP8[$175>>0]|0;
   $177 = $176&255;
   $178 = ($177|0)==(0);
   if ($178) {
    $179 = HEAP32[$1>>2]|0;
    $180 = ((($179)) + 8|0);
    $181 = HEAP32[$180>>2]|0;
    _sqlite3VdbeDelete($181);
    $182 = HEAP32[$1>>2]|0;
    $183 = ((($182)) + 8|0);
    HEAP32[$183>>2] = 0;
   }
  }
 }
 $184 = HEAP32[$1>>2]|0;
 $185 = ((($184)) + 18|0);
 $186 = HEAP8[$185>>0]|0;
 $187 = $186&255;
 $188 = ($187|0)==(0);
 if ($188) {
  $189 = HEAP32[$db>>2]|0;
  $190 = HEAP32[$1>>2]|0;
  $191 = ((($190)) + 408|0);
  $192 = HEAP32[$191>>2]|0;
  _sqlite3DbFree($189,$192);
  $193 = HEAP32[$1>>2]|0;
  $194 = ((($193)) + 408|0);
  HEAP32[$194>>2] = 0;
  $195 = HEAP32[$1>>2]|0;
  $196 = ((($195)) + 404|0);
  HEAP32[$196>>2] = 0;
 }
 $197 = HEAP32[$1>>2]|0;
 $198 = ((($197)) + 524|0);
 $199 = HEAP32[$198>>2]|0;
 _sqlite3_free($199);
 $200 = HEAP32[$1>>2]|0;
 $201 = ((($200)) + 454|0);
 $202 = HEAP8[$201>>0]|0;
 $203 = ($202<<24>>24)!=(0);
 if (!($203)) {
  $204 = HEAP32[$db>>2]|0;
  $205 = HEAP32[$1>>2]|0;
  $206 = ((($205)) + 488|0);
  $207 = HEAP32[$206>>2]|0;
  _sqlite3DeleteTable($204,$207);
 }
 $208 = HEAP32[$db>>2]|0;
 $209 = HEAP32[$1>>2]|0;
 $210 = ((($209)) + 540|0);
 $211 = HEAP32[$210>>2]|0;
 _sqlite3WithDelete($208,$211);
 $212 = HEAP32[$db>>2]|0;
 $213 = HEAP32[$1>>2]|0;
 $214 = ((($213)) + 492|0);
 $215 = HEAP32[$214>>2]|0;
 _sqlite3DeleteTrigger($212,$215);
 $216 = HEAP32[$1>>2]|0;
 $217 = ((($216)) + 448|0);
 $218 = HEAP32[$217>>2]|0;
 $219 = (($218) - 1)|0;
 HEAP32[$i>>2] = $219;
 while(1) {
  $220 = HEAP32[$i>>2]|0;
  $221 = ($220|0)>=(0);
  $222 = HEAP32[$db>>2]|0;
  if (!($221)) {
   break;
  }
  $223 = HEAP32[$i>>2]|0;
  $224 = HEAP32[$1>>2]|0;
  $225 = ((($224)) + 476|0);
  $226 = HEAP32[$225>>2]|0;
  $227 = (($226) + ($223<<2)|0);
  $228 = HEAP32[$227>>2]|0;
  _sqlite3DbFree($222,$228);
  $229 = HEAP32[$i>>2]|0;
  $230 = (($229) + -1)|0;
  HEAP32[$i>>2] = $230;
 }
 $231 = HEAP32[$1>>2]|0;
 $232 = ((($231)) + 476|0);
 $233 = HEAP32[$232>>2]|0;
 _sqlite3DbFree($222,$233);
 while(1) {
  $234 = HEAP32[$1>>2]|0;
  $235 = ((($234)) + 412|0);
  $236 = HEAP32[$235>>2]|0;
  $237 = ($236|0)!=(0|0);
  if (!($237)) {
   break;
  }
  $238 = HEAP32[$1>>2]|0;
  $239 = ((($238)) + 412|0);
  $240 = HEAP32[$239>>2]|0;
  HEAP32[$p>>2] = $240;
  $241 = HEAP32[$p>>2]|0;
  $242 = HEAP32[$241>>2]|0;
  $243 = HEAP32[$1>>2]|0;
  $244 = ((($243)) + 412|0);
  HEAP32[$244>>2] = $242;
  $245 = HEAP32[$db>>2]|0;
  $246 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($245,$246);
 }
 while(1) {
  $247 = HEAP32[$1>>2]|0;
  $248 = ((($247)) + 528|0);
  $249 = HEAP32[$248>>2]|0;
  $250 = ($249|0)!=(0|0);
  if (!($250)) {
   break;
  }
  $251 = HEAP32[$1>>2]|0;
  $252 = ((($251)) + 528|0);
  $253 = HEAP32[$252>>2]|0;
  HEAP32[$p1>>2] = $253;
  $254 = HEAP32[$p1>>2]|0;
  $255 = ((($254)) + 68|0);
  $256 = HEAP32[$255>>2]|0;
  $257 = HEAP32[$1>>2]|0;
  $258 = ((($257)) + 528|0);
  HEAP32[$258>>2] = $256;
  $259 = HEAP32[$db>>2]|0;
  $260 = HEAP32[$p1>>2]|0;
  _sqlite3DeleteTable($259,$260);
 }
 $261 = HEAP32[$nErr>>2]|0;
 HEAP32[$0>>2] = $261;
 $262 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($262|0);
}
function _sqlite3_vtab_on_conflict($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 70|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = (($4) - 1)|0;
 $6 = (17789 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_vtab_config($db,$op,$varargs) {
 $db = $db|0;
 $op = $op|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $cond = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$1>>2]|0;
 $cond = ($2|0)==(1);
 do {
  if ($cond) {
   $3 = HEAP32[$0>>2]|0;
   $4 = ((($3)) + 328|0);
   $5 = HEAP32[$4>>2]|0;
   HEAP32[$p>>2] = $5;
   $6 = HEAP32[$p>>2]|0;
   $7 = ($6|0)!=(0|0);
   if ($7) {
    $arglist_current = HEAP32[$ap>>2]|0;
    $9 = $arglist_current;
    $10 = ((0) + 4|0);
    $expanded2 = $10;
    $expanded = (($expanded2) - 1)|0;
    $11 = (($9) + ($expanded))|0;
    $12 = ((0) + 4|0);
    $expanded6 = $12;
    $expanded5 = (($expanded6) - 1)|0;
    $expanded4 = $expanded5 ^ -1;
    $13 = $11 & $expanded4;
    $14 = $13;
    $15 = HEAP32[$14>>2]|0;
    $arglist_next = ((($14)) + 4|0);
    HEAP32[$ap>>2] = $arglist_next;
    $16 = $15&255;
    $17 = HEAP32[$p>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = ((($18)) + 16|0);
    HEAP8[$19>>0] = $16;
    break;
   } else {
    $8 = (_sqlite3MisuseError(119587)|0);
    HEAP32[$rc>>2] = $8;
    break;
   }
  } else {
   $20 = (_sqlite3MisuseError(119595)|0);
   HEAP32[$rc>>2] = $20;
  }
 } while(0);
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $25 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$rc>>2]|0;
 _sqlite3Error($23,$24);
 $25 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3ValueNew($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3DbMallocZero($1,40,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $10 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 32|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3ValueSetStr($v,$n,$z,$enc,$xDel) {
 $v = $v|0;
 $n = $n|0;
 $z = $z|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $4 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $z;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP8[$3>>0]|0;
 $11 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeMemSetStr($7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return;
}
function _sqlite3_libversion() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return (16690|0);
}
function _sqlite3_sourceid() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return (17807|0);
}
function _sqlite3_libversion_number() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 3011000;
}
function _sqlite3MallocInit() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pSlot = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $n = sp + 8|0;
 $sz = sp + 4|0;
 $pSlot = sp;
 $0 = HEAP32[(44)>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  _sqlite3MemSetDefault();
 }
 ;HEAP32[280>>2]=0|0;HEAP32[280+4>>2]=0|0;HEAP32[280+8>>2]=0|0;HEAP32[280+12>>2]=0|0;HEAP32[280+16>>2]=0|0;HEAP32[280+20>>2]=0|0;HEAP32[280+24>>2]=0|0;HEAP32[280+28>>2]=0|0;
 HEAP32[280>>2] = (8);
 $2 = HEAP32[(200)>>2]|0;
 $3 = ($2|0)!=(0|0);
 $4 = HEAP32[(204)>>2]|0;
 $5 = ($4|0)>=(100);
 $or$cond = $3 & $5;
 $6 = HEAP32[(208)>>2]|0;
 $7 = ($6|0)>(0);
 $or$cond3 = $or$cond & $7;
 if ($or$cond3) {
  $8 = HEAP32[(204)>>2]|0;
  $9 = $8 & -8;
  HEAP32[$sz>>2] = $9;
  $10 = HEAP32[$sz>>2]|0;
  HEAP32[(204)>>2] = $10;
  $11 = HEAP32[(200)>>2]|0;
  HEAP32[$pSlot>>2] = $11;
  $12 = HEAP32[(208)>>2]|0;
  HEAP32[$n>>2] = $12;
  $13 = HEAP32[$pSlot>>2]|0;
  HEAP32[(300)>>2] = $13;
  $14 = HEAP32[$n>>2]|0;
  HEAP32[(304)>>2] = $14;
  HEAP32[$i>>2] = 0;
  while(1) {
   $15 = HEAP32[$i>>2]|0;
   $16 = HEAP32[$n>>2]|0;
   $17 = (($16) - 1)|0;
   $18 = ($15|0)<($17|0);
   if (!($18)) {
    break;
   }
   $19 = HEAP32[$sz>>2]|0;
   $20 = HEAP32[$pSlot>>2]|0;
   $21 = (($20) + ($19)|0);
   $22 = HEAP32[$pSlot>>2]|0;
   HEAP32[$22>>2] = $21;
   $23 = HEAP32[$pSlot>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$pSlot>>2] = $24;
   $25 = HEAP32[$i>>2]|0;
   $26 = (($25) + 1)|0;
   HEAP32[$i>>2] = $26;
  }
  $27 = HEAP32[$pSlot>>2]|0;
  HEAP32[$27>>2] = 0;
  $28 = HEAP32[$pSlot>>2]|0;
  $29 = ((($28)) + 4|0);
  HEAP32[(296)>>2] = $29;
 } else {
  HEAP32[(296)>>2] = 0;
  HEAP32[(200)>>2] = 0;
  HEAP32[(204)>>2] = 0;
  HEAP32[(208)>>2] = 0;
 }
 $30 = HEAP32[(212)>>2]|0;
 $31 = ($30|0)==(0|0);
 $32 = HEAP32[(216)>>2]|0;
 $33 = ($32|0)<(512);
 $or$cond5 = $31 | $33;
 $34 = HEAP32[(220)>>2]|0;
 $35 = ($34|0)<=(0);
 $or$cond7 = $or$cond5 | $35;
 if ($or$cond7) {
  HEAP32[(212)>>2] = 0;
  HEAP32[(216)>>2] = 0;
 }
 $36 = HEAP32[(64)>>2]|0;
 $37 = HEAP32[(72)>>2]|0;
 $38 = (FUNCTION_TABLE_ii[$36 & 255]($37)|0);
 HEAP32[$rc>>2] = $38;
 $39 = HEAP32[$rc>>2]|0;
 $40 = ($39|0)!=(0);
 if (!($40)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 ;HEAP32[280>>2]=0|0;HEAP32[280+4>>2]=0|0;HEAP32[280+8>>2]=0|0;HEAP32[280+12>>2]=0|0;HEAP32[280+16>>2]=0|0;HEAP32[280+20>>2]=0|0;HEAP32[280+24>>2]=0|0;HEAP32[280+28>>2]=0|0;
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3RegisterGlobalFunctions() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aFunc = 0, $i = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 8|0;
 $pHash = sp + 4|0;
 $aFunc = sp;
 HEAP32[$pHash>>2] = 848;
 HEAP32[$aFunc>>2] = 3848;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0|0)<(57);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$pHash>>2]|0;
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$aFunc>>2]|0;
  $5 = (($4) + ($3<<5)|0);
  _sqlite3FuncDefInsert($2,$5);
  $6 = HEAP32[$i>>2]|0;
  $7 = (($6) + 1)|0;
  HEAP32[$i>>2] = $7;
 }
 _sqlite3RegisterDateTimeFunctions();
 _sqlite3AlterFunctions();
 STACKTOP = sp;return;
}
function _sqlite3PcacheInitialize() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(120)>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  _sqlite3PCacheSetDefault();
 }
 $2 = HEAP32[(120)>>2]|0;
 $3 = HEAP32[(116)>>2]|0;
 $4 = (FUNCTION_TABLE_ii[$2 & 255]($3)|0);
 return ($4|0);
}
function _sqlite3OsInit() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 $1 = (_sqlite3_malloc(10)|0);
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 7;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 } else {
  $4 = HEAP32[$p>>2]|0;
  _sqlite3_free($4);
  $5 = (_sqlite3_os_init()|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 return (0)|0;
}
function _sqlite3PCacheBufferSetup($pBuf,$sz,$n) {
 $pBuf = $pBuf|0;
 $sz = $sz|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBuf;
 HEAP32[$1>>2] = $sz;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[(1636)>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$2>>2] = 0;
  HEAP32[$1>>2] = 0;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = $7 & -8;
 HEAP32[$1>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[(1648)>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[(1676)>>2] = $10;
 HEAP32[(1652)>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)>(90);
 if ($12) {
  $16 = 10;
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13|0) / 10)&-1;
  $15 = (($14) + 1)|0;
  $16 = $15;
 }
 HEAP32[(1656)>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 HEAP32[(1660)>>2] = $17;
 HEAP32[(1672)>>2] = 0;
 HEAP32[(1680)>>2] = 0;
 while(1) {
  $18 = HEAP32[$2>>2]|0;
  $19 = (($18) + -1)|0;
  HEAP32[$2>>2] = $19;
  $20 = ($18|0)!=(0);
  $21 = HEAP32[$0>>2]|0;
  if (!($20)) {
   break;
  }
  HEAP32[$p>>2] = $21;
  $22 = HEAP32[(1672)>>2]|0;
  $23 = HEAP32[$p>>2]|0;
  HEAP32[$23>>2] = $22;
  $24 = HEAP32[$p>>2]|0;
  HEAP32[(1672)>>2] = $24;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + ($25)|0);
  HEAP32[$0>>2] = $27;
 }
 HEAP32[(1664)>>2] = $21;
 STACKTOP = sp;return;
}
function _sqlite3_config($op,$varargs) {
 $op = $op|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ap = 0, $arglist_current = 0, $arglist_current12 = 0, $arglist_current15 = 0, $arglist_current18 = 0, $arglist_current21 = 0, $arglist_current24 = 0, $arglist_current27 = 0, $arglist_current3 = 0, $arglist_current30 = 0, $arglist_current33 = 0, $arglist_current36 = 0, $arglist_current39 = 0, $arglist_current42 = 0, $arglist_current45 = 0, $arglist_current48 = 0;
 var $arglist_current51 = 0, $arglist_current54 = 0, $arglist_current57 = 0, $arglist_current6 = 0, $arglist_current60 = 0, $arglist_current9 = 0, $arglist_next = 0, $arglist_next10 = 0, $arglist_next13 = 0, $arglist_next16 = 0, $arglist_next19 = 0, $arglist_next22 = 0, $arglist_next25 = 0, $arglist_next28 = 0, $arglist_next31 = 0, $arglist_next34 = 0, $arglist_next37 = 0, $arglist_next4 = 0, $arglist_next40 = 0, $arglist_next43 = 0;
 var $arglist_next46 = 0, $arglist_next49 = 0, $arglist_next52 = 0, $arglist_next55 = 0, $arglist_next58 = 0, $arglist_next61 = 0, $arglist_next7 = 0, $expanded = 0, $expanded100 = 0, $expanded102 = 0, $expanded103 = 0, $expanded104 = 0, $expanded106 = 0, $expanded107 = 0, $expanded109 = 0, $expanded11 = 0, $expanded110 = 0, $expanded111 = 0, $expanded113 = 0, $expanded114 = 0;
 var $expanded116 = 0, $expanded117 = 0, $expanded118 = 0, $expanded12 = 0, $expanded120 = 0, $expanded121 = 0, $expanded123 = 0, $expanded124 = 0, $expanded125 = 0, $expanded127 = 0, $expanded128 = 0, $expanded13 = 0, $expanded130 = 0, $expanded131 = 0, $expanded132 = 0, $expanded134 = 0, $expanded135 = 0, $expanded137 = 0, $expanded138 = 0, $expanded139 = 0;
 var $expanded141 = 0, $expanded142 = 0, $expanded144 = 0, $expanded145 = 0, $expanded146 = 0, $expanded15 = 0, $expanded16 = 0, $expanded18 = 0, $expanded19 = 0, $expanded2 = 0, $expanded20 = 0, $expanded22 = 0, $expanded23 = 0, $expanded25 = 0, $expanded26 = 0, $expanded27 = 0, $expanded29 = 0, $expanded30 = 0, $expanded32 = 0, $expanded33 = 0;
 var $expanded34 = 0, $expanded36 = 0, $expanded37 = 0, $expanded39 = 0, $expanded4 = 0, $expanded40 = 0, $expanded41 = 0, $expanded43 = 0, $expanded44 = 0, $expanded46 = 0, $expanded47 = 0, $expanded48 = 0, $expanded5 = 0, $expanded50 = 0, $expanded51 = 0, $expanded53 = 0, $expanded54 = 0, $expanded55 = 0, $expanded57 = 0, $expanded58 = 0;
 var $expanded6 = 0, $expanded60 = 0, $expanded61 = 0, $expanded62 = 0, $expanded64 = 0, $expanded65 = 0, $expanded67 = 0, $expanded68 = 0, $expanded69 = 0, $expanded71 = 0, $expanded72 = 0, $expanded74 = 0, $expanded75 = 0, $expanded76 = 0, $expanded78 = 0, $expanded79 = 0, $expanded8 = 0, $expanded81 = 0, $expanded82 = 0, $expanded83 = 0;
 var $expanded85 = 0, $expanded86 = 0, $expanded88 = 0, $expanded89 = 0, $expanded9 = 0, $expanded90 = 0, $expanded92 = 0, $expanded93 = 0, $expanded95 = 0, $expanded96 = 0, $expanded97 = 0, $expanded99 = 0, $mxMmap = 0, $or$cond = 0, $rc = 0, $szMmap = 0, dest = 0, label = 0, sp = 0, src = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $ap = sp + 24|0;
 $rc = sp + 16|0;
 $szMmap = sp + 8|0;
 $mxMmap = sp;
 HEAP32[$1>>2] = $op;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[(236)>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = (_sqlite3MisuseError(133154)|0);
  HEAP32[$0>>2] = $4;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 HEAP32[$ap>>2] = $varargs;
 $5 = HEAP32[$1>>2]|0;
 do {
  switch ($5|0) {
  case 4:  {
   $arglist_current = HEAP32[$ap>>2]|0;
   $6 = $arglist_current;
   $7 = ((0) + 4|0);
   $expanded2 = $7;
   $expanded = (($expanded2) - 1)|0;
   $8 = (($6) + ($expanded))|0;
   $9 = ((0) + 4|0);
   $expanded6 = $9;
   $expanded5 = (($expanded6) - 1)|0;
   $expanded4 = $expanded5 ^ -1;
   $10 = $8 & $expanded4;
   $11 = $10;
   $12 = HEAP32[$11>>2]|0;
   $arglist_next = ((($11)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next;
   ;HEAP32[(44)>>2]=HEAP32[$12>>2]|0;HEAP32[(44)+4>>2]=HEAP32[$12+4>>2]|0;HEAP32[(44)+8>>2]=HEAP32[$12+8>>2]|0;HEAP32[(44)+12>>2]=HEAP32[$12+12>>2]|0;HEAP32[(44)+16>>2]=HEAP32[$12+16>>2]|0;HEAP32[(44)+20>>2]=HEAP32[$12+20>>2]|0;HEAP32[(44)+24>>2]=HEAP32[$12+24>>2]|0;HEAP32[(44)+28>>2]=HEAP32[$12+28>>2]|0;
   break;
  }
  case 5:  {
   $13 = HEAP32[(44)>>2]|0;
   $14 = ($13|0)==(0|0);
   if ($14) {
    _sqlite3MemSetDefault();
   }
   $arglist_current3 = HEAP32[$ap>>2]|0;
   $15 = $arglist_current3;
   $16 = ((0) + 4|0);
   $expanded9 = $16;
   $expanded8 = (($expanded9) - 1)|0;
   $17 = (($15) + ($expanded8))|0;
   $18 = ((0) + 4|0);
   $expanded13 = $18;
   $expanded12 = (($expanded13) - 1)|0;
   $expanded11 = $expanded12 ^ -1;
   $19 = $17 & $expanded11;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $arglist_next4 = ((($20)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next4;
   ;HEAP32[$21>>2]=HEAP32[(44)>>2]|0;HEAP32[$21+4>>2]=HEAP32[(44)+4>>2]|0;HEAP32[$21+8>>2]=HEAP32[(44)+8>>2]|0;HEAP32[$21+12>>2]=HEAP32[(44)+12>>2]|0;HEAP32[$21+16>>2]=HEAP32[(44)+16>>2]|0;HEAP32[$21+20>>2]=HEAP32[(44)+20>>2]|0;HEAP32[$21+24>>2]=HEAP32[(44)+24>>2]|0;HEAP32[$21+28>>2]=HEAP32[(44)+28>>2]|0;
   break;
  }
  case 9:  {
   $arglist_current6 = HEAP32[$ap>>2]|0;
   $22 = $arglist_current6;
   $23 = ((0) + 4|0);
   $expanded16 = $23;
   $expanded15 = (($expanded16) - 1)|0;
   $24 = (($22) + ($expanded15))|0;
   $25 = ((0) + 4|0);
   $expanded20 = $25;
   $expanded19 = (($expanded20) - 1)|0;
   $expanded18 = $expanded19 ^ -1;
   $26 = $24 & $expanded18;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $arglist_next7 = ((($27)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next7;
   HEAP32[8>>2] = $28;
   break;
  }
  case 6:  {
   $arglist_current9 = HEAP32[$ap>>2]|0;
   $29 = $arglist_current9;
   $30 = ((0) + 4|0);
   $expanded23 = $30;
   $expanded22 = (($expanded23) - 1)|0;
   $31 = (($29) + ($expanded22))|0;
   $32 = ((0) + 4|0);
   $expanded27 = $32;
   $expanded26 = (($expanded27) - 1)|0;
   $expanded25 = $expanded26 ^ -1;
   $33 = $31 & $expanded25;
   $34 = $33;
   $35 = HEAP32[$34>>2]|0;
   $arglist_next10 = ((($34)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next10;
   HEAP32[(200)>>2] = $35;
   $arglist_current12 = HEAP32[$ap>>2]|0;
   $36 = $arglist_current12;
   $37 = ((0) + 4|0);
   $expanded30 = $37;
   $expanded29 = (($expanded30) - 1)|0;
   $38 = (($36) + ($expanded29))|0;
   $39 = ((0) + 4|0);
   $expanded34 = $39;
   $expanded33 = (($expanded34) - 1)|0;
   $expanded32 = $expanded33 ^ -1;
   $40 = $38 & $expanded32;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $arglist_next13 = ((($41)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next13;
   HEAP32[(204)>>2] = $42;
   $arglist_current15 = HEAP32[$ap>>2]|0;
   $43 = $arglist_current15;
   $44 = ((0) + 4|0);
   $expanded37 = $44;
   $expanded36 = (($expanded37) - 1)|0;
   $45 = (($43) + ($expanded36))|0;
   $46 = ((0) + 4|0);
   $expanded41 = $46;
   $expanded40 = (($expanded41) - 1)|0;
   $expanded39 = $expanded40 ^ -1;
   $47 = $45 & $expanded39;
   $48 = $47;
   $49 = HEAP32[$48>>2]|0;
   $arglist_next16 = ((($48)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next16;
   HEAP32[(208)>>2] = $49;
   break;
  }
  case 7:  {
   $arglist_current18 = HEAP32[$ap>>2]|0;
   $50 = $arglist_current18;
   $51 = ((0) + 4|0);
   $expanded44 = $51;
   $expanded43 = (($expanded44) - 1)|0;
   $52 = (($50) + ($expanded43))|0;
   $53 = ((0) + 4|0);
   $expanded48 = $53;
   $expanded47 = (($expanded48) - 1)|0;
   $expanded46 = $expanded47 ^ -1;
   $54 = $52 & $expanded46;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $arglist_next19 = ((($55)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next19;
   HEAP32[(212)>>2] = $56;
   $arglist_current21 = HEAP32[$ap>>2]|0;
   $57 = $arglist_current21;
   $58 = ((0) + 4|0);
   $expanded51 = $58;
   $expanded50 = (($expanded51) - 1)|0;
   $59 = (($57) + ($expanded50))|0;
   $60 = ((0) + 4|0);
   $expanded55 = $60;
   $expanded54 = (($expanded55) - 1)|0;
   $expanded53 = $expanded54 ^ -1;
   $61 = $59 & $expanded53;
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $arglist_next22 = ((($62)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next22;
   HEAP32[(216)>>2] = $63;
   $arglist_current24 = HEAP32[$ap>>2]|0;
   $64 = $arglist_current24;
   $65 = ((0) + 4|0);
   $expanded58 = $65;
   $expanded57 = (($expanded58) - 1)|0;
   $66 = (($64) + ($expanded57))|0;
   $67 = ((0) + 4|0);
   $expanded62 = $67;
   $expanded61 = (($expanded62) - 1)|0;
   $expanded60 = $expanded61 ^ -1;
   $68 = $66 & $expanded60;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $arglist_next25 = ((($69)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next25;
   HEAP32[(220)>>2] = $70;
   break;
  }
  case 24:  {
   $71 = (_sqlite3HeaderSizeBtree()|0);
   $72 = (_sqlite3HeaderSizePcache()|0);
   $73 = (($71) + ($72))|0;
   $74 = (_sqlite3HeaderSizePcache1()|0);
   $75 = (($73) + ($74))|0;
   $arglist_current27 = HEAP32[$ap>>2]|0;
   $76 = $arglist_current27;
   $77 = ((0) + 4|0);
   $expanded65 = $77;
   $expanded64 = (($expanded65) - 1)|0;
   $78 = (($76) + ($expanded64))|0;
   $79 = ((0) + 4|0);
   $expanded69 = $79;
   $expanded68 = (($expanded69) - 1)|0;
   $expanded67 = $expanded68 ^ -1;
   $80 = $78 & $expanded67;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $arglist_next28 = ((($81)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next28;
   HEAP32[$82>>2] = $75;
   break;
  }
  case 14:  {
   break;
  }
  case 15:  {
   HEAP32[$rc>>2] = 1;
   break;
  }
  case 18:  {
   $arglist_current30 = HEAP32[$ap>>2]|0;
   $83 = $arglist_current30;
   $84 = ((0) + 4|0);
   $expanded72 = $84;
   $expanded71 = (($expanded72) - 1)|0;
   $85 = (($83) + ($expanded71))|0;
   $86 = ((0) + 4|0);
   $expanded76 = $86;
   $expanded75 = (($expanded76) - 1)|0;
   $expanded74 = $expanded75 ^ -1;
   $87 = $85 & $expanded74;
   $88 = $87;
   $89 = HEAP32[$88>>2]|0;
   $arglist_next31 = ((($88)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next31;
   dest=(112); src=$89; stop=dest+52|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   break;
  }
  case 19:  {
   $90 = HEAP32[(120)>>2]|0;
   $91 = ($90|0)==(0|0);
   if ($91) {
    _sqlite3PCacheSetDefault();
   }
   $arglist_current33 = HEAP32[$ap>>2]|0;
   $92 = $arglist_current33;
   $93 = ((0) + 4|0);
   $expanded79 = $93;
   $expanded78 = (($expanded79) - 1)|0;
   $94 = (($92) + ($expanded78))|0;
   $95 = ((0) + 4|0);
   $expanded83 = $95;
   $expanded82 = (($expanded83) - 1)|0;
   $expanded81 = $expanded82 ^ -1;
   $96 = $94 & $expanded81;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $arglist_next34 = ((($97)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next34;
   dest=$98; src=(112); stop=dest+52|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   break;
  }
  case 13:  {
   $arglist_current36 = HEAP32[$ap>>2]|0;
   $99 = $arglist_current36;
   $100 = ((0) + 4|0);
   $expanded86 = $100;
   $expanded85 = (($expanded86) - 1)|0;
   $101 = (($99) + ($expanded85))|0;
   $102 = ((0) + 4|0);
   $expanded90 = $102;
   $expanded89 = (($expanded90) - 1)|0;
   $expanded88 = $expanded89 ^ -1;
   $103 = $101 & $expanded88;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $arglist_next37 = ((($104)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next37;
   HEAP32[(36)>>2] = $105;
   $arglist_current39 = HEAP32[$ap>>2]|0;
   $106 = $arglist_current39;
   $107 = ((0) + 4|0);
   $expanded93 = $107;
   $expanded92 = (($expanded93) - 1)|0;
   $108 = (($106) + ($expanded92))|0;
   $109 = ((0) + 4|0);
   $expanded97 = $109;
   $expanded96 = (($expanded97) - 1)|0;
   $expanded95 = $expanded96 ^ -1;
   $110 = $108 & $expanded95;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $arglist_next40 = ((($111)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next40;
   HEAP32[(40)>>2] = $112;
   break;
  }
  case 16:  {
   $arglist_current42 = HEAP32[$ap>>2]|0;
   $113 = $arglist_current42;
   $114 = ((0) + 4|0);
   $expanded100 = $114;
   $expanded99 = (($expanded100) - 1)|0;
   $115 = (($113) + ($expanded99))|0;
   $116 = ((0) + 4|0);
   $expanded104 = $116;
   $expanded103 = (($expanded104) - 1)|0;
   $expanded102 = $expanded103 ^ -1;
   $117 = $115 & $expanded102;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $arglist_next43 = ((($118)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next43;
   HEAP32[(264)>>2] = $119;
   $arglist_current45 = HEAP32[$ap>>2]|0;
   $120 = $arglist_current45;
   $121 = ((0) + 4|0);
   $expanded107 = $121;
   $expanded106 = (($expanded107) - 1)|0;
   $122 = (($120) + ($expanded106))|0;
   $123 = ((0) + 4|0);
   $expanded111 = $123;
   $expanded110 = (($expanded111) - 1)|0;
   $expanded109 = $expanded110 ^ -1;
   $124 = $122 & $expanded109;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $arglist_next46 = ((($125)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next46;
   HEAP32[(268)>>2] = $126;
   break;
  }
  case 17:  {
   $arglist_current48 = HEAP32[$ap>>2]|0;
   $127 = $arglist_current48;
   $128 = ((0) + 4|0);
   $expanded114 = $128;
   $expanded113 = (($expanded114) - 1)|0;
   $129 = (($127) + ($expanded113))|0;
   $130 = ((0) + 4|0);
   $expanded118 = $130;
   $expanded117 = (($expanded118) - 1)|0;
   $expanded116 = $expanded117 ^ -1;
   $131 = $129 & $expanded116;
   $132 = $131;
   $133 = HEAP32[$132>>2]|0;
   $arglist_next49 = ((($132)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next49;
   HEAP32[(20)>>2] = $133;
   break;
  }
  case 20:  {
   $arglist_current51 = HEAP32[$ap>>2]|0;
   $134 = $arglist_current51;
   $135 = ((0) + 4|0);
   $expanded121 = $135;
   $expanded120 = (($expanded121) - 1)|0;
   $136 = (($134) + ($expanded120))|0;
   $137 = ((0) + 4|0);
   $expanded125 = $137;
   $expanded124 = (($expanded125) - 1)|0;
   $expanded123 = $expanded124 ^ -1;
   $138 = $136 & $expanded123;
   $139 = $138;
   $140 = HEAP32[$139>>2]|0;
   $arglist_next52 = ((($139)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next52;
   HEAP32[(24)>>2] = $140;
   break;
  }
  case 22:  {
   $arglist_current54 = HEAP32[$ap>>2]|0;
   $141 = $arglist_current54;
   $142 = ((0) + 8|0);
   $expanded128 = $142;
   $expanded127 = (($expanded128) - 1)|0;
   $143 = (($141) + ($expanded127))|0;
   $144 = ((0) + 8|0);
   $expanded132 = $144;
   $expanded131 = (($expanded132) - 1)|0;
   $expanded130 = $expanded131 ^ -1;
   $145 = $143 & $expanded130;
   $146 = $145;
   $147 = $146;
   $148 = $147;
   $149 = HEAP32[$148>>2]|0;
   $150 = (($147) + 4)|0;
   $151 = $150;
   $152 = HEAP32[$151>>2]|0;
   $arglist_next55 = ((($146)) + 8|0);
   HEAP32[$ap>>2] = $arglist_next55;
   $153 = $szMmap;
   $154 = $153;
   HEAP32[$154>>2] = $149;
   $155 = (($153) + 4)|0;
   $156 = $155;
   HEAP32[$156>>2] = $152;
   $arglist_current57 = HEAP32[$ap>>2]|0;
   $157 = $arglist_current57;
   $158 = ((0) + 8|0);
   $expanded135 = $158;
   $expanded134 = (($expanded135) - 1)|0;
   $159 = (($157) + ($expanded134))|0;
   $160 = ((0) + 8|0);
   $expanded139 = $160;
   $expanded138 = (($expanded139) - 1)|0;
   $expanded137 = $expanded138 ^ -1;
   $161 = $159 & $expanded137;
   $162 = $161;
   $163 = $162;
   $164 = $163;
   $165 = HEAP32[$164>>2]|0;
   $166 = (($163) + 4)|0;
   $167 = $166;
   $168 = HEAP32[$167>>2]|0;
   $arglist_next58 = ((($162)) + 8|0);
   HEAP32[$ap>>2] = $arglist_next58;
   $169 = $mxMmap;
   $170 = $169;
   HEAP32[$170>>2] = $165;
   $171 = (($169) + 4)|0;
   $172 = $171;
   HEAP32[$172>>2] = $168;
   $173 = $mxMmap;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($173) + 4)|0;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = ($178|0)<(0);
   $180 = $mxMmap;
   $181 = $180;
   $182 = HEAP32[$181>>2]|0;
   $183 = (($180) + 4)|0;
   $184 = $183;
   $185 = HEAP32[$184>>2]|0;
   $186 = ($185|0)>(0);
   $187 = ($182>>>0)>(0);
   $188 = ($185|0)==(0);
   $189 = $188 & $187;
   $190 = $186 | $189;
   $or$cond = $179 | $190;
   $191 = $or$cond ? 0 : $165;
   $192 = $or$cond ? 0 : $168;
   $193 = $mxMmap;
   $194 = $193;
   HEAP32[$194>>2] = $191;
   $195 = (($193) + 4)|0;
   $196 = $195;
   HEAP32[$196>>2] = $192;
   $197 = $szMmap;
   $198 = $197;
   $199 = HEAP32[$198>>2]|0;
   $200 = (($197) + 4)|0;
   $201 = $200;
   $202 = HEAP32[$201>>2]|0;
   $203 = ($202|0)<(0);
   if ($203) {
    $204 = $szMmap;
    $205 = $204;
    HEAP32[$205>>2] = 0;
    $206 = (($204) + 4)|0;
    $207 = $206;
    HEAP32[$207>>2] = 0;
   }
   $208 = $szMmap;
   $209 = $208;
   $210 = HEAP32[$209>>2]|0;
   $211 = (($208) + 4)|0;
   $212 = $211;
   $213 = HEAP32[$212>>2]|0;
   $214 = $mxMmap;
   $215 = $214;
   $216 = HEAP32[$215>>2]|0;
   $217 = (($214) + 4)|0;
   $218 = $217;
   $219 = HEAP32[$218>>2]|0;
   $220 = ($213|0)>($219|0);
   $221 = ($210>>>0)>($216>>>0);
   $222 = ($213|0)==($219|0);
   $223 = $222 & $221;
   $224 = $220 | $223;
   if ($224) {
    $225 = $mxMmap;
    $226 = $225;
    $227 = HEAP32[$226>>2]|0;
    $228 = (($225) + 4)|0;
    $229 = $228;
    $230 = HEAP32[$229>>2]|0;
    $231 = $szMmap;
    $232 = $231;
    HEAP32[$232>>2] = $227;
    $233 = (($231) + 4)|0;
    $234 = $233;
    HEAP32[$234>>2] = $230;
   }
   $235 = $mxMmap;
   $236 = $235;
   $237 = HEAP32[$236>>2]|0;
   $238 = (($235) + 4)|0;
   $239 = $238;
   $240 = HEAP32[$239>>2]|0;
   $241 = (192);
   $242 = $241;
   HEAP32[$242>>2] = $237;
   $243 = (($241) + 4)|0;
   $244 = $243;
   HEAP32[$244>>2] = $240;
   $245 = $szMmap;
   $246 = $245;
   $247 = HEAP32[$246>>2]|0;
   $248 = (($245) + 4)|0;
   $249 = $248;
   $250 = HEAP32[$249>>2]|0;
   $251 = (184);
   $252 = $251;
   HEAP32[$252>>2] = $247;
   $253 = (($251) + 4)|0;
   $254 = $253;
   HEAP32[$254>>2] = $250;
   break;
  }
  case 25:  {
   $arglist_current60 = HEAP32[$ap>>2]|0;
   $255 = $arglist_current60;
   $256 = ((0) + 4|0);
   $expanded142 = $256;
   $expanded141 = (($expanded142) - 1)|0;
   $257 = (($255) + ($expanded141))|0;
   $258 = ((0) + 4|0);
   $expanded146 = $258;
   $expanded145 = (($expanded146) - 1)|0;
   $expanded144 = $expanded145 ^ -1;
   $259 = $257 & $expanded144;
   $260 = $259;
   $261 = HEAP32[$260>>2]|0;
   $arglist_next61 = ((($260)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next61;
   HEAP32[(232)>>2] = $261;
   break;
  }
  default: {
   HEAP32[$rc>>2] = 1;
  }
  }
 } while(0);
 $262 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $262;
 $263 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($263|0);
}
function _sqlite3MemSetDefault() {
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = 6028;
 (_sqlite3_config(4,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _sqlite3HeaderSizeBtree() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 88;
}
function _sqlite3HeaderSizePcache() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 40;
}
function _sqlite3HeaderSizePcache1() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 32;
}
function _sqlite3PCacheSetDefault() {
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = 6060;
 (_sqlite3_config(18,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _sqlite3_db_release_memory($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pBt = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pBt>>2] = $13;
  $14 = HEAP32[$pBt>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_sqlite3BtreePager($16)|0);
   HEAP32[$pPager>>2] = $17;
   $18 = HEAP32[$pPager>>2]|0;
   _sqlite3PagerShrink($18);
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3PagerShrink($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 212|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3PcacheShrink($3);
 STACKTOP = sp;return;
}
function _sqlite3BtreeIsInTrans($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _setupLookaside($db,$pBuf,$sz,$cnt) {
 $db = $db|0;
 $pBuf = $pBuf|0;
 $sz = $sz|0;
 $cnt = $cnt|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $or$cond = 0, $p = 0, $pStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pStart = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pBuf;
 HEAP32[$3>>2] = $sz;
 HEAP32[$4>>2] = $cnt;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 248|0);
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 5;
  $103 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 248|0);
 $12 = ((($11)) + 6|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = ($13<<24>>24)!=(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 248|0);
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3_free($18);
 }
 $19 = HEAP32[$3>>2]|0;
 $20 = $19 & -8;
 HEAP32[$3>>2] = $20;
 $21 = HEAP32[$3>>2]|0;
 $22 = ($21|0)<=(4);
 $$ = $22 ? 0 : $20;
 HEAP32[$3>>2] = $$;
 $23 = HEAP32[$4>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  HEAP32[$4>>2] = 0;
 }
 $25 = HEAP32[$3>>2]|0;
 $26 = ($25|0)==(0);
 $27 = HEAP32[$4>>2]|0;
 $28 = ($27|0)==(0);
 $or$cond = $26 | $28;
 do {
  if ($or$cond) {
   HEAP32[$3>>2] = 0;
   HEAP32[$pStart>>2] = 0;
  } else {
   $29 = HEAP32[$2>>2]|0;
   $30 = ($29|0)==(0|0);
   if (!($30)) {
    $43 = HEAP32[$2>>2]|0;
    HEAP32[$pStart>>2] = $43;
    break;
   }
   _sqlite3BeginBenignMalloc();
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = Math_imul($31, $32)|0;
   $34 = ($33|0)<(0);
   $35 = $34 << 31 >> 31;
   $36 = (_sqlite3Malloc($33,$35)|0);
   HEAP32[$pStart>>2] = $36;
   _sqlite3EndBenignMalloc();
   $37 = HEAP32[$pStart>>2]|0;
   $38 = ($37|0)!=(0|0);
   if ($38) {
    $39 = HEAP32[$pStart>>2]|0;
    $40 = (_sqlite3MallocSize($39)|0);
    $41 = HEAP32[$3>>2]|0;
    $42 = (($40|0) / ($41|0))&-1;
    HEAP32[$4>>2] = $42;
   }
  }
 } while(0);
 $44 = HEAP32[$pStart>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 248|0);
 $47 = ((($46)) + 32|0);
 HEAP32[$47>>2] = $44;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 248|0);
 $50 = ((($49)) + 28|0);
 HEAP32[$50>>2] = 0;
 $51 = HEAP32[$3>>2]|0;
 $52 = $51&65535;
 $53 = HEAP32[$1>>2]|0;
 $54 = ((($53)) + 248|0);
 $55 = ((($54)) + 4|0);
 HEAP16[$55>>1] = $52;
 $56 = HEAP32[$pStart>>2]|0;
 $57 = ($56|0)!=(0|0);
 if ($57) {
  $58 = HEAP32[$pStart>>2]|0;
  HEAP32[$p>>2] = $58;
  $59 = HEAP32[$4>>2]|0;
  $60 = (($59) - 1)|0;
  HEAP32[$i>>2] = $60;
  while(1) {
   $61 = HEAP32[$i>>2]|0;
   $62 = ($61|0)>=(0);
   if (!($62)) {
    break;
   }
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 248|0);
   $65 = ((($64)) + 28|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$p>>2]|0;
   HEAP32[$67>>2] = $66;
   $68 = HEAP32[$p>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 248|0);
   $71 = ((($70)) + 28|0);
   HEAP32[$71>>2] = $68;
   $72 = HEAP32[$3>>2]|0;
   $73 = HEAP32[$p>>2]|0;
   $74 = (($73) + ($72)|0);
   HEAP32[$p>>2] = $74;
   $75 = HEAP32[$i>>2]|0;
   $76 = (($75) + -1)|0;
   HEAP32[$i>>2] = $76;
  }
  $77 = HEAP32[$p>>2]|0;
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 248|0);
  $80 = ((($79)) + 36|0);
  HEAP32[$80>>2] = $77;
  $81 = HEAP32[$1>>2]|0;
  $82 = ((($81)) + 248|0);
  HEAP32[$82>>2] = 0;
  $83 = HEAP32[$2>>2]|0;
  $84 = ($83|0)==(0|0);
  $85 = $84 ? 1 : 0;
  $86 = $85&255;
  $87 = HEAP32[$1>>2]|0;
  $88 = ((($87)) + 248|0);
  $89 = ((($88)) + 6|0);
  HEAP8[$89>>0] = $86;
 } else {
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 248|0);
  $93 = ((($92)) + 32|0);
  HEAP32[$93>>2] = $90;
  $94 = HEAP32[$1>>2]|0;
  $95 = HEAP32[$1>>2]|0;
  $96 = ((($95)) + 248|0);
  $97 = ((($96)) + 36|0);
  HEAP32[$97>>2] = $94;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 248|0);
  HEAP32[$99>>2] = 1;
  $100 = HEAP32[$1>>2]|0;
  $101 = ((($100)) + 248|0);
  $102 = ((($101)) + 6|0);
  HEAP8[$102>>0] = 0;
 }
 HEAP32[$0>>2] = 0;
 $103 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($103|0);
}
function _sqlite3_last_insert_rowid($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = $2;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 tempRet0 = ($8);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3_changes($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_total_changes($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 84|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_close($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3Close($1,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3Close($db,$forceZombie) {
 $db = $db|0;
 $forceZombie = $forceZombie|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $forceZombie;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3SafetyCheckSickOrOk($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MisuseError(133819)|0);
  HEAP32[$0>>2] = $8;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $9 = HEAP32[$1>>2]|0;
 _disconnectAllVtab($9);
 $10 = HEAP32[$1>>2]|0;
 (_sqlite3VtabRollback($10)|0);
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0);
 if (!($12)) {
  $13 = HEAP32[$1>>2]|0;
  $14 = (_connectionIsBusy($13)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   _sqlite3ErrorWithMsg($16,5,39039,$vararg_buffer);
   HEAP32[$0>>2] = 5;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 76|0);
 HEAP32[$18>>2] = 1691352191;
 $19 = HEAP32[$1>>2]|0;
 _sqlite3LeaveMutexAndCloseZombie($19);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3_close_v2($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3Close($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_busy_handler($db,$xBusy,$pArg) {
 $db = $db|0;
 $xBusy = $xBusy|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $xBusy;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 448|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 448|0);
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = $6;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 448|0);
 $12 = ((($11)) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 496|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return 0;
}
function _sqlite3_busy_timeout($db,$ms) {
 $db = $db|0;
 $ms = $ms|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $ms;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 $4 = HEAP32[$0>>2]|0;
 if ($3) {
  $5 = HEAP32[$0>>2]|0;
  (_sqlite3_busy_handler($4,177,$5)|0);
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 496|0);
  HEAP32[$8>>2] = $6;
  STACKTOP = sp;return 0;
 } else {
  (_sqlite3_busy_handler($4,0,0)|0);
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _sqliteDefaultBusyCallback($ptr,$count) {
 $ptr = $ptr|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $timeout = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 $timeout = sp;
 HEAP32[$1>>2] = $ptr;
 HEAP32[$2>>2] = $count;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 496|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$timeout>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 1)|0;
 $9 = ($8*1000)|0;
 $10 = HEAP32[$timeout>>2]|0;
 $11 = ($9|0)>($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 } else {
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3OsSleep($13,1000000)|0);
  HEAP32[$0>>2] = 1;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return (0)|0;
}
function _sqlite3_create_function($db,$zFunc,$nArg,$enc,$p,$xSFunc,$xStep,$xFinal) {
 $db = $db|0;
 $zFunc = $zFunc|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $p = $p|0;
 $xSFunc = $xSFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $7 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $nArg;
 HEAP32[$3>>2] = $enc;
 HEAP32[$4>>2] = $p;
 HEAP32[$5>>2] = $xSFunc;
 HEAP32[$6>>2] = $xStep;
 HEAP32[$7>>2] = $xFinal;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$5>>2]|0;
 $14 = HEAP32[$6>>2]|0;
 $15 = HEAP32[$7>>2]|0;
 $16 = (_sqlite3_create_function_v2($8,$9,$10,$11,$12,$13,$14,$15,0)|0);
 STACKTOP = sp;return ($16|0);
}
function _sqlite3_create_function_v2($db,$zFunc,$nArg,$enc,$p,$xSFunc,$xStep,$xFinal,$xDestroy) {
 $db = $db|0;
 $zFunc = $zFunc|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $p = $p|0;
 $xSFunc = $xSFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 $xDestroy = $xDestroy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $7 = sp + 12|0;
 $8 = sp + 8|0;
 $rc = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $nArg;
 HEAP32[$3>>2] = $enc;
 HEAP32[$4>>2] = $p;
 HEAP32[$5>>2] = $xSFunc;
 HEAP32[$6>>2] = $xStep;
 HEAP32[$7>>2] = $xFinal;
 HEAP32[$8>>2] = $xDestroy;
 HEAP32[$rc>>2] = 1;
 HEAP32[$pArg>>2] = 0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = (_sqlite3DbMallocZero($11,12,0)|0);
   HEAP32[$pArg>>2] = $12;
   $13 = HEAP32[$pArg>>2]|0;
   $14 = ($13|0)!=(0|0);
   $15 = HEAP32[$8>>2]|0;
   if ($14) {
    $17 = HEAP32[$pArg>>2]|0;
    $18 = ((($17)) + 4|0);
    HEAP32[$18>>2] = $15;
    $19 = HEAP32[$4>>2]|0;
    $20 = HEAP32[$pArg>>2]|0;
    $21 = ((($20)) + 8|0);
    HEAP32[$21>>2] = $19;
    label = 5;
    break;
   } else {
    $16 = HEAP32[$4>>2]|0;
    FUNCTION_TABLE_vi[$15 & 255]($16);
    break;
   }
  } else {
   label = 5;
  }
 } while(0);
 if ((label|0) == 5) {
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = HEAP32[$5>>2]|0;
  $28 = HEAP32[$6>>2]|0;
  $29 = HEAP32[$7>>2]|0;
  $30 = HEAP32[$pArg>>2]|0;
  $31 = (_sqlite3CreateFunc($22,$23,$24,$25,$26,$27,$28,$29,$30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$pArg>>2]|0;
  $33 = ($32|0)!=(0|0);
  if ($33) {
   $34 = HEAP32[$pArg>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$8>>2]|0;
    $38 = HEAP32[$4>>2]|0;
    FUNCTION_TABLE_vi[$37 & 255]($38);
    $39 = HEAP32[$0>>2]|0;
    $40 = HEAP32[$pArg>>2]|0;
    _sqlite3DbFree($39,$40);
   }
  }
 }
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$rc>>2]|0;
 $43 = (_sqlite3ApiExit($41,$42)|0);
 HEAP32[$rc>>2] = $43;
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3CreateFunc($db,$zFunctionName,$nArg,$enc,$pUserData,$xSFunc,$xStep,$xFinal,$pDestructor) {
 $db = $db|0;
 $zFunctionName = $zFunctionName|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $pUserData = $pUserData|0;
 $xSFunc = $xSFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 $pDestructor = $pDestructor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $extraFlags = 0, $nName = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond3 = 0, $or$cond3$not = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond7$not = 0, $or$cond9 = 0;
 var $p = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $8 = sp + 24|0;
 $9 = sp + 20|0;
 $p = sp + 16|0;
 $nName = sp + 12|0;
 $extraFlags = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zFunctionName;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $enc;
 HEAP32[$5>>2] = $pUserData;
 HEAP32[$6>>2] = $xSFunc;
 HEAP32[$7>>2] = $xStep;
 HEAP32[$8>>2] = $xFinal;
 HEAP32[$9>>2] = $pDestructor;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 do {
  if (!($11)) {
   $12 = HEAP32[$6>>2]|0;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $14 = HEAP32[$8>>2]|0;
    $15 = ($14|0)!=(0|0);
    $16 = HEAP32[$7>>2]|0;
    $17 = ($16|0)!=(0|0);
    $or$cond = $15 | $17;
    if ($or$cond) {
     break;
    }
   }
   $18 = HEAP32[$6>>2]|0;
   $19 = ($18|0)==(0|0);
   $20 = HEAP32[$8>>2]|0;
   $21 = ($20|0)!=(0|0);
   $or$cond3 = $19 & $21;
   $or$cond3$not = $or$cond3 ^ 1;
   $22 = HEAP32[$7>>2]|0;
   $23 = ($22|0)!=(0|0);
   $or$cond5 = $or$cond3$not | $23;
   if ($or$cond5) {
    $24 = HEAP32[$6>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$8>>2]|0;
    $27 = ($26|0)!=(0|0);
    $or$cond7 = $25 | $27;
    $or$cond7$not = $or$cond7 ^ 1;
    $28 = HEAP32[$7>>2]|0;
    $29 = ($28|0)!=(0|0);
    $or$cond9 = $or$cond7$not & $29;
    $30 = HEAP32[$3>>2]|0;
    $31 = ($30|0)<(-1);
    $or$cond11 = $or$cond9 | $31;
    $32 = HEAP32[$3>>2]|0;
    $33 = ($32|0)>(127);
    $or$cond13 = $or$cond11 | $33;
    if (!($or$cond13)) {
     $34 = HEAP32[$2>>2]|0;
     $35 = (_sqlite3Strlen30($34)|0);
     HEAP32[$nName>>2] = $35;
     $36 = (255)<($35|0);
     if (!($36)) {
      $38 = HEAP32[$4>>2]|0;
      $39 = $38 & 2048;
      HEAP32[$extraFlags>>2] = $39;
      $40 = HEAP32[$4>>2]|0;
      $41 = $40 & 7;
      HEAP32[$4>>2] = $41;
      $42 = HEAP32[$4>>2]|0;
      $43 = ($42|0)==(4);
      do {
       if ($43) {
        $44 = HEAP8[352>>0]|0;
        $45 = $44 << 24 >> 24;
        $46 = ($45|0)==(0);
        $47 = $46 ? 3 : 2;
        HEAP32[$4>>2] = $47;
       } else {
        $48 = HEAP32[$4>>2]|0;
        $49 = ($48|0)==(5);
        if ($49) {
         $50 = HEAP32[$1>>2]|0;
         $51 = HEAP32[$2>>2]|0;
         $52 = HEAP32[$3>>2]|0;
         $53 = HEAP32[$extraFlags>>2]|0;
         $54 = 1 | $53;
         $55 = HEAP32[$5>>2]|0;
         $56 = HEAP32[$6>>2]|0;
         $57 = HEAP32[$7>>2]|0;
         $58 = HEAP32[$8>>2]|0;
         $59 = HEAP32[$9>>2]|0;
         $60 = (_sqlite3CreateFunc($50,$51,$52,$54,$55,$56,$57,$58,$59)|0);
         HEAP32[$rc>>2] = $60;
         $61 = HEAP32[$rc>>2]|0;
         $62 = ($61|0)==(0);
         if ($62) {
          $63 = HEAP32[$1>>2]|0;
          $64 = HEAP32[$2>>2]|0;
          $65 = HEAP32[$3>>2]|0;
          $66 = HEAP32[$extraFlags>>2]|0;
          $67 = 2 | $66;
          $68 = HEAP32[$5>>2]|0;
          $69 = HEAP32[$6>>2]|0;
          $70 = HEAP32[$7>>2]|0;
          $71 = HEAP32[$8>>2]|0;
          $72 = HEAP32[$9>>2]|0;
          $73 = (_sqlite3CreateFunc($63,$64,$65,$67,$68,$69,$70,$71,$72)|0);
          HEAP32[$rc>>2] = $73;
         }
         $74 = HEAP32[$rc>>2]|0;
         $75 = ($74|0)!=(0);
         if (!($75)) {
          HEAP32[$4>>2] = 3;
          break;
         }
         $76 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $76;
         $148 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($148|0);
        }
       }
      } while(0);
      $77 = HEAP32[$1>>2]|0;
      $78 = HEAP32[$2>>2]|0;
      $79 = HEAP32[$nName>>2]|0;
      $80 = HEAP32[$3>>2]|0;
      $81 = HEAP32[$4>>2]|0;
      $82 = $81&255;
      $83 = (_sqlite3FindFunction($77,$78,$79,$80,$82,0)|0);
      HEAP32[$p>>2] = $83;
      $84 = HEAP32[$p>>2]|0;
      $85 = ($84|0)!=(0|0);
      do {
       if ($85) {
        $86 = HEAP32[$p>>2]|0;
        $87 = ((($86)) + 2|0);
        $88 = HEAP16[$87>>1]|0;
        $89 = $88&65535;
        $90 = $89 & 3;
        $91 = HEAP32[$4>>2]|0;
        $92 = ($90|0)==($91|0);
        if ($92) {
         $93 = HEAP32[$p>>2]|0;
         $94 = HEAP16[$93>>1]|0;
         $95 = $94 << 16 >> 16;
         $96 = HEAP32[$3>>2]|0;
         $97 = ($95|0)==($96|0);
         if ($97) {
          $98 = HEAP32[$1>>2]|0;
          $99 = ((($98)) + 148|0);
          $100 = HEAP32[$99>>2]|0;
          $101 = ($100|0)!=(0);
          $102 = HEAP32[$1>>2]|0;
          if (!($101)) {
           _sqlite3ExpirePreparedStatements($102);
           break;
          }
          _sqlite3ErrorWithMsg($102,5,39733,$vararg_buffer);
          HEAP32[$0>>2] = 5;
          $148 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($148|0);
         }
        }
       }
      } while(0);
      $103 = HEAP32[$1>>2]|0;
      $104 = HEAP32[$2>>2]|0;
      $105 = HEAP32[$nName>>2]|0;
      $106 = HEAP32[$3>>2]|0;
      $107 = HEAP32[$4>>2]|0;
      $108 = $107&255;
      $109 = (_sqlite3FindFunction($103,$104,$105,$106,$108,1)|0);
      HEAP32[$p>>2] = $109;
      $110 = HEAP32[$p>>2]|0;
      $111 = ($110|0)!=(0|0);
      if (!($111)) {
       HEAP32[$0>>2] = 7;
       $148 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($148|0);
      }
      $112 = HEAP32[$1>>2]|0;
      $113 = HEAP32[$p>>2]|0;
      _functionDestroy($112,$113);
      $114 = HEAP32[$9>>2]|0;
      $115 = ($114|0)!=(0|0);
      if ($115) {
       $116 = HEAP32[$9>>2]|0;
       $117 = HEAP32[$116>>2]|0;
       $118 = (($117) + 1)|0;
       HEAP32[$116>>2] = $118;
      }
      $119 = HEAP32[$9>>2]|0;
      $120 = HEAP32[$p>>2]|0;
      $121 = ((($120)) + 28|0);
      HEAP32[$121>>2] = $119;
      $122 = HEAP32[$p>>2]|0;
      $123 = ((($122)) + 2|0);
      $124 = HEAP16[$123>>1]|0;
      $125 = $124&65535;
      $126 = $125 & 3;
      $127 = HEAP32[$extraFlags>>2]|0;
      $128 = $126 | $127;
      $129 = $128&65535;
      $130 = HEAP32[$p>>2]|0;
      $131 = ((($130)) + 2|0);
      HEAP16[$131>>1] = $129;
      $132 = HEAP32[$6>>2]|0;
      $133 = ($132|0)!=(0|0);
      $134 = HEAP32[$6>>2]|0;
      $135 = HEAP32[$7>>2]|0;
      $136 = $133 ? $134 : $135;
      $137 = HEAP32[$p>>2]|0;
      $138 = ((($137)) + 12|0);
      HEAP32[$138>>2] = $136;
      $139 = HEAP32[$8>>2]|0;
      $140 = HEAP32[$p>>2]|0;
      $141 = ((($140)) + 16|0);
      HEAP32[$141>>2] = $139;
      $142 = HEAP32[$5>>2]|0;
      $143 = HEAP32[$p>>2]|0;
      $144 = ((($143)) + 4|0);
      HEAP32[$144>>2] = $142;
      $145 = HEAP32[$3>>2]|0;
      $146 = $145&65535;
      $147 = HEAP32[$p>>2]|0;
      HEAP16[$147>>1] = $146;
      HEAP32[$0>>2] = 0;
      $148 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($148|0);
     }
    }
   }
  }
 } while(0);
 $37 = (_sqlite3MisuseError(134386)|0);
 HEAP32[$0>>2] = $37;
 $148 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($148|0);
}
function _sqlite3_overload_function($db,$zName,$nArg) {
 $db = $db|0;
 $zName = $zName|0;
 $nArg = $nArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nName = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $nArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3Strlen30($3)|0);
 HEAP32[$nName>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$nName>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3FindFunction($5,$6,$7,$8,1,0)|0);
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (_sqlite3CreateFunc($11,$12,$13,1,0,184,0,0,0)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$rc>>2]|0;
 $17 = (_sqlite3ApiExit($15,$16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3FindFunction($db,$zName,$nName,$nArg,$enc,$createFlag) {
 $db = $db|0;
 $zName = $zName|0;
 $nName = $nName|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bestScore = 0, $h = 0, $or$cond = 0, $p = 0, $pBest = 0, $pHash = 0, $score = 0, $score1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 49|0;
 $6 = sp + 48|0;
 $p = sp + 24|0;
 $pBest = sp + 20|0;
 $bestScore = sp + 16|0;
 $h = sp + 12|0;
 $score = sp + 8|0;
 $pHash = sp + 4|0;
 $score1 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $nName;
 HEAP32[$4>>2] = $nArg;
 HEAP8[$5>>0] = $enc;
 HEAP8[$6>>0] = $createFlag;
 HEAP32[$pBest>>2] = 0;
 HEAP32[$bestScore>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = (16974 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = HEAP32[$3>>2]|0;
 $14 = (($12) + ($13))|0;
 $15 = (($14|0) % 23)&-1;
 HEAP32[$h>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 340|0);
 $18 = HEAP32[$h>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (_functionSearch($17,$18,$19,$20)|0);
 HEAP32[$p>>2] = $21;
 while(1) {
  $22 = HEAP32[$p>>2]|0;
  $23 = ($22|0)!=(0|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$p>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = HEAP8[$5>>0]|0;
  $27 = (_matchQuality($24,$25,$26)|0);
  HEAP32[$score>>2] = $27;
  $28 = HEAP32[$score>>2]|0;
  $29 = HEAP32[$bestScore>>2]|0;
  $30 = ($28|0)>($29|0);
  if ($30) {
   $31 = HEAP32[$p>>2]|0;
   HEAP32[$pBest>>2] = $31;
   $32 = HEAP32[$score>>2]|0;
   HEAP32[$bestScore>>2] = $32;
  }
  $33 = HEAP32[$p>>2]|0;
  $34 = ((($33)) + 8|0);
  $35 = HEAP32[$34>>2]|0;
  HEAP32[$p>>2] = $35;
 }
 $36 = HEAP8[$6>>0]|0;
 $37 = ($36<<24>>24)!=(0);
 L8: do {
  if (!($37)) {
   $38 = HEAP32[$pBest>>2]|0;
   $39 = ($38|0)==(0|0);
   if (!($39)) {
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = $42 & 2097152;
    $44 = ($43|0)!=(0);
    if (!($44)) {
     break;
    }
   }
   HEAP32[$pHash>>2] = 848;
   HEAP32[$bestScore>>2] = 0;
   $45 = HEAP32[$pHash>>2]|0;
   $46 = HEAP32[$h>>2]|0;
   $47 = HEAP32[$2>>2]|0;
   $48 = HEAP32[$3>>2]|0;
   $49 = (_functionSearch($45,$46,$47,$48)|0);
   HEAP32[$p>>2] = $49;
   while(1) {
    $50 = HEAP32[$p>>2]|0;
    $51 = ($50|0)!=(0|0);
    if (!($51)) {
     break L8;
    }
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP32[$4>>2]|0;
    $54 = HEAP8[$5>>0]|0;
    $55 = (_matchQuality($52,$53,$54)|0);
    HEAP32[$score1>>2] = $55;
    $56 = HEAP32[$score1>>2]|0;
    $57 = HEAP32[$bestScore>>2]|0;
    $58 = ($56|0)>($57|0);
    if ($58) {
     $59 = HEAP32[$p>>2]|0;
     HEAP32[$pBest>>2] = $59;
     $60 = HEAP32[$score1>>2]|0;
     HEAP32[$bestScore>>2] = $60;
    }
    $61 = HEAP32[$p>>2]|0;
    $62 = ((($61)) + 8|0);
    $63 = HEAP32[$62>>2]|0;
    HEAP32[$p>>2] = $63;
   }
  }
 } while(0);
 $64 = HEAP8[$6>>0]|0;
 $65 = $64&255;
 $66 = ($65|0)!=(0);
 $67 = HEAP32[$bestScore>>2]|0;
 $68 = ($67|0)<(6);
 $or$cond = $66 & $68;
 if ($or$cond) {
  $69 = HEAP32[$1>>2]|0;
  $70 = HEAP32[$3>>2]|0;
  $71 = (32 + ($70))|0;
  $72 = (($71) + 1)|0;
  $73 = (_sqlite3DbMallocZero($69,$72,0)|0);
  HEAP32[$pBest>>2] = $73;
  $74 = ($73|0)!=(0|0);
  if ($74) {
   $75 = HEAP32[$pBest>>2]|0;
   $76 = ((($75)) + 32|0);
   $77 = HEAP32[$pBest>>2]|0;
   $78 = ((($77)) + 20|0);
   HEAP32[$78>>2] = $76;
   $79 = HEAP32[$4>>2]|0;
   $80 = $79&65535;
   $81 = HEAP32[$pBest>>2]|0;
   HEAP16[$81>>1] = $80;
   $82 = HEAP8[$5>>0]|0;
   $83 = $82&255;
   $84 = HEAP32[$pBest>>2]|0;
   $85 = ((($84)) + 2|0);
   HEAP16[$85>>1] = $83;
   $86 = HEAP32[$pBest>>2]|0;
   $87 = ((($86)) + 20|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = HEAP32[$3>>2]|0;
   _memcpy(($88|0),($89|0),($90|0))|0;
   $91 = HEAP32[$3>>2]|0;
   $92 = HEAP32[$pBest>>2]|0;
   $93 = ((($92)) + 20|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) + ($91)|0);
   HEAP8[$95>>0] = 0;
   $96 = HEAP32[$1>>2]|0;
   $97 = ((($96)) + 340|0);
   $98 = HEAP32[$pBest>>2]|0;
   _sqlite3FuncDefInsert($97,$98);
  }
 }
 $99 = HEAP32[$pBest>>2]|0;
 $100 = ($99|0)!=(0|0);
 do {
  if ($100) {
   $101 = HEAP32[$pBest>>2]|0;
   $102 = ((($101)) + 12|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = ($103|0)!=(0|0);
   if (!($104)) {
    $105 = HEAP8[$6>>0]|0;
    $106 = $105&255;
    $107 = ($106|0)!=(0);
    if (!($107)) {
     break;
    }
   }
   $108 = HEAP32[$pBest>>2]|0;
   HEAP32[$0>>2] = $108;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $109 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($109|0);
}
function _sqlite3InvalidFunction($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zName = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$zName>>2] = $7;
 $8 = HEAP32[$zName>>2]|0;
 HEAP32[$vararg_buffer>>2] = $8;
 $9 = (_sqlite3_mprintf(39796,$vararg_buffer)|0);
 HEAP32[$zErr>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$zErr>>2]|0;
 _sqlite3_result_error($10,$11,-1);
 $12 = HEAP32[$zErr>>2]|0;
 _sqlite3_free($12);
 STACKTOP = sp;return;
}
function _sqlite3_wal_autocheckpoint($db,$nFrame) {
 $db = $db|0;
 $nFrame = $nFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nFrame;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 $4 = HEAP32[$0>>2]|0;
 if ($3) {
  $5 = HEAP32[$1>>2]|0;
  $6 = $5;
  (_sqlite3_wal_hook($4,139,$6)|0);
  STACKTOP = sp;return 0;
 } else {
  (_sqlite3_wal_hook($4,0,0)|0);
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _sqlite3_wal_hook($db,$xCallback,$pArg) {
 $db = $db|0;
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $xCallback;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 220|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pRet>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 216|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 220|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3WalDefaultHook($pClientData,$db,$zDb,$nFrame) {
 $pClientData = $pClientData|0;
 $db = $db|0;
 $zDb = $zDb|0;
 $nFrame = $nFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pClientData;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $nFrame;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = $5;
 $7 = ($4|0)>=($6|0);
 if (!($7)) {
  STACKTOP = sp;return 0;
 }
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3_wal_checkpoint($8,$9)|0);
 _sqlite3EndBenignMalloc();
 STACKTOP = sp;return 0;
}
function _sqlite3_wal_checkpoint_v2($db,$zDb,$eMode,$pnLog,$pnCkpt) {
 $db = $db|0;
 $zDb = $zDb|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $rc = sp + 8|0;
 $iDb = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $eMode;
 HEAP32[$4>>2] = $pnLog;
 HEAP32[$5>>2] = $pnCkpt;
 HEAP32[$iDb>>2] = 10;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$4>>2]|0;
  HEAP32[$8>>2] = -1;
 }
 $9 = HEAP32[$5>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$5>>2]|0;
  HEAP32[$11>>2] = -1;
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = ($12|0)<(0);
 $14 = HEAP32[$3>>2]|0;
 $15 = ($14|0)>(3);
 $or$cond = $13 | $15;
 if ($or$cond) {
  HEAP32[$0>>2] = 21;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$2>>2]|0;
   $24 = (_sqlite3FindDbName($22,$23)|0);
   HEAP32[$iDb>>2] = $24;
  }
 }
 $25 = HEAP32[$iDb>>2]|0;
 $26 = ($25|0)<(0);
 if ($26) {
  HEAP32[$rc>>2] = 1;
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $28;
  _sqlite3ErrorWithMsg($27,1,17868,$vararg_buffer);
 } else {
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 448|0);
  $31 = ((($30)) + 8|0);
  HEAP32[$31>>2] = 0;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$iDb>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  $36 = HEAP32[$5>>2]|0;
  $37 = (_sqlite3Checkpoint($32,$33,$34,$35,$36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$rc>>2]|0;
  _sqlite3Error($38,$39);
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$rc>>2]|0;
 $42 = (_sqlite3ApiExit($40,$41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3FindDbName($db,$zName) {
 $db = $db|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $pDb = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$i>>2] = -1;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $31 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 HEAP32[$i>>2] = $9;
 $10 = HEAP32[$i>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<4)|0);
 HEAP32[$pDb>>2] = $14;
 while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = ($15|0)>=(0);
  if (!($16)) {
   label = 7;
   break;
  }
  $17 = HEAP32[$n>>2]|0;
  $18 = HEAP32[$pDb>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3Strlen30($19)|0);
  $21 = ($17|0)==($20|0);
  if ($21) {
   $22 = HEAP32[$pDb>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_sqlite3_stricmp($23,$24)|0);
   $26 = (0)==($25|0);
   if ($26) {
    label = 7;
    break;
   }
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + -1)|0;
  HEAP32[$i>>2] = $28;
  $29 = HEAP32[$pDb>>2]|0;
  $30 = ((($29)) + -16|0);
  HEAP32[$pDb>>2] = $30;
 }
 if ((label|0) == 7) {
  $31 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 return (0)|0;
}
function _sqlite3Checkpoint($db,$iDb,$eMode,$pnLog,$pnCkpt) {
 $db = $db|0;
 $iDb = $iDb|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bBusy = 0;
 var $i = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $bBusy = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $pnLog;
 HEAP32[$4>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bBusy>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(0);
  $12 = $9 ? $11 : 0;
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ($13|0)==($14|0);
  $16 = HEAP32[$1>>2]|0;
  $17 = ($16|0)==(10);
  $or$cond = $15 | $17;
  if ($or$cond) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18<<4)|0);
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$2>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$4>>2]|0;
   $28 = (_sqlite3BtreeCheckpoint($24,$25,$26,$27)|0);
   HEAP32[$rc>>2] = $28;
   HEAP32[$3>>2] = 0;
   HEAP32[$4>>2] = 0;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(5);
   if ($30) {
    HEAP32[$bBusy>>2] = 1;
    HEAP32[$rc>>2] = 0;
   }
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
 }
 $33 = HEAP32[$rc>>2]|0;
 $34 = ($33|0)==(0);
 $35 = HEAP32[$bBusy>>2]|0;
 $36 = ($35|0)!=(0);
 $or$cond3 = $34 & $36;
 $37 = HEAP32[$rc>>2]|0;
 $38 = $or$cond3 ? 5 : $37;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3_wal_checkpoint($db,$zDb) {
 $db = $db|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_wal_checkpoint_v2($2,$3,0,0,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3SafetyCheckSickOrOk($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $magic = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $magic = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 76|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$magic>>2] = $4;
 $5 = HEAP32[$magic>>2]|0;
 $6 = ($5|0)!=(1266094736);
 $7 = HEAP32[$magic>>2]|0;
 $8 = ($7|0)!=(-1607883113);
 $or$cond = $6 & $8;
 $9 = HEAP32[$magic>>2]|0;
 $10 = ($9|0)!=(-264537850);
 $or$cond3 = $or$cond & $10;
 if ($or$cond3) {
  _logBadConnection(39847);
  HEAP32[$0>>2] = 0;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 } else {
  HEAP32[$0>>2] = 1;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 return (0)|0;
}
function _sqlite3OomClear($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 65|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 160|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 65|0);
 HEAP8[$11>>0] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 240|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 248|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + -1)|0;
 HEAP32[$15>>2] = $17;
 STACKTOP = sp;return;
}
function _sqlite3_errcode($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3SafetyCheckSickOrOk($4)|0);
  $6 = ($5|0)!=(0);
  if (!($6)) {
   $7 = (_sqlite3MisuseError(135016)|0);
   HEAP32[$0>>2] = $7;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 65|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 52|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 56|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $17 & $20;
   HEAP32[$0>>2] = $21;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 HEAP32[$0>>2] = 7;
 $22 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3_limit($db,$limitId,$newLimit) {
 $db = $db|0;
 $limitId = $limitId|0;
 $newLimit = $newLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $oldLimit = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $oldLimit = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $limitId;
 HEAP32[$3>>2] = $newLimit;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)<(0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>=(12);
 $or$cond = $5 | $7;
 if ($or$cond) {
  HEAP32[$0>>2] = -1;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 88|0);
 $11 = (($10) + ($8<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$oldLimit>>2] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)>=(0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (940 + ($16<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)>($18|0);
  if ($19) {
   $20 = HEAP32[$2>>2]|0;
   $21 = (940 + ($20<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$3>>2] = $22;
  }
  $23 = HEAP32[$3>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 88|0);
  $27 = (($26) + ($24<<2)|0);
  HEAP32[$27>>2] = $23;
 }
 $28 = HEAP32[$oldLimit>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3_open($zFilename,$ppDb) {
 $zFilename = $zFilename|0;
 $ppDb = $ppDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $ppDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_openDatabase($2,$3,6,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _openDatabase($zFilename,$ppDb,$flags,$zVfs) {
 $zFilename = $zFilename|0;
 $ppDb = $ppDb|0;
 $flags = $flags|0;
 $zVfs = $zVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $db = 0, $isThreadsafe = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, $zErrMsg = 0, $zOpen = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $db = sp + 20|0;
 $rc = sp + 16|0;
 $isThreadsafe = sp + 12|0;
 $zOpen = sp + 8|0;
 $zErrMsg = sp + 4|0;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $ppDb;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $zVfs;
 HEAP32[$zOpen>>2] = 0;
 HEAP32[$zErrMsg>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $239 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($239|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = $10 & 7;
 $12 = 1 << $11;
 $13 = $12 & 70;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = (_sqlite3MisuseError(135519)|0);
  HEAP32[$0>>2] = $15;
  $239 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($239|0);
 }
 $16 = HEAP32[(12)>>2]|0;
 $17 = ($16|0)==(0);
 do {
  if ($17) {
   HEAP32[$isThreadsafe>>2] = 0;
  } else {
   $18 = HEAP32[$3>>2]|0;
   $19 = $18 & 32768;
   $20 = ($19|0)!=(0);
   if ($20) {
    HEAP32[$isThreadsafe>>2] = 0;
    break;
   }
   $21 = HEAP32[$3>>2]|0;
   $22 = $21 & 65536;
   $23 = ($22|0)!=(0);
   if ($23) {
    HEAP32[$isThreadsafe>>2] = 1;
    break;
   } else {
    $24 = HEAP32[(16)>>2]|0;
    HEAP32[$isThreadsafe>>2] = $24;
    break;
   }
  }
 } while(0);
 $25 = HEAP32[$3>>2]|0;
 $26 = $25 & 262144;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$3>>2]|0;
  $29 = $28 & -131073;
  HEAP32[$3>>2] = $29;
 } else {
  $30 = HEAP32[(228)>>2]|0;
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$3>>2]|0;
   $33 = $32 | 131072;
   HEAP32[$3>>2] = $33;
  }
 }
 $34 = HEAP32[$3>>2]|0;
 $35 = $34 & -655129;
 HEAP32[$3>>2] = $35;
 $36 = (_sqlite3MallocZero(536,0)|0);
 HEAP32[$db>>2] = $36;
 $37 = HEAP32[$db>>2]|0;
 $38 = ($37|0)==(0|0);
 do {
  if (!($38)) {
   $39 = HEAP32[$isThreadsafe>>2]|0;
   $40 = ($39|0)!=(0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = ((($41)) + 12|0);
    HEAP32[$42>>2] = (8);
    $43 = HEAP32[$db>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)==(0|0);
    if ($46) {
     $47 = HEAP32[$db>>2]|0;
     _sqlite3_free($47);
     HEAP32[$db>>2] = 0;
     break;
    }
   }
   $48 = HEAP32[$db>>2]|0;
   $49 = ((($48)) + 56|0);
   HEAP32[$49>>2] = 255;
   $50 = HEAP32[$db>>2]|0;
   $51 = ((($50)) + 20|0);
   HEAP32[$51>>2] = 2;
   $52 = HEAP32[$db>>2]|0;
   $53 = ((($52)) + 76|0);
   HEAP32[$53>>2] = -264537850;
   $54 = HEAP32[$db>>2]|0;
   $55 = ((($54)) + 460|0);
   $56 = HEAP32[$db>>2]|0;
   $57 = ((($56)) + 16|0);
   HEAP32[$57>>2] = $55;
   $58 = HEAP32[$db>>2]|0;
   $59 = ((($58)) + 88|0);
   dest=$59; src=940; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   $60 = HEAP32[$db>>2]|0;
   $61 = ((($60)) + 88|0);
   $62 = ((($61)) + 44|0);
   HEAP32[$62>>2] = 0;
   $63 = HEAP32[$db>>2]|0;
   $64 = ((($63)) + 63|0);
   HEAP8[$64>>0] = 1;
   $65 = HEAP32[$db>>2]|0;
   $66 = ((($65)) + 68|0);
   HEAP8[$66>>0] = -1;
   $67 = (184);
   $68 = $67;
   $69 = HEAP32[$68>>2]|0;
   $70 = (($67) + 4)|0;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$db>>2]|0;
   $74 = ((($73)) + 40|0);
   $75 = $74;
   $76 = $75;
   HEAP32[$76>>2] = $69;
   $77 = (($75) + 4)|0;
   $78 = $77;
   HEAP32[$78>>2] = $72;
   $79 = HEAP32[$db>>2]|0;
   $80 = ((($79)) + 72|0);
   HEAP32[$80>>2] = 0;
   $81 = HEAP32[$db>>2]|0;
   $82 = ((($81)) + 136|0);
   HEAP32[$82>>2] = 2147483647;
   $83 = HEAP32[$db>>2]|0;
   $84 = ((($83)) + 24|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $85 | 9437280;
   HEAP32[$84>>2] = $86;
   $87 = HEAP32[$db>>2]|0;
   $88 = ((($87)) + 432|0);
   _sqlite3HashInit($88);
   $89 = HEAP32[$db>>2]|0;
   $90 = ((($89)) + 312|0);
   _sqlite3HashInit($90);
   $91 = HEAP32[$db>>2]|0;
   (_createCollation($91,20716,1,0,141,0)|0);
   $92 = HEAP32[$db>>2]|0;
   (_createCollation($92,20716,3,0,141,0)|0);
   $93 = HEAP32[$db>>2]|0;
   (_createCollation($93,20716,2,0,141,0)|0);
   $94 = HEAP32[$db>>2]|0;
   (_createCollation($94,32558,1,0,142,0)|0);
   $95 = HEAP32[$db>>2]|0;
   (_createCollation($95,39855,1,(1),141,0)|0);
   $96 = HEAP32[$db>>2]|0;
   $97 = ((($96)) + 65|0);
   $98 = HEAP8[$97>>0]|0;
   $99 = ($98<<24>>24)!=(0);
   if (!($99)) {
    $100 = HEAP32[$db>>2]|0;
    $101 = (_sqlite3FindCollSeq($100,1,20716,0)|0);
    $102 = HEAP32[$db>>2]|0;
    $103 = ((($102)) + 8|0);
    HEAP32[$103>>2] = $101;
    $104 = HEAP32[$3>>2]|0;
    $105 = HEAP32[$db>>2]|0;
    $106 = ((($105)) + 48|0);
    HEAP32[$106>>2] = $104;
    $107 = HEAP32[$4>>2]|0;
    $108 = HEAP32[$1>>2]|0;
    $109 = HEAP32[$db>>2]|0;
    $110 = (_sqlite3ParseUri($107,$108,$3,$109,$zOpen,$zErrMsg)|0);
    HEAP32[$rc>>2] = $110;
    $111 = HEAP32[$rc>>2]|0;
    $112 = ($111|0)!=(0);
    if ($112) {
     $113 = HEAP32[$rc>>2]|0;
     $114 = ($113|0)==(7);
     if ($114) {
      $115 = HEAP32[$db>>2]|0;
      _sqlite3OomFault($115);
     }
     $116 = HEAP32[$db>>2]|0;
     $117 = HEAP32[$rc>>2]|0;
     $118 = HEAP32[$zErrMsg>>2]|0;
     $119 = ($118|0)!=(0|0);
     $120 = $119 ? 17521 : 0;
     $121 = HEAP32[$zErrMsg>>2]|0;
     HEAP32[$vararg_buffer>>2] = $121;
     _sqlite3ErrorWithMsg($116,$117,$120,$vararg_buffer);
     $122 = HEAP32[$zErrMsg>>2]|0;
     _sqlite3_free($122);
     break;
    }
    $123 = HEAP32[$db>>2]|0;
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP32[$zOpen>>2]|0;
    $126 = HEAP32[$db>>2]|0;
    $127 = HEAP32[$db>>2]|0;
    $128 = ((($127)) + 16|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = ((($129)) + 4|0);
    $131 = HEAP32[$3>>2]|0;
    $132 = $131 | 256;
    $133 = (_sqlite3BtreeOpen($124,$125,$126,$130,0,$132)|0);
    HEAP32[$rc>>2] = $133;
    $134 = HEAP32[$rc>>2]|0;
    $135 = ($134|0)!=(0);
    if ($135) {
     $136 = HEAP32[$rc>>2]|0;
     $137 = ($136|0)==(3082);
     if ($137) {
      HEAP32[$rc>>2] = 7;
     }
     $138 = HEAP32[$db>>2]|0;
     $139 = HEAP32[$rc>>2]|0;
     _sqlite3Error($138,$139);
     break;
    }
    $140 = HEAP32[$db>>2]|0;
    $141 = ((($140)) + 16|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ((($142)) + 4|0);
    $144 = HEAP32[$143>>2]|0;
    _sqlite3BtreeEnter($144);
    $145 = HEAP32[$db>>2]|0;
    $146 = HEAP32[$db>>2]|0;
    $147 = ((($146)) + 16|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = ((($148)) + 4|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = (_sqlite3SchemaGet($145,$150)|0);
    $152 = HEAP32[$db>>2]|0;
    $153 = ((($152)) + 16|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = ((($154)) + 12|0);
    HEAP32[$155>>2] = $151;
    $156 = HEAP32[$db>>2]|0;
    $157 = ((($156)) + 65|0);
    $158 = HEAP8[$157>>0]|0;
    $159 = ($158<<24>>24)!=(0);
    if (!($159)) {
     $160 = HEAP32[$db>>2]|0;
     $161 = ((($160)) + 16|0);
     $162 = HEAP32[$161>>2]|0;
     $163 = ((($162)) + 12|0);
     $164 = HEAP32[$163>>2]|0;
     $165 = ((($164)) + 77|0);
     $166 = HEAP8[$165>>0]|0;
     $167 = HEAP32[$db>>2]|0;
     $168 = ((($167)) + 62|0);
     HEAP8[$168>>0] = $166;
    }
    $169 = HEAP32[$db>>2]|0;
    $170 = (_sqlite3SchemaGet($169,0)|0);
    $171 = HEAP32[$db>>2]|0;
    $172 = ((($171)) + 16|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = ((($173)) + 16|0);
    $175 = ((($174)) + 12|0);
    HEAP32[$175>>2] = $170;
    $176 = HEAP32[$db>>2]|0;
    $177 = ((($176)) + 16|0);
    $178 = HEAP32[$177>>2]|0;
    HEAP32[$178>>2] = 39861;
    $179 = HEAP32[$db>>2]|0;
    $180 = ((($179)) + 16|0);
    $181 = HEAP32[$180>>2]|0;
    $182 = ((($181)) + 8|0);
    HEAP8[$182>>0] = 3;
    $183 = HEAP32[$db>>2]|0;
    $184 = ((($183)) + 16|0);
    $185 = HEAP32[$184>>2]|0;
    $186 = ((($185)) + 16|0);
    HEAP32[$186>>2] = 17802;
    $187 = HEAP32[$db>>2]|0;
    $188 = ((($187)) + 16|0);
    $189 = HEAP32[$188>>2]|0;
    $190 = ((($189)) + 16|0);
    $191 = ((($190)) + 8|0);
    HEAP8[$191>>0] = 1;
    $192 = HEAP32[$db>>2]|0;
    $193 = ((($192)) + 76|0);
    HEAP32[$193>>2] = -1607883113;
    $194 = HEAP32[$db>>2]|0;
    $195 = ((($194)) + 65|0);
    $196 = HEAP8[$195>>0]|0;
    $197 = ($196<<24>>24)!=(0);
    if (!($197)) {
     $198 = HEAP32[$db>>2]|0;
     _sqlite3Error($198,0);
     $199 = HEAP32[$db>>2]|0;
     _sqlite3RegisterBuiltinFunctions($199);
     $200 = HEAP32[$db>>2]|0;
     $201 = (_sqlite3_errcode($200)|0);
     HEAP32[$rc>>2] = $201;
     $202 = HEAP32[$rc>>2]|0;
     $203 = ($202|0)==(0);
     if ($203) {
      $204 = HEAP32[$db>>2]|0;
      _sqlite3AutoLoadExtensions($204);
      $205 = HEAP32[$db>>2]|0;
      $206 = (_sqlite3_errcode($205)|0);
      HEAP32[$rc>>2] = $206;
      $207 = HEAP32[$rc>>2]|0;
      $208 = ($207|0)!=(0);
      if ($208) {
       break;
      }
     }
     $209 = HEAP32[$db>>2]|0;
     $210 = ((($209)) + 65|0);
     $211 = HEAP8[$210>>0]|0;
     $212 = ($211<<24>>24)==(0);
     $213 = HEAP32[$rc>>2]|0;
     $214 = ($213|0)==(0);
     $or$cond = $212 & $214;
     if ($or$cond) {
      $215 = HEAP32[$db>>2]|0;
      $216 = (_sqlite3Fts3Init($215)|0);
      HEAP32[$rc>>2] = $216;
     }
     $217 = HEAP32[$rc>>2]|0;
     $218 = ($217|0)!=(0);
     if ($218) {
      $219 = HEAP32[$db>>2]|0;
      $220 = HEAP32[$rc>>2]|0;
      _sqlite3Error($219,$220);
     }
     $221 = HEAP32[$db>>2]|0;
     $222 = HEAP32[(36)>>2]|0;
     $223 = HEAP32[(40)>>2]|0;
     (_setupLookaside($221,0,$222,$223)|0);
     $224 = HEAP32[$db>>2]|0;
     (_sqlite3_wal_autocheckpoint($224,1000)|0);
    }
   }
  }
 } while(0);
 $225 = HEAP32[$db>>2]|0;
 $226 = (_sqlite3_errcode($225)|0);
 HEAP32[$rc>>2] = $226;
 $227 = HEAP32[$rc>>2]|0;
 $228 = ($227|0)==(7);
 if ($228) {
  $229 = HEAP32[$db>>2]|0;
  (_sqlite3_close($229)|0);
  HEAP32[$db>>2] = 0;
 } else {
  $230 = HEAP32[$rc>>2]|0;
  $231 = ($230|0)!=(0);
  if ($231) {
   $232 = HEAP32[$db>>2]|0;
   $233 = ((($232)) + 76|0);
   HEAP32[$233>>2] = 1266094736;
  }
 }
 $234 = HEAP32[$db>>2]|0;
 $235 = HEAP32[$2>>2]|0;
 HEAP32[$235>>2] = $234;
 $236 = HEAP32[$zOpen>>2]|0;
 _sqlite3_free($236);
 $237 = HEAP32[$rc>>2]|0;
 $238 = $237 & 255;
 HEAP32[$0>>2] = $238;
 $239 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($239|0);
}
function _createCollation($db,$zName,$enc,$pCtx,$xCompare,$xDel) {
 $db = $db|0;
 $zName = $zName|0;
 $enc = $enc|0;
 $pCtx = $pCtx|0;
 $xCompare = $xCompare|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $aColl = 0, $enc2 = 0, $j = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pColl = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 48|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $pColl = sp + 20|0;
 $enc2 = sp + 16|0;
 $aColl = sp + 12|0;
 $j = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zName;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $pCtx;
 HEAP32[$5>>2] = $xCompare;
 HEAP32[$6>>2] = $xDel;
 $7 = HEAP8[$3>>0]|0;
 $8 = $7&255;
 HEAP32[$enc2>>2] = $8;
 $9 = HEAP32[$enc2>>2]|0;
 $10 = ($9|0)==(4);
 $11 = HEAP32[$enc2>>2]|0;
 $12 = ($11|0)==(8);
 $or$cond = $10 | $12;
 if ($or$cond) {
  $13 = HEAP8[352>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = ($14|0)==(0);
  $16 = $15 ? 3 : 2;
  HEAP32[$enc2>>2] = $16;
 }
 $17 = HEAP32[$enc2>>2]|0;
 $18 = ($17|0)<(1);
 $19 = HEAP32[$enc2>>2]|0;
 $20 = ($19|0)>(3);
 $or$cond3 = $18 | $20;
 if ($or$cond3) {
  $21 = (_sqlite3MisuseError(135070)|0);
  HEAP32[$0>>2] = $21;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$enc2>>2]|0;
 $24 = $23&255;
 $25 = HEAP32[$2>>2]|0;
 $26 = (_sqlite3FindCollSeq($22,$24,$25,0)|0);
 HEAP32[$pColl>>2] = $26;
 $27 = HEAP32[$pColl>>2]|0;
 $28 = ($27|0)!=(0|0);
 L8: do {
  if ($28) {
   $29 = HEAP32[$pColl>>2]|0;
   $30 = ((($29)) + 12|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 148|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)!=(0);
    $37 = HEAP32[$1>>2]|0;
    if ($36) {
     _sqlite3ErrorWithMsg($37,5,45141,$vararg_buffer);
     HEAP32[$0>>2] = 5;
     $102 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($102|0);
    }
    _sqlite3ExpirePreparedStatements($37);
    $38 = HEAP32[$pColl>>2]|0;
    $39 = ((($38)) + 4|0);
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    $42 = $41 & -9;
    $43 = HEAP32[$enc2>>2]|0;
    $44 = ($42|0)==($43|0);
    if ($44) {
     $45 = HEAP32[$1>>2]|0;
     $46 = ((($45)) + 432|0);
     $47 = HEAP32[$2>>2]|0;
     $48 = (_sqlite3HashFind($46,$47)|0);
     HEAP32[$aColl>>2] = $48;
     HEAP32[$j>>2] = 0;
     while(1) {
      $49 = HEAP32[$j>>2]|0;
      $50 = ($49|0)<(3);
      if (!($50)) {
       break L8;
      }
      $51 = HEAP32[$j>>2]|0;
      $52 = HEAP32[$aColl>>2]|0;
      $53 = (($52) + (($51*20)|0)|0);
      HEAP32[$p>>2] = $53;
      $54 = HEAP32[$p>>2]|0;
      $55 = ((($54)) + 4|0);
      $56 = HEAP8[$55>>0]|0;
      $57 = $56&255;
      $58 = HEAP32[$pColl>>2]|0;
      $59 = ((($58)) + 4|0);
      $60 = HEAP8[$59>>0]|0;
      $61 = $60&255;
      $62 = ($57|0)==($61|0);
      if ($62) {
       $63 = HEAP32[$p>>2]|0;
       $64 = ((($63)) + 16|0);
       $65 = HEAP32[$64>>2]|0;
       $66 = ($65|0)!=(0|0);
       if ($66) {
        $67 = HEAP32[$p>>2]|0;
        $68 = ((($67)) + 16|0);
        $69 = HEAP32[$68>>2]|0;
        $70 = HEAP32[$p>>2]|0;
        $71 = ((($70)) + 8|0);
        $72 = HEAP32[$71>>2]|0;
        FUNCTION_TABLE_vi[$69 & 255]($72);
       }
       $73 = HEAP32[$p>>2]|0;
       $74 = ((($73)) + 12|0);
       HEAP32[$74>>2] = 0;
      }
      $75 = HEAP32[$j>>2]|0;
      $76 = (($75) + 1)|0;
      HEAP32[$j>>2] = $76;
     }
    }
   }
  }
 } while(0);
 $77 = HEAP32[$1>>2]|0;
 $78 = HEAP32[$enc2>>2]|0;
 $79 = $78&255;
 $80 = HEAP32[$2>>2]|0;
 $81 = (_sqlite3FindCollSeq($77,$79,$80,1)|0);
 HEAP32[$pColl>>2] = $81;
 $82 = HEAP32[$pColl>>2]|0;
 $83 = ($82|0)==(0|0);
 if ($83) {
  HEAP32[$0>>2] = 7;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 } else {
  $84 = HEAP32[$5>>2]|0;
  $85 = HEAP32[$pColl>>2]|0;
  $86 = ((($85)) + 12|0);
  HEAP32[$86>>2] = $84;
  $87 = HEAP32[$4>>2]|0;
  $88 = HEAP32[$pColl>>2]|0;
  $89 = ((($88)) + 8|0);
  HEAP32[$89>>2] = $87;
  $90 = HEAP32[$6>>2]|0;
  $91 = HEAP32[$pColl>>2]|0;
  $92 = ((($91)) + 16|0);
  HEAP32[$92>>2] = $90;
  $93 = HEAP32[$enc2>>2]|0;
  $94 = HEAP8[$3>>0]|0;
  $95 = $94&255;
  $96 = $95 & 8;
  $97 = $93 | $96;
  $98 = $97&255;
  $99 = HEAP32[$pColl>>2]|0;
  $100 = ((($99)) + 4|0);
  HEAP8[$100>>0] = $98;
  $101 = HEAP32[$1>>2]|0;
  _sqlite3Error($101,0);
  HEAP32[$0>>2] = 0;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 return (0)|0;
}
function _sqlite3Init($db,$pzErrMsg) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $commit_internal = 0, $i = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 $commit_internal = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pzErrMsg;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 $7 = $6 ^ 1;
 $8 = $7&1;
 HEAP32[$commit_internal>>2] = $8;
 HEAP32[$rc>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 140|0);
 $11 = ((($10)) + 5|0);
 HEAP8[$11>>0] = 1;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 77|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 62|0);
 HEAP8[$20>>0] = $18;
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$rc>>2]|0;
  $22 = ($21|0)==(0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28<<4)|0);
  $33 = ((($32)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ((($34)) + 78|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 & 1;
  $39 = ($38|0)==(1);
  $40 = HEAP32[$i>>2]|0;
  $41 = ($40|0)==(1);
  $or$cond = $39 | $41;
  if (!($or$cond)) {
   $42 = HEAP32[$0>>2]|0;
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = (_sqlite3InitOne($42,$43,$44)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$0>>2]|0;
    $49 = HEAP32[$i>>2]|0;
    _sqlite3ResetOneSchema($48,$49);
   }
  }
  $50 = HEAP32[$i>>2]|0;
  $51 = (($50) + 1)|0;
  HEAP32[$i>>2] = $51;
 }
 $52 = HEAP32[$rc>>2]|0;
 $53 = ($52|0)==(0);
 if ($53) {
  $54 = HEAP32[$0>>2]|0;
  $55 = ((($54)) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ((($56)) + 16|0);
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ((($59)) + 78|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61&65535;
  $63 = $62 & 1;
  $64 = ($63|0)==(1);
  if (!($64)) {
   $65 = HEAP32[$0>>2]|0;
   $66 = HEAP32[$1>>2]|0;
   $67 = (_sqlite3InitOne($65,1,$66)|0);
   HEAP32[$rc>>2] = $67;
   $68 = HEAP32[$rc>>2]|0;
   $69 = ($68|0)!=(0);
   if ($69) {
    $70 = HEAP32[$0>>2]|0;
    _sqlite3ResetOneSchema($70,1);
   }
  }
 }
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 140|0);
 $73 = ((($72)) + 5|0);
 HEAP8[$73>>0] = 0;
 $74 = HEAP32[$rc>>2]|0;
 $75 = ($74|0)==(0);
 $76 = HEAP32[$commit_internal>>2]|0;
 $77 = ($76|0)!=(0);
 $or$cond3 = $75 & $77;
 if (!($or$cond3)) {
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $78 = HEAP32[$0>>2]|0;
 _sqlite3CommitInternalChanges($78);
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _sqlite3FindTable($db,$zName,$zDatabase) {
 $db = $db|0;
 $zName = $zName|0;
 $zDatabase = $zDatabase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $j = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $p = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $zDatabase;
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)<(2);
  $10 = HEAP32[$i>>2]|0;
  $11 = $10 ^ 1;
  $12 = $9 ? $11 : $10;
  HEAP32[$j>>2] = $12;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$j>>2]|0;
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + ($16<<4)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (_sqlite3_stricmp($15,$21)|0);
   $23 = ($22|0)!=(0);
   if (!($23)) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $24 = HEAP32[$j>>2]|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<4)|0);
   $29 = ((($28)) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ((($30)) + 8|0);
   $32 = HEAP32[$1>>2]|0;
   $33 = (_sqlite3HashFind($31,$32)|0);
   HEAP32[$p>>2] = $33;
   $34 = HEAP32[$p>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    label = 7;
    break;
   }
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 7) {
  $38 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 return (0)|0;
}
function _sqlite3IsRowid($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_stricmp($2,45393)|0);
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 1;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = (_sqlite3_stricmp($5,29600)|0);
   $7 = ($6|0)==(0);
   if ($7) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (_sqlite3_stricmp($8,45401)|0);
   $10 = ($9|0)==(0);
   if ($10) {
    HEAP32[$0>>2] = 1;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3OsSleep($pVfs,$nMicro) {
 $pVfs = $pVfs|0;
 $nMicro = $nMicro|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $nMicro;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$4 & 255]($5,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_file_control($db,$zDbName,$op,$pArg) {
 $db = $db|0;
 $zDbName = $zDbName|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $pBtree = 0, $pPager = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pBtree = sp + 8|0;
 $pPager = sp + 4|0;
 $fd = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDbName;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pArg;
 HEAP32[$rc>>2] = 1;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3DbNameToBtree($4,$5)|0);
 HEAP32[$pBtree>>2] = $6;
 $7 = HEAP32[$pBtree>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $9 = HEAP32[$pBtree>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBtree>>2]|0;
 $11 = (_sqlite3BtreePager($10)|0);
 HEAP32[$pPager>>2] = $11;
 $12 = HEAP32[$pPager>>2]|0;
 $13 = (_sqlite3PagerFile($12)|0);
 HEAP32[$fd>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)==(7);
 if ($15) {
  $16 = HEAP32[$fd>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ($18|0)==(27);
 if ($19) {
  $20 = HEAP32[$pPager>>2]|0;
  $21 = (_sqlite3PagerVfs($20)|0);
  $22 = HEAP32[$3>>2]|0;
  HEAP32[$22>>2] = $21;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)==(28);
 if ($24) {
  $25 = HEAP32[$pPager>>2]|0;
  $26 = (_sqlite3PagerJrnlFile($25)|0);
  $27 = HEAP32[$3>>2]|0;
  HEAP32[$27>>2] = $26;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $28 = HEAP32[$fd>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $31 = HEAP32[$fd>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  $34 = (_sqlite3OsFileControl($31,$32,$33)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  HEAP32[$rc>>2] = 12;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _sqlite3DbNameToBtree($db,$zDbName) {
 $db = $db|0;
 $zDbName = $zDbName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDbName;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 8;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$2>>2]|0;
   $17 = ($16|0)==(0|0);
   if ($17) {
    label = 6;
    break;
   }
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + ($19<<4)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (_sqlite3_stricmp($18,$24)|0);
   $26 = ($25|0)==(0);
   if ($26) {
    label = 6;
    break;
   }
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 if ((label|0) == 6) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<4)|0);
  $32 = ((($31)) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$0>>2] = $33;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 return (0)|0;
}
function _sqlite3PagerVfs($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3PagerJrnlFile($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ((($5)) + 216|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3WalFile($7)|0);
  $11 = $8;
  STACKTOP = sp;return ($11|0);
 } else {
  $9 = ((($5)) + 68|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10;
  STACKTOP = sp;return ($11|0);
 }
 return (0)|0;
}
function _sqlite3OsFileControl($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (FUNCTION_TABLE_iiii[$6 & 255]($7,$8,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3FaultSim($iTest) {
 $iTest = $iTest|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $xCallback = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $xCallback = sp;
 HEAP32[$0>>2] = $iTest;
 $1 = HEAP32[(272)>>2]|0;
 HEAP32[$xCallback>>2] = $1;
 $2 = HEAP32[$xCallback>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$xCallback>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 255]($5)|0);
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3BtreeSetPageSize($p,$pageSize,$nReserve,$iFix) {
 $p = $p|0;
 $pageSize = $pageSize|0;
 $nReserve = $nReserve|0;
 $iFix = $iFix|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pageSize;
 HEAP32[$3>>2] = $nReserve;
 HEAP32[$4>>2] = $iFix;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$pBt>>2]|0;
 $10 = ((($9)) + 22|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 2;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 8;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = HEAP32[$pBt>>2]|0;
  $18 = ((($17)) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = ((($20)) + 36|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) - ($22))|0;
  HEAP32[$3>>2] = $23;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)>=(512);
 $26 = HEAP32[$2>>2]|0;
 $27 = ($26|0)<=(65536);
 $or$cond = $25 & $27;
 if ($or$cond) {
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) - 1)|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = $29 & $30;
  $32 = ($31|0)==(0);
  if ($32) {
   $33 = HEAP32[$2>>2]|0;
   $34 = HEAP32[$pBt>>2]|0;
   $35 = ((($34)) + 32|0);
   HEAP32[$35>>2] = $33;
   $36 = HEAP32[$pBt>>2]|0;
   _freeTempSpace($36);
  }
 }
 $37 = HEAP32[$pBt>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$pBt>>2]|0;
 $40 = ((($39)) + 32|0);
 $41 = HEAP32[$3>>2]|0;
 $42 = (_sqlite3PagerSetPagesize($38,$40,$41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$pBt>>2]|0;
 $44 = ((($43)) + 32|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$3>>2]|0;
 $47 = $46&65535;
 $48 = $47&65535;
 $49 = (($45) - ($48))|0;
 $50 = HEAP32[$pBt>>2]|0;
 $51 = ((($50)) + 36|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$4>>2]|0;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = HEAP32[$pBt>>2]|0;
  $55 = ((($54)) + 22|0);
  $56 = HEAP16[$55>>1]|0;
  $57 = $56&65535;
  $58 = $57 | 2;
  $59 = $58&65535;
  HEAP16[$55>>1] = $59;
 }
 $60 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $60;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3KeywordCode($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $id = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $id = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 HEAP32[$id>>2] = 27;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 (_keywordCode($2,$3,$id)|0);
 $4 = HEAP32[$id>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3ScratchMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3StatusHighwater(8,$1);
 $2 = HEAP32[(304)>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[(204)>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)>=($5|0);
  if ($6) {
   $7 = HEAP32[(300)>>2]|0;
   HEAP32[$p>>2] = $7;
   $8 = HEAP32[(300)>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[(300)>>2] = $9;
   $10 = HEAP32[(304)>>2]|0;
   $11 = (($10) + -1)|0;
   HEAP32[(304)>>2] = $11;
   _sqlite3StatusUp(3,1);
   $22 = HEAP32[$p>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_sqlite3Malloc($12,$14)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[8>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = HEAP32[$p>>2]|0;
 $19 = ($18|0)!=(0|0);
 $or$cond = $17 & $19;
 if (!($or$cond)) {
  $22 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = (_sqlite3MallocSize($20)|0);
 _sqlite3StatusUp(4,$21);
 $22 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3ScratchFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iSize = 0, $pSlot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pSlot = sp + 4|0;
 $iSize = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[(200)>>2]|0;
 $5 = ($3>>>0)>=($4>>>0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[(296)>>2]|0;
  $8 = ($6>>>0)<($7>>>0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   HEAP32[$pSlot>>2] = $9;
   $10 = HEAP32[(300)>>2]|0;
   $11 = HEAP32[$pSlot>>2]|0;
   HEAP32[$11>>2] = $10;
   $12 = HEAP32[$pSlot>>2]|0;
   HEAP32[(300)>>2] = $12;
   $13 = HEAP32[(304)>>2]|0;
   $14 = (($13) + 1)|0;
   HEAP32[(304)>>2] = $14;
   _sqlite3StatusDown(3,1);
   STACKTOP = sp;return;
  }
 }
 $15 = HEAP32[8>>2]|0;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = (_sqlite3MallocSize($17)|0);
  HEAP32[$iSize>>2] = $18;
  $19 = HEAP32[$iSize>>2]|0;
  _sqlite3StatusDown(4,$19);
  $20 = HEAP32[$iSize>>2]|0;
  _sqlite3StatusDown(0,$20);
  _sqlite3StatusDown(9,1);
  $21 = HEAP32[(48)>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$21 & 255]($22);
  STACKTOP = sp;return;
 } else {
  $23 = HEAP32[(48)>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$23 & 255]($24);
  STACKTOP = sp;return;
 }
}
function _sqlite3_uri_parameter($zFilename,$zParam) {
 $zFilename = $zFilename|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $zParam;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 $or$cond = $4 | $6;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 $9 = (($8) + 1)|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + ($9)|0);
 HEAP32[$1>>2] = $11;
 while(1) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if (!($14)) {
   label = 8;
   break;
  }
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (_strcmp($15,$16)|0);
  HEAP32[$x>>2] = $17;
  $18 = HEAP32[$1>>2]|0;
  $19 = (_sqlite3Strlen30($18)|0);
  $20 = (($19) + 1)|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + ($20)|0);
  HEAP32[$1>>2] = $22;
  $23 = HEAP32[$x>>2]|0;
  $24 = ($23|0)==(0);
  $25 = HEAP32[$1>>2]|0;
  if ($24) {
   label = 6;
   break;
  }
  $26 = (_sqlite3Strlen30($25)|0);
  $27 = (($26) + 1)|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + ($27)|0);
  HEAP32[$1>>2] = $29;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $25;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _sqlite3_uri_boolean($zFilename,$zParam,$bDflt) {
 $zFilename = $zFilename|0;
 $zParam = $zParam|0;
 $bDflt = $bDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$2>>2] = $bDflt;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3_uri_parameter($3,$4)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = $7&1;
 HEAP32[$2>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$z>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = $12&255;
  $14 = (_sqlite3GetBoolean($11,$13)|0);
  $15 = $14&255;
  $17 = $15;
  STACKTOP = sp;return ($17|0);
 } else {
  $16 = HEAP32[$2>>2]|0;
  $17 = $16;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3GetBoolean($z,$dflt) {
 $z = $z|0;
 $dflt = $dflt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $z;
 HEAP8[$1>>0] = $dflt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = (_getSafetyLevel($2,1,$3)|0);
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 $7 = $6&1;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3DecOrHexToI64($z,$pOut) {
 $z = $z|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $i = 0, $k = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $u = sp;
 $i = sp + 12|0;
 $k = sp + 8|0;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(48);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 1|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)==(120);
   if (!($11)) {
    $12 = HEAP32[$1>>2]|0;
    $13 = ((($12)) + 1|0);
    $14 = HEAP8[$13>>0]|0;
    $15 = $14 << 24 >> 24;
    $16 = ($15|0)==(88);
    if (!($16)) {
     break;
    }
   }
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 2|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = (17533 + ($20)|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 & 8;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = $u;
    $27 = $26;
    HEAP32[$27>>2] = 0;
    $28 = (($26) + 4)|0;
    $29 = $28;
    HEAP32[$29>>2] = 0;
    HEAP32[$i>>2] = 2;
    while(1) {
     $30 = HEAP32[$i>>2]|0;
     $31 = HEAP32[$1>>2]|0;
     $32 = (($31) + ($30)|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33 << 24 >> 24;
     $35 = ($34|0)==(48);
     $36 = HEAP32[$i>>2]|0;
     if (!($35)) {
      break;
     }
     $37 = (($36) + 1)|0;
     HEAP32[$i>>2] = $37;
    }
    HEAP32[$k>>2] = $36;
    while(1) {
     $38 = HEAP32[$k>>2]|0;
     $39 = HEAP32[$1>>2]|0;
     $40 = (($39) + ($38)|0);
     $41 = HEAP8[$40>>0]|0;
     $42 = $41&255;
     $43 = (17533 + ($42)|0);
     $44 = HEAP8[$43>>0]|0;
     $45 = $44&255;
     $46 = $45 & 8;
     $47 = ($46|0)!=(0);
     if (!($47)) {
      break;
     }
     $48 = $u;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (($48) + 4)|0;
     $52 = $51;
     $53 = HEAP32[$52>>2]|0;
     $54 = (___muldi3(($50|0),($53|0),16,0)|0);
     $55 = tempRet0;
     $56 = HEAP32[$k>>2]|0;
     $57 = HEAP32[$1>>2]|0;
     $58 = (($57) + ($56)|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59 << 24 >> 24;
     $61 = (_sqlite3HexToInt($60)|0);
     $62 = $61&255;
     $63 = (_i64Add(($54|0),($55|0),($62|0),0)|0);
     $64 = tempRet0;
     $65 = $u;
     $66 = $65;
     HEAP32[$66>>2] = $63;
     $67 = (($65) + 4)|0;
     $68 = $67;
     HEAP32[$68>>2] = $64;
     $69 = HEAP32[$k>>2]|0;
     $70 = (($69) + 1)|0;
     HEAP32[$k>>2] = $70;
    }
    $71 = HEAP32[$2>>2]|0;
    ;HEAP32[$71>>2]=HEAP32[$u>>2]|0;HEAP32[$71+4>>2]=HEAP32[$u+4>>2]|0;
    $72 = HEAP32[$k>>2]|0;
    $73 = HEAP32[$1>>2]|0;
    $74 = (($73) + ($72)|0);
    $75 = HEAP8[$74>>0]|0;
    $76 = $75 << 24 >> 24;
    $77 = ($76|0)==(0);
    if ($77) {
     $78 = HEAP32[$k>>2]|0;
     $79 = HEAP32[$i>>2]|0;
     $80 = (($78) - ($79))|0;
     $81 = ($80|0)<=(16);
     $83 = $81;
    } else {
     $83 = 0;
    }
    $82 = $83 ? 0 : 1;
    HEAP32[$0>>2] = $82;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
  }
 } while(0);
 $84 = HEAP32[$1>>2]|0;
 $85 = HEAP32[$2>>2]|0;
 $86 = HEAP32[$1>>2]|0;
 $87 = (_sqlite3Strlen30($86)|0);
 $88 = (_sqlite3Atoi64($84,$85,$87,1)|0);
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3BtreeGetFilename($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3PagerFilename($4,1)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3BtreeIsReadonly($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 22|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _mallocWithAlarm($n,$pp) {
 $n = $n|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFull = 0, $nUsed = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $nFull = sp + 12|0;
 $p = sp + 8|0;
 $nUsed = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $pp;
 $2 = HEAP32[(60)>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (FUNCTION_TABLE_ii[$2 & 255]($3)|0);
 HEAP32[$nFull>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3StatusHighwater(5,$5);
 $6 = (288);
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($6) + 4)|0;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 $13 = ($8>>>0)>(0);
 $14 = ($11|0)==(0);
 $15 = $14 & $13;
 $16 = $12 | $15;
 do {
  if ($16) {
   $17 = (_sqlite3StatusValue(0)|0);
   $18 = tempRet0;
   $19 = $nUsed;
   $20 = $19;
   HEAP32[$20>>2] = $17;
   $21 = (($19) + 4)|0;
   $22 = $21;
   HEAP32[$22>>2] = $18;
   $23 = $nUsed;
   $24 = $23;
   $25 = HEAP32[$24>>2]|0;
   $26 = (($23) + 4)|0;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (288);
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($29) + 4)|0;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$nFull>>2]|0;
   $36 = ($35|0)<(0);
   $37 = $36 << 31 >> 31;
   $38 = (_i64Subtract(($31|0),($34|0),($35|0),($37|0))|0);
   $39 = tempRet0;
   $40 = ($28|0)>($39|0);
   $41 = ($25>>>0)>=($38>>>0);
   $42 = ($28|0)==($39|0);
   $43 = $42 & $41;
   $44 = $40 | $43;
   if ($44) {
    HEAP32[(308)>>2] = 1;
    $45 = HEAP32[$nFull>>2]|0;
    _sqlite3MallocAlarm($45);
    break;
   } else {
    HEAP32[(308)>>2] = 0;
    break;
   }
  }
 } while(0);
 $46 = HEAP32[(44)>>2]|0;
 $47 = HEAP32[$nFull>>2]|0;
 $48 = (FUNCTION_TABLE_ii[$46 & 255]($47)|0);
 HEAP32[$p>>2] = $48;
 $49 = HEAP32[$p>>2]|0;
 $50 = ($49|0)!=(0|0);
 if (!($50)) {
  $54 = HEAP32[$p>>2]|0;
  $55 = HEAP32[$1>>2]|0;
  HEAP32[$55>>2] = $54;
  $56 = HEAP32[$nFull>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $51 = HEAP32[$p>>2]|0;
 $52 = (_sqlite3MallocSize($51)|0);
 HEAP32[$nFull>>2] = $52;
 $53 = HEAP32[$nFull>>2]|0;
 _sqlite3StatusUp(0,$53);
 _sqlite3StatusUp(9,1);
 $54 = HEAP32[$p>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 HEAP32[$55>>2] = $54;
 $56 = HEAP32[$nFull>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _sqlite3StatusHighwater($op,$X) {
 $op = $op|0;
 $X = $X|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $newValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $newValue = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $X;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$newValue>>2] = $2;
 $3 = HEAP32[$newValue>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((424) + ($4<<2)|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3>>>0)>($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$newValue>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((424) + ($9<<2)|0);
 HEAP32[$10>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3MallocAlarm($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $nByte;
 $1 = (288);
 $2 = $1;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($1) + 4)|0;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)<(0);
 $8 = ($3>>>0)<=(0);
 $9 = ($6|0)==(0);
 $10 = $9 & $8;
 $11 = $7 | $10;
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 (_sqlite3_release_memory($12)|0);
 STACKTOP = sp;return;
}
function _sqlite3StatusUp($op,$N) {
 $op = $op|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (384 + ($3<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2))|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (384 + ($7<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((424) + ($10<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9>>>0)>($12>>>0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (384 + ($14<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((424) + ($17<<2)|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _measureAllocationSize($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3DbMallocSize($2,$3)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 528|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($4))|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _isLookaside($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 248|0);
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($2>>>0)>=($6>>>0);
 if (!($7)) {
  $15 = 0;
  $14 = $15&1;
  STACKTOP = sp;return ($14|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 248|0);
 $11 = ((($10)) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($8>>>0)<($12>>>0);
 $15 = $13;
 $14 = $15&1;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3DbMallocSize($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (_isLookaside($5,$6)|0);
  $8 = ($7|0)!=(0);
  if ($8) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 248|0);
   $14 = ((($13)) + 4|0);
   $15 = HEAP16[$14>>1]|0;
   $16 = $15&65535;
   HEAP32[$0>>2] = $16;
   $17 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($17|0);
  }
 }
 $9 = HEAP32[(56)>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (FUNCTION_TABLE_ii[$9 & 255]($10)|0);
 HEAP32[$0>>2] = $11;
 $17 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3StrAccumAppend($p,$z,$N) {
 $p = $p|0;
 $z = $z|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($5) + ($6))|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7>>>0)>=($10>>>0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _enlargeAndAppend($12,$13,$14);
  STACKTOP = sp;return;
 } else {
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15))|0;
  HEAP32[$17>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + ($24)|0);
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  _memcpy(($28|0),($29|0),($30|0))|0;
  STACKTOP = sp;return;
 }
}
function _getIntArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  $8 = $0;
  $9 = $8;
  HEAP32[$9>>2] = 0;
  $10 = (($8) + 4)|0;
  $11 = $10;
  HEAP32[$11>>2] = 0;
  $27 = $0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  tempRet0 = ($32);
  STACKTOP = sp;return ($29|0);
 } else {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$13>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($14<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3_value_int64($20)|0);
  $22 = tempRet0;
  $23 = $0;
  $24 = $23;
  HEAP32[$24>>2] = $21;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  tempRet0 = ($32);
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _setStrAccumError($p,$eError) {
 $p = $p|0;
 $eError = $eError|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $eError;
 $2 = HEAP8[$1>>0]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 24|0);
 HEAP8[$4>>0] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _getDoubleArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  HEAPF64[$0>>3] = 0.0;
  $18 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$18);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($10<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (+_sqlite3_value_double($16));
  HEAPF64[$0>>3] = $17;
  $18 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$18);
 }
 return +(0.0);
}
function _sqlite3IsNaN($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0, $rc = 0, $y = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $rc = sp + 24|0;
 $y = sp + 8|0;
 $z = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 HEAPF64[$y>>3] = $1;
 $2 = +HEAPF64[$y>>3];
 HEAPF64[$z>>3] = $2;
 $3 = +HEAPF64[$y>>3];
 $4 = +HEAPF64[$z>>3];
 $5 = $3 != $4;
 $6 = $5&1;
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _et_getdigit($val,$cnt) {
 $val = $val|0;
 $cnt = $cnt|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, $digit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $digit = sp + 8|0;
 $d = sp;
 HEAP32[$1>>2] = $val;
 HEAP32[$2>>2] = $cnt;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  HEAP8[$0>>0] = 48;
  $24 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) + -1)|0;
  HEAP32[$6>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = +HEAPF64[$9>>3];
  $11 = (~~(($10)));
  HEAP32[$digit>>2] = $11;
  $12 = HEAP32[$digit>>2]|0;
  $13 = (+($12|0));
  HEAPF64[$d>>3] = $13;
  $14 = HEAP32[$digit>>2]|0;
  $15 = (($14) + 48)|0;
  HEAP32[$digit>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = +HEAPF64[$16>>3];
  $18 = +HEAPF64[$d>>3];
  $19 = $17 - $18;
  $20 = $19 * 10.0;
  $21 = HEAP32[$1>>2]|0;
  HEAPF64[$21>>3] = $20;
  $22 = HEAP32[$digit>>2]|0;
  $23 = $22&255;
  HEAP8[$0>>0] = $23;
  $24 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _getTextArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($10<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3_value_text($16)|0);
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3AppendChar($p,$N,$c) {
 $p = $p|0;
 $N = $N|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 HEAP8[$2>>0] = $c;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_i64Add(($5|0),0,($6|0),($8|0))|0);
 $10 = tempRet0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)>(0);
 $15 = ($9>>>0)>=($13>>>0);
 $16 = ($10|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (_sqlite3StrAccumEnlarge($19,$20)|0);
  HEAP32[$1>>2] = $21;
  $22 = ($21|0)<=(0);
  if ($22) {
   STACKTOP = sp;return;
  }
 }
 while(1) {
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + -1)|0;
  HEAP32[$1>>2] = $24;
  $25 = ($23|0)>(0);
  if (!($25)) {
   break;
  }
  $26 = HEAP8[$2>>0]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$28>>2] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($29)|0);
  HEAP8[$34>>0] = $26;
 }
 STACKTOP = sp;return;
}
function _sqlite3StrAccumAppendAll($p,$z) {
 $p = $p|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 _sqlite3StrAccumAppend($2,$3,$5);
 STACKTOP = sp;return;
}
function _enlargeAndAppend($p,$z,$N) {
 $p = $p|0;
 $z = $z|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3StrAccumEnlarge($3,$4)|0);
 HEAP32[$2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10)|0);
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _memcpy(($14|0),($15|0),($16|0))|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17))|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _sqlite3StrAccumEnlarge($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $szNew = 0, $zNew = 0, $zOld = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zNew = sp + 12|0;
 $zOld = sp + 8|0;
 $szNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = ((($11)) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13) - ($16))|0;
  $18 = (($17) - 1)|0;
  HEAP32[$2>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  _setStrAccumError($19,2);
  $20 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $20;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $21 = ((($11)) + 25|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 & 4;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $28;
 } else {
  $29 = 0;
 }
 HEAP32[$zOld>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 12|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = $szNew;
 $34 = $33;
 HEAP32[$34>>2] = $32;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$2>>2]|0;
 $38 = (($37) + 1)|0;
 $39 = ($38|0)<(0);
 $40 = $39 << 31 >> 31;
 $41 = $szNew;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_i64Add(($43|0),($46|0),($38|0),($40|0))|0);
 $48 = tempRet0;
 $49 = $szNew;
 $50 = $49;
 HEAP32[$50>>2] = $47;
 $51 = (($49) + 4)|0;
 $52 = $51;
 HEAP32[$52>>2] = $48;
 $53 = $szNew;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($53) + 4)|0;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$1>>2]|0;
 $60 = ((($59)) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (_i64Add(($55|0),($58|0),($61|0),0)|0);
 $63 = tempRet0;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 20|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($63|0)<(0);
 $68 = ($62>>>0)<=($66>>>0);
 $69 = ($63|0)==(0);
 $70 = $69 & $68;
 $71 = $67 | $70;
 if ($71) {
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 12|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $szNew;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (_i64Add(($77|0),($80|0),($74|0),0)|0);
  $82 = tempRet0;
  $83 = $szNew;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
 }
 $87 = $szNew;
 $88 = $87;
 $89 = HEAP32[$88>>2]|0;
 $90 = (($87) + 4)|0;
 $91 = $90;
 $92 = HEAP32[$91>>2]|0;
 $93 = HEAP32[$1>>2]|0;
 $94 = ((($93)) + 20|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = ($92|0)>(0);
 $97 = ($89>>>0)>($95>>>0);
 $98 = ($92|0)==(0);
 $99 = $98 & $97;
 $100 = $96 | $99;
 if ($100) {
  $101 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumReset($101);
  $102 = HEAP32[$1>>2]|0;
  _setStrAccumError($102,2);
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $103 = $szNew;
 $104 = $103;
 $105 = HEAP32[$104>>2]|0;
 $106 = (($103) + 4)|0;
 $107 = $106;
 $108 = HEAP32[$107>>2]|0;
 $109 = HEAP32[$1>>2]|0;
 $110 = ((($109)) + 16|0);
 HEAP32[$110>>2] = $105;
 $111 = HEAP32[$1>>2]|0;
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)!=(0|0);
 if ($113) {
  $114 = HEAP32[$1>>2]|0;
  $115 = HEAP32[$114>>2]|0;
  $116 = HEAP32[$zOld>>2]|0;
  $117 = HEAP32[$1>>2]|0;
  $118 = ((($117)) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = (_sqlite3DbRealloc($115,$116,$119,0)|0);
  HEAP32[$zNew>>2] = $120;
 } else {
  $121 = HEAP32[$zOld>>2]|0;
  $122 = HEAP32[$1>>2]|0;
  $123 = ((($122)) + 16|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = (_sqlite3_realloc64($121,$124,0)|0);
  HEAP32[$zNew>>2] = $125;
 }
 $126 = HEAP32[$zNew>>2]|0;
 $127 = ($126|0)!=(0|0);
 $128 = HEAP32[$1>>2]|0;
 if (!($127)) {
  _sqlite3StrAccumReset($128);
  $161 = HEAP32[$1>>2]|0;
  _setStrAccumError($161,1);
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $129 = ((($128)) + 25|0);
 $130 = HEAP8[$129>>0]|0;
 $131 = $130&255;
 $132 = $131 & 4;
 $133 = ($132|0)!=(0);
 if (!($133)) {
  $134 = HEAP32[$1>>2]|0;
  $135 = ((($134)) + 12|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = ($136>>>0)>(0);
  if ($137) {
   $138 = HEAP32[$zNew>>2]|0;
   $139 = HEAP32[$1>>2]|0;
   $140 = ((($139)) + 8|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = HEAP32[$1>>2]|0;
   $143 = ((($142)) + 12|0);
   $144 = HEAP32[$143>>2]|0;
   _memcpy(($138|0),($141|0),($144|0))|0;
  }
 }
 $145 = HEAP32[$zNew>>2]|0;
 $146 = HEAP32[$1>>2]|0;
 $147 = ((($146)) + 8|0);
 HEAP32[$147>>2] = $145;
 $148 = HEAP32[$1>>2]|0;
 $149 = HEAP32[$148>>2]|0;
 $150 = HEAP32[$zNew>>2]|0;
 $151 = (_sqlite3DbMallocSize($149,$150)|0);
 $152 = HEAP32[$1>>2]|0;
 $153 = ((($152)) + 16|0);
 HEAP32[$153>>2] = $151;
 $154 = HEAP32[$1>>2]|0;
 $155 = ((($154)) + 25|0);
 $156 = HEAP8[$155>>0]|0;
 $157 = $156&255;
 $158 = $157 | 4;
 $159 = $158&255;
 HEAP8[$155>>0] = $159;
 $160 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $160;
 $162 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _sqlite3StrAccumReset($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 25|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = $4 & 4;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 8|0);
  HEAP32[$19>>2] = 0;
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DbFree($8,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 25|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = $15 & -5;
 $17 = $16&255;
 HEAP8[$13>>0] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3DbRealloc($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $p;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)==(0|0);
 $12 = HEAP32[$3>>2]|0;
 if ($11) {
  $13 = $5;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3DbMallocRawNN($12,$15,$18)|0);
  HEAP32[$2>>2] = $19;
  $49 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $20 = HEAP32[$4>>2]|0;
 $21 = (_isLookaside($12,$20)|0);
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $5;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = ((($29)) + 248|0);
  $31 = ((($30)) + 4|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32&65535;
  $34 = ($28>>>0)<(0);
  $35 = ($25>>>0)<=($33>>>0);
  $36 = ($28|0)==(0);
  $37 = $36 & $35;
  $38 = $34 | $37;
  if ($38) {
   $39 = HEAP32[$4>>2]|0;
   HEAP32[$2>>2] = $39;
   $49 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 }
 $40 = HEAP32[$3>>2]|0;
 $41 = HEAP32[$4>>2]|0;
 $42 = $5;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (_dbReallocFinish($40,$41,$44,$47)|0);
 HEAP32[$2>>2] = $48;
 $49 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3DbMallocRawNN($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pBuf = sp + 8|0;
 HEAP32[$3>>2] = $db;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 248|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = $4;
   $14 = $13;
   $15 = HEAP32[$14>>2]|0;
   $16 = (($13) + 4)|0;
   $17 = $16;
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$3>>2]|0;
   $20 = ((($19)) + 248|0);
   $21 = ((($20)) + 4|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22&65535;
   $24 = ($18>>>0)>(0);
   $25 = ($15>>>0)>($23>>>0);
   $26 = ($18|0)==(0);
   $27 = $26 & $25;
   $28 = $24 | $27;
   $29 = HEAP32[$3>>2]|0;
   $30 = ((($29)) + 248|0);
   if ($28) {
    $31 = ((($30)) + 16|0);
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 1)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $35 = ((($30)) + 28|0);
   $36 = HEAP32[$35>>2]|0;
   HEAP32[$pBuf>>2] = $36;
   $37 = ($36|0)==(0|0);
   if ($37) {
    $38 = HEAP32[$3>>2]|0;
    $39 = ((($38)) + 248|0);
    $40 = ((($39)) + 16|0);
    $41 = ((($40)) + 8|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (($42) + 1)|0;
    HEAP32[$41>>2] = $43;
    break;
   }
   $44 = HEAP32[$pBuf>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$3>>2]|0;
   $47 = ((($46)) + 248|0);
   $48 = ((($47)) + 28|0);
   HEAP32[$48>>2] = $45;
   $49 = HEAP32[$3>>2]|0;
   $50 = ((($49)) + 248|0);
   $51 = ((($50)) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$51>>2] = $53;
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 248|0);
   $56 = ((($55)) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 1)|0;
   HEAP32[$56>>2] = $58;
   $59 = HEAP32[$3>>2]|0;
   $60 = ((($59)) + 248|0);
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = ((($63)) + 248|0);
   $65 = ((($64)) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = ($62|0)>($66|0);
   if ($67) {
    $68 = HEAP32[$3>>2]|0;
    $69 = ((($68)) + 248|0);
    $70 = ((($69)) + 8|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = HEAP32[$3>>2]|0;
    $73 = ((($72)) + 248|0);
    $74 = ((($73)) + 12|0);
    HEAP32[$74>>2] = $71;
   }
   $75 = HEAP32[$pBuf>>2]|0;
   HEAP32[$2>>2] = $75;
   $88 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($88|0);
  } else {
   $76 = HEAP32[$3>>2]|0;
   $77 = ((($76)) + 65|0);
   $78 = HEAP8[$77>>0]|0;
   $79 = ($78<<24>>24)!=(0);
   if ($79) {
    HEAP32[$2>>2] = 0;
    $88 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($88|0);
   }
  }
 } while(0);
 $80 = HEAP32[$3>>2]|0;
 $81 = $4;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = (_dbMallocRawFinish($80,$83,$86)|0);
 HEAP32[$2>>2] = $87;
 $88 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _dbReallocFinish($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pNew = sp + 8|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$pNew>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 65|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(0);
 if (!($13)) {
  $48 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = (_isLookaside($14,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3DbMallocRawNN($18,$21,$24)|0);
  HEAP32[$pNew>>2] = $25;
  $26 = HEAP32[$pNew>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   $48 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
  $28 = HEAP32[$pNew>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 248|0);
  $32 = ((($31)) + 4|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  _memcpy(($28|0),($29|0),($34|0))|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  _sqlite3DbFree($35,$36);
  $48 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($48|0);
 } else {
  $37 = HEAP32[$3>>2]|0;
  $38 = $4;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($38) + 4)|0;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (_sqlite3_realloc64($37,$40,$43)|0);
  HEAP32[$pNew>>2] = $44;
  $45 = HEAP32[$pNew>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $48 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
  $47 = HEAP32[$2>>2]|0;
  _sqlite3OomFault($47);
  $48 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 return (0)|0;
}
function _dbMallocRawFinish($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $db;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3Malloc($10,$13)|0);
 HEAP32[$p>>2] = $14;
 $15 = HEAP32[$p>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $18 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $17 = HEAP32[$2>>2]|0;
 _sqlite3OomFault($17);
 $18 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _posixIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (988|0);
}
function _unixClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$pFile>>2]|0;
 _verifyDbFile($2);
 $3 = HEAP32[$0>>2]|0;
 (_unixUnlock($3,0)|0);
 _unixEnterMutex();
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$pFile>>2]|0;
  $9 = ((($8)) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$pFile>>2]|0;
   _setPendingFd($14);
  }
 }
 $15 = HEAP32[$pFile>>2]|0;
 _releaseInodeInfo($15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (_closeUnixFile($16)|0);
 HEAP32[$rc>>2] = $17;
 _unixLeaveMutex();
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _unixRead($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $got = 0, $pFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $pFile = sp + 12|0;
 $got = sp + 8|0;
 HEAP32[$3>>2] = $id;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $amt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $11;
 $12 = HEAP32[$pFile>>2]|0;
 $13 = $6;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = (_seekAndRead($12,$15,$18,$19,$20)|0);
 HEAP32[$got>>2] = $21;
 $22 = HEAP32[$got>>2]|0;
 $23 = HEAP32[$5>>2]|0;
 $24 = ($22|0)==($23|0);
 if ($24) {
  HEAP32[$2>>2] = 0;
  $34 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $25 = HEAP32[$got>>2]|0;
 $26 = ($25|0)<(0);
 if ($26) {
  HEAP32[$2>>2] = 266;
  $34 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $27 = HEAP32[$pFile>>2]|0;
  _storeLastErrno($27,0);
  $28 = HEAP32[$got>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP32[$5>>2]|0;
  $32 = HEAP32[$got>>2]|0;
  $33 = (($31) - ($32))|0;
  _memset(($30|0),0,($33|0))|0;
  HEAP32[$2>>2] = 522;
  $34 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _unixWrite($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $wrote = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $pFile = sp + 12|0;
 $wrote = sp + 8|0;
 HEAP32[$3>>2] = $id;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $amt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $11;
 HEAP32[$wrote>>2] = 0;
 while(1) {
  $12 = HEAP32[$pFile>>2]|0;
  $13 = $6;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$4>>2]|0;
  $20 = HEAP32[$5>>2]|0;
  $21 = (_seekAndWrite($12,$15,$18,$19,$20)|0);
  HEAP32[$wrote>>2] = $21;
  $22 = HEAP32[$5>>2]|0;
  $23 = ($21|0)<($22|0);
  $24 = HEAP32[$wrote>>2]|0;
  $25 = ($24|0)>(0);
  $26 = $23 ? $25 : 0;
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$wrote>>2]|0;
  $28 = HEAP32[$5>>2]|0;
  $29 = (($28) - ($27))|0;
  HEAP32[$5>>2] = $29;
  $30 = HEAP32[$wrote>>2]|0;
  $31 = ($30|0)<(0);
  $32 = $31 << 31 >> 31;
  $33 = $6;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($33) + 4)|0;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (_i64Add(($35|0),($38|0),($30|0),($32|0))|0);
  $40 = tempRet0;
  $41 = $6;
  $42 = $41;
  HEAP32[$42>>2] = $39;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $40;
  $45 = HEAP32[$wrote>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  $47 = (($46) + ($45)|0);
  HEAP32[$4>>2] = $47;
 }
 $48 = HEAP32[$5>>2]|0;
 $49 = HEAP32[$wrote>>2]|0;
 $50 = ($48|0)>($49|0);
 if (!($50)) {
  HEAP32[$2>>2] = 0;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $51 = HEAP32[$wrote>>2]|0;
 $52 = ($51|0)<(0);
 if ($52) {
  $53 = HEAP32[$pFile>>2]|0;
  $54 = ((($53)) + 20|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(28);
  if ($56) {
   HEAP32[$2>>2] = 778;
   $58 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($58|0);
  }
 }
 $57 = HEAP32[$pFile>>2]|0;
 _storeLastErrno($57,0);
 HEAP32[$2>>2] = 13;
 $58 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _unixTruncate($id,$0,$1) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $pFile = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $id;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $9;
 $10 = HEAP32[$pFile>>2]|0;
 $11 = ((($10)) + 40|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 if ($13) {
  $14 = $4;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pFile>>2]|0;
  $21 = ((($20)) + 40|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)<(0);
  $24 = $23 << 31 >> 31;
  $25 = (_i64Add(($16|0),($19|0),($22|0),($24|0))|0);
  $26 = tempRet0;
  $27 = (_i64Subtract(($25|0),($26|0),1,0)|0);
  $28 = tempRet0;
  $29 = HEAP32[$pFile>>2]|0;
  $30 = ((($29)) + 40|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)<(0);
  $33 = $32 << 31 >> 31;
  $34 = (___divdi3(($27|0),($28|0),($31|0),($33|0))|0);
  $35 = tempRet0;
  $36 = HEAP32[$pFile>>2]|0;
  $37 = ((($36)) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = (___muldi3(($34|0),($35|0),($38|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $4;
  $44 = $43;
  HEAP32[$44>>2] = $41;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
 }
 $47 = HEAP32[$pFile>>2]|0;
 $48 = ((($47)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $4;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + 4)|0;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (_robust_ftruncate($49,$52,$55)|0);
 HEAP32[$rc>>2] = $56;
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = HEAP32[$pFile>>2]|0;
  $60 = (___errno_location()|0);
  $61 = HEAP32[$60>>2]|0;
  _storeLastErrno($59,$61);
  $62 = HEAP32[$pFile>>2]|0;
  $63 = ((($62)) + 32|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (_unixLogErrorAtLine(1546,18358,$64,30893)|0);
  HEAP32[$2>>2] = $65;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 } else {
  HEAP32[$2>>2] = 0;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return (0)|0;
}
function _unixSync($id,$flags) {
 $id = $id|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dirfd = 0, $isDataOnly = 0, $isFullsync = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $pFile = sp + 12|0;
 $isDataOnly = sp + 8|0;
 $isFullsync = sp + 4|0;
 $dirfd = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = $4 & 16;
 HEAP32[$isDataOnly>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = $6 & 15;
 $8 = ($7|0)==(3);
 $9 = $8&1;
 HEAP32[$isFullsync>>2] = $9;
 $10 = HEAP32[$pFile>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$isFullsync>>2]|0;
 $14 = HEAP32[$isDataOnly>>2]|0;
 $15 = (_full_fsync($12,$13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = HEAP32[$pFile>>2]|0;
 if ($17) {
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  _storeLastErrno($18,$20);
  $21 = HEAP32[$pFile>>2]|0;
  $22 = ((($21)) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (_unixLogErrorAtLine(1034,18649,$23,30848)|0);
  HEAP32[$0>>2] = $24;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $25 = ((($18)) + 18|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 8;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[(1272)>>2]|0;
  $31 = HEAP32[$pFile>>2]|0;
  $32 = ((($31)) + 32|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (FUNCTION_TABLE_iii[$30 & 255]($33,$dirfd)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$dirfd>>2]|0;
   (_full_fsync($37,0,0)|0);
   $38 = HEAP32[$pFile>>2]|0;
   $39 = HEAP32[$dirfd>>2]|0;
   _robust_close($38,$39,30862);
  } else {
   HEAP32[$rc>>2] = 0;
  }
  $40 = HEAP32[$pFile>>2]|0;
  $41 = ((($40)) + 18|0);
  $42 = HEAP16[$41>>1]|0;
  $43 = $42&65535;
  $44 = $43 & -9;
  $45 = $44&65535;
  HEAP16[$41>>1] = $45;
 }
 $46 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $46;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _unixFileSize($id,$pSize) {
 $id = $id|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $rc = sp + 76|0;
 $buf = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $pSize;
 $3 = HEAP32[(1128)>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$3 & 255]($6,$buf)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (___errno_location()|0);
  $12 = HEAP32[$11>>2]|0;
  _storeLastErrno($10,$12);
  HEAP32[$0>>2] = 1802;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $13 = ((($buf)) + 36|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)<(0);
 $16 = $15 << 31 >> 31;
 $17 = HEAP32[$2>>2]|0;
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $14;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $16;
 $22 = HEAP32[$2>>2]|0;
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = ($25|0)==(1);
 $30 = ($28|0)==(0);
 $31 = $29 & $30;
 if ($31) {
  $32 = HEAP32[$2>>2]|0;
  $33 = $32;
  $34 = $33;
  HEAP32[$34>>2] = 0;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = 0;
 }
 HEAP32[$0>>2] = 0;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _unixLock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $lock = 0;
 var $or$cond = 0, $or$cond3 = 0, $pFile = 0, $pInode = 0, $rc = 0, $tErrno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $pFile = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 HEAP32[$tErrno>>2] = 0;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 _unixEnterMutex();
 $10 = HEAP32[$pFile>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pInode>>2] = $12;
 $13 = HEAP32[$pFile>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = HEAP32[$pInode>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($16|0)!=($20|0);
 if ($21) {
  $22 = HEAP32[$pInode>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)>=(3);
  $27 = HEAP32[$2>>2]|0;
  $28 = ($27|0)>(1);
  $or$cond = $26 | $28;
  if ($or$cond) {
   HEAP32[$rc>>2] = 5;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 L8: do {
  if ((label|0) == 6) {
   $29 = HEAP32[$2>>2]|0;
   $30 = ($29|0)==(1);
   do {
    if ($30) {
     $31 = HEAP32[$pInode>>2]|0;
     $32 = ((($31)) + 12|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33&255;
     $35 = ($34|0)==(1);
     if (!($35)) {
      $36 = HEAP32[$pInode>>2]|0;
      $37 = ((($36)) + 12|0);
      $38 = HEAP8[$37>>0]|0;
      $39 = $38&255;
      $40 = ($39|0)==(2);
      if (!($40)) {
       break;
      }
     }
     $41 = HEAP32[$pFile>>2]|0;
     $42 = ((($41)) + 16|0);
     HEAP8[$42>>0] = 1;
     $43 = HEAP32[$pInode>>2]|0;
     $44 = ((($43)) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (($45) + 1)|0;
     HEAP32[$44>>2] = $46;
     $47 = HEAP32[$pInode>>2]|0;
     $48 = ((($47)) + 24|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = (($49) + 1)|0;
     HEAP32[$48>>2] = $50;
     break L8;
    }
   } while(0);
   $51 = ((($lock)) + 8|0);
   HEAP32[$51>>2] = 1;
   $52 = ((($lock)) + 2|0);
   HEAP16[$52>>1] = 0;
   $53 = HEAP32[$2>>2]|0;
   $54 = ($53|0)==(1);
   if ($54) {
    label = 13;
   } else {
    $55 = HEAP32[$2>>2]|0;
    $56 = ($55|0)==(4);
    if ($56) {
     $57 = HEAP32[$pFile>>2]|0;
     $58 = ((($57)) + 16|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59&255;
     $61 = ($60|0)<(3);
     if ($61) {
      label = 13;
     }
    }
   }
   if ((label|0) == 13) {
    $62 = HEAP32[$2>>2]|0;
    $63 = ($62|0)==(1);
    $64 = $63 ? 0 : 1;
    $65 = $64&65535;
    HEAP16[$lock>>1] = $65;
    $66 = HEAP32[832>>2]|0;
    $67 = ((($lock)) + 4|0);
    HEAP32[$67>>2] = $66;
    $68 = HEAP32[$pFile>>2]|0;
    $69 = (_unixFileLock($68,$lock)|0);
    $70 = ($69|0)!=(0);
    if ($70) {
     $71 = (___errno_location()|0);
     $72 = HEAP32[$71>>2]|0;
     HEAP32[$tErrno>>2] = $72;
     $73 = HEAP32[$tErrno>>2]|0;
     $74 = (_sqliteErrorFromPosixError($73,3850)|0);
     HEAP32[$rc>>2] = $74;
     $75 = HEAP32[$rc>>2]|0;
     $76 = ($75|0)!=(5);
     if (!($76)) {
      break;
     }
     $77 = HEAP32[$pFile>>2]|0;
     $78 = HEAP32[$tErrno>>2]|0;
     _storeLastErrno($77,$78);
     break;
    }
   }
   $79 = HEAP32[$2>>2]|0;
   $80 = ($79|0)==(1);
   do {
    if ($80) {
     $81 = HEAP32[832>>2]|0;
     $82 = (($81) + 2)|0;
     $83 = ((($lock)) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = ((($lock)) + 8|0);
     HEAP32[$84>>2] = 510;
     $85 = HEAP32[$pFile>>2]|0;
     $86 = (_unixFileLock($85,$lock)|0);
     $87 = ($86|0)!=(0);
     if ($87) {
      $88 = (___errno_location()|0);
      $89 = HEAP32[$88>>2]|0;
      HEAP32[$tErrno>>2] = $89;
      $90 = HEAP32[$tErrno>>2]|0;
      $91 = (_sqliteErrorFromPosixError($90,3850)|0);
      HEAP32[$rc>>2] = $91;
     }
     $92 = HEAP32[832>>2]|0;
     $93 = ((($lock)) + 4|0);
     HEAP32[$93>>2] = $92;
     $94 = ((($lock)) + 8|0);
     HEAP32[$94>>2] = 1;
     HEAP16[$lock>>1] = 2;
     $95 = HEAP32[$pFile>>2]|0;
     $96 = (_unixFileLock($95,$lock)|0);
     $97 = ($96|0)!=(0);
     $98 = HEAP32[$rc>>2]|0;
     $99 = ($98|0)==(0);
     $or$cond3 = $97 & $99;
     if ($or$cond3) {
      $100 = (___errno_location()|0);
      $101 = HEAP32[$100>>2]|0;
      HEAP32[$tErrno>>2] = $101;
      HEAP32[$rc>>2] = 2058;
     }
     $102 = HEAP32[$rc>>2]|0;
     $103 = ($102|0)!=(0);
     if (!($103)) {
      $108 = HEAP32[$pFile>>2]|0;
      $109 = ((($108)) + 16|0);
      HEAP8[$109>>0] = 1;
      $110 = HEAP32[$pInode>>2]|0;
      $111 = ((($110)) + 24|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = (($112) + 1)|0;
      HEAP32[$111>>2] = $113;
      $114 = HEAP32[$pInode>>2]|0;
      $115 = ((($114)) + 8|0);
      HEAP32[$115>>2] = 1;
      break;
     }
     $104 = HEAP32[$rc>>2]|0;
     $105 = ($104|0)!=(5);
     if (!($105)) {
      break L8;
     }
     $106 = HEAP32[$pFile>>2]|0;
     $107 = HEAP32[$tErrno>>2]|0;
     _storeLastErrno($106,$107);
     break L8;
    } else {
     $116 = HEAP32[$2>>2]|0;
     $117 = ($116|0)==(4);
     if ($117) {
      $118 = HEAP32[$pInode>>2]|0;
      $119 = ((($118)) + 8|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = ($120|0)>(1);
      if ($121) {
       HEAP32[$rc>>2] = 5;
       break;
      }
     }
     HEAP16[$lock>>1] = 1;
     $122 = HEAP32[$2>>2]|0;
     $123 = ($122|0)==(2);
     $124 = HEAP32[832>>2]|0;
     if ($123) {
      $125 = (($124) + 1)|0;
      $126 = ((($lock)) + 4|0);
      HEAP32[$126>>2] = $125;
      $127 = ((($lock)) + 8|0);
      HEAP32[$127>>2] = 1;
     } else {
      $128 = (($124) + 2)|0;
      $129 = ((($lock)) + 4|0);
      HEAP32[$129>>2] = $128;
      $130 = ((($lock)) + 8|0);
      HEAP32[$130>>2] = 510;
     }
     $131 = HEAP32[$pFile>>2]|0;
     $132 = (_unixFileLock($131,$lock)|0);
     $133 = ($132|0)!=(0);
     if ($133) {
      $134 = (___errno_location()|0);
      $135 = HEAP32[$134>>2]|0;
      HEAP32[$tErrno>>2] = $135;
      $136 = HEAP32[$tErrno>>2]|0;
      $137 = (_sqliteErrorFromPosixError($136,3850)|0);
      HEAP32[$rc>>2] = $137;
      $138 = HEAP32[$rc>>2]|0;
      $139 = ($138|0)!=(5);
      if ($139) {
       $140 = HEAP32[$pFile>>2]|0;
       $141 = HEAP32[$tErrno>>2]|0;
       _storeLastErrno($140,$141);
      }
     }
    }
   } while(0);
   $142 = HEAP32[$rc>>2]|0;
   $143 = ($142|0)==(0);
   $144 = HEAP32[$2>>2]|0;
   if ($143) {
    $145 = $144&255;
    $146 = HEAP32[$pFile>>2]|0;
    $147 = ((($146)) + 16|0);
    HEAP8[$147>>0] = $145;
    $148 = HEAP32[$2>>2]|0;
    $149 = $148&255;
    $150 = HEAP32[$pInode>>2]|0;
    $151 = ((($150)) + 12|0);
    HEAP8[$151>>0] = $149;
    break;
   }
   $152 = ($144|0)==(4);
   if ($152) {
    $153 = HEAP32[$pFile>>2]|0;
    $154 = ((($153)) + 16|0);
    HEAP8[$154>>0] = 3;
    $155 = HEAP32[$pInode>>2]|0;
    $156 = ((($155)) + 12|0);
    HEAP8[$156>>0] = 3;
   }
  }
 } while(0);
 _unixLeaveMutex();
 $157 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $157;
 $158 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($158|0);
}
function _unixUnlock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $eFileLock;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_posixUnlock($2,$3,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _unixCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lock = 0, $pFile = 0;
 var $rc = 0, $reserved = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $rc = sp + 32|0;
 $reserved = sp + 28|0;
 $pFile = sp + 24|0;
 $lock = sp + 8|0;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$reserved>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $2;
 _unixEnterMutex();
 $3 = HEAP32[$pFile>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)>(1);
 if ($9) {
  HEAP32[$reserved>>2] = 1;
 }
 $10 = HEAP32[$reserved>>2]|0;
 $11 = ($10|0)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$pFile>>2]|0;
   $13 = ((($12)) + 8|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ((($14)) + 13|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if (!($17)) {
    $18 = ((($lock)) + 2|0);
    HEAP16[$18>>1] = 0;
    $19 = HEAP32[832>>2]|0;
    $20 = (($19) + 1)|0;
    $21 = ((($lock)) + 4|0);
    HEAP32[$21>>2] = $20;
    $22 = ((($lock)) + 8|0);
    HEAP32[$22>>2] = 1;
    HEAP16[$lock>>1] = 1;
    $23 = HEAP32[(1152)>>2]|0;
    $24 = HEAP32[$pFile>>2]|0;
    $25 = ((($24)) + 12|0);
    $26 = HEAP32[$25>>2]|0;
    HEAP32[$vararg_buffer>>2] = $lock;
    $27 = (FUNCTION_TABLE_iiii[$23 & 255]($26,12,$vararg_buffer)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$rc>>2] = 3594;
     $29 = HEAP32[$pFile>>2]|0;
     $30 = (___errno_location()|0);
     $31 = HEAP32[$30>>2]|0;
     _storeLastErrno($29,$31);
     break;
    }
    $32 = HEAP16[$lock>>1]|0;
    $33 = $32 << 16 >> 16;
    $34 = ($33|0)!=(2);
    if ($34) {
     HEAP32[$reserved>>2] = 1;
    }
   }
  }
 } while(0);
 _unixLeaveMutex();
 $35 = HEAP32[$reserved>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _unixFileControl($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pFile = 0, $rc = 0, $vararg_buffer = 0, $zTFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pFile = sp + 12|0;
 $rc = sp + 8|0;
 $zTFile = sp + 4|0;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pArg;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 do {
  switch ($5|0) {
  case 1:  {
   $6 = HEAP32[$pFile>>2]|0;
   $7 = ((($6)) + 16|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = HEAP32[$3>>2]|0;
   HEAP32[$10>>2] = $9;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 4:  {
   $11 = HEAP32[$pFile>>2]|0;
   $12 = ((($11)) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   HEAP32[$14>>2] = $13;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 6:  {
   $15 = HEAP32[$3>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pFile>>2]|0;
   $18 = ((($17)) + 40|0);
   HEAP32[$18>>2] = $16;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 5:  {
   $19 = HEAP32[$pFile>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = $20;
   $22 = $21;
   $23 = HEAP32[$22>>2]|0;
   $24 = (($21) + 4)|0;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (_fcntlSizeHint($19,$23,$26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $28;
   break;
  }
  case 10:  {
   $29 = HEAP32[$pFile>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   _unixModeBit($29,4,$30);
   HEAP32[$0>>2] = 0;
   break;
  }
  case 13:  {
   $31 = HEAP32[$pFile>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   _unixModeBit($31,16,$32);
   HEAP32[$0>>2] = 0;
   break;
  }
  case 12:  {
   $33 = HEAP32[$pFile>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ((($35)) + 16|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$vararg_buffer>>2] = $37;
   $38 = (_sqlite3_mprintf(17521,$vararg_buffer)|0);
   $39 = HEAP32[$3>>2]|0;
   HEAP32[$39>>2] = $38;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 16:  {
   $40 = HEAP32[$pFile>>2]|0;
   $41 = ((($40)) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ((($42)) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)<(0);
   $46 = $45 << 31 >> 31;
   $47 = (_sqlite3_malloc64($44,$46)|0);
   HEAP32[$zTFile>>2] = $47;
   $48 = HEAP32[$zTFile>>2]|0;
   $49 = ($48|0)!=(0|0);
   if ($49) {
    $50 = HEAP32[$pFile>>2]|0;
    $51 = ((($50)) + 4|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = ((($52)) + 8|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = HEAP32[$zTFile>>2]|0;
    (_unixGetTempname($54,$55)|0);
    $56 = HEAP32[$zTFile>>2]|0;
    $57 = HEAP32[$3>>2]|0;
    HEAP32[$57>>2] = $56;
   }
   HEAP32[$0>>2] = 0;
   break;
  }
  case 20:  {
   $58 = HEAP32[$pFile>>2]|0;
   $59 = (_fileHasMoved($58)|0);
   $60 = HEAP32[$3>>2]|0;
   HEAP32[$60>>2] = $59;
   HEAP32[$0>>2] = 0;
   break;
  }
  default: {
   HEAP32[$0>>2] = 12;
  }
  }
 } while(0);
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _unixSectorSize($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return 4096;
}
function _unixDeviceCharacteristics($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 18|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $8 = HEAP32[$rc>>2]|0;
 $9 = $8 | 4096;
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _unixShmMap($fd,$iRegion,$szRegion,$bExtend,$pp) {
 $fd = $fd|0;
 $iRegion = $iRegion|0;
 $szRegion = $szRegion|0;
 $bExtend = $bExtend|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apNew = 0, $i = 0, $iPg = 0, $nByte = 0, $nMap = 0, $nReqRegion = 0, $nShmPerMap = 0, $or$cond = 0, $p = 0;
 var $pDbFd = 0, $pMem = 0, $pShmNode = 0, $rc = 0, $sStat = 0, $x = 0, $zFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 152|0;
 $1 = sp + 148|0;
 $2 = sp + 144|0;
 $3 = sp + 140|0;
 $4 = sp + 136|0;
 $5 = sp + 132|0;
 $pDbFd = sp + 128|0;
 $p = sp + 124|0;
 $pShmNode = sp + 120|0;
 $rc = sp + 116|0;
 $nShmPerMap = sp + 112|0;
 $nReqRegion = sp + 108|0;
 $apNew = sp + 104|0;
 $nByte = sp + 100|0;
 $sStat = sp + 24|0;
 $iPg = sp + 20|0;
 $x = sp + 16|0;
 $zFile = sp + 12|0;
 $nMap = sp + 8|0;
 $i = sp + 4|0;
 $pMem = sp;
 HEAP32[$1>>2] = $fd;
 HEAP32[$2>>2] = $iRegion;
 HEAP32[$3>>2] = $szRegion;
 HEAP32[$4>>2] = $bExtend;
 HEAP32[$5>>2] = $pp;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pDbFd>>2] = $6;
 HEAP32[$rc>>2] = 0;
 $7 = (_unixShmRegionPerMap()|0);
 HEAP32[$nShmPerMap>>2] = $7;
 $8 = HEAP32[$pDbFd>>2]|0;
 $9 = ((($8)) + 36|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[$pDbFd>>2]|0;
  $13 = (_unixOpenSharedMemory($12)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $16;
   $190 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($190|0);
  }
 }
 $17 = HEAP32[$pDbFd>>2]|0;
 $18 = ((($17)) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$p>>2] = $19;
 $20 = HEAP32[$p>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pShmNode>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$nShmPerMap>>2]|0;
 $24 = (($22) + ($23))|0;
 $25 = HEAP32[$nShmPerMap>>2]|0;
 $26 = (($24|0) / ($25|0))&-1;
 $27 = HEAP32[$nShmPerMap>>2]|0;
 $28 = Math_imul($26, $27)|0;
 HEAP32[$nReqRegion>>2] = $28;
 $29 = HEAP32[$pShmNode>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31&65535;
 $33 = HEAP32[$nReqRegion>>2]|0;
 $34 = ($32|0)<($33|0);
 L6: do {
  if ($34) {
   $35 = HEAP32[$nReqRegion>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = Math_imul($35, $36)|0;
   HEAP32[$nByte>>2] = $37;
   $38 = HEAP32[$3>>2]|0;
   $39 = HEAP32[$pShmNode>>2]|0;
   $40 = ((($39)) + 16|0);
   HEAP32[$40>>2] = $38;
   $41 = HEAP32[$pShmNode>>2]|0;
   $42 = ((($41)) + 12|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)>=(0);
   L8: do {
    if ($44) {
     $45 = HEAP32[(1128)>>2]|0;
     $46 = HEAP32[$pShmNode>>2]|0;
     $47 = ((($46)) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (FUNCTION_TABLE_iii[$45 & 255]($48,$sStat)|0);
     $50 = ($49|0)!=(0);
     if ($50) {
      HEAP32[$rc>>2] = 4874;
      break L6;
     }
     $51 = ((($sStat)) + 36|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = HEAP32[$nByte>>2]|0;
     $54 = ($52|0)<($53|0);
     if ($54) {
      $55 = HEAP32[$4>>2]|0;
      $56 = ($55|0)!=(0);
      if (!($56)) {
       break L6;
      }
      $57 = ((($sStat)) + 36|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = (($58|0) / 4096)&-1;
      HEAP32[$iPg>>2] = $59;
      while(1) {
       $60 = HEAP32[$iPg>>2]|0;
       $61 = HEAP32[$nByte>>2]|0;
       $62 = (($61|0) / 4096)&-1;
       $63 = ($60|0)<($62|0);
       if (!($63)) {
        break L8;
       }
       HEAP32[$x>>2] = 0;
       $64 = HEAP32[$pShmNode>>2]|0;
       $65 = ((($64)) + 12|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = HEAP32[$iPg>>2]|0;
       $68 = $67<<12;
       $69 = (($68) + 4096)|0;
       $70 = (($69) - 1)|0;
       $71 = ($70|0)<(0);
       $72 = $71 << 31 >> 31;
       $73 = (_seekAndWriteFd($66,$70,$72,17532,1,$x)|0);
       $74 = ($73|0)!=(1);
       if ($74) {
        break;
       }
       $80 = HEAP32[$iPg>>2]|0;
       $81 = (($80) + 1)|0;
       HEAP32[$iPg>>2] = $81;
      }
      $75 = HEAP32[$pShmNode>>2]|0;
      $76 = ((($75)) + 8|0);
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$zFile>>2] = $77;
      $78 = HEAP32[$zFile>>2]|0;
      $79 = (_unixLogErrorAtLine(4874,18393,$78,31732)|0);
      HEAP32[$rc>>2] = $79;
      break L6;
     }
    }
   } while(0);
   $82 = HEAP32[$pShmNode>>2]|0;
   $83 = ((($82)) + 24|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$nReqRegion>>2]|0;
   $86 = $85<<2;
   $87 = (_sqlite3_realloc($84,$86)|0);
   HEAP32[$apNew>>2] = $87;
   $88 = HEAP32[$apNew>>2]|0;
   $89 = ($88|0)!=(0|0);
   if (!($89)) {
    HEAP32[$rc>>2] = 3082;
    break;
   }
   $90 = HEAP32[$apNew>>2]|0;
   $91 = HEAP32[$pShmNode>>2]|0;
   $92 = ((($91)) + 24|0);
   HEAP32[$92>>2] = $90;
   while(1) {
    $93 = HEAP32[$pShmNode>>2]|0;
    $94 = ((($93)) + 20|0);
    $95 = HEAP16[$94>>1]|0;
    $96 = $95&65535;
    $97 = HEAP32[$nReqRegion>>2]|0;
    $98 = ($96|0)<($97|0);
    if (!($98)) {
     break L6;
    }
    $99 = HEAP32[$3>>2]|0;
    $100 = HEAP32[$nShmPerMap>>2]|0;
    $101 = Math_imul($99, $100)|0;
    HEAP32[$nMap>>2] = $101;
    $102 = HEAP32[$pShmNode>>2]|0;
    $103 = ((($102)) + 12|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = ($104|0)>=(0);
    if ($105) {
     $106 = HEAP32[(1332)>>2]|0;
     $107 = HEAP32[$nMap>>2]|0;
     $108 = HEAP32[$pShmNode>>2]|0;
     $109 = ((($108)) + 22|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     $112 = ($111|0)!=(0);
     $113 = $112 ? 1 : 3;
     $114 = HEAP32[$pShmNode>>2]|0;
     $115 = ((($114)) + 12|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$3>>2]|0;
     $118 = ($117|0)<(0);
     $119 = $118 << 31 >> 31;
     $120 = HEAP32[$pShmNode>>2]|0;
     $121 = ((($120)) + 20|0);
     $122 = HEAP16[$121>>1]|0;
     $123 = $122&65535;
     $124 = (___muldi3(($117|0),($119|0),($123|0),0)|0);
     $125 = tempRet0;
     $126 = (FUNCTION_TABLE_iiiiiii[$106 & 255](0,$107,$113,1,$116,$124)|0);
     HEAP32[$pMem>>2] = $126;
     $127 = HEAP32[$pMem>>2]|0;
     $128 = ($127|0)==((-1)|0);
     if ($128) {
      label = 21;
      break;
     }
    } else {
     $133 = HEAP32[$3>>2]|0;
     $134 = ($133|0)<(0);
     $135 = $134 << 31 >> 31;
     $136 = (_sqlite3_malloc64($133,$135)|0);
     HEAP32[$pMem>>2] = $136;
     $137 = HEAP32[$pMem>>2]|0;
     $138 = ($137|0)==(0|0);
     if ($138) {
      label = 23;
      break;
     }
     $139 = HEAP32[$pMem>>2]|0;
     $140 = HEAP32[$3>>2]|0;
     _memset(($139|0),0,($140|0))|0;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $141 = HEAP32[$i>>2]|0;
     $142 = HEAP32[$nShmPerMap>>2]|0;
     $143 = ($141|0)<($142|0);
     if (!($143)) {
      break;
     }
     $144 = HEAP32[$3>>2]|0;
     $145 = HEAP32[$i>>2]|0;
     $146 = Math_imul($144, $145)|0;
     $147 = HEAP32[$pMem>>2]|0;
     $148 = (($147) + ($146)|0);
     $149 = HEAP32[$pShmNode>>2]|0;
     $150 = ((($149)) + 20|0);
     $151 = HEAP16[$150>>1]|0;
     $152 = $151&65535;
     $153 = HEAP32[$i>>2]|0;
     $154 = (($152) + ($153))|0;
     $155 = HEAP32[$pShmNode>>2]|0;
     $156 = ((($155)) + 24|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = (($157) + ($154<<2)|0);
     HEAP32[$158>>2] = $148;
     $159 = HEAP32[$i>>2]|0;
     $160 = (($159) + 1)|0;
     HEAP32[$i>>2] = $160;
    }
    $161 = HEAP32[$nShmPerMap>>2]|0;
    $162 = HEAP32[$pShmNode>>2]|0;
    $163 = ((($162)) + 20|0);
    $164 = HEAP16[$163>>1]|0;
    $165 = $164&65535;
    $166 = (($165) + ($161))|0;
    $167 = $166&65535;
    HEAP16[$163>>1] = $167;
   }
   if ((label|0) == 21) {
    $129 = HEAP32[$pShmNode>>2]|0;
    $130 = ((($129)) + 8|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = (_unixLogErrorAtLine(5386,18480,$131,31759)|0);
    HEAP32[$rc>>2] = $132;
    break;
   }
   else if ((label|0) == 23) {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $168 = HEAP32[$pShmNode>>2]|0;
 $169 = ((($168)) + 20|0);
 $170 = HEAP16[$169>>1]|0;
 $171 = $170&65535;
 $172 = HEAP32[$2>>2]|0;
 $173 = ($171|0)>($172|0);
 if ($173) {
  $174 = HEAP32[$2>>2]|0;
  $175 = HEAP32[$pShmNode>>2]|0;
  $176 = ((($175)) + 24|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = (($177) + ($174<<2)|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = HEAP32[$5>>2]|0;
  HEAP32[$180>>2] = $179;
 } else {
  $181 = HEAP32[$5>>2]|0;
  HEAP32[$181>>2] = 0;
 }
 $182 = HEAP32[$pShmNode>>2]|0;
 $183 = ((($182)) + 22|0);
 $184 = HEAP8[$183>>0]|0;
 $185 = $184&255;
 $186 = ($185|0)!=(0);
 $187 = HEAP32[$rc>>2]|0;
 $188 = ($187|0)==(0);
 $or$cond = $186 & $188;
 if ($or$cond) {
  HEAP32[$rc>>2] = 8;
 }
 $189 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $189;
 $190 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($190|0);
}
function _unixShmLock($fd,$ofst,$n,$flags) {
 $fd = $fd|0;
 $ofst = $ofst|0;
 $n = $n|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $allMask = 0, $allShared = 0, $mask = 0, $p = 0, $pDbFd = 0, $pShmNode = 0, $pX = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pDbFd = sp + 16|0;
 $p = sp + 12|0;
 $pX = sp + 8|0;
 $pShmNode = sp + 4|0;
 $rc = sp;
 $mask = sp + 40|0;
 $allMask = sp + 38|0;
 $allShared = sp + 36|0;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $ofst;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$pDbFd>>2] = $4;
 $5 = HEAP32[$pDbFd>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pShmNode>>2] = $9;
 HEAP32[$rc>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = 1 << $12;
 $14 = HEAP32[$1>>2]|0;
 $15 = 1 << $14;
 $16 = (($13) - ($15))|0;
 $17 = $16&65535;
 HEAP16[$mask>>1] = $17;
 $18 = HEAP32[$3>>2]|0;
 $19 = $18 & 1;
 $20 = ($19|0)!=(0);
 if ($20) {
  HEAP16[$allMask>>1] = 0;
  $21 = HEAP32[$pShmNode>>2]|0;
  $22 = ((($21)) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pX>>2] = $23;
  while(1) {
   $24 = HEAP32[$pX>>2]|0;
   $25 = ($24|0)!=(0|0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$pX>>2]|0;
   $27 = HEAP32[$p>>2]|0;
   $28 = ($26|0)==($27|0);
   if (!($28)) {
    $29 = HEAP32[$pX>>2]|0;
    $30 = ((($29)) + 10|0);
    $31 = HEAP16[$30>>1]|0;
    $32 = $31&65535;
    $33 = HEAP16[$allMask>>1]|0;
    $34 = $33&65535;
    $35 = $34 | $32;
    $36 = $35&65535;
    HEAP16[$allMask>>1] = $36;
   }
   $37 = HEAP32[$pX>>2]|0;
   $38 = ((($37)) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   HEAP32[$pX>>2] = $39;
  }
  $40 = HEAP16[$mask>>1]|0;
  $41 = $40&65535;
  $42 = HEAP16[$allMask>>1]|0;
  $43 = $42&65535;
  $44 = $41 & $43;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$pDbFd>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = (($47) + 120)|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = (_unixShmSystemLock($46,2,$48,$49)|0);
   HEAP32[$rc>>2] = $50;
  } else {
   HEAP32[$rc>>2] = 0;
  }
  $51 = HEAP32[$rc>>2]|0;
  $52 = ($51|0)==(0);
  if (!($52)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $53 = HEAP16[$mask>>1]|0;
  $54 = $53&65535;
  $55 = $54 ^ -1;
  $56 = HEAP32[$p>>2]|0;
  $57 = ((($56)) + 12|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & $55;
  $61 = $60&65535;
  HEAP16[$57>>1] = $61;
  $62 = HEAP16[$mask>>1]|0;
  $63 = $62&65535;
  $64 = $63 ^ -1;
  $65 = HEAP32[$p>>2]|0;
  $66 = ((($65)) + 10|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = $68 & $64;
  $70 = $69&65535;
  HEAP16[$66>>1] = $70;
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $71 = HEAP32[$3>>2]|0;
 $72 = $71 & 4;
 $73 = ($72|0)!=(0);
 if (!($73)) {
  $121 = HEAP32[$pShmNode>>2]|0;
  $122 = ((($121)) + 32|0);
  $123 = HEAP32[$122>>2]|0;
  HEAP32[$pX>>2] = $123;
  while(1) {
   $124 = HEAP32[$pX>>2]|0;
   $125 = ($124|0)!=(0|0);
   if (!($125)) {
    break;
   }
   $126 = HEAP32[$pX>>2]|0;
   $127 = ((($126)) + 12|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128&65535;
   $130 = HEAP16[$mask>>1]|0;
   $131 = $130&65535;
   $132 = $129 & $131;
   $133 = ($132|0)!=(0);
   if ($133) {
    label = 28;
    break;
   }
   $134 = HEAP32[$pX>>2]|0;
   $135 = ((($134)) + 10|0);
   $136 = HEAP16[$135>>1]|0;
   $137 = $136&65535;
   $138 = HEAP16[$mask>>1]|0;
   $139 = $138&65535;
   $140 = $137 & $139;
   $141 = ($140|0)!=(0);
   if ($141) {
    label = 28;
    break;
   }
   $142 = HEAP32[$pX>>2]|0;
   $143 = ((($142)) + 4|0);
   $144 = HEAP32[$143>>2]|0;
   HEAP32[$pX>>2] = $144;
  }
  if ((label|0) == 28) {
   HEAP32[$rc>>2] = 5;
  }
  $145 = HEAP32[$rc>>2]|0;
  $146 = ($145|0)==(0);
  if (!($146)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $147 = HEAP32[$pDbFd>>2]|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = (($148) + 120)|0;
  $150 = HEAP32[$2>>2]|0;
  $151 = (_unixShmSystemLock($147,1,$149,$150)|0);
  HEAP32[$rc>>2] = $151;
  $152 = HEAP32[$rc>>2]|0;
  $153 = ($152|0)==(0);
  if (!($153)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $154 = HEAP16[$mask>>1]|0;
  $155 = $154&65535;
  $156 = HEAP32[$p>>2]|0;
  $157 = ((($156)) + 12|0);
  $158 = HEAP16[$157>>1]|0;
  $159 = $158&65535;
  $160 = $159 | $155;
  $161 = $160&65535;
  HEAP16[$157>>1] = $161;
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 HEAP16[$allShared>>1] = 0;
 $74 = HEAP32[$pShmNode>>2]|0;
 $75 = ((($74)) + 32|0);
 $76 = HEAP32[$75>>2]|0;
 HEAP32[$pX>>2] = $76;
 while(1) {
  $77 = HEAP32[$pX>>2]|0;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$pX>>2]|0;
  $80 = ((($79)) + 12|0);
  $81 = HEAP16[$80>>1]|0;
  $82 = $81&65535;
  $83 = HEAP16[$mask>>1]|0;
  $84 = $83&65535;
  $85 = $82 & $84;
  $86 = ($85|0)!=(0);
  if ($86) {
   label = 16;
   break;
  }
  $87 = HEAP32[$pX>>2]|0;
  $88 = ((($87)) + 10|0);
  $89 = HEAP16[$88>>1]|0;
  $90 = $89&65535;
  $91 = HEAP16[$allShared>>1]|0;
  $92 = $91&65535;
  $93 = $92 | $90;
  $94 = $93&65535;
  HEAP16[$allShared>>1] = $94;
  $95 = HEAP32[$pX>>2]|0;
  $96 = ((($95)) + 4|0);
  $97 = HEAP32[$96>>2]|0;
  HEAP32[$pX>>2] = $97;
 }
 if ((label|0) == 16) {
  HEAP32[$rc>>2] = 5;
 }
 $98 = HEAP32[$rc>>2]|0;
 $99 = ($98|0)==(0);
 do {
  if ($99) {
   $100 = HEAP16[$allShared>>1]|0;
   $101 = $100&65535;
   $102 = HEAP16[$mask>>1]|0;
   $103 = $102&65535;
   $104 = $101 & $103;
   $105 = ($104|0)==(0);
   if ($105) {
    $106 = HEAP32[$pDbFd>>2]|0;
    $107 = HEAP32[$1>>2]|0;
    $108 = (($107) + 120)|0;
    $109 = HEAP32[$2>>2]|0;
    $110 = (_unixShmSystemLock($106,0,$108,$109)|0);
    HEAP32[$rc>>2] = $110;
    break;
   } else {
    HEAP32[$rc>>2] = 0;
    break;
   }
  }
 } while(0);
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 if (!($112)) {
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $113 = HEAP16[$mask>>1]|0;
 $114 = $113&65535;
 $115 = HEAP32[$p>>2]|0;
 $116 = ((($115)) + 10|0);
 $117 = HEAP16[$116>>1]|0;
 $118 = $117&65535;
 $119 = $118 | $114;
 $120 = $119&65535;
 HEAP16[$116>>1] = $120;
 $162 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _unixShmBarrier($fd) {
 $fd = $fd|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $fd;
 _unixEnterMutex();
 _unixLeaveMutex();
 STACKTOP = sp;return;
}
function _unixShmUnmap($fd,$deleteFlag) {
 $fd = $fd|0;
 $deleteFlag = $deleteFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pDbFd = 0, $pShmNode = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $pShmNode = sp + 8|0;
 $pp = sp + 4|0;
 $pDbFd = sp;
 HEAP32[$1>>2] = $fd;
 HEAP32[$2>>2] = $deleteFlag;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pDbFd>>2] = $3;
 $4 = HEAP32[$pDbFd>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pShmNode>>2] = $10;
 $11 = HEAP32[$pShmNode>>2]|0;
 $12 = ((($11)) + 32|0);
 HEAP32[$pp>>2] = $12;
 while(1) {
  $13 = HEAP32[$pp>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ($14|0)!=($15|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pp>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ((($18)) + 4|0);
  HEAP32[$pp>>2] = $19;
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pp>>2]|0;
 HEAP32[$23>>2] = $22;
 $24 = HEAP32[$p>>2]|0;
 _sqlite3_free($24);
 $25 = HEAP32[$pDbFd>>2]|0;
 $26 = ((($25)) + 36|0);
 HEAP32[$26>>2] = 0;
 _unixEnterMutex();
 $27 = HEAP32[$pShmNode>>2]|0;
 $28 = ((($27)) + 28|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + -1)|0;
 HEAP32[$28>>2] = $30;
 $31 = HEAP32[$pShmNode>>2]|0;
 $32 = ((($31)) + 28|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)==(0);
 if ($34) {
  $35 = HEAP32[$2>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$pShmNode>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)>=(0);
   if ($40) {
    $41 = HEAP32[(1260)>>2]|0;
    $42 = HEAP32[$pShmNode>>2]|0;
    $43 = ((($42)) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    (FUNCTION_TABLE_ii[$41 & 255]($44)|0);
   }
  }
  $45 = HEAP32[$pDbFd>>2]|0;
  _unixShmPurge($45);
 }
 _unixLeaveMutex();
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _unixFetch($fd,$0,$1,$nAmt,$pp) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $nAmt = $nAmt|0;
 $pp = $pp|0;
 var $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nAmt;
 HEAP32[$5>>2] = $pp;
 $10 = HEAP32[$5>>2]|0;
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return 0;
}
function _unixUnfetch($fd,$0,$1,$p) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $p = $p|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $p;
 STACKTOP = sp;return 0;
}
function _verifyDbFile($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $buf = sp + 32|0;
 $rc = sp + 28|0;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[(1128)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (FUNCTION_TABLE_iii[$1 & 255]($4,$buf)|0);
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 32|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$vararg_buffer>>2] = $10;
  _sqlite3_log(28,18214,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $11 = ((($buf)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 18|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = $17 & 32;
  $19 = ($18|0)==(0);
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 32|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $22;
   _sqlite3_log(28,18238,$vararg_buffer1);
   STACKTOP = sp;return;
  }
 }
 $23 = ((($buf)) + 16|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24>>>0)>(1);
 $26 = HEAP32[$0>>2]|0;
 if ($25) {
  $27 = ((($26)) + 32|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $28;
  _sqlite3_log(28,18267,$vararg_buffer4);
  STACKTOP = sp;return;
 }
 $29 = (_fileHasMoved($26)|0);
 $30 = ($29|0)!=(0);
 if (!($30)) {
  STACKTOP = sp;return;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 32|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$vararg_buffer7>>2] = $33;
 _sqlite3_log(28,18294,$vararg_buffer7);
 STACKTOP = sp;return;
}
function _unixEnterMutex() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _setPendingFd($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pInode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pInode = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$pInode>>2]|0;
 $8 = ((($7)) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 8|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$p>>2]|0;
 $13 = HEAP32[$pInode>>2]|0;
 $14 = ((($13)) + 28|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = -1;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 28|0);
 HEAP32[$18>>2] = 0;
 STACKTOP = sp;return;
}
function _releaseInodeInfo($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInode = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pInode = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$pInode>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pInode>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$pInode>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 _closePendingFds($14);
 $15 = HEAP32[$pInode>>2]|0;
 $16 = ((($15)) + 36|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$pInode>>2]|0;
 $20 = ((($19)) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 if ($18) {
  $22 = HEAP32[$pInode>>2]|0;
  $23 = ((($22)) + 36|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 32|0);
  HEAP32[$25>>2] = $21;
 } else {
  HEAP32[1400>>2] = $21;
 }
 $26 = HEAP32[$pInode>>2]|0;
 $27 = ((($26)) + 32|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$pInode>>2]|0;
  $31 = ((($30)) + 36|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$pInode>>2]|0;
  $34 = ((($33)) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ((($35)) + 36|0);
  HEAP32[$36>>2] = $32;
 }
 $37 = HEAP32[$pInode>>2]|0;
 _sqlite3_free($37);
 STACKTOP = sp;return;
}
function _closeUnixFile($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$pFile>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(0);
 if ($5) {
  $6 = HEAP32[$pFile>>2]|0;
  $7 = HEAP32[$pFile>>2]|0;
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  _robust_close($6,$9,29216);
  $10 = HEAP32[$pFile>>2]|0;
  $11 = ((($10)) + 12|0);
  HEAP32[$11>>2] = -1;
 }
 $12 = HEAP32[$pFile>>2]|0;
 $13 = ((($12)) + 28|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3_free($14);
 $15 = HEAP32[$pFile>>2]|0;
 dest=$15; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return 0;
}
function _unixLeaveMutex() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _fileHasMoved($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $buf = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $20 = 0;
  $19 = $20&1;
  STACKTOP = sp;return ($19|0);
 }
 $5 = HEAP32[(1116)>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (FUNCTION_TABLE_iii[$5 & 255]($8,$buf)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $20 = 1;
  $19 = $20&1;
  STACKTOP = sp;return ($19|0);
 }
 $11 = ((($buf)) + 72|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($12|0)!=($17|0);
 $20 = $18;
 $19 = $20&1;
 STACKTOP = sp;return ($19|0);
}
function _posixOpen($zFile,$flags,$mode) {
 $zFile = $zFile|0;
 $flags = $flags|0;
 $mode = $mode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $zFile;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $mode;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $6 = (_open($3,$4,$vararg_buffer)|0);
 STACKTOP = sp;return ($6|0);
}
function _openDirectory($zFilename,$pFd) {
 $zFilename = $zFilename|0;
 $pFd = $pFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $ii = 0, $vararg_buffer = 0, $zDirname = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 544|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $ii = sp + 8|0;
 $fd = sp + 4|0;
 $zDirname = sp + 24|0;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $pFd;
 HEAP32[$fd>>2] = -1;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $3;
 (_sqlite3_snprintf(512,$zDirname,17521,$vararg_buffer)|0);
 $4 = (_strlen($zDirname)|0);
 HEAP32[$ii>>2] = $4;
 while(1) {
  $5 = HEAP32[$ii>>2]|0;
  $6 = ($5|0)>(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$ii>>2]|0;
  $8 = (($zDirname) + ($7)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = ($10|0)!=(47);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$ii>>2]|0;
  $13 = (($12) + -1)|0;
  HEAP32[$ii>>2] = $13;
 }
 $14 = HEAP32[$ii>>2]|0;
 $15 = ($14|0)>(0);
 if ($15) {
  $16 = HEAP32[$ii>>2]|0;
  $17 = (($zDirname) + ($16)|0);
  HEAP8[$17>>0] = 0;
 } else {
  $18 = HEAP8[$zDirname>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = ($19|0)!=(47);
  if ($20) {
   HEAP8[$zDirname>>0] = 46;
  }
  $21 = ((($zDirname)) + 1|0);
  HEAP8[$21>>0] = 0;
 }
 $22 = (_robust_open($zDirname,0,0)|0);
 HEAP32[$fd>>2] = $22;
 $23 = HEAP32[$fd>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$fd>>2]|0;
 $26 = ($25|0)>=(0);
 if ($26) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 } else {
  $27 = (_sqlite3CantopenError(30807)|0);
  $28 = (_unixLogErrorAtLine($27,18439,$zDirname,30807)|0);
  HEAP32[$0>>2] = $28;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _unixGetpagesize() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_sysconf(30)|0);
 return ($0|0);
}
function _robust_open($z,$f,$m) {
 $z = $z|0;
 $f = $f|0;
 $m = $m|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $m2 = 0, $or$cond = 0, $statbuf = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $fd = sp + 88|0;
 $m2 = sp + 84|0;
 $statbuf = sp + 8|0;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $f;
 HEAP32[$2>>2] = $m;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 $5 = HEAP32[$2>>2]|0;
 $6 = $4 ? $5 : 420;
 HEAP32[$m2>>2] = $6;
 while(1) {
  $7 = HEAP32[(1068)>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = $9 | 524288;
  $11 = HEAP32[$m2>>2]|0;
  $12 = (FUNCTION_TABLE_iiii[$7 & 255]($8,$10,$11)|0);
  HEAP32[$fd>>2] = $12;
  $13 = HEAP32[$fd>>2]|0;
  $14 = ($13|0)<(0);
  if ($14) {
   $15 = (___errno_location()|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)==(4);
   if ($17) {
    continue;
   } else {
    break;
   }
  }
  $18 = HEAP32[$fd>>2]|0;
  $19 = ($18|0)>=(3);
  if ($19) {
   break;
  }
  $20 = HEAP32[(1080)>>2]|0;
  $21 = HEAP32[$fd>>2]|0;
  (FUNCTION_TABLE_ii[$20 & 255]($21)|0);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$fd>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr2>>2] = $23;
  _sqlite3_log(28,18526,$vararg_buffer);
  HEAP32[$fd>>2] = -1;
  $24 = HEAP32[(1068)>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (FUNCTION_TABLE_iiii[$24 & 255](18569,$25,$26)|0);
  $28 = ($27|0)<(0);
  if ($28) {
   break;
  }
 }
 $29 = HEAP32[$fd>>2]|0;
 $30 = ($29|0)>=(0);
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0);
 $or$cond = $30 & $32;
 if (!($or$cond)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $33 = HEAP32[(1128)>>2]|0;
 $34 = HEAP32[$fd>>2]|0;
 $35 = (FUNCTION_TABLE_iii[$33 & 255]($34,$statbuf)|0);
 $36 = ($35|0)==(0);
 if (!($36)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $37 = ((($statbuf)) + 36|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)==(0);
 if (!($39)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $40 = ((($statbuf)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 & 511;
 $43 = HEAP32[$2>>2]|0;
 $44 = ($42|0)!=($43|0);
 if (!($44)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $45 = HEAP32[(1236)>>2]|0;
 $46 = HEAP32[$fd>>2]|0;
 $47 = HEAP32[$2>>2]|0;
 (FUNCTION_TABLE_iii[$45 & 255]($46,$47)|0);
 $48 = HEAP32[$fd>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _unixLogErrorAtLine($errcode,$zFunc,$zPath,$iLine) {
 $errcode = $errcode|0;
 $zFunc = $zFunc|0;
 $zPath = $zPath|0;
 $iLine = $iLine|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iErrno = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $zErr = sp + 24|0;
 $iErrno = sp + 20|0;
 HEAP32[$0>>2] = $errcode;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $iLine;
 $4 = (___errno_location()|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$iErrno>>2] = $5;
 $6 = HEAP32[$iErrno>>2]|0;
 $7 = (_strerror($6)|0);
 HEAP32[$zErr>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$2>>2] = 17532;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$iErrno>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$zErr>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $12;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $14;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $15;
 _sqlite3_log($10,18579,$vararg_buffer);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3CantopenError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_sourceid()|0);
 $3 = ((($2)) + 20|0);
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $3;
 _sqlite3_log(14,18610,$vararg_buffer);
 STACKTOP = sp;return 14;
}
function _closePendingFds($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pInode = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pInode = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$pInode>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$p>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pNext>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  _robust_close($12,$14,28492);
  $15 = HEAP32[$p>>2]|0;
  _sqlite3_free($15);
  $16 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $16;
 }
 $17 = HEAP32[$pInode>>2]|0;
 $18 = ((($17)) + 28|0);
 HEAP32[$18>>2] = 0;
 STACKTOP = sp;return;
}
function _robust_close($pFile,$h,$lineno) {
 $pFile = $pFile|0;
 $h = $h|0;
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $lineno;
 $3 = HEAP32[(1080)>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (FUNCTION_TABLE_ii[$3 & 255]($4)|0);
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 32|0);
  $11 = HEAP32[$10>>2]|0;
  $13 = $11;
 } else {
  $13 = 0;
 }
 $12 = HEAP32[$2>>2]|0;
 (_unixLogErrorAtLine(4106,18327,$13,$12)|0);
 STACKTOP = sp;return;
}
function _seekAndRead($id,$0,$1,$pBuf,$cnt) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $cnt = $cnt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $got = 0, $newOffset = 0, $prior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 8|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $got = sp + 20|0;
 $prior = sp + 16|0;
 $newOffset = sp;
 HEAP32[$3>>2] = $id;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $pBuf;
 HEAP32[$6>>2] = $cnt;
 HEAP32[$prior>>2] = 0;
 while(1) {
  $11 = HEAP32[$3>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $4;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_lseek($13,$16,0)|0);
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = $newOffset;
  $24 = $23;
  HEAP32[$24>>2] = $20;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $newOffset;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)<(0);
  if ($33) {
   label = 3;
   break;
  }
  $37 = HEAP32[(1164)>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = ((($38)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$5>>2]|0;
  $42 = HEAP32[$6>>2]|0;
  $43 = (FUNCTION_TABLE_iiii[$37 & 255]($40,$41,$42)|0);
  HEAP32[$got>>2] = $43;
  $44 = HEAP32[$got>>2]|0;
  $45 = HEAP32[$6>>2]|0;
  $46 = ($44|0)==($45|0);
  if ($46) {
   break;
  }
  $47 = HEAP32[$got>>2]|0;
  $48 = ($47|0)<(0);
  if ($48) {
   $49 = (___errno_location()|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($50|0)==(4);
   if (!($51)) {
    label = 8;
    break;
   }
   HEAP32[$got>>2] = 1;
  } else {
   $55 = HEAP32[$got>>2]|0;
   $56 = ($55|0)>(0);
   if ($56) {
    $57 = HEAP32[$got>>2]|0;
    $58 = HEAP32[$6>>2]|0;
    $59 = (($58) - ($57))|0;
    HEAP32[$6>>2] = $59;
    $60 = HEAP32[$got>>2]|0;
    $61 = ($60|0)<(0);
    $62 = $61 << 31 >> 31;
    $63 = $4;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (($63) + 4)|0;
    $67 = $66;
    $68 = HEAP32[$67>>2]|0;
    $69 = (_i64Add(($65|0),($68|0),($60|0),($62|0))|0);
    $70 = tempRet0;
    $71 = $4;
    $72 = $71;
    HEAP32[$72>>2] = $69;
    $73 = (($71) + 4)|0;
    $74 = $73;
    HEAP32[$74>>2] = $70;
    $75 = HEAP32[$got>>2]|0;
    $76 = HEAP32[$prior>>2]|0;
    $77 = (($76) + ($75))|0;
    HEAP32[$prior>>2] = $77;
    $78 = HEAP32[$got>>2]|0;
    $79 = HEAP32[$5>>2]|0;
    $80 = (($79) + ($78)|0);
    HEAP32[$5>>2] = $80;
   }
  }
  $81 = HEAP32[$got>>2]|0;
  $82 = ($81|0)>(0);
  if (!($82)) {
   break;
  }
 }
 if ((label|0) == 3) {
  $34 = HEAP32[$3>>2]|0;
  $35 = (___errno_location()|0);
  $36 = HEAP32[$35>>2]|0;
  _storeLastErrno($34,$36);
  HEAP32[$2>>2] = -1;
  $86 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$prior>>2] = 0;
  $52 = HEAP32[$3>>2]|0;
  $53 = (___errno_location()|0);
  $54 = HEAP32[$53>>2]|0;
  _storeLastErrno($52,$54);
 }
 $83 = HEAP32[$got>>2]|0;
 $84 = HEAP32[$prior>>2]|0;
 $85 = (($83) + ($84))|0;
 HEAP32[$2>>2] = $85;
 $86 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _storeLastErrno($pFile,$error) {
 $pFile = $pFile|0;
 $error = $error|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $error;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = $2;
 STACKTOP = sp;return;
}
function _seekAndWrite($id,$0,$1,$pBuf,$cnt) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $cnt = $cnt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $cnt;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $3;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 20|0);
 $23 = (_seekAndWriteFd($12,$15,$18,$19,$20,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _seekAndWriteFd($fd,$0,$1,$pBuf,$nBuf,$piErrno) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $nBuf = $nBuf|0;
 $piErrno = $piErrno|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iSeek = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 8|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $rc = sp + 16|0;
 $iSeek = sp;
 HEAP32[$2>>2] = $fd;
 $7 = $3;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $nBuf;
 HEAP32[$6>>2] = $piErrno;
 HEAP32[$rc>>2] = 0;
 $11 = HEAP32[$5>>2]|0;
 $12 = $11 & 131071;
 HEAP32[$5>>2] = $12;
 while(1) {
  $13 = HEAP32[$2>>2]|0;
  $14 = $3;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_lseek($13,$16,0)|0);
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = $iSeek;
  $24 = $23;
  HEAP32[$24>>2] = $20;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $iSeek;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)<(0);
  if ($33) {
   label = 3;
   break;
  }
  $34 = HEAP32[(1200)>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = HEAP32[$4>>2]|0;
  $37 = HEAP32[$5>>2]|0;
  $38 = (FUNCTION_TABLE_iiii[$34 & 255]($35,$36,$37)|0);
  HEAP32[$rc>>2] = $38;
  $39 = HEAP32[$rc>>2]|0;
  $40 = ($39|0)<(0);
  if (!($40)) {
   break;
  }
  $41 = (___errno_location()|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)==(4);
  if (!($43)) {
   break;
  }
 }
 if ((label|0) == 3) {
  HEAP32[$rc>>2] = -1;
 }
 $44 = HEAP32[$rc>>2]|0;
 $45 = ($44|0)<(0);
 if (!($45)) {
  $49 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $46 = (___errno_location()|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$6>>2]|0;
 HEAP32[$48>>2] = $47;
 $49 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _robust_ftruncate($h,$0,$1) {
 $h = $h|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $h;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 while(1) {
  $8 = HEAP32[(1140)>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = $3;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (FUNCTION_TABLE_iii[$8 & 255]($9,$12)|0);
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$rc>>2]|0;
  $18 = ($17|0)<(0);
  if (!($18)) {
   label = 4;
   break;
  }
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(4);
  if (!($21)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _full_fsync($fd,$fullSync,$dataOnly) {
 $fd = $fd|0;
 $fullSync = $fullSync|0;
 $dataOnly = $dataOnly|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $fullSync;
 HEAP32[$2>>2] = $dataOnly;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_fsync($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _unixFileLock($pFile,$pLock) {
 $pFile = $pFile|0;
 $pLock = $pLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lock = 0, $pInode = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $rc = sp + 36|0;
 $pInode = sp + 32|0;
 $lock = sp + 16|0;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $pLock;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pInode>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 18|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 $10 = $9 & 3;
 $11 = ($10|0)==(1);
 do {
  if ($11) {
   $12 = HEAP32[$pInode>>2]|0;
   $13 = ((($12)) + 13|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)==(0);
   if (!($16)) {
    HEAP32[$rc>>2] = 0;
    break;
   }
   $17 = ((($lock)) + 2|0);
   HEAP16[$17>>1] = 0;
   $18 = HEAP32[832>>2]|0;
   $19 = (($18) + 2)|0;
   $20 = ((($lock)) + 4|0);
   HEAP32[$20>>2] = $19;
   $21 = ((($lock)) + 8|0);
   HEAP32[$21>>2] = 510;
   HEAP16[$lock>>1] = 1;
   $22 = HEAP32[(1152)>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 12|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$vararg_buffer>>2] = $lock;
   $26 = (FUNCTION_TABLE_iiii[$22 & 255]($25,13,$vararg_buffer)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$rc>>2]|0;
   $28 = ($27|0)<(0);
   if (!($28)) {
    $30 = HEAP32[$pInode>>2]|0;
    $31 = ((($30)) + 13|0);
    HEAP8[$31>>0] = 1;
    $32 = HEAP32[$pInode>>2]|0;
    $33 = ((($32)) + 24|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = (($34) + 1)|0;
    HEAP32[$33>>2] = $35;
    break;
   }
   $29 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $29;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  } else {
   $36 = HEAP32[(1152)>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $40;
   $41 = (FUNCTION_TABLE_iiii[$36 & 255]($39,13,$vararg_buffer1)|0);
   HEAP32[$rc>>2] = $41;
  }
 } while(0);
 $42 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $42;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _sqliteErrorFromPosixError($posixError,$sqliteIOErr) {
 $posixError = $posixError|0;
 $sqliteIOErr = $sqliteIOErr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $posixError;
 HEAP32[$2>>2] = $sqliteIOErr;
 $3 = HEAP32[$1>>2]|0;
 switch ($3|0) {
 case 37: case 4: case 16: case 110: case 11: case 13:  {
  HEAP32[$0>>2] = 5;
  break;
 }
 case 1:  {
  HEAP32[$0>>2] = 3;
  break;
 }
 default: {
  $4 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $4;
 }
 }
 $5 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _posixUnlock($id,$eFileLock,$handleNFSUnlock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 $handleNFSUnlock = $handleNFSUnlock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $lock = 0, $pFile = 0, $pInode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pFile = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 $rc = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 HEAP32[$3>>2] = $handleNFSUnlock;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pFile>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($8|0)<=($9|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 _unixEnterMutex();
 $11 = HEAP32[$pFile>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pInode>>2] = $13;
 $14 = HEAP32[$pFile>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = ($17|0)>(1);
 do {
  if ($18) {
   $19 = HEAP32[$2>>2]|0;
   $20 = ($19|0)==(1);
   if ($20) {
    HEAP16[$lock>>1] = 0;
    $21 = ((($lock)) + 2|0);
    HEAP16[$21>>1] = 0;
    $22 = HEAP32[832>>2]|0;
    $23 = (($22) + 2)|0;
    $24 = ((($lock)) + 4|0);
    HEAP32[$24>>2] = $23;
    $25 = ((($lock)) + 8|0);
    HEAP32[$25>>2] = 510;
    $26 = HEAP32[$pFile>>2]|0;
    $27 = (_unixFileLock($26,$lock)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$rc>>2] = 2314;
     $29 = HEAP32[$pFile>>2]|0;
     $30 = (___errno_location()|0);
     $31 = HEAP32[$30>>2]|0;
     _storeLastErrno($29,$31);
     break;
    }
   }
   HEAP16[$lock>>1] = 2;
   $32 = ((($lock)) + 2|0);
   HEAP16[$32>>1] = 0;
   $33 = HEAP32[832>>2]|0;
   $34 = ((($lock)) + 4|0);
   HEAP32[$34>>2] = $33;
   $35 = ((($lock)) + 8|0);
   HEAP32[$35>>2] = 2;
   $36 = HEAP32[$pFile>>2]|0;
   $37 = (_unixFileLock($36,$lock)|0);
   $38 = ($37|0)==(0);
   if ($38) {
    $39 = HEAP32[$pInode>>2]|0;
    $40 = ((($39)) + 12|0);
    HEAP8[$40>>0] = 1;
    label = 10;
    break;
   } else {
    HEAP32[$rc>>2] = 2058;
    $41 = HEAP32[$pFile>>2]|0;
    $42 = (___errno_location()|0);
    $43 = HEAP32[$42>>2]|0;
    _storeLastErrno($41,$43);
    break;
   }
  } else {
   label = 10;
  }
 } while(0);
 if ((label|0) == 10) {
  $44 = HEAP32[$2>>2]|0;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$pInode>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + -1)|0;
   HEAP32[$47>>2] = $49;
   $50 = HEAP32[$pInode>>2]|0;
   $51 = ((($50)) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)==(0);
   do {
    if ($53) {
     HEAP16[$lock>>1] = 2;
     $54 = ((($lock)) + 2|0);
     HEAP16[$54>>1] = 0;
     $55 = ((($lock)) + 8|0);
     HEAP32[$55>>2] = 0;
     $56 = ((($lock)) + 4|0);
     HEAP32[$56>>2] = 0;
     $57 = HEAP32[$pFile>>2]|0;
     $58 = (_unixFileLock($57,$lock)|0);
     $59 = ($58|0)==(0);
     if ($59) {
      $60 = HEAP32[$pInode>>2]|0;
      $61 = ((($60)) + 12|0);
      HEAP8[$61>>0] = 0;
      break;
     } else {
      HEAP32[$rc>>2] = 2058;
      $62 = HEAP32[$pFile>>2]|0;
      $63 = (___errno_location()|0);
      $64 = HEAP32[$63>>2]|0;
      _storeLastErrno($62,$64);
      $65 = HEAP32[$pInode>>2]|0;
      $66 = ((($65)) + 12|0);
      HEAP8[$66>>0] = 0;
      $67 = HEAP32[$pFile>>2]|0;
      $68 = ((($67)) + 16|0);
      HEAP8[$68>>0] = 0;
      break;
     }
    }
   } while(0);
   $69 = HEAP32[$pInode>>2]|0;
   $70 = ((($69)) + 24|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + -1)|0;
   HEAP32[$70>>2] = $72;
   $73 = HEAP32[$pInode>>2]|0;
   $74 = ((($73)) + 24|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75|0)==(0);
   if ($76) {
    $77 = HEAP32[$pFile>>2]|0;
    _closePendingFds($77);
   }
  }
 }
 _unixLeaveMutex();
 $78 = HEAP32[$rc>>2]|0;
 $79 = ($78|0)==(0);
 if ($79) {
  $80 = HEAP32[$2>>2]|0;
  $81 = $80&255;
  $82 = HEAP32[$pFile>>2]|0;
  $83 = ((($82)) + 16|0);
  HEAP8[$83>>0] = $81;
 }
 $84 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $84;
 $85 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($85|0);
}
function _fcntlSizeHint($pFile,$0,$1) {
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $buf = 0, $iWrite = 0, $nBlk = 0, $nSize = 0, $nWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $4 = sp + 16|0;
 $nSize = sp + 8|0;
 $buf = sp + 32|0;
 $nBlk = sp + 28|0;
 $nWrite = sp + 24|0;
 $iWrite = sp;
 HEAP32[$3>>2] = $pFile;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 L1: do {
  if ($12) {
   $13 = HEAP32[(1128)>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = ((($14)) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (FUNCTION_TABLE_iii[$13 & 255]($16,$buf)|0);
   $18 = ($17|0)!=(0);
   if ($18) {
    HEAP32[$2>>2] = 1802;
    $162 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($162|0);
   }
   $19 = $4;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $22 = (($19) + 4)|0;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = ((($25)) + 40|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)<(0);
   $29 = $28 << 31 >> 31;
   $30 = (_i64Add(($21|0),($24|0),($27|0),($29|0))|0);
   $31 = tempRet0;
   $32 = (_i64Subtract(($30|0),($31|0),1,0)|0);
   $33 = tempRet0;
   $34 = HEAP32[$3>>2]|0;
   $35 = ((($34)) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 << 31 >> 31;
   $39 = (___divdi3(($32|0),($33|0),($36|0),($38|0))|0);
   $40 = tempRet0;
   $41 = HEAP32[$3>>2]|0;
   $42 = ((($41)) + 40|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)<(0);
   $45 = $44 << 31 >> 31;
   $46 = (___muldi3(($39|0),($40|0),($43|0),($45|0))|0);
   $47 = tempRet0;
   $48 = $nSize;
   $49 = $48;
   HEAP32[$49>>2] = $46;
   $50 = (($48) + 4)|0;
   $51 = $50;
   HEAP32[$51>>2] = $47;
   $52 = $nSize;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($52) + 4)|0;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = ((($buf)) + 36|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)<(0);
   $61 = $60 << 31 >> 31;
   $62 = ($57|0)>($61|0);
   $63 = ($54>>>0)>($59>>>0);
   $64 = ($57|0)==($61|0);
   $65 = $64 & $63;
   $66 = $62 | $65;
   if ($66) {
    $67 = ((($buf)) + 40|0);
    $68 = HEAP32[$67>>2]|0;
    HEAP32[$nBlk>>2] = $68;
    HEAP32[$nWrite>>2] = 0;
    $69 = ((($buf)) + 36|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = HEAP32[$nBlk>>2]|0;
    $72 = (($70|0) / ($71|0))&-1;
    $73 = HEAP32[$nBlk>>2]|0;
    $74 = Math_imul($72, $73)|0;
    $75 = HEAP32[$nBlk>>2]|0;
    $76 = (($74) + ($75))|0;
    $77 = (($76) - 1)|0;
    $78 = ($77|0)<(0);
    $79 = $78 << 31 >> 31;
    $80 = $iWrite;
    $81 = $80;
    HEAP32[$81>>2] = $77;
    $82 = (($80) + 4)|0;
    $83 = $82;
    HEAP32[$83>>2] = $79;
    while(1) {
     $84 = $iWrite;
     $85 = $84;
     $86 = HEAP32[$85>>2]|0;
     $87 = (($84) + 4)|0;
     $88 = $87;
     $89 = HEAP32[$88>>2]|0;
     $90 = $nSize;
     $91 = $90;
     $92 = HEAP32[$91>>2]|0;
     $93 = (($90) + 4)|0;
     $94 = $93;
     $95 = HEAP32[$94>>2]|0;
     $96 = HEAP32[$nBlk>>2]|0;
     $97 = ($96|0)<(0);
     $98 = $97 << 31 >> 31;
     $99 = (_i64Add(($92|0),($95|0),($96|0),($98|0))|0);
     $100 = tempRet0;
     $101 = (_i64Subtract(($99|0),($100|0),1,0)|0);
     $102 = tempRet0;
     $103 = ($89|0)<($102|0);
     $104 = ($86>>>0)<($101>>>0);
     $105 = ($89|0)==($102|0);
     $106 = $105 & $104;
     $107 = $103 | $106;
     if (!($107)) {
      break L1;
     }
     $108 = $iWrite;
     $109 = $108;
     $110 = HEAP32[$109>>2]|0;
     $111 = (($108) + 4)|0;
     $112 = $111;
     $113 = HEAP32[$112>>2]|0;
     $114 = $nSize;
     $115 = $114;
     $116 = HEAP32[$115>>2]|0;
     $117 = (($114) + 4)|0;
     $118 = $117;
     $119 = HEAP32[$118>>2]|0;
     $120 = ($113|0)>($119|0);
     $121 = ($110>>>0)>=($116>>>0);
     $122 = ($113|0)==($119|0);
     $123 = $122 & $121;
     $124 = $120 | $123;
     if ($124) {
      $125 = $nSize;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = (($125) + 4)|0;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = (_i64Subtract(($127|0),($130|0),1,0)|0);
      $132 = tempRet0;
      $133 = $iWrite;
      $134 = $133;
      HEAP32[$134>>2] = $131;
      $135 = (($133) + 4)|0;
      $136 = $135;
      HEAP32[$136>>2] = $132;
     }
     $137 = HEAP32[$3>>2]|0;
     $138 = $iWrite;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (($138) + 4)|0;
     $142 = $141;
     $143 = HEAP32[$142>>2]|0;
     $144 = (_seekAndWrite($137,$140,$143,17532,1)|0);
     HEAP32[$nWrite>>2] = $144;
     $145 = HEAP32[$nWrite>>2]|0;
     $146 = ($145|0)!=(1);
     if ($146) {
      break;
     }
     $147 = HEAP32[$nBlk>>2]|0;
     $148 = ($147|0)<(0);
     $149 = $148 << 31 >> 31;
     $150 = $iWrite;
     $151 = $150;
     $152 = HEAP32[$151>>2]|0;
     $153 = (($150) + 4)|0;
     $154 = $153;
     $155 = HEAP32[$154>>2]|0;
     $156 = (_i64Add(($152|0),($155|0),($147|0),($149|0))|0);
     $157 = tempRet0;
     $158 = $iWrite;
     $159 = $158;
     HEAP32[$159>>2] = $156;
     $160 = (($158) + 4)|0;
     $161 = $160;
     HEAP32[$161>>2] = $157;
    }
    HEAP32[$2>>2] = 778;
    $162 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($162|0);
   }
  }
 } while(0);
 HEAP32[$2>>2] = 0;
 $162 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _unixModeBit($pFile,$mask,$pArg) {
 $pFile = $pFile|0;
 $mask = $mask|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $2 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP8[$1>>0] = $mask;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 18|0);
  $8 = HEAP16[$7>>1]|0;
  $9 = $8&65535;
  $10 = HEAP8[$1>>0]|0;
  $11 = $10&255;
  $12 = $9 & $11;
  $13 = ($12|0)!=(0);
  $14 = $13&1;
  $15 = HEAP32[$2>>2]|0;
  HEAP32[$15>>2] = $14;
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0);
 $19 = HEAP8[$1>>0]|0;
 $20 = $19&255;
 if ($18) {
  $21 = $20 ^ -1;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 18|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24&65535;
  $26 = $25 & $21;
  $27 = $26&65535;
  HEAP16[$23>>1] = $27;
  STACKTOP = sp;return;
 } else {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 18|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 | $20;
  $33 = $32&65535;
  HEAP16[$29>>1] = $33;
  STACKTOP = sp;return;
 }
}
function _unixGetTempname($nBuf,$zBuf) {
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iLimit = 0, $r = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, $zDir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $zDir = sp + 36|0;
 $iLimit = sp + 32|0;
 $r = sp;
 HEAP32[$1>>2] = $nBuf;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$iLimit>>2] = 0;
 $3 = (_unixTempFileDir()|0);
 HEAP32[$zDir>>2] = $3;
 while(1) {
  _sqlite3_randomness(8,$r);
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) - 2)|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + ($5)|0);
  HEAP8[$7>>0] = 0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$zDir>>2]|0;
  $11 = $r;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($11) + 4)|0;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$vararg_buffer>>2] = $10;
  $vararg_ptr1 = ((($vararg_buffer)) + 8|0);
  $17 = $vararg_ptr1;
  $18 = $17;
  HEAP32[$18>>2] = $13;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $vararg_ptr2 = ((($vararg_buffer)) + 16|0);
  HEAP32[$vararg_ptr2>>2] = 0;
  (_sqlite3_snprintf($8,$9,18660,$vararg_buffer)|0);
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) - 2)|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + ($22)|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 4;
   break;
  }
  $28 = HEAP32[$iLimit>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$iLimit>>2] = $29;
  $30 = ($28|0)>(10);
  if ($30) {
   label = 4;
   break;
  }
  $31 = HEAP32[(1092)>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (FUNCTION_TABLE_iii[$31 & 255]($32,0)|0);
  $34 = ($33|0)==(0);
  if (!($34)) {
   label = 6;
   break;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _unixTempFileDir() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $i = 0, $zDir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $i = sp + 80|0;
 $buf = sp + 4|0;
 $zDir = sp;
 $0 = HEAP32[844>>2]|0;
 HEAP32[$zDir>>2] = $0;
 $1 = HEAP32[1404>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $3 = (_getenv((18700|0))|0);
  HEAP32[1404>>2] = $3;
 }
 $4 = HEAP32[(1408)>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $6 = (_getenv((18714|0))|0);
  HEAP32[(1408)>>2] = $6;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = ($7>>>0)<(6);
  if (!($8)) {
   label = 12;
   break;
  }
  $9 = HEAP32[$zDir>>2]|0;
  $10 = ($9|0)==(0|0);
  if (!($10)) {
   $11 = HEAP32[(1116)>>2]|0;
   $12 = HEAP32[$zDir>>2]|0;
   $13 = (FUNCTION_TABLE_iii[$11 & 255]($12,$buf)|0);
   $14 = ($13|0)!=(0);
   if (!($14)) {
    $15 = ((($buf)) + 12|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = $16 & 61440;
    $18 = ($17|0)==(16384);
    if ($18) {
     $19 = HEAP32[(1092)>>2]|0;
     $20 = HEAP32[$zDir>>2]|0;
     $21 = (FUNCTION_TABLE_iii[$19 & 255]($20,7)|0);
     $22 = ($21|0)!=(0);
     if (!($22)) {
      label = 12;
      break;
     }
    }
   }
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
  $25 = (1404 + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$zDir>>2] = $26;
 }
 if ((label|0) == 12) {
  $27 = HEAP32[$zDir>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _unixShmRegionPerMap() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pgsz = 0, $shmsz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $shmsz = sp + 4|0;
 $pgsz = sp;
 HEAP32[$shmsz>>2] = 32768;
 $1 = HEAP32[(1368)>>2]|0;
 $2 = (FUNCTION_TABLE_i[$1 & 255]()|0);
 HEAP32[$pgsz>>2] = $2;
 $3 = HEAP32[$pgsz>>2]|0;
 $4 = HEAP32[$shmsz>>2]|0;
 $5 = ($3|0)<($4|0);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  $6 = HEAP32[$pgsz>>2]|0;
  $7 = HEAP32[$shmsz>>2]|0;
  $8 = (($6|0) / ($7|0))&-1;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _unixOpenSharedMemory($pDbFd) {
 $pDbFd = $pDbFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nShmFilename = 0, $openFlags = 0, $p = 0, $pInode = 0, $pShmNode = 0, $rc = 0, $sStat = 0, $vararg_buffer = 0, $zBasePath = 0, $zShmFilename = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $p = sp + 108|0;
 $pShmNode = sp + 104|0;
 $rc = sp + 100|0;
 $pInode = sp + 96|0;
 $zShmFilename = sp + 92|0;
 $nShmFilename = sp + 88|0;
 $sStat = sp + 12|0;
 $zBasePath = sp + 8|0;
 $openFlags = sp + 4|0;
 HEAP32[$1>>2] = $pDbFd;
 HEAP32[$p>>2] = 0;
 $2 = (_sqlite3_malloc64(16,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 7;
  $128 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($128|0);
 }
 $5 = HEAP32[$p>>2]|0;
 ;HEAP32[$5>>2]=0|0;HEAP32[$5+4>>2]=0|0;HEAP32[$5+8>>2]=0|0;HEAP32[$5+12>>2]=0|0;
 _unixEnterMutex();
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pInode>>2] = $8;
 $9 = HEAP32[$pInode>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pShmNode>>2] = $11;
 $12 = HEAP32[$pShmNode>>2]|0;
 $13 = ($12|0)==(0|0);
 L5: do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 32|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$zBasePath>>2] = $16;
   $17 = HEAP32[(1128)>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (FUNCTION_TABLE_iii[$17 & 255]($20,$sStat)|0);
   $22 = ($21|0)!=(0);
   do {
    if ($22) {
     HEAP32[$rc>>2] = 1802;
    } else {
     $23 = HEAP32[$zBasePath>>2]|0;
     $24 = (_strlen($23)|0);
     $25 = (6 + ($24))|0;
     HEAP32[$nShmFilename>>2] = $25;
     $26 = HEAP32[$nShmFilename>>2]|0;
     $27 = (36 + ($26))|0;
     $28 = (_sqlite3_malloc64($27,0)|0);
     HEAP32[$pShmNode>>2] = $28;
     $29 = HEAP32[$pShmNode>>2]|0;
     $30 = ($29|0)==(0|0);
     if ($30) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $31 = HEAP32[$pShmNode>>2]|0;
     $32 = HEAP32[$nShmFilename>>2]|0;
     $33 = (36 + ($32))|0;
     _memset(($31|0),0,($33|0))|0;
     $34 = HEAP32[$pShmNode>>2]|0;
     $35 = ((($34)) + 36|0);
     $36 = HEAP32[$pShmNode>>2]|0;
     $37 = ((($36)) + 8|0);
     HEAP32[$37>>2] = $35;
     HEAP32[$zShmFilename>>2] = $35;
     $38 = HEAP32[$nShmFilename>>2]|0;
     $39 = HEAP32[$zShmFilename>>2]|0;
     $40 = HEAP32[$zBasePath>>2]|0;
     HEAP32[$vararg_buffer>>2] = $40;
     (_sqlite3_snprintf($38,$39,18721,$vararg_buffer)|0);
     $41 = HEAP32[$pShmNode>>2]|0;
     $42 = ((($41)) + 12|0);
     HEAP32[$42>>2] = -1;
     $43 = HEAP32[$pShmNode>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = ((($44)) + 8|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ((($46)) + 20|0);
     HEAP32[$47>>2] = $43;
     $48 = HEAP32[$1>>2]|0;
     $49 = ((($48)) + 8|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = HEAP32[$pShmNode>>2]|0;
     HEAP32[$51>>2] = $50;
     $52 = HEAP32[$pShmNode>>2]|0;
     $53 = ((($52)) + 4|0);
     HEAP32[$53>>2] = (8);
     $54 = HEAP32[$pShmNode>>2]|0;
     $55 = ((($54)) + 4|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = ($56|0)==(0|0);
     if ($57) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $58 = HEAP32[$pInode>>2]|0;
     $59 = ((($58)) + 13|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60&255;
     $62 = ($61|0)==(0);
     if (!($62)) {
      break L5;
     }
     HEAP32[$openFlags>>2] = 66;
     $63 = HEAP32[$1>>2]|0;
     $64 = ((($63)) + 32|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = (_sqlite3_uri_boolean($65,18728,0)|0);
     $67 = ($66|0)!=(0);
     if ($67) {
      HEAP32[$openFlags>>2] = 0;
      $68 = HEAP32[$pShmNode>>2]|0;
      $69 = ((($68)) + 22|0);
      HEAP8[$69>>0] = 1;
     }
     $70 = HEAP32[$zShmFilename>>2]|0;
     $71 = HEAP32[$openFlags>>2]|0;
     $72 = ((($sStat)) + 12|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = $73 & 511;
     $75 = (_robust_open($70,$71,$74)|0);
     $76 = HEAP32[$pShmNode>>2]|0;
     $77 = ((($76)) + 12|0);
     HEAP32[$77>>2] = $75;
     $78 = HEAP32[$pShmNode>>2]|0;
     $79 = ((($78)) + 12|0);
     $80 = HEAP32[$79>>2]|0;
     $81 = ($80|0)<(0);
     if ($81) {
      $82 = (_sqlite3CantopenError(31584)|0);
      $83 = HEAP32[$zShmFilename>>2]|0;
      $84 = (_unixLogErrorAtLine($82,18322,$83,31584)|0);
      HEAP32[$rc>>2] = $84;
      break;
     }
     $85 = HEAP32[$pShmNode>>2]|0;
     $86 = ((($85)) + 12|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = ((($sStat)) + 20|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = ((($sStat)) + 24|0);
     $91 = HEAP32[$90>>2]|0;
     (_robustFchown($87,$89,$91)|0);
     HEAP32[$rc>>2] = 0;
     $92 = HEAP32[$1>>2]|0;
     $93 = (_unixShmSystemLock($92,1,128,1)|0);
     $94 = ($93|0)==(0);
     if ($94) {
      $95 = HEAP32[$pShmNode>>2]|0;
      $96 = ((($95)) + 12|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = (_robust_ftruncate($97,0,0)|0);
      $99 = ($98|0)!=(0);
      if ($99) {
       $100 = HEAP32[$zShmFilename>>2]|0;
       $101 = (_unixLogErrorAtLine(4618,18358,$100,31600)|0);
       HEAP32[$rc>>2] = $101;
      }
     }
     $102 = HEAP32[$rc>>2]|0;
     $103 = ($102|0)==(0);
     if ($103) {
      $104 = HEAP32[$1>>2]|0;
      $105 = (_unixShmSystemLock($104,0,128,1)|0);
      HEAP32[$rc>>2] = $105;
     }
     $106 = HEAP32[$rc>>2]|0;
     $107 = ($106|0)!=(0);
     if (!($107)) {
      break L5;
     }
    }
   } while(0);
   $125 = HEAP32[$1>>2]|0;
   _unixShmPurge($125);
   $126 = HEAP32[$p>>2]|0;
   _sqlite3_free($126);
   _unixLeaveMutex();
   $127 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $127;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $108 = HEAP32[$pShmNode>>2]|0;
 $109 = HEAP32[$p>>2]|0;
 HEAP32[$109>>2] = $108;
 $110 = HEAP32[$pShmNode>>2]|0;
 $111 = ((($110)) + 28|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = (($112) + 1)|0;
 HEAP32[$111>>2] = $113;
 $114 = HEAP32[$p>>2]|0;
 $115 = HEAP32[$1>>2]|0;
 $116 = ((($115)) + 36|0);
 HEAP32[$116>>2] = $114;
 _unixLeaveMutex();
 $117 = HEAP32[$pShmNode>>2]|0;
 $118 = ((($117)) + 32|0);
 $119 = HEAP32[$118>>2]|0;
 $120 = HEAP32[$p>>2]|0;
 $121 = ((($120)) + 4|0);
 HEAP32[$121>>2] = $119;
 $122 = HEAP32[$p>>2]|0;
 $123 = HEAP32[$pShmNode>>2]|0;
 $124 = ((($123)) + 32|0);
 HEAP32[$124>>2] = $122;
 HEAP32[$0>>2] = 0;
 $128 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($128|0);
}
function _robustFchown($fd,$uid,$gid) {
 $fd = $fd|0;
 $uid = $uid|0;
 $gid = $gid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $uid;
 HEAP32[$2>>2] = $gid;
 $3 = HEAP32[(1320)>>2]|0;
 $4 = (FUNCTION_TABLE_i[$3 & 255]()|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $11 = 0;
  STACKTOP = sp;return ($11|0);
 }
 $6 = HEAP32[(1308)>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (FUNCTION_TABLE_iiii[$6 & 255]($7,$8,$9)|0);
 $11 = $10;
 STACKTOP = sp;return ($11|0);
}
function _unixShmSystemLock($pFile,$lockType,$ofst,$n) {
 $pFile = $pFile|0;
 $lockType = $lockType|0;
 $ofst = $ofst|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $f = 0, $pShmNode = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pShmNode = sp + 24|0;
 $f = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $lockType;
 HEAP32[$2>>2] = $ofst;
 HEAP32[$3>>2] = $n;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pShmNode>>2] = $8;
 $9 = HEAP32[$pShmNode>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>=(0);
 if (!($12)) {
  $28 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 ;HEAP32[$f>>2]=0|0;HEAP32[$f+4>>2]=0|0;HEAP32[$f+8>>2]=0|0;HEAP32[$f+12>>2]=0|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = $13&65535;
 HEAP16[$f>>1] = $14;
 $15 = ((($f)) + 2|0);
 HEAP16[$15>>1] = 0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($f)) + 4|0);
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$3>>2]|0;
 $19 = ((($f)) + 8|0);
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[(1152)>>2]|0;
 $21 = HEAP32[$pShmNode>>2]|0;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$vararg_buffer>>2] = $f;
 $24 = (FUNCTION_TABLE_iiii[$20 & 255]($23,13,$vararg_buffer)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)!=(-1);
 $27 = $26 ? 0 : 5;
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _unixShmPurge($pFd) {
 $pFd = $pFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nShmPerMap = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $nShmPerMap = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pFd;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = (_unixShmRegionPerMap()|0);
 HEAP32[$nShmPerMap>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = ($13|0)<($17|0);
  $19 = HEAP32[$p>>2]|0;
  if (!($18)) {
   break;
  }
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)>=(0);
  if ($22) {
   $23 = HEAP32[(1344)>>2]|0;
   $24 = HEAP32[$i>>2]|0;
   $25 = HEAP32[$p>>2]|0;
   $26 = ((($25)) + 24|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<2)|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 16|0);
   $32 = HEAP32[$31>>2]|0;
   (FUNCTION_TABLE_iii[$23 & 255]($29,$32)|0);
  } else {
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = ((($34)) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = (($36) + ($33<<2)|0);
   $38 = HEAP32[$37>>2]|0;
   _sqlite3_free($38);
  }
  $39 = HEAP32[$nShmPerMap>>2]|0;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + ($39))|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = ((($19)) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3_free($43);
 $44 = HEAP32[$p>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)>=(0);
 if ($47) {
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  _robust_close($48,$51,31469);
  $52 = HEAP32[$p>>2]|0;
  $53 = ((($52)) + 12|0);
  HEAP32[$53>>2] = -1;
 }
 $54 = HEAP32[$p>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = ((($55)) + 20|0);
 HEAP32[$56>>2] = 0;
 $57 = HEAP32[$p>>2]|0;
 _sqlite3_free($57);
 STACKTOP = sp;return;
}
function _findReusableFd($zPath,$flags) {
 $zPath = $zPath|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInode = 0, $pUnused = 0, $pp = 0, $sStat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $pUnused = sp + 84|0;
 $sStat = sp + 8|0;
 $pInode = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $zPath;
 HEAP32[$1>>2] = $flags;
 HEAP32[$pUnused>>2] = 0;
 $2 = HEAP32[(1116)>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (FUNCTION_TABLE_iii[$2 & 255]($3,$sStat)|0);
 $5 = (0)==($4|0);
 if (!($5)) {
  $45 = HEAP32[$pUnused>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 _unixEnterMutex();
 $6 = HEAP32[1400>>2]|0;
 HEAP32[$pInode>>2] = $6;
 while(1) {
  $7 = HEAP32[$pInode>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$pInode>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$sStat>>2]|0;
   $12 = ($10|0)!=($11|0);
   if ($12) {
    $46 = 1;
   } else {
    $13 = HEAP32[$pInode>>2]|0;
    $14 = ((($13)) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = ((($sStat)) + 72|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ($15|0)!=($17|0);
    $46 = $18;
   }
  } else {
   $46 = 0;
  }
  $19 = HEAP32[$pInode>>2]|0;
  if (!($46)) {
   break;
  }
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pInode>>2] = $21;
 }
 $22 = ($19|0)!=(0|0);
 if ($22) {
  $23 = HEAP32[$pInode>>2]|0;
  $24 = ((($23)) + 28|0);
  HEAP32[$pp>>2] = $24;
  while(1) {
   $25 = HEAP32[$pp>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0|0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$pp>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ((($29)) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$1>>2]|0;
   $33 = ($31|0)!=($32|0);
   if (!($33)) {
    break;
   }
   $34 = HEAP32[$pp>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = ((($35)) + 8|0);
   HEAP32[$pp>>2] = $36;
  }
  $37 = HEAP32[$pp>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$pUnused>>2] = $38;
  $39 = HEAP32[$pUnused>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$pUnused>>2]|0;
   $42 = ((($41)) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$pp>>2]|0;
   HEAP32[$44>>2] = $43;
  }
 }
 _unixLeaveMutex();
 $45 = HEAP32[$pUnused>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _findCreateFileMode($zPath,$flags,$pMode,$pUid,$pGid) {
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pMode = $pMode|0;
 $pUid = $pUid|0;
 $pGid = $pGid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nDb = 0, $rc = 0, $sStat = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $3 = sp + 88|0;
 $4 = sp + 84|0;
 $rc = sp + 80|0;
 $zDb = sp + 104|0;
 $nDb = sp + 76|0;
 $sStat = sp;
 HEAP32[$0>>2] = $zPath;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $pMode;
 HEAP32[$3>>2] = $pUid;
 HEAP32[$4>>2] = $pGid;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = $8 & 526336;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $39 = HEAP32[$1>>2]|0;
  $40 = $39 & 8;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   $43 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
  $42 = HEAP32[$2>>2]|0;
  HEAP32[$42>>2] = 384;
  $43 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 $13 = (($12) - 1)|0;
 HEAP32[$nDb>>2] = $13;
 while(1) {
  $14 = HEAP32[$nDb>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + ($14)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = ($18|0)!=(45);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$nDb>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$nDb>>2] = $21;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$nDb>>2]|0;
 _memcpy(($zDb|0),($22|0),($23|0))|0;
 $24 = HEAP32[$nDb>>2]|0;
 $25 = (($zDb) + ($24)|0);
 HEAP8[$25>>0] = 0;
 $26 = HEAP32[(1116)>>2]|0;
 $27 = (FUNCTION_TABLE_iii[$26 & 255]($zDb,$sStat)|0);
 $28 = (0)==($27|0);
 if ($28) {
  $29 = ((($sStat)) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30 & 511;
  $32 = HEAP32[$2>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = ((($sStat)) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$3>>2]|0;
  HEAP32[$35>>2] = $34;
  $36 = ((($sStat)) + 24|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$38>>2] = $37;
  $43 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($43|0);
 } else {
  HEAP32[$rc>>2] = 1802;
  $43 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 return (0)|0;
}
function _fillInUnixFile($pVfs,$h,$pId,$zFilename,$ctrlFlags) {
 $pVfs = $pVfs|0;
 $h = $h|0;
 $pId = $pId|0;
 $zFilename = $zFilename|0;
 $ctrlFlags = $ctrlFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $nFilename = 0, $pLockingStyle = 0, $pNew = 0, $rc = 0, $vararg_buffer = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pLockingStyle = sp + 20|0;
 $pNew = sp + 16|0;
 $rc = sp + 12|0;
 $zLockFile = sp + 8|0;
 $nFilename = sp + 4|0;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $pId;
 HEAP32[$3>>2] = $zFilename;
 HEAP32[$4>>2] = $ctrlFlags;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$pNew>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$pNew>>2]|0;
 $8 = ((($7)) + 12|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$pNew>>2]|0;
 $11 = ((($10)) + 4|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$pNew>>2]|0;
 $14 = ((($13)) + 32|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$4>>2]|0;
 $16 = $15&255;
 $17 = $16&255;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 18|0);
 HEAP16[$19>>1] = $17;
 $20 = HEAP32[$4>>2]|0;
 $21 = $20 & 64;
 $22 = ($21|0)!=(0);
 $23 = HEAP32[$3>>2]|0;
 $24 = $22 ? $23 : 0;
 $25 = (_sqlite3_uri_boolean($24,18741,1)|0);
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP32[$pNew>>2]|0;
  $28 = ((($27)) + 18|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  $31 = $30 | 16;
  $32 = $31&65535;
  HEAP16[$28>>1] = $32;
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (_strcmp($35,17258)|0);
 $37 = ($36|0)==(0);
 if ($37) {
  $38 = HEAP32[$pNew>>2]|0;
  $39 = ((($38)) + 18|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40&65535;
  $42 = $41 | 1;
  $43 = $42&65535;
  HEAP16[$39>>1] = $43;
 }
 $44 = HEAP32[$4>>2]|0;
 $45 = $44 & 128;
 $46 = ($45|0)!=(0);
 if ($46) {
  HEAP32[$pLockingStyle>>2] = 1432;
 } else {
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 20|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$3>>2]|0;
  $52 = HEAP32[$pNew>>2]|0;
  $53 = (FUNCTION_TABLE_iii[$50 & 255]($51,$52)|0);
  HEAP32[$pLockingStyle>>2] = $53;
 }
 $54 = HEAP32[$pLockingStyle>>2]|0;
 $55 = ($54|0)==(988|0);
 if ($55) {
  _unixEnterMutex();
  $56 = HEAP32[$pNew>>2]|0;
  $57 = HEAP32[$pNew>>2]|0;
  $58 = ((($57)) + 8|0);
  $59 = (_findInodeInfo($56,$58)|0);
  HEAP32[$rc>>2] = $59;
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)!=(0);
  if ($61) {
   $62 = HEAP32[$pNew>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   _robust_close($62,$63,32585);
   HEAP32[$1>>2] = -1;
  }
  _unixLeaveMutex();
 } else {
  $64 = HEAP32[$pLockingStyle>>2]|0;
  $65 = ($64|0)==(1508|0);
  if ($65) {
   $66 = HEAP32[$3>>2]|0;
   $67 = (_strlen($66)|0);
   $68 = (($67) + 6)|0;
   HEAP32[$nFilename>>2] = $68;
   $69 = HEAP32[$nFilename>>2]|0;
   $70 = ($69|0)<(0);
   $71 = $70 << 31 >> 31;
   $72 = (_sqlite3_malloc64($69,$71)|0);
   HEAP32[$zLockFile>>2] = $72;
   $73 = HEAP32[$zLockFile>>2]|0;
   $74 = ($73|0)==(0|0);
   if ($74) {
    HEAP32[$rc>>2] = 7;
   } else {
    $75 = HEAP32[$nFilename>>2]|0;
    $76 = HEAP32[$zLockFile>>2]|0;
    $77 = HEAP32[$3>>2]|0;
    HEAP32[$vararg_buffer>>2] = $77;
    (_sqlite3_snprintf($75,$76,18746,$vararg_buffer)|0);
   }
   $78 = HEAP32[$zLockFile>>2]|0;
   $79 = HEAP32[$pNew>>2]|0;
   $80 = ((($79)) + 24|0);
   HEAP32[$80>>2] = $78;
  }
 }
 $81 = HEAP32[$pNew>>2]|0;
 _storeLastErrno($81,0);
 $82 = HEAP32[$rc>>2]|0;
 $83 = ($82|0)!=(0);
 if (!($83)) {
  $88 = HEAP32[$pLockingStyle>>2]|0;
  $89 = HEAP32[$pNew>>2]|0;
  HEAP32[$89>>2] = $88;
  $90 = HEAP32[$pNew>>2]|0;
  _verifyDbFile($90);
  $91 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $84 = HEAP32[$1>>2]|0;
 $85 = ($84|0)>=(0);
 if (!($85)) {
  $91 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $86 = HEAP32[$pNew>>2]|0;
 $87 = HEAP32[$1>>2]|0;
 _robust_close($86,$87,32670);
 $91 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _findInodeInfo($pFile,$ppInode) {
 $pFile = $pFile|0;
 $ppInode = $ppInode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $fileId = 0;
 var $pInode = 0, $rc = 0, $statbuf = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $rc = sp + 92|0;
 $fd = sp + 88|0;
 $fileId = sp + 80|0;
 $statbuf = sp + 4|0;
 $pInode = sp;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $ppInode;
 HEAP32[$pInode>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$fd>>2] = $5;
 $6 = HEAP32[(1128)>>2]|0;
 $7 = HEAP32[$fd>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$6 & 255]($7,$statbuf)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = (___errno_location()|0);
  $13 = HEAP32[$12>>2]|0;
  _storeLastErrno($11,$13);
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(75);
  if ($17) {
   HEAP32[$0>>2] = 22;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  } else {
   HEAP32[$0>>2] = 10;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 }
 ;HEAP32[$fileId>>2]=0|0;HEAP32[$fileId+4>>2]=0|0;
 $18 = HEAP32[$statbuf>>2]|0;
 HEAP32[$fileId>>2] = $18;
 $19 = ((($statbuf)) + 72|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($fileId)) + 4|0);
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[1400>>2]|0;
 HEAP32[$pInode>>2] = $22;
 while(1) {
  $23 = HEAP32[$pInode>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$pInode>>2]|0;
   $26 = (_memcmp($fileId,$25,8)|0);
   $27 = ($26|0)!=(0);
   $57 = $27;
  } else {
   $57 = 0;
  }
  $28 = HEAP32[$pInode>>2]|0;
  if (!($57)) {
   break;
  }
  $29 = ((($28)) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pInode>>2] = $30;
 }
 $31 = ($28|0)==(0|0);
 if ($31) {
  $32 = (_sqlite3_malloc64(40,0)|0);
  HEAP32[$pInode>>2] = $32;
  $33 = HEAP32[$pInode>>2]|0;
  $34 = ($33|0)==(0|0);
  if ($34) {
   HEAP32[$0>>2] = 7;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
  $35 = HEAP32[$pInode>>2]|0;
  dest=$35; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $36 = HEAP32[$pInode>>2]|0;
  ;HEAP32[$36>>2]=HEAP32[$fileId>>2]|0;HEAP32[$36+4>>2]=HEAP32[$fileId+4>>2]|0;
  $37 = HEAP32[$pInode>>2]|0;
  $38 = ((($37)) + 16|0);
  HEAP32[$38>>2] = 1;
  $39 = HEAP32[1400>>2]|0;
  $40 = HEAP32[$pInode>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pInode>>2]|0;
  $43 = ((($42)) + 36|0);
  HEAP32[$43>>2] = 0;
  $44 = HEAP32[1400>>2]|0;
  $45 = ($44|0)!=(0|0);
  if ($45) {
   $46 = HEAP32[$pInode>>2]|0;
   $47 = HEAP32[1400>>2]|0;
   $48 = ((($47)) + 36|0);
   HEAP32[$48>>2] = $46;
  }
  $49 = HEAP32[$pInode>>2]|0;
  HEAP32[1400>>2] = $49;
 } else {
  $50 = HEAP32[$pInode>>2]|0;
  $51 = ((($50)) + 16|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$51>>2] = $53;
 }
 $54 = HEAP32[$pInode>>2]|0;
 $55 = HEAP32[$2>>2]|0;
 HEAP32[$55>>2] = $54;
 HEAP32[$0>>2] = 0;
 $56 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _nolockClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_closeUnixFile($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _nolockLock($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _nolockUnlock($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _nolockCheckReservedLock($NotUsed,$pResOut) {
 $NotUsed = $NotUsed|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $pResOut;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 STACKTOP = sp;return 0;
}
function _dotlockClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $pFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$0>>2]|0;
 (_dotlockUnlock($2,0)|0);
 $3 = HEAP32[$pFile>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3_free($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = (_closeUnixFile($6)|0);
 STACKTOP = sp;return ($7|0);
}
function _dotlockLock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0;
 var $rc = 0, $tErrno = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFile = sp + 12|0;
 $zLockFile = sp + 8|0;
 $rc = sp + 4|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$zLockFile>>2] = $6;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$pFile>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)>(0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = $12&255;
  $14 = HEAP32[$pFile>>2]|0;
  $15 = ((($14)) + 16|0);
  HEAP8[$15>>0] = $13;
  $16 = HEAP32[$zLockFile>>2]|0;
  (_utimes(($16|0),(0|0))|0);
  HEAP32[$0>>2] = 0;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[(1284)>>2]|0;
 $18 = HEAP32[$zLockFile>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 255]($18,511)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)<(0);
 if (!($21)) {
  $33 = HEAP32[$2>>2]|0;
  $34 = $33&255;
  $35 = HEAP32[$pFile>>2]|0;
  $36 = ((($35)) + 16|0);
  HEAP8[$36>>0] = $34;
  $37 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $22 = (___errno_location()|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$tErrno>>2] = $23;
 $24 = HEAP32[$tErrno>>2]|0;
 $25 = (17)==($24|0);
 if ($25) {
  HEAP32[$rc>>2] = 5;
 } else {
  $26 = HEAP32[$tErrno>>2]|0;
  $27 = (_sqliteErrorFromPosixError($26,3850)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$rc>>2]|0;
  $29 = ($28|0)!=(5);
  if ($29) {
   $30 = HEAP32[$pFile>>2]|0;
   $31 = HEAP32[$tErrno>>2]|0;
   _storeLastErrno($30,$31);
  }
 }
 $32 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $32;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _dotlockUnlock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, $tErrno = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFile = sp + 12|0;
 $zLockFile = sp + 8|0;
 $rc = sp + 4|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$zLockFile>>2] = $6;
 $7 = HEAP32[$pFile>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($10|0)==($11|0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ($13|0)==(1);
 if ($14) {
  $15 = HEAP32[$pFile>>2]|0;
  $16 = ((($15)) + 16|0);
  HEAP8[$16>>0] = 1;
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $17 = HEAP32[(1296)>>2]|0;
 $18 = HEAP32[$zLockFile>>2]|0;
 $19 = (FUNCTION_TABLE_ii[$17 & 255]($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)<(0);
 if (!($21)) {
  $29 = HEAP32[$pFile>>2]|0;
  $30 = ((($29)) + 16|0);
  HEAP8[$30>>0] = 0;
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $22 = (___errno_location()|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$tErrno>>2] = $23;
 $24 = HEAP32[$tErrno>>2]|0;
 $25 = ($24|0)==(2);
 if ($25) {
  HEAP32[$rc>>2] = 0;
 } else {
  HEAP32[$rc>>2] = 2058;
  $26 = HEAP32[$pFile>>2]|0;
  $27 = HEAP32[$tErrno>>2]|0;
  _storeLastErrno($26,$27);
 }
 $28 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $28;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _dotlockCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, $reserved = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $reserved = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$reserved>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $2;
 $3 = HEAP32[(1092)>>2]|0;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$3 & 255]($6,0)|0);
 $8 = ($7|0)==(0);
 $9 = $8&1;
 HEAP32[$reserved>>2] = $9;
 $10 = HEAP32[$reserved>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _mkFullPathname($zPath,$zOut,$nOut) {
 $zPath = $zPath|0;
 $zOut = $zOut|0;
 $nOut = $nOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iOff = 0, $nPath = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $nPath = sp + 8|0;
 $iOff = sp + 4|0;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $zOut;
 HEAP32[$3>>2] = $nOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 HEAP32[$nPath>>2] = $5;
 HEAP32[$iOff>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP8[$6>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)!=(47);
 do {
  if ($9) {
   $10 = HEAP32[(1104)>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   $12 = HEAP32[$3>>2]|0;
   $13 = (($12) - 2)|0;
   $14 = (FUNCTION_TABLE_iii[$10 & 255]($11,$13)|0);
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $19 = HEAP32[$2>>2]|0;
    $20 = (_sqlite3Strlen30($19)|0);
    HEAP32[$iOff>>2] = $20;
    $21 = HEAP32[$iOff>>2]|0;
    $22 = (($21) + 1)|0;
    HEAP32[$iOff>>2] = $22;
    $23 = HEAP32[$2>>2]|0;
    $24 = (($23) + ($21)|0);
    HEAP8[$24>>0] = 47;
    break;
   }
   $16 = (_sqlite3CantopenError(33245)|0);
   $17 = HEAP32[$1>>2]|0;
   $18 = (_unixLogErrorAtLine($16,18340,$17,33245)|0);
   HEAP32[$0>>2] = $18;
   $42 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($42|0);
  }
 } while(0);
 $25 = HEAP32[$iOff>>2]|0;
 $26 = HEAP32[$nPath>>2]|0;
 $27 = (($25) + ($26))|0;
 $28 = (($27) + 1)|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = ($28|0)>($29|0);
 if ($30) {
  $31 = HEAP32[$iOff>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (($32) + ($31)|0);
  HEAP8[$33>>0] = 0;
  $34 = (_sqlite3CantopenError(33254)|0);
  HEAP32[$0>>2] = $34;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 } else {
  $35 = HEAP32[$3>>2]|0;
  $36 = HEAP32[$iOff>>2]|0;
  $37 = (($35) - ($36))|0;
  $38 = HEAP32[$iOff>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = (($39) + ($38)|0);
  $41 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $41;
  (_sqlite3_snprintf($37,$40,17521,$vararg_buffer)|0);
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _nolockIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (1432|0);
}
function _dotlockIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (1508|0);
}
function _sqlite3PcachePagecount($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[(136)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (FUNCTION_TABLE_ii[$1 & 255]($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3VMPrintf($db,$zFormat,$ap) {
 $db = $db|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $z = 0, $zBase = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $z = sp + 28|0;
 $zBase = sp + 44|0;
 $acc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 88|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3StrAccumInit($acc,$3,$zBase,70,$6);
 $7 = ((($acc)) + 25|0);
 HEAP8[$7>>0] = 1;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($acc,$8,$9);
 $10 = (_sqlite3StrAccumFinish($acc)|0);
 HEAP32[$z>>2] = $10;
 $11 = ((($acc)) + 24|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(1);
 if (!($14)) {
  $16 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $15 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($15);
 $16 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3ValueSetNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($1);
 STACKTOP = sp;return;
}
function _sqlite3OpenTempDatabase($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pBt = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $db = sp + 12|0;
 $rc = sp + 8|0;
 $pBt = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 453|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if (!($14)) {
   $15 = HEAP32[$db>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$db>>2]|0;
   $18 = (_sqlite3BtreeOpen($16,0,$17,$pBt,0,542)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($21,18787,$vararg_buffer);
    $22 = HEAP32[$rc>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 12|0);
    HEAP32[$24>>2] = $22;
    HEAP32[$0>>2] = 1;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
   $25 = HEAP32[$pBt>>2]|0;
   $26 = HEAP32[$db>>2]|0;
   $27 = ((($26)) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ((($28)) + 16|0);
   $30 = ((($29)) + 4|0);
   HEAP32[$30>>2] = $25;
   $31 = HEAP32[$pBt>>2]|0;
   $32 = HEAP32[$db>>2]|0;
   $33 = ((($32)) + 72|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3BtreeSetPageSize($31,$34,-1,0)|0);
   $36 = (7)==($35|0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    _sqlite3OomFault($37);
    HEAP32[$0>>2] = 1;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3BtreeOpen($pVfs,$zFilename,$db,$ppBtree,$flags,$vfsFlags) {
 $pVfs = $pVfs|0;
 $zFilename = $zFilename|0;
 $db = $db|0;
 $ppBtree = $ppBtree|0;
 $flags = $flags|0;
 $vfsFlags = $vfsFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDb = 0, $isMemdb = 0;
 var $isTempDb = 0, $mutexOpen = 0, $nFilename = 0, $nFullPathname = 0, $nReserve = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pBt = 0, $pExisting = 0, $pSib = 0, $rc = 0, $zDbHeader = 0, $zFullPathname = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $pBt = sp + 48|0;
 $p = sp + 44|0;
 $mutexOpen = sp + 40|0;
 $rc = sp + 36|0;
 $nReserve = sp + 180|0;
 $zDbHeader = sp + 80|0;
 $isTempDb = sp + 32|0;
 $isMemdb = sp + 28|0;
 $nFilename = sp + 24|0;
 $nFullPathname = sp + 20|0;
 $zFullPathname = sp + 16|0;
 $iDb = sp + 12|0;
 $pExisting = sp + 8|0;
 $i = sp + 4|0;
 $pSib = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zFilename;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $ppBtree;
 HEAP32[$5>>2] = $flags;
 HEAP32[$6>>2] = $vfsFlags;
 HEAP32[$pBt>>2] = 0;
 HEAP32[$mutexOpen>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $14 = 1;
 } else {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  $14 = $12;
 }
 $13 = $14&1;
 HEAP32[$isTempDb>>2] = $13;
 $15 = HEAP32[$2>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$2>>2]|0;
  $18 = (_strcmp($17,18857)|0);
  $19 = ($18|0)==(0);
  if ($19) {
   $29 = 1;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 do {
  if ((label|0) == 5) {
   $20 = HEAP32[$isTempDb>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$3>>2]|0;
    $23 = (_sqlite3TempInMemory($22)|0);
    $24 = ($23|0)!=(0);
    if ($24) {
     $29 = 1;
     break;
    }
   }
   $25 = HEAP32[$6>>2]|0;
   $26 = $25 & 128;
   $27 = ($26|0)!=(0);
   $29 = $27;
  }
 } while(0);
 $28 = $29&1;
 HEAP32[$isMemdb>>2] = $28;
 $30 = HEAP32[$isMemdb>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$5>>2]|0;
  $33 = $32 | 2;
  HEAP32[$5>>2] = $33;
 }
 $34 = HEAP32[$6>>2]|0;
 $35 = $34 & 256;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$isMemdb>>2]|0;
  $38 = ($37|0)!=(0);
  $39 = HEAP32[$isTempDb>>2]|0;
  $40 = ($39|0)!=(0);
  $or$cond = $38 | $40;
  if ($or$cond) {
   $41 = HEAP32[$6>>2]|0;
   $42 = $41 & -257;
   $43 = $42 | 512;
   HEAP32[$6>>2] = $43;
  }
 }
 $44 = (_sqlite3MallocZero(48,0)|0);
 HEAP32[$p>>2] = $44;
 $45 = HEAP32[$p>>2]|0;
 $46 = ($45|0)!=(0|0);
 if (!($46)) {
  HEAP32[$0>>2] = 7;
  $395 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($395|0);
 }
 $47 = HEAP32[$p>>2]|0;
 $48 = ((($47)) + 8|0);
 HEAP8[$48>>0] = 0;
 $49 = HEAP32[$3>>2]|0;
 $50 = HEAP32[$p>>2]|0;
 HEAP32[$50>>2] = $49;
 $51 = HEAP32[$p>>2]|0;
 $52 = HEAP32[$p>>2]|0;
 $53 = ((($52)) + 32|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$p>>2]|0;
 $55 = ((($54)) + 32|0);
 $56 = ((($55)) + 4|0);
 HEAP32[$56>>2] = 1;
 $57 = HEAP32[$isTempDb>>2]|0;
 $58 = ($57|0)==(0);
 do {
  if ($58) {
   $59 = HEAP32[$isMemdb>>2]|0;
   $60 = ($59|0)==(0);
   if (!($60)) {
    $61 = HEAP32[$6>>2]|0;
    $62 = $61 & 64;
    $63 = ($62|0)!=(0);
    if (!($63)) {
     break;
    }
   }
   $64 = HEAP32[$6>>2]|0;
   $65 = $64 & 131072;
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = HEAP32[$2>>2]|0;
    $68 = (_sqlite3Strlen30($67)|0);
    $69 = (($68) + 1)|0;
    HEAP32[$nFilename>>2] = $69;
    $70 = HEAP32[$1>>2]|0;
    $71 = ((($70)) + 8|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (($72) + 1)|0;
    HEAP32[$nFullPathname>>2] = $73;
    $74 = HEAP32[$nFullPathname>>2]|0;
    $75 = HEAP32[$nFilename>>2]|0;
    $76 = ($74|0)>($75|0);
    $77 = HEAP32[$nFullPathname>>2]|0;
    $78 = HEAP32[$nFilename>>2]|0;
    $79 = $76 ? $77 : $78;
    $80 = ($79|0)<(0);
    $81 = $80 << 31 >> 31;
    $82 = (_sqlite3Malloc($79,$81)|0);
    HEAP32[$zFullPathname>>2] = $82;
    $83 = HEAP32[$p>>2]|0;
    $84 = ((($83)) + 9|0);
    HEAP8[$84>>0] = 1;
    $85 = HEAP32[$zFullPathname>>2]|0;
    $86 = ($85|0)!=(0|0);
    if (!($86)) {
     $87 = HEAP32[$p>>2]|0;
     _sqlite3_free($87);
     HEAP32[$0>>2] = 7;
     $395 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($395|0);
    }
    $88 = HEAP32[$isMemdb>>2]|0;
    $89 = ($88|0)!=(0);
    if ($89) {
     $90 = HEAP32[$zFullPathname>>2]|0;
     $91 = HEAP32[$2>>2]|0;
     $92 = HEAP32[$nFilename>>2]|0;
     _memcpy(($90|0),($91|0),($92|0))|0;
    } else {
     $93 = HEAP32[$1>>2]|0;
     $94 = HEAP32[$2>>2]|0;
     $95 = HEAP32[$nFullPathname>>2]|0;
     $96 = HEAP32[$zFullPathname>>2]|0;
     $97 = (_sqlite3OsFullPathname($93,$94,$95,$96)|0);
     HEAP32[$rc>>2] = $97;
     $98 = HEAP32[$rc>>2]|0;
     $99 = ($98|0)!=(0);
     if ($99) {
      $100 = HEAP32[$zFullPathname>>2]|0;
      _sqlite3_free($100);
      $101 = HEAP32[$p>>2]|0;
      _sqlite3_free($101);
      $102 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $102;
      $395 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($395|0);
     }
    }
    $103 = HEAP32[1684>>2]|0;
    HEAP32[$pBt>>2] = $103;
    while(1) {
     $104 = HEAP32[$pBt>>2]|0;
     $105 = ($104|0)!=(0|0);
     if (!($105)) {
      break;
     }
     $106 = HEAP32[$zFullPathname>>2]|0;
     $107 = HEAP32[$pBt>>2]|0;
     $108 = HEAP32[$107>>2]|0;
     $109 = (_sqlite3PagerFilename($108,0)|0);
     $110 = (_strcmp($106,$109)|0);
     $111 = (0)==($110|0);
     if ($111) {
      $112 = HEAP32[$pBt>>2]|0;
      $113 = HEAP32[$112>>2]|0;
      $114 = (_sqlite3PagerVfs($113)|0);
      $115 = HEAP32[$1>>2]|0;
      $116 = ($114|0)==($115|0);
      if ($116) {
       label = 29;
       break;
      }
     }
     $148 = HEAP32[$pBt>>2]|0;
     $149 = ((($148)) + 68|0);
     $150 = HEAP32[$149>>2]|0;
     HEAP32[$pBt>>2] = $150;
    }
    do {
     if ((label|0) == 29) {
      $117 = HEAP32[$3>>2]|0;
      $118 = ((($117)) + 20|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = (($119) - 1)|0;
      HEAP32[$iDb>>2] = $120;
      while(1) {
       $121 = HEAP32[$iDb>>2]|0;
       $122 = ($121|0)>=(0);
       if (!($122)) {
        label = 35;
        break;
       }
       $123 = HEAP32[$iDb>>2]|0;
       $124 = HEAP32[$3>>2]|0;
       $125 = ((($124)) + 16|0);
       $126 = HEAP32[$125>>2]|0;
       $127 = (($126) + ($123<<4)|0);
       $128 = ((($127)) + 4|0);
       $129 = HEAP32[$128>>2]|0;
       HEAP32[$pExisting>>2] = $129;
       $130 = HEAP32[$pExisting>>2]|0;
       $131 = ($130|0)!=(0|0);
       if ($131) {
        $132 = HEAP32[$pExisting>>2]|0;
        $133 = ((($132)) + 4|0);
        $134 = HEAP32[$133>>2]|0;
        $135 = HEAP32[$pBt>>2]|0;
        $136 = ($134|0)==($135|0);
        if ($136) {
         break;
        }
       }
       $139 = HEAP32[$iDb>>2]|0;
       $140 = (($139) + -1)|0;
       HEAP32[$iDb>>2] = $140;
      }
      if ((label|0) == 35) {
       $141 = HEAP32[$pBt>>2]|0;
       $142 = HEAP32[$p>>2]|0;
       $143 = ((($142)) + 4|0);
       HEAP32[$143>>2] = $141;
       $144 = HEAP32[$pBt>>2]|0;
       $145 = ((($144)) + 64|0);
       $146 = HEAP32[$145>>2]|0;
       $147 = (($146) + 1)|0;
       HEAP32[$145>>2] = $147;
       break;
      }
      $137 = HEAP32[$zFullPathname>>2]|0;
      _sqlite3_free($137);
      $138 = HEAP32[$p>>2]|0;
      _sqlite3_free($138);
      HEAP32[$0>>2] = 19;
      $395 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($395|0);
     }
    } while(0);
    $151 = HEAP32[$zFullPathname>>2]|0;
    _sqlite3_free($151);
   }
  }
 } while(0);
 $152 = HEAP32[$pBt>>2]|0;
 $153 = ($152|0)==(0|0);
 do {
  if ($153) {
   $154 = (_sqlite3MallocZero(84,0)|0);
   HEAP32[$pBt>>2] = $154;
   $155 = HEAP32[$pBt>>2]|0;
   $156 = ($155|0)==(0|0);
   if ($156) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $157 = HEAP32[$1>>2]|0;
   $158 = HEAP32[$pBt>>2]|0;
   $159 = HEAP32[$2>>2]|0;
   $160 = HEAP32[$5>>2]|0;
   $161 = HEAP32[$6>>2]|0;
   $162 = (_sqlite3PagerOpen($157,$158,$159,88,$160,$161,143)|0);
   HEAP32[$rc>>2] = $162;
   $163 = HEAP32[$rc>>2]|0;
   $164 = ($163|0)==(0);
   if ($164) {
    $165 = HEAP32[$pBt>>2]|0;
    $166 = HEAP32[$165>>2]|0;
    $167 = HEAP32[$3>>2]|0;
    $168 = ((($167)) + 40|0);
    $169 = $168;
    $170 = $169;
    $171 = HEAP32[$170>>2]|0;
    $172 = (($169) + 4)|0;
    $173 = $172;
    $174 = HEAP32[$173>>2]|0;
    _sqlite3PagerSetMmapLimit($166,$171,$174);
    $175 = HEAP32[$pBt>>2]|0;
    $176 = HEAP32[$175>>2]|0;
    $177 = (_sqlite3PagerReadFileheader($176,100,$zDbHeader)|0);
    HEAP32[$rc>>2] = $177;
   }
   $178 = HEAP32[$rc>>2]|0;
   $179 = ($178|0)!=(0);
   if (!($179)) {
    $180 = HEAP32[$5>>2]|0;
    $181 = $180&255;
    $182 = HEAP32[$pBt>>2]|0;
    $183 = ((($182)) + 16|0);
    HEAP8[$183>>0] = $181;
    $184 = HEAP32[$3>>2]|0;
    $185 = HEAP32[$pBt>>2]|0;
    $186 = ((($185)) + 4|0);
    HEAP32[$186>>2] = $184;
    $187 = HEAP32[$pBt>>2]|0;
    $188 = HEAP32[$187>>2]|0;
    $189 = HEAP32[$pBt>>2]|0;
    _sqlite3PagerSetBusyhandler($188,171,$189);
    $190 = HEAP32[$pBt>>2]|0;
    $191 = HEAP32[$p>>2]|0;
    $192 = ((($191)) + 4|0);
    HEAP32[$192>>2] = $190;
    $193 = HEAP32[$pBt>>2]|0;
    $194 = ((($193)) + 8|0);
    HEAP32[$194>>2] = 0;
    $195 = HEAP32[$pBt>>2]|0;
    $196 = ((($195)) + 12|0);
    HEAP32[$196>>2] = 0;
    $197 = HEAP32[$pBt>>2]|0;
    $198 = HEAP32[$197>>2]|0;
    $199 = (_sqlite3PagerIsreadonly($198)|0);
    $200 = ($199<<24>>24)!=(0);
    if ($200) {
     $201 = HEAP32[$pBt>>2]|0;
     $202 = ((($201)) + 22|0);
     $203 = HEAP16[$202>>1]|0;
     $204 = $203&65535;
     $205 = $204 | 1;
     $206 = $205&65535;
     HEAP16[$202>>1] = $206;
    }
    $207 = ((($zDbHeader)) + 16|0);
    $208 = HEAP8[$207>>0]|0;
    $209 = $208&255;
    $210 = $209 << 8;
    $211 = ((($zDbHeader)) + 17|0);
    $212 = HEAP8[$211>>0]|0;
    $213 = $212&255;
    $214 = $213 << 16;
    $215 = $210 | $214;
    $216 = HEAP32[$pBt>>2]|0;
    $217 = ((($216)) + 32|0);
    HEAP32[$217>>2] = $215;
    $218 = HEAP32[$pBt>>2]|0;
    $219 = ((($218)) + 32|0);
    $220 = HEAP32[$219>>2]|0;
    $221 = ($220>>>0)<(512);
    if ($221) {
     label = 49;
    } else {
     $222 = HEAP32[$pBt>>2]|0;
     $223 = ((($222)) + 32|0);
     $224 = HEAP32[$223>>2]|0;
     $225 = ($224>>>0)>(65536);
     if ($225) {
      label = 49;
     } else {
      $226 = HEAP32[$pBt>>2]|0;
      $227 = ((($226)) + 32|0);
      $228 = HEAP32[$227>>2]|0;
      $229 = (($228) - 1)|0;
      $230 = HEAP32[$pBt>>2]|0;
      $231 = ((($230)) + 32|0);
      $232 = HEAP32[$231>>2]|0;
      $233 = $229 & $232;
      $234 = ($233|0)!=(0);
      if ($234) {
       label = 49;
      } else {
       $245 = ((($zDbHeader)) + 20|0);
       $246 = HEAP8[$245>>0]|0;
       HEAP8[$nReserve>>0] = $246;
       $247 = HEAP32[$pBt>>2]|0;
       $248 = ((($247)) + 22|0);
       $249 = HEAP16[$248>>1]|0;
       $250 = $249&65535;
       $251 = $250 | 2;
       $252 = $251&65535;
       HEAP16[$248>>1] = $252;
       $253 = ((($zDbHeader)) + 52|0);
       $254 = (_sqlite3Get4byte($253)|0);
       $255 = ($254|0)!=(0);
       $256 = $255 ? 1 : 0;
       $257 = $256&255;
       $258 = HEAP32[$pBt>>2]|0;
       $259 = ((($258)) + 17|0);
       HEAP8[$259>>0] = $257;
       $260 = ((($zDbHeader)) + 64|0);
       $261 = (_sqlite3Get4byte($260)|0);
       $262 = ($261|0)!=(0);
       $263 = $262 ? 1 : 0;
       $264 = $263&255;
       $265 = HEAP32[$pBt>>2]|0;
       $266 = ((($265)) + 18|0);
       HEAP8[$266>>0] = $264;
      }
     }
    }
    if ((label|0) == 49) {
     $235 = HEAP32[$pBt>>2]|0;
     $236 = ((($235)) + 32|0);
     HEAP32[$236>>2] = 0;
     $237 = HEAP32[$2>>2]|0;
     $238 = ($237|0)==(0|0);
     $239 = HEAP32[$isMemdb>>2]|0;
     $240 = ($239|0)!=(0);
     $or$cond3 = $238 | $240;
     if (!($or$cond3)) {
      $241 = HEAP32[$pBt>>2]|0;
      $242 = ((($241)) + 17|0);
      HEAP8[$242>>0] = 0;
      $243 = HEAP32[$pBt>>2]|0;
      $244 = ((($243)) + 18|0);
      HEAP8[$244>>0] = 0;
     }
     HEAP8[$nReserve>>0] = 0;
    }
    $267 = HEAP32[$pBt>>2]|0;
    $268 = HEAP32[$267>>2]|0;
    $269 = HEAP32[$pBt>>2]|0;
    $270 = ((($269)) + 32|0);
    $271 = HEAP8[$nReserve>>0]|0;
    $272 = $271&255;
    $273 = (_sqlite3PagerSetPagesize($268,$270,$272)|0);
    HEAP32[$rc>>2] = $273;
    $274 = HEAP32[$rc>>2]|0;
    $275 = ($274|0)!=(0);
    if (!($275)) {
     $276 = HEAP32[$pBt>>2]|0;
     $277 = ((($276)) + 32|0);
     $278 = HEAP32[$277>>2]|0;
     $279 = HEAP8[$nReserve>>0]|0;
     $280 = $279&255;
     $281 = (($278) - ($280))|0;
     $282 = HEAP32[$pBt>>2]|0;
     $283 = ((($282)) + 36|0);
     HEAP32[$283>>2] = $281;
     $284 = HEAP32[$p>>2]|0;
     $285 = ((($284)) + 9|0);
     $286 = HEAP8[$285>>0]|0;
     $287 = ($286<<24>>24)!=(0);
     if ($287) {
      $288 = HEAP32[$pBt>>2]|0;
      $289 = ((($288)) + 64|0);
      HEAP32[$289>>2] = 1;
      $290 = HEAP32[1684>>2]|0;
      $291 = HEAP32[$pBt>>2]|0;
      $292 = ((($291)) + 68|0);
      HEAP32[$292>>2] = $290;
      $293 = HEAP32[$pBt>>2]|0;
      HEAP32[1684>>2] = $293;
      label = 56;
     } else {
      label = 56;
     }
    }
   }
  } else {
   label = 56;
  }
 } while(0);
 if ((label|0) == 56) {
  $294 = HEAP32[$p>>2]|0;
  $295 = ((($294)) + 9|0);
  $296 = HEAP8[$295>>0]|0;
  $297 = ($296<<24>>24)!=(0);
  L85: do {
   if ($297) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $298 = HEAP32[$i>>2]|0;
     $299 = HEAP32[$3>>2]|0;
     $300 = ((($299)) + 20|0);
     $301 = HEAP32[$300>>2]|0;
     $302 = ($298|0)<($301|0);
     if (!($302)) {
      break L85;
     }
     $303 = HEAP32[$i>>2]|0;
     $304 = HEAP32[$3>>2]|0;
     $305 = ((($304)) + 16|0);
     $306 = HEAP32[$305>>2]|0;
     $307 = (($306) + ($303<<4)|0);
     $308 = ((($307)) + 4|0);
     $309 = HEAP32[$308>>2]|0;
     HEAP32[$pSib>>2] = $309;
     $310 = ($309|0)!=(0|0);
     if ($310) {
      $311 = HEAP32[$pSib>>2]|0;
      $312 = ((($311)) + 9|0);
      $313 = HEAP8[$312>>0]|0;
      $314 = $313&255;
      $315 = ($314|0)!=(0);
      if ($315) {
       break;
      }
     }
     $371 = HEAP32[$i>>2]|0;
     $372 = (($371) + 1)|0;
     HEAP32[$i>>2] = $372;
    }
    while(1) {
     $316 = HEAP32[$pSib>>2]|0;
     $317 = ((($316)) + 28|0);
     $318 = HEAP32[$317>>2]|0;
     $319 = ($318|0)!=(0|0);
     if (!($319)) {
      break;
     }
     $320 = HEAP32[$pSib>>2]|0;
     $321 = ((($320)) + 28|0);
     $322 = HEAP32[$321>>2]|0;
     HEAP32[$pSib>>2] = $322;
    }
    $323 = HEAP32[$p>>2]|0;
    $324 = ((($323)) + 4|0);
    $325 = HEAP32[$324>>2]|0;
    $326 = HEAP32[$pSib>>2]|0;
    $327 = ((($326)) + 4|0);
    $328 = HEAP32[$327>>2]|0;
    $329 = ($325>>>0)<($328>>>0);
    if ($329) {
     $330 = HEAP32[$pSib>>2]|0;
     $331 = HEAP32[$p>>2]|0;
     $332 = ((($331)) + 24|0);
     HEAP32[$332>>2] = $330;
     $333 = HEAP32[$p>>2]|0;
     $334 = ((($333)) + 28|0);
     HEAP32[$334>>2] = 0;
     $335 = HEAP32[$p>>2]|0;
     $336 = HEAP32[$pSib>>2]|0;
     $337 = ((($336)) + 28|0);
     HEAP32[$337>>2] = $335;
     break;
    }
    while(1) {
     $338 = HEAP32[$pSib>>2]|0;
     $339 = ((($338)) + 24|0);
     $340 = HEAP32[$339>>2]|0;
     $341 = ($340|0)!=(0|0);
     if ($341) {
      $342 = HEAP32[$pSib>>2]|0;
      $343 = ((($342)) + 24|0);
      $344 = HEAP32[$343>>2]|0;
      $345 = ((($344)) + 4|0);
      $346 = HEAP32[$345>>2]|0;
      $347 = HEAP32[$p>>2]|0;
      $348 = ((($347)) + 4|0);
      $349 = HEAP32[$348>>2]|0;
      $350 = ($346>>>0)<($349>>>0);
      $396 = $350;
     } else {
      $396 = 0;
     }
     $351 = HEAP32[$pSib>>2]|0;
     $352 = ((($351)) + 24|0);
     $353 = HEAP32[$352>>2]|0;
     if (!($396)) {
      break;
     }
     HEAP32[$pSib>>2] = $353;
    }
    $354 = HEAP32[$p>>2]|0;
    $355 = ((($354)) + 24|0);
    HEAP32[$355>>2] = $353;
    $356 = HEAP32[$pSib>>2]|0;
    $357 = HEAP32[$p>>2]|0;
    $358 = ((($357)) + 28|0);
    HEAP32[$358>>2] = $356;
    $359 = HEAP32[$p>>2]|0;
    $360 = ((($359)) + 24|0);
    $361 = HEAP32[$360>>2]|0;
    $362 = ($361|0)!=(0|0);
    if ($362) {
     $363 = HEAP32[$p>>2]|0;
     $364 = HEAP32[$p>>2]|0;
     $365 = ((($364)) + 24|0);
     $366 = HEAP32[$365>>2]|0;
     $367 = ((($366)) + 28|0);
     HEAP32[$367>>2] = $363;
    }
    $368 = HEAP32[$p>>2]|0;
    $369 = HEAP32[$pSib>>2]|0;
    $370 = ((($369)) + 24|0);
    HEAP32[$370>>2] = $368;
   }
  } while(0);
  $373 = HEAP32[$p>>2]|0;
  $374 = HEAP32[$4>>2]|0;
  HEAP32[$374>>2] = $373;
 }
 $375 = HEAP32[$rc>>2]|0;
 $376 = ($375|0)!=(0);
 if ($376) {
  $377 = HEAP32[$pBt>>2]|0;
  $378 = ($377|0)!=(0|0);
  if ($378) {
   $379 = HEAP32[$pBt>>2]|0;
   $380 = HEAP32[$379>>2]|0;
   $381 = ($380|0)!=(0|0);
   if ($381) {
    $382 = HEAP32[$pBt>>2]|0;
    $383 = HEAP32[$382>>2]|0;
    (_sqlite3PagerClose($383)|0);
   }
  }
  $384 = HEAP32[$pBt>>2]|0;
  _sqlite3_free($384);
  $385 = HEAP32[$p>>2]|0;
  _sqlite3_free($385);
  $386 = HEAP32[$4>>2]|0;
  HEAP32[$386>>2] = 0;
 } else {
  $387 = HEAP32[$p>>2]|0;
  $388 = (_sqlite3BtreeSchema($387,0,0)|0);
  $389 = ($388|0)==(0|0);
  if ($389) {
   $390 = HEAP32[$p>>2]|0;
   $391 = ((($390)) + 4|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = HEAP32[$392>>2]|0;
   _sqlite3PagerSetCachesize($393,2000);
  }
 }
 $394 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $394;
 $395 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($395|0);
}
function _sqlite3TempInMemory($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3OsFullPathname($pVfs,$zPath,$nPathOut,$zPathOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $nPathOut = $nPathOut|0;
 $zPathOut = $zPathOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $nPathOut;
 HEAP32[$3>>2] = $zPathOut;
 $4 = HEAP32[$3>>2]|0;
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iiiii[$7 & 255]($8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _sqlite3PagerFilename($pPager,$nullIfMemDb) {
 $pPager = $pPager|0;
 $nullIfMemDb = $nullIfMemDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nullIfMemDb;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 16|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)!=(0);
  if ($8) {
   $12 = 17532;
   STACKTOP = sp;return ($12|0);
  }
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 176|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3PagerOpen($pVfs,$ppPager,$zFilename,$nExtra,$flags,$vfsFlags,$xReinit) {
 $pVfs = $pVfs|0;
 $ppPager = $ppPager|0;
 $zFilename = $zFilename|0;
 $nExtra = $nExtra|0;
 $flags = $flags|0;
 $vfsFlags = $vfsFlags|0;
 $xReinit = $xReinit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $fout = 0, $iDc = 0, $journalFileSize = 0, $memDb = 0, $nPathname = 0, $nUri = 0, $pPager = 0, $pPtr = 0, $pcacheSize = 0, $rc = 0, $readOnly = 0, $szPageDflt = 0;
 var $tempFile = 0, $useJournal = 0, $z = 0, $zPathname = 0, $zUri = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $7 = sp + 68|0;
 $pPtr = sp + 64|0;
 $pPager = sp + 60|0;
 $rc = sp + 56|0;
 $tempFile = sp + 52|0;
 $memDb = sp + 48|0;
 $readOnly = sp + 44|0;
 $journalFileSize = sp + 40|0;
 $zPathname = sp + 36|0;
 $nPathname = sp + 32|0;
 $useJournal = sp + 28|0;
 $pcacheSize = sp + 24|0;
 $szPageDflt = sp + 20|0;
 $zUri = sp + 16|0;
 $nUri = sp + 12|0;
 $z = sp + 8|0;
 $fout = sp + 4|0;
 $iDc = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $ppPager;
 HEAP32[$3>>2] = $zFilename;
 HEAP32[$4>>2] = $nExtra;
 HEAP32[$5>>2] = $flags;
 HEAP32[$6>>2] = $vfsFlags;
 HEAP32[$7>>2] = $xReinit;
 HEAP32[$pPager>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$tempFile>>2] = 0;
 HEAP32[$memDb>>2] = 0;
 HEAP32[$readOnly>>2] = 0;
 HEAP32[$zPathname>>2] = 0;
 HEAP32[$nPathname>>2] = 0;
 $8 = HEAP32[$5>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$useJournal>>2] = $11;
 $12 = (_sqlite3PcacheSize()|0);
 HEAP32[$pcacheSize>>2] = $12;
 HEAP32[$szPageDflt>>2] = 1024;
 HEAP32[$zUri>>2] = 0;
 HEAP32[$nUri>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3MemJournalSize()|0);
 $17 = ($15|0)>($16|0);
 if ($17) {
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + 7)|0;
  $22 = $21 & -8;
  HEAP32[$journalFileSize>>2] = $22;
 } else {
  $23 = (_sqlite3MemJournalSize()|0);
  $24 = (($23) + 7)|0;
  $25 = $24 & -8;
  HEAP32[$journalFileSize>>2] = $25;
 }
 $26 = HEAP32[$2>>2]|0;
 HEAP32[$26>>2] = 0;
 $27 = HEAP32[$5>>2]|0;
 $28 = $27 & 2;
 $29 = ($28|0)!=(0);
 do {
  if ($29) {
   HEAP32[$memDb>>2] = 1;
   $30 = HEAP32[$3>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$3>>2]|0;
    $33 = HEAP8[$32>>0]|0;
    $34 = $33 << 24 >> 24;
    $35 = ($34|0)!=(0);
    if ($35) {
     $36 = HEAP32[$3>>2]|0;
     $37 = (_sqlite3DbStrDup(0,$36)|0);
     HEAP32[$zPathname>>2] = $37;
     $38 = HEAP32[$zPathname>>2]|0;
     $39 = ($38|0)==(0|0);
     if (!($39)) {
      $40 = HEAP32[$zPathname>>2]|0;
      $41 = (_sqlite3Strlen30($40)|0);
      HEAP32[$nPathname>>2] = $41;
      HEAP32[$3>>2] = 0;
      break;
     }
     HEAP32[$0>>2] = 7;
     $387 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($387|0);
    }
   }
  }
 } while(0);
 $42 = HEAP32[$3>>2]|0;
 $43 = ($42|0)!=(0|0);
 if ($43) {
  $44 = HEAP32[$3>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = ($46|0)!=(0);
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + 1)|0;
   HEAP32[$nPathname>>2] = $51;
   $52 = HEAP32[$nPathname>>2]|0;
   $53 = $52<<1;
   $54 = ($53|0)<(0);
   $55 = $54 << 31 >> 31;
   $56 = (_sqlite3DbMallocRaw(0,$53,$55)|0);
   HEAP32[$zPathname>>2] = $56;
   $57 = HEAP32[$zPathname>>2]|0;
   $58 = ($57|0)==(0|0);
   if ($58) {
    HEAP32[$0>>2] = 7;
    $387 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($387|0);
   }
   $59 = HEAP32[$zPathname>>2]|0;
   HEAP8[$59>>0] = 0;
   $60 = HEAP32[$1>>2]|0;
   $61 = HEAP32[$3>>2]|0;
   $62 = HEAP32[$nPathname>>2]|0;
   $63 = HEAP32[$zPathname>>2]|0;
   $64 = (_sqlite3OsFullPathname($60,$61,$62,$63)|0);
   HEAP32[$rc>>2] = $64;
   $65 = HEAP32[$zPathname>>2]|0;
   $66 = (_sqlite3Strlen30($65)|0);
   HEAP32[$nPathname>>2] = $66;
   $67 = HEAP32[$3>>2]|0;
   $68 = (_sqlite3Strlen30($67)|0);
   $69 = (($68) + 1)|0;
   $70 = HEAP32[$3>>2]|0;
   $71 = (($70) + ($69)|0);
   HEAP32[$zUri>>2] = $71;
   HEAP32[$z>>2] = $71;
   while(1) {
    $72 = HEAP32[$z>>2]|0;
    $73 = HEAP8[$72>>0]|0;
    $74 = ($73<<24>>24)!=(0);
    $75 = HEAP32[$z>>2]|0;
    if (!($74)) {
     break;
    }
    $76 = (_sqlite3Strlen30($75)|0);
    $77 = (($76) + 1)|0;
    $78 = HEAP32[$z>>2]|0;
    $79 = (($78) + ($77)|0);
    HEAP32[$z>>2] = $79;
    $80 = HEAP32[$z>>2]|0;
    $81 = (_sqlite3Strlen30($80)|0);
    $82 = (($81) + 1)|0;
    $83 = HEAP32[$z>>2]|0;
    $84 = (($83) + ($82)|0);
    HEAP32[$z>>2] = $84;
   }
   $85 = ((($75)) + 1|0);
   $86 = HEAP32[$zUri>>2]|0;
   $87 = $85;
   $88 = $86;
   $89 = (($87) - ($88))|0;
   HEAP32[$nUri>>2] = $89;
   $90 = HEAP32[$rc>>2]|0;
   $91 = ($90|0)==(0);
   if ($91) {
    $92 = HEAP32[$nPathname>>2]|0;
    $93 = (($92) + 8)|0;
    $94 = HEAP32[$1>>2]|0;
    $95 = ((($94)) + 8|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = ($93|0)>($96|0);
    if ($97) {
     $98 = (_sqlite3CantopenError(48375)|0);
     HEAP32[$rc>>2] = $98;
    }
   }
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)!=(0);
   if ($100) {
    $101 = HEAP32[$zPathname>>2]|0;
    _sqlite3DbFree(0,$101);
    $102 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $102;
    $387 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($387|0);
   }
  }
 }
 $103 = HEAP32[$pcacheSize>>2]|0;
 $104 = (($103) + 7)|0;
 $105 = $104 & -8;
 $106 = (224 + ($105))|0;
 $107 = HEAP32[$1>>2]|0;
 $108 = ((($107)) + 4|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = (($109) + 7)|0;
 $111 = $110 & -8;
 $112 = (($106) + ($111))|0;
 $113 = HEAP32[$journalFileSize>>2]|0;
 $114 = $113<<1;
 $115 = (($112) + ($114))|0;
 $116 = HEAP32[$nPathname>>2]|0;
 $117 = (($115) + ($116))|0;
 $118 = (($117) + 1)|0;
 $119 = HEAP32[$nUri>>2]|0;
 $120 = (($118) + ($119))|0;
 $121 = HEAP32[$nPathname>>2]|0;
 $122 = (($120) + ($121))|0;
 $123 = (($122) + 8)|0;
 $124 = (($123) + 2)|0;
 $125 = HEAP32[$nPathname>>2]|0;
 $126 = (($124) + ($125))|0;
 $127 = (($126) + 4)|0;
 $128 = (($127) + 2)|0;
 $129 = (_sqlite3MallocZero($128,0)|0);
 HEAP32[$pPtr>>2] = $129;
 $130 = HEAP32[$pPtr>>2]|0;
 $131 = ($130|0)!=(0|0);
 if (!($131)) {
  $132 = HEAP32[$zPathname>>2]|0;
  _sqlite3DbFree(0,$132);
  HEAP32[$0>>2] = 7;
  $387 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($387|0);
 }
 $133 = HEAP32[$pPtr>>2]|0;
 HEAP32[$pPager>>2] = $133;
 $134 = HEAP32[$pPtr>>2]|0;
 $135 = ((($134)) + 224|0);
 HEAP32[$pPtr>>2] = $135;
 $136 = HEAP32[$pPager>>2]|0;
 $137 = ((($136)) + 212|0);
 HEAP32[$137>>2] = $135;
 $138 = HEAP32[$pcacheSize>>2]|0;
 $139 = (($138) + 7)|0;
 $140 = $139 & -8;
 $141 = HEAP32[$pPtr>>2]|0;
 $142 = (($141) + ($140)|0);
 HEAP32[$pPtr>>2] = $142;
 $143 = HEAP32[$pPager>>2]|0;
 $144 = ((($143)) + 64|0);
 HEAP32[$144>>2] = $142;
 $145 = HEAP32[$1>>2]|0;
 $146 = ((($145)) + 4|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = (($147) + 7)|0;
 $149 = $148 & -8;
 $150 = HEAP32[$pPtr>>2]|0;
 $151 = (($150) + ($149)|0);
 HEAP32[$pPtr>>2] = $151;
 $152 = HEAP32[$pPager>>2]|0;
 $153 = ((($152)) + 72|0);
 HEAP32[$153>>2] = $151;
 $154 = HEAP32[$journalFileSize>>2]|0;
 $155 = HEAP32[$pPtr>>2]|0;
 $156 = (($155) + ($154)|0);
 HEAP32[$pPtr>>2] = $156;
 $157 = HEAP32[$pPager>>2]|0;
 $158 = ((($157)) + 68|0);
 HEAP32[$158>>2] = $156;
 $159 = HEAP32[$journalFileSize>>2]|0;
 $160 = HEAP32[$pPtr>>2]|0;
 $161 = (($160) + ($159)|0);
 HEAP32[$pPtr>>2] = $161;
 $162 = HEAP32[$pPager>>2]|0;
 $163 = ((($162)) + 176|0);
 HEAP32[$163>>2] = $161;
 $164 = HEAP32[$zPathname>>2]|0;
 $165 = ($164|0)!=(0|0);
 if ($165) {
  $166 = HEAP32[$nPathname>>2]|0;
  $167 = (($166) + 1)|0;
  $168 = HEAP32[$nUri>>2]|0;
  $169 = (($167) + ($168))|0;
  $170 = HEAP32[$pPtr>>2]|0;
  $171 = (($170) + ($169)|0);
  HEAP32[$pPtr>>2] = $171;
  $172 = HEAP32[$pPager>>2]|0;
  $173 = ((($172)) + 180|0);
  HEAP32[$173>>2] = $171;
  $174 = HEAP32[$pPager>>2]|0;
  $175 = ((($174)) + 176|0);
  $176 = HEAP32[$175>>2]|0;
  $177 = HEAP32[$zPathname>>2]|0;
  $178 = HEAP32[$nPathname>>2]|0;
  _memcpy(($176|0),($177|0),($178|0))|0;
  $179 = HEAP32[$nUri>>2]|0;
  $180 = ($179|0)!=(0);
  if ($180) {
   $181 = HEAP32[$nPathname>>2]|0;
   $182 = (($181) + 1)|0;
   $183 = HEAP32[$pPager>>2]|0;
   $184 = ((($183)) + 176|0);
   $185 = HEAP32[$184>>2]|0;
   $186 = (($185) + ($182)|0);
   $187 = HEAP32[$zUri>>2]|0;
   $188 = HEAP32[$nUri>>2]|0;
   _memcpy(($186|0),($187|0),($188|0))|0;
  }
  $189 = HEAP32[$pPager>>2]|0;
  $190 = ((($189)) + 180|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = HEAP32[$zPathname>>2]|0;
  $193 = HEAP32[$nPathname>>2]|0;
  _memcpy(($191|0),($192|0),($193|0))|0;
  $194 = HEAP32[$nPathname>>2]|0;
  $195 = HEAP32[$pPager>>2]|0;
  $196 = ((($195)) + 180|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = (($197) + ($194)|0);
  dest=$198; src=18866; stop=dest+10|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $199 = HEAP32[$nPathname>>2]|0;
  $200 = (($199) + 8)|0;
  $201 = (($200) + 1)|0;
  $202 = HEAP32[$pPager>>2]|0;
  $203 = ((($202)) + 180|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = (($204) + ($201)|0);
  $206 = HEAP32[$pPager>>2]|0;
  $207 = ((($206)) + 220|0);
  HEAP32[$207>>2] = $205;
  $208 = HEAP32[$pPager>>2]|0;
  $209 = ((($208)) + 220|0);
  $210 = HEAP32[$209>>2]|0;
  $211 = HEAP32[$zPathname>>2]|0;
  $212 = HEAP32[$nPathname>>2]|0;
  _memcpy(($210|0),($211|0),($212|0))|0;
  $213 = HEAP32[$nPathname>>2]|0;
  $214 = HEAP32[$pPager>>2]|0;
  $215 = ((($214)) + 220|0);
  $216 = HEAP32[$215>>2]|0;
  $217 = (($216) + ($213)|0);
  ;HEAP8[$217>>0]=HEAP8[18876>>0]|0;HEAP8[$217+1>>0]=HEAP8[18876+1>>0]|0;HEAP8[$217+2>>0]=HEAP8[18876+2>>0]|0;HEAP8[$217+3>>0]=HEAP8[18876+3>>0]|0;HEAP8[$217+4>>0]=HEAP8[18876+4>>0]|0;
  $218 = HEAP32[$zPathname>>2]|0;
  _sqlite3DbFree(0,$218);
 }
 $219 = HEAP32[$1>>2]|0;
 $220 = HEAP32[$pPager>>2]|0;
 HEAP32[$220>>2] = $219;
 $221 = HEAP32[$6>>2]|0;
 $222 = HEAP32[$pPager>>2]|0;
 $223 = ((($222)) + 152|0);
 HEAP32[$223>>2] = $221;
 $224 = HEAP32[$3>>2]|0;
 $225 = ($224|0)!=(0|0);
 do {
  if ($225) {
   $226 = HEAP32[$3>>2]|0;
   $227 = HEAP8[$226>>0]|0;
   $228 = $227 << 24 >> 24;
   $229 = ($228|0)!=(0);
   if ($229) {
    HEAP32[$fout>>2] = 0;
    $230 = HEAP32[$1>>2]|0;
    $231 = HEAP32[$pPager>>2]|0;
    $232 = ((($231)) + 176|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = HEAP32[$pPager>>2]|0;
    $235 = ((($234)) + 64|0);
    $236 = HEAP32[$235>>2]|0;
    $237 = HEAP32[$6>>2]|0;
    $238 = (_sqlite3OsOpen($230,$233,$236,$237,$fout)|0);
    HEAP32[$rc>>2] = $238;
    $239 = HEAP32[$fout>>2]|0;
    $240 = $239 & 1;
    HEAP32[$readOnly>>2] = $240;
    $241 = HEAP32[$rc>>2]|0;
    $242 = ($241|0)==(0);
    if ($242) {
     $243 = HEAP32[$pPager>>2]|0;
     $244 = ((($243)) + 64|0);
     $245 = HEAP32[$244>>2]|0;
     $246 = (_sqlite3OsDeviceCharacteristics($245)|0);
     HEAP32[$iDc>>2] = $246;
     $247 = HEAP32[$readOnly>>2]|0;
     $248 = ($247|0)!=(0);
     do {
      if (!($248)) {
       $249 = HEAP32[$pPager>>2]|0;
       _setSectorSize($249);
       $250 = HEAP32[$szPageDflt>>2]|0;
       $251 = HEAP32[$pPager>>2]|0;
       $252 = ((($251)) + 156|0);
       $253 = HEAP32[$252>>2]|0;
       $254 = ($250>>>0)<($253>>>0);
       if ($254) {
        $255 = HEAP32[$pPager>>2]|0;
        $256 = ((($255)) + 156|0);
        $257 = HEAP32[$256>>2]|0;
        $258 = ($257>>>0)>(8192);
        if ($258) {
         HEAP32[$szPageDflt>>2] = 8192;
         break;
        } else {
         $259 = HEAP32[$pPager>>2]|0;
         $260 = ((($259)) + 156|0);
         $261 = HEAP32[$260>>2]|0;
         HEAP32[$szPageDflt>>2] = $261;
         break;
        }
       }
      }
     } while(0);
     $262 = HEAP32[$3>>2]|0;
     $263 = (_sqlite3_uri_boolean($262,18882,0)|0);
     $264 = $263&255;
     $265 = HEAP32[$pPager>>2]|0;
     $266 = ((($265)) + 14|0);
     HEAP8[$266>>0] = $264;
     $267 = HEAP32[$iDc>>2]|0;
     $268 = $267 & 8192;
     $269 = ($268|0)!=(0);
     if (!($269)) {
      $270 = HEAP32[$3>>2]|0;
      $271 = (_sqlite3_uri_boolean($270,18889,0)|0);
      $272 = ($271|0)!=(0);
      if (!($272)) {
       break;
      }
     }
     $273 = HEAP32[$6>>2]|0;
     $274 = $273 | 1;
     HEAP32[$6>>2] = $274;
     label = 39;
    }
   } else {
    label = 39;
   }
  } else {
   label = 39;
  }
 } while(0);
 if ((label|0) == 39) {
  HEAP32[$tempFile>>2] = 1;
  $275 = HEAP32[$pPager>>2]|0;
  $276 = ((($275)) + 17|0);
  HEAP8[$276>>0] = 1;
  $277 = HEAP32[$pPager>>2]|0;
  $278 = ((($277)) + 18|0);
  HEAP8[$278>>0] = 4;
  $279 = HEAP32[$pPager>>2]|0;
  $280 = ((($279)) + 14|0);
  HEAP8[$280>>0] = 1;
  $281 = HEAP32[$6>>2]|0;
  $282 = $281 & 1;
  HEAP32[$readOnly>>2] = $282;
 }
 $283 = HEAP32[$rc>>2]|0;
 $284 = ($283|0)==(0);
 if ($284) {
  $285 = HEAP32[$pPager>>2]|0;
  $286 = (_sqlite3PagerSetPagesize($285,$szPageDflt,-1)|0);
  HEAP32[$rc>>2] = $286;
 }
 $287 = HEAP32[$rc>>2]|0;
 $288 = ($287|0)==(0);
 if ($288) {
  $289 = HEAP32[$4>>2]|0;
  $290 = (($289) + 7)|0;
  $291 = $290 & -8;
  HEAP32[$4>>2] = $291;
  $292 = HEAP32[$szPageDflt>>2]|0;
  $293 = HEAP32[$4>>2]|0;
  $294 = HEAP32[$memDb>>2]|0;
  $295 = ($294|0)!=(0);
  $296 = $295 ^ 1;
  $297 = $296&1;
  $298 = HEAP32[$memDb>>2]|0;
  $299 = ($298|0)!=(0);
  $300 = $299 ^ 1;
  $301 = $300 ? 178 : 0;
  $302 = HEAP32[$pPager>>2]|0;
  $303 = HEAP32[$pPager>>2]|0;
  $304 = ((($303)) + 212|0);
  $305 = HEAP32[$304>>2]|0;
  $306 = (_sqlite3PcacheOpen($292,$293,$297,$301,$302,$305)|0);
  HEAP32[$rc>>2] = $306;
 }
 $307 = HEAP32[$rc>>2]|0;
 $308 = ($307|0)!=(0);
 if ($308) {
  $309 = HEAP32[$pPager>>2]|0;
  $310 = ((($309)) + 64|0);
  $311 = HEAP32[$310>>2]|0;
  (_sqlite3OsClose($311)|0);
  $312 = HEAP32[$pPager>>2]|0;
  $313 = ((($312)) + 208|0);
  $314 = HEAP32[$313>>2]|0;
  _sqlite3PageFree($314);
  $315 = HEAP32[$pPager>>2]|0;
  _sqlite3_free($315);
  $316 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $316;
  $387 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($387|0);
 }
 $317 = HEAP32[$useJournal>>2]|0;
 $318 = $317&255;
 $319 = HEAP32[$pPager>>2]|0;
 $320 = ((($319)) + 6|0);
 HEAP8[$320>>0] = $318;
 $321 = HEAP32[$pPager>>2]|0;
 $322 = ((($321)) + 164|0);
 HEAP32[$322>>2] = 1073741823;
 $323 = HEAP32[$tempFile>>2]|0;
 $324 = $323&255;
 $325 = HEAP32[$pPager>>2]|0;
 $326 = ((($325)) + 13|0);
 HEAP8[$326>>0] = $324;
 $327 = HEAP32[$tempFile>>2]|0;
 $328 = $327&255;
 $329 = HEAP32[$pPager>>2]|0;
 $330 = ((($329)) + 4|0);
 HEAP8[$330>>0] = $328;
 $331 = HEAP32[$pPager>>2]|0;
 $332 = ((($331)) + 13|0);
 $333 = HEAP8[$332>>0]|0;
 $334 = HEAP32[$pPager>>2]|0;
 $335 = ((($334)) + 19|0);
 HEAP8[$335>>0] = $333;
 $336 = HEAP32[$memDb>>2]|0;
 $337 = $336&255;
 $338 = HEAP32[$pPager>>2]|0;
 $339 = ((($338)) + 16|0);
 HEAP8[$339>>0] = $337;
 $340 = HEAP32[$readOnly>>2]|0;
 $341 = $340&255;
 $342 = HEAP32[$pPager>>2]|0;
 $343 = ((($342)) + 15|0);
 HEAP8[$343>>0] = $341;
 $344 = HEAP32[$pPager>>2]|0;
 $345 = ((($344)) + 13|0);
 $346 = HEAP8[$345>>0]|0;
 $347 = HEAP32[$pPager>>2]|0;
 $348 = ((($347)) + 7|0);
 HEAP8[$348>>0] = $346;
 $349 = HEAP32[$pPager>>2]|0;
 $350 = ((($349)) + 7|0);
 $351 = HEAP8[$350>>0]|0;
 $352 = ($351<<24>>24)!=(0);
 if (!($352)) {
  $353 = HEAP32[$pPager>>2]|0;
  $354 = ((($353)) + 8|0);
  HEAP8[$354>>0] = 1;
  $355 = HEAP32[$pPager>>2]|0;
  $356 = ((($355)) + 9|0);
  HEAP8[$356>>0] = 0;
  $357 = HEAP32[$pPager>>2]|0;
  $358 = ((($357)) + 12|0);
  HEAP8[$358>>0] = 2;
  $359 = HEAP32[$pPager>>2]|0;
  $360 = ((($359)) + 11|0);
  HEAP8[$360>>0] = 34;
  $361 = HEAP32[$pPager>>2]|0;
  $362 = ((($361)) + 10|0);
  HEAP8[$362>>0] = 2;
 }
 $363 = HEAP32[$4>>2]|0;
 $364 = $363&65535;
 $365 = HEAP32[$pPager>>2]|0;
 $366 = ((($365)) + 148|0);
 HEAP16[$366>>1] = $364;
 $367 = HEAP32[$pPager>>2]|0;
 $368 = ((($367)) + 168|0);
 $369 = $368;
 $370 = $369;
 HEAP32[$370>>2] = -1;
 $371 = (($369) + 4)|0;
 $372 = $371;
 HEAP32[$372>>2] = -1;
 $373 = HEAP32[$pPager>>2]|0;
 _setSectorSize($373);
 $374 = HEAP32[$useJournal>>2]|0;
 $375 = ($374|0)!=(0);
 if ($375) {
  $378 = HEAP32[$memDb>>2]|0;
  $379 = ($378|0)!=(0);
  if ($379) {
   $380 = HEAP32[$pPager>>2]|0;
   $381 = ((($380)) + 5|0);
   HEAP8[$381>>0] = 4;
  }
 } else {
  $376 = HEAP32[$pPager>>2]|0;
  $377 = ((($376)) + 5|0);
  HEAP8[$377>>0] = 2;
 }
 $382 = HEAP32[$7>>2]|0;
 $383 = HEAP32[$pPager>>2]|0;
 $384 = ((($383)) + 204|0);
 HEAP32[$384>>2] = $382;
 $385 = HEAP32[$pPager>>2]|0;
 $386 = HEAP32[$2>>2]|0;
 HEAP32[$386>>2] = $385;
 HEAP32[$0>>2] = 0;
 $387 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($387|0);
}
function _pageReinit($pData) {
 $pData = $pData|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pData;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PagerGetExtra($1)|0);
 HEAP32[$pPage>>2] = $2;
 $3 = HEAP32[$pPage>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pPage>>2]|0;
 HEAP8[$6>>0] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3PagerPageRefcount($7)|0);
 $9 = ($8|0)>(1);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pPage>>2]|0;
 (_btreeInitPage($10)|0);
 STACKTOP = sp;return;
}
function _sqlite3PagerSetMmapLimit($pPager,$0,$1) {
 $pPager = $pPager|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pPager;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 136|0);
 $16 = $15;
 $17 = $16;
 HEAP32[$17>>2] = $10;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $13;
 $20 = HEAP32[$2>>2]|0;
 _pagerFixMaplimit($20);
 STACKTOP = sp;return;
}
function _sqlite3PagerReadFileheader($pPager,$N,$pDest) {
 $pPager = $pPager|0;
 $N = $N|0;
 $pDest = $pDest|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memset(($3|0),0,($4|0))|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3OsRead($12,$13,$14,0,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(522);
 $$ = $17 ? 0 : $15;
 HEAP32[$rc>>2] = $$;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3PagerSetBusyhandler($pPager,$xBusyHandler,$pBusyHandlerArg) {
 $pPager = $pPager|0;
 $xBusyHandler = $xBusyHandler|0;
 $pBusyHandlerArg = $pBusyHandlerArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $ap = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $xBusyHandler;
 HEAP32[$2>>2] = $pBusyHandlerArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 184|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 188|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 64|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 184|0);
 HEAP32[$ap>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$ap>>2]|0;
 _sqlite3OsFileControlHint($18,15,$19);
 STACKTOP = sp;return;
}
function _btreeInvokeBusyHandler($pArg) {
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pArg;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pBt>>2] = $1;
 $2 = HEAP32[$pBt>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 448|0);
 $6 = (_sqlite3InvokeBusyHandler($5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3PagerIsreadonly($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 15|0);
 $3 = HEAP8[$2>>0]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3Get4byte($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = $3 << 24;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 << 16;
 $10 = $4 | $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 2|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 << 8;
 $16 = $10 | $15;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 3|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = $16 | $20;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3PagerSetPagesize($pPager,$pPageSize,$nReserve) {
 $pPager = $pPager|0;
 $pPageSize = $pPageSize|0;
 $nReserve = $nReserve|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nByte = 0, $or$cond = 0, $pNew = 0, $pageSize = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $pageSize = sp + 12|0;
 $pNew = sp + 8|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pPageSize;
 HEAP32[$2>>2] = $nReserve;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pageSize>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if ($9) {
  label = 3;
 } else {
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 28|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)==(0);
  if ($13) {
   label = 3;
  }
 }
 do {
  if ((label|0) == 3) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 212|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3PcacheRefCount($16)|0);
   $18 = ($17|0)==(0);
   $19 = HEAP32[$pageSize>>2]|0;
   $20 = ($19|0)!=(0);
   $or$cond = $18 & $20;
   if ($or$cond) {
    $21 = HEAP32[$pageSize>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 160|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($21|0)!=($24|0);
    if ($25) {
     HEAP32[$pNew>>2] = 0;
     $26 = $nByte;
     $27 = $26;
     HEAP32[$27>>2] = 0;
     $28 = (($26) + 4)|0;
     $29 = $28;
     HEAP32[$29>>2] = 0;
     $30 = HEAP32[$0>>2]|0;
     $31 = ((($30)) + 17|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = ($33|0)>(0);
     if ($34) {
      $35 = HEAP32[$0>>2]|0;
      $36 = ((($35)) + 64|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$37>>2]|0;
      $39 = ($38|0)!=(0|0);
      if ($39) {
       $40 = HEAP32[$0>>2]|0;
       $41 = ((($40)) + 64|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = (_sqlite3OsFileSize($42,$nByte)|0);
       HEAP32[$rc>>2] = $43;
      }
     }
     $44 = HEAP32[$rc>>2]|0;
     $45 = ($44|0)==(0);
     if ($45) {
      $46 = HEAP32[$pageSize>>2]|0;
      $47 = (_sqlite3PageMalloc($46)|0);
      HEAP32[$pNew>>2] = $47;
      $48 = HEAP32[$pNew>>2]|0;
      $49 = ($48|0)!=(0|0);
      if (!($49)) {
       HEAP32[$rc>>2] = 7;
      }
     }
     $50 = HEAP32[$rc>>2]|0;
     $51 = ($50|0)==(0);
     if ($51) {
      $52 = HEAP32[$0>>2]|0;
      _pager_reset($52);
      $53 = HEAP32[$0>>2]|0;
      $54 = ((($53)) + 212|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$pageSize>>2]|0;
      $57 = (_sqlite3PcacheSetPageSize($55,$56)|0);
      HEAP32[$rc>>2] = $57;
     }
     $58 = HEAP32[$rc>>2]|0;
     $59 = ($58|0)==(0);
     if ($59) {
      $60 = HEAP32[$0>>2]|0;
      $61 = ((($60)) + 208|0);
      $62 = HEAP32[$61>>2]|0;
      _sqlite3PageFree($62);
      $63 = HEAP32[$pNew>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = ((($64)) + 208|0);
      HEAP32[$65>>2] = $63;
      $66 = $nByte;
      $67 = $66;
      $68 = HEAP32[$67>>2]|0;
      $69 = (($66) + 4)|0;
      $70 = $69;
      $71 = HEAP32[$70>>2]|0;
      $72 = HEAP32[$pageSize>>2]|0;
      $73 = (_i64Add(($68|0),($71|0),($72|0),0)|0);
      $74 = tempRet0;
      $75 = (_i64Subtract(($73|0),($74|0),1,0)|0);
      $76 = tempRet0;
      $77 = HEAP32[$pageSize>>2]|0;
      $78 = (___divdi3(($75|0),($76|0),($77|0),0)|0);
      $79 = tempRet0;
      $80 = HEAP32[$0>>2]|0;
      $81 = ((($80)) + 28|0);
      HEAP32[$81>>2] = $78;
      $82 = HEAP32[$pageSize>>2]|0;
      $83 = HEAP32[$0>>2]|0;
      $84 = ((($83)) + 160|0);
      HEAP32[$84>>2] = $82;
      break;
     } else {
      $85 = HEAP32[$pNew>>2]|0;
      _sqlite3PageFree($85);
      break;
     }
    }
   }
  }
 } while(0);
 $86 = HEAP32[$0>>2]|0;
 $87 = ((($86)) + 160|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 HEAP32[$89>>2] = $88;
 $90 = HEAP32[$rc>>2]|0;
 $91 = ($90|0)==(0);
 if (!($91)) {
  $103 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 $92 = HEAP32[$2>>2]|0;
 $93 = ($92|0)<(0);
 if ($93) {
  $94 = HEAP32[$0>>2]|0;
  $95 = ((($94)) + 150|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96 << 16 >> 16;
  HEAP32[$2>>2] = $97;
 }
 $98 = HEAP32[$2>>2]|0;
 $99 = $98&65535;
 $100 = HEAP32[$0>>2]|0;
 $101 = ((($100)) + 150|0);
 HEAP16[$101>>1] = $99;
 $102 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($102);
 $103 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($103|0);
}
function _sqlite3PagerClose($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTmp>>2] = $3;
 _sqlite3BeginBenignMalloc();
 $4 = HEAP32[$0>>2]|0;
 _pagerFreeMapHdrs($4);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP8[$6>>0] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 216|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 10|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 160|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$pTmp>>2]|0;
 (_sqlite3WalClose($9,$13,$16,$17)|0);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 216|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 _pager_reset($20);
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = ($23<<24>>24)!=(0);
 $25 = HEAP32[$0>>2]|0;
 if ($24) {
  _pager_unlock($25);
 } else {
  $26 = ((($25)) + 68|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$0>>2]|0;
   $32 = (_pagerSyncHotJournal($31)|0);
   (_pager_error($30,$32)|0);
  }
  $33 = HEAP32[$0>>2]|0;
  _pagerUnlockAndRollback($33);
 }
 _sqlite3EndBenignMalloc();
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 68|0);
 $36 = HEAP32[$35>>2]|0;
 (_sqlite3OsClose($36)|0);
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 64|0);
 $39 = HEAP32[$38>>2]|0;
 (_sqlite3OsClose($39)|0);
 $40 = HEAP32[$pTmp>>2]|0;
 _sqlite3PageFree($40);
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 212|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3PcacheClose($43);
 $44 = HEAP32[$0>>2]|0;
 _sqlite3_free($44);
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeSchema($p,$nBytes,$xFree) {
 $p = $p|0;
 $nBytes = $nBytes|0;
 $xFree = $xFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nBytes;
 HEAP32[$2>>2] = $xFree;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 $or$cond = $10 & $12;
 if (!($or$cond)) {
  $22 = HEAP32[$pBt>>2]|0;
  $23 = ((($22)) + 48|0);
  $24 = HEAP32[$23>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)<(0);
 $15 = $14 << 31 >> 31;
 $16 = (_sqlite3DbMallocZero(0,$13,$15)|0);
 $17 = HEAP32[$pBt>>2]|0;
 $18 = ((($17)) + 48|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$pBt>>2]|0;
 $21 = ((($20)) + 52|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pBt>>2]|0;
 $23 = ((($22)) + 48|0);
 $24 = HEAP32[$23>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3PagerSetCachesize($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 212|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3PcacheSetCachesize($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3PcacheSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 48;
}
function _sqlite3MemJournalSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 40;
}
function _sqlite3OsOpen($pVfs,$zPath,$pFile,$flags,$pFlagsOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $pFile = $pFile|0;
 $flags = $flags|0;
 $pFlagsOut = $pFlagsOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $pFile;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $pFlagsOut;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = $11 & 556927;
 $13 = HEAP32[$4>>2]|0;
 $14 = (FUNCTION_TABLE_iiiiii[$7 & 255]($8,$9,$10,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3OsDeviceCharacteristics($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 255]($5)|0);
 STACKTOP = sp;return ($6|0);
}
function _setSectorSize($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 13|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 64|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3OsDeviceCharacteristics($8)|0);
  $10 = $9 & 4096;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 64|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3SectorSize($16)|0);
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 156|0);
   HEAP32[$19>>2] = $17;
   STACKTOP = sp;return;
  }
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 156|0);
 HEAP32[$13>>2] = 512;
 STACKTOP = sp;return;
}
function _sqlite3PcacheOpen($szPage,$szExtra,$bPurgeable,$xStress,$pStress,$p) {
 $szPage = $szPage|0;
 $szExtra = $szExtra|0;
 $bPurgeable = $bPurgeable|0;
 $xStress = $xStress|0;
 $pStress = $pStress|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $szPage;
 HEAP32[$1>>2] = $szExtra;
 HEAP32[$2>>2] = $bPurgeable;
 HEAP32[$3>>2] = $xStress;
 HEAP32[$4>>2] = $pStress;
 HEAP32[$5>>2] = $p;
 $6 = HEAP32[$5>>2]|0;
 dest=$6; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$5>>2]|0;
 $8 = ((($7)) + 24|0);
 HEAP32[$8>>2] = 1;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$5>>2]|0;
 $11 = ((($10)) + 28|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = $12&255;
 $14 = HEAP32[$5>>2]|0;
 $15 = ((($14)) + 32|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$5>>2]|0;
 $17 = ((($16)) + 33|0);
 HEAP8[$17>>0] = 2;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = ((($19)) + 36|0);
 HEAP32[$20>>2] = $18;
 $21 = HEAP32[$4>>2]|0;
 $22 = HEAP32[$5>>2]|0;
 $23 = ((($22)) + 40|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$5>>2]|0;
 $25 = ((($24)) + 16|0);
 HEAP32[$25>>2] = 100;
 $26 = HEAP32[$5>>2]|0;
 $27 = ((($26)) + 20|0);
 HEAP32[$27>>2] = 1;
 $28 = HEAP32[$5>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = (_sqlite3PcacheSetPageSize($28,$29)|0);
 STACKTOP = sp;return ($30|0);
}
function _pagerStress($p,$pPg) {
 $p = $p|0;
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pPager = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pPg;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pPager>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $8 = HEAP32[$pPager>>2]|0;
 $9 = ((($8)) + 21|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(0);
 do {
  if ($12) {
   $13 = HEAP32[$pPager>>2]|0;
   $14 = ((($13)) + 21|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = $16 & 3;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    $19 = HEAP32[$2>>2]|0;
    $20 = ((($19)) + 24|0);
    $21 = HEAP16[$20>>1]|0;
    $22 = $21&65535;
    $23 = $22 & 8;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $60 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($60|0);
  }
 } while(0);
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 12|0);
 HEAP32[$26>>2] = 0;
 $27 = HEAP32[$pPager>>2]|0;
 $28 = (_pagerUseWal($27)|0);
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$2>>2]|0;
 if ($29) {
  $31 = (_subjournalPageIfRequired($30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$rc>>2]|0;
  $33 = ($32|0)==(0);
  if ($33) {
   $34 = HEAP32[$pPager>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   $36 = (_pagerWalFrames($34,$35,0,0)|0);
   HEAP32[$rc>>2] = $36;
  }
 } else {
  $37 = ((($30)) + 24|0);
  $38 = HEAP16[$37>>1]|0;
  $39 = $38&65535;
  $40 = $39 & 8;
  $41 = ($40|0)!=(0);
  if ($41) {
   label = 12;
  } else {
   $42 = HEAP32[$pPager>>2]|0;
   $43 = ((($42)) + 17|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = ($45|0)==(3);
   if ($46) {
    label = 12;
   }
  }
  if ((label|0) == 12) {
   $47 = HEAP32[$pPager>>2]|0;
   $48 = (_syncJournal($47,1)|0);
   HEAP32[$rc>>2] = $48;
  }
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)==(0);
  if ($50) {
   $51 = HEAP32[$pPager>>2]|0;
   $52 = HEAP32[$2>>2]|0;
   $53 = (_pager_write_pagelist($51,$52)|0);
   HEAP32[$rc>>2] = $53;
  }
 }
 $54 = HEAP32[$rc>>2]|0;
 $55 = ($54|0)==(0);
 if ($55) {
  $56 = HEAP32[$2>>2]|0;
  _sqlite3PcacheMakeClean($56);
 }
 $57 = HEAP32[$pPager>>2]|0;
 $58 = HEAP32[$rc>>2]|0;
 $59 = (_pager_error($57,$58)|0);
 HEAP32[$0>>2] = $59;
 $60 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _sqlite3OsClose($pId) {
 $pId = $pId|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pId;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (FUNCTION_TABLE_ii[$7 & 255]($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3PageFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _pcache1Free($1);
 STACKTOP = sp;return;
}
function _sqlite3SectorSize($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $iRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iRet = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3OsSectorSize($1)|0);
 HEAP32[$iRet>>2] = $2;
 $3 = HEAP32[$iRet>>2]|0;
 $4 = ($3|0)<(32);
 if ($4) {
  HEAP32[$iRet>>2] = 512;
 } else {
  $5 = HEAP32[$iRet>>2]|0;
  $6 = ($5|0)>(65536);
  if ($6) {
   HEAP32[$iRet>>2] = 65536;
  }
 }
 $7 = HEAP32[$iRet>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3OsSectorSize($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $xSectorSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $xSectorSize = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$xSectorSize>>2] = $4;
 $5 = HEAP32[$xSectorSize>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = 4096;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$xSectorSize>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (FUNCTION_TABLE_ii[$7 & 255]($8)|0);
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3PcacheSetPageSize($pCache,$szPage) {
 $pCache = $pCache|0;
 $szPage = $szPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $szPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[(128)>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 28|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 40)|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = (FUNCTION_TABLE_iiii[$7 & 255]($8,$12,$16)|0);
  HEAP32[$pNew>>2] = $17;
  $18 = HEAP32[$pNew>>2]|0;
  $19 = ($18|0)==(0|0);
  if ($19) {
   HEAP32[$0>>2] = 7;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
  $20 = HEAP32[(132)>>2]|0;
  $21 = HEAP32[$pNew>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = (_numberOfCachePages($22)|0);
  FUNCTION_TABLE_vii[$20 & 255]($21,$23);
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 44|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[(156)>>2]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 44|0);
   $31 = HEAP32[$30>>2]|0;
   FUNCTION_TABLE_vi[$28 & 255]($31);
  }
  $32 = HEAP32[$pNew>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 44|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$2>>2]|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 24|0);
  HEAP32[$37>>2] = $35;
 }
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _numberOfCachePages($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(0);
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 if ($5) {
  HEAP32[$0>>2] = $8;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $9 = ($8|0)<(0);
  $10 = $9 << 31 >> 31;
  $11 = (___muldi3(-1024,-1,($8|0),($10|0))|0);
  $12 = tempRet0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($15) + ($18))|0;
  $20 = ($19|0)<(0);
  $21 = $20 << 31 >> 31;
  $22 = (___divdi3(($11|0),($12|0),($19|0),($21|0))|0);
  $23 = tempRet0;
  HEAP32[$0>>2] = $22;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _pagerUseWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = $4&1;
 STACKTOP = sp;return ($5|0);
}
function _subjournalPageIfRequired($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_subjRequiresPage($2)|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (_subjournalPage($5)|0);
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 } else {
  HEAP32[$0>>2] = 0;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 return (0)|0;
}
function _pagerWalFrames($pPager,$pList,$nTruncate,$isCommit) {
 $pPager = $pPager|0;
 $pList = $pList|0;
 $nTruncate = $nTruncate|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $nList = 0, $p = 0, $ppNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $nList = sp + 8|0;
 $p = sp + 4|0;
 $ppNext = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $nTruncate;
 HEAP32[$3>>2] = $isCommit;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 L1: do {
  if ($5) {
   HEAP32[$ppNext>>2] = $1;
   HEAP32[$nList>>2] = 0;
   $6 = HEAP32[$1>>2]|0;
   HEAP32[$p>>2] = $6;
   while(1) {
    $7 = HEAP32[$p>>2]|0;
    $8 = HEAP32[$ppNext>>2]|0;
    HEAP32[$8>>2] = $7;
    $9 = ($7|0)!=(0|0);
    if (!($9)) {
     break L1;
    }
    $10 = HEAP32[$p>>2]|0;
    $11 = ((($10)) + 20|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$2>>2]|0;
    $14 = ($12>>>0)<=($13>>>0);
    if ($14) {
     $15 = HEAP32[$p>>2]|0;
     $16 = ((($15)) + 12|0);
     HEAP32[$ppNext>>2] = $16;
     $17 = HEAP32[$nList>>2]|0;
     $18 = (($17) + 1)|0;
     HEAP32[$nList>>2] = $18;
    }
    $19 = HEAP32[$p>>2]|0;
    $20 = ((($19)) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$p>>2] = $21;
   }
  } else {
   HEAP32[$nList>>2] = 1;
  }
 } while(0);
 $22 = HEAP32[$nList>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 192|0);
 $25 = ((($24)) + 8|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($22))|0;
 HEAP32[$25>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 20|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(1);
 if ($31) {
  $32 = HEAP32[$1>>2]|0;
  _pager_write_changecounter($32);
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 216|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 160|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$3>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 11|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = (_sqlite3WalFrames($35,$38,$39,$40,$41,$45)|0);
 HEAP32[$rc>>2] = $46;
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if (!($48)) {
  $68 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 96|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if (!($52)) {
  $68 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $53 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $53;
 while(1) {
  $54 = HEAP32[$p>>2]|0;
  $55 = ($54|0)!=(0|0);
  if (!($55)) {
   break;
  }
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 96|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$p>>2]|0;
  $60 = ((($59)) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$p>>2]|0;
  $63 = ((($62)) + 4|0);
  $64 = HEAP32[$63>>2]|0;
  _sqlite3BackupUpdate($58,$61,$64);
  $65 = HEAP32[$p>>2]|0;
  $66 = ((($65)) + 12|0);
  $67 = HEAP32[$66>>2]|0;
  HEAP32[$p>>2] = $67;
 }
 $68 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($68|0);
}
function _syncJournal($pPager,$newHdr) {
 $pPager = $pPager|0;
 $newHdr = $newHdr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMagic = 0, $iDc = 0, $iNextHdrOffset = 0, $or$cond = 0, $rc = 0, $zHeader = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $iDc = sp + 8|0;
 $iNextHdrOffset = sp;
 $aMagic = sp + 40|0;
 $zHeader = sp + 28|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $newHdr;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3PagerExclusiveLock($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 7|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 68|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$1>>2]|0;
    $18 = ((($17)) + 5|0);
    $19 = HEAP8[$18>>0]|0;
    $20 = $19&255;
    $21 = ($20|0)!=(4);
    if ($21) {
     $22 = HEAP32[$1>>2]|0;
     $23 = ((($22)) + 64|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = (_sqlite3OsDeviceCharacteristics($24)|0);
     HEAP32[$iDc>>2] = $25;
     $26 = HEAP32[$iDc>>2]|0;
     $27 = $26 & 512;
     $28 = (0)==($27|0);
     if ($28) {
      ;HEAP8[$zHeader>>0]=HEAP8[19004>>0]|0;HEAP8[$zHeader+1>>0]=HEAP8[19004+1>>0]|0;HEAP8[$zHeader+2>>0]=HEAP8[19004+2>>0]|0;HEAP8[$zHeader+3>>0]=HEAP8[19004+3>>0]|0;HEAP8[$zHeader+4>>0]=HEAP8[19004+4>>0]|0;HEAP8[$zHeader+5>>0]=HEAP8[19004+5>>0]|0;HEAP8[$zHeader+6>>0]=HEAP8[19004+6>>0]|0;HEAP8[$zHeader+7>>0]=HEAP8[19004+7>>0]|0;
      $29 = ((($zHeader)) + 8|0);
      $30 = HEAP32[$1>>2]|0;
      $31 = ((($30)) + 48|0);
      $32 = HEAP32[$31>>2]|0;
      _sqlite3Put4byte($29,$32);
      $33 = HEAP32[$1>>2]|0;
      $34 = (_journalHdrOffset($33)|0);
      $35 = tempRet0;
      $36 = $iNextHdrOffset;
      $37 = $36;
      HEAP32[$37>>2] = $34;
      $38 = (($36) + 4)|0;
      $39 = $38;
      HEAP32[$39>>2] = $35;
      $40 = HEAP32[$1>>2]|0;
      $41 = ((($40)) + 68|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = $iNextHdrOffset;
      $44 = $43;
      $45 = HEAP32[$44>>2]|0;
      $46 = (($43) + 4)|0;
      $47 = $46;
      $48 = HEAP32[$47>>2]|0;
      $49 = (_sqlite3OsRead($42,$aMagic,8,$45,$48)|0);
      HEAP32[$rc>>2] = $49;
      $50 = HEAP32[$rc>>2]|0;
      $51 = ($50|0)==(0);
      if ($51) {
       $52 = (_memcmp($aMagic,19004,8)|0);
       $53 = (0)==($52|0);
       if ($53) {
        $54 = HEAP32[$1>>2]|0;
        $55 = ((($54)) + 68|0);
        $56 = HEAP32[$55>>2]|0;
        $57 = $iNextHdrOffset;
        $58 = $57;
        $59 = HEAP32[$58>>2]|0;
        $60 = (($57) + 4)|0;
        $61 = $60;
        $62 = HEAP32[$61>>2]|0;
        $63 = (_sqlite3OsWrite($56,19012,1,$59,$62)|0);
        HEAP32[$rc>>2] = $63;
       }
      }
      $64 = HEAP32[$rc>>2]|0;
      $65 = ($64|0)!=(0);
      $66 = HEAP32[$rc>>2]|0;
      $67 = ($66|0)!=(522);
      $or$cond = $65 & $67;
      if ($or$cond) {
       $68 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $68;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
      $69 = HEAP32[$1>>2]|0;
      $70 = ((($69)) + 8|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = ($72|0)!=(0);
      if ($73) {
       $74 = HEAP32[$iDc>>2]|0;
       $75 = $74 & 1024;
       $76 = (0)==($75|0);
       if ($76) {
        $77 = HEAP32[$1>>2]|0;
        $78 = ((($77)) + 68|0);
        $79 = HEAP32[$78>>2]|0;
        $80 = HEAP32[$1>>2]|0;
        $81 = ((($80)) + 12|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $84 = (_sqlite3OsSync($79,$83)|0);
        HEAP32[$rc>>2] = $84;
        $85 = HEAP32[$rc>>2]|0;
        $86 = ($85|0)!=(0);
        if ($86) {
         $87 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $87;
         $169 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($169|0);
        }
       }
      }
      $88 = HEAP32[$1>>2]|0;
      $89 = ((($88)) + 68|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = HEAP32[$1>>2]|0;
      $92 = ((($91)) + 88|0);
      $93 = $92;
      $94 = $93;
      $95 = HEAP32[$94>>2]|0;
      $96 = (($93) + 4)|0;
      $97 = $96;
      $98 = HEAP32[$97>>2]|0;
      $99 = (_sqlite3OsWrite($90,$zHeader,12,$95,$98)|0);
      HEAP32[$rc>>2] = $99;
      $100 = HEAP32[$rc>>2]|0;
      $101 = ($100|0)!=(0);
      if ($101) {
       $102 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $102;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
     }
     $103 = HEAP32[$iDc>>2]|0;
     $104 = $103 & 1024;
     $105 = (0)==($104|0);
     if ($105) {
      $106 = HEAP32[$1>>2]|0;
      $107 = ((($106)) + 68|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = HEAP32[$1>>2]|0;
      $110 = ((($109)) + 12|0);
      $111 = HEAP8[$110>>0]|0;
      $112 = $111&255;
      $113 = HEAP32[$1>>2]|0;
      $114 = ((($113)) + 12|0);
      $115 = HEAP8[$114>>0]|0;
      $116 = $115&255;
      $117 = ($116|0)==(3);
      $118 = $117 ? 16 : 0;
      $119 = $112 | $118;
      $120 = (_sqlite3OsSync($108,$119)|0);
      HEAP32[$rc>>2] = $120;
      $121 = HEAP32[$rc>>2]|0;
      $122 = ($121|0)!=(0);
      if ($122) {
       $123 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $123;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
     }
     $124 = HEAP32[$1>>2]|0;
     $125 = ((($124)) + 80|0);
     $126 = $125;
     $127 = $126;
     $128 = HEAP32[$127>>2]|0;
     $129 = (($126) + 4)|0;
     $130 = $129;
     $131 = HEAP32[$130>>2]|0;
     $132 = HEAP32[$1>>2]|0;
     $133 = ((($132)) + 88|0);
     $134 = $133;
     $135 = $134;
     HEAP32[$135>>2] = $128;
     $136 = (($134) + 4)|0;
     $137 = $136;
     HEAP32[$137>>2] = $131;
     $138 = HEAP32[$2>>2]|0;
     $139 = ($138|0)!=(0);
     if (!($139)) {
      break;
     }
     $140 = HEAP32[$iDc>>2]|0;
     $141 = $140 & 512;
     $142 = (0)==($141|0);
     if (!($142)) {
      break;
     }
     $143 = HEAP32[$1>>2]|0;
     $144 = ((($143)) + 48|0);
     HEAP32[$144>>2] = 0;
     $145 = HEAP32[$1>>2]|0;
     $146 = (_writeJournalHdr($145)|0);
     HEAP32[$rc>>2] = $146;
     $147 = HEAP32[$rc>>2]|0;
     $148 = ($147|0)!=(0);
     if (!($148)) {
      break;
     }
     $149 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $149;
     $169 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($169|0);
    }
   }
   $150 = HEAP32[$1>>2]|0;
   $151 = ((($150)) + 80|0);
   $152 = $151;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = (($152) + 4)|0;
   $156 = $155;
   $157 = HEAP32[$156>>2]|0;
   $158 = HEAP32[$1>>2]|0;
   $159 = ((($158)) + 88|0);
   $160 = $159;
   $161 = $160;
   HEAP32[$161>>2] = $154;
   $162 = (($160) + 4)|0;
   $163 = $162;
   HEAP32[$163>>2] = $157;
  }
 } while(0);
 $164 = HEAP32[$1>>2]|0;
 $165 = ((($164)) + 212|0);
 $166 = HEAP32[$165>>2]|0;
 _sqlite3PcacheClearSyncFlags($166);
 $167 = HEAP32[$1>>2]|0;
 $168 = ((($167)) + 17|0);
 HEAP8[$168>>0] = 4;
 HEAP32[$0>>2] = 0;
 $169 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _pager_write_pagelist($pPager,$pList) {
 $pPager = $pPager|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $offset = 0, $pData = 0, $pgno = 0, $rc = 0, $szFile = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $szFile = sp + 8|0;
 $pgno = sp + 20|0;
 $offset = sp;
 $pData = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pList;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 152|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_pagerOpentemp($7,$10,$13)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 do {
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 40|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 28|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19>>>0)<($22>>>0);
   if ($23) {
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 12|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ($26|0)!=(0|0);
    if (!($27)) {
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 20|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$0>>2]|0;
     $32 = ((($31)) + 40|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($30>>>0)>($33>>>0);
     if (!($34)) {
      break;
     }
    }
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 160|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)<(0);
    $39 = $38 << 31 >> 31;
    $40 = HEAP32[$0>>2]|0;
    $41 = ((($40)) + 28|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (___muldi3(($37|0),($39|0),($42|0),0)|0);
    $44 = tempRet0;
    $45 = $szFile;
    $46 = $45;
    HEAP32[$46>>2] = $43;
    $47 = (($45) + 4)|0;
    $48 = $47;
    HEAP32[$48>>2] = $44;
    $49 = HEAP32[$0>>2]|0;
    $50 = ((($49)) + 64|0);
    $51 = HEAP32[$50>>2]|0;
    _sqlite3OsFileControlHint($51,5,$szFile);
    $52 = HEAP32[$0>>2]|0;
    $53 = ((($52)) + 28|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = HEAP32[$0>>2]|0;
    $56 = ((($55)) + 40|0);
    HEAP32[$56>>2] = $54;
   }
  }
 } while(0);
 while(1) {
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  $59 = HEAP32[$1>>2]|0;
  $60 = ($59|0)!=(0|0);
  $61 = $58 ? $60 : 0;
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 20|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$pgno>>2] = $64;
  $65 = HEAP32[$pgno>>2]|0;
  $66 = HEAP32[$0>>2]|0;
  $67 = ((($66)) + 28|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = ($65>>>0)<=($68>>>0);
  if ($69) {
   $70 = HEAP32[$1>>2]|0;
   $71 = ((($70)) + 24|0);
   $72 = HEAP16[$71>>1]|0;
   $73 = $72&65535;
   $74 = $73 & 32;
   $75 = (0)==($74|0);
   if ($75) {
    $76 = HEAP32[$pgno>>2]|0;
    $77 = (($76) - 1)|0;
    $78 = HEAP32[$0>>2]|0;
    $79 = ((($78)) + 160|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ($80|0)<(0);
    $82 = $81 << 31 >> 31;
    $83 = (___muldi3(($77|0),0,($80|0),($82|0))|0);
    $84 = tempRet0;
    $85 = $offset;
    $86 = $85;
    HEAP32[$86>>2] = $83;
    $87 = (($85) + 4)|0;
    $88 = $87;
    HEAP32[$88>>2] = $84;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 20|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(1);
    if ($92) {
     $93 = HEAP32[$1>>2]|0;
     _pager_write_changecounter($93);
    }
    $94 = HEAP32[$1>>2]|0;
    $95 = ((($94)) + 4|0);
    $96 = HEAP32[$95>>2]|0;
    HEAP32[$pData>>2] = $96;
    $97 = HEAP32[$0>>2]|0;
    $98 = ((($97)) + 64|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = HEAP32[$pData>>2]|0;
    $101 = HEAP32[$0>>2]|0;
    $102 = ((($101)) + 160|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = $offset;
    $105 = $104;
    $106 = HEAP32[$105>>2]|0;
    $107 = (($104) + 4)|0;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (_sqlite3OsWrite($99,$100,$103,$106,$109)|0);
    HEAP32[$rc>>2] = $110;
    $111 = HEAP32[$pgno>>2]|0;
    $112 = ($111|0)==(1);
    if ($112) {
     $113 = HEAP32[$0>>2]|0;
     $114 = ((($113)) + 112|0);
     $115 = HEAP32[$pData>>2]|0;
     $116 = ((($115)) + 24|0);
     dest=$114; src=$116; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
    }
    $117 = HEAP32[$pgno>>2]|0;
    $118 = HEAP32[$0>>2]|0;
    $119 = ((($118)) + 36|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = ($117>>>0)>($120>>>0);
    if ($121) {
     $122 = HEAP32[$pgno>>2]|0;
     $123 = HEAP32[$0>>2]|0;
     $124 = ((($123)) + 36|0);
     HEAP32[$124>>2] = $122;
    }
    $125 = HEAP32[$0>>2]|0;
    $126 = ((($125)) + 192|0);
    $127 = ((($126)) + 8|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = (($128) + 1)|0;
    HEAP32[$127>>2] = $129;
    $130 = HEAP32[$0>>2]|0;
    $131 = ((($130)) + 96|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = HEAP32[$pgno>>2]|0;
    $134 = HEAP32[$1>>2]|0;
    $135 = ((($134)) + 4|0);
    $136 = HEAP32[$135>>2]|0;
    _sqlite3BackupUpdate($132,$133,$136);
   }
  }
  $137 = HEAP32[$1>>2]|0;
  $138 = ((($137)) + 12|0);
  $139 = HEAP32[$138>>2]|0;
  HEAP32[$1>>2] = $139;
 }
 $140 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($140|0);
}
function _sqlite3PcacheMakeClean($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($7,1);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & -15;
 $13 = $12&65535;
 HEAP16[$9>>1] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 | 1;
 $19 = $18&65535;
 HEAP16[$15>>1] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 26|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)==(0);
 if (!($24)) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$0>>2]|0;
 _pcacheUnpin($25);
 STACKTOP = sp;return;
}
function _pager_error($pPager,$rc) {
 $pPager = $pPager|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $rc;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 255;
 HEAP32[$rc2>>2] = $3;
 $4 = HEAP32[$rc2>>2]|0;
 $5 = ($4|0)==(13);
 $6 = HEAP32[$rc2>>2]|0;
 $7 = ($6|0)==(10);
 $or$cond = $5 | $7;
 if (!($or$cond)) {
  $13 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 44|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 17|0);
 HEAP8[$12>>0] = 6;
 $13 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _subjRequiresPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pPager = 0, $pgno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pPager = sp + 12|0;
 $p = sp + 8|0;
 $pgno = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgno>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$pPager>>2]|0;
  $10 = ((($9)) + 104|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   label = 7;
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pPager>>2]|0;
  $15 = ((($14)) + 100|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*48)|0)|0);
  HEAP32[$p>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pgno>>2]|0;
  $22 = ($20>>>0)>=($21>>>0);
  if ($22) {
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$pgno>>2]|0;
   $27 = (_sqlite3BitvecTestNotNull($25,$26)|0);
   $28 = (0)==($27|0);
   if ($28) {
    label = 5;
    break;
   }
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 return (0)|0;
}
function _subjournalPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $offset = 0, $pData = 0, $pData2 = 0, $pPager = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $rc = sp + 20|0;
 $pPager = sp + 16|0;
 $pData = sp + 12|0;
 $offset = sp;
 $pData2 = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)!=(2);
 if ($8) {
  $9 = HEAP32[$pPager>>2]|0;
  $10 = (_openSubJournal($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)==(0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 4|0);
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$pData>>2] = $15;
   $16 = HEAP32[$pPager>>2]|0;
   $17 = ((($16)) + 56|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$pPager>>2]|0;
   $20 = ((($19)) + 160|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (4 + ($21))|0;
   $23 = ($22|0)<(0);
   $24 = $23 << 31 >> 31;
   $25 = (___muldi3(($18|0),0,($22|0),($24|0))|0);
   $26 = tempRet0;
   $27 = $offset;
   $28 = $27;
   HEAP32[$28>>2] = $25;
   $29 = (($27) + 4)|0;
   $30 = $29;
   HEAP32[$30>>2] = $26;
   $31 = HEAP32[$pData>>2]|0;
   HEAP32[$pData2>>2] = $31;
   $32 = HEAP32[$pPager>>2]|0;
   $33 = ((($32)) + 72|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $offset;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$0>>2]|0;
   $42 = ((($41)) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_write32bits($34,$37,$40,$43)|0);
   HEAP32[$rc>>2] = $44;
   $45 = HEAP32[$rc>>2]|0;
   $46 = ($45|0)==(0);
   if ($46) {
    $47 = HEAP32[$pPager>>2]|0;
    $48 = ((($47)) + 72|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = HEAP32[$pData2>>2]|0;
    $51 = HEAP32[$pPager>>2]|0;
    $52 = ((($51)) + 160|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = $offset;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = (($54) + 4)|0;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (_i64Add(($56|0),($59|0),4,0)|0);
    $61 = tempRet0;
    $62 = (_sqlite3OsWrite($49,$50,$53,$60,$61)|0);
    HEAP32[$rc>>2] = $62;
   }
  }
 }
 $63 = HEAP32[$rc>>2]|0;
 $64 = ($63|0)==(0);
 if (!($64)) {
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 $65 = HEAP32[$pPager>>2]|0;
 $66 = ((($65)) + 56|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 HEAP32[$66>>2] = $68;
 $69 = HEAP32[$pPager>>2]|0;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = (_addToSavepointBitvecs($69,$72)|0);
 HEAP32[$rc>>2] = $73;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _sqlite3BitvecTestNotNull($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $bin = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $bin = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($5>>>0)>=($7>>>0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 while(1) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13>>>0) / ($16>>>0))&-1;
  HEAP32[$bin>>2] = $17;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($18>>>0) % ($21>>>0))&-1;
  HEAP32[$2>>2] = $22;
  $23 = HEAP32[$bin>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$1>>2] = $27;
  $28 = HEAP32[$1>>2]|0;
  $29 = ($28|0)!=(0|0);
  if (!($29)) {
   label = 5;
   break;
  }
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31>>>0)<=(4000);
 $33 = HEAP32[$2>>2]|0;
 if ($32) {
  $34 = (($33>>>0) / 8)&-1;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = (($36) + ($34)|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = $38&255;
  $40 = HEAP32[$2>>2]|0;
  $41 = $40 & 7;
  $42 = 1 << $41;
  $43 = $39 & $42;
  $44 = ($43|0)!=(0);
  $45 = $44&1;
  HEAP32[$0>>2] = $45;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $46 = (($33) + 1)|0;
 HEAP32[$2>>2] = $46;
 $47 = $33;
 $48 = (($47>>>0) % 125)&-1;
 HEAP32[$h>>2] = $48;
 while(1) {
  $49 = HEAP32[$h>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 12|0);
  $52 = (($51) + ($49<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($53|0)!=(0);
  if (!($54)) {
   label = 13;
   break;
  }
  $55 = HEAP32[$h>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 12|0);
  $58 = (($57) + ($55<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ($59|0)==($60|0);
  if ($61) {
   label = 11;
   break;
  }
  $62 = HEAP32[$h>>2]|0;
  $63 = (($62) + 1)|0;
  $64 = (($63>>>0) % 125)&-1;
  HEAP32[$h>>2] = $64;
 }
 if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _openSubJournal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 5|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(4);
 if (!($10)) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 22|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)!=(0);
  if (!($15)) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 72|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_pagerOpentemp($19,$22,8192)|0);
   HEAP32[$rc>>2] = $23;
   $24 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($24|0);
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 72|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3MemJournalOpen($18);
 $24 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _write32bits($fd,$0,$1,$val) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $val = $val|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ac = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 $ac = sp + 16|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $val;
 $9 = HEAP32[$4>>2]|0;
 _sqlite3Put4byte($ac,$9);
 $10 = HEAP32[$2>>2]|0;
 $11 = $3;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3OsWrite($10,$ac,4,$13,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _addToSavepointBitvecs($pPager,$pgno) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ii = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $2 = HEAP32[$ii>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 104|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$ii>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 100|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*48)|0)|0);
  HEAP32[$p>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12>>>0)<=($15>>>0);
  if ($16) {
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = (_sqlite3BitvecSet($19,$20)|0);
   $22 = HEAP32[$rc>>2]|0;
   $23 = $22 | $21;
   HEAP32[$rc>>2] = $23;
  }
  $24 = HEAP32[$ii>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$ii>>2] = $25;
 }
 $26 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3MemJournalOpen($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (_sqlite3MemJournalSize()|0);
 _memset(($2|0),0,($3|0))|0;
 $4 = HEAP32[$p>>2]|0;
 HEAP32[$4>>2] = 1688;
 STACKTOP = sp;return;
}
function _pagerOpentemp($pPager,$pFile,$vfsFlags) {
 $pPager = $pPager|0;
 $pFile = $pFile|0;
 $vfsFlags = $vfsFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $vfsFlags;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 | 30;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3OsOpen($6,0,$7,$8,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _memjrnlClose($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 (_memjrnlTruncate($1,0,0)|0);
 STACKTOP = sp;return 0;
}
function _memjrnlRead($pJfd,$zBuf,$iAmt,$0,$1) {
 $pJfd = $pJfd|0;
 $zBuf = $zBuf|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $iChunkOffset = 0, $iOff = 0, $iSpace = 0, $nCopy = 0, $nRead = 0, $or$cond = 0, $or$cond5 = 0, $p = 0, $pChunk = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 8|0;
 $p = sp + 40|0;
 $zOut = sp + 36|0;
 $nRead = sp + 32|0;
 $iChunkOffset = sp + 28|0;
 $pChunk = sp + 24|0;
 $iOff = sp;
 $iSpace = sp + 20|0;
 $nCopy = sp + 16|0;
 HEAP32[$2>>2] = $pJfd;
 HEAP32[$3>>2] = $zBuf;
 HEAP32[$4>>2] = $iAmt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$zOut>>2] = $11;
 $12 = HEAP32[$4>>2]|0;
 HEAP32[$nRead>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $5;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = ($17|0)!=($23|0);
 $28 = ($20|0)!=($26|0);
 $29 = $27 | $28;
 $30 = $5;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($32|0)==(0);
 $37 = ($35|0)==(0);
 $38 = $36 & $37;
 $or$cond = $29 | $38;
 L1: do {
  if ($or$cond) {
   $39 = $iOff;
   $40 = $39;
   HEAP32[$40>>2] = 0;
   $41 = (($39) + 4)|0;
   $42 = $41;
   HEAP32[$42>>2] = 0;
   $43 = HEAP32[$p>>2]|0;
   $44 = ((($43)) + 4|0);
   $45 = HEAP32[$44>>2]|0;
   HEAP32[$pChunk>>2] = $45;
   while(1) {
    $46 = HEAP32[$pChunk>>2]|0;
    $47 = ($46|0)!=(0|0);
    if (!($47)) {
     break L1;
    }
    $48 = $iOff;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (_i64Add(($50|0),($53|0),1020,0)|0);
    $55 = tempRet0;
    $56 = $5;
    $57 = $56;
    $58 = HEAP32[$57>>2]|0;
    $59 = (($56) + 4)|0;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = ($55|0)<($61|0);
    $63 = ($54>>>0)<=($58>>>0);
    $64 = ($55|0)==($61|0);
    $65 = $64 & $63;
    $66 = $62 | $65;
    if (!($66)) {
     break L1;
    }
    $67 = $iOff;
    $68 = $67;
    $69 = HEAP32[$68>>2]|0;
    $70 = (($67) + 4)|0;
    $71 = $70;
    $72 = HEAP32[$71>>2]|0;
    $73 = (_i64Add(($69|0),($72|0),1020,0)|0);
    $74 = tempRet0;
    $75 = $iOff;
    $76 = $75;
    HEAP32[$76>>2] = $73;
    $77 = (($75) + 4)|0;
    $78 = $77;
    HEAP32[$78>>2] = $74;
    $79 = HEAP32[$pChunk>>2]|0;
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$pChunk>>2] = $80;
   }
  } else {
   $81 = HEAP32[$p>>2]|0;
   $82 = ((($81)) + 24|0);
   $83 = ((($82)) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   HEAP32[$pChunk>>2] = $84;
  }
 } while(0);
 $85 = $5;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = (___remdi3(($87|0),($90|0),1020,0)|0);
 $92 = tempRet0;
 HEAP32[$iChunkOffset>>2] = $91;
 while(1) {
  $93 = HEAP32[$iChunkOffset>>2]|0;
  $94 = (1020 - ($93))|0;
  HEAP32[$iSpace>>2] = $94;
  $95 = HEAP32[$nRead>>2]|0;
  $96 = HEAP32[$iChunkOffset>>2]|0;
  $97 = (1020 - ($96))|0;
  $98 = ($95|0)<($97|0);
  $99 = HEAP32[$nRead>>2]|0;
  $100 = HEAP32[$iChunkOffset>>2]|0;
  $101 = (1020 - ($100))|0;
  $102 = $98 ? $99 : $101;
  HEAP32[$nCopy>>2] = $102;
  $103 = HEAP32[$zOut>>2]|0;
  $104 = HEAP32[$iChunkOffset>>2]|0;
  $105 = HEAP32[$pChunk>>2]|0;
  $106 = ((($105)) + 4|0);
  $107 = (($106) + ($104)|0);
  $108 = HEAP32[$nCopy>>2]|0;
  _memcpy(($103|0),($107|0),($108|0))|0;
  $109 = HEAP32[$nCopy>>2]|0;
  $110 = HEAP32[$zOut>>2]|0;
  $111 = (($110) + ($109)|0);
  HEAP32[$zOut>>2] = $111;
  $112 = HEAP32[$iSpace>>2]|0;
  $113 = HEAP32[$nRead>>2]|0;
  $114 = (($113) - ($112))|0;
  HEAP32[$nRead>>2] = $114;
  HEAP32[$iChunkOffset>>2] = 0;
  $115 = HEAP32[$nRead>>2]|0;
  $116 = ($115|0)>=(0);
  if (!($116)) {
   label = 10;
   break;
  }
  $117 = HEAP32[$pChunk>>2]|0;
  $118 = HEAP32[$117>>2]|0;
  HEAP32[$pChunk>>2] = $118;
  $119 = ($118|0)!=(0|0);
  $120 = HEAP32[$nRead>>2]|0;
  $121 = ($120|0)>(0);
  $or$cond5 = $119 & $121;
  if (!($or$cond5)) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 10) {
  $122 = $5;
  $123 = $122;
  $124 = HEAP32[$123>>2]|0;
  $125 = (($122) + 4)|0;
  $126 = $125;
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$4>>2]|0;
  $129 = ($128|0)<(0);
  $130 = $129 << 31 >> 31;
  $131 = (_i64Add(($124|0),($127|0),($128|0),($130|0))|0);
  $132 = tempRet0;
  $133 = HEAP32[$p>>2]|0;
  $134 = ((($133)) + 24|0);
  $135 = $134;
  $136 = $135;
  HEAP32[$136>>2] = $131;
  $137 = (($135) + 4)|0;
  $138 = $137;
  HEAP32[$138>>2] = $132;
  $139 = HEAP32[$pChunk>>2]|0;
  $140 = HEAP32[$p>>2]|0;
  $141 = ((($140)) + 24|0);
  $142 = ((($141)) + 8|0);
  HEAP32[$142>>2] = $139;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _memjrnlWrite($pJfd,$zBuf,$iAmt,$0,$1) {
 $pJfd = $pJfd|0;
 $zBuf = $zBuf|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $iChunkOffset = 0, $iSpace = 0, $nWrite = 0, $p = 0, $pChunk = 0, $pNew = 0, $zWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp;
 $p = sp + 32|0;
 $nWrite = sp + 28|0;
 $zWrite = sp + 24|0;
 $pChunk = sp + 20|0;
 $iChunkOffset = sp + 16|0;
 $iSpace = sp + 12|0;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $pJfd;
 HEAP32[$4>>2] = $zBuf;
 HEAP32[$5>>2] = $iAmt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $11;
 $12 = HEAP32[$5>>2]|0;
 HEAP32[$nWrite>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 HEAP32[$zWrite>>2] = $13;
 while(1) {
  $14 = HEAP32[$nWrite>>2]|0;
  $15 = ($14|0)>(0);
  if (!($15)) {
   label = 11;
   break;
  }
  $16 = HEAP32[$p>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pChunk>>2] = $19;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = $21;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (___remdi3(($24|0),($27|0),1020,0)|0);
  $29 = tempRet0;
  HEAP32[$iChunkOffset>>2] = $28;
  $30 = HEAP32[$nWrite>>2]|0;
  $31 = HEAP32[$iChunkOffset>>2]|0;
  $32 = (1020 - ($31))|0;
  $33 = ($30|0)<($32|0);
  $34 = HEAP32[$nWrite>>2]|0;
  $35 = HEAP32[$iChunkOffset>>2]|0;
  $36 = (1020 - ($35))|0;
  $37 = $33 ? $34 : $36;
  HEAP32[$iSpace>>2] = $37;
  $38 = HEAP32[$iChunkOffset>>2]|0;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = (_sqlite3_malloc(1024)|0);
   HEAP32[$pNew>>2] = $40;
   $41 = HEAP32[$pNew>>2]|0;
   $42 = ($41|0)!=(0|0);
   if (!($42)) {
    label = 5;
    break;
   }
   $43 = HEAP32[$pNew>>2]|0;
   HEAP32[$43>>2] = 0;
   $44 = HEAP32[$pChunk>>2]|0;
   $45 = ($44|0)!=(0|0);
   $46 = HEAP32[$pNew>>2]|0;
   if ($45) {
    $47 = HEAP32[$pChunk>>2]|0;
    HEAP32[$47>>2] = $46;
   } else {
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 4|0);
    HEAP32[$49>>2] = $46;
   }
   $50 = HEAP32[$pNew>>2]|0;
   $51 = HEAP32[$p>>2]|0;
   $52 = ((($51)) + 8|0);
   $53 = ((($52)) + 8|0);
   HEAP32[$53>>2] = $50;
  }
  $54 = HEAP32[$iChunkOffset>>2]|0;
  $55 = HEAP32[$p>>2]|0;
  $56 = ((($55)) + 8|0);
  $57 = ((($56)) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ((($58)) + 4|0);
  $60 = (($59) + ($54)|0);
  $61 = HEAP32[$zWrite>>2]|0;
  $62 = HEAP32[$iSpace>>2]|0;
  _memcpy(($60|0),($61|0),($62|0))|0;
  $63 = HEAP32[$iSpace>>2]|0;
  $64 = HEAP32[$zWrite>>2]|0;
  $65 = (($64) + ($63)|0);
  HEAP32[$zWrite>>2] = $65;
  $66 = HEAP32[$iSpace>>2]|0;
  $67 = HEAP32[$nWrite>>2]|0;
  $68 = (($67) - ($66))|0;
  HEAP32[$nWrite>>2] = $68;
  $69 = HEAP32[$iSpace>>2]|0;
  $70 = ($69|0)<(0);
  $71 = $70 << 31 >> 31;
  $72 = HEAP32[$p>>2]|0;
  $73 = ((($72)) + 8|0);
  $74 = $73;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($74) + 4)|0;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = (_i64Add(($76|0),($79|0),($69|0),($71|0))|0);
  $81 = tempRet0;
  $82 = $73;
  $83 = $82;
  HEAP32[$83>>2] = $80;
  $84 = (($82) + 4)|0;
  $85 = $84;
  HEAP32[$85>>2] = $81;
 }
 if ((label|0) == 5) {
  HEAP32[$2>>2] = 3082;
  $86 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$2>>2] = 0;
  $86 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 return (0)|0;
}
function _memjrnlTruncate($pJfd,$0,$1) {
 $pJfd = $pJfd|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pChunk = 0, $pTmp = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $p = sp + 16|0;
 $pChunk = sp + 12|0;
 $pTmp = sp + 8|0;
 HEAP32[$2>>2] = $pJfd;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pChunk>>2] = $11;
 while(1) {
  $12 = HEAP32[$pChunk>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pChunk>>2]|0;
  HEAP32[$pTmp>>2] = $14;
  $15 = HEAP32[$pChunk>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pChunk>>2] = $16;
  $17 = HEAP32[$pTmp>>2]|0;
  _sqlite3_free($17);
 }
 $18 = HEAP32[$2>>2]|0;
 _sqlite3MemJournalOpen($18);
 STACKTOP = sp;return 0;
}
function _memjrnlSync($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _memjrnlFileSize($pJfd,$pSize) {
 $pJfd = $pJfd|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pJfd;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3Put4byte($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $v;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 >>> 24;
 $4 = $3&255;
 $5 = HEAP32[$0>>2]|0;
 HEAP8[$5>>0] = $4;
 $6 = HEAP32[$1>>2]|0;
 $7 = $6 >>> 16;
 $8 = $7&255;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 1|0);
 HEAP8[$10>>0] = $8;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11 >>> 8;
 $13 = $12&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 2|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$1>>2]|0;
 $17 = $16&255;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 3|0);
 HEAP8[$19>>0] = $17;
 STACKTOP = sp;return;
}
function _sqlite3BitvecSet($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiValues = 0, $bin = 0, $h = 0, $j = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $h = sp + 16|0;
 $bin = sp + 12|0;
 $j = sp + 8|0;
 $rc = sp + 4|0;
 $aiValues = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$2>>2] = $6;
 while(1) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8>>>0)>(4000);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($14>>>0) / ($17>>>0))&-1;
  HEAP32[$bin>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19>>>0) % ($22>>>0))&-1;
  HEAP32[$2>>2] = $23;
  $24 = HEAP32[$bin>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 12|0);
  $27 = (($26) + ($24<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 8|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3BitvecCreate($32)|0);
   $34 = HEAP32[$bin>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = (($36) + ($34<<2)|0);
   HEAP32[$37>>2] = $33;
   $38 = HEAP32[$bin>>2]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 12|0);
   $41 = (($40) + ($38<<2)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==(0|0);
   if ($43) {
    label = 8;
    break;
   }
  }
  $44 = HEAP32[$bin>>2]|0;
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 12|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$1>>2] = $48;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 7;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50>>>0)<=(4000);
 $52 = HEAP32[$2>>2]|0;
 if ($51) {
  $53 = $52 & 7;
  $54 = 1 << $53;
  $55 = HEAP32[$2>>2]|0;
  $56 = (($55>>>0) / 8)&-1;
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = (($58) + ($56)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = $61 | $54;
  $63 = $62&255;
  HEAP8[$59>>0] = $63;
  HEAP32[$0>>2] = 0;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $64 = (($52) + 1)|0;
 HEAP32[$2>>2] = $64;
 $65 = $52;
 $66 = (($65>>>0) % 125)&-1;
 HEAP32[$h>>2] = $66;
 $67 = HEAP32[$h>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = ((($68)) + 12|0);
 $70 = (($69) + ($67<<2)|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0);
 L20: do {
  if ($72) {
   while(1) {
    $77 = HEAP32[$h>>2]|0;
    $78 = HEAP32[$1>>2]|0;
    $79 = ((($78)) + 12|0);
    $80 = (($79) + ($77<<2)|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = HEAP32[$2>>2]|0;
    $83 = ($81|0)==($82|0);
    if ($83) {
     break;
    }
    $84 = HEAP32[$h>>2]|0;
    $85 = (($84) + 1)|0;
    HEAP32[$h>>2] = $85;
    $86 = HEAP32[$h>>2]|0;
    $87 = ($86>>>0)>=(125);
    $$ = $87 ? 0 : $85;
    HEAP32[$h>>2] = $$;
    $88 = HEAP32[$h>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 12|0);
    $91 = (($90) + ($88<<2)|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)!=(0);
    if (!($93)) {
     label = 17;
     break L20;
    }
   }
   HEAP32[$0>>2] = 0;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  } else {
   $73 = HEAP32[$1>>2]|0;
   $74 = ((($73)) + 4|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75>>>0)<(124);
   if (!($76)) {
    label = 17;
   }
  }
 } while(0);
 if ((label|0) == 17) {
  $94 = HEAP32[$1>>2]|0;
  $95 = ((($94)) + 4|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96>>>0)>=(62);
  if ($97) {
   $98 = (_sqlite3DbMallocRaw(0,500,0)|0);
   HEAP32[$aiValues>>2] = $98;
   $99 = HEAP32[$aiValues>>2]|0;
   $100 = ($99|0)==(0|0);
   if ($100) {
    HEAP32[$0>>2] = 7;
    $144 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($144|0);
   }
   $101 = HEAP32[$aiValues>>2]|0;
   $102 = HEAP32[$1>>2]|0;
   $103 = ((($102)) + 12|0);
   _memcpy(($101|0),($103|0),500)|0;
   $104 = HEAP32[$1>>2]|0;
   $105 = ((($104)) + 12|0);
   _memset(($105|0),0,500)|0;
   $106 = HEAP32[$1>>2]|0;
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + 125)|0;
   $109 = (($108) - 1)|0;
   $110 = (($109>>>0) / 125)&-1;
   $111 = HEAP32[$1>>2]|0;
   $112 = ((($111)) + 8|0);
   HEAP32[$112>>2] = $110;
   $113 = HEAP32[$1>>2]|0;
   $114 = HEAP32[$2>>2]|0;
   $115 = (_sqlite3BitvecSet($113,$114)|0);
   HEAP32[$rc>>2] = $115;
   HEAP32[$j>>2] = 0;
   while(1) {
    $116 = HEAP32[$j>>2]|0;
    $117 = ($116>>>0)<(125);
    if (!($117)) {
     break;
    }
    $118 = HEAP32[$j>>2]|0;
    $119 = HEAP32[$aiValues>>2]|0;
    $120 = (($119) + ($118<<2)|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = ($121|0)!=(0);
    if ($122) {
     $123 = HEAP32[$1>>2]|0;
     $124 = HEAP32[$j>>2]|0;
     $125 = HEAP32[$aiValues>>2]|0;
     $126 = (($125) + ($124<<2)|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = (_sqlite3BitvecSet($123,$127)|0);
     $129 = HEAP32[$rc>>2]|0;
     $130 = $129 | $128;
     HEAP32[$rc>>2] = $130;
    }
    $131 = HEAP32[$j>>2]|0;
    $132 = (($131) + 1)|0;
    HEAP32[$j>>2] = $132;
   }
   $133 = HEAP32[$aiValues>>2]|0;
   _sqlite3DbFree(0,$133);
   $134 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $134;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  }
 }
 $135 = HEAP32[$1>>2]|0;
 $136 = ((($135)) + 4|0);
 $137 = HEAP32[$136>>2]|0;
 $138 = (($137) + 1)|0;
 HEAP32[$136>>2] = $138;
 $139 = HEAP32[$2>>2]|0;
 $140 = HEAP32[$h>>2]|0;
 $141 = HEAP32[$1>>2]|0;
 $142 = ((($141)) + 12|0);
 $143 = (($142) + ($140<<2)|0);
 HEAP32[$143>>2] = $139;
 HEAP32[$0>>2] = 0;
 $144 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($144|0);
}
function _sqlite3BitvecCreate($iSize) {
 $iSize = $iSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $iSize;
 $1 = (_sqlite3MallocZero(512,0)|0);
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $6 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _pager_write_changecounter($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $change_counter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $change_counter = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 112|0);
 $5 = (_sqlite3Get4byte($4)|0);
 $6 = (($5) + 1)|0;
 HEAP32[$change_counter>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$change_counter>>2]|0;
 _sqlite3Put4byte($10,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 92|0);
 $16 = HEAP32[$change_counter>>2]|0;
 _sqlite3Put4byte($15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 96|0);
 _sqlite3Put4byte($20,3011000);
 STACKTOP = sp;return;
}
function _sqlite3WalFrames($pWal,$szPage,$pList,$nTruncate,$isCommit,$sync_flags) {
 $pWal = $pWal|0;
 $szPage = $szPage|0;
 $pList = $pList|0;
 $nTruncate = $nTruncate|0;
 $isCommit = $isCommit|0;
 $sync_flags = $sync_flags|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCksum = 0, $aWalHdr = 0, $iFirst = 0, $iFrame = 0;
 var $iOff = 0, $iOffset = 0, $iWrite = 0, $nDbSize = 0, $nExtra = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pData = 0, $pLast = 0, $pLive = 0, $rc = 0, $sectorSize = 0, $sz = 0, $szFrame = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 112|0;
 $5 = sp + 108|0;
 $6 = sp + 104|0;
 $rc = sp + 100|0;
 $iFrame = sp + 96|0;
 $p = sp + 92|0;
 $pLast = sp + 88|0;
 $nExtra = sp + 84|0;
 $szFrame = sp + 80|0;
 $iOffset = sp + 40|0;
 $w = sp + 16|0;
 $iFirst = sp + 76|0;
 $pLive = sp + 72|0;
 $aWalHdr = sp + 136|0;
 $aCksum = sp + 64|0;
 $nDbSize = sp + 60|0;
 $iWrite = sp + 56|0;
 $iOff = sp + 8|0;
 $pData = sp + 52|0;
 $sectorSize = sp + 48|0;
 $sz = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $szPage;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nTruncate;
 HEAP32[$5>>2] = $isCommit;
 HEAP32[$6>>2] = $sync_flags;
 HEAP32[$pLast>>2] = 0;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$iFirst>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_walIndexHdr($7)|0);
 HEAP32[$pLive>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$pLive>>2]|0;
 $12 = (_memcmp($10,$11,48)|0);
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$pLive>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$iFirst>>2] = $17;
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = (_walRestartLog($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = (0)!=($19|0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $514 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($514|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 52|0);
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$iFrame>>2] = $25;
 $26 = HEAP32[$iFrame>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP8[352>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)==(0);
  $31 = $30&1;
  $32 = 931071618 | $31;
  _sqlite3Put4byte($aWalHdr,$32);
  $33 = ((($aWalHdr)) + 4|0);
  _sqlite3Put4byte($33,3007000);
  $34 = ((($aWalHdr)) + 8|0);
  $35 = HEAP32[$2>>2]|0;
  _sqlite3Put4byte($34,$35);
  $36 = ((($aWalHdr)) + 12|0);
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 112|0);
  $39 = HEAP32[$38>>2]|0;
  _sqlite3Put4byte($36,$39);
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 112|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$1>>2]|0;
   $45 = ((($44)) + 52|0);
   $46 = ((($45)) + 32|0);
   _sqlite3_randomness(8,$46);
  }
  $47 = ((($aWalHdr)) + 16|0);
  $48 = HEAP32[$1>>2]|0;
  $49 = ((($48)) + 52|0);
  $50 = ((($49)) + 32|0);
  ;HEAP8[$47>>0]=HEAP8[$50>>0]|0;HEAP8[$47+1>>0]=HEAP8[$50+1>>0]|0;HEAP8[$47+2>>0]=HEAP8[$50+2>>0]|0;HEAP8[$47+3>>0]=HEAP8[$50+3>>0]|0;HEAP8[$47+4>>0]=HEAP8[$50+4>>0]|0;HEAP8[$47+5>>0]=HEAP8[$50+5>>0]|0;HEAP8[$47+6>>0]=HEAP8[$50+6>>0]|0;HEAP8[$47+7>>0]=HEAP8[$50+7>>0]|0;
  _walChecksumBytes(1,$aWalHdr,24,0,$aCksum);
  $51 = ((($aWalHdr)) + 24|0);
  $52 = HEAP32[$aCksum>>2]|0;
  _sqlite3Put4byte($51,$52);
  $53 = ((($aWalHdr)) + 28|0);
  $54 = ((($aCksum)) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  _sqlite3Put4byte($53,$55);
  $56 = HEAP32[$2>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 36|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP8[352>>0]|0;
  $60 = $59 << 24 >> 24;
  $61 = ($60|0)==(0);
  $62 = $61&1;
  $63 = $62&255;
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 52|0);
  $66 = ((($65)) + 13|0);
  HEAP8[$66>>0] = $63;
  $67 = HEAP32[$aCksum>>2]|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = ((($68)) + 52|0);
  $70 = ((($69)) + 24|0);
  HEAP32[$70>>2] = $67;
  $71 = ((($aCksum)) + 4|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 52|0);
  $75 = ((($74)) + 24|0);
  $76 = ((($75)) + 4|0);
  HEAP32[$76>>2] = $72;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 47|0);
  HEAP8[$78>>0] = 1;
  $79 = HEAP32[$1>>2]|0;
  $80 = ((($79)) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (_sqlite3OsWrite($81,$aWalHdr,32,0,0)|0);
  HEAP32[$rc>>2] = $82;
  $83 = HEAP32[$rc>>2]|0;
  $84 = ($83|0)!=(0);
  if ($84) {
   $85 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $85;
   $514 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($514|0);
  }
  $86 = HEAP32[$1>>2]|0;
  $87 = ((($86)) + 48|0);
  $88 = HEAP8[$87>>0]|0;
  $89 = $88&255;
  $90 = ($89|0)!=(0);
  $91 = HEAP32[$6>>2]|0;
  $92 = ($91|0)!=(0);
  $or$cond = $90 & $92;
  if ($or$cond) {
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 8|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = HEAP32[$6>>2]|0;
   $97 = $96 & 19;
   $98 = (_sqlite3OsSync($95,$97)|0);
   HEAP32[$rc>>2] = $98;
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)!=(0);
   if ($100) {
    $101 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $101;
    $514 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($514|0);
   }
  }
 }
 $102 = HEAP32[$1>>2]|0;
 HEAP32[$w>>2] = $102;
 $103 = HEAP32[$1>>2]|0;
 $104 = ((($103)) + 8|0);
 $105 = HEAP32[$104>>2]|0;
 $106 = ((($w)) + 4|0);
 HEAP32[$106>>2] = $105;
 $107 = ((($w)) + 8|0);
 $108 = $107;
 $109 = $108;
 HEAP32[$109>>2] = 0;
 $110 = (($108) + 4)|0;
 $111 = $110;
 HEAP32[$111>>2] = 0;
 $112 = HEAP32[$6>>2]|0;
 $113 = ((($w)) + 16|0);
 HEAP32[$113>>2] = $112;
 $114 = HEAP32[$2>>2]|0;
 $115 = ((($w)) + 20|0);
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$iFrame>>2]|0;
 $117 = (($116) + 1)|0;
 $118 = (($117) - 1)|0;
 $119 = HEAP32[$2>>2]|0;
 $120 = (($119) + 24)|0;
 $121 = ($120|0)<(0);
 $122 = $121 << 31 >> 31;
 $123 = (___muldi3(($118|0),0,($120|0),($122|0))|0);
 $124 = tempRet0;
 $125 = (_i64Add(32,0,($123|0),($124|0))|0);
 $126 = tempRet0;
 $127 = $iOffset;
 $128 = $127;
 HEAP32[$128>>2] = $125;
 $129 = (($127) + 4)|0;
 $130 = $129;
 HEAP32[$130>>2] = $126;
 $131 = HEAP32[$2>>2]|0;
 $132 = (($131) + 24)|0;
 HEAP32[$szFrame>>2] = $132;
 $133 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $133;
 while(1) {
  $134 = HEAP32[$p>>2]|0;
  $135 = ($134|0)!=(0|0);
  if (!($135)) {
   label = 30;
   break;
  }
  $136 = HEAP32[$iFirst>>2]|0;
  $137 = ($136|0)!=(0);
  if ($137) {
   $138 = HEAP32[$p>>2]|0;
   $139 = ((($138)) + 12|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = ($140|0)!=(0|0);
   $142 = HEAP32[$5>>2]|0;
   $143 = ($142|0)==(0);
   $or$cond3 = $141 | $143;
   if ($or$cond3) {
    HEAP32[$iWrite>>2] = 0;
    $144 = HEAP32[$1>>2]|0;
    $145 = HEAP32[$p>>2]|0;
    $146 = ((($145)) + 20|0);
    $147 = HEAP32[$146>>2]|0;
    (_sqlite3WalFindFrame($144,$147,$iWrite)|0);
    $148 = HEAP32[$iWrite>>2]|0;
    $149 = HEAP32[$iFirst>>2]|0;
    $150 = ($148>>>0)>=($149>>>0);
    if ($150) {
     $151 = HEAP32[$iWrite>>2]|0;
     $152 = (($151) - 1)|0;
     $153 = HEAP32[$2>>2]|0;
     $154 = (($153) + 24)|0;
     $155 = ($154|0)<(0);
     $156 = $155 << 31 >> 31;
     $157 = (___muldi3(($152|0),0,($154|0),($156|0))|0);
     $158 = tempRet0;
     $159 = (_i64Add(32,0,($157|0),($158|0))|0);
     $160 = tempRet0;
     $161 = (_i64Add(($159|0),($160|0),24,0)|0);
     $162 = tempRet0;
     $163 = $iOff;
     $164 = $163;
     HEAP32[$164>>2] = $161;
     $165 = (($163) + 4)|0;
     $166 = $165;
     HEAP32[$166>>2] = $162;
     $167 = HEAP32[$1>>2]|0;
     $168 = ((($167)) + 104|0);
     $169 = HEAP32[$168>>2]|0;
     $170 = ($169|0)==(0);
     if ($170) {
      label = 20;
     } else {
      $171 = HEAP32[$iWrite>>2]|0;
      $172 = HEAP32[$1>>2]|0;
      $173 = ((($172)) + 104|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = ($171>>>0)<($174>>>0);
      if ($175) {
       label = 20;
      }
     }
     if ((label|0) == 20) {
      label = 0;
      $176 = HEAP32[$iWrite>>2]|0;
      $177 = HEAP32[$1>>2]|0;
      $178 = ((($177)) + 104|0);
      HEAP32[$178>>2] = $176;
     }
     $179 = HEAP32[$p>>2]|0;
     $180 = ((($179)) + 4|0);
     $181 = HEAP32[$180>>2]|0;
     HEAP32[$pData>>2] = $181;
     $182 = HEAP32[$1>>2]|0;
     $183 = ((($182)) + 8|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = HEAP32[$pData>>2]|0;
     $186 = HEAP32[$2>>2]|0;
     $187 = $iOff;
     $188 = $187;
     $189 = HEAP32[$188>>2]|0;
     $190 = (($187) + 4)|0;
     $191 = $190;
     $192 = HEAP32[$191>>2]|0;
     $193 = (_sqlite3OsWrite($184,$185,$186,$189,$192)|0);
     HEAP32[$rc>>2] = $193;
     $194 = HEAP32[$rc>>2]|0;
     $195 = ($194|0)!=(0);
     if ($195) {
      label = 22;
      break;
     }
     $197 = HEAP32[$p>>2]|0;
     $198 = ((($197)) + 24|0);
     $199 = HEAP16[$198>>1]|0;
     $200 = $199&65535;
     $201 = $200 & -129;
     $202 = $201&65535;
     HEAP16[$198>>1] = $202;
    } else {
     label = 24;
    }
   } else {
    label = 24;
   }
  } else {
   label = 24;
  }
  if ((label|0) == 24) {
   label = 0;
   $203 = HEAP32[$iFrame>>2]|0;
   $204 = (($203) + 1)|0;
   HEAP32[$iFrame>>2] = $204;
   $205 = HEAP32[$5>>2]|0;
   $206 = ($205|0)!=(0);
   if ($206) {
    $207 = HEAP32[$p>>2]|0;
    $208 = ((($207)) + 12|0);
    $209 = HEAP32[$208>>2]|0;
    $210 = ($209|0)==(0|0);
    $211 = HEAP32[$4>>2]|0;
    $$ = $210 ? $211 : 0;
    $212 = $$;
   } else {
    $212 = 0;
   }
   HEAP32[$nDbSize>>2] = $212;
   $213 = HEAP32[$p>>2]|0;
   $214 = HEAP32[$nDbSize>>2]|0;
   $215 = $iOffset;
   $216 = $215;
   $217 = HEAP32[$216>>2]|0;
   $218 = (($215) + 4)|0;
   $219 = $218;
   $220 = HEAP32[$219>>2]|0;
   $221 = (_walWriteOneFrame($w,$213,$214,$217,$220)|0);
   HEAP32[$rc>>2] = $221;
   $222 = HEAP32[$rc>>2]|0;
   $223 = ($222|0)!=(0);
   if ($223) {
    label = 27;
    break;
   }
   $225 = HEAP32[$p>>2]|0;
   HEAP32[$pLast>>2] = $225;
   $226 = HEAP32[$szFrame>>2]|0;
   $227 = ($226|0)<(0);
   $228 = $227 << 31 >> 31;
   $229 = $iOffset;
   $230 = $229;
   $231 = HEAP32[$230>>2]|0;
   $232 = (($229) + 4)|0;
   $233 = $232;
   $234 = HEAP32[$233>>2]|0;
   $235 = (_i64Add(($231|0),($234|0),($226|0),($228|0))|0);
   $236 = tempRet0;
   $237 = $iOffset;
   $238 = $237;
   HEAP32[$238>>2] = $235;
   $239 = (($237) + 4)|0;
   $240 = $239;
   HEAP32[$240>>2] = $236;
   $241 = HEAP32[$p>>2]|0;
   $242 = ((($241)) + 24|0);
   $243 = HEAP16[$242>>1]|0;
   $244 = $243&65535;
   $245 = $244 | 128;
   $246 = $245&65535;
   HEAP16[$242>>1] = $246;
  }
  $247 = HEAP32[$p>>2]|0;
  $248 = ((($247)) + 12|0);
  $249 = HEAP32[$248>>2]|0;
  HEAP32[$p>>2] = $249;
 }
 if ((label|0) == 22) {
  $196 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $196;
  $514 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($514|0);
 }
 else if ((label|0) == 27) {
  $224 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $224;
  $514 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($514|0);
 }
 else if ((label|0) == 30) {
  $250 = HEAP32[$5>>2]|0;
  $251 = ($250|0)!=(0);
  if ($251) {
   $252 = HEAP32[$1>>2]|0;
   $253 = ((($252)) + 104|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = ($254|0)!=(0);
   if ($255) {
    $256 = HEAP32[$1>>2]|0;
    $257 = HEAP32[$iFrame>>2]|0;
    $258 = (_walRewriteChecksums($256,$257)|0);
    HEAP32[$rc>>2] = $258;
    $259 = HEAP32[$rc>>2]|0;
    $260 = ($259|0)!=(0);
    if ($260) {
     $261 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $261;
     $514 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($514|0);
    }
   }
  }
  $262 = HEAP32[$5>>2]|0;
  $263 = ($262|0)!=(0);
  L49: do {
   if ($263) {
    $264 = HEAP32[$6>>2]|0;
    $265 = $264 & 32;
    $266 = ($265|0)!=(0);
    if ($266) {
     $267 = HEAP32[$1>>2]|0;
     $268 = ((($267)) + 49|0);
     $269 = HEAP8[$268>>0]|0;
     $270 = ($269<<24>>24)!=(0);
     if (!($270)) {
      $350 = ((($w)) + 4|0);
      $351 = HEAP32[$350>>2]|0;
      $352 = HEAP32[$6>>2]|0;
      $353 = $352 & 19;
      $354 = (_sqlite3OsSync($351,$353)|0);
      HEAP32[$rc>>2] = $354;
      break;
     }
     $271 = HEAP32[$1>>2]|0;
     $272 = ((($271)) + 8|0);
     $273 = HEAP32[$272>>2]|0;
     $274 = (_sqlite3SectorSize($273)|0);
     HEAP32[$sectorSize>>2] = $274;
     $275 = $iOffset;
     $276 = $275;
     $277 = HEAP32[$276>>2]|0;
     $278 = (($275) + 4)|0;
     $279 = $278;
     $280 = HEAP32[$279>>2]|0;
     $281 = HEAP32[$sectorSize>>2]|0;
     $282 = ($281|0)<(0);
     $283 = $282 << 31 >> 31;
     $284 = (_i64Add(($277|0),($280|0),($281|0),($283|0))|0);
     $285 = tempRet0;
     $286 = (_i64Subtract(($284|0),($285|0),1,0)|0);
     $287 = tempRet0;
     $288 = HEAP32[$sectorSize>>2]|0;
     $289 = ($288|0)<(0);
     $290 = $289 << 31 >> 31;
     $291 = (___divdi3(($286|0),($287|0),($288|0),($290|0))|0);
     $292 = tempRet0;
     $293 = HEAP32[$sectorSize>>2]|0;
     $294 = ($293|0)<(0);
     $295 = $294 << 31 >> 31;
     $296 = (___muldi3(($291|0),($292|0),($293|0),($295|0))|0);
     $297 = tempRet0;
     $298 = ((($w)) + 8|0);
     $299 = $298;
     $300 = $299;
     HEAP32[$300>>2] = $296;
     $301 = (($299) + 4)|0;
     $302 = $301;
     HEAP32[$302>>2] = $297;
     while(1) {
      $303 = $iOffset;
      $304 = $303;
      $305 = HEAP32[$304>>2]|0;
      $306 = (($303) + 4)|0;
      $307 = $306;
      $308 = HEAP32[$307>>2]|0;
      $309 = ((($w)) + 8|0);
      $310 = $309;
      $311 = $310;
      $312 = HEAP32[$311>>2]|0;
      $313 = (($310) + 4)|0;
      $314 = $313;
      $315 = HEAP32[$314>>2]|0;
      $316 = ($308|0)<($315|0);
      $317 = ($305>>>0)<($312>>>0);
      $318 = ($308|0)==($315|0);
      $319 = $318 & $317;
      $320 = $316 | $319;
      if (!($320)) {
       break L49;
      }
      $321 = HEAP32[$pLast>>2]|0;
      $322 = HEAP32[$4>>2]|0;
      $323 = $iOffset;
      $324 = $323;
      $325 = HEAP32[$324>>2]|0;
      $326 = (($323) + 4)|0;
      $327 = $326;
      $328 = HEAP32[$327>>2]|0;
      $329 = (_walWriteOneFrame($w,$321,$322,$325,$328)|0);
      HEAP32[$rc>>2] = $329;
      $330 = HEAP32[$rc>>2]|0;
      $331 = ($330|0)!=(0);
      if ($331) {
       break;
      }
      $333 = HEAP32[$szFrame>>2]|0;
      $334 = ($333|0)<(0);
      $335 = $334 << 31 >> 31;
      $336 = $iOffset;
      $337 = $336;
      $338 = HEAP32[$337>>2]|0;
      $339 = (($336) + 4)|0;
      $340 = $339;
      $341 = HEAP32[$340>>2]|0;
      $342 = (_i64Add(($338|0),($341|0),($333|0),($335|0))|0);
      $343 = tempRet0;
      $344 = $iOffset;
      $345 = $344;
      HEAP32[$345>>2] = $342;
      $346 = (($344) + 4)|0;
      $347 = $346;
      HEAP32[$347>>2] = $343;
      $348 = HEAP32[$nExtra>>2]|0;
      $349 = (($348) + 1)|0;
      HEAP32[$nExtra>>2] = $349;
     }
     $332 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $332;
     $514 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($514|0);
    }
   }
  } while(0);
  $355 = HEAP32[$5>>2]|0;
  $356 = ($355|0)!=(0);
  if ($356) {
   $357 = HEAP32[$1>>2]|0;
   $358 = ((($357)) + 47|0);
   $359 = HEAP8[$358>>0]|0;
   $360 = $359&255;
   $361 = ($360|0)!=(0);
   if ($361) {
    $362 = HEAP32[$1>>2]|0;
    $363 = ((($362)) + 16|0);
    $364 = $363;
    $365 = $364;
    $366 = HEAP32[$365>>2]|0;
    $367 = (($364) + 4)|0;
    $368 = $367;
    $369 = HEAP32[$368>>2]|0;
    $370 = ($369|0)>(0);
    $371 = ($366>>>0)>=(0);
    $372 = ($369|0)==(0);
    $373 = $372 & $371;
    $374 = $370 | $373;
    if ($374) {
     $375 = HEAP32[$1>>2]|0;
     $376 = ((($375)) + 16|0);
     $377 = $376;
     $378 = $377;
     $379 = HEAP32[$378>>2]|0;
     $380 = (($377) + 4)|0;
     $381 = $380;
     $382 = HEAP32[$381>>2]|0;
     $383 = $sz;
     $384 = $383;
     HEAP32[$384>>2] = $379;
     $385 = (($383) + 4)|0;
     $386 = $385;
     HEAP32[$386>>2] = $382;
     $387 = HEAP32[$iFrame>>2]|0;
     $388 = HEAP32[$nExtra>>2]|0;
     $389 = (($387) + ($388))|0;
     $390 = (($389) + 1)|0;
     $391 = (($390) - 1)|0;
     $392 = HEAP32[$2>>2]|0;
     $393 = (($392) + 24)|0;
     $394 = ($393|0)<(0);
     $395 = $394 << 31 >> 31;
     $396 = (___muldi3(($391|0),0,($393|0),($395|0))|0);
     $397 = tempRet0;
     $398 = (_i64Add(32,0,($396|0),($397|0))|0);
     $399 = tempRet0;
     $400 = HEAP32[$1>>2]|0;
     $401 = ((($400)) + 16|0);
     $402 = $401;
     $403 = $402;
     $404 = HEAP32[$403>>2]|0;
     $405 = (($402) + 4)|0;
     $406 = $405;
     $407 = HEAP32[$406>>2]|0;
     $408 = ($399|0)>($407|0);
     $409 = ($398>>>0)>($404>>>0);
     $410 = ($399|0)==($407|0);
     $411 = $410 & $409;
     $412 = $408 | $411;
     if ($412) {
      $413 = HEAP32[$iFrame>>2]|0;
      $414 = HEAP32[$nExtra>>2]|0;
      $415 = (($413) + ($414))|0;
      $416 = (($415) + 1)|0;
      $417 = (($416) - 1)|0;
      $418 = HEAP32[$2>>2]|0;
      $419 = (($418) + 24)|0;
      $420 = ($419|0)<(0);
      $421 = $420 << 31 >> 31;
      $422 = (___muldi3(($417|0),0,($419|0),($421|0))|0);
      $423 = tempRet0;
      $424 = (_i64Add(32,0,($422|0),($423|0))|0);
      $425 = tempRet0;
      $426 = $sz;
      $427 = $426;
      HEAP32[$427>>2] = $424;
      $428 = (($426) + 4)|0;
      $429 = $428;
      HEAP32[$429>>2] = $425;
     }
     $430 = HEAP32[$1>>2]|0;
     $431 = $sz;
     $432 = $431;
     $433 = HEAP32[$432>>2]|0;
     $434 = (($431) + 4)|0;
     $435 = $434;
     $436 = HEAP32[$435>>2]|0;
     _walLimitSize($430,$433,$436);
     $437 = HEAP32[$1>>2]|0;
     $438 = ((($437)) + 47|0);
     HEAP8[$438>>0] = 0;
    }
   }
  }
  $439 = HEAP32[$1>>2]|0;
  $440 = ((($439)) + 52|0);
  $441 = ((($440)) + 16|0);
  $442 = HEAP32[$441>>2]|0;
  HEAP32[$iFrame>>2] = $442;
  $443 = HEAP32[$3>>2]|0;
  HEAP32[$p>>2] = $443;
  while(1) {
   $444 = HEAP32[$p>>2]|0;
   $445 = ($444|0)!=(0|0);
   $446 = HEAP32[$rc>>2]|0;
   $447 = ($446|0)==(0);
   $448 = $445 ? $447 : 0;
   if (!($448)) {
    break;
   }
   $449 = HEAP32[$p>>2]|0;
   $450 = ((($449)) + 24|0);
   $451 = HEAP16[$450>>1]|0;
   $452 = $451&65535;
   $453 = $452 & 128;
   $454 = ($453|0)==(0);
   if (!($454)) {
    $455 = HEAP32[$iFrame>>2]|0;
    $456 = (($455) + 1)|0;
    HEAP32[$iFrame>>2] = $456;
    $457 = HEAP32[$1>>2]|0;
    $458 = HEAP32[$iFrame>>2]|0;
    $459 = HEAP32[$p>>2]|0;
    $460 = ((($459)) + 20|0);
    $461 = HEAP32[$460>>2]|0;
    $462 = (_walIndexAppend($457,$458,$461)|0);
    HEAP32[$rc>>2] = $462;
   }
   $463 = HEAP32[$p>>2]|0;
   $464 = ((($463)) + 12|0);
   $465 = HEAP32[$464>>2]|0;
   HEAP32[$p>>2] = $465;
  }
  while(1) {
   $466 = HEAP32[$rc>>2]|0;
   $467 = ($466|0)==(0);
   $468 = HEAP32[$nExtra>>2]|0;
   $469 = ($468|0)>(0);
   $470 = $467 ? $469 : 0;
   if (!($470)) {
    break;
   }
   $471 = HEAP32[$iFrame>>2]|0;
   $472 = (($471) + 1)|0;
   HEAP32[$iFrame>>2] = $472;
   $473 = HEAP32[$nExtra>>2]|0;
   $474 = (($473) + -1)|0;
   HEAP32[$nExtra>>2] = $474;
   $475 = HEAP32[$1>>2]|0;
   $476 = HEAP32[$iFrame>>2]|0;
   $477 = HEAP32[$pLast>>2]|0;
   $478 = ((($477)) + 20|0);
   $479 = HEAP32[$478>>2]|0;
   $480 = (_walIndexAppend($475,$476,$479)|0);
   HEAP32[$rc>>2] = $480;
  }
  $481 = HEAP32[$rc>>2]|0;
  $482 = ($481|0)==(0);
  if ($482) {
   $483 = HEAP32[$2>>2]|0;
   $484 = $483 & 65280;
   $485 = HEAP32[$2>>2]|0;
   $486 = $485 >> 16;
   $487 = $484 | $486;
   $488 = $487&65535;
   $489 = HEAP32[$1>>2]|0;
   $490 = ((($489)) + 52|0);
   $491 = ((($490)) + 14|0);
   HEAP16[$491>>1] = $488;
   $492 = HEAP32[$iFrame>>2]|0;
   $493 = HEAP32[$1>>2]|0;
   $494 = ((($493)) + 52|0);
   $495 = ((($494)) + 16|0);
   HEAP32[$495>>2] = $492;
   $496 = HEAP32[$5>>2]|0;
   $497 = ($496|0)!=(0);
   if ($497) {
    $498 = HEAP32[$1>>2]|0;
    $499 = ((($498)) + 52|0);
    $500 = ((($499)) + 8|0);
    $501 = HEAP32[$500>>2]|0;
    $502 = (($501) + 1)|0;
    HEAP32[$500>>2] = $502;
    $503 = HEAP32[$4>>2]|0;
    $504 = HEAP32[$1>>2]|0;
    $505 = ((($504)) + 52|0);
    $506 = ((($505)) + 20|0);
    HEAP32[$506>>2] = $503;
   }
   $507 = HEAP32[$5>>2]|0;
   $508 = ($507|0)!=(0);
   if ($508) {
    $509 = HEAP32[$1>>2]|0;
    _walIndexWriteHdr($509);
    $510 = HEAP32[$iFrame>>2]|0;
    $511 = HEAP32[$1>>2]|0;
    $512 = ((($511)) + 12|0);
    HEAP32[$512>>2] = $510;
   }
  }
  $513 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $513;
  $514 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($514|0);
 }
 return (0)|0;
}
function _sqlite3BackupUpdate($pBackup,$iPage,$aData) {
 $pBackup = $pBackup|0;
 $iPage = $iPage|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pBackup;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $aData;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 _backupUpdate($5,$6,$7);
 STACKTOP = sp;return;
}
function _walIndexHdr($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _walRestartLog($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $notUsed = 0, $pInfo = 0, $rc = 0, $salt1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $cnt = sp + 12|0;
 $pInfo = sp + 8|0;
 $salt1 = sp + 4|0;
 $notUsed = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (_walCkptInfo($7)|0);
  HEAP32[$pInfo>>2] = $8;
  $9 = HEAP32[$pInfo>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10>>>0)>(0);
  do {
   if ($11) {
    _sqlite3_randomness(4,$salt1);
    $12 = HEAP32[$1>>2]|0;
    $13 = (_walLockExclusive($12,4,4)|0);
    HEAP32[$rc>>2] = $13;
    $14 = HEAP32[$rc>>2]|0;
    $15 = ($14|0)==(0);
    if ($15) {
     $16 = HEAP32[$1>>2]|0;
     $17 = HEAP32[$salt1>>2]|0;
     _walRestartHdr($16,$17);
     $18 = HEAP32[$1>>2]|0;
     _walUnlockExclusive($18,4,4);
     break;
    }
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)!=(5);
    if ($20) {
     $21 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $21;
     $32 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($32|0);
    }
   }
  } while(0);
  $22 = HEAP32[$1>>2]|0;
  _walUnlockShared($22,3);
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 40|0);
  HEAP16[$24>>1] = -1;
  HEAP32[$cnt>>2] = 0;
  while(1) {
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$cnt>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$cnt>>2] = $27;
   $28 = (_walTryBeginRead($25,$notUsed,1,$27)|0);
   HEAP32[$rc>>2] = $28;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(-1);
   if (!($30)) {
    break;
   }
  }
 }
 $31 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _walChecksumBytes($nativeCksum,$a,$nByte,$aIn,$aOut) {
 $nativeCksum = $nativeCksum|0;
 $a = $a|0;
 $nByte = $nByte|0;
 $aIn = $aIn|0;
 $aOut = $aOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $aData = 0, $aEnd = 0, $s1 = 0, $s2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $s1 = sp + 12|0;
 $s2 = sp + 8|0;
 $aData = sp + 4|0;
 $aEnd = sp;
 HEAP32[$0>>2] = $nativeCksum;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $aIn;
 HEAP32[$4>>2] = $aOut;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$aData>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + ($6)|0);
 HEAP32[$aEnd>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$s1>>2] = $12;
  $13 = HEAP32[$3>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$s2>>2] = $15;
 } else {
  HEAP32[$s2>>2] = 0;
  HEAP32[$s1>>2] = 0;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  while(1) {
   $18 = HEAP32[$aData>>2]|0;
   $19 = ((($18)) + 4|0);
   HEAP32[$aData>>2] = $19;
   $20 = HEAP32[$18>>2]|0;
   $21 = HEAP32[$s2>>2]|0;
   $22 = (($20) + ($21))|0;
   $23 = HEAP32[$s1>>2]|0;
   $24 = (($23) + ($22))|0;
   HEAP32[$s1>>2] = $24;
   $25 = HEAP32[$aData>>2]|0;
   $26 = ((($25)) + 4|0);
   HEAP32[$aData>>2] = $26;
   $27 = HEAP32[$25>>2]|0;
   $28 = HEAP32[$s1>>2]|0;
   $29 = (($27) + ($28))|0;
   $30 = HEAP32[$s2>>2]|0;
   $31 = (($30) + ($29))|0;
   HEAP32[$s2>>2] = $31;
   $32 = HEAP32[$aData>>2]|0;
   $33 = HEAP32[$aEnd>>2]|0;
   $34 = ($32>>>0)<($33>>>0);
   if (!($34)) {
    break;
   }
  }
  $90 = HEAP32[$s1>>2]|0;
  $91 = HEAP32[$4>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$s2>>2]|0;
  $93 = HEAP32[$4>>2]|0;
  $94 = ((($93)) + 4|0);
  HEAP32[$94>>2] = $92;
  STACKTOP = sp;return;
 } else {
  while(1) {
   $35 = HEAP32[$aData>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $36 & 255;
   $38 = $37 << 24;
   $39 = HEAP32[$aData>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = $40 & 65280;
   $42 = $41 << 8;
   $43 = (($38) + ($42))|0;
   $44 = HEAP32[$aData>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $45 & 16711680;
   $47 = $46 >>> 8;
   $48 = (($43) + ($47))|0;
   $49 = HEAP32[$aData>>2]|0;
   $50 = HEAP32[$49>>2]|0;
   $51 = $50 & -16777216;
   $52 = $51 >>> 24;
   $53 = (($48) + ($52))|0;
   $54 = HEAP32[$s2>>2]|0;
   $55 = (($53) + ($54))|0;
   $56 = HEAP32[$s1>>2]|0;
   $57 = (($56) + ($55))|0;
   HEAP32[$s1>>2] = $57;
   $58 = HEAP32[$aData>>2]|0;
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $60 & 255;
   $62 = $61 << 24;
   $63 = HEAP32[$aData>>2]|0;
   $64 = ((($63)) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 & 65280;
   $67 = $66 << 8;
   $68 = (($62) + ($67))|0;
   $69 = HEAP32[$aData>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = $71 & 16711680;
   $73 = $72 >>> 8;
   $74 = (($68) + ($73))|0;
   $75 = HEAP32[$aData>>2]|0;
   $76 = ((($75)) + 4|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = $77 & -16777216;
   $79 = $78 >>> 24;
   $80 = (($74) + ($79))|0;
   $81 = HEAP32[$s1>>2]|0;
   $82 = (($80) + ($81))|0;
   $83 = HEAP32[$s2>>2]|0;
   $84 = (($83) + ($82))|0;
   HEAP32[$s2>>2] = $84;
   $85 = HEAP32[$aData>>2]|0;
   $86 = ((($85)) + 8|0);
   HEAP32[$aData>>2] = $86;
   $87 = HEAP32[$aData>>2]|0;
   $88 = HEAP32[$aEnd>>2]|0;
   $89 = ($87>>>0)<($88>>>0);
   if (!($89)) {
    break;
   }
  }
  $90 = HEAP32[$s1>>2]|0;
  $91 = HEAP32[$4>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$s2>>2]|0;
  $93 = HEAP32[$4>>2]|0;
  $94 = ((($93)) + 4|0);
  HEAP32[$94>>2] = $92;
  STACKTOP = sp;return;
 }
}
function _sqlite3OsSync($id,$flags) {
 $id = $id|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3WalFindFrame($pWal,$pgno,$piRead) {
 $pWal = $pWal|0;
 $pgno = $pgno|0;
 $piRead = $piRead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0;
 var $iFrame = 0, $iHash = 0, $iKey = 0, $iLast = 0, $iMinHash = 0, $iRead = 0, $iZero = 0, $nCollide = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $iRead = sp + 40|0;
 $iLast = sp + 36|0;
 $iHash = sp + 32|0;
 $iMinHash = sp + 28|0;
 $aHash = sp + 24|0;
 $aPgno = sp + 20|0;
 $iZero = sp + 16|0;
 $iKey = sp + 12|0;
 $nCollide = sp + 8|0;
 $rc = sp + 4|0;
 $iFrame = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $piRead;
 HEAP32[$iRead>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$iLast>>2] = $7;
 $8 = HEAP32[$iLast>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 40|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = ($13|0)==(0);
  if (!($14)) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 100|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_walFramePage($18)|0);
   HEAP32[$iMinHash>>2] = $19;
   $20 = HEAP32[$iLast>>2]|0;
   $21 = (_walFramePage($20)|0);
   HEAP32[$iHash>>2] = $21;
   L4: while(1) {
    $22 = HEAP32[$iHash>>2]|0;
    $23 = HEAP32[$iMinHash>>2]|0;
    $24 = ($22|0)>=($23|0);
    $25 = HEAP32[$iRead>>2]|0;
    $26 = ($25|0)==(0);
    $27 = $24 ? $26 : 0;
    if (!($27)) {
     label = 18;
     break;
    }
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$iHash>>2]|0;
    $30 = (_walHashGet($28,$29,$aHash,$aPgno,$iZero)|0);
    HEAP32[$rc>>2] = $30;
    $31 = HEAP32[$rc>>2]|0;
    $32 = ($31|0)!=(0);
    if ($32) {
     label = 7;
     break;
    }
    HEAP32[$nCollide>>2] = 8192;
    $34 = HEAP32[$2>>2]|0;
    $35 = (_walHash($34)|0);
    HEAP32[$iKey>>2] = $35;
    while(1) {
     $36 = HEAP32[$iKey>>2]|0;
     $37 = HEAP32[$aHash>>2]|0;
     $38 = (($37) + ($36<<1)|0);
     $39 = HEAP16[$38>>1]|0;
     $40 = ($39<<16>>16)!=(0);
     if (!($40)) {
      break;
     }
     $41 = HEAP32[$iKey>>2]|0;
     $42 = HEAP32[$aHash>>2]|0;
     $43 = (($42) + ($41<<1)|0);
     $44 = HEAP16[$43>>1]|0;
     $45 = $44&65535;
     $46 = HEAP32[$iZero>>2]|0;
     $47 = (($45) + ($46))|0;
     HEAP32[$iFrame>>2] = $47;
     $48 = HEAP32[$iFrame>>2]|0;
     $49 = HEAP32[$iLast>>2]|0;
     $50 = ($48>>>0)<=($49>>>0);
     if ($50) {
      $51 = HEAP32[$iFrame>>2]|0;
      $52 = HEAP32[$1>>2]|0;
      $53 = ((($52)) + 100|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($51>>>0)>=($54>>>0);
      if ($55) {
       $56 = HEAP32[$iKey>>2]|0;
       $57 = HEAP32[$aHash>>2]|0;
       $58 = (($57) + ($56<<1)|0);
       $59 = HEAP16[$58>>1]|0;
       $60 = $59&65535;
       $61 = HEAP32[$aPgno>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       $63 = HEAP32[$62>>2]|0;
       $64 = HEAP32[$2>>2]|0;
       $65 = ($63|0)==($64|0);
       if ($65) {
        $66 = HEAP32[$iFrame>>2]|0;
        HEAP32[$iRead>>2] = $66;
       }
      }
     }
     $67 = HEAP32[$nCollide>>2]|0;
     $68 = (($67) + -1)|0;
     HEAP32[$nCollide>>2] = $68;
     $69 = ($67|0)==(0);
     if ($69) {
      label = 15;
      break L4;
     }
     $71 = HEAP32[$iKey>>2]|0;
     $72 = (_walNextHash($71)|0);
     HEAP32[$iKey>>2] = $72;
    }
    $73 = HEAP32[$iHash>>2]|0;
    $74 = (($73) + -1)|0;
    HEAP32[$iHash>>2] = $74;
   }
   if ((label|0) == 7) {
    $33 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $33;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   else if ((label|0) == 15) {
    $70 = (_sqlite3CorruptError(53690)|0);
    HEAP32[$0>>2] = $70;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   else if ((label|0) == 18) {
    $75 = HEAP32[$iRead>>2]|0;
    $76 = HEAP32[$3>>2]|0;
    HEAP32[$76>>2] = $75;
    HEAP32[$0>>2] = 0;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
  }
 }
 $15 = HEAP32[$3>>2]|0;
 HEAP32[$15>>2] = 0;
 HEAP32[$0>>2] = 0;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _walWriteOneFrame($p,$pPage,$nTruncate,$0,$1) {
 $p = $p|0;
 $pPage = $pPage|0;
 $nTruncate = $nTruncate|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aFrame = 0, $pData = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $rc = sp + 12|0;
 $pData = sp + 8|0;
 $aFrame = sp + 32|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pPage;
 HEAP32[$5>>2] = $nTruncate;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$4>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pData>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$pData>>2]|0;
 _walEncodeFrame($15,$18,$19,$20,$aFrame);
 $21 = HEAP32[$3>>2]|0;
 $22 = $6;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (_walWriteToLog($21,$aFrame,24,$24,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $31;
  $47 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($47|0);
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = HEAP32[$pData>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = ((($34)) + 20|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $6;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),24,0)|0);
  $44 = tempRet0;
  $45 = (_walWriteToLog($32,$33,$36,$43,$44)|0);
  HEAP32[$rc>>2] = $45;
  $46 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $46;
  $47 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 return (0)|0;
}
function _walRewriteChecksums($pWal,$iLast) {
 $pWal = $pWal|0;
 $iLast = $iLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBuf = 0, $aFrame = 0, $iCksumOff = 0, $iOff = 0, $iPgno = 0, $iRead = 0, $nDbSize = 0, $rc = 0, $szPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $szPage = sp + 36|0;
 $rc = sp + 32|0;
 $aBuf = sp + 28|0;
 $aFrame = sp + 56|0;
 $iRead = sp + 24|0;
 $iCksumOff = sp + 8|0;
 $iOff = sp;
 $iPgno = sp + 20|0;
 $nDbSize = sp + 16|0;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iLast;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$szPage>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$szPage>>2]|0;
 $7 = (($6) + 24)|0;
 $8 = (_sqlite3_malloc($7)|0);
 HEAP32[$aBuf>>2] = $8;
 $9 = HEAP32[$aBuf>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $124 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($124|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 104|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(1);
 if ($14) {
  $15 = $iCksumOff;
  $16 = $15;
  HEAP32[$16>>2] = 24;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = 0;
 } else {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 104|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) - 1)|0;
  $23 = (($22) - 1)|0;
  $24 = HEAP32[$szPage>>2]|0;
  $25 = (($24) + 24)|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (___muldi3(($23|0),0,($25|0),($27|0))|0);
  $29 = tempRet0;
  $30 = (_i64Add(32,0,($28|0),($29|0))|0);
  $31 = tempRet0;
  $32 = (_i64Add(($30|0),($31|0),16,0)|0);
  $33 = tempRet0;
  $34 = $iCksumOff;
  $35 = $34;
  HEAP32[$35>>2] = $32;
  $36 = (($34) + 4)|0;
  $37 = $36;
  HEAP32[$37>>2] = $33;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$aBuf>>2]|0;
 $42 = $iCksumOff;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (_sqlite3OsRead($40,$41,8,$44,$47)|0);
 HEAP32[$rc>>2] = $48;
 $49 = HEAP32[$aBuf>>2]|0;
 $50 = (_sqlite3Get4byte($49)|0);
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 52|0);
 $53 = ((($52)) + 24|0);
 HEAP32[$53>>2] = $50;
 $54 = HEAP32[$aBuf>>2]|0;
 $55 = ((($54)) + 4|0);
 $56 = (_sqlite3Get4byte($55)|0);
 $57 = HEAP32[$1>>2]|0;
 $58 = ((($57)) + 52|0);
 $59 = ((($58)) + 24|0);
 $60 = ((($59)) + 4|0);
 HEAP32[$60>>2] = $56;
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 104|0);
 $63 = HEAP32[$62>>2]|0;
 HEAP32[$iRead>>2] = $63;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 104|0);
 HEAP32[$65>>2] = 0;
 while(1) {
  $66 = HEAP32[$rc>>2]|0;
  $67 = ($66|0)==(0);
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$iRead>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = ($68>>>0)<=($69>>>0);
  if (!($70)) {
   break;
  }
  $71 = HEAP32[$iRead>>2]|0;
  $72 = (($71) - 1)|0;
  $73 = HEAP32[$szPage>>2]|0;
  $74 = (($73) + 24)|0;
  $75 = ($74|0)<(0);
  $76 = $75 << 31 >> 31;
  $77 = (___muldi3(($72|0),0,($74|0),($76|0))|0);
  $78 = tempRet0;
  $79 = (_i64Add(32,0,($77|0),($78|0))|0);
  $80 = tempRet0;
  $81 = $iOff;
  $82 = $81;
  HEAP32[$82>>2] = $79;
  $83 = (($81) + 4)|0;
  $84 = $83;
  HEAP32[$84>>2] = $80;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = HEAP32[$aBuf>>2]|0;
  $89 = HEAP32[$szPage>>2]|0;
  $90 = (($89) + 24)|0;
  $91 = $iOff;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (($91) + 4)|0;
  $95 = $94;
  $96 = HEAP32[$95>>2]|0;
  $97 = (_sqlite3OsRead($87,$88,$90,$93,$96)|0);
  HEAP32[$rc>>2] = $97;
  $98 = HEAP32[$rc>>2]|0;
  $99 = ($98|0)==(0);
  if ($99) {
   $100 = HEAP32[$aBuf>>2]|0;
   $101 = (_sqlite3Get4byte($100)|0);
   HEAP32[$iPgno>>2] = $101;
   $102 = HEAP32[$aBuf>>2]|0;
   $103 = ((($102)) + 4|0);
   $104 = (_sqlite3Get4byte($103)|0);
   HEAP32[$nDbSize>>2] = $104;
   $105 = HEAP32[$1>>2]|0;
   $106 = HEAP32[$iPgno>>2]|0;
   $107 = HEAP32[$nDbSize>>2]|0;
   $108 = HEAP32[$aBuf>>2]|0;
   $109 = ((($108)) + 24|0);
   _walEncodeFrame($105,$106,$107,$109,$aFrame);
   $110 = HEAP32[$1>>2]|0;
   $111 = ((($110)) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = $iOff;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = (($113) + 4)|0;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (_sqlite3OsWrite($112,$aFrame,24,$115,$118)|0);
   HEAP32[$rc>>2] = $119;
  }
  $120 = HEAP32[$iRead>>2]|0;
  $121 = (($120) + 1)|0;
  HEAP32[$iRead>>2] = $121;
 }
 $122 = HEAP32[$aBuf>>2]|0;
 _sqlite3_free($122);
 $123 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $123;
 $124 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($124|0);
}
function _walLimitSize($pWal,$0,$1) {
 $pWal = $pWal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rx = 0, $sz = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp + 16|0;
 $2 = sp + 24|0;
 $3 = sp + 8|0;
 $sz = sp;
 $rx = sp + 20|0;
 HEAP32[$2>>2] = $pWal;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3OsFileSize($10,$sz)|0);
 HEAP32[$rx>>2] = $11;
 $12 = HEAP32[$rx>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $sz;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = $3;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($19|0)>($25|0);
  $27 = ($16>>>0)>($22>>>0);
  $28 = ($19|0)==($25|0);
  $29 = $28 & $27;
  $30 = $26 | $29;
  if ($30) {
   $31 = HEAP32[$2>>2]|0;
   $32 = ((($31)) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $3;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = (($34) + 4)|0;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3OsTruncate($33,$36,$39)|0);
   HEAP32[$rx>>2] = $40;
  }
 }
 _sqlite3EndBenignMalloc();
 $41 = HEAP32[$rx>>2]|0;
 $42 = ($41|0)!=(0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$rx>>2]|0;
 $44 = HEAP32[$2>>2]|0;
 $45 = ((($44)) + 108|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$vararg_buffer>>2] = $46;
 _sqlite3_log($43,18978,$vararg_buffer);
 STACKTOP = sp;return;
}
function _walIndexAppend($pWal,$iFrame,$iPage) {
 $pWal = $pWal|0;
 $iFrame = $iFrame|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iKey = 0, $iZero = 0;
 var $idx = 0, $nByte = 0, $nCollide = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $rc = sp + 28|0;
 $iZero = sp + 24|0;
 $aPgno = sp + 20|0;
 $aHash = sp + 16|0;
 $iKey = sp + 12|0;
 $idx = sp + 8|0;
 $nCollide = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iFrame;
 HEAP32[$3>>2] = $iPage;
 HEAP32[$iZero>>2] = 0;
 HEAP32[$aPgno>>2] = 0;
 HEAP32[$aHash>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_walFramePage($5)|0);
 $7 = (_walHashGet($4,$6,$aHash,$aPgno,$iZero)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 do {
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$iZero>>2]|0;
   $12 = (($10) - ($11))|0;
   HEAP32[$idx>>2] = $12;
   $13 = HEAP32[$idx>>2]|0;
   $14 = ($13|0)==(1);
   if ($14) {
    $15 = HEAP32[$aHash>>2]|0;
    $16 = ((($15)) + 16384|0);
    $17 = HEAP32[$aPgno>>2]|0;
    $18 = ((($17)) + 4|0);
    $19 = $16;
    $20 = $18;
    $21 = (($19) - ($20))|0;
    HEAP32[$nByte>>2] = $21;
    $22 = HEAP32[$aPgno>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP32[$nByte>>2]|0;
    _memset(($23|0),0,($24|0))|0;
   }
   $25 = HEAP32[$idx>>2]|0;
   $26 = HEAP32[$aPgno>>2]|0;
   $27 = (($26) + ($25<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[$1>>2]|0;
    _walCleanupHash($30);
   }
   $31 = HEAP32[$idx>>2]|0;
   HEAP32[$nCollide>>2] = $31;
   $32 = HEAP32[$3>>2]|0;
   $33 = (_walHash($32)|0);
   HEAP32[$iKey>>2] = $33;
   while(1) {
    $34 = HEAP32[$iKey>>2]|0;
    $35 = HEAP32[$aHash>>2]|0;
    $36 = (($35) + ($34<<1)|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = ($37<<16>>16)!=(0);
    if (!($38)) {
     label = 11;
     break;
    }
    $39 = HEAP32[$nCollide>>2]|0;
    $40 = (($39) + -1)|0;
    HEAP32[$nCollide>>2] = $40;
    $41 = ($39|0)==(0);
    if ($41) {
     break;
    }
    $43 = HEAP32[$iKey>>2]|0;
    $44 = (_walNextHash($43)|0);
    HEAP32[$iKey>>2] = $44;
   }
   if ((label|0) == 11) {
    $45 = HEAP32[$3>>2]|0;
    $46 = HEAP32[$idx>>2]|0;
    $47 = HEAP32[$aPgno>>2]|0;
    $48 = (($47) + ($46<<2)|0);
    HEAP32[$48>>2] = $45;
    $49 = HEAP32[$idx>>2]|0;
    $50 = $49&65535;
    $51 = HEAP32[$iKey>>2]|0;
    $52 = HEAP32[$aHash>>2]|0;
    $53 = (($52) + ($51<<1)|0);
    HEAP16[$53>>1] = $50;
    break;
   }
   $42 = (_sqlite3CorruptError(52166)|0);
   HEAP32[$0>>2] = $42;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 } while(0);
 $54 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _walIndexWriteHdr($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aHdr = 0, $nCksum = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $aHdr = sp + 4|0;
 $nCksum = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_walIndexHdr($1)|0);
 HEAP32[$aHdr>>2] = $2;
 HEAP32[$nCksum>>2] = 40;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = ((($4)) + 12|0);
 HEAP8[$5>>0] = 1;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 HEAP32[$7>>2] = 3007000;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 52|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 52|0);
 $12 = ((($11)) + 40|0);
 _walChecksumBytes(1,$9,40,0,$12);
 $13 = HEAP32[$aHdr>>2]|0;
 $14 = ((($13)) + 48|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 52|0);
 dest=$14; src=$16; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $17 = HEAP32[$0>>2]|0;
 _walShmBarrier($17);
 $18 = HEAP32[$aHdr>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 52|0);
 dest=$18; src=$20; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _walCkptInfo($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 96|0);
 STACKTOP = sp;return ($5|0);
}
function _walLockExclusive($pWal,$lockIdx,$n) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $lockIdx;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 43|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = (_sqlite3OsShmLock($10,$11,$12,10)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _walRestartHdr($pWal,$salt1) {
 $pWal = $pWal|0;
 $salt1 = $salt1|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aSalt = 0, $i = 0, $pInfo = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pInfo = sp + 8|0;
 $i = sp + 4|0;
 $aSalt = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $salt1;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_walCkptInfo($2)|0);
 HEAP32[$pInfo>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = ((($5)) + 32|0);
 HEAP32[$aSalt>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 112|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 52|0);
 $13 = ((($12)) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$aSalt>>2]|0;
 $15 = HEAP32[$aSalt>>2]|0;
 $16 = (_sqlite3Get4byte($15)|0);
 $17 = (1 + ($16))|0;
 _sqlite3Put4byte($14,$17);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 52|0);
 $20 = ((($19)) + 32|0);
 $21 = ((($20)) + 4|0);
 ;HEAP32[$21>>2]=HEAP32[$1>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 _walIndexWriteHdr($22);
 $23 = HEAP32[$pInfo>>2]|0;
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$pInfo>>2]|0;
 $25 = ((($24)) + 32|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$pInfo>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = ((($27)) + 4|0);
 HEAP32[$28>>2] = 0;
 HEAP32[$i>>2] = 2;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = ($29|0)<(5);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pInfo>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = (($33) + ($31<<2)|0);
  HEAP32[$34>>2] = -1;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$i>>2] = $36;
 }
 STACKTOP = sp;return;
}
function _walUnlockExclusive($pWal,$lockIdx,$n) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $lockIdx;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 (_sqlite3OsShmLock($9,$10,$11,9)|0);
 STACKTOP = sp;return;
}
function _walUnlockShared($pWal,$lockIdx) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $lockIdx;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 43|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 (_sqlite3OsShmLock($8,$9,1,5)|0);
 STACKTOP = sp;return;
}
function _walTryBeginRead($pWal,$pChanged,$useWal,$cnt) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 $useWal = $useWal|0;
 $cnt = $cnt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $mxFrame = 0, $mxI = 0, $mxReadMark = 0, $nDelay = 0, $or$cond = 0, $pInfo = 0, $rc = 0, $thisMark = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $pInfo = sp + 28|0;
 $mxReadMark = sp + 24|0;
 $mxI = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $mxFrame = sp + 8|0;
 $nDelay = sp + 4|0;
 $thisMark = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 HEAP32[$3>>2] = $useWal;
 HEAP32[$4>>2] = $cnt;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(5);
 if ($6) {
  HEAP32[$nDelay>>2] = 1;
  $7 = HEAP32[$4>>2]|0;
  $8 = ($7|0)>(100);
  if ($8) {
   HEAP32[$0>>2] = 15;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
  $9 = HEAP32[$4>>2]|0;
  $10 = ($9|0)>=(10);
  if ($10) {
   $11 = HEAP32[$4>>2]|0;
   $12 = (($11) - 9)|0;
   $13 = HEAP32[$4>>2]|0;
   $14 = (($13) - 9)|0;
   $15 = Math_imul($12, $14)|0;
   $16 = ($15*39)|0;
   HEAP32[$nDelay>>2] = $16;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$nDelay>>2]|0;
  (_sqlite3OsSleep($18,$19)|0);
 }
 $20 = HEAP32[$3>>2]|0;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (_walIndexReadHdr($22,$23)|0);
  HEAP32[$rc>>2] = $24;
  $25 = HEAP32[$rc>>2]|0;
  $26 = ($25|0)==(5);
  do {
   if ($26) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 32|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = (_walLockShared($32,2)|0);
    HEAP32[$rc>>2] = $33;
    $34 = (0)==($33|0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     _walUnlockShared($35,2);
     HEAP32[$rc>>2] = -1;
     break;
    }
    $36 = HEAP32[$rc>>2]|0;
    $37 = ($36|0)==(5);
    if ($37) {
     HEAP32[$rc>>2] = 261;
    }
   }
  } while(0);
  $38 = HEAP32[$rc>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $40;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = (_walCkptInfo($41)|0);
 HEAP32[$pInfo>>2] = $42;
 $43 = HEAP32[$3>>2]|0;
 $44 = ($43|0)!=(0);
 do {
  if (!($44)) {
   $45 = HEAP32[$pInfo>>2]|0;
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 52|0);
   $49 = ((($48)) + 16|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($46|0)==($50|0);
   if ($51) {
    $52 = HEAP32[$1>>2]|0;
    $53 = (_walLockShared($52,3)|0);
    HEAP32[$rc>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    _walShmBarrier($54);
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)==(0);
    if (!($56)) {
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)!=(5);
     if (!($66)) {
      break;
     }
     $67 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $67;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    }
    $57 = HEAP32[$1>>2]|0;
    $58 = (_walIndexHdr($57)|0);
    $59 = HEAP32[$1>>2]|0;
    $60 = ((($59)) + 52|0);
    $61 = (_memcmp($58,$60,48)|0);
    $62 = ($61|0)!=(0);
    $63 = HEAP32[$1>>2]|0;
    if ($62) {
     _walUnlockShared($63,3);
     HEAP32[$0>>2] = -1;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    } else {
     $64 = ((($63)) + 40|0);
     HEAP16[$64>>1] = 0;
     HEAP32[$0>>2] = 0;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    }
   }
  }
 } while(0);
 HEAP32[$mxReadMark>>2] = 0;
 HEAP32[$mxI>>2] = 0;
 $68 = HEAP32[$1>>2]|0;
 $69 = ((($68)) + 52|0);
 $70 = ((($69)) + 16|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$mxFrame>>2] = $71;
 HEAP32[$i>>2] = 1;
 while(1) {
  $72 = HEAP32[$i>>2]|0;
  $73 = ($72|0)<(5);
  if (!($73)) {
   break;
  }
  $74 = HEAP32[$i>>2]|0;
  $75 = HEAP32[$pInfo>>2]|0;
  $76 = ((($75)) + 4|0);
  $77 = (($76) + ($74<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  HEAP32[$thisMark>>2] = $78;
  $79 = HEAP32[$mxReadMark>>2]|0;
  $80 = HEAP32[$thisMark>>2]|0;
  $81 = ($79>>>0)<=($80>>>0);
  if ($81) {
   $82 = HEAP32[$thisMark>>2]|0;
   $83 = HEAP32[$mxFrame>>2]|0;
   $84 = ($82>>>0)<=($83>>>0);
   if ($84) {
    $85 = HEAP32[$thisMark>>2]|0;
    HEAP32[$mxReadMark>>2] = $85;
    $86 = HEAP32[$i>>2]|0;
    HEAP32[$mxI>>2] = $86;
   }
  }
  $87 = HEAP32[$i>>2]|0;
  $88 = (($87) + 1)|0;
  HEAP32[$i>>2] = $88;
 }
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 46|0);
 $91 = HEAP8[$90>>0]|0;
 $92 = $91&255;
 $93 = $92 & 2;
 $94 = ($93|0)==(0);
 L48: do {
  if ($94) {
   $95 = HEAP32[$mxReadMark>>2]|0;
   $96 = HEAP32[$mxFrame>>2]|0;
   $97 = ($95>>>0)<($96>>>0);
   $98 = HEAP32[$mxI>>2]|0;
   $99 = ($98|0)==(0);
   $or$cond = $97 | $99;
   if ($or$cond) {
    HEAP32[$i>>2] = 1;
    while(1) {
     $100 = HEAP32[$i>>2]|0;
     $101 = ($100|0)<(5);
     if (!($101)) {
      break L48;
     }
     $102 = HEAP32[$1>>2]|0;
     $103 = HEAP32[$i>>2]|0;
     $104 = (3 + ($103))|0;
     $105 = (_walLockExclusive($102,$104,1)|0);
     HEAP32[$rc>>2] = $105;
     $106 = HEAP32[$rc>>2]|0;
     $107 = ($106|0)==(0);
     if ($107) {
      label = 36;
      break;
     }
     $117 = HEAP32[$rc>>2]|0;
     $118 = ($117|0)!=(5);
     if ($118) {
      break;
     }
     $120 = HEAP32[$i>>2]|0;
     $121 = (($120) + 1)|0;
     HEAP32[$i>>2] = $121;
    }
    if ((label|0) == 36) {
     $108 = HEAP32[$mxFrame>>2]|0;
     $109 = HEAP32[$i>>2]|0;
     $110 = HEAP32[$pInfo>>2]|0;
     $111 = ((($110)) + 4|0);
     $112 = (($111) + ($109<<2)|0);
     HEAP32[$112>>2] = $108;
     HEAP32[$mxReadMark>>2] = $108;
     $113 = HEAP32[$i>>2]|0;
     HEAP32[$mxI>>2] = $113;
     $114 = HEAP32[$1>>2]|0;
     $115 = HEAP32[$i>>2]|0;
     $116 = (3 + ($115))|0;
     _walUnlockExclusive($114,$116,1);
     break;
    }
    $119 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $119;
    $164 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($164|0);
   }
  }
 } while(0);
 $122 = HEAP32[$mxI>>2]|0;
 $123 = ($122|0)==(0);
 if ($123) {
  $124 = HEAP32[$rc>>2]|0;
  $125 = ($124|0)==(5);
  $126 = $125 ? -1 : 520;
  HEAP32[$0>>2] = $126;
  $164 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($164|0);
 }
 $127 = HEAP32[$1>>2]|0;
 $128 = HEAP32[$mxI>>2]|0;
 $129 = (3 + ($128))|0;
 $130 = (_walLockShared($127,$129)|0);
 HEAP32[$rc>>2] = $130;
 $131 = HEAP32[$rc>>2]|0;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = HEAP32[$rc>>2]|0;
  $134 = ($133|0)==(5);
  $135 = HEAP32[$rc>>2]|0;
  $136 = $134 ? -1 : $135;
  HEAP32[$0>>2] = $136;
  $164 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($164|0);
 }
 $137 = HEAP32[$pInfo>>2]|0;
 $138 = HEAP32[$137>>2]|0;
 $139 = (($138) + 1)|0;
 $140 = HEAP32[$1>>2]|0;
 $141 = ((($140)) + 100|0);
 HEAP32[$141>>2] = $139;
 $142 = HEAP32[$1>>2]|0;
 _walShmBarrier($142);
 $143 = HEAP32[$mxI>>2]|0;
 $144 = HEAP32[$pInfo>>2]|0;
 $145 = ((($144)) + 4|0);
 $146 = (($145) + ($143<<2)|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = HEAP32[$mxReadMark>>2]|0;
 $149 = ($147|0)!=($148|0);
 if (!($149)) {
  $150 = HEAP32[$1>>2]|0;
  $151 = (_walIndexHdr($150)|0);
  $152 = HEAP32[$1>>2]|0;
  $153 = ((($152)) + 52|0);
  $154 = (_memcmp($151,$153,48)|0);
  $155 = ($154|0)!=(0);
  if (!($155)) {
   $159 = HEAP32[$mxI>>2]|0;
   $160 = $159&65535;
   $161 = HEAP32[$1>>2]|0;
   $162 = ((($161)) + 40|0);
   HEAP16[$162>>1] = $160;
   $163 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $163;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
 }
 $156 = HEAP32[$1>>2]|0;
 $157 = HEAP32[$mxI>>2]|0;
 $158 = (3 + ($157))|0;
 _walUnlockShared($156,$158);
 HEAP32[$0>>2] = -1;
 $164 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($164|0);
}
function _sqlite3OsShmLock($id,$offset,$n,$flags) {
 $id = $id|0;
 $offset = $offset|0;
 $n = $n|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iiiii[$7 & 255]($8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _walIndexReadHdr($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $badHdr = 0, $page0 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $badHdr = sp + 4|0;
 $page0 = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_walIndexPage($3,0,$page0)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $8 = HEAP32[$page0>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_walIndexTryHdr($10,$11)|0);
  $13 = $12;
 } else {
  $13 = 1;
 }
 HEAP32[$badHdr>>2] = $13;
 $14 = HEAP32[$badHdr>>2]|0;
 $15 = ($14|0)!=(0);
 do {
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 46|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 2;
   $21 = ($20|0)!=(0);
   $22 = HEAP32[$1>>2]|0;
   if ($21) {
    $23 = (_walLockShared($22,0)|0);
    HEAP32[$rc>>2] = $23;
    $24 = (0)==($23|0);
    if (!($24)) {
     break;
    }
    $25 = HEAP32[$1>>2]|0;
    _walUnlockShared($25,0);
    HEAP32[$rc>>2] = 264;
    break;
   }
   $26 = (_walLockExclusive($22,0,1)|0);
   HEAP32[$rc>>2] = $26;
   $27 = (0)==($26|0);
   if ($27) {
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 44|0);
    HEAP8[$29>>0] = 1;
    $30 = HEAP32[$1>>2]|0;
    $31 = (_walIndexPage($30,0,$page0)|0);
    HEAP32[$rc>>2] = $31;
    $32 = (0)==($31|0);
    if ($32) {
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = (_walIndexTryHdr($33,$34)|0);
     HEAP32[$badHdr>>2] = $35;
     $36 = HEAP32[$badHdr>>2]|0;
     $37 = ($36|0)!=(0);
     if ($37) {
      $38 = HEAP32[$1>>2]|0;
      $39 = (_walIndexRecover($38)|0);
      HEAP32[$rc>>2] = $39;
      $40 = HEAP32[$2>>2]|0;
      HEAP32[$40>>2] = 1;
     }
    }
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 44|0);
    HEAP8[$42>>0] = 0;
    $43 = HEAP32[$1>>2]|0;
    _walUnlockExclusive($43,0,1);
   }
  }
 } while(0);
 $44 = HEAP32[$badHdr>>2]|0;
 $45 = ($44|0)==(0);
 if ($45) {
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 52|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)!=(3007000);
  if ($49) {
   $50 = (_sqlite3CantopenError(53243)|0);
   HEAP32[$rc>>2] = $50;
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _walLockShared($pWal,$lockIdx) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $lockIdx;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_sqlite3OsShmLock($9,$10,1,6)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _walShmBarrier($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 43|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(2);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3OsShmBarrier($8);
 STACKTOP = sp;return;
}
function _walIndexPage($pWal,$iPage,$ppPage) {
 $pWal = $pWal|0;
 $iPage = $iPage|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $apNew = 0, $nByte = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $nByte = sp + 4|0;
 $apNew = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($6|0)<=($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$2>>2]|0;
   $10 = (($9) + 1)|0;
   $11 = $10<<2;
   HEAP32[$nByte>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 32|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$nByte>>2]|0;
   $16 = ($15|0)<(0);
   $17 = $16 << 31 >> 31;
   $18 = (_sqlite3_realloc64($14,$15,$17)|0);
   HEAP32[$apNew>>2] = $18;
   $19 = HEAP32[$apNew>>2]|0;
   $20 = ($19|0)!=(0|0);
   if ($20) {
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 24|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$apNew>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    $27 = HEAP32[$2>>2]|0;
    $28 = (($27) + 1)|0;
    $29 = HEAP32[$1>>2]|0;
    $30 = ((($29)) + 24|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (($28) - ($31))|0;
    $33 = $32<<2;
    _memset(($26|0),0,($33|0))|0;
    $34 = HEAP32[$apNew>>2]|0;
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 32|0);
    HEAP32[$36>>2] = $34;
    $37 = HEAP32[$2>>2]|0;
    $38 = (($37) + 1)|0;
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 24|0);
    HEAP32[$40>>2] = $38;
    break;
   }
   $21 = HEAP32[$3>>2]|0;
   HEAP32[$21>>2] = 0;
   HEAP32[$0>>2] = 7;
   $96 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($96|0);
  }
 } while(0);
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = ((($42)) + 32|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<2)|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0|0);
 do {
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 43|0);
   $50 = HEAP8[$49>>0]|0;
   $51 = $50&255;
   $52 = ($51|0)==(2);
   if ($52) {
    $53 = (_sqlite3MallocZero(32768,0)|0);
    $54 = HEAP32[$2>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 32|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + ($54<<2)|0);
    HEAP32[$58>>2] = $53;
    $59 = HEAP32[$2>>2]|0;
    $60 = HEAP32[$1>>2]|0;
    $61 = ((($60)) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($59<<2)|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ($64|0)!=(0|0);
    if ($65) {
     break;
    }
    HEAP32[$rc>>2] = 7;
    break;
   } else {
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 4|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP32[$2>>2]|0;
    $70 = HEAP32[$1>>2]|0;
    $71 = ((($70)) + 44|0);
    $72 = HEAP8[$71>>0]|0;
    $73 = $72&255;
    $74 = HEAP32[$2>>2]|0;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 32|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = (($77) + ($74<<2)|0);
    $79 = (_sqlite3OsShmMap($68,$69,32768,$73,$78)|0);
    HEAP32[$rc>>2] = $79;
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)==(8);
    if (!($81)) {
     break;
    }
    $82 = HEAP32[$1>>2]|0;
    $83 = ((($82)) + 46|0);
    $84 = HEAP8[$83>>0]|0;
    $85 = $84&255;
    $86 = $85 | 2;
    $87 = $86&255;
    HEAP8[$83>>0] = $87;
    HEAP32[$rc>>2] = 0;
    break;
   }
  }
 } while(0);
 $88 = HEAP32[$2>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 32|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = (($91) + ($88<<2)|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = HEAP32[$3>>2]|0;
 HEAP32[$94>>2] = $93;
 $95 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $95;
 $96 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($96|0);
}
function _walIndexTryHdr($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aCksum = 0, $aHdr = 0, $h1 = 0, $h2 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $aCksum = sp + 104|0;
 $h1 = sp + 56|0;
 $h2 = sp + 8|0;
 $aHdr = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_walIndexHdr($3)|0);
 HEAP32[$aHdr>>2] = $4;
 $5 = HEAP32[$aHdr>>2]|0;
 dest=$h1; src=$5; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $6 = HEAP32[$1>>2]|0;
 _walShmBarrier($6);
 $7 = HEAP32[$aHdr>>2]|0;
 $8 = ((($7)) + 48|0);
 dest=$h2; src=$8; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $9 = (_memcmp($h1,$h2,48)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $11 = ((($h1)) + 12|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(0);
 if ($14) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 _walChecksumBytes(1,$h1,40,0,$aCksum);
 $15 = HEAP32[$aCksum>>2]|0;
 $16 = ((($h1)) + 40|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($15|0)!=($17|0);
 if (!($18)) {
  $19 = ((($aCksum)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ((($h1)) + 40|0);
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)!=($23|0);
  if (!($24)) {
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 52|0);
   $27 = (_memcmp($26,$h1,48)|0);
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$2>>2]|0;
    HEAP32[$29>>2] = 1;
    $30 = HEAP32[$1>>2]|0;
    $31 = ((($30)) + 52|0);
    dest=$31; src=$h1; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 52|0);
    $34 = ((($33)) + 14|0);
    $35 = HEAP16[$34>>1]|0;
    $36 = $35&65535;
    $37 = $36 & 65024;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 52|0);
    $40 = ((($39)) + 14|0);
    $41 = HEAP16[$40>>1]|0;
    $42 = $41&65535;
    $43 = $42 & 1;
    $44 = $43 << 16;
    $45 = (($37) + ($44))|0;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 36|0);
    HEAP32[$47>>2] = $45;
   }
   HEAP32[$0>>2] = 0;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _walIndexRecover($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBuf = 0, $aData = 0, $aFrame = 0;
 var $aFrameCksum = 0, $i = 0, $iFrame = 0, $iLock = 0, $iOffset = 0, $isValid = 0, $magic = 0, $nLock = 0, $nSize = 0, $nTruncate = 0, $or$cond = 0, $or$cond3 = 0, $pInfo = 0, $pgno = 0, $rc = 0, $szFrame = 0, $szPage = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $version = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $rc = sp + 88|0;
 $nSize = sp + 8|0;
 $aFrameCksum = sp + 80|0;
 $iLock = sp + 76|0;
 $nLock = sp + 72|0;
 $aBuf = sp + 104|0;
 $aFrame = sp + 68|0;
 $szFrame = sp + 64|0;
 $aData = sp + 60|0;
 $iFrame = sp + 56|0;
 $iOffset = sp;
 $szPage = sp + 52|0;
 $magic = sp + 48|0;
 $version = sp + 44|0;
 $isValid = sp + 40|0;
 $pgno = sp + 36|0;
 $nTruncate = sp + 32|0;
 $pInfo = sp + 28|0;
 $i = sp + 24|0;
 HEAP32[$1>>2] = $pWal;
 ;HEAP32[$aFrameCksum>>2]=0|0;HEAP32[$aFrameCksum+4>>2]=0|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 45|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = (1 + ($5))|0;
 HEAP32[$iLock>>2] = $6;
 $7 = HEAP32[$iLock>>2]|0;
 $8 = (8 - ($7))|0;
 HEAP32[$nLock>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$iLock>>2]|0;
 $11 = HEAP32[$nLock>>2]|0;
 $12 = (_walLockExclusive($9,$10,$11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $276 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($276|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 52|0);
 dest=$17; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3OsFileSize($20,$nSize)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)!=(0);
 L5: do {
  if (!($23)) {
   $24 = $nSize;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + 4)|0;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)>(0);
   $31 = ($26>>>0)>(32);
   $32 = ($29|0)==(0);
   $33 = $32 & $31;
   $34 = $30 | $33;
   do {
    if ($34) {
     HEAP32[$aFrame>>2] = 0;
     $35 = HEAP32[$1>>2]|0;
     $36 = ((($35)) + 8|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = (_sqlite3OsRead($37,$aBuf,32,0,0)|0);
     HEAP32[$rc>>2] = $38;
     $39 = HEAP32[$rc>>2]|0;
     $40 = ($39|0)!=(0);
     if ($40) {
      break L5;
     }
     $41 = (_sqlite3Get4byte($aBuf)|0);
     HEAP32[$magic>>2] = $41;
     $42 = ((($aBuf)) + 8|0);
     $43 = (_sqlite3Get4byte($42)|0);
     HEAP32[$szPage>>2] = $43;
     $44 = HEAP32[$magic>>2]|0;
     $45 = $44 & -2;
     $46 = ($45|0)!=(931071618);
     if (!($46)) {
      $47 = HEAP32[$szPage>>2]|0;
      $48 = HEAP32[$szPage>>2]|0;
      $49 = (($48) - 1)|0;
      $50 = $47 & $49;
      $51 = ($50|0)!=(0);
      $52 = HEAP32[$szPage>>2]|0;
      $53 = ($52|0)>(65536);
      $or$cond = $51 | $53;
      $54 = HEAP32[$szPage>>2]|0;
      $55 = ($54|0)<(512);
      $or$cond3 = $or$cond | $55;
      if (!($or$cond3)) {
       $56 = HEAP32[$magic>>2]|0;
       $57 = $56 & 1;
       $58 = $57&255;
       $59 = HEAP32[$1>>2]|0;
       $60 = ((($59)) + 52|0);
       $61 = ((($60)) + 13|0);
       HEAP8[$61>>0] = $58;
       $62 = HEAP32[$szPage>>2]|0;
       $63 = HEAP32[$1>>2]|0;
       $64 = ((($63)) + 36|0);
       HEAP32[$64>>2] = $62;
       $65 = ((($aBuf)) + 12|0);
       $66 = (_sqlite3Get4byte($65)|0);
       $67 = HEAP32[$1>>2]|0;
       $68 = ((($67)) + 112|0);
       HEAP32[$68>>2] = $66;
       $69 = HEAP32[$1>>2]|0;
       $70 = ((($69)) + 52|0);
       $71 = ((($70)) + 32|0);
       $72 = ((($aBuf)) + 16|0);
       ;HEAP8[$71>>0]=HEAP8[$72>>0]|0;HEAP8[$71+1>>0]=HEAP8[$72+1>>0]|0;HEAP8[$71+2>>0]=HEAP8[$72+2>>0]|0;HEAP8[$71+3>>0]=HEAP8[$72+3>>0]|0;HEAP8[$71+4>>0]=HEAP8[$72+4>>0]|0;HEAP8[$71+5>>0]=HEAP8[$72+5>>0]|0;HEAP8[$71+6>>0]=HEAP8[$72+6>>0]|0;HEAP8[$71+7>>0]=HEAP8[$72+7>>0]|0;
       $73 = HEAP32[$1>>2]|0;
       $74 = ((($73)) + 52|0);
       $75 = ((($74)) + 13|0);
       $76 = HEAP8[$75>>0]|0;
       $77 = $76&255;
       $78 = HEAP8[352>>0]|0;
       $79 = $78 << 24 >> 24;
       $80 = ($79|0)==(0);
       $81 = $80&1;
       $82 = ($77|0)==($81|0);
       $83 = $82&1;
       $84 = HEAP32[$1>>2]|0;
       $85 = ((($84)) + 52|0);
       $86 = ((($85)) + 24|0);
       _walChecksumBytes($83,$aBuf,24,0,$86);
       $87 = HEAP32[$1>>2]|0;
       $88 = ((($87)) + 52|0);
       $89 = ((($88)) + 24|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = ((($aBuf)) + 24|0);
       $92 = (_sqlite3Get4byte($91)|0);
       $93 = ($90|0)!=($92|0);
       if (!($93)) {
        $94 = HEAP32[$1>>2]|0;
        $95 = ((($94)) + 52|0);
        $96 = ((($95)) + 24|0);
        $97 = ((($96)) + 4|0);
        $98 = HEAP32[$97>>2]|0;
        $99 = ((($aBuf)) + 28|0);
        $100 = (_sqlite3Get4byte($99)|0);
        $101 = ($98|0)!=($100|0);
        if (!($101)) {
         $102 = ((($aBuf)) + 4|0);
         $103 = (_sqlite3Get4byte($102)|0);
         HEAP32[$version>>2] = $103;
         $104 = HEAP32[$version>>2]|0;
         $105 = ($104|0)!=(3007000);
         if ($105) {
          $106 = (_sqlite3CantopenError(52297)|0);
          HEAP32[$rc>>2] = $106;
          break;
         }
         $107 = HEAP32[$szPage>>2]|0;
         $108 = (($107) + 24)|0;
         HEAP32[$szFrame>>2] = $108;
         $109 = HEAP32[$szFrame>>2]|0;
         $110 = ($109|0)<(0);
         $111 = $110 << 31 >> 31;
         $112 = (_sqlite3_malloc64($109,$111)|0);
         HEAP32[$aFrame>>2] = $112;
         $113 = HEAP32[$aFrame>>2]|0;
         $114 = ($113|0)!=(0|0);
         if (!($114)) {
          HEAP32[$rc>>2] = 7;
          break L5;
         }
         $115 = HEAP32[$aFrame>>2]|0;
         $116 = ((($115)) + 24|0);
         HEAP32[$aData>>2] = $116;
         HEAP32[$iFrame>>2] = 0;
         $117 = $iOffset;
         $118 = $117;
         HEAP32[$118>>2] = 32;
         $119 = (($117) + 4)|0;
         $120 = $119;
         HEAP32[$120>>2] = 0;
         while(1) {
          $121 = $iOffset;
          $122 = $121;
          $123 = HEAP32[$122>>2]|0;
          $124 = (($121) + 4)|0;
          $125 = $124;
          $126 = HEAP32[$125>>2]|0;
          $127 = HEAP32[$szFrame>>2]|0;
          $128 = ($127|0)<(0);
          $129 = $128 << 31 >> 31;
          $130 = (_i64Add(($123|0),($126|0),($127|0),($129|0))|0);
          $131 = tempRet0;
          $132 = $nSize;
          $133 = $132;
          $134 = HEAP32[$133>>2]|0;
          $135 = (($132) + 4)|0;
          $136 = $135;
          $137 = HEAP32[$136>>2]|0;
          $138 = ($131|0)<($137|0);
          $139 = ($130>>>0)<=($134>>>0);
          $140 = ($131|0)==($137|0);
          $141 = $140 & $139;
          $142 = $138 | $141;
          if (!($142)) {
           break;
          }
          $143 = HEAP32[$iFrame>>2]|0;
          $144 = (($143) + 1)|0;
          HEAP32[$iFrame>>2] = $144;
          $145 = HEAP32[$1>>2]|0;
          $146 = ((($145)) + 8|0);
          $147 = HEAP32[$146>>2]|0;
          $148 = HEAP32[$aFrame>>2]|0;
          $149 = HEAP32[$szFrame>>2]|0;
          $150 = $iOffset;
          $151 = $150;
          $152 = HEAP32[$151>>2]|0;
          $153 = (($150) + 4)|0;
          $154 = $153;
          $155 = HEAP32[$154>>2]|0;
          $156 = (_sqlite3OsRead($147,$148,$149,$152,$155)|0);
          HEAP32[$rc>>2] = $156;
          $157 = HEAP32[$rc>>2]|0;
          $158 = ($157|0)!=(0);
          if ($158) {
           break;
          }
          $159 = HEAP32[$1>>2]|0;
          $160 = HEAP32[$aData>>2]|0;
          $161 = HEAP32[$aFrame>>2]|0;
          $162 = (_walDecodeFrame($159,$pgno,$nTruncate,$160,$161)|0);
          HEAP32[$isValid>>2] = $162;
          $163 = HEAP32[$isValid>>2]|0;
          $164 = ($163|0)!=(0);
          if (!($164)) {
           break;
          }
          $165 = HEAP32[$1>>2]|0;
          $166 = HEAP32[$iFrame>>2]|0;
          $167 = HEAP32[$pgno>>2]|0;
          $168 = (_walIndexAppend($165,$166,$167)|0);
          HEAP32[$rc>>2] = $168;
          $169 = HEAP32[$rc>>2]|0;
          $170 = ($169|0)!=(0);
          if ($170) {
           break;
          }
          $171 = HEAP32[$nTruncate>>2]|0;
          $172 = ($171|0)!=(0);
          if ($172) {
           $173 = HEAP32[$iFrame>>2]|0;
           $174 = HEAP32[$1>>2]|0;
           $175 = ((($174)) + 52|0);
           $176 = ((($175)) + 16|0);
           HEAP32[$176>>2] = $173;
           $177 = HEAP32[$nTruncate>>2]|0;
           $178 = HEAP32[$1>>2]|0;
           $179 = ((($178)) + 52|0);
           $180 = ((($179)) + 20|0);
           HEAP32[$180>>2] = $177;
           $181 = HEAP32[$szPage>>2]|0;
           $182 = $181 & 65280;
           $183 = HEAP32[$szPage>>2]|0;
           $184 = $183 >> 16;
           $185 = $182 | $184;
           $186 = $185&65535;
           $187 = HEAP32[$1>>2]|0;
           $188 = ((($187)) + 52|0);
           $189 = ((($188)) + 14|0);
           HEAP16[$189>>1] = $186;
           $190 = HEAP32[$1>>2]|0;
           $191 = ((($190)) + 52|0);
           $192 = ((($191)) + 24|0);
           $193 = HEAP32[$192>>2]|0;
           HEAP32[$aFrameCksum>>2] = $193;
           $194 = HEAP32[$1>>2]|0;
           $195 = ((($194)) + 52|0);
           $196 = ((($195)) + 24|0);
           $197 = ((($196)) + 4|0);
           $198 = HEAP32[$197>>2]|0;
           $199 = ((($aFrameCksum)) + 4|0);
           HEAP32[$199>>2] = $198;
          }
          $200 = HEAP32[$szFrame>>2]|0;
          $201 = ($200|0)<(0);
          $202 = $201 << 31 >> 31;
          $203 = $iOffset;
          $204 = $203;
          $205 = HEAP32[$204>>2]|0;
          $206 = (($203) + 4)|0;
          $207 = $206;
          $208 = HEAP32[$207>>2]|0;
          $209 = (_i64Add(($205|0),($208|0),($200|0),($202|0))|0);
          $210 = tempRet0;
          $211 = $iOffset;
          $212 = $211;
          HEAP32[$212>>2] = $209;
          $213 = (($211) + 4)|0;
          $214 = $213;
          HEAP32[$214>>2] = $210;
         }
         $215 = HEAP32[$aFrame>>2]|0;
         _sqlite3_free($215);
        }
       }
      }
     }
    }
   } while(0);
   $216 = HEAP32[$rc>>2]|0;
   $217 = ($216|0)==(0);
   if ($217) {
    $218 = HEAP32[$aFrameCksum>>2]|0;
    $219 = HEAP32[$1>>2]|0;
    $220 = ((($219)) + 52|0);
    $221 = ((($220)) + 24|0);
    HEAP32[$221>>2] = $218;
    $222 = ((($aFrameCksum)) + 4|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = HEAP32[$1>>2]|0;
    $225 = ((($224)) + 52|0);
    $226 = ((($225)) + 24|0);
    $227 = ((($226)) + 4|0);
    HEAP32[$227>>2] = $223;
    $228 = HEAP32[$1>>2]|0;
    _walIndexWriteHdr($228);
    $229 = HEAP32[$1>>2]|0;
    $230 = (_walCkptInfo($229)|0);
    HEAP32[$pInfo>>2] = $230;
    $231 = HEAP32[$pInfo>>2]|0;
    HEAP32[$231>>2] = 0;
    $232 = HEAP32[$1>>2]|0;
    $233 = ((($232)) + 52|0);
    $234 = ((($233)) + 16|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = HEAP32[$pInfo>>2]|0;
    $237 = ((($236)) + 32|0);
    HEAP32[$237>>2] = $235;
    $238 = HEAP32[$pInfo>>2]|0;
    $239 = ((($238)) + 4|0);
    HEAP32[$239>>2] = 0;
    HEAP32[$i>>2] = 1;
    while(1) {
     $240 = HEAP32[$i>>2]|0;
     $241 = ($240|0)<(5);
     if (!($241)) {
      break;
     }
     $242 = HEAP32[$i>>2]|0;
     $243 = HEAP32[$pInfo>>2]|0;
     $244 = ((($243)) + 4|0);
     $245 = (($244) + ($242<<2)|0);
     HEAP32[$245>>2] = -1;
     $246 = HEAP32[$i>>2]|0;
     $247 = (($246) + 1)|0;
     HEAP32[$i>>2] = $247;
    }
    $248 = HEAP32[$1>>2]|0;
    $249 = ((($248)) + 52|0);
    $250 = ((($249)) + 16|0);
    $251 = HEAP32[$250>>2]|0;
    $252 = ($251|0)!=(0);
    if ($252) {
     $253 = HEAP32[$1>>2]|0;
     $254 = ((($253)) + 52|0);
     $255 = ((($254)) + 16|0);
     $256 = HEAP32[$255>>2]|0;
     $257 = HEAP32[$pInfo>>2]|0;
     $258 = ((($257)) + 4|0);
     $259 = ((($258)) + 4|0);
     HEAP32[$259>>2] = $256;
    }
    $260 = HEAP32[$1>>2]|0;
    $261 = ((($260)) + 52|0);
    $262 = ((($261)) + 20|0);
    $263 = HEAP32[$262>>2]|0;
    $264 = ($263|0)!=(0);
    if ($264) {
     $265 = HEAP32[$1>>2]|0;
     $266 = ((($265)) + 52|0);
     $267 = ((($266)) + 16|0);
     $268 = HEAP32[$267>>2]|0;
     $269 = HEAP32[$1>>2]|0;
     $270 = ((($269)) + 108|0);
     $271 = HEAP32[$270>>2]|0;
     HEAP32[$vararg_buffer>>2] = $268;
     $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr4>>2] = $271;
     _sqlite3_log(283,18899,$vararg_buffer);
    }
   }
  }
 } while(0);
 $272 = HEAP32[$1>>2]|0;
 $273 = HEAP32[$iLock>>2]|0;
 $274 = HEAP32[$nLock>>2]|0;
 _walUnlockExclusive($272,$273,$274);
 $275 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $275;
 $276 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($276|0);
}
function _sqlite3OsShmMap($id,$iPage,$pgsz,$bExtend,$pp) {
 $id = $id|0;
 $iPage = $iPage|0;
 $pgsz = $pgsz|0;
 $bExtend = $bExtend|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $pgsz;
 HEAP32[$3>>2] = $bExtend;
 HEAP32[$4>>2] = $pp;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = (FUNCTION_TABLE_iiiiii[$8 & 255]($9,$10,$11,$12,$13)|0);
 STACKTOP = sp;return ($14|0);
}
function _sqlite3OsFileSize($id,$pSize) {
 $id = $id|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3OsRead($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $id;
 HEAP32[$3>>2] = $pBuf;
 HEAP32[$4>>2] = $amt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = $5;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiiii[$13 & 255]($14,$15,$16,$19,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _walDecodeFrame($pWal,$piPage,$pnTruncate,$aData,$aFrame) {
 $pWal = $pWal|0;
 $piPage = $piPage|0;
 $pnTruncate = $pnTruncate|0;
 $aData = $aData|0;
 $aFrame = $aFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aCksum = 0, $nativeCksum = 0, $pgno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $nativeCksum = sp + 8|0;
 $aCksum = sp + 4|0;
 $pgno = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $piPage;
 HEAP32[$3>>2] = $pnTruncate;
 HEAP32[$4>>2] = $aData;
 HEAP32[$5>>2] = $aFrame;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = ((($7)) + 24|0);
 HEAP32[$aCksum>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = ((($10)) + 32|0);
 $12 = HEAP32[$5>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = (_memcmp($11,$13,8)|0);
 $15 = ($14|0)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $16 = HEAP32[$5>>2]|0;
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$pgno>>2] = $17;
 $18 = HEAP32[$pgno>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 52|0);
 $22 = ((($21)) + 13|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = HEAP8[352>>0]|0;
 $26 = $25 << 24 >> 24;
 $27 = ($26|0)==(0);
 $28 = $27&1;
 $29 = ($24|0)==($28|0);
 $30 = $29&1;
 HEAP32[$nativeCksum>>2] = $30;
 $31 = HEAP32[$nativeCksum>>2]|0;
 $32 = HEAP32[$5>>2]|0;
 $33 = HEAP32[$aCksum>>2]|0;
 $34 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($31,$32,8,$33,$34);
 $35 = HEAP32[$nativeCksum>>2]|0;
 $36 = HEAP32[$4>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 36|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$aCksum>>2]|0;
 $41 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($35,$36,$39,$40,$41);
 $42 = HEAP32[$aCksum>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$5>>2]|0;
 $45 = ((($44)) + 16|0);
 $46 = (_sqlite3Get4byte($45)|0);
 $47 = ($43|0)!=($46|0);
 if (!($47)) {
  $48 = HEAP32[$aCksum>>2]|0;
  $49 = ((($48)) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$5>>2]|0;
  $52 = ((($51)) + 20|0);
  $53 = (_sqlite3Get4byte($52)|0);
  $54 = ($50|0)!=($53|0);
  if (!($54)) {
   $55 = HEAP32[$pgno>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   HEAP32[$56>>2] = $55;
   $57 = HEAP32[$5>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = (_sqlite3Get4byte($58)|0);
   $60 = HEAP32[$3>>2]|0;
   HEAP32[$60>>2] = $59;
   HEAP32[$0>>2] = 1;
   $61 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($61|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3OsShmBarrier($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 FUNCTION_TABLE_vi[$4 & 255]($5);
 STACKTOP = sp;return;
}
function _walFramePage($iFrame) {
 $iFrame = $iFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $iHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iHash = sp;
 HEAP32[$0>>2] = $iFrame;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4096)|0;
 $3 = (($2) - 4062)|0;
 $4 = (($3) - 1)|0;
 $5 = (($4>>>0) / 4096)&-1;
 HEAP32[$iHash>>2] = $5;
 $6 = HEAP32[$iHash>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _walHashGet($pWal,$iHash,$paHash,$paPgno,$piZero) {
 $pWal = $pWal|0;
 $iHash = $iHash|0;
 $paHash = $paHash|0;
 $paPgno = $paPgno|0;
 $piZero = $piZero|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iZero = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $aPgno = sp + 8|0;
 $iZero = sp + 4|0;
 $aHash = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iHash;
 HEAP32[$2>>2] = $paHash;
 HEAP32[$3>>2] = $paPgno;
 HEAP32[$4>>2] = $piZero;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_walIndexPage($5,$6,$aPgno)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $10 = HEAP32[$aPgno>>2]|0;
 $11 = ((($10)) + 16384|0);
 HEAP32[$aHash>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$aPgno>>2]|0;
  $15 = ((($14)) + 136|0);
  HEAP32[$aPgno>>2] = $15;
  HEAP32[$iZero>>2] = 0;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) - 1)|0;
  $18 = $17<<12;
  $19 = (4062 + ($18))|0;
  HEAP32[$iZero>>2] = $19;
 }
 $20 = HEAP32[$aPgno>>2]|0;
 $21 = ((($20)) + -4|0);
 $22 = HEAP32[$3>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$aHash>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$iZero>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _walHash($iPage) {
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $iPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1*383)|0;
 $3 = $2 & 8191;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3CorruptError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_sourceid()|0);
 $3 = ((($2)) + 20|0);
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $3;
 _sqlite3_log(11,18936,$vararg_buffer);
 STACKTOP = sp;return 11;
}
function _walNextHash($iPriorHash) {
 $iPriorHash = $iPriorHash|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $iPriorHash;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 1)|0;
 $3 = $2 & 8191;
 STACKTOP = sp;return ($3|0);
}
function _walEncodeFrame($pWal,$iPage,$nTruncate,$aData,$aFrame) {
 $pWal = $pWal|0;
 $iPage = $iPage|0;
 $nTruncate = $nTruncate|0;
 $aData = $aData|0;
 $aFrame = $aFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aCksum = 0, $nativeCksum = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $nativeCksum = sp + 4|0;
 $aCksum = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $nTruncate;
 HEAP32[$3>>2] = $aData;
 HEAP32[$4>>2] = $aFrame;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = ((($6)) + 24|0);
 HEAP32[$aCksum>>2] = $7;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _sqlite3Put4byte($8,$9);
 $10 = HEAP32[$4>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$2>>2]|0;
 _sqlite3Put4byte($11,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 104|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0);
 $17 = HEAP32[$4>>2]|0;
 $18 = ((($17)) + 8|0);
 if ($16) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 52|0);
  $21 = ((($20)) + 32|0);
  ;HEAP8[$18>>0]=HEAP8[$21>>0]|0;HEAP8[$18+1>>0]=HEAP8[$21+1>>0]|0;HEAP8[$18+2>>0]=HEAP8[$21+2>>0]|0;HEAP8[$18+3>>0]=HEAP8[$21+3>>0]|0;HEAP8[$18+4>>0]=HEAP8[$21+4>>0]|0;HEAP8[$18+5>>0]=HEAP8[$21+5>>0]|0;HEAP8[$18+6>>0]=HEAP8[$21+6>>0]|0;HEAP8[$18+7>>0]=HEAP8[$21+7>>0]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 52|0);
  $24 = ((($23)) + 13|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = HEAP8[352>>0]|0;
  $28 = $27 << 24 >> 24;
  $29 = ($28|0)==(0);
  $30 = $29&1;
  $31 = ($26|0)==($30|0);
  $32 = $31&1;
  HEAP32[$nativeCksum>>2] = $32;
  $33 = HEAP32[$nativeCksum>>2]|0;
  $34 = HEAP32[$4>>2]|0;
  $35 = HEAP32[$aCksum>>2]|0;
  $36 = HEAP32[$aCksum>>2]|0;
  _walChecksumBytes($33,$34,8,$35,$36);
  $37 = HEAP32[$nativeCksum>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 36|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$aCksum>>2]|0;
  $43 = HEAP32[$aCksum>>2]|0;
  _walChecksumBytes($37,$38,$41,$42,$43);
  $44 = HEAP32[$4>>2]|0;
  $45 = ((($44)) + 16|0);
  $46 = HEAP32[$aCksum>>2]|0;
  $47 = HEAP32[$46>>2]|0;
  _sqlite3Put4byte($45,$47);
  $48 = HEAP32[$4>>2]|0;
  $49 = ((($48)) + 20|0);
  $50 = HEAP32[$aCksum>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  _sqlite3Put4byte($49,$52);
  STACKTOP = sp;return;
 } else {
  dest=$18; stop=dest+16|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  STACKTOP = sp;return;
 }
}
function _walWriteToLog($p,$pContent,$iAmt,$0,$1) {
 $p = $p|0;
 $pContent = $pContent|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iFirstAmt = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $rc = sp + 12|0;
 $iFirstAmt = sp + 8|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pContent;
 HEAP32[$5>>2] = $iAmt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($16|0)<($24|0);
 $26 = ($13>>>0)<($21>>>0);
 $27 = ($16|0)==($24|0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  $30 = $6;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$5>>2]|0;
  $37 = ($36|0)<(0);
  $38 = $37 << 31 >> 31;
  $39 = (_i64Add(($32|0),($35|0),($36|0),($38|0))|0);
  $40 = tempRet0;
  $41 = HEAP32[$3>>2]|0;
  $42 = ((($41)) + 8|0);
  $43 = $42;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($40|0)>($48|0);
  $50 = ($39>>>0)>=($45>>>0);
  $51 = ($40|0)==($48|0);
  $52 = $51 & $50;
  $53 = $49 | $52;
  if ($53) {
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 8|0);
   $56 = $55;
   $57 = $56;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($56) + 4)|0;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = $6;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Subtract(($58|0),($61|0),($64|0),($67|0))|0);
   $69 = tempRet0;
   HEAP32[$iFirstAmt>>2] = $68;
   $70 = HEAP32[$3>>2]|0;
   $71 = ((($70)) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$4>>2]|0;
   $74 = HEAP32[$iFirstAmt>>2]|0;
   $75 = $6;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (($75) + 4)|0;
   $79 = $78;
   $80 = HEAP32[$79>>2]|0;
   $81 = (_sqlite3OsWrite($72,$73,$74,$77,$80)|0);
   HEAP32[$rc>>2] = $81;
   $82 = HEAP32[$rc>>2]|0;
   $83 = ($82|0)!=(0);
   if ($83) {
    $84 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $84;
    $132 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($132|0);
   }
   $85 = HEAP32[$iFirstAmt>>2]|0;
   $86 = ($85|0)<(0);
   $87 = $86 << 31 >> 31;
   $88 = $6;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (($88) + 4)|0;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = (_i64Add(($90|0),($93|0),($85|0),($87|0))|0);
   $95 = tempRet0;
   $96 = $6;
   $97 = $96;
   HEAP32[$97>>2] = $94;
   $98 = (($96) + 4)|0;
   $99 = $98;
   HEAP32[$99>>2] = $95;
   $100 = HEAP32[$iFirstAmt>>2]|0;
   $101 = HEAP32[$5>>2]|0;
   $102 = (($101) - ($100))|0;
   HEAP32[$5>>2] = $102;
   $103 = HEAP32[$iFirstAmt>>2]|0;
   $104 = HEAP32[$4>>2]|0;
   $105 = (($104) + ($103)|0);
   HEAP32[$4>>2] = $105;
   $106 = HEAP32[$3>>2]|0;
   $107 = ((($106)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = HEAP32[$3>>2]|0;
   $110 = ((($109)) + 16|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $111 & 19;
   $113 = (_sqlite3OsSync($108,$112)|0);
   HEAP32[$rc>>2] = $113;
   $114 = HEAP32[$5>>2]|0;
   $115 = ($114|0)==(0);
   $116 = HEAP32[$rc>>2]|0;
   $117 = ($116|0)!=(0);
   $or$cond = $115 | $117;
   if ($or$cond) {
    $118 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $118;
    $132 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($132|0);
   }
  }
 }
 $119 = HEAP32[$3>>2]|0;
 $120 = ((($119)) + 4|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$4>>2]|0;
 $123 = HEAP32[$5>>2]|0;
 $124 = $6;
 $125 = $124;
 $126 = HEAP32[$125>>2]|0;
 $127 = (($124) + 4)|0;
 $128 = $127;
 $129 = HEAP32[$128>>2]|0;
 $130 = (_sqlite3OsWrite($121,$122,$123,$126,$129)|0);
 HEAP32[$rc>>2] = $130;
 $131 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $131;
 $132 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _sqlite3BeginBenignMalloc() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[1764>>2]|0;
 $1 = ($0|0)!=(0|0);
 if (!($1)) {
  return;
 }
 $2 = HEAP32[1764>>2]|0;
 FUNCTION_TABLE_v[$2 & 255]();
 return;
}
function _sqlite3OsTruncate($id,$0,$1) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $id;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = $3;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (FUNCTION_TABLE_iiii[$11 & 255]($12,$15,$18)|0);
 STACKTOP = sp;return ($19|0);
}
function _sqlite3EndBenignMalloc() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(1768)>>2]|0;
 $1 = ($0|0)!=(0|0);
 if (!($1)) {
  return;
 }
 $2 = HEAP32[(1768)>>2]|0;
 FUNCTION_TABLE_v[$2 & 255]();
 return;
}
function _walCleanupHash($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $i = 0, $iLimit = 0, $iZero = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $aHash = sp + 20|0;
 $aPgno = sp + 16|0;
 $iZero = sp + 12|0;
 $iLimit = sp + 8|0;
 $nByte = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$aHash>>2] = 0;
 HEAP32[$aPgno>>2] = 0;
 HEAP32[$iZero>>2] = 0;
 HEAP32[$iLimit>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_walFramePage($10)|0);
 (_walHashGet($6,$11,$aHash,$aPgno,$iZero)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$iZero>>2]|0;
 $17 = (($15) - ($16))|0;
 HEAP32[$iLimit>>2] = $17;
 HEAP32[$i>>2] = 0;
 while(1) {
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)<(8192);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$aHash>>2]|0;
  $22 = (($21) + ($20<<1)|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23&65535;
  $25 = HEAP32[$iLimit>>2]|0;
  $26 = ($24|0)>($25|0);
  if ($26) {
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$aHash>>2]|0;
   $29 = (($28) + ($27<<1)|0);
   HEAP16[$29>>1] = 0;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$aHash>>2]|0;
 $33 = HEAP32[$iLimit>>2]|0;
 $34 = (($33) + 1)|0;
 $35 = HEAP32[$aPgno>>2]|0;
 $36 = (($35) + ($34<<2)|0);
 $37 = $32;
 $38 = $36;
 $39 = (($37) - ($38))|0;
 HEAP32[$nByte>>2] = $39;
 $40 = HEAP32[$iLimit>>2]|0;
 $41 = (($40) + 1)|0;
 $42 = HEAP32[$aPgno>>2]|0;
 $43 = (($42) + ($41<<2)|0);
 $44 = HEAP32[$nByte>>2]|0;
 _memset(($43|0),0,($44|0))|0;
 STACKTOP = sp;return;
}
function _backupUpdate($p,$iPage,$aData) {
 $p = $p|0;
 $iPage = $iPage|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $aData;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 28|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = (_isFatalError($5)|0);
  $7 = ($6|0)!=(0);
  if (!($7)) {
   $8 = HEAP32[$1>>2]|0;
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 16|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = ($8>>>0)<($11>>>0);
   if ($12) {
    $13 = HEAP32[$0>>2]|0;
    $14 = HEAP32[$1>>2]|0;
    $15 = HEAP32[$2>>2]|0;
    $16 = (_backupOnePage($13,$14,$15,1)|0);
    HEAP32[$rc>>2] = $16;
    $17 = HEAP32[$rc>>2]|0;
    $18 = ($17|0)!=(0);
    if ($18) {
     $19 = HEAP32[$rc>>2]|0;
     $20 = HEAP32[$0>>2]|0;
     $21 = ((($20)) + 28|0);
     HEAP32[$21>>2] = $19;
    }
   }
  }
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 44|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$0>>2] = $24;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3PagerExclusiveLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (_pagerUseWal($6)|0);
  $8 = (0)==($7|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = (_pager_wait_on_lock($9,4)|0);
   HEAP32[$rc>>2] = $10;
  }
 }
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _journalHdrOffset($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $offset = sp + 8|0;
 $c = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = $offset;
 $2 = $1;
 HEAP32[$2>>2] = 0;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $c;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 $17 = $c;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($19|0)!=(0);
 $24 = ($22|0)!=(0);
 $25 = $23 | $24;
 if (!($25)) {
  $50 = $offset;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  tempRet0 = ($55);
  STACKTOP = sp;return ($52|0);
 }
 $26 = $c;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (_i64Subtract(($28|0),($31|0),1,0)|0);
 $33 = tempRet0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 156|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (___divdi3(($32|0),($33|0),($36|0),0)|0);
 $38 = tempRet0;
 $39 = (_i64Add(($37|0),($38|0),1,0)|0);
 $40 = tempRet0;
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 156|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (___muldi3(($39|0),($40|0),($43|0),0)|0);
 $45 = tempRet0;
 $46 = $offset;
 $47 = $46;
 HEAP32[$47>>2] = $44;
 $48 = (($46) + 4)|0;
 $49 = $48;
 HEAP32[$49>>2] = $45;
 $50 = $offset;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + 4)|0;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 tempRet0 = ($55);
 STACKTOP = sp;return ($52|0);
}
function _writeJournalHdr($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ii = 0, $nHeader = 0, $nWrite = 0, $rc = 0;
 var $zHeader = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $rc = sp + 16|0;
 $zHeader = sp + 12|0;
 $nHeader = sp + 8|0;
 $nWrite = sp + 4|0;
 $ii = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$zHeader>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 160|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$nHeader>>2] = $6;
 $7 = HEAP32[$nHeader>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 156|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7>>>0)>($10>>>0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 156|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$nHeader>>2] = $14;
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $15 = HEAP32[$ii>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 104|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$ii>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = ((($21)) + 100|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*48)|0)|0);
  $25 = ((($24)) + 8|0);
  $26 = $25;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)==(0);
  $33 = ($31|0)==(0);
  $34 = $32 & $33;
  if ($34) {
   $35 = HEAP32[$0>>2]|0;
   $36 = ((($35)) + 80|0);
   $37 = $36;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($37) + 4)|0;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$ii>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = ((($44)) + 100|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + (($43*48)|0)|0);
   $48 = ((($47)) + 8|0);
   $49 = $48;
   $50 = $49;
   HEAP32[$50>>2] = $39;
   $51 = (($49) + 4)|0;
   $52 = $51;
   HEAP32[$52>>2] = $42;
  }
  $53 = HEAP32[$ii>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$ii>>2] = $54;
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (_journalHdrOffset($55)|0);
 $57 = tempRet0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 80|0);
 $60 = $59;
 $61 = $60;
 HEAP32[$61>>2] = $56;
 $62 = (($60) + 4)|0;
 $63 = $62;
 HEAP32[$63>>2] = $57;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 88|0);
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $56;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $57;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 7|0);
 $72 = HEAP8[$71>>0]|0;
 $73 = $72&255;
 $74 = ($73|0)!=(0);
 if ($74) {
  label = 11;
 } else {
  $75 = HEAP32[$0>>2]|0;
  $76 = ((($75)) + 5|0);
  $77 = HEAP8[$76>>0]|0;
  $78 = $77&255;
  $79 = ($78|0)==(4);
  if ($79) {
   label = 11;
  } else {
   $80 = HEAP32[$0>>2]|0;
   $81 = ((($80)) + 64|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (_sqlite3OsDeviceCharacteristics($82)|0);
   $84 = $83 & 512;
   $85 = ($84|0)!=(0);
   if ($85) {
    label = 11;
   } else {
    $89 = HEAP32[$zHeader>>2]|0;
    dest=$89; stop=dest+12|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
   }
  }
 }
 if ((label|0) == 11) {
  $86 = HEAP32[$zHeader>>2]|0;
  ;HEAP8[$86>>0]=HEAP8[19004>>0]|0;HEAP8[$86+1>>0]=HEAP8[19004+1>>0]|0;HEAP8[$86+2>>0]=HEAP8[19004+2>>0]|0;HEAP8[$86+3>>0]=HEAP8[19004+3>>0]|0;HEAP8[$86+4>>0]=HEAP8[19004+4>>0]|0;HEAP8[$86+5>>0]=HEAP8[19004+5>>0]|0;HEAP8[$86+6>>0]=HEAP8[19004+6>>0]|0;HEAP8[$86+7>>0]=HEAP8[19004+7>>0]|0;
  $87 = HEAP32[$zHeader>>2]|0;
  $88 = ((($87)) + 8|0);
  _sqlite3Put4byte($88,-1);
 }
 $90 = HEAP32[$0>>2]|0;
 $91 = ((($90)) + 52|0);
 _sqlite3_randomness(4,$91);
 $92 = HEAP32[$zHeader>>2]|0;
 $93 = ((($92)) + 12|0);
 $94 = HEAP32[$0>>2]|0;
 $95 = ((($94)) + 52|0);
 $96 = HEAP32[$95>>2]|0;
 _sqlite3Put4byte($93,$96);
 $97 = HEAP32[$zHeader>>2]|0;
 $98 = ((($97)) + 16|0);
 $99 = HEAP32[$0>>2]|0;
 $100 = ((($99)) + 32|0);
 $101 = HEAP32[$100>>2]|0;
 _sqlite3Put4byte($98,$101);
 $102 = HEAP32[$zHeader>>2]|0;
 $103 = ((($102)) + 20|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = ((($104)) + 156|0);
 $106 = HEAP32[$105>>2]|0;
 _sqlite3Put4byte($103,$106);
 $107 = HEAP32[$zHeader>>2]|0;
 $108 = ((($107)) + 24|0);
 $109 = HEAP32[$0>>2]|0;
 $110 = ((($109)) + 160|0);
 $111 = HEAP32[$110>>2]|0;
 _sqlite3Put4byte($108,$111);
 $112 = HEAP32[$zHeader>>2]|0;
 $113 = ((($112)) + 28|0);
 $114 = HEAP32[$nHeader>>2]|0;
 $115 = (($114) - 28)|0;
 _memset(($113|0),0,($115|0))|0;
 HEAP32[$nWrite>>2] = 0;
 while(1) {
  $116 = HEAP32[$rc>>2]|0;
  $117 = ($116|0)==(0);
  if (!($117)) {
   label = 17;
   break;
  }
  $118 = HEAP32[$nWrite>>2]|0;
  $119 = HEAP32[$0>>2]|0;
  $120 = ((($119)) + 156|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($118>>>0)<($121>>>0);
  if (!($122)) {
   label = 17;
   break;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = ((($123)) + 68|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = HEAP32[$zHeader>>2]|0;
  $127 = HEAP32[$nHeader>>2]|0;
  $128 = HEAP32[$0>>2]|0;
  $129 = ((($128)) + 80|0);
  $130 = $129;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  $136 = (_sqlite3OsWrite($125,$126,$127,$132,$135)|0);
  HEAP32[$rc>>2] = $136;
  $137 = HEAP32[$nHeader>>2]|0;
  $138 = HEAP32[$0>>2]|0;
  $139 = ((($138)) + 80|0);
  $140 = $139;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = (($140) + 4)|0;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (_i64Add(($142|0),($145|0),($137|0),0)|0);
  $147 = tempRet0;
  $148 = $139;
  $149 = $148;
  HEAP32[$149>>2] = $146;
  $150 = (($148) + 4)|0;
  $151 = $150;
  HEAP32[$151>>2] = $147;
  $152 = HEAP32[$nHeader>>2]|0;
  $153 = HEAP32[$nWrite>>2]|0;
  $154 = (($153) + ($152))|0;
  HEAP32[$nWrite>>2] = $154;
 }
 if ((label|0) == 17) {
  $155 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 return (0)|0;
}
function _sqlite3PcacheClearSyncFlags($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 24|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7&65535;
  $9 = $8 & -9;
  $10 = $9&65535;
  HEAP16[$6>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _pager_wait_on_lock($pPager,$locktype) {
 $pPager = $pPager|0;
 $locktype = $locktype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $locktype;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = (_pagerLockDb($2,$3)|0);
  HEAP32[$rc>>2] = $4;
  $5 = HEAP32[$rc>>2]|0;
  $6 = ($5|0)==(5);
  if (!($6)) {
   label = 4;
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 184|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 188|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (FUNCTION_TABLE_ii[$9 & 255]($12)|0);
  $14 = ($13|0)!=(0);
  if (!($14)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _pagerLockDb($pPager,$eLock) {
 $pPager = $pPager|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eLock;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 18|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5|0)<($6|0);
 if (!($7)) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 18|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)==(5);
  if (!($12)) {
   $37 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 14|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(0);
 if ($17) {
  $23 = 0;
 } else {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 64|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (_sqlite3OsLock($20,$21)|0);
  $23 = $22;
 }
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if (!($25)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 18|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)!=(5);
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)==(4);
 $or$cond = $30 | $32;
 if (!($or$cond)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = $33&255;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 18|0);
 HEAP8[$36>>0] = $34;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3OsLock($id,$lockType) {
 $id = $id|0;
 $lockType = $lockType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $lockType;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3OsFileControlHint($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (FUNCTION_TABLE_iiii[$6 & 255]($7,$8,$9)|0);
 STACKTOP = sp;return;
}
function _pcacheManageDirtyList($pPage,$addRemove) {
 $pPage = $pPage|0;
 $addRemove = $addRemove|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $p = 0, $pSynced = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 12|0;
 $p = sp + 4|0;
 $pSynced = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP8[$1>>0] = $addRemove;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP8[$1>>0]|0;
 $6 = $5&255;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ($11|0)==($12|0);
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 36|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pSynced>>2] = $16;
   while(1) {
    $17 = HEAP32[$pSynced>>2]|0;
    $18 = ($17|0)!=(0|0);
    if ($18) {
     $19 = HEAP32[$pSynced>>2]|0;
     $20 = ((($19)) + 24|0);
     $21 = HEAP16[$20>>1]|0;
     $22 = $21&65535;
     $23 = $22 & 8;
     $24 = ($23|0)!=(0);
     $109 = $24;
    } else {
     $109 = 0;
    }
    $25 = HEAP32[$pSynced>>2]|0;
    if (!($109)) {
     break;
    }
    $26 = ((($25)) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pSynced>>2] = $27;
   }
   $28 = HEAP32[$p>>2]|0;
   $29 = ((($28)) + 8|0);
   HEAP32[$29>>2] = $25;
  }
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)!=(0|0);
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  if ($33) {
   $37 = HEAP32[$0>>2]|0;
   $38 = ((($37)) + 32|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 36|0);
   HEAP32[$40>>2] = $36;
  } else {
   $41 = HEAP32[$p>>2]|0;
   $42 = ((($41)) + 4|0);
   HEAP32[$42>>2] = $36;
  }
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)!=(0|0);
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 32|0);
  $49 = HEAP32[$48>>2]|0;
  if ($46) {
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 36|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ((($52)) + 32|0);
   HEAP32[$53>>2] = $49;
  } else {
   $54 = HEAP32[$p>>2]|0;
   HEAP32[$54>>2] = $49;
   $55 = HEAP32[$p>>2]|0;
   $56 = HEAP32[$55>>2]|0;
   $57 = ($56|0)==(0|0);
   if ($57) {
    $58 = HEAP32[$p>>2]|0;
    $59 = ((($58)) + 32|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60&255;
    $62 = ($61|0)!=(0);
    if ($62) {
     $63 = HEAP32[$p>>2]|0;
     $64 = ((($63)) + 33|0);
     HEAP8[$64>>0] = 2;
    }
   }
  }
  $65 = HEAP32[$0>>2]|0;
  $66 = ((($65)) + 32|0);
  HEAP32[$66>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  $68 = ((($67)) + 36|0);
  HEAP32[$68>>2] = 0;
 }
 $69 = HEAP8[$1>>0]|0;
 $70 = $69&255;
 $71 = $70 & 2;
 $72 = ($71|0)!=(0);
 if (!($72)) {
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$p>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 32|0);
 HEAP32[$76>>2] = $74;
 $77 = HEAP32[$0>>2]|0;
 $78 = ((($77)) + 32|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = ($79|0)!=(0|0);
 $81 = HEAP32[$0>>2]|0;
 if ($80) {
  $82 = HEAP32[$0>>2]|0;
  $83 = ((($82)) + 32|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = ((($84)) + 36|0);
  HEAP32[$85>>2] = $81;
 } else {
  $86 = HEAP32[$p>>2]|0;
  $87 = ((($86)) + 4|0);
  HEAP32[$87>>2] = $81;
  $88 = HEAP32[$p>>2]|0;
  $89 = ((($88)) + 32|0);
  $90 = HEAP8[$89>>0]|0;
  $91 = ($90<<24>>24)!=(0);
  if ($91) {
   $92 = HEAP32[$p>>2]|0;
   $93 = ((($92)) + 33|0);
   HEAP8[$93>>0] = 1;
  }
 }
 $94 = HEAP32[$0>>2]|0;
 $95 = HEAP32[$p>>2]|0;
 HEAP32[$95>>2] = $94;
 $96 = HEAP32[$p>>2]|0;
 $97 = ((($96)) + 8|0);
 $98 = HEAP32[$97>>2]|0;
 $99 = ($98|0)!=(0|0);
 if ($99) {
  STACKTOP = sp;return;
 }
 $100 = HEAP32[$0>>2]|0;
 $101 = ((($100)) + 24|0);
 $102 = HEAP16[$101>>1]|0;
 $103 = $102&65535;
 $104 = $103 & 8;
 $105 = (0)==($104|0);
 if (!($105)) {
  STACKTOP = sp;return;
 }
 $106 = HEAP32[$0>>2]|0;
 $107 = HEAP32[$p>>2]|0;
 $108 = ((($107)) + 8|0);
 HEAP32[$108>>2] = $106;
 STACKTOP = sp;return;
}
function _pcacheUnpin($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[(144)>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 44|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 FUNCTION_TABLE_viii[$7 & 255]($12,$14,0);
 STACKTOP = sp;return;
}
function _pcache1Free($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $nFreed = 0, $pSlot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $nFreed = sp + 4|0;
 $pSlot = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nFreed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[(1660)>>2]|0;
 $5 = ($3>>>0)>=($4>>>0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[(1664)>>2]|0;
  $8 = ($6>>>0)<($7>>>0);
  if ($8) {
   _sqlite3StatusDown(1,1);
   $9 = HEAP32[$0>>2]|0;
   HEAP32[$pSlot>>2] = $9;
   $10 = HEAP32[(1672)>>2]|0;
   $11 = HEAP32[$pSlot>>2]|0;
   HEAP32[$11>>2] = $10;
   $12 = HEAP32[$pSlot>>2]|0;
   HEAP32[(1672)>>2] = $12;
   $13 = HEAP32[(1676)>>2]|0;
   $14 = (($13) + 1)|0;
   HEAP32[(1676)>>2] = $14;
   $15 = HEAP32[(1676)>>2]|0;
   $16 = HEAP32[(1656)>>2]|0;
   $17 = ($15|0)<($16|0);
   $18 = $17&1;
   HEAP32[(1680)>>2] = $18;
   STACKTOP = sp;return;
  }
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3MallocSize($19)|0);
 HEAP32[$nFreed>>2] = $20;
 $21 = HEAP32[$nFreed>>2]|0;
 _sqlite3StatusDown(2,$21);
 $22 = HEAP32[$0>>2]|0;
 _sqlite3_free($22);
 STACKTOP = sp;return;
}
function _sqlite3PagerGetExtra($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerPageRefcount($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PcachePageRefcount($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _btreeInitPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cellOffset = 0, $data = 0, $hdr = 0, $i = 0, $iCellFirst = 0, $iCellLast = 0, $nFree = 0, $next = 0;
 var $pBt = 0, $pc = 0, $size = 0, $sz = 0, $top = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $pc = sp + 50|0;
 $hdr = sp + 52|0;
 $data = sp + 32|0;
 $pBt = sp + 28|0;
 $usableSize = sp + 24|0;
 $cellOffset = sp + 48|0;
 $nFree = sp + 20|0;
 $top = sp + 16|0;
 $iCellFirst = sp + 12|0;
 $iCellLast = sp + 8|0;
 $i = sp + 4|0;
 $sz = sp;
 $next = sp + 46|0;
 $size = sp + 44|0;
 HEAP32[$1>>2] = $pPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 do {
  if (!($4)) {
   $5 = HEAP32[$1>>2]|0;
   $6 = ((($5)) + 52|0);
   $7 = HEAP32[$6>>2]|0;
   HEAP32[$pBt>>2] = $7;
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 5|0);
   $10 = HEAP8[$9>>0]|0;
   HEAP8[$hdr>>0] = $10;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 56|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$data>>2] = $13;
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP8[$hdr>>0]|0;
   $16 = $15&255;
   $17 = HEAP32[$data>>2]|0;
   $18 = (($17) + ($16)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = (_decodeFlags($14,$20)|0);
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = (_sqlite3CorruptError(57320)|0);
    HEAP32[$0>>2] = $23;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   $24 = HEAP32[$pBt>>2]|0;
   $25 = ((($24)) + 32|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) - 1)|0;
   $28 = $27&65535;
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 20|0);
   HEAP16[$30>>1] = $28;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 1|0);
   HEAP8[$32>>0] = 0;
   $33 = HEAP32[$pBt>>2]|0;
   $34 = ((($33)) + 36|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$usableSize>>2] = $35;
   $36 = HEAP8[$hdr>>0]|0;
   $37 = $36&255;
   $38 = (($37) + 8)|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 6|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = (($38) + ($42))|0;
   $44 = $43&65535;
   HEAP16[$cellOffset>>1] = $44;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 14|0);
   HEAP16[$46>>1] = $44;
   $47 = HEAP32[$usableSize>>2]|0;
   $48 = HEAP32[$data>>2]|0;
   $49 = (($48) + ($47)|0);
   $50 = HEAP32[$1>>2]|0;
   $51 = ((($50)) + 60|0);
   HEAP32[$51>>2] = $49;
   $52 = HEAP16[$cellOffset>>1]|0;
   $53 = $52&65535;
   $54 = HEAP32[$data>>2]|0;
   $55 = (($54) + ($53)|0);
   $56 = HEAP32[$1>>2]|0;
   $57 = ((($56)) + 64|0);
   HEAP32[$57>>2] = $55;
   $58 = HEAP32[$1>>2]|0;
   $59 = ((($58)) + 6|0);
   $60 = HEAP8[$59>>0]|0;
   $61 = $60&255;
   $62 = HEAP32[$data>>2]|0;
   $63 = (($62) + ($61)|0);
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 68|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP8[$hdr>>0]|0;
   $67 = $66&255;
   $68 = (($67) + 5)|0;
   $69 = HEAP32[$data>>2]|0;
   $70 = (($69) + ($68)|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = $72 << 8;
   $74 = HEAP8[$hdr>>0]|0;
   $75 = $74&255;
   $76 = (($75) + 5)|0;
   $77 = HEAP32[$data>>2]|0;
   $78 = (($77) + ($76)|0);
   $79 = ((($78)) + 1|0);
   $80 = HEAP8[$79>>0]|0;
   $81 = $80&255;
   $82 = $73 | $81;
   $83 = (($82) - 1)|0;
   $84 = $83 & 65535;
   $85 = (($84) + 1)|0;
   HEAP32[$top>>2] = $85;
   $86 = HEAP8[$hdr>>0]|0;
   $87 = $86&255;
   $88 = (($87) + 3)|0;
   $89 = HEAP32[$data>>2]|0;
   $90 = (($89) + ($88)|0);
   $91 = HEAP8[$90>>0]|0;
   $92 = $91&255;
   $93 = $92 << 8;
   $94 = HEAP8[$hdr>>0]|0;
   $95 = $94&255;
   $96 = (($95) + 3)|0;
   $97 = HEAP32[$data>>2]|0;
   $98 = (($97) + ($96)|0);
   $99 = ((($98)) + 1|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = $93 | $101;
   $103 = $102&65535;
   $104 = HEAP32[$1>>2]|0;
   $105 = ((($104)) + 18|0);
   HEAP16[$105>>1] = $103;
   $106 = HEAP32[$1>>2]|0;
   $107 = ((($106)) + 18|0);
   $108 = HEAP16[$107>>1]|0;
   $109 = $108&65535;
   $110 = HEAP32[$pBt>>2]|0;
   $111 = ((($110)) + 32|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = (($112) - 8)|0;
   $114 = (($113>>>0) / 6)&-1;
   $115 = ($109>>>0)>($114>>>0);
   if ($115) {
    $116 = (_sqlite3CorruptError(57338)|0);
    HEAP32[$0>>2] = $116;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   $117 = HEAP16[$cellOffset>>1]|0;
   $118 = $117&65535;
   $119 = HEAP32[$1>>2]|0;
   $120 = ((($119)) + 18|0);
   $121 = HEAP16[$120>>1]|0;
   $122 = $121&65535;
   $123 = $122<<1;
   $124 = (($118) + ($123))|0;
   HEAP32[$iCellFirst>>2] = $124;
   $125 = HEAP32[$usableSize>>2]|0;
   $126 = (($125) - 4)|0;
   HEAP32[$iCellLast>>2] = $126;
   $127 = HEAP32[$pBt>>2]|0;
   $128 = ((($127)) + 4|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = ((($129)) + 24|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = $131 & 268435456;
   $133 = ($132|0)!=(0);
   do {
    if ($133) {
     $134 = HEAP32[$1>>2]|0;
     $135 = ((($134)) + 4|0);
     $136 = HEAP8[$135>>0]|0;
     $137 = ($136<<24>>24)!=(0);
     if (!($137)) {
      $138 = HEAP32[$iCellLast>>2]|0;
      $139 = (($138) + -1)|0;
      HEAP32[$iCellLast>>2] = $139;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $140 = HEAP32[$i>>2]|0;
      $141 = HEAP32[$1>>2]|0;
      $142 = ((($141)) + 18|0);
      $143 = HEAP16[$142>>1]|0;
      $144 = $143&65535;
      $145 = ($140|0)<($144|0);
      if (!($145)) {
       label = 17;
       break;
      }
      $146 = HEAP16[$cellOffset>>1]|0;
      $147 = $146&65535;
      $148 = HEAP32[$i>>2]|0;
      $149 = $148<<1;
      $150 = (($147) + ($149))|0;
      $151 = HEAP32[$data>>2]|0;
      $152 = (($151) + ($150)|0);
      $153 = HEAP8[$152>>0]|0;
      $154 = $153&255;
      $155 = $154 << 8;
      $156 = HEAP16[$cellOffset>>1]|0;
      $157 = $156&65535;
      $158 = HEAP32[$i>>2]|0;
      $159 = $158<<1;
      $160 = (($157) + ($159))|0;
      $161 = HEAP32[$data>>2]|0;
      $162 = (($161) + ($160)|0);
      $163 = ((($162)) + 1|0);
      $164 = HEAP8[$163>>0]|0;
      $165 = $164&255;
      $166 = $155 | $165;
      $167 = $166&65535;
      HEAP16[$pc>>1] = $167;
      $168 = HEAP16[$pc>>1]|0;
      $169 = $168&65535;
      $170 = HEAP32[$iCellFirst>>2]|0;
      $171 = ($169|0)<($170|0);
      if ($171) {
       label = 13;
       break;
      }
      $172 = HEAP16[$pc>>1]|0;
      $173 = $172&65535;
      $174 = HEAP32[$iCellLast>>2]|0;
      $175 = ($173|0)>($174|0);
      if ($175) {
       label = 13;
       break;
      }
      $177 = HEAP32[$1>>2]|0;
      $178 = ((($177)) + 76|0);
      $179 = HEAP32[$178>>2]|0;
      $180 = HEAP32[$1>>2]|0;
      $181 = HEAP16[$pc>>1]|0;
      $182 = $181&65535;
      $183 = HEAP32[$data>>2]|0;
      $184 = (($183) + ($182)|0);
      $185 = (FUNCTION_TABLE_iii[$179 & 255]($180,$184)|0);
      $186 = $185&65535;
      HEAP32[$sz>>2] = $186;
      $187 = HEAP16[$pc>>1]|0;
      $188 = $187&65535;
      $189 = HEAP32[$sz>>2]|0;
      $190 = (($188) + ($189))|0;
      $191 = HEAP32[$usableSize>>2]|0;
      $192 = ($190|0)>($191|0);
      if ($192) {
       label = 15;
       break;
      }
      $194 = HEAP32[$i>>2]|0;
      $195 = (($194) + 1)|0;
      HEAP32[$i>>2] = $195;
     }
     if ((label|0) == 13) {
      $176 = (_sqlite3CorruptError(57366)|0);
      HEAP32[$0>>2] = $176;
      $311 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($311|0);
     }
     else if ((label|0) == 15) {
      $193 = (_sqlite3CorruptError(57371)|0);
      HEAP32[$0>>2] = $193;
      $311 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($311|0);
     }
     else if ((label|0) == 17) {
      $196 = HEAP32[$1>>2]|0;
      $197 = ((($196)) + 4|0);
      $198 = HEAP8[$197>>0]|0;
      $199 = ($198<<24>>24)!=(0);
      if ($199) {
       break;
      }
      $200 = HEAP32[$iCellLast>>2]|0;
      $201 = (($200) + 1)|0;
      HEAP32[$iCellLast>>2] = $201;
      break;
     }
    }
   } while(0);
   $202 = HEAP8[$hdr>>0]|0;
   $203 = $202&255;
   $204 = (($203) + 1)|0;
   $205 = HEAP32[$data>>2]|0;
   $206 = (($205) + ($204)|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $207&255;
   $209 = $208 << 8;
   $210 = HEAP8[$hdr>>0]|0;
   $211 = $210&255;
   $212 = (($211) + 1)|0;
   $213 = HEAP32[$data>>2]|0;
   $214 = (($213) + ($212)|0);
   $215 = ((($214)) + 1|0);
   $216 = HEAP8[$215>>0]|0;
   $217 = $216&255;
   $218 = $209 | $217;
   $219 = $218&65535;
   HEAP16[$pc>>1] = $219;
   $220 = HEAP8[$hdr>>0]|0;
   $221 = $220&255;
   $222 = (($221) + 7)|0;
   $223 = HEAP32[$data>>2]|0;
   $224 = (($223) + ($222)|0);
   $225 = HEAP8[$224>>0]|0;
   $226 = $225&255;
   $227 = HEAP32[$top>>2]|0;
   $228 = (($226) + ($227))|0;
   HEAP32[$nFree>>2] = $228;
   while(1) {
    $229 = HEAP16[$pc>>1]|0;
    $230 = $229&65535;
    $231 = ($230|0)>(0);
    if (!($231)) {
     label = 29;
     break;
    }
    $232 = HEAP16[$pc>>1]|0;
    $233 = $232&65535;
    $234 = HEAP32[$iCellFirst>>2]|0;
    $235 = ($233|0)<($234|0);
    if ($235) {
     label = 23;
     break;
    }
    $236 = HEAP16[$pc>>1]|0;
    $237 = $236&65535;
    $238 = HEAP32[$iCellLast>>2]|0;
    $239 = ($237|0)>($238|0);
    if ($239) {
     label = 23;
     break;
    }
    $241 = HEAP16[$pc>>1]|0;
    $242 = $241&65535;
    $243 = HEAP32[$data>>2]|0;
    $244 = (($243) + ($242)|0);
    $245 = HEAP8[$244>>0]|0;
    $246 = $245&255;
    $247 = $246 << 8;
    $248 = HEAP16[$pc>>1]|0;
    $249 = $248&65535;
    $250 = HEAP32[$data>>2]|0;
    $251 = (($250) + ($249)|0);
    $252 = ((($251)) + 1|0);
    $253 = HEAP8[$252>>0]|0;
    $254 = $253&255;
    $255 = $247 | $254;
    $256 = $255&65535;
    HEAP16[$next>>1] = $256;
    $257 = HEAP16[$pc>>1]|0;
    $258 = $257&65535;
    $259 = (($258) + 2)|0;
    $260 = HEAP32[$data>>2]|0;
    $261 = (($260) + ($259)|0);
    $262 = HEAP8[$261>>0]|0;
    $263 = $262&255;
    $264 = $263 << 8;
    $265 = HEAP16[$pc>>1]|0;
    $266 = $265&65535;
    $267 = (($266) + 2)|0;
    $268 = HEAP32[$data>>2]|0;
    $269 = (($268) + ($267)|0);
    $270 = ((($269)) + 1|0);
    $271 = HEAP8[$270>>0]|0;
    $272 = $271&255;
    $273 = $264 | $272;
    $274 = $273&65535;
    HEAP16[$size>>1] = $274;
    $275 = HEAP16[$next>>1]|0;
    $276 = $275&65535;
    $277 = ($276|0)>(0);
    if ($277) {
     $278 = HEAP16[$next>>1]|0;
     $279 = $278&65535;
     $280 = HEAP16[$pc>>1]|0;
     $281 = $280&65535;
     $282 = HEAP16[$size>>1]|0;
     $283 = $282&65535;
     $284 = (($281) + ($283))|0;
     $285 = (($284) + 3)|0;
     $286 = ($279|0)<=($285|0);
     if ($286) {
      label = 27;
      break;
     }
    }
    $287 = HEAP16[$pc>>1]|0;
    $288 = $287&65535;
    $289 = HEAP16[$size>>1]|0;
    $290 = $289&65535;
    $291 = (($288) + ($290))|0;
    $292 = HEAP32[$usableSize>>2]|0;
    $293 = ($291|0)>($292|0);
    if ($293) {
     label = 27;
     break;
    }
    $295 = HEAP32[$nFree>>2]|0;
    $296 = HEAP16[$size>>1]|0;
    $297 = $296&65535;
    $298 = (($295) + ($297))|0;
    HEAP32[$nFree>>2] = $298;
    $299 = HEAP16[$next>>1]|0;
    HEAP16[$pc>>1] = $299;
   }
   if ((label|0) == 23) {
    $240 = (_sqlite3CorruptError(57391)|0);
    HEAP32[$0>>2] = $240;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   else if ((label|0) == 27) {
    $294 = (_sqlite3CorruptError(57398)|0);
    HEAP32[$0>>2] = $294;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   else if ((label|0) == 29) {
    $300 = HEAP32[$nFree>>2]|0;
    $301 = HEAP32[$usableSize>>2]|0;
    $302 = ($300|0)>($301|0);
    if (!($302)) {
     $304 = HEAP32[$nFree>>2]|0;
     $305 = HEAP32[$iCellFirst>>2]|0;
     $306 = (($304) - ($305))|0;
     $307 = $306&65535;
     $308 = HEAP32[$1>>2]|0;
     $309 = ((($308)) + 16|0);
     HEAP16[$309>>1] = $307;
     $310 = HEAP32[$1>>2]|0;
     HEAP8[$310>>0] = 1;
     break;
    }
    $303 = (_sqlite3CorruptError(57412)|0);
    HEAP32[$0>>2] = $303;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $311 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($311|0);
}
function _sqlite3PcachePageRefcount($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 STACKTOP = sp;return ($4|0);
}
function _decodeFlags($pPage,$flagByte) {
 $pPage = $pPage|0;
 $flagByte = $flagByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $flagByte;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 >> 3;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$2>>2]|0;
 $9 = $8 & -9;
 HEAP32[$2>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13<<2;
 $15 = (4 - ($14))|0;
 $16 = $15&255;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 6|0);
 HEAP8[$18>>0] = $16;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 76|0);
 HEAP32[$20>>2] = 179;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 52|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pBt>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)==(5);
 do {
  if ($25) {
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 2|0);
   HEAP8[$27>>0] = 1;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = ($30<<24>>24)!=(0);
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 3|0);
   if ($31) {
    HEAP8[$33>>0] = 1;
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 80|0);
    HEAP32[$35>>2] = 185;
   } else {
    HEAP8[$33>>0] = 0;
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 76|0);
    HEAP32[$37>>2] = 180;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 80|0);
    HEAP32[$39>>2] = 186;
   }
   $40 = HEAP32[$pBt>>2]|0;
   $41 = ((($40)) + 28|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = HEAP32[$1>>2]|0;
   $44 = ((($43)) + 10|0);
   HEAP16[$44>>1] = $42;
   $45 = HEAP32[$pBt>>2]|0;
   $46 = ((($45)) + 30|0);
   $47 = HEAP16[$46>>1]|0;
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 12|0);
   HEAP16[$49>>1] = $47;
  } else {
   $50 = HEAP32[$2>>2]|0;
   $51 = ($50|0)==(2);
   if ($51) {
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 2|0);
    HEAP8[$53>>0] = 0;
    $54 = HEAP32[$1>>2]|0;
    $55 = ((($54)) + 3|0);
    HEAP8[$55>>0] = 0;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 80|0);
    HEAP32[$57>>2] = 187;
    $58 = HEAP32[$pBt>>2]|0;
    $59 = ((($58)) + 24|0);
    $60 = HEAP16[$59>>1]|0;
    $61 = HEAP32[$1>>2]|0;
    $62 = ((($61)) + 10|0);
    HEAP16[$62>>1] = $60;
    $63 = HEAP32[$pBt>>2]|0;
    $64 = ((($63)) + 26|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 12|0);
    HEAP16[$67>>1] = $65;
    break;
   }
   $68 = (_sqlite3CorruptError(57278)|0);
   HEAP32[$0>>2] = $68;
   $74 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($74|0);
  }
 } while(0);
 $69 = HEAP32[$pBt>>2]|0;
 $70 = ((($69)) + 21|0);
 $71 = HEAP8[$70>>0]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 7|0);
 HEAP8[$73>>0] = $71;
 HEAP32[$0>>2] = 0;
 $74 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _cellSizePtr($pPage,$pCell) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $minLocal = 0, $nSize = 0, $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pIter = sp + 12|0;
 $pEnd = sp + 8|0;
 $nSize = sp + 4|0;
 $minLocal = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 6|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (($2) + ($6)|0);
 HEAP32[$pIter>>2] = $7;
 $8 = HEAP32[$pIter>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$nSize>>2] = $10;
 $11 = HEAP32[$nSize>>2]|0;
 $12 = ($11>>>0)>=(128);
 L1: do {
  if ($12) {
   $13 = HEAP32[$pIter>>2]|0;
   $14 = ((($13)) + 8|0);
   HEAP32[$pEnd>>2] = $14;
   $15 = HEAP32[$nSize>>2]|0;
   $16 = $15 & 127;
   HEAP32[$nSize>>2] = $16;
   while(1) {
    $17 = HEAP32[$nSize>>2]|0;
    $18 = $17 << 7;
    $19 = HEAP32[$pIter>>2]|0;
    $20 = ((($19)) + 1|0);
    HEAP32[$pIter>>2] = $20;
    $21 = HEAP8[$20>>0]|0;
    $22 = $21&255;
    $23 = $22 & 127;
    $24 = $18 | $23;
    HEAP32[$nSize>>2] = $24;
    $25 = HEAP32[$pIter>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)>=(128);
    if (!($28)) {
     break L1;
    }
    $29 = HEAP32[$pIter>>2]|0;
    $30 = HEAP32[$pEnd>>2]|0;
    $31 = ($29>>>0)<($30>>>0);
    if (!($31)) {
     break;
    }
   }
  }
 } while(0);
 $32 = HEAP32[$pIter>>2]|0;
 $33 = ((($32)) + 1|0);
 HEAP32[$pIter>>2] = $33;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 2|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = ($36<<24>>24)!=(0);
 L7: do {
  if ($37) {
   $38 = HEAP32[$pIter>>2]|0;
   $39 = ((($38)) + 9|0);
   HEAP32[$pEnd>>2] = $39;
   while(1) {
    $40 = HEAP32[$pIter>>2]|0;
    $41 = ((($40)) + 1|0);
    HEAP32[$pIter>>2] = $41;
    $42 = HEAP8[$40>>0]|0;
    $43 = $42&255;
    $44 = $43 & 128;
    $45 = ($44|0)!=(0);
    if (!($45)) {
     break L7;
    }
    $46 = HEAP32[$pIter>>2]|0;
    $47 = HEAP32[$pEnd>>2]|0;
    $48 = ($46>>>0)<($47>>>0);
    if (!($48)) {
     break;
    }
   }
  }
 } while(0);
 $49 = HEAP32[$nSize>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 10|0);
 $52 = HEAP16[$51>>1]|0;
 $53 = $52&65535;
 $54 = ($49>>>0)<=($53>>>0);
 if ($54) {
  $55 = HEAP32[$pIter>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = $55;
  $58 = $56;
  $59 = (($57) - ($58))|0;
  $60 = HEAP32[$nSize>>2]|0;
  $61 = (($60) + ($59))|0;
  HEAP32[$nSize>>2] = $61;
  $62 = HEAP32[$nSize>>2]|0;
  $63 = ($62>>>0)<(4);
  $$ = $63 ? 4 : $61;
  HEAP32[$nSize>>2] = $$;
  $97 = HEAP32[$nSize>>2]|0;
  $98 = $97&65535;
  STACKTOP = sp;return ($98|0);
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 12|0);
 $66 = HEAP16[$65>>1]|0;
 $67 = $66&65535;
 HEAP32[$minLocal>>2] = $67;
 $68 = HEAP32[$minLocal>>2]|0;
 $69 = HEAP32[$nSize>>2]|0;
 $70 = HEAP32[$minLocal>>2]|0;
 $71 = (($69) - ($70))|0;
 $72 = HEAP32[$0>>2]|0;
 $73 = ((($72)) + 52|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = ((($74)) + 36|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = (($76) - 4)|0;
 $78 = (($71>>>0) % ($77>>>0))&-1;
 $79 = (($68) + ($78))|0;
 HEAP32[$nSize>>2] = $79;
 $80 = HEAP32[$nSize>>2]|0;
 $81 = HEAP32[$0>>2]|0;
 $82 = ((($81)) + 10|0);
 $83 = HEAP16[$82>>1]|0;
 $84 = $83&65535;
 $85 = ($80>>>0)>($84>>>0);
 if ($85) {
  $86 = HEAP32[$minLocal>>2]|0;
  HEAP32[$nSize>>2] = $86;
 }
 $87 = HEAP32[$pIter>>2]|0;
 $88 = HEAP32[$1>>2]|0;
 $89 = $87;
 $90 = $88;
 $91 = (($89) - ($90))|0;
 $92 = $91&65535;
 $93 = $92&65535;
 $94 = (4 + ($93))|0;
 $95 = HEAP32[$nSize>>2]|0;
 $96 = (($95) + ($94))|0;
 HEAP32[$nSize>>2] = $96;
 $97 = HEAP32[$nSize>>2]|0;
 $98 = $97&65535;
 STACKTOP = sp;return ($98|0);
}
function _btreeParseCellPtr($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iKey = 0, $nPayload = 0, $pEnd = 0;
 var $pEnd1 = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pIter = sp + 20|0;
 $nPayload = sp + 16|0;
 $iKey = sp;
 $pEnd = sp + 12|0;
 $pEnd1 = sp + 8|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pIter>>2] = $3;
 $4 = HEAP32[$pIter>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 HEAP32[$nPayload>>2] = $6;
 $7 = HEAP32[$nPayload>>2]|0;
 $8 = ($7>>>0)>=(128);
 L1: do {
  if ($8) {
   $9 = HEAP32[$pIter>>2]|0;
   $10 = ((($9)) + 8|0);
   HEAP32[$pEnd>>2] = $10;
   $11 = HEAP32[$nPayload>>2]|0;
   $12 = $11 & 127;
   HEAP32[$nPayload>>2] = $12;
   while(1) {
    $13 = HEAP32[$nPayload>>2]|0;
    $14 = $13 << 7;
    $15 = HEAP32[$pIter>>2]|0;
    $16 = ((($15)) + 1|0);
    HEAP32[$pIter>>2] = $16;
    $17 = HEAP8[$16>>0]|0;
    $18 = $17&255;
    $19 = $18 & 127;
    $20 = $14 | $19;
    HEAP32[$nPayload>>2] = $20;
    $21 = HEAP32[$pIter>>2]|0;
    $22 = HEAP8[$21>>0]|0;
    $23 = $22&255;
    $24 = ($23|0)>=(128);
    if (!($24)) {
     break L1;
    }
    $25 = HEAP32[$pIter>>2]|0;
    $26 = HEAP32[$pEnd>>2]|0;
    $27 = ($25>>>0)<($26>>>0);
    if (!($27)) {
     break;
    }
   }
  }
 } while(0);
 $28 = HEAP32[$pIter>>2]|0;
 $29 = ((($28)) + 1|0);
 HEAP32[$pIter>>2] = $29;
 $30 = HEAP32[$pIter>>2]|0;
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = $iKey;
 $34 = $33;
 HEAP32[$34>>2] = $32;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = 0;
 $37 = $iKey;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42>>>0)>(0);
 $44 = ($39>>>0)>=(128);
 $45 = ($42|0)==(0);
 $46 = $45 & $44;
 $47 = $43 | $46;
 L7: do {
  if ($47) {
   $48 = HEAP32[$pIter>>2]|0;
   $49 = ((($48)) + 7|0);
   HEAP32[$pEnd1>>2] = $49;
   $50 = $iKey;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = $52 & 127;
   $57 = $iKey;
   $58 = $57;
   HEAP32[$58>>2] = $56;
   $59 = (($57) + 4)|0;
   $60 = $59;
   HEAP32[$60>>2] = 0;
   while(1) {
    $61 = $iKey;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (($61) + 4)|0;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = (_bitshift64Shl(($63|0),($66|0),7)|0);
    $68 = tempRet0;
    $69 = HEAP32[$pIter>>2]|0;
    $70 = ((($69)) + 1|0);
    HEAP32[$pIter>>2] = $70;
    $71 = HEAP8[$70>>0]|0;
    $72 = $71&255;
    $73 = $72 & 127;
    $74 = ($73|0)<(0);
    $75 = $74 << 31 >> 31;
    $76 = $67 | $73;
    $77 = $68 | $75;
    $78 = $iKey;
    $79 = $78;
    HEAP32[$79>>2] = $76;
    $80 = (($78) + 4)|0;
    $81 = $80;
    HEAP32[$81>>2] = $77;
    $82 = HEAP32[$pIter>>2]|0;
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = ($84|0)<(128);
    if ($85) {
     break L7;
    }
    $86 = HEAP32[$pIter>>2]|0;
    $87 = HEAP32[$pEnd1>>2]|0;
    $88 = ($86>>>0)>=($87>>>0);
    if ($88) {
     break;
    }
   }
   $89 = $iKey;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = (($89) + 4)|0;
   $93 = $92;
   $94 = HEAP32[$93>>2]|0;
   $95 = (_bitshift64Shl(($91|0),($94|0),8)|0);
   $96 = tempRet0;
   $97 = HEAP32[$pIter>>2]|0;
   $98 = ((($97)) + 1|0);
   HEAP32[$pIter>>2] = $98;
   $99 = HEAP8[$98>>0]|0;
   $100 = $99&255;
   $101 = $95 | $100;
   $102 = $iKey;
   $103 = $102;
   HEAP32[$103>>2] = $101;
   $104 = (($102) + 4)|0;
   $105 = $104;
   HEAP32[$105>>2] = $96;
  }
 } while(0);
 $106 = HEAP32[$pIter>>2]|0;
 $107 = ((($106)) + 1|0);
 HEAP32[$pIter>>2] = $107;
 $108 = $iKey;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = (($108) + 4)|0;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = HEAP32[$2>>2]|0;
 $115 = $114;
 $116 = $115;
 HEAP32[$116>>2] = $110;
 $117 = (($115) + 4)|0;
 $118 = $117;
 HEAP32[$118>>2] = $113;
 $119 = HEAP32[$nPayload>>2]|0;
 $120 = HEAP32[$2>>2]|0;
 $121 = ((($120)) + 12|0);
 HEAP32[$121>>2] = $119;
 $122 = HEAP32[$pIter>>2]|0;
 $123 = HEAP32[$2>>2]|0;
 $124 = ((($123)) + 8|0);
 HEAP32[$124>>2] = $122;
 $125 = HEAP32[$nPayload>>2]|0;
 $126 = HEAP32[$0>>2]|0;
 $127 = ((($126)) + 10|0);
 $128 = HEAP16[$127>>1]|0;
 $129 = $128&65535;
 $130 = ($125>>>0)<=($129>>>0);
 if (!($130)) {
  $154 = HEAP32[$0>>2]|0;
  $155 = HEAP32[$1>>2]|0;
  $156 = HEAP32[$2>>2]|0;
  _btreeParseCellAdjustSizeForOverflow($154,$155,$156);
  STACKTOP = sp;return;
 }
 $131 = HEAP32[$nPayload>>2]|0;
 $132 = HEAP32[$pIter>>2]|0;
 $133 = HEAP32[$1>>2]|0;
 $134 = $132;
 $135 = $133;
 $136 = (($134) - ($135))|0;
 $137 = $136&65535;
 $138 = $137&65535;
 $139 = (($131) + ($138))|0;
 $140 = $139&65535;
 $141 = HEAP32[$2>>2]|0;
 $142 = ((($141)) + 18|0);
 HEAP16[$142>>1] = $140;
 $143 = HEAP32[$2>>2]|0;
 $144 = ((($143)) + 18|0);
 $145 = HEAP16[$144>>1]|0;
 $146 = $145&65535;
 $147 = ($146|0)<(4);
 if ($147) {
  $148 = HEAP32[$2>>2]|0;
  $149 = ((($148)) + 18|0);
  HEAP16[$149>>1] = 4;
 }
 $150 = HEAP32[$nPayload>>2]|0;
 $151 = $150&65535;
 $152 = HEAP32[$2>>2]|0;
 $153 = ((($152)) + 16|0);
 HEAP16[$153>>1] = $151;
 STACKTOP = sp;return;
}
function _cellSizePtrNoPayload($pPage,$pCell) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pIter = sp + 4|0;
 $pEnd = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 HEAP32[$pIter>>2] = $3;
 $4 = HEAP32[$pIter>>2]|0;
 $5 = ((($4)) + 9|0);
 HEAP32[$pEnd>>2] = $5;
 while(1) {
  $6 = HEAP32[$pIter>>2]|0;
  $7 = ((($6)) + 1|0);
  HEAP32[$pIter>>2] = $7;
  $8 = HEAP8[$6>>0]|0;
  $9 = $8&255;
  $10 = $9 & 128;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = HEAP32[$pEnd>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if (!($14)) {
   break;
  }
 }
 $15 = HEAP32[$pIter>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = $15;
 $18 = $16;
 $19 = (($17) - ($18))|0;
 $20 = $19&65535;
 STACKTOP = sp;return ($20|0);
}
function _btreeParseCellPtrNoPayload($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3GetVarint($4,$5)|0);
 $7 = $6&255;
 $8 = (4 + ($7))|0;
 $9 = $8&65535;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 18|0);
 HEAP16[$11>>1] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP16[$15>>1] = 0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeParseCellPtrIndex($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $nPayload = 0;
 var $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pIter = sp + 8|0;
 $nPayload = sp + 4|0;
 $pEnd = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 6|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = (($3) + ($7)|0);
 HEAP32[$pIter>>2] = $8;
 $9 = HEAP32[$pIter>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$nPayload>>2] = $11;
 $12 = HEAP32[$nPayload>>2]|0;
 $13 = ($12>>>0)>=(128);
 L1: do {
  if ($13) {
   $14 = HEAP32[$pIter>>2]|0;
   $15 = ((($14)) + 8|0);
   HEAP32[$pEnd>>2] = $15;
   $16 = HEAP32[$nPayload>>2]|0;
   $17 = $16 & 127;
   HEAP32[$nPayload>>2] = $17;
   while(1) {
    $18 = HEAP32[$nPayload>>2]|0;
    $19 = $18 << 7;
    $20 = HEAP32[$pIter>>2]|0;
    $21 = ((($20)) + 1|0);
    HEAP32[$pIter>>2] = $21;
    $22 = HEAP8[$21>>0]|0;
    $23 = $22&255;
    $24 = $23 & 127;
    $25 = $19 | $24;
    HEAP32[$nPayload>>2] = $25;
    $26 = HEAP32[$pIter>>2]|0;
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = ($28|0)>=(128);
    if (!($29)) {
     break L1;
    }
    $30 = HEAP32[$pIter>>2]|0;
    $31 = HEAP32[$pEnd>>2]|0;
    $32 = ($30>>>0)<($31>>>0);
    if (!($32)) {
     break;
    }
   }
  }
 } while(0);
 $33 = HEAP32[$pIter>>2]|0;
 $34 = ((($33)) + 1|0);
 HEAP32[$pIter>>2] = $34;
 $35 = HEAP32[$nPayload>>2]|0;
 $36 = HEAP32[$2>>2]|0;
 $37 = $36;
 $38 = $37;
 HEAP32[$38>>2] = $35;
 $39 = (($37) + 4)|0;
 $40 = $39;
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$nPayload>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = ((($42)) + 12|0);
 HEAP32[$43>>2] = $41;
 $44 = HEAP32[$pIter>>2]|0;
 $45 = HEAP32[$2>>2]|0;
 $46 = ((($45)) + 8|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$nPayload>>2]|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 10|0);
 $50 = HEAP16[$49>>1]|0;
 $51 = $50&65535;
 $52 = ($47>>>0)<=($51>>>0);
 if (!($52)) {
  $76 = HEAP32[$0>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  _btreeParseCellAdjustSizeForOverflow($76,$77,$78);
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$nPayload>>2]|0;
 $54 = HEAP32[$pIter>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 $56 = $54;
 $57 = $55;
 $58 = (($56) - ($57))|0;
 $59 = $58&65535;
 $60 = $59&65535;
 $61 = (($53) + ($60))|0;
 $62 = $61&65535;
 $63 = HEAP32[$2>>2]|0;
 $64 = ((($63)) + 18|0);
 HEAP16[$64>>1] = $62;
 $65 = HEAP32[$2>>2]|0;
 $66 = ((($65)) + 18|0);
 $67 = HEAP16[$66>>1]|0;
 $68 = $67&65535;
 $69 = ($68|0)<(4);
 if ($69) {
  $70 = HEAP32[$2>>2]|0;
  $71 = ((($70)) + 18|0);
  HEAP16[$71>>1] = 4;
 }
 $72 = HEAP32[$nPayload>>2]|0;
 $73 = $72&65535;
 $74 = HEAP32[$2>>2]|0;
 $75 = ((($74)) + 16|0);
 HEAP16[$75>>1] = $73;
 STACKTOP = sp;return;
}
function _btreeParseCellAdjustSizeForOverflow($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $maxLocal = 0, $minLocal = 0, $surplus = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $minLocal = sp + 8|0;
 $maxLocal = sp + 4|0;
 $surplus = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 HEAP32[$minLocal>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 10|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 HEAP32[$maxLocal>>2] = $10;
 $11 = HEAP32[$minLocal>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$minLocal>>2]|0;
 $16 = (($14) - ($15))|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 4)|0;
 $23 = (($16>>>0) % ($22>>>0))&-1;
 $24 = (($11) + ($23))|0;
 HEAP32[$surplus>>2] = $24;
 $25 = HEAP32[$surplus>>2]|0;
 $26 = HEAP32[$maxLocal>>2]|0;
 $27 = ($25|0)<=($26|0);
 if ($27) {
  $28 = HEAP32[$surplus>>2]|0;
  $29 = $28&65535;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 16|0);
  HEAP16[$31>>1] = $29;
 } else {
  $32 = HEAP32[$minLocal>>2]|0;
  $33 = $32&65535;
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 16|0);
  HEAP16[$35>>1] = $33;
 }
 $36 = HEAP32[$2>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = HEAP16[$37>>1]|0;
 $39 = $38&65535;
 $40 = HEAP32[$2>>2]|0;
 $41 = ((($40)) + 8|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($39)|0);
 $44 = HEAP32[$1>>2]|0;
 $45 = $43;
 $46 = $44;
 $47 = (($45) - ($46))|0;
 $48 = $47&65535;
 $49 = $48&65535;
 $50 = (($49) + 4)|0;
 $51 = $50&65535;
 $52 = HEAP32[$2>>2]|0;
 $53 = ((($52)) + 18|0);
 HEAP16[$53>>1] = $51;
 STACKTOP = sp;return;
}
function _sqlite3GetVarint($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $s = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $a = sp + 8|0;
 $b = sp + 4|0;
 $s = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 HEAP32[$a>>2] = $5;
 $6 = HEAP32[$a>>2]|0;
 $7 = $6 & 128;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = HEAP32[$a>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = $10;
  $12 = $11;
  HEAP32[$12>>2] = $9;
  $13 = (($11) + 4)|0;
  $14 = $13;
  HEAP32[$14>>2] = 0;
  HEAP8[$0>>0] = 1;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 1|0);
 HEAP32[$1>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 HEAP32[$b>>2] = $19;
 $20 = HEAP32[$b>>2]|0;
 $21 = $20 & 128;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $23 = HEAP32[$a>>2]|0;
  $24 = $23 & 127;
  HEAP32[$a>>2] = $24;
  $25 = HEAP32[$a>>2]|0;
  $26 = $25 << 7;
  HEAP32[$a>>2] = $26;
  $27 = HEAP32[$b>>2]|0;
  $28 = HEAP32[$a>>2]|0;
  $29 = $28 | $27;
  HEAP32[$a>>2] = $29;
  $30 = HEAP32[$a>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = $31;
  $33 = $32;
  HEAP32[$33>>2] = $30;
  $34 = (($32) + 4)|0;
  $35 = $34;
  HEAP32[$35>>2] = 0;
  HEAP8[$0>>0] = 2;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 1|0);
 HEAP32[$1>>2] = $37;
 $38 = HEAP32[$a>>2]|0;
 $39 = $38 << 14;
 HEAP32[$a>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = HEAP32[$a>>2]|0;
 $44 = $43 | $42;
 HEAP32[$a>>2] = $44;
 $45 = HEAP32[$a>>2]|0;
 $46 = $45 & 128;
 $47 = ($46|0)!=(0);
 $48 = HEAP32[$a>>2]|0;
 $49 = $48 & 2080895;
 HEAP32[$a>>2] = $49;
 if (!($47)) {
  $50 = HEAP32[$b>>2]|0;
  $51 = $50 & 127;
  HEAP32[$b>>2] = $51;
  $52 = HEAP32[$b>>2]|0;
  $53 = $52 << 7;
  HEAP32[$b>>2] = $53;
  $54 = HEAP32[$b>>2]|0;
  $55 = HEAP32[$a>>2]|0;
  $56 = $55 | $54;
  HEAP32[$a>>2] = $56;
  $57 = HEAP32[$a>>2]|0;
  $58 = HEAP32[$2>>2]|0;
  $59 = $58;
  $60 = $59;
  HEAP32[$60>>2] = $57;
  $61 = (($59) + 4)|0;
  $62 = $61;
  HEAP32[$62>>2] = 0;
  HEAP8[$0>>0] = 3;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = ((($63)) + 1|0);
 HEAP32[$1>>2] = $64;
 $65 = HEAP32[$b>>2]|0;
 $66 = $65 << 14;
 HEAP32[$b>>2] = $66;
 $67 = HEAP32[$1>>2]|0;
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = HEAP32[$b>>2]|0;
 $71 = $70 | $69;
 HEAP32[$b>>2] = $71;
 $72 = HEAP32[$b>>2]|0;
 $73 = $72 & 128;
 $74 = ($73|0)!=(0);
 $75 = HEAP32[$b>>2]|0;
 $76 = $75 & 2080895;
 HEAP32[$b>>2] = $76;
 $77 = HEAP32[$a>>2]|0;
 if (!($74)) {
  $78 = $77 << 7;
  HEAP32[$a>>2] = $78;
  $79 = HEAP32[$b>>2]|0;
  $80 = HEAP32[$a>>2]|0;
  $81 = $80 | $79;
  HEAP32[$a>>2] = $81;
  $82 = HEAP32[$a>>2]|0;
  $83 = HEAP32[$2>>2]|0;
  $84 = $83;
  $85 = $84;
  HEAP32[$85>>2] = $82;
  $86 = (($84) + 4)|0;
  $87 = $86;
  HEAP32[$87>>2] = 0;
  HEAP8[$0>>0] = 4;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 HEAP32[$s>>2] = $77;
 $88 = HEAP32[$1>>2]|0;
 $89 = ((($88)) + 1|0);
 HEAP32[$1>>2] = $89;
 $90 = HEAP32[$a>>2]|0;
 $91 = $90 << 14;
 HEAP32[$a>>2] = $91;
 $92 = HEAP32[$1>>2]|0;
 $93 = HEAP8[$92>>0]|0;
 $94 = $93&255;
 $95 = HEAP32[$a>>2]|0;
 $96 = $95 | $94;
 HEAP32[$a>>2] = $96;
 $97 = HEAP32[$a>>2]|0;
 $98 = $97 & 128;
 $99 = ($98|0)!=(0);
 if (!($99)) {
  $100 = HEAP32[$b>>2]|0;
  $101 = $100 << 7;
  HEAP32[$b>>2] = $101;
  $102 = HEAP32[$b>>2]|0;
  $103 = HEAP32[$a>>2]|0;
  $104 = $103 | $102;
  HEAP32[$a>>2] = $104;
  $105 = HEAP32[$s>>2]|0;
  $106 = $105 >>> 18;
  HEAP32[$s>>2] = $106;
  $107 = HEAP32[$s>>2]|0;
  $108 = HEAP32[$a>>2]|0;
  $109 = HEAP32[$2>>2]|0;
  $110 = $109;
  $111 = $110;
  HEAP32[$111>>2] = $108;
  $112 = (($110) + 4)|0;
  $113 = $112;
  HEAP32[$113>>2] = $107;
  HEAP8[$0>>0] = 5;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $114 = HEAP32[$s>>2]|0;
 $115 = $114 << 7;
 HEAP32[$s>>2] = $115;
 $116 = HEAP32[$b>>2]|0;
 $117 = HEAP32[$s>>2]|0;
 $118 = $117 | $116;
 HEAP32[$s>>2] = $118;
 $119 = HEAP32[$1>>2]|0;
 $120 = ((($119)) + 1|0);
 HEAP32[$1>>2] = $120;
 $121 = HEAP32[$b>>2]|0;
 $122 = $121 << 14;
 HEAP32[$b>>2] = $122;
 $123 = HEAP32[$1>>2]|0;
 $124 = HEAP8[$123>>0]|0;
 $125 = $124&255;
 $126 = HEAP32[$b>>2]|0;
 $127 = $126 | $125;
 HEAP32[$b>>2] = $127;
 $128 = HEAP32[$b>>2]|0;
 $129 = $128 & 128;
 $130 = ($129|0)!=(0);
 if (!($130)) {
  $131 = HEAP32[$a>>2]|0;
  $132 = $131 & 2080895;
  HEAP32[$a>>2] = $132;
  $133 = HEAP32[$a>>2]|0;
  $134 = $133 << 7;
  HEAP32[$a>>2] = $134;
  $135 = HEAP32[$b>>2]|0;
  $136 = HEAP32[$a>>2]|0;
  $137 = $136 | $135;
  HEAP32[$a>>2] = $137;
  $138 = HEAP32[$s>>2]|0;
  $139 = $138 >>> 18;
  HEAP32[$s>>2] = $139;
  $140 = HEAP32[$s>>2]|0;
  $141 = HEAP32[$a>>2]|0;
  $142 = HEAP32[$2>>2]|0;
  $143 = $142;
  $144 = $143;
  HEAP32[$144>>2] = $141;
  $145 = (($143) + 4)|0;
  $146 = $145;
  HEAP32[$146>>2] = $140;
  HEAP8[$0>>0] = 6;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $147 = HEAP32[$1>>2]|0;
 $148 = ((($147)) + 1|0);
 HEAP32[$1>>2] = $148;
 $149 = HEAP32[$a>>2]|0;
 $150 = $149 << 14;
 HEAP32[$a>>2] = $150;
 $151 = HEAP32[$1>>2]|0;
 $152 = HEAP8[$151>>0]|0;
 $153 = $152&255;
 $154 = HEAP32[$a>>2]|0;
 $155 = $154 | $153;
 HEAP32[$a>>2] = $155;
 $156 = HEAP32[$a>>2]|0;
 $157 = $156 & 128;
 $158 = ($157|0)!=(0);
 $159 = HEAP32[$a>>2]|0;
 if (!($158)) {
  $160 = $159 & -266354561;
  HEAP32[$a>>2] = $160;
  $161 = HEAP32[$b>>2]|0;
  $162 = $161 & 2080895;
  HEAP32[$b>>2] = $162;
  $163 = HEAP32[$b>>2]|0;
  $164 = $163 << 7;
  HEAP32[$b>>2] = $164;
  $165 = HEAP32[$b>>2]|0;
  $166 = HEAP32[$a>>2]|0;
  $167 = $166 | $165;
  HEAP32[$a>>2] = $167;
  $168 = HEAP32[$s>>2]|0;
  $169 = $168 >>> 11;
  HEAP32[$s>>2] = $169;
  $170 = HEAP32[$s>>2]|0;
  $171 = HEAP32[$a>>2]|0;
  $172 = HEAP32[$2>>2]|0;
  $173 = $172;
  $174 = $173;
  HEAP32[$174>>2] = $171;
  $175 = (($173) + 4)|0;
  $176 = $175;
  HEAP32[$176>>2] = $170;
  HEAP8[$0>>0] = 7;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $177 = $159 & 2080895;
 HEAP32[$a>>2] = $177;
 $178 = HEAP32[$1>>2]|0;
 $179 = ((($178)) + 1|0);
 HEAP32[$1>>2] = $179;
 $180 = HEAP32[$b>>2]|0;
 $181 = $180 << 14;
 HEAP32[$b>>2] = $181;
 $182 = HEAP32[$1>>2]|0;
 $183 = HEAP8[$182>>0]|0;
 $184 = $183&255;
 $185 = HEAP32[$b>>2]|0;
 $186 = $185 | $184;
 HEAP32[$b>>2] = $186;
 $187 = HEAP32[$b>>2]|0;
 $188 = $187 & 128;
 $189 = ($188|0)!=(0);
 if ($189) {
  $206 = HEAP32[$1>>2]|0;
  $207 = ((($206)) + 1|0);
  HEAP32[$1>>2] = $207;
  $208 = HEAP32[$a>>2]|0;
  $209 = $208 << 15;
  HEAP32[$a>>2] = $209;
  $210 = HEAP32[$1>>2]|0;
  $211 = HEAP8[$210>>0]|0;
  $212 = $211&255;
  $213 = HEAP32[$a>>2]|0;
  $214 = $213 | $212;
  HEAP32[$a>>2] = $214;
  $215 = HEAP32[$b>>2]|0;
  $216 = $215 & 2080895;
  HEAP32[$b>>2] = $216;
  $217 = HEAP32[$b>>2]|0;
  $218 = $217 << 8;
  HEAP32[$b>>2] = $218;
  $219 = HEAP32[$b>>2]|0;
  $220 = HEAP32[$a>>2]|0;
  $221 = $220 | $219;
  HEAP32[$a>>2] = $221;
  $222 = HEAP32[$s>>2]|0;
  $223 = $222 << 4;
  HEAP32[$s>>2] = $223;
  $224 = HEAP32[$1>>2]|0;
  $225 = ((($224)) + -4|0);
  $226 = HEAP8[$225>>0]|0;
  $227 = $226&255;
  HEAP32[$b>>2] = $227;
  $228 = HEAP32[$b>>2]|0;
  $229 = $228 & 127;
  HEAP32[$b>>2] = $229;
  $230 = HEAP32[$b>>2]|0;
  $231 = $230 >>> 3;
  HEAP32[$b>>2] = $231;
  $232 = HEAP32[$b>>2]|0;
  $233 = HEAP32[$s>>2]|0;
  $234 = $233 | $232;
  HEAP32[$s>>2] = $234;
  $235 = HEAP32[$s>>2]|0;
  $236 = HEAP32[$a>>2]|0;
  $237 = HEAP32[$2>>2]|0;
  $238 = $237;
  $239 = $238;
  HEAP32[$239>>2] = $236;
  $240 = (($238) + 4)|0;
  $241 = $240;
  HEAP32[$241>>2] = $235;
  HEAP8[$0>>0] = 9;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 } else {
  $190 = HEAP32[$b>>2]|0;
  $191 = $190 & -266354561;
  HEAP32[$b>>2] = $191;
  $192 = HEAP32[$a>>2]|0;
  $193 = $192 << 7;
  HEAP32[$a>>2] = $193;
  $194 = HEAP32[$b>>2]|0;
  $195 = HEAP32[$a>>2]|0;
  $196 = $195 | $194;
  HEAP32[$a>>2] = $196;
  $197 = HEAP32[$s>>2]|0;
  $198 = $197 >>> 4;
  HEAP32[$s>>2] = $198;
  $199 = HEAP32[$s>>2]|0;
  $200 = HEAP32[$a>>2]|0;
  $201 = HEAP32[$2>>2]|0;
  $202 = $201;
  $203 = $202;
  HEAP32[$203>>2] = $200;
  $204 = (($202) + 4)|0;
  $205 = $204;
  HEAP32[$205>>2] = $199;
  HEAP8[$0>>0] = 8;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 return (0)|0;
}
function _pagerFixMaplimit($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 STACKTOP = sp;return;
}
function _sqlite3InvokeBusyHandler($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 8|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)<(0);
   if (!($10)) {
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$1>>2]|0;
    $14 = ((($13)) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (FUNCTION_TABLE_iii[$12 & 255]($15,$18)|0);
    HEAP32[$rc>>2] = $19;
    $20 = HEAP32[$rc>>2]|0;
    $21 = ($20|0)==(0);
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 8|0);
    if ($21) {
     HEAP32[$23>>2] = -1;
    } else {
     $24 = HEAP32[$23>>2]|0;
     $25 = (($24) + 1)|0;
     HEAP32[$23>>2] = $25;
    }
    $26 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $26;
    $27 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($27|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3PcacheRefCount($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PageMalloc($sz) {
 $sz = $sz|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $sz;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pcache1Alloc($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _pager_reset($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 108|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 96|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3BackupRestart($7);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 212|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3PcacheClear($10);
 STACKTOP = sp;return;
}
function _pcache1Alloc($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $sz = sp;
 HEAP32[$0>>2] = $nByte;
 HEAP32[$p>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[(1648)>>2]|0;
 $3 = ($1|0)<=($2|0);
 if ($3) {
  $4 = HEAP32[(1672)>>2]|0;
  HEAP32[$p>>2] = $4;
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if ($6) {
   $7 = HEAP32[(1672)>>2]|0;
   $8 = HEAP32[$7>>2]|0;
   HEAP32[(1672)>>2] = $8;
   $9 = HEAP32[(1676)>>2]|0;
   $10 = (($9) + -1)|0;
   HEAP32[(1676)>>2] = $10;
   $11 = HEAP32[(1676)>>2]|0;
   $12 = HEAP32[(1656)>>2]|0;
   $13 = ($11|0)<($12|0);
   $14 = $13&1;
   HEAP32[(1680)>>2] = $14;
   $15 = HEAP32[$0>>2]|0;
   _sqlite3StatusHighwater(7,$15);
   _sqlite3StatusUp(1,1);
  }
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = ($16|0)==(0|0);
 if (!($17)) {
  $28 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = (_sqlite3Malloc($18,$20)|0);
 HEAP32[$p>>2] = $21;
 $22 = HEAP32[$p>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $28 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = (_sqlite3MallocSize($24)|0);
 HEAP32[$sz>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 _sqlite3StatusHighwater(7,$26);
 $27 = HEAP32[$sz>>2]|0;
 _sqlite3StatusUp(2,$27);
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3BackupRestart($pBackup) {
 $pBackup = $pBackup|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBackup;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 while(1) {
  $2 = HEAP32[$p>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 16|0);
  HEAP32[$5>>2] = 1;
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$p>>2] = $8;
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheClear($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3PcacheTruncate($1,0);
 STACKTOP = sp;return;
}
function _sqlite3PcacheTruncate($pCache,$pgno) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, $pPage1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $p = sp + 8|0;
 $pNext = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pNext>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15>>>0)>($16>>>0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   _sqlite3PcacheMakeClean($18);
  }
  $19 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $19;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   $26 = HEAP32[(140)>>2]|0;
   $27 = HEAP32[$0>>2]|0;
   $28 = ((($27)) + 44|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (FUNCTION_TABLE_iiii[$26 & 255]($29,1,0)|0);
   HEAP32[$pPage1>>2] = $30;
   $31 = HEAP32[$pPage1>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    $33 = HEAP32[$pPage1>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 24|0);
    $37 = HEAP32[$36>>2]|0;
    _memset(($34|0),0,($37|0))|0;
    HEAP32[$1>>2] = 1;
   }
  }
 }
 $38 = HEAP32[(152)>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 44|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = (($42) + 1)|0;
 FUNCTION_TABLE_vii[$38 & 255]($41,$43);
 STACKTOP = sp;return;
}
function _pagerFreeMapHdrs($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 144|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$pNext>>2] = $8;
  $9 = HEAP32[$p>>2]|0;
  _sqlite3_free($9);
  $10 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $10;
 }
 STACKTOP = sp;return;
}
function _sqlite3WalClose($pWal,$sync_flags,$nBuf,$zBuf) {
 $pWal = $pWal|0;
 $sync_flags = $sync_flags|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bPersist = 0, $isDelete = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $isDelete = sp + 4|0;
 $bPersist = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $sync_flags;
 HEAP32[$2>>2] = $nBuf;
 HEAP32[$3>>2] = $zBuf;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 HEAP32[$isDelete>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3OsLock($8,4)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 43|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)==(0);
   if ($16) {
    $17 = HEAP32[$0>>2]|0;
    $18 = ((($17)) + 43|0);
    HEAP8[$18>>0] = 1;
   }
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (_sqlite3WalCheckpoint($19,0,0,0,$20,$21,$22,0,0)|0);
   HEAP32[$rc>>2] = $23;
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   if ($25) {
    HEAP32[$bPersist>>2] = -1;
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    _sqlite3OsFileControlHint($28,10,$bPersist);
    $29 = HEAP32[$bPersist>>2]|0;
    $30 = ($29|0)!=(1);
    if ($30) {
     HEAP32[$isDelete>>2] = 1;
     break;
    }
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 16|0);
    $33 = $32;
    $34 = $33;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($33) + 4)|0;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = ($38|0)>(0);
    $40 = ($35>>>0)>=(0);
    $41 = ($38|0)==(0);
    $42 = $41 & $40;
    $43 = $39 | $42;
    if ($43) {
     $44 = HEAP32[$0>>2]|0;
     _walLimitSize($44,0,0);
    }
   }
  }
 } while(0);
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$isDelete>>2]|0;
 _walIndexClose($45,$46);
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 (_sqlite3OsClose($49)|0);
 $50 = HEAP32[$isDelete>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  _sqlite3BeginBenignMalloc();
  $52 = HEAP32[$0>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$0>>2]|0;
  $55 = ((($54)) + 108|0);
  $56 = HEAP32[$55>>2]|0;
  (_sqlite3OsDelete($53,$56,0)|0);
  _sqlite3EndBenignMalloc();
 }
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 32|0);
 $59 = HEAP32[$58>>2]|0;
 _sqlite3_free($59);
 $60 = HEAP32[$0>>2]|0;
 _sqlite3_free($60);
 $61 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _pager_unlock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $iDc = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $iDc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BitvecDestroy($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 60|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 _releaseAllSavepoints($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (_pagerUseWal($7)|0);
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = ((($10)) + 216|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3WalEndReadTransaction($12);
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 17|0);
  HEAP8[$14>>0] = 0;
 } else {
  $15 = ((($10)) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 64|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$0>>2]|0;
    $24 = ((($23)) + 64|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (_sqlite3OsDeviceCharacteristics($25)|0);
    $27 = $26;
   } else {
    $27 = 0;
   }
   HEAP32[$iDc>>2] = $27;
   $28 = HEAP32[$iDc>>2]|0;
   $29 = $28 & 2048;
   $30 = (0)==($29|0);
   if ($30) {
    label = 8;
   } else {
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 5|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = $33&255;
    $35 = $34 & 5;
    $36 = (1)!=($35|0);
    if ($36) {
     label = 8;
    }
   }
   if ((label|0) == 8) {
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 68|0);
    $39 = HEAP32[$38>>2]|0;
    (_sqlite3OsClose($39)|0);
   }
   $40 = HEAP32[$0>>2]|0;
   $41 = (_pagerUnlockDb($40,0)|0);
   HEAP32[$rc>>2] = $41;
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = HEAP32[$0>>2]|0;
    $45 = ((($44)) + 17|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = ($47|0)==(6);
    if ($48) {
     $49 = HEAP32[$0>>2]|0;
     $50 = ((($49)) + 18|0);
     HEAP8[$50>>0] = 5;
    }
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = ((($51)) + 19|0);
   HEAP8[$52>>0] = 0;
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 17|0);
   HEAP8[$54>>0] = 0;
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 44|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)!=(0);
 if (!($58)) {
  $69 = HEAP32[$0>>2]|0;
  $70 = ((($69)) + 80|0);
  $71 = $70;
  $72 = $71;
  HEAP32[$72>>2] = 0;
  $73 = (($71) + 4)|0;
  $74 = $73;
  HEAP32[$74>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  $76 = ((($75)) + 88|0);
  $77 = $76;
  $78 = $77;
  HEAP32[$78>>2] = 0;
  $79 = (($77) + 4)|0;
  $80 = $79;
  HEAP32[$80>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  $82 = ((($81)) + 20|0);
  HEAP8[$82>>0] = 0;
  STACKTOP = sp;return;
 }
 $59 = HEAP32[$0>>2]|0;
 _pager_reset($59);
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 13|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 19|0);
 HEAP8[$64>>0] = $62;
 $65 = HEAP32[$0>>2]|0;
 $66 = ((($65)) + 17|0);
 HEAP8[$66>>0] = 0;
 $67 = HEAP32[$0>>2]|0;
 $68 = ((($67)) + 44|0);
 HEAP32[$68>>2] = 0;
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 80|0);
 $71 = $70;
 $72 = $71;
 HEAP32[$72>>2] = 0;
 $73 = (($71) + 4)|0;
 $74 = $73;
 HEAP32[$74>>2] = 0;
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 88|0);
 $77 = $76;
 $78 = $77;
 HEAP32[$78>>2] = 0;
 $79 = (($77) + 4)|0;
 $80 = $79;
 HEAP32[$80>>2] = 0;
 $81 = HEAP32[$0>>2]|0;
 $82 = ((($81)) + 20|0);
 HEAP8[$82>>0] = 0;
 STACKTOP = sp;return;
}
function _pagerSyncHotJournal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 7|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 68|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3OsSync($7,2)|0);
  HEAP32[$rc>>2] = $8;
 }
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 if (!($10)) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 88|0);
 $16 = (_sqlite3OsFileSize($13,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _pagerUnlockAndRollback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 17|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(6);
 do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 17|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = ($9|0)!=(0);
   if ($10) {
    $11 = HEAP32[$0>>2]|0;
    $12 = ((($11)) + 17|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13&255;
    $15 = ($14|0)>=(2);
    if ($15) {
     _sqlite3BeginBenignMalloc();
     $16 = HEAP32[$0>>2]|0;
     (_sqlite3PagerRollback($16)|0);
     _sqlite3EndBenignMalloc();
     break;
    }
    $17 = HEAP32[$0>>2]|0;
    $18 = ((($17)) + 4|0);
    $19 = HEAP8[$18>>0]|0;
    $20 = ($19<<24>>24)!=(0);
    if (!($20)) {
     $21 = HEAP32[$0>>2]|0;
     (_pager_end_transaction($21,0,0)|0);
    }
   }
  }
 } while(0);
 $22 = HEAP32[$0>>2]|0;
 _pager_unlock($22);
 STACKTOP = sp;return;
}
function _sqlite3PcacheClose($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[(156)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$1 & 255]($4);
 STACKTOP = sp;return;
}
function _sqlite3WalCheckpoint($pWal,$eMode,$xBusy,$pBusyArg,$sync_flags,$nBuf,$zBuf,$pnLog,$pnCkpt) {
 $pWal = $pWal|0;
 $eMode = $eMode|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $sync_flags = $sync_flags|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $eMode2 = 0, $isChanged = 0, $or$cond = 0, $rc = 0, $xBusy2 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $9 = sp + 16|0;
 $rc = sp + 12|0;
 $isChanged = sp + 8|0;
 $eMode2 = sp + 4|0;
 $xBusy2 = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $xBusy;
 HEAP32[$4>>2] = $pBusyArg;
 HEAP32[$5>>2] = $sync_flags;
 HEAP32[$6>>2] = $nBuf;
 HEAP32[$7>>2] = $zBuf;
 HEAP32[$8>>2] = $pnLog;
 HEAP32[$9>>2] = $pnCkpt;
 HEAP32[$isChanged>>2] = 0;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$eMode2>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$xBusy2>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 46|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = ($14<<24>>24)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 8;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (_walLockExclusive($16,1,1)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 45|0);
 HEAP8[$22>>0] = 1;
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0);
 do {
  if ($24) {
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$4>>2]|0;
   $28 = (_walBusyLock($25,$26,$27,0,1)|0);
   HEAP32[$rc>>2] = $28;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(0);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 44|0);
    HEAP8[$32>>0] = 1;
    break;
   }
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(5);
   if ($34) {
    HEAP32[$eMode2>>2] = 0;
    HEAP32[$xBusy2>>2] = 0;
    HEAP32[$rc>>2] = 0;
   }
  }
 } while(0);
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = (_walIndexReadHdr($37,$isChanged)|0);
  HEAP32[$rc>>2] = $38;
  $39 = HEAP32[$isChanged>>2]|0;
  $40 = ($39|0)!=(0);
  if ($40) {
   $41 = HEAP32[$1>>2]|0;
   $42 = ((($41)) + 4|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)>=(3);
   if ($46) {
    $47 = HEAP32[$1>>2]|0;
    $48 = ((($47)) + 4|0);
    $49 = HEAP32[$48>>2]|0;
    (_sqlite3OsUnfetch($49,0,0,0)|0);
   }
  }
 }
 $50 = HEAP32[$rc>>2]|0;
 $51 = ($50|0)==(0);
 if ($51) {
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 52|0);
  $54 = ((($53)) + 16|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0);
  if ($56) {
   $57 = HEAP32[$1>>2]|0;
   $58 = (_walPagesize($57)|0);
   $59 = HEAP32[$6>>2]|0;
   $60 = ($58|0)!=($59|0);
   if ($60) {
    $61 = (_sqlite3CorruptError(54390)|0);
    HEAP32[$rc>>2] = $61;
   } else {
    label = 18;
   }
  } else {
   label = 18;
  }
  if ((label|0) == 18) {
   $62 = HEAP32[$1>>2]|0;
   $63 = HEAP32[$eMode2>>2]|0;
   $64 = HEAP32[$xBusy2>>2]|0;
   $65 = HEAP32[$4>>2]|0;
   $66 = HEAP32[$5>>2]|0;
   $67 = HEAP32[$7>>2]|0;
   $68 = (_walCheckpoint($62,$63,$64,$65,$66,$67)|0);
   HEAP32[$rc>>2] = $68;
  }
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(0);
  $71 = HEAP32[$rc>>2]|0;
  $72 = ($71|0)==(5);
  $or$cond = $70 | $72;
  if ($or$cond) {
   $73 = HEAP32[$8>>2]|0;
   $74 = ($73|0)!=(0|0);
   if ($74) {
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 52|0);
    $77 = ((($76)) + 16|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = HEAP32[$8>>2]|0;
    HEAP32[$79>>2] = $78;
   }
   $80 = HEAP32[$9>>2]|0;
   $81 = ($80|0)!=(0|0);
   if ($81) {
    $82 = HEAP32[$1>>2]|0;
    $83 = (_walCkptInfo($82)|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = HEAP32[$9>>2]|0;
    HEAP32[$85>>2] = $84;
   }
  }
 }
 $86 = HEAP32[$isChanged>>2]|0;
 $87 = ($86|0)!=(0);
 if ($87) {
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 52|0);
  dest=$89; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 }
 $90 = HEAP32[$1>>2]|0;
 (_sqlite3WalEndWriteTransaction($90)|0);
 $91 = HEAP32[$1>>2]|0;
 _walUnlockExclusive($91,1,1);
 $92 = HEAP32[$1>>2]|0;
 $93 = ((($92)) + 45|0);
 HEAP8[$93>>0] = 0;
 $94 = HEAP32[$rc>>2]|0;
 $95 = ($94|0)==(0);
 if ($95) {
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$eMode2>>2]|0;
  $98 = ($96|0)!=($97|0);
  if ($98) {
   $100 = 5;
  } else {
   label = 28;
  }
 } else {
  label = 28;
 }
 if ((label|0) == 28) {
  $99 = HEAP32[$rc>>2]|0;
  $100 = $99;
 }
 HEAP32[$0>>2] = $100;
 $101 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($101|0);
}
function _walIndexClose($pWal,$isDelete) {
 $pWal = $pWal|0;
 $isDelete = $isDelete|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $isDelete;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 43|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(2);
 if (!($6)) {
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  (_sqlite3OsShmUnmap($27,$28)|0);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_free($17);
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<2)|0);
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _sqlite3OsDelete($pVfs,$zPath,$dirSync) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $dirSync = $dirSync|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $dirSync;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$5 & 255]($6,$7,$8)|0);
 STACKTOP = sp;return ($9|0);
}
function _walBusyLock($pWal,$xBusy,$pBusyArg,$lockIdx,$n) {
 $pWal = $pWal|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $xBusy;
 HEAP32[$2>>2] = $pBusyArg;
 HEAP32[$3>>2] = $lockIdx;
 HEAP32[$4>>2] = $n;
 while(1) {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$4>>2]|0;
  $8 = (_walLockExclusive($5,$6,$7)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)==(5);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   label = 4;
   break;
  }
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (FUNCTION_TABLE_ii[$13 & 255]($14)|0);
  $16 = ($15|0)!=(0);
  if (!($16)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3OsUnfetch($id,$0,$1,$p) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $p = $p|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $p;
 STACKTOP = sp;return 0;
}
function _walPagesize($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = ((($2)) + 14|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 65024;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 14|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 1;
 $13 = $12 << 16;
 $14 = (($6) + ($13))|0;
 STACKTOP = sp;return ($14|0);
}
function _walCheckpoint($pWal,$eMode,$xBusy,$pBusyArg,$sync_flags,$zBuf) {
 $pWal = $pWal|0;
 $eMode = $eMode|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $sync_flags = $sync_flags|0;
 $zBuf = $zBuf|0;
 var $$old = 0, $$old4 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDbpage = 0, $iFrame = 0, $iOffset = 0, $mxPage = 0, $mxSafeFrame = 0, $nBackfill = 0;
 var $nReq = 0, $nSize = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pInfo = 0, $pIter = 0, $rc = 0, $salt1 = 0, $szDb = 0, $szPage = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 80|0;
 $rc = sp + 76|0;
 $szPage = sp + 72|0;
 $pIter = sp + 68|0;
 $iDbpage = sp + 64|0;
 $iFrame = sp + 60|0;
 $mxSafeFrame = sp + 56|0;
 $mxPage = sp + 52|0;
 $i = sp + 48|0;
 $pInfo = sp + 44|0;
 $y = sp + 40|0;
 $nSize = sp + 24|0;
 $nBackfill = sp + 36|0;
 $nReq = sp + 16|0;
 $iOffset = sp + 8|0;
 $szDb = sp;
 $salt1 = sp + 32|0;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $xBusy;
 HEAP32[$4>>2] = $pBusyArg;
 HEAP32[$5>>2] = $sync_flags;
 HEAP32[$6>>2] = $zBuf;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pIter>>2] = 0;
 HEAP32[$iDbpage>>2] = 0;
 HEAP32[$iFrame>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_walPagesize($7)|0);
 HEAP32[$szPage>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_walCkptInfo($9)|0);
 HEAP32[$pInfo>>2] = $10;
 $11 = HEAP32[$pInfo>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($12>>>0)<($16>>>0);
 L1: do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (_walIteratorInit($18,$pIter)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $22;
    $270 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($270|0);
   }
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 52|0);
   $25 = ((($24)) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$mxSafeFrame>>2] = $26;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 52|0);
   $29 = ((($28)) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$mxPage>>2] = $30;
   HEAP32[$i>>2] = 1;
   while(1) {
    $31 = HEAP32[$i>>2]|0;
    $32 = ($31|0)<(5);
    if (!($32)) {
     break;
    }
    $33 = HEAP32[$i>>2]|0;
    $34 = HEAP32[$pInfo>>2]|0;
    $35 = ((($34)) + 4|0);
    $36 = (($35) + ($33<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    HEAP32[$y>>2] = $37;
    $38 = HEAP32[$mxSafeFrame>>2]|0;
    $39 = HEAP32[$y>>2]|0;
    $40 = ($38>>>0)>($39>>>0);
    do {
     if ($40) {
      $41 = HEAP32[$1>>2]|0;
      $42 = HEAP32[$3>>2]|0;
      $43 = HEAP32[$4>>2]|0;
      $44 = HEAP32[$i>>2]|0;
      $45 = (3 + ($44))|0;
      $46 = (_walBusyLock($41,$42,$43,$45,1)|0);
      HEAP32[$rc>>2] = $46;
      $47 = HEAP32[$rc>>2]|0;
      $48 = ($47|0)==(0);
      if ($48) {
       $49 = HEAP32[$i>>2]|0;
       $50 = ($49|0)==(1);
       $51 = HEAP32[$mxSafeFrame>>2]|0;
       $52 = $50 ? $51 : -1;
       $53 = HEAP32[$i>>2]|0;
       $54 = HEAP32[$pInfo>>2]|0;
       $55 = ((($54)) + 4|0);
       $56 = (($55) + ($53<<2)|0);
       HEAP32[$56>>2] = $52;
       $57 = HEAP32[$1>>2]|0;
       $58 = HEAP32[$i>>2]|0;
       $59 = (3 + ($58))|0;
       _walUnlockExclusive($57,$59,1);
       break;
      }
      $60 = HEAP32[$rc>>2]|0;
      $61 = ($60|0)==(5);
      if (!($61)) {
       break L1;
      }
      $62 = HEAP32[$y>>2]|0;
      HEAP32[$mxSafeFrame>>2] = $62;
      HEAP32[$3>>2] = 0;
     }
    } while(0);
    $63 = HEAP32[$i>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[$i>>2] = $64;
   }
   $65 = HEAP32[$pInfo>>2]|0;
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$mxSafeFrame>>2]|0;
   $68 = ($66>>>0)<($67>>>0);
   if ($68) {
    $69 = HEAP32[$1>>2]|0;
    $70 = HEAP32[$3>>2]|0;
    $71 = HEAP32[$4>>2]|0;
    $72 = (_walBusyLock($69,$70,$71,3,1)|0);
    HEAP32[$rc>>2] = $72;
    $73 = ($72|0)==(0);
    if ($73) {
     $74 = HEAP32[$pInfo>>2]|0;
     $75 = HEAP32[$74>>2]|0;
     HEAP32[$nBackfill>>2] = $75;
     $76 = HEAP32[$mxSafeFrame>>2]|0;
     $77 = HEAP32[$pInfo>>2]|0;
     $78 = ((($77)) + 32|0);
     HEAP32[$78>>2] = $76;
     $79 = HEAP32[$5>>2]|0;
     $80 = ($79|0)!=(0);
     if ($80) {
      $81 = HEAP32[$1>>2]|0;
      $82 = ((($81)) + 8|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = HEAP32[$5>>2]|0;
      $85 = (_sqlite3OsSync($83,$84)|0);
      HEAP32[$rc>>2] = $85;
     }
     $86 = HEAP32[$rc>>2]|0;
     $87 = ($86|0)==(0);
     if ($87) {
      $88 = HEAP32[$mxPage>>2]|0;
      $89 = HEAP32[$szPage>>2]|0;
      $90 = ($89|0)<(0);
      $91 = $90 << 31 >> 31;
      $92 = (___muldi3(($88|0),0,($89|0),($91|0))|0);
      $93 = tempRet0;
      $94 = $nReq;
      $95 = $94;
      HEAP32[$95>>2] = $92;
      $96 = (($94) + 4)|0;
      $97 = $96;
      HEAP32[$97>>2] = $93;
      $98 = HEAP32[$1>>2]|0;
      $99 = ((($98)) + 4|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = (_sqlite3OsFileSize($100,$nSize)|0);
      HEAP32[$rc>>2] = $101;
      $102 = HEAP32[$rc>>2]|0;
      $103 = ($102|0)==(0);
      if ($103) {
       $104 = $nSize;
       $105 = $104;
       $106 = HEAP32[$105>>2]|0;
       $107 = (($104) + 4)|0;
       $108 = $107;
       $109 = HEAP32[$108>>2]|0;
       $110 = $nReq;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = (($110) + 4)|0;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = ($109|0)<($115|0);
       $117 = ($106>>>0)<($112>>>0);
       $118 = ($109|0)==($115|0);
       $119 = $118 & $117;
       $120 = $116 | $119;
       if ($120) {
        $121 = HEAP32[$1>>2]|0;
        $122 = ((($121)) + 4|0);
        $123 = HEAP32[$122>>2]|0;
        _sqlite3OsFileControlHint($123,5,$nReq);
       }
      }
     }
     L28: while(1) {
      $$old = HEAP32[$rc>>2]|0;
      $$old4 = ($$old|0)==(0);
      if (!($$old4)) {
       break;
      }
      while(1) {
       $124 = HEAP32[$pIter>>2]|0;
       $125 = (_walIteratorNext($124,$iDbpage,$iFrame)|0);
       $126 = (0)==($125|0);
       if (!($126)) {
        break L28;
       }
       $127 = HEAP32[$iFrame>>2]|0;
       $128 = HEAP32[$nBackfill>>2]|0;
       $129 = ($127>>>0)<=($128>>>0);
       if ($129) {
        continue L28;
       }
       $130 = HEAP32[$iFrame>>2]|0;
       $131 = HEAP32[$mxSafeFrame>>2]|0;
       $132 = ($130>>>0)>($131>>>0);
       if ($132) {
        continue L28;
       }
       $133 = HEAP32[$iDbpage>>2]|0;
       $134 = HEAP32[$mxPage>>2]|0;
       $135 = ($133>>>0)>($134>>>0);
       if ($135) {
        continue L28;
       }
       $136 = HEAP32[$iFrame>>2]|0;
       $137 = (($136) - 1)|0;
       $138 = HEAP32[$szPage>>2]|0;
       $139 = (($138) + 24)|0;
       $140 = ($139|0)<(0);
       $141 = $140 << 31 >> 31;
       $142 = (___muldi3(($137|0),0,($139|0),($141|0))|0);
       $143 = tempRet0;
       $144 = (_i64Add(32,0,($142|0),($143|0))|0);
       $145 = tempRet0;
       $146 = (_i64Add(($144|0),($145|0),24,0)|0);
       $147 = tempRet0;
       $148 = $iOffset;
       $149 = $148;
       HEAP32[$149>>2] = $146;
       $150 = (($148) + 4)|0;
       $151 = $150;
       HEAP32[$151>>2] = $147;
       $152 = HEAP32[$1>>2]|0;
       $153 = ((($152)) + 8|0);
       $154 = HEAP32[$153>>2]|0;
       $155 = HEAP32[$6>>2]|0;
       $156 = HEAP32[$szPage>>2]|0;
       $157 = $iOffset;
       $158 = $157;
       $159 = HEAP32[$158>>2]|0;
       $160 = (($157) + 4)|0;
       $161 = $160;
       $162 = HEAP32[$161>>2]|0;
       $163 = (_sqlite3OsRead($154,$155,$156,$159,$162)|0);
       HEAP32[$rc>>2] = $163;
       $164 = HEAP32[$rc>>2]|0;
       $165 = ($164|0)!=(0);
       if ($165) {
        break L28;
       }
       $166 = HEAP32[$iDbpage>>2]|0;
       $167 = (($166) - 1)|0;
       $168 = HEAP32[$szPage>>2]|0;
       $169 = ($168|0)<(0);
       $170 = $169 << 31 >> 31;
       $171 = (___muldi3(($167|0),0,($168|0),($170|0))|0);
       $172 = tempRet0;
       $173 = $iOffset;
       $174 = $173;
       HEAP32[$174>>2] = $171;
       $175 = (($173) + 4)|0;
       $176 = $175;
       HEAP32[$176>>2] = $172;
       $177 = HEAP32[$1>>2]|0;
       $178 = ((($177)) + 4|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = HEAP32[$6>>2]|0;
       $181 = HEAP32[$szPage>>2]|0;
       $182 = $iOffset;
       $183 = $182;
       $184 = HEAP32[$183>>2]|0;
       $185 = (($182) + 4)|0;
       $186 = $185;
       $187 = HEAP32[$186>>2]|0;
       $188 = (_sqlite3OsWrite($179,$180,$181,$184,$187)|0);
       HEAP32[$rc>>2] = $188;
       $189 = HEAP32[$rc>>2]|0;
       $190 = ($189|0)==(0);
       $191 = HEAP32[$rc>>2]|0;
       $192 = ($191|0)==(0);
       $or$cond5 = $190 & $192;
       if (!($or$cond5)) {
        break L28;
       }
      }
     }
     $193 = HEAP32[$rc>>2]|0;
     $194 = ($193|0)==(0);
     if ($194) {
      $195 = HEAP32[$mxSafeFrame>>2]|0;
      $196 = HEAP32[$1>>2]|0;
      $197 = (_walIndexHdr($196)|0);
      $198 = ((($197)) + 16|0);
      $199 = HEAP32[$198>>2]|0;
      $200 = ($195|0)==($199|0);
      if ($200) {
       $201 = HEAP32[$1>>2]|0;
       $202 = ((($201)) + 52|0);
       $203 = ((($202)) + 20|0);
       $204 = HEAP32[$203>>2]|0;
       $205 = HEAP32[$szPage>>2]|0;
       $206 = ($205|0)<(0);
       $207 = $206 << 31 >> 31;
       $208 = (___muldi3(($204|0),0,($205|0),($207|0))|0);
       $209 = tempRet0;
       $210 = $szDb;
       $211 = $210;
       HEAP32[$211>>2] = $208;
       $212 = (($210) + 4)|0;
       $213 = $212;
       HEAP32[$213>>2] = $209;
       $214 = HEAP32[$1>>2]|0;
       $215 = ((($214)) + 4|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $szDb;
       $218 = $217;
       $219 = HEAP32[$218>>2]|0;
       $220 = (($217) + 4)|0;
       $221 = $220;
       $222 = HEAP32[$221>>2]|0;
       $223 = (_sqlite3OsTruncate($216,$219,$222)|0);
       HEAP32[$rc>>2] = $223;
       $224 = HEAP32[$rc>>2]|0;
       $225 = ($224|0)==(0);
       $226 = HEAP32[$5>>2]|0;
       $227 = ($226|0)!=(0);
       $or$cond = $225 & $227;
       if ($or$cond) {
        $228 = HEAP32[$1>>2]|0;
        $229 = ((($228)) + 4|0);
        $230 = HEAP32[$229>>2]|0;
        $231 = HEAP32[$5>>2]|0;
        $232 = (_sqlite3OsSync($230,$231)|0);
        HEAP32[$rc>>2] = $232;
       }
      }
      $233 = HEAP32[$rc>>2]|0;
      $234 = ($233|0)==(0);
      if ($234) {
       $235 = HEAP32[$mxSafeFrame>>2]|0;
       $236 = HEAP32[$pInfo>>2]|0;
       HEAP32[$236>>2] = $235;
      }
     }
     $237 = HEAP32[$1>>2]|0;
     _walUnlockExclusive($237,3,1);
    }
   }
   $238 = HEAP32[$rc>>2]|0;
   $239 = ($238|0)==(5);
   if ($239) {
    HEAP32[$rc>>2] = 0;
    label = 36;
   } else {
    label = 36;
   }
  } else {
   label = 36;
  }
 } while(0);
 do {
  if ((label|0) == 36) {
   $240 = HEAP32[$rc>>2]|0;
   $241 = ($240|0)==(0);
   $242 = HEAP32[$2>>2]|0;
   $243 = ($242|0)!=(0);
   $or$cond3 = $241 & $243;
   if ($or$cond3) {
    $244 = HEAP32[$pInfo>>2]|0;
    $245 = HEAP32[$244>>2]|0;
    $246 = HEAP32[$1>>2]|0;
    $247 = ((($246)) + 52|0);
    $248 = ((($247)) + 16|0);
    $249 = HEAP32[$248>>2]|0;
    $250 = ($245>>>0)<($249>>>0);
    if ($250) {
     HEAP32[$rc>>2] = 5;
     break;
    }
    $251 = HEAP32[$2>>2]|0;
    $252 = ($251|0)>=(2);
    if ($252) {
     _sqlite3_randomness(4,$salt1);
     $253 = HEAP32[$1>>2]|0;
     $254 = HEAP32[$3>>2]|0;
     $255 = HEAP32[$4>>2]|0;
     $256 = (_walBusyLock($253,$254,$255,4,4)|0);
     HEAP32[$rc>>2] = $256;
     $257 = HEAP32[$rc>>2]|0;
     $258 = ($257|0)==(0);
     if ($258) {
      $259 = HEAP32[$2>>2]|0;
      $260 = ($259|0)==(3);
      if ($260) {
       $261 = HEAP32[$1>>2]|0;
       $262 = HEAP32[$salt1>>2]|0;
       _walRestartHdr($261,$262);
       $263 = HEAP32[$1>>2]|0;
       $264 = ((($263)) + 8|0);
       $265 = HEAP32[$264>>2]|0;
       $266 = (_sqlite3OsTruncate($265,0,0)|0);
       HEAP32[$rc>>2] = $266;
      }
      $267 = HEAP32[$1>>2]|0;
      _walUnlockExclusive($267,4,4);
     }
    }
   }
  }
 } while(0);
 $268 = HEAP32[$pIter>>2]|0;
 _walIteratorFree($268);
 $269 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $269;
 $270 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($270|0);
}
function _sqlite3WalEndWriteTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  STACKTOP = sp;return 0;
 }
 $5 = HEAP32[$0>>2]|0;
 _walUnlockExclusive($5,0,1);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 44|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 104|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 47|0);
 HEAP8[$11>>0] = 0;
 STACKTOP = sp;return 0;
}
function _walIteratorInit($pWal,$pp) {
 $pWal = $pWal|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aHash = 0, $aIndex = 0;
 var $aPgno = 0, $aTmp = 0, $i = 0, $iLast = 0, $iZero = 0, $j = 0, $nByte = 0, $nEntry = 0, $nSegment = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $p = sp + 48|0;
 $nSegment = sp + 44|0;
 $iLast = sp + 40|0;
 $nByte = sp + 36|0;
 $i = sp + 32|0;
 $aTmp = sp + 28|0;
 $rc = sp + 24|0;
 $aHash = sp + 20|0;
 $iZero = sp + 16|0;
 $aPgno = sp + 12|0;
 $j = sp + 8|0;
 $nEntry = sp + 4|0;
 $aIndex = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iLast>>2] = $6;
 $7 = HEAP32[$iLast>>2]|0;
 $8 = (_walFramePage($7)|0);
 $9 = (($8) + 1)|0;
 HEAP32[$nSegment>>2] = $9;
 $10 = HEAP32[$nSegment>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = ($11*20)|0;
 $13 = (28 + ($12))|0;
 $14 = HEAP32[$iLast>>2]|0;
 $15 = $14<<1;
 $16 = (($13) + ($15))|0;
 HEAP32[$nByte>>2] = $16;
 $17 = HEAP32[$nByte>>2]|0;
 $18 = ($17|0)<(0);
 $19 = $18 << 31 >> 31;
 $20 = (_sqlite3_malloc64($17,$19)|0);
 HEAP32[$p>>2] = $20;
 $21 = HEAP32[$p>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  HEAP32[$0>>2] = 7;
  $117 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($117|0);
 }
 $23 = HEAP32[$p>>2]|0;
 $24 = HEAP32[$nByte>>2]|0;
 _memset(($23|0),0,($24|0))|0;
 $25 = HEAP32[$nSegment>>2]|0;
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 4|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$iLast>>2]|0;
 $29 = ($28>>>0)>(4096);
 $30 = HEAP32[$iLast>>2]|0;
 $31 = $29 ? 4096 : $30;
 $32 = $31<<1;
 $33 = (_sqlite3_malloc64($32,0)|0);
 HEAP32[$aTmp>>2] = $33;
 $34 = HEAP32[$aTmp>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  HEAP32[$rc>>2] = 7;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$rc>>2]|0;
  $37 = ($36|0)==(0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$nSegment>>2]|0;
  $40 = ($38|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$i>>2]|0;
  $43 = (_walHashGet($41,$42,$aHash,$aPgno,$iZero)|0);
  HEAP32[$rc>>2] = $43;
  $44 = HEAP32[$rc>>2]|0;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$aPgno>>2]|0;
   $47 = ((($46)) + 4|0);
   HEAP32[$aPgno>>2] = $47;
   $48 = HEAP32[$i>>2]|0;
   $49 = (($48) + 1)|0;
   $50 = HEAP32[$nSegment>>2]|0;
   $51 = ($49|0)==($50|0);
   if ($51) {
    $52 = HEAP32[$iLast>>2]|0;
    $53 = HEAP32[$iZero>>2]|0;
    $54 = (($52) - ($53))|0;
    HEAP32[$nEntry>>2] = $54;
   } else {
    $55 = HEAP32[$aHash>>2]|0;
    $56 = HEAP32[$aPgno>>2]|0;
    $57 = $55;
    $58 = $56;
    $59 = (($57) - ($58))|0;
    $60 = (($59|0) / 4)&-1;
    HEAP32[$nEntry>>2] = $60;
   }
   $61 = HEAP32[$iZero>>2]|0;
   $62 = HEAP32[$p>>2]|0;
   $63 = ((($62)) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = HEAP32[$p>>2]|0;
   $66 = ((($65)) + 8|0);
   $67 = (($66) + (($64*20)|0)|0);
   $68 = (($67) + ($61<<1)|0);
   HEAP32[$aIndex>>2] = $68;
   $69 = HEAP32[$iZero>>2]|0;
   $70 = (($69) + 1)|0;
   HEAP32[$iZero>>2] = $70;
   HEAP32[$j>>2] = 0;
   while(1) {
    $71 = HEAP32[$j>>2]|0;
    $72 = HEAP32[$nEntry>>2]|0;
    $73 = ($71|0)<($72|0);
    if (!($73)) {
     break;
    }
    $74 = HEAP32[$j>>2]|0;
    $75 = $74&65535;
    $76 = HEAP32[$j>>2]|0;
    $77 = HEAP32[$aIndex>>2]|0;
    $78 = (($77) + ($76<<1)|0);
    HEAP16[$78>>1] = $75;
    $79 = HEAP32[$j>>2]|0;
    $80 = (($79) + 1)|0;
    HEAP32[$j>>2] = $80;
   }
   $81 = HEAP32[$aPgno>>2]|0;
   $82 = HEAP32[$aTmp>>2]|0;
   $83 = HEAP32[$aIndex>>2]|0;
   _walMergesort($81,$82,$83,$nEntry);
   $84 = HEAP32[$iZero>>2]|0;
   $85 = HEAP32[$i>>2]|0;
   $86 = HEAP32[$p>>2]|0;
   $87 = ((($86)) + 8|0);
   $88 = (($87) + (($85*20)|0)|0);
   $89 = ((($88)) + 16|0);
   HEAP32[$89>>2] = $84;
   $90 = HEAP32[$nEntry>>2]|0;
   $91 = HEAP32[$i>>2]|0;
   $92 = HEAP32[$p>>2]|0;
   $93 = ((($92)) + 8|0);
   $94 = (($93) + (($91*20)|0)|0);
   $95 = ((($94)) + 12|0);
   HEAP32[$95>>2] = $90;
   $96 = HEAP32[$aIndex>>2]|0;
   $97 = HEAP32[$i>>2]|0;
   $98 = HEAP32[$p>>2]|0;
   $99 = ((($98)) + 8|0);
   $100 = (($99) + (($97*20)|0)|0);
   $101 = ((($100)) + 4|0);
   HEAP32[$101>>2] = $96;
   $102 = HEAP32[$aPgno>>2]|0;
   $103 = HEAP32[$i>>2]|0;
   $104 = HEAP32[$p>>2]|0;
   $105 = ((($104)) + 8|0);
   $106 = (($105) + (($103*20)|0)|0);
   $107 = ((($106)) + 8|0);
   HEAP32[$107>>2] = $102;
  }
  $108 = HEAP32[$i>>2]|0;
  $109 = (($108) + 1)|0;
  HEAP32[$i>>2] = $109;
 }
 $110 = HEAP32[$aTmp>>2]|0;
 _sqlite3_free($110);
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)!=(0);
 if ($112) {
  $113 = HEAP32[$p>>2]|0;
  _walIteratorFree($113);
 }
 $114 = HEAP32[$p>>2]|0;
 $115 = HEAP32[$2>>2]|0;
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $116;
 $117 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($117|0);
}
function _walIteratorNext($p,$piPage,$piFrame) {
 $p = $p|0;
 $piPage = $piPage|0;
 $piFrame = $piFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iMin = 0, $iPg = 0, $iRet = 0, $pSegment = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $iMin = sp + 16|0;
 $iRet = sp + 12|0;
 $i = sp + 8|0;
 $pSegment = sp + 4|0;
 $iPg = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piPage;
 HEAP32[$2>>2] = $piFrame;
 HEAP32[$iRet>>2] = -1;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$iMin>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 HEAP32[$i>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = (($13) + (($11*20)|0)|0);
  HEAP32[$pSegment>>2] = $14;
  while(1) {
   $15 = HEAP32[$pSegment>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pSegment>>2]|0;
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$pSegment>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$pSegment>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + ($22<<1)|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27&65535;
   $29 = HEAP32[$pSegment>>2]|0;
   $30 = ((($29)) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + ($28<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$iPg>>2] = $33;
   $34 = HEAP32[$iPg>>2]|0;
   $35 = HEAP32[$iMin>>2]|0;
   $36 = ($34>>>0)>($35>>>0);
   if ($36) {
    label = 6;
    break;
   }
   $54 = HEAP32[$pSegment>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$54>>2] = $56;
  }
  if ((label|0) == 6) {
   label = 0;
   $37 = HEAP32[$iPg>>2]|0;
   $38 = HEAP32[$iRet>>2]|0;
   $39 = ($37>>>0)<($38>>>0);
   if ($39) {
    $40 = HEAP32[$iPg>>2]|0;
    HEAP32[$iRet>>2] = $40;
    $41 = HEAP32[$pSegment>>2]|0;
    $42 = ((($41)) + 16|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$pSegment>>2]|0;
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$pSegment>>2]|0;
    $47 = ((($46)) + 4|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + ($45<<1)|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = (($43) + ($51))|0;
    $53 = HEAP32[$2>>2]|0;
    HEAP32[$53>>2] = $52;
   }
  }
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + -1)|0;
  HEAP32[$i>>2] = $58;
 }
 $59 = HEAP32[$iRet>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 HEAP32[$60>>2] = $59;
 $61 = HEAP32[$1>>2]|0;
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$iRet>>2]|0;
 $63 = ($62|0)==(-1);
 $64 = $63&1;
 STACKTOP = sp;return ($64|0);
}
function _walIteratorFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return;
}
function _walMergesort($aContent,$aBuffer,$aList,$pnList) {
 $aContent = $aContent|0;
 $aBuffer = $aBuffer|0;
 $aList = $aList|0;
 $pnList = $pnList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aMerge = 0;
 var $aSub = 0, $iList = 0, $iSub = 0, $nList = 0, $nMerge = 0, $p = 0, $p1 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 144|0;
 $1 = sp + 140|0;
 $2 = sp + 136|0;
 $3 = sp + 132|0;
 $nList = sp + 128|0;
 $nMerge = sp + 124|0;
 $aMerge = sp + 120|0;
 $iList = sp + 116|0;
 $iSub = sp + 112|0;
 $aSub = sp + 8|0;
 $p = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $aContent;
 HEAP32[$1>>2] = $aBuffer;
 HEAP32[$2>>2] = $aList;
 HEAP32[$3>>2] = $pnList;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nList>>2] = $5;
 HEAP32[$nMerge>>2] = 0;
 HEAP32[$aMerge>>2] = 0;
 HEAP32[$iSub>>2] = 0;
 dest=$aSub; stop=dest+104|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$iList>>2] = 0;
 while(1) {
  $6 = HEAP32[$iList>>2]|0;
  $7 = HEAP32[$nList>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  HEAP32[$nMerge>>2] = 1;
  $9 = HEAP32[$iList>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (($10) + ($9<<1)|0);
  HEAP32[$aMerge>>2] = $11;
  HEAP32[$iSub>>2] = 0;
  while(1) {
   $12 = HEAP32[$iList>>2]|0;
   $13 = HEAP32[$iSub>>2]|0;
   $14 = 1 << $13;
   $15 = $12 & $14;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$iSub>>2]|0;
   $18 = (($aSub) + ($17<<3)|0);
   HEAP32[$p>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$p>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   _walMerge($19,$22,$24,$aMerge,$nMerge,$25);
   $26 = HEAP32[$iSub>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$iSub>>2] = $27;
  }
  $28 = HEAP32[$aMerge>>2]|0;
  $29 = HEAP32[$iSub>>2]|0;
  $30 = (($aSub) + ($29<<3)|0);
  $31 = ((($30)) + 4|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$nMerge>>2]|0;
  $33 = HEAP32[$iSub>>2]|0;
  $34 = (($aSub) + ($33<<3)|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$iList>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$iList>>2] = $36;
 }
 $37 = HEAP32[$iSub>>2]|0;
 $38 = (($37) + 1)|0;
 HEAP32[$iSub>>2] = $38;
 while(1) {
  $39 = HEAP32[$iSub>>2]|0;
  $40 = ($39>>>0)<(13);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$nList>>2]|0;
  $42 = HEAP32[$iSub>>2]|0;
  $43 = 1 << $42;
  $44 = $41 & $43;
  $45 = ($44|0)!=(0);
  if ($45) {
   $46 = HEAP32[$iSub>>2]|0;
   $47 = (($aSub) + ($46<<3)|0);
   HEAP32[$p1>>2] = $47;
   $48 = HEAP32[$0>>2]|0;
   $49 = HEAP32[$p1>>2]|0;
   $50 = ((($49)) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$p1>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   _walMerge($48,$51,$53,$aMerge,$nMerge,$54);
  }
  $55 = HEAP32[$iSub>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$iSub>>2] = $56;
 }
 $57 = HEAP32[$nMerge>>2]|0;
 $58 = HEAP32[$3>>2]|0;
 HEAP32[$58>>2] = $57;
 STACKTOP = sp;return;
}
function _walMerge($aContent,$aLeft,$nLeft,$paRight,$pnRight,$aTmp) {
 $aContent = $aContent|0;
 $aLeft = $aLeft|0;
 $nLeft = $nLeft|0;
 $paRight = $paRight|0;
 $pnRight = $pnRight|0;
 $aTmp = $aTmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aRight = 0, $dbpage = 0, $iLeft = 0, $iOut = 0, $iRight = 0, $logpage = 0, $nRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $iLeft = sp + 20|0;
 $iRight = sp + 16|0;
 $iOut = sp + 12|0;
 $nRight = sp + 8|0;
 $aRight = sp + 4|0;
 $logpage = sp + 48|0;
 $dbpage = sp;
 HEAP32[$0>>2] = $aContent;
 HEAP32[$1>>2] = $aLeft;
 HEAP32[$2>>2] = $nLeft;
 HEAP32[$3>>2] = $paRight;
 HEAP32[$4>>2] = $pnRight;
 HEAP32[$5>>2] = $aTmp;
 HEAP32[$iLeft>>2] = 0;
 HEAP32[$iRight>>2] = 0;
 HEAP32[$iOut>>2] = 0;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nRight>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aRight>>2] = $9;
 while(1) {
  $10 = HEAP32[$iRight>>2]|0;
  $11 = HEAP32[$nRight>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   $13 = HEAP32[$iLeft>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ($13|0)<($14|0);
   if (!($15)) {
    break;
   }
  }
  $16 = HEAP32[$iLeft>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ($16|0)<($17|0);
  do {
   if ($18) {
    $19 = HEAP32[$iRight>>2]|0;
    $20 = HEAP32[$nRight>>2]|0;
    $21 = ($19|0)>=($20|0);
    if (!($21)) {
     $22 = HEAP32[$iLeft>>2]|0;
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + ($22<<1)|0);
     $25 = HEAP16[$24>>1]|0;
     $26 = $25&65535;
     $27 = HEAP32[$0>>2]|0;
     $28 = (($27) + ($26<<2)|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$iRight>>2]|0;
     $31 = HEAP32[$aRight>>2]|0;
     $32 = (($31) + ($30<<1)|0);
     $33 = HEAP16[$32>>1]|0;
     $34 = $33&65535;
     $35 = HEAP32[$0>>2]|0;
     $36 = (($35) + ($34<<2)|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = ($29>>>0)<($37>>>0);
     if (!($38)) {
      label = 8;
      break;
     }
    }
    $39 = HEAP32[$iLeft>>2]|0;
    $40 = (($39) + 1)|0;
    HEAP32[$iLeft>>2] = $40;
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + ($39<<1)|0);
    $43 = HEAP16[$42>>1]|0;
    HEAP16[$logpage>>1] = $43;
   } else {
    label = 8;
   }
  } while(0);
  if ((label|0) == 8) {
   label = 0;
   $44 = HEAP32[$iRight>>2]|0;
   $45 = (($44) + 1)|0;
   HEAP32[$iRight>>2] = $45;
   $46 = HEAP32[$aRight>>2]|0;
   $47 = (($46) + ($44<<1)|0);
   $48 = HEAP16[$47>>1]|0;
   HEAP16[$logpage>>1] = $48;
  }
  $49 = HEAP16[$logpage>>1]|0;
  $50 = $49&65535;
  $51 = HEAP32[$0>>2]|0;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  HEAP32[$dbpage>>2] = $53;
  $54 = HEAP16[$logpage>>1]|0;
  $55 = HEAP32[$iOut>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$iOut>>2] = $56;
  $57 = HEAP32[$5>>2]|0;
  $58 = (($57) + ($55<<1)|0);
  HEAP16[$58>>1] = $54;
  $59 = HEAP32[$iLeft>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   continue;
  }
  $62 = HEAP32[$iLeft>>2]|0;
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + ($62<<1)|0);
  $65 = HEAP16[$64>>1]|0;
  $66 = $65&65535;
  $67 = HEAP32[$0>>2]|0;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$dbpage>>2]|0;
  $71 = ($69|0)==($70|0);
  if (!($71)) {
   continue;
  }
  $72 = HEAP32[$iLeft>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$iLeft>>2] = $73;
 }
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$3>>2]|0;
 HEAP32[$75>>2] = $74;
 $76 = HEAP32[$iOut>>2]|0;
 $77 = HEAP32[$4>>2]|0;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$1>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = HEAP32[$iOut>>2]|0;
 $81 = $80<<1;
 _memcpy(($78|0),($79|0),($81|0))|0;
 STACKTOP = sp;return;
}
function _sqlite3OsShmUnmap($id,$deleteFlag) {
 $id = $id|0;
 $deleteFlag = $deleteFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $deleteFlag;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 64|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3BitvecDestroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 L4: do {
  if ($6) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $7 = HEAP32[$i>>2]|0;
    $8 = ($7>>>0)<(125);
    if (!($8)) {
     break L4;
    }
    $9 = HEAP32[$i>>2]|0;
    $10 = HEAP32[$0>>2]|0;
    $11 = ((($10)) + 12|0);
    $12 = (($11) + ($9<<2)|0);
    $13 = HEAP32[$12>>2]|0;
    _sqlite3BitvecDestroy($13);
    $14 = HEAP32[$i>>2]|0;
    $15 = (($14) + 1)|0;
    HEAP32[$i>>2] = $15;
   }
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 _sqlite3_free($16);
 STACKTOP = sp;return;
}
function _releaseAllSavepoints($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $ii = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $1 = HEAP32[$ii>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 104|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$ii>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 100|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + (($6*48)|0)|0);
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3BitvecDestroy($12);
  $13 = HEAP32[$ii>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$ii>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 72|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3IsMemJournal($21)|0);
  $23 = ($22|0)!=(0);
  if ($23) {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 72|0);
  $26 = HEAP32[$25>>2]|0;
  (_sqlite3OsClose($26)|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 100|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3_free($29);
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 100|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 104|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 56|0);
 HEAP32[$35>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3WalEndReadTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 (_sqlite3WalEndWriteTransaction($1)|0);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)>=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (3 + ($11))|0;
 _walUnlockShared($7,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 40|0);
 HEAP16[$14>>1] = -1;
 STACKTOP = sp;return;
}
function _pagerUnlockDb($pPager,$eLock) {
 $pPager = $pPager|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eLock;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 14|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if ($11) {
  $17 = 0;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 64|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3OsUnlock($14,$15)|0);
  $17 = $16;
 }
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 18|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)!=(5);
 if (!($22)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = $23&255;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 18|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3IsMemJournal($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(1688|0);
 $4 = $3&1;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3OsUnlock($id,$lockType) {
 $id = $id|0;
 $lockType = $lockType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $lockType;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3PagerRollback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $eState = 0, $or$cond = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $rc2 = sp + 4|0;
 $eState = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 17|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(6);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 44|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$0>>2] = $9;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $10 = ((($7)) + 17|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)<=(1);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_pagerUseWal($14)|0);
 $16 = ($15|0)!=(0);
 $17 = HEAP32[$1>>2]|0;
 do {
  if ($16) {
   $18 = (_sqlite3PagerSavepoint($17,2,-1)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = (_pager_end_transaction($19,$23,0)|0);
   HEAP32[$rc2>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    $27 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $27;
   }
  } else {
   $28 = ((($17)) + 68|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 17|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = ($35|0)==(2);
    if (!($36)) {
     $54 = HEAP32[$1>>2]|0;
     $55 = (_pager_playback($54,0)|0);
     HEAP32[$rc>>2] = $55;
     break;
    }
   }
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 17|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   HEAP32[$eState>>2] = $40;
   $41 = HEAP32[$1>>2]|0;
   $42 = (_pager_end_transaction($41,0,0)|0);
   HEAP32[$rc>>2] = $42;
   $43 = HEAP32[$1>>2]|0;
   $44 = ((($43)) + 16|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = ($45<<24>>24)==(0);
   $47 = HEAP32[$eState>>2]|0;
   $48 = ($47|0)>(2);
   $or$cond = $46 & $48;
   if ($or$cond) {
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 44|0);
    HEAP32[$50>>2] = 4;
    $51 = HEAP32[$1>>2]|0;
    $52 = ((($51)) + 17|0);
    HEAP8[$52>>0] = 6;
    $53 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $53;
    $59 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($59|0);
   }
  }
 } while(0);
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$rc>>2]|0;
 $58 = (_pager_error($56,$57)|0);
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _pager_end_transaction($pPager,$hasMaster,$bCommit) {
 $pPager = $pPager|0;
 $hasMaster = $hasMaster|0;
 $bCommit = $bCommit|0;
 var $$ = 0, $$4 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDelete = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $rc2 = sp + 4|0;
 $bDelete = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $hasMaster;
 HEAP32[$3>>2] = $bCommit;
 HEAP32[$rc>>2] = 0;
 HEAP32[$rc2>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(2);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 18|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)<(2);
  if ($13) {
   HEAP32[$0>>2] = 0;
   $190 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($190|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 _releaseAllSavepoints($14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 68|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 L6: do {
  if ($19) {
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 68|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3IsMemJournal($22)|0);
   $24 = ($23|0)!=(0);
   $25 = HEAP32[$1>>2]|0;
   if ($24) {
    $26 = ((($25)) + 68|0);
    $27 = HEAP32[$26>>2]|0;
    (_sqlite3OsClose($27)|0);
    break;
   }
   $28 = ((($25)) + 5|0);
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   $31 = ($30|0)==(3);
   $32 = HEAP32[$1>>2]|0;
   if ($31) {
    $33 = ((($32)) + 80|0);
    $34 = $33;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)==(0);
    $41 = ($39|0)==(0);
    $42 = $40 & $41;
    if ($42) {
     HEAP32[$rc>>2] = 0;
    } else {
     $43 = HEAP32[$1>>2]|0;
     $44 = ((($43)) + 68|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (_sqlite3OsTruncate($45,0,0)|0);
     HEAP32[$rc>>2] = $46;
     $47 = HEAP32[$rc>>2]|0;
     $48 = ($47|0)==(0);
     if ($48) {
      $49 = HEAP32[$1>>2]|0;
      $50 = ((($49)) + 8|0);
      $51 = HEAP8[$50>>0]|0;
      $52 = $51&255;
      $53 = ($52|0)!=(0);
      if ($53) {
       $54 = HEAP32[$1>>2]|0;
       $55 = ((($54)) + 68|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = HEAP32[$1>>2]|0;
       $58 = ((($57)) + 12|0);
       $59 = HEAP8[$58>>0]|0;
       $60 = $59&255;
       $61 = (_sqlite3OsSync($56,$60)|0);
       HEAP32[$rc>>2] = $61;
      }
     }
    }
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 80|0);
    $64 = $63;
    $65 = $64;
    HEAP32[$65>>2] = 0;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = 0;
    break;
   } else {
    $68 = ((($32)) + 5|0);
    $69 = HEAP8[$68>>0]|0;
    $70 = $69&255;
    $71 = ($70|0)==(1);
    do {
     if (!($71)) {
      $72 = HEAP32[$1>>2]|0;
      $73 = ((($72)) + 4|0);
      $74 = HEAP8[$73>>0]|0;
      $75 = $74&255;
      $76 = ($75|0)!=(0);
      if ($76) {
       $77 = HEAP32[$1>>2]|0;
       $78 = ((($77)) + 5|0);
       $79 = HEAP8[$78>>0]|0;
       $80 = $79&255;
       $81 = ($80|0)!=(5);
       if ($81) {
        break;
       }
      }
      $91 = HEAP32[$1>>2]|0;
      $92 = ((($91)) + 13|0);
      $93 = HEAP8[$92>>0]|0;
      $94 = ($93<<24>>24)!=(0);
      $$ = $94 ? 0 : 1;
      $95 = $$&1;
      HEAP32[$bDelete>>2] = $95;
      $96 = HEAP32[$1>>2]|0;
      $97 = ((($96)) + 68|0);
      $98 = HEAP32[$97>>2]|0;
      (_sqlite3OsClose($98)|0);
      $99 = HEAP32[$bDelete>>2]|0;
      $100 = ($99|0)!=(0);
      if (!($100)) {
       break L6;
      }
      $101 = HEAP32[$1>>2]|0;
      $102 = HEAP32[$101>>2]|0;
      $103 = HEAP32[$1>>2]|0;
      $104 = ((($103)) + 180|0);
      $105 = HEAP32[$104>>2]|0;
      $106 = HEAP32[$1>>2]|0;
      $107 = ((($106)) + 9|0);
      $108 = HEAP8[$107>>0]|0;
      $109 = $108&255;
      $110 = (_sqlite3OsDelete($102,$105,$109)|0);
      HEAP32[$rc>>2] = $110;
      break L6;
     }
    } while(0);
    $82 = HEAP32[$1>>2]|0;
    $83 = HEAP32[$2>>2]|0;
    $84 = (_zeroJournalHdr($82,$83)|0);
    HEAP32[$rc>>2] = $84;
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 80|0);
    $87 = $86;
    $88 = $87;
    HEAP32[$88>>2] = 0;
    $89 = (($87) + 4)|0;
    $90 = $89;
    HEAP32[$90>>2] = 0;
    break;
   }
  }
 } while(0);
 $111 = HEAP32[$1>>2]|0;
 $112 = ((($111)) + 60|0);
 $113 = HEAP32[$112>>2]|0;
 _sqlite3BitvecDestroy($113);
 $114 = HEAP32[$1>>2]|0;
 $115 = ((($114)) + 60|0);
 HEAP32[$115>>2] = 0;
 $116 = HEAP32[$1>>2]|0;
 $117 = ((($116)) + 48|0);
 HEAP32[$117>>2] = 0;
 $118 = HEAP32[$1>>2]|0;
 $119 = ((($118)) + 212|0);
 $120 = HEAP32[$119>>2]|0;
 _sqlite3PcacheCleanAll($120);
 $121 = HEAP32[$1>>2]|0;
 $122 = ((($121)) + 212|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = HEAP32[$1>>2]|0;
 $125 = ((($124)) + 28|0);
 $126 = HEAP32[$125>>2]|0;
 _sqlite3PcacheTruncate($123,$126);
 $127 = HEAP32[$1>>2]|0;
 $128 = (_pagerUseWal($127)|0);
 $129 = ($128|0)!=(0);
 if ($129) {
  $130 = HEAP32[$1>>2]|0;
  $131 = ((($130)) + 216|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = (_sqlite3WalEndWriteTransaction($132)|0);
  HEAP32[$rc2>>2] = $133;
 } else {
  $134 = HEAP32[$rc>>2]|0;
  $135 = ($134|0)==(0);
  $136 = HEAP32[$3>>2]|0;
  $137 = ($136|0)!=(0);
  $or$cond = $135 & $137;
  if ($or$cond) {
   $138 = HEAP32[$1>>2]|0;
   $139 = ((($138)) + 36|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = HEAP32[$1>>2]|0;
   $142 = ((($141)) + 28|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = ($140>>>0)>($143>>>0);
   if ($144) {
    $145 = HEAP32[$1>>2]|0;
    $146 = HEAP32[$1>>2]|0;
    $147 = ((($146)) + 28|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = (_pager_truncate($145,$148)|0);
    HEAP32[$rc>>2] = $149;
   }
  }
 }
 $150 = HEAP32[$rc>>2]|0;
 $151 = ($150|0)==(0);
 $152 = HEAP32[$3>>2]|0;
 $153 = ($152|0)!=(0);
 $or$cond3 = $151 & $153;
 if ($or$cond3) {
  $154 = HEAP32[$1>>2]|0;
  $155 = ((($154)) + 64|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = HEAP32[$156>>2]|0;
  $158 = ($157|0)!=(0|0);
  if ($158) {
   $159 = HEAP32[$1>>2]|0;
   $160 = ((($159)) + 64|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = (_sqlite3OsFileControl($161,22,0)|0);
   HEAP32[$rc>>2] = $162;
   $163 = HEAP32[$rc>>2]|0;
   $164 = ($163|0)==(12);
   $$4 = $164 ? 0 : $162;
   HEAP32[$rc>>2] = $$4;
  }
 }
 $165 = HEAP32[$1>>2]|0;
 $166 = ((($165)) + 4|0);
 $167 = HEAP8[$166>>0]|0;
 $168 = ($167<<24>>24)!=(0);
 do {
  if (!($168)) {
   $169 = HEAP32[$1>>2]|0;
   $170 = (_pagerUseWal($169)|0);
   $171 = ($170|0)!=(0);
   if ($171) {
    $172 = HEAP32[$1>>2]|0;
    $173 = ((($172)) + 216|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = (_sqlite3WalExclusiveMode($174,0)|0);
    $176 = ($175|0)!=(0);
    if (!($176)) {
     break;
    }
   }
   $177 = HEAP32[$1>>2]|0;
   $178 = (_pagerUnlockDb($177,1)|0);
   HEAP32[$rc2>>2] = $178;
   $179 = HEAP32[$1>>2]|0;
   $180 = ((($179)) + 19|0);
   HEAP8[$180>>0] = 0;
  }
 } while(0);
 $181 = HEAP32[$1>>2]|0;
 $182 = ((($181)) + 17|0);
 HEAP8[$182>>0] = 1;
 $183 = HEAP32[$1>>2]|0;
 $184 = ((($183)) + 20|0);
 HEAP8[$184>>0] = 0;
 $185 = HEAP32[$rc>>2]|0;
 $186 = ($185|0)==(0);
 $187 = HEAP32[$rc2>>2]|0;
 $188 = HEAP32[$rc>>2]|0;
 $189 = $186 ? $187 : $188;
 HEAP32[$0>>2] = $189;
 $190 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($190|0);
}
function _sqlite3PagerSavepoint($pPager,$op,$iSavepoint) {
 $pPager = $pPager|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $ii = 0, $nNew = 0, $pSavepoint = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $ii = sp + 8|0;
 $nNew = sp + 4|0;
 $pSavepoint = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 104|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)<($11|0);
 if (!($12)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ($14|0)==(1);
 $16 = $15 ? 0 : 1;
 $17 = (($13) + ($16))|0;
 HEAP32[$nNew>>2] = $17;
 $18 = HEAP32[$nNew>>2]|0;
 HEAP32[$ii>>2] = $18;
 while(1) {
  $19 = HEAP32[$ii>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 104|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($19|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$ii>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 100|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($24*48)|0)|0);
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3BitvecDestroy($30);
  $31 = HEAP32[$ii>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$ii>>2] = $32;
 }
 $33 = HEAP32[$nNew>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 104|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$1>>2]|0;
 $37 = ($36|0)==(1);
 if (!($37)) {
  $56 = HEAP32[$0>>2]|0;
  $57 = (_pagerUseWal($56)|0);
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = HEAP32[$0>>2]|0;
   $60 = ((($59)) + 68|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$61>>2]|0;
   $63 = ($62|0)!=(0|0);
   if (!($63)) {
    $76 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($76|0);
   }
  }
  $64 = HEAP32[$nNew>>2]|0;
  $65 = ($64|0)==(0);
  if ($65) {
   $72 = 0;
  } else {
   $66 = HEAP32[$nNew>>2]|0;
   $67 = (($66) - 1)|0;
   $68 = HEAP32[$0>>2]|0;
   $69 = ((($68)) + 100|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + (($67*48)|0)|0);
   $72 = $71;
  }
  HEAP32[$pSavepoint>>2] = $72;
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$pSavepoint>>2]|0;
  $75 = (_pagerPlaybackSavepoint($73,$74)|0);
  HEAP32[$rc>>2] = $75;
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $38 = HEAP32[$nNew>>2]|0;
 $39 = ($38|0)==(0);
 if (!($39)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 72|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0|0);
 if (!($44)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 72|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (_sqlite3IsMemJournal($47)|0);
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$0>>2]|0;
  $51 = ((($50)) + 72|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (_sqlite3OsTruncate($52,0,0)|0);
  HEAP32[$rc>>2] = $53;
 }
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 56|0);
 HEAP32[$55>>2] = 0;
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _pager_playback($pPager,$isHot) {
 $pPager = $pPager|0;
 $isHot = $isHot|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $mxPg = 0, $nPlayback = 0, $nRec = 0, $needPagerReset = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pVfs = 0, $rc = 0, $res = 0, $szJ = 0, $u = 0;
 var $vararg_buffer = 0, $vararg_ptr8 = 0, $zMaster = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $pVfs = sp + 48|0;
 $szJ = sp;
 $nRec = sp + 44|0;
 $u = sp + 40|0;
 $mxPg = sp + 36|0;
 $rc = sp + 32|0;
 $res = sp + 28|0;
 $zMaster = sp + 24|0;
 $needPagerReset = sp + 20|0;
 $nPlayback = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $isHot;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$mxPg>>2] = 0;
 HEAP32[$res>>2] = 1;
 HEAP32[$zMaster>>2] = 0;
 HEAP32[$nPlayback>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3OsFileSize($6,$szJ)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 L1: do {
  if (!($9)) {
   $10 = HEAP32[$0>>2]|0;
   $11 = ((($10)) + 208|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$zMaster>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 68|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$zMaster>>2]|0;
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = ((($18)) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 1)|0;
   $22 = (_readMasterJournal($15,$16,$21)|0);
   HEAP32[$rc>>2] = $22;
   $23 = HEAP32[$rc>>2]|0;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = HEAP32[$zMaster>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)!=(0);
    if ($28) {
     $29 = HEAP32[$pVfs>>2]|0;
     $30 = HEAP32[$zMaster>>2]|0;
     $31 = (_sqlite3OsAccess($29,$30,0,$res)|0);
     HEAP32[$rc>>2] = $31;
    }
   }
   HEAP32[$zMaster>>2] = 0;
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(0);
   $34 = HEAP32[$res>>2]|0;
   $35 = ($34|0)!=(0);
   $or$cond = $33 & $35;
   if ($or$cond) {
    $36 = HEAP32[$0>>2]|0;
    $37 = ((($36)) + 80|0);
    $38 = $37;
    $39 = $38;
    HEAP32[$39>>2] = 0;
    $40 = (($38) + 4)|0;
    $41 = $40;
    HEAP32[$41>>2] = 0;
    $42 = HEAP32[$1>>2]|0;
    HEAP32[$needPagerReset>>2] = $42;
    L8: while(1) {
     $43 = HEAP32[$0>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = $szJ;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($45) + 4)|0;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (_readJournalHdr($43,$44,$47,$50,$nRec,$mxPg)|0);
     HEAP32[$rc>>2] = $51;
     $52 = HEAP32[$rc>>2]|0;
     $53 = ($52|0)!=(0);
     if ($53) {
      label = 8;
      break;
     }
     $56 = HEAP32[$nRec>>2]|0;
     $57 = ($56|0)==(-1);
     if ($57) {
      $58 = $szJ;
      $59 = $58;
      $60 = HEAP32[$59>>2]|0;
      $61 = (($58) + 4)|0;
      $62 = $61;
      $63 = HEAP32[$62>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = ((($64)) + 156|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = (_i64Subtract(($60|0),($63|0),($66|0),0)|0);
      $68 = tempRet0;
      $69 = HEAP32[$0>>2]|0;
      $70 = ((($69)) + 160|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = (($71) + 8)|0;
      $73 = ($72|0)<(0);
      $74 = $73 << 31 >> 31;
      $75 = (___divdi3(($67|0),($68|0),($72|0),($74|0))|0);
      $76 = tempRet0;
      HEAP32[$nRec>>2] = $75;
     }
     $77 = HEAP32[$nRec>>2]|0;
     $78 = ($77|0)!=(0);
     $79 = HEAP32[$1>>2]|0;
     $80 = ($79|0)!=(0);
     $or$cond3 = $78 | $80;
     if (!($or$cond3)) {
      $81 = HEAP32[$0>>2]|0;
      $82 = ((($81)) + 88|0);
      $83 = $82;
      $84 = $83;
      $85 = HEAP32[$84>>2]|0;
      $86 = (($83) + 4)|0;
      $87 = $86;
      $88 = HEAP32[$87>>2]|0;
      $89 = HEAP32[$0>>2]|0;
      $90 = ((($89)) + 156|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (_i64Add(($85|0),($88|0),($91|0),0)|0);
      $93 = tempRet0;
      $94 = HEAP32[$0>>2]|0;
      $95 = ((($94)) + 80|0);
      $96 = $95;
      $97 = $96;
      $98 = HEAP32[$97>>2]|0;
      $99 = (($96) + 4)|0;
      $100 = $99;
      $101 = HEAP32[$100>>2]|0;
      $102 = ($92|0)==($98|0);
      $103 = ($93|0)==($101|0);
      $104 = $102 & $103;
      if ($104) {
       $105 = $szJ;
       $106 = $105;
       $107 = HEAP32[$106>>2]|0;
       $108 = (($105) + 4)|0;
       $109 = $108;
       $110 = HEAP32[$109>>2]|0;
       $111 = HEAP32[$0>>2]|0;
       $112 = ((($111)) + 80|0);
       $113 = $112;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (($113) + 4)|0;
       $117 = $116;
       $118 = HEAP32[$117>>2]|0;
       $119 = (_i64Subtract(($107|0),($110|0),($115|0),($118|0))|0);
       $120 = tempRet0;
       $121 = HEAP32[$0>>2]|0;
       $122 = ((($121)) + 160|0);
       $123 = HEAP32[$122>>2]|0;
       $124 = (($123) + 8)|0;
       $125 = ($124|0)<(0);
       $126 = $125 << 31 >> 31;
       $127 = (___divdi3(($119|0),($120|0),($124|0),($126|0))|0);
       $128 = tempRet0;
       HEAP32[$nRec>>2] = $127;
      }
     }
     $129 = HEAP32[$0>>2]|0;
     $130 = ((($129)) + 80|0);
     $131 = $130;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = (($131) + 4)|0;
     $135 = $134;
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$0>>2]|0;
     $138 = ((($137)) + 156|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($133|0)==($139|0);
     $141 = ($136|0)==(0);
     $142 = $140 & $141;
     if ($142) {
      $143 = HEAP32[$0>>2]|0;
      $144 = HEAP32[$mxPg>>2]|0;
      $145 = (_pager_truncate($143,$144)|0);
      HEAP32[$rc>>2] = $145;
      $146 = HEAP32[$rc>>2]|0;
      $147 = ($146|0)!=(0);
      if ($147) {
       break L1;
      }
      $148 = HEAP32[$mxPg>>2]|0;
      $149 = HEAP32[$0>>2]|0;
      $150 = ((($149)) + 28|0);
      HEAP32[$150>>2] = $148;
     }
     HEAP32[$u>>2] = 0;
     while(1) {
      $151 = HEAP32[$u>>2]|0;
      $152 = HEAP32[$nRec>>2]|0;
      $153 = ($151>>>0)<($152>>>0);
      if (!($153)) {
       continue L8;
      }
      $154 = HEAP32[$needPagerReset>>2]|0;
      $155 = ($154|0)!=(0);
      if ($155) {
       $156 = HEAP32[$0>>2]|0;
       _pager_reset($156);
       HEAP32[$needPagerReset>>2] = 0;
      }
      $157 = HEAP32[$0>>2]|0;
      $158 = HEAP32[$0>>2]|0;
      $159 = ((($158)) + 80|0);
      $160 = (_pager_playback_one_page($157,$159,0,1,0)|0);
      HEAP32[$rc>>2] = $160;
      $161 = HEAP32[$rc>>2]|0;
      $162 = ($161|0)==(0);
      if (!($162)) {
       break;
      }
      $163 = HEAP32[$nPlayback>>2]|0;
      $164 = (($163) + 1)|0;
      HEAP32[$nPlayback>>2] = $164;
      $165 = HEAP32[$u>>2]|0;
      $166 = (($165) + 1)|0;
      HEAP32[$u>>2] = $166;
     }
     $167 = HEAP32[$rc>>2]|0;
     $168 = ($167|0)==(101);
     if (!($168)) {
      label = 26;
      break;
     }
     $169 = $szJ;
     $170 = $169;
     $171 = HEAP32[$170>>2]|0;
     $172 = (($169) + 4)|0;
     $173 = $172;
     $174 = HEAP32[$173>>2]|0;
     $175 = HEAP32[$0>>2]|0;
     $176 = ((($175)) + 80|0);
     $177 = $176;
     $178 = $177;
     HEAP32[$178>>2] = $171;
     $179 = (($177) + 4)|0;
     $180 = $179;
     HEAP32[$180>>2] = $174;
    }
    if ((label|0) == 8) {
     $54 = HEAP32[$rc>>2]|0;
     $55 = ($54|0)==(101);
     if (!($55)) {
      break;
     }
     HEAP32[$rc>>2] = 0;
     break;
    }
    else if ((label|0) == 26) {
     $181 = HEAP32[$rc>>2]|0;
     $182 = ($181|0)==(522);
     if (!($182)) {
      break;
     }
     HEAP32[$rc>>2] = 0;
     break;
    }
   }
  }
 } while(0);
 $183 = HEAP32[$0>>2]|0;
 $184 = ((($183)) + 13|0);
 $185 = HEAP8[$184>>0]|0;
 $186 = HEAP32[$0>>2]|0;
 $187 = ((($186)) + 19|0);
 HEAP8[$187>>0] = $185;
 $188 = HEAP32[$rc>>2]|0;
 $189 = ($188|0)==(0);
 if ($189) {
  $190 = HEAP32[$0>>2]|0;
  $191 = ((($190)) + 208|0);
  $192 = HEAP32[$191>>2]|0;
  HEAP32[$zMaster>>2] = $192;
  $193 = HEAP32[$0>>2]|0;
  $194 = ((($193)) + 68|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = HEAP32[$zMaster>>2]|0;
  $197 = HEAP32[$0>>2]|0;
  $198 = HEAP32[$197>>2]|0;
  $199 = ((($198)) + 8|0);
  $200 = HEAP32[$199>>2]|0;
  $201 = (($200) + 1)|0;
  $202 = (_readMasterJournal($195,$196,$201)|0);
  HEAP32[$rc>>2] = $202;
 }
 $203 = HEAP32[$rc>>2]|0;
 $204 = ($203|0)==(0);
 do {
  if ($204) {
   $205 = HEAP32[$0>>2]|0;
   $206 = ((($205)) + 17|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $207&255;
   $209 = ($208|0)>=(4);
   if (!($209)) {
    $210 = HEAP32[$0>>2]|0;
    $211 = ((($210)) + 17|0);
    $212 = HEAP8[$211>>0]|0;
    $213 = $212&255;
    $214 = ($213|0)==(0);
    if (!($214)) {
     break;
    }
   }
   $215 = HEAP32[$0>>2]|0;
   $216 = (_sqlite3PagerSync($215,0)|0);
   HEAP32[$rc>>2] = $216;
  }
 } while(0);
 $217 = HEAP32[$rc>>2]|0;
 $218 = ($217|0)==(0);
 if ($218) {
  $219 = HEAP32[$0>>2]|0;
  $220 = HEAP32[$zMaster>>2]|0;
  $221 = HEAP8[$220>>0]|0;
  $222 = $221 << 24 >> 24;
  $223 = ($222|0)!=(0);
  $224 = $223&1;
  $225 = (_pager_end_transaction($219,$224,0)|0);
  HEAP32[$rc>>2] = $225;
 }
 $226 = HEAP32[$rc>>2]|0;
 $227 = ($226|0)==(0);
 if ($227) {
  $228 = HEAP32[$zMaster>>2]|0;
  $229 = HEAP8[$228>>0]|0;
  $230 = $229 << 24 >> 24;
  $231 = ($230|0)!=(0);
  $232 = HEAP32[$res>>2]|0;
  $233 = ($232|0)!=(0);
  $or$cond5 = $231 & $233;
  if ($or$cond5) {
   $234 = HEAP32[$0>>2]|0;
   $235 = HEAP32[$zMaster>>2]|0;
   $236 = (_pager_delmaster($234,$235)|0);
   HEAP32[$rc>>2] = $236;
  }
 }
 $237 = HEAP32[$1>>2]|0;
 $238 = ($237|0)!=(0);
 $239 = HEAP32[$nPlayback>>2]|0;
 $240 = ($239|0)!=(0);
 $or$cond7 = $238 & $240;
 if (!($or$cond7)) {
  $245 = HEAP32[$0>>2]|0;
  _setSectorSize($245);
  $246 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($246|0);
 }
 $241 = HEAP32[$nPlayback>>2]|0;
 $242 = HEAP32[$0>>2]|0;
 $243 = ((($242)) + 180|0);
 $244 = HEAP32[$243>>2]|0;
 HEAP32[$vararg_buffer>>2] = $241;
 $vararg_ptr8 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $244;
 _sqlite3_log(539,19013,$vararg_buffer);
 $245 = HEAP32[$0>>2]|0;
 _setSectorSize($245);
 $246 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($246|0);
}
function _pagerPlaybackSavepoint($pPager,$pSavepoint) {
 $pPager = $pPager|0;
 $pSavepoint = $pSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dummy = 0, $iHdrOff = 0, $ii = 0, $ii1 = 0, $nJRec = 0;
 var $offset = 0, $pDone = 0, $rc = 0, $szJ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $szJ = sp + 16|0;
 $iHdrOff = sp + 8|0;
 $rc = sp + 44|0;
 $pDone = sp + 40|0;
 $ii = sp + 36|0;
 $nJRec = sp + 32|0;
 $dummy = sp + 28|0;
 $ii1 = sp + 24|0;
 $offset = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pSavepoint;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDone>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3BitvecCreate($7)|0);
  HEAP32[$pDone>>2] = $8;
  $9 = HEAP32[$pDone>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   HEAP32[$0>>2] = 7;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $21 = $15;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $21 = $18;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 28|0);
 HEAP32[$20>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 13|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 19|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$2>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $29 = HEAP32[$1>>2]|0;
  $30 = (_pagerUseWal($29)|0);
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$1>>2]|0;
   $33 = (_pagerRollbackWal($32)|0);
   HEAP32[$0>>2] = $33;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 80|0);
 $36 = $35;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = $szJ;
 $43 = $42;
 HEAP32[$43>>2] = $38;
 $44 = (($42) + 4)|0;
 $45 = $44;
 HEAP32[$45>>2] = $41;
 $46 = HEAP32[$2>>2]|0;
 $47 = ($46|0)!=(0|0);
 L15: do {
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = (_pagerUseWal($48)|0);
   $50 = ($49|0)!=(0);
   if ($50) {
    label = 19;
   } else {
    $51 = HEAP32[$2>>2]|0;
    $52 = ((($51)) + 8|0);
    $53 = $52;
    $54 = $53;
    $55 = HEAP32[$54>>2]|0;
    $56 = (($53) + 4)|0;
    $57 = $56;
    $58 = HEAP32[$57>>2]|0;
    $59 = ($55|0)!=(0);
    $60 = ($58|0)!=(0);
    $61 = $59 | $60;
    if ($61) {
     $62 = HEAP32[$2>>2]|0;
     $63 = ((($62)) + 8|0);
     $64 = $63;
     $65 = $64;
     $66 = HEAP32[$65>>2]|0;
     $67 = (($64) + 4)|0;
     $68 = $67;
     $69 = HEAP32[$68>>2]|0;
     $78 = $66;$81 = $69;
    } else {
     $70 = $szJ;
     $71 = $70;
     $72 = HEAP32[$71>>2]|0;
     $73 = (($70) + 4)|0;
     $74 = $73;
     $75 = HEAP32[$74>>2]|0;
     $78 = $72;$81 = $75;
    }
    $76 = $iHdrOff;
    $77 = $76;
    HEAP32[$77>>2] = $78;
    $79 = (($76) + 4)|0;
    $80 = $79;
    HEAP32[$80>>2] = $81;
    $82 = HEAP32[$2>>2]|0;
    $83 = $82;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $86 = (($83) + 4)|0;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 80|0);
    $91 = $90;
    $92 = $91;
    HEAP32[$92>>2] = $85;
    $93 = (($91) + 4)|0;
    $94 = $93;
    HEAP32[$94>>2] = $88;
    while(1) {
     $95 = HEAP32[$rc>>2]|0;
     $96 = ($95|0)==(0);
     if (!($96)) {
      break L15;
     }
     $97 = HEAP32[$1>>2]|0;
     $98 = ((($97)) + 80|0);
     $99 = $98;
     $100 = $99;
     $101 = HEAP32[$100>>2]|0;
     $102 = (($99) + 4)|0;
     $103 = $102;
     $104 = HEAP32[$103>>2]|0;
     $105 = $iHdrOff;
     $106 = $105;
     $107 = HEAP32[$106>>2]|0;
     $108 = (($105) + 4)|0;
     $109 = $108;
     $110 = HEAP32[$109>>2]|0;
     $111 = ($104|0)<($110|0);
     $112 = ($101>>>0)<($107>>>0);
     $113 = ($104|0)==($110|0);
     $114 = $113 & $112;
     $115 = $111 | $114;
     if (!($115)) {
      break L15;
     }
     $116 = HEAP32[$1>>2]|0;
     $117 = HEAP32[$1>>2]|0;
     $118 = ((($117)) + 80|0);
     $119 = HEAP32[$pDone>>2]|0;
     $120 = (_pager_playback_one_page($116,$118,$119,1,1)|0);
     HEAP32[$rc>>2] = $120;
    }
   }
  } else {
   label = 19;
  }
 } while(0);
 if ((label|0) == 19) {
  $121 = HEAP32[$1>>2]|0;
  $122 = ((($121)) + 80|0);
  $123 = $122;
  $124 = $123;
  HEAP32[$124>>2] = 0;
  $125 = (($123) + 4)|0;
  $126 = $125;
  HEAP32[$126>>2] = 0;
 }
 L28: while(1) {
  $127 = HEAP32[$rc>>2]|0;
  $128 = ($127|0)==(0);
  if (!($128)) {
   break;
  }
  $129 = HEAP32[$1>>2]|0;
  $130 = ((($129)) + 80|0);
  $131 = $130;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = (($131) + 4)|0;
  $135 = $134;
  $136 = HEAP32[$135>>2]|0;
  $137 = $szJ;
  $138 = $137;
  $139 = HEAP32[$138>>2]|0;
  $140 = (($137) + 4)|0;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = ($136|0)<($142|0);
  $144 = ($133>>>0)<($139>>>0);
  $145 = ($136|0)==($142|0);
  $146 = $145 & $144;
  $147 = $143 | $146;
  if (!($147)) {
   break;
  }
  HEAP32[$nJRec>>2] = 0;
  $148 = HEAP32[$1>>2]|0;
  $149 = $szJ;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = (($149) + 4)|0;
  $153 = $152;
  $154 = HEAP32[$153>>2]|0;
  $155 = (_readJournalHdr($148,0,$151,$154,$nJRec,$dummy)|0);
  HEAP32[$rc>>2] = $155;
  $156 = HEAP32[$nJRec>>2]|0;
  $157 = ($156|0)==(0);
  if ($157) {
   $158 = HEAP32[$1>>2]|0;
   $159 = ((($158)) + 88|0);
   $160 = $159;
   $161 = $160;
   $162 = HEAP32[$161>>2]|0;
   $163 = (($160) + 4)|0;
   $164 = $163;
   $165 = HEAP32[$164>>2]|0;
   $166 = HEAP32[$1>>2]|0;
   $167 = ((($166)) + 156|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (_i64Add(($162|0),($165|0),($168|0),0)|0);
   $170 = tempRet0;
   $171 = HEAP32[$1>>2]|0;
   $172 = ((($171)) + 80|0);
   $173 = $172;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($173) + 4)|0;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = ($169|0)==($175|0);
   $180 = ($170|0)==($178|0);
   $181 = $179 & $180;
   if ($181) {
    $182 = $szJ;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = (($182) + 4)|0;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = HEAP32[$1>>2]|0;
    $189 = ((($188)) + 80|0);
    $190 = $189;
    $191 = $190;
    $192 = HEAP32[$191>>2]|0;
    $193 = (($190) + 4)|0;
    $194 = $193;
    $195 = HEAP32[$194>>2]|0;
    $196 = (_i64Subtract(($184|0),($187|0),($192|0),($195|0))|0);
    $197 = tempRet0;
    $198 = HEAP32[$1>>2]|0;
    $199 = ((($198)) + 160|0);
    $200 = HEAP32[$199>>2]|0;
    $201 = (($200) + 8)|0;
    $202 = ($201|0)<(0);
    $203 = $202 << 31 >> 31;
    $204 = (___divdi3(($196|0),($197|0),($201|0),($203|0))|0);
    $205 = tempRet0;
    HEAP32[$nJRec>>2] = $204;
   }
  }
  HEAP32[$ii>>2] = 0;
  while(1) {
   $206 = HEAP32[$rc>>2]|0;
   $207 = ($206|0)==(0);
   if (!($207)) {
    continue L28;
   }
   $208 = HEAP32[$ii>>2]|0;
   $209 = HEAP32[$nJRec>>2]|0;
   $210 = ($208>>>0)<($209>>>0);
   if (!($210)) {
    continue L28;
   }
   $211 = HEAP32[$1>>2]|0;
   $212 = ((($211)) + 80|0);
   $213 = $212;
   $214 = $213;
   $215 = HEAP32[$214>>2]|0;
   $216 = (($213) + 4)|0;
   $217 = $216;
   $218 = HEAP32[$217>>2]|0;
   $219 = $szJ;
   $220 = $219;
   $221 = HEAP32[$220>>2]|0;
   $222 = (($219) + 4)|0;
   $223 = $222;
   $224 = HEAP32[$223>>2]|0;
   $225 = ($218|0)<($224|0);
   $226 = ($215>>>0)<($221>>>0);
   $227 = ($218|0)==($224|0);
   $228 = $227 & $226;
   $229 = $225 | $228;
   if (!($229)) {
    continue L28;
   }
   $230 = HEAP32[$1>>2]|0;
   $231 = HEAP32[$1>>2]|0;
   $232 = ((($231)) + 80|0);
   $233 = HEAP32[$pDone>>2]|0;
   $234 = (_pager_playback_one_page($230,$232,$233,1,1)|0);
   HEAP32[$rc>>2] = $234;
   $235 = HEAP32[$ii>>2]|0;
   $236 = (($235) + 1)|0;
   HEAP32[$ii>>2] = $236;
  }
 }
 $237 = HEAP32[$2>>2]|0;
 $238 = ($237|0)!=(0|0);
 L42: do {
  if ($238) {
   $239 = HEAP32[$2>>2]|0;
   $240 = ((($239)) + 24|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[$1>>2]|0;
   $243 = ((($242)) + 160|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (4 + ($244))|0;
   $246 = ($245|0)<(0);
   $247 = $246 << 31 >> 31;
   $248 = (___muldi3(($241|0),0,($245|0),($247|0))|0);
   $249 = tempRet0;
   $250 = $offset;
   $251 = $250;
   HEAP32[$251>>2] = $248;
   $252 = (($250) + 4)|0;
   $253 = $252;
   HEAP32[$253>>2] = $249;
   $254 = HEAP32[$1>>2]|0;
   $255 = (_pagerUseWal($254)|0);
   $256 = ($255|0)!=(0);
   if ($256) {
    $257 = HEAP32[$1>>2]|0;
    $258 = ((($257)) + 216|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = HEAP32[$2>>2]|0;
    $261 = ((($260)) + 28|0);
    $262 = (_sqlite3WalSavepointUndo($259,$261)|0);
    HEAP32[$rc>>2] = $262;
   }
   $263 = HEAP32[$2>>2]|0;
   $264 = ((($263)) + 24|0);
   $265 = HEAP32[$264>>2]|0;
   HEAP32[$ii1>>2] = $265;
   while(1) {
    $266 = HEAP32[$rc>>2]|0;
    $267 = ($266|0)==(0);
    if (!($267)) {
     break L42;
    }
    $268 = HEAP32[$ii1>>2]|0;
    $269 = HEAP32[$1>>2]|0;
    $270 = ((($269)) + 56|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = ($268>>>0)<($271>>>0);
    if (!($272)) {
     break L42;
    }
    $273 = HEAP32[$1>>2]|0;
    $274 = HEAP32[$pDone>>2]|0;
    $275 = (_pager_playback_one_page($273,$offset,$274,0,1)|0);
    HEAP32[$rc>>2] = $275;
    $276 = HEAP32[$ii1>>2]|0;
    $277 = (($276) + 1)|0;
    HEAP32[$ii1>>2] = $277;
   }
  }
 } while(0);
 $278 = HEAP32[$pDone>>2]|0;
 _sqlite3BitvecDestroy($278);
 $279 = HEAP32[$rc>>2]|0;
 $280 = ($279|0)==(0);
 if ($280) {
  $281 = $szJ;
  $282 = $281;
  $283 = HEAP32[$282>>2]|0;
  $284 = (($281) + 4)|0;
  $285 = $284;
  $286 = HEAP32[$285>>2]|0;
  $287 = HEAP32[$1>>2]|0;
  $288 = ((($287)) + 80|0);
  $289 = $288;
  $290 = $289;
  HEAP32[$290>>2] = $283;
  $291 = (($289) + 4)|0;
  $292 = $291;
  HEAP32[$292>>2] = $286;
 }
 $293 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $293;
 $294 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($294|0);
}
function _pagerRollbackWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $pNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $pList = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 28|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 216|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3WalUndo($8,181,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 212|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3PcacheDirtyList($13)|0);
 HEAP32[$pList>>2] = $14;
 while(1) {
  $15 = HEAP32[$pList>>2]|0;
  $16 = ($15|0)!=(0|0);
  $17 = HEAP32[$rc>>2]|0;
  $18 = ($17|0)==(0);
  $19 = $16 ? $18 : 0;
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$pList>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$pNext>>2] = $22;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$pList>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_pagerUndoCallback($23,$26)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$pNext>>2]|0;
  HEAP32[$pList>>2] = $28;
 }
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _pager_playback_one_page($pPager,$pOffset,$pDone,$isMainJrnl,$isSavepnt) {
 $pPager = $pPager|0;
 $pOffset = $pOffset|0;
 $pDone = $pDone|0;
 $isMainJrnl = $isMainJrnl|0;
 $isSavepnt = $isSavepnt|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $aData = 0, $cksum = 0, $isSynced = 0, $jfd = 0, $ofst = 0, $or$cond = 0, $or$cond4 = 0, $pData = 0, $pPg = 0, $pgno = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $rc = sp + 36|0;
 $pPg = sp + 32|0;
 $pgno = sp + 28|0;
 $cksum = sp + 24|0;
 $aData = sp + 20|0;
 $jfd = sp + 16|0;
 $isSynced = sp + 12|0;
 $ofst = sp;
 $pData = sp + 8|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pOffset;
 HEAP32[$3>>2] = $pDone;
 HEAP32[$4>>2] = $isMainJrnl;
 HEAP32[$5>>2] = $isSavepnt;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 208|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aData>>2] = $8;
 $9 = HEAP32[$4>>2]|0;
 $10 = ($9|0)!=(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = ((($11)) + 68|0);
  $13 = HEAP32[$12>>2]|0;
  $16 = $13;
 } else {
  $14 = ((($11)) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15;
 }
 HEAP32[$jfd>>2] = $16;
 $17 = HEAP32[$jfd>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (_read32bits($17,$21,$24,$pgno)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $318 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($318|0);
 }
 $29 = HEAP32[$jfd>>2]|0;
 $30 = HEAP32[$aData>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 160|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$2>>2]|0;
 $35 = $34;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (_i64Add(($37|0),($40|0),4,0)|0);
 $42 = tempRet0;
 $43 = (_sqlite3OsRead($29,$30,$33,$41,$42)|0);
 HEAP32[$rc>>2] = $43;
 $44 = HEAP32[$rc>>2]|0;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $46;
  $318 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($318|0);
 }
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 160|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 4)|0;
 $51 = HEAP32[$4>>2]|0;
 $52 = $51<<2;
 $53 = (($50) + ($52))|0;
 $54 = ($53|0)<(0);
 $55 = $54 << 31 >> 31;
 $56 = HEAP32[$2>>2]|0;
 $57 = $56;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (_i64Add(($59|0),($62|0),($53|0),($55|0))|0);
 $64 = tempRet0;
 $65 = $56;
 $66 = $65;
 HEAP32[$66>>2] = $63;
 $67 = (($65) + 4)|0;
 $68 = $67;
 HEAP32[$68>>2] = $64;
 $69 = HEAP32[$pgno>>2]|0;
 $70 = ($69|0)==(0);
 if (!($70)) {
  $71 = HEAP32[$pgno>>2]|0;
  $72 = HEAP32[832>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 160|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($72|0) / ($75|0))&-1;
  $77 = (($76) + 1)|0;
  $78 = ($71|0)==($77|0);
  if (!($78)) {
   $79 = HEAP32[$pgno>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 28|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = ($79>>>0)>($82>>>0);
   if (!($83)) {
    $84 = HEAP32[$3>>2]|0;
    $85 = HEAP32[$pgno>>2]|0;
    $86 = (_sqlite3BitvecTest($84,$85)|0);
    $87 = ($86|0)!=(0);
    if (!($87)) {
     $88 = HEAP32[$4>>2]|0;
     $89 = ($88|0)!=(0);
     if ($89) {
      $90 = HEAP32[$jfd>>2]|0;
      $91 = HEAP32[$2>>2]|0;
      $92 = $91;
      $93 = $92;
      $94 = HEAP32[$93>>2]|0;
      $95 = (($92) + 4)|0;
      $96 = $95;
      $97 = HEAP32[$96>>2]|0;
      $98 = (_i64Subtract(($94|0),($97|0),4,0)|0);
      $99 = tempRet0;
      $100 = (_read32bits($90,$98,$99,$cksum)|0);
      HEAP32[$rc>>2] = $100;
      $101 = HEAP32[$rc>>2]|0;
      $102 = ($101|0)!=(0);
      if ($102) {
       $103 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $103;
       $318 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($318|0);
      }
      $104 = HEAP32[$5>>2]|0;
      $105 = ($104|0)!=(0);
      if (!($105)) {
       $106 = HEAP32[$1>>2]|0;
       $107 = HEAP32[$aData>>2]|0;
       $108 = (_pager_cksum($106,$107)|0);
       $109 = HEAP32[$cksum>>2]|0;
       $110 = ($108|0)!=($109|0);
       if ($110) {
        HEAP32[$0>>2] = 101;
        $318 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($318|0);
       }
      }
     }
     $111 = HEAP32[$3>>2]|0;
     $112 = ($111|0)!=(0|0);
     if ($112) {
      $113 = HEAP32[$3>>2]|0;
      $114 = HEAP32[$pgno>>2]|0;
      $115 = (_sqlite3BitvecSet($113,$114)|0);
      HEAP32[$rc>>2] = $115;
      $116 = ($115|0)!=(0);
      if ($116) {
       $117 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $117;
       $318 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($318|0);
      }
     }
     $118 = HEAP32[$pgno>>2]|0;
     $119 = ($118|0)==(1);
     if ($119) {
      $120 = HEAP32[$1>>2]|0;
      $121 = ((($120)) + 150|0);
      $122 = HEAP16[$121>>1]|0;
      $123 = $122 << 16 >> 16;
      $124 = HEAP32[$aData>>2]|0;
      $125 = ((($124)) + 20|0);
      $126 = HEAP8[$125>>0]|0;
      $127 = $126&255;
      $128 = ($123|0)!=($127|0);
      if ($128) {
       $129 = HEAP32[$aData>>2]|0;
       $130 = ((($129)) + 20|0);
       $131 = HEAP8[$130>>0]|0;
       $132 = $131&255;
       $133 = HEAP32[$1>>2]|0;
       $134 = ((($133)) + 150|0);
       HEAP16[$134>>1] = $132;
      }
     }
     $135 = HEAP32[$1>>2]|0;
     $136 = (_pagerUseWal($135)|0);
     $137 = ($136|0)!=(0);
     if ($137) {
      HEAP32[$pPg>>2] = 0;
     } else {
      $138 = HEAP32[$1>>2]|0;
      $139 = HEAP32[$pgno>>2]|0;
      $140 = (_sqlite3PagerLookup($138,$139)|0);
      HEAP32[$pPg>>2] = $140;
     }
     $141 = HEAP32[$4>>2]|0;
     $142 = ($141|0)!=(0);
     if ($142) {
      $143 = HEAP32[$1>>2]|0;
      $144 = ((($143)) + 7|0);
      $145 = HEAP8[$144>>0]|0;
      $146 = $145&255;
      $147 = ($146|0)!=(0);
      if ($147) {
       $169 = 1;
      } else {
       $148 = HEAP32[$2>>2]|0;
       $149 = $148;
       $150 = $149;
       $151 = HEAP32[$150>>2]|0;
       $152 = (($149) + 4)|0;
       $153 = $152;
       $154 = HEAP32[$153>>2]|0;
       $155 = HEAP32[$1>>2]|0;
       $156 = ((($155)) + 88|0);
       $157 = $156;
       $158 = $157;
       $159 = HEAP32[$158>>2]|0;
       $160 = (($157) + 4)|0;
       $161 = $160;
       $162 = HEAP32[$161>>2]|0;
       $163 = ($154|0)<($162|0);
       $164 = ($151>>>0)<=($159>>>0);
       $165 = ($154|0)==($162|0);
       $166 = $165 & $164;
       $167 = $163 | $166;
       $169 = $167;
      }
      $168 = $169&1;
      HEAP32[$isSynced>>2] = $168;
     } else {
      $170 = HEAP32[$pPg>>2]|0;
      $171 = ($170|0)==(0|0);
      if ($171) {
       $179 = 1;
      } else {
       $172 = HEAP32[$pPg>>2]|0;
       $173 = ((($172)) + 24|0);
       $174 = HEAP16[$173>>1]|0;
       $175 = $174&65535;
       $176 = $175 & 8;
       $177 = (0)==($176|0);
       $179 = $177;
      }
      $178 = $179&1;
      HEAP32[$isSynced>>2] = $178;
     }
     $180 = HEAP32[$1>>2]|0;
     $181 = ((($180)) + 64|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = HEAP32[$182>>2]|0;
     $184 = ($183|0)!=(0|0);
     do {
      if ($184) {
       $185 = HEAP32[$1>>2]|0;
       $186 = ((($185)) + 17|0);
       $187 = HEAP8[$186>>0]|0;
       $188 = $187&255;
       $189 = ($188|0)>=(4);
       if ($189) {
        $$old = HEAP32[$isSynced>>2]|0;
        $$old1 = ($$old|0)!=(0);
        if (!($$old1)) {
         label = 44;
         break;
        }
       } else {
        $190 = HEAP32[$1>>2]|0;
        $191 = ((($190)) + 17|0);
        $192 = HEAP8[$191>>0]|0;
        $193 = $192&255;
        $194 = ($193|0)==(0);
        $195 = HEAP32[$isSynced>>2]|0;
        $196 = ($195|0)!=(0);
        $or$cond = $194 & $196;
        if (!($or$cond)) {
         label = 44;
         break;
        }
       }
       $197 = HEAP32[$pgno>>2]|0;
       $198 = (($197) - 1)|0;
       $199 = HEAP32[$1>>2]|0;
       $200 = ((($199)) + 160|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)<(0);
       $203 = $202 << 31 >> 31;
       $204 = (___muldi3(($198|0),0,($201|0),($203|0))|0);
       $205 = tempRet0;
       $206 = $ofst;
       $207 = $206;
       HEAP32[$207>>2] = $204;
       $208 = (($206) + 4)|0;
       $209 = $208;
       HEAP32[$209>>2] = $205;
       $210 = HEAP32[$1>>2]|0;
       $211 = ((($210)) + 64|0);
       $212 = HEAP32[$211>>2]|0;
       $213 = HEAP32[$aData>>2]|0;
       $214 = HEAP32[$1>>2]|0;
       $215 = ((($214)) + 160|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $ofst;
       $218 = $217;
       $219 = HEAP32[$218>>2]|0;
       $220 = (($217) + 4)|0;
       $221 = $220;
       $222 = HEAP32[$221>>2]|0;
       $223 = (_sqlite3OsWrite($212,$213,$216,$219,$222)|0);
       HEAP32[$rc>>2] = $223;
       $224 = HEAP32[$pgno>>2]|0;
       $225 = HEAP32[$1>>2]|0;
       $226 = ((($225)) + 36|0);
       $227 = HEAP32[$226>>2]|0;
       $228 = ($224>>>0)>($227>>>0);
       if ($228) {
        $229 = HEAP32[$pgno>>2]|0;
        $230 = HEAP32[$1>>2]|0;
        $231 = ((($230)) + 36|0);
        HEAP32[$231>>2] = $229;
       }
       $232 = HEAP32[$1>>2]|0;
       $233 = ((($232)) + 96|0);
       $234 = HEAP32[$233>>2]|0;
       $235 = ($234|0)!=(0|0);
       if ($235) {
        $236 = HEAP32[$1>>2]|0;
        $237 = ((($236)) + 96|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = HEAP32[$pgno>>2]|0;
        $240 = HEAP32[$aData>>2]|0;
        _sqlite3BackupUpdate($238,$239,$240);
        $241 = HEAP32[$aData>>2]|0;
        HEAP32[$aData>>2] = $241;
       }
      } else {
       label = 44;
      }
     } while(0);
     do {
      if ((label|0) == 44) {
       $242 = HEAP32[$4>>2]|0;
       $243 = ($242|0)==(0);
       $244 = HEAP32[$pPg>>2]|0;
       $245 = ($244|0)==(0|0);
       $or$cond4 = $243 & $245;
       if ($or$cond4) {
        $246 = HEAP32[$1>>2]|0;
        $247 = ((($246)) + 21|0);
        $248 = HEAP8[$247>>0]|0;
        $249 = $248&255;
        $250 = $249 | 2;
        $251 = $250&255;
        HEAP8[$247>>0] = $251;
        $252 = HEAP32[$1>>2]|0;
        $253 = HEAP32[$pgno>>2]|0;
        $254 = (_sqlite3PagerGet($252,$253,$pPg,1)|0);
        HEAP32[$rc>>2] = $254;
        $255 = HEAP32[$1>>2]|0;
        $256 = ((($255)) + 21|0);
        $257 = HEAP8[$256>>0]|0;
        $258 = $257&255;
        $259 = $258 & -3;
        $260 = $259&255;
        HEAP8[$256>>0] = $260;
        $261 = HEAP32[$rc>>2]|0;
        $262 = ($261|0)!=(0);
        if (!($262)) {
         $264 = HEAP32[$pPg>>2]|0;
         $265 = ((($264)) + 24|0);
         $266 = HEAP16[$265>>1]|0;
         $267 = $266&65535;
         $268 = $267 & -17;
         $269 = $268&65535;
         HEAP16[$265>>1] = $269;
         $270 = HEAP32[$pPg>>2]|0;
         _sqlite3PcacheMakeDirty($270);
         break;
        }
        $263 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $263;
        $318 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($318|0);
       }
      }
     } while(0);
     $271 = HEAP32[$pPg>>2]|0;
     $272 = ($271|0)!=(0|0);
     if ($272) {
      $273 = HEAP32[$pPg>>2]|0;
      $274 = ((($273)) + 4|0);
      $275 = HEAP32[$274>>2]|0;
      HEAP32[$pData>>2] = $275;
      $276 = HEAP32[$pData>>2]|0;
      $277 = HEAP32[$aData>>2]|0;
      $278 = HEAP32[$1>>2]|0;
      $279 = ((($278)) + 160|0);
      $280 = HEAP32[$279>>2]|0;
      _memcpy(($276|0),($277|0),($280|0))|0;
      $281 = HEAP32[$1>>2]|0;
      $282 = ((($281)) + 204|0);
      $283 = HEAP32[$282>>2]|0;
      $284 = HEAP32[$pPg>>2]|0;
      FUNCTION_TABLE_vi[$283 & 255]($284);
      $285 = HEAP32[$4>>2]|0;
      $286 = ($285|0)!=(0);
      do {
       if ($286) {
        $287 = HEAP32[$5>>2]|0;
        $288 = ($287|0)!=(0);
        if ($288) {
         $289 = HEAP32[$2>>2]|0;
         $290 = $289;
         $291 = $290;
         $292 = HEAP32[$291>>2]|0;
         $293 = (($290) + 4)|0;
         $294 = $293;
         $295 = HEAP32[$294>>2]|0;
         $296 = HEAP32[$1>>2]|0;
         $297 = ((($296)) + 88|0);
         $298 = $297;
         $299 = $298;
         $300 = HEAP32[$299>>2]|0;
         $301 = (($298) + 4)|0;
         $302 = $301;
         $303 = HEAP32[$302>>2]|0;
         $304 = ($295|0)<($303|0);
         $305 = ($292>>>0)<=($300>>>0);
         $306 = ($295|0)==($303|0);
         $307 = $306 & $305;
         $308 = $304 | $307;
         if (!($308)) {
          break;
         }
        }
        $309 = HEAP32[$pPg>>2]|0;
        _sqlite3PcacheMakeClean($309);
       }
      } while(0);
      $310 = HEAP32[$pgno>>2]|0;
      $311 = ($310|0)==(1);
      if ($311) {
       $312 = HEAP32[$1>>2]|0;
       $313 = ((($312)) + 112|0);
       $314 = HEAP32[$pData>>2]|0;
       $315 = ((($314)) + 24|0);
       dest=$313; src=$315; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
      }
      $316 = HEAP32[$pPg>>2]|0;
      _sqlite3PcacheRelease($316);
     }
     $317 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $317;
     $318 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($318|0);
    }
   }
   HEAP32[$0>>2] = 0;
   $318 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($318|0);
  }
 }
 HEAP32[$0>>2] = 101;
 $318 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($318|0);
}
function _readJournalHdr($pPager,$isHot,$0,$1,$pNRec,$pDbSize) {
 $pPager = $pPager|0;
 $isHot = $isHot|0;
 $0 = $0|0;
 $1 = $1|0;
 $pNRec = $pNRec|0;
 $pDbSize = $pDbSize|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMagic = 0, $iHdrOff = 0;
 var $iPageSize = 0, $iSectorSize = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 8|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $rc = sp + 24|0;
 $aMagic = sp + 48|0;
 $iHdrOff = sp;
 $iPageSize = sp + 20|0;
 $iSectorSize = sp + 16|0;
 HEAP32[$3>>2] = $pPager;
 HEAP32[$4>>2] = $isHot;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $pNRec;
 HEAP32[$7>>2] = $pDbSize;
 $12 = HEAP32[$3>>2]|0;
 $13 = (_journalHdrOffset($12)|0);
 $14 = tempRet0;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 80|0);
 $17 = $16;
 $18 = $17;
 HEAP32[$18>>2] = $13;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $14;
 $21 = HEAP32[$3>>2]|0;
 $22 = ((($21)) + 80|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = ((($29)) + 156|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_i64Add(($25|0),($28|0),($31|0),0)|0);
 $33 = tempRet0;
 $34 = $5;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = ($33|0)>($39|0);
 $41 = ($32>>>0)>($36>>>0);
 $42 = ($33|0)==($39|0);
 $43 = $42 & $41;
 $44 = $40 | $43;
 if ($44) {
  HEAP32[$2>>2] = 101;
  $219 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($219|0);
 }
 $45 = HEAP32[$3>>2]|0;
 $46 = ((($45)) + 80|0);
 $47 = $46;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) + 4)|0;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = $iHdrOff;
 $54 = $53;
 HEAP32[$54>>2] = $49;
 $55 = (($53) + 4)|0;
 $56 = $55;
 HEAP32[$56>>2] = $52;
 $57 = HEAP32[$4>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  label = 5;
 } else {
  $59 = $iHdrOff;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) + 4)|0;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$3>>2]|0;
  $66 = ((($65)) + 88|0);
  $67 = $66;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($61|0)!=($69|0);
  $74 = ($64|0)!=($72|0);
  $75 = $73 | $74;
  if ($75) {
   label = 5;
  }
 }
 if ((label|0) == 5) {
  $76 = HEAP32[$3>>2]|0;
  $77 = ((($76)) + 68|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = $iHdrOff;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (($79) + 4)|0;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3OsRead($78,$aMagic,8,$81,$84)|0);
  HEAP32[$rc>>2] = $85;
  $86 = HEAP32[$rc>>2]|0;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $88;
   $219 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($219|0);
  }
  $89 = (_memcmp($aMagic,19004,8)|0);
  $90 = ($89|0)!=(0);
  if ($90) {
   HEAP32[$2>>2] = 101;
   $219 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($219|0);
  }
 }
 $91 = HEAP32[$3>>2]|0;
 $92 = ((($91)) + 68|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = $iHdrOff;
 $95 = $94;
 $96 = HEAP32[$95>>2]|0;
 $97 = (($94) + 4)|0;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (_i64Add(($96|0),($99|0),8,0)|0);
 $101 = tempRet0;
 $102 = HEAP32[$6>>2]|0;
 $103 = (_read32bits($93,$100,$101,$102)|0);
 HEAP32[$rc>>2] = $103;
 $104 = (0)!=($103|0);
 if (!($104)) {
  $105 = HEAP32[$3>>2]|0;
  $106 = ((($105)) + 68|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $iHdrOff;
  $109 = $108;
  $110 = HEAP32[$109>>2]|0;
  $111 = (($108) + 4)|0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (_i64Add(($110|0),($113|0),12,0)|0);
  $115 = tempRet0;
  $116 = HEAP32[$3>>2]|0;
  $117 = ((($116)) + 52|0);
  $118 = (_read32bits($107,$114,$115,$117)|0);
  HEAP32[$rc>>2] = $118;
  $119 = (0)!=($118|0);
  if (!($119)) {
   $120 = HEAP32[$3>>2]|0;
   $121 = ((($120)) + 68|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = $iHdrOff;
   $124 = $123;
   $125 = HEAP32[$124>>2]|0;
   $126 = (($123) + 4)|0;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = (_i64Add(($125|0),($128|0),16,0)|0);
   $130 = tempRet0;
   $131 = HEAP32[$7>>2]|0;
   $132 = (_read32bits($122,$129,$130,$131)|0);
   HEAP32[$rc>>2] = $132;
   $133 = (0)!=($132|0);
   if (!($133)) {
    $135 = HEAP32[$3>>2]|0;
    $136 = ((($135)) + 80|0);
    $137 = $136;
    $138 = $137;
    $139 = HEAP32[$138>>2]|0;
    $140 = (($137) + 4)|0;
    $141 = $140;
    $142 = HEAP32[$141>>2]|0;
    $143 = ($139|0)==(0);
    $144 = ($142|0)==(0);
    $145 = $143 & $144;
    do {
     if ($145) {
      $146 = HEAP32[$3>>2]|0;
      $147 = ((($146)) + 68|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = $iHdrOff;
      $150 = $149;
      $151 = HEAP32[$150>>2]|0;
      $152 = (($149) + 4)|0;
      $153 = $152;
      $154 = HEAP32[$153>>2]|0;
      $155 = (_i64Add(($151|0),($154|0),20,0)|0);
      $156 = tempRet0;
      $157 = (_read32bits($148,$155,$156,$iSectorSize)|0);
      HEAP32[$rc>>2] = $157;
      $158 = (0)!=($157|0);
      if (!($158)) {
       $159 = HEAP32[$3>>2]|0;
       $160 = ((($159)) + 68|0);
       $161 = HEAP32[$160>>2]|0;
       $162 = $iHdrOff;
       $163 = $162;
       $164 = HEAP32[$163>>2]|0;
       $165 = (($162) + 4)|0;
       $166 = $165;
       $167 = HEAP32[$166>>2]|0;
       $168 = (_i64Add(($164|0),($167|0),24,0)|0);
       $169 = tempRet0;
       $170 = (_read32bits($161,$168,$169,$iPageSize)|0);
       HEAP32[$rc>>2] = $170;
       $171 = (0)!=($170|0);
       if (!($171)) {
        $173 = HEAP32[$iPageSize>>2]|0;
        $174 = ($173|0)==(0);
        if ($174) {
         $175 = HEAP32[$3>>2]|0;
         $176 = ((($175)) + 160|0);
         $177 = HEAP32[$176>>2]|0;
         HEAP32[$iPageSize>>2] = $177;
        }
        $178 = HEAP32[$iPageSize>>2]|0;
        $179 = ($178>>>0)<(512);
        $180 = HEAP32[$iSectorSize>>2]|0;
        $181 = ($180>>>0)<(32);
        $or$cond = $179 | $181;
        $182 = HEAP32[$iPageSize>>2]|0;
        $183 = ($182>>>0)>(65536);
        $or$cond3 = $or$cond | $183;
        $184 = HEAP32[$iSectorSize>>2]|0;
        $185 = ($184>>>0)>(65536);
        $or$cond5 = $or$cond3 | $185;
        if (!($or$cond5)) {
         $186 = HEAP32[$iPageSize>>2]|0;
         $187 = (($186) - 1)|0;
         $188 = HEAP32[$iPageSize>>2]|0;
         $189 = $187 & $188;
         $190 = ($189|0)!=(0);
         if (!($190)) {
          $191 = HEAP32[$iSectorSize>>2]|0;
          $192 = (($191) - 1)|0;
          $193 = HEAP32[$iSectorSize>>2]|0;
          $194 = $192 & $193;
          $195 = ($194|0)!=(0);
          if (!($195)) {
           $196 = HEAP32[$3>>2]|0;
           $197 = (_sqlite3PagerSetPagesize($196,$iPageSize,-1)|0);
           HEAP32[$rc>>2] = $197;
           $198 = HEAP32[$iSectorSize>>2]|0;
           $199 = HEAP32[$3>>2]|0;
           $200 = ((($199)) + 156|0);
           HEAP32[$200>>2] = $198;
           break;
          }
         }
        }
        HEAP32[$2>>2] = 101;
        $219 = HEAP32[$2>>2]|0;
        STACKTOP = sp;return ($219|0);
       }
      }
      $172 = HEAP32[$rc>>2]|0;
      HEAP32[$2>>2] = $172;
      $219 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($219|0);
     }
    } while(0);
    $201 = HEAP32[$3>>2]|0;
    $202 = ((($201)) + 156|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = HEAP32[$3>>2]|0;
    $205 = ((($204)) + 80|0);
    $206 = $205;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = (($206) + 4)|0;
    $210 = $209;
    $211 = HEAP32[$210>>2]|0;
    $212 = (_i64Add(($208|0),($211|0),($203|0),0)|0);
    $213 = tempRet0;
    $214 = $205;
    $215 = $214;
    HEAP32[$215>>2] = $212;
    $216 = (($214) + 4)|0;
    $217 = $216;
    HEAP32[$217>>2] = $213;
    $218 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $218;
    $219 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($219|0);
   }
  }
 }
 $134 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $134;
 $219 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($219|0);
}
function _sqlite3WalSavepointUndo($pWal,$aWalData) {
 $pWal = $pWal|0;
 $aWalData = $aWalData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $aWalData;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 112|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($4|0)!=($7|0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  HEAP32[$9>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 112|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 12|0);
  HEAP32[$14>>2] = $12;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($16>>>0)<($20>>>0);
 if (!($21)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 52|0);
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $23;
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 52|0);
 $32 = ((($31)) + 24|0);
 HEAP32[$32>>2] = $29;
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 52|0);
 $38 = ((($37)) + 24|0);
 $39 = ((($38)) + 4|0);
 HEAP32[$39>>2] = $35;
 $40 = HEAP32[$0>>2]|0;
 _walCleanupHash($40);
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3WalUndo($pWal,$xUndo,$pUndoCtx) {
 $pWal = $pWal|0;
 $xUndo = $xUndo|0;
 $pUndoCtx = $pUndoCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iFrame = 0, $iMax = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $iMax = sp + 4|0;
 $iFrame = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $xUndo;
 HEAP32[$2>>2] = $pUndoCtx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$iMax>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 52|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = (_walIndexHdr($13)|0);
 dest=$12; src=$14; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 52|0);
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 1)|0;
 HEAP32[$iFrame>>2] = $19;
 while(1) {
  $20 = HEAP32[$rc>>2]|0;
  $21 = ($20|0)==(0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$iFrame>>2]|0;
  $23 = HEAP32[$iMax>>2]|0;
  $24 = ($22>>>0)<=($23>>>0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$iFrame>>2]|0;
  $29 = (_walFramePgno($27,$28)|0);
  $30 = (FUNCTION_TABLE_iii[$25 & 255]($26,$29)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$iFrame>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$iFrame>>2] = $32;
 }
 $33 = HEAP32[$iMax>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 52|0);
 $36 = ((($35)) + 16|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ($33|0)!=($37|0);
 if (!($38)) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $39 = HEAP32[$0>>2]|0;
 _walCleanupHash($39);
 $40 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($40|0);
}
function _pagerUndoCallback($pCtx,$iPg) {
 $pCtx = $pCtx|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iFrame = 0, $pPager = 0, $pPg = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pPager = sp + 8|0;
 $pPg = sp + 4|0;
 $iFrame = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $iPg;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pPager>>2] = $2;
 $3 = HEAP32[$pPager>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3PagerLookup($3,$4)|0);
 HEAP32[$pPg>>2] = $5;
 $6 = HEAP32[$pPg>>2]|0;
 $7 = ($6|0)!=(0|0);
 do {
  if ($7) {
   $8 = HEAP32[$pPg>>2]|0;
   $9 = (_sqlite3PcachePageRefcount($8)|0);
   $10 = ($9|0)==(1);
   if ($10) {
    $11 = HEAP32[$pPg>>2]|0;
    _sqlite3PcacheDrop($11);
    break;
   }
   HEAP32[$iFrame>>2] = 0;
   $12 = HEAP32[$pPager>>2]|0;
   $13 = ((($12)) + 216|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$pPg>>2]|0;
   $16 = ((($15)) + 20|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3WalFindFrame($14,$17,$iFrame)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)==(0);
   if ($20) {
    $21 = HEAP32[$pPg>>2]|0;
    $22 = HEAP32[$iFrame>>2]|0;
    $23 = (_readDbPage($21,$22)|0);
    HEAP32[$rc>>2] = $23;
   }
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   if ($25) {
    $26 = HEAP32[$pPager>>2]|0;
    $27 = ((($26)) + 204|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$pPg>>2]|0;
    FUNCTION_TABLE_vi[$28 & 255]($29);
   }
   $30 = HEAP32[$pPg>>2]|0;
   _sqlite3PagerUnrefNotNull($30);
  }
 } while(0);
 $31 = HEAP32[$pPager>>2]|0;
 $32 = ((($31)) + 96|0);
 $33 = HEAP32[$32>>2]|0;
 _sqlite3BackupRestart($33);
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _sqlite3PcacheDirtyList($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 32|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$p>>2]|0;
  $9 = ((($8)) + 12|0);
  HEAP32[$9>>2] = $7;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_pcacheSortDirtyList($14)|0);
 STACKTOP = sp;return ($15|0);
}
function _walFramePgno($pWal,$iFrame) {
 $pWal = $pWal|0;
 $iFrame = $iFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $iHash = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iFrame;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_walFramePage($3)|0);
 HEAP32[$iHash>>2] = $4;
 $5 = HEAP32[$iHash>>2]|0;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$2>>2]|0;
 if ($6) {
  $8 = (34 + ($7))|0;
  $9 = (($8) - 1)|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($9<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$0>>2] = $15;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $16 = (($7) - 1)|0;
  $17 = (($16) - 4062)|0;
  $18 = (($17>>>0) % 4096)&-1;
  $19 = HEAP32[$iHash>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($18<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3PagerLookup($pPager,$pgno) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 212|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3PcacheFetch($5,$6,0)|0);
 HEAP32[$pPage>>2] = $7;
 $8 = HEAP32[$pPage>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 212|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$pPage>>2]|0;
  $15 = (_sqlite3PcacheFetchFinish($12,$13,$14)|0);
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _sqlite3PcacheDrop($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  _pcacheManageDirtyList($7,1);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + -1)|0;
 HEAP32[$11>>2] = $13;
 $14 = HEAP32[(144)>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 28|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 FUNCTION_TABLE_viii[$14 & 255]($19,$21,1);
 STACKTOP = sp;return;
}
function _readDbPage($pPg,$iFrame) {
 $pPg = $pPg|0;
 $iFrame = $iFrame|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $dbFileVers = 0, $iOffset = 0, $pPager = 0, $pgno = 0, $pgsz = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pPager = sp + 24|0;
 $pgno = sp + 20|0;
 $rc = sp + 16|0;
 $pgsz = sp + 12|0;
 $iOffset = sp;
 $dbFileVers = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$1>>2] = $iFrame;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgno>>2] = $7;
 HEAP32[$rc>>2] = 0;
 $8 = HEAP32[$pPager>>2]|0;
 $9 = ((($8)) + 160|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pgsz>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$pPager>>2]|0;
  $14 = ((($13)) + 216|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pgsz>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3WalReadFrame($15,$16,$17,$20)|0);
  HEAP32[$rc>>2] = $21;
 } else {
  $22 = HEAP32[$pgno>>2]|0;
  $23 = (($22) - 1)|0;
  $24 = HEAP32[$pPager>>2]|0;
  $25 = ((($24)) + 160|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = (___muldi3(($23|0),0,($26|0),($28|0))|0);
  $30 = tempRet0;
  $31 = $iOffset;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
  $35 = HEAP32[$pPager>>2]|0;
  $36 = ((($35)) + 64|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pgsz>>2]|0;
  $42 = $iOffset;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (_sqlite3OsRead($37,$40,$41,$44,$47)|0);
  HEAP32[$rc>>2] = $48;
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)==(522);
  $$ = $50 ? 0 : $48;
  HEAP32[$rc>>2] = $$;
 }
 $51 = HEAP32[$pgno>>2]|0;
 $52 = ($51|0)==(1);
 if (!($52)) {
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $53 = HEAP32[$rc>>2]|0;
 $54 = ($53|0)!=(0);
 if ($54) {
  $55 = HEAP32[$pPager>>2]|0;
  $56 = ((($55)) + 112|0);
  dest=$56; stop=dest+16|0; do { HEAP8[dest>>0]=255|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  $57 = HEAP32[$0>>2]|0;
  $58 = ((($57)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ((($59)) + 24|0);
  HEAP32[$dbFileVers>>2] = $60;
  $61 = HEAP32[$pPager>>2]|0;
  $62 = ((($61)) + 112|0);
  $63 = HEAP32[$dbFileVers>>2]|0;
  dest=$62; src=$63; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _sqlite3PagerUnrefNotNull($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 64;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  _pagerReleaseMapPage($10);
  $11 = HEAP32[$pPager>>2]|0;
  _pagerUnlockIfUnused($11);
  STACKTOP = sp;return;
 } else {
  _sqlite3PcacheRelease($10);
  $11 = HEAP32[$pPager>>2]|0;
  _pagerUnlockIfUnused($11);
  STACKTOP = sp;return;
 }
}
function _sqlite3PcacheFetch($pCache,$pgno,$createFlag) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eCreate = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $eCreate = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $createFlag;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 33|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $3 & $7;
 HEAP32[$eCreate>>2] = $8;
 $9 = HEAP32[(140)>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 44|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$eCreate>>2]|0;
 $15 = (FUNCTION_TABLE_iiii[$9 & 255]($12,$13,$14)|0);
 STACKTOP = sp;return ($15|0);
}
function _sqlite3PcacheFetchFinish($pCache,$pgno,$pPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $pPage;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pPgHdr>>2] = $6;
 $7 = HEAP32[$pPgHdr>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$1>>2]|0;
 if ($9) {
  $14 = ((($10)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$14>>2] = $16;
  $17 = HEAP32[$pPgHdr>>2]|0;
  $18 = ((($17)) + 26|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = (($19) + 1)<<16>>16;
  HEAP16[$18>>1] = $20;
  $21 = HEAP32[$pPgHdr>>2]|0;
  HEAP32[$0>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = (_pcacheFetchFinishWithInit($10,$11,$12)|0);
  HEAP32[$0>>2] = $13;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _pcacheFetchFinishWithInit($pCache,$pgno,$pPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $pPage;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPgHdr>>2] = $5;
 $6 = HEAP32[$pPgHdr>>2]|0;
 dest=$6; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$pPgHdr>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$pPgHdr>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$pPgHdr>>2]|0;
 $14 = ((($13)) + 40|0);
 $15 = HEAP32[$pPgHdr>>2]|0;
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$pPgHdr>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 28|0);
 $22 = HEAP32[$21>>2]|0;
 _memset(($19|0),0,($22|0))|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$pPgHdr>>2]|0;
 $25 = ((($24)) + 28|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$1>>2]|0;
 $27 = HEAP32[$pPgHdr>>2]|0;
 $28 = ((($27)) + 20|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pPgHdr>>2]|0;
 $30 = ((($29)) + 24|0);
 HEAP16[$30>>1] = 1;
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = (_sqlite3PcacheFetchFinish($31,$32,$33)|0);
 STACKTOP = sp;return ($34|0);
}
function _sqlite3WalReadFrame($pWal,$iRead,$nOut,$pOut) {
 $pWal = $pWal|0;
 $iRead = $iRead|0;
 $nOut = $nOut|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iOffset = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $sz = sp + 8|0;
 $iOffset = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iRead;
 HEAP32[$2>>2] = $nOut;
 HEAP32[$3>>2] = $pOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = ((($5)) + 14|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 HEAP32[$sz>>2] = $8;
 $9 = HEAP32[$sz>>2]|0;
 $10 = $9 & 65024;
 $11 = HEAP32[$sz>>2]|0;
 $12 = $11 & 1;
 $13 = $12 << 16;
 $14 = (($10) + ($13))|0;
 HEAP32[$sz>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$sz>>2]|0;
 $18 = (($17) + 24)|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = (___muldi3(($16|0),0,($18|0),($20|0))|0);
 $22 = tempRet0;
 $23 = (_i64Add(32,0,($21|0),($22|0))|0);
 $24 = tempRet0;
 $25 = (_i64Add(($23|0),($24|0),24,0)|0);
 $26 = tempRet0;
 $27 = $iOffset;
 $28 = $27;
 HEAP32[$28>>2] = $25;
 $29 = (($27) + 4)|0;
 $30 = $29;
 HEAP32[$30>>2] = $26;
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$sz>>2]|0;
 $37 = ($35|0)>($36|0);
 $38 = HEAP32[$sz>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = $37 ? $38 : $39;
 $41 = $iOffset;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3OsRead($33,$34,$40,$43,$46)|0);
 STACKTOP = sp;return ($47|0);
}
function _pagerReleaseMapPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 128|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + -1)|0;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$pPager>>2]|0;
 $9 = ((($8)) + 144|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pPager>>2]|0;
 $15 = ((($14)) + 144|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$pPager>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = HEAP32[$pPager>>2]|0;
 $24 = ((($23)) + 160|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)<(0);
 $27 = $26 << 31 >> 31;
 $28 = (___muldi3(($22|0),0,($25|0),($27|0))|0);
 $29 = tempRet0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3OsUnfetch($18,$28,$29,$32)|0);
 STACKTOP = sp;return;
}
function _sqlite3PcacheRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 26|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = (($9) + -1)<<16>>16;
 HEAP16[$8>>1] = $10;
 $11 = $10 << 16 >> 16;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 1;
 $18 = ($17|0)!=(0);
 $19 = HEAP32[$0>>2]|0;
 if ($18) {
  _pcacheUnpin($19);
  STACKTOP = sp;return;
 }
 $20 = ((($19)) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($23,3);
 STACKTOP = sp;return;
}
function _pagerUnlockIfUnused($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 128|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 212|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PcacheRefCount($7)|0);
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 _pagerUnlockAndRollback($10);
 STACKTOP = sp;return;
}
function _pcacheSortDirtyList($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $i = 0, $p = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $a = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pIn;
 dest=$a; stop=dest+128|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if (!($2)) {
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  HEAP32[$p>>2] = $3;
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 12|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$p>>2]|0;
  $8 = ((($7)) + 12|0);
  HEAP32[$8>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $9 = HEAP32[$i>>2]|0;
   $10 = ($9|0)<(31);
   if (!($10)) {
    break;
   }
   $11 = HEAP32[$i>>2]|0;
   $12 = (($a) + ($11<<2)|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if ($14) {
    label = 6;
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = (($a) + ($18<<2)|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$p>>2]|0;
   $22 = (_pcacheMergeDirtyList($20,$21)|0);
   HEAP32[$p>>2] = $22;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($a) + ($23<<2)|0);
   HEAP32[$24>>2] = 0;
   $25 = HEAP32[$i>>2]|0;
   $26 = (($25) + 1)|0;
   HEAP32[$i>>2] = $26;
  }
  if ((label|0) == 6) {
   label = 0;
   $15 = HEAP32[$p>>2]|0;
   $16 = HEAP32[$i>>2]|0;
   $17 = (($a) + ($16<<2)|0);
   HEAP32[$17>>2] = $15;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = ($27|0)==(31);
  if (!($28)) {
   continue;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($a) + ($29<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$p>>2]|0;
  $33 = (_pcacheMergeDirtyList($31,$32)|0);
  $34 = HEAP32[$i>>2]|0;
  $35 = (($a) + ($34<<2)|0);
  HEAP32[$35>>2] = $33;
 }
 $36 = HEAP32[$a>>2]|0;
 HEAP32[$p>>2] = $36;
 HEAP32[$i>>2] = 1;
 while(1) {
  $37 = HEAP32[$i>>2]|0;
  $38 = ($37|0)<(32);
  $39 = HEAP32[$p>>2]|0;
  if (!($38)) {
   break;
  }
  $40 = HEAP32[$i>>2]|0;
  $41 = (($a) + ($40<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_pcacheMergeDirtyList($39,$42)|0);
  HEAP32[$p>>2] = $43;
  $44 = HEAP32[$i>>2]|0;
  $45 = (($44) + 1)|0;
  HEAP32[$i>>2] = $45;
 }
 STACKTOP = sp;return ($39|0);
}
function _pcacheMergeDirtyList($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pTail = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $result = sp + 8|0;
 $pTail = sp;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 HEAP32[$pTail>>2] = $result;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  $4 = HEAP32[$1>>2]|0;
  $5 = ($4|0)!=(0|0);
  $6 = $3 ? $5 : 0;
  $7 = HEAP32[$0>>2]|0;
  if (!($6)) {
   break;
  }
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9>>>0)<($12>>>0);
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$pTail>>2]|0;
   $16 = ((($15)) + 12|0);
   HEAP32[$16>>2] = $14;
   $17 = HEAP32[$0>>2]|0;
   HEAP32[$pTail>>2] = $17;
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$0>>2] = $20;
   continue;
  } else {
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$pTail>>2]|0;
   $23 = ((($22)) + 12|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$1>>2]|0;
   HEAP32[$pTail>>2] = $24;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 12|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$1>>2] = $27;
   continue;
  }
 }
 $28 = ($7|0)!=(0|0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pTail>>2]|0;
  $31 = ((($30)) + 12|0);
  HEAP32[$31>>2] = $29;
  $39 = ((($result)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ($32|0)!=(0|0);
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$pTail>>2]|0;
  $36 = ((($35)) + 12|0);
  HEAP32[$36>>2] = $34;
  $39 = ((($result)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  STACKTOP = sp;return ($40|0);
 } else {
  $37 = HEAP32[$pTail>>2]|0;
  $38 = ((($37)) + 12|0);
  HEAP32[$38>>2] = 0;
  $39 = ((($result)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 return (0)|0;
}
function _read32bits($fd,$0,$1,$pRes) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $pRes = $pRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ac = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $ac = sp + 20|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pRes;
 $9 = HEAP32[$2>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3OsRead($9,$ac,4,$12,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $19 = (_sqlite3Get4byte($ac)|0);
 $20 = HEAP32[$4>>2]|0;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3BitvecTest($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3BitvecTestNotNull($4,$5)|0);
 $7 = ($6|0)!=(0);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _pager_cksum($pPager,$aData) {
 $pPager = $pPager|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $cksum = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $cksum = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $aData;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$cksum>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 160|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 200)|0;
 HEAP32[$i>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)>(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$cksum>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$cksum>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) - 200)|0;
  HEAP32[$i>>2] = $19;
 }
 $20 = HEAP32[$cksum>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3PcacheMakeDirty($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 33;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10 & -33;
 $12 = $11&65535;
 HEAP16[$8>>1] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 1;
 $18 = ($17|0)!=(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 24|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $23 = $22 ^ 3;
 $24 = $23&65535;
 HEAP16[$20>>1] = $24;
 $25 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($25,2);
 STACKTOP = sp;return;
}
function _readMasterJournal($pJrnl,$zMaster,$nMaster) {
 $pJrnl = $pJrnl|0;
 $zMaster = $zMaster|0;
 $nMaster = $nMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aMagic = 0, $cksum = 0, $len = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0;
 var $szJ = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $rc = sp + 20|0;
 $len = sp + 16|0;
 $szJ = sp;
 $cksum = sp + 12|0;
 $u = sp + 8|0;
 $aMagic = sp + 40|0;
 HEAP32[$1>>2] = $pJrnl;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$3>>2] = $nMaster;
 $4 = HEAP32[$2>>2]|0;
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3OsFileSize($5,$szJ)|0);
 HEAP32[$rc>>2] = $6;
 $7 = (0)!=($6|0);
 $8 = $szJ;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)<(0);
 $15 = ($10>>>0)<(16);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 $or$cond = $7 | $18;
 if (!($or$cond)) {
  $19 = HEAP32[$1>>2]|0;
  $20 = $szJ;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (_i64Subtract(($22|0),($25|0),16,0)|0);
  $27 = tempRet0;
  $28 = (_read32bits($19,$26,$27,$len)|0);
  HEAP32[$rc>>2] = $28;
  $29 = (0)!=($28|0);
  if (!($29)) {
   $30 = HEAP32[$len>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = ($30>>>0)>=($31>>>0);
   $33 = HEAP32[$len>>2]|0;
   $34 = ($33|0)==(0);
   $or$cond3 = $32 | $34;
   if (!($or$cond3)) {
    $35 = HEAP32[$1>>2]|0;
    $36 = $szJ;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = (($36) + 4)|0;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (_i64Subtract(($38|0),($41|0),12,0)|0);
    $43 = tempRet0;
    $44 = (_read32bits($35,$42,$43,$cksum)|0);
    HEAP32[$rc>>2] = $44;
    $45 = (0)!=($44|0);
    if (!($45)) {
     $46 = HEAP32[$1>>2]|0;
     $47 = $szJ;
     $48 = $47;
     $49 = HEAP32[$48>>2]|0;
     $50 = (($47) + 4)|0;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (_i64Subtract(($49|0),($52|0),8,0)|0);
     $54 = tempRet0;
     $55 = (_sqlite3OsRead($46,$aMagic,8,$53,$54)|0);
     HEAP32[$rc>>2] = $55;
     $56 = (0)!=($55|0);
     if (!($56)) {
      $57 = (_memcmp($aMagic,19004,8)|0);
      $58 = ($57|0)!=(0);
      if (!($58)) {
       $59 = HEAP32[$1>>2]|0;
       $60 = HEAP32[$2>>2]|0;
       $61 = HEAP32[$len>>2]|0;
       $62 = $szJ;
       $63 = $62;
       $64 = HEAP32[$63>>2]|0;
       $65 = (($62) + 4)|0;
       $66 = $65;
       $67 = HEAP32[$66>>2]|0;
       $68 = (_i64Subtract(($64|0),($67|0),16,0)|0);
       $69 = tempRet0;
       $70 = HEAP32[$len>>2]|0;
       $71 = (_i64Subtract(($68|0),($69|0),($70|0),0)|0);
       $72 = tempRet0;
       $73 = (_sqlite3OsRead($59,$60,$61,$71,$72)|0);
       HEAP32[$rc>>2] = $73;
       $74 = (0)!=($73|0);
       if (!($74)) {
        HEAP32[$u>>2] = 0;
        while(1) {
         $76 = HEAP32[$u>>2]|0;
         $77 = HEAP32[$len>>2]|0;
         $78 = ($76>>>0)<($77>>>0);
         if (!($78)) {
          break;
         }
         $79 = HEAP32[$u>>2]|0;
         $80 = HEAP32[$2>>2]|0;
         $81 = (($80) + ($79)|0);
         $82 = HEAP8[$81>>0]|0;
         $83 = $82 << 24 >> 24;
         $84 = HEAP32[$cksum>>2]|0;
         $85 = (($84) - ($83))|0;
         HEAP32[$cksum>>2] = $85;
         $86 = HEAP32[$u>>2]|0;
         $87 = (($86) + 1)|0;
         HEAP32[$u>>2] = $87;
        }
        $88 = HEAP32[$cksum>>2]|0;
        $89 = ($88|0)!=(0);
        if ($89) {
         HEAP32[$len>>2] = 0;
        }
        $90 = HEAP32[$len>>2]|0;
        $91 = HEAP32[$2>>2]|0;
        $92 = (($91) + ($90)|0);
        HEAP8[$92>>0] = 0;
        HEAP32[$0>>2] = 0;
        $93 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($93|0);
       }
      }
     }
    }
   }
  }
 }
 $75 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $75;
 $93 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3OsAccess($pVfs,$zPath,$flags,$pResOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pResOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (FUNCTION_TABLE_iiiii[$6 & 255]($7,$8,$9,$10)|0);
 STACKTOP = sp;return ($11|0);
}
function _pager_truncate($pPager,$nPage) {
 $pPager = $pPager|0;
 $nPage = $nPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $currentSize = 0, $newSize = 0, $pTmp = 0, $rc = 0, $szPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $currentSize = sp + 8|0;
 $newSize = sp;
 $szPage = sp + 20|0;
 $pTmp = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nPage;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 17|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)>=(4);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 17|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(0);
  if (!($16)) {
   $127 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($127|0);
  }
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 160|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$szPage>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3OsFileSize($22,$currentSize)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$szPage>>2]|0;
 $25 = ($24|0)<(0);
 $26 = $25 << 31 >> 31;
 $27 = HEAP32[$1>>2]|0;
 $28 = (___muldi3(($24|0),($26|0),($27|0),0)|0);
 $29 = tempRet0;
 $30 = $newSize;
 $31 = $30;
 HEAP32[$31>>2] = $28;
 $32 = (($30) + 4)|0;
 $33 = $32;
 HEAP32[$33>>2] = $29;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if (!($35)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $36 = $currentSize;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = $newSize;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = ($38|0)!=($44|0);
 $49 = ($41|0)!=($47|0);
 $50 = $48 | $49;
 if (!($50)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $51 = $currentSize;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = $newSize;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = ($56|0)>($62|0);
 $64 = ($53>>>0)>($59>>>0);
 $65 = ($56|0)==($62|0);
 $66 = $65 & $64;
 $67 = $63 | $66;
 if ($67) {
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 64|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $newSize;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = (($71) + 4)|0;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (_sqlite3OsTruncate($70,$73,$76)|0);
  HEAP32[$rc>>2] = $77;
 } else {
  $78 = $currentSize;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = HEAP32[$szPage>>2]|0;
  $85 = ($84|0)<(0);
  $86 = $85 << 31 >> 31;
  $87 = (_i64Add(($80|0),($83|0),($84|0),($86|0))|0);
  $88 = tempRet0;
  $89 = $newSize;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = ($88|0)<($94|0);
  $96 = ($87>>>0)<=($91>>>0);
  $97 = ($88|0)==($94|0);
  $98 = $97 & $96;
  $99 = $95 | $98;
  if ($99) {
   $100 = HEAP32[$0>>2]|0;
   $101 = ((($100)) + 208|0);
   $102 = HEAP32[$101>>2]|0;
   HEAP32[$pTmp>>2] = $102;
   $103 = HEAP32[$pTmp>>2]|0;
   $104 = HEAP32[$szPage>>2]|0;
   _memset(($103|0),0,($104|0))|0;
   $105 = HEAP32[$0>>2]|0;
   $106 = ((($105)) + 64|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = HEAP32[$pTmp>>2]|0;
   $109 = HEAP32[$szPage>>2]|0;
   $110 = $newSize;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($110) + 4)|0;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = HEAP32[$szPage>>2]|0;
   $117 = ($116|0)<(0);
   $118 = $117 << 31 >> 31;
   $119 = (_i64Subtract(($112|0),($115|0),($116|0),($118|0))|0);
   $120 = tempRet0;
   $121 = (_sqlite3OsWrite($107,$108,$109,$119,$120)|0);
   HEAP32[$rc>>2] = $121;
  }
 }
 $122 = HEAP32[$rc>>2]|0;
 $123 = ($122|0)==(0);
 if (!($123)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $124 = HEAP32[$1>>2]|0;
 $125 = HEAP32[$0>>2]|0;
 $126 = ((($125)) + 36|0);
 HEAP32[$126>>2] = $124;
 $127 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($127|0);
}
function _pager_delmaster($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $exists = 0, $flags = 0, $flags1 = 0, $nMasterJournal = 0, $nMasterPtr = 0, $pJournal = 0, $pMaster = 0, $pVfs = 0, $rc = 0, $zJournal = 0, $zMasterJournal = 0, $zMasterPtr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $pVfs = sp + 52|0;
 $rc = sp + 48|0;
 $pMaster = sp + 44|0;
 $pJournal = sp + 40|0;
 $zMasterJournal = sp + 36|0;
 $nMasterJournal = sp;
 $zJournal = sp + 32|0;
 $zMasterPtr = sp + 28|0;
 $nMasterPtr = sp + 24|0;
 $flags = sp + 20|0;
 $exists = sp + 16|0;
 $c = sp + 12|0;
 $flags1 = sp + 8|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $zMaster;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$zMasterJournal>>2] = 0;
 $4 = HEAP32[$pVfs>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6<<1;
 $8 = ($7|0)<(0);
 $9 = $8 << 31 >> 31;
 $10 = (_sqlite3MallocZero($7,$9)|0);
 HEAP32[$pMaster>>2] = $10;
 $11 = HEAP32[$pMaster>>2]|0;
 $12 = HEAP32[$pVfs>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($11) + ($14)|0);
 HEAP32[$pJournal>>2] = $15;
 $16 = HEAP32[$pMaster>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  HEAP32[$flags>>2] = 16385;
  $18 = HEAP32[$pVfs>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$pMaster>>2]|0;
  $21 = (_sqlite3OsOpen($18,$19,$20,16385,0)|0);
  HEAP32[$rc>>2] = $21;
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)!=(0);
 L5: do {
  if (!($23)) {
   $24 = HEAP32[$pMaster>>2]|0;
   $25 = (_sqlite3OsFileSize($24,$nMasterJournal)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $28 = HEAP32[$pVfs>>2]|0;
    $29 = ((($28)) + 8|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$nMasterPtr>>2] = $31;
    $32 = $nMasterJournal;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$nMasterPtr>>2]|0;
    $39 = ($38|0)<(0);
    $40 = $39 << 31 >> 31;
    $41 = (_i64Add(($34|0),($37|0),($38|0),($40|0))|0);
    $42 = tempRet0;
    $43 = (_i64Add(($41|0),($42|0),1,0)|0);
    $44 = tempRet0;
    $45 = (_sqlite3Malloc($43,$44)|0);
    HEAP32[$zMasterJournal>>2] = $45;
    $46 = HEAP32[$zMasterJournal>>2]|0;
    $47 = ($46|0)!=(0|0);
    if (!($47)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    $48 = $nMasterJournal;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (_i64Add(($50|0),($53|0),1,0)|0);
    $55 = tempRet0;
    $56 = HEAP32[$zMasterJournal>>2]|0;
    $57 = (($56) + ($54)|0);
    HEAP32[$zMasterPtr>>2] = $57;
    $58 = HEAP32[$pMaster>>2]|0;
    $59 = HEAP32[$zMasterJournal>>2]|0;
    $60 = $nMasterJournal;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($60) + 4)|0;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (_sqlite3OsRead($58,$59,$62,0,0)|0);
    HEAP32[$rc>>2] = $66;
    $67 = HEAP32[$rc>>2]|0;
    $68 = ($67|0)!=(0);
    if (!($68)) {
     $69 = $nMasterJournal;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = (($69) + 4)|0;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$zMasterJournal>>2]|0;
     $76 = (($75) + ($71)|0);
     HEAP8[$76>>0] = 0;
     $77 = HEAP32[$zMasterJournal>>2]|0;
     HEAP32[$zJournal>>2] = $77;
     while(1) {
      $78 = HEAP32[$zJournal>>2]|0;
      $79 = HEAP32[$zMasterJournal>>2]|0;
      $80 = $78;
      $81 = $79;
      $82 = (($80) - ($81))|0;
      $83 = ($82|0)<(0);
      $84 = $83 << 31 >> 31;
      $85 = $nMasterJournal;
      $86 = $85;
      $87 = HEAP32[$86>>2]|0;
      $88 = (($85) + 4)|0;
      $89 = $88;
      $90 = HEAP32[$89>>2]|0;
      $91 = ($84|0)<($90|0);
      $92 = ($82>>>0)<($87>>>0);
      $93 = ($84|0)==($90|0);
      $94 = $93 & $92;
      $95 = $91 | $94;
      if (!($95)) {
       break;
      }
      $96 = HEAP32[$pVfs>>2]|0;
      $97 = HEAP32[$zJournal>>2]|0;
      $98 = (_sqlite3OsAccess($96,$97,0,$exists)|0);
      HEAP32[$rc>>2] = $98;
      $99 = HEAP32[$rc>>2]|0;
      $100 = ($99|0)!=(0);
      if ($100) {
       break L5;
      }
      $101 = HEAP32[$exists>>2]|0;
      $102 = ($101|0)!=(0);
      if ($102) {
       HEAP32[$flags1>>2] = 2049;
       $103 = HEAP32[$pVfs>>2]|0;
       $104 = HEAP32[$zJournal>>2]|0;
       $105 = HEAP32[$pJournal>>2]|0;
       $106 = HEAP32[$flags1>>2]|0;
       $107 = (_sqlite3OsOpen($103,$104,$105,$106,0)|0);
       HEAP32[$rc>>2] = $107;
       $108 = HEAP32[$rc>>2]|0;
       $109 = ($108|0)!=(0);
       if ($109) {
        break L5;
       }
       $110 = HEAP32[$pJournal>>2]|0;
       $111 = HEAP32[$zMasterPtr>>2]|0;
       $112 = HEAP32[$nMasterPtr>>2]|0;
       $113 = (_readMasterJournal($110,$111,$112)|0);
       HEAP32[$rc>>2] = $113;
       $114 = HEAP32[$pJournal>>2]|0;
       (_sqlite3OsClose($114)|0);
       $115 = HEAP32[$rc>>2]|0;
       $116 = ($115|0)!=(0);
       if ($116) {
        break L5;
       }
       $117 = HEAP32[$zMasterPtr>>2]|0;
       $118 = HEAP8[$117>>0]|0;
       $119 = $118 << 24 >> 24;
       $120 = ($119|0)!=(0);
       if ($120) {
        $121 = HEAP32[$zMasterPtr>>2]|0;
        $122 = HEAP32[$1>>2]|0;
        $123 = (_strcmp($121,$122)|0);
        $124 = ($123|0)==(0);
        $126 = $124;
       } else {
        $126 = 0;
       }
       $125 = $126&1;
       HEAP32[$c>>2] = $125;
       $127 = HEAP32[$c>>2]|0;
       $128 = ($127|0)!=(0);
       if ($128) {
        break L5;
       }
      }
      $129 = HEAP32[$zJournal>>2]|0;
      $130 = (_sqlite3Strlen30($129)|0);
      $131 = (($130) + 1)|0;
      $132 = HEAP32[$zJournal>>2]|0;
      $133 = (($132) + ($131)|0);
      HEAP32[$zJournal>>2] = $133;
     }
     $134 = HEAP32[$pMaster>>2]|0;
     (_sqlite3OsClose($134)|0);
     $135 = HEAP32[$pVfs>>2]|0;
     $136 = HEAP32[$1>>2]|0;
     $137 = (_sqlite3OsDelete($135,$136,0)|0);
     HEAP32[$rc>>2] = $137;
    }
   }
  }
 } while(0);
 $138 = HEAP32[$zMasterJournal>>2]|0;
 _sqlite3_free($138);
 $139 = HEAP32[$pMaster>>2]|0;
 $140 = ($139|0)!=(0|0);
 if (!($140)) {
  $143 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($143|0);
 }
 $141 = HEAP32[$pMaster>>2]|0;
 (_sqlite3OsClose($141)|0);
 $142 = HEAP32[$pMaster>>2]|0;
 _sqlite3_free($142);
 $143 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($143|0);
}
function _zeroJournalHdr($pPager,$doTruncate) {
 $pPager = $pPager|0;
 $doTruncate = $doTruncate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iLimit = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $iLimit = sp + 8|0;
 $sz = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $doTruncate;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 80|0);
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)!=(0);
 $11 = ($9|0)!=(0);
 $12 = $10 | $11;
 if (!($12)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 168|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $iLimit;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25|0)!=(0);
 $27 = $iLimit;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($29|0)==(0);
 $34 = ($32|0)==(0);
 $35 = $33 & $34;
 $or$cond = $26 | $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 68|0);
 $38 = HEAP32[$37>>2]|0;
 if ($or$cond) {
  $39 = (_sqlite3OsTruncate($38,0,0)|0);
  HEAP32[$rc>>2] = $39;
 } else {
  $40 = (_sqlite3OsWrite($38,19040,28,0,0)|0);
  HEAP32[$rc>>2] = $40;
 }
 $41 = HEAP32[$rc>>2]|0;
 $42 = ($41|0)==(0);
 if ($42) {
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 7|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = ($45<<24>>24)!=(0);
  if (!($46)) {
   $47 = HEAP32[$0>>2]|0;
   $48 = ((($47)) + 68|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = 16 | $53;
   $55 = (_sqlite3OsSync($49,$54)|0);
   HEAP32[$rc>>2] = $55;
  }
 }
 $56 = HEAP32[$rc>>2]|0;
 $57 = ($56|0)==(0);
 $58 = $iLimit;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)>(0);
 $65 = ($60>>>0)>(0);
 $66 = ($63|0)==(0);
 $67 = $66 & $65;
 $68 = $64 | $67;
 $or$cond3 = $57 & $68;
 if (!($or$cond3)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 68|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (_sqlite3OsFileSize($71,$sz)|0);
 HEAP32[$rc>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)==(0);
 if (!($74)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $75 = $sz;
 $76 = $75;
 $77 = HEAP32[$76>>2]|0;
 $78 = (($75) + 4)|0;
 $79 = $78;
 $80 = HEAP32[$79>>2]|0;
 $81 = $iLimit;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = ($80|0)>($86|0);
 $88 = ($77>>>0)>($83>>>0);
 $89 = ($80|0)==($86|0);
 $90 = $89 & $88;
 $91 = $87 | $90;
 if (!($91)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $92 = HEAP32[$0>>2]|0;
 $93 = ((($92)) + 68|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $iLimit;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (($95) + 4)|0;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (_sqlite3OsTruncate($94,$97,$100)|0);
 HEAP32[$rc>>2] = $101;
 $102 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _sqlite3PcacheCleanAll($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $2;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  _sqlite3PcacheMakeClean($4);
 }
 STACKTOP = sp;return;
}
function _sqlite3WalExclusiveMode($pWal,$op) {
 $pWal = $pWal|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $op;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0);
 if (!($3)) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ($26|0)>(0);
  $28 = HEAP32[$0>>2]|0;
  if ($27) {
   $29 = HEAP32[$0>>2]|0;
   $30 = ((($29)) + 40|0);
   $31 = HEAP16[$30>>1]|0;
   $32 = $31 << 16 >> 16;
   $33 = (3 + ($32))|0;
   _walUnlockShared($28,$33);
   $34 = HEAP32[$0>>2]|0;
   $35 = ((($34)) + 43|0);
   HEAP8[$35>>0] = 1;
   HEAP32[$rc>>2] = 1;
   $41 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($41|0);
  } else {
   $36 = ((($28)) + 43|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)==(0);
   $40 = $39&1;
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 43|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  HEAP32[$rc>>2] = 0;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 43|0);
 HEAP8[$9>>0] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = (3 + ($14))|0;
 $16 = (_walLockShared($10,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 43|0);
  HEAP8[$19>>0] = 1;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 43|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)==(0);
 $25 = $24&1;
 HEAP32[$rc>>2] = $25;
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3PcacheSetCachesize($pCache,$mxPage) {
 $pCache = $pCache|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[(132)>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_numberOfCachePages($9)|0);
 FUNCTION_TABLE_vii[$5 & 255]($8,$10);
 STACKTOP = sp;return;
}
function _querySharedCacheTableLock($p,$iTab,$eLock) {
 $p = $p|0;
 $iTab = $iTab|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $pBt = sp + 4|0;
 $pIter = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTab;
 HEAP8[$3>>0] = $eLock;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 9|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if (!($10)) {
  HEAP32[$0>>2] = 0;
  $55 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 76|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ($13|0)!=($14|0);
 if ($15) {
  $16 = HEAP32[$pBt>>2]|0;
  $17 = ((($16)) + 22|0);
  $18 = HEAP16[$17>>1]|0;
  $19 = $18&65535;
  $20 = $19 & 32;
  $21 = ($20|0)!=(0);
  if ($21) {
   HEAP32[$0>>2] = 262;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 }
 $22 = HEAP32[$pBt>>2]|0;
 $23 = ((($22)) + 72|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$pIter>>2] = $24;
 while(1) {
  $25 = HEAP32[$pIter>>2]|0;
  $26 = ($25|0)!=(0|0);
  if (!($26)) {
   label = 15;
   break;
  }
  $27 = HEAP32[$pIter>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = ($28|0)!=($29|0);
  if ($30) {
   $31 = HEAP32[$pIter>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = ($33|0)==($34|0);
   if ($35) {
    $36 = HEAP32[$pIter>>2]|0;
    $37 = ((($36)) + 8|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = HEAP8[$3>>0]|0;
    $41 = $40&255;
    $42 = ($39|0)!=($41|0);
    if ($42) {
     break;
    }
   }
  }
  $52 = HEAP32[$pIter>>2]|0;
  $53 = ((($52)) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pIter>>2] = $54;
 }
 if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $55 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $43 = HEAP8[$3>>0]|0;
 $44 = $43&255;
 $45 = ($44|0)==(2);
 if ($45) {
  $46 = HEAP32[$pBt>>2]|0;
  $47 = ((($46)) + 22|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $48&65535;
  $50 = $49 | 64;
  $51 = $50&65535;
  HEAP16[$47>>1] = $51;
 }
 HEAP32[$0>>2] = 262;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _lockBtree($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $isOpen = 0, $nPage = 0, $nPageFile = 0, $nPageHeader = 0, $or$cond = 0, $or$cond3 = 0, $pPage1 = 0, $page1 = 0, $pageSize = 0, $rc = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $rc = sp + 32|0;
 $pPage1 = sp + 28|0;
 $nPage = sp + 24|0;
 $nPageFile = sp + 20|0;
 $nPageHeader = sp + 16|0;
 $pageSize = sp + 12|0;
 $usableSize = sp + 8|0;
 $page1 = sp + 4|0;
 $isOpen = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$nPageFile>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3PagerSharedLock($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $222 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (_btreeGetPage($8,1,$pPage1,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $222 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $13 = HEAP32[$pPage1>>2]|0;
 $14 = ((($13)) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 28|0);
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$nPageHeader>>2] = $17;
 HEAP32[$nPage>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 _sqlite3PagerPagecount($19,$nPageFile);
 $20 = HEAP32[$nPage>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  label = 7;
 } else {
  $22 = HEAP32[$pPage1>>2]|0;
  $23 = ((($22)) + 56|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 24|0);
  $26 = HEAP32[$pPage1>>2]|0;
  $27 = ((($26)) + 56|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 92|0);
  $30 = (_memcmp($25,$29,4)|0);
  $31 = ($30|0)!=(0);
  if ($31) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$nPageFile>>2]|0;
  HEAP32[$nPage>>2] = $32;
 }
 $33 = HEAP32[$nPage>>2]|0;
 $34 = ($33|0)>(0);
 L14: do {
  if ($34) {
   $35 = HEAP32[$pPage1>>2]|0;
   $36 = ((($35)) + 56|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$page1>>2] = $37;
   HEAP32[$rc>>2] = 26;
   $38 = HEAP32[$page1>>2]|0;
   $39 = (_memcmp($38,19068,16)|0);
   $40 = ($39|0)!=(0);
   L16: do {
    if (!($40)) {
     $41 = HEAP32[$page1>>2]|0;
     $42 = ((($41)) + 18|0);
     $43 = HEAP8[$42>>0]|0;
     $44 = $43&255;
     $45 = ($44|0)>(2);
     if ($45) {
      $46 = HEAP32[$1>>2]|0;
      $47 = ((($46)) + 22|0);
      $48 = HEAP16[$47>>1]|0;
      $49 = $48&65535;
      $50 = $49 | 1;
      $51 = $50&65535;
      HEAP16[$47>>1] = $51;
     }
     $52 = HEAP32[$page1>>2]|0;
     $53 = ((($52)) + 19|0);
     $54 = HEAP8[$53>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)>(2);
     if (!($56)) {
      $57 = HEAP32[$page1>>2]|0;
      $58 = ((($57)) + 19|0);
      $59 = HEAP8[$58>>0]|0;
      $60 = $59&255;
      $61 = ($60|0)==(2);
      do {
       if ($61) {
        $62 = HEAP32[$1>>2]|0;
        $63 = ((($62)) + 22|0);
        $64 = HEAP16[$63>>1]|0;
        $65 = $64&65535;
        $66 = $65 & 16;
        $67 = ($66|0)==(0);
        if ($67) {
         HEAP32[$isOpen>>2] = 0;
         $68 = HEAP32[$1>>2]|0;
         $69 = HEAP32[$68>>2]|0;
         $70 = (_sqlite3PagerOpenWal($69,$isOpen)|0);
         HEAP32[$rc>>2] = $70;
         $71 = HEAP32[$rc>>2]|0;
         $72 = ($71|0)!=(0);
         if ($72) {
          break L16;
         }
         $73 = HEAP32[$isOpen>>2]|0;
         $74 = ($73|0)==(0);
         if (!($74)) {
          HEAP32[$rc>>2] = 26;
          break;
         }
         $75 = HEAP32[$pPage1>>2]|0;
         _releasePage($75);
         HEAP32[$0>>2] = 0;
         $222 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($222|0);
        }
       }
      } while(0);
      $76 = HEAP32[$page1>>2]|0;
      $77 = ((($76)) + 21|0);
      $78 = (_memcmp($77,19084,3)|0);
      $79 = ($78|0)!=(0);
      if (!($79)) {
       $80 = HEAP32[$page1>>2]|0;
       $81 = ((($80)) + 16|0);
       $82 = HEAP8[$81>>0]|0;
       $83 = $82&255;
       $84 = $83 << 8;
       $85 = HEAP32[$page1>>2]|0;
       $86 = ((($85)) + 17|0);
       $87 = HEAP8[$86>>0]|0;
       $88 = $87&255;
       $89 = $88 << 16;
       $90 = $84 | $89;
       HEAP32[$pageSize>>2] = $90;
       $91 = HEAP32[$pageSize>>2]|0;
       $92 = (($91) - 1)|0;
       $93 = HEAP32[$pageSize>>2]|0;
       $94 = $92 & $93;
       $95 = ($94|0)!=(0);
       $96 = HEAP32[$pageSize>>2]|0;
       $97 = ($96>>>0)>(65536);
       $or$cond = $95 | $97;
       $98 = HEAP32[$pageSize>>2]|0;
       $99 = ($98>>>0)<=(256);
       $or$cond3 = $or$cond | $99;
       if (!($or$cond3)) {
        $100 = HEAP32[$pageSize>>2]|0;
        $101 = HEAP32[$page1>>2]|0;
        $102 = ((($101)) + 20|0);
        $103 = HEAP8[$102>>0]|0;
        $104 = $103&255;
        $105 = (($100) - ($104))|0;
        HEAP32[$usableSize>>2] = $105;
        $106 = HEAP32[$pageSize>>2]|0;
        $107 = HEAP32[$1>>2]|0;
        $108 = ((($107)) + 32|0);
        $109 = HEAP32[$108>>2]|0;
        $110 = ($106|0)!=($109|0);
        if ($110) {
         $111 = HEAP32[$pPage1>>2]|0;
         _releasePage($111);
         $112 = HEAP32[$usableSize>>2]|0;
         $113 = HEAP32[$1>>2]|0;
         $114 = ((($113)) + 36|0);
         HEAP32[$114>>2] = $112;
         $115 = HEAP32[$pageSize>>2]|0;
         $116 = HEAP32[$1>>2]|0;
         $117 = ((($116)) + 32|0);
         HEAP32[$117>>2] = $115;
         $118 = HEAP32[$1>>2]|0;
         _freeTempSpace($118);
         $119 = HEAP32[$1>>2]|0;
         $120 = HEAP32[$119>>2]|0;
         $121 = HEAP32[$1>>2]|0;
         $122 = ((($121)) + 32|0);
         $123 = HEAP32[$pageSize>>2]|0;
         $124 = HEAP32[$usableSize>>2]|0;
         $125 = (($123) - ($124))|0;
         $126 = (_sqlite3PagerSetPagesize($120,$122,$125)|0);
         HEAP32[$rc>>2] = $126;
         $127 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $127;
         $222 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($222|0);
        }
        $128 = HEAP32[$1>>2]|0;
        $129 = ((($128)) + 4|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = ((($130)) + 24|0);
        $132 = HEAP32[$131>>2]|0;
        $133 = $132 & 65536;
        $134 = ($133|0)==(0);
        if ($134) {
         $135 = HEAP32[$nPage>>2]|0;
         $136 = HEAP32[$nPageFile>>2]|0;
         $137 = ($135|0)>($136|0);
         if ($137) {
          $138 = (_sqlite3CorruptError(58503)|0);
          HEAP32[$rc>>2] = $138;
          break;
         }
        }
        $139 = HEAP32[$usableSize>>2]|0;
        $140 = ($139>>>0)<(480);
        if (!($140)) {
         $141 = HEAP32[$pageSize>>2]|0;
         $142 = HEAP32[$1>>2]|0;
         $143 = ((($142)) + 32|0);
         HEAP32[$143>>2] = $141;
         $144 = HEAP32[$usableSize>>2]|0;
         $145 = HEAP32[$1>>2]|0;
         $146 = ((($145)) + 36|0);
         HEAP32[$146>>2] = $144;
         $147 = HEAP32[$page1>>2]|0;
         $148 = ((($147)) + 52|0);
         $149 = (_sqlite3Get4byte($148)|0);
         $150 = ($149|0)!=(0);
         $151 = $150 ? 1 : 0;
         $152 = $151&255;
         $153 = HEAP32[$1>>2]|0;
         $154 = ((($153)) + 17|0);
         HEAP8[$154>>0] = $152;
         $155 = HEAP32[$page1>>2]|0;
         $156 = ((($155)) + 64|0);
         $157 = (_sqlite3Get4byte($156)|0);
         $158 = ($157|0)!=(0);
         $159 = $158 ? 1 : 0;
         $160 = $159&255;
         $161 = HEAP32[$1>>2]|0;
         $162 = ((($161)) + 18|0);
         HEAP8[$162>>0] = $160;
         break L14;
        }
       }
      }
     }
    }
   } while(0);
   $218 = HEAP32[$pPage1>>2]|0;
   _releasePage($218);
   $219 = HEAP32[$1>>2]|0;
   $220 = ((($219)) + 12|0);
   HEAP32[$220>>2] = 0;
   $221 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $221;
   $222 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($222|0);
  }
 } while(0);
 $163 = HEAP32[$1>>2]|0;
 $164 = ((($163)) + 36|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = (($165) - 12)|0;
 $167 = $166<<6;
 $168 = (($167>>>0) / 255)&-1;
 $169 = (($168) - 23)|0;
 $170 = $169&65535;
 $171 = HEAP32[$1>>2]|0;
 $172 = ((($171)) + 24|0);
 HEAP16[$172>>1] = $170;
 $173 = HEAP32[$1>>2]|0;
 $174 = ((($173)) + 36|0);
 $175 = HEAP32[$174>>2]|0;
 $176 = (($175) - 12)|0;
 $177 = $176<<5;
 $178 = (($177>>>0) / 255)&-1;
 $179 = (($178) - 23)|0;
 $180 = $179&65535;
 $181 = HEAP32[$1>>2]|0;
 $182 = ((($181)) + 26|0);
 HEAP16[$182>>1] = $180;
 $183 = HEAP32[$1>>2]|0;
 $184 = ((($183)) + 36|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = (($185) - 35)|0;
 $187 = $186&65535;
 $188 = HEAP32[$1>>2]|0;
 $189 = ((($188)) + 28|0);
 HEAP16[$189>>1] = $187;
 $190 = HEAP32[$1>>2]|0;
 $191 = ((($190)) + 36|0);
 $192 = HEAP32[$191>>2]|0;
 $193 = (($192) - 12)|0;
 $194 = $193<<5;
 $195 = (($194>>>0) / 255)&-1;
 $196 = (($195) - 23)|0;
 $197 = $196&65535;
 $198 = HEAP32[$1>>2]|0;
 $199 = ((($198)) + 30|0);
 HEAP16[$199>>1] = $197;
 $200 = HEAP32[$1>>2]|0;
 $201 = ((($200)) + 24|0);
 $202 = HEAP16[$201>>1]|0;
 $203 = $202&65535;
 $204 = ($203|0)>(127);
 $205 = HEAP32[$1>>2]|0;
 if ($204) {
  $206 = ((($205)) + 21|0);
  HEAP8[$206>>0] = 127;
 } else {
  $207 = ((($205)) + 24|0);
  $208 = HEAP16[$207>>1]|0;
  $209 = $208&255;
  $210 = HEAP32[$1>>2]|0;
  $211 = ((($210)) + 21|0);
  HEAP8[$211>>0] = $209;
 }
 $212 = HEAP32[$pPage1>>2]|0;
 $213 = HEAP32[$1>>2]|0;
 $214 = ((($213)) + 12|0);
 HEAP32[$214>>2] = $212;
 $215 = HEAP32[$nPage>>2]|0;
 $216 = HEAP32[$1>>2]|0;
 $217 = ((($216)) + 44|0);
 HEAP32[$217>>2] = $215;
 HEAP32[$0>>2] = 0;
 $222 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($222|0);
}
function _sqlite3PagerBegin($pPager,$exFlag,$subjInMemory) {
 $pPager = $pPager|0;
 $exFlag = $exFlag|0;
 $subjInMemory = $subjInMemory|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $exFlag;
 HEAP32[$3>>2] = $subjInMemory;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = HEAP32[$3>>2]|0;
 $12 = $11&255;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 22|0);
 HEAP8[$14>>0] = $12;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 17|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(1);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = (_pagerUseWal($20)|0);
  $22 = ($21|0)!=(0);
  $23 = HEAP32[$1>>2]|0;
  if ($22) {
   $24 = ((($23)) + 4|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)!=(0);
   do {
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 216|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = (_sqlite3WalExclusiveMode($30,-1)|0);
     $32 = ($31|0)!=(0);
     if ($32) {
      $33 = HEAP32[$1>>2]|0;
      $34 = (_pagerLockDb($33,4)|0);
      HEAP32[$rc>>2] = $34;
      $35 = HEAP32[$rc>>2]|0;
      $36 = ($35|0)!=(0);
      if (!($36)) {
       $38 = HEAP32[$1>>2]|0;
       $39 = ((($38)) + 216|0);
       $40 = HEAP32[$39>>2]|0;
       (_sqlite3WalExclusiveMode($40,1)|0);
       break;
      }
      $37 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $37;
      $78 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($78|0);
     }
    }
   } while(0);
   $41 = HEAP32[$1>>2]|0;
   $42 = ((($41)) + 216|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_sqlite3WalBeginWriteTransaction($43)|0);
   HEAP32[$rc>>2] = $44;
  } else {
   $45 = (_pagerLockDb($23,2)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)==(0);
   $48 = HEAP32[$2>>2]|0;
   $49 = ($48|0)!=(0);
   $or$cond = $47 & $49;
   if ($or$cond) {
    $50 = HEAP32[$1>>2]|0;
    $51 = (_pager_wait_on_lock($50,4)|0);
    HEAP32[$rc>>2] = $51;
   }
  }
  $52 = HEAP32[$rc>>2]|0;
  $53 = ($52|0)==(0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 17|0);
   HEAP8[$55>>0] = 2;
   $56 = HEAP32[$1>>2]|0;
   $57 = ((($56)) + 28|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 40|0);
   HEAP32[$60>>2] = $58;
   $61 = HEAP32[$1>>2]|0;
   $62 = ((($61)) + 28|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 36|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP32[$1>>2]|0;
   $67 = ((($66)) + 28|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 32|0);
   HEAP32[$70>>2] = $68;
   $71 = HEAP32[$1>>2]|0;
   $72 = ((($71)) + 80|0);
   $73 = $72;
   $74 = $73;
   HEAP32[$74>>2] = 0;
   $75 = (($73) + 4)|0;
   $76 = $75;
   HEAP32[$76>>2] = 0;
  }
 }
 $77 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _newDatabase($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $data = 0, $pP1 = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pP1 = sp + 8|0;
 $data = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4>>>0)>(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pP1>>2] = $8;
 $9 = HEAP32[$pP1>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$data>>2] = $11;
 $12 = HEAP32[$pP1>>2]|0;
 $13 = ((($12)) + 72|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3PagerWrite($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $18;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 } else {
  $19 = HEAP32[$data>>2]|0;
  dest=$19; src=19068; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $22 >>> 8;
  $24 = $23 & 255;
  $25 = $24&255;
  $26 = HEAP32[$data>>2]|0;
  $27 = ((($26)) + 16|0);
  HEAP8[$27>>0] = $25;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30 >>> 16;
  $32 = $31 & 255;
  $33 = $32&255;
  $34 = HEAP32[$data>>2]|0;
  $35 = ((($34)) + 17|0);
  HEAP8[$35>>0] = $33;
  $36 = HEAP32[$data>>2]|0;
  $37 = ((($36)) + 18|0);
  HEAP8[$37>>0] = 1;
  $38 = HEAP32[$data>>2]|0;
  $39 = ((($38)) + 19|0);
  HEAP8[$39>>0] = 1;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 32|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($42) - ($45))|0;
  $47 = $46&255;
  $48 = HEAP32[$data>>2]|0;
  $49 = ((($48)) + 20|0);
  HEAP8[$49>>0] = $47;
  $50 = HEAP32[$data>>2]|0;
  $51 = ((($50)) + 21|0);
  HEAP8[$51>>0] = 64;
  $52 = HEAP32[$data>>2]|0;
  $53 = ((($52)) + 22|0);
  HEAP8[$53>>0] = 32;
  $54 = HEAP32[$data>>2]|0;
  $55 = ((($54)) + 23|0);
  HEAP8[$55>>0] = 32;
  $56 = HEAP32[$data>>2]|0;
  $57 = ((($56)) + 24|0);
  dest=$57; stop=dest+76|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  $58 = HEAP32[$pP1>>2]|0;
  _zeroPage($58,13);
  $59 = HEAP32[$1>>2]|0;
  $60 = ((($59)) + 22|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61&65535;
  $63 = $62 | 2;
  $64 = $63&65535;
  HEAP16[$60>>1] = $64;
  $65 = HEAP32[$data>>2]|0;
  $66 = ((($65)) + 52|0);
  $67 = HEAP32[$1>>2]|0;
  $68 = ((($67)) + 17|0);
  $69 = HEAP8[$68>>0]|0;
  $70 = $69&255;
  _sqlite3Put4byte($66,$70);
  $71 = HEAP32[$data>>2]|0;
  $72 = ((($71)) + 64|0);
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 18|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  _sqlite3Put4byte($72,$76);
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 44|0);
  HEAP32[$78>>2] = 1;
  $79 = HEAP32[$data>>2]|0;
  $80 = ((($79)) + 31|0);
  HEAP8[$80>>0] = 1;
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 return (0)|0;
}
function _unlockBtreeIfUnused($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pPage1>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 12|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$pPage1>>2]|0;
 _releasePageNotNull($15);
 STACKTOP = sp;return;
}
function _sqlite3PagerOpenSavepoint($pPager,$nSavepoint) {
 $pPager = $pPager|0;
 $nSavepoint = $nSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $nSavepoint;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 104|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)>($6|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 6|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = (_pagerOpenSavepoint($13,$14)|0);
   HEAP32[$0>>2] = $15;
   $16 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($16|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3PagerSharedLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bExists = 0, $bHotJournal = 0, $dbFileVers = 0, $f = 0, $fout = 0, $nPage = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pVfs = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $bHotJournal = sp + 20|0;
 $pVfs = sp + 16|0;
 $bExists = sp + 12|0;
 $fout = sp + 8|0;
 $f = sp + 4|0;
 $nPage = sp;
 $dbFileVers = sp + 40|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 44|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 44|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$0>>2] = $13;
   $153 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($153|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_pagerUseWal($14)|0);
 $16 = ($15|0)!=(0);
 do {
  if ($16) {
   label = 35;
  } else {
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 17|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = ($20|0)==(0);
   if ($21) {
    HEAP32[$bHotJournal>>2] = 1;
    $22 = HEAP32[$1>>2]|0;
    $23 = (_pager_wait_on_lock($22,1)|0);
    HEAP32[$rc>>2] = $23;
    $24 = HEAP32[$rc>>2]|0;
    $25 = ($24|0)!=(0);
    if (!($25)) {
     $26 = HEAP32[$1>>2]|0;
     $27 = ((($26)) + 18|0);
     $28 = HEAP8[$27>>0]|0;
     $29 = $28&255;
     $30 = ($29|0)<=(1);
     if ($30) {
      $31 = HEAP32[$1>>2]|0;
      $32 = (_hasHotJournal($31,$bHotJournal)|0);
      HEAP32[$rc>>2] = $32;
     }
     $33 = HEAP32[$rc>>2]|0;
     $34 = ($33|0)!=(0);
     if (!($34)) {
      $35 = HEAP32[$bHotJournal>>2]|0;
      $36 = ($35|0)!=(0);
      if ($36) {
       $37 = HEAP32[$1>>2]|0;
       $38 = ((($37)) + 15|0);
       $39 = HEAP8[$38>>0]|0;
       $40 = ($39<<24>>24)!=(0);
       if ($40) {
        HEAP32[$rc>>2] = 776;
        break;
       }
       $41 = HEAP32[$1>>2]|0;
       $42 = (_pagerLockDb($41,4)|0);
       HEAP32[$rc>>2] = $42;
       $43 = HEAP32[$rc>>2]|0;
       $44 = ($43|0)!=(0);
       if ($44) {
        break;
       }
       $45 = HEAP32[$1>>2]|0;
       $46 = ((($45)) + 68|0);
       $47 = HEAP32[$46>>2]|0;
       $48 = HEAP32[$47>>2]|0;
       $49 = ($48|0)!=(0|0);
       if (!($49)) {
        $50 = HEAP32[$1>>2]|0;
        $51 = HEAP32[$50>>2]|0;
        HEAP32[$pVfs>>2] = $51;
        $52 = HEAP32[$pVfs>>2]|0;
        $53 = HEAP32[$1>>2]|0;
        $54 = ((($53)) + 180|0);
        $55 = HEAP32[$54>>2]|0;
        $56 = (_sqlite3OsAccess($52,$55,0,$bExists)|0);
        HEAP32[$rc>>2] = $56;
        $57 = HEAP32[$rc>>2]|0;
        $58 = ($57|0)==(0);
        $59 = HEAP32[$bExists>>2]|0;
        $60 = ($59|0)!=(0);
        $or$cond = $58 & $60;
        if ($or$cond) {
         HEAP32[$fout>>2] = 0;
         HEAP32[$f>>2] = 2050;
         $61 = HEAP32[$pVfs>>2]|0;
         $62 = HEAP32[$1>>2]|0;
         $63 = ((($62)) + 180|0);
         $64 = HEAP32[$63>>2]|0;
         $65 = HEAP32[$1>>2]|0;
         $66 = ((($65)) + 68|0);
         $67 = HEAP32[$66>>2]|0;
         $68 = HEAP32[$f>>2]|0;
         $69 = (_sqlite3OsOpen($61,$64,$67,$68,$fout)|0);
         HEAP32[$rc>>2] = $69;
         $70 = HEAP32[$rc>>2]|0;
         $71 = ($70|0)==(0);
         if ($71) {
          $72 = HEAP32[$fout>>2]|0;
          $73 = $72 & 1;
          $74 = ($73|0)!=(0);
          if ($74) {
           $75 = (_sqlite3CantopenError(48855)|0);
           HEAP32[$rc>>2] = $75;
           $76 = HEAP32[$1>>2]|0;
           $77 = ((($76)) + 68|0);
           $78 = HEAP32[$77>>2]|0;
           (_sqlite3OsClose($78)|0);
          }
         }
        }
       }
       $79 = HEAP32[$1>>2]|0;
       $80 = ((($79)) + 68|0);
       $81 = HEAP32[$80>>2]|0;
       $82 = HEAP32[$81>>2]|0;
       $83 = ($82|0)!=(0|0);
       $84 = HEAP32[$1>>2]|0;
       if ($83) {
        $85 = (_pagerSyncHotJournal($84)|0);
        HEAP32[$rc>>2] = $85;
        $86 = HEAP32[$rc>>2]|0;
        $87 = ($86|0)==(0);
        if ($87) {
         $88 = HEAP32[$1>>2]|0;
         $89 = (_pager_playback($88,1)|0);
         HEAP32[$rc>>2] = $89;
         $90 = HEAP32[$1>>2]|0;
         $91 = ((($90)) + 17|0);
         HEAP8[$91>>0] = 0;
        }
       } else {
        $92 = ((($84)) + 4|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = ($93<<24>>24)!=(0);
        if (!($94)) {
         $95 = HEAP32[$1>>2]|0;
         (_pagerUnlockDb($95,1)|0);
        }
       }
       $96 = HEAP32[$rc>>2]|0;
       $97 = ($96|0)!=(0);
       if ($97) {
        $98 = HEAP32[$1>>2]|0;
        $99 = HEAP32[$rc>>2]|0;
        (_pager_error($98,$99)|0);
        break;
       }
      }
      $100 = HEAP32[$1>>2]|0;
      $101 = ((($100)) + 13|0);
      $102 = HEAP8[$101>>0]|0;
      $103 = ($102<<24>>24)!=(0);
      if (!($103)) {
       $104 = HEAP32[$1>>2]|0;
       $105 = ((($104)) + 24|0);
       $106 = HEAP8[$105>>0]|0;
       $107 = $106&255;
       $108 = ($107|0)!=(0);
       if ($108) {
        HEAP32[$nPage>>2] = 0;
        $109 = HEAP32[$1>>2]|0;
        $110 = (_pagerPagecount($109,$nPage)|0);
        HEAP32[$rc>>2] = $110;
        $111 = HEAP32[$rc>>2]|0;
        $112 = ($111|0)!=(0);
        if ($112) {
         break;
        }
        $113 = HEAP32[$nPage>>2]|0;
        $114 = ($113>>>0)>(0);
        if ($114) {
         $115 = HEAP32[$1>>2]|0;
         $116 = ((($115)) + 64|0);
         $117 = HEAP32[$116>>2]|0;
         $118 = (_sqlite3OsRead($117,$dbFileVers,16,24,0)|0);
         HEAP32[$rc>>2] = $118;
         $119 = HEAP32[$rc>>2]|0;
         $120 = ($119|0)!=(0);
         $121 = HEAP32[$rc>>2]|0;
         $122 = ($121|0)!=(522);
         $or$cond3 = $120 & $122;
         if ($or$cond3) {
          break;
         }
        } else {
         dest=$dbFileVers; stop=dest+16|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
        }
        $123 = HEAP32[$1>>2]|0;
        $124 = ((($123)) + 112|0);
        $125 = (_memcmp($124,$dbFileVers,16)|0);
        $126 = ($125|0)!=(0);
        if ($126) {
         $127 = HEAP32[$1>>2]|0;
         _pager_reset($127);
        }
       }
      }
      $128 = HEAP32[$1>>2]|0;
      $129 = (_pagerOpenWalIfPresent($128)|0);
      HEAP32[$rc>>2] = $129;
      label = 35;
     }
    }
   } else {
    label = 35;
   }
  }
 } while(0);
 if ((label|0) == 35) {
  $130 = HEAP32[$1>>2]|0;
  $131 = (_pagerUseWal($130)|0);
  $132 = ($131|0)!=(0);
  if ($132) {
   $133 = HEAP32[$1>>2]|0;
   $134 = (_pagerBeginReadTransaction($133)|0);
   HEAP32[$rc>>2] = $134;
  }
  $135 = HEAP32[$1>>2]|0;
  $136 = ((($135)) + 17|0);
  $137 = HEAP8[$136>>0]|0;
  $138 = $137&255;
  $139 = ($138|0)==(0);
  $140 = HEAP32[$rc>>2]|0;
  $141 = ($140|0)==(0);
  $or$cond5 = $139 & $141;
  if ($or$cond5) {
   $142 = HEAP32[$1>>2]|0;
   $143 = HEAP32[$1>>2]|0;
   $144 = ((($143)) + 28|0);
   $145 = (_pagerPagecount($142,$144)|0);
   HEAP32[$rc>>2] = $145;
  }
 }
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)!=(0);
 $148 = HEAP32[$1>>2]|0;
 if ($147) {
  _pager_unlock($148);
 } else {
  $149 = ((($148)) + 17|0);
  HEAP8[$149>>0] = 1;
  $150 = HEAP32[$1>>2]|0;
  $151 = ((($150)) + 24|0);
  HEAP8[$151>>0] = 1;
 }
 $152 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $152;
 $153 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _btreeGetPage($pBt,$pgno,$ppPage,$flags) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 $9 = (_sqlite3PagerGet($6,$7,$pDbPage,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $13 = HEAP32[$pDbPage>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_btreePageFromDbPage($13,$14,$15)|0);
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3PagerOpenWal($pPager,$pbOpen) {
 $pPager = $pPager|0;
 $pbOpen = $pbOpen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pbOpen;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 13|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 do {
  if ($6) {
   label = 7;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 216|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    label = 7;
   } else {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_sqlite3PagerWalSupported($11)|0);
    $13 = ($12|0)!=(0);
    if ($13) {
     $14 = HEAP32[$1>>2]|0;
     $15 = ((($14)) + 68|0);
     $16 = HEAP32[$15>>2]|0;
     (_sqlite3OsClose($16)|0);
     $17 = HEAP32[$1>>2]|0;
     $18 = (_pagerOpenWal($17)|0);
     HEAP32[$rc>>2] = $18;
     $19 = HEAP32[$rc>>2]|0;
     $20 = ($19|0)==(0);
     if (!($20)) {
      break;
     }
     $21 = HEAP32[$1>>2]|0;
     $22 = ((($21)) + 5|0);
     HEAP8[$22>>0] = 5;
     $23 = HEAP32[$1>>2]|0;
     $24 = ((($23)) + 17|0);
     HEAP8[$24>>0] = 0;
     break;
    } else {
     HEAP32[$0>>2] = 14;
     $27 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($27|0);
    }
   }
  }
 } while(0);
 if ((label|0) == 7) {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$25>>2] = 1;
 }
 $26 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _releasePage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _releasePageNotNull($3);
 STACKTOP = sp;return;
}
function _freeTempSpace($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + -4|0);
 HEAP32[$6>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 80|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3PageFree($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 80|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _hasHotJournal($pPager,$pExists) {
 $pPager = $pPager|0;
 $pExists = $pExists|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $exists = 0, $f = 0, $first = 0, $jrnlOpen = 0, $locked = 0, $nPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pVfs = sp + 24|0;
 $rc = sp + 20|0;
 $exists = sp + 16|0;
 $jrnlOpen = sp + 12|0;
 $locked = sp + 8|0;
 $nPage = sp + 4|0;
 $f = sp;
 $first = sp + 36|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pExists;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$rc>>2] = 0;
 HEAP32[$exists>>2] = 1;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8 ^ 1;
 $10 = $9 ^ 1;
 $11 = $10&1;
 HEAP32[$jrnlOpen>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$jrnlOpen>>2]|0;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$pVfs>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 180|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3OsAccess($15,$18,0,$exists)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 $22 = HEAP32[$exists>>2]|0;
 $23 = ($22|0)!=(0);
 $or$cond = $21 & $23;
 if (!($or$cond)) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 HEAP32[$locked>>2] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 64|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3OsCheckReservedLock($26,$locked)|0);
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$locked>>2]|0;
 $31 = ($30|0)!=(0);
 $or$cond3 = $29 | $31;
 if ($or$cond3) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (_pagerPagecount($32,$nPage)|0);
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if (!($35)) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $36 = HEAP32[$nPage>>2]|0;
 $37 = ($36|0)!=(0);
 $38 = HEAP32[$jrnlOpen>>2]|0;
 $39 = ($38|0)!=(0);
 $or$cond5 = $37 | $39;
 if (!($or$cond5)) {
  _sqlite3BeginBenignMalloc();
  $40 = HEAP32[$0>>2]|0;
  $41 = (_pagerLockDb($40,2)|0);
  $42 = ($41|0)==(0);
  if ($42) {
   $43 = HEAP32[$pVfs>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = ((($44)) + 180|0);
   $46 = HEAP32[$45>>2]|0;
   (_sqlite3OsDelete($43,$46,0)|0);
   $47 = HEAP32[$0>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = ($49<<24>>24)!=(0);
   if (!($50)) {
    $51 = HEAP32[$0>>2]|0;
    (_pagerUnlockDb($51,1)|0);
   }
  }
  _sqlite3EndBenignMalloc();
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $52 = HEAP32[$jrnlOpen>>2]|0;
 $53 = ($52|0)!=(0);
 if (!($53)) {
  HEAP32[$f>>2] = 2049;
  $54 = HEAP32[$pVfs>>2]|0;
  $55 = HEAP32[$0>>2]|0;
  $56 = ((($55)) + 180|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = HEAP32[$0>>2]|0;
  $59 = ((($58)) + 68|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$f>>2]|0;
  $62 = (_sqlite3OsOpen($54,$57,$60,$61,$f)|0);
  HEAP32[$rc>>2] = $62;
 }
 $63 = HEAP32[$rc>>2]|0;
 $64 = ($63|0)==(0);
 if ($64) {
  HEAP8[$first>>0] = 0;
  $65 = HEAP32[$0>>2]|0;
  $66 = ((($65)) + 68|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (_sqlite3OsRead($67,$first,1,0,0)|0);
  HEAP32[$rc>>2] = $68;
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(522);
  $$ = $70 ? 0 : $68;
  HEAP32[$rc>>2] = $$;
  $71 = HEAP32[$jrnlOpen>>2]|0;
  $72 = ($71|0)!=(0);
  if (!($72)) {
   $73 = HEAP32[$0>>2]|0;
   $74 = ((($73)) + 68|0);
   $75 = HEAP32[$74>>2]|0;
   (_sqlite3OsClose($75)|0);
  }
  $76 = HEAP8[$first>>0]|0;
  $77 = $76&255;
  $78 = ($77|0)!=(0);
  $79 = $78&1;
  $80 = HEAP32[$1>>2]|0;
  HEAP32[$80>>2] = $79;
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 } else {
  $81 = HEAP32[$rc>>2]|0;
  $82 = ($81|0)==(14);
  if (!($82)) {
   $84 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($84|0);
  }
  $83 = HEAP32[$1>>2]|0;
  HEAP32[$83>>2] = 1;
  HEAP32[$rc>>2] = 0;
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 return (0)|0;
}
function _pagerPagecount($pPager,$pnPage) {
 $pPager = $pPager|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nPage = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $nPage = sp + 12|0;
 $n = sp;
 $rc = sp + 8|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pnPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 216|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3WalDbsize($5)|0);
 HEAP32[$nPage>>2] = $6;
 $7 = HEAP32[$nPage>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = $n;
  $10 = $9;
  HEAP32[$10>>2] = 0;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = 0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 64|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 64|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3OsFileSize($20,$n)|0);
   HEAP32[$rc>>2] = $21;
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $24;
    $57 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($57|0);
   }
  }
  $25 = $n;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 160|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)<(0);
  $35 = $34 << 31 >> 31;
  $36 = (_i64Add(($27|0),($30|0),($33|0),($35|0))|0);
  $37 = tempRet0;
  $38 = (_i64Subtract(($36|0),($37|0),1,0)|0);
  $39 = tempRet0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 160|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)<(0);
  $44 = $43 << 31 >> 31;
  $45 = (___divdi3(($38|0),($39|0),($42|0),($44|0))|0);
  $46 = tempRet0;
  HEAP32[$nPage>>2] = $45;
 }
 $47 = HEAP32[$nPage>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 164|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($47>>>0)>($50>>>0);
 if ($51) {
  $52 = HEAP32[$nPage>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = ((($53)) + 164|0);
  HEAP32[$54>>2] = $52;
 }
 $55 = HEAP32[$nPage>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 HEAP32[$56>>2] = $55;
 HEAP32[$0>>2] = 0;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _pagerOpenWalIfPresent($pPager) {
 $pPager = $pPager|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $isWal = 0, $nPage = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $isWal = sp + 4|0;
 $nPage = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 13|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 do {
  if (!($5)) {
   $6 = HEAP32[$1>>2]|0;
   $7 = (_pagerPagecount($6,$nPage)|0);
   HEAP32[$rc>>2] = $7;
   $8 = HEAP32[$rc>>2]|0;
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $10;
    $35 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
   $11 = HEAP32[$nPage>>2]|0;
   $12 = ($11|0)==(0);
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 220|0);
   $17 = HEAP32[$16>>2]|0;
   if ($12) {
    $18 = (_sqlite3OsDelete($14,$17,0)|0);
    HEAP32[$rc>>2] = $18;
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)==(5898);
    $$ = $20 ? 0 : $18;
    HEAP32[$rc>>2] = $$;
    HEAP32[$isWal>>2] = 0;
   } else {
    $21 = (_sqlite3OsAccess($14,$17,0,$isWal)|0);
    HEAP32[$rc>>2] = $21;
   }
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$isWal>>2]|0;
    $25 = ($24|0)!=(0);
    $26 = HEAP32[$1>>2]|0;
    if ($25) {
     $27 = (_sqlite3PagerOpenWal($26,0)|0);
     HEAP32[$rc>>2] = $27;
     break;
    }
    $28 = ((($26)) + 5|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(5);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = ((($32)) + 5|0);
     HEAP8[$33>>0] = 0;
    }
   }
  }
 } while(0);
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _pagerBeginReadTransaction($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $changed = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $changed = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$changed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3WalEndReadTransaction($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3WalBeginReadTransaction($6,$changed)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$changed>>2]|0;
 $11 = ($10|0)!=(0);
 $or$cond = $9 | $11;
 if (!($or$cond)) {
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $12 = HEAP32[$0>>2]|0;
 _pager_reset($12);
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3OsCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3WalDbsize($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pWal;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 40|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6 << 16 >> 16;
  $8 = ($7|0)>=(0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 52|0);
   $11 = ((($10)) + 20|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$0>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($13|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3WalBeginReadTransaction($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $cnt = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $pChanged;
 HEAP32[$cnt>>2] = 0;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = HEAP32[$cnt>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[$cnt>>2] = $5;
  $6 = (_walTryBeginRead($2,$3,0,$5)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(-1);
  if (!($8)) {
   break;
  }
 }
 $9 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _btreePageFromDbPage($pDbPage,$pgno,$pBt) {
 $pDbPage = $pDbPage|0;
 $pgno = $pgno|0;
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pDbPage;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $pBt;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3PagerGetExtra($3)|0);
 HEAP32[$pPage>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$pPage>>2]|0;
 $7 = ((($6)) + 84|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5|0)!=($8|0);
 if (!($9)) {
  $29 = HEAP32[$pPage>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3PagerGetData($10)|0);
 $12 = HEAP32[$pPage>>2]|0;
 $13 = ((($12)) + 56|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$pPage>>2]|0;
 $16 = ((($15)) + 72|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$pPage>>2]|0;
 $19 = ((($18)) + 52|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$pPage>>2]|0;
 $22 = ((($21)) + 84|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$1>>2]|0;
 $24 = ($23|0)==(1);
 $25 = $24 ? 100 : 0;
 $26 = $25&255;
 $27 = HEAP32[$pPage>>2]|0;
 $28 = ((($27)) + 5|0);
 HEAP8[$28>>0] = $26;
 $29 = HEAP32[$pPage>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PagerWalSupported($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMethods = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pMethods = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pMethods>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)!=(0);
 if ($9) {
  $18 = 1;
  $17 = $18&1;
  STACKTOP = sp;return ($17|0);
 }
 $10 = HEAP32[$pMethods>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>=(2);
 if (!($12)) {
  $18 = 0;
  $17 = $18&1;
  STACKTOP = sp;return ($17|0);
 }
 $13 = HEAP32[$pMethods>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 $18 = $16;
 $17 = $18&1;
 STACKTOP = sp;return ($17|0);
}
function _pagerOpenWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_pagerExclusiveLock($5)|0);
  HEAP32[$rc>>2] = $6;
 }
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $32 = HEAP32[$0>>2]|0;
  _pagerFixMaplimit($32);
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 220|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 168|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 216|0);
 $31 = (_sqlite3WalOpen($10,$13,$16,$20,$25,$28,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($32);
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _pagerExclusiveLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pagerLockDb($1,4)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  $6 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $5 = HEAP32[$0>>2]|0;
 (_pagerUnlockDb($5,1)|0);
 $6 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3WalOpen($pVfs,$pDbFd,$zWalName,$bNoShm,$0,$1,$ppWal) {
 $pVfs = $pVfs|0;
 $pDbFd = $pDbFd|0;
 $zWalName = $zWalName|0;
 $bNoShm = $bNoShm|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppWal = $ppWal|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $flags = 0, $iDC = 0, $pRet = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp;
 $8 = sp + 24|0;
 $rc = sp + 20|0;
 $pRet = sp + 16|0;
 $flags = sp + 12|0;
 $iDC = sp + 8|0;
 HEAP32[$3>>2] = $pVfs;
 HEAP32[$4>>2] = $pDbFd;
 HEAP32[$5>>2] = $zWalName;
 HEAP32[$6>>2] = $bNoShm;
 $9 = $7;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$8>>2] = $ppWal;
 $13 = HEAP32[$8>>2]|0;
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (120 + ($16))|0;
 $18 = (_sqlite3MallocZero($17,0)|0);
 HEAP32[$pRet>>2] = $18;
 $19 = HEAP32[$pRet>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  HEAP32[$2>>2] = 7;
  $93 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$pRet>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$pRet>>2]|0;
 $24 = ((($23)) + 120|0);
 $25 = HEAP32[$pRet>>2]|0;
 $26 = ((($25)) + 8|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$4>>2]|0;
 $28 = HEAP32[$pRet>>2]|0;
 $29 = ((($28)) + 4|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$pRet>>2]|0;
 $31 = ((($30)) + 40|0);
 HEAP16[$31>>1] = -1;
 $32 = $7;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$pRet>>2]|0;
 $39 = ((($38)) + 16|0);
 $40 = $39;
 $41 = $40;
 HEAP32[$41>>2] = $34;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $37;
 $44 = HEAP32[$5>>2]|0;
 $45 = HEAP32[$pRet>>2]|0;
 $46 = ((($45)) + 108|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$pRet>>2]|0;
 $48 = ((($47)) + 48|0);
 HEAP8[$48>>0] = 1;
 $49 = HEAP32[$pRet>>2]|0;
 $50 = ((($49)) + 49|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$6>>2]|0;
 $52 = ($51|0)!=(0);
 $53 = $52 ? 2 : 0;
 $54 = $53&255;
 $55 = HEAP32[$pRet>>2]|0;
 $56 = ((($55)) + 43|0);
 HEAP8[$56>>0] = $54;
 HEAP32[$flags>>2] = 524294;
 $57 = HEAP32[$3>>2]|0;
 $58 = HEAP32[$5>>2]|0;
 $59 = HEAP32[$pRet>>2]|0;
 $60 = ((($59)) + 8|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$flags>>2]|0;
 $63 = (_sqlite3OsOpen($57,$58,$61,$62,$flags)|0);
 HEAP32[$rc>>2] = $63;
 $64 = HEAP32[$rc>>2]|0;
 $65 = ($64|0)==(0);
 if ($65) {
  $66 = HEAP32[$flags>>2]|0;
  $67 = $66 & 1;
  $68 = ($67|0)!=(0);
  if ($68) {
   $69 = HEAP32[$pRet>>2]|0;
   $70 = ((($69)) + 46|0);
   HEAP8[$70>>0] = 1;
  }
 }
 $71 = HEAP32[$rc>>2]|0;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$pRet>>2]|0;
  _walIndexClose($73,0);
  $74 = HEAP32[$pRet>>2]|0;
  $75 = ((($74)) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  (_sqlite3OsClose($76)|0);
  $77 = HEAP32[$pRet>>2]|0;
  _sqlite3_free($77);
 } else {
  $78 = HEAP32[$4>>2]|0;
  $79 = (_sqlite3OsDeviceCharacteristics($78)|0);
  HEAP32[$iDC>>2] = $79;
  $80 = HEAP32[$iDC>>2]|0;
  $81 = $80 & 1024;
  $82 = ($81|0)!=(0);
  if ($82) {
   $83 = HEAP32[$pRet>>2]|0;
   $84 = ((($83)) + 48|0);
   HEAP8[$84>>0] = 0;
  }
  $85 = HEAP32[$iDC>>2]|0;
  $86 = $85 & 4096;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$pRet>>2]|0;
   $89 = ((($88)) + 49|0);
   HEAP8[$89>>0] = 0;
  }
  $90 = HEAP32[$pRet>>2]|0;
  $91 = HEAP32[$8>>2]|0;
  HEAP32[$91>>2] = $90;
 }
 $92 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $92;
 $93 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _releasePageNotNull($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3PagerUnrefNotNull($3);
 STACKTOP = sp;return;
}
function _sqlite3WalBeginWriteTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 46|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  HEAP32[$0>>2] = 8;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_walLockExclusive($6,0,1)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $10;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 44|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = HEAP32[$1>>2]|0;
 $16 = (_walIndexHdr($15)|0);
 $17 = (_memcmp($14,$16,48)|0);
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$1>>2]|0;
  _walUnlockExclusive($19,0,1);
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 44|0);
  HEAP8[$21>>0] = 0;
  HEAP32[$rc>>2] = 517;
 }
 $22 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _zeroPage($pPage,$flags) {
 $pPage = $pPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $first = 0;
 var $hdr = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $data = sp + 4|0;
 $pBt = sp;
 $hdr = sp + 18|0;
 $first = sp + 16|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$data>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 5|0);
 $10 = HEAP8[$9>>0]|0;
 HEAP8[$hdr>>0] = $10;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 22|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 4;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP8[$hdr>>0]|0;
  $18 = $17&255;
  $19 = HEAP32[$data>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP32[$pBt>>2]|0;
  $22 = ((($21)) + 36|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$hdr>>0]|0;
  $25 = $24&255;
  $26 = (($23) - ($25))|0;
  _memset(($20|0),0,($26|0))|0;
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = $27&255;
 $29 = HEAP8[$hdr>>0]|0;
 $30 = $29&255;
 $31 = HEAP32[$data>>2]|0;
 $32 = (($31) + ($30)|0);
 HEAP8[$32>>0] = $28;
 $33 = HEAP8[$hdr>>0]|0;
 $34 = $33&255;
 $35 = HEAP32[$1>>2]|0;
 $36 = $35 & 8;
 $37 = ($36|0)==(0);
 $38 = $37 ? 12 : 8;
 $39 = (($34) + ($38))|0;
 $40 = $39&65535;
 HEAP16[$first>>1] = $40;
 $41 = HEAP8[$hdr>>0]|0;
 $42 = $41&255;
 $43 = (($42) + 1)|0;
 $44 = HEAP32[$data>>2]|0;
 $45 = (($44) + ($43)|0);
 ;HEAP8[$45>>0]=0|0;HEAP8[$45+1>>0]=0|0;HEAP8[$45+2>>0]=0|0;HEAP8[$45+3>>0]=0|0;
 $46 = HEAP8[$hdr>>0]|0;
 $47 = $46&255;
 $48 = (($47) + 7)|0;
 $49 = HEAP32[$data>>2]|0;
 $50 = (($49) + ($48)|0);
 HEAP8[$50>>0] = 0;
 $51 = HEAP32[$pBt>>2]|0;
 $52 = ((($51)) + 36|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = $53 >>> 8;
 $55 = $54&255;
 $56 = HEAP8[$hdr>>0]|0;
 $57 = $56&255;
 $58 = (($57) + 5)|0;
 $59 = HEAP32[$data>>2]|0;
 $60 = (($59) + ($58)|0);
 HEAP8[$60>>0] = $55;
 $61 = HEAP32[$pBt>>2]|0;
 $62 = ((($61)) + 36|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = $63&255;
 $65 = HEAP8[$hdr>>0]|0;
 $66 = $65&255;
 $67 = (($66) + 5)|0;
 $68 = HEAP32[$data>>2]|0;
 $69 = (($68) + ($67)|0);
 $70 = ((($69)) + 1|0);
 HEAP8[$70>>0] = $64;
 $71 = HEAP32[$pBt>>2]|0;
 $72 = ((($71)) + 36|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP16[$first>>1]|0;
 $75 = $74&65535;
 $76 = (($73) - ($75))|0;
 $77 = $76&65535;
 $78 = HEAP32[$0>>2]|0;
 $79 = ((($78)) + 16|0);
 HEAP16[$79>>1] = $77;
 $80 = HEAP32[$0>>2]|0;
 $81 = HEAP32[$1>>2]|0;
 (_decodeFlags($80,$81)|0);
 $82 = HEAP16[$first>>1]|0;
 $83 = HEAP32[$0>>2]|0;
 $84 = ((($83)) + 14|0);
 HEAP16[$84>>1] = $82;
 $85 = HEAP32[$pBt>>2]|0;
 $86 = ((($85)) + 36|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$data>>2]|0;
 $89 = (($88) + ($87)|0);
 $90 = HEAP32[$0>>2]|0;
 $91 = ((($90)) + 60|0);
 HEAP32[$91>>2] = $89;
 $92 = HEAP16[$first>>1]|0;
 $93 = $92&65535;
 $94 = HEAP32[$data>>2]|0;
 $95 = (($94) + ($93)|0);
 $96 = HEAP32[$0>>2]|0;
 $97 = ((($96)) + 64|0);
 HEAP32[$97>>2] = $95;
 $98 = HEAP32[$0>>2]|0;
 $99 = ((($98)) + 6|0);
 $100 = HEAP8[$99>>0]|0;
 $101 = $100&255;
 $102 = HEAP32[$data>>2]|0;
 $103 = (($102) + ($101)|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = ((($104)) + 68|0);
 HEAP32[$105>>2] = $103;
 $106 = HEAP32[$0>>2]|0;
 $107 = ((($106)) + 1|0);
 HEAP8[$107>>0] = 0;
 $108 = HEAP32[$pBt>>2]|0;
 $109 = ((($108)) + 32|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = (($110) - 1)|0;
 $112 = $111&65535;
 $113 = HEAP32[$0>>2]|0;
 $114 = ((($113)) + 20|0);
 HEAP16[$114>>1] = $112;
 $115 = HEAP32[$0>>2]|0;
 $116 = ((($115)) + 18|0);
 HEAP16[$116>>1] = 0;
 $117 = HEAP32[$0>>2]|0;
 HEAP8[$117>>0] = 1;
 STACKTOP = sp;return;
}
function _pagerOpenSavepoint($pPager,$nSavepoint) {
 $pPager = $pPager|0;
 $nSavepoint = $nSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $ii = 0, $nCurrent = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $nCurrent = sp + 8|0;
 $ii = sp + 4|0;
 $aNew = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $nSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 104|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nCurrent>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 100|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9*48)|0;
 $11 = (_sqlite3Realloc($8,$10,0)|0);
 HEAP32[$aNew>>2] = $11;
 $12 = HEAP32[$aNew>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  HEAP32[$0>>2] = 7;
  $116 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $14 = HEAP32[$nCurrent>>2]|0;
 $15 = HEAP32[$aNew>>2]|0;
 $16 = (($15) + (($14*48)|0)|0);
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$nCurrent>>2]|0;
 $19 = (($17) - ($18))|0;
 $20 = ($19*48)|0;
 _memset(($16|0),0,($20|0))|0;
 $21 = HEAP32[$aNew>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 100|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$nCurrent>>2]|0;
 HEAP32[$ii>>2] = $24;
 while(1) {
  $25 = HEAP32[$ii>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = ($25|0)<($26|0);
  if (!($27)) {
   label = 14;
   break;
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 28|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$ii>>2]|0;
  $32 = HEAP32[$aNew>>2]|0;
  $33 = (($32) + (($31*48)|0)|0);
  $34 = ((($33)) + 20|0);
  HEAP32[$34>>2] = $30;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 68|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(0|0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 80|0);
   $42 = $41;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = ($47|0)>(0);
   $49 = ($44>>>0)>(0);
   $50 = ($47|0)==(0);
   $51 = $50 & $49;
   $52 = $48 | $51;
   if ($52) {
    $53 = HEAP32[$1>>2]|0;
    $54 = ((($53)) + 80|0);
    $55 = $54;
    $56 = $55;
    $57 = HEAP32[$56>>2]|0;
    $58 = (($55) + 4)|0;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$ii>>2]|0;
    $62 = HEAP32[$aNew>>2]|0;
    $63 = (($62) + (($61*48)|0)|0);
    $64 = $63;
    $65 = $64;
    HEAP32[$65>>2] = $57;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = $60;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  if ((label|0) == 8) {
   label = 0;
   $68 = HEAP32[$1>>2]|0;
   $69 = ((($68)) + 156|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$ii>>2]|0;
   $72 = HEAP32[$aNew>>2]|0;
   $73 = (($72) + (($71*48)|0)|0);
   $74 = $73;
   $75 = $74;
   HEAP32[$75>>2] = $70;
   $76 = (($74) + 4)|0;
   $77 = $76;
   HEAP32[$77>>2] = 0;
  }
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 56|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = HEAP32[$ii>>2]|0;
  $82 = HEAP32[$aNew>>2]|0;
  $83 = (($82) + (($81*48)|0)|0);
  $84 = ((($83)) + 24|0);
  HEAP32[$84>>2] = $80;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 28|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (_sqlite3BitvecCreate($87)|0);
  $89 = HEAP32[$ii>>2]|0;
  $90 = HEAP32[$aNew>>2]|0;
  $91 = (($90) + (($89*48)|0)|0);
  $92 = ((($91)) + 16|0);
  HEAP32[$92>>2] = $88;
  $93 = HEAP32[$ii>>2]|0;
  $94 = HEAP32[$aNew>>2]|0;
  $95 = (($94) + (($93*48)|0)|0);
  $96 = ((($95)) + 16|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($97|0)!=(0|0);
  if (!($98)) {
   label = 10;
   break;
  }
  $99 = HEAP32[$1>>2]|0;
  $100 = (_pagerUseWal($99)|0);
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$1>>2]|0;
   $103 = ((($102)) + 216|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = HEAP32[$ii>>2]|0;
   $106 = HEAP32[$aNew>>2]|0;
   $107 = (($106) + (($105*48)|0)|0);
   $108 = ((($107)) + 28|0);
   _sqlite3WalSavepoint($104,$108);
  }
  $109 = HEAP32[$ii>>2]|0;
  $110 = (($109) + 1)|0;
  $111 = HEAP32[$1>>2]|0;
  $112 = ((($111)) + 104|0);
  HEAP32[$112>>2] = $110;
  $113 = HEAP32[$ii>>2]|0;
  $114 = (($113) + 1)|0;
  HEAP32[$ii>>2] = $114;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 7;
  $116 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 else if ((label|0) == 14) {
  $115 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $115;
  $116 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 return (0)|0;
}
function _sqlite3WalSavepoint($pWal,$aWalData) {
 $pWal = $pWal|0;
 $aWalData = $aWalData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $aWalData;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = ((($14)) + 24|0);
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 112|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 12|0);
 HEAP32[$24>>2] = $22;
 STACKTOP = sp;return;
}
function _sqlite3PagerDataVersion($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 108|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _btreePagecount($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3OsFetch($id,$0,$1,$iAmt,$pp) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $iAmt = $iAmt|0;
 $pp = $pp|0;
 var $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $iAmt;
 HEAP32[$5>>2] = $pp;
 $10 = HEAP32[$5>>2]|0;
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return 0;
}
function _pagerAcquireMapPage($pPager,$pgno,$pData,$ppPage) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 $pData = $pData|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $pData;
 HEAP32[$4>>2] = $ppPage;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 144|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 do {
  if ($8) {
   $10 = ((($9)) + 144|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$p>>2] = $11;
   $12 = HEAP32[$4>>2]|0;
   HEAP32[$12>>2] = $11;
   $13 = HEAP32[$p>>2]|0;
   $14 = ((($13)) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 144|0);
   HEAP32[$17>>2] = $15;
   $18 = HEAP32[$p>>2]|0;
   $19 = ((($18)) + 12|0);
   HEAP32[$19>>2] = 0;
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 148|0);
   $25 = HEAP16[$24>>1]|0;
   $26 = $25&65535;
   _memset(($22|0),0,($26|0))|0;
  } else {
   $27 = ((($9)) + 148|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = $28&65535;
   $30 = (40 + ($29))|0;
   $31 = (_sqlite3MallocZero($30,0)|0);
   HEAP32[$p>>2] = $31;
   $32 = HEAP32[$4>>2]|0;
   HEAP32[$32>>2] = $31;
   $33 = HEAP32[$p>>2]|0;
   $34 = ($33|0)==(0|0);
   if (!($34)) {
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 40|0);
    $50 = HEAP32[$p>>2]|0;
    $51 = ((($50)) + 8|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$p>>2]|0;
    $53 = ((($52)) + 24|0);
    HEAP16[$53>>1] = 64;
    $54 = HEAP32[$p>>2]|0;
    $55 = ((($54)) + 26|0);
    HEAP16[$55>>1] = 1;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$p>>2]|0;
    $58 = ((($57)) + 16|0);
    HEAP32[$58>>2] = $56;
    break;
   }
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 64|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = (($38) - 1)|0;
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 160|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)<(0);
   $44 = $43 << 31 >> 31;
   $45 = (___muldi3(($39|0),0,($42|0),($44|0))|0);
   $46 = tempRet0;
   $47 = HEAP32[$3>>2]|0;
   (_sqlite3OsUnfetch($37,$45,$46,$47)|0);
   HEAP32[$0>>2] = 7;
   $69 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($69|0);
  }
 } while(0);
 $59 = HEAP32[$2>>2]|0;
 $60 = HEAP32[$p>>2]|0;
 $61 = ((($60)) + 20|0);
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$3>>2]|0;
 $63 = HEAP32[$p>>2]|0;
 $64 = ((($63)) + 4|0);
 HEAP32[$64>>2] = $62;
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 128|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 HEAP32[$66>>2] = $68;
 HEAP32[$0>>2] = 0;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3PcacheFetchStress($pCache,$pgno,$ppPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $pPg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pPg = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 33|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(2);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3PcachePagecount($9)|0);
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)>($13|0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pPg>>2] = $17;
  while(1) {
   $18 = HEAP32[$pPg>>2]|0;
   $19 = ($18|0)!=(0|0);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$pPg>>2]|0;
   $21 = ((($20)) + 26|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)!=(0);
   if (!($24)) {
    $25 = HEAP32[$pPg>>2]|0;
    $26 = ((($25)) + 24|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27&65535;
    $29 = $28 & 8;
    $30 = ($29|0)!=(0);
    if (!($30)) {
     break;
    }
   }
   $31 = HEAP32[$pPg>>2]|0;
   $32 = ((($31)) + 36|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$pPg>>2] = $33;
  }
  $34 = HEAP32[$pPg>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$pPg>>2]|0;
  $38 = ($37|0)!=(0|0);
  L14: do {
   if (!($38)) {
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 4|0);
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$pPg>>2] = $41;
    while(1) {
     $42 = HEAP32[$pPg>>2]|0;
     $43 = ($42|0)!=(0|0);
     if (!($43)) {
      break L14;
     }
     $44 = HEAP32[$pPg>>2]|0;
     $45 = ((($44)) + 26|0);
     $46 = HEAP16[$45>>1]|0;
     $47 = $46 << 16 >> 16;
     $48 = ($47|0)!=(0);
     if (!($48)) {
      break L14;
     }
     $49 = HEAP32[$pPg>>2]|0;
     $50 = ((($49)) + 36|0);
     $51 = HEAP32[$50>>2]|0;
     HEAP32[$pPg>>2] = $51;
    }
   }
  } while(0);
  $52 = HEAP32[$pPg>>2]|0;
  $53 = ($52|0)!=(0|0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 36|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 40|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$pPg>>2]|0;
   $61 = (FUNCTION_TABLE_iii[$56 & 255]($59,$60)|0);
   HEAP32[$rc>>2] = $61;
   $62 = HEAP32[$rc>>2]|0;
   $63 = ($62|0)!=(0);
   $64 = HEAP32[$rc>>2]|0;
   $65 = ($64|0)!=(5);
   $or$cond = $63 & $65;
   if ($or$cond) {
    $66 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $66;
    $78 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
  }
 }
 $67 = HEAP32[(140)>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = ((($68)) + 44|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$2>>2]|0;
 $72 = (FUNCTION_TABLE_iiii[$67 & 255]($70,$71,2)|0);
 $73 = HEAP32[$3>>2]|0;
 HEAP32[$73>>2] = $72;
 $74 = HEAP32[$3>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = ($75|0)==(0|0);
 $77 = $76 ? 7 : 0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3PagerIsMemdb($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _pagerWriteLargeSector($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ii = 0, $nPage = 0, $nPageCount = 0, $nPagePerSector = 0;
 var $needSync = 0, $or$cond = 0, $pPage = 0, $pPage1 = 0, $pPager = 0, $pg = 0, $pg1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $rc = sp + 40|0;
 $nPageCount = sp + 36|0;
 $pg1 = sp + 32|0;
 $nPage = sp + 28|0;
 $ii = sp + 24|0;
 $needSync = sp + 20|0;
 $pPager = sp + 16|0;
 $nPagePerSector = sp + 12|0;
 $pg = sp + 8|0;
 $pPage = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nPage>>2] = 0;
 HEAP32[$needSync>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 156|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$pPager>>2]|0;
 $8 = ((($7)) + 160|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) / ($9>>>0))&-1;
 HEAP32[$nPagePerSector>>2] = $10;
 $11 = HEAP32[$pPager>>2]|0;
 $12 = ((($11)) + 21|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 | 4;
 $16 = $15&255;
 HEAP8[$12>>0] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) - 1)|0;
 $21 = HEAP32[$nPagePerSector>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = $22 ^ -1;
 $24 = $20 & $23;
 $25 = (($24) + 1)|0;
 HEAP32[$pg1>>2] = $25;
 $26 = HEAP32[$pPager>>2]|0;
 $27 = ((($26)) + 28|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$nPageCount>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$nPageCount>>2]|0;
 $33 = ($31>>>0)>($32>>>0);
 do {
  if ($33) {
   $34 = HEAP32[$0>>2]|0;
   $35 = ((($34)) + 20|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pg1>>2]|0;
   $38 = (($36) - ($37))|0;
   $39 = (($38) + 1)|0;
   HEAP32[$nPage>>2] = $39;
  } else {
   $40 = HEAP32[$pg1>>2]|0;
   $41 = HEAP32[$nPagePerSector>>2]|0;
   $42 = (($40) + ($41))|0;
   $43 = (($42) - 1)|0;
   $44 = HEAP32[$nPageCount>>2]|0;
   $45 = ($43>>>0)>($44>>>0);
   if ($45) {
    $46 = HEAP32[$nPageCount>>2]|0;
    $47 = (($46) + 1)|0;
    $48 = HEAP32[$pg1>>2]|0;
    $49 = (($47) - ($48))|0;
    HEAP32[$nPage>>2] = $49;
    break;
   } else {
    $50 = HEAP32[$nPagePerSector>>2]|0;
    HEAP32[$nPage>>2] = $50;
    break;
   }
  }
 } while(0);
 HEAP32[$ii>>2] = 0;
 while(1) {
  $51 = HEAP32[$ii>>2]|0;
  $52 = HEAP32[$nPage>>2]|0;
  $53 = ($51|0)<($52|0);
  $54 = HEAP32[$rc>>2]|0;
  $55 = ($54|0)==(0);
  $56 = $53 ? $55 : 0;
  if (!($56)) {
   break;
  }
  $57 = HEAP32[$pg1>>2]|0;
  $58 = HEAP32[$ii>>2]|0;
  $59 = (($57) + ($58))|0;
  HEAP32[$pg>>2] = $59;
  $60 = HEAP32[$pg>>2]|0;
  $61 = HEAP32[$0>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($60|0)==($63|0);
  if ($64) {
   label = 10;
  } else {
   $65 = HEAP32[$pPager>>2]|0;
   $66 = ((($65)) + 60|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = HEAP32[$pg>>2]|0;
   $69 = (_sqlite3BitvecTest($67,$68)|0);
   $70 = ($69|0)!=(0);
   if ($70) {
    $93 = HEAP32[$pPager>>2]|0;
    $94 = HEAP32[$pg>>2]|0;
    $95 = (_sqlite3PagerLookup($93,$94)|0);
    HEAP32[$pPage>>2] = $95;
    $96 = ($95|0)!=(0|0);
    if ($96) {
     $97 = HEAP32[$pPage>>2]|0;
     $98 = ((($97)) + 24|0);
     $99 = HEAP16[$98>>1]|0;
     $100 = $99&65535;
     $101 = $100 & 8;
     $102 = ($101|0)!=(0);
     if ($102) {
      HEAP32[$needSync>>2] = 1;
     }
     $103 = HEAP32[$pPage>>2]|0;
     _sqlite3PagerUnrefNotNull($103);
    }
   } else {
    label = 10;
   }
  }
  if ((label|0) == 10) {
   label = 0;
   $71 = HEAP32[$pg>>2]|0;
   $72 = HEAP32[832>>2]|0;
   $73 = HEAP32[$pPager>>2]|0;
   $74 = ((($73)) + 160|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = (($72|0) / ($75|0))&-1;
   $77 = (($76) + 1)|0;
   $78 = ($71|0)!=($77|0);
   if ($78) {
    $79 = HEAP32[$pPager>>2]|0;
    $80 = HEAP32[$pg>>2]|0;
    $81 = (_sqlite3PagerGet($79,$80,$pPage,0)|0);
    HEAP32[$rc>>2] = $81;
    $82 = HEAP32[$rc>>2]|0;
    $83 = ($82|0)==(0);
    if ($83) {
     $84 = HEAP32[$pPage>>2]|0;
     $85 = (_pager_write($84)|0);
     HEAP32[$rc>>2] = $85;
     $86 = HEAP32[$pPage>>2]|0;
     $87 = ((($86)) + 24|0);
     $88 = HEAP16[$87>>1]|0;
     $89 = $88&65535;
     $90 = $89 & 8;
     $91 = ($90|0)!=(0);
     if ($91) {
      HEAP32[$needSync>>2] = 1;
     }
     $92 = HEAP32[$pPage>>2]|0;
     _sqlite3PagerUnrefNotNull($92);
    }
   }
  }
  $104 = HEAP32[$ii>>2]|0;
  $105 = (($104) + 1)|0;
  HEAP32[$ii>>2] = $105;
 }
 $106 = HEAP32[$rc>>2]|0;
 $107 = ($106|0)==(0);
 $108 = HEAP32[$needSync>>2]|0;
 $109 = ($108|0)!=(0);
 $or$cond = $107 & $109;
 if (!($or$cond)) {
  $129 = HEAP32[$pPager>>2]|0;
  $130 = ((($129)) + 21|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = $131&255;
  $133 = $132 & -5;
  $134 = $133&255;
  HEAP8[$130>>0] = $134;
  $135 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $110 = HEAP32[$ii>>2]|0;
  $111 = HEAP32[$nPage>>2]|0;
  $112 = ($110|0)<($111|0);
  if (!($112)) {
   break;
  }
  $113 = HEAP32[$pPager>>2]|0;
  $114 = HEAP32[$pg1>>2]|0;
  $115 = HEAP32[$ii>>2]|0;
  $116 = (($114) + ($115))|0;
  $117 = (_sqlite3PagerLookup($113,$116)|0);
  HEAP32[$pPage1>>2] = $117;
  $118 = HEAP32[$pPage1>>2]|0;
  $119 = ($118|0)!=(0|0);
  if ($119) {
   $120 = HEAP32[$pPage1>>2]|0;
   $121 = ((($120)) + 24|0);
   $122 = HEAP16[$121>>1]|0;
   $123 = $122&65535;
   $124 = $123 | 8;
   $125 = $124&65535;
   HEAP16[$121>>1] = $125;
   $126 = HEAP32[$pPage1>>2]|0;
   _sqlite3PagerUnrefNotNull($126);
  }
  $127 = HEAP32[$ii>>2]|0;
  $128 = (($127) + 1)|0;
  HEAP32[$ii>>2] = $128;
 }
 $129 = HEAP32[$pPager>>2]|0;
 $130 = ((($129)) + 21|0);
 $131 = HEAP8[$130>>0]|0;
 $132 = $131&255;
 $133 = $132 & -5;
 $134 = $133&255;
 HEAP8[$130>>0] = $134;
 $135 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _pager_write($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $pPager = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pPager = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = ((($5)) + 17|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$pPager>>2]|0;
  $11 = (_pager_open_journal($10)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $14;
   $76 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($76|0);
  }
 }
 $15 = HEAP32[$1>>2]|0;
 _sqlite3PcacheMakeDirty($15);
 $16 = HEAP32[$pPager>>2]|0;
 $17 = ((($16)) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 do {
  if ($19) {
   $20 = HEAP32[$pPager>>2]|0;
   $21 = ((($20)) + 60|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 20|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (_sqlite3BitvecTestNotNull($22,$25)|0);
   $27 = ($26|0)==(0);
   if ($27) {
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 20|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$pPager>>2]|0;
    $32 = ((($31)) + 32|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30>>>0)<=($33>>>0);
    if (!($34)) {
     $40 = HEAP32[$pPager>>2]|0;
     $41 = ((($40)) + 17|0);
     $42 = HEAP8[$41>>0]|0;
     $43 = $42&255;
     $44 = ($43|0)!=(4);
     if (!($44)) {
      break;
     }
     $45 = HEAP32[$1>>2]|0;
     $46 = ((($45)) + 24|0);
     $47 = HEAP16[$46>>1]|0;
     $48 = $47&65535;
     $49 = $48 | 8;
     $50 = $49&65535;
     HEAP16[$46>>1] = $50;
     break;
    }
    $35 = HEAP32[$1>>2]|0;
    $36 = (_pagerAddPageToRollbackJournal($35)|0);
    HEAP32[$rc>>2] = $36;
    $37 = HEAP32[$rc>>2]|0;
    $38 = ($37|0)!=(0);
    if ($38) {
     $39 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $39;
     $76 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($76|0);
    }
   }
  }
 } while(0);
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 24|0);
 $53 = HEAP16[$52>>1]|0;
 $54 = $53&65535;
 $55 = $54 | 4;
 $56 = $55&65535;
 HEAP16[$52>>1] = $56;
 $57 = HEAP32[$pPager>>2]|0;
 $58 = ((($57)) + 104|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)>(0);
 if ($60) {
  $61 = HEAP32[$1>>2]|0;
  $62 = (_subjournalPageIfRequired($61)|0);
  HEAP32[$rc>>2] = $62;
 }
 $63 = HEAP32[$pPager>>2]|0;
 $64 = ((($63)) + 28|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($65>>>0)<($68>>>0);
 if ($69) {
  $70 = HEAP32[$1>>2]|0;
  $71 = ((($70)) + 20|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$pPager>>2]|0;
  $74 = ((($73)) + 28|0);
  HEAP32[$74>>2] = $72;
 }
 $75 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $75;
 $76 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _pager_open_journal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $flags = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $pVfs = sp + 4|0;
 $flags = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $89 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($89|0);
 }
 $11 = (_pagerUseWal($8)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 5|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(2);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 28|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3BitvecCreate($20)|0);
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 60|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 60|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)==(0|0);
   if ($27) {
    HEAP32[$0>>2] = 7;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 68|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   do {
    if (!($32)) {
     $33 = HEAP32[$1>>2]|0;
     $34 = ((($33)) + 5|0);
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     $37 = ($36|0)==(4);
     $38 = HEAP32[$1>>2]|0;
     if ($37) {
      $39 = ((($38)) + 68|0);
      $40 = HEAP32[$39>>2]|0;
      _sqlite3MemJournalOpen($40);
      break;
     }
     $41 = ((($38)) + 13|0);
     $42 = HEAP8[$41>>0]|0;
     $43 = $42&255;
     $44 = ($43|0)!=(0);
     $45 = $44 ? 4104 : 2048;
     $46 = 6 | $45;
     HEAP32[$flags>>2] = $46;
     $47 = HEAP32[$1>>2]|0;
     $48 = (_databaseIsUnmoved($47)|0);
     HEAP32[$rc>>2] = $48;
     $49 = HEAP32[$rc>>2]|0;
     $50 = ($49|0)==(0);
     if ($50) {
      $51 = HEAP32[$pVfs>>2]|0;
      $52 = HEAP32[$1>>2]|0;
      $53 = ((($52)) + 180|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = HEAP32[$1>>2]|0;
      $56 = ((($55)) + 68|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$flags>>2]|0;
      $59 = (_sqlite3OsOpen($51,$54,$57,$58,0)|0);
      HEAP32[$rc>>2] = $59;
     }
    }
   } while(0);
   $60 = HEAP32[$rc>>2]|0;
   $61 = ($60|0)==(0);
   if ($61) {
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 48|0);
    HEAP32[$63>>2] = 0;
    $64 = HEAP32[$1>>2]|0;
    $65 = ((($64)) + 80|0);
    $66 = $65;
    $67 = $66;
    HEAP32[$67>>2] = 0;
    $68 = (($66) + 4)|0;
    $69 = $68;
    HEAP32[$69>>2] = 0;
    $70 = HEAP32[$1>>2]|0;
    $71 = ((($70)) + 20|0);
    HEAP8[$71>>0] = 0;
    $72 = HEAP32[$1>>2]|0;
    $73 = ((($72)) + 88|0);
    $74 = $73;
    $75 = $74;
    HEAP32[$75>>2] = 0;
    $76 = (($74) + 4)|0;
    $77 = $76;
    HEAP32[$77>>2] = 0;
    $78 = HEAP32[$1>>2]|0;
    $79 = (_writeJournalHdr($78)|0);
    HEAP32[$rc>>2] = $79;
   }
  }
 }
 $80 = HEAP32[$rc>>2]|0;
 $81 = ($80|0)!=(0);
 $82 = HEAP32[$1>>2]|0;
 if ($81) {
  $83 = ((($82)) + 60|0);
  $84 = HEAP32[$83>>2]|0;
  _sqlite3BitvecDestroy($84);
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 60|0);
  HEAP32[$86>>2] = 0;
 } else {
  $87 = ((($82)) + 17|0);
  HEAP8[$87>>0] = 3;
 }
 $88 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _pagerAddPageToRollbackJournal($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $iOff = 0, $pData2 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pPager = sp + 20|0;
 $rc = sp + 16|0;
 $cksum = sp + 12|0;
 $pData2 = sp + 8|0;
 $iOff = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $iOff;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pData2>>2] = $19;
 $20 = HEAP32[$pPager>>2]|0;
 $21 = HEAP32[$pData2>>2]|0;
 $22 = (_pager_cksum($20,$21)|0);
 HEAP32[$cksum>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 24|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26 | 8;
 $28 = $27&65535;
 HEAP16[$24>>1] = $28;
 $29 = HEAP32[$pPager>>2]|0;
 $30 = ((($29)) + 68|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = $iOff;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 20|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (_write32bits($31,$34,$37,$40)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $44;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 }
 $45 = HEAP32[$pPager>>2]|0;
 $46 = ((($45)) + 68|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$pData2>>2]|0;
 $49 = HEAP32[$pPager>>2]|0;
 $50 = ((($49)) + 160|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = $iOff;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = (_i64Add(($54|0),($57|0),4,0)|0);
 $59 = tempRet0;
 $60 = (_sqlite3OsWrite($47,$48,$51,$58,$59)|0);
 HEAP32[$rc>>2] = $60;
 $61 = HEAP32[$rc>>2]|0;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $63;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 }
 $64 = HEAP32[$pPager>>2]|0;
 $65 = ((($64)) + 68|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $iOff;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = HEAP32[$pPager>>2]|0;
 $74 = ((($73)) + 160|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = ($75|0)<(0);
 $77 = $76 << 31 >> 31;
 $78 = (_i64Add(($69|0),($72|0),($75|0),($77|0))|0);
 $79 = tempRet0;
 $80 = (_i64Add(($78|0),($79|0),4,0)|0);
 $81 = tempRet0;
 $82 = HEAP32[$cksum>>2]|0;
 $83 = (_write32bits($66,$80,$81,$82)|0);
 HEAP32[$rc>>2] = $83;
 $84 = HEAP32[$rc>>2]|0;
 $85 = ($84|0)!=(0);
 if ($85) {
  $86 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $86;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 } else {
  $87 = HEAP32[$pPager>>2]|0;
  $88 = ((($87)) + 160|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = (8 + ($89))|0;
  $91 = ($90|0)<(0);
  $92 = $91 << 31 >> 31;
  $93 = HEAP32[$pPager>>2]|0;
  $94 = ((($93)) + 80|0);
  $95 = $94;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = (($95) + 4)|0;
  $99 = $98;
  $100 = HEAP32[$99>>2]|0;
  $101 = (_i64Add(($97|0),($100|0),($90|0),($92|0))|0);
  $102 = tempRet0;
  $103 = $94;
  $104 = $103;
  HEAP32[$104>>2] = $101;
  $105 = (($103) + 4)|0;
  $106 = $105;
  HEAP32[$106>>2] = $102;
  $107 = HEAP32[$pPager>>2]|0;
  $108 = ((($107)) + 48|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = (($109) + 1)|0;
  HEAP32[$108>>2] = $110;
  $111 = HEAP32[$pPager>>2]|0;
  $112 = ((($111)) + 60|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = HEAP32[$1>>2]|0;
  $115 = ((($114)) + 20|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (_sqlite3BitvecSet($113,$116)|0);
  HEAP32[$rc>>2] = $117;
  $118 = HEAP32[$pPager>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = ((($119)) + 20|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = (_addToSavepointBitvecs($118,$121)|0);
  $123 = HEAP32[$rc>>2]|0;
  $124 = $123 | $122;
  HEAP32[$rc>>2] = $124;
  $125 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $125;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 }
 return (0)|0;
}
function _databaseIsUnmoved($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bHasMoved = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $bHasMoved = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$bHasMoved>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 13|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3OsFileControl($12,20,$bHasMoved)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(12);
 if ($15) {
  HEAP32[$rc>>2] = 0;
 } else {
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)==(0);
  $18 = HEAP32[$bHasMoved>>2]|0;
  $19 = ($18|0)!=(0);
  $or$cond = $17 & $19;
  if ($or$cond) {
   HEAP32[$rc>>2] = 1032;
  }
 }
 $20 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _pager_incr_changecounter($pPager,$isDirectMode) {
 $pPager = $pPager|0;
 $isDirectMode = $isDirectMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $isDirectMode;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 19|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8>>>0)>(0);
 if (!($9)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3PagerGet($10,1,$pPgHdr,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$pPgHdr>>2]|0;
  $15 = (_sqlite3PagerWrite($14)|0);
  HEAP32[$rc>>2] = $15;
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = HEAP32[$pPgHdr>>2]|0;
  _pager_write_changecounter($18);
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 19|0);
  HEAP8[$20>>0] = 1;
 }
 $21 = HEAP32[$pPgHdr>>2]|0;
 _sqlite3PagerUnref($21);
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _writeMasterJournal($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $iHdrOff = 0, $jrnlSize = 0, $nMaster = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $rc = sp + 24|0;
 $nMaster = sp + 20|0;
 $iHdrOff = sp + 8|0;
 $jrnlSize = sp;
 $cksum = sp + 16|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$cksum>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 5|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(4);
  if (!($9)) {
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 68|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 20|0);
    HEAP8[$16>>0] = 1;
    HEAP32[$nMaster>>2] = 0;
    while(1) {
     $17 = HEAP32[$nMaster>>2]|0;
     $18 = HEAP32[$2>>2]|0;
     $19 = (($18) + ($17)|0);
     $20 = HEAP8[$19>>0]|0;
     $21 = ($20<<24>>24)!=(0);
     if (!($21)) {
      break;
     }
     $22 = HEAP32[$nMaster>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = (($23) + ($22)|0);
     $25 = HEAP8[$24>>0]|0;
     $26 = $25 << 24 >> 24;
     $27 = HEAP32[$cksum>>2]|0;
     $28 = (($27) + ($26))|0;
     HEAP32[$cksum>>2] = $28;
     $29 = HEAP32[$nMaster>>2]|0;
     $30 = (($29) + 1)|0;
     HEAP32[$nMaster>>2] = $30;
    }
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = ($33<<24>>24)!=(0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     $36 = (_journalHdrOffset($35)|0);
     $37 = tempRet0;
     $38 = HEAP32[$1>>2]|0;
     $39 = ((($38)) + 80|0);
     $40 = $39;
     $41 = $40;
     HEAP32[$41>>2] = $36;
     $42 = (($40) + 4)|0;
     $43 = $42;
     HEAP32[$43>>2] = $37;
    }
    $44 = HEAP32[$1>>2]|0;
    $45 = ((($44)) + 80|0);
    $46 = $45;
    $47 = $46;
    $48 = HEAP32[$47>>2]|0;
    $49 = (($46) + 4)|0;
    $50 = $49;
    $51 = HEAP32[$50>>2]|0;
    $52 = $iHdrOff;
    $53 = $52;
    HEAP32[$53>>2] = $48;
    $54 = (($52) + 4)|0;
    $55 = $54;
    HEAP32[$55>>2] = $51;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 68|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $iHdrOff;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59) + 4)|0;
    $63 = $62;
    $64 = HEAP32[$63>>2]|0;
    $65 = HEAP32[832>>2]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 160|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($65|0) / ($68|0))&-1;
    $70 = (($69) + 1)|0;
    $71 = (_write32bits($58,$61,$64,$70)|0);
    HEAP32[$rc>>2] = $71;
    $72 = (0)!=($71|0);
    if (!($72)) {
     $73 = HEAP32[$1>>2]|0;
     $74 = ((($73)) + 68|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = HEAP32[$2>>2]|0;
     $77 = HEAP32[$nMaster>>2]|0;
     $78 = $iHdrOff;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = (($78) + 4)|0;
     $82 = $81;
     $83 = HEAP32[$82>>2]|0;
     $84 = (_i64Add(($80|0),($83|0),4,0)|0);
     $85 = tempRet0;
     $86 = (_sqlite3OsWrite($75,$76,$77,$84,$85)|0);
     HEAP32[$rc>>2] = $86;
     $87 = (0)!=($86|0);
     if (!($87)) {
      $88 = HEAP32[$1>>2]|0;
      $89 = ((($88)) + 68|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = $iHdrOff;
      $92 = $91;
      $93 = HEAP32[$92>>2]|0;
      $94 = (($91) + 4)|0;
      $95 = $94;
      $96 = HEAP32[$95>>2]|0;
      $97 = (_i64Add(($93|0),($96|0),4,0)|0);
      $98 = tempRet0;
      $99 = HEAP32[$nMaster>>2]|0;
      $100 = ($99|0)<(0);
      $101 = $100 << 31 >> 31;
      $102 = (_i64Add(($97|0),($98|0),($99|0),($101|0))|0);
      $103 = tempRet0;
      $104 = HEAP32[$nMaster>>2]|0;
      $105 = (_write32bits($90,$102,$103,$104)|0);
      HEAP32[$rc>>2] = $105;
      $106 = (0)!=($105|0);
      if (!($106)) {
       $107 = HEAP32[$1>>2]|0;
       $108 = ((($107)) + 68|0);
       $109 = HEAP32[$108>>2]|0;
       $110 = $iHdrOff;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = (($110) + 4)|0;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (_i64Add(($112|0),($115|0),4,0)|0);
       $117 = tempRet0;
       $118 = HEAP32[$nMaster>>2]|0;
       $119 = ($118|0)<(0);
       $120 = $119 << 31 >> 31;
       $121 = (_i64Add(($116|0),($117|0),($118|0),($120|0))|0);
       $122 = tempRet0;
       $123 = (_i64Add(($121|0),($122|0),4,0)|0);
       $124 = tempRet0;
       $125 = HEAP32[$cksum>>2]|0;
       $126 = (_write32bits($109,$123,$124,$125)|0);
       HEAP32[$rc>>2] = $126;
       $127 = (0)!=($126|0);
       if (!($127)) {
        $128 = HEAP32[$1>>2]|0;
        $129 = ((($128)) + 68|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = $iHdrOff;
        $132 = $131;
        $133 = HEAP32[$132>>2]|0;
        $134 = (($131) + 4)|0;
        $135 = $134;
        $136 = HEAP32[$135>>2]|0;
        $137 = (_i64Add(($133|0),($136|0),4,0)|0);
        $138 = tempRet0;
        $139 = HEAP32[$nMaster>>2]|0;
        $140 = ($139|0)<(0);
        $141 = $140 << 31 >> 31;
        $142 = (_i64Add(($137|0),($138|0),($139|0),($141|0))|0);
        $143 = tempRet0;
        $144 = (_i64Add(($142|0),($143|0),8,0)|0);
        $145 = tempRet0;
        $146 = (_sqlite3OsWrite($130,19004,8,$144,$145)|0);
        HEAP32[$rc>>2] = $146;
        $147 = (0)!=($146|0);
        if (!($147)) {
         $149 = HEAP32[$nMaster>>2]|0;
         $150 = (($149) + 20)|0;
         $151 = ($150|0)<(0);
         $152 = $151 << 31 >> 31;
         $153 = HEAP32[$1>>2]|0;
         $154 = ((($153)) + 80|0);
         $155 = $154;
         $156 = $155;
         $157 = HEAP32[$156>>2]|0;
         $158 = (($155) + 4)|0;
         $159 = $158;
         $160 = HEAP32[$159>>2]|0;
         $161 = (_i64Add(($157|0),($160|0),($150|0),($152|0))|0);
         $162 = tempRet0;
         $163 = $154;
         $164 = $163;
         HEAP32[$164>>2] = $161;
         $165 = (($163) + 4)|0;
         $166 = $165;
         HEAP32[$166>>2] = $162;
         $167 = HEAP32[$1>>2]|0;
         $168 = ((($167)) + 68|0);
         $169 = HEAP32[$168>>2]|0;
         $170 = (_sqlite3OsFileSize($169,$jrnlSize)|0);
         HEAP32[$rc>>2] = $170;
         $171 = (0)==($170|0);
         if ($171) {
          $172 = $jrnlSize;
          $173 = $172;
          $174 = HEAP32[$173>>2]|0;
          $175 = (($172) + 4)|0;
          $176 = $175;
          $177 = HEAP32[$176>>2]|0;
          $178 = HEAP32[$1>>2]|0;
          $179 = ((($178)) + 80|0);
          $180 = $179;
          $181 = $180;
          $182 = HEAP32[$181>>2]|0;
          $183 = (($180) + 4)|0;
          $184 = $183;
          $185 = HEAP32[$184>>2]|0;
          $186 = ($177|0)>($185|0);
          $187 = ($174>>>0)>($182>>>0);
          $188 = ($177|0)==($185|0);
          $189 = $188 & $187;
          $190 = $186 | $189;
          if ($190) {
           $191 = HEAP32[$1>>2]|0;
           $192 = ((($191)) + 68|0);
           $193 = HEAP32[$192>>2]|0;
           $194 = HEAP32[$1>>2]|0;
           $195 = ((($194)) + 80|0);
           $196 = $195;
           $197 = $196;
           $198 = HEAP32[$197>>2]|0;
           $199 = (($196) + 4)|0;
           $200 = $199;
           $201 = HEAP32[$200>>2]|0;
           $202 = (_sqlite3OsTruncate($193,$198,$201)|0);
           HEAP32[$rc>>2] = $202;
          }
         }
         $203 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $203;
         $204 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($204|0);
        }
       }
      }
     }
    }
    $148 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $148;
    $204 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($204|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $204 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($204|0);
}
function _sqlite3PagerCommitPhaseTwo($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = ((($6)) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$0>>2] = $8;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $9 = ((($6)) + 17|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(2);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 5|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = ($21|0)==(1);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 17|0);
    HEAP8[$24>>0] = 1;
    HEAP32[$0>>2] = 0;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 108|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$26>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = (_pager_end_transaction($29,$33,1)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$rc>>2]|0;
 $37 = (_pager_error($35,$36)|0);
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _btreeClearHasContent($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BitvecDestroy($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 60|0);
 HEAP32[$5>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeEndTransaction($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$pBt>>2]|0;
 $7 = ((($6)) + 19|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)>(0);
 if ($12) {
  $13 = HEAP32[$db>>2]|0;
  $14 = ((($13)) + 152|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)>(1);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   _downgradeAllSharedCacheTableLocks($17);
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 8|0);
   HEAP8[$19>>0] = 1;
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$0>>2]|0;
  _clearAllSharedCacheTableLocks($25);
  $26 = HEAP32[$pBt>>2]|0;
  $27 = ((($26)) + 40|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + -1)|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$pBt>>2]|0;
  $31 = ((($30)) + 40|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (0)==($32|0);
  if ($33) {
   $34 = HEAP32[$pBt>>2]|0;
   $35 = ((($34)) + 20|0);
   HEAP8[$35>>0] = 0;
  }
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 8|0);
 HEAP8[$37>>0] = 0;
 $38 = HEAP32[$pBt>>2]|0;
 _unlockBtreeIfUnused($38);
 STACKTOP = sp;return;
}
function _downgradeAllSharedCacheTableLocks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pLock = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $pLock = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$pBt>>2]|0;
 $5 = ((($4)) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ($6|0)==($7|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$pBt>>2]|0;
 $10 = ((($9)) + 76|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 22|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -97;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$pBt>>2]|0;
 $18 = ((($17)) + 72|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pLock>>2] = $19;
 while(1) {
  $20 = HEAP32[$pLock>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pLock>>2]|0;
  $23 = ((($22)) + 8|0);
  HEAP8[$23>>0] = 1;
  $24 = HEAP32[$pLock>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pLock>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _clearAllSharedCacheTableLocks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pLock = 0, $ppIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBt = sp + 8|0;
 $ppIter = sp + 4|0;
 $pLock = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$pBt>>2]|0;
 $5 = ((($4)) + 72|0);
 HEAP32[$ppIter>>2] = $5;
 while(1) {
  $6 = HEAP32[$ppIter>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$ppIter>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pLock>>2] = $10;
  $11 = HEAP32[$pLock>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ($12|0)==($13|0);
  $15 = HEAP32[$pLock>>2]|0;
  $16 = ((($15)) + 12|0);
  if (!($14)) {
   HEAP32[$ppIter>>2] = $16;
   continue;
  }
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$ppIter>>2]|0;
  HEAP32[$18>>2] = $17;
  $19 = HEAP32[$pLock>>2]|0;
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)!=(1);
  if (!($22)) {
   continue;
  }
  $23 = HEAP32[$pLock>>2]|0;
  _sqlite3_free($23);
 }
 $24 = HEAP32[$pBt>>2]|0;
 $25 = ((($24)) + 76|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ($26|0)==($27|0);
 $29 = HEAP32[$pBt>>2]|0;
 if ($28) {
  $30 = ((($29)) + 76|0);
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$pBt>>2]|0;
  $32 = ((($31)) + 22|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = $34 & -97;
  $36 = $35&65535;
  HEAP16[$32>>1] = $36;
  STACKTOP = sp;return;
 }
 $37 = ((($29)) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)==(2);
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$pBt>>2]|0;
 $41 = ((($40)) + 22|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = $43 & -65;
 $45 = $44&65535;
 HEAP16[$41>>1] = $45;
 STACKTOP = sp;return;
}
function _autoVacuumCommit($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $iFree = 0;
 var $nFin = 0, $nFree = 0, $nOrig = 0, $or$cond = 0, $or$cond3 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $rc = sp + 20|0;
 $pPager = sp + 16|0;
 $nFin = sp + 12|0;
 $nFree = sp + 8|0;
 $iFree = sp + 4|0;
 $nOrig = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 _invalidateAllOverflowCache($4);
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 18|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 do {
  if (!($8)) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (_btreePagecount($9)|0);
   HEAP32[$nOrig>>2] = $10;
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$nOrig>>2]|0;
   $13 = (_ptrmapPageno($11,$12)|0);
   $14 = HEAP32[$nOrig>>2]|0;
   $15 = ($13|0)==($14|0);
   if (!($15)) {
    $16 = HEAP32[$nOrig>>2]|0;
    $17 = HEAP32[832>>2]|0;
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 32|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($17>>>0) / ($20>>>0))&-1;
    $22 = (($21) + 1)|0;
    $23 = ($16|0)==($22|0);
    if (!($23)) {
     $25 = HEAP32[$1>>2]|0;
     $26 = ((($25)) + 12|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = ((($27)) + 56|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = ((($29)) + 36|0);
     $31 = (_sqlite3Get4byte($30)|0);
     HEAP32[$nFree>>2] = $31;
     $32 = HEAP32[$1>>2]|0;
     $33 = HEAP32[$nOrig>>2]|0;
     $34 = HEAP32[$nFree>>2]|0;
     $35 = (_finalDbSize($32,$33,$34)|0);
     HEAP32[$nFin>>2] = $35;
     $36 = HEAP32[$nFin>>2]|0;
     $37 = HEAP32[$nOrig>>2]|0;
     $38 = ($36>>>0)>($37>>>0);
     if ($38) {
      $39 = (_sqlite3CorruptError(59231)|0);
      HEAP32[$0>>2] = $39;
      $98 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($98|0);
     }
     $40 = HEAP32[$nFin>>2]|0;
     $41 = HEAP32[$nOrig>>2]|0;
     $42 = ($40>>>0)<($41>>>0);
     if ($42) {
      $43 = HEAP32[$1>>2]|0;
      $44 = (_saveAllCursors($43,0,0)|0);
      HEAP32[$rc>>2] = $44;
     }
     $45 = HEAP32[$nOrig>>2]|0;
     HEAP32[$iFree>>2] = $45;
     while(1) {
      $46 = HEAP32[$iFree>>2]|0;
      $47 = HEAP32[$nFin>>2]|0;
      $48 = ($46>>>0)>($47>>>0);
      $49 = HEAP32[$rc>>2]|0;
      $50 = ($49|0)==(0);
      $51 = $48 ? $50 : 0;
      if (!($51)) {
       break;
      }
      $52 = HEAP32[$1>>2]|0;
      $53 = HEAP32[$nFin>>2]|0;
      $54 = HEAP32[$iFree>>2]|0;
      $55 = (_incrVacuumStep($52,$53,$54,1)|0);
      HEAP32[$rc>>2] = $55;
      $56 = HEAP32[$iFree>>2]|0;
      $57 = (($56) + -1)|0;
      HEAP32[$iFree>>2] = $57;
     }
     $58 = HEAP32[$rc>>2]|0;
     $59 = ($58|0)==(101);
     $60 = HEAP32[$rc>>2]|0;
     $61 = ($60|0)==(0);
     $or$cond = $59 | $61;
     $62 = HEAP32[$nFree>>2]|0;
     $63 = ($62>>>0)>(0);
     $or$cond3 = $or$cond & $63;
     if ($or$cond3) {
      $64 = HEAP32[$1>>2]|0;
      $65 = ((($64)) + 12|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = ((($66)) + 72|0);
      $68 = HEAP32[$67>>2]|0;
      $69 = (_sqlite3PagerWrite($68)|0);
      HEAP32[$rc>>2] = $69;
      $70 = HEAP32[$1>>2]|0;
      $71 = ((($70)) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ((($72)) + 56|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = ((($74)) + 32|0);
      _sqlite3Put4byte($75,0);
      $76 = HEAP32[$1>>2]|0;
      $77 = ((($76)) + 12|0);
      $78 = HEAP32[$77>>2]|0;
      $79 = ((($78)) + 56|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = ((($80)) + 36|0);
      _sqlite3Put4byte($81,0);
      $82 = HEAP32[$1>>2]|0;
      $83 = ((($82)) + 12|0);
      $84 = HEAP32[$83>>2]|0;
      $85 = ((($84)) + 56|0);
      $86 = HEAP32[$85>>2]|0;
      $87 = ((($86)) + 28|0);
      $88 = HEAP32[$nFin>>2]|0;
      _sqlite3Put4byte($87,$88);
      $89 = HEAP32[$1>>2]|0;
      $90 = ((($89)) + 19|0);
      HEAP8[$90>>0] = 1;
      $91 = HEAP32[$nFin>>2]|0;
      $92 = HEAP32[$1>>2]|0;
      $93 = ((($92)) + 44|0);
      HEAP32[$93>>2] = $91;
     }
     $94 = HEAP32[$rc>>2]|0;
     $95 = ($94|0)!=(0);
     if (!($95)) {
      break;
     }
     $96 = HEAP32[$pPager>>2]|0;
     (_sqlite3PagerRollback($96)|0);
     break;
    }
   }
   $24 = (_sqlite3CorruptError(59226)|0);
   HEAP32[$0>>2] = $24;
   $98 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($98|0);
  }
 } while(0);
 $97 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $97;
 $98 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($98|0);
}
function _invalidateAllOverflowCache($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 64|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  $10 = $9 & -5;
  $11 = $10&255;
  HEAP8[$7>>0] = $11;
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p>>2] = $14;
 }
 STACKTOP = sp;return;
}
function _ptrmapPageno($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrMap = 0, $nPagesPerMapPage = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nPagesPerMapPage = sp + 8|0;
 $iPtrMap = sp + 4|0;
 $ret = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3>>>0)<(2);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7>>>0) / 5)&-1;
 $9 = (($8) + 1)|0;
 HEAP32[$nPagesPerMapPage>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) - 2)|0;
 $12 = HEAP32[$nPagesPerMapPage>>2]|0;
 $13 = (($11>>>0) / ($12>>>0))&-1;
 HEAP32[$iPtrMap>>2] = $13;
 $14 = HEAP32[$iPtrMap>>2]|0;
 $15 = HEAP32[$nPagesPerMapPage>>2]|0;
 $16 = Math_imul($14, $15)|0;
 $17 = (($16) + 2)|0;
 HEAP32[$ret>>2] = $17;
 $18 = HEAP32[$ret>>2]|0;
 $19 = HEAP32[832>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 32|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($19>>>0) / ($22>>>0))&-1;
 $24 = (($23) + 1)|0;
 $25 = ($18|0)==($24|0);
 if ($25) {
  $26 = HEAP32[$ret>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$ret>>2] = $27;
 }
 $28 = HEAP32[$ret>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _finalDbSize($pBt,$nOrig,$nFree) {
 $pBt = $pBt|0;
 $nOrig = $nOrig|0;
 $nFree = $nFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nEntry = 0, $nFin = 0, $nPtrmap = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nEntry = sp + 8|0;
 $nPtrmap = sp + 4|0;
 $nFin = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $nOrig;
 HEAP32[$2>>2] = $nFree;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5>>>0) / 5)&-1;
 HEAP32[$nEntry>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($7) - ($8))|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (_ptrmapPageno($10,$11)|0);
 $13 = (($9) + ($12))|0;
 $14 = HEAP32[$nEntry>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = HEAP32[$nEntry>>2]|0;
 $17 = (($15>>>0) / ($16>>>0))&-1;
 HEAP32[$nPtrmap>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (($18) - ($19))|0;
 $21 = HEAP32[$nPtrmap>>2]|0;
 $22 = (($20) - ($21))|0;
 HEAP32[$nFin>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[832>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($24>>>0) / ($27>>>0))&-1;
 $29 = (($28) + 1)|0;
 $30 = ($23>>>0)>($29>>>0);
 if ($30) {
  $31 = HEAP32[$nFin>>2]|0;
  $32 = HEAP32[832>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($32>>>0) / ($35>>>0))&-1;
  $37 = (($36) + 1)|0;
  $38 = ($31>>>0)<($37>>>0);
  if ($38) {
   $39 = HEAP32[$nFin>>2]|0;
   $40 = (($39) + -1)|0;
   HEAP32[$nFin>>2] = $40;
  }
 }
 while(1) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$nFin>>2]|0;
  $43 = (_ptrmapPageno($41,$42)|0);
  $44 = HEAP32[$nFin>>2]|0;
  $45 = ($43|0)==($44|0);
  if ($45) {
   $56 = 1;
  } else {
   $46 = HEAP32[$nFin>>2]|0;
   $47 = HEAP32[832>>2]|0;
   $48 = HEAP32[$0>>2]|0;
   $49 = ((($48)) + 32|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($47>>>0) / ($50>>>0))&-1;
   $52 = (($51) + 1)|0;
   $53 = ($46|0)==($52|0);
   $56 = $53;
  }
  $54 = HEAP32[$nFin>>2]|0;
  if (!($56)) {
   break;
  }
  $55 = (($54) + -1)|0;
  HEAP32[$nFin>>2] = $55;
 }
 STACKTOP = sp;return ($54|0);
}
function _saveAllCursors($pBt,$iRoot,$pExcept) {
 $pBt = $pBt|0;
 $iRoot = $iRoot|0;
 $pExcept = $pExcept|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $iRoot;
 HEAP32[$3>>2] = $pExcept;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$p>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP32[$3>>2]|0;
  $11 = ($9|0)!=($10|0);
  if ($11) {
   $12 = HEAP32[$2>>2]|0;
   $13 = (0)==($12|0);
   if ($13) {
    break;
   }
   $14 = HEAP32[$p>>2]|0;
   $15 = ((($14)) + 52|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$2>>2]|0;
   $18 = ($16|0)==($17|0);
   if ($18) {
    break;
   }
  }
  $19 = HEAP32[$p>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$p>>2] = $21;
 }
 $22 = HEAP32[$p>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$p>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$3>>2]|0;
  $27 = (_saveCursorsOnList($24,$25,$26)|0);
  HEAP32[$0>>2] = $27;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $28 = HEAP32[$3>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$3>>2]|0;
  $31 = ((($30)) + 64|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = $33 & -33;
  $35 = $34&255;
  HEAP8[$31>>0] = $35;
 }
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _incrVacuumStep($pBt,$nFin,$iLastPg,$bCommit) {
 $pBt = $pBt|0;
 $nFin = $nFin|0;
 $iLastPg = $iLastPg|0;
 $bCommit = $bCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eMode = 0, $eType = 0, $iFreePg = 0, $iFreePg1 = 0, $iNear = 0, $iPtrPage = 0, $nFreeList = 0, $pFreePg = 0, $pFreePg2 = 0, $pLastPg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $nFreeList = sp + 32|0;
 $rc = sp + 28|0;
 $eType = sp + 57|0;
 $iPtrPage = sp + 24|0;
 $iFreePg = sp + 20|0;
 $pFreePg = sp + 16|0;
 $iFreePg1 = sp + 12|0;
 $pLastPg = sp + 8|0;
 $eMode = sp + 56|0;
 $iNear = sp + 4|0;
 $pFreePg2 = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $nFin;
 HEAP32[$3>>2] = $iLastPg;
 HEAP32[$4>>2] = $bCommit;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_ptrmapPageno($5,$6)|0);
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)==($8|0);
 do {
  if (!($9)) {
   $10 = HEAP32[$3>>2]|0;
   $11 = HEAP32[832>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 32|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = (($11>>>0) / ($14>>>0))&-1;
   $16 = (($15) + 1)|0;
   $17 = ($10|0)!=($16|0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 12|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = ((($20)) + 56|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ((($22)) + 36|0);
    $24 = (_sqlite3Get4byte($23)|0);
    HEAP32[$nFreeList>>2] = $24;
    $25 = HEAP32[$nFreeList>>2]|0;
    $26 = ($25|0)==(0);
    if ($26) {
     HEAP32[$0>>2] = 101;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = (_ptrmapGet($27,$28,$eType,$iPtrPage)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$rc>>2]|0;
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $32;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $33 = HEAP8[$eType>>0]|0;
    $34 = $33&255;
    $35 = ($34|0)==(1);
    if ($35) {
     $36 = (_sqlite3CorruptError(59063)|0);
     HEAP32[$0>>2] = $36;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $37 = HEAP8[$eType>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)==(2);
    if ($39) {
     $40 = HEAP32[$4>>2]|0;
     $41 = ($40|0)==(0);
     if (!($41)) {
      break;
     }
     $42 = HEAP32[$1>>2]|0;
     $43 = HEAP32[$3>>2]|0;
     $44 = (_allocateBtreePage($42,$pFreePg,$iFreePg,$43,1)|0);
     HEAP32[$rc>>2] = $44;
     $45 = HEAP32[$rc>>2]|0;
     $46 = ($45|0)!=(0);
     if (!($46)) {
      $48 = HEAP32[$pFreePg>>2]|0;
      _releasePage($48);
      break;
     }
     $47 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $47;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    HEAP8[$eMode>>0] = 0;
    HEAP32[$iNear>>2] = 0;
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$3>>2]|0;
    $51 = (_btreeGetPage($49,$50,$pLastPg,0)|0);
    HEAP32[$rc>>2] = $51;
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)!=(0);
    if ($53) {
     $54 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $54;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $55 = HEAP32[$4>>2]|0;
    $56 = ($55|0)==(0);
    if ($56) {
     HEAP8[$eMode>>0] = 2;
     $57 = HEAP32[$2>>2]|0;
     HEAP32[$iNear>>2] = $57;
    }
    while(1) {
     $58 = HEAP32[$1>>2]|0;
     $59 = HEAP32[$iNear>>2]|0;
     $60 = HEAP8[$eMode>>0]|0;
     $61 = (_allocateBtreePage($58,$pFreePg2,$iFreePg1,$59,$60)|0);
     HEAP32[$rc>>2] = $61;
     $62 = HEAP32[$rc>>2]|0;
     $63 = ($62|0)!=(0);
     if ($63) {
      label = 19;
      break;
     }
     $66 = HEAP32[$pFreePg2>>2]|0;
     _releasePage($66);
     $67 = HEAP32[$4>>2]|0;
     $68 = ($67|0)!=(0);
     if (!($68)) {
      break;
     }
     $69 = HEAP32[$iFreePg1>>2]|0;
     $70 = HEAP32[$2>>2]|0;
     $71 = ($69>>>0)>($70>>>0);
     if (!($71)) {
      break;
     }
    }
    if ((label|0) == 19) {
     $64 = HEAP32[$pLastPg>>2]|0;
     _releasePage($64);
     $65 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $65;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$pLastPg>>2]|0;
    $74 = HEAP8[$eType>>0]|0;
    $75 = HEAP32[$iPtrPage>>2]|0;
    $76 = HEAP32[$iFreePg1>>2]|0;
    $77 = HEAP32[$4>>2]|0;
    $78 = (_relocatePage($72,$73,$74,$75,$76,$77)|0);
    HEAP32[$rc>>2] = $78;
    $79 = HEAP32[$pLastPg>>2]|0;
    _releasePage($79);
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     $82 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $82;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
   }
  }
 } while(0);
 $83 = HEAP32[$4>>2]|0;
 $84 = ($83|0)==(0);
 if ($84) {
  while(1) {
   $85 = HEAP32[$3>>2]|0;
   $86 = (($85) + -1)|0;
   HEAP32[$3>>2] = $86;
   $87 = HEAP32[$3>>2]|0;
   $88 = HEAP32[832>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = ((($89)) + 32|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($88>>>0) / ($91>>>0))&-1;
   $93 = (($92) + 1)|0;
   $94 = ($87|0)==($93|0);
   if ($94) {
    continue;
   }
   $95 = HEAP32[$1>>2]|0;
   $96 = HEAP32[$3>>2]|0;
   $97 = (_ptrmapPageno($95,$96)|0);
   $98 = HEAP32[$3>>2]|0;
   $99 = ($97|0)==($98|0);
   if (!($99)) {
    break;
   }
  }
  $100 = HEAP32[$1>>2]|0;
  $101 = ((($100)) + 19|0);
  HEAP8[$101>>0] = 1;
  $102 = HEAP32[$3>>2]|0;
  $103 = HEAP32[$1>>2]|0;
  $104 = ((($103)) + 44|0);
  HEAP32[$104>>2] = $102;
 }
 HEAP32[$0>>2] = 0;
 $105 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($105|0);
}
function _saveCursorsOnList($p,$iRoot,$pExcept) {
 $p = $p|0;
 $iRoot = $iRoot|0;
 $pExcept = $pExcept|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iRoot;
 HEAP32[$3>>2] = $pExcept;
 L1: while(1) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$3>>2]|0;
  $6 = ($4|0)!=($5|0);
  do {
   if ($6) {
    $7 = HEAP32[$2>>2]|0;
    $8 = (0)==($7|0);
    if (!($8)) {
     $9 = HEAP32[$1>>2]|0;
     $10 = ((($9)) + 52|0);
     $11 = HEAP32[$10>>2]|0;
     $12 = HEAP32[$2>>2]|0;
     $13 = ($11|0)==($12|0);
     if (!($13)) {
      break;
     }
    }
    $14 = HEAP32[$1>>2]|0;
    $15 = ((($14)) + 66|0);
    $16 = HEAP8[$15>>0]|0;
    $17 = $16&255;
    $18 = ($17|0)==(1);
    if (!($18)) {
     $19 = HEAP32[$1>>2]|0;
     $20 = ((($19)) + 66|0);
     $21 = HEAP8[$20>>0]|0;
     $22 = $21&255;
     $23 = ($22|0)==(2);
     if (!($23)) {
      $29 = HEAP32[$1>>2]|0;
      _btreeReleaseAllCursorPages($29);
      break;
     }
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = (_saveCursorPosition($24)|0);
    HEAP32[$rc>>2] = $25;
    $26 = HEAP32[$rc>>2]|0;
    $27 = (0)!=($26|0);
    if ($27) {
     label = 8;
     break L1;
    }
   }
  } while(0);
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$1>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 8) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _saveCursorPosition($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 66|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 66|0);
  HEAP8[$7>>0] = 1;
 } else {
  $8 = ((($6)) + 60|0);
  HEAP32[$8>>2] = 0;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_saveCursorKey($9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  _btreeReleaseAllCursorPages($13);
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 66|0);
  HEAP8[$15>>0] = 3;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = $19 & -15;
 $21 = $20&255;
 HEAP8[$17>>0] = $21;
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _btreeReleaseAllCursorPages($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 68|0);
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($1|0)<=($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 120|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _releasePage($11);
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$i>>2] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 68|0);
 HEAP8[$19>>0] = -1;
 STACKTOP = sp;return;
}
function _saveCursorKey($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pKey = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pKey = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = (_sqlite3BtreeKeySize($1,$3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 69|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = (0)==($8|0);
 if (!($9)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 40|0);
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3Malloc($14,$17)|0);
 HEAP32[$pKey>>2] = $18;
 $19 = HEAP32[$pKey>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  HEAP32[$rc>>2] = 7;
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 40|0);
 $24 = $23;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$pKey>>2]|0;
 $31 = (_sqlite3BtreeKey($21,0,$26,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)==(0);
 $34 = HEAP32[$pKey>>2]|0;
 if ($33) {
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 48|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 } else {
  _sqlite3_free($34);
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3BtreeKeySize($pCur,$pSize) {
 $pCur = $pCur|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 _getCellInfo($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeKey($pCur,$offset,$amt,$pBuf) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $pBuf;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_accessPayload($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _getCellInfo($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iPage = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = ((($2)) + 18|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = ($5|0)==(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 HEAP32[$iPage>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 | 2;
 $16 = $15&255;
 HEAP8[$12>>0] = $16;
 $17 = HEAP32[$iPage>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 120|0);
 $20 = (($19) + ($17<<2)|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$iPage>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 80|0);
 $25 = (($24) + ($22<<1)|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 16|0);
 _btreeParseCell($21,$27,$29);
 STACKTOP = sp;return;
}
function _btreeParseCell($pPage,$iCell,$pInfo) {
 $pPage = $pPage|0;
 $iCell = $iCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $iCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 80|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = HEAP32[$1>>2]|0;
 $15 = $14<<1;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + ($15)|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 << 8;
 $23 = HEAP32[$1>>2]|0;
 $24 = $23<<1;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 64|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24)|0);
 $29 = ((($28)) + 1|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $22 | $31;
 $33 = $13 & $32;
 $34 = (($9) + ($33)|0);
 $35 = HEAP32[$2>>2]|0;
 FUNCTION_TABLE_viii[$5 & 255]($6,$34,$35);
 STACKTOP = sp;return;
}
function _accessPayload($pCur,$offset,$amt,$pBuf,$eOp) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 $eOp = $eOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $a1 = 0, $aNew = 0;
 var $aPayload = 0, $iIdx = 0, $nOvfl = 0, $nextPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $ovflSize = 0, $pBt = 0, $pDbPage = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $aPayload = sp + 44|0;
 $rc = sp + 40|0;
 $iIdx = sp + 36|0;
 $pPage = sp + 32|0;
 $pBt = sp + 28|0;
 $a = sp + 24|0;
 $ovflSize = sp + 20|0;
 $nextPage = sp + 16|0;
 $nOvfl = sp + 12|0;
 $aNew = sp + 8|0;
 $a1 = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $offset;
 HEAP32[$3>>2] = $amt;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $eOp;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iIdx>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 68|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 120|0);
 $12 = (($11) + ($9<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pPage>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pBt>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 _getCellInfo($17);
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = ((($19)) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$aPayload>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 16|0);
 $24 = ((($23)) + 16|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = HEAP32[$aPayload>>2]|0;
 $28 = (($27) + ($26)|0);
 $29 = HEAP32[$pBt>>2]|0;
 $30 = ((($29)) + 36|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$pPage>>2]|0;
 $33 = ((($32)) + 56|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31)|0);
 $36 = ($28>>>0)>($35>>>0);
 if ($36) {
  $37 = (_sqlite3CorruptError(60043)|0);
  HEAP32[$0>>2] = $37;
  $276 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($276|0);
 }
 $38 = HEAP32[$2>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 16|0);
 $41 = ((($40)) + 16|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = ($38>>>0)<($43>>>0);
 if ($44) {
  $45 = HEAP32[$3>>2]|0;
  HEAP32[$a>>2] = $45;
  $46 = HEAP32[$a>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = (($46) + ($47))|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 16|0);
  $51 = ((($50)) + 16|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = ($48>>>0)>($53>>>0);
  if ($54) {
   $55 = HEAP32[$1>>2]|0;
   $56 = ((($55)) + 16|0);
   $57 = ((($56)) + 16|0);
   $58 = HEAP16[$57>>1]|0;
   $59 = $58&65535;
   $60 = HEAP32[$2>>2]|0;
   $61 = (($59) - ($60))|0;
   HEAP32[$a>>2] = $61;
  }
  $62 = HEAP32[$2>>2]|0;
  $63 = HEAP32[$aPayload>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP32[$4>>2]|0;
  $66 = HEAP32[$a>>2]|0;
  $67 = HEAP32[$5>>2]|0;
  $68 = $67 & 1;
  $69 = HEAP32[$pPage>>2]|0;
  $70 = ((($69)) + 72|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (_copyPayload($64,$65,$66,$68,$71)|0);
  HEAP32[$rc>>2] = $72;
  HEAP32[$2>>2] = 0;
  $73 = HEAP32[$a>>2]|0;
  $74 = HEAP32[$4>>2]|0;
  $75 = (($74) + ($73)|0);
  HEAP32[$4>>2] = $75;
  $76 = HEAP32[$a>>2]|0;
  $77 = HEAP32[$3>>2]|0;
  $78 = (($77) - ($76))|0;
  HEAP32[$3>>2] = $78;
 } else {
  $79 = HEAP32[$1>>2]|0;
  $80 = ((($79)) + 16|0);
  $81 = ((($80)) + 16|0);
  $82 = HEAP16[$81>>1]|0;
  $83 = $82&65535;
  $84 = HEAP32[$2>>2]|0;
  $85 = (($84) - ($83))|0;
  HEAP32[$2>>2] = $85;
 }
 $86 = HEAP32[$rc>>2]|0;
 $87 = ($86|0)==(0);
 $88 = HEAP32[$3>>2]|0;
 $89 = ($88>>>0)>(0);
 $or$cond = $87 & $89;
 L12: do {
  if ($or$cond) {
   $90 = HEAP32[$pBt>>2]|0;
   $91 = ((($90)) + 36|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = (($92) - 4)|0;
   HEAP32[$ovflSize>>2] = $93;
   $94 = HEAP32[$1>>2]|0;
   $95 = ((($94)) + 16|0);
   $96 = ((($95)) + 16|0);
   $97 = HEAP16[$96>>1]|0;
   $98 = $97&65535;
   $99 = HEAP32[$aPayload>>2]|0;
   $100 = (($99) + ($98)|0);
   $101 = (_sqlite3Get4byte($100)|0);
   HEAP32[$nextPage>>2] = $101;
   $102 = HEAP32[$5>>2]|0;
   $103 = ($102|0)!=(2);
   if ($103) {
    $104 = HEAP32[$1>>2]|0;
    $105 = ((($104)) + 64|0);
    $106 = HEAP8[$105>>0]|0;
    $107 = $106&255;
    $108 = $107 & 4;
    $109 = ($108|0)==(0);
    if ($109) {
     $110 = HEAP32[$1>>2]|0;
     $111 = ((($110)) + 16|0);
     $112 = ((($111)) + 12|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = HEAP32[$1>>2]|0;
     $115 = ((($114)) + 16|0);
     $116 = ((($115)) + 16|0);
     $117 = HEAP16[$116>>1]|0;
     $118 = $117&65535;
     $119 = (($113) - ($118))|0;
     $120 = HEAP32[$ovflSize>>2]|0;
     $121 = (($119) + ($120))|0;
     $122 = (($121) - 1)|0;
     $123 = HEAP32[$ovflSize>>2]|0;
     $124 = (($122>>>0) / ($123>>>0))&-1;
     HEAP32[$nOvfl>>2] = $124;
     $125 = HEAP32[$nOvfl>>2]|0;
     $126 = HEAP32[$1>>2]|0;
     $127 = ((($126)) + 56|0);
     $128 = HEAP32[$127>>2]|0;
     $129 = ($125|0)>($128|0);
     do {
      if ($129) {
       $130 = HEAP32[$1>>2]|0;
       $131 = ((($130)) + 12|0);
       $132 = HEAP32[$131>>2]|0;
       $133 = HEAP32[$nOvfl>>2]|0;
       $134 = $133<<1;
       $135 = $134<<2;
       $136 = (_sqlite3Realloc($132,$135,0)|0);
       HEAP32[$aNew>>2] = $136;
       $137 = HEAP32[$aNew>>2]|0;
       $138 = ($137|0)==(0|0);
       if ($138) {
        HEAP32[$rc>>2] = 7;
        break;
       } else {
        $139 = HEAP32[$nOvfl>>2]|0;
        $140 = $139<<1;
        $141 = HEAP32[$1>>2]|0;
        $142 = ((($141)) + 56|0);
        HEAP32[$142>>2] = $140;
        $143 = HEAP32[$aNew>>2]|0;
        $144 = HEAP32[$1>>2]|0;
        $145 = ((($144)) + 12|0);
        HEAP32[$145>>2] = $143;
        break;
       }
      }
     } while(0);
     $146 = HEAP32[$rc>>2]|0;
     $147 = ($146|0)==(0);
     if ($147) {
      $148 = HEAP32[$1>>2]|0;
      $149 = ((($148)) + 12|0);
      $150 = HEAP32[$149>>2]|0;
      $151 = HEAP32[$nOvfl>>2]|0;
      $152 = $151<<2;
      _memset(($150|0),0,($152|0))|0;
      $153 = HEAP32[$1>>2]|0;
      $154 = ((($153)) + 64|0);
      $155 = HEAP8[$154>>0]|0;
      $156 = $155&255;
      $157 = $156 | 4;
      $158 = $157&255;
      HEAP8[$154>>0] = $158;
     }
    }
   }
   $159 = HEAP32[$1>>2]|0;
   $160 = ((($159)) + 64|0);
   $161 = HEAP8[$160>>0]|0;
   $162 = $161&255;
   $163 = $162 & 4;
   $164 = ($163|0)!=(0);
   if ($164) {
    $165 = HEAP32[$2>>2]|0;
    $166 = HEAP32[$ovflSize>>2]|0;
    $167 = (($165>>>0) / ($166>>>0))&-1;
    $168 = HEAP32[$1>>2]|0;
    $169 = ((($168)) + 12|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = (($170) + ($167<<2)|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = ($172|0)!=(0);
    if ($173) {
     $174 = HEAP32[$2>>2]|0;
     $175 = HEAP32[$ovflSize>>2]|0;
     $176 = (($174>>>0) / ($175>>>0))&-1;
     HEAP32[$iIdx>>2] = $176;
     $177 = HEAP32[$iIdx>>2]|0;
     $178 = HEAP32[$1>>2]|0;
     $179 = ((($178)) + 12|0);
     $180 = HEAP32[$179>>2]|0;
     $181 = (($180) + ($177<<2)|0);
     $182 = HEAP32[$181>>2]|0;
     HEAP32[$nextPage>>2] = $182;
     $183 = HEAP32[$2>>2]|0;
     $184 = HEAP32[$ovflSize>>2]|0;
     $185 = (($183>>>0) % ($184>>>0))&-1;
     HEAP32[$2>>2] = $185;
    }
   }
   while(1) {
    $186 = HEAP32[$rc>>2]|0;
    $187 = ($186|0)==(0);
    $188 = HEAP32[$3>>2]|0;
    $189 = ($188>>>0)>(0);
    $or$cond3 = $187 & $189;
    $190 = HEAP32[$nextPage>>2]|0;
    $191 = ($190|0)!=(0);
    $or$cond7 = $or$cond3 & $191;
    if (!($or$cond7)) {
     break L12;
    }
    $192 = HEAP32[$1>>2]|0;
    $193 = ((($192)) + 64|0);
    $194 = HEAP8[$193>>0]|0;
    $195 = $194&255;
    $196 = $195 & 4;
    $197 = ($196|0)!=(0);
    if ($197) {
     $198 = HEAP32[$nextPage>>2]|0;
     $199 = HEAP32[$iIdx>>2]|0;
     $200 = HEAP32[$1>>2]|0;
     $201 = ((($200)) + 12|0);
     $202 = HEAP32[$201>>2]|0;
     $203 = (($202) + ($199<<2)|0);
     HEAP32[$203>>2] = $198;
    }
    $204 = HEAP32[$2>>2]|0;
    $205 = HEAP32[$ovflSize>>2]|0;
    $206 = ($204>>>0)>=($205>>>0);
    if ($206) {
     $207 = HEAP32[$iIdx>>2]|0;
     $208 = (($207) + 1)|0;
     $209 = HEAP32[$1>>2]|0;
     $210 = ((($209)) + 12|0);
     $211 = HEAP32[$210>>2]|0;
     $212 = (($211) + ($208<<2)|0);
     $213 = HEAP32[$212>>2]|0;
     $214 = ($213|0)!=(0);
     if ($214) {
      $215 = HEAP32[$iIdx>>2]|0;
      $216 = (($215) + 1)|0;
      $217 = HEAP32[$1>>2]|0;
      $218 = ((($217)) + 12|0);
      $219 = HEAP32[$218>>2]|0;
      $220 = (($219) + ($216<<2)|0);
      $221 = HEAP32[$220>>2]|0;
      HEAP32[$nextPage>>2] = $221;
     } else {
      $222 = HEAP32[$pBt>>2]|0;
      $223 = HEAP32[$nextPage>>2]|0;
      $224 = (_getOverflowPage($222,$223,0,$nextPage)|0);
      HEAP32[$rc>>2] = $224;
     }
     $225 = HEAP32[$ovflSize>>2]|0;
     $226 = HEAP32[$2>>2]|0;
     $227 = (($226) - ($225))|0;
     HEAP32[$2>>2] = $227;
    } else {
     $228 = HEAP32[$3>>2]|0;
     HEAP32[$a1>>2] = $228;
     $229 = HEAP32[$a1>>2]|0;
     $230 = HEAP32[$2>>2]|0;
     $231 = (($229) + ($230))|0;
     $232 = HEAP32[$ovflSize>>2]|0;
     $233 = ($231>>>0)>($232>>>0);
     if ($233) {
      $234 = HEAP32[$ovflSize>>2]|0;
      $235 = HEAP32[$2>>2]|0;
      $236 = (($234) - ($235))|0;
      HEAP32[$a1>>2] = $236;
     }
     $237 = HEAP32[$pBt>>2]|0;
     $238 = HEAP32[$237>>2]|0;
     $239 = HEAP32[$nextPage>>2]|0;
     $240 = HEAP32[$5>>2]|0;
     $241 = $240 & 1;
     $242 = ($241|0)==(0);
     $243 = $242 ? 2 : 0;
     $244 = (_sqlite3PagerGet($238,$239,$pDbPage,$243)|0);
     HEAP32[$rc>>2] = $244;
     $245 = HEAP32[$rc>>2]|0;
     $246 = ($245|0)==(0);
     if ($246) {
      $247 = HEAP32[$pDbPage>>2]|0;
      $248 = (_sqlite3PagerGetData($247)|0);
      HEAP32[$aPayload>>2] = $248;
      $249 = HEAP32[$aPayload>>2]|0;
      $250 = (_sqlite3Get4byte($249)|0);
      HEAP32[$nextPage>>2] = $250;
      $251 = HEAP32[$2>>2]|0;
      $252 = (($251) + 4)|0;
      $253 = HEAP32[$aPayload>>2]|0;
      $254 = (($253) + ($252)|0);
      $255 = HEAP32[$4>>2]|0;
      $256 = HEAP32[$a1>>2]|0;
      $257 = HEAP32[$5>>2]|0;
      $258 = $257 & 1;
      $259 = HEAP32[$pDbPage>>2]|0;
      $260 = (_copyPayload($254,$255,$256,$258,$259)|0);
      HEAP32[$rc>>2] = $260;
      $261 = HEAP32[$pDbPage>>2]|0;
      _sqlite3PagerUnref($261);
      HEAP32[$2>>2] = 0;
     }
     $262 = HEAP32[$a1>>2]|0;
     $263 = HEAP32[$3>>2]|0;
     $264 = (($263) - ($262))|0;
     HEAP32[$3>>2] = $264;
     $265 = HEAP32[$a1>>2]|0;
     $266 = HEAP32[$4>>2]|0;
     $267 = (($266) + ($265)|0);
     HEAP32[$4>>2] = $267;
    }
    $268 = HEAP32[$iIdx>>2]|0;
    $269 = (($268) + 1)|0;
    HEAP32[$iIdx>>2] = $269;
   }
  }
 } while(0);
 $270 = HEAP32[$rc>>2]|0;
 $271 = ($270|0)==(0);
 $272 = HEAP32[$3>>2]|0;
 $273 = ($272>>>0)>(0);
 $or$cond5 = $271 & $273;
 if ($or$cond5) {
  $274 = (_sqlite3CorruptError(60200)|0);
  HEAP32[$0>>2] = $274;
  $276 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($276|0);
 } else {
  $275 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $275;
  $276 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($276|0);
 }
 return (0)|0;
}
function _copyPayload($pPayload,$pBuf,$nByte,$eOp,$pDbPage) {
 $pPayload = $pPayload|0;
 $pBuf = $pBuf|0;
 $nByte = $nByte|0;
 $eOp = $eOp|0;
 $pDbPage = $pDbPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPayload;
 HEAP32[$2>>2] = $pBuf;
 HEAP32[$3>>2] = $nByte;
 HEAP32[$4>>2] = $eOp;
 HEAP32[$5>>2] = $pDbPage;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = HEAP32[$5>>2]|0;
   $9 = (_sqlite3PagerWrite($8)|0);
   HEAP32[$rc>>2] = $9;
   $10 = HEAP32[$rc>>2]|0;
   $11 = ($10|0)!=(0);
   if (!($11)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = HEAP32[$2>>2]|0;
    $15 = HEAP32[$3>>2]|0;
    _memcpy(($13|0),($14|0),($15|0))|0;
    break;
   }
   $12 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $12;
   $19 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($19|0);
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$3>>2]|0;
   _memcpy(($16|0),($17|0),($18|0))|0;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _getOverflowPage($pBt,$ovfl,$ppPage,$pPgnoNext) {
 $pBt = $pBt|0;
 $ovfl = $ovfl|0;
 $ppPage = $ppPage|0;
 $pPgnoNext = $pPgnoNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $iGuess = 0, $next = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $next = sp + 16|0;
 $pPage = sp + 12|0;
 $rc = sp + 8|0;
 $pgno = sp + 4|0;
 $iGuess = sp;
 $eType = sp + 36|0;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $ovfl;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $pPgnoNext;
 HEAP32[$next>>2] = 0;
 HEAP32[$pPage>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 1)|0;
  HEAP32[$iGuess>>2] = $9;
  while(1) {
   $10 = HEAP32[$0>>2]|0;
   $11 = HEAP32[$iGuess>>2]|0;
   $12 = (_ptrmapPageno($10,$11)|0);
   $13 = HEAP32[$iGuess>>2]|0;
   $14 = ($12|0)==($13|0);
   if ($14) {
    $64 = 1;
   } else {
    $15 = HEAP32[$iGuess>>2]|0;
    $16 = HEAP32[832>>2]|0;
    $17 = HEAP32[$0>>2]|0;
    $18 = ((($17)) + 32|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = (($16>>>0) / ($19>>>0))&-1;
    $21 = (($20) + 1)|0;
    $22 = ($15|0)==($21|0);
    $64 = $22;
   }
   $23 = HEAP32[$iGuess>>2]|0;
   if (!($64)) {
    break;
   }
   $24 = (($23) + 1)|0;
   HEAP32[$iGuess>>2] = $24;
  }
  $25 = HEAP32[$0>>2]|0;
  $26 = (_btreePagecount($25)|0);
  $27 = ($23>>>0)<=($26>>>0);
  if ($27) {
   $28 = HEAP32[$0>>2]|0;
   $29 = HEAP32[$iGuess>>2]|0;
   $30 = (_ptrmapGet($28,$29,$eType,$pgno)|0);
   HEAP32[$rc>>2] = $30;
   $31 = HEAP32[$rc>>2]|0;
   $32 = ($31|0)==(0);
   if ($32) {
    $33 = HEAP8[$eType>>0]|0;
    $34 = $33&255;
    $35 = ($34|0)==(4);
    if ($35) {
     $36 = HEAP32[$pgno>>2]|0;
     $37 = HEAP32[$1>>2]|0;
     $38 = ($36|0)==($37|0);
     if ($38) {
      $39 = HEAP32[$iGuess>>2]|0;
      HEAP32[$next>>2] = $39;
      HEAP32[$rc>>2] = 101;
     }
    }
   }
  }
 }
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(0);
 if ($41) {
  $42 = HEAP32[$0>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$2>>2]|0;
  $45 = ($44|0)==(0|0);
  $46 = $45 ? 2 : 0;
  $47 = (_btreeGetPage($42,$43,$pPage,$46)|0);
  HEAP32[$rc>>2] = $47;
  $48 = HEAP32[$rc>>2]|0;
  $49 = ($48|0)==(0);
  if ($49) {
   $50 = HEAP32[$pPage>>2]|0;
   $51 = ((($50)) + 56|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3Get4byte($52)|0);
   HEAP32[$next>>2] = $53;
  }
 }
 $54 = HEAP32[$next>>2]|0;
 $55 = HEAP32[$3>>2]|0;
 HEAP32[$55>>2] = $54;
 $56 = HEAP32[$2>>2]|0;
 $57 = ($56|0)!=(0|0);
 $58 = HEAP32[$pPage>>2]|0;
 if ($57) {
  $59 = HEAP32[$2>>2]|0;
  HEAP32[$59>>2] = $58;
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(101);
  $62 = HEAP32[$rc>>2]|0;
  $63 = $61 ? 0 : $62;
  STACKTOP = sp;return ($63|0);
 } else {
  _releasePage($58);
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(101);
  $62 = HEAP32[$rc>>2]|0;
  $63 = $61 ? 0 : $62;
  STACKTOP = sp;return ($63|0);
 }
 return (0)|0;
}
function _ptrmapGet($pBt,$key,$pEType,$pPgno) {
 $pBt = $pBt|0;
 $key = $key|0;
 $pEType = $pEType|0;
 $pPgno = $pPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrmap = 0, $offset = 0, $pDbPage = 0, $pPtrmap = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pDbPage = sp + 16|0;
 $iPtrmap = sp + 12|0;
 $pPtrmap = sp + 8|0;
 $offset = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $key;
 HEAP32[$3>>2] = $pEType;
 HEAP32[$4>>2] = $pPgno;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_ptrmapPageno($5,$6)|0);
 HEAP32[$iPtrmap>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$iPtrmap>>2]|0;
 $11 = (_sqlite3PagerGet($9,$10,$pDbPage,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $15 = HEAP32[$pDbPage>>2]|0;
 $16 = (_sqlite3PagerGetData($15)|0);
 HEAP32[$pPtrmap>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$iPtrmap>>2]|0;
 $19 = (($17) - ($18))|0;
 $20 = (($19) - 1)|0;
 $21 = ($20*5)|0;
 HEAP32[$offset>>2] = $21;
 $22 = HEAP32[$offset>>2]|0;
 $23 = ($22|0)<(0);
 if ($23) {
  $24 = HEAP32[$pDbPage>>2]|0;
  _sqlite3PagerUnref($24);
  $25 = (_sqlite3CorruptError(56557)|0);
  HEAP32[$0>>2] = $25;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $26 = HEAP32[$offset>>2]|0;
 $27 = HEAP32[$pPtrmap>>2]|0;
 $28 = (($27) + ($26)|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = HEAP32[$3>>2]|0;
 HEAP8[$30>>0] = $29;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$offset>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = HEAP32[$pPtrmap>>2]|0;
  $36 = (($35) + ($34)|0);
  $37 = (_sqlite3Get4byte($36)|0);
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$38>>2] = $37;
 }
 $39 = HEAP32[$pDbPage>>2]|0;
 _sqlite3PagerUnref($39);
 $40 = HEAP32[$3>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($42|0)<(1);
 if (!($43)) {
  $44 = HEAP32[$3>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = ($46|0)>(5);
  if (!($47)) {
   HEAP32[$0>>2] = 0;
   $49 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 }
 $48 = (_sqlite3CorruptError(56565)|0);
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _allocateBtreePage($pBt,$ppPage,$pPgno,$nearby,$eMode) {
 $pBt = $pBt|0;
 $ppPage = $ppPage|0;
 $pPgno = $pPgno|0;
 $nearby = $nearby|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0;
 var $bNoContent = 0, $closest = 0, $d2 = 0, $dist = 0, $eType = 0, $i = 0, $iNewTrunk = 0, $iPage = 0, $iTrunk = 0, $k = 0, $mxPage = 0, $n = 0, $nSearch = 0, $noContent = 0, $or$cond = 0, $pNewTrunk = 0, $pPage1 = 0, $pPg = 0, $pPrevTrunk = 0, $pTrunk = 0;
 var $rc = 0, $searchList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 102|0;
 $pPage1 = sp + 76|0;
 $rc = sp + 72|0;
 $n = sp + 68|0;
 $k = sp + 64|0;
 $pTrunk = sp + 60|0;
 $pPrevTrunk = sp + 56|0;
 $mxPage = sp + 52|0;
 $iTrunk = sp + 48|0;
 $searchList = sp + 101|0;
 $nSearch = sp + 44|0;
 $eType = sp + 100|0;
 $pNewTrunk = sp + 40|0;
 $iNewTrunk = sp + 36|0;
 $closest = sp + 32|0;
 $iPage = sp + 28|0;
 $aData = sp + 24|0;
 $i = sp + 20|0;
 $dist = sp + 16|0;
 $d2 = sp + 12|0;
 $noContent = sp + 8|0;
 $bNoContent = sp + 4|0;
 $pPg = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $pPgno;
 HEAP32[$4>>2] = $nearby;
 HEAP8[$5>>0] = $eMode;
 HEAP32[$pTrunk>>2] = 0;
 HEAP32[$pPrevTrunk>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPage1>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_btreePagecount($9)|0);
 HEAP32[$mxPage>>2] = $10;
 $11 = HEAP32[$pPage1>>2]|0;
 $12 = ((($11)) + 56|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 36|0);
 $15 = (_sqlite3Get4byte($14)|0);
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$n>>2]|0;
 $17 = HEAP32[$mxPage>>2]|0;
 $18 = ($16>>>0)>=($17>>>0);
 if ($18) {
  $19 = (_sqlite3CorruptError(61115)|0);
  HEAP32[$0>>2] = $19;
  $458 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($458|0);
 }
 $20 = HEAP32[$n>>2]|0;
 $21 = ($20>>>0)>(0);
 L5: do {
  if ($21) {
   HEAP8[$searchList>>0] = 0;
   HEAP32[$nSearch>>2] = 0;
   $22 = HEAP8[$5>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)==(1);
   do {
    if ($24) {
     $25 = HEAP32[$4>>2]|0;
     $26 = HEAP32[$mxPage>>2]|0;
     $27 = ($25>>>0)<=($26>>>0);
     if ($27) {
      $28 = HEAP32[$1>>2]|0;
      $29 = HEAP32[$4>>2]|0;
      $30 = (_ptrmapGet($28,$29,$eType,0)|0);
      HEAP32[$rc>>2] = $30;
      $31 = HEAP32[$rc>>2]|0;
      $32 = ($31|0)!=(0);
      if ($32) {
       $33 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $33;
       $458 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($458|0);
      } else {
       $34 = HEAP8[$eType>>0]|0;
       $35 = $34&255;
       $36 = ($35|0)==(2);
       if (!($36)) {
        break;
       }
       HEAP8[$searchList>>0] = 1;
       break;
      }
     }
    } else {
     $37 = HEAP8[$5>>0]|0;
     $38 = $37&255;
     $39 = ($38|0)==(2);
     if ($39) {
      HEAP8[$searchList>>0] = 1;
     }
    }
   } while(0);
   $40 = HEAP32[$pPage1>>2]|0;
   $41 = ((($40)) + 72|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (_sqlite3PagerWrite($42)|0);
   HEAP32[$rc>>2] = $43;
   $44 = HEAP32[$rc>>2]|0;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $46;
    $458 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($458|0);
   }
   $47 = HEAP32[$pPage1>>2]|0;
   $48 = ((($47)) + 56|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ((($49)) + 36|0);
   $51 = HEAP32[$n>>2]|0;
   $52 = (($51) - 1)|0;
   _sqlite3Put4byte($50,$52);
   L22: while(1) {
    $53 = HEAP32[$pTrunk>>2]|0;
    HEAP32[$pPrevTrunk>>2] = $53;
    $54 = HEAP32[$pPrevTrunk>>2]|0;
    $55 = ($54|0)!=(0|0);
    if ($55) {
     $56 = HEAP32[$pPrevTrunk>>2]|0;
     $57 = ((($56)) + 56|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = (_sqlite3Get4byte($58)|0);
     HEAP32[$iTrunk>>2] = $59;
    } else {
     $60 = HEAP32[$pPage1>>2]|0;
     $61 = ((($60)) + 56|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = ((($62)) + 32|0);
     $64 = (_sqlite3Get4byte($63)|0);
     HEAP32[$iTrunk>>2] = $64;
    }
    $65 = HEAP32[$iTrunk>>2]|0;
    $66 = HEAP32[$mxPage>>2]|0;
    $67 = ($65>>>0)>($66>>>0);
    if ($67) {
     label = 20;
    } else {
     $68 = HEAP32[$nSearch>>2]|0;
     $69 = (($68) + 1)|0;
     HEAP32[$nSearch>>2] = $69;
     $70 = HEAP32[$n>>2]|0;
     $71 = ($68>>>0)>($70>>>0);
     if ($71) {
      label = 20;
     } else {
      $73 = HEAP32[$1>>2]|0;
      $74 = HEAP32[$iTrunk>>2]|0;
      $75 = (_btreeGetUnusedPage($73,$74,$pTrunk,0)|0);
      HEAP32[$rc>>2] = $75;
     }
    }
    if ((label|0) == 20) {
     label = 0;
     $72 = (_sqlite3CorruptError(61171)|0);
     HEAP32[$rc>>2] = $72;
    }
    $76 = HEAP32[$rc>>2]|0;
    $77 = ($76|0)!=(0);
    if ($77) {
     label = 23;
     break;
    }
    $78 = HEAP32[$pTrunk>>2]|0;
    $79 = ((($78)) + 56|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ((($80)) + 4|0);
    $82 = (_sqlite3Get4byte($81)|0);
    HEAP32[$k>>2] = $82;
    $83 = HEAP32[$k>>2]|0;
    $84 = ($83|0)!=(0);
    $85 = HEAP8[$searchList>>0]|0;
    $86 = ($85<<24>>24)!=(0);
    $or$cond = $84 | $86;
    L35: do {
     if ($or$cond) {
      $104 = HEAP32[$k>>2]|0;
      $105 = HEAP32[$1>>2]|0;
      $106 = ((($105)) + 36|0);
      $107 = HEAP32[$106>>2]|0;
      $108 = (($107>>>0) / 4)&-1;
      $109 = (($108) - 2)|0;
      $110 = ($104>>>0)>($109>>>0);
      if ($110) {
       label = 28;
       break L22;
      }
      $112 = HEAP8[$searchList>>0]|0;
      $113 = $112&255;
      $114 = ($113|0)!=(0);
      do {
       if ($114) {
        $115 = HEAP32[$4>>2]|0;
        $116 = HEAP32[$iTrunk>>2]|0;
        $117 = ($115|0)==($116|0);
        if (!($117)) {
         $118 = HEAP32[$iTrunk>>2]|0;
         $119 = HEAP32[$4>>2]|0;
         $120 = ($118>>>0)<($119>>>0);
         if (!($120)) {
          break;
         }
         $121 = HEAP8[$5>>0]|0;
         $122 = $121&255;
         $123 = ($122|0)==(2);
         if (!($123)) {
          break;
         }
        }
        $124 = HEAP32[$iTrunk>>2]|0;
        $125 = HEAP32[$3>>2]|0;
        HEAP32[$125>>2] = $124;
        $126 = HEAP32[$pTrunk>>2]|0;
        $127 = HEAP32[$2>>2]|0;
        HEAP32[$127>>2] = $126;
        HEAP8[$searchList>>0] = 0;
        $128 = HEAP32[$pTrunk>>2]|0;
        $129 = ((($128)) + 72|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = (_sqlite3PagerWrite($130)|0);
        HEAP32[$rc>>2] = $131;
        $132 = HEAP32[$rc>>2]|0;
        $133 = ($132|0)!=(0);
        if ($133) {
         break L5;
        }
        $134 = HEAP32[$k>>2]|0;
        $135 = ($134|0)==(0);
        do {
         if ($135) {
          $136 = HEAP32[$pPrevTrunk>>2]|0;
          $137 = ($136|0)!=(0|0);
          if (!($137)) {
           $138 = HEAP32[$pPage1>>2]|0;
           $139 = ((($138)) + 56|0);
           $140 = HEAP32[$139>>2]|0;
           $141 = ((($140)) + 32|0);
           $142 = HEAP32[$pTrunk>>2]|0;
           $143 = ((($142)) + 56|0);
           $144 = HEAP32[$143>>2]|0;
           ;HEAP8[$141>>0]=HEAP8[$144>>0]|0;HEAP8[$141+1>>0]=HEAP8[$144+1>>0]|0;HEAP8[$141+2>>0]=HEAP8[$144+2>>0]|0;HEAP8[$141+3>>0]=HEAP8[$144+3>>0]|0;
           break;
          }
          $145 = HEAP32[$pPrevTrunk>>2]|0;
          $146 = ((($145)) + 72|0);
          $147 = HEAP32[$146>>2]|0;
          $148 = (_sqlite3PagerWrite($147)|0);
          HEAP32[$rc>>2] = $148;
          $149 = HEAP32[$rc>>2]|0;
          $150 = ($149|0)!=(0);
          if ($150) {
           break L5;
          }
          $151 = HEAP32[$pPrevTrunk>>2]|0;
          $152 = ((($151)) + 56|0);
          $153 = HEAP32[$152>>2]|0;
          $154 = HEAP32[$pTrunk>>2]|0;
          $155 = ((($154)) + 56|0);
          $156 = HEAP32[$155>>2]|0;
          ;HEAP8[$153>>0]=HEAP8[$156>>0]|0;HEAP8[$153+1>>0]=HEAP8[$156+1>>0]|0;HEAP8[$153+2>>0]=HEAP8[$156+2>>0]|0;HEAP8[$153+3>>0]=HEAP8[$156+3>>0]|0;
         } else {
          $157 = HEAP32[$pTrunk>>2]|0;
          $158 = ((($157)) + 56|0);
          $159 = HEAP32[$158>>2]|0;
          $160 = ((($159)) + 8|0);
          $161 = (_sqlite3Get4byte($160)|0);
          HEAP32[$iNewTrunk>>2] = $161;
          $162 = HEAP32[$iNewTrunk>>2]|0;
          $163 = HEAP32[$mxPage>>2]|0;
          $164 = ($162>>>0)>($163>>>0);
          if ($164) {
           label = 40;
           break L22;
          }
          $166 = HEAP32[$1>>2]|0;
          $167 = HEAP32[$iNewTrunk>>2]|0;
          $168 = (_btreeGetUnusedPage($166,$167,$pNewTrunk,0)|0);
          HEAP32[$rc>>2] = $168;
          $169 = HEAP32[$rc>>2]|0;
          $170 = ($169|0)!=(0);
          if ($170) {
           break L5;
          }
          $171 = HEAP32[$pNewTrunk>>2]|0;
          $172 = ((($171)) + 72|0);
          $173 = HEAP32[$172>>2]|0;
          $174 = (_sqlite3PagerWrite($173)|0);
          HEAP32[$rc>>2] = $174;
          $175 = HEAP32[$rc>>2]|0;
          $176 = ($175|0)!=(0);
          $177 = HEAP32[$pNewTrunk>>2]|0;
          if ($176) {
           label = 43;
           break L22;
          }
          $178 = ((($177)) + 56|0);
          $179 = HEAP32[$178>>2]|0;
          $180 = HEAP32[$pTrunk>>2]|0;
          $181 = ((($180)) + 56|0);
          $182 = HEAP32[$181>>2]|0;
          ;HEAP8[$179>>0]=HEAP8[$182>>0]|0;HEAP8[$179+1>>0]=HEAP8[$182+1>>0]|0;HEAP8[$179+2>>0]=HEAP8[$182+2>>0]|0;HEAP8[$179+3>>0]=HEAP8[$182+3>>0]|0;
          $183 = HEAP32[$pNewTrunk>>2]|0;
          $184 = ((($183)) + 56|0);
          $185 = HEAP32[$184>>2]|0;
          $186 = ((($185)) + 4|0);
          $187 = HEAP32[$k>>2]|0;
          $188 = (($187) - 1)|0;
          _sqlite3Put4byte($186,$188);
          $189 = HEAP32[$pNewTrunk>>2]|0;
          $190 = ((($189)) + 56|0);
          $191 = HEAP32[$190>>2]|0;
          $192 = ((($191)) + 8|0);
          $193 = HEAP32[$pTrunk>>2]|0;
          $194 = ((($193)) + 56|0);
          $195 = HEAP32[$194>>2]|0;
          $196 = ((($195)) + 12|0);
          $197 = HEAP32[$k>>2]|0;
          $198 = (($197) - 1)|0;
          $199 = $198<<2;
          _memcpy(($192|0),($196|0),($199|0))|0;
          $200 = HEAP32[$pNewTrunk>>2]|0;
          _releasePage($200);
          $201 = HEAP32[$pPrevTrunk>>2]|0;
          $202 = ($201|0)!=(0|0);
          if (!($202)) {
           $203 = HEAP32[$pPage1>>2]|0;
           $204 = ((($203)) + 56|0);
           $205 = HEAP32[$204>>2]|0;
           $206 = ((($205)) + 32|0);
           $207 = HEAP32[$iNewTrunk>>2]|0;
           _sqlite3Put4byte($206,$207);
           break;
          }
          $208 = HEAP32[$pPrevTrunk>>2]|0;
          $209 = ((($208)) + 72|0);
          $210 = HEAP32[$209>>2]|0;
          $211 = (_sqlite3PagerWrite($210)|0);
          HEAP32[$rc>>2] = $211;
          $212 = HEAP32[$rc>>2]|0;
          $213 = ($212|0)!=(0);
          if ($213) {
           break L5;
          }
          $214 = HEAP32[$pPrevTrunk>>2]|0;
          $215 = ((($214)) + 56|0);
          $216 = HEAP32[$215>>2]|0;
          $217 = HEAP32[$iNewTrunk>>2]|0;
          _sqlite3Put4byte($216,$217);
         }
        } while(0);
        HEAP32[$pTrunk>>2] = 0;
        break L35;
       }
      } while(0);
      $218 = HEAP32[$k>>2]|0;
      $219 = ($218>>>0)>(0);
      if ($219) {
       $220 = HEAP32[$pTrunk>>2]|0;
       $221 = ((($220)) + 56|0);
       $222 = HEAP32[$221>>2]|0;
       HEAP32[$aData>>2] = $222;
       $223 = HEAP32[$4>>2]|0;
       $224 = ($223>>>0)>(0);
       HEAP32[$closest>>2] = 0;
       L62: do {
        if ($224) {
         $225 = HEAP8[$5>>0]|0;
         $226 = $225&255;
         $227 = ($226|0)==(2);
         if ($227) {
          HEAP32[$i>>2] = 0;
          while(1) {
           $228 = HEAP32[$i>>2]|0;
           $229 = HEAP32[$k>>2]|0;
           $230 = ($228>>>0)<($229>>>0);
           if (!($230)) {
            break L62;
           }
           $231 = HEAP32[$i>>2]|0;
           $232 = $231<<2;
           $233 = (8 + ($232))|0;
           $234 = HEAP32[$aData>>2]|0;
           $235 = (($234) + ($233)|0);
           $236 = (_sqlite3Get4byte($235)|0);
           HEAP32[$iPage>>2] = $236;
           $237 = HEAP32[$iPage>>2]|0;
           $238 = HEAP32[$4>>2]|0;
           $239 = ($237>>>0)<=($238>>>0);
           $240 = HEAP32[$i>>2]|0;
           if ($239) {
            break;
           }
           $241 = (($240) + 1)|0;
           HEAP32[$i>>2] = $241;
          }
          HEAP32[$closest>>2] = $240;
          break;
         } else {
          $242 = HEAP32[$aData>>2]|0;
          $243 = ((($242)) + 8|0);
          $244 = (_sqlite3Get4byte($243)|0);
          $245 = HEAP32[$4>>2]|0;
          $246 = (($244) - ($245))|0;
          $247 = (_sqlite3AbsInt32($246)|0);
          HEAP32[$dist>>2] = $247;
          HEAP32[$i>>2] = 1;
          while(1) {
           $248 = HEAP32[$i>>2]|0;
           $249 = HEAP32[$k>>2]|0;
           $250 = ($248>>>0)<($249>>>0);
           if (!($250)) {
            break L62;
           }
           $251 = HEAP32[$i>>2]|0;
           $252 = $251<<2;
           $253 = (8 + ($252))|0;
           $254 = HEAP32[$aData>>2]|0;
           $255 = (($254) + ($253)|0);
           $256 = (_sqlite3Get4byte($255)|0);
           $257 = HEAP32[$4>>2]|0;
           $258 = (($256) - ($257))|0;
           $259 = (_sqlite3AbsInt32($258)|0);
           HEAP32[$d2>>2] = $259;
           $260 = HEAP32[$d2>>2]|0;
           $261 = HEAP32[$dist>>2]|0;
           $262 = ($260|0)<($261|0);
           if ($262) {
            $263 = HEAP32[$i>>2]|0;
            HEAP32[$closest>>2] = $263;
            $264 = HEAP32[$d2>>2]|0;
            HEAP32[$dist>>2] = $264;
           }
           $265 = HEAP32[$i>>2]|0;
           $266 = (($265) + 1)|0;
           HEAP32[$i>>2] = $266;
          }
         }
        }
       } while(0);
       $267 = HEAP32[$closest>>2]|0;
       $268 = $267<<2;
       $269 = (8 + ($268))|0;
       $270 = HEAP32[$aData>>2]|0;
       $271 = (($270) + ($269)|0);
       $272 = (_sqlite3Get4byte($271)|0);
       HEAP32[$iPage>>2] = $272;
       $273 = HEAP32[$iPage>>2]|0;
       $274 = HEAP32[$mxPage>>2]|0;
       $275 = ($273>>>0)>($274>>>0);
       if ($275) {
        label = 63;
        break L22;
       }
       $277 = HEAP8[$searchList>>0]|0;
       $278 = ($277<<24>>24)!=(0);
       if ($278) {
        $279 = HEAP32[$iPage>>2]|0;
        $280 = HEAP32[$4>>2]|0;
        $281 = ($279|0)==($280|0);
        if (!($281)) {
         $282 = HEAP32[$iPage>>2]|0;
         $283 = HEAP32[$4>>2]|0;
         $284 = ($282>>>0)<($283>>>0);
         if (!($284)) {
          break;
         }
         $285 = HEAP8[$5>>0]|0;
         $286 = $285&255;
         $287 = ($286|0)==(2);
         if (!($287)) {
          break;
         }
        }
       }
       $288 = HEAP32[$iPage>>2]|0;
       $289 = HEAP32[$3>>2]|0;
       HEAP32[$289>>2] = $288;
       $290 = HEAP32[$pTrunk>>2]|0;
       $291 = ((($290)) + 72|0);
       $292 = HEAP32[$291>>2]|0;
       $293 = (_sqlite3PagerWrite($292)|0);
       HEAP32[$rc>>2] = $293;
       $294 = HEAP32[$rc>>2]|0;
       $295 = ($294|0)!=(0);
       if ($295) {
        break L5;
       }
       $296 = HEAP32[$closest>>2]|0;
       $297 = HEAP32[$k>>2]|0;
       $298 = (($297) - 1)|0;
       $299 = ($296>>>0)<($298>>>0);
       if ($299) {
        $300 = HEAP32[$closest>>2]|0;
        $301 = $300<<2;
        $302 = (8 + ($301))|0;
        $303 = HEAP32[$aData>>2]|0;
        $304 = (($303) + ($302)|0);
        $305 = HEAP32[$k>>2]|0;
        $306 = $305<<2;
        $307 = (4 + ($306))|0;
        $308 = HEAP32[$aData>>2]|0;
        $309 = (($308) + ($307)|0);
        ;HEAP8[$304>>0]=HEAP8[$309>>0]|0;HEAP8[$304+1>>0]=HEAP8[$309+1>>0]|0;HEAP8[$304+2>>0]=HEAP8[$309+2>>0]|0;HEAP8[$304+3>>0]=HEAP8[$309+3>>0]|0;
       }
       $310 = HEAP32[$aData>>2]|0;
       $311 = ((($310)) + 4|0);
       $312 = HEAP32[$k>>2]|0;
       $313 = (($312) - 1)|0;
       _sqlite3Put4byte($311,$313);
       $314 = HEAP32[$1>>2]|0;
       $315 = HEAP32[$3>>2]|0;
       $316 = HEAP32[$315>>2]|0;
       $317 = (_btreeGetHasContent($314,$316)|0);
       $318 = ($317|0)!=(0);
       $319 = $318 ^ 1;
       $320 = $319 ? 1 : 0;
       HEAP32[$noContent>>2] = $320;
       $321 = HEAP32[$1>>2]|0;
       $322 = HEAP32[$3>>2]|0;
       $323 = HEAP32[$322>>2]|0;
       $324 = HEAP32[$2>>2]|0;
       $325 = HEAP32[$noContent>>2]|0;
       $326 = (_btreeGetUnusedPage($321,$323,$324,$325)|0);
       HEAP32[$rc>>2] = $326;
       $327 = HEAP32[$rc>>2]|0;
       $328 = ($327|0)==(0);
       do {
        if ($328) {
         $329 = HEAP32[$2>>2]|0;
         $330 = HEAP32[$329>>2]|0;
         $331 = ((($330)) + 72|0);
         $332 = HEAP32[$331>>2]|0;
         $333 = (_sqlite3PagerWrite($332)|0);
         HEAP32[$rc>>2] = $333;
         $334 = HEAP32[$rc>>2]|0;
         $335 = ($334|0)!=(0);
         if (!($335)) {
          break;
         }
         $336 = HEAP32[$2>>2]|0;
         $337 = HEAP32[$336>>2]|0;
         _releasePage($337);
         $338 = HEAP32[$2>>2]|0;
         HEAP32[$338>>2] = 0;
        }
       } while(0);
       HEAP8[$searchList>>0] = 0;
      }
     } else {
      $87 = HEAP32[$pTrunk>>2]|0;
      $88 = ((($87)) + 72|0);
      $89 = HEAP32[$88>>2]|0;
      $90 = (_sqlite3PagerWrite($89)|0);
      HEAP32[$rc>>2] = $90;
      $91 = HEAP32[$rc>>2]|0;
      $92 = ($91|0)!=(0);
      if ($92) {
       break L5;
      }
      $93 = HEAP32[$iTrunk>>2]|0;
      $94 = HEAP32[$3>>2]|0;
      HEAP32[$94>>2] = $93;
      $95 = HEAP32[$pPage1>>2]|0;
      $96 = ((($95)) + 56|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ((($97)) + 32|0);
      $99 = HEAP32[$pTrunk>>2]|0;
      $100 = ((($99)) + 56|0);
      $101 = HEAP32[$100>>2]|0;
      ;HEAP8[$98>>0]=HEAP8[$101>>0]|0;HEAP8[$98+1>>0]=HEAP8[$101+1>>0]|0;HEAP8[$98+2>>0]=HEAP8[$101+2>>0]|0;HEAP8[$98+3>>0]=HEAP8[$101+3>>0]|0;
      $102 = HEAP32[$pTrunk>>2]|0;
      $103 = HEAP32[$2>>2]|0;
      HEAP32[$103>>2] = $102;
      HEAP32[$pTrunk>>2] = 0;
     }
    } while(0);
    $339 = HEAP32[$pPrevTrunk>>2]|0;
    _releasePage($339);
    HEAP32[$pPrevTrunk>>2] = 0;
    $340 = HEAP8[$searchList>>0]|0;
    $341 = ($340<<24>>24)!=(0);
    if (!($341)) {
     break L5;
    }
   }
   if ((label|0) == 23) {
    HEAP32[$pTrunk>>2] = 0;
    break;
   }
   else if ((label|0) == 28) {
    $111 = (_sqlite3CorruptError(61200)|0);
    HEAP32[$rc>>2] = $111;
    break;
   }
   else if ((label|0) == 40) {
    $165 = (_sqlite3CorruptError(61234)|0);
    HEAP32[$rc>>2] = $165;
    break;
   }
   else if ((label|0) == 43) {
    _releasePage($177);
    break;
   }
   else if ((label|0) == 63) {
    $276 = (_sqlite3CorruptError(61299)|0);
    HEAP32[$rc>>2] = $276;
    break;
   }
  } else {
   $342 = HEAP32[$1>>2]|0;
   $343 = ((($342)) + 19|0);
   $344 = HEAP8[$343>>0]|0;
   $345 = $344&255;
   $346 = (0)==($345|0);
   $347 = $346 ? 1 : 0;
   HEAP32[$bNoContent>>2] = $347;
   $348 = HEAP32[$1>>2]|0;
   $349 = ((($348)) + 12|0);
   $350 = HEAP32[$349>>2]|0;
   $351 = ((($350)) + 72|0);
   $352 = HEAP32[$351>>2]|0;
   $353 = (_sqlite3PagerWrite($352)|0);
   HEAP32[$rc>>2] = $353;
   $354 = HEAP32[$rc>>2]|0;
   $355 = ($354|0)!=(0);
   if ($355) {
    $356 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $356;
    $458 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($458|0);
   }
   $357 = HEAP32[$1>>2]|0;
   $358 = ((($357)) + 44|0);
   $359 = HEAP32[$358>>2]|0;
   $360 = (($359) + 1)|0;
   HEAP32[$358>>2] = $360;
   $361 = HEAP32[$1>>2]|0;
   $362 = ((($361)) + 44|0);
   $363 = HEAP32[$362>>2]|0;
   $364 = HEAP32[832>>2]|0;
   $365 = HEAP32[$1>>2]|0;
   $366 = ((($365)) + 32|0);
   $367 = HEAP32[$366>>2]|0;
   $368 = (($364>>>0) / ($367>>>0))&-1;
   $369 = (($368) + 1)|0;
   $370 = ($363|0)==($369|0);
   if ($370) {
    $371 = HEAP32[$1>>2]|0;
    $372 = ((($371)) + 44|0);
    $373 = HEAP32[$372>>2]|0;
    $374 = (($373) + 1)|0;
    HEAP32[$372>>2] = $374;
   }
   $375 = HEAP32[$1>>2]|0;
   $376 = ((($375)) + 17|0);
   $377 = HEAP8[$376>>0]|0;
   $378 = $377&255;
   $379 = ($378|0)!=(0);
   do {
    if ($379) {
     $380 = HEAP32[$1>>2]|0;
     $381 = HEAP32[$1>>2]|0;
     $382 = ((($381)) + 44|0);
     $383 = HEAP32[$382>>2]|0;
     $384 = (_ptrmapPageno($380,$383)|0);
     $385 = HEAP32[$1>>2]|0;
     $386 = ((($385)) + 44|0);
     $387 = HEAP32[$386>>2]|0;
     $388 = ($384|0)==($387|0);
     if ($388) {
      HEAP32[$pPg>>2] = 0;
      $389 = HEAP32[$1>>2]|0;
      $390 = HEAP32[$1>>2]|0;
      $391 = ((($390)) + 44|0);
      $392 = HEAP32[$391>>2]|0;
      $393 = HEAP32[$bNoContent>>2]|0;
      $394 = (_btreeGetUnusedPage($389,$392,$pPg,$393)|0);
      HEAP32[$rc>>2] = $394;
      $395 = HEAP32[$rc>>2]|0;
      $396 = ($395|0)==(0);
      if ($396) {
       $397 = HEAP32[$pPg>>2]|0;
       $398 = ((($397)) + 72|0);
       $399 = HEAP32[$398>>2]|0;
       $400 = (_sqlite3PagerWrite($399)|0);
       HEAP32[$rc>>2] = $400;
       $401 = HEAP32[$pPg>>2]|0;
       _releasePage($401);
      }
      $402 = HEAP32[$rc>>2]|0;
      $403 = ($402|0)!=(0);
      if ($403) {
       $404 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $404;
       $458 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($458|0);
      } else {
       $405 = HEAP32[$1>>2]|0;
       $406 = ((($405)) + 44|0);
       $407 = HEAP32[$406>>2]|0;
       $408 = (($407) + 1)|0;
       HEAP32[$406>>2] = $408;
       $409 = HEAP32[$1>>2]|0;
       $410 = ((($409)) + 44|0);
       $411 = HEAP32[$410>>2]|0;
       $412 = HEAP32[832>>2]|0;
       $413 = HEAP32[$1>>2]|0;
       $414 = ((($413)) + 32|0);
       $415 = HEAP32[$414>>2]|0;
       $416 = (($412>>>0) / ($415>>>0))&-1;
       $417 = (($416) + 1)|0;
       $418 = ($411|0)==($417|0);
       if (!($418)) {
        break;
       }
       $419 = HEAP32[$1>>2]|0;
       $420 = ((($419)) + 44|0);
       $421 = HEAP32[$420>>2]|0;
       $422 = (($421) + 1)|0;
       HEAP32[$420>>2] = $422;
       break;
      }
     }
    }
   } while(0);
   $423 = HEAP32[$1>>2]|0;
   $424 = ((($423)) + 12|0);
   $425 = HEAP32[$424>>2]|0;
   $426 = ((($425)) + 56|0);
   $427 = HEAP32[$426>>2]|0;
   $428 = ((($427)) + 28|0);
   $429 = HEAP32[$1>>2]|0;
   $430 = ((($429)) + 44|0);
   $431 = HEAP32[$430>>2]|0;
   _sqlite3Put4byte($428,$431);
   $432 = HEAP32[$1>>2]|0;
   $433 = ((($432)) + 44|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = HEAP32[$3>>2]|0;
   HEAP32[$435>>2] = $434;
   $436 = HEAP32[$1>>2]|0;
   $437 = HEAP32[$3>>2]|0;
   $438 = HEAP32[$437>>2]|0;
   $439 = HEAP32[$2>>2]|0;
   $440 = HEAP32[$bNoContent>>2]|0;
   $441 = (_btreeGetUnusedPage($436,$438,$439,$440)|0);
   HEAP32[$rc>>2] = $441;
   $442 = HEAP32[$rc>>2]|0;
   $443 = ($442|0)!=(0);
   if ($443) {
    $444 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $444;
    $458 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($458|0);
   } else {
    $445 = HEAP32[$2>>2]|0;
    $446 = HEAP32[$445>>2]|0;
    $447 = ((($446)) + 72|0);
    $448 = HEAP32[$447>>2]|0;
    $449 = (_sqlite3PagerWrite($448)|0);
    HEAP32[$rc>>2] = $449;
    $450 = HEAP32[$rc>>2]|0;
    $451 = ($450|0)!=(0);
    if (!($451)) {
     break;
    }
    $452 = HEAP32[$2>>2]|0;
    $453 = HEAP32[$452>>2]|0;
    _releasePage($453);
    $454 = HEAP32[$2>>2]|0;
    HEAP32[$454>>2] = 0;
    break;
   }
  }
 } while(0);
 $455 = HEAP32[$pTrunk>>2]|0;
 _releasePage($455);
 $456 = HEAP32[$pPrevTrunk>>2]|0;
 _releasePage($456);
 $457 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $457;
 $458 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($458|0);
}
function _relocatePage($pBt,$pDbPage,$eType,$iPtrPage,$iFreePage,$isCommit) {
 $pBt = $pBt|0;
 $pDbPage = $pDbPage|0;
 $eType = $eType|0;
 $iPtrPage = $iPtrPage|0;
 $iFreePage = $iFreePage|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $iDbPage = 0, $nextOvfl = 0;
 var $pPager = 0, $pPtrPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 44|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $pPtrPage = sp + 16|0;
 $iDbPage = sp + 12|0;
 $pPager = sp + 8|0;
 $rc = sp + 4|0;
 $nextOvfl = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pDbPage;
 HEAP8[$3>>0] = $eType;
 HEAP32[$4>>2] = $iPtrPage;
 HEAP32[$5>>2] = $iFreePage;
 HEAP32[$6>>2] = $isCommit;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 84|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$iDbPage>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pPager>>2] = $11;
 $12 = HEAP32[$pPager>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 72|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$6>>2]|0;
 $18 = (_sqlite3PagerMovepage($12,$15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $22 = HEAP32[$5>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 84|0);
 HEAP32[$24>>2] = $22;
 $25 = HEAP8[$3>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)==(5);
 if ($27) {
  label = 5;
 } else {
  $28 = HEAP8[$3>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(1);
  if ($30) {
   label = 5;
  } else {
   $36 = HEAP32[$2>>2]|0;
   $37 = ((($36)) + 56|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3Get4byte($38)|0);
   HEAP32[$nextOvfl>>2] = $39;
   $40 = HEAP32[$nextOvfl>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP32[$nextOvfl>>2]|0;
    $44 = HEAP32[$5>>2]|0;
    _ptrmapPut($42,$43,4,$44,$rc);
    $45 = HEAP32[$rc>>2]|0;
    $46 = ($45|0)!=(0);
    if ($46) {
     $47 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $47;
     $77 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($77|0);
    }
   }
  }
 }
 if ((label|0) == 5) {
  $31 = HEAP32[$2>>2]|0;
  $32 = (_setChildPtrmaps($31)|0);
  HEAP32[$rc>>2] = $32;
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $35;
   $77 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($77|0);
  }
 }
 $48 = HEAP8[$3>>0]|0;
 $49 = $48&255;
 $50 = ($49|0)!=(1);
 do {
  if ($50) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$4>>2]|0;
   $53 = (_btreeGetPage($51,$52,$pPtrPage,0)|0);
   HEAP32[$rc>>2] = $53;
   $54 = HEAP32[$rc>>2]|0;
   $55 = ($54|0)!=(0);
   if ($55) {
    $56 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $56;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   $57 = HEAP32[$pPtrPage>>2]|0;
   $58 = ((($57)) + 72|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3PagerWrite($59)|0);
   HEAP32[$rc>>2] = $60;
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)!=(0);
   $63 = HEAP32[$pPtrPage>>2]|0;
   if ($62) {
    _releasePage($63);
    $64 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $64;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   } else {
    $65 = HEAP32[$iDbPage>>2]|0;
    $66 = HEAP32[$5>>2]|0;
    $67 = HEAP8[$3>>0]|0;
    $68 = (_modifyPagePointer($63,$65,$66,$67)|0);
    HEAP32[$rc>>2] = $68;
    $69 = HEAP32[$pPtrPage>>2]|0;
    _releasePage($69);
    $70 = HEAP32[$rc>>2]|0;
    $71 = ($70|0)==(0);
    if (!($71)) {
     break;
    }
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$5>>2]|0;
    $74 = HEAP8[$3>>0]|0;
    $75 = HEAP32[$4>>2]|0;
    _ptrmapPut($72,$73,$74,$75,$rc);
    break;
   }
  }
 } while(0);
 $76 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $76;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _btreeGetUnusedPage($pBt,$pgno,$ppPage,$flags) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 $9 = (_btreeGetPage($5,$6,$7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 $12 = HEAP32[$3>>2]|0;
 do {
  if ($11) {
   $13 = HEAP32[$12>>2]|0;
   $14 = ((($13)) + 72|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (_sqlite3PagerPageRefcount($15)|0);
   $17 = ($16|0)>(1);
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   if (!($17)) {
    HEAP8[$19>>0] = 0;
    break;
   }
   _releasePage($19);
   $20 = HEAP32[$3>>2]|0;
   HEAP32[$20>>2] = 0;
   $21 = (_sqlite3CorruptError(57630)|0);
   HEAP32[$0>>2] = $21;
   $23 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($23|0);
  } else {
   HEAP32[$12>>2] = 0;
  }
 } while(0);
 $22 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3AbsInt32($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 $4 = HEAP32[$1>>2]|0;
 do {
  if ($3) {
   HEAP32[$0>>2] = $4;
  } else {
   $5 = ($4|0)==(-2147483648);
   if ($5) {
    HEAP32[$0>>2] = 2147483647;
    break;
   } else {
    $6 = HEAP32[$1>>2]|0;
    $7 = (0 - ($6))|0;
    HEAP32[$0>>2] = $7;
    break;
   }
  }
 } while(0);
 $8 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _btreeGetHasContent($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $16 = 0;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$p>>2]|0;
 $9 = (_sqlite3BitvecSize($8)|0);
 $10 = ($7>>>0)>($9>>>0);
 if ($10) {
  $16 = 1;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (_sqlite3BitvecTest($11,$12)|0);
 $14 = ($13|0)!=(0);
 $16 = $14;
 $15 = $16&1;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3BitvecSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3PagerMovepage($pPager,$pPg,$pgno,$isCommit) {
 $pPager = $pPager|0;
 $pPg = $pPg|0;
 $pgno = $pgno|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $needSyncPgno = 0, $or$cond = 0, $origPgno = 0, $pPgHdr = 0, $pPgOld = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pPgOld = sp + 16|0;
 $needSyncPgno = sp + 12|0;
 $rc = sp + 8|0;
 $origPgno = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pPg;
 HEAP32[$3>>2] = $pgno;
 HEAP32[$4>>2] = $isCommit;
 HEAP32[$needSyncPgno>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3PagerWrite($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $13;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  }
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 & 2;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$2>>2]|0;
  $21 = (_subjournalPageIfRequired($20)|0);
  HEAP32[$rc>>2] = $21;
  $22 = (0)!=($21|0);
  if ($22) {
   $23 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $23;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 8;
 $29 = ($28|0)==(0);
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0);
 $or$cond = $29 | $31;
 if (!($or$cond)) {
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$needSyncPgno>>2] = $34;
 }
 $35 = HEAP32[$2>>2]|0;
 $36 = ((($35)) + 24|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37&65535;
 $39 = $38 & -9;
 $40 = $39&65535;
 HEAP16[$36>>1] = $40;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 $43 = (_sqlite3PagerLookup($41,$42)|0);
 HEAP32[$pPgOld>>2] = $43;
 $44 = HEAP32[$pPgOld>>2]|0;
 $45 = ($44|0)!=(0|0);
 do {
  if ($45) {
   $46 = HEAP32[$pPgOld>>2]|0;
   $47 = ((($46)) + 24|0);
   $48 = HEAP16[$47>>1]|0;
   $49 = $48&65535;
   $50 = $49 & 8;
   $51 = HEAP32[$2>>2]|0;
   $52 = ((($51)) + 24|0);
   $53 = HEAP16[$52>>1]|0;
   $54 = $53&65535;
   $55 = $54 | $50;
   $56 = $55&65535;
   HEAP16[$52>>1] = $56;
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 16|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = ($59<<24>>24)!=(0);
   $61 = HEAP32[$pPgOld>>2]|0;
   if ($60) {
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 28|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (($64) + 1)|0;
    _sqlite3PcacheMove($61,$65);
    break;
   } else {
    _sqlite3PcacheDrop($61);
    break;
   }
  }
 } while(0);
 $66 = HEAP32[$2>>2]|0;
 $67 = ((($66)) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 HEAP32[$origPgno>>2] = $68;
 $69 = HEAP32[$2>>2]|0;
 $70 = HEAP32[$3>>2]|0;
 _sqlite3PcacheMove($69,$70);
 $71 = HEAP32[$2>>2]|0;
 _sqlite3PcacheMakeDirty($71);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 16|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = ($74<<24>>24)!=(0);
 if ($75) {
  $76 = HEAP32[$pPgOld>>2]|0;
  $77 = HEAP32[$origPgno>>2]|0;
  _sqlite3PcacheMove($76,$77);
  $78 = HEAP32[$pPgOld>>2]|0;
  _sqlite3PagerUnrefNotNull($78);
 }
 $79 = HEAP32[$needSyncPgno>>2]|0;
 $80 = ($79|0)!=(0);
 do {
  if ($80) {
   $81 = HEAP32[$1>>2]|0;
   $82 = HEAP32[$needSyncPgno>>2]|0;
   $83 = (_sqlite3PagerGet($81,$82,$pPgHdr,0)|0);
   HEAP32[$rc>>2] = $83;
   $84 = HEAP32[$rc>>2]|0;
   $85 = ($84|0)!=(0);
   if (!($85)) {
    $99 = HEAP32[$pPgHdr>>2]|0;
    $100 = ((($99)) + 24|0);
    $101 = HEAP16[$100>>1]|0;
    $102 = $101&65535;
    $103 = $102 | 8;
    $104 = $103&65535;
    HEAP16[$100>>1] = $104;
    $105 = HEAP32[$pPgHdr>>2]|0;
    _sqlite3PcacheMakeDirty($105);
    $106 = HEAP32[$pPgHdr>>2]|0;
    _sqlite3PagerUnrefNotNull($106);
    break;
   }
   $86 = HEAP32[$needSyncPgno>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = ((($87)) + 32|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = ($86>>>0)<=($89>>>0);
   if ($90) {
    $91 = HEAP32[$1>>2]|0;
    $92 = ((($91)) + 60|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = HEAP32[$needSyncPgno>>2]|0;
    $95 = HEAP32[$1>>2]|0;
    $96 = ((($95)) + 208|0);
    $97 = HEAP32[$96>>2]|0;
    _sqlite3BitvecClear($93,$94,$97);
   }
   $98 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $98;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $107 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($107|0);
}
function _setChildPtrmaps($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $childPgno = 0, $childPgno1 = 0, $i = 0, $isInitOrig = 0, $nCell = 0, $pBt = 0, $pCell = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $i = sp + 28|0;
 $nCell = sp + 24|0;
 $rc = sp + 20|0;
 $pBt = sp + 16|0;
 $isInitOrig = sp + 36|0;
 $pgno = sp + 12|0;
 $pCell = sp + 8|0;
 $childPgno = sp + 4|0;
 $childPgno1 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 HEAP8[$isInitOrig>>0] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 84|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pgno>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_btreeInitPage($9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $77 = HEAP8[$isInitOrig>>0]|0;
  $78 = HEAP32[$0>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 18|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 HEAP32[$nCell>>2] = $16;
 HEAP32[$i>>2] = 0;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$nCell>>2]|0;
  $19 = ($17|0)<($18|0);
  $20 = HEAP32[$0>>2]|0;
  if (!($19)) {
   break;
  }
  $21 = ((($20)) + 56|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 20|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = HEAP32[$i>>2]|0;
  $28 = $27<<1;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 64|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28)|0);
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = $34 << 8;
  $36 = HEAP32[$i>>2]|0;
  $37 = $36<<1;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 64|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + ($37)|0);
  $42 = ((($41)) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = $35 | $44;
  $46 = $26 & $45;
  $47 = (($22) + ($46)|0);
  HEAP32[$pCell>>2] = $47;
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$pCell>>2]|0;
  _ptrmapPutOvflPtr($48,$49,$rc);
  $50 = HEAP32[$0>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = ($52<<24>>24)!=(0);
  if (!($53)) {
   $54 = HEAP32[$pCell>>2]|0;
   $55 = (_sqlite3Get4byte($54)|0);
   HEAP32[$childPgno>>2] = $55;
   $56 = HEAP32[$pBt>>2]|0;
   $57 = HEAP32[$childPgno>>2]|0;
   $58 = HEAP32[$pgno>>2]|0;
   _ptrmapPut($56,$57,5,$58,$rc);
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$i>>2] = $60;
 }
 $61 = ((($20)) + 4|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = ($62<<24>>24)!=(0);
 if ($63) {
  $77 = HEAP8[$isInitOrig>>0]|0;
  $78 = HEAP32[$0>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 5|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = $66&255;
 $68 = (($67) + 8)|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 56|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + ($68)|0);
 $73 = (_sqlite3Get4byte($72)|0);
 HEAP32[$childPgno1>>2] = $73;
 $74 = HEAP32[$pBt>>2]|0;
 $75 = HEAP32[$childPgno1>>2]|0;
 $76 = HEAP32[$pgno>>2]|0;
 _ptrmapPut($74,$75,5,$76,$rc);
 $77 = HEAP8[$isInitOrig>>0]|0;
 $78 = HEAP32[$0>>2]|0;
 HEAP8[$78>>0] = $77;
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _ptrmapPut($pBt,$key,$eType,$parent,$pRC) {
 $pBt = $pBt|0;
 $key = $key|0;
 $eType = $eType|0;
 $parent = $parent|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrmap = 0, $offset = 0, $pDbPage = 0, $pPtrmap = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 36|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pDbPage = sp + 16|0;
 $pPtrmap = sp + 12|0;
 $iPtrmap = sp + 8|0;
 $offset = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $key;
 HEAP8[$2>>0] = $eType;
 HEAP32[$3>>2] = $parent;
 HEAP32[$4>>2] = $pRC;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = (_sqlite3CorruptError(56501)|0);
  $11 = HEAP32[$4>>2]|0;
  HEAP32[$11>>2] = $10;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_ptrmapPageno($12,$13)|0);
 HEAP32[$iPtrmap>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$iPtrmap>>2]|0;
 $18 = (_sqlite3PagerGet($16,$17,$pDbPage,0)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  HEAP32[$22>>2] = $21;
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$iPtrmap>>2]|0;
 $25 = (($23) - ($24))|0;
 $26 = (($25) - 1)|0;
 $27 = ($26*5)|0;
 HEAP32[$offset>>2] = $27;
 $28 = HEAP32[$offset>>2]|0;
 $29 = ($28|0)<(0);
 do {
  if ($29) {
   $30 = (_sqlite3CorruptError(56512)|0);
   $31 = HEAP32[$4>>2]|0;
   HEAP32[$31>>2] = $30;
  } else {
   $32 = HEAP32[$pDbPage>>2]|0;
   $33 = (_sqlite3PagerGetData($32)|0);
   HEAP32[$pPtrmap>>2] = $33;
   $34 = HEAP8[$2>>0]|0;
   $35 = $34&255;
   $36 = HEAP32[$offset>>2]|0;
   $37 = HEAP32[$pPtrmap>>2]|0;
   $38 = (($37) + ($36)|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = ($35|0)!=($40|0);
   if (!($41)) {
    $42 = HEAP32[$offset>>2]|0;
    $43 = (($42) + 1)|0;
    $44 = HEAP32[$pPtrmap>>2]|0;
    $45 = (($44) + ($43)|0);
    $46 = (_sqlite3Get4byte($45)|0);
    $47 = HEAP32[$3>>2]|0;
    $48 = ($46|0)!=($47|0);
    if (!($48)) {
     break;
    }
   }
   $49 = HEAP32[$pDbPage>>2]|0;
   $50 = (_sqlite3PagerWrite($49)|0);
   HEAP32[$rc>>2] = $50;
   $51 = HEAP32[$4>>2]|0;
   HEAP32[$51>>2] = $50;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP8[$2>>0]|0;
    $55 = HEAP32[$offset>>2]|0;
    $56 = HEAP32[$pPtrmap>>2]|0;
    $57 = (($56) + ($55)|0);
    HEAP8[$57>>0] = $54;
    $58 = HEAP32[$offset>>2]|0;
    $59 = (($58) + 1)|0;
    $60 = HEAP32[$pPtrmap>>2]|0;
    $61 = (($60) + ($59)|0);
    $62 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($61,$62);
   }
  }
 } while(0);
 $63 = HEAP32[$pDbPage>>2]|0;
 _sqlite3PagerUnref($63);
 STACKTOP = sp;return;
}
function _modifyPagePointer($pPage,$iFrom,$iTo,$eType) {
 $pPage = $pPage|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 $eType = $eType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $info = 0, $isInitOrig = 0, $nCell = 0, $pCell = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 57|0;
 $isInitOrig = sp + 56|0;
 $i = sp + 36|0;
 $nCell = sp + 32|0;
 $rc = sp + 28|0;
 $pCell = sp + 24|0;
 $info = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $iFrom;
 HEAP32[$3>>2] = $iTo;
 HEAP8[$4>>0] = $eType;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(4);
 $8 = HEAP32[$1>>2]|0;
 do {
  if ($7) {
   $9 = ((($8)) + 56|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3Get4byte($10)|0);
   $12 = HEAP32[$2>>2]|0;
   $13 = ($11|0)!=($12|0);
   if (!($13)) {
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 56|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($17,$18);
    break;
   }
   $14 = (_sqlite3CorruptError(58893)|0);
   HEAP32[$0>>2] = $14;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  } else {
   $19 = HEAP8[$8>>0]|0;
   HEAP8[$isInitOrig>>0] = $19;
   $20 = HEAP32[$1>>2]|0;
   $21 = (_btreeInitPage($20)|0);
   HEAP32[$rc>>2] = $21;
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $24;
    $144 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($144|0);
   }
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 18|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27&65535;
   HEAP32[$nCell>>2] = $28;
   HEAP32[$i>>2] = 0;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$nCell>>2]|0;
    $31 = ($29|0)<($30|0);
    if (!($31)) {
     break;
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 56|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 20|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = $37&65535;
    $39 = HEAP32[$i>>2]|0;
    $40 = $39<<1;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 64|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = (($43) + ($40)|0);
    $45 = HEAP8[$44>>0]|0;
    $46 = $45&255;
    $47 = $46 << 8;
    $48 = HEAP32[$i>>2]|0;
    $49 = $48<<1;
    $50 = HEAP32[$1>>2]|0;
    $51 = ((($50)) + 64|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (($52) + ($49)|0);
    $54 = ((($53)) + 1|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = $47 | $56;
    $58 = $38 & $57;
    $59 = (($34) + ($58)|0);
    HEAP32[$pCell>>2] = $59;
    $60 = HEAP8[$4>>0]|0;
    $61 = $60&255;
    $62 = ($61|0)==(3);
    if ($62) {
     $63 = HEAP32[$1>>2]|0;
     $64 = ((($63)) + 80|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$1>>2]|0;
     $67 = HEAP32[$pCell>>2]|0;
     FUNCTION_TABLE_viii[$65 & 255]($66,$67,$info);
     $68 = ((($info)) + 16|0);
     $69 = HEAP16[$68>>1]|0;
     $70 = $69&65535;
     $71 = ((($info)) + 12|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = ($70>>>0)<($72>>>0);
     if ($73) {
      $74 = HEAP32[$pCell>>2]|0;
      $75 = ((($info)) + 18|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76&65535;
      $78 = (($74) + ($77)|0);
      $79 = ((($78)) + -1|0);
      $80 = HEAP32[$1>>2]|0;
      $81 = ((($80)) + 56|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = HEAP32[$1>>2]|0;
      $84 = ((($83)) + 20|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85&65535;
      $87 = (($82) + ($86)|0);
      $88 = ($79>>>0)<=($87>>>0);
      if ($88) {
       $89 = HEAP32[$2>>2]|0;
       $90 = HEAP32[$pCell>>2]|0;
       $91 = ((($info)) + 18|0);
       $92 = HEAP16[$91>>1]|0;
       $93 = $92&65535;
       $94 = (($90) + ($93)|0);
       $95 = ((($94)) + -4|0);
       $96 = (_sqlite3Get4byte($95)|0);
       $97 = ($89|0)==($96|0);
       if ($97) {
        label = 13;
        break;
       }
      }
     }
    } else {
     $105 = HEAP32[$pCell>>2]|0;
     $106 = (_sqlite3Get4byte($105)|0);
     $107 = HEAP32[$2>>2]|0;
     $108 = ($106|0)==($107|0);
     if ($108) {
      label = 15;
      break;
     }
    }
    $111 = HEAP32[$i>>2]|0;
    $112 = (($111) + 1)|0;
    HEAP32[$i>>2] = $112;
   }
   if ((label|0) == 13) {
    $98 = HEAP32[$pCell>>2]|0;
    $99 = ((($info)) + 18|0);
    $100 = HEAP16[$99>>1]|0;
    $101 = $100&65535;
    $102 = (($98) + ($101)|0);
    $103 = ((($102)) + -4|0);
    $104 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($103,$104);
   }
   else if ((label|0) == 15) {
    $109 = HEAP32[$pCell>>2]|0;
    $110 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($109,$110);
   }
   $113 = HEAP32[$i>>2]|0;
   $114 = HEAP32[$nCell>>2]|0;
   $115 = ($113|0)==($114|0);
   do {
    if ($115) {
     $116 = HEAP8[$4>>0]|0;
     $117 = $116&255;
     $118 = ($117|0)!=(5);
     if (!($118)) {
      $119 = HEAP32[$1>>2]|0;
      $120 = ((($119)) + 5|0);
      $121 = HEAP8[$120>>0]|0;
      $122 = $121&255;
      $123 = (($122) + 8)|0;
      $124 = HEAP32[$1>>2]|0;
      $125 = ((($124)) + 56|0);
      $126 = HEAP32[$125>>2]|0;
      $127 = (($126) + ($123)|0);
      $128 = (_sqlite3Get4byte($127)|0);
      $129 = HEAP32[$2>>2]|0;
      $130 = ($128|0)!=($129|0);
      if (!($130)) {
       $132 = HEAP32[$1>>2]|0;
       $133 = ((($132)) + 5|0);
       $134 = HEAP8[$133>>0]|0;
       $135 = $134&255;
       $136 = (($135) + 8)|0;
       $137 = HEAP32[$1>>2]|0;
       $138 = ((($137)) + 56|0);
       $139 = HEAP32[$138>>2]|0;
       $140 = (($139) + ($136)|0);
       $141 = HEAP32[$3>>2]|0;
       _sqlite3Put4byte($140,$141);
       break;
      }
     }
     $131 = (_sqlite3CorruptError(58929)|0);
     HEAP32[$0>>2] = $131;
     $144 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($144|0);
    }
   } while(0);
   $142 = HEAP8[$isInitOrig>>0]|0;
   $143 = HEAP32[$1>>2]|0;
   HEAP8[$143>>0] = $142;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $144 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($144|0);
}
function _sqlite3PcacheMove($p,$newPgno) {
 $p = $p|0;
 $newPgno = $newPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $newPgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[(148)>>2]|0;
 $6 = HEAP32[$pCache>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 FUNCTION_TABLE_viiii[$5 & 255]($8,$10,$13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 20|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & 2;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 8;
 $29 = ($28|0)!=(0);
 if (!($29)) {
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($30,3);
 STACKTOP = sp;return;
}
function _sqlite3BitvecClear($p,$i,$pBuf) {
 $p = $p|0;
 $i = $i|0;
 $pBuf = $pBuf|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $aiValues = 0, $bin = 0;
 var $h = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $bin = sp + 12|0;
 $j = sp + 8|0;
 $aiValues = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pBuf;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$1>>2] = $6;
 while(1) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11>>>0) / ($14>>>0))&-1;
  HEAP32[$bin>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($16>>>0) % ($19>>>0))&-1;
  HEAP32[$1>>2] = $20;
  $21 = HEAP32[$bin>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$0>>2] = $25;
  $26 = HEAP32[$0>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 16) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29>>>0)<=(4000);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = $31 & 7;
  $33 = 1 << $32;
  $34 = $33 ^ -1;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35>>>0) / 8)&-1;
  $37 = HEAP32[$0>>2]|0;
  $38 = ((($37)) + 12|0);
  $39 = (($38) + ($36)|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $41 & $34;
  $43 = $42&255;
  HEAP8[$39>>0] = $43;
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$2>>2]|0;
 HEAP32[$aiValues>>2] = $44;
 $45 = HEAP32[$aiValues>>2]|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 12|0);
 _memcpy(($45|0),($47|0),500)|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 12|0);
 _memset(($49|0),0,500)|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 4|0);
 HEAP32[$51>>2] = 0;
 HEAP32[$j>>2] = 0;
 while(1) {
  $52 = HEAP32[$j>>2]|0;
  $53 = ($52>>>0)<(125);
  if (!($53)) {
   break;
  }
  $54 = HEAP32[$j>>2]|0;
  $55 = HEAP32[$aiValues>>2]|0;
  $56 = (($55) + ($54<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ($57|0)!=(0);
  if ($58) {
   $59 = HEAP32[$j>>2]|0;
   $60 = HEAP32[$aiValues>>2]|0;
   $61 = (($60) + ($59<<2)|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = ($62|0)!=($64|0);
   if ($65) {
    $66 = HEAP32[$j>>2]|0;
    $67 = HEAP32[$aiValues>>2]|0;
    $68 = (($67) + ($66<<2)|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) - 1)|0;
    $71 = $70;
    $72 = (($71>>>0) % 125)&-1;
    HEAP32[$h>>2] = $72;
    $73 = HEAP32[$0>>2]|0;
    $74 = ((($73)) + 4|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$74>>2] = $76;
    while(1) {
     $77 = HEAP32[$h>>2]|0;
     $78 = HEAP32[$0>>2]|0;
     $79 = ((($78)) + 12|0);
     $80 = (($79) + ($77<<2)|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = ($81|0)!=(0);
     if (!($82)) {
      break;
     }
     $83 = HEAP32[$h>>2]|0;
     $84 = (($83) + 1)|0;
     HEAP32[$h>>2] = $84;
     $85 = HEAP32[$h>>2]|0;
     $86 = ($85>>>0)>=(125);
     $$ = $86 ? 0 : $84;
     HEAP32[$h>>2] = $$;
    }
    $87 = HEAP32[$j>>2]|0;
    $88 = HEAP32[$aiValues>>2]|0;
    $89 = (($88) + ($87<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$h>>2]|0;
    $92 = HEAP32[$0>>2]|0;
    $93 = ((($92)) + 12|0);
    $94 = (($93) + ($91<<2)|0);
    HEAP32[$94>>2] = $90;
   }
  }
  $95 = HEAP32[$j>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$j>>2] = $96;
 }
 STACKTOP = sp;return;
}
function _ptrmapPutOvflPtr($pPage,$pCell,$pRC) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $info = 0, $ovfl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $info = sp;
 $ovfl = sp + 24|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pRC;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 80|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 FUNCTION_TABLE_viii[$8 & 255]($9,$10,$info);
 $11 = ((($info)) + 16|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = ((($info)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($13>>>0)<($15>>>0);
 if (!($16)) {
  STACKTOP = sp;return;
 }
 $17 = ((($info)) + 18|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($19) - 4)|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + ($20)|0);
 $23 = (_sqlite3Get4byte($22)|0);
 HEAP32[$ovfl>>2] = $23;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 52|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$ovfl>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 84|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 _ptrmapPut($26,$27,3,$30,$31);
 STACKTOP = sp;return;
}
function _sqlite3BtreeTripAllCursors($pBtree,$errCode,$writeOnly) {
 $pBtree = $pBtree|0;
 $errCode = $errCode|0;
 $writeOnly = $writeOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pBtree;
 HEAP32[$1>>2] = $errCode;
 HEAP32[$2>>2] = $writeOnly;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 L4: while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   label = 15;
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)!=(0);
  do {
   if ($14) {
    $15 = HEAP32[$p>>2]|0;
    $16 = ((($15)) + 64|0);
    $17 = HEAP8[$16>>0]|0;
    $18 = $17&255;
    $19 = $18 & 1;
    $20 = ($19|0)==(0);
    if ($20) {
     $21 = HEAP32[$p>>2]|0;
     $22 = ((($21)) + 66|0);
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)==(1);
     if (!($25)) {
      $26 = HEAP32[$p>>2]|0;
      $27 = ((($26)) + 66|0);
      $28 = HEAP8[$27>>0]|0;
      $29 = $28&255;
      $30 = ($29|0)==(2);
      if (!($30)) {
       break;
      }
     }
     $31 = HEAP32[$p>>2]|0;
     $32 = (_saveCursorPosition($31)|0);
     HEAP32[$rc>>2] = $32;
     $33 = HEAP32[$rc>>2]|0;
     $34 = ($33|0)!=(0);
     if ($34) {
      break L4;
     }
    } else {
     label = 10;
    }
   } else {
    label = 10;
   }
  } while(0);
  if ((label|0) == 10) {
   label = 0;
   $37 = HEAP32[$p>>2]|0;
   _sqlite3BtreeClearCursor($37);
   $38 = HEAP32[$p>>2]|0;
   $39 = ((($38)) + 66|0);
   HEAP8[$39>>0] = 4;
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP32[$p>>2]|0;
   $42 = ((($41)) + 60|0);
   HEAP32[$42>>2] = $40;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$p>>2]|0;
   $45 = ((($44)) + 68|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = $46 << 24 >> 24;
   $48 = ($43|0)<=($47|0);
   if (!($48)) {
    break;
   }
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$p>>2]|0;
   $51 = ((($50)) + 120|0);
   $52 = (($51) + ($49<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   _releasePage($53);
   $54 = HEAP32[$i>>2]|0;
   $55 = HEAP32[$p>>2]|0;
   $56 = ((($55)) + 120|0);
   $57 = (($56) + ($54<<2)|0);
   HEAP32[$57>>2] = 0;
   $58 = HEAP32[$i>>2]|0;
   $59 = (($58) + 1)|0;
   HEAP32[$i>>2] = $59;
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = ((($60)) + 8|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$p>>2] = $62;
 }
 if ((label|0) == 15) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$rc>>2]|0;
 (_sqlite3BtreeTripAllCursors($35,$36,0)|0);
 $63 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _sqlite3BtreeClearCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 48|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 66|0);
 HEAP8[$7>>0] = 0;
 STACKTOP = sp;return;
}
function _releaseMemArray($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pEnd = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0);
 $or$cond = $3 & $5;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + (($6*40)|0)|0);
 HEAP32[$pEnd>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 528|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0|0);
 if ($15) {
  while(1) {
   $16 = HEAP32[$0>>2]|0;
   $17 = ((($16)) + 24|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$db>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    _sqlite3DbFree($20,$23);
   }
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 40|0);
   HEAP32[$0>>2] = $25;
   $26 = HEAP32[$pEnd>>2]|0;
   $27 = ($25>>>0)<($26>>>0);
   if (!($27)) {
    break;
   }
  }
  STACKTOP = sp;return;
 }
 while(1) {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 & 9312;
  $33 = ($32|0)!=(0);
  $34 = HEAP32[$0>>2]|0;
  if ($33) {
   _sqlite3VdbeMemRelease($34);
  } else {
   $35 = ((($34)) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = HEAP32[$db>>2]|0;
    $39 = HEAP32[$0>>2]|0;
    $40 = ((($39)) + 20|0);
    $41 = HEAP32[$40>>2]|0;
    _sqlite3DbFree($38,$41);
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 24|0);
    HEAP32[$43>>2] = 0;
   }
  }
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  HEAP16[$45>>1] = 128;
  $46 = HEAP32[$0>>2]|0;
  $47 = ((($46)) + 40|0);
  HEAP32[$0>>2] = $47;
  $48 = HEAP32[$pEnd>>2]|0;
  $49 = ($47>>>0)<($48>>>0);
  if (!($49)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _vdbeFreeOpArray($db,$aOp,$nOp) {
 $db = $db|0;
 $aOp = $aOp|0;
 $nOp = $nOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $aOp;
 HEAP32[$2>>2] = $nOp;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($25,$26);
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pOp>>2] = $5;
 while(1) {
  $6 = HEAP32[$pOp>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + (($7*20)|0)|0);
  $10 = ($6>>>0)<($9>>>0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$pOp>>2]|0;
  $12 = ((($11)) + 1|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$pOp>>2]|0;
   $17 = ((($16)) + 1|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   $20 = HEAP32[$pOp>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   _freeP4($15,$19,$22);
  }
  $23 = HEAP32[$pOp>>2]|0;
  $24 = ((($23)) + 20|0);
  HEAP32[$pOp>>2] = $24;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($25,$26);
 STACKTOP = sp;return;
}
function _freeP4($db,$p4type,$p4) {
 $db = $db|0;
 $p4type = $p4type|0;
 $p4 = $p4|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p4type;
 HEAP32[$2>>2] = $p4;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 switch ($5|0) {
 case -20:  {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  _freeEphemeralFunction($6,$9);
  break;
 }
 case -15: case -1: case -13: case -12:  {
  break;
 }
 case -6:  {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 528|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0|0);
  if (!($15)) {
   STACKTOP = sp;return;
  }
  $16 = HEAP32[$2>>2]|0;
  _sqlite3KeyInfoUnref($16);
  STACKTOP = sp;return;
  break;
 }
 case -11:  {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 528|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)==(0|0);
  if (!($20)) {
   STACKTOP = sp;return;
  }
  $21 = HEAP32[$2>>2]|0;
  _sqlite3_free($21);
  STACKTOP = sp;return;
  break;
 }
 case -5:  {
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  _freeEphemeralFunction($22,$23);
  STACKTOP = sp;return;
  break;
 }
 case -8:  {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 528|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)==(0|0);
  $28 = HEAP32[$2>>2]|0;
  if ($27) {
   _sqlite3ValueFree($28);
   STACKTOP = sp;return;
  }
  HEAP32[$p>>2] = $28;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 24|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = HEAP32[$0>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = ((($34)) + 20|0);
   $36 = HEAP32[$35>>2]|0;
   _sqlite3DbFree($33,$36);
  }
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($37,$38);
  STACKTOP = sp;return;
  break;
 }
 case -10:  {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 528|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)==(0|0);
  if (!($42)) {
   STACKTOP = sp;return;
  }
  $43 = HEAP32[$2>>2]|0;
  _sqlite3VtabUnlock($43);
  STACKTOP = sp;return;
  break;
 }
 default: {
  STACKTOP = sp;return;
 }
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 _sqlite3DbFree($10,$11);
 STACKTOP = sp;return;
}
function _freeEphemeralFunction($db,$pDef) {
 $db = $db|0;
 $pDef = $pDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pDef;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 2|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 16;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($10,$11);
 STACKTOP = sp;return;
}
function _sqlite3KeyInfoUnref($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + -1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 _sqlite3DbFree(0,$9);
 STACKTOP = sp;return;
}
function _sqlite3VtabUnlock($pVTab) {
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)!=(0|0);
 if ($15) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  (FUNCTION_TABLE_ii[$19 & 255]($20)|0);
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _sqlite3OsCurrentTimeInt64($pVfs,$pTimeOut) {
 $pVfs = $pVfs|0;
 $pTimeOut = $pTimeOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $r = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $pTimeOut;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>=(2);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 72|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 72|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = (FUNCTION_TABLE_iii[$11 & 255]($12,$13)|0);
   HEAP32[$rc>>2] = $14;
   $29 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 255]($18,$r)|0);
 HEAP32[$rc>>2] = $19;
 $20 = +HEAPF64[$r>>3];
 $21 = $20 * 8.64E+7;
 $22 = (~~$21)>>>0;
 $23 = +Math_abs($21) >= 1.0 ? $21 > 0.0 ? (~~+Math_min(+Math_floor($21 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($21 - +(~~$21 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $24 = HEAP32[$1>>2]|0;
 $25 = $24;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = (($25) + 4)|0;
 $28 = $27;
 HEAP32[$28>>2] = $23;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3VdbeDelete($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3VdbeClearObject($5,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 52|0);
 $13 = HEAP32[$12>>2]|0;
 if ($10) {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 48|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 52|0);
  HEAP32[$17>>2] = $13;
 } else {
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 4|0);
  HEAP32[$19>>2] = $13;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 52|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 52|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 48|0);
  HEAP32[$30>>2] = $26;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 40|0);
 HEAP32[$32>>2] = -1241070648;
 $33 = HEAP32[$0>>2]|0;
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$db>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($34,$35);
 STACKTOP = sp;return;
}
function _apiOomError($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3OomClear($1);
 $2 = HEAP32[$0>>2]|0;
 _sqlite3Error($2,7);
 STACKTOP = sp;return 7;
}
function _sqlite3VdbeHalt($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $eStatementOp = 0, $isSpecialError = 0, $mrc = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $db = sp + 12|0;
 $mrc = sp + 8|0;
 $eStatementOp = sp + 4|0;
 $isSpecialError = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 65|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 80|0);
  HEAP32[$9>>2] = 7;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 200|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 200|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 196|0);
  $19 = HEAP32[$18>>2]|0;
  _memset(($16|0),0,($19|0))|0;
 }
 $20 = HEAP32[$1>>2]|0;
 _closeAllCursors($20);
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 40|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(-1108210269);
 if ($24) {
  HEAP32[$0>>2] = 0;
  $284 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($284|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 76|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)>=(0);
 if ($28) {
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 88|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = ($31&65535) >>> 7;
  $33 = $32 & 1;
  $34 = $33&65535;
  $35 = ($34|0)!=(0);
  if ($35) {
   HEAP32[$eStatementOp>>2] = 0;
   $36 = HEAP32[$1>>2]|0;
   _sqlite3VdbeEnter($36);
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 80|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = $39 & 255;
   HEAP32[$mrc>>2] = $40;
   $41 = HEAP32[$mrc>>2]|0;
   $42 = ($41|0)==(7);
   $43 = HEAP32[$mrc>>2]|0;
   $44 = ($43|0)==(10);
   $or$cond = $42 | $44;
   $45 = HEAP32[$mrc>>2]|0;
   $46 = ($45|0)==(9);
   $or$cond3 = $or$cond | $46;
   if ($or$cond3) {
    $50 = 1;
   } else {
    $47 = HEAP32[$mrc>>2]|0;
    $48 = ($47|0)==(13);
    $50 = $48;
   }
   $49 = $50&1;
   HEAP32[$isSpecialError>>2] = $49;
   $51 = HEAP32[$isSpecialError>>2]|0;
   $52 = ($51|0)!=(0);
   do {
    if ($52) {
     $53 = HEAP32[$1>>2]|0;
     $54 = ((($53)) + 88|0);
     $55 = HEAP16[$54>>1]|0;
     $56 = ($55&65535) >>> 6;
     $57 = $56 & 1;
     $58 = $57&65535;
     $59 = ($58|0)==(0);
     $60 = HEAP32[$mrc>>2]|0;
     $61 = ($60|0)!=(9);
     $or$cond5 = $59 | $61;
     if ($or$cond5) {
      $62 = HEAP32[$mrc>>2]|0;
      $63 = ($62|0)==(7);
      $64 = HEAP32[$mrc>>2]|0;
      $65 = ($64|0)==(13);
      $or$cond7 = $63 | $65;
      if ($or$cond7) {
       $66 = HEAP32[$1>>2]|0;
       $67 = ((($66)) + 88|0);
       $68 = HEAP16[$67>>1]|0;
       $69 = ($68&65535) >>> 5;
       $70 = $69 & 1;
       $71 = $70&65535;
       $72 = ($71|0)!=(0);
       if ($72) {
        HEAP32[$eStatementOp>>2] = 2;
        break;
       }
      }
      $73 = HEAP32[$db>>2]|0;
      _sqlite3RollbackAll($73,516);
      $74 = HEAP32[$db>>2]|0;
      _sqlite3CloseSavepoints($74);
      $75 = HEAP32[$db>>2]|0;
      $76 = ((($75)) + 63|0);
      HEAP8[$76>>0] = 1;
      $77 = HEAP32[$1>>2]|0;
      $78 = ((($77)) + 92|0);
      HEAP32[$78>>2] = 0;
     }
    }
   } while(0);
   $79 = HEAP32[$1>>2]|0;
   $80 = ((($79)) + 80|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = ($81|0)==(0);
   if ($82) {
    $83 = HEAP32[$1>>2]|0;
    (_sqlite3VdbeCheckFk($83,0)|0);
   }
   $84 = HEAP32[$db>>2]|0;
   $85 = ((($84)) + 308|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($86|0)>(0);
   if ($87) {
    $88 = HEAP32[$db>>2]|0;
    $89 = ((($88)) + 332|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($90|0)==(0|0);
    if ($91) {
     label = 38;
    } else {
     label = 21;
    }
   } else {
    label = 21;
   }
   if ((label|0) == 21) {
    $92 = HEAP32[$db>>2]|0;
    $93 = ((($92)) + 63|0);
    $94 = HEAP8[$93>>0]|0;
    $95 = $94&255;
    $96 = ($95|0)!=(0);
    if ($96) {
     $97 = HEAP32[$db>>2]|0;
     $98 = ((($97)) + 156|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$1>>2]|0;
     $101 = ((($100)) + 88|0);
     $102 = HEAP16[$101>>1]|0;
     $103 = ($102&65535) >>> 6;
     $104 = $103 & 1;
     $105 = $104&65535;
     $106 = ($105|0)==(0);
     $107 = $106&1;
     $108 = ($99|0)==($107|0);
     if ($108) {
      $109 = HEAP32[$1>>2]|0;
      $110 = ((($109)) + 80|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = ($111|0)==(0);
      if ($112) {
       label = 25;
      } else {
       $113 = HEAP32[$1>>2]|0;
       $114 = ((($113)) + 86|0);
       $115 = HEAP8[$114>>0]|0;
       $116 = $115&255;
       $117 = ($116|0)!=(3);
       $118 = HEAP32[$isSpecialError>>2]|0;
       $119 = ($118|0)!=(0);
       $or$cond9 = $117 | $119;
       if ($or$cond9) {
        $168 = HEAP32[$db>>2]|0;
        _sqlite3RollbackAll($168,0);
        $169 = HEAP32[$1>>2]|0;
        $170 = ((($169)) + 92|0);
        HEAP32[$170>>2] = 0;
       } else {
        label = 25;
       }
      }
      do {
       if ((label|0) == 25) {
        $120 = HEAP32[$1>>2]|0;
        $121 = (_sqlite3VdbeCheckFk($120,1)|0);
        HEAP32[$rc>>2] = $121;
        $122 = HEAP32[$rc>>2]|0;
        $123 = ($122|0)!=(0);
        do {
         if ($123) {
          $124 = HEAP32[$1>>2]|0;
          $125 = ((($124)) + 88|0);
          $126 = HEAP16[$125>>1]|0;
          $127 = ($126&65535) >>> 6;
          $128 = $127 & 1;
          $129 = $128&65535;
          $130 = ($129|0)!=(0);
          if (!($130)) {
           HEAP32[$rc>>2] = 787;
           break;
          }
          HEAP32[$0>>2] = 1;
          $284 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($284|0);
         } else {
          $131 = HEAP32[$db>>2]|0;
          $132 = HEAP32[$1>>2]|0;
          $133 = (_vdbeCommit($131,$132)|0);
          HEAP32[$rc>>2] = $133;
         }
        } while(0);
        $134 = HEAP32[$rc>>2]|0;
        $135 = ($134|0)==(5);
        if ($135) {
         $136 = HEAP32[$1>>2]|0;
         $137 = ((($136)) + 88|0);
         $138 = HEAP16[$137>>1]|0;
         $139 = ($138&65535) >>> 6;
         $140 = $139 & 1;
         $141 = $140&65535;
         $142 = ($141|0)!=(0);
         if ($142) {
          HEAP32[$0>>2] = 5;
          $284 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($284|0);
         }
        }
        $143 = HEAP32[$rc>>2]|0;
        $144 = ($143|0)!=(0);
        if ($144) {
         $145 = HEAP32[$rc>>2]|0;
         $146 = HEAP32[$1>>2]|0;
         $147 = ((($146)) + 80|0);
         HEAP32[$147>>2] = $145;
         $148 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($148,0);
         $149 = HEAP32[$1>>2]|0;
         $150 = ((($149)) + 92|0);
         HEAP32[$150>>2] = 0;
         break;
        } else {
         $151 = HEAP32[$db>>2]|0;
         $152 = ((($151)) + 512|0);
         $153 = $152;
         $154 = $153;
         HEAP32[$154>>2] = 0;
         $155 = (($153) + 4)|0;
         $156 = $155;
         HEAP32[$156>>2] = 0;
         $157 = HEAP32[$db>>2]|0;
         $158 = ((($157)) + 520|0);
         $159 = $158;
         $160 = $159;
         HEAP32[$160>>2] = 0;
         $161 = (($159) + 4)|0;
         $162 = $161;
         HEAP32[$162>>2] = 0;
         $163 = HEAP32[$db>>2]|0;
         $164 = ((($163)) + 24|0);
         $165 = HEAP32[$164>>2]|0;
         $166 = $165 & -16777217;
         HEAP32[$164>>2] = $166;
         $167 = HEAP32[$db>>2]|0;
         _sqlite3CommitInternalChanges($167);
         break;
        }
       }
      } while(0);
      $171 = HEAP32[$db>>2]|0;
      $172 = ((($171)) + 504|0);
      HEAP32[$172>>2] = 0;
     } else {
      label = 38;
     }
    } else {
     label = 38;
    }
   }
   do {
    if ((label|0) == 38) {
     $173 = HEAP32[$eStatementOp>>2]|0;
     $174 = ($173|0)==(0);
     if ($174) {
      $175 = HEAP32[$1>>2]|0;
      $176 = ((($175)) + 80|0);
      $177 = HEAP32[$176>>2]|0;
      $178 = ($177|0)==(0);
      if (!($178)) {
       $179 = HEAP32[$1>>2]|0;
       $180 = ((($179)) + 86|0);
       $181 = HEAP8[$180>>0]|0;
       $182 = $181&255;
       $183 = ($182|0)==(3);
       if (!($183)) {
        $184 = HEAP32[$1>>2]|0;
        $185 = ((($184)) + 86|0);
        $186 = HEAP8[$185>>0]|0;
        $187 = $186&255;
        $188 = ($187|0)==(2);
        if ($188) {
         HEAP32[$eStatementOp>>2] = 2;
         break;
        } else {
         $189 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($189,516);
         $190 = HEAP32[$db>>2]|0;
         _sqlite3CloseSavepoints($190);
         $191 = HEAP32[$db>>2]|0;
         $192 = ((($191)) + 63|0);
         HEAP8[$192>>0] = 1;
         $193 = HEAP32[$1>>2]|0;
         $194 = ((($193)) + 92|0);
         HEAP32[$194>>2] = 0;
         break;
        }
       }
      }
      HEAP32[$eStatementOp>>2] = 1;
     }
    }
   } while(0);
   $195 = HEAP32[$eStatementOp>>2]|0;
   $196 = ($195|0)!=(0);
   if ($196) {
    $197 = HEAP32[$1>>2]|0;
    $198 = HEAP32[$eStatementOp>>2]|0;
    $199 = (_sqlite3VdbeCloseStatement($197,$198)|0);
    HEAP32[$rc>>2] = $199;
    $200 = HEAP32[$rc>>2]|0;
    $201 = ($200|0)!=(0);
    if ($201) {
     $202 = HEAP32[$1>>2]|0;
     $203 = ((($202)) + 80|0);
     $204 = HEAP32[$203>>2]|0;
     $205 = ($204|0)==(0);
     if ($205) {
      label = 49;
     } else {
      $206 = HEAP32[$1>>2]|0;
      $207 = ((($206)) + 80|0);
      $208 = HEAP32[$207>>2]|0;
      $209 = $208 & 255;
      $210 = ($209|0)==(19);
      if ($210) {
       label = 49;
      }
     }
     if ((label|0) == 49) {
      $211 = HEAP32[$rc>>2]|0;
      $212 = HEAP32[$1>>2]|0;
      $213 = ((($212)) + 80|0);
      HEAP32[$213>>2] = $211;
      $214 = HEAP32[$db>>2]|0;
      $215 = HEAP32[$1>>2]|0;
      $216 = ((($215)) + 44|0);
      $217 = HEAP32[$216>>2]|0;
      _sqlite3DbFree($214,$217);
      $218 = HEAP32[$1>>2]|0;
      $219 = ((($218)) + 44|0);
      HEAP32[$219>>2] = 0;
     }
     $220 = HEAP32[$db>>2]|0;
     _sqlite3RollbackAll($220,516);
     $221 = HEAP32[$db>>2]|0;
     _sqlite3CloseSavepoints($221);
     $222 = HEAP32[$db>>2]|0;
     $223 = ((($222)) + 63|0);
     HEAP8[$223>>0] = 1;
     $224 = HEAP32[$1>>2]|0;
     $225 = ((($224)) + 92|0);
     HEAP32[$225>>2] = 0;
    }
   }
   $226 = HEAP32[$1>>2]|0;
   $227 = ((($226)) + 88|0);
   $228 = HEAP16[$227>>1]|0;
   $229 = ($228&65535) >>> 2;
   $230 = $229 & 1;
   $231 = $230&65535;
   $232 = ($231|0)!=(0);
   if ($232) {
    $233 = HEAP32[$eStatementOp>>2]|0;
    $234 = ($233|0)!=(2);
    $235 = HEAP32[$db>>2]|0;
    if ($234) {
     $236 = HEAP32[$1>>2]|0;
     $237 = ((($236)) + 92|0);
     $238 = HEAP32[$237>>2]|0;
     _sqlite3VdbeSetChanges($235,$238);
    } else {
     _sqlite3VdbeSetChanges($235,0);
    }
    $239 = HEAP32[$1>>2]|0;
    $240 = ((($239)) + 92|0);
    HEAP32[$240>>2] = 0;
   }
  }
 }
 $241 = HEAP32[$1>>2]|0;
 $242 = ((($241)) + 76|0);
 $243 = HEAP32[$242>>2]|0;
 $244 = ($243|0)>=(0);
 if ($244) {
  $245 = HEAP32[$db>>2]|0;
  $246 = ((($245)) + 148|0);
  $247 = HEAP32[$246>>2]|0;
  $248 = (($247) + -1)|0;
  HEAP32[$246>>2] = $248;
  $249 = HEAP32[$1>>2]|0;
  $250 = ((($249)) + 88|0);
  $251 = HEAP16[$250>>1]|0;
  $252 = ($251&65535) >>> 6;
  $253 = $252 & 1;
  $254 = $253&65535;
  $255 = ($254|0)!=(0);
  if (!($255)) {
   $256 = HEAP32[$db>>2]|0;
   $257 = ((($256)) + 156|0);
   $258 = HEAP32[$257>>2]|0;
   $259 = (($258) + -1)|0;
   HEAP32[$257>>2] = $259;
  }
  $260 = HEAP32[$1>>2]|0;
  $261 = ((($260)) + 88|0);
  $262 = HEAP16[$261>>1]|0;
  $263 = ($262&65535) >>> 7;
  $264 = $263 & 1;
  $265 = $264&65535;
  $266 = ($265|0)!=(0);
  if ($266) {
   $267 = HEAP32[$db>>2]|0;
   $268 = ((($267)) + 152|0);
   $269 = HEAP32[$268>>2]|0;
   $270 = (($269) + -1)|0;
   HEAP32[$268>>2] = $270;
  }
 }
 $271 = HEAP32[$1>>2]|0;
 $272 = ((($271)) + 40|0);
 HEAP32[$272>>2] = 1369188723;
 $273 = HEAP32[$db>>2]|0;
 $274 = ((($273)) + 65|0);
 $275 = HEAP8[$274>>0]|0;
 $276 = ($275<<24>>24)!=(0);
 if ($276) {
  $277 = HEAP32[$1>>2]|0;
  $278 = ((($277)) + 80|0);
  HEAP32[$278>>2] = 7;
 }
 $279 = HEAP32[$1>>2]|0;
 $280 = ((($279)) + 80|0);
 $281 = HEAP32[$280>>2]|0;
 $282 = ($281|0)==(5);
 $283 = $282 ? 5 : 0;
 HEAP32[$0>>2] = $283;
 $284 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($284|0);
}
function _sqlite3VdbeTransferError($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 80|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$db>>2]|0;
 if (!($9)) {
  $35 = HEAP32[$rc>>2]|0;
  _sqlite3Error($10,$35);
  $36 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $11 = ((($10)) + 66|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = (($12) + 1)<<24>>24;
 HEAP8[$11>>0] = $13;
 _sqlite3BeginBenignMalloc();
 $14 = HEAP32[$db>>2]|0;
 $15 = ((($14)) + 236|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = (_sqlite3ValueNew($18)|0);
  $20 = HEAP32[$db>>2]|0;
  $21 = ((($20)) + 236|0);
  HEAP32[$21>>2] = $19;
 }
 $22 = HEAP32[$db>>2]|0;
 $23 = ((($22)) + 236|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 44|0);
 $27 = HEAP32[$26>>2]|0;
 _sqlite3ValueSetStr($24,-1,$27,1,(-1));
 _sqlite3EndBenignMalloc();
 $28 = HEAP32[$db>>2]|0;
 $29 = ((($28)) + 66|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = (($30) + -1)<<24>>24;
 HEAP8[$29>>0] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = HEAP32[$db>>2]|0;
 $34 = ((($33)) + 52|0);
 HEAP32[$34>>2] = $32;
 $36 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _Cleanup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3DbFree($3,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 20|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _closeAllCursors($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDel = 0, $pFrame = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pFrame = sp + 4|0;
 $pDel = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 176|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 176|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pFrame>>2] = $7;
  while(1) {
   $8 = HEAP32[$pFrame>>2]|0;
   $9 = ((($8)) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   $12 = HEAP32[$pFrame>>2]|0;
   if (!($11)) {
    break;
   }
   $13 = ((($12)) + 4|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$pFrame>>2] = $14;
  }
  (_sqlite3VdbeFrameRestore($12)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 176|0);
  HEAP32[$16>>2] = 0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 184|0);
  HEAP32[$18>>2] = 0;
 }
 $19 = HEAP32[$0>>2]|0;
 _closeCursorsInFrame($19);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 40|0);
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 28|0);
  $30 = HEAP32[$29>>2]|0;
  _releaseMemArray($27,$30);
 }
 while(1) {
  $31 = HEAP32[$0>>2]|0;
  $32 = ((($31)) + 180|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)!=(0|0);
  $35 = HEAP32[$0>>2]|0;
  if (!($34)) {
   break;
  }
  $36 = ((($35)) + 180|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$pDel>>2] = $37;
  $38 = HEAP32[$pDel>>2]|0;
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$0>>2]|0;
  $42 = ((($41)) + 180|0);
  HEAP32[$42>>2] = $40;
  $43 = HEAP32[$pDel>>2]|0;
  _sqlite3VdbeFrameDelete($43);
 }
 $44 = ((($35)) + 204|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)!=(0|0);
 if (!($46)) {
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$0>>2]|0;
 _sqlite3VdbeDeleteAuxData($47,-1,0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeEnter($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aDb = 0, $db = 0;
 var $i = 0, $nDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $db = sp + 8|0;
 $aDb = sp + 4|0;
 $nDb = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aDb>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$nDb>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$nDb>>2]|0;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)!=(1);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 100|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$i>>2]|0;
   $22 = 1 << $21;
   $23 = $20 & $22;
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$i>>2]|0;
    $26 = HEAP32[$aDb>>2]|0;
    $27 = (($26) + ($25<<4)|0);
    $28 = ((($27)) + 4|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($29|0)!=(0|0);
    if ($30) {
     $31 = HEAP32[$i>>2]|0;
     $32 = HEAP32[$aDb>>2]|0;
     $33 = (($32) + ($31<<4)|0);
     $34 = ((($33)) + 4|0);
     $35 = HEAP32[$34>>2]|0;
     _sqlite3BtreeEnter($35);
    }
   }
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _sqlite3RollbackAll($db,$tripCode) {
 $db = $db|0;
 $tripCode = $tripCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $i = 0, $inTrans = 0, $p = 0, $schemaChange = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $i = sp + 12|0;
 $inTrans = sp + 8|0;
 $schemaChange = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $tripCode;
 HEAP32[$inTrans>>2] = 0;
 _sqlite3BeginBenignMalloc();
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 2;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 140|0);
  $10 = ((($9)) + 5|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(0);
  $15 = $13;
 } else {
  $15 = 0;
 }
 $14 = $15&1;
 HEAP32[$schemaChange>>2] = $14;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$p>>2] = $27;
  $28 = HEAP32[$p>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = (_sqlite3BtreeIsInTrans($30)|0);
   $32 = ($31|0)!=(0);
   if ($32) {
    HEAP32[$inTrans>>2] = 1;
   }
   $33 = HEAP32[$p>>2]|0;
   $34 = HEAP32[$1>>2]|0;
   $35 = HEAP32[$schemaChange>>2]|0;
   $36 = ($35|0)!=(0);
   $37 = $36 ^ 1;
   $38 = $37&1;
   (_sqlite3BtreeRollback($33,$34,$38)|0);
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 1)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$0>>2]|0;
 (_sqlite3VtabRollback($41)|0);
 _sqlite3EndBenignMalloc();
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 24|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = $44 & 2;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 140|0);
  $49 = ((($48)) + 5|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = ($51|0)==(0);
  if ($52) {
   $53 = HEAP32[$0>>2]|0;
   _sqlite3ExpirePreparedStatements($53);
   $54 = HEAP32[$0>>2]|0;
   _sqlite3ResetAllSchemasOfConnection($54);
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 512|0);
 $57 = $56;
 $58 = $57;
 HEAP32[$58>>2] = 0;
 $59 = (($57) + 4)|0;
 $60 = $59;
 HEAP32[$60>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 520|0);
 $63 = $62;
 $64 = $63;
 HEAP32[$64>>2] = 0;
 $65 = (($63) + 4)|0;
 $66 = $65;
 HEAP32[$66>>2] = 0;
 $67 = HEAP32[$0>>2]|0;
 $68 = ((($67)) + 24|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $69 & -16777217;
 HEAP32[$68>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 204|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = ($73|0)!=(0|0);
 if (!($74)) {
  STACKTOP = sp;return;
 }
 $75 = HEAP32[$inTrans>>2]|0;
 $76 = ($75|0)!=(0);
 if (!($76)) {
  $77 = HEAP32[$0>>2]|0;
  $78 = ((($77)) + 63|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = ($79<<24>>24)!=(0);
  if ($80) {
   STACKTOP = sp;return;
  }
 }
 $81 = HEAP32[$0>>2]|0;
 $82 = ((($81)) + 204|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = HEAP32[$0>>2]|0;
 $85 = ((($84)) + 200|0);
 $86 = HEAP32[$85>>2]|0;
 FUNCTION_TABLE_vi[$83 & 255]($86);
 STACKTOP = sp;return;
}
function _sqlite3CloseSavepoints($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ((($1)) + 492|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  $5 = HEAP32[$0>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 492|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pTmp>>2] = $7;
  $8 = HEAP32[$pTmp>>2]|0;
  $9 = ((($8)) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 492|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$pTmp>>2]|0;
  _sqlite3DbFree($13,$14);
 }
 $15 = ((($5)) + 500|0);
 HEAP32[$15>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 504|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 71|0);
 HEAP8[$19>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeCheckFk($p,$deferred) {
 $p = $p|0;
 $deferred = $deferred|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $deferred;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$db>>2]|0;
  $8 = ((($7)) + 512|0);
  $9 = $8;
  $10 = $9;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9) + 4)|0;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$db>>2]|0;
  $16 = ((($15)) + 520|0);
  $17 = $16;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($17) + 4)|0;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (_i64Add(($11|0),($14|0),($19|0),($22|0))|0);
  $24 = tempRet0;
  $25 = ($24|0)>(0);
  $26 = ($23>>>0)>(0);
  $27 = ($24|0)==(0);
  $28 = $27 & $26;
  $29 = $25 | $28;
  if (!($29)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $30 = HEAP32[$2>>2]|0;
   $31 = ($30|0)!=(0);
   if (!($31)) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 144|0);
    $34 = $33;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)>(0);
    $41 = ($36>>>0)>(0);
    $42 = ($39|0)==(0);
    $43 = $42 & $41;
    $44 = $40 | $43;
    if ($44) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $50 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 } while(0);
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 80|0);
 HEAP32[$46>>2] = 787;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 86|0);
 HEAP8[$48>>0] = 2;
 $49 = HEAP32[$1>>2]|0;
 _sqlite3VdbeError($49,19133,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $50 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _vdbeCommit($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iRandom = 0, $nMainFile = 0, $nTrans = 0, $needSync = 0;
 var $needXcommit = 0, $offset = 0, $or$cond = 0, $pBt = 0, $pBt1 = 0, $pBt2 = 0, $pBt3 = 0, $pBt4 = 0, $pBt5 = 0, $pMaster = 0, $pVfs = 0, $rc = 0, $res = 0, $retryCount = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr14 = 0, $zFile = 0;
 var $zMainFile = 0, $zMaster = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $i = sp + 116|0;
 $nTrans = sp + 112|0;
 $rc = sp + 108|0;
 $needXcommit = sp + 104|0;
 $pBt = sp + 100|0;
 $pBt1 = sp + 96|0;
 $pBt2 = sp + 92|0;
 $pVfs = sp + 88|0;
 $needSync = sp + 84|0;
 $zMaster = sp + 80|0;
 $zMainFile = sp + 76|0;
 $pMaster = sp + 72|0;
 $offset = sp;
 $res = sp + 68|0;
 $retryCount = sp + 64|0;
 $nMainFile = sp + 60|0;
 $iRandom = sp + 56|0;
 $pBt3 = sp + 52|0;
 $zFile = sp + 48|0;
 $pBt4 = sp + 44|0;
 $pBt5 = sp + 40|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$nTrans>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$needXcommit>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3VtabSync($3,$4)|0);
 HEAP32[$rc>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$rc>>2]|0;
  $7 = ($6|0)==(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pBt>>2] = $19;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = (_sqlite3BtreeIsInTrans($20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   HEAP32[$needXcommit>>2] = 1;
   $23 = HEAP32[$i>>2]|0;
   $24 = ($23|0)!=(1);
   if ($24) {
    $25 = HEAP32[$nTrans>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[$nTrans>>2] = $26;
   }
   $27 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeEnter($27);
   $28 = HEAP32[$pBt>>2]|0;
   $29 = (_sqlite3BtreePager($28)|0);
   $30 = (_sqlite3PagerExclusiveLock($29)|0);
   HEAP32[$rc>>2] = $30;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
 }
 $33 = HEAP32[$rc>>2]|0;
 $34 = ($33|0)!=(0);
 if ($34) {
  $35 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $35;
  $294 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($294|0);
 }
 $36 = HEAP32[$needXcommit>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 196|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = ($40|0)!=(0|0);
  if ($41) {
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 196|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 192|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (FUNCTION_TABLE_ii[$44 & 255]($47)|0);
   HEAP32[$rc>>2] = $48;
   $49 = HEAP32[$rc>>2]|0;
   $50 = ($49|0)!=(0);
   if ($50) {
    HEAP32[$0>>2] = 531;
    $294 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($294|0);
   }
  }
 }
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = ((($53)) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (_sqlite3BtreeGetFilename($55)|0);
 $57 = (_sqlite3Strlen30($56)|0);
 $58 = (0)==($57|0);
 $59 = HEAP32[$nTrans>>2]|0;
 $60 = ($59|0)<=(1);
 $or$cond = $58 | $60;
 if ($or$cond) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)==(0);
   if (!($62)) {
    break;
   }
   $63 = HEAP32[$i>>2]|0;
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 20|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = ($63|0)<($66|0);
   if (!($67)) {
    break;
   }
   $68 = HEAP32[$i>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 16|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + ($68<<4)|0);
   $73 = ((($72)) + 4|0);
   $74 = HEAP32[$73>>2]|0;
   HEAP32[$pBt1>>2] = $74;
   $75 = HEAP32[$pBt1>>2]|0;
   $76 = ($75|0)!=(0|0);
   if ($76) {
    $77 = HEAP32[$pBt1>>2]|0;
    $78 = (_sqlite3BtreeCommitPhaseOne($77,0)|0);
    HEAP32[$rc>>2] = $78;
   }
   $79 = HEAP32[$i>>2]|0;
   $80 = (($79) + 1)|0;
   HEAP32[$i>>2] = $80;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $81 = HEAP32[$rc>>2]|0;
   $82 = ($81|0)==(0);
   if (!($82)) {
    break;
   }
   $83 = HEAP32[$i>>2]|0;
   $84 = HEAP32[$1>>2]|0;
   $85 = ((($84)) + 20|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($83|0)<($86|0);
   if (!($87)) {
    break;
   }
   $88 = HEAP32[$i>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = ((($89)) + 16|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($91) + ($88<<4)|0);
   $93 = ((($92)) + 4|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$pBt2>>2] = $94;
   $95 = HEAP32[$pBt2>>2]|0;
   $96 = ($95|0)!=(0|0);
   if ($96) {
    $97 = HEAP32[$pBt2>>2]|0;
    $98 = (_sqlite3BtreeCommitPhaseTwo($97,0)|0);
    HEAP32[$rc>>2] = $98;
   }
   $99 = HEAP32[$i>>2]|0;
   $100 = (($99) + 1)|0;
   HEAP32[$i>>2] = $100;
  }
  $101 = HEAP32[$rc>>2]|0;
  $102 = ($101|0)==(0);
  if ($102) {
   $103 = HEAP32[$1>>2]|0;
   (_sqlite3VtabCommit($103)|0);
  }
 } else {
  $104 = HEAP32[$1>>2]|0;
  $105 = HEAP32[$104>>2]|0;
  HEAP32[$pVfs>>2] = $105;
  HEAP32[$needSync>>2] = 0;
  HEAP32[$zMaster>>2] = 0;
  $106 = HEAP32[$1>>2]|0;
  $107 = ((($106)) + 16|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = ((($108)) + 4|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = (_sqlite3BtreeGetFilename($110)|0);
  HEAP32[$zMainFile>>2] = $111;
  HEAP32[$pMaster>>2] = 0;
  $112 = $offset;
  $113 = $112;
  HEAP32[$113>>2] = 0;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = 0;
  HEAP32[$retryCount>>2] = 0;
  $116 = HEAP32[$zMainFile>>2]|0;
  $117 = (_sqlite3Strlen30($116)|0);
  HEAP32[$nMainFile>>2] = $117;
  $118 = HEAP32[$1>>2]|0;
  $119 = HEAP32[$zMainFile>>2]|0;
  HEAP32[$vararg_buffer>>2] = $119;
  $120 = (_sqlite3MPrintf($118,19163,$vararg_buffer)|0);
  HEAP32[$zMaster>>2] = $120;
  $121 = HEAP32[$zMaster>>2]|0;
  $122 = ($121|0)==(0|0);
  if ($122) {
   HEAP32[$0>>2] = 7;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
  while(1) {
   $123 = HEAP32[$retryCount>>2]|0;
   $124 = ($123|0)!=(0);
   if ($124) {
    $125 = HEAP32[$retryCount>>2]|0;
    $126 = ($125|0)>(100);
    if ($126) {
     label = 34;
     break;
    }
    $130 = HEAP32[$retryCount>>2]|0;
    $131 = ($130|0)==(1);
    if ($131) {
     $132 = HEAP32[$zMaster>>2]|0;
     HEAP32[$vararg_buffer8>>2] = $132;
     _sqlite3_log(13,19193,$vararg_buffer8);
    }
   }
   $133 = HEAP32[$retryCount>>2]|0;
   $134 = (($133) + 1)|0;
   HEAP32[$retryCount>>2] = $134;
   _sqlite3_randomness(4,$iRandom);
   $135 = HEAP32[$nMainFile>>2]|0;
   $136 = HEAP32[$zMaster>>2]|0;
   $137 = (($136) + ($135)|0);
   $138 = HEAP32[$iRandom>>2]|0;
   $139 = $138 >>> 8;
   $140 = $139 & 16777215;
   $141 = HEAP32[$iRandom>>2]|0;
   $142 = $141 & 255;
   HEAP32[$vararg_buffer11>>2] = $140;
   $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
   HEAP32[$vararg_ptr14>>2] = $142;
   (_sqlite3_snprintf(13,$137,19208,$vararg_buffer11)|0);
   $143 = HEAP32[$pVfs>>2]|0;
   $144 = HEAP32[$zMaster>>2]|0;
   $145 = (_sqlite3OsAccess($143,$144,0,$res)|0);
   HEAP32[$rc>>2] = $145;
   $146 = HEAP32[$rc>>2]|0;
   $147 = ($146|0)==(0);
   $148 = HEAP32[$res>>2]|0;
   $149 = ($148|0)!=(0);
   $150 = $147 ? $149 : 0;
   if (!($150)) {
    break;
   }
  }
  if ((label|0) == 34) {
   $127 = HEAP32[$zMaster>>2]|0;
   HEAP32[$vararg_buffer5>>2] = $127;
   _sqlite3_log(13,19179,$vararg_buffer5);
   $128 = HEAP32[$pVfs>>2]|0;
   $129 = HEAP32[$zMaster>>2]|0;
   (_sqlite3OsDelete($128,$129,0)|0);
  }
  $151 = HEAP32[$rc>>2]|0;
  $152 = ($151|0)==(0);
  if ($152) {
   $153 = HEAP32[$pVfs>>2]|0;
   $154 = HEAP32[$zMaster>>2]|0;
   $155 = (_sqlite3OsOpenMalloc($153,$154,$pMaster,16406,0)|0);
   HEAP32[$rc>>2] = $155;
  }
  $156 = HEAP32[$rc>>2]|0;
  $157 = ($156|0)!=(0);
  if ($157) {
   $158 = HEAP32[$1>>2]|0;
   $159 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($158,$159);
   $160 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $160;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $161 = HEAP32[$i>>2]|0;
   $162 = HEAP32[$1>>2]|0;
   $163 = ((($162)) + 20|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = ($161|0)<($164|0);
   if (!($165)) {
    break;
   }
   $166 = HEAP32[$i>>2]|0;
   $167 = HEAP32[$1>>2]|0;
   $168 = ((($167)) + 16|0);
   $169 = HEAP32[$168>>2]|0;
   $170 = (($169) + ($166<<4)|0);
   $171 = ((($170)) + 4|0);
   $172 = HEAP32[$171>>2]|0;
   HEAP32[$pBt3>>2] = $172;
   $173 = HEAP32[$pBt3>>2]|0;
   $174 = (_sqlite3BtreeIsInTrans($173)|0);
   $175 = ($174|0)!=(0);
   if ($175) {
    $176 = HEAP32[$pBt3>>2]|0;
    $177 = (_sqlite3BtreeGetJournalname($176)|0);
    HEAP32[$zFile>>2] = $177;
    $178 = HEAP32[$zFile>>2]|0;
    $179 = ($178|0)==(0|0);
    if (!($179)) {
     $180 = HEAP32[$needSync>>2]|0;
     $181 = ($180|0)!=(0);
     if (!($181)) {
      $182 = HEAP32[$pBt3>>2]|0;
      $183 = (_sqlite3BtreeSyncDisabled($182)|0);
      $184 = ($183|0)!=(0);
      if (!($184)) {
       HEAP32[$needSync>>2] = 1;
      }
     }
     $185 = HEAP32[$pMaster>>2]|0;
     $186 = HEAP32[$zFile>>2]|0;
     $187 = HEAP32[$zFile>>2]|0;
     $188 = (_sqlite3Strlen30($187)|0);
     $189 = (($188) + 1)|0;
     $190 = $offset;
     $191 = $190;
     $192 = HEAP32[$191>>2]|0;
     $193 = (($190) + 4)|0;
     $194 = $193;
     $195 = HEAP32[$194>>2]|0;
     $196 = (_sqlite3OsWrite($185,$186,$189,$192,$195)|0);
     HEAP32[$rc>>2] = $196;
     $197 = HEAP32[$zFile>>2]|0;
     $198 = (_sqlite3Strlen30($197)|0);
     $199 = (($198) + 1)|0;
     $200 = ($199|0)<(0);
     $201 = $200 << 31 >> 31;
     $202 = $offset;
     $203 = $202;
     $204 = HEAP32[$203>>2]|0;
     $205 = (($202) + 4)|0;
     $206 = $205;
     $207 = HEAP32[$206>>2]|0;
     $208 = (_i64Add(($204|0),($207|0),($199|0),($201|0))|0);
     $209 = tempRet0;
     $210 = $offset;
     $211 = $210;
     HEAP32[$211>>2] = $208;
     $212 = (($210) + 4)|0;
     $213 = $212;
     HEAP32[$213>>2] = $209;
     $214 = HEAP32[$rc>>2]|0;
     $215 = ($214|0)!=(0);
     if ($215) {
      label = 50;
      break;
     }
    }
   }
   $222 = HEAP32[$i>>2]|0;
   $223 = (($222) + 1)|0;
   HEAP32[$i>>2] = $223;
  }
  if ((label|0) == 50) {
   $216 = HEAP32[$pMaster>>2]|0;
   (_sqlite3OsCloseFree($216)|0);
   $217 = HEAP32[$pVfs>>2]|0;
   $218 = HEAP32[$zMaster>>2]|0;
   (_sqlite3OsDelete($217,$218,0)|0);
   $219 = HEAP32[$1>>2]|0;
   $220 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($219,$220);
   $221 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $221;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
  $224 = HEAP32[$needSync>>2]|0;
  $225 = ($224|0)!=(0);
  if ($225) {
   $226 = HEAP32[$pMaster>>2]|0;
   $227 = (_sqlite3OsDeviceCharacteristics($226)|0);
   $228 = $227 & 1024;
   $229 = (0)==($228|0);
   if ($229) {
    $230 = HEAP32[$pMaster>>2]|0;
    $231 = (_sqlite3OsSync($230,2)|0);
    HEAP32[$rc>>2] = $231;
    $232 = (0)!=($231|0);
    if ($232) {
     $233 = HEAP32[$pMaster>>2]|0;
     (_sqlite3OsCloseFree($233)|0);
     $234 = HEAP32[$pVfs>>2]|0;
     $235 = HEAP32[$zMaster>>2]|0;
     (_sqlite3OsDelete($234,$235,0)|0);
     $236 = HEAP32[$1>>2]|0;
     $237 = HEAP32[$zMaster>>2]|0;
     _sqlite3DbFree($236,$237);
     $238 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $238;
     $294 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($294|0);
    }
   }
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $239 = HEAP32[$rc>>2]|0;
   $240 = ($239|0)==(0);
   if (!($240)) {
    break;
   }
   $241 = HEAP32[$i>>2]|0;
   $242 = HEAP32[$1>>2]|0;
   $243 = ((($242)) + 20|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = ($241|0)<($244|0);
   if (!($245)) {
    break;
   }
   $246 = HEAP32[$i>>2]|0;
   $247 = HEAP32[$1>>2]|0;
   $248 = ((($247)) + 16|0);
   $249 = HEAP32[$248>>2]|0;
   $250 = (($249) + ($246<<4)|0);
   $251 = ((($250)) + 4|0);
   $252 = HEAP32[$251>>2]|0;
   HEAP32[$pBt4>>2] = $252;
   $253 = HEAP32[$pBt4>>2]|0;
   $254 = ($253|0)!=(0|0);
   if ($254) {
    $255 = HEAP32[$pBt4>>2]|0;
    $256 = HEAP32[$zMaster>>2]|0;
    $257 = (_sqlite3BtreeCommitPhaseOne($255,$256)|0);
    HEAP32[$rc>>2] = $257;
   }
   $258 = HEAP32[$i>>2]|0;
   $259 = (($258) + 1)|0;
   HEAP32[$i>>2] = $259;
  }
  $260 = HEAP32[$pMaster>>2]|0;
  (_sqlite3OsCloseFree($260)|0);
  $261 = HEAP32[$rc>>2]|0;
  $262 = ($261|0)!=(0);
  if ($262) {
   $263 = HEAP32[$1>>2]|0;
   $264 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($263,$264);
   $265 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $265;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
  $266 = HEAP32[$pVfs>>2]|0;
  $267 = HEAP32[$zMaster>>2]|0;
  $268 = HEAP32[$needSync>>2]|0;
  $269 = (_sqlite3OsDelete($266,$267,$268)|0);
  HEAP32[$rc>>2] = $269;
  $270 = HEAP32[$1>>2]|0;
  $271 = HEAP32[$zMaster>>2]|0;
  _sqlite3DbFree($270,$271);
  HEAP32[$zMaster>>2] = 0;
  $272 = HEAP32[$rc>>2]|0;
  $273 = ($272|0)!=(0);
  if ($273) {
   $274 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $274;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
  _sqlite3BeginBenignMalloc();
  HEAP32[$i>>2] = 0;
  while(1) {
   $275 = HEAP32[$i>>2]|0;
   $276 = HEAP32[$1>>2]|0;
   $277 = ((($276)) + 20|0);
   $278 = HEAP32[$277>>2]|0;
   $279 = ($275|0)<($278|0);
   if (!($279)) {
    break;
   }
   $280 = HEAP32[$i>>2]|0;
   $281 = HEAP32[$1>>2]|0;
   $282 = ((($281)) + 16|0);
   $283 = HEAP32[$282>>2]|0;
   $284 = (($283) + ($280<<4)|0);
   $285 = ((($284)) + 4|0);
   $286 = HEAP32[$285>>2]|0;
   HEAP32[$pBt5>>2] = $286;
   $287 = HEAP32[$pBt5>>2]|0;
   $288 = ($287|0)!=(0|0);
   if ($288) {
    $289 = HEAP32[$pBt5>>2]|0;
    (_sqlite3BtreeCommitPhaseTwo($289,1)|0);
   }
   $290 = HEAP32[$i>>2]|0;
   $291 = (($290) + 1)|0;
   HEAP32[$i>>2] = $291;
  }
  _sqlite3EndBenignMalloc();
  $292 = HEAP32[$1>>2]|0;
  (_sqlite3VtabCommit($292)|0);
 }
 $293 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $293;
 $294 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($294|0);
}
function _sqlite3CommitInternalChanges($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 & -3;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3VdbeCloseStatement($p,$eOp) {
 $p = $p|0;
 $eOp = $eOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $db = 0, $i = 0, $iSavepoint = 0, $pBt = 0, $rc = 0, $rc2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $db = sp + 20|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $iSavepoint = sp + 8|0;
 $rc2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eOp;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 504|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 104|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 104|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) - 1)|0;
 HEAP32[$iSavepoint>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$db>>2]|0;
  $18 = ((($17)) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  HEAP32[$rc2>>2] = 0;
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$db>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pBt>>2] = $27;
  $28 = HEAP32[$pBt>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = ($30|0)==(2);
   if ($31) {
    $32 = HEAP32[$pBt>>2]|0;
    $33 = HEAP32[$iSavepoint>>2]|0;
    $34 = (_sqlite3BtreeSavepoint($32,2,$33)|0);
    HEAP32[$rc2>>2] = $34;
   }
   $35 = HEAP32[$rc2>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$pBt>>2]|0;
    $38 = HEAP32[$iSavepoint>>2]|0;
    $39 = (_sqlite3BtreeSavepoint($37,1,$38)|0);
    HEAP32[$rc2>>2] = $39;
   }
   $40 = HEAP32[$rc>>2]|0;
   $41 = ($40|0)==(0);
   if ($41) {
    $42 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $42;
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$db>>2]|0;
 $46 = ((($45)) + 504|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + -1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 104|0);
 HEAP32[$50>>2] = 0;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = ($53|0)==(2);
  if ($54) {
   $55 = HEAP32[$db>>2]|0;
   $56 = HEAP32[$iSavepoint>>2]|0;
   $57 = (_sqlite3VtabSavepoint($55,2,$56)|0);
   HEAP32[$rc>>2] = $57;
  }
  $58 = HEAP32[$rc>>2]|0;
  $59 = ($58|0)==(0);
  if ($59) {
   $60 = HEAP32[$db>>2]|0;
   $61 = HEAP32[$iSavepoint>>2]|0;
   $62 = (_sqlite3VtabSavepoint($60,1,$61)|0);
   HEAP32[$rc>>2] = $62;
  }
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = ($63|0)==(2);
 if (!($64)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = ((($65)) + 152|0);
 $67 = $66;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = HEAP32[$db>>2]|0;
 $74 = ((($73)) + 512|0);
 $75 = $74;
 $76 = $75;
 HEAP32[$76>>2] = $69;
 $77 = (($75) + 4)|0;
 $78 = $77;
 HEAP32[$78>>2] = $72;
 $79 = HEAP32[$0>>2]|0;
 $80 = ((($79)) + 160|0);
 $81 = $80;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = HEAP32[$db>>2]|0;
 $88 = ((($87)) + 520|0);
 $89 = $88;
 $90 = $89;
 HEAP32[$90>>2] = $83;
 $91 = (($89) + 4)|0;
 $92 = $91;
 HEAP32[$92>>2] = $86;
 $93 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3VdbeSetChanges($db,$nChange) {
 $db = $db|0;
 $nChange = $nChange|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nChange;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 80|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 84|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($5))|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3VdbeFrameRestore($pFrame) {
 $pFrame = $pFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pFrame;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$v>>2]|0;
 _closeCursorsInFrame($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ((($7)) + 200|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$v>>2]|0;
 $13 = ((($12)) + 196|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$v>>2]|0;
 $18 = ((($17)) + 4|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 48|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$v>>2]|0;
 $23 = ((($22)) + 32|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$v>>2]|0;
 $28 = ((($27)) + 8|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 52|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$v>>2]|0;
 $33 = ((($32)) + 28|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 24|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$v>>2]|0;
 $38 = ((($37)) + 56|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 40|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$v>>2]|0;
 $43 = ((($42)) + 36|0);
 HEAP32[$43>>2] = $41;
 $44 = HEAP32[$0>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = $45;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) + 4)|0;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$v>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = ((($53)) + 32|0);
 $55 = $54;
 $56 = $55;
 HEAP32[$56>>2] = $48;
 $57 = (($55) + 4)|0;
 $58 = $57;
 HEAP32[$58>>2] = $51;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 68|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$v>>2]|0;
 $63 = ((($62)) + 92|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 72|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$v>>2]|0;
 $68 = HEAP32[$67>>2]|0;
 $69 = ((($68)) + 80|0);
 HEAP32[$69>>2] = $66;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 44|0);
 $72 = HEAP32[$71>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _closeCursorsInFrame($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pC = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $pC = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 56|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 36|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 56|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$pC>>2] = $15;
  $16 = HEAP32[$pC>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$pC>>2]|0;
   _sqlite3VdbeFreeCursor($18,$19);
   $20 = HEAP32[$i>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 56|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + ($20<<2)|0);
   HEAP32[$24>>2] = 0;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeFrameDelete($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aMem = 0, $apCsr = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $aMem = sp + 4|0;
 $apCsr = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 HEAP32[$aMem>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$aMem>>2]|0;
 $7 = (($6) + (($5*40)|0)|0);
 HEAP32[$apCsr>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 64|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$apCsr>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3VdbeFreeCursor($14,$18);
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$aMem>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 60|0);
 $24 = HEAP32[$23>>2]|0;
 _releaseMemArray($21,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($27,$28);
 STACKTOP = sp;return;
}
function _sqlite3VdbeDeleteAuxData($pVdbe,$iOp,$mask) {
 $pVdbe = $pVdbe|0;
 $iOp = $iOp|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAux = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pp = sp + 4|0;
 $pAux = sp;
 HEAP32[$0>>2] = $pVdbe;
 HEAP32[$1>>2] = $iOp;
 HEAP32[$2>>2] = $mask;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 204|0);
 HEAP32[$pp>>2] = $4;
 L1: while(1) {
  $5 = HEAP32[$pp>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$pp>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pAux>>2] = $9;
  $10 = HEAP32[$1>>2]|0;
  $11 = ($10|0)<(0);
  do {
   if (!($11)) {
    $12 = HEAP32[$pAux>>2]|0;
    $13 = HEAP32[$12>>2]|0;
    $14 = HEAP32[$1>>2]|0;
    $15 = ($13|0)==($14|0);
    if ($15) {
     $16 = HEAP32[$pAux>>2]|0;
     $17 = ((($16)) + 4|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = ($18|0)>(31);
     if ($19) {
      break;
     }
     $20 = HEAP32[$2>>2]|0;
     $21 = HEAP32[$pAux>>2]|0;
     $22 = ((($21)) + 4|0);
     $23 = HEAP32[$22>>2]|0;
     $24 = 1 << $23;
     $25 = $20 & $24;
     $26 = ($25|0)!=(0);
     if (!($26)) {
      break;
     }
    }
    $44 = HEAP32[$pAux>>2]|0;
    $45 = ((($44)) + 16|0);
    HEAP32[$pp>>2] = $45;
    continue L1;
   }
  } while(0);
  $27 = HEAP32[$pAux>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)!=(0|0);
  if ($30) {
   $31 = HEAP32[$pAux>>2]|0;
   $32 = ((($31)) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$pAux>>2]|0;
   $35 = ((($34)) + 8|0);
   $36 = HEAP32[$35>>2]|0;
   FUNCTION_TABLE_vi[$33 & 255]($36);
  }
  $37 = HEAP32[$pAux>>2]|0;
  $38 = ((($37)) + 16|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$pp>>2]|0;
  HEAP32[$40>>2] = $39;
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$pAux>>2]|0;
  _sqlite3DbFree($42,$43);
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeFreeCursor($p,$pCx) {
 $p = $p|0;
 $pCx = $pCx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pModule = 0, $pVCur = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVCur = sp + 4|0;
 $pModule = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pCx;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 switch ($6|0) {
 case 1:  {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  _sqlite3VdbeSorterClose($8,$9);
  STACKTOP = sp;return;
  break;
 }
 case 0:  {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0|0);
  $14 = HEAP32[$1>>2]|0;
  if ($13) {
   $15 = ((($14)) + 20|0);
   $16 = HEAP32[$15>>2]|0;
   (_sqlite3BtreeClose($16)|0);
   STACKTOP = sp;return;
  } else {
   $17 = ((($14)) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   (_sqlite3BtreeCloseCursor($18)|0);
   STACKTOP = sp;return;
  }
  break;
 }
 case 2:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pVCur>>2] = $21;
  $22 = HEAP32[$pVCur>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pModule>>2] = $24;
  $25 = HEAP32[$pVCur>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + -1)|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$pModule>>2]|0;
  $31 = ((($30)) + 28|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$pVCur>>2]|0;
  (FUNCTION_TABLE_ii[$32 & 255]($33)|0);
  STACKTOP = sp;return;
  break;
 }
 default: {
  STACKTOP = sp;return;
 }
 }
}
function _sqlite3VdbeSorterClose($db,$pCsr) {
 $db = $db|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSorter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pSorter = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pCsr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pSorter>>2] = $4;
 $5 = HEAP32[$pSorter>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pSorter>>2]|0;
 _sqlite3VdbeSorterReset($7,$8);
 $9 = HEAP32[$pSorter>>2]|0;
 $10 = ((($9)) + 36|0);
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3_free($12);
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pSorter>>2]|0;
 _sqlite3DbFree($13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 16|0);
 HEAP32[$16>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeClose($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $pBt = 0, $pCur = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBt = sp + 8|0;
 $pCur = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pCur>>2] = $7;
 while(1) {
  $8 = HEAP32[$pCur>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pCur>>2]|0;
  HEAP32[$pTmp>>2] = $10;
  $11 = HEAP32[$pCur>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pCur>>2] = $13;
  $14 = HEAP32[$pTmp>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ($15|0)==($16|0);
  if (!($17)) {
   continue;
  }
  $18 = HEAP32[$pTmp>>2]|0;
  (_sqlite3BtreeCloseCursor($18)|0);
 }
 $19 = HEAP32[$0>>2]|0;
 (_sqlite3BtreeRollback($19,0,0)|0);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 9|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = ($22<<24>>24)!=(0);
 if ($23) {
  $24 = HEAP32[$pBt>>2]|0;
  $25 = (_removeFromSharingList($24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   label = 7;
  }
 } else {
  label = 7;
 }
 if ((label|0) == 7) {
  $27 = HEAP32[$pBt>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  (_sqlite3PagerClose($28)|0);
  $29 = HEAP32[$pBt>>2]|0;
  $30 = ((($29)) + 52|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = HEAP32[$pBt>>2]|0;
   $34 = ((($33)) + 48|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$pBt>>2]|0;
    $38 = ((($37)) + 52|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pBt>>2]|0;
    $41 = ((($40)) + 48|0);
    $42 = HEAP32[$41>>2]|0;
    FUNCTION_TABLE_vi[$39 & 255]($42);
   }
  }
  $43 = HEAP32[$pBt>>2]|0;
  $44 = ((($43)) + 48|0);
  $45 = HEAP32[$44>>2]|0;
  _sqlite3DbFree(0,$45);
  $46 = HEAP32[$pBt>>2]|0;
  _freeTempSpace($46);
  $47 = HEAP32[$pBt>>2]|0;
  _sqlite3_free($47);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 28|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0|0);
 if ($51) {
  $52 = HEAP32[$0>>2]|0;
  $53 = ((($52)) + 24|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$0>>2]|0;
  $56 = ((($55)) + 28|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ((($57)) + 24|0);
  HEAP32[$58>>2] = $54;
 }
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 24|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = ($61|0)!=(0|0);
 if (!($62)) {
  $70 = HEAP32[$0>>2]|0;
  _sqlite3_free($70);
  STACKTOP = sp;return 0;
 }
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 28|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = ((($66)) + 24|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ((($68)) + 28|0);
 HEAP32[$69>>2] = $65;
 $70 = HEAP32[$0>>2]|0;
 _sqlite3_free($70);
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeCloseCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pBt = 0, $pBtree = 0, $pPrev = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pBtree = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $pPrev = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pBtree>>2] = $2;
 $3 = HEAP32[$pBtree>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return 0;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$pBtree>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$0>>2]|0;
 _sqlite3BtreeClearCursor($9);
 $10 = HEAP32[$pBt>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ($12|0)==($13|0);
 L4: do {
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$pBt>>2]|0;
   $19 = ((($18)) + 8|0);
   HEAP32[$19>>2] = $17;
  } else {
   $20 = HEAP32[$pBt>>2]|0;
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pPrev>>2] = $22;
   while(1) {
    $23 = HEAP32[$pPrev>>2]|0;
    $24 = ((($23)) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$0>>2]|0;
    $27 = ($25|0)==($26|0);
    if ($27) {
     break;
    }
    $33 = HEAP32[$pPrev>>2]|0;
    $34 = ((($33)) + 8|0);
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pPrev>>2] = $35;
    $36 = HEAP32[$pPrev>>2]|0;
    $37 = ($36|0)!=(0|0);
    if (!($37)) {
     break L4;
    }
   }
   $28 = HEAP32[$0>>2]|0;
   $29 = ((($28)) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$pPrev>>2]|0;
   $32 = ((($31)) + 8|0);
   HEAP32[$32>>2] = $30;
  }
 } while(0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 68|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = ($38|0)<=($42|0);
  if (!($43)) {
   break;
  }
  $44 = HEAP32[$i>>2]|0;
  $45 = HEAP32[$0>>2]|0;
  $46 = ((($45)) + 120|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  _releasePage($48);
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$pBt>>2]|0;
 _unlockBtreeIfUnused($51);
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 _sqlite3_free($54);
 STACKTOP = sp;return 0;
}
function _sqlite3VdbeSorterReset($db,$pSorter) {
 $db = $db|0;
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pTask = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSorter;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _vdbeMergeEngineFree($4);
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 HEAP32[$6>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 59|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($7|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 64|0);
  $16 = (($15) + (($13*72)|0)|0);
  HEAP32[$pTask>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$pTask>>2]|0;
  _vdbeSortSubtaskCleanup($17,$18);
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$pTask>>2]|0;
  $21 = ((($20)) + 8|0);
  HEAP32[$21>>2] = $19;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 36|0);
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 36|0);
  $31 = HEAP32[$30>>2]|0;
  _vdbeSorterRecordFree(0,$31);
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 36|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 36|0);
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 56|0);
 HEAP8[$38>>0] = 0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 48|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 8|0);
 HEAP32[$42>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = HEAP32[$45>>2]|0;
 _sqlite3DbFree($43,$46);
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 32|0);
 HEAP32[$48>>2] = 0;
 STACKTOP = sp;return;
}
function _vdbeMergeEngineFree($pMerger) {
 $pMerger = $pMerger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pMerger;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_free($14);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*56)|0)|0);
  _vdbePmaReaderClear($11);
  $12 = HEAP32[$i>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$i>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_free($14);
 STACKTOP = sp;return;
}
function _vdbeSortSubtaskCleanup($db,$pTask) {
 $db = $db|0;
 $pTask = $pTask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTask;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3DbFree($2,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 _vdbeSorterRecordFree(0,$8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 40|0);
  $15 = HEAP32[$14>>2]|0;
  (_sqlite3OsCloseFree($15)|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 56|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $23 = HEAP32[$1>>2]|0;
  dest=$23; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 56|0);
 $22 = HEAP32[$21>>2]|0;
 (_sqlite3OsCloseFree($22)|0);
 $23 = HEAP32[$1>>2]|0;
 dest=$23; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _vdbeSorterRecordFree($db,$pRecord) {
 $db = $db|0;
 $pRecord = $pRecord|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pRecord;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pNext>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($8,$9);
  $10 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $10;
 }
 STACKTOP = sp;return;
}
function _vdbePmaReaderClear($pReadr) {
 $pReadr = $pReadr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pReadr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 44|0);
  $16 = HEAP32[$15>>2]|0;
  (_sqlite3OsUnfetch($13,0,0,$16)|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 _vdbeIncrFree($19);
 $20 = HEAP32[$0>>2]|0;
 dest=$20; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _vdbeIncrFree($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pIncr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _vdbeMergeEngineFree($5);
 $6 = HEAP32[$0>>2]|0;
 _sqlite3_free($6);
 STACKTOP = sp;return;
}
function _sqlite3OsCloseFree($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3OsClose($1)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _removeFromSharingList($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $removed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pList = sp + 4|0;
 $removed = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$removed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<=(0);
 if (!($8)) {
  $32 = HEAP32[$removed>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $9 = HEAP32[1684>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ($9|0)==($10|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 68|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[1684>>2] = $14;
 } else {
  $15 = HEAP32[1684>>2]|0;
  HEAP32[$pList>>2] = $15;
  while(1) {
   $16 = HEAP32[$pList>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    $18 = HEAP32[$pList>>2]|0;
    $19 = ((($18)) + 68|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ($20|0)!=($21|0);
    $33 = $22;
   } else {
    $33 = 0;
   }
   $23 = HEAP32[$pList>>2]|0;
   if (!($33)) {
    break;
   }
   $24 = ((($23)) + 68|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pList>>2] = $25;
  }
  $26 = ($23|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$0>>2]|0;
   $28 = ((($27)) + 68|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$pList>>2]|0;
   $31 = ((($30)) + 68|0);
   HEAP32[$31>>2] = $29;
  }
 }
 HEAP32[$removed>>2] = 1;
 $32 = HEAP32[$removed>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3VtabRollback($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _callFinaliser($1,68);
 STACKTOP = sp;return 0;
}
function _callFinaliser($db,$offset) {
 $db = $db|0;
 $offset = $offset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $aVTrans = 0, $i = 0, $p = 0, $pVTab = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $i = sp + 16|0;
 $aVTrans = sp + 12|0;
 $pVTab = sp + 8|0;
 $p = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $offset;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 332|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 332|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aVTrans>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 332|0);
 HEAP32[$10>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 308|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$aVTrans>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVTab>>2] = $19;
  $20 = HEAP32[$pVTab>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$p>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = (($26) + ($27)|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$x>>2] = $29;
   $30 = HEAP32[$x>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$x>>2]|0;
    $33 = HEAP32[$p>>2]|0;
    (FUNCTION_TABLE_ii[$32 & 255]($33)|0);
   }
  }
  $34 = HEAP32[$pVTab>>2]|0;
  $35 = ((($34)) + 20|0);
  HEAP32[$35>>2] = 0;
  $36 = HEAP32[$pVTab>>2]|0;
  _sqlite3VtabUnlock($36);
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$aVTrans>>2]|0;
 _sqlite3DbFree($39,$40);
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 308|0);
 HEAP32[$42>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeError($p,$zFormat,$varargs) {
 $p = $p|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3DbFree($3,$6);
 HEAP32[$ap>>2] = $varargs;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3VMPrintf($8,$9,$ap)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 44|0);
 HEAP32[$12>>2] = $10;
 STACKTOP = sp;return;
}
function _sqlite3VtabSync($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aVTrans = 0, $i = 0;
 var $pVtab = 0, $rc = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $aVTrans = sp + 8|0;
 $x = sp + 4|0;
 $pVtab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 332|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$aVTrans>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 332|0);
 HEAP32[$6>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 8;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 308|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   label = 8;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$aVTrans>>2]|0;
  $16 = (($15) + ($14<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVtab>>2] = $19;
  $20 = HEAP32[$pVtab>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$pVtab>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = ((($23)) + 60|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$x>>2] = $25;
   $26 = ($25|0)!=(0|0);
   if ($26) {
    $27 = HEAP32[$x>>2]|0;
    $28 = HEAP32[$pVtab>>2]|0;
    $29 = (FUNCTION_TABLE_ii[$27 & 255]($28)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$1>>2]|0;
    $31 = HEAP32[$pVtab>>2]|0;
    _sqlite3VtabImportErrmsg($30,$31);
   }
  }
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 if ((label|0) == 8) {
  $34 = HEAP32[$aVTrans>>2]|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 332|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3VtabCommit($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _callFinaliser($1,64);
 STACKTOP = sp;return 0;
}
function _sqlite3OsOpenMalloc($pVfs,$zFile,$ppFile,$flags,$pOutFlags) {
 $pVfs = $pVfs|0;
 $zFile = $zFile|0;
 $ppFile = $ppFile|0;
 $flags = $flags|0;
 $pOutFlags = $pOutFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zFile;
 HEAP32[$2>>2] = $ppFile;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $pOutFlags;
 HEAP32[$rc>>2] = 7;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<(0);
 $9 = $8 << 31 >> 31;
 $10 = (_sqlite3MallocZero($7,$9)|0);
 HEAP32[$pFile>>2] = $10;
 $11 = HEAP32[$pFile>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$pFile>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = (_sqlite3OsOpen($13,$14,$15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 $21 = HEAP32[$pFile>>2]|0;
 if ($20) {
  _sqlite3_free($21);
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  $22 = HEAP32[$2>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _sqlite3BtreeGetJournalname($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3PagerJournalname($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3BtreeSyncDisabled($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3PagerNosync($6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VtabImportErrmsg($p,$pVtab) {
 $p = $p|0;
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pVtab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DbFree($8,$11);
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3DbStrDup($12,$15)|0);
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 44|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 _sqlite3_free($21);
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 8|0);
 HEAP32[$23>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3PagerJournalname($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 180|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerNosync($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 7|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeSavepoint($p,$op,$iSavepoint) {
 $p = $p|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if (!($9)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pBt>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($13);
 $14 = HEAP32[$pBt>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = (_sqlite3PagerSavepoint($15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)==(0);
 if (!($20)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = ($21|0)<(0);
 if ($22) {
  $23 = HEAP32[$pBt>>2]|0;
  $24 = ((($23)) + 22|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26 & 8;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$pBt>>2]|0;
   $30 = ((($29)) + 44|0);
   HEAP32[$30>>2] = 0;
  }
 }
 $31 = HEAP32[$pBt>>2]|0;
 $32 = (_newDatabase($31)|0);
 HEAP32[$rc>>2] = $32;
 $33 = HEAP32[$pBt>>2]|0;
 $34 = ((($33)) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ((($35)) + 56|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ((($37)) + 28|0);
 $39 = (_sqlite3Get4byte($38)|0);
 $40 = HEAP32[$pBt>>2]|0;
 $41 = ((($40)) + 44|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3VtabSavepoint($db,$op,$iSavepoint) {
 $db = $db|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pMod = 0, $pVTab = 0, $rc = 0, $xMethod = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $pVTab = sp + 8|0;
 $pMod = sp + 4|0;
 $xMethod = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 332|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 15;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 308|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   label = 15;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 332|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVTab>>2] = $19;
  $20 = HEAP32[$pVTab>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pMod>>2] = $23;
  $24 = HEAP32[$pVTab>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[$pMod>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)>=(2);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    switch ($31|0) {
    case 0:  {
     $32 = HEAP32[$pMod>>2]|0;
     $33 = ((($32)) + 80|0);
     $34 = HEAP32[$33>>2]|0;
     HEAP32[$xMethod>>2] = $34;
     $35 = HEAP32[$2>>2]|0;
     $36 = (($35) + 1)|0;
     $37 = HEAP32[$pVTab>>2]|0;
     $38 = ((($37)) + 20|0);
     HEAP32[$38>>2] = $36;
     break;
    }
    case 2:  {
     $39 = HEAP32[$pMod>>2]|0;
     $40 = ((($39)) + 88|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$xMethod>>2] = $41;
     break;
    }
    default: {
     $42 = HEAP32[$pMod>>2]|0;
     $43 = ((($42)) + 84|0);
     $44 = HEAP32[$43>>2]|0;
     HEAP32[$xMethod>>2] = $44;
    }
    }
    $45 = HEAP32[$xMethod>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$pVTab>>2]|0;
     $48 = ((($47)) + 20|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = HEAP32[$2>>2]|0;
     $51 = ($49|0)>($50|0);
     if ($51) {
      $52 = HEAP32[$xMethod>>2]|0;
      $53 = HEAP32[$pVTab>>2]|0;
      $54 = ((($53)) + 8|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$2>>2]|0;
      $57 = (FUNCTION_TABLE_iii[$52 & 255]($55,$56)|0);
      HEAP32[$rc>>2] = $57;
     }
    }
   }
  }
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 if ((label|0) == 15) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return (0)|0;
}
function _vdbeMemClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($7);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 16|0);
  HEAP32[$21>>2] = 0;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 32|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3DbFree($14,$17);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 16|0);
 HEAP32[$21>>2] = 0;
 STACKTOP = sp;return;
}
function _vdbeMemClearExternAndSetNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFrame = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFrame = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 8192;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  (_sqlite3VdbeMemFinalize($7,$9)|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 & 1024;
 $15 = ($14|0)!=(0);
 $16 = HEAP32[$0>>2]|0;
 do {
  if ($15) {
   $17 = ((($16)) + 36|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   FUNCTION_TABLE_vi[$18 & 255]($21);
  } else {
   $22 = ((($16)) + 8|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   $25 = $24 & 32;
   $26 = ($25|0)!=(0);
   $27 = HEAP32[$0>>2]|0;
   if ($26) {
    $28 = HEAP32[$27>>2]|0;
    _sqlite3RowSetClear($28);
    break;
   }
   $29 = ((($27)) + 8|0);
   $30 = HEAP16[$29>>1]|0;
   $31 = $30&65535;
   $32 = $31 & 64;
   $33 = ($32|0)!=(0);
   if ($33) {
    $34 = HEAP32[$0>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pFrame>>2] = $35;
    $36 = HEAP32[$pFrame>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = ((($37)) + 180|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pFrame>>2]|0;
    $41 = ((($40)) + 4|0);
    HEAP32[$41>>2] = $39;
    $42 = HEAP32[$pFrame>>2]|0;
    $43 = HEAP32[$pFrame>>2]|0;
    $44 = HEAP32[$43>>2]|0;
    $45 = ((($44)) + 180|0);
    HEAP32[$45>>2] = $42;
   }
  }
 } while(0);
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 8|0);
 HEAP16[$47>>1] = 1;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemFinalize($pMem,$pFunc) {
 $pMem = $pMem|0;
 $pFunc = $pFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ctx = 0, $rc = 0, $t = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $rc = sp + 72|0;
 $ctx = sp + 40|0;
 $t = sp;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $pFunc;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 ;HEAP32[$ctx>>2]=0|0;HEAP32[$ctx+4>>2]=0|0;HEAP32[$ctx+8>>2]=0|0;HEAP32[$ctx+12>>2]=0|0;HEAP32[$ctx+16>>2]=0|0;HEAP32[$ctx+20>>2]=0|0;HEAP32[$ctx+24>>2]=0|0;HEAP32[$ctx+28>>2]=0|0;
 dest=$t; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = ((($t)) + 8|0);
 HEAP16[$8>>1] = 1;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($t)) + 32|0);
 HEAP32[$12>>2] = $11;
 HEAP32[$ctx>>2] = $t;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($ctx)) + 8|0);
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($ctx)) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 FUNCTION_TABLE_vi[$19 & 255]($ctx);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 24|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)>(0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 32|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 20|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3DbFree($26,$29);
 }
 $30 = HEAP32[$0>>2]|0;
 dest=$30; src=$t; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $31 = ((($ctx)) + 20|0);
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$rc>>2] = $32;
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3RowSetClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pChunk = 0, $pNextChunk = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pChunk = sp + 4|0;
 $pNextChunk = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pChunk>>2] = $2;
 while(1) {
  $3 = HEAP32[$pChunk>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$pChunk>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$pNextChunk>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$pChunk>>2]|0;
  _sqlite3DbFree($9,$10);
  $11 = HEAP32[$pNextChunk>>2]|0;
  HEAP32[$pChunk>>2] = $11;
 }
 $12 = HEAP32[$0>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 24|0);
 HEAP16[$14>>1] = 0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = 0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 12|0);
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 20|0);
 HEAP32[$20>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 26|0);
 HEAP16[$22>>1] = 1;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemGrow($pMem,$n,$bPreserve) {
 $pMem = $pMem|0;
 $n = $n|0;
 $bPreserve = $bPreserve|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $bPreserve;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($6|0)<($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$2>>2]|0;
   $10 = ($9|0)<(32);
   if ($10) {
    HEAP32[$2>>2] = 32;
   }
   $11 = HEAP32[$3>>2]|0;
   $12 = ($11|0)!=(0);
   if ($12) {
    $13 = HEAP32[$1>>2]|0;
    $14 = ((($13)) + 24|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = ($15|0)>(0);
    if ($16) {
     $17 = HEAP32[$1>>2]|0;
     $18 = ((($17)) + 16|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = HEAP32[$1>>2]|0;
     $21 = ((($20)) + 20|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = ($19|0)==($22|0);
     if ($23) {
      $24 = HEAP32[$1>>2]|0;
      $25 = ((($24)) + 32|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = HEAP32[$1>>2]|0;
      $28 = ((($27)) + 16|0);
      $29 = HEAP32[$28>>2]|0;
      $30 = HEAP32[$2>>2]|0;
      $31 = ($30|0)<(0);
      $32 = $31 << 31 >> 31;
      $33 = (_sqlite3DbReallocOrFree($26,$29,$30,$32)|0);
      $34 = HEAP32[$1>>2]|0;
      $35 = ((($34)) + 20|0);
      HEAP32[$35>>2] = $33;
      $36 = HEAP32[$1>>2]|0;
      $37 = ((($36)) + 16|0);
      HEAP32[$37>>2] = $33;
      HEAP32[$3>>2] = 0;
     } else {
      label = 8;
     }
    } else {
     label = 8;
    }
   } else {
    label = 8;
   }
   if ((label|0) == 8) {
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 24|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = ($40|0)>(0);
    if ($41) {
     $42 = HEAP32[$1>>2]|0;
     $43 = ((($42)) + 32|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$1>>2]|0;
     $46 = ((($45)) + 20|0);
     $47 = HEAP32[$46>>2]|0;
     _sqlite3DbFree($44,$47);
    }
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 32|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$2>>2]|0;
    $52 = ($51|0)<(0);
    $53 = $52 << 31 >> 31;
    $54 = (_sqlite3DbMallocRaw($50,$51,$53)|0);
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 20|0);
    HEAP32[$56>>2] = $54;
   }
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 20|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)==(0|0);
   $61 = HEAP32[$1>>2]|0;
   if (!($60)) {
    $66 = ((($61)) + 32|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = HEAP32[$1>>2]|0;
    $69 = ((($68)) + 20|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (_sqlite3DbMallocSize($67,$70)|0);
    $72 = HEAP32[$1>>2]|0;
    $73 = ((($72)) + 24|0);
    HEAP32[$73>>2] = $71;
    break;
   }
   _sqlite3VdbeMemSetNull($61);
   $62 = HEAP32[$1>>2]|0;
   $63 = ((($62)) + 16|0);
   HEAP32[$63>>2] = 0;
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 24|0);
   HEAP32[$65>>2] = 0;
   HEAP32[$0>>2] = 7;
   $119 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($119|0);
  }
 } while(0);
 $74 = HEAP32[$3>>2]|0;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$1>>2]|0;
  $77 = ((($76)) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(0|0);
  if ($79) {
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 16|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 20|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = ($82|0)!=($85|0);
   if ($86) {
    $87 = HEAP32[$1>>2]|0;
    $88 = ((($87)) + 20|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$1>>2]|0;
    $91 = ((($90)) + 16|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = HEAP32[$1>>2]|0;
    $94 = ((($93)) + 12|0);
    $95 = HEAP32[$94>>2]|0;
    _memcpy(($89|0),($92|0),($95|0))|0;
   }
  }
 }
 $96 = HEAP32[$1>>2]|0;
 $97 = ((($96)) + 8|0);
 $98 = HEAP16[$97>>1]|0;
 $99 = $98&65535;
 $100 = $99 & 1024;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = HEAP32[$1>>2]|0;
  $103 = ((($102)) + 36|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 16|0);
  $107 = HEAP32[$106>>2]|0;
  FUNCTION_TABLE_vi[$104 & 255]($107);
 }
 $108 = HEAP32[$1>>2]|0;
 $109 = ((($108)) + 20|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$1>>2]|0;
 $112 = ((($111)) + 16|0);
 HEAP32[$112>>2] = $110;
 $113 = HEAP32[$1>>2]|0;
 $114 = ((($113)) + 8|0);
 $115 = HEAP16[$114>>1]|0;
 $116 = $115&65535;
 $117 = $116 & -7169;
 $118 = $117&65535;
 HEAP16[$114>>1] = $118;
 HEAP32[$0>>2] = 0;
 $119 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($119|0);
}
function _sqlite3DbReallocOrFree($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pNew = sp + 8|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3DbRealloc($9,$10,$13,$16)|0);
 HEAP32[$pNew>>2] = $17;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $22 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$3>>2]|0;
 _sqlite3DbFree($20,$21);
 $22 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _valueBytes($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $pVal;
 HEAP8[$1>>0] = $enc;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = (_valueToText($2,$3)|0);
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $9 = 0;
  STACKTOP = sp;return ($9|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8;
 STACKTOP = sp;return ($9|0);
}
function _valueToText($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 18;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$1>>2]|0;
 if ($8) {
  $10 = ((($9)) + 8|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12 | 2;
  $14 = $13&65535;
  HEAP16[$10>>1] = $14;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & 16384;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   (_sqlite3VdbeMemExpandBlob($21)|0);
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 10|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = HEAP8[$2>>0]|0;
  $27 = $26&255;
  $28 = $27 & -9;
  $29 = ($25|0)!=($28|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = HEAP8[$2>>0]|0;
   $32 = $31&255;
   $33 = $32 & -9;
   (_sqlite3VdbeChangeEncoding($30,$33)|0);
  }
  $34 = HEAP8[$2>>0]|0;
  $35 = $34&255;
  $36 = $35 & 8;
  $37 = ($36|0)!=(0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = $40;
   $42 = 1 & $41;
   $43 = (1)==($42|0);
   if ($43) {
    $44 = HEAP32[$1>>2]|0;
    $45 = (_sqlite3VdbeMemMakeWriteable($44)|0);
    $46 = ($45|0)!=(0);
    if ($46) {
     HEAP32[$0>>2] = 0;
     $60 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($60|0);
    }
   }
  }
  $47 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemNulTerminate($47)|0);
 } else {
  $48 = HEAP8[$2>>0]|0;
  (_sqlite3VdbeMemStringify($9,$48,0)|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 10|0);
 $51 = HEAP8[$50>>0]|0;
 $52 = $51&255;
 $53 = HEAP8[$2>>0]|0;
 $54 = $53&255;
 $55 = $54 & -9;
 $56 = ($52|0)==($55|0);
 if ($56) {
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 16|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$0>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 } else {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return (0)|0;
}
function _sqlite3VdbeChangeEncoding($pMem,$desiredEnc) {
 $pMem = $pMem|0;
 $desiredEnc = $desiredEnc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $desiredEnc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 10|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12|0)==($13|0);
  if (!($14)) {
   $15 = HEAP32[$1>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = $16&255;
   $18 = (_sqlite3VdbeMemTranslate($15,$17)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $19;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3VdbeMemNulTerminate($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 514;
 $7 = ($6|0)!=(2);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_vdbeMemAddTerminator($8)|0);
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemStringify($pMem,$enc,$bForce) {
 $pMem = $pMem|0;
 $enc = $enc|0;
 $bForce = $bForce|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fg = 0, $nByte = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 33|0;
 $3 = sp + 32|0;
 $fg = sp + 20|0;
 $nByte = sp + 16|0;
 HEAP32[$1>>2] = $pMem;
 HEAP8[$2>>0] = $enc;
 HEAP8[$3>>0] = $bForce;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$fg>>2] = $7;
 HEAP32[$nByte>>2] = 32;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3VdbeMemClearAndResize($8,32)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $54 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $11 = HEAP32[$fg>>2]|0;
 $12 = $11 & 4;
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 if ($13) {
  $18 = $17;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($18) + 4)|0;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = $vararg_buffer;
  $25 = $24;
  HEAP32[$25>>2] = $20;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  (_sqlite3_snprintf(32,$16,19285,$vararg_buffer)|0);
 } else {
  $28 = +HEAPF64[$17>>3];
  HEAPF64[$vararg_buffer1>>3] = $28;
  (_sqlite3_snprintf(32,$16,19290,$vararg_buffer1)|0);
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 16|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3Strlen30($31)|0);
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 12|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 10|0);
 HEAP8[$36>>0] = 1;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 8|0);
 $39 = HEAP16[$38>>1]|0;
 $40 = $39&65535;
 $41 = $40 | 514;
 $42 = $41&65535;
 HEAP16[$38>>1] = $42;
 $43 = HEAP8[$3>>0]|0;
 $44 = ($43<<24>>24)!=(0);
 if ($44) {
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 8|0);
  $47 = HEAP16[$46>>1]|0;
  $48 = $47&65535;
  $49 = $48 & -13;
  $50 = $49&65535;
  HEAP16[$46>>1] = $50;
 }
 $51 = HEAP32[$1>>2]|0;
 $52 = HEAP8[$2>>0]|0;
 $53 = $52&255;
 (_sqlite3VdbeChangeEncoding($51,$53)|0);
 HEAP32[$0>>2] = 0;
 $54 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _sqlite3VdbeMemTranslate($pMem,$desiredEnc) {
 $pMem = $pMem|0;
 $desiredEnc = $desiredEnc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c2 = 0, $c21 = 0, $len = 0, $or$cond = 0;
 var $or$cond3 = 0, $rc = 0, $temp = 0, $z = 0, $zIn = 0, $zOut = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 45|0;
 $len = sp + 32|0;
 $zOut = sp + 28|0;
 $zIn = sp + 24|0;
 $zTerm = sp + 20|0;
 $z = sp + 16|0;
 $c = sp + 12|0;
 $temp = sp + 44|0;
 $rc = sp + 8|0;
 $c2 = sp + 4|0;
 $c21 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP8[$2>>0] = $desiredEnc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 10|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP8[$2>>0]|0;
  $9 = $8&255;
  $10 = ($9|0)!=(1);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (_sqlite3VdbeMemMakeWriteable($11)|0);
   HEAP32[$rc>>2] = $12;
   $13 = HEAP32[$rc>>2]|0;
   $14 = ($13|0)!=(0);
   if ($14) {
    HEAP32[$0>>2] = 7;
    $552 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($552|0);
   }
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 16|0);
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$zIn>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $20 & -2;
   $22 = HEAP32[$zIn>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zTerm>>2] = $23;
   while(1) {
    $24 = HEAP32[$zIn>>2]|0;
    $25 = HEAP32[$zTerm>>2]|0;
    $26 = ($24>>>0)<($25>>>0);
    if (!($26)) {
     break;
    }
    $27 = HEAP32[$zIn>>2]|0;
    $28 = HEAP8[$27>>0]|0;
    HEAP8[$temp>>0] = $28;
    $29 = HEAP32[$zIn>>2]|0;
    $30 = ((($29)) + 1|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = HEAP32[$zIn>>2]|0;
    HEAP8[$32>>0] = $31;
    $33 = HEAP32[$zIn>>2]|0;
    $34 = ((($33)) + 1|0);
    HEAP32[$zIn>>2] = $34;
    $35 = HEAP8[$temp>>0]|0;
    $36 = HEAP32[$zIn>>2]|0;
    $37 = ((($36)) + 1|0);
    HEAP32[$zIn>>2] = $37;
    HEAP8[$36>>0] = $35;
   }
   $38 = HEAP8[$2>>0]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 10|0);
   HEAP8[$40>>0] = $38;
  } else {
   label = 9;
  }
 } else {
  label = 9;
 }
 if ((label|0) == 9) {
  $41 = HEAP8[$2>>0]|0;
  $42 = $41&255;
  $43 = ($42|0)==(1);
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 12|0);
  $46 = HEAP32[$45>>2]|0;
  if ($43) {
   $47 = $46 & -2;
   HEAP32[$45>>2] = $47;
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $50<<1;
   $52 = (($51) + 1)|0;
   HEAP32[$len>>2] = $52;
  } else {
   $53 = $46<<1;
   $54 = (($53) + 2)|0;
   HEAP32[$len>>2] = $54;
  }
  $55 = HEAP32[$1>>2]|0;
  $56 = ((($55)) + 16|0);
  $57 = HEAP32[$56>>2]|0;
  HEAP32[$zIn>>2] = $57;
  $58 = HEAP32[$1>>2]|0;
  $59 = ((($58)) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$zIn>>2]|0;
  $62 = (($61) + ($60)|0);
  HEAP32[$zTerm>>2] = $62;
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 32|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$len>>2]|0;
  $67 = ($66|0)<(0);
  $68 = $67 << 31 >> 31;
  $69 = (_sqlite3DbMallocRaw($65,$66,$68)|0);
  HEAP32[$zOut>>2] = $69;
  $70 = HEAP32[$zOut>>2]|0;
  $71 = ($70|0)!=(0|0);
  if (!($71)) {
   HEAP32[$0>>2] = 7;
   $552 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($552|0);
  }
  $72 = HEAP32[$zOut>>2]|0;
  HEAP32[$z>>2] = $72;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 10|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  $77 = ($76|0)==(1);
  if ($77) {
   $78 = HEAP8[$2>>0]|0;
   $79 = $78&255;
   $80 = ($79|0)==(2);
   L24: do {
    if ($80) {
     while(1) {
      $81 = HEAP32[$zIn>>2]|0;
      $82 = HEAP32[$zTerm>>2]|0;
      $83 = ($81>>>0)<($82>>>0);
      if (!($83)) {
       break L24;
      }
      $84 = HEAP32[$zIn>>2]|0;
      $85 = ((($84)) + 1|0);
      HEAP32[$zIn>>2] = $85;
      $86 = HEAP8[$84>>0]|0;
      $87 = $86&255;
      HEAP32[$c>>2] = $87;
      $88 = HEAP32[$c>>2]|0;
      $89 = ($88>>>0)>=(192);
      do {
       if ($89) {
        $90 = HEAP32[$c>>2]|0;
        $91 = (($90) - 192)|0;
        $92 = (19221 + ($91)|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = $93&255;
        HEAP32[$c>>2] = $94;
        while(1) {
         $95 = HEAP32[$zIn>>2]|0;
         $96 = HEAP32[$zTerm>>2]|0;
         $97 = ($95|0)!=($96|0);
         if ($97) {
          $98 = HEAP32[$zIn>>2]|0;
          $99 = HEAP8[$98>>0]|0;
          $100 = $99&255;
          $101 = $100 & 192;
          $102 = ($101|0)==(128);
          $553 = $102;
         } else {
          $553 = 0;
         }
         $103 = HEAP32[$c>>2]|0;
         if (!($553)) {
          break;
         }
         $104 = $103 << 6;
         $105 = HEAP32[$zIn>>2]|0;
         $106 = ((($105)) + 1|0);
         HEAP32[$zIn>>2] = $106;
         $107 = HEAP8[$105>>0]|0;
         $108 = $107&255;
         $109 = 63 & $108;
         $110 = (($104) + ($109))|0;
         HEAP32[$c>>2] = $110;
        }
        $111 = ($103>>>0)<(128);
        if (!($111)) {
         $112 = HEAP32[$c>>2]|0;
         $113 = $112 & -2048;
         $114 = ($113|0)==(55296);
         if (!($114)) {
          $115 = HEAP32[$c>>2]|0;
          $116 = $115 & -2;
          $117 = ($116|0)==(65534);
          if (!($117)) {
           break;
          }
         }
        }
        HEAP32[$c>>2] = 65533;
       }
      } while(0);
      $118 = HEAP32[$c>>2]|0;
      $119 = ($118>>>0)<=(65535);
      $120 = HEAP32[$c>>2]|0;
      if ($119) {
       $121 = $120 & 255;
       $122 = $121&255;
       $123 = HEAP32[$z>>2]|0;
       $124 = ((($123)) + 1|0);
       HEAP32[$z>>2] = $124;
       HEAP8[$123>>0] = $122;
       $125 = HEAP32[$c>>2]|0;
       $126 = $125 >>> 8;
       $127 = $126 & 255;
       $128 = $127&255;
       $129 = HEAP32[$z>>2]|0;
       $130 = ((($129)) + 1|0);
       HEAP32[$z>>2] = $130;
       HEAP8[$129>>0] = $128;
       continue;
      } else {
       $131 = $120 >>> 10;
       $132 = $131 & 63;
       $133 = HEAP32[$c>>2]|0;
       $134 = (($133) - 65536)|0;
       $135 = $134 >>> 10;
       $136 = $135 & 192;
       $137 = (($132) + ($136))|0;
       $138 = $137&255;
       $139 = HEAP32[$z>>2]|0;
       $140 = ((($139)) + 1|0);
       HEAP32[$z>>2] = $140;
       HEAP8[$139>>0] = $138;
       $141 = HEAP32[$c>>2]|0;
       $142 = (($141) - 65536)|0;
       $143 = $142 >>> 18;
       $144 = $143 & 3;
       $145 = (216 + ($144))|0;
       $146 = $145&255;
       $147 = HEAP32[$z>>2]|0;
       $148 = ((($147)) + 1|0);
       HEAP32[$z>>2] = $148;
       HEAP8[$147>>0] = $146;
       $149 = HEAP32[$c>>2]|0;
       $150 = $149 & 255;
       $151 = $150&255;
       $152 = HEAP32[$z>>2]|0;
       $153 = ((($152)) + 1|0);
       HEAP32[$z>>2] = $153;
       HEAP8[$152>>0] = $151;
       $154 = HEAP32[$c>>2]|0;
       $155 = $154 >>> 8;
       $156 = $155 & 3;
       $157 = (220 + ($156))|0;
       $158 = $157&255;
       $159 = HEAP32[$z>>2]|0;
       $160 = ((($159)) + 1|0);
       HEAP32[$z>>2] = $160;
       HEAP8[$159>>0] = $158;
       continue;
      }
     }
    } else {
     while(1) {
      $161 = HEAP32[$zIn>>2]|0;
      $162 = HEAP32[$zTerm>>2]|0;
      $163 = ($161>>>0)<($162>>>0);
      if (!($163)) {
       break L24;
      }
      $164 = HEAP32[$zIn>>2]|0;
      $165 = ((($164)) + 1|0);
      HEAP32[$zIn>>2] = $165;
      $166 = HEAP8[$164>>0]|0;
      $167 = $166&255;
      HEAP32[$c>>2] = $167;
      $168 = HEAP32[$c>>2]|0;
      $169 = ($168>>>0)>=(192);
      do {
       if ($169) {
        $170 = HEAP32[$c>>2]|0;
        $171 = (($170) - 192)|0;
        $172 = (19221 + ($171)|0);
        $173 = HEAP8[$172>>0]|0;
        $174 = $173&255;
        HEAP32[$c>>2] = $174;
        while(1) {
         $175 = HEAP32[$zIn>>2]|0;
         $176 = HEAP32[$zTerm>>2]|0;
         $177 = ($175|0)!=($176|0);
         if ($177) {
          $178 = HEAP32[$zIn>>2]|0;
          $179 = HEAP8[$178>>0]|0;
          $180 = $179&255;
          $181 = $180 & 192;
          $182 = ($181|0)==(128);
          $554 = $182;
         } else {
          $554 = 0;
         }
         $183 = HEAP32[$c>>2]|0;
         if (!($554)) {
          break;
         }
         $184 = $183 << 6;
         $185 = HEAP32[$zIn>>2]|0;
         $186 = ((($185)) + 1|0);
         HEAP32[$zIn>>2] = $186;
         $187 = HEAP8[$185>>0]|0;
         $188 = $187&255;
         $189 = 63 & $188;
         $190 = (($184) + ($189))|0;
         HEAP32[$c>>2] = $190;
        }
        $191 = ($183>>>0)<(128);
        if (!($191)) {
         $192 = HEAP32[$c>>2]|0;
         $193 = $192 & -2048;
         $194 = ($193|0)==(55296);
         if (!($194)) {
          $195 = HEAP32[$c>>2]|0;
          $196 = $195 & -2;
          $197 = ($196|0)==(65534);
          if (!($197)) {
           break;
          }
         }
        }
        HEAP32[$c>>2] = 65533;
       }
      } while(0);
      $198 = HEAP32[$c>>2]|0;
      $199 = ($198>>>0)<=(65535);
      $200 = HEAP32[$c>>2]|0;
      if ($199) {
       $201 = $200 >>> 8;
       $202 = $201 & 255;
       $203 = $202&255;
       $204 = HEAP32[$z>>2]|0;
       $205 = ((($204)) + 1|0);
       HEAP32[$z>>2] = $205;
       HEAP8[$204>>0] = $203;
       $206 = HEAP32[$c>>2]|0;
       $207 = $206 & 255;
       $208 = $207&255;
       $209 = HEAP32[$z>>2]|0;
       $210 = ((($209)) + 1|0);
       HEAP32[$z>>2] = $210;
       HEAP8[$209>>0] = $208;
       continue;
      } else {
       $211 = (($200) - 65536)|0;
       $212 = $211 >>> 18;
       $213 = $212 & 3;
       $214 = (216 + ($213))|0;
       $215 = $214&255;
       $216 = HEAP32[$z>>2]|0;
       $217 = ((($216)) + 1|0);
       HEAP32[$z>>2] = $217;
       HEAP8[$216>>0] = $215;
       $218 = HEAP32[$c>>2]|0;
       $219 = $218 >>> 10;
       $220 = $219 & 63;
       $221 = HEAP32[$c>>2]|0;
       $222 = (($221) - 65536)|0;
       $223 = $222 >>> 10;
       $224 = $223 & 192;
       $225 = (($220) + ($224))|0;
       $226 = $225&255;
       $227 = HEAP32[$z>>2]|0;
       $228 = ((($227)) + 1|0);
       HEAP32[$z>>2] = $228;
       HEAP8[$227>>0] = $226;
       $229 = HEAP32[$c>>2]|0;
       $230 = $229 >>> 8;
       $231 = $230 & 3;
       $232 = (220 + ($231))|0;
       $233 = $232&255;
       $234 = HEAP32[$z>>2]|0;
       $235 = ((($234)) + 1|0);
       HEAP32[$z>>2] = $235;
       HEAP8[$234>>0] = $233;
       $236 = HEAP32[$c>>2]|0;
       $237 = $236 & 255;
       $238 = $237&255;
       $239 = HEAP32[$z>>2]|0;
       $240 = ((($239)) + 1|0);
       HEAP32[$z>>2] = $240;
       HEAP8[$239>>0] = $238;
       continue;
      }
     }
    }
   } while(0);
   $241 = HEAP32[$z>>2]|0;
   $242 = HEAP32[$zOut>>2]|0;
   $243 = $241;
   $244 = $242;
   $245 = (($243) - ($244))|0;
   $246 = HEAP32[$1>>2]|0;
   $247 = ((($246)) + 12|0);
   HEAP32[$247>>2] = $245;
   $248 = HEAP32[$z>>2]|0;
   $249 = ((($248)) + 1|0);
   HEAP32[$z>>2] = $249;
   HEAP8[$248>>0] = 0;
  } else {
   $250 = HEAP32[$1>>2]|0;
   $251 = ((($250)) + 10|0);
   $252 = HEAP8[$251>>0]|0;
   $253 = $252&255;
   $254 = ($253|0)==(2);
   L67: do {
    if ($254) {
     while(1) {
      $255 = HEAP32[$zIn>>2]|0;
      $256 = HEAP32[$zTerm>>2]|0;
      $257 = ($255>>>0)<($256>>>0);
      if (!($257)) {
       break L67;
      }
      $258 = HEAP32[$zIn>>2]|0;
      $259 = ((($258)) + 1|0);
      HEAP32[$zIn>>2] = $259;
      $260 = HEAP8[$258>>0]|0;
      $261 = $260&255;
      HEAP32[$c>>2] = $261;
      $262 = HEAP32[$zIn>>2]|0;
      $263 = ((($262)) + 1|0);
      HEAP32[$zIn>>2] = $263;
      $264 = HEAP8[$262>>0]|0;
      $265 = $264&255;
      $266 = $265 << 8;
      $267 = HEAP32[$c>>2]|0;
      $268 = (($267) + ($266))|0;
      HEAP32[$c>>2] = $268;
      $269 = HEAP32[$c>>2]|0;
      $270 = ($269>>>0)>=(55296);
      $271 = HEAP32[$c>>2]|0;
      $272 = ($271>>>0)<(57344);
      $or$cond = $270 & $272;
      if ($or$cond) {
       $273 = HEAP32[$zIn>>2]|0;
       $274 = HEAP32[$zTerm>>2]|0;
       $275 = ($273>>>0)<($274>>>0);
       if ($275) {
        $276 = HEAP32[$zIn>>2]|0;
        $277 = ((($276)) + 1|0);
        HEAP32[$zIn>>2] = $277;
        $278 = HEAP8[$276>>0]|0;
        $279 = $278&255;
        HEAP32[$c2>>2] = $279;
        $280 = HEAP32[$zIn>>2]|0;
        $281 = ((($280)) + 1|0);
        HEAP32[$zIn>>2] = $281;
        $282 = HEAP8[$280>>0]|0;
        $283 = $282&255;
        $284 = $283 << 8;
        $285 = HEAP32[$c2>>2]|0;
        $286 = (($285) + ($284))|0;
        HEAP32[$c2>>2] = $286;
        $287 = HEAP32[$c2>>2]|0;
        $288 = $287 & 1023;
        $289 = HEAP32[$c>>2]|0;
        $290 = $289 & 63;
        $291 = $290 << 10;
        $292 = (($288) + ($291))|0;
        $293 = HEAP32[$c>>2]|0;
        $294 = $293 & 960;
        $295 = (($294) + 64)|0;
        $296 = $295 << 10;
        $297 = (($292) + ($296))|0;
        HEAP32[$c>>2] = $297;
       }
      }
      $298 = HEAP32[$c>>2]|0;
      $299 = ($298>>>0)<(128);
      $300 = HEAP32[$c>>2]|0;
      if ($299) {
       $301 = $300 & 255;
       $302 = $301&255;
       $303 = HEAP32[$z>>2]|0;
       $304 = ((($303)) + 1|0);
       HEAP32[$z>>2] = $304;
       HEAP8[$303>>0] = $302;
       continue;
      }
      $305 = ($300>>>0)<(2048);
      $306 = HEAP32[$c>>2]|0;
      if ($305) {
       $307 = $306 >>> 6;
       $308 = $307 & 31;
       $309 = $308&255;
       $310 = $309&255;
       $311 = (192 + ($310))|0;
       $312 = $311&255;
       $313 = HEAP32[$z>>2]|0;
       $314 = ((($313)) + 1|0);
       HEAP32[$z>>2] = $314;
       HEAP8[$313>>0] = $312;
       $315 = HEAP32[$c>>2]|0;
       $316 = $315 & 63;
       $317 = $316&255;
       $318 = $317&255;
       $319 = (128 + ($318))|0;
       $320 = $319&255;
       $321 = HEAP32[$z>>2]|0;
       $322 = ((($321)) + 1|0);
       HEAP32[$z>>2] = $322;
       HEAP8[$321>>0] = $320;
       continue;
      }
      $323 = ($306>>>0)<(65536);
      $324 = HEAP32[$c>>2]|0;
      if ($323) {
       $325 = $324 >>> 12;
       $326 = $325 & 15;
       $327 = $326&255;
       $328 = $327&255;
       $329 = (224 + ($328))|0;
       $330 = $329&255;
       $331 = HEAP32[$z>>2]|0;
       $332 = ((($331)) + 1|0);
       HEAP32[$z>>2] = $332;
       HEAP8[$331>>0] = $330;
       $333 = HEAP32[$c>>2]|0;
       $334 = $333 >>> 6;
       $335 = $334 & 63;
       $336 = $335&255;
       $337 = $336&255;
       $338 = (128 + ($337))|0;
       $339 = $338&255;
       $340 = HEAP32[$z>>2]|0;
       $341 = ((($340)) + 1|0);
       HEAP32[$z>>2] = $341;
       HEAP8[$340>>0] = $339;
       $342 = HEAP32[$c>>2]|0;
       $343 = $342 & 63;
       $344 = $343&255;
       $345 = $344&255;
       $346 = (128 + ($345))|0;
       $347 = $346&255;
       $348 = HEAP32[$z>>2]|0;
       $349 = ((($348)) + 1|0);
       HEAP32[$z>>2] = $349;
       HEAP8[$348>>0] = $347;
       continue;
      } else {
       $350 = $324 >>> 18;
       $351 = $350 & 7;
       $352 = $351&255;
       $353 = $352&255;
       $354 = (240 + ($353))|0;
       $355 = $354&255;
       $356 = HEAP32[$z>>2]|0;
       $357 = ((($356)) + 1|0);
       HEAP32[$z>>2] = $357;
       HEAP8[$356>>0] = $355;
       $358 = HEAP32[$c>>2]|0;
       $359 = $358 >>> 12;
       $360 = $359 & 63;
       $361 = $360&255;
       $362 = $361&255;
       $363 = (128 + ($362))|0;
       $364 = $363&255;
       $365 = HEAP32[$z>>2]|0;
       $366 = ((($365)) + 1|0);
       HEAP32[$z>>2] = $366;
       HEAP8[$365>>0] = $364;
       $367 = HEAP32[$c>>2]|0;
       $368 = $367 >>> 6;
       $369 = $368 & 63;
       $370 = $369&255;
       $371 = $370&255;
       $372 = (128 + ($371))|0;
       $373 = $372&255;
       $374 = HEAP32[$z>>2]|0;
       $375 = ((($374)) + 1|0);
       HEAP32[$z>>2] = $375;
       HEAP8[$374>>0] = $373;
       $376 = HEAP32[$c>>2]|0;
       $377 = $376 & 63;
       $378 = $377&255;
       $379 = $378&255;
       $380 = (128 + ($379))|0;
       $381 = $380&255;
       $382 = HEAP32[$z>>2]|0;
       $383 = ((($382)) + 1|0);
       HEAP32[$z>>2] = $383;
       HEAP8[$382>>0] = $381;
       continue;
      }
     }
    } else {
     while(1) {
      $384 = HEAP32[$zIn>>2]|0;
      $385 = HEAP32[$zTerm>>2]|0;
      $386 = ($384>>>0)<($385>>>0);
      if (!($386)) {
       break L67;
      }
      $387 = HEAP32[$zIn>>2]|0;
      $388 = ((($387)) + 1|0);
      HEAP32[$zIn>>2] = $388;
      $389 = HEAP8[$387>>0]|0;
      $390 = $389&255;
      $391 = $390 << 8;
      HEAP32[$c>>2] = $391;
      $392 = HEAP32[$zIn>>2]|0;
      $393 = ((($392)) + 1|0);
      HEAP32[$zIn>>2] = $393;
      $394 = HEAP8[$392>>0]|0;
      $395 = $394&255;
      $396 = HEAP32[$c>>2]|0;
      $397 = (($396) + ($395))|0;
      HEAP32[$c>>2] = $397;
      $398 = HEAP32[$c>>2]|0;
      $399 = ($398>>>0)>=(55296);
      $400 = HEAP32[$c>>2]|0;
      $401 = ($400>>>0)<(57344);
      $or$cond3 = $399 & $401;
      if ($or$cond3) {
       $402 = HEAP32[$zIn>>2]|0;
       $403 = HEAP32[$zTerm>>2]|0;
       $404 = ($402>>>0)<($403>>>0);
       if ($404) {
        $405 = HEAP32[$zIn>>2]|0;
        $406 = ((($405)) + 1|0);
        HEAP32[$zIn>>2] = $406;
        $407 = HEAP8[$405>>0]|0;
        $408 = $407&255;
        $409 = $408 << 8;
        HEAP32[$c21>>2] = $409;
        $410 = HEAP32[$zIn>>2]|0;
        $411 = ((($410)) + 1|0);
        HEAP32[$zIn>>2] = $411;
        $412 = HEAP8[$410>>0]|0;
        $413 = $412&255;
        $414 = HEAP32[$c21>>2]|0;
        $415 = (($414) + ($413))|0;
        HEAP32[$c21>>2] = $415;
        $416 = HEAP32[$c21>>2]|0;
        $417 = $416 & 1023;
        $418 = HEAP32[$c>>2]|0;
        $419 = $418 & 63;
        $420 = $419 << 10;
        $421 = (($417) + ($420))|0;
        $422 = HEAP32[$c>>2]|0;
        $423 = $422 & 960;
        $424 = (($423) + 64)|0;
        $425 = $424 << 10;
        $426 = (($421) + ($425))|0;
        HEAP32[$c>>2] = $426;
       }
      }
      $427 = HEAP32[$c>>2]|0;
      $428 = ($427>>>0)<(128);
      $429 = HEAP32[$c>>2]|0;
      if ($428) {
       $430 = $429 & 255;
       $431 = $430&255;
       $432 = HEAP32[$z>>2]|0;
       $433 = ((($432)) + 1|0);
       HEAP32[$z>>2] = $433;
       HEAP8[$432>>0] = $431;
       continue;
      }
      $434 = ($429>>>0)<(2048);
      $435 = HEAP32[$c>>2]|0;
      if ($434) {
       $436 = $435 >>> 6;
       $437 = $436 & 31;
       $438 = $437&255;
       $439 = $438&255;
       $440 = (192 + ($439))|0;
       $441 = $440&255;
       $442 = HEAP32[$z>>2]|0;
       $443 = ((($442)) + 1|0);
       HEAP32[$z>>2] = $443;
       HEAP8[$442>>0] = $441;
       $444 = HEAP32[$c>>2]|0;
       $445 = $444 & 63;
       $446 = $445&255;
       $447 = $446&255;
       $448 = (128 + ($447))|0;
       $449 = $448&255;
       $450 = HEAP32[$z>>2]|0;
       $451 = ((($450)) + 1|0);
       HEAP32[$z>>2] = $451;
       HEAP8[$450>>0] = $449;
       continue;
      }
      $452 = ($435>>>0)<(65536);
      $453 = HEAP32[$c>>2]|0;
      if ($452) {
       $454 = $453 >>> 12;
       $455 = $454 & 15;
       $456 = $455&255;
       $457 = $456&255;
       $458 = (224 + ($457))|0;
       $459 = $458&255;
       $460 = HEAP32[$z>>2]|0;
       $461 = ((($460)) + 1|0);
       HEAP32[$z>>2] = $461;
       HEAP8[$460>>0] = $459;
       $462 = HEAP32[$c>>2]|0;
       $463 = $462 >>> 6;
       $464 = $463 & 63;
       $465 = $464&255;
       $466 = $465&255;
       $467 = (128 + ($466))|0;
       $468 = $467&255;
       $469 = HEAP32[$z>>2]|0;
       $470 = ((($469)) + 1|0);
       HEAP32[$z>>2] = $470;
       HEAP8[$469>>0] = $468;
       $471 = HEAP32[$c>>2]|0;
       $472 = $471 & 63;
       $473 = $472&255;
       $474 = $473&255;
       $475 = (128 + ($474))|0;
       $476 = $475&255;
       $477 = HEAP32[$z>>2]|0;
       $478 = ((($477)) + 1|0);
       HEAP32[$z>>2] = $478;
       HEAP8[$477>>0] = $476;
       continue;
      } else {
       $479 = $453 >>> 18;
       $480 = $479 & 7;
       $481 = $480&255;
       $482 = $481&255;
       $483 = (240 + ($482))|0;
       $484 = $483&255;
       $485 = HEAP32[$z>>2]|0;
       $486 = ((($485)) + 1|0);
       HEAP32[$z>>2] = $486;
       HEAP8[$485>>0] = $484;
       $487 = HEAP32[$c>>2]|0;
       $488 = $487 >>> 12;
       $489 = $488 & 63;
       $490 = $489&255;
       $491 = $490&255;
       $492 = (128 + ($491))|0;
       $493 = $492&255;
       $494 = HEAP32[$z>>2]|0;
       $495 = ((($494)) + 1|0);
       HEAP32[$z>>2] = $495;
       HEAP8[$494>>0] = $493;
       $496 = HEAP32[$c>>2]|0;
       $497 = $496 >>> 6;
       $498 = $497 & 63;
       $499 = $498&255;
       $500 = $499&255;
       $501 = (128 + ($500))|0;
       $502 = $501&255;
       $503 = HEAP32[$z>>2]|0;
       $504 = ((($503)) + 1|0);
       HEAP32[$z>>2] = $504;
       HEAP8[$503>>0] = $502;
       $505 = HEAP32[$c>>2]|0;
       $506 = $505 & 63;
       $507 = $506&255;
       $508 = $507&255;
       $509 = (128 + ($508))|0;
       $510 = $509&255;
       $511 = HEAP32[$z>>2]|0;
       $512 = ((($511)) + 1|0);
       HEAP32[$z>>2] = $512;
       HEAP8[$511>>0] = $510;
       continue;
      }
     }
    }
   } while(0);
   $513 = HEAP32[$z>>2]|0;
   $514 = HEAP32[$zOut>>2]|0;
   $515 = $513;
   $516 = $514;
   $517 = (($515) - ($516))|0;
   $518 = HEAP32[$1>>2]|0;
   $519 = ((($518)) + 12|0);
   HEAP32[$519>>2] = $517;
  }
  $520 = HEAP32[$z>>2]|0;
  HEAP8[$520>>0] = 0;
  $521 = HEAP32[$1>>2]|0;
  $522 = ((($521)) + 8|0);
  $523 = HEAP16[$522>>1]|0;
  $524 = $523&65535;
  HEAP32[$c>>2] = $524;
  $525 = HEAP32[$1>>2]|0;
  _sqlite3VdbeMemRelease($525);
  $526 = HEAP32[$c>>2]|0;
  $527 = $526 & 32799;
  $528 = 514 | $527;
  $529 = $528&65535;
  $530 = HEAP32[$1>>2]|0;
  $531 = ((($530)) + 8|0);
  HEAP16[$531>>1] = $529;
  $532 = HEAP8[$2>>0]|0;
  $533 = HEAP32[$1>>2]|0;
  $534 = ((($533)) + 10|0);
  HEAP8[$534>>0] = $532;
  $535 = HEAP32[$zOut>>2]|0;
  $536 = HEAP32[$1>>2]|0;
  $537 = ((($536)) + 16|0);
  HEAP32[$537>>2] = $535;
  $538 = HEAP32[$1>>2]|0;
  $539 = ((($538)) + 16|0);
  $540 = HEAP32[$539>>2]|0;
  $541 = HEAP32[$1>>2]|0;
  $542 = ((($541)) + 20|0);
  HEAP32[$542>>2] = $540;
  $543 = HEAP32[$1>>2]|0;
  $544 = ((($543)) + 32|0);
  $545 = HEAP32[$544>>2]|0;
  $546 = HEAP32[$1>>2]|0;
  $547 = ((($546)) + 16|0);
  $548 = HEAP32[$547>>2]|0;
  $549 = (_sqlite3DbMallocSize($545,$548)|0);
  $550 = HEAP32[$1>>2]|0;
  $551 = ((($550)) + 24|0);
  HEAP32[$551>>2] = $549;
 }
 HEAP32[$0>>2] = 0;
 $552 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($552|0);
}
function _vdbeMemAddTerminator($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 2)|0;
 $7 = (_sqlite3VdbeMemGrow($2,$6,1)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 7;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11)|0);
  HEAP8[$15>>0] = 0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 1)|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19)|0);
  HEAP8[$23>>0] = 0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP16[$25>>1]|0;
  $27 = $26&65535;
  $28 = $27 | 512;
  $29 = $28&65535;
  HEAP16[$25>>1] = $29;
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemClearAndResize($pMem,$szNew) {
 $pMem = $pMem|0;
 $szNew = $szNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $szNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($5|0)<($6|0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3VdbeMemGrow($8,$9,0)|0);
  HEAP32[$0>>2] = $10;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $11 = ((($8)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 16|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & 13;
  $20 = $19&65535;
  HEAP16[$16>>1] = $20;
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3AtoF($z,$pResult,$length,$enc) {
 $z = $z|0;
 $pResult = $pResult|0;
 $length = $length|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0.0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0.0, $41 = 0, $410 = 0.0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0.0;
 var $422 = 0.0, $423 = 0.0, $424 = 0.0, $425 = 0.0, $426 = 0.0, $427 = 0.0, $428 = 0.0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0.0, $44 = 0;
 var $440 = 0.0, $441 = 0.0, $442 = 0, $443 = 0, $444 = 0, $445 = 0.0, $446 = 0.0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0.0, $452 = 0.0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0.0, $464 = 0.0, $465 = 0.0, $466 = 0.0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0.0, $474 = 0.0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d = 0, $e = 0, $eValid = 0, $esign = 0, $i = 0, $incr = 0, $nDigits = 0, $nonNum = 0, $or$cond = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0;
 var $or$cond19 = 0, $or$cond21 = 0, $result = 0, $s = 0, $scale = 0, $sign = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 80|0;
 $incr = sp + 60|0;
 $zEnd = sp + 56|0;
 $sign = sp + 52|0;
 $s = sp + 16|0;
 $d = sp + 48|0;
 $esign = sp + 44|0;
 $e = sp + 40|0;
 $eValid = sp + 36|0;
 $result = sp + 8|0;
 $nDigits = sp + 32|0;
 $nonNum = sp + 28|0;
 $i = sp + 24|0;
 $scale = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pResult;
 HEAP32[$3>>2] = $length;
 HEAP8[$4>>0] = $enc;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (($5) + ($6)|0);
 HEAP32[$zEnd>>2] = $7;
 HEAP32[$sign>>2] = 1;
 $8 = $s;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 HEAP32[$d>>2] = 0;
 HEAP32[$esign>>2] = 1;
 HEAP32[$e>>2] = 0;
 HEAP32[$eValid>>2] = 1;
 HEAP32[$nDigits>>2] = 0;
 HEAP32[$nonNum>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 HEAPF64[$12>>3] = 0.0;
 $13 = HEAP8[$4>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(1);
 if ($15) {
  HEAP32[$incr>>2] = 1;
 } else {
  HEAP32[$incr>>2] = 2;
  $16 = HEAP8[$4>>0]|0;
  $17 = $16&255;
  $18 = (3 - ($17))|0;
  HEAP32[$i>>2] = $18;
  while(1) {
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = ($19|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$i>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + ($22)|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = ($26|0)==(0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = (($28) + 2)|0;
   HEAP32[$i>>2] = $29;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ($30|0)<($31|0);
  $33 = $32&1;
  HEAP32[$nonNum>>2] = $33;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($34) + ($35)|0);
  $37 = HEAP8[$4>>0]|0;
  $38 = $37&255;
  $39 = (($36) + ($38)|0);
  $40 = ((($39)) + -3|0);
  HEAP32[$zEnd>>2] = $40;
  $41 = HEAP8[$4>>0]|0;
  $42 = $41&255;
  $43 = $42 & 1;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + ($43)|0);
  HEAP32[$1>>2] = $45;
 }
 while(1) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$zEnd>>2]|0;
  $48 = ($46>>>0)<($47>>>0);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = (17533 + ($51)|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $54 & 1;
  $56 = ($55|0)!=(0);
  if (!($56)) {
   break;
  }
  $57 = HEAP32[$incr>>2]|0;
  $58 = HEAP32[$1>>2]|0;
  $59 = (($58) + ($57)|0);
  HEAP32[$1>>2] = $59;
 }
 $60 = HEAP32[$1>>2]|0;
 $61 = HEAP32[$zEnd>>2]|0;
 $62 = ($60>>>0)>=($61>>>0);
 if ($62) {
  HEAP32[$0>>2] = 0;
  $487 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($487|0);
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = HEAP8[$63>>0]|0;
 $65 = $64 << 24 >> 24;
 $66 = ($65|0)==(45);
 if ($66) {
  HEAP32[$sign>>2] = -1;
  $67 = HEAP32[$incr>>2]|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = (($68) + ($67)|0);
  HEAP32[$1>>2] = $69;
 } else {
  $70 = HEAP32[$1>>2]|0;
  $71 = HEAP8[$70>>0]|0;
  $72 = $71 << 24 >> 24;
  $73 = ($72|0)==(43);
  if ($73) {
   $74 = HEAP32[$incr>>2]|0;
   $75 = HEAP32[$1>>2]|0;
   $76 = (($75) + ($74)|0);
   HEAP32[$1>>2] = $76;
  }
 }
 while(1) {
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$zEnd>>2]|0;
  $79 = ($77>>>0)<($78>>>0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$1>>2]|0;
  $81 = HEAP8[$80>>0]|0;
  $82 = $81 << 24 >> 24;
  $83 = ($82|0)==(48);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$incr>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + ($84)|0);
  HEAP32[$1>>2] = $86;
  $87 = HEAP32[$nDigits>>2]|0;
  $88 = (($87) + 1)|0;
  HEAP32[$nDigits>>2] = $88;
 }
 while(1) {
  $89 = HEAP32[$1>>2]|0;
  $90 = HEAP32[$zEnd>>2]|0;
  $91 = ($89>>>0)<($90>>>0);
  if (!($91)) {
   break;
  }
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP8[$92>>0]|0;
  $94 = $93&255;
  $95 = (17533 + ($94)|0);
  $96 = HEAP8[$95>>0]|0;
  $97 = $96&255;
  $98 = $97 & 4;
  $99 = ($98|0)!=(0);
  $100 = $s;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($100) + 4)|0;
  $104 = $103;
  $105 = HEAP32[$104>>2]|0;
  $106 = ($105|0)<(214748364);
  $107 = ($102>>>0)<(3435973835);
  $108 = ($105|0)==(214748364);
  $109 = $108 & $107;
  $110 = $106 | $109;
  $or$cond19 = $99 & $110;
  if (!($or$cond19)) {
   break;
  }
  $111 = $s;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  $117 = (___muldi3(($113|0),($116|0),10,0)|0);
  $118 = tempRet0;
  $119 = HEAP32[$1>>2]|0;
  $120 = HEAP8[$119>>0]|0;
  $121 = $120 << 24 >> 24;
  $122 = (($121) - 48)|0;
  $123 = ($122|0)<(0);
  $124 = $123 << 31 >> 31;
  $125 = (_i64Add(($117|0),($118|0),($122|0),($124|0))|0);
  $126 = tempRet0;
  $127 = $s;
  $128 = $127;
  HEAP32[$128>>2] = $125;
  $129 = (($127) + 4)|0;
  $130 = $129;
  HEAP32[$130>>2] = $126;
  $131 = HEAP32[$incr>>2]|0;
  $132 = HEAP32[$1>>2]|0;
  $133 = (($132) + ($131)|0);
  HEAP32[$1>>2] = $133;
  $134 = HEAP32[$nDigits>>2]|0;
  $135 = (($134) + 1)|0;
  HEAP32[$nDigits>>2] = $135;
 }
 while(1) {
  $136 = HEAP32[$1>>2]|0;
  $137 = HEAP32[$zEnd>>2]|0;
  $138 = ($136>>>0)<($137>>>0);
  if (!($138)) {
   break;
  }
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP8[$139>>0]|0;
  $141 = $140&255;
  $142 = (17533 + ($141)|0);
  $143 = HEAP8[$142>>0]|0;
  $144 = $143&255;
  $145 = $144 & 4;
  $146 = ($145|0)!=(0);
  if (!($146)) {
   break;
  }
  $147 = HEAP32[$incr>>2]|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = (($148) + ($147)|0);
  HEAP32[$1>>2] = $149;
  $150 = HEAP32[$nDigits>>2]|0;
  $151 = (($150) + 1)|0;
  HEAP32[$nDigits>>2] = $151;
  $152 = HEAP32[$d>>2]|0;
  $153 = (($152) + 1)|0;
  HEAP32[$d>>2] = $153;
 }
 $154 = HEAP32[$1>>2]|0;
 $155 = HEAP32[$zEnd>>2]|0;
 $156 = ($154>>>0)>=($155>>>0);
 L35: do {
  if (!($156)) {
   $157 = HEAP32[$1>>2]|0;
   $158 = HEAP8[$157>>0]|0;
   $159 = $158 << 24 >> 24;
   $160 = ($159|0)==(46);
   L37: do {
    if ($160) {
     $161 = HEAP32[$incr>>2]|0;
     $162 = HEAP32[$1>>2]|0;
     $163 = (($162) + ($161)|0);
     HEAP32[$1>>2] = $163;
     while(1) {
      $164 = HEAP32[$1>>2]|0;
      $165 = HEAP32[$zEnd>>2]|0;
      $166 = ($164>>>0)<($165>>>0);
      if (!($166)) {
       break;
      }
      $167 = HEAP32[$1>>2]|0;
      $168 = HEAP8[$167>>0]|0;
      $169 = $168&255;
      $170 = (17533 + ($169)|0);
      $171 = HEAP8[$170>>0]|0;
      $172 = $171&255;
      $173 = $172 & 4;
      $174 = ($173|0)!=(0);
      $175 = $s;
      $176 = $175;
      $177 = HEAP32[$176>>2]|0;
      $178 = (($175) + 4)|0;
      $179 = $178;
      $180 = HEAP32[$179>>2]|0;
      $181 = ($180|0)<(214748364);
      $182 = ($177>>>0)<(3435973835);
      $183 = ($180|0)==(214748364);
      $184 = $183 & $182;
      $185 = $181 | $184;
      $or$cond21 = $174 & $185;
      if (!($or$cond21)) {
       break;
      }
      $186 = $s;
      $187 = $186;
      $188 = HEAP32[$187>>2]|0;
      $189 = (($186) + 4)|0;
      $190 = $189;
      $191 = HEAP32[$190>>2]|0;
      $192 = (___muldi3(($188|0),($191|0),10,0)|0);
      $193 = tempRet0;
      $194 = HEAP32[$1>>2]|0;
      $195 = HEAP8[$194>>0]|0;
      $196 = $195 << 24 >> 24;
      $197 = (($196) - 48)|0;
      $198 = ($197|0)<(0);
      $199 = $198 << 31 >> 31;
      $200 = (_i64Add(($192|0),($193|0),($197|0),($199|0))|0);
      $201 = tempRet0;
      $202 = $s;
      $203 = $202;
      HEAP32[$203>>2] = $200;
      $204 = (($202) + 4)|0;
      $205 = $204;
      HEAP32[$205>>2] = $201;
      $206 = HEAP32[$incr>>2]|0;
      $207 = HEAP32[$1>>2]|0;
      $208 = (($207) + ($206)|0);
      HEAP32[$1>>2] = $208;
      $209 = HEAP32[$nDigits>>2]|0;
      $210 = (($209) + 1)|0;
      HEAP32[$nDigits>>2] = $210;
      $211 = HEAP32[$d>>2]|0;
      $212 = (($211) + -1)|0;
      HEAP32[$d>>2] = $212;
     }
     while(1) {
      $213 = HEAP32[$1>>2]|0;
      $214 = HEAP32[$zEnd>>2]|0;
      $215 = ($213>>>0)<($214>>>0);
      if (!($215)) {
       break L37;
      }
      $216 = HEAP32[$1>>2]|0;
      $217 = HEAP8[$216>>0]|0;
      $218 = $217&255;
      $219 = (17533 + ($218)|0);
      $220 = HEAP8[$219>>0]|0;
      $221 = $220&255;
      $222 = $221 & 4;
      $223 = ($222|0)!=(0);
      if (!($223)) {
       break L37;
      }
      $224 = HEAP32[$incr>>2]|0;
      $225 = HEAP32[$1>>2]|0;
      $226 = (($225) + ($224)|0);
      HEAP32[$1>>2] = $226;
      $227 = HEAP32[$nDigits>>2]|0;
      $228 = (($227) + 1)|0;
      HEAP32[$nDigits>>2] = $228;
     }
    }
   } while(0);
   $229 = HEAP32[$1>>2]|0;
   $230 = HEAP32[$zEnd>>2]|0;
   $231 = ($229>>>0)>=($230>>>0);
   if (!($231)) {
    $232 = HEAP32[$1>>2]|0;
    $233 = HEAP8[$232>>0]|0;
    $234 = $233 << 24 >> 24;
    $235 = ($234|0)==(101);
    if ($235) {
     label = 38;
    } else {
     $236 = HEAP32[$1>>2]|0;
     $237 = HEAP8[$236>>0]|0;
     $238 = $237 << 24 >> 24;
     $239 = ($238|0)==(69);
     if ($239) {
      label = 38;
     }
    }
    L51: do {
     if ((label|0) == 38) {
      $240 = HEAP32[$incr>>2]|0;
      $241 = HEAP32[$1>>2]|0;
      $242 = (($241) + ($240)|0);
      HEAP32[$1>>2] = $242;
      HEAP32[$eValid>>2] = 0;
      $243 = HEAP32[$1>>2]|0;
      $244 = HEAP32[$zEnd>>2]|0;
      $245 = ($243>>>0)>=($244>>>0);
      if ($245) {
       break L35;
      }
      $246 = HEAP32[$1>>2]|0;
      $247 = HEAP8[$246>>0]|0;
      $248 = $247 << 24 >> 24;
      $249 = ($248|0)==(45);
      if ($249) {
       HEAP32[$esign>>2] = -1;
       $250 = HEAP32[$incr>>2]|0;
       $251 = HEAP32[$1>>2]|0;
       $252 = (($251) + ($250)|0);
       HEAP32[$1>>2] = $252;
      } else {
       $253 = HEAP32[$1>>2]|0;
       $254 = HEAP8[$253>>0]|0;
       $255 = $254 << 24 >> 24;
       $256 = ($255|0)==(43);
       if ($256) {
        $257 = HEAP32[$incr>>2]|0;
        $258 = HEAP32[$1>>2]|0;
        $259 = (($258) + ($257)|0);
        HEAP32[$1>>2] = $259;
       }
      }
      while(1) {
       $260 = HEAP32[$1>>2]|0;
       $261 = HEAP32[$zEnd>>2]|0;
       $262 = ($260>>>0)<($261>>>0);
       if (!($262)) {
        break L51;
       }
       $263 = HEAP32[$1>>2]|0;
       $264 = HEAP8[$263>>0]|0;
       $265 = $264&255;
       $266 = (17533 + ($265)|0);
       $267 = HEAP8[$266>>0]|0;
       $268 = $267&255;
       $269 = $268 & 4;
       $270 = ($269|0)!=(0);
       if (!($270)) {
        break L51;
       }
       $271 = HEAP32[$e>>2]|0;
       $272 = ($271|0)<(10000);
       if ($272) {
        $273 = HEAP32[$e>>2]|0;
        $274 = ($273*10)|0;
        $275 = HEAP32[$1>>2]|0;
        $276 = HEAP8[$275>>0]|0;
        $277 = $276 << 24 >> 24;
        $278 = (($277) - 48)|0;
        $279 = (($274) + ($278))|0;
        $280 = $279;
       } else {
        $280 = 10000;
       }
       HEAP32[$e>>2] = $280;
       $281 = HEAP32[$incr>>2]|0;
       $282 = HEAP32[$1>>2]|0;
       $283 = (($282) + ($281)|0);
       HEAP32[$1>>2] = $283;
       HEAP32[$eValid>>2] = 1;
      }
     }
    } while(0);
    $284 = HEAP32[$nDigits>>2]|0;
    $285 = ($284|0)!=(0);
    $286 = HEAP32[$eValid>>2]|0;
    $287 = ($286|0)!=(0);
    $or$cond = $285 & $287;
    if ($or$cond) {
     while(1) {
      $288 = HEAP32[$1>>2]|0;
      $289 = HEAP32[$zEnd>>2]|0;
      $290 = ($288>>>0)<($289>>>0);
      if (!($290)) {
       break L35;
      }
      $291 = HEAP32[$1>>2]|0;
      $292 = HEAP8[$291>>0]|0;
      $293 = $292&255;
      $294 = (17533 + ($293)|0);
      $295 = HEAP8[$294>>0]|0;
      $296 = $295&255;
      $297 = $296 & 1;
      $298 = ($297|0)!=(0);
      if (!($298)) {
       break L35;
      }
      $299 = HEAP32[$incr>>2]|0;
      $300 = HEAP32[$1>>2]|0;
      $301 = (($300) + ($299)|0);
      HEAP32[$1>>2] = $301;
     }
    }
   }
  }
 } while(0);
 $302 = HEAP32[$e>>2]|0;
 $303 = HEAP32[$esign>>2]|0;
 $304 = Math_imul($302, $303)|0;
 $305 = HEAP32[$d>>2]|0;
 $306 = (($304) + ($305))|0;
 HEAP32[$e>>2] = $306;
 $307 = HEAP32[$e>>2]|0;
 $308 = ($307|0)<(0);
 if ($308) {
  HEAP32[$esign>>2] = -1;
  $309 = HEAP32[$e>>2]|0;
  $310 = Math_imul($309, -1)|0;
  HEAP32[$e>>2] = $310;
 } else {
  HEAP32[$esign>>2] = 1;
 }
 $311 = $s;
 $312 = $311;
 $313 = HEAP32[$312>>2]|0;
 $314 = (($311) + 4)|0;
 $315 = $314;
 $316 = HEAP32[$315>>2]|0;
 $317 = ($313|0)!=(0);
 $318 = ($316|0)!=(0);
 $319 = $317 | $318;
 do {
  if ($319) {
   $326 = HEAP32[$esign>>2]|0;
   $327 = ($326|0)>(0);
   L77: do {
    if ($327) {
     while(1) {
      $328 = $s;
      $329 = $328;
      $330 = HEAP32[$329>>2]|0;
      $331 = (($328) + 4)|0;
      $332 = $331;
      $333 = HEAP32[$332>>2]|0;
      $334 = ($333|0)<(214748364);
      $335 = ($330>>>0)<(3435973836);
      $336 = ($333|0)==(214748364);
      $337 = $336 & $335;
      $338 = $334 | $337;
      $339 = HEAP32[$e>>2]|0;
      $340 = ($339|0)>(0);
      $341 = $338 ? $340 : 0;
      if (!($341)) {
       break L77;
      }
      $342 = HEAP32[$e>>2]|0;
      $343 = (($342) + -1)|0;
      HEAP32[$e>>2] = $343;
      $344 = $s;
      $345 = $344;
      $346 = HEAP32[$345>>2]|0;
      $347 = (($344) + 4)|0;
      $348 = $347;
      $349 = HEAP32[$348>>2]|0;
      $350 = (___muldi3(($346|0),($349|0),10,0)|0);
      $351 = tempRet0;
      $352 = $s;
      $353 = $352;
      HEAP32[$353>>2] = $350;
      $354 = (($352) + 4)|0;
      $355 = $354;
      HEAP32[$355>>2] = $351;
     }
    } else {
     while(1) {
      $356 = $s;
      $357 = $356;
      $358 = HEAP32[$357>>2]|0;
      $359 = (($356) + 4)|0;
      $360 = $359;
      $361 = HEAP32[$360>>2]|0;
      $362 = (___remdi3(($358|0),($361|0),10,0)|0);
      $363 = tempRet0;
      $364 = ($362|0)!=(0);
      $365 = ($363|0)!=(0);
      $366 = $364 | $365;
      $367 = HEAP32[$e>>2]|0;
      $368 = ($367|0)>(0);
      $369 = $366 ? 0 : $368;
      if (!($369)) {
       break L77;
      }
      $370 = HEAP32[$e>>2]|0;
      $371 = (($370) + -1)|0;
      HEAP32[$e>>2] = $371;
      $372 = $s;
      $373 = $372;
      $374 = HEAP32[$373>>2]|0;
      $375 = (($372) + 4)|0;
      $376 = $375;
      $377 = HEAP32[$376>>2]|0;
      $378 = (___divdi3(($374|0),($377|0),10,0)|0);
      $379 = tempRet0;
      $380 = $s;
      $381 = $380;
      HEAP32[$381>>2] = $378;
      $382 = (($380) + 4)|0;
      $383 = $382;
      HEAP32[$383>>2] = $379;
     }
    }
   } while(0);
   $384 = HEAP32[$sign>>2]|0;
   $385 = ($384|0)<(0);
   $386 = $s;
   $387 = $386;
   $388 = HEAP32[$387>>2]|0;
   $389 = (($386) + 4)|0;
   $390 = $389;
   $391 = HEAP32[$390>>2]|0;
   $392 = (_i64Subtract(0,0,($388|0),($391|0))|0);
   $393 = tempRet0;
   $394 = $385 ? $392 : $388;
   $395 = $385 ? $393 : $391;
   $396 = $s;
   $397 = $396;
   HEAP32[$397>>2] = $394;
   $398 = (($396) + 4)|0;
   $399 = $398;
   HEAP32[$399>>2] = $395;
   $400 = HEAP32[$e>>2]|0;
   $401 = ($400|0)!=(0);
   if (!($401)) {
    $467 = $s;
    $468 = $467;
    $469 = HEAP32[$468>>2]|0;
    $470 = (($467) + 4)|0;
    $471 = $470;
    $472 = HEAP32[$471>>2]|0;
    $473 = (+($469>>>0)) + (4294967296.0*(+($472|0)));
    HEAPF64[$result>>3] = $473;
    break;
   }
   HEAPF64[$scale>>3] = 1.0;
   $402 = HEAP32[$e>>2]|0;
   $403 = ($402|0)>(307);
   $404 = HEAP32[$e>>2]|0;
   $405 = ($404|0)<(342);
   $or$cond13 = $403 & $405;
   if ($or$cond13) {
    while(1) {
     $406 = HEAP32[$e>>2]|0;
     $407 = (($406|0) % 308)&-1;
     $408 = ($407|0)!=(0);
     if (!($408)) {
      break;
     }
     $409 = +HEAPF64[$scale>>3];
     $410 = $409 * 10.0;
     HEAPF64[$scale>>3] = $410;
     $411 = HEAP32[$e>>2]|0;
     $412 = (($411) - 1)|0;
     HEAP32[$e>>2] = $412;
    }
    $413 = HEAP32[$esign>>2]|0;
    $414 = ($413|0)<(0);
    $415 = $s;
    $416 = $415;
    $417 = HEAP32[$416>>2]|0;
    $418 = (($415) + 4)|0;
    $419 = $418;
    $420 = HEAP32[$419>>2]|0;
    $421 = (+($417>>>0)) + (4294967296.0*(+($420|0)));
    $422 = +HEAPF64[$scale>>3];
    if ($414) {
     $423 = $421 / $422;
     HEAPF64[$result>>3] = $423;
     $424 = +HEAPF64[$result>>3];
     $425 = $424 / 1.0E+308;
     HEAPF64[$result>>3] = $425;
     break;
    } else {
     $426 = $421 * $422;
     HEAPF64[$result>>3] = $426;
     $427 = +HEAPF64[$result>>3];
     $428 = $427 * 1.0E+308;
     HEAPF64[$result>>3] = $428;
     break;
    }
   }
   $429 = HEAP32[$e>>2]|0;
   $430 = ($429|0)>=(342);
   if ($430) {
    $431 = HEAP32[$esign>>2]|0;
    $432 = ($431|0)<(0);
    $433 = $s;
    $434 = $433;
    $435 = HEAP32[$434>>2]|0;
    $436 = (($433) + 4)|0;
    $437 = $436;
    $438 = HEAP32[$437>>2]|0;
    $439 = (+($435>>>0)) + (4294967296.0*(+($438|0)));
    if ($432) {
     $440 = 0.0 * $439;
     HEAPF64[$result>>3] = $440;
     break;
    } else {
     $441 = inf * $439;
     HEAPF64[$result>>3] = $441;
     break;
    }
   }
   while(1) {
    $442 = HEAP32[$e>>2]|0;
    $443 = (($442|0) % 22)&-1;
    $444 = ($443|0)!=(0);
    if (!($444)) {
     break;
    }
    $445 = +HEAPF64[$scale>>3];
    $446 = $445 * 10.0;
    HEAPF64[$scale>>3] = $446;
    $447 = HEAP32[$e>>2]|0;
    $448 = (($447) - 1)|0;
    HEAP32[$e>>2] = $448;
   }
   while(1) {
    $449 = HEAP32[$e>>2]|0;
    $450 = ($449|0)>(0);
    if (!($450)) {
     break;
    }
    $451 = +HEAPF64[$scale>>3];
    $452 = $451 * 1.0E+22;
    HEAPF64[$scale>>3] = $452;
    $453 = HEAP32[$e>>2]|0;
    $454 = (($453) - 22)|0;
    HEAP32[$e>>2] = $454;
   }
   $455 = HEAP32[$esign>>2]|0;
   $456 = ($455|0)<(0);
   $457 = $s;
   $458 = $457;
   $459 = HEAP32[$458>>2]|0;
   $460 = (($457) + 4)|0;
   $461 = $460;
   $462 = HEAP32[$461>>2]|0;
   $463 = (+($459>>>0)) + (4294967296.0*(+($462|0)));
   $464 = +HEAPF64[$scale>>3];
   if ($456) {
    $465 = $463 / $464;
    HEAPF64[$result>>3] = $465;
    break;
   } else {
    $466 = $463 * $464;
    HEAPF64[$result>>3] = $466;
    break;
   }
  } else {
   $320 = HEAP32[$sign>>2]|0;
   $321 = ($320|0)<(0);
   $322 = HEAP32[$nDigits>>2]|0;
   $323 = ($322|0)!=(0);
   $324 = $321 ? $323 : 0;
   $325 = $324 ? -0.0 : 0.0;
   HEAPF64[$result>>3] = $325;
  }
 } while(0);
 $474 = +HEAPF64[$result>>3];
 $475 = HEAP32[$2>>2]|0;
 HEAPF64[$475>>3] = $474;
 $476 = HEAP32[$1>>2]|0;
 $477 = HEAP32[$zEnd>>2]|0;
 $478 = ($476>>>0)>=($477>>>0);
 $479 = HEAP32[$nDigits>>2]|0;
 $480 = ($479|0)>(0);
 $or$cond15 = $478 & $480;
 $481 = HEAP32[$eValid>>2]|0;
 $482 = ($481|0)!=(0);
 $or$cond17 = $or$cond15 & $482;
 if ($or$cond17) {
  $483 = HEAP32[$nonNum>>2]|0;
  $484 = ($483|0)==(0);
  $486 = $484;
 } else {
  $486 = 0;
 }
 $485 = $486&1;
 HEAP32[$0>>2] = $485;
 $487 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($487|0);
}
function _doubleToInt64($r) {
 $r = +$r;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0.0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAPF64[$1>>3] = $r;
 $2 = +HEAPF64[$1>>3];
 $3 = $2 <= -9.2233720368547758E+18;
 do {
  if ($3) {
   $4 = $0;
   $5 = $4;
   HEAP32[$5>>2] = 0;
   $6 = (($4) + 4)|0;
   $7 = $6;
   HEAP32[$7>>2] = -2147483648;
  } else {
   $8 = +HEAPF64[$1>>3];
   $9 = $8 >= 9.2233720368547758E+18;
   if ($9) {
    $10 = $0;
    $11 = $10;
    HEAP32[$11>>2] = -1;
    $12 = (($10) + 4)|0;
    $13 = $12;
    HEAP32[$13>>2] = 2147483647;
    break;
   } else {
    $14 = +HEAPF64[$1>>3];
    $15 = (~~$14)>>>0;
    $16 = +Math_abs($14) >= 1.0 ? $14 > 0.0 ? (~~+Math_min(+Math_floor($14 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($14 - +(~~$14 >>> 0)) / 4294967296.0) >>> 0 : 0;
    $17 = $0;
    $18 = $17;
    HEAP32[$18>>2] = $15;
    $19 = (($17) + 4)|0;
    $20 = $19;
    HEAP32[$20>>2] = $16;
    break;
   }
  }
 } while(0);
 $21 = $0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 tempRet0 = ($26);
 STACKTOP = sp;return ($23|0);
}
function _sqlite3Atoi64($zNum,$pNum,$length,$enc) {
 $zNum = $zNum|0;
 $pNum = $pNum|0;
 $length = $length|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $incr = 0, $neg = 0, $nonNum = 0, $or$cond = 0, $u = 0, $zEnd = 0, $zStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 52|0;
 $incr = sp + 32|0;
 $u = sp;
 $neg = sp + 28|0;
 $i = sp + 24|0;
 $c = sp + 20|0;
 $nonNum = sp + 16|0;
 $zStart = sp + 12|0;
 $zEnd = sp + 8|0;
 HEAP32[$1>>2] = $zNum;
 HEAP32[$2>>2] = $pNum;
 HEAP32[$3>>2] = $length;
 HEAP8[$4>>0] = $enc;
 $5 = $u;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAP32[$neg>>2] = 0;
 HEAP32[$c>>2] = 0;
 HEAP32[$nonNum>>2] = 0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (($9) + ($10)|0);
 HEAP32[$zEnd>>2] = $11;
 $12 = HEAP8[$4>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(1);
 if ($14) {
  HEAP32[$incr>>2] = 1;
 } else {
  HEAP32[$incr>>2] = 2;
  $15 = HEAP8[$4>>0]|0;
  $16 = $15&255;
  $17 = (3 - ($16))|0;
  HEAP32[$i>>2] = $17;
  while(1) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$3>>2]|0;
   $20 = ($18|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + ($21)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 << 24 >> 24;
   $26 = ($25|0)==(0);
   if (!($26)) {
    break;
   }
   $27 = HEAP32[$i>>2]|0;
   $28 = (($27) + 2)|0;
   HEAP32[$i>>2] = $28;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = ($29|0)<($30|0);
  $32 = $31&1;
  HEAP32[$nonNum>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$i>>2]|0;
  $35 = (($33) + ($34)|0);
  $36 = HEAP8[$4>>0]|0;
  $37 = $36&255;
  $38 = (($35) + ($37)|0);
  $39 = ((($38)) + -3|0);
  HEAP32[$zEnd>>2] = $39;
  $40 = HEAP8[$4>>0]|0;
  $41 = $40&255;
  $42 = $41 & 1;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + ($42)|0);
  HEAP32[$1>>2] = $44;
 }
 while(1) {
  $45 = HEAP32[$1>>2]|0;
  $46 = HEAP32[$zEnd>>2]|0;
  $47 = ($45>>>0)<($46>>>0);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$1>>2]|0;
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = (17533 + ($50)|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = $53 & 1;
  $55 = ($54|0)!=(0);
  if (!($55)) {
   break;
  }
  $56 = HEAP32[$incr>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + ($56)|0);
  HEAP32[$1>>2] = $58;
 }
 $59 = HEAP32[$1>>2]|0;
 $60 = HEAP32[$zEnd>>2]|0;
 $61 = ($59>>>0)<($60>>>0);
 do {
  if ($61) {
   $62 = HEAP32[$1>>2]|0;
   $63 = HEAP8[$62>>0]|0;
   $64 = $63 << 24 >> 24;
   $65 = ($64|0)==(45);
   if ($65) {
    HEAP32[$neg>>2] = 1;
    $66 = HEAP32[$incr>>2]|0;
    $67 = HEAP32[$1>>2]|0;
    $68 = (($67) + ($66)|0);
    HEAP32[$1>>2] = $68;
    break;
   }
   $69 = HEAP32[$1>>2]|0;
   $70 = HEAP8[$69>>0]|0;
   $71 = $70 << 24 >> 24;
   $72 = ($71|0)==(43);
   if ($72) {
    $73 = HEAP32[$incr>>2]|0;
    $74 = HEAP32[$1>>2]|0;
    $75 = (($74) + ($73)|0);
    HEAP32[$1>>2] = $75;
   }
  }
 } while(0);
 $76 = HEAP32[$1>>2]|0;
 HEAP32[$zStart>>2] = $76;
 while(1) {
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$zEnd>>2]|0;
  $79 = ($77>>>0)<($78>>>0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$1>>2]|0;
  $81 = HEAP8[$80>>0]|0;
  $82 = $81 << 24 >> 24;
  $83 = ($82|0)==(48);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$incr>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + ($84)|0);
  HEAP32[$1>>2] = $86;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $87 = HEAP32[$i>>2]|0;
  $88 = HEAP32[$1>>2]|0;
  $89 = (($88) + ($87)|0);
  $90 = HEAP32[$zEnd>>2]|0;
  $91 = ($89>>>0)<($90>>>0);
  if ($91) {
   $92 = HEAP32[$i>>2]|0;
   $93 = HEAP32[$1>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = $95 << 24 >> 24;
   HEAP32[$c>>2] = $96;
   $97 = ($96|0)>=(48);
   if ($97) {
    $98 = HEAP32[$c>>2]|0;
    $99 = ($98|0)<=(57);
    $187 = $99;
   } else {
    $187 = 0;
   }
  } else {
   $187 = 0;
  }
  $100 = $u;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($100) + 4)|0;
  $104 = $103;
  $105 = HEAP32[$104>>2]|0;
  if (!($187)) {
   break;
  }
  $106 = (___muldi3(($102|0),($105|0),10,0)|0);
  $107 = tempRet0;
  $108 = HEAP32[$c>>2]|0;
  $109 = ($108|0)<(0);
  $110 = $109 << 31 >> 31;
  $111 = (_i64Add(($106|0),($107|0),($108|0),($110|0))|0);
  $112 = tempRet0;
  $113 = (_i64Subtract(($111|0),($112|0),48,0)|0);
  $114 = tempRet0;
  $115 = $u;
  $116 = $115;
  HEAP32[$116>>2] = $113;
  $117 = (($115) + 4)|0;
  $118 = $117;
  HEAP32[$118>>2] = $114;
  $119 = HEAP32[$incr>>2]|0;
  $120 = HEAP32[$i>>2]|0;
  $121 = (($120) + ($119))|0;
  HEAP32[$i>>2] = $121;
 }
 $122 = ($105>>>0)>(2147483647);
 $123 = ($102>>>0)>(4294967295);
 $124 = ($105|0)==(2147483647);
 $125 = $124 & $123;
 $126 = $122 | $125;
 $127 = HEAP32[$neg>>2]|0;
 $128 = ($127|0)!=(0);
 do {
  if ($126) {
   $129 = $128 ? 0 : -1;
   $130 = $128 ? -2147483648 : 2147483647;
   $131 = HEAP32[$2>>2]|0;
   $132 = $131;
   $133 = $132;
   HEAP32[$133>>2] = $129;
   $134 = (($132) + 4)|0;
   $135 = $134;
   HEAP32[$135>>2] = $130;
  } else {
   $136 = $u;
   $137 = $136;
   $138 = HEAP32[$137>>2]|0;
   $139 = (($136) + 4)|0;
   $140 = $139;
   $141 = HEAP32[$140>>2]|0;
   if ($128) {
    $142 = (_i64Subtract(0,0,($138|0),($141|0))|0);
    $143 = tempRet0;
    $144 = HEAP32[$2>>2]|0;
    $145 = $144;
    $146 = $145;
    HEAP32[$146>>2] = $142;
    $147 = (($145) + 4)|0;
    $148 = $147;
    HEAP32[$148>>2] = $143;
    break;
   } else {
    $149 = HEAP32[$2>>2]|0;
    $150 = $149;
    $151 = $150;
    HEAP32[$151>>2] = $138;
    $152 = (($150) + 4)|0;
    $153 = $152;
    HEAP32[$153>>2] = $141;
    break;
   }
  }
 } while(0);
 $154 = HEAP32[$c>>2]|0;
 $155 = ($154|0)!=(0);
 if ($155) {
  $156 = HEAP32[$i>>2]|0;
  $157 = HEAP32[$1>>2]|0;
  $158 = (($157) + ($156)|0);
  $159 = HEAP32[$zEnd>>2]|0;
  $160 = ($158>>>0)<($159>>>0);
  if (!($160)) {
   label = 33;
  }
 } else {
  label = 33;
 }
 do {
  if ((label|0) == 33) {
   $161 = HEAP32[$i>>2]|0;
   $162 = ($161|0)==(0);
   if ($162) {
    $163 = HEAP32[$zStart>>2]|0;
    $164 = HEAP32[$1>>2]|0;
    $165 = ($163|0)==($164|0);
    if ($165) {
     break;
    }
   }
   $166 = HEAP32[$i>>2]|0;
   $167 = HEAP32[$incr>>2]|0;
   $168 = ($167*19)|0;
   $169 = ($166|0)>($168|0);
   $170 = HEAP32[$nonNum>>2]|0;
   $171 = ($170|0)!=(0);
   $or$cond = $169 | $171;
   if (!($or$cond)) {
    $172 = HEAP32[$i>>2]|0;
    $173 = HEAP32[$incr>>2]|0;
    $174 = ($173*19)|0;
    $175 = ($172|0)<($174|0);
    if ($175) {
     HEAP32[$0>>2] = 0;
     $186 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($186|0);
    }
    $176 = HEAP32[$1>>2]|0;
    $177 = HEAP32[$incr>>2]|0;
    $178 = (_compare2pow63($176,$177)|0);
    HEAP32[$c>>2] = $178;
    $179 = HEAP32[$c>>2]|0;
    $180 = ($179|0)<(0);
    if ($180) {
     HEAP32[$0>>2] = 0;
     $186 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($186|0);
    }
    $181 = HEAP32[$c>>2]|0;
    $182 = ($181|0)>(0);
    if ($182) {
     HEAP32[$0>>2] = 1;
     $186 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($186|0);
    } else {
     $183 = HEAP32[$neg>>2]|0;
     $184 = ($183|0)!=(0);
     $185 = $184 ? 0 : 2;
     HEAP32[$0>>2] = $185;
     $186 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($186|0);
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 1;
 $186 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($186|0);
}
function _compare2pow63($zNum,$incr) {
 $zNum = $zNum|0;
 $incr = $incr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $pow63 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $c = sp + 8|0;
 $i = sp + 4|0;
 $pow63 = sp;
 HEAP32[$0>>2] = $zNum;
 HEAP32[$1>>2] = $incr;
 HEAP32[$c>>2] = 0;
 HEAP32[$pow63>>2] = 19297;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$c>>2]|0;
  $3 = ($2|0)==(0);
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)<(18);
  $6 = $3 ? $5 : 0;
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = Math_imul($7, $8)|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + ($9)|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$pow63>>2]|0;
  $16 = (($15) + ($14)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = (($13) - ($18))|0;
  $20 = ($19*10)|0;
  HEAP32[$c>>2] = $20;
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 $23 = HEAP32[$c>>2]|0;
 $24 = ($23|0)==(0);
 if (!($24)) {
  $32 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25*18)|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + ($26)|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = $29 << 24 >> 24;
 $31 = (($30) - 56)|0;
 HEAP32[$c>>2] = $31;
 $32 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3VdbeMemHandleBom($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $b1 = 0, $b2 = 0, $bom = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 $bom = sp + 10|0;
 $b1 = sp + 9|0;
 $b2 = sp + 8|0;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$rc>>2] = 0;
 HEAP8[$bom>>0] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(1);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 16|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  HEAP8[$b1>>0] = $8;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ((($11)) + 1|0);
  $13 = HEAP8[$12>>0]|0;
  HEAP8[$b2>>0] = $13;
  $14 = HEAP8[$b1>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(254);
  if ($16) {
   $17 = HEAP8[$b2>>0]|0;
   $18 = $17&255;
   $19 = ($18|0)==(255);
   if ($19) {
    HEAP8[$bom>>0] = 3;
   }
  }
  $20 = HEAP8[$b1>>0]|0;
  $21 = $20&255;
  $22 = ($21|0)==(255);
  if ($22) {
   $23 = HEAP8[$b2>>0]|0;
   $24 = $23&255;
   $25 = ($24|0)==(254);
   if ($25) {
    HEAP8[$bom>>0] = 2;
   }
  }
 }
 $26 = HEAP8[$bom>>0]|0;
 $27 = ($26<<24>>24)!=(0);
 if (!($27)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = (_sqlite3VdbeMemMakeWriteable($28)|0);
 HEAP32[$rc>>2] = $29;
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(0);
 if (!($31)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) - 2)|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 16|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ((($41)) + 2|0);
 $43 = HEAP32[$0>>2]|0;
 $44 = ((($43)) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 _memmove(($38|0),($42|0),($45|0))|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48)|0);
 HEAP8[$52>>0] = 0;
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 12|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + 1)|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + ($56)|0);
 HEAP8[$60>>0] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 8|0);
 $63 = HEAP16[$62>>1]|0;
 $64 = $63&65535;
 $65 = $64 | 512;
 $66 = $65&65535;
 HEAP16[$62>>1] = $66;
 $67 = HEAP8[$bom>>0]|0;
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 10|0);
 HEAP8[$69>>0] = $67;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _vdbeReleaseAndSetInt64($pMem,$0,$1) {
 $pMem = $pMem|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pMem;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemSetNull($8);
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = $15;
 $17 = $16;
 HEAP32[$17>>2] = $11;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $14;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 8|0);
 HEAP16[$21>>1] = 4;
 STACKTOP = sp;return;
}
function _sqlite3VdbeList($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apSub = 0, $db = 0, $i = 0;
 var $j = 0, $j1 = 0, $nByte = 0, $nRow = 0, $nSub = 0, $pMem = 0, $pOp = 0, $pSub = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $zP4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $nRow = sp + 60|0;
 $nSub = sp + 56|0;
 $apSub = sp + 52|0;
 $pSub = sp + 48|0;
 $db = sp + 44|0;
 $i = sp + 40|0;
 $rc = sp + 36|0;
 $pMem = sp + 32|0;
 $zP4 = sp + 28|0;
 $pOp = sp + 24|0;
 $j = sp + 20|0;
 $nByte = sp + 16|0;
 $j1 = sp + 12|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$nSub>>2] = 0;
 HEAP32[$apSub>>2] = 0;
 HEAP32[$pSub>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 40|0);
 HEAP32[$pMem>>2] = $7;
 $8 = HEAP32[$pMem>>2]|0;
 _releaseMemArray($8,8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 20|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 80|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(7);
 if ($14) {
  $15 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($15);
  HEAP32[$0>>2] = 1;
  $336 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($336|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 32|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$nRow>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 88|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21 & 3;
 $23 = $22&65535;
 $24 = ($23|0)==(1);
 L5: do {
  if ($24) {
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ((($27)) + 360|0);
   HEAP32[$pSub>>2] = $28;
   $29 = HEAP32[$pSub>>2]|0;
   $30 = ((($29)) + 8|0);
   $31 = HEAP16[$30>>1]|0;
   $32 = $31&65535;
   $33 = $32 & 16;
   $34 = ($33|0)!=(0);
   if ($34) {
    $35 = HEAP32[$pSub>>2]|0;
    $36 = ((($35)) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37>>>0) / 4)&-1;
    HEAP32[$nSub>>2] = $38;
    $39 = HEAP32[$pSub>>2]|0;
    $40 = ((($39)) + 16|0);
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$apSub>>2] = $41;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $42 = HEAP32[$i>>2]|0;
    $43 = HEAP32[$nSub>>2]|0;
    $44 = ($42|0)<($43|0);
    if (!($44)) {
     break L5;
    }
    $45 = HEAP32[$i>>2]|0;
    $46 = HEAP32[$apSub>>2]|0;
    $47 = (($46) + ($45<<2)|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = ((($48)) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$nRow>>2]|0;
    $52 = (($51) + ($50))|0;
    HEAP32[$nRow>>2] = $52;
    $53 = HEAP32[$i>>2]|0;
    $54 = (($53) + 1)|0;
    HEAP32[$i>>2] = $54;
   }
  }
 } while(0);
 while(1) {
  $55 = HEAP32[$1>>2]|0;
  $56 = ((($55)) + 76|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$56>>2] = $58;
  HEAP32[$i>>2] = $57;
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$nRow>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 88|0);
  $64 = HEAP16[$63>>1]|0;
  $65 = $64 & 3;
  $66 = $65&65535;
  $67 = ($66|0)==(2);
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 4|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + (($68*20)|0)|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = ($74|0)!=(161);
  if (!($75)) {
   break;
  }
 }
 $76 = HEAP32[$i>>2]|0;
 $77 = HEAP32[$nRow>>2]|0;
 $78 = ($76|0)>=($77|0);
 do {
  if ($78) {
   $79 = HEAP32[$1>>2]|0;
   $80 = ((($79)) + 80|0);
   HEAP32[$80>>2] = 0;
   HEAP32[$rc>>2] = 101;
  } else {
   $81 = HEAP32[$db>>2]|0;
   $82 = ((($81)) + 240|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = ($83|0)!=(0);
   if ($84) {
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 80|0);
    HEAP32[$86>>2] = 9;
    HEAP32[$rc>>2] = 1;
    $87 = HEAP32[$1>>2]|0;
    $88 = HEAP32[$1>>2]|0;
    $89 = ((($88)) + 80|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = (_sqlite3ErrStr($90)|0);
    _sqlite3VdbeError($87,$91,$vararg_buffer);
    break;
   }
   $92 = HEAP32[$i>>2]|0;
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 32|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = ($92|0)<($95|0);
   if ($96) {
    $97 = HEAP32[$i>>2]|0;
    $98 = HEAP32[$1>>2]|0;
    $99 = ((($98)) + 4|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = (($100) + (($97*20)|0)|0);
    HEAP32[$pOp>>2] = $101;
   } else {
    $102 = HEAP32[$1>>2]|0;
    $103 = ((($102)) + 32|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = HEAP32[$i>>2]|0;
    $106 = (($105) - ($104))|0;
    HEAP32[$i>>2] = $106;
    HEAP32[$j>>2] = 0;
    while(1) {
     $107 = HEAP32[$i>>2]|0;
     $108 = HEAP32[$j>>2]|0;
     $109 = HEAP32[$apSub>>2]|0;
     $110 = (($109) + ($108<<2)|0);
     $111 = HEAP32[$110>>2]|0;
     $112 = ((($111)) + 4|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = ($107|0)>=($113|0);
     if (!($114)) {
      break;
     }
     $115 = HEAP32[$j>>2]|0;
     $116 = HEAP32[$apSub>>2]|0;
     $117 = (($116) + ($115<<2)|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = ((($118)) + 4|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = HEAP32[$i>>2]|0;
     $122 = (($121) - ($120))|0;
     HEAP32[$i>>2] = $122;
     $123 = HEAP32[$j>>2]|0;
     $124 = (($123) + 1)|0;
     HEAP32[$j>>2] = $124;
    }
    $125 = HEAP32[$i>>2]|0;
    $126 = HEAP32[$j>>2]|0;
    $127 = HEAP32[$apSub>>2]|0;
    $128 = (($127) + ($126<<2)|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = HEAP32[$129>>2]|0;
    $131 = (($130) + (($125*20)|0)|0);
    HEAP32[$pOp>>2] = $131;
   }
   $132 = HEAP32[$1>>2]|0;
   $133 = ((($132)) + 88|0);
   $134 = HEAP16[$133>>1]|0;
   $135 = $134 & 3;
   $136 = $135&65535;
   $137 = ($136|0)==(1);
   if ($137) {
    $138 = HEAP32[$pMem>>2]|0;
    $139 = ((($138)) + 8|0);
    HEAP16[$139>>1] = 4;
    $140 = HEAP32[$i>>2]|0;
    $141 = ($140|0)<(0);
    $142 = $141 << 31 >> 31;
    $143 = HEAP32[$pMem>>2]|0;
    $144 = $143;
    $145 = $144;
    HEAP32[$145>>2] = $140;
    $146 = (($144) + 4)|0;
    $147 = $146;
    HEAP32[$147>>2] = $142;
    $148 = HEAP32[$pMem>>2]|0;
    $149 = ((($148)) + 40|0);
    HEAP32[$pMem>>2] = $149;
    $150 = HEAP32[$pMem>>2]|0;
    $151 = ((($150)) + 8|0);
    HEAP16[$151>>1] = 2562;
    $152 = HEAP32[$pOp>>2]|0;
    $153 = HEAP8[$152>>0]|0;
    $154 = $153&255;
    $155 = (_sqlite3OpcodeName($154)|0);
    $156 = HEAP32[$pMem>>2]|0;
    $157 = ((($156)) + 16|0);
    HEAP32[$157>>2] = $155;
    $158 = HEAP32[$pMem>>2]|0;
    $159 = ((($158)) + 16|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (_sqlite3Strlen30($160)|0);
    $162 = HEAP32[$pMem>>2]|0;
    $163 = ((($162)) + 12|0);
    HEAP32[$163>>2] = $161;
    $164 = HEAP32[$pMem>>2]|0;
    $165 = ((($164)) + 10|0);
    HEAP8[$165>>0] = 1;
    $166 = HEAP32[$pMem>>2]|0;
    $167 = ((($166)) + 40|0);
    HEAP32[$pMem>>2] = $167;
    $168 = HEAP32[$pOp>>2]|0;
    $169 = ((($168)) + 1|0);
    $170 = HEAP8[$169>>0]|0;
    $171 = $170 << 24 >> 24;
    $172 = ($171|0)==(-18);
    if ($172) {
     $173 = HEAP32[$nSub>>2]|0;
     $174 = (($173) + 1)|0;
     $175 = $174<<2;
     HEAP32[$nByte>>2] = $175;
     HEAP32[$j1>>2] = 0;
     while(1) {
      $176 = HEAP32[$j1>>2]|0;
      $177 = HEAP32[$nSub>>2]|0;
      $178 = ($176|0)<($177|0);
      if (!($178)) {
       break;
      }
      $179 = HEAP32[$j1>>2]|0;
      $180 = HEAP32[$apSub>>2]|0;
      $181 = (($180) + ($179<<2)|0);
      $182 = HEAP32[$181>>2]|0;
      $183 = HEAP32[$pOp>>2]|0;
      $184 = ((($183)) + 16|0);
      $185 = HEAP32[$184>>2]|0;
      $186 = ($182|0)==($185|0);
      if ($186) {
       break;
      }
      $187 = HEAP32[$j1>>2]|0;
      $188 = (($187) + 1)|0;
      HEAP32[$j1>>2] = $188;
     }
     $189 = HEAP32[$j1>>2]|0;
     $190 = HEAP32[$nSub>>2]|0;
     $191 = ($189|0)==($190|0);
     if ($191) {
      $192 = HEAP32[$pSub>>2]|0;
      $193 = HEAP32[$nByte>>2]|0;
      $194 = HEAP32[$nSub>>2]|0;
      $195 = ($194|0)!=(0);
      $196 = $195&1;
      $197 = (_sqlite3VdbeMemGrow($192,$193,$196)|0);
      $198 = (0)==($197|0);
      if ($198) {
       $199 = HEAP32[$pSub>>2]|0;
       $200 = ((($199)) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       HEAP32[$apSub>>2] = $201;
       $202 = HEAP32[$pOp>>2]|0;
       $203 = ((($202)) + 16|0);
       $204 = HEAP32[$203>>2]|0;
       $205 = HEAP32[$nSub>>2]|0;
       $206 = (($205) + 1)|0;
       HEAP32[$nSub>>2] = $206;
       $207 = HEAP32[$apSub>>2]|0;
       $208 = (($207) + ($205<<2)|0);
       HEAP32[$208>>2] = $204;
       $209 = HEAP32[$pSub>>2]|0;
       $210 = ((($209)) + 8|0);
       $211 = HEAP16[$210>>1]|0;
       $212 = $211&65535;
       $213 = $212 | 16;
       $214 = $213&65535;
       HEAP16[$210>>1] = $214;
       $215 = HEAP32[$nSub>>2]|0;
       $216 = $215<<2;
       $217 = HEAP32[$pSub>>2]|0;
       $218 = ((($217)) + 12|0);
       HEAP32[$218>>2] = $216;
      }
     }
    }
   }
   $219 = HEAP32[$pMem>>2]|0;
   $220 = ((($219)) + 8|0);
   HEAP16[$220>>1] = 4;
   $221 = HEAP32[$pOp>>2]|0;
   $222 = ((($221)) + 4|0);
   $223 = HEAP32[$222>>2]|0;
   $224 = ($223|0)<(0);
   $225 = $224 << 31 >> 31;
   $226 = HEAP32[$pMem>>2]|0;
   $227 = $226;
   $228 = $227;
   HEAP32[$228>>2] = $223;
   $229 = (($227) + 4)|0;
   $230 = $229;
   HEAP32[$230>>2] = $225;
   $231 = HEAP32[$pMem>>2]|0;
   $232 = ((($231)) + 40|0);
   HEAP32[$pMem>>2] = $232;
   $233 = HEAP32[$pMem>>2]|0;
   $234 = ((($233)) + 8|0);
   HEAP16[$234>>1] = 4;
   $235 = HEAP32[$pOp>>2]|0;
   $236 = ((($235)) + 8|0);
   $237 = HEAP32[$236>>2]|0;
   $238 = ($237|0)<(0);
   $239 = $238 << 31 >> 31;
   $240 = HEAP32[$pMem>>2]|0;
   $241 = $240;
   $242 = $241;
   HEAP32[$242>>2] = $237;
   $243 = (($241) + 4)|0;
   $244 = $243;
   HEAP32[$244>>2] = $239;
   $245 = HEAP32[$pMem>>2]|0;
   $246 = ((($245)) + 40|0);
   HEAP32[$pMem>>2] = $246;
   $247 = HEAP32[$pMem>>2]|0;
   $248 = ((($247)) + 8|0);
   HEAP16[$248>>1] = 4;
   $249 = HEAP32[$pOp>>2]|0;
   $250 = ((($249)) + 12|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = ($251|0)<(0);
   $253 = $252 << 31 >> 31;
   $254 = HEAP32[$pMem>>2]|0;
   $255 = $254;
   $256 = $255;
   HEAP32[$256>>2] = $251;
   $257 = (($255) + 4)|0;
   $258 = $257;
   HEAP32[$258>>2] = $253;
   $259 = HEAP32[$pMem>>2]|0;
   $260 = ((($259)) + 40|0);
   HEAP32[$pMem>>2] = $260;
   $261 = HEAP32[$pMem>>2]|0;
   $262 = (_sqlite3VdbeMemClearAndResize($261,100)|0);
   $263 = ($262|0)!=(0);
   if ($263) {
    HEAP32[$0>>2] = 1;
    $336 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($336|0);
   }
   $264 = HEAP32[$pMem>>2]|0;
   $265 = ((($264)) + 8|0);
   HEAP16[$265>>1] = 514;
   $266 = HEAP32[$pOp>>2]|0;
   $267 = HEAP32[$pMem>>2]|0;
   $268 = ((($267)) + 16|0);
   $269 = HEAP32[$268>>2]|0;
   $270 = HEAP32[$pMem>>2]|0;
   $271 = ((($270)) + 24|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = (_displayP4($266,$269,$272)|0);
   HEAP32[$zP4>>2] = $273;
   $274 = HEAP32[$zP4>>2]|0;
   $275 = HEAP32[$pMem>>2]|0;
   $276 = ((($275)) + 16|0);
   $277 = HEAP32[$276>>2]|0;
   $278 = ($274|0)!=($277|0);
   $279 = HEAP32[$pMem>>2]|0;
   if ($278) {
    $280 = HEAP32[$zP4>>2]|0;
    (_sqlite3VdbeMemSetStr($279,$280,-1,1,0)|0);
   } else {
    $281 = ((($279)) + 16|0);
    $282 = HEAP32[$281>>2]|0;
    $283 = (_sqlite3Strlen30($282)|0);
    $284 = HEAP32[$pMem>>2]|0;
    $285 = ((($284)) + 12|0);
    HEAP32[$285>>2] = $283;
    $286 = HEAP32[$pMem>>2]|0;
    $287 = ((($286)) + 10|0);
    HEAP8[$287>>0] = 1;
   }
   $288 = HEAP32[$pMem>>2]|0;
   $289 = ((($288)) + 40|0);
   HEAP32[$pMem>>2] = $289;
   $290 = HEAP32[$1>>2]|0;
   $291 = ((($290)) + 88|0);
   $292 = HEAP16[$291>>1]|0;
   $293 = $292 & 3;
   $294 = $293&65535;
   $295 = ($294|0)==(1);
   do {
    if ($295) {
     $296 = HEAP32[$pMem>>2]|0;
     $297 = (_sqlite3VdbeMemClearAndResize($296,4)|0);
     $298 = ($297|0)!=(0);
     if (!($298)) {
      $299 = HEAP32[$pMem>>2]|0;
      $300 = ((($299)) + 8|0);
      HEAP16[$300>>1] = 514;
      $301 = HEAP32[$pMem>>2]|0;
      $302 = ((($301)) + 12|0);
      HEAP32[$302>>2] = 2;
      $303 = HEAP32[$pMem>>2]|0;
      $304 = ((($303)) + 16|0);
      $305 = HEAP32[$304>>2]|0;
      $306 = HEAP32[$pOp>>2]|0;
      $307 = ((($306)) + 3|0);
      $308 = HEAP8[$307>>0]|0;
      $309 = $308&255;
      HEAP32[$vararg_buffer2>>2] = $309;
      (_sqlite3_snprintf(3,$305,19356,$vararg_buffer2)|0);
      $310 = HEAP32[$pMem>>2]|0;
      $311 = ((($310)) + 10|0);
      HEAP8[$311>>0] = 1;
      $312 = HEAP32[$pMem>>2]|0;
      $313 = ((($312)) + 40|0);
      HEAP32[$pMem>>2] = $313;
      $314 = HEAP32[$pMem>>2]|0;
      $315 = ((($314)) + 8|0);
      HEAP16[$315>>1] = 1;
      break;
     }
     HEAP32[$0>>2] = 1;
     $336 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($336|0);
    }
   } while(0);
   $316 = HEAP32[$1>>2]|0;
   $317 = ((($316)) + 88|0);
   $318 = HEAP16[$317>>1]|0;
   $319 = $318 & 3;
   $320 = $319&65535;
   $321 = (($320) - 1)|0;
   $322 = $321<<2;
   $323 = (8 - ($322))|0;
   $324 = $323&65535;
   $325 = HEAP32[$1>>2]|0;
   $326 = ((($325)) + 84|0);
   HEAP16[$326>>1] = $324;
   $327 = HEAP32[$1>>2]|0;
   $328 = ((($327)) + 8|0);
   $329 = HEAP32[$328>>2]|0;
   $330 = ((($329)) + 40|0);
   $331 = HEAP32[$1>>2]|0;
   $332 = ((($331)) + 20|0);
   HEAP32[$332>>2] = $330;
   $333 = HEAP32[$1>>2]|0;
   $334 = ((($333)) + 80|0);
   HEAP32[$334>>2] = 0;
   HEAP32[$rc>>2] = 100;
  }
 } while(0);
 $335 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $335;
 $336 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($336|0);
}
function _sqlite3VdbeExec($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0.0, $1129 = 0, $113 = 0, $1130 = 0.0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0.0, $1135 = 0.0, $1136 = 0.0, $1137 = 0.0, $1138 = 0.0, $1139 = 0.0, $114 = 0, $1140 = 0.0;
 var $1141 = 0.0, $1142 = 0.0, $1143 = 0.0, $1144 = 0, $1145 = 0.0, $1146 = 0.0, $1147 = 0.0, $1148 = 0.0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0.0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0.0, $1199 = 0.0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0.0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0, $1988 = 0;
 var $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0, $2004 = 0;
 var $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0, $2022 = 0;
 var $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0, $2040 = 0;
 var $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0, $2058 = 0, $2059 = 0;
 var $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0, $2076 = 0, $2077 = 0;
 var $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0, $2094 = 0, $2095 = 0;
 var $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0, $2111 = 0, $2112 = 0;
 var $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0, $213 = 0, $2130 = 0;
 var $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0, $2148 = 0, $2149 = 0;
 var $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0, $2166 = 0, $2167 = 0;
 var $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0.0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0, $2184 = 0, $2185 = 0;
 var $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0, $2201 = 0, $2202 = 0;
 var $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0, $222 = 0, $2220 = 0;
 var $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0, $2238 = 0, $2239 = 0;
 var $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0, $2253 = 0, $2254 = 0, $2255 = 0, $2256 = 0, $2257 = 0;
 var $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0, $2274 = 0, $2275 = 0;
 var $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0, $2292 = 0, $2293 = 0;
 var $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0, $231 = 0, $2310 = 0;
 var $2311 = 0, $2312 = 0, $2313 = 0, $2314 = 0, $2315 = 0, $2316 = 0, $2317 = 0, $2318 = 0, $2319 = 0, $232 = 0, $2320 = 0, $2321 = 0, $2322 = 0, $2323 = 0, $2324 = 0, $2325 = 0, $2326 = 0, $2327 = 0, $2328 = 0, $2329 = 0;
 var $233 = 0, $2330 = 0, $2331 = 0, $2332 = 0, $2333 = 0, $2334 = 0, $2335 = 0, $2336 = 0, $2337 = 0, $2338 = 0, $2339 = 0, $234 = 0, $2340 = 0, $2341 = 0, $2342 = 0, $2343 = 0, $2344 = 0, $2345 = 0, $2346 = 0, $2347 = 0;
 var $2348 = 0, $2349 = 0, $235 = 0, $2350 = 0, $2351 = 0, $2352 = 0, $2353 = 0, $2354 = 0, $2355 = 0, $2356 = 0, $2357 = 0, $2358 = 0, $2359 = 0, $236 = 0, $2360 = 0, $2361 = 0, $2362 = 0, $2363 = 0, $2364 = 0, $2365 = 0;
 var $2366 = 0, $2367 = 0, $2368 = 0, $2369 = 0, $237 = 0, $2370 = 0, $2371 = 0, $2372 = 0, $2373 = 0, $2374 = 0, $2375 = 0, $2376 = 0, $2377 = 0, $2378 = 0, $2379 = 0, $238 = 0, $2380 = 0, $2381 = 0, $2382 = 0, $2383 = 0;
 var $2384 = 0, $2385 = 0, $2386 = 0, $2387 = 0, $2388 = 0, $2389 = 0, $239 = 0, $2390 = 0, $2391 = 0, $2392 = 0, $2393 = 0, $2394 = 0, $2395 = 0, $2396 = 0, $2397 = 0, $2398 = 0, $2399 = 0, $24 = 0, $240 = 0, $2400 = 0;
 var $2401 = 0, $2402 = 0, $2403 = 0, $2404 = 0, $2405 = 0, $2406 = 0, $2407 = 0, $2408 = 0, $2409 = 0, $241 = 0, $2410 = 0, $2411 = 0, $2412 = 0, $2413 = 0, $2414 = 0, $2415 = 0, $2416 = 0, $2417 = 0, $2418 = 0, $2419 = 0;
 var $242 = 0, $2420 = 0, $2421 = 0, $2422 = 0, $2423 = 0, $2424 = 0, $2425 = 0, $2426 = 0, $2427 = 0, $2428 = 0, $2429 = 0, $243 = 0, $2430 = 0, $2431 = 0, $2432 = 0, $2433 = 0, $2434 = 0, $2435 = 0, $2436 = 0, $2437 = 0;
 var $2438 = 0, $2439 = 0, $244 = 0, $2440 = 0, $2441 = 0, $2442 = 0, $2443 = 0, $2444 = 0, $2445 = 0, $2446 = 0, $2447 = 0, $2448 = 0, $2449 = 0, $245 = 0, $2450 = 0, $2451 = 0, $2452 = 0, $2453 = 0, $2454 = 0, $2455 = 0;
 var $2456 = 0, $2457 = 0, $2458 = 0, $2459 = 0, $246 = 0, $2460 = 0, $2461 = 0, $2462 = 0, $2463 = 0, $2464 = 0, $2465 = 0, $2466 = 0, $2467 = 0, $2468 = 0, $2469 = 0, $247 = 0, $2470 = 0, $2471 = 0, $2472 = 0, $2473 = 0;
 var $2474 = 0, $2475 = 0, $2476 = 0, $2477 = 0, $2478 = 0, $2479 = 0, $248 = 0, $2480 = 0, $2481 = 0, $2482 = 0, $2483 = 0, $2484 = 0, $2485 = 0, $2486 = 0, $2487 = 0, $2488 = 0, $2489 = 0, $249 = 0, $2490 = 0, $2491 = 0;
 var $2492 = 0, $2493 = 0, $2494 = 0, $2495 = 0, $2496 = 0, $2497 = 0, $2498 = 0, $2499 = 0, $25 = 0, $250 = 0, $2500 = 0, $2501 = 0, $2502 = 0, $2503 = 0, $2504 = 0, $2505 = 0, $2506 = 0, $2507 = 0, $2508 = 0, $2509 = 0;
 var $251 = 0, $2510 = 0, $2511 = 0, $2512 = 0, $2513 = 0, $2514 = 0, $2515 = 0, $2516 = 0, $2517 = 0, $2518 = 0, $2519 = 0, $252 = 0, $2520 = 0, $2521 = 0, $2522 = 0, $2523 = 0, $2524 = 0, $2525 = 0, $2526 = 0, $2527 = 0;
 var $2528 = 0, $2529 = 0, $253 = 0, $2530 = 0, $2531 = 0, $2532 = 0, $2533 = 0, $2534 = 0, $2535 = 0, $2536 = 0, $2537 = 0, $2538 = 0, $2539 = 0, $254 = 0, $2540 = 0, $2541 = 0, $2542 = 0, $2543 = 0, $2544 = 0, $2545 = 0;
 var $2546 = 0, $2547 = 0, $2548 = 0, $2549 = 0, $255 = 0, $2550 = 0, $2551 = 0, $2552 = 0, $2553 = 0, $2554 = 0, $2555 = 0, $2556 = 0, $2557 = 0, $2558 = 0, $2559 = 0, $256 = 0, $2560 = 0, $2561 = 0, $2562 = 0, $2563 = 0;
 var $2564 = 0, $2565 = 0, $2566 = 0, $2567 = 0, $2568 = 0, $2569 = 0, $257 = 0, $2570 = 0, $2571 = 0, $2572 = 0, $2573 = 0, $2574 = 0, $2575 = 0, $2576 = 0, $2577 = 0, $2578 = 0, $2579 = 0, $258 = 0, $2580 = 0, $2581 = 0;
 var $2582 = 0, $2583 = 0, $2584 = 0, $2585 = 0, $2586 = 0, $2587 = 0, $2588 = 0, $2589 = 0, $259 = 0, $2590 = 0, $2591 = 0, $2592 = 0, $2593 = 0, $2594 = 0, $2595 = 0, $2596 = 0, $2597 = 0, $2598 = 0, $2599 = 0, $26 = 0;
 var $260 = 0, $2600 = 0, $2601 = 0, $2602 = 0, $2603 = 0, $2604 = 0, $2605 = 0, $2606 = 0, $2607 = 0, $2608 = 0, $2609 = 0, $261 = 0, $2610 = 0, $2611 = 0, $2612 = 0, $2613 = 0, $2614 = 0, $2615 = 0, $2616 = 0, $2617 = 0;
 var $2618 = 0, $2619 = 0, $262 = 0, $2620 = 0, $2621 = 0, $2622 = 0, $2623 = 0, $2624 = 0, $2625 = 0, $2626 = 0, $2627 = 0, $2628 = 0, $2629 = 0, $263 = 0, $2630 = 0, $2631 = 0, $2632 = 0, $2633 = 0, $2634 = 0, $2635 = 0;
 var $2636 = 0, $2637 = 0, $2638 = 0, $2639 = 0, $264 = 0, $2640 = 0, $2641 = 0, $2642 = 0, $2643 = 0, $2644 = 0, $2645 = 0, $2646 = 0, $2647 = 0, $2648 = 0, $2649 = 0, $265 = 0, $2650 = 0, $2651 = 0, $2652 = 0, $2653 = 0;
 var $2654 = 0, $2655 = 0, $2656 = 0, $2657 = 0, $2658 = 0, $2659 = 0, $266 = 0, $2660 = 0, $2661 = 0, $2662 = 0, $2663 = 0, $2664 = 0, $2665 = 0, $2666 = 0, $2667 = 0, $2668 = 0, $2669 = 0, $267 = 0, $2670 = 0, $2671 = 0;
 var $2672 = 0, $2673 = 0, $2674 = 0, $2675 = 0, $2676 = 0, $2677 = 0, $2678 = 0, $2679 = 0, $268 = 0, $2680 = 0, $2681 = 0, $2682 = 0, $2683 = 0, $2684 = 0, $2685 = 0, $2686 = 0, $2687 = 0, $2688 = 0, $2689 = 0, $269 = 0;
 var $2690 = 0, $2691 = 0, $2692 = 0, $2693 = 0, $2694 = 0, $2695 = 0, $2696 = 0, $2697 = 0, $2698 = 0, $2699 = 0, $27 = 0, $270 = 0, $2700 = 0, $2701 = 0, $2702 = 0, $2703 = 0, $2704 = 0, $2705 = 0, $2706 = 0, $2707 = 0;
 var $2708 = 0, $2709 = 0, $271 = 0, $2710 = 0, $2711 = 0, $2712 = 0, $2713 = 0, $2714 = 0, $2715 = 0, $2716 = 0, $2717 = 0, $2718 = 0, $2719 = 0, $272 = 0, $2720 = 0, $2721 = 0, $2722 = 0, $2723 = 0, $2724 = 0, $2725 = 0;
 var $2726 = 0, $2727 = 0, $2728 = 0, $2729 = 0, $273 = 0, $2730 = 0, $2731 = 0, $2732 = 0, $2733 = 0, $2734 = 0, $2735 = 0, $2736 = 0, $2737 = 0, $2738 = 0, $2739 = 0, $274 = 0, $2740 = 0, $2741 = 0, $2742 = 0, $2743 = 0;
 var $2744 = 0, $2745 = 0, $2746 = 0, $2747 = 0, $2748 = 0, $2749 = 0, $275 = 0, $2750 = 0, $2751 = 0, $2752 = 0, $2753 = 0, $2754 = 0, $2755 = 0, $2756 = 0, $2757 = 0, $2758 = 0, $2759 = 0, $276 = 0, $2760 = 0, $2761 = 0;
 var $2762 = 0, $2763 = 0, $2764 = 0, $2765 = 0, $2766 = 0, $2767 = 0, $2768 = 0, $2769 = 0, $277 = 0, $2770 = 0, $2771 = 0, $2772 = 0, $2773 = 0, $2774 = 0, $2775 = 0, $2776 = 0, $2777 = 0, $2778 = 0, $2779 = 0, $278 = 0;
 var $2780 = 0, $2781 = 0, $2782 = 0, $2783 = 0, $2784 = 0, $2785 = 0, $2786 = 0, $2787 = 0, $2788 = 0, $2789 = 0, $279 = 0, $2790 = 0, $2791 = 0, $2792 = 0, $2793 = 0, $2794 = 0, $2795 = 0, $2796 = 0, $2797 = 0, $2798 = 0;
 var $2799 = 0, $28 = 0, $280 = 0, $2800 = 0, $2801 = 0, $2802 = 0, $2803 = 0, $2804 = 0, $2805 = 0, $2806 = 0, $2807 = 0, $2808 = 0, $2809 = 0, $281 = 0, $2810 = 0, $2811 = 0, $2812 = 0, $2813 = 0, $2814 = 0, $2815 = 0;
 var $2816 = 0, $2817 = 0, $2818 = 0, $2819 = 0, $282 = 0, $2820 = 0, $2821 = 0, $2822 = 0, $2823 = 0, $2824 = 0, $2825 = 0, $2826 = 0, $2827 = 0, $2828 = 0, $2829 = 0, $283 = 0, $2830 = 0, $2831 = 0, $2832 = 0, $2833 = 0;
 var $2834 = 0, $2835 = 0, $2836 = 0, $2837 = 0, $2838 = 0, $2839 = 0, $284 = 0, $2840 = 0, $2841 = 0, $2842 = 0, $2843 = 0, $2844 = 0, $2845 = 0, $2846 = 0, $2847 = 0, $2848 = 0, $2849 = 0, $285 = 0, $2850 = 0, $2851 = 0;
 var $2852 = 0, $2853 = 0, $2854 = 0, $2855 = 0, $2856 = 0, $2857 = 0, $2858 = 0, $2859 = 0, $286 = 0, $2860 = 0, $2861 = 0, $2862 = 0, $2863 = 0, $2864 = 0, $2865 = 0, $2866 = 0, $2867 = 0, $2868 = 0, $2869 = 0, $287 = 0;
 var $2870 = 0, $2871 = 0, $2872 = 0, $2873 = 0, $2874 = 0, $2875 = 0, $2876 = 0, $2877 = 0, $2878 = 0, $2879 = 0, $288 = 0, $2880 = 0, $2881 = 0, $2882 = 0, $2883 = 0, $2884 = 0, $2885 = 0, $2886 = 0, $2887 = 0, $2888 = 0;
 var $2889 = 0, $289 = 0, $2890 = 0, $2891 = 0, $2892 = 0, $2893 = 0, $2894 = 0, $2895 = 0, $2896 = 0, $2897 = 0, $2898 = 0, $2899 = 0, $29 = 0, $290 = 0, $2900 = 0, $2901 = 0, $2902 = 0, $2903 = 0, $2904 = 0, $2905 = 0;
 var $2906 = 0, $2907 = 0, $2908 = 0, $2909 = 0, $291 = 0, $2910 = 0, $2911 = 0, $2912 = 0, $2913 = 0, $2914 = 0, $2915 = 0, $2916 = 0, $2917 = 0, $2918 = 0, $2919 = 0, $292 = 0, $2920 = 0, $2921 = 0, $2922 = 0, $2923 = 0;
 var $2924 = 0, $2925 = 0, $2926 = 0, $2927 = 0, $2928 = 0, $2929 = 0, $293 = 0, $2930 = 0, $2931 = 0, $2932 = 0, $2933 = 0, $2934 = 0, $2935 = 0, $2936 = 0, $2937 = 0, $2938 = 0, $2939 = 0, $294 = 0, $2940 = 0, $2941 = 0;
 var $2942 = 0, $2943 = 0, $2944 = 0, $2945 = 0, $2946 = 0, $2947 = 0, $2948 = 0, $2949 = 0, $295 = 0, $2950 = 0, $2951 = 0, $2952 = 0, $2953 = 0, $2954 = 0, $2955 = 0, $2956 = 0, $2957 = 0, $2958 = 0, $2959 = 0, $296 = 0;
 var $2960 = 0, $2961 = 0, $2962 = 0, $2963 = 0, $2964 = 0, $2965 = 0, $2966 = 0, $2967 = 0, $2968 = 0, $2969 = 0, $297 = 0, $2970 = 0, $2971 = 0, $2972 = 0, $2973 = 0, $2974 = 0, $2975 = 0, $2976 = 0, $2977 = 0, $2978 = 0;
 var $2979 = 0, $298 = 0, $2980 = 0, $2981 = 0, $2982 = 0, $2983 = 0, $2984 = 0, $2985 = 0, $2986 = 0, $2987 = 0, $2988 = 0, $2989 = 0, $299 = 0, $2990 = 0, $2991 = 0, $2992 = 0, $2993 = 0, $2994 = 0, $2995 = 0, $2996 = 0;
 var $2997 = 0, $2998 = 0, $2999 = 0, $3 = 0, $30 = 0, $300 = 0, $3000 = 0, $3001 = 0, $3002 = 0, $3003 = 0, $3004 = 0, $3005 = 0, $3006 = 0, $3007 = 0, $3008 = 0, $3009 = 0, $301 = 0, $3010 = 0, $3011 = 0, $3012 = 0;
 var $3013 = 0, $3014 = 0, $3015 = 0, $3016 = 0, $3017 = 0, $3018 = 0, $3019 = 0, $302 = 0, $3020 = 0, $3021 = 0, $3022 = 0, $3023 = 0, $3024 = 0, $3025 = 0, $3026 = 0, $3027 = 0, $3028 = 0, $3029 = 0, $303 = 0, $3030 = 0;
 var $3031 = 0, $3032 = 0, $3033 = 0, $3034 = 0, $3035 = 0, $3036 = 0, $3037 = 0, $3038 = 0, $3039 = 0, $304 = 0, $3040 = 0, $3041 = 0, $3042 = 0, $3043 = 0, $3044 = 0, $3045 = 0, $3046 = 0, $3047 = 0, $3048 = 0, $3049 = 0;
 var $305 = 0, $3050 = 0, $3051 = 0, $3052 = 0, $3053 = 0, $3054 = 0, $3055 = 0, $3056 = 0, $3057 = 0, $3058 = 0, $3059 = 0, $306 = 0, $3060 = 0, $3061 = 0, $3062 = 0, $3063 = 0, $3064 = 0, $3065 = 0, $3066 = 0, $3067 = 0;
 var $3068 = 0, $3069 = 0, $307 = 0, $3070 = 0, $3071 = 0, $3072 = 0, $3073 = 0, $3074 = 0, $3075 = 0, $3076 = 0, $3077 = 0, $3078 = 0, $3079 = 0, $308 = 0, $3080 = 0, $3081 = 0, $3082 = 0, $3083 = 0, $3084 = 0, $3085 = 0;
 var $3086 = 0, $3087 = 0, $3088 = 0, $3089 = 0, $309 = 0, $3090 = 0, $3091 = 0, $3092 = 0, $3093 = 0, $3094 = 0, $3095 = 0, $3096 = 0, $3097 = 0, $3098 = 0, $3099 = 0, $31 = 0, $310 = 0, $3100 = 0, $3101 = 0, $3102 = 0;
 var $3103 = 0, $3104 = 0, $3105 = 0, $3106 = 0, $3107 = 0, $3108 = 0, $3109 = 0, $311 = 0, $3110 = 0, $3111 = 0, $3112 = 0, $3113 = 0, $3114 = 0, $3115 = 0, $3116 = 0, $3117 = 0, $3118 = 0, $3119 = 0, $312 = 0, $3120 = 0;
 var $3121 = 0, $3122 = 0, $3123 = 0, $3124 = 0, $3125 = 0, $3126 = 0, $3127 = 0, $3128 = 0, $3129 = 0, $313 = 0, $3130 = 0, $3131 = 0, $3132 = 0, $3133 = 0, $3134 = 0, $3135 = 0, $3136 = 0, $3137 = 0, $3138 = 0, $3139 = 0;
 var $314 = 0, $3140 = 0, $3141 = 0, $3142 = 0, $3143 = 0, $3144 = 0, $3145 = 0, $3146 = 0, $3147 = 0, $3148 = 0, $3149 = 0, $315 = 0, $3150 = 0, $3151 = 0, $3152 = 0, $3153 = 0, $3154 = 0, $3155 = 0, $3156 = 0, $3157 = 0;
 var $3158 = 0, $3159 = 0, $316 = 0, $3160 = 0, $3161 = 0, $3162 = 0, $3163 = 0, $3164 = 0, $3165 = 0, $3166 = 0, $3167 = 0, $3168 = 0, $3169 = 0, $317 = 0, $3170 = 0, $3171 = 0, $3172 = 0, $3173 = 0, $3174 = 0, $3175 = 0;
 var $3176 = 0, $3177 = 0, $3178 = 0, $3179 = 0, $318 = 0, $3180 = 0, $3181 = 0, $3182 = 0, $3183 = 0, $3184 = 0, $3185 = 0, $3186 = 0, $3187 = 0, $3188 = 0, $3189 = 0, $319 = 0, $3190 = 0, $3191 = 0, $3192 = 0, $3193 = 0;
 var $3194 = 0, $3195 = 0, $3196 = 0, $3197 = 0, $3198 = 0, $3199 = 0, $32 = 0, $320 = 0, $3200 = 0, $3201 = 0, $3202 = 0, $3203 = 0, $3204 = 0, $3205 = 0, $3206 = 0, $3207 = 0, $3208 = 0, $3209 = 0, $321 = 0, $3210 = 0;
 var $3211 = 0, $3212 = 0, $3213 = 0, $3214 = 0, $3215 = 0, $3216 = 0, $3217 = 0, $3218 = 0, $3219 = 0, $322 = 0, $3220 = 0, $3221 = 0, $3222 = 0, $3223 = 0, $3224 = 0, $3225 = 0, $3226 = 0, $3227 = 0, $3228 = 0, $3229 = 0;
 var $323 = 0, $3230 = 0, $3231 = 0, $3232 = 0, $3233 = 0, $3234 = 0, $3235 = 0, $3236 = 0, $3237 = 0, $3238 = 0, $3239 = 0, $324 = 0, $3240 = 0, $3241 = 0, $3242 = 0, $3243 = 0, $3244 = 0, $3245 = 0, $3246 = 0, $3247 = 0;
 var $3248 = 0, $3249 = 0, $325 = 0, $3250 = 0, $3251 = 0, $3252 = 0, $3253 = 0, $3254 = 0, $3255 = 0, $3256 = 0, $3257 = 0, $3258 = 0, $3259 = 0, $326 = 0, $3260 = 0, $3261 = 0, $3262 = 0, $3263 = 0, $3264 = 0, $3265 = 0;
 var $3266 = 0, $3267 = 0, $3268 = 0, $3269 = 0, $327 = 0, $3270 = 0, $3271 = 0, $3272 = 0, $3273 = 0, $3274 = 0, $3275 = 0, $3276 = 0, $3277 = 0, $3278 = 0, $3279 = 0, $328 = 0, $3280 = 0, $3281 = 0, $3282 = 0, $3283 = 0;
 var $3284 = 0, $3285 = 0, $3286 = 0, $3287 = 0, $3288 = 0, $3289 = 0, $329 = 0, $3290 = 0, $3291 = 0, $3292 = 0, $3293 = 0, $3294 = 0, $3295 = 0, $3296 = 0, $3297 = 0, $3298 = 0, $3299 = 0, $33 = 0, $330 = 0, $3300 = 0;
 var $3301 = 0, $3302 = 0, $3303 = 0, $3304 = 0, $3305 = 0, $3306 = 0, $3307 = 0, $3308 = 0, $3309 = 0, $331 = 0, $3310 = 0, $3311 = 0, $3312 = 0, $3313 = 0, $3314 = 0, $3315 = 0, $3316 = 0, $3317 = 0, $3318 = 0, $3319 = 0;
 var $332 = 0, $3320 = 0, $3321 = 0, $3322 = 0, $3323 = 0, $3324 = 0, $3325 = 0, $3326 = 0, $3327 = 0, $3328 = 0, $3329 = 0, $333 = 0, $3330 = 0, $3331 = 0, $3332 = 0, $3333 = 0, $3334 = 0, $3335 = 0, $3336 = 0, $3337 = 0;
 var $3338 = 0, $3339 = 0, $334 = 0, $3340 = 0, $3341 = 0, $3342 = 0, $3343 = 0, $3344 = 0, $3345 = 0, $3346 = 0, $3347 = 0, $3348 = 0, $3349 = 0, $335 = 0, $3350 = 0, $3351 = 0, $3352 = 0, $3353 = 0, $3354 = 0, $3355 = 0;
 var $3356 = 0, $3357 = 0, $3358 = 0, $3359 = 0, $336 = 0, $3360 = 0, $3361 = 0, $3362 = 0, $3363 = 0, $3364 = 0, $3365 = 0, $3366 = 0, $3367 = 0, $3368 = 0, $3369 = 0, $337 = 0, $3370 = 0, $3371 = 0, $3372 = 0, $3373 = 0;
 var $3374 = 0, $3375 = 0, $3376 = 0, $3377 = 0, $3378 = 0, $3379 = 0, $338 = 0, $3380 = 0, $3381 = 0, $3382 = 0, $3383 = 0, $3384 = 0, $3385 = 0, $3386 = 0, $3387 = 0, $3388 = 0, $3389 = 0, $339 = 0, $3390 = 0, $3391 = 0;
 var $3392 = 0, $3393 = 0, $3394 = 0, $3395 = 0, $3396 = 0, $3397 = 0, $3398 = 0, $3399 = 0, $34 = 0, $340 = 0, $3400 = 0, $3401 = 0, $3402 = 0, $3403 = 0, $3404 = 0, $3405 = 0, $3406 = 0, $3407 = 0, $3408 = 0, $3409 = 0;
 var $341 = 0, $3410 = 0, $3411 = 0, $3412 = 0, $3413 = 0, $3414 = 0, $3415 = 0, $3416 = 0, $3417 = 0, $3418 = 0, $3419 = 0, $342 = 0, $3420 = 0, $3421 = 0, $3422 = 0, $3423 = 0, $3424 = 0, $3425 = 0, $3426 = 0, $3427 = 0;
 var $3428 = 0, $3429 = 0, $343 = 0, $3430 = 0, $3431 = 0, $3432 = 0, $3433 = 0, $3434 = 0, $3435 = 0, $3436 = 0, $3437 = 0, $3438 = 0, $3439 = 0, $344 = 0, $3440 = 0, $3441 = 0, $3442 = 0, $3443 = 0, $3444 = 0, $3445 = 0;
 var $3446 = 0, $3447 = 0, $3448 = 0, $3449 = 0, $345 = 0, $3450 = 0, $3451 = 0, $3452 = 0, $3453 = 0, $3454 = 0, $3455 = 0, $3456 = 0, $3457 = 0, $3458 = 0, $3459 = 0, $346 = 0, $3460 = 0, $3461 = 0, $3462 = 0, $3463 = 0;
 var $3464 = 0, $3465 = 0, $3466 = 0, $3467 = 0, $3468 = 0, $3469 = 0, $347 = 0, $3470 = 0, $3471 = 0, $3472 = 0, $3473 = 0, $3474 = 0, $3475 = 0, $3476 = 0, $3477 = 0, $3478 = 0, $3479 = 0, $348 = 0, $3480 = 0, $3481 = 0;
 var $3482 = 0, $3483 = 0, $3484 = 0, $3485 = 0, $3486 = 0, $3487 = 0, $3488 = 0, $3489 = 0, $349 = 0, $3490 = 0, $3491 = 0, $3492 = 0, $3493 = 0, $3494 = 0, $3495 = 0, $3496 = 0, $3497 = 0, $3498 = 0, $3499 = 0, $35 = 0;
 var $350 = 0, $3500 = 0, $3501 = 0, $3502 = 0, $3503 = 0, $3504 = 0, $3505 = 0, $3506 = 0, $3507 = 0, $3508 = 0, $3509 = 0, $351 = 0, $3510 = 0, $3511 = 0, $3512 = 0, $3513 = 0, $3514 = 0, $3515 = 0, $3516 = 0, $3517 = 0;
 var $3518 = 0, $3519 = 0, $352 = 0, $3520 = 0, $3521 = 0, $3522 = 0, $3523 = 0, $3524 = 0, $3525 = 0, $3526 = 0, $3527 = 0, $3528 = 0, $3529 = 0, $353 = 0, $3530 = 0, $3531 = 0, $3532 = 0, $3533 = 0, $3534 = 0, $3535 = 0;
 var $3536 = 0, $3537 = 0, $3538 = 0, $3539 = 0, $354 = 0, $3540 = 0, $3541 = 0, $3542 = 0, $3543 = 0, $3544 = 0, $3545 = 0, $3546 = 0, $3547 = 0, $3548 = 0, $3549 = 0, $355 = 0, $3550 = 0, $3551 = 0, $3552 = 0, $3553 = 0;
 var $3554 = 0, $3555 = 0, $3556 = 0, $3557 = 0, $3558 = 0, $3559 = 0, $356 = 0, $3560 = 0, $3561 = 0, $3562 = 0, $3563 = 0, $3564 = 0, $3565 = 0, $3566 = 0, $3567 = 0, $3568 = 0, $3569 = 0, $357 = 0, $3570 = 0, $3571 = 0;
 var $3572 = 0, $3573 = 0, $3574 = 0, $3575 = 0, $3576 = 0, $3577 = 0, $3578 = 0, $3579 = 0, $358 = 0, $3580 = 0, $3581 = 0, $3582 = 0, $3583 = 0, $3584 = 0, $3585 = 0, $3586 = 0, $3587 = 0, $3588 = 0, $3589 = 0, $359 = 0;
 var $3590 = 0, $3591 = 0, $3592 = 0, $3593 = 0, $3594 = 0, $3595 = 0, $3596 = 0, $3597 = 0, $3598 = 0, $3599 = 0, $36 = 0, $360 = 0, $3600 = 0, $3601 = 0, $3602 = 0, $3603 = 0, $3604 = 0, $3605 = 0, $3606 = 0, $3607 = 0;
 var $3608 = 0, $3609 = 0, $361 = 0, $3610 = 0, $3611 = 0, $3612 = 0, $3613 = 0, $3614 = 0, $3615 = 0, $3616 = 0, $3617 = 0, $3618 = 0, $3619 = 0, $362 = 0, $3620 = 0, $3621 = 0, $3622 = 0, $3623 = 0, $3624 = 0, $3625 = 0;
 var $3626 = 0, $3627 = 0, $3628 = 0, $3629 = 0, $363 = 0, $3630 = 0, $3631 = 0, $3632 = 0, $3633 = 0, $3634 = 0, $3635 = 0, $3636 = 0, $3637 = 0, $3638 = 0, $3639 = 0, $364 = 0, $3640 = 0, $3641 = 0, $3642 = 0, $3643 = 0;
 var $3644 = 0, $3645 = 0, $3646 = 0, $3647 = 0, $3648 = 0, $3649 = 0, $365 = 0, $3650 = 0, $3651 = 0, $3652 = 0, $3653 = 0, $3654 = 0, $3655 = 0, $3656 = 0, $3657 = 0, $3658 = 0, $3659 = 0, $366 = 0, $3660 = 0, $3661 = 0;
 var $3662 = 0, $3663 = 0, $3664 = 0, $3665 = 0, $3666 = 0, $3667 = 0, $3668 = 0, $3669 = 0, $367 = 0, $3670 = 0, $3671 = 0, $3672 = 0, $3673 = 0, $3674 = 0, $3675 = 0, $3676 = 0, $3677 = 0, $3678 = 0, $3679 = 0, $368 = 0;
 var $3680 = 0, $3681 = 0, $3682 = 0, $3683 = 0, $3684 = 0, $3685 = 0, $3686 = 0, $3687 = 0, $3688 = 0, $3689 = 0, $369 = 0, $3690 = 0, $3691 = 0, $3692 = 0, $3693 = 0, $3694 = 0, $3695 = 0, $3696 = 0, $3697 = 0, $3698 = 0;
 var $3699 = 0, $37 = 0, $370 = 0, $3700 = 0, $3701 = 0, $3702 = 0, $3703 = 0, $3704 = 0, $3705 = 0, $3706 = 0, $3707 = 0, $3708 = 0, $3709 = 0, $371 = 0, $3710 = 0, $3711 = 0, $3712 = 0, $3713 = 0, $3714 = 0, $3715 = 0;
 var $3716 = 0, $3717 = 0, $3718 = 0, $3719 = 0, $372 = 0, $3720 = 0, $3721 = 0, $3722 = 0, $3723 = 0, $3724 = 0, $3725 = 0, $3726 = 0, $3727 = 0, $3728 = 0, $3729 = 0, $373 = 0, $3730 = 0, $3731 = 0, $3732 = 0, $3733 = 0;
 var $3734 = 0, $3735 = 0, $3736 = 0, $3737 = 0, $3738 = 0, $3739 = 0, $374 = 0, $3740 = 0, $3741 = 0, $3742 = 0, $3743 = 0, $3744 = 0, $3745 = 0, $3746 = 0, $3747 = 0, $3748 = 0, $3749 = 0, $375 = 0, $3750 = 0, $3751 = 0;
 var $3752 = 0, $3753 = 0, $3754 = 0, $3755 = 0, $3756 = 0, $3757 = 0, $3758 = 0, $3759 = 0, $376 = 0, $3760 = 0, $3761 = 0, $3762 = 0, $3763 = 0, $3764 = 0, $3765 = 0, $3766 = 0, $3767 = 0, $3768 = 0, $3769 = 0, $377 = 0;
 var $3770 = 0, $3771 = 0, $3772 = 0, $3773 = 0, $3774 = 0, $3775 = 0, $3776 = 0, $3777 = 0, $3778 = 0, $3779 = 0, $378 = 0, $3780 = 0, $3781 = 0, $3782 = 0, $3783 = 0, $3784 = 0, $3785 = 0, $3786 = 0, $3787 = 0, $3788 = 0;
 var $3789 = 0, $379 = 0, $3790 = 0, $3791 = 0, $3792 = 0, $3793 = 0, $3794 = 0, $3795 = 0, $3796 = 0, $3797 = 0, $3798 = 0, $3799 = 0, $38 = 0, $380 = 0, $3800 = 0, $3801 = 0, $3802 = 0, $3803 = 0, $3804 = 0, $3805 = 0;
 var $3806 = 0, $3807 = 0, $3808 = 0, $3809 = 0, $381 = 0, $3810 = 0, $3811 = 0, $3812 = 0, $3813 = 0, $3814 = 0, $3815 = 0, $3816 = 0, $3817 = 0, $3818 = 0, $3819 = 0, $382 = 0, $3820 = 0, $3821 = 0, $3822 = 0, $3823 = 0;
 var $3824 = 0, $3825 = 0, $3826 = 0, $3827 = 0, $3828 = 0, $3829 = 0, $383 = 0, $3830 = 0, $3831 = 0, $3832 = 0, $3833 = 0, $3834 = 0, $3835 = 0, $3836 = 0, $3837 = 0, $3838 = 0, $3839 = 0, $384 = 0, $3840 = 0, $3841 = 0;
 var $3842 = 0, $3843 = 0, $3844 = 0, $3845 = 0, $3846 = 0, $3847 = 0, $3848 = 0, $3849 = 0, $385 = 0, $3850 = 0, $3851 = 0, $3852 = 0, $3853 = 0, $3854 = 0, $3855 = 0, $3856 = 0, $3857 = 0, $3858 = 0, $3859 = 0, $386 = 0;
 var $3860 = 0, $3861 = 0, $3862 = 0, $3863 = 0, $3864 = 0, $3865 = 0, $3866 = 0, $3867 = 0, $3868 = 0, $3869 = 0, $387 = 0, $3870 = 0, $3871 = 0, $3872 = 0, $3873 = 0, $3874 = 0, $3875 = 0, $3876 = 0, $3877 = 0, $3878 = 0;
 var $3879 = 0, $388 = 0, $3880 = 0, $3881 = 0, $3882 = 0, $3883 = 0, $3884 = 0, $3885 = 0, $3886 = 0, $3887 = 0, $3888 = 0, $3889 = 0, $389 = 0, $3890 = 0, $3891 = 0, $3892 = 0, $3893 = 0, $3894 = 0, $3895 = 0, $3896 = 0;
 var $3897 = 0, $3898 = 0, $3899 = 0, $39 = 0, $390 = 0, $3900 = 0, $3901 = 0, $3902 = 0, $3903 = 0, $3904 = 0, $3905 = 0, $3906 = 0, $3907 = 0, $3908 = 0, $3909 = 0, $391 = 0, $3910 = 0, $3911 = 0, $3912 = 0, $3913 = 0;
 var $3914 = 0, $3915 = 0, $3916 = 0, $3917 = 0, $3918 = 0, $3919 = 0, $392 = 0, $3920 = 0, $3921 = 0, $3922 = 0, $3923 = 0, $3924 = 0, $3925 = 0, $3926 = 0, $3927 = 0, $3928 = 0, $3929 = 0, $393 = 0, $3930 = 0, $3931 = 0;
 var $3932 = 0, $3933 = 0, $3934 = 0, $3935 = 0, $3936 = 0, $3937 = 0, $3938 = 0, $3939 = 0, $394 = 0, $3940 = 0, $3941 = 0, $3942 = 0, $3943 = 0.0, $3944 = 0, $3945 = 0, $3946 = 0, $3947 = 0, $3948 = 0, $3949 = 0, $395 = 0;
 var $3950 = 0.0, $3951 = 0, $3952 = 0, $3953 = 0, $3954 = 0, $3955 = 0, $3956 = 0, $3957 = 0, $3958 = 0.0, $3959 = 0, $396 = 0, $3960 = 0, $3961 = 0, $3962 = 0, $3963 = 0, $3964 = 0, $3965 = 0.0, $3966 = 0, $3967 = 0, $3968 = 0;
 var $3969 = 0, $397 = 0, $3970 = 0, $3971 = 0, $3972 = 0, $3973 = 0, $3974 = 0, $3975 = 0, $3976 = 0, $3977 = 0, $3978 = 0, $3979 = 0, $398 = 0, $3980 = 0, $3981 = 0, $3982 = 0, $3983 = 0, $3984 = 0, $3985 = 0, $3986 = 0;
 var $3987 = 0, $3988 = 0, $3989 = 0, $399 = 0, $3990 = 0, $3991 = 0, $3992 = 0, $3993 = 0, $3994 = 0, $3995 = 0, $3996 = 0, $3997 = 0, $3998 = 0, $3999 = 0, $4 = 0, $40 = 0, $400 = 0, $4000 = 0, $4001 = 0, $4002 = 0;
 var $4003 = 0, $4004 = 0, $4005 = 0, $4006 = 0, $4007 = 0, $4008 = 0, $4009 = 0, $401 = 0, $4010 = 0, $4011 = 0, $4012 = 0, $4013 = 0, $4014 = 0, $4015 = 0, $4016 = 0, $4017 = 0, $4018 = 0, $4019 = 0, $402 = 0, $4020 = 0;
 var $4021 = 0, $4022 = 0, $4023 = 0, $4024 = 0, $4025 = 0, $4026 = 0, $4027 = 0, $4028 = 0, $4029 = 0, $403 = 0, $4030 = 0, $4031 = 0, $4032 = 0, $4033 = 0, $4034 = 0, $4035 = 0, $4036 = 0, $4037 = 0, $4038 = 0, $4039 = 0;
 var $404 = 0, $4040 = 0, $4041 = 0, $4042 = 0, $4043 = 0, $4044 = 0, $4045 = 0, $4046 = 0, $4047 = 0, $4048 = 0, $4049 = 0, $405 = 0, $4050 = 0, $4051 = 0, $4052 = 0, $4053 = 0, $4054 = 0, $4055 = 0, $4056 = 0, $4057 = 0;
 var $4058 = 0, $4059 = 0, $406 = 0, $4060 = 0, $4061 = 0, $4062 = 0, $4063 = 0, $4064 = 0, $4065 = 0, $4066 = 0, $4067 = 0, $4068 = 0, $4069 = 0, $407 = 0, $4070 = 0, $4071 = 0, $4072 = 0, $4073 = 0, $4074 = 0, $4075 = 0;
 var $4076 = 0, $4077 = 0, $4078 = 0, $4079 = 0, $408 = 0, $4080 = 0, $4081 = 0, $4082 = 0, $4083 = 0, $4084 = 0, $4085 = 0, $4086 = 0, $4087 = 0, $4088 = 0, $4089 = 0, $409 = 0, $4090 = 0, $4091 = 0, $4092 = 0, $4093 = 0;
 var $4094 = 0, $4095 = 0, $4096 = 0, $4097 = 0, $4098 = 0, $4099 = 0, $41 = 0, $410 = 0, $4100 = 0, $4101 = 0, $4102 = 0, $4103 = 0, $4104 = 0, $4105 = 0, $4106 = 0, $4107 = 0, $4108 = 0, $4109 = 0, $411 = 0, $4110 = 0;
 var $4111 = 0, $4112 = 0, $4113 = 0, $4114 = 0, $4115 = 0, $4116 = 0, $4117 = 0, $4118 = 0, $4119 = 0, $412 = 0, $4120 = 0, $4121 = 0, $4122 = 0, $4123 = 0, $4124 = 0, $4125 = 0, $4126 = 0, $4127 = 0, $4128 = 0, $4129 = 0;
 var $413 = 0.0, $4130 = 0, $4131 = 0, $4132 = 0, $4133 = 0, $4134 = 0, $4135 = 0, $4136 = 0, $4137 = 0, $4138 = 0, $4139 = 0, $414 = 0, $4140 = 0, $4141 = 0, $4142 = 0, $4143 = 0, $4144 = 0, $4145 = 0, $4146 = 0, $4147 = 0;
 var $4148 = 0, $4149 = 0, $415 = 0, $4150 = 0, $4151 = 0, $4152 = 0, $4153 = 0, $4154 = 0, $4155 = 0, $4156 = 0, $4157 = 0, $4158 = 0, $4159 = 0, $416 = 0, $4160 = 0, $4161 = 0, $4162 = 0, $4163 = 0, $4164 = 0, $4165 = 0;
 var $4166 = 0, $4167 = 0, $4168 = 0, $4169 = 0, $417 = 0, $4170 = 0, $4171 = 0, $4172 = 0, $4173 = 0, $4174 = 0, $4175 = 0, $4176 = 0, $4177 = 0, $4178 = 0, $4179 = 0, $418 = 0, $4180 = 0, $4181 = 0, $4182 = 0, $4183 = 0;
 var $4184 = 0, $4185 = 0, $4186 = 0, $4187 = 0, $4188 = 0, $4189 = 0, $419 = 0, $4190 = 0, $4191 = 0, $4192 = 0, $4193 = 0, $4194 = 0, $4195 = 0, $4196 = 0, $4197 = 0, $4198 = 0, $4199 = 0, $42 = 0, $420 = 0, $4200 = 0;
 var $4201 = 0, $4202 = 0, $4203 = 0, $4204 = 0, $4205 = 0, $4206 = 0, $4207 = 0, $4208 = 0, $4209 = 0, $421 = 0, $4210 = 0, $4211 = 0, $4212 = 0, $4213 = 0, $4214 = 0, $4215 = 0, $4216 = 0, $4217 = 0, $4218 = 0, $4219 = 0;
 var $422 = 0, $4220 = 0, $4221 = 0, $4222 = 0, $4223 = 0, $4224 = 0, $4225 = 0, $4226 = 0, $4227 = 0, $4228 = 0, $4229 = 0, $423 = 0, $4230 = 0, $4231 = 0, $4232 = 0, $4233 = 0, $4234 = 0, $4235 = 0, $4236 = 0, $4237 = 0;
 var $4238 = 0, $4239 = 0, $424 = 0, $4240 = 0, $4241 = 0, $4242 = 0, $4243 = 0, $4244 = 0, $4245 = 0, $4246 = 0, $4247 = 0, $4248 = 0, $4249 = 0, $425 = 0, $4250 = 0, $4251 = 0, $4252 = 0, $4253 = 0, $4254 = 0, $4255 = 0;
 var $4256 = 0, $4257 = 0, $4258 = 0, $4259 = 0, $426 = 0, $4260 = 0, $4261 = 0, $4262 = 0, $4263 = 0, $4264 = 0, $4265 = 0, $4266 = 0, $4267 = 0, $4268 = 0, $4269 = 0, $427 = 0, $4270 = 0, $4271 = 0, $4272 = 0, $4273 = 0;
 var $4274 = 0, $4275 = 0, $4276 = 0, $4277 = 0, $4278 = 0, $4279 = 0, $428 = 0, $4280 = 0, $4281 = 0, $4282 = 0, $4283 = 0, $4284 = 0, $4285 = 0, $4286 = 0, $4287 = 0, $4288 = 0, $4289 = 0, $429 = 0, $4290 = 0, $4291 = 0;
 var $4292 = 0, $4293 = 0, $4294 = 0, $4295 = 0, $4296 = 0, $4297 = 0, $4298 = 0, $4299 = 0, $43 = 0, $430 = 0, $4300 = 0, $4301 = 0, $4302 = 0, $4303 = 0, $4304 = 0, $4305 = 0, $4306 = 0, $4307 = 0, $4308 = 0, $4309 = 0;
 var $431 = 0, $4310 = 0, $4311 = 0, $4312 = 0, $4313 = 0, $4314 = 0, $4315 = 0, $4316 = 0, $4317 = 0, $4318 = 0, $4319 = 0, $432 = 0, $4320 = 0, $4321 = 0, $4322 = 0, $4323 = 0, $4324 = 0, $4325 = 0, $4326 = 0, $4327 = 0;
 var $4328 = 0, $4329 = 0, $433 = 0, $4330 = 0, $4331 = 0, $4332 = 0, $4333 = 0, $4334 = 0, $4335 = 0, $4336 = 0, $4337 = 0, $4338 = 0, $4339 = 0, $434 = 0, $4340 = 0, $4341 = 0, $4342 = 0, $4343 = 0, $4344 = 0, $4345 = 0;
 var $4346 = 0, $4347 = 0, $4348 = 0, $4349 = 0, $435 = 0, $4350 = 0, $4351 = 0, $4352 = 0, $4353 = 0, $4354 = 0, $4355 = 0, $4356 = 0, $4357 = 0, $4358 = 0, $4359 = 0, $436 = 0, $4360 = 0, $4361 = 0, $4362 = 0, $4363 = 0;
 var $4364 = 0, $4365 = 0, $4366 = 0, $4367 = 0, $4368 = 0, $4369 = 0, $437 = 0, $4370 = 0, $4371 = 0, $4372 = 0, $4373 = 0, $4374 = 0, $4375 = 0, $4376 = 0, $4377 = 0, $4378 = 0, $4379 = 0, $438 = 0, $4380 = 0, $4381 = 0;
 var $4382 = 0, $4383 = 0, $4384 = 0, $4385 = 0, $4386 = 0, $4387 = 0, $4388 = 0, $4389 = 0, $439 = 0, $4390 = 0, $4391 = 0, $4392 = 0, $4393 = 0, $4394 = 0, $4395 = 0, $4396 = 0, $4397 = 0, $4398 = 0, $4399 = 0, $44 = 0;
 var $440 = 0, $4400 = 0, $4401 = 0, $4402 = 0, $4403 = 0, $4404 = 0, $4405 = 0, $4406 = 0, $4407 = 0, $4408 = 0, $4409 = 0, $441 = 0, $4410 = 0, $4411 = 0, $4412 = 0, $4413 = 0, $4414 = 0, $4415 = 0, $4416 = 0, $4417 = 0;
 var $4418 = 0, $4419 = 0, $442 = 0, $4420 = 0, $4421 = 0, $4422 = 0, $4423 = 0, $4424 = 0, $4425 = 0, $4426 = 0, $4427 = 0, $4428 = 0, $4429 = 0, $443 = 0, $4430 = 0, $4431 = 0, $4432 = 0, $4433 = 0, $4434 = 0, $4435 = 0;
 var $4436 = 0, $4437 = 0, $4438 = 0, $4439 = 0, $444 = 0, $4440 = 0, $4441 = 0, $4442 = 0, $4443 = 0, $4444 = 0, $4445 = 0, $4446 = 0, $4447 = 0, $4448 = 0, $4449 = 0, $445 = 0, $4450 = 0, $4451 = 0, $4452 = 0, $4453 = 0;
 var $4454 = 0, $4455 = 0, $4456 = 0, $4457 = 0, $4458 = 0, $4459 = 0, $446 = 0, $4460 = 0, $4461 = 0, $4462 = 0, $4463 = 0, $4464 = 0, $4465 = 0, $4466 = 0, $4467 = 0, $4468 = 0, $4469 = 0, $447 = 0, $4470 = 0, $4471 = 0;
 var $4472 = 0, $4473 = 0, $4474 = 0, $4475 = 0, $4476 = 0, $4477 = 0, $4478 = 0, $4479 = 0, $448 = 0, $4480 = 0, $4481 = 0, $4482 = 0, $4483 = 0, $4484 = 0, $4485 = 0, $4486 = 0, $4487 = 0, $4488 = 0, $4489 = 0, $449 = 0;
 var $4490 = 0, $4491 = 0, $4492 = 0, $4493 = 0, $4494 = 0, $4495 = 0, $4496 = 0, $4497 = 0, $4498 = 0, $4499 = 0, $45 = 0, $450 = 0, $4500 = 0, $4501 = 0, $4502 = 0, $4503 = 0, $4504 = 0, $4505 = 0, $4506 = 0, $4507 = 0;
 var $4508 = 0, $4509 = 0, $451 = 0, $4510 = 0, $4511 = 0, $4512 = 0, $4513 = 0, $4514 = 0, $4515 = 0, $4516 = 0, $4517 = 0, $4518 = 0, $4519 = 0, $452 = 0, $4520 = 0, $4521 = 0, $4522 = 0, $4523 = 0, $4524 = 0, $4525 = 0;
 var $4526 = 0, $4527 = 0, $4528 = 0, $4529 = 0, $453 = 0, $4530 = 0, $4531 = 0, $4532 = 0, $4533 = 0, $4534 = 0, $4535 = 0, $4536 = 0, $4537 = 0, $4538 = 0, $4539 = 0, $454 = 0, $4540 = 0, $4541 = 0, $4542 = 0, $4543 = 0;
 var $4544 = 0, $4545 = 0, $4546 = 0, $4547 = 0, $4548 = 0, $4549 = 0, $455 = 0, $4550 = 0, $4551 = 0, $4552 = 0, $4553 = 0, $4554 = 0, $4555 = 0, $4556 = 0, $4557 = 0, $4558 = 0, $4559 = 0, $456 = 0, $4560 = 0, $4561 = 0;
 var $4562 = 0, $4563 = 0, $4564 = 0, $4565 = 0, $4566 = 0, $4567 = 0, $4568 = 0, $4569 = 0, $457 = 0, $4570 = 0, $4571 = 0, $4572 = 0, $4573 = 0, $4574 = 0, $4575 = 0, $4576 = 0, $4577 = 0, $4578 = 0, $4579 = 0, $458 = 0;
 var $4580 = 0, $4581 = 0, $4582 = 0, $4583 = 0, $4584 = 0, $4585 = 0, $4586 = 0, $4587 = 0, $4588 = 0, $4589 = 0, $459 = 0, $4590 = 0, $4591 = 0, $4592 = 0, $4593 = 0, $4594 = 0, $4595 = 0, $4596 = 0, $4597 = 0, $4598 = 0;
 var $4599 = 0, $46 = 0, $460 = 0, $4600 = 0, $4601 = 0, $4602 = 0, $4603 = 0, $4604 = 0, $4605 = 0, $4606 = 0, $4607 = 0, $4608 = 0, $4609 = 0, $461 = 0, $4610 = 0, $4611 = 0, $4612 = 0, $4613 = 0, $4614 = 0, $4615 = 0;
 var $4616 = 0, $4617 = 0, $4618 = 0, $4619 = 0, $462 = 0, $4620 = 0, $4621 = 0, $4622 = 0, $4623 = 0, $4624 = 0, $4625 = 0, $4626 = 0, $4627 = 0, $4628 = 0, $4629 = 0, $463 = 0, $4630 = 0, $4631 = 0, $4632 = 0, $4633 = 0;
 var $4634 = 0, $4635 = 0, $4636 = 0, $4637 = 0, $4638 = 0, $4639 = 0, $464 = 0, $4640 = 0, $4641 = 0, $4642 = 0, $4643 = 0, $4644 = 0, $4645 = 0, $4646 = 0, $4647 = 0, $4648 = 0, $4649 = 0, $465 = 0, $4650 = 0, $4651 = 0;
 var $4652 = 0, $4653 = 0, $4654 = 0, $4655 = 0, $4656 = 0, $4657 = 0, $4658 = 0, $4659 = 0, $466 = 0, $4660 = 0, $4661 = 0, $4662 = 0, $4663 = 0, $4664 = 0, $4665 = 0, $4666 = 0, $4667 = 0, $4668 = 0, $4669 = 0, $467 = 0;
 var $4670 = 0, $4671 = 0, $4672 = 0, $4673 = 0, $4674 = 0, $4675 = 0, $4676 = 0, $4677 = 0, $4678 = 0, $4679 = 0, $468 = 0, $4680 = 0, $4681 = 0, $4682 = 0, $4683 = 0, $4684 = 0, $4685 = 0, $4686 = 0, $4687 = 0, $4688 = 0;
 var $4689 = 0, $469 = 0, $4690 = 0, $4691 = 0, $4692 = 0, $4693 = 0, $4694 = 0, $4695 = 0, $4696 = 0, $4697 = 0, $4698 = 0, $4699 = 0, $47 = 0, $470 = 0, $4700 = 0, $4701 = 0, $4702 = 0, $4703 = 0, $4704 = 0, $4705 = 0;
 var $4706 = 0, $4707 = 0, $4708 = 0, $4709 = 0, $471 = 0, $4710 = 0, $4711 = 0, $4712 = 0, $4713 = 0, $4714 = 0, $4715 = 0, $4716 = 0, $4717 = 0, $4718 = 0, $4719 = 0, $472 = 0, $4720 = 0, $4721 = 0, $4722 = 0, $4723 = 0;
 var $4724 = 0, $4725 = 0, $4726 = 0, $4727 = 0, $4728 = 0, $4729 = 0, $473 = 0, $4730 = 0, $4731 = 0, $4732 = 0, $4733 = 0, $4734 = 0, $4735 = 0, $4736 = 0, $4737 = 0, $4738 = 0, $4739 = 0, $474 = 0, $4740 = 0, $4741 = 0;
 var $4742 = 0, $4743 = 0, $4744 = 0, $4745 = 0, $4746 = 0, $4747 = 0, $4748 = 0, $4749 = 0, $475 = 0, $4750 = 0, $4751 = 0, $4752 = 0, $4753 = 0, $4754 = 0, $4755 = 0, $4756 = 0, $4757 = 0, $4758 = 0, $4759 = 0, $476 = 0;
 var $4760 = 0, $4761 = 0, $4762 = 0, $4763 = 0, $4764 = 0, $4765 = 0, $4766 = 0, $4767 = 0, $4768 = 0, $4769 = 0, $477 = 0, $4770 = 0, $4771 = 0, $4772 = 0, $4773 = 0, $4774 = 0, $4775 = 0, $4776 = 0, $4777 = 0, $4778 = 0;
 var $4779 = 0, $478 = 0, $4780 = 0, $4781 = 0, $4782 = 0, $4783 = 0, $4784 = 0, $4785 = 0, $4786 = 0, $4787 = 0, $4788 = 0, $4789 = 0, $479 = 0, $4790 = 0, $4791 = 0, $4792 = 0, $4793 = 0, $4794 = 0, $4795 = 0, $4796 = 0;
 var $4797 = 0, $4798 = 0, $4799 = 0, $48 = 0, $480 = 0, $4800 = 0, $4801 = 0, $4802 = 0, $4803 = 0, $4804 = 0, $4805 = 0, $4806 = 0, $4807 = 0, $4808 = 0, $4809 = 0, $481 = 0, $4810 = 0, $4811 = 0, $4812 = 0, $4813 = 0;
 var $4814 = 0, $4815 = 0, $4816 = 0, $4817 = 0, $4818 = 0, $4819 = 0, $482 = 0, $4820 = 0, $4821 = 0, $4822 = 0, $4823 = 0, $4824 = 0, $4825 = 0, $4826 = 0, $4827 = 0, $4828 = 0, $4829 = 0, $483 = 0, $4830 = 0, $4831 = 0;
 var $4832 = 0, $4833 = 0, $4834 = 0, $4835 = 0, $4836 = 0, $4837 = 0, $4838 = 0, $4839 = 0, $484 = 0, $4840 = 0, $4841 = 0, $4842 = 0, $4843 = 0, $4844 = 0, $4845 = 0, $4846 = 0, $4847 = 0, $4848 = 0, $4849 = 0, $485 = 0;
 var $4850 = 0, $4851 = 0, $4852 = 0, $4853 = 0, $4854 = 0, $4855 = 0, $4856 = 0, $4857 = 0, $4858 = 0, $4859 = 0, $486 = 0, $4860 = 0, $4861 = 0, $4862 = 0, $4863 = 0, $4864 = 0, $4865 = 0, $4866 = 0, $4867 = 0, $4868 = 0;
 var $4869 = 0, $487 = 0, $4870 = 0, $4871 = 0, $4872 = 0, $4873 = 0, $4874 = 0, $4875 = 0, $4876 = 0, $4877 = 0, $4878 = 0, $4879 = 0, $488 = 0, $4880 = 0, $4881 = 0, $4882 = 0, $4883 = 0, $4884 = 0, $4885 = 0, $4886 = 0;
 var $4887 = 0, $4888 = 0, $4889 = 0, $489 = 0, $4890 = 0, $4891 = 0, $4892 = 0, $4893 = 0, $4894 = 0, $4895 = 0, $4896 = 0, $4897 = 0, $4898 = 0, $4899 = 0, $49 = 0, $490 = 0, $4900 = 0, $4901 = 0, $4902 = 0, $4903 = 0;
 var $4904 = 0, $4905 = 0, $4906 = 0, $4907 = 0, $4908 = 0, $4909 = 0, $491 = 0, $4910 = 0, $4911 = 0, $4912 = 0, $4913 = 0, $4914 = 0, $4915 = 0, $4916 = 0, $4917 = 0, $4918 = 0, $4919 = 0, $492 = 0, $4920 = 0, $4921 = 0;
 var $4922 = 0, $4923 = 0, $4924 = 0, $4925 = 0, $4926 = 0, $4927 = 0, $4928 = 0, $4929 = 0, $493 = 0, $4930 = 0, $4931 = 0, $4932 = 0, $4933 = 0, $4934 = 0, $4935 = 0, $4936 = 0, $4937 = 0, $4938 = 0, $4939 = 0, $494 = 0;
 var $4940 = 0, $4941 = 0, $4942 = 0, $4943 = 0, $4944 = 0, $4945 = 0, $4946 = 0, $4947 = 0, $4948 = 0, $4949 = 0, $495 = 0, $4950 = 0, $4951 = 0, $4952 = 0, $4953 = 0, $4954 = 0, $4955 = 0, $4956 = 0, $4957 = 0, $4958 = 0;
 var $4959 = 0, $496 = 0, $4960 = 0, $4961 = 0, $4962 = 0, $4963 = 0, $4964 = 0, $4965 = 0, $4966 = 0, $4967 = 0, $4968 = 0, $4969 = 0, $497 = 0, $4970 = 0, $4971 = 0, $4972 = 0, $4973 = 0, $4974 = 0, $4975 = 0, $4976 = 0;
 var $4977 = 0, $4978 = 0, $4979 = 0, $498 = 0, $4980 = 0, $4981 = 0, $4982 = 0, $4983 = 0, $4984 = 0, $4985 = 0, $4986 = 0, $4987 = 0, $4988 = 0, $4989 = 0, $499 = 0, $4990 = 0, $4991 = 0, $4992 = 0, $4993 = 0, $4994 = 0;
 var $4995 = 0, $4996 = 0, $4997 = 0, $4998 = 0, $4999 = 0, $5 = 0, $50 = 0, $500 = 0, $5000 = 0, $5001 = 0, $5002 = 0, $5003 = 0, $5004 = 0, $5005 = 0, $5006 = 0, $5007 = 0, $5008 = 0, $5009 = 0, $501 = 0, $5010 = 0;
 var $5011 = 0, $5012 = 0, $5013 = 0, $5014 = 0, $5015 = 0, $5016 = 0, $5017 = 0, $5018 = 0, $5019 = 0, $502 = 0, $5020 = 0, $5021 = 0, $5022 = 0, $5023 = 0, $5024 = 0, $5025 = 0, $5026 = 0, $5027 = 0, $5028 = 0, $5029 = 0;
 var $503 = 0, $5030 = 0, $5031 = 0, $5032 = 0, $5033 = 0, $5034 = 0, $5035 = 0, $5036 = 0, $5037 = 0, $5038 = 0, $5039 = 0, $504 = 0, $5040 = 0, $5041 = 0, $5042 = 0, $5043 = 0, $5044 = 0, $5045 = 0, $5046 = 0, $5047 = 0;
 var $5048 = 0, $5049 = 0, $505 = 0, $5050 = 0, $5051 = 0, $5052 = 0, $5053 = 0, $5054 = 0, $5055 = 0, $5056 = 0, $5057 = 0, $5058 = 0, $5059 = 0, $506 = 0, $5060 = 0, $5061 = 0, $5062 = 0, $5063 = 0, $5064 = 0, $5065 = 0;
 var $5066 = 0, $5067 = 0, $5068 = 0, $5069 = 0, $507 = 0, $5070 = 0, $5071 = 0, $5072 = 0, $5073 = 0, $5074 = 0, $5075 = 0, $5076 = 0, $5077 = 0, $5078 = 0, $5079 = 0, $508 = 0, $5080 = 0, $5081 = 0, $5082 = 0, $5083 = 0;
 var $5084 = 0, $5085 = 0, $5086 = 0, $5087 = 0, $5088 = 0, $5089 = 0, $509 = 0, $5090 = 0, $5091 = 0, $5092 = 0, $5093 = 0, $5094 = 0, $5095 = 0, $5096 = 0, $5097 = 0, $5098 = 0, $5099 = 0, $51 = 0, $510 = 0, $5100 = 0;
 var $5101 = 0, $5102 = 0, $5103 = 0, $5104 = 0, $5105 = 0, $5106 = 0, $5107 = 0, $5108 = 0, $5109 = 0, $511 = 0, $5110 = 0, $5111 = 0, $5112 = 0, $5113 = 0, $5114 = 0, $5115 = 0, $5116 = 0, $5117 = 0, $5118 = 0, $5119 = 0;
 var $512 = 0, $5120 = 0, $5121 = 0, $5122 = 0, $5123 = 0, $5124 = 0, $5125 = 0, $5126 = 0, $5127 = 0, $5128 = 0, $5129 = 0, $513 = 0, $5130 = 0, $5131 = 0, $5132 = 0, $5133 = 0, $5134 = 0, $5135 = 0, $5136 = 0, $5137 = 0;
 var $5138 = 0, $5139 = 0, $514 = 0, $5140 = 0, $5141 = 0, $5142 = 0, $5143 = 0, $5144 = 0, $5145 = 0, $5146 = 0, $5147 = 0, $5148 = 0, $5149 = 0, $515 = 0, $5150 = 0, $5151 = 0, $5152 = 0, $5153 = 0, $5154 = 0, $5155 = 0;
 var $5156 = 0, $5157 = 0, $5158 = 0, $5159 = 0, $516 = 0, $5160 = 0, $5161 = 0, $5162 = 0, $5163 = 0, $5164 = 0, $5165 = 0, $5166 = 0, $5167 = 0, $5168 = 0, $5169 = 0, $517 = 0, $5170 = 0, $5171 = 0, $5172 = 0, $5173 = 0;
 var $5174 = 0, $5175 = 0, $5176 = 0, $5177 = 0, $5178 = 0, $5179 = 0, $518 = 0, $5180 = 0, $5181 = 0, $5182 = 0, $5183 = 0, $5184 = 0, $5185 = 0, $5186 = 0, $5187 = 0, $5188 = 0, $5189 = 0, $519 = 0, $5190 = 0, $5191 = 0;
 var $5192 = 0, $5193 = 0, $5194 = 0, $5195 = 0, $5196 = 0, $5197 = 0, $5198 = 0, $5199 = 0, $52 = 0, $520 = 0, $5200 = 0, $5201 = 0, $5202 = 0, $5203 = 0, $5204 = 0, $5205 = 0, $5206 = 0, $5207 = 0, $5208 = 0, $5209 = 0;
 var $521 = 0, $5210 = 0, $5211 = 0, $5212 = 0, $5213 = 0, $5214 = 0, $5215 = 0, $5216 = 0, $5217 = 0, $5218 = 0, $5219 = 0, $522 = 0, $5220 = 0, $5221 = 0, $5222 = 0, $5223 = 0, $5224 = 0, $5225 = 0, $5226 = 0, $5227 = 0;
 var $5228 = 0, $5229 = 0, $523 = 0, $5230 = 0, $5231 = 0, $5232 = 0, $5233 = 0, $5234 = 0, $5235 = 0, $5236 = 0, $5237 = 0, $5238 = 0, $5239 = 0, $524 = 0, $5240 = 0, $5241 = 0, $5242 = 0, $5243 = 0, $5244 = 0, $5245 = 0;
 var $5246 = 0, $5247 = 0, $5248 = 0, $5249 = 0, $525 = 0, $5250 = 0, $5251 = 0, $5252 = 0, $5253 = 0, $5254 = 0, $5255 = 0, $5256 = 0, $5257 = 0, $5258 = 0, $5259 = 0, $526 = 0, $5260 = 0, $5261 = 0, $5262 = 0, $5263 = 0;
 var $5264 = 0, $5265 = 0, $5266 = 0, $5267 = 0, $5268 = 0, $5269 = 0, $527 = 0, $5270 = 0, $5271 = 0, $5272 = 0, $5273 = 0, $5274 = 0, $5275 = 0, $5276 = 0, $5277 = 0, $5278 = 0, $5279 = 0, $528 = 0, $5280 = 0, $5281 = 0;
 var $5282 = 0, $5283 = 0, $5284 = 0, $5285 = 0, $5286 = 0, $5287 = 0, $5288 = 0, $5289 = 0, $529 = 0, $5290 = 0, $5291 = 0, $5292 = 0, $5293 = 0, $5294 = 0, $5295 = 0, $5296 = 0, $5297 = 0, $5298 = 0, $5299 = 0, $53 = 0;
 var $530 = 0, $5300 = 0, $5301 = 0, $5302 = 0, $5303 = 0, $5304 = 0, $5305 = 0, $5306 = 0, $5307 = 0, $5308 = 0, $5309 = 0, $531 = 0, $5310 = 0, $5311 = 0, $5312 = 0, $5313 = 0, $5314 = 0, $5315 = 0, $5316 = 0, $5317 = 0;
 var $5318 = 0, $5319 = 0, $532 = 0, $5320 = 0, $5321 = 0, $5322 = 0, $5323 = 0, $5324 = 0, $5325 = 0, $5326 = 0, $5327 = 0, $5328 = 0, $5329 = 0, $533 = 0, $5330 = 0, $5331 = 0, $5332 = 0, $5333 = 0, $5334 = 0, $5335 = 0;
 var $5336 = 0, $5337 = 0, $5338 = 0, $5339 = 0, $534 = 0, $5340 = 0, $5341 = 0, $5342 = 0, $5343 = 0, $5344 = 0, $5345 = 0, $5346 = 0, $5347 = 0, $5348 = 0, $5349 = 0, $535 = 0, $5350 = 0, $5351 = 0, $5352 = 0, $5353 = 0;
 var $5354 = 0, $5355 = 0, $5356 = 0, $5357 = 0, $5358 = 0, $5359 = 0, $536 = 0, $5360 = 0, $5361 = 0, $5362 = 0, $5363 = 0, $5364 = 0, $5365 = 0, $5366 = 0, $5367 = 0, $5368 = 0, $5369 = 0, $537 = 0, $5370 = 0, $5371 = 0;
 var $5372 = 0, $5373 = 0, $5374 = 0, $5375 = 0, $5376 = 0, $5377 = 0, $5378 = 0, $5379 = 0, $538 = 0, $5380 = 0, $5381 = 0, $5382 = 0, $5383 = 0, $5384 = 0, $5385 = 0, $5386 = 0, $5387 = 0, $5388 = 0, $5389 = 0, $539 = 0;
 var $5390 = 0, $5391 = 0, $5392 = 0, $5393 = 0, $5394 = 0, $5395 = 0, $5396 = 0, $5397 = 0, $5398 = 0, $5399 = 0, $54 = 0, $540 = 0, $5400 = 0, $5401 = 0, $5402 = 0, $5403 = 0, $5404 = 0, $5405 = 0, $5406 = 0, $5407 = 0;
 var $5408 = 0, $5409 = 0, $541 = 0, $5410 = 0, $5411 = 0, $5412 = 0, $5413 = 0, $5414 = 0, $5415 = 0, $5416 = 0, $5417 = 0, $5418 = 0, $5419 = 0, $542 = 0, $5420 = 0, $5421 = 0, $5422 = 0, $5423 = 0, $5424 = 0, $5425 = 0;
 var $5426 = 0, $5427 = 0, $5428 = 0, $5429 = 0, $543 = 0, $5430 = 0, $5431 = 0, $5432 = 0, $5433 = 0, $5434 = 0, $5435 = 0, $5436 = 0, $5437 = 0, $5438 = 0, $5439 = 0, $544 = 0, $5440 = 0, $5441 = 0, $5442 = 0, $5443 = 0;
 var $5444 = 0, $5445 = 0, $5446 = 0, $5447 = 0, $5448 = 0, $5449 = 0, $545 = 0, $5450 = 0, $5451 = 0, $5452 = 0, $5453 = 0, $5454 = 0, $5455 = 0, $5456 = 0, $5457 = 0, $5458 = 0, $5459 = 0, $546 = 0, $5460 = 0, $5461 = 0;
 var $5462 = 0, $5463 = 0, $5464 = 0, $5465 = 0, $5466 = 0, $5467 = 0, $5468 = 0, $5469 = 0, $547 = 0, $5470 = 0, $5471 = 0, $5472 = 0, $5473 = 0, $5474 = 0, $5475 = 0, $5476 = 0, $5477 = 0, $5478 = 0, $5479 = 0, $548 = 0;
 var $5480 = 0, $5481 = 0, $5482 = 0, $5483 = 0, $5484 = 0, $5485 = 0, $5486 = 0, $5487 = 0, $5488 = 0, $5489 = 0, $549 = 0, $5490 = 0, $5491 = 0, $5492 = 0, $5493 = 0, $5494 = 0, $5495 = 0, $5496 = 0, $5497 = 0, $5498 = 0;
 var $5499 = 0, $55 = 0, $550 = 0, $5500 = 0, $5501 = 0, $5502 = 0, $5503 = 0, $5504 = 0, $5505 = 0, $5506 = 0, $5507 = 0, $5508 = 0, $5509 = 0, $551 = 0, $5510 = 0, $5511 = 0, $5512 = 0, $5513 = 0, $5514 = 0, $5515 = 0;
 var $5516 = 0, $5517 = 0, $5518 = 0, $5519 = 0, $552 = 0, $5520 = 0, $5521 = 0, $5522 = 0, $5523 = 0, $5524 = 0, $5525 = 0, $5526 = 0, $5527 = 0, $5528 = 0, $5529 = 0, $553 = 0, $5530 = 0, $5531 = 0, $5532 = 0, $5533 = 0;
 var $5534 = 0, $5535 = 0, $5536 = 0, $5537 = 0, $5538 = 0, $5539 = 0, $554 = 0, $5540 = 0, $5541 = 0, $5542 = 0, $5543 = 0, $5544 = 0, $5545 = 0, $5546 = 0, $5547 = 0, $5548 = 0, $5549 = 0, $555 = 0, $5550 = 0, $5551 = 0;
 var $5552 = 0, $5553 = 0, $5554 = 0, $5555 = 0, $5556 = 0, $5557 = 0, $5558 = 0, $5559 = 0, $556 = 0, $5560 = 0, $5561 = 0, $5562 = 0, $5563 = 0, $5564 = 0, $5565 = 0, $5566 = 0, $5567 = 0, $5568 = 0, $5569 = 0, $557 = 0;
 var $5570 = 0, $5571 = 0, $5572 = 0, $5573 = 0, $5574 = 0, $5575 = 0, $5576 = 0, $5577 = 0, $5578 = 0, $5579 = 0, $558 = 0, $5580 = 0, $5581 = 0, $5582 = 0, $5583 = 0, $5584 = 0, $5585 = 0, $5586 = 0, $5587 = 0, $5588 = 0;
 var $5589 = 0, $559 = 0, $5590 = 0, $5591 = 0, $5592 = 0, $5593 = 0, $5594 = 0, $5595 = 0, $5596 = 0, $5597 = 0, $5598 = 0, $5599 = 0, $56 = 0, $560 = 0, $5600 = 0, $5601 = 0, $5602 = 0, $5603 = 0, $5604 = 0, $5605 = 0;
 var $5606 = 0, $5607 = 0, $5608 = 0, $5609 = 0, $561 = 0, $5610 = 0, $5611 = 0, $5612 = 0, $5613 = 0, $5614 = 0, $5615 = 0, $5616 = 0, $5617 = 0, $5618 = 0, $5619 = 0, $562 = 0, $5620 = 0, $5621 = 0, $5622 = 0, $5623 = 0;
 var $5624 = 0, $5625 = 0, $5626 = 0, $5627 = 0, $5628 = 0, $5629 = 0, $563 = 0, $5630 = 0, $5631 = 0, $5632 = 0, $5633 = 0, $5634 = 0, $5635 = 0, $5636 = 0, $5637 = 0, $5638 = 0, $5639 = 0, $564 = 0, $5640 = 0, $5641 = 0;
 var $5642 = 0, $5643 = 0, $5644 = 0, $5645 = 0, $5646 = 0, $5647 = 0, $5648 = 0, $5649 = 0, $565 = 0, $5650 = 0, $5651 = 0, $5652 = 0, $5653 = 0, $5654 = 0, $5655 = 0, $5656 = 0, $5657 = 0, $5658 = 0, $5659 = 0, $566 = 0;
 var $5660 = 0, $5661 = 0, $5662 = 0, $5663 = 0, $5664 = 0, $5665 = 0, $5666 = 0, $5667 = 0, $5668 = 0, $5669 = 0, $567 = 0, $5670 = 0, $5671 = 0, $5672 = 0, $5673 = 0, $5674 = 0, $5675 = 0, $5676 = 0, $5677 = 0, $5678 = 0;
 var $5679 = 0, $568 = 0, $5680 = 0, $5681 = 0, $5682 = 0, $5683 = 0, $5684 = 0, $5685 = 0, $5686 = 0, $5687 = 0, $5688 = 0, $5689 = 0, $569 = 0, $5690 = 0, $5691 = 0, $5692 = 0, $5693 = 0, $5694 = 0, $5695 = 0, $5696 = 0;
 var $5697 = 0, $5698 = 0, $5699 = 0, $57 = 0, $570 = 0, $5700 = 0, $5701 = 0, $5702 = 0, $5703 = 0, $5704 = 0, $5705 = 0, $5706 = 0, $5707 = 0, $5708 = 0, $5709 = 0, $571 = 0, $5710 = 0, $5711 = 0, $5712 = 0, $5713 = 0;
 var $5714 = 0, $5715 = 0, $5716 = 0, $5717 = 0, $5718 = 0, $5719 = 0, $572 = 0, $5720 = 0, $5721 = 0, $5722 = 0, $5723 = 0, $5724 = 0, $5725 = 0, $5726 = 0, $5727 = 0, $5728 = 0, $5729 = 0, $573 = 0, $5730 = 0, $5731 = 0;
 var $5732 = 0, $5733 = 0, $5734 = 0, $5735 = 0, $5736 = 0, $5737 = 0, $5738 = 0, $5739 = 0, $574 = 0, $5740 = 0, $5741 = 0, $5742 = 0, $5743 = 0, $5744 = 0, $5745 = 0, $5746 = 0, $5747 = 0, $5748 = 0, $5749 = 0, $575 = 0;
 var $5750 = 0, $5751 = 0, $5752 = 0, $5753 = 0, $5754 = 0, $5755 = 0, $5756 = 0, $5757 = 0, $5758 = 0, $5759 = 0, $576 = 0, $5760 = 0, $5761 = 0, $5762 = 0, $5763 = 0, $5764 = 0, $5765 = 0, $5766 = 0, $5767 = 0, $5768 = 0;
 var $5769 = 0, $577 = 0, $5770 = 0, $5771 = 0, $5772 = 0, $5773 = 0, $5774 = 0, $5775 = 0, $5776 = 0, $5777 = 0, $5778 = 0, $5779 = 0, $578 = 0, $5780 = 0, $5781 = 0, $5782 = 0, $5783 = 0, $5784 = 0, $5785 = 0, $5786 = 0;
 var $5787 = 0, $5788 = 0, $5789 = 0, $579 = 0, $5790 = 0, $5791 = 0, $5792 = 0, $5793 = 0, $5794 = 0, $5795 = 0, $5796 = 0, $5797 = 0, $5798 = 0, $5799 = 0, $58 = 0, $580 = 0, $5800 = 0, $5801 = 0, $5802 = 0, $5803 = 0;
 var $5804 = 0, $5805 = 0, $5806 = 0, $5807 = 0, $5808 = 0, $5809 = 0, $581 = 0, $5810 = 0, $5811 = 0, $5812 = 0, $5813 = 0, $5814 = 0, $5815 = 0, $5816 = 0, $5817 = 0, $5818 = 0, $5819 = 0, $582 = 0, $5820 = 0, $5821 = 0;
 var $5822 = 0, $5823 = 0, $5824 = 0, $5825 = 0, $5826 = 0, $5827 = 0, $5828 = 0, $5829 = 0, $583 = 0, $5830 = 0, $5831 = 0, $5832 = 0, $5833 = 0, $5834 = 0, $5835 = 0, $5836 = 0, $5837 = 0, $5838 = 0, $5839 = 0, $584 = 0;
 var $5840 = 0, $5841 = 0, $5842 = 0, $5843 = 0, $5844 = 0, $5845 = 0, $5846 = 0, $5847 = 0, $5848 = 0, $5849 = 0, $585 = 0, $5850 = 0, $5851 = 0, $5852 = 0, $5853 = 0, $5854 = 0, $5855 = 0, $5856 = 0, $5857 = 0, $5858 = 0;
 var $5859 = 0, $586 = 0, $5860 = 0, $5861 = 0, $5862 = 0, $5863 = 0, $5864 = 0, $5865 = 0, $5866 = 0, $5867 = 0, $5868 = 0, $5869 = 0, $587 = 0, $5870 = 0, $5871 = 0, $5872 = 0, $5873 = 0, $5874 = 0, $5875 = 0, $5876 = 0;
 var $5877 = 0, $5878 = 0, $5879 = 0, $588 = 0, $5880 = 0, $5881 = 0, $5882 = 0, $5883 = 0, $5884 = 0, $5885 = 0, $5886 = 0, $5887 = 0, $5888 = 0, $5889 = 0, $589 = 0, $5890 = 0, $5891 = 0, $5892 = 0, $5893 = 0, $5894 = 0;
 var $5895 = 0, $5896 = 0, $5897 = 0, $5898 = 0, $5899 = 0, $59 = 0, $590 = 0, $5900 = 0, $5901 = 0, $5902 = 0, $5903 = 0, $5904 = 0, $5905 = 0, $5906 = 0, $5907 = 0, $5908 = 0, $5909 = 0, $591 = 0, $5910 = 0, $5911 = 0;
 var $5912 = 0, $5913 = 0, $5914 = 0, $5915 = 0, $5916 = 0, $5917 = 0, $5918 = 0, $5919 = 0, $592 = 0, $5920 = 0, $5921 = 0, $5922 = 0, $5923 = 0, $5924 = 0, $5925 = 0, $5926 = 0, $5927 = 0, $5928 = 0, $5929 = 0, $593 = 0;
 var $5930 = 0, $5931 = 0, $5932 = 0, $5933 = 0, $5934 = 0, $5935 = 0, $5936 = 0, $5937 = 0, $5938 = 0, $5939 = 0, $594 = 0, $5940 = 0, $5941 = 0, $5942 = 0, $5943 = 0, $5944 = 0, $5945 = 0, $5946 = 0, $5947 = 0, $5948 = 0;
 var $5949 = 0, $595 = 0, $5950 = 0, $5951 = 0, $5952 = 0, $5953 = 0, $5954 = 0, $5955 = 0, $5956 = 0, $5957 = 0, $5958 = 0, $5959 = 0, $596 = 0, $5960 = 0, $5961 = 0, $5962 = 0, $5963 = 0, $5964 = 0, $5965 = 0, $5966 = 0;
 var $5967 = 0, $5968 = 0, $5969 = 0, $597 = 0, $5970 = 0, $5971 = 0, $5972 = 0, $5973 = 0, $5974 = 0, $5975 = 0, $5976 = 0, $5977 = 0, $5978 = 0, $5979 = 0, $598 = 0, $5980 = 0, $5981 = 0, $5982 = 0, $5983 = 0, $5984 = 0;
 var $5985 = 0, $5986 = 0, $5987 = 0, $5988 = 0, $5989 = 0, $599 = 0, $5990 = 0, $5991 = 0, $5992 = 0, $5993 = 0, $5994 = 0, $5995 = 0, $5996 = 0, $5997 = 0, $5998 = 0, $5999 = 0, $6 = 0, $60 = 0, $600 = 0, $6000 = 0;
 var $6001 = 0, $6002 = 0, $6003 = 0, $6004 = 0, $6005 = 0, $6006 = 0, $6007 = 0, $6008 = 0, $6009 = 0, $601 = 0, $6010 = 0, $6011 = 0, $6012 = 0, $6013 = 0, $6014 = 0, $6015 = 0, $6016 = 0, $6017 = 0, $6018 = 0, $6019 = 0;
 var $602 = 0, $6020 = 0, $6021 = 0, $6022 = 0, $6023 = 0, $6024 = 0, $6025 = 0, $6026 = 0, $6027 = 0, $6028 = 0, $6029 = 0, $603 = 0, $6030 = 0, $6031 = 0, $6032 = 0, $6033 = 0, $6034 = 0, $6035 = 0, $6036 = 0, $6037 = 0;
 var $6038 = 0, $6039 = 0, $604 = 0, $6040 = 0, $6041 = 0, $6042 = 0, $6043 = 0, $6044 = 0, $6045 = 0, $6046 = 0, $6047 = 0, $6048 = 0, $6049 = 0, $605 = 0, $6050 = 0, $6051 = 0, $6052 = 0, $6053 = 0, $6054 = 0, $6055 = 0;
 var $6056 = 0, $6057 = 0, $6058 = 0, $6059 = 0, $606 = 0, $6060 = 0, $6061 = 0, $6062 = 0, $6063 = 0, $6064 = 0, $6065 = 0, $6066 = 0, $6067 = 0, $6068 = 0, $6069 = 0, $607 = 0, $6070 = 0, $6071 = 0, $6072 = 0, $6073 = 0;
 var $6074 = 0, $6075 = 0, $6076 = 0, $6077 = 0, $6078 = 0, $6079 = 0, $608 = 0, $6080 = 0, $6081 = 0, $6082 = 0, $6083 = 0, $6084 = 0, $6085 = 0, $6086 = 0, $6087 = 0, $6088 = 0, $6089 = 0, $609 = 0, $6090 = 0, $6091 = 0;
 var $6092 = 0, $6093 = 0, $6094 = 0, $6095 = 0, $6096 = 0, $6097 = 0, $6098 = 0, $6099 = 0, $61 = 0, $610 = 0, $6100 = 0, $6101 = 0, $6102 = 0, $6103 = 0, $6104 = 0, $6105 = 0, $6106 = 0, $6107 = 0, $6108 = 0, $6109 = 0;
 var $611 = 0, $6110 = 0, $6111 = 0, $6112 = 0, $6113 = 0, $6114 = 0, $6115 = 0, $6116 = 0, $6117 = 0, $6118 = 0, $6119 = 0, $612 = 0, $6120 = 0, $6121 = 0, $6122 = 0, $6123 = 0, $6124 = 0, $6125 = 0, $6126 = 0, $6127 = 0;
 var $6128 = 0, $6129 = 0, $613 = 0, $6130 = 0, $6131 = 0, $6132 = 0, $6133 = 0, $6134 = 0, $6135 = 0, $6136 = 0, $6137 = 0, $6138 = 0, $6139 = 0, $614 = 0, $6140 = 0, $6141 = 0, $6142 = 0, $6143 = 0, $6144 = 0, $6145 = 0;
 var $6146 = 0, $6147 = 0, $6148 = 0, $6149 = 0, $615 = 0, $6150 = 0, $6151 = 0, $6152 = 0, $6153 = 0, $6154 = 0, $6155 = 0, $6156 = 0, $6157 = 0, $6158 = 0, $6159 = 0, $616 = 0, $6160 = 0, $6161 = 0, $6162 = 0, $6163 = 0;
 var $6164 = 0, $6165 = 0, $6166 = 0, $6167 = 0, $6168 = 0, $6169 = 0, $617 = 0, $6170 = 0, $6171 = 0, $6172 = 0, $6173 = 0, $6174 = 0, $6175 = 0, $6176 = 0, $6177 = 0, $6178 = 0, $6179 = 0, $618 = 0, $6180 = 0, $6181 = 0;
 var $6182 = 0, $6183 = 0, $6184 = 0, $6185 = 0, $6186 = 0, $6187 = 0, $6188 = 0, $6189 = 0, $619 = 0, $6190 = 0, $6191 = 0, $6192 = 0, $6193 = 0, $6194 = 0, $6195 = 0, $6196 = 0, $6197 = 0, $6198 = 0, $6199 = 0, $62 = 0;
 var $620 = 0, $6200 = 0, $6201 = 0, $6202 = 0, $6203 = 0, $6204 = 0, $6205 = 0, $6206 = 0, $6207 = 0, $6208 = 0, $6209 = 0, $621 = 0, $6210 = 0, $6211 = 0, $6212 = 0, $6213 = 0, $6214 = 0, $6215 = 0, $6216 = 0, $6217 = 0;
 var $6218 = 0, $6219 = 0, $622 = 0, $6220 = 0, $6221 = 0, $6222 = 0, $6223 = 0, $6224 = 0, $6225 = 0, $6226 = 0, $6227 = 0, $6228 = 0, $6229 = 0, $623 = 0, $6230 = 0, $6231 = 0, $6232 = 0, $6233 = 0, $6234 = 0, $6235 = 0;
 var $6236 = 0, $6237 = 0, $6238 = 0, $6239 = 0, $624 = 0, $6240 = 0, $6241 = 0, $6242 = 0, $6243 = 0, $6244 = 0, $6245 = 0, $6246 = 0, $6247 = 0, $6248 = 0, $6249 = 0, $625 = 0, $6250 = 0, $6251 = 0, $6252 = 0, $6253 = 0;
 var $6254 = 0, $6255 = 0, $6256 = 0, $6257 = 0, $6258 = 0, $6259 = 0, $626 = 0, $6260 = 0, $6261 = 0, $6262 = 0, $6263 = 0, $6264 = 0, $6265 = 0, $6266 = 0, $6267 = 0, $6268 = 0, $6269 = 0, $627 = 0, $6270 = 0, $6271 = 0;
 var $6272 = 0, $6273 = 0, $6274 = 0, $6275 = 0, $6276 = 0, $6277 = 0, $6278 = 0, $6279 = 0, $628 = 0, $6280 = 0, $6281 = 0, $6282 = 0, $6283 = 0, $6284 = 0, $6285 = 0, $6286 = 0, $6287 = 0, $6288 = 0, $6289 = 0, $629 = 0;
 var $6290 = 0, $6291 = 0, $6292 = 0, $6293 = 0, $6294 = 0, $6295 = 0, $6296 = 0, $6297 = 0, $6298 = 0, $6299 = 0, $63 = 0, $630 = 0, $6300 = 0, $6301 = 0, $6302 = 0, $6303 = 0, $6304 = 0, $6305 = 0, $6306 = 0, $6307 = 0;
 var $6308 = 0, $6309 = 0, $631 = 0, $6310 = 0, $6311 = 0, $6312 = 0, $6313 = 0, $6314 = 0, $6315 = 0, $6316 = 0, $6317 = 0, $6318 = 0, $6319 = 0, $632 = 0, $6320 = 0, $6321 = 0, $6322 = 0, $6323 = 0, $6324 = 0, $6325 = 0;
 var $6326 = 0, $6327 = 0, $6328 = 0, $6329 = 0, $633 = 0, $6330 = 0, $6331 = 0, $6332 = 0, $6333 = 0, $6334 = 0, $6335 = 0, $6336 = 0, $6337 = 0, $6338 = 0, $6339 = 0, $634 = 0, $6340 = 0, $6341 = 0, $6342 = 0, $6343 = 0;
 var $6344 = 0, $6345 = 0, $6346 = 0, $6347 = 0, $6348 = 0, $6349 = 0, $635 = 0, $6350 = 0, $6351 = 0, $6352 = 0, $6353 = 0, $6354 = 0, $6355 = 0, $6356 = 0, $6357 = 0, $6358 = 0, $6359 = 0, $636 = 0, $6360 = 0, $6361 = 0;
 var $6362 = 0, $6363 = 0, $6364 = 0, $6365 = 0, $6366 = 0, $6367 = 0, $6368 = 0, $6369 = 0, $637 = 0, $6370 = 0, $6371 = 0, $6372 = 0, $6373 = 0, $6374 = 0, $6375 = 0, $6376 = 0, $6377 = 0, $6378 = 0, $6379 = 0, $638 = 0;
 var $6380 = 0, $6381 = 0, $6382 = 0, $6383 = 0, $6384 = 0, $6385 = 0, $6386 = 0, $6387 = 0, $6388 = 0, $6389 = 0, $639 = 0, $6390 = 0, $6391 = 0, $6392 = 0, $6393 = 0, $6394 = 0, $6395 = 0, $6396 = 0, $6397 = 0, $6398 = 0;
 var $6399 = 0, $64 = 0, $640 = 0, $6400 = 0, $6401 = 0, $6402 = 0, $6403 = 0, $6404 = 0, $6405 = 0, $6406 = 0, $6407 = 0, $6408 = 0, $6409 = 0, $641 = 0, $6410 = 0, $6411 = 0, $6412 = 0, $6413 = 0, $6414 = 0, $6415 = 0;
 var $6416 = 0, $6417 = 0, $6418 = 0, $6419 = 0, $642 = 0, $6420 = 0, $6421 = 0, $6422 = 0, $6423 = 0, $6424 = 0, $6425 = 0, $6426 = 0, $6427 = 0, $6428 = 0, $6429 = 0, $643 = 0, $6430 = 0, $6431 = 0, $6432 = 0, $6433 = 0;
 var $6434 = 0, $6435 = 0, $6436 = 0, $6437 = 0, $6438 = 0, $6439 = 0, $644 = 0, $6440 = 0, $6441 = 0, $6442 = 0, $6443 = 0, $6444 = 0, $6445 = 0, $6446 = 0, $6447 = 0, $6448 = 0, $6449 = 0, $645 = 0, $6450 = 0, $6451 = 0;
 var $6452 = 0, $6453 = 0, $6454 = 0, $6455 = 0, $6456 = 0, $6457 = 0, $6458 = 0, $6459 = 0, $646 = 0, $6460 = 0, $6461 = 0, $6462 = 0, $6463 = 0, $6464 = 0, $6465 = 0, $6466 = 0, $6467 = 0, $6468 = 0, $6469 = 0, $647 = 0;
 var $6470 = 0, $6471 = 0, $6472 = 0, $6473 = 0, $6474 = 0, $6475 = 0, $6476 = 0, $6477 = 0, $6478 = 0, $6479 = 0, $648 = 0, $6480 = 0, $6481 = 0, $6482 = 0, $6483 = 0, $6484 = 0, $6485 = 0, $6486 = 0, $6487 = 0, $6488 = 0;
 var $6489 = 0, $649 = 0, $6490 = 0, $6491 = 0, $6492 = 0, $6493 = 0, $6494 = 0, $6495 = 0, $6496 = 0, $6497 = 0, $6498 = 0, $6499 = 0, $65 = 0, $650 = 0, $6500 = 0, $6501 = 0, $6502 = 0, $6503 = 0, $6504 = 0, $6505 = 0;
 var $6506 = 0, $6507 = 0, $6508 = 0, $6509 = 0, $651 = 0, $6510 = 0, $6511 = 0, $6512 = 0, $6513 = 0, $6514 = 0, $6515 = 0, $6516 = 0, $6517 = 0, $6518 = 0, $6519 = 0, $652 = 0, $6520 = 0, $6521 = 0, $6522 = 0, $6523 = 0;
 var $6524 = 0, $6525 = 0, $6526 = 0, $6527 = 0, $6528 = 0, $6529 = 0, $653 = 0, $6530 = 0, $6531 = 0, $6532 = 0, $6533 = 0, $6534 = 0, $6535 = 0, $6536 = 0, $6537 = 0, $6538 = 0, $6539 = 0, $654 = 0, $6540 = 0, $6541 = 0;
 var $6542 = 0, $6543 = 0, $6544 = 0, $6545 = 0, $6546 = 0, $6547 = 0, $6548 = 0, $6549 = 0, $655 = 0, $6550 = 0, $6551 = 0, $6552 = 0, $6553 = 0, $6554 = 0, $6555 = 0, $6556 = 0, $6557 = 0, $6558 = 0, $6559 = 0, $656 = 0;
 var $6560 = 0, $6561 = 0, $6562 = 0, $6563 = 0, $6564 = 0, $6565 = 0, $6566 = 0, $6567 = 0, $6568 = 0, $6569 = 0, $657 = 0, $6570 = 0, $6571 = 0, $6572 = 0, $6573 = 0, $6574 = 0, $6575 = 0, $6576 = 0, $6577 = 0, $6578 = 0;
 var $6579 = 0, $658 = 0, $6580 = 0, $6581 = 0, $6582 = 0, $6583 = 0, $6584 = 0, $6585 = 0, $6586 = 0, $6587 = 0, $6588 = 0, $6589 = 0, $659 = 0, $6590 = 0, $6591 = 0, $6592 = 0, $6593 = 0, $6594 = 0, $6595 = 0, $6596 = 0;
 var $6597 = 0, $6598 = 0, $6599 = 0, $66 = 0, $660 = 0, $6600 = 0, $6601 = 0, $6602 = 0, $6603 = 0, $6604 = 0, $6605 = 0, $6606 = 0, $6607 = 0, $6608 = 0, $6609 = 0, $661 = 0, $6610 = 0, $6611 = 0, $6612 = 0, $6613 = 0;
 var $6614 = 0, $6615 = 0, $6616 = 0, $6617 = 0, $6618 = 0, $6619 = 0, $662 = 0, $6620 = 0, $6621 = 0, $6622 = 0, $6623 = 0, $6624 = 0, $6625 = 0, $6626 = 0, $6627 = 0, $6628 = 0, $6629 = 0, $663 = 0, $6630 = 0, $6631 = 0;
 var $6632 = 0, $6633 = 0, $6634 = 0, $6635 = 0, $6636 = 0, $6637 = 0, $6638 = 0, $6639 = 0, $664 = 0, $6640 = 0, $6641 = 0, $6642 = 0, $6643 = 0, $6644 = 0, $6645 = 0, $6646 = 0, $6647 = 0, $6648 = 0, $6649 = 0, $665 = 0;
 var $6650 = 0, $6651 = 0, $6652 = 0, $6653 = 0, $6654 = 0, $6655 = 0, $6656 = 0, $6657 = 0, $6658 = 0, $6659 = 0, $666 = 0, $6660 = 0, $6661 = 0, $6662 = 0, $6663 = 0, $6664 = 0, $6665 = 0, $6666 = 0, $6667 = 0, $6668 = 0;
 var $6669 = 0, $667 = 0, $6670 = 0, $6671 = 0, $6672 = 0, $6673 = 0, $6674 = 0, $6675 = 0, $6676 = 0, $6677 = 0, $6678 = 0, $6679 = 0, $668 = 0, $6680 = 0, $6681 = 0, $6682 = 0, $6683 = 0, $6684 = 0, $6685 = 0, $6686 = 0;
 var $6687 = 0, $6688 = 0, $6689 = 0, $669 = 0, $6690 = 0, $6691 = 0, $6692 = 0, $6693 = 0, $6694 = 0, $6695 = 0, $6696 = 0, $6697 = 0, $6698 = 0, $6699 = 0, $67 = 0, $670 = 0, $6700 = 0, $6701 = 0, $6702 = 0, $6703 = 0;
 var $6704 = 0, $6705 = 0, $6706 = 0, $6707 = 0, $6708 = 0, $6709 = 0, $671 = 0, $6710 = 0, $6711 = 0, $6712 = 0, $6713 = 0, $6714 = 0, $6715 = 0, $6716 = 0, $6717 = 0, $6718 = 0, $6719 = 0, $672 = 0, $6720 = 0, $6721 = 0;
 var $6722 = 0, $6723 = 0, $6724 = 0, $6725 = 0, $6726 = 0, $6727 = 0, $6728 = 0, $6729 = 0, $673 = 0, $6730 = 0, $6731 = 0, $6732 = 0, $6733 = 0, $6734 = 0, $6735 = 0, $6736 = 0, $6737 = 0, $6738 = 0, $6739 = 0, $674 = 0;
 var $6740 = 0, $6741 = 0, $6742 = 0, $6743 = 0, $6744 = 0, $6745 = 0, $6746 = 0, $6747 = 0, $6748 = 0, $6749 = 0, $675 = 0, $6750 = 0, $6751 = 0, $6752 = 0, $6753 = 0, $6754 = 0, $6755 = 0, $6756 = 0, $6757 = 0, $6758 = 0;
 var $6759 = 0, $676 = 0, $6760 = 0, $6761 = 0, $6762 = 0, $6763 = 0, $6764 = 0, $6765 = 0, $6766 = 0, $6767 = 0, $6768 = 0, $6769 = 0, $677 = 0, $6770 = 0, $6771 = 0, $6772 = 0, $6773 = 0, $6774 = 0, $6775 = 0, $6776 = 0;
 var $6777 = 0, $6778 = 0, $6779 = 0, $678 = 0, $6780 = 0, $6781 = 0, $6782 = 0, $6783 = 0, $6784 = 0, $6785 = 0, $6786 = 0, $6787 = 0, $6788 = 0, $6789 = 0, $679 = 0, $6790 = 0, $6791 = 0, $6792 = 0, $6793 = 0, $6794 = 0;
 var $6795 = 0, $6796 = 0, $6797 = 0, $6798 = 0, $6799 = 0, $68 = 0, $680 = 0, $6800 = 0, $6801 = 0, $6802 = 0, $6803 = 0, $6804 = 0, $6805 = 0, $6806 = 0, $6807 = 0, $6808 = 0, $6809 = 0, $681 = 0, $6810 = 0, $6811 = 0;
 var $6812 = 0, $6813 = 0, $6814 = 0, $6815 = 0, $6816 = 0, $6817 = 0, $6818 = 0, $6819 = 0, $682 = 0, $6820 = 0, $6821 = 0, $6822 = 0, $6823 = 0, $6824 = 0, $6825 = 0, $6826 = 0, $6827 = 0, $6828 = 0, $6829 = 0, $683 = 0;
 var $6830 = 0, $6831 = 0, $6832 = 0, $6833 = 0, $6834 = 0, $6835 = 0, $6836 = 0, $6837 = 0, $6838 = 0, $6839 = 0, $684 = 0, $6840 = 0, $6841 = 0, $6842 = 0, $6843 = 0, $6844 = 0, $6845 = 0, $6846 = 0, $6847 = 0, $6848 = 0;
 var $6849 = 0, $685 = 0, $6850 = 0, $6851 = 0, $6852 = 0, $6853 = 0, $6854 = 0, $6855 = 0, $6856 = 0, $6857 = 0, $6858 = 0, $6859 = 0, $686 = 0, $6860 = 0, $6861 = 0, $6862 = 0, $6863 = 0, $6864 = 0, $6865 = 0, $6866 = 0;
 var $6867 = 0, $6868 = 0, $6869 = 0, $687 = 0, $6870 = 0, $6871 = 0, $6872 = 0, $6873 = 0, $6874 = 0, $6875 = 0, $6876 = 0, $6877 = 0, $6878 = 0, $6879 = 0, $688 = 0, $6880 = 0, $6881 = 0, $6882 = 0, $6883 = 0, $6884 = 0;
 var $6885 = 0, $6886 = 0, $6887 = 0, $6888 = 0, $6889 = 0, $689 = 0, $6890 = 0, $6891 = 0, $6892 = 0, $6893 = 0, $6894 = 0, $6895 = 0, $6896 = 0, $6897 = 0, $6898 = 0, $6899 = 0, $69 = 0, $690 = 0, $6900 = 0, $6901 = 0;
 var $6902 = 0, $6903 = 0, $6904 = 0, $6905 = 0, $6906 = 0, $6907 = 0, $6908 = 0, $6909 = 0, $691 = 0, $6910 = 0, $6911 = 0, $6912 = 0, $6913 = 0, $6914 = 0, $6915 = 0, $6916 = 0, $6917 = 0, $6918 = 0, $6919 = 0, $692 = 0;
 var $6920 = 0, $6921 = 0, $6922 = 0, $6923 = 0, $6924 = 0, $6925 = 0, $6926 = 0, $6927 = 0, $6928 = 0, $6929 = 0, $693 = 0, $6930 = 0, $6931 = 0, $6932 = 0, $6933 = 0, $6934 = 0, $6935 = 0, $6936 = 0, $6937 = 0, $6938 = 0;
 var $6939 = 0, $694 = 0, $6940 = 0, $6941 = 0, $6942 = 0, $6943 = 0, $6944 = 0, $6945 = 0, $6946 = 0, $6947 = 0, $6948 = 0, $6949 = 0, $695 = 0, $6950 = 0, $6951 = 0, $6952 = 0, $6953 = 0, $6954 = 0, $6955 = 0, $6956 = 0;
 var $6957 = 0, $6958 = 0, $6959 = 0, $696 = 0, $6960 = 0, $6961 = 0, $6962 = 0, $6963 = 0, $6964 = 0, $6965 = 0, $6966 = 0, $6967 = 0, $6968 = 0, $6969 = 0, $697 = 0, $6970 = 0, $6971 = 0, $6972 = 0, $6973 = 0, $6974 = 0;
 var $6975 = 0, $6976 = 0, $6977 = 0, $6978 = 0, $6979 = 0, $698 = 0, $6980 = 0, $6981 = 0, $6982 = 0, $6983 = 0, $6984 = 0, $6985 = 0, $6986 = 0, $6987 = 0, $6988 = 0, $6989 = 0, $699 = 0, $6990 = 0, $6991 = 0, $6992 = 0;
 var $6993 = 0, $6994 = 0, $6995 = 0, $6996 = 0, $6997 = 0, $6998 = 0, $6999 = 0, $7 = 0, $70 = 0, $700 = 0, $7000 = 0, $7001 = 0, $7002 = 0, $7003 = 0, $7004 = 0, $7005 = 0, $7006 = 0, $7007 = 0, $7008 = 0, $7009 = 0;
 var $701 = 0, $7010 = 0, $7011 = 0, $7012 = 0, $7013 = 0, $7014 = 0, $7015 = 0, $7016 = 0, $7017 = 0, $7018 = 0, $7019 = 0, $702 = 0, $7020 = 0, $7021 = 0, $7022 = 0, $7023 = 0, $7024 = 0, $7025 = 0, $7026 = 0, $7027 = 0;
 var $7028 = 0, $7029 = 0, $703 = 0, $7030 = 0, $7031 = 0, $7032 = 0, $7033 = 0, $7034 = 0, $7035 = 0, $7036 = 0, $7037 = 0, $7038 = 0, $7039 = 0, $704 = 0, $7040 = 0, $7041 = 0, $7042 = 0, $7043 = 0, $7044 = 0, $7045 = 0;
 var $7046 = 0, $7047 = 0, $7048 = 0, $7049 = 0, $705 = 0, $7050 = 0, $7051 = 0, $7052 = 0, $7053 = 0, $7054 = 0, $7055 = 0, $7056 = 0, $7057 = 0, $7058 = 0, $7059 = 0, $706 = 0, $7060 = 0, $7061 = 0, $7062 = 0, $7063 = 0;
 var $7064 = 0, $7065 = 0, $7066 = 0, $7067 = 0, $7068 = 0, $7069 = 0, $707 = 0, $7070 = 0, $7071 = 0, $7072 = 0, $7073 = 0, $7074 = 0, $7075 = 0, $7076 = 0, $7077 = 0, $7078 = 0, $7079 = 0, $708 = 0, $7080 = 0, $7081 = 0;
 var $7082 = 0, $7083 = 0, $7084 = 0, $7085 = 0, $7086 = 0, $7087 = 0, $7088 = 0, $7089 = 0, $709 = 0, $7090 = 0, $7091 = 0, $7092 = 0, $7093 = 0, $7094 = 0, $7095 = 0, $7096 = 0, $7097 = 0, $7098 = 0, $7099 = 0, $71 = 0;
 var $710 = 0, $7100 = 0, $7101 = 0, $7102 = 0, $7103 = 0, $7104 = 0, $7105 = 0, $7106 = 0, $7107 = 0, $7108 = 0, $7109 = 0, $711 = 0, $7110 = 0, $7111 = 0, $7112 = 0, $7113 = 0, $7114 = 0, $7115 = 0, $7116 = 0, $7117 = 0;
 var $7118 = 0, $7119 = 0, $712 = 0, $7120 = 0, $7121 = 0, $7122 = 0, $7123 = 0, $7124 = 0, $7125 = 0, $7126 = 0, $7127 = 0, $7128 = 0, $7129 = 0, $713 = 0, $7130 = 0, $7131 = 0, $7132 = 0, $7133 = 0, $7134 = 0, $7135 = 0;
 var $7136 = 0, $7137 = 0, $7138 = 0, $7139 = 0, $714 = 0, $7140 = 0, $7141 = 0, $7142 = 0, $7143 = 0, $7144 = 0, $7145 = 0, $7146 = 0, $7147 = 0, $7148 = 0, $7149 = 0, $715 = 0, $7150 = 0, $7151 = 0, $7152 = 0, $7153 = 0;
 var $7154 = 0, $7155 = 0, $7156 = 0, $7157 = 0, $7158 = 0, $7159 = 0, $716 = 0, $7160 = 0, $7161 = 0, $7162 = 0, $7163 = 0, $7164 = 0, $7165 = 0, $7166 = 0, $7167 = 0, $7168 = 0, $7169 = 0, $717 = 0, $7170 = 0, $7171 = 0;
 var $7172 = 0, $7173 = 0, $7174 = 0, $7175 = 0, $7176 = 0, $7177 = 0, $7178 = 0, $7179 = 0, $718 = 0, $7180 = 0, $7181 = 0, $7182 = 0, $7183 = 0, $7184 = 0, $7185 = 0, $7186 = 0, $7187 = 0, $7188 = 0, $7189 = 0, $719 = 0;
 var $7190 = 0, $7191 = 0, $7192 = 0, $7193 = 0, $7194 = 0, $7195 = 0, $7196 = 0, $7197 = 0, $7198 = 0, $7199 = 0, $72 = 0, $720 = 0, $7200 = 0, $7201 = 0, $7202 = 0, $7203 = 0, $7204 = 0, $7205 = 0, $7206 = 0, $7207 = 0;
 var $7208 = 0, $7209 = 0, $721 = 0, $7210 = 0, $7211 = 0, $7212 = 0, $7213 = 0, $7214 = 0, $7215 = 0, $7216 = 0, $7217 = 0, $7218 = 0, $7219 = 0, $722 = 0, $7220 = 0, $7221 = 0, $7222 = 0, $7223 = 0, $7224 = 0, $7225 = 0;
 var $7226 = 0, $7227 = 0, $7228 = 0, $7229 = 0, $723 = 0, $7230 = 0, $7231 = 0, $7232 = 0, $7233 = 0, $7234 = 0, $7235 = 0, $7236 = 0, $7237 = 0, $7238 = 0, $7239 = 0, $724 = 0, $7240 = 0, $7241 = 0, $7242 = 0, $7243 = 0;
 var $7244 = 0, $7245 = 0, $7246 = 0, $7247 = 0, $7248 = 0, $7249 = 0, $725 = 0, $7250 = 0, $7251 = 0, $7252 = 0, $7253 = 0, $7254 = 0, $7255 = 0, $7256 = 0, $7257 = 0, $7258 = 0, $7259 = 0, $726 = 0, $7260 = 0, $7261 = 0;
 var $7262 = 0, $7263 = 0, $7264 = 0, $7265 = 0, $7266 = 0, $7267 = 0, $7268 = 0, $7269 = 0, $727 = 0, $7270 = 0, $7271 = 0, $7272 = 0, $7273 = 0, $7274 = 0, $7275 = 0, $7276 = 0, $7277 = 0, $7278 = 0, $7279 = 0, $728 = 0;
 var $7280 = 0, $7281 = 0, $7282 = 0, $7283 = 0, $7284 = 0, $7285 = 0, $7286 = 0, $7287 = 0, $7288 = 0, $7289 = 0, $729 = 0, $7290 = 0, $7291 = 0, $7292 = 0, $7293 = 0, $7294 = 0, $7295 = 0, $7296 = 0, $7297 = 0, $7298 = 0;
 var $7299 = 0, $73 = 0, $730 = 0, $7300 = 0, $7301 = 0, $7302 = 0, $7303 = 0, $7304 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0;
 var $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0;
 var $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0;
 var $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0;
 var $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0;
 var $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0;
 var $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0;
 var $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0;
 var $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0;
 var $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0;
 var $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0;
 var $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0;
 var $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0;
 var $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0;
 var $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0;
 var $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aMem = 0, $aOffset = 0, $aOp = 0, $aPermute = 0, $aRes = 0, $aRoot = 0, $aTempRec = 0, $affinity = 0, $alreadyExists = 0, $apArg = 0, $apArg126 = 0, $avail = 0, $bIntint = 0, $bRev = 0, $c = 0;
 var $cAff = 0, $cnt = 0, $cnt42 = 0, $db = 0, $desiredAutoCommit = 0, $eNew = 0, $eOld = 0, $encoding = 0, $eqOnly = 0, $exists = 0, $file_format = 0, $flags = 0, $flags1 = 0, $flags3 = 0, $flags79 = 0, $hasUpdateCallback = 0, $i = 0, $i111 = 0, $i12 = 0, $i124 = 0;
 var $i15 = 0, $i3 = 0, $i8 = 0, $i91 = 0, $i96 = 0, $iA = 0, $iA5 = 0, $iB = 0, $iB6 = 0, $iCompare = 0, $iCookie = 0, $iDb = 0, $iDb23 = 0, $iDb76 = 0, $iDb81 = 0, $iGen = 0, $iKey = 0, $iKey39 = 0, $iKey45 = 0, $iMeta = 0;
 var $iMeta19 = 0, $iMoved = 0, $iPrior = 0, $iQuery = 0, $iRollback = 0, $iSavepoint = 0, $iSet = 0, $idx = 0, $ii = 0, $ii32 = 0, $initData = 0, $isSchemaChange = 0, $isTransaction = 0, $isWriteLock = 0, $j = 0, $j82 = 0, $lastRowid = 0, $len = 0, $len16 = 0, $n = 0;
 var $n1 = 0, $n2 = 0, $n55 = 0, $n64 = 0, $n7 = 0, $n89 = 0, $nArg = 0, $nArg123 = 0, $nByte = 0, $nByte13 = 0, $nByte83 = 0, $nChange = 0, $nChange$ = 0, $nData = 0, $nEntry = 0, $nErr = 0, $nField = 0, $nField20 = 0, $nField31 = 0, $nHdr = 0;
 var $nKey = 0, $nKeyCol = 0, $nMem = 0, $nName = 0, $nProgressLimit = 0, $nRoot = 0, $nVarint = 0, $nVmStep = 0, $nZero = 0, $nZero47 = 0, $newMax = 0, $nullFlag = 0, $oc = 0, $offset = 0, $offset64 = 0, $op = 0, $op48 = 0, $or$cond = 0, $or$cond10 = 0, $or$cond12 = 0;
 var $or$cond14 = 0, $or$cond16 = 0, $or$cond18 = 0, $or$cond21 = 0, $or$cond23 = 0, $or$cond25 = 0, $or$cond28 = 0, $or$cond3 = 0, $or$cond30 = 0, $or$cond6 = 0, $or$cond8 = 0, $p1 = 0, $p1100 = 0, $p118 = 0, $p19 = 0, $p2 = 0, $p210 = 0, $p211 = 0, $p222 = 0, $pArgc = 0;
 var $pBt = 0, $pBt128 = 0, $pBt98 = 0, $pBt99 = 0, $pC = 0, $pC27 = 0, $pC30 = 0, $pC33 = 0, $pC36 = 0, $pC40 = 0, $pC46 = 0, $pC49 = 0, $pC50 = 0, $pC52 = 0, $pC53 = 0, $pC56 = 0, $pC58 = 0, $pC59 = 0, $pC62 = 0, $pC65 = 0;
 var $pC67 = 0, $pC68 = 0, $pC72 = 0, $pC73 = 0, $pC77 = 0, $pCaller = 0, $pColl = 0, $pCrsr = 0, $pCrsr17 = 0, $pCrsr37 = 0, $pCrsr54 = 0, $pCrsr60 = 0, $pCrsr63 = 0, $pCrsr69 = 0, $pCtx = 0, $pCtx4 = 0, $pCtx90 = 0, $pCtx92 = 0, $pCur = 0, $pCur103 = 0;
 var $pCur109 = 0, $pCur115 = 0, $pCur119 = 0, $pCx = 0, $pCx26 = 0, $pCx28 = 0, $pData = 0, $pData0 = 0, $pDb = 0, $pDb24 = 0, $pDb80 = 0, $pDest = 0, $pDest114 = 0, $pEnd = 0, $pFrame = 0, $pFrame44 = 0, $pFrame85 = 0, $pFrame87 = 0, $pFrame88 = 0, $pFree = 0;
 var $pIdxKey = 0, $pIn = 0, $pIn1 = 0, $pIn2 = 0, $pIn3 = 0, $pKey = 0, $pKeyInfo = 0, $pKeyInfo21 = 0, $pKeyInfo25 = 0, $pLast = 0, $pMem = 0, $pMem43 = 0, $pMem84 = 0, $pMem93 = 0, $pMem95 = 0, $pMem97 = 0, $pModule = 0, $pModule105 = 0, $pModule106 = 0, $pModule113 = 0;
 var $pModule117 = 0, $pModule122 = 0, $pName = 0, $pNew = 0, $pOp = 0, $pOut = 0, $pPager = 0, $pProgram = 0, $pQuery = 0, $pRec = 0, $pReg = 0, $pRt = 0, $pSavepoint = 0, $pTabCur = 0, $pTmp = 0, $pVCur = 0, $pVCur107 = 0, $pVTab = 0, $pVar = 0, $pVtab = 0;
 var $pVtab104 = 0, $pVtab108 = 0, $pVtab112 = 0, $pVtab116 = 0, $pVtab120 = 0, $pVtab121 = 0, $pX = 0, $pX127 = 0, $payloadSize64 = 0, $pcDest = 0, $pcx = 0, $pgno = 0, $pgno78 = 0, $pnErr = 0, $r = 0, $r35 = 0, $r71 = 0, $r75 = 0, $rA = 0, $rB = 0;
 var $rc = 0, $res = 0, $res110 = 0, $res118 = 0, $res29 = 0, $res34 = 0, $res38 = 0, $res41 = 0, $res51 = 0, $res61 = 0, $res64 = 0, $res66 = 0, $res70 = 0, $res74 = 0, $resetSchemaOnFault = 0, $rowid = 0, $rowid125 = 0, $sContext = 0, $sMem = 0, $sMem102 = 0;
 var $seekResult = 0, $serial_type = 0, $t = 0, $t86 = 0, $t94 = 0, $takeJump = 0, $type1 = 0, $type2 = 0, $uA = 0, $v = 0, $v1 = 0, $v2 = 0, $v57 = 0, $val = 0, $vararg_buffer = 0, $vararg_buffer32 = 0, $vararg_buffer35 = 0, $vararg_buffer38 = 0, $vararg_buffer43 = 0, $vararg_buffer46 = 0;
 var $vararg_buffer48 = 0, $vararg_buffer51 = 0, $vararg_buffer53 = 0, $vararg_buffer55 = 0, $vararg_buffer57 = 0, $vararg_buffer62 = 0, $vararg_buffer64 = 0, $vararg_buffer67 = 0, $vararg_buffer70 = 0, $vararg_buffer73 = 0, $vararg_buffer76 = 0, $vararg_buffer81 = 0, $vararg_buffer83 = 0, $vararg_buffer85 = 0, $vararg_buffer88 = 0, $vararg_ptr31 = 0, $vararg_ptr41 = 0, $vararg_ptr42 = 0, $vararg_ptr60 = 0, $vararg_ptr61 = 0;
 var $vararg_ptr79 = 0, $vararg_ptr80 = 0, $vtabOnConflict = 0, $wrFlag = 0, $z = 0, $z101 = 0, $z129 = 0, $zAffinity = 0, $zAffinity14 = 0, $zData = 0, $zDb = 0, $zEndHdr = 0, $zFilename = 0, $zHdr = 0, $zKey = 0, $zLogFmt = 0, $zMaster = 0, $zName = 0, $zNewRecord = 0, $zSql = 0;
 var $zTab = 0, $zTbl = 0, $zTrace = 0, $zType = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1824|0;
 $vararg_buffer88 = sp + 496|0;
 $vararg_buffer85 = sp + 488|0;
 $vararg_buffer83 = sp + 480|0;
 $vararg_buffer81 = sp + 472|0;
 $vararg_buffer76 = sp + 456|0;
 $vararg_buffer73 = sp + 448|0;
 $vararg_buffer70 = sp + 440|0;
 $vararg_buffer67 = sp + 432|0;
 $vararg_buffer64 = sp + 424|0;
 $vararg_buffer62 = sp + 416|0;
 $vararg_buffer57 = sp + 400|0;
 $vararg_buffer55 = sp + 392|0;
 $vararg_buffer53 = sp + 384|0;
 $vararg_buffer51 = sp + 376|0;
 $vararg_buffer48 = sp + 368|0;
 $vararg_buffer46 = sp + 360|0;
 $vararg_buffer43 = sp + 352|0;
 $vararg_buffer38 = sp + 336|0;
 $vararg_buffer35 = sp + 328|0;
 $vararg_buffer32 = sp + 320|0;
 $vararg_buffer = sp + 312|0;
 $0 = sp + 1612|0;
 $aOp = sp + 1608|0;
 $pOp = sp + 1604|0;
 $rc = sp + 1600|0;
 $db = sp + 1596|0;
 $resetSchemaOnFault = sp + 1819|0;
 $encoding = sp + 1818|0;
 $iCompare = sp + 1592|0;
 $nVmStep = sp + 1588|0;
 $nProgressLimit = sp + 1584|0;
 $aMem = sp + 1580|0;
 $pIn1 = sp + 1576|0;
 $pIn2 = sp + 1572|0;
 $pIn3 = sp + 1568|0;
 $pOut = sp + 1564|0;
 $aPermute = sp + 1560|0;
 $lastRowid = sp + 304|0;
 $iPrior = sp + 1556|0;
 $pCaller = sp + 1552|0;
 $pcDest = sp + 1548|0;
 $zType = sp + 1544|0;
 $zLogFmt = sp + 1540|0;
 $pFrame = sp + 1536|0;
 $pcx = sp + 1532|0;
 $cnt = sp + 1528|0;
 $nullFlag = sp + 1626|0;
 $pVar = sp + 1524|0;
 $n = sp + 1520|0;
 $p1 = sp + 1516|0;
 $p2 = sp + 1512|0;
 $n1 = sp + 1508|0;
 $pMem = sp + 1504|0;
 $i = sp + 1500|0;
 $nByte = sp + 296|0;
 $bIntint = sp + 1817|0;
 $flags = sp + 1624|0;
 $type1 = sp + 1622|0;
 $type2 = sp + 1620|0;
 $iA = sp + 288|0;
 $iB = sp + 280|0;
 $rA = sp + 272|0;
 $rB = sp + 264|0;
 $n2 = sp + 1496|0;
 $pCtx = sp + 1492|0;
 $i3 = sp + 1488|0;
 $pCtx4 = sp + 1484|0;
 $iA5 = sp + 256|0;
 $uA = sp + 248|0;
 $iB6 = sp + 240|0;
 $op = sp + 1816|0;
 $res = sp + 1480|0;
 $affinity = sp + 1815|0;
 $flags1 = sp + 1618|0;
 $flags3 = sp + 1616|0;
 $n7 = sp + 1476|0;
 $i8 = sp + 1472|0;
 $p19 = sp + 1468|0;
 $p210 = sp + 1464|0;
 $pKeyInfo = sp + 1460|0;
 $idx = sp + 1456|0;
 $pColl = sp + 1452|0;
 $bRev = sp + 1448|0;
 $v1 = sp + 1444|0;
 $v2 = sp + 1440|0;
 $c = sp + 1436|0;
 $payloadSize64 = sp + 232|0;
 $p211 = sp + 1432|0;
 $pC = sp + 1428|0;
 $pCrsr = sp + 1424|0;
 $aOffset = sp + 1420|0;
 $len = sp + 1416|0;
 $i12 = sp + 1412|0;
 $pDest = sp + 1408|0;
 $sMem = sp + 192|0;
 $zData = sp + 1404|0;
 $zHdr = sp + 1400|0;
 $zEndHdr = sp + 1396|0;
 $offset = sp + 1392|0;
 $offset64 = sp + 184|0;
 $avail = sp + 1388|0;
 $t = sp + 1384|0;
 $pReg = sp + 1380|0;
 $zAffinity = sp + 1376|0;
 $cAff = sp + 1814|0;
 $zNewRecord = sp + 1372|0;
 $pRec = sp + 1368|0;
 $nData = sp + 176|0;
 $nHdr = sp + 1364|0;
 $nByte13 = sp + 168|0;
 $nZero = sp + 160|0;
 $nVarint = sp + 1360|0;
 $serial_type = sp + 1356|0;
 $pData0 = sp + 1352|0;
 $pLast = sp + 1348|0;
 $nField = sp + 1344|0;
 $zAffinity14 = sp + 1340|0;
 $file_format = sp + 1336|0;
 $i15 = sp + 1332|0;
 $j = sp + 1328|0;
 $len16 = sp + 1324|0;
 $nEntry = sp + 152|0;
 $pCrsr17 = sp + 1320|0;
 $p118 = sp + 1316|0;
 $zName = sp + 1312|0;
 $nName = sp + 1308|0;
 $pNew = sp + 1304|0;
 $pSavepoint = sp + 1300|0;
 $pTmp = sp + 1296|0;
 $iSavepoint = sp + 1292|0;
 $ii = sp + 1288|0;
 $isTransaction = sp + 1284|0;
 $isSchemaChange = sp + 1280|0;
 $desiredAutoCommit = sp + 1276|0;
 $iRollback = sp + 1272|0;
 $pBt = sp + 1268|0;
 $iMeta = sp + 1264|0;
 $iGen = sp + 1260|0;
 $iMeta19 = sp + 1256|0;
 $iDb = sp + 1252|0;
 $iCookie = sp + 1248|0;
 $pDb = sp + 1244|0;
 $nField20 = sp + 1240|0;
 $pKeyInfo21 = sp + 1236|0;
 $p222 = sp + 1232|0;
 $iDb23 = sp + 1228|0;
 $wrFlag = sp + 1224|0;
 $pX = sp + 1220|0;
 $pCur = sp + 1216|0;
 $pDb24 = sp + 1212|0;
 $pCx = sp + 1208|0;
 $pKeyInfo25 = sp + 1204|0;
 $pgno = sp + 1200|0;
 $pCx26 = sp + 1196|0;
 $pC27 = sp + 1192|0;
 $pCx28 = sp + 1188|0;
 $res29 = sp + 1184|0;
 $oc = sp + 1180|0;
 $pC30 = sp + 1176|0;
 $r = sp + 1160|0;
 $nField31 = sp + 1152|0;
 $iKey = sp + 144|0;
 $eqOnly = sp + 1148|0;
 $alreadyExists = sp + 1144|0;
 $takeJump = sp + 1140|0;
 $ii32 = sp + 1136|0;
 $pC33 = sp + 1132|0;
 $res34 = sp + 1128|0;
 $pFree = sp + 1124|0;
 $pIdxKey = sp + 1120|0;
 $r35 = sp + 1104|0;
 $aTempRec = sp + 1631|0;
 $pC36 = sp + 1100|0;
 $pCrsr37 = sp + 1096|0;
 $res38 = sp + 1092|0;
 $iKey39 = sp + 136|0;
 $v = sp + 128|0;
 $pC40 = sp + 1088|0;
 $res41 = sp + 1084|0;
 $cnt42 = sp + 1080|0;
 $pMem43 = sp + 1076|0;
 $pFrame44 = sp + 1072|0;
 $pData = sp + 1068|0;
 $pKey = sp + 1064|0;
 $iKey45 = sp + 120|0;
 $pC46 = sp + 1060|0;
 $nZero47 = sp + 1056|0;
 $seekResult = sp + 1052|0;
 $zDb = sp + 1048|0;
 $zTbl = sp + 1044|0;
 $op48 = sp + 1040|0;
 $pC49 = sp + 1036|0;
 $hasUpdateCallback = sp + 1630|0;
 $pC50 = sp + 1032|0;
 $res51 = sp + 1028|0;
 $nKeyCol = sp + 1024|0;
 $pC52 = sp + 1020|0;
 $pC53 = sp + 1016|0;
 $pCrsr54 = sp + 1012|0;
 $n55 = sp + 1008|0;
 $n64 = sp + 112|0;
 $pC56 = sp + 1004|0;
 $v57 = sp + 104|0;
 $pVtab = sp + 1000|0;
 $pModule = sp + 996|0;
 $pC58 = sp + 992|0;
 $pC59 = sp + 988|0;
 $pCrsr60 = sp + 984|0;
 $res61 = sp + 980|0;
 $pC62 = sp + 976|0;
 $pCrsr63 = sp + 972|0;
 $res64 = sp + 968|0;
 $pC65 = sp + 964|0;
 $res66 = sp + 960|0;
 $pC67 = sp + 956|0;
 $nKey = sp + 952|0;
 $zKey = sp + 948|0;
 $pC68 = sp + 944|0;
 $pCrsr69 = sp + 940|0;
 $res70 = sp + 936|0;
 $r71 = sp + 920|0;
 $pC72 = sp + 916|0;
 $pTabCur = sp + 912|0;
 $rowid = sp + 96|0;
 $pC73 = sp + 908|0;
 $res74 = sp + 904|0;
 $r75 = sp + 888|0;
 $iMoved = sp + 884|0;
 $iDb76 = sp + 880|0;
 $nChange = sp + 876|0;
 $pC77 = sp + 872|0;
 $pgno78 = sp + 868|0;
 $flags79 = sp + 864|0;
 $pDb80 = sp + 860|0;
 $iDb81 = sp + 856|0;
 $zMaster = sp + 852|0;
 $zSql = sp + 848|0;
 $initData = sp + 832|0;
 $nRoot = sp + 828|0;
 $aRoot = sp + 824|0;
 $j82 = sp + 820|0;
 $nErr = sp + 816|0;
 $z = sp + 812|0;
 $pnErr = sp + 808|0;
 $val = sp + 88|0;
 $iSet = sp + 804|0;
 $exists = sp + 800|0;
 $nMem = sp + 796|0;
 $nByte83 = sp + 792|0;
 $pRt = sp + 788|0;
 $pMem84 = sp + 784|0;
 $pEnd = sp + 780|0;
 $pFrame85 = sp + 776|0;
 $pProgram = sp + 772|0;
 $t86 = sp + 768|0;
 $pFrame87 = sp + 764|0;
 $pIn = sp + 760|0;
 $pFrame88 = sp + 756|0;
 $n89 = sp + 752|0;
 $pCtx90 = sp + 748|0;
 $i91 = sp + 744|0;
 $pCtx92 = sp + 740|0;
 $pMem93 = sp + 736|0;
 $t94 = sp + 48|0;
 $pMem95 = sp + 732|0;
 $i96 = sp + 728|0;
 $aRes = sp + 716|0;
 $pMem97 = sp + 712|0;
 $pBt98 = sp + 708|0;
 $pPager = sp + 704|0;
 $eNew = sp + 700|0;
 $eOld = sp + 696|0;
 $zFilename = sp + 692|0;
 $pBt99 = sp + 688|0;
 $isWriteLock = sp + 1629|0;
 $p1100 = sp + 684|0;
 $z101 = sp + 680|0;
 $pVTab = sp + 676|0;
 $sMem102 = sp + 8|0;
 $zTab = sp + 672|0;
 $pCur103 = sp + 668|0;
 $pVCur = sp + 664|0;
 $pVtab104 = sp + 660|0;
 $pModule105 = sp + 656|0;
 $nArg = sp + 652|0;
 $iQuery = sp + 648|0;
 $pModule106 = sp + 644|0;
 $pQuery = sp + 640|0;
 $pArgc = sp + 636|0;
 $pVCur107 = sp + 632|0;
 $pVtab108 = sp + 628|0;
 $pCur109 = sp + 624|0;
 $res110 = sp + 620|0;
 $i111 = sp + 616|0;
 $apArg = sp + 612|0;
 $pVtab112 = sp + 608|0;
 $pModule113 = sp + 604|0;
 $pDest114 = sp + 600|0;
 $sContext = sp + 568|0;
 $pCur115 = sp + 564|0;
 $pVtab116 = sp + 560|0;
 $pModule117 = sp + 556|0;
 $res118 = sp + 552|0;
 $pCur119 = sp + 548|0;
 $pVtab120 = sp + 544|0;
 $pName = sp + 540|0;
 $pVtab121 = sp + 536|0;
 $pModule122 = sp + 532|0;
 $nArg123 = sp + 528|0;
 $i124 = sp + 524|0;
 $rowid125 = sp;
 $apArg126 = sp + 520|0;
 $pX127 = sp + 516|0;
 $vtabOnConflict = sp + 1628|0;
 $newMax = sp + 512|0;
 $pBt128 = sp + 508|0;
 $zTrace = sp + 504|0;
 $z129 = sp + 500|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$aOp>>2] = $3;
 $4 = HEAP32[$aOp>>2]|0;
 HEAP32[$pOp>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP8[$resetSchemaOnFault>>0] = 0;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 62|0);
 $9 = HEAP8[$8>>0]|0;
 HEAP8[$encoding>>0] = $9;
 HEAP32[$iCompare>>2] = 0;
 HEAP32[$nVmStep>>2] = 0;
 HEAP32[$nProgressLimit>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$aMem>>2] = $12;
 HEAP32[$pIn1>>2] = 0;
 HEAP32[$pIn2>>2] = 0;
 HEAP32[$pIn3>>2] = 0;
 HEAP32[$pOut>>2] = 0;
 HEAP32[$aPermute>>2] = 0;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 32|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $lastRowid;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = HEAP32[$0>>2]|0;
 _sqlite3VdbeEnter($25);
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 80|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)==(7);
 L1: do {
  if ($29) {
   label = 860;
  } else {
   $30 = HEAP32[$0>>2]|0;
   $31 = ((($30)) + 80|0);
   HEAP32[$31>>2] = 0;
   $32 = HEAP32[$0>>2]|0;
   $33 = ((($32)) + 136|0);
   $34 = $33;
   $35 = $34;
   HEAP32[$35>>2] = 0;
   $36 = (($34) + 4)|0;
   $37 = $36;
   HEAP32[$37>>2] = 0;
   $38 = HEAP32[$0>>2]|0;
   $39 = ((($38)) + 20|0);
   HEAP32[$39>>2] = 0;
   $40 = HEAP32[$db>>2]|0;
   $41 = ((($40)) + 448|0);
   $42 = ((($41)) + 8|0);
   HEAP32[$42>>2] = 0;
   $43 = HEAP32[$db>>2]|0;
   $44 = ((($43)) + 240|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)!=(0);
   L3: do {
    if (!($46)) {
     $47 = HEAP32[$db>>2]|0;
     $48 = ((($47)) + 296|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49|0)!=(0|0);
     if ($50) {
      $51 = HEAP32[$0>>2]|0;
      $52 = ((($51)) + 108|0);
      $53 = ((($52)) + 16|0);
      $54 = HEAP32[$53>>2]|0;
      HEAP32[$iPrior>>2] = $54;
      $55 = HEAP32[$db>>2]|0;
      $56 = ((($55)) + 304|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$iPrior>>2]|0;
      $59 = HEAP32[$db>>2]|0;
      $60 = ((($59)) + 304|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = (($58>>>0) % ($61>>>0))&-1;
      $63 = (($57) - ($62))|0;
      HEAP32[$nProgressLimit>>2] = $63;
     }
     $64 = HEAP32[$0>>2]|0;
     $65 = ((($64)) + 76|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$aOp>>2]|0;
     $68 = (($67) + (($66*20)|0)|0);
     HEAP32[$pOp>>2] = $68;
     L8: while(1) {
      $69 = HEAP32[$rc>>2]|0;
      $70 = ($69|0)==(0);
      if (!($70)) {
       break L1;
      }
      $71 = HEAP32[$nVmStep>>2]|0;
      $72 = (($71) + 1)|0;
      HEAP32[$nVmStep>>2] = $72;
      $73 = HEAP32[$pOp>>2]|0;
      $74 = HEAP8[$73>>0]|0;
      $75 = $74&255;
      L11: do {
       switch ($75|0) {
       case 13:  {
        label = 8;
        break;
       }
       case 14:  {
        $112 = HEAP32[$pOp>>2]|0;
        $113 = ((($112)) + 4|0);
        $114 = HEAP32[$113>>2]|0;
        $115 = HEAP32[$aMem>>2]|0;
        $116 = (($115) + (($114*40)|0)|0);
        HEAP32[$pIn1>>2] = $116;
        $117 = HEAP32[$pIn1>>2]|0;
        $118 = ((($117)) + 8|0);
        HEAP16[$118>>1] = 4;
        $119 = HEAP32[$pOp>>2]|0;
        $120 = HEAP32[$aOp>>2]|0;
        $121 = $119;
        $122 = $120;
        $123 = (($121) - ($122))|0;
        $124 = (($123|0) / 20)&-1;
        $125 = ($124|0)<(0);
        $126 = $125 << 31 >> 31;
        $127 = HEAP32[$pIn1>>2]|0;
        $128 = $127;
        $129 = $128;
        HEAP32[$129>>2] = $124;
        $130 = (($128) + 4)|0;
        $131 = $130;
        HEAP32[$131>>2] = $126;
        label = 15;
        break;
       }
       case 15:  {
        $138 = HEAP32[$pOp>>2]|0;
        $139 = ((($138)) + 4|0);
        $140 = HEAP32[$139>>2]|0;
        $141 = HEAP32[$aMem>>2]|0;
        $142 = (($141) + (($140*40)|0)|0);
        HEAP32[$pIn1>>2] = $142;
        $143 = HEAP32[$pIn1>>2]|0;
        $144 = $143;
        $145 = $144;
        $146 = HEAP32[$145>>2]|0;
        $147 = (($144) + 4)|0;
        $148 = $147;
        $149 = HEAP32[$148>>2]|0;
        $150 = HEAP32[$aOp>>2]|0;
        $151 = (($150) + (($146*20)|0)|0);
        HEAP32[$pOp>>2] = $151;
        $152 = HEAP32[$pIn1>>2]|0;
        $153 = ((($152)) + 8|0);
        HEAP16[$153>>1] = 128;
        break;
       }
       case 16:  {
        $154 = HEAP32[$pOp>>2]|0;
        $155 = ((($154)) + 4|0);
        $156 = HEAP32[$155>>2]|0;
        $157 = HEAP32[$aMem>>2]|0;
        $158 = (($157) + (($156*40)|0)|0);
        HEAP32[$pOut>>2] = $158;
        $159 = HEAP32[$pOp>>2]|0;
        $160 = ((($159)) + 12|0);
        $161 = HEAP32[$160>>2]|0;
        $162 = (($161) - 1)|0;
        $163 = ($162|0)<(0);
        $164 = $163 << 31 >> 31;
        $165 = HEAP32[$pOut>>2]|0;
        $166 = $165;
        $167 = $166;
        HEAP32[$167>>2] = $162;
        $168 = (($166) + 4)|0;
        $169 = $168;
        HEAP32[$169>>2] = $164;
        $170 = HEAP32[$pOut>>2]|0;
        $171 = ((($170)) + 8|0);
        HEAP16[$171>>1] = 4;
        $172 = HEAP32[$pOp>>2]|0;
        $173 = ((($172)) + 8|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)!=(0);
        if ($175) {
         label = 15;
        }
        break;
       }
       case 17:  {
        $176 = HEAP32[$pOp>>2]|0;
        $177 = ((($176)) + 4|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = HEAP32[$aMem>>2]|0;
        $180 = (($179) + (($178*40)|0)|0);
        HEAP32[$pIn1>>2] = $180;
        $181 = HEAP32[$pIn1>>2]|0;
        $182 = $181;
        $183 = $182;
        $184 = HEAP32[$183>>2]|0;
        $185 = (($182) + 4)|0;
        $186 = $185;
        $187 = HEAP32[$186>>2]|0;
        $188 = HEAP32[$aOp>>2]|0;
        $189 = (($188) + (($184*20)|0)|0);
        HEAP32[$pCaller>>2] = $189;
        $190 = HEAP32[$pCaller>>2]|0;
        $191 = ((($190)) + 8|0);
        $192 = HEAP32[$191>>2]|0;
        $193 = (($192) - 1)|0;
        $194 = HEAP32[$aOp>>2]|0;
        $195 = (($194) + (($193*20)|0)|0);
        HEAP32[$pOp>>2] = $195;
        $196 = HEAP32[$pIn1>>2]|0;
        $197 = ((($196)) + 8|0);
        HEAP16[$197>>1] = 128;
        break;
       }
       case 18:  {
        $198 = HEAP32[$pOp>>2]|0;
        $199 = ((($198)) + 4|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = HEAP32[$aMem>>2]|0;
        $202 = (($201) + (($200*40)|0)|0);
        HEAP32[$pIn1>>2] = $202;
        $203 = HEAP32[$pIn1>>2]|0;
        $204 = ((($203)) + 8|0);
        HEAP16[$204>>1] = 4;
        $205 = HEAP32[$pIn1>>2]|0;
        $206 = $205;
        $207 = $206;
        $208 = HEAP32[$207>>2]|0;
        $209 = (($206) + 4)|0;
        $210 = $209;
        $211 = HEAP32[$210>>2]|0;
        HEAP32[$pcDest>>2] = $208;
        $212 = HEAP32[$pOp>>2]|0;
        $213 = HEAP32[$aOp>>2]|0;
        $214 = $212;
        $215 = $213;
        $216 = (($214) - ($215))|0;
        $217 = (($216|0) / 20)&-1;
        $218 = ($217|0)<(0);
        $219 = $218 << 31 >> 31;
        $220 = HEAP32[$pIn1>>2]|0;
        $221 = $220;
        $222 = $221;
        HEAP32[$222>>2] = $217;
        $223 = (($221) + 4)|0;
        $224 = $223;
        HEAP32[$224>>2] = $219;
        $225 = HEAP32[$pcDest>>2]|0;
        $226 = HEAP32[$aOp>>2]|0;
        $227 = (($226) + (($225*20)|0)|0);
        HEAP32[$pOp>>2] = $227;
        break;
       }
       case 20:  {
        $228 = HEAP32[$pOp>>2]|0;
        $229 = ((($228)) + 12|0);
        $230 = HEAP32[$229>>2]|0;
        $231 = HEAP32[$aMem>>2]|0;
        $232 = (($231) + (($230*40)|0)|0);
        HEAP32[$pIn3>>2] = $232;
        $233 = HEAP32[$pIn3>>2]|0;
        $234 = ((($233)) + 8|0);
        $235 = HEAP16[$234>>1]|0;
        $236 = $235&65535;
        $237 = $236 & 1;
        $238 = ($237|0)==(0);
        if (!($238)) {
         label = 21;
        }
        break;
       }
       case 21:  {
        label = 21;
        break;
       }
       case 22:  {
        $375 = HEAP32[$0>>2]|0;
        $376 = HEAP32[$pOp>>2]|0;
        $377 = (_out2Prerelease($375,$376)|0);
        HEAP32[$pOut>>2] = $377;
        $378 = HEAP32[$pOp>>2]|0;
        $379 = ((($378)) + 4|0);
        $380 = HEAP32[$379>>2]|0;
        $381 = ($380|0)<(0);
        $382 = $381 << 31 >> 31;
        $383 = HEAP32[$pOut>>2]|0;
        $384 = $383;
        $385 = $384;
        HEAP32[$385>>2] = $380;
        $386 = (($384) + 4)|0;
        $387 = $386;
        HEAP32[$387>>2] = $382;
        break;
       }
       case 23:  {
        $388 = HEAP32[$0>>2]|0;
        $389 = HEAP32[$pOp>>2]|0;
        $390 = (_out2Prerelease($388,$389)|0);
        HEAP32[$pOut>>2] = $390;
        $391 = HEAP32[$pOp>>2]|0;
        $392 = ((($391)) + 16|0);
        $393 = HEAP32[$392>>2]|0;
        $394 = $393;
        $395 = $394;
        $396 = HEAP32[$395>>2]|0;
        $397 = (($394) + 4)|0;
        $398 = $397;
        $399 = HEAP32[$398>>2]|0;
        $400 = HEAP32[$pOut>>2]|0;
        $401 = $400;
        $402 = $401;
        HEAP32[$402>>2] = $396;
        $403 = (($401) + 4)|0;
        $404 = $403;
        HEAP32[$404>>2] = $399;
        break;
       }
       case 133:  {
        $405 = HEAP32[$0>>2]|0;
        $406 = HEAP32[$pOp>>2]|0;
        $407 = (_out2Prerelease($405,$406)|0);
        HEAP32[$pOut>>2] = $407;
        $408 = HEAP32[$pOut>>2]|0;
        $409 = ((($408)) + 8|0);
        HEAP16[$409>>1] = 8;
        $410 = HEAP32[$pOp>>2]|0;
        $411 = ((($410)) + 16|0);
        $412 = HEAP32[$411>>2]|0;
        $413 = +HEAPF64[$412>>3];
        $414 = HEAP32[$pOut>>2]|0;
        HEAPF64[$414>>3] = $413;
        break;
       }
       case 97:  {
        $415 = HEAP32[$0>>2]|0;
        $416 = HEAP32[$pOp>>2]|0;
        $417 = (_out2Prerelease($415,$416)|0);
        HEAP32[$pOut>>2] = $417;
        $418 = HEAP32[$pOp>>2]|0;
        HEAP8[$418>>0] = 24;
        $419 = HEAP32[$pOp>>2]|0;
        $420 = ((($419)) + 16|0);
        $421 = HEAP32[$420>>2]|0;
        $422 = (_sqlite3Strlen30($421)|0);
        $423 = HEAP32[$pOp>>2]|0;
        $424 = ((($423)) + 4|0);
        HEAP32[$424>>2] = $422;
        $425 = HEAP8[$encoding>>0]|0;
        $426 = $425&255;
        $427 = ($426|0)!=(1);
        if ($427) {
         $428 = HEAP32[$pOut>>2]|0;
         $429 = HEAP32[$pOp>>2]|0;
         $430 = ((($429)) + 16|0);
         $431 = HEAP32[$430>>2]|0;
         $432 = (_sqlite3VdbeMemSetStr($428,$431,-1,1,0)|0);
         HEAP32[$rc>>2] = $432;
         $433 = HEAP32[$rc>>2]|0;
         $434 = ($433|0)==(18);
         if ($434) {
          label = 859;
          break L8;
         }
         $435 = HEAP32[$pOut>>2]|0;
         $436 = HEAP8[$encoding>>0]|0;
         $437 = $436&255;
         $438 = (_sqlite3VdbeChangeEncoding($435,$437)|0);
         $439 = (0)!=($438|0);
         if ($439) {
          label = 860;
          break L1;
         }
         $440 = HEAP32[$pOut>>2]|0;
         $441 = ((($440)) + 24|0);
         HEAP32[$441>>2] = 0;
         $442 = HEAP32[$pOut>>2]|0;
         $443 = ((($442)) + 8|0);
         $444 = HEAP16[$443>>1]|0;
         $445 = $444&65535;
         $446 = $445 | 2048;
         $447 = $446&65535;
         HEAP16[$443>>1] = $447;
         $448 = HEAP32[$pOp>>2]|0;
         $449 = ((($448)) + 1|0);
         $450 = HEAP8[$449>>0]|0;
         $451 = $450 << 24 >> 24;
         $452 = ($451|0)==(-1);
         if ($452) {
          $453 = HEAP32[$db>>2]|0;
          $454 = HEAP32[$pOp>>2]|0;
          $455 = ((($454)) + 16|0);
          $456 = HEAP32[$455>>2]|0;
          _sqlite3DbFree($453,$456);
         }
         $457 = HEAP32[$pOp>>2]|0;
         $458 = ((($457)) + 1|0);
         HEAP8[$458>>0] = -1;
         $459 = HEAP32[$pOut>>2]|0;
         $460 = ((($459)) + 16|0);
         $461 = HEAP32[$460>>2]|0;
         $462 = HEAP32[$pOp>>2]|0;
         $463 = ((($462)) + 16|0);
         HEAP32[$463>>2] = $461;
         $464 = HEAP32[$pOut>>2]|0;
         $465 = ((($464)) + 12|0);
         $466 = HEAP32[$465>>2]|0;
         $467 = HEAP32[$pOp>>2]|0;
         $468 = ((($467)) + 4|0);
         HEAP32[$468>>2] = $466;
        }
        $469 = HEAP32[$pOp>>2]|0;
        $470 = ((($469)) + 4|0);
        $471 = HEAP32[$470>>2]|0;
        $472 = HEAP32[$db>>2]|0;
        $473 = ((($472)) + 88|0);
        $474 = HEAP32[$473>>2]|0;
        $475 = ($471|0)>($474|0);
        if ($475) {
         label = 859;
         break L8;
        } else {
         label = 50;
        }
        break;
       }
       case 24:  {
        label = 50;
        break;
       }
       case 25:  {
        $515 = HEAP32[$0>>2]|0;
        $516 = HEAP32[$pOp>>2]|0;
        $517 = (_out2Prerelease($515,$516)|0);
        HEAP32[$pOut>>2] = $517;
        $518 = HEAP32[$pOp>>2]|0;
        $519 = ((($518)) + 12|0);
        $520 = HEAP32[$519>>2]|0;
        $521 = HEAP32[$pOp>>2]|0;
        $522 = ((($521)) + 8|0);
        $523 = HEAP32[$522>>2]|0;
        $524 = (($520) - ($523))|0;
        HEAP32[$cnt>>2] = $524;
        $525 = HEAP32[$pOp>>2]|0;
        $526 = ((($525)) + 4|0);
        $527 = HEAP32[$526>>2]|0;
        $528 = ($527|0)!=(0);
        $529 = $528 ? 257 : 1;
        $530 = $529&65535;
        HEAP16[$nullFlag>>1] = $530;
        $531 = HEAP32[$pOut>>2]|0;
        $532 = ((($531)) + 8|0);
        HEAP16[$532>>1] = $530;
        while(1) {
         $533 = HEAP32[$cnt>>2]|0;
         $534 = ($533|0)>(0);
         if (!($534)) {
          break L11;
         }
         $535 = HEAP32[$pOut>>2]|0;
         $536 = ((($535)) + 40|0);
         HEAP32[$pOut>>2] = $536;
         $537 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($537);
         $538 = HEAP16[$nullFlag>>1]|0;
         $539 = HEAP32[$pOut>>2]|0;
         $540 = ((($539)) + 8|0);
         HEAP16[$540>>1] = $538;
         $541 = HEAP32[$cnt>>2]|0;
         $542 = (($541) + -1)|0;
         HEAP32[$cnt>>2] = $542;
        }
        break;
       }
       case 26:  {
        $543 = HEAP32[$pOp>>2]|0;
        $544 = ((($543)) + 4|0);
        $545 = HEAP32[$544>>2]|0;
        $546 = HEAP32[$aMem>>2]|0;
        $547 = (($546) + (($545*40)|0)|0);
        HEAP32[$pOut>>2] = $547;
        $548 = HEAP32[$pOut>>2]|0;
        $549 = ((($548)) + 8|0);
        $550 = HEAP16[$549>>1]|0;
        $551 = $550&65535;
        $552 = $551 | 1;
        $553 = $552 & -129;
        $554 = $553&65535;
        $555 = HEAP32[$pOut>>2]|0;
        $556 = ((($555)) + 8|0);
        HEAP16[$556>>1] = $554;
        break;
       }
       case 27:  {
        $557 = HEAP32[$0>>2]|0;
        $558 = HEAP32[$pOp>>2]|0;
        $559 = (_out2Prerelease($557,$558)|0);
        HEAP32[$pOut>>2] = $559;
        $560 = HEAP32[$pOut>>2]|0;
        $561 = HEAP32[$pOp>>2]|0;
        $562 = ((($561)) + 16|0);
        $563 = HEAP32[$562>>2]|0;
        $564 = HEAP32[$pOp>>2]|0;
        $565 = ((($564)) + 4|0);
        $566 = HEAP32[$565>>2]|0;
        (_sqlite3VdbeMemSetStr($560,$563,$566,0,0)|0);
        $567 = HEAP8[$encoding>>0]|0;
        $568 = HEAP32[$pOut>>2]|0;
        $569 = ((($568)) + 10|0);
        HEAP8[$569>>0] = $567;
        break;
       }
       case 28:  {
        $570 = HEAP32[$pOp>>2]|0;
        $571 = ((($570)) + 4|0);
        $572 = HEAP32[$571>>2]|0;
        $573 = (($572) - 1)|0;
        $574 = HEAP32[$0>>2]|0;
        $575 = ((($574)) + 60|0);
        $576 = HEAP32[$575>>2]|0;
        $577 = (($576) + (($573*40)|0)|0);
        HEAP32[$pVar>>2] = $577;
        $578 = HEAP32[$pVar>>2]|0;
        $579 = (_sqlite3VdbeMemTooBig($578)|0);
        $580 = ($579|0)!=(0);
        if ($580) {
         label = 859;
         break L8;
        }
        $581 = HEAP32[$0>>2]|0;
        $582 = HEAP32[$pOp>>2]|0;
        $583 = (_out2Prerelease($581,$582)|0);
        HEAP32[$pOut>>2] = $583;
        $584 = HEAP32[$pOut>>2]|0;
        $585 = HEAP32[$pVar>>2]|0;
        _sqlite3VdbeMemShallowCopy($584,$585,2048);
        break;
       }
       case 29:  {
        $586 = HEAP32[$pOp>>2]|0;
        $587 = ((($586)) + 12|0);
        $588 = HEAP32[$587>>2]|0;
        HEAP32[$n>>2] = $588;
        $589 = HEAP32[$pOp>>2]|0;
        $590 = ((($589)) + 4|0);
        $591 = HEAP32[$590>>2]|0;
        HEAP32[$p1>>2] = $591;
        $592 = HEAP32[$pOp>>2]|0;
        $593 = ((($592)) + 8|0);
        $594 = HEAP32[$593>>2]|0;
        HEAP32[$p2>>2] = $594;
        $595 = HEAP32[$p1>>2]|0;
        $596 = HEAP32[$aMem>>2]|0;
        $597 = (($596) + (($595*40)|0)|0);
        HEAP32[$pIn1>>2] = $597;
        $598 = HEAP32[$p2>>2]|0;
        $599 = HEAP32[$aMem>>2]|0;
        $600 = (($599) + (($598*40)|0)|0);
        HEAP32[$pOut>>2] = $600;
        while(1) {
         $601 = HEAP32[$pOut>>2]|0;
         $602 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemMove($601,$602);
         $603 = HEAP32[$pOut>>2]|0;
         $604 = ((($603)) + 8|0);
         $605 = HEAP16[$604>>1]|0;
         $606 = $605&65535;
         $607 = $606 & 4096;
         $608 = ($607|0)!=(0);
         if ($608) {
          $609 = HEAP32[$pOut>>2]|0;
          $610 = (_sqlite3VdbeMemMakeWriteable($609)|0);
          $611 = ($610|0)!=(0);
          if ($611) {
           label = 860;
           break L1;
          }
         }
         $612 = HEAP32[$pIn1>>2]|0;
         $613 = ((($612)) + 40|0);
         HEAP32[$pIn1>>2] = $613;
         $614 = HEAP32[$pOut>>2]|0;
         $615 = ((($614)) + 40|0);
         HEAP32[$pOut>>2] = $615;
         $616 = HEAP32[$n>>2]|0;
         $617 = (($616) + -1)|0;
         HEAP32[$n>>2] = $617;
         $618 = ($617|0)!=(0);
         if (!($618)) {
          break;
         }
        }
        break;
       }
       case 30:  {
        $619 = HEAP32[$pOp>>2]|0;
        $620 = ((($619)) + 12|0);
        $621 = HEAP32[$620>>2]|0;
        HEAP32[$n1>>2] = $621;
        $622 = HEAP32[$pOp>>2]|0;
        $623 = ((($622)) + 4|0);
        $624 = HEAP32[$623>>2]|0;
        $625 = HEAP32[$aMem>>2]|0;
        $626 = (($625) + (($624*40)|0)|0);
        HEAP32[$pIn1>>2] = $626;
        $627 = HEAP32[$pOp>>2]|0;
        $628 = ((($627)) + 8|0);
        $629 = HEAP32[$628>>2]|0;
        $630 = HEAP32[$aMem>>2]|0;
        $631 = (($630) + (($629*40)|0)|0);
        HEAP32[$pOut>>2] = $631;
        while(1) {
         $632 = HEAP32[$pOut>>2]|0;
         $633 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemShallowCopy($632,$633,4096);
         $634 = HEAP32[$pOut>>2]|0;
         $635 = ((($634)) + 8|0);
         $636 = HEAP16[$635>>1]|0;
         $637 = $636&65535;
         $638 = $637 & 4096;
         $639 = ($638|0)!=(0);
         if ($639) {
          $640 = HEAP32[$pOut>>2]|0;
          $641 = (_sqlite3VdbeMemMakeWriteable($640)|0);
          $642 = ($641|0)!=(0);
          if ($642) {
           label = 860;
           break L1;
          }
         }
         $643 = HEAP32[$n1>>2]|0;
         $644 = (($643) + -1)|0;
         HEAP32[$n1>>2] = $644;
         $645 = ($643|0)==(0);
         if ($645) {
          break L11;
         }
         $646 = HEAP32[$pOut>>2]|0;
         $647 = ((($646)) + 40|0);
         HEAP32[$pOut>>2] = $647;
         $648 = HEAP32[$pIn1>>2]|0;
         $649 = ((($648)) + 40|0);
         HEAP32[$pIn1>>2] = $649;
        }
        break;
       }
       case 31:  {
        $650 = HEAP32[$pOp>>2]|0;
        $651 = ((($650)) + 4|0);
        $652 = HEAP32[$651>>2]|0;
        $653 = HEAP32[$aMem>>2]|0;
        $654 = (($653) + (($652*40)|0)|0);
        HEAP32[$pIn1>>2] = $654;
        $655 = HEAP32[$pOp>>2]|0;
        $656 = ((($655)) + 8|0);
        $657 = HEAP32[$656>>2]|0;
        $658 = HEAP32[$aMem>>2]|0;
        $659 = (($658) + (($657*40)|0)|0);
        HEAP32[$pOut>>2] = $659;
        $660 = HEAP32[$pOut>>2]|0;
        $661 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemShallowCopy($660,$661,4096);
        break;
       }
       case 32:  {
        $662 = HEAP32[$pOp>>2]|0;
        $663 = ((($662)) + 4|0);
        $664 = HEAP32[$663>>2]|0;
        $665 = HEAP32[$aMem>>2]|0;
        $666 = (($665) + (($664*40)|0)|0);
        HEAP32[$pIn1>>2] = $666;
        $667 = HEAP32[$pOp>>2]|0;
        $668 = ((($667)) + 8|0);
        $669 = HEAP32[$668>>2]|0;
        $670 = HEAP32[$aMem>>2]|0;
        $671 = (($670) + (($669*40)|0)|0);
        HEAP32[$pOut>>2] = $671;
        $672 = HEAP32[$pOut>>2]|0;
        $673 = HEAP32[$pIn1>>2]|0;
        $674 = $673;
        $675 = $674;
        $676 = HEAP32[$675>>2]|0;
        $677 = (($674) + 4)|0;
        $678 = $677;
        $679 = HEAP32[$678>>2]|0;
        _sqlite3VdbeMemSetInt64($672,$676,$679);
        break;
       }
       case 33:  {
        $680 = HEAP32[$db>>2]|0;
        $681 = ((($680)) + 296|0);
        $682 = HEAP32[$681>>2]|0;
        $683 = ($682|0)!=(0|0);
        if ($683) {
         $684 = HEAP32[$nVmStep>>2]|0;
         $685 = HEAP32[$nProgressLimit>>2]|0;
         $686 = ($684>>>0)>=($685>>>0);
         if ($686) {
          $687 = HEAP32[$db>>2]|0;
          $688 = ((($687)) + 296|0);
          $689 = HEAP32[$688>>2]|0;
          $690 = HEAP32[$db>>2]|0;
          $691 = ((($690)) + 300|0);
          $692 = HEAP32[$691>>2]|0;
          $693 = (FUNCTION_TABLE_ii[$689 & 255]($692)|0);
          $694 = ($693|0)!=(0);
          if ($694) {
           label = 74;
           break L8;
          }
         }
        }
        $695 = HEAP32[$0>>2]|0;
        $696 = (_sqlite3VdbeCheckFk($695,0)|0);
        HEAP32[$rc>>2] = $696;
        $697 = (0)!=($696|0);
        if (!($697)) {
         $698 = HEAP32[$0>>2]|0;
         $699 = (_sqlite3VdbeCloseStatement($698,1)|0);
         HEAP32[$rc>>2] = $699;
         $700 = HEAP32[$rc>>2]|0;
         $701 = ($700|0)!=(0);
         if (!($701)) {
          label = 77;
          break L8;
         }
        }
        break;
       }
       case 94:  {
        $752 = HEAP32[$pOp>>2]|0;
        $753 = ((($752)) + 4|0);
        $754 = HEAP32[$753>>2]|0;
        $755 = HEAP32[$aMem>>2]|0;
        $756 = (($755) + (($754*40)|0)|0);
        HEAP32[$pIn1>>2] = $756;
        $757 = HEAP32[$pOp>>2]|0;
        $758 = ((($757)) + 8|0);
        $759 = HEAP32[$758>>2]|0;
        $760 = HEAP32[$aMem>>2]|0;
        $761 = (($760) + (($759*40)|0)|0);
        HEAP32[$pIn2>>2] = $761;
        $762 = HEAP32[$pOp>>2]|0;
        $763 = ((($762)) + 12|0);
        $764 = HEAP32[$763>>2]|0;
        $765 = HEAP32[$aMem>>2]|0;
        $766 = (($765) + (($764*40)|0)|0);
        HEAP32[$pOut>>2] = $766;
        $767 = HEAP32[$pIn1>>2]|0;
        $768 = ((($767)) + 8|0);
        $769 = HEAP16[$768>>1]|0;
        $770 = $769&65535;
        $771 = HEAP32[$pIn2>>2]|0;
        $772 = ((($771)) + 8|0);
        $773 = HEAP16[$772>>1]|0;
        $774 = $773&65535;
        $775 = $770 | $774;
        $776 = $775 & 1;
        $777 = ($776|0)!=(0);
        if ($777) {
         $778 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($778);
         break L11;
        }
        $779 = HEAP32[$pIn1>>2]|0;
        $780 = ((($779)) + 8|0);
        $781 = HEAP16[$780>>1]|0;
        $782 = $781&65535;
        $783 = $782 & 16384;
        $784 = ($783|0)!=(0);
        if ($784) {
         $785 = HEAP32[$pIn1>>2]|0;
         $786 = (_sqlite3VdbeMemExpandBlob($785)|0);
         $787 = ($786|0)!=(0);
         if ($787) {
          label = 860;
          break L1;
         }
        }
        $788 = HEAP32[$pIn2>>2]|0;
        $789 = ((($788)) + 8|0);
        $790 = HEAP16[$789>>1]|0;
        $791 = $790&65535;
        $792 = $791 & 16384;
        $793 = ($792|0)!=(0);
        if ($793) {
         $794 = HEAP32[$pIn2>>2]|0;
         $795 = (_sqlite3VdbeMemExpandBlob($794)|0);
         $796 = ($795|0)!=(0);
         if ($796) {
          label = 860;
          break L1;
         }
        }
        $797 = HEAP32[$pIn1>>2]|0;
        $798 = ((($797)) + 8|0);
        $799 = HEAP16[$798>>1]|0;
        $800 = $799&65535;
        $801 = $800 & 18;
        $802 = ($801|0)==(0);
        if ($802) {
         $803 = HEAP32[$pIn1>>2]|0;
         $804 = HEAP8[$encoding>>0]|0;
         $805 = (_sqlite3VdbeMemStringify($803,$804,0)|0);
         $806 = ($805|0)!=(0);
         if ($806) {
          label = 860;
          break L1;
         }
        }
        $807 = HEAP32[$pIn2>>2]|0;
        $808 = ((($807)) + 8|0);
        $809 = HEAP16[$808>>1]|0;
        $810 = $809&65535;
        $811 = $810 & 18;
        $812 = ($811|0)==(0);
        if ($812) {
         $813 = HEAP32[$pIn2>>2]|0;
         $814 = HEAP8[$encoding>>0]|0;
         $815 = (_sqlite3VdbeMemStringify($813,$814,0)|0);
         $816 = ($815|0)!=(0);
         if ($816) {
          label = 860;
          break L1;
         }
        }
        $817 = HEAP32[$pIn1>>2]|0;
        $818 = ((($817)) + 12|0);
        $819 = HEAP32[$818>>2]|0;
        $820 = HEAP32[$pIn2>>2]|0;
        $821 = ((($820)) + 12|0);
        $822 = HEAP32[$821>>2]|0;
        $823 = (($819) + ($822))|0;
        $824 = ($823|0)<(0);
        $825 = $824 << 31 >> 31;
        $826 = $nByte;
        $827 = $826;
        HEAP32[$827>>2] = $823;
        $828 = (($826) + 4)|0;
        $829 = $828;
        HEAP32[$829>>2] = $825;
        $830 = $nByte;
        $831 = $830;
        $832 = HEAP32[$831>>2]|0;
        $833 = (($830) + 4)|0;
        $834 = $833;
        $835 = HEAP32[$834>>2]|0;
        $836 = HEAP32[$db>>2]|0;
        $837 = ((($836)) + 88|0);
        $838 = HEAP32[$837>>2]|0;
        $839 = ($838|0)<(0);
        $840 = $839 << 31 >> 31;
        $841 = ($835|0)>($840|0);
        $842 = ($832>>>0)>($838>>>0);
        $843 = ($835|0)==($840|0);
        $844 = $843 & $842;
        $845 = $841 | $844;
        if ($845) {
         label = 859;
         break L8;
        }
        $846 = HEAP32[$pOut>>2]|0;
        $847 = $nByte;
        $848 = $847;
        $849 = HEAP32[$848>>2]|0;
        $850 = (($847) + 4)|0;
        $851 = $850;
        $852 = HEAP32[$851>>2]|0;
        $853 = (($849) + 2)|0;
        $854 = HEAP32[$pOut>>2]|0;
        $855 = HEAP32[$pIn2>>2]|0;
        $856 = ($854|0)==($855|0);
        $857 = $856&1;
        $858 = (_sqlite3VdbeMemGrow($846,$853,$857)|0);
        $859 = ($858|0)!=(0);
        if ($859) {
         label = 860;
         break L1;
        }
        $860 = HEAP32[$pOut>>2]|0;
        $861 = ((($860)) + 8|0);
        $862 = HEAP16[$861>>1]|0;
        $863 = $862&65535;
        $864 = $863 & -49664;
        $865 = $864 | 2;
        $866 = $865&65535;
        $867 = HEAP32[$pOut>>2]|0;
        $868 = ((($867)) + 8|0);
        HEAP16[$868>>1] = $866;
        $869 = HEAP32[$pOut>>2]|0;
        $870 = HEAP32[$pIn2>>2]|0;
        $871 = ($869|0)!=($870|0);
        if ($871) {
         $872 = HEAP32[$pOut>>2]|0;
         $873 = ((($872)) + 16|0);
         $874 = HEAP32[$873>>2]|0;
         $875 = HEAP32[$pIn2>>2]|0;
         $876 = ((($875)) + 16|0);
         $877 = HEAP32[$876>>2]|0;
         $878 = HEAP32[$pIn2>>2]|0;
         $879 = ((($878)) + 12|0);
         $880 = HEAP32[$879>>2]|0;
         _memcpy(($874|0),($877|0),($880|0))|0;
        }
        $881 = HEAP32[$pIn2>>2]|0;
        $882 = ((($881)) + 12|0);
        $883 = HEAP32[$882>>2]|0;
        $884 = HEAP32[$pOut>>2]|0;
        $885 = ((($884)) + 16|0);
        $886 = HEAP32[$885>>2]|0;
        $887 = (($886) + ($883)|0);
        $888 = HEAP32[$pIn1>>2]|0;
        $889 = ((($888)) + 16|0);
        $890 = HEAP32[$889>>2]|0;
        $891 = HEAP32[$pIn1>>2]|0;
        $892 = ((($891)) + 12|0);
        $893 = HEAP32[$892>>2]|0;
        _memcpy(($887|0),($890|0),($893|0))|0;
        $894 = $nByte;
        $895 = $894;
        $896 = HEAP32[$895>>2]|0;
        $897 = (($894) + 4)|0;
        $898 = $897;
        $899 = HEAP32[$898>>2]|0;
        $900 = HEAP32[$pOut>>2]|0;
        $901 = ((($900)) + 16|0);
        $902 = HEAP32[$901>>2]|0;
        $903 = (($902) + ($896)|0);
        HEAP8[$903>>0] = 0;
        $904 = $nByte;
        $905 = $904;
        $906 = HEAP32[$905>>2]|0;
        $907 = (($904) + 4)|0;
        $908 = $907;
        $909 = HEAP32[$908>>2]|0;
        $910 = (_i64Add(($906|0),($909|0),1,0)|0);
        $911 = tempRet0;
        $912 = HEAP32[$pOut>>2]|0;
        $913 = ((($912)) + 16|0);
        $914 = HEAP32[$913>>2]|0;
        $915 = (($914) + ($910)|0);
        HEAP8[$915>>0] = 0;
        $916 = HEAP32[$pOut>>2]|0;
        $917 = ((($916)) + 8|0);
        $918 = HEAP16[$917>>1]|0;
        $919 = $918&65535;
        $920 = $919 | 512;
        $921 = $920&65535;
        HEAP16[$917>>1] = $921;
        $922 = $nByte;
        $923 = $922;
        $924 = HEAP32[$923>>2]|0;
        $925 = (($922) + 4)|0;
        $926 = $925;
        $927 = HEAP32[$926>>2]|0;
        $928 = HEAP32[$pOut>>2]|0;
        $929 = ((($928)) + 12|0);
        HEAP32[$929>>2] = $924;
        $930 = HEAP8[$encoding>>0]|0;
        $931 = HEAP32[$pOut>>2]|0;
        $932 = ((($931)) + 10|0);
        HEAP8[$932>>0] = $930;
        break;
       }
       case 93: case 92: case 91: case 90: case 89:  {
        $933 = HEAP32[$pOp>>2]|0;
        $934 = ((($933)) + 4|0);
        $935 = HEAP32[$934>>2]|0;
        $936 = HEAP32[$aMem>>2]|0;
        $937 = (($936) + (($935*40)|0)|0);
        HEAP32[$pIn1>>2] = $937;
        $938 = HEAP32[$pIn1>>2]|0;
        $939 = (_numericType($938)|0);
        HEAP16[$type1>>1] = $939;
        $940 = HEAP32[$pOp>>2]|0;
        $941 = ((($940)) + 8|0);
        $942 = HEAP32[$941>>2]|0;
        $943 = HEAP32[$aMem>>2]|0;
        $944 = (($943) + (($942*40)|0)|0);
        HEAP32[$pIn2>>2] = $944;
        $945 = HEAP32[$pIn2>>2]|0;
        $946 = (_numericType($945)|0);
        HEAP16[$type2>>1] = $946;
        $947 = HEAP32[$pOp>>2]|0;
        $948 = ((($947)) + 12|0);
        $949 = HEAP32[$948>>2]|0;
        $950 = HEAP32[$aMem>>2]|0;
        $951 = (($950) + (($949*40)|0)|0);
        HEAP32[$pOut>>2] = $951;
        $952 = HEAP32[$pIn1>>2]|0;
        $953 = ((($952)) + 8|0);
        $954 = HEAP16[$953>>1]|0;
        $955 = $954&65535;
        $956 = HEAP32[$pIn2>>2]|0;
        $957 = ((($956)) + 8|0);
        $958 = HEAP16[$957>>1]|0;
        $959 = $958&65535;
        $960 = $955 | $959;
        $961 = $960&65535;
        HEAP16[$flags>>1] = $961;
        $962 = HEAP16[$flags>>1]|0;
        $963 = $962&65535;
        $964 = $963 & 1;
        $965 = ($964|0)!=(0);
        L81: do {
         if (!($965)) {
          $966 = HEAP16[$type1>>1]|0;
          $967 = $966&65535;
          $968 = HEAP16[$type2>>1]|0;
          $969 = $968&65535;
          $970 = $967 & $969;
          $971 = $970 & 4;
          $972 = ($971|0)!=(0);
          L83: do {
           if ($972) {
            $973 = HEAP32[$pIn1>>2]|0;
            $974 = $973;
            $975 = $974;
            $976 = HEAP32[$975>>2]|0;
            $977 = (($974) + 4)|0;
            $978 = $977;
            $979 = HEAP32[$978>>2]|0;
            $980 = $iA;
            $981 = $980;
            HEAP32[$981>>2] = $976;
            $982 = (($980) + 4)|0;
            $983 = $982;
            HEAP32[$983>>2] = $979;
            $984 = HEAP32[$pIn2>>2]|0;
            $985 = $984;
            $986 = $985;
            $987 = HEAP32[$986>>2]|0;
            $988 = (($985) + 4)|0;
            $989 = $988;
            $990 = HEAP32[$989>>2]|0;
            $991 = $iB;
            $992 = $991;
            HEAP32[$992>>2] = $987;
            $993 = (($991) + 4)|0;
            $994 = $993;
            HEAP32[$994>>2] = $990;
            HEAP8[$bIntint>>0] = 1;
            $995 = HEAP32[$pOp>>2]|0;
            $996 = HEAP8[$995>>0]|0;
            $997 = $996&255;
            switch ($997|0) {
            case 89:  {
             $998 = $iA;
             $999 = $998;
             $1000 = HEAP32[$999>>2]|0;
             $1001 = (($998) + 4)|0;
             $1002 = $1001;
             $1003 = HEAP32[$1002>>2]|0;
             $1004 = (_sqlite3AddInt64($iB,$1000,$1003)|0);
             $1005 = ($1004|0)!=(0);
             if ($1005) {
              break L83;
             }
             break;
            }
            case 90:  {
             $1006 = $iA;
             $1007 = $1006;
             $1008 = HEAP32[$1007>>2]|0;
             $1009 = (($1006) + 4)|0;
             $1010 = $1009;
             $1011 = HEAP32[$1010>>2]|0;
             $1012 = (_sqlite3SubInt64($iB,$1008,$1011)|0);
             $1013 = ($1012|0)!=(0);
             if ($1013) {
              break L83;
             }
             break;
            }
            case 91:  {
             $1014 = $iA;
             $1015 = $1014;
             $1016 = HEAP32[$1015>>2]|0;
             $1017 = (($1014) + 4)|0;
             $1018 = $1017;
             $1019 = HEAP32[$1018>>2]|0;
             $1020 = (_sqlite3MulInt64($iB,$1016,$1019)|0);
             $1021 = ($1020|0)!=(0);
             if ($1021) {
              break L83;
             }
             break;
            }
            case 92:  {
             $1022 = $iA;
             $1023 = $1022;
             $1024 = HEAP32[$1023>>2]|0;
             $1025 = (($1022) + 4)|0;
             $1026 = $1025;
             $1027 = HEAP32[$1026>>2]|0;
             $1028 = ($1024|0)==(0);
             $1029 = ($1027|0)==(0);
             $1030 = $1028 & $1029;
             if ($1030) {
              break L81;
             }
             $1031 = $iA;
             $1032 = $1031;
             $1033 = HEAP32[$1032>>2]|0;
             $1034 = (($1031) + 4)|0;
             $1035 = $1034;
             $1036 = HEAP32[$1035>>2]|0;
             $1037 = ($1033|0)==(-1);
             $1038 = ($1036|0)==(-1);
             $1039 = $1037 & $1038;
             $1040 = $iB;
             $1041 = $1040;
             $1042 = HEAP32[$1041>>2]|0;
             $1043 = (($1040) + 4)|0;
             $1044 = $1043;
             $1045 = HEAP32[$1044>>2]|0;
             $1046 = ($1042|0)==(0);
             $1047 = ($1045|0)==(-2147483648);
             $1048 = $1046 & $1047;
             $or$cond = $1039 & $1048;
             if ($or$cond) {
              break L83;
             }
             $1049 = $iA;
             $1050 = $1049;
             $1051 = HEAP32[$1050>>2]|0;
             $1052 = (($1049) + 4)|0;
             $1053 = $1052;
             $1054 = HEAP32[$1053>>2]|0;
             $1055 = $iB;
             $1056 = $1055;
             $1057 = HEAP32[$1056>>2]|0;
             $1058 = (($1055) + 4)|0;
             $1059 = $1058;
             $1060 = HEAP32[$1059>>2]|0;
             $1061 = (___divdi3(($1057|0),($1060|0),($1051|0),($1054|0))|0);
             $1062 = tempRet0;
             $1063 = $iB;
             $1064 = $1063;
             HEAP32[$1064>>2] = $1061;
             $1065 = (($1063) + 4)|0;
             $1066 = $1065;
             HEAP32[$1066>>2] = $1062;
             break;
            }
            default: {
             $1067 = $iA;
             $1068 = $1067;
             $1069 = HEAP32[$1068>>2]|0;
             $1070 = (($1067) + 4)|0;
             $1071 = $1070;
             $1072 = HEAP32[$1071>>2]|0;
             $1073 = ($1069|0)==(0);
             $1074 = ($1072|0)==(0);
             $1075 = $1073 & $1074;
             if ($1075) {
              break L81;
             }
             $1076 = $iA;
             $1077 = $1076;
             $1078 = HEAP32[$1077>>2]|0;
             $1079 = (($1076) + 4)|0;
             $1080 = $1079;
             $1081 = HEAP32[$1080>>2]|0;
             $1082 = ($1078|0)==(-1);
             $1083 = ($1081|0)==(-1);
             $1084 = $1082 & $1083;
             if ($1084) {
              $1085 = $iA;
              $1086 = $1085;
              HEAP32[$1086>>2] = 1;
              $1087 = (($1085) + 4)|0;
              $1088 = $1087;
              HEAP32[$1088>>2] = 0;
             }
             $1089 = $iA;
             $1090 = $1089;
             $1091 = HEAP32[$1090>>2]|0;
             $1092 = (($1089) + 4)|0;
             $1093 = $1092;
             $1094 = HEAP32[$1093>>2]|0;
             $1095 = $iB;
             $1096 = $1095;
             $1097 = HEAP32[$1096>>2]|0;
             $1098 = (($1095) + 4)|0;
             $1099 = $1098;
             $1100 = HEAP32[$1099>>2]|0;
             $1101 = (___remdi3(($1097|0),($1100|0),($1091|0),($1094|0))|0);
             $1102 = tempRet0;
             $1103 = $iB;
             $1104 = $1103;
             HEAP32[$1104>>2] = $1101;
             $1105 = (($1103) + 4)|0;
             $1106 = $1105;
             HEAP32[$1106>>2] = $1102;
            }
            }
            $1107 = $iB;
            $1108 = $1107;
            $1109 = HEAP32[$1108>>2]|0;
            $1110 = (($1107) + 4)|0;
            $1111 = $1110;
            $1112 = HEAP32[$1111>>2]|0;
            $1113 = HEAP32[$pOut>>2]|0;
            $1114 = $1113;
            $1115 = $1114;
            HEAP32[$1115>>2] = $1109;
            $1116 = (($1114) + 4)|0;
            $1117 = $1116;
            HEAP32[$1117>>2] = $1112;
            $1118 = HEAP32[$pOut>>2]|0;
            $1119 = ((($1118)) + 8|0);
            $1120 = HEAP16[$1119>>1]|0;
            $1121 = $1120&65535;
            $1122 = $1121 & -49664;
            $1123 = $1122 | 4;
            $1124 = $1123&65535;
            $1125 = HEAP32[$pOut>>2]|0;
            $1126 = ((($1125)) + 8|0);
            HEAP16[$1126>>1] = $1124;
            break L11;
           } else {
            HEAP8[$bIntint>>0] = 0;
           }
          } while(0);
          $1127 = HEAP32[$pIn1>>2]|0;
          $1128 = (+_sqlite3VdbeRealValue($1127));
          HEAPF64[$rA>>3] = $1128;
          $1129 = HEAP32[$pIn2>>2]|0;
          $1130 = (+_sqlite3VdbeRealValue($1129));
          HEAPF64[$rB>>3] = $1130;
          $1131 = HEAP32[$pOp>>2]|0;
          $1132 = HEAP8[$1131>>0]|0;
          $1133 = $1132&255;
          switch ($1133|0) {
          case 89:  {
           $1134 = +HEAPF64[$rA>>3];
           $1135 = +HEAPF64[$rB>>3];
           $1136 = $1135 + $1134;
           HEAPF64[$rB>>3] = $1136;
           break;
          }
          case 90:  {
           $1137 = +HEAPF64[$rA>>3];
           $1138 = +HEAPF64[$rB>>3];
           $1139 = $1138 - $1137;
           HEAPF64[$rB>>3] = $1139;
           break;
          }
          case 91:  {
           $1140 = +HEAPF64[$rA>>3];
           $1141 = +HEAPF64[$rB>>3];
           $1142 = $1141 * $1140;
           HEAPF64[$rB>>3] = $1142;
           break;
          }
          case 92:  {
           $1143 = +HEAPF64[$rA>>3];
           $1144 = $1143 == 0.0;
           if ($1144) {
            break L81;
           }
           $1145 = +HEAPF64[$rA>>3];
           $1146 = +HEAPF64[$rB>>3];
           $1147 = $1146 / $1145;
           HEAPF64[$rB>>3] = $1147;
           break;
          }
          default: {
           $1148 = +HEAPF64[$rA>>3];
           $1149 = (~~$1148)>>>0;
           $1150 = +Math_abs($1148) >= 1.0 ? $1148 > 0.0 ? (~~+Math_min(+Math_floor($1148 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($1148 - +(~~$1148 >>> 0)) / 4294967296.0) >>> 0 : 0;
           $1151 = $iA;
           $1152 = $1151;
           HEAP32[$1152>>2] = $1149;
           $1153 = (($1151) + 4)|0;
           $1154 = $1153;
           HEAP32[$1154>>2] = $1150;
           $1155 = +HEAPF64[$rB>>3];
           $1156 = (~~$1155)>>>0;
           $1157 = +Math_abs($1155) >= 1.0 ? $1155 > 0.0 ? (~~+Math_min(+Math_floor($1155 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($1155 - +(~~$1155 >>> 0)) / 4294967296.0) >>> 0 : 0;
           $1158 = $iB;
           $1159 = $1158;
           HEAP32[$1159>>2] = $1156;
           $1160 = (($1158) + 4)|0;
           $1161 = $1160;
           HEAP32[$1161>>2] = $1157;
           $1162 = $iA;
           $1163 = $1162;
           $1164 = HEAP32[$1163>>2]|0;
           $1165 = (($1162) + 4)|0;
           $1166 = $1165;
           $1167 = HEAP32[$1166>>2]|0;
           $1168 = ($1164|0)==(0);
           $1169 = ($1167|0)==(0);
           $1170 = $1168 & $1169;
           if ($1170) {
            break L81;
           }
           $1171 = $iA;
           $1172 = $1171;
           $1173 = HEAP32[$1172>>2]|0;
           $1174 = (($1171) + 4)|0;
           $1175 = $1174;
           $1176 = HEAP32[$1175>>2]|0;
           $1177 = ($1173|0)==(-1);
           $1178 = ($1176|0)==(-1);
           $1179 = $1177 & $1178;
           if ($1179) {
            $1180 = $iA;
            $1181 = $1180;
            HEAP32[$1181>>2] = 1;
            $1182 = (($1180) + 4)|0;
            $1183 = $1182;
            HEAP32[$1183>>2] = 0;
           }
           $1184 = $iB;
           $1185 = $1184;
           $1186 = HEAP32[$1185>>2]|0;
           $1187 = (($1184) + 4)|0;
           $1188 = $1187;
           $1189 = HEAP32[$1188>>2]|0;
           $1190 = $iA;
           $1191 = $1190;
           $1192 = HEAP32[$1191>>2]|0;
           $1193 = (($1190) + 4)|0;
           $1194 = $1193;
           $1195 = HEAP32[$1194>>2]|0;
           $1196 = (___remdi3(($1186|0),($1189|0),($1192|0),($1195|0))|0);
           $1197 = tempRet0;
           $1198 = (+($1196>>>0)) + (4294967296.0*(+($1197|0)));
           HEAPF64[$rB>>3] = $1198;
          }
          }
          $1199 = +HEAPF64[$rB>>3];
          $1200 = (_sqlite3IsNaN($1199)|0);
          $1201 = ($1200|0)!=(0);
          if (!($1201)) {
           $1202 = +HEAPF64[$rB>>3];
           $1203 = HEAP32[$pOut>>2]|0;
           HEAPF64[$1203>>3] = $1202;
           $1204 = HEAP32[$pOut>>2]|0;
           $1205 = ((($1204)) + 8|0);
           $1206 = HEAP16[$1205>>1]|0;
           $1207 = $1206&65535;
           $1208 = $1207 & -49664;
           $1209 = $1208 | 8;
           $1210 = $1209&65535;
           $1211 = HEAP32[$pOut>>2]|0;
           $1212 = ((($1211)) + 8|0);
           HEAP16[$1212>>1] = $1210;
           $1213 = HEAP16[$type1>>1]|0;
           $1214 = $1213&65535;
           $1215 = HEAP16[$type2>>1]|0;
           $1216 = $1215&65535;
           $1217 = $1214 | $1216;
           $1218 = $1217 & 8;
           $1219 = ($1218|0)!=(0);
           $1220 = HEAP8[$bIntint>>0]|0;
           $1221 = ($1220<<24>>24)!=(0);
           $or$cond3 = $1219 | $1221;
           if ($or$cond3) {
            break L11;
           }
           $1222 = HEAP32[$pOut>>2]|0;
           _sqlite3VdbeIntegerAffinity($1222);
           break L11;
          }
         }
        } while(0);
        $1223 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($1223);
        break;
       }
       case 34:  {
        $1224 = HEAP32[$pOp>>2]|0;
        $1225 = ((($1224)) + 4|0);
        $1226 = HEAP32[$1225>>2]|0;
        $1227 = ($1226|0)!=(0);
        if ($1227) {
         $1228 = HEAP32[$pOp>>2]|0;
         $1229 = ((($1228)) + 4|0);
         $1230 = HEAP32[$1229>>2]|0;
         $1231 = HEAP32[$aMem>>2]|0;
         $1232 = (($1231) + (($1230*40)|0)|0);
         _sqlite3VdbeMemSetInt64($1232,0,0);
        }
        break;
       }
       case 35:  {
        $1233 = HEAP32[$pOp>>2]|0;
        $1234 = ((($1233)) + 3|0);
        $1235 = HEAP8[$1234>>0]|0;
        $1236 = $1235&255;
        HEAP32[$n2>>2] = $1236;
        $1237 = HEAP32[$db>>2]|0;
        $1238 = HEAP32[$n2>>2]|0;
        $1239 = (($1238) - 1)|0;
        $1240 = $1239<<2;
        $1241 = (32 + ($1240))|0;
        $1242 = (_sqlite3DbMallocRawNN($1237,$1241,0)|0);
        HEAP32[$pCtx>>2] = $1242;
        $1243 = HEAP32[$pCtx>>2]|0;
        $1244 = ($1243|0)==(0|0);
        if ($1244) {
         label = 860;
         break L1;
        }
        $1245 = HEAP32[$pCtx>>2]|0;
        HEAP32[$1245>>2] = 0;
        $1246 = HEAP32[$pOp>>2]|0;
        $1247 = ((($1246)) + 16|0);
        $1248 = HEAP32[$1247>>2]|0;
        $1249 = HEAP32[$pCtx>>2]|0;
        $1250 = ((($1249)) + 4|0);
        HEAP32[$1250>>2] = $1248;
        $1251 = HEAP32[$pOp>>2]|0;
        $1252 = HEAP32[$aOp>>2]|0;
        $1253 = $1251;
        $1254 = $1252;
        $1255 = (($1253) - ($1254))|0;
        $1256 = (($1255|0) / 20)&-1;
        $1257 = HEAP32[$pCtx>>2]|0;
        $1258 = ((($1257)) + 16|0);
        HEAP32[$1258>>2] = $1256;
        $1259 = HEAP32[$0>>2]|0;
        $1260 = HEAP32[$pCtx>>2]|0;
        $1261 = ((($1260)) + 12|0);
        HEAP32[$1261>>2] = $1259;
        $1262 = HEAP32[$n2>>2]|0;
        $1263 = $1262&255;
        $1264 = HEAP32[$pCtx>>2]|0;
        $1265 = ((($1264)) + 26|0);
        HEAP8[$1265>>0] = $1263;
        $1266 = HEAP32[$pOp>>2]|0;
        $1267 = ((($1266)) + 1|0);
        HEAP8[$1267>>0] = -20;
        $1268 = HEAP32[$pCtx>>2]|0;
        $1269 = HEAP32[$pOp>>2]|0;
        $1270 = ((($1269)) + 16|0);
        HEAP32[$1270>>2] = $1268;
        $1271 = HEAP32[$pOp>>2]|0;
        HEAP8[$1271>>0] = 36;
        label = 132;
        break;
       }
       case 36:  {
        label = 132;
        break;
       }
       case 88: case 87: case 86: case 85:  {
        $1391 = HEAP32[$pOp>>2]|0;
        $1392 = ((($1391)) + 4|0);
        $1393 = HEAP32[$1392>>2]|0;
        $1394 = HEAP32[$aMem>>2]|0;
        $1395 = (($1394) + (($1393*40)|0)|0);
        HEAP32[$pIn1>>2] = $1395;
        $1396 = HEAP32[$pOp>>2]|0;
        $1397 = ((($1396)) + 8|0);
        $1398 = HEAP32[$1397>>2]|0;
        $1399 = HEAP32[$aMem>>2]|0;
        $1400 = (($1399) + (($1398*40)|0)|0);
        HEAP32[$pIn2>>2] = $1400;
        $1401 = HEAP32[$pOp>>2]|0;
        $1402 = ((($1401)) + 12|0);
        $1403 = HEAP32[$1402>>2]|0;
        $1404 = HEAP32[$aMem>>2]|0;
        $1405 = (($1404) + (($1403*40)|0)|0);
        HEAP32[$pOut>>2] = $1405;
        $1406 = HEAP32[$pIn1>>2]|0;
        $1407 = ((($1406)) + 8|0);
        $1408 = HEAP16[$1407>>1]|0;
        $1409 = $1408&65535;
        $1410 = HEAP32[$pIn2>>2]|0;
        $1411 = ((($1410)) + 8|0);
        $1412 = HEAP16[$1411>>1]|0;
        $1413 = $1412&65535;
        $1414 = $1409 | $1413;
        $1415 = $1414 & 1;
        $1416 = ($1415|0)!=(0);
        if ($1416) {
         $1417 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($1417);
         break L11;
        }
        $1418 = HEAP32[$pIn2>>2]|0;
        $1419 = (_sqlite3VdbeIntValue($1418)|0);
        $1420 = tempRet0;
        $1421 = $iA5;
        $1422 = $1421;
        HEAP32[$1422>>2] = $1419;
        $1423 = (($1421) + 4)|0;
        $1424 = $1423;
        HEAP32[$1424>>2] = $1420;
        $1425 = HEAP32[$pIn1>>2]|0;
        $1426 = (_sqlite3VdbeIntValue($1425)|0);
        $1427 = tempRet0;
        $1428 = $iB6;
        $1429 = $1428;
        HEAP32[$1429>>2] = $1426;
        $1430 = (($1428) + 4)|0;
        $1431 = $1430;
        HEAP32[$1431>>2] = $1427;
        $1432 = HEAP32[$pOp>>2]|0;
        $1433 = HEAP8[$1432>>0]|0;
        HEAP8[$op>>0] = $1433;
        $1434 = HEAP8[$op>>0]|0;
        $1435 = $1434&255;
        $1436 = ($1435|0)==(85);
        do {
         if ($1436) {
          $1437 = $iB6;
          $1438 = $1437;
          $1439 = HEAP32[$1438>>2]|0;
          $1440 = (($1437) + 4)|0;
          $1441 = $1440;
          $1442 = HEAP32[$1441>>2]|0;
          $1443 = $iA5;
          $1444 = $1443;
          $1445 = HEAP32[$1444>>2]|0;
          $1446 = (($1443) + 4)|0;
          $1447 = $1446;
          $1448 = HEAP32[$1447>>2]|0;
          $1449 = $1445 & $1439;
          $1450 = $1448 & $1442;
          $1451 = $iA5;
          $1452 = $1451;
          HEAP32[$1452>>2] = $1449;
          $1453 = (($1451) + 4)|0;
          $1454 = $1453;
          HEAP32[$1454>>2] = $1450;
         } else {
          $1455 = HEAP8[$op>>0]|0;
          $1456 = $1455&255;
          $1457 = ($1456|0)==(86);
          $1458 = $iB6;
          $1459 = $1458;
          $1460 = HEAP32[$1459>>2]|0;
          $1461 = (($1458) + 4)|0;
          $1462 = $1461;
          $1463 = HEAP32[$1462>>2]|0;
          if ($1457) {
           $1464 = $iA5;
           $1465 = $1464;
           $1466 = HEAP32[$1465>>2]|0;
           $1467 = (($1464) + 4)|0;
           $1468 = $1467;
           $1469 = HEAP32[$1468>>2]|0;
           $1470 = $1466 | $1460;
           $1471 = $1469 | $1463;
           $1472 = $iA5;
           $1473 = $1472;
           HEAP32[$1473>>2] = $1470;
           $1474 = (($1472) + 4)|0;
           $1475 = $1474;
           HEAP32[$1475>>2] = $1471;
           break;
          }
          $1476 = ($1460|0)!=(0);
          $1477 = ($1463|0)!=(0);
          $1478 = $1476 | $1477;
          if ($1478) {
           $1479 = $iB6;
           $1480 = $1479;
           $1481 = HEAP32[$1480>>2]|0;
           $1482 = (($1479) + 4)|0;
           $1483 = $1482;
           $1484 = HEAP32[$1483>>2]|0;
           $1485 = ($1484|0)<(0);
           if ($1485) {
            $1486 = HEAP8[$op>>0]|0;
            $1487 = $1486&255;
            $1488 = (175 - ($1487))|0;
            $1489 = $1488&255;
            HEAP8[$op>>0] = $1489;
            $1490 = $iB6;
            $1491 = $1490;
            $1492 = HEAP32[$1491>>2]|0;
            $1493 = (($1490) + 4)|0;
            $1494 = $1493;
            $1495 = HEAP32[$1494>>2]|0;
            $1496 = ($1495|0)>(-1);
            $1497 = ($1492>>>0)>(4294967232);
            $1498 = ($1495|0)==(-1);
            $1499 = $1498 & $1497;
            $1500 = $1496 | $1499;
            $1501 = $iB6;
            $1502 = $1501;
            $1503 = HEAP32[$1502>>2]|0;
            $1504 = (($1501) + 4)|0;
            $1505 = $1504;
            $1506 = HEAP32[$1505>>2]|0;
            $1507 = (_i64Subtract(0,0,($1503|0),($1506|0))|0);
            $1508 = tempRet0;
            $1509 = $1500 ? $1507 : 64;
            $1510 = $1500 ? $1508 : 0;
            $1511 = $iB6;
            $1512 = $1511;
            HEAP32[$1512>>2] = $1509;
            $1513 = (($1511) + 4)|0;
            $1514 = $1513;
            HEAP32[$1514>>2] = $1510;
           }
           $1515 = $iB6;
           $1516 = $1515;
           $1517 = HEAP32[$1516>>2]|0;
           $1518 = (($1515) + 4)|0;
           $1519 = $1518;
           $1520 = HEAP32[$1519>>2]|0;
           $1521 = ($1520|0)>(0);
           $1522 = ($1517>>>0)>=(64);
           $1523 = ($1520|0)==(0);
           $1524 = $1523 & $1522;
           $1525 = $1521 | $1524;
           if ($1525) {
            $1526 = $iA5;
            $1527 = $1526;
            $1528 = HEAP32[$1527>>2]|0;
            $1529 = (($1526) + 4)|0;
            $1530 = $1529;
            $1531 = HEAP32[$1530>>2]|0;
            $1532 = ($1531|0)>(0);
            $1533 = ($1528>>>0)>=(0);
            $1534 = ($1531|0)==(0);
            $1535 = $1534 & $1533;
            $1536 = $1532 | $1535;
            if ($1536) {
             $1541 = 1;
            } else {
             $1537 = HEAP8[$op>>0]|0;
             $1538 = $1537&255;
             $1539 = ($1538|0)==(87);
             $1541 = $1539;
            }
            $1540 = $1541 ? 0 : -1;
            $1542 = ($1540|0)<(0);
            $1543 = $1542 << 31 >> 31;
            $1544 = $iA5;
            $1545 = $1544;
            HEAP32[$1545>>2] = $1540;
            $1546 = (($1544) + 4)|0;
            $1547 = $1546;
            HEAP32[$1547>>2] = $1543;
            break;
           }
           ;HEAP32[$uA>>2]=HEAP32[$iA5>>2]|0;HEAP32[$uA+4>>2]=HEAP32[$iA5+4>>2]|0;
           $1548 = HEAP8[$op>>0]|0;
           $1549 = $1548&255;
           $1550 = ($1549|0)==(87);
           $1551 = $iB6;
           $1552 = $1551;
           $1553 = HEAP32[$1552>>2]|0;
           $1554 = (($1551) + 4)|0;
           $1555 = $1554;
           $1556 = HEAP32[$1555>>2]|0;
           $1557 = $uA;
           $1558 = $1557;
           $1559 = HEAP32[$1558>>2]|0;
           $1560 = (($1557) + 4)|0;
           $1561 = $1560;
           $1562 = HEAP32[$1561>>2]|0;
           if ($1550) {
            $1563 = (_bitshift64Shl(($1559|0),($1562|0),($1553|0))|0);
            $1564 = tempRet0;
            $1565 = $uA;
            $1566 = $1565;
            HEAP32[$1566>>2] = $1563;
            $1567 = (($1565) + 4)|0;
            $1568 = $1567;
            HEAP32[$1568>>2] = $1564;
           } else {
            $1569 = (_bitshift64Lshr(($1559|0),($1562|0),($1553|0))|0);
            $1570 = tempRet0;
            $1571 = $uA;
            $1572 = $1571;
            HEAP32[$1572>>2] = $1569;
            $1573 = (($1571) + 4)|0;
            $1574 = $1573;
            HEAP32[$1574>>2] = $1570;
            $1575 = $iA5;
            $1576 = $1575;
            $1577 = HEAP32[$1576>>2]|0;
            $1578 = (($1575) + 4)|0;
            $1579 = $1578;
            $1580 = HEAP32[$1579>>2]|0;
            $1581 = ($1580|0)<(0);
            if ($1581) {
             $1582 = $iB6;
             $1583 = $1582;
             $1584 = HEAP32[$1583>>2]|0;
             $1585 = (($1582) + 4)|0;
             $1586 = $1585;
             $1587 = HEAP32[$1586>>2]|0;
             $1588 = (_i64Subtract(64,0,($1584|0),($1587|0))|0);
             $1589 = tempRet0;
             $1590 = (_bitshift64Shl(-1,-1,($1588|0))|0);
             $1591 = tempRet0;
             $1592 = $uA;
             $1593 = $1592;
             $1594 = HEAP32[$1593>>2]|0;
             $1595 = (($1592) + 4)|0;
             $1596 = $1595;
             $1597 = HEAP32[$1596>>2]|0;
             $1598 = $1594 | $1590;
             $1599 = $1597 | $1591;
             $1600 = $uA;
             $1601 = $1600;
             HEAP32[$1601>>2] = $1598;
             $1602 = (($1600) + 4)|0;
             $1603 = $1602;
             HEAP32[$1603>>2] = $1599;
            }
           }
           ;HEAP32[$iA5>>2]=HEAP32[$uA>>2]|0;HEAP32[$iA5+4>>2]=HEAP32[$uA+4>>2]|0;
          }
         }
        } while(0);
        $1604 = $iA5;
        $1605 = $1604;
        $1606 = HEAP32[$1605>>2]|0;
        $1607 = (($1604) + 4)|0;
        $1608 = $1607;
        $1609 = HEAP32[$1608>>2]|0;
        $1610 = HEAP32[$pOut>>2]|0;
        $1611 = $1610;
        $1612 = $1611;
        HEAP32[$1612>>2] = $1606;
        $1613 = (($1611) + 4)|0;
        $1614 = $1613;
        HEAP32[$1614>>2] = $1609;
        $1615 = HEAP32[$pOut>>2]|0;
        $1616 = ((($1615)) + 8|0);
        $1617 = HEAP16[$1616>>1]|0;
        $1618 = $1617&65535;
        $1619 = $1618 & -49664;
        $1620 = $1619 | 4;
        $1621 = $1620&65535;
        $1622 = HEAP32[$pOut>>2]|0;
        $1623 = ((($1622)) + 8|0);
        HEAP16[$1623>>1] = $1621;
        break;
       }
       case 37:  {
        $1624 = HEAP32[$pOp>>2]|0;
        $1625 = ((($1624)) + 4|0);
        $1626 = HEAP32[$1625>>2]|0;
        $1627 = HEAP32[$aMem>>2]|0;
        $1628 = (($1627) + (($1626*40)|0)|0);
        HEAP32[$pIn1>>2] = $1628;
        $1629 = HEAP32[$pIn1>>2]|0;
        (_sqlite3VdbeMemIntegerify($1629)|0);
        $1630 = HEAP32[$pOp>>2]|0;
        $1631 = ((($1630)) + 8|0);
        $1632 = HEAP32[$1631>>2]|0;
        $1633 = ($1632|0)<(0);
        $1634 = $1633 << 31 >> 31;
        $1635 = HEAP32[$pIn1>>2]|0;
        $1636 = $1635;
        $1637 = $1636;
        $1638 = HEAP32[$1637>>2]|0;
        $1639 = (($1636) + 4)|0;
        $1640 = $1639;
        $1641 = HEAP32[$1640>>2]|0;
        $1642 = (_i64Add(($1638|0),($1641|0),($1632|0),($1634|0))|0);
        $1643 = tempRet0;
        $1644 = $1635;
        $1645 = $1644;
        HEAP32[$1645>>2] = $1642;
        $1646 = (($1644) + 4)|0;
        $1647 = $1646;
        HEAP32[$1647>>2] = $1643;
        break;
       }
       case 38:  {
        $1648 = HEAP32[$pOp>>2]|0;
        $1649 = ((($1648)) + 4|0);
        $1650 = HEAP32[$1649>>2]|0;
        $1651 = HEAP32[$aMem>>2]|0;
        $1652 = (($1651) + (($1650*40)|0)|0);
        HEAP32[$pIn1>>2] = $1652;
        $1653 = HEAP32[$pIn1>>2]|0;
        $1654 = ((($1653)) + 8|0);
        $1655 = HEAP16[$1654>>1]|0;
        $1656 = $1655&65535;
        $1657 = $1656 & 4;
        $1658 = ($1657|0)==(0);
        if ($1658) {
         $1659 = HEAP32[$pIn1>>2]|0;
         $1660 = HEAP8[$encoding>>0]|0;
         _applyAffinity($1659,67,$1660);
         $1661 = HEAP32[$pIn1>>2]|0;
         $1662 = ((($1661)) + 8|0);
         $1663 = HEAP16[$1662>>1]|0;
         $1664 = $1663&65535;
         $1665 = $1664 & 4;
         $1666 = ($1665|0)==(0);
         if ($1666) {
          $1667 = HEAP32[$pOp>>2]|0;
          $1668 = ((($1667)) + 8|0);
          $1669 = HEAP32[$1668>>2]|0;
          $1670 = ($1669|0)==(0);
          if ($1670) {
           label = 165;
           break L8;
          } else {
           label = 15;
           break L11;
          }
         }
        }
        $1671 = HEAP32[$pIn1>>2]|0;
        $1672 = ((($1671)) + 8|0);
        $1673 = HEAP16[$1672>>1]|0;
        $1674 = $1673&65535;
        $1675 = $1674 & -49664;
        $1676 = $1675 | 4;
        $1677 = $1676&65535;
        $1678 = HEAP32[$pIn1>>2]|0;
        $1679 = ((($1678)) + 8|0);
        HEAP16[$1679>>1] = $1677;
        break;
       }
       case 39:  {
        $1680 = HEAP32[$pOp>>2]|0;
        $1681 = ((($1680)) + 4|0);
        $1682 = HEAP32[$1681>>2]|0;
        $1683 = HEAP32[$aMem>>2]|0;
        $1684 = (($1683) + (($1682*40)|0)|0);
        HEAP32[$pIn1>>2] = $1684;
        $1685 = HEAP32[$pIn1>>2]|0;
        $1686 = ((($1685)) + 8|0);
        $1687 = HEAP16[$1686>>1]|0;
        $1688 = $1687&65535;
        $1689 = $1688 & 4;
        $1690 = ($1689|0)!=(0);
        if ($1690) {
         $1691 = HEAP32[$pIn1>>2]|0;
         (_sqlite3VdbeMemRealify($1691)|0);
        }
        break;
       }
       case 40:  {
        $1692 = HEAP32[$pOp>>2]|0;
        $1693 = ((($1692)) + 4|0);
        $1694 = HEAP32[$1693>>2]|0;
        $1695 = HEAP32[$aMem>>2]|0;
        $1696 = (($1695) + (($1694*40)|0)|0);
        HEAP32[$pIn1>>2] = $1696;
        $1697 = HEAP32[$pIn1>>2]|0;
        $1698 = ((($1697)) + 8|0);
        $1699 = HEAP16[$1698>>1]|0;
        $1700 = $1699&65535;
        $1701 = $1700 & 16384;
        $1702 = ($1701|0)!=(0);
        if ($1702) {
         $1703 = HEAP32[$pIn1>>2]|0;
         $1704 = (_sqlite3VdbeMemExpandBlob($1703)|0);
         $1705 = $1704;
        } else {
         $1705 = 0;
        }
        HEAP32[$rc>>2] = $1705;
        $1706 = HEAP32[$pIn1>>2]|0;
        $1707 = HEAP32[$pOp>>2]|0;
        $1708 = ((($1707)) + 8|0);
        $1709 = HEAP32[$1708>>2]|0;
        $1710 = $1709&255;
        $1711 = HEAP8[$encoding>>0]|0;
        _sqlite3VdbeMemCast($1706,$1710,$1711);
        break;
       }
       case 83: case 80: case 81: case 82: case 78: case 79:  {
        $1712 = HEAP32[$pOp>>2]|0;
        $1713 = ((($1712)) + 4|0);
        $1714 = HEAP32[$1713>>2]|0;
        $1715 = HEAP32[$aMem>>2]|0;
        $1716 = (($1715) + (($1714*40)|0)|0);
        HEAP32[$pIn1>>2] = $1716;
        $1717 = HEAP32[$pOp>>2]|0;
        $1718 = ((($1717)) + 12|0);
        $1719 = HEAP32[$1718>>2]|0;
        $1720 = HEAP32[$aMem>>2]|0;
        $1721 = (($1720) + (($1719*40)|0)|0);
        HEAP32[$pIn3>>2] = $1721;
        $1722 = HEAP32[$pIn1>>2]|0;
        $1723 = ((($1722)) + 8|0);
        $1724 = HEAP16[$1723>>1]|0;
        HEAP16[$flags1>>1] = $1724;
        $1725 = HEAP32[$pIn3>>2]|0;
        $1726 = ((($1725)) + 8|0);
        $1727 = HEAP16[$1726>>1]|0;
        HEAP16[$flags3>>1] = $1727;
        $1728 = HEAP16[$flags1>>1]|0;
        $1729 = $1728&65535;
        $1730 = HEAP16[$flags3>>1]|0;
        $1731 = $1730&65535;
        $1732 = $1729 | $1731;
        $1733 = $1732 & 1;
        $1734 = ($1733|0)!=(0);
        $1735 = HEAP32[$pOp>>2]|0;
        $1736 = ((($1735)) + 3|0);
        $1737 = HEAP8[$1736>>0]|0;
        $1738 = $1737&255;
        do {
         if ($1734) {
          $1739 = $1738 & 128;
          $1740 = ($1739|0)!=(0);
          if (!($1740)) {
           $1753 = HEAP32[$pOp>>2]|0;
           $1754 = ((($1753)) + 3|0);
           $1755 = HEAP8[$1754>>0]|0;
           $1756 = $1755&255;
           $1757 = $1756 & 32;
           $1758 = ($1757|0)!=(0);
           $1759 = HEAP32[$pOp>>2]|0;
           if ($1758) {
            $1760 = ((($1759)) + 8|0);
            $1761 = HEAP32[$1760>>2]|0;
            $1762 = HEAP32[$aMem>>2]|0;
            $1763 = (($1762) + (($1761*40)|0)|0);
            HEAP32[$pOut>>2] = $1763;
            $1764 = HEAP32[$pOut>>2]|0;
            $1765 = ((($1764)) + 8|0);
            $1766 = HEAP16[$1765>>1]|0;
            $1767 = $1766&65535;
            $1768 = $1767 & -49664;
            $1769 = $1768 | 1;
            $1770 = $1769&65535;
            $1771 = HEAP32[$pOut>>2]|0;
            $1772 = ((($1771)) + 8|0);
            HEAP16[$1772>>1] = $1770;
            break L11;
           } else {
            $1773 = ((($1759)) + 3|0);
            $1774 = HEAP8[$1773>>0]|0;
            $1775 = $1774&255;
            $1776 = $1775 & 16;
            $1777 = ($1776|0)!=(0);
            if ($1777) {
             label = 15;
             break L11;
            } else {
             break L11;
            }
           }
          }
          $1741 = HEAP16[$flags1>>1]|0;
          $1742 = $1741&65535;
          $1743 = $1742 & 1;
          $1744 = ($1743|0)!=(0);
          if ($1744) {
           $1745 = HEAP16[$flags3>>1]|0;
           $1746 = $1745&65535;
           $1747 = $1746 & 1;
           $1748 = ($1747|0)!=(0);
           if ($1748) {
            $1749 = HEAP16[$flags3>>1]|0;
            $1750 = $1749&65535;
            $1751 = $1750 & 256;
            $1752 = ($1751|0)==(0);
            if ($1752) {
             HEAP32[$res>>2] = 0;
             break;
            }
           }
          }
          HEAP32[$res>>2] = 1;
         } else {
          $1778 = $1738 & 71;
          $1779 = $1778&255;
          HEAP8[$affinity>>0] = $1779;
          $1780 = HEAP8[$affinity>>0]|0;
          $1781 = $1780 << 24 >> 24;
          $1782 = ($1781|0)>=(67);
          if ($1782) {
           $1783 = HEAP16[$flags1>>1]|0;
           $1784 = $1783&65535;
           $1785 = $1784 & 14;
           $1786 = ($1785|0)==(2);
           if ($1786) {
            $1787 = HEAP32[$pIn1>>2]|0;
            _applyNumericAffinity($1787,0);
           }
           $1788 = HEAP16[$flags3>>1]|0;
           $1789 = $1788&65535;
           $1790 = $1789 & 14;
           $1791 = ($1790|0)==(2);
           if ($1791) {
            $1792 = HEAP32[$pIn3>>2]|0;
            _applyNumericAffinity($1792,0);
           }
          } else {
           $1793 = HEAP8[$affinity>>0]|0;
           $1794 = $1793 << 24 >> 24;
           $1795 = ($1794|0)==(66);
           if ($1795) {
            $1796 = HEAP16[$flags1>>1]|0;
            $1797 = $1796&65535;
            $1798 = $1797 & 2;
            $1799 = ($1798|0)==(0);
            if ($1799) {
             $1800 = HEAP16[$flags1>>1]|0;
             $1801 = $1800&65535;
             $1802 = $1801 & 12;
             $1803 = ($1802|0)!=(0);
             if ($1803) {
              $1804 = HEAP32[$pIn1>>2]|0;
              $1805 = HEAP8[$encoding>>0]|0;
              (_sqlite3VdbeMemStringify($1804,$1805,1)|0);
              $1806 = HEAP32[$pIn1>>2]|0;
              $1807 = ((($1806)) + 8|0);
              $1808 = HEAP16[$1807>>1]|0;
              $1809 = $1808&65535;
              $1810 = $1809 & -33280;
              $1811 = HEAP16[$flags1>>1]|0;
              $1812 = $1811&65535;
              $1813 = $1812 & 33279;
              $1814 = $1810 | $1813;
              $1815 = $1814&65535;
              HEAP16[$flags1>>1] = $1815;
             }
            }
            $1816 = HEAP16[$flags3>>1]|0;
            $1817 = $1816&65535;
            $1818 = $1817 & 2;
            $1819 = ($1818|0)==(0);
            if ($1819) {
             $1820 = HEAP16[$flags3>>1]|0;
             $1821 = $1820&65535;
             $1822 = $1821 & 12;
             $1823 = ($1822|0)!=(0);
             if ($1823) {
              $1824 = HEAP32[$pIn3>>2]|0;
              $1825 = HEAP8[$encoding>>0]|0;
              (_sqlite3VdbeMemStringify($1824,$1825,1)|0);
              $1826 = HEAP32[$pIn3>>2]|0;
              $1827 = ((($1826)) + 8|0);
              $1828 = HEAP16[$1827>>1]|0;
              $1829 = $1828&65535;
              $1830 = $1829 & -33280;
              $1831 = HEAP16[$flags3>>1]|0;
              $1832 = $1831&65535;
              $1833 = $1832 & 33279;
              $1834 = $1830 | $1833;
              $1835 = $1834&65535;
              HEAP16[$flags3>>1] = $1835;
             }
            }
           }
          }
          $1836 = HEAP16[$flags1>>1]|0;
          $1837 = $1836&65535;
          $1838 = $1837 & 16384;
          $1839 = ($1838|0)!=(0);
          if ($1839) {
           $1840 = HEAP32[$pIn1>>2]|0;
           (_sqlite3VdbeMemExpandBlob($1840)|0);
           $1841 = HEAP16[$flags1>>1]|0;
           $1842 = $1841&65535;
           $1843 = $1842 & -16385;
           $1844 = $1843&65535;
           HEAP16[$flags1>>1] = $1844;
          }
          $1845 = HEAP16[$flags3>>1]|0;
          $1846 = $1845&65535;
          $1847 = $1846 & 16384;
          $1848 = ($1847|0)!=(0);
          if ($1848) {
           $1849 = HEAP32[$pIn3>>2]|0;
           (_sqlite3VdbeMemExpandBlob($1849)|0);
           $1850 = HEAP16[$flags3>>1]|0;
           $1851 = $1850&65535;
           $1852 = $1851 & -16385;
           $1853 = $1852&65535;
           HEAP16[$flags3>>1] = $1853;
          }
          $1854 = HEAP32[$pIn3>>2]|0;
          $1855 = HEAP32[$pIn1>>2]|0;
          $1856 = HEAP32[$pOp>>2]|0;
          $1857 = ((($1856)) + 16|0);
          $1858 = HEAP32[$1857>>2]|0;
          $1859 = (_sqlite3MemCompare($1854,$1855,$1858)|0);
          HEAP32[$res>>2] = $1859;
         }
        } while(0);
        $1860 = HEAP32[$pOp>>2]|0;
        $1861 = HEAP8[$1860>>0]|0;
        $1862 = $1861&255;
        switch ($1862|0) {
        case 79:  {
         $1863 = HEAP32[$res>>2]|0;
         $1864 = ($1863|0)==(0);
         $1865 = $1864&1;
         HEAP32[$res>>2] = $1865;
         break;
        }
        case 78:  {
         $1866 = HEAP32[$res>>2]|0;
         $1867 = ($1866|0)!=(0);
         $1868 = $1867&1;
         HEAP32[$res>>2] = $1868;
         break;
        }
        case 82:  {
         $1869 = HEAP32[$res>>2]|0;
         $1870 = ($1869|0)<(0);
         $1871 = $1870&1;
         HEAP32[$res>>2] = $1871;
         break;
        }
        case 81:  {
         $1872 = HEAP32[$res>>2]|0;
         $1873 = ($1872|0)<=(0);
         $1874 = $1873&1;
         HEAP32[$res>>2] = $1874;
         break;
        }
        case 80:  {
         $1875 = HEAP32[$res>>2]|0;
         $1876 = ($1875|0)>(0);
         $1877 = $1876&1;
         HEAP32[$res>>2] = $1877;
         break;
        }
        default: {
         $1878 = HEAP32[$res>>2]|0;
         $1879 = ($1878|0)>=(0);
         $1880 = $1879&1;
         HEAP32[$res>>2] = $1880;
        }
        }
        $1881 = HEAP16[$flags1>>1]|0;
        $1882 = HEAP32[$pIn1>>2]|0;
        $1883 = ((($1882)) + 8|0);
        HEAP16[$1883>>1] = $1881;
        $1884 = HEAP16[$flags3>>1]|0;
        $1885 = HEAP32[$pIn3>>2]|0;
        $1886 = ((($1885)) + 8|0);
        HEAP16[$1886>>1] = $1884;
        $1887 = HEAP32[$pOp>>2]|0;
        $1888 = ((($1887)) + 3|0);
        $1889 = HEAP8[$1888>>0]|0;
        $1890 = $1889&255;
        $1891 = $1890 & 32;
        $1892 = ($1891|0)!=(0);
        if ($1892) {
         $1893 = HEAP32[$pOp>>2]|0;
         $1894 = ((($1893)) + 8|0);
         $1895 = HEAP32[$1894>>2]|0;
         $1896 = HEAP32[$aMem>>2]|0;
         $1897 = (($1896) + (($1895*40)|0)|0);
         HEAP32[$pOut>>2] = $1897;
         $1898 = HEAP32[$pOut>>2]|0;
         $1899 = ((($1898)) + 8|0);
         $1900 = HEAP16[$1899>>1]|0;
         $1901 = $1900&65535;
         $1902 = $1901 & -49664;
         $1903 = $1902 | 4;
         $1904 = $1903&65535;
         $1905 = HEAP32[$pOut>>2]|0;
         $1906 = ((($1905)) + 8|0);
         HEAP16[$1906>>1] = $1904;
         $1907 = HEAP32[$res>>2]|0;
         $1908 = ($1907|0)<(0);
         $1909 = $1908 << 31 >> 31;
         $1910 = HEAP32[$pOut>>2]|0;
         $1911 = $1910;
         $1912 = $1911;
         HEAP32[$1912>>2] = $1907;
         $1913 = (($1911) + 4)|0;
         $1914 = $1913;
         HEAP32[$1914>>2] = $1909;
         break L11;
        } else {
         $1915 = HEAP32[$res>>2]|0;
         $1916 = ($1915|0)!=(0);
         if ($1916) {
          label = 15;
          break L11;
         } else {
          break L11;
         }
        }
        break;
       }
       case 41:  {
        $1917 = HEAP32[$pOp>>2]|0;
        $1918 = ((($1917)) + 16|0);
        $1919 = HEAP32[$1918>>2]|0;
        $1920 = ((($1919)) + 4|0);
        HEAP32[$aPermute>>2] = $1920;
        break;
       }
       case 42:  {
        $1921 = HEAP32[$pOp>>2]|0;
        $1922 = ((($1921)) + 3|0);
        $1923 = HEAP8[$1922>>0]|0;
        $1924 = $1923&255;
        $1925 = $1924 & 1;
        $1926 = ($1925|0)==(0);
        if ($1926) {
         HEAP32[$aPermute>>2] = 0;
        }
        $1927 = HEAP32[$pOp>>2]|0;
        $1928 = ((($1927)) + 12|0);
        $1929 = HEAP32[$1928>>2]|0;
        HEAP32[$n7>>2] = $1929;
        $1930 = HEAP32[$pOp>>2]|0;
        $1931 = ((($1930)) + 16|0);
        $1932 = HEAP32[$1931>>2]|0;
        HEAP32[$pKeyInfo>>2] = $1932;
        $1933 = HEAP32[$pOp>>2]|0;
        $1934 = ((($1933)) + 4|0);
        $1935 = HEAP32[$1934>>2]|0;
        HEAP32[$p19>>2] = $1935;
        $1936 = HEAP32[$pOp>>2]|0;
        $1937 = ((($1936)) + 8|0);
        $1938 = HEAP32[$1937>>2]|0;
        HEAP32[$p210>>2] = $1938;
        HEAP32[$i8>>2] = 0;
        while(1) {
         $1939 = HEAP32[$i8>>2]|0;
         $1940 = HEAP32[$n7>>2]|0;
         $1941 = ($1939|0)<($1940|0);
         if (!($1941)) {
          break;
         }
         $1942 = HEAP32[$aPermute>>2]|0;
         $1943 = ($1942|0)!=(0|0);
         $1944 = HEAP32[$i8>>2]|0;
         if ($1943) {
          $1945 = HEAP32[$aPermute>>2]|0;
          $1946 = (($1945) + ($1944<<2)|0);
          $1947 = HEAP32[$1946>>2]|0;
          $1948 = $1947;
         } else {
          $1948 = $1944;
         }
         HEAP32[$idx>>2] = $1948;
         $1949 = HEAP32[$i8>>2]|0;
         $1950 = HEAP32[$pKeyInfo>>2]|0;
         $1951 = ((($1950)) + 20|0);
         $1952 = (($1951) + ($1949<<2)|0);
         $1953 = HEAP32[$1952>>2]|0;
         HEAP32[$pColl>>2] = $1953;
         $1954 = HEAP32[$i8>>2]|0;
         $1955 = HEAP32[$pKeyInfo>>2]|0;
         $1956 = ((($1955)) + 16|0);
         $1957 = HEAP32[$1956>>2]|0;
         $1958 = (($1957) + ($1954)|0);
         $1959 = HEAP8[$1958>>0]|0;
         $1960 = $1959&255;
         HEAP32[$bRev>>2] = $1960;
         $1961 = HEAP32[$p19>>2]|0;
         $1962 = HEAP32[$idx>>2]|0;
         $1963 = (($1961) + ($1962))|0;
         $1964 = HEAP32[$aMem>>2]|0;
         $1965 = (($1964) + (($1963*40)|0)|0);
         $1966 = HEAP32[$p210>>2]|0;
         $1967 = HEAP32[$idx>>2]|0;
         $1968 = (($1966) + ($1967))|0;
         $1969 = HEAP32[$aMem>>2]|0;
         $1970 = (($1969) + (($1968*40)|0)|0);
         $1971 = HEAP32[$pColl>>2]|0;
         $1972 = (_sqlite3MemCompare($1965,$1970,$1971)|0);
         HEAP32[$iCompare>>2] = $1972;
         $1973 = HEAP32[$iCompare>>2]|0;
         $1974 = ($1973|0)!=(0);
         if ($1974) {
          label = 217;
          break;
         }
         $1979 = HEAP32[$i8>>2]|0;
         $1980 = (($1979) + 1)|0;
         HEAP32[$i8>>2] = $1980;
        }
        if ((label|0) == 217) {
         label = 0;
         $1975 = HEAP32[$bRev>>2]|0;
         $1976 = ($1975|0)!=(0);
         if ($1976) {
          $1977 = HEAP32[$iCompare>>2]|0;
          $1978 = (0 - ($1977))|0;
          HEAP32[$iCompare>>2] = $1978;
         }
        }
        HEAP32[$aPermute>>2] = 0;
        break;
       }
       case 43:  {
        $1981 = HEAP32[$iCompare>>2]|0;
        $1982 = ($1981|0)<(0);
        if ($1982) {
         $1983 = HEAP32[$pOp>>2]|0;
         $1984 = ((($1983)) + 4|0);
         $1985 = HEAP32[$1984>>2]|0;
         $1986 = (($1985) - 1)|0;
         $1987 = HEAP32[$aOp>>2]|0;
         $1988 = (($1987) + (($1986*20)|0)|0);
         HEAP32[$pOp>>2] = $1988;
         break L11;
        }
        $1989 = HEAP32[$iCompare>>2]|0;
        $1990 = ($1989|0)==(0);
        $1991 = HEAP32[$pOp>>2]|0;
        if ($1990) {
         $1992 = ((($1991)) + 8|0);
         $1993 = HEAP32[$1992>>2]|0;
         $1994 = (($1993) - 1)|0;
         $1995 = HEAP32[$aOp>>2]|0;
         $1996 = (($1995) + (($1994*20)|0)|0);
         HEAP32[$pOp>>2] = $1996;
         break L11;
        } else {
         $1997 = ((($1991)) + 12|0);
         $1998 = HEAP32[$1997>>2]|0;
         $1999 = (($1998) - 1)|0;
         $2000 = HEAP32[$aOp>>2]|0;
         $2001 = (($2000) + (($1999*20)|0)|0);
         HEAP32[$pOp>>2] = $2001;
         break L11;
        }
        break;
       }
       case 71: case 72:  {
        $2002 = HEAP32[$pOp>>2]|0;
        $2003 = ((($2002)) + 4|0);
        $2004 = HEAP32[$2003>>2]|0;
        $2005 = HEAP32[$aMem>>2]|0;
        $2006 = (($2005) + (($2004*40)|0)|0);
        HEAP32[$pIn1>>2] = $2006;
        $2007 = HEAP32[$pIn1>>2]|0;
        $2008 = ((($2007)) + 8|0);
        $2009 = HEAP16[$2008>>1]|0;
        $2010 = $2009&65535;
        $2011 = $2010 & 1;
        $2012 = ($2011|0)!=(0);
        if ($2012) {
         HEAP32[$v1>>2] = 2;
        } else {
         $2013 = HEAP32[$pIn1>>2]|0;
         $2014 = (_sqlite3VdbeIntValue($2013)|0);
         $2015 = tempRet0;
         $2016 = ($2014|0)!=(0);
         $2017 = ($2015|0)!=(0);
         $2018 = $2016 | $2017;
         $2019 = $2018&1;
         HEAP32[$v1>>2] = $2019;
        }
        $2020 = HEAP32[$pOp>>2]|0;
        $2021 = ((($2020)) + 8|0);
        $2022 = HEAP32[$2021>>2]|0;
        $2023 = HEAP32[$aMem>>2]|0;
        $2024 = (($2023) + (($2022*40)|0)|0);
        HEAP32[$pIn2>>2] = $2024;
        $2025 = HEAP32[$pIn2>>2]|0;
        $2026 = ((($2025)) + 8|0);
        $2027 = HEAP16[$2026>>1]|0;
        $2028 = $2027&65535;
        $2029 = $2028 & 1;
        $2030 = ($2029|0)!=(0);
        if ($2030) {
         HEAP32[$v2>>2] = 2;
        } else {
         $2031 = HEAP32[$pIn2>>2]|0;
         $2032 = (_sqlite3VdbeIntValue($2031)|0);
         $2033 = tempRet0;
         $2034 = ($2032|0)!=(0);
         $2035 = ($2033|0)!=(0);
         $2036 = $2034 | $2035;
         $2037 = $2036&1;
         HEAP32[$v2>>2] = $2037;
        }
        $2038 = HEAP32[$pOp>>2]|0;
        $2039 = HEAP8[$2038>>0]|0;
        $2040 = $2039&255;
        $2041 = ($2040|0)==(72);
        $2042 = HEAP32[$v1>>2]|0;
        $2043 = ($2042*3)|0;
        $2044 = HEAP32[$v2>>2]|0;
        $2045 = (($2043) + ($2044))|0;
        if ($2041) {
         $2046 = (20892 + ($2045)|0);
         $2047 = HEAP8[$2046>>0]|0;
         $2048 = $2047&255;
         HEAP32[$v1>>2] = $2048;
        } else {
         $2049 = (20901 + ($2045)|0);
         $2050 = HEAP8[$2049>>0]|0;
         $2051 = $2050&255;
         HEAP32[$v1>>2] = $2051;
        }
        $2052 = HEAP32[$pOp>>2]|0;
        $2053 = ((($2052)) + 12|0);
        $2054 = HEAP32[$2053>>2]|0;
        $2055 = HEAP32[$aMem>>2]|0;
        $2056 = (($2055) + (($2054*40)|0)|0);
        HEAP32[$pOut>>2] = $2056;
        $2057 = HEAP32[$v1>>2]|0;
        $2058 = ($2057|0)==(2);
        if ($2058) {
         $2059 = HEAP32[$pOut>>2]|0;
         $2060 = ((($2059)) + 8|0);
         $2061 = HEAP16[$2060>>1]|0;
         $2062 = $2061&65535;
         $2063 = $2062 & -49664;
         $2064 = $2063 | 1;
         $2065 = $2064&65535;
         $2066 = HEAP32[$pOut>>2]|0;
         $2067 = ((($2066)) + 8|0);
         HEAP16[$2067>>1] = $2065;
         break L11;
        } else {
         $2068 = HEAP32[$v1>>2]|0;
         $2069 = ($2068|0)<(0);
         $2070 = $2069 << 31 >> 31;
         $2071 = HEAP32[$pOut>>2]|0;
         $2072 = $2071;
         $2073 = $2072;
         HEAP32[$2073>>2] = $2068;
         $2074 = (($2072) + 4)|0;
         $2075 = $2074;
         HEAP32[$2075>>2] = $2070;
         $2076 = HEAP32[$pOut>>2]|0;
         $2077 = ((($2076)) + 8|0);
         $2078 = HEAP16[$2077>>1]|0;
         $2079 = $2078&65535;
         $2080 = $2079 & -49664;
         $2081 = $2080 | 4;
         $2082 = $2081&65535;
         $2083 = HEAP32[$pOut>>2]|0;
         $2084 = ((($2083)) + 8|0);
         HEAP16[$2084>>1] = $2082;
         break L11;
        }
        break;
       }
       case 19:  {
        $2085 = HEAP32[$pOp>>2]|0;
        $2086 = ((($2085)) + 4|0);
        $2087 = HEAP32[$2086>>2]|0;
        $2088 = HEAP32[$aMem>>2]|0;
        $2089 = (($2088) + (($2087*40)|0)|0);
        HEAP32[$pIn1>>2] = $2089;
        $2090 = HEAP32[$pOp>>2]|0;
        $2091 = ((($2090)) + 8|0);
        $2092 = HEAP32[$2091>>2]|0;
        $2093 = HEAP32[$aMem>>2]|0;
        $2094 = (($2093) + (($2092*40)|0)|0);
        HEAP32[$pOut>>2] = $2094;
        $2095 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($2095);
        $2096 = HEAP32[$pIn1>>2]|0;
        $2097 = ((($2096)) + 8|0);
        $2098 = HEAP16[$2097>>1]|0;
        $2099 = $2098&65535;
        $2100 = $2099 & 1;
        $2101 = ($2100|0)==(0);
        if ($2101) {
         $2102 = HEAP32[$pOut>>2]|0;
         $2103 = ((($2102)) + 8|0);
         HEAP16[$2103>>1] = 4;
         $2104 = HEAP32[$pIn1>>2]|0;
         $2105 = (_sqlite3VdbeIntValue($2104)|0);
         $2106 = tempRet0;
         $2107 = ($2105|0)!=(0);
         $2108 = ($2106|0)!=(0);
         $2109 = $2107 | $2108;
         $2110 = $2109 ^ 1;
         $2111 = $2110&1;
         $2112 = ($2111|0)<(0);
         $2113 = $2112 << 31 >> 31;
         $2114 = HEAP32[$pOut>>2]|0;
         $2115 = $2114;
         $2116 = $2115;
         HEAP32[$2116>>2] = $2111;
         $2117 = (($2115) + 4)|0;
         $2118 = $2117;
         HEAP32[$2118>>2] = $2113;
        }
        break;
       }
       case 96:  {
        $2119 = HEAP32[$pOp>>2]|0;
        $2120 = ((($2119)) + 4|0);
        $2121 = HEAP32[$2120>>2]|0;
        $2122 = HEAP32[$aMem>>2]|0;
        $2123 = (($2122) + (($2121*40)|0)|0);
        HEAP32[$pIn1>>2] = $2123;
        $2124 = HEAP32[$pOp>>2]|0;
        $2125 = ((($2124)) + 8|0);
        $2126 = HEAP32[$2125>>2]|0;
        $2127 = HEAP32[$aMem>>2]|0;
        $2128 = (($2127) + (($2126*40)|0)|0);
        HEAP32[$pOut>>2] = $2128;
        $2129 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($2129);
        $2130 = HEAP32[$pIn1>>2]|0;
        $2131 = ((($2130)) + 8|0);
        $2132 = HEAP16[$2131>>1]|0;
        $2133 = $2132&65535;
        $2134 = $2133 & 1;
        $2135 = ($2134|0)==(0);
        if ($2135) {
         $2136 = HEAP32[$pOut>>2]|0;
         $2137 = ((($2136)) + 8|0);
         HEAP16[$2137>>1] = 4;
         $2138 = HEAP32[$pIn1>>2]|0;
         $2139 = (_sqlite3VdbeIntValue($2138)|0);
         $2140 = tempRet0;
         $2141 = $2139 ^ -1;
         $2142 = $2140 ^ -1;
         $2143 = HEAP32[$pOut>>2]|0;
         $2144 = $2143;
         $2145 = $2144;
         HEAP32[$2145>>2] = $2141;
         $2146 = (($2144) + 4)|0;
         $2147 = $2146;
         HEAP32[$2147>>2] = $2142;
        }
        break;
       }
       case 44:  {
        $2148 = HEAP32[$pOp>>2]|0;
        $2149 = ((($2148)) + 4|0);
        $2150 = HEAP32[$2149>>2]|0;
        $2151 = HEAP32[$0>>2]|0;
        $2152 = ((($2151)) + 200|0);
        $2153 = HEAP32[$2152>>2]|0;
        $2154 = (($2153) + ($2150)|0);
        $2155 = HEAP8[$2154>>0]|0;
        $2156 = ($2155<<24>>24)!=(0);
        if ($2156) {
         label = 15;
        } else {
         $2157 = HEAP32[$pOp>>2]|0;
         $2158 = ((($2157)) + 4|0);
         $2159 = HEAP32[$2158>>2]|0;
         $2160 = HEAP32[$0>>2]|0;
         $2161 = ((($2160)) + 200|0);
         $2162 = HEAP32[$2161>>2]|0;
         $2163 = (($2162) + ($2159)|0);
         HEAP8[$2163>>0] = 1;
        }
        break;
       }
       case 46: case 45:  {
        $2164 = HEAP32[$pOp>>2]|0;
        $2165 = ((($2164)) + 4|0);
        $2166 = HEAP32[$2165>>2]|0;
        $2167 = HEAP32[$aMem>>2]|0;
        $2168 = (($2167) + (($2166*40)|0)|0);
        HEAP32[$pIn1>>2] = $2168;
        $2169 = HEAP32[$pIn1>>2]|0;
        $2170 = ((($2169)) + 8|0);
        $2171 = HEAP16[$2170>>1]|0;
        $2172 = $2171&65535;
        $2173 = $2172 & 1;
        $2174 = ($2173|0)!=(0);
        if ($2174) {
         $2175 = HEAP32[$pOp>>2]|0;
         $2176 = ((($2175)) + 12|0);
         $2177 = HEAP32[$2176>>2]|0;
         HEAP32[$c>>2] = $2177;
        } else {
         $2178 = HEAP32[$pIn1>>2]|0;
         $2179 = (+_sqlite3VdbeRealValue($2178));
         $2180 = $2179 != 0.0;
         $2181 = $2180&1;
         HEAP32[$c>>2] = $2181;
         $2182 = HEAP32[$pOp>>2]|0;
         $2183 = HEAP8[$2182>>0]|0;
         $2184 = $2183&255;
         $2185 = ($2184|0)==(46);
         if ($2185) {
          $2186 = HEAP32[$c>>2]|0;
          $2187 = ($2186|0)!=(0);
          $2188 = $2187 ^ 1;
          $2189 = $2188&1;
          HEAP32[$c>>2] = $2189;
         }
        }
        $2190 = HEAP32[$c>>2]|0;
        $2191 = ($2190|0)!=(0);
        if ($2191) {
         label = 15;
        }
        break;
       }
       case 76:  {
        $2192 = HEAP32[$pOp>>2]|0;
        $2193 = ((($2192)) + 4|0);
        $2194 = HEAP32[$2193>>2]|0;
        $2195 = HEAP32[$aMem>>2]|0;
        $2196 = (($2195) + (($2194*40)|0)|0);
        HEAP32[$pIn1>>2] = $2196;
        $2197 = HEAP32[$pIn1>>2]|0;
        $2198 = ((($2197)) + 8|0);
        $2199 = HEAP16[$2198>>1]|0;
        $2200 = $2199&65535;
        $2201 = $2200 & 1;
        $2202 = ($2201|0)!=(0);
        if ($2202) {
         label = 15;
        }
        break;
       }
       case 77:  {
        $2203 = HEAP32[$pOp>>2]|0;
        $2204 = ((($2203)) + 4|0);
        $2205 = HEAP32[$2204>>2]|0;
        $2206 = HEAP32[$aMem>>2]|0;
        $2207 = (($2206) + (($2205*40)|0)|0);
        HEAP32[$pIn1>>2] = $2207;
        $2208 = HEAP32[$pIn1>>2]|0;
        $2209 = ((($2208)) + 8|0);
        $2210 = HEAP16[$2209>>1]|0;
        $2211 = $2210&65535;
        $2212 = $2211 & 1;
        $2213 = ($2212|0)==(0);
        if ($2213) {
         label = 15;
        }
        break;
       }
       case 47:  {
        $2214 = HEAP32[$pOp>>2]|0;
        $2215 = ((($2214)) + 4|0);
        $2216 = HEAP32[$2215>>2]|0;
        $2217 = HEAP32[$0>>2]|0;
        $2218 = ((($2217)) + 56|0);
        $2219 = HEAP32[$2218>>2]|0;
        $2220 = (($2219) + ($2216<<2)|0);
        $2221 = HEAP32[$2220>>2]|0;
        HEAP32[$pC>>2] = $2221;
        $2222 = HEAP32[$pOp>>2]|0;
        $2223 = ((($2222)) + 8|0);
        $2224 = HEAP32[$2223>>2]|0;
        HEAP32[$p211>>2] = $2224;
        $2225 = (_sqlite3VdbeCursorMoveto($pC,$p211)|0);
        HEAP32[$rc>>2] = $2225;
        $2226 = HEAP32[$pOp>>2]|0;
        $2227 = ((($2226)) + 12|0);
        $2228 = HEAP32[$2227>>2]|0;
        $2229 = HEAP32[$aMem>>2]|0;
        $2230 = (($2229) + (($2228*40)|0)|0);
        HEAP32[$pDest>>2] = $2230;
        $2231 = HEAP32[$pC>>2]|0;
        $2232 = ((($2231)) + 76|0);
        $2233 = HEAP32[$2232>>2]|0;
        HEAP32[$aOffset>>2] = $2233;
        $2234 = HEAP32[$pC>>2]|0;
        $2235 = ((($2234)) + 16|0);
        $2236 = HEAP32[$2235>>2]|0;
        HEAP32[$pCrsr>>2] = $2236;
        $2237 = HEAP32[$rc>>2]|0;
        $2238 = ($2237|0)!=(0);
        if ($2238) {
         label = 861;
         break L8;
        }
        $2239 = HEAP32[$pC>>2]|0;
        $2240 = ((($2239)) + 56|0);
        $2241 = HEAP32[$2240>>2]|0;
        $2242 = HEAP32[$0>>2]|0;
        $2243 = ((($2242)) + 72|0);
        $2244 = HEAP32[$2243>>2]|0;
        $2245 = ($2241|0)!=($2244|0);
        $2246 = HEAP32[$pC>>2]|0;
        do {
         if ($2245) {
          $2247 = ((($2246)) + 2|0);
          $2248 = HEAP8[$2247>>0]|0;
          $2249 = ($2248<<24>>24)!=(0);
          $2250 = HEAP32[$pC>>2]|0;
          do {
           if ($2249) {
            $2251 = HEAP8[$2250>>0]|0;
            $2252 = $2251&255;
            $2253 = ($2252|0)==(3);
            if ($2253) {
             $2254 = HEAP32[$pC>>2]|0;
             $2255 = ((($2254)) + 16|0);
             $2256 = HEAP32[$2255>>2]|0;
             $2257 = HEAP32[$aMem>>2]|0;
             $2258 = (($2257) + (($2256*40)|0)|0);
             HEAP32[$pReg>>2] = $2258;
             $2259 = HEAP32[$pReg>>2]|0;
             $2260 = ((($2259)) + 12|0);
             $2261 = HEAP32[$2260>>2]|0;
             HEAP32[$avail>>2] = $2261;
             $2262 = HEAP32[$pC>>2]|0;
             $2263 = ((($2262)) + 64|0);
             HEAP32[$2263>>2] = $2261;
             $2264 = HEAP32[$pC>>2]|0;
             $2265 = ((($2264)) + 60|0);
             HEAP32[$2265>>2] = $2261;
             $2266 = HEAP32[$pReg>>2]|0;
             $2267 = ((($2266)) + 16|0);
             $2268 = HEAP32[$2267>>2]|0;
             $2269 = HEAP32[$pC>>2]|0;
             $2270 = ((($2269)) + 72|0);
             HEAP32[$2270>>2] = $2268;
             break;
            } else {
             $2271 = HEAP32[$pDest>>2]|0;
             _sqlite3VdbeMemSetNull($2271);
             break L11;
            }
           } else {
            $2272 = ((($2250)) + 4|0);
            $2273 = HEAP8[$2272>>0]|0;
            $2274 = $2273&255;
            $2275 = ($2274|0)==(0);
            $2276 = HEAP32[$pCrsr>>2]|0;
            if ($2275) {
             (_sqlite3BtreeKeySize($2276,$payloadSize64)|0);
             $2277 = HEAP32[$pCrsr>>2]|0;
             $2278 = (_sqlite3BtreeKeyFetch($2277,$avail)|0);
             $2279 = HEAP32[$pC>>2]|0;
             $2280 = ((($2279)) + 72|0);
             HEAP32[$2280>>2] = $2278;
             $2281 = $payloadSize64;
             $2282 = $2281;
             $2283 = HEAP32[$2282>>2]|0;
             $2284 = (($2281) + 4)|0;
             $2285 = $2284;
             $2286 = HEAP32[$2285>>2]|0;
             $2287 = HEAP32[$pC>>2]|0;
             $2288 = ((($2287)) + 60|0);
             HEAP32[$2288>>2] = $2283;
            } else {
             $2289 = HEAP32[$pC>>2]|0;
             $2290 = ((($2289)) + 60|0);
             (_sqlite3BtreeDataSize($2276,$2290)|0);
             $2291 = HEAP32[$pCrsr>>2]|0;
             $2292 = (_sqlite3BtreeDataFetch($2291,$avail)|0);
             $2293 = HEAP32[$pC>>2]|0;
             $2294 = ((($2293)) + 72|0);
             HEAP32[$2294>>2] = $2292;
            }
            $2295 = HEAP32[$pC>>2]|0;
            $2296 = ((($2295)) + 60|0);
            $2297 = HEAP32[$2296>>2]|0;
            $2298 = HEAP32[$avail>>2]|0;
            $2299 = ($2297>>>0)<=($2298>>>0);
            $2300 = HEAP32[$pC>>2]|0;
            $2301 = ((($2300)) + 60|0);
            $2302 = HEAP32[$2301>>2]|0;
            if ($2299) {
             $2303 = HEAP32[$pC>>2]|0;
             $2304 = ((($2303)) + 64|0);
             HEAP32[$2304>>2] = $2302;
             break;
            }
            $2305 = HEAP32[$db>>2]|0;
            $2306 = ((($2305)) + 88|0);
            $2307 = HEAP32[$2306>>2]|0;
            $2308 = ($2302>>>0)>($2307>>>0);
            if ($2308) {
             label = 859;
             break L8;
            }
            $2309 = HEAP32[$avail>>2]|0;
            $2310 = HEAP32[$pC>>2]|0;
            $2311 = ((($2310)) + 64|0);
            HEAP32[$2311>>2] = $2309;
           }
          } while(0);
          $2312 = HEAP32[$0>>2]|0;
          $2313 = ((($2312)) + 72|0);
          $2314 = HEAP32[$2313>>2]|0;
          $2315 = HEAP32[$pC>>2]|0;
          $2316 = ((($2315)) + 56|0);
          HEAP32[$2316>>2] = $2314;
          $2317 = HEAP32[$pC>>2]|0;
          $2318 = ((($2317)) + 72|0);
          $2319 = HEAP32[$2318>>2]|0;
          $2320 = HEAP8[$2319>>0]|0;
          $2321 = $2320&255;
          $2322 = ($2321|0)<(128);
          $2323 = HEAP32[$pC>>2]|0;
          $2324 = ((($2323)) + 72|0);
          $2325 = HEAP32[$2324>>2]|0;
          if ($2322) {
           $2326 = HEAP8[$2325>>0]|0;
           $2327 = $2326&255;
           HEAP32[$offset>>2] = $2327;
           $2331 = 1;
          } else {
           $2328 = (_sqlite3GetVarint32($2325,$offset)|0);
           $2329 = $2328&255;
           $2331 = $2329;
          }
          $2330 = $2331&255;
          $2332 = $2330&255;
          $2333 = HEAP32[$pC>>2]|0;
          $2334 = ((($2333)) + 68|0);
          HEAP32[$2334>>2] = $2332;
          $2335 = HEAP32[$pC>>2]|0;
          $2336 = ((($2335)) + 14|0);
          HEAP16[$2336>>1] = 0;
          $2337 = HEAP32[$offset>>2]|0;
          $2338 = HEAP32[$aOffset>>2]|0;
          HEAP32[$2338>>2] = $2337;
          $2339 = HEAP32[$avail>>2]|0;
          $2340 = HEAP32[$offset>>2]|0;
          $2341 = ($2339>>>0)<($2340>>>0);
          if ($2341) {
           $2342 = HEAP32[$pC>>2]|0;
           $2343 = ((($2342)) + 72|0);
           HEAP32[$2343>>2] = 0;
           $2344 = HEAP32[$pC>>2]|0;
           $2345 = ((($2344)) + 64|0);
           HEAP32[$2345>>2] = 0;
           $2346 = HEAP32[$offset>>2]|0;
           $2347 = ($2346>>>0)>(98307);
           if (!($2347)) {
            $2348 = HEAP32[$offset>>2]|0;
            $2349 = HEAP32[$pC>>2]|0;
            $2350 = ((($2349)) + 60|0);
            $2351 = HEAP32[$2350>>2]|0;
            $2352 = ($2348>>>0)>($2351>>>0);
            if (!($2352)) {
             label = 272;
             break;
            }
           }
           $2353 = (_sqlite3CorruptError(76686)|0);
           HEAP32[$rc>>2] = $2353;
           break L11;
          } else {
           label = 272;
          }
         } else {
          $2354 = ((($2246)) + 14|0);
          $2355 = HEAP16[$2354>>1]|0;
          $2356 = $2355&65535;
          $2357 = HEAP32[$p211>>2]|0;
          $2358 = ($2356|0)<=($2357|0);
          if ($2358) {
           label = 272;
          } else {
           $2531 = HEAP32[$p211>>2]|0;
           $2532 = HEAP32[$pC>>2]|0;
           $2533 = ((($2532)) + 80|0);
           $2534 = (($2533) + ($2531<<2)|0);
           $2535 = HEAP32[$2534>>2]|0;
           HEAP32[$t>>2] = $2535;
          }
         }
        } while(0);
        if ((label|0) == 272) {
         label = 0;
         $2359 = HEAP32[$pC>>2]|0;
         $2360 = ((($2359)) + 68|0);
         $2361 = HEAP32[$2360>>2]|0;
         $2362 = HEAP32[$aOffset>>2]|0;
         $2363 = HEAP32[$2362>>2]|0;
         $2364 = ($2361>>>0)<($2363>>>0);
         do {
          if ($2364) {
           $2365 = HEAP32[$pC>>2]|0;
           $2366 = ((($2365)) + 72|0);
           $2367 = HEAP32[$2366>>2]|0;
           $2368 = ($2367|0)==(0|0);
           if ($2368) {
            dest=$sMem; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
            $2369 = HEAP32[$pCrsr>>2]|0;
            $2370 = HEAP32[$aOffset>>2]|0;
            $2371 = HEAP32[$2370>>2]|0;
            $2372 = HEAP32[$pC>>2]|0;
            $2373 = ((($2372)) + 4|0);
            $2374 = HEAP8[$2373>>0]|0;
            $2375 = ($2374<<24>>24)!=(0);
            $2376 = $2375 ^ 1;
            $2377 = $2376&1;
            $2378 = (_sqlite3VdbeMemFromBtree($2369,0,$2371,$2377,$sMem)|0);
            HEAP32[$rc>>2] = $2378;
            $2379 = HEAP32[$rc>>2]|0;
            $2380 = ($2379|0)!=(0);
            if ($2380) {
             break L11;
            }
            $2381 = ((($sMem)) + 16|0);
            $2382 = HEAP32[$2381>>2]|0;
            HEAP32[$zData>>2] = $2382;
           } else {
            $2383 = HEAP32[$pC>>2]|0;
            $2384 = ((($2383)) + 72|0);
            $2385 = HEAP32[$2384>>2]|0;
            HEAP32[$zData>>2] = $2385;
           }
           $2386 = HEAP32[$pC>>2]|0;
           $2387 = ((($2386)) + 14|0);
           $2388 = HEAP16[$2387>>1]|0;
           $2389 = $2388&65535;
           HEAP32[$i12>>2] = $2389;
           $2390 = HEAP32[$i12>>2]|0;
           $2391 = HEAP32[$aOffset>>2]|0;
           $2392 = (($2391) + ($2390<<2)|0);
           $2393 = HEAP32[$2392>>2]|0;
           $2394 = $offset64;
           $2395 = $2394;
           HEAP32[$2395>>2] = $2393;
           $2396 = (($2394) + 4)|0;
           $2397 = $2396;
           HEAP32[$2397>>2] = 0;
           $2398 = HEAP32[$zData>>2]|0;
           $2399 = HEAP32[$pC>>2]|0;
           $2400 = ((($2399)) + 68|0);
           $2401 = HEAP32[$2400>>2]|0;
           $2402 = (($2398) + ($2401)|0);
           HEAP32[$zHdr>>2] = $2402;
           $2403 = HEAP32[$zData>>2]|0;
           $2404 = HEAP32[$aOffset>>2]|0;
           $2405 = HEAP32[$2404>>2]|0;
           $2406 = (($2403) + ($2405)|0);
           HEAP32[$zEndHdr>>2] = $2406;
           while(1) {
            $2407 = HEAP32[$zHdr>>2]|0;
            $2408 = HEAP8[$2407>>0]|0;
            $2409 = $2408&255;
            HEAP32[$t>>2] = $2409;
            $2410 = ($2409>>>0)<(128);
            $2411 = HEAP32[$zHdr>>2]|0;
            if ($2410) {
             $2412 = ((($2411)) + 1|0);
             HEAP32[$zHdr>>2] = $2412;
             $2413 = HEAP32[$t>>2]|0;
             $2414 = $2413&255;
             $2415 = (_sqlite3VdbeOneByteSerialTypeLen($2414)|0);
             $2416 = $2415&255;
             $2417 = $offset64;
             $2418 = $2417;
             $2419 = HEAP32[$2418>>2]|0;
             $2420 = (($2417) + 4)|0;
             $2421 = $2420;
             $2422 = HEAP32[$2421>>2]|0;
             $2423 = (_i64Add(($2419|0),($2422|0),($2416|0),0)|0);
             $2424 = tempRet0;
             $2425 = $offset64;
             $2426 = $2425;
             HEAP32[$2426>>2] = $2423;
             $2427 = (($2425) + 4)|0;
             $2428 = $2427;
             HEAP32[$2428>>2] = $2424;
            } else {
             $2429 = (_sqlite3GetVarint32($2411,$t)|0);
             $2430 = $2429&255;
             $2431 = HEAP32[$zHdr>>2]|0;
             $2432 = (($2431) + ($2430)|0);
             HEAP32[$zHdr>>2] = $2432;
             $2433 = HEAP32[$t>>2]|0;
             $2434 = (_sqlite3VdbeSerialTypeLen($2433)|0);
             $2435 = $offset64;
             $2436 = $2435;
             $2437 = HEAP32[$2436>>2]|0;
             $2438 = (($2435) + 4)|0;
             $2439 = $2438;
             $2440 = HEAP32[$2439>>2]|0;
             $2441 = (_i64Add(($2437|0),($2440|0),($2434|0),0)|0);
             $2442 = tempRet0;
             $2443 = $offset64;
             $2444 = $2443;
             HEAP32[$2444>>2] = $2441;
             $2445 = (($2443) + 4)|0;
             $2446 = $2445;
             HEAP32[$2446>>2] = $2442;
            }
            $2447 = HEAP32[$t>>2]|0;
            $2448 = HEAP32[$i12>>2]|0;
            $2449 = (($2448) + 1)|0;
            HEAP32[$i12>>2] = $2449;
            $2450 = HEAP32[$pC>>2]|0;
            $2451 = ((($2450)) + 80|0);
            $2452 = (($2451) + ($2448<<2)|0);
            HEAP32[$2452>>2] = $2447;
            $2453 = $offset64;
            $2454 = $2453;
            $2455 = HEAP32[$2454>>2]|0;
            $2456 = (($2453) + 4)|0;
            $2457 = $2456;
            $2458 = HEAP32[$2457>>2]|0;
            $2459 = HEAP32[$i12>>2]|0;
            $2460 = HEAP32[$aOffset>>2]|0;
            $2461 = (($2460) + ($2459<<2)|0);
            HEAP32[$2461>>2] = $2455;
            $2462 = HEAP32[$i12>>2]|0;
            $2463 = HEAP32[$p211>>2]|0;
            $2464 = ($2462|0)<=($2463|0);
            if (!($2464)) {
             break;
            }
            $2465 = HEAP32[$zHdr>>2]|0;
            $2466 = HEAP32[$zEndHdr>>2]|0;
            $2467 = ($2465>>>0)<($2466>>>0);
            if (!($2467)) {
             break;
            }
           }
           $2468 = HEAP32[$i12>>2]|0;
           $2469 = $2468&65535;
           $2470 = HEAP32[$pC>>2]|0;
           $2471 = ((($2470)) + 14|0);
           HEAP16[$2471>>1] = $2469;
           $2472 = HEAP32[$zHdr>>2]|0;
           $2473 = HEAP32[$zData>>2]|0;
           $2474 = $2472;
           $2475 = $2473;
           $2476 = (($2474) - ($2475))|0;
           $2477 = HEAP32[$pC>>2]|0;
           $2478 = ((($2477)) + 68|0);
           HEAP32[$2478>>2] = $2476;
           $2479 = HEAP32[$pC>>2]|0;
           $2480 = ((($2479)) + 72|0);
           $2481 = HEAP32[$2480>>2]|0;
           $2482 = ($2481|0)==(0|0);
           if ($2482) {
            _sqlite3VdbeMemRelease($sMem);
           }
           $2483 = HEAP32[$zHdr>>2]|0;
           $2484 = HEAP32[$zEndHdr>>2]|0;
           $2485 = ($2483>>>0)>=($2484>>>0);
           if ($2485) {
            $2486 = HEAP32[$zHdr>>2]|0;
            $2487 = HEAP32[$zEndHdr>>2]|0;
            $2488 = ($2486>>>0)>($2487>>>0);
            if (!($2488)) {
             $2489 = $offset64;
             $2490 = $2489;
             $2491 = HEAP32[$2490>>2]|0;
             $2492 = (($2489) + 4)|0;
             $2493 = $2492;
             $2494 = HEAP32[$2493>>2]|0;
             $2495 = HEAP32[$pC>>2]|0;
             $2496 = ((($2495)) + 60|0);
             $2497 = HEAP32[$2496>>2]|0;
             $2498 = ($2491|0)!=($2497|0);
             $2499 = ($2494|0)!=(0);
             $2500 = $2498 | $2499;
             if (!($2500)) {
              label = 288;
             }
            }
           } else {
            label = 288;
           }
           if ((label|0) == 288) {
            label = 0;
            $2501 = $offset64;
            $2502 = $2501;
            $2503 = HEAP32[$2502>>2]|0;
            $2504 = (($2501) + 4)|0;
            $2505 = $2504;
            $2506 = HEAP32[$2505>>2]|0;
            $2507 = HEAP32[$pC>>2]|0;
            $2508 = ((($2507)) + 60|0);
            $2509 = HEAP32[$2508>>2]|0;
            $2510 = ($2506>>>0)>(0);
            $2511 = ($2503>>>0)>($2509>>>0);
            $2512 = ($2506|0)==(0);
            $2513 = $2512 & $2511;
            $2514 = $2510 | $2513;
            if (!($2514)) {
             break;
            }
           }
           $2515 = (_sqlite3CorruptError(76747)|0);
           HEAP32[$rc>>2] = $2515;
           break L11;
          } else {
           HEAP32[$t>>2] = 0;
          }
         } while(0);
         $2516 = HEAP32[$pC>>2]|0;
         $2517 = ((($2516)) + 14|0);
         $2518 = HEAP16[$2517>>1]|0;
         $2519 = $2518&65535;
         $2520 = HEAP32[$p211>>2]|0;
         $2521 = ($2519|0)<=($2520|0);
         if ($2521) {
          $2522 = HEAP32[$pOp>>2]|0;
          $2523 = ((($2522)) + 1|0);
          $2524 = HEAP8[$2523>>0]|0;
          $2525 = $2524 << 24 >> 24;
          $2526 = ($2525|0)==(-8);
          $2527 = HEAP32[$pDest>>2]|0;
          if ($2526) {
           $2528 = HEAP32[$pOp>>2]|0;
           $2529 = ((($2528)) + 16|0);
           $2530 = HEAP32[$2529>>2]|0;
           _sqlite3VdbeMemShallowCopy($2527,$2530,2048);
           break L11;
          } else {
           _sqlite3VdbeMemSetNull($2527);
           break L11;
          }
         }
        }
        $2536 = HEAP32[$pDest>>2]|0;
        $2537 = ((($2536)) + 8|0);
        $2538 = HEAP16[$2537>>1]|0;
        $2539 = $2538&65535;
        $2540 = $2539 & 9312;
        $2541 = ($2540|0)!=(0);
        if ($2541) {
         $2542 = HEAP32[$pDest>>2]|0;
         _sqlite3VdbeMemSetNull($2542);
        }
        $2543 = HEAP8[$encoding>>0]|0;
        $2544 = HEAP32[$pDest>>2]|0;
        $2545 = ((($2544)) + 10|0);
        HEAP8[$2545>>0] = $2543;
        $2546 = HEAP32[$pC>>2]|0;
        $2547 = ((($2546)) + 64|0);
        $2548 = HEAP32[$2547>>2]|0;
        $2549 = HEAP32[$p211>>2]|0;
        $2550 = (($2549) + 1)|0;
        $2551 = HEAP32[$aOffset>>2]|0;
        $2552 = (($2551) + ($2550<<2)|0);
        $2553 = HEAP32[$2552>>2]|0;
        $2554 = ($2548>>>0)>=($2553>>>0);
        if ($2554) {
         $2555 = HEAP32[$pC>>2]|0;
         $2556 = ((($2555)) + 72|0);
         $2557 = HEAP32[$2556>>2]|0;
         $2558 = HEAP32[$p211>>2]|0;
         $2559 = HEAP32[$aOffset>>2]|0;
         $2560 = (($2559) + ($2558<<2)|0);
         $2561 = HEAP32[$2560>>2]|0;
         $2562 = (($2557) + ($2561)|0);
         HEAP32[$zData>>2] = $2562;
         $2563 = HEAP32[$t>>2]|0;
         $2564 = ($2563>>>0)<(12);
         if ($2564) {
          $2565 = HEAP32[$zData>>2]|0;
          $2566 = HEAP32[$t>>2]|0;
          $2567 = HEAP32[$pDest>>2]|0;
          (_sqlite3VdbeSerialGet($2565,$2566,$2567)|0);
          break L11;
         }
         $2568 = HEAP32[$t>>2]|0;
         $2569 = (($2568) - 12)|0;
         $2570 = (($2569>>>0) / 2)&-1;
         HEAP32[$len>>2] = $2570;
         $2571 = HEAP32[$pDest>>2]|0;
         $2572 = ((($2571)) + 12|0);
         HEAP32[$2572>>2] = $2570;
         $2573 = HEAP32[$pDest>>2]|0;
         $2574 = ((($2573)) + 24|0);
         $2575 = HEAP32[$2574>>2]|0;
         $2576 = HEAP32[$len>>2]|0;
         $2577 = (($2576) + 2)|0;
         $2578 = ($2575|0)<($2577|0);
         $2579 = HEAP32[$pDest>>2]|0;
         if ($2578) {
          $2580 = ((($2579)) + 8|0);
          HEAP16[$2580>>1] = 1;
          $2581 = HEAP32[$pDest>>2]|0;
          $2582 = HEAP32[$len>>2]|0;
          $2583 = (($2582) + 2)|0;
          $2584 = (_sqlite3VdbeMemGrow($2581,$2583,0)|0);
          $2585 = ($2584|0)!=(0);
          if ($2585) {
           label = 860;
           break L1;
          }
         } else {
          $2586 = ((($2579)) + 20|0);
          $2587 = HEAP32[$2586>>2]|0;
          $2588 = HEAP32[$pDest>>2]|0;
          $2589 = ((($2588)) + 16|0);
          HEAP32[$2589>>2] = $2587;
         }
         $2590 = HEAP32[$pDest>>2]|0;
         $2591 = ((($2590)) + 16|0);
         $2592 = HEAP32[$2591>>2]|0;
         $2593 = HEAP32[$zData>>2]|0;
         $2594 = HEAP32[$len>>2]|0;
         _memcpy(($2592|0),($2593|0),($2594|0))|0;
         $2595 = HEAP32[$len>>2]|0;
         $2596 = HEAP32[$pDest>>2]|0;
         $2597 = ((($2596)) + 16|0);
         $2598 = HEAP32[$2597>>2]|0;
         $2599 = (($2598) + ($2595)|0);
         HEAP8[$2599>>0] = 0;
         $2600 = HEAP32[$len>>2]|0;
         $2601 = (($2600) + 1)|0;
         $2602 = HEAP32[$pDest>>2]|0;
         $2603 = ((($2602)) + 16|0);
         $2604 = HEAP32[$2603>>2]|0;
         $2605 = (($2604) + ($2601)|0);
         HEAP8[$2605>>0] = 0;
         $2606 = HEAP32[$t>>2]|0;
         $2607 = $2606 & 1;
         $2608 = (10020 + ($2607<<1)|0);
         $2609 = HEAP16[$2608>>1]|0;
         $2610 = HEAP32[$pDest>>2]|0;
         $2611 = ((($2610)) + 8|0);
         HEAP16[$2611>>1] = $2609;
         break L11;
        }
        $2612 = HEAP32[$pOp>>2]|0;
        $2613 = ((($2612)) + 3|0);
        $2614 = HEAP8[$2613>>0]|0;
        $2615 = $2614&255;
        $2616 = $2615 & 192;
        $2617 = ($2616|0)!=(0);
        do {
         if ($2617) {
          $2618 = HEAP32[$t>>2]|0;
          $2619 = ($2618>>>0)>=(12);
          if ($2619) {
           $2620 = HEAP32[$t>>2]|0;
           $2621 = $2620 & 1;
           $2622 = ($2621|0)==(0);
           if ($2622) {
            break;
           }
          }
          $2623 = HEAP32[$pOp>>2]|0;
          $2624 = ((($2623)) + 3|0);
          $2625 = HEAP8[$2624>>0]|0;
          $2626 = $2625&255;
          $2627 = $2626 & 128;
          $2628 = ($2627|0)!=(0);
          if (!($2628)) {
           label = 309;
          }
         } else {
          label = 309;
         }
        } while(0);
        if ((label|0) == 309) {
         label = 0;
         $2629 = HEAP32[$t>>2]|0;
         $2630 = (_sqlite3VdbeSerialTypeLen($2629)|0);
         HEAP32[$len>>2] = $2630;
         $2631 = ($2630|0)==(0);
         if (!($2631)) {
          $2634 = HEAP32[$pCrsr>>2]|0;
          $2635 = HEAP32[$p211>>2]|0;
          $2636 = HEAP32[$aOffset>>2]|0;
          $2637 = (($2636) + ($2635<<2)|0);
          $2638 = HEAP32[$2637>>2]|0;
          $2639 = HEAP32[$len>>2]|0;
          $2640 = HEAP32[$pC>>2]|0;
          $2641 = ((($2640)) + 4|0);
          $2642 = HEAP8[$2641>>0]|0;
          $2643 = ($2642<<24>>24)!=(0);
          $2644 = $2643 ^ 1;
          $2645 = $2644&1;
          $2646 = HEAP32[$pDest>>2]|0;
          $2647 = (_sqlite3VdbeMemFromBtree($2634,$2638,$2639,$2645,$2646)|0);
          HEAP32[$rc>>2] = $2647;
          $2648 = HEAP32[$rc>>2]|0;
          $2649 = ($2648|0)==(0);
          if (!($2649)) {
           break L11;
          }
          $2650 = HEAP32[$pDest>>2]|0;
          $2651 = ((($2650)) + 16|0);
          $2652 = HEAP32[$2651>>2]|0;
          $2653 = HEAP32[$t>>2]|0;
          $2654 = HEAP32[$pDest>>2]|0;
          (_sqlite3VdbeSerialGet($2652,$2653,$2654)|0);
          $2655 = HEAP32[$pDest>>2]|0;
          $2656 = ((($2655)) + 8|0);
          $2657 = HEAP16[$2656>>1]|0;
          $2658 = $2657&65535;
          $2659 = $2658 & -4097;
          $2660 = $2659&65535;
          HEAP16[$2656>>1] = $2660;
          break L11;
         }
        }
        $2632 = HEAP32[$t>>2]|0;
        $2633 = HEAP32[$pDest>>2]|0;
        (_sqlite3VdbeSerialGet(20910,$2632,$2633)|0);
        break;
       }
       case 48:  {
        $2661 = HEAP32[$pOp>>2]|0;
        $2662 = ((($2661)) + 16|0);
        $2663 = HEAP32[$2662>>2]|0;
        HEAP32[$zAffinity>>2] = $2663;
        $2664 = HEAP32[$pOp>>2]|0;
        $2665 = ((($2664)) + 4|0);
        $2666 = HEAP32[$2665>>2]|0;
        $2667 = HEAP32[$aMem>>2]|0;
        $2668 = (($2667) + (($2666*40)|0)|0);
        HEAP32[$pIn1>>2] = $2668;
        while(1) {
         $2669 = HEAP32[$zAffinity>>2]|0;
         $2670 = ((($2669)) + 1|0);
         HEAP32[$zAffinity>>2] = $2670;
         $2671 = HEAP8[$2669>>0]|0;
         HEAP8[$cAff>>0] = $2671;
         $2672 = $2671 << 24 >> 24;
         $2673 = ($2672|0)!=(0);
         if (!($2673)) {
          break L11;
         }
         $2674 = HEAP32[$pIn1>>2]|0;
         $2675 = HEAP8[$cAff>>0]|0;
         $2676 = HEAP8[$encoding>>0]|0;
         _applyAffinity($2674,$2675,$2676);
         $2677 = HEAP32[$pIn1>>2]|0;
         $2678 = ((($2677)) + 40|0);
         HEAP32[$pIn1>>2] = $2678;
        }
        break;
       }
       case 49:  {
        $2679 = $nData;
        $2680 = $2679;
        HEAP32[$2680>>2] = 0;
        $2681 = (($2679) + 4)|0;
        $2682 = $2681;
        HEAP32[$2682>>2] = 0;
        HEAP32[$nHdr>>2] = 0;
        $2683 = $nZero;
        $2684 = $2683;
        HEAP32[$2684>>2] = 0;
        $2685 = (($2683) + 4)|0;
        $2686 = $2685;
        HEAP32[$2686>>2] = 0;
        $2687 = HEAP32[$pOp>>2]|0;
        $2688 = ((($2687)) + 4|0);
        $2689 = HEAP32[$2688>>2]|0;
        HEAP32[$nField>>2] = $2689;
        $2690 = HEAP32[$pOp>>2]|0;
        $2691 = ((($2690)) + 16|0);
        $2692 = HEAP32[$2691>>2]|0;
        HEAP32[$zAffinity14>>2] = $2692;
        $2693 = HEAP32[$nField>>2]|0;
        $2694 = HEAP32[$aMem>>2]|0;
        $2695 = (($2694) + (($2693*40)|0)|0);
        HEAP32[$pData0>>2] = $2695;
        $2696 = HEAP32[$pOp>>2]|0;
        $2697 = ((($2696)) + 8|0);
        $2698 = HEAP32[$2697>>2]|0;
        HEAP32[$nField>>2] = $2698;
        $2699 = HEAP32[$nField>>2]|0;
        $2700 = (($2699) - 1)|0;
        $2701 = HEAP32[$pData0>>2]|0;
        $2702 = (($2701) + (($2700*40)|0)|0);
        HEAP32[$pLast>>2] = $2702;
        $2703 = HEAP32[$0>>2]|0;
        $2704 = ((($2703)) + 87|0);
        $2705 = HEAP8[$2704>>0]|0;
        $2706 = $2705&255;
        HEAP32[$file_format>>2] = $2706;
        $2707 = HEAP32[$pOp>>2]|0;
        $2708 = ((($2707)) + 12|0);
        $2709 = HEAP32[$2708>>2]|0;
        $2710 = HEAP32[$aMem>>2]|0;
        $2711 = (($2710) + (($2709*40)|0)|0);
        HEAP32[$pOut>>2] = $2711;
        $2712 = HEAP32[$zAffinity14>>2]|0;
        $2713 = ($2712|0)!=(0|0);
        if ($2713) {
         $2714 = HEAP32[$pData0>>2]|0;
         HEAP32[$pRec>>2] = $2714;
         while(1) {
          $2715 = HEAP32[$pRec>>2]|0;
          $2716 = ((($2715)) + 40|0);
          HEAP32[$pRec>>2] = $2716;
          $2717 = HEAP32[$zAffinity14>>2]|0;
          $2718 = ((($2717)) + 1|0);
          HEAP32[$zAffinity14>>2] = $2718;
          $2719 = HEAP8[$2717>>0]|0;
          $2720 = HEAP8[$encoding>>0]|0;
          _applyAffinity($2715,$2719,$2720);
          $2721 = HEAP32[$zAffinity14>>2]|0;
          $2722 = HEAP8[$2721>>0]|0;
          $2723 = ($2722<<24>>24)!=(0);
          if (!($2723)) {
           break;
          }
         }
        }
        $2724 = HEAP32[$pLast>>2]|0;
        HEAP32[$pRec>>2] = $2724;
        while(1) {
         $2725 = HEAP32[$pRec>>2]|0;
         $2726 = HEAP32[$file_format>>2]|0;
         $2727 = (_sqlite3VdbeSerialType($2725,$2726,$len16)|0);
         HEAP32[$serial_type>>2] = $2727;
         $2728 = HEAP32[$pRec>>2]|0;
         $2729 = ((($2728)) + 28|0);
         HEAP32[$2729>>2] = $2727;
         $2730 = HEAP32[$pRec>>2]|0;
         $2731 = ((($2730)) + 8|0);
         $2732 = HEAP16[$2731>>1]|0;
         $2733 = $2732&65535;
         $2734 = $2733 & 16384;
         $2735 = ($2734|0)!=(0);
         do {
          if ($2735) {
           $2736 = $nData;
           $2737 = $2736;
           $2738 = HEAP32[$2737>>2]|0;
           $2739 = (($2736) + 4)|0;
           $2740 = $2739;
           $2741 = HEAP32[$2740>>2]|0;
           $2742 = ($2738|0)!=(0);
           $2743 = ($2741|0)!=(0);
           $2744 = $2742 | $2743;
           $2745 = HEAP32[$pRec>>2]|0;
           if ($2744) {
            $2746 = (_sqlite3VdbeMemExpandBlob($2745)|0);
            $2747 = ($2746|0)!=(0);
            if ($2747) {
             label = 860;
             break L1;
            } else {
             break;
            }
           } else {
            $2748 = HEAP32[$2745>>2]|0;
            $2749 = ($2748|0)<(0);
            $2750 = $2749 << 31 >> 31;
            $2751 = $nZero;
            $2752 = $2751;
            $2753 = HEAP32[$2752>>2]|0;
            $2754 = (($2751) + 4)|0;
            $2755 = $2754;
            $2756 = HEAP32[$2755>>2]|0;
            $2757 = (_i64Add(($2753|0),($2756|0),($2748|0),($2750|0))|0);
            $2758 = tempRet0;
            $2759 = $nZero;
            $2760 = $2759;
            HEAP32[$2760>>2] = $2757;
            $2761 = (($2759) + 4)|0;
            $2762 = $2761;
            HEAP32[$2762>>2] = $2758;
            $2763 = HEAP32[$pRec>>2]|0;
            $2764 = HEAP32[$2763>>2]|0;
            $2765 = HEAP32[$len16>>2]|0;
            $2766 = (($2765) - ($2764))|0;
            HEAP32[$len16>>2] = $2766;
            break;
           }
          }
         } while(0);
         $2767 = HEAP32[$len16>>2]|0;
         $2768 = $nData;
         $2769 = $2768;
         $2770 = HEAP32[$2769>>2]|0;
         $2771 = (($2768) + 4)|0;
         $2772 = $2771;
         $2773 = HEAP32[$2772>>2]|0;
         $2774 = (_i64Add(($2770|0),($2773|0),($2767|0),0)|0);
         $2775 = tempRet0;
         $2776 = $nData;
         $2777 = $2776;
         HEAP32[$2777>>2] = $2774;
         $2778 = (($2776) + 4)|0;
         $2779 = $2778;
         HEAP32[$2779>>2] = $2775;
         $2780 = HEAP32[$serial_type>>2]|0;
         $2781 = ($2780>>>0)<=(127);
         if ($2781) {
          $2786 = 1;
         } else {
          $2782 = HEAP32[$serial_type>>2]|0;
          $2783 = (_sqlite3VarintLen($2782,0)|0);
          $2786 = $2783;
         }
         $2784 = HEAP32[$nHdr>>2]|0;
         $2785 = (($2784) + ($2786))|0;
         HEAP32[$nHdr>>2] = $2785;
         $2787 = HEAP32[$pRec>>2]|0;
         $2788 = ((($2787)) + -40|0);
         HEAP32[$pRec>>2] = $2788;
         $2789 = HEAP32[$pData0>>2]|0;
         $2790 = ($2788>>>0)>=($2789>>>0);
         if (!($2790)) {
          break;
         }
        }
        $2791 = HEAP32[$nHdr>>2]|0;
        $2792 = ($2791|0)<=(126);
        $2793 = HEAP32[$nHdr>>2]|0;
        if ($2792) {
         $2794 = (($2793) + 1)|0;
         HEAP32[$nHdr>>2] = $2794;
        } else {
         $2795 = ($2793|0)<(0);
         $2796 = $2795 << 31 >> 31;
         $2797 = (_sqlite3VarintLen($2793,$2796)|0);
         HEAP32[$nVarint>>2] = $2797;
         $2798 = HEAP32[$nVarint>>2]|0;
         $2799 = HEAP32[$nHdr>>2]|0;
         $2800 = (($2799) + ($2798))|0;
         HEAP32[$nHdr>>2] = $2800;
         $2801 = HEAP32[$nVarint>>2]|0;
         $2802 = HEAP32[$nHdr>>2]|0;
         $2803 = ($2802|0)<(0);
         $2804 = $2803 << 31 >> 31;
         $2805 = (_sqlite3VarintLen($2802,$2804)|0);
         $2806 = ($2801|0)<($2805|0);
         if ($2806) {
          $2807 = HEAP32[$nHdr>>2]|0;
          $2808 = (($2807) + 1)|0;
          HEAP32[$nHdr>>2] = $2808;
         }
        }
        $2809 = HEAP32[$nHdr>>2]|0;
        $2810 = ($2809|0)<(0);
        $2811 = $2810 << 31 >> 31;
        $2812 = $nData;
        $2813 = $2812;
        $2814 = HEAP32[$2813>>2]|0;
        $2815 = (($2812) + 4)|0;
        $2816 = $2815;
        $2817 = HEAP32[$2816>>2]|0;
        $2818 = (_i64Add(($2809|0),($2811|0),($2814|0),($2817|0))|0);
        $2819 = tempRet0;
        $2820 = $nByte13;
        $2821 = $2820;
        HEAP32[$2821>>2] = $2818;
        $2822 = (($2820) + 4)|0;
        $2823 = $2822;
        HEAP32[$2823>>2] = $2819;
        $2824 = $nByte13;
        $2825 = $2824;
        $2826 = HEAP32[$2825>>2]|0;
        $2827 = (($2824) + 4)|0;
        $2828 = $2827;
        $2829 = HEAP32[$2828>>2]|0;
        $2830 = $nZero;
        $2831 = $2830;
        $2832 = HEAP32[$2831>>2]|0;
        $2833 = (($2830) + 4)|0;
        $2834 = $2833;
        $2835 = HEAP32[$2834>>2]|0;
        $2836 = (_i64Add(($2826|0),($2829|0),($2832|0),($2835|0))|0);
        $2837 = tempRet0;
        $2838 = HEAP32[$db>>2]|0;
        $2839 = ((($2838)) + 88|0);
        $2840 = HEAP32[$2839>>2]|0;
        $2841 = ($2840|0)<(0);
        $2842 = $2841 << 31 >> 31;
        $2843 = ($2837|0)>($2842|0);
        $2844 = ($2836>>>0)>($2840>>>0);
        $2845 = ($2837|0)==($2842|0);
        $2846 = $2845 & $2844;
        $2847 = $2843 | $2846;
        if ($2847) {
         label = 859;
         break L8;
        }
        $2848 = HEAP32[$pOut>>2]|0;
        $2849 = $nByte13;
        $2850 = $2849;
        $2851 = HEAP32[$2850>>2]|0;
        $2852 = (($2849) + 4)|0;
        $2853 = $2852;
        $2854 = HEAP32[$2853>>2]|0;
        $2855 = (_sqlite3VdbeMemClearAndResize($2848,$2851)|0);
        $2856 = ($2855|0)!=(0);
        if ($2856) {
         label = 860;
         break L1;
        }
        $2857 = HEAP32[$pOut>>2]|0;
        $2858 = ((($2857)) + 16|0);
        $2859 = HEAP32[$2858>>2]|0;
        HEAP32[$zNewRecord>>2] = $2859;
        $2860 = HEAP32[$nHdr>>2]|0;
        $2861 = ($2860>>>0)<(128);
        if ($2861) {
         $2862 = HEAP32[$nHdr>>2]|0;
         $2863 = $2862&255;
         $2864 = HEAP32[$zNewRecord>>2]|0;
         HEAP8[$2864>>0] = $2863;
         $2871 = 1;
        } else {
         $2865 = HEAP32[$zNewRecord>>2]|0;
         $2866 = HEAP32[$nHdr>>2]|0;
         $2867 = ($2866|0)<(0);
         $2868 = $2867 << 31 >> 31;
         $2869 = (_sqlite3PutVarint($2865,$2866,$2868)|0);
         $2871 = $2869;
        }
        $2870 = $2871&255;
        $2872 = $2870&255;
        HEAP32[$i15>>2] = $2872;
        $2873 = HEAP32[$nHdr>>2]|0;
        HEAP32[$j>>2] = $2873;
        $2874 = HEAP32[$pData0>>2]|0;
        HEAP32[$pRec>>2] = $2874;
        while(1) {
         $2875 = HEAP32[$pRec>>2]|0;
         $2876 = ((($2875)) + 28|0);
         $2877 = HEAP32[$2876>>2]|0;
         HEAP32[$serial_type>>2] = $2877;
         $2878 = HEAP32[$serial_type>>2]|0;
         $2879 = ($2878>>>0)<(128);
         if ($2879) {
          $2880 = HEAP32[$serial_type>>2]|0;
          $2881 = $2880&255;
          $2882 = HEAP32[$i15>>2]|0;
          $2883 = HEAP32[$zNewRecord>>2]|0;
          $2884 = (($2883) + ($2882)|0);
          HEAP8[$2884>>0] = $2881;
          $2891 = 1;
         } else {
          $2885 = HEAP32[$i15>>2]|0;
          $2886 = HEAP32[$zNewRecord>>2]|0;
          $2887 = (($2886) + ($2885)|0);
          $2888 = HEAP32[$serial_type>>2]|0;
          $2889 = (_sqlite3PutVarint($2887,$2888,0)|0);
          $2891 = $2889;
         }
         $2890 = $2891&255;
         $2892 = $2890&255;
         $2893 = HEAP32[$i15>>2]|0;
         $2894 = (($2893) + ($2892))|0;
         HEAP32[$i15>>2] = $2894;
         $2895 = HEAP32[$j>>2]|0;
         $2896 = HEAP32[$zNewRecord>>2]|0;
         $2897 = (($2896) + ($2895)|0);
         $2898 = HEAP32[$pRec>>2]|0;
         $2899 = HEAP32[$serial_type>>2]|0;
         $2900 = (_sqlite3VdbeSerialPut($2897,$2898,$2899)|0);
         $2901 = HEAP32[$j>>2]|0;
         $2902 = (($2901) + ($2900))|0;
         HEAP32[$j>>2] = $2902;
         $2903 = HEAP32[$pRec>>2]|0;
         $2904 = ((($2903)) + 40|0);
         HEAP32[$pRec>>2] = $2904;
         $2905 = HEAP32[$pLast>>2]|0;
         $2906 = ($2904>>>0)<=($2905>>>0);
         if (!($2906)) {
          break;
         }
        }
        $2907 = $nByte13;
        $2908 = $2907;
        $2909 = HEAP32[$2908>>2]|0;
        $2910 = (($2907) + 4)|0;
        $2911 = $2910;
        $2912 = HEAP32[$2911>>2]|0;
        $2913 = HEAP32[$pOut>>2]|0;
        $2914 = ((($2913)) + 12|0);
        HEAP32[$2914>>2] = $2909;
        $2915 = HEAP32[$pOut>>2]|0;
        $2916 = ((($2915)) + 8|0);
        HEAP16[$2916>>1] = 16;
        $2917 = $nZero;
        $2918 = $2917;
        $2919 = HEAP32[$2918>>2]|0;
        $2920 = (($2917) + 4)|0;
        $2921 = $2920;
        $2922 = HEAP32[$2921>>2]|0;
        $2923 = ($2919|0)!=(0);
        $2924 = ($2922|0)!=(0);
        $2925 = $2923 | $2924;
        if ($2925) {
         $2926 = $nZero;
         $2927 = $2926;
         $2928 = HEAP32[$2927>>2]|0;
         $2929 = (($2926) + 4)|0;
         $2930 = $2929;
         $2931 = HEAP32[$2930>>2]|0;
         $2932 = HEAP32[$pOut>>2]|0;
         HEAP32[$2932>>2] = $2928;
         $2933 = HEAP32[$pOut>>2]|0;
         $2934 = ((($2933)) + 8|0);
         $2935 = HEAP16[$2934>>1]|0;
         $2936 = $2935&65535;
         $2937 = $2936 | 16384;
         $2938 = $2937&65535;
         HEAP16[$2934>>1] = $2938;
        }
        $2939 = HEAP32[$pOut>>2]|0;
        $2940 = ((($2939)) + 10|0);
        HEAP8[$2940>>0] = 1;
        break;
       }
       case 50:  {
        $2941 = HEAP32[$pOp>>2]|0;
        $2942 = ((($2941)) + 4|0);
        $2943 = HEAP32[$2942>>2]|0;
        $2944 = HEAP32[$0>>2]|0;
        $2945 = ((($2944)) + 56|0);
        $2946 = HEAP32[$2945>>2]|0;
        $2947 = (($2946) + ($2943<<2)|0);
        $2948 = HEAP32[$2947>>2]|0;
        $2949 = ((($2948)) + 16|0);
        $2950 = HEAP32[$2949>>2]|0;
        HEAP32[$pCrsr17>>2] = $2950;
        $2951 = $nEntry;
        $2952 = $2951;
        HEAP32[$2952>>2] = 0;
        $2953 = (($2951) + 4)|0;
        $2954 = $2953;
        HEAP32[$2954>>2] = 0;
        $2955 = HEAP32[$pCrsr17>>2]|0;
        $2956 = (_sqlite3BtreeCount($2955,$nEntry)|0);
        HEAP32[$rc>>2] = $2956;
        $2957 = HEAP32[$0>>2]|0;
        $2958 = HEAP32[$pOp>>2]|0;
        $2959 = (_out2Prerelease($2957,$2958)|0);
        HEAP32[$pOut>>2] = $2959;
        $2960 = $nEntry;
        $2961 = $2960;
        $2962 = HEAP32[$2961>>2]|0;
        $2963 = (($2960) + 4)|0;
        $2964 = $2963;
        $2965 = HEAP32[$2964>>2]|0;
        $2966 = HEAP32[$pOut>>2]|0;
        $2967 = $2966;
        $2968 = $2967;
        HEAP32[$2968>>2] = $2962;
        $2969 = (($2967) + 4)|0;
        $2970 = $2969;
        HEAP32[$2970>>2] = $2965;
        break;
       }
       case 0:  {
        $2971 = HEAP32[$pOp>>2]|0;
        $2972 = ((($2971)) + 4|0);
        $2973 = HEAP32[$2972>>2]|0;
        HEAP32[$p118>>2] = $2973;
        $2974 = HEAP32[$pOp>>2]|0;
        $2975 = ((($2974)) + 16|0);
        $2976 = HEAP32[$2975>>2]|0;
        HEAP32[$zName>>2] = $2976;
        $2977 = HEAP32[$p118>>2]|0;
        $2978 = ($2977|0)==(0);
        if ($2978) {
         $2979 = HEAP32[$db>>2]|0;
         $2980 = ((($2979)) + 156|0);
         $2981 = HEAP32[$2980>>2]|0;
         $2982 = ($2981|0)>(0);
         if ($2982) {
          $2983 = HEAP32[$0>>2]|0;
          _sqlite3VdbeError($2983,20918,$vararg_buffer46);
          HEAP32[$rc>>2] = 5;
          break L11;
         }
         $2984 = HEAP32[$zName>>2]|0;
         $2985 = (_sqlite3Strlen30($2984)|0);
         HEAP32[$nName>>2] = $2985;
         $2986 = HEAP32[$db>>2]|0;
         $2987 = HEAP32[$db>>2]|0;
         $2988 = ((($2987)) + 504|0);
         $2989 = HEAP32[$2988>>2]|0;
         $2990 = HEAP32[$db>>2]|0;
         $2991 = ((($2990)) + 500|0);
         $2992 = HEAP32[$2991>>2]|0;
         $2993 = (($2989) + ($2992))|0;
         $2994 = (_sqlite3VtabSavepoint($2986,0,$2993)|0);
         HEAP32[$rc>>2] = $2994;
         $2995 = HEAP32[$rc>>2]|0;
         $2996 = ($2995|0)!=(0);
         if ($2996) {
          label = 861;
          break L8;
         }
         $2997 = HEAP32[$db>>2]|0;
         $2998 = HEAP32[$nName>>2]|0;
         $2999 = (32 + ($2998))|0;
         $3000 = (($2999) + 1)|0;
         $3001 = (_sqlite3DbMallocRawNN($2997,$3000,0)|0);
         HEAP32[$pNew>>2] = $3001;
         $3002 = HEAP32[$pNew>>2]|0;
         $3003 = ($3002|0)!=(0|0);
         if (!($3003)) {
          break L11;
         }
         $3004 = HEAP32[$pNew>>2]|0;
         $3005 = ((($3004)) + 32|0);
         $3006 = HEAP32[$pNew>>2]|0;
         HEAP32[$3006>>2] = $3005;
         $3007 = HEAP32[$pNew>>2]|0;
         $3008 = HEAP32[$3007>>2]|0;
         $3009 = HEAP32[$zName>>2]|0;
         $3010 = HEAP32[$nName>>2]|0;
         $3011 = (($3010) + 1)|0;
         _memcpy(($3008|0),($3009|0),($3011|0))|0;
         $3012 = HEAP32[$db>>2]|0;
         $3013 = ((($3012)) + 63|0);
         $3014 = HEAP8[$3013>>0]|0;
         $3015 = ($3014<<24>>24)!=(0);
         $3016 = HEAP32[$db>>2]|0;
         if ($3015) {
          $3017 = ((($3016)) + 63|0);
          HEAP8[$3017>>0] = 0;
          $3018 = HEAP32[$db>>2]|0;
          $3019 = ((($3018)) + 71|0);
          HEAP8[$3019>>0] = 1;
         } else {
          $3020 = ((($3016)) + 500|0);
          $3021 = HEAP32[$3020>>2]|0;
          $3022 = (($3021) + 1)|0;
          HEAP32[$3020>>2] = $3022;
         }
         $3023 = HEAP32[$db>>2]|0;
         $3024 = ((($3023)) + 492|0);
         $3025 = HEAP32[$3024>>2]|0;
         $3026 = HEAP32[$pNew>>2]|0;
         $3027 = ((($3026)) + 24|0);
         HEAP32[$3027>>2] = $3025;
         $3028 = HEAP32[$pNew>>2]|0;
         $3029 = HEAP32[$db>>2]|0;
         $3030 = ((($3029)) + 492|0);
         HEAP32[$3030>>2] = $3028;
         $3031 = HEAP32[$db>>2]|0;
         $3032 = ((($3031)) + 512|0);
         $3033 = $3032;
         $3034 = $3033;
         $3035 = HEAP32[$3034>>2]|0;
         $3036 = (($3033) + 4)|0;
         $3037 = $3036;
         $3038 = HEAP32[$3037>>2]|0;
         $3039 = HEAP32[$pNew>>2]|0;
         $3040 = ((($3039)) + 8|0);
         $3041 = $3040;
         $3042 = $3041;
         HEAP32[$3042>>2] = $3035;
         $3043 = (($3041) + 4)|0;
         $3044 = $3043;
         HEAP32[$3044>>2] = $3038;
         $3045 = HEAP32[$db>>2]|0;
         $3046 = ((($3045)) + 520|0);
         $3047 = $3046;
         $3048 = $3047;
         $3049 = HEAP32[$3048>>2]|0;
         $3050 = (($3047) + 4)|0;
         $3051 = $3050;
         $3052 = HEAP32[$3051>>2]|0;
         $3053 = HEAP32[$pNew>>2]|0;
         $3054 = ((($3053)) + 16|0);
         $3055 = $3054;
         $3056 = $3055;
         HEAP32[$3056>>2] = $3049;
         $3057 = (($3055) + 4)|0;
         $3058 = $3057;
         HEAP32[$3058>>2] = $3052;
         break L11;
        }
        HEAP32[$iSavepoint>>2] = 0;
        $3059 = HEAP32[$db>>2]|0;
        $3060 = ((($3059)) + 492|0);
        $3061 = HEAP32[$3060>>2]|0;
        HEAP32[$pSavepoint>>2] = $3061;
        while(1) {
         $3062 = HEAP32[$pSavepoint>>2]|0;
         $3063 = ($3062|0)!=(0|0);
         if (!($3063)) {
          break;
         }
         $3064 = HEAP32[$pSavepoint>>2]|0;
         $3065 = HEAP32[$3064>>2]|0;
         $3066 = HEAP32[$zName>>2]|0;
         $3067 = (_sqlite3_stricmp($3065,$3066)|0);
         $3068 = ($3067|0)!=(0);
         if (!($3068)) {
          break;
         }
         $3069 = HEAP32[$iSavepoint>>2]|0;
         $3070 = (($3069) + 1)|0;
         HEAP32[$iSavepoint>>2] = $3070;
         $3071 = HEAP32[$pSavepoint>>2]|0;
         $3072 = ((($3071)) + 24|0);
         $3073 = HEAP32[$3072>>2]|0;
         HEAP32[$pSavepoint>>2] = $3073;
        }
        $3074 = HEAP32[$pSavepoint>>2]|0;
        $3075 = ($3074|0)!=(0|0);
        if (!($3075)) {
         $3076 = HEAP32[$0>>2]|0;
         $3077 = HEAP32[$zName>>2]|0;
         HEAP32[$vararg_buffer48>>2] = $3077;
         _sqlite3VdbeError($3076,20969,$vararg_buffer48);
         HEAP32[$rc>>2] = 1;
         break L11;
        }
        $3078 = HEAP32[$db>>2]|0;
        $3079 = ((($3078)) + 156|0);
        $3080 = HEAP32[$3079>>2]|0;
        $3081 = ($3080|0)>(0);
        $3082 = HEAP32[$p118>>2]|0;
        $3083 = ($3082|0)==(1);
        $or$cond6 = $3081 & $3083;
        if ($or$cond6) {
         $3084 = HEAP32[$0>>2]|0;
         _sqlite3VdbeError($3084,20991,$vararg_buffer51);
         HEAP32[$rc>>2] = 5;
         break L11;
        }
        $3085 = HEAP32[$pSavepoint>>2]|0;
        $3086 = ((($3085)) + 24|0);
        $3087 = HEAP32[$3086>>2]|0;
        $3088 = ($3087|0)==(0|0);
        if ($3088) {
         $3089 = HEAP32[$db>>2]|0;
         $3090 = ((($3089)) + 71|0);
         $3091 = HEAP8[$3090>>0]|0;
         $3092 = $3091&255;
         $3093 = ($3092|0)!=(0);
         $3095 = $3093;
        } else {
         $3095 = 0;
        }
        $3094 = $3095&1;
        HEAP32[$isTransaction>>2] = $3094;
        $3096 = HEAP32[$isTransaction>>2]|0;
        $3097 = ($3096|0)!=(0);
        $3098 = HEAP32[$p118>>2]|0;
        $3099 = ($3098|0)==(1);
        $or$cond8 = $3097 & $3099;
        if ($or$cond8) {
         $3100 = HEAP32[$0>>2]|0;
         $3101 = (_sqlite3VdbeCheckFk($3100,1)|0);
         HEAP32[$rc>>2] = $3101;
         $3102 = ($3101|0)!=(0);
         if ($3102) {
          label = 858;
          break L8;
         }
         $3103 = HEAP32[$db>>2]|0;
         $3104 = ((($3103)) + 63|0);
         HEAP8[$3104>>0] = 1;
         $3105 = HEAP32[$0>>2]|0;
         $3106 = (_sqlite3VdbeHalt($3105)|0);
         $3107 = ($3106|0)==(5);
         if ($3107) {
          label = 367;
          break L8;
         }
         $3120 = HEAP32[$db>>2]|0;
         $3121 = ((($3120)) + 71|0);
         HEAP8[$3121>>0] = 0;
         $3122 = HEAP32[$0>>2]|0;
         $3123 = ((($3122)) + 80|0);
         $3124 = HEAP32[$3123>>2]|0;
         HEAP32[$rc>>2] = $3124;
        } else {
         $3125 = HEAP32[$db>>2]|0;
         $3126 = ((($3125)) + 500|0);
         $3127 = HEAP32[$3126>>2]|0;
         $3128 = HEAP32[$iSavepoint>>2]|0;
         $3129 = (($3127) - ($3128))|0;
         $3130 = (($3129) - 1)|0;
         HEAP32[$iSavepoint>>2] = $3130;
         $3131 = HEAP32[$p118>>2]|0;
         $3132 = ($3131|0)==(2);
         L419: do {
          if ($3132) {
           $3133 = HEAP32[$db>>2]|0;
           $3134 = ((($3133)) + 24|0);
           $3135 = HEAP32[$3134>>2]|0;
           $3136 = $3135 & 2;
           $3137 = ($3136|0)!=(0);
           $3138 = $3137&1;
           HEAP32[$isSchemaChange>>2] = $3138;
           HEAP32[$ii>>2] = 0;
           while(1) {
            $3139 = HEAP32[$ii>>2]|0;
            $3140 = HEAP32[$db>>2]|0;
            $3141 = ((($3140)) + 20|0);
            $3142 = HEAP32[$3141>>2]|0;
            $3143 = ($3139|0)<($3142|0);
            if (!($3143)) {
             break L419;
            }
            $3144 = HEAP32[$ii>>2]|0;
            $3145 = HEAP32[$db>>2]|0;
            $3146 = ((($3145)) + 16|0);
            $3147 = HEAP32[$3146>>2]|0;
            $3148 = (($3147) + ($3144<<4)|0);
            $3149 = ((($3148)) + 4|0);
            $3150 = HEAP32[$3149>>2]|0;
            $3151 = HEAP32[$isSchemaChange>>2]|0;
            $3152 = ($3151|0)==(0);
            $3153 = $3152&1;
            $3154 = (_sqlite3BtreeTripAllCursors($3150,516,$3153)|0);
            HEAP32[$rc>>2] = $3154;
            $3155 = HEAP32[$rc>>2]|0;
            $3156 = ($3155|0)!=(0);
            if ($3156) {
             label = 861;
             break L8;
            }
            $3157 = HEAP32[$ii>>2]|0;
            $3158 = (($3157) + 1)|0;
            HEAP32[$ii>>2] = $3158;
           }
          } else {
           HEAP32[$isSchemaChange>>2] = 0;
          }
         } while(0);
         HEAP32[$ii>>2] = 0;
         while(1) {
          $3159 = HEAP32[$ii>>2]|0;
          $3160 = HEAP32[$db>>2]|0;
          $3161 = ((($3160)) + 20|0);
          $3162 = HEAP32[$3161>>2]|0;
          $3163 = ($3159|0)<($3162|0);
          if (!($3163)) {
           break;
          }
          $3164 = HEAP32[$ii>>2]|0;
          $3165 = HEAP32[$db>>2]|0;
          $3166 = ((($3165)) + 16|0);
          $3167 = HEAP32[$3166>>2]|0;
          $3168 = (($3167) + ($3164<<4)|0);
          $3169 = ((($3168)) + 4|0);
          $3170 = HEAP32[$3169>>2]|0;
          $3171 = HEAP32[$p118>>2]|0;
          $3172 = HEAP32[$iSavepoint>>2]|0;
          $3173 = (_sqlite3BtreeSavepoint($3170,$3171,$3172)|0);
          HEAP32[$rc>>2] = $3173;
          $3174 = HEAP32[$rc>>2]|0;
          $3175 = ($3174|0)!=(0);
          if ($3175) {
           label = 861;
           break L8;
          }
          $3176 = HEAP32[$ii>>2]|0;
          $3177 = (($3176) + 1)|0;
          HEAP32[$ii>>2] = $3177;
         }
         $3178 = HEAP32[$isSchemaChange>>2]|0;
         $3179 = ($3178|0)!=(0);
         if ($3179) {
          $3180 = HEAP32[$db>>2]|0;
          _sqlite3ExpirePreparedStatements($3180);
          $3181 = HEAP32[$db>>2]|0;
          _sqlite3ResetAllSchemasOfConnection($3181);
          $3182 = HEAP32[$db>>2]|0;
          $3183 = ((($3182)) + 24|0);
          $3184 = HEAP32[$3183>>2]|0;
          $3185 = $3184 | 2;
          $3186 = HEAP32[$db>>2]|0;
          $3187 = ((($3186)) + 24|0);
          HEAP32[$3187>>2] = $3185;
         }
        }
        while(1) {
         $3188 = HEAP32[$db>>2]|0;
         $3189 = ((($3188)) + 492|0);
         $3190 = HEAP32[$3189>>2]|0;
         $3191 = HEAP32[$pSavepoint>>2]|0;
         $3192 = ($3190|0)!=($3191|0);
         if (!($3192)) {
          break;
         }
         $3193 = HEAP32[$db>>2]|0;
         $3194 = ((($3193)) + 492|0);
         $3195 = HEAP32[$3194>>2]|0;
         HEAP32[$pTmp>>2] = $3195;
         $3196 = HEAP32[$pTmp>>2]|0;
         $3197 = ((($3196)) + 24|0);
         $3198 = HEAP32[$3197>>2]|0;
         $3199 = HEAP32[$db>>2]|0;
         $3200 = ((($3199)) + 492|0);
         HEAP32[$3200>>2] = $3198;
         $3201 = HEAP32[$db>>2]|0;
         $3202 = HEAP32[$pTmp>>2]|0;
         _sqlite3DbFree($3201,$3202);
         $3203 = HEAP32[$db>>2]|0;
         $3204 = ((($3203)) + 500|0);
         $3205 = HEAP32[$3204>>2]|0;
         $3206 = (($3205) + -1)|0;
         HEAP32[$3204>>2] = $3206;
        }
        $3207 = HEAP32[$p118>>2]|0;
        $3208 = ($3207|0)==(1);
        $3209 = HEAP32[$pSavepoint>>2]|0;
        if ($3208) {
         $3210 = ((($3209)) + 24|0);
         $3211 = HEAP32[$3210>>2]|0;
         $3212 = HEAP32[$db>>2]|0;
         $3213 = ((($3212)) + 492|0);
         HEAP32[$3213>>2] = $3211;
         $3214 = HEAP32[$db>>2]|0;
         $3215 = HEAP32[$pSavepoint>>2]|0;
         _sqlite3DbFree($3214,$3215);
         $3216 = HEAP32[$isTransaction>>2]|0;
         $3217 = ($3216|0)!=(0);
         if (!($3217)) {
          $3218 = HEAP32[$db>>2]|0;
          $3219 = ((($3218)) + 500|0);
          $3220 = HEAP32[$3219>>2]|0;
          $3221 = (($3220) + -1)|0;
          HEAP32[$3219>>2] = $3221;
         }
        } else {
         $3222 = ((($3209)) + 8|0);
         $3223 = $3222;
         $3224 = $3223;
         $3225 = HEAP32[$3224>>2]|0;
         $3226 = (($3223) + 4)|0;
         $3227 = $3226;
         $3228 = HEAP32[$3227>>2]|0;
         $3229 = HEAP32[$db>>2]|0;
         $3230 = ((($3229)) + 512|0);
         $3231 = $3230;
         $3232 = $3231;
         HEAP32[$3232>>2] = $3225;
         $3233 = (($3231) + 4)|0;
         $3234 = $3233;
         HEAP32[$3234>>2] = $3228;
         $3235 = HEAP32[$pSavepoint>>2]|0;
         $3236 = ((($3235)) + 16|0);
         $3237 = $3236;
         $3238 = $3237;
         $3239 = HEAP32[$3238>>2]|0;
         $3240 = (($3237) + 4)|0;
         $3241 = $3240;
         $3242 = HEAP32[$3241>>2]|0;
         $3243 = HEAP32[$db>>2]|0;
         $3244 = ((($3243)) + 520|0);
         $3245 = $3244;
         $3246 = $3245;
         HEAP32[$3246>>2] = $3239;
         $3247 = (($3245) + 4)|0;
         $3248 = $3247;
         HEAP32[$3248>>2] = $3242;
        }
        $3249 = HEAP32[$isTransaction>>2]|0;
        $3250 = ($3249|0)==(0);
        $3251 = HEAP32[$p118>>2]|0;
        $3252 = ($3251|0)==(2);
        $or$cond10 = $3250 | $3252;
        if ($or$cond10) {
         $3253 = HEAP32[$db>>2]|0;
         $3254 = HEAP32[$p118>>2]|0;
         $3255 = HEAP32[$iSavepoint>>2]|0;
         $3256 = (_sqlite3VtabSavepoint($3253,$3254,$3255)|0);
         HEAP32[$rc>>2] = $3256;
         $3257 = HEAP32[$rc>>2]|0;
         $3258 = ($3257|0)!=(0);
         if ($3258) {
          label = 861;
          break L8;
         }
        }
        break;
       }
       case 1:  {
        $3259 = HEAP32[$pOp>>2]|0;
        $3260 = ((($3259)) + 4|0);
        $3261 = HEAP32[$3260>>2]|0;
        HEAP32[$desiredAutoCommit>>2] = $3261;
        $3262 = HEAP32[$pOp>>2]|0;
        $3263 = ((($3262)) + 8|0);
        $3264 = HEAP32[$3263>>2]|0;
        HEAP32[$iRollback>>2] = $3264;
        $3265 = HEAP32[$desiredAutoCommit>>2]|0;
        $3266 = HEAP32[$db>>2]|0;
        $3267 = ((($3266)) + 63|0);
        $3268 = HEAP8[$3267>>0]|0;
        $3269 = $3268&255;
        $3270 = ($3265|0)!=($3269|0);
        if (!($3270)) {
         $3313 = HEAP32[$0>>2]|0;
         $3314 = HEAP32[$desiredAutoCommit>>2]|0;
         $3315 = ($3314|0)!=(0);
         if ($3315) {
          $3316 = HEAP32[$iRollback>>2]|0;
          $3317 = ($3316|0)!=(0);
          $3318 = $3317 ? 21148 : 21191;
          $3319 = $3318;
         } else {
          $3319 = 21100;
         }
         _sqlite3VdbeError($3313,$3319,$vararg_buffer55);
         HEAP32[$rc>>2] = 1;
         break L11;
        }
        $3271 = HEAP32[$iRollback>>2]|0;
        $3272 = ($3271|0)!=(0);
        if ($3272) {
         label = 391;
         break L8;
        }
        $3276 = HEAP32[$desiredAutoCommit>>2]|0;
        $3277 = ($3276|0)!=(0);
        if (!($3277)) {
         label = 395;
         break L8;
        }
        $3278 = HEAP32[$db>>2]|0;
        $3279 = ((($3278)) + 156|0);
        $3280 = HEAP32[$3279>>2]|0;
        $3281 = ($3280|0)>(0);
        if (!($3281)) {
         label = 395;
         break L8;
        }
        $3282 = HEAP32[$0>>2]|0;
        _sqlite3VdbeError($3282,21045,$vararg_buffer53);
        HEAP32[$rc>>2] = 5;
        break;
       }
       case 2:  {
        $3320 = HEAP32[$pOp>>2]|0;
        $3321 = ((($3320)) + 8|0);
        $3322 = HEAP32[$3321>>2]|0;
        $3323 = ($3322|0)!=(0);
        if ($3323) {
         $3324 = HEAP32[$db>>2]|0;
         $3325 = ((($3324)) + 24|0);
         $3326 = HEAP32[$3325>>2]|0;
         $3327 = $3326 & 33554432;
         $3328 = ($3327|0)!=(0);
         if ($3328) {
          label = 407;
          break L8;
         }
        }
        $3329 = HEAP32[$pOp>>2]|0;
        $3330 = ((($3329)) + 4|0);
        $3331 = HEAP32[$3330>>2]|0;
        $3332 = HEAP32[$db>>2]|0;
        $3333 = ((($3332)) + 16|0);
        $3334 = HEAP32[$3333>>2]|0;
        $3335 = (($3334) + ($3331<<4)|0);
        $3336 = ((($3335)) + 4|0);
        $3337 = HEAP32[$3336>>2]|0;
        HEAP32[$pBt>>2] = $3337;
        $3338 = HEAP32[$pBt>>2]|0;
        $3339 = ($3338|0)!=(0|0);
        if ($3339) {
         $3340 = HEAP32[$pBt>>2]|0;
         $3341 = HEAP32[$pOp>>2]|0;
         $3342 = ((($3341)) + 8|0);
         $3343 = HEAP32[$3342>>2]|0;
         $3344 = (_sqlite3BtreeBeginTrans($3340,$3343)|0);
         HEAP32[$rc>>2] = $3344;
         $3345 = HEAP32[$rc>>2]|0;
         $3346 = $3345 & 255;
         $3347 = ($3346|0)==(5);
         if ($3347) {
          label = 410;
          break L8;
         }
         $3359 = HEAP32[$rc>>2]|0;
         $3360 = ($3359|0)!=(0);
         if ($3360) {
          label = 861;
          break L8;
         }
         $3361 = HEAP32[$pOp>>2]|0;
         $3362 = ((($3361)) + 8|0);
         $3363 = HEAP32[$3362>>2]|0;
         $3364 = ($3363|0)!=(0);
         do {
          if ($3364) {
           $3365 = HEAP32[$0>>2]|0;
           $3366 = ((($3365)) + 88|0);
           $3367 = HEAP16[$3366>>1]|0;
           $3368 = ($3367&65535) >>> 5;
           $3369 = $3368 & 1;
           $3370 = $3369&65535;
           $3371 = ($3370|0)!=(0);
           if ($3371) {
            $3372 = HEAP32[$db>>2]|0;
            $3373 = ((($3372)) + 63|0);
            $3374 = HEAP8[$3373>>0]|0;
            $3375 = $3374&255;
            $3376 = ($3375|0)==(0);
            if (!($3376)) {
             $3377 = HEAP32[$db>>2]|0;
             $3378 = ((($3377)) + 152|0);
             $3379 = HEAP32[$3378>>2]|0;
             $3380 = ($3379|0)>(1);
             if (!($3380)) {
              break;
             }
            }
            $3381 = HEAP32[$0>>2]|0;
            $3382 = ((($3381)) + 104|0);
            $3383 = HEAP32[$3382>>2]|0;
            $3384 = ($3383|0)==(0);
            if ($3384) {
             $3385 = HEAP32[$db>>2]|0;
             $3386 = ((($3385)) + 504|0);
             $3387 = HEAP32[$3386>>2]|0;
             $3388 = (($3387) + 1)|0;
             HEAP32[$3386>>2] = $3388;
             $3389 = HEAP32[$db>>2]|0;
             $3390 = ((($3389)) + 500|0);
             $3391 = HEAP32[$3390>>2]|0;
             $3392 = HEAP32[$db>>2]|0;
             $3393 = ((($3392)) + 504|0);
             $3394 = HEAP32[$3393>>2]|0;
             $3395 = (($3391) + ($3394))|0;
             $3396 = HEAP32[$0>>2]|0;
             $3397 = ((($3396)) + 104|0);
             HEAP32[$3397>>2] = $3395;
            }
            $3398 = HEAP32[$db>>2]|0;
            $3399 = HEAP32[$0>>2]|0;
            $3400 = ((($3399)) + 104|0);
            $3401 = HEAP32[$3400>>2]|0;
            $3402 = (($3401) - 1)|0;
            $3403 = (_sqlite3VtabSavepoint($3398,0,$3402)|0);
            HEAP32[$rc>>2] = $3403;
            $3404 = HEAP32[$rc>>2]|0;
            $3405 = ($3404|0)==(0);
            if ($3405) {
             $3406 = HEAP32[$pBt>>2]|0;
             $3407 = HEAP32[$0>>2]|0;
             $3408 = ((($3407)) + 104|0);
             $3409 = HEAP32[$3408>>2]|0;
             $3410 = (_sqlite3BtreeBeginStmt($3406,$3409)|0);
             HEAP32[$rc>>2] = $3410;
            }
            $3411 = HEAP32[$db>>2]|0;
            $3412 = ((($3411)) + 512|0);
            $3413 = $3412;
            $3414 = $3413;
            $3415 = HEAP32[$3414>>2]|0;
            $3416 = (($3413) + 4)|0;
            $3417 = $3416;
            $3418 = HEAP32[$3417>>2]|0;
            $3419 = HEAP32[$0>>2]|0;
            $3420 = ((($3419)) + 152|0);
            $3421 = $3420;
            $3422 = $3421;
            HEAP32[$3422>>2] = $3415;
            $3423 = (($3421) + 4)|0;
            $3424 = $3423;
            HEAP32[$3424>>2] = $3418;
            $3425 = HEAP32[$db>>2]|0;
            $3426 = ((($3425)) + 520|0);
            $3427 = $3426;
            $3428 = $3427;
            $3429 = HEAP32[$3428>>2]|0;
            $3430 = (($3427) + 4)|0;
            $3431 = $3430;
            $3432 = HEAP32[$3431>>2]|0;
            $3433 = HEAP32[$0>>2]|0;
            $3434 = ((($3433)) + 160|0);
            $3435 = $3434;
            $3436 = $3435;
            HEAP32[$3436>>2] = $3429;
            $3437 = (($3435) + 4)|0;
            $3438 = $3437;
            HEAP32[$3438>>2] = $3432;
           }
          }
         } while(0);
         $3439 = HEAP32[$pBt>>2]|0;
         _sqlite3BtreeGetMeta($3439,1,$iMeta);
         $3440 = HEAP32[$pOp>>2]|0;
         $3441 = ((($3440)) + 4|0);
         $3442 = HEAP32[$3441>>2]|0;
         $3443 = HEAP32[$db>>2]|0;
         $3444 = ((($3443)) + 16|0);
         $3445 = HEAP32[$3444>>2]|0;
         $3446 = (($3445) + ($3442<<4)|0);
         $3447 = ((($3446)) + 12|0);
         $3448 = HEAP32[$3447>>2]|0;
         $3449 = ((($3448)) + 4|0);
         $3450 = HEAP32[$3449>>2]|0;
         HEAP32[$iGen>>2] = $3450;
        } else {
         HEAP32[$iMeta>>2] = 0;
         HEAP32[$iGen>>2] = 0;
        }
        $3451 = HEAP32[$pOp>>2]|0;
        $3452 = ((($3451)) + 3|0);
        $3453 = HEAP8[$3452>>0]|0;
        $3454 = $3453&255;
        $3455 = ($3454|0)!=(0);
        if ($3455) {
         $3456 = HEAP32[$iMeta>>2]|0;
         $3457 = HEAP32[$pOp>>2]|0;
         $3458 = ((($3457)) + 12|0);
         $3459 = HEAP32[$3458>>2]|0;
         $3460 = ($3456|0)!=($3459|0);
         if (!($3460)) {
          $3461 = HEAP32[$iGen>>2]|0;
          $3462 = HEAP32[$pOp>>2]|0;
          $3463 = ((($3462)) + 16|0);
          $3464 = HEAP32[$3463>>2]|0;
          $3465 = ($3461|0)!=($3464|0);
          if (!($3465)) {
           break L11;
          }
         }
         $3466 = HEAP32[$db>>2]|0;
         $3467 = HEAP32[$0>>2]|0;
         $3468 = ((($3467)) + 44|0);
         $3469 = HEAP32[$3468>>2]|0;
         _sqlite3DbFree($3466,$3469);
         $3470 = HEAP32[$db>>2]|0;
         $3471 = (_sqlite3DbStrDup($3470,21232)|0);
         $3472 = HEAP32[$0>>2]|0;
         $3473 = ((($3472)) + 44|0);
         HEAP32[$3473>>2] = $3471;
         $3474 = HEAP32[$pOp>>2]|0;
         $3475 = ((($3474)) + 4|0);
         $3476 = HEAP32[$3475>>2]|0;
         $3477 = HEAP32[$db>>2]|0;
         $3478 = ((($3477)) + 16|0);
         $3479 = HEAP32[$3478>>2]|0;
         $3480 = (($3479) + ($3476<<4)|0);
         $3481 = ((($3480)) + 12|0);
         $3482 = HEAP32[$3481>>2]|0;
         $3483 = HEAP32[$3482>>2]|0;
         $3484 = HEAP32[$iMeta>>2]|0;
         $3485 = ($3483|0)!=($3484|0);
         if ($3485) {
          $3486 = HEAP32[$db>>2]|0;
          $3487 = HEAP32[$pOp>>2]|0;
          $3488 = ((($3487)) + 4|0);
          $3489 = HEAP32[$3488>>2]|0;
          _sqlite3ResetOneSchema($3486,$3489);
         }
         $3490 = HEAP32[$0>>2]|0;
         $3491 = ((($3490)) + 88|0);
         $3492 = HEAP16[$3491>>1]|0;
         $3493 = $3492 & -9;
         $3494 = $3493 | 8;
         HEAP16[$3491>>1] = $3494;
         HEAP32[$rc>>2] = 17;
        }
        break;
       }
       case 51:  {
        $3495 = HEAP32[$pOp>>2]|0;
        $3496 = ((($3495)) + 4|0);
        $3497 = HEAP32[$3496>>2]|0;
        HEAP32[$iDb>>2] = $3497;
        $3498 = HEAP32[$pOp>>2]|0;
        $3499 = ((($3498)) + 12|0);
        $3500 = HEAP32[$3499>>2]|0;
        HEAP32[$iCookie>>2] = $3500;
        $3501 = HEAP32[$iDb>>2]|0;
        $3502 = HEAP32[$db>>2]|0;
        $3503 = ((($3502)) + 16|0);
        $3504 = HEAP32[$3503>>2]|0;
        $3505 = (($3504) + ($3501<<4)|0);
        $3506 = ((($3505)) + 4|0);
        $3507 = HEAP32[$3506>>2]|0;
        $3508 = HEAP32[$iCookie>>2]|0;
        _sqlite3BtreeGetMeta($3507,$3508,$iMeta19);
        $3509 = HEAP32[$0>>2]|0;
        $3510 = HEAP32[$pOp>>2]|0;
        $3511 = (_out2Prerelease($3509,$3510)|0);
        HEAP32[$pOut>>2] = $3511;
        $3512 = HEAP32[$iMeta19>>2]|0;
        $3513 = ($3512|0)<(0);
        $3514 = $3513 << 31 >> 31;
        $3515 = HEAP32[$pOut>>2]|0;
        $3516 = $3515;
        $3517 = $3516;
        HEAP32[$3517>>2] = $3512;
        $3518 = (($3516) + 4)|0;
        $3519 = $3518;
        HEAP32[$3519>>2] = $3514;
        break;
       }
       case 52:  {
        $3520 = HEAP32[$pOp>>2]|0;
        $3521 = ((($3520)) + 4|0);
        $3522 = HEAP32[$3521>>2]|0;
        $3523 = HEAP32[$db>>2]|0;
        $3524 = ((($3523)) + 16|0);
        $3525 = HEAP32[$3524>>2]|0;
        $3526 = (($3525) + ($3522<<4)|0);
        HEAP32[$pDb>>2] = $3526;
        $3527 = HEAP32[$pDb>>2]|0;
        $3528 = ((($3527)) + 4|0);
        $3529 = HEAP32[$3528>>2]|0;
        $3530 = HEAP32[$pOp>>2]|0;
        $3531 = ((($3530)) + 8|0);
        $3532 = HEAP32[$3531>>2]|0;
        $3533 = HEAP32[$pOp>>2]|0;
        $3534 = ((($3533)) + 12|0);
        $3535 = HEAP32[$3534>>2]|0;
        $3536 = (_sqlite3BtreeUpdateMeta($3529,$3532,$3535)|0);
        HEAP32[$rc>>2] = $3536;
        $3537 = HEAP32[$pOp>>2]|0;
        $3538 = ((($3537)) + 8|0);
        $3539 = HEAP32[$3538>>2]|0;
        $3540 = ($3539|0)==(1);
        $3541 = HEAP32[$pOp>>2]|0;
        if ($3540) {
         $3542 = ((($3541)) + 12|0);
         $3543 = HEAP32[$3542>>2]|0;
         $3544 = HEAP32[$pDb>>2]|0;
         $3545 = ((($3544)) + 12|0);
         $3546 = HEAP32[$3545>>2]|0;
         HEAP32[$3546>>2] = $3543;
         $3547 = HEAP32[$db>>2]|0;
         $3548 = ((($3547)) + 24|0);
         $3549 = HEAP32[$3548>>2]|0;
         $3550 = $3549 | 2;
         HEAP32[$3548>>2] = $3550;
        } else {
         $3551 = ((($3541)) + 8|0);
         $3552 = HEAP32[$3551>>2]|0;
         $3553 = ($3552|0)==(2);
         if ($3553) {
          $3554 = HEAP32[$pOp>>2]|0;
          $3555 = ((($3554)) + 12|0);
          $3556 = HEAP32[$3555>>2]|0;
          $3557 = $3556&255;
          $3558 = HEAP32[$pDb>>2]|0;
          $3559 = ((($3558)) + 12|0);
          $3560 = HEAP32[$3559>>2]|0;
          $3561 = ((($3560)) + 76|0);
          HEAP8[$3561>>0] = $3557;
         }
        }
        $3562 = HEAP32[$pOp>>2]|0;
        $3563 = ((($3562)) + 4|0);
        $3564 = HEAP32[$3563>>2]|0;
        $3565 = ($3564|0)==(1);
        if ($3565) {
         $3566 = HEAP32[$db>>2]|0;
         _sqlite3ExpirePreparedStatements($3566);
         $3567 = HEAP32[$0>>2]|0;
         $3568 = ((($3567)) + 88|0);
         $3569 = HEAP16[$3568>>1]|0;
         $3570 = $3569 & -9;
         HEAP16[$3568>>1] = $3570;
        }
        break;
       }
       case 53:  {
        $3571 = HEAP32[$pOp>>2]|0;
        $3572 = ((($3571)) + 4|0);
        $3573 = HEAP32[$3572>>2]|0;
        $3574 = HEAP32[$0>>2]|0;
        $3575 = ((($3574)) + 56|0);
        $3576 = HEAP32[$3575>>2]|0;
        $3577 = (($3576) + ($3573<<2)|0);
        $3578 = HEAP32[$3577>>2]|0;
        HEAP32[$pCur>>2] = $3578;
        $3579 = HEAP32[$pCur>>2]|0;
        $3580 = ($3579|0)!=(0|0);
        if ($3580) {
         $3581 = HEAP32[$pCur>>2]|0;
         $3582 = ((($3581)) + 8|0);
         $3583 = HEAP32[$3582>>2]|0;
         $3584 = HEAP32[$pOp>>2]|0;
         $3585 = ((($3584)) + 8|0);
         $3586 = HEAP32[$3585>>2]|0;
         $3587 = ($3583|0)==($3586|0);
         if ($3587) {
          label = 453;
         } else {
          label = 438;
         }
        } else {
         label = 438;
        }
        break;
       }
       case 55: case 54:  {
        label = 438;
        break;
       }
       case 57: case 56:  {
        $3728 = HEAP32[$0>>2]|0;
        $3729 = HEAP32[$pOp>>2]|0;
        $3730 = ((($3729)) + 4|0);
        $3731 = HEAP32[$3730>>2]|0;
        $3732 = HEAP32[$pOp>>2]|0;
        $3733 = ((($3732)) + 8|0);
        $3734 = HEAP32[$3733>>2]|0;
        $3735 = (_allocateCursor($3728,$3731,$3734,-1,0)|0);
        HEAP32[$pCx>>2] = $3735;
        $3736 = HEAP32[$pCx>>2]|0;
        $3737 = ($3736|0)==(0|0);
        if ($3737) {
         label = 860;
         break L1;
        }
        $3738 = HEAP32[$pCx>>2]|0;
        $3739 = ((($3738)) + 2|0);
        HEAP8[$3739>>0] = 1;
        $3740 = HEAP32[$pCx>>2]|0;
        $3741 = ((($3740)) + 5|0);
        $3742 = HEAP8[$3741>>0]|0;
        $3743 = $3742 & -2;
        $3744 = $3743 | 1;
        HEAP8[$3741>>0] = $3744;
        $3745 = HEAP32[$db>>2]|0;
        $3746 = HEAP32[$3745>>2]|0;
        $3747 = HEAP32[$db>>2]|0;
        $3748 = HEAP32[$pCx>>2]|0;
        $3749 = ((($3748)) + 20|0);
        $3750 = HEAP32[$pOp>>2]|0;
        $3751 = ((($3750)) + 3|0);
        $3752 = HEAP8[$3751>>0]|0;
        $3753 = $3752&255;
        $3754 = 5 | $3753;
        $3755 = (_sqlite3BtreeOpen($3746,0,$3747,$3749,$3754,1054)|0);
        HEAP32[$rc>>2] = $3755;
        $3756 = HEAP32[$rc>>2]|0;
        $3757 = ($3756|0)==(0);
        if ($3757) {
         $3758 = HEAP32[$pCx>>2]|0;
         $3759 = ((($3758)) + 20|0);
         $3760 = HEAP32[$3759>>2]|0;
         $3761 = (_sqlite3BtreeBeginTrans($3760,1)|0);
         HEAP32[$rc>>2] = $3761;
        }
        $3762 = HEAP32[$rc>>2]|0;
        $3763 = ($3762|0)==(0);
        do {
         if ($3763) {
          $3764 = HEAP32[$pOp>>2]|0;
          $3765 = ((($3764)) + 16|0);
          $3766 = HEAP32[$3765>>2]|0;
          HEAP32[$pKeyInfo25>>2] = $3766;
          $3767 = ($3766|0)!=(0|0);
          $3768 = HEAP32[$pCx>>2]|0;
          $3769 = ((($3768)) + 20|0);
          $3770 = HEAP32[$3769>>2]|0;
          if (!($3767)) {
           $3793 = HEAP32[$pCx>>2]|0;
           $3794 = ((($3793)) + 16|0);
           $3795 = HEAP32[$3794>>2]|0;
           $3796 = (_sqlite3BtreeCursor($3770,1,4,0,$3795)|0);
           HEAP32[$rc>>2] = $3796;
           $3797 = HEAP32[$pCx>>2]|0;
           $3798 = ((($3797)) + 4|0);
           HEAP8[$3798>>0] = 1;
           break;
          }
          $3771 = HEAP32[$pOp>>2]|0;
          $3772 = ((($3771)) + 3|0);
          $3773 = HEAP8[$3772>>0]|0;
          $3774 = $3773&255;
          $3775 = 2 | $3774;
          $3776 = (_sqlite3BtreeCreateTable($3770,$pgno,$3775)|0);
          HEAP32[$rc>>2] = $3776;
          $3777 = HEAP32[$rc>>2]|0;
          $3778 = ($3777|0)==(0);
          if ($3778) {
           $3779 = HEAP32[$pKeyInfo25>>2]|0;
           $3780 = HEAP32[$pCx>>2]|0;
           $3781 = ((($3780)) + 24|0);
           HEAP32[$3781>>2] = $3779;
           $3782 = HEAP32[$pCx>>2]|0;
           $3783 = ((($3782)) + 20|0);
           $3784 = HEAP32[$3783>>2]|0;
           $3785 = HEAP32[$pgno>>2]|0;
           $3786 = HEAP32[$pKeyInfo25>>2]|0;
           $3787 = HEAP32[$pCx>>2]|0;
           $3788 = ((($3787)) + 16|0);
           $3789 = HEAP32[$3788>>2]|0;
           $3790 = (_sqlite3BtreeCursor($3784,$3785,4,$3786,$3789)|0);
           HEAP32[$rc>>2] = $3790;
          }
          $3791 = HEAP32[$pCx>>2]|0;
          $3792 = ((($3791)) + 4|0);
          HEAP8[$3792>>0] = 0;
         }
        } while(0);
        $3799 = HEAP32[$pOp>>2]|0;
        $3800 = ((($3799)) + 3|0);
        $3801 = HEAP8[$3800>>0]|0;
        $3802 = $3801&255;
        $3803 = ($3802|0)!=(8);
        $3804 = $3803&1;
        $3805 = HEAP32[$pCx>>2]|0;
        $3806 = ((($3805)) + 5|0);
        $3807 = $3804&255;
        $3808 = HEAP8[$3806>>0]|0;
        $3809 = $3807 & 1;
        $3810 = ($3809 << 2)&255;
        $3811 = $3808 & -5;
        $3812 = $3811 | $3810;
        HEAP8[$3806>>0] = $3812;
        break;
       }
       case 58:  {
        $3813 = HEAP32[$0>>2]|0;
        $3814 = HEAP32[$pOp>>2]|0;
        $3815 = ((($3814)) + 4|0);
        $3816 = HEAP32[$3815>>2]|0;
        $3817 = HEAP32[$pOp>>2]|0;
        $3818 = ((($3817)) + 8|0);
        $3819 = HEAP32[$3818>>2]|0;
        $3820 = (_allocateCursor($3813,$3816,$3819,-1,1)|0);
        HEAP32[$pCx26>>2] = $3820;
        $3821 = HEAP32[$pCx26>>2]|0;
        $3822 = ($3821|0)==(0|0);
        if ($3822) {
         label = 860;
         break L1;
        }
        $3823 = HEAP32[$pOp>>2]|0;
        $3824 = ((($3823)) + 16|0);
        $3825 = HEAP32[$3824>>2]|0;
        $3826 = HEAP32[$pCx26>>2]|0;
        $3827 = ((($3826)) + 24|0);
        HEAP32[$3827>>2] = $3825;
        $3828 = HEAP32[$db>>2]|0;
        $3829 = HEAP32[$pOp>>2]|0;
        $3830 = ((($3829)) + 12|0);
        $3831 = HEAP32[$3830>>2]|0;
        $3832 = HEAP32[$pCx26>>2]|0;
        $3833 = (_sqlite3VdbeSorterInit($3828,$3831,$3832)|0);
        HEAP32[$rc>>2] = $3833;
        break;
       }
       case 59:  {
        $3834 = HEAP32[$pOp>>2]|0;
        $3835 = ((($3834)) + 4|0);
        $3836 = HEAP32[$3835>>2]|0;
        $3837 = HEAP32[$0>>2]|0;
        $3838 = ((($3837)) + 56|0);
        $3839 = HEAP32[$3838>>2]|0;
        $3840 = (($3839) + ($3836<<2)|0);
        $3841 = HEAP32[$3840>>2]|0;
        HEAP32[$pC27>>2] = $3841;
        $3842 = HEAP32[$pC27>>2]|0;
        $3843 = ((($3842)) + 32|0);
        $3844 = $3843;
        $3845 = $3844;
        $3846 = HEAP32[$3845>>2]|0;
        $3847 = (($3844) + 4)|0;
        $3848 = $3847;
        $3849 = HEAP32[$3848>>2]|0;
        $3850 = (_i64Add(($3846|0),($3849|0),1,0)|0);
        $3851 = tempRet0;
        $3852 = $3843;
        $3853 = $3852;
        HEAP32[$3853>>2] = $3850;
        $3854 = (($3852) + 4)|0;
        $3855 = $3854;
        HEAP32[$3855>>2] = $3851;
        $3856 = ($3846|0)==(0);
        $3857 = ($3849|0)==(0);
        $3858 = $3856 & $3857;
        if ($3858) {
         label = 15;
        }
        break;
       }
       case 60:  {
        $3859 = HEAP32[$0>>2]|0;
        $3860 = HEAP32[$pOp>>2]|0;
        $3861 = ((($3860)) + 4|0);
        $3862 = HEAP32[$3861>>2]|0;
        $3863 = HEAP32[$pOp>>2]|0;
        $3864 = ((($3863)) + 12|0);
        $3865 = HEAP32[$3864>>2]|0;
        $3866 = (_allocateCursor($3859,$3862,$3865,-1,3)|0);
        HEAP32[$pCx28>>2] = $3866;
        $3867 = HEAP32[$pCx28>>2]|0;
        $3868 = ($3867|0)==(0|0);
        if ($3868) {
         label = 860;
         break L1;
        }
        $3869 = HEAP32[$pCx28>>2]|0;
        $3870 = ((($3869)) + 2|0);
        HEAP8[$3870>>0] = 1;
        $3871 = HEAP32[$pOp>>2]|0;
        $3872 = ((($3871)) + 8|0);
        $3873 = HEAP32[$3872>>2]|0;
        $3874 = HEAP32[$pCx28>>2]|0;
        $3875 = ((($3874)) + 16|0);
        HEAP32[$3875>>2] = $3873;
        $3876 = HEAP32[$pCx28>>2]|0;
        $3877 = ((($3876)) + 4|0);
        HEAP8[$3877>>0] = 1;
        break;
       }
       case 61:  {
        $3878 = HEAP32[$0>>2]|0;
        $3879 = HEAP32[$pOp>>2]|0;
        $3880 = ((($3879)) + 4|0);
        $3881 = HEAP32[$3880>>2]|0;
        $3882 = HEAP32[$0>>2]|0;
        $3883 = ((($3882)) + 56|0);
        $3884 = HEAP32[$3883>>2]|0;
        $3885 = (($3884) + ($3881<<2)|0);
        $3886 = HEAP32[$3885>>2]|0;
        _sqlite3VdbeFreeCursor($3878,$3886);
        $3887 = HEAP32[$pOp>>2]|0;
        $3888 = ((($3887)) + 4|0);
        $3889 = HEAP32[$3888>>2]|0;
        $3890 = HEAP32[$0>>2]|0;
        $3891 = ((($3890)) + 56|0);
        $3892 = HEAP32[$3891>>2]|0;
        $3893 = (($3892) + ($3889<<2)|0);
        HEAP32[$3893>>2] = 0;
        break;
       }
       case 66: case 65: case 64: case 63:  {
        $3894 = HEAP32[$pOp>>2]|0;
        $3895 = ((($3894)) + 4|0);
        $3896 = HEAP32[$3895>>2]|0;
        $3897 = HEAP32[$0>>2]|0;
        $3898 = ((($3897)) + 56|0);
        $3899 = HEAP32[$3898>>2]|0;
        $3900 = (($3899) + ($3896<<2)|0);
        $3901 = HEAP32[$3900>>2]|0;
        HEAP32[$pC30>>2] = $3901;
        $3902 = HEAP32[$pOp>>2]|0;
        $3903 = HEAP8[$3902>>0]|0;
        $3904 = $3903&255;
        HEAP32[$oc>>2] = $3904;
        HEAP32[$eqOnly>>2] = 0;
        $3905 = HEAP32[$pC30>>2]|0;
        $3906 = ((($3905)) + 2|0);
        HEAP8[$3906>>0] = 0;
        $3907 = HEAP32[$pC30>>2]|0;
        $3908 = ((($3907)) + 4|0);
        $3909 = HEAP8[$3908>>0]|0;
        $3910 = ($3909<<24>>24)!=(0);
        if ($3910) {
         $3911 = HEAP32[$pOp>>2]|0;
         $3912 = ((($3911)) + 12|0);
         $3913 = HEAP32[$3912>>2]|0;
         $3914 = HEAP32[$aMem>>2]|0;
         $3915 = (($3914) + (($3913*40)|0)|0);
         HEAP32[$pIn3>>2] = $3915;
         $3916 = HEAP32[$pIn3>>2]|0;
         $3917 = ((($3916)) + 8|0);
         $3918 = HEAP16[$3917>>1]|0;
         $3919 = $3918&65535;
         $3920 = $3919 & 14;
         $3921 = ($3920|0)==(2);
         if ($3921) {
          $3922 = HEAP32[$pIn3>>2]|0;
          _applyNumericAffinity($3922,0);
         }
         $3923 = HEAP32[$pIn3>>2]|0;
         $3924 = (_sqlite3VdbeIntValue($3923)|0);
         $3925 = tempRet0;
         $3926 = $iKey;
         $3927 = $3926;
         HEAP32[$3927>>2] = $3924;
         $3928 = (($3926) + 4)|0;
         $3929 = $3928;
         HEAP32[$3929>>2] = $3925;
         $3930 = HEAP32[$pIn3>>2]|0;
         $3931 = ((($3930)) + 8|0);
         $3932 = HEAP16[$3931>>1]|0;
         $3933 = $3932&65535;
         $3934 = $3933 & 4;
         $3935 = ($3934|0)==(0);
         do {
          if ($3935) {
           $3936 = HEAP32[$pIn3>>2]|0;
           $3937 = ((($3936)) + 8|0);
           $3938 = HEAP16[$3937>>1]|0;
           $3939 = $3938&65535;
           $3940 = $3939 & 8;
           $3941 = ($3940|0)==(0);
           if ($3941) {
            label = 15;
            break L11;
           }
           $3942 = HEAP32[$pIn3>>2]|0;
           $3943 = +HEAPF64[$3942>>3];
           $3944 = $iKey;
           $3945 = $3944;
           $3946 = HEAP32[$3945>>2]|0;
           $3947 = (($3944) + 4)|0;
           $3948 = $3947;
           $3949 = HEAP32[$3948>>2]|0;
           $3950 = (+($3946>>>0)) + (4294967296.0*(+($3949|0)));
           $3951 = $3943 < $3950;
           if ($3951) {
            $3952 = HEAP32[$oc>>2]|0;
            $3953 = $3952 & 1;
            $3954 = ($3953|0)==(0);
            if (!($3954)) {
             break;
            }
            $3955 = HEAP32[$oc>>2]|0;
            $3956 = (($3955) + -1)|0;
            HEAP32[$oc>>2] = $3956;
            break;
           }
           $3957 = HEAP32[$pIn3>>2]|0;
           $3958 = +HEAPF64[$3957>>3];
           $3959 = $iKey;
           $3960 = $3959;
           $3961 = HEAP32[$3960>>2]|0;
           $3962 = (($3959) + 4)|0;
           $3963 = $3962;
           $3964 = HEAP32[$3963>>2]|0;
           $3965 = (+($3961>>>0)) + (4294967296.0*(+($3964|0)));
           $3966 = $3958 > $3965;
           if ($3966) {
            $3967 = HEAP32[$oc>>2]|0;
            $3968 = $3967 & 1;
            $3969 = ($3968|0)==(1);
            if ($3969) {
             $3970 = HEAP32[$oc>>2]|0;
             $3971 = (($3970) + 1)|0;
             HEAP32[$oc>>2] = $3971;
            }
           }
          }
         } while(0);
         $3972 = HEAP32[$pC30>>2]|0;
         $3973 = ((($3972)) + 16|0);
         $3974 = HEAP32[$3973>>2]|0;
         $3975 = $iKey;
         $3976 = $3975;
         $3977 = HEAP32[$3976>>2]|0;
         $3978 = (($3975) + 4)|0;
         $3979 = $3978;
         $3980 = HEAP32[$3979>>2]|0;
         $3981 = (_sqlite3BtreeMovetoUnpacked($3974,0,$3977,$3980,0,$res29)|0);
         HEAP32[$rc>>2] = $3981;
         $3982 = $iKey;
         $3983 = $3982;
         $3984 = HEAP32[$3983>>2]|0;
         $3985 = (($3982) + 4)|0;
         $3986 = $3985;
         $3987 = HEAP32[$3986>>2]|0;
         $3988 = HEAP32[$pC30>>2]|0;
         $3989 = ((($3988)) + 40|0);
         $3990 = $3989;
         $3991 = $3990;
         HEAP32[$3991>>2] = $3984;
         $3992 = (($3990) + 4)|0;
         $3993 = $3992;
         HEAP32[$3993>>2] = $3987;
         $3994 = HEAP32[$rc>>2]|0;
         $3995 = ($3994|0)!=(0);
         if ($3995) {
          label = 861;
          break L8;
         } else {
          label = 489;
         }
        } else {
         $3996 = HEAP32[$pC30>>2]|0;
         $3997 = ((($3996)) + 16|0);
         $3998 = HEAP32[$3997>>2]|0;
         $3999 = (_sqlite3BtreeCursorHasHint($3998,2)|0);
         $4000 = ($3999|0)!=(0);
         if ($4000) {
          HEAP32[$eqOnly>>2] = 1;
         }
         $4001 = HEAP32[$pOp>>2]|0;
         $4002 = ((($4001)) + 16|0);
         $4003 = HEAP32[$4002>>2]|0;
         HEAP32[$nField31>>2] = $4003;
         $4004 = HEAP32[$pC30>>2]|0;
         $4005 = ((($4004)) + 24|0);
         $4006 = HEAP32[$4005>>2]|0;
         HEAP32[$r>>2] = $4006;
         $4007 = HEAP32[$nField31>>2]|0;
         $4008 = $4007&65535;
         $4009 = ((($r)) + 8|0);
         HEAP16[$4009>>1] = $4008;
         $4010 = HEAP32[$oc>>2]|0;
         $4011 = (($4010) - 63)|0;
         $4012 = 1 & $4011;
         $4013 = ($4012|0)!=(0);
         $4014 = $4013 ? -1 : 1;
         $4015 = $4014&255;
         $4016 = ((($r)) + 10|0);
         HEAP8[$4016>>0] = $4015;
         $4017 = HEAP32[$pOp>>2]|0;
         $4018 = ((($4017)) + 12|0);
         $4019 = HEAP32[$4018>>2]|0;
         $4020 = HEAP32[$aMem>>2]|0;
         $4021 = (($4020) + (($4019*40)|0)|0);
         $4022 = ((($r)) + 4|0);
         HEAP32[$4022>>2] = $4021;
         $4023 = ((($r)) + 4|0);
         $4024 = HEAP32[$4023>>2]|0;
         $4025 = ((($4024)) + 8|0);
         $4026 = HEAP16[$4025>>1]|0;
         $4027 = $4026&65535;
         $4028 = $4027 & 16384;
         $4029 = ($4028|0)!=(0);
         if ($4029) {
          $4030 = ((($r)) + 4|0);
          $4031 = HEAP32[$4030>>2]|0;
          (_sqlite3VdbeMemExpandBlob($4031)|0);
         }
         $4032 = ((($r)) + 14|0);
         HEAP8[$4032>>0] = 0;
         $4033 = HEAP32[$pC30>>2]|0;
         $4034 = ((($4033)) + 16|0);
         $4035 = HEAP32[$4034>>2]|0;
         $4036 = (_sqlite3BtreeMovetoUnpacked($4035,$r,0,0,0,$res29)|0);
         HEAP32[$rc>>2] = $4036;
         $4037 = HEAP32[$rc>>2]|0;
         $4038 = ($4037|0)!=(0);
         if ($4038) {
          label = 861;
          break L8;
         }
         $4039 = HEAP32[$eqOnly>>2]|0;
         $4040 = ($4039|0)!=(0);
         if ($4040) {
          $4041 = ((($r)) + 14|0);
          $4042 = HEAP8[$4041>>0]|0;
          $4043 = $4042&255;
          $4044 = ($4043|0)==(0);
          if (!($4044)) {
           label = 489;
          }
         } else {
          label = 489;
         }
        }
        do {
         if ((label|0) == 489) {
          label = 0;
          $4045 = HEAP32[$pC30>>2]|0;
          $4046 = ((($4045)) + 3|0);
          HEAP8[$4046>>0] = 0;
          $4047 = HEAP32[$pC30>>2]|0;
          $4048 = ((($4047)) + 56|0);
          HEAP32[$4048>>2] = 0;
          $4049 = HEAP32[$oc>>2]|0;
          $4050 = ($4049|0)>=(65);
          $4051 = HEAP32[$res29>>2]|0;
          if ($4050) {
           $4052 = ($4051|0)<(0);
           if (!($4052)) {
            $4053 = HEAP32[$res29>>2]|0;
            $4054 = ($4053|0)==(0);
            $4055 = HEAP32[$oc>>2]|0;
            $4056 = ($4055|0)==(66);
            $or$cond12 = $4054 & $4056;
            if (!($or$cond12)) {
             HEAP32[$res29>>2] = 0;
             break;
            }
           }
           HEAP32[$res29>>2] = 0;
           $4057 = HEAP32[$pC30>>2]|0;
           $4058 = ((($4057)) + 16|0);
           $4059 = HEAP32[$4058>>2]|0;
           $4060 = (_sqlite3BtreeNext($4059,$res29)|0);
           HEAP32[$rc>>2] = $4060;
           $4061 = HEAP32[$rc>>2]|0;
           $4062 = ($4061|0)!=(0);
           if ($4062) {
            label = 861;
            break L8;
           } else {
            break;
           }
          } else {
           $4063 = ($4051|0)>(0);
           if (!($4063)) {
            $4064 = HEAP32[$res29>>2]|0;
            $4065 = ($4064|0)==(0);
            $4066 = HEAP32[$oc>>2]|0;
            $4067 = ($4066|0)==(63);
            $or$cond14 = $4065 & $4067;
            if (!($or$cond14)) {
             $4074 = HEAP32[$pC30>>2]|0;
             $4075 = ((($4074)) + 16|0);
             $4076 = HEAP32[$4075>>2]|0;
             $4077 = (_sqlite3BtreeEof($4076)|0);
             HEAP32[$res29>>2] = $4077;
             break;
            }
           }
           HEAP32[$res29>>2] = 0;
           $4068 = HEAP32[$pC30>>2]|0;
           $4069 = ((($4068)) + 16|0);
           $4070 = HEAP32[$4069>>2]|0;
           $4071 = (_sqlite3BtreePrevious($4070,$res29)|0);
           HEAP32[$rc>>2] = $4071;
           $4072 = HEAP32[$rc>>2]|0;
           $4073 = ($4072|0)!=(0);
           if ($4073) {
            label = 861;
            break L8;
           } else {
            break;
           }
          }
         }
        } while(0);
        $4078 = HEAP32[$res29>>2]|0;
        $4079 = ($4078|0)!=(0);
        if ($4079) {
         label = 15;
        } else {
         $4080 = HEAP32[$eqOnly>>2]|0;
         $4081 = ($4080|0)!=(0);
         if ($4081) {
          $4082 = HEAP32[$pOp>>2]|0;
          $4083 = ((($4082)) + 20|0);
          HEAP32[$pOp>>2] = $4083;
         }
        }
        break;
       }
       case 69: case 68: case 67:  {
        $4084 = HEAP32[$pOp>>2]|0;
        $4085 = ((($4084)) + 4|0);
        $4086 = HEAP32[$4085>>2]|0;
        $4087 = HEAP32[$0>>2]|0;
        $4088 = ((($4087)) + 56|0);
        $4089 = HEAP32[$4088>>2]|0;
        $4090 = (($4089) + ($4086<<2)|0);
        $4091 = HEAP32[$4090>>2]|0;
        HEAP32[$pC33>>2] = $4091;
        $4092 = HEAP32[$pOp>>2]|0;
        $4093 = ((($4092)) + 12|0);
        $4094 = HEAP32[$4093>>2]|0;
        $4095 = HEAP32[$aMem>>2]|0;
        $4096 = (($4095) + (($4094*40)|0)|0);
        HEAP32[$pIn3>>2] = $4096;
        HEAP32[$pFree>>2] = 0;
        $4097 = HEAP32[$pOp>>2]|0;
        $4098 = ((($4097)) + 16|0);
        $4099 = HEAP32[$4098>>2]|0;
        $4100 = ($4099|0)>(0);
        $4101 = HEAP32[$pC33>>2]|0;
        $4102 = ((($4101)) + 24|0);
        $4103 = HEAP32[$4102>>2]|0;
        if ($4100) {
         HEAP32[$r35>>2] = $4103;
         $4104 = HEAP32[$pOp>>2]|0;
         $4105 = ((($4104)) + 16|0);
         $4106 = HEAP32[$4105>>2]|0;
         $4107 = $4106&65535;
         $4108 = ((($r35)) + 8|0);
         HEAP16[$4108>>1] = $4107;
         $4109 = HEAP32[$pIn3>>2]|0;
         $4110 = ((($r35)) + 4|0);
         HEAP32[$4110>>2] = $4109;
         HEAP32[$ii32>>2] = 0;
         while(1) {
          $4111 = HEAP32[$ii32>>2]|0;
          $4112 = ((($r35)) + 8|0);
          $4113 = HEAP16[$4112>>1]|0;
          $4114 = $4113&65535;
          $4115 = ($4111|0)<($4114|0);
          if (!($4115)) {
           break;
          }
          $4116 = HEAP32[$ii32>>2]|0;
          $4117 = ((($r35)) + 4|0);
          $4118 = HEAP32[$4117>>2]|0;
          $4119 = (($4118) + (($4116*40)|0)|0);
          $4120 = ((($4119)) + 8|0);
          $4121 = HEAP16[$4120>>1]|0;
          $4122 = $4121&65535;
          $4123 = $4122 & 16384;
          $4124 = ($4123|0)!=(0);
          if ($4124) {
           $4125 = HEAP32[$ii32>>2]|0;
           $4126 = ((($r35)) + 4|0);
           $4127 = HEAP32[$4126>>2]|0;
           $4128 = (($4127) + (($4125*40)|0)|0);
           (_sqlite3VdbeMemExpandBlob($4128)|0);
          }
          $4129 = HEAP32[$ii32>>2]|0;
          $4130 = (($4129) + 1)|0;
          HEAP32[$ii32>>2] = $4130;
         }
         HEAP32[$pIdxKey>>2] = $r35;
        } else {
         $4131 = (_sqlite3VdbeAllocUnpackedRecord($4103,$aTempRec,183,$pFree)|0);
         HEAP32[$pIdxKey>>2] = $4131;
         $4132 = HEAP32[$pIdxKey>>2]|0;
         $4133 = ($4132|0)==(0|0);
         if ($4133) {
          label = 860;
          break L1;
         }
         $4134 = HEAP32[$pIn3>>2]|0;
         $4135 = ((($4134)) + 8|0);
         $4136 = HEAP16[$4135>>1]|0;
         $4137 = $4136&65535;
         $4138 = $4137 & 16384;
         $4139 = ($4138|0)!=(0);
         if ($4139) {
          $4140 = HEAP32[$pIn3>>2]|0;
          (_sqlite3VdbeMemExpandBlob($4140)|0);
         }
         $4141 = HEAP32[$pC33>>2]|0;
         $4142 = ((($4141)) + 24|0);
         $4143 = HEAP32[$4142>>2]|0;
         $4144 = HEAP32[$pIn3>>2]|0;
         $4145 = ((($4144)) + 12|0);
         $4146 = HEAP32[$4145>>2]|0;
         $4147 = HEAP32[$pIn3>>2]|0;
         $4148 = ((($4147)) + 16|0);
         $4149 = HEAP32[$4148>>2]|0;
         $4150 = HEAP32[$pIdxKey>>2]|0;
         _sqlite3VdbeRecordUnpack($4143,$4146,$4149,$4150);
        }
        $4151 = HEAP32[$pIdxKey>>2]|0;
        $4152 = ((($4151)) + 10|0);
        HEAP8[$4152>>0] = 0;
        HEAP32[$takeJump>>2] = 0;
        $4153 = HEAP32[$pOp>>2]|0;
        $4154 = HEAP8[$4153>>0]|0;
        $4155 = $4154&255;
        $4156 = ($4155|0)==(67);
        L570: do {
         if ($4156) {
          HEAP32[$ii32>>2] = 0;
          while(1) {
           $4157 = HEAP32[$ii32>>2]|0;
           $4158 = HEAP32[$pIdxKey>>2]|0;
           $4159 = ((($4158)) + 8|0);
           $4160 = HEAP16[$4159>>1]|0;
           $4161 = $4160&65535;
           $4162 = ($4157|0)<($4161|0);
           if (!($4162)) {
            break L570;
           }
           $4163 = HEAP32[$ii32>>2]|0;
           $4164 = HEAP32[$pIdxKey>>2]|0;
           $4165 = ((($4164)) + 4|0);
           $4166 = HEAP32[$4165>>2]|0;
           $4167 = (($4166) + (($4163*40)|0)|0);
           $4168 = ((($4167)) + 8|0);
           $4169 = HEAP16[$4168>>1]|0;
           $4170 = $4169&65535;
           $4171 = $4170 & 1;
           $4172 = ($4171|0)!=(0);
           if ($4172) {
            break;
           }
           $4173 = HEAP32[$ii32>>2]|0;
           $4174 = (($4173) + 1)|0;
           HEAP32[$ii32>>2] = $4174;
          }
          HEAP32[$takeJump>>2] = 1;
         }
        } while(0);
        $4175 = HEAP32[$pC33>>2]|0;
        $4176 = ((($4175)) + 16|0);
        $4177 = HEAP32[$4176>>2]|0;
        $4178 = HEAP32[$pIdxKey>>2]|0;
        $4179 = (_sqlite3BtreeMovetoUnpacked($4177,$4178,0,0,0,$res34)|0);
        HEAP32[$rc>>2] = $4179;
        $4180 = HEAP32[$db>>2]|0;
        $4181 = HEAP32[$pFree>>2]|0;
        _sqlite3DbFree($4180,$4181);
        $4182 = HEAP32[$rc>>2]|0;
        $4183 = ($4182|0)!=(0);
        if (!($4183)) {
         $4184 = HEAP32[$res34>>2]|0;
         $4185 = HEAP32[$pC33>>2]|0;
         $4186 = ((($4185)) + 28|0);
         HEAP32[$4186>>2] = $4184;
         $4187 = HEAP32[$res34>>2]|0;
         $4188 = ($4187|0)==(0);
         $4189 = $4188&1;
         HEAP32[$alreadyExists>>2] = $4189;
         $4190 = HEAP32[$alreadyExists>>2]|0;
         $4191 = (1 - ($4190))|0;
         $4192 = $4191&255;
         $4193 = HEAP32[$pC33>>2]|0;
         $4194 = ((($4193)) + 2|0);
         HEAP8[$4194>>0] = $4192;
         $4195 = HEAP32[$pC33>>2]|0;
         $4196 = ((($4195)) + 3|0);
         HEAP8[$4196>>0] = 0;
         $4197 = HEAP32[$pC33>>2]|0;
         $4198 = ((($4197)) + 56|0);
         HEAP32[$4198>>2] = 0;
         $4199 = HEAP32[$pOp>>2]|0;
         $4200 = HEAP8[$4199>>0]|0;
         $4201 = $4200&255;
         $4202 = ($4201|0)==(69);
         if ($4202) {
          $4203 = HEAP32[$alreadyExists>>2]|0;
          $4204 = ($4203|0)!=(0);
          if ($4204) {
           label = 15;
           break L11;
          } else {
           break L11;
          }
         } else {
          $4205 = HEAP32[$takeJump>>2]|0;
          $4206 = ($4205|0)==(0);
          $4207 = HEAP32[$alreadyExists>>2]|0;
          $4208 = ($4207|0)!=(0);
          $or$cond16 = $4206 & $4208;
          if ($or$cond16) {
           break L11;
          } else {
           label = 15;
           break L11;
          }
         }
        }
        break;
       }
       case 70:  {
        $4209 = HEAP32[$pOp>>2]|0;
        $4210 = ((($4209)) + 12|0);
        $4211 = HEAP32[$4210>>2]|0;
        $4212 = HEAP32[$aMem>>2]|0;
        $4213 = (($4212) + (($4211*40)|0)|0);
        HEAP32[$pIn3>>2] = $4213;
        $4214 = HEAP32[$pOp>>2]|0;
        $4215 = ((($4214)) + 4|0);
        $4216 = HEAP32[$4215>>2]|0;
        $4217 = HEAP32[$0>>2]|0;
        $4218 = ((($4217)) + 56|0);
        $4219 = HEAP32[$4218>>2]|0;
        $4220 = (($4219) + ($4216<<2)|0);
        $4221 = HEAP32[$4220>>2]|0;
        HEAP32[$pC36>>2] = $4221;
        $4222 = HEAP32[$pC36>>2]|0;
        $4223 = ((($4222)) + 16|0);
        $4224 = HEAP32[$4223>>2]|0;
        HEAP32[$pCrsr37>>2] = $4224;
        HEAP32[$res38>>2] = 0;
        $4225 = HEAP32[$pIn3>>2]|0;
        $4226 = $4225;
        $4227 = $4226;
        $4228 = HEAP32[$4227>>2]|0;
        $4229 = (($4226) + 4)|0;
        $4230 = $4229;
        $4231 = HEAP32[$4230>>2]|0;
        $4232 = $iKey39;
        $4233 = $4232;
        HEAP32[$4233>>2] = $4228;
        $4234 = (($4232) + 4)|0;
        $4235 = $4234;
        HEAP32[$4235>>2] = $4231;
        $4236 = HEAP32[$pCrsr37>>2]|0;
        $4237 = $iKey39;
        $4238 = $4237;
        $4239 = HEAP32[$4238>>2]|0;
        $4240 = (($4237) + 4)|0;
        $4241 = $4240;
        $4242 = HEAP32[$4241>>2]|0;
        $4243 = (_sqlite3BtreeMovetoUnpacked($4236,0,$4239,$4242,0,$res38)|0);
        HEAP32[$rc>>2] = $4243;
        $4244 = $iKey39;
        $4245 = $4244;
        $4246 = HEAP32[$4245>>2]|0;
        $4247 = (($4244) + 4)|0;
        $4248 = $4247;
        $4249 = HEAP32[$4248>>2]|0;
        $4250 = HEAP32[$pC36>>2]|0;
        $4251 = ((($4250)) + 40|0);
        $4252 = $4251;
        $4253 = $4252;
        HEAP32[$4253>>2] = $4246;
        $4254 = (($4252) + 4)|0;
        $4255 = $4254;
        HEAP32[$4255>>2] = $4249;
        $4256 = HEAP32[$pC36>>2]|0;
        $4257 = ((($4256)) + 2|0);
        HEAP8[$4257>>0] = 0;
        $4258 = HEAP32[$pC36>>2]|0;
        $4259 = ((($4258)) + 56|0);
        HEAP32[$4259>>2] = 0;
        $4260 = HEAP32[$pC36>>2]|0;
        $4261 = ((($4260)) + 3|0);
        HEAP8[$4261>>0] = 0;
        $4262 = HEAP32[$res38>>2]|0;
        $4263 = HEAP32[$pC36>>2]|0;
        $4264 = ((($4263)) + 28|0);
        HEAP32[$4264>>2] = $4262;
        $4265 = HEAP32[$res38>>2]|0;
        $4266 = ($4265|0)!=(0);
        if ($4266) {
         $4267 = HEAP32[$pOp>>2]|0;
         $4268 = ((($4267)) + 8|0);
         $4269 = HEAP32[$4268>>2]|0;
         $4270 = ($4269|0)==(0);
         if ($4270) {
          $4271 = (_sqlite3CorruptError(78267)|0);
          HEAP32[$rc>>2] = $4271;
         } else {
          label = 15;
         }
        }
        break;
       }
       case 73:  {
        $4272 = HEAP32[$0>>2]|0;
        $4273 = HEAP32[$pOp>>2]|0;
        $4274 = (_out2Prerelease($4272,$4273)|0);
        HEAP32[$pOut>>2] = $4274;
        $4275 = HEAP32[$pOp>>2]|0;
        $4276 = ((($4275)) + 4|0);
        $4277 = HEAP32[$4276>>2]|0;
        $4278 = HEAP32[$0>>2]|0;
        $4279 = ((($4278)) + 56|0);
        $4280 = HEAP32[$4279>>2]|0;
        $4281 = (($4280) + ($4277<<2)|0);
        $4282 = HEAP32[$4281>>2]|0;
        $4283 = ((($4282)) + 32|0);
        $4284 = $4283;
        $4285 = $4284;
        $4286 = HEAP32[$4285>>2]|0;
        $4287 = (($4284) + 4)|0;
        $4288 = $4287;
        $4289 = HEAP32[$4288>>2]|0;
        $4290 = (_i64Add(($4286|0),($4289|0),1,0)|0);
        $4291 = tempRet0;
        $4292 = $4283;
        $4293 = $4292;
        HEAP32[$4293>>2] = $4290;
        $4294 = (($4292) + 4)|0;
        $4295 = $4294;
        HEAP32[$4295>>2] = $4291;
        $4296 = HEAP32[$pOut>>2]|0;
        $4297 = $4296;
        $4298 = $4297;
        HEAP32[$4298>>2] = $4286;
        $4299 = (($4297) + 4)|0;
        $4300 = $4299;
        HEAP32[$4300>>2] = $4289;
        break;
       }
       case 74:  {
        $4301 = $v;
        $4302 = $4301;
        HEAP32[$4302>>2] = 0;
        $4303 = (($4301) + 4)|0;
        $4304 = $4303;
        HEAP32[$4304>>2] = 0;
        HEAP32[$res41>>2] = 0;
        $4305 = HEAP32[$0>>2]|0;
        $4306 = HEAP32[$pOp>>2]|0;
        $4307 = (_out2Prerelease($4305,$4306)|0);
        HEAP32[$pOut>>2] = $4307;
        $4308 = HEAP32[$pOp>>2]|0;
        $4309 = ((($4308)) + 4|0);
        $4310 = HEAP32[$4309>>2]|0;
        $4311 = HEAP32[$0>>2]|0;
        $4312 = ((($4311)) + 56|0);
        $4313 = HEAP32[$4312>>2]|0;
        $4314 = (($4313) + ($4310<<2)|0);
        $4315 = HEAP32[$4314>>2]|0;
        HEAP32[$pC40>>2] = $4315;
        $4316 = HEAP32[$pC40>>2]|0;
        $4317 = ((($4316)) + 5|0);
        $4318 = HEAP8[$4317>>0]|0;
        $4319 = ($4318&255) >>> 1;
        $4320 = $4319 & 1;
        $4321 = $4320&255;
        $4322 = ($4321|0)!=(0);
        do {
         if (!($4322)) {
          $4323 = HEAP32[$pC40>>2]|0;
          $4324 = ((($4323)) + 16|0);
          $4325 = HEAP32[$4324>>2]|0;
          $4326 = (_sqlite3BtreeLast($4325,$res41)|0);
          HEAP32[$rc>>2] = $4326;
          $4327 = HEAP32[$rc>>2]|0;
          $4328 = ($4327|0)!=(0);
          if ($4328) {
           label = 861;
           break L8;
          }
          $4329 = HEAP32[$res41>>2]|0;
          $4330 = ($4329|0)!=(0);
          if ($4330) {
           $4331 = $v;
           $4332 = $4331;
           HEAP32[$4332>>2] = 1;
           $4333 = (($4331) + 4)|0;
           $4334 = $4333;
           HEAP32[$4334>>2] = 0;
           break;
          }
          $4335 = HEAP32[$pC40>>2]|0;
          $4336 = ((($4335)) + 16|0);
          $4337 = HEAP32[$4336>>2]|0;
          $4338 = (_sqlite3BtreeKeySize($4337,$v)|0);
          HEAP32[$rc>>2] = $4338;
          $4339 = $v;
          $4340 = $4339;
          $4341 = HEAP32[$4340>>2]|0;
          $4342 = (($4339) + 4)|0;
          $4343 = $4342;
          $4344 = HEAP32[$4343>>2]|0;
          $4345 = ($4344|0)>(2147483647);
          $4346 = ($4341>>>0)>=(4294967295);
          $4347 = ($4344|0)==(2147483647);
          $4348 = $4347 & $4346;
          $4349 = $4345 | $4348;
          if ($4349) {
           $4350 = HEAP32[$pC40>>2]|0;
           $4351 = ((($4350)) + 5|0);
           $4352 = HEAP8[$4351>>0]|0;
           $4353 = $4352 & -3;
           $4354 = $4353 | 2;
           HEAP8[$4351>>0] = $4354;
           break;
          } else {
           $4355 = $v;
           $4356 = $4355;
           $4357 = HEAP32[$4356>>2]|0;
           $4358 = (($4355) + 4)|0;
           $4359 = $4358;
           $4360 = HEAP32[$4359>>2]|0;
           $4361 = (_i64Add(($4357|0),($4360|0),1,0)|0);
           $4362 = tempRet0;
           $4363 = $v;
           $4364 = $4363;
           HEAP32[$4364>>2] = $4361;
           $4365 = (($4363) + 4)|0;
           $4366 = $4365;
           HEAP32[$4366>>2] = $4362;
           break;
          }
         }
        } while(0);
        $4367 = HEAP32[$pOp>>2]|0;
        $4368 = ((($4367)) + 12|0);
        $4369 = HEAP32[$4368>>2]|0;
        $4370 = ($4369|0)!=(0);
        if ($4370) {
         $4371 = HEAP32[$0>>2]|0;
         $4372 = ((($4371)) + 176|0);
         $4373 = HEAP32[$4372>>2]|0;
         $4374 = ($4373|0)!=(0|0);
         if ($4374) {
          $4375 = HEAP32[$0>>2]|0;
          $4376 = ((($4375)) + 176|0);
          $4377 = HEAP32[$4376>>2]|0;
          HEAP32[$pFrame44>>2] = $4377;
          while(1) {
           $4378 = HEAP32[$pFrame44>>2]|0;
           $4379 = ((($4378)) + 4|0);
           $4380 = HEAP32[$4379>>2]|0;
           $4381 = ($4380|0)!=(0|0);
           if (!($4381)) {
            break;
           }
           $4382 = HEAP32[$pFrame44>>2]|0;
           $4383 = ((($4382)) + 4|0);
           $4384 = HEAP32[$4383>>2]|0;
           HEAP32[$pFrame44>>2] = $4384;
          }
          $4385 = HEAP32[$pOp>>2]|0;
          $4386 = ((($4385)) + 12|0);
          $4387 = HEAP32[$4386>>2]|0;
          $4388 = HEAP32[$pFrame44>>2]|0;
          $4389 = ((($4388)) + 16|0);
          $4390 = HEAP32[$4389>>2]|0;
          $4391 = (($4390) + (($4387*40)|0)|0);
          HEAP32[$pMem43>>2] = $4391;
         } else {
          $4392 = HEAP32[$pOp>>2]|0;
          $4393 = ((($4392)) + 12|0);
          $4394 = HEAP32[$4393>>2]|0;
          $4395 = HEAP32[$aMem>>2]|0;
          $4396 = (($4395) + (($4394*40)|0)|0);
          HEAP32[$pMem43>>2] = $4396;
         }
         $4397 = HEAP32[$pMem43>>2]|0;
         (_sqlite3VdbeMemIntegerify($4397)|0);
         $4398 = HEAP32[$pMem43>>2]|0;
         $4399 = $4398;
         $4400 = $4399;
         $4401 = HEAP32[$4400>>2]|0;
         $4402 = (($4399) + 4)|0;
         $4403 = $4402;
         $4404 = HEAP32[$4403>>2]|0;
         $4405 = ($4401|0)==(-1);
         $4406 = ($4404|0)==(2147483647);
         $4407 = $4405 & $4406;
         if ($4407) {
          label = 542;
          break L8;
         }
         $4408 = HEAP32[$pC40>>2]|0;
         $4409 = ((($4408)) + 5|0);
         $4410 = HEAP8[$4409>>0]|0;
         $4411 = ($4410&255) >>> 1;
         $4412 = $4411 & 1;
         $4413 = $4412&255;
         $4414 = ($4413|0)!=(0);
         if ($4414) {
          label = 542;
          break L8;
         }
         $4415 = $v;
         $4416 = $4415;
         $4417 = HEAP32[$4416>>2]|0;
         $4418 = (($4415) + 4)|0;
         $4419 = $4418;
         $4420 = HEAP32[$4419>>2]|0;
         $4421 = HEAP32[$pMem43>>2]|0;
         $4422 = $4421;
         $4423 = $4422;
         $4424 = HEAP32[$4423>>2]|0;
         $4425 = (($4422) + 4)|0;
         $4426 = $4425;
         $4427 = HEAP32[$4426>>2]|0;
         $4428 = (_i64Add(($4424|0),($4427|0),1,0)|0);
         $4429 = tempRet0;
         $4430 = ($4420|0)<($4429|0);
         $4431 = ($4417>>>0)<($4428>>>0);
         $4432 = ($4420|0)==($4429|0);
         $4433 = $4432 & $4431;
         $4434 = $4430 | $4433;
         if ($4434) {
          $4435 = HEAP32[$pMem43>>2]|0;
          $4436 = $4435;
          $4437 = $4436;
          $4438 = HEAP32[$4437>>2]|0;
          $4439 = (($4436) + 4)|0;
          $4440 = $4439;
          $4441 = HEAP32[$4440>>2]|0;
          $4442 = (_i64Add(($4438|0),($4441|0),1,0)|0);
          $4443 = tempRet0;
          $4444 = $v;
          $4445 = $4444;
          HEAP32[$4445>>2] = $4442;
          $4446 = (($4444) + 4)|0;
          $4447 = $4446;
          HEAP32[$4447>>2] = $4443;
         }
         $4448 = $v;
         $4449 = $4448;
         $4450 = HEAP32[$4449>>2]|0;
         $4451 = (($4448) + 4)|0;
         $4452 = $4451;
         $4453 = HEAP32[$4452>>2]|0;
         $4454 = HEAP32[$pMem43>>2]|0;
         $4455 = $4454;
         $4456 = $4455;
         HEAP32[$4456>>2] = $4450;
         $4457 = (($4455) + 4)|0;
         $4458 = $4457;
         HEAP32[$4458>>2] = $4453;
        }
        $4459 = HEAP32[$pC40>>2]|0;
        $4460 = ((($4459)) + 5|0);
        $4461 = HEAP8[$4460>>0]|0;
        $4462 = ($4461&255) >>> 1;
        $4463 = $4462 & 1;
        $4464 = $4463&255;
        $4465 = ($4464|0)!=(0);
        if ($4465) {
         HEAP32[$cnt42>>2] = 0;
         while(1) {
          _sqlite3_randomness(8,$v);
          $4466 = $v;
          $4467 = $4466;
          $4468 = HEAP32[$4467>>2]|0;
          $4469 = (($4466) + 4)|0;
          $4470 = $4469;
          $4471 = HEAP32[$4470>>2]|0;
          $4472 = $4471 & 1073741823;
          $4473 = $v;
          $4474 = $4473;
          HEAP32[$4474>>2] = $4468;
          $4475 = (($4473) + 4)|0;
          $4476 = $4475;
          HEAP32[$4476>>2] = $4472;
          $4477 = $v;
          $4478 = $4477;
          $4479 = HEAP32[$4478>>2]|0;
          $4480 = (($4477) + 4)|0;
          $4481 = $4480;
          $4482 = HEAP32[$4481>>2]|0;
          $4483 = (_i64Add(($4479|0),($4482|0),1,0)|0);
          $4484 = tempRet0;
          $4485 = $v;
          $4486 = $4485;
          HEAP32[$4486>>2] = $4483;
          $4487 = (($4485) + 4)|0;
          $4488 = $4487;
          HEAP32[$4488>>2] = $4484;
          $4489 = HEAP32[$pC40>>2]|0;
          $4490 = ((($4489)) + 16|0);
          $4491 = HEAP32[$4490>>2]|0;
          $4492 = $v;
          $4493 = $4492;
          $4494 = HEAP32[$4493>>2]|0;
          $4495 = (($4492) + 4)|0;
          $4496 = $4495;
          $4497 = HEAP32[$4496>>2]|0;
          $4498 = (_sqlite3BtreeMovetoUnpacked($4491,0,$4494,$4497,0,$res41)|0);
          HEAP32[$rc>>2] = $4498;
          $4499 = ($4498|0)==(0);
          $4500 = HEAP32[$res41>>2]|0;
          $4501 = ($4500|0)==(0);
          $or$cond18 = $4499 & $4501;
          if (!($or$cond18)) {
           break;
          }
          $4502 = HEAP32[$cnt42>>2]|0;
          $4503 = (($4502) + 1)|0;
          HEAP32[$cnt42>>2] = $4503;
          $4504 = ($4503|0)<(100);
          if (!($4504)) {
           break;
          }
         }
         $4505 = HEAP32[$rc>>2]|0;
         $4506 = ($4505|0)==(0);
         $4507 = HEAP32[$res41>>2]|0;
         $4508 = ($4507|0)==(0);
         $or$cond21 = $4506 & $4508;
         if ($or$cond21) {
          label = 551;
          break L8;
         }
        }
        $4509 = HEAP32[$pC40>>2]|0;
        $4510 = ((($4509)) + 3|0);
        HEAP8[$4510>>0] = 0;
        $4511 = HEAP32[$pC40>>2]|0;
        $4512 = ((($4511)) + 56|0);
        HEAP32[$4512>>2] = 0;
        $4513 = $v;
        $4514 = $4513;
        $4515 = HEAP32[$4514>>2]|0;
        $4516 = (($4513) + 4)|0;
        $4517 = $4516;
        $4518 = HEAP32[$4517>>2]|0;
        $4519 = HEAP32[$pOut>>2]|0;
        $4520 = $4519;
        $4521 = $4520;
        HEAP32[$4521>>2] = $4515;
        $4522 = (($4520) + 4)|0;
        $4523 = $4522;
        HEAP32[$4523>>2] = $4518;
        break;
       }
       case 84: case 75:  {
        $4524 = HEAP32[$pOp>>2]|0;
        $4525 = ((($4524)) + 8|0);
        $4526 = HEAP32[$4525>>2]|0;
        $4527 = HEAP32[$aMem>>2]|0;
        $4528 = (($4527) + (($4526*40)|0)|0);
        HEAP32[$pData>>2] = $4528;
        $4529 = HEAP32[$pOp>>2]|0;
        $4530 = ((($4529)) + 4|0);
        $4531 = HEAP32[$4530>>2]|0;
        $4532 = HEAP32[$0>>2]|0;
        $4533 = ((($4532)) + 56|0);
        $4534 = HEAP32[$4533>>2]|0;
        $4535 = (($4534) + ($4531<<2)|0);
        $4536 = HEAP32[$4535>>2]|0;
        HEAP32[$pC46>>2] = $4536;
        $4537 = HEAP32[$pOp>>2]|0;
        $4538 = HEAP8[$4537>>0]|0;
        $4539 = $4538&255;
        $4540 = ($4539|0)==(75);
        $4541 = HEAP32[$pOp>>2]|0;
        $4542 = ((($4541)) + 12|0);
        $4543 = HEAP32[$4542>>2]|0;
        if ($4540) {
         $4544 = HEAP32[$aMem>>2]|0;
         $4545 = (($4544) + (($4543*40)|0)|0);
         HEAP32[$pKey>>2] = $4545;
         $4546 = HEAP32[$pKey>>2]|0;
         $4547 = $4546;
         $4548 = $4547;
         $4549 = HEAP32[$4548>>2]|0;
         $4550 = (($4547) + 4)|0;
         $4551 = $4550;
         $4552 = HEAP32[$4551>>2]|0;
         $4553 = $iKey45;
         $4554 = $4553;
         HEAP32[$4554>>2] = $4549;
         $4555 = (($4553) + 4)|0;
         $4556 = $4555;
         HEAP32[$4556>>2] = $4552;
        } else {
         $4557 = ($4543|0)<(0);
         $4558 = $4557 << 31 >> 31;
         $4559 = $iKey45;
         $4560 = $4559;
         HEAP32[$4560>>2] = $4543;
         $4561 = (($4559) + 4)|0;
         $4562 = $4561;
         HEAP32[$4562>>2] = $4558;
        }
        $4563 = HEAP32[$pOp>>2]|0;
        $4564 = ((($4563)) + 3|0);
        $4565 = HEAP8[$4564>>0]|0;
        $4566 = $4565&255;
        $4567 = $4566 & 1;
        $4568 = ($4567|0)!=(0);
        if ($4568) {
         $4569 = HEAP32[$0>>2]|0;
         $4570 = ((($4569)) + 92|0);
         $4571 = HEAP32[$4570>>2]|0;
         $4572 = (($4571) + 1)|0;
         HEAP32[$4570>>2] = $4572;
        }
        $4573 = HEAP32[$pOp>>2]|0;
        $4574 = ((($4573)) + 3|0);
        $4575 = HEAP8[$4574>>0]|0;
        $4576 = $4575&255;
        $4577 = $4576 & 2;
        $4578 = ($4577|0)!=(0);
        if ($4578) {
         $4579 = $iKey45;
         $4580 = $4579;
         $4581 = HEAP32[$4580>>2]|0;
         $4582 = (($4579) + 4)|0;
         $4583 = $4582;
         $4584 = HEAP32[$4583>>2]|0;
         $4585 = $lastRowid;
         $4586 = $4585;
         HEAP32[$4586>>2] = $4581;
         $4587 = (($4585) + 4)|0;
         $4588 = $4587;
         HEAP32[$4588>>2] = $4584;
         $4589 = HEAP32[$db>>2]|0;
         $4590 = ((($4589)) + 32|0);
         $4591 = $4590;
         $4592 = $4591;
         HEAP32[$4592>>2] = $4581;
         $4593 = (($4591) + 4)|0;
         $4594 = $4593;
         HEAP32[$4594>>2] = $4584;
        }
        $4595 = HEAP32[$pData>>2]|0;
        $4596 = ((($4595)) + 8|0);
        $4597 = HEAP16[$4596>>1]|0;
        $4598 = $4597&65535;
        $4599 = $4598 & 1;
        $4600 = ($4599|0)!=(0);
        if ($4600) {
         $4601 = HEAP32[$pData>>2]|0;
         $4602 = ((($4601)) + 16|0);
         HEAP32[$4602>>2] = 0;
         $4603 = HEAP32[$pData>>2]|0;
         $4604 = ((($4603)) + 12|0);
         HEAP32[$4604>>2] = 0;
        }
        $4605 = HEAP32[$pOp>>2]|0;
        $4606 = ((($4605)) + 3|0);
        $4607 = HEAP8[$4606>>0]|0;
        $4608 = $4607&255;
        $4609 = $4608 & 16;
        $4610 = ($4609|0)!=(0);
        if ($4610) {
         $4611 = HEAP32[$pC46>>2]|0;
         $4612 = ((($4611)) + 28|0);
         $4613 = HEAP32[$4612>>2]|0;
         $4614 = $4613;
        } else {
         $4614 = 0;
        }
        HEAP32[$seekResult>>2] = $4614;
        $4615 = HEAP32[$pData>>2]|0;
        $4616 = ((($4615)) + 8|0);
        $4617 = HEAP16[$4616>>1]|0;
        $4618 = $4617&65535;
        $4619 = $4618 & 16384;
        $4620 = ($4619|0)!=(0);
        if ($4620) {
         $4621 = HEAP32[$pData>>2]|0;
         $4622 = HEAP32[$4621>>2]|0;
         HEAP32[$nZero47>>2] = $4622;
        } else {
         HEAP32[$nZero47>>2] = 0;
        }
        $4623 = HEAP32[$pC46>>2]|0;
        $4624 = ((($4623)) + 16|0);
        $4625 = HEAP32[$4624>>2]|0;
        $4626 = $iKey45;
        $4627 = $4626;
        $4628 = HEAP32[$4627>>2]|0;
        $4629 = (($4626) + 4)|0;
        $4630 = $4629;
        $4631 = HEAP32[$4630>>2]|0;
        $4632 = HEAP32[$pData>>2]|0;
        $4633 = ((($4632)) + 16|0);
        $4634 = HEAP32[$4633>>2]|0;
        $4635 = HEAP32[$pData>>2]|0;
        $4636 = ((($4635)) + 12|0);
        $4637 = HEAP32[$4636>>2]|0;
        $4638 = HEAP32[$nZero47>>2]|0;
        $4639 = HEAP32[$pOp>>2]|0;
        $4640 = ((($4639)) + 3|0);
        $4641 = HEAP8[$4640>>0]|0;
        $4642 = $4641&255;
        $4643 = $4642 & 8;
        $4644 = ($4643|0)!=(0);
        $4645 = $4644&1;
        $4646 = HEAP32[$seekResult>>2]|0;
        $4647 = (_sqlite3BtreeInsert($4625,0,$4628,$4631,$4634,$4637,$4638,$4645,$4646)|0);
        HEAP32[$rc>>2] = $4647;
        $4648 = HEAP32[$pC46>>2]|0;
        $4649 = ((($4648)) + 3|0);
        HEAP8[$4649>>0] = 0;
        $4650 = HEAP32[$pC46>>2]|0;
        $4651 = ((($4650)) + 56|0);
        HEAP32[$4651>>2] = 0;
        $4652 = HEAP32[$rc>>2]|0;
        $4653 = ($4652|0)==(0);
        if ($4653) {
         $4654 = HEAP32[$db>>2]|0;
         $4655 = ((($4654)) + 212|0);
         $4656 = HEAP32[$4655>>2]|0;
         $4657 = ($4656|0)!=(0|0);
         if ($4657) {
          $4658 = HEAP32[$pOp>>2]|0;
          $4659 = ((($4658)) + 16|0);
          $4660 = HEAP32[$4659>>2]|0;
          $4661 = ($4660|0)!=(0|0);
          if ($4661) {
           $4662 = HEAP32[$pC46>>2]|0;
           $4663 = ((($4662)) + 1|0);
           $4664 = HEAP8[$4663>>0]|0;
           $4665 = $4664 << 24 >> 24;
           $4666 = HEAP32[$db>>2]|0;
           $4667 = ((($4666)) + 16|0);
           $4668 = HEAP32[$4667>>2]|0;
           $4669 = (($4668) + ($4665<<4)|0);
           $4670 = HEAP32[$4669>>2]|0;
           HEAP32[$zDb>>2] = $4670;
           $4671 = HEAP32[$pOp>>2]|0;
           $4672 = ((($4671)) + 16|0);
           $4673 = HEAP32[$4672>>2]|0;
           HEAP32[$zTbl>>2] = $4673;
           $4674 = HEAP32[$pOp>>2]|0;
           $4675 = ((($4674)) + 3|0);
           $4676 = HEAP8[$4675>>0]|0;
           $4677 = $4676&255;
           $4678 = $4677 & 4;
           $4679 = ($4678|0)!=(0);
           $4680 = $4679 ? 23 : 18;
           HEAP32[$op48>>2] = $4680;
           $4681 = HEAP32[$db>>2]|0;
           $4682 = ((($4681)) + 212|0);
           $4683 = HEAP32[$4682>>2]|0;
           $4684 = HEAP32[$db>>2]|0;
           $4685 = ((($4684)) + 208|0);
           $4686 = HEAP32[$4685>>2]|0;
           $4687 = HEAP32[$op48>>2]|0;
           $4688 = HEAP32[$zDb>>2]|0;
           $4689 = HEAP32[$zTbl>>2]|0;
           $4690 = $iKey45;
           $4691 = $4690;
           $4692 = HEAP32[$4691>>2]|0;
           $4693 = (($4690) + 4)|0;
           $4694 = $4693;
           $4695 = HEAP32[$4694>>2]|0;
           FUNCTION_TABLE_viiiiii[$4683 & 255]($4686,$4687,$4688,$4689,$4692,$4695);
          }
         }
        }
        break;
       }
       case 95:  {
        $4696 = HEAP32[$pOp>>2]|0;
        $4697 = ((($4696)) + 4|0);
        $4698 = HEAP32[$4697>>2]|0;
        $4699 = HEAP32[$0>>2]|0;
        $4700 = ((($4699)) + 56|0);
        $4701 = HEAP32[$4700>>2]|0;
        $4702 = (($4701) + ($4698<<2)|0);
        $4703 = HEAP32[$4702>>2]|0;
        HEAP32[$pC49>>2] = $4703;
        $4704 = HEAP32[$db>>2]|0;
        $4705 = ((($4704)) + 212|0);
        $4706 = HEAP32[$4705>>2]|0;
        $4707 = ($4706|0)!=(0|0);
        if ($4707) {
         $4708 = HEAP32[$pOp>>2]|0;
         $4709 = ((($4708)) + 16|0);
         $4710 = HEAP32[$4709>>2]|0;
         $4711 = ($4710|0)!=(0|0);
         if ($4711) {
          $4712 = HEAP32[$pC49>>2]|0;
          $4713 = ((($4712)) + 4|0);
          $4714 = HEAP8[$4713>>0]|0;
          $4715 = $4714&255;
          $4716 = ($4715|0)!=(0);
          $4718 = $4716;
         } else {
          $4718 = 0;
         }
        } else {
         $4718 = 0;
        }
        $4717 = $4718&1;
        $4719 = $4717&255;
        HEAP8[$hasUpdateCallback>>0] = $4719;
        $4720 = HEAP32[$pOp>>2]|0;
        $4721 = ((($4720)) + 3|0);
        $4722 = HEAP8[$4721>>0]|0;
        $4723 = $4722&255;
        $4724 = ($4723|0)!=(0);
        if ($4724) {
         $4725 = HEAP8[$hasUpdateCallback>>0]|0;
         $4726 = $4725&255;
         $4727 = ($4726|0)!=(0);
         if ($4727) {
          $4728 = HEAP32[$pC49>>2]|0;
          $4729 = ((($4728)) + 16|0);
          $4730 = HEAP32[$4729>>2]|0;
          $4731 = HEAP32[$pC49>>2]|0;
          $4732 = ((($4731)) + 40|0);
          (_sqlite3BtreeKeySize($4730,$4732)|0);
         }
        }
        $4733 = HEAP32[$pC49>>2]|0;
        $4734 = ((($4733)) + 16|0);
        $4735 = HEAP32[$4734>>2]|0;
        $4736 = HEAP32[$pOp>>2]|0;
        $4737 = ((($4736)) + 3|0);
        $4738 = HEAP8[$4737>>0]|0;
        $4739 = (_sqlite3BtreeDelete($4735,$4738)|0);
        HEAP32[$rc>>2] = $4739;
        $4740 = HEAP32[$pC49>>2]|0;
        $4741 = ((($4740)) + 56|0);
        HEAP32[$4741>>2] = 0;
        $4742 = HEAP32[$rc>>2]|0;
        $4743 = ($4742|0)==(0);
        if ($4743) {
         $4744 = HEAP8[$hasUpdateCallback>>0]|0;
         $4745 = $4744&255;
         $4746 = ($4745|0)!=(0);
         if ($4746) {
          $4747 = HEAP32[$db>>2]|0;
          $4748 = ((($4747)) + 212|0);
          $4749 = HEAP32[$4748>>2]|0;
          $4750 = HEAP32[$db>>2]|0;
          $4751 = ((($4750)) + 208|0);
          $4752 = HEAP32[$4751>>2]|0;
          $4753 = HEAP32[$pC49>>2]|0;
          $4754 = ((($4753)) + 1|0);
          $4755 = HEAP8[$4754>>0]|0;
          $4756 = $4755 << 24 >> 24;
          $4757 = HEAP32[$db>>2]|0;
          $4758 = ((($4757)) + 16|0);
          $4759 = HEAP32[$4758>>2]|0;
          $4760 = (($4759) + ($4756<<4)|0);
          $4761 = HEAP32[$4760>>2]|0;
          $4762 = HEAP32[$pOp>>2]|0;
          $4763 = ((($4762)) + 16|0);
          $4764 = HEAP32[$4763>>2]|0;
          $4765 = HEAP32[$pC49>>2]|0;
          $4766 = ((($4765)) + 40|0);
          $4767 = $4766;
          $4768 = $4767;
          $4769 = HEAP32[$4768>>2]|0;
          $4770 = (($4767) + 4)|0;
          $4771 = $4770;
          $4772 = HEAP32[$4771>>2]|0;
          FUNCTION_TABLE_viiiiii[$4749 & 255]($4752,9,$4761,$4764,$4769,$4772);
         }
        }
        $4773 = HEAP32[$pOp>>2]|0;
        $4774 = ((($4773)) + 8|0);
        $4775 = HEAP32[$4774>>2]|0;
        $4776 = $4775 & 1;
        $4777 = ($4776|0)!=(0);
        if ($4777) {
         $4778 = HEAP32[$0>>2]|0;
         $4779 = ((($4778)) + 92|0);
         $4780 = HEAP32[$4779>>2]|0;
         $4781 = (($4780) + 1)|0;
         HEAP32[$4779>>2] = $4781;
        }
        break;
       }
       case 98:  {
        $4782 = HEAP32[$db>>2]|0;
        $4783 = HEAP32[$0>>2]|0;
        $4784 = ((($4783)) + 92|0);
        $4785 = HEAP32[$4784>>2]|0;
        _sqlite3VdbeSetChanges($4782,$4785);
        $4786 = HEAP32[$0>>2]|0;
        $4787 = ((($4786)) + 92|0);
        HEAP32[$4787>>2] = 0;
        break;
       }
       case 99:  {
        $4788 = HEAP32[$pOp>>2]|0;
        $4789 = ((($4788)) + 4|0);
        $4790 = HEAP32[$4789>>2]|0;
        $4791 = HEAP32[$0>>2]|0;
        $4792 = ((($4791)) + 56|0);
        $4793 = HEAP32[$4792>>2]|0;
        $4794 = (($4793) + ($4790<<2)|0);
        $4795 = HEAP32[$4794>>2]|0;
        HEAP32[$pC50>>2] = $4795;
        $4796 = HEAP32[$pOp>>2]|0;
        $4797 = ((($4796)) + 12|0);
        $4798 = HEAP32[$4797>>2]|0;
        $4799 = HEAP32[$aMem>>2]|0;
        $4800 = (($4799) + (($4798*40)|0)|0);
        HEAP32[$pIn3>>2] = $4800;
        $4801 = HEAP32[$pOp>>2]|0;
        $4802 = ((($4801)) + 16|0);
        $4803 = HEAP32[$4802>>2]|0;
        HEAP32[$nKeyCol>>2] = $4803;
        HEAP32[$res51>>2] = 0;
        $4804 = HEAP32[$pC50>>2]|0;
        $4805 = HEAP32[$pIn3>>2]|0;
        $4806 = HEAP32[$nKeyCol>>2]|0;
        $4807 = (_sqlite3VdbeSorterCompare($4804,$4805,$4806,$res51)|0);
        HEAP32[$rc>>2] = $4807;
        $4808 = HEAP32[$res51>>2]|0;
        $4809 = ($4808|0)!=(0);
        if ($4809) {
         label = 15;
        }
        break;
       }
       case 100:  {
        $4810 = HEAP32[$pOp>>2]|0;
        $4811 = ((($4810)) + 8|0);
        $4812 = HEAP32[$4811>>2]|0;
        $4813 = HEAP32[$aMem>>2]|0;
        $4814 = (($4813) + (($4812*40)|0)|0);
        HEAP32[$pOut>>2] = $4814;
        $4815 = HEAP32[$pOp>>2]|0;
        $4816 = ((($4815)) + 4|0);
        $4817 = HEAP32[$4816>>2]|0;
        $4818 = HEAP32[$0>>2]|0;
        $4819 = ((($4818)) + 56|0);
        $4820 = HEAP32[$4819>>2]|0;
        $4821 = (($4820) + ($4817<<2)|0);
        $4822 = HEAP32[$4821>>2]|0;
        HEAP32[$pC52>>2] = $4822;
        $4823 = HEAP32[$pC52>>2]|0;
        $4824 = HEAP32[$pOut>>2]|0;
        $4825 = (_sqlite3VdbeSorterRowkey($4823,$4824)|0);
        HEAP32[$rc>>2] = $4825;
        $4826 = HEAP32[$pOp>>2]|0;
        $4827 = ((($4826)) + 12|0);
        $4828 = HEAP32[$4827>>2]|0;
        $4829 = HEAP32[$0>>2]|0;
        $4830 = ((($4829)) + 56|0);
        $4831 = HEAP32[$4830>>2]|0;
        $4832 = (($4831) + ($4828<<2)|0);
        $4833 = HEAP32[$4832>>2]|0;
        $4834 = ((($4833)) + 56|0);
        HEAP32[$4834>>2] = 0;
        break;
       }
       case 102: case 101:  {
        $4835 = HEAP32[$pOp>>2]|0;
        $4836 = ((($4835)) + 8|0);
        $4837 = HEAP32[$4836>>2]|0;
        $4838 = HEAP32[$aMem>>2]|0;
        $4839 = (($4838) + (($4837*40)|0)|0);
        HEAP32[$pOut>>2] = $4839;
        $4840 = HEAP32[$pOp>>2]|0;
        $4841 = ((($4840)) + 4|0);
        $4842 = HEAP32[$4841>>2]|0;
        $4843 = HEAP32[$0>>2]|0;
        $4844 = ((($4843)) + 56|0);
        $4845 = HEAP32[$4844>>2]|0;
        $4846 = (($4845) + ($4842<<2)|0);
        $4847 = HEAP32[$4846>>2]|0;
        HEAP32[$pC53>>2] = $4847;
        $4848 = HEAP32[$pC53>>2]|0;
        $4849 = ((($4848)) + 16|0);
        $4850 = HEAP32[$4849>>2]|0;
        HEAP32[$pCrsr54>>2] = $4850;
        $4851 = HEAP32[$pC53>>2]|0;
        $4852 = ((($4851)) + 4|0);
        $4853 = HEAP8[$4852>>0]|0;
        $4854 = $4853&255;
        $4855 = ($4854|0)==(0);
        $4856 = HEAP32[$pCrsr54>>2]|0;
        if ($4855) {
         (_sqlite3BtreeKeySize($4856,$n64)|0);
         $4857 = $n64;
         $4858 = $4857;
         $4859 = HEAP32[$4858>>2]|0;
         $4860 = (($4857) + 4)|0;
         $4861 = $4860;
         $4862 = HEAP32[$4861>>2]|0;
         $4863 = HEAP32[$db>>2]|0;
         $4864 = ((($4863)) + 88|0);
         $4865 = HEAP32[$4864>>2]|0;
         $4866 = ($4865|0)<(0);
         $4867 = $4866 << 31 >> 31;
         $4868 = ($4862|0)>($4867|0);
         $4869 = ($4859>>>0)>($4865>>>0);
         $4870 = ($4862|0)==($4867|0);
         $4871 = $4870 & $4869;
         $4872 = $4868 | $4871;
         if ($4872) {
          label = 859;
          break L8;
         }
         $4873 = $n64;
         $4874 = $4873;
         $4875 = HEAP32[$4874>>2]|0;
         $4876 = (($4873) + 4)|0;
         $4877 = $4876;
         $4878 = HEAP32[$4877>>2]|0;
         HEAP32[$n55>>2] = $4875;
        } else {
         (_sqlite3BtreeDataSize($4856,$n55)|0);
         $4879 = HEAP32[$n55>>2]|0;
         $4880 = HEAP32[$db>>2]|0;
         $4881 = ((($4880)) + 88|0);
         $4882 = HEAP32[$4881>>2]|0;
         $4883 = ($4879>>>0)>($4882>>>0);
         if ($4883) {
          label = 859;
          break L8;
         }
        }
        $4884 = HEAP32[$pOut>>2]|0;
        $4885 = HEAP32[$n55>>2]|0;
        $4886 = ($4885>>>0)>(32);
        $4887 = HEAP32[$n55>>2]|0;
        $4888 = $4886 ? $4887 : 32;
        $4889 = (_sqlite3VdbeMemClearAndResize($4884,$4888)|0);
        $4890 = ($4889|0)!=(0);
        if ($4890) {
         label = 860;
         break L1;
        }
        $4891 = HEAP32[$n55>>2]|0;
        $4892 = HEAP32[$pOut>>2]|0;
        $4893 = ((($4892)) + 12|0);
        HEAP32[$4893>>2] = $4891;
        $4894 = HEAP32[$pOut>>2]|0;
        $4895 = ((($4894)) + 8|0);
        $4896 = HEAP16[$4895>>1]|0;
        $4897 = $4896&65535;
        $4898 = $4897 & -49664;
        $4899 = $4898 | 16;
        $4900 = $4899&65535;
        $4901 = HEAP32[$pOut>>2]|0;
        $4902 = ((($4901)) + 8|0);
        HEAP16[$4902>>1] = $4900;
        $4903 = HEAP32[$pC53>>2]|0;
        $4904 = ((($4903)) + 4|0);
        $4905 = HEAP8[$4904>>0]|0;
        $4906 = $4905&255;
        $4907 = ($4906|0)==(0);
        $4908 = HEAP32[$pCrsr54>>2]|0;
        $4909 = HEAP32[$n55>>2]|0;
        $4910 = HEAP32[$pOut>>2]|0;
        $4911 = ((($4910)) + 16|0);
        $4912 = HEAP32[$4911>>2]|0;
        if ($4907) {
         $4913 = (_sqlite3BtreeKey($4908,0,$4909,$4912)|0);
         HEAP32[$rc>>2] = $4913;
        } else {
         $4914 = (_sqlite3BtreeData($4908,0,$4909,$4912)|0);
         HEAP32[$rc>>2] = $4914;
        }
        $4915 = HEAP32[$pOut>>2]|0;
        $4916 = ((($4915)) + 10|0);
        HEAP8[$4916>>0] = 1;
        break;
       }
       case 103:  {
        $4917 = HEAP32[$0>>2]|0;
        $4918 = HEAP32[$pOp>>2]|0;
        $4919 = (_out2Prerelease($4917,$4918)|0);
        HEAP32[$pOut>>2] = $4919;
        $4920 = HEAP32[$pOp>>2]|0;
        $4921 = ((($4920)) + 4|0);
        $4922 = HEAP32[$4921>>2]|0;
        $4923 = HEAP32[$0>>2]|0;
        $4924 = ((($4923)) + 56|0);
        $4925 = HEAP32[$4924>>2]|0;
        $4926 = (($4925) + ($4922<<2)|0);
        $4927 = HEAP32[$4926>>2]|0;
        HEAP32[$pC56>>2] = $4927;
        $4928 = HEAP32[$pC56>>2]|0;
        $4929 = ((($4928)) + 2|0);
        $4930 = HEAP8[$4929>>0]|0;
        $4931 = ($4930<<24>>24)!=(0);
        if ($4931) {
         $4932 = HEAP32[$pOut>>2]|0;
         $4933 = ((($4932)) + 8|0);
         HEAP16[$4933>>1] = 1;
         break L11;
        }
        $4934 = HEAP32[$pC56>>2]|0;
        $4935 = ((($4934)) + 3|0);
        $4936 = HEAP8[$4935>>0]|0;
        $4937 = ($4936<<24>>24)!=(0);
        $4938 = HEAP32[$pC56>>2]|0;
        do {
         if ($4937) {
          $4939 = ((($4938)) + 40|0);
          $4940 = $4939;
          $4941 = $4940;
          $4942 = HEAP32[$4941>>2]|0;
          $4943 = (($4940) + 4)|0;
          $4944 = $4943;
          $4945 = HEAP32[$4944>>2]|0;
          $4946 = $v57;
          $4947 = $4946;
          HEAP32[$4947>>2] = $4942;
          $4948 = (($4946) + 4)|0;
          $4949 = $4948;
          HEAP32[$4949>>2] = $4945;
         } else {
          $4950 = HEAP8[$4938>>0]|0;
          $4951 = $4950&255;
          $4952 = ($4951|0)==(2);
          $4953 = HEAP32[$pC56>>2]|0;
          if ($4952) {
           $4954 = ((($4953)) + 16|0);
           $4955 = HEAP32[$4954>>2]|0;
           $4956 = HEAP32[$4955>>2]|0;
           HEAP32[$pVtab>>2] = $4956;
           $4957 = HEAP32[$pVtab>>2]|0;
           $4958 = HEAP32[$4957>>2]|0;
           HEAP32[$pModule>>2] = $4958;
           $4959 = HEAP32[$pModule>>2]|0;
           $4960 = ((($4959)) + 48|0);
           $4961 = HEAP32[$4960>>2]|0;
           $4962 = HEAP32[$pC56>>2]|0;
           $4963 = ((($4962)) + 16|0);
           $4964 = HEAP32[$4963>>2]|0;
           $4965 = (FUNCTION_TABLE_iii[$4961 & 255]($4964,$v57)|0);
           HEAP32[$rc>>2] = $4965;
           $4966 = HEAP32[$0>>2]|0;
           $4967 = HEAP32[$pVtab>>2]|0;
           _sqlite3VtabImportErrmsg($4966,$4967);
           break;
          }
          $4968 = (_sqlite3VdbeCursorRestore($4953)|0);
          HEAP32[$rc>>2] = $4968;
          $4969 = HEAP32[$rc>>2]|0;
          $4970 = ($4969|0)!=(0);
          if ($4970) {
           label = 861;
           break L8;
          }
          $4971 = HEAP32[$pC56>>2]|0;
          $4972 = ((($4971)) + 2|0);
          $4973 = HEAP8[$4972>>0]|0;
          $4974 = ($4973<<24>>24)!=(0);
          if ($4974) {
           $4975 = HEAP32[$pOut>>2]|0;
           $4976 = ((($4975)) + 8|0);
           HEAP16[$4976>>1] = 1;
           break L11;
          } else {
           $4977 = HEAP32[$pC56>>2]|0;
           $4978 = ((($4977)) + 16|0);
           $4979 = HEAP32[$4978>>2]|0;
           $4980 = (_sqlite3BtreeKeySize($4979,$v57)|0);
           HEAP32[$rc>>2] = $4980;
           break;
          }
         }
        } while(0);
        $4981 = $v57;
        $4982 = $4981;
        $4983 = HEAP32[$4982>>2]|0;
        $4984 = (($4981) + 4)|0;
        $4985 = $4984;
        $4986 = HEAP32[$4985>>2]|0;
        $4987 = HEAP32[$pOut>>2]|0;
        $4988 = $4987;
        $4989 = $4988;
        HEAP32[$4989>>2] = $4983;
        $4990 = (($4988) + 4)|0;
        $4991 = $4990;
        HEAP32[$4991>>2] = $4986;
        break;
       }
       case 104:  {
        $4992 = HEAP32[$pOp>>2]|0;
        $4993 = ((($4992)) + 4|0);
        $4994 = HEAP32[$4993>>2]|0;
        $4995 = HEAP32[$0>>2]|0;
        $4996 = ((($4995)) + 56|0);
        $4997 = HEAP32[$4996>>2]|0;
        $4998 = (($4997) + ($4994<<2)|0);
        $4999 = HEAP32[$4998>>2]|0;
        HEAP32[$pC58>>2] = $4999;
        $5000 = HEAP32[$pC58>>2]|0;
        $5001 = ((($5000)) + 2|0);
        HEAP8[$5001>>0] = 1;
        $5002 = HEAP32[$pC58>>2]|0;
        $5003 = ((($5002)) + 56|0);
        HEAP32[$5003>>2] = 0;
        $5004 = HEAP32[$pC58>>2]|0;
        $5005 = HEAP8[$5004>>0]|0;
        $5006 = $5005&255;
        $5007 = ($5006|0)==(0);
        if ($5007) {
         $5008 = HEAP32[$pC58>>2]|0;
         $5009 = ((($5008)) + 16|0);
         $5010 = HEAP32[$5009>>2]|0;
         _sqlite3BtreeClearCursor($5010);
        }
        break;
       }
       case 105:  {
        $5011 = HEAP32[$pOp>>2]|0;
        $5012 = ((($5011)) + 4|0);
        $5013 = HEAP32[$5012>>2]|0;
        $5014 = HEAP32[$0>>2]|0;
        $5015 = ((($5014)) + 56|0);
        $5016 = HEAP32[$5015>>2]|0;
        $5017 = (($5016) + ($5013<<2)|0);
        $5018 = HEAP32[$5017>>2]|0;
        HEAP32[$pC59>>2] = $5018;
        $5019 = HEAP32[$pC59>>2]|0;
        $5020 = ((($5019)) + 16|0);
        $5021 = HEAP32[$5020>>2]|0;
        HEAP32[$pCrsr60>>2] = $5021;
        HEAP32[$res61>>2] = 0;
        $5022 = HEAP32[$pCrsr60>>2]|0;
        $5023 = (_sqlite3BtreeLast($5022,$res61)|0);
        HEAP32[$rc>>2] = $5023;
        $5024 = HEAP32[$res61>>2]|0;
        $5025 = $5024&255;
        $5026 = HEAP32[$pC59>>2]|0;
        $5027 = ((($5026)) + 2|0);
        HEAP8[$5027>>0] = $5025;
        $5028 = HEAP32[$pC59>>2]|0;
        $5029 = ((($5028)) + 3|0);
        HEAP8[$5029>>0] = 0;
        $5030 = HEAP32[$pC59>>2]|0;
        $5031 = ((($5030)) + 56|0);
        HEAP32[$5031>>2] = 0;
        $5032 = HEAP32[$pOp>>2]|0;
        $5033 = ((($5032)) + 12|0);
        $5034 = HEAP32[$5033>>2]|0;
        $5035 = HEAP32[$pC59>>2]|0;
        $5036 = ((($5035)) + 28|0);
        HEAP32[$5036>>2] = $5034;
        $5037 = HEAP32[$pOp>>2]|0;
        $5038 = ((($5037)) + 8|0);
        $5039 = HEAP32[$5038>>2]|0;
        $5040 = ($5039|0)>(0);
        $5041 = HEAP32[$res61>>2]|0;
        $5042 = ($5041|0)!=(0);
        $or$cond30 = $5040 & $5042;
        if ($or$cond30) {
         label = 15;
        }
        break;
       }
       case 107: case 106:  {
        $5043 = HEAP32[$0>>2]|0;
        $5044 = ((($5043)) + 108|0);
        $5045 = ((($5044)) + 8|0);
        $5046 = HEAP32[$5045>>2]|0;
        $5047 = (($5046) + 1)|0;
        HEAP32[$5045>>2] = $5047;
        label = 609;
        break;
       }
       case 108:  {
        label = 609;
        break;
       }
       case 3:  {
        $5076 = HEAP32[$pOp>>2]|0;
        $5077 = ((($5076)) + 4|0);
        $5078 = HEAP32[$5077>>2]|0;
        $5079 = HEAP32[$0>>2]|0;
        $5080 = ((($5079)) + 56|0);
        $5081 = HEAP32[$5080>>2]|0;
        $5082 = (($5081) + ($5078<<2)|0);
        $5083 = HEAP32[$5082>>2]|0;
        HEAP32[$pC65>>2] = $5083;
        HEAP32[$res66>>2] = 0;
        $5084 = HEAP32[$db>>2]|0;
        $5085 = HEAP32[$pC65>>2]|0;
        $5086 = (_sqlite3VdbeSorterNext($5084,$5085,$res66)|0);
        HEAP32[$rc>>2] = $5086;
        label = 616;
        break;
       }
       case 5: case 4:  {
        $5087 = HEAP32[$pOp>>2]|0;
        $5088 = ((($5087)) + 4|0);
        $5089 = HEAP32[$5088>>2]|0;
        $5090 = HEAP32[$0>>2]|0;
        $5091 = ((($5090)) + 56|0);
        $5092 = HEAP32[$5091>>2]|0;
        $5093 = (($5092) + ($5089<<2)|0);
        $5094 = HEAP32[$5093>>2]|0;
        $5095 = ($5094|0)==(0|0);
        if (!($5095)) {
         label = 615;
        }
        break;
       }
       case 7: case 6:  {
        label = 615;
        break;
       }
       case 110: case 109:  {
        $5129 = HEAP32[$pOp>>2]|0;
        $5130 = ((($5129)) + 4|0);
        $5131 = HEAP32[$5130>>2]|0;
        $5132 = HEAP32[$0>>2]|0;
        $5133 = ((($5132)) + 56|0);
        $5134 = HEAP32[$5133>>2]|0;
        $5135 = (($5134) + ($5131<<2)|0);
        $5136 = HEAP32[$5135>>2]|0;
        HEAP32[$pC67>>2] = $5136;
        $5137 = HEAP32[$pOp>>2]|0;
        $5138 = ((($5137)) + 8|0);
        $5139 = HEAP32[$5138>>2]|0;
        $5140 = HEAP32[$aMem>>2]|0;
        $5141 = (($5140) + (($5139*40)|0)|0);
        HEAP32[$pIn2>>2] = $5141;
        $5142 = HEAP32[$pOp>>2]|0;
        $5143 = ((($5142)) + 3|0);
        $5144 = HEAP8[$5143>>0]|0;
        $5145 = $5144&255;
        $5146 = $5145 & 1;
        $5147 = ($5146|0)!=(0);
        if ($5147) {
         $5148 = HEAP32[$0>>2]|0;
         $5149 = ((($5148)) + 92|0);
         $5150 = HEAP32[$5149>>2]|0;
         $5151 = (($5150) + 1)|0;
         HEAP32[$5149>>2] = $5151;
        }
        $5152 = HEAP32[$pIn2>>2]|0;
        $5153 = ((($5152)) + 8|0);
        $5154 = HEAP16[$5153>>1]|0;
        $5155 = $5154&65535;
        $5156 = $5155 & 16384;
        $5157 = ($5156|0)!=(0);
        if ($5157) {
         $5158 = HEAP32[$pIn2>>2]|0;
         $5159 = (_sqlite3VdbeMemExpandBlob($5158)|0);
         $5160 = $5159;
        } else {
         $5160 = 0;
        }
        HEAP32[$rc>>2] = $5160;
        $5161 = HEAP32[$rc>>2]|0;
        $5162 = ($5161|0)==(0);
        if ($5162) {
         $5163 = HEAP32[$pOp>>2]|0;
         $5164 = HEAP8[$5163>>0]|0;
         $5165 = $5164&255;
         $5166 = ($5165|0)==(109);
         if ($5166) {
          $5167 = HEAP32[$pC67>>2]|0;
          $5168 = HEAP32[$pIn2>>2]|0;
          $5169 = (_sqlite3VdbeSorterWrite($5167,$5168)|0);
          HEAP32[$rc>>2] = $5169;
          break L11;
         }
         $5170 = HEAP32[$pIn2>>2]|0;
         $5171 = ((($5170)) + 12|0);
         $5172 = HEAP32[$5171>>2]|0;
         HEAP32[$nKey>>2] = $5172;
         $5173 = HEAP32[$pIn2>>2]|0;
         $5174 = ((($5173)) + 16|0);
         $5175 = HEAP32[$5174>>2]|0;
         HEAP32[$zKey>>2] = $5175;
         $5176 = HEAP32[$pC67>>2]|0;
         $5177 = ((($5176)) + 16|0);
         $5178 = HEAP32[$5177>>2]|0;
         $5179 = HEAP32[$zKey>>2]|0;
         $5180 = HEAP32[$nKey>>2]|0;
         $5181 = ($5180|0)<(0);
         $5182 = $5181 << 31 >> 31;
         $5183 = HEAP32[$pOp>>2]|0;
         $5184 = ((($5183)) + 12|0);
         $5185 = HEAP32[$5184>>2]|0;
         $5186 = HEAP32[$pOp>>2]|0;
         $5187 = ((($5186)) + 3|0);
         $5188 = HEAP8[$5187>>0]|0;
         $5189 = $5188&255;
         $5190 = $5189 & 16;
         $5191 = ($5190|0)!=(0);
         if ($5191) {
          $5192 = HEAP32[$pC67>>2]|0;
          $5193 = ((($5192)) + 28|0);
          $5194 = HEAP32[$5193>>2]|0;
          $5195 = $5194;
         } else {
          $5195 = 0;
         }
         $5196 = (_sqlite3BtreeInsert($5178,$5179,$5180,$5182,17532,0,0,$5185,$5195)|0);
         HEAP32[$rc>>2] = $5196;
         $5197 = HEAP32[$pC67>>2]|0;
         $5198 = ((($5197)) + 56|0);
         HEAP32[$5198>>2] = 0;
        }
        break;
       }
       case 111:  {
        $5199 = HEAP32[$pOp>>2]|0;
        $5200 = ((($5199)) + 4|0);
        $5201 = HEAP32[$5200>>2]|0;
        $5202 = HEAP32[$0>>2]|0;
        $5203 = ((($5202)) + 56|0);
        $5204 = HEAP32[$5203>>2]|0;
        $5205 = (($5204) + ($5201<<2)|0);
        $5206 = HEAP32[$5205>>2]|0;
        HEAP32[$pC68>>2] = $5206;
        $5207 = HEAP32[$pC68>>2]|0;
        $5208 = ((($5207)) + 16|0);
        $5209 = HEAP32[$5208>>2]|0;
        HEAP32[$pCrsr69>>2] = $5209;
        $5210 = HEAP32[$pC68>>2]|0;
        $5211 = ((($5210)) + 24|0);
        $5212 = HEAP32[$5211>>2]|0;
        HEAP32[$r71>>2] = $5212;
        $5213 = HEAP32[$pOp>>2]|0;
        $5214 = ((($5213)) + 12|0);
        $5215 = HEAP32[$5214>>2]|0;
        $5216 = $5215&65535;
        $5217 = ((($r71)) + 8|0);
        HEAP16[$5217>>1] = $5216;
        $5218 = ((($r71)) + 10|0);
        HEAP8[$5218>>0] = 0;
        $5219 = HEAP32[$pOp>>2]|0;
        $5220 = ((($5219)) + 8|0);
        $5221 = HEAP32[$5220>>2]|0;
        $5222 = HEAP32[$aMem>>2]|0;
        $5223 = (($5222) + (($5221*40)|0)|0);
        $5224 = ((($r71)) + 4|0);
        HEAP32[$5224>>2] = $5223;
        $5225 = HEAP32[$pCrsr69>>2]|0;
        $5226 = (_sqlite3BtreeMovetoUnpacked($5225,$r71,0,0,0,$res70)|0);
        HEAP32[$rc>>2] = $5226;
        $5227 = HEAP32[$rc>>2]|0;
        $5228 = ($5227|0)==(0);
        $5229 = HEAP32[$res70>>2]|0;
        $5230 = ($5229|0)==(0);
        $or$cond23 = $5228 & $5230;
        if ($or$cond23) {
         $5231 = HEAP32[$pCrsr69>>2]|0;
         $5232 = (_sqlite3BtreeDelete($5231,4)|0);
         HEAP32[$rc>>2] = $5232;
        }
        $5233 = HEAP32[$pC68>>2]|0;
        $5234 = ((($5233)) + 56|0);
        HEAP32[$5234>>2] = 0;
        break;
       }
       case 113: case 112:  {
        $5235 = HEAP32[$pOp>>2]|0;
        $5236 = ((($5235)) + 4|0);
        $5237 = HEAP32[$5236>>2]|0;
        $5238 = HEAP32[$0>>2]|0;
        $5239 = ((($5238)) + 56|0);
        $5240 = HEAP32[$5239>>2]|0;
        $5241 = (($5240) + ($5237<<2)|0);
        $5242 = HEAP32[$5241>>2]|0;
        HEAP32[$pC72>>2] = $5242;
        $5243 = HEAP32[$pC72>>2]|0;
        $5244 = (_sqlite3VdbeCursorRestore($5243)|0);
        HEAP32[$rc>>2] = $5244;
        $5245 = HEAP32[$rc>>2]|0;
        $5246 = ($5245|0)!=(0);
        if ($5246) {
         label = 861;
         break L8;
        }
        $5247 = HEAP32[$pC72>>2]|0;
        $5248 = ((($5247)) + 2|0);
        $5249 = HEAP8[$5248>>0]|0;
        $5250 = ($5249<<24>>24)!=(0);
        if ($5250) {
         $5314 = HEAP32[$pOp>>2]|0;
         $5315 = ((($5314)) + 8|0);
         $5316 = HEAP32[$5315>>2]|0;
         $5317 = HEAP32[$aMem>>2]|0;
         $5318 = (($5317) + (($5316*40)|0)|0);
         _sqlite3VdbeMemSetNull($5318);
         break L11;
        }
        $5251 = $rowid;
        $5252 = $5251;
        HEAP32[$5252>>2] = 0;
        $5253 = (($5251) + 4)|0;
        $5254 = $5253;
        HEAP32[$5254>>2] = 0;
        $5255 = HEAP32[$db>>2]|0;
        $5256 = HEAP32[$pC72>>2]|0;
        $5257 = ((($5256)) + 16|0);
        $5258 = HEAP32[$5257>>2]|0;
        $5259 = (_sqlite3VdbeIdxRowid($5255,$5258,$rowid)|0);
        HEAP32[$rc>>2] = $5259;
        $5260 = HEAP32[$rc>>2]|0;
        $5261 = ($5260|0)!=(0);
        if ($5261) {
         label = 861;
         break L8;
        }
        $5262 = HEAP32[$pOp>>2]|0;
        $5263 = HEAP8[$5262>>0]|0;
        $5264 = $5263&255;
        $5265 = ($5264|0)==(112);
        if ($5265) {
         $5266 = HEAP32[$pOp>>2]|0;
         $5267 = ((($5266)) + 12|0);
         $5268 = HEAP32[$5267>>2]|0;
         $5269 = HEAP32[$0>>2]|0;
         $5270 = ((($5269)) + 56|0);
         $5271 = HEAP32[$5270>>2]|0;
         $5272 = (($5271) + ($5268<<2)|0);
         $5273 = HEAP32[$5272>>2]|0;
         HEAP32[$pTabCur>>2] = $5273;
         $5274 = HEAP32[$pTabCur>>2]|0;
         $5275 = ((($5274)) + 2|0);
         HEAP8[$5275>>0] = 0;
         $5276 = $rowid;
         $5277 = $5276;
         $5278 = HEAP32[$5277>>2]|0;
         $5279 = (($5276) + 4)|0;
         $5280 = $5279;
         $5281 = HEAP32[$5280>>2]|0;
         $5282 = HEAP32[$pTabCur>>2]|0;
         $5283 = ((($5282)) + 40|0);
         $5284 = $5283;
         $5285 = $5284;
         HEAP32[$5285>>2] = $5278;
         $5286 = (($5284) + 4)|0;
         $5287 = $5286;
         HEAP32[$5287>>2] = $5281;
         $5288 = HEAP32[$pTabCur>>2]|0;
         $5289 = ((($5288)) + 3|0);
         HEAP8[$5289>>0] = 1;
         $5290 = HEAP32[$pOp>>2]|0;
         $5291 = ((($5290)) + 16|0);
         $5292 = HEAP32[$5291>>2]|0;
         $5293 = HEAP32[$pTabCur>>2]|0;
         $5294 = ((($5293)) + 52|0);
         HEAP32[$5294>>2] = $5292;
         $5295 = HEAP32[$pC72>>2]|0;
         $5296 = HEAP32[$pTabCur>>2]|0;
         $5297 = ((($5296)) + 48|0);
         HEAP32[$5297>>2] = $5295;
         break L11;
        } else {
         $5298 = HEAP32[$0>>2]|0;
         $5299 = HEAP32[$pOp>>2]|0;
         $5300 = (_out2Prerelease($5298,$5299)|0);
         HEAP32[$pOut>>2] = $5300;
         $5301 = $rowid;
         $5302 = $5301;
         $5303 = HEAP32[$5302>>2]|0;
         $5304 = (($5301) + 4)|0;
         $5305 = $5304;
         $5306 = HEAP32[$5305>>2]|0;
         $5307 = HEAP32[$pOut>>2]|0;
         $5308 = $5307;
         $5309 = $5308;
         HEAP32[$5309>>2] = $5303;
         $5310 = (($5308) + 4)|0;
         $5311 = $5310;
         HEAP32[$5311>>2] = $5306;
         $5312 = HEAP32[$pOut>>2]|0;
         $5313 = ((($5312)) + 8|0);
         HEAP16[$5313>>1] = 4;
         break L11;
        }
        break;
       }
       case 117: case 116: case 115: case 114:  {
        $5319 = HEAP32[$pOp>>2]|0;
        $5320 = ((($5319)) + 4|0);
        $5321 = HEAP32[$5320>>2]|0;
        $5322 = HEAP32[$0>>2]|0;
        $5323 = ((($5322)) + 56|0);
        $5324 = HEAP32[$5323>>2]|0;
        $5325 = (($5324) + ($5321<<2)|0);
        $5326 = HEAP32[$5325>>2]|0;
        HEAP32[$pC73>>2] = $5326;
        $5327 = HEAP32[$pC73>>2]|0;
        $5328 = ((($5327)) + 24|0);
        $5329 = HEAP32[$5328>>2]|0;
        HEAP32[$r75>>2] = $5329;
        $5330 = HEAP32[$pOp>>2]|0;
        $5331 = ((($5330)) + 16|0);
        $5332 = HEAP32[$5331>>2]|0;
        $5333 = $5332&65535;
        $5334 = ((($r75)) + 8|0);
        HEAP16[$5334>>1] = $5333;
        $5335 = HEAP32[$pOp>>2]|0;
        $5336 = HEAP8[$5335>>0]|0;
        $5337 = $5336&255;
        $5338 = ($5337|0)<(116);
        $5339 = ((($r75)) + 10|0);
        if ($5338) {
         HEAP8[$5339>>0] = -1;
        } else {
         HEAP8[$5339>>0] = 0;
        }
        $5340 = HEAP32[$pOp>>2]|0;
        $5341 = ((($5340)) + 12|0);
        $5342 = HEAP32[$5341>>2]|0;
        $5343 = HEAP32[$aMem>>2]|0;
        $5344 = (($5343) + (($5342*40)|0)|0);
        $5345 = ((($r75)) + 4|0);
        HEAP32[$5345>>2] = $5344;
        HEAP32[$res74>>2] = 0;
        $5346 = HEAP32[$db>>2]|0;
        $5347 = HEAP32[$pC73>>2]|0;
        $5348 = (_sqlite3VdbeIdxKeyCompare($5346,$5347,$r75,$res74)|0);
        HEAP32[$rc>>2] = $5348;
        $5349 = HEAP32[$pOp>>2]|0;
        $5350 = HEAP8[$5349>>0]|0;
        $5351 = $5350&255;
        $5352 = $5351 & 1;
        $5353 = ($5352|0)==(0);
        $5354 = HEAP32[$res74>>2]|0;
        if ($5353) {
         $5355 = (0 - ($5354))|0;
         HEAP32[$res74>>2] = $5355;
        } else {
         $5356 = (($5354) + 1)|0;
         HEAP32[$res74>>2] = $5356;
        }
        $5357 = HEAP32[$res74>>2]|0;
        $5358 = ($5357|0)>(0);
        if ($5358) {
         label = 15;
        }
        break;
       }
       case 118:  {
        $5359 = HEAP32[$0>>2]|0;
        $5360 = HEAP32[$pOp>>2]|0;
        $5361 = (_out2Prerelease($5359,$5360)|0);
        HEAP32[$pOut>>2] = $5361;
        $5362 = HEAP32[$pOut>>2]|0;
        $5363 = ((($5362)) + 8|0);
        HEAP16[$5363>>1] = 1;
        $5364 = HEAP32[$db>>2]|0;
        $5365 = ((($5364)) + 152|0);
        $5366 = HEAP32[$5365>>2]|0;
        $5367 = HEAP32[$db>>2]|0;
        $5368 = ((($5367)) + 164|0);
        $5369 = HEAP32[$5368>>2]|0;
        $5370 = (($5369) + 1)|0;
        $5371 = ($5366|0)>($5370|0);
        if ($5371) {
         HEAP32[$rc>>2] = 6;
         $5372 = HEAP32[$0>>2]|0;
         $5373 = ((($5372)) + 86|0);
         HEAP8[$5373>>0] = 2;
         break L11;
        }
        $5374 = HEAP32[$pOp>>2]|0;
        $5375 = ((($5374)) + 12|0);
        $5376 = HEAP32[$5375>>2]|0;
        HEAP32[$iDb76>>2] = $5376;
        HEAP32[$iMoved>>2] = 0;
        $5377 = HEAP32[$iDb76>>2]|0;
        $5378 = HEAP32[$db>>2]|0;
        $5379 = ((($5378)) + 16|0);
        $5380 = HEAP32[$5379>>2]|0;
        $5381 = (($5380) + ($5377<<4)|0);
        $5382 = ((($5381)) + 4|0);
        $5383 = HEAP32[$5382>>2]|0;
        $5384 = HEAP32[$pOp>>2]|0;
        $5385 = ((($5384)) + 4|0);
        $5386 = HEAP32[$5385>>2]|0;
        $5387 = (_sqlite3BtreeDropTable($5383,$5386,$iMoved)|0);
        HEAP32[$rc>>2] = $5387;
        $5388 = HEAP32[$pOut>>2]|0;
        $5389 = ((($5388)) + 8|0);
        HEAP16[$5389>>1] = 4;
        $5390 = HEAP32[$iMoved>>2]|0;
        $5391 = ($5390|0)<(0);
        $5392 = $5391 << 31 >> 31;
        $5393 = HEAP32[$pOut>>2]|0;
        $5394 = $5393;
        $5395 = $5394;
        HEAP32[$5395>>2] = $5390;
        $5396 = (($5394) + 4)|0;
        $5397 = $5396;
        HEAP32[$5397>>2] = $5392;
        $5398 = HEAP32[$rc>>2]|0;
        $5399 = ($5398|0)==(0);
        $5400 = HEAP32[$iMoved>>2]|0;
        $5401 = ($5400|0)!=(0);
        $or$cond25 = $5399 & $5401;
        if ($or$cond25) {
         $5402 = HEAP32[$db>>2]|0;
         $5403 = HEAP32[$iDb76>>2]|0;
         $5404 = HEAP32[$iMoved>>2]|0;
         $5405 = HEAP32[$pOp>>2]|0;
         $5406 = ((($5405)) + 4|0);
         $5407 = HEAP32[$5406>>2]|0;
         _sqlite3RootPageMoved($5402,$5403,$5404,$5407);
         $5408 = HEAP32[$iDb76>>2]|0;
         $5409 = (($5408) + 1)|0;
         $5410 = $5409&255;
         HEAP8[$resetSchemaOnFault>>0] = $5410;
        }
        break;
       }
       case 119:  {
        HEAP32[$nChange>>2] = 0;
        $5411 = HEAP32[$pOp>>2]|0;
        $5412 = ((($5411)) + 8|0);
        $5413 = HEAP32[$5412>>2]|0;
        $5414 = HEAP32[$db>>2]|0;
        $5415 = ((($5414)) + 16|0);
        $5416 = HEAP32[$5415>>2]|0;
        $5417 = (($5416) + ($5413<<4)|0);
        $5418 = ((($5417)) + 4|0);
        $5419 = HEAP32[$5418>>2]|0;
        $5420 = HEAP32[$pOp>>2]|0;
        $5421 = ((($5420)) + 4|0);
        $5422 = HEAP32[$5421>>2]|0;
        $5423 = HEAP32[$pOp>>2]|0;
        $5424 = ((($5423)) + 12|0);
        $5425 = HEAP32[$5424>>2]|0;
        $5426 = ($5425|0)!=(0);
        $nChange$ = $5426 ? $nChange : 0;
        $5427 = (_sqlite3BtreeClearTable($5419,$5422,$nChange$)|0);
        HEAP32[$rc>>2] = $5427;
        $5428 = HEAP32[$pOp>>2]|0;
        $5429 = ((($5428)) + 12|0);
        $5430 = HEAP32[$5429>>2]|0;
        $5431 = ($5430|0)!=(0);
        if ($5431) {
         $5432 = HEAP32[$nChange>>2]|0;
         $5433 = HEAP32[$0>>2]|0;
         $5434 = ((($5433)) + 92|0);
         $5435 = HEAP32[$5434>>2]|0;
         $5436 = (($5435) + ($5432))|0;
         HEAP32[$5434>>2] = $5436;
         $5437 = HEAP32[$pOp>>2]|0;
         $5438 = ((($5437)) + 12|0);
         $5439 = HEAP32[$5438>>2]|0;
         $5440 = ($5439|0)>(0);
         if ($5440) {
          $5441 = HEAP32[$nChange>>2]|0;
          $5442 = ($5441|0)<(0);
          $5443 = $5442 << 31 >> 31;
          $5444 = HEAP32[$pOp>>2]|0;
          $5445 = ((($5444)) + 12|0);
          $5446 = HEAP32[$5445>>2]|0;
          $5447 = HEAP32[$aMem>>2]|0;
          $5448 = (($5447) + (($5446*40)|0)|0);
          $5449 = $5448;
          $5450 = $5449;
          $5451 = HEAP32[$5450>>2]|0;
          $5452 = (($5449) + 4)|0;
          $5453 = $5452;
          $5454 = HEAP32[$5453>>2]|0;
          $5455 = (_i64Add(($5451|0),($5454|0),($5441|0),($5443|0))|0);
          $5456 = tempRet0;
          $5457 = $5448;
          $5458 = $5457;
          HEAP32[$5458>>2] = $5455;
          $5459 = (($5457) + 4)|0;
          $5460 = $5459;
          HEAP32[$5460>>2] = $5456;
         }
        }
        break;
       }
       case 120:  {
        $5461 = HEAP32[$pOp>>2]|0;
        $5462 = ((($5461)) + 4|0);
        $5463 = HEAP32[$5462>>2]|0;
        $5464 = HEAP32[$0>>2]|0;
        $5465 = ((($5464)) + 56|0);
        $5466 = HEAP32[$5465>>2]|0;
        $5467 = (($5466) + ($5463<<2)|0);
        $5468 = HEAP32[$5467>>2]|0;
        HEAP32[$pC77>>2] = $5468;
        $5469 = HEAP32[$pC77>>2]|0;
        $5470 = HEAP8[$5469>>0]|0;
        $5471 = $5470&255;
        $5472 = ($5471|0)==(1);
        if ($5472) {
         $5473 = HEAP32[$db>>2]|0;
         $5474 = HEAP32[$pC77>>2]|0;
         $5475 = ((($5474)) + 16|0);
         $5476 = HEAP32[$5475>>2]|0;
         _sqlite3VdbeSorterReset($5473,$5476);
         break L11;
        } else {
         $5477 = HEAP32[$pC77>>2]|0;
         $5478 = ((($5477)) + 16|0);
         $5479 = HEAP32[$5478>>2]|0;
         $5480 = (_sqlite3BtreeClearTableOfCursor($5479)|0);
         HEAP32[$rc>>2] = $5480;
         break L11;
        }
        break;
       }
       case 122: case 121:  {
        $5481 = HEAP32[$0>>2]|0;
        $5482 = HEAP32[$pOp>>2]|0;
        $5483 = (_out2Prerelease($5481,$5482)|0);
        HEAP32[$pOut>>2] = $5483;
        HEAP32[$pgno78>>2] = 0;
        $5484 = HEAP32[$pOp>>2]|0;
        $5485 = ((($5484)) + 4|0);
        $5486 = HEAP32[$5485>>2]|0;
        $5487 = HEAP32[$db>>2]|0;
        $5488 = ((($5487)) + 16|0);
        $5489 = HEAP32[$5488>>2]|0;
        $5490 = (($5489) + ($5486<<4)|0);
        HEAP32[$pDb80>>2] = $5490;
        $5491 = HEAP32[$pOp>>2]|0;
        $5492 = HEAP8[$5491>>0]|0;
        $5493 = $5492&255;
        $5494 = ($5493|0)==(122);
        if ($5494) {
         HEAP32[$flags79>>2] = 1;
        } else {
         HEAP32[$flags79>>2] = 2;
        }
        $5495 = HEAP32[$pDb80>>2]|0;
        $5496 = ((($5495)) + 4|0);
        $5497 = HEAP32[$5496>>2]|0;
        $5498 = HEAP32[$flags79>>2]|0;
        $5499 = (_sqlite3BtreeCreateTable($5497,$pgno78,$5498)|0);
        HEAP32[$rc>>2] = $5499;
        $5500 = HEAP32[$pgno78>>2]|0;
        $5501 = ($5500|0)<(0);
        $5502 = $5501 << 31 >> 31;
        $5503 = HEAP32[$pOut>>2]|0;
        $5504 = $5503;
        $5505 = $5504;
        HEAP32[$5505>>2] = $5500;
        $5506 = (($5504) + 4)|0;
        $5507 = $5506;
        HEAP32[$5507>>2] = $5502;
        break;
       }
       case 123:  {
        $5508 = HEAP32[$pOp>>2]|0;
        $5509 = ((($5508)) + 4|0);
        $5510 = HEAP32[$5509>>2]|0;
        HEAP32[$iDb81>>2] = $5510;
        $5511 = HEAP32[$iDb81>>2]|0;
        $5512 = ($5511|0)==(1);
        $5513 = $5512 ? 21260 : 21279;
        HEAP32[$zMaster>>2] = $5513;
        $5514 = HEAP32[$db>>2]|0;
        HEAP32[$initData>>2] = $5514;
        $5515 = HEAP32[$pOp>>2]|0;
        $5516 = ((($5515)) + 4|0);
        $5517 = HEAP32[$5516>>2]|0;
        $5518 = ((($initData)) + 8|0);
        HEAP32[$5518>>2] = $5517;
        $5519 = HEAP32[$0>>2]|0;
        $5520 = ((($5519)) + 44|0);
        $5521 = ((($initData)) + 4|0);
        HEAP32[$5521>>2] = $5520;
        $5522 = HEAP32[$db>>2]|0;
        $5523 = HEAP32[$iDb81>>2]|0;
        $5524 = HEAP32[$db>>2]|0;
        $5525 = ((($5524)) + 16|0);
        $5526 = HEAP32[$5525>>2]|0;
        $5527 = (($5526) + ($5523<<4)|0);
        $5528 = HEAP32[$5527>>2]|0;
        $5529 = HEAP32[$zMaster>>2]|0;
        $5530 = HEAP32[$pOp>>2]|0;
        $5531 = ((($5530)) + 16|0);
        $5532 = HEAP32[$5531>>2]|0;
        HEAP32[$vararg_buffer57>>2] = $5528;
        $vararg_ptr60 = ((($vararg_buffer57)) + 4|0);
        HEAP32[$vararg_ptr60>>2] = $5529;
        $vararg_ptr61 = ((($vararg_buffer57)) + 8|0);
        HEAP32[$vararg_ptr61>>2] = $5532;
        $5533 = (_sqlite3MPrintf($5522,21293,$vararg_buffer57)|0);
        HEAP32[$zSql>>2] = $5533;
        $5534 = HEAP32[$zSql>>2]|0;
        $5535 = ($5534|0)==(0|0);
        if ($5535) {
         HEAP32[$rc>>2] = 7;
        } else {
         $5536 = HEAP32[$db>>2]|0;
         $5537 = ((($5536)) + 140|0);
         $5538 = ((($5537)) + 5|0);
         HEAP8[$5538>>0] = 1;
         $5539 = ((($initData)) + 12|0);
         HEAP32[$5539>>2] = 0;
         $5540 = HEAP32[$db>>2]|0;
         $5541 = HEAP32[$zSql>>2]|0;
         $5542 = (_sqlite3_exec($5540,$5541,140,$initData,0)|0);
         HEAP32[$rc>>2] = $5542;
         $5543 = HEAP32[$rc>>2]|0;
         $5544 = ($5543|0)==(0);
         if ($5544) {
          $5545 = ((($initData)) + 12|0);
          $5546 = HEAP32[$5545>>2]|0;
          HEAP32[$rc>>2] = $5546;
         }
         $5547 = HEAP32[$db>>2]|0;
         $5548 = HEAP32[$zSql>>2]|0;
         _sqlite3DbFree($5547,$5548);
         $5549 = HEAP32[$db>>2]|0;
         $5550 = ((($5549)) + 140|0);
         $5551 = ((($5550)) + 5|0);
         HEAP8[$5551>>0] = 0;
        }
        $5552 = HEAP32[$rc>>2]|0;
        $5553 = ($5552|0)!=(0);
        if ($5553) {
         $5554 = HEAP32[$db>>2]|0;
         _sqlite3ResetAllSchemasOfConnection($5554);
        }
        $5555 = HEAP32[$rc>>2]|0;
        $5556 = ($5555|0)==(7);
        if ($5556) {
         label = 860;
         break L1;
        }
        break;
       }
       case 124:  {
        $5557 = HEAP32[$db>>2]|0;
        $5558 = HEAP32[$pOp>>2]|0;
        $5559 = ((($5558)) + 4|0);
        $5560 = HEAP32[$5559>>2]|0;
        $5561 = (_sqlite3AnalysisLoad($5557,$5560)|0);
        HEAP32[$rc>>2] = $5561;
        break;
       }
       case 125:  {
        $5562 = HEAP32[$db>>2]|0;
        $5563 = HEAP32[$pOp>>2]|0;
        $5564 = ((($5563)) + 4|0);
        $5565 = HEAP32[$5564>>2]|0;
        $5566 = HEAP32[$pOp>>2]|0;
        $5567 = ((($5566)) + 16|0);
        $5568 = HEAP32[$5567>>2]|0;
        _sqlite3UnlinkAndDeleteTable($5562,$5565,$5568);
        break;
       }
       case 126:  {
        $5569 = HEAP32[$db>>2]|0;
        $5570 = HEAP32[$pOp>>2]|0;
        $5571 = ((($5570)) + 4|0);
        $5572 = HEAP32[$5571>>2]|0;
        $5573 = HEAP32[$pOp>>2]|0;
        $5574 = ((($5573)) + 16|0);
        $5575 = HEAP32[$5574>>2]|0;
        _sqlite3UnlinkAndDeleteIndex($5569,$5572,$5575);
        break;
       }
       case 127:  {
        $5576 = HEAP32[$db>>2]|0;
        $5577 = HEAP32[$pOp>>2]|0;
        $5578 = ((($5577)) + 4|0);
        $5579 = HEAP32[$5578>>2]|0;
        $5580 = HEAP32[$pOp>>2]|0;
        $5581 = ((($5580)) + 16|0);
        $5582 = HEAP32[$5581>>2]|0;
        _sqlite3UnlinkAndDeleteTrigger($5576,$5579,$5582);
        break;
       }
       case 128:  {
        $5583 = HEAP32[$pOp>>2]|0;
        $5584 = ((($5583)) + 8|0);
        $5585 = HEAP32[$5584>>2]|0;
        HEAP32[$nRoot>>2] = $5585;
        $5586 = HEAP32[$db>>2]|0;
        $5587 = HEAP32[$nRoot>>2]|0;
        $5588 = (($5587) + 1)|0;
        $5589 = $5588<<2;
        $5590 = (_sqlite3DbMallocRawNN($5586,$5589,0)|0);
        HEAP32[$aRoot>>2] = $5590;
        $5591 = HEAP32[$aRoot>>2]|0;
        $5592 = ($5591|0)==(0|0);
        if ($5592) {
         label = 860;
         break L1;
        }
        $5593 = HEAP32[$pOp>>2]|0;
        $5594 = ((($5593)) + 12|0);
        $5595 = HEAP32[$5594>>2]|0;
        $5596 = HEAP32[$aMem>>2]|0;
        $5597 = (($5596) + (($5595*40)|0)|0);
        HEAP32[$pnErr>>2] = $5597;
        $5598 = HEAP32[$pOp>>2]|0;
        $5599 = ((($5598)) + 4|0);
        $5600 = HEAP32[$5599>>2]|0;
        $5601 = HEAP32[$aMem>>2]|0;
        $5602 = (($5601) + (($5600*40)|0)|0);
        HEAP32[$pIn1>>2] = $5602;
        HEAP32[$j82>>2] = 0;
        while(1) {
         $5603 = HEAP32[$j82>>2]|0;
         $5604 = HEAP32[$nRoot>>2]|0;
         $5605 = ($5603|0)<($5604|0);
         $5606 = HEAP32[$j82>>2]|0;
         if (!($5605)) {
          break;
         }
         $5607 = HEAP32[$pIn1>>2]|0;
         $5608 = (($5607) + (($5606*40)|0)|0);
         $5609 = (_sqlite3VdbeIntValue($5608)|0);
         $5610 = tempRet0;
         $5611 = HEAP32[$j82>>2]|0;
         $5612 = HEAP32[$aRoot>>2]|0;
         $5613 = (($5612) + ($5611<<2)|0);
         HEAP32[$5613>>2] = $5609;
         $5614 = HEAP32[$j82>>2]|0;
         $5615 = (($5614) + 1)|0;
         HEAP32[$j82>>2] = $5615;
        }
        $5616 = HEAP32[$aRoot>>2]|0;
        $5617 = (($5616) + ($5606<<2)|0);
        HEAP32[$5617>>2] = 0;
        $5618 = HEAP32[$pOp>>2]|0;
        $5619 = ((($5618)) + 3|0);
        $5620 = HEAP8[$5619>>0]|0;
        $5621 = $5620&255;
        $5622 = HEAP32[$db>>2]|0;
        $5623 = ((($5622)) + 16|0);
        $5624 = HEAP32[$5623>>2]|0;
        $5625 = (($5624) + ($5621<<4)|0);
        $5626 = ((($5625)) + 4|0);
        $5627 = HEAP32[$5626>>2]|0;
        $5628 = HEAP32[$aRoot>>2]|0;
        $5629 = HEAP32[$nRoot>>2]|0;
        $5630 = HEAP32[$pnErr>>2]|0;
        $5631 = $5630;
        $5632 = $5631;
        $5633 = HEAP32[$5632>>2]|0;
        $5634 = (($5631) + 4)|0;
        $5635 = $5634;
        $5636 = HEAP32[$5635>>2]|0;
        $5637 = (_sqlite3BtreeIntegrityCheck($5627,$5628,$5629,$5633,$nErr)|0);
        HEAP32[$z>>2] = $5637;
        $5638 = HEAP32[$db>>2]|0;
        $5639 = HEAP32[$aRoot>>2]|0;
        _sqlite3DbFree($5638,$5639);
        $5640 = HEAP32[$nErr>>2]|0;
        $5641 = ($5640|0)<(0);
        $5642 = $5641 << 31 >> 31;
        $5643 = HEAP32[$pnErr>>2]|0;
        $5644 = $5643;
        $5645 = $5644;
        $5646 = HEAP32[$5645>>2]|0;
        $5647 = (($5644) + 4)|0;
        $5648 = $5647;
        $5649 = HEAP32[$5648>>2]|0;
        $5650 = (_i64Subtract(($5646|0),($5649|0),($5640|0),($5642|0))|0);
        $5651 = tempRet0;
        $5652 = $5643;
        $5653 = $5652;
        HEAP32[$5653>>2] = $5650;
        $5654 = (($5652) + 4)|0;
        $5655 = $5654;
        HEAP32[$5655>>2] = $5651;
        $5656 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemSetNull($5656);
        $5657 = HEAP32[$nErr>>2]|0;
        $5658 = ($5657|0)==(0);
        if (!($5658)) {
         $5659 = HEAP32[$z>>2]|0;
         $5660 = ($5659|0)==(0|0);
         if ($5660) {
          label = 860;
          break L1;
         }
         $5661 = HEAP32[$pIn1>>2]|0;
         $5662 = HEAP32[$z>>2]|0;
         (_sqlite3VdbeMemSetStr($5661,$5662,-1,1,142)|0);
        }
        $5663 = HEAP32[$pIn1>>2]|0;
        $5664 = HEAP8[$encoding>>0]|0;
        $5665 = $5664&255;
        (_sqlite3VdbeChangeEncoding($5663,$5665)|0);
        break;
       }
       case 129:  {
        $5666 = HEAP32[$pOp>>2]|0;
        $5667 = ((($5666)) + 4|0);
        $5668 = HEAP32[$5667>>2]|0;
        $5669 = HEAP32[$aMem>>2]|0;
        $5670 = (($5669) + (($5668*40)|0)|0);
        HEAP32[$pIn1>>2] = $5670;
        $5671 = HEAP32[$pOp>>2]|0;
        $5672 = ((($5671)) + 8|0);
        $5673 = HEAP32[$5672>>2]|0;
        $5674 = HEAP32[$aMem>>2]|0;
        $5675 = (($5674) + (($5673*40)|0)|0);
        HEAP32[$pIn2>>2] = $5675;
        $5676 = HEAP32[$pIn1>>2]|0;
        $5677 = ((($5676)) + 8|0);
        $5678 = HEAP16[$5677>>1]|0;
        $5679 = $5678&65535;
        $5680 = $5679 & 32;
        $5681 = ($5680|0)==(0);
        if ($5681) {
         $5682 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemSetRowSet($5682);
         $5683 = HEAP32[$pIn1>>2]|0;
         $5684 = ((($5683)) + 8|0);
         $5685 = HEAP16[$5684>>1]|0;
         $5686 = $5685&65535;
         $5687 = $5686 & 32;
         $5688 = ($5687|0)==(0);
         if ($5688) {
          label = 860;
          break L1;
         }
        }
        $5689 = HEAP32[$pIn1>>2]|0;
        $5690 = HEAP32[$5689>>2]|0;
        $5691 = HEAP32[$pIn2>>2]|0;
        $5692 = $5691;
        $5693 = $5692;
        $5694 = HEAP32[$5693>>2]|0;
        $5695 = (($5692) + 4)|0;
        $5696 = $5695;
        $5697 = HEAP32[$5696>>2]|0;
        _sqlite3RowSetInsert($5690,$5694,$5697);
        break;
       }
       case 130:  {
        $5698 = HEAP32[$pOp>>2]|0;
        $5699 = ((($5698)) + 4|0);
        $5700 = HEAP32[$5699>>2]|0;
        $5701 = HEAP32[$aMem>>2]|0;
        $5702 = (($5701) + (($5700*40)|0)|0);
        HEAP32[$pIn1>>2] = $5702;
        $5703 = HEAP32[$pIn1>>2]|0;
        $5704 = ((($5703)) + 8|0);
        $5705 = HEAP16[$5704>>1]|0;
        $5706 = $5705&65535;
        $5707 = $5706 & 32;
        $5708 = ($5707|0)==(0);
        if (!($5708)) {
         $5709 = HEAP32[$pIn1>>2]|0;
         $5710 = HEAP32[$5709>>2]|0;
         $5711 = (_sqlite3RowSetNext($5710,$val)|0);
         $5712 = ($5711|0)==(0);
         if (!($5712)) {
          $5714 = HEAP32[$pOp>>2]|0;
          $5715 = ((($5714)) + 12|0);
          $5716 = HEAP32[$5715>>2]|0;
          $5717 = HEAP32[$aMem>>2]|0;
          $5718 = (($5717) + (($5716*40)|0)|0);
          $5719 = $val;
          $5720 = $5719;
          $5721 = HEAP32[$5720>>2]|0;
          $5722 = (($5719) + 4)|0;
          $5723 = $5722;
          $5724 = HEAP32[$5723>>2]|0;
          _sqlite3VdbeMemSetInt64($5718,$5721,$5724);
          label = 9;
          break L11;
         }
        }
        $5713 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemSetNull($5713);
        label = 8;
        break;
       }
       case 131:  {
        $5725 = HEAP32[$pOp>>2]|0;
        $5726 = ((($5725)) + 4|0);
        $5727 = HEAP32[$5726>>2]|0;
        $5728 = HEAP32[$aMem>>2]|0;
        $5729 = (($5728) + (($5727*40)|0)|0);
        HEAP32[$pIn1>>2] = $5729;
        $5730 = HEAP32[$pOp>>2]|0;
        $5731 = ((($5730)) + 12|0);
        $5732 = HEAP32[$5731>>2]|0;
        $5733 = HEAP32[$aMem>>2]|0;
        $5734 = (($5733) + (($5732*40)|0)|0);
        HEAP32[$pIn3>>2] = $5734;
        $5735 = HEAP32[$pOp>>2]|0;
        $5736 = ((($5735)) + 16|0);
        $5737 = HEAP32[$5736>>2]|0;
        HEAP32[$iSet>>2] = $5737;
        $5738 = HEAP32[$pIn1>>2]|0;
        $5739 = ((($5738)) + 8|0);
        $5740 = HEAP16[$5739>>1]|0;
        $5741 = $5740&65535;
        $5742 = $5741 & 32;
        $5743 = ($5742|0)==(0);
        if ($5743) {
         $5744 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemSetRowSet($5744);
         $5745 = HEAP32[$pIn1>>2]|0;
         $5746 = ((($5745)) + 8|0);
         $5747 = HEAP16[$5746>>1]|0;
         $5748 = $5747&65535;
         $5749 = $5748 & 32;
         $5750 = ($5749|0)==(0);
         if ($5750) {
          label = 860;
          break L1;
         }
        }
        $5751 = HEAP32[$iSet>>2]|0;
        $5752 = ($5751|0)!=(0);
        if ($5752) {
         $5753 = HEAP32[$pIn1>>2]|0;
         $5754 = HEAP32[$5753>>2]|0;
         $5755 = HEAP32[$iSet>>2]|0;
         $5756 = HEAP32[$pIn3>>2]|0;
         $5757 = $5756;
         $5758 = $5757;
         $5759 = HEAP32[$5758>>2]|0;
         $5760 = (($5757) + 4)|0;
         $5761 = $5760;
         $5762 = HEAP32[$5761>>2]|0;
         $5763 = (_sqlite3RowSetTest($5754,$5755,$5759,$5762)|0);
         HEAP32[$exists>>2] = $5763;
         $5764 = HEAP32[$exists>>2]|0;
         $5765 = ($5764|0)!=(0);
         if ($5765) {
          label = 15;
          break L11;
         }
        }
        $5766 = HEAP32[$iSet>>2]|0;
        $5767 = ($5766|0)>=(0);
        if ($5767) {
         $5768 = HEAP32[$pIn1>>2]|0;
         $5769 = HEAP32[$5768>>2]|0;
         $5770 = HEAP32[$pIn3>>2]|0;
         $5771 = $5770;
         $5772 = $5771;
         $5773 = HEAP32[$5772>>2]|0;
         $5774 = (($5771) + 4)|0;
         $5775 = $5774;
         $5776 = HEAP32[$5775>>2]|0;
         _sqlite3RowSetInsert($5769,$5773,$5776);
        }
        break;
       }
       case 132:  {
        $5777 = HEAP32[$pOp>>2]|0;
        $5778 = ((($5777)) + 16|0);
        $5779 = HEAP32[$5778>>2]|0;
        HEAP32[$pProgram>>2] = $5779;
        $5780 = HEAP32[$pOp>>2]|0;
        $5781 = ((($5780)) + 12|0);
        $5782 = HEAP32[$5781>>2]|0;
        $5783 = HEAP32[$aMem>>2]|0;
        $5784 = (($5783) + (($5782*40)|0)|0);
        HEAP32[$pRt>>2] = $5784;
        $5785 = HEAP32[$pOp>>2]|0;
        $5786 = ((($5785)) + 3|0);
        $5787 = HEAP8[$5786>>0]|0;
        $5788 = ($5787<<24>>24)!=(0);
        if ($5788) {
         $5789 = HEAP32[$pProgram>>2]|0;
         $5790 = ((($5789)) + 20|0);
         $5791 = HEAP32[$5790>>2]|0;
         HEAP32[$t86>>2] = $5791;
         $5792 = HEAP32[$0>>2]|0;
         $5793 = ((($5792)) + 176|0);
         $5794 = HEAP32[$5793>>2]|0;
         HEAP32[$pFrame85>>2] = $5794;
         while(1) {
          $5795 = HEAP32[$pFrame85>>2]|0;
          $5796 = ($5795|0)!=(0|0);
          if (!($5796)) {
           break;
          }
          $5797 = HEAP32[$pFrame85>>2]|0;
          $5798 = ((($5797)) + 28|0);
          $5799 = HEAP32[$5798>>2]|0;
          $5800 = HEAP32[$t86>>2]|0;
          $5801 = ($5799|0)!=($5800|0);
          if (!($5801)) {
           break;
          }
          $5802 = HEAP32[$pFrame85>>2]|0;
          $5803 = ((($5802)) + 4|0);
          $5804 = HEAP32[$5803>>2]|0;
          HEAP32[$pFrame85>>2] = $5804;
         }
         $5805 = HEAP32[$pFrame85>>2]|0;
         $5806 = ($5805|0)!=(0|0);
         if ($5806) {
          break L11;
         }
        }
        $5807 = HEAP32[$0>>2]|0;
        $5808 = ((($5807)) + 184|0);
        $5809 = HEAP32[$5808>>2]|0;
        $5810 = HEAP32[$db>>2]|0;
        $5811 = ((($5810)) + 88|0);
        $5812 = ((($5811)) + 40|0);
        $5813 = HEAP32[$5812>>2]|0;
        $5814 = ($5809|0)>=($5813|0);
        if ($5814) {
         HEAP32[$rc>>2] = 1;
         $5815 = HEAP32[$0>>2]|0;
         _sqlite3VdbeError($5815,21357,$vararg_buffer62);
         break L11;
        }
        $5816 = HEAP32[$pRt>>2]|0;
        $5817 = ((($5816)) + 8|0);
        $5818 = HEAP16[$5817>>1]|0;
        $5819 = $5818&65535;
        $5820 = $5819 & 64;
        $5821 = ($5820|0)==(0);
        L800: do {
         if ($5821) {
          $5822 = HEAP32[$pProgram>>2]|0;
          $5823 = ((($5822)) + 8|0);
          $5824 = HEAP32[$5823>>2]|0;
          $5825 = HEAP32[$pProgram>>2]|0;
          $5826 = ((($5825)) + 12|0);
          $5827 = HEAP32[$5826>>2]|0;
          $5828 = (($5824) + ($5827))|0;
          HEAP32[$nMem>>2] = $5828;
          $5829 = HEAP32[$nMem>>2]|0;
          $5830 = ($5829*40)|0;
          $5831 = (80 + ($5830))|0;
          $5832 = HEAP32[$pProgram>>2]|0;
          $5833 = ((($5832)) + 12|0);
          $5834 = HEAP32[$5833>>2]|0;
          $5835 = $5834<<2;
          $5836 = (($5831) + ($5835))|0;
          $5837 = HEAP32[$pProgram>>2]|0;
          $5838 = ((($5837)) + 16|0);
          $5839 = HEAP32[$5838>>2]|0;
          $5840 = $5839;
          $5841 = (($5836) + ($5840))|0;
          HEAP32[$nByte83>>2] = $5841;
          $5842 = HEAP32[$db>>2]|0;
          $5843 = HEAP32[$nByte83>>2]|0;
          $5844 = ($5843|0)<(0);
          $5845 = $5844 << 31 >> 31;
          $5846 = (_sqlite3DbMallocZero($5842,$5843,$5845)|0);
          HEAP32[$pFrame85>>2] = $5846;
          $5847 = HEAP32[$pFrame85>>2]|0;
          $5848 = ($5847|0)!=(0|0);
          if (!($5848)) {
           label = 860;
           break L1;
          }
          $5849 = HEAP32[$pRt>>2]|0;
          _sqlite3VdbeMemRelease($5849);
          $5850 = HEAP32[$pRt>>2]|0;
          $5851 = ((($5850)) + 8|0);
          HEAP16[$5851>>1] = 64;
          $5852 = HEAP32[$pFrame85>>2]|0;
          $5853 = HEAP32[$pRt>>2]|0;
          HEAP32[$5853>>2] = $5852;
          $5854 = HEAP32[$0>>2]|0;
          $5855 = HEAP32[$pFrame85>>2]|0;
          HEAP32[$5855>>2] = $5854;
          $5856 = HEAP32[$nMem>>2]|0;
          $5857 = HEAP32[$pFrame85>>2]|0;
          $5858 = ((($5857)) + 60|0);
          HEAP32[$5858>>2] = $5856;
          $5859 = HEAP32[$pProgram>>2]|0;
          $5860 = ((($5859)) + 12|0);
          $5861 = HEAP32[$5860>>2]|0;
          $5862 = HEAP32[$pFrame85>>2]|0;
          $5863 = ((($5862)) + 64|0);
          HEAP32[$5863>>2] = $5861;
          $5864 = HEAP32[$pOp>>2]|0;
          $5865 = HEAP32[$aOp>>2]|0;
          $5866 = $5864;
          $5867 = $5865;
          $5868 = (($5866) - ($5867))|0;
          $5869 = (($5868|0) / 20)&-1;
          $5870 = HEAP32[$pFrame85>>2]|0;
          $5871 = ((($5870)) + 44|0);
          HEAP32[$5871>>2] = $5869;
          $5872 = HEAP32[$0>>2]|0;
          $5873 = ((($5872)) + 8|0);
          $5874 = HEAP32[$5873>>2]|0;
          $5875 = HEAP32[$pFrame85>>2]|0;
          $5876 = ((($5875)) + 16|0);
          HEAP32[$5876>>2] = $5874;
          $5877 = HEAP32[$0>>2]|0;
          $5878 = ((($5877)) + 28|0);
          $5879 = HEAP32[$5878>>2]|0;
          $5880 = HEAP32[$pFrame85>>2]|0;
          $5881 = ((($5880)) + 52|0);
          HEAP32[$5881>>2] = $5879;
          $5882 = HEAP32[$0>>2]|0;
          $5883 = ((($5882)) + 56|0);
          $5884 = HEAP32[$5883>>2]|0;
          $5885 = HEAP32[$pFrame85>>2]|0;
          $5886 = ((($5885)) + 24|0);
          HEAP32[$5886>>2] = $5884;
          $5887 = HEAP32[$0>>2]|0;
          $5888 = ((($5887)) + 36|0);
          $5889 = HEAP32[$5888>>2]|0;
          $5890 = HEAP32[$pFrame85>>2]|0;
          $5891 = ((($5890)) + 40|0);
          HEAP32[$5891>>2] = $5889;
          $5892 = HEAP32[$0>>2]|0;
          $5893 = ((($5892)) + 4|0);
          $5894 = HEAP32[$5893>>2]|0;
          $5895 = HEAP32[$pFrame85>>2]|0;
          $5896 = ((($5895)) + 8|0);
          HEAP32[$5896>>2] = $5894;
          $5897 = HEAP32[$0>>2]|0;
          $5898 = ((($5897)) + 32|0);
          $5899 = HEAP32[$5898>>2]|0;
          $5900 = HEAP32[$pFrame85>>2]|0;
          $5901 = ((($5900)) + 48|0);
          HEAP32[$5901>>2] = $5899;
          $5902 = HEAP32[$pProgram>>2]|0;
          $5903 = ((($5902)) + 20|0);
          $5904 = HEAP32[$5903>>2]|0;
          $5905 = HEAP32[$pFrame85>>2]|0;
          $5906 = ((($5905)) + 28|0);
          HEAP32[$5906>>2] = $5904;
          $5907 = HEAP32[$0>>2]|0;
          $5908 = ((($5907)) + 200|0);
          $5909 = HEAP32[$5908>>2]|0;
          $5910 = HEAP32[$pFrame85>>2]|0;
          $5911 = ((($5910)) + 20|0);
          HEAP32[$5911>>2] = $5909;
          $5912 = HEAP32[$0>>2]|0;
          $5913 = ((($5912)) + 196|0);
          $5914 = HEAP32[$5913>>2]|0;
          $5915 = HEAP32[$pFrame85>>2]|0;
          $5916 = ((($5915)) + 56|0);
          HEAP32[$5916>>2] = $5914;
          $5917 = HEAP32[$pFrame85>>2]|0;
          $5918 = ((($5917)) + 60|0);
          $5919 = HEAP32[$5918>>2]|0;
          $5920 = HEAP32[$pFrame85>>2]|0;
          $5921 = ((($5920)) + 80|0);
          $5922 = (($5921) + (($5919*40)|0)|0);
          HEAP32[$pEnd>>2] = $5922;
          $5923 = HEAP32[$pFrame85>>2]|0;
          $5924 = ((($5923)) + 80|0);
          HEAP32[$pMem84>>2] = $5924;
          while(1) {
           $5925 = HEAP32[$pMem84>>2]|0;
           $5926 = HEAP32[$pEnd>>2]|0;
           $5927 = ($5925|0)!=($5926|0);
           if (!($5927)) {
            break L800;
           }
           $5928 = HEAP32[$pMem84>>2]|0;
           $5929 = ((($5928)) + 8|0);
           HEAP16[$5929>>1] = 128;
           $5930 = HEAP32[$db>>2]|0;
           $5931 = HEAP32[$pMem84>>2]|0;
           $5932 = ((($5931)) + 32|0);
           HEAP32[$5932>>2] = $5930;
           $5933 = HEAP32[$pMem84>>2]|0;
           $5934 = ((($5933)) + 40|0);
           HEAP32[$pMem84>>2] = $5934;
          }
         } else {
          $5935 = HEAP32[$pRt>>2]|0;
          $5936 = HEAP32[$5935>>2]|0;
          HEAP32[$pFrame85>>2] = $5936;
         }
        } while(0);
        $5937 = HEAP32[$0>>2]|0;
        $5938 = ((($5937)) + 184|0);
        $5939 = HEAP32[$5938>>2]|0;
        $5940 = (($5939) + 1)|0;
        HEAP32[$5938>>2] = $5940;
        $5941 = HEAP32[$0>>2]|0;
        $5942 = ((($5941)) + 176|0);
        $5943 = HEAP32[$5942>>2]|0;
        $5944 = HEAP32[$pFrame85>>2]|0;
        $5945 = ((($5944)) + 4|0);
        HEAP32[$5945>>2] = $5943;
        $5946 = $lastRowid;
        $5947 = $5946;
        $5948 = HEAP32[$5947>>2]|0;
        $5949 = (($5946) + 4)|0;
        $5950 = $5949;
        $5951 = HEAP32[$5950>>2]|0;
        $5952 = HEAP32[$pFrame85>>2]|0;
        $5953 = ((($5952)) + 32|0);
        $5954 = $5953;
        $5955 = $5954;
        HEAP32[$5955>>2] = $5948;
        $5956 = (($5954) + 4)|0;
        $5957 = $5956;
        HEAP32[$5957>>2] = $5951;
        $5958 = HEAP32[$0>>2]|0;
        $5959 = ((($5958)) + 92|0);
        $5960 = HEAP32[$5959>>2]|0;
        $5961 = HEAP32[$pFrame85>>2]|0;
        $5962 = ((($5961)) + 68|0);
        HEAP32[$5962>>2] = $5960;
        $5963 = HEAP32[$0>>2]|0;
        $5964 = HEAP32[$5963>>2]|0;
        $5965 = ((($5964)) + 80|0);
        $5966 = HEAP32[$5965>>2]|0;
        $5967 = HEAP32[$pFrame85>>2]|0;
        $5968 = ((($5967)) + 72|0);
        HEAP32[$5968>>2] = $5966;
        $5969 = HEAP32[$0>>2]|0;
        $5970 = ((($5969)) + 92|0);
        HEAP32[$5970>>2] = 0;
        $5971 = HEAP32[$pFrame85>>2]|0;
        $5972 = HEAP32[$0>>2]|0;
        $5973 = ((($5972)) + 176|0);
        HEAP32[$5973>>2] = $5971;
        $5974 = HEAP32[$pFrame85>>2]|0;
        $5975 = ((($5974)) + 80|0);
        $5976 = ((($5975)) + -40|0);
        HEAP32[$aMem>>2] = $5976;
        $5977 = HEAP32[$0>>2]|0;
        $5978 = ((($5977)) + 8|0);
        HEAP32[$5978>>2] = $5976;
        $5979 = HEAP32[$pFrame85>>2]|0;
        $5980 = ((($5979)) + 60|0);
        $5981 = HEAP32[$5980>>2]|0;
        $5982 = HEAP32[$0>>2]|0;
        $5983 = ((($5982)) + 28|0);
        HEAP32[$5983>>2] = $5981;
        $5984 = HEAP32[$pFrame85>>2]|0;
        $5985 = ((($5984)) + 64|0);
        $5986 = HEAP32[$5985>>2]|0;
        $5987 = $5986&65535;
        $5988 = $5987&65535;
        $5989 = HEAP32[$0>>2]|0;
        $5990 = ((($5989)) + 36|0);
        HEAP32[$5990>>2] = $5988;
        $5991 = HEAP32[$0>>2]|0;
        $5992 = ((($5991)) + 28|0);
        $5993 = HEAP32[$5992>>2]|0;
        $5994 = (($5993) + 1)|0;
        $5995 = HEAP32[$aMem>>2]|0;
        $5996 = (($5995) + (($5994*40)|0)|0);
        $5997 = HEAP32[$0>>2]|0;
        $5998 = ((($5997)) + 56|0);
        HEAP32[$5998>>2] = $5996;
        $5999 = HEAP32[$pProgram>>2]|0;
        $6000 = HEAP32[$5999>>2]|0;
        HEAP32[$aOp>>2] = $6000;
        $6001 = HEAP32[$0>>2]|0;
        $6002 = ((($6001)) + 4|0);
        HEAP32[$6002>>2] = $6000;
        $6003 = HEAP32[$pProgram>>2]|0;
        $6004 = ((($6003)) + 4|0);
        $6005 = HEAP32[$6004>>2]|0;
        $6006 = HEAP32[$0>>2]|0;
        $6007 = ((($6006)) + 32|0);
        HEAP32[$6007>>2] = $6005;
        $6008 = HEAP32[$0>>2]|0;
        $6009 = ((($6008)) + 36|0);
        $6010 = HEAP32[$6009>>2]|0;
        $6011 = HEAP32[$0>>2]|0;
        $6012 = ((($6011)) + 56|0);
        $6013 = HEAP32[$6012>>2]|0;
        $6014 = (($6013) + ($6010<<2)|0);
        $6015 = HEAP32[$0>>2]|0;
        $6016 = ((($6015)) + 200|0);
        HEAP32[$6016>>2] = $6014;
        $6017 = HEAP32[$pProgram>>2]|0;
        $6018 = ((($6017)) + 16|0);
        $6019 = HEAP32[$6018>>2]|0;
        $6020 = HEAP32[$0>>2]|0;
        $6021 = ((($6020)) + 196|0);
        HEAP32[$6021>>2] = $6019;
        $6022 = HEAP32[$aOp>>2]|0;
        $6023 = ((($6022)) + -20|0);
        HEAP32[$pOp>>2] = $6023;
        $6024 = HEAP32[$0>>2]|0;
        $6025 = ((($6024)) + 200|0);
        $6026 = HEAP32[$6025>>2]|0;
        $6027 = HEAP32[$0>>2]|0;
        $6028 = ((($6027)) + 196|0);
        $6029 = HEAP32[$6028>>2]|0;
        _memset(($6026|0),0,($6029|0))|0;
        break;
       }
       case 134:  {
        $6030 = HEAP32[$0>>2]|0;
        $6031 = HEAP32[$pOp>>2]|0;
        $6032 = (_out2Prerelease($6030,$6031)|0);
        HEAP32[$pOut>>2] = $6032;
        $6033 = HEAP32[$0>>2]|0;
        $6034 = ((($6033)) + 176|0);
        $6035 = HEAP32[$6034>>2]|0;
        HEAP32[$pFrame87>>2] = $6035;
        $6036 = HEAP32[$pOp>>2]|0;
        $6037 = ((($6036)) + 4|0);
        $6038 = HEAP32[$6037>>2]|0;
        $6039 = HEAP32[$pFrame87>>2]|0;
        $6040 = ((($6039)) + 44|0);
        $6041 = HEAP32[$6040>>2]|0;
        $6042 = HEAP32[$pFrame87>>2]|0;
        $6043 = ((($6042)) + 8|0);
        $6044 = HEAP32[$6043>>2]|0;
        $6045 = (($6044) + (($6041*20)|0)|0);
        $6046 = ((($6045)) + 4|0);
        $6047 = HEAP32[$6046>>2]|0;
        $6048 = (($6038) + ($6047))|0;
        $6049 = HEAP32[$pFrame87>>2]|0;
        $6050 = ((($6049)) + 16|0);
        $6051 = HEAP32[$6050>>2]|0;
        $6052 = (($6051) + (($6048*40)|0)|0);
        HEAP32[$pIn>>2] = $6052;
        $6053 = HEAP32[$pOut>>2]|0;
        $6054 = HEAP32[$pIn>>2]|0;
        _sqlite3VdbeMemShallowCopy($6053,$6054,4096);
        break;
       }
       case 135:  {
        $6055 = HEAP32[$db>>2]|0;
        $6056 = ((($6055)) + 24|0);
        $6057 = HEAP32[$6056>>2]|0;
        $6058 = $6057 & 16777216;
        $6059 = ($6058|0)!=(0);
        $6060 = HEAP32[$pOp>>2]|0;
        if ($6059) {
         $6061 = ((($6060)) + 8|0);
         $6062 = HEAP32[$6061>>2]|0;
         $6063 = ($6062|0)<(0);
         $6064 = $6063 << 31 >> 31;
         $6065 = HEAP32[$db>>2]|0;
         $6066 = ((($6065)) + 520|0);
         $6067 = $6066;
         $6068 = $6067;
         $6069 = HEAP32[$6068>>2]|0;
         $6070 = (($6067) + 4)|0;
         $6071 = $6070;
         $6072 = HEAP32[$6071>>2]|0;
         $6073 = (_i64Add(($6069|0),($6072|0),($6062|0),($6064|0))|0);
         $6074 = tempRet0;
         $6075 = $6066;
         $6076 = $6075;
         HEAP32[$6076>>2] = $6073;
         $6077 = (($6075) + 4)|0;
         $6078 = $6077;
         HEAP32[$6078>>2] = $6074;
         break L11;
        }
        $6079 = ((($6060)) + 4|0);
        $6080 = HEAP32[$6079>>2]|0;
        $6081 = ($6080|0)!=(0);
        $6082 = HEAP32[$pOp>>2]|0;
        $6083 = ((($6082)) + 8|0);
        $6084 = HEAP32[$6083>>2]|0;
        $6085 = ($6084|0)<(0);
        $6086 = $6085 << 31 >> 31;
        if ($6081) {
         $6087 = HEAP32[$db>>2]|0;
         $6088 = ((($6087)) + 512|0);
         $6089 = $6088;
         $6090 = $6089;
         $6091 = HEAP32[$6090>>2]|0;
         $6092 = (($6089) + 4)|0;
         $6093 = $6092;
         $6094 = HEAP32[$6093>>2]|0;
         $6095 = (_i64Add(($6091|0),($6094|0),($6084|0),($6086|0))|0);
         $6096 = tempRet0;
         $6097 = $6088;
         $6098 = $6097;
         HEAP32[$6098>>2] = $6095;
         $6099 = (($6097) + 4)|0;
         $6100 = $6099;
         HEAP32[$6100>>2] = $6096;
         break L11;
        } else {
         $6101 = HEAP32[$0>>2]|0;
         $6102 = ((($6101)) + 144|0);
         $6103 = $6102;
         $6104 = $6103;
         $6105 = HEAP32[$6104>>2]|0;
         $6106 = (($6103) + 4)|0;
         $6107 = $6106;
         $6108 = HEAP32[$6107>>2]|0;
         $6109 = (_i64Add(($6105|0),($6108|0),($6084|0),($6086|0))|0);
         $6110 = tempRet0;
         $6111 = $6102;
         $6112 = $6111;
         HEAP32[$6112>>2] = $6109;
         $6113 = (($6111) + 4)|0;
         $6114 = $6113;
         HEAP32[$6114>>2] = $6110;
         break L11;
        }
        break;
       }
       case 136:  {
        $6115 = HEAP32[$pOp>>2]|0;
        $6116 = ((($6115)) + 4|0);
        $6117 = HEAP32[$6116>>2]|0;
        $6118 = ($6117|0)!=(0);
        if ($6118) {
         $6119 = HEAP32[$db>>2]|0;
         $6120 = ((($6119)) + 512|0);
         $6121 = $6120;
         $6122 = $6121;
         $6123 = HEAP32[$6122>>2]|0;
         $6124 = (($6121) + 4)|0;
         $6125 = $6124;
         $6126 = HEAP32[$6125>>2]|0;
         $6127 = ($6123|0)==(0);
         $6128 = ($6126|0)==(0);
         $6129 = $6127 & $6128;
         if (!($6129)) {
          break L11;
         }
         $6130 = HEAP32[$db>>2]|0;
         $6131 = ((($6130)) + 520|0);
         $6132 = $6131;
         $6133 = $6132;
         $6134 = HEAP32[$6133>>2]|0;
         $6135 = (($6132) + 4)|0;
         $6136 = $6135;
         $6137 = HEAP32[$6136>>2]|0;
         $6138 = ($6134|0)==(0);
         $6139 = ($6137|0)==(0);
         $6140 = $6138 & $6139;
         if ($6140) {
          label = 15;
          break L11;
         } else {
          break L11;
         }
        } else {
         $6141 = HEAP32[$0>>2]|0;
         $6142 = ((($6141)) + 144|0);
         $6143 = $6142;
         $6144 = $6143;
         $6145 = HEAP32[$6144>>2]|0;
         $6146 = (($6143) + 4)|0;
         $6147 = $6146;
         $6148 = HEAP32[$6147>>2]|0;
         $6149 = ($6145|0)==(0);
         $6150 = ($6148|0)==(0);
         $6151 = $6149 & $6150;
         if (!($6151)) {
          break L11;
         }
         $6152 = HEAP32[$db>>2]|0;
         $6153 = ((($6152)) + 520|0);
         $6154 = $6153;
         $6155 = $6154;
         $6156 = HEAP32[$6155>>2]|0;
         $6157 = (($6154) + 4)|0;
         $6158 = $6157;
         $6159 = HEAP32[$6158>>2]|0;
         $6160 = ($6156|0)==(0);
         $6161 = ($6159|0)==(0);
         $6162 = $6160 & $6161;
         if ($6162) {
          label = 15;
          break L11;
         } else {
          break L11;
         }
        }
        break;
       }
       case 137:  {
        $6163 = HEAP32[$0>>2]|0;
        $6164 = ((($6163)) + 176|0);
        $6165 = HEAP32[$6164>>2]|0;
        $6166 = ($6165|0)!=(0|0);
        if ($6166) {
         $6167 = HEAP32[$0>>2]|0;
         $6168 = ((($6167)) + 176|0);
         $6169 = HEAP32[$6168>>2]|0;
         HEAP32[$pFrame88>>2] = $6169;
         while(1) {
          $6170 = HEAP32[$pFrame88>>2]|0;
          $6171 = ((($6170)) + 4|0);
          $6172 = HEAP32[$6171>>2]|0;
          $6173 = ($6172|0)!=(0|0);
          if (!($6173)) {
           break;
          }
          $6174 = HEAP32[$pFrame88>>2]|0;
          $6175 = ((($6174)) + 4|0);
          $6176 = HEAP32[$6175>>2]|0;
          HEAP32[$pFrame88>>2] = $6176;
         }
         $6177 = HEAP32[$pOp>>2]|0;
         $6178 = ((($6177)) + 4|0);
         $6179 = HEAP32[$6178>>2]|0;
         $6180 = HEAP32[$pFrame88>>2]|0;
         $6181 = ((($6180)) + 16|0);
         $6182 = HEAP32[$6181>>2]|0;
         $6183 = (($6182) + (($6179*40)|0)|0);
         HEAP32[$pIn1>>2] = $6183;
        } else {
         $6184 = HEAP32[$pOp>>2]|0;
         $6185 = ((($6184)) + 4|0);
         $6186 = HEAP32[$6185>>2]|0;
         $6187 = HEAP32[$aMem>>2]|0;
         $6188 = (($6187) + (($6186*40)|0)|0);
         HEAP32[$pIn1>>2] = $6188;
        }
        $6189 = HEAP32[$pIn1>>2]|0;
        (_sqlite3VdbeMemIntegerify($6189)|0);
        $6190 = HEAP32[$pOp>>2]|0;
        $6191 = ((($6190)) + 8|0);
        $6192 = HEAP32[$6191>>2]|0;
        $6193 = HEAP32[$aMem>>2]|0;
        $6194 = (($6193) + (($6192*40)|0)|0);
        HEAP32[$pIn2>>2] = $6194;
        $6195 = HEAP32[$pIn2>>2]|0;
        (_sqlite3VdbeMemIntegerify($6195)|0);
        $6196 = HEAP32[$pIn1>>2]|0;
        $6197 = $6196;
        $6198 = $6197;
        $6199 = HEAP32[$6198>>2]|0;
        $6200 = (($6197) + 4)|0;
        $6201 = $6200;
        $6202 = HEAP32[$6201>>2]|0;
        $6203 = HEAP32[$pIn2>>2]|0;
        $6204 = $6203;
        $6205 = $6204;
        $6206 = HEAP32[$6205>>2]|0;
        $6207 = (($6204) + 4)|0;
        $6208 = $6207;
        $6209 = HEAP32[$6208>>2]|0;
        $6210 = ($6202|0)<($6209|0);
        $6211 = ($6199>>>0)<($6206>>>0);
        $6212 = ($6202|0)==($6209|0);
        $6213 = $6212 & $6211;
        $6214 = $6210 | $6213;
        if ($6214) {
         $6215 = HEAP32[$pIn2>>2]|0;
         $6216 = $6215;
         $6217 = $6216;
         $6218 = HEAP32[$6217>>2]|0;
         $6219 = (($6216) + 4)|0;
         $6220 = $6219;
         $6221 = HEAP32[$6220>>2]|0;
         $6222 = HEAP32[$pIn1>>2]|0;
         $6223 = $6222;
         $6224 = $6223;
         HEAP32[$6224>>2] = $6218;
         $6225 = (($6223) + 4)|0;
         $6226 = $6225;
         HEAP32[$6226>>2] = $6221;
        }
        break;
       }
       case 138:  {
        $6227 = HEAP32[$pOp>>2]|0;
        $6228 = ((($6227)) + 4|0);
        $6229 = HEAP32[$6228>>2]|0;
        $6230 = HEAP32[$aMem>>2]|0;
        $6231 = (($6230) + (($6229*40)|0)|0);
        HEAP32[$pIn1>>2] = $6231;
        $6232 = HEAP32[$pIn1>>2]|0;
        $6233 = $6232;
        $6234 = $6233;
        $6235 = HEAP32[$6234>>2]|0;
        $6236 = (($6233) + 4)|0;
        $6237 = $6236;
        $6238 = HEAP32[$6237>>2]|0;
        $6239 = ($6238|0)>(0);
        $6240 = ($6235>>>0)>(0);
        $6241 = ($6238|0)==(0);
        $6242 = $6241 & $6240;
        $6243 = $6239 | $6242;
        if ($6243) {
         $6244 = HEAP32[$pOp>>2]|0;
         $6245 = ((($6244)) + 12|0);
         $6246 = HEAP32[$6245>>2]|0;
         $6247 = ($6246|0)<(0);
         $6248 = $6247 << 31 >> 31;
         $6249 = HEAP32[$pIn1>>2]|0;
         $6250 = $6249;
         $6251 = $6250;
         $6252 = HEAP32[$6251>>2]|0;
         $6253 = (($6250) + 4)|0;
         $6254 = $6253;
         $6255 = HEAP32[$6254>>2]|0;
         $6256 = (_i64Subtract(($6252|0),($6255|0),($6246|0),($6248|0))|0);
         $6257 = tempRet0;
         $6258 = $6249;
         $6259 = $6258;
         HEAP32[$6259>>2] = $6256;
         $6260 = (($6258) + 4)|0;
         $6261 = $6260;
         HEAP32[$6261>>2] = $6257;
         label = 15;
        }
        break;
       }
       case 139:  {
        $6262 = HEAP32[$pOp>>2]|0;
        $6263 = ((($6262)) + 4|0);
        $6264 = HEAP32[$6263>>2]|0;
        $6265 = HEAP32[$aMem>>2]|0;
        $6266 = (($6265) + (($6264*40)|0)|0);
        HEAP32[$pIn1>>2] = $6266;
        $6267 = HEAP32[$pOp>>2]|0;
        $6268 = ((($6267)) + 12|0);
        $6269 = HEAP32[$6268>>2]|0;
        $6270 = HEAP32[$aMem>>2]|0;
        $6271 = (($6270) + (($6269*40)|0)|0);
        HEAP32[$pIn3>>2] = $6271;
        $6272 = HEAP32[$0>>2]|0;
        $6273 = HEAP32[$pOp>>2]|0;
        $6274 = (_out2Prerelease($6272,$6273)|0);
        HEAP32[$pOut>>2] = $6274;
        $6275 = HEAP32[$pIn1>>2]|0;
        $6276 = $6275;
        $6277 = $6276;
        $6278 = HEAP32[$6277>>2]|0;
        $6279 = (($6276) + 4)|0;
        $6280 = $6279;
        $6281 = HEAP32[$6280>>2]|0;
        $6282 = ($6281|0)<(0);
        $6283 = ($6278>>>0)<=(0);
        $6284 = ($6281|0)==(0);
        $6285 = $6284 & $6283;
        $6286 = $6282 | $6285;
        if ($6286) {
         $6320 = -1;$6323 = -1;
        } else {
         $6287 = HEAP32[$pIn1>>2]|0;
         $6288 = $6287;
         $6289 = $6288;
         $6290 = HEAP32[$6289>>2]|0;
         $6291 = (($6288) + 4)|0;
         $6292 = $6291;
         $6293 = HEAP32[$6292>>2]|0;
         $6294 = HEAP32[$pIn3>>2]|0;
         $6295 = $6294;
         $6296 = $6295;
         $6297 = HEAP32[$6296>>2]|0;
         $6298 = (($6295) + 4)|0;
         $6299 = $6298;
         $6300 = HEAP32[$6299>>2]|0;
         $6301 = ($6300|0)>(0);
         $6302 = ($6297>>>0)>(0);
         $6303 = ($6300|0)==(0);
         $6304 = $6303 & $6302;
         $6305 = $6301 | $6304;
         if ($6305) {
          $6306 = HEAP32[$pIn3>>2]|0;
          $6307 = $6306;
          $6308 = $6307;
          $6309 = HEAP32[$6308>>2]|0;
          $6310 = (($6307) + 4)|0;
          $6311 = $6310;
          $6312 = HEAP32[$6311>>2]|0;
          $6313 = $6309;$6314 = $6312;
         } else {
          $6313 = 0;$6314 = 0;
         }
         $6315 = (_i64Add(($6290|0),($6293|0),($6313|0),($6314|0))|0);
         $6316 = tempRet0;
         $6320 = $6315;$6323 = $6316;
        }
        $6317 = HEAP32[$pOut>>2]|0;
        $6318 = $6317;
        $6319 = $6318;
        HEAP32[$6319>>2] = $6320;
        $6321 = (($6318) + 4)|0;
        $6322 = $6321;
        HEAP32[$6322>>2] = $6323;
        break;
       }
       case 140:  {
        $6324 = HEAP32[$pOp>>2]|0;
        $6325 = ((($6324)) + 4|0);
        $6326 = HEAP32[$6325>>2]|0;
        $6327 = HEAP32[$aMem>>2]|0;
        $6328 = (($6327) + (($6326*40)|0)|0);
        HEAP32[$pIn1>>2] = $6328;
        $6329 = HEAP32[$pIn1>>2]|0;
        $6330 = $6329;
        $6331 = $6330;
        $6332 = HEAP32[$6331>>2]|0;
        $6333 = (($6330) + 4)|0;
        $6334 = $6333;
        $6335 = HEAP32[$6334>>2]|0;
        $6336 = ($6332|0)!=(0);
        $6337 = ($6335|0)!=(0);
        $6338 = $6336 | $6337;
        if ($6338) {
         $6339 = HEAP32[$pOp>>2]|0;
         $6340 = ((($6339)) + 12|0);
         $6341 = HEAP32[$6340>>2]|0;
         $6342 = ($6341|0)<(0);
         $6343 = $6342 << 31 >> 31;
         $6344 = HEAP32[$pIn1>>2]|0;
         $6345 = $6344;
         $6346 = $6345;
         $6347 = HEAP32[$6346>>2]|0;
         $6348 = (($6345) + 4)|0;
         $6349 = $6348;
         $6350 = HEAP32[$6349>>2]|0;
         $6351 = (_i64Subtract(($6347|0),($6350|0),($6341|0),($6343|0))|0);
         $6352 = tempRet0;
         $6353 = $6344;
         $6354 = $6353;
         HEAP32[$6354>>2] = $6351;
         $6355 = (($6353) + 4)|0;
         $6356 = $6355;
         HEAP32[$6356>>2] = $6352;
         label = 15;
        }
        break;
       }
       case 141:  {
        $6357 = HEAP32[$pOp>>2]|0;
        $6358 = ((($6357)) + 4|0);
        $6359 = HEAP32[$6358>>2]|0;
        $6360 = HEAP32[$aMem>>2]|0;
        $6361 = (($6360) + (($6359*40)|0)|0);
        HEAP32[$pIn1>>2] = $6361;
        $6362 = HEAP32[$pIn1>>2]|0;
        $6363 = $6362;
        $6364 = $6363;
        $6365 = HEAP32[$6364>>2]|0;
        $6366 = (($6363) + 4)|0;
        $6367 = $6366;
        $6368 = HEAP32[$6367>>2]|0;
        $6369 = (_i64Add(($6365|0),($6368|0),-1,-1)|0);
        $6370 = tempRet0;
        $6371 = $6362;
        $6372 = $6371;
        HEAP32[$6372>>2] = $6369;
        $6373 = (($6371) + 4)|0;
        $6374 = $6373;
        HEAP32[$6374>>2] = $6370;
        $6375 = HEAP32[$pIn1>>2]|0;
        $6376 = $6375;
        $6377 = $6376;
        $6378 = HEAP32[$6377>>2]|0;
        $6379 = (($6376) + 4)|0;
        $6380 = $6379;
        $6381 = HEAP32[$6380>>2]|0;
        $6382 = ($6378|0)==(0);
        $6383 = ($6381|0)==(0);
        $6384 = $6382 & $6383;
        if ($6384) {
         label = 15;
        }
        break;
       }
       case 142:  {
        $6385 = HEAP32[$pOp>>2]|0;
        $6386 = ((($6385)) + 4|0);
        $6387 = HEAP32[$6386>>2]|0;
        $6388 = HEAP32[$aMem>>2]|0;
        $6389 = (($6388) + (($6387*40)|0)|0);
        HEAP32[$pIn1>>2] = $6389;
        $6390 = HEAP32[$pIn1>>2]|0;
        $6391 = $6390;
        $6392 = $6391;
        $6393 = HEAP32[$6392>>2]|0;
        $6394 = (($6391) + 4)|0;
        $6395 = $6394;
        $6396 = HEAP32[$6395>>2]|0;
        $6397 = (_i64Add(($6393|0),($6396|0),1,0)|0);
        $6398 = tempRet0;
        $6399 = $6390;
        $6400 = $6399;
        HEAP32[$6400>>2] = $6397;
        $6401 = (($6399) + 4)|0;
        $6402 = $6401;
        HEAP32[$6402>>2] = $6398;
        $6403 = ($6393|0)==(0);
        $6404 = ($6396|0)==(0);
        $6405 = $6403 & $6404;
        if ($6405) {
         label = 15;
        }
        break;
       }
       case 143:  {
        $6406 = HEAP32[$pOp>>2]|0;
        $6407 = ((($6406)) + 3|0);
        $6408 = HEAP8[$6407>>0]|0;
        $6409 = $6408&255;
        HEAP32[$n89>>2] = $6409;
        $6410 = HEAP32[$db>>2]|0;
        $6411 = HEAP32[$n89>>2]|0;
        $6412 = (($6411) - 1)|0;
        $6413 = $6412<<2;
        $6414 = (32 + ($6413))|0;
        $6415 = (_sqlite3DbMallocRawNN($6410,$6414,0)|0);
        HEAP32[$pCtx90>>2] = $6415;
        $6416 = HEAP32[$pCtx90>>2]|0;
        $6417 = ($6416|0)==(0|0);
        if ($6417) {
         label = 860;
         break L1;
        }
        $6418 = HEAP32[$pCtx90>>2]|0;
        $6419 = ((($6418)) + 8|0);
        HEAP32[$6419>>2] = 0;
        $6420 = HEAP32[$pOp>>2]|0;
        $6421 = ((($6420)) + 16|0);
        $6422 = HEAP32[$6421>>2]|0;
        $6423 = HEAP32[$pCtx90>>2]|0;
        $6424 = ((($6423)) + 4|0);
        HEAP32[$6424>>2] = $6422;
        $6425 = HEAP32[$pOp>>2]|0;
        $6426 = HEAP32[$aOp>>2]|0;
        $6427 = $6425;
        $6428 = $6426;
        $6429 = (($6427) - ($6428))|0;
        $6430 = (($6429|0) / 20)&-1;
        $6431 = HEAP32[$pCtx90>>2]|0;
        $6432 = ((($6431)) + 16|0);
        HEAP32[$6432>>2] = $6430;
        $6433 = HEAP32[$0>>2]|0;
        $6434 = HEAP32[$pCtx90>>2]|0;
        $6435 = ((($6434)) + 12|0);
        HEAP32[$6435>>2] = $6433;
        $6436 = HEAP32[$n89>>2]|0;
        $6437 = $6436&255;
        $6438 = HEAP32[$pCtx90>>2]|0;
        $6439 = ((($6438)) + 26|0);
        HEAP8[$6439>>0] = $6437;
        $6440 = HEAP32[$pOp>>2]|0;
        $6441 = ((($6440)) + 1|0);
        HEAP8[$6441>>0] = -20;
        $6442 = HEAP32[$pCtx90>>2]|0;
        $6443 = HEAP32[$pOp>>2]|0;
        $6444 = ((($6443)) + 16|0);
        HEAP32[$6444>>2] = $6442;
        $6445 = HEAP32[$pOp>>2]|0;
        HEAP8[$6445>>0] = -112;
        label = 740;
        break;
       }
       case 144:  {
        label = 740;
        break;
       }
       case 145:  {
        $6530 = HEAP32[$pOp>>2]|0;
        $6531 = ((($6530)) + 4|0);
        $6532 = HEAP32[$6531>>2]|0;
        $6533 = HEAP32[$aMem>>2]|0;
        $6534 = (($6533) + (($6532*40)|0)|0);
        HEAP32[$pMem95>>2] = $6534;
        $6535 = HEAP32[$pMem95>>2]|0;
        $6536 = HEAP32[$pOp>>2]|0;
        $6537 = ((($6536)) + 16|0);
        $6538 = HEAP32[$6537>>2]|0;
        $6539 = (_sqlite3VdbeMemFinalize($6535,$6538)|0);
        HEAP32[$rc>>2] = $6539;
        $6540 = HEAP32[$rc>>2]|0;
        $6541 = ($6540|0)!=(0);
        if ($6541) {
         $6542 = HEAP32[$0>>2]|0;
         $6543 = HEAP32[$pMem95>>2]|0;
         $6544 = (_sqlite3_value_text($6543)|0);
         HEAP32[$vararg_buffer67>>2] = $6544;
         _sqlite3VdbeError($6542,17521,$vararg_buffer67);
        }
        $6545 = HEAP32[$pMem95>>2]|0;
        $6546 = HEAP8[$encoding>>0]|0;
        $6547 = $6546&255;
        (_sqlite3VdbeChangeEncoding($6545,$6547)|0);
        $6548 = HEAP32[$pMem95>>2]|0;
        $6549 = (_sqlite3VdbeMemTooBig($6548)|0);
        $6550 = ($6549|0)!=(0);
        if ($6550) {
         label = 859;
         break L8;
        }
        break;
       }
       case 8:  {
        HEAP32[$aRes>>2] = 0;
        $6551 = ((($aRes)) + 8|0);
        HEAP32[$6551>>2] = -1;
        $6552 = ((($aRes)) + 4|0);
        HEAP32[$6552>>2] = -1;
        $6553 = HEAP32[$db>>2]|0;
        $6554 = HEAP32[$pOp>>2]|0;
        $6555 = ((($6554)) + 4|0);
        $6556 = HEAP32[$6555>>2]|0;
        $6557 = HEAP32[$pOp>>2]|0;
        $6558 = ((($6557)) + 8|0);
        $6559 = HEAP32[$6558>>2]|0;
        $6560 = ((($aRes)) + 4|0);
        $6561 = ((($aRes)) + 8|0);
        $6562 = (_sqlite3Checkpoint($6553,$6556,$6559,$6560,$6561)|0);
        HEAP32[$rc>>2] = $6562;
        $6563 = HEAP32[$rc>>2]|0;
        $6564 = ($6563|0)==(5);
        if ($6564) {
         HEAP32[$rc>>2] = 0;
         HEAP32[$aRes>>2] = 1;
        }
        HEAP32[$i96>>2] = 0;
        $6565 = HEAP32[$pOp>>2]|0;
        $6566 = ((($6565)) + 12|0);
        $6567 = HEAP32[$6566>>2]|0;
        $6568 = HEAP32[$aMem>>2]|0;
        $6569 = (($6568) + (($6567*40)|0)|0);
        HEAP32[$pMem97>>2] = $6569;
        while(1) {
         $6570 = HEAP32[$i96>>2]|0;
         $6571 = ($6570|0)<(3);
         if (!($6571)) {
          break L11;
         }
         $6572 = HEAP32[$pMem97>>2]|0;
         $6573 = HEAP32[$i96>>2]|0;
         $6574 = (($aRes) + ($6573<<2)|0);
         $6575 = HEAP32[$6574>>2]|0;
         $6576 = ($6575|0)<(0);
         $6577 = $6576 << 31 >> 31;
         _sqlite3VdbeMemSetInt64($6572,$6575,$6577);
         $6578 = HEAP32[$i96>>2]|0;
         $6579 = (($6578) + 1)|0;
         HEAP32[$i96>>2] = $6579;
         $6580 = HEAP32[$pMem97>>2]|0;
         $6581 = ((($6580)) + 40|0);
         HEAP32[$pMem97>>2] = $6581;
        }
        break;
       }
       case 9:  {
        $6582 = HEAP32[$0>>2]|0;
        $6583 = HEAP32[$pOp>>2]|0;
        $6584 = (_out2Prerelease($6582,$6583)|0);
        HEAP32[$pOut>>2] = $6584;
        $6585 = HEAP32[$pOp>>2]|0;
        $6586 = ((($6585)) + 12|0);
        $6587 = HEAP32[$6586>>2]|0;
        HEAP32[$eNew>>2] = $6587;
        $6588 = HEAP32[$pOp>>2]|0;
        $6589 = ((($6588)) + 4|0);
        $6590 = HEAP32[$6589>>2]|0;
        $6591 = HEAP32[$db>>2]|0;
        $6592 = ((($6591)) + 16|0);
        $6593 = HEAP32[$6592>>2]|0;
        $6594 = (($6593) + ($6590<<4)|0);
        $6595 = ((($6594)) + 4|0);
        $6596 = HEAP32[$6595>>2]|0;
        HEAP32[$pBt98>>2] = $6596;
        $6597 = HEAP32[$pBt98>>2]|0;
        $6598 = (_sqlite3BtreePager($6597)|0);
        HEAP32[$pPager>>2] = $6598;
        $6599 = HEAP32[$pPager>>2]|0;
        $6600 = (_sqlite3PagerGetJournalMode($6599)|0);
        HEAP32[$eOld>>2] = $6600;
        $6601 = HEAP32[$eNew>>2]|0;
        $6602 = ($6601|0)==(-1);
        if ($6602) {
         $6603 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6603;
        }
        $6604 = HEAP32[$pPager>>2]|0;
        $6605 = (_sqlite3PagerOkToChangeJournalMode($6604)|0);
        $6606 = ($6605|0)!=(0);
        if (!($6606)) {
         $6607 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6607;
        }
        $6608 = HEAP32[$pPager>>2]|0;
        $6609 = (_sqlite3PagerFilename($6608,1)|0);
        HEAP32[$zFilename>>2] = $6609;
        $6610 = HEAP32[$eNew>>2]|0;
        $6611 = ($6610|0)==(5);
        do {
         if ($6611) {
          $6612 = HEAP32[$zFilename>>2]|0;
          $6613 = (_sqlite3Strlen30($6612)|0);
          $6614 = ($6613|0)==(0);
          if (!($6614)) {
           $6615 = HEAP32[$pPager>>2]|0;
           $6616 = (_sqlite3PagerWalSupported($6615)|0);
           $6617 = ($6616|0)!=(0);
           if ($6617) {
            break;
           }
          }
          $6618 = HEAP32[$eOld>>2]|0;
          HEAP32[$eNew>>2] = $6618;
         }
        } while(0);
        $6619 = HEAP32[$eNew>>2]|0;
        $6620 = HEAP32[$eOld>>2]|0;
        $6621 = ($6619|0)!=($6620|0);
        do {
         if ($6621) {
          $6622 = HEAP32[$eOld>>2]|0;
          $6623 = ($6622|0)==(5);
          $6624 = HEAP32[$eNew>>2]|0;
          $6625 = ($6624|0)==(5);
          $or$cond28 = $6623 | $6625;
          if ($or$cond28) {
           $6626 = HEAP32[$db>>2]|0;
           $6627 = ((($6626)) + 63|0);
           $6628 = HEAP8[$6627>>0]|0;
           $6629 = ($6628<<24>>24)!=(0);
           if ($6629) {
            $6630 = HEAP32[$db>>2]|0;
            $6631 = ((($6630)) + 152|0);
            $6632 = HEAP32[$6631>>2]|0;
            $6633 = ($6632|0)>(1);
            if (!($6633)) {
             $6638 = HEAP32[$eOld>>2]|0;
             $6639 = ($6638|0)==(5);
             if ($6639) {
              $6640 = HEAP32[$pPager>>2]|0;
              $6641 = (_sqlite3PagerCloseWal($6640)|0);
              HEAP32[$rc>>2] = $6641;
              $6642 = HEAP32[$rc>>2]|0;
              $6643 = ($6642|0)==(0);
              if ($6643) {
               $6644 = HEAP32[$pPager>>2]|0;
               $6645 = HEAP32[$eNew>>2]|0;
               (_sqlite3PagerSetJournalMode($6644,$6645)|0);
              }
             } else {
              $6646 = HEAP32[$eOld>>2]|0;
              $6647 = ($6646|0)==(4);
              if ($6647) {
               $6648 = HEAP32[$pPager>>2]|0;
               (_sqlite3PagerSetJournalMode($6648,2)|0);
              }
             }
             $6649 = HEAP32[$rc>>2]|0;
             $6650 = ($6649|0)==(0);
             if (!($6650)) {
              break;
             }
             $6651 = HEAP32[$pBt98>>2]|0;
             $6652 = HEAP32[$eNew>>2]|0;
             $6653 = ($6652|0)==(5);
             $6654 = $6653 ? 2 : 1;
             $6655 = (_sqlite3BtreeSetVersion($6651,$6654)|0);
             HEAP32[$rc>>2] = $6655;
             break;
            }
           }
           HEAP32[$rc>>2] = 1;
           $6634 = HEAP32[$0>>2]|0;
           $6635 = HEAP32[$eNew>>2]|0;
           $6636 = ($6635|0)==(5);
           $6637 = $6636 ? 21446 : 21451;
           HEAP32[$vararg_buffer70>>2] = $6637;
           _sqlite3VdbeError($6634,21394,$vararg_buffer70);
           break L11;
          }
         }
        } while(0);
        $6656 = HEAP32[$rc>>2]|0;
        $6657 = ($6656|0)!=(0);
        if ($6657) {
         $6658 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6658;
        }
        $6659 = HEAP32[$pPager>>2]|0;
        $6660 = HEAP32[$eNew>>2]|0;
        $6661 = (_sqlite3PagerSetJournalMode($6659,$6660)|0);
        HEAP32[$eNew>>2] = $6661;
        $6662 = HEAP32[$pOut>>2]|0;
        $6663 = ((($6662)) + 8|0);
        HEAP16[$6663>>1] = 2562;
        $6664 = HEAP32[$eNew>>2]|0;
        $6665 = (_sqlite3JournalModename($6664)|0);
        $6666 = HEAP32[$pOut>>2]|0;
        $6667 = ((($6666)) + 16|0);
        HEAP32[$6667>>2] = $6665;
        $6668 = HEAP32[$pOut>>2]|0;
        $6669 = ((($6668)) + 16|0);
        $6670 = HEAP32[$6669>>2]|0;
        $6671 = (_sqlite3Strlen30($6670)|0);
        $6672 = HEAP32[$pOut>>2]|0;
        $6673 = ((($6672)) + 12|0);
        HEAP32[$6673>>2] = $6671;
        $6674 = HEAP32[$pOut>>2]|0;
        $6675 = ((($6674)) + 10|0);
        HEAP8[$6675>>0] = 1;
        $6676 = HEAP32[$pOut>>2]|0;
        $6677 = HEAP8[$encoding>>0]|0;
        $6678 = $6677&255;
        (_sqlite3VdbeChangeEncoding($6676,$6678)|0);
        break;
       }
       case 10:  {
        $6679 = HEAP32[$0>>2]|0;
        $6680 = ((($6679)) + 44|0);
        $6681 = HEAP32[$db>>2]|0;
        $6682 = (_sqlite3RunVacuum($6680,$6681)|0);
        HEAP32[$rc>>2] = $6682;
        break;
       }
       case 146:  {
        $6683 = HEAP32[$pOp>>2]|0;
        $6684 = ((($6683)) + 4|0);
        $6685 = HEAP32[$6684>>2]|0;
        $6686 = HEAP32[$db>>2]|0;
        $6687 = ((($6686)) + 16|0);
        $6688 = HEAP32[$6687>>2]|0;
        $6689 = (($6688) + ($6685<<4)|0);
        $6690 = ((($6689)) + 4|0);
        $6691 = HEAP32[$6690>>2]|0;
        HEAP32[$pBt99>>2] = $6691;
        $6692 = HEAP32[$pBt99>>2]|0;
        $6693 = (_sqlite3BtreeIncrVacuum($6692)|0);
        HEAP32[$rc>>2] = $6693;
        $6694 = HEAP32[$rc>>2]|0;
        $6695 = ($6694|0)==(101);
        if ($6695) {
         HEAP32[$rc>>2] = 0;
         label = 15;
        }
        break;
       }
       case 147:  {
        $6696 = HEAP32[$pOp>>2]|0;
        $6697 = ((($6696)) + 4|0);
        $6698 = HEAP32[$6697>>2]|0;
        $6699 = ($6698|0)!=(0);
        if ($6699) {
         $6701 = HEAP32[$0>>2]|0;
         $6702 = ((($6701)) + 88|0);
         $6703 = HEAP16[$6702>>1]|0;
         $6704 = $6703 & -9;
         $6705 = $6704 | 8;
         HEAP16[$6702>>1] = $6705;
         break L11;
        } else {
         $6700 = HEAP32[$db>>2]|0;
         _sqlite3ExpirePreparedStatements($6700);
         break L11;
        }
        break;
       }
       case 148:  {
        $6706 = HEAP32[$pOp>>2]|0;
        $6707 = ((($6706)) + 12|0);
        $6708 = HEAP32[$6707>>2]|0;
        $6709 = $6708&255;
        HEAP8[$isWriteLock>>0] = $6709;
        $6710 = HEAP8[$isWriteLock>>0]|0;
        $6711 = $6710&255;
        $6712 = ($6711|0)!=(0);
        if (!($6712)) {
         $6713 = HEAP32[$db>>2]|0;
         $6714 = ((($6713)) + 24|0);
         $6715 = HEAP32[$6714>>2]|0;
         $6716 = $6715 & 16384;
         $6717 = (0)==($6716|0);
         if (!($6717)) {
          break L11;
         }
        }
        $6718 = HEAP32[$pOp>>2]|0;
        $6719 = ((($6718)) + 4|0);
        $6720 = HEAP32[$6719>>2]|0;
        HEAP32[$p1100>>2] = $6720;
        $6721 = HEAP32[$p1100>>2]|0;
        $6722 = HEAP32[$db>>2]|0;
        $6723 = ((($6722)) + 16|0);
        $6724 = HEAP32[$6723>>2]|0;
        $6725 = (($6724) + ($6721<<4)|0);
        $6726 = ((($6725)) + 4|0);
        $6727 = HEAP32[$6726>>2]|0;
        $6728 = HEAP32[$pOp>>2]|0;
        $6729 = ((($6728)) + 8|0);
        $6730 = HEAP32[$6729>>2]|0;
        $6731 = HEAP8[$isWriteLock>>0]|0;
        $6732 = (_sqlite3BtreeLockTable($6727,$6730,$6731)|0);
        HEAP32[$rc>>2] = $6732;
        $6733 = HEAP32[$rc>>2]|0;
        $6734 = $6733 & 255;
        $6735 = ($6734|0)==(6);
        if ($6735) {
         $6736 = HEAP32[$pOp>>2]|0;
         $6737 = ((($6736)) + 16|0);
         $6738 = HEAP32[$6737>>2]|0;
         HEAP32[$z101>>2] = $6738;
         $6739 = HEAP32[$0>>2]|0;
         $6740 = HEAP32[$z101>>2]|0;
         HEAP32[$vararg_buffer73>>2] = $6740;
         _sqlite3VdbeError($6739,21458,$vararg_buffer73);
        }
        break;
       }
       case 149:  {
        $6741 = HEAP32[$pOp>>2]|0;
        $6742 = ((($6741)) + 16|0);
        $6743 = HEAP32[$6742>>2]|0;
        HEAP32[$pVTab>>2] = $6743;
        $6744 = HEAP32[$db>>2]|0;
        $6745 = HEAP32[$pVTab>>2]|0;
        $6746 = (_sqlite3VtabBegin($6744,$6745)|0);
        HEAP32[$rc>>2] = $6746;
        $6747 = HEAP32[$pVTab>>2]|0;
        $6748 = ($6747|0)!=(0|0);
        if ($6748) {
         $6749 = HEAP32[$0>>2]|0;
         $6750 = HEAP32[$pVTab>>2]|0;
         $6751 = ((($6750)) + 8|0);
         $6752 = HEAP32[$6751>>2]|0;
         _sqlite3VtabImportErrmsg($6749,$6752);
        }
        break;
       }
       case 150:  {
        dest=$sMem102; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
        $6753 = HEAP32[$db>>2]|0;
        $6754 = ((($sMem102)) + 32|0);
        HEAP32[$6754>>2] = $6753;
        $6755 = HEAP32[$pOp>>2]|0;
        $6756 = ((($6755)) + 8|0);
        $6757 = HEAP32[$6756>>2]|0;
        $6758 = HEAP32[$aMem>>2]|0;
        $6759 = (($6758) + (($6757*40)|0)|0);
        $6760 = (_sqlite3VdbeMemCopy($sMem102,$6759)|0);
        HEAP32[$rc>>2] = $6760;
        $6761 = (_sqlite3_value_text($sMem102)|0);
        HEAP32[$zTab>>2] = $6761;
        $6762 = HEAP32[$zTab>>2]|0;
        $6763 = ($6762|0)!=(0|0);
        if ($6763) {
         $6764 = HEAP32[$db>>2]|0;
         $6765 = HEAP32[$pOp>>2]|0;
         $6766 = ((($6765)) + 4|0);
         $6767 = HEAP32[$6766>>2]|0;
         $6768 = HEAP32[$zTab>>2]|0;
         $6769 = HEAP32[$0>>2]|0;
         $6770 = ((($6769)) + 44|0);
         $6771 = (_sqlite3VtabCallCreate($6764,$6767,$6768,$6770)|0);
         HEAP32[$rc>>2] = $6771;
        }
        _sqlite3VdbeMemRelease($sMem102);
        break;
       }
       case 151:  {
        $6772 = HEAP32[$db>>2]|0;
        $6773 = ((($6772)) + 164|0);
        $6774 = HEAP32[$6773>>2]|0;
        $6775 = (($6774) + 1)|0;
        HEAP32[$6773>>2] = $6775;
        $6776 = HEAP32[$db>>2]|0;
        $6777 = HEAP32[$pOp>>2]|0;
        $6778 = ((($6777)) + 4|0);
        $6779 = HEAP32[$6778>>2]|0;
        $6780 = HEAP32[$pOp>>2]|0;
        $6781 = ((($6780)) + 16|0);
        $6782 = HEAP32[$6781>>2]|0;
        $6783 = (_sqlite3VtabCallDestroy($6776,$6779,$6782)|0);
        HEAP32[$rc>>2] = $6783;
        $6784 = HEAP32[$db>>2]|0;
        $6785 = ((($6784)) + 164|0);
        $6786 = HEAP32[$6785>>2]|0;
        $6787 = (($6786) + -1)|0;
        HEAP32[$6785>>2] = $6787;
        break;
       }
       case 152:  {
        HEAP32[$pCur103>>2] = 0;
        HEAP32[$pVCur>>2] = 0;
        $6788 = HEAP32[$pOp>>2]|0;
        $6789 = ((($6788)) + 16|0);
        $6790 = HEAP32[$6789>>2]|0;
        $6791 = ((($6790)) + 8|0);
        $6792 = HEAP32[$6791>>2]|0;
        HEAP32[$pVtab104>>2] = $6792;
        $6793 = HEAP32[$pVtab104>>2]|0;
        $6794 = ($6793|0)==(0|0);
        if (!($6794)) {
         $6795 = HEAP32[$pVtab104>>2]|0;
         $6796 = HEAP32[$6795>>2]|0;
         $6797 = ($6796|0)==(0|0);
         if (!($6797)) {
          $6798 = HEAP32[$pVtab104>>2]|0;
          $6799 = HEAP32[$6798>>2]|0;
          HEAP32[$pModule105>>2] = $6799;
          $6800 = HEAP32[$pModule105>>2]|0;
          $6801 = ((($6800)) + 24|0);
          $6802 = HEAP32[$6801>>2]|0;
          $6803 = HEAP32[$pVtab104>>2]|0;
          $6804 = (FUNCTION_TABLE_iii[$6802 & 255]($6803,$pVCur)|0);
          HEAP32[$rc>>2] = $6804;
          $6805 = HEAP32[$0>>2]|0;
          $6806 = HEAP32[$pVtab104>>2]|0;
          _sqlite3VtabImportErrmsg($6805,$6806);
          $6807 = HEAP32[$rc>>2]|0;
          $6808 = (0)==($6807|0);
          if (!($6808)) {
           break L11;
          }
          $6809 = HEAP32[$pVtab104>>2]|0;
          $6810 = HEAP32[$pVCur>>2]|0;
          HEAP32[$6810>>2] = $6809;
          $6811 = HEAP32[$0>>2]|0;
          $6812 = HEAP32[$pOp>>2]|0;
          $6813 = ((($6812)) + 4|0);
          $6814 = HEAP32[$6813>>2]|0;
          $6815 = (_allocateCursor($6811,$6814,0,-1,2)|0);
          HEAP32[$pCur103>>2] = $6815;
          $6816 = HEAP32[$pCur103>>2]|0;
          $6817 = ($6816|0)!=(0|0);
          if (!($6817)) {
           label = 804;
           break L8;
          }
          $6818 = HEAP32[$pVCur>>2]|0;
          $6819 = HEAP32[$pCur103>>2]|0;
          $6820 = ((($6819)) + 16|0);
          HEAP32[$6820>>2] = $6818;
          $6821 = HEAP32[$pVtab104>>2]|0;
          $6822 = ((($6821)) + 4|0);
          $6823 = HEAP32[$6822>>2]|0;
          $6824 = (($6823) + 1)|0;
          HEAP32[$6822>>2] = $6824;
          break L11;
         }
        }
        HEAP32[$rc>>2] = 6;
        break;
       }
       case 11:  {
        $6829 = HEAP32[$pOp>>2]|0;
        $6830 = ((($6829)) + 12|0);
        $6831 = HEAP32[$6830>>2]|0;
        $6832 = HEAP32[$aMem>>2]|0;
        $6833 = (($6832) + (($6831*40)|0)|0);
        HEAP32[$pQuery>>2] = $6833;
        $6834 = HEAP32[$pQuery>>2]|0;
        $6835 = ((($6834)) + 40|0);
        HEAP32[$pArgc>>2] = $6835;
        $6836 = HEAP32[$pOp>>2]|0;
        $6837 = ((($6836)) + 4|0);
        $6838 = HEAP32[$6837>>2]|0;
        $6839 = HEAP32[$0>>2]|0;
        $6840 = ((($6839)) + 56|0);
        $6841 = HEAP32[$6840>>2]|0;
        $6842 = (($6841) + ($6838<<2)|0);
        $6843 = HEAP32[$6842>>2]|0;
        HEAP32[$pCur109>>2] = $6843;
        $6844 = HEAP32[$pCur109>>2]|0;
        $6845 = ((($6844)) + 16|0);
        $6846 = HEAP32[$6845>>2]|0;
        HEAP32[$pVCur107>>2] = $6846;
        $6847 = HEAP32[$pVCur107>>2]|0;
        $6848 = HEAP32[$6847>>2]|0;
        HEAP32[$pVtab108>>2] = $6848;
        $6849 = HEAP32[$pVtab108>>2]|0;
        $6850 = HEAP32[$6849>>2]|0;
        HEAP32[$pModule106>>2] = $6850;
        $6851 = HEAP32[$pArgc>>2]|0;
        $6852 = $6851;
        $6853 = $6852;
        $6854 = HEAP32[$6853>>2]|0;
        $6855 = (($6852) + 4)|0;
        $6856 = $6855;
        $6857 = HEAP32[$6856>>2]|0;
        HEAP32[$nArg>>2] = $6854;
        $6858 = HEAP32[$pQuery>>2]|0;
        $6859 = $6858;
        $6860 = $6859;
        $6861 = HEAP32[$6860>>2]|0;
        $6862 = (($6859) + 4)|0;
        $6863 = $6862;
        $6864 = HEAP32[$6863>>2]|0;
        HEAP32[$iQuery>>2] = $6861;
        HEAP32[$res110>>2] = 0;
        $6865 = HEAP32[$0>>2]|0;
        $6866 = ((($6865)) + 12|0);
        $6867 = HEAP32[$6866>>2]|0;
        HEAP32[$apArg>>2] = $6867;
        HEAP32[$i111>>2] = 0;
        while(1) {
         $6868 = HEAP32[$i111>>2]|0;
         $6869 = HEAP32[$nArg>>2]|0;
         $6870 = ($6868|0)<($6869|0);
         if (!($6870)) {
          break;
         }
         $6871 = HEAP32[$i111>>2]|0;
         $6872 = (($6871) + 1)|0;
         $6873 = HEAP32[$pArgc>>2]|0;
         $6874 = (($6873) + (($6872*40)|0)|0);
         $6875 = HEAP32[$i111>>2]|0;
         $6876 = HEAP32[$apArg>>2]|0;
         $6877 = (($6876) + ($6875<<2)|0);
         HEAP32[$6877>>2] = $6874;
         $6878 = HEAP32[$i111>>2]|0;
         $6879 = (($6878) + 1)|0;
         HEAP32[$i111>>2] = $6879;
        }
        $6880 = HEAP32[$pModule106>>2]|0;
        $6881 = ((($6880)) + 32|0);
        $6882 = HEAP32[$6881>>2]|0;
        $6883 = HEAP32[$pVCur107>>2]|0;
        $6884 = HEAP32[$iQuery>>2]|0;
        $6885 = HEAP32[$pOp>>2]|0;
        $6886 = ((($6885)) + 16|0);
        $6887 = HEAP32[$6886>>2]|0;
        $6888 = HEAP32[$nArg>>2]|0;
        $6889 = HEAP32[$apArg>>2]|0;
        $6890 = (FUNCTION_TABLE_iiiiii[$6882 & 255]($6883,$6884,$6887,$6888,$6889)|0);
        HEAP32[$rc>>2] = $6890;
        $6891 = HEAP32[$0>>2]|0;
        $6892 = HEAP32[$pVtab108>>2]|0;
        _sqlite3VtabImportErrmsg($6891,$6892);
        $6893 = HEAP32[$rc>>2]|0;
        $6894 = ($6893|0)==(0);
        if ($6894) {
         $6895 = HEAP32[$pModule106>>2]|0;
         $6896 = ((($6895)) + 40|0);
         $6897 = HEAP32[$6896>>2]|0;
         $6898 = HEAP32[$pVCur107>>2]|0;
         $6899 = (FUNCTION_TABLE_ii[$6897 & 255]($6898)|0);
         HEAP32[$res110>>2] = $6899;
        }
        $6900 = HEAP32[$pCur109>>2]|0;
        $6901 = ((($6900)) + 2|0);
        HEAP8[$6901>>0] = 0;
        $6902 = HEAP32[$res110>>2]|0;
        $6903 = ($6902|0)!=(0);
        if ($6903) {
         label = 15;
        }
        break;
       }
       case 153:  {
        $6904 = HEAP32[$pOp>>2]|0;
        $6905 = ((($6904)) + 4|0);
        $6906 = HEAP32[$6905>>2]|0;
        $6907 = HEAP32[$0>>2]|0;
        $6908 = ((($6907)) + 56|0);
        $6909 = HEAP32[$6908>>2]|0;
        $6910 = (($6909) + ($6906<<2)|0);
        $6911 = HEAP32[$6910>>2]|0;
        HEAP32[$pCur115>>2] = $6911;
        $6912 = HEAP32[$pOp>>2]|0;
        $6913 = ((($6912)) + 12|0);
        $6914 = HEAP32[$6913>>2]|0;
        $6915 = HEAP32[$aMem>>2]|0;
        $6916 = (($6915) + (($6914*40)|0)|0);
        HEAP32[$pDest114>>2] = $6916;
        $6917 = HEAP32[$pCur115>>2]|0;
        $6918 = ((($6917)) + 2|0);
        $6919 = HEAP8[$6918>>0]|0;
        $6920 = ($6919<<24>>24)!=(0);
        if ($6920) {
         $6921 = HEAP32[$pDest114>>2]|0;
         _sqlite3VdbeMemSetNull($6921);
         break L11;
        }
        $6922 = HEAP32[$pCur115>>2]|0;
        $6923 = ((($6922)) + 16|0);
        $6924 = HEAP32[$6923>>2]|0;
        $6925 = HEAP32[$6924>>2]|0;
        HEAP32[$pVtab112>>2] = $6925;
        $6926 = HEAP32[$pVtab112>>2]|0;
        $6927 = HEAP32[$6926>>2]|0;
        HEAP32[$pModule113>>2] = $6927;
        ;HEAP32[$sContext>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;HEAP32[$sContext+8>>2]=0|0;HEAP32[$sContext+12>>2]=0|0;HEAP32[$sContext+16>>2]=0|0;HEAP32[$sContext+20>>2]=0|0;HEAP32[$sContext+24>>2]=0|0;HEAP32[$sContext+28>>2]=0|0;
        $6928 = HEAP32[$pDest114>>2]|0;
        HEAP32[$sContext>>2] = $6928;
        $6929 = HEAP32[$pDest114>>2]|0;
        $6930 = ((($6929)) + 8|0);
        $6931 = HEAP16[$6930>>1]|0;
        $6932 = $6931&65535;
        $6933 = $6932 & -49664;
        $6934 = $6933 | 1;
        $6935 = $6934&65535;
        $6936 = HEAP32[$pDest114>>2]|0;
        $6937 = ((($6936)) + 8|0);
        HEAP16[$6937>>1] = $6935;
        $6938 = HEAP32[$pModule113>>2]|0;
        $6939 = ((($6938)) + 44|0);
        $6940 = HEAP32[$6939>>2]|0;
        $6941 = HEAP32[$pCur115>>2]|0;
        $6942 = ((($6941)) + 16|0);
        $6943 = HEAP32[$6942>>2]|0;
        $6944 = HEAP32[$pOp>>2]|0;
        $6945 = ((($6944)) + 8|0);
        $6946 = HEAP32[$6945>>2]|0;
        $6947 = (FUNCTION_TABLE_iiii[$6940 & 255]($6943,$sContext,$6946)|0);
        HEAP32[$rc>>2] = $6947;
        $6948 = HEAP32[$0>>2]|0;
        $6949 = HEAP32[$pVtab112>>2]|0;
        _sqlite3VtabImportErrmsg($6948,$6949);
        $6950 = ((($sContext)) + 20|0);
        $6951 = HEAP32[$6950>>2]|0;
        $6952 = ($6951|0)!=(0);
        if ($6952) {
         $6953 = ((($sContext)) + 20|0);
         $6954 = HEAP32[$6953>>2]|0;
         HEAP32[$rc>>2] = $6954;
        }
        $6955 = HEAP32[$pDest114>>2]|0;
        $6956 = HEAP8[$encoding>>0]|0;
        $6957 = $6956&255;
        (_sqlite3VdbeChangeEncoding($6955,$6957)|0);
        $6958 = HEAP32[$pDest114>>2]|0;
        $6959 = (_sqlite3VdbeMemTooBig($6958)|0);
        $6960 = ($6959|0)!=(0);
        if ($6960) {
         label = 859;
         break L8;
        }
        break;
       }
       case 154:  {
        HEAP32[$res118>>2] = 0;
        $6961 = HEAP32[$pOp>>2]|0;
        $6962 = ((($6961)) + 4|0);
        $6963 = HEAP32[$6962>>2]|0;
        $6964 = HEAP32[$0>>2]|0;
        $6965 = ((($6964)) + 56|0);
        $6966 = HEAP32[$6965>>2]|0;
        $6967 = (($6966) + ($6963<<2)|0);
        $6968 = HEAP32[$6967>>2]|0;
        HEAP32[$pCur119>>2] = $6968;
        $6969 = HEAP32[$pCur119>>2]|0;
        $6970 = ((($6969)) + 2|0);
        $6971 = HEAP8[$6970>>0]|0;
        $6972 = ($6971<<24>>24)!=(0);
        if (!($6972)) {
         $6973 = HEAP32[$pCur119>>2]|0;
         $6974 = ((($6973)) + 16|0);
         $6975 = HEAP32[$6974>>2]|0;
         $6976 = HEAP32[$6975>>2]|0;
         HEAP32[$pVtab116>>2] = $6976;
         $6977 = HEAP32[$pVtab116>>2]|0;
         $6978 = HEAP32[$6977>>2]|0;
         HEAP32[$pModule117>>2] = $6978;
         $6979 = HEAP32[$pModule117>>2]|0;
         $6980 = ((($6979)) + 36|0);
         $6981 = HEAP32[$6980>>2]|0;
         $6982 = HEAP32[$pCur119>>2]|0;
         $6983 = ((($6982)) + 16|0);
         $6984 = HEAP32[$6983>>2]|0;
         $6985 = (FUNCTION_TABLE_ii[$6981 & 255]($6984)|0);
         HEAP32[$rc>>2] = $6985;
         $6986 = HEAP32[$0>>2]|0;
         $6987 = HEAP32[$pVtab116>>2]|0;
         _sqlite3VtabImportErrmsg($6986,$6987);
         $6988 = HEAP32[$rc>>2]|0;
         $6989 = ($6988|0)==(0);
         if ($6989) {
          $6990 = HEAP32[$pModule117>>2]|0;
          $6991 = ((($6990)) + 40|0);
          $6992 = HEAP32[$6991>>2]|0;
          $6993 = HEAP32[$pCur119>>2]|0;
          $6994 = ((($6993)) + 16|0);
          $6995 = HEAP32[$6994>>2]|0;
          $6996 = (FUNCTION_TABLE_ii[$6992 & 255]($6995)|0);
          HEAP32[$res118>>2] = $6996;
         }
         $6997 = HEAP32[$res118>>2]|0;
         $6998 = ($6997|0)!=(0);
         if ($6998) {
          label = 9;
         } else {
          label = 8;
         }
        }
        break;
       }
       case 155:  {
        $6999 = HEAP32[$pOp>>2]|0;
        $7000 = ((($6999)) + 16|0);
        $7001 = HEAP32[$7000>>2]|0;
        $7002 = ((($7001)) + 8|0);
        $7003 = HEAP32[$7002>>2]|0;
        HEAP32[$pVtab120>>2] = $7003;
        $7004 = HEAP32[$pOp>>2]|0;
        $7005 = ((($7004)) + 4|0);
        $7006 = HEAP32[$7005>>2]|0;
        $7007 = HEAP32[$aMem>>2]|0;
        $7008 = (($7007) + (($7006*40)|0)|0);
        HEAP32[$pName>>2] = $7008;
        $7009 = HEAP32[$pName>>2]|0;
        $7010 = (_sqlite3VdbeChangeEncoding($7009,1)|0);
        HEAP32[$rc>>2] = $7010;
        $7011 = HEAP32[$rc>>2]|0;
        $7012 = ($7011|0)==(0);
        if ($7012) {
         $7013 = HEAP32[$pVtab120>>2]|0;
         $7014 = HEAP32[$7013>>2]|0;
         $7015 = ((($7014)) + 76|0);
         $7016 = HEAP32[$7015>>2]|0;
         $7017 = HEAP32[$pVtab120>>2]|0;
         $7018 = HEAP32[$pName>>2]|0;
         $7019 = ((($7018)) + 16|0);
         $7020 = HEAP32[$7019>>2]|0;
         $7021 = (FUNCTION_TABLE_iii[$7016 & 255]($7017,$7020)|0);
         HEAP32[$rc>>2] = $7021;
         $7022 = HEAP32[$0>>2]|0;
         $7023 = HEAP32[$pVtab120>>2]|0;
         _sqlite3VtabImportErrmsg($7022,$7023);
         $7024 = HEAP32[$0>>2]|0;
         $7025 = ((($7024)) + 88|0);
         $7026 = HEAP16[$7025>>1]|0;
         $7027 = $7026 & -9;
         HEAP16[$7025>>1] = $7027;
        }
        break;
       }
       case 12:  {
        $7028 = HEAP32[$pOp>>2]|0;
        $7029 = ((($7028)) + 16|0);
        $7030 = HEAP32[$7029>>2]|0;
        $7031 = ((($7030)) + 8|0);
        $7032 = HEAP32[$7031>>2]|0;
        HEAP32[$pVtab121>>2] = $7032;
        $7033 = HEAP32[$pVtab121>>2]|0;
        $7034 = ($7033|0)==(0|0);
        if (!($7034)) {
         $7035 = HEAP32[$pVtab121>>2]|0;
         $7036 = HEAP32[$7035>>2]|0;
         $7037 = ($7036|0)==(0|0);
         if (!($7037)) {
          $7038 = HEAP32[$pVtab121>>2]|0;
          $7039 = HEAP32[$7038>>2]|0;
          HEAP32[$pModule122>>2] = $7039;
          $7040 = HEAP32[$pOp>>2]|0;
          $7041 = ((($7040)) + 8|0);
          $7042 = HEAP32[$7041>>2]|0;
          HEAP32[$nArg123>>2] = $7042;
          $7043 = HEAP32[$pModule122>>2]|0;
          $7044 = ((($7043)) + 52|0);
          $7045 = HEAP32[$7044>>2]|0;
          $7046 = ($7045|0)!=(0|0);
          if (!($7046)) {
           break L11;
          }
          $7047 = HEAP32[$db>>2]|0;
          $7048 = ((($7047)) + 70|0);
          $7049 = HEAP8[$7048>>0]|0;
          HEAP8[$vtabOnConflict>>0] = $7049;
          $7050 = HEAP32[$0>>2]|0;
          $7051 = ((($7050)) + 12|0);
          $7052 = HEAP32[$7051>>2]|0;
          HEAP32[$apArg126>>2] = $7052;
          $7053 = HEAP32[$pOp>>2]|0;
          $7054 = ((($7053)) + 12|0);
          $7055 = HEAP32[$7054>>2]|0;
          $7056 = HEAP32[$aMem>>2]|0;
          $7057 = (($7056) + (($7055*40)|0)|0);
          HEAP32[$pX127>>2] = $7057;
          HEAP32[$i124>>2] = 0;
          while(1) {
           $7058 = HEAP32[$i124>>2]|0;
           $7059 = HEAP32[$nArg123>>2]|0;
           $7060 = ($7058|0)<($7059|0);
           if (!($7060)) {
            break;
           }
           $7061 = HEAP32[$pX127>>2]|0;
           $7062 = HEAP32[$i124>>2]|0;
           $7063 = HEAP32[$apArg126>>2]|0;
           $7064 = (($7063) + ($7062<<2)|0);
           HEAP32[$7064>>2] = $7061;
           $7065 = HEAP32[$pX127>>2]|0;
           $7066 = ((($7065)) + 40|0);
           HEAP32[$pX127>>2] = $7066;
           $7067 = HEAP32[$i124>>2]|0;
           $7068 = (($7067) + 1)|0;
           HEAP32[$i124>>2] = $7068;
          }
          $7069 = HEAP32[$pOp>>2]|0;
          $7070 = ((($7069)) + 3|0);
          $7071 = HEAP8[$7070>>0]|0;
          $7072 = HEAP32[$db>>2]|0;
          $7073 = ((($7072)) + 70|0);
          HEAP8[$7073>>0] = $7071;
          $7074 = HEAP32[$pModule122>>2]|0;
          $7075 = ((($7074)) + 52|0);
          $7076 = HEAP32[$7075>>2]|0;
          $7077 = HEAP32[$pVtab121>>2]|0;
          $7078 = HEAP32[$nArg123>>2]|0;
          $7079 = HEAP32[$apArg126>>2]|0;
          $7080 = (FUNCTION_TABLE_iiiii[$7076 & 255]($7077,$7078,$7079,$rowid125)|0);
          HEAP32[$rc>>2] = $7080;
          $7081 = HEAP8[$vtabOnConflict>>0]|0;
          $7082 = HEAP32[$db>>2]|0;
          $7083 = ((($7082)) + 70|0);
          HEAP8[$7083>>0] = $7081;
          $7084 = HEAP32[$0>>2]|0;
          $7085 = HEAP32[$pVtab121>>2]|0;
          _sqlite3VtabImportErrmsg($7084,$7085);
          $7086 = HEAP32[$rc>>2]|0;
          $7087 = ($7086|0)==(0);
          if ($7087) {
           $7088 = HEAP32[$pOp>>2]|0;
           $7089 = ((($7088)) + 4|0);
           $7090 = HEAP32[$7089>>2]|0;
           $7091 = ($7090|0)!=(0);
           if ($7091) {
            $7092 = $rowid125;
            $7093 = $7092;
            $7094 = HEAP32[$7093>>2]|0;
            $7095 = (($7092) + 4)|0;
            $7096 = $7095;
            $7097 = HEAP32[$7096>>2]|0;
            $7098 = $lastRowid;
            $7099 = $7098;
            HEAP32[$7099>>2] = $7094;
            $7100 = (($7098) + 4)|0;
            $7101 = $7100;
            HEAP32[$7101>>2] = $7097;
            $7102 = HEAP32[$db>>2]|0;
            $7103 = ((($7102)) + 32|0);
            $7104 = $7103;
            $7105 = $7104;
            HEAP32[$7105>>2] = $7094;
            $7106 = (($7104) + 4)|0;
            $7107 = $7106;
            HEAP32[$7107>>2] = $7097;
           }
          }
          $7108 = HEAP32[$rc>>2]|0;
          $7109 = $7108 & 255;
          $7110 = ($7109|0)==(19);
          if ($7110) {
           $7111 = HEAP32[$pOp>>2]|0;
           $7112 = ((($7111)) + 16|0);
           $7113 = HEAP32[$7112>>2]|0;
           $7114 = ((($7113)) + 16|0);
           $7115 = HEAP8[$7114>>0]|0;
           $7116 = $7115&255;
           $7117 = ($7116|0)!=(0);
           if ($7117) {
            $7118 = HEAP32[$pOp>>2]|0;
            $7119 = ((($7118)) + 3|0);
            $7120 = HEAP8[$7119>>0]|0;
            $7121 = $7120&255;
            $7122 = ($7121|0)==(4);
            if ($7122) {
             HEAP32[$rc>>2] = 0;
             break L11;
            }
            $7123 = HEAP32[$pOp>>2]|0;
            $7124 = ((($7123)) + 3|0);
            $7125 = HEAP8[$7124>>0]|0;
            $7126 = $7125&255;
            $7127 = ($7126|0)==(5);
            if ($7127) {
             $7133 = 2;
            } else {
             $7128 = HEAP32[$pOp>>2]|0;
             $7129 = ((($7128)) + 3|0);
             $7130 = HEAP8[$7129>>0]|0;
             $7131 = $7130&255;
             $7133 = $7131;
            }
            $7132 = $7133&255;
            $7134 = HEAP32[$0>>2]|0;
            $7135 = ((($7134)) + 86|0);
            HEAP8[$7135>>0] = $7132;
            break L11;
           }
          }
          $7136 = HEAP32[$0>>2]|0;
          $7137 = ((($7136)) + 92|0);
          $7138 = HEAP32[$7137>>2]|0;
          $7139 = (($7138) + 1)|0;
          HEAP32[$7137>>2] = $7139;
          break L11;
         }
        }
        HEAP32[$rc>>2] = 6;
        break;
       }
       case 156:  {
        $7140 = HEAP32[$0>>2]|0;
        $7141 = HEAP32[$pOp>>2]|0;
        $7142 = (_out2Prerelease($7140,$7141)|0);
        HEAP32[$pOut>>2] = $7142;
        $7143 = HEAP32[$pOp>>2]|0;
        $7144 = ((($7143)) + 4|0);
        $7145 = HEAP32[$7144>>2]|0;
        $7146 = HEAP32[$db>>2]|0;
        $7147 = ((($7146)) + 16|0);
        $7148 = HEAP32[$7147>>2]|0;
        $7149 = (($7148) + ($7145<<4)|0);
        $7150 = ((($7149)) + 4|0);
        $7151 = HEAP32[$7150>>2]|0;
        $7152 = (_sqlite3BtreeLastPage($7151)|0);
        $7153 = HEAP32[$pOut>>2]|0;
        $7154 = $7153;
        $7155 = $7154;
        HEAP32[$7155>>2] = $7152;
        $7156 = (($7154) + 4)|0;
        $7157 = $7156;
        HEAP32[$7157>>2] = 0;
        break;
       }
       case 157:  {
        $7158 = HEAP32[$0>>2]|0;
        $7159 = HEAP32[$pOp>>2]|0;
        $7160 = (_out2Prerelease($7158,$7159)|0);
        HEAP32[$pOut>>2] = $7160;
        $7161 = HEAP32[$pOp>>2]|0;
        $7162 = ((($7161)) + 4|0);
        $7163 = HEAP32[$7162>>2]|0;
        $7164 = HEAP32[$db>>2]|0;
        $7165 = ((($7164)) + 16|0);
        $7166 = HEAP32[$7165>>2]|0;
        $7167 = (($7166) + ($7163<<4)|0);
        $7168 = ((($7167)) + 4|0);
        $7169 = HEAP32[$7168>>2]|0;
        HEAP32[$pBt128>>2] = $7169;
        HEAP32[$newMax>>2] = 0;
        $7170 = HEAP32[$pOp>>2]|0;
        $7171 = ((($7170)) + 12|0);
        $7172 = HEAP32[$7171>>2]|0;
        $7173 = ($7172|0)!=(0);
        if ($7173) {
         $7174 = HEAP32[$pBt128>>2]|0;
         $7175 = (_sqlite3BtreeLastPage($7174)|0);
         HEAP32[$newMax>>2] = $7175;
         $7176 = HEAP32[$newMax>>2]|0;
         $7177 = HEAP32[$pOp>>2]|0;
         $7178 = ((($7177)) + 12|0);
         $7179 = HEAP32[$7178>>2]|0;
         $7180 = ($7176>>>0)<($7179>>>0);
         if ($7180) {
          $7181 = HEAP32[$pOp>>2]|0;
          $7182 = ((($7181)) + 12|0);
          $7183 = HEAP32[$7182>>2]|0;
          HEAP32[$newMax>>2] = $7183;
         }
        }
        $7184 = HEAP32[$pBt128>>2]|0;
        $7185 = HEAP32[$newMax>>2]|0;
        $7186 = (_sqlite3BtreeMaxPageCount($7184,$7185)|0);
        $7187 = ($7186|0)<(0);
        $7188 = $7187 << 31 >> 31;
        $7189 = HEAP32[$pOut>>2]|0;
        $7190 = $7189;
        $7191 = $7190;
        HEAP32[$7191>>2] = $7186;
        $7192 = (($7190) + 4)|0;
        $7193 = $7192;
        HEAP32[$7193>>2] = $7188;
        break;
       }
       case 158:  {
        $7194 = HEAP32[$db>>2]|0;
        $7195 = ((($7194)) + 176|0);
        $7196 = HEAP32[$7195>>2]|0;
        $7197 = ($7196|0)!=(0|0);
        if ($7197) {
         $7198 = HEAP32[$0>>2]|0;
         $7199 = ((($7198)) + 88|0);
         $7200 = HEAP16[$7199>>1]|0;
         $7201 = ($7200&65535) >>> 9;
         $7202 = $7201 & 1;
         $7203 = $7202&65535;
         $7204 = ($7203|0)!=(0);
         if (!($7204)) {
          $7205 = HEAP32[$pOp>>2]|0;
          $7206 = ((($7205)) + 16|0);
          $7207 = HEAP32[$7206>>2]|0;
          $7208 = ($7207|0)!=(0|0);
          if ($7208) {
           $7209 = HEAP32[$pOp>>2]|0;
           $7210 = ((($7209)) + 16|0);
           $7211 = HEAP32[$7210>>2]|0;
           $7215 = $7211;
          } else {
           $7212 = HEAP32[$0>>2]|0;
           $7213 = ((($7212)) + 168|0);
           $7214 = HEAP32[$7213>>2]|0;
           $7215 = $7214;
          }
          HEAP32[$zTrace>>2] = $7215;
          $7216 = ($7215|0)!=(0|0);
          if ($7216) {
           $7217 = HEAP32[$0>>2]|0;
           $7218 = HEAP32[$zTrace>>2]|0;
           $7219 = (_sqlite3VdbeExpandSql($7217,$7218)|0);
           HEAP32[$z129>>2] = $7219;
           $7220 = HEAP32[$db>>2]|0;
           $7221 = ((($7220)) + 176|0);
           $7222 = HEAP32[$7221>>2]|0;
           $7223 = HEAP32[$db>>2]|0;
           $7224 = ((($7223)) + 180|0);
           $7225 = HEAP32[$7224>>2]|0;
           $7226 = HEAP32[$z129>>2]|0;
           FUNCTION_TABLE_vii[$7222 & 255]($7225,$7226);
           $7227 = HEAP32[$db>>2]|0;
           $7228 = HEAP32[$z129>>2]|0;
           _sqlite3DbFree($7227,$7228);
          }
         }
        }
        $7229 = HEAP32[$pOp>>2]|0;
        $7230 = ((($7229)) + 8|0);
        $7231 = HEAP32[$7230>>2]|0;
        $7232 = ($7231|0)!=(0);
        if ($7232) {
         label = 15;
        }
        break;
       }
       default: {
       }
       }
      } while(0);
      do {
       if ((label|0) == 21) {
        label = 0;
        $239 = HEAP32[$pOp>>2]|0;
        $240 = HEAP32[$aOp>>2]|0;
        $241 = $239;
        $242 = $240;
        $243 = (($241) - ($242))|0;
        $244 = (($243|0) / 20)&-1;
        HEAP32[$pcx>>2] = $244;
        $245 = HEAP32[$pOp>>2]|0;
        $246 = ((($245)) + 4|0);
        $247 = HEAP32[$246>>2]|0;
        $248 = ($247|0)==(0);
        if (!($248)) {
         label = 26;
         break L8;
        }
        $249 = HEAP32[$0>>2]|0;
        $250 = ((($249)) + 176|0);
        $251 = HEAP32[$250>>2]|0;
        $252 = ($251|0)!=(0|0);
        if (!($252)) {
         label = 26;
         break L8;
        }
        $253 = HEAP32[$0>>2]|0;
        $254 = ((($253)) + 176|0);
        $255 = HEAP32[$254>>2]|0;
        HEAP32[$pFrame>>2] = $255;
        $256 = HEAP32[$pFrame>>2]|0;
        $257 = ((($256)) + 4|0);
        $258 = HEAP32[$257>>2]|0;
        $259 = HEAP32[$0>>2]|0;
        $260 = ((($259)) + 176|0);
        HEAP32[$260>>2] = $258;
        $261 = HEAP32[$0>>2]|0;
        $262 = ((($261)) + 184|0);
        $263 = HEAP32[$262>>2]|0;
        $264 = (($263) + -1)|0;
        HEAP32[$262>>2] = $264;
        $265 = HEAP32[$db>>2]|0;
        $266 = HEAP32[$0>>2]|0;
        $267 = ((($266)) + 92|0);
        $268 = HEAP32[$267>>2]|0;
        _sqlite3VdbeSetChanges($265,$268);
        $269 = HEAP32[$pFrame>>2]|0;
        $270 = (_sqlite3VdbeFrameRestore($269)|0);
        HEAP32[$pcx>>2] = $270;
        $271 = HEAP32[$db>>2]|0;
        $272 = ((($271)) + 32|0);
        $273 = $272;
        $274 = $273;
        $275 = HEAP32[$274>>2]|0;
        $276 = (($273) + 4)|0;
        $277 = $276;
        $278 = HEAP32[$277>>2]|0;
        $279 = $lastRowid;
        $280 = $279;
        HEAP32[$280>>2] = $275;
        $281 = (($279) + 4)|0;
        $282 = $281;
        HEAP32[$282>>2] = $278;
        $283 = HEAP32[$pOp>>2]|0;
        $284 = ((($283)) + 8|0);
        $285 = HEAP32[$284>>2]|0;
        $286 = ($285|0)==(4);
        if ($286) {
         $287 = HEAP32[$pcx>>2]|0;
         $288 = HEAP32[$0>>2]|0;
         $289 = ((($288)) + 4|0);
         $290 = HEAP32[$289>>2]|0;
         $291 = (($290) + (($287*20)|0)|0);
         $292 = ((($291)) + 8|0);
         $293 = HEAP32[$292>>2]|0;
         $294 = (($293) - 1)|0;
         HEAP32[$pcx>>2] = $294;
        }
        $295 = HEAP32[$0>>2]|0;
        $296 = ((($295)) + 4|0);
        $297 = HEAP32[$296>>2]|0;
        HEAP32[$aOp>>2] = $297;
        $298 = HEAP32[$0>>2]|0;
        $299 = ((($298)) + 8|0);
        $300 = HEAP32[$299>>2]|0;
        HEAP32[$aMem>>2] = $300;
        $301 = HEAP32[$pcx>>2]|0;
        $302 = HEAP32[$aOp>>2]|0;
        $303 = (($302) + (($301*20)|0)|0);
        HEAP32[$pOp>>2] = $303;
       }
       else if ((label|0) == 50) {
        label = 0;
        $476 = HEAP32[$0>>2]|0;
        $477 = HEAP32[$pOp>>2]|0;
        $478 = (_out2Prerelease($476,$477)|0);
        HEAP32[$pOut>>2] = $478;
        $479 = HEAP32[$pOut>>2]|0;
        $480 = ((($479)) + 8|0);
        HEAP16[$480>>1] = 2562;
        $481 = HEAP32[$pOp>>2]|0;
        $482 = ((($481)) + 16|0);
        $483 = HEAP32[$482>>2]|0;
        $484 = HEAP32[$pOut>>2]|0;
        $485 = ((($484)) + 16|0);
        HEAP32[$485>>2] = $483;
        $486 = HEAP32[$pOp>>2]|0;
        $487 = ((($486)) + 4|0);
        $488 = HEAP32[$487>>2]|0;
        $489 = HEAP32[$pOut>>2]|0;
        $490 = ((($489)) + 12|0);
        HEAP32[$490>>2] = $488;
        $491 = HEAP8[$encoding>>0]|0;
        $492 = HEAP32[$pOut>>2]|0;
        $493 = ((($492)) + 10|0);
        HEAP8[$493>>0] = $491;
        $494 = HEAP32[$pOp>>2]|0;
        $495 = ((($494)) + 3|0);
        $496 = HEAP8[$495>>0]|0;
        $497 = ($496<<24>>24)!=(0);
        if ($497) {
         $498 = HEAP32[$pOp>>2]|0;
         $499 = ((($498)) + 12|0);
         $500 = HEAP32[$499>>2]|0;
         $501 = HEAP32[$aMem>>2]|0;
         $502 = (($501) + (($500*40)|0)|0);
         HEAP32[$pIn3>>2] = $502;
         $503 = HEAP32[$pIn3>>2]|0;
         $504 = $503;
         $505 = $504;
         $506 = HEAP32[$505>>2]|0;
         $507 = (($504) + 4)|0;
         $508 = $507;
         $509 = HEAP32[$508>>2]|0;
         $510 = ($506|0)!=(0);
         $511 = ($509|0)!=(0);
         $512 = $510 | $511;
         if ($512) {
          $513 = HEAP32[$pOut>>2]|0;
          $514 = ((($513)) + 8|0);
          HEAP16[$514>>1] = 2576;
         }
        }
       }
       else if ((label|0) == 132) {
        label = 0;
        $1272 = HEAP32[$pOp>>2]|0;
        $1273 = ((($1272)) + 16|0);
        $1274 = HEAP32[$1273>>2]|0;
        HEAP32[$pCtx4>>2] = $1274;
        $1275 = HEAP32[$pOp>>2]|0;
        $1276 = ((($1275)) + 12|0);
        $1277 = HEAP32[$1276>>2]|0;
        $1278 = HEAP32[$aMem>>2]|0;
        $1279 = (($1278) + (($1277*40)|0)|0);
        HEAP32[$pOut>>2] = $1279;
        $1280 = HEAP32[$pCtx4>>2]|0;
        $1281 = HEAP32[$1280>>2]|0;
        $1282 = HEAP32[$pOut>>2]|0;
        $1283 = ($1281|0)!=($1282|0);
        L988: do {
         if ($1283) {
          $1284 = HEAP32[$pOut>>2]|0;
          $1285 = HEAP32[$pCtx4>>2]|0;
          HEAP32[$1285>>2] = $1284;
          $1286 = HEAP32[$pCtx4>>2]|0;
          $1287 = ((($1286)) + 26|0);
          $1288 = HEAP8[$1287>>0]|0;
          $1289 = $1288&255;
          $1290 = (($1289) - 1)|0;
          HEAP32[$i3>>2] = $1290;
          while(1) {
           $1291 = HEAP32[$i3>>2]|0;
           $1292 = ($1291|0)>=(0);
           if (!($1292)) {
            break L988;
           }
           $1293 = HEAP32[$pOp>>2]|0;
           $1294 = ((($1293)) + 8|0);
           $1295 = HEAP32[$1294>>2]|0;
           $1296 = HEAP32[$i3>>2]|0;
           $1297 = (($1295) + ($1296))|0;
           $1298 = HEAP32[$aMem>>2]|0;
           $1299 = (($1298) + (($1297*40)|0)|0);
           $1300 = HEAP32[$i3>>2]|0;
           $1301 = HEAP32[$pCtx4>>2]|0;
           $1302 = ((($1301)) + 28|0);
           $1303 = (($1302) + ($1300<<2)|0);
           HEAP32[$1303>>2] = $1299;
           $1304 = HEAP32[$i3>>2]|0;
           $1305 = (($1304) + -1)|0;
           HEAP32[$i3>>2] = $1305;
          }
         }
        } while(0);
        $1306 = HEAP32[$pCtx4>>2]|0;
        $1307 = HEAP32[$1306>>2]|0;
        $1308 = ((($1307)) + 8|0);
        $1309 = HEAP16[$1308>>1]|0;
        $1310 = $1309&65535;
        $1311 = $1310 & -49664;
        $1312 = $1311 | 1;
        $1313 = $1312&65535;
        $1314 = HEAP32[$pCtx4>>2]|0;
        $1315 = HEAP32[$1314>>2]|0;
        $1316 = ((($1315)) + 8|0);
        HEAP16[$1316>>1] = $1313;
        $1317 = HEAP32[$pCtx4>>2]|0;
        $1318 = ((($1317)) + 25|0);
        HEAP8[$1318>>0] = 0;
        $1319 = $lastRowid;
        $1320 = $1319;
        $1321 = HEAP32[$1320>>2]|0;
        $1322 = (($1319) + 4)|0;
        $1323 = $1322;
        $1324 = HEAP32[$1323>>2]|0;
        $1325 = HEAP32[$db>>2]|0;
        $1326 = ((($1325)) + 32|0);
        $1327 = $1326;
        $1328 = $1327;
        HEAP32[$1328>>2] = $1321;
        $1329 = (($1327) + 4)|0;
        $1330 = $1329;
        HEAP32[$1330>>2] = $1324;
        $1331 = HEAP32[$pCtx4>>2]|0;
        $1332 = ((($1331)) + 4|0);
        $1333 = HEAP32[$1332>>2]|0;
        $1334 = ((($1333)) + 12|0);
        $1335 = HEAP32[$1334>>2]|0;
        $1336 = HEAP32[$pCtx4>>2]|0;
        $1337 = HEAP32[$pCtx4>>2]|0;
        $1338 = ((($1337)) + 26|0);
        $1339 = HEAP8[$1338>>0]|0;
        $1340 = $1339&255;
        $1341 = HEAP32[$pCtx4>>2]|0;
        $1342 = ((($1341)) + 28|0);
        FUNCTION_TABLE_viii[$1335 & 255]($1336,$1340,$1342);
        $1343 = HEAP32[$db>>2]|0;
        $1344 = ((($1343)) + 32|0);
        $1345 = $1344;
        $1346 = $1345;
        $1347 = HEAP32[$1346>>2]|0;
        $1348 = (($1345) + 4)|0;
        $1349 = $1348;
        $1350 = HEAP32[$1349>>2]|0;
        $1351 = $lastRowid;
        $1352 = $1351;
        HEAP32[$1352>>2] = $1347;
        $1353 = (($1351) + 4)|0;
        $1354 = $1353;
        HEAP32[$1354>>2] = $1350;
        $1355 = HEAP32[$pCtx4>>2]|0;
        $1356 = ((($1355)) + 25|0);
        $1357 = HEAP8[$1356>>0]|0;
        $1358 = ($1357<<24>>24)!=(0);
        if ($1358) {
         $1359 = HEAP32[$pCtx4>>2]|0;
         $1360 = ((($1359)) + 20|0);
         $1361 = HEAP32[$1360>>2]|0;
         $1362 = ($1361|0)!=(0);
         if ($1362) {
          $1363 = HEAP32[$0>>2]|0;
          $1364 = HEAP32[$pCtx4>>2]|0;
          $1365 = HEAP32[$1364>>2]|0;
          $1366 = (_sqlite3_value_text($1365)|0);
          HEAP32[$vararg_buffer43>>2] = $1366;
          _sqlite3VdbeError($1363,17521,$vararg_buffer43);
          $1367 = HEAP32[$pCtx4>>2]|0;
          $1368 = ((($1367)) + 20|0);
          $1369 = HEAP32[$1368>>2]|0;
          HEAP32[$rc>>2] = $1369;
         }
         $1370 = HEAP32[$0>>2]|0;
         $1371 = HEAP32[$pCtx4>>2]|0;
         $1372 = ((($1371)) + 16|0);
         $1373 = HEAP32[$1372>>2]|0;
         $1374 = HEAP32[$pOp>>2]|0;
         $1375 = ((($1374)) + 4|0);
         $1376 = HEAP32[$1375>>2]|0;
         _sqlite3VdbeDeleteAuxData($1370,$1373,$1376);
        }
        $1377 = HEAP32[$pOut>>2]|0;
        $1378 = ((($1377)) + 8|0);
        $1379 = HEAP16[$1378>>1]|0;
        $1380 = $1379&65535;
        $1381 = $1380 & 18;
        $1382 = ($1381|0)!=(0);
        if ($1382) {
         $1383 = HEAP32[$pCtx4>>2]|0;
         $1384 = HEAP32[$1383>>2]|0;
         $1385 = HEAP8[$encoding>>0]|0;
         $1386 = $1385&255;
         (_sqlite3VdbeChangeEncoding($1384,$1386)|0);
         $1387 = HEAP32[$pCtx4>>2]|0;
         $1388 = HEAP32[$1387>>2]|0;
         $1389 = (_sqlite3VdbeMemTooBig($1388)|0);
         $1390 = ($1389|0)!=(0);
         if ($1390) {
          label = 859;
          break L8;
         }
        }
       }
       else if ((label|0) == 438) {
        label = 0;
        $3588 = HEAP32[$0>>2]|0;
        $3589 = ((($3588)) + 88|0);
        $3590 = HEAP16[$3589>>1]|0;
        $3591 = ($3590&65535) >>> 3;
        $3592 = $3591 & 1;
        $3593 = $3592&65535;
        $3594 = ($3593|0)!=(0);
        if ($3594) {
         HEAP32[$rc>>2] = 516;
         break;
        }
        HEAP32[$nField20>>2] = 0;
        HEAP32[$pKeyInfo21>>2] = 0;
        $3595 = HEAP32[$pOp>>2]|0;
        $3596 = ((($3595)) + 8|0);
        $3597 = HEAP32[$3596>>2]|0;
        HEAP32[$p222>>2] = $3597;
        $3598 = HEAP32[$pOp>>2]|0;
        $3599 = ((($3598)) + 12|0);
        $3600 = HEAP32[$3599>>2]|0;
        HEAP32[$iDb23>>2] = $3600;
        $3601 = HEAP32[$iDb23>>2]|0;
        $3602 = HEAP32[$db>>2]|0;
        $3603 = ((($3602)) + 16|0);
        $3604 = HEAP32[$3603>>2]|0;
        $3605 = (($3604) + ($3601<<4)|0);
        HEAP32[$pDb24>>2] = $3605;
        $3606 = HEAP32[$pDb24>>2]|0;
        $3607 = ((($3606)) + 4|0);
        $3608 = HEAP32[$3607>>2]|0;
        HEAP32[$pX>>2] = $3608;
        $3609 = HEAP32[$pOp>>2]|0;
        $3610 = HEAP8[$3609>>0]|0;
        $3611 = $3610&255;
        $3612 = ($3611|0)==(55);
        if ($3612) {
         $3613 = HEAP32[$pOp>>2]|0;
         $3614 = ((($3613)) + 3|0);
         $3615 = HEAP8[$3614>>0]|0;
         $3616 = $3615&255;
         $3617 = $3616 & 8;
         $3618 = 4 | $3617;
         HEAP32[$wrFlag>>2] = $3618;
         $3619 = HEAP32[$pDb24>>2]|0;
         $3620 = ((($3619)) + 12|0);
         $3621 = HEAP32[$3620>>2]|0;
         $3622 = ((($3621)) + 76|0);
         $3623 = HEAP8[$3622>>0]|0;
         $3624 = $3623&255;
         $3625 = HEAP32[$0>>2]|0;
         $3626 = ((($3625)) + 87|0);
         $3627 = HEAP8[$3626>>0]|0;
         $3628 = $3627&255;
         $3629 = ($3624|0)<($3628|0);
         if ($3629) {
          $3630 = HEAP32[$pDb24>>2]|0;
          $3631 = ((($3630)) + 12|0);
          $3632 = HEAP32[$3631>>2]|0;
          $3633 = ((($3632)) + 76|0);
          $3634 = HEAP8[$3633>>0]|0;
          $3635 = HEAP32[$0>>2]|0;
          $3636 = ((($3635)) + 87|0);
          HEAP8[$3636>>0] = $3634;
         }
        } else {
         HEAP32[$wrFlag>>2] = 0;
        }
        $3637 = HEAP32[$pOp>>2]|0;
        $3638 = ((($3637)) + 3|0);
        $3639 = HEAP8[$3638>>0]|0;
        $3640 = $3639&255;
        $3641 = $3640 & 16;
        $3642 = ($3641|0)!=(0);
        if ($3642) {
         $3643 = HEAP32[$p222>>2]|0;
         $3644 = HEAP32[$aMem>>2]|0;
         $3645 = (($3644) + (($3643*40)|0)|0);
         HEAP32[$pIn2>>2] = $3645;
         $3646 = HEAP32[$pIn2>>2]|0;
         (_sqlite3VdbeMemIntegerify($3646)|0);
         $3647 = HEAP32[$pIn2>>2]|0;
         $3648 = $3647;
         $3649 = $3648;
         $3650 = HEAP32[$3649>>2]|0;
         $3651 = (($3648) + 4)|0;
         $3652 = $3651;
         $3653 = HEAP32[$3652>>2]|0;
         HEAP32[$p222>>2] = $3650;
         $3654 = HEAP32[$p222>>2]|0;
         $3655 = ($3654|0)<(2);
         if ($3655) {
          label = 446;
          break L8;
         }
        }
        $3657 = HEAP32[$pOp>>2]|0;
        $3658 = ((($3657)) + 1|0);
        $3659 = HEAP8[$3658>>0]|0;
        $3660 = $3659 << 24 >> 24;
        $3661 = ($3660|0)==(-6);
        $3662 = HEAP32[$pOp>>2]|0;
        if ($3661) {
         $3663 = ((($3662)) + 16|0);
         $3664 = HEAP32[$3663>>2]|0;
         HEAP32[$pKeyInfo21>>2] = $3664;
         $3665 = HEAP32[$pKeyInfo21>>2]|0;
         $3666 = ((($3665)) + 6|0);
         $3667 = HEAP16[$3666>>1]|0;
         $3668 = $3667&65535;
         $3669 = HEAP32[$pKeyInfo21>>2]|0;
         $3670 = ((($3669)) + 8|0);
         $3671 = HEAP16[$3670>>1]|0;
         $3672 = $3671&65535;
         $3673 = (($3668) + ($3672))|0;
         HEAP32[$nField20>>2] = $3673;
        } else {
         $3674 = ((($3662)) + 1|0);
         $3675 = HEAP8[$3674>>0]|0;
         $3676 = $3675 << 24 >> 24;
         $3677 = ($3676|0)==(-14);
         if ($3677) {
          $3678 = HEAP32[$pOp>>2]|0;
          $3679 = ((($3678)) + 16|0);
          $3680 = HEAP32[$3679>>2]|0;
          HEAP32[$nField20>>2] = $3680;
         }
        }
        $3681 = HEAP32[$0>>2]|0;
        $3682 = HEAP32[$pOp>>2]|0;
        $3683 = ((($3682)) + 4|0);
        $3684 = HEAP32[$3683>>2]|0;
        $3685 = HEAP32[$nField20>>2]|0;
        $3686 = HEAP32[$iDb23>>2]|0;
        $3687 = (_allocateCursor($3681,$3684,$3685,$3686,0)|0);
        HEAP32[$pCur>>2] = $3687;
        $3688 = HEAP32[$pCur>>2]|0;
        $3689 = ($3688|0)==(0|0);
        if ($3689) {
         label = 860;
         break L1;
        }
        $3690 = HEAP32[$pCur>>2]|0;
        $3691 = ((($3690)) + 2|0);
        HEAP8[$3691>>0] = 1;
        $3692 = HEAP32[$pCur>>2]|0;
        $3693 = ((($3692)) + 5|0);
        $3694 = HEAP8[$3693>>0]|0;
        $3695 = $3694 & -5;
        $3696 = $3695 | 4;
        HEAP8[$3693>>0] = $3696;
        $3697 = HEAP32[$p222>>2]|0;
        $3698 = HEAP32[$pCur>>2]|0;
        $3699 = ((($3698)) + 8|0);
        HEAP32[$3699>>2] = $3697;
        $3700 = HEAP32[$pX>>2]|0;
        $3701 = HEAP32[$p222>>2]|0;
        $3702 = HEAP32[$wrFlag>>2]|0;
        $3703 = HEAP32[$pKeyInfo21>>2]|0;
        $3704 = HEAP32[$pCur>>2]|0;
        $3705 = ((($3704)) + 16|0);
        $3706 = HEAP32[$3705>>2]|0;
        $3707 = (_sqlite3BtreeCursor($3700,$3701,$3702,$3703,$3706)|0);
        HEAP32[$rc>>2] = $3707;
        $3708 = HEAP32[$pKeyInfo21>>2]|0;
        $3709 = HEAP32[$pCur>>2]|0;
        $3710 = ((($3709)) + 24|0);
        HEAP32[$3710>>2] = $3708;
        $3711 = HEAP32[$pOp>>2]|0;
        $3712 = ((($3711)) + 1|0);
        $3713 = HEAP8[$3712>>0]|0;
        $3714 = $3713 << 24 >> 24;
        $3715 = ($3714|0)!=(-6);
        $3716 = $3715&1;
        $3717 = $3716&255;
        $3718 = HEAP32[$pCur>>2]|0;
        $3719 = ((($3718)) + 4|0);
        HEAP8[$3719>>0] = $3717;
        label = 453;
       }
       else if ((label|0) == 609) {
        label = 0;
        $5048 = HEAP32[$pOp>>2]|0;
        $5049 = ((($5048)) + 4|0);
        $5050 = HEAP32[$5049>>2]|0;
        $5051 = HEAP32[$0>>2]|0;
        $5052 = ((($5051)) + 56|0);
        $5053 = HEAP32[$5052>>2]|0;
        $5054 = (($5053) + ($5050<<2)|0);
        $5055 = HEAP32[$5054>>2]|0;
        HEAP32[$pC62>>2] = $5055;
        HEAP32[$res64>>2] = 1;
        $5056 = HEAP32[$pC62>>2]|0;
        $5057 = HEAP8[$5056>>0]|0;
        $5058 = $5057&255;
        $5059 = ($5058|0)==(1);
        $5060 = HEAP32[$pC62>>2]|0;
        if ($5059) {
         $5061 = (_sqlite3VdbeSorterRewind($5060,$res64)|0);
         HEAP32[$rc>>2] = $5061;
        } else {
         $5062 = ((($5060)) + 16|0);
         $5063 = HEAP32[$5062>>2]|0;
         HEAP32[$pCrsr63>>2] = $5063;
         $5064 = HEAP32[$pCrsr63>>2]|0;
         $5065 = (_sqlite3BtreeFirst($5064,$res64)|0);
         HEAP32[$rc>>2] = $5065;
         $5066 = HEAP32[$pC62>>2]|0;
         $5067 = ((($5066)) + 3|0);
         HEAP8[$5067>>0] = 0;
         $5068 = HEAP32[$pC62>>2]|0;
         $5069 = ((($5068)) + 56|0);
         HEAP32[$5069>>2] = 0;
        }
        $5070 = HEAP32[$res64>>2]|0;
        $5071 = $5070&255;
        $5072 = HEAP32[$pC62>>2]|0;
        $5073 = ((($5072)) + 2|0);
        HEAP8[$5073>>0] = $5071;
        $5074 = HEAP32[$res64>>2]|0;
        $5075 = ($5074|0)!=(0);
        if ($5075) {
         label = 15;
        }
       }
       else if ((label|0) == 615) {
        label = 0;
        $5096 = HEAP32[$pOp>>2]|0;
        $5097 = ((($5096)) + 4|0);
        $5098 = HEAP32[$5097>>2]|0;
        $5099 = HEAP32[$0>>2]|0;
        $5100 = ((($5099)) + 56|0);
        $5101 = HEAP32[$5100>>2]|0;
        $5102 = (($5101) + ($5098<<2)|0);
        $5103 = HEAP32[$5102>>2]|0;
        HEAP32[$pC65>>2] = $5103;
        $5104 = HEAP32[$pOp>>2]|0;
        $5105 = ((($5104)) + 12|0);
        $5106 = HEAP32[$5105>>2]|0;
        HEAP32[$res66>>2] = $5106;
        $5107 = HEAP32[$pOp>>2]|0;
        $5108 = ((($5107)) + 16|0);
        $5109 = HEAP32[$5108>>2]|0;
        $5110 = HEAP32[$pC65>>2]|0;
        $5111 = ((($5110)) + 16|0);
        $5112 = HEAP32[$5111>>2]|0;
        $5113 = (FUNCTION_TABLE_iii[$5109 & 255]($5112,$res66)|0);
        HEAP32[$rc>>2] = $5113;
        label = 616;
       }
       else if ((label|0) == 740) {
        label = 0;
        $6446 = HEAP32[$pOp>>2]|0;
        $6447 = ((($6446)) + 16|0);
        $6448 = HEAP32[$6447>>2]|0;
        HEAP32[$pCtx92>>2] = $6448;
        $6449 = HEAP32[$pOp>>2]|0;
        $6450 = ((($6449)) + 12|0);
        $6451 = HEAP32[$6450>>2]|0;
        $6452 = HEAP32[$aMem>>2]|0;
        $6453 = (($6452) + (($6451*40)|0)|0);
        HEAP32[$pMem93>>2] = $6453;
        $6454 = HEAP32[$pCtx92>>2]|0;
        $6455 = ((($6454)) + 8|0);
        $6456 = HEAP32[$6455>>2]|0;
        $6457 = HEAP32[$pMem93>>2]|0;
        $6458 = ($6456|0)!=($6457|0);
        L1026: do {
         if ($6458) {
          $6459 = HEAP32[$pMem93>>2]|0;
          $6460 = HEAP32[$pCtx92>>2]|0;
          $6461 = ((($6460)) + 8|0);
          HEAP32[$6461>>2] = $6459;
          $6462 = HEAP32[$pCtx92>>2]|0;
          $6463 = ((($6462)) + 26|0);
          $6464 = HEAP8[$6463>>0]|0;
          $6465 = $6464&255;
          $6466 = (($6465) - 1)|0;
          HEAP32[$i91>>2] = $6466;
          while(1) {
           $6467 = HEAP32[$i91>>2]|0;
           $6468 = ($6467|0)>=(0);
           if (!($6468)) {
            break L1026;
           }
           $6469 = HEAP32[$pOp>>2]|0;
           $6470 = ((($6469)) + 8|0);
           $6471 = HEAP32[$6470>>2]|0;
           $6472 = HEAP32[$i91>>2]|0;
           $6473 = (($6471) + ($6472))|0;
           $6474 = HEAP32[$aMem>>2]|0;
           $6475 = (($6474) + (($6473*40)|0)|0);
           $6476 = HEAP32[$i91>>2]|0;
           $6477 = HEAP32[$pCtx92>>2]|0;
           $6478 = ((($6477)) + 28|0);
           $6479 = (($6478) + ($6476<<2)|0);
           HEAP32[$6479>>2] = $6475;
           $6480 = HEAP32[$i91>>2]|0;
           $6481 = (($6480) + -1)|0;
           HEAP32[$i91>>2] = $6481;
          }
         }
        } while(0);
        $6482 = HEAP32[$pMem93>>2]|0;
        $6483 = ((($6482)) + 12|0);
        $6484 = HEAP32[$6483>>2]|0;
        $6485 = (($6484) + 1)|0;
        HEAP32[$6483>>2] = $6485;
        $6486 = HEAP32[$db>>2]|0;
        _sqlite3VdbeMemInit($t94,$6486,1);
        $6487 = HEAP32[$pCtx92>>2]|0;
        HEAP32[$6487>>2] = $t94;
        $6488 = HEAP32[$pCtx92>>2]|0;
        $6489 = ((($6488)) + 25|0);
        HEAP8[$6489>>0] = 0;
        $6490 = HEAP32[$pCtx92>>2]|0;
        $6491 = ((($6490)) + 24|0);
        HEAP8[$6491>>0] = 0;
        $6492 = HEAP32[$pCtx92>>2]|0;
        $6493 = ((($6492)) + 4|0);
        $6494 = HEAP32[$6493>>2]|0;
        $6495 = ((($6494)) + 12|0);
        $6496 = HEAP32[$6495>>2]|0;
        $6497 = HEAP32[$pCtx92>>2]|0;
        $6498 = HEAP32[$pCtx92>>2]|0;
        $6499 = ((($6498)) + 26|0);
        $6500 = HEAP8[$6499>>0]|0;
        $6501 = $6500&255;
        $6502 = HEAP32[$pCtx92>>2]|0;
        $6503 = ((($6502)) + 28|0);
        FUNCTION_TABLE_viii[$6496 & 255]($6497,$6501,$6503);
        $6504 = HEAP32[$pCtx92>>2]|0;
        $6505 = ((($6504)) + 25|0);
        $6506 = HEAP8[$6505>>0]|0;
        $6507 = ($6506<<24>>24)!=(0);
        if ($6507) {
         $6508 = HEAP32[$pCtx92>>2]|0;
         $6509 = ((($6508)) + 20|0);
         $6510 = HEAP32[$6509>>2]|0;
         $6511 = ($6510|0)!=(0);
         if ($6511) {
          $6512 = HEAP32[$0>>2]|0;
          $6513 = (_sqlite3_value_text($t94)|0);
          HEAP32[$vararg_buffer64>>2] = $6513;
          _sqlite3VdbeError($6512,17521,$vararg_buffer64);
          $6514 = HEAP32[$pCtx92>>2]|0;
          $6515 = ((($6514)) + 20|0);
          $6516 = HEAP32[$6515>>2]|0;
          HEAP32[$rc>>2] = $6516;
         }
         _sqlite3VdbeMemRelease($t94);
        }
        $6517 = HEAP32[$pCtx92>>2]|0;
        $6518 = ((($6517)) + 24|0);
        $6519 = HEAP8[$6518>>0]|0;
        $6520 = ($6519<<24>>24)!=(0);
        if ($6520) {
         $6521 = HEAP32[$pOp>>2]|0;
         $6522 = ((($6521)) + -20|0);
         $6523 = ((($6522)) + 4|0);
         $6524 = HEAP32[$6523>>2]|0;
         HEAP32[$i91>>2] = $6524;
         $6525 = HEAP32[$i91>>2]|0;
         $6526 = ($6525|0)!=(0);
         if ($6526) {
          $6527 = HEAP32[$i91>>2]|0;
          $6528 = HEAP32[$aMem>>2]|0;
          $6529 = (($6528) + (($6527*40)|0)|0);
          _sqlite3VdbeMemSetInt64($6529,1,0);
         }
        }
       }
      } while(0);
      do {
       if ((label|0) == 15) {
        label = 0;
        $132 = HEAP32[$pOp>>2]|0;
        $133 = ((($132)) + 8|0);
        $134 = HEAP32[$133>>2]|0;
        $135 = (($134) - 1)|0;
        $136 = HEAP32[$aOp>>2]|0;
        $137 = (($136) + (($135*20)|0)|0);
        HEAP32[$pOp>>2] = $137;
       }
       else if ((label|0) == 453) {
        label = 0;
        $3720 = HEAP32[$pCur>>2]|0;
        $3721 = ((($3720)) + 16|0);
        $3722 = HEAP32[$3721>>2]|0;
        $3723 = HEAP32[$pOp>>2]|0;
        $3724 = ((($3723)) + 3|0);
        $3725 = HEAP8[$3724>>0]|0;
        $3726 = $3725&255;
        $3727 = $3726 & 3;
        _sqlite3BtreeCursorHintFlags($3722,$3727);
       }
       else if ((label|0) == 616) {
        label = 0;
        $5114 = HEAP32[$pC65>>2]|0;
        $5115 = ((($5114)) + 56|0);
        HEAP32[$5115>>2] = 0;
        $5116 = HEAP32[$res66>>2]|0;
        $5117 = ($5116|0)==(0);
        $5118 = HEAP32[$pC65>>2]|0;
        $5119 = ((($5118)) + 2|0);
        if ($5117) {
         HEAP8[$5119>>0] = 0;
         $5120 = HEAP32[$pOp>>2]|0;
         $5121 = ((($5120)) + 3|0);
         $5122 = HEAP8[$5121>>0]|0;
         $5123 = $5122&255;
         $5124 = HEAP32[$0>>2]|0;
         $5125 = ((($5124)) + 108|0);
         $5126 = (($5125) + ($5123<<2)|0);
         $5127 = HEAP32[$5126>>2]|0;
         $5128 = (($5127) + 1)|0;
         HEAP32[$5126>>2] = $5128;
         label = 8;
         break;
        } else {
         HEAP8[$5119>>0] = 1;
         label = 9;
         break;
        }
       }
      } while(0);
      if ((label|0) == 8) {
       label = 0;
       $76 = HEAP32[$pOp>>2]|0;
       $77 = ((($76)) + 8|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = (($78) - 1)|0;
       $80 = HEAP32[$aOp>>2]|0;
       $81 = (($80) + (($79*20)|0)|0);
       HEAP32[$pOp>>2] = $81;
       label = 9;
      }
      if ((label|0) == 9) {
       label = 0;
       $82 = HEAP32[$db>>2]|0;
       $83 = ((($82)) + 240|0);
       $84 = HEAP32[$83>>2]|0;
       $85 = ($84|0)!=(0);
       if ($85) {
        break L3;
       }
       $86 = HEAP32[$db>>2]|0;
       $87 = ((($86)) + 296|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = ($88|0)!=(0|0);
       if ($89) {
        $90 = HEAP32[$nVmStep>>2]|0;
        $91 = HEAP32[$nProgressLimit>>2]|0;
        $92 = ($90>>>0)>=($91>>>0);
        if ($92) {
         $93 = HEAP32[$nVmStep>>2]|0;
         $94 = HEAP32[$db>>2]|0;
         $95 = ((($94)) + 304|0);
         $96 = HEAP32[$95>>2]|0;
         $97 = (($93) + ($96))|0;
         $98 = HEAP32[$nVmStep>>2]|0;
         $99 = HEAP32[$db>>2]|0;
         $100 = ((($99)) + 304|0);
         $101 = HEAP32[$100>>2]|0;
         $102 = (($98>>>0) % ($101>>>0))&-1;
         $103 = (($97) - ($102))|0;
         HEAP32[$nProgressLimit>>2] = $103;
         $104 = HEAP32[$db>>2]|0;
         $105 = ((($104)) + 296|0);
         $106 = HEAP32[$105>>2]|0;
         $107 = HEAP32[$db>>2]|0;
         $108 = ((($107)) + 300|0);
         $109 = HEAP32[$108>>2]|0;
         $110 = (FUNCTION_TABLE_ii[$106 & 255]($109)|0);
         $111 = ($110|0)!=(0);
         if ($111) {
          label = 13;
          break;
         }
        }
       }
      }
      $7233 = HEAP32[$pOp>>2]|0;
      $7234 = ((($7233)) + 20|0);
      HEAP32[$pOp>>2] = $7234;
     }
     switch (label|0) {
      case 13: {
       HEAP32[$rc>>2] = 9;
       break L1;
       break;
      }
      case 26: {
       $304 = HEAP32[$pOp>>2]|0;
       $305 = ((($304)) + 4|0);
       $306 = HEAP32[$305>>2]|0;
       $307 = HEAP32[$0>>2]|0;
       $308 = ((($307)) + 80|0);
       HEAP32[$308>>2] = $306;
       $309 = HEAP32[$pOp>>2]|0;
       $310 = ((($309)) + 8|0);
       $311 = HEAP32[$310>>2]|0;
       $312 = $311&255;
       $313 = HEAP32[$0>>2]|0;
       $314 = ((($313)) + 86|0);
       HEAP8[$314>>0] = $312;
       $315 = HEAP32[$pcx>>2]|0;
       $316 = HEAP32[$0>>2]|0;
       $317 = ((($316)) + 76|0);
       HEAP32[$317>>2] = $315;
       $318 = HEAP32[$0>>2]|0;
       $319 = ((($318)) + 80|0);
       $320 = HEAP32[$319>>2]|0;
       $321 = ($320|0)!=(0);
       if ($321) {
        $322 = HEAP32[$pOp>>2]|0;
        $323 = ((($322)) + 3|0);
        $324 = HEAP8[$323>>0]|0;
        $325 = ($324<<24>>24)!=(0);
        if ($325) {
         $326 = HEAP32[$pOp>>2]|0;
         $327 = ((($326)) + 3|0);
         $328 = HEAP8[$327>>0]|0;
         $329 = $328&255;
         $330 = (($329) - 1)|0;
         $331 = (2420 + ($330<<2)|0);
         $332 = HEAP32[$331>>2]|0;
         HEAP32[$zType>>2] = $332;
        } else {
         HEAP32[$zType>>2] = 0;
        }
        HEAP32[$zLogFmt>>2] = 20822;
        $333 = HEAP32[$zType>>2]|0;
        $334 = ($333|0)!=(0|0);
        if ($334) {
         $335 = HEAP32[$pOp>>2]|0;
         $336 = ((($335)) + 16|0);
         $337 = HEAP32[$336>>2]|0;
         $338 = ($337|0)!=(0|0);
         if ($338) {
          $339 = HEAP32[$0>>2]|0;
          $340 = HEAP32[$zType>>2]|0;
          $341 = HEAP32[$pOp>>2]|0;
          $342 = ((($341)) + 16|0);
          $343 = HEAP32[$342>>2]|0;
          HEAP32[$vararg_buffer>>2] = $340;
          $vararg_ptr31 = ((($vararg_buffer)) + 4|0);
          HEAP32[$vararg_ptr31>>2] = $343;
          _sqlite3VdbeError($339,20846,$vararg_buffer);
         } else {
          label = 33;
         }
        } else {
         label = 33;
        }
        do {
         if ((label|0) == 33) {
          $344 = HEAP32[$pOp>>2]|0;
          $345 = ((($344)) + 16|0);
          $346 = HEAP32[$345>>2]|0;
          $347 = ($346|0)!=(0|0);
          $348 = HEAP32[$0>>2]|0;
          if ($347) {
           $349 = HEAP32[$pOp>>2]|0;
           $350 = ((($349)) + 16|0);
           $351 = HEAP32[$350>>2]|0;
           HEAP32[$vararg_buffer32>>2] = $351;
           _sqlite3VdbeError($348,17521,$vararg_buffer32);
           break;
          } else {
           $352 = HEAP32[$zType>>2]|0;
           HEAP32[$vararg_buffer35>>2] = $352;
           _sqlite3VdbeError($348,20871,$vararg_buffer35);
           break;
          }
         }
        } while(0);
        $353 = HEAP32[$pOp>>2]|0;
        $354 = ((($353)) + 4|0);
        $355 = HEAP32[$354>>2]|0;
        $356 = HEAP32[$zLogFmt>>2]|0;
        $357 = HEAP32[$pcx>>2]|0;
        $358 = HEAP32[$0>>2]|0;
        $359 = ((($358)) + 168|0);
        $360 = HEAP32[$359>>2]|0;
        $361 = HEAP32[$0>>2]|0;
        $362 = ((($361)) + 44|0);
        $363 = HEAP32[$362>>2]|0;
        HEAP32[$vararg_buffer38>>2] = $357;
        $vararg_ptr41 = ((($vararg_buffer38)) + 4|0);
        HEAP32[$vararg_ptr41>>2] = $360;
        $vararg_ptr42 = ((($vararg_buffer38)) + 8|0);
        HEAP32[$vararg_ptr42>>2] = $363;
        _sqlite3_log($355,$356,$vararg_buffer38);
       }
       $364 = HEAP32[$0>>2]|0;
       $365 = (_sqlite3VdbeHalt($364)|0);
       HEAP32[$rc>>2] = $365;
       $366 = HEAP32[$rc>>2]|0;
       $367 = ($366|0)==(5);
       if ($367) {
        HEAP32[$rc>>2] = 5;
        $368 = HEAP32[$0>>2]|0;
        $369 = ((($368)) + 80|0);
        HEAP32[$369>>2] = 5;
        $7262 = $lastRowid;
        $7263 = $7262;
        $7264 = HEAP32[$7263>>2]|0;
        $7265 = (($7262) + 4)|0;
        $7266 = $7265;
        $7267 = HEAP32[$7266>>2]|0;
        $7268 = HEAP32[$db>>2]|0;
        $7269 = ((($7268)) + 32|0);
        $7270 = $7269;
        $7271 = $7270;
        HEAP32[$7271>>2] = $7264;
        $7272 = (($7270) + 4)|0;
        $7273 = $7272;
        HEAP32[$7273>>2] = $7267;
        $7274 = HEAP32[$nVmStep>>2]|0;
        $7275 = HEAP32[$0>>2]|0;
        $7276 = ((($7275)) + 108|0);
        $7277 = ((($7276)) + 16|0);
        $7278 = HEAP32[$7277>>2]|0;
        $7279 = (($7278) + ($7274))|0;
        HEAP32[$7277>>2] = $7279;
        $7280 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7280|0);
       } else {
        $370 = HEAP32[$0>>2]|0;
        $371 = ((($370)) + 80|0);
        $372 = HEAP32[$371>>2]|0;
        $373 = ($372|0)!=(0);
        $374 = $373 ? 1 : 101;
        HEAP32[$rc>>2] = $374;
        $7262 = $lastRowid;
        $7263 = $7262;
        $7264 = HEAP32[$7263>>2]|0;
        $7265 = (($7262) + 4)|0;
        $7266 = $7265;
        $7267 = HEAP32[$7266>>2]|0;
        $7268 = HEAP32[$db>>2]|0;
        $7269 = ((($7268)) + 32|0);
        $7270 = $7269;
        $7271 = $7270;
        HEAP32[$7271>>2] = $7264;
        $7272 = (($7270) + 4)|0;
        $7273 = $7272;
        HEAP32[$7273>>2] = $7267;
        $7274 = HEAP32[$nVmStep>>2]|0;
        $7275 = HEAP32[$0>>2]|0;
        $7276 = ((($7275)) + 108|0);
        $7277 = ((($7276)) + 16|0);
        $7278 = HEAP32[$7277>>2]|0;
        $7279 = (($7278) + ($7274))|0;
        HEAP32[$7277>>2] = $7279;
        $7280 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7280|0);
       }
       break;
      }
      case 74: {
       HEAP32[$rc>>2] = 9;
       break L1;
       break;
      }
      case 77: {
       $702 = HEAP32[$0>>2]|0;
       $703 = ((($702)) + 72|0);
       $704 = HEAP32[$703>>2]|0;
       $705 = (($704) + 2)|0;
       $706 = $705 | 1;
       $707 = HEAP32[$0>>2]|0;
       $708 = ((($707)) + 72|0);
       HEAP32[$708>>2] = $706;
       $709 = HEAP32[$pOp>>2]|0;
       $710 = ((($709)) + 4|0);
       $711 = HEAP32[$710>>2]|0;
       $712 = HEAP32[$aMem>>2]|0;
       $713 = (($712) + (($711*40)|0)|0);
       $714 = HEAP32[$0>>2]|0;
       $715 = ((($714)) + 20|0);
       HEAP32[$715>>2] = $713;
       HEAP32[$pMem>>2] = $713;
       HEAP32[$i>>2] = 0;
       while(1) {
        $716 = HEAP32[$i>>2]|0;
        $717 = HEAP32[$pOp>>2]|0;
        $718 = ((($717)) + 8|0);
        $719 = HEAP32[$718>>2]|0;
        $720 = ($716|0)<($719|0);
        if (!($720)) {
         break;
        }
        $721 = HEAP32[$i>>2]|0;
        $722 = HEAP32[$pMem>>2]|0;
        $723 = (($722) + (($721*40)|0)|0);
        $724 = ((($723)) + 8|0);
        $725 = HEAP16[$724>>1]|0;
        $726 = $725&65535;
        $727 = $726 & 4096;
        $728 = ($727|0)!=(0);
        if ($728) {
         $729 = HEAP32[$i>>2]|0;
         $730 = HEAP32[$pMem>>2]|0;
         $731 = (($730) + (($729*40)|0)|0);
         $732 = (_sqlite3VdbeMemMakeWriteable($731)|0);
         $733 = ($732|0)!=(0);
         if ($733) {
          label = 860;
          break L1;
         }
        }
        $734 = HEAP32[$i>>2]|0;
        $735 = HEAP32[$pMem>>2]|0;
        $736 = (($735) + (($734*40)|0)|0);
        (_sqlite3VdbeMemNulTerminate($736)|0);
        $737 = HEAP32[$i>>2]|0;
        $738 = (($737) + 1)|0;
        HEAP32[$i>>2] = $738;
       }
       $739 = HEAP32[$db>>2]|0;
       $740 = ((($739)) + 65|0);
       $741 = HEAP8[$740>>0]|0;
       $742 = ($741<<24>>24)!=(0);
       if ($742) {
        label = 860;
        break L1;
       }
       $743 = HEAP32[$pOp>>2]|0;
       $744 = HEAP32[$aOp>>2]|0;
       $745 = $743;
       $746 = $744;
       $747 = (($745) - ($746))|0;
       $748 = (($747|0) / 20)&-1;
       $749 = (($748) + 1)|0;
       $750 = HEAP32[$0>>2]|0;
       $751 = ((($750)) + 76|0);
       HEAP32[$751>>2] = $749;
       HEAP32[$rc>>2] = 100;
       $7262 = $lastRowid;
       $7263 = $7262;
       $7264 = HEAP32[$7263>>2]|0;
       $7265 = (($7262) + 4)|0;
       $7266 = $7265;
       $7267 = HEAP32[$7266>>2]|0;
       $7268 = HEAP32[$db>>2]|0;
       $7269 = ((($7268)) + 32|0);
       $7270 = $7269;
       $7271 = $7270;
       HEAP32[$7271>>2] = $7264;
       $7272 = (($7270) + 4)|0;
       $7273 = $7272;
       HEAP32[$7273>>2] = $7267;
       $7274 = HEAP32[$nVmStep>>2]|0;
       $7275 = HEAP32[$0>>2]|0;
       $7276 = ((($7275)) + 108|0);
       $7277 = ((($7276)) + 16|0);
       $7278 = HEAP32[$7277>>2]|0;
       $7279 = (($7278) + ($7274))|0;
       HEAP32[$7277>>2] = $7279;
       $7280 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7280|0);
       break;
      }
      case 165: {
       HEAP32[$rc>>2] = 20;
       label = 861;
       break;
      }
      case 367: {
       $3108 = HEAP32[$pOp>>2]|0;
       $3109 = HEAP32[$aOp>>2]|0;
       $3110 = $3108;
       $3111 = $3109;
       $3112 = (($3110) - ($3111))|0;
       $3113 = (($3112|0) / 20)&-1;
       $3114 = HEAP32[$0>>2]|0;
       $3115 = ((($3114)) + 76|0);
       HEAP32[$3115>>2] = $3113;
       $3116 = HEAP32[$db>>2]|0;
       $3117 = ((($3116)) + 63|0);
       HEAP8[$3117>>0] = 0;
       HEAP32[$rc>>2] = 5;
       $3118 = HEAP32[$0>>2]|0;
       $3119 = ((($3118)) + 80|0);
       HEAP32[$3119>>2] = 5;
       $7262 = $lastRowid;
       $7263 = $7262;
       $7264 = HEAP32[$7263>>2]|0;
       $7265 = (($7262) + 4)|0;
       $7266 = $7265;
       $7267 = HEAP32[$7266>>2]|0;
       $7268 = HEAP32[$db>>2]|0;
       $7269 = ((($7268)) + 32|0);
       $7270 = $7269;
       $7271 = $7270;
       HEAP32[$7271>>2] = $7264;
       $7272 = (($7270) + 4)|0;
       $7273 = $7272;
       HEAP32[$7273>>2] = $7267;
       $7274 = HEAP32[$nVmStep>>2]|0;
       $7275 = HEAP32[$0>>2]|0;
       $7276 = ((($7275)) + 108|0);
       $7277 = ((($7276)) + 16|0);
       $7278 = HEAP32[$7277>>2]|0;
       $7279 = (($7278) + ($7274))|0;
       HEAP32[$7277>>2] = $7279;
       $7280 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7280|0);
       break;
      }
      case 391: {
       $3273 = HEAP32[$db>>2]|0;
       _sqlite3RollbackAll($3273,516);
       $3274 = HEAP32[$db>>2]|0;
       $3275 = ((($3274)) + 63|0);
       HEAP8[$3275>>0] = 1;
       break;
      }
      case 395: {
       $3283 = HEAP32[$0>>2]|0;
       $3284 = (_sqlite3VdbeCheckFk($3283,1)|0);
       HEAP32[$rc>>2] = $3284;
       $3285 = ($3284|0)!=(0);
       if ($3285) {
        $7262 = $lastRowid;
        $7263 = $7262;
        $7264 = HEAP32[$7263>>2]|0;
        $7265 = (($7262) + 4)|0;
        $7266 = $7265;
        $7267 = HEAP32[$7266>>2]|0;
        $7268 = HEAP32[$db>>2]|0;
        $7269 = ((($7268)) + 32|0);
        $7270 = $7269;
        $7271 = $7270;
        HEAP32[$7271>>2] = $7264;
        $7272 = (($7270) + 4)|0;
        $7273 = $7272;
        HEAP32[$7273>>2] = $7267;
        $7274 = HEAP32[$nVmStep>>2]|0;
        $7275 = HEAP32[$0>>2]|0;
        $7276 = ((($7275)) + 108|0);
        $7277 = ((($7276)) + 16|0);
        $7278 = HEAP32[$7277>>2]|0;
        $7279 = (($7278) + ($7274))|0;
        HEAP32[$7277>>2] = $7279;
        $7280 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7280|0);
       } else {
        $3286 = HEAP32[$desiredAutoCommit>>2]|0;
        $3287 = $3286&255;
        $3288 = HEAP32[$db>>2]|0;
        $3289 = ((($3288)) + 63|0);
        HEAP8[$3289>>0] = $3287;
        break;
       }
       break;
      }
      case 407: {
       HEAP32[$rc>>2] = 8;
       label = 861;
       break;
      }
      case 410: {
       $3348 = HEAP32[$pOp>>2]|0;
       $3349 = HEAP32[$aOp>>2]|0;
       $3350 = $3348;
       $3351 = $3349;
       $3352 = (($3350) - ($3351))|0;
       $3353 = (($3352|0) / 20)&-1;
       $3354 = HEAP32[$0>>2]|0;
       $3355 = ((($3354)) + 76|0);
       HEAP32[$3355>>2] = $3353;
       $3356 = HEAP32[$rc>>2]|0;
       $3357 = HEAP32[$0>>2]|0;
       $3358 = ((($3357)) + 80|0);
       HEAP32[$3358>>2] = $3356;
       $7262 = $lastRowid;
       $7263 = $7262;
       $7264 = HEAP32[$7263>>2]|0;
       $7265 = (($7262) + 4)|0;
       $7266 = $7265;
       $7267 = HEAP32[$7266>>2]|0;
       $7268 = HEAP32[$db>>2]|0;
       $7269 = ((($7268)) + 32|0);
       $7270 = $7269;
       $7271 = $7270;
       HEAP32[$7271>>2] = $7264;
       $7272 = (($7270) + 4)|0;
       $7273 = $7272;
       HEAP32[$7273>>2] = $7267;
       $7274 = HEAP32[$nVmStep>>2]|0;
       $7275 = HEAP32[$0>>2]|0;
       $7276 = ((($7275)) + 108|0);
       $7277 = ((($7276)) + 16|0);
       $7278 = HEAP32[$7277>>2]|0;
       $7279 = (($7278) + ($7274))|0;
       HEAP32[$7277>>2] = $7279;
       $7280 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7280|0);
       break;
      }
      case 446: {
       $3656 = (_sqlite3CorruptError(77597)|0);
       HEAP32[$rc>>2] = $3656;
       label = 861;
       break;
      }
      case 542: {
       HEAP32[$rc>>2] = 13;
       label = 861;
       break;
      }
      case 551: {
       HEAP32[$rc>>2] = 13;
       label = 861;
       break;
      }
      case 804: {
       $6825 = HEAP32[$pModule105>>2]|0;
       $6826 = ((($6825)) + 28|0);
       $6827 = HEAP32[$6826>>2]|0;
       $6828 = HEAP32[$pVCur>>2]|0;
       (FUNCTION_TABLE_ii[$6827 & 255]($6828)|0);
       label = 860;
       break L1;
       break;
      }
      case 858: {
       $7262 = $lastRowid;
       $7263 = $7262;
       $7264 = HEAP32[$7263>>2]|0;
       $7265 = (($7262) + 4)|0;
       $7266 = $7265;
       $7267 = HEAP32[$7266>>2]|0;
       $7268 = HEAP32[$db>>2]|0;
       $7269 = ((($7268)) + 32|0);
       $7270 = $7269;
       $7271 = $7270;
       HEAP32[$7271>>2] = $7264;
       $7272 = (($7270) + 4)|0;
       $7273 = $7272;
       HEAP32[$7273>>2] = $7267;
       $7274 = HEAP32[$nVmStep>>2]|0;
       $7275 = HEAP32[$0>>2]|0;
       $7276 = ((($7275)) + 108|0);
       $7277 = ((($7276)) + 16|0);
       $7278 = HEAP32[$7277>>2]|0;
       $7279 = (($7278) + ($7274))|0;
       HEAP32[$7277>>2] = $7279;
       $7280 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7280|0);
       break;
      }
      case 859: {
       $7281 = HEAP32[$0>>2]|0;
       _sqlite3VdbeError($7281,17300,$vararg_buffer81);
       HEAP32[$rc>>2] = 18;
       break L1;
       break;
      }
     }
     if ((label|0) == 861) {
      $7284 = HEAP32[$db>>2]|0;
      $7285 = ((($7284)) + 65|0);
      $7286 = HEAP8[$7285>>0]|0;
      $7287 = ($7286<<24>>24)!=(0);
      if ($7287) {
       HEAP32[$rc>>2] = 7;
      }
      $7288 = HEAP32[$rc>>2]|0;
      $7289 = ($7288|0)!=(3082);
      if (!($7289)) {
       break L1;
      }
      $7290 = HEAP32[$0>>2]|0;
      $7291 = HEAP32[$rc>>2]|0;
      $7292 = (_sqlite3ErrStr($7291)|0);
      HEAP32[$vararg_buffer85>>2] = $7292;
      _sqlite3VdbeError($7290,17521,$vararg_buffer85);
      break L1;
     }
     $3290 = HEAP32[$0>>2]|0;
     $3291 = (_sqlite3VdbeHalt($3290)|0);
     $3292 = ($3291|0)==(5);
     if ($3292) {
      $3293 = HEAP32[$pOp>>2]|0;
      $3294 = HEAP32[$aOp>>2]|0;
      $3295 = $3293;
      $3296 = $3294;
      $3297 = (($3295) - ($3296))|0;
      $3298 = (($3297|0) / 20)&-1;
      $3299 = HEAP32[$0>>2]|0;
      $3300 = ((($3299)) + 76|0);
      HEAP32[$3300>>2] = $3298;
      $3301 = HEAP32[$desiredAutoCommit>>2]|0;
      $3302 = (1 - ($3301))|0;
      $3303 = $3302&255;
      $3304 = HEAP32[$db>>2]|0;
      $3305 = ((($3304)) + 63|0);
      HEAP8[$3305>>0] = $3303;
      HEAP32[$rc>>2] = 5;
      $3306 = HEAP32[$0>>2]|0;
      $3307 = ((($3306)) + 80|0);
      HEAP32[$3307>>2] = 5;
      $7262 = $lastRowid;
      $7263 = $7262;
      $7264 = HEAP32[$7263>>2]|0;
      $7265 = (($7262) + 4)|0;
      $7266 = $7265;
      $7267 = HEAP32[$7266>>2]|0;
      $7268 = HEAP32[$db>>2]|0;
      $7269 = ((($7268)) + 32|0);
      $7270 = $7269;
      $7271 = $7270;
      HEAP32[$7271>>2] = $7264;
      $7272 = (($7270) + 4)|0;
      $7273 = $7272;
      HEAP32[$7273>>2] = $7267;
      $7274 = HEAP32[$nVmStep>>2]|0;
      $7275 = HEAP32[$0>>2]|0;
      $7276 = ((($7275)) + 108|0);
      $7277 = ((($7276)) + 16|0);
      $7278 = HEAP32[$7277>>2]|0;
      $7279 = (($7278) + ($7274))|0;
      HEAP32[$7277>>2] = $7279;
      $7280 = HEAP32[$rc>>2]|0;
      STACKTOP = sp;return ($7280|0);
     }
     $3308 = HEAP32[$db>>2]|0;
     _sqlite3CloseSavepoints($3308);
     $3309 = HEAP32[$0>>2]|0;
     $3310 = ((($3309)) + 80|0);
     $3311 = HEAP32[$3310>>2]|0;
     $3312 = ($3311|0)==(0);
     if ($3312) {
      HEAP32[$rc>>2] = 101;
      $7262 = $lastRowid;
      $7263 = $7262;
      $7264 = HEAP32[$7263>>2]|0;
      $7265 = (($7262) + 4)|0;
      $7266 = $7265;
      $7267 = HEAP32[$7266>>2]|0;
      $7268 = HEAP32[$db>>2]|0;
      $7269 = ((($7268)) + 32|0);
      $7270 = $7269;
      $7271 = $7270;
      HEAP32[$7271>>2] = $7264;
      $7272 = (($7270) + 4)|0;
      $7273 = $7272;
      HEAP32[$7273>>2] = $7267;
      $7274 = HEAP32[$nVmStep>>2]|0;
      $7275 = HEAP32[$0>>2]|0;
      $7276 = ((($7275)) + 108|0);
      $7277 = ((($7276)) + 16|0);
      $7278 = HEAP32[$7277>>2]|0;
      $7279 = (($7278) + ($7274))|0;
      HEAP32[$7277>>2] = $7279;
      $7280 = HEAP32[$rc>>2]|0;
      STACKTOP = sp;return ($7280|0);
     } else {
      HEAP32[$rc>>2] = 1;
      $7262 = $lastRowid;
      $7263 = $7262;
      $7264 = HEAP32[$7263>>2]|0;
      $7265 = (($7262) + 4)|0;
      $7266 = $7265;
      $7267 = HEAP32[$7266>>2]|0;
      $7268 = HEAP32[$db>>2]|0;
      $7269 = ((($7268)) + 32|0);
      $7270 = $7269;
      $7271 = $7270;
      HEAP32[$7271>>2] = $7264;
      $7272 = (($7270) + 4)|0;
      $7273 = $7272;
      HEAP32[$7273>>2] = $7267;
      $7274 = HEAP32[$nVmStep>>2]|0;
      $7275 = HEAP32[$0>>2]|0;
      $7276 = ((($7275)) + 108|0);
      $7277 = ((($7276)) + 16|0);
      $7278 = HEAP32[$7277>>2]|0;
      $7279 = (($7278) + ($7274))|0;
      HEAP32[$7277>>2] = $7279;
      $7280 = HEAP32[$rc>>2]|0;
      STACKTOP = sp;return ($7280|0);
     }
    }
   } while(0);
   $7293 = HEAP32[$db>>2]|0;
   $7294 = ((($7293)) + 65|0);
   $7295 = HEAP8[$7294>>0]|0;
   $7296 = $7295&255;
   $7297 = ($7296|0)!=(0);
   $7298 = $7297 ? 7 : 9;
   HEAP32[$rc>>2] = $7298;
   $7299 = HEAP32[$rc>>2]|0;
   $7300 = HEAP32[$0>>2]|0;
   $7301 = ((($7300)) + 80|0);
   HEAP32[$7301>>2] = $7299;
   $7302 = HEAP32[$0>>2]|0;
   $7303 = HEAP32[$rc>>2]|0;
   $7304 = (_sqlite3ErrStr($7303)|0);
   HEAP32[$vararg_buffer88>>2] = $7304;
   _sqlite3VdbeError($7302,17521,$vararg_buffer88);
  }
 } while(0);
 if ((label|0) == 860) {
  $7282 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($7282);
  $7283 = HEAP32[$0>>2]|0;
  _sqlite3VdbeError($7283,18773,$vararg_buffer83);
  HEAP32[$rc>>2] = 7;
 }
 $7235 = HEAP32[$rc>>2]|0;
 $7236 = HEAP32[$0>>2]|0;
 $7237 = ((($7236)) + 80|0);
 HEAP32[$7237>>2] = $7235;
 $7238 = HEAP32[$rc>>2]|0;
 $7239 = HEAP32[$pOp>>2]|0;
 $7240 = HEAP32[$aOp>>2]|0;
 $7241 = $7239;
 $7242 = $7240;
 $7243 = (($7241) - ($7242))|0;
 $7244 = (($7243|0) / 20)&-1;
 $7245 = HEAP32[$0>>2]|0;
 $7246 = ((($7245)) + 168|0);
 $7247 = HEAP32[$7246>>2]|0;
 $7248 = HEAP32[$0>>2]|0;
 $7249 = ((($7248)) + 44|0);
 $7250 = HEAP32[$7249>>2]|0;
 HEAP32[$vararg_buffer76>>2] = $7244;
 $vararg_ptr79 = ((($vararg_buffer76)) + 4|0);
 HEAP32[$vararg_ptr79>>2] = $7247;
 $vararg_ptr80 = ((($vararg_buffer76)) + 8|0);
 HEAP32[$vararg_ptr80>>2] = $7250;
 _sqlite3_log($7238,21487,$vararg_buffer76);
 $7251 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeHalt($7251)|0);
 $7252 = HEAP32[$rc>>2]|0;
 $7253 = ($7252|0)==(3082);
 if ($7253) {
  $7254 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($7254);
 }
 HEAP32[$rc>>2] = 1;
 $7255 = HEAP8[$resetSchemaOnFault>>0]|0;
 $7256 = $7255&255;
 $7257 = ($7256|0)>(0);
 if (!($7257)) {
  $7262 = $lastRowid;
  $7263 = $7262;
  $7264 = HEAP32[$7263>>2]|0;
  $7265 = (($7262) + 4)|0;
  $7266 = $7265;
  $7267 = HEAP32[$7266>>2]|0;
  $7268 = HEAP32[$db>>2]|0;
  $7269 = ((($7268)) + 32|0);
  $7270 = $7269;
  $7271 = $7270;
  HEAP32[$7271>>2] = $7264;
  $7272 = (($7270) + 4)|0;
  $7273 = $7272;
  HEAP32[$7273>>2] = $7267;
  $7274 = HEAP32[$nVmStep>>2]|0;
  $7275 = HEAP32[$0>>2]|0;
  $7276 = ((($7275)) + 108|0);
  $7277 = ((($7276)) + 16|0);
  $7278 = HEAP32[$7277>>2]|0;
  $7279 = (($7278) + ($7274))|0;
  HEAP32[$7277>>2] = $7279;
  $7280 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($7280|0);
 }
 $7258 = HEAP32[$db>>2]|0;
 $7259 = HEAP8[$resetSchemaOnFault>>0]|0;
 $7260 = $7259&255;
 $7261 = (($7260) - 1)|0;
 _sqlite3ResetOneSchema($7258,$7261);
 $7262 = $lastRowid;
 $7263 = $7262;
 $7264 = HEAP32[$7263>>2]|0;
 $7265 = (($7262) + 4)|0;
 $7266 = $7265;
 $7267 = HEAP32[$7266>>2]|0;
 $7268 = HEAP32[$db>>2]|0;
 $7269 = ((($7268)) + 32|0);
 $7270 = $7269;
 $7271 = $7270;
 HEAP32[$7271>>2] = $7264;
 $7272 = (($7270) + 4)|0;
 $7273 = $7272;
 HEAP32[$7273>>2] = $7267;
 $7274 = HEAP32[$nVmStep>>2]|0;
 $7275 = HEAP32[$0>>2]|0;
 $7276 = ((($7275)) + 108|0);
 $7277 = ((($7276)) + 16|0);
 $7278 = HEAP32[$7277>>2]|0;
 $7279 = (($7278) + ($7274))|0;
 HEAP32[$7277>>2] = $7279;
 $7280 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7280|0);
}
function _doWalCallbacks($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nEntry = 0, $or$cond = 0, $or$cond3 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $rc = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $nEntry = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pBt>>2] = $12;
  $13 = HEAP32[$pBt>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeEnter($15);
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_sqlite3BtreePager($16)|0);
   $18 = (_sqlite3PagerWalCallback($17)|0);
   HEAP32[$nEntry>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 216|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0|0);
   $23 = HEAP32[$nEntry>>2]|0;
   $24 = ($23|0)>(0);
   $or$cond = $22 & $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   $or$cond3 = $or$cond & $26;
   if ($or$cond3) {
    $27 = HEAP32[$0>>2]|0;
    $28 = ((($27)) + 216|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 220|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$0>>2]|0;
    $34 = HEAP32[$i>>2]|0;
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 16|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + ($34<<4)|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$nEntry>>2]|0;
    $41 = (FUNCTION_TABLE_iiiii[$29 & 255]($32,$33,$39,$40)|0);
    HEAP32[$rc>>2] = $41;
   }
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$i>>2] = $43;
 }
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3OpcodeName($i) {
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $i;
 $1 = HEAP32[$0>>2]|0;
 $2 = (1772 + ($1<<2)|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _displayP4($pOp,$zTemp,$nTemp) {
 $pOp = $pOp|0;
 $zTemp = $zTemp|0;
 $nTemp = $nTemp|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0.0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0.0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ai = 0, $i = 0, $j = 0, $n = 0, $pColl = 0, $pColl1 = 0, $pDef = 0;
 var $pKeyInfo = 0, $pMem = 0, $pVtab = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer15 = 0, $vararg_buffer18 = 0, $vararg_buffer21 = 0, $vararg_buffer24 = 0, $vararg_buffer27 = 0, $vararg_buffer30 = 0, $vararg_buffer33 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr4 = 0, $x = 0, $zColl = 0, $zP4 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer33 = sp + 88|0;
 $vararg_buffer30 = sp + 80|0;
 $vararg_buffer27 = sp + 72|0;
 $vararg_buffer24 = sp + 64|0;
 $vararg_buffer21 = sp + 56|0;
 $vararg_buffer18 = sp + 48|0;
 $vararg_buffer15 = sp + 40|0;
 $vararg_buffer12 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $zP4 = sp + 164|0;
 $x = sp + 136|0;
 $j = sp + 132|0;
 $pKeyInfo = sp + 128|0;
 $pColl = sp + 124|0;
 $zColl = sp + 120|0;
 $pColl1 = sp + 116|0;
 $pDef = sp + 112|0;
 $pMem = sp + 108|0;
 $pVtab = sp + 104|0;
 $i = sp + 100|0;
 $ai = sp + 96|0;
 $n = sp + 92|0;
 HEAP32[$0>>2] = $pOp;
 HEAP32[$1>>2] = $zTemp;
 HEAP32[$2>>2] = $nTemp;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$zP4>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3StrAccumInit($x,0,$4,$5,0);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 1|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 L1: do {
  switch ($9|0) {
  case -6:  {
   $10 = HEAP32[$0>>2]|0;
   $11 = ((($10)) + 16|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$pKeyInfo>>2] = $12;
   $13 = HEAP32[$pKeyInfo>>2]|0;
   $14 = ((($13)) + 6|0);
   $15 = HEAP16[$14>>1]|0;
   $16 = $15&65535;
   HEAP32[$vararg_buffer>>2] = $16;
   _sqlite3XPrintf($x,20711,$vararg_buffer);
   HEAP32[$j>>2] = 0;
   while(1) {
    $17 = HEAP32[$j>>2]|0;
    $18 = HEAP32[$pKeyInfo>>2]|0;
    $19 = ((($18)) + 6|0);
    $20 = HEAP16[$19>>1]|0;
    $21 = $20&65535;
    $22 = ($17|0)<($21|0);
    if (!($22)) {
     break;
    }
    $23 = HEAP32[$j>>2]|0;
    $24 = HEAP32[$pKeyInfo>>2]|0;
    $25 = ((($24)) + 20|0);
    $26 = (($25) + ($23<<2)|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pColl>>2] = $27;
    $28 = HEAP32[$pColl>>2]|0;
    $29 = ($28|0)!=(0|0);
    if ($29) {
     $30 = HEAP32[$pColl>>2]|0;
     $31 = HEAP32[$30>>2]|0;
     $32 = $31;
    } else {
     $32 = 17532;
    }
    HEAP32[$zColl>>2] = $32;
    $33 = HEAP32[$zColl>>2]|0;
    $34 = (_strcmp($33,20716)|0);
    $35 = ($34|0)==(0);
    $$ = $35 ? 20723 : $32;
    HEAP32[$zColl>>2] = $$;
    $36 = HEAP32[$j>>2]|0;
    $37 = HEAP32[$pKeyInfo>>2]|0;
    $38 = ((($37)) + 16|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + ($36)|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    $43 = ($42|0)!=(0);
    $44 = $43 ? 20731 : 17532;
    $45 = HEAP32[$zColl>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $44;
    $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $45;
    _sqlite3XPrintf($x,20725,$vararg_buffer1);
    $46 = HEAP32[$j>>2]|0;
    $47 = (($46) + 1)|0;
    HEAP32[$j>>2] = $47;
   }
   _sqlite3StrAccumAppend($x,20733,1);
   break;
  }
  case -4:  {
   $48 = HEAP32[$0>>2]|0;
   $49 = ((($48)) + 16|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pColl1>>2] = $50;
   $51 = HEAP32[$pColl1>>2]|0;
   $52 = HEAP32[$51>>2]|0;
   HEAP32[$vararg_buffer5>>2] = $52;
   _sqlite3XPrintf($x,20735,$vararg_buffer5);
   break;
  }
  case -5:  {
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 16|0);
   $55 = HEAP32[$54>>2]|0;
   HEAP32[$pDef>>2] = $55;
   $56 = HEAP32[$pDef>>2]|0;
   $57 = ((($56)) + 20|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$pDef>>2]|0;
   $60 = HEAP16[$59>>1]|0;
   $61 = $60 << 16 >> 16;
   HEAP32[$vararg_buffer8>>2] = $58;
   $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
   HEAP32[$vararg_ptr11>>2] = $61;
   _sqlite3XPrintf($x,20743,$vararg_buffer8);
   break;
  }
  case -13:  {
   $62 = HEAP32[$0>>2]|0;
   $63 = ((($62)) + 16|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $64;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (($65) + 4)|0;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = $vararg_buffer12;
   $72 = $71;
   HEAP32[$72>>2] = $67;
   $73 = (($71) + 4)|0;
   $74 = $73;
   HEAP32[$74>>2] = $70;
   _sqlite3XPrintf($x,19285,$vararg_buffer12);
   break;
  }
  case -14:  {
   $75 = HEAP32[$0>>2]|0;
   $76 = ((($75)) + 16|0);
   $77 = HEAP32[$76>>2]|0;
   HEAP32[$vararg_buffer15>>2] = $77;
   _sqlite3XPrintf($x,20750,$vararg_buffer15);
   break;
  }
  case -12:  {
   $78 = HEAP32[$0>>2]|0;
   $79 = ((($78)) + 16|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = +HEAPF64[$80>>3];
   HEAPF64[$vararg_buffer18>>3] = $81;
   _sqlite3XPrintf($x,20753,$vararg_buffer18);
   break;
  }
  case -8:  {
   $82 = HEAP32[$0>>2]|0;
   $83 = ((($82)) + 16|0);
   $84 = HEAP32[$83>>2]|0;
   HEAP32[$pMem>>2] = $84;
   $85 = HEAP32[$pMem>>2]|0;
   $86 = ((($85)) + 8|0);
   $87 = HEAP16[$86>>1]|0;
   $88 = $87&65535;
   $89 = $88 & 2;
   $90 = ($89|0)!=(0);
   $91 = HEAP32[$pMem>>2]|0;
   if ($90) {
    $92 = ((($91)) + 16|0);
    $93 = HEAP32[$92>>2]|0;
    HEAP32[$zP4>>2] = $93;
    break L1;
   }
   $94 = ((($91)) + 8|0);
   $95 = HEAP16[$94>>1]|0;
   $96 = $95&65535;
   $97 = $96 & 4;
   $98 = ($97|0)!=(0);
   $99 = HEAP32[$pMem>>2]|0;
   if ($98) {
    $100 = $99;
    $101 = $100;
    $102 = HEAP32[$101>>2]|0;
    $103 = (($100) + 4)|0;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = $vararg_buffer21;
    $107 = $106;
    HEAP32[$107>>2] = $102;
    $108 = (($106) + 4)|0;
    $109 = $108;
    HEAP32[$109>>2] = $105;
    _sqlite3XPrintf($x,19285,$vararg_buffer21);
    break L1;
   }
   $110 = ((($99)) + 8|0);
   $111 = HEAP16[$110>>1]|0;
   $112 = $111&65535;
   $113 = $112 & 8;
   $114 = ($113|0)!=(0);
   $115 = HEAP32[$pMem>>2]|0;
   if ($114) {
    $116 = +HEAPF64[$115>>3];
    HEAPF64[$vararg_buffer24>>3] = $116;
    _sqlite3XPrintf($x,20753,$vararg_buffer24);
    break L1;
   }
   $117 = ((($115)) + 8|0);
   $118 = HEAP16[$117>>1]|0;
   $119 = $118&65535;
   $120 = $119 & 1;
   $121 = ($120|0)!=(0);
   if ($121) {
    HEAP32[$zP4>>2] = 18200;
    break L1;
   } else {
    HEAP32[$zP4>>2] = 20759;
    break L1;
   }
   break;
  }
  case -10:  {
   $122 = HEAP32[$0>>2]|0;
   $123 = ((($122)) + 16|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = ((($124)) + 8|0);
   $126 = HEAP32[$125>>2]|0;
   HEAP32[$pVtab>>2] = $126;
   $127 = HEAP32[$pVtab>>2]|0;
   HEAP32[$vararg_buffer27>>2] = $127;
   _sqlite3XPrintf($x,20766,$vararg_buffer27);
   break;
  }
  case -15:  {
   $128 = HEAP32[$0>>2]|0;
   $129 = ((($128)) + 16|0);
   $130 = HEAP32[$129>>2]|0;
   HEAP32[$ai>>2] = $130;
   $131 = HEAP32[$ai>>2]|0;
   $132 = HEAP32[$131>>2]|0;
   HEAP32[$n>>2] = $132;
   HEAP32[$i>>2] = 1;
   while(1) {
    $133 = HEAP32[$i>>2]|0;
    $134 = HEAP32[$n>>2]|0;
    $135 = ($133|0)<($134|0);
    if (!($135)) {
     break;
    }
    $136 = HEAP32[$i>>2]|0;
    $137 = HEAP32[$ai>>2]|0;
    $138 = (($137) + ($136<<2)|0);
    $139 = HEAP32[$138>>2]|0;
    HEAP32[$vararg_buffer30>>2] = $139;
    _sqlite3XPrintf($x,20774,$vararg_buffer30);
    $140 = HEAP32[$i>>2]|0;
    $141 = (($140) + 1)|0;
    HEAP32[$i>>2] = $141;
   }
   $142 = HEAP32[$1>>2]|0;
   HEAP8[$142>>0] = 91;
   _sqlite3StrAccumAppend($x,20778,1);
   break;
  }
  case -18:  {
   _sqlite3XPrintf($x,20780,$vararg_buffer33);
   break;
  }
  case -19:  {
   $143 = HEAP32[$1>>2]|0;
   HEAP8[$143>>0] = 0;
   break;
  }
  default: {
   $144 = HEAP32[$0>>2]|0;
   $145 = ((($144)) + 16|0);
   $146 = HEAP32[$145>>2]|0;
   HEAP32[$zP4>>2] = $146;
   $147 = HEAP32[$zP4>>2]|0;
   $148 = ($147|0)==(0|0);
   if ($148) {
    $149 = HEAP32[$1>>2]|0;
    HEAP32[$zP4>>2] = $149;
    $150 = HEAP32[$1>>2]|0;
    HEAP8[$150>>0] = 0;
   }
  }
  }
 } while(0);
 (_sqlite3StrAccumFinish($x)|0);
 $151 = HEAP32[$zP4>>2]|0;
 STACKTOP = sp;return ($151|0);
}
function _sqlite3XPrintf($p,$zFormat,$varargs) {
 $p = $p|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _sqlite3VXPrintf($2,$3,$ap);
 STACKTOP = sp;return;
}
function _out2Prerelease($p,$pOp) {
 $p = $p|0;
 $pOp = $pOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOut = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOp;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + (($5*40)|0)|0);
 HEAP32[$pOut>>2] = $9;
 $10 = HEAP32[$pOut>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 & 9312;
 $15 = ($14|0)!=(0);
 $16 = HEAP32[$pOut>>2]|0;
 if ($15) {
  $17 = (_out2PrereleaseWithClear($16)|0);
  HEAP32[$0>>2] = $17;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $18 = ((($16)) + 8|0);
  HEAP16[$18>>1] = 4;
  $19 = HEAP32[$pOut>>2]|0;
  HEAP32[$0>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemTooBig($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 18;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$n>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 16384;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$n>>2]|0;
  $20 = (($19) + ($18))|0;
  HEAP32[$n>>2] = $20;
 }
 $21 = HEAP32[$n>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 32|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ((($24)) + 88|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($21|0)>($26|0);
 $28 = $27&1;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3VdbeMemShallowCopy($pTo,$pFrom,$srcType) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 $srcType = $srcType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $srcType;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 9312;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  _vdbeClrCopy($9,$10,$11);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 ;HEAP32[$9>>2]=HEAP32[$12>>2]|0;HEAP32[$9+4>>2]=HEAP32[$12+4>>2]|0;HEAP32[$9+8>>2]=HEAP32[$12+8>>2]|0;HEAP32[$9+12>>2]=HEAP32[$12+12>>2]|0;HEAP32[$9+16>>2]=HEAP32[$12+16>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 2048;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 8|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $23 = $22 & -7169;
 $24 = $23&65535;
 HEAP16[$20>>1] = $24;
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 8|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = $29 | $25;
 $31 = $30&65535;
 HEAP16[$27>>1] = $31;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemMove($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 dest=$3; src=$4; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 24|0);
 HEAP32[$8>>2] = 0;
 STACKTOP = sp;return;
}
function _numericType($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 12;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 do {
  if ($7) {
   $12 = $11 & 12;
   $13 = $12&65535;
   HEAP16[$0>>1] = $13;
  } else {
   $14 = $11 & 18;
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = HEAP32[$1>>2]|0;
    $17 = (_computeNumericType($16)|0);
    HEAP16[$0>>1] = $17;
    break;
   } else {
    HEAP16[$0>>1] = 0;
    break;
   }
  }
 } while(0);
 $18 = HEAP16[$0>>1]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3AddInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iA = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 8|0;
 $iA = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = $iA;
 $17 = $16;
 HEAP32[$17>>2] = $12;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)>(0);
 $27 = ($22>>>0)>=(0);
 $28 = ($25|0)==(0);
 $29 = $28 & $27;
 $30 = $26 | $29;
 $31 = $iA;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 if ($30) {
  $37 = ($36|0)>(0);
  $38 = ($33>>>0)>(0);
  $39 = ($36|0)==(0);
  $40 = $39 & $38;
  $41 = $37 | $40;
  if ($41) {
   $42 = $iA;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_i64Subtract(-1,2147483647,($44|0),($47|0))|0);
   $49 = tempRet0;
   $50 = $4;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = ($49|0)<($55|0);
   $57 = ($48>>>0)<($52>>>0);
   $58 = ($49|0)==($55|0);
   $59 = $58 & $57;
   $60 = $56 | $59;
   if ($60) {
    HEAP32[$2>>2] = 1;
    $104 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($104|0);
   }
  }
 } else {
  $61 = ($36|0)<(0);
  if ($61) {
   $62 = $iA;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Add(($64|0),($67|0),-1,2147483647)|0);
   $69 = tempRet0;
   $70 = (_i64Subtract(0,0,($68|0),($69|0))|0);
   $71 = tempRet0;
   $72 = $4;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (_i64Add(($74|0),($77|0),1,0)|0);
   $79 = tempRet0;
   $80 = ($71|0)>($79|0);
   $81 = ($70>>>0)>($78>>>0);
   $82 = ($71|0)==($79|0);
   $83 = $82 & $81;
   $84 = $80 | $83;
   if ($84) {
    HEAP32[$2>>2] = 1;
    $104 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($104|0);
   }
  }
 }
 $85 = $4;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = HEAP32[$3>>2]|0;
 $92 = $91;
 $93 = $92;
 $94 = HEAP32[$93>>2]|0;
 $95 = (($92) + 4)|0;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (_i64Add(($94|0),($97|0),($87|0),($90|0))|0);
 $99 = tempRet0;
 $100 = $91;
 $101 = $100;
 HEAP32[$101>>2] = $98;
 $102 = (($100) + 4)|0;
 $103 = $102;
 HEAP32[$103>>2] = $99;
 HEAP32[$2>>2] = 0;
 $104 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _sqlite3SubInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)==(0);
 $16 = ($14|0)==(-2147483648);
 $17 = $15 & $16;
 $18 = HEAP32[$3>>2]|0;
 if (!($17)) {
  $49 = $4;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (_i64Subtract(0,0,($51|0),($54|0))|0);
  $56 = tempRet0;
  $57 = (_sqlite3AddInt64($18,$55,$56)|0);
  HEAP32[$2>>2] = $57;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)>(0);
 $26 = ($21>>>0)>=(0);
 $27 = ($24|0)==(0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  HEAP32[$2>>2] = 1;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 } else {
  $30 = $4;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  $37 = $36;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Subtract(($39|0),($42|0),($32|0),($35|0))|0);
  $44 = tempRet0;
  $45 = $36;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  HEAP32[$2>>2] = 0;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 return (0)|0;
}
function _sqlite3MulInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $iA = 0, $iA0 = 0, $iA1 = 0, $iB0 = 0, $iB1 = 0, $or$cond = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 48|0;
 $iA = sp + 40|0;
 $iA1 = sp + 32|0;
 $iA0 = sp + 24|0;
 $iB1 = sp + 16|0;
 $iB0 = sp + 8|0;
 $r = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = $iA;
 $17 = $16;
 HEAP32[$17>>2] = $12;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $iA;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (___divdi3(($22|0),($25|0),0,1)|0);
 $27 = tempRet0;
 $28 = $iA1;
 $29 = $28;
 HEAP32[$29>>2] = $26;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 $32 = $iA;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (___remdi3(($34|0),($37|0),0,1)|0);
 $39 = tempRet0;
 $40 = $iA0;
 $41 = $40;
 HEAP32[$41>>2] = $38;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = $4;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (___divdi3(($46|0),($49|0),0,1)|0);
 $51 = tempRet0;
 $52 = $iB1;
 $53 = $52;
 HEAP32[$53>>2] = $50;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $51;
 $56 = $4;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($56) + 4)|0;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = (___remdi3(($58|0),($61|0),0,1)|0);
 $63 = tempRet0;
 $64 = $iB0;
 $65 = $64;
 HEAP32[$65>>2] = $62;
 $66 = (($64) + 4)|0;
 $67 = $66;
 HEAP32[$67>>2] = $63;
 $68 = $iA1;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($68) + 4)|0;
 $72 = $71;
 $73 = HEAP32[$72>>2]|0;
 $74 = ($70|0)==(0);
 $75 = ($73|0)==(0);
 $76 = $74 & $75;
 $77 = $iB1;
 $78 = $77;
 $79 = HEAP32[$78>>2]|0;
 $80 = (($77) + 4)|0;
 $81 = $80;
 $82 = HEAP32[$81>>2]|0;
 $83 = ($79|0)==(0);
 $84 = ($82|0)==(0);
 $85 = $83 & $84;
 do {
  if ($76) {
   if (!($85)) {
    $105 = $iA0;
    $106 = $105;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($105) + 4)|0;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = $iB1;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = (($111) + 4)|0;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (___muldi3(($107|0),($110|0),($113|0),($116|0))|0);
    $118 = tempRet0;
    $119 = $r;
    $120 = $119;
    HEAP32[$120>>2] = $117;
    $121 = (($119) + 4)|0;
    $122 = $121;
    HEAP32[$122>>2] = $118;
    break;
   }
   $86 = $4;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = (($86) + 4)|0;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = HEAP32[$3>>2]|0;
   $93 = $92;
   $94 = $93;
   $95 = HEAP32[$94>>2]|0;
   $96 = (($93) + 4)|0;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (___muldi3(($95|0),($98|0),($88|0),($91|0))|0);
   $100 = tempRet0;
   $101 = $92;
   $102 = $101;
   HEAP32[$102>>2] = $99;
   $103 = (($101) + 4)|0;
   $104 = $103;
   HEAP32[$104>>2] = $100;
   HEAP32[$2>>2] = 0;
   $202 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($202|0);
  } else {
   if ($85) {
    $123 = $iA1;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = (($123) + 4)|0;
    $127 = $126;
    $128 = HEAP32[$127>>2]|0;
    $129 = $iB0;
    $130 = $129;
    $131 = HEAP32[$130>>2]|0;
    $132 = (($129) + 4)|0;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (___muldi3(($125|0),($128|0),($131|0),($134|0))|0);
    $136 = tempRet0;
    $137 = $r;
    $138 = $137;
    HEAP32[$138>>2] = $135;
    $139 = (($137) + 4)|0;
    $140 = $139;
    HEAP32[$140>>2] = $136;
    break;
   }
   HEAP32[$2>>2] = 1;
   $202 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($202|0);
  }
 } while(0);
 $141 = $r;
 $142 = $141;
 $143 = HEAP32[$142>>2]|0;
 $144 = (($141) + 4)|0;
 $145 = $144;
 $146 = HEAP32[$145>>2]|0;
 $147 = ($146|0)<(-1);
 $148 = ($143>>>0)<(2147483648);
 $149 = ($146|0)==(-1);
 $150 = $149 & $148;
 $151 = $147 | $150;
 $152 = $r;
 $153 = $152;
 $154 = HEAP32[$153>>2]|0;
 $155 = (($152) + 4)|0;
 $156 = $155;
 $157 = HEAP32[$156>>2]|0;
 $158 = ($157|0)>(0);
 $159 = ($154>>>0)>=(2147483648);
 $160 = ($157|0)==(0);
 $161 = $160 & $159;
 $162 = $158 | $161;
 $or$cond = $151 | $162;
 if ($or$cond) {
  HEAP32[$2>>2] = 1;
  $202 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($202|0);
 }
 $163 = $r;
 $164 = $163;
 $165 = HEAP32[$164>>2]|0;
 $166 = (($163) + 4)|0;
 $167 = $166;
 $168 = HEAP32[$167>>2]|0;
 $169 = (___muldi3(($165|0),($168|0),0,1)|0);
 $170 = tempRet0;
 $171 = $r;
 $172 = $171;
 HEAP32[$172>>2] = $169;
 $173 = (($171) + 4)|0;
 $174 = $173;
 HEAP32[$174>>2] = $170;
 $175 = $iA0;
 $176 = $175;
 $177 = HEAP32[$176>>2]|0;
 $178 = (($175) + 4)|0;
 $179 = $178;
 $180 = HEAP32[$179>>2]|0;
 $181 = $iB0;
 $182 = $181;
 $183 = HEAP32[$182>>2]|0;
 $184 = (($181) + 4)|0;
 $185 = $184;
 $186 = HEAP32[$185>>2]|0;
 $187 = (___muldi3(($177|0),($180|0),($183|0),($186|0))|0);
 $188 = tempRet0;
 $189 = (_sqlite3AddInt64($r,$187,$188)|0);
 $190 = ($189|0)!=(0);
 if ($190) {
  HEAP32[$2>>2] = 1;
  $202 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($202|0);
 } else {
  $191 = $r;
  $192 = $191;
  $193 = HEAP32[$192>>2]|0;
  $194 = (($191) + 4)|0;
  $195 = $194;
  $196 = HEAP32[$195>>2]|0;
  $197 = HEAP32[$3>>2]|0;
  $198 = $197;
  $199 = $198;
  HEAP32[$199>>2] = $193;
  $200 = (($198) + 4)|0;
  $201 = $200;
  HEAP32[$201>>2] = $196;
  HEAP32[$2>>2] = 0;
  $202 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($202|0);
 }
 return (0)|0;
}
function _sqlite3VdbeIntegerAffinity($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ix = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $ix = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = +HEAPF64[$1>>3];
 $3 = (_doubleToInt64($2)|0);
 $4 = tempRet0;
 $5 = $ix;
 $6 = $5;
 HEAP32[$6>>2] = $3;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = HEAP32[$0>>2]|0;
 $10 = +HEAPF64[$9>>3];
 $11 = $ix;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (+($13>>>0)) + (4294967296.0*(+($16|0)));
 $18 = $10 == $17;
 $19 = $ix;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)>(-2147483648);
 $26 = ($21>>>0)>(0);
 $27 = ($24|0)==(-2147483648);
 $28 = $27 & $26;
 $29 = $25 | $28;
 $or$cond = $18 & $29;
 $30 = $ix;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)<(2147483647);
 $37 = ($32>>>0)<(4294967295);
 $38 = ($35|0)==(2147483647);
 $39 = $38 & $37;
 $40 = $36 | $39;
 $or$cond3 = $or$cond & $40;
 if (!($or$cond3)) {
  STACKTOP = sp;return;
 }
 $41 = $ix;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = $47;
 $49 = $48;
 HEAP32[$49>>2] = $43;
 $50 = (($48) + 4)|0;
 $51 = $50;
 HEAP32[$51>>2] = $46;
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 8|0);
 $54 = HEAP16[$53>>1]|0;
 $55 = $54&65535;
 $56 = $55 & -49664;
 $57 = $56 | 4;
 $58 = $57&65535;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 8|0);
 HEAP16[$60>>1] = $58;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemIntegerify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 $4 = HEAP32[$0>>2]|0;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $2;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $3;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & -49664;
 $14 = $13 | 4;
 $15 = $14&65535;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP16[$17>>1] = $15;
 STACKTOP = sp;return 0;
}
function _applyAffinity($pRec,$affinity,$enc) {
 $pRec = $pRec|0;
 $affinity = $affinity|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pRec;
 HEAP8[$1>>0] = $affinity;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)>=(67);
 if (!($5)) {
  $25 = HEAP8[$1>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($26|0)==(66);
  if (!($27)) {
   STACKTOP = sp;return;
  }
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 & 2;
  $33 = (0)==($32|0);
  if ($33) {
   $34 = HEAP32[$0>>2]|0;
   $35 = ((($34)) + 8|0);
   $36 = HEAP16[$35>>1]|0;
   $37 = $36&65535;
   $38 = $37 & 12;
   $39 = ($38|0)!=(0);
   if ($39) {
    $40 = HEAP32[$0>>2]|0;
    $41 = HEAP8[$2>>0]|0;
    (_sqlite3VdbeMemStringify($40,$41,1)|0);
   }
  }
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 & -13;
  $47 = $46&65535;
  HEAP16[$43>>1] = $47;
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 $10 = $9 & 4;
 $11 = ($10|0)==(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 8;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if (!($17)) {
  _sqlite3VdbeIntegerAffinity($18);
  STACKTOP = sp;return;
 }
 $19 = ((($18)) + 8|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & 2;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 _applyNumericAffinity($24,1);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemRealify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (+_sqlite3VdbeRealValue($1));
 $3 = HEAP32[$0>>2]|0;
 HEAPF64[$3>>3] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & -49664;
 $9 = $8 | 8;
 $10 = $9&65535;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 HEAP16[$12>>1] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3VdbeMemCast($pMem,$aff,$encoding) {
 $pMem = $pMem|0;
 $aff = $aff|0;
 $encoding = $encoding|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pMem;
 HEAP8[$1>>0] = $aff;
 HEAP8[$2>>0] = $encoding;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP8[$1>>0]|0;
 $10 = $9&255;
 switch ($10|0) {
 case 65:  {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  $15 = $14 & 16;
  $16 = ($15|0)==(0);
  $17 = HEAP32[$0>>2]|0;
  if ($16) {
   $18 = HEAP8[$2>>0]|0;
   _sqlite3ValueApplyAffinity($17,66,$18);
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = HEAP16[$20>>1]|0;
   $22 = $21&65535;
   $23 = $22 & -49664;
   $24 = $23 | 16;
   $25 = $24&65535;
   $26 = HEAP32[$0>>2]|0;
   $27 = ((($26)) + 8|0);
   HEAP16[$27>>1] = $25;
   STACKTOP = sp;return;
  } else {
   $28 = ((($17)) + 8|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29&65535;
   $31 = $30 & -33264;
   $32 = $31&65535;
   HEAP16[$28>>1] = $32;
   STACKTOP = sp;return;
  }
  break;
 }
 case 67:  {
  $33 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemNumerify($33)|0);
  STACKTOP = sp;return;
  break;
 }
 case 68:  {
  $34 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemIntegerify($34)|0);
  STACKTOP = sp;return;
  break;
 }
 case 69:  {
  $35 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemRealify($35)|0);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $36 = HEAP32[$0>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = HEAP16[$37>>1]|0;
  $39 = $38&65535;
  $40 = $39 & 16;
  $41 = $40 >> 3;
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 | $41;
  $47 = $46&65535;
  HEAP16[$43>>1] = $47;
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP8[$2>>0]|0;
  _sqlite3ValueApplyAffinity($48,66,$49);
  $50 = HEAP32[$0>>2]|0;
  $51 = ((($50)) + 8|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = $53 & -16413;
  $55 = $54&65535;
  HEAP16[$51>>1] = $55;
  STACKTOP = sp;return;
 }
 }
}
function _sqlite3MemCompare($pMem1,$pMem2,$pColl) {
 $pMem1 = $pMem1|0;
 $pMem2 = $pMem2|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0, $77 = 0.0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0, $combined_flags = 0, $f1 = 0, $f2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $f1 = sp + 8|0;
 $f2 = sp + 4|0;
 $combined_flags = sp;
 HEAP32[$1>>2] = $pMem1;
 HEAP32[$2>>2] = $pMem2;
 HEAP32[$3>>2] = $pColl;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$f1>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 HEAP32[$f2>>2] = $11;
 $12 = HEAP32[$f1>>2]|0;
 $13 = HEAP32[$f2>>2]|0;
 $14 = $12 | $13;
 HEAP32[$combined_flags>>2] = $14;
 $15 = HEAP32[$combined_flags>>2]|0;
 $16 = $15 & 1;
 $17 = ($16|0)!=(0);
 do {
  if ($17) {
   $18 = HEAP32[$f2>>2]|0;
   $19 = $18 & 1;
   $20 = HEAP32[$f1>>2]|0;
   $21 = $20 & 1;
   $22 = (($19) - ($21))|0;
   HEAP32[$0>>2] = $22;
  } else {
   $23 = HEAP32[$combined_flags>>2]|0;
   $24 = $23 & 12;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $117 = HEAP32[$combined_flags>>2]|0;
    $118 = $117 & 2;
    $119 = ($118|0)!=(0);
    if ($119) {
     $120 = HEAP32[$f1>>2]|0;
     $121 = $120 & 2;
     $122 = ($121|0)==(0);
     if ($122) {
      HEAP32[$0>>2] = 1;
      break;
     }
     $123 = HEAP32[$f2>>2]|0;
     $124 = $123 & 2;
     $125 = ($124|0)==(0);
     if ($125) {
      HEAP32[$0>>2] = -1;
      break;
     }
     $126 = HEAP32[$3>>2]|0;
     $127 = ($126|0)!=(0|0);
     if ($127) {
      $128 = HEAP32[$1>>2]|0;
      $129 = HEAP32[$2>>2]|0;
      $130 = HEAP32[$3>>2]|0;
      $131 = (_vdbeCompareMemString($128,$129,$130,0)|0);
      HEAP32[$0>>2] = $131;
      break;
     }
    }
    $132 = HEAP32[$1>>2]|0;
    $133 = HEAP32[$2>>2]|0;
    $134 = (_sqlite3BlobCompare($132,$133)|0);
    HEAP32[$0>>2] = $134;
    break;
   }
   $26 = HEAP32[$f1>>2]|0;
   $27 = HEAP32[$f2>>2]|0;
   $28 = $26 & $27;
   $29 = $28 & 4;
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = $31;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$2>>2]|0;
    $39 = $38;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (($39) + 4)|0;
    $43 = $42;
    $44 = HEAP32[$43>>2]|0;
    $45 = ($37|0)<($44|0);
    $46 = ($34>>>0)<($41>>>0);
    $47 = ($37|0)==($44|0);
    $48 = $47 & $46;
    $49 = $45 | $48;
    if ($49) {
     HEAP32[$0>>2] = -1;
     break;
    }
    $50 = HEAP32[$1>>2]|0;
    $51 = $50;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (($51) + 4)|0;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$2>>2]|0;
    $58 = $57;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = ($56|0)>($63|0);
    $65 = ($53>>>0)>($60>>>0);
    $66 = ($56|0)==($63|0);
    $67 = $66 & $65;
    $68 = $64 | $67;
    if ($68) {
     HEAP32[$0>>2] = 1;
     break;
    } else {
     HEAP32[$0>>2] = 0;
     break;
    }
   }
   $69 = HEAP32[$f1>>2]|0;
   $70 = HEAP32[$f2>>2]|0;
   $71 = $69 & $70;
   $72 = $71 & 8;
   $73 = ($72|0)!=(0);
   if ($73) {
    $74 = HEAP32[$1>>2]|0;
    $75 = +HEAPF64[$74>>3];
    $76 = HEAP32[$2>>2]|0;
    $77 = +HEAPF64[$76>>3];
    $78 = $75 < $77;
    if ($78) {
     HEAP32[$0>>2] = -1;
     break;
    }
    $79 = HEAP32[$1>>2]|0;
    $80 = +HEAPF64[$79>>3];
    $81 = HEAP32[$2>>2]|0;
    $82 = +HEAPF64[$81>>3];
    $83 = $80 > $82;
    if ($83) {
     HEAP32[$0>>2] = 1;
     break;
    } else {
     HEAP32[$0>>2] = 0;
     break;
    }
   }
   $84 = HEAP32[$f1>>2]|0;
   $85 = $84 & 4;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$f2>>2]|0;
    $88 = $87 & 8;
    $89 = ($88|0)!=(0);
    if ($89) {
     $90 = HEAP32[$1>>2]|0;
     $91 = $90;
     $92 = $91;
     $93 = HEAP32[$92>>2]|0;
     $94 = (($91) + 4)|0;
     $95 = $94;
     $96 = HEAP32[$95>>2]|0;
     $97 = HEAP32[$2>>2]|0;
     $98 = +HEAPF64[$97>>3];
     $99 = (_sqlite3IntFloatCompare($93,$96,$98)|0);
     HEAP32[$0>>2] = $99;
     break;
    } else {
     HEAP32[$0>>2] = -1;
     break;
    }
   }
   $100 = HEAP32[$f1>>2]|0;
   $101 = $100 & 8;
   $102 = ($101|0)!=(0);
   if (!($102)) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $103 = HEAP32[$f2>>2]|0;
   $104 = $103 & 4;
   $105 = ($104|0)!=(0);
   if ($105) {
    $106 = HEAP32[$2>>2]|0;
    $107 = $106;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$1>>2]|0;
    $114 = +HEAPF64[$113>>3];
    $115 = (_sqlite3IntFloatCompare($109,$112,$114)|0);
    $116 = (0 - ($115))|0;
    HEAP32[$0>>2] = $116;
    break;
   } else {
    HEAP32[$0>>2] = -1;
    break;
   }
  }
 } while(0);
 $135 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _sqlite3VdbeCursorMoveto($pp,$piCol) {
 $pp = $pp|0;
 $piCol = $piCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iMap = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $iMap = sp;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $piCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(0);
 do {
  if ($8) {
   $9 = HEAP32[$p>>2]|0;
   $10 = ((($9)) + 3|0);
   $11 = HEAP8[$10>>0]|0;
   $12 = ($11<<24>>24)!=(0);
   $13 = HEAP32[$p>>2]|0;
   if (!($12)) {
    $35 = ((($13)) + 16|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = (_sqlite3BtreeCursorHasMoved($36)|0);
    $38 = ($37|0)!=(0);
    if (!($38)) {
     break;
    }
    $39 = HEAP32[$p>>2]|0;
    $40 = (_handleMovedCursor($39)|0);
    HEAP32[$0>>2] = $40;
    $41 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($41|0);
   }
   $14 = ((($13)) + 52|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = (1 + ($18))|0;
    $20 = HEAP32[$p>>2]|0;
    $21 = ((($20)) + 52|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + ($19<<2)|0);
    $24 = HEAP32[$23>>2]|0;
    HEAP32[$iMap>>2] = $24;
    $25 = ($24|0)>(0);
    if ($25) {
     $26 = HEAP32[$p>>2]|0;
     $27 = ((($26)) + 48|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = HEAP32[$1>>2]|0;
     HEAP32[$29>>2] = $28;
     $30 = HEAP32[$iMap>>2]|0;
     $31 = (($30) - 1)|0;
     $32 = HEAP32[$2>>2]|0;
     HEAP32[$32>>2] = $31;
     HEAP32[$0>>2] = 0;
     $41 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($41|0);
    }
   }
   $33 = HEAP32[$p>>2]|0;
   $34 = (_handleDeferredMoveto($33)|0);
   HEAP32[$0>>2] = $34;
   $41 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3BtreeKeyFetch($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_fetchPayload($2,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeDataSize($pCur,$pSize) {
 $pCur = $pCur|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 _getCellInfo($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$7>>2] = $6;
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeDataFetch($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_fetchPayload($2,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3GetVarint32($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, $n = 0, $v64 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 25|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $a = sp + 12|0;
 $b = sp + 8|0;
 $v64 = sp;
 $n = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 HEAP32[$a>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 1|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$b>>2] = $10;
 $11 = HEAP32[$b>>2]|0;
 $12 = $11 & 128;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $14 = HEAP32[$a>>2]|0;
  $15 = $14 & 127;
  HEAP32[$a>>2] = $15;
  $16 = HEAP32[$a>>2]|0;
  $17 = $16 << 7;
  HEAP32[$a>>2] = $17;
  $18 = HEAP32[$a>>2]|0;
  $19 = HEAP32[$b>>2]|0;
  $20 = $18 | $19;
  $21 = HEAP32[$2>>2]|0;
  HEAP32[$21>>2] = $20;
  HEAP8[$0>>0] = 2;
  $72 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($72|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 1|0);
 HEAP32[$1>>2] = $23;
 $24 = HEAP32[$a>>2]|0;
 $25 = $24 << 14;
 HEAP32[$a>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = HEAP32[$a>>2]|0;
 $30 = $29 | $28;
 HEAP32[$a>>2] = $30;
 $31 = HEAP32[$a>>2]|0;
 $32 = $31 & 128;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  $34 = HEAP32[$a>>2]|0;
  $35 = $34 & 2080895;
  HEAP32[$a>>2] = $35;
  $36 = HEAP32[$b>>2]|0;
  $37 = $36 & 127;
  HEAP32[$b>>2] = $37;
  $38 = HEAP32[$b>>2]|0;
  $39 = $38 << 7;
  HEAP32[$b>>2] = $39;
  $40 = HEAP32[$a>>2]|0;
  $41 = HEAP32[$b>>2]|0;
  $42 = $40 | $41;
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  HEAP8[$0>>0] = 3;
  $72 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($72|0);
 }
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + -2|0);
 HEAP32[$1>>2] = $45;
 $46 = HEAP32[$1>>2]|0;
 $47 = (_sqlite3GetVarint($46,$v64)|0);
 HEAP8[$n>>0] = $47;
 $48 = $v64;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = $v64;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = (($54) + 4)|0;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($50|0)!=($56|0);
 $61 = (0)!=($59|0);
 $62 = $60 | $61;
 if ($62) {
  $63 = HEAP32[$2>>2]|0;
  HEAP32[$63>>2] = -1;
 } else {
  $64 = $v64;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = (($64) + 4)|0;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  HEAP32[$70>>2] = $66;
 }
 $71 = HEAP8[$n>>0]|0;
 HEAP8[$0>>0] = $71;
 $72 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($72|0);
}
function _sqlite3VdbeMemFromBtree($pCur,$offset,$amt,$key,$pMem) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $key = $key|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $available = 0, $rc = 0, $zData = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $zData = sp + 8|0;
 $available = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $key;
 HEAP32[$4>>2] = $pMem;
 HEAP32[$available>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = (_sqlite3BtreeKeyFetch($7,$available)|0);
  HEAP32[$zData>>2] = $8;
 } else {
  $9 = (_sqlite3BtreeDataFetch($7,$available)|0);
  HEAP32[$zData>>2] = $9;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = HEAP32[$available>>2]|0;
 $14 = ($12>>>0)<=($13>>>0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$zData>>2]|0;
  $17 = (($16) + ($15)|0);
  $18 = HEAP32[$4>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$4>>2]|0;
  $21 = ((($20)) + 8|0);
  HEAP16[$21>>1] = 4112;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$4>>2]|0;
  $24 = ((($23)) + 12|0);
  HEAP32[$24>>2] = $22;
  $31 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($31|0);
 } else {
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = (_vdbeMemFromBtreeResize($25,$26,$27,$28,$29)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 return (0)|0;
}
function _sqlite3VdbeOneByteSerialTypeLen($serial_type) {
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $serial_type;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 $3 = (21519 + ($2)|0);
 $4 = HEAP8[$3>>0]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeSerialTypeLen($serial_type) {
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $serial_type;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2>>>0)>=(128);
 $4 = HEAP32[$1>>2]|0;
 if ($3) {
  $5 = (($4) - 12)|0;
  $6 = (($5>>>0) / 2)&-1;
  HEAP32[$0>>2] = $6;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $7 = (21519 + ($4)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSerialGet($buf,$serial_type,$pMem) {
 $buf = $buf|0;
 $serial_type = $serial_type|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $buf;
 HEAP32[$2>>2] = $serial_type;
 HEAP32[$3>>2] = $pMem;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 0: case 11: case 10:  {
  $5 = HEAP32[$3>>2]|0;
  $6 = ((($5)) + 8|0);
  HEAP16[$6>>1] = 1;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 1:  {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)<(0);
  $11 = $10 << 31 >> 31;
  $12 = HEAP32[$3>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $9;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 8|0);
  HEAP16[$18>>1] = 4;
  HEAP32[$0>>2] = 1;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 2:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = $21<<8;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 1|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $22 | $26;
  $28 = ($27|0)<(0);
  $29 = $28 << 31 >> 31;
  $30 = HEAP32[$3>>2]|0;
  $31 = $30;
  $32 = $31;
  HEAP32[$32>>2] = $27;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $29;
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP16[$36>>1] = 4;
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 3:  {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP8[$37>>0]|0;
  $39 = $38 << 24 >> 24;
  $40 = $39<<16;
  $41 = HEAP32[$1>>2]|0;
  $42 = ((($41)) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = $44 << 8;
  $46 = $40 | $45;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 2|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $46 | $50;
  $52 = ($51|0)<(0);
  $53 = $52 << 31 >> 31;
  $54 = HEAP32[$3>>2]|0;
  $55 = $54;
  $56 = $55;
  HEAP32[$56>>2] = $51;
  $57 = (($55) + 4)|0;
  $58 = $57;
  HEAP32[$58>>2] = $53;
  $59 = HEAP32[$3>>2]|0;
  $60 = ((($59)) + 8|0);
  HEAP16[$60>>1] = 4;
  HEAP32[$0>>2] = 3;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 4:  {
  $61 = HEAP32[$1>>2]|0;
  $62 = HEAP8[$61>>0]|0;
  $63 = $62 << 24 >> 24;
  $64 = $63<<24;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 1|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $68 << 16;
  $70 = $64 | $69;
  $71 = HEAP32[$1>>2]|0;
  $72 = ((($71)) + 2|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = $74 << 8;
  $76 = $70 | $75;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 3|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $76 | $80;
  $82 = ($81|0)<(0);
  $83 = $82 << 31 >> 31;
  $84 = HEAP32[$3>>2]|0;
  $85 = $84;
  $86 = $85;
  HEAP32[$86>>2] = $81;
  $87 = (($85) + 4)|0;
  $88 = $87;
  HEAP32[$88>>2] = $83;
  $89 = HEAP32[$3>>2]|0;
  $90 = ((($89)) + 8|0);
  HEAP16[$90>>1] = 4;
  HEAP32[$0>>2] = 4;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 5:  {
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 2|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93&255;
  $95 = $94 << 24;
  $96 = HEAP32[$1>>2]|0;
  $97 = ((($96)) + 2|0);
  $98 = ((($97)) + 1|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = $100 << 16;
  $102 = $95 | $101;
  $103 = HEAP32[$1>>2]|0;
  $104 = ((($103)) + 2|0);
  $105 = ((($104)) + 2|0);
  $106 = HEAP8[$105>>0]|0;
  $107 = $106&255;
  $108 = $107 << 8;
  $109 = $102 | $108;
  $110 = HEAP32[$1>>2]|0;
  $111 = ((($110)) + 2|0);
  $112 = ((($111)) + 3|0);
  $113 = HEAP8[$112>>0]|0;
  $114 = $113&255;
  $115 = $109 | $114;
  $116 = HEAP32[$1>>2]|0;
  $117 = HEAP8[$116>>0]|0;
  $118 = $117 << 24 >> 24;
  $119 = $118<<8;
  $120 = HEAP32[$1>>2]|0;
  $121 = ((($120)) + 1|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $119 | $123;
  $125 = ($124|0)<(0);
  $126 = $125 << 31 >> 31;
  $127 = (___muldi3(0,1,($124|0),($126|0))|0);
  $128 = tempRet0;
  $129 = (_i64Add(($115|0),0,($127|0),($128|0))|0);
  $130 = tempRet0;
  $131 = HEAP32[$3>>2]|0;
  $132 = $131;
  $133 = $132;
  HEAP32[$133>>2] = $129;
  $134 = (($132) + 4)|0;
  $135 = $134;
  HEAP32[$135>>2] = $130;
  $136 = HEAP32[$3>>2]|0;
  $137 = ((($136)) + 8|0);
  HEAP16[$137>>1] = 4;
  HEAP32[$0>>2] = 6;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 7: case 6:  {
  $138 = HEAP32[$1>>2]|0;
  $139 = HEAP32[$2>>2]|0;
  $140 = HEAP32[$3>>2]|0;
  $141 = (_serialGet($138,$139,$140)|0);
  HEAP32[$0>>2] = $141;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 9: case 8:  {
  $142 = HEAP32[$2>>2]|0;
  $143 = (($142) - 8)|0;
  $144 = HEAP32[$3>>2]|0;
  $145 = $144;
  $146 = $145;
  HEAP32[$146>>2] = $143;
  $147 = (($145) + 4)|0;
  $148 = $147;
  HEAP32[$148>>2] = 0;
  $149 = HEAP32[$3>>2]|0;
  $150 = ((($149)) + 8|0);
  HEAP16[$150>>1] = 4;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 default: {
  $151 = HEAP32[$1>>2]|0;
  $152 = HEAP32[$3>>2]|0;
  $153 = ((($152)) + 16|0);
  HEAP32[$153>>2] = $151;
  $154 = HEAP32[$2>>2]|0;
  $155 = (($154) - 12)|0;
  $156 = (($155>>>0) / 2)&-1;
  $157 = HEAP32[$3>>2]|0;
  $158 = ((($157)) + 12|0);
  HEAP32[$158>>2] = $156;
  $159 = HEAP32[$2>>2]|0;
  $160 = $159 & 1;
  $161 = (10024 + ($160<<1)|0);
  $162 = HEAP16[$161>>1]|0;
  $163 = HEAP32[$3>>2]|0;
  $164 = ((($163)) + 8|0);
  HEAP16[$164>>1] = $162;
  $165 = HEAP32[$3>>2]|0;
  $166 = ((($165)) + 12|0);
  $167 = HEAP32[$166>>2]|0;
  HEAP32[$0>>2] = $167;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 }
 return (0)|0;
}
function _sqlite3VdbeSerialType($pMem,$file_format,$pLen) {
 $pMem = $pMem|0;
 $file_format = $file_format|0;
 $pLen = $pLen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flags = 0;
 var $i = 0, $n = 0, $or$cond = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $flags = sp + 20|0;
 $n = sp + 16|0;
 $i = sp + 8|0;
 $u = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $file_format;
 HEAP32[$3>>2] = $pLen;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$flags>>2] = $7;
 $8 = HEAP32[$flags>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  HEAP32[$11>>2] = 0;
  HEAP32[$0>>2] = 0;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $12 = HEAP32[$flags>>2]|0;
 $13 = $12 & 4;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $134 = HEAP32[$flags>>2]|0;
  $135 = $134 & 8;
  $136 = ($135|0)!=(0);
  if ($136) {
   $137 = HEAP32[$3>>2]|0;
   HEAP32[$137>>2] = 8;
   HEAP32[$0>>2] = 7;
   $158 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($158|0);
  }
  $138 = HEAP32[$1>>2]|0;
  $139 = ((($138)) + 12|0);
  $140 = HEAP32[$139>>2]|0;
  HEAP32[$n>>2] = $140;
  $141 = HEAP32[$flags>>2]|0;
  $142 = $141 & 16384;
  $143 = ($142|0)!=(0);
  if ($143) {
   $144 = HEAP32[$1>>2]|0;
   $145 = HEAP32[$144>>2]|0;
   $146 = HEAP32[$n>>2]|0;
   $147 = (($146) + ($145))|0;
   HEAP32[$n>>2] = $147;
  }
  $148 = HEAP32[$n>>2]|0;
  $149 = HEAP32[$3>>2]|0;
  HEAP32[$149>>2] = $148;
  $150 = HEAP32[$n>>2]|0;
  $151 = $150<<1;
  $152 = (($151) + 12)|0;
  $153 = HEAP32[$flags>>2]|0;
  $154 = $153 & 2;
  $155 = ($154|0)!=(0);
  $156 = $155&1;
  $157 = (($152) + ($156))|0;
  HEAP32[$0>>2] = $157;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = $15;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = $i;
 $23 = $22;
 HEAP32[$23>>2] = $18;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = $i;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)<(0);
 $33 = $i;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 if ($32) {
  $39 = $35 ^ -1;
  $40 = $38 ^ -1;
  $41 = $u;
  $42 = $41;
  HEAP32[$42>>2] = $39;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $40;
 } else {
  $45 = $u;
  $46 = $45;
  HEAP32[$46>>2] = $35;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $38;
 }
 $49 = $u;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = (($49) + 4)|0;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = ($54>>>0)<(0);
 $56 = ($51>>>0)<=(127);
 $57 = ($54|0)==(0);
 $58 = $57 & $56;
 $59 = $55 | $58;
 if ($59) {
  $60 = $i;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (($60) + 4)|0;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = $62 & 1;
  $67 = $i;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($66|0)==($69|0);
  $74 = (0)==($72|0);
  $75 = $73 & $74;
  $76 = HEAP32[$2>>2]|0;
  $77 = ($76|0)>=(4);
  $or$cond = $75 & $77;
  $78 = HEAP32[$3>>2]|0;
  if ($or$cond) {
   HEAP32[$78>>2] = 0;
   $79 = $u;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (($79) + 4)|0;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = (8 + ($81))|0;
   HEAP32[$0>>2] = $85;
   $158 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($158|0);
  } else {
   HEAP32[$78>>2] = 1;
   HEAP32[$0>>2] = 1;
   $158 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($158|0);
  }
 }
 $86 = $u;
 $87 = $86;
 $88 = HEAP32[$87>>2]|0;
 $89 = (($86) + 4)|0;
 $90 = $89;
 $91 = HEAP32[$90>>2]|0;
 $92 = ($91>>>0)<(0);
 $93 = ($88>>>0)<=(32767);
 $94 = ($91|0)==(0);
 $95 = $94 & $93;
 $96 = $92 | $95;
 if ($96) {
  $97 = HEAP32[$3>>2]|0;
  HEAP32[$97>>2] = 2;
  HEAP32[$0>>2] = 2;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $98 = $u;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (($98) + 4)|0;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = ($103>>>0)<(0);
 $105 = ($100>>>0)<=(8388607);
 $106 = ($103|0)==(0);
 $107 = $106 & $105;
 $108 = $104 | $107;
 if ($108) {
  $109 = HEAP32[$3>>2]|0;
  HEAP32[$109>>2] = 3;
  HEAP32[$0>>2] = 3;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $110 = $u;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = (($110) + 4)|0;
 $114 = $113;
 $115 = HEAP32[$114>>2]|0;
 $116 = ($115>>>0)<(0);
 $117 = ($112>>>0)<=(2147483647);
 $118 = ($115|0)==(0);
 $119 = $118 & $117;
 $120 = $116 | $119;
 if ($120) {
  $121 = HEAP32[$3>>2]|0;
  HEAP32[$121>>2] = 4;
  HEAP32[$0>>2] = 4;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $122 = $u;
 $123 = $122;
 $124 = HEAP32[$123>>2]|0;
 $125 = (($122) + 4)|0;
 $126 = $125;
 $127 = HEAP32[$126>>2]|0;
 $128 = ($127>>>0)<(32767);
 $129 = ($124>>>0)<=(4294967295);
 $130 = ($127|0)==(32767);
 $131 = $130 & $129;
 $132 = $128 | $131;
 $133 = HEAP32[$3>>2]|0;
 if ($132) {
  HEAP32[$133>>2] = 6;
  HEAP32[$0>>2] = 5;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 } else {
  HEAP32[$133>>2] = 8;
  HEAP32[$0>>2] = 6;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 return (0)|0;
}
function _sqlite3VarintLen($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $i = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 HEAP32[$i>>2] = 1;
 while(1) {
  $7 = $2;
  $8 = $7;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($7) + 4)|0;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (_bitshift64Lshr(($9|0),($12|0),7)|0);
  $14 = tempRet0;
  $15 = $2;
  $16 = $15;
  HEAP32[$16>>2] = $13;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = $14;
  $19 = ($13|0)!=(0);
  $20 = ($14|0)!=(0);
  $21 = $19 | $20;
  $22 = HEAP32[$i>>2]|0;
  if (!($21)) {
   break;
  }
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 STACKTOP = sp;return ($22|0);
}
function _sqlite3PutVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14>>>0)<(0);
 $16 = ($11>>>0)<=(127);
 $17 = ($14|0)==(0);
 $18 = $17 & $16;
 $19 = $15 | $18;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 if ($19) {
  $26 = $22 & 127;
  $27 = $26&255;
  $28 = HEAP32[$3>>2]|0;
  HEAP8[$28>>0] = $27;
  HEAP32[$2>>2] = 1;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $29 = ($25>>>0)<(0);
 $30 = ($22>>>0)<=(16383);
 $31 = ($25|0)==(0);
 $32 = $31 & $30;
 $33 = $29 | $32;
 if ($33) {
  $34 = $4;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (_bitshift64Lshr(($36|0),($39|0),7)|0);
  $41 = tempRet0;
  $42 = $40 & 127;
  $43 = $42 | 128;
  $44 = $43&255;
  $45 = HEAP32[$3>>2]|0;
  HEAP8[$45>>0] = $44;
  $46 = $4;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = $48 & 127;
  $53 = $52&255;
  $54 = HEAP32[$3>>2]|0;
  $55 = ((($54)) + 1|0);
  HEAP8[$55>>0] = $53;
  HEAP32[$2>>2] = 2;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  $56 = HEAP32[$3>>2]|0;
  $57 = $4;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_putVarint64($56,$59,$62)|0);
  HEAP32[$2>>2] = $63;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSerialPut($buf,$pMem,$serial_type) {
 $buf = $buf|0;
 $pMem = $pMem|0;
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $len = 0, $or$cond = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $len = sp + 12|0;
 $v = sp;
 $i = sp + 8|0;
 HEAP32[$1>>2] = $buf;
 HEAP32[$2>>2] = $pMem;
 HEAP32[$3>>2] = $serial_type;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4>>>0)<=(7);
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6>>>0)>(0);
 $or$cond = $5 & $7;
 $8 = HEAP32[$3>>2]|0;
 if ($or$cond) {
  $9 = ($8|0)==(7);
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   ;HEAP32[$v>>2]=HEAP32[$10>>2]|0;HEAP32[$v+4>>2]=HEAP32[$10+4>>2]|0;
  } else {
   $11 = HEAP32[$2>>2]|0;
   $12 = $11;
   $13 = $12;
   $14 = HEAP32[$13>>2]|0;
   $15 = (($12) + 4)|0;
   $16 = $15;
   $17 = HEAP32[$16>>2]|0;
   $18 = $v;
   $19 = $18;
   HEAP32[$19>>2] = $14;
   $20 = (($18) + 4)|0;
   $21 = $20;
   HEAP32[$21>>2] = $17;
  }
  $22 = HEAP32[$3>>2]|0;
  $23 = (21519 + ($22)|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  HEAP32[$i>>2] = $25;
  HEAP32[$len>>2] = $25;
  while(1) {
   $26 = $v;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = $28 & 255;
   $33 = $32&255;
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + -1)|0;
   HEAP32[$i>>2] = $35;
   $36 = HEAP32[$1>>2]|0;
   $37 = (($36) + ($35)|0);
   HEAP8[$37>>0] = $33;
   $38 = $v;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = (($38) + 4)|0;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (_bitshift64Lshr(($40|0),($43|0),8)|0);
   $45 = tempRet0;
   $46 = $v;
   $47 = $46;
   HEAP32[$47>>2] = $44;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = HEAP32[$i>>2]|0;
   $51 = ($50|0)!=(0);
   if (!($51)) {
    break;
   }
  }
  $52 = HEAP32[$len>>2]|0;
  HEAP32[$0>>2] = $52;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 } else {
  $53 = ($8>>>0)>=(12);
  if (!($53)) {
   HEAP32[$0>>2] = 0;
   $65 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($65|0);
  }
  $54 = HEAP32[$2>>2]|0;
  $55 = ((($54)) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  HEAP32[$len>>2] = $56;
  $57 = HEAP32[$len>>2]|0;
  $58 = ($57>>>0)>(0);
  if ($58) {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$len>>2]|0;
   _memcpy(($59|0),($62|0),($63|0))|0;
  }
  $64 = HEAP32[$len>>2]|0;
  HEAP32[$0>>2] = $64;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCount($pCur,$pnEntry) {
 $pCur = $pCur|0;
 $pnEntry = $pnEntry|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iIdx = 0, $nEntry = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $nEntry = sp;
 $rc = sp + 16|0;
 $iIdx = sp + 12|0;
 $pPage = sp + 8|0;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pnEntry;
 $3 = $nEntry;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = $11;
  $13 = $12;
  HEAP32[$13>>2] = 0;
  $14 = (($12) + 4)|0;
  $15 = $14;
  HEAP32[$15>>2] = 0;
  HEAP32[$0>>2] = 0;
  $171 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($171|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (_moveToRoot($16)|0);
 HEAP32[$rc>>2] = $17;
 L5: while(1) {
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)==(0);
  if (!($19)) {
   label = 16;
   break;
  }
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 68|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22 << 24 >> 24;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 120|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pPage>>2] = $27;
  $28 = HEAP32[$pPage>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = ($31|0)!=(0);
  if ($32) {
   label = 7;
  } else {
   $33 = HEAP32[$pPage>>2]|0;
   $34 = ((($33)) + 2|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = ($35<<24>>24)!=(0);
   if (!($36)) {
    label = 7;
   }
  }
  if ((label|0) == 7) {
   label = 0;
   $37 = HEAP32[$pPage>>2]|0;
   $38 = ((($37)) + 18|0);
   $39 = HEAP16[$38>>1]|0;
   $40 = $39&65535;
   $41 = $nEntry;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($41) + 4)|0;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = (_i64Add(($43|0),($46|0),($40|0),0)|0);
   $48 = tempRet0;
   $49 = $nEntry;
   $50 = $49;
   HEAP32[$50>>2] = $47;
   $51 = (($49) + 4)|0;
   $52 = $51;
   HEAP32[$52>>2] = $48;
  }
  $53 = HEAP32[$pPage>>2]|0;
  $54 = ((($53)) + 4|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = ($55<<24>>24)!=(0);
  if ($56) {
   while(1) {
    $57 = HEAP32[$1>>2]|0;
    $58 = ((($57)) + 68|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59 << 24 >> 24;
    $61 = ($60|0)==(0);
    if ($61) {
     label = 10;
     break L5;
    }
    $75 = HEAP32[$1>>2]|0;
    _moveToParent($75);
    $76 = HEAP32[$1>>2]|0;
    $77 = ((($76)) + 68|0);
    $78 = HEAP8[$77>>0]|0;
    $79 = $78 << 24 >> 24;
    $80 = HEAP32[$1>>2]|0;
    $81 = ((($80)) + 80|0);
    $82 = (($81) + ($79<<1)|0);
    $83 = HEAP16[$82>>1]|0;
    $84 = $83&65535;
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 68|0);
    $87 = HEAP8[$86>>0]|0;
    $88 = $87 << 24 >> 24;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 120|0);
    $91 = (($90) + ($88<<2)|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ((($92)) + 18|0);
    $94 = HEAP16[$93>>1]|0;
    $95 = $94&65535;
    $96 = ($84|0)>=($95|0);
    if (!($96)) {
     break;
    }
   }
   $97 = HEAP32[$1>>2]|0;
   $98 = ((($97)) + 68|0);
   $99 = HEAP8[$98>>0]|0;
   $100 = $99 << 24 >> 24;
   $101 = HEAP32[$1>>2]|0;
   $102 = ((($101)) + 80|0);
   $103 = (($102) + ($100<<1)|0);
   $104 = HEAP16[$103>>1]|0;
   $105 = (($104) + 1)<<16>>16;
   HEAP16[$103>>1] = $105;
   $106 = HEAP32[$1>>2]|0;
   $107 = ((($106)) + 68|0);
   $108 = HEAP8[$107>>0]|0;
   $109 = $108 << 24 >> 24;
   $110 = HEAP32[$1>>2]|0;
   $111 = ((($110)) + 120|0);
   $112 = (($111) + ($109<<2)|0);
   $113 = HEAP32[$112>>2]|0;
   HEAP32[$pPage>>2] = $113;
  }
  $114 = HEAP32[$1>>2]|0;
  $115 = ((($114)) + 68|0);
  $116 = HEAP8[$115>>0]|0;
  $117 = $116 << 24 >> 24;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 80|0);
  $120 = (($119) + ($117<<1)|0);
  $121 = HEAP16[$120>>1]|0;
  $122 = $121&65535;
  HEAP32[$iIdx>>2] = $122;
  $123 = HEAP32[$iIdx>>2]|0;
  $124 = HEAP32[$pPage>>2]|0;
  $125 = ((($124)) + 18|0);
  $126 = HEAP16[$125>>1]|0;
  $127 = $126&65535;
  $128 = ($123|0)==($127|0);
  $129 = HEAP32[$1>>2]|0;
  $130 = HEAP32[$pPage>>2]|0;
  if ($128) {
   $131 = ((($130)) + 5|0);
   $132 = HEAP8[$131>>0]|0;
   $133 = $132&255;
   $134 = (($133) + 8)|0;
   $135 = HEAP32[$pPage>>2]|0;
   $136 = ((($135)) + 56|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = (($137) + ($134)|0);
   $139 = (_sqlite3Get4byte($138)|0);
   $140 = (_moveToChild($129,$139)|0);
   HEAP32[$rc>>2] = $140;
   continue;
  } else {
   $141 = ((($130)) + 56|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = HEAP32[$pPage>>2]|0;
   $144 = ((($143)) + 20|0);
   $145 = HEAP16[$144>>1]|0;
   $146 = $145&65535;
   $147 = HEAP32[$iIdx>>2]|0;
   $148 = $147<<1;
   $149 = HEAP32[$pPage>>2]|0;
   $150 = ((($149)) + 64|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = (($151) + ($148)|0);
   $153 = HEAP8[$152>>0]|0;
   $154 = $153&255;
   $155 = $154 << 8;
   $156 = HEAP32[$iIdx>>2]|0;
   $157 = $156<<1;
   $158 = HEAP32[$pPage>>2]|0;
   $159 = ((($158)) + 64|0);
   $160 = HEAP32[$159>>2]|0;
   $161 = (($160) + ($157)|0);
   $162 = ((($161)) + 1|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163&255;
   $165 = $155 | $164;
   $166 = $146 & $165;
   $167 = (($142) + ($166)|0);
   $168 = (_sqlite3Get4byte($167)|0);
   $169 = (_moveToChild($129,$168)|0);
   HEAP32[$rc>>2] = $169;
   continue;
  }
 }
 if ((label|0) == 10) {
  $62 = $nEntry;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  $69 = $68;
  $70 = $69;
  HEAP32[$70>>2] = $64;
  $71 = (($69) + 4)|0;
  $72 = $71;
  HEAP32[$72>>2] = $67;
  $73 = HEAP32[$1>>2]|0;
  $74 = (_moveToRoot($73)|0);
  HEAP32[$0>>2] = $74;
  $171 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($171|0);
 }
 else if ((label|0) == 16) {
  $170 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $170;
  $171 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($171|0);
 }
 return (0)|0;
}
function _sqlite3BtreeBeginStmt($p,$iStatement) {
 $p = $p|0;
 $iStatement = $iStatement|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iStatement;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3PagerOpenSavepoint($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3ResetOneSchema($db,$iDb) {
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDb = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 HEAP32[$pDb>>2] = $6;
 $7 = HEAP32[$pDb>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 _sqlite3SchemaClear($9);
 $10 = HEAP32[$1>>2]|0;
 $11 = ($10|0)!=(1);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP32[$pDb>>2] = $15;
 $16 = HEAP32[$pDb>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3SchemaClear($18);
 STACKTOP = sp;return;
}
function _allocateCursor($p,$iCur,$nField,$iDb,$eCurType) {
 $p = $p|0;
 $iCur = $iCur|0;
 $nField = $nField|0;
 $iDb = $iDb|0;
 $eCurType = $eCurType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $nByte = 0, $pCx = 0, $pMem = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 28|0;
 $pMem = sp + 8|0;
 $nByte = sp + 4|0;
 $pCx = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $nField;
 HEAP32[$3>>2] = $iDb;
 HEAP8[$4>>0] = $eCurType;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($7) - ($8))|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*40)|0)|0);
 HEAP32[$pMem>>2] = $13;
 HEAP32[$pCx>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = $14<<3;
 $16 = (88 + ($15))|0;
 $17 = HEAP8[$4>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = (_sqlite3BtreeCursorSize()|0);
  $22 = $20;
 } else {
  $22 = 0;
 }
 $21 = (($16) + ($22))|0;
 HEAP32[$nByte>>2] = $21;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 56|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($23<<2)|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 56|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31<<2)|0);
  $36 = HEAP32[$35>>2]|0;
  _sqlite3VdbeFreeCursor($30,$36);
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 56|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + ($37<<2)|0);
  HEAP32[$41>>2] = 0;
 }
 $42 = HEAP32[$pMem>>2]|0;
 $43 = HEAP32[$nByte>>2]|0;
 $44 = (_sqlite3VdbeMemClearAndResize($42,$43)|0);
 $45 = (0)==($44|0);
 if (!($45)) {
  $86 = HEAP32[$pCx>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 $46 = HEAP32[$pMem>>2]|0;
 $47 = ((($46)) + 16|0);
 $48 = HEAP32[$47>>2]|0;
 HEAP32[$pCx>>2] = $48;
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 56|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + ($49<<2)|0);
 HEAP32[$53>>2] = $48;
 $54 = HEAP32[$pCx>>2]|0;
 dest=$54; stop=dest+88|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $55 = HEAP8[$4>>0]|0;
 $56 = HEAP32[$pCx>>2]|0;
 HEAP8[$56>>0] = $55;
 $57 = HEAP32[$3>>2]|0;
 $58 = $57&255;
 $59 = HEAP32[$pCx>>2]|0;
 $60 = ((($59)) + 1|0);
 HEAP8[$60>>0] = $58;
 $61 = HEAP32[$2>>2]|0;
 $62 = $61&65535;
 $63 = HEAP32[$pCx>>2]|0;
 $64 = ((($63)) + 12|0);
 HEAP16[$64>>1] = $62;
 $65 = HEAP32[$2>>2]|0;
 $66 = HEAP32[$pCx>>2]|0;
 $67 = ((($66)) + 80|0);
 $68 = (($67) + ($65<<2)|0);
 $69 = HEAP32[$pCx>>2]|0;
 $70 = ((($69)) + 76|0);
 HEAP32[$70>>2] = $68;
 $71 = HEAP8[$4>>0]|0;
 $72 = $71&255;
 $73 = ($72|0)==(0);
 if (!($73)) {
  $86 = HEAP32[$pCx>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 $74 = HEAP32[$2>>2]|0;
 $75 = $74<<3;
 $76 = (88 + ($75))|0;
 $77 = HEAP32[$pMem>>2]|0;
 $78 = ((($77)) + 16|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = (($79) + ($76)|0);
 $81 = HEAP32[$pCx>>2]|0;
 $82 = ((($81)) + 16|0);
 HEAP32[$82>>2] = $80;
 $83 = HEAP32[$pCx>>2]|0;
 $84 = ((($83)) + 16|0);
 $85 = HEAP32[$84>>2]|0;
 _sqlite3BtreeCursorZero($85);
 $86 = HEAP32[$pCx>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _sqlite3BtreeCursor($p,$iTable,$wrFlag,$pKeyInfo,$pCur) {
 $p = $p|0;
 $iTable = $iTable|0;
 $wrFlag = $wrFlag|0;
 $pKeyInfo = $pKeyInfo|0;
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $wrFlag;
 HEAP32[$3>>2] = $pKeyInfo;
 HEAP32[$4>>2] = $pCur;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<(1);
 if ($6) {
  $7 = (_sqlite3CorruptError(59722)|0);
  HEAP32[$rc>>2] = $7;
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3BtreeEnter($8);
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$4>>2]|0;
  $14 = (_btreeCursor($9,$10,$11,$12,$13)|0);
  HEAP32[$rc>>2] = $14;
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCursorHintFlags($pCur,$x) {
 $pCur = $pCur|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2&255;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 67|0);
 HEAP8[$5>>0] = $3;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCreateTable($p,$piTable,$flags) {
 $p = $p|0;
 $piTable = $piTable|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piTable;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_btreeCreateTable($4,$5,$6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeSorterInit($db,$nField,$pCsr) {
 $db = $db|0;
 $nField = $nField|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $mxCache = 0;
 var $pKeyInfo = 0, $pSorter = 0, $pTask = 0, $pgsz = 0, $rc = 0, $sz = 0, $szKeyInfo = 0, $szPma = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $pgsz = sp + 36|0;
 $i = sp + 32|0;
 $mxCache = sp + 28|0;
 $pSorter = sp + 24|0;
 $pKeyInfo = sp + 20|0;
 $szKeyInfo = sp + 16|0;
 $sz = sp + 12|0;
 $rc = sp + 8|0;
 $pTask = sp + 4|0;
 $szPma = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nField;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 6|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = (($8) - 1)|0;
 $10 = $9<<2;
 $11 = (24 + ($10))|0;
 HEAP32[$szKeyInfo>>2] = $11;
 HEAP32[$sz>>2] = 136;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$sz>>2]|0;
 $14 = HEAP32[$szKeyInfo>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = ($15|0)<(0);
 $17 = $16 << 31 >> 31;
 $18 = (_sqlite3DbMallocZero($12,$15,$17)|0);
 HEAP32[$pSorter>>2] = $18;
 $19 = HEAP32[$pSorter>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 16|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pSorter>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  HEAP32[$rc>>2] = 7;
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $24 = HEAP32[$pSorter>>2]|0;
 $25 = HEAP32[$sz>>2]|0;
 $26 = (($24) + ($25)|0);
 HEAP32[$pKeyInfo>>2] = $26;
 $27 = HEAP32[$pSorter>>2]|0;
 $28 = ((($27)) + 28|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pKeyInfo>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 $31 = ((($30)) + 24|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$szKeyInfo>>2]|0;
 _memcpy(($29|0),($32|0),($33|0))|0;
 $34 = HEAP32[$pKeyInfo>>2]|0;
 $35 = ((($34)) + 12|0);
 HEAP32[$35>>2] = 0;
 $36 = HEAP32[$1>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$pKeyInfo>>2]|0;
  $39 = ((($38)) + 6|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40&65535;
  $42 = HEAP32[$1>>2]|0;
  $43 = (($41) - ($42))|0;
  $44 = HEAP32[$pKeyInfo>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46&65535;
  $48 = (($47) + ($43))|0;
  $49 = $48&65535;
  HEAP16[$45>>1] = $49;
  $50 = HEAP32[$1>>2]|0;
  $51 = $50&65535;
  $52 = HEAP32[$pKeyInfo>>2]|0;
  $53 = ((($52)) + 6|0);
  HEAP16[$53>>1] = $51;
 }
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 16|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = (_sqlite3BtreeGetPageSize($58)|0);
 HEAP32[$pgsz>>2] = $59;
 $60 = HEAP32[$pSorter>>2]|0;
 $61 = ((($60)) + 12|0);
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$pSorter>>2]|0;
 $63 = ((($62)) + 59|0);
 HEAP8[$63>>0] = 1;
 $64 = HEAP32[$pSorter>>2]|0;
 $65 = ((($64)) + 58|0);
 HEAP8[$65>>0] = -1;
 $66 = HEAP32[$pSorter>>2]|0;
 $67 = ((($66)) + 59|0);
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = ($69|0)>(1);
 $71 = $70&1;
 $72 = $71&255;
 $73 = HEAP32[$pSorter>>2]|0;
 $74 = ((($73)) + 57|0);
 HEAP8[$74>>0] = $72;
 $75 = HEAP32[$0>>2]|0;
 $76 = HEAP32[$pSorter>>2]|0;
 $77 = ((($76)) + 24|0);
 HEAP32[$77>>2] = $75;
 HEAP32[$i>>2] = 0;
 while(1) {
  $78 = HEAP32[$i>>2]|0;
  $79 = HEAP32[$pSorter>>2]|0;
  $80 = ((($79)) + 59|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = $81&255;
  $83 = ($78|0)<($82|0);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$i>>2]|0;
  $85 = HEAP32[$pSorter>>2]|0;
  $86 = ((($85)) + 64|0);
  $87 = (($86) + (($84*72)|0)|0);
  HEAP32[$pTask>>2] = $87;
  $88 = HEAP32[$pSorter>>2]|0;
  $89 = HEAP32[$pTask>>2]|0;
  $90 = ((($89)) + 8|0);
  HEAP32[$90>>2] = $88;
  $91 = HEAP32[$i>>2]|0;
  $92 = (($91) + 1)|0;
  HEAP32[$i>>2] = $92;
 }
 $93 = HEAP32[$0>>2]|0;
 $94 = (_sqlite3TempInMemory($93)|0);
 $95 = ($94|0)!=(0);
 if (!($95)) {
  $96 = HEAP32[(232)>>2]|0;
  HEAP32[$szPma>>2] = $96;
  $97 = HEAP32[$szPma>>2]|0;
  $98 = HEAP32[$pgsz>>2]|0;
  $99 = Math_imul($97, $98)|0;
  $100 = HEAP32[$pSorter>>2]|0;
  HEAP32[$100>>2] = $99;
  $101 = HEAP32[$0>>2]|0;
  $102 = ((($101)) + 16|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = ((($103)) + 12|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = ((($105)) + 80|0);
  $107 = HEAP32[$106>>2]|0;
  HEAP32[$mxCache>>2] = $107;
  $108 = HEAP32[$mxCache>>2]|0;
  $109 = HEAP32[$szPma>>2]|0;
  $110 = ($108|0)<($109|0);
  if ($110) {
   $111 = HEAP32[$szPma>>2]|0;
   HEAP32[$mxCache>>2] = $111;
  }
  $112 = HEAP32[$mxCache>>2]|0;
  $113 = ($112|0)<(0);
  $114 = $113 << 31 >> 31;
  $115 = HEAP32[$pgsz>>2]|0;
  $116 = ($115|0)<(0);
  $117 = $116 << 31 >> 31;
  $118 = (___muldi3(($112|0),($114|0),($115|0),($117|0))|0);
  $119 = tempRet0;
  $120 = ($119|0)<(0);
  $121 = ($118>>>0)<(536870912);
  $122 = ($119|0)==(0);
  $123 = $122 & $121;
  $124 = $120 | $123;
  if ($124) {
   $125 = HEAP32[$mxCache>>2]|0;
   $126 = ($125|0)<(0);
   $127 = $126 << 31 >> 31;
   $128 = HEAP32[$pgsz>>2]|0;
   $129 = ($128|0)<(0);
   $130 = $129 << 31 >> 31;
   $131 = (___muldi3(($125|0),($127|0),($128|0),($130|0))|0);
   $132 = tempRet0;
   $135 = $131;$177 = $132;
  } else {
   $135 = 536870912;$177 = 0;
  }
  $133 = HEAP32[$pSorter>>2]|0;
  $134 = ((($133)) + 4|0);
  HEAP32[$134>>2] = $135;
  $136 = HEAP32[(200)>>2]|0;
  $137 = ($136|0)==(0|0);
  if ($137) {
   $138 = HEAP32[$pgsz>>2]|0;
   $139 = HEAP32[$pSorter>>2]|0;
   $140 = ((($139)) + 52|0);
   HEAP32[$140>>2] = $138;
   $141 = HEAP32[$pgsz>>2]|0;
   $142 = ($141|0)<(0);
   $143 = $142 << 31 >> 31;
   $144 = (_sqlite3Malloc($141,$143)|0);
   $145 = HEAP32[$pSorter>>2]|0;
   $146 = ((($145)) + 36|0);
   $147 = ((($146)) + 4|0);
   HEAP32[$147>>2] = $144;
   $148 = HEAP32[$pSorter>>2]|0;
   $149 = ((($148)) + 36|0);
   $150 = ((($149)) + 4|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = ($151|0)!=(0|0);
   if (!($152)) {
    HEAP32[$rc>>2] = 7;
   }
  }
 }
 $153 = HEAP32[$pKeyInfo>>2]|0;
 $154 = ((($153)) + 6|0);
 $155 = HEAP16[$154>>1]|0;
 $156 = $155&65535;
 $157 = HEAP32[$pKeyInfo>>2]|0;
 $158 = ((($157)) + 8|0);
 $159 = HEAP16[$158>>1]|0;
 $160 = $159&65535;
 $161 = (($156) + ($160))|0;
 $162 = ($161|0)<(13);
 if (!($162)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $163 = HEAP32[$pKeyInfo>>2]|0;
 $164 = ((($163)) + 20|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = ($165|0)==(0|0);
 if (!($166)) {
  $167 = HEAP32[$pKeyInfo>>2]|0;
  $168 = ((($167)) + 20|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = HEAP32[$0>>2]|0;
  $171 = ((($170)) + 8|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = ($169|0)==($172|0);
  if (!($173)) {
   $176 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($176|0);
  }
 }
 $174 = HEAP32[$pSorter>>2]|0;
 $175 = ((($174)) + 60|0);
 HEAP8[$175>>0] = 3;
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _sqlite3BtreeMovetoUnpacked($pCur,$pIdxKey,$0,$1,$biasRight,$pRes) {
 $pCur = $pCur|0;
 $pIdxKey = $pIdxKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $biasRight = $biasRight|0;
 $pRes = $pRes|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $chldPg = 0, $idx = 0, $lwr = 0, $nCell = 0;
 var $nCellKey = 0, $pCell = 0, $pCellBody = 0, $pCellKey = 0, $pPage = 0, $rc = 0, $upr = 0, $xRecordCompare = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 8|0;
 $6 = sp + 68|0;
 $7 = sp + 64|0;
 $rc = sp + 60|0;
 $xRecordCompare = sp + 56|0;
 $lwr = sp + 52|0;
 $upr = sp + 48|0;
 $idx = sp + 44|0;
 $c = sp + 40|0;
 $chldPg = sp + 36|0;
 $pPage = sp + 32|0;
 $pCell = sp + 28|0;
 $nCellKey = sp;
 $nCell = sp + 24|0;
 $pCellKey = sp + 20|0;
 $pCellBody = sp + 16|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pIdxKey;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $biasRight;
 HEAP32[$7>>2] = $pRes;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 66|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(1);
 if ($16) {
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 64|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = $20 & 2;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$3>>2]|0;
   $24 = ((($23)) + 69|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)!=(0);
   if ($27) {
    $28 = HEAP32[$3>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = $29;
    $31 = $30;
    $32 = HEAP32[$31>>2]|0;
    $33 = (($30) + 4)|0;
    $34 = $33;
    $35 = HEAP32[$34>>2]|0;
    $36 = $5;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = (($36) + 4)|0;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = ($32|0)==($38|0);
    $43 = ($35|0)==($41|0);
    $44 = $42 & $43;
    if ($44) {
     $45 = HEAP32[$7>>2]|0;
     HEAP32[$45>>2] = 0;
     HEAP32[$2>>2] = 0;
     $476 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($476|0);
    }
    $46 = HEAP32[$3>>2]|0;
    $47 = ((($46)) + 64|0);
    $48 = HEAP8[$47>>0]|0;
    $49 = $48&255;
    $50 = $49 & 8;
    $51 = ($50|0)!=(0);
    if ($51) {
     $52 = HEAP32[$3>>2]|0;
     $53 = ((($52)) + 16|0);
     $54 = $53;
     $55 = $54;
     $56 = HEAP32[$55>>2]|0;
     $57 = (($54) + 4)|0;
     $58 = $57;
     $59 = HEAP32[$58>>2]|0;
     $60 = $5;
     $61 = $60;
     $62 = HEAP32[$61>>2]|0;
     $63 = (($60) + 4)|0;
     $64 = $63;
     $65 = HEAP32[$64>>2]|0;
     $66 = ($59|0)<($65|0);
     $67 = ($56>>>0)<($62>>>0);
     $68 = ($59|0)==($65|0);
     $69 = $68 & $67;
     $70 = $66 | $69;
     if ($70) {
      $71 = HEAP32[$7>>2]|0;
      HEAP32[$71>>2] = -1;
      HEAP32[$2>>2] = 0;
      $476 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($476|0);
     }
    }
   }
  }
 }
 $72 = HEAP32[$4>>2]|0;
 $73 = ($72|0)!=(0|0);
 if ($73) {
  $74 = HEAP32[$4>>2]|0;
  $75 = (_sqlite3VdbeFindCompare($74)|0);
  HEAP32[$xRecordCompare>>2] = $75;
  $76 = HEAP32[$4>>2]|0;
  $77 = ((($76)) + 11|0);
  HEAP8[$77>>0] = 0;
 } else {
  HEAP32[$xRecordCompare>>2] = 0;
 }
 $78 = HEAP32[$3>>2]|0;
 $79 = (_moveToRoot($78)|0);
 HEAP32[$rc>>2] = $79;
 $80 = HEAP32[$rc>>2]|0;
 $81 = ($80|0)!=(0);
 if ($81) {
  $82 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $82;
  $476 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($476|0);
 }
 $83 = HEAP32[$3>>2]|0;
 $84 = ((($83)) + 66|0);
 $85 = HEAP8[$84>>0]|0;
 $86 = $85&255;
 $87 = ($86|0)==(0);
 if ($87) {
  $88 = HEAP32[$7>>2]|0;
  HEAP32[$88>>2] = -1;
  HEAP32[$2>>2] = 0;
  $476 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($476|0);
 }
 L24: while(1) {
  $89 = HEAP32[$3>>2]|0;
  $90 = ((($89)) + 68|0);
  $91 = HEAP8[$90>>0]|0;
  $92 = $91 << 24 >> 24;
  $93 = HEAP32[$3>>2]|0;
  $94 = ((($93)) + 120|0);
  $95 = (($94) + ($92<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  HEAP32[$pPage>>2] = $96;
  HEAP32[$lwr>>2] = 0;
  $97 = HEAP32[$pPage>>2]|0;
  $98 = ((($97)) + 18|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $99&65535;
  $101 = (($100) - 1)|0;
  HEAP32[$upr>>2] = $101;
  $102 = HEAP32[$upr>>2]|0;
  $103 = HEAP32[$6>>2]|0;
  $104 = (1 - ($103))|0;
  $105 = $102 >> $104;
  HEAP32[$idx>>2] = $105;
  $106 = HEAP32[$idx>>2]|0;
  $107 = $106&65535;
  $108 = HEAP32[$3>>2]|0;
  $109 = ((($108)) + 68|0);
  $110 = HEAP8[$109>>0]|0;
  $111 = $110 << 24 >> 24;
  $112 = HEAP32[$3>>2]|0;
  $113 = ((($112)) + 80|0);
  $114 = (($113) + ($111<<1)|0);
  HEAP16[$114>>1] = $107;
  $115 = HEAP32[$xRecordCompare>>2]|0;
  $116 = ($115|0)==(0|0);
  L26: do {
   if ($116) {
    while(1) {
     $117 = HEAP32[$pPage>>2]|0;
     $118 = ((($117)) + 68|0);
     $119 = HEAP32[$118>>2]|0;
     $120 = HEAP32[$pPage>>2]|0;
     $121 = ((($120)) + 20|0);
     $122 = HEAP16[$121>>1]|0;
     $123 = $122&65535;
     $124 = HEAP32[$idx>>2]|0;
     $125 = $124<<1;
     $126 = HEAP32[$pPage>>2]|0;
     $127 = ((($126)) + 64|0);
     $128 = HEAP32[$127>>2]|0;
     $129 = (($128) + ($125)|0);
     $130 = HEAP8[$129>>0]|0;
     $131 = $130&255;
     $132 = $131 << 8;
     $133 = HEAP32[$idx>>2]|0;
     $134 = $133<<1;
     $135 = HEAP32[$pPage>>2]|0;
     $136 = ((($135)) + 64|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = (($137) + ($134)|0);
     $139 = ((($138)) + 1|0);
     $140 = HEAP8[$139>>0]|0;
     $141 = $140&255;
     $142 = $132 | $141;
     $143 = $123 & $142;
     $144 = (($119) + ($143)|0);
     HEAP32[$pCell>>2] = $144;
     $145 = HEAP32[$pPage>>2]|0;
     $146 = ((($145)) + 3|0);
     $147 = HEAP8[$146>>0]|0;
     $148 = ($147<<24>>24)!=(0);
     L29: do {
      if ($148) {
       while(1) {
        $149 = HEAP32[$pCell>>2]|0;
        $150 = ((($149)) + 1|0);
        HEAP32[$pCell>>2] = $150;
        $151 = HEAP8[$149>>0]|0;
        $152 = $151&255;
        $153 = (128)<=($152|0);
        if (!($153)) {
         break L29;
        }
        $154 = HEAP32[$pCell>>2]|0;
        $155 = HEAP32[$pPage>>2]|0;
        $156 = ((($155)) + 60|0);
        $157 = HEAP32[$156>>2]|0;
        $158 = ($154>>>0)>=($157>>>0);
        if ($158) {
         label = 20;
         break L24;
        }
       }
      }
     } while(0);
     $160 = HEAP32[$pCell>>2]|0;
     (_sqlite3GetVarint($160,$nCellKey)|0);
     $161 = $nCellKey;
     $162 = $161;
     $163 = HEAP32[$162>>2]|0;
     $164 = (($161) + 4)|0;
     $165 = $164;
     $166 = HEAP32[$165>>2]|0;
     $167 = $5;
     $168 = $167;
     $169 = HEAP32[$168>>2]|0;
     $170 = (($167) + 4)|0;
     $171 = $170;
     $172 = HEAP32[$171>>2]|0;
     $173 = ($166|0)<($172|0);
     $174 = ($163>>>0)<($169>>>0);
     $175 = ($166|0)==($172|0);
     $176 = $175 & $174;
     $177 = $173 | $176;
     if ($177) {
      $178 = HEAP32[$idx>>2]|0;
      $179 = (($178) + 1)|0;
      HEAP32[$lwr>>2] = $179;
      $180 = HEAP32[$lwr>>2]|0;
      $181 = HEAP32[$upr>>2]|0;
      $182 = ($180|0)>($181|0);
      if ($182) {
       label = 23;
       break;
      }
     } else {
      $183 = $nCellKey;
      $184 = $183;
      $185 = HEAP32[$184>>2]|0;
      $186 = (($183) + 4)|0;
      $187 = $186;
      $188 = HEAP32[$187>>2]|0;
      $189 = $5;
      $190 = $189;
      $191 = HEAP32[$190>>2]|0;
      $192 = (($189) + 4)|0;
      $193 = $192;
      $194 = HEAP32[$193>>2]|0;
      $195 = ($188|0)>($194|0);
      $196 = ($185>>>0)>($191>>>0);
      $197 = ($188|0)==($194|0);
      $198 = $197 & $196;
      $199 = $195 | $198;
      if (!($199)) {
       label = 27;
       break;
      }
      $200 = HEAP32[$idx>>2]|0;
      $201 = (($200) - 1)|0;
      HEAP32[$upr>>2] = $201;
      $202 = HEAP32[$lwr>>2]|0;
      $203 = HEAP32[$upr>>2]|0;
      $204 = ($202|0)>($203|0);
      if ($204) {
       label = 26;
       break;
      }
     }
     $238 = HEAP32[$lwr>>2]|0;
     $239 = HEAP32[$upr>>2]|0;
     $240 = (($238) + ($239))|0;
     $241 = $240 >> 1;
     HEAP32[$idx>>2] = $241;
    }
    if ((label|0) == 23) {
     label = 0;
     HEAP32[$c>>2] = -1;
     label = 51;
     break;
    }
    else if ((label|0) == 26) {
     label = 0;
     HEAP32[$c>>2] = 1;
     label = 51;
     break;
    }
    else if ((label|0) == 27) {
     label = 0;
     $205 = HEAP32[$3>>2]|0;
     $206 = ((($205)) + 64|0);
     $207 = HEAP8[$206>>0]|0;
     $208 = $207&255;
     $209 = $208 | 2;
     $210 = $209&255;
     HEAP8[$206>>0] = $210;
     $211 = $nCellKey;
     $212 = $211;
     $213 = HEAP32[$212>>2]|0;
     $214 = (($211) + 4)|0;
     $215 = $214;
     $216 = HEAP32[$215>>2]|0;
     $217 = HEAP32[$3>>2]|0;
     $218 = ((($217)) + 16|0);
     $219 = $218;
     $220 = $219;
     HEAP32[$220>>2] = $213;
     $221 = (($219) + 4)|0;
     $222 = $221;
     HEAP32[$222>>2] = $216;
     $223 = HEAP32[$idx>>2]|0;
     $224 = $223&65535;
     $225 = HEAP32[$3>>2]|0;
     $226 = ((($225)) + 68|0);
     $227 = HEAP8[$226>>0]|0;
     $228 = $227 << 24 >> 24;
     $229 = HEAP32[$3>>2]|0;
     $230 = ((($229)) + 80|0);
     $231 = (($230) + ($228<<1)|0);
     HEAP16[$231>>1] = $224;
     $232 = HEAP32[$pPage>>2]|0;
     $233 = ((($232)) + 4|0);
     $234 = HEAP8[$233>>0]|0;
     $235 = ($234<<24>>24)!=(0);
     if ($235) {
      label = 29;
      break L24;
     }
     $236 = HEAP32[$idx>>2]|0;
     HEAP32[$lwr>>2] = $236;
     break;
    }
   } else {
    while(1) {
     $242 = HEAP32[$pPage>>2]|0;
     $243 = ((($242)) + 68|0);
     $244 = HEAP32[$243>>2]|0;
     $245 = HEAP32[$pPage>>2]|0;
     $246 = ((($245)) + 20|0);
     $247 = HEAP16[$246>>1]|0;
     $248 = $247&65535;
     $249 = HEAP32[$idx>>2]|0;
     $250 = $249<<1;
     $251 = HEAP32[$pPage>>2]|0;
     $252 = ((($251)) + 64|0);
     $253 = HEAP32[$252>>2]|0;
     $254 = (($253) + ($250)|0);
     $255 = HEAP8[$254>>0]|0;
     $256 = $255&255;
     $257 = $256 << 8;
     $258 = HEAP32[$idx>>2]|0;
     $259 = $258<<1;
     $260 = HEAP32[$pPage>>2]|0;
     $261 = ((($260)) + 64|0);
     $262 = HEAP32[$261>>2]|0;
     $263 = (($262) + ($259)|0);
     $264 = ((($263)) + 1|0);
     $265 = HEAP8[$264>>0]|0;
     $266 = $265&255;
     $267 = $257 | $266;
     $268 = $248 & $267;
     $269 = (($244) + ($268)|0);
     HEAP32[$pCell>>2] = $269;
     $270 = HEAP32[$pCell>>2]|0;
     $271 = HEAP8[$270>>0]|0;
     $272 = $271&255;
     HEAP32[$nCell>>2] = $272;
     $273 = HEAP32[$nCell>>2]|0;
     $274 = HEAP32[$pPage>>2]|0;
     $275 = ((($274)) + 7|0);
     $276 = HEAP8[$275>>0]|0;
     $277 = $276&255;
     $278 = ($273|0)<=($277|0);
     do {
      if ($278) {
       $279 = HEAP32[$xRecordCompare>>2]|0;
       $280 = HEAP32[$nCell>>2]|0;
       $281 = HEAP32[$pCell>>2]|0;
       $282 = ((($281)) + 1|0);
       $283 = HEAP32[$4>>2]|0;
       $284 = (FUNCTION_TABLE_iiii[$279 & 255]($280,$282,$283)|0);
       HEAP32[$c>>2] = $284;
      } else {
       $285 = HEAP32[$pCell>>2]|0;
       $286 = ((($285)) + 1|0);
       $287 = HEAP8[$286>>0]|0;
       $288 = $287&255;
       $289 = $288 & 128;
       $290 = ($289|0)!=(0);
       if (!($290)) {
        $291 = HEAP32[$nCell>>2]|0;
        $292 = $291 & 127;
        $293 = $292 << 7;
        $294 = HEAP32[$pCell>>2]|0;
        $295 = ((($294)) + 1|0);
        $296 = HEAP8[$295>>0]|0;
        $297 = $296&255;
        $298 = (($293) + ($297))|0;
        HEAP32[$nCell>>2] = $298;
        $299 = HEAP32[$pPage>>2]|0;
        $300 = ((($299)) + 10|0);
        $301 = HEAP16[$300>>1]|0;
        $302 = $301&65535;
        $303 = ($298|0)<=($302|0);
        if ($303) {
         $304 = HEAP32[$xRecordCompare>>2]|0;
         $305 = HEAP32[$nCell>>2]|0;
         $306 = HEAP32[$pCell>>2]|0;
         $307 = ((($306)) + 2|0);
         $308 = HEAP32[$4>>2]|0;
         $309 = (FUNCTION_TABLE_iiii[$304 & 255]($305,$307,$308)|0);
         HEAP32[$c>>2] = $309;
         break;
        }
       }
       $310 = HEAP32[$pCell>>2]|0;
       $311 = HEAP32[$pPage>>2]|0;
       $312 = ((($311)) + 6|0);
       $313 = HEAP8[$312>>0]|0;
       $314 = $313&255;
       $315 = (0 - ($314))|0;
       $316 = (($310) + ($315)|0);
       HEAP32[$pCellBody>>2] = $316;
       $317 = HEAP32[$pPage>>2]|0;
       $318 = ((($317)) + 80|0);
       $319 = HEAP32[$318>>2]|0;
       $320 = HEAP32[$pPage>>2]|0;
       $321 = HEAP32[$pCellBody>>2]|0;
       $322 = HEAP32[$3>>2]|0;
       $323 = ((($322)) + 16|0);
       FUNCTION_TABLE_viii[$319 & 255]($320,$321,$323);
       $324 = HEAP32[$3>>2]|0;
       $325 = ((($324)) + 16|0);
       $326 = $325;
       $327 = $326;
       $328 = HEAP32[$327>>2]|0;
       $329 = (($326) + 4)|0;
       $330 = $329;
       $331 = HEAP32[$330>>2]|0;
       HEAP32[$nCell>>2] = $328;
       $332 = HEAP32[$nCell>>2]|0;
       $333 = ($332|0)<(2);
       if ($333) {
        label = 37;
        break L24;
       }
       $335 = HEAP32[$nCell>>2]|0;
       $336 = (($335) + 18)|0;
       $337 = ($336|0)<(0);
       $338 = $337 << 31 >> 31;
       $339 = (_sqlite3Malloc($336,$338)|0);
       HEAP32[$pCellKey>>2] = $339;
       $340 = HEAP32[$pCellKey>>2]|0;
       $341 = ($340|0)==(0|0);
       if ($341) {
        label = 39;
        break L24;
       }
       $342 = HEAP32[$idx>>2]|0;
       $343 = $342&65535;
       $344 = HEAP32[$3>>2]|0;
       $345 = ((($344)) + 68|0);
       $346 = HEAP8[$345>>0]|0;
       $347 = $346 << 24 >> 24;
       $348 = HEAP32[$3>>2]|0;
       $349 = ((($348)) + 80|0);
       $350 = (($349) + ($347<<1)|0);
       HEAP16[$350>>1] = $343;
       $351 = HEAP32[$3>>2]|0;
       $352 = HEAP32[$nCell>>2]|0;
       $353 = HEAP32[$pCellKey>>2]|0;
       $354 = (_accessPayload($351,0,$352,$353,2)|0);
       HEAP32[$rc>>2] = $354;
       $355 = HEAP32[$rc>>2]|0;
       $356 = ($355|0)!=(0);
       if ($356) {
        label = 41;
        break L24;
       }
       $358 = HEAP32[$xRecordCompare>>2]|0;
       $359 = HEAP32[$nCell>>2]|0;
       $360 = HEAP32[$pCellKey>>2]|0;
       $361 = HEAP32[$4>>2]|0;
       $362 = (FUNCTION_TABLE_iiii[$358 & 255]($359,$360,$361)|0);
       HEAP32[$c>>2] = $362;
       $363 = HEAP32[$pCellKey>>2]|0;
       _sqlite3_free($363);
      }
     } while(0);
     $364 = HEAP32[$c>>2]|0;
     $365 = ($364|0)<(0);
     if ($365) {
      $366 = HEAP32[$idx>>2]|0;
      $367 = (($366) + 1)|0;
      HEAP32[$lwr>>2] = $367;
     } else {
      $368 = HEAP32[$c>>2]|0;
      $369 = ($368|0)>(0);
      if (!($369)) {
       label = 47;
       break L24;
      }
      $370 = HEAP32[$idx>>2]|0;
      $371 = (($370) - 1)|0;
      HEAP32[$upr>>2] = $371;
     }
     $386 = HEAP32[$lwr>>2]|0;
     $387 = HEAP32[$upr>>2]|0;
     $388 = ($386|0)>($387|0);
     if ($388) {
      label = 51;
      break L26;
     }
     $389 = HEAP32[$lwr>>2]|0;
     $390 = HEAP32[$upr>>2]|0;
     $391 = (($389) + ($390))|0;
     $392 = $391 >> 1;
     HEAP32[$idx>>2] = $392;
    }
   }
  } while(0);
  if ((label|0) == 51) {
   label = 0;
   $393 = HEAP32[$pPage>>2]|0;
   $394 = ((($393)) + 4|0);
   $395 = HEAP8[$394>>0]|0;
   $396 = ($395<<24>>24)!=(0);
   if ($396) {
    label = 52;
    break;
   }
  }
  $408 = HEAP32[$lwr>>2]|0;
  $409 = HEAP32[$pPage>>2]|0;
  $410 = ((($409)) + 18|0);
  $411 = HEAP16[$410>>1]|0;
  $412 = $411&65535;
  $413 = ($408|0)>=($412|0);
  $414 = HEAP32[$pPage>>2]|0;
  if ($413) {
   $415 = ((($414)) + 5|0);
   $416 = HEAP8[$415>>0]|0;
   $417 = $416&255;
   $418 = (($417) + 8)|0;
   $419 = HEAP32[$pPage>>2]|0;
   $420 = ((($419)) + 56|0);
   $421 = HEAP32[$420>>2]|0;
   $422 = (($421) + ($418)|0);
   $423 = (_sqlite3Get4byte($422)|0);
   HEAP32[$chldPg>>2] = $423;
  } else {
   $424 = ((($414)) + 56|0);
   $425 = HEAP32[$424>>2]|0;
   $426 = HEAP32[$pPage>>2]|0;
   $427 = ((($426)) + 20|0);
   $428 = HEAP16[$427>>1]|0;
   $429 = $428&65535;
   $430 = HEAP32[$lwr>>2]|0;
   $431 = $430<<1;
   $432 = HEAP32[$pPage>>2]|0;
   $433 = ((($432)) + 64|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = (($434) + ($431)|0);
   $436 = HEAP8[$435>>0]|0;
   $437 = $436&255;
   $438 = $437 << 8;
   $439 = HEAP32[$lwr>>2]|0;
   $440 = $439<<1;
   $441 = HEAP32[$pPage>>2]|0;
   $442 = ((($441)) + 64|0);
   $443 = HEAP32[$442>>2]|0;
   $444 = (($443) + ($440)|0);
   $445 = ((($444)) + 1|0);
   $446 = HEAP8[$445>>0]|0;
   $447 = $446&255;
   $448 = $438 | $447;
   $449 = $429 & $448;
   $450 = (($425) + ($449)|0);
   $451 = (_sqlite3Get4byte($450)|0);
   HEAP32[$chldPg>>2] = $451;
  }
  $452 = HEAP32[$lwr>>2]|0;
  $453 = $452&65535;
  $454 = HEAP32[$3>>2]|0;
  $455 = ((($454)) + 68|0);
  $456 = HEAP8[$455>>0]|0;
  $457 = $456 << 24 >> 24;
  $458 = HEAP32[$3>>2]|0;
  $459 = ((($458)) + 80|0);
  $460 = (($459) + ($457<<1)|0);
  HEAP16[$460>>1] = $453;
  $461 = HEAP32[$3>>2]|0;
  $462 = HEAP32[$chldPg>>2]|0;
  $463 = (_moveToChild($461,$462)|0);
  HEAP32[$rc>>2] = $463;
  $464 = HEAP32[$rc>>2]|0;
  $465 = ($464|0)!=(0);
  if ($465) {
   break;
  }
 }
 if ((label|0) == 20) {
  $159 = (_sqlite3CorruptError(60709)|0);
  HEAP32[$2>>2] = $159;
  $476 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($476|0);
 }
 else if ((label|0) == 29) {
  $237 = HEAP32[$7>>2]|0;
  HEAP32[$237>>2] = 0;
  HEAP32[$rc>>2] = 0;
 }
 else if ((label|0) == 37) {
  $334 = (_sqlite3CorruptError(60782)|0);
  HEAP32[$rc>>2] = $334;
 }
 else if ((label|0) == 39) {
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 41) {
  $357 = HEAP32[$pCellKey>>2]|0;
  _sqlite3_free($357);
 }
 else if ((label|0) == 47) {
  $372 = HEAP32[$7>>2]|0;
  HEAP32[$372>>2] = 0;
  HEAP32[$rc>>2] = 0;
  $373 = HEAP32[$idx>>2]|0;
  $374 = $373&65535;
  $375 = HEAP32[$3>>2]|0;
  $376 = ((($375)) + 68|0);
  $377 = HEAP8[$376>>0]|0;
  $378 = $377 << 24 >> 24;
  $379 = HEAP32[$3>>2]|0;
  $380 = ((($379)) + 80|0);
  $381 = (($380) + ($378<<1)|0);
  HEAP16[$381>>1] = $374;
  $382 = HEAP32[$4>>2]|0;
  $383 = ((($382)) + 11|0);
  $384 = HEAP8[$383>>0]|0;
  $385 = ($384<<24>>24)!=(0);
  if ($385) {
   HEAP32[$rc>>2] = 11;
  }
 }
 else if ((label|0) == 52) {
  $397 = HEAP32[$idx>>2]|0;
  $398 = $397&65535;
  $399 = HEAP32[$3>>2]|0;
  $400 = ((($399)) + 68|0);
  $401 = HEAP8[$400>>0]|0;
  $402 = $401 << 24 >> 24;
  $403 = HEAP32[$3>>2]|0;
  $404 = ((($403)) + 80|0);
  $405 = (($404) + ($402<<1)|0);
  HEAP16[$405>>1] = $398;
  $406 = HEAP32[$c>>2]|0;
  $407 = HEAP32[$7>>2]|0;
  HEAP32[$407>>2] = $406;
  HEAP32[$rc>>2] = 0;
 }
 $466 = HEAP32[$3>>2]|0;
 $467 = ((($466)) + 16|0);
 $468 = ((($467)) + 18|0);
 HEAP16[$468>>1] = 0;
 $469 = HEAP32[$3>>2]|0;
 $470 = ((($469)) + 64|0);
 $471 = HEAP8[$470>>0]|0;
 $472 = $471&255;
 $473 = $472 & -7;
 $474 = $473&255;
 HEAP8[$470>>0] = $474;
 $475 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $475;
 $476 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($476|0);
}
function _sqlite3BtreeCursorHasHint($pCsr,$mask) {
 $pCsr = $pCsr|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $mask;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 67|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = $5 & $6;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3BtreeNext($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = ((($4)) + 18|0);
 HEAP16[$5>>1] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 64|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & -7;
 $11 = $10&255;
 HEAP8[$7>>0] = $11;
 $12 = HEAP32[$2>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 66|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(1);
 $18 = HEAP32[$1>>2]|0;
 if ($17) {
  $19 = HEAP32[$2>>2]|0;
  $20 = (_btreeNext($18,$19)|0);
  HEAP32[$0>>2] = $20;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $21 = ((($18)) + 68|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22 << 24 >> 24;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 120|0);
 $26 = (($25) + ($23<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pPage>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 68|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30 << 24 >> 24;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 80|0);
 $34 = (($33) + ($31<<1)|0);
 $35 = HEAP16[$34>>1]|0;
 $36 = (($35) + 1)<<16>>16;
 HEAP16[$34>>1] = $36;
 $37 = $36&65535;
 $38 = HEAP32[$pPage>>2]|0;
 $39 = ((($38)) + 18|0);
 $40 = HEAP16[$39>>1]|0;
 $41 = $40&65535;
 $42 = ($37|0)>=($41|0);
 if ($42) {
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 68|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 80|0);
  $49 = (($48) + ($46<<1)|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = (($50) + -1)<<16>>16;
  HEAP16[$49>>1] = $51;
  $52 = HEAP32[$1>>2]|0;
  $53 = HEAP32[$2>>2]|0;
  $54 = (_btreeNext($52,$53)|0);
  HEAP32[$0>>2] = $54;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $55 = HEAP32[$pPage>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = HEAP8[$56>>0]|0;
 $58 = ($57<<24>>24)!=(0);
 if ($58) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $59 = HEAP32[$1>>2]|0;
  $60 = (_moveToLeftmost($59)|0);
  HEAP32[$0>>2] = $60;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _sqlite3BtreePrevious($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 64|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & -15;
 $9 = $8&255;
 HEAP8[$5>>0] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = ((($11)) + 18|0);
 HEAP16[$12>>1] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 66|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(1);
 if (!($17)) {
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 68|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 80|0);
  $24 = (($23) + ($21<<1)|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = ($26|0)==(0);
  if (!($27)) {
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 68|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30 << 24 >> 24;
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 120|0);
   $34 = (($33) + ($31<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ((($35)) + 4|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)==(0);
   if (!($39)) {
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 68|0);
    $45 = HEAP8[$44>>0]|0;
    $46 = $45 << 24 >> 24;
    $47 = HEAP32[$1>>2]|0;
    $48 = ((($47)) + 80|0);
    $49 = (($48) + ($46<<1)|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = (($50) + -1)<<16>>16;
    HEAP16[$49>>1] = $51;
    HEAP32[$0>>2] = 0;
    $52 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($52|0);
   }
  }
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = (_btreePrevious($40,$41)|0);
 HEAP32[$0>>2] = $42;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3BtreeEof($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 66|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = (1)!=($4|0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeAllocUnpackedRecord($pKeyInfo,$pSpace,$szSpace,$ppFree) {
 $pKeyInfo = $pKeyInfo|0;
 $pSpace = $pSpace|0;
 $szSpace = $szSpace|0;
 $ppFree = $ppFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $nOff = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 $nOff = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pKeyInfo;
 HEAP32[$2>>2] = $pSpace;
 HEAP32[$3>>2] = $szSpace;
 HEAP32[$4>>2] = $ppFree;
 $5 = HEAP32[$2>>2]|0;
 $6 = $5;
 $7 = $6 & 7;
 $8 = (8 - ($7))|0;
 $9 = $8 & 7;
 HEAP32[$nOff>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 6|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = (($13) + 1)|0;
 $15 = ($14*40)|0;
 $16 = (16 + ($15))|0;
 HEAP32[$nByte>>2] = $16;
 $17 = HEAP32[$nByte>>2]|0;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$nOff>>2]|0;
 $20 = (($18) + ($19))|0;
 $21 = ($17|0)>($20|0);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$nByte>>2]|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (_sqlite3DbMallocRaw($24,$25,$27)|0);
  HEAP32[$p>>2] = $28;
  $29 = HEAP32[$p>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  HEAP32[$30>>2] = $29;
  $31 = HEAP32[$p>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 } else {
  $33 = HEAP32[$nOff>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + ($33)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$4>>2]|0;
  HEAP32[$36>>2] = 0;
 }
 $37 = HEAP32[$p>>2]|0;
 $38 = ((($37)) + 16|0);
 $39 = HEAP32[$p>>2]|0;
 $40 = ((($39)) + 4|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$p>>2]|0;
 HEAP32[$42>>2] = $41;
 $43 = HEAP32[$1>>2]|0;
 $44 = ((($43)) + 6|0);
 $45 = HEAP16[$44>>1]|0;
 $46 = $45&65535;
 $47 = (($46) + 1)|0;
 $48 = $47&65535;
 $49 = HEAP32[$p>>2]|0;
 $50 = ((($49)) + 8|0);
 HEAP16[$50>>1] = $48;
 $51 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3VdbeRecordUnpack($pKeyInfo,$nKey,$pKey,$p) {
 $pKeyInfo = $pKeyInfo|0;
 $nKey = $nKey|0;
 $pKey = $pKey|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $aKey = 0, $d = 0, $idx = 0, $pMem = 0, $serial_type = 0, $szHdr = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aKey = sp + 20|0;
 $d = sp + 16|0;
 $idx = sp + 12|0;
 $u = sp + 40|0;
 $szHdr = sp + 8|0;
 $pMem = sp + 4|0;
 $serial_type = sp;
 HEAP32[$0>>2] = $pKeyInfo;
 HEAP32[$1>>2] = $nKey;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $p;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$aKey>>2] = $4;
 $5 = HEAP32[$3>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pMem>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 10|0);
 HEAP8[$9>>0] = 0;
 $10 = HEAP32[$aKey>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)<(128);
 $14 = HEAP32[$aKey>>2]|0;
 if ($13) {
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  HEAP32[$szHdr>>2] = $16;
  $20 = 1;
 } else {
  $17 = (_sqlite3GetVarint32($14,$szHdr)|0);
  $18 = $17&255;
  $20 = $18;
 }
 $19 = $20&255;
 $21 = $19&255;
 HEAP32[$idx>>2] = $21;
 $22 = HEAP32[$szHdr>>2]|0;
 HEAP32[$d>>2] = $22;
 HEAP16[$u>>1] = 0;
 while(1) {
  $23 = HEAP32[$idx>>2]|0;
  $24 = HEAP32[$szHdr>>2]|0;
  $25 = ($23>>>0)<($24>>>0);
  if (!($25)) {
   label = 11;
   break;
  }
  $26 = HEAP32[$d>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ($26|0)<=($27|0);
  if (!($28)) {
   label = 11;
   break;
  }
  $29 = HEAP32[$idx>>2]|0;
  $30 = HEAP32[$aKey>>2]|0;
  $31 = (($30) + ($29)|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)<(128);
  $35 = HEAP32[$idx>>2]|0;
  $36 = HEAP32[$aKey>>2]|0;
  $37 = (($36) + ($35)|0);
  if ($34) {
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   HEAP32[$serial_type>>2] = $39;
   $43 = 1;
  } else {
   $40 = (_sqlite3GetVarint32($37,$serial_type)|0);
   $41 = $40&255;
   $43 = $41;
  }
  $42 = $43&255;
  $44 = $42&255;
  $45 = HEAP32[$idx>>2]|0;
  $46 = (($45) + ($44))|0;
  HEAP32[$idx>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 4|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = HEAP32[$pMem>>2]|0;
  $51 = ((($50)) + 10|0);
  HEAP8[$51>>0] = $49;
  $52 = HEAP32[$0>>2]|0;
  $53 = ((($52)) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$pMem>>2]|0;
  $56 = ((($55)) + 32|0);
  HEAP32[$56>>2] = $54;
  $57 = HEAP32[$pMem>>2]|0;
  $58 = ((($57)) + 24|0);
  HEAP32[$58>>2] = 0;
  $59 = HEAP32[$d>>2]|0;
  $60 = HEAP32[$aKey>>2]|0;
  $61 = (($60) + ($59)|0);
  $62 = HEAP32[$serial_type>>2]|0;
  $63 = HEAP32[$pMem>>2]|0;
  $64 = (_sqlite3VdbeSerialGet($61,$62,$63)|0);
  $65 = HEAP32[$d>>2]|0;
  $66 = (($65) + ($64))|0;
  HEAP32[$d>>2] = $66;
  $67 = HEAP32[$pMem>>2]|0;
  $68 = ((($67)) + 40|0);
  HEAP32[$pMem>>2] = $68;
  $69 = HEAP16[$u>>1]|0;
  $70 = (($69) + 1)<<16>>16;
  HEAP16[$u>>1] = $70;
  $71 = $70&65535;
  $72 = HEAP32[$3>>2]|0;
  $73 = ((($72)) + 8|0);
  $74 = HEAP16[$73>>1]|0;
  $75 = $74&65535;
  $76 = ($71|0)>=($75|0);
  if ($76) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  $77 = HEAP16[$u>>1]|0;
  $78 = HEAP32[$3>>2]|0;
  $79 = ((($78)) + 8|0);
  HEAP16[$79>>1] = $77;
  STACKTOP = sp;return;
 }
}
function _sqlite3BtreeLast($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (1)==($6|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = $11 & 8;
  $13 = ($12|0)!=(0);
  if ($13) {
   HEAP32[$0>>2] = 0;
   $37 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_moveToRoot($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 66|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = (0)==($21|0);
   $23 = HEAP32[$2>>2]|0;
   if ($22) {
    HEAP32[$23>>2] = 1;
    break;
   }
   HEAP32[$23>>2] = 0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_moveToRightmost($24)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)==(0);
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 64|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   if ($27) {
    $32 = $31 | 8;
    $33 = $32&255;
    HEAP8[$29>>0] = $33;
    break;
   } else {
    $34 = $31 & -9;
    $35 = $34&255;
    HEAP8[$29>>0] = $35;
    break;
   }
  }
 } while(0);
 $36 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3BtreeInsert($pCur,$pKey,$0,$1,$pData,$nData,$nZero,$appendBias,$seekResult) {
 $pCur = $pCur|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $pData = $pData|0;
 $nData = $nData|0;
 $nZero = $nZero|0;
 $appendBias = $appendBias|0;
 $seekResult = $seekResult|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $idx = 0, $loc = 0, $newCell = 0, $oldCell = 0, $or$cond = 0;
 var $p = 0, $pBt = 0, $pPage = 0, $rc = 0, $szNew = 0, $szOld = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $8 = sp + 52|0;
 $9 = sp + 48|0;
 $10 = sp + 44|0;
 $rc = sp + 40|0;
 $loc = sp + 36|0;
 $szNew = sp + 32|0;
 $idx = sp + 28|0;
 $pPage = sp + 24|0;
 $p = sp + 20|0;
 $pBt = sp + 16|0;
 $oldCell = sp + 12|0;
 $newCell = sp + 8|0;
 $szOld = sp + 76|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pKey;
 $11 = $5;
 $12 = $11;
 HEAP32[$12>>2] = $0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $1;
 HEAP32[$6>>2] = $pData;
 HEAP32[$7>>2] = $nData;
 HEAP32[$8>>2] = $nZero;
 HEAP32[$9>>2] = $appendBias;
 HEAP32[$10>>2] = $seekResult;
 $15 = HEAP32[$10>>2]|0;
 HEAP32[$loc>>2] = $15;
 HEAP32[$szNew>>2] = 0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$p>>2] = $17;
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pBt>>2] = $20;
 HEAP32[$newCell>>2] = 0;
 $21 = HEAP32[$3>>2]|0;
 $22 = ((($21)) + 66|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)==(4);
 $26 = HEAP32[$3>>2]|0;
 if ($25) {
  $27 = ((($26)) + 60|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$2>>2] = $28;
  $256 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($256|0);
 }
 $29 = ((($26)) + 64|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $31 & 32;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$pBt>>2]|0;
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 52|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = (_saveAllCursors($34,$37,$38)|0);
  HEAP32[$rc>>2] = $39;
  $40 = HEAP32[$rc>>2]|0;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $42;
   $256 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($256|0);
  }
 }
 $43 = HEAP32[$3>>2]|0;
 $44 = ((($43)) + 72|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)==(0|0);
 do {
  if ($46) {
   $47 = HEAP32[$p>>2]|0;
   $48 = $5;
   $49 = $48;
   $50 = HEAP32[$49>>2]|0;
   $51 = (($48) + 4)|0;
   $52 = $51;
   $53 = HEAP32[$52>>2]|0;
   _invalidateIncrblobCursors($47,$50,$53,0);
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 64|0);
   $56 = HEAP8[$55>>0]|0;
   $57 = $56&255;
   $58 = $57 & 2;
   $59 = ($58|0)!=(0);
   $60 = $5;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($60) + 4)|0;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = ($65|0)>(0);
   $67 = ($62>>>0)>(0);
   $68 = ($65|0)==(0);
   $69 = $68 & $67;
   $70 = $66 | $69;
   $or$cond = $59 & $70;
   if ($or$cond) {
    $71 = HEAP32[$3>>2]|0;
    $72 = ((($71)) + 16|0);
    $73 = $72;
    $74 = $73;
    $75 = HEAP32[$74>>2]|0;
    $76 = (($73) + 4)|0;
    $77 = $76;
    $78 = HEAP32[$77>>2]|0;
    $79 = $5;
    $80 = $79;
    $81 = HEAP32[$80>>2]|0;
    $82 = (($79) + 4)|0;
    $83 = $82;
    $84 = HEAP32[$83>>2]|0;
    $85 = (_i64Subtract(($81|0),($84|0),1,0)|0);
    $86 = tempRet0;
    $87 = ($75|0)==($85|0);
    $88 = ($78|0)==($86|0);
    $89 = $87 & $88;
    if ($89) {
     HEAP32[$loc>>2] = -1;
     break;
    }
   }
   $90 = HEAP32[$loc>>2]|0;
   $91 = ($90|0)==(0);
   if ($91) {
    $92 = HEAP32[$3>>2]|0;
    $93 = $5;
    $94 = $93;
    $95 = HEAP32[$94>>2]|0;
    $96 = (($93) + 4)|0;
    $97 = $96;
    $98 = HEAP32[$97>>2]|0;
    $99 = HEAP32[$9>>2]|0;
    $100 = (_sqlite3BtreeMovetoUnpacked($92,0,$95,$98,$99,$loc)|0);
    HEAP32[$rc>>2] = $100;
    $101 = HEAP32[$rc>>2]|0;
    $102 = ($101|0)!=(0);
    if ($102) {
     $103 = HEAP32[$rc>>2]|0;
     HEAP32[$2>>2] = $103;
     $256 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($256|0);
    }
   }
  } else {
   $104 = HEAP32[$loc>>2]|0;
   $105 = ($104|0)==(0);
   if ($105) {
    $106 = HEAP32[$3>>2]|0;
    $107 = HEAP32[$4>>2]|0;
    $108 = $5;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (($108) + 4)|0;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = HEAP32[$9>>2]|0;
    $115 = (_btreeMoveto($106,$107,$110,$113,$114,$loc)|0);
    HEAP32[$rc>>2] = $115;
    $116 = HEAP32[$rc>>2]|0;
    $117 = ($116|0)!=(0);
    if ($117) {
     $118 = HEAP32[$rc>>2]|0;
     HEAP32[$2>>2] = $118;
     $256 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($256|0);
    }
   }
  }
 } while(0);
 $119 = HEAP32[$3>>2]|0;
 $120 = ((($119)) + 68|0);
 $121 = HEAP8[$120>>0]|0;
 $122 = $121 << 24 >> 24;
 $123 = HEAP32[$3>>2]|0;
 $124 = ((($123)) + 120|0);
 $125 = (($124) + ($122<<2)|0);
 $126 = HEAP32[$125>>2]|0;
 HEAP32[$pPage>>2] = $126;
 $127 = HEAP32[$pBt>>2]|0;
 $128 = ((($127)) + 80|0);
 $129 = HEAP32[$128>>2]|0;
 HEAP32[$newCell>>2] = $129;
 $130 = HEAP32[$pPage>>2]|0;
 $131 = HEAP32[$newCell>>2]|0;
 $132 = HEAP32[$4>>2]|0;
 $133 = $5;
 $134 = $133;
 $135 = HEAP32[$134>>2]|0;
 $136 = (($133) + 4)|0;
 $137 = $136;
 $138 = HEAP32[$137>>2]|0;
 $139 = HEAP32[$6>>2]|0;
 $140 = HEAP32[$7>>2]|0;
 $141 = HEAP32[$8>>2]|0;
 $142 = (_fillInCell($130,$131,$132,$135,$138,$139,$140,$141,$szNew)|0);
 HEAP32[$rc>>2] = $142;
 $143 = HEAP32[$rc>>2]|0;
 $144 = ($143|0)!=(0);
 do {
  if (!($144)) {
   $145 = HEAP32[$3>>2]|0;
   $146 = ((($145)) + 68|0);
   $147 = HEAP8[$146>>0]|0;
   $148 = $147 << 24 >> 24;
   $149 = HEAP32[$3>>2]|0;
   $150 = ((($149)) + 80|0);
   $151 = (($150) + ($148<<1)|0);
   $152 = HEAP16[$151>>1]|0;
   $153 = $152&65535;
   HEAP32[$idx>>2] = $153;
   $154 = HEAP32[$loc>>2]|0;
   $155 = ($154|0)==(0);
   if ($155) {
    $156 = HEAP32[$pPage>>2]|0;
    $157 = ((($156)) + 72|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = (_sqlite3PagerWrite($158)|0);
    HEAP32[$rc>>2] = $159;
    $160 = HEAP32[$rc>>2]|0;
    $161 = ($160|0)!=(0);
    if ($161) {
     break;
    }
    $162 = HEAP32[$pPage>>2]|0;
    $163 = ((($162)) + 56|0);
    $164 = HEAP32[$163>>2]|0;
    $165 = HEAP32[$pPage>>2]|0;
    $166 = ((($165)) + 20|0);
    $167 = HEAP16[$166>>1]|0;
    $168 = $167&65535;
    $169 = HEAP32[$idx>>2]|0;
    $170 = $169<<1;
    $171 = HEAP32[$pPage>>2]|0;
    $172 = ((($171)) + 64|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = (($173) + ($170)|0);
    $175 = HEAP8[$174>>0]|0;
    $176 = $175&255;
    $177 = $176 << 8;
    $178 = HEAP32[$idx>>2]|0;
    $179 = $178<<1;
    $180 = HEAP32[$pPage>>2]|0;
    $181 = ((($180)) + 64|0);
    $182 = HEAP32[$181>>2]|0;
    $183 = (($182) + ($179)|0);
    $184 = ((($183)) + 1|0);
    $185 = HEAP8[$184>>0]|0;
    $186 = $185&255;
    $187 = $177 | $186;
    $188 = $168 & $187;
    $189 = (($164) + ($188)|0);
    HEAP32[$oldCell>>2] = $189;
    $190 = HEAP32[$pPage>>2]|0;
    $191 = ((($190)) + 4|0);
    $192 = HEAP8[$191>>0]|0;
    $193 = ($192<<24>>24)!=(0);
    if (!($193)) {
     $194 = HEAP32[$newCell>>2]|0;
     $195 = HEAP32[$oldCell>>2]|0;
     ;HEAP8[$194>>0]=HEAP8[$195>>0]|0;HEAP8[$194+1>>0]=HEAP8[$195+1>>0]|0;HEAP8[$194+2>>0]=HEAP8[$195+2>>0]|0;HEAP8[$194+3>>0]=HEAP8[$195+3>>0]|0;
    }
    $196 = HEAP32[$pPage>>2]|0;
    $197 = HEAP32[$oldCell>>2]|0;
    $198 = (_clearCell($196,$197,$szOld)|0);
    HEAP32[$rc>>2] = $198;
    $199 = HEAP32[$pPage>>2]|0;
    $200 = HEAP32[$idx>>2]|0;
    $201 = HEAP16[$szOld>>1]|0;
    $202 = $201&65535;
    _dropCell($199,$200,$202,$rc);
    $203 = HEAP32[$rc>>2]|0;
    $204 = ($203|0)!=(0);
    if ($204) {
     break;
    }
   } else {
    $205 = HEAP32[$loc>>2]|0;
    $206 = ($205|0)<(0);
    if ($206) {
     $207 = HEAP32[$pPage>>2]|0;
     $208 = ((($207)) + 18|0);
     $209 = HEAP16[$208>>1]|0;
     $210 = $209&65535;
     $211 = ($210|0)>(0);
     if ($211) {
      $212 = HEAP32[$3>>2]|0;
      $213 = ((($212)) + 68|0);
      $214 = HEAP8[$213>>0]|0;
      $215 = $214 << 24 >> 24;
      $216 = HEAP32[$3>>2]|0;
      $217 = ((($216)) + 80|0);
      $218 = (($217) + ($215<<1)|0);
      $219 = HEAP16[$218>>1]|0;
      $220 = (($219) + 1)<<16>>16;
      HEAP16[$218>>1] = $220;
      $221 = $220&65535;
      HEAP32[$idx>>2] = $221;
     }
    }
   }
   $222 = HEAP32[$pPage>>2]|0;
   $223 = HEAP32[$idx>>2]|0;
   $224 = HEAP32[$newCell>>2]|0;
   $225 = HEAP32[$szNew>>2]|0;
   _insertCell($222,$223,$224,$225,0,0,$rc);
   $226 = HEAP32[$3>>2]|0;
   $227 = ((($226)) + 16|0);
   $228 = ((($227)) + 18|0);
   HEAP16[$228>>1] = 0;
   $229 = HEAP32[$rc>>2]|0;
   $230 = ($229|0)==(0);
   if ($230) {
    $231 = HEAP32[$pPage>>2]|0;
    $232 = ((($231)) + 1|0);
    $233 = HEAP8[$232>>0]|0;
    $234 = $233&255;
    $235 = ($234|0)!=(0);
    if ($235) {
     $236 = HEAP32[$3>>2]|0;
     $237 = ((($236)) + 64|0);
     $238 = HEAP8[$237>>0]|0;
     $239 = $238&255;
     $240 = $239 & -3;
     $241 = $240&255;
     HEAP8[$237>>0] = $241;
     $242 = HEAP32[$3>>2]|0;
     $243 = (_balance($242)|0);
     HEAP32[$rc>>2] = $243;
     $244 = HEAP32[$3>>2]|0;
     $245 = ((($244)) + 68|0);
     $246 = HEAP8[$245>>0]|0;
     $247 = $246 << 24 >> 24;
     $248 = HEAP32[$3>>2]|0;
     $249 = ((($248)) + 120|0);
     $250 = (($249) + ($247<<2)|0);
     $251 = HEAP32[$250>>2]|0;
     $252 = ((($251)) + 1|0);
     HEAP8[$252>>0] = 0;
     $253 = HEAP32[$3>>2]|0;
     $254 = ((($253)) + 66|0);
     HEAP8[$254>>0] = 0;
    }
   }
  }
 } while(0);
 $255 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $255;
 $256 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($256|0);
}
function _sqlite3BtreeDelete($pCur,$flags) {
 $pCur = $pCur|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bPreserve = 0, $bSkipnext = 0, $iCellDepth = 0, $iCellIdx = 0, $n = 0, $nCell = 0, $notUsed = 0, $p = 0, $pBt = 0, $pCell = 0, $pLeaf = 0, $pPage = 0;
 var $pTmp = 0, $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 63|0;
 $p = sp + 48|0;
 $pBt = sp + 44|0;
 $rc = sp + 40|0;
 $pPage = sp + 36|0;
 $pCell = sp + 32|0;
 $iCellIdx = sp + 28|0;
 $iCellDepth = sp + 24|0;
 $szCell = sp + 60|0;
 $bSkipnext = sp + 20|0;
 $bPreserve = sp + 62|0;
 $notUsed = sp + 16|0;
 $pLeaf = sp + 12|0;
 $nCell = sp + 8|0;
 $n = sp + 4|0;
 $pTmp = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP8[$2>>0] = $flags;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 HEAP32[$bSkipnext>>2] = 0;
 $8 = HEAP8[$2>>0]|0;
 $9 = $8&255;
 $10 = $9 & 2;
 $11 = $10&255;
 HEAP8[$bPreserve>>0] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 68|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14 << 24 >> 24;
 HEAP32[$iCellDepth>>2] = $15;
 $16 = HEAP32[$iCellDepth>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 80|0);
 $19 = (($18) + ($16<<1)|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 HEAP32[$iCellIdx>>2] = $21;
 $22 = HEAP32[$iCellDepth>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 120|0);
 $25 = (($24) + ($22<<2)|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pPage>>2] = $26;
 $27 = HEAP32[$pPage>>2]|0;
 $28 = ((($27)) + 56|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$pPage>>2]|0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP16[$31>>1]|0;
 $33 = $32&65535;
 $34 = HEAP32[$iCellIdx>>2]|0;
 $35 = $34<<1;
 $36 = HEAP32[$pPage>>2]|0;
 $37 = ((($36)) + 64|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35)|0);
 $40 = HEAP8[$39>>0]|0;
 $41 = $40&255;
 $42 = $41 << 8;
 $43 = HEAP32[$iCellIdx>>2]|0;
 $44 = $43<<1;
 $45 = HEAP32[$pPage>>2]|0;
 $46 = ((($45)) + 64|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44)|0);
 $49 = ((($48)) + 1|0);
 $50 = HEAP8[$49>>0]|0;
 $51 = $50&255;
 $52 = $42 | $51;
 $53 = $33 & $52;
 $54 = (($29) + ($53)|0);
 HEAP32[$pCell>>2] = $54;
 $55 = HEAP32[$pPage>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = HEAP8[$56>>0]|0;
 $58 = ($57<<24>>24)!=(0);
 if (!($58)) {
  HEAP32[$notUsed>>2] = 0;
  $59 = HEAP32[$1>>2]|0;
  $60 = (_sqlite3BtreePrevious($59,$notUsed)|0);
  HEAP32[$rc>>2] = $60;
  $61 = HEAP32[$rc>>2]|0;
  $62 = ($61|0)!=(0);
  if ($62) {
   $63 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $63;
   $287 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($287|0);
  }
 }
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 64|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = $66&255;
 $68 = $67 & 32;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$pBt>>2]|0;
  $71 = HEAP32[$1>>2]|0;
  $72 = ((($71)) + 52|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$1>>2]|0;
  $75 = (_saveAllCursors($70,$73,$74)|0);
  HEAP32[$rc>>2] = $75;
  $76 = HEAP32[$rc>>2]|0;
  $77 = ($76|0)!=(0);
  if ($77) {
   $78 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $78;
   $287 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($287|0);
  }
 }
 $79 = HEAP32[$1>>2]|0;
 $80 = ((($79)) + 72|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = ($81|0)==(0|0);
 if ($82) {
  $83 = HEAP32[$p>>2]|0;
  $84 = HEAP32[$1>>2]|0;
  $85 = ((($84)) + 16|0);
  $86 = $85;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (($86) + 4)|0;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  _invalidateIncrblobCursors($83,$88,$91,0);
 }
 $92 = HEAP8[$bPreserve>>0]|0;
 $93 = ($92<<24>>24)!=(0);
 do {
  if ($93) {
   $94 = HEAP32[$pPage>>2]|0;
   $95 = ((($94)) + 4|0);
   $96 = HEAP8[$95>>0]|0;
   $97 = ($96<<24>>24)!=(0);
   if ($97) {
    $98 = HEAP32[$pPage>>2]|0;
    $99 = ((($98)) + 16|0);
    $100 = HEAP16[$99>>1]|0;
    $101 = $100&65535;
    $102 = HEAP32[$pPage>>2]|0;
    $103 = HEAP32[$pCell>>2]|0;
    $104 = (_cellSizePtr($102,$103)|0);
    $105 = $104&65535;
    $106 = (($101) + ($105))|0;
    $107 = (($106) + 2)|0;
    $108 = HEAP32[$pBt>>2]|0;
    $109 = ((($108)) + 36|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = $110<<1;
    $112 = (($111>>>0) / 3)&-1;
    $113 = ($107|0)>($112|0);
    if (!($113)) {
     HEAP32[$bSkipnext>>2] = 1;
     break;
    }
   }
   $114 = HEAP32[$1>>2]|0;
   $115 = (_saveCursorKey($114)|0);
   HEAP32[$rc>>2] = $115;
   $116 = HEAP32[$rc>>2]|0;
   $117 = ($116|0)!=(0);
   if ($117) {
    $118 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $118;
    $287 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($287|0);
   }
  }
 } while(0);
 $119 = HEAP32[$pPage>>2]|0;
 $120 = ((($119)) + 72|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = (_sqlite3PagerWrite($121)|0);
 HEAP32[$rc>>2] = $122;
 $123 = HEAP32[$rc>>2]|0;
 $124 = ($123|0)!=(0);
 if ($124) {
  $125 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $125;
  $287 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($287|0);
 }
 $126 = HEAP32[$pPage>>2]|0;
 $127 = HEAP32[$pCell>>2]|0;
 $128 = (_clearCell($126,$127,$szCell)|0);
 HEAP32[$rc>>2] = $128;
 $129 = HEAP32[$pPage>>2]|0;
 $130 = HEAP32[$iCellIdx>>2]|0;
 $131 = HEAP16[$szCell>>1]|0;
 $132 = $131&65535;
 _dropCell($129,$130,$132,$rc);
 $133 = HEAP32[$rc>>2]|0;
 $134 = ($133|0)!=(0);
 if ($134) {
  $135 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $135;
  $287 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($287|0);
 }
 $136 = HEAP32[$pPage>>2]|0;
 $137 = ((($136)) + 4|0);
 $138 = HEAP8[$137>>0]|0;
 $139 = ($138<<24>>24)!=(0);
 if (!($139)) {
  $140 = HEAP32[$1>>2]|0;
  $141 = ((($140)) + 68|0);
  $142 = HEAP8[$141>>0]|0;
  $143 = $142 << 24 >> 24;
  $144 = HEAP32[$1>>2]|0;
  $145 = ((($144)) + 120|0);
  $146 = (($145) + ($143<<2)|0);
  $147 = HEAP32[$146>>2]|0;
  HEAP32[$pLeaf>>2] = $147;
  $148 = HEAP32[$iCellDepth>>2]|0;
  $149 = (($148) + 1)|0;
  $150 = HEAP32[$1>>2]|0;
  $151 = ((($150)) + 120|0);
  $152 = (($151) + ($149<<2)|0);
  $153 = HEAP32[$152>>2]|0;
  $154 = ((($153)) + 84|0);
  $155 = HEAP32[$154>>2]|0;
  HEAP32[$n>>2] = $155;
  $156 = HEAP32[$pLeaf>>2]|0;
  $157 = ((($156)) + 56|0);
  $158 = HEAP32[$157>>2]|0;
  $159 = HEAP32[$pLeaf>>2]|0;
  $160 = ((($159)) + 20|0);
  $161 = HEAP16[$160>>1]|0;
  $162 = $161&65535;
  $163 = HEAP32[$pLeaf>>2]|0;
  $164 = ((($163)) + 18|0);
  $165 = HEAP16[$164>>1]|0;
  $166 = $165&65535;
  $167 = (($166) - 1)|0;
  $168 = $167<<1;
  $169 = HEAP32[$pLeaf>>2]|0;
  $170 = ((($169)) + 64|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = (($171) + ($168)|0);
  $173 = HEAP8[$172>>0]|0;
  $174 = $173&255;
  $175 = $174 << 8;
  $176 = HEAP32[$pLeaf>>2]|0;
  $177 = ((($176)) + 18|0);
  $178 = HEAP16[$177>>1]|0;
  $179 = $178&65535;
  $180 = (($179) - 1)|0;
  $181 = $180<<1;
  $182 = HEAP32[$pLeaf>>2]|0;
  $183 = ((($182)) + 64|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = (($184) + ($181)|0);
  $186 = ((($185)) + 1|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $187&255;
  $189 = $175 | $188;
  $190 = $162 & $189;
  $191 = (($158) + ($190)|0);
  HEAP32[$pCell>>2] = $191;
  $192 = HEAP32[$pCell>>2]|0;
  $193 = HEAP32[$pLeaf>>2]|0;
  $194 = ((($193)) + 56|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = ((($195)) + 4|0);
  $197 = ($192>>>0)<($196>>>0);
  if ($197) {
   $198 = (_sqlite3CorruptError(63773)|0);
   HEAP32[$0>>2] = $198;
   $287 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($287|0);
  }
  $199 = HEAP32[$pLeaf>>2]|0;
  $200 = ((($199)) + 76|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = HEAP32[$pLeaf>>2]|0;
  $203 = HEAP32[$pCell>>2]|0;
  $204 = (FUNCTION_TABLE_iii[$201 & 255]($202,$203)|0);
  $205 = $204&65535;
  HEAP32[$nCell>>2] = $205;
  $206 = HEAP32[$pBt>>2]|0;
  $207 = ((($206)) + 80|0);
  $208 = HEAP32[$207>>2]|0;
  HEAP32[$pTmp>>2] = $208;
  $209 = HEAP32[$pLeaf>>2]|0;
  $210 = ((($209)) + 72|0);
  $211 = HEAP32[$210>>2]|0;
  $212 = (_sqlite3PagerWrite($211)|0);
  HEAP32[$rc>>2] = $212;
  $213 = HEAP32[$pPage>>2]|0;
  $214 = HEAP32[$iCellIdx>>2]|0;
  $215 = HEAP32[$pCell>>2]|0;
  $216 = ((($215)) + -4|0);
  $217 = HEAP32[$nCell>>2]|0;
  $218 = (($217) + 4)|0;
  $219 = HEAP32[$pTmp>>2]|0;
  $220 = HEAP32[$n>>2]|0;
  _insertCell($213,$214,$216,$218,$219,$220,$rc);
  $221 = HEAP32[$pLeaf>>2]|0;
  $222 = HEAP32[$pLeaf>>2]|0;
  $223 = ((($222)) + 18|0);
  $224 = HEAP16[$223>>1]|0;
  $225 = $224&65535;
  $226 = (($225) - 1)|0;
  $227 = HEAP32[$nCell>>2]|0;
  _dropCell($221,$226,$227,$rc);
  $228 = HEAP32[$rc>>2]|0;
  $229 = ($228|0)!=(0);
  if ($229) {
   $230 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $230;
   $287 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($287|0);
  }
 }
 $231 = HEAP32[$1>>2]|0;
 $232 = (_balance($231)|0);
 HEAP32[$rc>>2] = $232;
 $233 = HEAP32[$rc>>2]|0;
 $234 = ($233|0)==(0);
 if ($234) {
  $235 = HEAP32[$1>>2]|0;
  $236 = ((($235)) + 68|0);
  $237 = HEAP8[$236>>0]|0;
  $238 = $237 << 24 >> 24;
  $239 = HEAP32[$iCellDepth>>2]|0;
  $240 = ($238|0)>($239|0);
  if ($240) {
   while(1) {
    $241 = HEAP32[$1>>2]|0;
    $242 = ((($241)) + 68|0);
    $243 = HEAP8[$242>>0]|0;
    $244 = $243 << 24 >> 24;
    $245 = HEAP32[$iCellDepth>>2]|0;
    $246 = ($244|0)>($245|0);
    $247 = HEAP32[$1>>2]|0;
    if (!($246)) {
     break;
    }
    $248 = ((($247)) + 68|0);
    $249 = HEAP8[$248>>0]|0;
    $250 = (($249) + -1)<<24>>24;
    HEAP8[$248>>0] = $250;
    $251 = $249 << 24 >> 24;
    $252 = HEAP32[$1>>2]|0;
    $253 = ((($252)) + 120|0);
    $254 = (($253) + ($251<<2)|0);
    $255 = HEAP32[$254>>2]|0;
    _releasePage($255);
   }
   $256 = (_balance($247)|0);
   HEAP32[$rc>>2] = $256;
  }
 }
 $257 = HEAP32[$rc>>2]|0;
 $258 = ($257|0)==(0);
 do {
  if ($258) {
   $259 = HEAP32[$bSkipnext>>2]|0;
   $260 = ($259|0)!=(0);
   $261 = HEAP32[$1>>2]|0;
   if (!($260)) {
    $281 = (_moveToRoot($261)|0);
    HEAP32[$rc>>2] = $281;
    $282 = HEAP8[$bPreserve>>0]|0;
    $283 = ($282<<24>>24)!=(0);
    if (!($283)) {
     break;
    }
    $284 = HEAP32[$1>>2]|0;
    $285 = ((($284)) + 66|0);
    HEAP8[$285>>0] = 3;
    break;
   }
   $262 = ((($261)) + 66|0);
   HEAP8[$262>>0] = 2;
   $263 = HEAP32[$iCellIdx>>2]|0;
   $264 = HEAP32[$pPage>>2]|0;
   $265 = ((($264)) + 18|0);
   $266 = HEAP16[$265>>1]|0;
   $267 = $266&65535;
   $268 = ($263|0)>=($267|0);
   $269 = HEAP32[$1>>2]|0;
   $270 = ((($269)) + 60|0);
   if ($268) {
    HEAP32[$270>>2] = -1;
    $271 = HEAP32[$pPage>>2]|0;
    $272 = ((($271)) + 18|0);
    $273 = HEAP16[$272>>1]|0;
    $274 = $273&65535;
    $275 = (($274) - 1)|0;
    $276 = $275&65535;
    $277 = HEAP32[$iCellDepth>>2]|0;
    $278 = HEAP32[$1>>2]|0;
    $279 = ((($278)) + 80|0);
    $280 = (($279) + ($277<<1)|0);
    HEAP16[$280>>1] = $276;
    break;
   } else {
    HEAP32[$270>>2] = 1;
    break;
   }
  }
 } while(0);
 $286 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $286;
 $287 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($287|0);
}
function _sqlite3VdbeSorterCompare($pCsr,$pVal,$nKeyCol,$pRes) {
 $pCsr = $pCsr|0;
 $pVal = $pVal|0;
 $nKeyCol = $nKeyCol|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nKey = 0;
 var $p = 0, $pKey = 0, $pKeyInfo = 0, $pSorter = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $pSorter = sp + 24|0;
 $r2 = sp + 20|0;
 $pKeyInfo = sp + 16|0;
 $i = sp + 12|0;
 $pKey = sp + 8|0;
 $nKey = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pVal;
 HEAP32[$3>>2] = $nKeyCol;
 HEAP32[$4>>2] = $pRes;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pSorter>>2] = $7;
 $8 = HEAP32[$pSorter>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$r2>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pKeyInfo>>2] = $13;
 $14 = HEAP32[$r2>>2]|0;
 $15 = ($14|0)==(0|0);
 do {
  if ($15) {
   $16 = HEAP32[$pKeyInfo>>2]|0;
   $17 = (_sqlite3VdbeAllocUnpackedRecord($16,0,0,$p)|0);
   $18 = HEAP32[$pSorter>>2]|0;
   $19 = ((($18)) + 32|0);
   HEAP32[$19>>2] = $17;
   HEAP32[$r2>>2] = $17;
   $20 = HEAP32[$r2>>2]|0;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $22 = HEAP32[$3>>2]|0;
    $23 = $22&65535;
    $24 = HEAP32[$r2>>2]|0;
    $25 = ((($24)) + 8|0);
    HEAP16[$25>>1] = $23;
    break;
   }
   HEAP32[$0>>2] = 7;
   $57 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($57|0);
  }
 } while(0);
 $26 = HEAP32[$pSorter>>2]|0;
 $27 = (_vdbeSorterRowkey($26,$nKey)|0);
 HEAP32[$pKey>>2] = $27;
 $28 = HEAP32[$pKeyInfo>>2]|0;
 $29 = HEAP32[$nKey>>2]|0;
 $30 = HEAP32[$pKey>>2]|0;
 $31 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($28,$29,$30,$31);
 HEAP32[$i>>2] = 0;
 while(1) {
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  $34 = ($32|0)<($33|0);
  if (!($34)) {
   label = 10;
   break;
  }
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$r2>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + (($35*40)|0)|0);
  $40 = ((($39)) + 8|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42 & 1;
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 8;
   break;
  }
  $46 = HEAP32[$i>>2]|0;
  $47 = (($46) + 1)|0;
  HEAP32[$i>>2] = $47;
 }
 if ((label|0) == 8) {
  $45 = HEAP32[$4>>2]|0;
  HEAP32[$45>>2] = -1;
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 10) {
  $48 = HEAP32[$2>>2]|0;
  $49 = ((($48)) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$2>>2]|0;
  $52 = ((($51)) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$r2>>2]|0;
  $55 = (_sqlite3VdbeRecordCompare($50,$53,$54)|0);
  $56 = HEAP32[$4>>2]|0;
  HEAP32[$56>>2] = $55;
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSorterRowkey($pCsr,$pOut) {
 $pCsr = $pCsr|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nKey = 0, $pKey = 0, $pSorter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSorter = sp + 8|0;
 $pKey = sp + 4|0;
 $nKey = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = (_vdbeSorterRowkey($6,$nKey)|0);
 HEAP32[$pKey>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$nKey>>2]|0;
 $10 = (_sqlite3VdbeMemClearAndResize($8,$9)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  HEAP32[$0>>2] = 7;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 } else {
  $12 = HEAP32[$nKey>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 12|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$2>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & -49664;
  $20 = $19 | 16;
  $21 = $20&65535;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 8|0);
  HEAP16[$23>>1] = $21;
  $24 = HEAP32[$2>>2]|0;
  $25 = ((($24)) + 16|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$pKey>>2]|0;
  $28 = HEAP32[$nKey>>2]|0;
  _memcpy(($26|0),($27|0),($28|0))|0;
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _sqlite3VdbeCursorRestore($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3BtreeCursorHasMoved($4)|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (_handleMovedCursor($7)|0);
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSorterRewind($pCsr,$pbEof) {
 $pCsr = $pCsr|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pSorter = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = ((($6)) + 56|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$pSorter>>2]|0;
 if (!($10)) {
  $22 = (_vdbeSorterFlushPMA($11)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(0);
  if ($25) {
   $26 = HEAP32[$pSorter>>2]|0;
   $27 = (_vdbeSorterSetupMerge($26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$2>>2]|0;
   HEAP32[$28>>2] = 0;
  }
  $29 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $29;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $12 = ((($11)) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 $15 = HEAP32[$2>>2]|0;
 if ($14) {
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$pSorter>>2]|0;
  $17 = ((($16)) + 64|0);
  $18 = HEAP32[$pSorter>>2]|0;
  $19 = ((($18)) + 36|0);
  $20 = (_vdbeSorterSort($17,$19)|0);
  HEAP32[$rc>>2] = $20;
 } else {
  HEAP32[$15>>2] = 1;
 }
 $21 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $21;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3BtreeFirst($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pRes;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_moveToRoot($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 66|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = ($9|0)==(0);
   $11 = HEAP32[$1>>2]|0;
   if ($10) {
    HEAP32[$11>>2] = 1;
    break;
   } else {
    HEAP32[$11>>2] = 0;
    $12 = HEAP32[$0>>2]|0;
    $13 = (_moveToLeftmost($12)|0);
    HEAP32[$rc>>2] = $13;
    break;
   }
  }
 } while(0);
 $14 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3VdbeSorterNext($db,$pCsr,$pbEof) {
 $db = $db|0;
 $pCsr = $pCsr|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFree = 0;
 var $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSorter = sp + 8|0;
 $rc = sp + 4|0;
 $pFree = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pbEof;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = ((($6)) + 56|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 $10 = HEAP32[$pSorter>>2]|0;
 if ($9) {
  $11 = ((($10)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (_vdbeMergeEngineStep($12,$13)|0);
  HEAP32[$rc>>2] = $14;
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $15 = ((($10)) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pFree>>2] = $16;
 $17 = HEAP32[$pFree>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$pSorter>>2]|0;
 $21 = ((($20)) + 36|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pFree>>2]|0;
 $23 = ((($22)) + 4|0);
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$pSorter>>2]|0;
 $25 = ((($24)) + 36|0);
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pFree>>2]|0;
  _vdbeSorterRecordFree($29,$30);
 }
 $31 = HEAP32[$pSorter>>2]|0;
 $32 = ((($31)) + 36|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 $35 = $34 ^ 1;
 $36 = $35&1;
 $37 = HEAP32[$2>>2]|0;
 HEAP32[$37>>2] = $36;
 HEAP32[$rc>>2] = 0;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3VdbeSorterWrite($pCsr,$pVal) {
 $pCsr = $pCsr|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $bFlush = 0, $iListOff = 0, $nMin = 0, $nNew = 0, $nPMA = 0, $nReq = 0, $or$cond = 0, $or$cond3 = 0, $pNew = 0, $pSorter = 0;
 var $rc = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $pSorter = sp + 40|0;
 $rc = sp + 36|0;
 $pNew = sp + 32|0;
 $bFlush = sp + 28|0;
 $nReq = sp + 24|0;
 $nPMA = sp + 20|0;
 $t = sp + 16|0;
 $nMin = sp + 12|0;
 $aNew = sp + 8|0;
 $iListOff = sp + 4|0;
 $nNew = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pVal;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)<(128);
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 1|0);
 if ($12) {
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  HEAP32[$t>>2] = $18;
 } else {
  (_sqlite3GetVarint32($16,$t)|0);
 }
 $19 = HEAP32[$t>>2]|0;
 $20 = ($19|0)>(0);
 $21 = HEAP32[$t>>2]|0;
 $22 = ($21|0)<(10);
 $or$cond = $20 & $22;
 $23 = HEAP32[$t>>2]|0;
 $24 = ($23|0)!=(7);
 $or$cond3 = $or$cond & $24;
 do {
  if ($or$cond3) {
   $25 = HEAP32[$pSorter>>2]|0;
   $26 = ((($25)) + 60|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27&255;
   $29 = $28 & 1;
   $30 = $29&255;
   HEAP8[$26>>0] = $30;
  } else {
   $31 = HEAP32[$t>>2]|0;
   $32 = ($31|0)>(10);
   if ($32) {
    $33 = HEAP32[$t>>2]|0;
    $34 = $33 & 1;
    $35 = ($34|0)!=(0);
    if ($35) {
     $36 = HEAP32[$pSorter>>2]|0;
     $37 = ((($36)) + 60|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38&255;
     $40 = $39 & 2;
     $41 = $40&255;
     HEAP8[$37>>0] = $41;
     break;
    }
   }
   $42 = HEAP32[$pSorter>>2]|0;
   $43 = ((($42)) + 60|0);
   HEAP8[$43>>0] = 0;
  }
 } while(0);
 $44 = HEAP32[$2>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 8)|0;
 HEAP32[$nReq>>2] = $47;
 $48 = HEAP32[$2>>2]|0;
 $49 = ((($48)) + 12|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$2>>2]|0;
 $52 = ((($51)) + 12|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = ($53|0)<(0);
 $55 = $54 << 31 >> 31;
 $56 = (_sqlite3VarintLen($53,$55)|0);
 $57 = (($50) + ($56))|0;
 HEAP32[$nPMA>>2] = $57;
 $58 = HEAP32[$pSorter>>2]|0;
 $59 = ((($58)) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)!=(0);
 if ($61) {
  $62 = HEAP32[$pSorter>>2]|0;
  $63 = ((($62)) + 36|0);
  $64 = ((($63)) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)!=(0|0);
  $67 = HEAP32[$pSorter>>2]|0;
  if ($66) {
   $68 = ((($67)) + 48|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)!=(0);
   if ($70) {
    $71 = HEAP32[$pSorter>>2]|0;
    $72 = ((($71)) + 48|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = HEAP32[$nReq>>2]|0;
    $75 = (($73) + ($74))|0;
    $76 = HEAP32[$pSorter>>2]|0;
    $77 = ((($76)) + 4|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = ($75|0)>($78|0);
    $81 = $79;
   } else {
    $81 = 0;
   }
   $80 = $81&1;
   HEAP32[$bFlush>>2] = $80;
  } else {
   $82 = ((($67)) + 36|0);
   $83 = ((($82)) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$pSorter>>2]|0;
   $86 = ((($85)) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = ($84|0)>($87|0);
   if ($88) {
    $99 = 1;
   } else {
    $89 = HEAP32[$pSorter>>2]|0;
    $90 = ((($89)) + 36|0);
    $91 = ((($90)) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = HEAP32[$pSorter>>2]|0;
    $94 = HEAP32[$93>>2]|0;
    $95 = ($92|0)>($94|0);
    if ($95) {
     $96 = (_sqlite3HeapNearlyFull()|0);
     $97 = ($96|0)!=(0);
     $99 = $97;
    } else {
     $99 = 0;
    }
   }
   $98 = $99&1;
   HEAP32[$bFlush>>2] = $98;
  }
  $100 = HEAP32[$bFlush>>2]|0;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$pSorter>>2]|0;
   $103 = (_vdbeSorterFlushPMA($102)|0);
   HEAP32[$rc>>2] = $103;
   $104 = HEAP32[$pSorter>>2]|0;
   $105 = ((($104)) + 36|0);
   $106 = ((($105)) + 8|0);
   HEAP32[$106>>2] = 0;
   $107 = HEAP32[$pSorter>>2]|0;
   $108 = ((($107)) + 48|0);
   HEAP32[$108>>2] = 0;
  }
 }
 $109 = HEAP32[$nPMA>>2]|0;
 $110 = HEAP32[$pSorter>>2]|0;
 $111 = ((($110)) + 36|0);
 $112 = ((($111)) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = (($113) + ($109))|0;
 HEAP32[$112>>2] = $114;
 $115 = HEAP32[$nPMA>>2]|0;
 $116 = HEAP32[$pSorter>>2]|0;
 $117 = ((($116)) + 8|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = ($115|0)>($118|0);
 if ($119) {
  $120 = HEAP32[$nPMA>>2]|0;
  $121 = HEAP32[$pSorter>>2]|0;
  $122 = ((($121)) + 8|0);
  HEAP32[$122>>2] = $120;
 }
 $123 = HEAP32[$pSorter>>2]|0;
 $124 = ((($123)) + 36|0);
 $125 = ((($124)) + 4|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = ($126|0)!=(0|0);
 do {
  if ($127) {
   $128 = HEAP32[$pSorter>>2]|0;
   $129 = ((($128)) + 48|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = HEAP32[$nReq>>2]|0;
   $132 = (($130) + ($131))|0;
   HEAP32[$nMin>>2] = $132;
   $133 = HEAP32[$nMin>>2]|0;
   $134 = HEAP32[$pSorter>>2]|0;
   $135 = ((($134)) + 52|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = ($133|0)>($136|0);
   do {
    if ($137) {
     $138 = HEAP32[$pSorter>>2]|0;
     $139 = ((($138)) + 36|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = HEAP32[$pSorter>>2]|0;
     $142 = ((($141)) + 36|0);
     $143 = ((($142)) + 4|0);
     $144 = HEAP32[$143>>2]|0;
     $145 = $140;
     $146 = $144;
     $147 = (($145) - ($146))|0;
     HEAP32[$iListOff>>2] = $147;
     $148 = HEAP32[$pSorter>>2]|0;
     $149 = ((($148)) + 52|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = $150<<1;
     HEAP32[$nNew>>2] = $151;
     while(1) {
      $152 = HEAP32[$nNew>>2]|0;
      $153 = HEAP32[$nMin>>2]|0;
      $154 = ($152|0)<($153|0);
      $155 = HEAP32[$nNew>>2]|0;
      if (!($154)) {
       break;
      }
      $156 = $155<<1;
      HEAP32[$nNew>>2] = $156;
     }
     $157 = HEAP32[$pSorter>>2]|0;
     $158 = ((($157)) + 4|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ($155|0)>($159|0);
     if ($160) {
      $161 = HEAP32[$pSorter>>2]|0;
      $162 = ((($161)) + 4|0);
      $163 = HEAP32[$162>>2]|0;
      HEAP32[$nNew>>2] = $163;
     }
     $164 = HEAP32[$nNew>>2]|0;
     $165 = HEAP32[$nMin>>2]|0;
     $166 = ($164|0)<($165|0);
     if ($166) {
      $167 = HEAP32[$nMin>>2]|0;
      HEAP32[$nNew>>2] = $167;
     }
     $168 = HEAP32[$pSorter>>2]|0;
     $169 = ((($168)) + 36|0);
     $170 = ((($169)) + 4|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = HEAP32[$nNew>>2]|0;
     $173 = ($172|0)<(0);
     $174 = $173 << 31 >> 31;
     $175 = (_sqlite3Realloc($171,$172,$174)|0);
     HEAP32[$aNew>>2] = $175;
     $176 = HEAP32[$aNew>>2]|0;
     $177 = ($176|0)!=(0|0);
     if ($177) {
      $178 = HEAP32[$iListOff>>2]|0;
      $179 = HEAP32[$aNew>>2]|0;
      $180 = (($179) + ($178)|0);
      $181 = HEAP32[$pSorter>>2]|0;
      $182 = ((($181)) + 36|0);
      HEAP32[$182>>2] = $180;
      $183 = HEAP32[$aNew>>2]|0;
      $184 = HEAP32[$pSorter>>2]|0;
      $185 = ((($184)) + 36|0);
      $186 = ((($185)) + 4|0);
      HEAP32[$186>>2] = $183;
      $187 = HEAP32[$nNew>>2]|0;
      $188 = HEAP32[$pSorter>>2]|0;
      $189 = ((($188)) + 52|0);
      HEAP32[$189>>2] = $187;
      break;
     }
     HEAP32[$0>>2] = 7;
     $248 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($248|0);
    }
   } while(0);
   $190 = HEAP32[$pSorter>>2]|0;
   $191 = ((($190)) + 48|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = HEAP32[$pSorter>>2]|0;
   $194 = ((($193)) + 36|0);
   $195 = ((($194)) + 4|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = (($196) + ($192)|0);
   HEAP32[$pNew>>2] = $197;
   $198 = HEAP32[$nReq>>2]|0;
   $199 = (($198) + 7)|0;
   $200 = $199 & -8;
   $201 = HEAP32[$pSorter>>2]|0;
   $202 = ((($201)) + 48|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = (($203) + ($200))|0;
   HEAP32[$202>>2] = $204;
   $205 = HEAP32[$pSorter>>2]|0;
   $206 = ((($205)) + 36|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = ($207|0)!=(0|0);
   if ($208) {
    $209 = HEAP32[$pSorter>>2]|0;
    $210 = ((($209)) + 36|0);
    $211 = HEAP32[$210>>2]|0;
    $212 = HEAP32[$pSorter>>2]|0;
    $213 = ((($212)) + 36|0);
    $214 = ((($213)) + 4|0);
    $215 = HEAP32[$214>>2]|0;
    $216 = $211;
    $217 = $215;
    $218 = (($216) - ($217))|0;
    $219 = HEAP32[$pNew>>2]|0;
    $220 = ((($219)) + 4|0);
    HEAP32[$220>>2] = $218;
   }
  } else {
   $221 = HEAP32[$nReq>>2]|0;
   $222 = ($221|0)<(0);
   $223 = $222 << 31 >> 31;
   $224 = (_sqlite3Malloc($221,$223)|0);
   HEAP32[$pNew>>2] = $224;
   $225 = HEAP32[$pNew>>2]|0;
   $226 = ($225|0)==(0|0);
   if (!($226)) {
    $227 = HEAP32[$pSorter>>2]|0;
    $228 = ((($227)) + 36|0);
    $229 = HEAP32[$228>>2]|0;
    $230 = HEAP32[$pNew>>2]|0;
    $231 = ((($230)) + 4|0);
    HEAP32[$231>>2] = $229;
    break;
   }
   HEAP32[$0>>2] = 7;
   $248 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($248|0);
  }
 } while(0);
 $232 = HEAP32[$pNew>>2]|0;
 $233 = ((($232)) + 8|0);
 $234 = HEAP32[$2>>2]|0;
 $235 = ((($234)) + 16|0);
 $236 = HEAP32[$235>>2]|0;
 $237 = HEAP32[$2>>2]|0;
 $238 = ((($237)) + 12|0);
 $239 = HEAP32[$238>>2]|0;
 _memcpy(($233|0),($236|0),($239|0))|0;
 $240 = HEAP32[$2>>2]|0;
 $241 = ((($240)) + 12|0);
 $242 = HEAP32[$241>>2]|0;
 $243 = HEAP32[$pNew>>2]|0;
 HEAP32[$243>>2] = $242;
 $244 = HEAP32[$pNew>>2]|0;
 $245 = HEAP32[$pSorter>>2]|0;
 $246 = ((($245)) + 36|0);
 HEAP32[$246>>2] = $244;
 $247 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $247;
 $248 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($248|0);
}
function _sqlite3VdbeIdxRowid($db,$pCur,$rowid) {
 $db = $db|0;
 $pCur = $pCur|0;
 $rowid = $rowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $lenRowid = 0, $m = 0, $nCellKey = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $szHdr = 0, $typeRowid = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $nCellKey = sp + 80|0;
 $rc = sp + 100|0;
 $szHdr = sp + 96|0;
 $typeRowid = sp + 92|0;
 $lenRowid = sp + 88|0;
 $m = sp + 40|0;
 $v = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pCur;
 HEAP32[$3>>2] = $rowid;
 $4 = $nCellKey;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 (_sqlite3BtreeKeySize($8,$nCellKey)|0);
 $9 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemInit($m,$9,0);
 $10 = HEAP32[$2>>2]|0;
 $11 = $nCellKey;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3VdbeMemFromBtree($10,0,$13,1,$m)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 $21 = ((($m)) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)<(128);
 $26 = ((($m)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 if ($25) {
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  HEAP32[$szHdr>>2] = $29;
 } else {
  (_sqlite3GetVarint32($27,$szHdr)|0);
 }
 $30 = HEAP32[$szHdr>>2]|0;
 $31 = ($30>>>0)<(3);
 if (!($31)) {
  $32 = HEAP32[$szHdr>>2]|0;
  $33 = ((($m)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($32|0)>($34|0);
  if (!($35)) {
   $36 = HEAP32[$szHdr>>2]|0;
   $37 = (($36) - 1)|0;
   $38 = ((($m)) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($37)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = ($42|0)<(128);
   $44 = HEAP32[$szHdr>>2]|0;
   $45 = (($44) - 1)|0;
   $46 = ((($m)) + 16|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($45)|0);
   if ($43) {
    $49 = HEAP8[$48>>0]|0;
    $50 = $49&255;
    HEAP32[$typeRowid>>2] = $50;
   } else {
    (_sqlite3GetVarint32($48,$typeRowid)|0);
   }
   $51 = HEAP32[$typeRowid>>2]|0;
   $52 = ($51>>>0)<(1);
   $53 = HEAP32[$typeRowid>>2]|0;
   $54 = ($53>>>0)>(9);
   $or$cond = $52 | $54;
   $55 = HEAP32[$typeRowid>>2]|0;
   $56 = ($55|0)==(7);
   $or$cond3 = $or$cond | $56;
   if (!($or$cond3)) {
    $57 = HEAP32[$typeRowid>>2]|0;
    $58 = (21519 + ($57)|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    HEAP32[$lenRowid>>2] = $60;
    $61 = ((($m)) + 12|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = HEAP32[$szHdr>>2]|0;
    $64 = HEAP32[$lenRowid>>2]|0;
    $65 = (($63) + ($64))|0;
    $66 = ($62>>>0)<($65>>>0);
    if (!($66)) {
     $67 = ((($m)) + 12|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = HEAP32[$lenRowid>>2]|0;
     $70 = (($68) - ($69))|0;
     $71 = ((($m)) + 16|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = (($72) + ($70)|0);
     $74 = HEAP32[$typeRowid>>2]|0;
     (_sqlite3VdbeSerialGet($73,$74,$v)|0);
     $75 = $v;
     $76 = $75;
     $77 = HEAP32[$76>>2]|0;
     $78 = (($75) + 4)|0;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = HEAP32[$3>>2]|0;
     $82 = $81;
     $83 = $82;
     HEAP32[$83>>2] = $77;
     $84 = (($82) + 4)|0;
     $85 = $84;
     HEAP32[$85>>2] = $80;
     _sqlite3VdbeMemRelease($m);
     HEAP32[$0>>2] = 0;
     $87 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($87|0);
    }
   }
  }
 }
 _sqlite3VdbeMemRelease($m);
 $86 = (_sqlite3CorruptError(72119)|0);
 HEAP32[$0>>2] = $86;
 $87 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($87|0);
}
function _sqlite3VdbeIdxKeyCompare($db,$pC,$pUnpacked,$res) {
 $db = $db|0;
 $pC = $pC|0;
 $pUnpacked = $pUnpacked|0;
 $res = $res|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $m = 0, $nCellKey = 0, $or$cond = 0;
 var $pCur = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $nCellKey = sp + 40|0;
 $rc = sp + 52|0;
 $pCur = sp + 48|0;
 $m = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pC;
 HEAP32[$3>>2] = $pUnpacked;
 HEAP32[$4>>2] = $res;
 $5 = $nCellKey;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pCur>>2] = $11;
 $12 = HEAP32[$pCur>>2]|0;
 (_sqlite3BtreeKeySize($12,$nCellKey)|0);
 $13 = $nCellKey;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)<(0);
 $20 = ($15>>>0)<=(0);
 $21 = ($18|0)==(0);
 $22 = $21 & $20;
 $23 = $19 | $22;
 $24 = $nCellKey;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>(0);
 $31 = ($26>>>0)>(2147483647);
 $32 = ($29|0)==(0);
 $33 = $32 & $31;
 $34 = $30 | $33;
 $or$cond = $23 | $34;
 if ($or$cond) {
  $35 = HEAP32[$4>>2]|0;
  HEAP32[$35>>2] = 0;
  $36 = (_sqlite3CorruptError(72153)|0);
  HEAP32[$0>>2] = $36;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $37 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemInit($m,$37,0);
 $38 = HEAP32[$pCur>>2]|0;
 $39 = $nCellKey;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($39) + 4)|0;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (_sqlite3VdbeMemFromBtree($38,0,$41,1,$m)|0);
 HEAP32[$rc>>2] = $45;
 $46 = HEAP32[$rc>>2]|0;
 $47 = ($46|0)!=(0);
 if ($47) {
  $48 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $48;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 } else {
  $49 = ((($m)) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = ((($m)) + 16|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$3>>2]|0;
  $54 = (_sqlite3VdbeRecordCompare($50,$52,$53)|0);
  $55 = HEAP32[$4>>2]|0;
  HEAP32[$55>>2] = $54;
  _sqlite3VdbeMemRelease($m);
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return (0)|0;
}
function _sqlite3BtreeDropTable($p,$iTable,$piMoved) {
 $p = $p|0;
 $iTable = $iTable|0;
 $piMoved = $piMoved|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $piMoved;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_btreeDropTable($4,$5,$6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3RootPageMoved($db,$iDb,$iFrom,$iTo) {
 $db = $db|0;
 $iDb = $iDb|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDb = 0, $pElem = 0, $pHash = 0, $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pElem = sp + 16|0;
 $pHash = sp + 12|0;
 $pDb = sp + 8|0;
 $pTab = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iFrom;
 HEAP32[$3>>2] = $iTo;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 HEAP32[$pDb>>2] = $8;
 $9 = HEAP32[$pDb>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 8|0);
 HEAP32[$pHash>>2] = $12;
 $13 = HEAP32[$pHash>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pElem>>2] = $15;
 while(1) {
  $16 = HEAP32[$pElem>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$pElem>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pTab>>2] = $20;
  $21 = HEAP32[$pTab>>2]|0;
  $22 = ((($21)) + 28|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ($23|0)==($24|0);
  if ($25) {
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$pTab>>2]|0;
   $28 = ((($27)) + 28|0);
   HEAP32[$28>>2] = $26;
  }
  $29 = HEAP32[$pElem>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pElem>>2] = $30;
 }
 $31 = HEAP32[$pDb>>2]|0;
 $32 = ((($31)) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ((($33)) + 24|0);
 HEAP32[$pHash>>2] = $34;
 $35 = HEAP32[$pHash>>2]|0;
 $36 = ((($35)) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$pElem>>2] = $37;
 while(1) {
  $38 = HEAP32[$pElem>>2]|0;
  $39 = ($38|0)!=(0|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$pElem>>2]|0;
  $41 = ((($40)) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$pIdx>>2] = $42;
  $43 = HEAP32[$pIdx>>2]|0;
  $44 = ((($43)) + 44|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ($45|0)==($46|0);
  if ($47) {
   $48 = HEAP32[$3>>2]|0;
   $49 = HEAP32[$pIdx>>2]|0;
   $50 = ((($49)) + 44|0);
   HEAP32[$50>>2] = $48;
  }
  $51 = HEAP32[$pElem>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  HEAP32[$pElem>>2] = $52;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreeClearTable($p,$iTable,$pnChange) {
 $p = $p|0;
 $iTable = $iTable|0;
 $pnChange = $pnChange|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $pnChange;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_saveAllCursors($7,$8,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = (0)==($10|0);
 if (!($11)) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $12 = HEAP32[$0>>2]|0;
 _invalidateIncrblobCursors($12,0,0,1);
 $13 = HEAP32[$pBt>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (_clearDatabasePage($13,$14,0,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3BtreeClearTableOfCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3BtreeClearTable($2,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3InitCallback($pInit,$argc,$argv,$NotUsed) {
 $pInit = $pInit|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $pData = 0, $pIndex = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pData = sp + 20|0;
 $db = sp + 16|0;
 $iDb = sp + 12|0;
 $rc = sp + 8|0;
 $pStmt = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $pInit;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $NotUsed;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pData>>2] = $5;
 $6 = HEAP32[$pData>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$pData>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$iDb>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<4)|0);
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 78|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & -5;
 $22 = $21&65535;
 HEAP16[$18>>1] = $22;
 $23 = HEAP32[$db>>2]|0;
 $24 = ((($23)) + 65|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25<<24>>24)!=(0);
 if ($26) {
  $27 = HEAP32[$pData>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  _corruptSchema($27,$29,0);
  HEAP32[$0>>2] = 1;
  $130 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($130|0);
 }
 $30 = HEAP32[$3>>2]|0;
 $31 = ($30|0)==(0|0);
 if ($31) {
  HEAP32[$0>>2] = 0;
  $130 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($130|0);
 }
 $32 = HEAP32[$3>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 L9: do {
  if ($35) {
   $36 = HEAP32[$pData>>2]|0;
   $37 = HEAP32[$3>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   _corruptSchema($36,$38,0);
  } else {
   $39 = HEAP32[$3>>2]|0;
   $40 = ((($39)) + 8|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (_sqlite3_strnicmp($41,21654,7)|0);
   $43 = ($42|0)==(0);
   if (!($43)) {
    $92 = HEAP32[$3>>2]|0;
    $93 = HEAP32[$92>>2]|0;
    $94 = ($93|0)==(0|0);
    do {
     if (!($94)) {
      $95 = HEAP32[$3>>2]|0;
      $96 = ((($95)) + 8|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ($97|0)!=(0|0);
      if ($98) {
       $99 = HEAP32[$3>>2]|0;
       $100 = ((($99)) + 8|0);
       $101 = HEAP32[$100>>2]|0;
       $102 = HEAP8[$101>>0]|0;
       $103 = $102 << 24 >> 24;
       $104 = ($103|0)!=(0);
       if ($104) {
        break;
       }
      }
      $108 = HEAP32[$db>>2]|0;
      $109 = HEAP32[$3>>2]|0;
      $110 = HEAP32[$109>>2]|0;
      $111 = HEAP32[$iDb>>2]|0;
      $112 = HEAP32[$db>>2]|0;
      $113 = ((($112)) + 16|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = (($114) + ($111<<4)|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = (_sqlite3FindIndex($108,$110,$116)|0);
      HEAP32[$pIndex>>2] = $117;
      $118 = HEAP32[$pIndex>>2]|0;
      $119 = ($118|0)==(0|0);
      if ($119) {
       break L9;
      }
      $120 = HEAP32[$3>>2]|0;
      $121 = ((($120)) + 4|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = HEAP32[$pIndex>>2]|0;
      $124 = ((($123)) + 44|0);
      $125 = (_sqlite3GetInt32($122,$124)|0);
      $126 = ($125|0)==(0);
      if (!($126)) {
       break L9;
      }
      $127 = HEAP32[$pData>>2]|0;
      $128 = HEAP32[$3>>2]|0;
      $129 = HEAP32[$128>>2]|0;
      _corruptSchema($127,$129,21662);
      break L9;
     }
    } while(0);
    $105 = HEAP32[$pData>>2]|0;
    $106 = HEAP32[$3>>2]|0;
    $107 = HEAP32[$106>>2]|0;
    _corruptSchema($105,$107,0);
    break;
   }
   $44 = HEAP32[$iDb>>2]|0;
   $45 = $44&255;
   $46 = HEAP32[$db>>2]|0;
   $47 = ((($46)) + 140|0);
   $48 = ((($47)) + 4|0);
   HEAP8[$48>>0] = $45;
   $49 = HEAP32[$3>>2]|0;
   $50 = ((($49)) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (_sqlite3Atoi($51)|0);
   $53 = HEAP32[$db>>2]|0;
   $54 = ((($53)) + 140|0);
   HEAP32[$54>>2] = $52;
   $55 = HEAP32[$db>>2]|0;
   $56 = ((($55)) + 140|0);
   $57 = ((($56)) + 6|0);
   HEAP8[$57>>0] = 0;
   $58 = HEAP32[$db>>2]|0;
   $59 = HEAP32[$3>>2]|0;
   $60 = ((($59)) + 8|0);
   $61 = HEAP32[$60>>2]|0;
   (_sqlite3_prepare($58,$61,-1,$pStmt,0)|0);
   $62 = HEAP32[$db>>2]|0;
   $63 = ((($62)) + 52|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$rc>>2] = $64;
   $65 = HEAP32[$db>>2]|0;
   $66 = ((($65)) + 140|0);
   $67 = ((($66)) + 4|0);
   HEAP8[$67>>0] = 0;
   $68 = HEAP32[$rc>>2]|0;
   $69 = (0)!=($68|0);
   do {
    if ($69) {
     $70 = HEAP32[$db>>2]|0;
     $71 = ((($70)) + 140|0);
     $72 = ((($71)) + 6|0);
     $73 = HEAP8[$72>>0]|0;
     $74 = ($73<<24>>24)!=(0);
     if (!($74)) {
      $75 = HEAP32[$rc>>2]|0;
      $76 = HEAP32[$pData>>2]|0;
      $77 = ((($76)) + 12|0);
      HEAP32[$77>>2] = $75;
      $78 = HEAP32[$rc>>2]|0;
      $79 = ($78|0)==(7);
      if ($79) {
       $80 = HEAP32[$db>>2]|0;
       _sqlite3OomFault($80);
       break;
      }
      $81 = HEAP32[$rc>>2]|0;
      $82 = ($81|0)!=(9);
      if ($82) {
       $83 = HEAP32[$rc>>2]|0;
       $84 = $83 & 255;
       $85 = ($84|0)!=(6);
       if ($85) {
        $86 = HEAP32[$pData>>2]|0;
        $87 = HEAP32[$3>>2]|0;
        $88 = HEAP32[$87>>2]|0;
        $89 = HEAP32[$db>>2]|0;
        $90 = (_sqlite3_errmsg($89)|0);
        _corruptSchema($86,$88,$90);
       }
      }
     }
    }
   } while(0);
   $91 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_finalize($91)|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $130 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($130|0);
}
function _sqlite3AnalysisLoad($db,$iDb) {
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pIdx = 0, $rc = 0, $sInfo = 0, $vararg_buffer = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $sInfo = sp + 24|0;
 $i = sp + 16|0;
 $zSql = sp + 12|0;
 $rc = sp + 8|0;
 $pIdx = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$i>>2] = $12;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pIdx>>2] = $17;
  $18 = HEAP32[$pIdx>>2]|0;
  _sqlite3DefaultRowEst($18);
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$1>>2]|0;
 HEAP32[$sInfo>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<4)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ((($sInfo)) + 4|0);
 HEAP32[$28>>2] = $27;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($sInfo)) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3FindTable($29,21720,$31)|0);
 $33 = ($32|0)==(0|0);
 if ($33) {
  HEAP32[$0>>2] = 1;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($sInfo)) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 HEAP32[$vararg_buffer>>2] = $36;
 $37 = (_sqlite3MPrintf($34,21733,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $37;
 $38 = HEAP32[$zSql>>2]|0;
 $39 = ($38|0)==(0|0);
 if ($39) {
  HEAP32[$rc>>2] = 7;
 } else {
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$zSql>>2]|0;
  $42 = (_sqlite3_exec($40,$41,141,$sInfo,0)|0);
  HEAP32[$rc>>2] = $42;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$zSql>>2]|0;
  _sqlite3DbFree($43,$44);
 }
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(7);
 if ($46) {
  $47 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($47);
 }
 $48 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3UnlinkAndDeleteTable($db,$iDb,$zTabName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTabName = $zTabName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTabName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 HEAP32[$pDb>>2] = $7;
 $8 = HEAP32[$pDb>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 _sqlite3DeleteTable($14,$15);
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $18 | 2;
 HEAP32[$17>>2] = $19;
 STACKTOP = sp;return;
}
function _sqlite3UnlinkAndDeleteIndex($db,$iDb,$zIdxName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zIdxName = $zIdxName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pHash = 0, $pIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pIndex = sp + 8|0;
 $pHash = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zIdxName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 HEAP32[$pHash>>2] = $10;
 $11 = HEAP32[$pHash>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$pIndex>>2] = $13;
 $14 = HEAP32[$pIndex>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $57 = HEAP32[$0>>2]|0;
  $58 = ((($57)) + 24|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59 | 2;
  HEAP32[$58>>2] = $60;
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$pIndex>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pIndex>>2]|0;
 $22 = ($20|0)==($21|0);
 $23 = HEAP32[$pIndex>>2]|0;
 if ($22) {
  $24 = ((($23)) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$pIndex>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 8|0);
  HEAP32[$29>>2] = $25;
 } else {
  $30 = ((($23)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$p>>2] = $33;
  while(1) {
   $34 = HEAP32[$p>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ((($36)) + 20|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$pIndex>>2]|0;
    $40 = ($38|0)!=($39|0);
    $61 = $40;
   } else {
    $61 = 0;
   }
   $41 = HEAP32[$p>>2]|0;
   if (!($61)) {
    break;
   }
   $42 = ((($41)) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   HEAP32[$p>>2] = $43;
  }
  $44 = ($41|0)!=(0|0);
  if ($44) {
   $45 = HEAP32[$p>>2]|0;
   $46 = ((($45)) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$pIndex>>2]|0;
   $49 = ($47|0)==($48|0);
   if ($49) {
    $50 = HEAP32[$pIndex>>2]|0;
    $51 = ((($50)) + 20|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$p>>2]|0;
    $54 = ((($53)) + 20|0);
    HEAP32[$54>>2] = $52;
   }
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$pIndex>>2]|0;
 _freeIndex($55,$56);
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 | 2;
 HEAP32[$58>>2] = $60;
 STACKTOP = sp;return;
}
function _sqlite3UnlinkAndDeleteTrigger($db,$iDb,$zName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pHash = 0, $pTab = 0, $pTrigger = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pTrigger = sp + 12|0;
 $pHash = sp + 8|0;
 $pTab = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 40|0);
 HEAP32[$pHash>>2] = $10;
 $11 = HEAP32[$pHash>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$pTrigger>>2] = $13;
 $14 = HEAP32[$pTrigger>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$pTrigger>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$pTrigger>>2]|0;
 $20 = ((($19)) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($18|0)==($21|0);
 if ($22) {
  $23 = HEAP32[$pTrigger>>2]|0;
  $24 = (_tableOfTrigger($23)|0);
  HEAP32[$pTab>>2] = $24;
  $25 = HEAP32[$pTab>>2]|0;
  $26 = ((($25)) + 60|0);
  HEAP32[$pp>>2] = $26;
  while(1) {
   $27 = HEAP32[$pp>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$pTrigger>>2]|0;
   $30 = ($28|0)!=($29|0);
   $31 = HEAP32[$pp>>2]|0;
   $32 = HEAP32[$31>>2]|0;
   $33 = ((($32)) + 32|0);
   if (!($30)) {
    break;
   }
   HEAP32[$pp>>2] = $33;
  }
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pp>>2]|0;
  HEAP32[$35>>2] = $34;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pTrigger>>2]|0;
 _sqlite3DeleteTrigger($36,$37);
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 24|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $40 | 2;
 HEAP32[$39>>2] = $41;
 STACKTOP = sp;return;
}
function _sqlite3BtreeIntegrityCheck($p,$aRoot,$nRoot,$mxErr,$pnErr) {
 $p = $p|0;
 $aRoot = $aRoot|0;
 $nRoot = $nRoot|0;
 $mxErr = $mxErr|0;
 $pnErr = $pnErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $notUsed = 0, $pBt = 0, $sCheck = 0, $savedDbFlags = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $i = sp + 104|0;
 $sCheck = sp + 32|0;
 $pBt = sp + 24|0;
 $savedDbFlags = sp + 20|0;
 $zErr = sp + 128|0;
 $notUsed = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $aRoot;
 HEAP32[$2>>2] = $nRoot;
 HEAP32[$3>>2] = $mxErr;
 HEAP32[$4>>2] = $pnErr;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$pBt>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$savedDbFlags>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($13);
 $14 = HEAP32[$pBt>>2]|0;
 HEAP32[$sCheck>>2] = $14;
 $15 = HEAP32[$pBt>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($sCheck)) + 4|0);
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$sCheck>>2]|0;
 $19 = (_btreePagecount($18)|0);
 $20 = ((($sCheck)) + 12|0);
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$3>>2]|0;
 $22 = ((($sCheck)) + 16|0);
 HEAP32[$22>>2] = $21;
 $23 = ((($sCheck)) + 20|0);
 HEAP32[$23>>2] = 0;
 $24 = ((($sCheck)) + 24|0);
 HEAP32[$24>>2] = 0;
 $25 = ((($sCheck)) + 28|0);
 HEAP32[$25>>2] = 0;
 $26 = ((($sCheck)) + 32|0);
 HEAP32[$26>>2] = 0;
 $27 = ((($sCheck)) + 36|0);
 HEAP32[$27>>2] = 0;
 $28 = ((($sCheck)) + 8|0);
 HEAP32[$28>>2] = 0;
 $29 = ((($sCheck)) + 68|0);
 HEAP32[$29>>2] = 0;
 $30 = ((($sCheck)) + 40|0);
 _sqlite3StrAccumInit($30,0,$zErr,100,1000000000);
 $31 = ((($sCheck)) + 40|0);
 $32 = ((($31)) + 25|0);
 HEAP8[$32>>0] = 1;
 $33 = ((($sCheck)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0);
 L1: do {
  if (!($35)) {
   $36 = ((($sCheck)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37>>>0) / 8)&-1;
   $39 = (($38) + 1)|0;
   $40 = (_sqlite3MallocZero($39,0)|0);
   $41 = ((($sCheck)) + 8|0);
   HEAP32[$41>>2] = $40;
   $42 = ((($sCheck)) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)!=(0|0);
   if (!($44)) {
    $45 = ((($sCheck)) + 24|0);
    HEAP32[$45>>2] = 1;
    break;
   }
   $46 = HEAP32[$pBt>>2]|0;
   $47 = ((($46)) + 32|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (_sqlite3PageMalloc($48)|0);
   $50 = ((($sCheck)) + 68|0);
   HEAP32[$50>>2] = $49;
   $51 = ((($sCheck)) + 68|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)==(0|0);
   if ($53) {
    $54 = ((($sCheck)) + 24|0);
    HEAP32[$54>>2] = 1;
    break;
   }
   $55 = HEAP32[832>>2]|0;
   $56 = HEAP32[$pBt>>2]|0;
   $57 = ((($56)) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = (($55>>>0) / ($58>>>0))&-1;
   $60 = (($59) + 1)|0;
   HEAP32[$i>>2] = $60;
   $61 = HEAP32[$i>>2]|0;
   $62 = ((($sCheck)) + 12|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($61>>>0)<=($63>>>0);
   if ($64) {
    $65 = HEAP32[$i>>2]|0;
    _setPageReferenced($sCheck,$65);
   }
   $66 = ((($sCheck)) + 28|0);
   HEAP32[$66>>2] = 21807;
   $67 = HEAP32[$pBt>>2]|0;
   $68 = ((($67)) + 12|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ((($69)) + 56|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ((($71)) + 32|0);
   $73 = (_sqlite3Get4byte($72)|0);
   $74 = HEAP32[$pBt>>2]|0;
   $75 = ((($74)) + 12|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = ((($76)) + 56|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ((($78)) + 36|0);
   $80 = (_sqlite3Get4byte($79)|0);
   _checkList($sCheck,1,$73,$80);
   $81 = ((($sCheck)) + 28|0);
   HEAP32[$81>>2] = 0;
   $82 = HEAP32[$pBt>>2]|0;
   $83 = ((($82)) + 4|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ((($84)) + 24|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $86 & -268435457;
   HEAP32[$85>>2] = $87;
   HEAP32[$i>>2] = 0;
   while(1) {
    $88 = HEAP32[$i>>2]|0;
    $89 = HEAP32[$2>>2]|0;
    $90 = ($88|0)<($89|0);
    $91 = ((($sCheck)) + 16|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)!=(0);
    $94 = $90 ? $93 : 0;
    if (!($94)) {
     break;
    }
    $95 = HEAP32[$i>>2]|0;
    $96 = HEAP32[$1>>2]|0;
    $97 = (($96) + ($95<<2)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0);
    if (!($99)) {
     $100 = HEAP32[$pBt>>2]|0;
     $101 = ((($100)) + 17|0);
     $102 = HEAP8[$101>>0]|0;
     $103 = $102&255;
     $104 = ($103|0)!=(0);
     if ($104) {
      $105 = HEAP32[$i>>2]|0;
      $106 = HEAP32[$1>>2]|0;
      $107 = (($106) + ($105<<2)|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = ($108|0)>(1);
      if ($109) {
       $110 = HEAP32[$i>>2]|0;
       $111 = HEAP32[$1>>2]|0;
       $112 = (($111) + ($110<<2)|0);
       $113 = HEAP32[$112>>2]|0;
       _checkPtrmap($sCheck,$113,1,0);
      }
     }
     $114 = HEAP32[$i>>2]|0;
     $115 = HEAP32[$1>>2]|0;
     $116 = (($115) + ($114<<2)|0);
     $117 = HEAP32[$116>>2]|0;
     (_checkTreePage($sCheck,$117,$notUsed,-1,2147483647)|0);
    }
    $118 = HEAP32[$i>>2]|0;
    $119 = (($118) + 1)|0;
    HEAP32[$i>>2] = $119;
   }
   $120 = HEAP32[$savedDbFlags>>2]|0;
   $121 = HEAP32[$pBt>>2]|0;
   $122 = ((($121)) + 4|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = ((($123)) + 24|0);
   HEAP32[$124>>2] = $120;
   HEAP32[$i>>2] = 1;
   while(1) {
    $125 = HEAP32[$i>>2]|0;
    $126 = ((($sCheck)) + 12|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = ($125>>>0)<=($127>>>0);
    $129 = ((($sCheck)) + 16|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = ($130|0)!=(0);
    $132 = $128 ? $131 : 0;
    if (!($132)) {
     break L1;
    }
    $133 = HEAP32[$i>>2]|0;
    $134 = (_getPageReferenced($sCheck,$133)|0);
    $135 = ($134|0)==(0);
    do {
     if ($135) {
      $136 = HEAP32[$pBt>>2]|0;
      $137 = HEAP32[$i>>2]|0;
      $138 = (_ptrmapPageno($136,$137)|0);
      $139 = HEAP32[$i>>2]|0;
      $140 = ($138|0)!=($139|0);
      if (!($140)) {
       $141 = HEAP32[$pBt>>2]|0;
       $142 = ((($141)) + 17|0);
       $143 = HEAP8[$142>>0]|0;
       $144 = ($143<<24>>24)!=(0);
       if ($144) {
        break;
       }
      }
      $145 = HEAP32[$i>>2]|0;
      HEAP32[$vararg_buffer>>2] = $145;
      _checkAppendMsg($sCheck,21823,$vararg_buffer);
     }
    } while(0);
    $146 = HEAP32[$i>>2]|0;
    $147 = (_getPageReferenced($sCheck,$146)|0);
    $148 = ($147|0)!=(0);
    if ($148) {
     $149 = HEAP32[$pBt>>2]|0;
     $150 = HEAP32[$i>>2]|0;
     $151 = (_ptrmapPageno($149,$150)|0);
     $152 = HEAP32[$i>>2]|0;
     $153 = ($151|0)==($152|0);
     if ($153) {
      $154 = HEAP32[$pBt>>2]|0;
      $155 = ((($154)) + 17|0);
      $156 = HEAP8[$155>>0]|0;
      $157 = $156&255;
      $158 = ($157|0)!=(0);
      if ($158) {
       $159 = HEAP32[$i>>2]|0;
       HEAP32[$vararg_buffer1>>2] = $159;
       _checkAppendMsg($sCheck,21845,$vararg_buffer1);
      }
     }
    }
    $160 = HEAP32[$i>>2]|0;
    $161 = (($160) + 1)|0;
    HEAP32[$i>>2] = $161;
   }
  }
 } while(0);
 $162 = ((($sCheck)) + 68|0);
 $163 = HEAP32[$162>>2]|0;
 _sqlite3PageFree($163);
 $164 = ((($sCheck)) + 8|0);
 $165 = HEAP32[$164>>2]|0;
 _sqlite3_free($165);
 $166 = ((($sCheck)) + 24|0);
 $167 = HEAP32[$166>>2]|0;
 $168 = ($167|0)!=(0);
 if ($168) {
  $169 = ((($sCheck)) + 40|0);
  _sqlite3StrAccumReset($169);
  $170 = ((($sCheck)) + 20|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = (($171) + 1)|0;
  HEAP32[$170>>2] = $172;
 }
 $173 = ((($sCheck)) + 20|0);
 $174 = HEAP32[$173>>2]|0;
 $175 = HEAP32[$4>>2]|0;
 HEAP32[$175>>2] = $174;
 $176 = ((($sCheck)) + 20|0);
 $177 = HEAP32[$176>>2]|0;
 $178 = ($177|0)==(0);
 if (!($178)) {
  $180 = ((($sCheck)) + 40|0);
  $181 = (_sqlite3StrAccumFinish($180)|0);
  STACKTOP = sp;return ($181|0);
 }
 $179 = ((($sCheck)) + 40|0);
 _sqlite3StrAccumReset($179);
 $180 = ((($sCheck)) + 40|0);
 $181 = (_sqlite3StrAccumFinish($180)|0);
 STACKTOP = sp;return ($181|0);
}
function _sqlite3VdbeMemSetRowSet($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($4);
 $5 = HEAP32[$db>>2]|0;
 $6 = (_sqlite3DbMallocRawNN($5,64,0)|0);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 20|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 65|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 8|0);
  HEAP16[$14>>1] = 1;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 24|0);
  HEAP32[$16>>2] = 0;
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$db>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3DbMallocSize($17,$20)|0);
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 24|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$db>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 20|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 24|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3RowSetInit($24,$27,$30)|0);
  $32 = HEAP32[$0>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 8|0);
  HEAP16[$34>>1] = 32;
  STACKTOP = sp;return;
 }
}
function _sqlite3RowSetInsert($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, $pLast = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $pEntry = sp + 12|0;
 $pLast = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_rowSetEntryAlloc($8)|0);
 HEAP32[$pEntry>>2] = $9;
 $10 = HEAP32[$pEntry>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = $3;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$pEntry>>2]|0;
 $19 = $18;
 $20 = $19;
 HEAP32[$20>>2] = $14;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = HEAP32[$pEntry>>2]|0;
 $24 = ((($23)) + 8|0);
 HEAP32[$24>>2] = 0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pLast>>2] = $27;
 $28 = HEAP32[$pLast>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 26|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32&65535;
  $34 = $33 & 1;
  $35 = ($34|0)!=(0);
  if ($35) {
   $36 = $3;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($36) + 4)|0;
   $40 = $39;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pLast>>2]|0;
   $43 = $42;
   $44 = $43;
   $45 = HEAP32[$44>>2]|0;
   $46 = (($43) + 4)|0;
   $47 = $46;
   $48 = HEAP32[$47>>2]|0;
   $49 = ($41|0)<($48|0);
   $50 = ($38>>>0)<=($45>>>0);
   $51 = ($41|0)==($48|0);
   $52 = $51 & $50;
   $53 = $49 | $52;
   if ($53) {
    $54 = HEAP32[$2>>2]|0;
    $55 = ((($54)) + 26|0);
    $56 = HEAP16[$55>>1]|0;
    $57 = $56&65535;
    $58 = $57 & -2;
    $59 = $58&65535;
    HEAP16[$55>>1] = $59;
   }
  }
  $60 = HEAP32[$pEntry>>2]|0;
  $61 = HEAP32[$pLast>>2]|0;
  $62 = ((($61)) + 8|0);
  HEAP32[$62>>2] = $60;
 } else {
  $63 = HEAP32[$pEntry>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = ((($64)) + 8|0);
  HEAP32[$65>>2] = $63;
 }
 $66 = HEAP32[$pEntry>>2]|0;
 $67 = HEAP32[$2>>2]|0;
 $68 = ((($67)) + 12|0);
 HEAP32[$68>>2] = $66;
 STACKTOP = sp;return;
}
function _sqlite3RowSetNext($p,$pRowid) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pRowid;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 26|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  _rowSetToList($9);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  HEAP32[$0>>2] = 0;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = $23;
 $25 = $24;
 HEAP32[$25>>2] = $19;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $22;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ((($30)) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ($37|0)==(0|0);
 if ($38) {
  $39 = HEAP32[$1>>2]|0;
  _sqlite3RowSetClear($39);
 }
 HEAP32[$0>>2] = 1;
 $40 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($40|0);
}
function _sqlite3RowSetTest($pRowSet,$iBatch,$0,$1) {
 $pRowSet = $pRowSet|0;
 $iBatch = $iBatch|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $p = 0, $pAux = 0, $pTail = 0, $pTree = 0, $ppPrevTree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp;
 $p = sp + 24|0;
 $pTree = sp + 20|0;
 $ppPrevTree = sp + 16|0;
 $pAux = sp + 12|0;
 $pTail = sp + 8|0;
 HEAP32[$3>>2] = $pRowSet;
 HEAP32[$4>>2] = $iBatch;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$p>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$3>>2]|0;
   $21 = ((($20)) + 20|0);
   HEAP32[$ppPrevTree>>2] = $21;
   $22 = HEAP32[$3>>2]|0;
   $23 = ((($22)) + 26|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24&65535;
   $26 = $25 & 1;
   $27 = ($26|0)==(0);
   if ($27) {
    $28 = HEAP32[$p>>2]|0;
    $29 = (_rowSetEntrySort($28)|0);
    HEAP32[$p>>2] = $29;
   }
   $30 = HEAP32[$3>>2]|0;
   $31 = ((($30)) + 20|0);
   $32 = HEAP32[$31>>2]|0;
   HEAP32[$pTree>>2] = $32;
   while(1) {
    $33 = HEAP32[$pTree>>2]|0;
    $34 = ($33|0)!=(0|0);
    if (!($34)) {
     break;
    }
    $35 = HEAP32[$pTree>>2]|0;
    $36 = ((($35)) + 8|0);
    HEAP32[$ppPrevTree>>2] = $36;
    $37 = HEAP32[$pTree>>2]|0;
    $38 = ((($37)) + 12|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)==(0|0);
    if ($40) {
     label = 8;
     break;
    }
    $45 = HEAP32[$pTree>>2]|0;
    $46 = ((($45)) + 12|0);
    $47 = HEAP32[$46>>2]|0;
    _rowSetTreeToList($47,$pAux,$pTail);
    $48 = HEAP32[$pTree>>2]|0;
    $49 = ((($48)) + 12|0);
    HEAP32[$49>>2] = 0;
    $50 = HEAP32[$pAux>>2]|0;
    $51 = HEAP32[$p>>2]|0;
    $52 = (_rowSetEntryMerge($50,$51)|0);
    HEAP32[$p>>2] = $52;
    $53 = HEAP32[$pTree>>2]|0;
    $54 = ((($53)) + 8|0);
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$pTree>>2] = $55;
   }
   if ((label|0) == 8) {
    $41 = HEAP32[$p>>2]|0;
    $42 = (_rowSetListToTree($41)|0);
    $43 = HEAP32[$pTree>>2]|0;
    $44 = ((($43)) + 12|0);
    HEAP32[$44>>2] = $42;
   }
   $56 = HEAP32[$pTree>>2]|0;
   $57 = ($56|0)==(0|0);
   if ($57) {
    $58 = HEAP32[$3>>2]|0;
    $59 = (_rowSetEntryAlloc($58)|0);
    HEAP32[$pTree>>2] = $59;
    $60 = HEAP32[$ppPrevTree>>2]|0;
    HEAP32[$60>>2] = $59;
    $61 = HEAP32[$pTree>>2]|0;
    $62 = ($61|0)!=(0|0);
    if ($62) {
     $63 = HEAP32[$pTree>>2]|0;
     $64 = $63;
     $65 = $64;
     HEAP32[$65>>2] = 0;
     $66 = (($64) + 4)|0;
     $67 = $66;
     HEAP32[$67>>2] = 0;
     $68 = HEAP32[$pTree>>2]|0;
     $69 = ((($68)) + 8|0);
     HEAP32[$69>>2] = 0;
     $70 = HEAP32[$p>>2]|0;
     $71 = (_rowSetListToTree($70)|0);
     $72 = HEAP32[$pTree>>2]|0;
     $73 = ((($72)) + 12|0);
     HEAP32[$73>>2] = $71;
    }
   }
   $74 = HEAP32[$3>>2]|0;
   $75 = ((($74)) + 8|0);
   HEAP32[$75>>2] = 0;
   $76 = HEAP32[$3>>2]|0;
   $77 = ((($76)) + 12|0);
   HEAP32[$77>>2] = 0;
   $78 = HEAP32[$3>>2]|0;
   $79 = ((($78)) + 26|0);
   $80 = HEAP16[$79>>1]|0;
   $81 = $80&65535;
   $82 = $81 | 1;
   $83 = $82&65535;
   HEAP16[$79>>1] = $83;
  }
  $84 = HEAP32[$4>>2]|0;
  $85 = HEAP32[$3>>2]|0;
  $86 = ((($85)) + 28|0);
  HEAP32[$86>>2] = $84;
 }
 $87 = HEAP32[$3>>2]|0;
 $88 = ((($87)) + 20|0);
 $89 = HEAP32[$88>>2]|0;
 HEAP32[$pTree>>2] = $89;
 L21: while(1) {
  $90 = HEAP32[$pTree>>2]|0;
  $91 = ($90|0)!=(0|0);
  if (!($91)) {
   label = 25;
   break;
  }
  $92 = HEAP32[$pTree>>2]|0;
  $93 = ((($92)) + 12|0);
  $94 = HEAP32[$93>>2]|0;
  HEAP32[$p>>2] = $94;
  while(1) {
   $95 = HEAP32[$p>>2]|0;
   $96 = ($95|0)!=(0|0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$p>>2]|0;
   $98 = $97;
   $99 = $98;
   $100 = HEAP32[$99>>2]|0;
   $101 = (($98) + 4)|0;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = $5;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (($104) + 4)|0;
   $108 = $107;
   $109 = HEAP32[$108>>2]|0;
   $110 = ($103|0)<($109|0);
   $111 = ($100>>>0)<($106>>>0);
   $112 = ($103|0)==($109|0);
   $113 = $112 & $111;
   $114 = $110 | $113;
   $115 = HEAP32[$p>>2]|0;
   if ($114) {
    $116 = ((($115)) + 8|0);
    $117 = HEAP32[$116>>2]|0;
    HEAP32[$p>>2] = $117;
    continue;
   }
   $118 = $115;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   $124 = $5;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = ($123|0)>($129|0);
   $131 = ($120>>>0)>($126>>>0);
   $132 = ($123|0)==($129|0);
   $133 = $132 & $131;
   $134 = $130 | $133;
   if (!($134)) {
    label = 23;
    break L21;
   }
   $135 = HEAP32[$p>>2]|0;
   $136 = ((($135)) + 12|0);
   $137 = HEAP32[$136>>2]|0;
   HEAP32[$p>>2] = $137;
  }
  $138 = HEAP32[$pTree>>2]|0;
  $139 = ((($138)) + 8|0);
  $140 = HEAP32[$139>>2]|0;
  HEAP32[$pTree>>2] = $140;
 }
 if ((label|0) == 23) {
  HEAP32[$2>>2] = 1;
  $141 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 else if ((label|0) == 25) {
  HEAP32[$2>>2] = 0;
  $141 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemInit($pMem,$db,$flags) {
 $pMem = $pMem|0;
 $db = $db|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $db;
 HEAP16[$2>>1] = $flags;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP16[$5>>1] = $3;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 32|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 24|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3PagerOkToChangeJournalMode($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pPager;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 17|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)>=(3);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 80|0);
  $14 = $13;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)>(0);
  $21 = ($16>>>0)>(0);
  $22 = ($19|0)==(0);
  $23 = $22 & $21;
  $24 = $20 | $23;
  if ($24) {
   HEAP32[$0>>2] = 0;
   $25 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($25|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3PagerCloseWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $logexists = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $logexists = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  HEAP32[$logexists>>2] = 0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (_pagerLockDb($5,1)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 220|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (_sqlite3OsAccess($10,$13,0,$logexists)|0);
   HEAP32[$rc>>2] = $14;
  }
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)==(0);
  $17 = HEAP32[$logexists>>2]|0;
  $18 = ($17|0)!=(0);
  $or$cond = $16 & $18;
  if ($or$cond) {
   $19 = HEAP32[$0>>2]|0;
   $20 = (_pagerOpenWal($19)|0);
   HEAP32[$rc>>2] = $20;
  }
 }
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if (!($22)) {
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 216|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (_pagerExclusiveLock($27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)==(0);
 if (!($30)) {
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 216|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 10|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 160|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 208|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3WalClose($33,$37,$40,$43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 216|0);
 HEAP32[$46>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($47);
 $48 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _sqlite3PagerSetJournalMode($pPager,$eMode) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $eOld = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $eOld = sp + 16|0;
 $rc = sp + 4|0;
 $state = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 5|0);
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$eOld>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(4);
  $11 = HEAP32[$1>>2]|0;
  $12 = ($11|0)!=(2);
  $or$cond = $10 & $12;
  if ($or$cond) {
   $13 = HEAP8[$eOld>>0]|0;
   $14 = $13&255;
   HEAP32[$1>>2] = $14;
  }
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP8[$eOld>>0]|0;
 $17 = $16&255;
 $18 = ($15|0)!=($17|0);
 do {
  if ($18) {
   $19 = HEAP32[$1>>2]|0;
   $20 = $19&255;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 5|0);
   HEAP8[$22>>0] = $20;
   $23 = HEAP32[$0>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = ($25<<24>>24)!=(0);
   if (!($26)) {
    $27 = HEAP8[$eOld>>0]|0;
    $28 = $27&255;
    $29 = $28 & 5;
    $30 = ($29|0)==(1);
    if ($30) {
     $31 = HEAP32[$1>>2]|0;
     $32 = $31 & 1;
     $33 = ($32|0)==(0);
     if ($33) {
      $34 = HEAP32[$0>>2]|0;
      $35 = ((($34)) + 68|0);
      $36 = HEAP32[$35>>2]|0;
      (_sqlite3OsClose($36)|0);
      $37 = HEAP32[$0>>2]|0;
      $38 = ((($37)) + 18|0);
      $39 = HEAP8[$38>>0]|0;
      $40 = $39&255;
      $41 = ($40|0)>=(2);
      if ($41) {
       $42 = HEAP32[$0>>2]|0;
       $43 = HEAP32[$42>>2]|0;
       $44 = HEAP32[$0>>2]|0;
       $45 = ((($44)) + 180|0);
       $46 = HEAP32[$45>>2]|0;
       (_sqlite3OsDelete($43,$46,0)|0);
       break;
      }
      HEAP32[$rc>>2] = 0;
      $47 = HEAP32[$0>>2]|0;
      $48 = ((($47)) + 17|0);
      $49 = HEAP8[$48>>0]|0;
      $50 = $49&255;
      HEAP32[$state>>2] = $50;
      $51 = HEAP32[$state>>2]|0;
      $52 = ($51|0)==(0);
      if ($52) {
       $53 = HEAP32[$0>>2]|0;
       $54 = (_sqlite3PagerSharedLock($53)|0);
       HEAP32[$rc>>2] = $54;
      }
      $55 = HEAP32[$0>>2]|0;
      $56 = ((($55)) + 17|0);
      $57 = HEAP8[$56>>0]|0;
      $58 = $57&255;
      $59 = ($58|0)==(1);
      if ($59) {
       $60 = HEAP32[$0>>2]|0;
       $61 = (_pagerLockDb($60,2)|0);
       HEAP32[$rc>>2] = $61;
      }
      $62 = HEAP32[$rc>>2]|0;
      $63 = ($62|0)==(0);
      if ($63) {
       $64 = HEAP32[$0>>2]|0;
       $65 = HEAP32[$64>>2]|0;
       $66 = HEAP32[$0>>2]|0;
       $67 = ((($66)) + 180|0);
       $68 = HEAP32[$67>>2]|0;
       (_sqlite3OsDelete($65,$68,0)|0);
      }
      $69 = HEAP32[$rc>>2]|0;
      $70 = ($69|0)==(0);
      $71 = HEAP32[$state>>2]|0;
      $72 = ($71|0)==(1);
      $or$cond3 = $70 & $72;
      if ($or$cond3) {
       $73 = HEAP32[$0>>2]|0;
       (_pagerUnlockDb($73,1)|0);
       break;
      }
      $74 = HEAP32[$state>>2]|0;
      $75 = ($74|0)==(0);
      if (!($75)) {
       break;
      }
      $76 = HEAP32[$0>>2]|0;
      _pager_unlock($76);
      break;
     }
    }
   }
   $77 = HEAP32[$1>>2]|0;
   $78 = ($77|0)==(2);
   if ($78) {
    $79 = HEAP32[$0>>2]|0;
    $80 = ((($79)) + 68|0);
    $81 = HEAP32[$80>>2]|0;
    (_sqlite3OsClose($81)|0);
   }
  }
 } while(0);
 $82 = HEAP32[$0>>2]|0;
 $83 = ((($82)) + 5|0);
 $84 = HEAP8[$83>>0]|0;
 $85 = $84&255;
 STACKTOP = sp;return ($85|0);
}
function _sqlite3JournalModename($eMode) {
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(6);
 if ($3) {
  HEAP32[$0>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (2436 + ($4<<2)|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$0>>2] = $6;
 }
 $7 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3RunVacuum($pzErrMsg,$db) {
 $pzErrMsg = $pzErrMsg|0;
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $isMemDb = 0, $meta = 0, $nDb = 0, $nRes = 0, $pDb = 0, $pMain = 0, $pTemp = 0, $rc = 0, $saved_flags = 0, $saved_nChange = 0, $saved_nTotalChange = 0, $saved_xTrace = 0, $zSql = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $rc = sp + 52|0;
 $pMain = sp + 48|0;
 $pTemp = sp + 44|0;
 $zSql = sp + 40|0;
 $saved_flags = sp + 36|0;
 $saved_nChange = sp + 32|0;
 $saved_nTotalChange = sp + 28|0;
 $saved_xTrace = sp + 24|0;
 $pDb = sp + 20|0;
 $isMemDb = sp + 16|0;
 $nRes = sp + 12|0;
 $nDb = sp + 8|0;
 $meta = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pzErrMsg;
 HEAP32[$2>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$zSql>>2] = 0;
 HEAP32[$pDb>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 63|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  _sqlite3SetString($7,$8,22540);
  HEAP32[$0>>2] = 1;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 148|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(1);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _sqlite3SetString($13,$14,22580);
  HEAP32[$0>>2] = 1;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$saved_flags>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 80|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$saved_nChange>>2] = $20;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 84|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$saved_nTotalChange>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = ((($24)) + 176|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$saved_xTrace>>2] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = ((($27)) + 24|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $29 | 2107392;
 HEAP32[$28>>2] = $30;
 $31 = HEAP32[$2>>2]|0;
 $32 = ((($31)) + 24|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $33 & -655361;
 HEAP32[$32>>2] = $34;
 $35 = HEAP32[$2>>2]|0;
 $36 = ((($35)) + 176|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$2>>2]|0;
 $38 = ((($37)) + 16|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = ((($39)) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 HEAP32[$pMain>>2] = $41;
 $42 = HEAP32[$pMain>>2]|0;
 $43 = (_sqlite3BtreePager($42)|0);
 $44 = (_sqlite3PagerIsMemdb($43)|0);
 HEAP32[$isMemDb>>2] = $44;
 $45 = HEAP32[$2>>2]|0;
 $46 = ((($45)) + 20|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$nDb>>2] = $47;
 $48 = HEAP32[$2>>2]|0;
 $49 = (_sqlite3TempInMemory($48)|0);
 $50 = ($49|0)!=(0);
 if ($50) {
  HEAP32[$zSql>>2] = 22623;
 } else {
  HEAP32[$zSql>>2] = 22655;
 }
 $51 = HEAP32[$2>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = HEAP32[$zSql>>2]|0;
 $54 = (_execSql($51,$52,$53)|0);
 HEAP32[$rc>>2] = $54;
 $55 = HEAP32[$2>>2]|0;
 $56 = ((($55)) + 20|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$nDb>>2]|0;
 $59 = ($57|0)>($58|0);
 if ($59) {
  $60 = HEAP32[$2>>2]|0;
  $61 = ((($60)) + 20|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) - 1)|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = ((($64)) + 16|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (($66) + ($63<<4)|0);
  HEAP32[$pDb>>2] = $67;
 }
 $68 = HEAP32[$rc>>2]|0;
 $69 = ($68|0)!=(0);
 L16: do {
  if (!($69)) {
   $70 = HEAP32[$2>>2]|0;
   $71 = ((($70)) + 20|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($72) - 1)|0;
   $74 = HEAP32[$2>>2]|0;
   $75 = ((($74)) + 16|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = (($76) + ($73<<4)|0);
   $78 = ((($77)) + 4|0);
   $79 = HEAP32[$78>>2]|0;
   HEAP32[$pTemp>>2] = $79;
   $80 = HEAP32[$pTemp>>2]|0;
   (_sqlite3BtreeCommit($80)|0);
   $81 = HEAP32[$pMain>>2]|0;
   $82 = (_sqlite3BtreeGetOptimalReserve($81)|0);
   HEAP32[$nRes>>2] = $82;
   $83 = HEAP32[$2>>2]|0;
   $84 = HEAP32[$1>>2]|0;
   $85 = (_execSql($83,$84,22679)|0);
   HEAP32[$rc>>2] = $85;
   $86 = HEAP32[$rc>>2]|0;
   $87 = ($86|0)!=(0);
   if (!($87)) {
    $88 = HEAP32[$2>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = (_execSql($88,$89,22712)|0);
    HEAP32[$rc>>2] = $90;
    $91 = HEAP32[$rc>>2]|0;
    $92 = ($91|0)!=(0);
    if (!($92)) {
     $93 = HEAP32[$pMain>>2]|0;
     $94 = (_sqlite3BtreeBeginTrans($93,2)|0);
     HEAP32[$rc>>2] = $94;
     $95 = HEAP32[$rc>>2]|0;
     $96 = ($95|0)!=(0);
     if (!($96)) {
      $97 = HEAP32[$pMain>>2]|0;
      $98 = (_sqlite3BtreePager($97)|0);
      $99 = (_sqlite3PagerGetJournalMode($98)|0);
      $100 = ($99|0)==(5);
      if ($100) {
       $101 = HEAP32[$2>>2]|0;
       $102 = ((($101)) + 72|0);
       HEAP32[$102>>2] = 0;
      }
      $103 = HEAP32[$pTemp>>2]|0;
      $104 = HEAP32[$pMain>>2]|0;
      $105 = (_sqlite3BtreeGetPageSize($104)|0);
      $106 = HEAP32[$nRes>>2]|0;
      $107 = (_sqlite3BtreeSetPageSize($103,$105,$106,0)|0);
      $108 = ($107|0)!=(0);
      do {
       if (!($108)) {
        $109 = HEAP32[$isMemDb>>2]|0;
        $110 = ($109|0)!=(0);
        if (!($110)) {
         $111 = HEAP32[$pTemp>>2]|0;
         $112 = HEAP32[$2>>2]|0;
         $113 = ((($112)) + 72|0);
         $114 = HEAP32[$113>>2]|0;
         $115 = HEAP32[$nRes>>2]|0;
         $116 = (_sqlite3BtreeSetPageSize($111,$114,$115,0)|0);
         $117 = ($116|0)!=(0);
         if ($117) {
          break;
         }
        }
        $118 = HEAP32[$2>>2]|0;
        $119 = ((($118)) + 65|0);
        $120 = HEAP8[$119>>0]|0;
        $121 = $120&255;
        $122 = ($121|0)!=(0);
        if (!($122)) {
         $123 = HEAP32[$pTemp>>2]|0;
         $124 = HEAP32[$2>>2]|0;
         $125 = ((($124)) + 68|0);
         $126 = HEAP8[$125>>0]|0;
         $127 = $126 << 24 >> 24;
         $128 = ($127|0)>=(0);
         if ($128) {
          $129 = HEAP32[$2>>2]|0;
          $130 = ((($129)) + 68|0);
          $131 = HEAP8[$130>>0]|0;
          $132 = $131 << 24 >> 24;
          $135 = $132;
         } else {
          $133 = HEAP32[$pMain>>2]|0;
          $134 = (_sqlite3BtreeGetAutoVacuum($133)|0);
          $135 = $134;
         }
         (_sqlite3BtreeSetAutoVacuum($123,$135)|0);
         $136 = HEAP32[$2>>2]|0;
         $137 = HEAP32[$1>>2]|0;
         $138 = (_execExecSql($136,$137,22719)|0);
         HEAP32[$rc>>2] = $138;
         $139 = HEAP32[$rc>>2]|0;
         $140 = ($139|0)!=(0);
         if ($140) {
          break L16;
         }
         $141 = HEAP32[$2>>2]|0;
         $142 = HEAP32[$1>>2]|0;
         $143 = (_execExecSql($141,$142,22867)|0);
         HEAP32[$rc>>2] = $143;
         $144 = HEAP32[$rc>>2]|0;
         $145 = ($144|0)!=(0);
         if ($145) {
          break L16;
         }
         $146 = HEAP32[$2>>2]|0;
         $147 = HEAP32[$1>>2]|0;
         $148 = (_execExecSql($146,$147,22971)|0);
         HEAP32[$rc>>2] = $148;
         $149 = HEAP32[$rc>>2]|0;
         $150 = ($149|0)!=(0);
         if ($150) {
          break L16;
         }
         $151 = HEAP32[$2>>2]|0;
         $152 = ((($151)) + 24|0);
         $153 = HEAP32[$152>>2]|0;
         $154 = $153 | 134217728;
         HEAP32[$152>>2] = $154;
         $155 = HEAP32[$2>>2]|0;
         $156 = HEAP32[$1>>2]|0;
         $157 = (_execExecSql($155,$156,23089)|0);
         HEAP32[$rc>>2] = $157;
         $158 = HEAP32[$2>>2]|0;
         $159 = ((($158)) + 24|0);
         $160 = HEAP32[$159>>2]|0;
         $161 = $160 & -134217729;
         HEAP32[$159>>2] = $161;
         $162 = HEAP32[$rc>>2]|0;
         $163 = ($162|0)!=(0);
         if ($163) {
          break L16;
         }
         $164 = HEAP32[$2>>2]|0;
         $165 = HEAP32[$1>>2]|0;
         $166 = (_execExecSql($164,$165,23285)|0);
         HEAP32[$rc>>2] = $166;
         $167 = HEAP32[$rc>>2]|0;
         $168 = ($167|0)!=(0);
         if ($168) {
          break L16;
         }
         $169 = HEAP32[$2>>2]|0;
         $170 = HEAP32[$1>>2]|0;
         $171 = (_execExecSql($169,$170,23398)|0);
         HEAP32[$rc>>2] = $171;
         $172 = HEAP32[$rc>>2]|0;
         $173 = ($172|0)!=(0);
         if ($173) {
          break L16;
         }
         $174 = HEAP32[$2>>2]|0;
         $175 = HEAP32[$1>>2]|0;
         $176 = (_execSql($174,$175,23553)|0);
         HEAP32[$rc>>2] = $176;
         $177 = HEAP32[$rc>>2]|0;
         $178 = ($177|0)!=(0);
         if ($178) {
          break L16;
         }
         HEAP32[$i>>2] = 0;
         while(1) {
          $179 = HEAP32[$i>>2]|0;
          $180 = ($179|0)<(10);
          $181 = HEAP32[$pMain>>2]|0;
          if (!($180)) {
           break;
          }
          $182 = HEAP32[$i>>2]|0;
          $183 = (23737 + ($182)|0);
          $184 = HEAP8[$183>>0]|0;
          $185 = $184&255;
          _sqlite3BtreeGetMeta($181,$185,$meta);
          $186 = HEAP32[$pTemp>>2]|0;
          $187 = HEAP32[$i>>2]|0;
          $188 = (23737 + ($187)|0);
          $189 = HEAP8[$188>>0]|0;
          $190 = $189&255;
          $191 = HEAP32[$meta>>2]|0;
          $192 = HEAP32[$i>>2]|0;
          $193 = (($192) + 1)|0;
          $194 = (23737 + ($193)|0);
          $195 = HEAP8[$194>>0]|0;
          $196 = $195&255;
          $197 = (($191) + ($196))|0;
          $198 = (_sqlite3BtreeUpdateMeta($186,$190,$197)|0);
          HEAP32[$rc>>2] = $198;
          $199 = HEAP32[$rc>>2]|0;
          $200 = ($199|0)!=(0);
          if ($200) {
           break L16;
          }
          $201 = HEAP32[$i>>2]|0;
          $202 = (($201) + 2)|0;
          HEAP32[$i>>2] = $202;
         }
         $203 = HEAP32[$pTemp>>2]|0;
         $204 = (_sqlite3BtreeCopyFile($181,$203)|0);
         HEAP32[$rc>>2] = $204;
         $205 = HEAP32[$rc>>2]|0;
         $206 = ($205|0)!=(0);
         if ($206) {
          break L16;
         }
         $207 = HEAP32[$pTemp>>2]|0;
         $208 = (_sqlite3BtreeCommit($207)|0);
         HEAP32[$rc>>2] = $208;
         $209 = HEAP32[$rc>>2]|0;
         $210 = ($209|0)!=(0);
         if ($210) {
          break L16;
         }
         $211 = HEAP32[$pMain>>2]|0;
         $212 = HEAP32[$pTemp>>2]|0;
         $213 = (_sqlite3BtreeGetAutoVacuum($212)|0);
         (_sqlite3BtreeSetAutoVacuum($211,$213)|0);
         $214 = HEAP32[$pMain>>2]|0;
         $215 = HEAP32[$pTemp>>2]|0;
         $216 = (_sqlite3BtreeGetPageSize($215)|0);
         $217 = HEAP32[$nRes>>2]|0;
         $218 = (_sqlite3BtreeSetPageSize($214,$216,$217,1)|0);
         HEAP32[$rc>>2] = $218;
         break L16;
        }
       }
      } while(0);
      HEAP32[$rc>>2] = 7;
     }
    }
   }
  }
 } while(0);
 $219 = HEAP32[$saved_flags>>2]|0;
 $220 = HEAP32[$2>>2]|0;
 $221 = ((($220)) + 24|0);
 HEAP32[$221>>2] = $219;
 $222 = HEAP32[$saved_nChange>>2]|0;
 $223 = HEAP32[$2>>2]|0;
 $224 = ((($223)) + 80|0);
 HEAP32[$224>>2] = $222;
 $225 = HEAP32[$saved_nTotalChange>>2]|0;
 $226 = HEAP32[$2>>2]|0;
 $227 = ((($226)) + 84|0);
 HEAP32[$227>>2] = $225;
 $228 = HEAP32[$saved_xTrace>>2]|0;
 $229 = HEAP32[$2>>2]|0;
 $230 = ((($229)) + 176|0);
 HEAP32[$230>>2] = $228;
 $231 = HEAP32[$pMain>>2]|0;
 (_sqlite3BtreeSetPageSize($231,-1,-1,1)|0);
 $232 = HEAP32[$2>>2]|0;
 $233 = ((($232)) + 63|0);
 HEAP8[$233>>0] = 1;
 $234 = HEAP32[$pDb>>2]|0;
 $235 = ($234|0)!=(0|0);
 if ($235) {
  $236 = HEAP32[$pDb>>2]|0;
  $237 = ((($236)) + 4|0);
  $238 = HEAP32[$237>>2]|0;
  (_sqlite3BtreeClose($238)|0);
  $239 = HEAP32[$pDb>>2]|0;
  $240 = ((($239)) + 4|0);
  HEAP32[$240>>2] = 0;
  $241 = HEAP32[$pDb>>2]|0;
  $242 = ((($241)) + 12|0);
  HEAP32[$242>>2] = 0;
 }
 $243 = HEAP32[$2>>2]|0;
 _sqlite3ResetAllSchemasOfConnection($243);
 $244 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $244;
 $245 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($245|0);
}
function _sqlite3BtreeIncrVacuum($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFin = 0, $nFree = 0, $nOrig = 0, $pBt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $rc = sp + 16|0;
 $pBt = sp + 12|0;
 $nOrig = sp + 8|0;
 $nFree = sp + 4|0;
 $nFin = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = ((($5)) + 17|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  HEAP32[$rc>>2] = 101;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $9 = HEAP32[$pBt>>2]|0;
 $10 = (_btreePagecount($9)|0);
 HEAP32[$nOrig>>2] = $10;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 36|0);
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$nFree>>2] = $17;
 $18 = HEAP32[$pBt>>2]|0;
 $19 = HEAP32[$nOrig>>2]|0;
 $20 = HEAP32[$nFree>>2]|0;
 $21 = (_finalDbSize($18,$19,$20)|0);
 HEAP32[$nFin>>2] = $21;
 $22 = HEAP32[$nOrig>>2]|0;
 $23 = HEAP32[$nFin>>2]|0;
 $24 = ($22>>>0)<($23>>>0);
 if ($24) {
  $25 = (_sqlite3CorruptError(59178)|0);
  HEAP32[$rc>>2] = $25;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $26 = HEAP32[$nFree>>2]|0;
 $27 = ($26>>>0)>(0);
 if (!($27)) {
  HEAP32[$rc>>2] = 101;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $28 = HEAP32[$pBt>>2]|0;
 $29 = (_saveAllCursors($28,0,0)|0);
 HEAP32[$rc>>2] = $29;
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(0);
 if ($31) {
  $32 = HEAP32[$pBt>>2]|0;
  _invalidateAllOverflowCache($32);
  $33 = HEAP32[$pBt>>2]|0;
  $34 = HEAP32[$nFin>>2]|0;
  $35 = HEAP32[$nOrig>>2]|0;
  $36 = (_incrVacuumStep($33,$34,$35,0)|0);
  HEAP32[$rc>>2] = $36;
 }
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if (!($38)) {
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $39 = HEAP32[$pBt>>2]|0;
 $40 = ((($39)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ((($41)) + 72|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3PagerWrite($43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$pBt>>2]|0;
 $46 = ((($45)) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = ((($47)) + 56|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($49)) + 28|0);
 $51 = HEAP32[$pBt>>2]|0;
 $52 = ((($51)) + 44|0);
 $53 = HEAP32[$52>>2]|0;
 _sqlite3Put4byte($50,$53);
 $54 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _sqlite3BtreeLockTable($p,$iTab,$isWriteLock) {
 $p = $p|0;
 $iTab = $iTab|0;
 $isWriteLock = $isWriteLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $lockType = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 13|0;
 $rc = sp;
 $lockType = sp + 12|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTab;
 HEAP8[$2>>0] = $isWriteLock;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 9|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $7 = HEAP8[$2>>0]|0;
 $8 = $7&255;
 $9 = (1 + ($8))|0;
 $10 = $9&255;
 HEAP8[$lockType>>0] = $10;
 $11 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP8[$lockType>>0]|0;
 $15 = (_querySharedCacheTableLock($12,$13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP8[$lockType>>0]|0;
 $21 = (_setSharedCacheTableLock($18,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3VtabBegin($db,$pVTab) {
 $db = $db|0;
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iSvpt = 0, $pModule = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pModule = sp + 8|0;
 $i = sp + 4|0;
 $iSvpt = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pVTab;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 308|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 332|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0|0);
  if ($10) {
   HEAP32[$0>>2] = 6;
   $65 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($65|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pModule>>2] = $16;
 $17 = HEAP32[$pModule>>2]|0;
 $18 = ((($17)) + 56|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 308|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($21|0)<($24|0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 332|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + ($26<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = ($31|0)==($32|0);
   if ($33) {
    label = 10;
    break;
   }
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$i>>2] = $35;
  }
  if ((label|0) == 10) {
   HEAP32[$0>>2] = 0;
   $65 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($65|0);
  }
  $36 = HEAP32[$1>>2]|0;
  $37 = (_growVTrans($36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$rc>>2]|0;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = HEAP32[$pModule>>2]|0;
   $41 = ((($40)) + 56|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$2>>2]|0;
   $44 = ((($43)) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (FUNCTION_TABLE_ii[$42 & 255]($45)|0);
   HEAP32[$rc>>2] = $46;
   $47 = HEAP32[$rc>>2]|0;
   $48 = ($47|0)==(0);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 504|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 500|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($51) + ($54))|0;
    HEAP32[$iSvpt>>2] = $55;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$2>>2]|0;
    _addToVTrans($56,$57);
    $58 = HEAP32[$iSvpt>>2]|0;
    $59 = ($58|0)!=(0);
    if ($59) {
     $60 = HEAP32[$1>>2]|0;
     $61 = HEAP32[$iSvpt>>2]|0;
     $62 = (($61) - 1)|0;
     $63 = (_sqlite3VtabSavepoint($60,0,$62)|0);
     HEAP32[$rc>>2] = $63;
    }
   }
  }
 }
 $64 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $64;
 $65 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _sqlite3VtabCallCreate($db,$iDb,$zTab,$pzErr) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTab = $zTab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pMod = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0, $zMod = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $pTab = sp + 12|0;
 $pMod = sp + 8|0;
 $zMod = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTab;
 HEAP32[$3>>2] = $pzErr;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3FindTable($4,$5,$11)|0);
 HEAP32[$pTab>>2] = $12;
 $13 = HEAP32[$pTab>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$zMod>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 312|0);
 $19 = HEAP32[$zMod>>2]|0;
 $20 = (_sqlite3HashFind($18,$19)|0);
 HEAP32[$pMod>>2] = $20;
 $21 = HEAP32[$pMod>>2]|0;
 $22 = ($21|0)==(0|0);
 if ($22) {
  label = 4;
 } else {
  $23 = HEAP32[$pMod>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)==(0|0);
  if ($27) {
   label = 4;
  } else {
   $28 = HEAP32[$pMod>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ((($29)) + 20|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)==(0|0);
   if ($32) {
    label = 4;
   } else {
    $37 = HEAP32[$0>>2]|0;
    $38 = HEAP32[$pTab>>2]|0;
    $39 = HEAP32[$pMod>>2]|0;
    $40 = HEAP32[$pMod>>2]|0;
    $41 = HEAP32[$40>>2]|0;
    $42 = ((($41)) + 4|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$3>>2]|0;
    $45 = (_vtabCallConstructor($37,$38,$39,$43,$44)|0);
    HEAP32[$rc>>2] = $45;
   }
  }
 }
 if ((label|0) == 4) {
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$zMod>>2]|0;
  HEAP32[$vararg_buffer>>2] = $34;
  $35 = (_sqlite3MPrintf($33,23747,$vararg_buffer)|0);
  $36 = HEAP32[$3>>2]|0;
  HEAP32[$36>>2] = $35;
  HEAP32[$rc>>2] = 1;
 }
 $46 = HEAP32[$rc>>2]|0;
 $47 = ($46|0)==(0);
 if (!($47)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$pTab>>2]|0;
 $50 = (_sqlite3GetVTable($48,$49)|0);
 $51 = ($50|0)!=(0|0);
 if (!($51)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $52 = HEAP32[$0>>2]|0;
 $53 = (_growVTrans($52)|0);
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$rc>>2]|0;
 $55 = ($54|0)==(0);
 if (!($55)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $56 = HEAP32[$0>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$pTab>>2]|0;
 $59 = (_sqlite3GetVTable($57,$58)|0);
 _addToVTrans($56,$59);
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _sqlite3VtabCallDestroy($db,$iDb,$zTab) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTab = $zTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pTab = 0, $rc = 0, $xDestroy = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pTab = sp + 8|0;
 $p = sp + 4|0;
 $xDestroy = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3FindTable($4,$5,$11)|0);
 HEAP32[$pTab>>2] = $12;
 $13 = HEAP32[$pTab>>2]|0;
 $14 = ($13|0)!=(0|0);
 do {
  if ($14) {
   $15 = HEAP32[$pTab>>2]|0;
   $16 = ((($15)) + 56|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $19 = HEAP32[$pTab>>2]|0;
    $20 = ((($19)) + 56|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$p>>2] = $21;
    while(1) {
     $22 = HEAP32[$p>>2]|0;
     $23 = ($22|0)!=(0|0);
     if (!($23)) {
      label = 8;
      break;
     }
     $24 = HEAP32[$p>>2]|0;
     $25 = ((($24)) + 8|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ((($26)) + 4|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = ($28|0)>(0);
     if ($29) {
      label = 6;
      break;
     }
     $30 = HEAP32[$p>>2]|0;
     $31 = ((($30)) + 24|0);
     $32 = HEAP32[$31>>2]|0;
     HEAP32[$p>>2] = $32;
    }
    if ((label|0) == 6) {
     HEAP32[$0>>2] = 6;
     $55 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($55|0);
    }
    else if ((label|0) == 8) {
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$pTab>>2]|0;
     $35 = (_vtabDisconnectAll($33,$34)|0);
     HEAP32[$p>>2] = $35;
     $36 = HEAP32[$p>>2]|0;
     $37 = ((($36)) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = HEAP32[$38>>2]|0;
     $40 = ((($39)) + 20|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$xDestroy>>2] = $41;
     $42 = HEAP32[$xDestroy>>2]|0;
     $43 = HEAP32[$p>>2]|0;
     $44 = ((($43)) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (FUNCTION_TABLE_ii[$42 & 255]($45)|0);
     HEAP32[$rc>>2] = $46;
     $47 = HEAP32[$rc>>2]|0;
     $48 = ($47|0)==(0);
     if (!($48)) {
      break;
     }
     $49 = HEAP32[$p>>2]|0;
     $50 = ((($49)) + 8|0);
     HEAP32[$50>>2] = 0;
     $51 = HEAP32[$pTab>>2]|0;
     $52 = ((($51)) + 56|0);
     HEAP32[$52>>2] = 0;
     $53 = HEAP32[$p>>2]|0;
     _sqlite3VtabUnlock($53);
     break;
    }
   }
  }
 } while(0);
 $54 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3BtreeMaxPageCount($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3PagerMaxPageCount($6,$7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3VdbeExpandSql($p,$zRawSql) {
 $p = $p|0;
 $zRawSql = $zRawSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0.0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $db = 0, $enc = 0, $i = 0, $idx = 0, $n = 0, $nOut = 0, $nOut1 = 0, $nToken = 0, $nextIndex = 0, $out = 0, $pVar = 0, $utf8 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr7 = 0, $zBase = 0, $zStart = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer11 = sp + 72|0;
 $vararg_buffer8 = sp + 64|0;
 $vararg_buffer4 = sp + 56|0;
 $vararg_buffer1 = sp + 48|0;
 $vararg_buffer = sp + 40|0;
 $0 = sp + 148|0;
 $1 = sp + 144|0;
 $db = sp + 140|0;
 $idx = sp + 136|0;
 $nextIndex = sp + 132|0;
 $n = sp + 128|0;
 $nToken = sp + 124|0;
 $i = sp + 120|0;
 $pVar = sp + 116|0;
 $out = sp + 88|0;
 $zBase = sp + 156|0;
 $zStart = sp + 84|0;
 $nOut = sp + 80|0;
 $enc = sp + 152|0;
 $utf8 = sp;
 $nOut1 = sp + 76|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zRawSql;
 HEAP32[$idx>>2] = 0;
 HEAP32[$nextIndex>>2] = 1;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 88|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3StrAccumInit($out,$4,$zBase,100,$7);
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 160|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)>(1);
 if ($11) {
  while(1) {
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP8[$12>>0]|0;
   $14 = ($13<<24>>24)!=(0);
   if (!($14)) {
    break;
   }
   $15 = HEAP32[$1>>2]|0;
   HEAP32[$zStart>>2] = $15;
   while(1) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 1|0);
    HEAP32[$1>>2] = $17;
    $18 = HEAP8[$16>>0]|0;
    $19 = $18 << 24 >> 24;
    $20 = ($19|0)!=(10);
    if (!($20)) {
     break;
    }
    $21 = HEAP32[$1>>2]|0;
    $22 = HEAP8[$21>>0]|0;
    $23 = $22 << 24 >> 24;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     break;
    }
   }
   _sqlite3StrAccumAppend($out,23899,3);
   $25 = HEAP32[$zStart>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$zStart>>2]|0;
   $28 = $26;
   $29 = $27;
   $30 = (($28) - ($29))|0;
   _sqlite3StrAccumAppend($out,$25,$30);
  }
  $159 = (_sqlite3StrAccumFinish($out)|0);
  STACKTOP = sp;return ($159|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 68|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33 << 16 >> 16;
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$1>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = (_sqlite3Strlen30($37)|0);
  _sqlite3StrAccumAppend($out,$36,$38);
  $159 = (_sqlite3StrAccumFinish($out)|0);
  STACKTOP = sp;return ($159|0);
 }
 while(1) {
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP8[$39>>0]|0;
  $41 = ($40<<24>>24)!=(0);
  if (!($41)) {
   label = 33;
   break;
  }
  $42 = HEAP32[$1>>2]|0;
  $43 = (_findNextHostParameter($42,$nToken)|0);
  HEAP32[$n>>2] = $43;
  $44 = HEAP32[$1>>2]|0;
  $45 = HEAP32[$n>>2]|0;
  _sqlite3StrAccumAppend($out,$44,$45);
  $46 = HEAP32[$n>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = (($47) + ($46)|0);
  HEAP32[$1>>2] = $48;
  $49 = HEAP32[$nToken>>2]|0;
  $50 = ($49|0)==(0);
  if ($50) {
   label = 33;
   break;
  }
  $51 = HEAP32[$1>>2]|0;
  $52 = HEAP8[$51>>0]|0;
  $53 = $52 << 24 >> 24;
  $54 = ($53|0)==(63);
  do {
   if ($54) {
    $55 = HEAP32[$nToken>>2]|0;
    $56 = ($55|0)>(1);
    if ($56) {
     $57 = HEAP32[$1>>2]|0;
     $58 = ((($57)) + 1|0);
     (_sqlite3GetInt32($58,$idx)|0);
     break;
    } else {
     $59 = HEAP32[$nextIndex>>2]|0;
     HEAP32[$idx>>2] = $59;
     break;
    }
   } else {
    $60 = HEAP32[$0>>2]|0;
    $61 = HEAP32[$1>>2]|0;
    $62 = HEAP32[$nToken>>2]|0;
    $63 = (_sqlite3VdbeParameterIndex($60,$61,$62)|0);
    HEAP32[$idx>>2] = $63;
   }
  } while(0);
  $64 = HEAP32[$nToken>>2]|0;
  $65 = HEAP32[$1>>2]|0;
  $66 = (($65) + ($64)|0);
  HEAP32[$1>>2] = $66;
  $67 = HEAP32[$idx>>2]|0;
  $68 = (($67) + 1)|0;
  HEAP32[$nextIndex>>2] = $68;
  $69 = HEAP32[$idx>>2]|0;
  $70 = (($69) - 1)|0;
  $71 = HEAP32[$0>>2]|0;
  $72 = ((($71)) + 60|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + (($70*40)|0)|0);
  HEAP32[$pVar>>2] = $74;
  $75 = HEAP32[$pVar>>2]|0;
  $76 = ((($75)) + 8|0);
  $77 = HEAP16[$76>>1]|0;
  $78 = $77&65535;
  $79 = $78 & 1;
  $80 = ($79|0)!=(0);
  if ($80) {
   _sqlite3StrAccumAppend($out,18200,4);
   continue;
  }
  $81 = HEAP32[$pVar>>2]|0;
  $82 = ((($81)) + 8|0);
  $83 = HEAP16[$82>>1]|0;
  $84 = $83&65535;
  $85 = $84 & 4;
  $86 = ($85|0)!=(0);
  $87 = HEAP32[$pVar>>2]|0;
  if ($86) {
   $88 = $87;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (($88) + 4)|0;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = $vararg_buffer;
   $95 = $94;
   HEAP32[$95>>2] = $90;
   $96 = (($94) + 4)|0;
   $97 = $96;
   HEAP32[$97>>2] = $93;
   _sqlite3XPrintf($out,19285,$vararg_buffer);
   continue;
  }
  $98 = ((($87)) + 8|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $99&65535;
  $101 = $100 & 8;
  $102 = ($101|0)!=(0);
  $103 = HEAP32[$pVar>>2]|0;
  if ($102) {
   $104 = +HEAPF64[$103>>3];
   HEAPF64[$vararg_buffer1>>3] = $104;
   _sqlite3XPrintf($out,19290,$vararg_buffer1);
   continue;
  }
  $105 = ((($103)) + 8|0);
  $106 = HEAP16[$105>>1]|0;
  $107 = $106&65535;
  $108 = $107 & 2;
  $109 = ($108|0)!=(0);
  if ($109) {
   $110 = HEAP32[$db>>2]|0;
   $111 = ((($110)) + 62|0);
   $112 = HEAP8[$111>>0]|0;
   HEAP8[$enc>>0] = $112;
   $113 = HEAP8[$enc>>0]|0;
   $114 = $113&255;
   $115 = ($114|0)!=(1);
   if ($115) {
    dest=$utf8; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
    $116 = HEAP32[$db>>2]|0;
    $117 = ((($utf8)) + 32|0);
    HEAP32[$117>>2] = $116;
    $118 = HEAP32[$pVar>>2]|0;
    $119 = ((($118)) + 16|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = HEAP32[$pVar>>2]|0;
    $122 = ((($121)) + 12|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = HEAP8[$enc>>0]|0;
    (_sqlite3VdbeMemSetStr($utf8,$120,$123,$124,0)|0);
    (_sqlite3VdbeChangeEncoding($utf8,1)|0);
    HEAP32[$pVar>>2] = $utf8;
   }
   $125 = HEAP32[$pVar>>2]|0;
   $126 = ((($125)) + 12|0);
   $127 = HEAP32[$126>>2]|0;
   HEAP32[$nOut>>2] = $127;
   $128 = HEAP32[$nOut>>2]|0;
   $129 = HEAP32[$pVar>>2]|0;
   $130 = ((($129)) + 16|0);
   $131 = HEAP32[$130>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $128;
   $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $131;
   _sqlite3XPrintf($out,23903,$vararg_buffer4);
   $132 = HEAP8[$enc>>0]|0;
   $133 = $132&255;
   $134 = ($133|0)!=(1);
   if (!($134)) {
    continue;
   }
   _sqlite3VdbeMemRelease($utf8);
   continue;
  }
  $135 = HEAP32[$pVar>>2]|0;
  $136 = ((($135)) + 8|0);
  $137 = HEAP16[$136>>1]|0;
  $138 = $137&65535;
  $139 = $138 & 16384;
  $140 = ($139|0)!=(0);
  if ($140) {
   $141 = HEAP32[$pVar>>2]|0;
   $142 = HEAP32[$141>>2]|0;
   HEAP32[$vararg_buffer8>>2] = $142;
   _sqlite3XPrintf($out,23910,$vararg_buffer8);
   continue;
  }
  _sqlite3StrAccumAppend($out,23923,2);
  $143 = HEAP32[$pVar>>2]|0;
  $144 = ((($143)) + 12|0);
  $145 = HEAP32[$144>>2]|0;
  HEAP32[$nOut1>>2] = $145;
  HEAP32[$i>>2] = 0;
  while(1) {
   $146 = HEAP32[$i>>2]|0;
   $147 = HEAP32[$nOut1>>2]|0;
   $148 = ($146|0)<($147|0);
   if (!($148)) {
    break;
   }
   $149 = HEAP32[$i>>2]|0;
   $150 = HEAP32[$pVar>>2]|0;
   $151 = ((($150)) + 16|0);
   $152 = HEAP32[$151>>2]|0;
   $153 = (($152) + ($149)|0);
   $154 = HEAP8[$153>>0]|0;
   $155 = $154 << 24 >> 24;
   $156 = $155 & 255;
   HEAP32[$vararg_buffer11>>2] = $156;
   _sqlite3XPrintf($out,23926,$vararg_buffer11);
   $157 = HEAP32[$i>>2]|0;
   $158 = (($157) + 1)|0;
   HEAP32[$i>>2] = $158;
  }
  _sqlite3StrAccumAppend($out,23931,1);
 }
 if ((label|0) == 33) {
  $159 = (_sqlite3StrAccumFinish($out)|0);
  STACKTOP = sp;return ($159|0);
 }
 return (0)|0;
}
function _out2PrereleaseWithClear($pOut) {
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pOut;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 HEAP16[$3>>1] = 4;
 $4 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _vdbeClrCopy($pTo,$pFrom,$eType) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 $eType = $eType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $eType;
 $3 = HEAP32[$0>>2]|0;
 _vdbeMemClearExternAndSetNull($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemShallowCopy($4,$5,$6);
 STACKTOP = sp;return;
}
function _computeNumericType($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 10|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = (_sqlite3AtoF($4,$5,$8,$11)|0);
 $13 = ($12|0)==(0);
 if ($13) {
  HEAP16[$0>>1] = 0;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 10|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = (_sqlite3Atoi64($16,$17,$20,$23)|0);
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP16[$0>>1] = 4;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 } else {
  HEAP16[$0>>1] = 8;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 }
 return (0)|0;
}
function _sqlite3ValueApplyAffinity($pVal,$affinity,$enc) {
 $pVal = $pVal|0;
 $affinity = $affinity|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pVal;
 HEAP8[$1>>0] = $affinity;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$1>>0]|0;
 $5 = HEAP8[$2>>0]|0;
 _applyAffinity($3,$4,$5);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemNumerify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 13;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = ((($7)) + 16|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$0>>2]|0;
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 10|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = (_sqlite3Atoi64($9,$10,$13,$16)|0);
   $18 = (0)==($17|0);
   $19 = HEAP32[$0>>2]|0;
   if ($18) {
    $20 = ((($19)) + 8|0);
    $21 = HEAP16[$20>>1]|0;
    $22 = $21&65535;
    $23 = $22 & -49664;
    $24 = $23 | 4;
    $25 = $24&65535;
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 8|0);
    HEAP16[$27>>1] = $25;
    break;
   } else {
    $28 = (+_sqlite3VdbeRealValue($19));
    $29 = HEAP32[$0>>2]|0;
    HEAPF64[$29>>3] = $28;
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 8|0);
    $32 = HEAP16[$31>>1]|0;
    $33 = $32&65535;
    $34 = $33 & -49664;
    $35 = $34 | 8;
    $36 = $35&65535;
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 8|0);
    HEAP16[$38>>1] = $36;
    $39 = HEAP32[$0>>2]|0;
    _sqlite3VdbeIntegerAffinity($39);
    break;
   }
  }
 } while(0);
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 8|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = $43 & -19;
 $45 = $44&65535;
 HEAP16[$41>>1] = $45;
 STACKTOP = sp;return 0;
}
function _sqlite3IntFloatCompare($0,$1,$r) {
 $0 = $0|0;
 $1 = $1|0;
 $r = +$r;
 var $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0.0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0, $8 = 0, $9 = 0.0, $or$cond = 0, $s = 0, $y = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 24|0;
 $4 = sp + 16|0;
 $y = sp + 8|0;
 $s = sp;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAPF64[$4>>3] = $r;
 $9 = +HEAPF64[$4>>3];
 $10 = $9 < -9.2233720368547758E+18;
 if ($10) {
  HEAP32[$2>>2] = 1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = +HEAPF64[$4>>3];
 $12 = $11 > 9.2233720368547758E+18;
 if ($12) {
  HEAP32[$2>>2] = -1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $13 = +HEAPF64[$4>>3];
 $14 = (~~$13)>>>0;
 $15 = +Math_abs($13) >= 1.0 ? $13 > 0.0 ? (~~+Math_min(+Math_floor($13 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($13 - +(~~$13 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $16 = $y;
 $17 = $16;
 HEAP32[$17>>2] = $14;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $3;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = $y;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($25|0)<($31|0);
 $33 = ($22>>>0)<($28>>>0);
 $34 = ($25|0)==($31|0);
 $35 = $34 & $33;
 $36 = $32 | $35;
 if ($36) {
  HEAP32[$2>>2] = -1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $37 = $3;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = $y;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 $46 = (($43) + 4)|0;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = ($42|0)>($48|0);
 $50 = ($39>>>0)>($45>>>0);
 $51 = ($42|0)==($48|0);
 $52 = $51 & $50;
 $53 = $49 | $52;
 if ($53) {
  $54 = $y;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = ($56|0)==(0);
  $61 = ($59|0)==(-2147483648);
  $62 = $60 & $61;
  $63 = +HEAPF64[$4>>3];
  $64 = $63 > 0.0;
  $or$cond = $62 & $64;
  if ($or$cond) {
   HEAP32[$2>>2] = -1;
   $78 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($78|0);
  } else {
   HEAP32[$2>>2] = 1;
   $78 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
 }
 $65 = $3;
 $66 = $65;
 $67 = HEAP32[$66>>2]|0;
 $68 = (($65) + 4)|0;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (+($67>>>0)) + (4294967296.0*(+($70|0)));
 HEAPF64[$s>>3] = $71;
 $72 = +HEAPF64[$s>>3];
 $73 = +HEAPF64[$4>>3];
 $74 = $72 < $73;
 if ($74) {
  HEAP32[$2>>2] = -1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $75 = +HEAPF64[$s>>3];
 $76 = +HEAPF64[$4>>3];
 $77 = $75 > $76;
 if ($77) {
  HEAP32[$2>>2] = 1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 } else {
  HEAP32[$2>>2] = 0;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 return (0)|0;
}
function _vdbeCompareMemString($pMem1,$pMem2,$pColl,$prcErr) {
 $pMem1 = $pMem1|0;
 $pMem2 = $pMem2|0;
 $pColl = $pColl|0;
 $prcErr = $prcErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $c1 = 0;
 var $c2 = 0, $n1 = 0, $n2 = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $rc = sp + 96|0;
 $v1 = sp + 92|0;
 $v2 = sp + 88|0;
 $n1 = sp + 84|0;
 $n2 = sp + 80|0;
 $c1 = sp + 40|0;
 $c2 = sp;
 HEAP32[$1>>2] = $pMem1;
 HEAP32[$2>>2] = $pMem2;
 HEAP32[$3>>2] = $pColl;
 HEAP32[$4>>2] = $prcErr;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 10|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($8|0)==($12|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (FUNCTION_TABLE_iiiiii[$16 & 255]($19,$22,$25,$28,$31)|0);
  HEAP32[$0>>2] = $32;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 32|0);
 $35 = HEAP32[$34>>2]|0;
 _sqlite3VdbeMemInit($c1,$35,1);
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 32|0);
 $38 = HEAP32[$37>>2]|0;
 _sqlite3VdbeMemInit($c2,$38,1);
 $39 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemShallowCopy($c1,$39,4096);
 $40 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemShallowCopy($c2,$40,4096);
 $41 = HEAP32[$3>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = (_sqlite3ValueText($c1,$43)|0);
 HEAP32[$v1>>2] = $44;
 $45 = HEAP32[$v1>>2]|0;
 $46 = ($45|0)==(0|0);
 $47 = ((($c1)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = $46 ? 0 : $48;
 HEAP32[$n1>>2] = $49;
 $50 = HEAP32[$3>>2]|0;
 $51 = ((($50)) + 4|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = (_sqlite3ValueText($c2,$52)|0);
 HEAP32[$v2>>2] = $53;
 $54 = HEAP32[$v2>>2]|0;
 $55 = ($54|0)==(0|0);
 $56 = ((($c2)) + 12|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $55 ? 0 : $57;
 HEAP32[$n2>>2] = $58;
 $59 = HEAP32[$3>>2]|0;
 $60 = ((($59)) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$3>>2]|0;
 $63 = ((($62)) + 8|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$n1>>2]|0;
 $66 = HEAP32[$v1>>2]|0;
 $67 = HEAP32[$n2>>2]|0;
 $68 = HEAP32[$v2>>2]|0;
 $69 = (FUNCTION_TABLE_iiiiii[$61 & 255]($64,$65,$66,$67,$68)|0);
 HEAP32[$rc>>2] = $69;
 $70 = HEAP32[$v1>>2]|0;
 $71 = ($70|0)==(0|0);
 $72 = HEAP32[$v2>>2]|0;
 $73 = ($72|0)==(0|0);
 $or$cond = $71 | $73;
 $74 = HEAP32[$4>>2]|0;
 $75 = ($74|0)!=(0|0);
 $or$cond3 = $or$cond & $75;
 if ($or$cond3) {
  $76 = HEAP32[$4>>2]|0;
  HEAP8[$76>>0] = 7;
 }
 _sqlite3VdbeMemRelease($c1);
 _sqlite3VdbeMemRelease($c2);
 $77 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3BlobCompare($pB1,$pB2) {
 $pB1 = $pB1|0;
 $pB2 = $pB2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $c = sp;
 HEAP32[$1>>2] = $pB1;
 HEAP32[$2>>2] = $pB2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)>($14|0);
 if ($15) {
  $16 = HEAP32[$2>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $22 = $18;
 } else {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21;
 }
 $23 = (_memcmp($5,$8,$22)|0);
 HEAP32[$c>>2] = $23;
 $24 = HEAP32[$c>>2]|0;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP32[$c>>2]|0;
  HEAP32[$0>>2] = $26;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($29) - ($32))|0;
  HEAP32[$0>>2] = $33;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _handleDeferredMoveto($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $res = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3BtreeMovetoUnpacked($4,0,$9,$12,0,$res)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $17 = HEAP32[$res>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = (_sqlite3CorruptError(70762)|0);
  HEAP32[$0>>2] = $19;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 3|0);
  HEAP8[$21>>0] = 0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 56|0);
  HEAP32[$23>>2] = 0;
  HEAP32[$0>>2] = 0;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCursorHasMoved($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 66|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(1);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _handleMovedCursor($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $isDifferentRow = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $isDifferentRow = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3BtreeCursorRestore($3,$isDifferentRow)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 56|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$isDifferentRow>>2]|0;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 2|0);
 HEAP8[$10>>0] = 1;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3BtreeCursorRestore($pCur,$pDifferentRow) {
 $pCur = $pCur|0;
 $pDifferentRow = $pDifferentRow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pDifferentRow;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)>=(3);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_btreeRestoreCursorPosition($8)|0);
  $10 = $9;
 } else {
  $10 = 0;
 }
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  HEAP32[$13>>2] = 1;
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 66|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)!=(1);
 $20 = HEAP32[$2>>2]|0;
 if ($19) {
  HEAP32[$20>>2] = 1;
 } else {
  HEAP32[$20>>2] = 0;
 }
 HEAP32[$0>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _btreeRestoreCursorPosition($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $skipNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $skipNext = sp;
 HEAP32[$1>>2] = $pCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 66|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(4);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 60|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$0>>2] = $9;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $10 = ((($7)) + 66|0);
 HEAP8[$10>>0] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 40|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (_btreeMoveto($11,$14,$19,$22,0,$skipNext)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  _sqlite3_free($28);
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 48|0);
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$skipNext>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 60|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $34 | $31;
  HEAP32[$33>>2] = $35;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 60|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 66|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = $42&255;
   $44 = ($43|0)==(1);
   if ($44) {
    $45 = HEAP32[$1>>2]|0;
    $46 = ((($45)) + 66|0);
    HEAP8[$46>>0] = 2;
   }
  }
 }
 $47 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $47;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _btreeMoveto($pCur,$pKey,$0,$1,$bias,$pRes) {
 $pCur = $pCur|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $bias = $bias|0;
 $pRes = $pRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aSpace = 0, $pFree = 0, $pIdxKey = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $rc = sp + 16|0;
 $pIdxKey = sp + 12|0;
 $aSpace = sp + 40|0;
 $pFree = sp + 8|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pKey;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $bias;
 HEAP32[$7>>2] = $pRes;
 HEAP32[$pFree>>2] = 0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 72|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3VdbeAllocUnpackedRecord($16,$aSpace,200,$pFree)|0);
  HEAP32[$pIdxKey>>2] = $17;
  $18 = HEAP32[$pIdxKey>>2]|0;
  $19 = ($18|0)==(0|0);
  if ($19) {
   HEAP32[$2>>2] = 7;
   $63 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
  $20 = HEAP32[$3>>2]|0;
  $21 = ((($20)) + 72|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $5;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = HEAP32[$pIdxKey>>2]|0;
  _sqlite3VdbeRecordUnpack($22,$25,$29,$30);
  $31 = HEAP32[$pIdxKey>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = ($34|0)==(0);
  if ($35) {
   $36 = HEAP32[$3>>2]|0;
   $37 = ((($36)) + 72|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ((($38)) + 12|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$pFree>>2]|0;
   _sqlite3DbFree($40,$41);
   $42 = (_sqlite3CorruptError(56338)|0);
   HEAP32[$2>>2] = $42;
   $63 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } else {
  HEAP32[$pIdxKey>>2] = 0;
 }
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$pIdxKey>>2]|0;
 $45 = $5;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$6>>2]|0;
 $52 = HEAP32[$7>>2]|0;
 $53 = (_sqlite3BtreeMovetoUnpacked($43,$44,$47,$50,$51,$52)|0);
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$pFree>>2]|0;
 $55 = ($54|0)!=(0|0);
 if ($55) {
  $56 = HEAP32[$3>>2]|0;
  $57 = ((($56)) + 72|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ((($58)) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$pFree>>2]|0;
  _sqlite3DbFree($60,$61);
 }
 $62 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $62;
 $63 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _fetchPayload($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $amt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $amt = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 68|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 120|0);
 $8 = (($7) + ($5<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 60|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $11;
 $17 = $15;
 $18 = (($16) - ($17))|0;
 HEAP32[$amt>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 16|0);
 $21 = ((($20)) + 16|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = HEAP32[$amt>>2]|0;
 $25 = ($23>>>0)<($24>>>0);
 if ($25) {
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 16|0);
  $28 = ((($27)) + 16|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  HEAP32[$amt>>2] = $30;
 }
 $31 = HEAP32[$amt>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _vdbeMemFromBtreeResize($pCur,$offset,$amt,$key,$pMem) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $key = $key|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $key;
 HEAP32[$4>>2] = $pMem;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 2)|0;
 $10 = (_sqlite3VdbeMemClearAndResize($7,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = (0)==($10|0);
 if (!($11)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 if ($13) {
  $20 = (_sqlite3BtreeKey($14,$15,$16,$19)|0);
  HEAP32[$rc>>2] = $20;
 } else {
  $21 = (_sqlite3BtreeData($14,$15,$16,$19)|0);
  HEAP32[$rc>>2] = $21;
 }
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = ((($25)) + 16|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + ($24)|0);
  HEAP8[$28>>0] = 0;
  $29 = HEAP32[$2>>2]|0;
  $30 = (($29) + 1)|0;
  $31 = HEAP32[$4>>2]|0;
  $32 = ((($31)) + 16|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30)|0);
  HEAP8[$34>>0] = 0;
  $35 = HEAP32[$4>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP16[$36>>1] = 528;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  $39 = ((($38)) + 12|0);
  HEAP32[$39>>2] = $37;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $40 = HEAP32[$4>>2]|0;
  _sqlite3VdbeMemRelease($40);
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _serialGet($buf,$serial_type,$pMem) {
 $buf = $buf|0;
 $serial_type = $serial_type|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $x = sp;
 $y = sp + 8|0;
 HEAP32[$0>>2] = $buf;
 HEAP32[$1>>2] = $serial_type;
 HEAP32[$2>>2] = $pMem;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 << 24;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 1|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10 << 16;
 $12 = $6 | $11;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 2|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = $16 << 8;
 $18 = $12 | $17;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 3|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $18 | $22;
 $24 = $x;
 $25 = $24;
 HEAP32[$25>>2] = $23;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 4|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $31 << 24;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 4|0);
 $35 = ((($34)) + 1|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $37 << 16;
 $39 = $32 | $38;
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 4|0);
 $42 = ((($41)) + 2|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43&255;
 $45 = $44 << 8;
 $46 = $39 | $45;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 4|0);
 $49 = ((($48)) + 3|0);
 $50 = HEAP8[$49>>0]|0;
 $51 = $50&255;
 $52 = $46 | $51;
 HEAP32[$y>>2] = $52;
 $53 = $x;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($53) + 4)|0;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$y>>2]|0;
 $60 = (_i64Add(0,($55|0),($59|0),0)|0);
 $61 = tempRet0;
 $62 = $x;
 $63 = $62;
 HEAP32[$63>>2] = $60;
 $64 = (($62) + 4)|0;
 $65 = $64;
 HEAP32[$65>>2] = $61;
 $66 = HEAP32[$1>>2]|0;
 $67 = ($66|0)==(6);
 if ($67) {
  $68 = $x;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  $75 = $74;
  $76 = $75;
  HEAP32[$76>>2] = $70;
  $77 = (($75) + 4)|0;
  $78 = $77;
  HEAP32[$78>>2] = $73;
  $79 = HEAP32[$2>>2]|0;
  $80 = ((($79)) + 8|0);
  HEAP16[$80>>1] = 4;
  STACKTOP = sp;return 8;
 } else {
  $81 = HEAP32[$2>>2]|0;
  ;HEAP32[$81>>2]=HEAP32[$x>>2]|0;HEAP32[$81+4>>2]=HEAP32[$x+4>>2]|0;
  $82 = HEAP32[$2>>2]|0;
  $83 = +HEAPF64[$82>>3];
  $84 = (_sqlite3IsNaN($83)|0);
  $85 = ($84|0)!=(0);
  $86 = $85 ? 1 : 8;
  $87 = $86&65535;
  $88 = HEAP32[$2>>2]|0;
  $89 = ((($88)) + 8|0);
  HEAP16[$89>>1] = $87;
  STACKTOP = sp;return 8;
 }
 return (0)|0;
}
function _putVarint64($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0;
 var $i = 0, $j = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp;
 $i = sp + 16|0;
 $j = sp + 12|0;
 $n = sp + 8|0;
 $buf = sp + 28|0;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 & -16777216;
 $16 = (0)!=(0);
 $17 = ($15|0)!=(0);
 $18 = $16 | $17;
 if ($18) {
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = $21&255;
  $26 = HEAP32[$3>>2]|0;
  $27 = ((($26)) + 8|0);
  HEAP8[$27>>0] = $25;
  $28 = $4;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($28) + 4)|0;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (_bitshift64Lshr(($30|0),($33|0),8)|0);
  $35 = tempRet0;
  $36 = $4;
  $37 = $36;
  HEAP32[$37>>2] = $34;
  $38 = (($36) + 4)|0;
  $39 = $38;
  HEAP32[$39>>2] = $35;
  HEAP32[$i>>2] = 7;
  while(1) {
   $40 = HEAP32[$i>>2]|0;
   $41 = ($40|0)>=(0);
   if (!($41)) {
    break;
   }
   $42 = $4;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = $44 & 127;
   $49 = $48 | 128;
   $50 = $49&255;
   $51 = HEAP32[$i>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = (($52) + ($51)|0);
   HEAP8[$53>>0] = $50;
   $54 = $4;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($54) + 4)|0;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (_bitshift64Lshr(($56|0),($59|0),7)|0);
   $61 = tempRet0;
   $62 = $4;
   $63 = $62;
   HEAP32[$63>>2] = $60;
   $64 = (($62) + 4)|0;
   $65 = $64;
   HEAP32[$65>>2] = $61;
   $66 = HEAP32[$i>>2]|0;
   $67 = (($66) + -1)|0;
   HEAP32[$i>>2] = $67;
  }
  HEAP32[$2>>2] = 9;
  $120 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 HEAP32[$n>>2] = 0;
 while(1) {
  $68 = $4;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = $70 & 127;
  $75 = $74 | 128;
  $76 = $75&255;
  $77 = HEAP32[$n>>2]|0;
  $78 = (($77) + 1)|0;
  HEAP32[$n>>2] = $78;
  $79 = (($buf) + ($77)|0);
  HEAP8[$79>>0] = $76;
  $80 = $4;
  $81 = $80;
  $82 = HEAP32[$81>>2]|0;
  $83 = (($80) + 4)|0;
  $84 = $83;
  $85 = HEAP32[$84>>2]|0;
  $86 = (_bitshift64Lshr(($82|0),($85|0),7)|0);
  $87 = tempRet0;
  $88 = $4;
  $89 = $88;
  HEAP32[$89>>2] = $86;
  $90 = (($88) + 4)|0;
  $91 = $90;
  HEAP32[$91>>2] = $87;
  $92 = $4;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($94|0)!=(0);
  $99 = ($97|0)!=(0);
  $100 = $98 | $99;
  if (!($100)) {
   break;
  }
 }
 $101 = HEAP8[$buf>>0]|0;
 $102 = $101&255;
 $103 = $102 & 127;
 $104 = $103&255;
 HEAP8[$buf>>0] = $104;
 HEAP32[$i>>2] = 0;
 $105 = HEAP32[$n>>2]|0;
 $106 = (($105) - 1)|0;
 HEAP32[$j>>2] = $106;
 while(1) {
  $107 = HEAP32[$j>>2]|0;
  $108 = ($107|0)>=(0);
  if (!($108)) {
   break;
  }
  $109 = HEAP32[$j>>2]|0;
  $110 = (($buf) + ($109)|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$3>>2]|0;
  $114 = (($113) + ($112)|0);
  HEAP8[$114>>0] = $111;
  $115 = HEAP32[$j>>2]|0;
  $116 = (($115) + -1)|0;
  HEAP32[$j>>2] = $116;
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + 1)|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = HEAP32[$n>>2]|0;
 HEAP32[$2>>2] = $119;
 $120 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($120|0);
}
function _moveToRoot($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pRoot = 0, $rc = 0, $subpage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRoot = sp + 8|0;
 $rc = sp + 4|0;
 $subpage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 66|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)>=(3);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 66|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9&255;
   $11 = ($10|0)==(4);
   $12 = HEAP32[$1>>2]|0;
   if (!($11)) {
    _sqlite3BtreeClearCursor($12);
    break;
   }
   $13 = ((($12)) + 60|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$0>>2] = $14;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 68|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17 << 24 >> 24;
 $19 = ($18|0)>=(0);
 L8: do {
  if ($19) {
   while(1) {
    $20 = HEAP32[$1>>2]|0;
    $21 = ((($20)) + 68|0);
    $22 = HEAP8[$21>>0]|0;
    $23 = ($22<<24>>24)!=(0);
    if (!($23)) {
     break L8;
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 68|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = (($26) + -1)<<24>>24;
    HEAP8[$25>>0] = $27;
    $28 = $26 << 24 >> 24;
    $29 = HEAP32[$1>>2]|0;
    $30 = ((($29)) + 120|0);
    $31 = (($30) + ($28<<2)|0);
    $32 = HEAP32[$31>>2]|0;
    _releasePageNotNull($32);
   }
  } else {
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 52|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(0);
   $37 = HEAP32[$1>>2]|0;
   if ($36) {
    $38 = ((($37)) + 66|0);
    HEAP8[$38>>0] = 0;
    HEAP32[$0>>2] = 0;
    $128 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($128|0);
   }
   $39 = HEAP32[$37>>2]|0;
   $40 = ((($39)) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 52|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 120|0);
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 65|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = (_getAndInitPage($41,$44,$46,0,$50)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)!=(0);
   $54 = HEAP32[$1>>2]|0;
   if (!($53)) {
    $57 = ((($54)) + 68|0);
    HEAP8[$57>>0] = 0;
    $58 = HEAP32[$1>>2]|0;
    $59 = ((($58)) + 120|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = ((($60)) + 2|0);
    $62 = HEAP8[$61>>0]|0;
    $63 = HEAP32[$1>>2]|0;
    $64 = ((($63)) + 69|0);
    HEAP8[$64>>0] = $62;
    break;
   }
   $55 = ((($54)) + 66|0);
   HEAP8[$55>>0] = 0;
   $56 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $56;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 120|0);
 $67 = HEAP32[$66>>2]|0;
 HEAP32[$pRoot>>2] = $67;
 $68 = HEAP32[$pRoot>>2]|0;
 $69 = HEAP8[$68>>0]|0;
 $70 = $69&255;
 $71 = ($70|0)==(0);
 if (!($71)) {
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 72|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = ($74|0)==(0|0);
  $76 = $75&1;
  $77 = HEAP32[$pRoot>>2]|0;
  $78 = ((($77)) + 2|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = ($76|0)!=($80|0);
  if (!($81)) {
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 80|0);
   HEAP16[$84>>1] = 0;
   $85 = HEAP32[$1>>2]|0;
   $86 = ((($85)) + 16|0);
   $87 = ((($86)) + 18|0);
   HEAP16[$87>>1] = 0;
   $88 = HEAP32[$1>>2]|0;
   $89 = ((($88)) + 64|0);
   $90 = HEAP8[$89>>0]|0;
   $91 = $90&255;
   $92 = $91 & -15;
   $93 = $92&255;
   HEAP8[$89>>0] = $93;
   $94 = HEAP32[$pRoot>>2]|0;
   $95 = ((($94)) + 18|0);
   $96 = HEAP16[$95>>1]|0;
   $97 = $96&65535;
   $98 = ($97|0)>(0);
   do {
    if ($98) {
     $99 = HEAP32[$1>>2]|0;
     $100 = ((($99)) + 66|0);
     HEAP8[$100>>0] = 1;
    } else {
     $101 = HEAP32[$pRoot>>2]|0;
     $102 = ((($101)) + 4|0);
     $103 = HEAP8[$102>>0]|0;
     $104 = ($103<<24>>24)!=(0);
     if ($104) {
      $125 = HEAP32[$1>>2]|0;
      $126 = ((($125)) + 66|0);
      HEAP8[$126>>0] = 0;
      break;
     }
     $105 = HEAP32[$pRoot>>2]|0;
     $106 = ((($105)) + 84|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = ($107|0)!=(1);
     if (!($108)) {
      $110 = HEAP32[$pRoot>>2]|0;
      $111 = ((($110)) + 5|0);
      $112 = HEAP8[$111>>0]|0;
      $113 = $112&255;
      $114 = (($113) + 8)|0;
      $115 = HEAP32[$pRoot>>2]|0;
      $116 = ((($115)) + 56|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = (($117) + ($114)|0);
      $119 = (_sqlite3Get4byte($118)|0);
      HEAP32[$subpage>>2] = $119;
      $120 = HEAP32[$1>>2]|0;
      $121 = ((($120)) + 66|0);
      HEAP8[$121>>0] = 1;
      $122 = HEAP32[$1>>2]|0;
      $123 = HEAP32[$subpage>>2]|0;
      $124 = (_moveToChild($122,$123)|0);
      HEAP32[$rc>>2] = $124;
      break;
     }
     $109 = (_sqlite3CorruptError(60470)|0);
     HEAP32[$0>>2] = $109;
     $128 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($128|0);
    }
   } while(0);
   $127 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $127;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 }
 $82 = (_sqlite3CorruptError(60459)|0);
 HEAP32[$0>>2] = $82;
 $128 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($128|0);
}
function _moveToParent($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = ((($2)) + 18|0);
 HEAP16[$3>>1] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 64|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & -7;
 $9 = $8&255;
 HEAP8[$5>>0] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 68|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = (($12) + -1)<<24>>24;
 HEAP8[$11>>0] = $13;
 $14 = $12 << 24 >> 24;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 120|0);
 $17 = (($16) + ($14<<2)|0);
 $18 = HEAP32[$17>>2]|0;
 _releasePageNotNull($18);
 STACKTOP = sp;return;
}
function _moveToChild($pCur,$newPgno) {
 $pCur = $pCur|0;
 $newPgno = $newPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $newPgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 68|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)>=(19);
 if ($10) {
  $11 = (_sqlite3CorruptError(60332)|0);
  HEAP32[$0>>2] = $11;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 } else {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = ((($13)) + 18|0);
  HEAP16[$14>>1] = 0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 64|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = $18 & -7;
  $20 = $19&255;
  HEAP8[$16>>0] = $20;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 68|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = (($23) + 1)<<24>>24;
  HEAP8[$22>>0] = $24;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 68|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27 << 24 >> 24;
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 80|0);
  $31 = (($30) + ($28<<1)|0);
  HEAP16[$31>>1] = 0;
  $32 = HEAP32[$pBt>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 68|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36 << 24 >> 24;
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 120|0);
  $40 = (($39) + ($37<<2)|0);
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$1>>2]|0;
  $43 = ((($42)) + 65|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = (_getAndInitPage($32,$33,$40,$41,$45)|0);
  HEAP32[$0>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 return (0)|0;
}
function _getAndInitPage($pBt,$pgno,$ppPage,$pCur,$bReadOnly) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $pCur = $pCur|0;
 $bReadOnly = $bReadOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $rc = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $pCur;
 HEAP32[$5>>2] = $bReadOnly;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_btreePagecount($7)|0);
 $9 = ($6>>>0)>($8>>>0);
 L1: do {
  if ($9) {
   $10 = (_sqlite3CorruptError(57558)|0);
   HEAP32[$rc>>2] = $10;
  } else {
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$5>>2]|0;
   $15 = (_sqlite3PagerGet($12,$13,$pDbPage,$14)|0);
   HEAP32[$rc>>2] = $15;
   $16 = HEAP32[$rc>>2]|0;
   $17 = ($16|0)!=(0);
   if (!($17)) {
    $18 = HEAP32[$pDbPage>>2]|0;
    $19 = (_sqlite3PagerGetExtra($18)|0);
    $20 = HEAP32[$3>>2]|0;
    HEAP32[$20>>2] = $19;
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP8[$22>>0]|0;
    $24 = $23&255;
    $25 = ($24|0)==(0);
    if ($25) {
     $26 = HEAP32[$pDbPage>>2]|0;
     $27 = HEAP32[$2>>2]|0;
     $28 = HEAP32[$1>>2]|0;
     (_btreePageFromDbPage($26,$27,$28)|0);
     $29 = HEAP32[$3>>2]|0;
     $30 = HEAP32[$29>>2]|0;
     $31 = (_btreeInitPage($30)|0);
     HEAP32[$rc>>2] = $31;
     $32 = HEAP32[$rc>>2]|0;
     $33 = ($32|0)!=(0);
     if ($33) {
      $34 = HEAP32[$3>>2]|0;
      $35 = HEAP32[$34>>2]|0;
      _releasePage($35);
      break;
     }
    }
    $36 = HEAP32[$4>>2]|0;
    $37 = ($36|0)!=(0|0);
    do {
     if ($37) {
      $38 = HEAP32[$3>>2]|0;
      $39 = HEAP32[$38>>2]|0;
      $40 = ((($39)) + 18|0);
      $41 = HEAP16[$40>>1]|0;
      $42 = $41&65535;
      $43 = ($42|0)<(1);
      if (!($43)) {
       $44 = HEAP32[$3>>2]|0;
       $45 = HEAP32[$44>>2]|0;
       $46 = ((($45)) + 2|0);
       $47 = HEAP8[$46>>0]|0;
       $48 = $47&255;
       $49 = HEAP32[$4>>2]|0;
       $50 = ((($49)) + 69|0);
       $51 = HEAP8[$50>>0]|0;
       $52 = $51&255;
       $53 = ($48|0)!=($52|0);
       if (!($53)) {
        break;
       }
      }
      $54 = (_sqlite3CorruptError(57580)|0);
      HEAP32[$rc>>2] = $54;
      $55 = HEAP32[$3>>2]|0;
      $56 = HEAP32[$55>>2]|0;
      _releasePage($56);
      break L1;
     }
    } while(0);
    HEAP32[$0>>2] = 0;
    $64 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($64|0);
   }
  }
 } while(0);
 $57 = HEAP32[$4>>2]|0;
 $58 = ($57|0)!=(0|0);
 if ($58) {
  $59 = HEAP32[$4>>2]|0;
  $60 = ((($59)) + 68|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = (($61) + -1)<<24>>24;
  HEAP8[$60>>0] = $62;
 }
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3SchemaClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pElem = 0, $pSchema = 0, $pTab = 0, $temp1 = 0, $temp2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $temp1 = sp + 32|0;
 $temp2 = sp + 16|0;
 $pElem = sp + 8|0;
 $pSchema = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pSchema>>2] = $1;
 $2 = HEAP32[$pSchema>>2]|0;
 $3 = ((($2)) + 8|0);
 ;HEAP32[$temp1>>2]=HEAP32[$3>>2]|0;HEAP32[$temp1+4>>2]=HEAP32[$3+4>>2]|0;HEAP32[$temp1+8>>2]=HEAP32[$3+8>>2]|0;HEAP32[$temp1+12>>2]=HEAP32[$3+12>>2]|0;
 $4 = HEAP32[$pSchema>>2]|0;
 $5 = ((($4)) + 40|0);
 ;HEAP32[$temp2>>2]=HEAP32[$5>>2]|0;HEAP32[$temp2+4>>2]=HEAP32[$5+4>>2]|0;HEAP32[$temp2+8>>2]=HEAP32[$5+8>>2]|0;HEAP32[$temp2+12>>2]=HEAP32[$5+12>>2]|0;
 $6 = HEAP32[$pSchema>>2]|0;
 $7 = ((($6)) + 40|0);
 _sqlite3HashInit($7);
 $8 = HEAP32[$pSchema>>2]|0;
 $9 = ((($8)) + 24|0);
 _sqlite3HashClear($9);
 $10 = ((($temp2)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pElem>>2] = $11;
 while(1) {
  $12 = HEAP32[$pElem>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pElem>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  _sqlite3DeleteTrigger(0,$16);
  $17 = HEAP32[$pElem>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$pElem>>2] = $18;
 }
 _sqlite3HashClear($temp2);
 $19 = HEAP32[$pSchema>>2]|0;
 $20 = ((($19)) + 8|0);
 _sqlite3HashInit($20);
 $21 = ((($temp1)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pElem>>2] = $22;
 while(1) {
  $23 = HEAP32[$pElem>>2]|0;
  $24 = ($23|0)!=(0|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$pElem>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pTab>>2] = $27;
  $28 = HEAP32[$pTab>>2]|0;
  _sqlite3DeleteTable(0,$28);
  $29 = HEAP32[$pElem>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pElem>>2] = $30;
 }
 _sqlite3HashClear($temp1);
 $31 = HEAP32[$pSchema>>2]|0;
 $32 = ((($31)) + 56|0);
 _sqlite3HashClear($32);
 $33 = HEAP32[$pSchema>>2]|0;
 $34 = ((($33)) + 72|0);
 HEAP32[$34>>2] = 0;
 $35 = HEAP32[$pSchema>>2]|0;
 $36 = ((($35)) + 78|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37&65535;
 $39 = $38 & 1;
 $40 = ($39|0)!=(0);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pSchema>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 1)|0;
 HEAP32[$42>>2] = $44;
 $45 = HEAP32[$pSchema>>2]|0;
 $46 = ((($45)) + 78|0);
 $47 = HEAP16[$46>>1]|0;
 $48 = $47&65535;
 $49 = $48 & -2;
 $50 = $49&65535;
 HEAP16[$46>>1] = $50;
 STACKTOP = sp;return;
}
function _sqlite3HashInit($pNew) {
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pNew;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3HashClear($pH) {
 $pH = $pH|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $elem = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $elem = sp + 4|0;
 $next_elem = sp;
 HEAP32[$0>>2] = $pH;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$elem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 HEAP32[$11>>2] = 0;
 while(1) {
  $12 = HEAP32[$elem>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$elem>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$next_elem>>2] = $15;
  $16 = HEAP32[$elem>>2]|0;
  _sqlite3_free($16);
  $17 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 4|0);
 HEAP32[$19>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCursorSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 200;
}
function _sqlite3BtreeCursorZero($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 dest=$1; stop=dest+68|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _btreeCursor($p,$iTable,$wrFlag,$pKeyInfo,$pCur) {
 $p = $p|0;
 $iTable = $iTable|0;
 $wrFlag = $wrFlag|0;
 $pKeyInfo = $pKeyInfo|0;
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $pBt = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pBt = sp + 4|0;
 $pX = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $wrFlag;
 HEAP32[$4>>2] = $pKeyInfo;
 HEAP32[$5>>2] = $pCur;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$pBt>>2]|0;
  _allocateTempSpace($11);
  $12 = HEAP32[$pBt>>2]|0;
  $13 = ((($12)) + 80|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0|0);
  if ($15) {
   HEAP32[$0>>2] = 7;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = HEAP32[$pBt>>2]|0;
  $19 = (_btreePagecount($18)|0);
  $20 = ($19|0)==(0);
  if ($20) {
   HEAP32[$2>>2] = 0;
  }
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$5>>2]|0;
 $23 = ((($22)) + 52|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$5>>2]|0;
 $25 = ((($24)) + 68|0);
 HEAP8[$25>>0] = -1;
 $26 = HEAP32[$4>>2]|0;
 $27 = HEAP32[$5>>2]|0;
 $28 = ((($27)) + 72|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$5>>2]|0;
 HEAP32[$30>>2] = $29;
 $31 = HEAP32[$pBt>>2]|0;
 $32 = HEAP32[$5>>2]|0;
 $33 = ((($32)) + 4|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$3>>2]|0;
 $35 = ($34|0)!=(0);
 $36 = $35 ? 1 : 0;
 $37 = $36&255;
 $38 = HEAP32[$5>>2]|0;
 $39 = ((($38)) + 64|0);
 HEAP8[$39>>0] = $37;
 $40 = HEAP32[$3>>2]|0;
 $41 = ($40|0)!=(0);
 $42 = $41 ? 0 : 2;
 $43 = $42&255;
 $44 = HEAP32[$5>>2]|0;
 $45 = ((($44)) + 65|0);
 HEAP8[$45>>0] = $43;
 $46 = HEAP32[$pBt>>2]|0;
 $47 = ((($46)) + 8|0);
 $48 = HEAP32[$47>>2]|0;
 HEAP32[$pX>>2] = $48;
 while(1) {
  $49 = HEAP32[$pX>>2]|0;
  $50 = ($49|0)!=(0|0);
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$pX>>2]|0;
  $52 = ((($51)) + 52|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  $55 = ($53|0)==($54|0);
  if ($55) {
   $56 = HEAP32[$pX>>2]|0;
   $57 = ((($56)) + 64|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = $59 | 32;
   $61 = $60&255;
   HEAP8[$57>>0] = $61;
   $62 = HEAP32[$5>>2]|0;
   $63 = ((($62)) + 64|0);
   $64 = HEAP8[$63>>0]|0;
   $65 = $64&255;
   $66 = $65 | 32;
   $67 = $66&255;
   HEAP8[$63>>0] = $67;
  }
  $68 = HEAP32[$pX>>2]|0;
  $69 = ((($68)) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$pX>>2] = $70;
 }
 $71 = HEAP32[$pBt>>2]|0;
 $72 = ((($71)) + 8|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP32[$5>>2]|0;
 $75 = ((($74)) + 8|0);
 HEAP32[$75>>2] = $73;
 $76 = HEAP32[$5>>2]|0;
 $77 = HEAP32[$pBt>>2]|0;
 $78 = ((($77)) + 8|0);
 HEAP32[$78>>2] = $76;
 $79 = HEAP32[$5>>2]|0;
 $80 = ((($79)) + 66|0);
 HEAP8[$80>>0] = 0;
 HEAP32[$0>>2] = 0;
 $81 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _allocateTempSpace($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 32|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PageMalloc($7)|0);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 80|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 80|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 80|0);
 $17 = HEAP32[$16>>2]|0;
 ;HEAP8[$17>>0]=0|0;HEAP8[$17+1>>0]=0|0;HEAP8[$17+2>>0]=0|0;HEAP8[$17+3>>0]=0|0;HEAP8[$17+4>>0]=0|0;HEAP8[$17+5>>0]=0|0;HEAP8[$17+6>>0]=0|0;HEAP8[$17+7>>0]=0|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 80|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _btreeCreateTable($p,$piTable,$createTabFlags) {
 $p = $p|0;
 $piTable = $piTable|0;
 $createTabFlags = $createTabFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eType = 0, $iPtrPage = 0;
 var $pBt = 0, $pPageMove = 0, $pRoot = 0, $pgnoMove = 0, $pgnoRoot = 0, $ptfFlags = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pBt = sp + 28|0;
 $pRoot = sp + 24|0;
 $pgnoRoot = sp + 20|0;
 $rc = sp + 16|0;
 $ptfFlags = sp + 12|0;
 $pgnoMove = sp + 8|0;
 $pPageMove = sp + 4|0;
 $eType = sp + 48|0;
 $iPtrPage = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $piTable;
 HEAP32[$3>>2] = $createTabFlags;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 17|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 $11 = HEAP32[$pBt>>2]|0;
 if ($10) {
  _invalidateAllOverflowCache($11);
  $12 = HEAP32[$1>>2]|0;
  _sqlite3BtreeGetMeta($12,4,$pgnoRoot);
  $13 = HEAP32[$pgnoRoot>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$pgnoRoot>>2] = $14;
  while(1) {
   $15 = HEAP32[$pgnoRoot>>2]|0;
   $16 = HEAP32[$pBt>>2]|0;
   $17 = HEAP32[$pgnoRoot>>2]|0;
   $18 = (_ptrmapPageno($16,$17)|0);
   $19 = ($15|0)==($18|0);
   if (!($19)) {
    $20 = HEAP32[$pgnoRoot>>2]|0;
    $21 = HEAP32[832>>2]|0;
    $22 = HEAP32[$pBt>>2]|0;
    $23 = ((($22)) + 32|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($21>>>0) / ($24>>>0))&-1;
    $26 = (($25) + 1)|0;
    $27 = ($20|0)==($26|0);
    if (!($27)) {
     break;
    }
   }
   $28 = HEAP32[$pgnoRoot>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$pgnoRoot>>2] = $29;
  }
  $30 = HEAP32[$pBt>>2]|0;
  $31 = HEAP32[$pgnoRoot>>2]|0;
  $32 = (_allocateBtreePage($30,$pPageMove,$pgnoMove,$31,1)|0);
  HEAP32[$rc>>2] = $32;
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $35;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
  $36 = HEAP32[$pgnoMove>>2]|0;
  $37 = HEAP32[$pgnoRoot>>2]|0;
  $38 = ($36|0)!=($37|0);
  if ($38) {
   HEAP8[$eType>>0] = 0;
   HEAP32[$iPtrPage>>2] = 0;
   $39 = HEAP32[$pBt>>2]|0;
   $40 = (_saveAllCursors($39,0,0)|0);
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$pPageMove>>2]|0;
   _releasePage($41);
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $44;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $45 = HEAP32[$pBt>>2]|0;
   $46 = HEAP32[$pgnoRoot>>2]|0;
   $47 = (_btreeGetPage($45,$46,$pRoot,0)|0);
   HEAP32[$rc>>2] = $47;
   $48 = HEAP32[$rc>>2]|0;
   $49 = ($48|0)!=(0);
   if ($49) {
    $50 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $50;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $51 = HEAP32[$pBt>>2]|0;
   $52 = HEAP32[$pgnoRoot>>2]|0;
   $53 = (_ptrmapGet($51,$52,$eType,$iPtrPage)|0);
   HEAP32[$rc>>2] = $53;
   $54 = HEAP8[$eType>>0]|0;
   $55 = $54&255;
   $56 = ($55|0)==(1);
   if ($56) {
    label = 15;
   } else {
    $57 = HEAP8[$eType>>0]|0;
    $58 = $57&255;
    $59 = ($58|0)==(2);
    if ($59) {
     label = 15;
    }
   }
   if ((label|0) == 15) {
    $60 = (_sqlite3CorruptError(63920)|0);
    HEAP32[$rc>>2] = $60;
   }
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)!=(0);
   if ($62) {
    $63 = HEAP32[$pRoot>>2]|0;
    _releasePage($63);
    $64 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $64;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $65 = HEAP32[$pBt>>2]|0;
   $66 = HEAP32[$pRoot>>2]|0;
   $67 = HEAP8[$eType>>0]|0;
   $68 = HEAP32[$iPtrPage>>2]|0;
   $69 = HEAP32[$pgnoMove>>2]|0;
   $70 = (_relocatePage($65,$66,$67,$68,$69,0)|0);
   HEAP32[$rc>>2] = $70;
   $71 = HEAP32[$pRoot>>2]|0;
   _releasePage($71);
   $72 = HEAP32[$rc>>2]|0;
   $73 = ($72|0)!=(0);
   if ($73) {
    $74 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $74;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $75 = HEAP32[$pBt>>2]|0;
   $76 = HEAP32[$pgnoRoot>>2]|0;
   $77 = (_btreeGetPage($75,$76,$pRoot,0)|0);
   HEAP32[$rc>>2] = $77;
   $78 = HEAP32[$rc>>2]|0;
   $79 = ($78|0)!=(0);
   if ($79) {
    $80 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $80;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $81 = HEAP32[$pRoot>>2]|0;
   $82 = ((($81)) + 72|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = (_sqlite3PagerWrite($83)|0);
   HEAP32[$rc>>2] = $84;
   $85 = HEAP32[$rc>>2]|0;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$pRoot>>2]|0;
    _releasePage($87);
    $88 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $88;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
  } else {
   $89 = HEAP32[$pPageMove>>2]|0;
   HEAP32[$pRoot>>2] = $89;
  }
  $90 = HEAP32[$pBt>>2]|0;
  $91 = HEAP32[$pgnoRoot>>2]|0;
  _ptrmapPut($90,$91,1,0,$rc);
  $92 = HEAP32[$rc>>2]|0;
  $93 = ($92|0)!=(0);
  if ($93) {
   $94 = HEAP32[$pRoot>>2]|0;
   _releasePage($94);
   $95 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $95;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
  $96 = HEAP32[$1>>2]|0;
  $97 = HEAP32[$pgnoRoot>>2]|0;
  $98 = (_sqlite3BtreeUpdateMeta($96,4,$97)|0);
  HEAP32[$rc>>2] = $98;
  $99 = HEAP32[$rc>>2]|0;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = HEAP32[$pRoot>>2]|0;
   _releasePage($101);
   $102 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $102;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 } else {
  $103 = (_allocateBtreePage($11,$pRoot,$pgnoRoot,1,0)|0);
  HEAP32[$rc>>2] = $103;
  $104 = HEAP32[$rc>>2]|0;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $106;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 }
 $107 = HEAP32[$3>>2]|0;
 $108 = $107 & 1;
 $109 = ($108|0)!=(0);
 if ($109) {
  HEAP32[$ptfFlags>>2] = 13;
 } else {
  HEAP32[$ptfFlags>>2] = 10;
 }
 $110 = HEAP32[$pRoot>>2]|0;
 $111 = HEAP32[$ptfFlags>>2]|0;
 _zeroPage($110,$111);
 $112 = HEAP32[$pRoot>>2]|0;
 $113 = ((($112)) + 72|0);
 $114 = HEAP32[$113>>2]|0;
 _sqlite3PagerUnref($114);
 $115 = HEAP32[$pgnoRoot>>2]|0;
 $116 = HEAP32[$2>>2]|0;
 HEAP32[$116>>2] = $115;
 HEAP32[$0>>2] = 0;
 $117 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($117|0);
}
function _sqlite3VdbeFindCompare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $flags = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 6|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = (($6) + ($11))|0;
 $13 = ($12|0)<=(13);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP16[$17>>1]|0;
  $19 = $18&65535;
  HEAP32[$flags>>2] = $19;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = ($24<<24>>24)!=(0);
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 12|0);
  if ($25) {
   HEAP8[$27>>0] = 1;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 13|0);
   HEAP8[$29>>0] = -1;
  } else {
   HEAP8[$27>>0] = -1;
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 13|0);
   HEAP8[$31>>0] = 1;
  }
  $32 = HEAP32[$flags>>2]|0;
  $33 = $32 & 4;
  $34 = ($33|0)!=(0);
  if ($34) {
   HEAP32[$0>>2] = 153;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
  $35 = HEAP32[$flags>>2]|0;
  $36 = $35 & 25;
  $37 = ($36|0)==(0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 20|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    HEAP32[$0>>2] = 154;
    $43 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($43|0);
   }
  }
 }
 HEAP32[$0>>2] = 155;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _vdbeRecordCompareInt($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey = 0, $lhs = 0, $res = 0, $serial_type = 0, $v = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $aKey = sp + 36|0;
 $serial_type = sp + 32|0;
 $res = sp + 28|0;
 $y = sp + 24|0;
 $x = sp + 16|0;
 $v = sp + 8|0;
 $lhs = sp;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 63;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($7)|0);
 HEAP32[$aKey>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$serial_type>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = $v;
 $24 = $23;
 HEAP32[$24>>2] = $19;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = HEAP32[$serial_type>>2]|0;
 do {
  switch ($27|0) {
  case 1:  {
   $28 = HEAP32[$aKey>>2]|0;
   $29 = HEAP8[$28>>0]|0;
   $30 = $29 << 24 >> 24;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = $lhs;
   $34 = $33;
   HEAP32[$34>>2] = $30;
   $35 = (($33) + 4)|0;
   $36 = $35;
   HEAP32[$36>>2] = $32;
   break;
  }
  case 2:  {
   $37 = HEAP32[$aKey>>2]|0;
   $38 = HEAP8[$37>>0]|0;
   $39 = $38 << 24 >> 24;
   $40 = $39<<8;
   $41 = HEAP32[$aKey>>2]|0;
   $42 = ((($41)) + 1|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $40 | $44;
   $46 = ($45|0)<(0);
   $47 = $46 << 31 >> 31;
   $48 = $lhs;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = (($48) + 4)|0;
   $51 = $50;
   HEAP32[$51>>2] = $47;
   break;
  }
  case 3:  {
   $52 = HEAP32[$aKey>>2]|0;
   $53 = HEAP8[$52>>0]|0;
   $54 = $53 << 24 >> 24;
   $55 = $54<<16;
   $56 = HEAP32[$aKey>>2]|0;
   $57 = ((($56)) + 1|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = $59 << 8;
   $61 = $55 | $60;
   $62 = HEAP32[$aKey>>2]|0;
   $63 = ((($62)) + 2|0);
   $64 = HEAP8[$63>>0]|0;
   $65 = $64&255;
   $66 = $61 | $65;
   $67 = ($66|0)<(0);
   $68 = $67 << 31 >> 31;
   $69 = $lhs;
   $70 = $69;
   HEAP32[$70>>2] = $66;
   $71 = (($69) + 4)|0;
   $72 = $71;
   HEAP32[$72>>2] = $68;
   break;
  }
  case 4:  {
   $73 = HEAP32[$aKey>>2]|0;
   $74 = HEAP8[$73>>0]|0;
   $75 = $74&255;
   $76 = $75 << 24;
   $77 = HEAP32[$aKey>>2]|0;
   $78 = ((($77)) + 1|0);
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = $80 << 16;
   $82 = $76 | $81;
   $83 = HEAP32[$aKey>>2]|0;
   $84 = ((($83)) + 2|0);
   $85 = HEAP8[$84>>0]|0;
   $86 = $85&255;
   $87 = $86 << 8;
   $88 = $82 | $87;
   $89 = HEAP32[$aKey>>2]|0;
   $90 = ((($89)) + 3|0);
   $91 = HEAP8[$90>>0]|0;
   $92 = $91&255;
   $93 = $88 | $92;
   HEAP32[$y>>2] = $93;
   $94 = HEAP32[$y>>2]|0;
   $95 = ($94|0)<(0);
   $96 = $95 << 31 >> 31;
   $97 = $lhs;
   $98 = $97;
   HEAP32[$98>>2] = $94;
   $99 = (($97) + 4)|0;
   $100 = $99;
   HEAP32[$100>>2] = $96;
   break;
  }
  case 5:  {
   $101 = HEAP32[$aKey>>2]|0;
   $102 = ((($101)) + 2|0);
   $103 = HEAP8[$102>>0]|0;
   $104 = $103&255;
   $105 = $104 << 24;
   $106 = HEAP32[$aKey>>2]|0;
   $107 = ((($106)) + 2|0);
   $108 = ((($107)) + 1|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = $109&255;
   $111 = $110 << 16;
   $112 = $105 | $111;
   $113 = HEAP32[$aKey>>2]|0;
   $114 = ((($113)) + 2|0);
   $115 = ((($114)) + 2|0);
   $116 = HEAP8[$115>>0]|0;
   $117 = $116&255;
   $118 = $117 << 8;
   $119 = $112 | $118;
   $120 = HEAP32[$aKey>>2]|0;
   $121 = ((($120)) + 2|0);
   $122 = ((($121)) + 3|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = $123&255;
   $125 = $119 | $124;
   $126 = HEAP32[$aKey>>2]|0;
   $127 = HEAP8[$126>>0]|0;
   $128 = $127 << 24 >> 24;
   $129 = $128<<8;
   $130 = HEAP32[$aKey>>2]|0;
   $131 = ((($130)) + 1|0);
   $132 = HEAP8[$131>>0]|0;
   $133 = $132&255;
   $134 = $129 | $133;
   $135 = ($134|0)<(0);
   $136 = $135 << 31 >> 31;
   $137 = (___muldi3(0,1,($134|0),($136|0))|0);
   $138 = tempRet0;
   $139 = (_i64Add(($125|0),0,($137|0),($138|0))|0);
   $140 = tempRet0;
   $141 = $lhs;
   $142 = $141;
   HEAP32[$142>>2] = $139;
   $143 = (($141) + 4)|0;
   $144 = $143;
   HEAP32[$144>>2] = $140;
   break;
  }
  case 6:  {
   $145 = HEAP32[$aKey>>2]|0;
   $146 = HEAP8[$145>>0]|0;
   $147 = $146&255;
   $148 = $147 << 24;
   $149 = HEAP32[$aKey>>2]|0;
   $150 = ((($149)) + 1|0);
   $151 = HEAP8[$150>>0]|0;
   $152 = $151&255;
   $153 = $152 << 16;
   $154 = $148 | $153;
   $155 = HEAP32[$aKey>>2]|0;
   $156 = ((($155)) + 2|0);
   $157 = HEAP8[$156>>0]|0;
   $158 = $157&255;
   $159 = $158 << 8;
   $160 = $154 | $159;
   $161 = HEAP32[$aKey>>2]|0;
   $162 = ((($161)) + 3|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163&255;
   $165 = $160 | $164;
   $166 = $x;
   $167 = $166;
   HEAP32[$167>>2] = $165;
   $168 = (($166) + 4)|0;
   $169 = $168;
   HEAP32[$169>>2] = 0;
   $170 = $x;
   $171 = $170;
   $172 = HEAP32[$171>>2]|0;
   $173 = (($170) + 4)|0;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = HEAP32[$aKey>>2]|0;
   $177 = ((($176)) + 4|0);
   $178 = HEAP8[$177>>0]|0;
   $179 = $178&255;
   $180 = $179 << 24;
   $181 = HEAP32[$aKey>>2]|0;
   $182 = ((($181)) + 4|0);
   $183 = ((($182)) + 1|0);
   $184 = HEAP8[$183>>0]|0;
   $185 = $184&255;
   $186 = $185 << 16;
   $187 = $180 | $186;
   $188 = HEAP32[$aKey>>2]|0;
   $189 = ((($188)) + 4|0);
   $190 = ((($189)) + 2|0);
   $191 = HEAP8[$190>>0]|0;
   $192 = $191&255;
   $193 = $192 << 8;
   $194 = $187 | $193;
   $195 = HEAP32[$aKey>>2]|0;
   $196 = ((($195)) + 4|0);
   $197 = ((($196)) + 3|0);
   $198 = HEAP8[$197>>0]|0;
   $199 = $198&255;
   $200 = $194 | $199;
   $201 = $x;
   $202 = $201;
   HEAP32[$202>>2] = $200;
   $203 = (($201) + 4)|0;
   $204 = $203;
   HEAP32[$204>>2] = $172;
   $205 = $x;
   $206 = $205;
   $207 = HEAP32[$206>>2]|0;
   $208 = (($205) + 4)|0;
   $209 = $208;
   $210 = HEAP32[$209>>2]|0;
   $211 = $lhs;
   $212 = $211;
   HEAP32[$212>>2] = $207;
   $213 = (($211) + 4)|0;
   $214 = $213;
   HEAP32[$214>>2] = $210;
   break;
  }
  case 8:  {
   $215 = $lhs;
   $216 = $215;
   HEAP32[$216>>2] = 0;
   $217 = (($215) + 4)|0;
   $218 = $217;
   HEAP32[$218>>2] = 0;
   break;
  }
  case 9:  {
   $219 = $lhs;
   $220 = $219;
   HEAP32[$220>>2] = 1;
   $221 = (($219) + 4)|0;
   $222 = $221;
   HEAP32[$222>>2] = 0;
   break;
  }
  case 7: case 0:  {
   $223 = HEAP32[$1>>2]|0;
   $224 = HEAP32[$2>>2]|0;
   $225 = HEAP32[$3>>2]|0;
   $226 = (_sqlite3VdbeRecordCompare($223,$224,$225)|0);
   HEAP32[$0>>2] = $226;
   $288 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($288|0);
   break;
  }
  default: {
   $227 = HEAP32[$1>>2]|0;
   $228 = HEAP32[$2>>2]|0;
   $229 = HEAP32[$3>>2]|0;
   $230 = (_sqlite3VdbeRecordCompare($227,$228,$229)|0);
   HEAP32[$0>>2] = $230;
   $288 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($288|0);
  }
  }
 } while(0);
 $231 = $v;
 $232 = $231;
 $233 = HEAP32[$232>>2]|0;
 $234 = (($231) + 4)|0;
 $235 = $234;
 $236 = HEAP32[$235>>2]|0;
 $237 = $lhs;
 $238 = $237;
 $239 = HEAP32[$238>>2]|0;
 $240 = (($237) + 4)|0;
 $241 = $240;
 $242 = HEAP32[$241>>2]|0;
 $243 = ($236|0)>($242|0);
 $244 = ($233>>>0)>($239>>>0);
 $245 = ($236|0)==($242|0);
 $246 = $245 & $244;
 $247 = $243 | $246;
 do {
  if ($247) {
   $248 = HEAP32[$3>>2]|0;
   $249 = ((($248)) + 12|0);
   $250 = HEAP8[$249>>0]|0;
   $251 = $250 << 24 >> 24;
   HEAP32[$res>>2] = $251;
  } else {
   $252 = $v;
   $253 = $252;
   $254 = HEAP32[$253>>2]|0;
   $255 = (($252) + 4)|0;
   $256 = $255;
   $257 = HEAP32[$256>>2]|0;
   $258 = $lhs;
   $259 = $258;
   $260 = HEAP32[$259>>2]|0;
   $261 = (($258) + 4)|0;
   $262 = $261;
   $263 = HEAP32[$262>>2]|0;
   $264 = ($257|0)<($263|0);
   $265 = ($254>>>0)<($260>>>0);
   $266 = ($257|0)==($263|0);
   $267 = $266 & $265;
   $268 = $264 | $267;
   $269 = HEAP32[$3>>2]|0;
   if ($268) {
    $270 = ((($269)) + 13|0);
    $271 = HEAP8[$270>>0]|0;
    $272 = $271 << 24 >> 24;
    HEAP32[$res>>2] = $272;
    break;
   }
   $273 = ((($269)) + 8|0);
   $274 = HEAP16[$273>>1]|0;
   $275 = $274&65535;
   $276 = ($275|0)>(1);
   if ($276) {
    $277 = HEAP32[$1>>2]|0;
    $278 = HEAP32[$2>>2]|0;
    $279 = HEAP32[$3>>2]|0;
    $280 = (_sqlite3VdbeRecordCompareWithSkip($277,$278,$279,1)|0);
    HEAP32[$res>>2] = $280;
    break;
   } else {
    $281 = HEAP32[$3>>2]|0;
    $282 = ((($281)) + 10|0);
    $283 = HEAP8[$282>>0]|0;
    $284 = $283 << 24 >> 24;
    HEAP32[$res>>2] = $284;
    $285 = HEAP32[$3>>2]|0;
    $286 = ((($285)) + 14|0);
    HEAP8[$286>>0] = 1;
    break;
   }
  }
 } while(0);
 $287 = HEAP32[$res>>2]|0;
 HEAP32[$0>>2] = $287;
 $288 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($288|0);
}
function _vdbeRecordCompareString($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey1 = 0, $nCmp = 0, $nStr = 0, $res = 0, $serial_type = 0, $szHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aKey1 = sp + 20|0;
 $serial_type = sp + 16|0;
 $res = sp + 12|0;
 $nCmp = sp + 8|0;
 $nStr = sp + 4|0;
 $szHdr = sp;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$aKey1>>2] = $4;
 $5 = HEAP32[$aKey1>>2]|0;
 $6 = ((($5)) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)<(128);
 $10 = HEAP32[$aKey1>>2]|0;
 $11 = ((($10)) + 1|0);
 if ($9) {
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  HEAP32[$serial_type>>2] = $13;
 } else {
  (_sqlite3GetVarint32($11,$serial_type)|0);
 }
 $14 = HEAP32[$serial_type>>2]|0;
 $15 = ($14|0)<(12);
 do {
  if ($15) {
   $16 = HEAP32[$3>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   HEAP32[$res>>2] = $19;
  } else {
   $20 = HEAP32[$serial_type>>2]|0;
   $21 = $20 & 1;
   $22 = ($21|0)!=(0);
   if (!($22)) {
    $23 = HEAP32[$3>>2]|0;
    $24 = ((($23)) + 13|0);
    $25 = HEAP8[$24>>0]|0;
    $26 = $25 << 24 >> 24;
    HEAP32[$res>>2] = $26;
    break;
   }
   $27 = HEAP32[$aKey1>>2]|0;
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   HEAP32[$szHdr>>2] = $29;
   $30 = HEAP32[$serial_type>>2]|0;
   $31 = (($30) - 12)|0;
   $32 = (($31|0) / 2)&-1;
   HEAP32[$nStr>>2] = $32;
   $33 = HEAP32[$szHdr>>2]|0;
   $34 = HEAP32[$nStr>>2]|0;
   $35 = (($33) + ($34))|0;
   $36 = HEAP32[$1>>2]|0;
   $37 = ($35|0)>($36|0);
   if ($37) {
    $38 = (_sqlite3CorruptError(71970)|0);
    $39 = $38&255;
    $40 = HEAP32[$3>>2]|0;
    $41 = ((($40)) + 11|0);
    HEAP8[$41>>0] = $39;
    HEAP32[$0>>2] = 0;
    $111 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($111|0);
   }
   $42 = HEAP32[$3>>2]|0;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ((($44)) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$nStr>>2]|0;
   $48 = ($46|0)<($47|0);
   if ($48) {
    $49 = HEAP32[$3>>2]|0;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ((($51)) + 12|0);
    $53 = HEAP32[$52>>2]|0;
    $55 = $53;
   } else {
    $54 = HEAP32[$nStr>>2]|0;
    $55 = $54;
   }
   HEAP32[$nCmp>>2] = $55;
   $56 = HEAP32[$szHdr>>2]|0;
   $57 = HEAP32[$aKey1>>2]|0;
   $58 = (($57) + ($56)|0);
   $59 = HEAP32[$3>>2]|0;
   $60 = ((($59)) + 4|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ((($61)) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$nCmp>>2]|0;
   $65 = (_memcmp($58,$63,$64)|0);
   HEAP32[$res>>2] = $65;
   $66 = HEAP32[$res>>2]|0;
   $67 = ($66|0)==(0);
   if (!($67)) {
    $101 = HEAP32[$res>>2]|0;
    $102 = ($101|0)>(0);
    $103 = HEAP32[$3>>2]|0;
    if ($102) {
     $104 = ((($103)) + 13|0);
     $105 = HEAP8[$104>>0]|0;
     $106 = $105 << 24 >> 24;
     HEAP32[$res>>2] = $106;
     break;
    } else {
     $107 = ((($103)) + 12|0);
     $108 = HEAP8[$107>>0]|0;
     $109 = $108 << 24 >> 24;
     HEAP32[$res>>2] = $109;
     break;
    }
   }
   $68 = HEAP32[$nStr>>2]|0;
   $69 = HEAP32[$3>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ((($71)) + 12|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($68) - ($73))|0;
   HEAP32[$res>>2] = $74;
   $75 = HEAP32[$res>>2]|0;
   $76 = ($75|0)==(0);
   if ($76) {
    $77 = HEAP32[$3>>2]|0;
    $78 = ((($77)) + 8|0);
    $79 = HEAP16[$78>>1]|0;
    $80 = $79&65535;
    $81 = ($80|0)>(1);
    if ($81) {
     $82 = HEAP32[$1>>2]|0;
     $83 = HEAP32[$2>>2]|0;
     $84 = HEAP32[$3>>2]|0;
     $85 = (_sqlite3VdbeRecordCompareWithSkip($82,$83,$84,1)|0);
     HEAP32[$res>>2] = $85;
     break;
    } else {
     $86 = HEAP32[$3>>2]|0;
     $87 = ((($86)) + 10|0);
     $88 = HEAP8[$87>>0]|0;
     $89 = $88 << 24 >> 24;
     HEAP32[$res>>2] = $89;
     $90 = HEAP32[$3>>2]|0;
     $91 = ((($90)) + 14|0);
     HEAP8[$91>>0] = 1;
     break;
    }
   } else {
    $92 = HEAP32[$res>>2]|0;
    $93 = ($92|0)>(0);
    $94 = HEAP32[$3>>2]|0;
    if ($93) {
     $95 = ((($94)) + 13|0);
     $96 = HEAP8[$95>>0]|0;
     $97 = $96 << 24 >> 24;
     HEAP32[$res>>2] = $97;
     break;
    } else {
     $98 = ((($94)) + 12|0);
     $99 = HEAP8[$98>>0]|0;
     $100 = $99 << 24 >> 24;
     HEAP32[$res>>2] = $100;
     break;
    }
   }
  }
 } while(0);
 $110 = HEAP32[$res>>2]|0;
 HEAP32[$0>>2] = $110;
 $111 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($111|0);
}
function _sqlite3VdbeRecordCompare($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $nKey1;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $pPKey2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeRecordCompareWithSkip($3,$4,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeRecordCompareWithSkip($nKey1,$pKey1,$pPKey2,$bSkip) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 $bSkip = $bSkip|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0.0, $164 = 0, $165 = 0.0, $166 = 0, $167 = 0.0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0.0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0.0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey1 = 0, $d1 = 0, $i = 0, $idx1 = 0;
 var $lhs = 0, $mem1 = 0, $nCmp = 0, $nCmp1 = 0, $nStr = 0, $pKeyInfo = 0, $pRhs = 0, $rc = 0, $rhs = 0, $s1 = 0, $serial_type = 0, $szHdr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $d1 = sp + 104|0;
 $i = sp + 100|0;
 $szHdr1 = sp + 96|0;
 $idx1 = sp + 92|0;
 $rc = sp + 88|0;
 $pRhs = sp + 84|0;
 $pKeyInfo = sp + 80|0;
 $aKey1 = sp + 76|0;
 $mem1 = sp + 16|0;
 $s1 = sp + 72|0;
 $serial_type = sp + 68|0;
 $lhs = sp + 8|0;
 $rhs = sp;
 $nCmp = sp + 64|0;
 $nStr = sp + 60|0;
 $nCmp1 = sp + 56|0;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 HEAP32[$4>>2] = $bSkip;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pRhs>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pKeyInfo>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$aKey1>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = HEAP32[$aKey1>>2]|0;
 do {
  if ($12) {
   $14 = ((($13)) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)<(128);
   $18 = HEAP32[$aKey1>>2]|0;
   $19 = ((($18)) + 1|0);
   if ($17) {
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    HEAP32[$s1>>2] = $21;
    $25 = 1;
   } else {
    $22 = (_sqlite3GetVarint32($19,$s1)|0);
    $23 = $22&255;
    $25 = $23;
   }
   $24 = $25&255;
   $26 = $24&255;
   $27 = (1 + ($26))|0;
   HEAP32[$idx1>>2] = $27;
   $28 = HEAP32[$aKey1>>2]|0;
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   HEAP32[$szHdr1>>2] = $30;
   $31 = HEAP32[$szHdr1>>2]|0;
   $32 = HEAP32[$s1>>2]|0;
   $33 = (_sqlite3VdbeSerialTypeLen($32)|0);
   $34 = (($31) + ($33))|0;
   HEAP32[$d1>>2] = $34;
   HEAP32[$i>>2] = 1;
   $35 = HEAP32[$pRhs>>2]|0;
   $36 = ((($35)) + 40|0);
   HEAP32[$pRhs>>2] = $36;
  } else {
   $37 = HEAP8[$13>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)<(128);
   $40 = HEAP32[$aKey1>>2]|0;
   if ($39) {
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    HEAP32[$szHdr1>>2] = $42;
    $46 = 1;
   } else {
    $43 = (_sqlite3GetVarint32($40,$szHdr1)|0);
    $44 = $43&255;
    $46 = $44;
   }
   $45 = $46&255;
   $47 = $45&255;
   HEAP32[$idx1>>2] = $47;
   $48 = HEAP32[$szHdr1>>2]|0;
   HEAP32[$d1>>2] = $48;
   $49 = HEAP32[$d1>>2]|0;
   $50 = HEAP32[$1>>2]|0;
   $51 = ($49>>>0)>($50>>>0);
   if (!($51)) {
    HEAP32[$i>>2] = 0;
    break;
   }
   $52 = (_sqlite3CorruptError(71688)|0);
   $53 = $52&255;
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 11|0);
   HEAP8[$55>>0] = $53;
   HEAP32[$0>>2] = 0;
   $375 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($375|0);
  }
 } while(0);
 L16: while(1) {
  $56 = HEAP32[$pRhs>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & 4;
  $61 = ($60|0)!=(0);
  do {
   if ($61) {
    $62 = HEAP32[$idx1>>2]|0;
    $63 = HEAP32[$aKey1>>2]|0;
    $64 = (($63) + ($62)|0);
    $65 = HEAP8[$64>>0]|0;
    $66 = $65&255;
    HEAP32[$serial_type>>2] = $66;
    $67 = HEAP32[$serial_type>>2]|0;
    $68 = ($67>>>0)>=(10);
    if ($68) {
     HEAP32[$rc>>2] = 1;
     break;
    }
    $69 = HEAP32[$serial_type>>2]|0;
    $70 = ($69|0)==(0);
    if ($70) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $71 = HEAP32[$serial_type>>2]|0;
    $72 = ($71|0)==(7);
    if ($72) {
     $73 = HEAP32[$d1>>2]|0;
     $74 = HEAP32[$aKey1>>2]|0;
     $75 = (($74) + ($73)|0);
     $76 = HEAP32[$serial_type>>2]|0;
     (_sqlite3VdbeSerialGet($75,$76,$mem1)|0);
     $77 = HEAP32[$pRhs>>2]|0;
     $78 = $77;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = (($78) + 4)|0;
     $82 = $81;
     $83 = HEAP32[$82>>2]|0;
     $84 = +HEAPF64[$mem1>>3];
     $85 = (_sqlite3IntFloatCompare($80,$83,$84)|0);
     $86 = (0 - ($85))|0;
     HEAP32[$rc>>2] = $86;
     break;
    }
    $87 = HEAP32[$serial_type>>2]|0;
    $88 = HEAP32[$d1>>2]|0;
    $89 = HEAP32[$aKey1>>2]|0;
    $90 = (($89) + ($88)|0);
    $91 = (_vdbeRecordDecodeInt($87,$90)|0);
    $92 = tempRet0;
    $93 = $lhs;
    $94 = $93;
    HEAP32[$94>>2] = $91;
    $95 = (($93) + 4)|0;
    $96 = $95;
    HEAP32[$96>>2] = $92;
    $97 = HEAP32[$pRhs>>2]|0;
    $98 = $97;
    $99 = $98;
    $100 = HEAP32[$99>>2]|0;
    $101 = (($98) + 4)|0;
    $102 = $101;
    $103 = HEAP32[$102>>2]|0;
    $104 = $rhs;
    $105 = $104;
    HEAP32[$105>>2] = $100;
    $106 = (($104) + 4)|0;
    $107 = $106;
    HEAP32[$107>>2] = $103;
    $108 = $lhs;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (($108) + 4)|0;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = $rhs;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = ($113|0)<($119|0);
    $121 = ($110>>>0)<($116>>>0);
    $122 = ($113|0)==($119|0);
    $123 = $122 & $121;
    $124 = $120 | $123;
    if ($124) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $125 = $lhs;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (($125) + 4)|0;
    $129 = $128;
    $130 = HEAP32[$129>>2]|0;
    $131 = $rhs;
    $132 = $131;
    $133 = HEAP32[$132>>2]|0;
    $134 = (($131) + 4)|0;
    $135 = $134;
    $136 = HEAP32[$135>>2]|0;
    $137 = ($130|0)>($136|0);
    $138 = ($127>>>0)>($133>>>0);
    $139 = ($130|0)==($136|0);
    $140 = $139 & $138;
    $141 = $137 | $140;
    if ($141) {
     HEAP32[$rc>>2] = 1;
    }
   } else {
    $142 = HEAP32[$pRhs>>2]|0;
    $143 = ((($142)) + 8|0);
    $144 = HEAP16[$143>>1]|0;
    $145 = $144&65535;
    $146 = $145 & 8;
    $147 = ($146|0)!=(0);
    if ($147) {
     $148 = HEAP32[$idx1>>2]|0;
     $149 = HEAP32[$aKey1>>2]|0;
     $150 = (($149) + ($148)|0);
     $151 = HEAP8[$150>>0]|0;
     $152 = $151&255;
     HEAP32[$serial_type>>2] = $152;
     $153 = HEAP32[$serial_type>>2]|0;
     $154 = ($153>>>0)>=(10);
     if ($154) {
      HEAP32[$rc>>2] = 1;
      break;
     }
     $155 = HEAP32[$serial_type>>2]|0;
     $156 = ($155|0)==(0);
     if ($156) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $157 = HEAP32[$d1>>2]|0;
     $158 = HEAP32[$aKey1>>2]|0;
     $159 = (($158) + ($157)|0);
     $160 = HEAP32[$serial_type>>2]|0;
     (_sqlite3VdbeSerialGet($159,$160,$mem1)|0);
     $161 = HEAP32[$serial_type>>2]|0;
     $162 = ($161|0)==(7);
     if (!($162)) {
      $171 = $mem1;
      $172 = $171;
      $173 = HEAP32[$172>>2]|0;
      $174 = (($171) + 4)|0;
      $175 = $174;
      $176 = HEAP32[$175>>2]|0;
      $177 = HEAP32[$pRhs>>2]|0;
      $178 = +HEAPF64[$177>>3];
      $179 = (_sqlite3IntFloatCompare($173,$176,$178)|0);
      HEAP32[$rc>>2] = $179;
      break;
     }
     $163 = +HEAPF64[$mem1>>3];
     $164 = HEAP32[$pRhs>>2]|0;
     $165 = +HEAPF64[$164>>3];
     $166 = $163 < $165;
     if ($166) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $167 = +HEAPF64[$mem1>>3];
     $168 = HEAP32[$pRhs>>2]|0;
     $169 = +HEAPF64[$168>>3];
     $170 = $167 > $169;
     if (!($170)) {
      break;
     }
     HEAP32[$rc>>2] = 1;
     break;
    }
    $180 = HEAP32[$pRhs>>2]|0;
    $181 = ((($180)) + 8|0);
    $182 = HEAP16[$181>>1]|0;
    $183 = $182&65535;
    $184 = $183 & 2;
    $185 = ($184|0)!=(0);
    if (!($185)) {
     $272 = HEAP32[$pRhs>>2]|0;
     $273 = ((($272)) + 8|0);
     $274 = HEAP16[$273>>1]|0;
     $275 = $274&65535;
     $276 = $275 & 16;
     $277 = ($276|0)!=(0);
     $278 = HEAP32[$idx1>>2]|0;
     $279 = HEAP32[$aKey1>>2]|0;
     $280 = (($279) + ($278)|0);
     $281 = HEAP8[$280>>0]|0;
     $282 = $281&255;
     if (!($277)) {
      HEAP32[$serial_type>>2] = $282;
      $331 = HEAP32[$serial_type>>2]|0;
      $332 = ($331|0)!=(0);
      $333 = $332&1;
      HEAP32[$rc>>2] = $333;
      break;
     }
     $283 = ($282|0)<(128);
     $284 = HEAP32[$idx1>>2]|0;
     $285 = HEAP32[$aKey1>>2]|0;
     $286 = (($285) + ($284)|0);
     if ($283) {
      $287 = HEAP8[$286>>0]|0;
      $288 = $287&255;
      HEAP32[$serial_type>>2] = $288;
     } else {
      (_sqlite3GetVarint32($286,$serial_type)|0);
     }
     $289 = HEAP32[$serial_type>>2]|0;
     $290 = ($289>>>0)<(12);
     if (!($290)) {
      $291 = HEAP32[$serial_type>>2]|0;
      $292 = $291 & 1;
      $293 = ($292|0)!=(0);
      if (!($293)) {
       $294 = HEAP32[$serial_type>>2]|0;
       $295 = (($294) - 12)|0;
       $296 = (($295>>>0) / 2)&-1;
       HEAP32[$nStr>>2] = $296;
       $297 = HEAP32[$d1>>2]|0;
       $298 = HEAP32[$nStr>>2]|0;
       $299 = (($297) + ($298))|0;
       $300 = HEAP32[$1>>2]|0;
       $301 = ($299>>>0)>($300>>>0);
       if ($301) {
        label = 59;
        break L16;
       }
       $306 = HEAP32[$nStr>>2]|0;
       $307 = HEAP32[$pRhs>>2]|0;
       $308 = ((($307)) + 12|0);
       $309 = HEAP32[$308>>2]|0;
       $310 = ($306|0)<($309|0);
       if ($310) {
        $311 = HEAP32[$nStr>>2]|0;
        $315 = $311;
       } else {
        $312 = HEAP32[$pRhs>>2]|0;
        $313 = ((($312)) + 12|0);
        $314 = HEAP32[$313>>2]|0;
        $315 = $314;
       }
       HEAP32[$nCmp1>>2] = $315;
       $316 = HEAP32[$d1>>2]|0;
       $317 = HEAP32[$aKey1>>2]|0;
       $318 = (($317) + ($316)|0);
       $319 = HEAP32[$pRhs>>2]|0;
       $320 = ((($319)) + 16|0);
       $321 = HEAP32[$320>>2]|0;
       $322 = HEAP32[$nCmp1>>2]|0;
       $323 = (_memcmp($318,$321,$322)|0);
       HEAP32[$rc>>2] = $323;
       $324 = HEAP32[$rc>>2]|0;
       $325 = ($324|0)==(0);
       if (!($325)) {
        break;
       }
       $326 = HEAP32[$nStr>>2]|0;
       $327 = HEAP32[$pRhs>>2]|0;
       $328 = ((($327)) + 12|0);
       $329 = HEAP32[$328>>2]|0;
       $330 = (($326) - ($329))|0;
       HEAP32[$rc>>2] = $330;
       break;
      }
     }
     HEAP32[$rc>>2] = -1;
     break;
    }
    $186 = HEAP32[$idx1>>2]|0;
    $187 = HEAP32[$aKey1>>2]|0;
    $188 = (($187) + ($186)|0);
    $189 = HEAP8[$188>>0]|0;
    $190 = $189&255;
    $191 = ($190|0)<(128);
    $192 = HEAP32[$idx1>>2]|0;
    $193 = HEAP32[$aKey1>>2]|0;
    $194 = (($193) + ($192)|0);
    if ($191) {
     $195 = HEAP8[$194>>0]|0;
     $196 = $195&255;
     HEAP32[$serial_type>>2] = $196;
    } else {
     (_sqlite3GetVarint32($194,$serial_type)|0);
    }
    $197 = HEAP32[$serial_type>>2]|0;
    $198 = ($197>>>0)<(12);
    if ($198) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $199 = HEAP32[$serial_type>>2]|0;
    $200 = $199 & 1;
    $201 = ($200|0)!=(0);
    if (!($201)) {
     HEAP32[$rc>>2] = 1;
     break;
    }
    $202 = HEAP32[$serial_type>>2]|0;
    $203 = (($202) - 12)|0;
    $204 = (($203>>>0) / 2)&-1;
    $205 = ((($mem1)) + 12|0);
    HEAP32[$205>>2] = $204;
    $206 = HEAP32[$d1>>2]|0;
    $207 = ((($mem1)) + 12|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = (($206) + ($208))|0;
    $210 = HEAP32[$1>>2]|0;
    $211 = ($209>>>0)>($210>>>0);
    if ($211) {
     label = 43;
     break L16;
    }
    $216 = HEAP32[$i>>2]|0;
    $217 = HEAP32[$pKeyInfo>>2]|0;
    $218 = ((($217)) + 20|0);
    $219 = (($218) + ($216<<2)|0);
    $220 = HEAP32[$219>>2]|0;
    $221 = ($220|0)!=(0|0);
    if ($221) {
     $222 = HEAP32[$pKeyInfo>>2]|0;
     $223 = ((($222)) + 4|0);
     $224 = HEAP8[$223>>0]|0;
     $225 = ((($mem1)) + 10|0);
     HEAP8[$225>>0] = $224;
     $226 = HEAP32[$pKeyInfo>>2]|0;
     $227 = ((($226)) + 12|0);
     $228 = HEAP32[$227>>2]|0;
     $229 = ((($mem1)) + 32|0);
     HEAP32[$229>>2] = $228;
     $230 = ((($mem1)) + 8|0);
     HEAP16[$230>>1] = 2;
     $231 = HEAP32[$d1>>2]|0;
     $232 = HEAP32[$aKey1>>2]|0;
     $233 = (($232) + ($231)|0);
     $234 = ((($mem1)) + 16|0);
     HEAP32[$234>>2] = $233;
     $235 = HEAP32[$pRhs>>2]|0;
     $236 = HEAP32[$i>>2]|0;
     $237 = HEAP32[$pKeyInfo>>2]|0;
     $238 = ((($237)) + 20|0);
     $239 = (($238) + ($236<<2)|0);
     $240 = HEAP32[$239>>2]|0;
     $241 = HEAP32[$3>>2]|0;
     $242 = ((($241)) + 11|0);
     $243 = (_vdbeCompareMemString($mem1,$235,$240,$242)|0);
     HEAP32[$rc>>2] = $243;
     break;
    }
    $244 = ((($mem1)) + 12|0);
    $245 = HEAP32[$244>>2]|0;
    $246 = HEAP32[$pRhs>>2]|0;
    $247 = ((($246)) + 12|0);
    $248 = HEAP32[$247>>2]|0;
    $249 = ($245|0)<($248|0);
    if ($249) {
     $250 = ((($mem1)) + 12|0);
     $251 = HEAP32[$250>>2]|0;
     $255 = $251;
    } else {
     $252 = HEAP32[$pRhs>>2]|0;
     $253 = ((($252)) + 12|0);
     $254 = HEAP32[$253>>2]|0;
     $255 = $254;
    }
    HEAP32[$nCmp>>2] = $255;
    $256 = HEAP32[$d1>>2]|0;
    $257 = HEAP32[$aKey1>>2]|0;
    $258 = (($257) + ($256)|0);
    $259 = HEAP32[$pRhs>>2]|0;
    $260 = ((($259)) + 16|0);
    $261 = HEAP32[$260>>2]|0;
    $262 = HEAP32[$nCmp>>2]|0;
    $263 = (_memcmp($258,$261,$262)|0);
    HEAP32[$rc>>2] = $263;
    $264 = HEAP32[$rc>>2]|0;
    $265 = ($264|0)==(0);
    if ($265) {
     $266 = ((($mem1)) + 12|0);
     $267 = HEAP32[$266>>2]|0;
     $268 = HEAP32[$pRhs>>2]|0;
     $269 = ((($268)) + 12|0);
     $270 = HEAP32[$269>>2]|0;
     $271 = (($267) - ($270))|0;
     HEAP32[$rc>>2] = $271;
    }
   }
  } while(0);
  $334 = HEAP32[$rc>>2]|0;
  $335 = ($334|0)!=(0);
  $336 = HEAP32[$i>>2]|0;
  if ($335) {
   label = 67;
   break;
  }
  $346 = (($336) + 1)|0;
  HEAP32[$i>>2] = $346;
  $347 = HEAP32[$pRhs>>2]|0;
  $348 = ((($347)) + 40|0);
  HEAP32[$pRhs>>2] = $348;
  $349 = HEAP32[$serial_type>>2]|0;
  $350 = (_sqlite3VdbeSerialTypeLen($349)|0);
  $351 = HEAP32[$d1>>2]|0;
  $352 = (($351) + ($350))|0;
  HEAP32[$d1>>2] = $352;
  $353 = HEAP32[$serial_type>>2]|0;
  $354 = (_sqlite3VarintLen($353,0)|0);
  $355 = HEAP32[$idx1>>2]|0;
  $356 = (($355) + ($354))|0;
  HEAP32[$idx1>>2] = $356;
  $357 = HEAP32[$idx1>>2]|0;
  $358 = HEAP32[$szHdr1>>2]|0;
  $359 = ($357>>>0)<($358>>>0);
  if (!($359)) {
   label = 73;
   break;
  }
  $360 = HEAP32[$i>>2]|0;
  $361 = HEAP32[$3>>2]|0;
  $362 = ((($361)) + 8|0);
  $363 = HEAP16[$362>>1]|0;
  $364 = $363&65535;
  $365 = ($360|0)<($364|0);
  if (!($365)) {
   label = 73;
   break;
  }
  $366 = HEAP32[$d1>>2]|0;
  $367 = HEAP32[$1>>2]|0;
  $368 = ($366>>>0)<=($367>>>0);
  if (!($368)) {
   label = 73;
   break;
  }
 }
 if ((label|0) == 43) {
  $212 = (_sqlite3CorruptError(71763)|0);
  $213 = $212&255;
  $214 = HEAP32[$3>>2]|0;
  $215 = ((($214)) + 11|0);
  HEAP8[$215>>0] = $213;
  HEAP32[$0>>2] = 0;
  $375 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($375|0);
 }
 else if ((label|0) == 59) {
  $302 = (_sqlite3CorruptError(71792)|0);
  $303 = $302&255;
  $304 = HEAP32[$3>>2]|0;
  $305 = ((($304)) + 11|0);
  HEAP8[$305>>0] = $303;
  HEAP32[$0>>2] = 0;
  $375 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($375|0);
 }
 else if ((label|0) == 67) {
  $337 = HEAP32[$pKeyInfo>>2]|0;
  $338 = ((($337)) + 16|0);
  $339 = HEAP32[$338>>2]|0;
  $340 = (($339) + ($336)|0);
  $341 = HEAP8[$340>>0]|0;
  $342 = ($341<<24>>24)!=(0);
  if ($342) {
   $343 = HEAP32[$rc>>2]|0;
   $344 = (0 - ($343))|0;
   HEAP32[$rc>>2] = $344;
  }
  $345 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $345;
  $375 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($375|0);
 }
 else if ((label|0) == 73) {
  $369 = HEAP32[$3>>2]|0;
  $370 = ((($369)) + 14|0);
  HEAP8[$370>>0] = 1;
  $371 = HEAP32[$3>>2]|0;
  $372 = ((($371)) + 10|0);
  $373 = HEAP8[$372>>0]|0;
  $374 = $373 << 24 >> 24;
  HEAP32[$0>>2] = $374;
  $375 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($375|0);
 }
 return (0)|0;
}
function _vdbeRecordDecodeInt($serial_type,$aKey) {
 $serial_type = $serial_type|0;
 $aKey = $aKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $y = sp + 16|0;
 $x = sp;
 HEAP32[$1>>2] = $serial_type;
 HEAP32[$2>>2] = $aKey;
 $3 = HEAP32[$1>>2]|0;
 switch ($3|0) {
 case 1: case 0:  {
  $4 = HEAP32[$2>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)<(0);
  $8 = $7 << 31 >> 31;
  $9 = $0;
  $10 = $9;
  HEAP32[$10>>2] = $6;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = $8;
  break;
 }
 case 2:  {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = $15<<8;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 1|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = $16 | $20;
  $22 = ($21|0)<(0);
  $23 = $22 << 31 >> 31;
  $24 = $0;
  $25 = $24;
  HEAP32[$25>>2] = $21;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  break;
 }
 case 3:  {
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = $30<<16;
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 1|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = $35 << 8;
  $37 = $31 | $36;
  $38 = HEAP32[$2>>2]|0;
  $39 = ((($38)) + 2|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $37 | $41;
  $43 = ($42|0)<(0);
  $44 = $43 << 31 >> 31;
  $45 = $0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  break;
 }
 case 4:  {
  $49 = HEAP32[$2>>2]|0;
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 << 24;
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 1|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = $56 << 16;
  $58 = $52 | $57;
  $59 = HEAP32[$2>>2]|0;
  $60 = ((($59)) + 2|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = $61&255;
  $63 = $62 << 8;
  $64 = $58 | $63;
  $65 = HEAP32[$2>>2]|0;
  $66 = ((($65)) + 3|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $64 | $68;
  HEAP32[$y>>2] = $69;
  $70 = HEAP32[$y>>2]|0;
  $71 = ($70|0)<(0);
  $72 = $71 << 31 >> 31;
  $73 = $0;
  $74 = $73;
  HEAP32[$74>>2] = $70;
  $75 = (($73) + 4)|0;
  $76 = $75;
  HEAP32[$76>>2] = $72;
  break;
 }
 case 5:  {
  $77 = HEAP32[$2>>2]|0;
  $78 = ((($77)) + 2|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $80 << 24;
  $82 = HEAP32[$2>>2]|0;
  $83 = ((($82)) + 2|0);
  $84 = ((($83)) + 1|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = $85&255;
  $87 = $86 << 16;
  $88 = $81 | $87;
  $89 = HEAP32[$2>>2]|0;
  $90 = ((($89)) + 2|0);
  $91 = ((($90)) + 2|0);
  $92 = HEAP8[$91>>0]|0;
  $93 = $92&255;
  $94 = $93 << 8;
  $95 = $88 | $94;
  $96 = HEAP32[$2>>2]|0;
  $97 = ((($96)) + 2|0);
  $98 = ((($97)) + 3|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = $95 | $100;
  $102 = HEAP32[$2>>2]|0;
  $103 = HEAP8[$102>>0]|0;
  $104 = $103 << 24 >> 24;
  $105 = $104<<8;
  $106 = HEAP32[$2>>2]|0;
  $107 = ((($106)) + 1|0);
  $108 = HEAP8[$107>>0]|0;
  $109 = $108&255;
  $110 = $105 | $109;
  $111 = ($110|0)<(0);
  $112 = $111 << 31 >> 31;
  $113 = (___muldi3(0,1,($110|0),($112|0))|0);
  $114 = tempRet0;
  $115 = (_i64Add(($101|0),0,($113|0),($114|0))|0);
  $116 = tempRet0;
  $117 = $0;
  $118 = $117;
  HEAP32[$118>>2] = $115;
  $119 = (($117) + 4)|0;
  $120 = $119;
  HEAP32[$120>>2] = $116;
  break;
 }
 case 6:  {
  $121 = HEAP32[$2>>2]|0;
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $123 << 24;
  $125 = HEAP32[$2>>2]|0;
  $126 = ((($125)) + 1|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = $128 << 16;
  $130 = $124 | $129;
  $131 = HEAP32[$2>>2]|0;
  $132 = ((($131)) + 2|0);
  $133 = HEAP8[$132>>0]|0;
  $134 = $133&255;
  $135 = $134 << 8;
  $136 = $130 | $135;
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 3|0);
  $139 = HEAP8[$138>>0]|0;
  $140 = $139&255;
  $141 = $136 | $140;
  $142 = $x;
  $143 = $142;
  HEAP32[$143>>2] = $141;
  $144 = (($142) + 4)|0;
  $145 = $144;
  HEAP32[$145>>2] = 0;
  $146 = $x;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (($146) + 4)|0;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$2>>2]|0;
  $153 = ((($152)) + 4|0);
  $154 = HEAP8[$153>>0]|0;
  $155 = $154&255;
  $156 = $155 << 24;
  $157 = HEAP32[$2>>2]|0;
  $158 = ((($157)) + 4|0);
  $159 = ((($158)) + 1|0);
  $160 = HEAP8[$159>>0]|0;
  $161 = $160&255;
  $162 = $161 << 16;
  $163 = $156 | $162;
  $164 = HEAP32[$2>>2]|0;
  $165 = ((($164)) + 4|0);
  $166 = ((($165)) + 2|0);
  $167 = HEAP8[$166>>0]|0;
  $168 = $167&255;
  $169 = $168 << 8;
  $170 = $163 | $169;
  $171 = HEAP32[$2>>2]|0;
  $172 = ((($171)) + 4|0);
  $173 = ((($172)) + 3|0);
  $174 = HEAP8[$173>>0]|0;
  $175 = $174&255;
  $176 = $170 | $175;
  $177 = $x;
  $178 = $177;
  HEAP32[$178>>2] = $176;
  $179 = (($177) + 4)|0;
  $180 = $179;
  HEAP32[$180>>2] = $148;
  $181 = $x;
  $182 = $181;
  $183 = HEAP32[$182>>2]|0;
  $184 = (($181) + 4)|0;
  $185 = $184;
  $186 = HEAP32[$185>>2]|0;
  $187 = $0;
  $188 = $187;
  HEAP32[$188>>2] = $183;
  $189 = (($187) + 4)|0;
  $190 = $189;
  HEAP32[$190>>2] = $186;
  break;
 }
 default: {
  $191 = HEAP32[$1>>2]|0;
  $192 = (($191) - 8)|0;
  $193 = $0;
  $194 = $193;
  HEAP32[$194>>2] = $192;
  $195 = (($193) + 4)|0;
  $196 = $195;
  HEAP32[$196>>2] = 0;
 }
 }
 $197 = $0;
 $198 = $197;
 $199 = HEAP32[$198>>2]|0;
 $200 = (($197) + 4)|0;
 $201 = $200;
 $202 = HEAP32[$201>>2]|0;
 tempRet0 = ($202);
 STACKTOP = sp;return ($199|0);
}
function _btreeNext($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $idx = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $idx = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 66|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)>=(3);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = (_btreeRestoreCursorPosition($13)|0);
   $15 = $14;
  } else {
   $15 = 0;
  }
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $18;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 66|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (0)==($22|0);
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   HEAP32[$24>>2] = 1;
   HEAP32[$0>>2] = 0;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 60|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 66|0);
   HEAP8[$30>>0] = 1;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 60|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)>(0);
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 60|0);
   HEAP32[$36>>2] = 0;
   if ($34) {
    HEAP32[$0>>2] = 0;
    $122 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($122|0);
   }
  }
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 68|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = $39 << 24 >> 24;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 120|0);
 $43 = (($42) + ($40<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pPage>>2] = $44;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 68|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47 << 24 >> 24;
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 80|0);
 $51 = (($50) + ($48<<1)|0);
 $52 = HEAP16[$51>>1]|0;
 $53 = (($52) + 1)<<16>>16;
 HEAP16[$51>>1] = $53;
 $54 = $53&65535;
 HEAP32[$idx>>2] = $54;
 $55 = HEAP32[$idx>>2]|0;
 $56 = HEAP32[$pPage>>2]|0;
 $57 = ((($56)) + 18|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = ($55|0)>=($59|0);
 $61 = HEAP32[$pPage>>2]|0;
 $62 = ((($61)) + 4|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = ($63<<24>>24)!=(0);
 if (!($60)) {
  if ($64) {
   HEAP32[$0>>2] = 0;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  } else {
   $120 = HEAP32[$1>>2]|0;
   $121 = (_moveToLeftmost($120)|0);
   HEAP32[$0>>2] = $121;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
 }
 if (!($64)) {
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$pPage>>2]|0;
  $67 = ((($66)) + 5|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = (($69) + 8)|0;
  $71 = HEAP32[$pPage>>2]|0;
  $72 = ((($71)) + 56|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($70)|0);
  $75 = (_sqlite3Get4byte($74)|0);
  $76 = (_moveToChild($65,$75)|0);
  HEAP32[$rc>>2] = $76;
  $77 = HEAP32[$rc>>2]|0;
  $78 = ($77|0)!=(0);
  if ($78) {
   $79 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $79;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  } else {
   $80 = HEAP32[$1>>2]|0;
   $81 = (_moveToLeftmost($80)|0);
   HEAP32[$0>>2] = $81;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
 }
 while(1) {
  $82 = HEAP32[$1>>2]|0;
  $83 = ((($82)) + 68|0);
  $84 = HEAP8[$83>>0]|0;
  $85 = $84 << 24 >> 24;
  $86 = ($85|0)==(0);
  if ($86) {
   label = 17;
   break;
  }
  $90 = HEAP32[$1>>2]|0;
  _moveToParent($90);
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 68|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93 << 24 >> 24;
  $95 = HEAP32[$1>>2]|0;
  $96 = ((($95)) + 120|0);
  $97 = (($96) + ($94<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  HEAP32[$pPage>>2] = $98;
  $99 = HEAP32[$1>>2]|0;
  $100 = ((($99)) + 68|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = $101 << 24 >> 24;
  $103 = HEAP32[$1>>2]|0;
  $104 = ((($103)) + 80|0);
  $105 = (($104) + ($102<<1)|0);
  $106 = HEAP16[$105>>1]|0;
  $107 = $106&65535;
  $108 = HEAP32[$pPage>>2]|0;
  $109 = ((($108)) + 18|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $110&65535;
  $112 = ($107|0)>=($111|0);
  if (!($112)) {
   break;
  }
 }
 if ((label|0) == 17) {
  $87 = HEAP32[$2>>2]|0;
  HEAP32[$87>>2] = 1;
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 66|0);
  HEAP8[$89>>0] = 0;
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $113 = HEAP32[$pPage>>2]|0;
 $114 = ((($113)) + 2|0);
 $115 = HEAP8[$114>>0]|0;
 $116 = ($115<<24>>24)!=(0);
 if ($116) {
  $117 = HEAP32[$1>>2]|0;
  $118 = HEAP32[$2>>2]|0;
  $119 = (_sqlite3BtreeNext($117,$118)|0);
  HEAP32[$0>>2] = $119;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 } else {
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 return (0)|0;
}
function _moveToLeftmost($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pgno = sp + 8|0;
 $rc = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $1 = HEAP32[$rc>>2]|0;
  $2 = ($1|0)==(0);
  if (!($2)) {
   label = 5;
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 68|0);
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 120|0);
  $9 = (($8) + ($6<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pPage>>2] = $10;
  $11 = ((($10)) + 4|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = ($12<<24>>24)!=(0);
  $14 = $13 ^ 1;
  if (!($14)) {
   label = 5;
   break;
  }
  $15 = HEAP32[$pPage>>2]|0;
  $16 = ((($15)) + 56|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pPage>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20&65535;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 68|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 80|0);
  $28 = (($27) + ($25<<1)|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  $31 = $30<<1;
  $32 = HEAP32[$pPage>>2]|0;
  $33 = ((($32)) + 64|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = $37 << 8;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 68|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 80|0);
  $45 = (($44) + ($42<<1)|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46&65535;
  $48 = $47<<1;
  $49 = HEAP32[$pPage>>2]|0;
  $50 = ((($49)) + 64|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48)|0);
  $53 = ((($52)) + 1|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = $38 | $55;
  $57 = $21 & $56;
  $58 = (($17) + ($57)|0);
  $59 = (_sqlite3Get4byte($58)|0);
  HEAP32[$pgno>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  $61 = HEAP32[$pgno>>2]|0;
  $62 = (_moveToChild($60,$61)|0);
  HEAP32[$rc>>2] = $62;
 }
 if ((label|0) == 5) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return (0)|0;
}
function _btreePrevious($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $idx = 0;
 var $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pPage = sp + 4|0;
 $idx = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 66|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)>=(3);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = (_btreeRestoreCursorPosition($13)|0);
   $15 = $14;
  } else {
   $15 = 0;
  }
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $18;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 66|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (0)==($22|0);
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   HEAP32[$24>>2] = 1;
   HEAP32[$0>>2] = 0;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 60|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 66|0);
   HEAP8[$30>>0] = 1;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 60|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)<(0);
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 60|0);
   HEAP32[$36>>2] = 0;
   if ($34) {
    HEAP32[$0>>2] = 0;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
  }
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 68|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = $39 << 24 >> 24;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 120|0);
 $43 = (($42) + ($40<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pPage>>2] = $44;
 $45 = HEAP32[$pPage>>2]|0;
 $46 = ((($45)) + 4|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = ($47<<24>>24)!=(0);
 do {
  if ($48) {
   while(1) {
    $94 = HEAP32[$1>>2]|0;
    $95 = ((($94)) + 68|0);
    $96 = HEAP8[$95>>0]|0;
    $97 = $96 << 24 >> 24;
    $98 = HEAP32[$1>>2]|0;
    $99 = ((($98)) + 80|0);
    $100 = (($99) + ($97<<1)|0);
    $101 = HEAP16[$100>>1]|0;
    $102 = $101&65535;
    $103 = ($102|0)==(0);
    $104 = HEAP32[$1>>2]|0;
    $105 = ((($104)) + 68|0);
    $106 = HEAP8[$105>>0]|0;
    $107 = $106 << 24 >> 24;
    if (!($103)) {
     break;
    }
    $108 = ($107|0)==(0);
    $109 = HEAP32[$1>>2]|0;
    if ($108) {
     label = 17;
     break;
    }
    _moveToParent($109);
   }
   if ((label|0) == 17) {
    $110 = ((($109)) + 66|0);
    HEAP8[$110>>0] = 0;
    $111 = HEAP32[$2>>2]|0;
    HEAP32[$111>>2] = 1;
    HEAP32[$0>>2] = 0;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
   $112 = HEAP32[$1>>2]|0;
   $113 = ((($112)) + 80|0);
   $114 = (($113) + ($107<<1)|0);
   $115 = HEAP16[$114>>1]|0;
   $116 = (($115) + -1)<<16>>16;
   HEAP16[$114>>1] = $116;
   $117 = HEAP32[$1>>2]|0;
   $118 = ((($117)) + 68|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = $119 << 24 >> 24;
   $121 = HEAP32[$1>>2]|0;
   $122 = ((($121)) + 120|0);
   $123 = (($122) + ($120<<2)|0);
   $124 = HEAP32[$123>>2]|0;
   HEAP32[$pPage>>2] = $124;
   $125 = HEAP32[$pPage>>2]|0;
   $126 = ((($125)) + 2|0);
   $127 = HEAP8[$126>>0]|0;
   $128 = $127&255;
   $129 = ($128|0)!=(0);
   if ($129) {
    $130 = HEAP32[$pPage>>2]|0;
    $131 = ((($130)) + 4|0);
    $132 = HEAP8[$131>>0]|0;
    $133 = ($132<<24>>24)!=(0);
    if (!($133)) {
     $134 = HEAP32[$1>>2]|0;
     $135 = HEAP32[$2>>2]|0;
     $136 = (_sqlite3BtreePrevious($134,$135)|0);
     HEAP32[$rc>>2] = $136;
     break;
    }
   }
   HEAP32[$rc>>2] = 0;
  } else {
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 68|0);
   $51 = HEAP8[$50>>0]|0;
   $52 = $51 << 24 >> 24;
   $53 = HEAP32[$1>>2]|0;
   $54 = ((($53)) + 80|0);
   $55 = (($54) + ($52<<1)|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   HEAP32[$idx>>2] = $57;
   $58 = HEAP32[$1>>2]|0;
   $59 = HEAP32[$pPage>>2]|0;
   $60 = ((($59)) + 56|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$pPage>>2]|0;
   $63 = ((($62)) + 20|0);
   $64 = HEAP16[$63>>1]|0;
   $65 = $64&65535;
   $66 = HEAP32[$idx>>2]|0;
   $67 = $66<<1;
   $68 = HEAP32[$pPage>>2]|0;
   $69 = ((($68)) + 64|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + ($67)|0);
   $72 = HEAP8[$71>>0]|0;
   $73 = $72&255;
   $74 = $73 << 8;
   $75 = HEAP32[$idx>>2]|0;
   $76 = $75<<1;
   $77 = HEAP32[$pPage>>2]|0;
   $78 = ((($77)) + 64|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76)|0);
   $81 = ((($80)) + 1|0);
   $82 = HEAP8[$81>>0]|0;
   $83 = $82&255;
   $84 = $74 | $83;
   $85 = $65 & $84;
   $86 = (($61) + ($85)|0);
   $87 = (_sqlite3Get4byte($86)|0);
   $88 = (_moveToChild($58,$87)|0);
   HEAP32[$rc>>2] = $88;
   $89 = HEAP32[$rc>>2]|0;
   $90 = ($89|0)!=(0);
   if (!($90)) {
    $92 = HEAP32[$1>>2]|0;
    $93 = (_moveToRightmost($92)|0);
    HEAP32[$rc>>2] = $93;
    break;
   }
   $91 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $91;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
 } while(0);
 $137 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $137;
 $138 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($138|0);
}
function _moveToRightmost($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pgno = sp + 8|0;
 $rc = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pPage>>2] = 0;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ((($2)) + 68|0);
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 120|0);
  $8 = (($7) + ($5<<2)|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pPage>>2] = $9;
  $10 = ((($9)) + 4|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  $13 = $12 ^ 1;
  $14 = HEAP32[$pPage>>2]|0;
  if (!($13)) {
   label = 5;
   break;
  }
  $15 = ((($14)) + 5|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($17) + 8)|0;
  $19 = HEAP32[$pPage>>2]|0;
  $20 = ((($19)) + 56|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18)|0);
  $23 = (_sqlite3Get4byte($22)|0);
  HEAP32[$pgno>>2] = $23;
  $24 = HEAP32[$pPage>>2]|0;
  $25 = ((($24)) + 18|0);
  $26 = HEAP16[$25>>1]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 68|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 80|0);
  $33 = (($32) + ($30<<1)|0);
  HEAP16[$33>>1] = $26;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$pgno>>2]|0;
  $36 = (_moveToChild($34,$35)|0);
  HEAP32[$rc>>2] = $36;
  $37 = HEAP32[$rc>>2]|0;
  $38 = ($37|0)!=(0);
  if ($38) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $39 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $39;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 else if ((label|0) == 5) {
  $40 = ((($14)) + 18|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = (($42) - 1)|0;
  $44 = $43&65535;
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 68|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47 << 24 >> 24;
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 80|0);
  $51 = (($50) + ($48<<1)|0);
  HEAP16[$51>>1] = $44;
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 return (0)|0;
}
function _invalidateIncrblobCursors($pBtree,$0,$1,$isClearTable) {
 $pBtree = $pBtree|0;
 $0 = $0|0;
 $1 = $1|0;
 $isClearTable = $isClearTable|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $pBtree;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $isClearTable;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 11|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 11|0);
 HEAP8[$15>>0] = 0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$p>>2] = $20;
 while(1) {
  $21 = HEAP32[$p>>2]|0;
  $22 = ($21|0)!=(0|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 64|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $26 & 16;
  $28 = ($27|0)!=(0);
  do {
   if ($28) {
    $29 = HEAP32[$2>>2]|0;
    $30 = ((($29)) + 11|0);
    HEAP8[$30>>0] = 1;
    $31 = HEAP32[$4>>2]|0;
    $32 = ($31|0)!=(0);
    if (!($32)) {
     $33 = HEAP32[$p>>2]|0;
     $34 = ((($33)) + 16|0);
     $35 = $34;
     $36 = $35;
     $37 = HEAP32[$36>>2]|0;
     $38 = (($35) + 4)|0;
     $39 = $38;
     $40 = HEAP32[$39>>2]|0;
     $41 = $3;
     $42 = $41;
     $43 = HEAP32[$42>>2]|0;
     $44 = (($41) + 4)|0;
     $45 = $44;
     $46 = HEAP32[$45>>2]|0;
     $47 = ($37|0)==($43|0);
     $48 = ($40|0)==($46|0);
     $49 = $47 & $48;
     if (!($49)) {
      break;
     }
    }
    $50 = HEAP32[$p>>2]|0;
    $51 = ((($50)) + 66|0);
    HEAP8[$51>>0] = 0;
   }
  } while(0);
  $52 = HEAP32[$p>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$p>>2] = $54;
 }
 STACKTOP = sp;return;
}
function _fillInCell($pPage,$pCell,$pKey,$0,$1,$pData,$nData,$nZero,$pnSize) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $pData = $pData|0;
 $nData = $nData|0;
 $nZero = $nZero|0;
 $pnSize = $pnSize|0;
 var $$ = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0;
 var $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0;
 var $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0;
 var $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0;
 var $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0;
 var $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0;
 var $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0;
 var $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eType = 0, $mn = 0, $n = 0, $nHeader = 0, $nPayload = 0, $nSrc = 0, $or$cond = 0, $pBt = 0;
 var $pOvfl = 0, $pPayload = 0, $pPrior = 0, $pSrc = 0, $pToRelease = 0, $pgnoOvfl = 0, $pgnoPtrmap = 0, $rc = 0, $spaceLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp;
 $7 = sp + 80|0;
 $8 = sp + 76|0;
 $9 = sp + 72|0;
 $10 = sp + 68|0;
 $nPayload = sp + 64|0;
 $pSrc = sp + 60|0;
 $nSrc = sp + 56|0;
 $n = sp + 52|0;
 $rc = sp + 48|0;
 $spaceLeft = sp + 44|0;
 $pOvfl = sp + 40|0;
 $pToRelease = sp + 36|0;
 $pPrior = sp + 32|0;
 $pPayload = sp + 28|0;
 $pBt = sp + 24|0;
 $pgnoOvfl = sp + 20|0;
 $nHeader = sp + 16|0;
 $mn = sp + 12|0;
 $pgnoPtrmap = sp + 8|0;
 $eType = sp + 100|0;
 HEAP32[$3>>2] = $pPage;
 HEAP32[$4>>2] = $pCell;
 HEAP32[$5>>2] = $pKey;
 $11 = $6;
 $12 = $11;
 HEAP32[$12>>2] = $0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $1;
 HEAP32[$7>>2] = $pData;
 HEAP32[$8>>2] = $nData;
 HEAP32[$9>>2] = $nZero;
 HEAP32[$10>>2] = $pnSize;
 HEAP32[$pOvfl>>2] = 0;
 HEAP32[$pToRelease>>2] = 0;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 52|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pBt>>2] = $17;
 HEAP32[$pgnoOvfl>>2] = 0;
 $18 = HEAP32[$3>>2]|0;
 $19 = ((($18)) + 6|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 HEAP32[$nHeader>>2] = $21;
 $22 = HEAP32[$8>>2]|0;
 $23 = HEAP32[$9>>2]|0;
 $24 = (($22) + ($23))|0;
 HEAP32[$nPayload>>2] = $24;
 $25 = HEAP32[$3>>2]|0;
 $26 = ((($25)) + 3|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = ($27<<24>>24)!=(0);
 if ($28) {
  $29 = HEAP32[$nPayload>>2]|0;
  $30 = ($29>>>0)<(128);
  if ($30) {
   $31 = HEAP32[$nPayload>>2]|0;
   $32 = $31&255;
   $33 = HEAP32[$nHeader>>2]|0;
   $34 = HEAP32[$4>>2]|0;
   $35 = (($34) + ($33)|0);
   HEAP8[$35>>0] = $32;
   $44 = 1;
  } else {
   $36 = HEAP32[$nHeader>>2]|0;
   $37 = HEAP32[$4>>2]|0;
   $38 = (($37) + ($36)|0);
   $39 = HEAP32[$nPayload>>2]|0;
   $40 = ($39|0)<(0);
   $41 = $40 << 31 >> 31;
   $42 = (_sqlite3PutVarint($38,$39,$41)|0);
   $44 = $42;
  }
  $43 = $44&255;
  $45 = $43&255;
  $46 = HEAP32[$nHeader>>2]|0;
  $47 = (($46) + ($45))|0;
  HEAP32[$nHeader>>2] = $47;
 }
 $48 = HEAP32[$nHeader>>2]|0;
 $49 = HEAP32[$4>>2]|0;
 $50 = (($49) + ($48)|0);
 $51 = $6;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = (_sqlite3PutVarint($50,$53,$56)|0);
 $58 = HEAP32[$nHeader>>2]|0;
 $59 = (($58) + ($57))|0;
 HEAP32[$nHeader>>2] = $59;
 $60 = HEAP32[$3>>2]|0;
 $61 = ((($60)) + 2|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = ($62<<24>>24)!=(0);
 if ($63) {
  $64 = HEAP32[$7>>2]|0;
  HEAP32[$pSrc>>2] = $64;
  $65 = HEAP32[$8>>2]|0;
  HEAP32[$nSrc>>2] = $65;
  HEAP32[$8>>2] = 0;
 } else {
  $66 = $6;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($66) + 4)|0;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  HEAP32[$nPayload>>2] = $68;
  $72 = HEAP32[$5>>2]|0;
  HEAP32[$pSrc>>2] = $72;
  $73 = $6;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = (($73) + 4)|0;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  HEAP32[$nSrc>>2] = $75;
 }
 $79 = HEAP32[$nPayload>>2]|0;
 $80 = HEAP32[$3>>2]|0;
 $81 = ((($80)) + 10|0);
 $82 = HEAP16[$81>>1]|0;
 $83 = $82&65535;
 $84 = ($79|0)<=($83|0);
 if ($84) {
  $85 = HEAP32[$nHeader>>2]|0;
  $86 = HEAP32[$nPayload>>2]|0;
  $87 = (($85) + ($86))|0;
  HEAP32[$n>>2] = $87;
  $88 = HEAP32[$n>>2]|0;
  $89 = ($88|0)<(4);
  $$ = $89 ? 4 : $87;
  HEAP32[$n>>2] = $$;
  $90 = HEAP32[$n>>2]|0;
  $91 = HEAP32[$10>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$nPayload>>2]|0;
  HEAP32[$spaceLeft>>2] = $92;
  $93 = HEAP32[$4>>2]|0;
  HEAP32[$pPrior>>2] = $93;
 } else {
  $94 = HEAP32[$3>>2]|0;
  $95 = ((($94)) + 12|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96&65535;
  HEAP32[$mn>>2] = $97;
  $98 = HEAP32[$mn>>2]|0;
  $99 = HEAP32[$nPayload>>2]|0;
  $100 = HEAP32[$mn>>2]|0;
  $101 = (($99) - ($100))|0;
  $102 = HEAP32[$3>>2]|0;
  $103 = ((($102)) + 52|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = ((($104)) + 36|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (($106) - 4)|0;
  $108 = (($101>>>0) % ($107>>>0))&-1;
  $109 = (($98) + ($108))|0;
  HEAP32[$n>>2] = $109;
  $110 = HEAP32[$n>>2]|0;
  $111 = HEAP32[$3>>2]|0;
  $112 = ((($111)) + 10|0);
  $113 = HEAP16[$112>>1]|0;
  $114 = $113&65535;
  $115 = ($110|0)>($114|0);
  if ($115) {
   $116 = HEAP32[$mn>>2]|0;
   HEAP32[$n>>2] = $116;
  }
  $117 = HEAP32[$n>>2]|0;
  HEAP32[$spaceLeft>>2] = $117;
  $118 = HEAP32[$n>>2]|0;
  $119 = HEAP32[$nHeader>>2]|0;
  $120 = (($118) + ($119))|0;
  $121 = (($120) + 4)|0;
  $122 = HEAP32[$10>>2]|0;
  HEAP32[$122>>2] = $121;
  $123 = HEAP32[$nHeader>>2]|0;
  $124 = HEAP32[$n>>2]|0;
  $125 = (($123) + ($124))|0;
  $126 = HEAP32[$4>>2]|0;
  $127 = (($126) + ($125)|0);
  HEAP32[$pPrior>>2] = $127;
 }
 $128 = HEAP32[$nHeader>>2]|0;
 $129 = HEAP32[$4>>2]|0;
 $130 = (($129) + ($128)|0);
 HEAP32[$pPayload>>2] = $130;
 while(1) {
  $131 = HEAP32[$nPayload>>2]|0;
  $132 = ($131|0)>(0);
  if (!($132)) {
   label = 35;
   break;
  }
  $133 = HEAP32[$spaceLeft>>2]|0;
  $134 = ($133|0)==(0);
  if ($134) {
   $135 = HEAP32[$pgnoOvfl>>2]|0;
   HEAP32[$pgnoPtrmap>>2] = $135;
   $136 = HEAP32[$pBt>>2]|0;
   $137 = ((($136)) + 17|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = ($138<<24>>24)!=(0);
   if ($139) {
    while(1) {
     $140 = HEAP32[$pgnoOvfl>>2]|0;
     $141 = (($140) + 1)|0;
     HEAP32[$pgnoOvfl>>2] = $141;
     $142 = HEAP32[$pBt>>2]|0;
     $143 = HEAP32[$pgnoOvfl>>2]|0;
     $144 = (_ptrmapPageno($142,$143)|0);
     $145 = HEAP32[$pgnoOvfl>>2]|0;
     $146 = ($144|0)==($145|0);
     if ($146) {
      continue;
     }
     $147 = HEAP32[$pgnoOvfl>>2]|0;
     $148 = HEAP32[832>>2]|0;
     $149 = HEAP32[$pBt>>2]|0;
     $150 = ((($149)) + 32|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = (($148>>>0) / ($151>>>0))&-1;
     $153 = (($152) + 1)|0;
     $154 = ($147|0)==($153|0);
     if (!($154)) {
      break;
     }
    }
   }
   $155 = HEAP32[$pBt>>2]|0;
   $156 = HEAP32[$pgnoOvfl>>2]|0;
   $157 = (_allocateBtreePage($155,$pOvfl,$pgnoOvfl,$156,0)|0);
   HEAP32[$rc>>2] = $157;
   $158 = HEAP32[$pBt>>2]|0;
   $159 = ((($158)) + 17|0);
   $160 = HEAP8[$159>>0]|0;
   $161 = $160&255;
   $162 = ($161|0)!=(0);
   $163 = HEAP32[$rc>>2]|0;
   $164 = ($163|0)==(0);
   $or$cond = $162 & $164;
   if ($or$cond) {
    $165 = HEAP32[$pgnoPtrmap>>2]|0;
    $166 = ($165|0)!=(0);
    $167 = $166 ? 4 : 3;
    $168 = $167&255;
    HEAP8[$eType>>0] = $168;
    $169 = HEAP32[$pBt>>2]|0;
    $170 = HEAP32[$pgnoOvfl>>2]|0;
    $171 = HEAP8[$eType>>0]|0;
    $172 = HEAP32[$pgnoPtrmap>>2]|0;
    _ptrmapPut($169,$170,$171,$172,$rc);
    $173 = HEAP32[$rc>>2]|0;
    $174 = ($173|0)!=(0);
    if ($174) {
     $175 = HEAP32[$pOvfl>>2]|0;
     _releasePage($175);
    }
   }
   $176 = HEAP32[$rc>>2]|0;
   $177 = ($176|0)!=(0);
   if ($177) {
    label = 24;
    break;
   }
   $180 = HEAP32[$pPrior>>2]|0;
   $181 = HEAP32[$pgnoOvfl>>2]|0;
   _sqlite3Put4byte($180,$181);
   $182 = HEAP32[$pToRelease>>2]|0;
   _releasePage($182);
   $183 = HEAP32[$pOvfl>>2]|0;
   HEAP32[$pToRelease>>2] = $183;
   $184 = HEAP32[$pOvfl>>2]|0;
   $185 = ((($184)) + 56|0);
   $186 = HEAP32[$185>>2]|0;
   HEAP32[$pPrior>>2] = $186;
   $187 = HEAP32[$pPrior>>2]|0;
   _sqlite3Put4byte($187,0);
   $188 = HEAP32[$pOvfl>>2]|0;
   $189 = ((($188)) + 56|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = ((($190)) + 4|0);
   HEAP32[$pPayload>>2] = $191;
   $192 = HEAP32[$pBt>>2]|0;
   $193 = ((($192)) + 36|0);
   $194 = HEAP32[$193>>2]|0;
   $195 = (($194) - 4)|0;
   HEAP32[$spaceLeft>>2] = $195;
  }
  $196 = HEAP32[$nPayload>>2]|0;
  HEAP32[$n>>2] = $196;
  $197 = HEAP32[$n>>2]|0;
  $198 = HEAP32[$spaceLeft>>2]|0;
  $199 = ($197|0)>($198|0);
  if ($199) {
   $200 = HEAP32[$spaceLeft>>2]|0;
   HEAP32[$n>>2] = $200;
  }
  $201 = HEAP32[$nSrc>>2]|0;
  $202 = ($201|0)>(0);
  if ($202) {
   $203 = HEAP32[$n>>2]|0;
   $204 = HEAP32[$nSrc>>2]|0;
   $205 = ($203|0)>($204|0);
   if ($205) {
    $206 = HEAP32[$nSrc>>2]|0;
    HEAP32[$n>>2] = $206;
   }
   $207 = HEAP32[$pPayload>>2]|0;
   $208 = HEAP32[$pSrc>>2]|0;
   $209 = HEAP32[$n>>2]|0;
   _memcpy(($207|0),($208|0),($209|0))|0;
  } else {
   $210 = HEAP32[$pPayload>>2]|0;
   $211 = HEAP32[$n>>2]|0;
   _memset(($210|0),0,($211|0))|0;
  }
  $212 = HEAP32[$n>>2]|0;
  $213 = HEAP32[$nPayload>>2]|0;
  $214 = (($213) - ($212))|0;
  HEAP32[$nPayload>>2] = $214;
  $215 = HEAP32[$n>>2]|0;
  $216 = HEAP32[$pPayload>>2]|0;
  $217 = (($216) + ($215)|0);
  HEAP32[$pPayload>>2] = $217;
  $218 = HEAP32[$n>>2]|0;
  $219 = HEAP32[$pSrc>>2]|0;
  $220 = (($219) + ($218)|0);
  HEAP32[$pSrc>>2] = $220;
  $221 = HEAP32[$n>>2]|0;
  $222 = HEAP32[$nSrc>>2]|0;
  $223 = (($222) - ($221))|0;
  HEAP32[$nSrc>>2] = $223;
  $224 = HEAP32[$n>>2]|0;
  $225 = HEAP32[$spaceLeft>>2]|0;
  $226 = (($225) - ($224))|0;
  HEAP32[$spaceLeft>>2] = $226;
  $227 = HEAP32[$nSrc>>2]|0;
  $228 = ($227|0)==(0);
  if (!($228)) {
   continue;
  }
  $229 = HEAP32[$8>>2]|0;
  HEAP32[$nSrc>>2] = $229;
  $230 = HEAP32[$7>>2]|0;
  HEAP32[$pSrc>>2] = $230;
 }
 if ((label|0) == 24) {
  $178 = HEAP32[$pToRelease>>2]|0;
  _releasePage($178);
  $179 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $179;
  $232 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($232|0);
 }
 else if ((label|0) == 35) {
  $231 = HEAP32[$pToRelease>>2]|0;
  _releasePage($231);
  HEAP32[$2>>2] = 0;
  $232 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($232|0);
 }
 return (0)|0;
}
function _clearCell($pPage,$pCell,$pnSize) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pnSize = $pnSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iNext = 0, $info = 0, $nOvfl = 0, $ovflPageSize = 0, $ovflPgno = 0, $pBt = 0, $pOvfl = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $pBt = sp + 48|0;
 $info = sp;
 $ovflPgno = sp + 44|0;
 $rc = sp + 40|0;
 $nOvfl = sp + 36|0;
 $ovflPageSize = sp + 32|0;
 $iNext = sp + 28|0;
 $pOvfl = sp + 24|0;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $pnSize;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 80|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 FUNCTION_TABLE_viii[$9 & 255]($10,$11,$info);
 $12 = ((($info)) + 18|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = HEAP32[$3>>2]|0;
 HEAP16[$14>>1] = $13;
 $15 = ((($info)) + 16|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = ((($info)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($17|0)==($19|0);
 if ($20) {
  HEAP32[$0>>2] = 0;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($info)) + 18|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = $23&65535;
 $25 = (($21) + ($24)|0);
 $26 = ((($25)) + -1|0);
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 56|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP16[$31>>1]|0;
 $33 = $32&65535;
 $34 = (($29) + ($33)|0);
 $35 = ($26>>>0)>($34>>>0);
 if ($35) {
  $36 = (_sqlite3CorruptError(61571)|0);
  HEAP32[$0>>2] = $36;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = ((($info)) + 18|0);
 $39 = HEAP16[$38>>1]|0;
 $40 = $39&65535;
 $41 = (($37) + ($40)|0);
 $42 = ((($41)) + -4|0);
 $43 = (_sqlite3Get4byte($42)|0);
 HEAP32[$ovflPgno>>2] = $43;
 $44 = HEAP32[$pBt>>2]|0;
 $45 = ((($44)) + 36|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) - 4)|0;
 HEAP32[$ovflPageSize>>2] = $47;
 $48 = ((($info)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($info)) + 16|0);
 $51 = HEAP16[$50>>1]|0;
 $52 = $51&65535;
 $53 = (($49) - ($52))|0;
 $54 = HEAP32[$ovflPageSize>>2]|0;
 $55 = (($53) + ($54))|0;
 $56 = (($55) - 1)|0;
 $57 = HEAP32[$ovflPageSize>>2]|0;
 $58 = (($56>>>0) / ($57>>>0))&-1;
 HEAP32[$nOvfl>>2] = $58;
 while(1) {
  $59 = HEAP32[$nOvfl>>2]|0;
  $60 = (($59) + -1)|0;
  HEAP32[$nOvfl>>2] = $60;
  $61 = ($59|0)!=(0);
  if (!($61)) {
   label = 23;
   break;
  }
  HEAP32[$iNext>>2] = 0;
  HEAP32[$pOvfl>>2] = 0;
  $62 = HEAP32[$ovflPgno>>2]|0;
  $63 = ($62>>>0)<(2);
  if ($63) {
   label = 9;
   break;
  }
  $64 = HEAP32[$ovflPgno>>2]|0;
  $65 = HEAP32[$pBt>>2]|0;
  $66 = (_btreePagecount($65)|0);
  $67 = ($64>>>0)>($66>>>0);
  if ($67) {
   label = 9;
   break;
  }
  $69 = HEAP32[$nOvfl>>2]|0;
  $70 = ($69|0)!=(0);
  if ($70) {
   $71 = HEAP32[$pBt>>2]|0;
   $72 = HEAP32[$ovflPgno>>2]|0;
   $73 = (_getOverflowPage($71,$72,$pOvfl,$iNext)|0);
   HEAP32[$rc>>2] = $73;
   $74 = HEAP32[$rc>>2]|0;
   $75 = ($74|0)!=(0);
   if ($75) {
    label = 12;
    break;
   }
  }
  $77 = HEAP32[$pOvfl>>2]|0;
  $78 = ($77|0)!=(0|0);
  if ($78) {
   label = 15;
  } else {
   $79 = HEAP32[$pBt>>2]|0;
   $80 = HEAP32[$ovflPgno>>2]|0;
   $81 = (_btreePageLookup($79,$80)|0);
   HEAP32[$pOvfl>>2] = $81;
   $82 = ($81|0)!=(0|0);
   if ($82) {
    label = 15;
   } else {
    label = 17;
   }
  }
  if ((label|0) == 15) {
   label = 0;
   $83 = HEAP32[$pOvfl>>2]|0;
   $84 = ((($83)) + 72|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = (_sqlite3PagerPageRefcount($85)|0);
   $87 = ($86|0)!=(1);
   if ($87) {
    $88 = (_sqlite3CorruptError(61607)|0);
    HEAP32[$rc>>2] = $88;
   } else {
    label = 17;
   }
  }
  if ((label|0) == 17) {
   label = 0;
   $89 = HEAP32[$pBt>>2]|0;
   $90 = HEAP32[$pOvfl>>2]|0;
   $91 = HEAP32[$ovflPgno>>2]|0;
   $92 = (_freePage2($89,$90,$91)|0);
   HEAP32[$rc>>2] = $92;
  }
  $93 = HEAP32[$pOvfl>>2]|0;
  $94 = ($93|0)!=(0|0);
  if ($94) {
   $95 = HEAP32[$pOvfl>>2]|0;
   $96 = ((($95)) + 72|0);
   $97 = HEAP32[$96>>2]|0;
   _sqlite3PagerUnref($97);
  }
  $98 = HEAP32[$rc>>2]|0;
  $99 = ($98|0)!=(0);
  if ($99) {
   label = 21;
   break;
  }
  $101 = HEAP32[$iNext>>2]|0;
  HEAP32[$ovflPgno>>2] = $101;
 }
 if ((label|0) == 9) {
  $68 = (_sqlite3CorruptError(61587)|0);
  HEAP32[$0>>2] = $68;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 else if ((label|0) == 12) {
  $76 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $76;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 else if ((label|0) == 21) {
  $100 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $100;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 else if ((label|0) == 23) {
  HEAP32[$0>>2] = 0;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 return (0)|0;
}
function _dropCell($pPage,$idx,$sz,$pRC) {
 $pPage = $pPage|0;
 $idx = $idx|0;
 $sz = $sz|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $data = 0, $hdr = 0, $pc = 0, $ptr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pc = sp + 16|0;
 $data = sp + 12|0;
 $ptr = sp + 8|0;
 $rc = sp + 4|0;
 $hdr = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $sz;
 HEAP32[$3>>2] = $pRC;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$data>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = $10<<1;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 64|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11)|0);
 HEAP32[$ptr>>2] = $15;
 $16 = HEAP32[$ptr>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 << 8;
 $20 = HEAP32[$ptr>>2]|0;
 $21 = ((($20)) + 1|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $19 | $23;
 HEAP32[$pc>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 5|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 HEAP32[$hdr>>2] = $28;
 $29 = HEAP32[$pc>>2]|0;
 $30 = HEAP32[$hdr>>2]|0;
 $31 = (($30) + 5)|0;
 $32 = HEAP32[$data>>2]|0;
 $33 = (($32) + ($31)|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = $35 << 8;
 $37 = HEAP32[$hdr>>2]|0;
 $38 = (($37) + 5)|0;
 $39 = HEAP32[$data>>2]|0;
 $40 = (($39) + ($38)|0);
 $41 = ((($40)) + 1|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = $36 | $43;
 $45 = ($29>>>0)<($44>>>0);
 if (!($45)) {
  $46 = HEAP32[$pc>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = (($46) + ($47))|0;
  $49 = HEAP32[$0>>2]|0;
  $50 = ((($49)) + 52|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ((($51)) + 36|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($48>>>0)>($53>>>0);
  if (!($54)) {
   $57 = HEAP32[$0>>2]|0;
   $58 = HEAP32[$pc>>2]|0;
   $59 = $58&65535;
   $60 = HEAP32[$2>>2]|0;
   $61 = $60&65535;
   $62 = (_freeSpace($57,$59,$61)|0);
   HEAP32[$rc>>2] = $62;
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)!=(0);
   if ($64) {
    $65 = HEAP32[$rc>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    HEAP32[$66>>2] = $65;
    STACKTOP = sp;return;
   }
   $67 = HEAP32[$0>>2]|0;
   $68 = ((($67)) + 18|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = (($69) + -1)<<16>>16;
   HEAP16[$68>>1] = $70;
   $71 = HEAP32[$0>>2]|0;
   $72 = ((($71)) + 18|0);
   $73 = HEAP16[$72>>1]|0;
   $74 = $73&65535;
   $75 = ($74|0)==(0);
   if ($75) {
    $76 = HEAP32[$hdr>>2]|0;
    $77 = (($76) + 1)|0;
    $78 = HEAP32[$data>>2]|0;
    $79 = (($78) + ($77)|0);
    ;HEAP8[$79>>0]=0|0;HEAP8[$79+1>>0]=0|0;HEAP8[$79+2>>0]=0|0;HEAP8[$79+3>>0]=0|0;
    $80 = HEAP32[$hdr>>2]|0;
    $81 = (($80) + 7)|0;
    $82 = HEAP32[$data>>2]|0;
    $83 = (($82) + ($81)|0);
    HEAP8[$83>>0] = 0;
    $84 = HEAP32[$0>>2]|0;
    $85 = ((($84)) + 52|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = ((($86)) + 36|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = $88 >>> 8;
    $90 = $89&255;
    $91 = HEAP32[$hdr>>2]|0;
    $92 = (($91) + 5)|0;
    $93 = HEAP32[$data>>2]|0;
    $94 = (($93) + ($92)|0);
    HEAP8[$94>>0] = $90;
    $95 = HEAP32[$0>>2]|0;
    $96 = ((($95)) + 52|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ((($97)) + 36|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = $99&255;
    $101 = HEAP32[$hdr>>2]|0;
    $102 = (($101) + 5)|0;
    $103 = HEAP32[$data>>2]|0;
    $104 = (($103) + ($102)|0);
    $105 = ((($104)) + 1|0);
    HEAP8[$105>>0] = $100;
    $106 = HEAP32[$0>>2]|0;
    $107 = ((($106)) + 52|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = ((($108)) + 36|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$0>>2]|0;
    $112 = ((($111)) + 5|0);
    $113 = HEAP8[$112>>0]|0;
    $114 = $113&255;
    $115 = (($110) - ($114))|0;
    $116 = HEAP32[$0>>2]|0;
    $117 = ((($116)) + 6|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = $118&255;
    $120 = (($115) - ($119))|0;
    $121 = (($120) - 8)|0;
    $122 = $121&65535;
    $123 = HEAP32[$0>>2]|0;
    $124 = ((($123)) + 16|0);
    HEAP16[$124>>1] = $122;
    STACKTOP = sp;return;
   } else {
    $125 = HEAP32[$ptr>>2]|0;
    $126 = HEAP32[$ptr>>2]|0;
    $127 = ((($126)) + 2|0);
    $128 = HEAP32[$0>>2]|0;
    $129 = ((($128)) + 18|0);
    $130 = HEAP16[$129>>1]|0;
    $131 = $130&65535;
    $132 = HEAP32[$1>>2]|0;
    $133 = (($131) - ($132))|0;
    $134 = $133<<1;
    _memmove(($125|0),($127|0),($134|0))|0;
    $135 = HEAP32[$0>>2]|0;
    $136 = ((($135)) + 18|0);
    $137 = HEAP16[$136>>1]|0;
    $138 = $137&65535;
    $139 = $138 >> 8;
    $140 = $139&255;
    $141 = HEAP32[$hdr>>2]|0;
    $142 = (($141) + 3)|0;
    $143 = HEAP32[$data>>2]|0;
    $144 = (($143) + ($142)|0);
    HEAP8[$144>>0] = $140;
    $145 = HEAP32[$0>>2]|0;
    $146 = ((($145)) + 18|0);
    $147 = HEAP16[$146>>1]|0;
    $148 = $147&255;
    $149 = HEAP32[$hdr>>2]|0;
    $150 = (($149) + 3)|0;
    $151 = HEAP32[$data>>2]|0;
    $152 = (($151) + ($150)|0);
    $153 = ((($152)) + 1|0);
    HEAP8[$153>>0] = $148;
    $154 = HEAP32[$0>>2]|0;
    $155 = ((($154)) + 16|0);
    $156 = HEAP16[$155>>1]|0;
    $157 = $156&65535;
    $158 = (($157) + 2)|0;
    $159 = $158&65535;
    HEAP16[$155>>1] = $159;
    STACKTOP = sp;return;
   }
  }
 }
 $55 = (_sqlite3CorruptError(61841)|0);
 $56 = HEAP32[$3>>2]|0;
 HEAP32[$56>>2] = $55;
 STACKTOP = sp;return;
}
function _insertCell($pPage,$i,$pCell,$sz,$pTemp,$iChild,$pRC) {
 $pPage = $pPage|0;
 $i = $i|0;
 $pCell = $pCell|0;
 $sz = $sz|0;
 $pTemp = $pTemp|0;
 $iChild = $iChild|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $idx = 0, $j = 0, $pIns = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $idx = sp + 16|0;
 $j = sp + 12|0;
 $data = sp + 8|0;
 $pIns = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $sz;
 HEAP32[$4>>2] = $pTemp;
 HEAP32[$5>>2] = $iChild;
 HEAP32[$6>>2] = $pRC;
 HEAP32[$idx>>2] = 0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + 2)|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19&65535;
  $21 = ($16|0)>($20|0);
  if (!($21)) {
   $48 = HEAP32[$0>>2]|0;
   $49 = ((($48)) + 72|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_sqlite3PagerWrite($50)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)!=(0);
   if ($53) {
    $54 = HEAP32[$rc>>2]|0;
    $55 = HEAP32[$6>>2]|0;
    HEAP32[$55>>2] = $54;
    STACKTOP = sp;return;
   }
   $56 = HEAP32[$0>>2]|0;
   $57 = ((($56)) + 56|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$data>>2] = $58;
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$3>>2]|0;
   $61 = (_allocateSpace($59,$60,$idx)|0);
   HEAP32[$rc>>2] = $61;
   $62 = HEAP32[$rc>>2]|0;
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = HEAP32[$rc>>2]|0;
    $65 = HEAP32[$6>>2]|0;
    HEAP32[$65>>2] = $64;
    STACKTOP = sp;return;
   }
   $66 = HEAP32[$3>>2]|0;
   $67 = (2 + ($66))|0;
   $68 = $67&65535;
   $69 = $68&65535;
   $70 = HEAP32[$0>>2]|0;
   $71 = ((($70)) + 16|0);
   $72 = HEAP16[$71>>1]|0;
   $73 = $72&65535;
   $74 = (($73) - ($69))|0;
   $75 = $74&65535;
   HEAP16[$71>>1] = $75;
   $76 = HEAP32[$idx>>2]|0;
   $77 = HEAP32[$data>>2]|0;
   $78 = (($77) + ($76)|0);
   $79 = HEAP32[$2>>2]|0;
   $80 = HEAP32[$3>>2]|0;
   _memcpy(($78|0),($79|0),($80|0))|0;
   $81 = HEAP32[$5>>2]|0;
   $82 = ($81|0)!=(0);
   if ($82) {
    $83 = HEAP32[$idx>>2]|0;
    $84 = HEAP32[$data>>2]|0;
    $85 = (($84) + ($83)|0);
    $86 = HEAP32[$5>>2]|0;
    _sqlite3Put4byte($85,$86);
   }
   $87 = HEAP32[$0>>2]|0;
   $88 = ((($87)) + 64|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = HEAP32[$1>>2]|0;
   $91 = $90<<1;
   $92 = (($89) + ($91)|0);
   HEAP32[$pIns>>2] = $92;
   $93 = HEAP32[$pIns>>2]|0;
   $94 = ((($93)) + 2|0);
   $95 = HEAP32[$pIns>>2]|0;
   $96 = HEAP32[$0>>2]|0;
   $97 = ((($96)) + 18|0);
   $98 = HEAP16[$97>>1]|0;
   $99 = $98&65535;
   $100 = HEAP32[$1>>2]|0;
   $101 = (($99) - ($100))|0;
   $102 = $101<<1;
   _memmove(($94|0),($95|0),($102|0))|0;
   $103 = HEAP32[$idx>>2]|0;
   $104 = $103 >> 8;
   $105 = $104&255;
   $106 = HEAP32[$pIns>>2]|0;
   HEAP8[$106>>0] = $105;
   $107 = HEAP32[$idx>>2]|0;
   $108 = $107&255;
   $109 = HEAP32[$pIns>>2]|0;
   $110 = ((($109)) + 1|0);
   HEAP8[$110>>0] = $108;
   $111 = HEAP32[$0>>2]|0;
   $112 = ((($111)) + 18|0);
   $113 = HEAP16[$112>>1]|0;
   $114 = (($113) + 1)<<16>>16;
   HEAP16[$112>>1] = $114;
   $115 = HEAP32[$0>>2]|0;
   $116 = ((($115)) + 5|0);
   $117 = HEAP8[$116>>0]|0;
   $118 = $117&255;
   $119 = (($118) + 4)|0;
   $120 = HEAP32[$data>>2]|0;
   $121 = (($120) + ($119)|0);
   $122 = HEAP8[$121>>0]|0;
   $123 = (($122) + 1)<<24>>24;
   HEAP8[$121>>0] = $123;
   $124 = $123&255;
   $125 = ($124|0)==(0);
   if ($125) {
    $126 = HEAP32[$0>>2]|0;
    $127 = ((($126)) + 5|0);
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = (($129) + 3)|0;
    $131 = HEAP32[$data>>2]|0;
    $132 = (($131) + ($130)|0);
    $133 = HEAP8[$132>>0]|0;
    $134 = (($133) + 1)<<24>>24;
    HEAP8[$132>>0] = $134;
   }
   $135 = HEAP32[$0>>2]|0;
   $136 = ((($135)) + 52|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = ((($137)) + 17|0);
   $139 = HEAP8[$138>>0]|0;
   $140 = ($139<<24>>24)!=(0);
   if (!($140)) {
    STACKTOP = sp;return;
   }
   $141 = HEAP32[$0>>2]|0;
   $142 = HEAP32[$2>>2]|0;
   $143 = HEAP32[$6>>2]|0;
   _ptrmapPutOvflPtr($141,$142,$143);
   STACKTOP = sp;return;
  }
 }
 $22 = HEAP32[$4>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$4>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$3>>2]|0;
  _memcpy(($24|0),($25|0),($26|0))|0;
  $27 = HEAP32[$4>>2]|0;
  HEAP32[$2>>2] = $27;
 }
 $28 = HEAP32[$5>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  _sqlite3Put4byte($30,$31);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 1|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = (($34) + 1)<<24>>24;
 HEAP8[$33>>0] = $35;
 $36 = $34&255;
 HEAP32[$j>>2] = $36;
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$j>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 32|0);
 $41 = (($40) + ($38<<2)|0);
 HEAP32[$41>>2] = $37;
 $42 = HEAP32[$1>>2]|0;
 $43 = $42&65535;
 $44 = HEAP32[$j>>2]|0;
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 22|0);
 $47 = (($46) + ($44<<1)|0);
 HEAP16[$47>>1] = $43;
 STACKTOP = sp;return;
}
function _balance($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBalanceQuickSpace = 0, $iIdx = 0, $iPage = 0, $nMin = 0, $pFree = 0, $pPage = 0, $pParent = 0, $pSpace = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 32|0;
 $rc = sp + 28|0;
 $nMin = sp + 24|0;
 $aBalanceQuickSpace = sp + 36|0;
 $pFree = sp + 20|0;
 $iPage = sp + 16|0;
 $pPage = sp + 12|0;
 $pParent = sp + 8|0;
 $iIdx = sp + 4|0;
 $pSpace = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<1;
 $7 = (($6>>>0) / 3)&-1;
 HEAP32[$nMin>>2] = $7;
 HEAP32[$pFree>>2] = 0;
 while(1) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 68|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  HEAP32[$iPage>>2] = $11;
  $12 = HEAP32[$iPage>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pPage>>2] = $16;
  $17 = HEAP32[$iPage>>2]|0;
  $18 = ($17|0)==(0);
  $19 = HEAP32[$pPage>>2]|0;
  $20 = ((($19)) + 1|0);
  $21 = HEAP8[$20>>0]|0;
  if ($18) {
   $22 = ($21<<24>>24)!=(0);
   if (!($22)) {
    break;
   }
   $23 = HEAP32[$pPage>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 120|0);
   $26 = ((($25)) + 4|0);
   $27 = (_balance_deeper($23,$26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 68|0);
    HEAP8[$31>>0] = 1;
    $32 = HEAP32[$0>>2]|0;
    $33 = ((($32)) + 80|0);
    HEAP16[$33>>1] = 0;
    $34 = HEAP32[$0>>2]|0;
    $35 = ((($34)) + 80|0);
    $36 = ((($35)) + 2|0);
    HEAP16[$36>>1] = 0;
   }
  } else {
   $37 = $21&255;
   $38 = ($37|0)==(0);
   if ($38) {
    $39 = HEAP32[$pPage>>2]|0;
    $40 = ((($39)) + 16|0);
    $41 = HEAP16[$40>>1]|0;
    $42 = $41&65535;
    $43 = HEAP32[$nMin>>2]|0;
    $44 = ($42|0)<=($43|0);
    if ($44) {
     break;
    }
   }
   $45 = HEAP32[$iPage>>2]|0;
   $46 = (($45) - 1)|0;
   $47 = HEAP32[$0>>2]|0;
   $48 = ((($47)) + 120|0);
   $49 = (($48) + ($46<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pParent>>2] = $50;
   $51 = HEAP32[$iPage>>2]|0;
   $52 = (($51) - 1)|0;
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 80|0);
   $55 = (($54) + ($52<<1)|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   HEAP32[$iIdx>>2] = $57;
   $58 = HEAP32[$pParent>>2]|0;
   $59 = ((($58)) + 72|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = (_sqlite3PagerWrite($60)|0);
   HEAP32[$rc>>2] = $61;
   $62 = HEAP32[$rc>>2]|0;
   $63 = ($62|0)==(0);
   do {
    if ($63) {
     $64 = HEAP32[$pPage>>2]|0;
     $65 = ((($64)) + 3|0);
     $66 = HEAP8[$65>>0]|0;
     $67 = $66&255;
     $68 = ($67|0)!=(0);
     if ($68) {
      $69 = HEAP32[$pPage>>2]|0;
      $70 = ((($69)) + 1|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = ($72|0)==(1);
      if ($73) {
       $74 = HEAP32[$pPage>>2]|0;
       $75 = ((($74)) + 22|0);
       $76 = HEAP16[$75>>1]|0;
       $77 = $76&65535;
       $78 = HEAP32[$pPage>>2]|0;
       $79 = ((($78)) + 18|0);
       $80 = HEAP16[$79>>1]|0;
       $81 = $80&65535;
       $82 = ($77|0)==($81|0);
       if ($82) {
        $83 = HEAP32[$pParent>>2]|0;
        $84 = ((($83)) + 84|0);
        $85 = HEAP32[$84>>2]|0;
        $86 = ($85|0)!=(1);
        if ($86) {
         $87 = HEAP32[$pParent>>2]|0;
         $88 = ((($87)) + 18|0);
         $89 = HEAP16[$88>>1]|0;
         $90 = $89&65535;
         $91 = HEAP32[$iIdx>>2]|0;
         $92 = ($90|0)==($91|0);
         if ($92) {
          $93 = HEAP32[$pParent>>2]|0;
          $94 = HEAP32[$pPage>>2]|0;
          $95 = (_balance_quick($93,$94,$aBalanceQuickSpace)|0);
          HEAP32[$rc>>2] = $95;
          break;
         }
        }
       }
      }
     }
     $96 = HEAP32[$0>>2]|0;
     $97 = ((($96)) + 4|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = ((($98)) + 32|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = (_sqlite3PageMalloc($100)|0);
     HEAP32[$pSpace>>2] = $101;
     $102 = HEAP32[$pParent>>2]|0;
     $103 = HEAP32[$iIdx>>2]|0;
     $104 = HEAP32[$pSpace>>2]|0;
     $105 = HEAP32[$iPage>>2]|0;
     $106 = ($105|0)==(1);
     $107 = $106&1;
     $108 = HEAP32[$0>>2]|0;
     $109 = ((($108)) + 67|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     $112 = $111 & 1;
     $113 = (_balance_nonroot($102,$103,$104,$107,$112)|0);
     HEAP32[$rc>>2] = $113;
     $114 = HEAP32[$pFree>>2]|0;
     $115 = ($114|0)!=(0|0);
     if ($115) {
      $116 = HEAP32[$pFree>>2]|0;
      _sqlite3PageFree($116);
     }
     $117 = HEAP32[$pSpace>>2]|0;
     HEAP32[$pFree>>2] = $117;
    }
   } while(0);
   $118 = HEAP32[$pPage>>2]|0;
   $119 = ((($118)) + 1|0);
   HEAP8[$119>>0] = 0;
   $120 = HEAP32[$pPage>>2]|0;
   _releasePage($120);
   $121 = HEAP32[$0>>2]|0;
   $122 = ((($121)) + 68|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = (($123) + -1)<<24>>24;
   HEAP8[$122>>0] = $124;
  }
  $125 = HEAP32[$rc>>2]|0;
  $126 = ($125|0)==(0);
  if (!($126)) {
   break;
  }
 }
 $127 = HEAP32[$pFree>>2]|0;
 $128 = ($127|0)!=(0|0);
 if (!($128)) {
  $130 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($130|0);
 }
 $129 = HEAP32[$pFree>>2]|0;
 _sqlite3PageFree($129);
 $130 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($130|0);
}
function _btreePageLookup($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3PagerLookup($4,$5)|0);
 HEAP32[$pDbPage>>2] = $6;
 $7 = HEAP32[$pDbPage>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$pDbPage>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = (_btreePageFromDbPage($9,$10,$11)|0);
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _freePage2($pBt,$pMemPage,$iPage) {
 $pBt = $pBt|0;
 $pMemPage = $pMemPage|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iTrunk = 0, $nFree = 0, $nLeaf = 0, $pPage = 0, $pPage1 = 0, $pTrunk = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pTrunk = sp + 24|0;
 $iTrunk = sp + 20|0;
 $pPage1 = sp + 16|0;
 $pPage = sp + 12|0;
 $rc = sp + 8|0;
 $nFree = sp + 4|0;
 $nLeaf = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pMemPage;
 HEAP32[$3>>2] = $iPage;
 HEAP32[$pTrunk>>2] = 0;
 HEAP32[$iTrunk>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pPage1>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7>>>0)<(2);
 if ($8) {
  $9 = (_sqlite3CorruptError(61425)|0);
  HEAP32[$0>>2] = $9;
  $166 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($166|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  HEAP32[$pPage>>2] = $12;
  $13 = HEAP32[$pPage>>2]|0;
  $14 = ((($13)) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3PagerRef($15);
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = (_btreePageLookup($16,$17)|0);
  HEAP32[$pPage>>2] = $18;
 }
 $19 = HEAP32[$pPage1>>2]|0;
 $20 = ((($19)) + 72|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (_sqlite3PagerWrite($21)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 do {
  if (!($24)) {
   $25 = HEAP32[$pPage1>>2]|0;
   $26 = ((($25)) + 56|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ((($27)) + 36|0);
   $29 = (_sqlite3Get4byte($28)|0);
   HEAP32[$nFree>>2] = $29;
   $30 = HEAP32[$pPage1>>2]|0;
   $31 = ((($30)) + 56|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ((($32)) + 36|0);
   $34 = HEAP32[$nFree>>2]|0;
   $35 = (($34) + 1)|0;
   _sqlite3Put4byte($33,$35);
   $36 = HEAP32[$1>>2]|0;
   $37 = ((($36)) + 22|0);
   $38 = HEAP16[$37>>1]|0;
   $39 = $38&65535;
   $40 = $39 & 4;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$pPage>>2]|0;
    $43 = ($42|0)!=(0|0);
    if (!($43)) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$3>>2]|0;
     $46 = (_btreeGetPage($44,$45,$pPage,0)|0);
     HEAP32[$rc>>2] = $46;
     $47 = ($46|0)!=(0);
     if ($47) {
      break;
     }
    }
    $48 = HEAP32[$pPage>>2]|0;
    $49 = ((($48)) + 72|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (_sqlite3PagerWrite($50)|0);
    HEAP32[$rc>>2] = $51;
    $52 = ($51|0)!=(0);
    if ($52) {
     break;
    }
    $53 = HEAP32[$pPage>>2]|0;
    $54 = ((($53)) + 56|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = HEAP32[$pPage>>2]|0;
    $57 = ((($56)) + 52|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = ((($58)) + 32|0);
    $60 = HEAP32[$59>>2]|0;
    _memset(($55|0),0,($60|0))|0;
   }
   $61 = HEAP32[$1>>2]|0;
   $62 = ((($61)) + 17|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = ($63<<24>>24)!=(0);
   if ($64) {
    $65 = HEAP32[$1>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    _ptrmapPut($65,$66,2,0,$rc);
    $67 = HEAP32[$rc>>2]|0;
    $68 = ($67|0)!=(0);
    if ($68) {
     break;
    }
   }
   $69 = HEAP32[$nFree>>2]|0;
   $70 = ($69|0)!=(0);
   if ($70) {
    $71 = HEAP32[$pPage1>>2]|0;
    $72 = ((($71)) + 56|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = ((($73)) + 32|0);
    $75 = (_sqlite3Get4byte($74)|0);
    HEAP32[$iTrunk>>2] = $75;
    $76 = HEAP32[$1>>2]|0;
    $77 = HEAP32[$iTrunk>>2]|0;
    $78 = (_btreeGetPage($76,$77,$pTrunk,0)|0);
    HEAP32[$rc>>2] = $78;
    $79 = HEAP32[$rc>>2]|0;
    $80 = ($79|0)!=(0);
    if ($80) {
     break;
    }
    $81 = HEAP32[$pTrunk>>2]|0;
    $82 = ((($81)) + 56|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ((($83)) + 4|0);
    $85 = (_sqlite3Get4byte($84)|0);
    HEAP32[$nLeaf>>2] = $85;
    $86 = HEAP32[$nLeaf>>2]|0;
    $87 = HEAP32[$1>>2]|0;
    $88 = ((($87)) + 36|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = (($89>>>0) / 4)&-1;
    $91 = (($90) - 2)|0;
    $92 = ($86>>>0)>($91>>>0);
    if ($92) {
     $93 = (_sqlite3CorruptError(61478)|0);
     HEAP32[$rc>>2] = $93;
     break;
    }
    $94 = HEAP32[$nLeaf>>2]|0;
    $95 = HEAP32[$1>>2]|0;
    $96 = ((($95)) + 36|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97>>>0) / 4)&-1;
    $99 = (($98) - 8)|0;
    $100 = ($94>>>0)<($99>>>0);
    if ($100) {
     $101 = HEAP32[$pTrunk>>2]|0;
     $102 = ((($101)) + 72|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = (_sqlite3PagerWrite($103)|0);
     HEAP32[$rc>>2] = $104;
     $105 = HEAP32[$rc>>2]|0;
     $106 = ($105|0)==(0);
     if (!($106)) {
      break;
     }
     $107 = HEAP32[$pTrunk>>2]|0;
     $108 = ((($107)) + 56|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = ((($109)) + 4|0);
     $111 = HEAP32[$nLeaf>>2]|0;
     $112 = (($111) + 1)|0;
     _sqlite3Put4byte($110,$112);
     $113 = HEAP32[$nLeaf>>2]|0;
     $114 = $113<<2;
     $115 = (8 + ($114))|0;
     $116 = HEAP32[$pTrunk>>2]|0;
     $117 = ((($116)) + 56|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = (($118) + ($115)|0);
     $120 = HEAP32[$3>>2]|0;
     _sqlite3Put4byte($119,$120);
     $121 = HEAP32[$pPage>>2]|0;
     $122 = ($121|0)!=(0|0);
     if ($122) {
      $123 = HEAP32[$1>>2]|0;
      $124 = ((($123)) + 22|0);
      $125 = HEAP16[$124>>1]|0;
      $126 = $125&65535;
      $127 = $126 & 4;
      $128 = ($127|0)==(0);
      if ($128) {
       $129 = HEAP32[$pPage>>2]|0;
       $130 = ((($129)) + 72|0);
       $131 = HEAP32[$130>>2]|0;
       _sqlite3PagerDontWrite($131);
      }
     }
     $132 = HEAP32[$1>>2]|0;
     $133 = HEAP32[$3>>2]|0;
     $134 = (_btreeSetHasContent($132,$133)|0);
     HEAP32[$rc>>2] = $134;
     break;
    }
   }
   $135 = HEAP32[$pPage>>2]|0;
   $136 = ($135|0)==(0|0);
   if ($136) {
    $137 = HEAP32[$1>>2]|0;
    $138 = HEAP32[$3>>2]|0;
    $139 = (_btreeGetPage($137,$138,$pPage,0)|0);
    HEAP32[$rc>>2] = $139;
    $140 = (0)!=($139|0);
    if ($140) {
     break;
    }
   }
   $141 = HEAP32[$pPage>>2]|0;
   $142 = ((($141)) + 72|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = (_sqlite3PagerWrite($143)|0);
   HEAP32[$rc>>2] = $144;
   $145 = HEAP32[$rc>>2]|0;
   $146 = ($145|0)!=(0);
   if (!($146)) {
    $147 = HEAP32[$pPage>>2]|0;
    $148 = ((($147)) + 56|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = HEAP32[$iTrunk>>2]|0;
    _sqlite3Put4byte($149,$150);
    $151 = HEAP32[$pPage>>2]|0;
    $152 = ((($151)) + 56|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = ((($153)) + 4|0);
    _sqlite3Put4byte($154,0);
    $155 = HEAP32[$pPage1>>2]|0;
    $156 = ((($155)) + 56|0);
    $157 = HEAP32[$156>>2]|0;
    $158 = ((($157)) + 32|0);
    $159 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($158,$159);
   }
  }
 } while(0);
 $160 = HEAP32[$pPage>>2]|0;
 $161 = ($160|0)!=(0|0);
 if ($161) {
  $162 = HEAP32[$pPage>>2]|0;
  HEAP8[$162>>0] = 0;
 }
 $163 = HEAP32[$pPage>>2]|0;
 _releasePage($163);
 $164 = HEAP32[$pTrunk>>2]|0;
 _releasePage($164);
 $165 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $165;
 $166 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($166|0);
}
function _sqlite3PagerRef($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3PcacheRef($1);
 STACKTOP = sp;return;
}
function _sqlite3PagerDontWrite($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 2;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pPager>>2]|0;
 $11 = ((($10)) + 104|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 | 32;
 $19 = $18&65535;
 HEAP16[$15>>1] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 24|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & -5;
 $25 = $24&65535;
 HEAP16[$21>>1] = $25;
 STACKTOP = sp;return;
}
function _btreeSetHasContent($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3BitvecCreate($8)|0);
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 60|0);
  HEAP32[$11>>2] = $9;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 60|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 60|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (_sqlite3BitvecSize($21)|0);
 $23 = ($18>>>0)<=($22>>>0);
 if (!($23)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = (_sqlite3BitvecSet($26,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PcacheRef($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = (($3) + 1)<<16>>16;
 HEAP16[$2>>1] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 STACKTOP = sp;return;
}
function _freeSpace($pPage,$iStart,$iSize) {
 $pPage = $pPage|0;
 $iStart = $iStart|0;
 $iSize = $iSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $hdr = 0, $iEnd = 0, $iFreeBlk = 0, $iLast = 0, $iOrigSize = 0, $iPtr = 0, $iPtrEnd = 0, $nFrag = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 32|0;
 $3 = sp + 30|0;
 $iPtr = sp + 28|0;
 $iFreeBlk = sp + 26|0;
 $hdr = sp + 35|0;
 $nFrag = sp + 34|0;
 $iOrigSize = sp + 24|0;
 $iLast = sp + 12|0;
 $iEnd = sp + 8|0;
 $data = sp + 4|0;
 $iPtrEnd = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP16[$2>>1] = $iStart;
 HEAP16[$3>>1] = $iSize;
 HEAP8[$nFrag>>0] = 0;
 $4 = HEAP16[$3>>1]|0;
 HEAP16[$iOrigSize>>1] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 4)|0;
 HEAP32[$iLast>>2] = $10;
 $11 = HEAP16[$2>>1]|0;
 $12 = $11&65535;
 $13 = HEAP16[$3>>1]|0;
 $14 = $13&65535;
 $15 = (($12) + ($14))|0;
 HEAP32[$iEnd>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 56|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$data>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 52|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ((($21)) + 22|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = $23&65535;
 $25 = $24 & 4;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP16[$2>>1]|0;
  $28 = $27&65535;
  $29 = HEAP32[$data>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP16[$3>>1]|0;
  $32 = $31&65535;
  _memset(($30|0),0,($32|0))|0;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 5|0);
 $35 = HEAP8[$34>>0]|0;
 HEAP8[$hdr>>0] = $35;
 $36 = HEAP8[$hdr>>0]|0;
 $37 = $36&255;
 $38 = (($37) + 1)|0;
 $39 = $38&65535;
 HEAP16[$iPtr>>1] = $39;
 $40 = HEAP16[$iPtr>>1]|0;
 $41 = $40&65535;
 $42 = (($41) + 1)|0;
 $43 = HEAP32[$data>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP8[$44>>0]|0;
 $46 = $45&255;
 $47 = ($46|0)==(0);
 if ($47) {
  $48 = HEAP16[$iPtr>>1]|0;
  $49 = $48&65535;
  $50 = HEAP32[$data>>2]|0;
  $51 = (($50) + ($49)|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = ($53|0)==(0);
  if ($54) {
   HEAP16[$iFreeBlk>>1] = 0;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 do {
  if ((label|0) == 6) {
   while(1) {
    label = 0;
    $55 = HEAP16[$iPtr>>1]|0;
    $56 = $55&65535;
    $57 = HEAP32[$data>>2]|0;
    $58 = (($57) + ($56)|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = $60 << 8;
    $62 = HEAP16[$iPtr>>1]|0;
    $63 = $62&65535;
    $64 = HEAP32[$data>>2]|0;
    $65 = (($64) + ($63)|0);
    $66 = ((($65)) + 1|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = $61 | $68;
    $70 = $69&65535;
    HEAP16[$iFreeBlk>>1] = $70;
    $71 = $70&65535;
    $72 = ($71|0)>(0);
    if ($72) {
     $73 = HEAP16[$iFreeBlk>>1]|0;
     $74 = $73&65535;
     $75 = HEAP16[$2>>1]|0;
     $76 = $75&65535;
     $77 = ($74|0)<($76|0);
     $344 = $77;
    } else {
     $344 = 0;
    }
    $78 = HEAP16[$iFreeBlk>>1]|0;
    $79 = $78&65535;
    if (!($344)) {
     break;
    }
    $80 = HEAP16[$iPtr>>1]|0;
    $81 = $80&65535;
    $82 = (($81) + 4)|0;
    $83 = ($79|0)<($82|0);
    if ($83) {
     label = 10;
     break;
    }
    $85 = HEAP16[$iFreeBlk>>1]|0;
    HEAP16[$iPtr>>1] = $85;
    label = 6;
   }
   if ((label|0) == 10) {
    $84 = (_sqlite3CorruptError(57169)|0);
    HEAP32[$0>>2] = $84;
    $343 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($343|0);
   }
   $86 = HEAP32[$iLast>>2]|0;
   $87 = ($79>>>0)>($86>>>0);
   if ($87) {
    $88 = (_sqlite3CorruptError(57172)|0);
    HEAP32[$0>>2] = $88;
    $343 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($343|0);
   }
   $89 = HEAP16[$iFreeBlk>>1]|0;
   $90 = $89&65535;
   $91 = ($90|0)!=(0);
   do {
    if ($91) {
     $92 = HEAP32[$iEnd>>2]|0;
     $93 = (($92) + 3)|0;
     $94 = HEAP16[$iFreeBlk>>1]|0;
     $95 = $94&65535;
     $96 = ($93>>>0)>=($95>>>0);
     if ($96) {
      $97 = HEAP16[$iFreeBlk>>1]|0;
      $98 = $97&65535;
      $99 = HEAP32[$iEnd>>2]|0;
      $100 = (($98) - ($99))|0;
      $101 = $100&255;
      HEAP8[$nFrag>>0] = $101;
      $102 = HEAP32[$iEnd>>2]|0;
      $103 = HEAP16[$iFreeBlk>>1]|0;
      $104 = $103&65535;
      $105 = ($102>>>0)>($104>>>0);
      if ($105) {
       $106 = (_sqlite3CorruptError(57183)|0);
       HEAP32[$0>>2] = $106;
       $343 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($343|0);
      }
      $107 = HEAP16[$iFreeBlk>>1]|0;
      $108 = $107&65535;
      $109 = HEAP16[$iFreeBlk>>1]|0;
      $110 = $109&65535;
      $111 = (($110) + 2)|0;
      $112 = HEAP32[$data>>2]|0;
      $113 = (($112) + ($111)|0);
      $114 = HEAP8[$113>>0]|0;
      $115 = $114&255;
      $116 = $115 << 8;
      $117 = HEAP16[$iFreeBlk>>1]|0;
      $118 = $117&65535;
      $119 = (($118) + 2)|0;
      $120 = HEAP32[$data>>2]|0;
      $121 = (($120) + ($119)|0);
      $122 = ((($121)) + 1|0);
      $123 = HEAP8[$122>>0]|0;
      $124 = $123&255;
      $125 = $116 | $124;
      $126 = (($108) + ($125))|0;
      HEAP32[$iEnd>>2] = $126;
      $127 = HEAP32[$iEnd>>2]|0;
      $128 = HEAP32[$1>>2]|0;
      $129 = ((($128)) + 52|0);
      $130 = HEAP32[$129>>2]|0;
      $131 = ((($130)) + 36|0);
      $132 = HEAP32[$131>>2]|0;
      $133 = ($127>>>0)>($132>>>0);
      if (!($133)) {
       $135 = HEAP32[$iEnd>>2]|0;
       $136 = HEAP16[$2>>1]|0;
       $137 = $136&65535;
       $138 = (($135) - ($137))|0;
       $139 = $138&65535;
       HEAP16[$3>>1] = $139;
       $140 = HEAP16[$iFreeBlk>>1]|0;
       $141 = $140&65535;
       $142 = HEAP32[$data>>2]|0;
       $143 = (($142) + ($141)|0);
       $144 = HEAP8[$143>>0]|0;
       $145 = $144&255;
       $146 = $145 << 8;
       $147 = HEAP16[$iFreeBlk>>1]|0;
       $148 = $147&65535;
       $149 = HEAP32[$data>>2]|0;
       $150 = (($149) + ($148)|0);
       $151 = ((($150)) + 1|0);
       $152 = HEAP8[$151>>0]|0;
       $153 = $152&255;
       $154 = $146 | $153;
       $155 = $154&65535;
       HEAP16[$iFreeBlk>>1] = $155;
       break;
      }
      $134 = (_sqlite3CorruptError(57185)|0);
      HEAP32[$0>>2] = $134;
      $343 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($343|0);
     }
    }
   } while(0);
   $156 = HEAP16[$iPtr>>1]|0;
   $157 = $156&65535;
   $158 = HEAP8[$hdr>>0]|0;
   $159 = $158&255;
   $160 = (($159) + 1)|0;
   $161 = ($157|0)>($160|0);
   do {
    if ($161) {
     $162 = HEAP16[$iPtr>>1]|0;
     $163 = $162&65535;
     $164 = HEAP16[$iPtr>>1]|0;
     $165 = $164&65535;
     $166 = (($165) + 2)|0;
     $167 = HEAP32[$data>>2]|0;
     $168 = (($167) + ($166)|0);
     $169 = HEAP8[$168>>0]|0;
     $170 = $169&255;
     $171 = $170 << 8;
     $172 = HEAP16[$iPtr>>1]|0;
     $173 = $172&65535;
     $174 = (($173) + 2)|0;
     $175 = HEAP32[$data>>2]|0;
     $176 = (($175) + ($174)|0);
     $177 = ((($176)) + 1|0);
     $178 = HEAP8[$177>>0]|0;
     $179 = $178&255;
     $180 = $171 | $179;
     $181 = (($163) + ($180))|0;
     HEAP32[$iPtrEnd>>2] = $181;
     $182 = HEAP32[$iPtrEnd>>2]|0;
     $183 = (($182) + 3)|0;
     $184 = HEAP16[$2>>1]|0;
     $185 = $184&65535;
     $186 = ($183|0)>=($185|0);
     if ($186) {
      $187 = HEAP32[$iPtrEnd>>2]|0;
      $188 = HEAP16[$2>>1]|0;
      $189 = $188&65535;
      $190 = ($187|0)>($189|0);
      if (!($190)) {
       $192 = HEAP16[$2>>1]|0;
       $193 = $192&65535;
       $194 = HEAP32[$iPtrEnd>>2]|0;
       $195 = (($193) - ($194))|0;
       $196 = HEAP8[$nFrag>>0]|0;
       $197 = $196&255;
       $198 = (($197) + ($195))|0;
       $199 = $198&255;
       HEAP8[$nFrag>>0] = $199;
       $200 = HEAP32[$iEnd>>2]|0;
       $201 = HEAP16[$iPtr>>1]|0;
       $202 = $201&65535;
       $203 = (($200) - ($202))|0;
       $204 = $203&65535;
       HEAP16[$3>>1] = $204;
       $205 = HEAP16[$iPtr>>1]|0;
       HEAP16[$2>>1] = $205;
       break;
      }
      $191 = (_sqlite3CorruptError(57197)|0);
      HEAP32[$0>>2] = $191;
      $343 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($343|0);
     }
    }
   } while(0);
   $206 = HEAP8[$nFrag>>0]|0;
   $207 = $206&255;
   $208 = HEAP8[$hdr>>0]|0;
   $209 = $208&255;
   $210 = (($209) + 7)|0;
   $211 = HEAP32[$data>>2]|0;
   $212 = (($211) + ($210)|0);
   $213 = HEAP8[$212>>0]|0;
   $214 = $213&255;
   $215 = ($207|0)>($214|0);
   if (!($215)) {
    $217 = HEAP8[$nFrag>>0]|0;
    $218 = $217&255;
    $219 = HEAP8[$hdr>>0]|0;
    $220 = $219&255;
    $221 = (($220) + 7)|0;
    $222 = HEAP32[$data>>2]|0;
    $223 = (($222) + ($221)|0);
    $224 = HEAP8[$223>>0]|0;
    $225 = $224&255;
    $226 = (($225) - ($218))|0;
    $227 = $226&255;
    HEAP8[$223>>0] = $227;
    break;
   }
   $216 = (_sqlite3CorruptError(57203)|0);
   HEAP32[$0>>2] = $216;
   $343 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($343|0);
  }
 } while(0);
 $228 = HEAP16[$2>>1]|0;
 $229 = $228&65535;
 $230 = HEAP8[$hdr>>0]|0;
 $231 = $230&255;
 $232 = (($231) + 5)|0;
 $233 = HEAP32[$data>>2]|0;
 $234 = (($233) + ($232)|0);
 $235 = HEAP8[$234>>0]|0;
 $236 = $235&255;
 $237 = $236 << 8;
 $238 = HEAP8[$hdr>>0]|0;
 $239 = $238&255;
 $240 = (($239) + 5)|0;
 $241 = HEAP32[$data>>2]|0;
 $242 = (($241) + ($240)|0);
 $243 = ((($242)) + 1|0);
 $244 = HEAP8[$243>>0]|0;
 $245 = $244&255;
 $246 = $237 | $245;
 $247 = ($229|0)==($246|0);
 do {
  if ($247) {
   $248 = HEAP16[$iPtr>>1]|0;
   $249 = $248&65535;
   $250 = HEAP8[$hdr>>0]|0;
   $251 = $250&255;
   $252 = (($251) + 1)|0;
   $253 = ($249|0)!=($252|0);
   if (!($253)) {
    $255 = HEAP16[$iFreeBlk>>1]|0;
    $256 = $255&65535;
    $257 = $256 >> 8;
    $258 = $257&255;
    $259 = HEAP8[$hdr>>0]|0;
    $260 = $259&255;
    $261 = (($260) + 1)|0;
    $262 = HEAP32[$data>>2]|0;
    $263 = (($262) + ($261)|0);
    HEAP8[$263>>0] = $258;
    $264 = HEAP16[$iFreeBlk>>1]|0;
    $265 = $264&255;
    $266 = HEAP8[$hdr>>0]|0;
    $267 = $266&255;
    $268 = (($267) + 1)|0;
    $269 = HEAP32[$data>>2]|0;
    $270 = (($269) + ($268)|0);
    $271 = ((($270)) + 1|0);
    HEAP8[$271>>0] = $265;
    $272 = HEAP32[$iEnd>>2]|0;
    $273 = $272 >>> 8;
    $274 = $273&255;
    $275 = HEAP8[$hdr>>0]|0;
    $276 = $275&255;
    $277 = (($276) + 5)|0;
    $278 = HEAP32[$data>>2]|0;
    $279 = (($278) + ($277)|0);
    HEAP8[$279>>0] = $274;
    $280 = HEAP32[$iEnd>>2]|0;
    $281 = $280&255;
    $282 = HEAP8[$hdr>>0]|0;
    $283 = $282&255;
    $284 = (($283) + 5)|0;
    $285 = HEAP32[$data>>2]|0;
    $286 = (($285) + ($284)|0);
    $287 = ((($286)) + 1|0);
    HEAP8[$287>>0] = $281;
    break;
   }
   $254 = (_sqlite3CorruptError(57210)|0);
   HEAP32[$0>>2] = $254;
   $343 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($343|0);
  } else {
   $288 = HEAP16[$2>>1]|0;
   $289 = $288&65535;
   $290 = $289 >> 8;
   $291 = $290&255;
   $292 = HEAP16[$iPtr>>1]|0;
   $293 = $292&65535;
   $294 = HEAP32[$data>>2]|0;
   $295 = (($294) + ($293)|0);
   HEAP8[$295>>0] = $291;
   $296 = HEAP16[$2>>1]|0;
   $297 = $296&255;
   $298 = HEAP16[$iPtr>>1]|0;
   $299 = $298&65535;
   $300 = HEAP32[$data>>2]|0;
   $301 = (($300) + ($299)|0);
   $302 = ((($301)) + 1|0);
   HEAP8[$302>>0] = $297;
   $303 = HEAP16[$iFreeBlk>>1]|0;
   $304 = $303&65535;
   $305 = $304 >> 8;
   $306 = $305&255;
   $307 = HEAP16[$2>>1]|0;
   $308 = $307&65535;
   $309 = HEAP32[$data>>2]|0;
   $310 = (($309) + ($308)|0);
   HEAP8[$310>>0] = $306;
   $311 = HEAP16[$iFreeBlk>>1]|0;
   $312 = $311&255;
   $313 = HEAP16[$2>>1]|0;
   $314 = $313&65535;
   $315 = HEAP32[$data>>2]|0;
   $316 = (($315) + ($314)|0);
   $317 = ((($316)) + 1|0);
   HEAP8[$317>>0] = $312;
   $318 = HEAP16[$3>>1]|0;
   $319 = $318&65535;
   $320 = $319 >> 8;
   $321 = $320&255;
   $322 = HEAP16[$2>>1]|0;
   $323 = $322&65535;
   $324 = (($323) + 2)|0;
   $325 = HEAP32[$data>>2]|0;
   $326 = (($325) + ($324)|0);
   HEAP8[$326>>0] = $321;
   $327 = HEAP16[$3>>1]|0;
   $328 = $327&255;
   $329 = HEAP16[$2>>1]|0;
   $330 = $329&65535;
   $331 = (($330) + 2)|0;
   $332 = HEAP32[$data>>2]|0;
   $333 = (($332) + ($331)|0);
   $334 = ((($333)) + 1|0);
   HEAP8[$334>>0] = $328;
  }
 } while(0);
 $335 = HEAP16[$iOrigSize>>1]|0;
 $336 = $335&65535;
 $337 = HEAP32[$1>>2]|0;
 $338 = ((($337)) + 16|0);
 $339 = HEAP16[$338>>1]|0;
 $340 = $339&65535;
 $341 = (($340) + ($336))|0;
 $342 = $341&65535;
 HEAP16[$338>>1] = $342;
 HEAP32[$0>>2] = 0;
 $343 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($343|0);
}
function _allocateSpace($pPage,$nByte,$pIdx) {
 $pPage = $pPage|0;
 $nByte = $nByte|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $gap = 0, $hdr = 0, $pSpace = 0, $rc = 0, $top = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $hdr = sp + 20|0;
 $data = sp + 16|0;
 $top = sp + 12|0;
 $rc = sp + 8|0;
 $gap = sp + 4|0;
 $pSpace = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $pIdx;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 HEAP32[$hdr>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$data>>2] = $10;
 HEAP32[$rc>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 14|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 18|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17&65535;
 $19 = $18<<1;
 $20 = (($14) + ($19))|0;
 HEAP32[$gap>>2] = $20;
 $21 = HEAP32[$hdr>>2]|0;
 $22 = (($21) + 5)|0;
 $23 = HEAP32[$data>>2]|0;
 $24 = (($23) + ($22)|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = $26 << 8;
 $28 = HEAP32[$hdr>>2]|0;
 $29 = (($28) + 5)|0;
 $30 = HEAP32[$data>>2]|0;
 $31 = (($30) + ($29)|0);
 $32 = ((($31)) + 1|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = $27 | $34;
 HEAP32[$top>>2] = $35;
 $36 = HEAP32[$gap>>2]|0;
 $37 = HEAP32[$top>>2]|0;
 $38 = ($36|0)>($37|0);
 do {
  if ($38) {
   $39 = HEAP32[$top>>2]|0;
   $40 = ($39|0)==(0);
   if ($40) {
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 52|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ((($43)) + 36|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)==(65536);
    if ($46) {
     HEAP32[$top>>2] = 65536;
     break;
    }
   }
   $47 = (_sqlite3CorruptError(57075)|0);
   HEAP32[$0>>2] = $47;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $48 = HEAP32[$hdr>>2]|0;
 $49 = (($48) + 2)|0;
 $50 = HEAP32[$data>>2]|0;
 $51 = (($50) + ($49)|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = $52&255;
 $54 = ($53|0)!=(0);
 if ($54) {
  label = 8;
 } else {
  $55 = HEAP32[$hdr>>2]|0;
  $56 = (($55) + 1)|0;
  $57 = HEAP32[$data>>2]|0;
  $58 = (($57) + ($56)|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = $59&255;
  $61 = ($60|0)!=(0);
  if ($61) {
   label = 8;
  }
 }
 if ((label|0) == 8) {
  $62 = HEAP32[$gap>>2]|0;
  $63 = (($62) + 2)|0;
  $64 = HEAP32[$top>>2]|0;
  $65 = ($63|0)<=($64|0);
  if ($65) {
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$2>>2]|0;
   $68 = (_pageFindSlot($66,$67,$rc)|0);
   HEAP32[$pSpace>>2] = $68;
   $69 = HEAP32[$pSpace>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    $71 = HEAP32[$pSpace>>2]|0;
    $72 = HEAP32[$data>>2]|0;
    $73 = $71;
    $74 = $72;
    $75 = (($73) - ($74))|0;
    $76 = HEAP32[$3>>2]|0;
    HEAP32[$76>>2] = $75;
    HEAP32[$0>>2] = 0;
    $128 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($128|0);
   }
   $77 = HEAP32[$rc>>2]|0;
   $78 = ($77|0)!=(0);
   if ($78) {
    $79 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $79;
    $128 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($128|0);
   }
  }
 }
 $80 = HEAP32[$gap>>2]|0;
 $81 = (($80) + 2)|0;
 $82 = HEAP32[$2>>2]|0;
 $83 = (($81) + ($82))|0;
 $84 = HEAP32[$top>>2]|0;
 $85 = ($83|0)>($84|0);
 do {
  if ($85) {
   $86 = HEAP32[$1>>2]|0;
   $87 = (_defragmentPage($86)|0);
   HEAP32[$rc>>2] = $87;
   $88 = HEAP32[$rc>>2]|0;
   $89 = ($88|0)!=(0);
   if (!($89)) {
    $91 = HEAP32[$hdr>>2]|0;
    $92 = (($91) + 5)|0;
    $93 = HEAP32[$data>>2]|0;
    $94 = (($93) + ($92)|0);
    $95 = HEAP8[$94>>0]|0;
    $96 = $95&255;
    $97 = $96 << 8;
    $98 = HEAP32[$hdr>>2]|0;
    $99 = (($98) + 5)|0;
    $100 = HEAP32[$data>>2]|0;
    $101 = (($100) + ($99)|0);
    $102 = ((($101)) + 1|0);
    $103 = HEAP8[$102>>0]|0;
    $104 = $103&255;
    $105 = $97 | $104;
    $106 = (($105) - 1)|0;
    $107 = $106 & 65535;
    $108 = (($107) + 1)|0;
    HEAP32[$top>>2] = $108;
    break;
   }
   $90 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $90;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $109 = HEAP32[$2>>2]|0;
 $110 = HEAP32[$top>>2]|0;
 $111 = (($110) - ($109))|0;
 HEAP32[$top>>2] = $111;
 $112 = HEAP32[$top>>2]|0;
 $113 = $112 >> 8;
 $114 = $113&255;
 $115 = HEAP32[$hdr>>2]|0;
 $116 = (($115) + 5)|0;
 $117 = HEAP32[$data>>2]|0;
 $118 = (($117) + ($116)|0);
 HEAP8[$118>>0] = $114;
 $119 = HEAP32[$top>>2]|0;
 $120 = $119&255;
 $121 = HEAP32[$hdr>>2]|0;
 $122 = (($121) + 5)|0;
 $123 = HEAP32[$data>>2]|0;
 $124 = (($123) + ($122)|0);
 $125 = ((($124)) + 1|0);
 HEAP8[$125>>0] = $120;
 $126 = HEAP32[$top>>2]|0;
 $127 = HEAP32[$3>>2]|0;
 HEAP32[$127>>2] = $126;
 HEAP32[$0>>2] = 0;
 $128 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($128|0);
}
function _pageFindSlot($pPg,$nByte,$pRc) {
 $pPg = $pPg|0;
 $nByte = $nByte|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aData = 0, $hdr = 0, $iAddr = 0, $pc = 0, $size = 0, $usableSize = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $hdr = sp + 24|0;
 $aData = sp + 20|0;
 $iAddr = sp + 16|0;
 $pc = sp + 12|0;
 $x = sp + 8|0;
 $usableSize = sp + 4|0;
 $size = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $pRc;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 HEAP32[$hdr>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$aData>>2] = $10;
 $11 = HEAP32[$hdr>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$iAddr>>2] = $12;
 $13 = HEAP32[$iAddr>>2]|0;
 $14 = HEAP32[$aData>>2]|0;
 $15 = (($14) + ($13)|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = $17 << 8;
 $19 = HEAP32[$iAddr>>2]|0;
 $20 = HEAP32[$aData>>2]|0;
 $21 = (($20) + ($19)|0);
 $22 = ((($21)) + 1|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $18 | $24;
 HEAP32[$pc>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 52|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($28)) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$usableSize>>2] = $30;
 while(1) {
  $31 = HEAP32[$pc>>2]|0;
  $32 = HEAP32[$usableSize>>2]|0;
  $33 = (($32) - 4)|0;
  $34 = ($31|0)>($33|0);
  if ($34) {
   label = 4;
   break;
  }
  $35 = HEAP32[$pc>>2]|0;
  $36 = HEAP32[$iAddr>>2]|0;
  $37 = (($36) + 4)|0;
  $38 = ($35|0)<($37|0);
  if ($38) {
   label = 4;
   break;
  }
  $41 = HEAP32[$pc>>2]|0;
  $42 = (($41) + 2)|0;
  $43 = HEAP32[$aData>>2]|0;
  $44 = (($43) + ($42)|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46 << 8;
  $48 = HEAP32[$pc>>2]|0;
  $49 = (($48) + 2)|0;
  $50 = HEAP32[$aData>>2]|0;
  $51 = (($50) + ($49)|0);
  $52 = ((($51)) + 1|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $47 | $54;
  HEAP32[$size>>2] = $55;
  $56 = HEAP32[$size>>2]|0;
  $57 = HEAP32[$2>>2]|0;
  $58 = (($56) - ($57))|0;
  HEAP32[$x>>2] = $58;
  $59 = ($58|0)>=(0);
  $60 = HEAP32[$pc>>2]|0;
  if ($59) {
   label = 6;
   break;
  }
  HEAP32[$iAddr>>2] = $60;
  $124 = HEAP32[$pc>>2]|0;
  $125 = HEAP32[$aData>>2]|0;
  $126 = (($125) + ($124)|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = $128 << 8;
  $130 = HEAP32[$pc>>2]|0;
  $131 = HEAP32[$aData>>2]|0;
  $132 = (($131) + ($130)|0);
  $133 = ((($132)) + 1|0);
  $134 = HEAP8[$133>>0]|0;
  $135 = $134&255;
  $136 = $129 | $135;
  HEAP32[$pc>>2] = $136;
  $137 = HEAP32[$pc>>2]|0;
  $138 = ($137|0)!=(0);
  if (!($138)) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 4) {
  $39 = (_sqlite3CorruptError(56997)|0);
  $40 = HEAP32[$3>>2]|0;
  HEAP32[$40>>2] = $39;
  HEAP32[$0>>2] = 0;
  $139 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 else if ((label|0) == 6) {
  $61 = HEAP32[$1>>2]|0;
  $62 = ((($61)) + 14|0);
  $63 = HEAP16[$62>>1]|0;
  $64 = $63&65535;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 18|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = $68<<1;
  $70 = (($64) + ($69))|0;
  $71 = ($60|0)<($70|0);
  if (!($71)) {
   $72 = HEAP32[$size>>2]|0;
   $73 = HEAP32[$pc>>2]|0;
   $74 = (($72) + ($73))|0;
   $75 = HEAP32[$usableSize>>2]|0;
   $76 = ($74|0)>($75|0);
   if (!($76)) {
    $79 = HEAP32[$x>>2]|0;
    $80 = ($79|0)<(4);
    do {
     if ($80) {
      $81 = HEAP32[$hdr>>2]|0;
      $82 = (($81) + 7)|0;
      $83 = HEAP32[$aData>>2]|0;
      $84 = (($83) + ($82)|0);
      $85 = HEAP8[$84>>0]|0;
      $86 = $85&255;
      $87 = ($86|0)>(57);
      if (!($87)) {
       $88 = HEAP32[$iAddr>>2]|0;
       $89 = HEAP32[$aData>>2]|0;
       $90 = (($89) + ($88)|0);
       $91 = HEAP32[$pc>>2]|0;
       $92 = HEAP32[$aData>>2]|0;
       $93 = (($92) + ($91)|0);
       ;HEAP8[$90>>0]=HEAP8[$93>>0]|0;HEAP8[$90+1>>0]=HEAP8[$93+1>>0]|0;
       $94 = HEAP32[$x>>2]|0;
       $95 = $94&255;
       $96 = $95&255;
       $97 = HEAP32[$hdr>>2]|0;
       $98 = (($97) + 7)|0;
       $99 = HEAP32[$aData>>2]|0;
       $100 = (($99) + ($98)|0);
       $101 = HEAP8[$100>>0]|0;
       $102 = $101&255;
       $103 = (($102) + ($96))|0;
       $104 = $103&255;
       HEAP8[$100>>0] = $104;
       break;
      }
      HEAP32[$0>>2] = 0;
      $139 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($139|0);
     } else {
      $105 = HEAP32[$x>>2]|0;
      $106 = $105 >> 8;
      $107 = $106&255;
      $108 = HEAP32[$pc>>2]|0;
      $109 = (($108) + 2)|0;
      $110 = HEAP32[$aData>>2]|0;
      $111 = (($110) + ($109)|0);
      HEAP8[$111>>0] = $107;
      $112 = HEAP32[$x>>2]|0;
      $113 = $112&255;
      $114 = HEAP32[$pc>>2]|0;
      $115 = (($114) + 2)|0;
      $116 = HEAP32[$aData>>2]|0;
      $117 = (($116) + ($115)|0);
      $118 = ((($117)) + 1|0);
      HEAP8[$118>>0] = $113;
     }
    } while(0);
    $119 = HEAP32[$pc>>2]|0;
    $120 = HEAP32[$x>>2]|0;
    $121 = (($119) + ($120))|0;
    $122 = HEAP32[$aData>>2]|0;
    $123 = (($122) + ($121)|0);
    HEAP32[$0>>2] = $123;
    $139 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($139|0);
   }
  }
  $77 = (_sqlite3CorruptError(57008)|0);
  $78 = HEAP32[$3>>2]|0;
  HEAP32[$78>>2] = $77;
  HEAP32[$0>>2] = 0;
  $139 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $139 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 return (0)|0;
}
function _defragmentPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cbrk = 0, $cellOffset = 0, $data = 0, $hdr = 0, $i = 0, $iCellFirst = 0, $iCellLast = 0, $nCell = 0, $pAddr = 0;
 var $pc = 0, $size = 0, $src = 0, $temp = 0, $usableSize = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $i = sp + 56|0;
 $pc = sp + 52|0;
 $hdr = sp + 48|0;
 $size = sp + 44|0;
 $usableSize = sp + 40|0;
 $cellOffset = sp + 36|0;
 $cbrk = sp + 32|0;
 $nCell = sp + 28|0;
 $data = sp + 24|0;
 $temp = sp + 20|0;
 $src = sp + 16|0;
 $iCellFirst = sp + 12|0;
 $iCellLast = sp + 8|0;
 $pAddr = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$temp>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$data>>2] = $4;
 HEAP32[$src>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$hdr>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 14|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 HEAP32[$cellOffset>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 18|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 HEAP32[$nCell>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$usableSize>>2] = $21;
 $22 = HEAP32[$usableSize>>2]|0;
 HEAP32[$cbrk>>2] = $22;
 $23 = HEAP32[$cellOffset>>2]|0;
 $24 = HEAP32[$nCell>>2]|0;
 $25 = $24<<1;
 $26 = (($23) + ($25))|0;
 HEAP32[$iCellFirst>>2] = $26;
 $27 = HEAP32[$usableSize>>2]|0;
 $28 = (($27) - 4)|0;
 HEAP32[$iCellLast>>2] = $28;
 HEAP32[$i>>2] = 0;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$nCell>>2]|0;
  $31 = ($29|0)<($30|0);
  if (!($31)) {
   label = 14;
   break;
  }
  $32 = HEAP32[$cellOffset>>2]|0;
  $33 = HEAP32[$i>>2]|0;
  $34 = $33<<1;
  $35 = (($32) + ($34))|0;
  $36 = HEAP32[$data>>2]|0;
  $37 = (($36) + ($35)|0);
  HEAP32[$pAddr>>2] = $37;
  $38 = HEAP32[$pAddr>>2]|0;
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = $40 << 8;
  $42 = HEAP32[$pAddr>>2]|0;
  $43 = ((($42)) + 1|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = $41 | $45;
  HEAP32[$pc>>2] = $46;
  $47 = HEAP32[$pc>>2]|0;
  $48 = HEAP32[$iCellFirst>>2]|0;
  $49 = ($47|0)<($48|0);
  if ($49) {
   label = 5;
   break;
  }
  $50 = HEAP32[$pc>>2]|0;
  $51 = HEAP32[$iCellLast>>2]|0;
  $52 = ($50|0)>($51|0);
  if ($52) {
   label = 5;
   break;
  }
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 76|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = HEAP32[$pc>>2]|0;
  $59 = HEAP32[$src>>2]|0;
  $60 = (($59) + ($58)|0);
  $61 = (FUNCTION_TABLE_iii[$56 & 255]($57,$60)|0);
  $62 = $61&65535;
  HEAP32[$size>>2] = $62;
  $63 = HEAP32[$size>>2]|0;
  $64 = HEAP32[$cbrk>>2]|0;
  $65 = (($64) - ($63))|0;
  HEAP32[$cbrk>>2] = $65;
  $66 = HEAP32[$cbrk>>2]|0;
  $67 = HEAP32[$iCellFirst>>2]|0;
  $68 = ($66|0)<($67|0);
  if ($68) {
   label = 8;
   break;
  }
  $69 = HEAP32[$pc>>2]|0;
  $70 = HEAP32[$size>>2]|0;
  $71 = (($69) + ($70))|0;
  $72 = HEAP32[$usableSize>>2]|0;
  $73 = ($71|0)>($72|0);
  if ($73) {
   label = 8;
   break;
  }
  $75 = HEAP32[$cbrk>>2]|0;
  $76 = $75 >> 8;
  $77 = $76&255;
  $78 = HEAP32[$pAddr>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$cbrk>>2]|0;
  $80 = $79&255;
  $81 = HEAP32[$pAddr>>2]|0;
  $82 = ((($81)) + 1|0);
  HEAP8[$82>>0] = $80;
  $83 = HEAP32[$temp>>2]|0;
  $84 = ($83|0)==(0|0);
  if ($84) {
   $85 = HEAP32[$cbrk>>2]|0;
   $86 = HEAP32[$pc>>2]|0;
   $87 = ($85|0)==($86|0);
   if (!($87)) {
    $88 = HEAP32[$1>>2]|0;
    $89 = ((($88)) + 52|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = (_sqlite3PagerTempSpace($91)|0);
    HEAP32[$temp>>2] = $92;
    $93 = HEAP32[$hdr>>2]|0;
    $94 = (($93) + 5)|0;
    $95 = HEAP32[$data>>2]|0;
    $96 = (($95) + ($94)|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $97&255;
    $99 = $98 << 8;
    $100 = HEAP32[$hdr>>2]|0;
    $101 = (($100) + 5)|0;
    $102 = HEAP32[$data>>2]|0;
    $103 = (($102) + ($101)|0);
    $104 = ((($103)) + 1|0);
    $105 = HEAP8[$104>>0]|0;
    $106 = $105&255;
    $107 = $99 | $106;
    HEAP32[$x>>2] = $107;
    $108 = HEAP32[$x>>2]|0;
    $109 = HEAP32[$temp>>2]|0;
    $110 = (($109) + ($108)|0);
    $111 = HEAP32[$x>>2]|0;
    $112 = HEAP32[$data>>2]|0;
    $113 = (($112) + ($111)|0);
    $114 = HEAP32[$cbrk>>2]|0;
    $115 = HEAP32[$size>>2]|0;
    $116 = (($114) + ($115))|0;
    $117 = HEAP32[$x>>2]|0;
    $118 = (($116) - ($117))|0;
    _memcpy(($110|0),($113|0),($118|0))|0;
    $119 = HEAP32[$temp>>2]|0;
    HEAP32[$src>>2] = $119;
    label = 12;
   }
  } else {
   label = 12;
  }
  if ((label|0) == 12) {
   label = 0;
   $120 = HEAP32[$cbrk>>2]|0;
   $121 = HEAP32[$data>>2]|0;
   $122 = (($121) + ($120)|0);
   $123 = HEAP32[$pc>>2]|0;
   $124 = HEAP32[$src>>2]|0;
   $125 = (($124) + ($123)|0);
   $126 = HEAP32[$size>>2]|0;
   _memcpy(($122|0),($125|0),($126|0))|0;
  }
  $127 = HEAP32[$i>>2]|0;
  $128 = (($127) + 1)|0;
  HEAP32[$i>>2] = $128;
 }
 if ((label|0) == 5) {
  $53 = (_sqlite3CorruptError(56934)|0);
  HEAP32[$0>>2] = $53;
  $170 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($170|0);
 }
 else if ((label|0) == 8) {
  $74 = (_sqlite3CorruptError(56940)|0);
  HEAP32[$0>>2] = $74;
  $170 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($170|0);
 }
 else if ((label|0) == 14) {
  $129 = HEAP32[$cbrk>>2]|0;
  $130 = $129 >> 8;
  $131 = $130&255;
  $132 = HEAP32[$hdr>>2]|0;
  $133 = (($132) + 5)|0;
  $134 = HEAP32[$data>>2]|0;
  $135 = (($134) + ($133)|0);
  HEAP8[$135>>0] = $131;
  $136 = HEAP32[$cbrk>>2]|0;
  $137 = $136&255;
  $138 = HEAP32[$hdr>>2]|0;
  $139 = (($138) + 5)|0;
  $140 = HEAP32[$data>>2]|0;
  $141 = (($140) + ($139)|0);
  $142 = ((($141)) + 1|0);
  HEAP8[$142>>0] = $137;
  $143 = HEAP32[$hdr>>2]|0;
  $144 = (($143) + 1)|0;
  $145 = HEAP32[$data>>2]|0;
  $146 = (($145) + ($144)|0);
  HEAP8[$146>>0] = 0;
  $147 = HEAP32[$hdr>>2]|0;
  $148 = (($147) + 2)|0;
  $149 = HEAP32[$data>>2]|0;
  $150 = (($149) + ($148)|0);
  HEAP8[$150>>0] = 0;
  $151 = HEAP32[$hdr>>2]|0;
  $152 = (($151) + 7)|0;
  $153 = HEAP32[$data>>2]|0;
  $154 = (($153) + ($152)|0);
  HEAP8[$154>>0] = 0;
  $155 = HEAP32[$iCellFirst>>2]|0;
  $156 = HEAP32[$data>>2]|0;
  $157 = (($156) + ($155)|0);
  $158 = HEAP32[$cbrk>>2]|0;
  $159 = HEAP32[$iCellFirst>>2]|0;
  $160 = (($158) - ($159))|0;
  _memset(($157|0),0,($160|0))|0;
  $161 = HEAP32[$cbrk>>2]|0;
  $162 = HEAP32[$iCellFirst>>2]|0;
  $163 = (($161) - ($162))|0;
  $164 = HEAP32[$1>>2]|0;
  $165 = ((($164)) + 16|0);
  $166 = HEAP16[$165>>1]|0;
  $167 = $166&65535;
  $168 = ($163|0)!=($167|0);
  if ($168) {
   $169 = (_sqlite3CorruptError(56964)|0);
   HEAP32[$0>>2] = $169;
   $170 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($170|0);
  } else {
   HEAP32[$0>>2] = 0;
   $170 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($170|0);
  }
 }
 return (0)|0;
}
function _sqlite3PagerTempSpace($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _balance_deeper($pRoot,$ppChild) {
 $pRoot = $pRoot|0;
 $ppChild = $ppChild|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $pBt = 0, $pChild = 0, $pgnoChild = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pChild = sp + 8|0;
 $pgnoChild = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $ppChild;
 HEAP32[$pChild>>2] = 0;
 HEAP32[$pgnoChild>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 72|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PagerWrite($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = HEAP32[$pBt>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 84|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (_allocateBtreePage($12,$pChild,$pgnoChild,$15,0)|0);
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$pChild>>2]|0;
  _copyNodeContent($17,$18,$rc);
  $19 = HEAP32[$pBt>>2]|0;
  $20 = ((($19)) + 17|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = ($21<<24>>24)!=(0);
  if ($22) {
   $23 = HEAP32[$pBt>>2]|0;
   $24 = HEAP32[$pgnoChild>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 84|0);
   $27 = HEAP32[$26>>2]|0;
   _ptrmapPut($23,$24,5,$27,$rc);
  }
 }
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$pChild>>2]|0;
  _releasePage($31);
  $32 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $32;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 } else {
  $33 = HEAP32[$pChild>>2]|0;
  $34 = ((($33)) + 22|0);
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 22|0);
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 1|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = $40<<1;
  _memcpy(($34|0),($36|0),($41|0))|0;
  $42 = HEAP32[$pChild>>2]|0;
  $43 = ((($42)) + 32|0);
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 32|0);
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 1|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = $49<<2;
  _memcpy(($43|0),($45|0),($50|0))|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 1|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = HEAP32[$pChild>>2]|0;
  $55 = ((($54)) + 1|0);
  HEAP8[$55>>0] = $53;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pChild>>2]|0;
  $58 = ((($57)) + 56|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = $61 & -9;
  _zeroPage($56,$62);
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 5|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = (($66) + 8)|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = ((($68)) + 56|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($70) + ($67)|0);
  $72 = HEAP32[$pgnoChild>>2]|0;
  _sqlite3Put4byte($71,$72);
  $73 = HEAP32[$pChild>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  HEAP32[$0>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 return (0)|0;
}
function _balance_quick($pParent,$pPage,$pSpace) {
 $pParent = $pParent|0;
 $pPage = $pPage|0;
 $pSpace = $pSpace|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pBt = 0, $pCell = 0, $pNew = 0, $pOut = 0, $pStop = 0, $pgnoNew = 0;
 var $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pBt = sp + 24|0;
 $pNew = sp + 20|0;
 $rc = sp + 16|0;
 $pgnoNew = sp + 12|0;
 $pOut = sp + 8|0;
 $pCell = sp + 4|0;
 $szCell = sp + 44|0;
 $pStop = sp;
 HEAP32[$1>>2] = $pParent;
 HEAP32[$2>>2] = $pPage;
 HEAP32[$3>>2] = $pSpace;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 18|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = (_sqlite3CorruptError(62338)|0);
  HEAP32[$0>>2] = $12;
  $153 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($153|0);
 }
 $13 = HEAP32[$pBt>>2]|0;
 $14 = (_allocateBtreePage($13,$pNew,$pgnoNew,0,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 4|0);
  HEAP32[$pOut>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pCell>>2] = $21;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 76|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$pCell>>2]|0;
  $27 = (FUNCTION_TABLE_iii[$24 & 255]($25,$26)|0);
  HEAP16[$szCell>>1] = $27;
  $28 = HEAP32[$pNew>>2]|0;
  _zeroPage($28,13);
  $29 = HEAP32[$pNew>>2]|0;
  $30 = (_rebuildPage($29,1,$pCell,$szCell)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$rc>>2]|0;
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $33;
   $153 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($153|0);
  }
  $34 = HEAP32[$pBt>>2]|0;
  $35 = ((($34)) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$pNew>>2]|0;
  $38 = ((($37)) + 14|0);
  $39 = HEAP16[$38>>1]|0;
  $40 = $39&65535;
  $41 = (($36) - ($40))|0;
  $42 = (($41) - 2)|0;
  $43 = HEAP16[$szCell>>1]|0;
  $44 = $43&65535;
  $45 = (($42) - ($44))|0;
  $46 = $45&65535;
  $47 = HEAP32[$pNew>>2]|0;
  $48 = ((($47)) + 16|0);
  HEAP16[$48>>1] = $46;
  $49 = HEAP32[$pBt>>2]|0;
  $50 = ((($49)) + 17|0);
  $51 = HEAP8[$50>>0]|0;
  $52 = ($51<<24>>24)!=(0);
  if ($52) {
   $53 = HEAP32[$pBt>>2]|0;
   $54 = HEAP32[$pgnoNew>>2]|0;
   $55 = HEAP32[$1>>2]|0;
   $56 = ((($55)) + 84|0);
   $57 = HEAP32[$56>>2]|0;
   _ptrmapPut($53,$54,5,$57,$rc);
   $58 = HEAP16[$szCell>>1]|0;
   $59 = $58&65535;
   $60 = HEAP32[$pNew>>2]|0;
   $61 = ((($60)) + 12|0);
   $62 = HEAP16[$61>>1]|0;
   $63 = $62&65535;
   $64 = ($59|0)>($63|0);
   if ($64) {
    $65 = HEAP32[$pNew>>2]|0;
    $66 = HEAP32[$pCell>>2]|0;
    _ptrmapPutOvflPtr($65,$66,$rc);
   }
  }
  $67 = HEAP32[$2>>2]|0;
  $68 = ((($67)) + 56|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = ((($70)) + 20|0);
  $72 = HEAP16[$71>>1]|0;
  $73 = $72&65535;
  $74 = HEAP32[$2>>2]|0;
  $75 = ((($74)) + 18|0);
  $76 = HEAP16[$75>>1]|0;
  $77 = $76&65535;
  $78 = (($77) - 1)|0;
  $79 = $78<<1;
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 64|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + ($79)|0);
  $84 = HEAP8[$83>>0]|0;
  $85 = $84&255;
  $86 = $85 << 8;
  $87 = HEAP32[$2>>2]|0;
  $88 = ((($87)) + 18|0);
  $89 = HEAP16[$88>>1]|0;
  $90 = $89&65535;
  $91 = (($90) - 1)|0;
  $92 = $91<<1;
  $93 = HEAP32[$2>>2]|0;
  $94 = ((($93)) + 64|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95) + ($92)|0);
  $97 = ((($96)) + 1|0);
  $98 = HEAP8[$97>>0]|0;
  $99 = $98&255;
  $100 = $86 | $99;
  $101 = $73 & $100;
  $102 = (($69) + ($101)|0);
  HEAP32[$pCell>>2] = $102;
  $103 = HEAP32[$pCell>>2]|0;
  $104 = ((($103)) + 9|0);
  HEAP32[$pStop>>2] = $104;
  while(1) {
   $105 = HEAP32[$pCell>>2]|0;
   $106 = ((($105)) + 1|0);
   HEAP32[$pCell>>2] = $106;
   $107 = HEAP8[$105>>0]|0;
   $108 = $107&255;
   $109 = $108 & 128;
   $110 = ($109|0)!=(0);
   if (!($110)) {
    break;
   }
   $111 = HEAP32[$pCell>>2]|0;
   $112 = HEAP32[$pStop>>2]|0;
   $113 = ($111>>>0)<($112>>>0);
   if (!($113)) {
    break;
   }
  }
  $114 = HEAP32[$pCell>>2]|0;
  $115 = ((($114)) + 9|0);
  HEAP32[$pStop>>2] = $115;
  while(1) {
   $116 = HEAP32[$pCell>>2]|0;
   $117 = ((($116)) + 1|0);
   HEAP32[$pCell>>2] = $117;
   $118 = HEAP8[$116>>0]|0;
   $119 = HEAP32[$pOut>>2]|0;
   $120 = ((($119)) + 1|0);
   HEAP32[$pOut>>2] = $120;
   HEAP8[$119>>0] = $118;
   $121 = $118&255;
   $122 = $121 & 128;
   $123 = ($122|0)!=(0);
   if (!($123)) {
    break;
   }
   $124 = HEAP32[$pCell>>2]|0;
   $125 = HEAP32[$pStop>>2]|0;
   $126 = ($124>>>0)<($125>>>0);
   if (!($126)) {
    break;
   }
  }
  $127 = HEAP32[$1>>2]|0;
  $128 = HEAP32[$1>>2]|0;
  $129 = ((($128)) + 18|0);
  $130 = HEAP16[$129>>1]|0;
  $131 = $130&65535;
  $132 = HEAP32[$3>>2]|0;
  $133 = HEAP32[$pOut>>2]|0;
  $134 = HEAP32[$3>>2]|0;
  $135 = $133;
  $136 = $134;
  $137 = (($135) - ($136))|0;
  $138 = HEAP32[$2>>2]|0;
  $139 = ((($138)) + 84|0);
  $140 = HEAP32[$139>>2]|0;
  _insertCell($127,$131,$132,$137,0,$140,$rc);
  $141 = HEAP32[$1>>2]|0;
  $142 = ((($141)) + 5|0);
  $143 = HEAP8[$142>>0]|0;
  $144 = $143&255;
  $145 = (($144) + 8)|0;
  $146 = HEAP32[$1>>2]|0;
  $147 = ((($146)) + 56|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = (($148) + ($145)|0);
  $150 = HEAP32[$pgnoNew>>2]|0;
  _sqlite3Put4byte($149,$150);
  $151 = HEAP32[$pNew>>2]|0;
  _releasePage($151);
 }
 $152 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $152;
 $153 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _balance_nonroot($pParent,$iParentIdx,$aOvflSpace,$isRoot,$bBulk) {
 $pParent = $pParent|0;
 $iParentIdx = $iParentIdx|0;
 $aOvflSpace = $aOvflSpace|0;
 $isRoot = $isRoot|0;
 $bBulk = $bBulk|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $138 = 0, $139 = 0, $14 = 0;
 var $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0;
 var $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0;
 var $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0;
 var $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0;
 var $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0;
 var $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0;
 var $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0;
 var $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0;
 var $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0;
 var $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0;
 var $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0;
 var $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0;
 var $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0;
 var $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0;
 var $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0;
 var $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0;
 var $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0;
 var $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0;
 var $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0;
 var $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0;
 var $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0;
 var $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0;
 var $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0;
 var $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0;
 var $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0;
 var $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0;
 var $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0;
 var $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0;
 var $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0;
 var $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0;
 var $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0;
 var $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0;
 var $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0;
 var $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0;
 var $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0;
 var $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0;
 var $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0;
 var $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0;
 var $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0;
 var $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0;
 var $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0;
 var $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0;
 var $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0;
 var $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0;
 var $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0;
 var $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0;
 var $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0;
 var $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aData = 0, $aOld = 0, $aPgFlags = 0, $aPgOrder = 0, $aPgno = 0, $aSpace1 = 0, $abDone = 0;
 var $apDiv = 0, $apNew = 0, $apOld = 0, $b = 0, $cntNew = 0, $cntOld = 0, $cntOldNext = 0, $d = 0, $i = 0, $iBest = 0, $iNew = 0, $iNew9 = 0, $iOff = 0, $iOld = 0, $iOld10 = 0, $iOvflSpace = 0, $iPg = 0, $iSpace1 = 0, $info = 0, $j = 0;
 var $k = 0, $key = 0, $leafCorrection = 0, $leafData = 0, $limit = 0, $maskPage = 0, $nMaxCells = 0, $nNew = 0, $nNewCell = 0, $nOld = 0, $nxDiv = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pBt = 0, $pCell = 0, $pCell5 = 0, $pNew = 0, $pNew3 = 0, $pNew8 = 0;
 var $pOld = 0, $pOld2 = 0, $pOld4 = 0, $pRight = 0, $pTemp = 0, $pTemp6 = 0, $pageFlags = 0, $pgno = 0, $piCell = 0, $piEnd = 0, $r = 0, $rc = 0, $sz = 0, $sz1 = 0, $sz7 = 0, $szLeft = 0, $szNew = 0, $szRight = 0, $szScratch = 0, $usableSize = 0;
 var $usableSpace = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 448|0;
 $0 = sp + 412|0;
 $1 = sp + 408|0;
 $2 = sp + 404|0;
 $3 = sp + 400|0;
 $4 = sp + 396|0;
 $5 = sp + 392|0;
 $pBt = sp + 388|0;
 $nMaxCells = sp + 384|0;
 $nNew = sp + 380|0;
 $nOld = sp + 376|0;
 $i = sp + 372|0;
 $j = sp + 368|0;
 $k = sp + 364|0;
 $nxDiv = sp + 360|0;
 $rc = sp + 356|0;
 $leafCorrection = sp + 430|0;
 $leafData = sp + 352|0;
 $usableSpace = sp + 348|0;
 $pageFlags = sp + 344|0;
 $iSpace1 = sp + 340|0;
 $iOvflSpace = sp + 336|0;
 $szScratch = sp + 332|0;
 $apOld = sp + 320|0;
 $apNew = sp + 300|0;
 $pRight = sp + 296|0;
 $apDiv = sp + 288|0;
 $cntNew = sp + 264|0;
 $cntOld = sp + 244|0;
 $szNew = sp + 224|0;
 $aSpace1 = sp + 220|0;
 $pgno = sp + 216|0;
 $abDone = sp + 432|0;
 $aPgno = sp + 196|0;
 $aPgOrder = sp + 176|0;
 $aPgFlags = sp + 420|0;
 $b = sp + 160|0;
 $iOff = sp + 152|0;
 $pOld = sp + 148|0;
 $limit = sp + 144|0;
 $aData = sp + 140|0;
 $maskPage = sp + 418|0;
 $piCell = sp + 136|0;
 $piEnd = sp + 132|0;
 $sz = sp + 416|0;
 $pTemp = sp + 128|0;
 $p = sp + 124|0;
 $sz1 = sp + 120|0;
 $szRight = sp + 116|0;
 $szLeft = sp + 112|0;
 $r = sp + 108|0;
 $d = sp + 104|0;
 $pNew = sp + 100|0;
 $iBest = sp + 96|0;
 $pOld2 = sp + 92|0;
 $pNew3 = sp + 88|0;
 $aOld = sp + 84|0;
 $cntOldNext = sp + 80|0;
 $usableSize = sp + 76|0;
 $iNew = sp + 72|0;
 $iOld = sp + 68|0;
 $pCell = sp + 64|0;
 $pOld4 = sp + 60|0;
 $pCell5 = sp + 56|0;
 $pTemp6 = sp + 52|0;
 $sz7 = sp + 48|0;
 $pNew8 = sp + 44|0;
 $info = sp;
 $iPg = sp + 40|0;
 $iNew9 = sp + 36|0;
 $iOld10 = sp + 32|0;
 $nNewCell = sp + 28|0;
 $key = sp + 24|0;
 HEAP32[$1>>2] = $pParent;
 HEAP32[$2>>2] = $iParentIdx;
 HEAP32[$3>>2] = $aOvflSpace;
 HEAP32[$4>>2] = $isRoot;
 HEAP32[$5>>2] = $bBulk;
 HEAP32[$nMaxCells>>2] = 0;
 HEAP32[$nNew>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iSpace1>>2] = 0;
 HEAP32[$iOvflSpace>>2] = 0;
 ;HEAP8[$abDone>>0]=0|0;HEAP8[$abDone+1>>0]=0|0;HEAP8[$abDone+2>>0]=0|0;HEAP8[$abDone+3>>0]=0|0;HEAP8[$abDone+4>>0]=0|0;
 HEAP32[$b>>2] = 0;
 $6 = ((($b)) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pBt>>2] = $9;
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  HEAP32[$0>>2] = 7;
  $1372 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($1372|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 1|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 18|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($15) + ($19))|0;
 HEAP32[$i>>2] = $20;
 $21 = HEAP32[$i>>2]|0;
 $22 = ($21|0)<(2);
 if ($22) {
  HEAP32[$nxDiv>>2] = 0;
 } else {
  $23 = HEAP32[$2>>2]|0;
  $24 = ($23|0)==(0);
  do {
   if ($24) {
    HEAP32[$nxDiv>>2] = 0;
   } else {
    $25 = HEAP32[$2>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    $27 = ($25|0)==($26|0);
    if ($27) {
     $28 = HEAP32[$i>>2]|0;
     $29 = (($28) - 2)|0;
     $30 = HEAP32[$5>>2]|0;
     $31 = (($29) + ($30))|0;
     HEAP32[$nxDiv>>2] = $31;
     break;
    } else {
     $32 = HEAP32[$2>>2]|0;
     $33 = (($32) - 1)|0;
     HEAP32[$nxDiv>>2] = $33;
     break;
    }
   }
  } while(0);
  $34 = HEAP32[$5>>2]|0;
  $35 = (2 - ($34))|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$i>>2]|0;
 $37 = (($36) + 1)|0;
 HEAP32[$nOld>>2] = $37;
 $38 = HEAP32[$i>>2]|0;
 $39 = HEAP32[$nxDiv>>2]|0;
 $40 = (($38) + ($39))|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 1|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43&255;
 $45 = (($40) - ($44))|0;
 $46 = HEAP32[$1>>2]|0;
 $47 = ((($46)) + 18|0);
 $48 = HEAP16[$47>>1]|0;
 $49 = $48&65535;
 $50 = ($45|0)==($49|0);
 $51 = HEAP32[$1>>2]|0;
 if ($50) {
  $52 = ((($51)) + 5|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = (($54) + 8)|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 56|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($55)|0);
  HEAP32[$pRight>>2] = $59;
 } else {
  $60 = ((($51)) + 56|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 20|0);
  $64 = HEAP16[$63>>1]|0;
  $65 = $64&65535;
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$nxDiv>>2]|0;
  $68 = (($66) + ($67))|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 1|0);
  $71 = HEAP8[$70>>0]|0;
  $72 = $71&255;
  $73 = (($68) - ($72))|0;
  $74 = $73<<1;
  $75 = HEAP32[$1>>2]|0;
  $76 = ((($75)) + 64|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (($77) + ($74)|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $80 << 8;
  $82 = HEAP32[$i>>2]|0;
  $83 = HEAP32[$nxDiv>>2]|0;
  $84 = (($82) + ($83))|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 1|0);
  $87 = HEAP8[$86>>0]|0;
  $88 = $87&255;
  $89 = (($84) - ($88))|0;
  $90 = $89<<1;
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 64|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + ($90)|0);
  $95 = ((($94)) + 1|0);
  $96 = HEAP8[$95>>0]|0;
  $97 = $96&255;
  $98 = $81 | $97;
  $99 = $65 & $98;
  $100 = (($61) + ($99)|0);
  HEAP32[$pRight>>2] = $100;
 }
 $101 = HEAP32[$pRight>>2]|0;
 $102 = (_sqlite3Get4byte($101)|0);
 HEAP32[$pgno>>2] = $102;
 while(1) {
  $103 = HEAP32[$pBt>>2]|0;
  $104 = HEAP32[$pgno>>2]|0;
  $105 = HEAP32[$i>>2]|0;
  $106 = (($apOld) + ($105<<2)|0);
  $107 = (_getAndInitPage($103,$104,$106,0,0)|0);
  HEAP32[$rc>>2] = $107;
  $108 = HEAP32[$rc>>2]|0;
  $109 = ($108|0)!=(0);
  if ($109) {
   label = 16;
   break;
  }
  $113 = HEAP32[$i>>2]|0;
  $114 = (($apOld) + ($113<<2)|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = ((($115)) + 18|0);
  $117 = HEAP16[$116>>1]|0;
  $118 = $117&65535;
  $119 = (1 + ($118))|0;
  $120 = HEAP32[$i>>2]|0;
  $121 = (($apOld) + ($120<<2)|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = ((($122)) + 1|0);
  $124 = HEAP8[$123>>0]|0;
  $125 = $124&255;
  $126 = (($119) + ($125))|0;
  $127 = HEAP32[$nMaxCells>>2]|0;
  $128 = (($127) + ($126))|0;
  HEAP32[$nMaxCells>>2] = $128;
  $129 = HEAP32[$i>>2]|0;
  $130 = (($129) + -1)|0;
  HEAP32[$i>>2] = $130;
  $131 = ($129|0)==(0);
  if ($131) {
   label = 26;
   break;
  }
  $132 = HEAP32[$i>>2]|0;
  $133 = HEAP32[$nxDiv>>2]|0;
  $134 = (($132) + ($133))|0;
  $135 = HEAP32[$1>>2]|0;
  $136 = ((($135)) + 22|0);
  $137 = HEAP16[$136>>1]|0;
  $138 = $137&65535;
  $139 = ($134|0)==($138|0);
  if ($139) {
   $140 = HEAP32[$1>>2]|0;
   $141 = ((($140)) + 1|0);
   $142 = HEAP8[$141>>0]|0;
   $143 = $142&255;
   $144 = ($143|0)!=(0);
   if ($144) {
    $145 = HEAP32[$1>>2]|0;
    $146 = ((($145)) + 32|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = HEAP32[$i>>2]|0;
    $149 = (($apDiv) + ($148<<2)|0);
    HEAP32[$149>>2] = $147;
    $150 = HEAP32[$i>>2]|0;
    $151 = (($apDiv) + ($150<<2)|0);
    $152 = HEAP32[$151>>2]|0;
    $153 = (_sqlite3Get4byte($152)|0);
    HEAP32[$pgno>>2] = $153;
    $154 = HEAP32[$1>>2]|0;
    $155 = ((($154)) + 76|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = HEAP32[$1>>2]|0;
    $158 = HEAP32[$i>>2]|0;
    $159 = (($apDiv) + ($158<<2)|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (FUNCTION_TABLE_iii[$156 & 255]($157,$160)|0);
    $162 = $161&65535;
    $163 = HEAP32[$i>>2]|0;
    $164 = (($szNew) + ($163<<2)|0);
    HEAP32[$164>>2] = $162;
    $165 = HEAP32[$1>>2]|0;
    $166 = ((($165)) + 1|0);
    HEAP8[$166>>0] = 0;
    continue;
   }
  }
  $167 = HEAP32[$1>>2]|0;
  $168 = ((($167)) + 56|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = HEAP32[$1>>2]|0;
  $171 = ((($170)) + 20|0);
  $172 = HEAP16[$171>>1]|0;
  $173 = $172&65535;
  $174 = HEAP32[$i>>2]|0;
  $175 = HEAP32[$nxDiv>>2]|0;
  $176 = (($174) + ($175))|0;
  $177 = HEAP32[$1>>2]|0;
  $178 = ((($177)) + 1|0);
  $179 = HEAP8[$178>>0]|0;
  $180 = $179&255;
  $181 = (($176) - ($180))|0;
  $182 = $181<<1;
  $183 = HEAP32[$1>>2]|0;
  $184 = ((($183)) + 64|0);
  $185 = HEAP32[$184>>2]|0;
  $186 = (($185) + ($182)|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $187&255;
  $189 = $188 << 8;
  $190 = HEAP32[$i>>2]|0;
  $191 = HEAP32[$nxDiv>>2]|0;
  $192 = (($190) + ($191))|0;
  $193 = HEAP32[$1>>2]|0;
  $194 = ((($193)) + 1|0);
  $195 = HEAP8[$194>>0]|0;
  $196 = $195&255;
  $197 = (($192) - ($196))|0;
  $198 = $197<<1;
  $199 = HEAP32[$1>>2]|0;
  $200 = ((($199)) + 64|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = (($201) + ($198)|0);
  $203 = ((($202)) + 1|0);
  $204 = HEAP8[$203>>0]|0;
  $205 = $204&255;
  $206 = $189 | $205;
  $207 = $173 & $206;
  $208 = (($169) + ($207)|0);
  $209 = HEAP32[$i>>2]|0;
  $210 = (($apDiv) + ($209<<2)|0);
  HEAP32[$210>>2] = $208;
  $211 = HEAP32[$i>>2]|0;
  $212 = (($apDiv) + ($211<<2)|0);
  $213 = HEAP32[$212>>2]|0;
  $214 = (_sqlite3Get4byte($213)|0);
  HEAP32[$pgno>>2] = $214;
  $215 = HEAP32[$1>>2]|0;
  $216 = ((($215)) + 76|0);
  $217 = HEAP32[$216>>2]|0;
  $218 = HEAP32[$1>>2]|0;
  $219 = HEAP32[$i>>2]|0;
  $220 = (($apDiv) + ($219<<2)|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = (FUNCTION_TABLE_iii[$217 & 255]($218,$221)|0);
  $223 = $222&65535;
  $224 = HEAP32[$i>>2]|0;
  $225 = (($szNew) + ($224<<2)|0);
  HEAP32[$225>>2] = $223;
  $226 = HEAP32[$pBt>>2]|0;
  $227 = ((($226)) + 22|0);
  $228 = HEAP16[$227>>1]|0;
  $229 = $228&65535;
  $230 = $229 & 4;
  $231 = ($230|0)!=(0);
  if ($231) {
   $232 = HEAP32[$i>>2]|0;
   $233 = (($apDiv) + ($232<<2)|0);
   $234 = HEAP32[$233>>2]|0;
   $235 = $234;
   $236 = HEAP32[$1>>2]|0;
   $237 = ((($236)) + 56|0);
   $238 = HEAP32[$237>>2]|0;
   $239 = $238;
   $240 = (($235) - ($239))|0;
   HEAP32[$iOff>>2] = $240;
   $241 = HEAP32[$iOff>>2]|0;
   $242 = HEAP32[$i>>2]|0;
   $243 = (($szNew) + ($242<<2)|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (($241) + ($244))|0;
   $246 = HEAP32[$pBt>>2]|0;
   $247 = ((($246)) + 36|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = ($245|0)>($248|0);
   if ($249) {
    label = 23;
    break;
   }
   $254 = HEAP32[$iOff>>2]|0;
   $255 = HEAP32[$3>>2]|0;
   $256 = (($255) + ($254)|0);
   $257 = HEAP32[$i>>2]|0;
   $258 = (($apDiv) + ($257<<2)|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = HEAP32[$i>>2]|0;
   $261 = (($szNew) + ($260<<2)|0);
   $262 = HEAP32[$261>>2]|0;
   _memcpy(($256|0),($259|0),($262|0))|0;
   $263 = HEAP32[$i>>2]|0;
   $264 = (($apDiv) + ($263<<2)|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = HEAP32[$1>>2]|0;
   $267 = ((($266)) + 56|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = $265;
   $270 = $268;
   $271 = (($269) - ($270))|0;
   $272 = HEAP32[$3>>2]|0;
   $273 = (($272) + ($271)|0);
   $274 = HEAP32[$i>>2]|0;
   $275 = (($apDiv) + ($274<<2)|0);
   HEAP32[$275>>2] = $273;
  }
  $276 = HEAP32[$1>>2]|0;
  $277 = HEAP32[$i>>2]|0;
  $278 = HEAP32[$nxDiv>>2]|0;
  $279 = (($277) + ($278))|0;
  $280 = HEAP32[$1>>2]|0;
  $281 = ((($280)) + 1|0);
  $282 = HEAP8[$281>>0]|0;
  $283 = $282&255;
  $284 = (($279) - ($283))|0;
  $285 = HEAP32[$i>>2]|0;
  $286 = (($szNew) + ($285<<2)|0);
  $287 = HEAP32[$286>>2]|0;
  _dropCell($276,$284,$287,$rc);
 }
 L32: do {
  if ((label|0) == 16) {
   $110 = HEAP32[$i>>2]|0;
   $111 = (($110) + 1)|0;
   $112 = $111<<2;
   _memset(($apOld|0),0,($112|0))|0;
  }
  else if ((label|0) == 23) {
   $250 = (_sqlite3CorruptError(62677)|0);
   HEAP32[$rc>>2] = $250;
   $251 = HEAP32[$i>>2]|0;
   $252 = (($251) + 1)|0;
   $253 = $252<<2;
   _memset(($apOld|0),0,($253|0))|0;
  }
  else if ((label|0) == 26) {
   $288 = HEAP32[$nMaxCells>>2]|0;
   $289 = (($288) + 3)|0;
   $290 = $289 & -4;
   HEAP32[$nMaxCells>>2] = $290;
   $291 = HEAP32[$nMaxCells>>2]|0;
   $292 = $291<<2;
   $293 = HEAP32[$nMaxCells>>2]|0;
   $294 = $293<<1;
   $295 = (($292) + ($294))|0;
   $296 = HEAP32[$pBt>>2]|0;
   $297 = ((($296)) + 32|0);
   $298 = HEAP32[$297>>2]|0;
   $299 = (($295) + ($298))|0;
   HEAP32[$szScratch>>2] = $299;
   $300 = HEAP32[$szScratch>>2]|0;
   $301 = (_sqlite3ScratchMalloc($300)|0);
   $302 = ((($b)) + 8|0);
   HEAP32[$302>>2] = $301;
   $303 = ((($b)) + 8|0);
   $304 = HEAP32[$303>>2]|0;
   $305 = ($304|0)==(0|0);
   if ($305) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $306 = HEAP32[$nMaxCells>>2]|0;
   $307 = ((($b)) + 8|0);
   $308 = HEAP32[$307>>2]|0;
   $309 = (($308) + ($306<<2)|0);
   $310 = ((($b)) + 12|0);
   HEAP32[$310>>2] = $309;
   $311 = HEAP32[$nMaxCells>>2]|0;
   $312 = ((($b)) + 12|0);
   $313 = HEAP32[$312>>2]|0;
   $314 = (($313) + ($311<<1)|0);
   HEAP32[$aSpace1>>2] = $314;
   $315 = HEAP32[$apOld>>2]|0;
   $316 = ((($b)) + 4|0);
   HEAP32[$316>>2] = $315;
   $317 = ((($b)) + 4|0);
   $318 = HEAP32[$317>>2]|0;
   $319 = ((($318)) + 4|0);
   $320 = HEAP8[$319>>0]|0;
   $321 = $320&255;
   $322 = $321<<2;
   $323 = $322&65535;
   HEAP16[$leafCorrection>>1] = $323;
   $324 = ((($b)) + 4|0);
   $325 = HEAP32[$324>>2]|0;
   $326 = ((($325)) + 3|0);
   $327 = HEAP8[$326>>0]|0;
   $328 = $327&255;
   HEAP32[$leafData>>2] = $328;
   HEAP32[$i>>2] = 0;
   while(1) {
    $329 = HEAP32[$i>>2]|0;
    $330 = HEAP32[$nOld>>2]|0;
    $331 = ($329|0)<($330|0);
    if (!($331)) {
     break;
    }
    $332 = HEAP32[$i>>2]|0;
    $333 = (($apOld) + ($332<<2)|0);
    $334 = HEAP32[$333>>2]|0;
    HEAP32[$pOld>>2] = $334;
    $335 = HEAP32[$pOld>>2]|0;
    $336 = ((($335)) + 18|0);
    $337 = HEAP16[$336>>1]|0;
    $338 = $337&65535;
    HEAP32[$limit>>2] = $338;
    $339 = HEAP32[$pOld>>2]|0;
    $340 = ((($339)) + 56|0);
    $341 = HEAP32[$340>>2]|0;
    HEAP32[$aData>>2] = $341;
    $342 = HEAP32[$pOld>>2]|0;
    $343 = ((($342)) + 20|0);
    $344 = HEAP16[$343>>1]|0;
    HEAP16[$maskPage>>1] = $344;
    $345 = HEAP32[$aData>>2]|0;
    $346 = HEAP32[$pOld>>2]|0;
    $347 = ((($346)) + 14|0);
    $348 = HEAP16[$347>>1]|0;
    $349 = $348&65535;
    $350 = (($345) + ($349)|0);
    HEAP32[$piCell>>2] = $350;
    $351 = HEAP32[$pOld>>2]|0;
    $352 = ((($351)) + 56|0);
    $353 = HEAP32[$352>>2]|0;
    $354 = HEAP8[$353>>0]|0;
    $355 = $354&255;
    $356 = HEAP32[$apOld>>2]|0;
    $357 = ((($356)) + 56|0);
    $358 = HEAP32[$357>>2]|0;
    $359 = HEAP8[$358>>0]|0;
    $360 = $359&255;
    $361 = ($355|0)!=($360|0);
    if ($361) {
     label = 31;
     break;
    }
    $363 = HEAP32[$b>>2]|0;
    $364 = ((($b)) + 12|0);
    $365 = HEAP32[$364>>2]|0;
    $366 = (($365) + ($363<<1)|0);
    $367 = HEAP32[$limit>>2]|0;
    $368 = HEAP32[$pOld>>2]|0;
    $369 = ((($368)) + 1|0);
    $370 = HEAP8[$369>>0]|0;
    $371 = $370&255;
    $372 = (($367) + ($371))|0;
    $373 = $372<<1;
    _memset(($366|0),0,($373|0))|0;
    $374 = HEAP32[$pOld>>2]|0;
    $375 = ((($374)) + 1|0);
    $376 = HEAP8[$375>>0]|0;
    $377 = $376&255;
    $378 = ($377|0)>(0);
    L43: do {
     if ($378) {
      $379 = HEAP32[$pOld>>2]|0;
      $380 = ((($379)) + 22|0);
      $381 = HEAP16[$380>>1]|0;
      $382 = $381&65535;
      HEAP32[$limit>>2] = $382;
      HEAP32[$j>>2] = 0;
      while(1) {
       $383 = HEAP32[$j>>2]|0;
       $384 = HEAP32[$limit>>2]|0;
       $385 = ($383|0)<($384|0);
       if (!($385)) {
        break;
       }
       $386 = HEAP32[$aData>>2]|0;
       $387 = HEAP16[$maskPage>>1]|0;
       $388 = $387&65535;
       $389 = HEAP32[$piCell>>2]|0;
       $390 = HEAP8[$389>>0]|0;
       $391 = $390&255;
       $392 = $391 << 8;
       $393 = HEAP32[$piCell>>2]|0;
       $394 = ((($393)) + 1|0);
       $395 = HEAP8[$394>>0]|0;
       $396 = $395&255;
       $397 = $392 | $396;
       $398 = $388 & $397;
       $399 = (($386) + ($398)|0);
       $400 = HEAP32[$b>>2]|0;
       $401 = ((($b)) + 8|0);
       $402 = HEAP32[$401>>2]|0;
       $403 = (($402) + ($400<<2)|0);
       HEAP32[$403>>2] = $399;
       $404 = HEAP32[$piCell>>2]|0;
       $405 = ((($404)) + 2|0);
       HEAP32[$piCell>>2] = $405;
       $406 = HEAP32[$b>>2]|0;
       $407 = (($406) + 1)|0;
       HEAP32[$b>>2] = $407;
       $408 = HEAP32[$j>>2]|0;
       $409 = (($408) + 1)|0;
       HEAP32[$j>>2] = $409;
      }
      HEAP32[$k>>2] = 0;
      while(1) {
       $410 = HEAP32[$k>>2]|0;
       $411 = HEAP32[$pOld>>2]|0;
       $412 = ((($411)) + 1|0);
       $413 = HEAP8[$412>>0]|0;
       $414 = $413&255;
       $415 = ($410|0)<($414|0);
       if (!($415)) {
        break L43;
       }
       $416 = HEAP32[$k>>2]|0;
       $417 = HEAP32[$pOld>>2]|0;
       $418 = ((($417)) + 32|0);
       $419 = (($418) + ($416<<2)|0);
       $420 = HEAP32[$419>>2]|0;
       $421 = HEAP32[$b>>2]|0;
       $422 = ((($b)) + 8|0);
       $423 = HEAP32[$422>>2]|0;
       $424 = (($423) + ($421<<2)|0);
       HEAP32[$424>>2] = $420;
       $425 = HEAP32[$b>>2]|0;
       $426 = (($425) + 1)|0;
       HEAP32[$b>>2] = $426;
       $427 = HEAP32[$k>>2]|0;
       $428 = (($427) + 1)|0;
       HEAP32[$k>>2] = $428;
      }
     }
    } while(0);
    $429 = HEAP32[$aData>>2]|0;
    $430 = HEAP32[$pOld>>2]|0;
    $431 = ((($430)) + 14|0);
    $432 = HEAP16[$431>>1]|0;
    $433 = $432&65535;
    $434 = (($429) + ($433)|0);
    $435 = HEAP32[$pOld>>2]|0;
    $436 = ((($435)) + 18|0);
    $437 = HEAP16[$436>>1]|0;
    $438 = $437&65535;
    $439 = $438<<1;
    $440 = (($434) + ($439)|0);
    HEAP32[$piEnd>>2] = $440;
    while(1) {
     $441 = HEAP32[$piCell>>2]|0;
     $442 = HEAP32[$piEnd>>2]|0;
     $443 = ($441>>>0)<($442>>>0);
     if (!($443)) {
      break;
     }
     $444 = HEAP32[$aData>>2]|0;
     $445 = HEAP16[$maskPage>>1]|0;
     $446 = $445&65535;
     $447 = HEAP32[$piCell>>2]|0;
     $448 = HEAP8[$447>>0]|0;
     $449 = $448&255;
     $450 = $449 << 8;
     $451 = HEAP32[$piCell>>2]|0;
     $452 = ((($451)) + 1|0);
     $453 = HEAP8[$452>>0]|0;
     $454 = $453&255;
     $455 = $450 | $454;
     $456 = $446 & $455;
     $457 = (($444) + ($456)|0);
     $458 = HEAP32[$b>>2]|0;
     $459 = ((($b)) + 8|0);
     $460 = HEAP32[$459>>2]|0;
     $461 = (($460) + ($458<<2)|0);
     HEAP32[$461>>2] = $457;
     $462 = HEAP32[$piCell>>2]|0;
     $463 = ((($462)) + 2|0);
     HEAP32[$piCell>>2] = $463;
     $464 = HEAP32[$b>>2]|0;
     $465 = (($464) + 1)|0;
     HEAP32[$b>>2] = $465;
    }
    $466 = HEAP32[$b>>2]|0;
    $467 = HEAP32[$i>>2]|0;
    $468 = (($cntOld) + ($467<<2)|0);
    HEAP32[$468>>2] = $466;
    $469 = HEAP32[$i>>2]|0;
    $470 = HEAP32[$nOld>>2]|0;
    $471 = (($470) - 1)|0;
    $472 = ($469|0)>=($471|0);
    $473 = HEAP32[$leafData>>2]|0;
    $474 = ($473|0)!=(0);
    $or$cond = $472 | $474;
    if (!($or$cond)) {
     $475 = HEAP32[$i>>2]|0;
     $476 = (($szNew) + ($475<<2)|0);
     $477 = HEAP32[$476>>2]|0;
     $478 = $477&65535;
     HEAP16[$sz>>1] = $478;
     $479 = HEAP16[$sz>>1]|0;
     $480 = HEAP32[$b>>2]|0;
     $481 = ((($b)) + 12|0);
     $482 = HEAP32[$481>>2]|0;
     $483 = (($482) + ($480<<1)|0);
     HEAP16[$483>>1] = $479;
     $484 = HEAP32[$iSpace1>>2]|0;
     $485 = HEAP32[$aSpace1>>2]|0;
     $486 = (($485) + ($484)|0);
     HEAP32[$pTemp>>2] = $486;
     $487 = HEAP16[$sz>>1]|0;
     $488 = $487&65535;
     $489 = HEAP32[$iSpace1>>2]|0;
     $490 = (($489) + ($488))|0;
     HEAP32[$iSpace1>>2] = $490;
     $491 = HEAP32[$pTemp>>2]|0;
     $492 = HEAP32[$i>>2]|0;
     $493 = (($apDiv) + ($492<<2)|0);
     $494 = HEAP32[$493>>2]|0;
     $495 = HEAP16[$sz>>1]|0;
     $496 = $495&65535;
     _memcpy(($491|0),($494|0),($496|0))|0;
     $497 = HEAP32[$pTemp>>2]|0;
     $498 = HEAP16[$leafCorrection>>1]|0;
     $499 = $498&65535;
     $500 = (($497) + ($499)|0);
     $501 = HEAP32[$b>>2]|0;
     $502 = ((($b)) + 8|0);
     $503 = HEAP32[$502>>2]|0;
     $504 = (($503) + ($501<<2)|0);
     HEAP32[$504>>2] = $500;
     $505 = HEAP32[$b>>2]|0;
     $506 = ((($b)) + 12|0);
     $507 = HEAP32[$506>>2]|0;
     $508 = (($507) + ($505<<1)|0);
     $509 = HEAP16[$508>>1]|0;
     $510 = $509&65535;
     $511 = HEAP16[$leafCorrection>>1]|0;
     $512 = $511&65535;
     $513 = (($510) - ($512))|0;
     $514 = $513&65535;
     $515 = HEAP32[$b>>2]|0;
     $516 = ((($b)) + 12|0);
     $517 = HEAP32[$516>>2]|0;
     $518 = (($517) + ($515<<1)|0);
     HEAP16[$518>>1] = $514;
     $519 = HEAP32[$pOld>>2]|0;
     $520 = ((($519)) + 4|0);
     $521 = HEAP8[$520>>0]|0;
     $522 = ($521<<24>>24)!=(0);
     L59: do {
      if ($522) {
       while(1) {
        $532 = HEAP32[$b>>2]|0;
        $533 = ((($b)) + 12|0);
        $534 = HEAP32[$533>>2]|0;
        $535 = (($534) + ($532<<1)|0);
        $536 = HEAP16[$535>>1]|0;
        $537 = $536&65535;
        $538 = ($537|0)<(4);
        if (!($538)) {
         break L59;
        }
        $539 = HEAP32[$iSpace1>>2]|0;
        $540 = (($539) + 1)|0;
        HEAP32[$iSpace1>>2] = $540;
        $541 = HEAP32[$aSpace1>>2]|0;
        $542 = (($541) + ($539)|0);
        HEAP8[$542>>0] = 0;
        $543 = HEAP32[$b>>2]|0;
        $544 = ((($b)) + 12|0);
        $545 = HEAP32[$544>>2]|0;
        $546 = (($545) + ($543<<1)|0);
        $547 = HEAP16[$546>>1]|0;
        $548 = (($547) + 1)<<16>>16;
        HEAP16[$546>>1] = $548;
       }
      } else {
       $523 = HEAP32[$b>>2]|0;
       $524 = ((($b)) + 8|0);
       $525 = HEAP32[$524>>2]|0;
       $526 = (($525) + ($523<<2)|0);
       $527 = HEAP32[$526>>2]|0;
       $528 = HEAP32[$pOld>>2]|0;
       $529 = ((($528)) + 56|0);
       $530 = HEAP32[$529>>2]|0;
       $531 = ((($530)) + 8|0);
       ;HEAP8[$527>>0]=HEAP8[$531>>0]|0;HEAP8[$527+1>>0]=HEAP8[$531+1>>0]|0;HEAP8[$527+2>>0]=HEAP8[$531+2>>0]|0;HEAP8[$527+3>>0]=HEAP8[$531+3>>0]|0;
      }
     } while(0);
     $549 = HEAP32[$b>>2]|0;
     $550 = (($549) + 1)|0;
     HEAP32[$b>>2] = $550;
    }
    $551 = HEAP32[$i>>2]|0;
    $552 = (($551) + 1)|0;
    HEAP32[$i>>2] = $552;
   }
   if ((label|0) == 31) {
    $362 = (_sqlite3CorruptError(62744)|0);
    HEAP32[$rc>>2] = $362;
    break;
   }
   $553 = HEAP32[$pBt>>2]|0;
   $554 = ((($553)) + 36|0);
   $555 = HEAP32[$554>>2]|0;
   $556 = (($555) - 12)|0;
   $557 = HEAP16[$leafCorrection>>1]|0;
   $558 = $557&65535;
   $559 = (($556) + ($558))|0;
   HEAP32[$usableSpace>>2] = $559;
   HEAP32[$i>>2] = 0;
   while(1) {
    $560 = HEAP32[$i>>2]|0;
    $561 = HEAP32[$nOld>>2]|0;
    $562 = ($560|0)<($561|0);
    if (!($562)) {
     break;
    }
    $563 = HEAP32[$i>>2]|0;
    $564 = (($apOld) + ($563<<2)|0);
    $565 = HEAP32[$564>>2]|0;
    HEAP32[$p>>2] = $565;
    $566 = HEAP32[$usableSpace>>2]|0;
    $567 = HEAP32[$p>>2]|0;
    $568 = ((($567)) + 16|0);
    $569 = HEAP16[$568>>1]|0;
    $570 = $569&65535;
    $571 = (($566) - ($570))|0;
    $572 = HEAP32[$i>>2]|0;
    $573 = (($szNew) + ($572<<2)|0);
    HEAP32[$573>>2] = $571;
    $574 = HEAP32[$i>>2]|0;
    $575 = (($szNew) + ($574<<2)|0);
    $576 = HEAP32[$575>>2]|0;
    $577 = ($576|0)<(0);
    if ($577) {
     label = 52;
     break;
    }
    HEAP32[$j>>2] = 0;
    while(1) {
     $579 = HEAP32[$j>>2]|0;
     $580 = HEAP32[$p>>2]|0;
     $581 = ((($580)) + 1|0);
     $582 = HEAP8[$581>>0]|0;
     $583 = $582&255;
     $584 = ($579|0)<($583|0);
     if (!($584)) {
      break;
     }
     $585 = HEAP32[$p>>2]|0;
     $586 = ((($585)) + 76|0);
     $587 = HEAP32[$586>>2]|0;
     $588 = HEAP32[$p>>2]|0;
     $589 = HEAP32[$j>>2]|0;
     $590 = HEAP32[$p>>2]|0;
     $591 = ((($590)) + 32|0);
     $592 = (($591) + ($589<<2)|0);
     $593 = HEAP32[$592>>2]|0;
     $594 = (FUNCTION_TABLE_iii[$587 & 255]($588,$593)|0);
     $595 = $594&65535;
     $596 = (2 + ($595))|0;
     $597 = HEAP32[$i>>2]|0;
     $598 = (($szNew) + ($597<<2)|0);
     $599 = HEAP32[$598>>2]|0;
     $600 = (($599) + ($596))|0;
     HEAP32[$598>>2] = $600;
     $601 = HEAP32[$j>>2]|0;
     $602 = (($601) + 1)|0;
     HEAP32[$j>>2] = $602;
    }
    $603 = HEAP32[$i>>2]|0;
    $604 = (($cntOld) + ($603<<2)|0);
    $605 = HEAP32[$604>>2]|0;
    $606 = HEAP32[$i>>2]|0;
    $607 = (($cntNew) + ($606<<2)|0);
    HEAP32[$607>>2] = $605;
    $608 = HEAP32[$i>>2]|0;
    $609 = (($608) + 1)|0;
    HEAP32[$i>>2] = $609;
   }
   if ((label|0) == 52) {
    $578 = (_sqlite3CorruptError(62842)|0);
    HEAP32[$rc>>2] = $578;
    break;
   }
   $610 = HEAP32[$nOld>>2]|0;
   HEAP32[$k>>2] = $610;
   HEAP32[$i>>2] = 0;
   L80: while(1) {
    $611 = HEAP32[$i>>2]|0;
    $612 = HEAP32[$k>>2]|0;
    $613 = ($611|0)<($612|0);
    if (!($613)) {
     label = 83;
     break;
    }
    while(1) {
     $614 = HEAP32[$i>>2]|0;
     $615 = (($szNew) + ($614<<2)|0);
     $616 = HEAP32[$615>>2]|0;
     $617 = HEAP32[$usableSpace>>2]|0;
     $618 = ($616|0)>($617|0);
     if (!($618)) {
      break;
     }
     $619 = HEAP32[$i>>2]|0;
     $620 = (($619) + 1)|0;
     $621 = HEAP32[$k>>2]|0;
     $622 = ($620|0)>=($621|0);
     if ($622) {
      $623 = HEAP32[$i>>2]|0;
      $624 = (($623) + 2)|0;
      HEAP32[$k>>2] = $624;
      $625 = HEAP32[$k>>2]|0;
      $626 = ($625|0)>(5);
      if ($626) {
       label = 62;
       break L80;
      }
      $628 = HEAP32[$k>>2]|0;
      $629 = (($628) - 1)|0;
      $630 = (($szNew) + ($629<<2)|0);
      HEAP32[$630>>2] = 0;
      $631 = HEAP32[$b>>2]|0;
      $632 = HEAP32[$k>>2]|0;
      $633 = (($632) - 1)|0;
      $634 = (($cntNew) + ($633<<2)|0);
      HEAP32[$634>>2] = $631;
     }
     $635 = HEAP32[$i>>2]|0;
     $636 = (($cntNew) + ($635<<2)|0);
     $637 = HEAP32[$636>>2]|0;
     $638 = (($637) - 1)|0;
     $639 = (_cachedCellSize($b,$638)|0);
     $640 = $639&65535;
     $641 = (2 + ($640))|0;
     HEAP32[$sz1>>2] = $641;
     $642 = HEAP32[$sz1>>2]|0;
     $643 = HEAP32[$i>>2]|0;
     $644 = (($szNew) + ($643<<2)|0);
     $645 = HEAP32[$644>>2]|0;
     $646 = (($645) - ($642))|0;
     HEAP32[$644>>2] = $646;
     $647 = HEAP32[$leafData>>2]|0;
     $648 = ($647|0)!=(0);
     do {
      if (!($648)) {
       $649 = HEAP32[$i>>2]|0;
       $650 = (($cntNew) + ($649<<2)|0);
       $651 = HEAP32[$650>>2]|0;
       $652 = HEAP32[$b>>2]|0;
       $653 = ($651|0)<($652|0);
       if ($653) {
        $654 = HEAP32[$i>>2]|0;
        $655 = (($cntNew) + ($654<<2)|0);
        $656 = HEAP32[$655>>2]|0;
        $657 = (_cachedCellSize($b,$656)|0);
        $658 = $657&65535;
        $659 = (2 + ($658))|0;
        HEAP32[$sz1>>2] = $659;
        break;
       } else {
        HEAP32[$sz1>>2] = 0;
        break;
       }
      }
     } while(0);
     $660 = HEAP32[$sz1>>2]|0;
     $661 = HEAP32[$i>>2]|0;
     $662 = (($661) + 1)|0;
     $663 = (($szNew) + ($662<<2)|0);
     $664 = HEAP32[$663>>2]|0;
     $665 = (($664) + ($660))|0;
     HEAP32[$663>>2] = $665;
     $666 = HEAP32[$i>>2]|0;
     $667 = (($cntNew) + ($666<<2)|0);
     $668 = HEAP32[$667>>2]|0;
     $669 = (($668) + -1)|0;
     HEAP32[$667>>2] = $669;
    }
    while(1) {
     $670 = HEAP32[$i>>2]|0;
     $671 = (($cntNew) + ($670<<2)|0);
     $672 = HEAP32[$671>>2]|0;
     $673 = HEAP32[$b>>2]|0;
     $674 = ($672|0)<($673|0);
     if (!($674)) {
      break;
     }
     $675 = HEAP32[$i>>2]|0;
     $676 = (($cntNew) + ($675<<2)|0);
     $677 = HEAP32[$676>>2]|0;
     $678 = (_cachedCellSize($b,$677)|0);
     $679 = $678&65535;
     $680 = (2 + ($679))|0;
     HEAP32[$sz1>>2] = $680;
     $681 = HEAP32[$i>>2]|0;
     $682 = (($szNew) + ($681<<2)|0);
     $683 = HEAP32[$682>>2]|0;
     $684 = HEAP32[$sz1>>2]|0;
     $685 = (($683) + ($684))|0;
     $686 = HEAP32[$usableSpace>>2]|0;
     $687 = ($685|0)>($686|0);
     if ($687) {
      break;
     }
     $688 = HEAP32[$sz1>>2]|0;
     $689 = HEAP32[$i>>2]|0;
     $690 = (($szNew) + ($689<<2)|0);
     $691 = HEAP32[$690>>2]|0;
     $692 = (($691) + ($688))|0;
     HEAP32[$690>>2] = $692;
     $693 = HEAP32[$i>>2]|0;
     $694 = (($cntNew) + ($693<<2)|0);
     $695 = HEAP32[$694>>2]|0;
     $696 = (($695) + 1)|0;
     HEAP32[$694>>2] = $696;
     $697 = HEAP32[$leafData>>2]|0;
     $698 = ($697|0)!=(0);
     do {
      if (!($698)) {
       $699 = HEAP32[$i>>2]|0;
       $700 = (($cntNew) + ($699<<2)|0);
       $701 = HEAP32[$700>>2]|0;
       $702 = HEAP32[$b>>2]|0;
       $703 = ($701|0)<($702|0);
       if ($703) {
        $704 = HEAP32[$i>>2]|0;
        $705 = (($cntNew) + ($704<<2)|0);
        $706 = HEAP32[$705>>2]|0;
        $707 = (_cachedCellSize($b,$706)|0);
        $708 = $707&65535;
        $709 = (2 + ($708))|0;
        HEAP32[$sz1>>2] = $709;
        break;
       } else {
        HEAP32[$sz1>>2] = 0;
        break;
       }
      }
     } while(0);
     $710 = HEAP32[$sz1>>2]|0;
     $711 = HEAP32[$i>>2]|0;
     $712 = (($711) + 1)|0;
     $713 = (($szNew) + ($712<<2)|0);
     $714 = HEAP32[$713>>2]|0;
     $715 = (($714) - ($710))|0;
     HEAP32[$713>>2] = $715;
    }
    $716 = HEAP32[$i>>2]|0;
    $717 = (($cntNew) + ($716<<2)|0);
    $718 = HEAP32[$717>>2]|0;
    $719 = HEAP32[$b>>2]|0;
    $720 = ($718|0)>=($719|0);
    $721 = HEAP32[$i>>2]|0;
    if ($720) {
     $722 = (($721) + 1)|0;
     HEAP32[$k>>2] = $722;
    } else {
     $723 = (($cntNew) + ($721<<2)|0);
     $724 = HEAP32[$723>>2]|0;
     $725 = HEAP32[$i>>2]|0;
     $726 = ($725|0)>(0);
     if ($726) {
      $727 = HEAP32[$i>>2]|0;
      $728 = (($727) - 1)|0;
      $729 = (($cntNew) + ($728<<2)|0);
      $730 = HEAP32[$729>>2]|0;
      $732 = $730;
     } else {
      $732 = 0;
     }
     $731 = ($724|0)<=($732|0);
     if ($731) {
      label = 81;
      break;
     }
    }
    $734 = HEAP32[$i>>2]|0;
    $735 = (($734) + 1)|0;
    HEAP32[$i>>2] = $735;
   }
   if ((label|0) == 62) {
    $627 = (_sqlite3CorruptError(62854)|0);
    HEAP32[$rc>>2] = $627;
    break;
   }
   else if ((label|0) == 81) {
    $733 = (_sqlite3CorruptError(62887)|0);
    HEAP32[$rc>>2] = $733;
    break;
   }
   else if ((label|0) == 83) {
    $736 = HEAP32[$k>>2]|0;
    $737 = (($736) - 1)|0;
    HEAP32[$i>>2] = $737;
    while(1) {
     $738 = HEAP32[$i>>2]|0;
     $739 = ($738|0)>(0);
     if (!($739)) {
      break;
     }
     $740 = HEAP32[$i>>2]|0;
     $741 = (($szNew) + ($740<<2)|0);
     $742 = HEAP32[$741>>2]|0;
     HEAP32[$szRight>>2] = $742;
     $743 = HEAP32[$i>>2]|0;
     $744 = (($743) - 1)|0;
     $745 = (($szNew) + ($744<<2)|0);
     $746 = HEAP32[$745>>2]|0;
     HEAP32[$szLeft>>2] = $746;
     $747 = HEAP32[$i>>2]|0;
     $748 = (($747) - 1)|0;
     $749 = (($cntNew) + ($748<<2)|0);
     $750 = HEAP32[$749>>2]|0;
     $751 = (($750) - 1)|0;
     HEAP32[$r>>2] = $751;
     $752 = HEAP32[$r>>2]|0;
     $753 = (($752) + 1)|0;
     $754 = HEAP32[$leafData>>2]|0;
     $755 = (($753) - ($754))|0;
     HEAP32[$d>>2] = $755;
     $756 = HEAP32[$d>>2]|0;
     (_cachedCellSize($b,$756)|0);
     while(1) {
      $757 = HEAP32[$r>>2]|0;
      (_cachedCellSize($b,$757)|0);
      $758 = HEAP32[$szRight>>2]|0;
      $759 = ($758|0)!=(0);
      if ($759) {
       $760 = HEAP32[$5>>2]|0;
       $761 = ($760|0)!=(0);
       if ($761) {
        break;
       }
       $762 = HEAP32[$szRight>>2]|0;
       $763 = HEAP32[$d>>2]|0;
       $764 = ((($b)) + 12|0);
       $765 = HEAP32[$764>>2]|0;
       $766 = (($765) + ($763<<1)|0);
       $767 = HEAP16[$766>>1]|0;
       $768 = $767&65535;
       $769 = (($762) + ($768))|0;
       $770 = (($769) + 2)|0;
       $771 = HEAP32[$szLeft>>2]|0;
       $772 = HEAP32[$r>>2]|0;
       $773 = ((($b)) + 12|0);
       $774 = HEAP32[$773>>2]|0;
       $775 = (($774) + ($772<<1)|0);
       $776 = HEAP16[$775>>1]|0;
       $777 = $776&65535;
       $778 = (($777) + 2)|0;
       $779 = (($771) - ($778))|0;
       $780 = ($770|0)>($779|0);
       if ($780) {
        break;
       }
      }
      $781 = HEAP32[$d>>2]|0;
      $782 = ((($b)) + 12|0);
      $783 = HEAP32[$782>>2]|0;
      $784 = (($783) + ($781<<1)|0);
      $785 = HEAP16[$784>>1]|0;
      $786 = $785&65535;
      $787 = (($786) + 2)|0;
      $788 = HEAP32[$szRight>>2]|0;
      $789 = (($788) + ($787))|0;
      HEAP32[$szRight>>2] = $789;
      $790 = HEAP32[$r>>2]|0;
      $791 = ((($b)) + 12|0);
      $792 = HEAP32[$791>>2]|0;
      $793 = (($792) + ($790<<1)|0);
      $794 = HEAP16[$793>>1]|0;
      $795 = $794&65535;
      $796 = (($795) + 2)|0;
      $797 = HEAP32[$szLeft>>2]|0;
      $798 = (($797) - ($796))|0;
      HEAP32[$szLeft>>2] = $798;
      $799 = HEAP32[$r>>2]|0;
      $800 = HEAP32[$i>>2]|0;
      $801 = (($800) - 1)|0;
      $802 = (($cntNew) + ($801<<2)|0);
      HEAP32[$802>>2] = $799;
      $803 = HEAP32[$r>>2]|0;
      $804 = (($803) + -1)|0;
      HEAP32[$r>>2] = $804;
      $805 = HEAP32[$d>>2]|0;
      $806 = (($805) + -1)|0;
      HEAP32[$d>>2] = $806;
      $807 = HEAP32[$r>>2]|0;
      $808 = ($807|0)>=(0);
      if (!($808)) {
       break;
      }
     }
     $809 = HEAP32[$szRight>>2]|0;
     $810 = HEAP32[$i>>2]|0;
     $811 = (($szNew) + ($810<<2)|0);
     HEAP32[$811>>2] = $809;
     $812 = HEAP32[$szLeft>>2]|0;
     $813 = HEAP32[$i>>2]|0;
     $814 = (($813) - 1)|0;
     $815 = (($szNew) + ($814<<2)|0);
     HEAP32[$815>>2] = $812;
     $816 = HEAP32[$i>>2]|0;
     $817 = (($816) - 1)|0;
     $818 = (($cntNew) + ($817<<2)|0);
     $819 = HEAP32[$818>>2]|0;
     $820 = HEAP32[$i>>2]|0;
     $821 = ($820|0)>(1);
     if ($821) {
      $822 = HEAP32[$i>>2]|0;
      $823 = (($822) - 2)|0;
      $824 = (($cntNew) + ($823<<2)|0);
      $825 = HEAP32[$824>>2]|0;
      $827 = $825;
     } else {
      $827 = 0;
     }
     $826 = ($819|0)<=($827|0);
     if ($826) {
      label = 93;
      break;
     }
     $829 = HEAP32[$i>>2]|0;
     $830 = (($829) + -1)|0;
     HEAP32[$i>>2] = $830;
    }
    if ((label|0) == 93) {
     $828 = (_sqlite3CorruptError(62929)|0);
     HEAP32[$rc>>2] = $828;
     break;
    }
    $831 = HEAP32[$apOld>>2]|0;
    $832 = ((($831)) + 56|0);
    $833 = HEAP32[$832>>2]|0;
    $834 = HEAP8[$833>>0]|0;
    $835 = $834&255;
    HEAP32[$pageFlags>>2] = $835;
    HEAP32[$i>>2] = 0;
    while(1) {
     $836 = HEAP32[$i>>2]|0;
     $837 = HEAP32[$k>>2]|0;
     $838 = ($836|0)<($837|0);
     if (!($838)) {
      break;
     }
     $839 = HEAP32[$i>>2]|0;
     $840 = HEAP32[$nOld>>2]|0;
     $841 = ($839|0)<($840|0);
     if ($841) {
      $842 = HEAP32[$i>>2]|0;
      $843 = (($apOld) + ($842<<2)|0);
      $844 = HEAP32[$843>>2]|0;
      $845 = HEAP32[$i>>2]|0;
      $846 = (($apNew) + ($845<<2)|0);
      HEAP32[$846>>2] = $844;
      HEAP32[$pNew>>2] = $844;
      $847 = HEAP32[$i>>2]|0;
      $848 = (($apOld) + ($847<<2)|0);
      HEAP32[$848>>2] = 0;
      $849 = HEAP32[$pNew>>2]|0;
      $850 = ((($849)) + 72|0);
      $851 = HEAP32[$850>>2]|0;
      $852 = (_sqlite3PagerWrite($851)|0);
      HEAP32[$rc>>2] = $852;
      $853 = HEAP32[$nNew>>2]|0;
      $854 = (($853) + 1)|0;
      HEAP32[$nNew>>2] = $854;
      $855 = HEAP32[$rc>>2]|0;
      $856 = ($855|0)!=(0);
      if ($856) {
       break L32;
      }
     } else {
      $857 = HEAP32[$pBt>>2]|0;
      $858 = HEAP32[$5>>2]|0;
      $859 = ($858|0)!=(0);
      $860 = HEAP32[$pgno>>2]|0;
      $861 = $859 ? 1 : $860;
      $862 = (_allocateBtreePage($857,$pNew,$pgno,$861,0)|0);
      HEAP32[$rc>>2] = $862;
      $863 = HEAP32[$rc>>2]|0;
      $864 = ($863|0)!=(0);
      if ($864) {
       break L32;
      }
      $865 = HEAP32[$pNew>>2]|0;
      $866 = HEAP32[$pageFlags>>2]|0;
      _zeroPage($865,$866);
      $867 = HEAP32[$pNew>>2]|0;
      $868 = HEAP32[$i>>2]|0;
      $869 = (($apNew) + ($868<<2)|0);
      HEAP32[$869>>2] = $867;
      $870 = HEAP32[$nNew>>2]|0;
      $871 = (($870) + 1)|0;
      HEAP32[$nNew>>2] = $871;
      $872 = HEAP32[$b>>2]|0;
      $873 = HEAP32[$i>>2]|0;
      $874 = (($cntOld) + ($873<<2)|0);
      HEAP32[$874>>2] = $872;
      $875 = HEAP32[$pBt>>2]|0;
      $876 = ((($875)) + 17|0);
      $877 = HEAP8[$876>>0]|0;
      $878 = ($877<<24>>24)!=(0);
      if ($878) {
       $879 = HEAP32[$pBt>>2]|0;
       $880 = HEAP32[$pNew>>2]|0;
       $881 = ((($880)) + 84|0);
       $882 = HEAP32[$881>>2]|0;
       $883 = HEAP32[$1>>2]|0;
       $884 = ((($883)) + 84|0);
       $885 = HEAP32[$884>>2]|0;
       _ptrmapPut($879,$882,5,$885,$rc);
       $886 = HEAP32[$rc>>2]|0;
       $887 = ($886|0)!=(0);
       if ($887) {
        break L32;
       }
      }
     }
     $888 = HEAP32[$i>>2]|0;
     $889 = (($888) + 1)|0;
     HEAP32[$i>>2] = $889;
    }
    HEAP32[$i>>2] = 0;
    L142: while(1) {
     $890 = HEAP32[$i>>2]|0;
     $891 = HEAP32[$nNew>>2]|0;
     $892 = ($890|0)<($891|0);
     if (!($892)) {
      break;
     }
     $893 = HEAP32[$i>>2]|0;
     $894 = (($apNew) + ($893<<2)|0);
     $895 = HEAP32[$894>>2]|0;
     $896 = ((($895)) + 84|0);
     $897 = HEAP32[$896>>2]|0;
     $898 = HEAP32[$i>>2]|0;
     $899 = (($aPgno) + ($898<<2)|0);
     HEAP32[$899>>2] = $897;
     $900 = HEAP32[$i>>2]|0;
     $901 = (($aPgOrder) + ($900<<2)|0);
     HEAP32[$901>>2] = $897;
     $902 = HEAP32[$i>>2]|0;
     $903 = (($apNew) + ($902<<2)|0);
     $904 = HEAP32[$903>>2]|0;
     $905 = ((($904)) + 72|0);
     $906 = HEAP32[$905>>2]|0;
     $907 = ((($906)) + 24|0);
     $908 = HEAP16[$907>>1]|0;
     $909 = HEAP32[$i>>2]|0;
     $910 = (($aPgFlags) + ($909<<1)|0);
     HEAP16[$910>>1] = $908;
     HEAP32[$j>>2] = 0;
     while(1) {
      $911 = HEAP32[$j>>2]|0;
      $912 = HEAP32[$i>>2]|0;
      $913 = ($911|0)<($912|0);
      if (!($913)) {
       break;
      }
      $914 = HEAP32[$j>>2]|0;
      $915 = (($aPgno) + ($914<<2)|0);
      $916 = HEAP32[$915>>2]|0;
      $917 = HEAP32[$i>>2]|0;
      $918 = (($aPgno) + ($917<<2)|0);
      $919 = HEAP32[$918>>2]|0;
      $920 = ($916|0)==($919|0);
      if ($920) {
       label = 108;
       break L142;
      }
      $922 = HEAP32[$j>>2]|0;
      $923 = (($922) + 1)|0;
      HEAP32[$j>>2] = $923;
     }
     $924 = HEAP32[$i>>2]|0;
     $925 = (($924) + 1)|0;
     HEAP32[$i>>2] = $925;
    }
    if ((label|0) == 108) {
     $921 = (_sqlite3CorruptError(63003)|0);
     HEAP32[$rc>>2] = $921;
     break;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $926 = HEAP32[$i>>2]|0;
     $927 = HEAP32[$nNew>>2]|0;
     $928 = ($926|0)<($927|0);
     if (!($928)) {
      break;
     }
     HEAP32[$iBest>>2] = 0;
     HEAP32[$j>>2] = 1;
     while(1) {
      $929 = HEAP32[$j>>2]|0;
      $930 = HEAP32[$nNew>>2]|0;
      $931 = ($929|0)<($930|0);
      if (!($931)) {
       break;
      }
      $932 = HEAP32[$j>>2]|0;
      $933 = (($aPgOrder) + ($932<<2)|0);
      $934 = HEAP32[$933>>2]|0;
      $935 = HEAP32[$iBest>>2]|0;
      $936 = (($aPgOrder) + ($935<<2)|0);
      $937 = HEAP32[$936>>2]|0;
      $938 = ($934>>>0)<($937>>>0);
      if ($938) {
       $939 = HEAP32[$j>>2]|0;
       HEAP32[$iBest>>2] = $939;
      }
      $940 = HEAP32[$j>>2]|0;
      $941 = (($940) + 1)|0;
      HEAP32[$j>>2] = $941;
     }
     $942 = HEAP32[$iBest>>2]|0;
     $943 = (($aPgOrder) + ($942<<2)|0);
     $944 = HEAP32[$943>>2]|0;
     HEAP32[$pgno>>2] = $944;
     $945 = HEAP32[$iBest>>2]|0;
     $946 = (($aPgOrder) + ($945<<2)|0);
     HEAP32[$946>>2] = -1;
     $947 = HEAP32[$iBest>>2]|0;
     $948 = HEAP32[$i>>2]|0;
     $949 = ($947|0)!=($948|0);
     if ($949) {
      $950 = HEAP32[$iBest>>2]|0;
      $951 = HEAP32[$i>>2]|0;
      $952 = ($950|0)>($951|0);
      if ($952) {
       $953 = HEAP32[$iBest>>2]|0;
       $954 = (($apNew) + ($953<<2)|0);
       $955 = HEAP32[$954>>2]|0;
       $956 = ((($955)) + 72|0);
       $957 = HEAP32[$956>>2]|0;
       $958 = HEAP32[$pBt>>2]|0;
       $959 = ((($958)) + 44|0);
       $960 = HEAP32[$959>>2]|0;
       $961 = HEAP32[$iBest>>2]|0;
       $962 = (($960) + ($961))|0;
       $963 = (($962) + 1)|0;
       _sqlite3PagerRekey($957,$963,0);
      }
      $964 = HEAP32[$i>>2]|0;
      $965 = (($apNew) + ($964<<2)|0);
      $966 = HEAP32[$965>>2]|0;
      $967 = ((($966)) + 72|0);
      $968 = HEAP32[$967>>2]|0;
      $969 = HEAP32[$pgno>>2]|0;
      $970 = HEAP32[$iBest>>2]|0;
      $971 = (($aPgFlags) + ($970<<1)|0);
      $972 = HEAP16[$971>>1]|0;
      _sqlite3PagerRekey($968,$969,$972);
      $973 = HEAP32[$pgno>>2]|0;
      $974 = HEAP32[$i>>2]|0;
      $975 = (($apNew) + ($974<<2)|0);
      $976 = HEAP32[$975>>2]|0;
      $977 = ((($976)) + 84|0);
      HEAP32[$977>>2] = $973;
     }
     $978 = HEAP32[$i>>2]|0;
     $979 = (($978) + 1)|0;
     HEAP32[$i>>2] = $979;
    }
    $980 = HEAP32[$pRight>>2]|0;
    $981 = HEAP32[$nNew>>2]|0;
    $982 = (($981) - 1)|0;
    $983 = (($apNew) + ($982<<2)|0);
    $984 = HEAP32[$983>>2]|0;
    $985 = ((($984)) + 84|0);
    $986 = HEAP32[$985>>2]|0;
    _sqlite3Put4byte($980,$986);
    $987 = HEAP32[$pageFlags>>2]|0;
    $988 = $987 & 8;
    $989 = ($988|0)==(0);
    if ($989) {
     $990 = HEAP32[$nOld>>2]|0;
     $991 = HEAP32[$nNew>>2]|0;
     $992 = ($990|0)!=($991|0);
     if ($992) {
      $993 = HEAP32[$nOld>>2]|0;
      $994 = (($993) - 1)|0;
      $995 = HEAP32[$nNew>>2]|0;
      $996 = HEAP32[$nOld>>2]|0;
      $997 = ($995|0)>($996|0);
      $998 = $997 ? $apNew : $apOld;
      $999 = (($998) + ($994<<2)|0);
      $1000 = HEAP32[$999>>2]|0;
      HEAP32[$pOld2>>2] = $1000;
      $1001 = HEAP32[$nNew>>2]|0;
      $1002 = (($1001) - 1)|0;
      $1003 = (($apNew) + ($1002<<2)|0);
      $1004 = HEAP32[$1003>>2]|0;
      $1005 = ((($1004)) + 56|0);
      $1006 = HEAP32[$1005>>2]|0;
      $1007 = ((($1006)) + 8|0);
      $1008 = HEAP32[$pOld2>>2]|0;
      $1009 = ((($1008)) + 56|0);
      $1010 = HEAP32[$1009>>2]|0;
      $1011 = ((($1010)) + 8|0);
      ;HEAP8[$1007>>0]=HEAP8[$1011>>0]|0;HEAP8[$1007+1>>0]=HEAP8[$1011+1>>0]|0;HEAP8[$1007+2>>0]=HEAP8[$1011+2>>0]|0;HEAP8[$1007+3>>0]=HEAP8[$1011+3>>0]|0;
     }
    }
    $1012 = HEAP32[$pBt>>2]|0;
    $1013 = ((($1012)) + 17|0);
    $1014 = HEAP8[$1013>>0]|0;
    $1015 = ($1014<<24>>24)!=(0);
    L174: do {
     if ($1015) {
      $1016 = HEAP32[$apNew>>2]|0;
      HEAP32[$pNew3>>2] = $1016;
      $1017 = HEAP32[$pNew3>>2]|0;
      $1018 = ((($1017)) + 56|0);
      $1019 = HEAP32[$1018>>2]|0;
      HEAP32[$aOld>>2] = $1019;
      $1020 = HEAP32[$pNew3>>2]|0;
      $1021 = ((($1020)) + 18|0);
      $1022 = HEAP16[$1021>>1]|0;
      $1023 = $1022&65535;
      $1024 = HEAP32[$pNew3>>2]|0;
      $1025 = ((($1024)) + 1|0);
      $1026 = HEAP8[$1025>>0]|0;
      $1027 = $1026&255;
      $1028 = (($1023) + ($1027))|0;
      HEAP32[$cntOldNext>>2] = $1028;
      $1029 = HEAP32[$pBt>>2]|0;
      $1030 = ((($1029)) + 36|0);
      $1031 = HEAP32[$1030>>2]|0;
      HEAP32[$usableSize>>2] = $1031;
      HEAP32[$iNew>>2] = 0;
      HEAP32[$iOld>>2] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $1032 = HEAP32[$i>>2]|0;
       $1033 = HEAP32[$b>>2]|0;
       $1034 = ($1032|0)<($1033|0);
       if (!($1034)) {
        break L174;
       }
       $1035 = HEAP32[$i>>2]|0;
       $1036 = ((($b)) + 8|0);
       $1037 = HEAP32[$1036>>2]|0;
       $1038 = (($1037) + ($1035<<2)|0);
       $1039 = HEAP32[$1038>>2]|0;
       HEAP32[$pCell>>2] = $1039;
       $1040 = HEAP32[$i>>2]|0;
       $1041 = HEAP32[$cntOldNext>>2]|0;
       $1042 = ($1040|0)==($1041|0);
       if ($1042) {
        $1043 = HEAP32[$iOld>>2]|0;
        $1044 = (($1043) + 1)|0;
        HEAP32[$iOld>>2] = $1044;
        $1045 = HEAP32[$nNew>>2]|0;
        $1046 = ($1044|0)<($1045|0);
        $1047 = HEAP32[$iOld>>2]|0;
        if ($1046) {
         $1048 = (($apNew) + ($1047<<2)|0);
         $1049 = HEAP32[$1048>>2]|0;
         $1052 = $1049;
        } else {
         $1050 = (($apOld) + ($1047<<2)|0);
         $1051 = HEAP32[$1050>>2]|0;
         $1052 = $1051;
        }
        HEAP32[$pOld4>>2] = $1052;
        $1053 = HEAP32[$pOld4>>2]|0;
        $1054 = ((($1053)) + 18|0);
        $1055 = HEAP16[$1054>>1]|0;
        $1056 = $1055&65535;
        $1057 = HEAP32[$pOld4>>2]|0;
        $1058 = ((($1057)) + 1|0);
        $1059 = HEAP8[$1058>>0]|0;
        $1060 = $1059&255;
        $1061 = (($1056) + ($1060))|0;
        $1062 = HEAP32[$leafData>>2]|0;
        $1063 = ($1062|0)!=(0);
        $1064 = $1063 ^ 1;
        $1065 = $1064&1;
        $1066 = (($1061) + ($1065))|0;
        $1067 = HEAP32[$cntOldNext>>2]|0;
        $1068 = (($1067) + ($1066))|0;
        HEAP32[$cntOldNext>>2] = $1068;
        $1069 = HEAP32[$pOld4>>2]|0;
        $1070 = ((($1069)) + 56|0);
        $1071 = HEAP32[$1070>>2]|0;
        HEAP32[$aOld>>2] = $1071;
       }
       $1072 = HEAP32[$i>>2]|0;
       $1073 = HEAP32[$iNew>>2]|0;
       $1074 = (($cntNew) + ($1073<<2)|0);
       $1075 = HEAP32[$1074>>2]|0;
       $1076 = ($1072|0)==($1075|0);
       if ($1076) {
        $1077 = HEAP32[$iNew>>2]|0;
        $1078 = (($1077) + 1)|0;
        HEAP32[$iNew>>2] = $1078;
        $1079 = (($apNew) + ($1078<<2)|0);
        $1080 = HEAP32[$1079>>2]|0;
        HEAP32[$pNew3>>2] = $1080;
        $1081 = HEAP32[$leafData>>2]|0;
        $1082 = ($1081|0)!=(0);
        if ($1082) {
         label = 136;
        }
       } else {
        label = 136;
       }
       do {
        if ((label|0) == 136) {
         label = 0;
         $1083 = HEAP32[$iOld>>2]|0;
         $1084 = HEAP32[$nNew>>2]|0;
         $1085 = ($1083|0)>=($1084|0);
         if (!($1085)) {
          $1086 = HEAP32[$pNew3>>2]|0;
          $1087 = ((($1086)) + 84|0);
          $1088 = HEAP32[$1087>>2]|0;
          $1089 = HEAP32[$iOld>>2]|0;
          $1090 = (($aPgno) + ($1089<<2)|0);
          $1091 = HEAP32[$1090>>2]|0;
          $1092 = ($1088|0)!=($1091|0);
          if (!($1092)) {
           $1093 = HEAP32[$pCell>>2]|0;
           $1094 = HEAP32[$aOld>>2]|0;
           $1095 = ($1093>>>0)>=($1094>>>0);
           if ($1095) {
            $1096 = HEAP32[$pCell>>2]|0;
            $1097 = HEAP32[$usableSize>>2]|0;
            $1098 = HEAP32[$aOld>>2]|0;
            $1099 = (($1098) + ($1097)|0);
            $1100 = ($1096>>>0)<($1099>>>0);
            if ($1100) {
             break;
            }
           }
          }
         }
         $1101 = HEAP16[$leafCorrection>>1]|0;
         $1102 = ($1101<<16>>16)!=(0);
         if (!($1102)) {
          $1103 = HEAP32[$pBt>>2]|0;
          $1104 = HEAP32[$pCell>>2]|0;
          $1105 = (_sqlite3Get4byte($1104)|0);
          $1106 = HEAP32[$pNew3>>2]|0;
          $1107 = ((($1106)) + 84|0);
          $1108 = HEAP32[$1107>>2]|0;
          _ptrmapPut($1103,$1105,5,$1108,$rc);
         }
         $1109 = HEAP32[$i>>2]|0;
         $1110 = (_cachedCellSize($b,$1109)|0);
         $1111 = $1110&65535;
         $1112 = HEAP32[$pNew3>>2]|0;
         $1113 = ((($1112)) + 12|0);
         $1114 = HEAP16[$1113>>1]|0;
         $1115 = $1114&65535;
         $1116 = ($1111|0)>($1115|0);
         if ($1116) {
          $1117 = HEAP32[$pNew3>>2]|0;
          $1118 = HEAP32[$pCell>>2]|0;
          _ptrmapPutOvflPtr($1117,$1118,$rc);
         }
         $1119 = HEAP32[$rc>>2]|0;
         $1120 = ($1119|0)!=(0);
         if ($1120) {
          break L32;
         }
        }
       } while(0);
       $1121 = HEAP32[$i>>2]|0;
       $1122 = (($1121) + 1)|0;
       HEAP32[$i>>2] = $1122;
      }
     }
    } while(0);
    HEAP32[$i>>2] = 0;
    while(1) {
     $1123 = HEAP32[$i>>2]|0;
     $1124 = HEAP32[$nNew>>2]|0;
     $1125 = (($1124) - 1)|0;
     $1126 = ($1123|0)<($1125|0);
     if (!($1126)) {
      break;
     }
     $1127 = HEAP32[$i>>2]|0;
     $1128 = (($apNew) + ($1127<<2)|0);
     $1129 = HEAP32[$1128>>2]|0;
     HEAP32[$pNew8>>2] = $1129;
     $1130 = HEAP32[$i>>2]|0;
     $1131 = (($cntNew) + ($1130<<2)|0);
     $1132 = HEAP32[$1131>>2]|0;
     HEAP32[$j>>2] = $1132;
     $1133 = HEAP32[$j>>2]|0;
     $1134 = ((($b)) + 8|0);
     $1135 = HEAP32[$1134>>2]|0;
     $1136 = (($1135) + ($1133<<2)|0);
     $1137 = HEAP32[$1136>>2]|0;
     HEAP32[$pCell5>>2] = $1137;
     $1138 = HEAP32[$j>>2]|0;
     $1139 = ((($b)) + 12|0);
     $1140 = HEAP32[$1139>>2]|0;
     $1141 = (($1140) + ($1138<<1)|0);
     $1142 = HEAP16[$1141>>1]|0;
     $1143 = $1142&65535;
     $1144 = HEAP16[$leafCorrection>>1]|0;
     $1145 = $1144&65535;
     $1146 = (($1143) + ($1145))|0;
     HEAP32[$sz7>>2] = $1146;
     $1147 = HEAP32[$iOvflSpace>>2]|0;
     $1148 = HEAP32[$3>>2]|0;
     $1149 = (($1148) + ($1147)|0);
     HEAP32[$pTemp6>>2] = $1149;
     $1150 = HEAP32[$pNew8>>2]|0;
     $1151 = ((($1150)) + 4|0);
     $1152 = HEAP8[$1151>>0]|0;
     $1153 = ($1152<<24>>24)!=(0);
     do {
      if ($1153) {
       $1159 = HEAP32[$leafData>>2]|0;
       $1160 = ($1159|0)!=(0);
       if ($1160) {
        $1161 = HEAP32[$j>>2]|0;
        $1162 = (($1161) + -1)|0;
        HEAP32[$j>>2] = $1162;
        $1163 = HEAP32[$pNew8>>2]|0;
        $1164 = ((($1163)) + 80|0);
        $1165 = HEAP32[$1164>>2]|0;
        $1166 = HEAP32[$pNew8>>2]|0;
        $1167 = HEAP32[$j>>2]|0;
        $1168 = ((($b)) + 8|0);
        $1169 = HEAP32[$1168>>2]|0;
        $1170 = (($1169) + ($1167<<2)|0);
        $1171 = HEAP32[$1170>>2]|0;
        FUNCTION_TABLE_viii[$1165 & 255]($1166,$1171,$info);
        $1172 = HEAP32[$pTemp6>>2]|0;
        HEAP32[$pCell5>>2] = $1172;
        $1173 = HEAP32[$pCell5>>2]|0;
        $1174 = ((($1173)) + 4|0);
        $1175 = $info;
        $1176 = $1175;
        $1177 = HEAP32[$1176>>2]|0;
        $1178 = (($1175) + 4)|0;
        $1179 = $1178;
        $1180 = HEAP32[$1179>>2]|0;
        $1181 = (_sqlite3PutVarint($1174,$1177,$1180)|0);
        $1182 = (4 + ($1181))|0;
        HEAP32[$sz7>>2] = $1182;
        HEAP32[$pTemp6>>2] = 0;
        break;
       }
       $1183 = HEAP32[$pCell5>>2]|0;
       $1184 = ((($1183)) + -4|0);
       HEAP32[$pCell5>>2] = $1184;
       $1185 = HEAP32[$j>>2]|0;
       $1186 = ((($b)) + 12|0);
       $1187 = HEAP32[$1186>>2]|0;
       $1188 = (($1187) + ($1185<<1)|0);
       $1189 = HEAP16[$1188>>1]|0;
       $1190 = $1189&65535;
       $1191 = ($1190|0)==(4);
       if ($1191) {
        $1192 = HEAP32[$1>>2]|0;
        $1193 = ((($1192)) + 76|0);
        $1194 = HEAP32[$1193>>2]|0;
        $1195 = HEAP32[$1>>2]|0;
        $1196 = HEAP32[$pCell5>>2]|0;
        $1197 = (FUNCTION_TABLE_iii[$1194 & 255]($1195,$1196)|0);
        $1198 = $1197&65535;
        HEAP32[$sz7>>2] = $1198;
       }
      } else {
       $1154 = HEAP32[$pNew8>>2]|0;
       $1155 = ((($1154)) + 56|0);
       $1156 = HEAP32[$1155>>2]|0;
       $1157 = ((($1156)) + 8|0);
       $1158 = HEAP32[$pCell5>>2]|0;
       ;HEAP8[$1157>>0]=HEAP8[$1158>>0]|0;HEAP8[$1157+1>>0]=HEAP8[$1158+1>>0]|0;HEAP8[$1157+2>>0]=HEAP8[$1158+2>>0]|0;HEAP8[$1157+3>>0]=HEAP8[$1158+3>>0]|0;
      }
     } while(0);
     $1199 = HEAP32[$sz7>>2]|0;
     $1200 = HEAP32[$iOvflSpace>>2]|0;
     $1201 = (($1200) + ($1199))|0;
     HEAP32[$iOvflSpace>>2] = $1201;
     $1202 = HEAP32[$1>>2]|0;
     $1203 = HEAP32[$nxDiv>>2]|0;
     $1204 = HEAP32[$i>>2]|0;
     $1205 = (($1203) + ($1204))|0;
     $1206 = HEAP32[$pCell5>>2]|0;
     $1207 = HEAP32[$sz7>>2]|0;
     $1208 = HEAP32[$pTemp6>>2]|0;
     $1209 = HEAP32[$pNew8>>2]|0;
     $1210 = ((($1209)) + 84|0);
     $1211 = HEAP32[$1210>>2]|0;
     _insertCell($1202,$1205,$1206,$1207,$1208,$1211,$rc);
     $1212 = HEAP32[$rc>>2]|0;
     $1213 = ($1212|0)!=(0);
     if ($1213) {
      break L32;
     }
     $1214 = HEAP32[$i>>2]|0;
     $1215 = (($1214) + 1)|0;
     HEAP32[$i>>2] = $1215;
    }
    $1216 = HEAP32[$nNew>>2]|0;
    $1217 = (1 - ($1216))|0;
    HEAP32[$i>>2] = $1217;
    while(1) {
     $1218 = HEAP32[$i>>2]|0;
     $1219 = HEAP32[$nNew>>2]|0;
     $1220 = ($1218|0)<($1219|0);
     if (!($1220)) {
      break;
     }
     $1221 = HEAP32[$i>>2]|0;
     $1222 = ($1221|0)<(0);
     $1223 = HEAP32[$i>>2]|0;
     $1224 = (0 - ($1223))|0;
     $1225 = $1222 ? $1224 : $1223;
     HEAP32[$iPg>>2] = $1225;
     $1226 = HEAP32[$iPg>>2]|0;
     $1227 = (($abDone) + ($1226)|0);
     $1228 = HEAP8[$1227>>0]|0;
     $1229 = ($1228<<24>>24)!=(0);
     do {
      if (!($1229)) {
       $1230 = HEAP32[$i>>2]|0;
       $1231 = ($1230|0)>=(0);
       if (!($1231)) {
        $1232 = HEAP32[$iPg>>2]|0;
        $1233 = (($1232) - 1)|0;
        $1234 = (($cntOld) + ($1233<<2)|0);
        $1235 = HEAP32[$1234>>2]|0;
        $1236 = HEAP32[$iPg>>2]|0;
        $1237 = (($1236) - 1)|0;
        $1238 = (($cntNew) + ($1237<<2)|0);
        $1239 = HEAP32[$1238>>2]|0;
        $1240 = ($1235|0)>=($1239|0);
        if (!($1240)) {
         break;
        }
       }
       $1241 = HEAP32[$iPg>>2]|0;
       $1242 = ($1241|0)==(0);
       if ($1242) {
        HEAP32[$iOld10>>2] = 0;
        HEAP32[$iNew9>>2] = 0;
        $1243 = HEAP32[$cntNew>>2]|0;
        HEAP32[$nNewCell>>2] = $1243;
       } else {
        $1244 = HEAP32[$iPg>>2]|0;
        $1245 = HEAP32[$nOld>>2]|0;
        $1246 = ($1244|0)<($1245|0);
        if ($1246) {
         $1247 = HEAP32[$iPg>>2]|0;
         $1248 = (($1247) - 1)|0;
         $1249 = (($cntOld) + ($1248<<2)|0);
         $1250 = HEAP32[$1249>>2]|0;
         $1251 = HEAP32[$leafData>>2]|0;
         $1252 = ($1251|0)!=(0);
         $1253 = $1252 ^ 1;
         $1254 = $1253&1;
         $1255 = (($1250) + ($1254))|0;
         $1257 = $1255;
        } else {
         $1256 = HEAP32[$b>>2]|0;
         $1257 = $1256;
        }
        HEAP32[$iOld10>>2] = $1257;
        $1258 = HEAP32[$iPg>>2]|0;
        $1259 = (($1258) - 1)|0;
        $1260 = (($cntNew) + ($1259<<2)|0);
        $1261 = HEAP32[$1260>>2]|0;
        $1262 = HEAP32[$leafData>>2]|0;
        $1263 = ($1262|0)!=(0);
        $1264 = $1263 ^ 1;
        $1265 = $1264&1;
        $1266 = (($1261) + ($1265))|0;
        HEAP32[$iNew9>>2] = $1266;
        $1267 = HEAP32[$iPg>>2]|0;
        $1268 = (($cntNew) + ($1267<<2)|0);
        $1269 = HEAP32[$1268>>2]|0;
        $1270 = HEAP32[$iNew9>>2]|0;
        $1271 = (($1269) - ($1270))|0;
        HEAP32[$nNewCell>>2] = $1271;
       }
       $1272 = HEAP32[$iPg>>2]|0;
       $1273 = (($apNew) + ($1272<<2)|0);
       $1274 = HEAP32[$1273>>2]|0;
       $1275 = HEAP32[$iOld10>>2]|0;
       $1276 = HEAP32[$iNew9>>2]|0;
       $1277 = HEAP32[$nNewCell>>2]|0;
       $1278 = (_editPage($1274,$1275,$1276,$1277,$b)|0);
       HEAP32[$rc>>2] = $1278;
       $1279 = HEAP32[$rc>>2]|0;
       $1280 = ($1279|0)!=(0);
       if ($1280) {
        break L32;
       }
       $1281 = HEAP32[$iPg>>2]|0;
       $1282 = (($abDone) + ($1281)|0);
       $1283 = HEAP8[$1282>>0]|0;
       $1284 = (($1283) + 1)<<24>>24;
       HEAP8[$1282>>0] = $1284;
       $1285 = HEAP32[$usableSpace>>2]|0;
       $1286 = HEAP32[$iPg>>2]|0;
       $1287 = (($szNew) + ($1286<<2)|0);
       $1288 = HEAP32[$1287>>2]|0;
       $1289 = (($1285) - ($1288))|0;
       $1290 = $1289&65535;
       $1291 = HEAP32[$iPg>>2]|0;
       $1292 = (($apNew) + ($1291<<2)|0);
       $1293 = HEAP32[$1292>>2]|0;
       $1294 = ((($1293)) + 16|0);
       HEAP16[$1294>>1] = $1290;
      }
     } while(0);
     $1295 = HEAP32[$i>>2]|0;
     $1296 = (($1295) + 1)|0;
     HEAP32[$i>>2] = $1296;
    }
    $1297 = HEAP32[$4>>2]|0;
    $1298 = ($1297|0)!=(0);
    if ($1298) {
     $1299 = HEAP32[$1>>2]|0;
     $1300 = ((($1299)) + 18|0);
     $1301 = HEAP16[$1300>>1]|0;
     $1302 = $1301&65535;
     $1303 = ($1302|0)==(0);
     if ($1303) {
      $1304 = HEAP32[$1>>2]|0;
      $1305 = ((($1304)) + 5|0);
      $1306 = HEAP8[$1305>>0]|0;
      $1307 = $1306&255;
      $1308 = HEAP32[$apNew>>2]|0;
      $1309 = ((($1308)) + 16|0);
      $1310 = HEAP16[$1309>>1]|0;
      $1311 = $1310&65535;
      $1312 = ($1307|0)<=($1311|0);
      if ($1312) {
       $1313 = HEAP32[$apNew>>2]|0;
       $1314 = (_defragmentPage($1313)|0);
       HEAP32[$rc>>2] = $1314;
       $1315 = HEAP32[$apNew>>2]|0;
       $1316 = HEAP32[$1>>2]|0;
       _copyNodeContent($1315,$1316,$rc);
       $1317 = HEAP32[$apNew>>2]|0;
       _freePage($1317,$rc);
      } else {
       label = 174;
      }
     } else {
      label = 174;
     }
    } else {
     label = 174;
    }
    L239: do {
     if ((label|0) == 174) {
      $1318 = HEAP32[$pBt>>2]|0;
      $1319 = ((($1318)) + 17|0);
      $1320 = HEAP8[$1319>>0]|0;
      $1321 = $1320&255;
      $1322 = ($1321|0)==(0);
      $1323 = HEAP16[$leafCorrection>>1]|0;
      $1324 = ($1323<<16>>16)!=(0);
      $or$cond3 = $1322 | $1324;
      if (!($or$cond3)) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $1325 = HEAP32[$i>>2]|0;
        $1326 = HEAP32[$nNew>>2]|0;
        $1327 = ($1325|0)<($1326|0);
        if (!($1327)) {
         break L239;
        }
        $1328 = HEAP32[$i>>2]|0;
        $1329 = (($apNew) + ($1328<<2)|0);
        $1330 = HEAP32[$1329>>2]|0;
        $1331 = ((($1330)) + 56|0);
        $1332 = HEAP32[$1331>>2]|0;
        $1333 = ((($1332)) + 8|0);
        $1334 = (_sqlite3Get4byte($1333)|0);
        HEAP32[$key>>2] = $1334;
        $1335 = HEAP32[$pBt>>2]|0;
        $1336 = HEAP32[$key>>2]|0;
        $1337 = HEAP32[$i>>2]|0;
        $1338 = (($apNew) + ($1337<<2)|0);
        $1339 = HEAP32[$1338>>2]|0;
        $1340 = ((($1339)) + 84|0);
        $1341 = HEAP32[$1340>>2]|0;
        _ptrmapPut($1335,$1336,5,$1341,$rc);
        $1342 = HEAP32[$i>>2]|0;
        $1343 = (($1342) + 1)|0;
        HEAP32[$i>>2] = $1343;
       }
      }
     }
    } while(0);
    $1344 = HEAP32[$nNew>>2]|0;
    HEAP32[$i>>2] = $1344;
    while(1) {
     $1345 = HEAP32[$i>>2]|0;
     $1346 = HEAP32[$nOld>>2]|0;
     $1347 = ($1345|0)<($1346|0);
     if (!($1347)) {
      break L32;
     }
     $1348 = HEAP32[$i>>2]|0;
     $1349 = (($apOld) + ($1348<<2)|0);
     $1350 = HEAP32[$1349>>2]|0;
     _freePage($1350,$rc);
     $1351 = HEAP32[$i>>2]|0;
     $1352 = (($1351) + 1)|0;
     HEAP32[$i>>2] = $1352;
    }
   }
  }
 } while(0);
 $1353 = ((($b)) + 8|0);
 $1354 = HEAP32[$1353>>2]|0;
 _sqlite3ScratchFree($1354);
 HEAP32[$i>>2] = 0;
 while(1) {
  $1355 = HEAP32[$i>>2]|0;
  $1356 = HEAP32[$nOld>>2]|0;
  $1357 = ($1355|0)<($1356|0);
  if (!($1357)) {
   break;
  }
  $1358 = HEAP32[$i>>2]|0;
  $1359 = (($apOld) + ($1358<<2)|0);
  $1360 = HEAP32[$1359>>2]|0;
  _releasePage($1360);
  $1361 = HEAP32[$i>>2]|0;
  $1362 = (($1361) + 1)|0;
  HEAP32[$i>>2] = $1362;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $1363 = HEAP32[$i>>2]|0;
  $1364 = HEAP32[$nNew>>2]|0;
  $1365 = ($1363|0)<($1364|0);
  if (!($1365)) {
   break;
  }
  $1366 = HEAP32[$i>>2]|0;
  $1367 = (($apNew) + ($1366<<2)|0);
  $1368 = HEAP32[$1367>>2]|0;
  _releasePage($1368);
  $1369 = HEAP32[$i>>2]|0;
  $1370 = (($1369) + 1)|0;
  HEAP32[$i>>2] = $1370;
 }
 $1371 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $1371;
 $1372 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($1372|0);
}
function _copyNodeContent($pFrom,$pTo,$pRC) {
 $pFrom = $pFrom|0;
 $pTo = $pTo|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $aFrom = 0, $aTo = 0, $iData = 0, $iFromHdr = 0, $iToHdr = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pBt = sp + 24|0;
 $aFrom = sp + 20|0;
 $aTo = sp + 16|0;
 $iFromHdr = sp + 12|0;
 $iToHdr = sp + 8|0;
 $rc = sp + 4|0;
 $iData = sp;
 HEAP32[$0>>2] = $pFrom;
 HEAP32[$1>>2] = $pTo;
 HEAP32[$2>>2] = $pRC;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aFrom>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 56|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$aTo>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 5|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 HEAP32[$iFromHdr>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 84|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)==(1);
 $23 = $22 ? 100 : 0;
 HEAP32[$iToHdr>>2] = $23;
 $24 = HEAP32[$iFromHdr>>2]|0;
 $25 = (($24) + 5)|0;
 $26 = HEAP32[$aFrom>>2]|0;
 $27 = (($26) + ($25)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = $29 << 8;
 $31 = HEAP32[$iFromHdr>>2]|0;
 $32 = (($31) + 5)|0;
 $33 = HEAP32[$aFrom>>2]|0;
 $34 = (($33) + ($32)|0);
 $35 = ((($34)) + 1|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $30 | $37;
 HEAP32[$iData>>2] = $38;
 $39 = HEAP32[$iData>>2]|0;
 $40 = HEAP32[$aTo>>2]|0;
 $41 = (($40) + ($39)|0);
 $42 = HEAP32[$iData>>2]|0;
 $43 = HEAP32[$aFrom>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP32[$pBt>>2]|0;
 $46 = ((($45)) + 36|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$iData>>2]|0;
 $49 = (($47) - ($48))|0;
 _memcpy(($41|0),($44|0),($49|0))|0;
 $50 = HEAP32[$iToHdr>>2]|0;
 $51 = HEAP32[$aTo>>2]|0;
 $52 = (($51) + ($50)|0);
 $53 = HEAP32[$iFromHdr>>2]|0;
 $54 = HEAP32[$aFrom>>2]|0;
 $55 = (($54) + ($53)|0);
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 14|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 18|0);
 $62 = HEAP16[$61>>1]|0;
 $63 = $62&65535;
 $64 = $63<<1;
 $65 = (($59) + ($64))|0;
 _memcpy(($52|0),($55|0),($65|0))|0;
 $66 = HEAP32[$1>>2]|0;
 HEAP8[$66>>0] = 0;
 $67 = HEAP32[$1>>2]|0;
 $68 = (_btreeInitPage($67)|0);
 HEAP32[$rc>>2] = $68;
 $69 = HEAP32[$rc>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$rc>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  HEAP32[$72>>2] = $71;
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$pBt>>2]|0;
 $74 = ((($73)) + 17|0);
 $75 = HEAP8[$74>>0]|0;
 $76 = ($75<<24>>24)!=(0);
 if (!($76)) {
  STACKTOP = sp;return;
 }
 $77 = HEAP32[$1>>2]|0;
 $78 = (_setChildPtrmaps($77)|0);
 $79 = HEAP32[$2>>2]|0;
 HEAP32[$79>>2] = $78;
 STACKTOP = sp;return;
}
function _rebuildPage($pPg,$nCell,$apCell,$szCell) {
 $pPg = $pPg|0;
 $nCell = $nCell|0;
 $apCell = $apCell|0;
 $szCell = $szCell|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0, $hdr = 0, $i = 0, $pCell = 0, $pCellptr = 0, $pData = 0, $pEnd = 0;
 var $pTmp = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $hdr = sp + 32|0;
 $aData = sp + 28|0;
 $usableSize = sp + 24|0;
 $pEnd = sp + 20|0;
 $i = sp + 16|0;
 $pCellptr = sp + 12|0;
 $pTmp = sp + 8|0;
 $pData = sp + 4|0;
 $pCell = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $nCell;
 HEAP32[$3>>2] = $apCell;
 HEAP32[$4>>2] = $szCell;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$hdr>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aData>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$usableSize>>2] = $16;
 $17 = HEAP32[$usableSize>>2]|0;
 $18 = HEAP32[$aData>>2]|0;
 $19 = (($18) + ($17)|0);
 HEAP32[$pEnd>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pCellptr>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 52|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3PagerTempSpace($26)|0);
 HEAP32[$pTmp>>2] = $27;
 $28 = HEAP32[$hdr>>2]|0;
 $29 = (($28) + 5)|0;
 $30 = HEAP32[$aData>>2]|0;
 $31 = (($30) + ($29)|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = $33 << 8;
 $35 = HEAP32[$hdr>>2]|0;
 $36 = (($35) + 5)|0;
 $37 = HEAP32[$aData>>2]|0;
 $38 = (($37) + ($36)|0);
 $39 = ((($38)) + 1|0);
 $40 = HEAP8[$39>>0]|0;
 $41 = $40&255;
 $42 = $34 | $41;
 HEAP32[$i>>2] = $42;
 $43 = HEAP32[$i>>2]|0;
 $44 = HEAP32[$pTmp>>2]|0;
 $45 = (($44) + ($43)|0);
 $46 = HEAP32[$i>>2]|0;
 $47 = HEAP32[$aData>>2]|0;
 $48 = (($47) + ($46)|0);
 $49 = HEAP32[$usableSize>>2]|0;
 $50 = HEAP32[$i>>2]|0;
 $51 = (($49) - ($50))|0;
 _memcpy(($45|0),($48|0),($51|0))|0;
 $52 = HEAP32[$pEnd>>2]|0;
 HEAP32[$pData>>2] = $52;
 HEAP32[$i>>2] = 0;
 while(1) {
  $53 = HEAP32[$i>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  $55 = ($53|0)<($54|0);
  if (!($55)) {
   label = 9;
   break;
  }
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$3>>2]|0;
  $58 = (($57) + ($56<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$pCell>>2] = $59;
  $60 = HEAP32[$pCell>>2]|0;
  $61 = HEAP32[$aData>>2]|0;
  $62 = ($60>>>0)>=($61>>>0);
  if ($62) {
   $63 = HEAP32[$pCell>>2]|0;
   $64 = HEAP32[$pEnd>>2]|0;
   $65 = ($63>>>0)<($64>>>0);
   if ($65) {
    $66 = HEAP32[$pCell>>2]|0;
    $67 = HEAP32[$aData>>2]|0;
    $68 = $66;
    $69 = $67;
    $70 = (($68) - ($69))|0;
    $71 = HEAP32[$pTmp>>2]|0;
    $72 = (($71) + ($70)|0);
    HEAP32[$pCell>>2] = $72;
   }
  }
  $73 = HEAP32[$i>>2]|0;
  $74 = HEAP32[$4>>2]|0;
  $75 = (($74) + ($73<<1)|0);
  $76 = HEAP16[$75>>1]|0;
  $77 = $76&65535;
  $78 = HEAP32[$pData>>2]|0;
  $79 = (0 - ($77))|0;
  $80 = (($78) + ($79)|0);
  HEAP32[$pData>>2] = $80;
  $81 = HEAP32[$pData>>2]|0;
  $82 = HEAP32[$aData>>2]|0;
  $83 = $81;
  $84 = $82;
  $85 = (($83) - ($84))|0;
  $86 = $85 >> 8;
  $87 = $86&255;
  $88 = HEAP32[$pCellptr>>2]|0;
  HEAP8[$88>>0] = $87;
  $89 = HEAP32[$pData>>2]|0;
  $90 = HEAP32[$aData>>2]|0;
  $91 = $89;
  $92 = $90;
  $93 = (($91) - ($92))|0;
  $94 = $93&255;
  $95 = HEAP32[$pCellptr>>2]|0;
  $96 = ((($95)) + 1|0);
  HEAP8[$96>>0] = $94;
  $97 = HEAP32[$pCellptr>>2]|0;
  $98 = ((($97)) + 2|0);
  HEAP32[$pCellptr>>2] = $98;
  $99 = HEAP32[$pData>>2]|0;
  $100 = HEAP32[$pCellptr>>2]|0;
  $101 = ($99>>>0)<($100>>>0);
  if ($101) {
   label = 7;
   break;
  }
  $103 = HEAP32[$pData>>2]|0;
  $104 = HEAP32[$pCell>>2]|0;
  $105 = HEAP32[$i>>2]|0;
  $106 = HEAP32[$4>>2]|0;
  $107 = (($106) + ($105<<1)|0);
  $108 = HEAP16[$107>>1]|0;
  $109 = $108&65535;
  _memcpy(($103|0),($104|0),($109|0))|0;
  $110 = HEAP32[$i>>2]|0;
  $111 = (($110) + 1)|0;
  HEAP32[$i>>2] = $111;
 }
 if ((label|0) == 7) {
  $102 = (_sqlite3CorruptError(62048)|0);
  HEAP32[$0>>2] = $102;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 else if ((label|0) == 9) {
  $112 = HEAP32[$2>>2]|0;
  $113 = $112&65535;
  $114 = HEAP32[$1>>2]|0;
  $115 = ((($114)) + 18|0);
  HEAP16[$115>>1] = $113;
  $116 = HEAP32[$1>>2]|0;
  $117 = ((($116)) + 1|0);
  HEAP8[$117>>0] = 0;
  $118 = HEAP32[$hdr>>2]|0;
  $119 = (($118) + 1)|0;
  $120 = HEAP32[$aData>>2]|0;
  $121 = (($120) + ($119)|0);
  HEAP8[$121>>0] = 0;
  $122 = HEAP32[$hdr>>2]|0;
  $123 = (($122) + 1)|0;
  $124 = HEAP32[$aData>>2]|0;
  $125 = (($124) + ($123)|0);
  $126 = ((($125)) + 1|0);
  HEAP8[$126>>0] = 0;
  $127 = HEAP32[$1>>2]|0;
  $128 = ((($127)) + 18|0);
  $129 = HEAP16[$128>>1]|0;
  $130 = $129&65535;
  $131 = $130 >> 8;
  $132 = $131&255;
  $133 = HEAP32[$hdr>>2]|0;
  $134 = (($133) + 3)|0;
  $135 = HEAP32[$aData>>2]|0;
  $136 = (($135) + ($134)|0);
  HEAP8[$136>>0] = $132;
  $137 = HEAP32[$1>>2]|0;
  $138 = ((($137)) + 18|0);
  $139 = HEAP16[$138>>1]|0;
  $140 = $139&255;
  $141 = HEAP32[$hdr>>2]|0;
  $142 = (($141) + 3)|0;
  $143 = HEAP32[$aData>>2]|0;
  $144 = (($143) + ($142)|0);
  $145 = ((($144)) + 1|0);
  HEAP8[$145>>0] = $140;
  $146 = HEAP32[$pData>>2]|0;
  $147 = HEAP32[$aData>>2]|0;
  $148 = $146;
  $149 = $147;
  $150 = (($148) - ($149))|0;
  $151 = $150 >> 8;
  $152 = $151&255;
  $153 = HEAP32[$hdr>>2]|0;
  $154 = (($153) + 5)|0;
  $155 = HEAP32[$aData>>2]|0;
  $156 = (($155) + ($154)|0);
  HEAP8[$156>>0] = $152;
  $157 = HEAP32[$pData>>2]|0;
  $158 = HEAP32[$aData>>2]|0;
  $159 = $157;
  $160 = $158;
  $161 = (($159) - ($160))|0;
  $162 = $161&255;
  $163 = HEAP32[$hdr>>2]|0;
  $164 = (($163) + 5)|0;
  $165 = HEAP32[$aData>>2]|0;
  $166 = (($165) + ($164)|0);
  $167 = ((($166)) + 1|0);
  HEAP8[$167>>0] = $162;
  $168 = HEAP32[$hdr>>2]|0;
  $169 = (($168) + 7)|0;
  $170 = HEAP32[$aData>>2]|0;
  $171 = (($170) + ($169)|0);
  HEAP8[$171>>0] = 0;
  HEAP32[$0>>2] = 0;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 return (0)|0;
}
function _cachedCellSize($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<1)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = ($8<<16>>16)!=(0);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<1)|0);
  $15 = HEAP16[$14>>1]|0;
  HEAP16[$0>>1] = $15;
  $19 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_computeCellSize($16,$17)|0);
  HEAP16[$0>>1] = $18;
  $19 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3PagerRekey($pPg,$iNew,$flags) {
 $pPg = $pPg|0;
 $iNew = $iNew|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$1>>2] = $iNew;
 HEAP16[$2>>1] = $flags;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 HEAP16[$5>>1] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3PcacheMove($6,$7);
 STACKTOP = sp;return;
}
function _editPage($pPg,$iOld,$iNew,$nNew,$pCArray) {
 $pPg = $pPg|0;
 $iOld = $iOld|0;
 $iNew = $iNew|0;
 $nNew = $nNew|0;
 $pCArray = $pCArray|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0;
 var $hdr = 0, $i = 0, $iCell = 0, $iNewEnd = 0, $iOldEnd = 0, $nAdd = 0, $nCell = 0, $nShift = 0, $pBegin = 0, $pCellptr = 0, $pData = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $aData = sp + 44|0;
 $hdr = sp + 40|0;
 $pBegin = sp + 36|0;
 $nCell = sp + 32|0;
 $pData = sp + 28|0;
 $pCellptr = sp + 24|0;
 $i = sp + 20|0;
 $iOldEnd = sp + 16|0;
 $iNewEnd = sp + 12|0;
 $nShift = sp + 8|0;
 $nAdd = sp + 4|0;
 $iCell = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iOld;
 HEAP32[$3>>2] = $iNew;
 HEAP32[$4>>2] = $nNew;
 HEAP32[$5>>2] = $pCArray;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 56|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aData>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 5|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 HEAP32[$hdr>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 $14 = $13<<1;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14)|0);
 HEAP32[$pBegin>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 18|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 HEAP32[$nCell>>2] = $22;
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 18|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = (($23) + ($27))|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 1|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = (($28) + ($32))|0;
 HEAP32[$iOldEnd>>2] = $33;
 $34 = HEAP32[$3>>2]|0;
 $35 = HEAP32[$4>>2]|0;
 $36 = (($34) + ($35))|0;
 HEAP32[$iNewEnd>>2] = $36;
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$3>>2]|0;
 $39 = ($37|0)<($38|0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = (($42) - ($43))|0;
  $45 = HEAP32[$5>>2]|0;
  $46 = (_pageFreeArray($40,$41,$44,$45)|0);
  HEAP32[$nShift>>2] = $46;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 64|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$nShift>>2]|0;
  $51 = $50<<1;
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 64|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51)|0);
  $56 = HEAP32[$nCell>>2]|0;
  $57 = $56<<1;
  _memmove(($49|0),($55|0),($57|0))|0;
  $58 = HEAP32[$nShift>>2]|0;
  $59 = HEAP32[$nCell>>2]|0;
  $60 = (($59) - ($58))|0;
  HEAP32[$nCell>>2] = $60;
 }
 $61 = HEAP32[$iNewEnd>>2]|0;
 $62 = HEAP32[$iOldEnd>>2]|0;
 $63 = ($61|0)<($62|0);
 if ($63) {
  $64 = HEAP32[$1>>2]|0;
  $65 = HEAP32[$iNewEnd>>2]|0;
  $66 = HEAP32[$iOldEnd>>2]|0;
  $67 = HEAP32[$iNewEnd>>2]|0;
  $68 = (($66) - ($67))|0;
  $69 = HEAP32[$5>>2]|0;
  $70 = (_pageFreeArray($64,$65,$68,$69)|0);
  $71 = HEAP32[$nCell>>2]|0;
  $72 = (($71) - ($70))|0;
  HEAP32[$nCell>>2] = $72;
 }
 $73 = HEAP32[$hdr>>2]|0;
 $74 = (($73) + 5)|0;
 $75 = HEAP32[$aData>>2]|0;
 $76 = (($75) + ($74)|0);
 $77 = HEAP8[$76>>0]|0;
 $78 = $77&255;
 $79 = $78 << 8;
 $80 = HEAP32[$hdr>>2]|0;
 $81 = (($80) + 5)|0;
 $82 = HEAP32[$aData>>2]|0;
 $83 = (($82) + ($81)|0);
 $84 = ((($83)) + 1|0);
 $85 = HEAP8[$84>>0]|0;
 $86 = $85&255;
 $87 = $79 | $86;
 $88 = (($87) - 1)|0;
 $89 = $88 & 65535;
 $90 = (($89) + 1)|0;
 $91 = HEAP32[$aData>>2]|0;
 $92 = (($91) + ($90)|0);
 HEAP32[$pData>>2] = $92;
 $93 = HEAP32[$pData>>2]|0;
 $94 = HEAP32[$pBegin>>2]|0;
 $95 = ($93>>>0)<($94>>>0);
 L7: do {
  if (!($95)) {
   $96 = HEAP32[$3>>2]|0;
   $97 = HEAP32[$2>>2]|0;
   $98 = ($96|0)<($97|0);
   if ($98) {
    $99 = HEAP32[$4>>2]|0;
    $100 = HEAP32[$2>>2]|0;
    $101 = HEAP32[$3>>2]|0;
    $102 = (($100) - ($101))|0;
    $103 = ($99|0)<($102|0);
    if ($103) {
     $104 = HEAP32[$4>>2]|0;
     $108 = $104;
    } else {
     $105 = HEAP32[$2>>2]|0;
     $106 = HEAP32[$3>>2]|0;
     $107 = (($105) - ($106))|0;
     $108 = $107;
    }
    HEAP32[$nAdd>>2] = $108;
    $109 = HEAP32[$1>>2]|0;
    $110 = ((($109)) + 64|0);
    $111 = HEAP32[$110>>2]|0;
    HEAP32[$pCellptr>>2] = $111;
    $112 = HEAP32[$nAdd>>2]|0;
    $113 = $112<<1;
    $114 = HEAP32[$pCellptr>>2]|0;
    $115 = (($114) + ($113)|0);
    $116 = HEAP32[$pCellptr>>2]|0;
    $117 = HEAP32[$nCell>>2]|0;
    $118 = $117<<1;
    _memmove(($115|0),($116|0),($118|0))|0;
    $119 = HEAP32[$1>>2]|0;
    $120 = HEAP32[$pBegin>>2]|0;
    $121 = HEAP32[$pCellptr>>2]|0;
    $122 = HEAP32[$3>>2]|0;
    $123 = HEAP32[$nAdd>>2]|0;
    $124 = HEAP32[$5>>2]|0;
    $125 = (_pageInsertArray($119,$120,$pData,$121,$122,$123,$124)|0);
    $126 = ($125|0)!=(0);
    if ($126) {
     break;
    }
    $127 = HEAP32[$nAdd>>2]|0;
    $128 = HEAP32[$nCell>>2]|0;
    $129 = (($128) + ($127))|0;
    HEAP32[$nCell>>2] = $129;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $130 = HEAP32[$i>>2]|0;
    $131 = HEAP32[$1>>2]|0;
    $132 = ((($131)) + 1|0);
    $133 = HEAP8[$132>>0]|0;
    $134 = $133&255;
    $135 = ($130|0)<($134|0);
    if (!($135)) {
     break;
    }
    $136 = HEAP32[$2>>2]|0;
    $137 = HEAP32[$i>>2]|0;
    $138 = HEAP32[$1>>2]|0;
    $139 = ((($138)) + 22|0);
    $140 = (($139) + ($137<<1)|0);
    $141 = HEAP16[$140>>1]|0;
    $142 = $141&65535;
    $143 = (($136) + ($142))|0;
    $144 = HEAP32[$3>>2]|0;
    $145 = (($143) - ($144))|0;
    HEAP32[$iCell>>2] = $145;
    $146 = HEAP32[$iCell>>2]|0;
    $147 = ($146|0)>=(0);
    if ($147) {
     $148 = HEAP32[$iCell>>2]|0;
     $149 = HEAP32[$4>>2]|0;
     $150 = ($148|0)<($149|0);
     if ($150) {
      $151 = HEAP32[$iCell>>2]|0;
      $152 = $151<<1;
      $153 = HEAP32[$1>>2]|0;
      $154 = ((($153)) + 64|0);
      $155 = HEAP32[$154>>2]|0;
      $156 = (($155) + ($152)|0);
      HEAP32[$pCellptr>>2] = $156;
      $157 = HEAP32[$pCellptr>>2]|0;
      $158 = ((($157)) + 2|0);
      $159 = HEAP32[$pCellptr>>2]|0;
      $160 = HEAP32[$nCell>>2]|0;
      $161 = HEAP32[$iCell>>2]|0;
      $162 = (($160) - ($161))|0;
      $163 = $162<<1;
      _memmove(($158|0),($159|0),($163|0))|0;
      $164 = HEAP32[$nCell>>2]|0;
      $165 = (($164) + 1)|0;
      HEAP32[$nCell>>2] = $165;
      $166 = HEAP32[$1>>2]|0;
      $167 = HEAP32[$pBegin>>2]|0;
      $168 = HEAP32[$pCellptr>>2]|0;
      $169 = HEAP32[$iCell>>2]|0;
      $170 = HEAP32[$3>>2]|0;
      $171 = (($169) + ($170))|0;
      $172 = HEAP32[$5>>2]|0;
      $173 = (_pageInsertArray($166,$167,$pData,$168,$171,1,$172)|0);
      $174 = ($173|0)!=(0);
      if ($174) {
       break L7;
      }
     }
    }
    $175 = HEAP32[$i>>2]|0;
    $176 = (($175) + 1)|0;
    HEAP32[$i>>2] = $176;
   }
   $177 = HEAP32[$nCell>>2]|0;
   $178 = $177<<1;
   $179 = HEAP32[$1>>2]|0;
   $180 = ((($179)) + 64|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = (($181) + ($178)|0);
   HEAP32[$pCellptr>>2] = $182;
   $183 = HEAP32[$1>>2]|0;
   $184 = HEAP32[$pBegin>>2]|0;
   $185 = HEAP32[$pCellptr>>2]|0;
   $186 = HEAP32[$3>>2]|0;
   $187 = HEAP32[$nCell>>2]|0;
   $188 = (($186) + ($187))|0;
   $189 = HEAP32[$4>>2]|0;
   $190 = HEAP32[$nCell>>2]|0;
   $191 = (($189) - ($190))|0;
   $192 = HEAP32[$5>>2]|0;
   $193 = (_pageInsertArray($183,$184,$pData,$185,$188,$191,$192)|0);
   $194 = ($193|0)!=(0);
   if (!($194)) {
    $195 = HEAP32[$4>>2]|0;
    $196 = $195&65535;
    $197 = HEAP32[$1>>2]|0;
    $198 = ((($197)) + 18|0);
    HEAP16[$198>>1] = $196;
    $199 = HEAP32[$1>>2]|0;
    $200 = ((($199)) + 1|0);
    HEAP8[$200>>0] = 0;
    $201 = HEAP32[$1>>2]|0;
    $202 = ((($201)) + 18|0);
    $203 = HEAP16[$202>>1]|0;
    $204 = $203&65535;
    $205 = $204 >> 8;
    $206 = $205&255;
    $207 = HEAP32[$hdr>>2]|0;
    $208 = (($207) + 3)|0;
    $209 = HEAP32[$aData>>2]|0;
    $210 = (($209) + ($208)|0);
    HEAP8[$210>>0] = $206;
    $211 = HEAP32[$1>>2]|0;
    $212 = ((($211)) + 18|0);
    $213 = HEAP16[$212>>1]|0;
    $214 = $213&255;
    $215 = HEAP32[$hdr>>2]|0;
    $216 = (($215) + 3)|0;
    $217 = HEAP32[$aData>>2]|0;
    $218 = (($217) + ($216)|0);
    $219 = ((($218)) + 1|0);
    HEAP8[$219>>0] = $214;
    $220 = HEAP32[$pData>>2]|0;
    $221 = HEAP32[$aData>>2]|0;
    $222 = $220;
    $223 = $221;
    $224 = (($222) - ($223))|0;
    $225 = $224 >> 8;
    $226 = $225&255;
    $227 = HEAP32[$hdr>>2]|0;
    $228 = (($227) + 5)|0;
    $229 = HEAP32[$aData>>2]|0;
    $230 = (($229) + ($228)|0);
    HEAP8[$230>>0] = $226;
    $231 = HEAP32[$pData>>2]|0;
    $232 = HEAP32[$aData>>2]|0;
    $233 = $231;
    $234 = $232;
    $235 = (($233) - ($234))|0;
    $236 = $235&255;
    $237 = HEAP32[$hdr>>2]|0;
    $238 = (($237) + 5)|0;
    $239 = HEAP32[$aData>>2]|0;
    $240 = (($239) + ($238)|0);
    $241 = ((($240)) + 1|0);
    HEAP8[$241>>0] = $236;
    HEAP32[$0>>2] = 0;
    $258 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($258|0);
   }
  }
 } while(0);
 $242 = HEAP32[$5>>2]|0;
 $243 = HEAP32[$3>>2]|0;
 $244 = HEAP32[$4>>2]|0;
 _populateCellCache($242,$243,$244);
 $245 = HEAP32[$1>>2]|0;
 $246 = HEAP32[$4>>2]|0;
 $247 = HEAP32[$3>>2]|0;
 $248 = HEAP32[$5>>2]|0;
 $249 = ((($248)) + 8|0);
 $250 = HEAP32[$249>>2]|0;
 $251 = (($250) + ($247<<2)|0);
 $252 = HEAP32[$3>>2]|0;
 $253 = HEAP32[$5>>2]|0;
 $254 = ((($253)) + 12|0);
 $255 = HEAP32[$254>>2]|0;
 $256 = (($255) + ($252<<1)|0);
 $257 = (_rebuildPage($245,$246,$251,$256)|0);
 HEAP32[$0>>2] = $257;
 $258 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($258|0);
}
function _freePage($pPage,$pRC) {
 $pPage = $pPage|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pRC;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 84|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_freePage2($7,$8,$11)|0);
 $13 = HEAP32[$1>>2]|0;
 HEAP32[$13>>2] = $12;
 STACKTOP = sp;return;
}
function _computeCellSize($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<2)|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (FUNCTION_TABLE_iii[$6 & 255]($9,$15)|0);
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17<<1)|0);
 HEAP16[$21>>1] = $16;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<1)|0);
 $27 = HEAP16[$26>>1]|0;
 STACKTOP = sp;return ($27|0);
}
function _pageFreeArray($pPg,$iFirst,$nCell,$pCArray) {
 $pPg = $pPg|0;
 $iFirst = $iFirst|0;
 $nCell = $nCell|0;
 $pCArray = $pCArray|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $aData = 0, $i = 0, $iEnd = 0;
 var $nRet = 0, $pCell = 0, $pEnd = 0, $pFree = 0, $pStart = 0, $sz = 0, $szFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $aData = sp + 36|0;
 $pEnd = sp + 32|0;
 $pStart = sp + 28|0;
 $nRet = sp + 24|0;
 $i = sp + 20|0;
 $iEnd = sp + 16|0;
 $pFree = sp + 12|0;
 $szFree = sp + 8|0;
 $pCell = sp + 4|0;
 $sz = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iFirst;
 HEAP32[$3>>2] = $nCell;
 HEAP32[$4>>2] = $pCArray;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$aData>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 52|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$aData>>2]|0;
 $14 = (($13) + ($12)|0);
 HEAP32[$pEnd>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 5|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = (($18) + 8)|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 6|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = (($19) + ($23))|0;
 $25 = HEAP32[$aData>>2]|0;
 $26 = (($25) + ($24)|0);
 HEAP32[$pStart>>2] = $26;
 HEAP32[$nRet>>2] = 0;
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (($27) + ($28))|0;
 HEAP32[$iEnd>>2] = $29;
 HEAP32[$pFree>>2] = 0;
 HEAP32[$szFree>>2] = 0;
 $30 = HEAP32[$2>>2]|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$iEnd>>2]|0;
  $33 = ($31|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  $36 = ((($35)) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  HEAP32[$pCell>>2] = $39;
  $40 = HEAP32[$pCell>>2]|0;
  $41 = HEAP32[$pStart>>2]|0;
  $42 = ($40>>>0)>=($41>>>0);
  if ($42) {
   $43 = HEAP32[$pCell>>2]|0;
   $44 = HEAP32[$pEnd>>2]|0;
   $45 = ($43>>>0)<($44>>>0);
   if ($45) {
    $46 = HEAP32[$i>>2]|0;
    $47 = HEAP32[$4>>2]|0;
    $48 = ((($47)) + 12|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + ($46<<1)|0);
    $51 = HEAP16[$50>>1]|0;
    $52 = $51&65535;
    HEAP32[$sz>>2] = $52;
    $53 = HEAP32[$pFree>>2]|0;
    $54 = HEAP32[$pCell>>2]|0;
    $55 = HEAP32[$sz>>2]|0;
    $56 = (($54) + ($55)|0);
    $57 = ($53|0)!=($56|0);
    if ($57) {
     $58 = HEAP32[$pFree>>2]|0;
     $59 = ($58|0)!=(0|0);
     if ($59) {
      $60 = HEAP32[$1>>2]|0;
      $61 = HEAP32[$pFree>>2]|0;
      $62 = HEAP32[$aData>>2]|0;
      $63 = $61;
      $64 = $62;
      $65 = (($63) - ($64))|0;
      $66 = $65&65535;
      $67 = HEAP32[$szFree>>2]|0;
      $68 = $67&65535;
      (_freeSpace($60,$66,$68)|0);
     }
     $69 = HEAP32[$pCell>>2]|0;
     HEAP32[$pFree>>2] = $69;
     $70 = HEAP32[$sz>>2]|0;
     HEAP32[$szFree>>2] = $70;
     $71 = HEAP32[$pFree>>2]|0;
     $72 = HEAP32[$sz>>2]|0;
     $73 = (($71) + ($72)|0);
     $74 = HEAP32[$pEnd>>2]|0;
     $75 = ($73>>>0)>($74>>>0);
     if ($75) {
      label = 9;
      break;
     }
    } else {
     $76 = HEAP32[$pCell>>2]|0;
     HEAP32[$pFree>>2] = $76;
     $77 = HEAP32[$sz>>2]|0;
     $78 = HEAP32[$szFree>>2]|0;
     $79 = (($78) + ($77))|0;
     HEAP32[$szFree>>2] = $79;
    }
    $80 = HEAP32[$nRet>>2]|0;
    $81 = (($80) + 1)|0;
    HEAP32[$nRet>>2] = $81;
   }
  }
  $82 = HEAP32[$i>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$i>>2] = $83;
 }
 if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $96 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 $84 = HEAP32[$pFree>>2]|0;
 $85 = ($84|0)!=(0|0);
 if ($85) {
  $86 = HEAP32[$1>>2]|0;
  $87 = HEAP32[$pFree>>2]|0;
  $88 = HEAP32[$aData>>2]|0;
  $89 = $87;
  $90 = $88;
  $91 = (($89) - ($90))|0;
  $92 = $91&65535;
  $93 = HEAP32[$szFree>>2]|0;
  $94 = $93&65535;
  (_freeSpace($86,$92,$94)|0);
 }
 $95 = HEAP32[$nRet>>2]|0;
 HEAP32[$0>>2] = $95;
 $96 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($96|0);
}
function _pageInsertArray($pPg,$pBegin,$ppData,$pCellptr,$iFirst,$nCell,$pCArray) {
 $pPg = $pPg|0;
 $pBegin = $pBegin|0;
 $ppData = $ppData|0;
 $pCellptr = $pCellptr|0;
 $iFirst = $iFirst|0;
 $nCell = $nCell|0;
 $pCArray = $pCArray|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $aData = 0, $i = 0, $iEnd = 0;
 var $pData = 0, $pSlot = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $i = sp + 24|0;
 $aData = sp + 20|0;
 $pData = sp + 16|0;
 $iEnd = sp + 12|0;
 $sz = sp + 8|0;
 $rc = sp + 4|0;
 $pSlot = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $pBegin;
 HEAP32[$3>>2] = $ppData;
 HEAP32[$4>>2] = $pCellptr;
 HEAP32[$5>>2] = $iFirst;
 HEAP32[$6>>2] = $nCell;
 HEAP32[$7>>2] = $pCArray;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$aData>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pData>>2] = $12;
 $13 = HEAP32[$5>>2]|0;
 $14 = HEAP32[$6>>2]|0;
 $15 = (($13) + ($14))|0;
 HEAP32[$iEnd>>2] = $15;
 $16 = HEAP32[$5>>2]|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$iEnd>>2]|0;
  $19 = ($17|0)<($18|0);
  if (!($19)) {
   label = 10;
   break;
  }
  $20 = HEAP32[$7>>2]|0;
  $21 = HEAP32[$i>>2]|0;
  $22 = (_cachedCellSize($20,$21)|0);
  $23 = $22&65535;
  HEAP32[$sz>>2] = $23;
  $24 = HEAP32[$aData>>2]|0;
  $25 = ((($24)) + 1|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = ($27|0)==(0);
  if ($28) {
   $29 = HEAP32[$aData>>2]|0;
   $30 = ((($29)) + 2|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $33 = ($32|0)==(0);
   if ($33) {
    label = 6;
   } else {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $34 = HEAP32[$1>>2]|0;
   $35 = HEAP32[$sz>>2]|0;
   $36 = (_pageFindSlot($34,$35,$rc)|0);
   HEAP32[$pSlot>>2] = $36;
   $37 = ($36|0)==(0|0);
   if ($37) {
    label = 6;
   }
  }
  if ((label|0) == 6) {
   label = 0;
   $38 = HEAP32[$sz>>2]|0;
   $39 = HEAP32[$pData>>2]|0;
   $40 = (0 - ($38))|0;
   $41 = (($39) + ($40)|0);
   HEAP32[$pData>>2] = $41;
   $42 = HEAP32[$pData>>2]|0;
   $43 = HEAP32[$2>>2]|0;
   $44 = ($42>>>0)<($43>>>0);
   if ($44) {
    label = 7;
    break;
   }
   $45 = HEAP32[$pData>>2]|0;
   HEAP32[$pSlot>>2] = $45;
  }
  $46 = HEAP32[$pSlot>>2]|0;
  $47 = HEAP32[$i>>2]|0;
  $48 = HEAP32[$7>>2]|0;
  $49 = ((($48)) + 8|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + ($47<<2)|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$sz>>2]|0;
  _memmove(($46|0),($52|0),($53|0))|0;
  $54 = HEAP32[$pSlot>>2]|0;
  $55 = HEAP32[$aData>>2]|0;
  $56 = $54;
  $57 = $55;
  $58 = (($56) - ($57))|0;
  $59 = $58 >> 8;
  $60 = $59&255;
  $61 = HEAP32[$4>>2]|0;
  HEAP8[$61>>0] = $60;
  $62 = HEAP32[$pSlot>>2]|0;
  $63 = HEAP32[$aData>>2]|0;
  $64 = $62;
  $65 = $63;
  $66 = (($64) - ($65))|0;
  $67 = $66&255;
  $68 = HEAP32[$4>>2]|0;
  $69 = ((($68)) + 1|0);
  HEAP8[$69>>0] = $67;
  $70 = HEAP32[$4>>2]|0;
  $71 = ((($70)) + 2|0);
  HEAP32[$4>>2] = $71;
  $72 = HEAP32[$i>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$i>>2] = $73;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $76 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 else if ((label|0) == 10) {
  $74 = HEAP32[$pData>>2]|0;
  $75 = HEAP32[$3>>2]|0;
  HEAP32[$75>>2] = $74;
  HEAP32[$0>>2] = 0;
  $76 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 return (0)|0;
}
function _populateCellCache($p,$idx,$N) {
 $p = $p|0;
 $idx = $idx|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $N;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)>(0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + ($5<<1)|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10&65535;
  $12 = ($11|0)==(0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 4|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ((($15)) + 76|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 8|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<2)|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (FUNCTION_TABLE_iii[$17 & 255]($20,$26)|0);
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$0>>2]|0;
   $30 = ((($29)) + 12|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + ($28<<1)|0);
   HEAP16[$32>>1] = $27;
  }
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$1>>2] = $34;
  $35 = HEAP32[$2>>2]|0;
  $36 = (($35) + -1)|0;
  HEAP32[$2>>2] = $36;
 }
 STACKTOP = sp;return;
}
function _vdbeSorterRowkey($pSorter,$pnKey) {
 $pSorter = $pSorter|0;
 $pnKey = $pnKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pKey = 0, $pReader = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pKey = sp + 4|0;
 $pReader = sp;
 HEAP32[$0>>2] = $pSorter;
 HEAP32[$1>>2] = $pnKey;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ((($8)) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($12*56)|0)|0);
  HEAP32[$pReader>>2] = $18;
  $19 = HEAP32[$pReader>>2]|0;
  $20 = ((($19)) + 20|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$pReader>>2]|0;
  $24 = ((($23)) + 32|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pKey>>2] = $25;
  $34 = HEAP32[$pKey>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $26 = ((($6)) + 36|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 36|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ((($32)) + 8|0);
  HEAP32[$pKey>>2] = $33;
  $34 = HEAP32[$pKey>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _vdbeSorterSort($pTask,$pList) {
 $pTask = $pTask|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aSlot = 0, $i = 0, $p = 0, $pNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $aSlot = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $pNext = sp;
 HEAP32[$1>>2] = $pTask;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_vdbeSortAllocUnpacked($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_vdbeSorterGetCompare($12)|0);
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 32|0);
 HEAP32[$15>>2] = $13;
 $16 = (_sqlite3MallocZero(256,0)|0);
 HEAP32[$aSlot>>2] = $16;
 $17 = HEAP32[$aSlot>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  HEAP32[$0>>2] = 7;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 while(1) {
  $19 = HEAP32[$p>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$2>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0|0);
  $25 = HEAP32[$p>>2]|0;
  do {
   if ($24) {
    $26 = HEAP32[$2>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = ($25|0)==($28|0);
    if ($29) {
     HEAP32[$pNext>>2] = 0;
     break;
    } else {
     $30 = HEAP32[$p>>2]|0;
     $31 = ((($30)) + 4|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$2>>2]|0;
     $34 = ((($33)) + 4|0);
     $35 = HEAP32[$34>>2]|0;
     $36 = (($35) + ($32)|0);
     HEAP32[$pNext>>2] = $36;
     break;
    }
   } else {
    $37 = ((($25)) + 4|0);
    $38 = HEAP32[$37>>2]|0;
    HEAP32[$pNext>>2] = $38;
   }
  } while(0);
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 4|0);
  HEAP32[$40>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $41 = HEAP32[$i>>2]|0;
   $42 = HEAP32[$aSlot>>2]|0;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(0|0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[$1>>2]|0;
   $47 = HEAP32[$p>>2]|0;
   $48 = HEAP32[$i>>2]|0;
   $49 = HEAP32[$aSlot>>2]|0;
   $50 = (($49) + ($48<<2)|0);
   $51 = HEAP32[$50>>2]|0;
   _vdbeSorterMerge($46,$47,$51,$p);
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$aSlot>>2]|0;
   $54 = (($53) + ($52<<2)|0);
   HEAP32[$54>>2] = 0;
   $55 = HEAP32[$i>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$i>>2] = $56;
  }
  $57 = HEAP32[$p>>2]|0;
  $58 = HEAP32[$i>>2]|0;
  $59 = HEAP32[$aSlot>>2]|0;
  $60 = (($59) + ($58<<2)|0);
  HEAP32[$60>>2] = $57;
  $61 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $61;
 }
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $62 = HEAP32[$i>>2]|0;
  $63 = ($62|0)<(64);
  if (!($63)) {
   break;
  }
  $64 = HEAP32[$1>>2]|0;
  $65 = HEAP32[$p>>2]|0;
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$aSlot>>2]|0;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  _vdbeSorterMerge($64,$65,$69,$p);
  $70 = HEAP32[$i>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$i>>2] = $71;
 }
 $72 = HEAP32[$p>>2]|0;
 $73 = HEAP32[$2>>2]|0;
 HEAP32[$73>>2] = $72;
 $74 = HEAP32[$aSlot>>2]|0;
 _sqlite3_free($74);
 $75 = HEAP32[$1>>2]|0;
 $76 = ((($75)) + 12|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = ((($77)) + 11|0);
 $79 = HEAP8[$78>>0]|0;
 $80 = $79&255;
 HEAP32[$0>>2] = $80;
 $81 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _vdbeSorterFlushPMA($pSorter) {
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSorter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 56|0);
 HEAP8[$2>>0] = 1;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 64|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = (_vdbeSorterListToPMA($4,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _vdbeSorterSetupMerge($pSorter) {
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMain = 0, $pTask0 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $pTask0 = sp + 4|0;
 $pMain = sp;
 HEAP32[$0>>2] = $pSorter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 HEAP32[$pTask0>>2] = $2;
 HEAP32[$pMain>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_vdbeSorterMergeTreeBuild($3,$pMain)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$pTask0>>2]|0;
  $8 = HEAP32[$pMain>>2]|0;
  $9 = (_vdbeMergeEngineInit($7,$8,0)|0);
  HEAP32[$rc>>2] = $9;
  $10 = HEAP32[$pMain>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 20|0);
  HEAP32[$12>>2] = $10;
  HEAP32[$pMain>>2] = 0;
 }
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $15 = HEAP32[$pMain>>2]|0;
 _vdbeMergeEngineFree($15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _vdbeSortAllocUnpacked($pTask) {
 $pTask = $pTask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pFree = sp;
 HEAP32[$1>>2] = $pTask;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 8|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = ((($8)) + 28|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3VdbeAllocUnpackedRecord($10,0,0,$pFree)|0);
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 12|0);
   HEAP32[$13>>2] = $11;
   $14 = HEAP32[$pFree>>2]|0;
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ((($18)) + 28|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = ((($20)) + 6|0);
    $22 = HEAP16[$21>>1]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ((($25)) + 8|0);
    HEAP16[$26>>1] = $22;
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 12|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ((($29)) + 11|0);
    HEAP8[$30>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $31 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($31|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _vdbeSorterGetCompare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(1);
 do {
  if ($6) {
   HEAP32[$0>>2] = 138;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 60|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9&255;
   $11 = ($10|0)==(2);
   if ($11) {
    HEAP32[$0>>2] = 139;
    break;
   } else {
    HEAP32[$0>>2] = 140;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _vdbeSorterMerge($pTask,$p1,$p2,$ppOut) {
 $pTask = $pTask|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bCached = 0, $pFinal = 0, $pp = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pFinal = sp + 12|0;
 $pp = sp + 8|0;
 $bCached = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $p1;
 HEAP32[$2>>2] = $p2;
 HEAP32[$3>>2] = $ppOut;
 HEAP32[$pFinal>>2] = 0;
 HEAP32[$pp>>2] = $pFinal;
 HEAP32[$bCached>>2] = 0;
 while(1) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ($4|0)!=(0|0);
  $6 = HEAP32[$2>>2]|0;
  $7 = ($6|0)!=(0|0);
  $8 = $5 ? $7 : 0;
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 32|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 8|0);
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (FUNCTION_TABLE_iiiiiii[$11 & 255]($12,$bCached,$14,$16,$18,$20)|0);
  HEAP32[$res>>2] = $21;
  $22 = HEAP32[$res>>2]|0;
  $23 = ($22|0)<=(0);
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$pp>>2]|0;
   HEAP32[$25>>2] = $24;
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 4|0);
   HEAP32[$pp>>2] = $27;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$1>>2] = $30;
   continue;
  } else {
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$pp>>2]|0;
   HEAP32[$32>>2] = $31;
   $33 = HEAP32[$2>>2]|0;
   $34 = ((($33)) + 4|0);
   HEAP32[$pp>>2] = $34;
   $35 = HEAP32[$2>>2]|0;
   $36 = ((($35)) + 4|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$2>>2] = $37;
   HEAP32[$bCached>>2] = 0;
   continue;
  }
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ($38|0)!=(0|0);
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = $39 ? $40 : $41;
 $43 = HEAP32[$pp>>2]|0;
 HEAP32[$43>>2] = $42;
 $44 = HEAP32[$pFinal>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 HEAP32[$45>>2] = $44;
 STACKTOP = sp;return;
}
function _vdbeSorterCompareInt($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $12 = 0, $13 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $or$cond = 0, $p1 = 0, $p2 = 0, $res = 0, $s1 = 0, $s2 = 0;
 var $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $p1 = sp + 28|0;
 $p2 = sp + 24|0;
 $s1 = sp + 20|0;
 $s2 = sp + 16|0;
 $v1 = sp + 12|0;
 $v2 = sp + 8|0;
 $res = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p1>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$p2>>2] = $7;
 $8 = HEAP32[$p1>>2]|0;
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$s1>>2] = $11;
 $12 = HEAP32[$p2>>2]|0;
 $13 = ((($12)) + 1|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 HEAP32[$s2>>2] = $15;
 $16 = HEAP32[$p1>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = HEAP32[$p1>>2]|0;
 $20 = (($19) + ($18)|0);
 HEAP32[$v1>>2] = $20;
 $21 = HEAP32[$p2>>2]|0;
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = HEAP32[$p2>>2]|0;
 $25 = (($24) + ($23)|0);
 HEAP32[$v2>>2] = $25;
 $26 = HEAP32[$s1>>2]|0;
 $27 = ($26|0)>(7);
 $28 = HEAP32[$s2>>2]|0;
 $29 = ($28|0)>(7);
 $or$cond = $27 & $29;
 $30 = HEAP32[$s1>>2]|0;
 $31 = HEAP32[$s2>>2]|0;
 L1: do {
  if ($or$cond) {
   $32 = (($30) - ($31))|0;
   HEAP32[$res>>2] = $32;
  } else {
   $33 = ($30|0)==($31|0);
   if ($33) {
    $34 = HEAP32[$v1>>2]|0;
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = HEAP32[$v2>>2]|0;
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = $36 ^ $39;
    $41 = $40 & 128;
    $42 = ($41|0)!=(0);
    if ($42) {
     $43 = HEAP32[$v1>>2]|0;
     $44 = HEAP8[$43>>0]|0;
     $45 = $44&255;
     $46 = $45 & 128;
     $47 = ($46|0)!=(0);
     $48 = $47 ? -1 : 1;
     HEAP32[$res>>2] = $48;
     break;
    }
    HEAP32[$res>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $49 = HEAP32[$i>>2]|0;
     $50 = HEAP32[$s1>>2]|0;
     $51 = (21647 + ($50)|0);
     $52 = HEAP8[$51>>0]|0;
     $53 = $52&255;
     $54 = ($49|0)<($53|0);
     if (!($54)) {
      break L1;
     }
     $55 = HEAP32[$i>>2]|0;
     $56 = HEAP32[$v1>>2]|0;
     $57 = (($56) + ($55)|0);
     $58 = HEAP8[$57>>0]|0;
     $59 = $58&255;
     $60 = HEAP32[$i>>2]|0;
     $61 = HEAP32[$v2>>2]|0;
     $62 = (($61) + ($60)|0);
     $63 = HEAP8[$62>>0]|0;
     $64 = $63&255;
     $65 = (($59) - ($64))|0;
     HEAP32[$res>>2] = $65;
     $66 = ($65|0)!=(0);
     if ($66) {
      break L1;
     }
     $67 = HEAP32[$i>>2]|0;
     $68 = (($67) + 1)|0;
     HEAP32[$i>>2] = $68;
    }
   }
   $69 = HEAP32[$s2>>2]|0;
   $70 = ($69|0)>(7);
   do {
    if ($70) {
     HEAP32[$res>>2] = 1;
    } else {
     $71 = HEAP32[$s1>>2]|0;
     $72 = ($71|0)>(7);
     if ($72) {
      HEAP32[$res>>2] = -1;
      break;
     } else {
      $73 = HEAP32[$s1>>2]|0;
      $74 = HEAP32[$s2>>2]|0;
      $75 = (($73) - ($74))|0;
      HEAP32[$res>>2] = $75;
      break;
     }
    }
   } while(0);
   $76 = HEAP32[$res>>2]|0;
   $77 = ($76|0)>(0);
   if ($77) {
    $78 = HEAP32[$v1>>2]|0;
    $79 = HEAP8[$78>>0]|0;
    $80 = $79&255;
    $81 = $80 & 128;
    $82 = ($81|0)!=(0);
    if (!($82)) {
     break;
    }
    HEAP32[$res>>2] = -1;
    break;
   } else {
    $83 = HEAP32[$v2>>2]|0;
    $84 = HEAP8[$83>>0]|0;
    $85 = $84&255;
    $86 = $85 & 128;
    $87 = ($86|0)!=(0);
    if (!($87)) {
     break;
    }
    HEAP32[$res>>2] = 1;
    break;
   }
  }
 } while(0);
 $88 = HEAP32[$res>>2]|0;
 $89 = ($88|0)==(0);
 $90 = HEAP32[$0>>2]|0;
 $91 = ((($90)) + 8|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = ((($92)) + 28|0);
 $94 = HEAP32[$93>>2]|0;
 if ($89) {
  $95 = ((($94)) + 6|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96&65535;
  $98 = ($97|0)>(1);
  if (!($98)) {
   $112 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($112|0);
  }
  $99 = HEAP32[$0>>2]|0;
  $100 = HEAP32[$1>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = HEAP32[$3>>2]|0;
  $103 = HEAP32[$4>>2]|0;
  $104 = HEAP32[$5>>2]|0;
  $105 = (_vdbeSorterCompareTail($99,$100,$101,$102,$103,$104)|0);
  HEAP32[$res>>2] = $105;
  $112 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($112|0);
 } else {
  $106 = ((($94)) + 16|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = HEAP8[$107>>0]|0;
  $109 = ($108<<24>>24)!=(0);
  if (!($109)) {
   $112 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($112|0);
  }
  $110 = HEAP32[$res>>2]|0;
  $111 = Math_imul($110, -1)|0;
  HEAP32[$res>>2] = $111;
  $112 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 return (0)|0;
}
function _vdbeSorterCompareText($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $n1 = 0, $n2 = 0, $p1 = 0, $p2 = 0, $res = 0, $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $p1 = sp + 24|0;
 $p2 = sp + 20|0;
 $v1 = sp + 16|0;
 $v2 = sp + 12|0;
 $n1 = sp + 8|0;
 $n2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p1>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$p2>>2] = $7;
 $8 = HEAP32[$p1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$p1>>2]|0;
 $12 = (($11) + ($10)|0);
 HEAP32[$v1>>2] = $12;
 $13 = HEAP32[$p2>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$p2>>2]|0;
 $17 = (($16) + ($15)|0);
 HEAP32[$v2>>2] = $17;
 $18 = HEAP32[$p1>>2]|0;
 $19 = ((($18)) + 1|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)<(128);
 $23 = HEAP32[$p1>>2]|0;
 $24 = ((($23)) + 1|0);
 if ($22) {
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  HEAP32[$n1>>2] = $26;
 } else {
  (_sqlite3GetVarint32($24,$n1)|0);
 }
 $27 = HEAP32[$n1>>2]|0;
 $28 = (($27) - 13)|0;
 $29 = (($28|0) / 2)&-1;
 HEAP32[$n1>>2] = $29;
 $30 = HEAP32[$p2>>2]|0;
 $31 = ((($30)) + 1|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = ($33|0)<(128);
 $35 = HEAP32[$p2>>2]|0;
 $36 = ((($35)) + 1|0);
 if ($34) {
  $37 = HEAP8[$36>>0]|0;
  $38 = $37&255;
  HEAP32[$n2>>2] = $38;
 } else {
  (_sqlite3GetVarint32($36,$n2)|0);
 }
 $39 = HEAP32[$n2>>2]|0;
 $40 = (($39) - 13)|0;
 $41 = (($40|0) / 2)&-1;
 HEAP32[$n2>>2] = $41;
 $42 = HEAP32[$v1>>2]|0;
 $43 = HEAP32[$v2>>2]|0;
 $44 = HEAP32[$n1>>2]|0;
 $45 = HEAP32[$n2>>2]|0;
 $46 = ($44|0)<($45|0);
 $47 = HEAP32[$n1>>2]|0;
 $48 = HEAP32[$n2>>2]|0;
 $49 = $46 ? $47 : $48;
 $50 = (_memcmp($42,$43,$49)|0);
 HEAP32[$res>>2] = $50;
 $51 = HEAP32[$res>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$n1>>2]|0;
  $54 = HEAP32[$n2>>2]|0;
  $55 = (($53) - ($54))|0;
  HEAP32[$res>>2] = $55;
 }
 $56 = HEAP32[$res>>2]|0;
 $57 = ($56|0)==(0);
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 8|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ((($60)) + 28|0);
 $62 = HEAP32[$61>>2]|0;
 if ($57) {
  $63 = ((($62)) + 6|0);
  $64 = HEAP16[$63>>1]|0;
  $65 = $64&65535;
  $66 = ($65|0)>(1);
  if (!($66)) {
   $80 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($80|0);
  }
  $67 = HEAP32[$0>>2]|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = HEAP32[$3>>2]|0;
  $71 = HEAP32[$4>>2]|0;
  $72 = HEAP32[$5>>2]|0;
  $73 = (_vdbeSorterCompareTail($67,$68,$69,$70,$71,$72)|0);
  HEAP32[$res>>2] = $73;
  $80 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($80|0);
 } else {
  $74 = ((($62)) + 16|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP8[$75>>0]|0;
  $77 = ($76<<24>>24)!=(0);
  if (!($77)) {
   $80 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($80|0);
  }
  $78 = HEAP32[$res>>2]|0;
  $79 = Math_imul($78, -1)|0;
  HEAP32[$res>>2] = $79;
  $80 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 return (0)|0;
}
function _vdbeSorterCompare($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $r2 = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$r2>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$r2>>2]|0;
  $24 = (_sqlite3VdbeRecordCompare($21,$22,$23)|0);
  STACKTOP = sp;return ($24|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$5>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($16,$17,$18,$19);
 $20 = HEAP32[$1>>2]|0;
 HEAP32[$20>>2] = 1;
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$r2>>2]|0;
 $24 = (_sqlite3VdbeRecordCompare($21,$22,$23)|0);
 STACKTOP = sp;return ($24|0);
}
function _vdbeSorterCompareTail($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $r2 = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$r2>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$r2>>2]|0;
  $24 = (_sqlite3VdbeRecordCompareWithSkip($21,$22,$23,1)|0);
  STACKTOP = sp;return ($24|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$5>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($16,$17,$18,$19);
 $20 = HEAP32[$1>>2]|0;
 HEAP32[$20>>2] = 1;
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$r2>>2]|0;
 $24 = (_sqlite3VdbeRecordCompareWithSkip($21,$22,$23,1)|0);
 STACKTOP = sp;return ($24|0);
}
function _vdbeSorterListToPMA($pTask,$pList) {
 $pTask = $pTask|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $pNext = 0, $rc = 0, $writer = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $db = sp + 52|0;
 $rc = sp + 48|0;
 $writer = sp;
 $p = sp + 44|0;
 $pNext = sp + 40|0;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP32[$rc>>2] = 0;
 dest=$writer; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 40|0);
  $14 = (_vdbeSorterOpenTempFile($11,0,0,$13)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (_vdbeSorterSort($17,$18)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if (!($21)) {
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 HEAP32[$pNext>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 40|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 40|0);
 $32 = ((($31)) + 8|0);
 $33 = $32;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 _vdbePmaWriterInit($24,$writer,$29,$35,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 28|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 1)|0;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$1>>2]|0;
 $44 = ((($43)) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)<(0);
 $47 = $46 << 31 >> 31;
 _vdbePmaWriteVarint($writer,$45,$47);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 HEAP32[$p>>2] = $49;
 while(1) {
  $50 = HEAP32[$p>>2]|0;
  $51 = ($50|0)!=(0|0);
  $52 = HEAP32[$p>>2]|0;
  if (!($51)) {
   break;
  }
  $53 = ((($52)) + 4|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pNext>>2] = $54;
  $55 = HEAP32[$p>>2]|0;
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  _vdbePmaWriteVarint($writer,$56,$58);
  $59 = HEAP32[$p>>2]|0;
  $60 = ((($59)) + 8|0);
  $61 = HEAP32[$p>>2]|0;
  $62 = HEAP32[$61>>2]|0;
  _vdbePmaWriteBlob($writer,$60,$62);
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)==(0|0);
  if ($66) {
   $67 = HEAP32[$p>>2]|0;
   _sqlite3_free($67);
  }
  $68 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $68;
 }
 $69 = HEAP32[$1>>2]|0;
 HEAP32[$69>>2] = $52;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 40|0);
 $72 = ((($71)) + 8|0);
 $73 = (_vdbePmaWriterFinish($writer,$72)|0);
 HEAP32[$rc>>2] = $73;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _vdbeSorterOpenTempFile($db,$0,$1,$ppFd) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppFd = $ppFd|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $max = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 8|0;
 $5 = sp + 20|0;
 $rc = sp + 16|0;
 $max = sp;
 HEAP32[$3>>2] = $db;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $ppFd;
 $10 = (_sqlite3FaultSim(202)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  HEAP32[$2>>2] = 3338;
  $25 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$5>>2]|0;
 $15 = (_sqlite3OsOpenMalloc($13,0,$14,4126,$rc)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = $max;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$5>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  _sqlite3OsFileControlHint($23,18,$max);
 }
 $24 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $24;
 $25 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _vdbePmaWriterInit($pFd,$p,$nBuf,$0,$1) {
 $pFd = $pFd|0;
 $p = $p|0;
 $nBuf = $nBuf|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0, $8 = 0, $9 = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $pFd;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $nBuf;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$3>>2]|0;
 dest=$10; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $11 = HEAP32[$4>>2]|0;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = (_sqlite3Malloc($11,$13)|0);
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 4|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $22 = $5;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$4>>2]|0;
  $29 = ($28|0)<(0);
  $30 = $29 << 31 >> 31;
  $31 = (___remdi3(($24|0),($27|0),($28|0),($30|0))|0);
  $32 = tempRet0;
  $33 = HEAP32[$3>>2]|0;
  $34 = ((($33)) + 12|0);
  HEAP32[$34>>2] = $31;
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 16|0);
  HEAP32[$36>>2] = $31;
  $37 = $5;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$3>>2]|0;
  $44 = ((($43)) + 12|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)<(0);
  $47 = $46 << 31 >> 31;
  $48 = (_i64Subtract(($39|0),($42|0),($45|0),($47|0))|0);
  $49 = tempRet0;
  $50 = HEAP32[$3>>2]|0;
  $51 = ((($50)) + 24|0);
  $52 = $51;
  $53 = $52;
  HEAP32[$53>>2] = $48;
  $54 = (($52) + 4)|0;
  $55 = $54;
  HEAP32[$55>>2] = $49;
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$3>>2]|0;
  $58 = ((($57)) + 8|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP32[$2>>2]|0;
  $60 = HEAP32[$3>>2]|0;
  $61 = ((($60)) + 32|0);
  HEAP32[$61>>2] = $59;
  STACKTOP = sp;return;
 } else {
  $21 = HEAP32[$3>>2]|0;
  HEAP32[$21>>2] = 7;
  STACKTOP = sp;return;
 }
}
function _vdbePmaWriteVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aByte = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 12|0;
 $3 = sp;
 $nByte = sp + 8|0;
 $aByte = sp + 16|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3PutVarint($aByte,$10,$13)|0);
 HEAP32[$nByte>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$nByte>>2]|0;
 _vdbePmaWriteBlob($15,$aByte,$16);
 STACKTOP = sp;return;
}
function _vdbePmaWriteBlob($p,$pData,$nData) {
 $p = $p|0;
 $pData = $pData|0;
 $nData = $nData|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nCopy = 0, $nRem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nRem = sp + 4|0;
 $nCopy = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pData;
 HEAP32[$2>>2] = $nData;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$nRem>>2] = $3;
 while(1) {
  $4 = HEAP32[$nRem>>2]|0;
  $5 = ($4|0)>(0);
  if (!($5)) {
   label = 9;
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 9;
   break;
  }
  $9 = HEAP32[$nRem>>2]|0;
  HEAP32[$nCopy>>2] = $9;
  $10 = HEAP32[$nCopy>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13) - ($16))|0;
  $18 = ($10|0)>($17|0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 16|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($21) - ($24))|0;
   HEAP32[$nCopy>>2] = $25;
  }
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28)|0);
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$nRem>>2]|0;
  $35 = (($33) - ($34))|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = (($36) + ($35)|0);
  $38 = HEAP32[$nCopy>>2]|0;
  _memcpy(($32|0),($37|0),($38|0))|0;
  $39 = HEAP32[$nCopy>>2]|0;
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 16|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + ($39))|0;
  HEAP32[$41>>2] = $43;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 16|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 8|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($46|0)==($49|0);
  if ($50) {
   $51 = HEAP32[$0>>2]|0;
   $52 = ((($51)) + 32|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$0>>2]|0;
   $55 = ((($54)) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + ($56)|0);
   $61 = HEAP32[$0>>2]|0;
   $62 = ((($61)) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$0>>2]|0;
   $65 = ((($64)) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($63) - ($66))|0;
   $68 = HEAP32[$0>>2]|0;
   $69 = ((($68)) + 24|0);
   $70 = $69;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($70) + 4)|0;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$0>>2]|0;
   $77 = ((($76)) + 12|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)<(0);
   $80 = $79 << 31 >> 31;
   $81 = (_i64Add(($72|0),($75|0),($78|0),($80|0))|0);
   $82 = tempRet0;
   $83 = (_sqlite3OsWrite($53,$60,$67,$81,$82)|0);
   $84 = HEAP32[$0>>2]|0;
   HEAP32[$84>>2] = $83;
   $85 = HEAP32[$0>>2]|0;
   $86 = ((($85)) + 16|0);
   HEAP32[$86>>2] = 0;
   $87 = HEAP32[$0>>2]|0;
   $88 = ((($87)) + 12|0);
   HEAP32[$88>>2] = 0;
   $89 = HEAP32[$0>>2]|0;
   $90 = ((($89)) + 8|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = ($91|0)<(0);
   $93 = $92 << 31 >> 31;
   $94 = HEAP32[$0>>2]|0;
   $95 = ((($94)) + 24|0);
   $96 = $95;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($96) + 4)|0;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = (_i64Add(($98|0),($101|0),($91|0),($93|0))|0);
   $103 = tempRet0;
   $104 = $95;
   $105 = $104;
   HEAP32[$105>>2] = $102;
   $106 = (($104) + 4)|0;
   $107 = $106;
   HEAP32[$107>>2] = $103;
  }
  $108 = HEAP32[$nCopy>>2]|0;
  $109 = HEAP32[$nRem>>2]|0;
  $110 = (($109) - ($108))|0;
  HEAP32[$nRem>>2] = $110;
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
}
function _vdbePmaWriterFinish($p,$piEof) {
 $p = $p|0;
 $piEof = $piEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $rc = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piEof;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 16|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($11|0)>($14|0);
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = ((($16)) + 32|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$0>>2]|0;
    $20 = ((($19)) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($24) + ($21)|0);
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$0>>2]|0;
    $30 = ((($29)) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (($28) - ($31))|0;
    $33 = HEAP32[$0>>2]|0;
    $34 = ((($33)) + 24|0);
    $35 = $34;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($35) + 4)|0;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$0>>2]|0;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (_i64Add(($37|0),($40|0),($43|0),($45|0))|0);
    $47 = tempRet0;
    $48 = (_sqlite3OsWrite($18,$25,$32,$46,$47)|0);
    $49 = HEAP32[$0>>2]|0;
    HEAP32[$49>>2] = $48;
   }
  }
 }
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = $51;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)<(0);
 $62 = $61 << 31 >> 31;
 $63 = (_i64Add(($54|0),($57|0),($60|0),($62|0))|0);
 $64 = tempRet0;
 $65 = HEAP32[$1>>2]|0;
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $63;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $64;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 4|0);
 $72 = HEAP32[$71>>2]|0;
 _sqlite3_free($72);
 $73 = HEAP32[$0>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 HEAP32[$rc>>2] = $74;
 $75 = HEAP32[$0>>2]|0;
 dest=$75; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _vdbeSorterMergeTreeBuild($pSorter,$ppOut) {
 $pSorter = $pSorter|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $i = 0;
 var $iReadOff = 0, $iSeq = 0, $iTask = 0, $nDepth = 0, $nReader = 0, $pMain = 0, $pMerger = 0, $pRoot = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pMain = sp + 44|0;
 $rc = sp + 40|0;
 $iTask = sp + 36|0;
 $pTask = sp + 32|0;
 $pRoot = sp + 28|0;
 $nDepth = sp + 24|0;
 $iReadOff = sp;
 $i = sp + 20|0;
 $iSeq = sp + 16|0;
 $pMerger = sp + 12|0;
 $nReader = sp + 8|0;
 HEAP32[$0>>2] = $pSorter;
 HEAP32[$1>>2] = $ppOut;
 HEAP32[$pMain>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iTask>>2] = 0;
 while(1) {
  $2 = HEAP32[$rc>>2]|0;
  $3 = ($2|0)==(0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$iTask>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 59|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$iTask>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 64|0);
  $13 = (($12) + (($10*72)|0)|0);
  HEAP32[$pTask>>2] = $13;
  HEAP32[$pRoot>>2] = 0;
  $14 = HEAP32[$pTask>>2]|0;
  $15 = ((($14)) + 28|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_vdbeSorterTreeDepth($16)|0);
  HEAP32[$nDepth>>2] = $17;
  $18 = $iReadOff;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$pTask>>2]|0;
  $23 = ((($22)) + 28|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)<=(16);
  L5: do {
   if ($25) {
    $26 = HEAP32[$pTask>>2]|0;
    $27 = HEAP32[$pTask>>2]|0;
    $28 = ((($27)) + 28|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = (_vdbeMergeEngineLevel0($26,$29,$iReadOff,$pRoot)|0);
    HEAP32[$rc>>2] = $30;
   } else {
    HEAP32[$iSeq>>2] = 0;
    $31 = (_vdbeMergeEngineNew(16)|0);
    HEAP32[$pRoot>>2] = $31;
    $32 = HEAP32[$pRoot>>2]|0;
    $33 = ($32|0)==(0|0);
    if ($33) {
     HEAP32[$rc>>2] = 7;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $34 = HEAP32[$i>>2]|0;
     $35 = HEAP32[$pTask>>2]|0;
     $36 = ((($35)) + 28|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = ($34|0)<($37|0);
     $39 = HEAP32[$rc>>2]|0;
     $40 = ($39|0)==(0);
     $41 = $38 ? $40 : 0;
     if (!($41)) {
      break L5;
     }
     HEAP32[$pMerger>>2] = 0;
     $42 = HEAP32[$pTask>>2]|0;
     $43 = ((($42)) + 28|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$i>>2]|0;
     $46 = (($44) - ($45))|0;
     $47 = ($46|0)<(16);
     if ($47) {
      $48 = HEAP32[$pTask>>2]|0;
      $49 = ((($48)) + 28|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = HEAP32[$i>>2]|0;
      $52 = (($50) - ($51))|0;
      $53 = $52;
     } else {
      $53 = 16;
     }
     HEAP32[$nReader>>2] = $53;
     $54 = HEAP32[$pTask>>2]|0;
     $55 = HEAP32[$nReader>>2]|0;
     $56 = (_vdbeMergeEngineLevel0($54,$55,$iReadOff,$pMerger)|0);
     HEAP32[$rc>>2] = $56;
     $57 = HEAP32[$rc>>2]|0;
     $58 = ($57|0)==(0);
     if ($58) {
      $59 = HEAP32[$pTask>>2]|0;
      $60 = HEAP32[$nDepth>>2]|0;
      $61 = HEAP32[$iSeq>>2]|0;
      $62 = (($61) + 1)|0;
      HEAP32[$iSeq>>2] = $62;
      $63 = HEAP32[$pRoot>>2]|0;
      $64 = HEAP32[$pMerger>>2]|0;
      $65 = (_vdbeSorterAddToTree($59,$60,$61,$63,$64)|0);
      HEAP32[$rc>>2] = $65;
     }
     $66 = HEAP32[$i>>2]|0;
     $67 = (($66) + 16)|0;
     HEAP32[$i>>2] = $67;
    }
   }
  } while(0);
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)==(0);
  $70 = HEAP32[$pRoot>>2]|0;
  if ($69) {
   HEAP32[$pMain>>2] = $70;
  } else {
   _vdbeMergeEngineFree($70);
  }
  $71 = HEAP32[$iTask>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$iTask>>2] = $72;
 }
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)!=(0);
 if (!($74)) {
  $76 = HEAP32[$pMain>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  HEAP32[$77>>2] = $76;
  $78 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $75 = HEAP32[$pMain>>2]|0;
 _vdbeMergeEngineFree($75);
 HEAP32[$pMain>>2] = 0;
 $76 = HEAP32[$pMain>>2]|0;
 $77 = HEAP32[$1>>2]|0;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _vdbeMergeEngineInit($pTask,$pMerger,$eMode) {
 $pTask = $pTask|0;
 $pMerger = $pMerger|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $nTree = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $nTree = sp;
 HEAP32[$1>>2] = $pTask;
 HEAP32[$2>>2] = $pMerger;
 HEAP32[$3>>2] = $eMode;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nTree>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$nTree>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*56)|0)|0);
  $17 = (_vdbePmaReaderIncrInit($16,0)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)!=(0);
  if ($19) {
   label = 4;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 if ((label|0) == 4) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) - 1)|0;
 HEAP32[$i>>2] = $25;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = ($26|0)>(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$i>>2]|0;
  _vdbeMergeEngineCompare($28,$29);
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + -1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ((($34)) + 11|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _vdbeSorterTreeDepth($nPMA) {
 $nPMA = $nPMA|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nDepth = 0, $nDiv = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $nDepth = sp + 8|0;
 $nDiv = sp;
 HEAP32[$0>>2] = $nPMA;
 HEAP32[$nDepth>>2] = 0;
 $1 = $nDiv;
 $2 = $1;
 HEAP32[$2>>2] = 16;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 while(1) {
  $5 = $nDiv;
  $6 = $5;
  $7 = HEAP32[$6>>2]|0;
  $8 = (($5) + 4)|0;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = ($10|0)<($13|0);
  $15 = ($7>>>0)<($11>>>0);
  $16 = ($10|0)==($13|0);
  $17 = $16 & $15;
  $18 = $14 | $17;
  if (!($18)) {
   break;
  }
  $19 = $nDiv;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (___muldi3(($21|0),($24|0),16,0)|0);
  $26 = tempRet0;
  $27 = $nDiv;
  $28 = $27;
  HEAP32[$28>>2] = $25;
  $29 = (($27) + 4)|0;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = HEAP32[$nDepth>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$nDepth>>2] = $32;
 }
 $33 = HEAP32[$nDepth>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeMergeEngineLevel0($pTask,$nPMA,$piOffset,$ppOut) {
 $pTask = $pTask|0;
 $nPMA = $nPMA|0;
 $piOffset = $piOffset|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $iOff = 0, $nDummy = 0, $pNew = 0, $pReadr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pNew = sp + 28|0;
 $iOff = sp + 8|0;
 $i = sp + 24|0;
 $rc = sp + 20|0;
 $nDummy = sp;
 $pReadr = sp + 16|0;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $nPMA;
 HEAP32[$2>>2] = $piOffset;
 HEAP32[$3>>2] = $ppOut;
 $4 = HEAP32[$2>>2]|0;
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = $iOff;
 $12 = $11;
 HEAP32[$12>>2] = $7;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $10;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = (_vdbeMergeEngineNew($15)|0);
 HEAP32[$pNew>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  HEAP32[$rc>>2] = 7;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ($20|0)<($21|0);
  $23 = HEAP32[$rc>>2]|0;
  $24 = ($23|0)==(0);
  $25 = $22 ? $24 : 0;
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$pNew>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + (($26*56)|0)|0);
  HEAP32[$pReadr>>2] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 40|0);
  $34 = $iOff;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$pReadr>>2]|0;
  $41 = (_vdbePmaReaderInit($31,$33,$36,$39,$40,$nDummy)|0);
  HEAP32[$rc>>2] = $41;
  $42 = HEAP32[$pReadr>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = $43;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = $iOff;
  $51 = $50;
  HEAP32[$51>>2] = $46;
  $52 = (($50) + 4)|0;
  $53 = $52;
  HEAP32[$53>>2] = $49;
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 $56 = HEAP32[$rc>>2]|0;
 $57 = ($56|0)!=(0);
 if (!($57)) {
  $60 = $iOff;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (($60) + 4)|0;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$2>>2]|0;
  $67 = $66;
  $68 = $67;
  HEAP32[$68>>2] = $62;
  $69 = (($67) + 4)|0;
  $70 = $69;
  HEAP32[$70>>2] = $65;
  $71 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $58 = HEAP32[$pNew>>2]|0;
 _vdbeMergeEngineFree($58);
 $59 = HEAP32[$3>>2]|0;
 HEAP32[$59>>2] = 0;
 $60 = $iOff;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (($60) + 4)|0;
 $64 = $63;
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$2>>2]|0;
 $67 = $66;
 $68 = $67;
 HEAP32[$68>>2] = $62;
 $69 = (($67) + 4)|0;
 $70 = $69;
 HEAP32[$70>>2] = $65;
 $71 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($71|0);
}
function _vdbeMergeEngineNew($nReader) {
 $nReader = $nReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $N = 0, $nByte = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $N = sp + 8|0;
 $nByte = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $nReader;
 HEAP32[$N>>2] = 2;
 while(1) {
  $1 = HEAP32[$N>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ($1|0)<($2|0);
  $4 = HEAP32[$N>>2]|0;
  if (!($3)) {
   break;
  }
  $5 = HEAP32[$N>>2]|0;
  $6 = (($5) + ($4))|0;
  HEAP32[$N>>2] = $6;
 }
 $7 = ($4*60)|0;
 $8 = (16 + ($7))|0;
 HEAP32[$nByte>>2] = $8;
 $9 = (_sqlite3FaultSim(100)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $15 = 0;
 } else {
  $11 = HEAP32[$nByte>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = (_sqlite3MallocZero($11,$13)|0);
  $15 = $14;
 }
 HEAP32[$pNew>>2] = $15;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $33 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $18 = HEAP32[$N>>2]|0;
 $19 = HEAP32[$pNew>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ((($22)) + 16|0);
 $24 = HEAP32[$pNew>>2]|0;
 $25 = ((($24)) + 12|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$N>>2]|0;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + (($26*56)|0)|0);
 $31 = HEAP32[$pNew>>2]|0;
 $32 = ((($31)) + 8|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeSorterAddToTree($pTask,$nDepth,$iSeq,$pRoot,$pLeaf) {
 $pTask = $pTask|0;
 $nDepth = $nDepth|0;
 $iSeq = $iSeq|0;
 $pRoot = $pRoot|0;
 $pLeaf = $pLeaf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iIter = 0, $nDiv = 0, $p = 0, $pIncr = 0, $pNew = 0, $pReadr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $rc = sp + 28|0;
 $nDiv = sp + 24|0;
 $i = sp + 20|0;
 $p = sp + 16|0;
 $pIncr = sp + 12|0;
 $iIter = sp + 8|0;
 $pReadr = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $nDepth;
 HEAP32[$2>>2] = $iSeq;
 HEAP32[$3>>2] = $pRoot;
 HEAP32[$4>>2] = $pLeaf;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nDiv>>2] = 1;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = (_vdbeIncrMergerNew($6,$7,$pIncr)|0);
 HEAP32[$rc>>2] = $8;
 HEAP32[$i>>2] = 1;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$nDiv>>2]|0;
  $13 = $12<<4;
  HEAP32[$nDiv>>2] = $13;
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = ($16|0)<($17|0);
  $19 = HEAP32[$rc>>2]|0;
  $20 = ($19|0)==(0);
  $21 = $18 ? $20 : 0;
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$nDiv>>2]|0;
  $24 = (($22|0) / ($23|0))&-1;
  $25 = (($24|0) % 16)&-1;
  HEAP32[$iIter>>2] = $25;
  $26 = HEAP32[$iIter>>2]|0;
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + (($26*56)|0)|0);
  HEAP32[$pReadr>>2] = $30;
  $31 = HEAP32[$pReadr>>2]|0;
  $32 = ((($31)) + 48|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)==(0|0);
  do {
   if ($34) {
    $35 = (_vdbeMergeEngineNew(16)|0);
    HEAP32[$pNew>>2] = $35;
    $36 = HEAP32[$pNew>>2]|0;
    $37 = ($36|0)==(0|0);
    if ($37) {
     HEAP32[$rc>>2] = 7;
     break;
    } else {
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$pNew>>2]|0;
     $40 = HEAP32[$pReadr>>2]|0;
     $41 = ((($40)) + 48|0);
     $42 = (_vdbeIncrMergerNew($38,$39,$41)|0);
     HEAP32[$rc>>2] = $42;
     break;
    }
   }
  } while(0);
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)==(0);
  if ($44) {
   $45 = HEAP32[$pReadr>>2]|0;
   $46 = ((($45)) + 48|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$p>>2] = $49;
   $50 = HEAP32[$nDiv>>2]|0;
   $51 = (($50|0) / 16)&-1;
   HEAP32[$nDiv>>2] = $51;
  }
  $52 = HEAP32[$i>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$i>>2] = $53;
 }
 $54 = HEAP32[$rc>>2]|0;
 $55 = ($54|0)==(0);
 $56 = HEAP32[$pIncr>>2]|0;
 if ($55) {
  $57 = HEAP32[$2>>2]|0;
  $58 = (($57|0) % 16)&-1;
  $59 = HEAP32[$p>>2]|0;
  $60 = ((($59)) + 12|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + (($58*56)|0)|0);
  $63 = ((($62)) + 48|0);
  HEAP32[$63>>2] = $56;
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  _vdbeIncrFree($56);
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _vdbePmaReaderInit($pTask,$pFile,$0,$1,$pReadr,$pnByte) {
 $pTask = $pTask|0;
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 $pReadr = $pReadr|0;
 $pnByte = $pnByte|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $nByte = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 8|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $rc = sp + 16|0;
 $nByte = sp;
 HEAP32[$2>>2] = $pTask;
 HEAP32[$3>>2] = $pFile;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $pReadr;
 HEAP32[$6>>2] = $pnByte;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$5>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = $4;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_vdbePmaReaderSeek($11,$12,$13,$16,$19)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = $nByte;
  $24 = $23;
  HEAP32[$24>>2] = 0;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = 0;
  $27 = HEAP32[$5>>2]|0;
  $28 = (_vdbePmaReadVarint($27,$nByte)|0);
  HEAP32[$rc>>2] = $28;
  $29 = HEAP32[$5>>2]|0;
  $30 = $29;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = $nByte;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (_i64Add(($32|0),($35|0),($38|0),($41|0))|0);
  $43 = tempRet0;
  $44 = HEAP32[$5>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = $45;
  $47 = $46;
  HEAP32[$47>>2] = $42;
  $48 = (($46) + 4)|0;
  $49 = $48;
  HEAP32[$49>>2] = $43;
  $50 = $nByte;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$6>>2]|0;
  $57 = $56;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_i64Add(($59|0),($62|0),($52|0),($55|0))|0);
  $64 = tempRet0;
  $65 = $56;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
 }
 $69 = HEAP32[$rc>>2]|0;
 $70 = ($69|0)==(0);
 if (!($70)) {
  $73 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $71 = HEAP32[$5>>2]|0;
 $72 = (_vdbePmaReaderNext($71)|0);
 HEAP32[$rc>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _vdbePmaReaderSeek($pTask,$pReadr,$pFile,$0,$1) {
 $pTask = $pTask|0;
 $pReadr = $pReadr|0;
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iBuf = 0, $nRead = 0, $or$cond = 0, $pgsz = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp;
 $rc = sp + 20|0;
 $pgsz = sp + 16|0;
 $iBuf = sp + 12|0;
 $nRead = sp + 8|0;
 HEAP32[$3>>2] = $pTask;
 HEAP32[$4>>2] = $pReadr;
 HEAP32[$5>>2] = $pFile;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $11 = (_sqlite3FaultSim(201)|0);
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$2>>2] = 266;
  $166 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($166|0);
 }
 $13 = HEAP32[$4>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$4>>2]|0;
  $18 = ((($17)) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = ((($20)) + 44|0);
  $22 = HEAP32[$21>>2]|0;
  (_sqlite3OsUnfetch($19,0,0,$22)|0);
  $23 = HEAP32[$4>>2]|0;
  $24 = ((($23)) + 44|0);
  HEAP32[$24>>2] = 0;
 }
 $25 = $6;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$4>>2]|0;
 $32 = $31;
 $33 = $32;
 HEAP32[$33>>2] = $27;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $30;
 $36 = HEAP32[$5>>2]|0;
 $37 = ((($36)) + 8|0);
 $38 = $37;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$4>>2]|0;
 $45 = ((($44)) + 8|0);
 $46 = $45;
 $47 = $46;
 HEAP32[$47>>2] = $40;
 $48 = (($46) + 4)|0;
 $49 = $48;
 HEAP32[$49>>2] = $43;
 $50 = HEAP32[$5>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$4>>2]|0;
 $53 = ((($52)) + 24|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$5>>2]|0;
 $56 = HEAP32[$4>>2]|0;
 $57 = ((($56)) + 44|0);
 $58 = (_vdbeSorterMapFile($54,$55,$57)|0);
 HEAP32[$rc>>2] = $58;
 $59 = HEAP32[$rc>>2]|0;
 $60 = ($59|0)==(0);
 if ($60) {
  $61 = HEAP32[$4>>2]|0;
  $62 = ((($61)) + 44|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)==(0|0);
  if ($64) {
   $65 = HEAP32[$3>>2]|0;
   $66 = ((($65)) + 8|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = ((($67)) + 12|0);
   $69 = HEAP32[$68>>2]|0;
   HEAP32[$pgsz>>2] = $69;
   $70 = HEAP32[$4>>2]|0;
   $71 = $70;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($71) + 4)|0;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = HEAP32[$pgsz>>2]|0;
   $78 = ($77|0)<(0);
   $79 = $78 << 31 >> 31;
   $80 = (___remdi3(($73|0),($76|0),($77|0),($79|0))|0);
   $81 = tempRet0;
   HEAP32[$iBuf>>2] = $80;
   $82 = HEAP32[$4>>2]|0;
   $83 = ((($82)) + 36|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)==(0|0);
   if ($85) {
    $86 = HEAP32[$pgsz>>2]|0;
    $87 = ($86|0)<(0);
    $88 = $87 << 31 >> 31;
    $89 = (_sqlite3Malloc($86,$88)|0);
    $90 = HEAP32[$4>>2]|0;
    $91 = ((($90)) + 36|0);
    HEAP32[$91>>2] = $89;
    $92 = HEAP32[$4>>2]|0;
    $93 = ((($92)) + 36|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==(0|0);
    if ($95) {
     HEAP32[$rc>>2] = 7;
    }
    $96 = HEAP32[$pgsz>>2]|0;
    $97 = HEAP32[$4>>2]|0;
    $98 = ((($97)) + 40|0);
    HEAP32[$98>>2] = $96;
   }
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)==(0);
   $101 = HEAP32[$iBuf>>2]|0;
   $102 = ($101|0)!=(0);
   $or$cond = $100 & $102;
   if ($or$cond) {
    $103 = HEAP32[$pgsz>>2]|0;
    $104 = HEAP32[$iBuf>>2]|0;
    $105 = (($103) - ($104))|0;
    HEAP32[$nRead>>2] = $105;
    $106 = HEAP32[$4>>2]|0;
    $107 = $106;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$nRead>>2]|0;
    $114 = ($113|0)<(0);
    $115 = $114 << 31 >> 31;
    $116 = (_i64Add(($109|0),($112|0),($113|0),($115|0))|0);
    $117 = tempRet0;
    $118 = HEAP32[$4>>2]|0;
    $119 = ((($118)) + 8|0);
    $120 = $119;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = (($120) + 4)|0;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = ($117|0)>($125|0);
    $127 = ($116>>>0)>($122>>>0);
    $128 = ($117|0)==($125|0);
    $129 = $128 & $127;
    $130 = $126 | $129;
    if ($130) {
     $131 = HEAP32[$4>>2]|0;
     $132 = ((($131)) + 8|0);
     $133 = $132;
     $134 = $133;
     $135 = HEAP32[$134>>2]|0;
     $136 = (($133) + 4)|0;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = HEAP32[$4>>2]|0;
     $140 = $139;
     $141 = $140;
     $142 = HEAP32[$141>>2]|0;
     $143 = (($140) + 4)|0;
     $144 = $143;
     $145 = HEAP32[$144>>2]|0;
     $146 = (_i64Subtract(($135|0),($138|0),($142|0),($145|0))|0);
     $147 = tempRet0;
     HEAP32[$nRead>>2] = $146;
    }
    $148 = HEAP32[$4>>2]|0;
    $149 = ((($148)) + 24|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = HEAP32[$iBuf>>2]|0;
    $152 = HEAP32[$4>>2]|0;
    $153 = ((($152)) + 36|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = (($154) + ($151)|0);
    $156 = HEAP32[$nRead>>2]|0;
    $157 = HEAP32[$4>>2]|0;
    $158 = $157;
    $159 = $158;
    $160 = HEAP32[$159>>2]|0;
    $161 = (($158) + 4)|0;
    $162 = $161;
    $163 = HEAP32[$162>>2]|0;
    $164 = (_sqlite3OsRead($150,$155,$156,$160,$163)|0);
    HEAP32[$rc>>2] = $164;
   }
  }
 }
 $165 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $165;
 $166 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($166|0);
}
function _vdbePmaReadVarint($p,$pnOut) {
 $p = $p|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $a = 0, $aVarint = 0, $i = 0, $iBuf = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iBuf = sp + 12|0;
 $aVarint = sp + 32|0;
 $a = sp + 8|0;
 $i = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pnOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = $7;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 do {
  if ($6) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 44|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($16) + ($10)|0);
   $18 = HEAP32[$2>>2]|0;
   $19 = (_sqlite3GetVarint($17,$18)|0);
   $20 = $19&255;
   $21 = HEAP32[$1>>2]|0;
   $22 = $21;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = (($22) + 4)|0;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (_i64Add(($24|0),($27|0),($20|0),0)|0);
   $29 = tempRet0;
   $30 = $21;
   $31 = $30;
   HEAP32[$31>>2] = $28;
   $32 = (($30) + 4)|0;
   $33 = $32;
   HEAP32[$33>>2] = $29;
  } else {
   $34 = HEAP32[$1>>2]|0;
   $35 = ((($34)) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 << 31 >> 31;
   $39 = (___remdi3(($10|0),($13|0),($36|0),($38|0))|0);
   $40 = tempRet0;
   HEAP32[$iBuf>>2] = $39;
   $41 = HEAP32[$iBuf>>2]|0;
   $42 = ($41|0)!=(0);
   if ($42) {
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 40|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$iBuf>>2]|0;
    $47 = (($45) - ($46))|0;
    $48 = ($47|0)>=(9);
    if ($48) {
     $49 = HEAP32[$iBuf>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = ((($50)) + 36|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = (($52) + ($49)|0);
     $54 = HEAP32[$2>>2]|0;
     $55 = (_sqlite3GetVarint($53,$54)|0);
     $56 = $55&255;
     $57 = HEAP32[$1>>2]|0;
     $58 = $57;
     $59 = $58;
     $60 = HEAP32[$59>>2]|0;
     $61 = (($58) + 4)|0;
     $62 = $61;
     $63 = HEAP32[$62>>2]|0;
     $64 = (_i64Add(($60|0),($63|0),($56|0),0)|0);
     $65 = tempRet0;
     $66 = $57;
     $67 = $66;
     HEAP32[$67>>2] = $64;
     $68 = (($66) + 4)|0;
     $69 = $68;
     HEAP32[$69>>2] = $65;
     break;
    }
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $70 = HEAP32[$1>>2]|0;
    $71 = (_vdbePmaReadBlob($70,1,$a)|0);
    HEAP32[$rc>>2] = $71;
    $72 = HEAP32[$rc>>2]|0;
    $73 = ($72|0)!=(0);
    if ($73) {
     break;
    }
    $75 = HEAP32[$a>>2]|0;
    $76 = HEAP8[$75>>0]|0;
    $77 = HEAP32[$i>>2]|0;
    $78 = (($77) + 1)|0;
    HEAP32[$i>>2] = $78;
    $79 = $77 & 15;
    $80 = (($aVarint) + ($79)|0);
    HEAP8[$80>>0] = $76;
    $81 = HEAP32[$a>>2]|0;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $83 & 128;
    $85 = ($84|0)!=(0);
    if (!($85)) {
     label = 10;
     break;
    }
   }
   if ((label|0) == 10) {
    $86 = HEAP32[$2>>2]|0;
    (_sqlite3GetVarint($aVarint,$86)|0);
    break;
   }
   $74 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $74;
   $87 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($87|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $87 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($87|0);
}
function _vdbePmaReaderNext($pReadr) {
 $pReadr = $pReadr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $bEof = 0, $nRec = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $nRec = sp;
 $pIncr = sp + 12|0;
 $bEof = sp + 8|0;
 HEAP32[$1>>2] = $pReadr;
 HEAP32[$rc>>2] = 0;
 $2 = $nRec;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($12|0)>($20|0);
 $22 = ($9>>>0)>=($17>>>0);
 $23 = ($12|0)==($20|0);
 $24 = $23 & $22;
 $25 = $21 | $24;
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pIncr>>2] = $28;
  HEAP32[$bEof>>2] = 1;
  $29 = HEAP32[$pIncr>>2]|0;
  $30 = ($29|0)!=(0|0);
  if ($30) {
   $31 = HEAP32[$pIncr>>2]|0;
   $32 = (_vdbeIncrSwap($31)|0);
   HEAP32[$rc>>2] = $32;
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$pIncr>>2]|0;
    $36 = ((($35)) + 20|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0);
    if ($38) {
     $39 = HEAP32[$pIncr>>2]|0;
     $40 = HEAP32[$39>>2]|0;
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$pIncr>>2]|0;
     $43 = ((($42)) + 32|0);
     $44 = HEAP32[$pIncr>>2]|0;
     $45 = ((($44)) + 8|0);
     $46 = $45;
     $47 = $46;
     $48 = HEAP32[$47>>2]|0;
     $49 = (($46) + 4)|0;
     $50 = $49;
     $51 = HEAP32[$50>>2]|0;
     $52 = (_vdbePmaReaderSeek($40,$41,$43,$48,$51)|0);
     HEAP32[$rc>>2] = $52;
     HEAP32[$bEof>>2] = 0;
    }
   }
  }
  $53 = HEAP32[$bEof>>2]|0;
  $54 = ($53|0)!=(0);
  if ($54) {
   $55 = HEAP32[$1>>2]|0;
   _vdbePmaReaderClear($55);
   $56 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $56;
   $82 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($82|0);
  }
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$1>>2]|0;
  $60 = (_vdbePmaReadVarint($59,$nRec)|0);
  HEAP32[$rc>>2] = $60;
 }
 $61 = HEAP32[$rc>>2]|0;
 $62 = ($61|0)==(0);
 if ($62) {
  $63 = $nRec;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($63) + 4)|0;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 20|0);
  HEAP32[$70>>2] = $65;
  $71 = HEAP32[$1>>2]|0;
  $72 = $nRec;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 32|0);
  $80 = (_vdbePmaReadBlob($71,$74,$79)|0);
  HEAP32[$rc>>2] = $80;
 }
 $81 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $81;
 $82 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($82|0);
}
function _vdbeSorterMapFile($pTask,$pFile,$pp) {
 $pTask = $pTask|0;
 $pFile = $pFile|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pFd = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pFd = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 136|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)<(0);
 $19 = $18 << 31 >> 31;
 $20 = ($10|0)<($19|0);
 $21 = ($7>>>0)<=($17>>>0);
 $22 = ($10|0)==($19|0);
 $23 = $22 & $21;
 $24 = $20 | $23;
 if (!($24)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pFd>>2] = $26;
 $27 = HEAP32[$pFd>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>=(3);
 if (!($30)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $31 = HEAP32[$pFd>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = $33;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (_sqlite3OsFetch($31,0,0,$36,$40)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _vdbePmaReadBlob($p,$nByte,$ppOut) {
 $p = $p|0;
 $nByte = $nByte|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $aNext = 0, $iBuf = 0, $nAvail = 0;
 var $nCopy = 0, $nNew = 0, $nRead = 0, $nRem = 0, $rc = 0, $rc1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $iBuf = sp + 36|0;
 $nAvail = sp + 32|0;
 $nRead = sp + 28|0;
 $rc = sp + 24|0;
 $nRem = sp + 20|0;
 $aNew = sp + 16|0;
 $nNew = sp + 12|0;
 $rc1 = sp + 8|0;
 $nCopy = sp + 4|0;
 $aNext = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $ppOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 if ($7) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 44|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($11)|0);
  $19 = HEAP32[$3>>2]|0;
  HEAP32[$19>>2] = $18;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = HEAP32[$1>>2]|0;
  $24 = $23;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($24) + 4)|0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (_i64Add(($26|0),($29|0),($20|0),($22|0))|0);
  $31 = tempRet0;
  $32 = $23;
  $33 = $32;
  HEAP32[$33>>2] = $30;
  $34 = (($32) + 4)|0;
  $35 = $34;
  HEAP32[$35>>2] = $31;
  HEAP32[$0>>2] = 0;
  $235 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($235|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)<(0);
 $40 = $39 << 31 >> 31;
 $41 = (___remdi3(($11|0),($14|0),($38|0),($40|0))|0);
 $42 = tempRet0;
 HEAP32[$iBuf>>2] = $41;
 $43 = HEAP32[$iBuf>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 8|0);
  $47 = $46;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = $53;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (_i64Subtract(($49|0),($52|0),($56|0),($59|0))|0);
  $61 = tempRet0;
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 40|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($64|0)<(0);
  $66 = $65 << 31 >> 31;
  $67 = ($61|0)>($66|0);
  $68 = ($60>>>0)>($64>>>0);
  $69 = ($61|0)==($66|0);
  $70 = $69 & $68;
  $71 = $67 | $70;
  $72 = HEAP32[$1>>2]|0;
  if ($71) {
   $73 = ((($72)) + 40|0);
   $74 = HEAP32[$73>>2]|0;
   HEAP32[$nRead>>2] = $74;
  } else {
   $75 = ((($72)) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = $82;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (($83) + 4)|0;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = (_i64Subtract(($78|0),($81|0),($85|0),($88|0))|0);
   $90 = tempRet0;
   HEAP32[$nRead>>2] = $89;
  }
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 24|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$1>>2]|0;
  $95 = ((($94)) + 36|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = HEAP32[$nRead>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = $98;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (($99) + 4)|0;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (_sqlite3OsRead($93,$96,$97,$101,$104)|0);
  HEAP32[$rc>>2] = $105;
  $106 = HEAP32[$rc>>2]|0;
  $107 = ($106|0)!=(0);
  if ($107) {
   $108 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $108;
   $235 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($235|0);
  }
 }
 $109 = HEAP32[$1>>2]|0;
 $110 = ((($109)) + 40|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = HEAP32[$iBuf>>2]|0;
 $113 = (($111) - ($112))|0;
 HEAP32[$nAvail>>2] = $113;
 $114 = HEAP32[$2>>2]|0;
 $115 = HEAP32[$nAvail>>2]|0;
 $116 = ($114|0)<=($115|0);
 do {
  if ($116) {
   $117 = HEAP32[$iBuf>>2]|0;
   $118 = HEAP32[$1>>2]|0;
   $119 = ((($118)) + 36|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (($120) + ($117)|0);
   $122 = HEAP32[$3>>2]|0;
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[$2>>2]|0;
   $124 = ($123|0)<(0);
   $125 = $124 << 31 >> 31;
   $126 = HEAP32[$1>>2]|0;
   $127 = $126;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   $133 = (_i64Add(($129|0),($132|0),($123|0),($125|0))|0);
   $134 = tempRet0;
   $135 = $126;
   $136 = $135;
   HEAP32[$136>>2] = $133;
   $137 = (($135) + 4)|0;
   $138 = $137;
   HEAP32[$138>>2] = $134;
  } else {
   $139 = HEAP32[$1>>2]|0;
   $140 = ((($139)) + 16|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = HEAP32[$2>>2]|0;
   $143 = ($141|0)<($142|0);
   do {
    if ($143) {
     $144 = HEAP32[$1>>2]|0;
     $145 = ((($144)) + 16|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = $146<<1;
     $148 = (128)>($147|0);
     if ($148) {
      $153 = 128;
     } else {
      $149 = HEAP32[$1>>2]|0;
      $150 = ((($149)) + 16|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = $151<<1;
      $153 = $152;
     }
     HEAP32[$nNew>>2] = $153;
     while(1) {
      $154 = HEAP32[$2>>2]|0;
      $155 = HEAP32[$nNew>>2]|0;
      $156 = ($154|0)>($155|0);
      if (!($156)) {
       break;
      }
      $157 = HEAP32[$nNew>>2]|0;
      $158 = $157<<1;
      HEAP32[$nNew>>2] = $158;
     }
     $159 = HEAP32[$1>>2]|0;
     $160 = ((($159)) + 28|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$nNew>>2]|0;
     $163 = ($162|0)<(0);
     $164 = $163 << 31 >> 31;
     $165 = (_sqlite3Realloc($161,$162,$164)|0);
     HEAP32[$aNew>>2] = $165;
     $166 = HEAP32[$aNew>>2]|0;
     $167 = ($166|0)!=(0|0);
     if ($167) {
      $168 = HEAP32[$nNew>>2]|0;
      $169 = HEAP32[$1>>2]|0;
      $170 = ((($169)) + 16|0);
      HEAP32[$170>>2] = $168;
      $171 = HEAP32[$aNew>>2]|0;
      $172 = HEAP32[$1>>2]|0;
      $173 = ((($172)) + 28|0);
      HEAP32[$173>>2] = $171;
      break;
     }
     HEAP32[$0>>2] = 7;
     $235 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($235|0);
    }
   } while(0);
   $174 = HEAP32[$1>>2]|0;
   $175 = ((($174)) + 28|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = HEAP32[$iBuf>>2]|0;
   $178 = HEAP32[$1>>2]|0;
   $179 = ((($178)) + 36|0);
   $180 = HEAP32[$179>>2]|0;
   $181 = (($180) + ($177)|0);
   $182 = HEAP32[$nAvail>>2]|0;
   _memcpy(($176|0),($181|0),($182|0))|0;
   $183 = HEAP32[$nAvail>>2]|0;
   $184 = ($183|0)<(0);
   $185 = $184 << 31 >> 31;
   $186 = HEAP32[$1>>2]|0;
   $187 = $186;
   $188 = $187;
   $189 = HEAP32[$188>>2]|0;
   $190 = (($187) + 4)|0;
   $191 = $190;
   $192 = HEAP32[$191>>2]|0;
   $193 = (_i64Add(($189|0),($192|0),($183|0),($185|0))|0);
   $194 = tempRet0;
   $195 = $186;
   $196 = $195;
   HEAP32[$196>>2] = $193;
   $197 = (($195) + 4)|0;
   $198 = $197;
   HEAP32[$198>>2] = $194;
   $199 = HEAP32[$2>>2]|0;
   $200 = HEAP32[$nAvail>>2]|0;
   $201 = (($199) - ($200))|0;
   HEAP32[$nRem>>2] = $201;
   while(1) {
    $202 = HEAP32[$nRem>>2]|0;
    $203 = ($202|0)>(0);
    if (!($203)) {
     label = 27;
     break;
    }
    $204 = HEAP32[$nRem>>2]|0;
    HEAP32[$nCopy>>2] = $204;
    $205 = HEAP32[$nRem>>2]|0;
    $206 = HEAP32[$1>>2]|0;
    $207 = ((($206)) + 40|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = ($205|0)>($208|0);
    if ($209) {
     $210 = HEAP32[$1>>2]|0;
     $211 = ((($210)) + 40|0);
     $212 = HEAP32[$211>>2]|0;
     HEAP32[$nCopy>>2] = $212;
    }
    $213 = HEAP32[$1>>2]|0;
    $214 = HEAP32[$nCopy>>2]|0;
    $215 = (_vdbePmaReadBlob($213,$214,$aNext)|0);
    HEAP32[$rc1>>2] = $215;
    $216 = HEAP32[$rc1>>2]|0;
    $217 = ($216|0)!=(0);
    if ($217) {
     break;
    }
    $219 = HEAP32[$2>>2]|0;
    $220 = HEAP32[$nRem>>2]|0;
    $221 = (($219) - ($220))|0;
    $222 = HEAP32[$1>>2]|0;
    $223 = ((($222)) + 28|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = (($224) + ($221)|0);
    $226 = HEAP32[$aNext>>2]|0;
    $227 = HEAP32[$nCopy>>2]|0;
    _memcpy(($225|0),($226|0),($227|0))|0;
    $228 = HEAP32[$nCopy>>2]|0;
    $229 = HEAP32[$nRem>>2]|0;
    $230 = (($229) - ($228))|0;
    HEAP32[$nRem>>2] = $230;
   }
   if ((label|0) == 27) {
    $231 = HEAP32[$1>>2]|0;
    $232 = ((($231)) + 28|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = HEAP32[$3>>2]|0;
    HEAP32[$234>>2] = $233;
    break;
   }
   $218 = HEAP32[$rc1>>2]|0;
   HEAP32[$0>>2] = $218;
   $235 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($235|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $235 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($235|0);
}
function _vdbeIncrSwap($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pIncr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_vdbeIncrPopulate($1)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 32|0);
 $7 = ((($6)) + 16|0);
 ;HEAP32[$4>>2]=HEAP32[$7>>2]|0;HEAP32[$4+4>>2]=HEAP32[$7+4>>2]|0;HEAP32[$4+8>>2]=HEAP32[$7+8>>2]|0;HEAP32[$4+12>>2]=HEAP32[$7+12>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = ((($9)) + 8|0);
 $11 = $10;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($13|0)==($21|0);
 $26 = ($16|0)==($24|0);
 $27 = $25 & $26;
 if (!($27)) {
  $30 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = 1;
 $30 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _vdbeIncrPopulate($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dummy = 0, $iEof = 0, $iStart = 0, $nKey = 0, $pMerger = 0, $pOut = 0, $pReader = 0, $pTask = 0, $rc = 0, $rc2 = 0, $writer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $rc = sp + 84|0;
 $rc2 = sp + 80|0;
 $iStart = sp + 48|0;
 $pOut = sp + 76|0;
 $pTask = sp + 72|0;
 $pMerger = sp + 68|0;
 $writer = sp + 8|0;
 $dummy = sp + 64|0;
 $pReader = sp + 60|0;
 $nKey = sp + 56|0;
 $iEof = sp;
 HEAP32[$0>>2] = $pIncr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = $2;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = $iStart;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $8;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 32|0);
 $15 = ((($14)) + 16|0);
 HEAP32[$pOut>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pTask>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pMerger>>2] = $20;
 $21 = HEAP32[$pOut>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pTask>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $iStart;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (($28) + 4)|0;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 _vdbePmaWriterInit($22,$writer,$27,$30,$33);
 while(1) {
  $34 = HEAP32[$rc>>2]|0;
  $35 = ($34|0)==(0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$pMerger>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pMerger>>2]|0;
  $42 = ((($41)) + 12|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + (($40*56)|0)|0);
  HEAP32[$pReader>>2] = $44;
  $45 = HEAP32[$pReader>>2]|0;
  $46 = ((($45)) + 20|0);
  $47 = HEAP32[$46>>2]|0;
  HEAP32[$nKey>>2] = $47;
  $48 = ((($writer)) + 24|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = ((($writer)) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  $59 = (_i64Add(($51|0),($54|0),($56|0),($58|0))|0);
  $60 = tempRet0;
  $61 = $iEof;
  $62 = $61;
  HEAP32[$62>>2] = $59;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = $60;
  $65 = HEAP32[$pReader>>2]|0;
  $66 = ((($65)) + 24|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)==(0|0);
  if ($68) {
   break;
  }
  $69 = $iEof;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = HEAP32[$nKey>>2]|0;
  $76 = ($75|0)<(0);
  $77 = $76 << 31 >> 31;
  $78 = (_i64Add(($71|0),($74|0),($75|0),($77|0))|0);
  $79 = tempRet0;
  $80 = HEAP32[$nKey>>2]|0;
  $81 = ($80|0)<(0);
  $82 = $81 << 31 >> 31;
  $83 = (_sqlite3VarintLen($80,$82)|0);
  $84 = ($83|0)<(0);
  $85 = $84 << 31 >> 31;
  $86 = (_i64Add(($78|0),($79|0),($83|0),($85|0))|0);
  $87 = tempRet0;
  $88 = $iStart;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$0>>2]|0;
  $95 = ((($94)) + 16|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)<(0);
  $98 = $97 << 31 >> 31;
  $99 = (_i64Add(($90|0),($93|0),($96|0),($98|0))|0);
  $100 = tempRet0;
  $101 = ($87|0)>($100|0);
  $102 = ($86>>>0)>($99>>>0);
  $103 = ($87|0)==($100|0);
  $104 = $103 & $102;
  $105 = $101 | $104;
  if ($105) {
   break;
  }
  $106 = HEAP32[$nKey>>2]|0;
  $107 = ($106|0)<(0);
  $108 = $107 << 31 >> 31;
  _vdbePmaWriteVarint($writer,$106,$108);
  $109 = HEAP32[$pReader>>2]|0;
  $110 = ((($109)) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = HEAP32[$nKey>>2]|0;
  _vdbePmaWriteBlob($writer,$111,$112);
  $113 = HEAP32[$0>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (_vdbeMergeEngineStep($115,$dummy)|0);
  HEAP32[$rc>>2] = $116;
 }
 $117 = HEAP32[$pOut>>2]|0;
 $118 = ((($117)) + 8|0);
 $119 = (_vdbePmaWriterFinish($writer,$118)|0);
 HEAP32[$rc2>>2] = $119;
 $120 = HEAP32[$rc>>2]|0;
 $121 = ($120|0)==(0);
 if (!($121)) {
  $123 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $122 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $122;
 $123 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _vdbeMergeEngineStep($pMerger,$pbEof) {
 $pMerger = $pMerger|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bCached = 0, $i = 0, $iPrev = 0, $iRes = 0, $pReadr1 = 0, $pReadr2 = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $rc = sp + 28|0;
 $iPrev = sp + 24|0;
 $pTask = sp + 20|0;
 $i = sp + 16|0;
 $pReadr1 = sp + 12|0;
 $pReadr2 = sp + 8|0;
 $bCached = sp + 4|0;
 $iRes = sp;
 HEAP32[$0>>2] = $pMerger;
 HEAP32[$1>>2] = $pbEof;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iPrev>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTask>>2] = $9;
 $10 = HEAP32[$iPrev>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*56)|0)|0);
 $15 = (_vdbePmaReaderNext($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  HEAP32[$bCached>>2] = 0;
  $18 = HEAP32[$iPrev>>2]|0;
  $19 = $18 & 65534;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + (($19*56)|0)|0);
  HEAP32[$pReadr1>>2] = $23;
  $24 = HEAP32[$iPrev>>2]|0;
  $25 = $24 | 1;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*56)|0)|0);
  HEAP32[$pReadr2>>2] = $29;
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$iPrev>>2]|0;
  $33 = (($31) + ($32))|0;
  $34 = (($33|0) / 2)&-1;
  HEAP32[$i>>2] = $34;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = ($35|0)>(0);
   if (!($36)) {
    break;
   }
   $37 = HEAP32[$pReadr1>>2]|0;
   $38 = ((($37)) + 24|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)==(0|0);
   do {
    if ($40) {
     HEAP32[$iRes>>2] = 1;
    } else {
     $41 = HEAP32[$pReadr2>>2]|0;
     $42 = ((($41)) + 24|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = ($43|0)==(0|0);
     if ($44) {
      HEAP32[$iRes>>2] = -1;
      break;
     } else {
      $45 = HEAP32[$pTask>>2]|0;
      $46 = ((($45)) + 32|0);
      $47 = HEAP32[$46>>2]|0;
      $48 = HEAP32[$pTask>>2]|0;
      $49 = HEAP32[$pReadr1>>2]|0;
      $50 = ((($49)) + 32|0);
      $51 = HEAP32[$50>>2]|0;
      $52 = HEAP32[$pReadr1>>2]|0;
      $53 = ((($52)) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = HEAP32[$pReadr2>>2]|0;
      $56 = ((($55)) + 32|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$pReadr2>>2]|0;
      $59 = ((($58)) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = (FUNCTION_TABLE_iiiiiii[$47 & 255]($48,$bCached,$51,$54,$57,$60)|0);
      HEAP32[$iRes>>2] = $61;
      break;
     }
    }
   } while(0);
   $62 = HEAP32[$iRes>>2]|0;
   $63 = ($62|0)<(0);
   do {
    if ($63) {
     label = 12;
    } else {
     $64 = HEAP32[$iRes>>2]|0;
     $65 = ($64|0)==(0);
     if ($65) {
      $66 = HEAP32[$pReadr1>>2]|0;
      $67 = HEAP32[$pReadr2>>2]|0;
      $68 = ($66>>>0)<($67>>>0);
      if ($68) {
       label = 12;
       break;
      }
     }
     $93 = HEAP32[$pReadr1>>2]|0;
     $94 = ((($93)) + 24|0);
     $95 = HEAP32[$94>>2]|0;
     $96 = ($95|0)!=(0|0);
     if ($96) {
      HEAP32[$bCached>>2] = 0;
     }
     $97 = HEAP32[$pReadr2>>2]|0;
     $98 = HEAP32[$0>>2]|0;
     $99 = ((($98)) + 12|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = $97;
     $102 = $100;
     $103 = (($101) - ($102))|0;
     $104 = (($103|0) / 56)&-1;
     $105 = HEAP32[$i>>2]|0;
     $106 = HEAP32[$0>>2]|0;
     $107 = ((($106)) + 8|0);
     $108 = HEAP32[$107>>2]|0;
     $109 = (($108) + ($105<<2)|0);
     HEAP32[$109>>2] = $104;
     $110 = HEAP32[$i>>2]|0;
     $111 = $110 ^ 1;
     $112 = HEAP32[$0>>2]|0;
     $113 = ((($112)) + 8|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + ($111<<2)|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$0>>2]|0;
     $118 = ((($117)) + 12|0);
     $119 = HEAP32[$118>>2]|0;
     $120 = (($119) + (($116*56)|0)|0);
     HEAP32[$pReadr1>>2] = $120;
    }
   } while(0);
   if ((label|0) == 12) {
    label = 0;
    $69 = HEAP32[$pReadr1>>2]|0;
    $70 = HEAP32[$0>>2]|0;
    $71 = ((($70)) + 12|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $69;
    $74 = $72;
    $75 = (($73) - ($74))|0;
    $76 = (($75|0) / 56)&-1;
    $77 = HEAP32[$i>>2]|0;
    $78 = HEAP32[$0>>2]|0;
    $79 = ((($78)) + 8|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (($80) + ($77<<2)|0);
    HEAP32[$81>>2] = $76;
    $82 = HEAP32[$i>>2]|0;
    $83 = $82 ^ 1;
    $84 = HEAP32[$0>>2]|0;
    $85 = ((($84)) + 8|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = (($86) + ($83<<2)|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = HEAP32[$0>>2]|0;
    $90 = ((($89)) + 12|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = (($91) + (($88*56)|0)|0);
    HEAP32[$pReadr2>>2] = $92;
    HEAP32[$bCached>>2] = 0;
   }
   $121 = HEAP32[$i>>2]|0;
   $122 = (($121|0) / 2)&-1;
   HEAP32[$i>>2] = $122;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = ((($123)) + 8|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = ((($125)) + 4|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$0>>2]|0;
  $129 = ((($128)) + 12|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = (($130) + (($127*56)|0)|0);
  $132 = ((($131)) + 24|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = ($133|0)==(0|0);
  $135 = $134&1;
  $136 = HEAP32[$1>>2]|0;
  HEAP32[$136>>2] = $135;
 }
 $137 = HEAP32[$rc>>2]|0;
 $138 = ($137|0)==(0);
 if ($138) {
  $139 = HEAP32[$pTask>>2]|0;
  $140 = ((($139)) + 12|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ((($141)) + 11|0);
  $143 = HEAP8[$142>>0]|0;
  $144 = $143&255;
  $146 = $144;
  STACKTOP = sp;return ($146|0);
 } else {
  $145 = HEAP32[$rc>>2]|0;
  $146 = $145;
  STACKTOP = sp;return ($146|0);
 }
 return (0)|0;
}
function _vdbeIncrMergerNew($pTask,$pMerger,$ppOut) {
 $pTask = $pTask|0;
 $pMerger = $pMerger|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pIncr = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pMerger;
 HEAP32[$2>>2] = $ppOut;
 HEAP32[$rc>>2] = 0;
 $3 = (_sqlite3FaultSim(100)|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  $7 = 0;
 } else {
  $5 = (_sqlite3MallocZero(64,0)|0);
  $7 = $5;
 }
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$6>>2] = $7;
 HEAP32[$pIncr>>2] = $7;
 $8 = HEAP32[$pIncr>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$1>>2]|0;
 if (!($9)) {
  _vdbeMergeEngineFree($10);
  HEAP32[$rc>>2] = 7;
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $11 = HEAP32[$pIncr>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pIncr>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 9)|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25|0) / 2)&-1;
 $27 = ($20|0)>($26|0);
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 if ($27) {
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 9)|0;
  $39 = $33;
 } else {
  $34 = ((($30)) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35|0) / 2)&-1;
  $39 = $36;
 }
 $37 = HEAP32[$pIncr>>2]|0;
 $38 = ((($37)) + 16|0);
 HEAP32[$38>>2] = $39;
 $40 = HEAP32[$pIncr>>2]|0;
 $41 = ((($40)) + 16|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 56|0);
 $47 = ((($46)) + 8|0);
 $48 = $47;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (_i64Add(($50|0),($53|0),($42|0),($44|0))|0);
 $55 = tempRet0;
 $56 = $47;
 $57 = $56;
 HEAP32[$57>>2] = $54;
 $58 = (($56) + 4)|0;
 $59 = $58;
 HEAP32[$59>>2] = $55;
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _vdbePmaReaderIncrInit($pReadr,$eMode) {
 $pReadr = $pReadr|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pIncr = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pReadr;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIncr>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pIncr>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_vdbePmaReaderIncrMergeInit($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _vdbeMergeEngineCompare($pMerger,$iOut) {
 $pMerger = $pMerger|0;
 $iOut = $iOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $bCached = 0;
 var $i1 = 0, $i2 = 0, $iRes = 0, $p1 = 0, $p2 = 0, $pTask = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $i1 = sp + 28|0;
 $i2 = sp + 24|0;
 $iRes = sp + 20|0;
 $p1 = sp + 16|0;
 $p2 = sp + 12|0;
 $pTask = sp + 8|0;
 $bCached = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pMerger;
 HEAP32[$1>>2] = $iOut;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4|0) / 2)&-1;
 $6 = ($2|0)>=($5|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9|0) / 2)&-1;
  $11 = (($7) - ($10))|0;
  $12 = $11<<1;
  HEAP32[$i1>>2] = $12;
  $13 = HEAP32[$i1>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i2>>2] = $14;
 } else {
  $15 = $7<<1;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$i1>>2] = $20;
  $21 = HEAP32[$1>>2]|0;
  $22 = $21<<1;
  $23 = (($22) + 1)|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($23<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$i2>>2] = $28;
 }
 $29 = HEAP32[$i1>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 12|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + (($29*56)|0)|0);
 HEAP32[$p1>>2] = $33;
 $34 = HEAP32[$i2>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + (($34*56)|0)|0);
 HEAP32[$p2>>2] = $38;
 $39 = HEAP32[$p1>>2]|0;
 $40 = ((($39)) + 24|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)==(0|0);
 do {
  if ($42) {
   $43 = HEAP32[$i2>>2]|0;
   HEAP32[$iRes>>2] = $43;
  } else {
   $44 = HEAP32[$p2>>2]|0;
   $45 = ((($44)) + 24|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==(0|0);
   if ($47) {
    $48 = HEAP32[$i1>>2]|0;
    HEAP32[$iRes>>2] = $48;
    break;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = ((($49)) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   HEAP32[$pTask>>2] = $51;
   HEAP32[$bCached>>2] = 0;
   $52 = HEAP32[$pTask>>2]|0;
   $53 = ((($52)) + 32|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = HEAP32[$pTask>>2]|0;
   $56 = HEAP32[$p1>>2]|0;
   $57 = ((($56)) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$p1>>2]|0;
   $60 = ((($59)) + 20|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$p2>>2]|0;
   $63 = ((($62)) + 32|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = HEAP32[$p2>>2]|0;
   $66 = ((($65)) + 20|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (FUNCTION_TABLE_iiiiiii[$54 & 255]($55,$bCached,$58,$61,$64,$67)|0);
   HEAP32[$res>>2] = $68;
   $69 = HEAP32[$res>>2]|0;
   $70 = ($69|0)<=(0);
   if ($70) {
    $71 = HEAP32[$i1>>2]|0;
    HEAP32[$iRes>>2] = $71;
    break;
   } else {
    $72 = HEAP32[$i2>>2]|0;
    HEAP32[$iRes>>2] = $72;
    break;
   }
  }
 } while(0);
 $73 = HEAP32[$iRes>>2]|0;
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 8|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = (($77) + ($74<<2)|0);
 HEAP32[$78>>2] = $73;
 STACKTOP = sp;return;
}
function _vdbePmaReaderIncrMergeInit($pReadr,$eMode) {
 $pReadr = $pReadr|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $db = 0, $mxSz = 0, $pIncr = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $pIncr = sp + 12|0;
 $pTask = sp + 8|0;
 $db = sp + 4|0;
 $mxSz = sp;
 HEAP32[$0>>2] = $pReadr;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIncr>>2] = $4;
 $5 = HEAP32[$pIncr>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTask>>2] = $6;
 $7 = HEAP32[$pTask>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$pTask>>2]|0;
 $13 = HEAP32[$pIncr>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_vdbeMergeEngineInit($12,$15,$16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$pIncr>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$mxSz>>2] = $22;
  $23 = HEAP32[$pTask>>2]|0;
  $24 = ((($23)) + 56|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)==(0|0);
  if ($26) {
   $27 = HEAP32[$db>>2]|0;
   $28 = HEAP32[$pTask>>2]|0;
   $29 = ((($28)) + 56|0);
   $30 = ((($29)) + 8|0);
   $31 = $30;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($31) + 4)|0;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pTask>>2]|0;
   $38 = ((($37)) + 56|0);
   $39 = (_vdbeSorterOpenTempFile($27,$33,$36,$38)|0);
   HEAP32[$rc>>2] = $39;
   $40 = HEAP32[$pTask>>2]|0;
   $41 = ((($40)) + 56|0);
   $42 = ((($41)) + 8|0);
   $43 = $42;
   $44 = $43;
   HEAP32[$44>>2] = 0;
   $45 = (($43) + 4)|0;
   $46 = $45;
   HEAP32[$46>>2] = 0;
  }
  $47 = HEAP32[$rc>>2]|0;
  $48 = ($47|0)==(0);
  if ($48) {
   $49 = HEAP32[$pTask>>2]|0;
   $50 = ((($49)) + 56|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$pIncr>>2]|0;
   $53 = ((($52)) + 32|0);
   $54 = ((($53)) + 16|0);
   HEAP32[$54>>2] = $51;
   $55 = HEAP32[$pTask>>2]|0;
   $56 = ((($55)) + 56|0);
   $57 = ((($56)) + 8|0);
   $58 = $57;
   $59 = $58;
   $60 = HEAP32[$59>>2]|0;
   $61 = (($58) + 4)|0;
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$pIncr>>2]|0;
   $65 = ((($64)) + 8|0);
   $66 = $65;
   $67 = $66;
   HEAP32[$67>>2] = $60;
   $68 = (($66) + 4)|0;
   $69 = $68;
   HEAP32[$69>>2] = $63;
   $70 = HEAP32[$mxSz>>2]|0;
   $71 = ($70|0)<(0);
   $72 = $71 << 31 >> 31;
   $73 = HEAP32[$pTask>>2]|0;
   $74 = ((($73)) + 56|0);
   $75 = ((($74)) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (_i64Add(($78|0),($81|0),($70|0),($72|0))|0);
   $83 = tempRet0;
   $84 = $75;
   $85 = $84;
   HEAP32[$85>>2] = $82;
   $86 = (($84) + 4)|0;
   $87 = $86;
   HEAP32[$87>>2] = $83;
  }
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = ($88|0)==(0);
 if (!($89)) {
  $92 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($92|0);
 }
 $90 = HEAP32[$0>>2]|0;
 $91 = (_vdbePmaReaderNext($90)|0);
 HEAP32[$rc>>2] = $91;
 $92 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($92|0);
}
function _sqlite3HeapNearlyFull() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(308)>>2]|0;
 return ($0|0);
}
function _btreeDropTable($p,$iTable,$piMoved) {
 $p = $p|0;
 $iTable = $iTable|0;
 $piMoved = $piMoved|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $maxRootPgno = 0, $pBt = 0, $pMove = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $pPage = sp + 12|0;
 $pBt = sp + 8|0;
 $maxRootPgno = sp + 4|0;
 $pMove = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $piMoved;
 HEAP32[$pPage>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 262;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11>>>0)<(2);
 if ($12) {
  $13 = (_sqlite3CorruptError(64136)|0);
  HEAP32[$0>>2] = $13;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $14 = HEAP32[$pBt>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (_btreeGetPage($14,$15,$pPage,0)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $19;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (_sqlite3BtreeClearTable($20,$21,0)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$pPage>>2]|0;
  _releasePage($25);
  $26 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $26;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $27 = HEAP32[$3>>2]|0;
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$pBt>>2]|0;
 $29 = ((($28)) + 17|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = ($30<<24>>24)!=(0);
 if ($31) {
  $32 = HEAP32[$1>>2]|0;
  _sqlite3BtreeGetMeta($32,4,$maxRootPgno);
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$maxRootPgno>>2]|0;
  $35 = ($33|0)==($34|0);
  $36 = HEAP32[$pPage>>2]|0;
  do {
   if ($35) {
    _freePage($36,$rc);
    $37 = HEAP32[$pPage>>2]|0;
    _releasePage($37);
    $38 = HEAP32[$rc>>2]|0;
    $39 = ($38|0)!=(0);
    if ($39) {
     $40 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $40;
     $88 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($88|0);
    }
   } else {
    _releasePage($36);
    $41 = HEAP32[$pBt>>2]|0;
    $42 = HEAP32[$maxRootPgno>>2]|0;
    $43 = (_btreeGetPage($41,$42,$pMove,0)|0);
    HEAP32[$rc>>2] = $43;
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)!=(0);
    if ($45) {
     $46 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $46;
     $88 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($88|0);
    }
    $47 = HEAP32[$pBt>>2]|0;
    $48 = HEAP32[$pMove>>2]|0;
    $49 = HEAP32[$2>>2]|0;
    $50 = (_relocatePage($47,$48,1,0,$49,0)|0);
    HEAP32[$rc>>2] = $50;
    $51 = HEAP32[$pMove>>2]|0;
    _releasePage($51);
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)!=(0);
    if ($53) {
     $54 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $54;
     $88 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($88|0);
    }
    HEAP32[$pMove>>2] = 0;
    $55 = HEAP32[$pBt>>2]|0;
    $56 = HEAP32[$maxRootPgno>>2]|0;
    $57 = (_btreeGetPage($55,$56,$pMove,0)|0);
    HEAP32[$rc>>2] = $57;
    $58 = HEAP32[$pMove>>2]|0;
    _freePage($58,$rc);
    $59 = HEAP32[$pMove>>2]|0;
    _releasePage($59);
    $60 = HEAP32[$rc>>2]|0;
    $61 = ($60|0)!=(0);
    if (!($61)) {
     $63 = HEAP32[$maxRootPgno>>2]|0;
     $64 = HEAP32[$3>>2]|0;
     HEAP32[$64>>2] = $63;
     break;
    }
    $62 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $62;
    $88 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($88|0);
   }
  } while(0);
  $65 = HEAP32[$maxRootPgno>>2]|0;
  $66 = (($65) + -1)|0;
  HEAP32[$maxRootPgno>>2] = $66;
  while(1) {
   $67 = HEAP32[$maxRootPgno>>2]|0;
   $68 = HEAP32[832>>2]|0;
   $69 = HEAP32[$pBt>>2]|0;
   $70 = ((($69)) + 32|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($68>>>0) / ($71>>>0))&-1;
   $73 = (($72) + 1)|0;
   $74 = ($67|0)==($73|0);
   if (!($74)) {
    $75 = HEAP32[$pBt>>2]|0;
    $76 = HEAP32[$maxRootPgno>>2]|0;
    $77 = (_ptrmapPageno($75,$76)|0);
    $78 = HEAP32[$maxRootPgno>>2]|0;
    $79 = ($77|0)==($78|0);
    if (!($79)) {
     break;
    }
   }
   $80 = HEAP32[$maxRootPgno>>2]|0;
   $81 = (($80) + -1)|0;
   HEAP32[$maxRootPgno>>2] = $81;
  }
  $82 = HEAP32[$1>>2]|0;
  $83 = HEAP32[$maxRootPgno>>2]|0;
  $84 = (_sqlite3BtreeUpdateMeta($82,4,$83)|0);
  HEAP32[$rc>>2] = $84;
 } else {
  $85 = HEAP32[$pPage>>2]|0;
  _freePage($85,$rc);
  $86 = HEAP32[$pPage>>2]|0;
  _releasePage($86);
 }
 $87 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $87;
 $88 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _clearDatabasePage($pBt,$pgno,$freePageFlag,$pnChange) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $freePageFlag = $freePageFlag|0;
 $pnChange = $pnChange|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $hdr = 0, $i = 0, $pCell = 0, $pPage = 0, $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pPage = sp + 16|0;
 $rc = sp + 12|0;
 $pCell = sp + 8|0;
 $i = sp + 4|0;
 $hdr = sp;
 $szCell = sp + 40|0;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $freePageFlag;
 HEAP32[$4>>2] = $pnChange;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_btreePagecount($6)|0);
 $8 = ($5>>>0)>($7>>>0);
 if ($8) {
  $9 = (_sqlite3CorruptError(64010)|0);
  HEAP32[$0>>2] = $9;
  $123 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (_getAndInitPage($10,$11,$pPage,0,0)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $123 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $16 = HEAP32[$pPage>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 L9: do {
  if ($19) {
   $20 = (_sqlite3CorruptError(64015)|0);
   HEAP32[$rc>>2] = $20;
  } else {
   $21 = HEAP32[$pPage>>2]|0;
   $22 = ((($21)) + 8|0);
   HEAP8[$22>>0] = 1;
   $23 = HEAP32[$pPage>>2]|0;
   $24 = ((($23)) + 5|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   HEAP32[$hdr>>2] = $26;
   HEAP32[$i>>2] = 0;
   while(1) {
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$pPage>>2]|0;
    $29 = ((($28)) + 18|0);
    $30 = HEAP16[$29>>1]|0;
    $31 = $30&65535;
    $32 = ($27|0)<($31|0);
    $33 = HEAP32[$pPage>>2]|0;
    if (!($32)) {
     break;
    }
    $34 = ((($33)) + 56|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = HEAP32[$pPage>>2]|0;
    $37 = ((($36)) + 20|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38&65535;
    $40 = HEAP32[$i>>2]|0;
    $41 = $40<<1;
    $42 = HEAP32[$pPage>>2]|0;
    $43 = ((($42)) + 64|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + ($41)|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = $47 << 8;
    $49 = HEAP32[$i>>2]|0;
    $50 = $49<<1;
    $51 = HEAP32[$pPage>>2]|0;
    $52 = ((($51)) + 64|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (($53) + ($50)|0);
    $55 = ((($54)) + 1|0);
    $56 = HEAP8[$55>>0]|0;
    $57 = $56&255;
    $58 = $48 | $57;
    $59 = $39 & $58;
    $60 = (($35) + ($59)|0);
    HEAP32[$pCell>>2] = $60;
    $61 = HEAP32[$pPage>>2]|0;
    $62 = ((($61)) + 4|0);
    $63 = HEAP8[$62>>0]|0;
    $64 = ($63<<24>>24)!=(0);
    if (!($64)) {
     $65 = HEAP32[$1>>2]|0;
     $66 = HEAP32[$pCell>>2]|0;
     $67 = (_sqlite3Get4byte($66)|0);
     $68 = HEAP32[$4>>2]|0;
     $69 = (_clearDatabasePage($65,$67,1,$68)|0);
     HEAP32[$rc>>2] = $69;
     $70 = HEAP32[$rc>>2]|0;
     $71 = ($70|0)!=(0);
     if ($71) {
      break L9;
     }
    }
    $72 = HEAP32[$pPage>>2]|0;
    $73 = HEAP32[$pCell>>2]|0;
    $74 = (_clearCell($72,$73,$szCell)|0);
    HEAP32[$rc>>2] = $74;
    $75 = HEAP32[$rc>>2]|0;
    $76 = ($75|0)!=(0);
    if ($76) {
     break L9;
    }
    $77 = HEAP32[$i>>2]|0;
    $78 = (($77) + 1)|0;
    HEAP32[$i>>2] = $78;
   }
   $79 = ((($33)) + 4|0);
   $80 = HEAP8[$79>>0]|0;
   $81 = ($80<<24>>24)!=(0);
   if ($81) {
    $94 = HEAP32[$4>>2]|0;
    $95 = ($94|0)!=(0|0);
    if ($95) {
     $96 = HEAP32[$pPage>>2]|0;
     $97 = ((($96)) + 18|0);
     $98 = HEAP16[$97>>1]|0;
     $99 = $98&65535;
     $100 = HEAP32[$4>>2]|0;
     $101 = HEAP32[$100>>2]|0;
     $102 = (($101) + ($99))|0;
     HEAP32[$100>>2] = $102;
    }
   } else {
    $82 = HEAP32[$1>>2]|0;
    $83 = HEAP32[$hdr>>2]|0;
    $84 = (($83) + 8)|0;
    $85 = HEAP32[$pPage>>2]|0;
    $86 = ((($85)) + 56|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = (($87) + ($84)|0);
    $89 = (_sqlite3Get4byte($88)|0);
    $90 = HEAP32[$4>>2]|0;
    $91 = (_clearDatabasePage($82,$89,1,$90)|0);
    HEAP32[$rc>>2] = $91;
    $92 = HEAP32[$rc>>2]|0;
    $93 = ($92|0)!=(0);
    if ($93) {
     break;
    }
   }
   $103 = HEAP32[$3>>2]|0;
   $104 = ($103|0)!=(0);
   $105 = HEAP32[$pPage>>2]|0;
   if ($104) {
    _freePage($105,$rc);
    break;
   }
   $106 = ((($105)) + 72|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = (_sqlite3PagerWrite($107)|0);
   HEAP32[$rc>>2] = $108;
   $109 = ($108|0)==(0);
   if ($109) {
    $110 = HEAP32[$pPage>>2]|0;
    $111 = HEAP32[$hdr>>2]|0;
    $112 = HEAP32[$pPage>>2]|0;
    $113 = ((($112)) + 56|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = (($114) + ($111)|0);
    $116 = HEAP8[$115>>0]|0;
    $117 = $116&255;
    $118 = $117 | 8;
    _zeroPage($110,$118);
   }
  }
 } while(0);
 $119 = HEAP32[$pPage>>2]|0;
 $120 = ((($119)) + 8|0);
 HEAP8[$120>>0] = 0;
 $121 = HEAP32[$pPage>>2]|0;
 _releasePage($121);
 $122 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $122;
 $123 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _corruptSchema($pData,$zObj,$zExtra) {
 $pData = $pData|0;
 $zObj = $zObj|0;
 $zExtra = $zExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $db = sp + 20|0;
 $z = sp + 16|0;
 HEAP32[$0>>2] = $pData;
 HEAP32[$1>>2] = $zObj;
 HEAP32[$2>>2] = $zExtra;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 65|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  $9 = HEAP32[$db>>2]|0;
  $10 = ((($9)) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 65536;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ($14|0)==(0|0);
   if ($15) {
    HEAP32[$1>>2] = 21679;
   }
   $16 = HEAP32[$db>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   HEAP32[$vararg_buffer>>2] = $17;
   $18 = (_sqlite3MPrintf($16,21681,$vararg_buffer)|0);
   HEAP32[$z>>2] = $18;
   $19 = HEAP32[$2>>2]|0;
   $20 = ($19|0)!=(0|0);
   if ($20) {
    $21 = HEAP32[$db>>2]|0;
    $22 = HEAP32[$z>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $22;
    $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $23;
    $24 = (_sqlite3MPrintf($21,21712,$vararg_buffer1)|0);
    HEAP32[$z>>2] = $24;
   }
   $25 = HEAP32[$db>>2]|0;
   $26 = HEAP32[$0>>2]|0;
   $27 = ((($26)) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   _sqlite3DbFree($25,$29);
   $30 = HEAP32[$z>>2]|0;
   $31 = HEAP32[$0>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$33>>2] = $30;
  }
 }
 $34 = HEAP32[$db>>2]|0;
 $35 = ((($34)) + 65|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = ($37|0)!=(0);
 if ($38) {
  $42 = 7;
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 12|0);
  HEAP32[$41>>2] = $42;
  STACKTOP = sp;return;
 }
 $39 = (_sqlite3CorruptError(109430)|0);
 $42 = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 12|0);
 HEAP32[$41>>2] = $42;
 STACKTOP = sp;return;
}
function _sqlite3Atoi($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$x>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = HEAP32[$0>>2]|0;
  (_sqlite3GetInt32($3,$x)|0);
 }
 $4 = HEAP32[$x>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3FindIndex($db,$zName,$zDb) {
 $db = $db|0;
 $zName = $zName|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $j = 0, $p = 0, $pSchema = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pSchema = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)<(2);
  $10 = HEAP32[$i>>2]|0;
  $11 = $10 ^ 1;
  $12 = $9 ? $11 : $10;
  HEAP32[$j>>2] = $12;
  $13 = HEAP32[$j>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  $18 = ((($17)) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pSchema>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($23<<4)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_sqlite3_stricmp($22,$28)|0);
   $30 = ($29|0)!=(0);
   if (!($30)) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $31 = HEAP32[$pSchema>>2]|0;
   $32 = ((($31)) + 24|0);
   $33 = HEAP32[$1>>2]|0;
   $34 = (_sqlite3HashFind($32,$33)|0);
   HEAP32[$p>>2] = $34;
   $35 = HEAP32[$p>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    label = 7;
    break;
   }
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 if ((label|0) == 7) {
  $39 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 return (0)|0;
}
function _sqlite3GetInt32($zNum,$pValue) {
 $zNum = $zNum|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $neg = 0, $or$cond = 0, $u = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $v = sp;
 $i = sp + 20|0;
 $c = sp + 16|0;
 $neg = sp + 12|0;
 $u = sp + 8|0;
 HEAP32[$1>>2] = $zNum;
 HEAP32[$2>>2] = $pValue;
 $3 = $v;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 HEAP32[$neg>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)==(45);
 do {
  if ($10) {
   HEAP32[$neg>>2] = 1;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 1|0);
   HEAP32[$1>>2] = $12;
  } else {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)==(43);
   $17 = HEAP32[$1>>2]|0;
   if ($16) {
    $18 = ((($17)) + 1|0);
    HEAP32[$1>>2] = $18;
    break;
   }
   $19 = HEAP8[$17>>0]|0;
   $20 = $19 << 24 >> 24;
   $21 = ($20|0)==(48);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 1|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = ($25|0)==(120);
    if (!($26)) {
     $27 = HEAP32[$1>>2]|0;
     $28 = ((($27)) + 1|0);
     $29 = HEAP8[$28>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(88);
     if (!($31)) {
      break;
     }
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 2|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = (17533 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    $39 = $38 & 8;
    $40 = ($39|0)!=(0);
    if ($40) {
     HEAP32[$u>>2] = 0;
     $41 = HEAP32[$1>>2]|0;
     $42 = ((($41)) + 2|0);
     HEAP32[$1>>2] = $42;
     while(1) {
      $43 = HEAP32[$1>>2]|0;
      $44 = HEAP8[$43>>0]|0;
      $45 = $44 << 24 >> 24;
      $46 = ($45|0)==(48);
      if (!($46)) {
       break;
      }
      $47 = HEAP32[$1>>2]|0;
      $48 = ((($47)) + 1|0);
      HEAP32[$1>>2] = $48;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $49 = HEAP32[$i>>2]|0;
      $50 = HEAP32[$1>>2]|0;
      $51 = (($50) + ($49)|0);
      $52 = HEAP8[$51>>0]|0;
      $53 = $52&255;
      $54 = (17533 + ($53)|0);
      $55 = HEAP8[$54>>0]|0;
      $56 = $55&255;
      $57 = $56 & 8;
      $58 = ($57|0)!=(0);
      $59 = HEAP32[$i>>2]|0;
      $60 = ($59|0)<(8);
      $61 = $58 ? $60 : 0;
      $62 = HEAP32[$u>>2]|0;
      if (!($61)) {
       break;
      }
      $63 = $62<<4;
      $64 = HEAP32[$i>>2]|0;
      $65 = HEAP32[$1>>2]|0;
      $66 = (($65) + ($64)|0);
      $67 = HEAP8[$66>>0]|0;
      $68 = $67 << 24 >> 24;
      $69 = (_sqlite3HexToInt($68)|0);
      $70 = $69&255;
      $71 = (($63) + ($70))|0;
      HEAP32[$u>>2] = $71;
      $72 = HEAP32[$i>>2]|0;
      $73 = (($72) + 1)|0;
      HEAP32[$i>>2] = $73;
     }
     $74 = $62 & -2147483648;
     $75 = ($74|0)==(0);
     if ($75) {
      $76 = HEAP32[$i>>2]|0;
      $77 = HEAP32[$1>>2]|0;
      $78 = (($77) + ($76)|0);
      $79 = HEAP8[$78>>0]|0;
      $80 = $79&255;
      $81 = (17533 + ($80)|0);
      $82 = HEAP8[$81>>0]|0;
      $83 = $82&255;
      $84 = $83 & 8;
      $85 = ($84|0)==(0);
      if ($85) {
       $86 = HEAP32[$2>>2]|0;
       ;HEAP32[$86>>2]=HEAP32[$u>>2]|0;
       HEAP32[$0>>2] = 1;
       $162 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($162|0);
      }
     }
     HEAP32[$0>>2] = 0;
     $162 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($162|0);
    }
   }
  }
 } while(0);
 while(1) {
  $87 = HEAP32[$1>>2]|0;
  $88 = HEAP8[$87>>0]|0;
  $89 = $88 << 24 >> 24;
  $90 = ($89|0)==(48);
  if (!($90)) {
   break;
  }
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 1|0);
  HEAP32[$1>>2] = $92;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $93 = HEAP32[$i>>2]|0;
  $94 = ($93|0)<(11);
  if (!($94)) {
   break;
  }
  $95 = HEAP32[$i>>2]|0;
  $96 = HEAP32[$1>>2]|0;
  $97 = (($96) + ($95)|0);
  $98 = HEAP8[$97>>0]|0;
  $99 = $98 << 24 >> 24;
  $100 = (($99) - 48)|0;
  HEAP32[$c>>2] = $100;
  $101 = ($100|0)>=(0);
  $102 = HEAP32[$c>>2]|0;
  $103 = ($102|0)<=(9);
  $or$cond = $101 & $103;
  if (!($or$cond)) {
   break;
  }
  $104 = $v;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = (___muldi3(($106|0),($109|0),10,0)|0);
  $111 = tempRet0;
  $112 = HEAP32[$c>>2]|0;
  $113 = ($112|0)<(0);
  $114 = $113 << 31 >> 31;
  $115 = (_i64Add(($110|0),($111|0),($112|0),($114|0))|0);
  $116 = tempRet0;
  $117 = $v;
  $118 = $117;
  HEAP32[$118>>2] = $115;
  $119 = (($117) + 4)|0;
  $120 = $119;
  HEAP32[$120>>2] = $116;
  $121 = HEAP32[$i>>2]|0;
  $122 = (($121) + 1)|0;
  HEAP32[$i>>2] = $122;
 }
 $123 = HEAP32[$i>>2]|0;
 $124 = ($123|0)>(10);
 if ($124) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $125 = $v;
 $126 = $125;
 $127 = HEAP32[$126>>2]|0;
 $128 = (($125) + 4)|0;
 $129 = $128;
 $130 = HEAP32[$129>>2]|0;
 $131 = HEAP32[$neg>>2]|0;
 $132 = ($131|0)<(0);
 $133 = $132 << 31 >> 31;
 $134 = (_i64Subtract(($127|0),($130|0),($131|0),($133|0))|0);
 $135 = tempRet0;
 $136 = ($135|0)>(0);
 $137 = ($134>>>0)>(2147483647);
 $138 = ($135|0)==(0);
 $139 = $138 & $137;
 $140 = $136 | $139;
 if ($140) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $141 = HEAP32[$neg>>2]|0;
 $142 = ($141|0)!=(0);
 if ($142) {
  $143 = $v;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (($143) + 4)|0;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (_i64Subtract(0,0,($145|0),($148|0))|0);
  $150 = tempRet0;
  $151 = $v;
  $152 = $151;
  HEAP32[$152>>2] = $149;
  $153 = (($151) + 4)|0;
  $154 = $153;
  HEAP32[$154>>2] = $150;
 }
 $155 = $v;
 $156 = $155;
 $157 = HEAP32[$156>>2]|0;
 $158 = (($155) + 4)|0;
 $159 = $158;
 $160 = HEAP32[$159>>2]|0;
 $161 = HEAP32[$2>>2]|0;
 HEAP32[$161>>2] = $157;
 HEAP32[$0>>2] = 1;
 $162 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _sqlite3HashFind($pH,$pKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $elem = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pKey;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_findElementWithHash($2,$3,$h)|0);
 HEAP32[$elem>>2] = $4;
 $5 = HEAP32[$elem>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = 0;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$elem>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _findElementWithHash($pH,$pKey,$pHash) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $pHash = $pHash|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $count = 0, $elem = 0, $h = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $elem = sp + 12|0;
 $count = sp + 8|0;
 $h = sp + 4|0;
 $pEntry = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $pHash;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  $9 = (_strHash($8)|0);
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9>>>0) % ($11>>>0))&-1;
  HEAP32[$h>>2] = $12;
  $13 = HEAP32[$h>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<3)|0);
  HEAP32[$pEntry>>2] = $17;
  $18 = HEAP32[$pEntry>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$elem>>2] = $20;
  $21 = HEAP32[$pEntry>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$count>>2] = $22;
 } else {
  HEAP32[$h>>2] = 0;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$elem>>2] = $25;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$count>>2] = $28;
 }
 $29 = HEAP32[$h>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 HEAP32[$30>>2] = $29;
 while(1) {
  $31 = HEAP32[$count>>2]|0;
  $32 = (($31) + -1)|0;
  HEAP32[$count>>2] = $32;
  $33 = ($31|0)!=(0);
  if (!($33)) {
   label = 9;
   break;
  }
  $34 = HEAP32[$elem>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (_sqlite3_stricmp($36,$37)|0);
  $39 = ($38|0)==(0);
  $40 = HEAP32[$elem>>2]|0;
  if ($39) {
   label = 7;
   break;
  }
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$elem>>2] = $41;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = $40;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _strHash($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $h = sp;
 $c = sp + 8|0;
 HEAP32[$0>>2] = $z;
 HEAP32[$h>>2] = 0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ((($1)) + 1|0);
  HEAP32[$0>>2] = $2;
  $3 = HEAP8[$1>>0]|0;
  HEAP8[$c>>0] = $3;
  $4 = $3&255;
  $5 = ($4|0)!=(0);
  $6 = HEAP32[$h>>2]|0;
  if (!($5)) {
   break;
  }
  $7 = $6 << 3;
  $8 = HEAP32[$h>>2]|0;
  $9 = $7 ^ $8;
  $10 = HEAP8[$c>>0]|0;
  $11 = $10&255;
  $12 = (16974 + ($11)|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = $9 ^ $14;
  HEAP32[$h>>2] = $15;
 }
 STACKTOP = sp;return ($6|0);
}
function _sqlite3HexToInt($h) {
 $h = $h|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $h;
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 >> 6;
 $3 = 1 & $2;
 $4 = ($3*9)|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + ($4))|0;
 HEAP32[$0>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = $7 & 15;
 $9 = $8&255;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3DefaultRowEst($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $aVal = 0, $i = 0, $nCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $aVal = sp + 16|0;
 $a = sp + 8|0;
 $nCopy = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pIdx;
 ;HEAP16[$aVal>>1]=HEAP16[10028>>1]|0;HEAP16[$aVal+2>>1]=HEAP16[10028+2>>1]|0;HEAP16[$aVal+4>>1]=HEAP16[10028+4>>1]|0;HEAP16[$aVal+6>>1]=HEAP16[10028+6>>1]|0;HEAP16[$aVal+8>>1]=HEAP16[10028+8>>1]|0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$a>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 50|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = (5)<($7|0);
 if ($8) {
  $13 = 5;
 } else {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 50|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12;
 }
 HEAP32[$nCopy>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 38|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = HEAP32[$a>>2]|0;
 HEAP16[$19>>1] = $18;
 $20 = HEAP32[$a>>2]|0;
 $21 = HEAP16[$20>>1]|0;
 $22 = $21 << 16 >> 16;
 $23 = ($22|0)<(33);
 if ($23) {
  $24 = HEAP32[$a>>2]|0;
  HEAP16[$24>>1] = 33;
 }
 $25 = HEAP32[$a>>2]|0;
 $26 = ((($25)) + 2|0);
 $27 = HEAP32[$nCopy>>2]|0;
 $28 = $27<<1;
 _memcpy(($26|0),($aVal|0),($28|0))|0;
 $29 = HEAP32[$nCopy>>2]|0;
 $30 = (($29) + 1)|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 50|0);
  $34 = HEAP16[$33>>1]|0;
  $35 = $34&65535;
  $36 = ($31|0)<=($35|0);
  if (!($36)) {
   break;
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = HEAP32[$a>>2]|0;
  $39 = (($38) + ($37<<1)|0);
  HEAP16[$39>>1] = 23;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 54|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = ($45|0)!=(0);
 if (!($46)) {
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 50|0);
 $49 = HEAP16[$48>>1]|0;
 $50 = $49&65535;
 $51 = HEAP32[$a>>2]|0;
 $52 = (($51) + ($50<<1)|0);
 HEAP16[$52>>1] = 0;
 STACKTOP = sp;return;
}
function _analysisLoader($pData,$argc,$argv,$NotUsed) {
 $pData = $pData|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $aiRowEst = 0, $fakeIdx = 0, $nCol = 0, $pIndex = 0, $pInfo = 0, $pTable = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $pInfo = sp + 76|0;
 $pIndex = sp + 72|0;
 $pTable = sp + 68|0;
 $z = sp + 64|0;
 $aiRowEst = sp + 60|0;
 $nCol = sp + 56|0;
 $fakeIdx = sp;
 HEAP32[$1>>2] = $pData;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $NotUsed;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pInfo>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6|0)==(0|0);
 if (!($7)) {
  $8 = HEAP32[$3>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0|0);
  if (!($10)) {
   $11 = HEAP32[$3>>2]|0;
   $12 = ((($11)) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if (!($14)) {
    $15 = HEAP32[$pInfo>>2]|0;
    $16 = HEAP32[$15>>2]|0;
    $17 = HEAP32[$3>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$pInfo>>2]|0;
    $20 = ((($19)) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (_sqlite3FindTable($16,$18,$21)|0);
    HEAP32[$pTable>>2] = $22;
    $23 = HEAP32[$pTable>>2]|0;
    $24 = ($23|0)==(0|0);
    if ($24) {
     HEAP32[$0>>2] = 0;
     $89 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($89|0);
    }
    $25 = HEAP32[$3>>2]|0;
    $26 = ((($25)) + 4|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27|0)==(0|0);
    do {
     if ($28) {
      HEAP32[$pIndex>>2] = 0;
     } else {
      $29 = HEAP32[$3>>2]|0;
      $30 = HEAP32[$29>>2]|0;
      $31 = HEAP32[$3>>2]|0;
      $32 = ((($31)) + 4|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = (_sqlite3_stricmp($30,$33)|0);
      $35 = ($34|0)==(0);
      if ($35) {
       $36 = HEAP32[$pTable>>2]|0;
       $37 = (_sqlite3PrimaryKeyIndex($36)|0);
       HEAP32[$pIndex>>2] = $37;
       break;
      } else {
       $38 = HEAP32[$pInfo>>2]|0;
       $39 = HEAP32[$38>>2]|0;
       $40 = HEAP32[$3>>2]|0;
       $41 = ((($40)) + 4|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = HEAP32[$pInfo>>2]|0;
       $44 = ((($43)) + 4|0);
       $45 = HEAP32[$44>>2]|0;
       $46 = (_sqlite3FindIndex($39,$42,$45)|0);
       HEAP32[$pIndex>>2] = $46;
       break;
      }
     }
    } while(0);
    $47 = HEAP32[$3>>2]|0;
    $48 = ((($47)) + 8|0);
    $49 = HEAP32[$48>>2]|0;
    HEAP32[$z>>2] = $49;
    $50 = HEAP32[$pIndex>>2]|0;
    $51 = ($50|0)!=(0|0);
    if ($51) {
     HEAP32[$aiRowEst>>2] = 0;
     $52 = HEAP32[$pIndex>>2]|0;
     $53 = ((($52)) + 50|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = (($55) + 1)|0;
     HEAP32[$nCol>>2] = $56;
     $57 = HEAP32[$pIndex>>2]|0;
     $58 = ((($57)) + 55|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59 & -5;
     HEAP8[$58>>0] = $60;
     $61 = HEAP32[$z>>2]|0;
     $62 = HEAP32[$nCol>>2]|0;
     $63 = HEAP32[$aiRowEst>>2]|0;
     $64 = HEAP32[$pIndex>>2]|0;
     $65 = ((($64)) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$pIndex>>2]|0;
     _decodeIntArray($61,$62,$63,$66,$67);
     $68 = HEAP32[$pIndex>>2]|0;
     $69 = ((($68)) + 36|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($70|0)==(0|0);
     if ($71) {
      $72 = HEAP32[$pIndex>>2]|0;
      $73 = ((($72)) + 8|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = HEAP16[$74>>1]|0;
      $76 = HEAP32[$pTable>>2]|0;
      $77 = ((($76)) + 38|0);
      HEAP16[$77>>1] = $75;
     }
    } else {
     $78 = HEAP32[$pTable>>2]|0;
     $79 = ((($78)) + 40|0);
     $80 = HEAP16[$79>>1]|0;
     $81 = ((($fakeIdx)) + 48|0);
     HEAP16[$81>>1] = $80;
     $82 = HEAP32[$z>>2]|0;
     $83 = HEAP32[$pTable>>2]|0;
     $84 = ((($83)) + 38|0);
     _decodeIntArray($82,1,0,$84,$fakeIdx);
     $85 = ((($fakeIdx)) + 48|0);
     $86 = HEAP16[$85>>1]|0;
     $87 = HEAP32[$pTable>>2]|0;
     $88 = ((($87)) + 40|0);
     HEAP16[$88>>1] = $86;
    }
    HEAP32[$0>>2] = 0;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3PrimaryKeyIndex($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   label = 5;
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 55|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 & 3;
  $10 = $9&255;
  $11 = ($10|0)==(2);
  $12 = $11 ^ 1;
  if (!($12)) {
   label = 5;
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$p>>2] = $15;
 }
 if ((label|0) == 5) {
  $16 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _decodeIntArray($zIntArray,$nOut,$aOut,$aLog,$pIndex) {
 $zIntArray = $zIntArray|0;
 $nOut = $nOut|0;
 $aOut = $aOut|0;
 $aLog = $aLog|0;
 $pIndex = $pIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $c = 0, $i = 0, $v = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $z = sp + 12|0;
 $c = sp + 8|0;
 $i = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $zIntArray;
 HEAP32[$1>>2] = $nOut;
 HEAP32[$2>>2] = $aOut;
 HEAP32[$3>>2] = $aLog;
 HEAP32[$4>>2] = $pIndex;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$z>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)!=(0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  HEAP32[$v>>2] = 0;
  while(1) {
   $13 = HEAP32[$z>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   HEAP32[$c>>2] = $15;
   $16 = ($15|0)>=(48);
   $17 = HEAP32[$c>>2]|0;
   $18 = ($17|0)<=(57);
   $19 = $16 ? $18 : 0;
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$v>>2]|0;
   $21 = ($20*10)|0;
   $22 = HEAP32[$c>>2]|0;
   $23 = (($21) + ($22))|0;
   $24 = (($23) - 48)|0;
   HEAP32[$v>>2] = $24;
   $25 = HEAP32[$z>>2]|0;
   $26 = ((($25)) + 1|0);
   HEAP32[$z>>2] = $26;
  }
  $27 = HEAP32[$v>>2]|0;
  $28 = (_sqlite3LogEst($27,0)|0);
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = (($30) + ($29<<1)|0);
  HEAP16[$31>>1] = $28;
  $32 = HEAP32[$z>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = $33 << 24 >> 24;
  $35 = ($34|0)==(32);
  if ($35) {
   $36 = HEAP32[$z>>2]|0;
   $37 = ((($36)) + 1|0);
   HEAP32[$z>>2] = $37;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 $40 = HEAP32[$4>>2]|0;
 $41 = ((($40)) + 55|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42 & -5;
 HEAP8[$41>>0] = $43;
 $44 = HEAP32[$4>>2]|0;
 $45 = ((($44)) + 55|0);
 $46 = HEAP8[$45>>0]|0;
 $47 = $46 & -65;
 HEAP8[$45>>0] = $47;
 L13: while(1) {
  $48 = HEAP32[$z>>2]|0;
  $49 = HEAP8[$48>>0]|0;
  $50 = ($49<<24>>24)!=(0);
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$z>>2]|0;
  $52 = (_sqlite3_strglob(21774,$51)|0);
  $53 = ($52|0)==(0);
  do {
   if ($53) {
    $54 = HEAP32[$4>>2]|0;
    $55 = ((($54)) + 55|0);
    $56 = HEAP8[$55>>0]|0;
    $57 = $56 & -5;
    $58 = $57 | 4;
    HEAP8[$55>>0] = $58;
   } else {
    $59 = HEAP32[$z>>2]|0;
    $60 = (_sqlite3_strglob(21785,$59)|0);
    $61 = ($60|0)==(0);
    $62 = HEAP32[$z>>2]|0;
    if ($61) {
     $63 = ((($62)) + 3|0);
     $64 = (_sqlite3Atoi($63)|0);
     $65 = ($64|0)<(0);
     $66 = $65 << 31 >> 31;
     $67 = (_sqlite3LogEst($64,$66)|0);
     $68 = HEAP32[$4>>2]|0;
     $69 = ((($68)) + 48|0);
     HEAP16[$69>>1] = $67;
     break;
    }
    $70 = (_sqlite3_strglob(21795,$62)|0);
    $71 = ($70|0)==(0);
    if ($71) {
     $72 = HEAP32[$4>>2]|0;
     $73 = ((($72)) + 55|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = $74 & -65;
     $76 = $75 | 64;
     HEAP8[$73>>0] = $76;
    }
   }
  } while(0);
  while(1) {
   $77 = HEAP32[$z>>2]|0;
   $78 = HEAP8[$77>>0]|0;
   $79 = $78 << 24 >> 24;
   $80 = ($79|0)!=(0);
   if (!($80)) {
    break;
   }
   $81 = HEAP32[$z>>2]|0;
   $82 = HEAP8[$81>>0]|0;
   $83 = $82 << 24 >> 24;
   $84 = ($83|0)!=(32);
   if (!($84)) {
    break;
   }
   $85 = HEAP32[$z>>2]|0;
   $86 = ((($85)) + 1|0);
   HEAP32[$z>>2] = $86;
  }
  while(1) {
   $87 = HEAP32[$z>>2]|0;
   $88 = HEAP8[$87>>0]|0;
   $89 = $88 << 24 >> 24;
   $90 = ($89|0)==(32);
   if (!($90)) {
    continue L13;
   }
   $91 = HEAP32[$z>>2]|0;
   $92 = ((($91)) + 1|0);
   HEAP32[$z>>2] = $92;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3LogEst($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 10|0;
 $3 = sp;
 $y = sp + 8|0;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP16[$y>>1] = 40;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13>>>0)<(0);
 $15 = ($10>>>0)<(8);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 L1: do {
  if ($18) {
   $19 = $3;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $22 = (($19) + 4)|0;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24>>>0)<(0);
   $26 = ($21>>>0)<(2);
   $27 = ($24|0)==(0);
   $28 = $27 & $26;
   $29 = $25 | $28;
   if ($29) {
    HEAP16[$2>>1] = 0;
    $126 = HEAP16[$2>>1]|0;
    STACKTOP = sp;return ($126|0);
   } else {
    while(1) {
     $30 = $3;
     $31 = $30;
     $32 = HEAP32[$31>>2]|0;
     $33 = (($30) + 4)|0;
     $34 = $33;
     $35 = HEAP32[$34>>2]|0;
     $36 = ($35>>>0)<(0);
     $37 = ($32>>>0)<(8);
     $38 = ($35|0)==(0);
     $39 = $38 & $37;
     $40 = $36 | $39;
     if (!($40)) {
      break L1;
     }
     $41 = HEAP16[$y>>1]|0;
     $42 = $41 << 16 >> 16;
     $43 = (($42) - 10)|0;
     $44 = $43&65535;
     HEAP16[$y>>1] = $44;
     $45 = $3;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($45) + 4)|0;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (_bitshift64Shl(($47|0),($50|0),1)|0);
     $52 = tempRet0;
     $53 = $3;
     $54 = $53;
     HEAP32[$54>>2] = $51;
     $55 = (($53) + 4)|0;
     $56 = $55;
     HEAP32[$56>>2] = $52;
    }
   }
  } else {
   while(1) {
    $57 = $3;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (($57) + 4)|0;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = ($62>>>0)>(0);
    $64 = ($59>>>0)>(255);
    $65 = ($62|0)==(0);
    $66 = $65 & $64;
    $67 = $63 | $66;
    if (!($67)) {
     break;
    }
    $68 = HEAP16[$y>>1]|0;
    $69 = $68 << 16 >> 16;
    $70 = (($69) + 40)|0;
    $71 = $70&65535;
    HEAP16[$y>>1] = $71;
    $72 = $3;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = (_bitshift64Lshr(($74|0),($77|0),4)|0);
    $79 = tempRet0;
    $80 = $3;
    $81 = $80;
    HEAP32[$81>>2] = $78;
    $82 = (($80) + 4)|0;
    $83 = $82;
    HEAP32[$83>>2] = $79;
   }
   while(1) {
    $84 = $3;
    $85 = $84;
    $86 = HEAP32[$85>>2]|0;
    $87 = (($84) + 4)|0;
    $88 = $87;
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89>>>0)>(0);
    $91 = ($86>>>0)>(15);
    $92 = ($89|0)==(0);
    $93 = $92 & $91;
    $94 = $90 | $93;
    if (!($94)) {
     break L1;
    }
    $95 = HEAP16[$y>>1]|0;
    $96 = $95 << 16 >> 16;
    $97 = (($96) + 10)|0;
    $98 = $97&65535;
    HEAP16[$y>>1] = $98;
    $99 = $3;
    $100 = $99;
    $101 = HEAP32[$100>>2]|0;
    $102 = (($99) + 4)|0;
    $103 = $102;
    $104 = HEAP32[$103>>2]|0;
    $105 = (_bitshift64Lshr(($101|0),($104|0),1)|0);
    $106 = tempRet0;
    $107 = $3;
    $108 = $107;
    HEAP32[$108>>2] = $105;
    $109 = (($107) + 4)|0;
    $110 = $109;
    HEAP32[$110>>2] = $106;
   }
  }
 } while(0);
 $111 = $3;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = $113 & 7;
 $118 = (10038 + ($117<<1)|0);
 $119 = HEAP16[$118>>1]|0;
 $120 = $119 << 16 >> 16;
 $121 = HEAP16[$y>>1]|0;
 $122 = $121 << 16 >> 16;
 $123 = (($120) + ($122))|0;
 $124 = (($123) - 10)|0;
 $125 = $124&65535;
 HEAP16[$2>>1] = $125;
 $126 = HEAP16[$2>>1]|0;
 STACKTOP = sp;return ($126|0);
}
function _sqlite3HashInsert($pH,$pKey,$data) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $data = $data|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_elem = 0, $old_data = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $h = sp + 12|0;
 $elem = sp + 8|0;
 $new_elem = sp + 4|0;
 $old_data = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $data;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_findElementWithHash($4,$5,$h)|0);
 HEAP32[$elem>>2] = $6;
 $7 = HEAP32[$elem>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$elem>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$old_data>>2] = $11;
  $12 = HEAP32[$3>>2]|0;
  $13 = ($12|0)==(0|0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$elem>>2]|0;
   $16 = HEAP32[$h>>2]|0;
   _removeElementGivenHash($14,$15,$16);
  } else {
   $17 = HEAP32[$3>>2]|0;
   $18 = HEAP32[$elem>>2]|0;
   $19 = ((($18)) + 8|0);
   HEAP32[$19>>2] = $17;
   $20 = HEAP32[$2>>2]|0;
   $21 = HEAP32[$elem>>2]|0;
   $22 = ((($21)) + 12|0);
   HEAP32[$22>>2] = $20;
  }
  $23 = HEAP32[$old_data>>2]|0;
  HEAP32[$0>>2] = $23;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $24 = HEAP32[$3>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $26 = (_sqlite3Malloc(16,0)|0);
 HEAP32[$new_elem>>2] = $26;
 $27 = HEAP32[$new_elem>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$3>>2]|0;
  HEAP32[$0>>2] = $29;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$new_elem>>2]|0;
 $32 = ((($31)) + 12|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$3>>2]|0;
 $34 = HEAP32[$new_elem>>2]|0;
 $35 = ((($34)) + 8|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42>>>0)>=(10);
 if ($43) {
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = $48<<1;
  $50 = ($46>>>0)>($49>>>0);
  if ($50) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$1>>2]|0;
   $53 = ((($52)) + 4|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = $54<<1;
   $56 = (_rehash($51,$55)|0);
   $57 = ($56|0)!=(0);
   if ($57) {
    $58 = HEAP32[$2>>2]|0;
    $59 = (_strHash($58)|0);
    $60 = HEAP32[$1>>2]|0;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59>>>0) % ($61>>>0))&-1;
    HEAP32[$h>>2] = $62;
   }
  }
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 12|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0|0);
 if ($67) {
  $68 = HEAP32[$h>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 12|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + ($68<<3)|0);
  $74 = $72;
 } else {
  $74 = 0;
 }
 $73 = HEAP32[$new_elem>>2]|0;
 _insertElement($63,$74,$73);
 HEAP32[$0>>2] = 0;
 $75 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _removeElementGivenHash($pH,$elem,$h) {
 $pH = $pH|0;
 $elem = $elem|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pEntry = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $elem;
 HEAP32[$2>>2] = $h;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 if ($6) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$11>>2] = $8;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  HEAP32[$13>>2] = $8;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  HEAP32[$22>>2] = $19;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<3)|0);
  HEAP32[$pEntry>>2] = $31;
  $32 = HEAP32[$pEntry>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ($34|0)==($35|0);
  if ($36) {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$pEntry>>2]|0;
   $40 = ((($39)) + 4|0);
   HEAP32[$40>>2] = $38;
  }
  $41 = HEAP32[$pEntry>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + -1)|0;
  HEAP32[$41>>2] = $43;
 }
 $44 = HEAP32[$1>>2]|0;
 _sqlite3_free($44);
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 4|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + -1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)==(0);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$0>>2]|0;
 _sqlite3HashClear($53);
 STACKTOP = sp;return;
}
function _rehash($pH,$new_size) {
 $pH = $pH|0;
 $new_size = $new_size|0;
 var $$new_size = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_ht = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $new_ht = sp + 12|0;
 $elem = sp + 8|0;
 $next_elem = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $new_size;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<3;
 $5 = ($4>>>0)>(1024);
 $$new_size = $5 ? 128 : $new_size;
 HEAP32[$2>>2] = $$new_size;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($6|0)==($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 _sqlite3BeginBenignMalloc();
 $10 = HEAP32[$2>>2]|0;
 $11 = $10<<3;
 $12 = (_sqlite3Malloc($11,0)|0);
 HEAP32[$new_ht>>2] = $12;
 _sqlite3EndBenignMalloc();
 $13 = HEAP32[$new_ht>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3_free($17);
 $18 = HEAP32[$new_ht>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = HEAP32[$new_ht>>2]|0;
 $22 = (_sqlite3MallocSize($21)|0);
 $23 = (($22>>>0) / 8)&-1;
 HEAP32[$2>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$new_ht>>2]|0;
 $26 = HEAP32[$2>>2]|0;
 $27 = $26<<3;
 _memset(($25|0),0,($27|0))|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$elem>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 8|0);
 HEAP32[$32>>2] = 0;
 while(1) {
  $33 = HEAP32[$elem>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$elem>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_strHash($37)|0);
  $39 = HEAP32[$2>>2]|0;
  $40 = (($38>>>0) % ($39>>>0))&-1;
  HEAP32[$h>>2] = $40;
  $41 = HEAP32[$elem>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$next_elem>>2] = $42;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$h>>2]|0;
  $45 = HEAP32[$new_ht>>2]|0;
  $46 = (($45) + ($44<<3)|0);
  $47 = HEAP32[$elem>>2]|0;
  _insertElement($43,$46,$47);
  $48 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $48;
 }
 HEAP32[$0>>2] = 1;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _insertElement($pH,$pEntry,$pNew) {
 $pH = $pH|0;
 $pEntry = $pEntry|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pHead = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pHead = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pEntry;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0);
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = $10;
  } else {
   $11 = 0;
  }
  HEAP32[$pHead>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$12>>2] = $14;
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 4|0);
  HEAP32[$17>>2] = $15;
 } else {
  HEAP32[$pHead>>2] = 0;
 }
 $18 = HEAP32[$pHead>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0|0);
  if ($47) {
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$0>>2]|0;
   $50 = ((($49)) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ((($51)) + 4|0);
   HEAP32[$52>>2] = $48;
  }
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 4|0);
  HEAP32[$54>>2] = 0;
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 8|0);
  HEAP32[$57>>2] = $55;
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$pHead>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[$pHead>>2]|0;
 $23 = ((($22)) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 4|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$pHead>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 $31 = HEAP32[$2>>2]|0;
 if ($30) {
  $32 = HEAP32[$pHead>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$34>>2] = $31;
 } else {
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP32[$36>>2] = $31;
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$pHead>>2]|0;
 $39 = ((($38)) + 4|0);
 HEAP32[$39>>2] = $37;
 STACKTOP = sp;return;
}
function _freeIndex($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _sqlite3DeleteIndexSamples($2,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3ExprDelete($4,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3ExprListDelete($8,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 _sqlite3DbFree($12,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 55|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18&255) >>> 4;
 $20 = $19 & 1;
 $21 = $20&255;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($27,$28);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 32|0);
 $26 = HEAP32[$25>>2]|0;
 _sqlite3DbFree($23,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($27,$28);
 STACKTOP = sp;return;
}
function _sqlite3DeleteIndexSamples($db,$pIdx) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pIdx;
 STACKTOP = sp;return;
}
function _sqlite3ExprDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 16384;
 $8 = ($7|0)!=(0);
 do {
  if (!($8)) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 12|0);
   $12 = HEAP32[$11>>2]|0;
   _sqlite3ExprDelete($9,$12);
   $13 = HEAP32[$0>>2]|0;
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 16|0);
   $16 = HEAP32[$15>>2]|0;
   _sqlite3ExprDelete($13,$16);
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = $19 & 65536;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$0>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    _sqlite3DbFree($22,$25);
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = $28 & 2048;
   $30 = ($29|0)!=(0);
   $31 = HEAP32[$0>>2]|0;
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 20|0);
   if ($30) {
    $34 = HEAP32[$33>>2]|0;
    _sqlite3SelectDelete($31,$34);
    break;
   } else {
    $35 = HEAP32[$33>>2]|0;
    _sqlite3ExprListDelete($31,$35);
    break;
   }
  }
 } while(0);
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = $38 & 32768;
 $40 = ($39|0)!=(0);
 if ($40) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($41,$42);
 STACKTOP = sp;return;
}
function _sqlite3ExprListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pItem>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  $11 = HEAP32[$0>>2]|0;
  if (!($10)) {
   break;
  }
  $12 = HEAP32[$pItem>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  _sqlite3ExprDelete($11,$13);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$pItem>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3DbFree($14,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3DbFree($18,$21);
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
  $24 = HEAP32[$pItem>>2]|0;
  $25 = ((($24)) + 20|0);
  HEAP32[$pItem>>2] = $25;
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 _sqlite3DbFree($11,$28);
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($29,$30);
 STACKTOP = sp;return;
}
function _sqlite3SelectDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _clearSelect($2,$3,1);
 STACKTOP = sp;return;
}
function _clearSelect($db,$p,$bFree) {
 $db = $db|0;
 $p = $p|0;
 $bFree = $bFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPrior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPrior = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bFree;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 52|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pPrior>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  _sqlite3ExprListDelete($8,$10);
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 32|0);
  $14 = HEAP32[$13>>2]|0;
  _sqlite3SrcListDelete($11,$14);
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 36|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3ExprDelete($15,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 40|0);
  $22 = HEAP32[$21>>2]|0;
  _sqlite3ExprListDelete($19,$22);
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 44|0);
  $26 = HEAP32[$25>>2]|0;
  _sqlite3ExprDelete($23,$26);
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 48|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3ExprListDelete($27,$30);
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 60|0);
  $34 = HEAP32[$33>>2]|0;
  _sqlite3ExprDelete($31,$34);
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 64|0);
  $38 = HEAP32[$37>>2]|0;
  _sqlite3ExprDelete($35,$38);
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 68|0);
  $42 = HEAP32[$41>>2]|0;
  _sqlite3WithDelete($39,$42);
  $43 = HEAP32[$2>>2]|0;
  $44 = ($43|0)!=(0);
  if ($44) {
   $45 = HEAP32[$0>>2]|0;
   $46 = HEAP32[$1>>2]|0;
   _sqlite3DbFree($45,$46);
  }
  $47 = HEAP32[$pPrior>>2]|0;
  HEAP32[$1>>2] = $47;
  HEAP32[$2>>2] = 1;
 }
 STACKTOP = sp;return;
}
function _sqlite3SrcListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$pItem>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  $10 = HEAP32[$0>>2]|0;
  if (!($9)) {
   break;
  }
  $11 = HEAP32[$pItem>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  _sqlite3DbFree($10,$13);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$pItem>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3DbFree($14,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3DbFree($18,$21);
  $22 = HEAP32[$pItem>>2]|0;
  $23 = ((($22)) + 36|0);
  $24 = ((($23)) + 1|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = ($25&255) >>> 1;
  $27 = $26 & 1;
  $28 = $27&255;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$pItem>>2]|0;
   $32 = ((($31)) + 64|0);
   $33 = HEAP32[$32>>2]|0;
   _sqlite3DbFree($30,$33);
  }
  $34 = HEAP32[$pItem>>2]|0;
  $35 = ((($34)) + 36|0);
  $36 = ((($35)) + 1|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = ($37&255) >>> 2;
  $39 = $38 & 1;
  $40 = $39&255;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = HEAP32[$0>>2]|0;
   $43 = HEAP32[$pItem>>2]|0;
   $44 = ((($43)) + 64|0);
   $45 = HEAP32[$44>>2]|0;
   _sqlite3ExprListDelete($42,$45);
  }
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$pItem>>2]|0;
  $48 = ((($47)) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  _sqlite3DeleteTable($46,$49);
  $50 = HEAP32[$0>>2]|0;
  $51 = HEAP32[$pItem>>2]|0;
  $52 = ((($51)) + 20|0);
  $53 = HEAP32[$52>>2]|0;
  _sqlite3SelectDelete($50,$53);
  $54 = HEAP32[$0>>2]|0;
  $55 = HEAP32[$pItem>>2]|0;
  $56 = ((($55)) + 48|0);
  $57 = HEAP32[$56>>2]|0;
  _sqlite3ExprDelete($54,$57);
  $58 = HEAP32[$0>>2]|0;
  $59 = HEAP32[$pItem>>2]|0;
  $60 = ((($59)) + 52|0);
  $61 = HEAP32[$60>>2]|0;
  _sqlite3IdListDelete($58,$61);
  $62 = HEAP32[$i>>2]|0;
  $63 = (($62) + 1)|0;
  HEAP32[$i>>2] = $63;
  $64 = HEAP32[$pItem>>2]|0;
  $65 = ((($64)) + 72|0);
  HEAP32[$pItem>>2] = $65;
 }
 $66 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($10,$66);
 STACKTOP = sp;return;
}
function _sqlite3WithDelete($db,$pWith) {
 $db = $db|0;
 $pWith = $pWith|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pCte = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pWith;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = (($10) + ($8<<4)|0);
  HEAP32[$pCte>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pCte>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3ExprListDelete($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pCte>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3SelectDelete($16,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$pCte>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  _sqlite3DbFree($20,$22);
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($25,$26);
 STACKTOP = sp;return;
}
function _sqlite3IdListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  $9 = HEAP32[$0>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  _sqlite3DbFree($9,$14);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 _sqlite3DbFree($9,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($19,$20);
 STACKTOP = sp;return;
}
function _tableOfTrigger($pTrigger) {
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTrigger;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3HashFind($4,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _setPageReferenced($pCheck,$iPg) {
 $pCheck = $pCheck|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 7;
 $4 = 1 << $3;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5>>>0) / 8)&-1;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 | $4;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 STACKTOP = sp;return;
}
function _checkList($pCheck,$isFreeList,$iPage,$N) {
 $pCheck = $pCheck|0;
 $isFreeList = $isFreeList|0;
 $iPage = $iPage|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $expected = 0, $i = 0, $iFirst = 0, $iFreePage = 0, $n = 0, $or$cond = 0, $pOvflData = 0, $pOvflPage = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $i = sp + 60|0;
 $expected = sp + 56|0;
 $iFirst = sp + 52|0;
 $pOvflPage = sp + 48|0;
 $pOvflData = sp + 44|0;
 $n = sp + 40|0;
 $iFreePage = sp + 36|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $isFreeList;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$expected>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$iFirst>>2] = $5;
 while(1) {
  $6 = HEAP32[$3>>2]|0;
  $7 = (($6) + -1)|0;
  HEAP32[$3>>2] = $7;
  $8 = ($6|0)>(0);
  if (!($8)) {
   label = 25;
   break;
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   label = 25;
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)<(1);
  $15 = HEAP32[$0>>2]|0;
  if ($14) {
   label = 5;
   break;
  }
  $20 = HEAP32[$2>>2]|0;
  $21 = (_checkRef($15,$20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   label = 25;
   break;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (_sqlite3PagerGet($25,$26,$pOvflPage,0)|0);
  $28 = ($27|0)!=(0);
  if ($28) {
   label = 8;
   break;
  }
  $31 = HEAP32[$pOvflPage>>2]|0;
  $32 = (_sqlite3PagerGetData($31)|0);
  HEAP32[$pOvflData>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = ($33|0)!=(0);
  do {
   if ($34) {
    $35 = HEAP32[$pOvflData>>2]|0;
    $36 = ((($35)) + 4|0);
    $37 = (_sqlite3Get4byte($36)|0);
    HEAP32[$n>>2] = $37;
    $38 = HEAP32[$0>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = ((($39)) + 17|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = ($41<<24>>24)!=(0);
    if ($42) {
     $43 = HEAP32[$0>>2]|0;
     $44 = HEAP32[$2>>2]|0;
     _checkPtrmap($43,$44,2,0);
    }
    $45 = HEAP32[$n>>2]|0;
    $46 = HEAP32[$0>>2]|0;
    $47 = HEAP32[$46>>2]|0;
    $48 = ((($47)) + 36|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49|0) / 4)&-1;
    $51 = (($50) - 2)|0;
    $52 = ($45|0)>($51|0);
    if ($52) {
     $53 = HEAP32[$0>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $54;
     _checkAppendMsg($53,21958,$vararg_buffer7);
     $55 = HEAP32[$3>>2]|0;
     $56 = (($55) + -1)|0;
     HEAP32[$3>>2] = $56;
     break;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $57 = HEAP32[$i>>2]|0;
     $58 = HEAP32[$n>>2]|0;
     $59 = ($57|0)<($58|0);
     if (!($59)) {
      break;
     }
     $60 = HEAP32[$i>>2]|0;
     $61 = $60<<2;
     $62 = (8 + ($61))|0;
     $63 = HEAP32[$pOvflData>>2]|0;
     $64 = (($63) + ($62)|0);
     $65 = (_sqlite3Get4byte($64)|0);
     HEAP32[$iFreePage>>2] = $65;
     $66 = HEAP32[$0>>2]|0;
     $67 = HEAP32[$66>>2]|0;
     $68 = ((($67)) + 17|0);
     $69 = HEAP8[$68>>0]|0;
     $70 = ($69<<24>>24)!=(0);
     if ($70) {
      $71 = HEAP32[$0>>2]|0;
      $72 = HEAP32[$iFreePage>>2]|0;
      _checkPtrmap($71,$72,2,0);
     }
     $73 = HEAP32[$0>>2]|0;
     $74 = HEAP32[$iFreePage>>2]|0;
     (_checkRef($73,$74)|0);
     $75 = HEAP32[$i>>2]|0;
     $76 = (($75) + 1)|0;
     HEAP32[$i>>2] = $76;
    }
    $77 = HEAP32[$n>>2]|0;
    $78 = HEAP32[$3>>2]|0;
    $79 = (($78) - ($77))|0;
    HEAP32[$3>>2] = $79;
   } else {
    $80 = HEAP32[$0>>2]|0;
    $81 = HEAP32[$80>>2]|0;
    $82 = ((($81)) + 17|0);
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = ($84|0)!=(0);
    $86 = HEAP32[$3>>2]|0;
    $87 = ($86|0)>(0);
    $or$cond = $85 & $87;
    if ($or$cond) {
     $88 = HEAP32[$pOvflData>>2]|0;
     $89 = (_sqlite3Get4byte($88)|0);
     HEAP32[$i>>2] = $89;
     $90 = HEAP32[$0>>2]|0;
     $91 = HEAP32[$i>>2]|0;
     $92 = HEAP32[$2>>2]|0;
     _checkPtrmap($90,$91,4,$92);
    }
   }
  } while(0);
  $93 = HEAP32[$pOvflData>>2]|0;
  $94 = (_sqlite3Get4byte($93)|0);
  HEAP32[$2>>2] = $94;
  $95 = HEAP32[$pOvflPage>>2]|0;
  _sqlite3PagerUnref($95);
  $96 = HEAP32[$1>>2]|0;
  $97 = ($96|0)!=(0);
  if (!($97)) {
   continue;
  }
  $98 = HEAP32[$3>>2]|0;
  $99 = HEAP32[$2>>2]|0;
  $100 = ($99|0)!=(0);
  $101 = $100&1;
  $102 = ($98|0)<($101|0);
  if (!($102)) {
   continue;
  }
  $103 = HEAP32[$0>>2]|0;
  _checkAppendMsg($103,21997,$vararg_buffer10);
 }
 if ((label|0) == 5) {
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + 1)|0;
  $18 = HEAP32[$expected>>2]|0;
  $19 = HEAP32[$iFirst>>2]|0;
  HEAP32[$vararg_buffer>>2] = $17;
  $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr2>>2] = $18;
  $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr3>>2] = $19;
  _checkAppendMsg($15,21879,$vararg_buffer);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 8) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $30;
  _checkAppendMsg($29,21936,$vararg_buffer4);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 25) {
  STACKTOP = sp;return;
 }
}
function _checkPtrmap($pCheck,$iChild,$eType,$iParent) {
 $pCheck = $pCheck|0;
 $iChild = $iChild|0;
 $eType = $eType|0;
 $iParent = $iParent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ePtrmapType = 0, $iPtrmapParent = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0;
 var $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 49|0;
 $3 = sp + 36|0;
 $rc = sp + 32|0;
 $ePtrmapType = sp + 48|0;
 $iPtrmapParent = sp + 28|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iChild;
 HEAP8[$2>>0] = $eType;
 HEAP32[$3>>2] = $iParent;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_ptrmapGet($5,$6,$ePtrmapType,$iPtrmapParent)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(7);
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)==(3082);
  $or$cond = $11 | $13;
  if ($or$cond) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 24|0);
   HEAP32[$15>>2] = 1;
  }
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $17;
  _checkAppendMsg($16,22084,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $18 = HEAP8[$ePtrmapType>>0]|0;
 $19 = $18&255;
 $20 = HEAP8[$2>>0]|0;
 $21 = $20&255;
 $22 = ($19|0)!=($21|0);
 if (!($22)) {
  $23 = HEAP32[$iPtrmapParent>>2]|0;
  $24 = HEAP32[$3>>2]|0;
  $25 = ($23|0)!=($24|0);
  if (!($25)) {
   STACKTOP = sp;return;
  }
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP8[$2>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP8[$ePtrmapType>>0]|0;
 $32 = $31&255;
 $33 = HEAP32[$iPtrmapParent>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $27;
 $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $29;
 $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
 HEAP32[$vararg_ptr6>>2] = $30;
 $vararg_ptr7 = ((($vararg_buffer2)) + 12|0);
 HEAP32[$vararg_ptr7>>2] = $32;
 $vararg_ptr8 = ((($vararg_buffer2)) + 16|0);
 HEAP32[$vararg_ptr8>>2] = $33;
 _checkAppendMsg($26,22113,$vararg_buffer2);
 STACKTOP = sp;return;
}
function _checkTreePage($pCheck,$iPage,$piMinKey,$0,$1) {
 $pCheck = $pCheck|0;
 $iPage = $iPage|0;
 $piMinKey = $piMinKey|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cellStart = 0;
 var $contentOffset = 0, $d2 = 0, $data = 0, $depth = 0, $doCoverageCheck = 0, $hdr = 0, $heap = 0, $i = 0, $info = 0, $j = 0, $keyCanBeEqual = 0, $nCell = 0, $nFrag = 0, $nPage = 0, $pBt = 0, $pCell = 0, $pCellIdx = 0, $pPage = 0, $pc = 0, $pgno = 0;
 var $pgnoOvfl = 0, $prev = 0, $rc = 0, $savedIsInit = 0, $saved_v1 = 0, $saved_v2 = 0, $saved_zPfx = 0, $size = 0, $size1 = 0, $usableSize = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer20 = 0, $vararg_buffer4 = 0, $vararg_buffer9 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0;
 var $vararg_ptr24 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer20 = sp + 96|0;
 $vararg_buffer16 = sp + 88|0;
 $vararg_buffer14 = sp + 80|0;
 $vararg_buffer11 = sp + 72|0;
 $vararg_buffer9 = sp + 64|0;
 $vararg_buffer4 = sp + 48|0;
 $vararg_buffer1 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $2 = sp + 240|0;
 $3 = sp + 236|0;
 $4 = sp + 232|0;
 $5 = sp + 228|0;
 $6 = sp + 24|0;
 $pPage = sp + 224|0;
 $i = sp + 220|0;
 $rc = sp + 216|0;
 $depth = sp + 212|0;
 $d2 = sp + 208|0;
 $pgno = sp + 204|0;
 $nFrag = sp + 200|0;
 $hdr = sp + 196|0;
 $cellStart = sp + 192|0;
 $nCell = sp + 188|0;
 $doCoverageCheck = sp + 184|0;
 $keyCanBeEqual = sp + 180|0;
 $data = sp + 176|0;
 $pCell = sp + 172|0;
 $pCellIdx = sp + 168|0;
 $pBt = sp + 164|0;
 $pc = sp + 160|0;
 $usableSize = sp + 156|0;
 $contentOffset = sp + 152|0;
 $heap = sp + 148|0;
 $x = sp + 144|0;
 $prev = sp + 140|0;
 $saved_zPfx = sp + 136|0;
 $saved_v1 = sp + 132|0;
 $saved_v2 = sp + 128|0;
 $savedIsInit = sp + 244|0;
 $info = sp;
 $nPage = sp + 124|0;
 $pgnoOvfl = sp + 120|0;
 $size = sp + 116|0;
 $size1 = sp + 112|0;
 $j = sp + 108|0;
 HEAP32[$3>>2] = $pCheck;
 HEAP32[$4>>2] = $iPage;
 HEAP32[$5>>2] = $piMinKey;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$pPage>>2] = 0;
 HEAP32[$depth>>2] = -1;
 HEAP32[$doCoverageCheck>>2] = 1;
 HEAP32[$keyCanBeEqual>>2] = 1;
 HEAP32[$heap>>2] = 0;
 HEAP32[$prev>>2] = 0;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$saved_zPfx>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($14)) + 32|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$saved_v1>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$saved_v2>>2] = $19;
 HEAP8[$savedIsInit>>0] = 0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pBt>>2] = $21;
 $22 = HEAP32[$pBt>>2]|0;
 $23 = ((($22)) + 36|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$usableSize>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = ($25|0)==(0);
 if ($26) {
  HEAP32[$2>>2] = 0;
  $500 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($500|0);
 }
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$4>>2]|0;
 $29 = (_checkRef($27,$28)|0);
 $30 = ($29|0)!=(0);
 if ($30) {
  HEAP32[$2>>2] = 0;
  $500 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($500|0);
 }
 $31 = HEAP32[$3>>2]|0;
 $32 = ((($31)) + 28|0);
 HEAP32[$32>>2] = 22167;
 $33 = HEAP32[$4>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = ((($34)) + 32|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pBt>>2]|0;
 $37 = HEAP32[$4>>2]|0;
 $38 = (_btreeGetPage($36,$37,$pPage,0)|0);
 HEAP32[$rc>>2] = $38;
 $39 = ($38|0)!=(0);
 do {
  if ($39) {
   $40 = HEAP32[$3>>2]|0;
   $41 = HEAP32[$rc>>2]|0;
   HEAP32[$vararg_buffer>>2] = $41;
   _checkAppendMsg($40,22177,$vararg_buffer);
  } else {
   $42 = HEAP32[$pPage>>2]|0;
   $43 = HEAP8[$42>>0]|0;
   HEAP8[$savedIsInit>>0] = $43;
   $44 = HEAP32[$pPage>>2]|0;
   HEAP8[$44>>0] = 0;
   $45 = HEAP32[$pPage>>2]|0;
   $46 = (_btreeInitPage($45)|0);
   HEAP32[$rc>>2] = $46;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$3>>2]|0;
    $49 = HEAP32[$rc>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $49;
    _checkAppendMsg($48,22215,$vararg_buffer1);
    break;
   }
   $50 = HEAP32[$pPage>>2]|0;
   $51 = ((($50)) + 56|0);
   $52 = HEAP32[$51>>2]|0;
   HEAP32[$data>>2] = $52;
   $53 = HEAP32[$pPage>>2]|0;
   $54 = ((($53)) + 5|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55&255;
   HEAP32[$hdr>>2] = $56;
   $57 = HEAP32[$3>>2]|0;
   $58 = ((($57)) + 28|0);
   HEAP32[$58>>2] = 22253;
   $59 = HEAP32[$hdr>>2]|0;
   $60 = (($59) + 5)|0;
   $61 = HEAP32[$data>>2]|0;
   $62 = (($61) + ($60)|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = $63&255;
   $65 = $64 << 8;
   $66 = HEAP32[$hdr>>2]|0;
   $67 = (($66) + 5)|0;
   $68 = HEAP32[$data>>2]|0;
   $69 = (($68) + ($67)|0);
   $70 = ((($69)) + 1|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = $65 | $72;
   $74 = (($73) - 1)|0;
   $75 = $74 & 65535;
   $76 = (($75) + 1)|0;
   HEAP32[$contentOffset>>2] = $76;
   $77 = HEAP32[$hdr>>2]|0;
   $78 = (($77) + 3)|0;
   $79 = HEAP32[$data>>2]|0;
   $80 = (($79) + ($78)|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = $81&255;
   $83 = $82 << 8;
   $84 = HEAP32[$hdr>>2]|0;
   $85 = (($84) + 3)|0;
   $86 = HEAP32[$data>>2]|0;
   $87 = (($86) + ($85)|0);
   $88 = ((($87)) + 1|0);
   $89 = HEAP8[$88>>0]|0;
   $90 = $89&255;
   $91 = $83 | $90;
   HEAP32[$nCell>>2] = $91;
   $92 = HEAP32[$hdr>>2]|0;
   $93 = (($92) + 12)|0;
   $94 = HEAP32[$pPage>>2]|0;
   $95 = ((($94)) + 4|0);
   $96 = HEAP8[$95>>0]|0;
   $97 = $96&255;
   $98 = $97<<2;
   $99 = (($93) - ($98))|0;
   HEAP32[$cellStart>>2] = $99;
   $100 = HEAP32[$cellStart>>2]|0;
   $101 = HEAP32[$nCell>>2]|0;
   $102 = (($101) - 1)|0;
   $103 = $102<<1;
   $104 = (($100) + ($103))|0;
   $105 = HEAP32[$data>>2]|0;
   $106 = (($105) + ($104)|0);
   HEAP32[$pCellIdx>>2] = $106;
   $107 = HEAP32[$pPage>>2]|0;
   $108 = ((($107)) + 4|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = ($109<<24>>24)!=(0);
   if ($110) {
    $134 = HEAP32[$3>>2]|0;
    $135 = ((($134)) + 68|0);
    $136 = HEAP32[$135>>2]|0;
    HEAP32[$heap>>2] = $136;
    $137 = HEAP32[$heap>>2]|0;
    HEAP32[$137>>2] = 0;
   } else {
    $111 = HEAP32[$hdr>>2]|0;
    $112 = (($111) + 8)|0;
    $113 = HEAP32[$data>>2]|0;
    $114 = (($113) + ($112)|0);
    $115 = (_sqlite3Get4byte($114)|0);
    HEAP32[$pgno>>2] = $115;
    $116 = HEAP32[$pBt>>2]|0;
    $117 = ((($116)) + 17|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = ($118<<24>>24)!=(0);
    if ($119) {
     $120 = HEAP32[$3>>2]|0;
     $121 = ((($120)) + 28|0);
     HEAP32[$121>>2] = 22279;
     $122 = HEAP32[$3>>2]|0;
     $123 = HEAP32[$pgno>>2]|0;
     $124 = HEAP32[$4>>2]|0;
     _checkPtrmap($122,$123,5,$124);
    }
    $125 = HEAP32[$3>>2]|0;
    $126 = HEAP32[$pgno>>2]|0;
    $127 = $6;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = (_checkTreePage($125,$126,$6,$129,$132)|0);
    HEAP32[$depth>>2] = $133;
    HEAP32[$keyCanBeEqual>>2] = 0;
   }
   $138 = HEAP32[$nCell>>2]|0;
   $139 = (($138) - 1)|0;
   HEAP32[$i>>2] = $139;
   while(1) {
    $140 = HEAP32[$i>>2]|0;
    $141 = ($140|0)>=(0);
    if (!($141)) {
     break;
    }
    $142 = HEAP32[$3>>2]|0;
    $143 = ((($142)) + 16|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = ($144|0)!=(0);
    if (!($145)) {
     break;
    }
    $146 = HEAP32[$i>>2]|0;
    $147 = HEAP32[$3>>2]|0;
    $148 = ((($147)) + 36|0);
    HEAP32[$148>>2] = $146;
    $149 = HEAP32[$pCellIdx>>2]|0;
    $150 = HEAP8[$149>>0]|0;
    $151 = $150&255;
    $152 = $151 << 8;
    $153 = HEAP32[$pCellIdx>>2]|0;
    $154 = ((($153)) + 1|0);
    $155 = HEAP8[$154>>0]|0;
    $156 = $155&255;
    $157 = $152 | $156;
    HEAP32[$pc>>2] = $157;
    $158 = HEAP32[$pCellIdx>>2]|0;
    $159 = ((($158)) + -2|0);
    HEAP32[$pCellIdx>>2] = $159;
    $160 = HEAP32[$pc>>2]|0;
    $161 = HEAP32[$contentOffset>>2]|0;
    $162 = ($160>>>0)<($161>>>0);
    do {
     if ($162) {
      label = 19;
     } else {
      $163 = HEAP32[$pc>>2]|0;
      $164 = HEAP32[$usableSize>>2]|0;
      $165 = (($164) - 4)|0;
      $166 = ($163>>>0)>($165>>>0);
      if ($166) {
       label = 19;
      } else {
       $172 = HEAP32[$pc>>2]|0;
       $173 = HEAP32[$data>>2]|0;
       $174 = (($173) + ($172)|0);
       HEAP32[$pCell>>2] = $174;
       $175 = HEAP32[$pPage>>2]|0;
       $176 = ((($175)) + 80|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = HEAP32[$pPage>>2]|0;
       $179 = HEAP32[$pCell>>2]|0;
       FUNCTION_TABLE_viii[$177 & 255]($178,$179,$info);
       $180 = HEAP32[$pc>>2]|0;
       $181 = ((($info)) + 18|0);
       $182 = HEAP16[$181>>1]|0;
       $183 = $182&65535;
       $184 = (($180) + ($183))|0;
       $185 = HEAP32[$usableSize>>2]|0;
       $186 = ($184>>>0)>($185>>>0);
       if ($186) {
        $187 = HEAP32[$3>>2]|0;
        _checkAppendMsg($187,22337,$vararg_buffer9);
        HEAP32[$doCoverageCheck>>2] = 0;
        break;
       }
       $188 = HEAP32[$pPage>>2]|0;
       $189 = ((($188)) + 2|0);
       $190 = HEAP8[$189>>0]|0;
       $191 = ($190<<24>>24)!=(0);
       if ($191) {
        $192 = HEAP32[$keyCanBeEqual>>2]|0;
        $193 = ($192|0)!=(0);
        $194 = $info;
        $195 = $194;
        $196 = HEAP32[$195>>2]|0;
        $197 = (($194) + 4)|0;
        $198 = $197;
        $199 = HEAP32[$198>>2]|0;
        $200 = $6;
        $201 = $200;
        $202 = HEAP32[$201>>2]|0;
        $203 = (($200) + 4)|0;
        $204 = $203;
        $205 = HEAP32[$204>>2]|0;
        if ($193) {
         $206 = ($199|0)>($205|0);
         $207 = ($196>>>0)>($202>>>0);
         $208 = ($199|0)==($205|0);
         $209 = $208 & $207;
         $210 = $206 | $209;
         if ($210) {
          label = 26;
         }
        } else {
         $211 = ($199|0)>($205|0);
         $212 = ($196>>>0)>=($202>>>0);
         $213 = ($199|0)==($205|0);
         $214 = $213 & $212;
         $215 = $211 | $214;
         if ($215) {
          label = 26;
         }
        }
        if ((label|0) == 26) {
         label = 0;
         $216 = HEAP32[$3>>2]|0;
         $217 = $info;
         $218 = $217;
         $219 = HEAP32[$218>>2]|0;
         $220 = (($217) + 4)|0;
         $221 = $220;
         $222 = HEAP32[$221>>2]|0;
         $223 = $vararg_buffer11;
         $224 = $223;
         HEAP32[$224>>2] = $219;
         $225 = (($223) + 4)|0;
         $226 = $225;
         HEAP32[$226>>2] = $222;
         _checkAppendMsg($216,22361,$vararg_buffer11);
        }
        $227 = $info;
        $228 = $227;
        $229 = HEAP32[$228>>2]|0;
        $230 = (($227) + 4)|0;
        $231 = $230;
        $232 = HEAP32[$231>>2]|0;
        $233 = $6;
        $234 = $233;
        HEAP32[$234>>2] = $229;
        $235 = (($233) + 4)|0;
        $236 = $235;
        HEAP32[$236>>2] = $232;
       }
       $237 = ((($info)) + 12|0);
       $238 = HEAP32[$237>>2]|0;
       $239 = ((($info)) + 16|0);
       $240 = HEAP16[$239>>1]|0;
       $241 = $240&65535;
       $242 = ($238>>>0)>($241>>>0);
       if ($242) {
        $243 = ((($info)) + 12|0);
        $244 = HEAP32[$243>>2]|0;
        $245 = ((($info)) + 16|0);
        $246 = HEAP16[$245>>1]|0;
        $247 = $246&65535;
        $248 = (($244) - ($247))|0;
        $249 = HEAP32[$usableSize>>2]|0;
        $250 = (($248) + ($249))|0;
        $251 = (($250) - 5)|0;
        $252 = HEAP32[$usableSize>>2]|0;
        $253 = (($252) - 4)|0;
        $254 = (($251>>>0) / ($253>>>0))&-1;
        HEAP32[$nPage>>2] = $254;
        $255 = ((($info)) + 18|0);
        $256 = HEAP16[$255>>1]|0;
        $257 = $256&65535;
        $258 = (($257) - 4)|0;
        $259 = HEAP32[$pCell>>2]|0;
        $260 = (($259) + ($258)|0);
        $261 = (_sqlite3Get4byte($260)|0);
        HEAP32[$pgnoOvfl>>2] = $261;
        $262 = HEAP32[$pBt>>2]|0;
        $263 = ((($262)) + 17|0);
        $264 = HEAP8[$263>>0]|0;
        $265 = ($264<<24>>24)!=(0);
        if ($265) {
         $266 = HEAP32[$3>>2]|0;
         $267 = HEAP32[$pgnoOvfl>>2]|0;
         $268 = HEAP32[$4>>2]|0;
         _checkPtrmap($266,$267,3,$268);
        }
        $269 = HEAP32[$3>>2]|0;
        $270 = HEAP32[$pgnoOvfl>>2]|0;
        $271 = HEAP32[$nPage>>2]|0;
        _checkList($269,0,$270,$271);
       }
       $272 = HEAP32[$pPage>>2]|0;
       $273 = ((($272)) + 4|0);
       $274 = HEAP8[$273>>0]|0;
       $275 = ($274<<24>>24)!=(0);
       if ($275) {
        $299 = HEAP32[$heap>>2]|0;
        $300 = HEAP32[$pc>>2]|0;
        $301 = $300 << 16;
        $302 = HEAP32[$pc>>2]|0;
        $303 = ((($info)) + 18|0);
        $304 = HEAP16[$303>>1]|0;
        $305 = $304&65535;
        $306 = (($302) + ($305))|0;
        $307 = (($306) - 1)|0;
        $308 = $301 | $307;
        _btreeHeapInsert($299,$308);
        break;
       }
       $276 = HEAP32[$pCell>>2]|0;
       $277 = (_sqlite3Get4byte($276)|0);
       HEAP32[$pgno>>2] = $277;
       $278 = HEAP32[$pBt>>2]|0;
       $279 = ((($278)) + 17|0);
       $280 = HEAP8[$279>>0]|0;
       $281 = ($280<<24>>24)!=(0);
       if ($281) {
        $282 = HEAP32[$3>>2]|0;
        $283 = HEAP32[$pgno>>2]|0;
        $284 = HEAP32[$4>>2]|0;
        _checkPtrmap($282,$283,5,$284);
       }
       $285 = HEAP32[$3>>2]|0;
       $286 = HEAP32[$pgno>>2]|0;
       $287 = $6;
       $288 = $287;
       $289 = HEAP32[$288>>2]|0;
       $290 = (($287) + 4)|0;
       $291 = $290;
       $292 = HEAP32[$291>>2]|0;
       $293 = (_checkTreePage($285,$286,$6,$289,$292)|0);
       HEAP32[$d2>>2] = $293;
       HEAP32[$keyCanBeEqual>>2] = 0;
       $294 = HEAP32[$d2>>2]|0;
       $295 = HEAP32[$depth>>2]|0;
       $296 = ($294|0)!=($295|0);
       if ($296) {
        $297 = HEAP32[$3>>2]|0;
        _checkAppendMsg($297,22385,$vararg_buffer14);
        $298 = HEAP32[$d2>>2]|0;
        HEAP32[$depth>>2] = $298;
       }
      }
     }
    } while(0);
    if ((label|0) == 19) {
     label = 0;
     $167 = HEAP32[$3>>2]|0;
     $168 = HEAP32[$pc>>2]|0;
     $169 = HEAP32[$contentOffset>>2]|0;
     $170 = HEAP32[$usableSize>>2]|0;
     $171 = (($170) - 4)|0;
     HEAP32[$vararg_buffer4>>2] = $168;
     $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $169;
     $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
     HEAP32[$vararg_ptr8>>2] = $171;
     _checkAppendMsg($167,22307,$vararg_buffer4);
     HEAP32[$doCoverageCheck>>2] = 0;
    }
    $309 = HEAP32[$i>>2]|0;
    $310 = (($309) + -1)|0;
    HEAP32[$i>>2] = $310;
   }
   $311 = $6;
   $312 = $311;
   $313 = HEAP32[$312>>2]|0;
   $314 = (($311) + 4)|0;
   $315 = $314;
   $316 = HEAP32[$315>>2]|0;
   $317 = HEAP32[$5>>2]|0;
   $318 = $317;
   $319 = $318;
   HEAP32[$319>>2] = $313;
   $320 = (($318) + 4)|0;
   $321 = $320;
   HEAP32[$321>>2] = $316;
   $322 = HEAP32[$3>>2]|0;
   $323 = ((($322)) + 28|0);
   HEAP32[$323>>2] = 0;
   $324 = HEAP32[$doCoverageCheck>>2]|0;
   $325 = ($324|0)!=(0);
   if ($325) {
    $326 = HEAP32[$3>>2]|0;
    $327 = ((($326)) + 16|0);
    $328 = HEAP32[$327>>2]|0;
    $329 = ($328|0)>(0);
    if ($329) {
     $330 = HEAP32[$pPage>>2]|0;
     $331 = ((($330)) + 4|0);
     $332 = HEAP8[$331>>0]|0;
     $333 = ($332<<24>>24)!=(0);
     L60: do {
      if (!($333)) {
       $334 = HEAP32[$3>>2]|0;
       $335 = ((($334)) + 68|0);
       $336 = HEAP32[$335>>2]|0;
       HEAP32[$heap>>2] = $336;
       $337 = HEAP32[$heap>>2]|0;
       HEAP32[$337>>2] = 0;
       $338 = HEAP32[$nCell>>2]|0;
       $339 = (($338) - 1)|0;
       HEAP32[$i>>2] = $339;
       while(1) {
        $340 = HEAP32[$i>>2]|0;
        $341 = ($340|0)>=(0);
        if (!($341)) {
         break L60;
        }
        $342 = HEAP32[$cellStart>>2]|0;
        $343 = HEAP32[$i>>2]|0;
        $344 = $343<<1;
        $345 = (($342) + ($344))|0;
        $346 = HEAP32[$data>>2]|0;
        $347 = (($346) + ($345)|0);
        $348 = HEAP8[$347>>0]|0;
        $349 = $348&255;
        $350 = $349 << 8;
        $351 = HEAP32[$cellStart>>2]|0;
        $352 = HEAP32[$i>>2]|0;
        $353 = $352<<1;
        $354 = (($351) + ($353))|0;
        $355 = HEAP32[$data>>2]|0;
        $356 = (($355) + ($354)|0);
        $357 = ((($356)) + 1|0);
        $358 = HEAP8[$357>>0]|0;
        $359 = $358&255;
        $360 = $350 | $359;
        HEAP32[$pc>>2] = $360;
        $361 = HEAP32[$pPage>>2]|0;
        $362 = ((($361)) + 76|0);
        $363 = HEAP32[$362>>2]|0;
        $364 = HEAP32[$pPage>>2]|0;
        $365 = HEAP32[$pc>>2]|0;
        $366 = HEAP32[$data>>2]|0;
        $367 = (($366) + ($365)|0);
        $368 = (FUNCTION_TABLE_iii[$363 & 255]($364,$367)|0);
        $369 = $368&65535;
        HEAP32[$size>>2] = $369;
        $370 = HEAP32[$heap>>2]|0;
        $371 = HEAP32[$pc>>2]|0;
        $372 = $371 << 16;
        $373 = HEAP32[$pc>>2]|0;
        $374 = HEAP32[$size>>2]|0;
        $375 = (($373) + ($374))|0;
        $376 = (($375) - 1)|0;
        $377 = $372 | $376;
        _btreeHeapInsert($370,$377);
        $378 = HEAP32[$i>>2]|0;
        $379 = (($378) + -1)|0;
        HEAP32[$i>>2] = $379;
       }
      }
     } while(0);
     $380 = HEAP32[$hdr>>2]|0;
     $381 = (($380) + 1)|0;
     $382 = HEAP32[$data>>2]|0;
     $383 = (($382) + ($381)|0);
     $384 = HEAP8[$383>>0]|0;
     $385 = $384&255;
     $386 = $385 << 8;
     $387 = HEAP32[$hdr>>2]|0;
     $388 = (($387) + 1)|0;
     $389 = HEAP32[$data>>2]|0;
     $390 = (($389) + ($388)|0);
     $391 = ((($390)) + 1|0);
     $392 = HEAP8[$391>>0]|0;
     $393 = $392&255;
     $394 = $386 | $393;
     HEAP32[$i>>2] = $394;
     while(1) {
      $395 = HEAP32[$i>>2]|0;
      $396 = ($395|0)>(0);
      if (!($396)) {
       break;
      }
      $397 = HEAP32[$i>>2]|0;
      $398 = (($397) + 2)|0;
      $399 = HEAP32[$data>>2]|0;
      $400 = (($399) + ($398)|0);
      $401 = HEAP8[$400>>0]|0;
      $402 = $401&255;
      $403 = $402 << 8;
      $404 = HEAP32[$i>>2]|0;
      $405 = (($404) + 2)|0;
      $406 = HEAP32[$data>>2]|0;
      $407 = (($406) + ($405)|0);
      $408 = ((($407)) + 1|0);
      $409 = HEAP8[$408>>0]|0;
      $410 = $409&255;
      $411 = $403 | $410;
      HEAP32[$size1>>2] = $411;
      $412 = HEAP32[$heap>>2]|0;
      $413 = HEAP32[$i>>2]|0;
      $414 = $413 << 16;
      $415 = HEAP32[$i>>2]|0;
      $416 = HEAP32[$size1>>2]|0;
      $417 = (($415) + ($416))|0;
      $418 = (($417) - 1)|0;
      $419 = $414 | $418;
      _btreeHeapInsert($412,$419);
      $420 = HEAP32[$i>>2]|0;
      $421 = HEAP32[$data>>2]|0;
      $422 = (($421) + ($420)|0);
      $423 = HEAP8[$422>>0]|0;
      $424 = $423&255;
      $425 = $424 << 8;
      $426 = HEAP32[$i>>2]|0;
      $427 = HEAP32[$data>>2]|0;
      $428 = (($427) + ($426)|0);
      $429 = ((($428)) + 1|0);
      $430 = HEAP8[$429>>0]|0;
      $431 = $430&255;
      $432 = $425 | $431;
      HEAP32[$j>>2] = $432;
      $433 = HEAP32[$j>>2]|0;
      HEAP32[$i>>2] = $433;
     }
     HEAP32[$nFrag>>2] = 0;
     $434 = HEAP32[$contentOffset>>2]|0;
     $435 = (($434) - 1)|0;
     HEAP32[$prev>>2] = $435;
     while(1) {
      $436 = HEAP32[$heap>>2]|0;
      $437 = (_btreeHeapPull($436,$x)|0);
      $438 = ($437|0)!=(0);
      if (!($438)) {
       break;
      }
      $439 = HEAP32[$prev>>2]|0;
      $440 = $439 & 65535;
      $441 = HEAP32[$x>>2]|0;
      $442 = $441 >>> 16;
      $443 = ($440>>>0)>=($442>>>0);
      if ($443) {
       label = 51;
       break;
      }
      $448 = HEAP32[$x>>2]|0;
      $449 = $448 >>> 16;
      $450 = HEAP32[$prev>>2]|0;
      $451 = $450 & 65535;
      $452 = (($449) - ($451))|0;
      $453 = (($452) - 1)|0;
      $454 = HEAP32[$nFrag>>2]|0;
      $455 = (($454) + ($453))|0;
      HEAP32[$nFrag>>2] = $455;
      $456 = HEAP32[$x>>2]|0;
      HEAP32[$prev>>2] = $456;
     }
     if ((label|0) == 51) {
      $444 = HEAP32[$3>>2]|0;
      $445 = HEAP32[$x>>2]|0;
      $446 = $445 >>> 16;
      $447 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer16>>2] = $446;
      $vararg_ptr19 = ((($vararg_buffer16)) + 4|0);
      HEAP32[$vararg_ptr19>>2] = $447;
      _checkAppendMsg($444,22410,$vararg_buffer16);
     }
     $457 = HEAP32[$usableSize>>2]|0;
     $458 = HEAP32[$prev>>2]|0;
     $459 = $458 & 65535;
     $460 = (($457) - ($459))|0;
     $461 = (($460) - 1)|0;
     $462 = HEAP32[$nFrag>>2]|0;
     $463 = (($462) + ($461))|0;
     HEAP32[$nFrag>>2] = $463;
     $464 = HEAP32[$heap>>2]|0;
     $465 = HEAP32[$464>>2]|0;
     $466 = ($465|0)==(0);
     if ($466) {
      $467 = HEAP32[$nFrag>>2]|0;
      $468 = HEAP32[$hdr>>2]|0;
      $469 = (($468) + 7)|0;
      $470 = HEAP32[$data>>2]|0;
      $471 = (($470) + ($469)|0);
      $472 = HEAP8[$471>>0]|0;
      $473 = $472&255;
      $474 = ($467|0)!=($473|0);
      if ($474) {
       $475 = HEAP32[$3>>2]|0;
       $476 = HEAP32[$nFrag>>2]|0;
       $477 = HEAP32[$hdr>>2]|0;
       $478 = (($477) + 7)|0;
       $479 = HEAP32[$data>>2]|0;
       $480 = (($479) + ($478)|0);
       $481 = HEAP8[$480>>0]|0;
       $482 = $481&255;
       $483 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer20>>2] = $476;
       $vararg_ptr23 = ((($vararg_buffer20)) + 4|0);
       HEAP32[$vararg_ptr23>>2] = $482;
       $vararg_ptr24 = ((($vararg_buffer20)) + 8|0);
       HEAP32[$vararg_ptr24>>2] = $483;
       _checkAppendMsg($475,22447,$vararg_buffer20);
      }
     }
    }
   }
  }
 } while(0);
 $484 = HEAP32[$doCoverageCheck>>2]|0;
 $485 = ($484|0)!=(0);
 if (!($485)) {
  $486 = HEAP8[$savedIsInit>>0]|0;
  $487 = HEAP32[$pPage>>2]|0;
  HEAP8[$487>>0] = $486;
 }
 $488 = HEAP32[$pPage>>2]|0;
 _releasePage($488);
 $489 = HEAP32[$saved_zPfx>>2]|0;
 $490 = HEAP32[$3>>2]|0;
 $491 = ((($490)) + 28|0);
 HEAP32[$491>>2] = $489;
 $492 = HEAP32[$saved_v1>>2]|0;
 $493 = HEAP32[$3>>2]|0;
 $494 = ((($493)) + 32|0);
 HEAP32[$494>>2] = $492;
 $495 = HEAP32[$saved_v2>>2]|0;
 $496 = HEAP32[$3>>2]|0;
 $497 = ((($496)) + 36|0);
 HEAP32[$497>>2] = $495;
 $498 = HEAP32[$depth>>2]|0;
 $499 = (($498) + 1)|0;
 HEAP32[$2>>2] = $499;
 $500 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($500|0);
}
function _getPageReferenced($pCheck,$iPg) {
 $pCheck = $pCheck|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2>>>0) / 8)&-1;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = HEAP32[$1>>2]|0;
 $11 = $10 & 7;
 $12 = 1 << $11;
 $13 = $9 & $12;
 STACKTOP = sp;return ($13|0);
}
function _checkAppendMsg($pCheck,$zFormat,$varargs) {
 $pCheck = $pCheck|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$11>>2] = $13;
 HEAP32[$ap>>2] = $varargs;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 40|0);
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 40|0);
  _sqlite3StrAccumAppend($20,22499,1);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 28|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 40|0);
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 28|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 36|0);
  $35 = HEAP32[$34>>2]|0;
  HEAP32[$vararg_buffer>>2] = $32;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $35;
  _sqlite3XPrintf($26,$29,$vararg_buffer);
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 40|0);
 $38 = HEAP32[$1>>2]|0;
 _sqlite3VXPrintf($37,$38,$ap);
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 40|0);
 $41 = ((($40)) + 24|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = ($43|0)==(1);
 if (!($44)) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 24|0);
 HEAP32[$46>>2] = 1;
 STACKTOP = sp;return;
}
function _checkRef($pCheck,$iPage) {
 $pCheck = $pCheck|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 HEAP32[$1>>2] = $pCheck;
 HEAP32[$2>>2] = $iPage;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 1;
  } else {
   $5 = HEAP32[$2>>2]|0;
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 12|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = ($5>>>0)>($8>>>0);
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   if ($9) {
    HEAP32[$vararg_buffer>>2] = $11;
    _checkAppendMsg($10,22036,$vararg_buffer);
    HEAP32[$0>>2] = 1;
    break;
   }
   $12 = (_getPageReferenced($10,$11)|0);
   $13 = ($12|0)!=(0);
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   if ($13) {
    HEAP32[$vararg_buffer1>>2] = $15;
    _checkAppendMsg($14,22059,$vararg_buffer1);
    HEAP32[$0>>2] = 1;
    break;
   } else {
    _setPageReferenced($14,$15);
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _btreeHeapInsert($aHeap,$x) {
 $aHeap = $aHeap|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $j = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $aHeap;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$i>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + ($6<<2)|0);
 HEAP32[$8>>2] = $5;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = (($9>>>0) / 2)&-1;
  HEAP32[$j>>2] = $10;
  $11 = ($10>>>0)>(0);
  if (!($11)) {
   label = 5;
   break;
  }
  $12 = HEAP32[$j>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + ($12<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($15>>>0)>($19>>>0);
  if (!($20)) {
   label = 5;
   break;
  }
  $21 = HEAP32[$j>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + ($21<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$1>>2] = $24;
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + ($25<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$j>>2]|0;
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + ($29<<2)|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + ($33<<2)|0);
  HEAP32[$35>>2] = $32;
  $36 = HEAP32[$j>>2]|0;
  HEAP32[$i>>2] = $36;
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _btreeHeapPull($aHeap,$pOut) {
 $aHeap = $aHeap|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $j = sp + 8|0;
 $i = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $aHeap;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$x>>2] = $4;
 $5 = ($4|0)==(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$x>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + ($10<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$x>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16<<2)|0);
 HEAP32[$18>>2] = -1;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + -1)|0;
 HEAP32[$19>>2] = $21;
 HEAP32[$i>>2] = 1;
 while(1) {
  $22 = HEAP32[$i>>2]|0;
  $23 = $22<<1;
  HEAP32[$j>>2] = $23;
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($23>>>0)<=($25>>>0);
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$j>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$j>>2]|0;
  $32 = (($31) + 1)|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($30>>>0)>($35>>>0);
  if ($36) {
   $37 = HEAP32[$j>>2]|0;
   $38 = (($37) + 1)|0;
   HEAP32[$j>>2] = $38;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + ($39<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$j>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + ($43<<2)|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($42>>>0)<($46>>>0);
  if ($47) {
   break;
  }
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = (($49) + ($48<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$x>>2] = $51;
  $52 = HEAP32[$j>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = (($53) + ($52<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + ($56<<2)|0);
  HEAP32[$58>>2] = $55;
  $59 = HEAP32[$x>>2]|0;
  $60 = HEAP32[$j>>2]|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = (($61) + ($60<<2)|0);
  HEAP32[$62>>2] = $59;
  $63 = HEAP32[$j>>2]|0;
  HEAP32[$i>>2] = $63;
 }
 HEAP32[$0>>2] = 1;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3RowSetInit($db,$pSpace,$N) {
 $db = $db|0;
 $pSpace = $pSpace|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSpace;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 12|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 20|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 32|0);
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 16|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) - 32)|0;
 $20 = (($19>>>0) / 16)&-1;
 $21 = $20&65535;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 24|0);
 HEAP16[$23>>1] = $21;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 26|0);
 HEAP16[$25>>1] = 1;
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 28|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _rowSetEntryAlloc($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = (_sqlite3DbMallocRawNN($9,1016,0)|0);
   HEAP32[$pNew>>2] = $10;
   $11 = HEAP32[$pNew>>2]|0;
   $12 = ($11|0)==(0|0);
   if (!($12)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$pNew>>2]|0;
    HEAP32[$15>>2] = $14;
    $16 = HEAP32[$pNew>>2]|0;
    $17 = HEAP32[$1>>2]|0;
    HEAP32[$17>>2] = $16;
    $18 = HEAP32[$pNew>>2]|0;
    $19 = ((($18)) + 8|0);
    $20 = HEAP32[$1>>2]|0;
    $21 = ((($20)) + 16|0);
    HEAP32[$21>>2] = $19;
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 24|0);
    HEAP16[$23>>1] = 63;
    break;
   }
   HEAP32[$0>>2] = 0;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = (($26) + -1)<<16>>16;
 HEAP16[$25>>1] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ((($30)) + 16|0);
 HEAP32[$29>>2] = $31;
 HEAP32[$0>>2] = $30;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _rowSetToList($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 1;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_rowSetEntrySort($9)|0);
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  HEAP32[$12>>2] = $10;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 26|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 | 2;
 $18 = $17&65535;
 HEAP16[$14>>1] = $18;
 STACKTOP = sp;return;
}
function _rowSetEntrySort($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aBucket = 0, $i = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 168|0;
 $i = sp + 164|0;
 $pNext = sp + 160|0;
 $aBucket = sp;
 HEAP32[$0>>2] = $pIn;
 _memset(($aBucket|0),0,160)|0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if (!($2)) {
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 8|0);
  $5 = HEAP32[$4>>2]|0;
  HEAP32[$pNext>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 8|0);
  HEAP32[$7>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $8 = HEAP32[$i>>2]|0;
   $9 = (($aBucket) + ($8<<2)|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if (!($11)) {
    break;
   }
   $12 = HEAP32[$i>>2]|0;
   $13 = (($aBucket) + ($12<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = (_rowSetEntryMerge($14,$15)|0);
   HEAP32[$0>>2] = $16;
   $17 = HEAP32[$i>>2]|0;
   $18 = (($aBucket) + ($17<<2)|0);
   HEAP32[$18>>2] = 0;
   $19 = HEAP32[$i>>2]|0;
   $20 = (($19) + 1)|0;
   HEAP32[$i>>2] = $20;
  }
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($aBucket) + ($22<<2)|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$pNext>>2]|0;
  HEAP32[$0>>2] = $24;
 }
 HEAP32[$0>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = ($25>>>0)<(40);
  $27 = HEAP32[$0>>2]|0;
  if (!($26)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = (($aBucket) + ($28<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_rowSetEntryMerge($27,$30)|0);
  HEAP32[$0>>2] = $31;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 STACKTOP = sp;return ($27|0);
}
function _rowSetEntryMerge($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $head = 0, $pTail = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $head = sp;
 $pTail = sp + 16|0;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 HEAP32[$pTail>>2] = $head;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  $4 = HEAP32[$1>>2]|0;
  $5 = ($4|0)!=(0|0);
  $6 = $3 ? $5 : 0;
  $7 = HEAP32[$0>>2]|0;
  if (!($6)) {
   break;
  }
  $8 = $7;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($8) + 4)|0;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = $14;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = ($13|0)<($20|0);
  $22 = ($10>>>0)<($17>>>0);
  $23 = ($13|0)==($20|0);
  $24 = $23 & $22;
  $25 = $21 | $24;
  if ($25) {
   $26 = HEAP32[$0>>2]|0;
   $27 = HEAP32[$pTail>>2]|0;
   $28 = ((($27)) + 8|0);
   HEAP32[$28>>2] = $26;
   $29 = HEAP32[$0>>2]|0;
   $30 = ((($29)) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$0>>2] = $31;
   $32 = HEAP32[$pTail>>2]|0;
   $33 = ((($32)) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pTail>>2] = $34;
   continue;
  }
  $35 = HEAP32[$1>>2]|0;
  $36 = $35;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$0>>2]|0;
  $43 = $42;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($41|0)<($48|0);
  $50 = ($38>>>0)<($45>>>0);
  $51 = ($41|0)==($48|0);
  $52 = $51 & $50;
  $53 = $49 | $52;
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = HEAP32[$pTail>>2]|0;
   $56 = ((($55)) + 8|0);
   HEAP32[$56>>2] = $54;
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   HEAP32[$1>>2] = $59;
   $60 = HEAP32[$pTail>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$pTail>>2] = $62;
   continue;
  } else {
   $63 = HEAP32[$0>>2]|0;
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$0>>2] = $65;
   continue;
  }
 }
 $66 = ($7|0)!=(0|0);
 if ($66) {
  $67 = HEAP32[$0>>2]|0;
  $68 = HEAP32[$pTail>>2]|0;
  $69 = ((($68)) + 8|0);
  HEAP32[$69>>2] = $67;
  $73 = ((($head)) + 8|0);
  $74 = HEAP32[$73>>2]|0;
  STACKTOP = sp;return ($74|0);
 } else {
  $70 = HEAP32[$1>>2]|0;
  $71 = HEAP32[$pTail>>2]|0;
  $72 = ((($71)) + 8|0);
  HEAP32[$72>>2] = $70;
  $73 = ((($head)) + 8|0);
  $74 = HEAP32[$73>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 return (0)|0;
}
function _rowSetListToTree($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDepth = 0, $p = 0, $pLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $iDepth = sp + 8|0;
 $p = sp + 4|0;
 $pLeft = sp;
 HEAP32[$0>>2] = $pList;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$0>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 12|0);
 HEAP32[$8>>2] = 0;
 HEAP32[$iDepth>>2] = 1;
 while(1) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$p>>2]|0;
  if (!($10)) {
   break;
  }
  HEAP32[$pLeft>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$pLeft>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 12|0);
  HEAP32[$18>>2] = $16;
  $19 = HEAP32[$iDepth>>2]|0;
  $20 = (_rowSetNDeepTree($0,$19)|0);
  $21 = HEAP32[$p>>2]|0;
  $22 = ((($21)) + 8|0);
  HEAP32[$22>>2] = $20;
  $23 = HEAP32[$iDepth>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$iDepth>>2] = $24;
 }
 STACKTOP = sp;return ($11|0);
}
function _rowSetTreeToList($pIn,$ppFirst,$ppLast) {
 $pIn = $pIn|0;
 $ppFirst = $ppFirst|0;
 $ppLast = $ppLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pIn;
 HEAP32[$1>>2] = $ppFirst;
 HEAP32[$2>>2] = $ppLast;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  _rowSetTreeToList($9,$10,$p);
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 8|0);
  HEAP32[$13>>2] = $11;
 } else {
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$14>>2] = $7;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$0>>2]|0;
 if ($18) {
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = HEAP32[$2>>2]|0;
  _rowSetTreeToList($21,$23,$24);
  STACKTOP = sp;return;
 } else {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$25>>2] = $19;
  STACKTOP = sp;return;
 }
}
function _rowSetNDeepTree($ppList,$iDepth) {
 $ppList = $ppList|0;
 $iDepth = $iDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $p = 0, $pLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pLeft = sp;
 HEAP32[$1>>2] = $ppList;
 HEAP32[$2>>2] = $iDepth;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(1);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$p>>2] = $9;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  HEAP32[$13>>2] = $12;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 8|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$p>>2]|0;
  $17 = ((($16)) + 12|0);
  HEAP32[$17>>2] = 0;
  $18 = HEAP32[$p>>2]|0;
  HEAP32[$0>>2] = $18;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $19 = HEAP32[$2>>2]|0;
 $20 = (($19) - 1)|0;
 $21 = (_rowSetNDeepTree($8,$20)|0);
 HEAP32[$pLeft>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$p>>2] = $23;
 $24 = HEAP32[$p>>2]|0;
 $25 = ($24|0)==(0|0);
 $26 = HEAP32[$pLeft>>2]|0;
 if ($25) {
  HEAP32[$0>>2] = $26;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 } else {
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 12|0);
  HEAP32[$28>>2] = $26;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) - 1)|0;
  $36 = (_rowSetNDeepTree($33,$35)|0);
  $37 = HEAP32[$p>>2]|0;
  $38 = ((($37)) + 8|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$p>>2]|0;
  HEAP32[$0>>2] = $39;
  $40 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 return (0)|0;
}
function _sqlite3SetString($pz,$db,$zNew) {
 $pz = $pz|0;
 $db = $db|0;
 $zNew = $zNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pz;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 _sqlite3DbFree($3,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (_sqlite3DbStrDup($6,$7)|0);
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$9>>2] = $8;
 STACKTOP = sp;return;
}
function _execSql($db,$pzErrMsg,$zSql) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 $zSql = $zSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pStmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pStmt = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pzErrMsg;
 HEAP32[$3>>2] = $zSql;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  HEAP32[$0>>2] = 7;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3_prepare($6,$7,-1,$pStmt,0)|0);
 $9 = (0)!=($8|0);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (_sqlite3_errmsg($12)|0);
  _sqlite3SetString($10,$11,$13);
  $14 = HEAP32[$1>>2]|0;
  $15 = (_sqlite3_errcode($14)|0);
  HEAP32[$0>>2] = $15;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $16 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($16)|0);
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$pStmt>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = (_vacuumFinalize($17,$18,$19)|0);
  HEAP32[$0>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCommit($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3BtreeCommitPhaseOne($2,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $8 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3BtreeCommitPhaseTwo($6,0)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3BtreeGetOptimalReserve($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3BtreeGetReserveNoMutex($2)|0);
 HEAP32[$n>>2] = $3;
 $4 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeSetAutoVacuum($p,$autoVacuum) {
 $p = $p|0;
 $autoVacuum = $autoVacuum|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $av = 0, $pBt = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pBt = sp + 4|0;
 $rc = sp;
 $av = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $autoVacuum;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = $5&255;
 HEAP8[$av>>0] = $6;
 $7 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($7);
 $8 = HEAP32[$pBt>>2]|0;
 $9 = ((($8)) + 22|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP8[$av>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)!=(0);
  $17 = $16 ? 1 : 0;
  $18 = HEAP32[$pBt>>2]|0;
  $19 = ((($18)) + 17|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = ($17|0)!=($21|0);
  if ($22) {
   HEAP32[$rc>>2] = 8;
   $37 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $23 = HEAP8[$av>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)!=(0);
 $26 = $25 ? 1 : 0;
 $27 = $26&255;
 $28 = HEAP32[$pBt>>2]|0;
 $29 = ((($28)) + 17|0);
 HEAP8[$29>>0] = $27;
 $30 = HEAP8[$av>>0]|0;
 $31 = $30&255;
 $32 = ($31|0)==(2);
 $33 = $32 ? 1 : 0;
 $34 = $33&255;
 $35 = HEAP32[$pBt>>2]|0;
 $36 = ((($35)) + 18|0);
 HEAP8[$36>>0] = $34;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3BtreeGetAutoVacuum($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $16 = 0;
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 18|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 $14 = $13 ^ 1;
 $15 = $14 ? 1 : 2;
 $16 = $15;
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _execExecSql($db,$pzErrMsg,$zSql) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 $zSql = $zSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pzErrMsg;
 HEAP32[$3>>2] = $zSql;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_sqlite3_prepare($4,$5,-1,$pStmt,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 while(1) {
  $10 = HEAP32[$pStmt>>2]|0;
  $11 = (_sqlite3_step($10)|0);
  $12 = (100)==($11|0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   label = 6;
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$pStmt>>2]|0;
  $16 = (_sqlite3_column_text($15,0)|0);
  $17 = (_execSql($13,$14,$16)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)!=(0);
  if ($19) {
   label = 5;
   break;
  }
 }
 if ((label|0) == 5) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$pStmt>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  (_vacuumFinalize($20,$21,$22)|0);
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 else if ((label|0) == 6) {
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (_vacuumFinalize($13,$24,$25)|0);
  HEAP32[$0>>2] = $26;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCopyFile($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $nByte = 0, $pFd = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $rc = sp + 60|0;
 $pFd = sp + 56|0;
 $b = sp + 8|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($2);
 $3 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3BtreePager($4)|0);
 $6 = (_sqlite3PagerFile($5)|0);
 HEAP32[$pFd>>2] = $6;
 $7 = HEAP32[$pFd>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_sqlite3BtreeGetPageSize($10)|0);
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = HEAP32[$1>>2]|0;
  $15 = (_sqlite3BtreeLastPage($14)|0);
  $16 = (___muldi3(($11|0),($13|0),($15|0),0)|0);
  $17 = tempRet0;
  $18 = $nByte;
  $19 = $18;
  HEAP32[$19>>2] = $16;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = $17;
  $22 = HEAP32[$pFd>>2]|0;
  $23 = (_sqlite3OsFileControl($22,11,$nByte)|0);
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(12);
  $$ = $25 ? 0 : $23;
  HEAP32[$rc>>2] = $$;
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)!=(0);
  if ($27) {
   $50 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 }
 dest=$b; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ((($b)) + 20|0);
 HEAP32[$30>>2] = $29;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($b)) + 24|0);
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($b)) + 4|0);
 HEAP32[$34>>2] = $33;
 $35 = ((($b)) + 16|0);
 HEAP32[$35>>2] = 1;
 (_sqlite3_backup_step($b,2147483647)|0);
 $36 = (_sqlite3_backup_finish($b)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ((($41)) + 22|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43&65535;
  $45 = $44 & -3;
  $46 = $45&65535;
  HEAP16[$42>>1] = $46;
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 } else {
  $47 = ((($b)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (_sqlite3BtreePager($48)|0);
  _sqlite3PagerClearCache($49);
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 return (0)|0;
}
function _vacuumFinalize($db,$pStmt,$pzErrMsg) {
 $db = $db|0;
 $pStmt = $pStmt|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pStmt;
 HEAP32[$2>>2] = $pzErrMsg;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeFinalize($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3_errmsg($9)|0);
 _sqlite3SetString($7,$8,$10);
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3BtreeGetReserveNoMutex($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 36|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($5) - ($10))|0;
 HEAP32[$n>>2] = $11;
 $12 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3PagerClearCache($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 13|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 _pager_reset($10);
 STACKTOP = sp;return;
}
function _setSharedCacheTableLock($p,$iTable,$eLock) {
 $p = $p|0;
 $iTable = $iTable|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pIter = 0, $pLock = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 24|0;
 $pBt = sp + 8|0;
 $pLock = sp + 4|0;
 $pIter = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP8[$3>>0] = $eLock;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 HEAP32[$pLock>>2] = 0;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 72|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pIter>>2] = $9;
 while(1) {
  $10 = HEAP32[$pIter>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($14|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$pIter>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ($18|0)==($19|0);
   if ($20) {
    label = 5;
    break;
   }
  }
  $22 = HEAP32[$pIter>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pIter>>2] = $24;
 }
 if ((label|0) == 5) {
  $21 = HEAP32[$pIter>>2]|0;
  HEAP32[$pLock>>2] = $21;
 }
 $25 = HEAP32[$pLock>>2]|0;
 $26 = ($25|0)!=(0|0);
 do {
  if (!($26)) {
   $27 = (_sqlite3MallocZero(16,0)|0);
   HEAP32[$pLock>>2] = $27;
   $28 = HEAP32[$pLock>>2]|0;
   $29 = ($28|0)!=(0|0);
   if ($29) {
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$pLock>>2]|0;
    $32 = ((($31)) + 4|0);
    HEAP32[$32>>2] = $30;
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$pLock>>2]|0;
    HEAP32[$34>>2] = $33;
    $35 = HEAP32[$pBt>>2]|0;
    $36 = ((($35)) + 72|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$pLock>>2]|0;
    $39 = ((($38)) + 12|0);
    HEAP32[$39>>2] = $37;
    $40 = HEAP32[$pLock>>2]|0;
    $41 = HEAP32[$pBt>>2]|0;
    $42 = ((($41)) + 72|0);
    HEAP32[$42>>2] = $40;
    break;
   }
   HEAP32[$0>>2] = 7;
   $53 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($53|0);
  }
 } while(0);
 $43 = HEAP8[$3>>0]|0;
 $44 = $43&255;
 $45 = HEAP32[$pLock>>2]|0;
 $46 = ((($45)) + 8|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47&255;
 $49 = ($44|0)>($48|0);
 if ($49) {
  $50 = HEAP8[$3>>0]|0;
  $51 = HEAP32[$pLock>>2]|0;
  $52 = ((($51)) + 8|0);
  HEAP8[$52>>0] = $50;
 }
 HEAP32[$0>>2] = 0;
 $53 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _growVTrans($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ARRAY_INCR = 0, $aVTrans = 0, $nBytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ARRAY_INCR = sp + 8|0;
 $aVTrans = sp + 4|0;
 $nBytes = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$ARRAY_INCR>>2] = 5;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 308|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4|0) % 5)&-1;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 308|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = (($9) + 5)|0;
   $11 = $10<<2;
   HEAP32[$nBytes>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 332|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$nBytes>>2]|0;
   $17 = ($16|0)<(0);
   $18 = $17 << 31 >> 31;
   $19 = (_sqlite3DbRealloc($12,$15,$16,$18)|0);
   HEAP32[$aVTrans>>2] = $19;
   $20 = HEAP32[$aVTrans>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 308|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$aVTrans>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    ;HEAP32[$26>>2]=0|0;HEAP32[$26+4>>2]=0|0;HEAP32[$26+8>>2]=0|0;HEAP32[$26+12>>2]=0|0;HEAP32[$26+16>>2]=0|0;
    $27 = HEAP32[$aVTrans>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 332|0);
    HEAP32[$29>>2] = $27;
    break;
   }
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _addToVTrans($db,$pVTab) {
 $db = $db|0;
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pVTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 308|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 332|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($5<<2)|0);
 HEAP32[$10>>2] = $2;
 $11 = HEAP32[$1>>2]|0;
 _sqlite3VtabLock($11);
 STACKTOP = sp;return;
}
function _sqlite3VtabLock($pVTab) {
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _vtabCallConstructor($db,$pTab,$pMod,$xConstruct,$pzErr) {
 $db = $db|0;
 $pTab = $pTab|0;
 $pMod = $pMod|0;
 $xConstruct = $xConstruct|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $azArg = 0, $i = 0, $iCol = 0, $iDb = 0, $j = 0, $nArg = 0, $nDel = 0, $nType = 0, $oooHidden = 0, $or$cond = 0, $pCtx = 0, $pVTable = 0, $rc = 0, $sCtx = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer2 = 0;
 var $vararg_buffer5 = 0, $vararg_buffer8 = 0, $zErr = 0, $zFormat = 0, $zModuleName = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $2 = sp + 124|0;
 $3 = sp + 120|0;
 $4 = sp + 116|0;
 $5 = sp + 112|0;
 $sCtx = sp + 96|0;
 $pVTable = sp + 92|0;
 $rc = sp + 88|0;
 $azArg = sp + 84|0;
 $nArg = sp + 80|0;
 $zErr = sp + 76|0;
 $zModuleName = sp + 72|0;
 $iDb = sp + 68|0;
 $pCtx = sp + 64|0;
 $zFormat = sp + 60|0;
 $iCol = sp + 56|0;
 $oooHidden = sp + 136|0;
 $zType = sp + 52|0;
 $nType = sp + 48|0;
 $i = sp + 44|0;
 $j = sp + 40|0;
 $nDel = sp + 36|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pMod;
 HEAP32[$4>>2] = $xConstruct;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$azArg>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 48|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$nArg>>2] = $11;
 HEAP32[$zErr>>2] = 0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 328|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pCtx>>2] = $14;
 while(1) {
  $15 = HEAP32[$pCtx>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pCtx>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($19|0)==($20|0);
  if ($21) {
   label = 4;
   break;
  }
  $27 = HEAP32[$pCtx>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  HEAP32[$pCtx>>2] = $29;
 }
 if ((label|0) == 4) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$vararg_buffer>>2] = $24;
  $25 = (_sqlite3MPrintf($22,23766,$vararg_buffer)|0);
  $26 = HEAP32[$5>>2]|0;
  HEAP32[$26>>2] = $25;
  HEAP32[$0>>2] = 6;
  $262 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($262|0);
 }
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $32;
 $33 = (_sqlite3MPrintf($30,17521,$vararg_buffer2)|0);
 HEAP32[$zModuleName>>2] = $33;
 $34 = HEAP32[$zModuleName>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  HEAP32[$0>>2] = 7;
  $262 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($262|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = (_sqlite3DbMallocZero($36,28,0)|0);
 HEAP32[$pVTable>>2] = $37;
 $38 = HEAP32[$pVTable>>2]|0;
 $39 = ($38|0)!=(0|0);
 $40 = HEAP32[$1>>2]|0;
 if (!($39)) {
  $41 = HEAP32[$zModuleName>>2]|0;
  _sqlite3DbFree($40,$41);
  HEAP32[$0>>2] = 7;
  $262 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($262|0);
 }
 $42 = HEAP32[$pVTable>>2]|0;
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$pVTable>>2]|0;
 $45 = ((($44)) + 4|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$2>>2]|0;
 $48 = ((($47)) + 64|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (_sqlite3SchemaToIndex($46,$49)|0);
 HEAP32[$iDb>>2] = $50;
 $51 = HEAP32[$iDb>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = ((($52)) + 16|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + ($51<<4)|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$2>>2]|0;
 $58 = ((($57)) + 52|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = ((($59)) + 4|0);
 HEAP32[$60>>2] = $56;
 $61 = HEAP32[$2>>2]|0;
 $62 = ((($sCtx)) + 4|0);
 HEAP32[$62>>2] = $61;
 $63 = HEAP32[$pVTable>>2]|0;
 HEAP32[$sCtx>>2] = $63;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 328|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ((($sCtx)) + 8|0);
 HEAP32[$67>>2] = $66;
 $68 = ((($sCtx)) + 12|0);
 HEAP32[$68>>2] = 0;
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 328|0);
 HEAP32[$70>>2] = $sCtx;
 $71 = HEAP32[$4>>2]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = HEAP32[$3>>2]|0;
 $74 = ((($73)) + 8|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = HEAP32[$nArg>>2]|0;
 $77 = HEAP32[$azArg>>2]|0;
 $78 = HEAP32[$pVTable>>2]|0;
 $79 = ((($78)) + 8|0);
 $80 = (FUNCTION_TABLE_iiiiiii[$71 & 255]($72,$75,$76,$77,$79,$zErr)|0);
 HEAP32[$rc>>2] = $80;
 $81 = ((($sCtx)) + 8|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = HEAP32[$1>>2]|0;
 $84 = ((($83)) + 328|0);
 HEAP32[$84>>2] = $82;
 $85 = HEAP32[$rc>>2]|0;
 $86 = ($85|0)==(7);
 if ($86) {
  $87 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($87);
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = (0)!=($88|0);
 L20: do {
  if ($89) {
   $90 = HEAP32[$zErr>>2]|0;
   $91 = ($90|0)==(0|0);
   $92 = HEAP32[$1>>2]|0;
   if ($91) {
    $93 = HEAP32[$zModuleName>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $93;
    $94 = (_sqlite3MPrintf($92,23808,$vararg_buffer5)|0);
    $95 = HEAP32[$5>>2]|0;
    HEAP32[$95>>2] = $94;
   } else {
    $96 = HEAP32[$zErr>>2]|0;
    HEAP32[$vararg_buffer8>>2] = $96;
    $97 = (_sqlite3MPrintf($92,17521,$vararg_buffer8)|0);
    $98 = HEAP32[$5>>2]|0;
    HEAP32[$98>>2] = $97;
    $99 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($99);
   }
   $100 = HEAP32[$1>>2]|0;
   $101 = HEAP32[$pVTable>>2]|0;
   _sqlite3DbFree($100,$101);
  } else {
   $102 = HEAP32[$pVTable>>2]|0;
   $103 = ((($102)) + 8|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)!=(0|0);
   if ($105) {
    $106 = HEAP32[$pVTable>>2]|0;
    $107 = ((($106)) + 8|0);
    $108 = HEAP32[$107>>2]|0;
    ;HEAP32[$108>>2]=0|0;HEAP32[$108+4>>2]=0|0;HEAP32[$108+8>>2]=0|0;
    $109 = HEAP32[$3>>2]|0;
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$pVTable>>2]|0;
    $112 = ((($111)) + 8|0);
    $113 = HEAP32[$112>>2]|0;
    HEAP32[$113>>2] = $110;
    $114 = HEAP32[$pVTable>>2]|0;
    $115 = ((($114)) + 12|0);
    HEAP32[$115>>2] = 1;
    $116 = ((($sCtx)) + 12|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = ($117|0)==(0);
    if ($118) {
     HEAP32[$zFormat>>2] = 23838;
     $119 = HEAP32[$1>>2]|0;
     $120 = HEAP32[$zFormat>>2]|0;
     $121 = HEAP32[$2>>2]|0;
     $122 = HEAP32[$121>>2]|0;
     HEAP32[$vararg_buffer11>>2] = $122;
     $123 = (_sqlite3MPrintf($119,$120,$vararg_buffer11)|0);
     $124 = HEAP32[$5>>2]|0;
     HEAP32[$124>>2] = $123;
     $125 = HEAP32[$pVTable>>2]|0;
     _sqlite3VtabUnlock($125);
     HEAP32[$rc>>2] = 1;
     break;
    }
    HEAP8[$oooHidden>>0] = 0;
    $126 = HEAP32[$2>>2]|0;
    $127 = ((($126)) + 56|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$pVTable>>2]|0;
    $130 = ((($129)) + 24|0);
    HEAP32[$130>>2] = $128;
    $131 = HEAP32[$pVTable>>2]|0;
    $132 = HEAP32[$2>>2]|0;
    $133 = ((($132)) + 56|0);
    HEAP32[$133>>2] = $131;
    HEAP32[$iCol>>2] = 0;
    while(1) {
     $134 = HEAP32[$iCol>>2]|0;
     $135 = HEAP32[$2>>2]|0;
     $136 = ((($135)) + 34|0);
     $137 = HEAP16[$136>>1]|0;
     $138 = $137 << 16 >> 16;
     $139 = ($134|0)<($138|0);
     if (!($139)) {
      break L20;
     }
     $140 = HEAP32[$iCol>>2]|0;
     $141 = HEAP32[$2>>2]|0;
     $142 = ((($141)) + 4|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = (($143) + (($140*24)|0)|0);
     $145 = ((($144)) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     HEAP32[$zType>>2] = $146;
     HEAP32[$i>>2] = 0;
     $147 = HEAP32[$zType>>2]|0;
     $148 = ($147|0)!=(0|0);
     do {
      if ($148) {
       $157 = HEAP32[$zType>>2]|0;
       $158 = (_sqlite3Strlen30($157)|0);
       HEAP32[$nType>>2] = $158;
       $159 = HEAP32[$zType>>2]|0;
       $160 = (_sqlite3_strnicmp(23884,$159,6)|0);
       $161 = ($160|0)!=(0);
       if ($161) {
        label = 27;
       } else {
        $162 = HEAP32[$zType>>2]|0;
        $163 = ((($162)) + 6|0);
        $164 = HEAP8[$163>>0]|0;
        $165 = $164 << 24 >> 24;
        $166 = ($165|0)!=(0);
        if ($166) {
         $167 = HEAP32[$zType>>2]|0;
         $168 = ((($167)) + 6|0);
         $169 = HEAP8[$168>>0]|0;
         $170 = $169 << 24 >> 24;
         $171 = ($170|0)!=(32);
         if ($171) {
          label = 27;
         }
        }
       }
       L39: do {
        if ((label|0) == 27) {
         label = 0;
         HEAP32[$i>>2] = 0;
         while(1) {
          $172 = HEAP32[$i>>2]|0;
          $173 = HEAP32[$nType>>2]|0;
          $174 = ($172|0)<($173|0);
          if (!($174)) {
           break L39;
          }
          $175 = HEAP32[$i>>2]|0;
          $176 = HEAP32[$zType>>2]|0;
          $177 = (($176) + ($175)|0);
          $178 = (_sqlite3_strnicmp(23891,$177,7)|0);
          $179 = (0)==($178|0);
          if ($179) {
           $180 = HEAP32[$i>>2]|0;
           $181 = (($180) + 7)|0;
           $182 = HEAP32[$zType>>2]|0;
           $183 = (($182) + ($181)|0);
           $184 = HEAP8[$183>>0]|0;
           $185 = $184 << 24 >> 24;
           $186 = ($185|0)==(0);
           if ($186) {
            break;
           }
           $187 = HEAP32[$i>>2]|0;
           $188 = (($187) + 7)|0;
           $189 = HEAP32[$zType>>2]|0;
           $190 = (($189) + ($188)|0);
           $191 = HEAP8[$190>>0]|0;
           $192 = $191 << 24 >> 24;
           $193 = ($192|0)==(32);
           if ($193) {
            break;
           }
          }
          $196 = HEAP32[$i>>2]|0;
          $197 = (($196) + 1)|0;
          HEAP32[$i>>2] = $197;
         }
         $194 = HEAP32[$i>>2]|0;
         $195 = (($194) + 1)|0;
         HEAP32[$i>>2] = $195;
        }
       } while(0);
       $198 = HEAP32[$i>>2]|0;
       $199 = HEAP32[$nType>>2]|0;
       $200 = ($198|0)<($199|0);
       if (!($200)) {
        $249 = HEAP8[$oooHidden>>0]|0;
        $250 = $249&255;
        $251 = HEAP32[$2>>2]|0;
        $252 = ((($251)) + 42|0);
        $253 = HEAP8[$252>>0]|0;
        $254 = $253&255;
        $255 = $254 | $250;
        $256 = $255&255;
        HEAP8[$252>>0] = $256;
        break;
       }
       $201 = HEAP32[$i>>2]|0;
       $202 = (($201) + 6)|0;
       $203 = HEAP32[$zType>>2]|0;
       $204 = (($203) + ($202)|0);
       $205 = HEAP8[$204>>0]|0;
       $206 = $205 << 24 >> 24;
       $207 = ($206|0)!=(0);
       $208 = $207 ? 1 : 0;
       $209 = (6 + ($208))|0;
       HEAP32[$nDel>>2] = $209;
       $210 = HEAP32[$i>>2]|0;
       HEAP32[$j>>2] = $210;
       while(1) {
        $211 = HEAP32[$j>>2]|0;
        $212 = HEAP32[$nDel>>2]|0;
        $213 = (($211) + ($212))|0;
        $214 = HEAP32[$nType>>2]|0;
        $215 = ($213|0)<=($214|0);
        if (!($215)) {
         break;
        }
        $216 = HEAP32[$j>>2]|0;
        $217 = HEAP32[$nDel>>2]|0;
        $218 = (($216) + ($217))|0;
        $219 = HEAP32[$zType>>2]|0;
        $220 = (($219) + ($218)|0);
        $221 = HEAP8[$220>>0]|0;
        $222 = HEAP32[$j>>2]|0;
        $223 = HEAP32[$zType>>2]|0;
        $224 = (($223) + ($222)|0);
        HEAP8[$224>>0] = $221;
        $225 = HEAP32[$j>>2]|0;
        $226 = (($225) + 1)|0;
        HEAP32[$j>>2] = $226;
       }
       $227 = HEAP32[$i>>2]|0;
       $228 = HEAP32[$zType>>2]|0;
       $229 = (($228) + ($227)|0);
       $230 = HEAP8[$229>>0]|0;
       $231 = $230 << 24 >> 24;
       $232 = ($231|0)==(0);
       $233 = HEAP32[$i>>2]|0;
       $234 = ($233|0)>(0);
       $or$cond = $232 & $234;
       if ($or$cond) {
        $235 = HEAP32[$i>>2]|0;
        $236 = (($235) - 1)|0;
        $237 = HEAP32[$zType>>2]|0;
        $238 = (($237) + ($236)|0);
        HEAP8[$238>>0] = 0;
       }
       $239 = HEAP32[$iCol>>2]|0;
       $240 = HEAP32[$2>>2]|0;
       $241 = ((($240)) + 4|0);
       $242 = HEAP32[$241>>2]|0;
       $243 = (($242) + (($239*24)|0)|0);
       $244 = ((($243)) + 23|0);
       $245 = HEAP8[$244>>0]|0;
       $246 = $245&255;
       $247 = $246 | 2;
       $248 = $247&255;
       HEAP8[$244>>0] = $248;
       HEAP8[$oooHidden>>0] = -128;
      } else {
       $149 = HEAP8[$oooHidden>>0]|0;
       $150 = $149&255;
       $151 = HEAP32[$2>>2]|0;
       $152 = ((($151)) + 42|0);
       $153 = HEAP8[$152>>0]|0;
       $154 = $153&255;
       $155 = $154 | $150;
       $156 = $155&255;
       HEAP8[$152>>0] = $156;
      }
     } while(0);
     $257 = HEAP32[$iCol>>2]|0;
     $258 = (($257) + 1)|0;
     HEAP32[$iCol>>2] = $258;
    }
   }
  }
 } while(0);
 $259 = HEAP32[$1>>2]|0;
 $260 = HEAP32[$zModuleName>>2]|0;
 _sqlite3DbFree($259,$260);
 $261 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $261;
 $262 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($262|0);
}
function _sqlite3GetVTable($db,$pTab) {
 $db = $db|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pVtab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pVtab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pVtab>>2] = $4;
 while(1) {
  $5 = HEAP32[$pVtab>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$pVtab>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ($8|0)!=($9|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$pVtab>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pVtab>>2] = $13;
 }
 if ((label|0) == 5) {
  $14 = HEAP32[$pVtab>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return (0)|0;
}
function _vtabDisconnectAll($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db2 = 0, $pNext = 0, $pRet = 0, $pVTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pRet = sp + 12|0;
 $pVTable = sp + 8|0;
 $db2 = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$pRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pVTable>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 56|0);
 HEAP32[$6>>2] = 0;
 while(1) {
  $7 = HEAP32[$pVTable>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$pVTable>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$db2>>2] = $10;
  $11 = HEAP32[$pVTable>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pNext>>2] = $13;
  $14 = HEAP32[$db2>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ($14|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$pVTable>>2]|0;
   HEAP32[$pRet>>2] = $17;
   $18 = HEAP32[$pRet>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 56|0);
   HEAP32[$20>>2] = $18;
   $21 = HEAP32[$pRet>>2]|0;
   $22 = ((($21)) + 24|0);
   HEAP32[$22>>2] = 0;
  } else {
   $23 = HEAP32[$db2>>2]|0;
   $24 = ((($23)) + 336|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$pVTable>>2]|0;
   $27 = ((($26)) + 24|0);
   HEAP32[$27>>2] = $25;
   $28 = HEAP32[$pVTable>>2]|0;
   $29 = HEAP32[$db2>>2]|0;
   $30 = ((($29)) + 336|0);
   HEAP32[$30>>2] = $28;
  }
  $31 = HEAP32[$pNext>>2]|0;
  HEAP32[$pVTable>>2] = $31;
 }
 $32 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3PagerMaxPageCount($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 164|0);
  HEAP32[$6>>2] = $4;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 164|0);
 $9 = HEAP32[$8>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _findNextHostParameter($zSql,$pnToken) {
 $zSql = $zSql|0;
 $pnToken = $pnToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nTotal = 0;
 var $tokenType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $tokenType = sp + 8|0;
 $nTotal = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $zSql;
 HEAP32[$1>>2] = $pnToken;
 HEAP32[$nTotal>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = ($4<<24>>24)!=(0);
  if (!($5)) {
   label = 6;
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = (_sqlite3GetToken($6,$tokenType)|0);
  HEAP32[$n>>2] = $7;
  $8 = HEAP32[$tokenType>>2]|0;
  $9 = ($8|0)==(135);
  $10 = HEAP32[$n>>2]|0;
  if ($9) {
   break;
  }
  $12 = HEAP32[$nTotal>>2]|0;
  $13 = (($12) + ($10))|0;
  HEAP32[$nTotal>>2] = $13;
  $14 = HEAP32[$n>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + ($14)|0);
  HEAP32[$0>>2] = $16;
 }
 if ((label|0) == 6) {
  $17 = HEAP32[$nTotal>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $17 = HEAP32[$nTotal>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3GetToken($z,$tokenType) {
 $z = $z|0;
 $tokenType = $tokenType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $c = 0, $delim = 0, $i = 0, $n = 0, $or$cond = 0, $or$cond7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $c = sp + 8|0;
 $delim = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $tokenType;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = (23933 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 L1: do {
  switch ($8|0) {
  case 7:  {
   HEAP32[$i>>2] = 1;
   while(1) {
    $9 = HEAP32[$i>>2]|0;
    $10 = HEAP32[$1>>2]|0;
    $11 = (($10) + ($9)|0);
    $12 = HEAP8[$11>>0]|0;
    $13 = $12&255;
    $14 = (17533 + ($13)|0);
    $15 = HEAP8[$14>>0]|0;
    $16 = $15&255;
    $17 = $16 & 1;
    $18 = ($17|0)!=(0);
    if (!($18)) {
     break;
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = (($19) + 1)|0;
    HEAP32[$i>>2] = $20;
   }
   $21 = HEAP32[$2>>2]|0;
   HEAP32[$21>>2] = 159;
   $22 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $22;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 11:  {
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 1|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)==(45);
   if (!($27)) {
    $41 = HEAP32[$2>>2]|0;
    HEAP32[$41>>2] = 90;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   HEAP32[$i>>2] = 2;
   while(1) {
    $28 = HEAP32[$i>>2]|0;
    $29 = HEAP32[$1>>2]|0;
    $30 = (($29) + ($28)|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = $31&255;
    HEAP32[$c>>2] = $32;
    $33 = ($32|0)!=(0);
    $34 = HEAP32[$c>>2]|0;
    $35 = ($34|0)!=(10);
    $36 = $33 ? $35 : 0;
    if (!($36)) {
     break;
    }
    $37 = HEAP32[$i>>2]|0;
    $38 = (($37) + 1)|0;
    HEAP32[$i>>2] = $38;
   }
   $39 = HEAP32[$2>>2]|0;
   HEAP32[$39>>2] = 159;
   $40 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $40;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 17:  {
   $42 = HEAP32[$2>>2]|0;
   HEAP32[$42>>2] = 22;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 18:  {
   $43 = HEAP32[$2>>2]|0;
   HEAP32[$43>>2] = 23;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 19:  {
   $44 = HEAP32[$2>>2]|0;
   HEAP32[$44>>2] = 1;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 20:  {
   $45 = HEAP32[$2>>2]|0;
   HEAP32[$45>>2] = 89;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 21:  {
   $46 = HEAP32[$2>>2]|0;
   HEAP32[$46>>2] = 91;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 16:  {
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 1|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = ($50|0)!=(42);
   if (!($51)) {
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 2|0);
    $54 = HEAP8[$53>>0]|0;
    $55 = $54&255;
    $56 = ($55|0)==(0);
    if (!($56)) {
     HEAP32[$i>>2] = 3;
     $58 = HEAP32[$1>>2]|0;
     $59 = ((($58)) + 2|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60&255;
     HEAP32[$c>>2] = $61;
     while(1) {
      $62 = HEAP32[$c>>2]|0;
      $63 = ($62|0)!=(42);
      if (!($63)) {
       $64 = HEAP32[$i>>2]|0;
       $65 = HEAP32[$1>>2]|0;
       $66 = (($65) + ($64)|0);
       $67 = HEAP8[$66>>0]|0;
       $68 = $67&255;
       $69 = ($68|0)!=(47);
       if (!($69)) {
        break;
       }
      }
      $70 = HEAP32[$i>>2]|0;
      $71 = HEAP32[$1>>2]|0;
      $72 = (($71) + ($70)|0);
      $73 = HEAP8[$72>>0]|0;
      $74 = $73&255;
      HEAP32[$c>>2] = $74;
      $75 = ($74|0)!=(0);
      if (!($75)) {
       break;
      }
      $76 = HEAP32[$i>>2]|0;
      $77 = (($76) + 1)|0;
      HEAP32[$i>>2] = $77;
     }
     $78 = HEAP32[$c>>2]|0;
     $79 = ($78|0)!=(0);
     if ($79) {
      $80 = HEAP32[$i>>2]|0;
      $81 = (($80) + 1)|0;
      HEAP32[$i>>2] = $81;
     }
     $82 = HEAP32[$2>>2]|0;
     HEAP32[$82>>2] = 159;
     $83 = HEAP32[$i>>2]|0;
     HEAP32[$0>>2] = $83;
     $502 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($502|0);
    }
   }
   $57 = HEAP32[$2>>2]|0;
   HEAP32[$57>>2] = 92;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 22:  {
   $84 = HEAP32[$2>>2]|0;
   HEAP32[$84>>2] = 93;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 14:  {
   $85 = HEAP32[$2>>2]|0;
   HEAP32[$85>>2] = 79;
   $86 = HEAP32[$1>>2]|0;
   $87 = ((($86)) + 1|0);
   $88 = HEAP8[$87>>0]|0;
   $89 = $88&255;
   $90 = ($89|0)==(61);
   $91 = $90&1;
   $92 = (1 + ($91))|0;
   HEAP32[$0>>2] = $92;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 12:  {
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 1|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = $95&255;
   HEAP32[$c>>2] = $96;
   $97 = ($96|0)==(61);
   if ($97) {
    $98 = HEAP32[$2>>2]|0;
    HEAP32[$98>>2] = 81;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $99 = HEAP32[$c>>2]|0;
   $100 = ($99|0)==(62);
   if ($100) {
    $101 = HEAP32[$2>>2]|0;
    HEAP32[$101>>2] = 78;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $102 = HEAP32[$c>>2]|0;
   $103 = ($102|0)==(60);
   $104 = HEAP32[$2>>2]|0;
   if ($103) {
    HEAP32[$104>>2] = 87;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$104>>2] = 82;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 13:  {
   $105 = HEAP32[$1>>2]|0;
   $106 = ((($105)) + 1|0);
   $107 = HEAP8[$106>>0]|0;
   $108 = $107&255;
   HEAP32[$c>>2] = $108;
   $109 = ($108|0)==(61);
   if ($109) {
    $110 = HEAP32[$2>>2]|0;
    HEAP32[$110>>2] = 83;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $111 = HEAP32[$c>>2]|0;
   $112 = ($111|0)==(62);
   $113 = HEAP32[$2>>2]|0;
   if ($112) {
    HEAP32[$113>>2] = 88;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$113>>2] = 80;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 15:  {
   $114 = HEAP32[$1>>2]|0;
   $115 = ((($114)) + 1|0);
   $116 = HEAP8[$115>>0]|0;
   $117 = $116&255;
   $118 = ($117|0)!=(61);
   $119 = HEAP32[$2>>2]|0;
   if ($118) {
    HEAP32[$119>>2] = 160;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$119>>2] = 78;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 10:  {
   $120 = HEAP32[$1>>2]|0;
   $121 = ((($120)) + 1|0);
   $122 = HEAP8[$121>>0]|0;
   $123 = $122&255;
   $124 = ($123|0)!=(124);
   $125 = HEAP32[$2>>2]|0;
   if ($124) {
    HEAP32[$125>>2] = 86;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$125>>2] = 94;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 23:  {
   $126 = HEAP32[$2>>2]|0;
   HEAP32[$126>>2] = 26;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 24:  {
   $127 = HEAP32[$2>>2]|0;
   HEAP32[$127>>2] = 85;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 25:  {
   $128 = HEAP32[$2>>2]|0;
   HEAP32[$128>>2] = 96;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 8:  {
   $129 = HEAP32[$1>>2]|0;
   $130 = HEAP8[$129>>0]|0;
   $131 = $130&255;
   HEAP32[$delim>>2] = $131;
   HEAP32[$i>>2] = 1;
   while(1) {
    $132 = HEAP32[$i>>2]|0;
    $133 = HEAP32[$1>>2]|0;
    $134 = (($133) + ($132)|0);
    $135 = HEAP8[$134>>0]|0;
    $136 = $135&255;
    HEAP32[$c>>2] = $136;
    $137 = ($136|0)!=(0);
    if (!($137)) {
     break;
    }
    $138 = HEAP32[$c>>2]|0;
    $139 = HEAP32[$delim>>2]|0;
    $140 = ($138|0)==($139|0);
    if ($140) {
     $141 = HEAP32[$i>>2]|0;
     $142 = (($141) + 1)|0;
     $143 = HEAP32[$1>>2]|0;
     $144 = (($143) + ($142)|0);
     $145 = HEAP8[$144>>0]|0;
     $146 = $145&255;
     $147 = HEAP32[$delim>>2]|0;
     $148 = ($146|0)==($147|0);
     if (!($148)) {
      break;
     }
     $149 = HEAP32[$i>>2]|0;
     $150 = (($149) + 1)|0;
     HEAP32[$i>>2] = $150;
    }
    $151 = HEAP32[$i>>2]|0;
    $152 = (($151) + 1)|0;
    HEAP32[$i>>2] = $152;
   }
   $153 = HEAP32[$c>>2]|0;
   $154 = ($153|0)==(39);
   if ($154) {
    $155 = HEAP32[$2>>2]|0;
    HEAP32[$155>>2] = 97;
    $156 = HEAP32[$i>>2]|0;
    $157 = (($156) + 1)|0;
    HEAP32[$0>>2] = $157;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $158 = HEAP32[$c>>2]|0;
   $159 = ($158|0)!=(0);
   $160 = HEAP32[$2>>2]|0;
   if ($159) {
    HEAP32[$160>>2] = 27;
    $161 = HEAP32[$i>>2]|0;
    $162 = (($161) + 1)|0;
    HEAP32[$0>>2] = $162;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$160>>2] = 160;
    $163 = HEAP32[$i>>2]|0;
    HEAP32[$0>>2] = $163;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 26:  {
   $164 = HEAP32[$1>>2]|0;
   $165 = ((($164)) + 1|0);
   $166 = HEAP8[$165>>0]|0;
   $167 = $166&255;
   $168 = (17533 + ($167)|0);
   $169 = HEAP8[$168>>0]|0;
   $170 = $169&255;
   $171 = $170 & 4;
   $172 = ($171|0)!=(0);
   if ($172) {
    label = 64;
   } else {
    $173 = HEAP32[$2>>2]|0;
    HEAP32[$173>>2] = 122;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 3:  {
   label = 64;
   break;
  }
  case 9:  {
   HEAP32[$i>>2] = 1;
   $319 = HEAP32[$1>>2]|0;
   $320 = HEAP8[$319>>0]|0;
   $321 = $320&255;
   HEAP32[$c>>2] = $321;
   while(1) {
    $322 = HEAP32[$c>>2]|0;
    $323 = ($322|0)!=(93);
    if (!($323)) {
     break;
    }
    $324 = HEAP32[$i>>2]|0;
    $325 = HEAP32[$1>>2]|0;
    $326 = (($325) + ($324)|0);
    $327 = HEAP8[$326>>0]|0;
    $328 = $327&255;
    HEAP32[$c>>2] = $328;
    $329 = ($328|0)!=(0);
    if (!($329)) {
     break;
    }
    $330 = HEAP32[$i>>2]|0;
    $331 = (($330) + 1)|0;
    HEAP32[$i>>2] = $331;
   }
   $332 = HEAP32[$c>>2]|0;
   $333 = ($332|0)==(93);
   $334 = $333 ? 27 : 160;
   $335 = HEAP32[$2>>2]|0;
   HEAP32[$335>>2] = $334;
   $336 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $336;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 6:  {
   $337 = HEAP32[$2>>2]|0;
   HEAP32[$337>>2] = 135;
   HEAP32[$i>>2] = 1;
   while(1) {
    $338 = HEAP32[$i>>2]|0;
    $339 = HEAP32[$1>>2]|0;
    $340 = (($339) + ($338)|0);
    $341 = HEAP8[$340>>0]|0;
    $342 = $341&255;
    $343 = (17533 + ($342)|0);
    $344 = HEAP8[$343>>0]|0;
    $345 = $344&255;
    $346 = $345 & 4;
    $347 = ($346|0)!=(0);
    $348 = HEAP32[$i>>2]|0;
    if (!($347)) {
     break;
    }
    $349 = (($348) + 1)|0;
    HEAP32[$i>>2] = $349;
   }
   HEAP32[$0>>2] = $348;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 5: case 4:  {
   HEAP32[$n>>2] = 0;
   $350 = HEAP32[$2>>2]|0;
   HEAP32[$350>>2] = 135;
   HEAP32[$i>>2] = 1;
   while(1) {
    $351 = HEAP32[$i>>2]|0;
    $352 = HEAP32[$1>>2]|0;
    $353 = (($352) + ($351)|0);
    $354 = HEAP8[$353>>0]|0;
    $355 = $354&255;
    HEAP32[$c>>2] = $355;
    $356 = ($355|0)!=(0);
    if (!($356)) {
     break;
    }
    $357 = HEAP32[$c>>2]|0;
    $358 = $357&255;
    $359 = $358&255;
    $360 = (17533 + ($359)|0);
    $361 = HEAP8[$360>>0]|0;
    $362 = $361&255;
    $363 = $362 & 70;
    $364 = ($363|0)!=(0);
    if ($364) {
     $365 = HEAP32[$n>>2]|0;
     $366 = (($365) + 1)|0;
     HEAP32[$n>>2] = $366;
    } else {
     $367 = HEAP32[$c>>2]|0;
     $368 = ($367|0)==(40);
     $369 = HEAP32[$n>>2]|0;
     $370 = ($369|0)>(0);
     $or$cond = $368 & $370;
     if ($or$cond) {
      label = 107;
      break;
     }
     $394 = HEAP32[$c>>2]|0;
     $395 = ($394|0)==(58);
     if (!($395)) {
      break;
     }
     $396 = HEAP32[$i>>2]|0;
     $397 = (($396) + 1)|0;
     $398 = HEAP32[$1>>2]|0;
     $399 = (($398) + ($397)|0);
     $400 = HEAP8[$399>>0]|0;
     $401 = $400&255;
     $402 = ($401|0)==(58);
     if (!($402)) {
      break;
     }
     $403 = HEAP32[$i>>2]|0;
     $404 = (($403) + 1)|0;
     HEAP32[$i>>2] = $404;
    }
    $405 = HEAP32[$i>>2]|0;
    $406 = (($405) + 1)|0;
    HEAP32[$i>>2] = $406;
   }
   do {
    if ((label|0) == 107) {
     while(1) {
      label = 0;
      $371 = HEAP32[$i>>2]|0;
      $372 = (($371) + 1)|0;
      HEAP32[$i>>2] = $372;
      $373 = HEAP32[$i>>2]|0;
      $374 = HEAP32[$1>>2]|0;
      $375 = (($374) + ($373)|0);
      $376 = HEAP8[$375>>0]|0;
      $377 = $376&255;
      HEAP32[$c>>2] = $377;
      $378 = ($377|0)!=(0);
      if (!($378)) {
       break;
      }
      $379 = HEAP32[$c>>2]|0;
      $380 = $379&255;
      $381 = $380&255;
      $382 = (17533 + ($381)|0);
      $383 = HEAP8[$382>>0]|0;
      $384 = $383&255;
      $385 = $384 & 1;
      $386 = ($385|0)==(0);
      $387 = HEAP32[$c>>2]|0;
      $388 = ($387|0)!=(41);
      $or$cond7 = $386 & $388;
      if ($or$cond7) {
       label = 107;
      } else {
       break;
      }
     }
     $389 = HEAP32[$c>>2]|0;
     $390 = ($389|0)==(41);
     if ($390) {
      $391 = HEAP32[$i>>2]|0;
      $392 = (($391) + 1)|0;
      HEAP32[$i>>2] = $392;
      break;
     } else {
      $393 = HEAP32[$2>>2]|0;
      HEAP32[$393>>2] = 160;
      break;
     }
    }
   } while(0);
   $407 = HEAP32[$n>>2]|0;
   $408 = ($407|0)==(0);
   if ($408) {
    $409 = HEAP32[$2>>2]|0;
    HEAP32[$409>>2] = 160;
   }
   $410 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $410;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 1:  {
   HEAP32[$i>>2] = 1;
   while(1) {
    $411 = HEAP32[$i>>2]|0;
    $412 = HEAP32[$1>>2]|0;
    $413 = (($412) + ($411)|0);
    $414 = HEAP8[$413>>0]|0;
    $415 = $414&255;
    $416 = (23933 + ($415)|0);
    $417 = HEAP8[$416>>0]|0;
    $418 = $417&255;
    $419 = ($418|0)<=(1);
    $420 = HEAP32[$i>>2]|0;
    if (!($419)) {
     break;
    }
    $421 = (($420) + 1)|0;
    HEAP32[$i>>2] = $421;
   }
   $422 = HEAP32[$1>>2]|0;
   $423 = (($422) + ($420)|0);
   $424 = HEAP8[$423>>0]|0;
   $425 = $424&255;
   $426 = (17533 + ($425)|0);
   $427 = HEAP8[$426>>0]|0;
   $428 = $427&255;
   $429 = $428 & 70;
   $430 = ($429|0)!=(0);
   if ($430) {
    $431 = HEAP32[$i>>2]|0;
    $432 = (($431) + 1)|0;
    HEAP32[$i>>2] = $432;
    break L1;
   }
   $433 = HEAP32[$2>>2]|0;
   HEAP32[$433>>2] = 27;
   $434 = HEAP32[$1>>2]|0;
   $435 = HEAP32[$i>>2]|0;
   $436 = HEAP32[$2>>2]|0;
   $437 = (_keywordCode($434,$435,$436)|0);
   HEAP32[$0>>2] = $437;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 0:  {
   $438 = HEAP32[$1>>2]|0;
   $439 = ((($438)) + 1|0);
   $440 = HEAP8[$439>>0]|0;
   $441 = $440&255;
   $442 = ($441|0)==(39);
   if ($442) {
    $443 = HEAP32[$2>>2]|0;
    HEAP32[$443>>2] = 134;
    HEAP32[$i>>2] = 2;
    while(1) {
     $444 = HEAP32[$i>>2]|0;
     $445 = HEAP32[$1>>2]|0;
     $446 = (($445) + ($444)|0);
     $447 = HEAP8[$446>>0]|0;
     $448 = $447&255;
     $449 = (17533 + ($448)|0);
     $450 = HEAP8[$449>>0]|0;
     $451 = $450&255;
     $452 = $451 & 8;
     $453 = ($452|0)!=(0);
     $454 = HEAP32[$i>>2]|0;
     if (!($453)) {
      break;
     }
     $455 = (($454) + 1)|0;
     HEAP32[$i>>2] = $455;
    }
    $456 = HEAP32[$1>>2]|0;
    $457 = (($456) + ($454)|0);
    $458 = HEAP8[$457>>0]|0;
    $459 = $458&255;
    $460 = ($459|0)!=(39);
    if ($460) {
     label = 131;
    } else {
     $461 = HEAP32[$i>>2]|0;
     $462 = (($461|0) % 2)&-1;
     $463 = ($462|0)!=(0);
     if ($463) {
      label = 131;
     }
    }
    L168: do {
     if ((label|0) == 131) {
      $464 = HEAP32[$2>>2]|0;
      HEAP32[$464>>2] = 160;
      while(1) {
       $465 = HEAP32[$i>>2]|0;
       $466 = HEAP32[$1>>2]|0;
       $467 = (($466) + ($465)|0);
       $468 = HEAP8[$467>>0]|0;
       $469 = $468&255;
       $470 = ($469|0)!=(0);
       if (!($470)) {
        break L168;
       }
       $471 = HEAP32[$i>>2]|0;
       $472 = HEAP32[$1>>2]|0;
       $473 = (($472) + ($471)|0);
       $474 = HEAP8[$473>>0]|0;
       $475 = $474&255;
       $476 = ($475|0)!=(39);
       if (!($476)) {
        break L168;
       }
       $477 = HEAP32[$i>>2]|0;
       $478 = (($477) + 1)|0;
       HEAP32[$i>>2] = $478;
      }
     }
    } while(0);
    $479 = HEAP32[$i>>2]|0;
    $480 = HEAP32[$1>>2]|0;
    $481 = (($480) + ($479)|0);
    $482 = HEAP8[$481>>0]|0;
    $483 = ($482<<24>>24)!=(0);
    if ($483) {
     $484 = HEAP32[$i>>2]|0;
     $485 = (($484) + 1)|0;
     HEAP32[$i>>2] = $485;
    }
    $486 = HEAP32[$i>>2]|0;
    HEAP32[$0>>2] = $486;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    label = 138;
   }
   break;
  }
  case 2:  {
   label = 138;
   break;
  }
  default: {
   $487 = HEAP32[$2>>2]|0;
   HEAP32[$487>>2] = 160;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
  }
  }
 } while(0);
 if ((label|0) == 64) {
  $174 = HEAP32[$2>>2]|0;
  HEAP32[$174>>2] = 132;
  $175 = HEAP32[$1>>2]|0;
  $176 = HEAP8[$175>>0]|0;
  $177 = $176&255;
  $178 = ($177|0)==(48);
  do {
   if ($178) {
    $179 = HEAP32[$1>>2]|0;
    $180 = ((($179)) + 1|0);
    $181 = HEAP8[$180>>0]|0;
    $182 = $181&255;
    $183 = ($182|0)==(120);
    if (!($183)) {
     $184 = HEAP32[$1>>2]|0;
     $185 = ((($184)) + 1|0);
     $186 = HEAP8[$185>>0]|0;
     $187 = $186&255;
     $188 = ($187|0)==(88);
     if (!($188)) {
      break;
     }
    }
    $189 = HEAP32[$1>>2]|0;
    $190 = ((($189)) + 2|0);
    $191 = HEAP8[$190>>0]|0;
    $192 = $191&255;
    $193 = (17533 + ($192)|0);
    $194 = HEAP8[$193>>0]|0;
    $195 = $194&255;
    $196 = $195 & 8;
    $197 = ($196|0)!=(0);
    if ($197) {
     HEAP32[$i>>2] = 3;
     while(1) {
      $198 = HEAP32[$i>>2]|0;
      $199 = HEAP32[$1>>2]|0;
      $200 = (($199) + ($198)|0);
      $201 = HEAP8[$200>>0]|0;
      $202 = $201&255;
      $203 = (17533 + ($202)|0);
      $204 = HEAP8[$203>>0]|0;
      $205 = $204&255;
      $206 = $205 & 8;
      $207 = ($206|0)!=(0);
      $208 = HEAP32[$i>>2]|0;
      if (!($207)) {
       break;
      }
      $209 = (($208) + 1)|0;
      HEAP32[$i>>2] = $209;
     }
     HEAP32[$0>>2] = $208;
     $502 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($502|0);
    }
   }
  } while(0);
  HEAP32[$i>>2] = 0;
  while(1) {
   $210 = HEAP32[$i>>2]|0;
   $211 = HEAP32[$1>>2]|0;
   $212 = (($211) + ($210)|0);
   $213 = HEAP8[$212>>0]|0;
   $214 = $213&255;
   $215 = (17533 + ($214)|0);
   $216 = HEAP8[$215>>0]|0;
   $217 = $216&255;
   $218 = $217 & 4;
   $219 = ($218|0)!=(0);
   $220 = HEAP32[$i>>2]|0;
   if (!($219)) {
    break;
   }
   $221 = (($220) + 1)|0;
   HEAP32[$i>>2] = $221;
  }
  $222 = HEAP32[$1>>2]|0;
  $223 = (($222) + ($220)|0);
  $224 = HEAP8[$223>>0]|0;
  $225 = $224&255;
  $226 = ($225|0)==(46);
  if ($226) {
   $227 = HEAP32[$i>>2]|0;
   $228 = (($227) + 1)|0;
   HEAP32[$i>>2] = $228;
   while(1) {
    $229 = HEAP32[$i>>2]|0;
    $230 = HEAP32[$1>>2]|0;
    $231 = (($230) + ($229)|0);
    $232 = HEAP8[$231>>0]|0;
    $233 = $232&255;
    $234 = (17533 + ($233)|0);
    $235 = HEAP8[$234>>0]|0;
    $236 = $235&255;
    $237 = $236 & 4;
    $238 = ($237|0)!=(0);
    if (!($238)) {
     break;
    }
    $239 = HEAP32[$i>>2]|0;
    $240 = (($239) + 1)|0;
    HEAP32[$i>>2] = $240;
   }
   $241 = HEAP32[$2>>2]|0;
   HEAP32[$241>>2] = 133;
  }
  $242 = HEAP32[$i>>2]|0;
  $243 = HEAP32[$1>>2]|0;
  $244 = (($243) + ($242)|0);
  $245 = HEAP8[$244>>0]|0;
  $246 = $245&255;
  $247 = ($246|0)==(101);
  if ($247) {
   label = 82;
  } else {
   $248 = HEAP32[$i>>2]|0;
   $249 = HEAP32[$1>>2]|0;
   $250 = (($249) + ($248)|0);
   $251 = HEAP8[$250>>0]|0;
   $252 = $251&255;
   $253 = ($252|0)==(69);
   if ($253) {
    label = 82;
   }
  }
  do {
   if ((label|0) == 82) {
    $254 = HEAP32[$i>>2]|0;
    $255 = (($254) + 1)|0;
    $256 = HEAP32[$1>>2]|0;
    $257 = (($256) + ($255)|0);
    $258 = HEAP8[$257>>0]|0;
    $259 = $258&255;
    $260 = (17533 + ($259)|0);
    $261 = HEAP8[$260>>0]|0;
    $262 = $261&255;
    $263 = $262 & 4;
    $264 = ($263|0)!=(0);
    if (!($264)) {
     $265 = HEAP32[$i>>2]|0;
     $266 = (($265) + 1)|0;
     $267 = HEAP32[$1>>2]|0;
     $268 = (($267) + ($266)|0);
     $269 = HEAP8[$268>>0]|0;
     $270 = $269&255;
     $271 = ($270|0)==(43);
     if (!($271)) {
      $272 = HEAP32[$i>>2]|0;
      $273 = (($272) + 1)|0;
      $274 = HEAP32[$1>>2]|0;
      $275 = (($274) + ($273)|0);
      $276 = HEAP8[$275>>0]|0;
      $277 = $276&255;
      $278 = ($277|0)==(45);
      if (!($278)) {
       break;
      }
     }
     $279 = HEAP32[$i>>2]|0;
     $280 = (($279) + 2)|0;
     $281 = HEAP32[$1>>2]|0;
     $282 = (($281) + ($280)|0);
     $283 = HEAP8[$282>>0]|0;
     $284 = $283&255;
     $285 = (17533 + ($284)|0);
     $286 = HEAP8[$285>>0]|0;
     $287 = $286&255;
     $288 = $287 & 4;
     $289 = ($288|0)!=(0);
     if (!($289)) {
      break;
     }
    }
    $290 = HEAP32[$i>>2]|0;
    $291 = (($290) + 2)|0;
    HEAP32[$i>>2] = $291;
    while(1) {
     $292 = HEAP32[$i>>2]|0;
     $293 = HEAP32[$1>>2]|0;
     $294 = (($293) + ($292)|0);
     $295 = HEAP8[$294>>0]|0;
     $296 = $295&255;
     $297 = (17533 + ($296)|0);
     $298 = HEAP8[$297>>0]|0;
     $299 = $298&255;
     $300 = $299 & 4;
     $301 = ($300|0)!=(0);
     if (!($301)) {
      break;
     }
     $302 = HEAP32[$i>>2]|0;
     $303 = (($302) + 1)|0;
     HEAP32[$i>>2] = $303;
    }
    $304 = HEAP32[$2>>2]|0;
    HEAP32[$304>>2] = 133;
   }
  } while(0);
  while(1) {
   $305 = HEAP32[$i>>2]|0;
   $306 = HEAP32[$1>>2]|0;
   $307 = (($306) + ($305)|0);
   $308 = HEAP8[$307>>0]|0;
   $309 = $308&255;
   $310 = (17533 + ($309)|0);
   $311 = HEAP8[$310>>0]|0;
   $312 = $311&255;
   $313 = $312 & 70;
   $314 = ($313|0)!=(0);
   if (!($314)) {
    break;
   }
   $315 = HEAP32[$2>>2]|0;
   HEAP32[$315>>2] = 160;
   $316 = HEAP32[$i>>2]|0;
   $317 = (($316) + 1)|0;
   HEAP32[$i>>2] = $317;
  }
  $318 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $318;
  $502 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($502|0);
 }
 else if ((label|0) == 138) {
  HEAP32[$i>>2] = 1;
 }
 while(1) {
  $488 = HEAP32[$i>>2]|0;
  $489 = HEAP32[$1>>2]|0;
  $490 = (($489) + ($488)|0);
  $491 = HEAP8[$490>>0]|0;
  $492 = $491&255;
  $493 = (17533 + ($492)|0);
  $494 = HEAP8[$493>>0]|0;
  $495 = $494&255;
  $496 = $495 & 70;
  $497 = ($496|0)!=(0);
  if (!($497)) {
   break;
  }
  $498 = HEAP32[$i>>2]|0;
  $499 = (($498) + 1)|0;
  HEAP32[$i>>2] = $499;
 }
 $500 = HEAP32[$2>>2]|0;
 HEAP32[$500>>2] = 27;
 $501 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $501;
 $502 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($502|0);
}
function _keywordCode($z,$n,$pType) {
 $z = $z|0;
 $n = $n|0;
 $pType = $pType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $zKW = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $zKW = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $pType;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>=(2);
 if (!($4)) {
  $73 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = (16974 + ($7)|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10<<2;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) - 1)|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + ($13)|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = (16974 + ($17)|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20*3)|0;
 $22 = $11 ^ $21;
 $23 = HEAP32[$1>>2]|0;
 $24 = $22 ^ $23;
 $25 = (($24|0) % 127)&-1;
 HEAP32[$i>>2] = $25;
 $26 = HEAP32[$i>>2]|0;
 $27 = (24742 + ($26)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = (($29) - 1)|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = ($31|0)>=(0);
  if (!($32)) {
   label = 13;
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = (24993 + ($33)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  $37 = HEAP32[$1>>2]|0;
  $38 = ($36|0)!=($37|0);
  if (!($38)) {
   HEAP32[$j>>2] = 0;
   $39 = HEAP32[$i>>2]|0;
   $40 = (10054 + ($39<<1)|0);
   $41 = HEAP16[$40>>1]|0;
   $42 = $41&65535;
   $43 = (24189 + ($42)|0);
   HEAP32[$zKW>>2] = $43;
   while(1) {
    $44 = HEAP32[$j>>2]|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = ($44|0)<($45|0);
    if ($46) {
     $47 = HEAP32[$j>>2]|0;
     $48 = HEAP32[$0>>2]|0;
     $49 = (($48) + ($47)|0);
     $50 = HEAP8[$49>>0]|0;
     $51 = $50 << 24 >> 24;
     $52 = $51 & -33;
     $53 = HEAP32[$j>>2]|0;
     $54 = HEAP32[$zKW>>2]|0;
     $55 = (($54) + ($53)|0);
     $56 = HEAP8[$55>>0]|0;
     $57 = $56 << 24 >> 24;
     $58 = ($52|0)==($57|0);
     $74 = $58;
    } else {
     $74 = 0;
    }
    $59 = HEAP32[$j>>2]|0;
    if (!($74)) {
     break;
    }
    $60 = (($59) + 1)|0;
    HEAP32[$j>>2] = $60;
   }
   $61 = HEAP32[$1>>2]|0;
   $62 = ($59|0)<($61|0);
   if (!($62)) {
    break;
   }
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = (24869 + ($68)|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70&255;
  $72 = (($71) - 1)|0;
  HEAP32[$i>>2] = $72;
 }
 if ((label|0) == 13) {
  $73 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $63 = HEAP32[$i>>2]|0;
 $64 = (25117 + ($63)|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = HEAP32[$2>>2]|0;
 HEAP32[$67>>2] = $66;
 $73 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _sqlite3PagerWalCallback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3WalCallback($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WalCallback($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $ret = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$ret>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $8 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$ret>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _columnNullValue() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return (312|0);
}
function _sqlite3VdbeAddOp3($p,$op,$p1,$p2,$p3) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $i = sp + 4|0;
 $pOp = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $p1;
 HEAP32[$4>>2] = $p2;
 HEAP32[$5>>2] = $p3;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 88|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$i>>2]|0;
 $15 = ($13|0)<=($14|0);
 $16 = HEAP32[$1>>2]|0;
 if ($15) {
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = HEAP32[$4>>2]|0;
  $20 = HEAP32[$5>>2]|0;
  $21 = (_growOp3($16,$17,$18,$19,$20)|0);
  HEAP32[$0>>2] = $21;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 } else {
  $22 = ((($16)) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$22>>2] = $24;
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*20)|0)|0);
  HEAP32[$pOp>>2] = $29;
  $30 = HEAP32[$2>>2]|0;
  $31 = $30&255;
  $32 = HEAP32[$pOp>>2]|0;
  HEAP8[$32>>0] = $31;
  $33 = HEAP32[$pOp>>2]|0;
  $34 = ((($33)) + 3|0);
  HEAP8[$34>>0] = 0;
  $35 = HEAP32[$3>>2]|0;
  $36 = HEAP32[$pOp>>2]|0;
  $37 = ((($36)) + 4|0);
  HEAP32[$37>>2] = $35;
  $38 = HEAP32[$4>>2]|0;
  $39 = HEAP32[$pOp>>2]|0;
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$5>>2]|0;
  $42 = HEAP32[$pOp>>2]|0;
  $43 = ((($42)) + 12|0);
  HEAP32[$43>>2] = $41;
  $44 = HEAP32[$pOp>>2]|0;
  $45 = ((($44)) + 16|0);
  HEAP32[$45>>2] = 0;
  $46 = HEAP32[$pOp>>2]|0;
  $47 = ((($46)) + 1|0);
  HEAP8[$47>>0] = 0;
  $48 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $48;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 return (0)|0;
}
function _growOp3($p,$op,$p1,$p2,$p3) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $p1;
 HEAP32[$4>>2] = $p2;
 HEAP32[$5>>2] = $p3;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_growOpArray($6,1)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  $13 = HEAP32[$5>>2]|0;
  $14 = (_sqlite3VdbeAddOp3($9,$10,$11,$12,$13)|0);
  HEAP32[$0>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _growOpArray($v,$nOp) {
 $v = $v|0;
 $nOp = $nOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $nNew = 0, $p = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pNew = sp + 8|0;
 $p = sp + 4|0;
 $nNew = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $nOp;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 88|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 88|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11<<1;
  $13 = $12;
 } else {
  $13 = 51;
 }
 HEAP32[$nNew>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$nNew>>2]|0;
 $20 = ($19*20)|0;
 $21 = (_sqlite3DbRealloc($15,$18,$20,0)|0);
 HEAP32[$pNew>>2] = $21;
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $39 = HEAP32[$pNew>>2]|0;
  $40 = ($39|0)!=(0|0);
  $41 = $40 ? 0 : 7;
  STACKTOP = sp;return ($41|0);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$pNew>>2]|0;
 $27 = (_sqlite3DbMallocSize($25,$26)|0);
 $28 = HEAP32[$p>>2]|0;
 $29 = ((($28)) + 92|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 92|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32>>>0) / 20)&-1;
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 88|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pNew>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 4|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$pNew>>2]|0;
 $40 = ($39|0)!=(0|0);
 $41 = $40 ? 0 : 7;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3BtreeSharable($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 9|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _vdbeChangeP4Full($p,$pOp,$zP4,$n) {
 $p = $p|0;
 $pOp = $pOp|0;
 $zP4 = $zP4|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pOp;
 HEAP32[$2>>2] = $zP4;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 1|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  _freeP4($9,$13,$16);
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 1|0);
  HEAP8[$18>>0] = 0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  HEAP32[$20>>2] = 0;
 }
 $21 = HEAP32[$3>>2]|0;
 $22 = ($21|0)<(0);
 if ($22) {
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $24;
  $29 = $27;
  $30 = (($28) - ($29))|0;
  $31 = (($30|0) / 20)&-1;
  $32 = HEAP32[$2>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  _sqlite3VdbeChangeP4($23,$31,$32,$33);
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$3>>2]|0;
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$2>>2]|0;
  $37 = (_sqlite3Strlen30($36)|0);
  HEAP32[$3>>2] = $37;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$3>>2]|0;
 $42 = ($41|0)<(0);
 $43 = $42 << 31 >> 31;
 $44 = (_sqlite3DbStrNDup($39,$40,$41,$43)|0);
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 16|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 1|0);
 HEAP8[$48>>0] = -1;
 STACKTOP = sp;return;
}
function _sqlite3DbStrNDup($db,$z,$0,$1) {
 $db = $db|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp;
 $zNew = sp + 8|0;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $z;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$2>>2] = 0;
  $41 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = $5;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (_i64Add(($15|0),($18|0),1,0)|0);
 $20 = tempRet0;
 $21 = (_sqlite3DbMallocRawNN($12,$19,$20)|0);
 HEAP32[$zNew>>2] = $21;
 $22 = HEAP32[$zNew>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$zNew>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = $5;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  _memcpy(($24|0),($25|0),($28|0))|0;
  $32 = $5;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$zNew>>2]|0;
  $39 = (($38) + ($34)|0);
  HEAP8[$39>>0] = 0;
 }
 $40 = HEAP32[$zNew>>2]|0;
 HEAP32[$2>>2] = $40;
 $41 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _resolveP2Values($p,$pMaxFuncArgs) {
 $p = $p|0;
 $pMaxFuncArgs = $pMaxFuncArgs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aLabel = 0, $i = 0;
 var $n = 0, $nMaxArgs = 0, $opcode = 0, $pOp = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $i = sp + 20|0;
 $nMaxArgs = sp + 16|0;
 $pOp = sp + 12|0;
 $pParse = sp + 8|0;
 $aLabel = sp + 4|0;
 $opcode = sp + 32|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pMaxFuncArgs;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$nMaxArgs>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = ((($7)) + 120|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aLabel>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 88|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 & -65;
 $14 = $13 | 64;
 HEAP16[$11>>1] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 88|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 & -129;
 HEAP16[$16>>1] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pOp>>2] = $21;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 32|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) - 1)|0;
 HEAP32[$i>>2] = $25;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = ($26|0)>=(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$pOp>>2]|0;
  $29 = HEAP8[$28>>0]|0;
  HEAP8[$opcode>>0] = $29;
  $30 = HEAP8[$opcode>>0]|0;
  $31 = $30&255;
  switch ($31|0) {
  case 2:  {
   $32 = HEAP32[$pOp>>2]|0;
   $33 = ((($32)) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = ($34|0)!=(0);
   if ($35) {
    $36 = HEAP32[$0>>2]|0;
    $37 = ((($36)) + 88|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38 & -65;
    HEAP16[$37>>1] = $39;
    label = 6;
   } else {
    label = 6;
   }
   break;
  }
  case 0: case 1:  {
   label = 6;
   break;
  }
  case 9: case 10: case 8:  {
   $45 = HEAP32[$0>>2]|0;
   $46 = ((($45)) + 88|0);
   $47 = HEAP16[$46>>1]|0;
   $48 = $47 & -65;
   HEAP16[$46>>1] = $48;
   $49 = HEAP32[$0>>2]|0;
   $50 = ((($49)) + 88|0);
   $51 = HEAP16[$50>>1]|0;
   $52 = $51 & -129;
   $53 = $52 | 128;
   HEAP16[$50>>1] = $53;
   break;
  }
  case 12:  {
   $54 = HEAP32[$pOp>>2]|0;
   $55 = ((($54)) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$nMaxArgs>>2]|0;
   $58 = ($56|0)>($57|0);
   if ($58) {
    $59 = HEAP32[$pOp>>2]|0;
    $60 = ((($59)) + 8|0);
    $61 = HEAP32[$60>>2]|0;
    HEAP32[$nMaxArgs>>2] = $61;
   }
   break;
  }
  case 11:  {
   $62 = HEAP32[$pOp>>2]|0;
   $63 = ((($62)) + -20|0);
   $64 = ((($63)) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$n>>2] = $65;
   $66 = HEAP32[$n>>2]|0;
   $67 = HEAP32[$nMaxArgs>>2]|0;
   $68 = ($66|0)>($67|0);
   if ($68) {
    $69 = HEAP32[$n>>2]|0;
    HEAP32[$nMaxArgs>>2] = $69;
   }
   break;
  }
  case 3: case 5: case 7:  {
   $70 = HEAP32[$pOp>>2]|0;
   $71 = ((($70)) + 16|0);
   HEAP32[$71>>2] = 182;
   $72 = HEAP32[$pOp>>2]|0;
   $73 = ((($72)) + 1|0);
   HEAP8[$73>>0] = -19;
   break;
  }
  case 4: case 6:  {
   $74 = HEAP32[$pOp>>2]|0;
   $75 = ((($74)) + 16|0);
   HEAP32[$75>>2] = 183;
   $76 = HEAP32[$pOp>>2]|0;
   $77 = ((($76)) + 1|0);
   HEAP8[$77>>0] = -19;
   break;
  }
  default: {
  }
  }
  if ((label|0) == 6) {
   label = 0;
   $40 = HEAP32[$0>>2]|0;
   $41 = ((($40)) + 88|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = $42 & -129;
   $44 = $43 | 128;
   HEAP16[$41>>1] = $44;
  }
  $78 = HEAP8[$opcode>>0]|0;
  $79 = $78&255;
  $80 = (25281 + ($79)|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = HEAP32[$pOp>>2]|0;
  $83 = ((($82)) + 2|0);
  HEAP8[$83>>0] = $81;
  $84 = HEAP32[$pOp>>2]|0;
  $85 = ((($84)) + 2|0);
  $86 = HEAP8[$85>>0]|0;
  $87 = $86&255;
  $88 = $87 & 1;
  $89 = ($88|0)!=(0);
  if ($89) {
   $90 = HEAP32[$pOp>>2]|0;
   $91 = ((($90)) + 8|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ($92|0)<(0);
   if ($93) {
    $94 = HEAP32[$pOp>>2]|0;
    $95 = ((($94)) + 8|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = (-1 - ($96))|0;
    $98 = HEAP32[$aLabel>>2]|0;
    $99 = (($98) + ($97<<2)|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = HEAP32[$pOp>>2]|0;
    $102 = ((($101)) + 8|0);
    HEAP32[$102>>2] = $100;
   }
  }
  $103 = HEAP32[$i>>2]|0;
  $104 = (($103) + -1)|0;
  HEAP32[$i>>2] = $104;
  $105 = HEAP32[$pOp>>2]|0;
  $106 = ((($105)) + 20|0);
  HEAP32[$pOp>>2] = $106;
 }
 $107 = HEAP32[$0>>2]|0;
 $108 = HEAP32[$107>>2]|0;
 $109 = HEAP32[$pParse>>2]|0;
 $110 = ((($109)) + 120|0);
 $111 = HEAP32[$110>>2]|0;
 _sqlite3DbFree($108,$111);
 $112 = HEAP32[$pParse>>2]|0;
 $113 = ((($112)) + 120|0);
 HEAP32[$113>>2] = 0;
 $114 = HEAP32[$pParse>>2]|0;
 $115 = ((($114)) + 116|0);
 HEAP32[$115>>2] = 0;
 $116 = HEAP32[$nMaxArgs>>2]|0;
 $117 = HEAP32[$1>>2]|0;
 HEAP32[$117>>2] = $116;
 STACKTOP = sp;return;
}
function _allocSpace($p,$pBuf,$nByte) {
 $p = $p|0;
 $pBuf = $pBuf|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pBuf;
 HEAP32[$2>>2] = $nByte;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $27 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + 7)|0;
 $7 = $6 & -8;
 HEAP32[$2>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)<=($11|0);
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 if ($12) {
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) - ($13))|0;
  HEAP32[$15>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($20)|0);
  HEAP32[$1>>2] = $23;
  $27 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $24 = ((($14)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($13))|0;
  HEAP32[$24>>2] = $26;
  $27 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3BtreeIncrblobCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = $4 | 16;
 $6 = $5&255;
 HEAP8[$2>>0] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 11|0);
 HEAP8[$9>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3BtreeEnterCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($2);
 STACKTOP = sp;return;
}
function _sqlite3ReadSchema($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = ((($3)) + 140|0);
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $8 = HEAP32[$db>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = (_sqlite3Init($8,$10)|0);
  HEAP32[$rc>>2] = $11;
 }
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $14 = HEAP32[$rc>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 68|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 1)|0;
 HEAP32[$18>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3VtabEponymousTableInit($pParse,$pMod) {
 $pParse = $pParse|0;
 $pMod = $pMod|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $db = 0, $nName = 0, $pModule = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0;
 var $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pModule = sp + 24|0;
 $pTab = sp + 20|0;
 $zErr = sp + 16|0;
 $nName = sp + 12|0;
 $rc = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pMod;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pModule>>2] = $4;
 HEAP32[$zErr>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $11 = HEAP32[$pModule>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$pModule>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pModule>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)!=($20|0);
  if ($21) {
   HEAP32[$0>>2] = 0;
   $93 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($93|0);
  }
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = ((($22)) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (_sqlite3Strlen30($24)|0);
 $26 = (($25) + 1)|0;
 HEAP32[$nName>>2] = $26;
 $27 = HEAP32[$db>>2]|0;
 $28 = HEAP32[$nName>>2]|0;
 $29 = (72 + ($28))|0;
 $30 = (_sqlite3DbMallocZero($27,$29,0)|0);
 HEAP32[$pTab>>2] = $30;
 $31 = HEAP32[$pTab>>2]|0;
 $32 = ($31|0)==(0|0);
 if ($32) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $33 = HEAP32[$pTab>>2]|0;
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 16|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pTab>>2]|0;
 $37 = ((($36)) + 72|0);
 $38 = HEAP32[$pTab>>2]|0;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$pTab>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$nName>>2]|0;
 _memcpy(($40|0),($43|0),($44|0))|0;
 $45 = HEAP32[$pTab>>2]|0;
 $46 = ((($45)) + 36|0);
 HEAP16[$46>>1] = 1;
 $47 = HEAP32[$db>>2]|0;
 $48 = ((($47)) + 16|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($49)) + 12|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$pTab>>2]|0;
 $53 = ((($52)) + 64|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$pTab>>2]|0;
 $55 = ((($54)) + 42|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = $56&255;
 $58 = $57 | 16;
 $59 = $58&255;
 HEAP8[$55>>0] = $59;
 $60 = HEAP32[$pTab>>2]|0;
 $61 = ((($60)) + 48|0);
 HEAP32[$61>>2] = 0;
 $62 = HEAP32[$pTab>>2]|0;
 $63 = ((($62)) + 32|0);
 HEAP16[$63>>1] = -1;
 $64 = HEAP32[$db>>2]|0;
 $65 = HEAP32[$pTab>>2]|0;
 $66 = HEAP32[$db>>2]|0;
 $67 = HEAP32[$pTab>>2]|0;
 $68 = HEAP32[$67>>2]|0;
 $69 = (_sqlite3DbStrDup($66,$68)|0);
 _addModuleArgument($64,$65,$69);
 $70 = HEAP32[$db>>2]|0;
 $71 = HEAP32[$pTab>>2]|0;
 _addModuleArgument($70,$71,0);
 $72 = HEAP32[$db>>2]|0;
 $73 = HEAP32[$pTab>>2]|0;
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$pTab>>2]|0;
 $76 = HEAP32[$75>>2]|0;
 $77 = (_sqlite3DbStrDup($74,$76)|0);
 _addModuleArgument($72,$73,$77);
 $78 = HEAP32[$db>>2]|0;
 $79 = HEAP32[$pTab>>2]|0;
 $80 = HEAP32[$2>>2]|0;
 $81 = HEAP32[$pModule>>2]|0;
 $82 = ((($81)) + 8|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (_vtabCallConstructor($78,$79,$80,$83,$zErr)|0);
 HEAP32[$rc>>2] = $84;
 $85 = HEAP32[$rc>>2]|0;
 $86 = ($85|0)!=(0);
 if ($86) {
  $87 = HEAP32[$1>>2]|0;
  $88 = HEAP32[$zErr>>2]|0;
  HEAP32[$vararg_buffer>>2] = $88;
  _sqlite3ErrorMsg($87,17521,$vararg_buffer);
  $89 = HEAP32[$db>>2]|0;
  $90 = HEAP32[$zErr>>2]|0;
  _sqlite3DbFree($89,$90);
  $91 = HEAP32[$db>>2]|0;
  $92 = HEAP32[$2>>2]|0;
  _sqlite3VtabEponymousTableClear($91,$92);
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 } else {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 return (0)|0;
}
function _addModuleArgument($db,$pTable,$zArg) {
 $db = $db|0;
 $pTable = $pTable|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $azModuleArg = 0, $i = 0, $nBytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nBytes = sp + 8|0;
 $azModuleArg = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (2 + ($5))|0;
 $7 = $6<<2;
 HEAP32[$nBytes>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$nBytes>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_sqlite3DbRealloc($8,$11,$12,$14)|0);
 HEAP32[$azModuleArg>>2] = $15;
 $16 = HEAP32[$azModuleArg>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  _sqlite3DbFree($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 48|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$21>>2] = $23;
  HEAP32[$i>>2] = $22;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$azModuleArg>>2]|0;
  $27 = (($26) + ($25<<2)|0);
  HEAP32[$27>>2] = $24;
  $28 = HEAP32[$i>>2]|0;
  $29 = (($28) + 1)|0;
  $30 = HEAP32[$azModuleArg>>2]|0;
  $31 = (($30) + ($29<<2)|0);
  HEAP32[$31>>2] = 0;
  $32 = HEAP32[$azModuleArg>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 52|0);
  HEAP32[$34>>2] = $32;
  STACKTOP = sp;return;
 }
}
function _sqlite3VtabEponymousTableClear($db,$pMod) {
 $db = $db|0;
 $pMod = $pMod|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pMod;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$pTab>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pTab>>2]|0;
 _sqlite3DeleteColumnNames($7,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$pTab>>2]|0;
 _sqlite3VtabClear($9,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$pTab>>2]|0;
 _sqlite3DbFree($11,$12);
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 16|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3DeleteColumnNames($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCol = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pCol = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCol>>2] = $4;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 34|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($6|0)<($10|0);
  $12 = HEAP32[$0>>2]|0;
  if (!($11)) {
   break;
  }
  $13 = HEAP32[$pCol>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  _sqlite3DbFree($12,$14);
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$pCol>>2]|0;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3ExprDelete($15,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$pCol>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  _sqlite3DbFree($19,$22);
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$pCol>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  _sqlite3DbFree($23,$26);
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$pCol>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3DbFree($27,$30);
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
  $33 = HEAP32[$pCol>>2]|0;
  $34 = ((($33)) + 24|0);
  HEAP32[$pCol>>2] = $34;
 }
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 _sqlite3DbFree($12,$37);
 STACKTOP = sp;return;
}
function _sqlite3VtabClear($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 528|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)==(0|0);
  if ($7) {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $8 = HEAP32[$1>>2]|0;
  (_vtabDisconnectAll(0,$8)|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 48|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)!=(1);
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 52|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<2)|0);
   $26 = HEAP32[$25>>2]|0;
   _sqlite3DbFree($20,$26);
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 52|0);
 $32 = HEAP32[$31>>2]|0;
 _sqlite3DbFree($29,$32);
 STACKTOP = sp;return;
}
function _sqlite3VtabUnlockList($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 336|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 336|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 _sqlite3ExpirePreparedStatements($8);
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pNext>>2] = $11;
  $12 = HEAP32[$p>>2]|0;
  _sqlite3VtabUnlock($12);
  $13 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $13;
  $14 = HEAP32[$p>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3CollapseDatabaseArray($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $j = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$j>>2] = 2;
 HEAP32[$i>>2] = 2;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  HEAP32[$pDb>>2] = $10;
  $11 = HEAP32[$pDb>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$pDb>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   _sqlite3DbFree($15,$17);
   $18 = HEAP32[$pDb>>2]|0;
   HEAP32[$18>>2] = 0;
  } else {
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$i>>2]|0;
   $21 = ($19|0)<($20|0);
   if ($21) {
    $22 = HEAP32[$j>>2]|0;
    $23 = HEAP32[$0>>2]|0;
    $24 = ((($23)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + ($22<<4)|0);
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$0>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    ;HEAP32[$26>>2]=HEAP32[$31>>2]|0;HEAP32[$26+4>>2]=HEAP32[$31+4>>2]|0;HEAP32[$26+8>>2]=HEAP32[$31+8>>2]|0;HEAP32[$26+12>>2]=HEAP32[$31+12>>2]|0;
   }
   $32 = HEAP32[$j>>2]|0;
   $33 = (($32) + 1)|0;
   HEAP32[$j>>2] = $33;
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$j>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 20|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 20|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)<=(2);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = ((($43)) + 16|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 460|0);
 $48 = ($45|0)!=($47|0);
 if (!($48)) {
  STACKTOP = sp;return;
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 460|0);
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 ;HEAP32[$50>>2]=HEAP32[$53>>2]|0;HEAP32[$50+4>>2]=HEAP32[$53+4>>2]|0;HEAP32[$50+8>>2]=HEAP32[$53+8>>2]|0;HEAP32[$50+12>>2]=HEAP32[$53+12>>2]|0;HEAP32[$50+16>>2]=HEAP32[$53+16>>2]|0;HEAP32[$50+20>>2]=HEAP32[$53+20>>2]|0;HEAP32[$50+24>>2]=HEAP32[$53+24>>2]|0;HEAP32[$50+28>>2]=HEAP32[$53+28>>2]|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 16|0);
 $57 = HEAP32[$56>>2]|0;
 _sqlite3DbFree($54,$57);
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 460|0);
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 16|0);
 HEAP32[$61>>2] = $59;
 STACKTOP = sp;return;
}
function _sqlite3FkDelete($db,$pTab) {
 $db = $db|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pFKey = 0, $pNext = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pFKey = sp + 12|0;
 $pNext = sp + 8|0;
 $p = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFKey>>2] = $4;
 while(1) {
  $5 = HEAP32[$pFKey>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 528|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = ($11|0)==(0|0);
   if ($12) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $13 = HEAP32[$pFKey>>2]|0;
   $14 = ((($13)) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   $17 = HEAP32[$pFKey>>2]|0;
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   if ($16) {
    $20 = HEAP32[$pFKey>>2]|0;
    $21 = ((($20)) + 16|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ((($22)) + 12|0);
    HEAP32[$23>>2] = $19;
   } else {
    HEAP32[$p>>2] = $19;
    $24 = HEAP32[$p>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$pFKey>>2]|0;
    if ($25) {
     $27 = ((($26)) + 12|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = ((($28)) + 8|0);
     $30 = HEAP32[$29>>2]|0;
     $33 = $30;
    } else {
     $31 = ((($26)) + 8|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = $32;
    }
    HEAP32[$z>>2] = $33;
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 64|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = ((($36)) + 56|0);
    $38 = HEAP32[$z>>2]|0;
    $39 = HEAP32[$p>>2]|0;
    (_sqlite3HashInsert($37,$38,$39)|0);
   }
   $40 = HEAP32[$pFKey>>2]|0;
   $41 = ((($40)) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)!=(0|0);
   if ($43) {
    $44 = HEAP32[$pFKey>>2]|0;
    $45 = ((($44)) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$pFKey>>2]|0;
    $48 = ((($47)) + 12|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ((($49)) + 16|0);
    HEAP32[$50>>2] = $46;
   }
  }
  $51 = HEAP32[$0>>2]|0;
  $52 = HEAP32[$pFKey>>2]|0;
  $53 = ((($52)) + 28|0);
  $54 = HEAP32[$53>>2]|0;
  _fkTriggerDelete($51,$54);
  $55 = HEAP32[$0>>2]|0;
  $56 = HEAP32[$pFKey>>2]|0;
  $57 = ((($56)) + 28|0);
  $58 = ((($57)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  _fkTriggerDelete($55,$59);
  $60 = HEAP32[$pFKey>>2]|0;
  $61 = ((($60)) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$pNext>>2] = $62;
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pFKey>>2]|0;
  _sqlite3DbFree($63,$64);
  $65 = HEAP32[$pNext>>2]|0;
  HEAP32[$pFKey>>2] = $65;
 }
 STACKTOP = sp;return;
}
function _fkTriggerDelete($dbMem,$p) {
 $dbMem = $dbMem|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pStep = sp;
 HEAP32[$0>>2] = $dbMem;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pStep>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pStep>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3ExprDelete($7,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$pStep>>2]|0;
 $13 = ((($12)) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3ExprListDelete($11,$14);
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$pStep>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3SelectDelete($15,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 12|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3ExprDelete($19,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($23,$24);
 STACKTOP = sp;return;
}
function _sqlite3Utf8Read($pz) {
 $pz = $pz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pz;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 1|0);
 HEAP32[$1>>2] = $3;
 $4 = HEAP8[$2>>0]|0;
 $5 = $4&255;
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6>>>0)>=(192);
 if (!($7)) {
  $35 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$c>>2]|0;
 $9 = (($8) - 192)|0;
 $10 = (19221 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 HEAP32[$c>>2] = $12;
 while(1) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = $16 & 192;
  $18 = ($17|0)==(128);
  $19 = HEAP32[$c>>2]|0;
  if (!($18)) {
   break;
  }
  $20 = $19 << 6;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 1|0);
  HEAP32[$21>>2] = $23;
  $24 = HEAP8[$22>>0]|0;
  $25 = $24&255;
  $26 = 63 & $25;
  $27 = (($20) + ($26))|0;
  HEAP32[$c>>2] = $27;
 }
 $28 = ($19>>>0)<(128);
 if (!($28)) {
  $29 = HEAP32[$c>>2]|0;
  $30 = $29 & -2048;
  $31 = ($30|0)==(55296);
  if (!($31)) {
   $32 = HEAP32[$c>>2]|0;
   $33 = $32 & -2;
   $34 = ($33|0)==(65534);
   if (!($34)) {
    $35 = HEAP32[$c>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
  }
 }
 HEAP32[$c>>2] = 65533;
 $35 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _logBadConnection($zType) {
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $zType;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 _sqlite3_log(21,25563,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3VdbeDb($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3VdbeSwap($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, $tmp = 0, $zTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 220|0;
 $1 = sp + 216|0;
 $tmp = sp;
 $pTmp = sp + 212|0;
 $zTmp = sp + 208|0;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 $2 = HEAP32[$0>>2]|0;
 _memcpy(($tmp|0),($2|0),208)|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memcpy(($3|0),($4|0),208)|0;
 $5 = HEAP32[$1>>2]|0;
 _memcpy(($5|0),($tmp|0),208)|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTmp>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 52|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$pTmp>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 52|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pTmp>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 48|0);
 HEAP32[$24>>2] = $22;
 $25 = HEAP32[$pTmp>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 48|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 168|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$zTmp>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 168|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 168|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$zTmp>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 168|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 88|0);
 $41 = HEAP16[$40>>1]|0;
 $42 = ($41&65535) >>> 8;
 $43 = $42 & 1;
 $44 = $43&65535;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 88|0);
 $47 = $44&65535;
 $48 = HEAP16[$46>>1]|0;
 $49 = $47 & 1;
 $50 = ($49 << 8)&65535;
 $51 = $48 & -257;
 $52 = $51 | $50;
 HEAP16[$46>>1] = $52;
 STACKTOP = sp;return;
}
function _sqlite3VdbeResetStepResult($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 HEAP32[$2>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Prepare($db,$zSql,$nBytes,$saveSqlFlag,$pReprepare,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $saveSqlFlag = $saveSqlFlag|0;
 $pReprepare = $pReprepare|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $i = 0, $iFirst = 0, $mx = 0, $mxLen = 0, $pBt = 0, $pParse = 0, $pT = 0, $pVdbe = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $zDb = 0, $zErrMsg = 0, $zSqlCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $4 = sp + 76|0;
 $5 = sp + 72|0;
 $6 = sp + 68|0;
 $pParse = sp + 64|0;
 $zErrMsg = sp + 60|0;
 $rc = sp + 56|0;
 $i = sp + 52|0;
 $pBt = sp + 48|0;
 $zDb = sp + 44|0;
 $zSqlCopy = sp + 40|0;
 $mxLen = sp + 36|0;
 $iFirst = sp + 32|0;
 $mx = sp + 28|0;
 $pVdbe = sp + 24|0;
 $pT = sp + 20|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $saveSqlFlag;
 HEAP32[$4>>2] = $pReprepare;
 HEAP32[$5>>2] = $ppStmt;
 HEAP32[$6>>2] = $pzTail;
 HEAP32[$zErrMsg>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3DbMallocZero($7,544,0)|0);
 HEAP32[$pParse>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$rc>>2] = 7;
  $216 = HEAP32[$pParse>>2]|0;
  _sqlite3ParserReset($216);
  $217 = HEAP32[$0>>2]|0;
  $218 = HEAP32[$pParse>>2]|0;
  _sqlite3DbFree($217,$218);
  $219 = HEAP32[$0>>2]|0;
  $220 = HEAP32[$rc>>2]|0;
  $221 = (_sqlite3ApiExit($219,$220)|0);
  HEAP32[$rc>>2] = $221;
  $222 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $11 = HEAP32[$4>>2]|0;
 $12 = HEAP32[$pParse>>2]|0;
 $13 = ((($12)) + 480|0);
 HEAP32[$13>>2] = $11;
 HEAP32[$i>>2] = 0;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($14|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19<<4)|0);
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pBt>>2] = $25;
  $26 = HEAP32[$pBt>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[$pBt>>2]|0;
   $29 = (_sqlite3BtreeSchemaLocked($28)|0);
   HEAP32[$rc>>2] = $29;
   $30 = HEAP32[$rc>>2]|0;
   $31 = ($30|0)!=(0);
   if ($31) {
    label = 7;
    break;
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 16|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + ($32<<4)|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$zDb>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$rc>>2]|0;
  $40 = HEAP32[$zDb>>2]|0;
  HEAP32[$vararg_buffer>>2] = $40;
  _sqlite3ErrorWithMsg($38,$39,25608,$vararg_buffer);
  $216 = HEAP32[$pParse>>2]|0;
  _sqlite3ParserReset($216);
  $217 = HEAP32[$0>>2]|0;
  $218 = HEAP32[$pParse>>2]|0;
  _sqlite3DbFree($217,$218);
  $219 = HEAP32[$0>>2]|0;
  $220 = HEAP32[$rc>>2]|0;
  $221 = (_sqlite3ApiExit($219,$220)|0);
  HEAP32[$rc>>2] = $221;
  $222 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $43 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$pParse>>2]|0;
 HEAP32[$45>>2] = $44;
 $46 = HEAP32[$pParse>>2]|0;
 $47 = ((($46)) + 428|0);
 HEAP32[$47>>2] = 0;
 $48 = HEAP32[$2>>2]|0;
 $49 = ($48|0)>=(0);
 do {
  if ($49) {
   $50 = HEAP32[$2>>2]|0;
   $51 = ($50|0)==(0);
   if (!($51)) {
    $52 = HEAP32[$2>>2]|0;
    $53 = (($52) - 1)|0;
    $54 = HEAP32[$1>>2]|0;
    $55 = (($54) + ($53)|0);
    $56 = HEAP8[$55>>0]|0;
    $57 = $56 << 24 >> 24;
    $58 = ($57|0)!=(0);
    if (!($58)) {
     label = 17;
     break;
    }
   }
   $59 = HEAP32[$0>>2]|0;
   $60 = ((($59)) + 88|0);
   $61 = ((($60)) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$mxLen>>2] = $62;
   $63 = HEAP32[$2>>2]|0;
   $64 = HEAP32[$mxLen>>2]|0;
   $65 = ($63|0)>($64|0);
   $66 = HEAP32[$0>>2]|0;
   if ($65) {
    _sqlite3ErrorWithMsg($66,18,25638,$vararg_buffer1);
    $67 = HEAP32[$0>>2]|0;
    $68 = (_sqlite3ApiExit($67,18)|0);
    HEAP32[$rc>>2] = $68;
    $216 = HEAP32[$pParse>>2]|0;
    _sqlite3ParserReset($216);
    $217 = HEAP32[$0>>2]|0;
    $218 = HEAP32[$pParse>>2]|0;
    _sqlite3DbFree($217,$218);
    $219 = HEAP32[$0>>2]|0;
    $220 = HEAP32[$rc>>2]|0;
    $221 = (_sqlite3ApiExit($219,$220)|0);
    HEAP32[$rc>>2] = $221;
    $222 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($222|0);
   }
   $69 = HEAP32[$1>>2]|0;
   $70 = HEAP32[$2>>2]|0;
   $71 = ($70|0)<(0);
   $72 = $71 << 31 >> 31;
   $73 = (_sqlite3DbStrNDup($66,$69,$70,$72)|0);
   HEAP32[$zSqlCopy>>2] = $73;
   $74 = HEAP32[$zSqlCopy>>2]|0;
   $75 = ($74|0)!=(0|0);
   if ($75) {
    $76 = HEAP32[$pParse>>2]|0;
    $77 = HEAP32[$zSqlCopy>>2]|0;
    (_sqlite3RunParser($76,$77,$zErrMsg)|0);
    $78 = HEAP32[$pParse>>2]|0;
    $79 = ((($78)) + 484|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = HEAP32[$zSqlCopy>>2]|0;
    $82 = $80;
    $83 = $81;
    $84 = (($82) - ($83))|0;
    $85 = HEAP32[$1>>2]|0;
    $86 = (($85) + ($84)|0);
    $87 = HEAP32[$pParse>>2]|0;
    $88 = ((($87)) + 484|0);
    HEAP32[$88>>2] = $86;
    $89 = HEAP32[$0>>2]|0;
    $90 = HEAP32[$zSqlCopy>>2]|0;
    _sqlite3DbFree($89,$90);
    break;
   } else {
    $91 = HEAP32[$2>>2]|0;
    $92 = HEAP32[$1>>2]|0;
    $93 = (($92) + ($91)|0);
    $94 = HEAP32[$pParse>>2]|0;
    $95 = ((($94)) + 484|0);
    HEAP32[$95>>2] = $93;
    break;
   }
  } else {
   label = 17;
  }
 } while(0);
 if ((label|0) == 17) {
  $96 = HEAP32[$pParse>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  (_sqlite3RunParser($96,$97,$zErrMsg)|0);
 }
 $98 = HEAP32[$pParse>>2]|0;
 $99 = ((($98)) + 12|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100|0)==(101);
 if ($101) {
  $102 = HEAP32[$pParse>>2]|0;
  $103 = ((($102)) + 12|0);
  HEAP32[$103>>2] = 0;
 }
 $104 = HEAP32[$pParse>>2]|0;
 $105 = ((($104)) + 17|0);
 $106 = HEAP8[$105>>0]|0;
 $107 = ($106<<24>>24)!=(0);
 if ($107) {
  $108 = HEAP32[$pParse>>2]|0;
  _schemaIsValid($108);
 }
 $109 = HEAP32[$0>>2]|0;
 $110 = ((($109)) + 65|0);
 $111 = HEAP8[$110>>0]|0;
 $112 = ($111<<24>>24)!=(0);
 if ($112) {
  $113 = HEAP32[$pParse>>2]|0;
  $114 = ((($113)) + 12|0);
  HEAP32[$114>>2] = 7;
 }
 $115 = HEAP32[$6>>2]|0;
 $116 = ($115|0)!=(0|0);
 if ($116) {
  $117 = HEAP32[$pParse>>2]|0;
  $118 = ((($117)) + 484|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = HEAP32[$6>>2]|0;
  HEAP32[$120>>2] = $119;
 }
 $121 = HEAP32[$pParse>>2]|0;
 $122 = ((($121)) + 12|0);
 $123 = HEAP32[$122>>2]|0;
 HEAP32[$rc>>2] = $123;
 $124 = HEAP32[$rc>>2]|0;
 $125 = ($124|0)==(0);
 L42: do {
  if ($125) {
   $126 = HEAP32[$pParse>>2]|0;
   $127 = ((($126)) + 8|0);
   $128 = HEAP32[$127>>2]|0;
   $129 = ($128|0)!=(0|0);
   if ($129) {
    $130 = HEAP32[$pParse>>2]|0;
    $131 = ((($130)) + 453|0);
    $132 = HEAP8[$131>>0]|0;
    $133 = $132&255;
    $134 = ($133|0)!=(0);
    if ($134) {
     $135 = HEAP32[$pParse>>2]|0;
     $136 = ((($135)) + 453|0);
     $137 = HEAP8[$136>>0]|0;
     $138 = $137&255;
     $139 = ($138|0)==(2);
     $140 = HEAP32[$pParse>>2]|0;
     $141 = ((($140)) + 8|0);
     $142 = HEAP32[$141>>2]|0;
     if ($139) {
      _sqlite3VdbeSetNumCols($142,4);
      HEAP32[$iFirst>>2] = 8;
      HEAP32[$mx>>2] = 12;
     } else {
      _sqlite3VdbeSetNumCols($142,8);
      HEAP32[$iFirst>>2] = 0;
      HEAP32[$mx>>2] = 8;
     }
     $143 = HEAP32[$iFirst>>2]|0;
     HEAP32[$i>>2] = $143;
     while(1) {
      $144 = HEAP32[$i>>2]|0;
      $145 = HEAP32[$mx>>2]|0;
      $146 = ($144|0)<($145|0);
      if (!($146)) {
       break L42;
      }
      $147 = HEAP32[$pParse>>2]|0;
      $148 = ((($147)) + 8|0);
      $149 = HEAP32[$148>>2]|0;
      $150 = HEAP32[$i>>2]|0;
      $151 = HEAP32[$iFirst>>2]|0;
      $152 = (($150) - ($151))|0;
      $153 = HEAP32[$i>>2]|0;
      $154 = (2460 + ($153<<2)|0);
      $155 = HEAP32[$154>>2]|0;
      (_sqlite3VdbeSetColName($149,$152,0,$155,0)|0);
      $156 = HEAP32[$i>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$i>>2] = $157;
     }
    }
   }
  }
 } while(0);
 $158 = HEAP32[$0>>2]|0;
 $159 = ((($158)) + 140|0);
 $160 = ((($159)) + 5|0);
 $161 = HEAP8[$160>>0]|0;
 $162 = $161&255;
 $163 = ($162|0)==(0);
 if ($163) {
  $164 = HEAP32[$pParse>>2]|0;
  $165 = ((($164)) + 8|0);
  $166 = HEAP32[$165>>2]|0;
  HEAP32[$pVdbe>>2] = $166;
  $167 = HEAP32[$pVdbe>>2]|0;
  $168 = HEAP32[$1>>2]|0;
  $169 = HEAP32[$pParse>>2]|0;
  $170 = ((($169)) + 484|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = HEAP32[$1>>2]|0;
  $173 = $171;
  $174 = $172;
  $175 = (($173) - ($174))|0;
  $176 = HEAP32[$3>>2]|0;
  _sqlite3VdbeSetSql($167,$168,$175,$176);
 }
 $177 = HEAP32[$pParse>>2]|0;
 $178 = ((($177)) + 8|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = ($179|0)!=(0|0);
 do {
  if ($180) {
   $181 = HEAP32[$rc>>2]|0;
   $182 = ($181|0)!=(0);
   if (!($182)) {
    $183 = HEAP32[$0>>2]|0;
    $184 = ((($183)) + 65|0);
    $185 = HEAP8[$184>>0]|0;
    $186 = $185&255;
    $187 = ($186|0)!=(0);
    if (!($187)) {
     label = 41;
     break;
    }
   }
   $188 = HEAP32[$pParse>>2]|0;
   $189 = ((($188)) + 8|0);
   $190 = HEAP32[$189>>2]|0;
   (_sqlite3VdbeFinalize($190)|0);
  } else {
   label = 41;
  }
 } while(0);
 if ((label|0) == 41) {
  $191 = HEAP32[$pParse>>2]|0;
  $192 = ((($191)) + 8|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = HEAP32[$5>>2]|0;
  HEAP32[$194>>2] = $193;
 }
 $195 = HEAP32[$zErrMsg>>2]|0;
 $196 = ($195|0)!=(0|0);
 $197 = HEAP32[$0>>2]|0;
 $198 = HEAP32[$rc>>2]|0;
 if ($196) {
  $199 = HEAP32[$zErrMsg>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $199;
  _sqlite3ErrorWithMsg($197,$198,17521,$vararg_buffer3);
  $200 = HEAP32[$0>>2]|0;
  $201 = HEAP32[$zErrMsg>>2]|0;
  _sqlite3DbFree($200,$201);
 } else {
  _sqlite3Error($197,$198);
 }
 while(1) {
  $202 = HEAP32[$pParse>>2]|0;
  $203 = ((($202)) + 532|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = ($204|0)!=(0|0);
  if (!($205)) {
   break;
  }
  $206 = HEAP32[$pParse>>2]|0;
  $207 = ((($206)) + 532|0);
  $208 = HEAP32[$207>>2]|0;
  HEAP32[$pT>>2] = $208;
  $209 = HEAP32[$pT>>2]|0;
  $210 = ((($209)) + 4|0);
  $211 = HEAP32[$210>>2]|0;
  $212 = HEAP32[$pParse>>2]|0;
  $213 = ((($212)) + 532|0);
  HEAP32[$213>>2] = $211;
  $214 = HEAP32[$0>>2]|0;
  $215 = HEAP32[$pT>>2]|0;
  _sqlite3DbFree($214,$215);
 }
 $216 = HEAP32[$pParse>>2]|0;
 _sqlite3ParserReset($216);
 $217 = HEAP32[$0>>2]|0;
 $218 = HEAP32[$pParse>>2]|0;
 _sqlite3DbFree($217,$218);
 $219 = HEAP32[$0>>2]|0;
 $220 = HEAP32[$rc>>2]|0;
 $221 = (_sqlite3ApiExit($219,$220)|0);
 HEAP32[$rc>>2] = $221;
 $222 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($222|0);
}
function _sqlite3BtreeSchemaLocked($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_querySharedCacheTableLock($2,1,1)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _schemaIsValid($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cookie = 0, $db = 0, $iDb = 0, $openedTransaction = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $db = sp + 20|0;
 $iDb = sp + 16|0;
 $rc = sp + 12|0;
 $cookie = sp + 8|0;
 $openedTransaction = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 HEAP32[$iDb>>2] = 0;
 while(1) {
  $3 = HEAP32[$iDb>>2]|0;
  $4 = HEAP32[$db>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 14;
   break;
  }
  HEAP32[$openedTransaction>>2] = 0;
  $8 = HEAP32[$iDb>>2]|0;
  $9 = HEAP32[$db>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pBt>>2] = $14;
  $15 = HEAP32[$pBt>>2]|0;
  $16 = ($15|0)==(0|0);
  if (!($16)) {
   $17 = HEAP32[$pBt>>2]|0;
   $18 = (_sqlite3BtreeIsInReadTrans($17)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$pBt>>2]|0;
    $21 = (_sqlite3BtreeBeginTrans($20,0)|0);
    HEAP32[$rc>>2] = $21;
    $22 = HEAP32[$rc>>2]|0;
    $23 = ($22|0)==(7);
    $24 = HEAP32[$rc>>2]|0;
    $25 = ($24|0)==(3082);
    $or$cond = $23 | $25;
    if ($or$cond) {
     $26 = HEAP32[$db>>2]|0;
     _sqlite3OomFault($26);
    }
    $27 = HEAP32[$rc>>2]|0;
    $28 = ($27|0)!=(0);
    if ($28) {
     label = 14;
     break;
    }
    HEAP32[$openedTransaction>>2] = 1;
   }
   $29 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeGetMeta($29,1,$cookie);
   $30 = HEAP32[$cookie>>2]|0;
   $31 = HEAP32[$iDb>>2]|0;
   $32 = HEAP32[$db>>2]|0;
   $33 = ((($32)) + 16|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<4)|0);
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = ($30|0)!=($38|0);
   if ($39) {
    $40 = HEAP32[$db>>2]|0;
    $41 = HEAP32[$iDb>>2]|0;
    _sqlite3ResetOneSchema($40,$41);
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 12|0);
    HEAP32[$43>>2] = 17;
   }
   $44 = HEAP32[$openedTransaction>>2]|0;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$pBt>>2]|0;
    (_sqlite3BtreeCommit($46)|0);
   }
  }
  $47 = HEAP32[$iDb>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$iDb>>2] = $48;
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeSetNumCols($p,$nResColumn) {
 $p = $p|0;
 $nResColumn = $nResColumn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $pColName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pColName = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nResColumn;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 84|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10<<1;
 _releaseMemArray($6,$11);
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 _sqlite3DbFree($12,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = $16<<1;
 HEAP32[$n>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = $18&65535;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 84|0);
 HEAP16[$21>>1] = $19;
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$n>>2]|0;
 $24 = ($23*40)|0;
 $25 = (_sqlite3DbMallocZero($22,$24,0)|0);
 HEAP32[$pColName>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 16|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(0|0);
 if ($31) {
  STACKTOP = sp;return;
 }
 while(1) {
  $32 = HEAP32[$n>>2]|0;
  $33 = (($32) + -1)|0;
  HEAP32[$n>>2] = $33;
  $34 = ($32|0)>(0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$pColName>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP16[$36>>1] = 1;
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$pColName>>2]|0;
  $40 = ((($39)) + 32|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$pColName>>2]|0;
  $42 = ((($41)) + 40|0);
  HEAP32[$pColName>>2] = $42;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeSetColName($p,$idx,$var,$zName,$xDel) {
 $p = $p|0;
 $idx = $idx|0;
 $var = $var|0;
 $zName = $zName|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColName = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $rc = sp + 4|0;
 $pColName = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $idx;
 HEAP32[$3>>2] = $var;
 HEAP32[$4>>2] = $zName;
 HEAP32[$5>>2] = $xDel;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 65|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 84|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15&65535;
  $17 = Math_imul($12, $16)|0;
  $18 = (($11) + ($17))|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + (($18*40)|0)|0);
  HEAP32[$pColName>>2] = $22;
  $23 = HEAP32[$pColName>>2]|0;
  $24 = HEAP32[$4>>2]|0;
  $25 = HEAP32[$5>>2]|0;
  $26 = (_sqlite3VdbeMemSetStr($23,$24,-1,1,$25)|0);
  HEAP32[$rc>>2] = $26;
  $27 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSetSql($p,$z,$n,$isPrepareV2) {
 $p = $p|0;
 $z = $z|0;
 $n = $n|0;
 $isPrepareV2 = $isPrepareV2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $isPrepareV2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3DbStrNDup($7,$8,$9,$11)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 168|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$3>>2]|0;
 $16 = $15&255;
 $17 = $16&255;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 88|0);
 $20 = $17&65535;
 $21 = HEAP16[$19>>1]|0;
 $22 = $20 & 1;
 $23 = ($22 << 8)&65535;
 $24 = $21 & -257;
 $25 = $24 | $23;
 HEAP16[$19>>1] = $25;
 STACKTOP = sp;return;
}
function _sqlite3Utf8CharLen($zIn,$nByte) {
 $zIn = $zIn|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $z = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $r = sp + 8|0;
 $z = sp + 4|0;
 $zTerm = sp;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$r>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>=(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$z>>2]|0;
  $7 = (($6) + ($5)|0);
  HEAP32[$zTerm>>2] = $7;
 } else {
  HEAP32[$zTerm>>2] = (-1);
 }
 while(1) {
  $8 = HEAP32[$z>>2]|0;
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   label = 10;
   break;
  }
  $12 = HEAP32[$z>>2]|0;
  $13 = HEAP32[$zTerm>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if (!($14)) {
   label = 10;
   break;
  }
  $15 = HEAP32[$z>>2]|0;
  $16 = ((($15)) + 1|0);
  HEAP32[$z>>2] = $16;
  $17 = HEAP8[$15>>0]|0;
  $18 = $17&255;
  $19 = ($18|0)>=(192);
  L8: do {
   if ($19) {
    while(1) {
     $20 = HEAP32[$z>>2]|0;
     $21 = HEAP8[$20>>0]|0;
     $22 = $21&255;
     $23 = $22 & 192;
     $24 = ($23|0)==(128);
     if (!($24)) {
      break L8;
     }
     $25 = HEAP32[$z>>2]|0;
     $26 = ((($25)) + 1|0);
     HEAP32[$z>>2] = $26;
    }
   }
  } while(0);
  $27 = HEAP32[$r>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$r>>2] = $28;
 }
 if ((label|0) == 10) {
  $29 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _sqlite3DeleteTriggerStep($db,$pTriggerStep) {
 $db = $db|0;
 $pTriggerStep = $pTriggerStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTriggerStep;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$1>>2]|0;
  HEAP32[$pTmp>>2] = $4;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 28|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$1>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$pTmp>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3ExprDelete($8,$11);
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pTmp>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3ExprListDelete($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pTmp>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3SelectDelete($16,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$pTmp>>2]|0;
  $22 = ((($21)) + 24|0);
  $23 = HEAP32[$22>>2]|0;
  _sqlite3IdListDelete($20,$23);
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$pTmp>>2]|0;
  _sqlite3DbFree($24,$25);
 }
 STACKTOP = sp;return;
}
function _sqlite3ParserAlloc($mallocProc) {
 $mallocProc = $mallocProc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pParser = sp;
 HEAP32[$0>>2] = $mallocProc;
 $1 = HEAP32[$0>>2]|0;
 $2 = (FUNCTION_TABLE_iii[$1 & 255](1612,0)|0);
 HEAP32[$pParser>>2] = $2;
 $3 = HEAP32[$pParser>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $6 = HEAP32[$pParser>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $5 = HEAP32[$pParser>>2]|0;
 HEAP32[$5>>2] = -1;
 $6 = HEAP32[$pParser>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3Parser($yyp,$yymajor,$yyminor,$pParse) {
 $yyp = $yyp|0;
 $yymajor = $yymajor|0;
 $yyminor = $yyminor|0;
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yyact = 0, $yyminorunion = 0, $yyminorunion$byval_copy = 0, $yypParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $yyminorunion$byval_copy = sp + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $yyminorunion = sp + 8|0;
 $yyact = sp + 4|0;
 $yypParser = sp;
 HEAP32[$0>>2] = $yyp;
 HEAP32[$1>>2] = $yymajor;
 HEAP32[$2>>2] = $pParse;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$yypParser>>2] = $3;
 $4 = HEAP32[$yypParser>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  $7 = HEAP32[$yypParser>>2]|0;
  HEAP32[$7>>2] = 0;
  $8 = HEAP32[$yypParser>>2]|0;
  $9 = ((($8)) + 4|0);
  HEAP32[$9>>2] = -1;
  $10 = HEAP32[$yypParser>>2]|0;
  $11 = ((($10)) + 12|0);
  HEAP16[$11>>1] = 0;
  $12 = HEAP32[$yypParser>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = ((($13)) + 2|0);
  HEAP8[$14>>0] = 0;
 }
 ;HEAP32[$yyminorunion>>2]=HEAP32[$yyminor>>2]|0;HEAP32[$yyminorunion+4>>2]=HEAP32[$yyminor+4>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$yypParser>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 while(1) {
  $18 = HEAP32[$yypParser>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = $19&255;
  $21 = (_yy_find_shift_action($18,$20)|0);
  HEAP32[$yyact>>2] = $21;
  $22 = HEAP32[$yyact>>2]|0;
  $23 = ($22|0)<=(976);
  $24 = HEAP32[$yyact>>2]|0;
  do {
   if ($23) {
    $25 = ($24|0)>(435);
    if ($25) {
     $26 = HEAP32[$yyact>>2]|0;
     $27 = (($26) + 328)|0;
     HEAP32[$yyact>>2] = $27;
    }
    $28 = HEAP32[$yypParser>>2]|0;
    $29 = HEAP32[$yyact>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    _yy_shift($28,$29,$30,$yyminorunion);
    $31 = HEAP32[$yypParser>>2]|0;
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + -1)|0;
    HEAP32[$32>>2] = $34;
    HEAP32[$1>>2] = 253;
   } else {
    $35 = ($24|0)<=(1304);
    $36 = HEAP32[$yypParser>>2]|0;
    if ($35) {
     $37 = HEAP32[$yyact>>2]|0;
     $38 = (($37) - 977)|0;
     _yy_reduce($36,$38);
     break;
    } else {
     $39 = HEAP32[$1>>2]|0;
     ;HEAP32[$yyminorunion$byval_copy>>2]=HEAP32[$yyminorunion>>2]|0;HEAP32[$yyminorunion$byval_copy+4>>2]=HEAP32[$yyminorunion+4>>2]|0;HEAP32[$yyminorunion$byval_copy+8>>2]=HEAP32[$yyminorunion+8>>2]|0;
     _yy_syntax_error($36,$39,$yyminorunion$byval_copy);
     $40 = HEAP32[$yypParser>>2]|0;
     $41 = HEAP32[$1>>2]|0;
     $42 = $41&255;
     _yy_destructor($40,$42,$yyminorunion);
     HEAP32[$1>>2] = 253;
     break;
    }
   }
  } while(0);
  $43 = HEAP32[$1>>2]|0;
  $44 = ($43|0)!=(253);
  if (!($44)) {
   label = 13;
   break;
  }
  $45 = HEAP32[$yypParser>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)>=(0);
  if (!($47)) {
   label = 13;
   break;
  }
 }
 if ((label|0) == 13) {
  STACKTOP = sp;return;
 }
}
function _sqlite3ParserFree($p,$freeProc) {
 $p = $p|0;
 $freeProc = $freeProc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pParser = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $freeProc;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pParser>>2] = $2;
 while(1) {
  $3 = HEAP32[$pParser>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)>=(0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$pParser>>2]|0;
  _yy_pop_parser_stack($6);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$pParser>>2]|0;
 FUNCTION_TABLE_vi[$7 & 255]($8);
 STACKTOP = sp;return;
}
function _yy_find_shift_action($pParser,$iLookAhead) {
 $pParser = $pParser|0;
 $iLookAhead = $iLookAhead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $iFallback = 0;
 var $j = 0, $or$cond = 0, $or$cond3 = 0, $stateno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 21|0;
 $i = sp + 8|0;
 $stateno = sp + 4|0;
 $iFallback = sp + 20|0;
 $j = sp;
 HEAP32[$1>>2] = $pParser;
 HEAP8[$2>>0] = $iLookAhead;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = (($6) + ($4<<4)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 HEAP32[$stateno>>2] = $9;
 $10 = HEAP32[$stateno>>2]|0;
 $11 = ($10|0)>=(977);
 if ($11) {
  $12 = HEAP32[$stateno>>2]|0;
  HEAP32[$0>>2] = $12;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 while(1) {
  $13 = HEAP32[$stateno>>2]|0;
  $14 = (10302 + ($13<<1)|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15 << 16 >> 16;
  HEAP32[$i>>2] = $16;
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)==(-89);
  if ($18) {
   label = 4;
   break;
  }
  $23 = HEAP8[$2>>0]|0;
  $24 = $23&255;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + ($24))|0;
  HEAP32[$i>>2] = $26;
  $27 = HEAP32[$i>>2]|0;
  $28 = ($27|0)<(0);
  $29 = HEAP32[$i>>2]|0;
  $30 = ($29|0)>=(1501);
  $or$cond = $28 | $30;
  if (!($or$cond)) {
   $31 = HEAP32[$i>>2]|0;
   $32 = (25744 + ($31)|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = HEAP8[$2>>0]|0;
   $36 = $35&255;
   $37 = ($34|0)!=($36|0);
   if (!($37)) {
    label = 15;
    break;
   }
  }
  $38 = HEAP8[$2>>0]|0;
  $39 = $38&255;
  $40 = ($39|0)>(0);
  if (!($40)) {
   break;
  }
  $41 = HEAP8[$2>>0]|0;
  $42 = $41&255;
  $43 = ($42>>>0)<(70);
  if (!($43)) {
   label = 11;
   break;
  }
  $44 = HEAP8[$2>>0]|0;
  $45 = $44&255;
  $46 = (27245 + ($45)|0);
  $47 = HEAP8[$46>>0]|0;
  HEAP8[$iFallback>>0] = $47;
  $48 = $47&255;
  $49 = ($48|0)!=(0);
  if (!($49)) {
   label = 11;
   break;
  }
  $50 = HEAP8[$iFallback>>0]|0;
  HEAP8[$2>>0] = $50;
 }
 if ((label|0) == 4) {
  $19 = HEAP32[$stateno>>2]|0;
  $20 = (11174 + ($19<<1)|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21&65535;
  HEAP32[$0>>2] = $22;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 else if ((label|0) == 11) {
  $51 = HEAP32[$i>>2]|0;
  $52 = HEAP8[$2>>0]|0;
  $53 = $52&255;
  $54 = (($51) - ($53))|0;
  $55 = (($54) + 70)|0;
  HEAP32[$j>>2] = $55;
  $56 = HEAP32[$j>>2]|0;
  $57 = ($56|0)>=(0);
  $58 = HEAP32[$j>>2]|0;
  $59 = ($58|0)<(1501);
  $or$cond3 = $57 & $59;
  if ($or$cond3) {
   $60 = HEAP32[$j>>2]|0;
   $61 = (25744 + ($60)|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = ($63|0)==(70);
   if ($64) {
    $65 = HEAP32[$j>>2]|0;
    $66 = (12046 + ($65<<1)|0);
    $67 = HEAP16[$66>>1]|0;
    $68 = $67&65535;
    HEAP32[$0>>2] = $68;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
  }
 }
 else if ((label|0) == 15) {
  $73 = HEAP32[$i>>2]|0;
  $74 = (12046 + ($73<<1)|0);
  $75 = HEAP16[$74>>1]|0;
  $76 = $75&65535;
  HEAP32[$0>>2] = $76;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $69 = HEAP32[$stateno>>2]|0;
 $70 = (11174 + ($69<<1)|0);
 $71 = HEAP16[$70>>1]|0;
 $72 = $71&65535;
 HEAP32[$0>>2] = $72;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _yy_shift($yypParser,$yyNewState,$yyMajor,$yypMinor) {
 $yypParser = $yypParser|0;
 $yyNewState = $yyNewState|0;
 $yyMajor = $yyMajor|0;
 $yypMinor = $yypMinor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yytos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $yytos = sp;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yyNewState;
 HEAP32[$2>>2] = $yyMajor;
 HEAP32[$3>>2] = $yypMinor;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)>=(100);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$3>>2]|0;
  _yyStackOverflow($10,$11);
  STACKTOP = sp;return;
 } else {
  $12 = HEAP32[$10>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = (($14) + ($12<<4)|0);
  HEAP32[$yytos>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = $16&65535;
  $18 = HEAP32[$yytos>>2]|0;
  HEAP16[$18>>1] = $17;
  $19 = HEAP32[$2>>2]|0;
  $20 = $19&255;
  $21 = HEAP32[$yytos>>2]|0;
  $22 = ((($21)) + 2|0);
  HEAP8[$22>>0] = $20;
  $23 = HEAP32[$yytos>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$3>>2]|0;
  ;HEAP32[$24>>2]=HEAP32[$25>>2]|0;HEAP32[$24+4>>2]=HEAP32[$25+4>>2]|0;HEAP32[$24+8>>2]=HEAP32[$25+8>>2]|0;
  STACKTOP = sp;return;
 }
}
function _yy_reduce($yypParser,$yyruleno) {
 $yypParser = $yypParser|0;
 $yyruleno = $yyruleno|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0;
 var $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0;
 var $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0;
 var $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0;
 var $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0;
 var $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0;
 var $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0;
 var $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0;
 var $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0;
 var $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0;
 var $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0;
 var $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0;
 var $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0;
 var $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0;
 var $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0;
 var $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0;
 var $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0;
 var $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0;
 var $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0;
 var $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0;
 var $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0, $1753 = 0;
 var $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0, $1771 = 0;
 var $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0, $179 = 0;
 var $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0, $1807 = 0;
 var $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0, $1825 = 0;
 var $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0, $1843 = 0;
 var $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0, $1861 = 0;
 var $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0, $188 = 0;
 var $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0, $1898 = 0;
 var $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0, $1915 = 0;
 var $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0, $1933 = 0;
 var $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0, $1951 = 0;
 var $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0, $197 = 0;
 var $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0, $1988 = 0;
 var $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0, $2004 = 0;
 var $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0, $2022 = 0;
 var $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0, $2040 = 0;
 var $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0, $2058 = 0, $2059 = 0;
 var $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0, $2076 = 0, $2077 = 0;
 var $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0, $2094 = 0, $2095 = 0;
 var $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0, $2111 = 0, $2112 = 0;
 var $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0, $213 = 0, $2130 = 0;
 var $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0, $2148 = 0, $2149 = 0;
 var $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0, $2166 = 0, $2167 = 0;
 var $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0, $2184 = 0, $2185 = 0;
 var $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0, $2201 = 0, $2202 = 0;
 var $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0, $222 = 0, $2220 = 0;
 var $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0, $2238 = 0, $2239 = 0;
 var $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0, $2253 = 0, $2254 = 0, $2255 = 0, $2256 = 0, $2257 = 0;
 var $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0, $2274 = 0, $2275 = 0;
 var $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0, $2292 = 0, $2293 = 0;
 var $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0, $231 = 0, $2310 = 0;
 var $2311 = 0, $2312 = 0, $2313 = 0, $2314 = 0, $2315 = 0, $2316 = 0, $2317 = 0, $2318 = 0, $2319 = 0, $232 = 0, $2320 = 0, $2321 = 0, $2322 = 0, $2323 = 0, $2324 = 0, $2325 = 0, $2326 = 0, $2327 = 0, $2328 = 0, $2329 = 0;
 var $233 = 0, $2330 = 0, $2331 = 0, $2332 = 0, $2333 = 0, $2334 = 0, $2335 = 0, $2336 = 0, $2337 = 0, $2338 = 0, $2339 = 0, $234 = 0, $2340 = 0, $2341 = 0, $2342 = 0, $2343 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0;
 var $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0;
 var $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0;
 var $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0;
 var $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0;
 var $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0;
 var $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0;
 var $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0;
 var $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0;
 var $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0;
 var $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0;
 var $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0;
 var $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0;
 var $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0;
 var $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0;
 var $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0;
 var $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0;
 var $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0;
 var $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0;
 var $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0;
 var $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0;
 var $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0;
 var $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0;
 var $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0;
 var $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0;
 var $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0;
 var $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0;
 var $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0;
 var $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0;
 var $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0;
 var $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0;
 var $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0;
 var $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0;
 var $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0;
 var $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0;
 var $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0;
 var $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0;
 var $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0;
 var $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0;
 var $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0;
 var $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0;
 var $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0;
 var $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0;
 var $996 = 0, $997 = 0, $998 = 0, $999 = 0, $all = 0, $dest = 0, $p = 0, $p2 = 0, $p9 = 0, $pDot = 0, $pFrom = 0, $pLeft = 0, $pLeft4 = 0, $pLhs = 0, $pList = 0, $pList7 = 0, $pList8 = 0, $pNew = 0, $pOld = 0, $pParse = 0;
 var $pRHS = 0, $pRhs = 0, $pRight = 0, $pRight3 = 0, $pSrc = 0, $pSubquery = 0, $temp1 = 0, $temp15 = 0, $temp2 = 0, $temp26 = 0, $temp3 = 0, $temp4 = 0, $v = 0, $v1 = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0;
 var $vararg_ptr1 = 0, $x = 0, $yyact = 0, $yygoto = 0, $yygotominor = 0, $yymsp = 0, $yysize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 248|0;
 $1 = sp + 244|0;
 $yygoto = sp + 240|0;
 $yyact = sp + 236|0;
 $yygotominor = sp + 224|0;
 $yymsp = sp + 220|0;
 $yysize = sp + 216|0;
 $pParse = sp + 212|0;
 $v = sp + 200|0;
 $v1 = sp + 188|0;
 $dest = sp + 168|0;
 $p = sp + 164|0;
 $pRhs = sp + 160|0;
 $pLhs = sp + 156|0;
 $pFrom = sp + 152|0;
 $x = sp + 144|0;
 $pRight = sp + 136|0;
 $pLeft = sp + 132|0;
 $p2 = sp + 128|0;
 $pRight3 = sp + 124|0;
 $pLeft4 = sp + 120|0;
 $pDot = sp + 116|0;
 $pNew = sp + 112|0;
 $pOld = sp + 108|0;
 $pSubquery = sp + 104|0;
 $temp1 = sp + 100|0;
 $temp2 = sp + 96|0;
 $temp15 = sp + 92|0;
 $temp26 = sp + 88|0;
 $temp3 = sp + 84|0;
 $temp4 = sp + 80|0;
 $pList = sp + 76|0;
 $pList7 = sp + 72|0;
 $pList8 = sp + 68|0;
 $pRHS = sp + 64|0;
 $pSrc = sp + 60|0;
 $p9 = sp + 56|0;
 $all = sp + 48|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yyruleno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = (($8) + ($6<<4)|0);
 HEAP32[$yymsp>>2] = $9;
 ;HEAP32[$yygotominor>>2]=HEAP32[2544>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[2544+4>>2]|0;HEAP32[$yygotominor+8>>2]=HEAP32[2544+8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 L1: do {
  switch ($10|0) {
  case 6:  {
   $11 = HEAP32[$pParse>>2]|0;
   $12 = ((($11)) + 453|0);
   HEAP8[$12>>0] = 1;
   break;
  }
  case 7:  {
   $13 = HEAP32[$pParse>>2]|0;
   $14 = ((($13)) + 453|0);
   HEAP8[$14>>0] = 2;
   break;
  }
  case 8:  {
   $15 = HEAP32[$pParse>>2]|0;
   _sqlite3FinishCoding($15);
   break;
  }
  case 9:  {
   $16 = HEAP32[$pParse>>2]|0;
   $17 = HEAP32[$yymsp>>2]|0;
   $18 = ((($17)) + -16|0);
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   _sqlite3BeginTransaction($16,$20);
   break;
  }
  case 13:  {
   HEAP32[$yygotominor>>2] = 7;
   break;
  }
  case 117: case 115: case 16: case 15: case 14:  {
   $21 = HEAP32[$yymsp>>2]|0;
   $22 = ((($21)) + 2|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   HEAP32[$yygotominor>>2] = $24;
   break;
  }
  case 18: case 17:  {
   $25 = HEAP32[$pParse>>2]|0;
   _sqlite3CommitTransaction($25);
   break;
  }
  case 19:  {
   $26 = HEAP32[$pParse>>2]|0;
   _sqlite3RollbackTransaction($26);
   break;
  }
  case 22:  {
   $27 = HEAP32[$pParse>>2]|0;
   $28 = HEAP32[$yymsp>>2]|0;
   $29 = ((($28)) + 4|0);
   _sqlite3Savepoint($27,0,$29);
   break;
  }
  case 23:  {
   $30 = HEAP32[$pParse>>2]|0;
   $31 = HEAP32[$yymsp>>2]|0;
   $32 = ((($31)) + 4|0);
   _sqlite3Savepoint($30,1,$32);
   break;
  }
  case 24:  {
   $33 = HEAP32[$pParse>>2]|0;
   $34 = HEAP32[$yymsp>>2]|0;
   $35 = ((($34)) + 4|0);
   _sqlite3Savepoint($33,2,$35);
   break;
  }
  case 26:  {
   $36 = HEAP32[$pParse>>2]|0;
   $37 = HEAP32[$yymsp>>2]|0;
   $38 = ((($37)) + -16|0);
   $39 = ((($38)) + 4|0);
   $40 = HEAP32[$yymsp>>2]|0;
   $41 = ((($40)) + 4|0);
   $42 = HEAP32[$yymsp>>2]|0;
   $43 = ((($42)) + -64|0);
   $44 = ((($43)) + 4|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$yymsp>>2]|0;
   $47 = ((($46)) + -32|0);
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   _sqlite3StartTable($36,$39,$41,$45,0,0,$49);
   break;
  }
  case 27:  {
   $50 = HEAP32[$pParse>>2]|0;
   _disableLookaside($50);
   $51 = HEAP32[$yymsp>>2]|0;
   $52 = ((($51)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$52>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$52+4>>2]|0;
   break;
  }
  case 247: case 222: case 219: case 124: case 108: case 97: case 85: case 83: case 81: case 68: case 34: case 31: case 28:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 248: case 223: case 220: case 107: case 84: case 69: case 30: case 29:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 32:  {
   $53 = HEAP32[$pParse>>2]|0;
   $54 = HEAP32[$yymsp>>2]|0;
   $55 = ((($54)) + -32|0);
   $56 = ((($55)) + 4|0);
   $57 = HEAP32[$yymsp>>2]|0;
   $58 = ((($57)) + -16|0);
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$yymsp>>2]|0;
   $61 = ((($60)) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = $62&255;
   _sqlite3EndTable($53,$56,$59,$63,0);
   break;
  }
  case 33:  {
   $64 = HEAP32[$pParse>>2]|0;
   $65 = HEAP32[$yymsp>>2]|0;
   $66 = ((($65)) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   _sqlite3EndTable($64,0,0,0,$67);
   $68 = HEAP32[$pParse>>2]|0;
   $69 = HEAP32[$68>>2]|0;
   $70 = HEAP32[$yymsp>>2]|0;
   $71 = ((($70)) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   _sqlite3SelectDelete($69,$72);
   break;
  }
  case 35:  {
   $73 = HEAP32[$yymsp>>2]|0;
   $74 = ((($73)) + 4|0);
   $75 = ((($74)) + 4|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = ($76|0)==(5);
   if ($77) {
    $78 = HEAP32[$yymsp>>2]|0;
    $79 = ((($78)) + 4|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (_sqlite3_strnicmp($80,27337,5)|0);
    $82 = ($81|0)==(0);
    if ($82) {
     HEAP32[$yygotominor>>2] = 96;
     break L1;
    }
   }
   HEAP32[$yygotominor>>2] = 0;
   $83 = HEAP32[$pParse>>2]|0;
   $84 = HEAP32[$yymsp>>2]|0;
   $85 = ((($84)) + 4|0);
   $86 = ((($85)) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$yymsp>>2]|0;
   $89 = ((($88)) + 4|0);
   $90 = HEAP32[$89>>2]|0;
   HEAP32[$vararg_buffer>>2] = $87;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $90;
   _sqlite3ErrorMsg($83,27343,$vararg_buffer);
   break;
  }
  case 38:  {
   $91 = HEAP32[$yymsp>>2]|0;
   $92 = ((($91)) + -32|0);
   $93 = ((($92)) + 4|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$yygotominor>>2] = $94;
   $95 = HEAP32[$pParse>>2]|0;
   $96 = ((($95)) + 508|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = HEAP32[$yymsp>>2]|0;
   $99 = ((($98)) + -32|0);
   $100 = ((($99)) + 4|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = $97;
   $103 = $101;
   $104 = (($102) - ($103))|0;
   $105 = HEAP32[$pParse>>2]|0;
   $106 = ((($105)) + 508|0);
   $107 = ((($106)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (($104) + ($108))|0;
   $110 = ((($yygotominor)) + 4|0);
   HEAP32[$110>>2] = $109;
   break;
  }
  case 39:  {
   $111 = HEAP32[$pParse>>2]|0;
   $112 = HEAP32[$yymsp>>2]|0;
   $113 = ((($112)) + 4|0);
   _sqlite3AddColumn($111,$113);
   $114 = HEAP32[$yymsp>>2]|0;
   $115 = ((($114)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$115>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$115+4>>2]|0;
   $116 = HEAP32[$pParse>>2]|0;
   $117 = ((($116)) + 328|0);
   $118 = ((($117)) + 4|0);
   HEAP32[$118>>2] = 0;
   break;
  }
  case 280: case 264: case 263: case 262: case 261: case 260: case 259: case 258: case 257: case 151: case 142: case 131: case 130: case 48: case 45: case 42: case 41: case 40:  {
   $119 = HEAP32[$yymsp>>2]|0;
   $120 = ((($119)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$120>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$120+4>>2]|0;
   break;
  }
  case 44:  {
   $121 = HEAP32[$pParse>>2]|0;
   $122 = HEAP32[$yymsp>>2]|0;
   $123 = ((($122)) + 4|0);
   _sqlite3AddColumnType($121,$123);
   break;
  }
  case 46:  {
   $124 = HEAP32[$yymsp>>2]|0;
   $125 = ((($124)) + -48|0);
   $126 = ((($125)) + 4|0);
   $127 = HEAP32[$126>>2]|0;
   HEAP32[$yygotominor>>2] = $127;
   $128 = HEAP32[$yymsp>>2]|0;
   $129 = ((($128)) + 4|0);
   $130 = ((($129)) + 4|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = HEAP32[$yymsp>>2]|0;
   $133 = ((($132)) + 4|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = (($134) + ($131)|0);
   $136 = HEAP32[$yymsp>>2]|0;
   $137 = ((($136)) + -48|0);
   $138 = ((($137)) + 4|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $135;
   $141 = $139;
   $142 = (($140) - ($141))|0;
   $143 = ((($yygotominor)) + 4|0);
   HEAP32[$143>>2] = $142;
   break;
  }
  case 47:  {
   $144 = HEAP32[$yymsp>>2]|0;
   $145 = ((($144)) + -80|0);
   $146 = ((($145)) + 4|0);
   $147 = HEAP32[$146>>2]|0;
   HEAP32[$yygotominor>>2] = $147;
   $148 = HEAP32[$yymsp>>2]|0;
   $149 = ((($148)) + 4|0);
   $150 = ((($149)) + 4|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = HEAP32[$yymsp>>2]|0;
   $153 = ((($152)) + 4|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = (($154) + ($151)|0);
   $156 = HEAP32[$yymsp>>2]|0;
   $157 = ((($156)) + -80|0);
   $158 = ((($157)) + 4|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = $155;
   $161 = $159;
   $162 = (($160) - ($161))|0;
   $163 = ((($yygotominor)) + 4|0);
   HEAP32[$163>>2] = $162;
   break;
  }
  case 49:  {
   $164 = HEAP32[$yymsp>>2]|0;
   $165 = ((($164)) + -16|0);
   $166 = ((($165)) + 4|0);
   $167 = HEAP32[$166>>2]|0;
   HEAP32[$yygotominor>>2] = $167;
   $168 = HEAP32[$yymsp>>2]|0;
   $169 = ((($168)) + 4|0);
   $170 = ((($169)) + 4|0);
   $171 = HEAP32[$170>>2]|0;
   $172 = HEAP32[$yymsp>>2]|0;
   $173 = ((($172)) + 4|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = HEAP32[$yymsp>>2]|0;
   $176 = ((($175)) + -16|0);
   $177 = ((($176)) + 4|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = $174;
   $180 = $178;
   $181 = (($179) - ($180))|0;
   $182 = (($171) + ($181))|0;
   $183 = ((($yygotominor)) + 4|0);
   HEAP32[$183>>2] = $182;
   break;
  }
  case 92: case 54:  {
   $184 = HEAP32[$pParse>>2]|0;
   $185 = ((($184)) + 328|0);
   $186 = HEAP32[$yymsp>>2]|0;
   $187 = ((($186)) + 4|0);
   ;HEAP32[$185>>2]=HEAP32[$187>>2]|0;HEAP32[$185+4>>2]=HEAP32[$187+4>>2]|0;
   break;
  }
  case 57: case 55:  {
   $188 = HEAP32[$pParse>>2]|0;
   $189 = HEAP32[$yymsp>>2]|0;
   $190 = ((($189)) + 4|0);
   _sqlite3AddDefaultValue($188,$190);
   break;
  }
  case 56:  {
   $191 = HEAP32[$pParse>>2]|0;
   $192 = HEAP32[$yymsp>>2]|0;
   $193 = ((($192)) + -16|0);
   $194 = ((($193)) + 4|0);
   _sqlite3AddDefaultValue($191,$194);
   break;
  }
  case 58:  {
   $195 = HEAP32[$pParse>>2]|0;
   $196 = HEAP32[$yymsp>>2]|0;
   $197 = ((($196)) + 4|0);
   $198 = HEAP32[$197>>2]|0;
   $199 = (_sqlite3PExpr($195,155,$198,0,0)|0);
   HEAP32[$v>>2] = $199;
   $200 = HEAP32[$yymsp>>2]|0;
   $201 = ((($200)) + -16|0);
   $202 = ((($201)) + 4|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = ((($v)) + 4|0);
   HEAP32[$204>>2] = $203;
   $205 = HEAP32[$yymsp>>2]|0;
   $206 = ((($205)) + 4|0);
   $207 = ((($206)) + 8|0);
   $208 = HEAP32[$207>>2]|0;
   $209 = ((($v)) + 8|0);
   HEAP32[$209>>2] = $208;
   $210 = HEAP32[$pParse>>2]|0;
   _sqlite3AddDefaultValue($210,$v);
   break;
  }
  case 59:  {
   $211 = HEAP32[$pParse>>2]|0;
   $212 = HEAP32[$yymsp>>2]|0;
   $213 = ((($212)) + 4|0);
   _spanExpr($v1,$211,97,$213);
   $214 = HEAP32[$pParse>>2]|0;
   _sqlite3AddDefaultValue($214,$v1);
   break;
  }
  case 61:  {
   $215 = HEAP32[$pParse>>2]|0;
   $216 = HEAP32[$yymsp>>2]|0;
   $217 = ((($216)) + 4|0);
   $218 = HEAP32[$217>>2]|0;
   _sqlite3AddNotNull($215,$218);
   break;
  }
  case 62:  {
   $219 = HEAP32[$pParse>>2]|0;
   $220 = HEAP32[$yymsp>>2]|0;
   $221 = ((($220)) + -16|0);
   $222 = ((($221)) + 4|0);
   $223 = HEAP32[$222>>2]|0;
   $224 = HEAP32[$yymsp>>2]|0;
   $225 = ((($224)) + 4|0);
   $226 = HEAP32[$225>>2]|0;
   $227 = HEAP32[$yymsp>>2]|0;
   $228 = ((($227)) + -32|0);
   $229 = ((($228)) + 4|0);
   $230 = HEAP32[$229>>2]|0;
   _sqlite3AddPrimaryKey($219,0,$223,$226,$230);
   break;
  }
  case 63:  {
   $231 = HEAP32[$pParse>>2]|0;
   $232 = HEAP32[$yymsp>>2]|0;
   $233 = ((($232)) + 4|0);
   $234 = HEAP32[$233>>2]|0;
   (_sqlite3CreateIndex($231,0,0,0,0,$234,0,0,0,0)|0);
   break;
  }
  case 64:  {
   $235 = HEAP32[$pParse>>2]|0;
   $236 = HEAP32[$yymsp>>2]|0;
   $237 = ((($236)) + -16|0);
   $238 = ((($237)) + 4|0);
   $239 = HEAP32[$238>>2]|0;
   _sqlite3AddCheckConstraint($235,$239);
   break;
  }
  case 65:  {
   $240 = HEAP32[$pParse>>2]|0;
   $241 = HEAP32[$yymsp>>2]|0;
   $242 = ((($241)) + -32|0);
   $243 = ((($242)) + 4|0);
   $244 = HEAP32[$yymsp>>2]|0;
   $245 = ((($244)) + -16|0);
   $246 = ((($245)) + 4|0);
   $247 = HEAP32[$246>>2]|0;
   $248 = HEAP32[$yymsp>>2]|0;
   $249 = ((($248)) + 4|0);
   $250 = HEAP32[$249>>2]|0;
   _sqlite3CreateForeignKey($240,0,$243,$247,$250);
   break;
  }
  case 66:  {
   $251 = HEAP32[$pParse>>2]|0;
   $252 = HEAP32[$yymsp>>2]|0;
   $253 = ((($252)) + 4|0);
   $254 = HEAP32[$253>>2]|0;
   _sqlite3DeferForeignKey($251,$254);
   break;
  }
  case 67:  {
   $255 = HEAP32[$pParse>>2]|0;
   $256 = HEAP32[$yymsp>>2]|0;
   $257 = ((($256)) + 4|0);
   _sqlite3AddCollateType($255,$257);
   break;
  }
  case 70:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 71:  {
   $258 = HEAP32[$yymsp>>2]|0;
   $259 = ((($258)) + -16|0);
   $260 = ((($259)) + 4|0);
   $261 = HEAP32[$260>>2]|0;
   $262 = HEAP32[$yymsp>>2]|0;
   $263 = ((($262)) + 4|0);
   $264 = ((($263)) + 4|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = $265 ^ -1;
   $267 = $261 & $266;
   $268 = HEAP32[$yymsp>>2]|0;
   $269 = ((($268)) + 4|0);
   $270 = HEAP32[$269>>2]|0;
   $271 = $267 | $270;
   HEAP32[$yygotominor>>2] = $271;
   break;
  }
  case 73: case 72:  {
   HEAP32[$yygotominor>>2] = 0;
   $272 = ((($yygotominor)) + 4|0);
   HEAP32[$272>>2] = 0;
   break;
  }
  case 74:  {
   $273 = HEAP32[$yymsp>>2]|0;
   $274 = ((($273)) + 4|0);
   $275 = HEAP32[$274>>2]|0;
   HEAP32[$yygotominor>>2] = $275;
   $276 = ((($yygotominor)) + 4|0);
   HEAP32[$276>>2] = 255;
   break;
  }
  case 75:  {
   $277 = HEAP32[$yymsp>>2]|0;
   $278 = ((($277)) + 4|0);
   $279 = HEAP32[$278>>2]|0;
   $280 = $279 << 8;
   HEAP32[$yygotominor>>2] = $280;
   $281 = ((($yygotominor)) + 4|0);
   HEAP32[$281>>2] = 65280;
   break;
  }
  case 76:  {
   HEAP32[$yygotominor>>2] = 7;
   break;
  }
  case 77:  {
   HEAP32[$yygotominor>>2] = 8;
   break;
  }
  case 78:  {
   HEAP32[$yygotominor>>2] = 9;
   break;
  }
  case 79:  {
   HEAP32[$yygotominor>>2] = 6;
   break;
  }
  case 80:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 178: case 103: case 102: case 100: case 98: case 82:  {
   $282 = HEAP32[$yymsp>>2]|0;
   $283 = ((($282)) + 4|0);
   $284 = HEAP32[$283>>2]|0;
   HEAP32[$yygotominor>>2] = $284;
   break;
  }
  case 86:  {
   $285 = ((($yygotominor)) + 4|0);
   HEAP32[$285>>2] = 0;
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 87:  {
   $286 = HEAP32[$yymsp>>2]|0;
   $287 = ((($286)) + -16|0);
   $288 = ((($287)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$288>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$288+4>>2]|0;
   break;
  }
  case 90:  {
   $289 = HEAP32[$pParse>>2]|0;
   $290 = ((($289)) + 328|0);
   $291 = ((($290)) + 4|0);
   HEAP32[$291>>2] = 0;
   break;
  }
  case 93:  {
   $292 = HEAP32[$pParse>>2]|0;
   $293 = HEAP32[$yymsp>>2]|0;
   $294 = ((($293)) + -48|0);
   $295 = ((($294)) + 4|0);
   $296 = HEAP32[$295>>2]|0;
   $297 = HEAP32[$yymsp>>2]|0;
   $298 = ((($297)) + 4|0);
   $299 = HEAP32[$298>>2]|0;
   $300 = HEAP32[$yymsp>>2]|0;
   $301 = ((($300)) + -32|0);
   $302 = ((($301)) + 4|0);
   $303 = HEAP32[$302>>2]|0;
   _sqlite3AddPrimaryKey($292,$296,$299,$303,0);
   break;
  }
  case 94:  {
   $304 = HEAP32[$pParse>>2]|0;
   $305 = HEAP32[$yymsp>>2]|0;
   $306 = ((($305)) + -32|0);
   $307 = ((($306)) + 4|0);
   $308 = HEAP32[$307>>2]|0;
   $309 = HEAP32[$yymsp>>2]|0;
   $310 = ((($309)) + 4|0);
   $311 = HEAP32[$310>>2]|0;
   (_sqlite3CreateIndex($304,0,0,0,$308,$311,0,0,0,0)|0);
   break;
  }
  case 95:  {
   $312 = HEAP32[$pParse>>2]|0;
   $313 = HEAP32[$yymsp>>2]|0;
   $314 = ((($313)) + -32|0);
   $315 = ((($314)) + 4|0);
   $316 = HEAP32[$315>>2]|0;
   _sqlite3AddCheckConstraint($312,$316);
   break;
  }
  case 96:  {
   $317 = HEAP32[$pParse>>2]|0;
   $318 = HEAP32[$yymsp>>2]|0;
   $319 = ((($318)) + -96|0);
   $320 = ((($319)) + 4|0);
   $321 = HEAP32[$320>>2]|0;
   $322 = HEAP32[$yymsp>>2]|0;
   $323 = ((($322)) + -48|0);
   $324 = ((($323)) + 4|0);
   $325 = HEAP32[$yymsp>>2]|0;
   $326 = ((($325)) + -32|0);
   $327 = ((($326)) + 4|0);
   $328 = HEAP32[$327>>2]|0;
   $329 = HEAP32[$yymsp>>2]|0;
   $330 = ((($329)) + -16|0);
   $331 = ((($330)) + 4|0);
   $332 = HEAP32[$331>>2]|0;
   _sqlite3CreateForeignKey($317,$321,$324,$328,$332);
   $333 = HEAP32[$pParse>>2]|0;
   $334 = HEAP32[$yymsp>>2]|0;
   $335 = ((($334)) + 4|0);
   $336 = HEAP32[$335>>2]|0;
   _sqlite3DeferForeignKey($333,$336);
   break;
  }
  case 101: case 99:  {
   HEAP32[$yygotominor>>2] = 10;
   break;
  }
  case 104:  {
   HEAP32[$yygotominor>>2] = 4;
   break;
  }
  case 179: case 105:  {
   HEAP32[$yygotominor>>2] = 5;
   break;
  }
  case 106:  {
   $337 = HEAP32[$pParse>>2]|0;
   $338 = HEAP32[$yymsp>>2]|0;
   $339 = ((($338)) + 4|0);
   $340 = HEAP32[$339>>2]|0;
   $341 = HEAP32[$yymsp>>2]|0;
   $342 = ((($341)) + -16|0);
   $343 = ((($342)) + 4|0);
   $344 = HEAP32[$343>>2]|0;
   _sqlite3DropTable($337,$340,0,$344);
   break;
  }
  case 109:  {
   $345 = HEAP32[$pParse>>2]|0;
   $346 = HEAP32[$yymsp>>2]|0;
   $347 = ((($346)) + -128|0);
   $348 = ((($347)) + 4|0);
   $349 = HEAP32[$yymsp>>2]|0;
   $350 = ((($349)) + -64|0);
   $351 = ((($350)) + 4|0);
   $352 = HEAP32[$yymsp>>2]|0;
   $353 = ((($352)) + -48|0);
   $354 = ((($353)) + 4|0);
   $355 = HEAP32[$yymsp>>2]|0;
   $356 = ((($355)) + -32|0);
   $357 = ((($356)) + 4|0);
   $358 = HEAP32[$357>>2]|0;
   $359 = HEAP32[$yymsp>>2]|0;
   $360 = ((($359)) + 4|0);
   $361 = HEAP32[$360>>2]|0;
   $362 = HEAP32[$yymsp>>2]|0;
   $363 = ((($362)) + -112|0);
   $364 = ((($363)) + 4|0);
   $365 = HEAP32[$364>>2]|0;
   $366 = HEAP32[$yymsp>>2]|0;
   $367 = ((($366)) + -80|0);
   $368 = ((($367)) + 4|0);
   $369 = HEAP32[$368>>2]|0;
   _sqlite3CreateView($345,$348,$351,$354,$358,$361,$365,$369);
   break;
  }
  case 110:  {
   $370 = HEAP32[$pParse>>2]|0;
   $371 = HEAP32[$yymsp>>2]|0;
   $372 = ((($371)) + 4|0);
   $373 = HEAP32[$372>>2]|0;
   $374 = HEAP32[$yymsp>>2]|0;
   $375 = ((($374)) + -16|0);
   $376 = ((($375)) + 4|0);
   $377 = HEAP32[$376>>2]|0;
   _sqlite3DropTable($370,$373,1,$377);
   break;
  }
  case 111:  {
   ;HEAP32[$dest>>2]=HEAP32[2508>>2]|0;HEAP32[$dest+4>>2]=HEAP32[2508+4>>2]|0;HEAP32[$dest+8>>2]=HEAP32[2508+8>>2]|0;HEAP32[$dest+12>>2]=HEAP32[2508+12>>2]|0;HEAP32[$dest+16>>2]=HEAP32[2508+16>>2]|0;
   $378 = HEAP32[$pParse>>2]|0;
   $379 = HEAP32[$yymsp>>2]|0;
   $380 = ((($379)) + 4|0);
   $381 = HEAP32[$380>>2]|0;
   (_sqlite3Select($378,$381,$dest)|0);
   $382 = HEAP32[$pParse>>2]|0;
   $383 = HEAP32[$382>>2]|0;
   $384 = HEAP32[$yymsp>>2]|0;
   $385 = ((($384)) + 4|0);
   $386 = HEAP32[$385>>2]|0;
   _sqlite3SelectDelete($383,$386);
   break;
  }
  case 112:  {
   $387 = HEAP32[$yymsp>>2]|0;
   $388 = ((($387)) + 4|0);
   $389 = HEAP32[$388>>2]|0;
   HEAP32[$p>>2] = $389;
   $390 = HEAP32[$p>>2]|0;
   $391 = ($390|0)!=(0|0);
   if ($391) {
    $392 = HEAP32[$yymsp>>2]|0;
    $393 = ((($392)) + -16|0);
    $394 = ((($393)) + 4|0);
    $395 = HEAP32[$394>>2]|0;
    $396 = HEAP32[$p>>2]|0;
    $397 = ((($396)) + 68|0);
    HEAP32[$397>>2] = $395;
    $398 = HEAP32[$pParse>>2]|0;
    $399 = HEAP32[$p>>2]|0;
    _parserDoubleLinkSelect($398,$399);
   } else {
    $400 = HEAP32[$pParse>>2]|0;
    $401 = HEAP32[$400>>2]|0;
    $402 = HEAP32[$yymsp>>2]|0;
    $403 = ((($402)) + -16|0);
    $404 = ((($403)) + 4|0);
    $405 = HEAP32[$404>>2]|0;
    _sqlite3WithDelete($401,$405);
   }
   $406 = HEAP32[$p>>2]|0;
   HEAP32[$yygotominor>>2] = $406;
   break;
  }
  case 119: case 113:  {
   $407 = HEAP32[$yymsp>>2]|0;
   $408 = ((($407)) + 4|0);
   $409 = HEAP32[$408>>2]|0;
   HEAP32[$yygotominor>>2] = $409;
   break;
  }
  case 114:  {
   $410 = HEAP32[$yymsp>>2]|0;
   $411 = ((($410)) + 4|0);
   $412 = HEAP32[$411>>2]|0;
   HEAP32[$pRhs>>2] = $412;
   $413 = HEAP32[$yymsp>>2]|0;
   $414 = ((($413)) + -32|0);
   $415 = ((($414)) + 4|0);
   $416 = HEAP32[$415>>2]|0;
   HEAP32[$pLhs>>2] = $416;
   $417 = HEAP32[$pRhs>>2]|0;
   $418 = ($417|0)!=(0|0);
   if ($418) {
    $419 = HEAP32[$pRhs>>2]|0;
    $420 = ((($419)) + 52|0);
    $421 = HEAP32[$420>>2]|0;
    $422 = ($421|0)!=(0|0);
    if ($422) {
     $423 = ((($x)) + 4|0);
     HEAP32[$423>>2] = 0;
     $424 = HEAP32[$pParse>>2]|0;
     $425 = HEAP32[$pRhs>>2]|0;
     _parserDoubleLinkSelect($424,$425);
     $426 = HEAP32[$pParse>>2]|0;
     $427 = HEAP32[$pRhs>>2]|0;
     $428 = (_sqlite3SrcListAppendFromTerm($426,0,0,0,$x,$427,0,0)|0);
     HEAP32[$pFrom>>2] = $428;
     $429 = HEAP32[$pParse>>2]|0;
     $430 = HEAP32[$pFrom>>2]|0;
     $431 = (_sqlite3SelectNew($429,0,$430,0,0,0,0,0,0,0)|0);
     HEAP32[$pRhs>>2] = $431;
    }
   }
   $432 = HEAP32[$pRhs>>2]|0;
   $433 = ($432|0)!=(0|0);
   if ($433) {
    $434 = HEAP32[$yymsp>>2]|0;
    $435 = ((($434)) + -16|0);
    $436 = ((($435)) + 4|0);
    $437 = HEAP32[$436>>2]|0;
    $438 = $437&255;
    $439 = HEAP32[$pRhs>>2]|0;
    $440 = ((($439)) + 4|0);
    HEAP8[$440>>0] = $438;
    $441 = HEAP32[$pLhs>>2]|0;
    $442 = HEAP32[$pRhs>>2]|0;
    $443 = ((($442)) + 52|0);
    HEAP32[$443>>2] = $441;
    $444 = HEAP32[$pLhs>>2]|0;
    $445 = ($444|0)!=(0|0);
    if ($445) {
     $446 = HEAP32[$pLhs>>2]|0;
     $447 = ((($446)) + 6|0);
     $448 = HEAP16[$447>>1]|0;
     $449 = $448&65535;
     $450 = $449 & -513;
     $451 = $450&65535;
     HEAP16[$447>>1] = $451;
    }
    $452 = HEAP32[$pRhs>>2]|0;
    $453 = ((($452)) + 6|0);
    $454 = HEAP16[$453>>1]|0;
    $455 = $454&65535;
    $456 = $455 & -513;
    $457 = $456&65535;
    HEAP16[$453>>1] = $457;
    $458 = HEAP32[$yymsp>>2]|0;
    $459 = ((($458)) + -16|0);
    $460 = ((($459)) + 4|0);
    $461 = HEAP32[$460>>2]|0;
    $462 = ($461|0)!=(116);
    if ($462) {
     $463 = HEAP32[$pParse>>2]|0;
     $464 = ((($463)) + 22|0);
     HEAP8[$464>>0] = 1;
    }
   } else {
    $465 = HEAP32[$pParse>>2]|0;
    $466 = HEAP32[$465>>2]|0;
    $467 = HEAP32[$pLhs>>2]|0;
    _sqlite3SelectDelete($466,$467);
   }
   $468 = HEAP32[$pRhs>>2]|0;
   HEAP32[$yygotominor>>2] = $468;
   break;
  }
  case 116:  {
   HEAP32[$yygotominor>>2] = 116;
   break;
  }
  case 118:  {
   $469 = HEAP32[$pParse>>2]|0;
   $470 = HEAP32[$yymsp>>2]|0;
   $471 = ((($470)) + -96|0);
   $472 = ((($471)) + 4|0);
   $473 = HEAP32[$472>>2]|0;
   $474 = HEAP32[$yymsp>>2]|0;
   $475 = ((($474)) + -80|0);
   $476 = ((($475)) + 4|0);
   $477 = HEAP32[$476>>2]|0;
   $478 = HEAP32[$yymsp>>2]|0;
   $479 = ((($478)) + -64|0);
   $480 = ((($479)) + 4|0);
   $481 = HEAP32[$480>>2]|0;
   $482 = HEAP32[$yymsp>>2]|0;
   $483 = ((($482)) + -48|0);
   $484 = ((($483)) + 4|0);
   $485 = HEAP32[$484>>2]|0;
   $486 = HEAP32[$yymsp>>2]|0;
   $487 = ((($486)) + -32|0);
   $488 = ((($487)) + 4|0);
   $489 = HEAP32[$488>>2]|0;
   $490 = HEAP32[$yymsp>>2]|0;
   $491 = ((($490)) + -16|0);
   $492 = ((($491)) + 4|0);
   $493 = HEAP32[$492>>2]|0;
   $494 = HEAP32[$yymsp>>2]|0;
   $495 = ((($494)) + -112|0);
   $496 = ((($495)) + 4|0);
   $497 = HEAP32[$496>>2]|0;
   $498 = $497&65535;
   $499 = HEAP32[$yymsp>>2]|0;
   $500 = ((($499)) + 4|0);
   $501 = HEAP32[$500>>2]|0;
   $502 = HEAP32[$yymsp>>2]|0;
   $503 = ((($502)) + 4|0);
   $504 = ((($503)) + 4|0);
   $505 = HEAP32[$504>>2]|0;
   $506 = (_sqlite3SelectNew($469,$473,$477,$481,$485,$489,$493,$498,$501,$505)|0);
   HEAP32[$yygotominor>>2] = $506;
   break;
  }
  case 120:  {
   $507 = HEAP32[$pParse>>2]|0;
   $508 = HEAP32[$yymsp>>2]|0;
   $509 = ((($508)) + -16|0);
   $510 = ((($509)) + 4|0);
   $511 = HEAP32[$510>>2]|0;
   $512 = (_sqlite3SelectNew($507,$511,0,0,0,0,0,256,0,0)|0);
   HEAP32[$yygotominor>>2] = $512;
   break;
  }
  case 121:  {
   $513 = HEAP32[$yymsp>>2]|0;
   $514 = ((($513)) + -64|0);
   $515 = ((($514)) + 4|0);
   $516 = HEAP32[$515>>2]|0;
   HEAP32[$pLeft>>2] = $516;
   $517 = HEAP32[$pParse>>2]|0;
   $518 = HEAP32[$yymsp>>2]|0;
   $519 = ((($518)) + -16|0);
   $520 = ((($519)) + 4|0);
   $521 = HEAP32[$520>>2]|0;
   $522 = (_sqlite3SelectNew($517,$521,0,0,0,0,0,768,0,0)|0);
   HEAP32[$pRight>>2] = $522;
   $523 = HEAP32[$pLeft>>2]|0;
   $524 = ($523|0)!=(0|0);
   if ($524) {
    $525 = HEAP32[$pLeft>>2]|0;
    $526 = ((($525)) + 6|0);
    $527 = HEAP16[$526>>1]|0;
    $528 = $527&65535;
    $529 = $528 & -513;
    $530 = $529&65535;
    HEAP16[$526>>1] = $530;
   }
   $531 = HEAP32[$pRight>>2]|0;
   $532 = ($531|0)!=(0|0);
   if ($532) {
    $533 = HEAP32[$pRight>>2]|0;
    $534 = ((($533)) + 4|0);
    HEAP8[$534>>0] = 116;
    $535 = HEAP32[$yymsp>>2]|0;
    $536 = ((($535)) + -64|0);
    $537 = ((($536)) + 4|0);
    $538 = HEAP32[$537>>2]|0;
    HEAP32[$pLeft>>2] = $538;
    $539 = HEAP32[$pLeft>>2]|0;
    $540 = HEAP32[$pRight>>2]|0;
    $541 = ((($540)) + 52|0);
    HEAP32[$541>>2] = $539;
    $542 = HEAP32[$pRight>>2]|0;
    HEAP32[$yygotominor>>2] = $542;
    break L1;
   } else {
    $543 = HEAP32[$pLeft>>2]|0;
    HEAP32[$yygotominor>>2] = $543;
    break L1;
   }
   break;
  }
  case 122:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 123:  {
   HEAP32[$yygotominor>>2] = 2;
   break;
  }
  case 244: case 125:  {
   $544 = HEAP32[$yymsp>>2]|0;
   $545 = ((($544)) + -16|0);
   $546 = ((($545)) + 4|0);
   $547 = HEAP32[$546>>2]|0;
   HEAP32[$yygotominor>>2] = $547;
   break;
  }
  case 243: case 237: case 162: case 155: case 126:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 127:  {
   $548 = HEAP32[$pParse>>2]|0;
   $549 = HEAP32[$yymsp>>2]|0;
   $550 = ((($549)) + -32|0);
   $551 = ((($550)) + 4|0);
   $552 = HEAP32[$551>>2]|0;
   $553 = HEAP32[$yymsp>>2]|0;
   $554 = ((($553)) + -16|0);
   $555 = ((($554)) + 4|0);
   $556 = HEAP32[$555>>2]|0;
   $557 = (_sqlite3ExprListAppend($548,$552,$556)|0);
   HEAP32[$yygotominor>>2] = $557;
   $558 = HEAP32[$yymsp>>2]|0;
   $559 = ((($558)) + 4|0);
   $560 = ((($559)) + 4|0);
   $561 = HEAP32[$560>>2]|0;
   $562 = ($561>>>0)>(0);
   if ($562) {
    $563 = HEAP32[$pParse>>2]|0;
    $564 = HEAP32[$yygotominor>>2]|0;
    $565 = HEAP32[$yymsp>>2]|0;
    $566 = ((($565)) + 4|0);
    _sqlite3ExprListSetName($563,$564,$566,1);
   }
   $567 = HEAP32[$pParse>>2]|0;
   $568 = HEAP32[$yygotominor>>2]|0;
   $569 = HEAP32[$yymsp>>2]|0;
   $570 = ((($569)) + -16|0);
   $571 = ((($570)) + 4|0);
   _sqlite3ExprListSetSpan($567,$568,$571);
   break;
  }
  case 128:  {
   $572 = HEAP32[$pParse>>2]|0;
   $573 = HEAP32[$572>>2]|0;
   $574 = (_sqlite3Expr($573,158,0)|0);
   HEAP32[$p2>>2] = $574;
   $575 = HEAP32[$pParse>>2]|0;
   $576 = HEAP32[$yymsp>>2]|0;
   $577 = ((($576)) + -16|0);
   $578 = ((($577)) + 4|0);
   $579 = HEAP32[$578>>2]|0;
   $580 = HEAP32[$p2>>2]|0;
   $581 = (_sqlite3ExprListAppend($575,$579,$580)|0);
   HEAP32[$yygotominor>>2] = $581;
   break;
  }
  case 129:  {
   $582 = HEAP32[$pParse>>2]|0;
   $583 = HEAP32[$yymsp>>2]|0;
   $584 = ((($583)) + 4|0);
   $585 = (_sqlite3PExpr($582,158,0,0,$584)|0);
   HEAP32[$pRight3>>2] = $585;
   $586 = HEAP32[$pParse>>2]|0;
   $587 = HEAP32[$yymsp>>2]|0;
   $588 = ((($587)) + -32|0);
   $589 = ((($588)) + 4|0);
   $590 = (_sqlite3PExpr($586,27,0,0,$589)|0);
   HEAP32[$pLeft4>>2] = $590;
   $591 = HEAP32[$pParse>>2]|0;
   $592 = HEAP32[$pLeft4>>2]|0;
   $593 = HEAP32[$pRight3>>2]|0;
   $594 = (_sqlite3PExpr($591,122,$592,$593,0)|0);
   HEAP32[$pDot>>2] = $594;
   $595 = HEAP32[$pParse>>2]|0;
   $596 = HEAP32[$yymsp>>2]|0;
   $597 = ((($596)) + -48|0);
   $598 = ((($597)) + 4|0);
   $599 = HEAP32[$598>>2]|0;
   $600 = HEAP32[$pDot>>2]|0;
   $601 = (_sqlite3ExprListAppend($595,$599,$600)|0);
   HEAP32[$yygotominor>>2] = $601;
   break;
  }
  case 132:  {
   $602 = ((($yygotominor)) + 4|0);
   HEAP32[$602>>2] = 0;
   break;
  }
  case 133:  {
   $603 = HEAP32[$pParse>>2]|0;
   $604 = HEAP32[$603>>2]|0;
   $605 = (_sqlite3DbMallocZero($604,80,0)|0);
   HEAP32[$yygotominor>>2] = $605;
   break;
  }
  case 134:  {
   $606 = HEAP32[$yymsp>>2]|0;
   $607 = ((($606)) + 4|0);
   $608 = HEAP32[$607>>2]|0;
   HEAP32[$yygotominor>>2] = $608;
   $609 = HEAP32[$yygotominor>>2]|0;
   _sqlite3SrcListShiftJoinType($609);
   break;
  }
  case 135:  {
   $610 = HEAP32[$yymsp>>2]|0;
   $611 = ((($610)) + -16|0);
   $612 = ((($611)) + 4|0);
   $613 = HEAP32[$612>>2]|0;
   HEAP32[$yygotominor>>2] = $613;
   $614 = HEAP32[$yygotominor>>2]|0;
   $615 = ($614|0)!=(0|0);
   if ($615) {
    $616 = HEAP32[$yygotominor>>2]|0;
    $617 = HEAP32[$616>>2]|0;
    $618 = ($617|0)>(0);
    if ($618) {
     $619 = HEAP32[$yymsp>>2]|0;
     $620 = ((($619)) + 4|0);
     $621 = HEAP32[$620>>2]|0;
     $622 = $621&255;
     $623 = HEAP32[$yygotominor>>2]|0;
     $624 = HEAP32[$623>>2]|0;
     $625 = (($624) - 1)|0;
     $626 = HEAP32[$yygotominor>>2]|0;
     $627 = ((($626)) + 8|0);
     $628 = (($627) + (($625*72)|0)|0);
     $629 = ((($628)) + 36|0);
     HEAP8[$629>>0] = $622;
    }
   }
   break;
  }
  case 136:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 137:  {
   $630 = HEAP32[$pParse>>2]|0;
   $631 = HEAP32[$yymsp>>2]|0;
   $632 = ((($631)) + -96|0);
   $633 = ((($632)) + 4|0);
   $634 = HEAP32[$633>>2]|0;
   $635 = HEAP32[$yymsp>>2]|0;
   $636 = ((($635)) + -80|0);
   $637 = ((($636)) + 4|0);
   $638 = HEAP32[$yymsp>>2]|0;
   $639 = ((($638)) + -64|0);
   $640 = ((($639)) + 4|0);
   $641 = HEAP32[$yymsp>>2]|0;
   $642 = ((($641)) + -48|0);
   $643 = ((($642)) + 4|0);
   $644 = HEAP32[$yymsp>>2]|0;
   $645 = ((($644)) + -16|0);
   $646 = ((($645)) + 4|0);
   $647 = HEAP32[$646>>2]|0;
   $648 = HEAP32[$yymsp>>2]|0;
   $649 = ((($648)) + 4|0);
   $650 = HEAP32[$649>>2]|0;
   $651 = (_sqlite3SrcListAppendFromTerm($630,$634,$637,$640,$643,0,$647,$650)|0);
   HEAP32[$yygotominor>>2] = $651;
   $652 = HEAP32[$pParse>>2]|0;
   $653 = HEAP32[$yygotominor>>2]|0;
   $654 = HEAP32[$yymsp>>2]|0;
   $655 = ((($654)) + -32|0);
   $656 = ((($655)) + 4|0);
   _sqlite3SrcListIndexedBy($652,$653,$656);
   break;
  }
  case 138:  {
   $657 = HEAP32[$pParse>>2]|0;
   $658 = HEAP32[$yymsp>>2]|0;
   $659 = ((($658)) + -128|0);
   $660 = ((($659)) + 4|0);
   $661 = HEAP32[$660>>2]|0;
   $662 = HEAP32[$yymsp>>2]|0;
   $663 = ((($662)) + -112|0);
   $664 = ((($663)) + 4|0);
   $665 = HEAP32[$yymsp>>2]|0;
   $666 = ((($665)) + -96|0);
   $667 = ((($666)) + 4|0);
   $668 = HEAP32[$yymsp>>2]|0;
   $669 = ((($668)) + -32|0);
   $670 = ((($669)) + 4|0);
   $671 = HEAP32[$yymsp>>2]|0;
   $672 = ((($671)) + -16|0);
   $673 = ((($672)) + 4|0);
   $674 = HEAP32[$673>>2]|0;
   $675 = HEAP32[$yymsp>>2]|0;
   $676 = ((($675)) + 4|0);
   $677 = HEAP32[$676>>2]|0;
   $678 = (_sqlite3SrcListAppendFromTerm($657,$661,$664,$667,$670,0,$674,$677)|0);
   HEAP32[$yygotominor>>2] = $678;
   $679 = HEAP32[$pParse>>2]|0;
   $680 = HEAP32[$yygotominor>>2]|0;
   $681 = HEAP32[$yymsp>>2]|0;
   $682 = ((($681)) + -64|0);
   $683 = ((($682)) + 4|0);
   $684 = HEAP32[$683>>2]|0;
   _sqlite3SrcListFuncArgs($679,$680,$684);
   break;
  }
  case 139:  {
   $685 = HEAP32[$pParse>>2]|0;
   $686 = HEAP32[$yymsp>>2]|0;
   $687 = ((($686)) + -96|0);
   $688 = ((($687)) + 4|0);
   $689 = HEAP32[$688>>2]|0;
   $690 = HEAP32[$yymsp>>2]|0;
   $691 = ((($690)) + -32|0);
   $692 = ((($691)) + 4|0);
   $693 = HEAP32[$yymsp>>2]|0;
   $694 = ((($693)) + -64|0);
   $695 = ((($694)) + 4|0);
   $696 = HEAP32[$695>>2]|0;
   $697 = HEAP32[$yymsp>>2]|0;
   $698 = ((($697)) + -16|0);
   $699 = ((($698)) + 4|0);
   $700 = HEAP32[$699>>2]|0;
   $701 = HEAP32[$yymsp>>2]|0;
   $702 = ((($701)) + 4|0);
   $703 = HEAP32[$702>>2]|0;
   $704 = (_sqlite3SrcListAppendFromTerm($685,$689,0,0,$692,$696,$700,$703)|0);
   HEAP32[$yygotominor>>2] = $704;
   break;
  }
  case 140:  {
   $705 = HEAP32[$yymsp>>2]|0;
   $706 = ((($705)) + -96|0);
   $707 = ((($706)) + 4|0);
   $708 = HEAP32[$707>>2]|0;
   $709 = ($708|0)==(0|0);
   if ($709) {
    $710 = HEAP32[$yymsp>>2]|0;
    $711 = ((($710)) + -32|0);
    $712 = ((($711)) + 4|0);
    $713 = ((($712)) + 4|0);
    $714 = HEAP32[$713>>2]|0;
    $715 = ($714|0)==(0);
    if ($715) {
     $716 = HEAP32[$yymsp>>2]|0;
     $717 = ((($716)) + -16|0);
     $718 = ((($717)) + 4|0);
     $719 = HEAP32[$718>>2]|0;
     $720 = ($719|0)==(0|0);
     if ($720) {
      $721 = HEAP32[$yymsp>>2]|0;
      $722 = ((($721)) + 4|0);
      $723 = HEAP32[$722>>2]|0;
      $724 = ($723|0)==(0|0);
      if ($724) {
       $725 = HEAP32[$yymsp>>2]|0;
       $726 = ((($725)) + -64|0);
       $727 = ((($726)) + 4|0);
       $728 = HEAP32[$727>>2]|0;
       HEAP32[$yygotominor>>2] = $728;
       break L1;
      }
     }
    }
   }
   $729 = HEAP32[$yymsp>>2]|0;
   $730 = ((($729)) + -64|0);
   $731 = ((($730)) + 4|0);
   $732 = HEAP32[$731>>2]|0;
   $733 = HEAP32[$732>>2]|0;
   $734 = ($733|0)==(1);
   if (!($734)) {
    $791 = HEAP32[$yymsp>>2]|0;
    $792 = ((($791)) + -64|0);
    $793 = ((($792)) + 4|0);
    $794 = HEAP32[$793>>2]|0;
    _sqlite3SrcListShiftJoinType($794);
    $795 = HEAP32[$pParse>>2]|0;
    $796 = HEAP32[$yymsp>>2]|0;
    $797 = ((($796)) + -64|0);
    $798 = ((($797)) + 4|0);
    $799 = HEAP32[$798>>2]|0;
    $800 = (_sqlite3SelectNew($795,0,$799,0,0,0,0,1024,0,0)|0);
    HEAP32[$pSubquery>>2] = $800;
    $801 = HEAP32[$pParse>>2]|0;
    $802 = HEAP32[$yymsp>>2]|0;
    $803 = ((($802)) + -96|0);
    $804 = ((($803)) + 4|0);
    $805 = HEAP32[$804>>2]|0;
    $806 = HEAP32[$yymsp>>2]|0;
    $807 = ((($806)) + -32|0);
    $808 = ((($807)) + 4|0);
    $809 = HEAP32[$pSubquery>>2]|0;
    $810 = HEAP32[$yymsp>>2]|0;
    $811 = ((($810)) + -16|0);
    $812 = ((($811)) + 4|0);
    $813 = HEAP32[$812>>2]|0;
    $814 = HEAP32[$yymsp>>2]|0;
    $815 = ((($814)) + 4|0);
    $816 = HEAP32[$815>>2]|0;
    $817 = (_sqlite3SrcListAppendFromTerm($801,$805,0,0,$808,$809,$813,$816)|0);
    HEAP32[$yygotominor>>2] = $817;
    break L1;
   }
   $735 = HEAP32[$pParse>>2]|0;
   $736 = HEAP32[$yymsp>>2]|0;
   $737 = ((($736)) + -96|0);
   $738 = ((($737)) + 4|0);
   $739 = HEAP32[$738>>2]|0;
   $740 = HEAP32[$yymsp>>2]|0;
   $741 = ((($740)) + -32|0);
   $742 = ((($741)) + 4|0);
   $743 = HEAP32[$yymsp>>2]|0;
   $744 = ((($743)) + -16|0);
   $745 = ((($744)) + 4|0);
   $746 = HEAP32[$745>>2]|0;
   $747 = HEAP32[$yymsp>>2]|0;
   $748 = ((($747)) + 4|0);
   $749 = HEAP32[$748>>2]|0;
   $750 = (_sqlite3SrcListAppendFromTerm($735,$739,0,0,$742,0,$746,$749)|0);
   HEAP32[$yygotominor>>2] = $750;
   $751 = HEAP32[$yygotominor>>2]|0;
   $752 = ($751|0)!=(0|0);
   if ($752) {
    $753 = HEAP32[$yygotominor>>2]|0;
    $754 = HEAP32[$753>>2]|0;
    $755 = (($754) - 1)|0;
    $756 = HEAP32[$yygotominor>>2]|0;
    $757 = ((($756)) + 8|0);
    $758 = (($757) + (($755*72)|0)|0);
    HEAP32[$pNew>>2] = $758;
    $759 = HEAP32[$yymsp>>2]|0;
    $760 = ((($759)) + -64|0);
    $761 = ((($760)) + 4|0);
    $762 = HEAP32[$761>>2]|0;
    $763 = ((($762)) + 8|0);
    HEAP32[$pOld>>2] = $763;
    $764 = HEAP32[$pOld>>2]|0;
    $765 = ((($764)) + 8|0);
    $766 = HEAP32[$765>>2]|0;
    $767 = HEAP32[$pNew>>2]|0;
    $768 = ((($767)) + 8|0);
    HEAP32[$768>>2] = $766;
    $769 = HEAP32[$pOld>>2]|0;
    $770 = ((($769)) + 4|0);
    $771 = HEAP32[$770>>2]|0;
    $772 = HEAP32[$pNew>>2]|0;
    $773 = ((($772)) + 4|0);
    HEAP32[$773>>2] = $771;
    $774 = HEAP32[$pOld>>2]|0;
    $775 = ((($774)) + 20|0);
    $776 = HEAP32[$775>>2]|0;
    $777 = HEAP32[$pNew>>2]|0;
    $778 = ((($777)) + 20|0);
    HEAP32[$778>>2] = $776;
    $779 = HEAP32[$pOld>>2]|0;
    $780 = ((($779)) + 4|0);
    HEAP32[$780>>2] = 0;
    $781 = HEAP32[$pOld>>2]|0;
    $782 = ((($781)) + 8|0);
    HEAP32[$782>>2] = 0;
    $783 = HEAP32[$pOld>>2]|0;
    $784 = ((($783)) + 20|0);
    HEAP32[$784>>2] = 0;
   }
   $785 = HEAP32[$pParse>>2]|0;
   $786 = HEAP32[$785>>2]|0;
   $787 = HEAP32[$yymsp>>2]|0;
   $788 = ((($787)) + -64|0);
   $789 = ((($788)) + 4|0);
   $790 = HEAP32[$789>>2]|0;
   _sqlite3SrcListDelete($786,$790);
   break;
  }
  case 150: case 141:  {
   HEAP32[$yygotominor>>2] = 0;
   $818 = ((($yygotominor)) + 4|0);
   HEAP32[$818>>2] = 0;
   break;
  }
  case 143:  {
   $819 = HEAP32[$pParse>>2]|0;
   $820 = HEAP32[$819>>2]|0;
   $821 = HEAP32[$yymsp>>2]|0;
   $822 = ((($821)) + -16|0);
   $823 = ((($822)) + 4|0);
   $824 = HEAP32[$yymsp>>2]|0;
   $825 = ((($824)) + 4|0);
   $826 = (_sqlite3SrcListAppend($820,0,$823,$825)|0);
   HEAP32[$yygotominor>>2] = $826;
   break;
  }
  case 144:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 145:  {
   $827 = HEAP32[$pParse>>2]|0;
   $828 = HEAP32[$yymsp>>2]|0;
   $829 = ((($828)) + -16|0);
   $830 = ((($829)) + 4|0);
   $831 = (_sqlite3JoinType($827,$830,0,0)|0);
   HEAP32[$yygotominor>>2] = $831;
   break;
  }
  case 146:  {
   $832 = HEAP32[$pParse>>2]|0;
   $833 = HEAP32[$yymsp>>2]|0;
   $834 = ((($833)) + -32|0);
   $835 = ((($834)) + 4|0);
   $836 = HEAP32[$yymsp>>2]|0;
   $837 = ((($836)) + -16|0);
   $838 = ((($837)) + 4|0);
   $839 = (_sqlite3JoinType($832,$835,$838,0)|0);
   HEAP32[$yygotominor>>2] = $839;
   break;
  }
  case 147:  {
   $840 = HEAP32[$pParse>>2]|0;
   $841 = HEAP32[$yymsp>>2]|0;
   $842 = ((($841)) + -48|0);
   $843 = ((($842)) + 4|0);
   $844 = HEAP32[$yymsp>>2]|0;
   $845 = ((($844)) + -32|0);
   $846 = ((($845)) + 4|0);
   $847 = HEAP32[$yymsp>>2]|0;
   $848 = ((($847)) + -16|0);
   $849 = ((($848)) + 4|0);
   $850 = (_sqlite3JoinType($840,$843,$846,$849)|0);
   HEAP32[$yygotominor>>2] = $850;
   break;
  }
  case 234: case 232: case 172: case 165: case 148:  {
   $851 = HEAP32[$yymsp>>2]|0;
   $852 = ((($851)) + 4|0);
   $853 = HEAP32[$852>>2]|0;
   HEAP32[$yygotominor>>2] = $853;
   break;
  }
  case 235: case 233: case 171: case 164: case 149:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 152:  {
   HEAP32[$yygotominor>>2] = 0;
   $854 = ((($yygotominor)) + 4|0);
   HEAP32[$854>>2] = 1;
   break;
  }
  case 181: case 153:  {
   $855 = HEAP32[$yymsp>>2]|0;
   $856 = ((($855)) + -16|0);
   $857 = ((($856)) + 4|0);
   $858 = HEAP32[$857>>2]|0;
   HEAP32[$yygotominor>>2] = $858;
   break;
  }
  case 180: case 154:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 236: case 163: case 156:  {
   $859 = HEAP32[$yymsp>>2]|0;
   $860 = ((($859)) + 4|0);
   $861 = HEAP32[$860>>2]|0;
   HEAP32[$yygotominor>>2] = $861;
   break;
  }
  case 157:  {
   $862 = HEAP32[$pParse>>2]|0;
   $863 = HEAP32[$yymsp>>2]|0;
   $864 = ((($863)) + -48|0);
   $865 = ((($864)) + 4|0);
   $866 = HEAP32[$865>>2]|0;
   $867 = HEAP32[$yymsp>>2]|0;
   $868 = ((($867)) + -16|0);
   $869 = ((($868)) + 4|0);
   $870 = HEAP32[$869>>2]|0;
   $871 = (_sqlite3ExprListAppend($862,$866,$870)|0);
   HEAP32[$yygotominor>>2] = $871;
   $872 = HEAP32[$yygotominor>>2]|0;
   $873 = HEAP32[$yymsp>>2]|0;
   $874 = ((($873)) + 4|0);
   $875 = HEAP32[$874>>2]|0;
   _sqlite3ExprListSetSortOrder($872,$875);
   break;
  }
  case 158:  {
   $876 = HEAP32[$pParse>>2]|0;
   $877 = HEAP32[$yymsp>>2]|0;
   $878 = ((($877)) + -16|0);
   $879 = ((($878)) + 4|0);
   $880 = HEAP32[$879>>2]|0;
   $881 = (_sqlite3ExprListAppend($876,0,$880)|0);
   HEAP32[$yygotominor>>2] = $881;
   $882 = HEAP32[$yygotominor>>2]|0;
   $883 = HEAP32[$yymsp>>2]|0;
   $884 = ((($883)) + 4|0);
   $885 = HEAP32[$884>>2]|0;
   _sqlite3ExprListSetSortOrder($882,$885);
   break;
  }
  case 159:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 160:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 161:  {
   HEAP32[$yygotominor>>2] = -1;
   break;
  }
  case 166:  {
   HEAP32[$yygotominor>>2] = 0;
   $886 = ((($yygotominor)) + 4|0);
   HEAP32[$886>>2] = 0;
   break;
  }
  case 167:  {
   $887 = HEAP32[$yymsp>>2]|0;
   $888 = ((($887)) + 4|0);
   $889 = HEAP32[$888>>2]|0;
   HEAP32[$yygotominor>>2] = $889;
   $890 = ((($yygotominor)) + 4|0);
   HEAP32[$890>>2] = 0;
   break;
  }
  case 168:  {
   $891 = HEAP32[$yymsp>>2]|0;
   $892 = ((($891)) + -32|0);
   $893 = ((($892)) + 4|0);
   $894 = HEAP32[$893>>2]|0;
   HEAP32[$yygotominor>>2] = $894;
   $895 = HEAP32[$yymsp>>2]|0;
   $896 = ((($895)) + 4|0);
   $897 = HEAP32[$896>>2]|0;
   $898 = ((($yygotominor)) + 4|0);
   HEAP32[$898>>2] = $897;
   break;
  }
  case 169:  {
   $899 = HEAP32[$yymsp>>2]|0;
   $900 = ((($899)) + -32|0);
   $901 = ((($900)) + 4|0);
   $902 = HEAP32[$901>>2]|0;
   $903 = ((($yygotominor)) + 4|0);
   HEAP32[$903>>2] = $902;
   $904 = HEAP32[$yymsp>>2]|0;
   $905 = ((($904)) + 4|0);
   $906 = HEAP32[$905>>2]|0;
   HEAP32[$yygotominor>>2] = $906;
   break;
  }
  case 170:  {
   $907 = HEAP32[$pParse>>2]|0;
   $908 = HEAP32[$yymsp>>2]|0;
   $909 = ((($908)) + -80|0);
   $910 = ((($909)) + 4|0);
   $911 = HEAP32[$910>>2]|0;
   _sqlite3WithPush($907,$911,1);
   $912 = HEAP32[$pParse>>2]|0;
   $913 = HEAP32[$yymsp>>2]|0;
   $914 = ((($913)) + -32|0);
   $915 = ((($914)) + 4|0);
   $916 = HEAP32[$915>>2]|0;
   $917 = HEAP32[$yymsp>>2]|0;
   $918 = ((($917)) + -16|0);
   $919 = ((($918)) + 4|0);
   _sqlite3SrcListIndexedBy($912,$916,$919);
   $920 = HEAP32[$pParse>>2]|0;
   $921 = HEAP32[$yymsp>>2]|0;
   $922 = ((($921)) + -32|0);
   $923 = ((($922)) + 4|0);
   $924 = HEAP32[$923>>2]|0;
   $925 = HEAP32[$yymsp>>2]|0;
   $926 = ((($925)) + 4|0);
   $927 = HEAP32[$926>>2]|0;
   _sqlite3DeleteFrom($920,$924,$927);
   break;
  }
  case 173:  {
   $928 = HEAP32[$pParse>>2]|0;
   $929 = HEAP32[$yymsp>>2]|0;
   $930 = ((($929)) + -112|0);
   $931 = ((($930)) + 4|0);
   $932 = HEAP32[$931>>2]|0;
   _sqlite3WithPush($928,$932,1);
   $933 = HEAP32[$pParse>>2]|0;
   $934 = HEAP32[$yymsp>>2]|0;
   $935 = ((($934)) + -64|0);
   $936 = ((($935)) + 4|0);
   $937 = HEAP32[$936>>2]|0;
   $938 = HEAP32[$yymsp>>2]|0;
   $939 = ((($938)) + -48|0);
   $940 = ((($939)) + 4|0);
   _sqlite3SrcListIndexedBy($933,$937,$940);
   $941 = HEAP32[$pParse>>2]|0;
   $942 = HEAP32[$yymsp>>2]|0;
   $943 = ((($942)) + -16|0);
   $944 = ((($943)) + 4|0);
   $945 = HEAP32[$944>>2]|0;
   _sqlite3ExprListCheckLength($941,$945,27370);
   $946 = HEAP32[$pParse>>2]|0;
   $947 = HEAP32[$yymsp>>2]|0;
   $948 = ((($947)) + -64|0);
   $949 = ((($948)) + 4|0);
   $950 = HEAP32[$949>>2]|0;
   $951 = HEAP32[$yymsp>>2]|0;
   $952 = ((($951)) + -16|0);
   $953 = ((($952)) + 4|0);
   $954 = HEAP32[$953>>2]|0;
   $955 = HEAP32[$yymsp>>2]|0;
   $956 = ((($955)) + 4|0);
   $957 = HEAP32[$956>>2]|0;
   $958 = HEAP32[$yymsp>>2]|0;
   $959 = ((($958)) + -80|0);
   $960 = ((($959)) + 4|0);
   $961 = HEAP32[$960>>2]|0;
   _sqlite3Update($946,$950,$954,$957,$961);
   break;
  }
  case 174:  {
   $962 = HEAP32[$pParse>>2]|0;
   $963 = HEAP32[$yymsp>>2]|0;
   $964 = ((($963)) + -64|0);
   $965 = ((($964)) + 4|0);
   $966 = HEAP32[$965>>2]|0;
   $967 = HEAP32[$yymsp>>2]|0;
   $968 = ((($967)) + 4|0);
   $969 = HEAP32[$968>>2]|0;
   $970 = (_sqlite3ExprListAppend($962,$966,$969)|0);
   HEAP32[$yygotominor>>2] = $970;
   $971 = HEAP32[$pParse>>2]|0;
   $972 = HEAP32[$yygotominor>>2]|0;
   $973 = HEAP32[$yymsp>>2]|0;
   $974 = ((($973)) + -32|0);
   $975 = ((($974)) + 4|0);
   _sqlite3ExprListSetName($971,$972,$975,1);
   break;
  }
  case 175:  {
   $976 = HEAP32[$pParse>>2]|0;
   $977 = HEAP32[$yymsp>>2]|0;
   $978 = ((($977)) + 4|0);
   $979 = HEAP32[$978>>2]|0;
   $980 = (_sqlite3ExprListAppend($976,0,$979)|0);
   HEAP32[$yygotominor>>2] = $980;
   $981 = HEAP32[$pParse>>2]|0;
   $982 = HEAP32[$yygotominor>>2]|0;
   $983 = HEAP32[$yymsp>>2]|0;
   $984 = ((($983)) + -32|0);
   $985 = ((($984)) + 4|0);
   _sqlite3ExprListSetName($981,$982,$985,1);
   break;
  }
  case 176:  {
   $986 = HEAP32[$pParse>>2]|0;
   $987 = HEAP32[$yymsp>>2]|0;
   $988 = ((($987)) + -80|0);
   $989 = ((($988)) + 4|0);
   $990 = HEAP32[$989>>2]|0;
   _sqlite3WithPush($986,$990,1);
   $991 = HEAP32[$pParse>>2]|0;
   $992 = HEAP32[$yymsp>>2]|0;
   $993 = ((($992)) + -32|0);
   $994 = ((($993)) + 4|0);
   $995 = HEAP32[$994>>2]|0;
   $996 = HEAP32[$yymsp>>2]|0;
   $997 = ((($996)) + 4|0);
   $998 = HEAP32[$997>>2]|0;
   $999 = HEAP32[$yymsp>>2]|0;
   $1000 = ((($999)) + -16|0);
   $1001 = ((($1000)) + 4|0);
   $1002 = HEAP32[$1001>>2]|0;
   $1003 = HEAP32[$yymsp>>2]|0;
   $1004 = ((($1003)) + -64|0);
   $1005 = ((($1004)) + 4|0);
   $1006 = HEAP32[$1005>>2]|0;
   _sqlite3Insert($991,$995,$998,$1002,$1006);
   break;
  }
  case 177:  {
   $1007 = HEAP32[$pParse>>2]|0;
   $1008 = HEAP32[$yymsp>>2]|0;
   $1009 = ((($1008)) + -96|0);
   $1010 = ((($1009)) + 4|0);
   $1011 = HEAP32[$1010>>2]|0;
   _sqlite3WithPush($1007,$1011,1);
   $1012 = HEAP32[$pParse>>2]|0;
   $1013 = HEAP32[$yymsp>>2]|0;
   $1014 = ((($1013)) + -48|0);
   $1015 = ((($1014)) + 4|0);
   $1016 = HEAP32[$1015>>2]|0;
   $1017 = HEAP32[$yymsp>>2]|0;
   $1018 = ((($1017)) + -32|0);
   $1019 = ((($1018)) + 4|0);
   $1020 = HEAP32[$1019>>2]|0;
   $1021 = HEAP32[$yymsp>>2]|0;
   $1022 = ((($1021)) + -80|0);
   $1023 = ((($1022)) + 4|0);
   $1024 = HEAP32[$1023>>2]|0;
   _sqlite3Insert($1012,$1016,0,$1020,$1024);
   break;
  }
  case 182:  {
   $1025 = HEAP32[$pParse>>2]|0;
   $1026 = HEAP32[$1025>>2]|0;
   $1027 = HEAP32[$yymsp>>2]|0;
   $1028 = ((($1027)) + -32|0);
   $1029 = ((($1028)) + 4|0);
   $1030 = HEAP32[$1029>>2]|0;
   $1031 = HEAP32[$yymsp>>2]|0;
   $1032 = ((($1031)) + 4|0);
   $1033 = (_sqlite3IdListAppend($1026,$1030,$1032)|0);
   HEAP32[$yygotominor>>2] = $1033;
   break;
  }
  case 183:  {
   $1034 = HEAP32[$pParse>>2]|0;
   $1035 = HEAP32[$1034>>2]|0;
   $1036 = HEAP32[$yymsp>>2]|0;
   $1037 = ((($1036)) + 4|0);
   $1038 = (_sqlite3IdListAppend($1035,0,$1037)|0);
   HEAP32[$yygotominor>>2] = $1038;
   break;
  }
  case 184:  {
   $1039 = HEAP32[$yymsp>>2]|0;
   $1040 = ((($1039)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$1040>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$1040+4>>2]|0;HEAP32[$yygotominor+8>>2]=HEAP32[$1040+8>>2]|0;
   break;
  }
  case 185:  {
   $1041 = HEAP32[$yymsp>>2]|0;
   $1042 = ((($1041)) + -16|0);
   $1043 = ((($1042)) + 4|0);
   $1044 = HEAP32[$1043>>2]|0;
   HEAP32[$yygotominor>>2] = $1044;
   $1045 = HEAP32[$yymsp>>2]|0;
   $1046 = ((($1045)) + -32|0);
   $1047 = ((($1046)) + 4|0);
   $1048 = HEAP32[$yymsp>>2]|0;
   $1049 = ((($1048)) + 4|0);
   _spanSet($yygotominor,$1047,$1049);
   break;
  }
  case 192: case 191: case 186:  {
   $1050 = HEAP32[$pParse>>2]|0;
   $1051 = HEAP32[$yymsp>>2]|0;
   $1052 = ((($1051)) + 2|0);
   $1053 = HEAP8[$1052>>0]|0;
   $1054 = $1053&255;
   $1055 = HEAP32[$yymsp>>2]|0;
   $1056 = ((($1055)) + 4|0);
   _spanExpr($yygotominor,$1050,$1054,$1056);
   break;
  }
  case 188: case 187:  {
   $1057 = HEAP32[$pParse>>2]|0;
   $1058 = HEAP32[$yymsp>>2]|0;
   $1059 = ((($1058)) + 4|0);
   _spanExpr($yygotominor,$1057,27,$1059);
   break;
  }
  case 189:  {
   $1060 = HEAP32[$pParse>>2]|0;
   $1061 = HEAP32[$yymsp>>2]|0;
   $1062 = ((($1061)) + -32|0);
   $1063 = ((($1062)) + 4|0);
   $1064 = (_sqlite3PExpr($1060,27,0,0,$1063)|0);
   HEAP32[$temp1>>2] = $1064;
   $1065 = HEAP32[$pParse>>2]|0;
   $1066 = HEAP32[$yymsp>>2]|0;
   $1067 = ((($1066)) + 4|0);
   $1068 = (_sqlite3PExpr($1065,27,0,0,$1067)|0);
   HEAP32[$temp2>>2] = $1068;
   $1069 = HEAP32[$pParse>>2]|0;
   $1070 = HEAP32[$temp1>>2]|0;
   $1071 = HEAP32[$temp2>>2]|0;
   $1072 = (_sqlite3PExpr($1069,122,$1070,$1071,0)|0);
   HEAP32[$yygotominor>>2] = $1072;
   $1073 = HEAP32[$yymsp>>2]|0;
   $1074 = ((($1073)) + -32|0);
   $1075 = ((($1074)) + 4|0);
   $1076 = HEAP32[$yymsp>>2]|0;
   $1077 = ((($1076)) + 4|0);
   _spanSet($yygotominor,$1075,$1077);
   break;
  }
  case 190:  {
   $1078 = HEAP32[$pParse>>2]|0;
   $1079 = HEAP32[$yymsp>>2]|0;
   $1080 = ((($1079)) + -64|0);
   $1081 = ((($1080)) + 4|0);
   $1082 = (_sqlite3PExpr($1078,27,0,0,$1081)|0);
   HEAP32[$temp15>>2] = $1082;
   $1083 = HEAP32[$pParse>>2]|0;
   $1084 = HEAP32[$yymsp>>2]|0;
   $1085 = ((($1084)) + -32|0);
   $1086 = ((($1085)) + 4|0);
   $1087 = (_sqlite3PExpr($1083,27,0,0,$1086)|0);
   HEAP32[$temp26>>2] = $1087;
   $1088 = HEAP32[$pParse>>2]|0;
   $1089 = HEAP32[$yymsp>>2]|0;
   $1090 = ((($1089)) + 4|0);
   $1091 = (_sqlite3PExpr($1088,27,0,0,$1090)|0);
   HEAP32[$temp3>>2] = $1091;
   $1092 = HEAP32[$pParse>>2]|0;
   $1093 = HEAP32[$temp26>>2]|0;
   $1094 = HEAP32[$temp3>>2]|0;
   $1095 = (_sqlite3PExpr($1092,122,$1093,$1094,0)|0);
   HEAP32[$temp4>>2] = $1095;
   $1096 = HEAP32[$pParse>>2]|0;
   $1097 = HEAP32[$temp15>>2]|0;
   $1098 = HEAP32[$temp4>>2]|0;
   $1099 = (_sqlite3PExpr($1096,122,$1097,$1098,0)|0);
   HEAP32[$yygotominor>>2] = $1099;
   $1100 = HEAP32[$yymsp>>2]|0;
   $1101 = ((($1100)) + -64|0);
   $1102 = ((($1101)) + 4|0);
   $1103 = HEAP32[$yymsp>>2]|0;
   $1104 = ((($1103)) + 4|0);
   _spanSet($yygotominor,$1102,$1104);
   break;
  }
  case 193:  {
   $1105 = HEAP32[$yymsp>>2]|0;
   $1106 = ((($1105)) + 4|0);
   $1107 = ((($1106)) + 4|0);
   $1108 = HEAP32[$1107>>2]|0;
   $1109 = ($1108>>>0)>=(2);
   do {
    if ($1109) {
     $1110 = HEAP32[$yymsp>>2]|0;
     $1111 = ((($1110)) + 4|0);
     $1112 = HEAP32[$1111>>2]|0;
     $1113 = HEAP8[$1112>>0]|0;
     $1114 = $1113 << 24 >> 24;
     $1115 = ($1114|0)==(35);
     if ($1115) {
      $1116 = HEAP32[$yymsp>>2]|0;
      $1117 = ((($1116)) + 4|0);
      $1118 = HEAP32[$1117>>2]|0;
      $1119 = ((($1118)) + 1|0);
      $1120 = HEAP8[$1119>>0]|0;
      $1121 = $1120&255;
      $1122 = (17533 + ($1121)|0);
      $1123 = HEAP8[$1122>>0]|0;
      $1124 = $1123&255;
      $1125 = $1124 & 4;
      $1126 = ($1125|0)!=(0);
      if ($1126) {
       $1127 = HEAP32[$pParse>>2]|0;
       $1128 = ((($1127)) + 18|0);
       $1129 = HEAP8[$1128>>0]|0;
       $1130 = $1129&255;
       $1131 = ($1130|0)==(0);
       $1132 = HEAP32[$pParse>>2]|0;
       $1133 = HEAP32[$yymsp>>2]|0;
       $1134 = ((($1133)) + 4|0);
       if ($1131) {
        HEAP32[$vararg_buffer2>>2] = $1134;
        _sqlite3ErrorMsg($1132,27379,$vararg_buffer2);
        HEAP32[$yygotominor>>2] = 0;
        break;
       }
       $1135 = (_sqlite3PExpr($1132,157,0,0,$1134)|0);
       HEAP32[$yygotominor>>2] = $1135;
       $1136 = HEAP32[$yygotominor>>2]|0;
       $1137 = ($1136|0)!=(0|0);
       if ($1137) {
        $1138 = HEAP32[$yymsp>>2]|0;
        $1139 = ((($1138)) + 4|0);
        $1140 = HEAP32[$1139>>2]|0;
        $1141 = ((($1140)) + 1|0);
        $1142 = HEAP32[$yygotominor>>2]|0;
        $1143 = ((($1142)) + 28|0);
        (_sqlite3GetInt32($1141,$1143)|0);
       }
      } else {
       label = 161;
      }
     } else {
      label = 161;
     }
    } else {
     label = 161;
    }
   } while(0);
   if ((label|0) == 161) {
    $1144 = HEAP32[$pParse>>2]|0;
    $1145 = HEAP32[$yymsp>>2]|0;
    $1146 = ((($1145)) + 4|0);
    _spanExpr($yygotominor,$1144,135,$1146);
    $1147 = HEAP32[$pParse>>2]|0;
    $1148 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprAssignVarNumber($1147,$1148);
   }
   $1149 = HEAP32[$yymsp>>2]|0;
   $1150 = ((($1149)) + 4|0);
   $1151 = HEAP32[$yymsp>>2]|0;
   $1152 = ((($1151)) + 4|0);
   _spanSet($yygotominor,$1150,$1152);
   break;
  }
  case 194:  {
   $1153 = HEAP32[$pParse>>2]|0;
   $1154 = HEAP32[$yymsp>>2]|0;
   $1155 = ((($1154)) + -32|0);
   $1156 = ((($1155)) + 4|0);
   $1157 = HEAP32[$1156>>2]|0;
   $1158 = HEAP32[$yymsp>>2]|0;
   $1159 = ((($1158)) + 4|0);
   $1160 = (_sqlite3ExprAddCollateToken($1153,$1157,$1159,1)|0);
   HEAP32[$yygotominor>>2] = $1160;
   $1161 = HEAP32[$yymsp>>2]|0;
   $1162 = ((($1161)) + -32|0);
   $1163 = ((($1162)) + 4|0);
   $1164 = ((($1163)) + 4|0);
   $1165 = HEAP32[$1164>>2]|0;
   $1166 = ((($yygotominor)) + 4|0);
   HEAP32[$1166>>2] = $1165;
   $1167 = HEAP32[$yymsp>>2]|0;
   $1168 = ((($1167)) + 4|0);
   $1169 = ((($1168)) + 4|0);
   $1170 = HEAP32[$1169>>2]|0;
   $1171 = HEAP32[$yymsp>>2]|0;
   $1172 = ((($1171)) + 4|0);
   $1173 = HEAP32[$1172>>2]|0;
   $1174 = (($1173) + ($1170)|0);
   $1175 = ((($yygotominor)) + 8|0);
   HEAP32[$1175>>2] = $1174;
   break;
  }
  case 195:  {
   $1176 = HEAP32[$pParse>>2]|0;
   $1177 = HEAP32[$yymsp>>2]|0;
   $1178 = ((($1177)) + -48|0);
   $1179 = ((($1178)) + 4|0);
   $1180 = HEAP32[$1179>>2]|0;
   $1181 = HEAP32[$yymsp>>2]|0;
   $1182 = ((($1181)) + -16|0);
   $1183 = ((($1182)) + 4|0);
   $1184 = (_sqlite3PExpr($1176,38,$1180,0,$1183)|0);
   HEAP32[$yygotominor>>2] = $1184;
   $1185 = HEAP32[$yymsp>>2]|0;
   $1186 = ((($1185)) + -80|0);
   $1187 = ((($1186)) + 4|0);
   $1188 = HEAP32[$yymsp>>2]|0;
   $1189 = ((($1188)) + 4|0);
   _spanSet($yygotominor,$1187,$1189);
   break;
  }
  case 196:  {
   $1190 = HEAP32[$yymsp>>2]|0;
   $1191 = ((($1190)) + -16|0);
   $1192 = ((($1191)) + 4|0);
   $1193 = HEAP32[$1192>>2]|0;
   $1194 = ($1193|0)!=(0|0);
   if ($1194) {
    $1195 = HEAP32[$yymsp>>2]|0;
    $1196 = ((($1195)) + -16|0);
    $1197 = ((($1196)) + 4|0);
    $1198 = HEAP32[$1197>>2]|0;
    $1199 = HEAP32[$1198>>2]|0;
    $1200 = HEAP32[$pParse>>2]|0;
    $1201 = HEAP32[$1200>>2]|0;
    $1202 = ((($1201)) + 88|0);
    $1203 = ((($1202)) + 24|0);
    $1204 = HEAP32[$1203>>2]|0;
    $1205 = ($1199|0)>($1204|0);
    if ($1205) {
     $1206 = HEAP32[$pParse>>2]|0;
     $1207 = HEAP32[$yymsp>>2]|0;
     $1208 = ((($1207)) + -64|0);
     $1209 = ((($1208)) + 4|0);
     HEAP32[$vararg_buffer5>>2] = $1209;
     _sqlite3ErrorMsg($1206,27403,$vararg_buffer5);
    }
   }
   $1210 = HEAP32[$pParse>>2]|0;
   $1211 = HEAP32[$yymsp>>2]|0;
   $1212 = ((($1211)) + -16|0);
   $1213 = ((($1212)) + 4|0);
   $1214 = HEAP32[$1213>>2]|0;
   $1215 = HEAP32[$yymsp>>2]|0;
   $1216 = ((($1215)) + -64|0);
   $1217 = ((($1216)) + 4|0);
   $1218 = (_sqlite3ExprFunction($1210,$1214,$1217)|0);
   HEAP32[$yygotominor>>2] = $1218;
   $1219 = HEAP32[$yymsp>>2]|0;
   $1220 = ((($1219)) + -64|0);
   $1221 = ((($1220)) + 4|0);
   $1222 = HEAP32[$yymsp>>2]|0;
   $1223 = ((($1222)) + 4|0);
   _spanSet($yygotominor,$1221,$1223);
   $1224 = HEAP32[$yymsp>>2]|0;
   $1225 = ((($1224)) + -32|0);
   $1226 = ((($1225)) + 4|0);
   $1227 = HEAP32[$1226>>2]|0;
   $1228 = ($1227|0)==(1);
   if ($1228) {
    $1229 = HEAP32[$yygotominor>>2]|0;
    $1230 = ($1229|0)!=(0|0);
    if ($1230) {
     $1231 = HEAP32[$yygotominor>>2]|0;
     $1232 = ((($1231)) + 4|0);
     $1233 = HEAP32[$1232>>2]|0;
     $1234 = $1233 | 16;
     HEAP32[$1232>>2] = $1234;
    }
   }
   break;
  }
  case 197:  {
   $1235 = HEAP32[$pParse>>2]|0;
   $1236 = HEAP32[$yymsp>>2]|0;
   $1237 = ((($1236)) + -48|0);
   $1238 = ((($1237)) + 4|0);
   $1239 = (_sqlite3ExprFunction($1235,0,$1238)|0);
   HEAP32[$yygotominor>>2] = $1239;
   $1240 = HEAP32[$yymsp>>2]|0;
   $1241 = ((($1240)) + -48|0);
   $1242 = ((($1241)) + 4|0);
   $1243 = HEAP32[$yymsp>>2]|0;
   $1244 = ((($1243)) + 4|0);
   _spanSet($yygotominor,$1242,$1244);
   break;
  }
  case 198:  {
   $1245 = HEAP32[$pParse>>2]|0;
   $1246 = HEAP32[$yymsp>>2]|0;
   $1247 = ((($1246)) + 4|0);
   $1248 = (_sqlite3ExprFunction($1245,0,$1247)|0);
   HEAP32[$yygotominor>>2] = $1248;
   $1249 = HEAP32[$yymsp>>2]|0;
   $1250 = ((($1249)) + 4|0);
   $1251 = HEAP32[$yymsp>>2]|0;
   $1252 = ((($1251)) + 4|0);
   _spanSet($yygotominor,$1250,$1252);
   break;
  }
  case 206: case 205: case 204: case 203: case 202: case 201: case 200: case 199:  {
   $1253 = HEAP32[$pParse>>2]|0;
   $1254 = HEAP32[$yymsp>>2]|0;
   $1255 = ((($1254)) + -16|0);
   $1256 = ((($1255)) + 2|0);
   $1257 = HEAP8[$1256>>0]|0;
   $1258 = $1257&255;
   $1259 = HEAP32[$yymsp>>2]|0;
   $1260 = ((($1259)) + -32|0);
   $1261 = ((($1260)) + 4|0);
   $1262 = HEAP32[$yymsp>>2]|0;
   $1263 = ((($1262)) + 4|0);
   _spanBinaryExpr($yygotominor,$1253,$1258,$1261,$1263);
   break;
  }
  case 207:  {
   $1264 = HEAP32[$yymsp>>2]|0;
   $1265 = ((($1264)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$1265>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$1265+4>>2]|0;
   $1266 = ((($yygotominor)) + 8|0);
   HEAP32[$1266>>2] = 0;
   break;
  }
  case 208:  {
   $1267 = HEAP32[$yymsp>>2]|0;
   $1268 = ((($1267)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$1268>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$1268+4>>2]|0;
   $1269 = ((($yygotominor)) + 8|0);
   HEAP32[$1269>>2] = 1;
   break;
  }
  case 209:  {
   $1270 = HEAP32[$pParse>>2]|0;
   $1271 = HEAP32[$yymsp>>2]|0;
   $1272 = ((($1271)) + 4|0);
   $1273 = HEAP32[$1272>>2]|0;
   $1274 = (_sqlite3ExprListAppend($1270,0,$1273)|0);
   HEAP32[$pList>>2] = $1274;
   $1275 = HEAP32[$pParse>>2]|0;
   $1276 = HEAP32[$pList>>2]|0;
   $1277 = HEAP32[$yymsp>>2]|0;
   $1278 = ((($1277)) + -32|0);
   $1279 = ((($1278)) + 4|0);
   $1280 = HEAP32[$1279>>2]|0;
   $1281 = (_sqlite3ExprListAppend($1275,$1276,$1280)|0);
   HEAP32[$pList>>2] = $1281;
   $1282 = HEAP32[$pParse>>2]|0;
   $1283 = HEAP32[$pList>>2]|0;
   $1284 = HEAP32[$yymsp>>2]|0;
   $1285 = ((($1284)) + -16|0);
   $1286 = ((($1285)) + 4|0);
   $1287 = (_sqlite3ExprFunction($1282,$1283,$1286)|0);
   HEAP32[$yygotominor>>2] = $1287;
   $1288 = HEAP32[$pParse>>2]|0;
   $1289 = HEAP32[$yymsp>>2]|0;
   $1290 = ((($1289)) + -16|0);
   $1291 = ((($1290)) + 4|0);
   $1292 = ((($1291)) + 8|0);
   $1293 = HEAP32[$1292>>2]|0;
   _exprNot($1288,$1293,$yygotominor);
   $1294 = HEAP32[$yymsp>>2]|0;
   $1295 = ((($1294)) + -32|0);
   $1296 = ((($1295)) + 4|0);
   $1297 = ((($1296)) + 4|0);
   $1298 = HEAP32[$1297>>2]|0;
   $1299 = ((($yygotominor)) + 4|0);
   HEAP32[$1299>>2] = $1298;
   $1300 = HEAP32[$yymsp>>2]|0;
   $1301 = ((($1300)) + 4|0);
   $1302 = ((($1301)) + 8|0);
   $1303 = HEAP32[$1302>>2]|0;
   $1304 = ((($yygotominor)) + 8|0);
   HEAP32[$1304>>2] = $1303;
   $1305 = HEAP32[$yygotominor>>2]|0;
   $1306 = ($1305|0)!=(0|0);
   if ($1306) {
    $1307 = HEAP32[$yygotominor>>2]|0;
    $1308 = ((($1307)) + 4|0);
    $1309 = HEAP32[$1308>>2]|0;
    $1310 = $1309 | 128;
    HEAP32[$1308>>2] = $1310;
   }
   break;
  }
  case 210:  {
   $1311 = HEAP32[$pParse>>2]|0;
   $1312 = HEAP32[$yymsp>>2]|0;
   $1313 = ((($1312)) + -32|0);
   $1314 = ((($1313)) + 4|0);
   $1315 = HEAP32[$1314>>2]|0;
   $1316 = (_sqlite3ExprListAppend($1311,0,$1315)|0);
   HEAP32[$pList7>>2] = $1316;
   $1317 = HEAP32[$pParse>>2]|0;
   $1318 = HEAP32[$pList7>>2]|0;
   $1319 = HEAP32[$yymsp>>2]|0;
   $1320 = ((($1319)) + -64|0);
   $1321 = ((($1320)) + 4|0);
   $1322 = HEAP32[$1321>>2]|0;
   $1323 = (_sqlite3ExprListAppend($1317,$1318,$1322)|0);
   HEAP32[$pList7>>2] = $1323;
   $1324 = HEAP32[$pParse>>2]|0;
   $1325 = HEAP32[$pList7>>2]|0;
   $1326 = HEAP32[$yymsp>>2]|0;
   $1327 = ((($1326)) + 4|0);
   $1328 = HEAP32[$1327>>2]|0;
   $1329 = (_sqlite3ExprListAppend($1324,$1325,$1328)|0);
   HEAP32[$pList7>>2] = $1329;
   $1330 = HEAP32[$pParse>>2]|0;
   $1331 = HEAP32[$pList7>>2]|0;
   $1332 = HEAP32[$yymsp>>2]|0;
   $1333 = ((($1332)) + -48|0);
   $1334 = ((($1333)) + 4|0);
   $1335 = (_sqlite3ExprFunction($1330,$1331,$1334)|0);
   HEAP32[$yygotominor>>2] = $1335;
   $1336 = HEAP32[$pParse>>2]|0;
   $1337 = HEAP32[$yymsp>>2]|0;
   $1338 = ((($1337)) + -48|0);
   $1339 = ((($1338)) + 4|0);
   $1340 = ((($1339)) + 8|0);
   $1341 = HEAP32[$1340>>2]|0;
   _exprNot($1336,$1341,$yygotominor);
   $1342 = HEAP32[$yymsp>>2]|0;
   $1343 = ((($1342)) + -64|0);
   $1344 = ((($1343)) + 4|0);
   $1345 = ((($1344)) + 4|0);
   $1346 = HEAP32[$1345>>2]|0;
   $1347 = ((($yygotominor)) + 4|0);
   HEAP32[$1347>>2] = $1346;
   $1348 = HEAP32[$yymsp>>2]|0;
   $1349 = ((($1348)) + 4|0);
   $1350 = ((($1349)) + 8|0);
   $1351 = HEAP32[$1350>>2]|0;
   $1352 = ((($yygotominor)) + 8|0);
   HEAP32[$1352>>2] = $1351;
   $1353 = HEAP32[$yygotominor>>2]|0;
   $1354 = ($1353|0)!=(0|0);
   if ($1354) {
    $1355 = HEAP32[$yygotominor>>2]|0;
    $1356 = ((($1355)) + 4|0);
    $1357 = HEAP32[$1356>>2]|0;
    $1358 = $1357 | 128;
    HEAP32[$1356>>2] = $1358;
   }
   break;
  }
  case 211:  {
   $1359 = HEAP32[$pParse>>2]|0;
   $1360 = HEAP32[$yymsp>>2]|0;
   $1361 = ((($1360)) + 2|0);
   $1362 = HEAP8[$1361>>0]|0;
   $1363 = $1362&255;
   $1364 = HEAP32[$yymsp>>2]|0;
   $1365 = ((($1364)) + -16|0);
   $1366 = ((($1365)) + 4|0);
   $1367 = HEAP32[$yymsp>>2]|0;
   $1368 = ((($1367)) + 4|0);
   _spanUnaryPostfix($yygotominor,$1359,$1363,$1366,$1368);
   break;
  }
  case 212:  {
   $1369 = HEAP32[$pParse>>2]|0;
   $1370 = HEAP32[$yymsp>>2]|0;
   $1371 = ((($1370)) + -32|0);
   $1372 = ((($1371)) + 4|0);
   $1373 = HEAP32[$yymsp>>2]|0;
   $1374 = ((($1373)) + 4|0);
   _spanUnaryPostfix($yygotominor,$1369,77,$1372,$1374);
   break;
  }
  case 213:  {
   $1375 = HEAP32[$pParse>>2]|0;
   $1376 = HEAP32[$yymsp>>2]|0;
   $1377 = ((($1376)) + -32|0);
   $1378 = ((($1377)) + 4|0);
   $1379 = HEAP32[$yymsp>>2]|0;
   $1380 = ((($1379)) + 4|0);
   _spanBinaryExpr($yygotominor,$1375,73,$1378,$1380);
   $1381 = HEAP32[$pParse>>2]|0;
   $1382 = HEAP32[$yymsp>>2]|0;
   $1383 = ((($1382)) + 4|0);
   $1384 = HEAP32[$1383>>2]|0;
   $1385 = HEAP32[$yygotominor>>2]|0;
   _binaryToUnaryIfNull($1381,$1384,$1385,76);
   break;
  }
  case 214:  {
   $1386 = HEAP32[$pParse>>2]|0;
   $1387 = HEAP32[$yymsp>>2]|0;
   $1388 = ((($1387)) + -48|0);
   $1389 = ((($1388)) + 4|0);
   $1390 = HEAP32[$yymsp>>2]|0;
   $1391 = ((($1390)) + 4|0);
   _spanBinaryExpr($yygotominor,$1386,148,$1389,$1391);
   $1392 = HEAP32[$pParse>>2]|0;
   $1393 = HEAP32[$yymsp>>2]|0;
   $1394 = ((($1393)) + 4|0);
   $1395 = HEAP32[$1394>>2]|0;
   $1396 = HEAP32[$yygotominor>>2]|0;
   _binaryToUnaryIfNull($1392,$1395,$1396,77);
   break;
  }
  case 216: case 215:  {
   $1397 = HEAP32[$pParse>>2]|0;
   $1398 = HEAP32[$yymsp>>2]|0;
   $1399 = ((($1398)) + -16|0);
   $1400 = ((($1399)) + 2|0);
   $1401 = HEAP8[$1400>>0]|0;
   $1402 = $1401&255;
   $1403 = HEAP32[$yymsp>>2]|0;
   $1404 = ((($1403)) + 4|0);
   $1405 = HEAP32[$yymsp>>2]|0;
   $1406 = ((($1405)) + -16|0);
   $1407 = ((($1406)) + 4|0);
   _spanUnaryPrefix($yygotominor,$1397,$1402,$1404,$1407);
   break;
  }
  case 217:  {
   $1408 = HEAP32[$pParse>>2]|0;
   $1409 = HEAP32[$yymsp>>2]|0;
   $1410 = ((($1409)) + 4|0);
   $1411 = HEAP32[$yymsp>>2]|0;
   $1412 = ((($1411)) + -16|0);
   $1413 = ((($1412)) + 4|0);
   _spanUnaryPrefix($yygotominor,$1408,155,$1410,$1413);
   break;
  }
  case 218:  {
   $1414 = HEAP32[$pParse>>2]|0;
   $1415 = HEAP32[$yymsp>>2]|0;
   $1416 = ((($1415)) + 4|0);
   $1417 = HEAP32[$yymsp>>2]|0;
   $1418 = ((($1417)) + -16|0);
   $1419 = ((($1418)) + 4|0);
   _spanUnaryPrefix($yygotominor,$1414,156,$1416,$1419);
   break;
  }
  case 221:  {
   $1420 = HEAP32[$pParse>>2]|0;
   $1421 = HEAP32[$yymsp>>2]|0;
   $1422 = ((($1421)) + -32|0);
   $1423 = ((($1422)) + 4|0);
   $1424 = HEAP32[$1423>>2]|0;
   $1425 = (_sqlite3ExprListAppend($1420,0,$1424)|0);
   HEAP32[$pList8>>2] = $1425;
   $1426 = HEAP32[$pParse>>2]|0;
   $1427 = HEAP32[$pList8>>2]|0;
   $1428 = HEAP32[$yymsp>>2]|0;
   $1429 = ((($1428)) + 4|0);
   $1430 = HEAP32[$1429>>2]|0;
   $1431 = (_sqlite3ExprListAppend($1426,$1427,$1430)|0);
   HEAP32[$pList8>>2] = $1431;
   $1432 = HEAP32[$pParse>>2]|0;
   $1433 = HEAP32[$yymsp>>2]|0;
   $1434 = ((($1433)) + -64|0);
   $1435 = ((($1434)) + 4|0);
   $1436 = HEAP32[$1435>>2]|0;
   $1437 = (_sqlite3PExpr($1432,74,$1436,0,0)|0);
   HEAP32[$yygotominor>>2] = $1437;
   $1438 = HEAP32[$yygotominor>>2]|0;
   $1439 = ($1438|0)!=(0|0);
   if ($1439) {
    $1440 = HEAP32[$pList8>>2]|0;
    $1441 = HEAP32[$yygotominor>>2]|0;
    $1442 = ((($1441)) + 20|0);
    HEAP32[$1442>>2] = $1440;
   } else {
    $1443 = HEAP32[$pParse>>2]|0;
    $1444 = HEAP32[$1443>>2]|0;
    $1445 = HEAP32[$pList8>>2]|0;
    _sqlite3ExprListDelete($1444,$1445);
   }
   $1446 = HEAP32[$pParse>>2]|0;
   $1447 = HEAP32[$yymsp>>2]|0;
   $1448 = ((($1447)) + -48|0);
   $1449 = ((($1448)) + 4|0);
   $1450 = HEAP32[$1449>>2]|0;
   _exprNot($1446,$1450,$yygotominor);
   $1451 = HEAP32[$yymsp>>2]|0;
   $1452 = ((($1451)) + -64|0);
   $1453 = ((($1452)) + 4|0);
   $1454 = ((($1453)) + 4|0);
   $1455 = HEAP32[$1454>>2]|0;
   $1456 = ((($yygotominor)) + 4|0);
   HEAP32[$1456>>2] = $1455;
   $1457 = HEAP32[$yymsp>>2]|0;
   $1458 = ((($1457)) + 4|0);
   $1459 = ((($1458)) + 8|0);
   $1460 = HEAP32[$1459>>2]|0;
   $1461 = ((($yygotominor)) + 8|0);
   HEAP32[$1461>>2] = $1460;
   break;
  }
  case 224:  {
   $1462 = HEAP32[$yymsp>>2]|0;
   $1463 = ((($1462)) + -16|0);
   $1464 = ((($1463)) + 4|0);
   $1465 = HEAP32[$1464>>2]|0;
   $1466 = ($1465|0)==(0|0);
   do {
    if ($1466) {
     $1467 = HEAP32[$pParse>>2]|0;
     $1468 = HEAP32[$yymsp>>2]|0;
     $1469 = ((($1468)) + -48|0);
     $1470 = ((($1469)) + 4|0);
     $1471 = HEAP32[$1470>>2]|0;
     $1472 = (2528 + ($1471<<3)|0);
     $1473 = (_sqlite3PExpr($1467,132,0,0,$1472)|0);
     HEAP32[$yygotominor>>2] = $1473;
     $1474 = HEAP32[$pParse>>2]|0;
     $1475 = HEAP32[$1474>>2]|0;
     $1476 = HEAP32[$yymsp>>2]|0;
     $1477 = ((($1476)) + -64|0);
     $1478 = ((($1477)) + 4|0);
     $1479 = HEAP32[$1478>>2]|0;
     _sqlite3ExprDelete($1475,$1479);
    } else {
     $1480 = HEAP32[$yymsp>>2]|0;
     $1481 = ((($1480)) + -16|0);
     $1482 = ((($1481)) + 4|0);
     $1483 = HEAP32[$1482>>2]|0;
     $1484 = HEAP32[$1483>>2]|0;
     $1485 = ($1484|0)==(1);
     if ($1485) {
      $1486 = HEAP32[$yymsp>>2]|0;
      $1487 = ((($1486)) + -16|0);
      $1488 = ((($1487)) + 4|0);
      $1489 = HEAP32[$1488>>2]|0;
      $1490 = ((($1489)) + 4|0);
      $1491 = HEAP32[$1490>>2]|0;
      $1492 = HEAP32[$1491>>2]|0;
      HEAP32[$pRHS>>2] = $1492;
      $1493 = HEAP32[$yymsp>>2]|0;
      $1494 = ((($1493)) + -16|0);
      $1495 = ((($1494)) + 4|0);
      $1496 = HEAP32[$1495>>2]|0;
      $1497 = ((($1496)) + 4|0);
      $1498 = HEAP32[$1497>>2]|0;
      HEAP32[$1498>>2] = 0;
      $1499 = HEAP32[$pParse>>2]|0;
      $1500 = HEAP32[$1499>>2]|0;
      $1501 = HEAP32[$yymsp>>2]|0;
      $1502 = ((($1501)) + -16|0);
      $1503 = ((($1502)) + 4|0);
      $1504 = HEAP32[$1503>>2]|0;
      _sqlite3ExprListDelete($1500,$1504);
      $1505 = HEAP32[$pRHS>>2]|0;
      $1506 = ($1505|0)!=(0|0);
      if ($1506) {
       $1507 = HEAP32[$pRHS>>2]|0;
       $1508 = ((($1507)) + 4|0);
       $1509 = HEAP32[$1508>>2]|0;
       $1510 = $1509 & -257;
       HEAP32[$1508>>2] = $1510;
       $1511 = HEAP32[$pRHS>>2]|0;
       $1512 = ((($1511)) + 4|0);
       $1513 = HEAP32[$1512>>2]|0;
       $1514 = $1513 | 512;
       HEAP32[$1512>>2] = $1514;
      }
      $1515 = HEAP32[$pParse>>2]|0;
      $1516 = HEAP32[$yymsp>>2]|0;
      $1517 = ((($1516)) + -48|0);
      $1518 = ((($1517)) + 4|0);
      $1519 = HEAP32[$1518>>2]|0;
      $1520 = ($1519|0)!=(0);
      $1521 = $1520 ? 78 : 79;
      $1522 = HEAP32[$yymsp>>2]|0;
      $1523 = ((($1522)) + -64|0);
      $1524 = ((($1523)) + 4|0);
      $1525 = HEAP32[$1524>>2]|0;
      $1526 = HEAP32[$pRHS>>2]|0;
      $1527 = (_sqlite3PExpr($1515,$1521,$1525,$1526,0)|0);
      HEAP32[$yygotominor>>2] = $1527;
      break;
     }
     $1528 = HEAP32[$pParse>>2]|0;
     $1529 = HEAP32[$yymsp>>2]|0;
     $1530 = ((($1529)) + -64|0);
     $1531 = ((($1530)) + 4|0);
     $1532 = HEAP32[$1531>>2]|0;
     $1533 = (_sqlite3PExpr($1528,75,$1532,0,0)|0);
     HEAP32[$yygotominor>>2] = $1533;
     $1534 = HEAP32[$yygotominor>>2]|0;
     $1535 = ($1534|0)!=(0|0);
     if ($1535) {
      $1536 = HEAP32[$yymsp>>2]|0;
      $1537 = ((($1536)) + -16|0);
      $1538 = ((($1537)) + 4|0);
      $1539 = HEAP32[$1538>>2]|0;
      $1540 = HEAP32[$yygotominor>>2]|0;
      $1541 = ((($1540)) + 20|0);
      HEAP32[$1541>>2] = $1539;
      $1542 = HEAP32[$pParse>>2]|0;
      $1543 = HEAP32[$yygotominor>>2]|0;
      _sqlite3ExprSetHeightAndFlags($1542,$1543);
     } else {
      $1544 = HEAP32[$pParse>>2]|0;
      $1545 = HEAP32[$1544>>2]|0;
      $1546 = HEAP32[$yymsp>>2]|0;
      $1547 = ((($1546)) + -16|0);
      $1548 = ((($1547)) + 4|0);
      $1549 = HEAP32[$1548>>2]|0;
      _sqlite3ExprListDelete($1545,$1549);
     }
     $1550 = HEAP32[$pParse>>2]|0;
     $1551 = HEAP32[$yymsp>>2]|0;
     $1552 = ((($1551)) + -48|0);
     $1553 = ((($1552)) + 4|0);
     $1554 = HEAP32[$1553>>2]|0;
     _exprNot($1550,$1554,$yygotominor);
    }
   } while(0);
   $1555 = HEAP32[$yymsp>>2]|0;
   $1556 = ((($1555)) + -64|0);
   $1557 = ((($1556)) + 4|0);
   $1558 = ((($1557)) + 4|0);
   $1559 = HEAP32[$1558>>2]|0;
   $1560 = ((($yygotominor)) + 4|0);
   HEAP32[$1560>>2] = $1559;
   $1561 = HEAP32[$yymsp>>2]|0;
   $1562 = ((($1561)) + 4|0);
   $1563 = ((($1562)) + 4|0);
   $1564 = HEAP32[$1563>>2]|0;
   $1565 = HEAP32[$yymsp>>2]|0;
   $1566 = ((($1565)) + 4|0);
   $1567 = HEAP32[$1566>>2]|0;
   $1568 = (($1567) + ($1564)|0);
   $1569 = ((($yygotominor)) + 8|0);
   HEAP32[$1569>>2] = $1568;
   break;
  }
  case 225:  {
   $1570 = HEAP32[$pParse>>2]|0;
   $1571 = (_sqlite3PExpr($1570,119,0,0,0)|0);
   HEAP32[$yygotominor>>2] = $1571;
   $1572 = HEAP32[$yygotominor>>2]|0;
   $1573 = ($1572|0)!=(0|0);
   if ($1573) {
    $1574 = HEAP32[$yymsp>>2]|0;
    $1575 = ((($1574)) + -16|0);
    $1576 = ((($1575)) + 4|0);
    $1577 = HEAP32[$1576>>2]|0;
    $1578 = HEAP32[$yygotominor>>2]|0;
    $1579 = ((($1578)) + 20|0);
    HEAP32[$1579>>2] = $1577;
    $1580 = HEAP32[$yygotominor>>2]|0;
    $1581 = ((($1580)) + 4|0);
    $1582 = HEAP32[$1581>>2]|0;
    $1583 = $1582 | 2099200;
    HEAP32[$1581>>2] = $1583;
    $1584 = HEAP32[$pParse>>2]|0;
    $1585 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeightAndFlags($1584,$1585);
   } else {
    $1586 = HEAP32[$pParse>>2]|0;
    $1587 = HEAP32[$1586>>2]|0;
    $1588 = HEAP32[$yymsp>>2]|0;
    $1589 = ((($1588)) + -16|0);
    $1590 = ((($1589)) + 4|0);
    $1591 = HEAP32[$1590>>2]|0;
    _sqlite3SelectDelete($1587,$1591);
   }
   $1592 = HEAP32[$yymsp>>2]|0;
   $1593 = ((($1592)) + -32|0);
   $1594 = ((($1593)) + 4|0);
   $1595 = HEAP32[$1594>>2]|0;
   $1596 = ((($yygotominor)) + 4|0);
   HEAP32[$1596>>2] = $1595;
   $1597 = HEAP32[$yymsp>>2]|0;
   $1598 = ((($1597)) + 4|0);
   $1599 = ((($1598)) + 4|0);
   $1600 = HEAP32[$1599>>2]|0;
   $1601 = HEAP32[$yymsp>>2]|0;
   $1602 = ((($1601)) + 4|0);
   $1603 = HEAP32[$1602>>2]|0;
   $1604 = (($1603) + ($1600)|0);
   $1605 = ((($yygotominor)) + 8|0);
   HEAP32[$1605>>2] = $1604;
   break;
  }
  case 226:  {
   $1606 = HEAP32[$pParse>>2]|0;
   $1607 = HEAP32[$yymsp>>2]|0;
   $1608 = ((($1607)) + -64|0);
   $1609 = ((($1608)) + 4|0);
   $1610 = HEAP32[$1609>>2]|0;
   $1611 = (_sqlite3PExpr($1606,75,$1610,0,0)|0);
   HEAP32[$yygotominor>>2] = $1611;
   $1612 = HEAP32[$yygotominor>>2]|0;
   $1613 = ($1612|0)!=(0|0);
   if ($1613) {
    $1614 = HEAP32[$yymsp>>2]|0;
    $1615 = ((($1614)) + -16|0);
    $1616 = ((($1615)) + 4|0);
    $1617 = HEAP32[$1616>>2]|0;
    $1618 = HEAP32[$yygotominor>>2]|0;
    $1619 = ((($1618)) + 20|0);
    HEAP32[$1619>>2] = $1617;
    $1620 = HEAP32[$yygotominor>>2]|0;
    $1621 = ((($1620)) + 4|0);
    $1622 = HEAP32[$1621>>2]|0;
    $1623 = $1622 | 2099200;
    HEAP32[$1621>>2] = $1623;
    $1624 = HEAP32[$pParse>>2]|0;
    $1625 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeightAndFlags($1624,$1625);
   } else {
    $1626 = HEAP32[$pParse>>2]|0;
    $1627 = HEAP32[$1626>>2]|0;
    $1628 = HEAP32[$yymsp>>2]|0;
    $1629 = ((($1628)) + -16|0);
    $1630 = ((($1629)) + 4|0);
    $1631 = HEAP32[$1630>>2]|0;
    _sqlite3SelectDelete($1627,$1631);
   }
   $1632 = HEAP32[$pParse>>2]|0;
   $1633 = HEAP32[$yymsp>>2]|0;
   $1634 = ((($1633)) + -48|0);
   $1635 = ((($1634)) + 4|0);
   $1636 = HEAP32[$1635>>2]|0;
   _exprNot($1632,$1636,$yygotominor);
   $1637 = HEAP32[$yymsp>>2]|0;
   $1638 = ((($1637)) + -64|0);
   $1639 = ((($1638)) + 4|0);
   $1640 = ((($1639)) + 4|0);
   $1641 = HEAP32[$1640>>2]|0;
   $1642 = ((($yygotominor)) + 4|0);
   HEAP32[$1642>>2] = $1641;
   $1643 = HEAP32[$yymsp>>2]|0;
   $1644 = ((($1643)) + 4|0);
   $1645 = ((($1644)) + 4|0);
   $1646 = HEAP32[$1645>>2]|0;
   $1647 = HEAP32[$yymsp>>2]|0;
   $1648 = ((($1647)) + 4|0);
   $1649 = HEAP32[$1648>>2]|0;
   $1650 = (($1649) + ($1646)|0);
   $1651 = ((($yygotominor)) + 8|0);
   HEAP32[$1651>>2] = $1650;
   break;
  }
  case 227:  {
   $1652 = HEAP32[$pParse>>2]|0;
   $1653 = HEAP32[$1652>>2]|0;
   $1654 = HEAP32[$yymsp>>2]|0;
   $1655 = ((($1654)) + -16|0);
   $1656 = ((($1655)) + 4|0);
   $1657 = HEAP32[$yymsp>>2]|0;
   $1658 = ((($1657)) + 4|0);
   $1659 = (_sqlite3SrcListAppend($1653,0,$1656,$1658)|0);
   HEAP32[$pSrc>>2] = $1659;
   $1660 = HEAP32[$pParse>>2]|0;
   $1661 = HEAP32[$yymsp>>2]|0;
   $1662 = ((($1661)) + -48|0);
   $1663 = ((($1662)) + 4|0);
   $1664 = HEAP32[$1663>>2]|0;
   $1665 = (_sqlite3PExpr($1660,75,$1664,0,0)|0);
   HEAP32[$yygotominor>>2] = $1665;
   $1666 = HEAP32[$yygotominor>>2]|0;
   $1667 = ($1666|0)!=(0|0);
   $1668 = HEAP32[$pParse>>2]|0;
   if ($1667) {
    $1669 = HEAP32[$pSrc>>2]|0;
    $1670 = (_sqlite3SelectNew($1668,0,$1669,0,0,0,0,0,0,0)|0);
    $1671 = HEAP32[$yygotominor>>2]|0;
    $1672 = ((($1671)) + 20|0);
    HEAP32[$1672>>2] = $1670;
    $1673 = HEAP32[$yygotominor>>2]|0;
    $1674 = ((($1673)) + 4|0);
    $1675 = HEAP32[$1674>>2]|0;
    $1676 = $1675 | 2099200;
    HEAP32[$1674>>2] = $1676;
    $1677 = HEAP32[$pParse>>2]|0;
    $1678 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeightAndFlags($1677,$1678);
   } else {
    $1679 = HEAP32[$1668>>2]|0;
    $1680 = HEAP32[$pSrc>>2]|0;
    _sqlite3SrcListDelete($1679,$1680);
   }
   $1681 = HEAP32[$pParse>>2]|0;
   $1682 = HEAP32[$yymsp>>2]|0;
   $1683 = ((($1682)) + -32|0);
   $1684 = ((($1683)) + 4|0);
   $1685 = HEAP32[$1684>>2]|0;
   _exprNot($1681,$1685,$yygotominor);
   $1686 = HEAP32[$yymsp>>2]|0;
   $1687 = ((($1686)) + -48|0);
   $1688 = ((($1687)) + 4|0);
   $1689 = ((($1688)) + 4|0);
   $1690 = HEAP32[$1689>>2]|0;
   $1691 = ((($yygotominor)) + 4|0);
   HEAP32[$1691>>2] = $1690;
   $1692 = HEAP32[$yymsp>>2]|0;
   $1693 = ((($1692)) + 4|0);
   $1694 = HEAP32[$1693>>2]|0;
   $1695 = ($1694|0)!=(0|0);
   $1696 = HEAP32[$yymsp>>2]|0;
   if ($1695) {
    $1697 = ((($1696)) + 4|0);
    $1698 = ((($1697)) + 4|0);
    $1699 = HEAP32[$1698>>2]|0;
    $1700 = HEAP32[$yymsp>>2]|0;
    $1701 = ((($1700)) + 4|0);
    $1702 = HEAP32[$1701>>2]|0;
    $1703 = (($1702) + ($1699)|0);
    $1714 = $1703;
   } else {
    $1704 = ((($1696)) + -16|0);
    $1705 = ((($1704)) + 4|0);
    $1706 = ((($1705)) + 4|0);
    $1707 = HEAP32[$1706>>2]|0;
    $1708 = HEAP32[$yymsp>>2]|0;
    $1709 = ((($1708)) + -16|0);
    $1710 = ((($1709)) + 4|0);
    $1711 = HEAP32[$1710>>2]|0;
    $1712 = (($1711) + ($1707)|0);
    $1714 = $1712;
   }
   $1713 = ((($yygotominor)) + 8|0);
   HEAP32[$1713>>2] = $1714;
   break;
  }
  case 228:  {
   $1715 = HEAP32[$pParse>>2]|0;
   $1716 = (_sqlite3PExpr($1715,20,0,0,0)|0);
   HEAP32[$yygotominor>>2] = $1716;
   HEAP32[$p9>>2] = $1716;
   $1717 = HEAP32[$p9>>2]|0;
   $1718 = ($1717|0)!=(0|0);
   if ($1718) {
    $1719 = HEAP32[$yymsp>>2]|0;
    $1720 = ((($1719)) + -16|0);
    $1721 = ((($1720)) + 4|0);
    $1722 = HEAP32[$1721>>2]|0;
    $1723 = HEAP32[$p9>>2]|0;
    $1724 = ((($1723)) + 20|0);
    HEAP32[$1724>>2] = $1722;
    $1725 = HEAP32[$p9>>2]|0;
    $1726 = ((($1725)) + 4|0);
    $1727 = HEAP32[$1726>>2]|0;
    $1728 = $1727 | 2099200;
    HEAP32[$1726>>2] = $1728;
    $1729 = HEAP32[$pParse>>2]|0;
    $1730 = HEAP32[$p9>>2]|0;
    _sqlite3ExprSetHeightAndFlags($1729,$1730);
   } else {
    $1731 = HEAP32[$pParse>>2]|0;
    $1732 = HEAP32[$1731>>2]|0;
    $1733 = HEAP32[$yymsp>>2]|0;
    $1734 = ((($1733)) + -16|0);
    $1735 = ((($1734)) + 4|0);
    $1736 = HEAP32[$1735>>2]|0;
    _sqlite3SelectDelete($1732,$1736);
   }
   $1737 = HEAP32[$yymsp>>2]|0;
   $1738 = ((($1737)) + -48|0);
   $1739 = ((($1738)) + 4|0);
   $1740 = HEAP32[$1739>>2]|0;
   $1741 = ((($yygotominor)) + 4|0);
   HEAP32[$1741>>2] = $1740;
   $1742 = HEAP32[$yymsp>>2]|0;
   $1743 = ((($1742)) + 4|0);
   $1744 = ((($1743)) + 4|0);
   $1745 = HEAP32[$1744>>2]|0;
   $1746 = HEAP32[$yymsp>>2]|0;
   $1747 = ((($1746)) + 4|0);
   $1748 = HEAP32[$1747>>2]|0;
   $1749 = (($1748) + ($1745)|0);
   $1750 = ((($yygotominor)) + 8|0);
   HEAP32[$1750>>2] = $1749;
   break;
  }
  case 229:  {
   $1751 = HEAP32[$pParse>>2]|0;
   $1752 = HEAP32[$yymsp>>2]|0;
   $1753 = ((($1752)) + -48|0);
   $1754 = ((($1753)) + 4|0);
   $1755 = HEAP32[$1754>>2]|0;
   $1756 = (_sqlite3PExpr($1751,136,$1755,0,0)|0);
   HEAP32[$yygotominor>>2] = $1756;
   $1757 = HEAP32[$yygotominor>>2]|0;
   $1758 = ($1757|0)!=(0|0);
   if ($1758) {
    $1759 = HEAP32[$yymsp>>2]|0;
    $1760 = ((($1759)) + -16|0);
    $1761 = ((($1760)) + 4|0);
    $1762 = HEAP32[$1761>>2]|0;
    $1763 = ($1762|0)!=(0|0);
    if ($1763) {
     $1764 = HEAP32[$pParse>>2]|0;
     $1765 = HEAP32[$yymsp>>2]|0;
     $1766 = ((($1765)) + -32|0);
     $1767 = ((($1766)) + 4|0);
     $1768 = HEAP32[$1767>>2]|0;
     $1769 = HEAP32[$yymsp>>2]|0;
     $1770 = ((($1769)) + -16|0);
     $1771 = ((($1770)) + 4|0);
     $1772 = HEAP32[$1771>>2]|0;
     $1773 = (_sqlite3ExprListAppend($1764,$1768,$1772)|0);
     $1780 = $1773;
    } else {
     $1774 = HEAP32[$yymsp>>2]|0;
     $1775 = ((($1774)) + -32|0);
     $1776 = ((($1775)) + 4|0);
     $1777 = HEAP32[$1776>>2]|0;
     $1780 = $1777;
    }
    $1778 = HEAP32[$yygotominor>>2]|0;
    $1779 = ((($1778)) + 20|0);
    HEAP32[$1779>>2] = $1780;
    $1781 = HEAP32[$pParse>>2]|0;
    $1782 = HEAP32[$yygotominor>>2]|0;
    _sqlite3ExprSetHeightAndFlags($1781,$1782);
   } else {
    $1783 = HEAP32[$pParse>>2]|0;
    $1784 = HEAP32[$1783>>2]|0;
    $1785 = HEAP32[$yymsp>>2]|0;
    $1786 = ((($1785)) + -32|0);
    $1787 = ((($1786)) + 4|0);
    $1788 = HEAP32[$1787>>2]|0;
    _sqlite3ExprListDelete($1784,$1788);
    $1789 = HEAP32[$pParse>>2]|0;
    $1790 = HEAP32[$1789>>2]|0;
    $1791 = HEAP32[$yymsp>>2]|0;
    $1792 = ((($1791)) + -16|0);
    $1793 = ((($1792)) + 4|0);
    $1794 = HEAP32[$1793>>2]|0;
    _sqlite3ExprDelete($1790,$1794);
   }
   $1795 = HEAP32[$yymsp>>2]|0;
   $1796 = ((($1795)) + -64|0);
   $1797 = ((($1796)) + 4|0);
   $1798 = HEAP32[$1797>>2]|0;
   $1799 = ((($yygotominor)) + 4|0);
   HEAP32[$1799>>2] = $1798;
   $1800 = HEAP32[$yymsp>>2]|0;
   $1801 = ((($1800)) + 4|0);
   $1802 = ((($1801)) + 4|0);
   $1803 = HEAP32[$1802>>2]|0;
   $1804 = HEAP32[$yymsp>>2]|0;
   $1805 = ((($1804)) + 4|0);
   $1806 = HEAP32[$1805>>2]|0;
   $1807 = (($1806) + ($1803)|0);
   $1808 = ((($yygotominor)) + 8|0);
   HEAP32[$1808>>2] = $1807;
   break;
  }
  case 230:  {
   $1809 = HEAP32[$pParse>>2]|0;
   $1810 = HEAP32[$yymsp>>2]|0;
   $1811 = ((($1810)) + -64|0);
   $1812 = ((($1811)) + 4|0);
   $1813 = HEAP32[$1812>>2]|0;
   $1814 = HEAP32[$yymsp>>2]|0;
   $1815 = ((($1814)) + -32|0);
   $1816 = ((($1815)) + 4|0);
   $1817 = HEAP32[$1816>>2]|0;
   $1818 = (_sqlite3ExprListAppend($1809,$1813,$1817)|0);
   HEAP32[$yygotominor>>2] = $1818;
   $1819 = HEAP32[$pParse>>2]|0;
   $1820 = HEAP32[$yygotominor>>2]|0;
   $1821 = HEAP32[$yymsp>>2]|0;
   $1822 = ((($1821)) + 4|0);
   $1823 = HEAP32[$1822>>2]|0;
   $1824 = (_sqlite3ExprListAppend($1819,$1820,$1823)|0);
   HEAP32[$yygotominor>>2] = $1824;
   break;
  }
  case 231:  {
   $1825 = HEAP32[$pParse>>2]|0;
   $1826 = HEAP32[$yymsp>>2]|0;
   $1827 = ((($1826)) + -32|0);
   $1828 = ((($1827)) + 4|0);
   $1829 = HEAP32[$1828>>2]|0;
   $1830 = (_sqlite3ExprListAppend($1825,0,$1829)|0);
   HEAP32[$yygotominor>>2] = $1830;
   $1831 = HEAP32[$pParse>>2]|0;
   $1832 = HEAP32[$yygotominor>>2]|0;
   $1833 = HEAP32[$yymsp>>2]|0;
   $1834 = ((($1833)) + 4|0);
   $1835 = HEAP32[$1834>>2]|0;
   $1836 = (_sqlite3ExprListAppend($1831,$1832,$1835)|0);
   HEAP32[$yygotominor>>2] = $1836;
   break;
  }
  case 238:  {
   $1837 = HEAP32[$pParse>>2]|0;
   $1838 = HEAP32[$yymsp>>2]|0;
   $1839 = ((($1838)) + -32|0);
   $1840 = ((($1839)) + 4|0);
   $1841 = HEAP32[$1840>>2]|0;
   $1842 = HEAP32[$yymsp>>2]|0;
   $1843 = ((($1842)) + 4|0);
   $1844 = HEAP32[$1843>>2]|0;
   $1845 = (_sqlite3ExprListAppend($1837,$1841,$1844)|0);
   HEAP32[$yygotominor>>2] = $1845;
   break;
  }
  case 239:  {
   $1846 = HEAP32[$pParse>>2]|0;
   $1847 = HEAP32[$yymsp>>2]|0;
   $1848 = ((($1847)) + 4|0);
   $1849 = HEAP32[$1848>>2]|0;
   $1850 = (_sqlite3ExprListAppend($1846,0,$1849)|0);
   HEAP32[$yygotominor>>2] = $1850;
   break;
  }
  case 240:  {
   $1851 = HEAP32[$pParse>>2]|0;
   $1852 = HEAP32[$yymsp>>2]|0;
   $1853 = ((($1852)) + -112|0);
   $1854 = ((($1853)) + 4|0);
   $1855 = HEAP32[$yymsp>>2]|0;
   $1856 = ((($1855)) + -96|0);
   $1857 = ((($1856)) + 4|0);
   $1858 = HEAP32[$pParse>>2]|0;
   $1859 = HEAP32[$1858>>2]|0;
   $1860 = HEAP32[$yymsp>>2]|0;
   $1861 = ((($1860)) + -64|0);
   $1862 = ((($1861)) + 4|0);
   $1863 = (_sqlite3SrcListAppend($1859,0,$1862,0)|0);
   $1864 = HEAP32[$yymsp>>2]|0;
   $1865 = ((($1864)) + -32|0);
   $1866 = ((($1865)) + 4|0);
   $1867 = HEAP32[$1866>>2]|0;
   $1868 = HEAP32[$yymsp>>2]|0;
   $1869 = ((($1868)) + -160|0);
   $1870 = ((($1869)) + 4|0);
   $1871 = HEAP32[$1870>>2]|0;
   $1872 = HEAP32[$yymsp>>2]|0;
   $1873 = ((($1872)) + -176|0);
   $1874 = ((($1873)) + 4|0);
   $1875 = HEAP32[$yymsp>>2]|0;
   $1876 = ((($1875)) + 4|0);
   $1877 = HEAP32[$1876>>2]|0;
   $1878 = HEAP32[$yymsp>>2]|0;
   $1879 = ((($1878)) + -128|0);
   $1880 = ((($1879)) + 4|0);
   $1881 = HEAP32[$1880>>2]|0;
   (_sqlite3CreateIndex($1851,$1854,$1857,$1863,$1867,$1871,$1874,$1877,0,$1881)|0);
   break;
  }
  case 292: case 241:  {
   HEAP32[$yygotominor>>2] = 2;
   break;
  }
  case 242:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 245:  {
   $1882 = HEAP32[$pParse>>2]|0;
   $1883 = HEAP32[$yymsp>>2]|0;
   $1884 = ((($1883)) + -64|0);
   $1885 = ((($1884)) + 4|0);
   $1886 = HEAP32[$1885>>2]|0;
   $1887 = HEAP32[$yymsp>>2]|0;
   $1888 = ((($1887)) + -32|0);
   $1889 = ((($1888)) + 4|0);
   $1890 = HEAP32[$yymsp>>2]|0;
   $1891 = ((($1890)) + -16|0);
   $1892 = ((($1891)) + 4|0);
   $1893 = HEAP32[$1892>>2]|0;
   $1894 = HEAP32[$yymsp>>2]|0;
   $1895 = ((($1894)) + 4|0);
   $1896 = HEAP32[$1895>>2]|0;
   $1897 = (_parserAddExprIdListTerm($1882,$1886,$1889,$1893,$1896)|0);
   HEAP32[$yygotominor>>2] = $1897;
   break;
  }
  case 246:  {
   $1898 = HEAP32[$pParse>>2]|0;
   $1899 = HEAP32[$yymsp>>2]|0;
   $1900 = ((($1899)) + -32|0);
   $1901 = ((($1900)) + 4|0);
   $1902 = HEAP32[$yymsp>>2]|0;
   $1903 = ((($1902)) + -16|0);
   $1904 = ((($1903)) + 4|0);
   $1905 = HEAP32[$1904>>2]|0;
   $1906 = HEAP32[$yymsp>>2]|0;
   $1907 = ((($1906)) + 4|0);
   $1908 = HEAP32[$1907>>2]|0;
   $1909 = (_parserAddExprIdListTerm($1898,0,$1901,$1905,$1908)|0);
   HEAP32[$yygotominor>>2] = $1909;
   break;
  }
  case 249:  {
   $1910 = HEAP32[$pParse>>2]|0;
   $1911 = HEAP32[$yymsp>>2]|0;
   $1912 = ((($1911)) + 4|0);
   $1913 = HEAP32[$1912>>2]|0;
   $1914 = HEAP32[$yymsp>>2]|0;
   $1915 = ((($1914)) + -16|0);
   $1916 = ((($1915)) + 4|0);
   $1917 = HEAP32[$1916>>2]|0;
   _sqlite3DropIndex($1910,$1913,$1917);
   break;
  }
  case 251: case 250:  {
   $1918 = HEAP32[$pParse>>2]|0;
   _sqlite3Vacuum($1918);
   break;
  }
  case 252:  {
   $1919 = HEAP32[$pParse>>2]|0;
   $1920 = HEAP32[$yymsp>>2]|0;
   $1921 = ((($1920)) + -16|0);
   $1922 = ((($1921)) + 4|0);
   $1923 = HEAP32[$yymsp>>2]|0;
   $1924 = ((($1923)) + 4|0);
   _sqlite3Pragma($1919,$1922,$1924,0,0);
   break;
  }
  case 253:  {
   $1925 = HEAP32[$pParse>>2]|0;
   $1926 = HEAP32[$yymsp>>2]|0;
   $1927 = ((($1926)) + -48|0);
   $1928 = ((($1927)) + 4|0);
   $1929 = HEAP32[$yymsp>>2]|0;
   $1930 = ((($1929)) + -32|0);
   $1931 = ((($1930)) + 4|0);
   $1932 = HEAP32[$yymsp>>2]|0;
   $1933 = ((($1932)) + 4|0);
   _sqlite3Pragma($1925,$1928,$1931,$1933,0);
   break;
  }
  case 254:  {
   $1934 = HEAP32[$pParse>>2]|0;
   $1935 = HEAP32[$yymsp>>2]|0;
   $1936 = ((($1935)) + -64|0);
   $1937 = ((($1936)) + 4|0);
   $1938 = HEAP32[$yymsp>>2]|0;
   $1939 = ((($1938)) + -48|0);
   $1940 = ((($1939)) + 4|0);
   $1941 = HEAP32[$yymsp>>2]|0;
   $1942 = ((($1941)) + -16|0);
   $1943 = ((($1942)) + 4|0);
   _sqlite3Pragma($1934,$1937,$1940,$1943,0);
   break;
  }
  case 255:  {
   $1944 = HEAP32[$pParse>>2]|0;
   $1945 = HEAP32[$yymsp>>2]|0;
   $1946 = ((($1945)) + -48|0);
   $1947 = ((($1946)) + 4|0);
   $1948 = HEAP32[$yymsp>>2]|0;
   $1949 = ((($1948)) + -32|0);
   $1950 = ((($1949)) + 4|0);
   $1951 = HEAP32[$yymsp>>2]|0;
   $1952 = ((($1951)) + 4|0);
   _sqlite3Pragma($1944,$1947,$1950,$1952,1);
   break;
  }
  case 256:  {
   $1953 = HEAP32[$pParse>>2]|0;
   $1954 = HEAP32[$yymsp>>2]|0;
   $1955 = ((($1954)) + -64|0);
   $1956 = ((($1955)) + 4|0);
   $1957 = HEAP32[$yymsp>>2]|0;
   $1958 = ((($1957)) + -48|0);
   $1959 = ((($1958)) + 4|0);
   $1960 = HEAP32[$yymsp>>2]|0;
   $1961 = ((($1960)) + -16|0);
   $1962 = ((($1961)) + 4|0);
   _sqlite3Pragma($1953,$1956,$1959,$1962,1);
   break;
  }
  case 265:  {
   $1963 = HEAP32[$yymsp>>2]|0;
   $1964 = ((($1963)) + -48|0);
   $1965 = ((($1964)) + 4|0);
   $1966 = HEAP32[$1965>>2]|0;
   HEAP32[$all>>2] = $1966;
   $1967 = HEAP32[$yymsp>>2]|0;
   $1968 = ((($1967)) + 4|0);
   $1969 = HEAP32[$1968>>2]|0;
   $1970 = HEAP32[$yymsp>>2]|0;
   $1971 = ((($1970)) + -48|0);
   $1972 = ((($1971)) + 4|0);
   $1973 = HEAP32[$1972>>2]|0;
   $1974 = $1969;
   $1975 = $1973;
   $1976 = (($1974) - ($1975))|0;
   $1977 = HEAP32[$yymsp>>2]|0;
   $1978 = ((($1977)) + 4|0);
   $1979 = ((($1978)) + 4|0);
   $1980 = HEAP32[$1979>>2]|0;
   $1981 = (($1976) + ($1980))|0;
   $1982 = ((($all)) + 4|0);
   HEAP32[$1982>>2] = $1981;
   $1983 = HEAP32[$pParse>>2]|0;
   $1984 = HEAP32[$yymsp>>2]|0;
   $1985 = ((($1984)) + -16|0);
   $1986 = ((($1985)) + 4|0);
   $1987 = HEAP32[$1986>>2]|0;
   _sqlite3FinishTrigger($1983,$1987,$all);
   break;
  }
  case 266:  {
   $1988 = HEAP32[$pParse>>2]|0;
   $1989 = HEAP32[$yymsp>>2]|0;
   $1990 = ((($1989)) + -112|0);
   $1991 = ((($1990)) + 4|0);
   $1992 = HEAP32[$yymsp>>2]|0;
   $1993 = ((($1992)) + -96|0);
   $1994 = ((($1993)) + 4|0);
   $1995 = HEAP32[$yymsp>>2]|0;
   $1996 = ((($1995)) + -80|0);
   $1997 = ((($1996)) + 4|0);
   $1998 = HEAP32[$1997>>2]|0;
   $1999 = HEAP32[$yymsp>>2]|0;
   $2000 = ((($1999)) + -64|0);
   $2001 = ((($2000)) + 4|0);
   $2002 = HEAP32[$2001>>2]|0;
   $2003 = HEAP32[$yymsp>>2]|0;
   $2004 = ((($2003)) + -64|0);
   $2005 = ((($2004)) + 4|0);
   $2006 = ((($2005)) + 4|0);
   $2007 = HEAP32[$2006>>2]|0;
   $2008 = HEAP32[$yymsp>>2]|0;
   $2009 = ((($2008)) + -32|0);
   $2010 = ((($2009)) + 4|0);
   $2011 = HEAP32[$2010>>2]|0;
   $2012 = HEAP32[$yymsp>>2]|0;
   $2013 = ((($2012)) + 4|0);
   $2014 = HEAP32[$2013>>2]|0;
   $2015 = HEAP32[$yymsp>>2]|0;
   $2016 = ((($2015)) + -160|0);
   $2017 = ((($2016)) + 4|0);
   $2018 = HEAP32[$2017>>2]|0;
   $2019 = HEAP32[$yymsp>>2]|0;
   $2020 = ((($2019)) + -128|0);
   $2021 = ((($2020)) + 4|0);
   $2022 = HEAP32[$2021>>2]|0;
   _sqlite3BeginTrigger($1988,$1991,$1994,$1998,$2002,$2007,$2011,$2014,$2018,$2022);
   $2023 = HEAP32[$yymsp>>2]|0;
   $2024 = ((($2023)) + -96|0);
   $2025 = ((($2024)) + 4|0);
   $2026 = ((($2025)) + 4|0);
   $2027 = HEAP32[$2026>>2]|0;
   $2028 = ($2027|0)==(0);
   $2029 = HEAP32[$yymsp>>2]|0;
   if ($2028) {
    $2030 = ((($2029)) + -112|0);
    $2031 = ((($2030)) + 4|0);
    ;HEAP32[$yygotominor>>2]=HEAP32[$2031>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$2031+4>>2]|0;
    break L1;
   } else {
    $2032 = ((($2029)) + -96|0);
    $2033 = ((($2032)) + 4|0);
    ;HEAP32[$yygotominor>>2]=HEAP32[$2033>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$2033+4>>2]|0;
    break L1;
   }
   break;
  }
  case 270: case 267:  {
   HEAP32[$yygotominor>>2] = 35;
   break;
  }
  case 268:  {
   HEAP32[$yygotominor>>2] = 31;
   break;
  }
  case 269:  {
   HEAP32[$yygotominor>>2] = 49;
   break;
  }
  case 272: case 271:  {
   $2034 = HEAP32[$yymsp>>2]|0;
   $2035 = ((($2034)) + 2|0);
   $2036 = HEAP8[$2035>>0]|0;
   $2037 = $2036&255;
   HEAP32[$yygotominor>>2] = $2037;
   $2038 = ((($yygotominor)) + 4|0);
   HEAP32[$2038>>2] = 0;
   break;
  }
  case 273:  {
   HEAP32[$yygotominor>>2] = 110;
   $2039 = HEAP32[$yymsp>>2]|0;
   $2040 = ((($2039)) + 4|0);
   $2041 = HEAP32[$2040>>2]|0;
   $2042 = ((($yygotominor)) + 4|0);
   HEAP32[$2042>>2] = $2041;
   break;
  }
  case 297: case 276:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 298: case 277:  {
   $2043 = HEAP32[$yymsp>>2]|0;
   $2044 = ((($2043)) + 4|0);
   $2045 = HEAP32[$2044>>2]|0;
   HEAP32[$yygotominor>>2] = $2045;
   break;
  }
  case 278:  {
   $2046 = HEAP32[$yymsp>>2]|0;
   $2047 = ((($2046)) + -16|0);
   $2048 = ((($2047)) + 4|0);
   $2049 = HEAP32[$2048>>2]|0;
   $2050 = HEAP32[$yymsp>>2]|0;
   $2051 = ((($2050)) + -32|0);
   $2052 = ((($2051)) + 4|0);
   $2053 = HEAP32[$2052>>2]|0;
   $2054 = ((($2053)) + 32|0);
   $2055 = HEAP32[$2054>>2]|0;
   $2056 = ((($2055)) + 28|0);
   HEAP32[$2056>>2] = $2049;
   $2057 = HEAP32[$yymsp>>2]|0;
   $2058 = ((($2057)) + -16|0);
   $2059 = ((($2058)) + 4|0);
   $2060 = HEAP32[$2059>>2]|0;
   $2061 = HEAP32[$yymsp>>2]|0;
   $2062 = ((($2061)) + -32|0);
   $2063 = ((($2062)) + 4|0);
   $2064 = HEAP32[$2063>>2]|0;
   $2065 = ((($2064)) + 32|0);
   HEAP32[$2065>>2] = $2060;
   $2066 = HEAP32[$yymsp>>2]|0;
   $2067 = ((($2066)) + -32|0);
   $2068 = ((($2067)) + 4|0);
   $2069 = HEAP32[$2068>>2]|0;
   HEAP32[$yygotominor>>2] = $2069;
   break;
  }
  case 279:  {
   $2070 = HEAP32[$yymsp>>2]|0;
   $2071 = ((($2070)) + -16|0);
   $2072 = ((($2071)) + 4|0);
   $2073 = HEAP32[$2072>>2]|0;
   $2074 = HEAP32[$yymsp>>2]|0;
   $2075 = ((($2074)) + -16|0);
   $2076 = ((($2075)) + 4|0);
   $2077 = HEAP32[$2076>>2]|0;
   $2078 = ((($2077)) + 32|0);
   HEAP32[$2078>>2] = $2073;
   $2079 = HEAP32[$yymsp>>2]|0;
   $2080 = ((($2079)) + -16|0);
   $2081 = ((($2080)) + 4|0);
   $2082 = HEAP32[$2081>>2]|0;
   HEAP32[$yygotominor>>2] = $2082;
   break;
  }
  case 281:  {
   $2083 = HEAP32[$yymsp>>2]|0;
   $2084 = ((($2083)) + 4|0);
   ;HEAP32[$yygotominor>>2]=HEAP32[$2084>>2]|0;HEAP32[$yygotominor+4>>2]=HEAP32[$2084+4>>2]|0;
   $2085 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2085,27437,$vararg_buffer8);
   break;
  }
  case 283:  {
   $2086 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2086,27532,$vararg_buffer10);
   break;
  }
  case 284:  {
   $2087 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2087,27616,$vararg_buffer12);
   break;
  }
  case 285:  {
   $2088 = HEAP32[$pParse>>2]|0;
   $2089 = HEAP32[$2088>>2]|0;
   $2090 = HEAP32[$yymsp>>2]|0;
   $2091 = ((($2090)) + -64|0);
   $2092 = ((($2091)) + 4|0);
   $2093 = HEAP32[$yymsp>>2]|0;
   $2094 = ((($2093)) + -16|0);
   $2095 = ((($2094)) + 4|0);
   $2096 = HEAP32[$2095>>2]|0;
   $2097 = HEAP32[$yymsp>>2]|0;
   $2098 = ((($2097)) + 4|0);
   $2099 = HEAP32[$2098>>2]|0;
   $2100 = HEAP32[$yymsp>>2]|0;
   $2101 = ((($2100)) + -80|0);
   $2102 = ((($2101)) + 4|0);
   $2103 = HEAP32[$2102>>2]|0;
   $2104 = $2103&255;
   $2105 = (_sqlite3TriggerUpdateStep($2089,$2092,$2096,$2099,$2104)|0);
   HEAP32[$yygotominor>>2] = $2105;
   break;
  }
  case 286:  {
   $2106 = HEAP32[$pParse>>2]|0;
   $2107 = HEAP32[$2106>>2]|0;
   $2108 = HEAP32[$yymsp>>2]|0;
   $2109 = ((($2108)) + -32|0);
   $2110 = ((($2109)) + 4|0);
   $2111 = HEAP32[$yymsp>>2]|0;
   $2112 = ((($2111)) + -16|0);
   $2113 = ((($2112)) + 4|0);
   $2114 = HEAP32[$2113>>2]|0;
   $2115 = HEAP32[$yymsp>>2]|0;
   $2116 = ((($2115)) + 4|0);
   $2117 = HEAP32[$2116>>2]|0;
   $2118 = HEAP32[$yymsp>>2]|0;
   $2119 = ((($2118)) + -64|0);
   $2120 = ((($2119)) + 4|0);
   $2121 = HEAP32[$2120>>2]|0;
   $2122 = $2121&255;
   $2123 = (_sqlite3TriggerInsertStep($2107,$2110,$2114,$2117,$2122)|0);
   HEAP32[$yygotominor>>2] = $2123;
   break;
  }
  case 287:  {
   $2124 = HEAP32[$pParse>>2]|0;
   $2125 = HEAP32[$2124>>2]|0;
   $2126 = HEAP32[$yymsp>>2]|0;
   $2127 = ((($2126)) + -32|0);
   $2128 = ((($2127)) + 4|0);
   $2129 = HEAP32[$yymsp>>2]|0;
   $2130 = ((($2129)) + 4|0);
   $2131 = HEAP32[$2130>>2]|0;
   $2132 = (_sqlite3TriggerDeleteStep($2125,$2128,$2131)|0);
   HEAP32[$yygotominor>>2] = $2132;
   break;
  }
  case 288:  {
   $2133 = HEAP32[$pParse>>2]|0;
   $2134 = HEAP32[$2133>>2]|0;
   $2135 = HEAP32[$yymsp>>2]|0;
   $2136 = ((($2135)) + 4|0);
   $2137 = HEAP32[$2136>>2]|0;
   $2138 = (_sqlite3TriggerSelectStep($2134,$2137)|0);
   HEAP32[$yygotominor>>2] = $2138;
   break;
  }
  case 289:  {
   $2139 = HEAP32[$pParse>>2]|0;
   $2140 = (_sqlite3PExpr($2139,57,0,0,0)|0);
   HEAP32[$yygotominor>>2] = $2140;
   $2141 = HEAP32[$yygotominor>>2]|0;
   $2142 = ($2141|0)!=(0|0);
   if ($2142) {
    $2143 = HEAP32[$yygotominor>>2]|0;
    $2144 = ((($2143)) + 1|0);
    HEAP8[$2144>>0] = 4;
   }
   $2145 = HEAP32[$yymsp>>2]|0;
   $2146 = ((($2145)) + -48|0);
   $2147 = ((($2146)) + 4|0);
   $2148 = HEAP32[$2147>>2]|0;
   $2149 = ((($yygotominor)) + 4|0);
   HEAP32[$2149>>2] = $2148;
   $2150 = HEAP32[$yymsp>>2]|0;
   $2151 = ((($2150)) + 4|0);
   $2152 = ((($2151)) + 4|0);
   $2153 = HEAP32[$2152>>2]|0;
   $2154 = HEAP32[$yymsp>>2]|0;
   $2155 = ((($2154)) + 4|0);
   $2156 = HEAP32[$2155>>2]|0;
   $2157 = (($2156) + ($2153)|0);
   $2158 = ((($yygotominor)) + 8|0);
   HEAP32[$2158>>2] = $2157;
   break;
  }
  case 290:  {
   $2159 = HEAP32[$pParse>>2]|0;
   $2160 = HEAP32[$yymsp>>2]|0;
   $2161 = ((($2160)) + -16|0);
   $2162 = ((($2161)) + 4|0);
   $2163 = (_sqlite3PExpr($2159,57,0,0,$2162)|0);
   HEAP32[$yygotominor>>2] = $2163;
   $2164 = HEAP32[$yygotominor>>2]|0;
   $2165 = ($2164|0)!=(0|0);
   if ($2165) {
    $2166 = HEAP32[$yymsp>>2]|0;
    $2167 = ((($2166)) + -48|0);
    $2168 = ((($2167)) + 4|0);
    $2169 = HEAP32[$2168>>2]|0;
    $2170 = $2169&255;
    $2171 = HEAP32[$yygotominor>>2]|0;
    $2172 = ((($2171)) + 1|0);
    HEAP8[$2172>>0] = $2170;
   }
   $2173 = HEAP32[$yymsp>>2]|0;
   $2174 = ((($2173)) + -80|0);
   $2175 = ((($2174)) + 4|0);
   $2176 = HEAP32[$2175>>2]|0;
   $2177 = ((($yygotominor)) + 4|0);
   HEAP32[$2177>>2] = $2176;
   $2178 = HEAP32[$yymsp>>2]|0;
   $2179 = ((($2178)) + 4|0);
   $2180 = ((($2179)) + 4|0);
   $2181 = HEAP32[$2180>>2]|0;
   $2182 = HEAP32[$yymsp>>2]|0;
   $2183 = ((($2182)) + 4|0);
   $2184 = HEAP32[$2183>>2]|0;
   $2185 = (($2184) + ($2181)|0);
   $2186 = ((($yygotominor)) + 8|0);
   HEAP32[$2186>>2] = $2185;
   break;
  }
  case 291:  {
   HEAP32[$yygotominor>>2] = 1;
   break;
  }
  case 293:  {
   HEAP32[$yygotominor>>2] = 3;
   break;
  }
  case 294:  {
   $2187 = HEAP32[$pParse>>2]|0;
   $2188 = HEAP32[$yymsp>>2]|0;
   $2189 = ((($2188)) + 4|0);
   $2190 = HEAP32[$2189>>2]|0;
   $2191 = HEAP32[$yymsp>>2]|0;
   $2192 = ((($2191)) + -16|0);
   $2193 = ((($2192)) + 4|0);
   $2194 = HEAP32[$2193>>2]|0;
   _sqlite3DropTrigger($2187,$2190,$2194);
   break;
  }
  case 295:  {
   $2195 = HEAP32[$pParse>>2]|0;
   $2196 = HEAP32[$yymsp>>2]|0;
   $2197 = ((($2196)) + -48|0);
   $2198 = ((($2197)) + 4|0);
   $2199 = HEAP32[$2198>>2]|0;
   $2200 = HEAP32[$yymsp>>2]|0;
   $2201 = ((($2200)) + -16|0);
   $2202 = ((($2201)) + 4|0);
   $2203 = HEAP32[$2202>>2]|0;
   $2204 = HEAP32[$yymsp>>2]|0;
   $2205 = ((($2204)) + 4|0);
   $2206 = HEAP32[$2205>>2]|0;
   _sqlite3Attach($2195,$2199,$2203,$2206);
   break;
  }
  case 296:  {
   $2207 = HEAP32[$pParse>>2]|0;
   $2208 = HEAP32[$yymsp>>2]|0;
   $2209 = ((($2208)) + 4|0);
   $2210 = HEAP32[$2209>>2]|0;
   _sqlite3Detach($2207,$2210);
   break;
  }
  case 301:  {
   $2211 = HEAP32[$pParse>>2]|0;
   _sqlite3Reindex($2211,0,0);
   break;
  }
  case 302:  {
   $2212 = HEAP32[$pParse>>2]|0;
   $2213 = HEAP32[$yymsp>>2]|0;
   $2214 = ((($2213)) + -16|0);
   $2215 = ((($2214)) + 4|0);
   $2216 = HEAP32[$yymsp>>2]|0;
   $2217 = ((($2216)) + 4|0);
   _sqlite3Reindex($2212,$2215,$2217);
   break;
  }
  case 303:  {
   $2218 = HEAP32[$pParse>>2]|0;
   _sqlite3Analyze($2218,0,0);
   break;
  }
  case 304:  {
   $2219 = HEAP32[$pParse>>2]|0;
   $2220 = HEAP32[$yymsp>>2]|0;
   $2221 = ((($2220)) + -16|0);
   $2222 = ((($2221)) + 4|0);
   $2223 = HEAP32[$yymsp>>2]|0;
   $2224 = ((($2223)) + 4|0);
   _sqlite3Analyze($2219,$2222,$2224);
   break;
  }
  case 305:  {
   $2225 = HEAP32[$pParse>>2]|0;
   $2226 = HEAP32[$yymsp>>2]|0;
   $2227 = ((($2226)) + -48|0);
   $2228 = ((($2227)) + 4|0);
   $2229 = HEAP32[$2228>>2]|0;
   $2230 = HEAP32[$yymsp>>2]|0;
   $2231 = ((($2230)) + 4|0);
   _sqlite3AlterRenameTable($2225,$2229,$2231);
   break;
  }
  case 306:  {
   $2232 = HEAP32[$pParse>>2]|0;
   $2233 = HEAP32[$yymsp>>2]|0;
   $2234 = ((($2233)) + 4|0);
   _sqlite3AlterFinishAddColumn($2232,$2234);
   break;
  }
  case 307:  {
   $2235 = HEAP32[$pParse>>2]|0;
   _disableLookaside($2235);
   $2236 = HEAP32[$pParse>>2]|0;
   $2237 = HEAP32[$yymsp>>2]|0;
   $2238 = ((($2237)) + 4|0);
   $2239 = HEAP32[$2238>>2]|0;
   _sqlite3AlterBeginAddColumn($2236,$2239);
   break;
  }
  case 310:  {
   $2240 = HEAP32[$pParse>>2]|0;
   _sqlite3VtabFinishParse($2240,0);
   break;
  }
  case 311:  {
   $2241 = HEAP32[$pParse>>2]|0;
   $2242 = HEAP32[$yymsp>>2]|0;
   $2243 = ((($2242)) + 4|0);
   _sqlite3VtabFinishParse($2241,$2243);
   break;
  }
  case 312:  {
   $2244 = HEAP32[$pParse>>2]|0;
   $2245 = HEAP32[$yymsp>>2]|0;
   $2246 = ((($2245)) + -48|0);
   $2247 = ((($2246)) + 4|0);
   $2248 = HEAP32[$yymsp>>2]|0;
   $2249 = ((($2248)) + -32|0);
   $2250 = ((($2249)) + 4|0);
   $2251 = HEAP32[$yymsp>>2]|0;
   $2252 = ((($2251)) + 4|0);
   $2253 = HEAP32[$yymsp>>2]|0;
   $2254 = ((($2253)) + -64|0);
   $2255 = ((($2254)) + 4|0);
   $2256 = HEAP32[$2255>>2]|0;
   _sqlite3VtabBeginParse($2244,$2247,$2250,$2252,$2256);
   break;
  }
  case 315:  {
   $2257 = HEAP32[$pParse>>2]|0;
   _sqlite3VtabArgInit($2257);
   break;
  }
  case 319: case 318: case 317:  {
   $2258 = HEAP32[$pParse>>2]|0;
   $2259 = HEAP32[$yymsp>>2]|0;
   $2260 = ((($2259)) + 4|0);
   _sqlite3VtabArgExtend($2258,$2260);
   break;
  }
  case 323:  {
   HEAP32[$yygotominor>>2] = 0;
   break;
  }
  case 325: case 324:  {
   $2261 = HEAP32[$yymsp>>2]|0;
   $2262 = ((($2261)) + 4|0);
   $2263 = HEAP32[$2262>>2]|0;
   HEAP32[$yygotominor>>2] = $2263;
   break;
  }
  case 326:  {
   $2264 = HEAP32[$pParse>>2]|0;
   $2265 = HEAP32[$yymsp>>2]|0;
   $2266 = ((($2265)) + -80|0);
   $2267 = ((($2266)) + 4|0);
   $2268 = HEAP32[$yymsp>>2]|0;
   $2269 = ((($2268)) + -64|0);
   $2270 = ((($2269)) + 4|0);
   $2271 = HEAP32[$2270>>2]|0;
   $2272 = HEAP32[$yymsp>>2]|0;
   $2273 = ((($2272)) + -16|0);
   $2274 = ((($2273)) + 4|0);
   $2275 = HEAP32[$2274>>2]|0;
   $2276 = (_sqlite3WithAdd($2264,0,$2267,$2271,$2275)|0);
   HEAP32[$yygotominor>>2] = $2276;
   break;
  }
  case 327:  {
   $2277 = HEAP32[$pParse>>2]|0;
   $2278 = HEAP32[$yymsp>>2]|0;
   $2279 = ((($2278)) + -112|0);
   $2280 = ((($2279)) + 4|0);
   $2281 = HEAP32[$2280>>2]|0;
   $2282 = HEAP32[$yymsp>>2]|0;
   $2283 = ((($2282)) + -80|0);
   $2284 = ((($2283)) + 4|0);
   $2285 = HEAP32[$yymsp>>2]|0;
   $2286 = ((($2285)) + -64|0);
   $2287 = ((($2286)) + 4|0);
   $2288 = HEAP32[$2287>>2]|0;
   $2289 = HEAP32[$yymsp>>2]|0;
   $2290 = ((($2289)) + -16|0);
   $2291 = ((($2290)) + 4|0);
   $2292 = HEAP32[$2291>>2]|0;
   $2293 = (_sqlite3WithAdd($2277,$2281,$2284,$2288,$2292)|0);
   HEAP32[$yygotominor>>2] = $2293;
   break;
  }
  default: {
  }
  }
 } while(0);
 $2294 = HEAP32[$1>>2]|0;
 $2295 = (27701 + ($2294<<1)|0);
 $2296 = HEAP8[$2295>>0]|0;
 $2297 = $2296&255;
 HEAP32[$yygoto>>2] = $2297;
 $2298 = HEAP32[$1>>2]|0;
 $2299 = (27701 + ($2298<<1)|0);
 $2300 = ((($2299)) + 1|0);
 $2301 = HEAP8[$2300>>0]|0;
 $2302 = $2301&255;
 HEAP32[$yysize>>2] = $2302;
 $2303 = HEAP32[$yysize>>2]|0;
 $2304 = HEAP32[$0>>2]|0;
 $2305 = HEAP32[$2304>>2]|0;
 $2306 = (($2305) - ($2303))|0;
 HEAP32[$2304>>2] = $2306;
 $2307 = HEAP32[$yysize>>2]|0;
 $2308 = (0 - ($2307))|0;
 $2309 = HEAP32[$yymsp>>2]|0;
 $2310 = (($2309) + ($2308<<4)|0);
 $2311 = HEAP16[$2310>>1]|0;
 $2312 = $2311&65535;
 $2313 = HEAP32[$yygoto>>2]|0;
 $2314 = $2313&255;
 $2315 = (_yy_find_reduce_action($2312,$2314)|0);
 HEAP32[$yyact>>2] = $2315;
 $2316 = HEAP32[$yyact>>2]|0;
 $2317 = ($2316|0)<=(976);
 if (!($2317)) {
  $2343 = HEAP32[$0>>2]|0;
  _yy_accept($2343);
  STACKTOP = sp;return;
 }
 $2318 = HEAP32[$yyact>>2]|0;
 $2319 = ($2318|0)>(435);
 if ($2319) {
  $2320 = HEAP32[$yyact>>2]|0;
  $2321 = (($2320) + 328)|0;
  HEAP32[$yyact>>2] = $2321;
 }
 $2322 = HEAP32[$yysize>>2]|0;
 $2323 = ($2322|0)!=(0);
 $2324 = HEAP32[$0>>2]|0;
 if ($2323) {
  $2325 = HEAP32[$2324>>2]|0;
  $2326 = (($2325) + 1)|0;
  HEAP32[$2324>>2] = $2326;
  $2327 = HEAP32[$yysize>>2]|0;
  $2328 = (($2327) - 1)|0;
  $2329 = HEAP32[$yymsp>>2]|0;
  $2330 = (0 - ($2328))|0;
  $2331 = (($2329) + ($2330<<4)|0);
  HEAP32[$yymsp>>2] = $2331;
  $2332 = HEAP32[$yyact>>2]|0;
  $2333 = $2332&65535;
  $2334 = HEAP32[$yymsp>>2]|0;
  HEAP16[$2334>>1] = $2333;
  $2335 = HEAP32[$yygoto>>2]|0;
  $2336 = $2335&255;
  $2337 = HEAP32[$yymsp>>2]|0;
  $2338 = ((($2337)) + 2|0);
  HEAP8[$2338>>0] = $2336;
  $2339 = HEAP32[$yymsp>>2]|0;
  $2340 = ((($2339)) + 4|0);
  ;HEAP32[$2340>>2]=HEAP32[$yygotominor>>2]|0;HEAP32[$2340+4>>2]=HEAP32[$yygotominor+4>>2]|0;HEAP32[$2340+8>>2]=HEAP32[$yygotominor+8>>2]|0;
  STACKTOP = sp;return;
 } else {
  $2341 = HEAP32[$yyact>>2]|0;
  $2342 = HEAP32[$yygoto>>2]|0;
  _yy_shift($2324,$2341,$2342,$yygotominor);
  STACKTOP = sp;return;
 }
}
function _yy_syntax_error($yypParser,$yymajor,$yyminor) {
 $yypParser = $yypParser|0;
 $yymajor = $yymajor|0;
 $yyminor = $yyminor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pParse = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yymajor;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 HEAP32[$vararg_buffer>>2] = $yyminor;
 _sqlite3ErrorMsg($5,27379,$vararg_buffer);
 $6 = HEAP32[$pParse>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 HEAP32[$8>>2] = $6;
 STACKTOP = sp;return;
}
function _yy_destructor($yypParser,$yymajor,$yypminor) {
 $yypParser = $yypParser|0;
 $yymajor = $yymajor|0;
 $yypminor = $yypminor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 12|0;
 $2 = sp + 4|0;
 $pParse = sp;
 HEAP32[$0>>2] = $yypParser;
 HEAP8[$1>>0] = $yymajor;
 HEAP32[$2>>2] = $yypminor;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP8[$1>>0]|0;
 $7 = $6&255;
 do {
  switch ($7|0) {
  case 208: case 197: case 196: case 163:  {
   $8 = HEAP32[$pParse>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   _sqlite3SelectDelete($9,$11);
   STACKTOP = sp;return;
   break;
  }
  case 175: case 174:  {
   $12 = HEAP32[$pParse>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   _sqlite3ExprDelete($13,$15);
   STACKTOP = sp;return;
   break;
  }
  case 227: case 220: case 211: case 210: case 209: case 206: case 204: case 201: case 189: case 188: case 179:  {
   $16 = HEAP32[$pParse>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   _sqlite3ExprListDelete($17,$19);
   STACKTOP = sp;return;
   break;
  }
  case 214: case 213: case 202: case 195:  {
   $20 = HEAP32[$pParse>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   _sqlite3SrcListDelete($21,$23);
   STACKTOP = sp;return;
   break;
  }
  case 251: case 198:  {
   $24 = HEAP32[$pParse>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   _sqlite3WithDelete($25,$27);
   STACKTOP = sp;return;
   break;
  }
  case 242: case 237: case 228: case 226: case 217: case 205: case 203:  {
   $28 = HEAP32[$pParse>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$2>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   _sqlite3ExprDelete($29,$31);
   STACKTOP = sp;return;
   break;
  }
  case 222: case 219: case 218:  {
   $32 = HEAP32[$pParse>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   _sqlite3IdListDelete($33,$35);
   STACKTOP = sp;return;
   break;
  }
  case 238: case 233:  {
   $36 = HEAP32[$pParse>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   _sqlite3DeleteTriggerStep($37,$39);
   STACKTOP = sp;return;
   break;
  }
  case 235:  {
   $40 = HEAP32[$pParse>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$2>>2]|0;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   _sqlite3IdListDelete($41,$44);
   STACKTOP = sp;return;
   break;
  }
  default: {
   STACKTOP = sp;return;
  }
  }
 } while(0);
}
function _yyStackOverflow($yypParser,$yypMinor) {
 $yypParser = $yypParser|0;
 $yypMinor = $yypMinor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yypMinor;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + -1)|0;
 HEAP32[$5>>2] = $7;
 while(1) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$0>>2]|0;
  _yy_pop_parser_stack($11);
 }
 $12 = HEAP32[$pParse>>2]|0;
 _sqlite3ErrorMsg($12,27315,$vararg_buffer);
 $13 = HEAP32[$pParse>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 8|0);
 HEAP32[$15>>2] = $13;
 STACKTOP = sp;return;
}
function _yy_pop_parser_stack($pParser) {
 $pParser = $pParser|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yytos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $yytos = sp;
 HEAP32[$0>>2] = $pParser;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + -1)|0;
 HEAP32[$1>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = (($5) + ($2<<4)|0);
 HEAP32[$yytos>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$yytos>>2]|0;
 $9 = ((($8)) + 2|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = HEAP32[$yytos>>2]|0;
 $12 = ((($11)) + 4|0);
 _yy_destructor($7,$10,$12);
 STACKTOP = sp;return;
}
function _sqlite3FinishCoding($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $pEL = 0, $v = 0, $vtab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $db = sp + 20|0;
 $v = sp + 16|0;
 $iDb = sp + 12|0;
 $i = sp + 8|0;
 $vtab = sp + 4|0;
 $pEL = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 18|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 65|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 68|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0);
  if (!($15)) {
   $22 = HEAP32[$0>>2]|0;
   $23 = (_sqlite3GetVdbe($22)|0);
   HEAP32[$v>>2] = $23;
   $24 = HEAP32[$v>>2]|0;
   $25 = ($24|0)!=(0|0);
   do {
    if ($25) {
     while(1) {
      $26 = HEAP32[$v>>2]|0;
      $27 = (_sqlite3VdbeDeletePriorOpcode($26,61)|0);
      $28 = ($27|0)!=(0);
      if (!($28)) {
       break;
      }
     }
     $29 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($29,21)|0);
     $30 = HEAP32[$db>>2]|0;
     $31 = ((($30)) + 65|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = HEAP32[$0>>2]|0;
      $36 = ((($35)) + 340|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = ($37|0)!=(0);
      if (!($38)) {
       $39 = HEAP32[$0>>2]|0;
       $40 = ((($39)) + 324|0);
       $41 = HEAP32[$40>>2]|0;
       $42 = ($41|0)!=(0|0);
       if (!($42)) {
        break;
       }
      }
      $43 = HEAP32[$v>>2]|0;
      _sqlite3VdbeJumpHere($43,0);
      HEAP32[$iDb>>2] = 0;
      while(1) {
       $44 = HEAP32[$iDb>>2]|0;
       $45 = HEAP32[$db>>2]|0;
       $46 = ((($45)) + 20|0);
       $47 = HEAP32[$46>>2]|0;
       $48 = ($44|0)<($47|0);
       if (!($48)) {
        break;
       }
       $49 = HEAP32[$0>>2]|0;
       $50 = ((($49)) + 340|0);
       $51 = HEAP32[$50>>2]|0;
       $52 = HEAP32[$iDb>>2]|0;
       $53 = 1 << $52;
       $54 = $51 & $53;
       $55 = ($54|0)!=(0);
       $56 = $55&1;
       $57 = ($56|0)==(0);
       if (!($57)) {
        $58 = HEAP32[$v>>2]|0;
        $59 = HEAP32[$iDb>>2]|0;
        _sqlite3VdbeUsesBtree($58,$59);
        $60 = HEAP32[$v>>2]|0;
        $61 = HEAP32[$iDb>>2]|0;
        $62 = HEAP32[$0>>2]|0;
        $63 = ((($62)) + 336|0);
        $64 = HEAP32[$63>>2]|0;
        $65 = HEAP32[$iDb>>2]|0;
        $66 = 1 << $65;
        $67 = $64 & $66;
        $68 = ($67|0)!=(0);
        $69 = $68&1;
        $70 = HEAP32[$iDb>>2]|0;
        $71 = HEAP32[$0>>2]|0;
        $72 = ((($71)) + 344|0);
        $73 = (($72) + ($70<<2)|0);
        $74 = HEAP32[$73>>2]|0;
        $75 = HEAP32[$iDb>>2]|0;
        $76 = HEAP32[$db>>2]|0;
        $77 = ((($76)) + 16|0);
        $78 = HEAP32[$77>>2]|0;
        $79 = (($78) + ($75<<4)|0);
        $80 = ((($79)) + 12|0);
        $81 = HEAP32[$80>>2]|0;
        $82 = ((($81)) + 4|0);
        $83 = HEAP32[$82>>2]|0;
        (_sqlite3VdbeAddOp4Int($60,2,$61,$69,$74,$83)|0);
        $84 = HEAP32[$db>>2]|0;
        $85 = ((($84)) + 140|0);
        $86 = ((($85)) + 5|0);
        $87 = HEAP8[$86>>0]|0;
        $88 = $87&255;
        $89 = ($88|0)==(0);
        if ($89) {
         $90 = HEAP32[$v>>2]|0;
         _sqlite3VdbeChangeP5($90,1);
        }
       }
       $91 = HEAP32[$iDb>>2]|0;
       $92 = (($91) + 1)|0;
       HEAP32[$iDb>>2] = $92;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $93 = HEAP32[$i>>2]|0;
       $94 = HEAP32[$0>>2]|0;
       $95 = ((($94)) + 456|0);
       $96 = HEAP32[$95>>2]|0;
       $97 = ($93|0)<($96|0);
       if (!($97)) {
        break;
       }
       $98 = HEAP32[$db>>2]|0;
       $99 = HEAP32[$i>>2]|0;
       $100 = HEAP32[$0>>2]|0;
       $101 = ((($100)) + 524|0);
       $102 = HEAP32[$101>>2]|0;
       $103 = (($102) + ($99<<2)|0);
       $104 = HEAP32[$103>>2]|0;
       $105 = (_sqlite3GetVTable($98,$104)|0);
       HEAP32[$vtab>>2] = $105;
       $106 = HEAP32[$v>>2]|0;
       $107 = HEAP32[$vtab>>2]|0;
       (_sqlite3VdbeAddOp4($106,149,0,0,0,$107,-10)|0);
       $108 = HEAP32[$i>>2]|0;
       $109 = (($108) + 1)|0;
       HEAP32[$i>>2] = $109;
      }
      $110 = HEAP32[$0>>2]|0;
      $111 = ((($110)) + 456|0);
      HEAP32[$111>>2] = 0;
      $112 = HEAP32[$0>>2]|0;
      _codeTableLocks($112);
      $113 = HEAP32[$0>>2]|0;
      _sqlite3AutoincrementBegin($113);
      $114 = HEAP32[$0>>2]|0;
      $115 = ((($114)) + 324|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($116|0)!=(0|0);
      L27: do {
       if ($117) {
        $118 = HEAP32[$0>>2]|0;
        $119 = ((($118)) + 324|0);
        $120 = HEAP32[$119>>2]|0;
        HEAP32[$pEL>>2] = $120;
        $121 = HEAP32[$0>>2]|0;
        $122 = ((($121)) + 23|0);
        HEAP8[$122>>0] = 0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $123 = HEAP32[$i>>2]|0;
         $124 = HEAP32[$pEL>>2]|0;
         $125 = HEAP32[$124>>2]|0;
         $126 = ($123|0)<($125|0);
         if (!($126)) {
          break L27;
         }
         $127 = HEAP32[$0>>2]|0;
         $128 = HEAP32[$i>>2]|0;
         $129 = HEAP32[$pEL>>2]|0;
         $130 = ((($129)) + 4|0);
         $131 = HEAP32[$130>>2]|0;
         $132 = (($131) + (($128*20)|0)|0);
         $133 = HEAP32[$132>>2]|0;
         $134 = HEAP32[$i>>2]|0;
         $135 = HEAP32[$pEL>>2]|0;
         $136 = ((($135)) + 4|0);
         $137 = HEAP32[$136>>2]|0;
         $138 = (($137) + (($134*20)|0)|0);
         $139 = ((($138)) + 16|0);
         $140 = HEAP32[$139>>2]|0;
         _sqlite3ExprCode($127,$133,$140);
         $141 = HEAP32[$i>>2]|0;
         $142 = (($141) + 1)|0;
         HEAP32[$i>>2] = $142;
        }
       }
      } while(0);
      $143 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeGoto($143,1)|0);
     }
    }
   } while(0);
   $144 = HEAP32[$v>>2]|0;
   $145 = ($144|0)!=(0|0);
   if ($145) {
    $146 = HEAP32[$0>>2]|0;
    $147 = ((($146)) + 68|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = ($148|0)==(0);
    if ($149) {
     $150 = HEAP32[$db>>2]|0;
     $151 = ((($150)) + 65|0);
     $152 = HEAP8[$151>>0]|0;
     $153 = ($152<<24>>24)!=(0);
     if (!($153)) {
      $154 = HEAP32[$0>>2]|0;
      $155 = ((($154)) + 412|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = ($156|0)!=(0|0);
      if ($157) {
       $158 = HEAP32[$0>>2]|0;
       $159 = ((($158)) + 72|0);
       $160 = HEAP32[$159>>2]|0;
       $161 = ($160|0)==(0);
       if ($161) {
        $162 = HEAP32[$0>>2]|0;
        $163 = ((($162)) + 72|0);
        HEAP32[$163>>2] = 1;
       }
      }
      $164 = HEAP32[$v>>2]|0;
      $165 = HEAP32[$0>>2]|0;
      _sqlite3VdbeMakeReady($164,$165);
      $166 = HEAP32[$0>>2]|0;
      $167 = ((($166)) + 12|0);
      HEAP32[$167>>2] = 101;
      STACKTOP = sp;return;
     }
    }
   }
   $168 = HEAP32[$0>>2]|0;
   $169 = ((($168)) + 12|0);
   HEAP32[$169>>2] = 1;
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 12|0);
 HEAP32[$21>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3BeginTransaction($pParse,$type) {
 $pParse = $pParse|0;
 $type = $type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $v = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $type;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3AuthCheck($4,22,28613,0,0)|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(7);
 L7: do {
  if ($12) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $13 = HEAP32[$i>>2]|0;
    $14 = HEAP32[$db>>2]|0;
    $15 = ((($14)) + 20|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($13|0)<($16|0);
    if (!($17)) {
     break L7;
    }
    $18 = HEAP32[$v>>2]|0;
    $19 = HEAP32[$i>>2]|0;
    $20 = HEAP32[$1>>2]|0;
    $21 = ($20|0)==(9);
    $22 = $21&1;
    $23 = (($22) + 1)|0;
    (_sqlite3VdbeAddOp2($18,2,$19,$23)|0);
    $24 = HEAP32[$v>>2]|0;
    $25 = HEAP32[$i>>2]|0;
    _sqlite3VdbeUsesBtree($24,$25);
    $26 = HEAP32[$i>>2]|0;
    $27 = (($26) + 1)|0;
    HEAP32[$i>>2] = $27;
   }
  }
 } while(0);
 $28 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp0($28,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3CommitTransaction($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3AuthCheck($1,22,28657,0,0)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp1($8,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3RollbackTransaction($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3AuthCheck($1,22,28664,0,0)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($8,1,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3Savepoint($pParse,$op,$pName) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zName = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pName;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3NameFromToken($4,$5)|0);
 HEAP32[$zName>>2] = $6;
 $7 = HEAP32[$zName>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (2576 + ($14<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$zName>>2]|0;
  $18 = (_sqlite3AuthCheck($13,32,$16,$17,0)|0);
  $19 = ($18|0)!=(0);
  if (!($19)) {
   $23 = HEAP32[$v>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$zName>>2]|0;
   (_sqlite3VdbeAddOp4($23,0,$24,0,0,$25,-1)|0);
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _sqlite3StartTable($pParse,$pName1,$pName2,$isTemp,$isView,$isVirtual,$noErr) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $isTemp = $isTemp|0;
 $isView = $isView|0;
 $isVirtual = $isVirtual|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $db = 0, $fileFormat = 0, $iDb = 0, $or$cond = 0, $or$cond3 = 0, $pName = 0, $pTable = 0, $reg1 = 0, $reg2 = 0, $reg3 = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_buffer6 = 0, $zDb = 0, $zDb1 = 0;
 var $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $pTable = sp + 68|0;
 $zName = sp + 64|0;
 $db = sp + 60|0;
 $v = sp + 56|0;
 $iDb = sp + 52|0;
 $pName = sp + 48|0;
 $zDb = sp + 44|0;
 $zDb1 = sp + 40|0;
 $addr1 = sp + 36|0;
 $fileFormat = sp + 32|0;
 $reg1 = sp + 28|0;
 $reg2 = sp + 24|0;
 $reg3 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $isTemp;
 HEAP32[$4>>2] = $isView;
 HEAP32[$5>>2] = $isVirtual;
 HEAP32[$6>>2] = $noErr;
 HEAP32[$zName>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 140|0);
 $11 = ((($10)) + 5|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$db>>2]|0;
  $16 = ((($15)) + 140|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(1);
  if ($18) {
   $19 = HEAP32[$db>>2]|0;
   $20 = ((($19)) + 140|0);
   $21 = ((($20)) + 4|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$db>>2]|0;
   $25 = HEAP32[$iDb>>2]|0;
   $26 = ($25|0)==(1);
   $27 = $26 ? 21260 : 21279;
   $28 = (_sqlite3DbStrDup($24,$27)|0);
   HEAP32[$zName>>2] = $28;
   $29 = HEAP32[$1>>2]|0;
   HEAP32[$pName>>2] = $29;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (_sqlite3TwoPartName($30,$31,$32,$pName)|0);
  HEAP32[$iDb>>2] = $33;
  $34 = HEAP32[$iDb>>2]|0;
  $35 = ($34|0)<(0);
  if ($35) {
   STACKTOP = sp;return;
  }
  $36 = HEAP32[$3>>2]|0;
  $37 = ($36|0)!=(0);
  if ($37) {
   $38 = HEAP32[$2>>2]|0;
   $39 = ((($38)) + 4|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40>>>0)>(0);
   $42 = HEAP32[$iDb>>2]|0;
   $43 = ($42|0)!=(1);
   $or$cond = $41 & $43;
   if ($or$cond) {
    $44 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($44,28681,$vararg_buffer);
    STACKTOP = sp;return;
   }
  }
  $45 = HEAP32[$3>>2]|0;
  $46 = ($45|0)!=(0);
  if ($46) {
   HEAP32[$iDb>>2] = 1;
  }
  $47 = HEAP32[$db>>2]|0;
  $48 = HEAP32[$pName>>2]|0;
  $49 = (_sqlite3NameFromToken($47,$48)|0);
  HEAP32[$zName>>2] = $49;
 }
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 500|0);
 $52 = HEAP32[$pName>>2]|0;
 ;HEAP32[$51>>2]=HEAP32[$52>>2]|0;HEAP32[$51+4>>2]=HEAP32[$52+4>>2]|0;
 $53 = HEAP32[$zName>>2]|0;
 $54 = ($53|0)==(0|0);
 if ($54) {
  STACKTOP = sp;return;
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$zName>>2]|0;
 $57 = (_sqlite3CheckObjectName($55,$56)|0);
 $58 = (0)!=($57|0);
 L21: do {
  if (!($58)) {
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 140|0);
   $61 = ((($60)) + 4|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = ($63|0)==(1);
   if ($64) {
    HEAP32[$3>>2] = 1;
   }
   $65 = HEAP32[$iDb>>2]|0;
   $66 = HEAP32[$db>>2]|0;
   $67 = ((($66)) + 16|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($68) + ($65<<4)|0);
   $70 = HEAP32[$69>>2]|0;
   HEAP32[$zDb>>2] = $70;
   $71 = HEAP32[$0>>2]|0;
   $72 = HEAP32[$3>>2]|0;
   $73 = ($72|0)==(1);
   $74 = $73 ? 21260 : 21279;
   $75 = HEAP32[$zDb>>2]|0;
   $76 = (_sqlite3AuthCheck($71,18,$74,0,$75)|0);
   $77 = ($76|0)!=(0);
   if (!($77)) {
    $78 = HEAP32[$5>>2]|0;
    $79 = ($78|0)!=(0);
    if (!($79)) {
     $80 = HEAP32[$0>>2]|0;
     $81 = HEAP32[$3>>2]|0;
     $82 = HEAP32[$4>>2]|0;
     $83 = $82<<1;
     $84 = (($81) + ($83))|0;
     $85 = (28722 + ($84)|0);
     $86 = HEAP8[$85>>0]|0;
     $87 = $86&255;
     $88 = HEAP32[$zName>>2]|0;
     $89 = HEAP32[$zDb>>2]|0;
     $90 = (_sqlite3AuthCheck($80,$87,$88,0,$89)|0);
     $91 = ($90|0)!=(0);
     if ($91) {
      break;
     }
    }
    $92 = HEAP32[$0>>2]|0;
    $93 = ((($92)) + 454|0);
    $94 = HEAP8[$93>>0]|0;
    $95 = ($94<<24>>24)!=(0);
    do {
     if (!($95)) {
      $96 = HEAP32[$iDb>>2]|0;
      $97 = HEAP32[$db>>2]|0;
      $98 = ((($97)) + 16|0);
      $99 = HEAP32[$98>>2]|0;
      $100 = (($99) + ($96<<4)|0);
      $101 = HEAP32[$100>>2]|0;
      HEAP32[$zDb1>>2] = $101;
      $102 = HEAP32[$0>>2]|0;
      $103 = (_sqlite3ReadSchema($102)|0);
      $104 = (0)!=($103|0);
      if ($104) {
       break L21;
      }
      $105 = HEAP32[$db>>2]|0;
      $106 = HEAP32[$zName>>2]|0;
      $107 = HEAP32[$zDb1>>2]|0;
      $108 = (_sqlite3FindTable($105,$106,$107)|0);
      HEAP32[$pTable>>2] = $108;
      $109 = HEAP32[$pTable>>2]|0;
      $110 = ($109|0)!=(0|0);
      if (!($110)) {
       $116 = HEAP32[$db>>2]|0;
       $117 = HEAP32[$zName>>2]|0;
       $118 = HEAP32[$zDb1>>2]|0;
       $119 = (_sqlite3FindIndex($116,$117,$118)|0);
       $120 = ($119|0)!=(0|0);
       if (!($120)) {
        break;
       }
       $121 = HEAP32[$0>>2]|0;
       $122 = HEAP32[$zName>>2]|0;
       HEAP32[$vararg_buffer6>>2] = $122;
       _sqlite3ErrorMsg($121,28750,$vararg_buffer6);
       break L21;
      }
      $111 = HEAP32[$6>>2]|0;
      $112 = ($111|0)!=(0);
      $113 = HEAP32[$0>>2]|0;
      if ($112) {
       $115 = HEAP32[$iDb>>2]|0;
       _sqlite3CodeVerifySchema($113,$115);
       break L21;
      } else {
       $114 = HEAP32[$pName>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $114;
       _sqlite3ErrorMsg($113,28726,$vararg_buffer4);
       break L21;
      }
     }
    } while(0);
    $123 = HEAP32[$db>>2]|0;
    $124 = (_sqlite3DbMallocZero($123,72,0)|0);
    HEAP32[$pTable>>2] = $124;
    $125 = HEAP32[$pTable>>2]|0;
    $126 = ($125|0)==(0|0);
    if ($126) {
     $127 = HEAP32[$0>>2]|0;
     $128 = ((($127)) + 12|0);
     HEAP32[$128>>2] = 7;
     $129 = HEAP32[$0>>2]|0;
     $130 = ((($129)) + 68|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = (($131) + 1)|0;
     HEAP32[$130>>2] = $132;
     break;
    }
    $133 = HEAP32[$zName>>2]|0;
    $134 = HEAP32[$pTable>>2]|0;
    HEAP32[$134>>2] = $133;
    $135 = HEAP32[$pTable>>2]|0;
    $136 = ((($135)) + 32|0);
    HEAP16[$136>>1] = -1;
    $137 = HEAP32[$iDb>>2]|0;
    $138 = HEAP32[$db>>2]|0;
    $139 = ((($138)) + 16|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = (($140) + ($137<<4)|0);
    $142 = ((($141)) + 12|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = HEAP32[$pTable>>2]|0;
    $145 = ((($144)) + 64|0);
    HEAP32[$145>>2] = $143;
    $146 = HEAP32[$pTable>>2]|0;
    $147 = ((($146)) + 36|0);
    HEAP16[$147>>1] = 1;
    $148 = HEAP32[$pTable>>2]|0;
    $149 = ((($148)) + 38|0);
    HEAP16[$149>>1] = 200;
    $150 = HEAP32[$pTable>>2]|0;
    $151 = HEAP32[$0>>2]|0;
    $152 = ((($151)) + 488|0);
    HEAP32[$152>>2] = $150;
    $153 = HEAP32[$0>>2]|0;
    $154 = ((($153)) + 18|0);
    $155 = HEAP8[$154>>0]|0;
    $156 = ($155<<24>>24)!=(0);
    if (!($156)) {
     $157 = HEAP32[$zName>>2]|0;
     $158 = (_strcmp($157,28785)|0);
     $159 = ($158|0)==(0);
     if ($159) {
      $160 = HEAP32[$pTable>>2]|0;
      $161 = HEAP32[$pTable>>2]|0;
      $162 = ((($161)) + 64|0);
      $163 = HEAP32[$162>>2]|0;
      $164 = ((($163)) + 72|0);
      HEAP32[$164>>2] = $160;
     }
    }
    $165 = HEAP32[$db>>2]|0;
    $166 = ((($165)) + 140|0);
    $167 = ((($166)) + 5|0);
    $168 = HEAP8[$167>>0]|0;
    $169 = ($168<<24>>24)!=(0);
    if ($169) {
     STACKTOP = sp;return;
    }
    $170 = HEAP32[$0>>2]|0;
    $171 = (_sqlite3GetVdbe($170)|0);
    HEAP32[$v>>2] = $171;
    $172 = ($171|0)!=(0|0);
    if (!($172)) {
     STACKTOP = sp;return;
    }
    $173 = HEAP32[$0>>2]|0;
    $174 = HEAP32[$iDb>>2]|0;
    _sqlite3BeginWriteOperation($173,1,$174);
    $175 = HEAP32[$5>>2]|0;
    $176 = ($175|0)!=(0);
    if ($176) {
     $177 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($177,149)|0);
    }
    $178 = HEAP32[$0>>2]|0;
    $179 = ((($178)) + 76|0);
    $180 = HEAP32[$179>>2]|0;
    $181 = (($180) + 1)|0;
    HEAP32[$179>>2] = $181;
    $182 = HEAP32[$0>>2]|0;
    $183 = ((($182)) + 392|0);
    HEAP32[$183>>2] = $181;
    HEAP32[$reg1>>2] = $181;
    $184 = HEAP32[$0>>2]|0;
    $185 = ((($184)) + 76|0);
    $186 = HEAP32[$185>>2]|0;
    $187 = (($186) + 1)|0;
    HEAP32[$185>>2] = $187;
    $188 = HEAP32[$0>>2]|0;
    $189 = ((($188)) + 396|0);
    HEAP32[$189>>2] = $187;
    HEAP32[$reg2>>2] = $187;
    $190 = HEAP32[$0>>2]|0;
    $191 = ((($190)) + 76|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = (($192) + 1)|0;
    HEAP32[$191>>2] = $193;
    HEAP32[$reg3>>2] = $193;
    $194 = HEAP32[$v>>2]|0;
    $195 = HEAP32[$iDb>>2]|0;
    $196 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp3($194,51,$195,$196,2)|0);
    $197 = HEAP32[$v>>2]|0;
    $198 = HEAP32[$iDb>>2]|0;
    _sqlite3VdbeUsesBtree($197,$198);
    $199 = HEAP32[$v>>2]|0;
    $200 = HEAP32[$reg3>>2]|0;
    $201 = (_sqlite3VdbeAddOp1($199,45,$200)|0);
    HEAP32[$addr1>>2] = $201;
    $202 = HEAP32[$db>>2]|0;
    $203 = ((($202)) + 24|0);
    $204 = HEAP32[$203>>2]|0;
    $205 = $204 & 32768;
    $206 = ($205|0)!=(0);
    $207 = $206 ? 1 : 4;
    HEAP32[$fileFormat>>2] = $207;
    $208 = HEAP32[$v>>2]|0;
    $209 = HEAP32[$iDb>>2]|0;
    $210 = HEAP32[$fileFormat>>2]|0;
    (_sqlite3VdbeAddOp3($208,52,$209,2,$210)|0);
    $211 = HEAP32[$v>>2]|0;
    $212 = HEAP32[$iDb>>2]|0;
    $213 = HEAP32[$db>>2]|0;
    $214 = ((($213)) + 62|0);
    $215 = HEAP8[$214>>0]|0;
    $216 = $215&255;
    (_sqlite3VdbeAddOp3($211,52,$212,5,$216)|0);
    $217 = HEAP32[$v>>2]|0;
    $218 = HEAP32[$addr1>>2]|0;
    _sqlite3VdbeJumpHere($217,$218);
    $219 = HEAP32[$4>>2]|0;
    $220 = ($219|0)!=(0);
    $221 = HEAP32[$5>>2]|0;
    $222 = ($221|0)!=(0);
    $or$cond3 = $220 | $222;
    $223 = HEAP32[$v>>2]|0;
    if ($or$cond3) {
     $224 = HEAP32[$reg2>>2]|0;
     (_sqlite3VdbeAddOp2($223,22,0,$224)|0);
    } else {
     $225 = HEAP32[$iDb>>2]|0;
     $226 = HEAP32[$reg2>>2]|0;
     $227 = (_sqlite3VdbeAddOp2($223,122,$225,$226)|0);
     $228 = HEAP32[$0>>2]|0;
     $229 = ((($228)) + 424|0);
     HEAP32[$229>>2] = $227;
    }
    $230 = HEAP32[$0>>2]|0;
    $231 = HEAP32[$iDb>>2]|0;
    _sqlite3OpenMasterTable($230,$231);
    $232 = HEAP32[$v>>2]|0;
    $233 = HEAP32[$reg1>>2]|0;
    (_sqlite3VdbeAddOp2($232,74,0,$233)|0);
    $234 = HEAP32[$v>>2]|0;
    $235 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp4($234,27,6,$235,0,28801,-2)|0);
    $236 = HEAP32[$v>>2]|0;
    $237 = HEAP32[$reg3>>2]|0;
    $238 = HEAP32[$reg1>>2]|0;
    (_sqlite3VdbeAddOp3($236,75,0,$237,$238)|0);
    $239 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($239,8);
    $240 = HEAP32[$v>>2]|0;
    (_sqlite3VdbeAddOp0($240,61)|0);
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $241 = HEAP32[$db>>2]|0;
 $242 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($241,$242);
 STACKTOP = sp;return;
}
function _disableLookaside($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = (($3) + 1)<<24>>24;
 HEAP8[$2>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 248|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3EndTable($pParse,$pCons,$pEnd,$tabOpts,$pSelect) {
 $pParse = $pParse|0;
 $pCons = $pCons|0;
 $pEnd = $pEnd|0;
 $tabOpts = $tabOpts|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addrInsLoop = 0, $addrTop = 0, $db = 0, $dest = 0, $iDb = 0, $n = 0, $nName = 0, $or$cond = 0, $p = 0, $pDb = 0, $pEnd2 = 0, $pIdx = 0, $pOld = 0, $pSchema = 0, $pSelTab = 0, $regRec = 0, $regRowid = 0, $regYield = 0, $v = 0;
 var $vararg_buffer = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer22 = 0, $vararg_buffer4 = 0, $vararg_buffer9 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zName = 0, $zStmt = 0, $zType = 0, $zType2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer22 = sp + 72|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 192|0;
 $1 = sp + 188|0;
 $2 = sp + 184|0;
 $3 = sp + 196|0;
 $4 = sp + 180|0;
 $p = sp + 176|0;
 $db = sp + 172|0;
 $iDb = sp + 168|0;
 $pIdx = sp + 164|0;
 $n = sp + 160|0;
 $v = sp + 156|0;
 $zType = sp + 152|0;
 $zType2 = sp + 148|0;
 $zStmt = sp + 144|0;
 $dest = sp + 124|0;
 $regYield = sp + 120|0;
 $addrTop = sp + 116|0;
 $regRec = sp + 112|0;
 $regRowid = sp + 108|0;
 $addrInsLoop = sp + 104|0;
 $pSelTab = sp + 100|0;
 $pEnd2 = sp + 96|0;
 $pDb = sp + 92|0;
 $pOld = sp + 88|0;
 $pSchema = sp + 84|0;
 $zName = sp + 80|0;
 $nName = sp + 76|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pCons;
 HEAP32[$2>>2] = $pEnd;
 HEAP8[$3>>0] = $tabOpts;
 HEAP32[$4>>2] = $pSelect;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$4>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 & $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 488|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 140|0);
 $18 = ((($17)) + 5|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if ($20) {
  $21 = HEAP32[$db>>2]|0;
  $22 = ((($21)) + 140|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 28|0);
  HEAP32[$25>>2] = $23;
  $26 = HEAP32[$p>>2]|0;
  $27 = ((($26)) + 28|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(1);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 42|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   $34 = $33 | 1;
   $35 = $34&255;
   HEAP8[$31>>0] = $35;
  }
 }
 $36 = HEAP8[$3>>0]|0;
 $37 = $36&255;
 $38 = $37 & 32;
 $39 = ($38|0)!=(0);
 do {
  if ($39) {
   $40 = HEAP32[$p>>2]|0;
   $41 = ((($40)) + 42|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = $42&255;
   $44 = $43 & 8;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($46,28894,$vararg_buffer);
    STACKTOP = sp;return;
   }
   $47 = HEAP32[$p>>2]|0;
   $48 = ((($47)) + 42|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = $50 & 4;
   $52 = ($51|0)==(0);
   if ($52) {
    $53 = HEAP32[$0>>2]|0;
    $54 = HEAP32[$p>>2]|0;
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$vararg_buffer2>>2] = $55;
    _sqlite3ErrorMsg($53,28944,$vararg_buffer2);
    break;
   } else {
    $56 = HEAP32[$p>>2]|0;
    $57 = ((($56)) + 42|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    $60 = $59 | 96;
    $61 = $60&255;
    HEAP8[$57>>0] = $61;
    $62 = HEAP32[$0>>2]|0;
    $63 = HEAP32[$p>>2]|0;
    _convertToWithoutRowidTable($62,$63);
    break;
   }
  }
 } while(0);
 $64 = HEAP32[$db>>2]|0;
 $65 = HEAP32[$p>>2]|0;
 $66 = ((($65)) + 64|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (_sqlite3SchemaToIndex($64,$67)|0);
 HEAP32[$iDb>>2] = $68;
 $69 = HEAP32[$p>>2]|0;
 $70 = ((($69)) + 24|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0|0);
 if ($72) {
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$p>>2]|0;
  $75 = HEAP32[$p>>2]|0;
  $76 = ((($75)) + 24|0);
  $77 = HEAP32[$76>>2]|0;
  _sqlite3ResolveSelfReference($73,$74,4,0,$77);
 }
 $78 = HEAP32[$p>>2]|0;
 _estimateTableWidth($78);
 $79 = HEAP32[$p>>2]|0;
 $80 = ((($79)) + 8|0);
 $81 = HEAP32[$80>>2]|0;
 HEAP32[$pIdx>>2] = $81;
 while(1) {
  $82 = HEAP32[$pIdx>>2]|0;
  $83 = ($82|0)!=(0|0);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$pIdx>>2]|0;
  _estimateIndexWidth($84);
  $85 = HEAP32[$pIdx>>2]|0;
  $86 = ((($85)) + 20|0);
  $87 = HEAP32[$86>>2]|0;
  HEAP32[$pIdx>>2] = $87;
 }
 $88 = HEAP32[$db>>2]|0;
 $89 = ((($88)) + 140|0);
 $90 = ((($89)) + 5|0);
 $91 = HEAP8[$90>>0]|0;
 $92 = ($91<<24>>24)!=(0);
 if (!($92)) {
  $93 = HEAP32[$0>>2]|0;
  $94 = (_sqlite3GetVdbe($93)|0);
  HEAP32[$v>>2] = $94;
  $95 = HEAP32[$v>>2]|0;
  $96 = ($95|0)==(0|0);
  if ($96) {
   STACKTOP = sp;return;
  }
  $97 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp1($97,61,0)|0);
  $98 = HEAP32[$p>>2]|0;
  $99 = ((($98)) + 12|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($100|0)==(0|0);
  if ($101) {
   HEAP32[$zType>>2] = 28976;
   HEAP32[$zType2>>2] = 28982;
  } else {
   HEAP32[$zType>>2] = 28988;
   HEAP32[$zType2>>2] = 28993;
  }
  $102 = HEAP32[$4>>2]|0;
  $103 = ($102|0)!=(0|0);
  do {
   if ($103) {
    $104 = HEAP32[$0>>2]|0;
    $105 = ((($104)) + 76|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = (($106) + 1)|0;
    HEAP32[$105>>2] = $107;
    HEAP32[$regYield>>2] = $107;
    $108 = HEAP32[$0>>2]|0;
    $109 = ((($108)) + 76|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = (($110) + 1)|0;
    HEAP32[$109>>2] = $111;
    HEAP32[$regRec>>2] = $111;
    $112 = HEAP32[$0>>2]|0;
    $113 = ((($112)) + 76|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = (($114) + 1)|0;
    HEAP32[$113>>2] = $115;
    HEAP32[$regRowid>>2] = $115;
    $116 = HEAP32[$0>>2]|0;
    _sqlite3MayAbort($116);
    $117 = HEAP32[$v>>2]|0;
    $118 = HEAP32[$0>>2]|0;
    $119 = ((($118)) + 396|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = HEAP32[$iDb>>2]|0;
    (_sqlite3VdbeAddOp3($117,55,1,$120,$121)|0);
    $122 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($122,16);
    $123 = HEAP32[$0>>2]|0;
    $124 = ((($123)) + 72|0);
    HEAP32[$124>>2] = 2;
    $125 = HEAP32[$v>>2]|0;
    $126 = (_sqlite3VdbeCurrentAddr($125)|0);
    $127 = (($126) + 1)|0;
    HEAP32[$addrTop>>2] = $127;
    $128 = HEAP32[$v>>2]|0;
    $129 = HEAP32[$regYield>>2]|0;
    $130 = HEAP32[$addrTop>>2]|0;
    (_sqlite3VdbeAddOp3($128,16,$129,0,$130)|0);
    $131 = HEAP32[$regYield>>2]|0;
    _sqlite3SelectDestInit($dest,13,$131);
    $132 = HEAP32[$0>>2]|0;
    $133 = HEAP32[$4>>2]|0;
    (_sqlite3Select($132,$133,$dest)|0);
    $134 = HEAP32[$v>>2]|0;
    $135 = HEAP32[$regYield>>2]|0;
    _sqlite3VdbeEndCoroutine($134,$135);
    $136 = HEAP32[$v>>2]|0;
    $137 = HEAP32[$addrTop>>2]|0;
    $138 = (($137) - 1)|0;
    _sqlite3VdbeJumpHere($136,$138);
    $139 = HEAP32[$0>>2]|0;
    $140 = ((($139)) + 68|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = ($141|0)!=(0);
    if ($142) {
     STACKTOP = sp;return;
    }
    $143 = HEAP32[$0>>2]|0;
    $144 = HEAP32[$4>>2]|0;
    $145 = (_sqlite3ResultSetOfSelect($143,$144)|0);
    HEAP32[$pSelTab>>2] = $145;
    $146 = HEAP32[$pSelTab>>2]|0;
    $147 = ($146|0)==(0|0);
    if ($147) {
     STACKTOP = sp;return;
    } else {
     $148 = HEAP32[$pSelTab>>2]|0;
     $149 = ((($148)) + 34|0);
     $150 = HEAP16[$149>>1]|0;
     $151 = HEAP32[$p>>2]|0;
     $152 = ((($151)) + 34|0);
     HEAP16[$152>>1] = $150;
     $153 = HEAP32[$pSelTab>>2]|0;
     $154 = ((($153)) + 4|0);
     $155 = HEAP32[$154>>2]|0;
     $156 = HEAP32[$p>>2]|0;
     $157 = ((($156)) + 4|0);
     HEAP32[$157>>2] = $155;
     $158 = HEAP32[$pSelTab>>2]|0;
     $159 = ((($158)) + 34|0);
     HEAP16[$159>>1] = 0;
     $160 = HEAP32[$pSelTab>>2]|0;
     $161 = ((($160)) + 4|0);
     HEAP32[$161>>2] = 0;
     $162 = HEAP32[$db>>2]|0;
     $163 = HEAP32[$pSelTab>>2]|0;
     _sqlite3DeleteTable($162,$163);
     $164 = HEAP32[$v>>2]|0;
     $165 = ((($dest)) + 4|0);
     $166 = HEAP32[$165>>2]|0;
     $167 = (_sqlite3VdbeAddOp1($164,18,$166)|0);
     HEAP32[$addrInsLoop>>2] = $167;
     $168 = HEAP32[$v>>2]|0;
     $169 = ((($dest)) + 8|0);
     $170 = HEAP32[$169>>2]|0;
     $171 = ((($dest)) + 12|0);
     $172 = HEAP32[$171>>2]|0;
     $173 = HEAP32[$regRec>>2]|0;
     (_sqlite3VdbeAddOp3($168,49,$170,$172,$173)|0);
     $174 = HEAP32[$v>>2]|0;
     $175 = HEAP32[$p>>2]|0;
     _sqlite3TableAffinity($174,$175,0);
     $176 = HEAP32[$v>>2]|0;
     $177 = HEAP32[$regRowid>>2]|0;
     (_sqlite3VdbeAddOp2($176,74,1,$177)|0);
     $178 = HEAP32[$v>>2]|0;
     $179 = HEAP32[$regRec>>2]|0;
     $180 = HEAP32[$regRowid>>2]|0;
     (_sqlite3VdbeAddOp3($178,75,1,$179,$180)|0);
     $181 = HEAP32[$v>>2]|0;
     $182 = HEAP32[$addrInsLoop>>2]|0;
     (_sqlite3VdbeGoto($181,$182)|0);
     $183 = HEAP32[$v>>2]|0;
     $184 = HEAP32[$addrInsLoop>>2]|0;
     _sqlite3VdbeJumpHere($183,$184);
     $185 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp1($185,61,1)|0);
     break;
    }
   }
  } while(0);
  $186 = HEAP32[$4>>2]|0;
  $187 = ($186|0)!=(0|0);
  if ($187) {
   $188 = HEAP32[$db>>2]|0;
   $189 = HEAP32[$p>>2]|0;
   $190 = (_createTableStmt($188,$189)|0);
   HEAP32[$zStmt>>2] = $190;
  } else {
   $191 = HEAP8[$3>>0]|0;
   $192 = $191&255;
   $193 = ($192|0)!=(0);
   $194 = HEAP32[$0>>2]|0;
   $195 = ((($194)) + 508|0);
   $196 = HEAP32[$2>>2]|0;
   $197 = $193 ? $195 : $196;
   HEAP32[$pEnd2>>2] = $197;
   $198 = HEAP32[$pEnd2>>2]|0;
   $199 = HEAP32[$198>>2]|0;
   $200 = HEAP32[$0>>2]|0;
   $201 = ((($200)) + 500|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = $199;
   $204 = $202;
   $205 = (($203) - ($204))|0;
   HEAP32[$n>>2] = $205;
   $206 = HEAP32[$pEnd2>>2]|0;
   $207 = HEAP32[$206>>2]|0;
   $208 = HEAP8[$207>>0]|0;
   $209 = $208 << 24 >> 24;
   $210 = ($209|0)!=(59);
   if ($210) {
    $211 = HEAP32[$pEnd2>>2]|0;
    $212 = ((($211)) + 4|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = HEAP32[$n>>2]|0;
    $215 = (($214) + ($213))|0;
    HEAP32[$n>>2] = $215;
   }
   $216 = HEAP32[$db>>2]|0;
   $217 = HEAP32[$zType2>>2]|0;
   $218 = HEAP32[$n>>2]|0;
   $219 = HEAP32[$0>>2]|0;
   $220 = ((($219)) + 500|0);
   $221 = HEAP32[$220>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $217;
   $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $218;
   $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
   HEAP32[$vararg_ptr8>>2] = $221;
   $222 = (_sqlite3MPrintf($216,28998,$vararg_buffer4)|0);
   HEAP32[$zStmt>>2] = $222;
  }
  $223 = HEAP32[$0>>2]|0;
  $224 = HEAP32[$iDb>>2]|0;
  $225 = HEAP32[$db>>2]|0;
  $226 = ((($225)) + 16|0);
  $227 = HEAP32[$226>>2]|0;
  $228 = (($227) + ($224<<4)|0);
  $229 = HEAP32[$228>>2]|0;
  $230 = HEAP32[$iDb>>2]|0;
  $231 = ($230|0)==(1);
  $232 = $231 ? 21260 : 21279;
  $233 = HEAP32[$zType>>2]|0;
  $234 = HEAP32[$p>>2]|0;
  $235 = HEAP32[$234>>2]|0;
  $236 = HEAP32[$p>>2]|0;
  $237 = HEAP32[$236>>2]|0;
  $238 = HEAP32[$0>>2]|0;
  $239 = ((($238)) + 396|0);
  $240 = HEAP32[$239>>2]|0;
  $241 = HEAP32[$zStmt>>2]|0;
  $242 = HEAP32[$0>>2]|0;
  $243 = ((($242)) + 392|0);
  $244 = HEAP32[$243>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $229;
  $vararg_ptr12 = ((($vararg_buffer9)) + 4|0);
  HEAP32[$vararg_ptr12>>2] = $232;
  $vararg_ptr13 = ((($vararg_buffer9)) + 8|0);
  HEAP32[$vararg_ptr13>>2] = $233;
  $vararg_ptr14 = ((($vararg_buffer9)) + 12|0);
  HEAP32[$vararg_ptr14>>2] = $235;
  $vararg_ptr15 = ((($vararg_buffer9)) + 16|0);
  HEAP32[$vararg_ptr15>>2] = $237;
  $vararg_ptr16 = ((($vararg_buffer9)) + 20|0);
  HEAP32[$vararg_ptr16>>2] = $240;
  $vararg_ptr17 = ((($vararg_buffer9)) + 24|0);
  HEAP32[$vararg_ptr17>>2] = $241;
  $vararg_ptr18 = ((($vararg_buffer9)) + 28|0);
  HEAP32[$vararg_ptr18>>2] = $244;
  _sqlite3NestedParse($223,29013,$vararg_buffer9);
  $245 = HEAP32[$db>>2]|0;
  $246 = HEAP32[$zStmt>>2]|0;
  _sqlite3DbFree($245,$246);
  $247 = HEAP32[$0>>2]|0;
  $248 = HEAP32[$iDb>>2]|0;
  _sqlite3ChangeCookie($247,$248);
  $249 = HEAP32[$p>>2]|0;
  $250 = ((($249)) + 42|0);
  $251 = HEAP8[$250>>0]|0;
  $252 = $251&255;
  $253 = $252 & 8;
  $254 = ($253|0)!=(0);
  if ($254) {
   $255 = HEAP32[$iDb>>2]|0;
   $256 = HEAP32[$db>>2]|0;
   $257 = ((($256)) + 16|0);
   $258 = HEAP32[$257>>2]|0;
   $259 = (($258) + ($255<<4)|0);
   HEAP32[$pDb>>2] = $259;
   $260 = HEAP32[$pDb>>2]|0;
   $261 = ((($260)) + 12|0);
   $262 = HEAP32[$261>>2]|0;
   $263 = ((($262)) + 72|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = ($264|0)==(0|0);
   if ($265) {
    $266 = HEAP32[$0>>2]|0;
    $267 = HEAP32[$pDb>>2]|0;
    $268 = HEAP32[$267>>2]|0;
    HEAP32[$vararg_buffer19>>2] = $268;
    _sqlite3NestedParse($266,29100,$vararg_buffer19);
   }
  }
  $269 = HEAP32[$v>>2]|0;
  $270 = HEAP32[$iDb>>2]|0;
  $271 = HEAP32[$db>>2]|0;
  $272 = HEAP32[$p>>2]|0;
  $273 = HEAP32[$272>>2]|0;
  HEAP32[$vararg_buffer22>>2] = $273;
  $274 = (_sqlite3MPrintf($271,29142,$vararg_buffer22)|0);
  _sqlite3VdbeAddParseSchemaOp($269,$270,$274);
 }
 $275 = HEAP32[$db>>2]|0;
 $276 = ((($275)) + 140|0);
 $277 = ((($276)) + 5|0);
 $278 = HEAP8[$277>>0]|0;
 $279 = ($278<<24>>24)!=(0);
 if (!($279)) {
  STACKTOP = sp;return;
 }
 $280 = HEAP32[$p>>2]|0;
 $281 = ((($280)) + 64|0);
 $282 = HEAP32[$281>>2]|0;
 HEAP32[$pSchema>>2] = $282;
 $283 = HEAP32[$pSchema>>2]|0;
 $284 = ((($283)) + 8|0);
 $285 = HEAP32[$p>>2]|0;
 $286 = HEAP32[$285>>2]|0;
 $287 = HEAP32[$p>>2]|0;
 $288 = (_sqlite3HashInsert($284,$286,$287)|0);
 HEAP32[$pOld>>2] = $288;
 $289 = HEAP32[$pOld>>2]|0;
 $290 = ($289|0)!=(0|0);
 if ($290) {
  $291 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($291);
  STACKTOP = sp;return;
 }
 $292 = HEAP32[$0>>2]|0;
 $293 = ((($292)) + 488|0);
 HEAP32[$293>>2] = 0;
 $294 = HEAP32[$db>>2]|0;
 $295 = ((($294)) + 24|0);
 $296 = HEAP32[$295>>2]|0;
 $297 = $296 | 2;
 HEAP32[$295>>2] = $297;
 $298 = HEAP32[$p>>2]|0;
 $299 = ((($298)) + 12|0);
 $300 = HEAP32[$299>>2]|0;
 $301 = ($300|0)!=(0|0);
 if ($301) {
  STACKTOP = sp;return;
 }
 $302 = HEAP32[$0>>2]|0;
 $303 = ((($302)) + 500|0);
 $304 = HEAP32[$303>>2]|0;
 HEAP32[$zName>>2] = $304;
 $305 = HEAP32[$1>>2]|0;
 $306 = HEAP32[$305>>2]|0;
 $307 = ($306|0)==(0|0);
 if ($307) {
  $308 = HEAP32[$2>>2]|0;
  HEAP32[$1>>2] = $308;
 }
 $309 = HEAP32[$1>>2]|0;
 $310 = HEAP32[$309>>2]|0;
 $311 = HEAP32[$zName>>2]|0;
 $312 = $310;
 $313 = $311;
 $314 = (($312) - ($313))|0;
 HEAP32[$nName>>2] = $314;
 $315 = HEAP32[$zName>>2]|0;
 $316 = HEAP32[$nName>>2]|0;
 $317 = (_sqlite3Utf8CharLen($315,$316)|0);
 $318 = (13 + ($317))|0;
 $319 = HEAP32[$p>>2]|0;
 $320 = ((($319)) + 44|0);
 HEAP32[$320>>2] = $318;
 STACKTOP = sp;return;
}
function _sqlite3AddColumn($pParse,$pName) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $aNew = 0, $db = 0, $i = 0, $p = 0, $pCol = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $p = sp + 32|0;
 $i = sp + 28|0;
 $z = sp + 24|0;
 $pCol = sp + 20|0;
 $db = sp + 16|0;
 $aNew = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 488|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 34|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (($11) + 1)|0;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 88|0);
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($12|0)>($16|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$p>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$vararg_buffer>>2] = $20;
  _sqlite3ErrorMsg($18,30823,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (_sqlite3NameFromToken($21,$22)|0);
 HEAP32[$z>>2] = $23;
 $24 = HEAP32[$z>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 34|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29 << 16 >> 16;
  $31 = ($26|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = HEAP32[$z>>2]|0;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*24)|0)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (_sqlite3_stricmp($32,$38)|0);
  $40 = ($39|0)==(0);
  if ($40) {
   label = 8;
   break;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
 }
 if ((label|0) == 8) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $42;
  _sqlite3ErrorMsg($41,30846,$vararg_buffer1);
  $43 = HEAP32[$db>>2]|0;
  $44 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($43,$44);
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$p>>2]|0;
 $48 = ((($47)) + 34|0);
 $49 = HEAP16[$48>>1]|0;
 $50 = $49 << 16 >> 16;
 $51 = $50 & 7;
 $52 = ($51|0)==(0);
 do {
  if ($52) {
   $53 = HEAP32[$db>>2]|0;
   $54 = HEAP32[$p>>2]|0;
   $55 = ((($54)) + 4|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$p>>2]|0;
   $58 = ((($57)) + 34|0);
   $59 = HEAP16[$58>>1]|0;
   $60 = $59 << 16 >> 16;
   $61 = (($60) + 8)|0;
   $62 = ($61*24)|0;
   $63 = (_sqlite3DbRealloc($53,$56,$62,0)|0);
   HEAP32[$aNew>>2] = $63;
   $64 = HEAP32[$aNew>>2]|0;
   $65 = ($64|0)==(0|0);
   if (!($65)) {
    $68 = HEAP32[$aNew>>2]|0;
    $69 = HEAP32[$p>>2]|0;
    $70 = ((($69)) + 4|0);
    HEAP32[$70>>2] = $68;
    break;
   }
   $66 = HEAP32[$db>>2]|0;
   $67 = HEAP32[$z>>2]|0;
   _sqlite3DbFree($66,$67);
   STACKTOP = sp;return;
  }
 } while(0);
 $71 = HEAP32[$p>>2]|0;
 $72 = ((($71)) + 34|0);
 $73 = HEAP16[$72>>1]|0;
 $74 = $73 << 16 >> 16;
 $75 = HEAP32[$p>>2]|0;
 $76 = ((($75)) + 4|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = (($77) + (($74*24)|0)|0);
 HEAP32[$pCol>>2] = $78;
 $79 = HEAP32[$pCol>>2]|0;
 ;HEAP32[$79>>2]=0|0;HEAP32[$79+4>>2]=0|0;HEAP32[$79+8>>2]=0|0;HEAP32[$79+12>>2]=0|0;HEAP32[$79+16>>2]=0|0;HEAP32[$79+20>>2]=0|0;
 $80 = HEAP32[$z>>2]|0;
 $81 = HEAP32[$pCol>>2]|0;
 HEAP32[$81>>2] = $80;
 $82 = HEAP32[$pCol>>2]|0;
 $83 = ((($82)) + 21|0);
 HEAP8[$83>>0] = 65;
 $84 = HEAP32[$pCol>>2]|0;
 $85 = ((($84)) + 22|0);
 HEAP8[$85>>0] = 1;
 $86 = HEAP32[$p>>2]|0;
 $87 = ((($86)) + 34|0);
 $88 = HEAP16[$87>>1]|0;
 $89 = (($88) + 1)<<16>>16;
 HEAP16[$87>>1] = $89;
 STACKTOP = sp;return;
}
function _sqlite3AddColumnType($pParse,$pType) {
 $pParse = $pParse|0;
 $pType = $pType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pType;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 488|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 34|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = ($10|0)<(1);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 34|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14 << 16 >> 16;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + (($16*24)|0)|0);
 HEAP32[$pCol>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pCol>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 _sqlite3DbFree($22,$25);
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = (_sqlite3NameFromToken($27,$28)|0);
 $30 = HEAP32[$pCol>>2]|0;
 $31 = ((($30)) + 12|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$pCol>>2]|0;
 $33 = ((($32)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$pCol>>2]|0;
 $36 = ((($35)) + 22|0);
 $37 = (_sqlite3AffinityType($34,$36)|0);
 $38 = HEAP32[$pCol>>2]|0;
 $39 = ((($38)) + 21|0);
 HEAP8[$39>>0] = $37;
 STACKTOP = sp;return;
}
function _sqlite3AddDefaultValue($pParse,$pSpan) {
 $pParse = $pParse|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $pCol = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $p = sp + 12|0;
 $pCol = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSpan;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 488|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 34|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = (($12) - 1)|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + (($13*24)|0)|0);
 HEAP32[$pCol>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$db>>2]|0;
 $21 = ((($20)) + 140|0);
 $22 = ((($21)) + 5|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = (_sqlite3ExprIsConstantOrFunction($19,$23)|0);
 $25 = ($24|0)!=(0);
 if ($25) {
  $29 = HEAP32[$db>>2]|0;
  $30 = HEAP32[$pCol>>2]|0;
  $31 = ((($30)) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  _sqlite3ExprDelete($29,$32);
  $33 = HEAP32[$db>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3ExprDup($33,$35,1)|0);
  $37 = HEAP32[$pCol>>2]|0;
  $38 = ((($37)) + 4|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$db>>2]|0;
  $40 = HEAP32[$pCol>>2]|0;
  $41 = ((($40)) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  _sqlite3DbFree($39,$42);
  $43 = HEAP32[$db>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 8|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = $49;
  $54 = $52;
  $55 = (($53) - ($54))|0;
  $56 = ($55|0)<(0);
  $57 = $56 << 31 >> 31;
  $58 = (_sqlite3DbStrNDup($43,$46,$55,$57)|0);
  $59 = HEAP32[$pCol>>2]|0;
  $60 = ((($59)) + 8|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 } else {
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pCol>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$vararg_buffer>>2] = $28;
  _sqlite3ErrorMsg($26,30872,$vararg_buffer);
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 }
}
function _sqlite3PExpr($pParse,$op,$pLeft,$pRight,$pToken) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 HEAP32[$4>>2] = $pToken;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(72);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 68|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = (_sqlite3ExprAnd($12,$13,$14)|0);
   HEAP32[$p>>2] = $15;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = $18 & 255;
  $20 = HEAP32[$4>>2]|0;
  $21 = (_sqlite3ExprAlloc($17,$19,$20,1)|0);
  HEAP32[$p>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$3>>2]|0;
  _sqlite3ExprAttachSubtrees($23,$24,$25,$26);
 }
 $27 = HEAP32[$p>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $33 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 24|0);
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3ExprCheckHeight($29,$32)|0);
 $33 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _spanExpr($pOut,$pParse,$op,$pValue) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pValue;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_sqlite3PExpr($4,$5,0,0,$6)|0);
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$3>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($15)|0);
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 8|0);
 HEAP32[$20>>2] = $18;
 STACKTOP = sp;return;
}
function _sqlite3AddNotNull($pParse,$onError) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 488|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 34|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = ($10|0)<(1);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = $12&255;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 34|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = (($17) - 1)|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + (($18*24)|0)|0);
 $23 = ((($22)) + 20|0);
 HEAP8[$23>>0] = $13;
 STACKTOP = sp;return;
}
function _sqlite3AddPrimaryKey($pParse,$pList,$onError,$autoInc,$sortOrder) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $onError = $onError|0;
 $autoInc = $autoInc|0;
 $sortOrder = $sortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $iCol = 0, $nTerm = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pCExpr = 0, $pTab = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $zCName = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $pTab = sp + 40|0;
 $zType = sp + 36|0;
 $iCol = sp + 32|0;
 $i = sp + 28|0;
 $nTerm = sp + 24|0;
 $pCExpr = sp + 20|0;
 $zCName = sp + 16|0;
 $p = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $onError;
 HEAP32[$3>>2] = $autoInc;
 HEAP32[$4>>2] = $sortOrder;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 488|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTab>>2] = $7;
 HEAP32[$zType>>2] = 0;
 HEAP32[$iCol>>2] = -1;
 $8 = HEAP32[$pTab>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $157 = HEAP32[$0>>2]|0;
  $158 = HEAP32[$157>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($158,$159);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 454|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  $157 = HEAP32[$0>>2]|0;
  $158 = HEAP32[$157>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($158,$159);
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$pTab>>2]|0;
 $16 = ((($15)) + 42|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 & 4;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$pTab>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$vararg_buffer>>2] = $23;
  _sqlite3ErrorMsg($21,30917,$vararg_buffer);
  $157 = HEAP32[$0>>2]|0;
  $158 = HEAP32[$157>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($158,$159);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$pTab>>2]|0;
 $25 = ((($24)) + 42|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = $27 | 4;
 $29 = $28&255;
 HEAP8[$25>>0] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($30|0)==(0|0);
 L11: do {
  if ($31) {
   $32 = HEAP32[$pTab>>2]|0;
   $33 = ((($32)) + 34|0);
   $34 = HEAP16[$33>>1]|0;
   $35 = $34 << 16 >> 16;
   $36 = (($35) - 1)|0;
   HEAP32[$iCol>>2] = $36;
   $37 = HEAP32[$iCol>>2]|0;
   $38 = HEAP32[$pTab>>2]|0;
   $39 = ((($38)) + 4|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + (($37*24)|0)|0);
   $42 = ((($41)) + 23|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = $43&255;
   $45 = $44 | 1;
   $46 = $45&255;
   HEAP8[$42>>0] = $46;
   $47 = HEAP32[$iCol>>2]|0;
   $48 = HEAP32[$pTab>>2]|0;
   $49 = ((($48)) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + (($47*24)|0)|0);
   $52 = ((($51)) + 12|0);
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$zType>>2] = $53;
   HEAP32[$nTerm>>2] = 1;
  } else {
   $54 = HEAP32[$1>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   HEAP32[$nTerm>>2] = $55;
   HEAP32[$i>>2] = 0;
   while(1) {
    $56 = HEAP32[$i>>2]|0;
    $57 = HEAP32[$nTerm>>2]|0;
    $58 = ($56|0)<($57|0);
    if (!($58)) {
     break L11;
    }
    $59 = HEAP32[$i>>2]|0;
    $60 = HEAP32[$1>>2]|0;
    $61 = ((($60)) + 4|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + (($59*20)|0)|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (_sqlite3ExprSkipCollate($64)|0);
    HEAP32[$pCExpr>>2] = $65;
    $66 = HEAP32[$pCExpr>>2]|0;
    _sqlite3StringToId($66);
    $67 = HEAP32[$pCExpr>>2]|0;
    $68 = HEAP8[$67>>0]|0;
    $69 = $68&255;
    $70 = ($69|0)==(27);
    L17: do {
     if ($70) {
      $71 = HEAP32[$pCExpr>>2]|0;
      $72 = ((($71)) + 8|0);
      $73 = HEAP32[$72>>2]|0;
      HEAP32[$zCName>>2] = $73;
      HEAP32[$iCol>>2] = 0;
      while(1) {
       $74 = HEAP32[$iCol>>2]|0;
       $75 = HEAP32[$pTab>>2]|0;
       $76 = ((($75)) + 34|0);
       $77 = HEAP16[$76>>1]|0;
       $78 = $77 << 16 >> 16;
       $79 = ($74|0)<($78|0);
       if (!($79)) {
        break L17;
       }
       $80 = HEAP32[$zCName>>2]|0;
       $81 = HEAP32[$iCol>>2]|0;
       $82 = HEAP32[$pTab>>2]|0;
       $83 = ((($82)) + 4|0);
       $84 = HEAP32[$83>>2]|0;
       $85 = (($84) + (($81*24)|0)|0);
       $86 = HEAP32[$85>>2]|0;
       $87 = (_sqlite3_stricmp($80,$86)|0);
       $88 = ($87|0)==(0);
       $89 = HEAP32[$iCol>>2]|0;
       if ($88) {
        break;
       }
       $106 = (($89) + 1)|0;
       HEAP32[$iCol>>2] = $106;
      }
      $90 = HEAP32[$pTab>>2]|0;
      $91 = ((($90)) + 4|0);
      $92 = HEAP32[$91>>2]|0;
      $93 = (($92) + (($89*24)|0)|0);
      $94 = ((($93)) + 23|0);
      $95 = HEAP8[$94>>0]|0;
      $96 = $95&255;
      $97 = $96 | 1;
      $98 = $97&255;
      HEAP8[$94>>0] = $98;
      $99 = HEAP32[$iCol>>2]|0;
      $100 = HEAP32[$pTab>>2]|0;
      $101 = ((($100)) + 4|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) + (($99*24)|0)|0);
      $104 = ((($103)) + 12|0);
      $105 = HEAP32[$104>>2]|0;
      HEAP32[$zType>>2] = $105;
     }
    } while(0);
    $107 = HEAP32[$i>>2]|0;
    $108 = (($107) + 1)|0;
    HEAP32[$i>>2] = $108;
   }
  }
 } while(0);
 $109 = HEAP32[$nTerm>>2]|0;
 $110 = ($109|0)==(1);
 $111 = HEAP32[$zType>>2]|0;
 $112 = ($111|0)!=(0|0);
 $or$cond = $110 & $112;
 if ($or$cond) {
  $113 = HEAP32[$zType>>2]|0;
  $114 = (_sqlite3_stricmp($113,17889)|0);
  $115 = ($114|0)==(0);
  $116 = HEAP32[$4>>2]|0;
  $117 = ($116|0)!=(1);
  $or$cond3 = $115 & $117;
  if ($or$cond3) {
   $118 = HEAP32[$iCol>>2]|0;
   $119 = $118&65535;
   $120 = HEAP32[$pTab>>2]|0;
   $121 = ((($120)) + 32|0);
   HEAP16[$121>>1] = $119;
   $122 = HEAP32[$2>>2]|0;
   $123 = $122&255;
   $124 = HEAP32[$pTab>>2]|0;
   $125 = ((($124)) + 43|0);
   HEAP8[$125>>0] = $123;
   $126 = HEAP32[$3>>2]|0;
   $127 = $126<<3;
   $128 = HEAP32[$pTab>>2]|0;
   $129 = ((($128)) + 42|0);
   $130 = HEAP8[$129>>0]|0;
   $131 = $130&255;
   $132 = $131 | $127;
   $133 = $132&255;
   HEAP8[$129>>0] = $133;
   $134 = HEAP32[$1>>2]|0;
   $135 = ($134|0)!=(0|0);
   if (!($135)) {
    $157 = HEAP32[$0>>2]|0;
    $158 = HEAP32[$157>>2]|0;
    $159 = HEAP32[$1>>2]|0;
    _sqlite3ExprListDelete($158,$159);
    STACKTOP = sp;return;
   }
   $136 = HEAP32[$1>>2]|0;
   $137 = ((($136)) + 4|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = ((($138)) + 12|0);
   $140 = HEAP8[$139>>0]|0;
   $141 = HEAP32[$0>>2]|0;
   $142 = ((($141)) + 452|0);
   HEAP8[$142>>0] = $140;
   $157 = HEAP32[$0>>2]|0;
   $158 = HEAP32[$157>>2]|0;
   $159 = HEAP32[$1>>2]|0;
   _sqlite3ExprListDelete($158,$159);
   STACKTOP = sp;return;
  }
 }
 $143 = HEAP32[$3>>2]|0;
 $144 = ($143|0)!=(0);
 $145 = HEAP32[$0>>2]|0;
 if ($144) {
  _sqlite3ErrorMsg($145,30958,$vararg_buffer4);
  $157 = HEAP32[$0>>2]|0;
  $158 = HEAP32[$157>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($158,$159);
  STACKTOP = sp;return;
 }
 $146 = HEAP32[$1>>2]|0;
 $147 = HEAP32[$2>>2]|0;
 $148 = HEAP32[$4>>2]|0;
 $149 = (_sqlite3CreateIndex($145,0,0,0,$146,$147,0,0,$148,0)|0);
 HEAP32[$p>>2] = $149;
 $150 = HEAP32[$p>>2]|0;
 $151 = ($150|0)!=(0|0);
 if ($151) {
  $152 = HEAP32[$p>>2]|0;
  $153 = ((($152)) + 55|0);
  $154 = HEAP8[$153>>0]|0;
  $155 = $154 & -4;
  $156 = $155 | 2;
  HEAP8[$153>>0] = $156;
 }
 HEAP32[$1>>2] = 0;
 $157 = HEAP32[$0>>2]|0;
 $158 = HEAP32[$157>>2]|0;
 $159 = HEAP32[$1>>2]|0;
 _sqlite3ExprListDelete($158,$159);
 STACKTOP = sp;return;
}
function _sqlite3CreateIndex($pParse,$pName1,$pName2,$pTblName,$pList,$onError,$pStart,$pPIWhere,$sortOrder,$ifNotExist) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pTblName = $pTblName|0;
 $pList = $pList|0;
 $onError = $onError|0;
 $pStart = $pStart|0;
 $pPIWhere = $pPIWhere|0;
 $sortOrder = $sortOrder|0;
 $ifNotExist = $ifNotExist|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0;
 var $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0;
 var $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0;
 var $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0;
 var $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0;
 var $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0;
 var $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0;
 var $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0;
 var $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0;
 var $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0;
 var $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0;
 var $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $iMem = 0, $j = 0, $k = 0, $n = 0, $n1 = 0, $nColl = 0, $nExtra = 0, $nExtraCol = 0, $nName = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $p = 0;
 var $pCExpr = 0, $pCopy = 0, $pDb = 0, $pExpr = 0, $pIdx = 0, $pIndex = 0, $pListItem = 0, $pLoop = 0, $pName = 0, $pOther = 0, $pPk = 0, $pRet = 0, $pTab = 0, $prevCol = 0, $requestedSortOrder = 0, $sFix = 0, $sortOrderMask = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer11 = 0;
 var $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer23 = 0, $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer33 = 0, $vararg_buffer41 = 0, $vararg_buffer6 = 0, $vararg_buffer9 = 0, $vararg_ptr22 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr37 = 0, $vararg_ptr38 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, $x = 0, $z1 = 0;
 var $z2 = 0, $zColl = 0, $zDb = 0, $zExtra = 0, $zName = 0, $zStmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 352|0;
 $vararg_buffer41 = sp + 112|0;
 $vararg_buffer33 = sp + 88|0;
 $vararg_buffer28 = sp + 72|0;
 $vararg_buffer25 = sp + 64|0;
 $vararg_buffer23 = sp + 56|0;
 $vararg_buffer19 = sp + 48|0;
 $vararg_buffer16 = sp + 40|0;
 $vararg_buffer13 = sp + 32|0;
 $vararg_buffer11 = sp + 24|0;
 $vararg_buffer9 = sp + 16|0;
 $vararg_buffer6 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 336|0;
 $1 = sp + 332|0;
 $2 = sp + 328|0;
 $3 = sp + 324|0;
 $4 = sp + 320|0;
 $5 = sp + 316|0;
 $6 = sp + 312|0;
 $7 = sp + 308|0;
 $8 = sp + 304|0;
 $9 = sp + 300|0;
 $pRet = sp + 296|0;
 $pTab = sp + 292|0;
 $pIndex = sp + 288|0;
 $zName = sp + 284|0;
 $nName = sp + 280|0;
 $i = sp + 276|0;
 $j = sp + 272|0;
 $sFix = sp + 248|0;
 $sortOrderMask = sp + 240|0;
 $db = sp + 236|0;
 $pDb = sp + 232|0;
 $iDb = sp + 228|0;
 $pName = sp + 224|0;
 $pListItem = sp + 220|0;
 $nExtra = sp + 216|0;
 $nExtraCol = sp + 212|0;
 $zExtra = sp + 208|0;
 $pPk = sp + 204|0;
 $n = sp + 200|0;
 $pLoop = sp + 196|0;
 $zDb = sp + 192|0;
 $prevCol = sp + 184|0;
 $pExpr = sp + 180|0;
 $pCExpr = sp + 176|0;
 $requestedSortOrder = sp + 172|0;
 $zColl = sp + 168|0;
 $pCopy = sp + 164|0;
 $nColl = sp + 160|0;
 $x = sp + 156|0;
 $pIdx = sp + 152|0;
 $k = sp + 148|0;
 $z1 = sp + 144|0;
 $z2 = sp + 140|0;
 $p = sp + 136|0;
 $v = sp + 132|0;
 $zStmt = sp + 128|0;
 $iMem = sp + 124|0;
 $n1 = sp + 120|0;
 $pOther = sp + 116|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $pTblName;
 HEAP32[$4>>2] = $pList;
 HEAP32[$5>>2] = $onError;
 HEAP32[$6>>2] = $pStart;
 HEAP32[$7>>2] = $pPIWhere;
 HEAP32[$8>>2] = $sortOrder;
 HEAP32[$9>>2] = $ifNotExist;
 HEAP32[$pRet>>2] = 0;
 HEAP32[$pTab>>2] = 0;
 HEAP32[$pIndex>>2] = 0;
 HEAP32[$zName>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 HEAP32[$pName>>2] = 0;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$zExtra>>2] = 0;
 HEAP32[$pPk>>2] = 0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 65|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)!=(0);
 L1: do {
  if (!($16)) {
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 454|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = ($20|0)!=(0);
   if (!($21)) {
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 68|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($24|0)>(0);
    if (!($25)) {
     $26 = HEAP32[$0>>2]|0;
     $27 = (_sqlite3ReadSchema($26)|0);
     $28 = (0)!=($27|0);
     if (!($28)) {
      $29 = HEAP32[$3>>2]|0;
      $30 = ($29|0)!=(0|0);
      $31 = HEAP32[$0>>2]|0;
      if ($30) {
       $32 = HEAP32[$1>>2]|0;
       $33 = HEAP32[$2>>2]|0;
       $34 = (_sqlite3TwoPartName($31,$32,$33,$pName)|0);
       HEAP32[$iDb>>2] = $34;
       $35 = HEAP32[$iDb>>2]|0;
       $36 = ($35|0)<(0);
       if ($36) {
        break;
       }
       $37 = HEAP32[$db>>2]|0;
       $38 = ((($37)) + 140|0);
       $39 = ((($38)) + 5|0);
       $40 = HEAP8[$39>>0]|0;
       $41 = ($40<<24>>24)!=(0);
       if (!($41)) {
        $42 = HEAP32[$0>>2]|0;
        $43 = HEAP32[$3>>2]|0;
        $44 = (_sqlite3SrcListLookup($42,$43)|0);
        HEAP32[$pTab>>2] = $44;
        $45 = HEAP32[$2>>2]|0;
        $46 = ((($45)) + 4|0);
        $47 = HEAP32[$46>>2]|0;
        $48 = ($47|0)==(0);
        $49 = HEAP32[$pTab>>2]|0;
        $50 = ($49|0)!=(0|0);
        $or$cond = $48 & $50;
        if ($or$cond) {
         $51 = HEAP32[$pTab>>2]|0;
         $52 = ((($51)) + 64|0);
         $53 = HEAP32[$52>>2]|0;
         $54 = HEAP32[$db>>2]|0;
         $55 = ((($54)) + 16|0);
         $56 = HEAP32[$55>>2]|0;
         $57 = ((($56)) + 16|0);
         $58 = ((($57)) + 12|0);
         $59 = HEAP32[$58>>2]|0;
         $60 = ($53|0)==($59|0);
         if ($60) {
          HEAP32[$iDb>>2] = 1;
         }
        }
       }
       $61 = HEAP32[$0>>2]|0;
       $62 = HEAP32[$iDb>>2]|0;
       $63 = HEAP32[$pName>>2]|0;
       _sqlite3FixInit($sFix,$61,$62,31014,$63);
       $64 = HEAP32[$3>>2]|0;
       (_sqlite3FixSrcList($sFix,$64)|0);
       $65 = HEAP32[$0>>2]|0;
       $66 = HEAP32[$3>>2]|0;
       $67 = ((($66)) + 8|0);
       $68 = (_sqlite3LocateTableItem($65,0,$67)|0);
       HEAP32[$pTab>>2] = $68;
       $69 = HEAP32[$pTab>>2]|0;
       $70 = ($69|0)==(0|0);
       if ($70) {
        break;
       }
       $71 = HEAP32[$iDb>>2]|0;
       $72 = ($71|0)==(1);
       if ($72) {
        $73 = HEAP32[$iDb>>2]|0;
        $74 = HEAP32[$db>>2]|0;
        $75 = ((($74)) + 16|0);
        $76 = HEAP32[$75>>2]|0;
        $77 = (($76) + ($73<<4)|0);
        $78 = ((($77)) + 12|0);
        $79 = HEAP32[$78>>2]|0;
        $80 = HEAP32[$pTab>>2]|0;
        $81 = ((($80)) + 64|0);
        $82 = HEAP32[$81>>2]|0;
        $83 = ($79|0)!=($82|0);
        if ($83) {
         $84 = HEAP32[$0>>2]|0;
         $85 = HEAP32[$pTab>>2]|0;
         $86 = HEAP32[$85>>2]|0;
         HEAP32[$vararg_buffer>>2] = $86;
         _sqlite3ErrorMsg($84,31020,$vararg_buffer);
         break;
        }
       }
       $87 = HEAP32[$pTab>>2]|0;
       $88 = ((($87)) + 42|0);
       $89 = HEAP8[$88>>0]|0;
       $90 = $89&255;
       $91 = $90 & 32;
       $92 = ($91|0)==(0);
       if (!($92)) {
        $93 = HEAP32[$pTab>>2]|0;
        $94 = (_sqlite3PrimaryKeyIndex($93)|0);
        HEAP32[$pPk>>2] = $94;
       }
      } else {
       $95 = ((($31)) + 488|0);
       $96 = HEAP32[$95>>2]|0;
       HEAP32[$pTab>>2] = $96;
       $97 = HEAP32[$pTab>>2]|0;
       $98 = ($97|0)!=(0|0);
       if (!($98)) {
        break;
       }
       $99 = HEAP32[$db>>2]|0;
       $100 = HEAP32[$pTab>>2]|0;
       $101 = ((($100)) + 64|0);
       $102 = HEAP32[$101>>2]|0;
       $103 = (_sqlite3SchemaToIndex($99,$102)|0);
       HEAP32[$iDb>>2] = $103;
      }
      $104 = HEAP32[$iDb>>2]|0;
      $105 = HEAP32[$db>>2]|0;
      $106 = ((($105)) + 16|0);
      $107 = HEAP32[$106>>2]|0;
      $108 = (($107) + ($104<<4)|0);
      HEAP32[$pDb>>2] = $108;
      $109 = HEAP32[$pTab>>2]|0;
      $110 = HEAP32[$109>>2]|0;
      $111 = (_sqlite3_strnicmp($110,28844,7)|0);
      $112 = ($111|0)==(0);
      if ($112) {
       $113 = HEAP32[$db>>2]|0;
       $114 = ((($113)) + 140|0);
       $115 = ((($114)) + 5|0);
       $116 = HEAP8[$115>>0]|0;
       $117 = $116&255;
       $118 = ($117|0)==(0);
       if ($118) {
        $119 = HEAP32[$pTab>>2]|0;
        $120 = HEAP32[$119>>2]|0;
        $121 = ((($120)) + 7|0);
        $122 = (_sqlite3_strnicmp($121,31070,9)|0);
        $123 = ($122|0)!=(0);
        if ($123) {
         $124 = HEAP32[$0>>2]|0;
         $125 = HEAP32[$pTab>>2]|0;
         $126 = HEAP32[$125>>2]|0;
         HEAP32[$vararg_buffer6>>2] = $126;
         _sqlite3ErrorMsg($124,31080,$vararg_buffer6);
         break;
        }
       }
      }
      $127 = HEAP32[$pTab>>2]|0;
      $128 = ((($127)) + 12|0);
      $129 = HEAP32[$128>>2]|0;
      $130 = ($129|0)!=(0|0);
      if ($130) {
       $131 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($131,31108,$vararg_buffer9);
       break;
      }
      $132 = HEAP32[$pTab>>2]|0;
      $133 = ((($132)) + 42|0);
      $134 = HEAP8[$133>>0]|0;
      $135 = $134&255;
      $136 = $135 & 16;
      $137 = ($136|0)!=(0);
      if ($137) {
       $138 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($138,31133,$vararg_buffer11);
       break;
      }
      $139 = HEAP32[$pName>>2]|0;
      $140 = ($139|0)!=(0|0);
      if ($140) {
       $141 = HEAP32[$db>>2]|0;
       $142 = HEAP32[$pName>>2]|0;
       $143 = (_sqlite3NameFromToken($141,$142)|0);
       HEAP32[$zName>>2] = $143;
       $144 = HEAP32[$zName>>2]|0;
       $145 = ($144|0)==(0|0);
       if ($145) {
        break;
       }
       $146 = HEAP32[$0>>2]|0;
       $147 = HEAP32[$zName>>2]|0;
       $148 = (_sqlite3CheckObjectName($146,$147)|0);
       $149 = (0)!=($148|0);
       if ($149) {
        break;
       }
       $150 = HEAP32[$db>>2]|0;
       $151 = ((($150)) + 140|0);
       $152 = ((($151)) + 5|0);
       $153 = HEAP8[$152>>0]|0;
       $154 = ($153<<24>>24)!=(0);
       if (!($154)) {
        $155 = HEAP32[$db>>2]|0;
        $156 = HEAP32[$zName>>2]|0;
        $157 = (_sqlite3FindTable($155,$156,0)|0);
        $158 = ($157|0)!=(0|0);
        if ($158) {
         $159 = HEAP32[$0>>2]|0;
         $160 = HEAP32[$zName>>2]|0;
         HEAP32[$vararg_buffer13>>2] = $160;
         _sqlite3ErrorMsg($159,31167,$vararg_buffer13);
         break;
        }
       }
       $161 = HEAP32[$db>>2]|0;
       $162 = HEAP32[$zName>>2]|0;
       $163 = HEAP32[$pDb>>2]|0;
       $164 = HEAP32[$163>>2]|0;
       $165 = (_sqlite3FindIndex($161,$162,$164)|0);
       $166 = ($165|0)!=(0|0);
       if ($166) {
        $167 = HEAP32[$9>>2]|0;
        $168 = ($167|0)!=(0);
        $169 = HEAP32[$0>>2]|0;
        if ($168) {
         $171 = HEAP32[$iDb>>2]|0;
         _sqlite3CodeVerifySchema($169,$171);
         break;
        } else {
         $170 = HEAP32[$zName>>2]|0;
         HEAP32[$vararg_buffer16>>2] = $170;
         _sqlite3ErrorMsg($169,31201,$vararg_buffer16);
         break;
        }
       }
      } else {
       $172 = HEAP32[$pTab>>2]|0;
       $173 = ((($172)) + 8|0);
       $174 = HEAP32[$173>>2]|0;
       HEAP32[$pLoop>>2] = $174;
       HEAP32[$n>>2] = 1;
       while(1) {
        $175 = HEAP32[$pLoop>>2]|0;
        $176 = ($175|0)!=(0|0);
        if (!($176)) {
         break;
        }
        $177 = HEAP32[$pLoop>>2]|0;
        $178 = ((($177)) + 20|0);
        $179 = HEAP32[$178>>2]|0;
        HEAP32[$pLoop>>2] = $179;
        $180 = HEAP32[$n>>2]|0;
        $181 = (($180) + 1)|0;
        HEAP32[$n>>2] = $181;
       }
       $182 = HEAP32[$db>>2]|0;
       $183 = HEAP32[$pTab>>2]|0;
       $184 = HEAP32[$183>>2]|0;
       $185 = HEAP32[$n>>2]|0;
       HEAP32[$vararg_buffer19>>2] = $184;
       $vararg_ptr22 = ((($vararg_buffer19)) + 4|0);
       HEAP32[$vararg_ptr22>>2] = $185;
       $186 = (_sqlite3MPrintf($182,31225,$vararg_buffer19)|0);
       HEAP32[$zName>>2] = $186;
       $187 = HEAP32[$zName>>2]|0;
       $188 = ($187|0)==(0|0);
       if ($188) {
        break;
       }
      }
      $189 = HEAP32[$pDb>>2]|0;
      $190 = HEAP32[$189>>2]|0;
      HEAP32[$zDb>>2] = $190;
      $191 = HEAP32[$0>>2]|0;
      $192 = HEAP32[$iDb>>2]|0;
      $193 = ($192|0)==(1);
      $194 = $193 ? 21260 : 21279;
      $195 = HEAP32[$zDb>>2]|0;
      $196 = (_sqlite3AuthCheck($191,18,$194,0,$195)|0);
      $197 = ($196|0)!=(0);
      if (!($197)) {
       HEAP32[$i>>2] = 1;
       $198 = HEAP32[$iDb>>2]|0;
       $199 = ($198|0)==(1);
       $$ = $199 ? 3 : 1;
       HEAP32[$i>>2] = $$;
       $200 = HEAP32[$0>>2]|0;
       $201 = HEAP32[$i>>2]|0;
       $202 = HEAP32[$zName>>2]|0;
       $203 = HEAP32[$pTab>>2]|0;
       $204 = HEAP32[$203>>2]|0;
       $205 = HEAP32[$zDb>>2]|0;
       $206 = (_sqlite3AuthCheck($200,$201,$202,$204,$205)|0);
       $207 = ($206|0)!=(0);
       if (!($207)) {
        $208 = HEAP32[$4>>2]|0;
        $209 = ($208|0)==(0|0);
        if ($209) {
         $210 = HEAP32[$pTab>>2]|0;
         $211 = ((($210)) + 34|0);
         $212 = HEAP16[$211>>1]|0;
         $213 = $212 << 16 >> 16;
         $214 = (($213) - 1)|0;
         $215 = HEAP32[$pTab>>2]|0;
         $216 = ((($215)) + 4|0);
         $217 = HEAP32[$216>>2]|0;
         $218 = (($217) + (($214*24)|0)|0);
         $219 = HEAP32[$218>>2]|0;
         _sqlite3TokenInit($prevCol,$219);
         $220 = HEAP32[$0>>2]|0;
         $221 = HEAP32[$db>>2]|0;
         $222 = (_sqlite3ExprAlloc($221,27,$prevCol,0)|0);
         $223 = (_sqlite3ExprListAppend($220,0,$222)|0);
         HEAP32[$4>>2] = $223;
         $224 = HEAP32[$4>>2]|0;
         $225 = ($224|0)==(0|0);
         if ($225) {
          break;
         }
         $226 = HEAP32[$4>>2]|0;
         $227 = HEAP32[$8>>2]|0;
         _sqlite3ExprListSetSortOrder($226,$227);
        } else {
         $228 = HEAP32[$0>>2]|0;
         $229 = HEAP32[$4>>2]|0;
         _sqlite3ExprListCheckLength($228,$229,31014);
        }
        HEAP32[$i>>2] = 0;
        while(1) {
         $230 = HEAP32[$i>>2]|0;
         $231 = HEAP32[$4>>2]|0;
         $232 = HEAP32[$231>>2]|0;
         $233 = ($230|0)<($232|0);
         if (!($233)) {
          break;
         }
         $234 = HEAP32[$i>>2]|0;
         $235 = HEAP32[$4>>2]|0;
         $236 = ((($235)) + 4|0);
         $237 = HEAP32[$236>>2]|0;
         $238 = (($237) + (($234*20)|0)|0);
         $239 = HEAP32[$238>>2]|0;
         HEAP32[$pExpr>>2] = $239;
         $240 = HEAP32[$pExpr>>2]|0;
         $241 = HEAP8[$240>>0]|0;
         $242 = $241&255;
         $243 = ($242|0)==(95);
         if ($243) {
          $244 = HEAP32[$pExpr>>2]|0;
          $245 = ((($244)) + 8|0);
          $246 = HEAP32[$245>>2]|0;
          $247 = (_sqlite3Strlen30($246)|0);
          $248 = (1 + ($247))|0;
          $249 = HEAP32[$nExtra>>2]|0;
          $250 = (($249) + ($248))|0;
          HEAP32[$nExtra>>2] = $250;
         }
         $251 = HEAP32[$i>>2]|0;
         $252 = (($251) + 1)|0;
         HEAP32[$i>>2] = $252;
        }
        $253 = HEAP32[$zName>>2]|0;
        $254 = (_sqlite3Strlen30($253)|0);
        HEAP32[$nName>>2] = $254;
        $255 = HEAP32[$pPk>>2]|0;
        $256 = ($255|0)!=(0|0);
        if ($256) {
         $257 = HEAP32[$pPk>>2]|0;
         $258 = ((($257)) + 50|0);
         $259 = HEAP16[$258>>1]|0;
         $260 = $259&65535;
         $261 = $260;
        } else {
         $261 = 1;
        }
        HEAP32[$nExtraCol>>2] = $261;
        $262 = HEAP32[$db>>2]|0;
        $263 = HEAP32[$4>>2]|0;
        $264 = HEAP32[$263>>2]|0;
        $265 = HEAP32[$nExtraCol>>2]|0;
        $266 = (($264) + ($265))|0;
        $267 = $266&65535;
        $268 = HEAP32[$nName>>2]|0;
        $269 = HEAP32[$nExtra>>2]|0;
        $270 = (($268) + ($269))|0;
        $271 = (($270) + 1)|0;
        $272 = (_sqlite3AllocateIndexObject($262,$267,$271,$zExtra)|0);
        HEAP32[$pIndex>>2] = $272;
        $273 = HEAP32[$db>>2]|0;
        $274 = ((($273)) + 65|0);
        $275 = HEAP8[$274>>0]|0;
        $276 = ($275<<24>>24)!=(0);
        if (!($276)) {
         $277 = HEAP32[$zExtra>>2]|0;
         $278 = HEAP32[$pIndex>>2]|0;
         HEAP32[$278>>2] = $277;
         $279 = HEAP32[$nName>>2]|0;
         $280 = (($279) + 1)|0;
         $281 = HEAP32[$zExtra>>2]|0;
         $282 = (($281) + ($280)|0);
         HEAP32[$zExtra>>2] = $282;
         $283 = HEAP32[$pIndex>>2]|0;
         $284 = HEAP32[$283>>2]|0;
         $285 = HEAP32[$zName>>2]|0;
         $286 = HEAP32[$nName>>2]|0;
         $287 = (($286) + 1)|0;
         _memcpy(($284|0),($285|0),($287|0))|0;
         $288 = HEAP32[$pTab>>2]|0;
         $289 = HEAP32[$pIndex>>2]|0;
         $290 = ((($289)) + 12|0);
         HEAP32[$290>>2] = $288;
         $291 = HEAP32[$5>>2]|0;
         $292 = $291&255;
         $293 = HEAP32[$pIndex>>2]|0;
         $294 = ((($293)) + 54|0);
         HEAP8[$294>>0] = $292;
         $295 = HEAP32[$5>>2]|0;
         $296 = ($295|0)!=(0);
         $297 = $296&1;
         $298 = HEAP32[$pIndex>>2]|0;
         $299 = ((($298)) + 55|0);
         $300 = $297&255;
         $301 = HEAP8[$299>>0]|0;
         $302 = $300 & 1;
         $303 = ($302 << 3)&255;
         $304 = $301 & -9;
         $305 = $304 | $303;
         HEAP8[$299>>0] = $305;
         $306 = HEAP32[$pName>>2]|0;
         $307 = ($306|0)!=(0|0);
         $308 = $307 ? 0 : 1;
         $309 = HEAP32[$pIndex>>2]|0;
         $310 = ((($309)) + 55|0);
         $311 = $308&255;
         $312 = HEAP8[$310>>0]|0;
         $313 = $311 & 3;
         $314 = $312 & -4;
         $315 = $314 | $313;
         HEAP8[$310>>0] = $315;
         $316 = HEAP32[$iDb>>2]|0;
         $317 = HEAP32[$db>>2]|0;
         $318 = ((($317)) + 16|0);
         $319 = HEAP32[$318>>2]|0;
         $320 = (($319) + ($316<<4)|0);
         $321 = ((($320)) + 12|0);
         $322 = HEAP32[$321>>2]|0;
         $323 = HEAP32[$pIndex>>2]|0;
         $324 = ((($323)) + 24|0);
         HEAP32[$324>>2] = $322;
         $325 = HEAP32[$4>>2]|0;
         $326 = HEAP32[$325>>2]|0;
         $327 = $326&65535;
         $328 = HEAP32[$pIndex>>2]|0;
         $329 = ((($328)) + 50|0);
         HEAP16[$329>>1] = $327;
         $330 = HEAP32[$7>>2]|0;
         $331 = ($330|0)!=(0|0);
         if ($331) {
          $332 = HEAP32[$0>>2]|0;
          $333 = HEAP32[$pTab>>2]|0;
          $334 = HEAP32[$7>>2]|0;
          _sqlite3ResolveSelfReference($332,$333,16,$334,0);
          $335 = HEAP32[$7>>2]|0;
          $336 = HEAP32[$pIndex>>2]|0;
          $337 = ((($336)) + 36|0);
          HEAP32[$337>>2] = $335;
          HEAP32[$7>>2] = 0;
         }
         $338 = HEAP32[$pDb>>2]|0;
         $339 = ((($338)) + 12|0);
         $340 = HEAP32[$339>>2]|0;
         $341 = ((($340)) + 76|0);
         $342 = HEAP8[$341>>0]|0;
         $343 = $342&255;
         $344 = ($343|0)>=(4);
         if ($344) {
          HEAP32[$sortOrderMask>>2] = -1;
         } else {
          HEAP32[$sortOrderMask>>2] = 0;
         }
         HEAP32[$i>>2] = 0;
         $345 = HEAP32[$4>>2]|0;
         $346 = ((($345)) + 4|0);
         $347 = HEAP32[$346>>2]|0;
         HEAP32[$pListItem>>2] = $347;
         while(1) {
          $348 = HEAP32[$i>>2]|0;
          $349 = HEAP32[$4>>2]|0;
          $350 = HEAP32[$349>>2]|0;
          $351 = ($348|0)<($350|0);
          if (!($351)) {
           break;
          }
          $352 = HEAP32[$pListItem>>2]|0;
          $353 = HEAP32[$352>>2]|0;
          _sqlite3StringToId($353);
          $354 = HEAP32[$0>>2]|0;
          $355 = HEAP32[$pTab>>2]|0;
          $356 = HEAP32[$pListItem>>2]|0;
          $357 = HEAP32[$356>>2]|0;
          _sqlite3ResolveSelfReference($354,$355,32,$357,0);
          $358 = HEAP32[$0>>2]|0;
          $359 = ((($358)) + 68|0);
          $360 = HEAP32[$359>>2]|0;
          $361 = ($360|0)!=(0);
          if ($361) {
           break L1;
          }
          $362 = HEAP32[$pListItem>>2]|0;
          $363 = HEAP32[$362>>2]|0;
          $364 = (_sqlite3ExprSkipCollate($363)|0);
          HEAP32[$pCExpr>>2] = $364;
          $365 = HEAP32[$pCExpr>>2]|0;
          $366 = HEAP8[$365>>0]|0;
          $367 = $366&255;
          $368 = ($367|0)!=(152);
          if ($368) {
           $369 = HEAP32[$pTab>>2]|0;
           $370 = HEAP32[$0>>2]|0;
           $371 = ((($370)) + 488|0);
           $372 = HEAP32[$371>>2]|0;
           $373 = ($369|0)==($372|0);
           if ($373) {
            label = 65;
            break;
           }
           $375 = HEAP32[$pIndex>>2]|0;
           $376 = ((($375)) + 40|0);
           $377 = HEAP32[$376>>2]|0;
           $378 = ($377|0)==(0|0);
           do {
            if ($378) {
             $379 = HEAP32[$db>>2]|0;
             $380 = HEAP32[$4>>2]|0;
             $381 = (_sqlite3ExprListDup($379,$380,0)|0);
             HEAP32[$pCopy>>2] = $381;
             $382 = HEAP32[$pCopy>>2]|0;
             $383 = HEAP32[$pIndex>>2]|0;
             $384 = ((($383)) + 40|0);
             HEAP32[$384>>2] = $382;
             $385 = HEAP32[$db>>2]|0;
             $386 = ((($385)) + 65|0);
             $387 = HEAP8[$386>>0]|0;
             $388 = ($387<<24>>24)!=(0);
             if ($388) {
              break;
             }
             $389 = HEAP32[$i>>2]|0;
             $390 = HEAP32[$pCopy>>2]|0;
             $391 = ((($390)) + 4|0);
             $392 = HEAP32[$391>>2]|0;
             $393 = (($392) + (($389*20)|0)|0);
             HEAP32[$pListItem>>2] = $393;
            }
           } while(0);
           HEAP32[$j>>2] = -2;
           $394 = HEAP32[$i>>2]|0;
           $395 = HEAP32[$pIndex>>2]|0;
           $396 = ((($395)) + 4|0);
           $397 = HEAP32[$396>>2]|0;
           $398 = (($397) + ($394<<1)|0);
           HEAP16[$398>>1] = -2;
           $399 = HEAP32[$pIndex>>2]|0;
           $400 = ((($399)) + 55|0);
           $401 = HEAP8[$400>>0]|0;
           $402 = $401 & -9;
           HEAP8[$400>>0] = $402;
          } else {
           $403 = HEAP32[$pCExpr>>2]|0;
           $404 = ((($403)) + 32|0);
           $405 = HEAP16[$404>>1]|0;
           $406 = $405 << 16 >> 16;
           HEAP32[$j>>2] = $406;
           $407 = HEAP32[$j>>2]|0;
           $408 = ($407|0)<(0);
           do {
            if ($408) {
             $409 = HEAP32[$pTab>>2]|0;
             $410 = ((($409)) + 32|0);
             $411 = HEAP16[$410>>1]|0;
             $412 = $411 << 16 >> 16;
             HEAP32[$j>>2] = $412;
            } else {
             $413 = HEAP32[$j>>2]|0;
             $414 = HEAP32[$pTab>>2]|0;
             $415 = ((($414)) + 4|0);
             $416 = HEAP32[$415>>2]|0;
             $417 = (($416) + (($413*24)|0)|0);
             $418 = ((($417)) + 20|0);
             $419 = HEAP8[$418>>0]|0;
             $420 = $419&255;
             $421 = ($420|0)==(0);
             if (!($421)) {
              break;
             }
             $422 = HEAP32[$pIndex>>2]|0;
             $423 = ((($422)) + 55|0);
             $424 = HEAP8[$423>>0]|0;
             $425 = $424 & -9;
             HEAP8[$423>>0] = $425;
            }
           } while(0);
           $426 = HEAP32[$j>>2]|0;
           $427 = $426&65535;
           $428 = HEAP32[$i>>2]|0;
           $429 = HEAP32[$pIndex>>2]|0;
           $430 = ((($429)) + 4|0);
           $431 = HEAP32[$430>>2]|0;
           $432 = (($431) + ($428<<1)|0);
           HEAP16[$432>>1] = $427;
          }
          HEAP32[$zColl>>2] = 0;
          $433 = HEAP32[$pListItem>>2]|0;
          $434 = HEAP32[$433>>2]|0;
          $435 = HEAP8[$434>>0]|0;
          $436 = $435&255;
          $437 = ($436|0)==(95);
          do {
           if ($437) {
            $438 = HEAP32[$pListItem>>2]|0;
            $439 = HEAP32[$438>>2]|0;
            $440 = ((($439)) + 8|0);
            $441 = HEAP32[$440>>2]|0;
            HEAP32[$zColl>>2] = $441;
            $442 = HEAP32[$zColl>>2]|0;
            $443 = (_sqlite3Strlen30($442)|0);
            $444 = (($443) + 1)|0;
            HEAP32[$nColl>>2] = $444;
            $445 = HEAP32[$zExtra>>2]|0;
            $446 = HEAP32[$zColl>>2]|0;
            $447 = HEAP32[$nColl>>2]|0;
            _memcpy(($445|0),($446|0),($447|0))|0;
            $448 = HEAP32[$zExtra>>2]|0;
            HEAP32[$zColl>>2] = $448;
            $449 = HEAP32[$nColl>>2]|0;
            $450 = HEAP32[$zExtra>>2]|0;
            $451 = (($450) + ($449)|0);
            HEAP32[$zExtra>>2] = $451;
            $452 = HEAP32[$nColl>>2]|0;
            $453 = HEAP32[$nExtra>>2]|0;
            $454 = (($453) - ($452))|0;
            HEAP32[$nExtra>>2] = $454;
           } else {
            $455 = HEAP32[$j>>2]|0;
            $456 = ($455|0)>=(0);
            if (!($456)) {
             break;
            }
            $457 = HEAP32[$j>>2]|0;
            $458 = HEAP32[$pTab>>2]|0;
            $459 = ((($458)) + 4|0);
            $460 = HEAP32[$459>>2]|0;
            $461 = (($460) + (($457*24)|0)|0);
            $462 = ((($461)) + 16|0);
            $463 = HEAP32[$462>>2]|0;
            HEAP32[$zColl>>2] = $463;
           }
          } while(0);
          $464 = HEAP32[$zColl>>2]|0;
          $465 = ($464|0)!=(0|0);
          if (!($465)) {
           HEAP32[$zColl>>2] = 20716;
          }
          $466 = HEAP32[$db>>2]|0;
          $467 = ((($466)) + 140|0);
          $468 = ((($467)) + 5|0);
          $469 = HEAP8[$468>>0]|0;
          $470 = ($469<<24>>24)!=(0);
          if (!($470)) {
           $471 = HEAP32[$0>>2]|0;
           $472 = HEAP32[$zColl>>2]|0;
           $473 = (_sqlite3LocateCollSeq($471,$472)|0);
           $474 = ($473|0)!=(0|0);
           if (!($474)) {
            break L1;
           }
          }
          $475 = HEAP32[$zColl>>2]|0;
          $476 = HEAP32[$i>>2]|0;
          $477 = HEAP32[$pIndex>>2]|0;
          $478 = ((($477)) + 32|0);
          $479 = HEAP32[$478>>2]|0;
          $480 = (($479) + ($476<<2)|0);
          HEAP32[$480>>2] = $475;
          $481 = HEAP32[$pListItem>>2]|0;
          $482 = ((($481)) + 12|0);
          $483 = HEAP8[$482>>0]|0;
          $484 = $483&255;
          $485 = HEAP32[$sortOrderMask>>2]|0;
          $486 = $484 & $485;
          HEAP32[$requestedSortOrder>>2] = $486;
          $487 = HEAP32[$requestedSortOrder>>2]|0;
          $488 = $487&255;
          $489 = HEAP32[$i>>2]|0;
          $490 = HEAP32[$pIndex>>2]|0;
          $491 = ((($490)) + 28|0);
          $492 = HEAP32[$491>>2]|0;
          $493 = (($492) + ($489)|0);
          HEAP8[$493>>0] = $488;
          $494 = HEAP32[$i>>2]|0;
          $495 = (($494) + 1)|0;
          HEAP32[$i>>2] = $495;
          $496 = HEAP32[$pListItem>>2]|0;
          $497 = ((($496)) + 20|0);
          HEAP32[$pListItem>>2] = $497;
         }
         if ((label|0) == 65) {
          $374 = HEAP32[$0>>2]|0;
          _sqlite3ErrorMsg($374,31248,$vararg_buffer23);
          break;
         }
         $498 = HEAP32[$pPk>>2]|0;
         $499 = ($498|0)!=(0|0);
         L109: do {
          if ($499) {
           HEAP32[$j>>2] = 0;
           while(1) {
            $500 = HEAP32[$j>>2]|0;
            $501 = HEAP32[$pPk>>2]|0;
            $502 = ((($501)) + 50|0);
            $503 = HEAP16[$502>>1]|0;
            $504 = $503&65535;
            $505 = ($500|0)<($504|0);
            if (!($505)) {
             break L109;
            }
            $506 = HEAP32[$j>>2]|0;
            $507 = HEAP32[$pPk>>2]|0;
            $508 = ((($507)) + 4|0);
            $509 = HEAP32[$508>>2]|0;
            $510 = (($509) + ($506<<1)|0);
            $511 = HEAP16[$510>>1]|0;
            $512 = $511 << 16 >> 16;
            HEAP32[$x>>2] = $512;
            $513 = HEAP32[$pIndex>>2]|0;
            $514 = ((($513)) + 4|0);
            $515 = HEAP32[$514>>2]|0;
            $516 = HEAP32[$pIndex>>2]|0;
            $517 = ((($516)) + 50|0);
            $518 = HEAP16[$517>>1]|0;
            $519 = $518&65535;
            $520 = HEAP32[$x>>2]|0;
            $521 = (_hasColumn($515,$519,$520)|0);
            $522 = ($521|0)!=(0);
            if ($522) {
             $523 = HEAP32[$pIndex>>2]|0;
             $524 = ((($523)) + 52|0);
             $525 = HEAP16[$524>>1]|0;
             $526 = (($525) + -1)<<16>>16;
             HEAP16[$524>>1] = $526;
            } else {
             $527 = HEAP32[$x>>2]|0;
             $528 = $527&65535;
             $529 = HEAP32[$i>>2]|0;
             $530 = HEAP32[$pIndex>>2]|0;
             $531 = ((($530)) + 4|0);
             $532 = HEAP32[$531>>2]|0;
             $533 = (($532) + ($529<<1)|0);
             HEAP16[$533>>1] = $528;
             $534 = HEAP32[$j>>2]|0;
             $535 = HEAP32[$pPk>>2]|0;
             $536 = ((($535)) + 32|0);
             $537 = HEAP32[$536>>2]|0;
             $538 = (($537) + ($534<<2)|0);
             $539 = HEAP32[$538>>2]|0;
             $540 = HEAP32[$i>>2]|0;
             $541 = HEAP32[$pIndex>>2]|0;
             $542 = ((($541)) + 32|0);
             $543 = HEAP32[$542>>2]|0;
             $544 = (($543) + ($540<<2)|0);
             HEAP32[$544>>2] = $539;
             $545 = HEAP32[$j>>2]|0;
             $546 = HEAP32[$pPk>>2]|0;
             $547 = ((($546)) + 28|0);
             $548 = HEAP32[$547>>2]|0;
             $549 = (($548) + ($545)|0);
             $550 = HEAP8[$549>>0]|0;
             $551 = HEAP32[$i>>2]|0;
             $552 = HEAP32[$pIndex>>2]|0;
             $553 = ((($552)) + 28|0);
             $554 = HEAP32[$553>>2]|0;
             $555 = (($554) + ($551)|0);
             HEAP8[$555>>0] = $550;
             $556 = HEAP32[$i>>2]|0;
             $557 = (($556) + 1)|0;
             HEAP32[$i>>2] = $557;
            }
            $558 = HEAP32[$j>>2]|0;
            $559 = (($558) + 1)|0;
            HEAP32[$j>>2] = $559;
           }
          } else {
           $560 = HEAP32[$i>>2]|0;
           $561 = HEAP32[$pIndex>>2]|0;
           $562 = ((($561)) + 4|0);
           $563 = HEAP32[$562>>2]|0;
           $564 = (($563) + ($560<<1)|0);
           HEAP16[$564>>1] = -1;
           $565 = HEAP32[$i>>2]|0;
           $566 = HEAP32[$pIndex>>2]|0;
           $567 = ((($566)) + 32|0);
           $568 = HEAP32[$567>>2]|0;
           $569 = (($568) + ($565<<2)|0);
           HEAP32[$569>>2] = 20716;
          }
         } while(0);
         $570 = HEAP32[$pIndex>>2]|0;
         _sqlite3DefaultRowEst($570);
         $571 = HEAP32[$0>>2]|0;
         $572 = ((($571)) + 488|0);
         $573 = HEAP32[$572>>2]|0;
         $574 = ($573|0)==(0|0);
         if ($574) {
          $575 = HEAP32[$pIndex>>2]|0;
          _estimateIndexWidth($575);
         }
         $576 = HEAP32[$pTab>>2]|0;
         $577 = HEAP32[$0>>2]|0;
         $578 = ((($577)) + 488|0);
         $579 = HEAP32[$578>>2]|0;
         $580 = ($576|0)==($579|0);
         L123: do {
          if ($580) {
           $581 = HEAP32[$pTab>>2]|0;
           $582 = ((($581)) + 8|0);
           $583 = HEAP32[$582>>2]|0;
           HEAP32[$pIdx>>2] = $583;
           while(1) {
            $584 = HEAP32[$pIdx>>2]|0;
            $585 = ($584|0)!=(0|0);
            if (!($585)) {
             break L123;
            }
            $586 = HEAP32[$pIdx>>2]|0;
            $587 = ((($586)) + 50|0);
            $588 = HEAP16[$587>>1]|0;
            $589 = $588&65535;
            $590 = HEAP32[$pIndex>>2]|0;
            $591 = ((($590)) + 50|0);
            $592 = HEAP16[$591>>1]|0;
            $593 = $592&65535;
            $594 = ($589|0)!=($593|0);
            if (!($594)) {
             HEAP32[$k>>2] = 0;
             while(1) {
              $595 = HEAP32[$k>>2]|0;
              $596 = HEAP32[$pIdx>>2]|0;
              $597 = ((($596)) + 50|0);
              $598 = HEAP16[$597>>1]|0;
              $599 = $598&65535;
              $600 = ($595|0)<($599|0);
              if (!($600)) {
               break;
              }
              $601 = HEAP32[$k>>2]|0;
              $602 = HEAP32[$pIdx>>2]|0;
              $603 = ((($602)) + 4|0);
              $604 = HEAP32[$603>>2]|0;
              $605 = (($604) + ($601<<1)|0);
              $606 = HEAP16[$605>>1]|0;
              $607 = $606 << 16 >> 16;
              $608 = HEAP32[$k>>2]|0;
              $609 = HEAP32[$pIndex>>2]|0;
              $610 = ((($609)) + 4|0);
              $611 = HEAP32[$610>>2]|0;
              $612 = (($611) + ($608<<1)|0);
              $613 = HEAP16[$612>>1]|0;
              $614 = $613 << 16 >> 16;
              $615 = ($607|0)!=($614|0);
              if ($615) {
               break;
              }
              $616 = HEAP32[$k>>2]|0;
              $617 = HEAP32[$pIdx>>2]|0;
              $618 = ((($617)) + 32|0);
              $619 = HEAP32[$618>>2]|0;
              $620 = (($619) + ($616<<2)|0);
              $621 = HEAP32[$620>>2]|0;
              HEAP32[$z1>>2] = $621;
              $622 = HEAP32[$k>>2]|0;
              $623 = HEAP32[$pIndex>>2]|0;
              $624 = ((($623)) + 32|0);
              $625 = HEAP32[$624>>2]|0;
              $626 = (($625) + ($622<<2)|0);
              $627 = HEAP32[$626>>2]|0;
              HEAP32[$z2>>2] = $627;
              $628 = HEAP32[$z1>>2]|0;
              $629 = HEAP32[$z2>>2]|0;
              $630 = ($628|0)!=($629|0);
              if ($630) {
               $631 = HEAP32[$z1>>2]|0;
               $632 = HEAP32[$z2>>2]|0;
               $633 = (_sqlite3_stricmp($631,$632)|0);
               $634 = ($633|0)!=(0);
               if ($634) {
                break;
               }
              }
              $635 = HEAP32[$k>>2]|0;
              $636 = (($635) + 1)|0;
              HEAP32[$k>>2] = $636;
             }
             $637 = HEAP32[$k>>2]|0;
             $638 = HEAP32[$pIdx>>2]|0;
             $639 = ((($638)) + 50|0);
             $640 = HEAP16[$639>>1]|0;
             $641 = $640&65535;
             $642 = ($637|0)==($641|0);
             if ($642) {
              break;
             }
            }
            $674 = HEAP32[$pIdx>>2]|0;
            $675 = ((($674)) + 20|0);
            $676 = HEAP32[$675>>2]|0;
            HEAP32[$pIdx>>2] = $676;
           }
           $643 = HEAP32[$pIdx>>2]|0;
           $644 = ((($643)) + 54|0);
           $645 = HEAP8[$644>>0]|0;
           $646 = $645&255;
           $647 = HEAP32[$pIndex>>2]|0;
           $648 = ((($647)) + 54|0);
           $649 = HEAP8[$648>>0]|0;
           $650 = $649&255;
           $651 = ($646|0)!=($650|0);
           do {
            if ($651) {
             $652 = HEAP32[$pIdx>>2]|0;
             $653 = ((($652)) + 54|0);
             $654 = HEAP8[$653>>0]|0;
             $655 = $654&255;
             $656 = ($655|0)==(10);
             do {
              if (!($656)) {
               $657 = HEAP32[$pIndex>>2]|0;
               $658 = ((($657)) + 54|0);
               $659 = HEAP8[$658>>0]|0;
               $660 = $659&255;
               $661 = ($660|0)==(10);
               if ($661) {
                break;
               }
               $662 = HEAP32[$0>>2]|0;
               HEAP32[$vararg_buffer25>>2] = 0;
               _sqlite3ErrorMsg($662,31309,$vararg_buffer25);
              }
             } while(0);
             $663 = HEAP32[$pIdx>>2]|0;
             $664 = ((($663)) + 54|0);
             $665 = HEAP8[$664>>0]|0;
             $666 = $665&255;
             $667 = ($666|0)==(10);
             if (!($667)) {
              break;
             }
             $668 = HEAP32[$pIndex>>2]|0;
             $669 = ((($668)) + 54|0);
             $670 = HEAP8[$669>>0]|0;
             $671 = HEAP32[$pIdx>>2]|0;
             $672 = ((($671)) + 54|0);
             HEAP8[$672>>0] = $670;
            }
           } while(0);
           $673 = HEAP32[$pIdx>>2]|0;
           HEAP32[$pRet>>2] = $673;
           break L1;
          }
         } while(0);
         $677 = HEAP32[$db>>2]|0;
         $678 = ((($677)) + 140|0);
         $679 = ((($678)) + 5|0);
         $680 = HEAP8[$679>>0]|0;
         $681 = ($680<<24>>24)!=(0);
         do {
          if ($681) {
           $682 = HEAP32[$pIndex>>2]|0;
           $683 = ((($682)) + 24|0);
           $684 = HEAP32[$683>>2]|0;
           $685 = ((($684)) + 24|0);
           $686 = HEAP32[$pIndex>>2]|0;
           $687 = HEAP32[$686>>2]|0;
           $688 = HEAP32[$pIndex>>2]|0;
           $689 = (_sqlite3HashInsert($685,$687,$688)|0);
           HEAP32[$p>>2] = $689;
           $690 = HEAP32[$p>>2]|0;
           $691 = ($690|0)!=(0|0);
           $692 = HEAP32[$db>>2]|0;
           if ($691) {
            _sqlite3OomFault($692);
            break L1;
           }
           $693 = ((($692)) + 24|0);
           $694 = HEAP32[$693>>2]|0;
           $695 = $694 | 2;
           HEAP32[$693>>2] = $695;
           $696 = HEAP32[$3>>2]|0;
           $697 = ($696|0)!=(0|0);
           if (!($697)) {
            break;
           }
           $698 = HEAP32[$db>>2]|0;
           $699 = ((($698)) + 140|0);
           $700 = HEAP32[$699>>2]|0;
           $701 = HEAP32[$pIndex>>2]|0;
           $702 = ((($701)) + 44|0);
           HEAP32[$702>>2] = $700;
          } else {
           $703 = HEAP32[$pTab>>2]|0;
           $704 = ((($703)) + 42|0);
           $705 = HEAP8[$704>>0]|0;
           $706 = $705&255;
           $707 = $706 & 32;
           $708 = ($707|0)==(0);
           $709 = HEAP32[$3>>2]|0;
           $710 = ($709|0)!=(0|0);
           $or$cond3 = $708 | $710;
           if (!($or$cond3)) {
            break;
           }
           $711 = HEAP32[$0>>2]|0;
           $712 = ((($711)) + 76|0);
           $713 = HEAP32[$712>>2]|0;
           $714 = (($713) + 1)|0;
           HEAP32[$712>>2] = $714;
           HEAP32[$iMem>>2] = $714;
           $715 = HEAP32[$0>>2]|0;
           $716 = (_sqlite3GetVdbe($715)|0);
           HEAP32[$v>>2] = $716;
           $717 = HEAP32[$v>>2]|0;
           $718 = ($717|0)==(0|0);
           if ($718) {
            break L1;
           }
           $719 = HEAP32[$0>>2]|0;
           $720 = HEAP32[$iDb>>2]|0;
           _sqlite3BeginWriteOperation($719,1,$720);
           $721 = HEAP32[$v>>2]|0;
           $722 = (_sqlite3VdbeAddOp0($721,160)|0);
           $723 = HEAP32[$pIndex>>2]|0;
           $724 = ((($723)) + 44|0);
           HEAP32[$724>>2] = $722;
           $725 = HEAP32[$v>>2]|0;
           $726 = HEAP32[$iDb>>2]|0;
           $727 = HEAP32[$iMem>>2]|0;
           (_sqlite3VdbeAddOp2($725,121,$726,$727)|0);
           $728 = HEAP32[$6>>2]|0;
           $729 = ($728|0)!=(0|0);
           if ($729) {
            $730 = HEAP32[$0>>2]|0;
            $731 = ((($730)) + 508|0);
            $732 = HEAP32[$731>>2]|0;
            $733 = HEAP32[$pName>>2]|0;
            $734 = HEAP32[$733>>2]|0;
            $735 = $732;
            $736 = $734;
            $737 = (($735) - ($736))|0;
            $738 = HEAP32[$0>>2]|0;
            $739 = ((($738)) + 508|0);
            $740 = ((($739)) + 4|0);
            $741 = HEAP32[$740>>2]|0;
            $742 = (($737) + ($741))|0;
            HEAP32[$n1>>2] = $742;
            $743 = HEAP32[$n1>>2]|0;
            $744 = (($743) - 1)|0;
            $745 = HEAP32[$pName>>2]|0;
            $746 = HEAP32[$745>>2]|0;
            $747 = (($746) + ($744)|0);
            $748 = HEAP8[$747>>0]|0;
            $749 = $748 << 24 >> 24;
            $750 = ($749|0)==(59);
            if ($750) {
             $751 = HEAP32[$n1>>2]|0;
             $752 = (($751) + -1)|0;
             HEAP32[$n1>>2] = $752;
            }
            $753 = HEAP32[$db>>2]|0;
            $754 = HEAP32[$5>>2]|0;
            $755 = ($754|0)==(0);
            $756 = $755 ? 17532 : 31371;
            $757 = HEAP32[$n1>>2]|0;
            $758 = HEAP32[$pName>>2]|0;
            $759 = HEAP32[$758>>2]|0;
            HEAP32[$vararg_buffer28>>2] = $756;
            $vararg_ptr31 = ((($vararg_buffer28)) + 4|0);
            HEAP32[$vararg_ptr31>>2] = $757;
            $vararg_ptr32 = ((($vararg_buffer28)) + 8|0);
            HEAP32[$vararg_ptr32>>2] = $759;
            $760 = (_sqlite3MPrintf($753,31351,$vararg_buffer28)|0);
            HEAP32[$zStmt>>2] = $760;
           } else {
            HEAP32[$zStmt>>2] = 0;
           }
           $761 = HEAP32[$0>>2]|0;
           $762 = HEAP32[$iDb>>2]|0;
           $763 = HEAP32[$db>>2]|0;
           $764 = ((($763)) + 16|0);
           $765 = HEAP32[$764>>2]|0;
           $766 = (($765) + ($762<<4)|0);
           $767 = HEAP32[$766>>2]|0;
           $768 = HEAP32[$iDb>>2]|0;
           $769 = ($768|0)==(1);
           $770 = $769 ? 21260 : 21279;
           $771 = HEAP32[$pIndex>>2]|0;
           $772 = HEAP32[$771>>2]|0;
           $773 = HEAP32[$pTab>>2]|0;
           $774 = HEAP32[$773>>2]|0;
           $775 = HEAP32[$iMem>>2]|0;
           $776 = HEAP32[$zStmt>>2]|0;
           HEAP32[$vararg_buffer33>>2] = $767;
           $vararg_ptr36 = ((($vararg_buffer33)) + 4|0);
           HEAP32[$vararg_ptr36>>2] = $770;
           $vararg_ptr37 = ((($vararg_buffer33)) + 8|0);
           HEAP32[$vararg_ptr37>>2] = $772;
           $vararg_ptr38 = ((($vararg_buffer33)) + 12|0);
           HEAP32[$vararg_ptr38>>2] = $774;
           $vararg_ptr39 = ((($vararg_buffer33)) + 16|0);
           HEAP32[$vararg_ptr39>>2] = $775;
           $vararg_ptr40 = ((($vararg_buffer33)) + 20|0);
           HEAP32[$vararg_ptr40>>2] = $776;
           _sqlite3NestedParse($761,31379,$vararg_buffer33);
           $777 = HEAP32[$db>>2]|0;
           $778 = HEAP32[$zStmt>>2]|0;
           _sqlite3DbFree($777,$778);
           $779 = HEAP32[$3>>2]|0;
           $780 = ($779|0)!=(0|0);
           if ($780) {
            $781 = HEAP32[$0>>2]|0;
            $782 = HEAP32[$pIndex>>2]|0;
            $783 = HEAP32[$iMem>>2]|0;
            _sqlite3RefillIndex($781,$782,$783);
            $784 = HEAP32[$0>>2]|0;
            $785 = HEAP32[$iDb>>2]|0;
            _sqlite3ChangeCookie($784,$785);
            $786 = HEAP32[$v>>2]|0;
            $787 = HEAP32[$iDb>>2]|0;
            $788 = HEAP32[$db>>2]|0;
            $789 = HEAP32[$pIndex>>2]|0;
            $790 = HEAP32[$789>>2]|0;
            HEAP32[$vararg_buffer41>>2] = $790;
            $791 = (_sqlite3MPrintf($788,31427,$vararg_buffer41)|0);
            _sqlite3VdbeAddParseSchemaOp($786,$787,$791);
            $792 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp1($792,147,0)|0);
           }
           $793 = HEAP32[$v>>2]|0;
           $794 = HEAP32[$pIndex>>2]|0;
           $795 = ((($794)) + 44|0);
           $796 = HEAP32[$795>>2]|0;
           _sqlite3VdbeJumpHere($793,$796);
          }
         } while(0);
         $797 = HEAP32[$db>>2]|0;
         $798 = ((($797)) + 140|0);
         $799 = ((($798)) + 5|0);
         $800 = HEAP8[$799>>0]|0;
         $801 = $800&255;
         $802 = ($801|0)!=(0);
         $803 = HEAP32[$3>>2]|0;
         $804 = ($803|0)==(0|0);
         $or$cond5 = $802 | $804;
         if (!($or$cond5)) {
          break;
         }
         $805 = HEAP32[$5>>2]|0;
         $806 = ($805|0)!=(5);
         do {
          if ($806) {
           label = 132;
          } else {
           $807 = HEAP32[$pTab>>2]|0;
           $808 = ((($807)) + 8|0);
           $809 = HEAP32[$808>>2]|0;
           $810 = ($809|0)==(0|0);
           if ($810) {
            label = 132;
            break;
           }
           $811 = HEAP32[$pTab>>2]|0;
           $812 = ((($811)) + 8|0);
           $813 = HEAP32[$812>>2]|0;
           $814 = ((($813)) + 54|0);
           $815 = HEAP8[$814>>0]|0;
           $816 = $815&255;
           $817 = ($816|0)==(5);
           if ($817) {
            label = 132;
            break;
           }
           $826 = HEAP32[$pTab>>2]|0;
           $827 = ((($826)) + 8|0);
           $828 = HEAP32[$827>>2]|0;
           HEAP32[$pOther>>2] = $828;
           while(1) {
            $829 = HEAP32[$pOther>>2]|0;
            $830 = ((($829)) + 20|0);
            $831 = HEAP32[$830>>2]|0;
            $832 = ($831|0)!=(0|0);
            if ($832) {
             $833 = HEAP32[$pOther>>2]|0;
             $834 = ((($833)) + 20|0);
             $835 = HEAP32[$834>>2]|0;
             $836 = ((($835)) + 54|0);
             $837 = HEAP8[$836>>0]|0;
             $838 = $837&255;
             $839 = ($838|0)!=(5);
             $862 = $839;
            } else {
             $862 = 0;
            }
            $840 = HEAP32[$pOther>>2]|0;
            $841 = ((($840)) + 20|0);
            $842 = HEAP32[$841>>2]|0;
            if (!($862)) {
             break;
            }
            HEAP32[$pOther>>2] = $842;
           }
           $843 = HEAP32[$pIndex>>2]|0;
           $844 = ((($843)) + 20|0);
           HEAP32[$844>>2] = $842;
           $845 = HEAP32[$pIndex>>2]|0;
           $846 = HEAP32[$pOther>>2]|0;
           $847 = ((($846)) + 20|0);
           HEAP32[$847>>2] = $845;
          }
         } while(0);
         if ((label|0) == 132) {
          $818 = HEAP32[$pTab>>2]|0;
          $819 = ((($818)) + 8|0);
          $820 = HEAP32[$819>>2]|0;
          $821 = HEAP32[$pIndex>>2]|0;
          $822 = ((($821)) + 20|0);
          HEAP32[$822>>2] = $820;
          $823 = HEAP32[$pIndex>>2]|0;
          $824 = HEAP32[$pTab>>2]|0;
          $825 = ((($824)) + 8|0);
          HEAP32[$825>>2] = $823;
         }
         $848 = HEAP32[$pIndex>>2]|0;
         HEAP32[$pRet>>2] = $848;
         HEAP32[$pIndex>>2] = 0;
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $849 = HEAP32[$pIndex>>2]|0;
 $850 = ($849|0)!=(0|0);
 if (!($850)) {
  $853 = HEAP32[$db>>2]|0;
  $854 = HEAP32[$7>>2]|0;
  _sqlite3ExprDelete($853,$854);
  $855 = HEAP32[$db>>2]|0;
  $856 = HEAP32[$4>>2]|0;
  _sqlite3ExprListDelete($855,$856);
  $857 = HEAP32[$db>>2]|0;
  $858 = HEAP32[$3>>2]|0;
  _sqlite3SrcListDelete($857,$858);
  $859 = HEAP32[$db>>2]|0;
  $860 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($859,$860);
  $861 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($861|0);
 }
 $851 = HEAP32[$db>>2]|0;
 $852 = HEAP32[$pIndex>>2]|0;
 _freeIndex($851,$852);
 $853 = HEAP32[$db>>2]|0;
 $854 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($853,$854);
 $855 = HEAP32[$db>>2]|0;
 $856 = HEAP32[$4>>2]|0;
 _sqlite3ExprListDelete($855,$856);
 $857 = HEAP32[$db>>2]|0;
 $858 = HEAP32[$3>>2]|0;
 _sqlite3SrcListDelete($857,$858);
 $859 = HEAP32[$db>>2]|0;
 $860 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($859,$860);
 $861 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($861|0);
}
function _sqlite3AddCheckConstraint($pParse,$pCheckExpr) {
 $pParse = $pParse|0;
 $pCheckExpr = $pCheckExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pTab = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pCheckExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 488|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 454|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  if (!($12)) {
   $13 = HEAP32[$db>>2]|0;
   $14 = ((($13)) + 140|0);
   $15 = ((($14)) + 4|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   $18 = HEAP32[$db>>2]|0;
   $19 = ((($18)) + 16|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + ($17<<4)|0);
   $22 = ((($21)) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3BtreeIsReadonly($23)|0);
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$0>>2]|0;
    $27 = HEAP32[$pTab>>2]|0;
    $28 = ((($27)) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = (_sqlite3ExprListAppend($26,$29,$30)|0);
    $32 = HEAP32[$pTab>>2]|0;
    $33 = ((($32)) + 24|0);
    HEAP32[$33>>2] = $31;
    $34 = HEAP32[$0>>2]|0;
    $35 = ((($34)) + 328|0);
    $36 = ((($35)) + 4|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)!=(0);
    if (!($38)) {
     STACKTOP = sp;return;
    }
    $39 = HEAP32[$0>>2]|0;
    $40 = HEAP32[$pTab>>2]|0;
    $41 = ((($40)) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$0>>2]|0;
    $44 = ((($43)) + 328|0);
    _sqlite3ExprListSetName($39,$42,$44,1);
    STACKTOP = sp;return;
   }
  }
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 _sqlite3ExprDelete($46,$47);
 STACKTOP = sp;return;
}
function _sqlite3CreateForeignKey($pParse,$pFromCol,$pTo,$pToCol,$flags) {
 $pParse = $pParse|0;
 $pFromCol = $pFromCol|0;
 $pTo = $pTo|0;
 $pToCol = $pToCol|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $j = 0, $n = 0;
 var $nByte = 0, $nCol = 0, $p = 0, $pFKey = 0, $pNextTo = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $db = sp + 60|0;
 $pFKey = sp + 56|0;
 $pNextTo = sp + 52|0;
 $p = sp + 48|0;
 $nByte = sp + 44|0;
 $i = sp + 40|0;
 $nCol = sp + 36|0;
 $z = sp + 32|0;
 $iCol = sp + 28|0;
 $j = sp + 24|0;
 $n = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pFromCol;
 HEAP32[$2>>2] = $pTo;
 HEAP32[$3>>2] = $pToCol;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP32[$pFKey>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 488|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)==(0|0);
 L1: do {
  if (!($11)) {
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 454|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $17 = HEAP32[$1>>2]|0;
    $18 = ($17|0)==(0|0);
    if ($18) {
     $19 = HEAP32[$p>>2]|0;
     $20 = ((($19)) + 34|0);
     $21 = HEAP16[$20>>1]|0;
     $22 = $21 << 16 >> 16;
     $23 = (($22) - 1)|0;
     HEAP32[$iCol>>2] = $23;
     $24 = HEAP32[$iCol>>2]|0;
     $25 = ($24|0)<(0);
     if ($25) {
      break;
     }
     $26 = HEAP32[$3>>2]|0;
     $27 = ($26|0)!=(0|0);
     if ($27) {
      $28 = HEAP32[$3>>2]|0;
      $29 = HEAP32[$28>>2]|0;
      $30 = ($29|0)!=(1);
      if ($30) {
       $31 = HEAP32[$0>>2]|0;
       $32 = HEAP32[$iCol>>2]|0;
       $33 = HEAP32[$p>>2]|0;
       $34 = ((($33)) + 4|0);
       $35 = HEAP32[$34>>2]|0;
       $36 = (($35) + (($32*24)|0)|0);
       $37 = HEAP32[$36>>2]|0;
       $38 = HEAP32[$2>>2]|0;
       HEAP32[$vararg_buffer>>2] = $37;
       $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
       HEAP32[$vararg_ptr1>>2] = $38;
       _sqlite3ErrorMsg($31,31538,$vararg_buffer);
       break;
      }
     }
     HEAP32[$nCol>>2] = 1;
    } else {
     $39 = HEAP32[$3>>2]|0;
     $40 = ($39|0)!=(0|0);
     if ($40) {
      $41 = HEAP32[$3>>2]|0;
      $42 = HEAP32[$41>>2]|0;
      $43 = HEAP32[$1>>2]|0;
      $44 = HEAP32[$43>>2]|0;
      $45 = ($42|0)!=($44|0);
      if ($45) {
       $46 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($46,31601,$vararg_buffer2);
       break;
      }
     }
     $47 = HEAP32[$1>>2]|0;
     $48 = HEAP32[$47>>2]|0;
     HEAP32[$nCol>>2] = $48;
    }
    $49 = HEAP32[$nCol>>2]|0;
    $50 = (($49) - 1)|0;
    $51 = $50<<3;
    $52 = (44 + ($51))|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = ((($53)) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($52) + ($55))|0;
    $57 = (($56) + 1)|0;
    HEAP32[$nByte>>2] = $57;
    $58 = HEAP32[$3>>2]|0;
    $59 = ($58|0)!=(0|0);
    L17: do {
     if ($59) {
      HEAP32[$i>>2] = 0;
      while(1) {
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$3>>2]|0;
       $62 = HEAP32[$61>>2]|0;
       $63 = ($60|0)<($62|0);
       if (!($63)) {
        break L17;
       }
       $64 = HEAP32[$i>>2]|0;
       $65 = HEAP32[$3>>2]|0;
       $66 = ((($65)) + 4|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = (($67) + (($64*20)|0)|0);
       $69 = ((($68)) + 4|0);
       $70 = HEAP32[$69>>2]|0;
       $71 = (_sqlite3Strlen30($70)|0);
       $72 = (($71) + 1)|0;
       $73 = HEAP32[$nByte>>2]|0;
       $74 = (($73) + ($72))|0;
       HEAP32[$nByte>>2] = $74;
       $75 = HEAP32[$i>>2]|0;
       $76 = (($75) + 1)|0;
       HEAP32[$i>>2] = $76;
      }
     }
    } while(0);
    $77 = HEAP32[$db>>2]|0;
    $78 = HEAP32[$nByte>>2]|0;
    $79 = ($78|0)<(0);
    $80 = $79 << 31 >> 31;
    $81 = (_sqlite3DbMallocZero($77,$78,$80)|0);
    HEAP32[$pFKey>>2] = $81;
    $82 = HEAP32[$pFKey>>2]|0;
    $83 = ($82|0)==(0|0);
    if (!($83)) {
     $84 = HEAP32[$p>>2]|0;
     $85 = HEAP32[$pFKey>>2]|0;
     HEAP32[$85>>2] = $84;
     $86 = HEAP32[$p>>2]|0;
     $87 = ((($86)) + 16|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = HEAP32[$pFKey>>2]|0;
     $90 = ((($89)) + 4|0);
     HEAP32[$90>>2] = $88;
     $91 = HEAP32[$nCol>>2]|0;
     $92 = HEAP32[$pFKey>>2]|0;
     $93 = ((($92)) + 36|0);
     $94 = (($93) + ($91<<3)|0);
     HEAP32[$z>>2] = $94;
     $95 = HEAP32[$z>>2]|0;
     $96 = HEAP32[$pFKey>>2]|0;
     $97 = ((($96)) + 8|0);
     HEAP32[$97>>2] = $95;
     $98 = HEAP32[$z>>2]|0;
     $99 = HEAP32[$2>>2]|0;
     $100 = HEAP32[$99>>2]|0;
     $101 = HEAP32[$2>>2]|0;
     $102 = ((($101)) + 4|0);
     $103 = HEAP32[$102>>2]|0;
     _memcpy(($98|0),($100|0),($103|0))|0;
     $104 = HEAP32[$2>>2]|0;
     $105 = ((($104)) + 4|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = HEAP32[$z>>2]|0;
     $108 = (($107) + ($106)|0);
     HEAP8[$108>>0] = 0;
     $109 = HEAP32[$z>>2]|0;
     (_sqlite3Dequote($109)|0);
     $110 = HEAP32[$2>>2]|0;
     $111 = ((($110)) + 4|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (($112) + 1)|0;
     $114 = HEAP32[$z>>2]|0;
     $115 = (($114) + ($113)|0);
     HEAP32[$z>>2] = $115;
     $116 = HEAP32[$nCol>>2]|0;
     $117 = HEAP32[$pFKey>>2]|0;
     $118 = ((($117)) + 20|0);
     HEAP32[$118>>2] = $116;
     $119 = HEAP32[$1>>2]|0;
     $120 = ($119|0)==(0|0);
     L24: do {
      if ($120) {
       $121 = HEAP32[$p>>2]|0;
       $122 = ((($121)) + 34|0);
       $123 = HEAP16[$122>>1]|0;
       $124 = $123 << 16 >> 16;
       $125 = (($124) - 1)|0;
       $126 = HEAP32[$pFKey>>2]|0;
       $127 = ((($126)) + 36|0);
       HEAP32[$127>>2] = $125;
      } else {
       HEAP32[$i>>2] = 0;
       while(1) {
        $128 = HEAP32[$i>>2]|0;
        $129 = HEAP32[$nCol>>2]|0;
        $130 = ($128|0)<($129|0);
        if (!($130)) {
         break L24;
        }
        HEAP32[$j>>2] = 0;
        while(1) {
         $131 = HEAP32[$j>>2]|0;
         $132 = HEAP32[$p>>2]|0;
         $133 = ((($132)) + 34|0);
         $134 = HEAP16[$133>>1]|0;
         $135 = $134 << 16 >> 16;
         $136 = ($131|0)<($135|0);
         if (!($136)) {
          break;
         }
         $137 = HEAP32[$j>>2]|0;
         $138 = HEAP32[$p>>2]|0;
         $139 = ((($138)) + 4|0);
         $140 = HEAP32[$139>>2]|0;
         $141 = (($140) + (($137*24)|0)|0);
         $142 = HEAP32[$141>>2]|0;
         $143 = HEAP32[$i>>2]|0;
         $144 = HEAP32[$1>>2]|0;
         $145 = ((($144)) + 4|0);
         $146 = HEAP32[$145>>2]|0;
         $147 = (($146) + (($143*20)|0)|0);
         $148 = ((($147)) + 4|0);
         $149 = HEAP32[$148>>2]|0;
         $150 = (_sqlite3_stricmp($142,$149)|0);
         $151 = ($150|0)==(0);
         $152 = HEAP32[$j>>2]|0;
         if ($151) {
          label = 25;
          break;
         }
         $157 = (($152) + 1)|0;
         HEAP32[$j>>2] = $157;
        }
        if ((label|0) == 25) {
         label = 0;
         $153 = HEAP32[$i>>2]|0;
         $154 = HEAP32[$pFKey>>2]|0;
         $155 = ((($154)) + 36|0);
         $156 = (($155) + ($153<<3)|0);
         HEAP32[$156>>2] = $152;
        }
        $158 = HEAP32[$j>>2]|0;
        $159 = HEAP32[$p>>2]|0;
        $160 = ((($159)) + 34|0);
        $161 = HEAP16[$160>>1]|0;
        $162 = $161 << 16 >> 16;
        $163 = ($158|0)>=($162|0);
        if ($163) {
         break;
        }
        $172 = HEAP32[$i>>2]|0;
        $173 = (($172) + 1)|0;
        HEAP32[$i>>2] = $173;
       }
       $164 = HEAP32[$0>>2]|0;
       $165 = HEAP32[$i>>2]|0;
       $166 = HEAP32[$1>>2]|0;
       $167 = ((($166)) + 4|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = (($168) + (($165*20)|0)|0);
       $170 = ((($169)) + 4|0);
       $171 = HEAP32[$170>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $171;
       _sqlite3ErrorMsg($164,31695,$vararg_buffer4);
       break L1;
      }
     } while(0);
     $174 = HEAP32[$3>>2]|0;
     $175 = ($174|0)!=(0|0);
     L40: do {
      if ($175) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $176 = HEAP32[$i>>2]|0;
        $177 = HEAP32[$nCol>>2]|0;
        $178 = ($176|0)<($177|0);
        if (!($178)) {
         break L40;
        }
        $179 = HEAP32[$i>>2]|0;
        $180 = HEAP32[$3>>2]|0;
        $181 = ((($180)) + 4|0);
        $182 = HEAP32[$181>>2]|0;
        $183 = (($182) + (($179*20)|0)|0);
        $184 = ((($183)) + 4|0);
        $185 = HEAP32[$184>>2]|0;
        $186 = (_sqlite3Strlen30($185)|0);
        HEAP32[$n>>2] = $186;
        $187 = HEAP32[$z>>2]|0;
        $188 = HEAP32[$i>>2]|0;
        $189 = HEAP32[$pFKey>>2]|0;
        $190 = ((($189)) + 36|0);
        $191 = (($190) + ($188<<3)|0);
        $192 = ((($191)) + 4|0);
        HEAP32[$192>>2] = $187;
        $193 = HEAP32[$z>>2]|0;
        $194 = HEAP32[$i>>2]|0;
        $195 = HEAP32[$3>>2]|0;
        $196 = ((($195)) + 4|0);
        $197 = HEAP32[$196>>2]|0;
        $198 = (($197) + (($194*20)|0)|0);
        $199 = ((($198)) + 4|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = HEAP32[$n>>2]|0;
        _memcpy(($193|0),($200|0),($201|0))|0;
        $202 = HEAP32[$n>>2]|0;
        $203 = HEAP32[$z>>2]|0;
        $204 = (($203) + ($202)|0);
        HEAP8[$204>>0] = 0;
        $205 = HEAP32[$n>>2]|0;
        $206 = (($205) + 1)|0;
        $207 = HEAP32[$z>>2]|0;
        $208 = (($207) + ($206)|0);
        HEAP32[$z>>2] = $208;
        $209 = HEAP32[$i>>2]|0;
        $210 = (($209) + 1)|0;
        HEAP32[$i>>2] = $210;
       }
      }
     } while(0);
     $211 = HEAP32[$pFKey>>2]|0;
     $212 = ((($211)) + 24|0);
     HEAP8[$212>>0] = 0;
     $213 = HEAP32[$4>>2]|0;
     $214 = $213 & 255;
     $215 = $214&255;
     $216 = HEAP32[$pFKey>>2]|0;
     $217 = ((($216)) + 25|0);
     HEAP8[$217>>0] = $215;
     $218 = HEAP32[$4>>2]|0;
     $219 = $218 >> 8;
     $220 = $219 & 255;
     $221 = $220&255;
     $222 = HEAP32[$pFKey>>2]|0;
     $223 = ((($222)) + 25|0);
     $224 = ((($223)) + 1|0);
     HEAP8[$224>>0] = $221;
     $225 = HEAP32[$p>>2]|0;
     $226 = ((($225)) + 64|0);
     $227 = HEAP32[$226>>2]|0;
     $228 = ((($227)) + 56|0);
     $229 = HEAP32[$pFKey>>2]|0;
     $230 = ((($229)) + 8|0);
     $231 = HEAP32[$230>>2]|0;
     $232 = HEAP32[$pFKey>>2]|0;
     $233 = (_sqlite3HashInsert($228,$231,$232)|0);
     HEAP32[$pNextTo>>2] = $233;
     $234 = HEAP32[$pNextTo>>2]|0;
     $235 = HEAP32[$pFKey>>2]|0;
     $236 = ($234|0)==($235|0);
     if ($236) {
      $237 = HEAP32[$db>>2]|0;
      _sqlite3OomFault($237);
      break;
     }
     $238 = HEAP32[$pNextTo>>2]|0;
     $239 = ($238|0)!=(0|0);
     if ($239) {
      $240 = HEAP32[$pNextTo>>2]|0;
      $241 = HEAP32[$pFKey>>2]|0;
      $242 = ((($241)) + 12|0);
      HEAP32[$242>>2] = $240;
      $243 = HEAP32[$pFKey>>2]|0;
      $244 = HEAP32[$pNextTo>>2]|0;
      $245 = ((($244)) + 16|0);
      HEAP32[$245>>2] = $243;
     }
     $246 = HEAP32[$pFKey>>2]|0;
     $247 = HEAP32[$p>>2]|0;
     $248 = ((($247)) + 16|0);
     HEAP32[$248>>2] = $246;
     HEAP32[$pFKey>>2] = 0;
    }
   }
  }
 } while(0);
 $249 = HEAP32[$db>>2]|0;
 $250 = HEAP32[$pFKey>>2]|0;
 _sqlite3DbFree($249,$250);
 $251 = HEAP32[$db>>2]|0;
 $252 = HEAP32[$1>>2]|0;
 _sqlite3ExprListDelete($251,$252);
 $253 = HEAP32[$db>>2]|0;
 $254 = HEAP32[$3>>2]|0;
 _sqlite3ExprListDelete($253,$254);
 STACKTOP = sp;return;
}
function _sqlite3DeferForeignKey($pParse,$isDeferred) {
 $pParse = $pParse|0;
 $isDeferred = $isDeferred|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFKey = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pTab = sp + 4|0;
 $pFKey = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $isDeferred;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 488|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pTab>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pFKey>>2] = $8;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = $10&255;
 $12 = HEAP32[$pFKey>>2]|0;
 $13 = ((($12)) + 24|0);
 HEAP8[$13>>0] = $11;
 STACKTOP = sp;return;
}
function _sqlite3AddCollateType($pParse,$pToken) {
 $pParse = $pParse|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $p = 0, $pIdx = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $p = sp + 16|0;
 $i = sp + 12|0;
 $zColl = sp + 8|0;
 $db = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pToken;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 488|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 34|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 $10 = (($9) - 1)|0;
 HEAP32[$i>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3NameFromToken($13,$14)|0);
 HEAP32[$zColl>>2] = $15;
 $16 = HEAP32[$zColl>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$zColl>>2]|0;
 $20 = (_sqlite3LocateCollSeq($18,$19)|0);
 $21 = ($20|0)!=(0|0);
 $22 = HEAP32[$db>>2]|0;
 if (!($21)) {
  $62 = HEAP32[$zColl>>2]|0;
  _sqlite3DbFree($22,$62);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$i>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + (($23*24)|0)|0);
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3DbFree($22,$29);
 $30 = HEAP32[$zColl>>2]|0;
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + (($31*24)|0)|0);
 $36 = ((($35)) + 16|0);
 HEAP32[$36>>2] = $30;
 $37 = HEAP32[$p>>2]|0;
 $38 = ((($37)) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$pIdx>>2] = $39;
 while(1) {
  $40 = HEAP32[$pIdx>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$pIdx>>2]|0;
  $43 = ((($42)) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP16[$44>>1]|0;
  $46 = $45 << 16 >> 16;
  $47 = HEAP32[$i>>2]|0;
  $48 = ($46|0)==($47|0);
  if ($48) {
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$p>>2]|0;
   $51 = ((($50)) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + (($49*24)|0)|0);
   $54 = ((($53)) + 16|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$pIdx>>2]|0;
   $57 = ((($56)) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$58>>2] = $55;
  }
  $59 = HEAP32[$pIdx>>2]|0;
  $60 = ((($59)) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  HEAP32[$pIdx>>2] = $61;
 }
 STACKTOP = sp;return;
}
function _sqlite3DropTable($pParse,$pName,$isView,$noErr) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $isView = $isView|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $zArg2 = 0, $zDb = 0, $zTab = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $pTab = sp + 48|0;
 $v = sp + 44|0;
 $db = sp + 40|0;
 $iDb = sp + 36|0;
 $code = sp + 32|0;
 $zTab = sp + 28|0;
 $zDb = sp + 24|0;
 $zArg2 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $isView;
 HEAP32[$3>>2] = $noErr;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 65|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 do {
  if (!($9)) {
   $10 = HEAP32[$0>>2]|0;
   $11 = (_sqlite3ReadSchema($10)|0);
   $12 = ($11|0)!=(0);
   if (!($12)) {
    $13 = HEAP32[$3>>2]|0;
    $14 = ($13|0)!=(0);
    if ($14) {
     $15 = HEAP32[$db>>2]|0;
     $16 = ((($15)) + 69|0);
     $17 = HEAP8[$16>>0]|0;
     $18 = (($17) + 1)<<24>>24;
     HEAP8[$16>>0] = $18;
    }
    $19 = HEAP32[$0>>2]|0;
    $20 = HEAP32[$2>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 8|0);
    $23 = (_sqlite3LocateTableItem($19,$20,$22)|0);
    HEAP32[$pTab>>2] = $23;
    $24 = HEAP32[$3>>2]|0;
    $25 = ($24|0)!=(0);
    if ($25) {
     $26 = HEAP32[$db>>2]|0;
     $27 = ((($26)) + 69|0);
     $28 = HEAP8[$27>>0]|0;
     $29 = (($28) + -1)<<24>>24;
     HEAP8[$27>>0] = $29;
    }
    $30 = HEAP32[$pTab>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     $32 = HEAP32[$3>>2]|0;
     $33 = ($32|0)!=(0);
     if (!($33)) {
      break;
     }
     $34 = HEAP32[$0>>2]|0;
     $35 = HEAP32[$1>>2]|0;
     $36 = ((($35)) + 8|0);
     $37 = ((($36)) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     _sqlite3CodeVerifyNamedSchema($34,$38);
     break;
    }
    $39 = HEAP32[$db>>2]|0;
    $40 = HEAP32[$pTab>>2]|0;
    $41 = ((($40)) + 64|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (_sqlite3SchemaToIndex($39,$42)|0);
    HEAP32[$iDb>>2] = $43;
    $44 = HEAP32[$pTab>>2]|0;
    $45 = ((($44)) + 42|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = $47 & 16;
    $49 = ($48|0)!=(0);
    if ($49) {
     $50 = HEAP32[$0>>2]|0;
     $51 = HEAP32[$pTab>>2]|0;
     $52 = (_sqlite3ViewGetColumnNames($50,$51)|0);
     $53 = ($52|0)!=(0);
     if ($53) {
      break;
     }
    }
    $54 = HEAP32[$iDb>>2]|0;
    $55 = ($54|0)==(1);
    $56 = $55 ? 21260 : 21279;
    HEAP32[$zTab>>2] = $56;
    $57 = HEAP32[$iDb>>2]|0;
    $58 = HEAP32[$db>>2]|0;
    $59 = ((($58)) + 16|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = (($60) + ($57<<4)|0);
    $62 = HEAP32[$61>>2]|0;
    HEAP32[$zDb>>2] = $62;
    HEAP32[$zArg2>>2] = 0;
    $63 = HEAP32[$0>>2]|0;
    $64 = HEAP32[$zTab>>2]|0;
    $65 = HEAP32[$zDb>>2]|0;
    $66 = (_sqlite3AuthCheck($63,9,$64,0,$65)|0);
    $67 = ($66|0)!=(0);
    if (!($67)) {
     $68 = HEAP32[$2>>2]|0;
     $69 = ($68|0)!=(0);
     do {
      if ($69) {
       $70 = HEAP32[$iDb>>2]|0;
       $71 = ($70|0)==(1);
       if ($71) {
        HEAP32[$code>>2] = 15;
        break;
       } else {
        HEAP32[$code>>2] = 17;
        break;
       }
      } else {
       $72 = HEAP32[$pTab>>2]|0;
       $73 = ((($72)) + 42|0);
       $74 = HEAP8[$73>>0]|0;
       $75 = $74&255;
       $76 = $75 & 16;
       $77 = ($76|0)!=(0);
       if ($77) {
        HEAP32[$code>>2] = 30;
        $78 = HEAP32[$db>>2]|0;
        $79 = HEAP32[$pTab>>2]|0;
        $80 = (_sqlite3GetVTable($78,$79)|0);
        $81 = ((($80)) + 4|0);
        $82 = HEAP32[$81>>2]|0;
        $83 = ((($82)) + 4|0);
        $84 = HEAP32[$83>>2]|0;
        HEAP32[$zArg2>>2] = $84;
        break;
       }
       $85 = HEAP32[$iDb>>2]|0;
       $86 = ($85|0)==(1);
       if ($86) {
        HEAP32[$code>>2] = 13;
        break;
       } else {
        HEAP32[$code>>2] = 11;
        break;
       }
      }
     } while(0);
     $87 = HEAP32[$0>>2]|0;
     $88 = HEAP32[$code>>2]|0;
     $89 = HEAP32[$pTab>>2]|0;
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP32[$zArg2>>2]|0;
     $92 = HEAP32[$zDb>>2]|0;
     $93 = (_sqlite3AuthCheck($87,$88,$90,$91,$92)|0);
     $94 = ($93|0)!=(0);
     if (!($94)) {
      $95 = HEAP32[$0>>2]|0;
      $96 = HEAP32[$pTab>>2]|0;
      $97 = HEAP32[$96>>2]|0;
      $98 = HEAP32[$zDb>>2]|0;
      $99 = (_sqlite3AuthCheck($95,9,$97,0,$98)|0);
      $100 = ($99|0)!=(0);
      if (!($100)) {
       $101 = HEAP32[$pTab>>2]|0;
       $102 = HEAP32[$101>>2]|0;
       $103 = (_sqlite3_strnicmp($102,28844,7)|0);
       $104 = ($103|0)==(0);
       if ($104) {
        $105 = HEAP32[$pTab>>2]|0;
        $106 = HEAP32[$105>>2]|0;
        $107 = (_sqlite3_strnicmp($106,31741,11)|0);
        $108 = ($107|0)!=(0);
        if ($108) {
         $109 = HEAP32[$0>>2]|0;
         $110 = HEAP32[$pTab>>2]|0;
         $111 = HEAP32[$110>>2]|0;
         HEAP32[$vararg_buffer>>2] = $111;
         _sqlite3ErrorMsg($109,31753,$vararg_buffer);
         break;
        }
       }
       $112 = HEAP32[$2>>2]|0;
       $113 = ($112|0)!=(0);
       if ($113) {
        $114 = HEAP32[$pTab>>2]|0;
        $115 = ((($114)) + 12|0);
        $116 = HEAP32[$115>>2]|0;
        $117 = ($116|0)==(0|0);
        if ($117) {
         $118 = HEAP32[$0>>2]|0;
         $119 = HEAP32[$pTab>>2]|0;
         $120 = HEAP32[$119>>2]|0;
         HEAP32[$vararg_buffer1>>2] = $120;
         _sqlite3ErrorMsg($118,31781,$vararg_buffer1);
         break;
        }
       }
       $121 = HEAP32[$2>>2]|0;
       $122 = ($121|0)!=(0);
       if (!($122)) {
        $123 = HEAP32[$pTab>>2]|0;
        $124 = ((($123)) + 12|0);
        $125 = HEAP32[$124>>2]|0;
        $126 = ($125|0)!=(0|0);
        if ($126) {
         $127 = HEAP32[$0>>2]|0;
         $128 = HEAP32[$pTab>>2]|0;
         $129 = HEAP32[$128>>2]|0;
         HEAP32[$vararg_buffer4>>2] = $129;
         _sqlite3ErrorMsg($127,31815,$vararg_buffer4);
         break;
        }
       }
       $130 = HEAP32[$0>>2]|0;
       $131 = (_sqlite3GetVdbe($130)|0);
       HEAP32[$v>>2] = $131;
       $132 = HEAP32[$v>>2]|0;
       $133 = ($132|0)!=(0|0);
       if ($133) {
        $134 = HEAP32[$0>>2]|0;
        $135 = HEAP32[$iDb>>2]|0;
        _sqlite3BeginWriteOperation($134,1,$135);
        $136 = HEAP32[$0>>2]|0;
        $137 = HEAP32[$iDb>>2]|0;
        $138 = HEAP32[$pTab>>2]|0;
        $139 = HEAP32[$138>>2]|0;
        _sqlite3ClearStatTables($136,$137,31847,$139);
        $140 = HEAP32[$0>>2]|0;
        $141 = HEAP32[$1>>2]|0;
        $142 = HEAP32[$pTab>>2]|0;
        _sqlite3FkDropTable($140,$141,$142);
        $143 = HEAP32[$0>>2]|0;
        $144 = HEAP32[$pTab>>2]|0;
        $145 = HEAP32[$iDb>>2]|0;
        $146 = HEAP32[$2>>2]|0;
        _sqlite3CodeDropTable($143,$144,$145,$146);
       }
      }
     }
    }
   }
  }
 } while(0);
 $147 = HEAP32[$db>>2]|0;
 $148 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($147,$148);
 STACKTOP = sp;return;
}
function _sqlite3CreateView($pParse,$pBegin,$pName1,$pName2,$pCNames,$pSelect,$isTemp,$noErr) {
 $pParse = $pParse|0;
 $pBegin = $pBegin|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pCNames = $pCNames|0;
 $pSelect = $pSelect|0;
 $isTemp = $isTemp|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $db = 0, $iDb = 0;
 var $n = 0, $p = 0, $pName = 0, $sEnd = 0, $sFix = 0, $vararg_buffer = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $p = sp + 56|0;
 $n = sp + 52|0;
 $z = sp + 48|0;
 $sEnd = sp + 40|0;
 $sFix = sp + 16|0;
 $pName = sp + 12|0;
 $iDb = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pBegin;
 HEAP32[$2>>2] = $pName1;
 HEAP32[$3>>2] = $pName2;
 HEAP32[$4>>2] = $pCNames;
 HEAP32[$5>>2] = $pSelect;
 HEAP32[$6>>2] = $isTemp;
 HEAP32[$7>>2] = $noErr;
 HEAP32[$pName>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 444|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 << 16 >> 16;
 $14 = ($13|0)>(0);
 $15 = HEAP32[$0>>2]|0;
 if ($14) {
  _sqlite3ErrorMsg($15,32103,$vararg_buffer);
 } else {
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = HEAP32[$6>>2]|0;
  $19 = HEAP32[$7>>2]|0;
  _sqlite3StartTable($15,$16,$17,$18,1,0,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 488|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = ($23|0)==(0|0);
  if (!($24)) {
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 68|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)!=(0);
   if (!($28)) {
    $29 = HEAP32[$0>>2]|0;
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$3>>2]|0;
    (_sqlite3TwoPartName($29,$30,$31,$pName)|0);
    $32 = HEAP32[$db>>2]|0;
    $33 = HEAP32[$p>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = (_sqlite3SchemaToIndex($32,$35)|0);
    HEAP32[$iDb>>2] = $36;
    $37 = HEAP32[$0>>2]|0;
    $38 = HEAP32[$iDb>>2]|0;
    $39 = HEAP32[$pName>>2]|0;
    _sqlite3FixInit($sFix,$37,$38,28988,$39);
    $40 = HEAP32[$5>>2]|0;
    $41 = (_sqlite3FixSelect($sFix,$40)|0);
    $42 = ($41|0)!=(0);
    if (!($42)) {
     $43 = HEAP32[$db>>2]|0;
     $44 = HEAP32[$5>>2]|0;
     $45 = (_sqlite3SelectDup($43,$44,1)|0);
     $46 = HEAP32[$p>>2]|0;
     $47 = ((($46)) + 12|0);
     HEAP32[$47>>2] = $45;
     $48 = HEAP32[$db>>2]|0;
     $49 = HEAP32[$4>>2]|0;
     $50 = (_sqlite3ExprListDup($48,$49,1)|0);
     $51 = HEAP32[$p>>2]|0;
     $52 = ((($51)) + 24|0);
     HEAP32[$52>>2] = $50;
     $53 = HEAP32[$db>>2]|0;
     $54 = ((($53)) + 65|0);
     $55 = HEAP8[$54>>0]|0;
     $56 = ($55<<24>>24)!=(0);
     if (!($56)) {
      $57 = HEAP32[$0>>2]|0;
      $58 = ((($57)) + 508|0);
      ;HEAP32[$sEnd>>2]=HEAP32[$58>>2]|0;HEAP32[$sEnd+4>>2]=HEAP32[$58+4>>2]|0;
      $59 = HEAP32[$sEnd>>2]|0;
      $60 = HEAP8[$59>>0]|0;
      $61 = $60 << 24 >> 24;
      $62 = ($61|0)!=(59);
      if ($62) {
       $63 = ((($sEnd)) + 4|0);
       $64 = HEAP32[$63>>2]|0;
       $65 = HEAP32[$sEnd>>2]|0;
       $66 = (($65) + ($64)|0);
       HEAP32[$sEnd>>2] = $66;
      }
      $67 = ((($sEnd)) + 4|0);
      HEAP32[$67>>2] = 0;
      $68 = HEAP32[$sEnd>>2]|0;
      $69 = HEAP32[$1>>2]|0;
      $70 = HEAP32[$69>>2]|0;
      $71 = $68;
      $72 = $70;
      $73 = (($71) - ($72))|0;
      HEAP32[$n>>2] = $73;
      $74 = HEAP32[$1>>2]|0;
      $75 = HEAP32[$74>>2]|0;
      HEAP32[$z>>2] = $75;
      while(1) {
       $76 = HEAP32[$n>>2]|0;
       $77 = (($76) - 1)|0;
       $78 = HEAP32[$z>>2]|0;
       $79 = (($78) + ($77)|0);
       $80 = HEAP8[$79>>0]|0;
       $81 = $80&255;
       $82 = (17533 + ($81)|0);
       $83 = HEAP8[$82>>0]|0;
       $84 = $83&255;
       $85 = $84 & 1;
       $86 = ($85|0)!=(0);
       $87 = HEAP32[$n>>2]|0;
       if (!($86)) {
        break;
       }
       $88 = (($87) + -1)|0;
       HEAP32[$n>>2] = $88;
      }
      $89 = (($87) - 1)|0;
      $90 = HEAP32[$z>>2]|0;
      $91 = (($90) + ($89)|0);
      HEAP32[$sEnd>>2] = $91;
      $92 = ((($sEnd)) + 4|0);
      HEAP32[$92>>2] = 1;
      $93 = HEAP32[$0>>2]|0;
      _sqlite3EndTable($93,0,$sEnd,0,0);
     }
    }
   }
  }
 }
 $94 = HEAP32[$db>>2]|0;
 $95 = HEAP32[$5>>2]|0;
 _sqlite3SelectDelete($94,$95);
 $96 = HEAP32[$db>>2]|0;
 $97 = HEAP32[$4>>2]|0;
 _sqlite3ExprListDelete($96,$97);
 STACKTOP = sp;return;
}
function _sqlite3Select($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0;
 var $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0;
 var $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0;
 var $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0;
 var $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0;
 var $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0;
 var $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0;
 var $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0;
 var $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0;
 var $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0;
 var $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0;
 var $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0;
 var $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0;
 var $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0;
 var $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0;
 var $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0;
 var $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0;
 var $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0;
 var $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0;
 var $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0;
 var $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0;
 var $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0;
 var $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0;
 var $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0;
 var $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0;
 var $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0;
 var $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0;
 var $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0;
 var $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0;
 var $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0;
 var $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0;
 var $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0;
 var $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0;
 var $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0;
 var $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0;
 var $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0;
 var $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0;
 var $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0;
 var $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0;
 var $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0;
 var $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0;
 var $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0;
 var $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0;
 var $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0;
 var $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0;
 var $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0;
 var $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0;
 var $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0;
 var $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0;
 var $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $addr1 = 0, $addrEnd = 0, $addrOutputRow = 0, $addrReset = 0, $addrSetAbort = 0, $addrSortingIdx = 0, $addrTop = 0, $addrTopOfLoop = 0, $db = 0, $dest = 0, $flag = 0, $groupBySort = 0, $i = 0, $iAMem = 0;
 var $iAbortFlag = 0, $iBMem = 0, $iCsr = 0, $iDb = 0, $iEnd = 0, $iRestoreSelectId = 0, $iRoot = 0, $iUseFlag = 0, $isAgg = 0, $isAggSub = 0, $j = 0, $k = 0, $nCol = 0, $nGroupBy = 0, $onceAddr = 0, $or$cond = 0, $orderByGrp = 0, $pBest = 0, $pCol = 0, $pDel = 0;
 var $pEList = 0, $pGroupBy = 0, $pHaving = 0, $pIdx = 0, $pItem = 0, $pItem1 = 0, $pItem3 = 0, $pKeyInfo = 0, $pKeyInfo4 = 0, $pKeyInfo6 = 0, $pMinMax = 0, $pSub = 0, $pSub2 = 0, $pTab = 0, $pTab5 = 0, $pTabList = 0, $pWInfo = 0, $pWhere = 0, $r1 = 0, $rc = 0;
 var $regBase = 0, $regOutputRow = 0, $regRecord = 0, $regReset = 0, $retAddr = 0, $sAggInfo = 0, $sDistinct = 0, $sNC = 0, $sSort = 0, $sortOut = 0, $sortPTab = 0, $topAddr = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $wctrlFlags = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 432|0;
 $vararg_buffer = sp;
 $0 = sp + 412|0;
 $1 = sp + 408|0;
 $2 = sp + 404|0;
 $3 = sp + 400|0;
 $i = sp + 396|0;
 $j = sp + 392|0;
 $pWInfo = sp + 388|0;
 $v = sp + 384|0;
 $isAgg = sp + 380|0;
 $pEList = sp + 376|0;
 $pTabList = sp + 372|0;
 $pWhere = sp + 368|0;
 $pGroupBy = sp + 364|0;
 $pHaving = sp + 360|0;
 $rc = sp + 356|0;
 $sDistinct = sp + 344|0;
 $sSort = sp + 312|0;
 $sAggInfo = sp + 264|0;
 $iEnd = sp + 256|0;
 $db = sp + 252|0;
 $iRestoreSelectId = sp + 248|0;
 $pItem = sp + 244|0;
 $pSub = sp + 240|0;
 $isAggSub = sp + 236|0;
 $pTab = sp + 232|0;
 $pItem1 = sp + 228|0;
 $dest = sp + 208|0;
 $pSub2 = sp + 204|0;
 $addrTop = sp + 200|0;
 $topAddr = sp + 196|0;
 $onceAddr = sp + 192|0;
 $retAddr = sp + 188|0;
 $pKeyInfo = sp + 184|0;
 $wctrlFlags = sp + 416|0;
 $sNC = sp + 152|0;
 $iAMem = sp + 148|0;
 $iBMem = sp + 144|0;
 $iUseFlag = sp + 140|0;
 $iAbortFlag = sp + 136|0;
 $groupBySort = sp + 132|0;
 $addrEnd = sp + 128|0;
 $sortPTab = sp + 124|0;
 $sortOut = sp + 120|0;
 $orderByGrp = sp + 116|0;
 $k = sp + 112|0;
 $pItem3 = sp + 108|0;
 $pKeyInfo4 = sp + 104|0;
 $addr1 = sp + 100|0;
 $addrOutputRow = sp + 96|0;
 $regOutputRow = sp + 92|0;
 $addrSetAbort = sp + 88|0;
 $addrTopOfLoop = sp + 84|0;
 $addrSortingIdx = sp + 80|0;
 $addrReset = sp + 76|0;
 $regReset = sp + 72|0;
 $regBase = sp + 68|0;
 $regRecord = sp + 64|0;
 $nCol = sp + 60|0;
 $nGroupBy = sp + 56|0;
 $pCol = sp + 52|0;
 $r1 = sp + 48|0;
 $pDel = sp + 44|0;
 $pTab5 = sp + 40|0;
 $iDb = sp + 36|0;
 $iCsr = sp + 32|0;
 $pIdx = sp + 28|0;
 $pKeyInfo6 = sp + 24|0;
 $pBest = sp + 20|0;
 $iRoot = sp + 16|0;
 $pMinMax = sp + 12|0;
 $flag = sp + 418|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$pEList>>2] = 0;
 HEAP32[$rc>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 468|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iRestoreSelectId>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 472|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 468|0);
 HEAP32[$12>>2] = $9;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ($15|0)==(0|0);
 if (!($16)) {
  $17 = HEAP32[$db>>2]|0;
  $18 = ((($17)) + 65|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)!=(0);
  if (!($21)) {
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 68|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$1>>2]|0;
    $27 = (_sqlite3AuthCheck($26,21,0,0,0)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$0>>2] = 1;
     $1225 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($1225|0);
    }
    dest=$sAggInfo; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
    $29 = HEAP32[$3>>2]|0;
    $30 = HEAP8[$29>>0]|0;
    $31 = $30&255;
    $32 = ($31|0)<=(8);
    if ($32) {
     $33 = HEAP32[$db>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = ((($34)) + 48|0);
     $36 = HEAP32[$35>>2]|0;
     _sqlite3ExprListDelete($33,$36);
     $37 = HEAP32[$2>>2]|0;
     $38 = ((($37)) + 48|0);
     HEAP32[$38>>2] = 0;
     $39 = HEAP32[$2>>2]|0;
     $40 = ((($39)) + 6|0);
     $41 = HEAP16[$40>>1]|0;
     $42 = $41&65535;
     $43 = $42 & -2;
     $44 = $43&65535;
     HEAP16[$40>>1] = $44;
    }
    $45 = HEAP32[$1>>2]|0;
    $46 = HEAP32[$2>>2]|0;
    _sqlite3SelectPrep($45,$46,0);
    ;HEAP32[$sSort>>2]=0|0;HEAP32[$sSort+4>>2]=0|0;HEAP32[$sSort+8>>2]=0|0;HEAP32[$sSort+12>>2]=0|0;HEAP32[$sSort+16>>2]=0|0;HEAP32[$sSort+20>>2]=0|0;HEAP32[$sSort+24>>2]=0|0;HEAP32[$sSort+28>>2]=0|0;
    $47 = HEAP32[$2>>2]|0;
    $48 = ((($47)) + 48|0);
    $49 = HEAP32[$48>>2]|0;
    HEAP32[$sSort>>2] = $49;
    $50 = HEAP32[$2>>2]|0;
    $51 = ((($50)) + 32|0);
    $52 = HEAP32[$51>>2]|0;
    HEAP32[$pTabList>>2] = $52;
    $53 = HEAP32[$1>>2]|0;
    $54 = ((($53)) + 68|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = ($55|0)!=(0);
    L12: do {
     if (!($56)) {
      $57 = HEAP32[$db>>2]|0;
      $58 = ((($57)) + 65|0);
      $59 = HEAP8[$58>>0]|0;
      $60 = $59&255;
      $61 = ($60|0)!=(0);
      if (!($61)) {
       $62 = HEAP32[$2>>2]|0;
       $63 = ((($62)) + 6|0);
       $64 = HEAP16[$63>>1]|0;
       $65 = $64&65535;
       $66 = $65 & 8;
       $67 = ($66|0)!=(0);
       $68 = $67&1;
       HEAP32[$isAgg>>2] = $68;
       $69 = HEAP32[$1>>2]|0;
       $70 = HEAP32[$3>>2]|0;
       $71 = HEAP32[$2>>2]|0;
       $72 = HEAP32[$71>>2]|0;
       $73 = HEAP32[$72>>2]|0;
       $74 = (_checkForMultiColumnSelectError($69,$70,$73)|0);
       $75 = ($74|0)!=(0);
       if (!($75)) {
        HEAP32[$i>>2] = 0;
        while(1) {
         $76 = HEAP32[$2>>2]|0;
         $77 = ((($76)) + 52|0);
         $78 = HEAP32[$77>>2]|0;
         $79 = ($78|0)!=(0|0);
         if ($79) {
          break;
         }
         $80 = HEAP32[$i>>2]|0;
         $81 = HEAP32[$pTabList>>2]|0;
         $82 = HEAP32[$81>>2]|0;
         $83 = ($80|0)<($82|0);
         if (!($83)) {
          break;
         }
         $84 = HEAP32[$i>>2]|0;
         $85 = HEAP32[$pTabList>>2]|0;
         $86 = ((($85)) + 8|0);
         $87 = (($86) + (($84*72)|0)|0);
         HEAP32[$pItem>>2] = $87;
         $88 = HEAP32[$pItem>>2]|0;
         $89 = ((($88)) + 20|0);
         $90 = HEAP32[$89>>2]|0;
         HEAP32[$pSub>>2] = $90;
         $91 = HEAP32[$pItem>>2]|0;
         $92 = ((($91)) + 16|0);
         $93 = HEAP32[$92>>2]|0;
         HEAP32[$pTab>>2] = $93;
         $94 = HEAP32[$pSub>>2]|0;
         $95 = ($94|0)==(0|0);
         if (!($95)) {
          $96 = HEAP32[$pTab>>2]|0;
          $97 = ((($96)) + 34|0);
          $98 = HEAP16[$97>>1]|0;
          $99 = $98 << 16 >> 16;
          $100 = HEAP32[$pSub>>2]|0;
          $101 = HEAP32[$100>>2]|0;
          $102 = HEAP32[$101>>2]|0;
          $103 = ($99|0)!=($102|0);
          if ($103) {
           label = 17;
           break;
          }
          $114 = HEAP32[$pSub>>2]|0;
          $115 = ((($114)) + 6|0);
          $116 = HEAP16[$115>>1]|0;
          $117 = $116&65535;
          $118 = $117 & 8;
          $119 = ($118|0)!=(0);
          $120 = $119&1;
          HEAP32[$isAggSub>>2] = $120;
          $121 = HEAP32[$1>>2]|0;
          $122 = HEAP32[$2>>2]|0;
          $123 = HEAP32[$i>>2]|0;
          $124 = HEAP32[$isAgg>>2]|0;
          $125 = HEAP32[$isAggSub>>2]|0;
          $126 = (_flattenSubquery($121,$122,$123,$124,$125)|0);
          $127 = ($126|0)!=(0);
          if ($127) {
           $128 = HEAP32[$isAggSub>>2]|0;
           $129 = ($128|0)!=(0);
           if ($129) {
            HEAP32[$isAgg>>2] = 1;
            $130 = HEAP32[$2>>2]|0;
            $131 = ((($130)) + 6|0);
            $132 = HEAP16[$131>>1]|0;
            $133 = $132&65535;
            $134 = $133 | 8;
            $135 = $134&65535;
            HEAP16[$131>>1] = $135;
           }
           HEAP32[$i>>2] = -1;
          }
          $136 = HEAP32[$2>>2]|0;
          $137 = ((($136)) + 32|0);
          $138 = HEAP32[$137>>2]|0;
          HEAP32[$pTabList>>2] = $138;
          $139 = HEAP32[$db>>2]|0;
          $140 = ((($139)) + 65|0);
          $141 = HEAP8[$140>>0]|0;
          $142 = ($141<<24>>24)!=(0);
          if ($142) {
           break L12;
          }
          $143 = HEAP32[$3>>2]|0;
          $144 = HEAP8[$143>>0]|0;
          $145 = $144&255;
          $146 = ($145|0)<=(8);
          if (!($146)) {
           $147 = HEAP32[$2>>2]|0;
           $148 = ((($147)) + 48|0);
           $149 = HEAP32[$148>>2]|0;
           HEAP32[$sSort>>2] = $149;
          }
         }
         $150 = HEAP32[$i>>2]|0;
         $151 = (($150) + 1)|0;
         HEAP32[$i>>2] = $151;
        }
        if ((label|0) == 17) {
         $104 = HEAP32[$1>>2]|0;
         $105 = HEAP32[$pTab>>2]|0;
         $106 = ((($105)) + 34|0);
         $107 = HEAP16[$106>>1]|0;
         $108 = $107 << 16 >> 16;
         $109 = HEAP32[$pTab>>2]|0;
         $110 = HEAP32[$109>>2]|0;
         $111 = HEAP32[$pSub>>2]|0;
         $112 = HEAP32[$111>>2]|0;
         $113 = HEAP32[$112>>2]|0;
         HEAP32[$vararg_buffer>>2] = $108;
         $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
         HEAP32[$vararg_ptr2>>2] = $110;
         $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
         HEAP32[$vararg_ptr3>>2] = $113;
         _sqlite3ErrorMsg($104,32139,$vararg_buffer);
         break;
        }
        $152 = HEAP32[$1>>2]|0;
        $153 = (_sqlite3GetVdbe($152)|0);
        HEAP32[$v>>2] = $153;
        $154 = HEAP32[$v>>2]|0;
        $155 = ($154|0)==(0|0);
        if (!($155)) {
         $156 = HEAP32[$2>>2]|0;
         $157 = ((($156)) + 52|0);
         $158 = HEAP32[$157>>2]|0;
         $159 = ($158|0)!=(0|0);
         if ($159) {
          $160 = HEAP32[$1>>2]|0;
          $161 = HEAP32[$2>>2]|0;
          $162 = HEAP32[$3>>2]|0;
          $163 = (_multiSelect($160,$161,$162)|0);
          HEAP32[$rc>>2] = $163;
          $164 = HEAP32[$iRestoreSelectId>>2]|0;
          $165 = HEAP32[$1>>2]|0;
          $166 = ((($165)) + 468|0);
          HEAP32[$166>>2] = $164;
          $167 = HEAP32[$rc>>2]|0;
          HEAP32[$0>>2] = $167;
          $1225 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($1225|0);
         }
         HEAP32[$i>>2] = 0;
         while(1) {
          $168 = HEAP32[$i>>2]|0;
          $169 = HEAP32[$pTabList>>2]|0;
          $170 = HEAP32[$169>>2]|0;
          $171 = ($168|0)<($170|0);
          if (!($171)) {
           break;
          }
          $172 = HEAP32[$i>>2]|0;
          $173 = HEAP32[$pTabList>>2]|0;
          $174 = ((($173)) + 8|0);
          $175 = (($174) + (($172*72)|0)|0);
          HEAP32[$pItem1>>2] = $175;
          $176 = HEAP32[$pItem1>>2]|0;
          $177 = ((($176)) + 20|0);
          $178 = HEAP32[$177>>2]|0;
          HEAP32[$pSub2>>2] = $178;
          $179 = HEAP32[$pSub2>>2]|0;
          $180 = ($179|0)==(0|0);
          do {
           if (!($180)) {
            $181 = HEAP32[$pItem1>>2]|0;
            $182 = ((($181)) + 24|0);
            $183 = HEAP32[$182>>2]|0;
            $184 = ($183|0)!=(0);
            if ($184) {
             $185 = HEAP32[$pItem1>>2]|0;
             $186 = ((($185)) + 36|0);
             $187 = ((($186)) + 1|0);
             $188 = HEAP8[$187>>0]|0;
             $189 = ($188&255) >>> 4;
             $190 = $189 & 1;
             $191 = $190&255;
             $192 = ($191|0)==(0);
             if (!($192)) {
              break;
             }
             $193 = HEAP32[$v>>2]|0;
             $194 = HEAP32[$pItem1>>2]|0;
             $195 = ((($194)) + 28|0);
             $196 = HEAP32[$195>>2]|0;
             $197 = HEAP32[$pItem1>>2]|0;
             $198 = ((($197)) + 24|0);
             $199 = HEAP32[$198>>2]|0;
             (_sqlite3VdbeAddOp2($193,14,$196,$199)|0);
             break;
            }
            $200 = HEAP32[$2>>2]|0;
            $201 = (_sqlite3SelectExprHeight($200)|0);
            $202 = HEAP32[$1>>2]|0;
            $203 = ((($202)) + 464|0);
            $204 = HEAP32[$203>>2]|0;
            $205 = (($204) + ($201))|0;
            HEAP32[$203>>2] = $205;
            $206 = HEAP32[$pItem1>>2]|0;
            $207 = ((($206)) + 36|0);
            $208 = HEAP8[$207>>0]|0;
            $209 = $208&255;
            $210 = $209 & 32;
            $211 = ($210|0)==(0);
            if ($211) {
             $212 = HEAP32[$db>>2]|0;
             $213 = HEAP32[$pSub2>>2]|0;
             $214 = HEAP32[$2>>2]|0;
             $215 = ((($214)) + 36|0);
             $216 = HEAP32[$215>>2]|0;
             $217 = HEAP32[$pItem1>>2]|0;
             $218 = ((($217)) + 44|0);
             $219 = HEAP32[$218>>2]|0;
             (_pushDownWhereTerms($212,$213,$216,$219)|0);
            }
            $220 = HEAP32[$pTabList>>2]|0;
            $221 = HEAP32[$220>>2]|0;
            $222 = ($221|0)==(1);
            do {
             if ($222) {
              $223 = HEAP32[$2>>2]|0;
              $224 = ((($223)) + 6|0);
              $225 = HEAP16[$224>>1]|0;
              $226 = $225&65535;
              $227 = $226 & 2;
              $228 = ($227|0)==(0);
              if (!($228)) {
               label = 41;
               break;
              }
              $229 = HEAP32[$db>>2]|0;
              $230 = ((($229)) + 60|0);
              $231 = HEAP16[$230>>1]|0;
              $232 = $231&65535;
              $233 = $232 & 256;
              $234 = ($233|0)==(0);
              if (!($234)) {
               label = 41;
               break;
              }
              $235 = HEAP32[$v>>2]|0;
              $236 = (_sqlite3VdbeCurrentAddr($235)|0);
              $237 = (($236) + 1)|0;
              HEAP32[$addrTop>>2] = $237;
              $238 = HEAP32[$1>>2]|0;
              $239 = ((($238)) + 76|0);
              $240 = HEAP32[$239>>2]|0;
              $241 = (($240) + 1)|0;
              HEAP32[$239>>2] = $241;
              $242 = HEAP32[$pItem1>>2]|0;
              $243 = ((($242)) + 28|0);
              HEAP32[$243>>2] = $241;
              $244 = HEAP32[$v>>2]|0;
              $245 = HEAP32[$pItem1>>2]|0;
              $246 = ((($245)) + 28|0);
              $247 = HEAP32[$246>>2]|0;
              $248 = HEAP32[$addrTop>>2]|0;
              (_sqlite3VdbeAddOp3($244,16,$247,0,$248)|0);
              $249 = HEAP32[$addrTop>>2]|0;
              $250 = HEAP32[$pItem1>>2]|0;
              $251 = ((($250)) + 24|0);
              HEAP32[$251>>2] = $249;
              $252 = HEAP32[$pItem1>>2]|0;
              $253 = ((($252)) + 28|0);
              $254 = HEAP32[$253>>2]|0;
              _sqlite3SelectDestInit($dest,13,$254);
              $255 = HEAP32[$1>>2]|0;
              $256 = ((($255)) + 472|0);
              $257 = HEAP32[$256>>2]|0;
              $258 = $257&255;
              $259 = HEAP32[$pItem1>>2]|0;
              $260 = ((($259)) + 40|0);
              HEAP8[$260>>0] = $258;
              $261 = HEAP32[$1>>2]|0;
              $262 = HEAP32[$pSub2>>2]|0;
              (_sqlite3Select($261,$262,$dest)|0);
              $263 = HEAP32[$pSub2>>2]|0;
              $264 = ((($263)) + 24|0);
              $265 = $264;
              $266 = $265;
              $267 = HEAP32[$266>>2]|0;
              $268 = (($265) + 4)|0;
              $269 = $268;
              $270 = HEAP32[$269>>2]|0;
              $271 = (_sqlite3LogEst($267,$270)|0);
              $272 = HEAP32[$pItem1>>2]|0;
              $273 = ((($272)) + 16|0);
              $274 = HEAP32[$273>>2]|0;
              $275 = ((($274)) + 38|0);
              HEAP16[$275>>1] = $271;
              $276 = HEAP32[$pItem1>>2]|0;
              $277 = ((($276)) + 36|0);
              $278 = ((($277)) + 1|0);
              $279 = HEAP8[$278>>0]|0;
              $280 = $279 & -17;
              $281 = $280 | 16;
              HEAP8[$278>>0] = $281;
              $282 = ((($dest)) + 8|0);
              $283 = HEAP32[$282>>2]|0;
              $284 = HEAP32[$pItem1>>2]|0;
              $285 = ((($284)) + 32|0);
              HEAP32[$285>>2] = $283;
              $286 = HEAP32[$v>>2]|0;
              $287 = HEAP32[$pItem1>>2]|0;
              $288 = ((($287)) + 28|0);
              $289 = HEAP32[$288>>2]|0;
              _sqlite3VdbeEndCoroutine($286,$289);
              $290 = HEAP32[$v>>2]|0;
              $291 = HEAP32[$addrTop>>2]|0;
              $292 = (($291) - 1)|0;
              _sqlite3VdbeJumpHere($290,$292);
              $293 = HEAP32[$1>>2]|0;
              _sqlite3ClearTempRegCache($293);
             } else {
              label = 41;
             }
            } while(0);
            if ((label|0) == 41) {
             label = 0;
             HEAP32[$onceAddr>>2] = 0;
             $294 = HEAP32[$1>>2]|0;
             $295 = ((($294)) + 76|0);
             $296 = HEAP32[$295>>2]|0;
             $297 = (($296) + 1)|0;
             HEAP32[$295>>2] = $297;
             $298 = HEAP32[$pItem1>>2]|0;
             $299 = ((($298)) + 28|0);
             HEAP32[$299>>2] = $297;
             $300 = HEAP32[$v>>2]|0;
             $301 = HEAP32[$pItem1>>2]|0;
             $302 = ((($301)) + 28|0);
             $303 = HEAP32[$302>>2]|0;
             $304 = (_sqlite3VdbeAddOp2($300,22,0,$303)|0);
             HEAP32[$topAddr>>2] = $304;
             $305 = HEAP32[$topAddr>>2]|0;
             $306 = (($305) + 1)|0;
             $307 = HEAP32[$pItem1>>2]|0;
             $308 = ((($307)) + 24|0);
             HEAP32[$308>>2] = $306;
             $309 = HEAP32[$pItem1>>2]|0;
             $310 = ((($309)) + 36|0);
             $311 = ((($310)) + 1|0);
             $312 = HEAP8[$311>>0]|0;
             $313 = ($312&255) >>> 3;
             $314 = $313 & 1;
             $315 = $314&255;
             $316 = ($315|0)==(0);
             if ($316) {
              $317 = HEAP32[$1>>2]|0;
              $318 = (_sqlite3CodeOnce($317)|0);
              HEAP32[$onceAddr>>2] = $318;
             }
             $319 = HEAP32[$pItem1>>2]|0;
             $320 = ((($319)) + 44|0);
             $321 = HEAP32[$320>>2]|0;
             _sqlite3SelectDestInit($dest,12,$321);
             $322 = HEAP32[$1>>2]|0;
             $323 = ((($322)) + 472|0);
             $324 = HEAP32[$323>>2]|0;
             $325 = $324&255;
             $326 = HEAP32[$pItem1>>2]|0;
             $327 = ((($326)) + 40|0);
             HEAP8[$327>>0] = $325;
             $328 = HEAP32[$1>>2]|0;
             $329 = HEAP32[$pSub2>>2]|0;
             (_sqlite3Select($328,$329,$dest)|0);
             $330 = HEAP32[$pSub2>>2]|0;
             $331 = ((($330)) + 24|0);
             $332 = $331;
             $333 = $332;
             $334 = HEAP32[$333>>2]|0;
             $335 = (($332) + 4)|0;
             $336 = $335;
             $337 = HEAP32[$336>>2]|0;
             $338 = (_sqlite3LogEst($334,$337)|0);
             $339 = HEAP32[$pItem1>>2]|0;
             $340 = ((($339)) + 16|0);
             $341 = HEAP32[$340>>2]|0;
             $342 = ((($341)) + 38|0);
             HEAP16[$342>>1] = $338;
             $343 = HEAP32[$onceAddr>>2]|0;
             $344 = ($343|0)!=(0);
             if ($344) {
              $345 = HEAP32[$v>>2]|0;
              $346 = HEAP32[$onceAddr>>2]|0;
              _sqlite3VdbeJumpHere($345,$346);
             }
             $347 = HEAP32[$v>>2]|0;
             $348 = HEAP32[$pItem1>>2]|0;
             $349 = ((($348)) + 28|0);
             $350 = HEAP32[$349>>2]|0;
             $351 = (_sqlite3VdbeAddOp1($347,15,$350)|0);
             HEAP32[$retAddr>>2] = $351;
             $352 = HEAP32[$v>>2]|0;
             $353 = HEAP32[$topAddr>>2]|0;
             $354 = HEAP32[$retAddr>>2]|0;
             _sqlite3VdbeChangeP1($352,$353,$354);
             $355 = HEAP32[$1>>2]|0;
             _sqlite3ClearTempRegCache($355);
            }
            $356 = HEAP32[$db>>2]|0;
            $357 = ((($356)) + 65|0);
            $358 = HEAP8[$357>>0]|0;
            $359 = ($358<<24>>24)!=(0);
            if ($359) {
             break L12;
            }
            $360 = HEAP32[$2>>2]|0;
            $361 = (_sqlite3SelectExprHeight($360)|0);
            $362 = HEAP32[$1>>2]|0;
            $363 = ((($362)) + 464|0);
            $364 = HEAP32[$363>>2]|0;
            $365 = (($364) - ($361))|0;
            HEAP32[$363>>2] = $365;
           }
          } while(0);
          $366 = HEAP32[$i>>2]|0;
          $367 = (($366) + 1)|0;
          HEAP32[$i>>2] = $367;
         }
         $368 = HEAP32[$2>>2]|0;
         $369 = HEAP32[$368>>2]|0;
         HEAP32[$pEList>>2] = $369;
         $370 = HEAP32[$2>>2]|0;
         $371 = ((($370)) + 36|0);
         $372 = HEAP32[$371>>2]|0;
         HEAP32[$pWhere>>2] = $372;
         $373 = HEAP32[$2>>2]|0;
         $374 = ((($373)) + 40|0);
         $375 = HEAP32[$374>>2]|0;
         HEAP32[$pGroupBy>>2] = $375;
         $376 = HEAP32[$2>>2]|0;
         $377 = ((($376)) + 44|0);
         $378 = HEAP32[$377>>2]|0;
         HEAP32[$pHaving>>2] = $378;
         $379 = HEAP32[$2>>2]|0;
         $380 = ((($379)) + 6|0);
         $381 = HEAP16[$380>>1]|0;
         $382 = $381&65535;
         $383 = $382 & 1;
         $384 = ($383|0)!=(0);
         $385 = $384&1;
         $386 = $385&255;
         HEAP8[$sDistinct>>0] = $386;
         $387 = HEAP32[$2>>2]|0;
         $388 = ((($387)) + 6|0);
         $389 = HEAP16[$388>>1]|0;
         $390 = $389&65535;
         $391 = $390 & 9;
         $392 = ($391|0)==(1);
         if ($392) {
          $393 = HEAP32[$sSort>>2]|0;
          $394 = HEAP32[$pEList>>2]|0;
          $395 = (_sqlite3ExprListCompare($393,$394,-1)|0);
          $396 = ($395|0)==(0);
          if ($396) {
           $397 = HEAP32[$2>>2]|0;
           $398 = ((($397)) + 6|0);
           $399 = HEAP16[$398>>1]|0;
           $400 = $399&65535;
           $401 = $400 & -2;
           $402 = $401&65535;
           HEAP16[$398>>1] = $402;
           $403 = HEAP32[$db>>2]|0;
           $404 = HEAP32[$pEList>>2]|0;
           $405 = (_sqlite3ExprListDup($403,$404,0)|0);
           $406 = HEAP32[$2>>2]|0;
           $407 = ((($406)) + 40|0);
           HEAP32[$407>>2] = $405;
           HEAP32[$pGroupBy>>2] = $405;
          }
         }
         $408 = HEAP32[$sSort>>2]|0;
         $409 = ($408|0)!=(0|0);
         if ($409) {
          $410 = HEAP32[$1>>2]|0;
          $411 = HEAP32[$sSort>>2]|0;
          $412 = HEAP32[$pEList>>2]|0;
          $413 = HEAP32[$412>>2]|0;
          $414 = (_keyInfoFromExprList($410,$411,0,$413)|0);
          HEAP32[$pKeyInfo>>2] = $414;
          $415 = HEAP32[$1>>2]|0;
          $416 = ((($415)) + 72|0);
          $417 = HEAP32[$416>>2]|0;
          $418 = (($417) + 1)|0;
          HEAP32[$416>>2] = $418;
          $419 = ((($sSort)) + 8|0);
          HEAP32[$419>>2] = $417;
          $420 = HEAP32[$v>>2]|0;
          $421 = ((($sSort)) + 8|0);
          $422 = HEAP32[$421>>2]|0;
          $423 = HEAP32[$sSort>>2]|0;
          $424 = HEAP32[$423>>2]|0;
          $425 = (($424) + 1)|0;
          $426 = HEAP32[$pEList>>2]|0;
          $427 = HEAP32[$426>>2]|0;
          $428 = (($425) + ($427))|0;
          $429 = HEAP32[$pKeyInfo>>2]|0;
          $430 = (_sqlite3VdbeAddOp4($420,57,$422,$428,0,$429,-6)|0);
          $431 = ((($sSort)) + 20|0);
          HEAP32[$431>>2] = $430;
         } else {
          $432 = ((($sSort)) + 20|0);
          HEAP32[$432>>2] = -1;
         }
         $433 = HEAP32[$3>>2]|0;
         $434 = HEAP8[$433>>0]|0;
         $435 = $434&255;
         $436 = ($435|0)==(12);
         if ($436) {
          $437 = HEAP32[$v>>2]|0;
          $438 = HEAP32[$3>>2]|0;
          $439 = ((($438)) + 4|0);
          $440 = HEAP32[$439>>2]|0;
          $441 = HEAP32[$pEList>>2]|0;
          $442 = HEAP32[$441>>2]|0;
          (_sqlite3VdbeAddOp2($437,57,$440,$442)|0);
         }
         $443 = HEAP32[$v>>2]|0;
         $444 = (_sqlite3VdbeMakeLabel($443)|0);
         HEAP32[$iEnd>>2] = $444;
         $445 = HEAP32[$2>>2]|0;
         $446 = ((($445)) + 24|0);
         $447 = $446;
         $448 = $447;
         HEAP32[$448>>2] = -1;
         $449 = (($447) + 4)|0;
         $450 = $449;
         HEAP32[$450>>2] = 2147483647;
         $451 = HEAP32[$1>>2]|0;
         $452 = HEAP32[$2>>2]|0;
         $453 = HEAP32[$iEnd>>2]|0;
         _computeLimitRegisters($451,$452,$453);
         $454 = HEAP32[$2>>2]|0;
         $455 = ((($454)) + 8|0);
         $456 = HEAP32[$455>>2]|0;
         $457 = ($456|0)==(0);
         if ($457) {
          $458 = ((($sSort)) + 20|0);
          $459 = HEAP32[$458>>2]|0;
          $460 = ($459|0)>=(0);
          if ($460) {
           $461 = HEAP32[$v>>2]|0;
           $462 = ((($sSort)) + 20|0);
           $463 = HEAP32[$462>>2]|0;
           _sqlite3VdbeChangeOpcode($461,$463,58);
           $464 = ((($sSort)) + 28|0);
           $465 = HEAP8[$464>>0]|0;
           $466 = $465&255;
           $467 = $466 | 1;
           $468 = $467&255;
           HEAP8[$464>>0] = $468;
          }
         }
         $469 = HEAP32[$2>>2]|0;
         $470 = ((($469)) + 6|0);
         $471 = HEAP16[$470>>1]|0;
         $472 = $471&65535;
         $473 = $472 & 1;
         $474 = ($473|0)!=(0);
         if ($474) {
          $475 = HEAP32[$1>>2]|0;
          $476 = ((($475)) + 72|0);
          $477 = HEAP32[$476>>2]|0;
          $478 = (($477) + 1)|0;
          HEAP32[$476>>2] = $478;
          $479 = ((($sDistinct)) + 4|0);
          HEAP32[$479>>2] = $477;
          $480 = HEAP32[$v>>2]|0;
          $481 = ((($sDistinct)) + 4|0);
          $482 = HEAP32[$481>>2]|0;
          $483 = HEAP32[$1>>2]|0;
          $484 = HEAP32[$2>>2]|0;
          $485 = HEAP32[$484>>2]|0;
          $486 = (_keyInfoFromExprList($483,$485,0,0)|0);
          $487 = (_sqlite3VdbeAddOp4($480,57,$482,0,0,$486,-6)|0);
          $488 = ((($sDistinct)) + 8|0);
          HEAP32[$488>>2] = $487;
          $489 = HEAP32[$v>>2]|0;
          _sqlite3VdbeChangeP5($489,8);
          $490 = ((($sDistinct)) + 1|0);
          HEAP8[$490>>0] = 3;
         } else {
          $491 = ((($sDistinct)) + 1|0);
          HEAP8[$491>>0] = 0;
         }
         $492 = HEAP32[$isAgg>>2]|0;
         $493 = ($492|0)==(0);
         $494 = HEAP32[$pGroupBy>>2]|0;
         $495 = ($494|0)==(0|0);
         $or$cond = $493 & $495;
         if ($or$cond) {
          $496 = HEAP8[$sDistinct>>0]|0;
          $497 = $496&255;
          $498 = ($497|0)!=(0);
          $499 = $498 ? 1024 : 0;
          $500 = $499&65535;
          HEAP16[$wctrlFlags>>1] = $500;
          $501 = HEAP32[$1>>2]|0;
          $502 = HEAP32[$pTabList>>2]|0;
          $503 = HEAP32[$pWhere>>2]|0;
          $504 = HEAP32[$sSort>>2]|0;
          $505 = HEAP32[$2>>2]|0;
          $506 = HEAP32[$505>>2]|0;
          $507 = HEAP16[$wctrlFlags>>1]|0;
          $508 = (_sqlite3WhereBegin($501,$502,$503,$504,$506,$507,0)|0);
          HEAP32[$pWInfo>>2] = $508;
          $509 = HEAP32[$pWInfo>>2]|0;
          $510 = ($509|0)==(0|0);
          if ($510) {
           break;
          }
          $511 = HEAP32[$pWInfo>>2]|0;
          $512 = (_sqlite3WhereOutputRowCount($511)|0);
          $513 = tempRet0;
          $514 = HEAP32[$2>>2]|0;
          $515 = ((($514)) + 24|0);
          $516 = $515;
          $517 = $516;
          $518 = HEAP32[$517>>2]|0;
          $519 = (($516) + 4)|0;
          $520 = $519;
          $521 = HEAP32[$520>>2]|0;
          $522 = ($513>>>0)<($521>>>0);
          $523 = ($512>>>0)<($518>>>0);
          $524 = ($513|0)==($521|0);
          $525 = $524 & $523;
          $526 = $522 | $525;
          if ($526) {
           $527 = HEAP32[$pWInfo>>2]|0;
           $528 = (_sqlite3WhereOutputRowCount($527)|0);
           $529 = tempRet0;
           $530 = HEAP32[$2>>2]|0;
           $531 = ((($530)) + 24|0);
           $532 = $531;
           $533 = $532;
           HEAP32[$533>>2] = $528;
           $534 = (($532) + 4)|0;
           $535 = $534;
           HEAP32[$535>>2] = $529;
          }
          $536 = HEAP8[$sDistinct>>0]|0;
          $537 = $536&255;
          $538 = ($537|0)!=(0);
          do {
           if ($538) {
            $539 = HEAP32[$pWInfo>>2]|0;
            $540 = (_sqlite3WhereIsDistinct($539)|0);
            $541 = ($540|0)!=(0);
            if (!($541)) {
             break;
            }
            $542 = HEAP32[$pWInfo>>2]|0;
            $543 = (_sqlite3WhereIsDistinct($542)|0);
            $544 = $543&255;
            $545 = ((($sDistinct)) + 1|0);
            HEAP8[$545>>0] = $544;
           }
          } while(0);
          $546 = HEAP32[$sSort>>2]|0;
          $547 = ($546|0)!=(0|0);
          do {
           if ($547) {
            $548 = HEAP32[$pWInfo>>2]|0;
            $549 = (_sqlite3WhereIsOrdered($548)|0);
            $550 = ((($sSort)) + 4|0);
            HEAP32[$550>>2] = $549;
            $551 = ((($sSort)) + 4|0);
            $552 = HEAP32[$551>>2]|0;
            $553 = HEAP32[$sSort>>2]|0;
            $554 = HEAP32[$553>>2]|0;
            $555 = ($552|0)==($554|0);
            if (!($555)) {
             break;
            }
            HEAP32[$sSort>>2] = 0;
           }
          } while(0);
          $556 = ((($sSort)) + 20|0);
          $557 = HEAP32[$556>>2]|0;
          $558 = ($557|0)>=(0);
          do {
           if ($558) {
            $559 = HEAP32[$sSort>>2]|0;
            $560 = ($559|0)==(0|0);
            if (!($560)) {
             break;
            }
            $561 = HEAP32[$v>>2]|0;
            $562 = ((($sSort)) + 20|0);
            $563 = HEAP32[$562>>2]|0;
            (_sqlite3VdbeChangeToNoop($561,$563)|0);
           }
          } while(0);
          $564 = HEAP32[$1>>2]|0;
          $565 = HEAP32[$2>>2]|0;
          $566 = HEAP32[$pEList>>2]|0;
          $567 = HEAP32[$3>>2]|0;
          $568 = HEAP32[$pWInfo>>2]|0;
          $569 = (_sqlite3WhereContinueLabel($568)|0);
          $570 = HEAP32[$pWInfo>>2]|0;
          $571 = (_sqlite3WhereBreakLabel($570)|0);
          _selectInnerLoop($564,$565,$566,-1,$sSort,$sDistinct,$567,$569,$571);
          $572 = HEAP32[$pWInfo>>2]|0;
          _sqlite3WhereEnd($572);
         } else {
          HEAP32[$sortPTab>>2] = 0;
          HEAP32[$sortOut>>2] = 0;
          HEAP32[$orderByGrp>>2] = 0;
          $573 = HEAP32[$pGroupBy>>2]|0;
          $574 = ($573|0)!=(0|0);
          $575 = HEAP32[$2>>2]|0;
          do {
           if ($574) {
            $576 = HEAP32[$575>>2]|0;
            $577 = HEAP32[$576>>2]|0;
            HEAP32[$k>>2] = $577;
            $578 = HEAP32[$2>>2]|0;
            $579 = HEAP32[$578>>2]|0;
            $580 = ((($579)) + 4|0);
            $581 = HEAP32[$580>>2]|0;
            HEAP32[$pItem3>>2] = $581;
            while(1) {
             $582 = HEAP32[$k>>2]|0;
             $583 = ($582|0)>(0);
             if (!($583)) {
              break;
             }
             $584 = HEAP32[$pItem3>>2]|0;
             $585 = ((($584)) + 16|0);
             $586 = ((($585)) + 2|0);
             HEAP16[$586>>1] = 0;
             $587 = HEAP32[$k>>2]|0;
             $588 = (($587) + -1)|0;
             HEAP32[$k>>2] = $588;
             $589 = HEAP32[$pItem3>>2]|0;
             $590 = ((($589)) + 20|0);
             HEAP32[$pItem3>>2] = $590;
            }
            $591 = HEAP32[$pGroupBy>>2]|0;
            $592 = HEAP32[$591>>2]|0;
            HEAP32[$k>>2] = $592;
            $593 = HEAP32[$pGroupBy>>2]|0;
            $594 = ((($593)) + 4|0);
            $595 = HEAP32[$594>>2]|0;
            HEAP32[$pItem3>>2] = $595;
            while(1) {
             $596 = HEAP32[$k>>2]|0;
             $597 = ($596|0)>(0);
             if (!($597)) {
              break;
             }
             $598 = HEAP32[$pItem3>>2]|0;
             $599 = ((($598)) + 16|0);
             $600 = ((($599)) + 2|0);
             HEAP16[$600>>1] = 0;
             $601 = HEAP32[$k>>2]|0;
             $602 = (($601) + -1)|0;
             HEAP32[$k>>2] = $602;
             $603 = HEAP32[$pItem3>>2]|0;
             $604 = ((($603)) + 20|0);
             HEAP32[$pItem3>>2] = $604;
            }
            $605 = HEAP32[$2>>2]|0;
            $606 = ((($605)) + 24|0);
            $607 = $606;
            $608 = $607;
            $609 = HEAP32[$608>>2]|0;
            $610 = (($607) + 4)|0;
            $611 = $610;
            $612 = HEAP32[$611>>2]|0;
            $613 = ($612>>>0)>(0);
            $614 = ($609>>>0)>(100);
            $615 = ($612|0)==(0);
            $616 = $615 & $614;
            $617 = $613 | $616;
            if (!($617)) {
             break;
            }
            $618 = HEAP32[$2>>2]|0;
            $619 = ((($618)) + 24|0);
            $620 = $619;
            $621 = $620;
            HEAP32[$621>>2] = 100;
            $622 = (($620) + 4)|0;
            $623 = $622;
            HEAP32[$623>>2] = 0;
           } else {
            $624 = ((($575)) + 24|0);
            $625 = $624;
            $626 = $625;
            HEAP32[$626>>2] = 1;
            $627 = (($625) + 4)|0;
            $628 = $627;
            HEAP32[$628>>2] = 0;
           }
          } while(0);
          $629 = HEAP32[$pGroupBy>>2]|0;
          $630 = HEAP32[$sSort>>2]|0;
          $631 = (_sqlite3ExprListCompare($629,$630,-1)|0);
          $632 = ($631|0)==(0);
          if ($632) {
           HEAP32[$orderByGrp>>2] = 1;
          }
          $633 = HEAP32[$v>>2]|0;
          $634 = (_sqlite3VdbeMakeLabel($633)|0);
          HEAP32[$addrEnd>>2] = $634;
          ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
          $635 = HEAP32[$1>>2]|0;
          HEAP32[$sNC>>2] = $635;
          $636 = HEAP32[$pTabList>>2]|0;
          $637 = ((($sNC)) + 4|0);
          HEAP32[$637>>2] = $636;
          $638 = ((($sNC)) + 12|0);
          HEAP32[$638>>2] = $sAggInfo;
          $639 = HEAP32[$1>>2]|0;
          $640 = ((($639)) + 76|0);
          $641 = HEAP32[$640>>2]|0;
          $642 = (($641) + 1)|0;
          $643 = ((($sAggInfo)) + 16|0);
          HEAP32[$643>>2] = $642;
          $644 = HEAP32[$pGroupBy>>2]|0;
          $645 = ($644|0)!=(0|0);
          if ($645) {
           $646 = HEAP32[$pGroupBy>>2]|0;
           $647 = HEAP32[$646>>2]|0;
           $649 = $647;
          } else {
           $649 = 0;
          }
          $648 = ((($sAggInfo)) + 12|0);
          HEAP32[$648>>2] = $649;
          $650 = HEAP32[$pGroupBy>>2]|0;
          $651 = ((($sAggInfo)) + 24|0);
          HEAP32[$651>>2] = $650;
          $652 = HEAP32[$pEList>>2]|0;
          _sqlite3ExprAnalyzeAggList($sNC,$652);
          $653 = HEAP32[$sSort>>2]|0;
          _sqlite3ExprAnalyzeAggList($sNC,$653);
          $654 = HEAP32[$pHaving>>2]|0;
          $655 = ($654|0)!=(0|0);
          if ($655) {
           $656 = HEAP32[$pHaving>>2]|0;
           _sqlite3ExprAnalyzeAggregates($sNC,$656);
          }
          $657 = ((($sAggInfo)) + 32|0);
          $658 = HEAP32[$657>>2]|0;
          $659 = ((($sAggInfo)) + 36|0);
          HEAP32[$659>>2] = $658;
          HEAP32[$i>>2] = 0;
          while(1) {
           $660 = HEAP32[$i>>2]|0;
           $661 = ((($sAggInfo)) + 44|0);
           $662 = HEAP32[$661>>2]|0;
           $663 = ($660|0)<($662|0);
           if (!($663)) {
            break;
           }
           $664 = ((($sNC)) + 28|0);
           $665 = HEAP16[$664>>1]|0;
           $666 = $665&65535;
           $667 = $666 | 8;
           $668 = $667&65535;
           HEAP16[$664>>1] = $668;
           $669 = HEAP32[$i>>2]|0;
           $670 = ((($sAggInfo)) + 40|0);
           $671 = HEAP32[$670>>2]|0;
           $672 = (($671) + ($669<<4)|0);
           $673 = HEAP32[$672>>2]|0;
           $674 = ((($673)) + 20|0);
           $675 = HEAP32[$674>>2]|0;
           _sqlite3ExprAnalyzeAggList($sNC,$675);
           $676 = ((($sNC)) + 28|0);
           $677 = HEAP16[$676>>1]|0;
           $678 = $677&65535;
           $679 = $678 & -9;
           $680 = $679&65535;
           HEAP16[$676>>1] = $680;
           $681 = HEAP32[$i>>2]|0;
           $682 = (($681) + 1)|0;
           HEAP32[$i>>2] = $682;
          }
          $683 = HEAP32[$1>>2]|0;
          $684 = ((($683)) + 76|0);
          $685 = HEAP32[$684>>2]|0;
          $686 = ((($sAggInfo)) + 20|0);
          HEAP32[$686>>2] = $685;
          $687 = HEAP32[$db>>2]|0;
          $688 = ((($687)) + 65|0);
          $689 = HEAP8[$688>>0]|0;
          $690 = ($689<<24>>24)!=(0);
          if ($690) {
           break;
          }
          $691 = HEAP32[$pGroupBy>>2]|0;
          $692 = ($691|0)!=(0|0);
          if ($692) {
           $693 = HEAP32[$1>>2]|0;
           $694 = ((($693)) + 72|0);
           $695 = HEAP32[$694>>2]|0;
           $696 = (($695) + 1)|0;
           HEAP32[$694>>2] = $696;
           $697 = ((($sAggInfo)) + 4|0);
           HEAP32[$697>>2] = $695;
           $698 = HEAP32[$1>>2]|0;
           $699 = HEAP32[$pGroupBy>>2]|0;
           $700 = ((($sAggInfo)) + 32|0);
           $701 = HEAP32[$700>>2]|0;
           $702 = (_keyInfoFromExprList($698,$699,0,$701)|0);
           HEAP32[$pKeyInfo4>>2] = $702;
           $703 = HEAP32[$v>>2]|0;
           $704 = ((($sAggInfo)) + 4|0);
           $705 = HEAP32[$704>>2]|0;
           $706 = ((($sAggInfo)) + 12|0);
           $707 = HEAP32[$706>>2]|0;
           $708 = HEAP32[$pKeyInfo4>>2]|0;
           $709 = (_sqlite3VdbeAddOp4($703,58,$705,$707,0,$708,-6)|0);
           HEAP32[$addrSortingIdx>>2] = $709;
           $710 = HEAP32[$1>>2]|0;
           $711 = ((($710)) + 76|0);
           $712 = HEAP32[$711>>2]|0;
           $713 = (($712) + 1)|0;
           HEAP32[$711>>2] = $713;
           HEAP32[$iUseFlag>>2] = $713;
           $714 = HEAP32[$1>>2]|0;
           $715 = ((($714)) + 76|0);
           $716 = HEAP32[$715>>2]|0;
           $717 = (($716) + 1)|0;
           HEAP32[$715>>2] = $717;
           HEAP32[$iAbortFlag>>2] = $717;
           $718 = HEAP32[$1>>2]|0;
           $719 = ((($718)) + 76|0);
           $720 = HEAP32[$719>>2]|0;
           $721 = (($720) + 1)|0;
           HEAP32[$719>>2] = $721;
           HEAP32[$regOutputRow>>2] = $721;
           $722 = HEAP32[$v>>2]|0;
           $723 = (_sqlite3VdbeMakeLabel($722)|0);
           HEAP32[$addrOutputRow>>2] = $723;
           $724 = HEAP32[$1>>2]|0;
           $725 = ((($724)) + 76|0);
           $726 = HEAP32[$725>>2]|0;
           $727 = (($726) + 1)|0;
           HEAP32[$725>>2] = $727;
           HEAP32[$regReset>>2] = $727;
           $728 = HEAP32[$v>>2]|0;
           $729 = (_sqlite3VdbeMakeLabel($728)|0);
           HEAP32[$addrReset>>2] = $729;
           $730 = HEAP32[$1>>2]|0;
           $731 = ((($730)) + 76|0);
           $732 = HEAP32[$731>>2]|0;
           $733 = (($732) + 1)|0;
           HEAP32[$iAMem>>2] = $733;
           $734 = HEAP32[$pGroupBy>>2]|0;
           $735 = HEAP32[$734>>2]|0;
           $736 = HEAP32[$1>>2]|0;
           $737 = ((($736)) + 76|0);
           $738 = HEAP32[$737>>2]|0;
           $739 = (($738) + ($735))|0;
           HEAP32[$737>>2] = $739;
           $740 = HEAP32[$1>>2]|0;
           $741 = ((($740)) + 76|0);
           $742 = HEAP32[$741>>2]|0;
           $743 = (($742) + 1)|0;
           HEAP32[$iBMem>>2] = $743;
           $744 = HEAP32[$pGroupBy>>2]|0;
           $745 = HEAP32[$744>>2]|0;
           $746 = HEAP32[$1>>2]|0;
           $747 = ((($746)) + 76|0);
           $748 = HEAP32[$747>>2]|0;
           $749 = (($748) + ($745))|0;
           HEAP32[$747>>2] = $749;
           $750 = HEAP32[$v>>2]|0;
           $751 = HEAP32[$iAbortFlag>>2]|0;
           (_sqlite3VdbeAddOp2($750,22,0,$751)|0);
           $752 = HEAP32[$v>>2]|0;
           $753 = HEAP32[$iUseFlag>>2]|0;
           (_sqlite3VdbeAddOp2($752,22,0,$753)|0);
           $754 = HEAP32[$v>>2]|0;
           $755 = HEAP32[$iAMem>>2]|0;
           $756 = HEAP32[$iAMem>>2]|0;
           $757 = HEAP32[$pGroupBy>>2]|0;
           $758 = HEAP32[$757>>2]|0;
           $759 = (($756) + ($758))|0;
           $760 = (($759) - 1)|0;
           (_sqlite3VdbeAddOp3($754,25,0,$755,$760)|0);
           $761 = HEAP32[$v>>2]|0;
           $762 = HEAP32[$regReset>>2]|0;
           $763 = HEAP32[$addrReset>>2]|0;
           (_sqlite3VdbeAddOp2($761,14,$762,$763)|0);
           $764 = HEAP32[$1>>2]|0;
           $765 = HEAP32[$pTabList>>2]|0;
           $766 = HEAP32[$pWhere>>2]|0;
           $767 = HEAP32[$pGroupBy>>2]|0;
           $768 = HEAP32[$orderByGrp>>2]|0;
           $769 = ($768|0)!=(0);
           $770 = $769 ? 2048 : 0;
           $771 = 256 | $770;
           $772 = $771&65535;
           $773 = (_sqlite3WhereBegin($764,$765,$766,$767,0,$772,0)|0);
           HEAP32[$pWInfo>>2] = $773;
           $774 = HEAP32[$pWInfo>>2]|0;
           $775 = ($774|0)==(0|0);
           if ($775) {
            break;
           }
           $776 = HEAP32[$pWInfo>>2]|0;
           $777 = (_sqlite3WhereIsOrdered($776)|0);
           $778 = HEAP32[$pGroupBy>>2]|0;
           $779 = HEAP32[$778>>2]|0;
           $780 = ($777|0)==($779|0);
           if ($780) {
            HEAP32[$groupBySort>>2] = 0;
           } else {
            $781 = HEAP32[$1>>2]|0;
            $782 = HEAP8[$sDistinct>>0]|0;
            $783 = $782&255;
            $784 = ($783|0)!=(0);
            if ($784) {
             $785 = HEAP32[$2>>2]|0;
             $786 = ((($785)) + 6|0);
             $787 = HEAP16[$786>>1]|0;
             $788 = $787&65535;
             $789 = $788 & 1;
             $790 = ($789|0)==(0);
             $792 = $790;
            } else {
             $792 = 0;
            }
            $791 = $792 ? 32179 : 32188;
            _explainTempTable($781,$791);
            HEAP32[$groupBySort>>2] = 1;
            $793 = HEAP32[$pGroupBy>>2]|0;
            $794 = HEAP32[$793>>2]|0;
            HEAP32[$nGroupBy>>2] = $794;
            $795 = HEAP32[$nGroupBy>>2]|0;
            HEAP32[$nCol>>2] = $795;
            $796 = HEAP32[$nGroupBy>>2]|0;
            HEAP32[$j>>2] = $796;
            HEAP32[$i>>2] = 0;
            while(1) {
             $797 = HEAP32[$i>>2]|0;
             $798 = ((($sAggInfo)) + 32|0);
             $799 = HEAP32[$798>>2]|0;
             $800 = ($797|0)<($799|0);
             if (!($800)) {
              break;
             }
             $801 = HEAP32[$i>>2]|0;
             $802 = ((($sAggInfo)) + 28|0);
             $803 = HEAP32[$802>>2]|0;
             $804 = (($803) + (($801*24)|0)|0);
             $805 = ((($804)) + 12|0);
             $806 = HEAP32[$805>>2]|0;
             $807 = HEAP32[$j>>2]|0;
             $808 = ($806|0)>=($807|0);
             if ($808) {
              $809 = HEAP32[$nCol>>2]|0;
              $810 = (($809) + 1)|0;
              HEAP32[$nCol>>2] = $810;
              $811 = HEAP32[$j>>2]|0;
              $812 = (($811) + 1)|0;
              HEAP32[$j>>2] = $812;
             }
             $813 = HEAP32[$i>>2]|0;
             $814 = (($813) + 1)|0;
             HEAP32[$i>>2] = $814;
            }
            $815 = HEAP32[$1>>2]|0;
            $816 = HEAP32[$nCol>>2]|0;
            $817 = (_sqlite3GetTempRange($815,$816)|0);
            HEAP32[$regBase>>2] = $817;
            $818 = HEAP32[$1>>2]|0;
            _sqlite3ExprCacheClear($818);
            $819 = HEAP32[$1>>2]|0;
            $820 = HEAP32[$pGroupBy>>2]|0;
            $821 = HEAP32[$regBase>>2]|0;
            (_sqlite3ExprCodeExprList($819,$820,$821,0,0)|0);
            $822 = HEAP32[$nGroupBy>>2]|0;
            HEAP32[$j>>2] = $822;
            HEAP32[$i>>2] = 0;
            while(1) {
             $823 = HEAP32[$i>>2]|0;
             $824 = ((($sAggInfo)) + 32|0);
             $825 = HEAP32[$824>>2]|0;
             $826 = ($823|0)<($825|0);
             if (!($826)) {
              break;
             }
             $827 = HEAP32[$i>>2]|0;
             $828 = ((($sAggInfo)) + 28|0);
             $829 = HEAP32[$828>>2]|0;
             $830 = (($829) + (($827*24)|0)|0);
             HEAP32[$pCol>>2] = $830;
             $831 = HEAP32[$pCol>>2]|0;
             $832 = ((($831)) + 12|0);
             $833 = HEAP32[$832>>2]|0;
             $834 = HEAP32[$j>>2]|0;
             $835 = ($833|0)>=($834|0);
             if ($835) {
              $836 = HEAP32[$j>>2]|0;
              $837 = HEAP32[$regBase>>2]|0;
              $838 = (($836) + ($837))|0;
              HEAP32[$r1>>2] = $838;
              $839 = HEAP32[$1>>2]|0;
              $840 = HEAP32[$pCol>>2]|0;
              $841 = HEAP32[$840>>2]|0;
              $842 = HEAP32[$pCol>>2]|0;
              $843 = ((($842)) + 8|0);
              $844 = HEAP32[$843>>2]|0;
              $845 = HEAP32[$pCol>>2]|0;
              $846 = ((($845)) + 4|0);
              $847 = HEAP32[$846>>2]|0;
              $848 = HEAP32[$r1>>2]|0;
              _sqlite3ExprCodeGetColumnToReg($839,$841,$844,$847,$848);
              $849 = HEAP32[$j>>2]|0;
              $850 = (($849) + 1)|0;
              HEAP32[$j>>2] = $850;
             }
             $851 = HEAP32[$i>>2]|0;
             $852 = (($851) + 1)|0;
             HEAP32[$i>>2] = $852;
            }
            $853 = HEAP32[$1>>2]|0;
            $854 = (_sqlite3GetTempReg($853)|0);
            HEAP32[$regRecord>>2] = $854;
            $855 = HEAP32[$v>>2]|0;
            $856 = HEAP32[$regBase>>2]|0;
            $857 = HEAP32[$nCol>>2]|0;
            $858 = HEAP32[$regRecord>>2]|0;
            (_sqlite3VdbeAddOp3($855,49,$856,$857,$858)|0);
            $859 = HEAP32[$v>>2]|0;
            $860 = ((($sAggInfo)) + 4|0);
            $861 = HEAP32[$860>>2]|0;
            $862 = HEAP32[$regRecord>>2]|0;
            (_sqlite3VdbeAddOp2($859,109,$861,$862)|0);
            $863 = HEAP32[$1>>2]|0;
            $864 = HEAP32[$regRecord>>2]|0;
            _sqlite3ReleaseTempReg($863,$864);
            $865 = HEAP32[$1>>2]|0;
            $866 = HEAP32[$regBase>>2]|0;
            $867 = HEAP32[$nCol>>2]|0;
            _sqlite3ReleaseTempRange($865,$866,$867);
            $868 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($868);
            $869 = HEAP32[$1>>2]|0;
            $870 = ((($869)) + 72|0);
            $871 = HEAP32[$870>>2]|0;
            $872 = (($871) + 1)|0;
            HEAP32[$870>>2] = $872;
            HEAP32[$sortPTab>>2] = $871;
            $873 = ((($sAggInfo)) + 8|0);
            HEAP32[$873>>2] = $871;
            $874 = HEAP32[$1>>2]|0;
            $875 = (_sqlite3GetTempReg($874)|0);
            HEAP32[$sortOut>>2] = $875;
            $876 = HEAP32[$v>>2]|0;
            $877 = HEAP32[$sortPTab>>2]|0;
            $878 = HEAP32[$sortOut>>2]|0;
            $879 = HEAP32[$nCol>>2]|0;
            (_sqlite3VdbeAddOp3($876,60,$877,$878,$879)|0);
            $880 = HEAP32[$v>>2]|0;
            $881 = ((($sAggInfo)) + 4|0);
            $882 = HEAP32[$881>>2]|0;
            $883 = HEAP32[$addrEnd>>2]|0;
            (_sqlite3VdbeAddOp2($880,106,$882,$883)|0);
            $884 = ((($sAggInfo)) + 1|0);
            HEAP8[$884>>0] = 1;
            $885 = HEAP32[$1>>2]|0;
            _sqlite3ExprCacheClear($885);
           }
           $886 = HEAP32[$orderByGrp>>2]|0;
           $887 = ($886|0)!=(0);
           do {
            if ($887) {
             $888 = HEAP32[$db>>2]|0;
             $889 = ((($888)) + 60|0);
             $890 = HEAP16[$889>>1]|0;
             $891 = $890&65535;
             $892 = $891 & 4;
             $893 = ($892|0)==(0);
             if (!($893)) {
              break;
             }
             $894 = HEAP32[$groupBySort>>2]|0;
             $895 = ($894|0)!=(0);
             if (!($895)) {
              $896 = HEAP32[$pWInfo>>2]|0;
              $897 = (_sqlite3WhereIsSorted($896)|0);
              $898 = ($897|0)!=(0);
              if (!($898)) {
               break;
              }
             }
             HEAP32[$sSort>>2] = 0;
             $899 = HEAP32[$v>>2]|0;
             $900 = ((($sSort)) + 20|0);
             $901 = HEAP32[$900>>2]|0;
             (_sqlite3VdbeChangeToNoop($899,$901)|0);
            }
           } while(0);
           $902 = HEAP32[$v>>2]|0;
           $903 = (_sqlite3VdbeCurrentAddr($902)|0);
           HEAP32[$addrTopOfLoop>>2] = $903;
           $904 = HEAP32[$1>>2]|0;
           _sqlite3ExprCacheClear($904);
           $905 = HEAP32[$groupBySort>>2]|0;
           $906 = ($905|0)!=(0);
           if ($906) {
            $907 = HEAP32[$v>>2]|0;
            $908 = ((($sAggInfo)) + 4|0);
            $909 = HEAP32[$908>>2]|0;
            $910 = HEAP32[$sortOut>>2]|0;
            $911 = HEAP32[$sortPTab>>2]|0;
            (_sqlite3VdbeAddOp3($907,100,$909,$910,$911)|0);
           }
           HEAP32[$j>>2] = 0;
           while(1) {
            $912 = HEAP32[$j>>2]|0;
            $913 = HEAP32[$pGroupBy>>2]|0;
            $914 = HEAP32[$913>>2]|0;
            $915 = ($912|0)<($914|0);
            if (!($915)) {
             break;
            }
            $916 = HEAP32[$groupBySort>>2]|0;
            $917 = ($916|0)!=(0);
            if ($917) {
             $918 = HEAP32[$v>>2]|0;
             $919 = HEAP32[$sortPTab>>2]|0;
             $920 = HEAP32[$j>>2]|0;
             $921 = HEAP32[$iBMem>>2]|0;
             $922 = HEAP32[$j>>2]|0;
             $923 = (($921) + ($922))|0;
             (_sqlite3VdbeAddOp3($918,47,$919,$920,$923)|0);
            } else {
             HEAP8[$sAggInfo>>0] = 1;
             $924 = HEAP32[$1>>2]|0;
             $925 = HEAP32[$j>>2]|0;
             $926 = HEAP32[$pGroupBy>>2]|0;
             $927 = ((($926)) + 4|0);
             $928 = HEAP32[$927>>2]|0;
             $929 = (($928) + (($925*20)|0)|0);
             $930 = HEAP32[$929>>2]|0;
             $931 = HEAP32[$iBMem>>2]|0;
             $932 = HEAP32[$j>>2]|0;
             $933 = (($931) + ($932))|0;
             _sqlite3ExprCode($924,$930,$933);
            }
            $934 = HEAP32[$j>>2]|0;
            $935 = (($934) + 1)|0;
            HEAP32[$j>>2] = $935;
           }
           $936 = HEAP32[$v>>2]|0;
           $937 = HEAP32[$iAMem>>2]|0;
           $938 = HEAP32[$iBMem>>2]|0;
           $939 = HEAP32[$pGroupBy>>2]|0;
           $940 = HEAP32[$939>>2]|0;
           $941 = HEAP32[$pKeyInfo4>>2]|0;
           $942 = (_sqlite3KeyInfoRef($941)|0);
           (_sqlite3VdbeAddOp4($936,42,$937,$938,$940,$942,-6)|0);
           $943 = HEAP32[$v>>2]|0;
           $944 = (_sqlite3VdbeCurrentAddr($943)|0);
           HEAP32[$addr1>>2] = $944;
           $945 = HEAP32[$v>>2]|0;
           $946 = HEAP32[$addr1>>2]|0;
           $947 = (($946) + 1)|0;
           $948 = HEAP32[$addr1>>2]|0;
           $949 = (($948) + 1)|0;
           (_sqlite3VdbeAddOp3($945,43,$947,0,$949)|0);
           $950 = HEAP32[$1>>2]|0;
           $951 = HEAP32[$iBMem>>2]|0;
           $952 = HEAP32[$iAMem>>2]|0;
           $953 = HEAP32[$pGroupBy>>2]|0;
           $954 = HEAP32[$953>>2]|0;
           _sqlite3ExprCodeMove($950,$951,$952,$954);
           $955 = HEAP32[$v>>2]|0;
           $956 = HEAP32[$regOutputRow>>2]|0;
           $957 = HEAP32[$addrOutputRow>>2]|0;
           (_sqlite3VdbeAddOp2($955,14,$956,$957)|0);
           $958 = HEAP32[$v>>2]|0;
           $959 = HEAP32[$iAbortFlag>>2]|0;
           $960 = HEAP32[$addrEnd>>2]|0;
           (_sqlite3VdbeAddOp2($958,138,$959,$960)|0);
           $961 = HEAP32[$v>>2]|0;
           $962 = HEAP32[$regReset>>2]|0;
           $963 = HEAP32[$addrReset>>2]|0;
           (_sqlite3VdbeAddOp2($961,14,$962,$963)|0);
           $964 = HEAP32[$v>>2]|0;
           $965 = HEAP32[$addr1>>2]|0;
           _sqlite3VdbeJumpHere($964,$965);
           $966 = HEAP32[$1>>2]|0;
           _updateAccumulator($966,$sAggInfo);
           $967 = HEAP32[$v>>2]|0;
           $968 = HEAP32[$iUseFlag>>2]|0;
           (_sqlite3VdbeAddOp2($967,22,1,$968)|0);
           $969 = HEAP32[$groupBySort>>2]|0;
           $970 = ($969|0)!=(0);
           if ($970) {
            $971 = HEAP32[$v>>2]|0;
            $972 = ((($sAggInfo)) + 4|0);
            $973 = HEAP32[$972>>2]|0;
            $974 = HEAP32[$addrTopOfLoop>>2]|0;
            (_sqlite3VdbeAddOp2($971,3,$973,$974)|0);
           } else {
            $975 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($975);
            $976 = HEAP32[$v>>2]|0;
            $977 = HEAP32[$addrSortingIdx>>2]|0;
            (_sqlite3VdbeChangeToNoop($976,$977)|0);
           }
           $978 = HEAP32[$v>>2]|0;
           $979 = HEAP32[$regOutputRow>>2]|0;
           $980 = HEAP32[$addrOutputRow>>2]|0;
           (_sqlite3VdbeAddOp2($978,14,$979,$980)|0);
           $981 = HEAP32[$v>>2]|0;
           $982 = HEAP32[$addrEnd>>2]|0;
           (_sqlite3VdbeGoto($981,$982)|0);
           $983 = HEAP32[$v>>2]|0;
           $984 = (_sqlite3VdbeCurrentAddr($983)|0);
           HEAP32[$addrSetAbort>>2] = $984;
           $985 = HEAP32[$v>>2]|0;
           $986 = HEAP32[$iAbortFlag>>2]|0;
           (_sqlite3VdbeAddOp2($985,22,1,$986)|0);
           $987 = HEAP32[$v>>2]|0;
           $988 = HEAP32[$regOutputRow>>2]|0;
           (_sqlite3VdbeAddOp1($987,15,$988)|0);
           $989 = HEAP32[$v>>2]|0;
           $990 = HEAP32[$addrOutputRow>>2]|0;
           _sqlite3VdbeResolveLabel($989,$990);
           $991 = HEAP32[$v>>2]|0;
           $992 = (_sqlite3VdbeCurrentAddr($991)|0);
           HEAP32[$addrOutputRow>>2] = $992;
           $993 = HEAP32[$v>>2]|0;
           $994 = HEAP32[$iUseFlag>>2]|0;
           $995 = HEAP32[$addrOutputRow>>2]|0;
           $996 = (($995) + 2)|0;
           (_sqlite3VdbeAddOp2($993,138,$994,$996)|0);
           $997 = HEAP32[$v>>2]|0;
           $998 = HEAP32[$regOutputRow>>2]|0;
           (_sqlite3VdbeAddOp1($997,15,$998)|0);
           $999 = HEAP32[$1>>2]|0;
           _finalizeAggFunctions($999,$sAggInfo);
           $1000 = HEAP32[$1>>2]|0;
           $1001 = HEAP32[$pHaving>>2]|0;
           $1002 = HEAP32[$addrOutputRow>>2]|0;
           $1003 = (($1002) + 1)|0;
           _sqlite3ExprIfFalse($1000,$1001,$1003,16);
           $1004 = HEAP32[$1>>2]|0;
           $1005 = HEAP32[$2>>2]|0;
           $1006 = HEAP32[$2>>2]|0;
           $1007 = HEAP32[$1006>>2]|0;
           $1008 = HEAP32[$3>>2]|0;
           $1009 = HEAP32[$addrOutputRow>>2]|0;
           $1010 = (($1009) + 1)|0;
           $1011 = HEAP32[$addrSetAbort>>2]|0;
           _selectInnerLoop($1004,$1005,$1007,-1,$sSort,$sDistinct,$1008,$1010,$1011);
           $1012 = HEAP32[$v>>2]|0;
           $1013 = HEAP32[$regOutputRow>>2]|0;
           (_sqlite3VdbeAddOp1($1012,15,$1013)|0);
           $1014 = HEAP32[$v>>2]|0;
           $1015 = HEAP32[$addrReset>>2]|0;
           _sqlite3VdbeResolveLabel($1014,$1015);
           $1016 = HEAP32[$1>>2]|0;
           _resetAccumulator($1016,$sAggInfo);
           $1017 = HEAP32[$v>>2]|0;
           $1018 = HEAP32[$regReset>>2]|0;
           (_sqlite3VdbeAddOp1($1017,15,$1018)|0);
          } else {
           HEAP32[$pDel>>2] = 0;
           $1019 = HEAP32[$2>>2]|0;
           $1020 = (_isSimpleCount($1019,$sAggInfo)|0);
           HEAP32[$pTab5>>2] = $1020;
           $1021 = ($1020|0)!=(0|0);
           if ($1021) {
            $1022 = HEAP32[$1>>2]|0;
            $1023 = HEAP32[$1022>>2]|0;
            $1024 = HEAP32[$pTab5>>2]|0;
            $1025 = ((($1024)) + 64|0);
            $1026 = HEAP32[$1025>>2]|0;
            $1027 = (_sqlite3SchemaToIndex($1023,$1026)|0);
            HEAP32[$iDb>>2] = $1027;
            $1028 = HEAP32[$1>>2]|0;
            $1029 = ((($1028)) + 72|0);
            $1030 = HEAP32[$1029>>2]|0;
            $1031 = (($1030) + 1)|0;
            HEAP32[$1029>>2] = $1031;
            HEAP32[$iCsr>>2] = $1030;
            HEAP32[$pKeyInfo6>>2] = 0;
            HEAP32[$pBest>>2] = 0;
            $1032 = HEAP32[$pTab5>>2]|0;
            $1033 = ((($1032)) + 28|0);
            $1034 = HEAP32[$1033>>2]|0;
            HEAP32[$iRoot>>2] = $1034;
            $1035 = HEAP32[$1>>2]|0;
            $1036 = HEAP32[$iDb>>2]|0;
            _sqlite3CodeVerifySchema($1035,$1036);
            $1037 = HEAP32[$1>>2]|0;
            $1038 = HEAP32[$iDb>>2]|0;
            $1039 = HEAP32[$pTab5>>2]|0;
            $1040 = ((($1039)) + 28|0);
            $1041 = HEAP32[$1040>>2]|0;
            $1042 = HEAP32[$pTab5>>2]|0;
            $1043 = HEAP32[$1042>>2]|0;
            _sqlite3TableLock($1037,$1038,$1041,0,$1043);
            $1044 = HEAP32[$pTab5>>2]|0;
            $1045 = ((($1044)) + 42|0);
            $1046 = HEAP8[$1045>>0]|0;
            $1047 = $1046&255;
            $1048 = $1047 & 32;
            $1049 = ($1048|0)==(0);
            if (!($1049)) {
             $1050 = HEAP32[$pTab5>>2]|0;
             $1051 = (_sqlite3PrimaryKeyIndex($1050)|0);
             HEAP32[$pBest>>2] = $1051;
            }
            $1052 = HEAP32[$pTab5>>2]|0;
            $1053 = ((($1052)) + 8|0);
            $1054 = HEAP32[$1053>>2]|0;
            HEAP32[$pIdx>>2] = $1054;
            while(1) {
             $1055 = HEAP32[$pIdx>>2]|0;
             $1056 = ($1055|0)!=(0|0);
             if (!($1056)) {
              break;
             }
             $1057 = HEAP32[$pIdx>>2]|0;
             $1058 = ((($1057)) + 55|0);
             $1059 = HEAP8[$1058>>0]|0;
             $1060 = ($1059&255) >>> 2;
             $1061 = $1060 & 1;
             $1062 = $1061&255;
             $1063 = ($1062|0)==(0);
             do {
              if ($1063) {
               $1064 = HEAP32[$pIdx>>2]|0;
               $1065 = ((($1064)) + 48|0);
               $1066 = HEAP16[$1065>>1]|0;
               $1067 = $1066 << 16 >> 16;
               $1068 = HEAP32[$pTab5>>2]|0;
               $1069 = ((($1068)) + 40|0);
               $1070 = HEAP16[$1069>>1]|0;
               $1071 = $1070 << 16 >> 16;
               $1072 = ($1067|0)<($1071|0);
               if (!($1072)) {
                break;
               }
               $1073 = HEAP32[$pIdx>>2]|0;
               $1074 = ((($1073)) + 36|0);
               $1075 = HEAP32[$1074>>2]|0;
               $1076 = ($1075|0)==(0|0);
               if (!($1076)) {
                break;
               }
               $1077 = HEAP32[$pBest>>2]|0;
               $1078 = ($1077|0)!=(0|0);
               if ($1078) {
                $1079 = HEAP32[$pIdx>>2]|0;
                $1080 = ((($1079)) + 48|0);
                $1081 = HEAP16[$1080>>1]|0;
                $1082 = $1081 << 16 >> 16;
                $1083 = HEAP32[$pBest>>2]|0;
                $1084 = ((($1083)) + 48|0);
                $1085 = HEAP16[$1084>>1]|0;
                $1086 = $1085 << 16 >> 16;
                $1087 = ($1082|0)<($1086|0);
                if (!($1087)) {
                 break;
                }
               }
               $1088 = HEAP32[$pIdx>>2]|0;
               HEAP32[$pBest>>2] = $1088;
              }
             } while(0);
             $1089 = HEAP32[$pIdx>>2]|0;
             $1090 = ((($1089)) + 20|0);
             $1091 = HEAP32[$1090>>2]|0;
             HEAP32[$pIdx>>2] = $1091;
            }
            $1092 = HEAP32[$pBest>>2]|0;
            $1093 = ($1092|0)!=(0|0);
            if ($1093) {
             $1094 = HEAP32[$pBest>>2]|0;
             $1095 = ((($1094)) + 44|0);
             $1096 = HEAP32[$1095>>2]|0;
             HEAP32[$iRoot>>2] = $1096;
             $1097 = HEAP32[$1>>2]|0;
             $1098 = HEAP32[$pBest>>2]|0;
             $1099 = (_sqlite3KeyInfoOfIndex($1097,$1098)|0);
             HEAP32[$pKeyInfo6>>2] = $1099;
            }
            $1100 = HEAP32[$v>>2]|0;
            $1101 = HEAP32[$iCsr>>2]|0;
            $1102 = HEAP32[$iRoot>>2]|0;
            $1103 = HEAP32[$iDb>>2]|0;
            (_sqlite3VdbeAddOp4Int($1100,54,$1101,$1102,$1103,1)|0);
            $1104 = HEAP32[$pKeyInfo6>>2]|0;
            $1105 = ($1104|0)!=(0|0);
            if ($1105) {
             $1106 = HEAP32[$v>>2]|0;
             $1107 = HEAP32[$pKeyInfo6>>2]|0;
             _sqlite3VdbeChangeP4($1106,-1,$1107,-6);
            }
            $1108 = HEAP32[$v>>2]|0;
            $1109 = HEAP32[$iCsr>>2]|0;
            $1110 = ((($sAggInfo)) + 40|0);
            $1111 = HEAP32[$1110>>2]|0;
            $1112 = ((($1111)) + 8|0);
            $1113 = HEAP32[$1112>>2]|0;
            (_sqlite3VdbeAddOp2($1108,50,$1109,$1113)|0);
            $1114 = HEAP32[$v>>2]|0;
            $1115 = HEAP32[$iCsr>>2]|0;
            (_sqlite3VdbeAddOp1($1114,61,$1115)|0);
            $1116 = HEAP32[$1>>2]|0;
            $1117 = HEAP32[$pTab5>>2]|0;
            $1118 = HEAP32[$pBest>>2]|0;
            _explainSimpleCount($1116,$1117,$1118);
           } else {
            HEAP32[$pMinMax>>2] = 0;
            HEAP8[$flag>>0] = 0;
            $1119 = HEAP32[$2>>2]|0;
            $1120 = ((($1119)) + 44|0);
            $1121 = HEAP32[$1120>>2]|0;
            $1122 = ($1121|0)==(0|0);
            if ($1122) {
             $1123 = (_minMaxQuery($sAggInfo,$pMinMax)|0);
             HEAP8[$flag>>0] = $1123;
            }
            $1124 = HEAP8[$flag>>0]|0;
            $1125 = ($1124<<24>>24)!=(0);
            do {
             if ($1125) {
              $1126 = HEAP32[$db>>2]|0;
              $1127 = HEAP32[$pMinMax>>2]|0;
              $1128 = (_sqlite3ExprListDup($1126,$1127,0)|0);
              HEAP32[$pMinMax>>2] = $1128;
              $1129 = HEAP32[$pMinMax>>2]|0;
              HEAP32[$pDel>>2] = $1129;
              $1130 = HEAP32[$db>>2]|0;
              $1131 = ((($1130)) + 65|0);
              $1132 = HEAP8[$1131>>0]|0;
              $1133 = ($1132<<24>>24)!=(0);
              if ($1133) {
               break;
              }
              $1134 = HEAP8[$flag>>0]|0;
              $1135 = $1134&255;
              $1136 = ($1135|0)!=(1);
              $1137 = $1136 ? 1 : 0;
              $1138 = $1137&255;
              $1139 = HEAP32[$pMinMax>>2]|0;
              $1140 = ((($1139)) + 4|0);
              $1141 = HEAP32[$1140>>2]|0;
              $1142 = ((($1141)) + 12|0);
              HEAP8[$1142>>0] = $1138;
              $1143 = HEAP32[$pMinMax>>2]|0;
              $1144 = ((($1143)) + 4|0);
              $1145 = HEAP32[$1144>>2]|0;
              $1146 = HEAP32[$1145>>2]|0;
              HEAP8[$1146>>0] = -104;
             }
            } while(0);
            $1147 = HEAP32[$1>>2]|0;
            _resetAccumulator($1147,$sAggInfo);
            $1148 = HEAP32[$1>>2]|0;
            $1149 = HEAP32[$pTabList>>2]|0;
            $1150 = HEAP32[$pWhere>>2]|0;
            $1151 = HEAP32[$pMinMax>>2]|0;
            $1152 = HEAP8[$flag>>0]|0;
            $1153 = $1152&255;
            $1154 = (_sqlite3WhereBegin($1148,$1149,$1150,$1151,0,$1153,0)|0);
            HEAP32[$pWInfo>>2] = $1154;
            $1155 = HEAP32[$pWInfo>>2]|0;
            $1156 = ($1155|0)==(0|0);
            if ($1156) {
             $1157 = HEAP32[$db>>2]|0;
             $1158 = HEAP32[$pDel>>2]|0;
             _sqlite3ExprListDelete($1157,$1158);
             break;
            }
            $1159 = HEAP32[$1>>2]|0;
            _updateAccumulator($1159,$sAggInfo);
            $1160 = HEAP32[$pWInfo>>2]|0;
            $1161 = (_sqlite3WhereIsOrdered($1160)|0);
            $1162 = ($1161|0)>(0);
            if ($1162) {
             $1163 = HEAP32[$v>>2]|0;
             $1164 = HEAP32[$pWInfo>>2]|0;
             $1165 = (_sqlite3WhereBreakLabel($1164)|0);
             (_sqlite3VdbeGoto($1163,$1165)|0);
            }
            $1166 = HEAP32[$pWInfo>>2]|0;
            _sqlite3WhereEnd($1166);
            $1167 = HEAP32[$1>>2]|0;
            _finalizeAggFunctions($1167,$sAggInfo);
           }
           HEAP32[$sSort>>2] = 0;
           $1168 = HEAP32[$1>>2]|0;
           $1169 = HEAP32[$pHaving>>2]|0;
           $1170 = HEAP32[$addrEnd>>2]|0;
           _sqlite3ExprIfFalse($1168,$1169,$1170,16);
           $1171 = HEAP32[$1>>2]|0;
           $1172 = HEAP32[$2>>2]|0;
           $1173 = HEAP32[$2>>2]|0;
           $1174 = HEAP32[$1173>>2]|0;
           $1175 = HEAP32[$3>>2]|0;
           $1176 = HEAP32[$addrEnd>>2]|0;
           $1177 = HEAP32[$addrEnd>>2]|0;
           _selectInnerLoop($1171,$1172,$1174,-1,0,0,$1175,$1176,$1177);
           $1178 = HEAP32[$db>>2]|0;
           $1179 = HEAP32[$pDel>>2]|0;
           _sqlite3ExprListDelete($1178,$1179);
          }
          $1180 = HEAP32[$v>>2]|0;
          $1181 = HEAP32[$addrEnd>>2]|0;
          _sqlite3VdbeResolveLabel($1180,$1181);
         }
         $1182 = ((($sDistinct)) + 1|0);
         $1183 = HEAP8[$1182>>0]|0;
         $1184 = $1183&255;
         $1185 = ($1184|0)==(3);
         if ($1185) {
          $1186 = HEAP32[$1>>2]|0;
          _explainTempTable($1186,32179);
         }
         $1187 = HEAP32[$sSort>>2]|0;
         $1188 = ($1187|0)!=(0|0);
         if ($1188) {
          $1189 = HEAP32[$1>>2]|0;
          $1190 = ((($sSort)) + 4|0);
          $1191 = HEAP32[$1190>>2]|0;
          $1192 = ($1191|0)>(0);
          $1193 = $1192 ? 32197 : 32220;
          _explainTempTable($1189,$1193);
          $1194 = HEAP32[$1>>2]|0;
          $1195 = HEAP32[$2>>2]|0;
          $1196 = HEAP32[$pEList>>2]|0;
          $1197 = HEAP32[$1196>>2]|0;
          $1198 = HEAP32[$3>>2]|0;
          _generateSortTail($1194,$1195,$sSort,$1197,$1198);
         }
         $1199 = HEAP32[$v>>2]|0;
         $1200 = HEAP32[$iEnd>>2]|0;
         _sqlite3VdbeResolveLabel($1199,$1200);
         $1201 = HEAP32[$1>>2]|0;
         $1202 = ((($1201)) + 68|0);
         $1203 = HEAP32[$1202>>2]|0;
         $1204 = ($1203|0)>(0);
         $1205 = $1204&1;
         HEAP32[$rc>>2] = $1205;
        }
       }
      }
     }
    } while(0);
    $1206 = HEAP32[$iRestoreSelectId>>2]|0;
    $1207 = HEAP32[$1>>2]|0;
    $1208 = ((($1207)) + 468|0);
    HEAP32[$1208>>2] = $1206;
    $1209 = HEAP32[$rc>>2]|0;
    $1210 = ($1209|0)==(0);
    if ($1210) {
     $1211 = HEAP32[$3>>2]|0;
     $1212 = HEAP8[$1211>>0]|0;
     $1213 = $1212&255;
     $1214 = ($1213|0)==(9);
     if ($1214) {
      $1215 = HEAP32[$1>>2]|0;
      $1216 = HEAP32[$pTabList>>2]|0;
      $1217 = HEAP32[$pEList>>2]|0;
      _generateColumnNames($1215,$1216,$1217);
     }
    }
    $1218 = HEAP32[$db>>2]|0;
    $1219 = ((($sAggInfo)) + 28|0);
    $1220 = HEAP32[$1219>>2]|0;
    _sqlite3DbFree($1218,$1220);
    $1221 = HEAP32[$db>>2]|0;
    $1222 = ((($sAggInfo)) + 40|0);
    $1223 = HEAP32[$1222>>2]|0;
    _sqlite3DbFree($1221,$1223);
    $1224 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $1224;
    $1225 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($1225|0);
   }
  }
 }
 HEAP32[$0>>2] = 1;
 $1225 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($1225|0);
}
function _parserDoubleLinkSelect($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $cnt = 0, $mxSelect = 0, $pLoop = 0, $pNext = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pNext = sp + 16|0;
 $pLoop = sp + 12|0;
 $mxSelect = sp + 8|0;
 $cnt = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$pNext>>2] = 0;
 HEAP32[$cnt>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pLoop>>2] = $6;
 while(1) {
  $7 = HEAP32[$pLoop>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$pNext>>2]|0;
  $10 = HEAP32[$pLoop>>2]|0;
  $11 = ((($10)) + 56|0);
  HEAP32[$11>>2] = $9;
  $12 = HEAP32[$pLoop>>2]|0;
  $13 = ((($12)) + 6|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = $15 | 128;
  $17 = $16&65535;
  HEAP16[$13>>1] = $17;
  $18 = HEAP32[$pLoop>>2]|0;
  HEAP32[$pNext>>2] = $18;
  $19 = HEAP32[$pLoop>>2]|0;
  $20 = ((($19)) + 52|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pLoop>>2] = $21;
  $22 = HEAP32[$cnt>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$cnt>>2] = $23;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 6|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 512;
 $29 = ($28|0)==(0);
 if (!($29)) {
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$0>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ((($31)) + 88|0);
 $33 = ((($32)) + 16|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$mxSelect>>2] = $34;
 $35 = ($34|0)>(0);
 if (!($35)) {
  STACKTOP = sp;return;
 }
 $36 = HEAP32[$cnt>>2]|0;
 $37 = HEAP32[$mxSelect>>2]|0;
 $38 = ($36|0)>($37|0);
 if (!($38)) {
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($39,33162,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3SrcListAppendFromTerm($pParse,$p,$pTable,$pDatabase,$pAlias,$pSubquery,$pOn,$pUsing) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pTable = $pTable|0;
 $pDatabase = $pDatabase|0;
 $pAlias = $pAlias|0;
 $pSubquery = $pSubquery|0;
 $pOn = $pOn|0;
 $pUsing = $pUsing|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pItem = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $8 = sp + 12|0;
 $pItem = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pTable;
 HEAP32[$4>>2] = $pDatabase;
 HEAP32[$5>>2] = $pAlias;
 HEAP32[$6>>2] = $pSubquery;
 HEAP32[$7>>2] = $pOn;
 HEAP32[$8>>2] = $pUsing;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  label = 4;
 } else {
  $13 = HEAP32[$7>>2]|0;
  $14 = ($13|0)!=(0|0);
  $15 = HEAP32[$8>>2]|0;
  $16 = ($15|0)!=(0|0);
  $or$cond = $14 | $16;
  if ($or$cond) {
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$7>>2]|0;
   $19 = ($18|0)!=(0|0);
   $20 = $19 ? 33232 : 33235;
   HEAP32[$vararg_buffer>>2] = $20;
   _sqlite3ErrorMsg($17,33196,$vararg_buffer);
  } else {
   label = 4;
  }
 }
 if ((label|0) == 4) {
  $21 = HEAP32[$db>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  $24 = HEAP32[$4>>2]|0;
  $25 = (_sqlite3SrcListAppend($21,$22,$23,$24)|0);
  HEAP32[$2>>2] = $25;
  $26 = HEAP32[$2>>2]|0;
  $27 = ($26|0)==(0|0);
  if (!($27)) {
   $28 = HEAP32[$2>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0);
   if (!($30)) {
    $31 = HEAP32[$2>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) - 1)|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = ((($34)) + 8|0);
    $36 = (($35) + (($33*72)|0)|0);
    HEAP32[$pItem>>2] = $36;
    $37 = HEAP32[$5>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$db>>2]|0;
     $42 = HEAP32[$5>>2]|0;
     $43 = (_sqlite3NameFromToken($41,$42)|0);
     $44 = HEAP32[$pItem>>2]|0;
     $45 = ((($44)) + 12|0);
     HEAP32[$45>>2] = $43;
    }
    $46 = HEAP32[$6>>2]|0;
    $47 = HEAP32[$pItem>>2]|0;
    $48 = ((($47)) + 20|0);
    HEAP32[$48>>2] = $46;
    $49 = HEAP32[$7>>2]|0;
    $50 = HEAP32[$pItem>>2]|0;
    $51 = ((($50)) + 48|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$8>>2]|0;
    $53 = HEAP32[$pItem>>2]|0;
    $54 = ((($53)) + 52|0);
    HEAP32[$54>>2] = $52;
    $55 = HEAP32[$2>>2]|0;
    HEAP32[$0>>2] = $55;
    $62 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($62|0);
   }
  }
 }
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($56,$57);
 $58 = HEAP32[$db>>2]|0;
 $59 = HEAP32[$8>>2]|0;
 _sqlite3IdListDelete($58,$59);
 $60 = HEAP32[$db>>2]|0;
 $61 = HEAP32[$6>>2]|0;
 _sqlite3SelectDelete($60,$61);
 HEAP32[$0>>2] = 0;
 $62 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3SelectNew($pParse,$pEList,$pSrc,$pWhere,$pGroupBy,$pHaving,$pOrderBy,$selFlags,$pLimit,$pOffset) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pSrc = $pSrc|0;
 $pWhere = $pWhere|0;
 $pGroupBy = $pGroupBy|0;
 $pHaving = $pHaving|0;
 $pOrderBy = $pOrderBy|0;
 $selFlags = $selFlags|0;
 $pLimit = $pLimit|0;
 $pOffset = $pOffset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $db = 0, $pNew = 0, $standin = 0, $standin$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 88|0;
 $7 = sp + 116|0;
 $8 = sp + 84|0;
 $9 = sp + 80|0;
 $pNew = sp + 76|0;
 $standin = sp;
 $db = sp + 72|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $pGroupBy;
 HEAP32[$5>>2] = $pHaving;
 HEAP32[$6>>2] = $pOrderBy;
 HEAP16[$7>>1] = $selFlags;
 HEAP32[$8>>2] = $pLimit;
 HEAP32[$9>>2] = $pOffset;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (_sqlite3DbMallocRawNN($12,72,0)|0);
 HEAP32[$pNew>>2] = $13;
 $14 = HEAP32[$pNew>>2]|0;
 $15 = ($14|0)==(0|0);
 $standin$ = $15 ? $standin : $13;
 HEAP32[$pNew>>2] = $standin$;
 $16 = HEAP32[$1>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$db>>2]|0;
  $20 = (_sqlite3Expr($19,158,0)|0);
  $21 = (_sqlite3ExprListAppend($18,0,$20)|0);
  HEAP32[$1>>2] = $21;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$pNew>>2]|0;
 HEAP32[$23>>2] = $22;
 $24 = HEAP32[$pNew>>2]|0;
 $25 = ((($24)) + 4|0);
 HEAP8[$25>>0] = 119;
 $26 = HEAP16[$7>>1]|0;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = ((($27)) + 6|0);
 HEAP16[$28>>1] = $26;
 $29 = HEAP32[$pNew>>2]|0;
 $30 = ((($29)) + 8|0);
 HEAP32[$30>>2] = 0;
 $31 = HEAP32[$pNew>>2]|0;
 $32 = ((($31)) + 12|0);
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$pNew>>2]|0;
 $34 = ((($33)) + 16|0);
 HEAP32[$34>>2] = -1;
 $35 = HEAP32[$pNew>>2]|0;
 $36 = ((($35)) + 16|0);
 $37 = ((($36)) + 4|0);
 HEAP32[$37>>2] = -1;
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 24|0);
 $40 = $39;
 $41 = $40;
 HEAP32[$41>>2] = 0;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = 0;
 $44 = HEAP32[$2>>2]|0;
 $45 = ($44|0)==(0|0);
 if ($45) {
  $46 = HEAP32[$db>>2]|0;
  $47 = (_sqlite3DbMallocZero($46,80,0)|0);
  HEAP32[$2>>2] = $47;
 }
 $48 = HEAP32[$2>>2]|0;
 $49 = HEAP32[$pNew>>2]|0;
 $50 = ((($49)) + 32|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$3>>2]|0;
 $52 = HEAP32[$pNew>>2]|0;
 $53 = ((($52)) + 36|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$4>>2]|0;
 $55 = HEAP32[$pNew>>2]|0;
 $56 = ((($55)) + 40|0);
 HEAP32[$56>>2] = $54;
 $57 = HEAP32[$5>>2]|0;
 $58 = HEAP32[$pNew>>2]|0;
 $59 = ((($58)) + 44|0);
 HEAP32[$59>>2] = $57;
 $60 = HEAP32[$6>>2]|0;
 $61 = HEAP32[$pNew>>2]|0;
 $62 = ((($61)) + 48|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$pNew>>2]|0;
 $64 = ((($63)) + 52|0);
 HEAP32[$64>>2] = 0;
 $65 = HEAP32[$pNew>>2]|0;
 $66 = ((($65)) + 56|0);
 HEAP32[$66>>2] = 0;
 $67 = HEAP32[$8>>2]|0;
 $68 = HEAP32[$pNew>>2]|0;
 $69 = ((($68)) + 60|0);
 HEAP32[$69>>2] = $67;
 $70 = HEAP32[$9>>2]|0;
 $71 = HEAP32[$pNew>>2]|0;
 $72 = ((($71)) + 64|0);
 HEAP32[$72>>2] = $70;
 $73 = HEAP32[$pNew>>2]|0;
 $74 = ((($73)) + 68|0);
 HEAP32[$74>>2] = 0;
 $75 = HEAP32[$db>>2]|0;
 $76 = ((($75)) + 65|0);
 $77 = HEAP8[$76>>0]|0;
 $78 = ($77<<24>>24)!=(0);
 if (!($78)) {
  $84 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $79 = HEAP32[$db>>2]|0;
 $80 = HEAP32[$pNew>>2]|0;
 $81 = HEAP32[$pNew>>2]|0;
 $82 = ($81|0)!=($standin|0);
 $83 = $82&1;
 _clearSelect($79,$80,$83);
 HEAP32[$pNew>>2] = 0;
 $84 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($84|0);
}
function _sqlite3ExprListAppend($pParse,$pList,$pExpr) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $db = 0;
 var $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $db = sp + 8|0;
 $a = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pExpr;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$db>>2]|0;
  $9 = (_sqlite3DbMallocRawNN($8,8,0)|0);
  HEAP32[$2>>2] = $9;
  $10 = HEAP32[$2>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   label = 8;
  } else {
   $12 = HEAP32[$2>>2]|0;
   HEAP32[$12>>2] = 0;
   $13 = HEAP32[$db>>2]|0;
   $14 = (_sqlite3DbMallocRawNN($13,20,0)|0);
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 4|0);
   HEAP32[$16>>2] = $14;
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    label = 8;
   } else {
    label = 7;
   }
  }
 } else {
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) - 1)|0;
  $26 = $22 & $25;
  $27 = ($26|0)==(0);
  if ($27) {
   $28 = HEAP32[$db>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = ((($29)) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = $33<<1;
   $35 = ($34*20)|0;
   $36 = (_sqlite3DbRealloc($28,$31,$35,0)|0);
   HEAP32[$a>>2] = $36;
   $37 = HEAP32[$a>>2]|0;
   $38 = ($37|0)==(0|0);
   if ($38) {
    label = 8;
   } else {
    $39 = HEAP32[$a>>2]|0;
    $40 = HEAP32[$2>>2]|0;
    $41 = ((($40)) + 4|0);
    HEAP32[$41>>2] = $39;
    label = 7;
   }
  } else {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $42 = HEAP32[$2>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$42>>2] = $44;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 4|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + (($43*20)|0)|0);
  HEAP32[$pItem>>2] = $48;
  $49 = HEAP32[$pItem>>2]|0;
  ;HEAP32[$49>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;
  $50 = HEAP32[$3>>2]|0;
  $51 = HEAP32[$pItem>>2]|0;
  HEAP32[$51>>2] = $50;
  $52 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $52;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 8) {
  $53 = HEAP32[$db>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($53,$54);
  $55 = HEAP32[$db>>2]|0;
  $56 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($55,$56);
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _sqlite3ExprListSetName($pParse,$pList,$pName,$dequote) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pName = $pName|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pName;
 HEAP32[$3>>2] = $dequote;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + (($8*20)|0)|0);
 HEAP32[$pItem>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3DbStrNDup($14,$16,$19,0)|0);
 $21 = HEAP32[$pItem>>2]|0;
 $22 = ((($21)) + 4|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$3>>2]|0;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$pItem>>2]|0;
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$pItem>>2]|0;
 $30 = ((($29)) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 (_sqlite3Dequote($31)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprListSetSpan($pParse,$pList,$pSpan) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pSpan;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*20)|0)|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pItem>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3DbFree($14,$17);
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $24;
 $29 = $27;
 $30 = (($28) - ($29))|0;
 $31 = ($30|0)<(0);
 $32 = $31 << 31 >> 31;
 $33 = (_sqlite3DbStrNDup($18,$21,$30,$32)|0);
 $34 = HEAP32[$pItem>>2]|0;
 $35 = ((($34)) + 8|0);
 HEAP32[$35>>2] = $33;
 STACKTOP = sp;return;
}
function _sqlite3Expr($db,$op,$zToken) {
 $db = $db|0;
 $op = $op|0;
 $zToken = $zToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $x = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $zToken;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$x>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_sqlite3Strlen30($6)|0);
  $9 = $7;
 } else {
  $9 = 0;
 }
 $8 = ((($x)) + 4|0);
 HEAP32[$8>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (_sqlite3ExprAlloc($10,$11,$x,0)|0);
 STACKTOP = sp;return ($12|0);
}
function _sqlite3SrcListShiftJoinType($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 HEAP32[$i>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)>(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = (($8) - 1)|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = (($11) + (($9*72)|0)|0);
  $13 = ((($12)) + 36|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = (($17) + (($15*72)|0)|0);
  $19 = ((($18)) + 36|0);
  HEAP8[$19>>0] = $14;
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$i>>2] = $21;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = ((($23)) + 36|0);
 HEAP8[$24>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3SrcListIndexedBy($pParse,$p,$pIndexedBy) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pIndexedBy = $pIndexedBy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pIndexedBy;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = (($12) + (($10*72)|0)|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   $21 = HEAP32[$pItem>>2]|0;
   $22 = ((($21)) + 36|0);
   $23 = ((($22)) + 1|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 & -2;
   $26 = $25 | 1;
   HEAP8[$23>>0] = $26;
   STACKTOP = sp;return;
  }
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (_sqlite3NameFromToken($28,$29)|0);
 $31 = HEAP32[$pItem>>2]|0;
 $32 = ((($31)) + 64|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$pItem>>2]|0;
 $34 = ((($33)) + 64|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)!=(0|0);
 $37 = $36&1;
 $38 = HEAP32[$pItem>>2]|0;
 $39 = ((($38)) + 36|0);
 $40 = ((($39)) + 1|0);
 $41 = $37&255;
 $42 = HEAP8[$40>>0]|0;
 $43 = $41 & 1;
 $44 = ($43 << 1)&255;
 $45 = $42 & -3;
 $46 = $45 | $44;
 HEAP8[$40>>0] = $46;
 STACKTOP = sp;return;
}
function _sqlite3SrcListFuncArgs($pParse,$p,$pList) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 8|0);
  $10 = (($9) + (($7*72)|0)|0);
  HEAP32[$pItem>>2] = $10;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$pItem>>2]|0;
  $13 = ((($12)) + 64|0);
  HEAP32[$13>>2] = $11;
  $14 = HEAP32[$pItem>>2]|0;
  $15 = ((($14)) + 36|0);
  $16 = ((($15)) + 1|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 & -5;
  $19 = $18 | 4;
  HEAP8[$16>>0] = $19;
  STACKTOP = sp;return;
 } else {
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($21,$22);
  STACKTOP = sp;return;
 }
}
function _sqlite3SrcListAppend($db,$pList,$pTable,$pDatabase) {
 $db = $db|0;
 $pList = $pList|0;
 $pTable = $pTable|0;
 $pDatabase = $pDatabase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, $pTemp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pItem = sp + 4|0;
 $pTemp = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pTable;
 HEAP32[$4>>2] = $pDatabase;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3DbMallocRawNN($7,80,0)|0);
   HEAP32[$2>>2] = $8;
   $9 = HEAP32[$2>>2]|0;
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = HEAP32[$2>>2]|0;
    $12 = ((($11)) + 4|0);
    HEAP32[$12>>2] = 1;
    $13 = HEAP32[$2>>2]|0;
    HEAP32[$13>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 } while(0);
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3SrcListEnlarge($14,$15,1,$17)|0);
 HEAP32[$2>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 65|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = ($21<<24>>24)!=(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  _sqlite3SrcListDelete($23,$24);
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) - 1)|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = (($29) + (($27*72)|0)|0);
 HEAP32[$pItem>>2] = $30;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$4>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)==(0|0);
  if ($35) {
   HEAP32[$4>>2] = 0;
  }
 }
 $36 = HEAP32[$4>>2]|0;
 $37 = ($36|0)!=(0|0);
 if ($37) {
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$pTemp>>2] = $38;
  $39 = HEAP32[$3>>2]|0;
  HEAP32[$4>>2] = $39;
  $40 = HEAP32[$pTemp>>2]|0;
  HEAP32[$3>>2] = $40;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 $43 = (_sqlite3NameFromToken($41,$42)|0);
 $44 = HEAP32[$pItem>>2]|0;
 $45 = ((($44)) + 8|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$4>>2]|0;
 $48 = (_sqlite3NameFromToken($46,$47)|0);
 $49 = HEAP32[$pItem>>2]|0;
 $50 = ((($49)) + 4|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3JoinType($pParse,$pA,$pB,$pC) {
 $pParse = $pParse|0;
 $pA = $pA|0;
 $pB = $pB|0;
 $pC = $pC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $apAll = 0;
 var $i = 0, $j = 0, $jointype = 0, $p = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zSp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $jointype = sp + 48|0;
 $apAll = sp + 36|0;
 $p = sp + 32|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $zSp = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $pC;
 HEAP32[$jointype>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$apAll>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($apAll)) + 4|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$3>>2]|0;
 $8 = ((($apAll)) + 8|0);
 HEAP32[$8>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(3);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = (($apAll) + ($11<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = (($apAll) + ($15<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$p>>2] = $17;
  HEAP32[$j>>2] = 0;
  while(1) {
   $18 = HEAP32[$j>>2]|0;
   $19 = ($18|0)<(7);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = (33275 + (($23*3)|0)|0);
   $25 = ((($24)) + 1|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = ($22|0)==($27|0);
   if ($28) {
    $29 = HEAP32[$p>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$j>>2]|0;
    $32 = (33275 + (($31*3)|0)|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = $33&255;
    $35 = (33241 + ($34)|0);
    $36 = HEAP32[$p>>2]|0;
    $37 = ((($36)) + 4|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (_sqlite3_strnicmp($30,$35,$38)|0);
    $40 = ($39|0)==(0);
    if ($40) {
     label = 8;
     break;
    }
   }
   $48 = HEAP32[$j>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$j>>2] = $49;
  }
  if ((label|0) == 8) {
   label = 0;
   $41 = HEAP32[$j>>2]|0;
   $42 = (33275 + (($41*3)|0)|0);
   $43 = ((($42)) + 2|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = HEAP32[$jointype>>2]|0;
   $47 = $46 | $45;
   HEAP32[$jointype>>2] = $47;
  }
  $50 = HEAP32[$j>>2]|0;
  $51 = ($50|0)>=(7);
  if ($51) {
   label = 11;
   break;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 11) {
  $52 = HEAP32[$jointype>>2]|0;
  $53 = $52 | 64;
  HEAP32[$jointype>>2] = $53;
 }
 $56 = HEAP32[$jointype>>2]|0;
 $57 = $56 & 33;
 $58 = ($57|0)==(33);
 if (!($58)) {
  $59 = HEAP32[$jointype>>2]|0;
  $60 = $59 & 64;
  $61 = ($60|0)!=(0);
  if (!($61)) {
   $71 = HEAP32[$jointype>>2]|0;
   $72 = $71 & 32;
   $73 = ($72|0)!=(0);
   if (!($73)) {
    $78 = HEAP32[$jointype>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
   $74 = HEAP32[$jointype>>2]|0;
   $75 = $74 & 24;
   $76 = ($75|0)!=(8);
   if (!($76)) {
    $78 = HEAP32[$jointype>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
   $77 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($77,33342,$vararg_buffer4);
   HEAP32[$jointype>>2] = 1;
   $78 = HEAP32[$jointype>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
 }
 HEAP32[$zSp>>2] = 33296;
 $62 = HEAP32[$3>>2]|0;
 $63 = ($62|0)==(0|0);
 if ($63) {
  $64 = HEAP32[$zSp>>2]|0;
  $65 = ((($64)) + 1|0);
  HEAP32[$zSp>>2] = $65;
 }
 $66 = HEAP32[$0>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = HEAP32[$2>>2]|0;
 $69 = HEAP32[$zSp>>2]|0;
 $70 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $67;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $68;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $69;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $70;
 _sqlite3ErrorMsg($66,33298,$vararg_buffer);
 HEAP32[$jointype>>2] = 1;
 $78 = HEAP32[$jointype>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3ExprListSetSortOrder($p,$iSortOrder) {
 $p = $p|0;
 $iSortOrder = $iSortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iSortOrder;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)==(0|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $or$cond = $3 | $5;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = $6&255;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*20)|0)|0);
 $15 = ((($14)) + 12|0);
 HEAP8[$15>>0] = $7;
 STACKTOP = sp;return;
}
function _sqlite3WithPush($pParse,$pWith,$bFree) {
 $pParse = $pParse|0;
 $pWith = $pWith|0;
 $bFree = $bFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pWith;
 HEAP8[$2>>0] = $bFree;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 536|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 536|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP8[$2>>0]|0;
 $14 = ($13<<24>>24)!=(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 540|0);
 HEAP32[$17>>2] = $15;
 STACKTOP = sp;return;
}
function _sqlite3DeleteFrom($pParse,$pTabList,$pWhere) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aToOpen = 0, $addrBypass = 0, $addrEphOpen = 0, $addrLoop = 0, $aiCurOnePass = 0, $bComplex = 0, $count = 0;
 var $db = 0, $eOnePass = 0, $i = 0, $iAddrOnce = 0, $iDataCur = 0, $iDb = 0, $iEphCur = 0, $iIdxCur = 0, $iIdxNoSeek = 0, $iKey = 0, $iPk = 0, $iRowSet = 0, $iTabCur = 0, $isView = 0, $memCnt = 0, $nIdx = 0, $nKey = 0, $nPk = 0, $or$cond = 0, $or$cond$not = 0;
 var $or$cond3 = 0, $pIdx = 0, $pPk = 0, $pTab = 0, $pTrigger = 0, $pVTab = 0, $pWInfo = 0, $rcauth = 0, $sContext = 0, $sNC = 0, $v = 0, $wcf = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 180|0;
 $1 = sp + 176|0;
 $2 = sp + 172|0;
 $v = sp + 168|0;
 $pTab = sp + 164|0;
 $zDb = sp + 160|0;
 $i = sp + 156|0;
 $pWInfo = sp + 152|0;
 $pIdx = sp + 148|0;
 $iTabCur = sp + 144|0;
 $iDataCur = sp + 140|0;
 $iIdxCur = sp + 136|0;
 $nIdx = sp + 132|0;
 $db = sp + 128|0;
 $sContext = sp + 120|0;
 $sNC = sp + 88|0;
 $iDb = sp + 84|0;
 $memCnt = sp + 80|0;
 $rcauth = sp + 76|0;
 $eOnePass = sp + 72|0;
 $aiCurOnePass = sp + 64|0;
 $aToOpen = sp + 60|0;
 $pPk = sp + 56|0;
 $iPk = sp + 52|0;
 $nPk = sp + 188|0;
 $iKey = sp + 48|0;
 $nKey = sp + 186|0;
 $iEphCur = sp + 44|0;
 $iRowSet = sp + 40|0;
 $addrBypass = sp + 36|0;
 $addrLoop = sp + 32|0;
 $addrEphOpen = sp + 28|0;
 $isView = sp + 24|0;
 $pTrigger = sp + 20|0;
 $bComplex = sp + 16|0;
 $wcf = sp + 184|0;
 $iAddrOnce = sp + 12|0;
 $pVTab = sp + 8|0;
 $count = sp + 4|0;
 $iIdxNoSeek = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pWhere;
 HEAP32[$iDataCur>>2] = 0;
 HEAP32[$iIdxCur>>2] = 0;
 HEAP32[$memCnt>>2] = -1;
 HEAP32[$aToOpen>>2] = 0;
 HEAP32[$iPk>>2] = 0;
 HEAP16[$nPk>>1] = 1;
 HEAP32[$iEphCur>>2] = 0;
 HEAP32[$iRowSet>>2] = 0;
 HEAP32[$addrBypass>>2] = 0;
 HEAP32[$addrLoop>>2] = 0;
 HEAP32[$addrEphOpen>>2] = 0;
 ;HEAP32[$sContext>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 68|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 L1: do {
  if (!($8)) {
   $9 = HEAP32[$db>>2]|0;
   $10 = ((($9)) + 65|0);
   $11 = HEAP8[$10>>0]|0;
   $12 = $11&255;
   $13 = ($12|0)!=(0);
   if (!($13)) {
    $14 = HEAP32[$0>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    $16 = (_sqlite3SrcListLookup($14,$15)|0);
    HEAP32[$pTab>>2] = $16;
    $17 = HEAP32[$pTab>>2]|0;
    $18 = ($17|0)==(0|0);
    if (!($18)) {
     $19 = HEAP32[$0>>2]|0;
     $20 = HEAP32[$pTab>>2]|0;
     $21 = (_sqlite3TriggersExist($19,$20,109,0,0)|0);
     HEAP32[$pTrigger>>2] = $21;
     $22 = HEAP32[$pTab>>2]|0;
     $23 = ((($22)) + 12|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = ($24|0)!=(0|0);
     $26 = $25&1;
     HEAP32[$isView>>2] = $26;
     $27 = HEAP32[$pTrigger>>2]|0;
     $28 = ($27|0)!=(0|0);
     if ($28) {
      $34 = 1;
     } else {
      $29 = HEAP32[$0>>2]|0;
      $30 = HEAP32[$pTab>>2]|0;
      $31 = (_sqlite3FkRequired($29,$30,0,0)|0);
      $32 = ($31|0)!=(0);
      $34 = $32;
     }
     $33 = $34&1;
     HEAP32[$bComplex>>2] = $33;
     $35 = HEAP32[$0>>2]|0;
     $36 = HEAP32[$pTab>>2]|0;
     $37 = (_sqlite3ViewGetColumnNames($35,$36)|0);
     $38 = ($37|0)!=(0);
     if (!($38)) {
      $39 = HEAP32[$0>>2]|0;
      $40 = HEAP32[$pTab>>2]|0;
      $41 = HEAP32[$pTrigger>>2]|0;
      $42 = ($41|0)!=(0|0);
      $43 = $42 ? 1 : 0;
      $44 = (_sqlite3IsReadOnly($39,$40,$43)|0);
      $45 = ($44|0)!=(0);
      if (!($45)) {
       $46 = HEAP32[$db>>2]|0;
       $47 = HEAP32[$pTab>>2]|0;
       $48 = ((($47)) + 64|0);
       $49 = HEAP32[$48>>2]|0;
       $50 = (_sqlite3SchemaToIndex($46,$49)|0);
       HEAP32[$iDb>>2] = $50;
       $51 = HEAP32[$iDb>>2]|0;
       $52 = HEAP32[$db>>2]|0;
       $53 = ((($52)) + 16|0);
       $54 = HEAP32[$53>>2]|0;
       $55 = (($54) + ($51<<4)|0);
       $56 = HEAP32[$55>>2]|0;
       HEAP32[$zDb>>2] = $56;
       $57 = HEAP32[$0>>2]|0;
       $58 = HEAP32[$pTab>>2]|0;
       $59 = HEAP32[$58>>2]|0;
       $60 = HEAP32[$zDb>>2]|0;
       $61 = (_sqlite3AuthCheck($57,9,$59,0,$60)|0);
       HEAP32[$rcauth>>2] = $61;
       $62 = HEAP32[$rcauth>>2]|0;
       $63 = ($62|0)==(1);
       if (!($63)) {
        $64 = HEAP32[$0>>2]|0;
        $65 = ((($64)) + 72|0);
        $66 = HEAP32[$65>>2]|0;
        $67 = (($66) + 1)|0;
        HEAP32[$65>>2] = $67;
        $68 = HEAP32[$1>>2]|0;
        $69 = ((($68)) + 8|0);
        $70 = ((($69)) + 44|0);
        HEAP32[$70>>2] = $66;
        HEAP32[$iTabCur>>2] = $66;
        HEAP32[$nIdx>>2] = 0;
        $71 = HEAP32[$pTab>>2]|0;
        $72 = ((($71)) + 8|0);
        $73 = HEAP32[$72>>2]|0;
        HEAP32[$pIdx>>2] = $73;
        while(1) {
         $74 = HEAP32[$pIdx>>2]|0;
         $75 = ($74|0)!=(0|0);
         if (!($75)) {
          break;
         }
         $76 = HEAP32[$0>>2]|0;
         $77 = ((($76)) + 72|0);
         $78 = HEAP32[$77>>2]|0;
         $79 = (($78) + 1)|0;
         HEAP32[$77>>2] = $79;
         $80 = HEAP32[$pIdx>>2]|0;
         $81 = ((($80)) + 20|0);
         $82 = HEAP32[$81>>2]|0;
         HEAP32[$pIdx>>2] = $82;
         $83 = HEAP32[$nIdx>>2]|0;
         $84 = (($83) + 1)|0;
         HEAP32[$nIdx>>2] = $84;
        }
        $85 = HEAP32[$isView>>2]|0;
        $86 = ($85|0)!=(0);
        if ($86) {
         $87 = HEAP32[$0>>2]|0;
         $88 = HEAP32[$pTab>>2]|0;
         $89 = HEAP32[$88>>2]|0;
         _sqlite3AuthContextPush($87,$sContext,$89);
        }
        $90 = HEAP32[$0>>2]|0;
        $91 = (_sqlite3GetVdbe($90)|0);
        HEAP32[$v>>2] = $91;
        $92 = HEAP32[$v>>2]|0;
        $93 = ($92|0)==(0|0);
        if (!($93)) {
         $94 = HEAP32[$0>>2]|0;
         $95 = ((($94)) + 18|0);
         $96 = HEAP8[$95>>0]|0;
         $97 = $96&255;
         $98 = ($97|0)==(0);
         if ($98) {
          $99 = HEAP32[$v>>2]|0;
          _sqlite3VdbeCountChanges($99);
         }
         $100 = HEAP32[$0>>2]|0;
         $101 = HEAP32[$iDb>>2]|0;
         _sqlite3BeginWriteOperation($100,1,$101);
         $102 = HEAP32[$isView>>2]|0;
         $103 = ($102|0)!=(0);
         if ($103) {
          $104 = HEAP32[$0>>2]|0;
          $105 = HEAP32[$pTab>>2]|0;
          $106 = HEAP32[$2>>2]|0;
          $107 = HEAP32[$iTabCur>>2]|0;
          _sqlite3MaterializeView($104,$105,$106,$107);
          $108 = HEAP32[$iTabCur>>2]|0;
          HEAP32[$iIdxCur>>2] = $108;
          HEAP32[$iDataCur>>2] = $108;
         }
         ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
         $109 = HEAP32[$0>>2]|0;
         HEAP32[$sNC>>2] = $109;
         $110 = HEAP32[$1>>2]|0;
         $111 = ((($sNC)) + 4|0);
         HEAP32[$111>>2] = $110;
         $112 = HEAP32[$2>>2]|0;
         $113 = (_sqlite3ResolveExprNames($sNC,$112)|0);
         $114 = ($113|0)!=(0);
         if (!($114)) {
          $115 = HEAP32[$db>>2]|0;
          $116 = ((($115)) + 24|0);
          $117 = HEAP32[$116>>2]|0;
          $118 = $117 & 128;
          $119 = ($118|0)!=(0);
          if ($119) {
           $120 = HEAP32[$0>>2]|0;
           $121 = ((($120)) + 76|0);
           $122 = HEAP32[$121>>2]|0;
           $123 = (($122) + 1)|0;
           HEAP32[$121>>2] = $123;
           HEAP32[$memCnt>>2] = $123;
           $124 = HEAP32[$v>>2]|0;
           $125 = HEAP32[$memCnt>>2]|0;
           (_sqlite3VdbeAddOp2($124,22,0,$125)|0);
          }
          $126 = HEAP32[$rcauth>>2]|0;
          $127 = ($126|0)==(0);
          $128 = HEAP32[$2>>2]|0;
          $129 = ($128|0)==(0|0);
          $or$cond = $127 & $129;
          $or$cond$not = $or$cond ^ 1;
          $130 = HEAP32[$bComplex>>2]|0;
          $131 = ($130|0)!=(0);
          $or$cond3 = $or$cond$not | $131;
          L29: do {
           if ($or$cond3) {
            label = 29;
           } else {
            $132 = HEAP32[$pTab>>2]|0;
            $133 = ((($132)) + 42|0);
            $134 = HEAP8[$133>>0]|0;
            $135 = $134&255;
            $136 = $135 & 16;
            $137 = ($136|0)!=(0);
            if ($137) {
             label = 29;
            } else {
             $138 = HEAP32[$0>>2]|0;
             $139 = HEAP32[$iDb>>2]|0;
             $140 = HEAP32[$pTab>>2]|0;
             $141 = ((($140)) + 28|0);
             $142 = HEAP32[$141>>2]|0;
             $143 = HEAP32[$pTab>>2]|0;
             $144 = HEAP32[$143>>2]|0;
             _sqlite3TableLock($138,$139,$142,1,$144);
             $145 = HEAP32[$pTab>>2]|0;
             $146 = ((($145)) + 42|0);
             $147 = HEAP8[$146>>0]|0;
             $148 = $147&255;
             $149 = $148 & 32;
             $150 = ($149|0)==(0);
             if ($150) {
              $151 = HEAP32[$v>>2]|0;
              $152 = HEAP32[$pTab>>2]|0;
              $153 = ((($152)) + 28|0);
              $154 = HEAP32[$153>>2]|0;
              $155 = HEAP32[$iDb>>2]|0;
              $156 = HEAP32[$memCnt>>2]|0;
              $157 = HEAP32[$pTab>>2]|0;
              $158 = HEAP32[$157>>2]|0;
              (_sqlite3VdbeAddOp4($151,119,$154,$155,$156,$158,-2)|0);
             }
             $159 = HEAP32[$pTab>>2]|0;
             $160 = ((($159)) + 8|0);
             $161 = HEAP32[$160>>2]|0;
             HEAP32[$pIdx>>2] = $161;
             while(1) {
              $162 = HEAP32[$pIdx>>2]|0;
              $163 = ($162|0)!=(0|0);
              if (!($163)) {
               break L29;
              }
              $164 = HEAP32[$v>>2]|0;
              $165 = HEAP32[$pIdx>>2]|0;
              $166 = ((($165)) + 44|0);
              $167 = HEAP32[$166>>2]|0;
              $168 = HEAP32[$iDb>>2]|0;
              (_sqlite3VdbeAddOp2($164,119,$167,$168)|0);
              $169 = HEAP32[$pIdx>>2]|0;
              $170 = ((($169)) + 20|0);
              $171 = HEAP32[$170>>2]|0;
              HEAP32[$pIdx>>2] = $171;
             }
            }
           }
          } while(0);
          L38: do {
           if ((label|0) == 29) {
            HEAP16[$wcf>>1] = 12;
            $172 = HEAP32[$bComplex>>2]|0;
            $173 = ($172|0)!=(0);
            $174 = $173 ? 0 : 8192;
            $175 = HEAP16[$wcf>>1]|0;
            $176 = $175&65535;
            $177 = $176 | $174;
            $178 = $177&65535;
            HEAP16[$wcf>>1] = $178;
            $179 = HEAP32[$pTab>>2]|0;
            $180 = ((($179)) + 42|0);
            $181 = HEAP8[$180>>0]|0;
            $182 = $181&255;
            $183 = $182 & 32;
            $184 = ($183|0)==(0);
            if ($184) {
             HEAP32[$pPk>>2] = 0;
             HEAP16[$nPk>>1] = 1;
             $185 = HEAP32[$0>>2]|0;
             $186 = ((($185)) + 76|0);
             $187 = HEAP32[$186>>2]|0;
             $188 = (($187) + 1)|0;
             HEAP32[$186>>2] = $188;
             HEAP32[$iRowSet>>2] = $188;
             $189 = HEAP32[$v>>2]|0;
             $190 = HEAP32[$iRowSet>>2]|0;
             (_sqlite3VdbeAddOp2($189,25,0,$190)|0);
            } else {
             $191 = HEAP32[$pTab>>2]|0;
             $192 = (_sqlite3PrimaryKeyIndex($191)|0);
             HEAP32[$pPk>>2] = $192;
             $193 = HEAP32[$pPk>>2]|0;
             $194 = ((($193)) + 50|0);
             $195 = HEAP16[$194>>1]|0;
             HEAP16[$nPk>>1] = $195;
             $196 = HEAP32[$0>>2]|0;
             $197 = ((($196)) + 76|0);
             $198 = HEAP32[$197>>2]|0;
             $199 = (($198) + 1)|0;
             HEAP32[$iPk>>2] = $199;
             $200 = HEAP16[$nPk>>1]|0;
             $201 = $200 << 16 >> 16;
             $202 = HEAP32[$0>>2]|0;
             $203 = ((($202)) + 76|0);
             $204 = HEAP32[$203>>2]|0;
             $205 = (($204) + ($201))|0;
             HEAP32[$203>>2] = $205;
             $206 = HEAP32[$0>>2]|0;
             $207 = ((($206)) + 72|0);
             $208 = HEAP32[$207>>2]|0;
             $209 = (($208) + 1)|0;
             HEAP32[$207>>2] = $209;
             HEAP32[$iEphCur>>2] = $208;
             $210 = HEAP32[$v>>2]|0;
             $211 = HEAP32[$iEphCur>>2]|0;
             $212 = HEAP16[$nPk>>1]|0;
             $213 = $212 << 16 >> 16;
             $214 = (_sqlite3VdbeAddOp2($210,57,$211,$213)|0);
             HEAP32[$addrEphOpen>>2] = $214;
             $215 = HEAP32[$0>>2]|0;
             $216 = HEAP32[$pPk>>2]|0;
             _sqlite3VdbeSetP4KeyInfo($215,$216);
            }
            $217 = HEAP32[$0>>2]|0;
            $218 = HEAP32[$1>>2]|0;
            $219 = HEAP32[$2>>2]|0;
            $220 = HEAP16[$wcf>>1]|0;
            $221 = HEAP32[$iTabCur>>2]|0;
            $222 = (($221) + 1)|0;
            $223 = (_sqlite3WhereBegin($217,$218,$219,0,0,$220,$222)|0);
            HEAP32[$pWInfo>>2] = $223;
            $224 = HEAP32[$pWInfo>>2]|0;
            $225 = ($224|0)==(0|0);
            if ($225) {
             break L1;
            }
            $226 = HEAP32[$pWInfo>>2]|0;
            $227 = (_sqlite3WhereOkOnePass($226,$aiCurOnePass)|0);
            HEAP32[$eOnePass>>2] = $227;
            $228 = HEAP32[$db>>2]|0;
            $229 = ((($228)) + 24|0);
            $230 = HEAP32[$229>>2]|0;
            $231 = $230 & 128;
            $232 = ($231|0)!=(0);
            if ($232) {
             $233 = HEAP32[$v>>2]|0;
             $234 = HEAP32[$memCnt>>2]|0;
             (_sqlite3VdbeAddOp2($233,37,$234,1)|0);
            }
            $235 = HEAP32[$pPk>>2]|0;
            $236 = ($235|0)!=(0|0);
            do {
             if ($236) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $237 = HEAP32[$i>>2]|0;
               $238 = HEAP16[$nPk>>1]|0;
               $239 = $238 << 16 >> 16;
               $240 = ($237|0)<($239|0);
               if (!($240)) {
                break;
               }
               $241 = HEAP32[$v>>2]|0;
               $242 = HEAP32[$pTab>>2]|0;
               $243 = HEAP32[$iTabCur>>2]|0;
               $244 = HEAP32[$i>>2]|0;
               $245 = HEAP32[$pPk>>2]|0;
               $246 = ((($245)) + 4|0);
               $247 = HEAP32[$246>>2]|0;
               $248 = (($247) + ($244<<1)|0);
               $249 = HEAP16[$248>>1]|0;
               $250 = $249 << 16 >> 16;
               $251 = HEAP32[$iPk>>2]|0;
               $252 = HEAP32[$i>>2]|0;
               $253 = (($251) + ($252))|0;
               _sqlite3ExprCodeGetColumnOfTable($241,$242,$243,$250,$253);
               $254 = HEAP32[$i>>2]|0;
               $255 = (($254) + 1)|0;
               HEAP32[$i>>2] = $255;
              }
              $256 = HEAP32[$iPk>>2]|0;
              HEAP32[$iKey>>2] = $256;
             } else {
              $257 = HEAP32[$0>>2]|0;
              $258 = ((($257)) + 76|0);
              $259 = HEAP32[$258>>2]|0;
              $260 = (($259) + 1)|0;
              HEAP32[$iKey>>2] = $260;
              $261 = HEAP32[$0>>2]|0;
              $262 = HEAP32[$pTab>>2]|0;
              $263 = HEAP32[$iTabCur>>2]|0;
              $264 = HEAP32[$iKey>>2]|0;
              $265 = (_sqlite3ExprCodeGetColumn($261,$262,-1,$263,$264,0)|0);
              HEAP32[$iKey>>2] = $265;
              $266 = HEAP32[$iKey>>2]|0;
              $267 = HEAP32[$0>>2]|0;
              $268 = ((($267)) + 76|0);
              $269 = HEAP32[$268>>2]|0;
              $270 = ($266|0)>($269|0);
              if (!($270)) {
               break;
              }
              $271 = HEAP32[$iKey>>2]|0;
              $272 = HEAP32[$0>>2]|0;
              $273 = ((($272)) + 76|0);
              HEAP32[$273>>2] = $271;
             }
            } while(0);
            $274 = HEAP32[$eOnePass>>2]|0;
            $275 = ($274|0)!=(0);
            do {
             if ($275) {
              $276 = HEAP16[$nPk>>1]|0;
              HEAP16[$nKey>>1] = $276;
              $277 = HEAP32[$db>>2]|0;
              $278 = HEAP32[$nIdx>>2]|0;
              $279 = (($278) + 2)|0;
              $280 = ($279|0)<(0);
              $281 = $280 << 31 >> 31;
              $282 = (_sqlite3DbMallocRawNN($277,$279,$281)|0);
              HEAP32[$aToOpen>>2] = $282;
              $283 = HEAP32[$aToOpen>>2]|0;
              $284 = ($283|0)==(0|0);
              if ($284) {
               $285 = HEAP32[$pWInfo>>2]|0;
               _sqlite3WhereEnd($285);
               break L1;
              }
              $286 = HEAP32[$aToOpen>>2]|0;
              $287 = HEAP32[$nIdx>>2]|0;
              $288 = (($287) + 1)|0;
              _memset(($286|0),1,($288|0))|0;
              $289 = HEAP32[$nIdx>>2]|0;
              $290 = (($289) + 1)|0;
              $291 = HEAP32[$aToOpen>>2]|0;
              $292 = (($291) + ($290)|0);
              HEAP8[$292>>0] = 0;
              $293 = HEAP32[$aiCurOnePass>>2]|0;
              $294 = ($293|0)>=(0);
              if ($294) {
               $295 = HEAP32[$aiCurOnePass>>2]|0;
               $296 = HEAP32[$iTabCur>>2]|0;
               $297 = (($295) - ($296))|0;
               $298 = HEAP32[$aToOpen>>2]|0;
               $299 = (($298) + ($297)|0);
               HEAP8[$299>>0] = 0;
              }
              $300 = ((($aiCurOnePass)) + 4|0);
              $301 = HEAP32[$300>>2]|0;
              $302 = ($301|0)>=(0);
              if ($302) {
               $303 = ((($aiCurOnePass)) + 4|0);
               $304 = HEAP32[$303>>2]|0;
               $305 = HEAP32[$iTabCur>>2]|0;
               $306 = (($304) - ($305))|0;
               $307 = HEAP32[$aToOpen>>2]|0;
               $308 = (($307) + ($306)|0);
               HEAP8[$308>>0] = 0;
              }
              $309 = HEAP32[$addrEphOpen>>2]|0;
              $310 = ($309|0)!=(0);
              if (!($310)) {
               break;
              }
              $311 = HEAP32[$v>>2]|0;
              $312 = HEAP32[$addrEphOpen>>2]|0;
              (_sqlite3VdbeChangeToNoop($311,$312)|0);
             } else {
              $313 = HEAP32[$pPk>>2]|0;
              $314 = ($313|0)!=(0|0);
              if ($314) {
               $315 = HEAP32[$0>>2]|0;
               $316 = ((($315)) + 76|0);
               $317 = HEAP32[$316>>2]|0;
               $318 = (($317) + 1)|0;
               HEAP32[$316>>2] = $318;
               HEAP32[$iKey>>2] = $318;
               HEAP16[$nKey>>1] = 0;
               $319 = HEAP32[$v>>2]|0;
               $320 = HEAP32[$iPk>>2]|0;
               $321 = HEAP16[$nPk>>1]|0;
               $322 = $321 << 16 >> 16;
               $323 = HEAP32[$iKey>>2]|0;
               $324 = HEAP32[$0>>2]|0;
               $325 = HEAP32[$324>>2]|0;
               $326 = HEAP32[$pPk>>2]|0;
               $327 = (_sqlite3IndexAffinityStr($325,$326)|0);
               $328 = HEAP16[$nPk>>1]|0;
               $329 = $328 << 16 >> 16;
               (_sqlite3VdbeAddOp4($319,49,$320,$322,$323,$327,$329)|0);
               $330 = HEAP32[$v>>2]|0;
               $331 = HEAP32[$iEphCur>>2]|0;
               $332 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($330,110,$331,$332)|0);
               break;
              } else {
               HEAP16[$nKey>>1] = 1;
               $333 = HEAP32[$v>>2]|0;
               $334 = HEAP32[$iRowSet>>2]|0;
               $335 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($333,129,$334,$335)|0);
               break;
              }
             }
            } while(0);
            $336 = HEAP32[$eOnePass>>2]|0;
            $337 = ($336|0)!=(0);
            if ($337) {
             $338 = HEAP32[$v>>2]|0;
             $339 = (_sqlite3VdbeMakeLabel($338)|0);
             HEAP32[$addrBypass>>2] = $339;
            } else {
             $340 = HEAP32[$pWInfo>>2]|0;
             _sqlite3WhereEnd($340);
            }
            $341 = HEAP32[$isView>>2]|0;
            $342 = ($341|0)!=(0);
            do {
             if (!($342)) {
              HEAP32[$iAddrOnce>>2] = 0;
              $343 = HEAP32[$eOnePass>>2]|0;
              $344 = ($343|0)==(2);
              if ($344) {
               $345 = HEAP32[$0>>2]|0;
               $346 = (_sqlite3CodeOnce($345)|0);
               HEAP32[$iAddrOnce>>2] = $346;
              }
              $347 = HEAP32[$0>>2]|0;
              $348 = HEAP32[$pTab>>2]|0;
              $349 = HEAP32[$iTabCur>>2]|0;
              $350 = HEAP32[$aToOpen>>2]|0;
              (_sqlite3OpenTableAndIndices($347,$348,55,8,$349,$350,$iDataCur,$iIdxCur)|0);
              $351 = HEAP32[$eOnePass>>2]|0;
              $352 = ($351|0)==(2);
              if (!($352)) {
               break;
              }
              $353 = HEAP32[$v>>2]|0;
              $354 = HEAP32[$iAddrOnce>>2]|0;
              _sqlite3VdbeJumpHere($353,$354);
             }
            } while(0);
            $355 = HEAP32[$eOnePass>>2]|0;
            $356 = ($355|0)!=(0);
            do {
             if ($356) {
              $357 = HEAP32[$pTab>>2]|0;
              $358 = ((($357)) + 42|0);
              $359 = HEAP8[$358>>0]|0;
              $360 = $359&255;
              $361 = $360 & 16;
              $362 = ($361|0)!=(0);
              if ($362) {
               break;
              }
              $363 = HEAP32[$iDataCur>>2]|0;
              $364 = HEAP32[$iTabCur>>2]|0;
              $365 = (($363) - ($364))|0;
              $366 = HEAP32[$aToOpen>>2]|0;
              $367 = (($366) + ($365)|0);
              $368 = HEAP8[$367>>0]|0;
              $369 = $368&255;
              $370 = ($369|0)!=(0);
              if (!($370)) {
               break;
              }
              $371 = HEAP32[$v>>2]|0;
              $372 = HEAP32[$iDataCur>>2]|0;
              $373 = HEAP32[$addrBypass>>2]|0;
              $374 = HEAP32[$iKey>>2]|0;
              $375 = HEAP16[$nKey>>1]|0;
              $376 = $375 << 16 >> 16;
              (_sqlite3VdbeAddOp4Int($371,68,$372,$373,$374,$376)|0);
             } else {
              $377 = HEAP32[$pPk>>2]|0;
              $378 = ($377|0)!=(0|0);
              $379 = HEAP32[$v>>2]|0;
              if ($378) {
               $380 = HEAP32[$iEphCur>>2]|0;
               $381 = (_sqlite3VdbeAddOp1($379,108,$380)|0);
               HEAP32[$addrLoop>>2] = $381;
               $382 = HEAP32[$v>>2]|0;
               $383 = HEAP32[$iEphCur>>2]|0;
               $384 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($382,101,$383,$384)|0);
               break;
              } else {
               $385 = HEAP32[$iRowSet>>2]|0;
               $386 = HEAP32[$iKey>>2]|0;
               $387 = (_sqlite3VdbeAddOp3($379,130,$385,0,$386)|0);
               HEAP32[$addrLoop>>2] = $387;
               break;
              }
             }
            } while(0);
            $388 = HEAP32[$pTab>>2]|0;
            $389 = ((($388)) + 42|0);
            $390 = HEAP8[$389>>0]|0;
            $391 = $390&255;
            $392 = $391 & 16;
            $393 = ($392|0)!=(0);
            do {
             if ($393) {
              $394 = HEAP32[$db>>2]|0;
              $395 = HEAP32[$pTab>>2]|0;
              $396 = (_sqlite3GetVTable($394,$395)|0);
              HEAP32[$pVTab>>2] = $396;
              $397 = HEAP32[$0>>2]|0;
              $398 = HEAP32[$pTab>>2]|0;
              _sqlite3VtabMakeWritable($397,$398);
              $399 = HEAP32[$v>>2]|0;
              $400 = HEAP32[$iKey>>2]|0;
              $401 = HEAP32[$pVTab>>2]|0;
              (_sqlite3VdbeAddOp4($399,12,0,1,$400,$401,-10)|0);
              $402 = HEAP32[$v>>2]|0;
              _sqlite3VdbeChangeP5($402,2);
              $403 = HEAP32[$0>>2]|0;
              _sqlite3MayAbort($403);
              $404 = HEAP32[$eOnePass>>2]|0;
              $405 = ($404|0)==(1);
              if (!($405)) {
               break;
              }
              $406 = HEAP32[$0>>2]|0;
              $407 = ((($406)) + 416|0);
              $408 = HEAP32[$407>>2]|0;
              $409 = ($408|0)==(0|0);
              if (!($409)) {
               break;
              }
              $410 = HEAP32[$0>>2]|0;
              $411 = ((($410)) + 20|0);
              HEAP8[$411>>0] = 0;
             } else {
              $412 = HEAP32[$0>>2]|0;
              $413 = ((($412)) + 18|0);
              $414 = HEAP8[$413>>0]|0;
              $415 = $414&255;
              $416 = ($415|0)==(0);
              $417 = $416&1;
              HEAP32[$count>>2] = $417;
              HEAP32[$iIdxNoSeek>>2] = -1;
              $418 = HEAP32[$bComplex>>2]|0;
              $419 = ($418|0)==(0);
              do {
               if ($419) {
                $420 = ((($aiCurOnePass)) + 4|0);
                $421 = HEAP32[$420>>2]|0;
                $422 = HEAP32[$iDataCur>>2]|0;
                $423 = ($421|0)!=($422|0);
                if (!($423)) {
                 break;
                }
                $424 = ((($aiCurOnePass)) + 4|0);
                $425 = HEAP32[$424>>2]|0;
                HEAP32[$iIdxNoSeek>>2] = $425;
               }
              } while(0);
              $426 = HEAP32[$0>>2]|0;
              $427 = HEAP32[$pTab>>2]|0;
              $428 = HEAP32[$pTrigger>>2]|0;
              $429 = HEAP32[$iDataCur>>2]|0;
              $430 = HEAP32[$iIdxCur>>2]|0;
              $431 = HEAP32[$iKey>>2]|0;
              $432 = HEAP16[$nKey>>1]|0;
              $433 = HEAP32[$count>>2]|0;
              $434 = $433&255;
              $435 = HEAP32[$eOnePass>>2]|0;
              $436 = $435&255;
              $437 = HEAP32[$iIdxNoSeek>>2]|0;
              _sqlite3GenerateRowDelete($426,$427,$428,$429,$430,$431,$432,$434,10,$436,$437);
             }
            } while(0);
            $438 = HEAP32[$eOnePass>>2]|0;
            $439 = ($438|0)!=(0);
            do {
             if ($439) {
              $440 = HEAP32[$v>>2]|0;
              $441 = HEAP32[$addrBypass>>2]|0;
              _sqlite3VdbeResolveLabel($440,$441);
              $442 = HEAP32[$pWInfo>>2]|0;
              _sqlite3WhereEnd($442);
             } else {
              $443 = HEAP32[$pPk>>2]|0;
              $444 = ($443|0)!=(0|0);
              $445 = HEAP32[$v>>2]|0;
              if ($444) {
               $446 = HEAP32[$iEphCur>>2]|0;
               $447 = HEAP32[$addrLoop>>2]|0;
               $448 = (($447) + 1)|0;
               (_sqlite3VdbeAddOp2($445,7,$446,$448)|0);
               $449 = HEAP32[$v>>2]|0;
               $450 = HEAP32[$addrLoop>>2]|0;
               _sqlite3VdbeJumpHere($449,$450);
               break;
              } else {
               $451 = HEAP32[$addrLoop>>2]|0;
               (_sqlite3VdbeGoto($445,$451)|0);
               $452 = HEAP32[$v>>2]|0;
               $453 = HEAP32[$addrLoop>>2]|0;
               _sqlite3VdbeJumpHere($452,$453);
               break;
              }
             }
            } while(0);
            $454 = HEAP32[$isView>>2]|0;
            $455 = ($454|0)!=(0);
            if ($455) {
             break;
            }
            $456 = HEAP32[$pTab>>2]|0;
            $457 = ((($456)) + 42|0);
            $458 = HEAP8[$457>>0]|0;
            $459 = $458&255;
            $460 = $459 & 16;
            $461 = ($460|0)!=(0);
            if ($461) {
             break;
            }
            $462 = HEAP32[$pPk>>2]|0;
            $463 = ($462|0)!=(0|0);
            if (!($463)) {
             $464 = HEAP32[$v>>2]|0;
             $465 = HEAP32[$iDataCur>>2]|0;
             (_sqlite3VdbeAddOp1($464,61,$465)|0);
            }
            HEAP32[$i>>2] = 0;
            $466 = HEAP32[$pTab>>2]|0;
            $467 = ((($466)) + 8|0);
            $468 = HEAP32[$467>>2]|0;
            HEAP32[$pIdx>>2] = $468;
            while(1) {
             $469 = HEAP32[$pIdx>>2]|0;
             $470 = ($469|0)!=(0|0);
             if (!($470)) {
              break L38;
             }
             $471 = HEAP32[$v>>2]|0;
             $472 = HEAP32[$iIdxCur>>2]|0;
             $473 = HEAP32[$i>>2]|0;
             $474 = (($472) + ($473))|0;
             (_sqlite3VdbeAddOp1($471,61,$474)|0);
             $475 = HEAP32[$i>>2]|0;
             $476 = (($475) + 1)|0;
             HEAP32[$i>>2] = $476;
             $477 = HEAP32[$pIdx>>2]|0;
             $478 = ((($477)) + 20|0);
             $479 = HEAP32[$478>>2]|0;
             HEAP32[$pIdx>>2] = $479;
            }
           }
          } while(0);
          $480 = HEAP32[$0>>2]|0;
          $481 = ((($480)) + 18|0);
          $482 = HEAP8[$481>>0]|0;
          $483 = $482&255;
          $484 = ($483|0)==(0);
          if ($484) {
           $485 = HEAP32[$0>>2]|0;
           $486 = ((($485)) + 420|0);
           $487 = HEAP32[$486>>2]|0;
           $488 = ($487|0)==(0|0);
           if ($488) {
            $489 = HEAP32[$0>>2]|0;
            _sqlite3AutoincrementEnd($489);
           }
          }
          $490 = HEAP32[$db>>2]|0;
          $491 = ((($490)) + 24|0);
          $492 = HEAP32[$491>>2]|0;
          $493 = $492 & 128;
          $494 = ($493|0)!=(0);
          if ($494) {
           $495 = HEAP32[$0>>2]|0;
           $496 = ((($495)) + 18|0);
           $497 = HEAP8[$496>>0]|0;
           $498 = ($497<<24>>24)!=(0);
           if (!($498)) {
            $499 = HEAP32[$0>>2]|0;
            $500 = ((($499)) + 420|0);
            $501 = HEAP32[$500>>2]|0;
            $502 = ($501|0)!=(0|0);
            if (!($502)) {
             $503 = HEAP32[$v>>2]|0;
             $504 = HEAP32[$memCnt>>2]|0;
             (_sqlite3VdbeAddOp2($503,33,$504,1)|0);
             $505 = HEAP32[$v>>2]|0;
             _sqlite3VdbeSetNumCols($505,1);
             $506 = HEAP32[$v>>2]|0;
             (_sqlite3VdbeSetColName($506,0,0,33397,0)|0);
            }
           }
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 _sqlite3AuthContextPop($sContext);
 $507 = HEAP32[$db>>2]|0;
 $508 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($507,$508);
 $509 = HEAP32[$db>>2]|0;
 $510 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($509,$510);
 $511 = HEAP32[$db>>2]|0;
 $512 = HEAP32[$aToOpen>>2]|0;
 _sqlite3DbFree($511,$512);
 STACKTOP = sp;return;
}
function _sqlite3ExprListCheckLength($pParse,$pEList,$zObject) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $zObject = $zObject|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mx = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $mx = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $zObject;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 88|0);
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$mx>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$mx>>2]|0;
 $13 = ($11|0)>($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $15;
 _sqlite3ErrorMsg($14,33556,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3Update($pParse,$pTabList,$pChanges,$pWhere,$onError) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pChanges = $pChanges|0;
 $pWhere = $pWhere|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0;
 var $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0;
 var $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0;
 var $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0;
 var $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0;
 var $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0;
 var $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0;
 var $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0;
 var $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0;
 var $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0;
 var $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0;
 var $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0;
 var $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0;
 var $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0;
 var $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0;
 var $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0;
 var $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0;
 var $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0;
 var $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0;
 var $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0;
 var $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0;
 var $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0;
 var $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0;
 var $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0;
 var $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0;
 var $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0;
 var $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0;
 var $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0;
 var $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0;
 var $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0;
 var $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0;
 var $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0;
 var $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0;
 var $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0;
 var $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0;
 var $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0;
 var $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0;
 var $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0;
 var $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aRegIdx = 0, $aToOpen = 0, $aXRef = 0, $addr1 = 0, $addrOpen = 0, $addrTop = 0, $aiCurOnePass = 0, $bReplace = 0, $chngKey = 0;
 var $chngPk = 0, $chngRowid = 0, $db = 0, $hasFK = 0, $i = 0, $iBaseCur = 0, $iDataCur = 0, $iDb = 0, $iEph = 0, $iIdxCol = 0, $iIdxCur = 0, $iPk = 0, $isView = 0, $j = 0, $labelBreak = 0, $labelContinue = 0, $nIdx = 0, $nKey = 0, $nPk = 0, $newmask = 0;
 var $okOnePass = 0, $oldmask = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond21 = 0, $or$cond23 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pIdx = 0, $pPk = 0, $pRowidExpr = 0, $pTab = 0, $pTrigger = 0, $pWInfo = 0;
 var $rc = 0, $reg = 0, $regKey = 0, $regNew = 0, $regNewRowid = 0, $regOld = 0, $regOldRowid = 0, $regRowCount = 0, $regRowSet = 0, $sContext = 0, $sNC = 0, $tmask = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer = sp;
 $0 = sp + 244|0;
 $1 = sp + 240|0;
 $2 = sp + 236|0;
 $3 = sp + 232|0;
 $4 = sp + 228|0;
 $i = sp + 224|0;
 $j = sp + 220|0;
 $pTab = sp + 216|0;
 $addrTop = sp + 212|0;
 $pWInfo = sp + 208|0;
 $v = sp + 204|0;
 $pIdx = sp + 200|0;
 $pPk = sp + 196|0;
 $nIdx = sp + 192|0;
 $iBaseCur = sp + 188|0;
 $iDataCur = sp + 184|0;
 $iIdxCur = sp + 180|0;
 $db = sp + 176|0;
 $aRegIdx = sp + 172|0;
 $aXRef = sp + 168|0;
 $aToOpen = sp + 164|0;
 $chngPk = sp + 254|0;
 $chngRowid = sp + 253|0;
 $chngKey = sp + 252|0;
 $pRowidExpr = sp + 160|0;
 $sContext = sp + 152|0;
 $sNC = sp + 120|0;
 $iDb = sp + 112|0;
 $okOnePass = sp + 108|0;
 $hasFK = sp + 104|0;
 $labelBreak = sp + 100|0;
 $labelContinue = sp + 96|0;
 $isView = sp + 92|0;
 $pTrigger = sp + 88|0;
 $tmask = sp + 84|0;
 $newmask = sp + 80|0;
 $iEph = sp + 76|0;
 $nKey = sp + 72|0;
 $aiCurOnePass = sp + 64|0;
 $regRowCount = sp + 56|0;
 $regOldRowid = sp + 52|0;
 $regNewRowid = sp + 48|0;
 $regNew = sp + 44|0;
 $regOld = sp + 40|0;
 $regRowSet = sp + 36|0;
 $regKey = sp + 32|0;
 $rc = sp + 28|0;
 $reg = sp + 24|0;
 $iIdxCol = sp + 250|0;
 $iPk = sp + 20|0;
 $nPk = sp + 248|0;
 $addrOpen = sp + 16|0;
 $oldmask = sp + 12|0;
 $addr1 = sp + 8|0;
 $bReplace = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $onError;
 HEAP32[$addrTop>>2] = 0;
 HEAP32[$aRegIdx>>2] = 0;
 HEAP32[$aXRef>>2] = 0;
 HEAP32[$pRowidExpr>>2] = 0;
 HEAP32[$iEph>>2] = 0;
 HEAP32[$nKey>>2] = 0;
 HEAP32[$regRowCount>>2] = 0;
 HEAP32[$regOldRowid>>2] = 0;
 HEAP32[$regNewRowid>>2] = 0;
 HEAP32[$regNew>>2] = 0;
 HEAP32[$regOld>>2] = 0;
 HEAP32[$regRowSet>>2] = 0;
 HEAP32[$regKey>>2] = 0;
 ;HEAP32[$sContext>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 65|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 if ($15) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (_sqlite3SrcListLookup($16,$17)|0);
 HEAP32[$pTab>>2] = $18;
 $19 = HEAP32[$pTab>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pTab>>2]|0;
 $24 = ((($23)) + 64|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3SchemaToIndex($22,$25)|0);
 HEAP32[$iDb>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$pTab>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (_sqlite3TriggersExist($27,$28,110,$29,$tmask)|0);
 HEAP32[$pTrigger>>2] = $30;
 $31 = HEAP32[$pTab>>2]|0;
 $32 = ((($31)) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 $35 = $34&1;
 HEAP32[$isView>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pTab>>2]|0;
 $38 = (_sqlite3ViewGetColumnNames($36,$37)|0);
 $39 = ($38|0)!=(0);
 if ($39) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$pTab>>2]|0;
 $42 = HEAP32[$tmask>>2]|0;
 $43 = (_sqlite3IsReadOnly($40,$41,$42)|0);
 $44 = ($43|0)!=(0);
 if ($44) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 72|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 1)|0;
 HEAP32[$46>>2] = $48;
 HEAP32[$iDataCur>>2] = $47;
 HEAP32[$iBaseCur>>2] = $47;
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 8|0);
 $51 = ((($50)) + 44|0);
 HEAP32[$51>>2] = $47;
 $52 = HEAP32[$iDataCur>>2]|0;
 $53 = (($52) + 1)|0;
 HEAP32[$iIdxCur>>2] = $53;
 $54 = HEAP32[$pTab>>2]|0;
 $55 = ((($54)) + 42|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = $56&255;
 $58 = $57 & 32;
 $59 = ($58|0)==(0);
 if ($59) {
  $62 = 0;
 } else {
  $60 = HEAP32[$pTab>>2]|0;
  $61 = (_sqlite3PrimaryKeyIndex($60)|0);
  $62 = $61;
 }
 HEAP32[$pPk>>2] = $62;
 HEAP32[$nIdx>>2] = 0;
 $63 = HEAP32[$pTab>>2]|0;
 $64 = ((($63)) + 8|0);
 $65 = HEAP32[$64>>2]|0;
 HEAP32[$pIdx>>2] = $65;
 while(1) {
  $66 = HEAP32[$pIdx>>2]|0;
  $67 = ($66|0)!=(0|0);
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$pIdx>>2]|0;
  $69 = ((($68)) + 55|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70 & 3;
  $72 = $71&255;
  $73 = ($72|0)==(2);
  $74 = HEAP32[$pPk>>2]|0;
  $75 = ($74|0)!=(0|0);
  $or$cond = $73 & $75;
  if ($or$cond) {
   $76 = HEAP32[$0>>2]|0;
   $77 = ((($76)) + 72|0);
   $78 = HEAP32[$77>>2]|0;
   HEAP32[$iDataCur>>2] = $78;
   $79 = HEAP32[$iDataCur>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 8|0);
   $82 = ((($81)) + 44|0);
   HEAP32[$82>>2] = $79;
  }
  $83 = HEAP32[$0>>2]|0;
  $84 = ((($83)) + 72|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (($85) + 1)|0;
  HEAP32[$84>>2] = $86;
  $87 = HEAP32[$pIdx>>2]|0;
  $88 = ((($87)) + 20|0);
  $89 = HEAP32[$88>>2]|0;
  HEAP32[$pIdx>>2] = $89;
  $90 = HEAP32[$nIdx>>2]|0;
  $91 = (($90) + 1)|0;
  HEAP32[$nIdx>>2] = $91;
 }
 $92 = HEAP32[$db>>2]|0;
 $93 = HEAP32[$pTab>>2]|0;
 $94 = ((($93)) + 34|0);
 $95 = HEAP16[$94>>1]|0;
 $96 = $95 << 16 >> 16;
 $97 = HEAP32[$nIdx>>2]|0;
 $98 = (($96) + ($97))|0;
 $99 = $98<<2;
 $100 = HEAP32[$nIdx>>2]|0;
 $101 = (($99) + ($100))|0;
 $102 = (($101) + 2)|0;
 $103 = (_sqlite3DbMallocRawNN($92,$102,0)|0);
 HEAP32[$aXRef>>2] = $103;
 $104 = HEAP32[$aXRef>>2]|0;
 $105 = ($104|0)==(0|0);
 if ($105) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $106 = HEAP32[$aXRef>>2]|0;
 $107 = HEAP32[$pTab>>2]|0;
 $108 = ((($107)) + 34|0);
 $109 = HEAP16[$108>>1]|0;
 $110 = $109 << 16 >> 16;
 $111 = (($106) + ($110<<2)|0);
 HEAP32[$aRegIdx>>2] = $111;
 $112 = HEAP32[$aRegIdx>>2]|0;
 $113 = HEAP32[$nIdx>>2]|0;
 $114 = (($112) + ($113<<2)|0);
 HEAP32[$aToOpen>>2] = $114;
 $115 = HEAP32[$aToOpen>>2]|0;
 $116 = HEAP32[$nIdx>>2]|0;
 $117 = (($116) + 1)|0;
 _memset(($115|0),1,($117|0))|0;
 $118 = HEAP32[$nIdx>>2]|0;
 $119 = (($118) + 1)|0;
 $120 = HEAP32[$aToOpen>>2]|0;
 $121 = (($120) + ($119)|0);
 HEAP8[$121>>0] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $122 = HEAP32[$i>>2]|0;
  $123 = HEAP32[$pTab>>2]|0;
  $124 = ((($123)) + 34|0);
  $125 = HEAP16[$124>>1]|0;
  $126 = $125 << 16 >> 16;
  $127 = ($122|0)<($126|0);
  if (!($127)) {
   break;
  }
  $128 = HEAP32[$i>>2]|0;
  $129 = HEAP32[$aXRef>>2]|0;
  $130 = (($129) + ($128<<2)|0);
  HEAP32[$130>>2] = -1;
  $131 = HEAP32[$i>>2]|0;
  $132 = (($131) + 1)|0;
  HEAP32[$i>>2] = $132;
 }
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $133 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $133;
 $134 = HEAP32[$1>>2]|0;
 $135 = ((($sNC)) + 4|0);
 HEAP32[$135>>2] = $134;
 HEAP8[$chngPk>>0] = 0;
 HEAP8[$chngRowid>>0] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $136 = HEAP32[$i>>2]|0;
  $137 = HEAP32[$2>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = ($136|0)<($138|0);
  if (!($139)) {
   label = 41;
   break;
  }
  $140 = HEAP32[$i>>2]|0;
  $141 = HEAP32[$2>>2]|0;
  $142 = ((($141)) + 4|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = (($143) + (($140*20)|0)|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = (_sqlite3ResolveExprNames($sNC,$145)|0);
  $147 = ($146|0)!=(0);
  if ($147) {
   label = 185;
   break;
  }
  HEAP32[$j>>2] = 0;
  while(1) {
   $148 = HEAP32[$j>>2]|0;
   $149 = HEAP32[$pTab>>2]|0;
   $150 = ((($149)) + 34|0);
   $151 = HEAP16[$150>>1]|0;
   $152 = $151 << 16 >> 16;
   $153 = ($148|0)<($152|0);
   if (!($153)) {
    break;
   }
   $154 = HEAP32[$j>>2]|0;
   $155 = HEAP32[$pTab>>2]|0;
   $156 = ((($155)) + 4|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (($157) + (($154*24)|0)|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = HEAP32[$i>>2]|0;
   $161 = HEAP32[$2>>2]|0;
   $162 = ((($161)) + 4|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = (($163) + (($160*20)|0)|0);
   $165 = ((($164)) + 4|0);
   $166 = HEAP32[$165>>2]|0;
   $167 = (_sqlite3_stricmp($159,$166)|0);
   $168 = ($167|0)==(0);
   $169 = HEAP32[$j>>2]|0;
   if ($168) {
    label = 23;
    break;
   }
   $197 = (($169) + 1)|0;
   HEAP32[$j>>2] = $197;
  }
  if ((label|0) == 23) {
   label = 0;
   $170 = HEAP32[$pTab>>2]|0;
   $171 = ((($170)) + 32|0);
   $172 = HEAP16[$171>>1]|0;
   $173 = $172 << 16 >> 16;
   $174 = ($169|0)==($173|0);
   if ($174) {
    HEAP8[$chngRowid>>0] = 1;
    $175 = HEAP32[$i>>2]|0;
    $176 = HEAP32[$2>>2]|0;
    $177 = ((($176)) + 4|0);
    $178 = HEAP32[$177>>2]|0;
    $179 = (($178) + (($175*20)|0)|0);
    $180 = HEAP32[$179>>2]|0;
    HEAP32[$pRowidExpr>>2] = $180;
   } else {
    $181 = HEAP32[$pPk>>2]|0;
    $182 = ($181|0)!=(0|0);
    if ($182) {
     $183 = HEAP32[$j>>2]|0;
     $184 = HEAP32[$pTab>>2]|0;
     $185 = ((($184)) + 4|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = (($186) + (($183*24)|0)|0);
     $188 = ((($187)) + 23|0);
     $189 = HEAP8[$188>>0]|0;
     $190 = $189&255;
     $191 = $190 & 1;
     $192 = ($191|0)!=(0);
     if ($192) {
      HEAP8[$chngPk>>0] = 1;
     }
    }
   }
   $193 = HEAP32[$i>>2]|0;
   $194 = HEAP32[$j>>2]|0;
   $195 = HEAP32[$aXRef>>2]|0;
   $196 = (($195) + ($194<<2)|0);
   HEAP32[$196>>2] = $193;
  }
  $198 = HEAP32[$j>>2]|0;
  $199 = HEAP32[$pTab>>2]|0;
  $200 = ((($199)) + 34|0);
  $201 = HEAP16[$200>>1]|0;
  $202 = $201 << 16 >> 16;
  $203 = ($198|0)>=($202|0);
  if ($203) {
   $204 = HEAP32[$pPk>>2]|0;
   $205 = ($204|0)==(0|0);
   if (!($205)) {
    label = 34;
    break;
   }
   $206 = HEAP32[$i>>2]|0;
   $207 = HEAP32[$2>>2]|0;
   $208 = ((($207)) + 4|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = (($209) + (($206*20)|0)|0);
   $211 = ((($210)) + 4|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = (_sqlite3IsRowid($212)|0);
   $214 = ($213|0)!=(0);
   if (!($214)) {
    label = 34;
    break;
   }
   HEAP32[$j>>2] = -1;
   HEAP8[$chngRowid>>0] = 1;
   $215 = HEAP32[$i>>2]|0;
   $216 = HEAP32[$2>>2]|0;
   $217 = ((($216)) + 4|0);
   $218 = HEAP32[$217>>2]|0;
   $219 = (($218) + (($215*20)|0)|0);
   $220 = HEAP32[$219>>2]|0;
   HEAP32[$pRowidExpr>>2] = $220;
  }
  $231 = HEAP32[$0>>2]|0;
  $232 = HEAP32[$pTab>>2]|0;
  $233 = HEAP32[$232>>2]|0;
  $234 = HEAP32[$j>>2]|0;
  $235 = ($234|0)<(0);
  if ($235) {
   $248 = 29600;
  } else {
   $236 = HEAP32[$j>>2]|0;
   $237 = HEAP32[$pTab>>2]|0;
   $238 = ((($237)) + 4|0);
   $239 = HEAP32[$238>>2]|0;
   $240 = (($239) + (($236*24)|0)|0);
   $241 = HEAP32[$240>>2]|0;
   $248 = $241;
  }
  $242 = HEAP32[$iDb>>2]|0;
  $243 = HEAP32[$db>>2]|0;
  $244 = ((($243)) + 16|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = (($245) + ($242<<4)|0);
  $247 = HEAP32[$246>>2]|0;
  $249 = (_sqlite3AuthCheck($231,23,$233,$248,$247)|0);
  HEAP32[$rc>>2] = $249;
  $250 = HEAP32[$rc>>2]|0;
  $251 = ($250|0)==(1);
  if ($251) {
   label = 185;
   break;
  }
  $252 = HEAP32[$rc>>2]|0;
  $253 = ($252|0)==(2);
  if ($253) {
   $254 = HEAP32[$j>>2]|0;
   $255 = HEAP32[$aXRef>>2]|0;
   $256 = (($255) + ($254<<2)|0);
   HEAP32[$256>>2] = -1;
  }
  $257 = HEAP32[$i>>2]|0;
  $258 = (($257) + 1)|0;
  HEAP32[$i>>2] = $258;
 }
 if ((label|0) == 34) {
  $221 = HEAP32[$0>>2]|0;
  $222 = HEAP32[$i>>2]|0;
  $223 = HEAP32[$2>>2]|0;
  $224 = ((($223)) + 4|0);
  $225 = HEAP32[$224>>2]|0;
  $226 = (($225) + (($222*20)|0)|0);
  $227 = ((($226)) + 4|0);
  $228 = HEAP32[$227>>2]|0;
  HEAP32[$vararg_buffer>>2] = $228;
  _sqlite3ErrorMsg($221,33579,$vararg_buffer);
  $229 = HEAP32[$0>>2]|0;
  $230 = ((($229)) + 17|0);
  HEAP8[$230>>0] = 1;
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 41) {
  $259 = HEAP8[$chngRowid>>0]|0;
  $260 = $259&255;
  $261 = HEAP8[$chngPk>>0]|0;
  $262 = $261&255;
  $263 = (($260) + ($262))|0;
  $264 = $263&255;
  HEAP8[$chngKey>>0] = $264;
  $265 = HEAP32[$pTab>>2]|0;
  $266 = ((($265)) + 42|0);
  $267 = HEAP8[$266>>0]|0;
  $268 = $267&255;
  $269 = $268 & 16;
  $270 = ($269|0)!=(0);
  $271 = $270 ? -1 : 0;
  $272 = $270 ? -1 : 0;
  $273 = HEAP32[$1>>2]|0;
  $274 = ((($273)) + 8|0);
  $275 = ((($274)) + 56|0);
  $276 = $275;
  $277 = $276;
  HEAP32[$277>>2] = $271;
  $278 = (($276) + 4)|0;
  $279 = $278;
  HEAP32[$279>>2] = $272;
  $280 = HEAP32[$0>>2]|0;
  $281 = HEAP32[$pTab>>2]|0;
  $282 = HEAP32[$aXRef>>2]|0;
  $283 = HEAP8[$chngKey>>0]|0;
  $284 = $283&255;
  $285 = (_sqlite3FkRequired($280,$281,$282,$284)|0);
  HEAP32[$hasFK>>2] = $285;
  HEAP32[$j>>2] = 0;
  $286 = HEAP32[$pTab>>2]|0;
  $287 = ((($286)) + 8|0);
  $288 = HEAP32[$287>>2]|0;
  HEAP32[$pIdx>>2] = $288;
  while(1) {
   $289 = HEAP32[$pIdx>>2]|0;
   $290 = ($289|0)!=(0|0);
   if (!($290)) {
    break;
   }
   $291 = HEAP8[$chngKey>>0]|0;
   $292 = $291&255;
   $293 = ($292|0)!=(0);
   $294 = HEAP32[$hasFK>>2]|0;
   $295 = ($294|0)!=(0);
   $or$cond3 = $293 | $295;
   L67: do {
    if ($or$cond3) {
     label = 46;
    } else {
     $296 = HEAP32[$pIdx>>2]|0;
     $297 = ((($296)) + 36|0);
     $298 = HEAP32[$297>>2]|0;
     $299 = ($298|0)!=(0|0);
     if ($299) {
      label = 46;
     } else {
      $300 = HEAP32[$pIdx>>2]|0;
      $301 = HEAP32[$pPk>>2]|0;
      $302 = ($300|0)==($301|0);
      if ($302) {
       label = 46;
      } else {
       HEAP32[$reg>>2] = 0;
       HEAP32[$i>>2] = 0;
       while(1) {
        $307 = HEAP32[$i>>2]|0;
        $308 = HEAP32[$pIdx>>2]|0;
        $309 = ((($308)) + 50|0);
        $310 = HEAP16[$309>>1]|0;
        $311 = $310&65535;
        $312 = ($307|0)<($311|0);
        if (!($312)) {
         break L67;
        }
        $313 = HEAP32[$i>>2]|0;
        $314 = HEAP32[$pIdx>>2]|0;
        $315 = ((($314)) + 4|0);
        $316 = HEAP32[$315>>2]|0;
        $317 = (($316) + ($313<<1)|0);
        $318 = HEAP16[$317>>1]|0;
        HEAP16[$iIdxCol>>1] = $318;
        $319 = HEAP16[$iIdxCol>>1]|0;
        $320 = $319 << 16 >> 16;
        $321 = ($320|0)<(0);
        if ($321) {
         break;
        }
        $322 = HEAP16[$iIdxCol>>1]|0;
        $323 = $322 << 16 >> 16;
        $324 = HEAP32[$aXRef>>2]|0;
        $325 = (($324) + ($323<<2)|0);
        $326 = HEAP32[$325>>2]|0;
        $327 = ($326|0)>=(0);
        if ($327) {
         break;
        }
        $332 = HEAP32[$i>>2]|0;
        $333 = (($332) + 1)|0;
        HEAP32[$i>>2] = $333;
       }
       $328 = HEAP32[$0>>2]|0;
       $329 = ((($328)) + 76|0);
       $330 = HEAP32[$329>>2]|0;
       $331 = (($330) + 1)|0;
       HEAP32[$329>>2] = $331;
       HEAP32[$reg>>2] = $331;
      }
     }
    }
   } while(0);
   if ((label|0) == 46) {
    label = 0;
    $303 = HEAP32[$0>>2]|0;
    $304 = ((($303)) + 76|0);
    $305 = HEAP32[$304>>2]|0;
    $306 = (($305) + 1)|0;
    HEAP32[$304>>2] = $306;
    HEAP32[$reg>>2] = $306;
   }
   $334 = HEAP32[$reg>>2]|0;
   $335 = ($334|0)==(0);
   if ($335) {
    $336 = HEAP32[$j>>2]|0;
    $337 = (($336) + 1)|0;
    $338 = HEAP32[$aToOpen>>2]|0;
    $339 = (($338) + ($337)|0);
    HEAP8[$339>>0] = 0;
   }
   $340 = HEAP32[$reg>>2]|0;
   $341 = HEAP32[$j>>2]|0;
   $342 = HEAP32[$aRegIdx>>2]|0;
   $343 = (($342) + ($341<<2)|0);
   HEAP32[$343>>2] = $340;
   $344 = HEAP32[$pIdx>>2]|0;
   $345 = ((($344)) + 20|0);
   $346 = HEAP32[$345>>2]|0;
   HEAP32[$pIdx>>2] = $346;
   $347 = HEAP32[$j>>2]|0;
   $348 = (($347) + 1)|0;
   HEAP32[$j>>2] = $348;
  }
  $349 = HEAP32[$0>>2]|0;
  $350 = (_sqlite3GetVdbe($349)|0);
  HEAP32[$v>>2] = $350;
  $351 = HEAP32[$v>>2]|0;
  $352 = ($351|0)==(0|0);
  if ($352) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $353 = HEAP32[$0>>2]|0;
  $354 = ((($353)) + 18|0);
  $355 = HEAP8[$354>>0]|0;
  $356 = $355&255;
  $357 = ($356|0)==(0);
  if ($357) {
   $358 = HEAP32[$v>>2]|0;
   _sqlite3VdbeCountChanges($358);
  }
  $359 = HEAP32[$0>>2]|0;
  $360 = HEAP32[$iDb>>2]|0;
  _sqlite3BeginWriteOperation($359,1,$360);
  $361 = HEAP32[$pTab>>2]|0;
  $362 = ((($361)) + 42|0);
  $363 = HEAP8[$362>>0]|0;
  $364 = $363&255;
  $365 = $364 & 16;
  $366 = ($365|0)!=(0);
  if (!($366)) {
   $367 = HEAP32[$0>>2]|0;
   $368 = ((($367)) + 76|0);
   $369 = HEAP32[$368>>2]|0;
   $370 = (($369) + 1)|0;
   HEAP32[$368>>2] = $370;
   HEAP32[$regRowSet>>2] = $370;
   $371 = HEAP32[$0>>2]|0;
   $372 = ((($371)) + 76|0);
   $373 = HEAP32[$372>>2]|0;
   $374 = (($373) + 1)|0;
   HEAP32[$372>>2] = $374;
   HEAP32[$regNewRowid>>2] = $374;
   HEAP32[$regOldRowid>>2] = $374;
   $375 = HEAP8[$chngPk>>0]|0;
   $376 = $375&255;
   $377 = ($376|0)!=(0);
   $378 = HEAP32[$pTrigger>>2]|0;
   $379 = ($378|0)!=(0|0);
   $or$cond5 = $377 | $379;
   $380 = HEAP32[$hasFK>>2]|0;
   $381 = ($380|0)!=(0);
   $or$cond7 = $or$cond5 | $381;
   if ($or$cond7) {
    $382 = HEAP32[$0>>2]|0;
    $383 = ((($382)) + 76|0);
    $384 = HEAP32[$383>>2]|0;
    $385 = (($384) + 1)|0;
    HEAP32[$regOld>>2] = $385;
    $386 = HEAP32[$pTab>>2]|0;
    $387 = ((($386)) + 34|0);
    $388 = HEAP16[$387>>1]|0;
    $389 = $388 << 16 >> 16;
    $390 = HEAP32[$0>>2]|0;
    $391 = ((($390)) + 76|0);
    $392 = HEAP32[$391>>2]|0;
    $393 = (($392) + ($389))|0;
    HEAP32[$391>>2] = $393;
   }
   $394 = HEAP8[$chngKey>>0]|0;
   $395 = $394&255;
   $396 = ($395|0)!=(0);
   $397 = HEAP32[$pTrigger>>2]|0;
   $398 = ($397|0)!=(0|0);
   $or$cond9 = $396 | $398;
   $399 = HEAP32[$hasFK>>2]|0;
   $400 = ($399|0)!=(0);
   $or$cond11 = $or$cond9 | $400;
   if ($or$cond11) {
    $401 = HEAP32[$0>>2]|0;
    $402 = ((($401)) + 76|0);
    $403 = HEAP32[$402>>2]|0;
    $404 = (($403) + 1)|0;
    HEAP32[$402>>2] = $404;
    HEAP32[$regNewRowid>>2] = $404;
   }
   $405 = HEAP32[$0>>2]|0;
   $406 = ((($405)) + 76|0);
   $407 = HEAP32[$406>>2]|0;
   $408 = (($407) + 1)|0;
   HEAP32[$regNew>>2] = $408;
   $409 = HEAP32[$pTab>>2]|0;
   $410 = ((($409)) + 34|0);
   $411 = HEAP16[$410>>1]|0;
   $412 = $411 << 16 >> 16;
   $413 = HEAP32[$0>>2]|0;
   $414 = ((($413)) + 76|0);
   $415 = HEAP32[$414>>2]|0;
   $416 = (($415) + ($412))|0;
   HEAP32[$414>>2] = $416;
  }
  $417 = HEAP32[$isView>>2]|0;
  $418 = ($417|0)!=(0);
  if ($418) {
   $419 = HEAP32[$0>>2]|0;
   $420 = HEAP32[$pTab>>2]|0;
   $421 = HEAP32[$420>>2]|0;
   _sqlite3AuthContextPush($419,$sContext,$421);
  }
  $422 = HEAP32[$isView>>2]|0;
  $423 = ($422|0)!=(0);
  if ($423) {
   $424 = HEAP32[$0>>2]|0;
   $425 = HEAP32[$pTab>>2]|0;
   $426 = HEAP32[$3>>2]|0;
   $427 = HEAP32[$iDataCur>>2]|0;
   _sqlite3MaterializeView($424,$425,$426,$427);
  }
  $428 = HEAP32[$3>>2]|0;
  $429 = (_sqlite3ResolveExprNames($sNC,$428)|0);
  $430 = ($429|0)!=(0);
  if ($430) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $431 = HEAP32[$pTab>>2]|0;
  $432 = ((($431)) + 42|0);
  $433 = HEAP8[$432>>0]|0;
  $434 = $433&255;
  $435 = $434 & 16;
  $436 = ($435|0)!=(0);
  if ($436) {
   $437 = HEAP32[$0>>2]|0;
   $438 = HEAP32[$1>>2]|0;
   $439 = HEAP32[$pTab>>2]|0;
   $440 = HEAP32[$2>>2]|0;
   $441 = HEAP32[$pRowidExpr>>2]|0;
   $442 = HEAP32[$aXRef>>2]|0;
   $443 = HEAP32[$3>>2]|0;
   $444 = HEAP32[$4>>2]|0;
   _updateVirtualTable($437,$438,$439,$440,$441,$442,$443,$444);
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $445 = HEAP32[$pTab>>2]|0;
  $446 = ((($445)) + 42|0);
  $447 = HEAP8[$446>>0]|0;
  $448 = $447&255;
  $449 = $448 & 32;
  $450 = ($449|0)==(0);
  if ($450) {
   $451 = HEAP32[$v>>2]|0;
   $452 = HEAP32[$regRowSet>>2]|0;
   $453 = HEAP32[$regOldRowid>>2]|0;
   (_sqlite3VdbeAddOp3($451,25,0,$452,$453)|0);
   $454 = HEAP32[$0>>2]|0;
   $455 = HEAP32[$1>>2]|0;
   $456 = HEAP32[$3>>2]|0;
   $457 = HEAP32[$iIdxCur>>2]|0;
   $458 = (_sqlite3WhereBegin($454,$455,$456,0,0,4,$457)|0);
   HEAP32[$pWInfo>>2] = $458;
   $459 = HEAP32[$pWInfo>>2]|0;
   $460 = ($459|0)==(0|0);
   if ($460) {
    _sqlite3AuthContextPop($sContext);
    $1003 = HEAP32[$db>>2]|0;
    $1004 = HEAP32[$aXRef>>2]|0;
    _sqlite3DbFree($1003,$1004);
    $1005 = HEAP32[$db>>2]|0;
    $1006 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($1005,$1006);
    $1007 = HEAP32[$db>>2]|0;
    $1008 = HEAP32[$2>>2]|0;
    _sqlite3ExprListDelete($1007,$1008);
    $1009 = HEAP32[$db>>2]|0;
    $1010 = HEAP32[$3>>2]|0;
    _sqlite3ExprDelete($1009,$1010);
    STACKTOP = sp;return;
   }
   $461 = HEAP32[$pWInfo>>2]|0;
   $462 = (_sqlite3WhereOkOnePass($461,$aiCurOnePass)|0);
   HEAP32[$okOnePass>>2] = $462;
   $463 = HEAP32[$v>>2]|0;
   $464 = HEAP32[$iDataCur>>2]|0;
   $465 = HEAP32[$regOldRowid>>2]|0;
   (_sqlite3VdbeAddOp2($463,103,$464,$465)|0);
   $466 = HEAP32[$okOnePass>>2]|0;
   $467 = ($466|0)!=(0);
   if (!($467)) {
    $468 = HEAP32[$v>>2]|0;
    $469 = HEAP32[$regRowSet>>2]|0;
    $470 = HEAP32[$regOldRowid>>2]|0;
    (_sqlite3VdbeAddOp2($468,129,$469,$470)|0);
   }
   $471 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($471);
  } else {
   $472 = HEAP32[$pPk>>2]|0;
   $473 = ((($472)) + 50|0);
   $474 = HEAP16[$473>>1]|0;
   HEAP16[$nPk>>1] = $474;
   $475 = HEAP32[$0>>2]|0;
   $476 = ((($475)) + 76|0);
   $477 = HEAP32[$476>>2]|0;
   $478 = (($477) + 1)|0;
   HEAP32[$iPk>>2] = $478;
   $479 = HEAP16[$nPk>>1]|0;
   $480 = $479 << 16 >> 16;
   $481 = HEAP32[$0>>2]|0;
   $482 = ((($481)) + 76|0);
   $483 = HEAP32[$482>>2]|0;
   $484 = (($483) + ($480))|0;
   HEAP32[$482>>2] = $484;
   $485 = HEAP32[$0>>2]|0;
   $486 = ((($485)) + 76|0);
   $487 = HEAP32[$486>>2]|0;
   $488 = (($487) + 1)|0;
   HEAP32[$486>>2] = $488;
   HEAP32[$regKey>>2] = $488;
   $489 = HEAP32[$0>>2]|0;
   $490 = ((($489)) + 72|0);
   $491 = HEAP32[$490>>2]|0;
   $492 = (($491) + 1)|0;
   HEAP32[$490>>2] = $492;
   HEAP32[$iEph>>2] = $491;
   $493 = HEAP32[$v>>2]|0;
   $494 = HEAP32[$iPk>>2]|0;
   (_sqlite3VdbeAddOp2($493,25,0,$494)|0);
   $495 = HEAP32[$v>>2]|0;
   $496 = HEAP32[$iEph>>2]|0;
   $497 = HEAP16[$nPk>>1]|0;
   $498 = $497 << 16 >> 16;
   $499 = (_sqlite3VdbeAddOp2($495,57,$496,$498)|0);
   HEAP32[$addrOpen>>2] = $499;
   $500 = HEAP32[$0>>2]|0;
   $501 = HEAP32[$pPk>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($500,$501);
   $502 = HEAP32[$0>>2]|0;
   $503 = HEAP32[$1>>2]|0;
   $504 = HEAP32[$3>>2]|0;
   $505 = HEAP32[$iIdxCur>>2]|0;
   $506 = (_sqlite3WhereBegin($502,$503,$504,0,0,4,$505)|0);
   HEAP32[$pWInfo>>2] = $506;
   $507 = HEAP32[$pWInfo>>2]|0;
   $508 = ($507|0)==(0|0);
   if ($508) {
    _sqlite3AuthContextPop($sContext);
    $1003 = HEAP32[$db>>2]|0;
    $1004 = HEAP32[$aXRef>>2]|0;
    _sqlite3DbFree($1003,$1004);
    $1005 = HEAP32[$db>>2]|0;
    $1006 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($1005,$1006);
    $1007 = HEAP32[$db>>2]|0;
    $1008 = HEAP32[$2>>2]|0;
    _sqlite3ExprListDelete($1007,$1008);
    $1009 = HEAP32[$db>>2]|0;
    $1010 = HEAP32[$3>>2]|0;
    _sqlite3ExprDelete($1009,$1010);
    STACKTOP = sp;return;
   }
   $509 = HEAP32[$pWInfo>>2]|0;
   $510 = (_sqlite3WhereOkOnePass($509,$aiCurOnePass)|0);
   HEAP32[$okOnePass>>2] = $510;
   HEAP32[$i>>2] = 0;
   while(1) {
    $511 = HEAP32[$i>>2]|0;
    $512 = HEAP16[$nPk>>1]|0;
    $513 = $512 << 16 >> 16;
    $514 = ($511|0)<($513|0);
    if (!($514)) {
     break;
    }
    $515 = HEAP32[$v>>2]|0;
    $516 = HEAP32[$pTab>>2]|0;
    $517 = HEAP32[$iDataCur>>2]|0;
    $518 = HEAP32[$i>>2]|0;
    $519 = HEAP32[$pPk>>2]|0;
    $520 = ((($519)) + 4|0);
    $521 = HEAP32[$520>>2]|0;
    $522 = (($521) + ($518<<1)|0);
    $523 = HEAP16[$522>>1]|0;
    $524 = $523 << 16 >> 16;
    $525 = HEAP32[$iPk>>2]|0;
    $526 = HEAP32[$i>>2]|0;
    $527 = (($525) + ($526))|0;
    _sqlite3ExprCodeGetColumnOfTable($515,$516,$517,$524,$527);
    $528 = HEAP32[$i>>2]|0;
    $529 = (($528) + 1)|0;
    HEAP32[$i>>2] = $529;
   }
   $530 = HEAP32[$okOnePass>>2]|0;
   $531 = ($530|0)!=(0);
   $532 = HEAP32[$v>>2]|0;
   if ($531) {
    $533 = HEAP32[$addrOpen>>2]|0;
    (_sqlite3VdbeChangeToNoop($532,$533)|0);
    $534 = HEAP16[$nPk>>1]|0;
    $535 = $534 << 16 >> 16;
    HEAP32[$nKey>>2] = $535;
    $536 = HEAP32[$iPk>>2]|0;
    HEAP32[$regKey>>2] = $536;
   } else {
    $537 = HEAP32[$iPk>>2]|0;
    $538 = HEAP16[$nPk>>1]|0;
    $539 = $538 << 16 >> 16;
    $540 = HEAP32[$regKey>>2]|0;
    $541 = HEAP32[$db>>2]|0;
    $542 = HEAP32[$pPk>>2]|0;
    $543 = (_sqlite3IndexAffinityStr($541,$542)|0);
    $544 = HEAP16[$nPk>>1]|0;
    $545 = $544 << 16 >> 16;
    (_sqlite3VdbeAddOp4($532,49,$537,$539,$540,$543,$545)|0);
    $546 = HEAP32[$v>>2]|0;
    $547 = HEAP32[$iEph>>2]|0;
    $548 = HEAP32[$regKey>>2]|0;
    (_sqlite3VdbeAddOp2($546,110,$547,$548)|0);
   }
   $549 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($549);
  }
  $550 = HEAP32[$db>>2]|0;
  $551 = ((($550)) + 24|0);
  $552 = HEAP32[$551>>2]|0;
  $553 = $552 & 128;
  $554 = ($553|0)!=(0);
  if ($554) {
   $555 = HEAP32[$0>>2]|0;
   $556 = ((($555)) + 420|0);
   $557 = HEAP32[$556>>2]|0;
   $558 = ($557|0)!=(0|0);
   if (!($558)) {
    $559 = HEAP32[$0>>2]|0;
    $560 = ((($559)) + 76|0);
    $561 = HEAP32[$560>>2]|0;
    $562 = (($561) + 1)|0;
    HEAP32[$560>>2] = $562;
    HEAP32[$regRowCount>>2] = $562;
    $563 = HEAP32[$v>>2]|0;
    $564 = HEAP32[$regRowCount>>2]|0;
    (_sqlite3VdbeAddOp2($563,22,0,$564)|0);
   }
  }
  $565 = HEAP32[$v>>2]|0;
  $566 = (_sqlite3VdbeMakeLabel($565)|0);
  HEAP32[$labelBreak>>2] = $566;
  $567 = HEAP32[$isView>>2]|0;
  $568 = ($567|0)!=(0);
  if (!($568)) {
   $569 = HEAP32[$4>>2]|0;
   $570 = ($569|0)==(5);
   L139: do {
    if ($570) {
     $571 = HEAP32[$aToOpen>>2]|0;
     $572 = HEAP32[$nIdx>>2]|0;
     $573 = (($572) + 1)|0;
     _memset(($571|0),1,($573|0))|0;
    } else {
     $574 = HEAP32[$pTab>>2]|0;
     $575 = ((($574)) + 8|0);
     $576 = HEAP32[$575>>2]|0;
     HEAP32[$pIdx>>2] = $576;
     while(1) {
      $577 = HEAP32[$pIdx>>2]|0;
      $578 = ($577|0)!=(0|0);
      if (!($578)) {
       break L139;
      }
      $579 = HEAP32[$pIdx>>2]|0;
      $580 = ((($579)) + 54|0);
      $581 = HEAP8[$580>>0]|0;
      $582 = $581&255;
      $583 = ($582|0)==(5);
      if ($583) {
       break;
      }
      $587 = HEAP32[$pIdx>>2]|0;
      $588 = ((($587)) + 20|0);
      $589 = HEAP32[$588>>2]|0;
      HEAP32[$pIdx>>2] = $589;
     }
     $584 = HEAP32[$aToOpen>>2]|0;
     $585 = HEAP32[$nIdx>>2]|0;
     $586 = (($585) + 1)|0;
     _memset(($584|0),1,($586|0))|0;
    }
   } while(0);
   $590 = HEAP32[$okOnePass>>2]|0;
   $591 = ($590|0)!=(0);
   do {
    if ($591) {
     $592 = HEAP32[$aiCurOnePass>>2]|0;
     $593 = ($592|0)>=(0);
     if ($593) {
      $594 = HEAP32[$aiCurOnePass>>2]|0;
      $595 = HEAP32[$iBaseCur>>2]|0;
      $596 = (($594) - ($595))|0;
      $597 = HEAP32[$aToOpen>>2]|0;
      $598 = (($597) + ($596)|0);
      HEAP8[$598>>0] = 0;
     }
     $599 = ((($aiCurOnePass)) + 4|0);
     $600 = HEAP32[$599>>2]|0;
     $601 = ($600|0)>=(0);
     if (!($601)) {
      break;
     }
     $602 = ((($aiCurOnePass)) + 4|0);
     $603 = HEAP32[$602>>2]|0;
     $604 = HEAP32[$iBaseCur>>2]|0;
     $605 = (($603) - ($604))|0;
     $606 = HEAP32[$aToOpen>>2]|0;
     $607 = (($606) + ($605)|0);
     HEAP8[$607>>0] = 0;
    }
   } while(0);
   $608 = HEAP32[$0>>2]|0;
   $609 = HEAP32[$pTab>>2]|0;
   $610 = HEAP32[$iBaseCur>>2]|0;
   $611 = HEAP32[$aToOpen>>2]|0;
   (_sqlite3OpenTableAndIndices($608,$609,55,0,$610,$611,0,0)|0);
  }
  $612 = HEAP32[$okOnePass>>2]|0;
  $613 = ($612|0)!=(0);
  do {
   if ($613) {
    $614 = HEAP32[$iDataCur>>2]|0;
    $615 = HEAP32[$iBaseCur>>2]|0;
    $616 = (($614) - ($615))|0;
    $617 = HEAP32[$aToOpen>>2]|0;
    $618 = (($617) + ($616)|0);
    $619 = HEAP8[$618>>0]|0;
    $620 = $619&255;
    $621 = ($620|0)==(0);
    $622 = HEAP32[$isView>>2]|0;
    $623 = ($622|0)!=(0);
    $or$cond13 = $621 | $623;
    if (!($or$cond13)) {
     $624 = HEAP32[$v>>2]|0;
     $625 = HEAP32[$iDataCur>>2]|0;
     $626 = HEAP32[$labelBreak>>2]|0;
     $627 = HEAP32[$regKey>>2]|0;
     $628 = HEAP32[$nKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($624,68,$625,$626,$627,$628)|0);
    }
    $629 = HEAP32[$labelBreak>>2]|0;
    HEAP32[$labelContinue>>2] = $629;
    $630 = HEAP32[$v>>2]|0;
    $631 = HEAP32[$pPk>>2]|0;
    $632 = ($631|0)!=(0|0);
    $633 = HEAP32[$regKey>>2]|0;
    $634 = HEAP32[$regOldRowid>>2]|0;
    $635 = $632 ? $633 : $634;
    $636 = HEAP32[$labelBreak>>2]|0;
    (_sqlite3VdbeAddOp2($630,76,$635,$636)|0);
   } else {
    $637 = HEAP32[$pPk>>2]|0;
    $638 = ($637|0)!=(0|0);
    $639 = HEAP32[$v>>2]|0;
    if ($638) {
     $640 = (_sqlite3VdbeMakeLabel($639)|0);
     HEAP32[$labelContinue>>2] = $640;
     $641 = HEAP32[$v>>2]|0;
     $642 = HEAP32[$iEph>>2]|0;
     $643 = HEAP32[$labelBreak>>2]|0;
     (_sqlite3VdbeAddOp2($641,108,$642,$643)|0);
     $644 = HEAP32[$v>>2]|0;
     $645 = HEAP32[$iEph>>2]|0;
     $646 = HEAP32[$regKey>>2]|0;
     $647 = (_sqlite3VdbeAddOp2($644,101,$645,$646)|0);
     HEAP32[$addrTop>>2] = $647;
     $648 = HEAP32[$v>>2]|0;
     $649 = HEAP32[$iDataCur>>2]|0;
     $650 = HEAP32[$labelContinue>>2]|0;
     $651 = HEAP32[$regKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($648,68,$649,$650,$651,0)|0);
     break;
    } else {
     $652 = HEAP32[$regRowSet>>2]|0;
     $653 = HEAP32[$labelBreak>>2]|0;
     $654 = HEAP32[$regOldRowid>>2]|0;
     $655 = (_sqlite3VdbeAddOp3($639,130,$652,$653,$654)|0);
     HEAP32[$labelContinue>>2] = $655;
     $656 = HEAP32[$v>>2]|0;
     $657 = HEAP32[$iDataCur>>2]|0;
     $658 = HEAP32[$labelContinue>>2]|0;
     $659 = HEAP32[$regOldRowid>>2]|0;
     (_sqlite3VdbeAddOp3($656,70,$657,$658,$659)|0);
     break;
    }
   }
  } while(0);
  $660 = HEAP8[$chngRowid>>0]|0;
  $661 = ($660<<24>>24)!=(0);
  if ($661) {
   $662 = HEAP32[$0>>2]|0;
   $663 = HEAP32[$pRowidExpr>>2]|0;
   $664 = HEAP32[$regNewRowid>>2]|0;
   _sqlite3ExprCode($662,$663,$664);
   $665 = HEAP32[$v>>2]|0;
   $666 = HEAP32[$regNewRowid>>2]|0;
   (_sqlite3VdbeAddOp1($665,38,$666)|0);
  }
  $667 = HEAP8[$chngPk>>0]|0;
  $668 = $667&255;
  $669 = ($668|0)!=(0);
  $670 = HEAP32[$hasFK>>2]|0;
  $671 = ($670|0)!=(0);
  $or$cond15 = $669 | $671;
  $672 = HEAP32[$pTrigger>>2]|0;
  $673 = ($672|0)!=(0|0);
  $or$cond17 = $or$cond15 | $673;
  do {
   if ($or$cond17) {
    $674 = HEAP32[$hasFK>>2]|0;
    $675 = ($674|0)!=(0);
    if ($675) {
     $676 = HEAP32[$0>>2]|0;
     $677 = HEAP32[$pTab>>2]|0;
     $678 = (_sqlite3FkOldmask($676,$677)|0);
     $679 = $678;
    } else {
     $679 = 0;
    }
    HEAP32[$oldmask>>2] = $679;
    $680 = HEAP32[$0>>2]|0;
    $681 = HEAP32[$pTrigger>>2]|0;
    $682 = HEAP32[$2>>2]|0;
    $683 = HEAP32[$pTab>>2]|0;
    $684 = HEAP32[$4>>2]|0;
    $685 = (_sqlite3TriggerColmask($680,$681,$682,0,3,$683,$684)|0);
    $686 = HEAP32[$oldmask>>2]|0;
    $687 = $686 | $685;
    HEAP32[$oldmask>>2] = $687;
    HEAP32[$i>>2] = 0;
    while(1) {
     $688 = HEAP32[$i>>2]|0;
     $689 = HEAP32[$pTab>>2]|0;
     $690 = ((($689)) + 34|0);
     $691 = HEAP16[$690>>1]|0;
     $692 = $691 << 16 >> 16;
     $693 = ($688|0)<($692|0);
     if (!($693)) {
      break;
     }
     $694 = HEAP32[$oldmask>>2]|0;
     $695 = ($694|0)==(-1);
     do {
      if ($695) {
       label = 120;
      } else {
       $696 = HEAP32[$i>>2]|0;
       $697 = ($696|0)<(32);
       if ($697) {
        $698 = HEAP32[$oldmask>>2]|0;
        $699 = HEAP32[$i>>2]|0;
        $700 = 1 << $699;
        $701 = $698 & $700;
        $702 = ($701|0)!=(0);
        if ($702) {
         label = 120;
         break;
        }
       }
       $703 = HEAP32[$i>>2]|0;
       $704 = HEAP32[$pTab>>2]|0;
       $705 = ((($704)) + 4|0);
       $706 = HEAP32[$705>>2]|0;
       $707 = (($706) + (($703*24)|0)|0);
       $708 = ((($707)) + 23|0);
       $709 = HEAP8[$708>>0]|0;
       $710 = $709&255;
       $711 = $710 & 1;
       $712 = ($711|0)!=(0);
       if ($712) {
        label = 120;
        break;
       }
       $720 = HEAP32[$v>>2]|0;
       $721 = HEAP32[$regOld>>2]|0;
       $722 = HEAP32[$i>>2]|0;
       $723 = (($721) + ($722))|0;
       (_sqlite3VdbeAddOp2($720,25,0,$723)|0);
      }
     } while(0);
     if ((label|0) == 120) {
      label = 0;
      $713 = HEAP32[$v>>2]|0;
      $714 = HEAP32[$pTab>>2]|0;
      $715 = HEAP32[$iDataCur>>2]|0;
      $716 = HEAP32[$i>>2]|0;
      $717 = HEAP32[$regOld>>2]|0;
      $718 = HEAP32[$i>>2]|0;
      $719 = (($717) + ($718))|0;
      _sqlite3ExprCodeGetColumnOfTable($713,$714,$715,$716,$719);
     }
     $724 = HEAP32[$i>>2]|0;
     $725 = (($724) + 1)|0;
     HEAP32[$i>>2] = $725;
    }
    $726 = HEAP8[$chngRowid>>0]|0;
    $727 = $726&255;
    $728 = ($727|0)==(0);
    $729 = HEAP32[$pPk>>2]|0;
    $730 = ($729|0)==(0|0);
    $or$cond19 = $728 & $730;
    if (!($or$cond19)) {
     break;
    }
    $731 = HEAP32[$v>>2]|0;
    $732 = HEAP32[$regOldRowid>>2]|0;
    $733 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp2($731,30,$732,$733)|0);
   }
  } while(0);
  $734 = HEAP32[$0>>2]|0;
  $735 = HEAP32[$pTrigger>>2]|0;
  $736 = HEAP32[$2>>2]|0;
  $737 = HEAP32[$pTab>>2]|0;
  $738 = HEAP32[$4>>2]|0;
  $739 = (_sqlite3TriggerColmask($734,$735,$736,1,1,$737,$738)|0);
  HEAP32[$newmask>>2] = $739;
  HEAP32[$i>>2] = 0;
  while(1) {
   $740 = HEAP32[$i>>2]|0;
   $741 = HEAP32[$pTab>>2]|0;
   $742 = ((($741)) + 34|0);
   $743 = HEAP16[$742>>1]|0;
   $744 = $743 << 16 >> 16;
   $745 = ($740|0)<($744|0);
   if (!($745)) {
    break;
   }
   $746 = HEAP32[$i>>2]|0;
   $747 = HEAP32[$pTab>>2]|0;
   $748 = ((($747)) + 32|0);
   $749 = HEAP16[$748>>1]|0;
   $750 = $749 << 16 >> 16;
   $751 = ($746|0)==($750|0);
   L192: do {
    if ($751) {
     $752 = HEAP32[$v>>2]|0;
     $753 = HEAP32[$regNew>>2]|0;
     $754 = HEAP32[$i>>2]|0;
     $755 = (($753) + ($754))|0;
     (_sqlite3VdbeAddOp2($752,25,0,$755)|0);
    } else {
     $756 = HEAP32[$i>>2]|0;
     $757 = HEAP32[$aXRef>>2]|0;
     $758 = (($757) + ($756<<2)|0);
     $759 = HEAP32[$758>>2]|0;
     HEAP32[$j>>2] = $759;
     $760 = HEAP32[$j>>2]|0;
     $761 = ($760|0)>=(0);
     if ($761) {
      $762 = HEAP32[$0>>2]|0;
      $763 = HEAP32[$j>>2]|0;
      $764 = HEAP32[$2>>2]|0;
      $765 = ((($764)) + 4|0);
      $766 = HEAP32[$765>>2]|0;
      $767 = (($766) + (($763*20)|0)|0);
      $768 = HEAP32[$767>>2]|0;
      $769 = HEAP32[$regNew>>2]|0;
      $770 = HEAP32[$i>>2]|0;
      $771 = (($769) + ($770))|0;
      _sqlite3ExprCode($762,$768,$771);
      break;
     }
     $772 = HEAP32[$tmask>>2]|0;
     $773 = $772 & 1;
     $774 = (0)==($773|0);
     $775 = HEAP32[$i>>2]|0;
     $776 = ($775|0)>(31);
     $or$cond21 = $774 | $776;
     do {
      if (!($or$cond21)) {
       $777 = HEAP32[$newmask>>2]|0;
       $778 = HEAP32[$i>>2]|0;
       $779 = 1 << $778;
       $780 = $777 & $779;
       $781 = ($780|0)!=(0);
       if ($781) {
        break;
       }
       $789 = HEAP32[$v>>2]|0;
       $790 = HEAP32[$regNew>>2]|0;
       $791 = HEAP32[$i>>2]|0;
       $792 = (($790) + ($791))|0;
       (_sqlite3VdbeAddOp2($789,25,0,$792)|0);
       break L192;
      }
     } while(0);
     $782 = HEAP32[$0>>2]|0;
     $783 = HEAP32[$pTab>>2]|0;
     $784 = HEAP32[$i>>2]|0;
     $785 = HEAP32[$iDataCur>>2]|0;
     $786 = HEAP32[$regNew>>2]|0;
     $787 = HEAP32[$i>>2]|0;
     $788 = (($786) + ($787))|0;
     _sqlite3ExprCodeGetColumnToReg($782,$783,$784,$785,$788);
    }
   } while(0);
   $793 = HEAP32[$i>>2]|0;
   $794 = (($793) + 1)|0;
   HEAP32[$i>>2] = $794;
  }
  $795 = HEAP32[$tmask>>2]|0;
  $796 = $795 & 1;
  $797 = ($796|0)!=(0);
  L204: do {
   if ($797) {
    $798 = HEAP32[$v>>2]|0;
    $799 = HEAP32[$pTab>>2]|0;
    $800 = HEAP32[$regNew>>2]|0;
    _sqlite3TableAffinity($798,$799,$800);
    $801 = HEAP32[$0>>2]|0;
    $802 = HEAP32[$pTrigger>>2]|0;
    $803 = HEAP32[$2>>2]|0;
    $804 = HEAP32[$pTab>>2]|0;
    $805 = HEAP32[$regOldRowid>>2]|0;
    $806 = HEAP32[$4>>2]|0;
    $807 = HEAP32[$labelContinue>>2]|0;
    _sqlite3CodeRowTrigger($801,$802,110,$803,1,$804,$805,$806,$807);
    $808 = HEAP32[$pPk>>2]|0;
    $809 = ($808|0)!=(0|0);
    $810 = HEAP32[$v>>2]|0;
    $811 = HEAP32[$iDataCur>>2]|0;
    $812 = HEAP32[$labelContinue>>2]|0;
    if ($809) {
     $813 = HEAP32[$regKey>>2]|0;
     $814 = HEAP32[$nKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($810,68,$811,$812,$813,$814)|0);
    } else {
     $815 = HEAP32[$regOldRowid>>2]|0;
     (_sqlite3VdbeAddOp3($810,70,$811,$812,$815)|0);
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $816 = HEAP32[$i>>2]|0;
     $817 = HEAP32[$pTab>>2]|0;
     $818 = ((($817)) + 34|0);
     $819 = HEAP16[$818>>1]|0;
     $820 = $819 << 16 >> 16;
     $821 = ($816|0)<($820|0);
     if (!($821)) {
      break L204;
     }
     $822 = HEAP32[$i>>2]|0;
     $823 = HEAP32[$aXRef>>2]|0;
     $824 = (($823) + ($822<<2)|0);
     $825 = HEAP32[$824>>2]|0;
     $826 = ($825|0)<(0);
     do {
      if ($826) {
       $827 = HEAP32[$i>>2]|0;
       $828 = HEAP32[$pTab>>2]|0;
       $829 = ((($828)) + 32|0);
       $830 = HEAP16[$829>>1]|0;
       $831 = $830 << 16 >> 16;
       $832 = ($827|0)!=($831|0);
       if (!($832)) {
        break;
       }
       $833 = HEAP32[$v>>2]|0;
       $834 = HEAP32[$pTab>>2]|0;
       $835 = HEAP32[$iDataCur>>2]|0;
       $836 = HEAP32[$i>>2]|0;
       $837 = HEAP32[$regNew>>2]|0;
       $838 = HEAP32[$i>>2]|0;
       $839 = (($837) + ($838))|0;
       _sqlite3ExprCodeGetColumnOfTable($833,$834,$835,$836,$839);
      }
     } while(0);
     $840 = HEAP32[$i>>2]|0;
     $841 = (($840) + 1)|0;
     HEAP32[$i>>2] = $841;
    }
   }
  } while(0);
  $842 = HEAP32[$isView>>2]|0;
  $843 = ($842|0)!=(0);
  do {
   if (!($843)) {
    HEAP32[$addr1>>2] = 0;
    HEAP32[$bReplace>>2] = 0;
    $844 = HEAP32[$0>>2]|0;
    $845 = HEAP32[$pTab>>2]|0;
    $846 = HEAP32[$aRegIdx>>2]|0;
    $847 = HEAP32[$iDataCur>>2]|0;
    $848 = HEAP32[$iIdxCur>>2]|0;
    $849 = HEAP32[$regNewRowid>>2]|0;
    $850 = HEAP32[$regOldRowid>>2]|0;
    $851 = HEAP8[$chngKey>>0]|0;
    $852 = HEAP32[$4>>2]|0;
    $853 = $852&255;
    $854 = HEAP32[$labelContinue>>2]|0;
    $855 = HEAP32[$aXRef>>2]|0;
    _sqlite3GenerateConstraintChecks($844,$845,$846,$847,$848,$849,$850,$851,$853,$854,$bReplace,$855);
    $856 = HEAP32[$hasFK>>2]|0;
    $857 = ($856|0)!=(0);
    if ($857) {
     $858 = HEAP32[$0>>2]|0;
     $859 = HEAP32[$pTab>>2]|0;
     $860 = HEAP32[$regOldRowid>>2]|0;
     $861 = HEAP32[$aXRef>>2]|0;
     $862 = HEAP8[$chngKey>>0]|0;
     $863 = $862&255;
     _sqlite3FkCheck($858,$859,$860,0,$861,$863);
    }
    $864 = HEAP32[$bReplace>>2]|0;
    $865 = ($864|0)!=(0);
    if ($865) {
     label = 151;
    } else {
     $866 = HEAP8[$chngKey>>0]|0;
     $867 = $866&255;
     $868 = ($867|0)!=(0);
     if ($868) {
      label = 151;
     }
    }
    do {
     if ((label|0) == 151) {
      $869 = HEAP32[$pPk>>2]|0;
      $870 = ($869|0)!=(0|0);
      $871 = HEAP32[$v>>2]|0;
      $872 = HEAP32[$iDataCur>>2]|0;
      if ($870) {
       $873 = HEAP32[$regKey>>2]|0;
       $874 = HEAP32[$nKey>>2]|0;
       $875 = (_sqlite3VdbeAddOp4Int($871,68,$872,0,$873,$874)|0);
       HEAP32[$addr1>>2] = $875;
       break;
      } else {
       $876 = HEAP32[$regOldRowid>>2]|0;
       $877 = (_sqlite3VdbeAddOp3($871,70,$872,0,$876)|0);
       HEAP32[$addr1>>2] = $877;
       break;
      }
     }
    } while(0);
    $878 = HEAP32[$0>>2]|0;
    $879 = HEAP32[$pTab>>2]|0;
    $880 = HEAP32[$iDataCur>>2]|0;
    $881 = HEAP32[$iIdxCur>>2]|0;
    $882 = HEAP32[$aRegIdx>>2]|0;
    _sqlite3GenerateRowIndexDelete($878,$879,$880,$881,$882,-1);
    $883 = HEAP32[$hasFK>>2]|0;
    $884 = ($883|0)!=(0);
    if ($884) {
     label = 156;
    } else {
     $885 = HEAP8[$chngKey>>0]|0;
     $886 = $885&255;
     $887 = ($886|0)!=(0);
     $888 = HEAP32[$pPk>>2]|0;
     $889 = ($888|0)!=(0|0);
     $or$cond23 = $887 | $889;
     if ($or$cond23) {
      label = 156;
     }
    }
    if ((label|0) == 156) {
     $890 = HEAP32[$v>>2]|0;
     $891 = HEAP32[$iDataCur>>2]|0;
     (_sqlite3VdbeAddOp2($890,95,$891,0)|0);
    }
    $892 = HEAP32[$bReplace>>2]|0;
    $893 = ($892|0)!=(0);
    if ($893) {
     label = 159;
    } else {
     $894 = HEAP8[$chngKey>>0]|0;
     $895 = $894&255;
     $896 = ($895|0)!=(0);
     if ($896) {
      label = 159;
     }
    }
    if ((label|0) == 159) {
     $897 = HEAP32[$v>>2]|0;
     $898 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($897,$898);
    }
    $899 = HEAP32[$hasFK>>2]|0;
    $900 = ($899|0)!=(0);
    if ($900) {
     $901 = HEAP32[$0>>2]|0;
     $902 = HEAP32[$pTab>>2]|0;
     $903 = HEAP32[$regNewRowid>>2]|0;
     $904 = HEAP32[$aXRef>>2]|0;
     $905 = HEAP8[$chngKey>>0]|0;
     $906 = $905&255;
     _sqlite3FkCheck($901,$902,0,$903,$904,$906);
    }
    $907 = HEAP32[$0>>2]|0;
    $908 = HEAP32[$pTab>>2]|0;
    $909 = HEAP32[$iDataCur>>2]|0;
    $910 = HEAP32[$iIdxCur>>2]|0;
    $911 = HEAP32[$regNewRowid>>2]|0;
    $912 = HEAP32[$aRegIdx>>2]|0;
    _sqlite3CompleteInsertion($907,$908,$909,$910,$911,$912,1,0,0);
    $913 = HEAP32[$hasFK>>2]|0;
    $914 = ($913|0)!=(0);
    if (!($914)) {
     break;
    }
    $915 = HEAP32[$0>>2]|0;
    $916 = HEAP32[$pTab>>2]|0;
    $917 = HEAP32[$2>>2]|0;
    $918 = HEAP32[$regOldRowid>>2]|0;
    $919 = HEAP32[$aXRef>>2]|0;
    $920 = HEAP8[$chngKey>>0]|0;
    $921 = $920&255;
    _sqlite3FkActions($915,$916,$917,$918,$919,$921);
   }
  } while(0);
  $922 = HEAP32[$db>>2]|0;
  $923 = ((($922)) + 24|0);
  $924 = HEAP32[$923>>2]|0;
  $925 = $924 & 128;
  $926 = ($925|0)!=(0);
  do {
   if ($926) {
    $927 = HEAP32[$0>>2]|0;
    $928 = ((($927)) + 420|0);
    $929 = HEAP32[$928>>2]|0;
    $930 = ($929|0)!=(0|0);
    if ($930) {
     break;
    }
    $931 = HEAP32[$v>>2]|0;
    $932 = HEAP32[$regRowCount>>2]|0;
    (_sqlite3VdbeAddOp2($931,37,$932,1)|0);
   }
  } while(0);
  $933 = HEAP32[$0>>2]|0;
  $934 = HEAP32[$pTrigger>>2]|0;
  $935 = HEAP32[$2>>2]|0;
  $936 = HEAP32[$pTab>>2]|0;
  $937 = HEAP32[$regOldRowid>>2]|0;
  $938 = HEAP32[$4>>2]|0;
  $939 = HEAP32[$labelContinue>>2]|0;
  _sqlite3CodeRowTrigger($933,$934,110,$935,2,$936,$937,$938,$939);
  $940 = HEAP32[$okOnePass>>2]|0;
  $941 = ($940|0)!=(0);
  do {
   if (!($941)) {
    $942 = HEAP32[$pPk>>2]|0;
    $943 = ($942|0)!=(0|0);
    $944 = HEAP32[$v>>2]|0;
    $945 = HEAP32[$labelContinue>>2]|0;
    if ($943) {
     _sqlite3VdbeResolveLabel($944,$945);
     $946 = HEAP32[$v>>2]|0;
     $947 = HEAP32[$iEph>>2]|0;
     $948 = HEAP32[$addrTop>>2]|0;
     (_sqlite3VdbeAddOp2($946,7,$947,$948)|0);
     break;
    } else {
     (_sqlite3VdbeGoto($944,$945)|0);
     break;
    }
   }
  } while(0);
  $949 = HEAP32[$v>>2]|0;
  $950 = HEAP32[$labelBreak>>2]|0;
  _sqlite3VdbeResolveLabel($949,$950);
  HEAP32[$i>>2] = 0;
  $951 = HEAP32[$pTab>>2]|0;
  $952 = ((($951)) + 8|0);
  $953 = HEAP32[$952>>2]|0;
  HEAP32[$pIdx>>2] = $953;
  while(1) {
   $954 = HEAP32[$pIdx>>2]|0;
   $955 = ($954|0)!=(0|0);
   if (!($955)) {
    break;
   }
   $956 = HEAP32[$i>>2]|0;
   $957 = (($956) + 1)|0;
   $958 = HEAP32[$aToOpen>>2]|0;
   $959 = (($958) + ($957)|0);
   $960 = HEAP8[$959>>0]|0;
   $961 = ($960<<24>>24)!=(0);
   if ($961) {
    $962 = HEAP32[$v>>2]|0;
    $963 = HEAP32[$iIdxCur>>2]|0;
    $964 = HEAP32[$i>>2]|0;
    $965 = (($963) + ($964))|0;
    (_sqlite3VdbeAddOp2($962,61,$965,0)|0);
   }
   $966 = HEAP32[$pIdx>>2]|0;
   $967 = ((($966)) + 20|0);
   $968 = HEAP32[$967>>2]|0;
   HEAP32[$pIdx>>2] = $968;
   $969 = HEAP32[$i>>2]|0;
   $970 = (($969) + 1)|0;
   HEAP32[$i>>2] = $970;
  }
  $971 = HEAP32[$iDataCur>>2]|0;
  $972 = HEAP32[$iIdxCur>>2]|0;
  $973 = ($971|0)<($972|0);
  if ($973) {
   $974 = HEAP32[$v>>2]|0;
   $975 = HEAP32[$iDataCur>>2]|0;
   (_sqlite3VdbeAddOp2($974,61,$975,0)|0);
  }
  $976 = HEAP32[$0>>2]|0;
  $977 = ((($976)) + 18|0);
  $978 = HEAP8[$977>>0]|0;
  $979 = $978&255;
  $980 = ($979|0)==(0);
  do {
   if ($980) {
    $981 = HEAP32[$0>>2]|0;
    $982 = ((($981)) + 420|0);
    $983 = HEAP32[$982>>2]|0;
    $984 = ($983|0)==(0|0);
    if (!($984)) {
     break;
    }
    $985 = HEAP32[$0>>2]|0;
    _sqlite3AutoincrementEnd($985);
   }
  } while(0);
  $986 = HEAP32[$db>>2]|0;
  $987 = ((($986)) + 24|0);
  $988 = HEAP32[$987>>2]|0;
  $989 = $988 & 128;
  $990 = ($989|0)!=(0);
  if (!($990)) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $991 = HEAP32[$0>>2]|0;
  $992 = ((($991)) + 420|0);
  $993 = HEAP32[$992>>2]|0;
  $994 = ($993|0)!=(0|0);
  if ($994) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $995 = HEAP32[$0>>2]|0;
  $996 = ((($995)) + 18|0);
  $997 = HEAP8[$996>>0]|0;
  $998 = ($997<<24>>24)!=(0);
  if ($998) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $999 = HEAP32[$v>>2]|0;
  $1000 = HEAP32[$regRowCount>>2]|0;
  (_sqlite3VdbeAddOp2($999,33,$1000,1)|0);
  $1001 = HEAP32[$v>>2]|0;
  _sqlite3VdbeSetNumCols($1001,1);
  $1002 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeSetColName($1002,0,0,33598,0)|0);
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 185) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
}
function _sqlite3Insert($pParse,$pTabList,$pSelect,$pColumn,$onError) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pSelect = $pSelect|0;
 $pColumn = $pColumn|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aRegIdx = 0, $addr1 = 0, $addr11 = 0, $addrCont = 0, $addrInsTop = 0, $addrL = 0, $addrTop = 0, $appendFlag = 0, $bIdListInOrder = 0;
 var $db = 0, $dest = 0, $endOfLoop = 0, $i = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $iRegStore = 0, $idx = 0, $ipkColumn = 0, $isReplace = 0, $isView = 0, $j = 0, $nColumn = 0, $nHidden = 0, $nIdx = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0;
 var $or$cond7 = 0, $or$cond9 = 0, $pDb = 0, $pIdx = 0, $pList = 0, $pOp = 0, $pTab = 0, $pTrigger = 0, $pVTab = 0, $rc = 0, $regAutoinc = 0, $regCols = 0, $regData = 0, $regFromSelect = 0, $regIns = 0, $regRec = 0, $regRowCount = 0, $regRowid = 0, $regTempRowid = 0, $regYield = 0;
 var $sNC = 0, $srcTab = 0, $tmask = 0, $useTempTable = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer14 = 0, $vararg_buffer20 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0, $withoutRowid = 0, $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $vararg_buffer20 = sp + 32|0;
 $vararg_buffer14 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 288|0;
 $1 = sp + 284|0;
 $2 = sp + 280|0;
 $3 = sp + 276|0;
 $4 = sp + 272|0;
 $db = sp + 268|0;
 $pTab = sp + 264|0;
 $zTab = sp + 260|0;
 $zDb = sp + 256|0;
 $i = sp + 252|0;
 $j = sp + 248|0;
 $idx = sp + 244|0;
 $v = sp + 240|0;
 $pIdx = sp + 236|0;
 $nColumn = sp + 232|0;
 $nHidden = sp + 228|0;
 $iDataCur = sp + 224|0;
 $iIdxCur = sp + 220|0;
 $ipkColumn = sp + 216|0;
 $endOfLoop = sp + 212|0;
 $srcTab = sp + 208|0;
 $addrInsTop = sp + 204|0;
 $addrCont = sp + 200|0;
 $dest = sp + 180|0;
 $iDb = sp + 176|0;
 $pDb = sp + 172|0;
 $useTempTable = sp + 295|0;
 $appendFlag = sp + 294|0;
 $withoutRowid = sp + 293|0;
 $bIdListInOrder = sp + 292|0;
 $pList = sp + 168|0;
 $regFromSelect = sp + 164|0;
 $regAutoinc = sp + 160|0;
 $regRowCount = sp + 156|0;
 $regIns = sp + 152|0;
 $regRowid = sp + 148|0;
 $regData = sp + 144|0;
 $aRegIdx = sp + 140|0;
 $isView = sp + 136|0;
 $pTrigger = sp + 132|0;
 $tmask = sp + 128|0;
 $regYield = sp + 124|0;
 $addrTop = sp + 120|0;
 $rc = sp + 116|0;
 $regRec = sp + 112|0;
 $regTempRowid = sp + 108|0;
 $addrL = sp + 104|0;
 $sNC = sp + 72|0;
 $nIdx = sp + 68|0;
 $regCols = sp + 64|0;
 $addr1 = sp + 60|0;
 $pOp = sp + 56|0;
 $addr11 = sp + 52|0;
 $iRegStore = sp + 48|0;
 $pVTab = sp + 44|0;
 $isReplace = sp + 40|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pColumn;
 HEAP32[$4>>2] = $onError;
 HEAP32[$nHidden>>2] = 0;
 HEAP32[$iDataCur>>2] = 0;
 HEAP32[$iIdxCur>>2] = 0;
 HEAP32[$ipkColumn>>2] = -1;
 HEAP32[$srcTab>>2] = 0;
 HEAP32[$addrInsTop>>2] = 0;
 HEAP32[$addrCont>>2] = 0;
 HEAP8[$useTempTable>>0] = 0;
 HEAP8[$appendFlag>>0] = 0;
 HEAP32[$pList>>2] = 0;
 HEAP32[$regFromSelect>>2] = 0;
 HEAP32[$regAutoinc>>2] = 0;
 HEAP32[$regRowCount>>2] = 0;
 HEAP32[$aRegIdx>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 ;HEAP32[$dest>>2]=0|0;HEAP32[$dest+4>>2]=0|0;HEAP32[$dest+8>>2]=0|0;HEAP32[$dest+12>>2]=0|0;HEAP32[$dest+16>>2]=0|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 L1: do {
  if (!($10)) {
   $11 = HEAP32[$db>>2]|0;
   $12 = ((($11)) + 65|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    $16 = HEAP32[$2>>2]|0;
    $17 = ($16|0)!=(0|0);
    if ($17) {
     $18 = HEAP32[$2>>2]|0;
     $19 = ((($18)) + 6|0);
     $20 = HEAP16[$19>>1]|0;
     $21 = $20&65535;
     $22 = $21 & 256;
     $23 = ($22|0)!=(0);
     if ($23) {
      $24 = HEAP32[$2>>2]|0;
      $25 = ((($24)) + 52|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==(0|0);
      if ($27) {
       $28 = HEAP32[$2>>2]|0;
       $29 = HEAP32[$28>>2]|0;
       HEAP32[$pList>>2] = $29;
       $30 = HEAP32[$2>>2]|0;
       HEAP32[$30>>2] = 0;
       $31 = HEAP32[$db>>2]|0;
       $32 = HEAP32[$2>>2]|0;
       _sqlite3SelectDelete($31,$32);
       HEAP32[$2>>2] = 0;
      }
     }
    }
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 8|0);
    $35 = ((($34)) + 8|0);
    $36 = HEAP32[$35>>2]|0;
    HEAP32[$zTab>>2] = $36;
    $37 = HEAP32[$zTab>>2]|0;
    $38 = ($37|0)==(0|0);
    if (!($38)) {
     $39 = HEAP32[$0>>2]|0;
     $40 = HEAP32[$1>>2]|0;
     $41 = (_sqlite3SrcListLookup($39,$40)|0);
     HEAP32[$pTab>>2] = $41;
     $42 = HEAP32[$pTab>>2]|0;
     $43 = ($42|0)==(0|0);
     if (!($43)) {
      $44 = HEAP32[$db>>2]|0;
      $45 = HEAP32[$pTab>>2]|0;
      $46 = ((($45)) + 64|0);
      $47 = HEAP32[$46>>2]|0;
      $48 = (_sqlite3SchemaToIndex($44,$47)|0);
      HEAP32[$iDb>>2] = $48;
      $49 = HEAP32[$iDb>>2]|0;
      $50 = HEAP32[$db>>2]|0;
      $51 = ((($50)) + 16|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = (($52) + ($49<<4)|0);
      HEAP32[$pDb>>2] = $53;
      $54 = HEAP32[$pDb>>2]|0;
      $55 = HEAP32[$54>>2]|0;
      HEAP32[$zDb>>2] = $55;
      $56 = HEAP32[$0>>2]|0;
      $57 = HEAP32[$pTab>>2]|0;
      $58 = HEAP32[$57>>2]|0;
      $59 = HEAP32[$zDb>>2]|0;
      $60 = (_sqlite3AuthCheck($56,18,$58,0,$59)|0);
      $61 = ($60|0)!=(0);
      if (!($61)) {
       $62 = HEAP32[$pTab>>2]|0;
       $63 = ((($62)) + 42|0);
       $64 = HEAP8[$63>>0]|0;
       $65 = $64&255;
       $66 = $65 & 32;
       $67 = ($66|0)==(0);
       $68 = $67 ^ 1;
       $69 = $68&1;
       $70 = $69&255;
       HEAP8[$withoutRowid>>0] = $70;
       $71 = HEAP32[$0>>2]|0;
       $72 = HEAP32[$pTab>>2]|0;
       $73 = (_sqlite3TriggersExist($71,$72,108,0,$tmask)|0);
       HEAP32[$pTrigger>>2] = $73;
       $74 = HEAP32[$pTab>>2]|0;
       $75 = ((($74)) + 12|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = ($76|0)!=(0|0);
       $78 = $77&1;
       HEAP32[$isView>>2] = $78;
       $79 = HEAP32[$0>>2]|0;
       $80 = HEAP32[$pTab>>2]|0;
       $81 = (_sqlite3ViewGetColumnNames($79,$80)|0);
       $82 = ($81|0)!=(0);
       if (!($82)) {
        $83 = HEAP32[$0>>2]|0;
        $84 = HEAP32[$pTab>>2]|0;
        $85 = HEAP32[$tmask>>2]|0;
        $86 = (_sqlite3IsReadOnly($83,$84,$85)|0);
        $87 = ($86|0)!=(0);
        if (!($87)) {
         $88 = HEAP32[$0>>2]|0;
         $89 = (_sqlite3GetVdbe($88)|0);
         HEAP32[$v>>2] = $89;
         $90 = HEAP32[$v>>2]|0;
         $91 = ($90|0)==(0|0);
         if (!($91)) {
          $92 = HEAP32[$0>>2]|0;
          $93 = ((($92)) + 18|0);
          $94 = HEAP8[$93>>0]|0;
          $95 = $94&255;
          $96 = ($95|0)==(0);
          if ($96) {
           $97 = HEAP32[$v>>2]|0;
           _sqlite3VdbeCountChanges($97);
          }
          $98 = HEAP32[$0>>2]|0;
          $99 = HEAP32[$2>>2]|0;
          $100 = ($99|0)!=(0|0);
          $101 = HEAP32[$pTrigger>>2]|0;
          $102 = ($101|0)!=(0|0);
          $103 = $100 ? 1 : $102;
          $104 = $103&1;
          $105 = HEAP32[$iDb>>2]|0;
          _sqlite3BeginWriteOperation($98,$104,$105);
          $106 = HEAP32[$3>>2]|0;
          $107 = ($106|0)==(0|0);
          if ($107) {
           $108 = HEAP32[$0>>2]|0;
           $109 = HEAP32[$pTab>>2]|0;
           $110 = HEAP32[$2>>2]|0;
           $111 = HEAP32[$4>>2]|0;
           $112 = HEAP32[$iDb>>2]|0;
           $113 = (_xferOptimization($108,$109,$110,$111,$112)|0);
           $114 = ($113|0)!=(0);
           if (!($114)) {
            label = 17;
           }
          } else {
           label = 17;
          }
          L20: do {
           if ((label|0) == 17) {
            $115 = HEAP32[$0>>2]|0;
            $116 = HEAP32[$iDb>>2]|0;
            $117 = HEAP32[$pTab>>2]|0;
            $118 = (_autoIncBegin($115,$116,$117)|0);
            HEAP32[$regAutoinc>>2] = $118;
            $119 = HEAP32[$0>>2]|0;
            $120 = ((($119)) + 76|0);
            $121 = HEAP32[$120>>2]|0;
            $122 = (($121) + 1)|0;
            HEAP32[$regIns>>2] = $122;
            HEAP32[$regRowid>>2] = $122;
            $123 = HEAP32[$pTab>>2]|0;
            $124 = ((($123)) + 34|0);
            $125 = HEAP16[$124>>1]|0;
            $126 = $125 << 16 >> 16;
            $127 = (($126) + 1)|0;
            $128 = HEAP32[$0>>2]|0;
            $129 = ((($128)) + 76|0);
            $130 = HEAP32[$129>>2]|0;
            $131 = (($130) + ($127))|0;
            HEAP32[$129>>2] = $131;
            $132 = HEAP32[$pTab>>2]|0;
            $133 = ((($132)) + 42|0);
            $134 = HEAP8[$133>>0]|0;
            $135 = $134&255;
            $136 = $135 & 16;
            $137 = ($136|0)!=(0);
            if ($137) {
             $138 = HEAP32[$regRowid>>2]|0;
             $139 = (($138) + 1)|0;
             HEAP32[$regRowid>>2] = $139;
             $140 = HEAP32[$0>>2]|0;
             $141 = ((($140)) + 76|0);
             $142 = HEAP32[$141>>2]|0;
             $143 = (($142) + 1)|0;
             HEAP32[$141>>2] = $143;
            }
            $144 = HEAP32[$regRowid>>2]|0;
            $145 = (($144) + 1)|0;
            HEAP32[$regData>>2] = $145;
            $146 = HEAP32[$pTab>>2]|0;
            $147 = ((($146)) + 42|0);
            $148 = HEAP8[$147>>0]|0;
            $149 = $148&255;
            $150 = $149 & 128;
            $151 = ($150|0)==(0);
            $152 = $151&1;
            $153 = $152&255;
            HEAP8[$bIdListInOrder>>0] = $153;
            $154 = HEAP32[$3>>2]|0;
            $155 = ($154|0)!=(0|0);
            L25: do {
             if ($155) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $156 = HEAP32[$i>>2]|0;
               $157 = HEAP32[$3>>2]|0;
               $158 = ((($157)) + 4|0);
               $159 = HEAP32[$158>>2]|0;
               $160 = ($156|0)<($159|0);
               if (!($160)) {
                break;
               }
               $161 = HEAP32[$i>>2]|0;
               $162 = HEAP32[$3>>2]|0;
               $163 = HEAP32[$162>>2]|0;
               $164 = (($163) + ($161<<3)|0);
               $165 = ((($164)) + 4|0);
               HEAP32[$165>>2] = -1;
               $166 = HEAP32[$i>>2]|0;
               $167 = (($166) + 1)|0;
               HEAP32[$i>>2] = $167;
              }
              HEAP32[$i>>2] = 0;
              while(1) {
               $168 = HEAP32[$i>>2]|0;
               $169 = HEAP32[$3>>2]|0;
               $170 = ((($169)) + 4|0);
               $171 = HEAP32[$170>>2]|0;
               $172 = ($168|0)<($171|0);
               if (!($172)) {
                break L25;
               }
               HEAP32[$j>>2] = 0;
               while(1) {
                $173 = HEAP32[$j>>2]|0;
                $174 = HEAP32[$pTab>>2]|0;
                $175 = ((($174)) + 34|0);
                $176 = HEAP16[$175>>1]|0;
                $177 = $176 << 16 >> 16;
                $178 = ($173|0)<($177|0);
                if (!($178)) {
                 break;
                }
                $179 = HEAP32[$i>>2]|0;
                $180 = HEAP32[$3>>2]|0;
                $181 = HEAP32[$180>>2]|0;
                $182 = (($181) + ($179<<3)|0);
                $183 = HEAP32[$182>>2]|0;
                $184 = HEAP32[$j>>2]|0;
                $185 = HEAP32[$pTab>>2]|0;
                $186 = ((($185)) + 4|0);
                $187 = HEAP32[$186>>2]|0;
                $188 = (($187) + (($184*24)|0)|0);
                $189 = HEAP32[$188>>2]|0;
                $190 = (_sqlite3_stricmp($183,$189)|0);
                $191 = ($190|0)==(0);
                $192 = HEAP32[$j>>2]|0;
                if ($191) {
                 label = 28;
                 break;
                }
                $208 = (($192) + 1)|0;
                HEAP32[$j>>2] = $208;
               }
               do {
                if ((label|0) == 28) {
                 label = 0;
                 $193 = HEAP32[$i>>2]|0;
                 $194 = HEAP32[$3>>2]|0;
                 $195 = HEAP32[$194>>2]|0;
                 $196 = (($195) + ($193<<3)|0);
                 $197 = ((($196)) + 4|0);
                 HEAP32[$197>>2] = $192;
                 $198 = HEAP32[$i>>2]|0;
                 $199 = HEAP32[$j>>2]|0;
                 $200 = ($198|0)!=($199|0);
                 if ($200) {
                  HEAP8[$bIdListInOrder>>0] = 0;
                 }
                 $201 = HEAP32[$j>>2]|0;
                 $202 = HEAP32[$pTab>>2]|0;
                 $203 = ((($202)) + 32|0);
                 $204 = HEAP16[$203>>1]|0;
                 $205 = $204 << 16 >> 16;
                 $206 = ($201|0)==($205|0);
                 if (!($206)) {
                  break;
                 }
                 $207 = HEAP32[$i>>2]|0;
                 HEAP32[$ipkColumn>>2] = $207;
                }
               } while(0);
               $209 = HEAP32[$j>>2]|0;
               $210 = HEAP32[$pTab>>2]|0;
               $211 = ((($210)) + 34|0);
               $212 = HEAP16[$211>>1]|0;
               $213 = $212 << 16 >> 16;
               $214 = ($209|0)>=($213|0);
               if ($214) {
                $215 = HEAP32[$i>>2]|0;
                $216 = HEAP32[$3>>2]|0;
                $217 = HEAP32[$216>>2]|0;
                $218 = (($217) + ($215<<3)|0);
                $219 = HEAP32[$218>>2]|0;
                $220 = (_sqlite3IsRowid($219)|0);
                $221 = ($220|0)==(0);
                $222 = HEAP8[$withoutRowid>>0]|0;
                $223 = ($222<<24>>24)!=(0);
                $or$cond = $221 | $223;
                if ($or$cond) {
                 break;
                }
                $224 = HEAP32[$i>>2]|0;
                HEAP32[$ipkColumn>>2] = $224;
                HEAP8[$bIdListInOrder>>0] = 0;
               }
               $234 = HEAP32[$i>>2]|0;
               $235 = (($234) + 1)|0;
               HEAP32[$i>>2] = $235;
              }
              $225 = HEAP32[$0>>2]|0;
              $226 = HEAP32[$1>>2]|0;
              $227 = HEAP32[$i>>2]|0;
              $228 = HEAP32[$3>>2]|0;
              $229 = HEAP32[$228>>2]|0;
              $230 = (($229) + ($227<<3)|0);
              $231 = HEAP32[$230>>2]|0;
              HEAP32[$vararg_buffer>>2] = $226;
              $vararg_ptr12 = ((($vararg_buffer)) + 4|0);
              HEAP32[$vararg_ptr12>>2] = 0;
              $vararg_ptr13 = ((($vararg_buffer)) + 8|0);
              HEAP32[$vararg_ptr13>>2] = $231;
              _sqlite3ErrorMsg($225,33620,$vararg_buffer);
              $232 = HEAP32[$0>>2]|0;
              $233 = ((($232)) + 17|0);
              HEAP8[$233>>0] = 1;
              break L1;
             }
            } while(0);
            $236 = HEAP32[$2>>2]|0;
            $237 = ($236|0)!=(0|0);
            do {
             if ($237) {
              $238 = HEAP32[$0>>2]|0;
              $239 = ((($238)) + 76|0);
              $240 = HEAP32[$239>>2]|0;
              $241 = (($240) + 1)|0;
              HEAP32[$239>>2] = $241;
              HEAP32[$regYield>>2] = $241;
              $242 = HEAP32[$v>>2]|0;
              $243 = (_sqlite3VdbeCurrentAddr($242)|0);
              $244 = (($243) + 1)|0;
              HEAP32[$addrTop>>2] = $244;
              $245 = HEAP32[$v>>2]|0;
              $246 = HEAP32[$regYield>>2]|0;
              $247 = HEAP32[$addrTop>>2]|0;
              (_sqlite3VdbeAddOp3($245,16,$246,0,$247)|0);
              $248 = HEAP32[$regYield>>2]|0;
              _sqlite3SelectDestInit($dest,13,$248);
              $249 = HEAP8[$bIdListInOrder>>0]|0;
              $250 = $249&255;
              $251 = ($250|0)!=(0);
              $252 = HEAP32[$regData>>2]|0;
              $253 = $251 ? $252 : 0;
              $254 = ((($dest)) + 8|0);
              HEAP32[$254>>2] = $253;
              $255 = HEAP32[$pTab>>2]|0;
              $256 = ((($255)) + 34|0);
              $257 = HEAP16[$256>>1]|0;
              $258 = $257 << 16 >> 16;
              $259 = ((($dest)) + 12|0);
              HEAP32[$259>>2] = $258;
              $260 = HEAP32[$0>>2]|0;
              $261 = HEAP32[$2>>2]|0;
              $262 = (_sqlite3Select($260,$261,$dest)|0);
              HEAP32[$rc>>2] = $262;
              $263 = ((($dest)) + 8|0);
              $264 = HEAP32[$263>>2]|0;
              HEAP32[$regFromSelect>>2] = $264;
              $265 = HEAP32[$rc>>2]|0;
              $266 = ($265|0)!=(0);
              if ($266) {
               break L1;
              }
              $267 = HEAP32[$db>>2]|0;
              $268 = ((($267)) + 65|0);
              $269 = HEAP8[$268>>0]|0;
              $270 = $269&255;
              $271 = ($270|0)!=(0);
              if ($271) {
               break L1;
              }
              $272 = HEAP32[$0>>2]|0;
              $273 = ((($272)) + 68|0);
              $274 = HEAP32[$273>>2]|0;
              $275 = ($274|0)!=(0);
              if ($275) {
               break L1;
              }
              $276 = HEAP32[$v>>2]|0;
              $277 = HEAP32[$regYield>>2]|0;
              _sqlite3VdbeEndCoroutine($276,$277);
              $278 = HEAP32[$v>>2]|0;
              $279 = HEAP32[$addrTop>>2]|0;
              $280 = (($279) - 1)|0;
              _sqlite3VdbeJumpHere($278,$280);
              $281 = HEAP32[$2>>2]|0;
              $282 = HEAP32[$281>>2]|0;
              $283 = HEAP32[$282>>2]|0;
              HEAP32[$nColumn>>2] = $283;
              $284 = HEAP32[$pTrigger>>2]|0;
              $285 = ($284|0)!=(0|0);
              if ($285) {
               label = 44;
              } else {
               $286 = HEAP32[$0>>2]|0;
               $287 = HEAP32[$iDb>>2]|0;
               $288 = HEAP32[$pTab>>2]|0;
               $289 = (_readsTable($286,$287,$288)|0);
               $290 = ($289|0)!=(0);
               if ($290) {
                label = 44;
               }
              }
              if ((label|0) == 44) {
               HEAP8[$useTempTable>>0] = 1;
              }
              $291 = HEAP8[$useTempTable>>0]|0;
              $292 = ($291<<24>>24)!=(0);
              if (!($292)) {
               break;
              }
              $293 = HEAP32[$0>>2]|0;
              $294 = ((($293)) + 72|0);
              $295 = HEAP32[$294>>2]|0;
              $296 = (($295) + 1)|0;
              HEAP32[$294>>2] = $296;
              HEAP32[$srcTab>>2] = $295;
              $297 = HEAP32[$0>>2]|0;
              $298 = (_sqlite3GetTempReg($297)|0);
              HEAP32[$regRec>>2] = $298;
              $299 = HEAP32[$0>>2]|0;
              $300 = (_sqlite3GetTempReg($299)|0);
              HEAP32[$regTempRowid>>2] = $300;
              $301 = HEAP32[$v>>2]|0;
              $302 = HEAP32[$srcTab>>2]|0;
              $303 = HEAP32[$nColumn>>2]|0;
              (_sqlite3VdbeAddOp2($301,57,$302,$303)|0);
              $304 = HEAP32[$v>>2]|0;
              $305 = ((($dest)) + 4|0);
              $306 = HEAP32[$305>>2]|0;
              $307 = (_sqlite3VdbeAddOp1($304,18,$306)|0);
              HEAP32[$addrL>>2] = $307;
              $308 = HEAP32[$v>>2]|0;
              $309 = HEAP32[$regFromSelect>>2]|0;
              $310 = HEAP32[$nColumn>>2]|0;
              $311 = HEAP32[$regRec>>2]|0;
              (_sqlite3VdbeAddOp3($308,49,$309,$310,$311)|0);
              $312 = HEAP32[$v>>2]|0;
              $313 = HEAP32[$srcTab>>2]|0;
              $314 = HEAP32[$regTempRowid>>2]|0;
              (_sqlite3VdbeAddOp2($312,74,$313,$314)|0);
              $315 = HEAP32[$v>>2]|0;
              $316 = HEAP32[$srcTab>>2]|0;
              $317 = HEAP32[$regRec>>2]|0;
              $318 = HEAP32[$regTempRowid>>2]|0;
              (_sqlite3VdbeAddOp3($315,75,$316,$317,$318)|0);
              $319 = HEAP32[$v>>2]|0;
              $320 = HEAP32[$addrL>>2]|0;
              (_sqlite3VdbeGoto($319,$320)|0);
              $321 = HEAP32[$v>>2]|0;
              $322 = HEAP32[$addrL>>2]|0;
              _sqlite3VdbeJumpHere($321,$322);
              $323 = HEAP32[$0>>2]|0;
              $324 = HEAP32[$regRec>>2]|0;
              _sqlite3ReleaseTempReg($323,$324);
              $325 = HEAP32[$0>>2]|0;
              $326 = HEAP32[$regTempRowid>>2]|0;
              _sqlite3ReleaseTempReg($325,$326);
             } else {
              ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
              $327 = HEAP32[$0>>2]|0;
              HEAP32[$sNC>>2] = $327;
              HEAP32[$srcTab>>2] = -1;
              $328 = HEAP32[$pList>>2]|0;
              $329 = ($328|0)!=(0|0);
              if ($329) {
               $330 = HEAP32[$pList>>2]|0;
               $331 = HEAP32[$330>>2]|0;
               HEAP32[$nColumn>>2] = $331;
               $332 = HEAP32[$pList>>2]|0;
               $333 = (_sqlite3ResolveExprListNames($sNC,$332)|0);
               $334 = ($333|0)!=(0);
               if ($334) {
                break L1;
               } else {
                break;
               }
              } else {
               HEAP32[$nColumn>>2] = 0;
               break;
              }
             }
            } while(0);
            $335 = HEAP32[$3>>2]|0;
            $336 = ($335|0)==(0|0);
            $337 = HEAP32[$nColumn>>2]|0;
            $338 = ($337|0)>(0);
            $or$cond3 = $336 & $338;
            if ($or$cond3) {
             $339 = HEAP32[$pTab>>2]|0;
             $340 = ((($339)) + 32|0);
             $341 = HEAP16[$340>>1]|0;
             $342 = $341 << 16 >> 16;
             HEAP32[$ipkColumn>>2] = $342;
            }
            HEAP32[$i>>2] = 0;
            while(1) {
             $343 = HEAP32[$i>>2]|0;
             $344 = HEAP32[$pTab>>2]|0;
             $345 = ((($344)) + 34|0);
             $346 = HEAP16[$345>>1]|0;
             $347 = $346 << 16 >> 16;
             $348 = ($343|0)<($347|0);
             if (!($348)) {
              break;
             }
             $349 = HEAP32[$i>>2]|0;
             $350 = HEAP32[$pTab>>2]|0;
             $351 = ((($350)) + 4|0);
             $352 = HEAP32[$351>>2]|0;
             $353 = (($352) + (($349*24)|0)|0);
             $354 = ((($353)) + 23|0);
             $355 = HEAP8[$354>>0]|0;
             $356 = $355&255;
             $357 = $356 & 2;
             $358 = ($357|0)!=(0);
             $359 = $358 ? 1 : 0;
             $360 = HEAP32[$nHidden>>2]|0;
             $361 = (($360) + ($359))|0;
             HEAP32[$nHidden>>2] = $361;
             $362 = HEAP32[$i>>2]|0;
             $363 = (($362) + 1)|0;
             HEAP32[$i>>2] = $363;
            }
            $364 = HEAP32[$3>>2]|0;
            $365 = ($364|0)==(0|0);
            $366 = HEAP32[$nColumn>>2]|0;
            $367 = ($366|0)!=(0);
            $or$cond5 = $365 & $367;
            if ($or$cond5) {
             $368 = HEAP32[$nColumn>>2]|0;
             $369 = HEAP32[$pTab>>2]|0;
             $370 = ((($369)) + 34|0);
             $371 = HEAP16[$370>>1]|0;
             $372 = $371 << 16 >> 16;
             $373 = HEAP32[$nHidden>>2]|0;
             $374 = (($372) - ($373))|0;
             $375 = ($368|0)!=($374|0);
             if ($375) {
              $376 = HEAP32[$0>>2]|0;
              $377 = HEAP32[$1>>2]|0;
              $378 = HEAP32[$pTab>>2]|0;
              $379 = ((($378)) + 34|0);
              $380 = HEAP16[$379>>1]|0;
              $381 = $380 << 16 >> 16;
              $382 = HEAP32[$nHidden>>2]|0;
              $383 = (($381) - ($382))|0;
              $384 = HEAP32[$nColumn>>2]|0;
              HEAP32[$vararg_buffer14>>2] = $377;
              $vararg_ptr17 = ((($vararg_buffer14)) + 4|0);
              HEAP32[$vararg_ptr17>>2] = 0;
              $vararg_ptr18 = ((($vararg_buffer14)) + 8|0);
              HEAP32[$vararg_ptr18>>2] = $383;
              $vararg_ptr19 = ((($vararg_buffer14)) + 12|0);
              HEAP32[$vararg_ptr19>>2] = $384;
              _sqlite3ErrorMsg($376,33652,$vararg_buffer14);
              break L1;
             }
            }
            $385 = HEAP32[$3>>2]|0;
            $386 = ($385|0)!=(0|0);
            do {
             if ($386) {
              $387 = HEAP32[$nColumn>>2]|0;
              $388 = HEAP32[$3>>2]|0;
              $389 = ((($388)) + 4|0);
              $390 = HEAP32[$389>>2]|0;
              $391 = ($387|0)!=($390|0);
              if (!($391)) {
               break;
              }
              $392 = HEAP32[$0>>2]|0;
              $393 = HEAP32[$nColumn>>2]|0;
              $394 = HEAP32[$3>>2]|0;
              $395 = ((($394)) + 4|0);
              $396 = HEAP32[$395>>2]|0;
              HEAP32[$vararg_buffer20>>2] = $393;
              $vararg_ptr23 = ((($vararg_buffer20)) + 4|0);
              HEAP32[$vararg_ptr23>>2] = $396;
              _sqlite3ErrorMsg($392,33704,$vararg_buffer20);
              break L1;
             }
            } while(0);
            $397 = HEAP32[$db>>2]|0;
            $398 = ((($397)) + 24|0);
            $399 = HEAP32[$398>>2]|0;
            $400 = $399 & 128;
            $401 = ($400|0)!=(0);
            if ($401) {
             $402 = HEAP32[$0>>2]|0;
             $403 = ((($402)) + 76|0);
             $404 = HEAP32[$403>>2]|0;
             $405 = (($404) + 1)|0;
             HEAP32[$403>>2] = $405;
             HEAP32[$regRowCount>>2] = $405;
             $406 = HEAP32[$v>>2]|0;
             $407 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($406,22,0,$407)|0);
            }
            $408 = HEAP32[$isView>>2]|0;
            $409 = ($408|0)!=(0);
            L85: do {
             if (!($409)) {
              $410 = HEAP32[$0>>2]|0;
              $411 = HEAP32[$pTab>>2]|0;
              $412 = (_sqlite3OpenTableAndIndices($410,$411,55,0,-1,0,$iDataCur,$iIdxCur)|0);
              HEAP32[$nIdx>>2] = $412;
              $413 = HEAP32[$db>>2]|0;
              $414 = HEAP32[$nIdx>>2]|0;
              $415 = (($414) + 1)|0;
              $416 = $415<<2;
              $417 = (_sqlite3DbMallocRawNN($413,$416,0)|0);
              HEAP32[$aRegIdx>>2] = $417;
              $418 = HEAP32[$aRegIdx>>2]|0;
              $419 = ($418|0)==(0|0);
              if ($419) {
               break L1;
              }
              HEAP32[$i>>2] = 0;
              while(1) {
               $420 = HEAP32[$i>>2]|0;
               $421 = HEAP32[$nIdx>>2]|0;
               $422 = ($420|0)<($421|0);
               if (!($422)) {
                break L85;
               }
               $423 = HEAP32[$0>>2]|0;
               $424 = ((($423)) + 76|0);
               $425 = HEAP32[$424>>2]|0;
               $426 = (($425) + 1)|0;
               HEAP32[$424>>2] = $426;
               $427 = HEAP32[$i>>2]|0;
               $428 = HEAP32[$aRegIdx>>2]|0;
               $429 = (($428) + ($427<<2)|0);
               HEAP32[$429>>2] = $426;
               $430 = HEAP32[$i>>2]|0;
               $431 = (($430) + 1)|0;
               HEAP32[$i>>2] = $431;
              }
             }
            } while(0);
            $432 = HEAP8[$useTempTable>>0]|0;
            $433 = ($432<<24>>24)!=(0);
            do {
             if ($433) {
              $434 = HEAP32[$v>>2]|0;
              $435 = HEAP32[$srcTab>>2]|0;
              $436 = (_sqlite3VdbeAddOp1($434,108,$435)|0);
              HEAP32[$addrInsTop>>2] = $436;
              $437 = HEAP32[$v>>2]|0;
              $438 = (_sqlite3VdbeCurrentAddr($437)|0);
              HEAP32[$addrCont>>2] = $438;
             } else {
              $439 = HEAP32[$2>>2]|0;
              $440 = ($439|0)!=(0|0);
              if (!($440)) {
               break;
              }
              $441 = HEAP32[$v>>2]|0;
              $442 = ((($dest)) + 4|0);
              $443 = HEAP32[$442>>2]|0;
              $444 = (_sqlite3VdbeAddOp1($441,18,$443)|0);
              HEAP32[$addrCont>>2] = $444;
              HEAP32[$addrInsTop>>2] = $444;
             }
            } while(0);
            $445 = HEAP32[$v>>2]|0;
            $446 = (_sqlite3VdbeMakeLabel($445)|0);
            HEAP32[$endOfLoop>>2] = $446;
            $447 = HEAP32[$tmask>>2]|0;
            $448 = $447 & 1;
            $449 = ($448|0)!=(0);
            if ($449) {
             $450 = HEAP32[$0>>2]|0;
             $451 = HEAP32[$pTab>>2]|0;
             $452 = ((($451)) + 34|0);
             $453 = HEAP16[$452>>1]|0;
             $454 = $453 << 16 >> 16;
             $455 = (($454) + 1)|0;
             $456 = (_sqlite3GetTempRange($450,$455)|0);
             HEAP32[$regCols>>2] = $456;
             $457 = HEAP32[$ipkColumn>>2]|0;
             $458 = ($457|0)<(0);
             if ($458) {
              $459 = HEAP32[$v>>2]|0;
              $460 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp2($459,22,-1,$460)|0);
             } else {
              $461 = HEAP8[$useTempTable>>0]|0;
              $462 = ($461<<24>>24)!=(0);
              if ($462) {
               $463 = HEAP32[$v>>2]|0;
               $464 = HEAP32[$srcTab>>2]|0;
               $465 = HEAP32[$ipkColumn>>2]|0;
               $466 = HEAP32[$regCols>>2]|0;
               (_sqlite3VdbeAddOp3($463,47,$464,$465,$466)|0);
              } else {
               $467 = HEAP32[$0>>2]|0;
               $468 = HEAP32[$ipkColumn>>2]|0;
               $469 = HEAP32[$pList>>2]|0;
               $470 = ((($469)) + 4|0);
               $471 = HEAP32[$470>>2]|0;
               $472 = (($471) + (($468*20)|0)|0);
               $473 = HEAP32[$472>>2]|0;
               $474 = HEAP32[$regCols>>2]|0;
               _sqlite3ExprCode($467,$473,$474);
              }
              $475 = HEAP32[$v>>2]|0;
              $476 = HEAP32[$regCols>>2]|0;
              $477 = (_sqlite3VdbeAddOp1($475,77,$476)|0);
              HEAP32[$addr1>>2] = $477;
              $478 = HEAP32[$v>>2]|0;
              $479 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp2($478,22,-1,$479)|0);
              $480 = HEAP32[$v>>2]|0;
              $481 = HEAP32[$addr1>>2]|0;
              _sqlite3VdbeJumpHere($480,$481);
              $482 = HEAP32[$v>>2]|0;
              $483 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp1($482,38,$483)|0);
             }
             HEAP32[$j>>2] = 0;
             HEAP32[$i>>2] = 0;
             while(1) {
              $484 = HEAP32[$i>>2]|0;
              $485 = HEAP32[$pTab>>2]|0;
              $486 = ((($485)) + 34|0);
              $487 = HEAP16[$486>>1]|0;
              $488 = $487 << 16 >> 16;
              $489 = ($484|0)<($488|0);
              if (!($489)) {
               break;
              }
              $490 = HEAP32[$3>>2]|0;
              $491 = ($490|0)!=(0|0);
              L110: do {
               if ($491) {
                HEAP32[$j>>2] = 0;
                while(1) {
                 $492 = HEAP32[$j>>2]|0;
                 $493 = HEAP32[$3>>2]|0;
                 $494 = ((($493)) + 4|0);
                 $495 = HEAP32[$494>>2]|0;
                 $496 = ($492|0)<($495|0);
                 if (!($496)) {
                  break L110;
                 }
                 $497 = HEAP32[$j>>2]|0;
                 $498 = HEAP32[$3>>2]|0;
                 $499 = HEAP32[$498>>2]|0;
                 $500 = (($499) + ($497<<3)|0);
                 $501 = ((($500)) + 4|0);
                 $502 = HEAP32[$501>>2]|0;
                 $503 = HEAP32[$i>>2]|0;
                 $504 = ($502|0)==($503|0);
                 if ($504) {
                  break L110;
                 }
                 $505 = HEAP32[$j>>2]|0;
                 $506 = (($505) + 1)|0;
                 HEAP32[$j>>2] = $506;
                }
               }
              } while(0);
              $507 = HEAP8[$useTempTable>>0]|0;
              $508 = ($507<<24>>24)!=(0);
              $509 = HEAP32[$pList>>2]|0;
              $510 = ($509|0)!=(0|0);
              $or$cond7 = $508 | $510;
              do {
               if ($or$cond7) {
                $511 = HEAP32[$3>>2]|0;
                $512 = ($511|0)!=(0|0);
                if ($512) {
                 $513 = HEAP32[$j>>2]|0;
                 $514 = HEAP32[$3>>2]|0;
                 $515 = ((($514)) + 4|0);
                 $516 = HEAP32[$515>>2]|0;
                 $517 = ($513|0)>=($516|0);
                 if ($517) {
                  label = 90;
                  break;
                 }
                }
                $518 = HEAP8[$useTempTable>>0]|0;
                $519 = ($518<<24>>24)!=(0);
                if ($519) {
                 $532 = HEAP32[$v>>2]|0;
                 $533 = HEAP32[$srcTab>>2]|0;
                 $534 = HEAP32[$j>>2]|0;
                 $535 = HEAP32[$regCols>>2]|0;
                 $536 = HEAP32[$i>>2]|0;
                 $537 = (($535) + ($536))|0;
                 $538 = (($537) + 1)|0;
                 (_sqlite3VdbeAddOp3($532,47,$533,$534,$538)|0);
                 break;
                } else {
                 $539 = HEAP32[$0>>2]|0;
                 $540 = HEAP32[$j>>2]|0;
                 $541 = HEAP32[$pList>>2]|0;
                 $542 = ((($541)) + 4|0);
                 $543 = HEAP32[$542>>2]|0;
                 $544 = (($543) + (($540*20)|0)|0);
                 $545 = HEAP32[$544>>2]|0;
                 $546 = HEAP32[$regCols>>2]|0;
                 $547 = HEAP32[$i>>2]|0;
                 $548 = (($546) + ($547))|0;
                 $549 = (($548) + 1)|0;
                 _sqlite3ExprCodeAndCache($539,$545,$549);
                 break;
                }
               } else {
                label = 90;
               }
              } while(0);
              if ((label|0) == 90) {
               label = 0;
               $520 = HEAP32[$0>>2]|0;
               $521 = HEAP32[$i>>2]|0;
               $522 = HEAP32[$pTab>>2]|0;
               $523 = ((($522)) + 4|0);
               $524 = HEAP32[$523>>2]|0;
               $525 = (($524) + (($521*24)|0)|0);
               $526 = ((($525)) + 4|0);
               $527 = HEAP32[$526>>2]|0;
               $528 = HEAP32[$regCols>>2]|0;
               $529 = HEAP32[$i>>2]|0;
               $530 = (($528) + ($529))|0;
               $531 = (($530) + 1)|0;
               _sqlite3ExprCode($520,$527,$531);
              }
              $550 = HEAP32[$3>>2]|0;
              $551 = ($550|0)==(0|0);
              if ($551) {
               $552 = HEAP32[$j>>2]|0;
               $553 = (($552) + 1)|0;
               HEAP32[$j>>2] = $553;
              }
              $554 = HEAP32[$i>>2]|0;
              $555 = (($554) + 1)|0;
              HEAP32[$i>>2] = $555;
             }
             $556 = HEAP32[$isView>>2]|0;
             $557 = ($556|0)!=(0);
             if (!($557)) {
              $558 = HEAP32[$v>>2]|0;
              $559 = HEAP32[$pTab>>2]|0;
              $560 = HEAP32[$regCols>>2]|0;
              $561 = (($560) + 1)|0;
              _sqlite3TableAffinity($558,$559,$561);
             }
             $562 = HEAP32[$0>>2]|0;
             $563 = HEAP32[$pTrigger>>2]|0;
             $564 = HEAP32[$pTab>>2]|0;
             $565 = HEAP32[$regCols>>2]|0;
             $566 = HEAP32[$pTab>>2]|0;
             $567 = ((($566)) + 34|0);
             $568 = HEAP16[$567>>1]|0;
             $569 = $568 << 16 >> 16;
             $570 = (($565) - ($569))|0;
             $571 = (($570) - 1)|0;
             $572 = HEAP32[$4>>2]|0;
             $573 = HEAP32[$endOfLoop>>2]|0;
             _sqlite3CodeRowTrigger($562,$563,108,0,1,$564,$571,$572,$573);
             $574 = HEAP32[$0>>2]|0;
             $575 = HEAP32[$regCols>>2]|0;
             $576 = HEAP32[$pTab>>2]|0;
             $577 = ((($576)) + 34|0);
             $578 = HEAP16[$577>>1]|0;
             $579 = $578 << 16 >> 16;
             $580 = (($579) + 1)|0;
             _sqlite3ReleaseTempRange($574,$575,$580);
            }
            $581 = HEAP32[$isView>>2]|0;
            $582 = ($581|0)!=(0);
            do {
             if (!($582)) {
              $583 = HEAP32[$pTab>>2]|0;
              $584 = ((($583)) + 42|0);
              $585 = HEAP8[$584>>0]|0;
              $586 = $585&255;
              $587 = $586 & 16;
              $588 = ($587|0)!=(0);
              if ($588) {
               $589 = HEAP32[$v>>2]|0;
               $590 = HEAP32[$regIns>>2]|0;
               (_sqlite3VdbeAddOp2($589,25,0,$590)|0);
              }
              $591 = HEAP32[$ipkColumn>>2]|0;
              $592 = ($591|0)>=(0);
              L141: do {
               if ($592) {
                $593 = HEAP8[$useTempTable>>0]|0;
                $594 = ($593<<24>>24)!=(0);
                do {
                 if ($594) {
                  $595 = HEAP32[$v>>2]|0;
                  $596 = HEAP32[$srcTab>>2]|0;
                  $597 = HEAP32[$ipkColumn>>2]|0;
                  $598 = HEAP32[$regRowid>>2]|0;
                  (_sqlite3VdbeAddOp3($595,47,$596,$597,$598)|0);
                 } else {
                  $599 = HEAP32[$2>>2]|0;
                  $600 = ($599|0)!=(0|0);
                  if ($600) {
                   $601 = HEAP32[$v>>2]|0;
                   $602 = HEAP32[$regFromSelect>>2]|0;
                   $603 = HEAP32[$ipkColumn>>2]|0;
                   $604 = (($602) + ($603))|0;
                   $605 = HEAP32[$regRowid>>2]|0;
                   (_sqlite3VdbeAddOp2($601,30,$604,$605)|0);
                   break;
                  }
                  $606 = HEAP32[$0>>2]|0;
                  $607 = HEAP32[$ipkColumn>>2]|0;
                  $608 = HEAP32[$pList>>2]|0;
                  $609 = ((($608)) + 4|0);
                  $610 = HEAP32[$609>>2]|0;
                  $611 = (($610) + (($607*20)|0)|0);
                  $612 = HEAP32[$611>>2]|0;
                  $613 = HEAP32[$regRowid>>2]|0;
                  _sqlite3ExprCode($606,$612,$613);
                  $614 = HEAP32[$v>>2]|0;
                  $615 = (_sqlite3VdbeGetOp($614,-1)|0);
                  HEAP32[$pOp>>2] = $615;
                  $616 = HEAP32[$pOp>>2]|0;
                  $617 = ($616|0)!=(0|0);
                  if (!($617)) {
                   break;
                  }
                  $618 = HEAP32[$pOp>>2]|0;
                  $619 = HEAP8[$618>>0]|0;
                  $620 = $619&255;
                  $621 = ($620|0)==(25);
                  if (!($621)) {
                   break;
                  }
                  $622 = HEAP32[$pTab>>2]|0;
                  $623 = ((($622)) + 42|0);
                  $624 = HEAP8[$623>>0]|0;
                  $625 = $624&255;
                  $626 = $625 & 16;
                  $627 = ($626|0)!=(0);
                  if ($627) {
                   break;
                  }
                  HEAP8[$appendFlag>>0] = 1;
                  $628 = HEAP32[$pOp>>2]|0;
                  HEAP8[$628>>0] = 74;
                  $629 = HEAP32[$iDataCur>>2]|0;
                  $630 = HEAP32[$pOp>>2]|0;
                  $631 = ((($630)) + 4|0);
                  HEAP32[$631>>2] = $629;
                  $632 = HEAP32[$regRowid>>2]|0;
                  $633 = HEAP32[$pOp>>2]|0;
                  $634 = ((($633)) + 8|0);
                  HEAP32[$634>>2] = $632;
                  $635 = HEAP32[$regAutoinc>>2]|0;
                  $636 = HEAP32[$pOp>>2]|0;
                  $637 = ((($636)) + 12|0);
                  HEAP32[$637>>2] = $635;
                 }
                } while(0);
                $638 = HEAP8[$appendFlag>>0]|0;
                $639 = ($638<<24>>24)!=(0);
                if ($639) {
                 break;
                }
                $640 = HEAP32[$pTab>>2]|0;
                $641 = ((($640)) + 42|0);
                $642 = HEAP8[$641>>0]|0;
                $643 = $642&255;
                $644 = $643 & 16;
                $645 = ($644|0)!=(0);
                $646 = HEAP32[$v>>2]|0;
                if ($645) {
                 $655 = (_sqlite3VdbeCurrentAddr($646)|0);
                 HEAP32[$addr11>>2] = $655;
                 $656 = HEAP32[$v>>2]|0;
                 $657 = HEAP32[$regRowid>>2]|0;
                 $658 = HEAP32[$addr11>>2]|0;
                 $659 = (($658) + 2)|0;
                 (_sqlite3VdbeAddOp2($656,76,$657,$659)|0);
                } else {
                 $647 = HEAP32[$regRowid>>2]|0;
                 $648 = (_sqlite3VdbeAddOp1($646,77,$647)|0);
                 HEAP32[$addr11>>2] = $648;
                 $649 = HEAP32[$v>>2]|0;
                 $650 = HEAP32[$iDataCur>>2]|0;
                 $651 = HEAP32[$regRowid>>2]|0;
                 $652 = HEAP32[$regAutoinc>>2]|0;
                 (_sqlite3VdbeAddOp3($649,74,$650,$651,$652)|0);
                 $653 = HEAP32[$v>>2]|0;
                 $654 = HEAP32[$addr11>>2]|0;
                 _sqlite3VdbeJumpHere($653,$654);
                }
                $660 = HEAP32[$v>>2]|0;
                $661 = HEAP32[$regRowid>>2]|0;
                (_sqlite3VdbeAddOp1($660,38,$661)|0);
               } else {
                $662 = HEAP32[$pTab>>2]|0;
                $663 = ((($662)) + 42|0);
                $664 = HEAP8[$663>>0]|0;
                $665 = $664&255;
                $666 = $665 & 16;
                $667 = ($666|0)!=(0);
                do {
                 if (!($667)) {
                  $668 = HEAP8[$withoutRowid>>0]|0;
                  $669 = $668&255;
                  $670 = ($669|0)!=(0);
                  if ($670) {
                   break;
                  }
                  $673 = HEAP32[$v>>2]|0;
                  $674 = HEAP32[$iDataCur>>2]|0;
                  $675 = HEAP32[$regRowid>>2]|0;
                  $676 = HEAP32[$regAutoinc>>2]|0;
                  (_sqlite3VdbeAddOp3($673,74,$674,$675,$676)|0);
                  HEAP8[$appendFlag>>0] = 1;
                  break L141;
                 }
                } while(0);
                $671 = HEAP32[$v>>2]|0;
                $672 = HEAP32[$regRowid>>2]|0;
                (_sqlite3VdbeAddOp2($671,25,0,$672)|0);
               }
              } while(0);
              $677 = HEAP32[$0>>2]|0;
              $678 = HEAP32[$regAutoinc>>2]|0;
              $679 = HEAP32[$regRowid>>2]|0;
              _autoIncStep($677,$678,$679);
              HEAP32[$nHidden>>2] = 0;
              HEAP32[$i>>2] = 0;
              while(1) {
               $680 = HEAP32[$i>>2]|0;
               $681 = HEAP32[$pTab>>2]|0;
               $682 = ((($681)) + 34|0);
               $683 = HEAP16[$682>>1]|0;
               $684 = $683 << 16 >> 16;
               $685 = ($680|0)<($684|0);
               if (!($685)) {
                break;
               }
               $686 = HEAP32[$regRowid>>2]|0;
               $687 = (($686) + 1)|0;
               $688 = HEAP32[$i>>2]|0;
               $689 = (($687) + ($688))|0;
               HEAP32[$iRegStore>>2] = $689;
               $690 = HEAP32[$i>>2]|0;
               $691 = HEAP32[$pTab>>2]|0;
               $692 = ((($691)) + 32|0);
               $693 = HEAP16[$692>>1]|0;
               $694 = $693 << 16 >> 16;
               $695 = ($690|0)==($694|0);
               L167: do {
                if ($695) {
                 $696 = HEAP32[$v>>2]|0;
                 $697 = HEAP32[$iRegStore>>2]|0;
                 (_sqlite3VdbeAddOp1($696,26,$697)|0);
                } else {
                 $698 = HEAP32[$3>>2]|0;
                 $699 = ($698|0)==(0|0);
                 L170: do {
                  if ($699) {
                   $700 = HEAP32[$i>>2]|0;
                   $701 = HEAP32[$pTab>>2]|0;
                   $702 = ((($701)) + 4|0);
                   $703 = HEAP32[$702>>2]|0;
                   $704 = (($703) + (($700*24)|0)|0);
                   $705 = ((($704)) + 23|0);
                   $706 = HEAP8[$705>>0]|0;
                   $707 = $706&255;
                   $708 = $707 & 2;
                   $709 = ($708|0)!=(0);
                   if ($709) {
                    HEAP32[$j>>2] = -1;
                    $710 = HEAP32[$nHidden>>2]|0;
                    $711 = (($710) + 1)|0;
                    HEAP32[$nHidden>>2] = $711;
                    break;
                   } else {
                    $712 = HEAP32[$i>>2]|0;
                    $713 = HEAP32[$nHidden>>2]|0;
                    $714 = (($712) - ($713))|0;
                    HEAP32[$j>>2] = $714;
                    break;
                   }
                  } else {
                   HEAP32[$j>>2] = 0;
                   while(1) {
                    $715 = HEAP32[$j>>2]|0;
                    $716 = HEAP32[$3>>2]|0;
                    $717 = ((($716)) + 4|0);
                    $718 = HEAP32[$717>>2]|0;
                    $719 = ($715|0)<($718|0);
                    if (!($719)) {
                     break L170;
                    }
                    $720 = HEAP32[$j>>2]|0;
                    $721 = HEAP32[$3>>2]|0;
                    $722 = HEAP32[$721>>2]|0;
                    $723 = (($722) + ($720<<3)|0);
                    $724 = ((($723)) + 4|0);
                    $725 = HEAP32[$724>>2]|0;
                    $726 = HEAP32[$i>>2]|0;
                    $727 = ($725|0)==($726|0);
                    if ($727) {
                     break L170;
                    }
                    $728 = HEAP32[$j>>2]|0;
                    $729 = (($728) + 1)|0;
                    HEAP32[$j>>2] = $729;
                   }
                  }
                 } while(0);
                 $730 = HEAP32[$j>>2]|0;
                 $731 = ($730|0)<(0);
                 $732 = HEAP32[$nColumn>>2]|0;
                 $733 = ($732|0)==(0);
                 $or$cond9 = $731 | $733;
                 do {
                  if (!($or$cond9)) {
                   $734 = HEAP32[$3>>2]|0;
                   $735 = ($734|0)!=(0|0);
                   if ($735) {
                    $736 = HEAP32[$j>>2]|0;
                    $737 = HEAP32[$3>>2]|0;
                    $738 = ((($737)) + 4|0);
                    $739 = HEAP32[$738>>2]|0;
                    $740 = ($736|0)>=($739|0);
                    if ($740) {
                     break;
                    }
                   }
                   $750 = HEAP8[$useTempTable>>0]|0;
                   $751 = ($750<<24>>24)!=(0);
                   if ($751) {
                    $752 = HEAP32[$v>>2]|0;
                    $753 = HEAP32[$srcTab>>2]|0;
                    $754 = HEAP32[$j>>2]|0;
                    $755 = HEAP32[$iRegStore>>2]|0;
                    (_sqlite3VdbeAddOp3($752,47,$753,$754,$755)|0);
                    break L167;
                   }
                   $756 = HEAP32[$2>>2]|0;
                   $757 = ($756|0)!=(0|0);
                   if (!($757)) {
                    $766 = HEAP32[$0>>2]|0;
                    $767 = HEAP32[$j>>2]|0;
                    $768 = HEAP32[$pList>>2]|0;
                    $769 = ((($768)) + 4|0);
                    $770 = HEAP32[$769>>2]|0;
                    $771 = (($770) + (($767*20)|0)|0);
                    $772 = HEAP32[$771>>2]|0;
                    $773 = HEAP32[$iRegStore>>2]|0;
                    _sqlite3ExprCode($766,$772,$773);
                    break L167;
                   }
                   $758 = HEAP32[$regFromSelect>>2]|0;
                   $759 = HEAP32[$regData>>2]|0;
                   $760 = ($758|0)!=($759|0);
                   if (!($760)) {
                    break L167;
                   }
                   $761 = HEAP32[$v>>2]|0;
                   $762 = HEAP32[$regFromSelect>>2]|0;
                   $763 = HEAP32[$j>>2]|0;
                   $764 = (($762) + ($763))|0;
                   $765 = HEAP32[$iRegStore>>2]|0;
                   (_sqlite3VdbeAddOp2($761,31,$764,$765)|0);
                   break L167;
                  }
                 } while(0);
                 $741 = HEAP32[$0>>2]|0;
                 $742 = HEAP32[$i>>2]|0;
                 $743 = HEAP32[$pTab>>2]|0;
                 $744 = ((($743)) + 4|0);
                 $745 = HEAP32[$744>>2]|0;
                 $746 = (($745) + (($742*24)|0)|0);
                 $747 = ((($746)) + 4|0);
                 $748 = HEAP32[$747>>2]|0;
                 $749 = HEAP32[$iRegStore>>2]|0;
                 _sqlite3ExprCodeFactorable($741,$748,$749);
                }
               } while(0);
               $774 = HEAP32[$i>>2]|0;
               $775 = (($774) + 1)|0;
               HEAP32[$i>>2] = $775;
              }
              $776 = HEAP32[$pTab>>2]|0;
              $777 = ((($776)) + 42|0);
              $778 = HEAP8[$777>>0]|0;
              $779 = $778&255;
              $780 = $779 & 16;
              $781 = ($780|0)!=(0);
              if ($781) {
               $782 = HEAP32[$db>>2]|0;
               $783 = HEAP32[$pTab>>2]|0;
               $784 = (_sqlite3GetVTable($782,$783)|0);
               HEAP32[$pVTab>>2] = $784;
               $785 = HEAP32[$0>>2]|0;
               $786 = HEAP32[$pTab>>2]|0;
               _sqlite3VtabMakeWritable($785,$786);
               $787 = HEAP32[$v>>2]|0;
               $788 = HEAP32[$pTab>>2]|0;
               $789 = ((($788)) + 34|0);
               $790 = HEAP16[$789>>1]|0;
               $791 = $790 << 16 >> 16;
               $792 = (($791) + 2)|0;
               $793 = HEAP32[$regIns>>2]|0;
               $794 = HEAP32[$pVTab>>2]|0;
               (_sqlite3VdbeAddOp4($787,12,1,$792,$793,$794,-10)|0);
               $795 = HEAP32[$v>>2]|0;
               $796 = HEAP32[$4>>2]|0;
               $797 = ($796|0)==(10);
               $798 = HEAP32[$4>>2]|0;
               $799 = $797 ? 2 : $798;
               $800 = $799&255;
               _sqlite3VdbeChangeP5($795,$800);
               $801 = HEAP32[$0>>2]|0;
               _sqlite3MayAbort($801);
               break;
              } else {
               $802 = HEAP32[$0>>2]|0;
               $803 = HEAP32[$pTab>>2]|0;
               $804 = HEAP32[$aRegIdx>>2]|0;
               $805 = HEAP32[$iDataCur>>2]|0;
               $806 = HEAP32[$iIdxCur>>2]|0;
               $807 = HEAP32[$regIns>>2]|0;
               $808 = HEAP32[$ipkColumn>>2]|0;
               $809 = ($808|0)>=(0);
               $810 = $809&1;
               $811 = $810&255;
               $812 = HEAP32[$4>>2]|0;
               $813 = $812&255;
               $814 = HEAP32[$endOfLoop>>2]|0;
               _sqlite3GenerateConstraintChecks($802,$803,$804,$805,$806,$807,0,$811,$813,$814,$isReplace,0);
               $815 = HEAP32[$0>>2]|0;
               $816 = HEAP32[$pTab>>2]|0;
               $817 = HEAP32[$regIns>>2]|0;
               _sqlite3FkCheck($815,$816,0,$817,0,0);
               $818 = HEAP32[$0>>2]|0;
               $819 = HEAP32[$pTab>>2]|0;
               $820 = HEAP32[$iDataCur>>2]|0;
               $821 = HEAP32[$iIdxCur>>2]|0;
               $822 = HEAP32[$regIns>>2]|0;
               $823 = HEAP32[$aRegIdx>>2]|0;
               $824 = HEAP8[$appendFlag>>0]|0;
               $825 = $824&255;
               $826 = HEAP32[$isReplace>>2]|0;
               $827 = ($826|0)==(0);
               $828 = $827&1;
               _sqlite3CompleteInsertion($818,$819,$820,$821,$822,$823,0,$825,$828);
               break;
              }
             }
            } while(0);
            $829 = HEAP32[$db>>2]|0;
            $830 = ((($829)) + 24|0);
            $831 = HEAP32[$830>>2]|0;
            $832 = $831 & 128;
            $833 = ($832|0)!=(0);
            if ($833) {
             $834 = HEAP32[$v>>2]|0;
             $835 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($834,37,$835,1)|0);
            }
            $836 = HEAP32[$pTrigger>>2]|0;
            $837 = ($836|0)!=(0|0);
            if ($837) {
             $838 = HEAP32[$0>>2]|0;
             $839 = HEAP32[$pTrigger>>2]|0;
             $840 = HEAP32[$pTab>>2]|0;
             $841 = HEAP32[$regData>>2]|0;
             $842 = (($841) - 2)|0;
             $843 = HEAP32[$pTab>>2]|0;
             $844 = ((($843)) + 34|0);
             $845 = HEAP16[$844>>1]|0;
             $846 = $845 << 16 >> 16;
             $847 = (($842) - ($846))|0;
             $848 = HEAP32[$4>>2]|0;
             $849 = HEAP32[$endOfLoop>>2]|0;
             _sqlite3CodeRowTrigger($838,$839,108,0,2,$840,$847,$848,$849);
            }
            $850 = HEAP32[$v>>2]|0;
            $851 = HEAP32[$endOfLoop>>2]|0;
            _sqlite3VdbeResolveLabel($850,$851);
            $852 = HEAP8[$useTempTable>>0]|0;
            $853 = ($852<<24>>24)!=(0);
            do {
             if ($853) {
              $854 = HEAP32[$v>>2]|0;
              $855 = HEAP32[$srcTab>>2]|0;
              $856 = HEAP32[$addrCont>>2]|0;
              (_sqlite3VdbeAddOp2($854,7,$855,$856)|0);
              $857 = HEAP32[$v>>2]|0;
              $858 = HEAP32[$addrInsTop>>2]|0;
              _sqlite3VdbeJumpHere($857,$858);
              $859 = HEAP32[$v>>2]|0;
              $860 = HEAP32[$srcTab>>2]|0;
              (_sqlite3VdbeAddOp1($859,61,$860)|0);
             } else {
              $861 = HEAP32[$2>>2]|0;
              $862 = ($861|0)!=(0|0);
              if (!($862)) {
               break;
              }
              $863 = HEAP32[$v>>2]|0;
              $864 = HEAP32[$addrCont>>2]|0;
              (_sqlite3VdbeGoto($863,$864)|0);
              $865 = HEAP32[$v>>2]|0;
              $866 = HEAP32[$addrInsTop>>2]|0;
              _sqlite3VdbeJumpHere($865,$866);
             }
            } while(0);
            $867 = HEAP32[$pTab>>2]|0;
            $868 = ((($867)) + 42|0);
            $869 = HEAP8[$868>>0]|0;
            $870 = $869&255;
            $871 = $870 & 16;
            $872 = ($871|0)!=(0);
            $873 = HEAP32[$isView>>2]|0;
            $874 = ($873|0)!=(0);
            $or$cond11 = $872 | $874;
            if ($or$cond11) {
             break;
            }
            $875 = HEAP32[$iDataCur>>2]|0;
            $876 = HEAP32[$iIdxCur>>2]|0;
            $877 = ($875|0)<($876|0);
            if ($877) {
             $878 = HEAP32[$v>>2]|0;
             $879 = HEAP32[$iDataCur>>2]|0;
             (_sqlite3VdbeAddOp1($878,61,$879)|0);
            }
            HEAP32[$idx>>2] = 0;
            $880 = HEAP32[$pTab>>2]|0;
            $881 = ((($880)) + 8|0);
            $882 = HEAP32[$881>>2]|0;
            HEAP32[$pIdx>>2] = $882;
            while(1) {
             $883 = HEAP32[$pIdx>>2]|0;
             $884 = ($883|0)!=(0|0);
             if (!($884)) {
              break L20;
             }
             $885 = HEAP32[$v>>2]|0;
             $886 = HEAP32[$idx>>2]|0;
             $887 = HEAP32[$iIdxCur>>2]|0;
             $888 = (($886) + ($887))|0;
             (_sqlite3VdbeAddOp1($885,61,$888)|0);
             $889 = HEAP32[$pIdx>>2]|0;
             $890 = ((($889)) + 20|0);
             $891 = HEAP32[$890>>2]|0;
             HEAP32[$pIdx>>2] = $891;
             $892 = HEAP32[$idx>>2]|0;
             $893 = (($892) + 1)|0;
             HEAP32[$idx>>2] = $893;
            }
           }
          } while(0);
          $894 = HEAP32[$0>>2]|0;
          $895 = ((($894)) + 18|0);
          $896 = HEAP8[$895>>0]|0;
          $897 = $896&255;
          $898 = ($897|0)==(0);
          if ($898) {
           $899 = HEAP32[$0>>2]|0;
           $900 = ((($899)) + 420|0);
           $901 = HEAP32[$900>>2]|0;
           $902 = ($901|0)==(0|0);
           if ($902) {
            $903 = HEAP32[$0>>2]|0;
            _sqlite3AutoincrementEnd($903);
           }
          }
          $904 = HEAP32[$db>>2]|0;
          $905 = ((($904)) + 24|0);
          $906 = HEAP32[$905>>2]|0;
          $907 = $906 & 128;
          $908 = ($907|0)!=(0);
          if ($908) {
           $909 = HEAP32[$0>>2]|0;
           $910 = ((($909)) + 18|0);
           $911 = HEAP8[$910>>0]|0;
           $912 = ($911<<24>>24)!=(0);
           if (!($912)) {
            $913 = HEAP32[$0>>2]|0;
            $914 = ((($913)) + 420|0);
            $915 = HEAP32[$914>>2]|0;
            $916 = ($915|0)!=(0|0);
            if (!($916)) {
             $917 = HEAP32[$v>>2]|0;
             $918 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($917,33,$918,1)|0);
             $919 = HEAP32[$v>>2]|0;
             _sqlite3VdbeSetNumCols($919,1);
             $920 = HEAP32[$v>>2]|0;
             (_sqlite3VdbeSetColName($920,0,0,33729,0)|0);
            }
           }
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $921 = HEAP32[$db>>2]|0;
 $922 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($921,$922);
 $923 = HEAP32[$db>>2]|0;
 $924 = HEAP32[$pList>>2]|0;
 _sqlite3ExprListDelete($923,$924);
 $925 = HEAP32[$db>>2]|0;
 $926 = HEAP32[$2>>2]|0;
 _sqlite3SelectDelete($925,$926);
 $927 = HEAP32[$db>>2]|0;
 $928 = HEAP32[$3>>2]|0;
 _sqlite3IdListDelete($927,$928);
 $929 = HEAP32[$db>>2]|0;
 $930 = HEAP32[$aRegIdx>>2]|0;
 _sqlite3DbFree($929,$930);
 STACKTOP = sp;return;
}
function _sqlite3IdListAppend($db,$pList,$pToken) {
 $db = $db|0;
 $pList = $pList|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pToken;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (_sqlite3DbMallocZero($6,8,0)|0);
  HEAP32[$2>>2] = $7;
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   HEAP32[$0>>2] = 0;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = (_sqlite3ArrayAllocate($10,$12,8,$14,$i)|0);
 $16 = HEAP32[$2>>2]|0;
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$i>>2]|0;
 $18 = ($17|0)<(0);
 $19 = HEAP32[$1>>2]|0;
 if ($18) {
  $20 = HEAP32[$2>>2]|0;
  _sqlite3IdListDelete($19,$20);
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $21 = HEAP32[$3>>2]|0;
  $22 = (_sqlite3NameFromToken($19,$21)|0);
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($23<<3)|0);
  HEAP32[$26>>2] = $22;
  $27 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _spanSet($pOut,$pStart,$pEnd) {
 $pOut = $pOut|0;
 $pStart = $pStart|0;
 $pEnd = $pEnd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pStart;
 HEAP32[$2>>2] = $pEnd;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($9)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _sqlite3ExprAssignVarNumber($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $bOk = 0, $db = 0, $i = 0, $i1 = 0, $n = 0, $or$cond = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $x = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $db = sp + 36|0;
 $z = sp + 32|0;
 $x = sp + 50|0;
 $n = sp + 28|0;
 $i = sp;
 $bOk = sp + 24|0;
 $i1 = sp + 48|0;
 $a = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$z>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = ((($9)) + 1|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = ($12|0)==(0);
 do {
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 444|0);
   $16 = HEAP16[$15>>1]|0;
   $17 = (($16) + 1)<<16>>16;
   HEAP16[$15>>1] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 32|0);
   HEAP16[$19>>1] = $17;
  } else {
   HEAP16[$x>>1] = 0;
   $20 = HEAP32[$z>>2]|0;
   $21 = (_sqlite3Strlen30($20)|0);
   HEAP32[$n>>2] = $21;
   $22 = HEAP32[$z>>2]|0;
   $23 = HEAP8[$22>>0]|0;
   $24 = $23 << 24 >> 24;
   $25 = ($24|0)==(63);
   if ($25) {
    $26 = HEAP32[$z>>2]|0;
    $27 = ((($26)) + 1|0);
    $28 = HEAP32[$n>>2]|0;
    $29 = (($28) - 1)|0;
    $30 = (_sqlite3Atoi64($27,$i,$29,1)|0);
    $31 = (0)==($30|0);
    $32 = $31&1;
    HEAP32[$bOk>>2] = $32;
    $33 = $i;
    $34 = $33;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($33) + 4)|0;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = $35&65535;
    HEAP16[$x>>1] = $39;
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 32|0);
    HEAP16[$41>>1] = $39;
    $42 = HEAP32[$bOk>>2]|0;
    $43 = ($42|0)==(0);
    $44 = $i;
    $45 = $44;
    $46 = HEAP32[$45>>2]|0;
    $47 = (($44) + 4)|0;
    $48 = $47;
    $49 = HEAP32[$48>>2]|0;
    $50 = ($49|0)<(0);
    $51 = ($46>>>0)<(1);
    $52 = ($49|0)==(0);
    $53 = $52 & $51;
    $54 = $50 | $53;
    $or$cond = $43 | $54;
    if ($or$cond) {
     label = 7;
    } else {
     $55 = $i;
     $56 = $55;
     $57 = HEAP32[$56>>2]|0;
     $58 = (($55) + 4)|0;
     $59 = $58;
     $60 = HEAP32[$59>>2]|0;
     $61 = HEAP32[$db>>2]|0;
     $62 = ((($61)) + 88|0);
     $63 = ((($62)) + 36|0);
     $64 = HEAP32[$63>>2]|0;
     $65 = ($64|0)<(0);
     $66 = $65 << 31 >> 31;
     $67 = ($60|0)>($66|0);
     $68 = ($57>>>0)>($64>>>0);
     $69 = ($60|0)==($66|0);
     $70 = $69 & $68;
     $71 = $67 | $70;
     if ($71) {
      label = 7;
     }
    }
    if ((label|0) == 7) {
     $72 = HEAP32[$0>>2]|0;
     $73 = HEAP32[$db>>2]|0;
     $74 = ((($73)) + 88|0);
     $75 = ((($74)) + 36|0);
     $76 = HEAP32[$75>>2]|0;
     HEAP32[$vararg_buffer>>2] = $76;
     _sqlite3ErrorMsg($72,33743,$vararg_buffer);
     HEAP16[$x>>1] = 0;
    }
    $77 = $i;
    $78 = $77;
    $79 = HEAP32[$78>>2]|0;
    $80 = (($77) + 4)|0;
    $81 = $80;
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$0>>2]|0;
    $84 = ((($83)) + 444|0);
    $85 = HEAP16[$84>>1]|0;
    $86 = $85 << 16 >> 16;
    $87 = ($86|0)<(0);
    $88 = $87 << 31 >> 31;
    $89 = ($82|0)>($88|0);
    $90 = ($79>>>0)>($86>>>0);
    $91 = ($82|0)==($88|0);
    $92 = $91 & $90;
    $93 = $89 | $92;
    if ($93) {
     $94 = $i;
     $95 = $94;
     $96 = HEAP32[$95>>2]|0;
     $97 = (($94) + 4)|0;
     $98 = $97;
     $99 = HEAP32[$98>>2]|0;
     $100 = $96&65535;
     $101 = HEAP32[$0>>2]|0;
     $102 = ((($101)) + 444|0);
     HEAP16[$102>>1] = $100;
    }
   } else {
    HEAP16[$i1>>1] = 0;
    while(1) {
     $103 = HEAP16[$i1>>1]|0;
     $104 = $103 << 16 >> 16;
     $105 = HEAP32[$0>>2]|0;
     $106 = ((($105)) + 448|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = ($104|0)<($107|0);
     if (!($108)) {
      break;
     }
     $109 = HEAP16[$i1>>1]|0;
     $110 = $109 << 16 >> 16;
     $111 = HEAP32[$0>>2]|0;
     $112 = ((($111)) + 476|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = (($113) + ($110<<2)|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = ($115|0)!=(0|0);
     if ($116) {
      $117 = HEAP16[$i1>>1]|0;
      $118 = $117 << 16 >> 16;
      $119 = HEAP32[$0>>2]|0;
      $120 = ((($119)) + 476|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = (($121) + ($118<<2)|0);
      $123 = HEAP32[$122>>2]|0;
      $124 = HEAP32[$z>>2]|0;
      $125 = (_strcmp($123,$124)|0);
      $126 = ($125|0)==(0);
      if ($126) {
       label = 14;
       break;
      }
     }
     $133 = HEAP16[$i1>>1]|0;
     $134 = (($133) + 1)<<16>>16;
     HEAP16[$i1>>1] = $134;
    }
    if ((label|0) == 14) {
     $127 = HEAP16[$i1>>1]|0;
     $128 = $127 << 16 >> 16;
     $129 = (($128) + 1)|0;
     $130 = $129&65535;
     HEAP16[$x>>1] = $130;
     $131 = HEAP32[$1>>2]|0;
     $132 = ((($131)) + 32|0);
     HEAP16[$132>>1] = $130;
    }
    $135 = HEAP16[$x>>1]|0;
    $136 = $135 << 16 >> 16;
    $137 = ($136|0)==(0);
    if ($137) {
     $138 = HEAP32[$0>>2]|0;
     $139 = ((($138)) + 444|0);
     $140 = HEAP16[$139>>1]|0;
     $141 = (($140) + 1)<<16>>16;
     HEAP16[$139>>1] = $141;
     $142 = HEAP32[$1>>2]|0;
     $143 = ((($142)) + 32|0);
     HEAP16[$143>>1] = $141;
     HEAP16[$x>>1] = $141;
    }
   }
   $144 = HEAP16[$x>>1]|0;
   $145 = $144 << 16 >> 16;
   $146 = ($145|0)>(0);
   if ($146) {
    $147 = HEAP16[$x>>1]|0;
    $148 = $147 << 16 >> 16;
    $149 = HEAP32[$0>>2]|0;
    $150 = ((($149)) + 448|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = ($148|0)>($151|0);
    do {
     if ($152) {
      $153 = HEAP32[$db>>2]|0;
      $154 = HEAP32[$0>>2]|0;
      $155 = ((($154)) + 476|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = HEAP16[$x>>1]|0;
      $158 = $157 << 16 >> 16;
      $159 = $158<<2;
      $160 = (_sqlite3DbRealloc($153,$156,$159,0)|0);
      HEAP32[$a>>2] = $160;
      $161 = HEAP32[$a>>2]|0;
      $162 = ($161|0)==(0|0);
      if ($162) {
       STACKTOP = sp;return;
      } else {
       $163 = HEAP32[$a>>2]|0;
       $164 = HEAP32[$0>>2]|0;
       $165 = ((($164)) + 476|0);
       HEAP32[$165>>2] = $163;
       $166 = HEAP32[$0>>2]|0;
       $167 = ((($166)) + 448|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = HEAP32[$a>>2]|0;
       $170 = (($169) + ($168<<2)|0);
       $171 = HEAP16[$x>>1]|0;
       $172 = $171 << 16 >> 16;
       $173 = HEAP32[$0>>2]|0;
       $174 = ((($173)) + 448|0);
       $175 = HEAP32[$174>>2]|0;
       $176 = (($172) - ($175))|0;
       $177 = $176<<2;
       _memset(($170|0),0,($177|0))|0;
       $178 = HEAP16[$x>>1]|0;
       $179 = $178 << 16 >> 16;
       $180 = HEAP32[$0>>2]|0;
       $181 = ((($180)) + 448|0);
       HEAP32[$181>>2] = $179;
       break;
      }
     }
    } while(0);
    $182 = HEAP32[$z>>2]|0;
    $183 = HEAP8[$182>>0]|0;
    $184 = $183 << 24 >> 24;
    $185 = ($184|0)!=(63);
    if (!($185)) {
     $186 = HEAP16[$x>>1]|0;
     $187 = $186 << 16 >> 16;
     $188 = (($187) - 1)|0;
     $189 = HEAP32[$0>>2]|0;
     $190 = ((($189)) + 476|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (($191) + ($188<<2)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($193|0)==(0|0);
     if (!($194)) {
      break;
     }
    }
    $195 = HEAP32[$db>>2]|0;
    $196 = HEAP16[$x>>1]|0;
    $197 = $196 << 16 >> 16;
    $198 = (($197) - 1)|0;
    $199 = HEAP32[$0>>2]|0;
    $200 = ((($199)) + 476|0);
    $201 = HEAP32[$200>>2]|0;
    $202 = (($201) + ($198<<2)|0);
    $203 = HEAP32[$202>>2]|0;
    _sqlite3DbFree($195,$203);
    $204 = HEAP32[$db>>2]|0;
    $205 = HEAP32[$z>>2]|0;
    $206 = HEAP32[$n>>2]|0;
    $207 = (_sqlite3DbStrNDup($204,$205,$206,0)|0);
    $208 = HEAP16[$x>>1]|0;
    $209 = $208 << 16 >> 16;
    $210 = (($209) - 1)|0;
    $211 = HEAP32[$0>>2]|0;
    $212 = ((($211)) + 476|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = (($213) + ($210<<2)|0);
    HEAP32[$214>>2] = $207;
   }
  }
 } while(0);
 $215 = HEAP32[$0>>2]|0;
 $216 = ((($215)) + 68|0);
 $217 = HEAP32[$216>>2]|0;
 $218 = ($217|0)!=(0);
 if ($218) {
  STACKTOP = sp;return;
 }
 $219 = HEAP32[$0>>2]|0;
 $220 = ((($219)) + 444|0);
 $221 = HEAP16[$220>>1]|0;
 $222 = $221 << 16 >> 16;
 $223 = HEAP32[$db>>2]|0;
 $224 = ((($223)) + 88|0);
 $225 = ((($224)) + 36|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = ($222|0)>($226|0);
 if (!($227)) {
  STACKTOP = sp;return;
 }
 $228 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($228,33786,$vararg_buffer2);
 STACKTOP = sp;return;
}
function _sqlite3ExprAddCollateToken($pParse,$pExpr,$pCollName,$dequote) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pCollName = $pCollName|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pCollName;
 HEAP32[$3>>2] = $dequote;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6>>>0)>(0);
 if (!($7)) {
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (_sqlite3ExprAlloc($9,95,$10,$11)|0);
 HEAP32[$pNew>>2] = $12;
 $13 = HEAP32[$pNew>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = ((($16)) + 12|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 | 4352;
 HEAP32[$19>>2] = $21;
 $22 = HEAP32[$pNew>>2]|0;
 HEAP32[$1>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3ExprFunction($pParse,$pList,$pToken) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pNew = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pToken;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3ExprAlloc($6,151,$7,1)|0);
 HEAP32[$pNew>>2] = $8;
 $9 = HEAP32[$pNew>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($11,$12);
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$pNew>>2]|0;
  $15 = ((($14)) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pNew>>2]|0;
  _sqlite3ExprSetHeightAndFlags($16,$17);
  $18 = HEAP32[$pNew>>2]|0;
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _spanBinaryExpr($pOut,$pParse,$op,$pLeft,$pRight) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pLeft;
 HEAP32[$4>>2] = $pRight;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3PExpr($5,$6,$8,$10,0)|0);
 $12 = HEAP32[$0>>2]|0;
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 4|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$4>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 8|0);
 HEAP32[$22>>2] = $20;
 STACKTOP = sp;return;
}
function _exprNot($pParse,$doNot,$ppExpr) {
 $pParse = $pParse|0;
 $doNot = $doNot|0;
 $ppExpr = $ppExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $doNot;
 HEAP32[$2>>2] = $ppExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PExpr($5,19,$7,0,0)|0);
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 STACKTOP = sp;return;
}
function _spanUnaryPostfix($pOut,$pParse,$op,$pOperand,$pPostOp) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pOperand = $pOperand|0;
 $pPostOp = $pPostOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pOperand;
 HEAP32[$4>>2] = $pPostOp;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PExpr($5,$6,$8,0,0)|0);
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$4>>2]|0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18)|0);
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 8|0);
 HEAP32[$23>>2] = $21;
 STACKTOP = sp;return;
}
function _binaryToUnaryIfNull($pParse,$pY,$pA,$op) {
 $pParse = $pParse|0;
 $pY = $pY|0;
 $pA = $pA|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pY;
 HEAP32[$2>>2] = $pA;
 HEAP32[$3>>2] = $op;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 $or$cond = $7 & $9;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(101);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = $14&255;
 $16 = HEAP32[$2>>2]|0;
 HEAP8[$16>>0] = $15;
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3ExprDelete($17,$20);
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 16|0);
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _spanUnaryPrefix($pOut,$pParse,$op,$pOperand,$pPreOp) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pOperand = $pOperand|0;
 $pPreOp = $pPreOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pOperand;
 HEAP32[$4>>2] = $pPreOp;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PExpr($5,$6,$8,0,0)|0);
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$4>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = $17;
 STACKTOP = sp;return;
}
function _sqlite3ExprSetHeightAndFlags($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 68|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 _exprSetHeight($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 (_sqlite3ExprCheckHeight($7,$10)|0);
 STACKTOP = sp;return;
}
function _parserAddExprIdListTerm($pParse,$pPrior,$pIdToken,$hasCollate,$sortOrder) {
 $pParse = $pParse|0;
 $pPrior = $pPrior|0;
 $pIdToken = $pIdToken|0;
 $hasCollate = $hasCollate|0;
 $sortOrder = $sortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pPrior;
 HEAP32[$2>>2] = $pIdToken;
 HEAP32[$3>>2] = $hasCollate;
 HEAP32[$4>>2] = $sortOrder;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3ExprListAppend($5,$6,0)|0);
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)!=(-1);
 $or$cond = $9 | $11;
 if ($or$cond) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ((($13)) + 140|0);
  $15 = ((($14)) + 5|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$2>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$vararg_buffer>>2] = $22;
   $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr2>>2] = $24;
   _sqlite3ErrorMsg($19,33813,$vararg_buffer);
  }
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$p>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 _sqlite3ExprListSetName($25,$26,$27,1);
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3DropIndex($pParse,$pName,$ifExists) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $ifExists = $ifExists|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $pIndex = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0;
 var $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $pIndex = sp + 56|0;
 $v = sp + 52|0;
 $db = sp + 48|0;
 $iDb = sp + 44|0;
 $code = sp + 40|0;
 $pTab = sp + 36|0;
 $zDb = sp + 32|0;
 $zTab = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $ifExists;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 65|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3ReadSchema($9)|0);
 $11 = (0)!=($10|0);
 if ($11) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3FindIndex($12,$16,$20)|0);
 HEAP32[$pIndex>>2] = $21;
 $22 = HEAP32[$pIndex>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$2>>2]|0;
  $25 = ($24|0)!=(0);
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  if ($25) {
   $28 = ((($27)) + 8|0);
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   _sqlite3CodeVerifyNamedSchema($26,$30);
  } else {
   HEAP32[$vararg_buffer>>2] = $27;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 0;
   _sqlite3ErrorMsg($26,33851,$vararg_buffer);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = ((($31)) + 17|0);
  HEAP8[$32>>0] = 1;
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$pIndex>>2]|0;
 $34 = ((($33)) + 55|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35 & 3;
 $37 = $36&255;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  HEAP32[$vararg_buffer2>>2] = 0;
  _sqlite3ErrorMsg($39,33869,$vararg_buffer2);
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$db>>2]|0;
 $41 = HEAP32[$pIndex>>2]|0;
 $42 = ((($41)) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3SchemaToIndex($40,$43)|0);
 HEAP32[$iDb>>2] = $44;
 HEAP32[$code>>2] = 10;
 $45 = HEAP32[$pIndex>>2]|0;
 $46 = ((($45)) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pTab>>2] = $47;
 $48 = HEAP32[$iDb>>2]|0;
 $49 = HEAP32[$db>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48<<4)|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$zDb>>2] = $53;
 $54 = HEAP32[$iDb>>2]|0;
 $55 = ($54|0)==(1);
 $56 = $55 ? 21260 : 21279;
 HEAP32[$zTab>>2] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$zTab>>2]|0;
 $59 = HEAP32[$zDb>>2]|0;
 $60 = (_sqlite3AuthCheck($57,9,$58,0,$59)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $62 = HEAP32[$iDb>>2]|0;
 $63 = ($62|0)!=(0);
 if ($63) {
  HEAP32[$code>>2] = 12;
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$code>>2]|0;
 $66 = HEAP32[$pIndex>>2]|0;
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$pTab>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$zDb>>2]|0;
 $71 = (_sqlite3AuthCheck($64,$65,$67,$69,$70)|0);
 $72 = ($71|0)!=(0);
 if ($72) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$0>>2]|0;
 $74 = (_sqlite3GetVdbe($73)|0);
 HEAP32[$v>>2] = $74;
 $75 = HEAP32[$v>>2]|0;
 $76 = ($75|0)!=(0|0);
 if (!($76)) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $77 = HEAP32[$0>>2]|0;
 $78 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($77,1,$78);
 $79 = HEAP32[$0>>2]|0;
 $80 = HEAP32[$iDb>>2]|0;
 $81 = HEAP32[$db>>2]|0;
 $82 = ((($81)) + 16|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (($83) + ($80<<4)|0);
 $85 = HEAP32[$84>>2]|0;
 $86 = HEAP32[$iDb>>2]|0;
 $87 = ($86|0)==(1);
 $88 = $87 ? 21260 : 21279;
 $89 = HEAP32[$pIndex>>2]|0;
 $90 = HEAP32[$89>>2]|0;
 HEAP32[$vararg_buffer5>>2] = $85;
 $vararg_ptr8 = ((($vararg_buffer5)) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $88;
 $vararg_ptr9 = ((($vararg_buffer5)) + 8|0);
 HEAP32[$vararg_ptr9>>2] = $90;
 _sqlite3NestedParse($79,33942,$vararg_buffer5);
 $91 = HEAP32[$0>>2]|0;
 $92 = HEAP32[$iDb>>2]|0;
 $93 = HEAP32[$pIndex>>2]|0;
 $94 = HEAP32[$93>>2]|0;
 _sqlite3ClearStatTables($91,$92,33991,$94);
 $95 = HEAP32[$0>>2]|0;
 $96 = HEAP32[$iDb>>2]|0;
 _sqlite3ChangeCookie($95,$96);
 $97 = HEAP32[$0>>2]|0;
 $98 = HEAP32[$pIndex>>2]|0;
 $99 = ((($98)) + 44|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = HEAP32[$iDb>>2]|0;
 _destroyRootPage($97,$100,$101);
 $102 = HEAP32[$v>>2]|0;
 $103 = HEAP32[$iDb>>2]|0;
 $104 = HEAP32[$pIndex>>2]|0;
 $105 = HEAP32[$104>>2]|0;
 (_sqlite3VdbeAddOp4($102,126,$103,0,0,$105,0)|0);
 $106 = HEAP32[$db>>2]|0;
 $107 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($106,$107);
 STACKTOP = sp;return;
}
function _sqlite3Vacuum($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$v>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($5,10,0,0)|0);
 $6 = HEAP32[$v>>2]|0;
 _sqlite3VdbeUsesBtree($6,0);
 STACKTOP = sp;return;
}
function _sqlite3Pragma($pParse,$pId1,$pId2,$pValue,$minusFlag) {
 $pParse = $pParse|0;
 $pId1 = $pId1|0;
 $pId2 = $pId2|0;
 $pValue = $pValue|0;
 $minusFlag = $minusFlag|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0;
 var $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0;
 var $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0;
 var $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0;
 var $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0;
 var $1087 = 0, $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0;
 var $1104 = 0, $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0;
 var $1122 = 0, $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0;
 var $1140 = 0, $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0;
 var $1159 = 0, $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0;
 var $1177 = 0, $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0;
 var $1195 = 0, $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0;
 var $1212 = 0, $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0;
 var $1230 = 0, $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0;
 var $1249 = 0, $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0;
 var $1267 = 0, $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0;
 var $1285 = 0, $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0;
 var $1302 = 0, $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0;
 var $1320 = 0, $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0;
 var $1339 = 0, $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0;
 var $1357 = 0, $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0;
 var $1375 = 0, $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0;
 var $1393 = 0, $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0;
 var $1410 = 0, $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0;
 var $1429 = 0, $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0;
 var $1447 = 0, $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0;
 var $1465 = 0, $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0;
 var $1483 = 0, $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0;
 var $1500 = 0, $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0;
 var $1519 = 0, $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0;
 var $1537 = 0, $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0;
 var $1555 = 0, $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0;
 var $1573 = 0, $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0;
 var $1591 = 0, $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0;
 var $1609 = 0, $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0;
 var $1627 = 0, $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0;
 var $1645 = 0, $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0;
 var $1663 = 0, $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0;
 var $1681 = 0, $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0;
 var $17 = 0, $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0;
 var $1717 = 0, $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0;
 var $1735 = 0, $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0;
 var $1753 = 0, $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0;
 var $1771 = 0, $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0;
 var $179 = 0, $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0;
 var $1807 = 0, $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0;
 var $1825 = 0, $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0;
 var $1843 = 0, $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0;
 var $1861 = 0, $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0;
 var $188 = 0, $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0;
 var $1898 = 0, $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0;
 var $1915 = 0, $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0;
 var $1933 = 0, $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0;
 var $1951 = 0, $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0;
 var $197 = 0, $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0;
 var $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0;
 var $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0;
 var $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0;
 var $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0;
 var $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0;
 var $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0;
 var $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0;
 var $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0;
 var $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0;
 var $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0;
 var $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0;
 var $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0;
 var $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0;
 var $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0;
 var $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0;
 var $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0;
 var $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0;
 var $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0;
 var $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0;
 var $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0;
 var $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0;
 var $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0;
 var $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0;
 var $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0;
 var $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0;
 var $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0;
 var $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0;
 var $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0;
 var $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0;
 var $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0;
 var $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0;
 var $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0;
 var $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0;
 var $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0;
 var $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0;
 var $998 = 0, $999 = 0, $N = 0, $N46 = 0, $aFcntl = 0, $aOp = 0, $aOp41 = 0, $aOp42 = 0, $aOp43 = 0, $aOp8 = 0, $addr = 0, $addr32 = 0, $addrOk = 0, $addrTop = 0, $aiCols = 0, $azOrigin = 0, $b = 0, $ckUniq = 0, $cnt = 0, $cnum = 0;
 var $db = 0, $eAuto = 0, $eMode = 0, $eMode4 = 0, $eMode45 = 0, $i = 0, $i12 = 0, $i16 = 0, $i19 = 0, $i20 = 0, $i21 = 0, $i23 = 0, $i27 = 0, $i30 = 0, $i44 = 0, $iAddr = 0, $iBt = 0, $iCol = 0, $iCookie = 0, $iDataCur = 0;
 var $iDb = 0, $iIdxCur = 0, $iKey = 0, $iLevel = 0, $iLimit = 0, $iLimit9 = 0, $iReg = 0, $ii = 0, $ii3 = 0, $ii5 = 0, $isQuick = 0, $j = 0, $j28 = 0, $j31 = 0, $jmp2 = 0, $jmp239 = 0, $jmp3 = 0, $jmp340 = 0, $jmp4 = 0, $jmp5 = 0;
 var $jmp6 = 0, $k = 0, $k29 = 0, $kk = 0, $loopTop = 0, $lwr = 0, $mask = 0, $mid = 0, $mx = 0, $mxErr = 0, $n = 0, $nHidden = 0, $or$cond = 0, $or$cond11 = 0, $or$cond14 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $p = 0;
 var $pBt = 0, $pBt2 = 0, $pBt7 = 0, $pCol = 0, $pColl = 0, $pDb = 0, $pEnc = 0, $pFK = 0, $pFK24 = 0, $pId = 0, $pIdx = 0, $pIdx14 = 0, $pIdx17 = 0, $pIdx26 = 0, $pIdx35 = 0, $pIdx37 = 0, $pPager = 0, $pPager6 = 0, $pParent = 0, $pPk = 0;
 var $pPk38 = 0, $pPragma = 0, $pPrior = 0, $pTab = 0, $pTab13 = 0, $pTab15 = 0, $pTab18 = 0, $pTab22 = 0, $pTab25 = 0, $pTab34 = 0, $pTab36 = 0, $pTbls = 0, $r1 = 0, $rc = 0, $regKey = 0, $regResult = 0, $regRow = 0, $res = 0, $size = 0, $size1 = 0;
 var $size10 = 0, $size11 = 0, $sz = 0, $uniqOk = 0, $upr = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer15 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer23 = 0, $vararg_buffer25 = 0, $vararg_buffer33 = 0, $vararg_buffer39 = 0, $vararg_buffer44 = 0, $vararg_buffer49 = 0, $vararg_buffer54 = 0, $vararg_buffer61 = 0, $vararg_buffer66 = 0, $vararg_buffer70 = 0;
 var $vararg_buffer80 = 0, $vararg_buffer84 = 0, $vararg_buffer87 = 0, $vararg_buffer91 = 0, $vararg_ptr28 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr37 = 0, $vararg_ptr38 = 0, $vararg_ptr42 = 0, $vararg_ptr43 = 0, $vararg_ptr47 = 0, $vararg_ptr48 = 0, $vararg_ptr52 = 0, $vararg_ptr53 = 0, $vararg_ptr57 = 0, $vararg_ptr58 = 0;
 var $vararg_ptr59 = 0, $vararg_ptr60 = 0, $vararg_ptr64 = 0, $vararg_ptr65 = 0, $vararg_ptr69 = 0, $vararg_ptr73 = 0, $vararg_ptr74 = 0, $vararg_ptr75 = 0, $vararg_ptr76 = 0, $vararg_ptr77 = 0, $vararg_ptr78 = 0, $vararg_ptr79 = 0, $vararg_ptr83 = 0, $vararg_ptr90 = 0, $x = 0, $x33 = 0, $zDb = 0, $zErr = 0, $zLeft = 0, $zMode = 0;
 var $zOpt = 0, $zRet = 0, $zRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 800|0;
 $vararg_buffer91 = sp + 264|0;
 $vararg_buffer87 = sp + 256|0;
 $vararg_buffer84 = sp + 248|0;
 $vararg_buffer80 = sp + 240|0;
 $vararg_buffer70 = sp + 208|0;
 $vararg_buffer66 = sp + 200|0;
 $vararg_buffer61 = sp + 184|0;
 $vararg_buffer54 = sp + 160|0;
 $vararg_buffer49 = sp + 144|0;
 $vararg_buffer44 = sp + 128|0;
 $vararg_buffer39 = sp + 112|0;
 $vararg_buffer33 = sp + 96|0;
 $vararg_buffer25 = sp + 72|0;
 $vararg_buffer23 = sp + 64|0;
 $vararg_buffer20 = sp + 56|0;
 $vararg_buffer18 = sp + 48|0;
 $vararg_buffer15 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $0 = sp + 792|0;
 $1 = sp + 788|0;
 $2 = sp + 784|0;
 $3 = sp + 780|0;
 $4 = sp + 776|0;
 $zLeft = sp + 772|0;
 $zRight = sp + 768|0;
 $zDb = sp + 764|0;
 $pId = sp + 760|0;
 $aFcntl = sp + 744|0;
 $iDb = sp + 736|0;
 $lwr = sp + 732|0;
 $upr = sp + 728|0;
 $mid = sp + 724|0;
 $rc = sp + 720|0;
 $db = sp + 716|0;
 $pDb = sp + 712|0;
 $v = sp + 708|0;
 $pPragma = sp + 704|0;
 $aOp = sp + 700|0;
 $size = sp + 696|0;
 $pBt = sp + 692|0;
 $size1 = sp + 688|0;
 $pBt2 = sp + 684|0;
 $b = sp + 680|0;
 $ii = sp + 676|0;
 $iReg = sp + 672|0;
 $zRet = sp + 668|0;
 $eMode = sp + 664|0;
 $pPager = sp + 660|0;
 $ii3 = sp + 656|0;
 $eMode4 = sp + 652|0;
 $ii5 = sp + 648|0;
 $zMode = sp + 644|0;
 $n = sp + 640|0;
 $pPager6 = sp + 636|0;
 $iLimit = sp + 24|0;
 $pBt7 = sp + 632|0;
 $eAuto = sp + 628|0;
 $aOp8 = sp + 624|0;
 $iAddr = sp + 620|0;
 $iLimit9 = sp + 616|0;
 $addr = sp + 612|0;
 $size10 = sp + 608|0;
 $size11 = sp + 604|0;
 $sz = sp + 16|0;
 $res = sp + 600|0;
 $iLevel = sp + 596|0;
 $mask = sp + 592|0;
 $pTab = sp + 588|0;
 $i = sp + 584|0;
 $k = sp + 580|0;
 $nHidden = sp + 576|0;
 $pCol = sp + 572|0;
 $pPk = sp + 568|0;
 $pIdx = sp + 564|0;
 $i12 = sp + 560|0;
 $pTab13 = sp + 556|0;
 $pIdx14 = sp + 552|0;
 $pTab15 = sp + 548|0;
 $i16 = sp + 544|0;
 $mx = sp + 540|0;
 $cnum = sp + 796|0;
 $pIdx17 = sp + 536|0;
 $pTab18 = sp + 532|0;
 $i19 = sp + 528|0;
 $azOrigin = sp + 516|0;
 $i20 = sp + 512|0;
 $i21 = sp + 508|0;
 $p = sp + 504|0;
 $pColl = sp + 500|0;
 $pFK = sp + 496|0;
 $pTab22 = sp + 492|0;
 $i23 = sp + 488|0;
 $j = sp + 484|0;
 $pFK24 = sp + 480|0;
 $pTab25 = sp + 476|0;
 $pParent = sp + 472|0;
 $pIdx26 = sp + 468|0;
 $i27 = sp + 464|0;
 $j28 = sp + 460|0;
 $k29 = sp + 456|0;
 $x = sp + 452|0;
 $regResult = sp + 448|0;
 $regKey = sp + 444|0;
 $regRow = sp + 440|0;
 $addrTop = sp + 436|0;
 $addrOk = sp + 432|0;
 $aiCols = sp + 428|0;
 $iKey = sp + 424|0;
 $i30 = sp + 420|0;
 $j31 = sp + 416|0;
 $addr32 = sp + 412|0;
 $mxErr = sp + 408|0;
 $isQuick = sp + 404|0;
 $x33 = sp + 400|0;
 $pTbls = sp + 396|0;
 $cnt = sp + 392|0;
 $pTab34 = sp + 388|0;
 $pIdx35 = sp + 384|0;
 $pTab36 = sp + 380|0;
 $pIdx37 = sp + 376|0;
 $pPk38 = sp + 372|0;
 $pPrior = sp + 368|0;
 $loopTop = sp + 364|0;
 $iDataCur = sp + 360|0;
 $iIdxCur = sp + 356|0;
 $r1 = sp + 352|0;
 $zErr = sp + 348|0;
 $jmp2 = sp + 344|0;
 $jmp3 = sp + 340|0;
 $jmp239 = sp + 336|0;
 $jmp340 = sp + 332|0;
 $jmp4 = sp + 328|0;
 $jmp5 = sp + 324|0;
 $ckUniq = sp + 320|0;
 $uniqOk = sp + 316|0;
 $jmp6 = sp + 312|0;
 $kk = sp + 308|0;
 $iCol = sp + 304|0;
 $aOp41 = sp + 300|0;
 $pEnc = sp + 296|0;
 $iCookie = sp + 292|0;
 $aOp42 = sp + 288|0;
 $aOp43 = sp + 284|0;
 $i44 = sp + 280|0;
 $zOpt = sp + 276|0;
 $iBt = sp + 272|0;
 $eMode45 = sp + 268|0;
 $N = sp + 8|0;
 $N46 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pId1;
 HEAP32[$2>>2] = $pId2;
 HEAP32[$3>>2] = $pValue;
 HEAP32[$4>>2] = $minusFlag;
 HEAP32[$zLeft>>2] = 0;
 HEAP32[$zRight>>2] = 0;
 HEAP32[$zDb>>2] = 0;
 HEAP32[$mid>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$v>>2]|0;
 _sqlite3VdbeRunOnlyOnce($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 76|0);
 HEAP32[$13>>2] = 2;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = (_sqlite3TwoPartName($14,$15,$16,$pId)|0);
 HEAP32[$iDb>>2] = $17;
 $18 = HEAP32[$iDb>>2]|0;
 $19 = ($18|0)<(0);
 if ($19) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$iDb>>2]|0;
 $21 = HEAP32[$db>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20<<4)|0);
 HEAP32[$pDb>>2] = $24;
 $25 = HEAP32[$iDb>>2]|0;
 $26 = ($25|0)==(1);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = (_sqlite3OpenTempDatabase($27)|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   STACKTOP = sp;return;
  }
 }
 $30 = HEAP32[$db>>2]|0;
 $31 = HEAP32[$pId>>2]|0;
 $32 = (_sqlite3NameFromToken($30,$31)|0);
 HEAP32[$zLeft>>2] = $32;
 $33 = HEAP32[$zLeft>>2]|0;
 $34 = ($33|0)!=(0|0);
 if (!($34)) {
  STACKTOP = sp;return;
 }
 $35 = HEAP32[$4>>2]|0;
 $36 = ($35|0)!=(0);
 $37 = HEAP32[$db>>2]|0;
 $38 = HEAP32[$3>>2]|0;
 if ($36) {
  HEAP32[$vararg_buffer>>2] = $38;
  $39 = (_sqlite3MPrintf($37,33995,$vararg_buffer)|0);
  HEAP32[$zRight>>2] = $39;
 } else {
  $40 = (_sqlite3NameFromToken($37,$38)|0);
  HEAP32[$zRight>>2] = $40;
 }
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43>>>0)>(0);
 if ($44) {
  $45 = HEAP32[$pDb>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $46;
 } else {
  $47 = 0;
 }
 HEAP32[$zDb>>2] = $47;
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$zLeft>>2]|0;
 $50 = HEAP32[$zRight>>2]|0;
 $51 = HEAP32[$zDb>>2]|0;
 $52 = (_sqlite3AuthCheck($48,19,$49,$50,$51)|0);
 $53 = ($52|0)!=(0);
 L21: do {
  if (!($53)) {
   HEAP32[$aFcntl>>2] = 0;
   $54 = HEAP32[$zLeft>>2]|0;
   $55 = ((($aFcntl)) + 4|0);
   HEAP32[$55>>2] = $54;
   $56 = HEAP32[$zRight>>2]|0;
   $57 = ((($aFcntl)) + 8|0);
   HEAP32[$57>>2] = $56;
   $58 = ((($aFcntl)) + 12|0);
   HEAP32[$58>>2] = 0;
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 448|0);
   $61 = ((($60)) + 8|0);
   HEAP32[$61>>2] = 0;
   $62 = HEAP32[$db>>2]|0;
   $63 = HEAP32[$zDb>>2]|0;
   $64 = (_sqlite3_file_control($62,$63,14,$aFcntl)|0);
   HEAP32[$rc>>2] = $64;
   $65 = HEAP32[$rc>>2]|0;
   $66 = ($65|0)==(0);
   if ($66) {
    $67 = HEAP32[$v>>2]|0;
    $68 = HEAP32[$aFcntl>>2]|0;
    _returnSingleText($67,33999,$68);
    $69 = HEAP32[$aFcntl>>2]|0;
    _sqlite3_free($69);
    break;
   }
   $70 = HEAP32[$rc>>2]|0;
   $71 = ($70|0)!=(12);
   if ($71) {
    $72 = HEAP32[$aFcntl>>2]|0;
    $73 = ($72|0)!=(0|0);
    if ($73) {
     $74 = HEAP32[$0>>2]|0;
     $75 = HEAP32[$aFcntl>>2]|0;
     HEAP32[$vararg_buffer15>>2] = $75;
     _sqlite3ErrorMsg($74,17521,$vararg_buffer15);
     $76 = HEAP32[$aFcntl>>2]|0;
     _sqlite3_free($76);
    }
    $77 = HEAP32[$0>>2]|0;
    $78 = ((($77)) + 68|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = (($79) + 1)|0;
    HEAP32[$78>>2] = $80;
    $81 = HEAP32[$rc>>2]|0;
    $82 = HEAP32[$0>>2]|0;
    $83 = ((($82)) + 12|0);
    HEAP32[$83>>2] = $81;
    break;
   }
   HEAP32[$lwr>>2] = 0;
   HEAP32[$upr>>2] = 57;
   while(1) {
    $84 = HEAP32[$lwr>>2]|0;
    $85 = HEAP32[$upr>>2]|0;
    $86 = ($84|0)<=($85|0);
    if (!($86)) {
     break;
    }
    $87 = HEAP32[$lwr>>2]|0;
    $88 = HEAP32[$upr>>2]|0;
    $89 = (($87) + ($88))|0;
    $90 = (($89|0) / 2)&-1;
    HEAP32[$mid>>2] = $90;
    $91 = HEAP32[$zLeft>>2]|0;
    $92 = HEAP32[$mid>>2]|0;
    $93 = (2656 + (($92*12)|0)|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = (_sqlite3_stricmp($91,$94)|0);
    HEAP32[$rc>>2] = $95;
    $96 = HEAP32[$rc>>2]|0;
    $97 = ($96|0)==(0);
    if ($97) {
     break;
    }
    $98 = HEAP32[$rc>>2]|0;
    $99 = ($98|0)<(0);
    $100 = HEAP32[$mid>>2]|0;
    if ($99) {
     $101 = (($100) - 1)|0;
     HEAP32[$upr>>2] = $101;
     continue;
    } else {
     $102 = (($100) + 1)|0;
     HEAP32[$lwr>>2] = $102;
     continue;
    }
   }
   $103 = HEAP32[$lwr>>2]|0;
   $104 = HEAP32[$upr>>2]|0;
   $105 = ($103|0)>($104|0);
   if (!($105)) {
    $106 = HEAP32[$mid>>2]|0;
    $107 = (2656 + (($106*12)|0)|0);
    HEAP32[$pPragma>>2] = $107;
    $108 = HEAP32[$pPragma>>2]|0;
    $109 = ((($108)) + 5|0);
    $110 = HEAP8[$109>>0]|0;
    $111 = $110&255;
    $112 = $111 & 1;
    $113 = ($112|0)!=(0);
    if ($113) {
     $114 = HEAP32[$0>>2]|0;
     $115 = (_sqlite3ReadSchema($114)|0);
     $116 = ($115|0)!=(0);
     if ($116) {
      break;
     }
    }
    $117 = HEAP32[$pPragma>>2]|0;
    $118 = ((($117)) + 4|0);
    $119 = HEAP8[$118>>0]|0;
    $120 = $119&255;
    do {
     switch ($120|0) {
     case 11:  {
      $121 = HEAP32[$v>>2]|0;
      $122 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($121,$122);
      $123 = HEAP32[$zRight>>2]|0;
      $124 = ($123|0)!=(0|0);
      if ($124) {
       $142 = HEAP32[$zRight>>2]|0;
       $143 = (_sqlite3Atoi($142)|0);
       $144 = (_sqlite3AbsInt32($143)|0);
       HEAP32[$size>>2] = $144;
       $145 = HEAP32[$0>>2]|0;
       $146 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($145,0,$146);
       $147 = HEAP32[$v>>2]|0;
       $148 = HEAP32[$iDb>>2]|0;
       $149 = HEAP32[$size>>2]|0;
       (_sqlite3VdbeAddOp3($147,52,$148,3,$149)|0);
       $150 = HEAP32[$size>>2]|0;
       $151 = HEAP32[$pDb>>2]|0;
       $152 = ((($151)) + 12|0);
       $153 = HEAP32[$152>>2]|0;
       $154 = ((($153)) + 80|0);
       HEAP32[$154>>2] = $150;
       $155 = HEAP32[$pDb>>2]|0;
       $156 = ((($155)) + 4|0);
       $157 = HEAP32[$156>>2]|0;
       $158 = HEAP32[$pDb>>2]|0;
       $159 = ((($158)) + 12|0);
       $160 = HEAP32[$159>>2]|0;
       $161 = ((($160)) + 80|0);
       $162 = HEAP32[$161>>2]|0;
       (_sqlite3BtreeSetCacheSize($157,$162)|0);
       break L21;
      } else {
       $125 = HEAP32[$v>>2]|0;
       _setOneColumnName($125,34042);
       $126 = HEAP32[$0>>2]|0;
       $127 = ((($126)) + 76|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = (($128) + 2)|0;
       HEAP32[$127>>2] = $129;
       $130 = HEAP32[$v>>2]|0;
       $131 = (_sqlite3VdbeAddOpList($130,9,34006,0)|0);
       HEAP32[$aOp>>2] = $131;
       $132 = HEAP32[$iDb>>2]|0;
       $133 = HEAP32[$aOp>>2]|0;
       $134 = ((($133)) + 4|0);
       HEAP32[$134>>2] = $132;
       $135 = HEAP32[$iDb>>2]|0;
       $136 = HEAP32[$aOp>>2]|0;
       $137 = ((($136)) + 20|0);
       $138 = ((($137)) + 4|0);
       HEAP32[$138>>2] = $135;
       $139 = HEAP32[$aOp>>2]|0;
       $140 = ((($139)) + 120|0);
       $141 = ((($140)) + 4|0);
       HEAP32[$141>>2] = 2000;
       break L21;
      }
      break;
     }
     case 25:  {
      $163 = HEAP32[$pDb>>2]|0;
      $164 = ((($163)) + 4|0);
      $165 = HEAP32[$164>>2]|0;
      HEAP32[$pBt>>2] = $165;
      $166 = HEAP32[$zRight>>2]|0;
      $167 = ($166|0)!=(0|0);
      if ($167) {
       $177 = HEAP32[$zRight>>2]|0;
       $178 = (_sqlite3Atoi($177)|0);
       $179 = HEAP32[$db>>2]|0;
       $180 = ((($179)) + 72|0);
       HEAP32[$180>>2] = $178;
       $181 = HEAP32[$pBt>>2]|0;
       $182 = HEAP32[$db>>2]|0;
       $183 = ((($182)) + 72|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = (_sqlite3BtreeSetPageSize($181,$184,-1,0)|0);
       $186 = (7)==($185|0);
       if (!($186)) {
        break L21;
       }
       $187 = HEAP32[$db>>2]|0;
       _sqlite3OomFault($187);
       break L21;
      }
      $168 = HEAP32[$pBt>>2]|0;
      $169 = ($168|0)!=(0|0);
      if ($169) {
       $170 = HEAP32[$pBt>>2]|0;
       $171 = (_sqlite3BtreeGetPageSize($170)|0);
       $172 = $171;
      } else {
       $172 = 0;
      }
      HEAP32[$size1>>2] = $172;
      $173 = HEAP32[$v>>2]|0;
      $174 = HEAP32[$size1>>2]|0;
      $175 = ($174|0)<(0);
      $176 = $175 << 31 >> 31;
      _returnSingleInt($173,34053,$174,$176);
      break L21;
      break;
     }
     case 26:  {
      $188 = HEAP32[$pDb>>2]|0;
      $189 = ((($188)) + 4|0);
      $190 = HEAP32[$189>>2]|0;
      HEAP32[$pBt2>>2] = $190;
      HEAP32[$b>>2] = -1;
      $191 = HEAP32[$zRight>>2]|0;
      $192 = ($191|0)!=(0|0);
      if ($192) {
       $193 = HEAP32[$zRight>>2]|0;
       $194 = (_sqlite3GetBoolean($193,0)|0);
       $195 = $194&255;
       HEAP32[$b>>2] = $195;
      }
      $196 = HEAP32[$2>>2]|0;
      $197 = ((($196)) + 4|0);
      $198 = HEAP32[$197>>2]|0;
      $199 = ($198|0)==(0);
      $200 = HEAP32[$b>>2]|0;
      $201 = ($200|0)>=(0);
      $or$cond = $199 & $201;
      L65: do {
       if ($or$cond) {
        HEAP32[$ii>>2] = 0;
        while(1) {
         $202 = HEAP32[$ii>>2]|0;
         $203 = HEAP32[$db>>2]|0;
         $204 = ((($203)) + 20|0);
         $205 = HEAP32[$204>>2]|0;
         $206 = ($202|0)<($205|0);
         if (!($206)) {
          break L65;
         }
         $207 = HEAP32[$ii>>2]|0;
         $208 = HEAP32[$db>>2]|0;
         $209 = ((($208)) + 16|0);
         $210 = HEAP32[$209>>2]|0;
         $211 = (($210) + ($207<<4)|0);
         $212 = ((($211)) + 4|0);
         $213 = HEAP32[$212>>2]|0;
         $214 = HEAP32[$b>>2]|0;
         (_sqlite3BtreeSecureDelete($213,$214)|0);
         $215 = HEAP32[$ii>>2]|0;
         $216 = (($215) + 1)|0;
         HEAP32[$ii>>2] = $216;
        }
       }
      } while(0);
      $217 = HEAP32[$pBt2>>2]|0;
      $218 = HEAP32[$b>>2]|0;
      $219 = (_sqlite3BtreeSecureDelete($217,$218)|0);
      HEAP32[$b>>2] = $219;
      $220 = HEAP32[$v>>2]|0;
      $221 = HEAP32[$b>>2]|0;
      $222 = ($221|0)<(0);
      $223 = $222 << 31 >> 31;
      _returnSingleInt($220,34063,$221,$223);
      break L21;
      break;
     }
     case 23:  {
      $224 = HEAP32[$0>>2]|0;
      $225 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($224,$225);
      $226 = HEAP32[$0>>2]|0;
      $227 = ((($226)) + 76|0);
      $228 = HEAP32[$227>>2]|0;
      $229 = (($228) + 1)|0;
      HEAP32[$227>>2] = $229;
      HEAP32[$iReg>>2] = $229;
      $230 = HEAP32[$zLeft>>2]|0;
      $231 = HEAP8[$230>>0]|0;
      $232 = $231&255;
      $233 = (16974 + ($232)|0);
      $234 = HEAP8[$233>>0]|0;
      $235 = $234&255;
      $236 = ($235|0)==(112);
      $237 = HEAP32[$v>>2]|0;
      $238 = HEAP32[$iDb>>2]|0;
      $239 = HEAP32[$iReg>>2]|0;
      if ($236) {
       (_sqlite3VdbeAddOp2($237,156,$238,$239)|0);
      } else {
       $240 = HEAP32[$zRight>>2]|0;
       $241 = (_sqlite3Atoi($240)|0);
       $242 = (_sqlite3AbsInt32($241)|0);
       (_sqlite3VdbeAddOp3($237,157,$238,$239,$242)|0);
      }
      $243 = HEAP32[$v>>2]|0;
      $244 = HEAP32[$iReg>>2]|0;
      (_sqlite3VdbeAddOp2($243,33,$244,1)|0);
      $245 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($245,1);
      $246 = HEAP32[$v>>2]|0;
      $247 = HEAP32[$zLeft>>2]|0;
      (_sqlite3VdbeSetColName($246,0,0,$247,(-1))|0);
      break L21;
      break;
     }
     case 22:  {
      HEAP32[$zRet>>2] = 34077;
      $248 = HEAP32[$zRight>>2]|0;
      $249 = (_getLockingMode($248)|0);
      HEAP32[$eMode>>2] = $249;
      $250 = HEAP32[$2>>2]|0;
      $251 = ((($250)) + 4|0);
      $252 = HEAP32[$251>>2]|0;
      $253 = ($252|0)==(0);
      $254 = HEAP32[$eMode>>2]|0;
      $255 = ($254|0)==(-1);
      $or$cond3 = $253 & $255;
      if ($or$cond3) {
       $256 = HEAP32[$db>>2]|0;
       $257 = ((($256)) + 67|0);
       $258 = HEAP8[$257>>0]|0;
       $259 = $258&255;
       HEAP32[$eMode>>2] = $259;
      } else {
       $260 = HEAP32[$2>>2]|0;
       $261 = ((($260)) + 4|0);
       $262 = HEAP32[$261>>2]|0;
       $263 = ($262|0)==(0);
       if ($263) {
        HEAP32[$ii3>>2] = 2;
        while(1) {
         $264 = HEAP32[$ii3>>2]|0;
         $265 = HEAP32[$db>>2]|0;
         $266 = ((($265)) + 20|0);
         $267 = HEAP32[$266>>2]|0;
         $268 = ($264|0)<($267|0);
         if (!($268)) {
          break;
         }
         $269 = HEAP32[$ii3>>2]|0;
         $270 = HEAP32[$db>>2]|0;
         $271 = ((($270)) + 16|0);
         $272 = HEAP32[$271>>2]|0;
         $273 = (($272) + ($269<<4)|0);
         $274 = ((($273)) + 4|0);
         $275 = HEAP32[$274>>2]|0;
         $276 = (_sqlite3BtreePager($275)|0);
         HEAP32[$pPager>>2] = $276;
         $277 = HEAP32[$pPager>>2]|0;
         $278 = HEAP32[$eMode>>2]|0;
         (_sqlite3PagerLockingMode($277,$278)|0);
         $279 = HEAP32[$ii3>>2]|0;
         $280 = (($279) + 1)|0;
         HEAP32[$ii3>>2] = $280;
        }
        $281 = HEAP32[$eMode>>2]|0;
        $282 = $281&255;
        $283 = HEAP32[$db>>2]|0;
        $284 = ((($283)) + 67|0);
        HEAP8[$284>>0] = $282;
       }
       $285 = HEAP32[$pDb>>2]|0;
       $286 = ((($285)) + 4|0);
       $287 = HEAP32[$286>>2]|0;
       $288 = (_sqlite3BtreePager($287)|0);
       HEAP32[$pPager>>2] = $288;
       $289 = HEAP32[$pPager>>2]|0;
       $290 = HEAP32[$eMode>>2]|0;
       $291 = (_sqlite3PagerLockingMode($289,$290)|0);
       HEAP32[$eMode>>2] = $291;
      }
      $292 = HEAP32[$eMode>>2]|0;
      $293 = ($292|0)==(1);
      if ($293) {
       HEAP32[$zRet>>2] = 34084;
      }
      $294 = HEAP32[$v>>2]|0;
      $295 = HEAP32[$zRet>>2]|0;
      _returnSingleText($294,34094,$295);
      break L21;
      break;
     }
     case 19:  {
      $296 = HEAP32[$v>>2]|0;
      _setOneColumnName($296,34107);
      $297 = HEAP32[$zRight>>2]|0;
      $298 = ($297|0)==(0|0);
      if ($298) {
       HEAP32[$eMode4>>2] = -1;
      } else {
       $299 = HEAP32[$zRight>>2]|0;
       $300 = (_sqlite3Strlen30($299)|0);
       HEAP32[$n>>2] = $300;
       HEAP32[$eMode4>>2] = 0;
       while(1) {
        $301 = HEAP32[$eMode4>>2]|0;
        $302 = (_sqlite3JournalModename($301)|0);
        HEAP32[$zMode>>2] = $302;
        $303 = ($302|0)!=(0|0);
        if (!($303)) {
         break;
        }
        $304 = HEAP32[$zRight>>2]|0;
        $305 = HEAP32[$zMode>>2]|0;
        $306 = HEAP32[$n>>2]|0;
        $307 = (_sqlite3_strnicmp($304,$305,$306)|0);
        $308 = ($307|0)==(0);
        if ($308) {
         break;
        }
        $309 = HEAP32[$eMode4>>2]|0;
        $310 = (($309) + 1)|0;
        HEAP32[$eMode4>>2] = $310;
       }
       $311 = HEAP32[$zMode>>2]|0;
       $312 = ($311|0)!=(0|0);
       if (!($312)) {
        HEAP32[$eMode4>>2] = -1;
       }
      }
      $313 = HEAP32[$eMode4>>2]|0;
      $314 = ($313|0)==(-1);
      if ($314) {
       $315 = HEAP32[$2>>2]|0;
       $316 = ((($315)) + 4|0);
       $317 = HEAP32[$316>>2]|0;
       $318 = ($317|0)==(0);
       if ($318) {
        HEAP32[$iDb>>2] = 0;
        $319 = HEAP32[$2>>2]|0;
        $320 = ((($319)) + 4|0);
        HEAP32[$320>>2] = 1;
       }
      }
      $321 = HEAP32[$db>>2]|0;
      $322 = ((($321)) + 20|0);
      $323 = HEAP32[$322>>2]|0;
      $324 = (($323) - 1)|0;
      HEAP32[$ii5>>2] = $324;
      while(1) {
       $325 = HEAP32[$ii5>>2]|0;
       $326 = ($325|0)>=(0);
       if (!($326)) {
        break;
       }
       $327 = HEAP32[$ii5>>2]|0;
       $328 = HEAP32[$db>>2]|0;
       $329 = ((($328)) + 16|0);
       $330 = HEAP32[$329>>2]|0;
       $331 = (($330) + ($327<<4)|0);
       $332 = ((($331)) + 4|0);
       $333 = HEAP32[$332>>2]|0;
       $334 = ($333|0)!=(0|0);
       do {
        if ($334) {
         $335 = HEAP32[$ii5>>2]|0;
         $336 = HEAP32[$iDb>>2]|0;
         $337 = ($335|0)==($336|0);
         if (!($337)) {
          $338 = HEAP32[$2>>2]|0;
          $339 = ((($338)) + 4|0);
          $340 = HEAP32[$339>>2]|0;
          $341 = ($340|0)==(0);
          if (!($341)) {
           break;
          }
         }
         $342 = HEAP32[$v>>2]|0;
         $343 = HEAP32[$ii5>>2]|0;
         _sqlite3VdbeUsesBtree($342,$343);
         $344 = HEAP32[$v>>2]|0;
         $345 = HEAP32[$ii5>>2]|0;
         $346 = HEAP32[$eMode4>>2]|0;
         (_sqlite3VdbeAddOp3($344,9,$345,1,$346)|0);
        }
       } while(0);
       $347 = HEAP32[$ii5>>2]|0;
       $348 = (($347) + -1)|0;
       HEAP32[$ii5>>2] = $348;
      }
      $349 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($349,33,1,1)|0);
      break L21;
      break;
     }
     case 20:  {
      $350 = HEAP32[$pDb>>2]|0;
      $351 = ((($350)) + 4|0);
      $352 = HEAP32[$351>>2]|0;
      $353 = (_sqlite3BtreePager($352)|0);
      HEAP32[$pPager6>>2] = $353;
      $354 = $iLimit;
      $355 = $354;
      HEAP32[$355>>2] = -2;
      $356 = (($354) + 4)|0;
      $357 = $356;
      HEAP32[$357>>2] = -1;
      $358 = HEAP32[$zRight>>2]|0;
      $359 = ($358|0)!=(0|0);
      if ($359) {
       $360 = HEAP32[$zRight>>2]|0;
       (_sqlite3DecOrHexToI64($360,$iLimit)|0);
       $361 = $iLimit;
       $362 = $361;
       $363 = HEAP32[$362>>2]|0;
       $364 = (($361) + 4)|0;
       $365 = $364;
       $366 = HEAP32[$365>>2]|0;
       $367 = ($366|0)<(-1);
       $368 = ($363>>>0)<(4294967295);
       $369 = ($366|0)==(-1);
       $370 = $369 & $368;
       $371 = $367 | $370;
       if ($371) {
        $372 = $iLimit;
        $373 = $372;
        HEAP32[$373>>2] = -1;
        $374 = (($372) + 4)|0;
        $375 = $374;
        HEAP32[$375>>2] = -1;
       }
      }
      $376 = HEAP32[$pPager6>>2]|0;
      $377 = $iLimit;
      $378 = $377;
      $379 = HEAP32[$378>>2]|0;
      $380 = (($377) + 4)|0;
      $381 = $380;
      $382 = HEAP32[$381>>2]|0;
      $383 = (_sqlite3PagerJournalSizeLimit($376,$379,$382)|0);
      $384 = tempRet0;
      $385 = $iLimit;
      $386 = $385;
      HEAP32[$386>>2] = $383;
      $387 = (($385) + 4)|0;
      $388 = $387;
      HEAP32[$388>>2] = $384;
      $389 = HEAP32[$v>>2]|0;
      $390 = $iLimit;
      $391 = $390;
      $392 = HEAP32[$391>>2]|0;
      $393 = (($390) + 4)|0;
      $394 = $393;
      $395 = HEAP32[$394>>2]|0;
      _returnSingleInt($389,34120,$392,$395);
      break L21;
      break;
     }
     case 1:  {
      $396 = HEAP32[$pDb>>2]|0;
      $397 = ((($396)) + 4|0);
      $398 = HEAP32[$397>>2]|0;
      HEAP32[$pBt7>>2] = $398;
      $399 = HEAP32[$zRight>>2]|0;
      $400 = ($399|0)!=(0|0);
      if (!($400)) {
       $401 = HEAP32[$v>>2]|0;
       $402 = HEAP32[$pBt7>>2]|0;
       $403 = (_sqlite3BtreeGetAutoVacuum($402)|0);
       $404 = ($403|0)<(0);
       $405 = $404 << 31 >> 31;
       _returnSingleInt($401,34139,$403,$405);
       break L21;
      }
      $406 = HEAP32[$zRight>>2]|0;
      $407 = (_getAutoVacuum($406)|0);
      HEAP32[$eAuto>>2] = $407;
      $408 = HEAP32[$eAuto>>2]|0;
      $409 = $408&255;
      $410 = HEAP32[$db>>2]|0;
      $411 = ((($410)) + 68|0);
      HEAP8[$411>>0] = $409;
      $412 = HEAP32[$pBt7>>2]|0;
      $413 = HEAP32[$eAuto>>2]|0;
      $414 = (_sqlite3BtreeSetAutoVacuum($412,$413)|0);
      HEAP32[$rc>>2] = $414;
      $415 = HEAP32[$rc>>2]|0;
      $416 = ($415|0)==(0);
      if (!($416)) {
       break L21;
      }
      $417 = HEAP32[$eAuto>>2]|0;
      $418 = ($417|0)==(1);
      $419 = HEAP32[$eAuto>>2]|0;
      $420 = ($419|0)==(2);
      $or$cond5 = $418 | $420;
      if (!($or$cond5)) {
       break L21;
      }
      $421 = HEAP32[$v>>2]|0;
      $422 = (_sqlite3VdbeCurrentAddr($421)|0);
      HEAP32[$iAddr>>2] = $422;
      $423 = HEAP32[$v>>2]|0;
      $424 = (_sqlite3VdbeAddOpList($423,5,34151,0)|0);
      HEAP32[$aOp8>>2] = $424;
      $425 = HEAP32[$iDb>>2]|0;
      $426 = HEAP32[$aOp8>>2]|0;
      $427 = ((($426)) + 4|0);
      HEAP32[$427>>2] = $425;
      $428 = HEAP32[$iDb>>2]|0;
      $429 = HEAP32[$aOp8>>2]|0;
      $430 = ((($429)) + 20|0);
      $431 = ((($430)) + 4|0);
      HEAP32[$431>>2] = $428;
      $432 = HEAP32[$iAddr>>2]|0;
      $433 = (($432) + 4)|0;
      $434 = HEAP32[$aOp8>>2]|0;
      $435 = ((($434)) + 40|0);
      $436 = ((($435)) + 8|0);
      HEAP32[$436>>2] = $433;
      $437 = HEAP32[$iDb>>2]|0;
      $438 = HEAP32[$aOp8>>2]|0;
      $439 = ((($438)) + 80|0);
      $440 = ((($439)) + 4|0);
      HEAP32[$440>>2] = $437;
      $441 = HEAP32[$eAuto>>2]|0;
      $442 = (($441) - 1)|0;
      $443 = HEAP32[$aOp8>>2]|0;
      $444 = ((($443)) + 80|0);
      $445 = ((($444)) + 12|0);
      HEAP32[$445>>2] = $442;
      $446 = HEAP32[$v>>2]|0;
      $447 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($446,$447);
      break L21;
      break;
     }
     case 15:  {
      $448 = HEAP32[$zRight>>2]|0;
      $449 = ($448|0)==(0|0);
      if ($449) {
       label = 89;
      } else {
       $450 = HEAP32[$zRight>>2]|0;
       $451 = (_sqlite3GetInt32($450,$iLimit9)|0);
       $452 = ($451|0)==(0);
       $453 = HEAP32[$iLimit9>>2]|0;
       $454 = ($453|0)<=(0);
       $or$cond7 = $452 | $454;
       if ($or$cond7) {
        label = 89;
       }
      }
      if ((label|0) == 89) {
       HEAP32[$iLimit9>>2] = 2147483647;
      }
      $455 = HEAP32[$0>>2]|0;
      $456 = HEAP32[$iDb>>2]|0;
      _sqlite3BeginWriteOperation($455,0,$456);
      $457 = HEAP32[$v>>2]|0;
      $458 = HEAP32[$iLimit9>>2]|0;
      (_sqlite3VdbeAddOp2($457,22,$458,1)|0);
      $459 = HEAP32[$v>>2]|0;
      $460 = HEAP32[$iDb>>2]|0;
      $461 = (_sqlite3VdbeAddOp1($459,146,$460)|0);
      HEAP32[$addr>>2] = $461;
      $462 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp1($462,33,1)|0);
      $463 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($463,37,1,-1)|0);
      $464 = HEAP32[$v>>2]|0;
      $465 = HEAP32[$addr>>2]|0;
      (_sqlite3VdbeAddOp2($464,138,1,$465)|0);
      $466 = HEAP32[$v>>2]|0;
      $467 = HEAP32[$addr>>2]|0;
      _sqlite3VdbeJumpHere($466,$467);
      break L21;
      break;
     }
     case 4:  {
      $468 = HEAP32[$zRight>>2]|0;
      $469 = ($468|0)!=(0|0);
      if ($469) {
       $478 = HEAP32[$zRight>>2]|0;
       $479 = (_sqlite3Atoi($478)|0);
       HEAP32[$size10>>2] = $479;
       $480 = HEAP32[$size10>>2]|0;
       $481 = HEAP32[$pDb>>2]|0;
       $482 = ((($481)) + 12|0);
       $483 = HEAP32[$482>>2]|0;
       $484 = ((($483)) + 80|0);
       HEAP32[$484>>2] = $480;
       $485 = HEAP32[$pDb>>2]|0;
       $486 = ((($485)) + 4|0);
       $487 = HEAP32[$486>>2]|0;
       $488 = HEAP32[$pDb>>2]|0;
       $489 = ((($488)) + 12|0);
       $490 = HEAP32[$489>>2]|0;
       $491 = ((($490)) + 80|0);
       $492 = HEAP32[$491>>2]|0;
       (_sqlite3BtreeSetCacheSize($487,$492)|0);
       break L21;
      } else {
       $470 = HEAP32[$v>>2]|0;
       $471 = HEAP32[$pDb>>2]|0;
       $472 = ((($471)) + 12|0);
       $473 = HEAP32[$472>>2]|0;
       $474 = ((($473)) + 80|0);
       $475 = HEAP32[$474>>2]|0;
       $476 = ($475|0)<(0);
       $477 = $476 << 31 >> 31;
       _returnSingleInt($470,34042,$475,$477);
       break L21;
      }
      break;
     }
     case 5:  {
      $493 = HEAP32[$zRight>>2]|0;
      $494 = ($493|0)!=(0|0);
      if (!($494)) {
       $495 = HEAP32[$v>>2]|0;
       $496 = HEAP32[$db>>2]|0;
       $497 = ((($496)) + 24|0);
       $498 = HEAP32[$497>>2]|0;
       $499 = $498 & 32;
       $500 = ($499|0)==(0);
       if ($500) {
        $506 = 0;
       } else {
        $501 = HEAP32[$pDb>>2]|0;
        $502 = ((($501)) + 4|0);
        $503 = HEAP32[$502>>2]|0;
        $504 = (_sqlite3BtreeSetSpillSize($503,0)|0);
        $506 = $504;
       }
       $505 = ($506|0)<(0);
       $507 = $505 << 31 >> 31;
       _returnSingleInt($495,34171,$506,$507);
       break L21;
      }
      HEAP32[$size11>>2] = 1;
      $508 = HEAP32[$zRight>>2]|0;
      $509 = (_sqlite3GetInt32($508,$size11)|0);
      $510 = ($509|0)!=(0);
      if ($510) {
       $511 = HEAP32[$pDb>>2]|0;
       $512 = ((($511)) + 4|0);
       $513 = HEAP32[$512>>2]|0;
       $514 = HEAP32[$size11>>2]|0;
       (_sqlite3BtreeSetSpillSize($513,$514)|0);
      }
      $515 = HEAP32[$zRight>>2]|0;
      $516 = HEAP32[$size11>>2]|0;
      $517 = ($516|0)!=(0);
      $518 = $517&1;
      $519 = $518&255;
      $520 = (_sqlite3GetBoolean($515,$519)|0);
      $521 = ($520<<24>>24)!=(0);
      $522 = HEAP32[$db>>2]|0;
      $523 = ((($522)) + 24|0);
      $524 = HEAP32[$523>>2]|0;
      if ($521) {
       $525 = $524 | 32;
       HEAP32[$523>>2] = $525;
      } else {
       $526 = $524 & -33;
       HEAP32[$523>>2] = $526;
      }
      $527 = HEAP32[$db>>2]|0;
      _setAllPagerFlags($527);
      break L21;
      break;
     }
     case 24:  {
      $528 = $sz;
      $529 = $528;
      HEAP32[$529>>2] = 0;
      $530 = (($528) + 4)|0;
      $531 = $530;
      HEAP32[$531>>2] = 0;
      HEAP32[$rc>>2] = 0;
      $532 = HEAP32[$rc>>2]|0;
      $533 = ($532|0)==(0);
      if ($533) {
       $534 = HEAP32[$v>>2]|0;
       $535 = $sz;
       $536 = $535;
       $537 = HEAP32[$536>>2]|0;
       $538 = (($535) + 4)|0;
       $539 = $538;
       $540 = HEAP32[$539>>2]|0;
       _returnSingleInt($534,34183,$537,$540);
       break L21;
      }
      $541 = HEAP32[$rc>>2]|0;
      $542 = ($541|0)!=(12);
      if (!($542)) {
       break L21;
      }
      $543 = HEAP32[$0>>2]|0;
      $544 = ((($543)) + 68|0);
      $545 = HEAP32[$544>>2]|0;
      $546 = (($545) + 1)|0;
      HEAP32[$544>>2] = $546;
      $547 = HEAP32[$rc>>2]|0;
      $548 = HEAP32[$0>>2]|0;
      $549 = ((($548)) + 12|0);
      HEAP32[$549>>2] = $547;
      break L21;
      break;
     }
     case 32:  {
      $550 = HEAP32[$zRight>>2]|0;
      $551 = ($550|0)!=(0|0);
      if ($551) {
       $557 = HEAP32[$0>>2]|0;
       $558 = HEAP32[$zRight>>2]|0;
       (_changeTempStorage($557,$558)|0);
       break L21;
      } else {
       $552 = HEAP32[$v>>2]|0;
       $553 = HEAP32[$db>>2]|0;
       $554 = ((($553)) + 64|0);
       $555 = HEAP8[$554>>0]|0;
       $556 = $555&255;
       _returnSingleInt($552,34193,$556,0);
       break L21;
      }
      break;
     }
     case 33:  {
      $559 = HEAP32[$zRight>>2]|0;
      $560 = ($559|0)!=(0|0);
      if (!($560)) {
       $561 = HEAP32[$v>>2]|0;
       $562 = HEAP32[844>>2]|0;
       _returnSingleText($561,34204,$562);
       break L21;
      }
      $563 = HEAP32[$zRight>>2]|0;
      $564 = HEAP8[$563>>0]|0;
      $565 = ($564<<24>>24)!=(0);
      if ($565) {
       $566 = HEAP32[$db>>2]|0;
       $567 = HEAP32[$566>>2]|0;
       $568 = HEAP32[$zRight>>2]|0;
       $569 = (_sqlite3OsAccess($567,$568,1,$res)|0);
       HEAP32[$rc>>2] = $569;
       $570 = HEAP32[$rc>>2]|0;
       $571 = ($570|0)!=(0);
       $572 = HEAP32[$res>>2]|0;
       $573 = ($572|0)==(0);
       $or$cond9 = $571 | $573;
       if ($or$cond9) {
        $574 = HEAP32[$0>>2]|0;
        _sqlite3ErrorMsg($574,34225,$vararg_buffer18);
        break L21;
       }
      }
      $575 = HEAP32[$db>>2]|0;
      $576 = ((($575)) + 64|0);
      $577 = HEAP8[$576>>0]|0;
      $578 = $577&255;
      $579 = ($578|0)<=(1);
      if ($579) {
       $580 = HEAP32[$0>>2]|0;
       (_invalidateTempStorage($580)|0);
      }
      $581 = HEAP32[844>>2]|0;
      _sqlite3_free($581);
      $582 = HEAP32[$zRight>>2]|0;
      $583 = HEAP8[$582>>0]|0;
      $584 = ($583<<24>>24)!=(0);
      if ($584) {
       $585 = HEAP32[$zRight>>2]|0;
       HEAP32[$vararg_buffer20>>2] = $585;
       $586 = (_sqlite3_mprintf(17521,$vararg_buffer20)|0);
       HEAP32[844>>2] = $586;
       break L21;
      } else {
       HEAP32[844>>2] = 0;
       break L21;
      }
      break;
     }
     case 30:  {
      $587 = HEAP32[$zRight>>2]|0;
      $588 = ($587|0)!=(0|0);
      if (!($588)) {
       $589 = HEAP32[$v>>2]|0;
       $590 = HEAP32[$pDb>>2]|0;
       $591 = ((($590)) + 8|0);
       $592 = HEAP8[$591>>0]|0;
       $593 = $592&255;
       $594 = (($593) - 1)|0;
       $595 = ($594|0)<(0);
       $596 = $595 << 31 >> 31;
       _returnSingleInt($589,34250,$594,$596);
       break L21;
      }
      $597 = HEAP32[$db>>2]|0;
      $598 = ((($597)) + 63|0);
      $599 = HEAP8[$598>>0]|0;
      $600 = ($599<<24>>24)!=(0);
      if ($600) {
       $602 = HEAP32[$zRight>>2]|0;
       $603 = (_getSafetyLevel($602,0,1)|0);
       $604 = $603&255;
       $605 = (($604) + 1)|0;
       $606 = $605 & 7;
       HEAP32[$iLevel>>2] = $606;
       $607 = HEAP32[$iLevel>>2]|0;
       $608 = ($607|0)==(0);
       $$ = $608 ? 1 : $606;
       HEAP32[$iLevel>>2] = $$;
       $609 = HEAP32[$iLevel>>2]|0;
       $610 = $609&255;
       $611 = HEAP32[$pDb>>2]|0;
       $612 = ((($611)) + 8|0);
       HEAP8[$612>>0] = $610;
       $613 = HEAP32[$db>>2]|0;
       _setAllPagerFlags($613);
       break L21;
      } else {
       $601 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($601,34262,$vararg_buffer23);
       break L21;
      }
      break;
     }
     case 2:  {
      $614 = HEAP32[$zRight>>2]|0;
      $615 = ($614|0)==(0|0);
      if ($615) {
       $616 = HEAP32[$v>>2]|0;
       $617 = HEAP32[$pPragma>>2]|0;
       $618 = HEAP32[$617>>2]|0;
       $619 = HEAP32[$db>>2]|0;
       $620 = ((($619)) + 24|0);
       $621 = HEAP32[$620>>2]|0;
       $622 = HEAP32[$pPragma>>2]|0;
       $623 = ((($622)) + 8|0);
       $624 = HEAP32[$623>>2]|0;
       $625 = $621 & $624;
       $626 = ($625|0)!=(0);
       $627 = $626&1;
       $628 = ($627|0)<(0);
       $629 = $628 << 31 >> 31;
       _returnSingleInt($616,$618,$627,$629);
       break L21;
      }
      $630 = HEAP32[$pPragma>>2]|0;
      $631 = ((($630)) + 8|0);
      $632 = HEAP32[$631>>2]|0;
      HEAP32[$mask>>2] = $632;
      $633 = HEAP32[$db>>2]|0;
      $634 = ((($633)) + 63|0);
      $635 = HEAP8[$634>>0]|0;
      $636 = $635&255;
      $637 = ($636|0)==(0);
      if ($637) {
       $638 = HEAP32[$mask>>2]|0;
       $639 = $638 & -524289;
       HEAP32[$mask>>2] = $639;
      }
      $640 = HEAP32[$zRight>>2]|0;
      $641 = (_sqlite3GetBoolean($640,0)|0);
      $642 = ($641<<24>>24)!=(0);
      $643 = HEAP32[$mask>>2]|0;
      if ($642) {
       $644 = HEAP32[$db>>2]|0;
       $645 = ((($644)) + 24|0);
       $646 = HEAP32[$645>>2]|0;
       $647 = $646 | $643;
       HEAP32[$645>>2] = $647;
      } else {
       $648 = $643 ^ -1;
       $649 = HEAP32[$db>>2]|0;
       $650 = ((($649)) + 24|0);
       $651 = HEAP32[$650>>2]|0;
       $652 = $651 & $648;
       HEAP32[$650>>2] = $652;
       $653 = HEAP32[$mask>>2]|0;
       $654 = ($653|0)==(16777216);
       if ($654) {
        $655 = HEAP32[$db>>2]|0;
        $656 = ((($655)) + 520|0);
        $657 = $656;
        $658 = $657;
        HEAP32[$658>>2] = 0;
        $659 = (($657) + 4)|0;
        $660 = $659;
        HEAP32[$660>>2] = 0;
       }
      }
      $661 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($661,147,0,0)|0);
      $662 = HEAP32[$db>>2]|0;
      _setAllPagerFlags($662);
      break L21;
      break;
     }
     case 31:  {
      $663 = HEAP32[$zRight>>2]|0;
      $664 = ($663|0)!=(0|0);
      if (!($664)) {
       break L21;
      }
      $665 = HEAP32[$db>>2]|0;
      $666 = HEAP32[$zRight>>2]|0;
      $667 = HEAP32[$zDb>>2]|0;
      $668 = (_sqlite3FindTable($665,$666,$667)|0);
      HEAP32[$pTab>>2] = $668;
      $669 = HEAP32[$pTab>>2]|0;
      $670 = ($669|0)!=(0|0);
      if (!($670)) {
       break L21;
      }
      HEAP32[$nHidden>>2] = 0;
      $671 = HEAP32[$pTab>>2]|0;
      $672 = (_sqlite3PrimaryKeyIndex($671)|0);
      HEAP32[$pPk>>2] = $672;
      $673 = HEAP32[$0>>2]|0;
      $674 = ((($673)) + 76|0);
      HEAP32[$674>>2] = 6;
      $675 = HEAP32[$0>>2]|0;
      $676 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($675,$676);
      $677 = HEAP32[$v>>2]|0;
      _setAllColumnNames($677,6,3352);
      $678 = HEAP32[$0>>2]|0;
      $679 = HEAP32[$pTab>>2]|0;
      (_sqlite3ViewGetColumnNames($678,$679)|0);
      HEAP32[$i>>2] = 0;
      $680 = HEAP32[$pTab>>2]|0;
      $681 = ((($680)) + 4|0);
      $682 = HEAP32[$681>>2]|0;
      HEAP32[$pCol>>2] = $682;
      while(1) {
       $683 = HEAP32[$i>>2]|0;
       $684 = HEAP32[$pTab>>2]|0;
       $685 = ((($684)) + 34|0);
       $686 = HEAP16[$685>>1]|0;
       $687 = $686 << 16 >> 16;
       $688 = ($683|0)<($687|0);
       if (!($688)) {
        break L21;
       }
       $689 = HEAP32[$pCol>>2]|0;
       $690 = ((($689)) + 23|0);
       $691 = HEAP8[$690>>0]|0;
       $692 = $691&255;
       $693 = $692 & 2;
       $694 = ($693|0)!=(0);
       if ($694) {
        $695 = HEAP32[$nHidden>>2]|0;
        $696 = (($695) + 1)|0;
        HEAP32[$nHidden>>2] = $696;
       } else {
        $697 = HEAP32[$pCol>>2]|0;
        $698 = ((($697)) + 23|0);
        $699 = HEAP8[$698>>0]|0;
        $700 = $699&255;
        $701 = $700 & 1;
        $702 = ($701|0)==(0);
        L202: do {
         if ($702) {
          HEAP32[$k>>2] = 0;
         } else {
          $703 = HEAP32[$pPk>>2]|0;
          $704 = ($703|0)==(0|0);
          HEAP32[$k>>2] = 1;
          if ($704) {
           break;
          }
          while(1) {
           $705 = HEAP32[$k>>2]|0;
           $706 = HEAP32[$pTab>>2]|0;
           $707 = ((($706)) + 34|0);
           $708 = HEAP16[$707>>1]|0;
           $709 = $708 << 16 >> 16;
           $710 = ($705|0)<=($709|0);
           if (!($710)) {
            break L202;
           }
           $711 = HEAP32[$k>>2]|0;
           $712 = (($711) - 1)|0;
           $713 = HEAP32[$pPk>>2]|0;
           $714 = ((($713)) + 4|0);
           $715 = HEAP32[$714>>2]|0;
           $716 = (($715) + ($712<<1)|0);
           $717 = HEAP16[$716>>1]|0;
           $718 = $717 << 16 >> 16;
           $719 = HEAP32[$i>>2]|0;
           $720 = ($718|0)!=($719|0);
           if (!($720)) {
            break L202;
           }
           $721 = HEAP32[$k>>2]|0;
           $722 = (($721) + 1)|0;
           HEAP32[$k>>2] = $722;
          }
         }
        } while(0);
        $723 = HEAP32[$v>>2]|0;
        $724 = HEAP32[$i>>2]|0;
        $725 = HEAP32[$nHidden>>2]|0;
        $726 = (($724) - ($725))|0;
        $727 = HEAP32[$pCol>>2]|0;
        $728 = HEAP32[$727>>2]|0;
        $729 = HEAP32[$pCol>>2]|0;
        $730 = ((($729)) + 12|0);
        $731 = HEAP32[$730>>2]|0;
        $732 = ($731|0)!=(0|0);
        if ($732) {
         $733 = HEAP32[$pCol>>2]|0;
         $734 = ((($733)) + 12|0);
         $735 = HEAP32[$734>>2]|0;
         $746 = $735;
        } else {
         $746 = 17532;
        }
        $736 = HEAP32[$pCol>>2]|0;
        $737 = ((($736)) + 20|0);
        $738 = HEAP8[$737>>0]|0;
        $739 = $738&255;
        $740 = ($739|0)!=(0);
        $741 = $740 ? 1 : 0;
        $742 = HEAP32[$pCol>>2]|0;
        $743 = ((($742)) + 8|0);
        $744 = HEAP32[$743>>2]|0;
        $745 = HEAP32[$k>>2]|0;
        HEAP32[$vararg_buffer25>>2] = $726;
        $vararg_ptr28 = ((($vararg_buffer25)) + 4|0);
        HEAP32[$vararg_ptr28>>2] = $728;
        $vararg_ptr29 = ((($vararg_buffer25)) + 8|0);
        HEAP32[$vararg_ptr29>>2] = $746;
        $vararg_ptr30 = ((($vararg_buffer25)) + 12|0);
        HEAP32[$vararg_ptr30>>2] = $741;
        $vararg_ptr31 = ((($vararg_buffer25)) + 16|0);
        HEAP32[$vararg_ptr31>>2] = $744;
        $vararg_ptr32 = ((($vararg_buffer25)) + 20|0);
        HEAP32[$vararg_ptr32>>2] = $745;
        _sqlite3VdbeMultiLoad($723,1,34351,$vararg_buffer25);
        $747 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($747,33,1,6)|0);
       }
       $748 = HEAP32[$i>>2]|0;
       $749 = (($748) + 1)|0;
       HEAP32[$i>>2] = $749;
       $750 = HEAP32[$pCol>>2]|0;
       $751 = ((($750)) + 24|0);
       HEAP32[$pCol>>2] = $751;
      }
      break;
     }
     case 29:  {
      $752 = HEAP32[$0>>2]|0;
      $753 = (_sqlite3GetVdbe($752)|0);
      HEAP32[$v>>2] = $753;
      $754 = HEAP32[$0>>2]|0;
      $755 = ((($754)) + 76|0);
      HEAP32[$755>>2] = 4;
      $756 = HEAP32[$0>>2]|0;
      $757 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($756,$757);
      $758 = HEAP32[$v>>2]|0;
      _setAllColumnNames($758,4,3376);
      $759 = HEAP32[$pDb>>2]|0;
      $760 = ((($759)) + 12|0);
      $761 = HEAP32[$760>>2]|0;
      $762 = ((($761)) + 8|0);
      $763 = ((($762)) + 8|0);
      $764 = HEAP32[$763>>2]|0;
      HEAP32[$i12>>2] = $764;
      while(1) {
       $765 = HEAP32[$i12>>2]|0;
       $766 = ($765|0)!=(0|0);
       if (!($766)) {
        break L21;
       }
       $767 = HEAP32[$i12>>2]|0;
       $768 = ((($767)) + 8|0);
       $769 = HEAP32[$768>>2]|0;
       HEAP32[$pTab13>>2] = $769;
       $770 = HEAP32[$v>>2]|0;
       $771 = HEAP32[$pTab13>>2]|0;
       $772 = HEAP32[$771>>2]|0;
       $773 = HEAP32[$pTab13>>2]|0;
       $774 = ((($773)) + 40|0);
       $775 = HEAP16[$774>>1]|0;
       $776 = (_sqlite3LogEstToInt($775)|0);
       $777 = tempRet0;
       $778 = HEAP32[$pTab13>>2]|0;
       $779 = ((($778)) + 38|0);
       $780 = HEAP16[$779>>1]|0;
       $781 = (_sqlite3LogEstToInt($780)|0);
       $782 = tempRet0;
       HEAP32[$vararg_buffer33>>2] = $772;
       $vararg_ptr36 = ((($vararg_buffer33)) + 4|0);
       HEAP32[$vararg_ptr36>>2] = 0;
       $vararg_ptr37 = ((($vararg_buffer33)) + 8|0);
       HEAP32[$vararg_ptr37>>2] = $776;
       $vararg_ptr38 = ((($vararg_buffer33)) + 12|0);
       HEAP32[$vararg_ptr38>>2] = $781;
       _sqlite3VdbeMultiLoad($770,1,34371,$vararg_buffer33);
       $783 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($783,33,1,4)|0);
       $784 = HEAP32[$pTab13>>2]|0;
       $785 = ((($784)) + 8|0);
       $786 = HEAP32[$785>>2]|0;
       HEAP32[$pIdx>>2] = $786;
       while(1) {
        $787 = HEAP32[$pIdx>>2]|0;
        $788 = ($787|0)!=(0|0);
        if (!($788)) {
         break;
        }
        $789 = HEAP32[$v>>2]|0;
        $790 = HEAP32[$pIdx>>2]|0;
        $791 = HEAP32[$790>>2]|0;
        $792 = HEAP32[$pIdx>>2]|0;
        $793 = ((($792)) + 48|0);
        $794 = HEAP16[$793>>1]|0;
        $795 = (_sqlite3LogEstToInt($794)|0);
        $796 = tempRet0;
        $797 = HEAP32[$pIdx>>2]|0;
        $798 = ((($797)) + 8|0);
        $799 = HEAP32[$798>>2]|0;
        $800 = HEAP16[$799>>1]|0;
        $801 = (_sqlite3LogEstToInt($800)|0);
        $802 = tempRet0;
        HEAP32[$vararg_buffer39>>2] = $791;
        $vararg_ptr42 = ((($vararg_buffer39)) + 4|0);
        HEAP32[$vararg_ptr42>>2] = $795;
        $vararg_ptr43 = ((($vararg_buffer39)) + 8|0);
        HEAP32[$vararg_ptr43>>2] = $801;
        _sqlite3VdbeMultiLoad($789,2,34376,$vararg_buffer39);
        $803 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($803,33,1,4)|0);
        $804 = HEAP32[$pIdx>>2]|0;
        $805 = ((($804)) + 20|0);
        $806 = HEAP32[$805>>2]|0;
        HEAP32[$pIdx>>2] = $806;
       }
       $807 = HEAP32[$i12>>2]|0;
       $808 = HEAP32[$807>>2]|0;
       HEAP32[$i12>>2] = $808;
      }
      break;
     }
     case 16:  {
      $809 = HEAP32[$zRight>>2]|0;
      $810 = ($809|0)!=(0|0);
      if (!($810)) {
       break L21;
      }
      $811 = HEAP32[$db>>2]|0;
      $812 = HEAP32[$zRight>>2]|0;
      $813 = HEAP32[$zDb>>2]|0;
      $814 = (_sqlite3FindIndex($811,$812,$813)|0);
      HEAP32[$pIdx14>>2] = $814;
      $815 = HEAP32[$pIdx14>>2]|0;
      $816 = ($815|0)!=(0|0);
      if (!($816)) {
       break L21;
      }
      $817 = HEAP32[$pPragma>>2]|0;
      $818 = ((($817)) + 8|0);
      $819 = HEAP32[$818>>2]|0;
      $820 = ($819|0)!=(0);
      $821 = HEAP32[$pIdx14>>2]|0;
      if ($820) {
       $822 = ((($821)) + 52|0);
       $823 = HEAP16[$822>>1]|0;
       $824 = $823&65535;
       HEAP32[$mx>>2] = $824;
       $825 = HEAP32[$0>>2]|0;
       $826 = ((($825)) + 76|0);
       HEAP32[$826>>2] = 6;
      } else {
       $827 = ((($821)) + 50|0);
       $828 = HEAP16[$827>>1]|0;
       $829 = $828&65535;
       HEAP32[$mx>>2] = $829;
       $830 = HEAP32[$0>>2]|0;
       $831 = ((($830)) + 76|0);
       HEAP32[$831>>2] = 3;
      }
      $832 = HEAP32[$pIdx14>>2]|0;
      $833 = ((($832)) + 12|0);
      $834 = HEAP32[$833>>2]|0;
      HEAP32[$pTab15>>2] = $834;
      $835 = HEAP32[$0>>2]|0;
      $836 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($835,$836);
      $837 = HEAP32[$v>>2]|0;
      $838 = HEAP32[$0>>2]|0;
      $839 = ((($838)) + 76|0);
      $840 = HEAP32[$839>>2]|0;
      _setAllColumnNames($837,$840,3392);
      HEAP32[$i16>>2] = 0;
      while(1) {
       $841 = HEAP32[$i16>>2]|0;
       $842 = HEAP32[$mx>>2]|0;
       $843 = ($841|0)<($842|0);
       if (!($843)) {
        break L21;
       }
       $844 = HEAP32[$i16>>2]|0;
       $845 = HEAP32[$pIdx14>>2]|0;
       $846 = ((($845)) + 4|0);
       $847 = HEAP32[$846>>2]|0;
       $848 = (($847) + ($844<<1)|0);
       $849 = HEAP16[$848>>1]|0;
       HEAP16[$cnum>>1] = $849;
       $850 = HEAP32[$v>>2]|0;
       $851 = HEAP32[$i16>>2]|0;
       $852 = HEAP16[$cnum>>1]|0;
       $853 = $852 << 16 >> 16;
       $854 = HEAP16[$cnum>>1]|0;
       $855 = $854 << 16 >> 16;
       $856 = ($855|0)<(0);
       if ($856) {
        $864 = 0;
       } else {
        $857 = HEAP16[$cnum>>1]|0;
        $858 = $857 << 16 >> 16;
        $859 = HEAP32[$pTab15>>2]|0;
        $860 = ((($859)) + 4|0);
        $861 = HEAP32[$860>>2]|0;
        $862 = (($861) + (($858*24)|0)|0);
        $863 = HEAP32[$862>>2]|0;
        $864 = $863;
       }
       HEAP32[$vararg_buffer44>>2] = $851;
       $vararg_ptr47 = ((($vararg_buffer44)) + 4|0);
       HEAP32[$vararg_ptr47>>2] = $853;
       $vararg_ptr48 = ((($vararg_buffer44)) + 8|0);
       HEAP32[$vararg_ptr48>>2] = $864;
       _sqlite3VdbeMultiLoad($850,1,34400,$vararg_buffer44);
       $865 = HEAP32[$pPragma>>2]|0;
       $866 = ((($865)) + 8|0);
       $867 = HEAP32[$866>>2]|0;
       $868 = ($867|0)!=(0);
       if ($868) {
        $869 = HEAP32[$v>>2]|0;
        $870 = HEAP32[$i16>>2]|0;
        $871 = HEAP32[$pIdx14>>2]|0;
        $872 = ((($871)) + 28|0);
        $873 = HEAP32[$872>>2]|0;
        $874 = (($873) + ($870)|0);
        $875 = HEAP8[$874>>0]|0;
        $876 = $875&255;
        $877 = HEAP32[$i16>>2]|0;
        $878 = HEAP32[$pIdx14>>2]|0;
        $879 = ((($878)) + 32|0);
        $880 = HEAP32[$879>>2]|0;
        $881 = (($880) + ($877<<2)|0);
        $882 = HEAP32[$881>>2]|0;
        $883 = HEAP32[$i16>>2]|0;
        $884 = HEAP32[$pIdx14>>2]|0;
        $885 = ((($884)) + 50|0);
        $886 = HEAP16[$885>>1]|0;
        $887 = $886&65535;
        $888 = ($883|0)<($887|0);
        $889 = $888&1;
        HEAP32[$vararg_buffer49>>2] = $876;
        $vararg_ptr52 = ((($vararg_buffer49)) + 4|0);
        HEAP32[$vararg_ptr52>>2] = $882;
        $vararg_ptr53 = ((($vararg_buffer49)) + 8|0);
        HEAP32[$vararg_ptr53>>2] = $889;
        _sqlite3VdbeMultiLoad($869,4,34404,$vararg_buffer49);
       }
       $890 = HEAP32[$v>>2]|0;
       $891 = HEAP32[$0>>2]|0;
       $892 = ((($891)) + 76|0);
       $893 = HEAP32[$892>>2]|0;
       (_sqlite3VdbeAddOp2($890,33,1,$893)|0);
       $894 = HEAP32[$i16>>2]|0;
       $895 = (($894) + 1)|0;
       HEAP32[$i16>>2] = $895;
      }
      break;
     }
     case 17:  {
      $896 = HEAP32[$zRight>>2]|0;
      $897 = ($896|0)!=(0|0);
      if (!($897)) {
       break L21;
      }
      $898 = HEAP32[$db>>2]|0;
      $899 = HEAP32[$zRight>>2]|0;
      $900 = HEAP32[$zDb>>2]|0;
      $901 = (_sqlite3FindTable($898,$899,$900)|0);
      HEAP32[$pTab18>>2] = $901;
      $902 = HEAP32[$pTab18>>2]|0;
      $903 = ($902|0)!=(0|0);
      if (!($903)) {
       break L21;
      }
      $904 = HEAP32[$0>>2]|0;
      $905 = (_sqlite3GetVdbe($904)|0);
      HEAP32[$v>>2] = $905;
      $906 = HEAP32[$0>>2]|0;
      $907 = ((($906)) + 76|0);
      HEAP32[$907>>2] = 5;
      $908 = HEAP32[$0>>2]|0;
      $909 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($908,$909);
      $910 = HEAP32[$v>>2]|0;
      _setAllColumnNames($910,5,3416);
      $911 = HEAP32[$pTab18>>2]|0;
      $912 = ((($911)) + 8|0);
      $913 = HEAP32[$912>>2]|0;
      HEAP32[$pIdx17>>2] = $913;
      HEAP32[$i19>>2] = 0;
      while(1) {
       $914 = HEAP32[$pIdx17>>2]|0;
       $915 = ($914|0)!=(0|0);
       if (!($915)) {
        break L21;
       }
       ;HEAP32[$azOrigin>>2]=HEAP32[3436>>2]|0;HEAP32[$azOrigin+4>>2]=HEAP32[3436+4>>2]|0;HEAP32[$azOrigin+8>>2]=HEAP32[3436+8>>2]|0;
       $916 = HEAP32[$v>>2]|0;
       $917 = HEAP32[$i19>>2]|0;
       $918 = HEAP32[$pIdx17>>2]|0;
       $919 = HEAP32[$918>>2]|0;
       $920 = HEAP32[$pIdx17>>2]|0;
       $921 = ((($920)) + 54|0);
       $922 = HEAP8[$921>>0]|0;
       $923 = $922&255;
       $924 = ($923|0)!=(0);
       $925 = $924&1;
       $926 = HEAP32[$pIdx17>>2]|0;
       $927 = ((($926)) + 55|0);
       $928 = HEAP8[$927>>0]|0;
       $929 = $928 & 3;
       $930 = $929&255;
       $931 = (($azOrigin) + ($930<<2)|0);
       $932 = HEAP32[$931>>2]|0;
       $933 = HEAP32[$pIdx17>>2]|0;
       $934 = ((($933)) + 36|0);
       $935 = HEAP32[$934>>2]|0;
       $936 = ($935|0)!=(0|0);
       $937 = $936&1;
       HEAP32[$vararg_buffer54>>2] = $917;
       $vararg_ptr57 = ((($vararg_buffer54)) + 4|0);
       HEAP32[$vararg_ptr57>>2] = $919;
       $vararg_ptr58 = ((($vararg_buffer54)) + 8|0);
       HEAP32[$vararg_ptr58>>2] = $925;
       $vararg_ptr59 = ((($vararg_buffer54)) + 12|0);
       HEAP32[$vararg_ptr59>>2] = $932;
       $vararg_ptr60 = ((($vararg_buffer54)) + 16|0);
       HEAP32[$vararg_ptr60>>2] = $937;
       _sqlite3VdbeMultiLoad($916,1,34438,$vararg_buffer54);
       $938 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($938,33,1,5)|0);
       $939 = HEAP32[$pIdx17>>2]|0;
       $940 = ((($939)) + 20|0);
       $941 = HEAP32[$940>>2]|0;
       HEAP32[$pIdx17>>2] = $941;
       $942 = HEAP32[$i19>>2]|0;
       $943 = (($942) + 1)|0;
       HEAP32[$i19>>2] = $943;
      }
      break;
     }
     case 10:  {
      $944 = HEAP32[$0>>2]|0;
      $945 = ((($944)) + 76|0);
      HEAP32[$945>>2] = 3;
      $946 = HEAP32[$v>>2]|0;
      _setAllColumnNames($946,3,3448);
      HEAP32[$i20>>2] = 0;
      while(1) {
       $947 = HEAP32[$i20>>2]|0;
       $948 = HEAP32[$db>>2]|0;
       $949 = ((($948)) + 20|0);
       $950 = HEAP32[$949>>2]|0;
       $951 = ($947|0)<($950|0);
       if (!($951)) {
        break L21;
       }
       $952 = HEAP32[$i20>>2]|0;
       $953 = HEAP32[$db>>2]|0;
       $954 = ((($953)) + 16|0);
       $955 = HEAP32[$954>>2]|0;
       $956 = (($955) + ($952<<4)|0);
       $957 = ((($956)) + 4|0);
       $958 = HEAP32[$957>>2]|0;
       $959 = ($958|0)==(0|0);
       if (!($959)) {
        $960 = HEAP32[$v>>2]|0;
        $961 = HEAP32[$i20>>2]|0;
        $962 = HEAP32[$i20>>2]|0;
        $963 = HEAP32[$db>>2]|0;
        $964 = ((($963)) + 16|0);
        $965 = HEAP32[$964>>2]|0;
        $966 = (($965) + ($962<<4)|0);
        $967 = HEAP32[$966>>2]|0;
        $968 = HEAP32[$i20>>2]|0;
        $969 = HEAP32[$db>>2]|0;
        $970 = ((($969)) + 16|0);
        $971 = HEAP32[$970>>2]|0;
        $972 = (($971) + ($968<<4)|0);
        $973 = ((($972)) + 4|0);
        $974 = HEAP32[$973>>2]|0;
        $975 = (_sqlite3BtreeGetFilename($974)|0);
        HEAP32[$vararg_buffer61>>2] = $961;
        $vararg_ptr64 = ((($vararg_buffer61)) + 4|0);
        HEAP32[$vararg_ptr64>>2] = $967;
        $vararg_ptr65 = ((($vararg_buffer61)) + 8|0);
        HEAP32[$vararg_ptr65>>2] = $975;
        _sqlite3VdbeMultiLoad($960,1,34449,$vararg_buffer61);
        $976 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($976,33,1,3)|0);
       }
       $977 = HEAP32[$i20>>2]|0;
       $978 = (($977) + 1)|0;
       HEAP32[$i20>>2] = $978;
      }
      break;
     }
     case 7:  {
      HEAP32[$i21>>2] = 0;
      $979 = HEAP32[$0>>2]|0;
      $980 = ((($979)) + 76|0);
      HEAP32[$980>>2] = 2;
      $981 = HEAP32[$v>>2]|0;
      _setAllColumnNames($981,2,3460);
      $982 = HEAP32[$db>>2]|0;
      $983 = ((($982)) + 432|0);
      $984 = ((($983)) + 8|0);
      $985 = HEAP32[$984>>2]|0;
      HEAP32[$p>>2] = $985;
      while(1) {
       $986 = HEAP32[$p>>2]|0;
       $987 = ($986|0)!=(0|0);
       if (!($987)) {
        break L21;
       }
       $988 = HEAP32[$p>>2]|0;
       $989 = ((($988)) + 8|0);
       $990 = HEAP32[$989>>2]|0;
       HEAP32[$pColl>>2] = $990;
       $991 = HEAP32[$v>>2]|0;
       $992 = HEAP32[$i21>>2]|0;
       $993 = (($992) + 1)|0;
       HEAP32[$i21>>2] = $993;
       $994 = HEAP32[$pColl>>2]|0;
       $995 = HEAP32[$994>>2]|0;
       HEAP32[$vararg_buffer66>>2] = $992;
       $vararg_ptr69 = ((($vararg_buffer66)) + 4|0);
       HEAP32[$vararg_ptr69>>2] = $995;
       _sqlite3VdbeMultiLoad($991,1,34453,$vararg_buffer66);
       $996 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($996,33,1,2)|0);
       $997 = HEAP32[$p>>2]|0;
       $998 = HEAP32[$997>>2]|0;
       HEAP32[$p>>2] = $998;
      }
      break;
     }
     case 14:  {
      $999 = HEAP32[$zRight>>2]|0;
      $1000 = ($999|0)!=(0|0);
      if (!($1000)) {
       break L21;
      }
      $1001 = HEAP32[$db>>2]|0;
      $1002 = HEAP32[$zRight>>2]|0;
      $1003 = HEAP32[$zDb>>2]|0;
      $1004 = (_sqlite3FindTable($1001,$1002,$1003)|0);
      HEAP32[$pTab22>>2] = $1004;
      $1005 = HEAP32[$pTab22>>2]|0;
      $1006 = ($1005|0)!=(0|0);
      if (!($1006)) {
       break L21;
      }
      $1007 = HEAP32[$0>>2]|0;
      $1008 = (_sqlite3GetVdbe($1007)|0);
      HEAP32[$v>>2] = $1008;
      $1009 = HEAP32[$pTab22>>2]|0;
      $1010 = ((($1009)) + 16|0);
      $1011 = HEAP32[$1010>>2]|0;
      HEAP32[$pFK>>2] = $1011;
      $1012 = HEAP32[$pFK>>2]|0;
      $1013 = ($1012|0)!=(0|0);
      if (!($1013)) {
       break L21;
      }
      HEAP32[$i23>>2] = 0;
      $1014 = HEAP32[$0>>2]|0;
      $1015 = ((($1014)) + 76|0);
      HEAP32[$1015>>2] = 8;
      $1016 = HEAP32[$0>>2]|0;
      $1017 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($1016,$1017);
      $1018 = HEAP32[$v>>2]|0;
      _setAllColumnNames($1018,8,3468);
      while(1) {
       $1019 = HEAP32[$pFK>>2]|0;
       $1020 = ($1019|0)!=(0|0);
       if (!($1020)) {
        break L21;
       }
       HEAP32[$j>>2] = 0;
       while(1) {
        $1021 = HEAP32[$j>>2]|0;
        $1022 = HEAP32[$pFK>>2]|0;
        $1023 = ((($1022)) + 20|0);
        $1024 = HEAP32[$1023>>2]|0;
        $1025 = ($1021|0)<($1024|0);
        if (!($1025)) {
         break;
        }
        $1026 = HEAP32[$v>>2]|0;
        $1027 = HEAP32[$i23>>2]|0;
        $1028 = HEAP32[$j>>2]|0;
        $1029 = HEAP32[$pFK>>2]|0;
        $1030 = ((($1029)) + 8|0);
        $1031 = HEAP32[$1030>>2]|0;
        $1032 = HEAP32[$j>>2]|0;
        $1033 = HEAP32[$pFK>>2]|0;
        $1034 = ((($1033)) + 36|0);
        $1035 = (($1034) + ($1032<<3)|0);
        $1036 = HEAP32[$1035>>2]|0;
        $1037 = HEAP32[$pTab22>>2]|0;
        $1038 = ((($1037)) + 4|0);
        $1039 = HEAP32[$1038>>2]|0;
        $1040 = (($1039) + (($1036*24)|0)|0);
        $1041 = HEAP32[$1040>>2]|0;
        $1042 = HEAP32[$j>>2]|0;
        $1043 = HEAP32[$pFK>>2]|0;
        $1044 = ((($1043)) + 36|0);
        $1045 = (($1044) + ($1042<<3)|0);
        $1046 = ((($1045)) + 4|0);
        $1047 = HEAP32[$1046>>2]|0;
        $1048 = HEAP32[$pFK>>2]|0;
        $1049 = ((($1048)) + 25|0);
        $1050 = ((($1049)) + 1|0);
        $1051 = HEAP8[$1050>>0]|0;
        $1052 = (_actionName($1051)|0);
        $1053 = HEAP32[$pFK>>2]|0;
        $1054 = ((($1053)) + 25|0);
        $1055 = HEAP8[$1054>>0]|0;
        $1056 = (_actionName($1055)|0);
        HEAP32[$vararg_buffer70>>2] = $1027;
        $vararg_ptr73 = ((($vararg_buffer70)) + 4|0);
        HEAP32[$vararg_ptr73>>2] = $1028;
        $vararg_ptr74 = ((($vararg_buffer70)) + 8|0);
        HEAP32[$vararg_ptr74>>2] = $1031;
        $vararg_ptr75 = ((($vararg_buffer70)) + 12|0);
        HEAP32[$vararg_ptr75>>2] = $1041;
        $vararg_ptr76 = ((($vararg_buffer70)) + 16|0);
        HEAP32[$vararg_ptr76>>2] = $1047;
        $vararg_ptr77 = ((($vararg_buffer70)) + 20|0);
        HEAP32[$vararg_ptr77>>2] = $1052;
        $vararg_ptr78 = ((($vararg_buffer70)) + 24|0);
        HEAP32[$vararg_ptr78>>2] = $1056;
        $vararg_ptr79 = ((($vararg_buffer70)) + 28|0);
        HEAP32[$vararg_ptr79>>2] = 34491;
        _sqlite3VdbeMultiLoad($1026,1,34482,$vararg_buffer70);
        $1057 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($1057,33,1,8)|0);
        $1058 = HEAP32[$j>>2]|0;
        $1059 = (($1058) + 1)|0;
        HEAP32[$j>>2] = $1059;
       }
       $1060 = HEAP32[$i23>>2]|0;
       $1061 = (($1060) + 1)|0;
       HEAP32[$i23>>2] = $1061;
       $1062 = HEAP32[$pFK>>2]|0;
       $1063 = ((($1062)) + 4|0);
       $1064 = HEAP32[$1063>>2]|0;
       HEAP32[$pFK>>2] = $1064;
      }
      break;
     }
     case 13:  {
      $1065 = HEAP32[$0>>2]|0;
      $1066 = ((($1065)) + 76|0);
      $1067 = HEAP32[$1066>>2]|0;
      $1068 = (($1067) + 1)|0;
      HEAP32[$regResult>>2] = $1068;
      $1069 = HEAP32[$0>>2]|0;
      $1070 = ((($1069)) + 76|0);
      $1071 = HEAP32[$1070>>2]|0;
      $1072 = (($1071) + 4)|0;
      HEAP32[$1070>>2] = $1072;
      $1073 = HEAP32[$0>>2]|0;
      $1074 = ((($1073)) + 76|0);
      $1075 = HEAP32[$1074>>2]|0;
      $1076 = (($1075) + 1)|0;
      HEAP32[$1074>>2] = $1076;
      HEAP32[$regKey>>2] = $1076;
      $1077 = HEAP32[$0>>2]|0;
      $1078 = ((($1077)) + 76|0);
      $1079 = HEAP32[$1078>>2]|0;
      $1080 = (($1079) + 1)|0;
      HEAP32[$1078>>2] = $1080;
      HEAP32[$regRow>>2] = $1080;
      $1081 = HEAP32[$0>>2]|0;
      $1082 = (_sqlite3GetVdbe($1081)|0);
      HEAP32[$v>>2] = $1082;
      $1083 = HEAP32[$v>>2]|0;
      _setAllColumnNames($1083,4,3500);
      $1084 = HEAP32[$0>>2]|0;
      $1085 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($1084,$1085);
      $1086 = HEAP32[$iDb>>2]|0;
      $1087 = HEAP32[$db>>2]|0;
      $1088 = ((($1087)) + 16|0);
      $1089 = HEAP32[$1088>>2]|0;
      $1090 = (($1089) + ($1086<<4)|0);
      $1091 = ((($1090)) + 12|0);
      $1092 = HEAP32[$1091>>2]|0;
      $1093 = ((($1092)) + 8|0);
      $1094 = ((($1093)) + 8|0);
      $1095 = HEAP32[$1094>>2]|0;
      HEAP32[$k29>>2] = $1095;
      while(1) {
       $1096 = HEAP32[$k29>>2]|0;
       $1097 = ($1096|0)!=(0|0);
       if (!($1097)) {
        break L21;
       }
       $1098 = HEAP32[$zRight>>2]|0;
       $1099 = ($1098|0)!=(0|0);
       if ($1099) {
        $1100 = HEAP32[$0>>2]|0;
        $1101 = HEAP32[$zRight>>2]|0;
        $1102 = HEAP32[$zDb>>2]|0;
        $1103 = (_sqlite3LocateTable($1100,0,$1101,$1102)|0);
        HEAP32[$pTab25>>2] = $1103;
        HEAP32[$k29>>2] = 0;
       } else {
        $1104 = HEAP32[$k29>>2]|0;
        $1105 = ((($1104)) + 8|0);
        $1106 = HEAP32[$1105>>2]|0;
        HEAP32[$pTab25>>2] = $1106;
        $1107 = HEAP32[$k29>>2]|0;
        $1108 = HEAP32[$1107>>2]|0;
        HEAP32[$k29>>2] = $1108;
       }
       $1109 = HEAP32[$pTab25>>2]|0;
       $1110 = ($1109|0)==(0|0);
       if ($1110) {
        continue;
       }
       $1111 = HEAP32[$pTab25>>2]|0;
       $1112 = ((($1111)) + 16|0);
       $1113 = HEAP32[$1112>>2]|0;
       $1114 = ($1113|0)==(0|0);
       if ($1114) {
        continue;
       }
       $1115 = HEAP32[$0>>2]|0;
       $1116 = HEAP32[$iDb>>2]|0;
       $1117 = HEAP32[$pTab25>>2]|0;
       $1118 = ((($1117)) + 28|0);
       $1119 = HEAP32[$1118>>2]|0;
       $1120 = HEAP32[$pTab25>>2]|0;
       $1121 = HEAP32[$1120>>2]|0;
       _sqlite3TableLock($1115,$1116,$1119,0,$1121);
       $1122 = HEAP32[$pTab25>>2]|0;
       $1123 = ((($1122)) + 34|0);
       $1124 = HEAP16[$1123>>1]|0;
       $1125 = $1124 << 16 >> 16;
       $1126 = HEAP32[$regRow>>2]|0;
       $1127 = (($1125) + ($1126))|0;
       $1128 = HEAP32[$0>>2]|0;
       $1129 = ((($1128)) + 76|0);
       $1130 = HEAP32[$1129>>2]|0;
       $1131 = ($1127|0)>($1130|0);
       if ($1131) {
        $1132 = HEAP32[$pTab25>>2]|0;
        $1133 = ((($1132)) + 34|0);
        $1134 = HEAP16[$1133>>1]|0;
        $1135 = $1134 << 16 >> 16;
        $1136 = HEAP32[$regRow>>2]|0;
        $1137 = (($1135) + ($1136))|0;
        $1138 = HEAP32[$0>>2]|0;
        $1139 = ((($1138)) + 76|0);
        HEAP32[$1139>>2] = $1137;
       }
       $1140 = HEAP32[$0>>2]|0;
       $1141 = HEAP32[$iDb>>2]|0;
       $1142 = HEAP32[$pTab25>>2]|0;
       _sqlite3OpenTable($1140,0,$1141,$1142,54);
       $1143 = HEAP32[$v>>2]|0;
       $1144 = HEAP32[$regResult>>2]|0;
       $1145 = HEAP32[$pTab25>>2]|0;
       $1146 = HEAP32[$1145>>2]|0;
       (_sqlite3VdbeLoadString($1143,$1144,$1146)|0);
       HEAP32[$i27>>2] = 1;
       $1147 = HEAP32[$pTab25>>2]|0;
       $1148 = ((($1147)) + 16|0);
       $1149 = HEAP32[$1148>>2]|0;
       HEAP32[$pFK24>>2] = $1149;
       L279: while(1) {
        $1150 = HEAP32[$pFK24>>2]|0;
        $1151 = ($1150|0)!=(0|0);
        if (!($1151)) {
         break;
        }
        $1152 = HEAP32[$db>>2]|0;
        $1153 = HEAP32[$pFK24>>2]|0;
        $1154 = ((($1153)) + 8|0);
        $1155 = HEAP32[$1154>>2]|0;
        $1156 = HEAP32[$zDb>>2]|0;
        $1157 = (_sqlite3FindTable($1152,$1155,$1156)|0);
        HEAP32[$pParent>>2] = $1157;
        $1158 = HEAP32[$pParent>>2]|0;
        $1159 = ($1158|0)==(0|0);
        do {
         if (!($1159)) {
          HEAP32[$pIdx26>>2] = 0;
          $1160 = HEAP32[$0>>2]|0;
          $1161 = HEAP32[$iDb>>2]|0;
          $1162 = HEAP32[$pParent>>2]|0;
          $1163 = ((($1162)) + 28|0);
          $1164 = HEAP32[$1163>>2]|0;
          $1165 = HEAP32[$pParent>>2]|0;
          $1166 = HEAP32[$1165>>2]|0;
          _sqlite3TableLock($1160,$1161,$1164,0,$1166);
          $1167 = HEAP32[$0>>2]|0;
          $1168 = HEAP32[$pParent>>2]|0;
          $1169 = HEAP32[$pFK24>>2]|0;
          $1170 = (_sqlite3FkLocateIndex($1167,$1168,$1169,$pIdx26,0)|0);
          HEAP32[$x>>2] = $1170;
          $1171 = HEAP32[$x>>2]|0;
          $1172 = ($1171|0)==(0);
          if (!($1172)) {
           label = 207;
           break L279;
          }
          $1173 = HEAP32[$pIdx26>>2]|0;
          $1174 = ($1173|0)==(0|0);
          if ($1174) {
           $1175 = HEAP32[$0>>2]|0;
           $1176 = HEAP32[$i27>>2]|0;
           $1177 = HEAP32[$iDb>>2]|0;
           $1178 = HEAP32[$pParent>>2]|0;
           _sqlite3OpenTable($1175,$1176,$1177,$1178,54);
           break;
          } else {
           $1179 = HEAP32[$v>>2]|0;
           $1180 = HEAP32[$i27>>2]|0;
           $1181 = HEAP32[$pIdx26>>2]|0;
           $1182 = ((($1181)) + 44|0);
           $1183 = HEAP32[$1182>>2]|0;
           $1184 = HEAP32[$iDb>>2]|0;
           (_sqlite3VdbeAddOp3($1179,54,$1180,$1183,$1184)|0);
           $1185 = HEAP32[$0>>2]|0;
           $1186 = HEAP32[$pIdx26>>2]|0;
           _sqlite3VdbeSetP4KeyInfo($1185,$1186);
           break;
          }
         }
        } while(0);
        $1187 = HEAP32[$i27>>2]|0;
        $1188 = (($1187) + 1)|0;
        HEAP32[$i27>>2] = $1188;
        $1189 = HEAP32[$pFK24>>2]|0;
        $1190 = ((($1189)) + 4|0);
        $1191 = HEAP32[$1190>>2]|0;
        HEAP32[$pFK24>>2] = $1191;
       }
       if ((label|0) == 207) {
        label = 0;
        HEAP32[$k29>>2] = 0;
       }
       $1192 = HEAP32[$pFK24>>2]|0;
       $1193 = ($1192|0)!=(0|0);
       if ($1193) {
        break L21;
       }
       $1194 = HEAP32[$0>>2]|0;
       $1195 = ((($1194)) + 72|0);
       $1196 = HEAP32[$1195>>2]|0;
       $1197 = HEAP32[$i27>>2]|0;
       $1198 = ($1196|0)<($1197|0);
       if ($1198) {
        $1199 = HEAP32[$i27>>2]|0;
        $1200 = HEAP32[$0>>2]|0;
        $1201 = ((($1200)) + 72|0);
        HEAP32[$1201>>2] = $1199;
       }
       $1202 = HEAP32[$v>>2]|0;
       $1203 = (_sqlite3VdbeAddOp1($1202,108,0)|0);
       HEAP32[$addrTop>>2] = $1203;
       HEAP32[$i27>>2] = 1;
       $1204 = HEAP32[$pTab25>>2]|0;
       $1205 = ((($1204)) + 16|0);
       $1206 = HEAP32[$1205>>2]|0;
       HEAP32[$pFK24>>2] = $1206;
       while(1) {
        $1207 = HEAP32[$pFK24>>2]|0;
        $1208 = ($1207|0)!=(0|0);
        if (!($1208)) {
         break;
        }
        $1209 = HEAP32[$db>>2]|0;
        $1210 = HEAP32[$pFK24>>2]|0;
        $1211 = ((($1210)) + 8|0);
        $1212 = HEAP32[$1211>>2]|0;
        $1213 = HEAP32[$zDb>>2]|0;
        $1214 = (_sqlite3FindTable($1209,$1212,$1213)|0);
        HEAP32[$pParent>>2] = $1214;
        HEAP32[$pIdx26>>2] = 0;
        HEAP32[$aiCols>>2] = 0;
        $1215 = HEAP32[$pParent>>2]|0;
        $1216 = ($1215|0)!=(0|0);
        if ($1216) {
         $1217 = HEAP32[$0>>2]|0;
         $1218 = HEAP32[$pParent>>2]|0;
         $1219 = HEAP32[$pFK24>>2]|0;
         $1220 = (_sqlite3FkLocateIndex($1217,$1218,$1219,$pIdx26,$aiCols)|0);
         HEAP32[$x>>2] = $1220;
        }
        $1221 = HEAP32[$v>>2]|0;
        $1222 = (_sqlite3VdbeMakeLabel($1221)|0);
        HEAP32[$addrOk>>2] = $1222;
        $1223 = HEAP32[$pParent>>2]|0;
        $1224 = ($1223|0)!=(0|0);
        $1225 = HEAP32[$pIdx26>>2]|0;
        $1226 = ($1225|0)==(0|0);
        $or$cond11 = $1224 & $1226;
        do {
         if ($or$cond11) {
          $1227 = HEAP32[$pFK24>>2]|0;
          $1228 = ((($1227)) + 36|0);
          $1229 = HEAP32[$1228>>2]|0;
          HEAP32[$iKey>>2] = $1229;
          $1230 = HEAP32[$iKey>>2]|0;
          $1231 = HEAP32[$pTab25>>2]|0;
          $1232 = ((($1231)) + 32|0);
          $1233 = HEAP16[$1232>>1]|0;
          $1234 = $1233 << 16 >> 16;
          $1235 = ($1230|0)!=($1234|0);
          $1236 = HEAP32[$v>>2]|0;
          if ($1235) {
           $1237 = HEAP32[$iKey>>2]|0;
           $1238 = HEAP32[$regRow>>2]|0;
           (_sqlite3VdbeAddOp3($1236,47,0,$1237,$1238)|0);
           $1239 = HEAP32[$v>>2]|0;
           $1240 = HEAP32[$pTab25>>2]|0;
           $1241 = HEAP32[$iKey>>2]|0;
           $1242 = HEAP32[$regRow>>2]|0;
           _sqlite3ColumnDefault($1239,$1240,$1241,$1242);
           $1243 = HEAP32[$v>>2]|0;
           $1244 = HEAP32[$regRow>>2]|0;
           $1245 = HEAP32[$addrOk>>2]|0;
           (_sqlite3VdbeAddOp2($1243,76,$1244,$1245)|0);
           $1246 = HEAP32[$v>>2]|0;
           $1247 = HEAP32[$regRow>>2]|0;
           $1248 = HEAP32[$v>>2]|0;
           $1249 = (_sqlite3VdbeCurrentAddr($1248)|0);
           $1250 = (($1249) + 3)|0;
           (_sqlite3VdbeAddOp2($1246,38,$1247,$1250)|0);
          } else {
           $1251 = HEAP32[$regRow>>2]|0;
           (_sqlite3VdbeAddOp2($1236,103,0,$1251)|0);
          }
          $1252 = HEAP32[$v>>2]|0;
          $1253 = HEAP32[$i27>>2]|0;
          $1254 = HEAP32[$regRow>>2]|0;
          (_sqlite3VdbeAddOp3($1252,70,$1253,0,$1254)|0);
          $1255 = HEAP32[$v>>2]|0;
          $1256 = HEAP32[$addrOk>>2]|0;
          (_sqlite3VdbeGoto($1255,$1256)|0);
          $1257 = HEAP32[$v>>2]|0;
          $1258 = HEAP32[$v>>2]|0;
          $1259 = (_sqlite3VdbeCurrentAddr($1258)|0);
          $1260 = (($1259) - 2)|0;
          _sqlite3VdbeJumpHere($1257,$1260);
         } else {
          HEAP32[$j28>>2] = 0;
          while(1) {
           $1261 = HEAP32[$j28>>2]|0;
           $1262 = HEAP32[$pFK24>>2]|0;
           $1263 = ((($1262)) + 20|0);
           $1264 = HEAP32[$1263>>2]|0;
           $1265 = ($1261|0)<($1264|0);
           if (!($1265)) {
            break;
           }
           $1266 = HEAP32[$v>>2]|0;
           $1267 = HEAP32[$pTab25>>2]|0;
           $1268 = HEAP32[$aiCols>>2]|0;
           $1269 = ($1268|0)!=(0|0);
           $1270 = HEAP32[$j28>>2]|0;
           if ($1269) {
            $1271 = HEAP32[$aiCols>>2]|0;
            $1272 = (($1271) + ($1270<<2)|0);
            $1273 = HEAP32[$1272>>2]|0;
            $1281 = $1273;
           } else {
            $1274 = HEAP32[$pFK24>>2]|0;
            $1275 = ((($1274)) + 36|0);
            $1276 = (($1275) + ($1270<<3)|0);
            $1277 = HEAP32[$1276>>2]|0;
            $1281 = $1277;
           }
           $1278 = HEAP32[$regRow>>2]|0;
           $1279 = HEAP32[$j28>>2]|0;
           $1280 = (($1278) + ($1279))|0;
           _sqlite3ExprCodeGetColumnOfTable($1266,$1267,0,$1281,$1280);
           $1282 = HEAP32[$v>>2]|0;
           $1283 = HEAP32[$regRow>>2]|0;
           $1284 = HEAP32[$j28>>2]|0;
           $1285 = (($1283) + ($1284))|0;
           $1286 = HEAP32[$addrOk>>2]|0;
           (_sqlite3VdbeAddOp2($1282,76,$1285,$1286)|0);
           $1287 = HEAP32[$j28>>2]|0;
           $1288 = (($1287) + 1)|0;
           HEAP32[$j28>>2] = $1288;
          }
          $1289 = HEAP32[$pParent>>2]|0;
          $1290 = ($1289|0)!=(0|0);
          if (!($1290)) {
           break;
          }
          $1291 = HEAP32[$v>>2]|0;
          $1292 = HEAP32[$regRow>>2]|0;
          $1293 = HEAP32[$pFK24>>2]|0;
          $1294 = ((($1293)) + 20|0);
          $1295 = HEAP32[$1294>>2]|0;
          $1296 = HEAP32[$regKey>>2]|0;
          $1297 = HEAP32[$db>>2]|0;
          $1298 = HEAP32[$pIdx26>>2]|0;
          $1299 = (_sqlite3IndexAffinityStr($1297,$1298)|0);
          $1300 = HEAP32[$pFK24>>2]|0;
          $1301 = ((($1300)) + 20|0);
          $1302 = HEAP32[$1301>>2]|0;
          (_sqlite3VdbeAddOp4($1291,49,$1292,$1295,$1296,$1299,$1302)|0);
          $1303 = HEAP32[$v>>2]|0;
          $1304 = HEAP32[$i27>>2]|0;
          $1305 = HEAP32[$addrOk>>2]|0;
          $1306 = HEAP32[$regKey>>2]|0;
          (_sqlite3VdbeAddOp4Int($1303,69,$1304,$1305,$1306,0)|0);
         }
        } while(0);
        $1307 = HEAP32[$v>>2]|0;
        $1308 = HEAP32[$regResult>>2]|0;
        $1309 = (($1308) + 1)|0;
        (_sqlite3VdbeAddOp2($1307,103,0,$1309)|0);
        $1310 = HEAP32[$v>>2]|0;
        $1311 = HEAP32[$regResult>>2]|0;
        $1312 = (($1311) + 2)|0;
        $1313 = HEAP32[$pFK24>>2]|0;
        $1314 = ((($1313)) + 8|0);
        $1315 = HEAP32[$1314>>2]|0;
        $1316 = HEAP32[$i27>>2]|0;
        $1317 = (($1316) - 1)|0;
        HEAP32[$vararg_buffer80>>2] = $1315;
        $vararg_ptr83 = ((($vararg_buffer80)) + 4|0);
        HEAP32[$vararg_ptr83>>2] = $1317;
        _sqlite3VdbeMultiLoad($1310,$1312,34508,$vararg_buffer80);
        $1318 = HEAP32[$v>>2]|0;
        $1319 = HEAP32[$regResult>>2]|0;
        (_sqlite3VdbeAddOp2($1318,33,$1319,4)|0);
        $1320 = HEAP32[$v>>2]|0;
        $1321 = HEAP32[$addrOk>>2]|0;
        _sqlite3VdbeResolveLabel($1320,$1321);
        $1322 = HEAP32[$db>>2]|0;
        $1323 = HEAP32[$aiCols>>2]|0;
        _sqlite3DbFree($1322,$1323);
        $1324 = HEAP32[$i27>>2]|0;
        $1325 = (($1324) + 1)|0;
        HEAP32[$i27>>2] = $1325;
        $1326 = HEAP32[$pFK24>>2]|0;
        $1327 = ((($1326)) + 4|0);
        $1328 = HEAP32[$1327>>2]|0;
        HEAP32[$pFK24>>2] = $1328;
       }
       $1329 = HEAP32[$v>>2]|0;
       $1330 = HEAP32[$addrTop>>2]|0;
       $1331 = (($1330) + 1)|0;
       (_sqlite3VdbeAddOp2($1329,7,0,$1331)|0);
       $1332 = HEAP32[$v>>2]|0;
       $1333 = HEAP32[$addrTop>>2]|0;
       _sqlite3VdbeJumpHere($1332,$1333);
      }
      break;
     }
     case 6:  {
      $1334 = HEAP32[$zRight>>2]|0;
      $1335 = ($1334|0)!=(0|0);
      if (!($1335)) {
       break L21;
      }
      $1336 = HEAP32[$db>>2]|0;
      $1337 = HEAP32[$zRight>>2]|0;
      $1338 = (_sqlite3GetBoolean($1337,0)|0);
      $1339 = $1338&255;
      _sqlite3RegisterLikeFunctions($1336,$1339);
      break L21;
      break;
     }
     case 18:  {
      $1340 = HEAP32[$zLeft>>2]|0;
      $1341 = HEAP8[$1340>>0]|0;
      $1342 = $1341&255;
      $1343 = (16974 + ($1342)|0);
      $1344 = HEAP8[$1343>>0]|0;
      $1345 = $1344&255;
      $1346 = ($1345|0)==(113);
      $1347 = $1346&1;
      HEAP32[$isQuick>>2] = $1347;
      $1348 = HEAP32[$2>>2]|0;
      $1349 = HEAP32[$1348>>2]|0;
      $1350 = ($1349|0)==(0|0);
      if ($1350) {
       HEAP32[$iDb>>2] = -1;
      }
      $1351 = HEAP32[$0>>2]|0;
      $1352 = ((($1351)) + 76|0);
      HEAP32[$1352>>2] = 6;
      $1353 = HEAP32[$v>>2]|0;
      _setOneColumnName($1353,34511);
      HEAP32[$mxErr>>2] = 100;
      $1354 = HEAP32[$zRight>>2]|0;
      $1355 = ($1354|0)!=(0|0);
      if ($1355) {
       $1356 = HEAP32[$zRight>>2]|0;
       (_sqlite3GetInt32($1356,$mxErr)|0);
       $1357 = HEAP32[$mxErr>>2]|0;
       $1358 = ($1357|0)<=(0);
       if ($1358) {
        HEAP32[$mxErr>>2] = 100;
       }
      }
      $1359 = HEAP32[$v>>2]|0;
      $1360 = HEAP32[$mxErr>>2]|0;
      (_sqlite3VdbeAddOp2($1359,22,$1360,1)|0);
      HEAP32[$i30>>2] = 0;
      while(1) {
       $1361 = HEAP32[$i30>>2]|0;
       $1362 = HEAP32[$db>>2]|0;
       $1363 = ((($1362)) + 20|0);
       $1364 = HEAP32[$1363>>2]|0;
       $1365 = ($1361|0)<($1364|0);
       if (!($1365)) {
        break;
       }
       HEAP32[$cnt>>2] = 0;
       $1366 = HEAP32[$iDb>>2]|0;
       $1367 = ($1366|0)>=(0);
       if ($1367) {
        $1368 = HEAP32[$i30>>2]|0;
        $1369 = HEAP32[$iDb>>2]|0;
        $1370 = ($1368|0)!=($1369|0);
        if (!($1370)) {
         label = 242;
        }
       } else {
        label = 242;
       }
       L335: do {
        if ((label|0) == 242) {
         label = 0;
         $1371 = HEAP32[$0>>2]|0;
         $1372 = HEAP32[$i30>>2]|0;
         _sqlite3CodeVerifySchema($1371,$1372);
         $1373 = HEAP32[$v>>2]|0;
         $1374 = (_sqlite3VdbeAddOp1($1373,138,1)|0);
         HEAP32[$addr32>>2] = $1374;
         $1375 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp2($1375,21,0,0)|0);
         $1376 = HEAP32[$v>>2]|0;
         $1377 = HEAP32[$addr32>>2]|0;
         _sqlite3VdbeJumpHere($1376,$1377);
         $1378 = HEAP32[$i30>>2]|0;
         $1379 = HEAP32[$db>>2]|0;
         $1380 = ((($1379)) + 16|0);
         $1381 = HEAP32[$1380>>2]|0;
         $1382 = (($1381) + ($1378<<4)|0);
         $1383 = ((($1382)) + 12|0);
         $1384 = HEAP32[$1383>>2]|0;
         $1385 = ((($1384)) + 8|0);
         HEAP32[$pTbls>>2] = $1385;
         $1386 = HEAP32[$pTbls>>2]|0;
         $1387 = ((($1386)) + 8|0);
         $1388 = HEAP32[$1387>>2]|0;
         HEAP32[$x33>>2] = $1388;
         while(1) {
          $1389 = HEAP32[$x33>>2]|0;
          $1390 = ($1389|0)!=(0|0);
          if (!($1390)) {
           break;
          }
          $1391 = HEAP32[$x33>>2]|0;
          $1392 = ((($1391)) + 8|0);
          $1393 = HEAP32[$1392>>2]|0;
          HEAP32[$pTab34>>2] = $1393;
          $1394 = HEAP32[$pTab34>>2]|0;
          $1395 = ((($1394)) + 42|0);
          $1396 = HEAP8[$1395>>0]|0;
          $1397 = $1396&255;
          $1398 = $1397 & 32;
          $1399 = ($1398|0)==(0);
          if ($1399) {
           $1400 = HEAP32[$v>>2]|0;
           $1401 = HEAP32[$pTab34>>2]|0;
           $1402 = ((($1401)) + 28|0);
           $1403 = HEAP32[$1402>>2]|0;
           $1404 = HEAP32[$cnt>>2]|0;
           $1405 = (2 + ($1404))|0;
           (_sqlite3VdbeAddOp2($1400,22,$1403,$1405)|0);
           $1406 = HEAP32[$cnt>>2]|0;
           $1407 = (($1406) + 1)|0;
           HEAP32[$cnt>>2] = $1407;
          }
          $1408 = HEAP32[$pTab34>>2]|0;
          $1409 = ((($1408)) + 8|0);
          $1410 = HEAP32[$1409>>2]|0;
          HEAP32[$pIdx35>>2] = $1410;
          while(1) {
           $1411 = HEAP32[$pIdx35>>2]|0;
           $1412 = ($1411|0)!=(0|0);
           if (!($1412)) {
            break;
           }
           $1413 = HEAP32[$v>>2]|0;
           $1414 = HEAP32[$pIdx35>>2]|0;
           $1415 = ((($1414)) + 44|0);
           $1416 = HEAP32[$1415>>2]|0;
           $1417 = HEAP32[$cnt>>2]|0;
           $1418 = (2 + ($1417))|0;
           (_sqlite3VdbeAddOp2($1413,22,$1416,$1418)|0);
           $1419 = HEAP32[$cnt>>2]|0;
           $1420 = (($1419) + 1)|0;
           HEAP32[$cnt>>2] = $1420;
           $1421 = HEAP32[$pIdx35>>2]|0;
           $1422 = ((($1421)) + 20|0);
           $1423 = HEAP32[$1422>>2]|0;
           HEAP32[$pIdx35>>2] = $1423;
          }
          $1424 = HEAP32[$x33>>2]|0;
          $1425 = HEAP32[$1424>>2]|0;
          HEAP32[$x33>>2] = $1425;
         }
         $1426 = HEAP32[$0>>2]|0;
         $1427 = ((($1426)) + 76|0);
         $1428 = HEAP32[$1427>>2]|0;
         $1429 = HEAP32[$cnt>>2]|0;
         $1430 = (($1429) + 8)|0;
         $1431 = ($1428|0)>($1430|0);
         if ($1431) {
          $1432 = HEAP32[$0>>2]|0;
          $1433 = ((($1432)) + 76|0);
          $1434 = HEAP32[$1433>>2]|0;
          $1439 = $1434;
         } else {
          $1435 = HEAP32[$cnt>>2]|0;
          $1436 = (($1435) + 8)|0;
          $1439 = $1436;
         }
         $1437 = HEAP32[$0>>2]|0;
         $1438 = ((($1437)) + 76|0);
         HEAP32[$1438>>2] = $1439;
         $1440 = HEAP32[$v>>2]|0;
         $1441 = HEAP32[$cnt>>2]|0;
         (_sqlite3VdbeAddOp3($1440,128,2,$1441,1)|0);
         $1442 = HEAP32[$v>>2]|0;
         $1443 = HEAP32[$i30>>2]|0;
         $1444 = $1443&255;
         _sqlite3VdbeChangeP5($1442,$1444);
         $1445 = HEAP32[$v>>2]|0;
         $1446 = (_sqlite3VdbeAddOp1($1445,76,2)|0);
         HEAP32[$addr32>>2] = $1446;
         $1447 = HEAP32[$v>>2]|0;
         $1448 = HEAP32[$db>>2]|0;
         $1449 = HEAP32[$i30>>2]|0;
         $1450 = HEAP32[$db>>2]|0;
         $1451 = ((($1450)) + 16|0);
         $1452 = HEAP32[$1451>>2]|0;
         $1453 = (($1452) + ($1449<<4)|0);
         $1454 = HEAP32[$1453>>2]|0;
         HEAP32[$vararg_buffer84>>2] = $1454;
         $1455 = (_sqlite3MPrintf($1448,34527,$vararg_buffer84)|0);
         (_sqlite3VdbeAddOp4($1447,97,0,3,0,$1455,-1)|0);
         $1456 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp3($1456,29,2,4,1)|0);
         $1457 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp3($1457,94,4,3,2)|0);
         $1458 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp2($1458,33,2,1)|0);
         $1459 = HEAP32[$v>>2]|0;
         $1460 = HEAP32[$addr32>>2]|0;
         _sqlite3VdbeJumpHere($1459,$1460);
         $1461 = HEAP32[$pTbls>>2]|0;
         $1462 = ((($1461)) + 8|0);
         $1463 = HEAP32[$1462>>2]|0;
         HEAP32[$x33>>2] = $1463;
         while(1) {
          $1464 = HEAP32[$x33>>2]|0;
          $1465 = ($1464|0)!=(0|0);
          if (!($1465)) {
           break L335;
          }
          $1466 = HEAP32[$isQuick>>2]|0;
          $1467 = ($1466|0)!=(0);
          $1468 = $1467 ^ 1;
          if (!($1468)) {
           break L335;
          }
          $1469 = HEAP32[$x33>>2]|0;
          $1470 = ((($1469)) + 8|0);
          $1471 = HEAP32[$1470>>2]|0;
          HEAP32[$pTab36>>2] = $1471;
          HEAP32[$pPrior>>2] = 0;
          HEAP32[$r1>>2] = -1;
          $1472 = HEAP32[$pTab36>>2]|0;
          $1473 = ((($1472)) + 8|0);
          $1474 = HEAP32[$1473>>2]|0;
          $1475 = ($1474|0)==(0|0);
          L356: do {
           if (!($1475)) {
            $1476 = HEAP32[$pTab36>>2]|0;
            $1477 = ((($1476)) + 42|0);
            $1478 = HEAP8[$1477>>0]|0;
            $1479 = $1478&255;
            $1480 = $1479 & 32;
            $1481 = ($1480|0)==(0);
            if ($1481) {
             $1484 = 0;
            } else {
             $1482 = HEAP32[$pTab36>>2]|0;
             $1483 = (_sqlite3PrimaryKeyIndex($1482)|0);
             $1484 = $1483;
            }
            HEAP32[$pPk38>>2] = $1484;
            $1485 = HEAP32[$v>>2]|0;
            $1486 = (_sqlite3VdbeAddOp1($1485,138,1)|0);
            HEAP32[$addr32>>2] = $1486;
            $1487 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp2($1487,21,0,0)|0);
            $1488 = HEAP32[$v>>2]|0;
            $1489 = HEAP32[$addr32>>2]|0;
            _sqlite3VdbeJumpHere($1488,$1489);
            $1490 = HEAP32[$0>>2]|0;
            _sqlite3ExprCacheClear($1490);
            $1491 = HEAP32[$0>>2]|0;
            $1492 = HEAP32[$pTab36>>2]|0;
            (_sqlite3OpenTableAndIndices($1491,$1492,54,0,1,0,$iDataCur,$iIdxCur)|0);
            $1493 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp2($1493,22,0,7)|0);
            HEAP32[$j31>>2] = 0;
            $1494 = HEAP32[$pTab36>>2]|0;
            $1495 = ((($1494)) + 8|0);
            $1496 = HEAP32[$1495>>2]|0;
            HEAP32[$pIdx37>>2] = $1496;
            while(1) {
             $1497 = HEAP32[$pIdx37>>2]|0;
             $1498 = ($1497|0)!=(0|0);
             if (!($1498)) {
              break;
             }
             $1499 = HEAP32[$v>>2]|0;
             $1500 = HEAP32[$j31>>2]|0;
             $1501 = (8 + ($1500))|0;
             (_sqlite3VdbeAddOp2($1499,22,0,$1501)|0);
             $1502 = HEAP32[$pIdx37>>2]|0;
             $1503 = ((($1502)) + 20|0);
             $1504 = HEAP32[$1503>>2]|0;
             HEAP32[$pIdx37>>2] = $1504;
             $1505 = HEAP32[$j31>>2]|0;
             $1506 = (($1505) + 1)|0;
             HEAP32[$j31>>2] = $1506;
            }
            $1507 = HEAP32[$0>>2]|0;
            $1508 = ((($1507)) + 76|0);
            $1509 = HEAP32[$1508>>2]|0;
            $1510 = HEAP32[$j31>>2]|0;
            $1511 = (8 + ($1510))|0;
            $1512 = ($1509|0)>($1511|0);
            if ($1512) {
             $1513 = HEAP32[$0>>2]|0;
             $1514 = ((($1513)) + 76|0);
             $1515 = HEAP32[$1514>>2]|0;
             $1520 = $1515;
            } else {
             $1516 = HEAP32[$j31>>2]|0;
             $1517 = (8 + ($1516))|0;
             $1520 = $1517;
            }
            $1518 = HEAP32[$0>>2]|0;
            $1519 = ((($1518)) + 76|0);
            HEAP32[$1519>>2] = $1520;
            $1521 = HEAP32[$v>>2]|0;
            $1522 = HEAP32[$iDataCur>>2]|0;
            (_sqlite3VdbeAddOp2($1521,108,$1522,0)|0);
            $1523 = HEAP32[$v>>2]|0;
            $1524 = (_sqlite3VdbeAddOp2($1523,37,7,1)|0);
            HEAP32[$loopTop>>2] = $1524;
            HEAP32[$j31>>2] = 0;
            while(1) {
             $1525 = HEAP32[$j31>>2]|0;
             $1526 = HEAP32[$pTab36>>2]|0;
             $1527 = ((($1526)) + 34|0);
             $1528 = HEAP16[$1527>>1]|0;
             $1529 = $1528 << 16 >> 16;
             $1530 = ($1525|0)<($1529|0);
             if (!($1530)) {
              break;
             }
             $1531 = HEAP32[$j31>>2]|0;
             $1532 = HEAP32[$pTab36>>2]|0;
             $1533 = ((($1532)) + 32|0);
             $1534 = HEAP16[$1533>>1]|0;
             $1535 = $1534 << 16 >> 16;
             $1536 = ($1531|0)==($1535|0);
             do {
              if (!($1536)) {
               $1537 = HEAP32[$j31>>2]|0;
               $1538 = HEAP32[$pTab36>>2]|0;
               $1539 = ((($1538)) + 4|0);
               $1540 = HEAP32[$1539>>2]|0;
               $1541 = (($1540) + (($1537*24)|0)|0);
               $1542 = ((($1541)) + 20|0);
               $1543 = HEAP8[$1542>>0]|0;
               $1544 = $1543&255;
               $1545 = ($1544|0)==(0);
               if ($1545) {
                break;
               }
               $1546 = HEAP32[$v>>2]|0;
               $1547 = HEAP32[$pTab36>>2]|0;
               $1548 = HEAP32[$iDataCur>>2]|0;
               $1549 = HEAP32[$j31>>2]|0;
               _sqlite3ExprCodeGetColumnOfTable($1546,$1547,$1548,$1549,3);
               $1550 = HEAP32[$v>>2]|0;
               _sqlite3VdbeChangeP5($1550,-128);
               $1551 = HEAP32[$v>>2]|0;
               $1552 = (_sqlite3VdbeAddOp1($1551,77,3)|0);
               HEAP32[$jmp2>>2] = $1552;
               $1553 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1553,37,1,-1)|0);
               $1554 = HEAP32[$db>>2]|0;
               $1555 = HEAP32[$pTab36>>2]|0;
               $1556 = HEAP32[$1555>>2]|0;
               $1557 = HEAP32[$j31>>2]|0;
               $1558 = HEAP32[$pTab36>>2]|0;
               $1559 = ((($1558)) + 4|0);
               $1560 = HEAP32[$1559>>2]|0;
               $1561 = (($1560) + (($1557*24)|0)|0);
               $1562 = HEAP32[$1561>>2]|0;
               HEAP32[$vararg_buffer87>>2] = $1556;
               $vararg_ptr90 = ((($vararg_buffer87)) + 4|0);
               HEAP32[$vararg_ptr90>>2] = $1562;
               $1563 = (_sqlite3MPrintf($1554,34551,$vararg_buffer87)|0);
               HEAP32[$zErr>>2] = $1563;
               $1564 = HEAP32[$v>>2]|0;
               $1565 = HEAP32[$zErr>>2]|0;
               (_sqlite3VdbeAddOp4($1564,97,0,3,0,$1565,-1)|0);
               $1566 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1566,33,3,1)|0);
               $1567 = HEAP32[$v>>2]|0;
               $1568 = (_sqlite3VdbeAddOp1($1567,138,1)|0);
               HEAP32[$jmp3>>2] = $1568;
               $1569 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp0($1569,21)|0);
               $1570 = HEAP32[$v>>2]|0;
               $1571 = HEAP32[$jmp2>>2]|0;
               _sqlite3VdbeJumpHere($1570,$1571);
               $1572 = HEAP32[$v>>2]|0;
               $1573 = HEAP32[$jmp3>>2]|0;
               _sqlite3VdbeJumpHere($1572,$1573);
              }
             } while(0);
             $1574 = HEAP32[$j31>>2]|0;
             $1575 = (($1574) + 1)|0;
             HEAP32[$j31>>2] = $1575;
            }
            HEAP32[$j31>>2] = 0;
            $1576 = HEAP32[$pTab36>>2]|0;
            $1577 = ((($1576)) + 8|0);
            $1578 = HEAP32[$1577>>2]|0;
            HEAP32[$pIdx37>>2] = $1578;
            while(1) {
             $1579 = HEAP32[$pIdx37>>2]|0;
             $1580 = ($1579|0)!=(0|0);
             $1581 = HEAP32[$v>>2]|0;
             if (!($1580)) {
              break;
             }
             $1582 = (_sqlite3VdbeMakeLabel($1581)|0);
             HEAP32[$ckUniq>>2] = $1582;
             $1583 = HEAP32[$pPk38>>2]|0;
             $1584 = HEAP32[$pIdx37>>2]|0;
             $1585 = ($1583|0)==($1584|0);
             if (!($1585)) {
              $1586 = HEAP32[$0>>2]|0;
              $1587 = HEAP32[$pIdx37>>2]|0;
              $1588 = HEAP32[$iDataCur>>2]|0;
              $1589 = HEAP32[$pPrior>>2]|0;
              $1590 = HEAP32[$r1>>2]|0;
              $1591 = (_sqlite3GenerateIndexKey($1586,$1587,$1588,0,0,$jmp340,$1589,$1590)|0);
              HEAP32[$r1>>2] = $1591;
              $1592 = HEAP32[$pIdx37>>2]|0;
              HEAP32[$pPrior>>2] = $1592;
              $1593 = HEAP32[$v>>2]|0;
              $1594 = HEAP32[$j31>>2]|0;
              $1595 = (8 + ($1594))|0;
              (_sqlite3VdbeAddOp2($1593,37,$1595,1)|0);
              $1596 = HEAP32[$v>>2]|0;
              $1597 = HEAP32[$iIdxCur>>2]|0;
              $1598 = HEAP32[$j31>>2]|0;
              $1599 = (($1597) + ($1598))|0;
              $1600 = HEAP32[$ckUniq>>2]|0;
              $1601 = HEAP32[$r1>>2]|0;
              $1602 = HEAP32[$pIdx37>>2]|0;
              $1603 = ((($1602)) + 52|0);
              $1604 = HEAP16[$1603>>1]|0;
              $1605 = $1604&65535;
              $1606 = (_sqlite3VdbeAddOp4Int($1596,69,$1599,$1600,$1601,$1605)|0);
              HEAP32[$jmp239>>2] = $1606;
              $1607 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1607,37,1,-1)|0);
              $1608 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeLoadString($1608,3,34571)|0);
              $1609 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1609,94,7,3,3)|0);
              $1610 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeLoadString($1610,4,34576)|0);
              $1611 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1611,94,4,3,3)|0);
              $1612 = HEAP32[$v>>2]|0;
              $1613 = HEAP32[$pIdx37>>2]|0;
              $1614 = HEAP32[$1613>>2]|0;
              $1615 = (_sqlite3VdbeLoadString($1612,4,$1614)|0);
              HEAP32[$jmp5>>2] = $1615;
              $1616 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1616,94,4,3,3)|0);
              $1617 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1617,33,3,1)|0);
              $1618 = HEAP32[$v>>2]|0;
              $1619 = (_sqlite3VdbeAddOp1($1618,138,1)|0);
              HEAP32[$jmp4>>2] = $1619;
              $1620 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp0($1620,21)|0);
              $1621 = HEAP32[$v>>2]|0;
              $1622 = HEAP32[$jmp239>>2]|0;
              _sqlite3VdbeJumpHere($1621,$1622);
              $1623 = HEAP32[$pIdx37>>2]|0;
              $1624 = ((($1623)) + 54|0);
              $1625 = HEAP8[$1624>>0]|0;
              $1626 = $1625&255;
              $1627 = ($1626|0)!=(0);
              if ($1627) {
               $1628 = HEAP32[$v>>2]|0;
               $1629 = (_sqlite3VdbeMakeLabel($1628)|0);
               HEAP32[$uniqOk>>2] = $1629;
               HEAP32[$kk>>2] = 0;
               while(1) {
                $1630 = HEAP32[$kk>>2]|0;
                $1631 = HEAP32[$pIdx37>>2]|0;
                $1632 = ((($1631)) + 50|0);
                $1633 = HEAP16[$1632>>1]|0;
                $1634 = $1633&65535;
                $1635 = ($1630|0)<($1634|0);
                if (!($1635)) {
                 break;
                }
                $1636 = HEAP32[$kk>>2]|0;
                $1637 = HEAP32[$pIdx37>>2]|0;
                $1638 = ((($1637)) + 4|0);
                $1639 = HEAP32[$1638>>2]|0;
                $1640 = (($1639) + ($1636<<1)|0);
                $1641 = HEAP16[$1640>>1]|0;
                $1642 = $1641 << 16 >> 16;
                HEAP32[$iCol>>2] = $1642;
                $1643 = HEAP32[$iCol>>2]|0;
                $1644 = ($1643|0)>=(0);
                if ($1644) {
                 $1645 = HEAP32[$iCol>>2]|0;
                 $1646 = HEAP32[$pTab36>>2]|0;
                 $1647 = ((($1646)) + 4|0);
                 $1648 = HEAP32[$1647>>2]|0;
                 $1649 = (($1648) + (($1645*24)|0)|0);
                 $1650 = ((($1649)) + 20|0);
                 $1651 = HEAP8[$1650>>0]|0;
                 $1652 = $1651&255;
                 $1653 = ($1652|0)!=(0);
                 if (!($1653)) {
                  label = 279;
                 }
                } else {
                 label = 279;
                }
                if ((label|0) == 279) {
                 label = 0;
                 $1654 = HEAP32[$v>>2]|0;
                 $1655 = HEAP32[$r1>>2]|0;
                 $1656 = HEAP32[$kk>>2]|0;
                 $1657 = (($1655) + ($1656))|0;
                 $1658 = HEAP32[$uniqOk>>2]|0;
                 (_sqlite3VdbeAddOp2($1654,76,$1657,$1658)|0);
                }
                $1659 = HEAP32[$kk>>2]|0;
                $1660 = (($1659) + 1)|0;
                HEAP32[$kk>>2] = $1660;
               }
               $1661 = HEAP32[$v>>2]|0;
               $1662 = HEAP32[$iIdxCur>>2]|0;
               $1663 = HEAP32[$j31>>2]|0;
               $1664 = (($1662) + ($1663))|0;
               $1665 = (_sqlite3VdbeAddOp1($1661,7,$1664)|0);
               HEAP32[$jmp6>>2] = $1665;
               $1666 = HEAP32[$v>>2]|0;
               $1667 = HEAP32[$uniqOk>>2]|0;
               (_sqlite3VdbeGoto($1666,$1667)|0);
               $1668 = HEAP32[$v>>2]|0;
               $1669 = HEAP32[$jmp6>>2]|0;
               _sqlite3VdbeJumpHere($1668,$1669);
               $1670 = HEAP32[$v>>2]|0;
               $1671 = HEAP32[$iIdxCur>>2]|0;
               $1672 = HEAP32[$j31>>2]|0;
               $1673 = (($1671) + ($1672))|0;
               $1674 = HEAP32[$uniqOk>>2]|0;
               $1675 = HEAP32[$r1>>2]|0;
               $1676 = HEAP32[$pIdx37>>2]|0;
               $1677 = ((($1676)) + 50|0);
               $1678 = HEAP16[$1677>>1]|0;
               $1679 = $1678&65535;
               (_sqlite3VdbeAddOp4Int($1670,115,$1673,$1674,$1675,$1679)|0);
               $1680 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1680,37,1,-1)|0);
               $1681 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeLoadString($1681,3,34597)|0);
               $1682 = HEAP32[$v>>2]|0;
               $1683 = HEAP32[$jmp5>>2]|0;
               (_sqlite3VdbeGoto($1682,$1683)|0);
               $1684 = HEAP32[$v>>2]|0;
               $1685 = HEAP32[$uniqOk>>2]|0;
               _sqlite3VdbeResolveLabel($1684,$1685);
              }
              $1686 = HEAP32[$v>>2]|0;
              $1687 = HEAP32[$jmp4>>2]|0;
              _sqlite3VdbeJumpHere($1686,$1687);
              $1688 = HEAP32[$0>>2]|0;
              $1689 = HEAP32[$jmp340>>2]|0;
              _sqlite3ResolvePartIdxLabel($1688,$1689);
             }
             $1690 = HEAP32[$pIdx37>>2]|0;
             $1691 = ((($1690)) + 20|0);
             $1692 = HEAP32[$1691>>2]|0;
             HEAP32[$pIdx37>>2] = $1692;
             $1693 = HEAP32[$j31>>2]|0;
             $1694 = (($1693) + 1)|0;
             HEAP32[$j31>>2] = $1694;
            }
            $1695 = HEAP32[$iDataCur>>2]|0;
            $1696 = HEAP32[$loopTop>>2]|0;
            (_sqlite3VdbeAddOp2($1581,7,$1695,$1696)|0);
            $1697 = HEAP32[$v>>2]|0;
            $1698 = HEAP32[$loopTop>>2]|0;
            $1699 = (($1698) - 1)|0;
            _sqlite3VdbeJumpHere($1697,$1699);
            $1700 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeLoadString($1700,2,34624)|0);
            HEAP32[$j31>>2] = 0;
            $1701 = HEAP32[$pTab36>>2]|0;
            $1702 = ((($1701)) + 8|0);
            $1703 = HEAP32[$1702>>2]|0;
            HEAP32[$pIdx37>>2] = $1703;
            while(1) {
             $1704 = HEAP32[$pIdx37>>2]|0;
             $1705 = ($1704|0)!=(0|0);
             if (!($1705)) {
              break L356;
             }
             $1706 = HEAP32[$pPk38>>2]|0;
             $1707 = HEAP32[$pIdx37>>2]|0;
             $1708 = ($1706|0)==($1707|0);
             if (!($1708)) {
              $1709 = HEAP32[$v>>2]|0;
              $1710 = (_sqlite3VdbeCurrentAddr($1709)|0);
              HEAP32[$addr32>>2] = $1710;
              $1711 = HEAP32[$v>>2]|0;
              $1712 = HEAP32[$addr32>>2]|0;
              $1713 = (($1712) + 2)|0;
              (_sqlite3VdbeAddOp2($1711,138,1,$1713)|0);
              $1714 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1714,21,0,0)|0);
              $1715 = HEAP32[$v>>2]|0;
              $1716 = HEAP32[$iIdxCur>>2]|0;
              $1717 = HEAP32[$j31>>2]|0;
              $1718 = (($1716) + ($1717))|0;
              (_sqlite3VdbeAddOp2($1715,50,$1718,3)|0);
              $1719 = HEAP32[$v>>2]|0;
              $1720 = HEAP32[$j31>>2]|0;
              $1721 = (8 + ($1720))|0;
              $1722 = HEAP32[$addr32>>2]|0;
              $1723 = (($1722) + 8)|0;
              (_sqlite3VdbeAddOp3($1719,79,$1721,$1723,3)|0);
              $1724 = HEAP32[$v>>2]|0;
              _sqlite3VdbeChangeP5($1724,-112);
              $1725 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1725,37,1,-1)|0);
              $1726 = HEAP32[$v>>2]|0;
              $1727 = HEAP32[$pIdx37>>2]|0;
              $1728 = HEAP32[$1727>>2]|0;
              (_sqlite3VdbeLoadString($1726,3,$1728)|0);
              $1729 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1729,94,3,2,7)|0);
              $1730 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1730,33,7,1)|0);
             }
             $1731 = HEAP32[$pIdx37>>2]|0;
             $1732 = ((($1731)) + 20|0);
             $1733 = HEAP32[$1732>>2]|0;
             HEAP32[$pIdx37>>2] = $1733;
             $1734 = HEAP32[$j31>>2]|0;
             $1735 = (($1734) + 1)|0;
             HEAP32[$j31>>2] = $1735;
            }
           }
          } while(0);
          $1736 = HEAP32[$x33>>2]|0;
          $1737 = HEAP32[$1736>>2]|0;
          HEAP32[$x33>>2] = $1737;
         }
        }
       } while(0);
       $1738 = HEAP32[$i30>>2]|0;
       $1739 = (($1738) + 1)|0;
       HEAP32[$i30>>2] = $1739;
      }
      $1740 = HEAP32[$v>>2]|0;
      $1741 = (_sqlite3VdbeAddOpList($1740,4,34653,0)|0);
      HEAP32[$aOp41>>2] = $1741;
      $1742 = HEAP32[$aOp41>>2]|0;
      $1743 = ($1742|0)!=(0|0);
      if (!($1743)) {
       break L21;
      }
      $1744 = HEAP32[$mxErr>>2]|0;
      $1745 = (0 - ($1744))|0;
      $1746 = HEAP32[$aOp41>>2]|0;
      $1747 = ((($1746)) + 8|0);
      HEAP32[$1747>>2] = $1745;
      $1748 = HEAP32[$aOp41>>2]|0;
      $1749 = ((($1748)) + 40|0);
      $1750 = ((($1749)) + 1|0);
      HEAP8[$1750>>0] = -2;
      $1751 = HEAP32[$aOp41>>2]|0;
      $1752 = ((($1751)) + 40|0);
      $1753 = ((($1752)) + 16|0);
      HEAP32[$1753>>2] = 34669;
      break L21;
      break;
     }
     case 12:  {
      $1754 = HEAP32[$zRight>>2]|0;
      $1755 = ($1754|0)!=(0|0);
      if (!($1755)) {
       $1756 = HEAP32[$0>>2]|0;
       $1757 = (_sqlite3ReadSchema($1756)|0);
       $1758 = ($1757|0)!=(0);
       if ($1758) {
        break L21;
       }
       $1759 = HEAP32[$v>>2]|0;
       $1760 = HEAP32[$0>>2]|0;
       $1761 = HEAP32[$1760>>2]|0;
       $1762 = ((($1761)) + 62|0);
       $1763 = HEAP8[$1762>>0]|0;
       $1764 = $1763&255;
       $1765 = (3516 + ($1764<<3)|0);
       $1766 = HEAP32[$1765>>2]|0;
       _returnSingleText($1759,34730,$1766);
       break L21;
      }
      $1767 = HEAP32[$db>>2]|0;
      $1768 = ((($1767)) + 16|0);
      $1769 = HEAP32[$1768>>2]|0;
      $1770 = ((($1769)) + 12|0);
      $1771 = HEAP32[$1770>>2]|0;
      $1772 = ((($1771)) + 78|0);
      $1773 = HEAP16[$1772>>1]|0;
      $1774 = $1773&65535;
      $1775 = $1774 & 1;
      $1776 = ($1775|0)==(1);
      if ($1776) {
       $1777 = HEAP32[$db>>2]|0;
       $1778 = ((($1777)) + 16|0);
       $1779 = HEAP32[$1778>>2]|0;
       $1780 = ((($1779)) + 12|0);
       $1781 = HEAP32[$1780>>2]|0;
       $1782 = ((($1781)) + 78|0);
       $1783 = HEAP16[$1782>>1]|0;
       $1784 = $1783&65535;
       $1785 = $1784 & 4;
       $1786 = ($1785|0)==(4);
       if (!($1786)) {
        break L21;
       }
      }
      HEAP32[$pEnc>>2] = 3516;
      while(1) {
       $1787 = HEAP32[$pEnc>>2]|0;
       $1788 = HEAP32[$1787>>2]|0;
       $1789 = ($1788|0)!=(0|0);
       if (!($1789)) {
        break;
       }
       $1790 = HEAP32[$zRight>>2]|0;
       $1791 = HEAP32[$pEnc>>2]|0;
       $1792 = HEAP32[$1791>>2]|0;
       $1793 = (_sqlite3_stricmp($1790,$1792)|0);
       $1794 = (0)==($1793|0);
       $1795 = HEAP32[$pEnc>>2]|0;
       if ($1794) {
        label = 301;
        break;
       }
       $1818 = ((($1795)) + 8|0);
       HEAP32[$pEnc>>2] = $1818;
      }
      if ((label|0) == 301) {
       $1796 = ((($1795)) + 4|0);
       $1797 = HEAP8[$1796>>0]|0;
       $1798 = $1797&255;
       $1799 = ($1798|0)!=(0);
       if ($1799) {
        $1800 = HEAP32[$pEnc>>2]|0;
        $1801 = ((($1800)) + 4|0);
        $1802 = HEAP8[$1801>>0]|0;
        $1803 = $1802&255;
        $1809 = $1803;
       } else {
        $1804 = HEAP8[352>>0]|0;
        $1805 = $1804 << 24 >> 24;
        $1806 = ($1805|0)==(0);
        $1807 = $1806 ? 3 : 2;
        $1809 = $1807;
       }
       $1808 = $1809&255;
       $1810 = HEAP32[$db>>2]|0;
       $1811 = ((($1810)) + 62|0);
       HEAP8[$1811>>0] = $1808;
       $1812 = HEAP32[$db>>2]|0;
       $1813 = ((($1812)) + 16|0);
       $1814 = HEAP32[$1813>>2]|0;
       $1815 = ((($1814)) + 12|0);
       $1816 = HEAP32[$1815>>2]|0;
       $1817 = ((($1816)) + 77|0);
       HEAP8[$1817>>0] = $1808;
      }
      $1819 = HEAP32[$pEnc>>2]|0;
      $1820 = HEAP32[$1819>>2]|0;
      $1821 = ($1820|0)!=(0|0);
      if ($1821) {
       break L21;
      }
      $1822 = HEAP32[$0>>2]|0;
      $1823 = HEAP32[$zRight>>2]|0;
      HEAP32[$vararg_buffer91>>2] = $1823;
      _sqlite3ErrorMsg($1822,34739,$vararg_buffer91);
      break L21;
      break;
     }
     case 0:  {
      $1824 = HEAP32[$pPragma>>2]|0;
      $1825 = ((($1824)) + 8|0);
      $1826 = HEAP32[$1825>>2]|0;
      HEAP32[$iCookie>>2] = $1826;
      $1827 = HEAP32[$v>>2]|0;
      $1828 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($1827,$1828);
      $1829 = HEAP32[$zRight>>2]|0;
      $1830 = ($1829|0)!=(0|0);
      if ($1830) {
       $1831 = HEAP32[$pPragma>>2]|0;
       $1832 = ((($1831)) + 5|0);
       $1833 = HEAP8[$1832>>0]|0;
       $1834 = $1833&255;
       $1835 = $1834 & 2;
       $1836 = ($1835|0)==(0);
       if ($1836) {
        $1837 = HEAP32[$v>>2]|0;
        $1838 = (_sqlite3VdbeAddOpList($1837,2,34764,0)|0);
        HEAP32[$aOp42>>2] = $1838;
        $1839 = HEAP32[$iDb>>2]|0;
        $1840 = HEAP32[$aOp42>>2]|0;
        $1841 = ((($1840)) + 4|0);
        HEAP32[$1841>>2] = $1839;
        $1842 = HEAP32[$iDb>>2]|0;
        $1843 = HEAP32[$aOp42>>2]|0;
        $1844 = ((($1843)) + 20|0);
        $1845 = ((($1844)) + 4|0);
        HEAP32[$1845>>2] = $1842;
        $1846 = HEAP32[$iCookie>>2]|0;
        $1847 = HEAP32[$aOp42>>2]|0;
        $1848 = ((($1847)) + 20|0);
        $1849 = ((($1848)) + 8|0);
        HEAP32[$1849>>2] = $1846;
        $1850 = HEAP32[$zRight>>2]|0;
        $1851 = (_sqlite3Atoi($1850)|0);
        $1852 = HEAP32[$aOp42>>2]|0;
        $1853 = ((($1852)) + 20|0);
        $1854 = ((($1853)) + 12|0);
        HEAP32[$1854>>2] = $1851;
        break L21;
       }
      }
      $1855 = HEAP32[$v>>2]|0;
      $1856 = (_sqlite3VdbeAddOpList($1855,3,34772,0)|0);
      HEAP32[$aOp43>>2] = $1856;
      $1857 = HEAP32[$iDb>>2]|0;
      $1858 = HEAP32[$aOp43>>2]|0;
      $1859 = ((($1858)) + 4|0);
      HEAP32[$1859>>2] = $1857;
      $1860 = HEAP32[$iDb>>2]|0;
      $1861 = HEAP32[$aOp43>>2]|0;
      $1862 = ((($1861)) + 20|0);
      $1863 = ((($1862)) + 4|0);
      HEAP32[$1863>>2] = $1860;
      $1864 = HEAP32[$iCookie>>2]|0;
      $1865 = HEAP32[$aOp43>>2]|0;
      $1866 = ((($1865)) + 20|0);
      $1867 = ((($1866)) + 12|0);
      HEAP32[$1867>>2] = $1864;
      $1868 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($1868,1);
      $1869 = HEAP32[$v>>2]|0;
      $1870 = HEAP32[$zLeft>>2]|0;
      (_sqlite3VdbeSetColName($1869,0,0,$1870,(-1))|0);
      break L21;
      break;
     }
     case 8:  {
      HEAP32[$i44>>2] = 0;
      $1871 = HEAP32[$0>>2]|0;
      $1872 = ((($1871)) + 76|0);
      HEAP32[$1872>>2] = 1;
      $1873 = HEAP32[$v>>2]|0;
      _setOneColumnName($1873,34784);
      while(1) {
       $1874 = HEAP32[$i44>>2]|0;
       $1875 = (($1874) + 1)|0;
       HEAP32[$i44>>2] = $1875;
       $1876 = (_sqlite3_compileoption_get($1874)|0);
       HEAP32[$zOpt>>2] = $1876;
       $1877 = ($1876|0)!=(0|0);
       if (!($1877)) {
        break L21;
       }
       $1878 = HEAP32[$v>>2]|0;
       $1879 = HEAP32[$zOpt>>2]|0;
       (_sqlite3VdbeLoadString($1878,1,$1879)|0);
       $1880 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($1880,33,1,1)|0);
      }
      break;
     }
     case 36:  {
      $1881 = HEAP32[$2>>2]|0;
      $1882 = HEAP32[$1881>>2]|0;
      $1883 = ($1882|0)!=(0|0);
      $1884 = HEAP32[$iDb>>2]|0;
      $1885 = $1883 ? $1884 : 10;
      HEAP32[$iBt>>2] = $1885;
      HEAP32[$eMode45>>2] = 0;
      $1886 = HEAP32[$zRight>>2]|0;
      $1887 = ($1886|0)!=(0|0);
      do {
       if ($1887) {
        $1888 = HEAP32[$zRight>>2]|0;
        $1889 = (_sqlite3_stricmp($1888,34821)|0);
        $1890 = ($1889|0)==(0);
        if ($1890) {
         HEAP32[$eMode45>>2] = 1;
         break;
        }
        $1891 = HEAP32[$zRight>>2]|0;
        $1892 = (_sqlite3_stricmp($1891,34826)|0);
        $1893 = ($1892|0)==(0);
        if ($1893) {
         HEAP32[$eMode45>>2] = 2;
         break;
        }
        $1894 = HEAP32[$zRight>>2]|0;
        $1895 = (_sqlite3_stricmp($1894,22520)|0);
        $1896 = ($1895|0)==(0);
        if ($1896) {
         HEAP32[$eMode45>>2] = 3;
        }
       }
      } while(0);
      $1897 = HEAP32[$v>>2]|0;
      _setAllColumnNames($1897,3,3588);
      $1898 = HEAP32[$0>>2]|0;
      $1899 = ((($1898)) + 76|0);
      HEAP32[$1899>>2] = 3;
      $1900 = HEAP32[$v>>2]|0;
      $1901 = HEAP32[$iBt>>2]|0;
      $1902 = HEAP32[$eMode45>>2]|0;
      (_sqlite3VdbeAddOp3($1900,8,$1901,$1902,1)|0);
      $1903 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($1903,33,1,3)|0);
      break L21;
      break;
     }
     case 35:  {
      $1904 = HEAP32[$zRight>>2]|0;
      $1905 = ($1904|0)!=(0|0);
      if ($1905) {
       $1906 = HEAP32[$db>>2]|0;
       $1907 = HEAP32[$zRight>>2]|0;
       $1908 = (_sqlite3Atoi($1907)|0);
       (_sqlite3_wal_autocheckpoint($1906,$1908)|0);
      }
      $1909 = HEAP32[$v>>2]|0;
      $1910 = HEAP32[$db>>2]|0;
      $1911 = ((($1910)) + 216|0);
      $1912 = HEAP32[$1911>>2]|0;
      $1913 = ($1912|0)==(139|0);
      if ($1913) {
       $1914 = HEAP32[$db>>2]|0;
       $1915 = ((($1914)) + 220|0);
       $1916 = HEAP32[$1915>>2]|0;
       $1917 = $1916;
       $1919 = $1917;
      } else {
       $1919 = 0;
      }
      $1918 = ($1919|0)<(0);
      $1920 = $1918 << 31 >> 31;
      _returnSingleInt($1909,34834,$1919,$1920);
      break L21;
      break;
     }
     case 27:  {
      $1921 = HEAP32[$db>>2]|0;
      (_sqlite3_db_release_memory($1921)|0);
      break L21;
      break;
     }
     case 28:  {
      $1933 = HEAP32[$zRight>>2]|0;
      $1934 = ($1933|0)!=(0|0);
      if ($1934) {
       $1935 = HEAP32[$zRight>>2]|0;
       $1936 = (_sqlite3DecOrHexToI64($1935,$N)|0);
       $1937 = ($1936|0)==(0);
       if ($1937) {
        $1938 = $N;
        $1939 = $1938;
        $1940 = HEAP32[$1939>>2]|0;
        $1941 = (($1938) + 4)|0;
        $1942 = $1941;
        $1943 = HEAP32[$1942>>2]|0;
        (_sqlite3_soft_heap_limit64($1940,$1943)|0);
        $1944 = tempRet0;
       }
      }
      $1945 = HEAP32[$v>>2]|0;
      $1946 = (_sqlite3_soft_heap_limit64(-1,-1)|0);
      $1947 = tempRet0;
      _returnSingleInt($1945,34861,$1946,$1947);
      break L21;
      break;
     }
     case 34:  {
      $1948 = HEAP32[$zRight>>2]|0;
      $1949 = ($1948|0)!=(0|0);
      if ($1949) {
       $1950 = HEAP32[$zRight>>2]|0;
       $1951 = (_sqlite3DecOrHexToI64($1950,$N46)|0);
       $1952 = ($1951|0)==(0);
       $1953 = $N46;
       $1954 = $1953;
       $1955 = HEAP32[$1954>>2]|0;
       $1956 = (($1953) + 4)|0;
       $1957 = $1956;
       $1958 = HEAP32[$1957>>2]|0;
       $1959 = ($1958|0)>(0);
       $1960 = ($1955>>>0)>=(0);
       $1961 = ($1958|0)==(0);
       $1962 = $1961 & $1960;
       $1963 = $1959 | $1962;
       $or$cond14 = $1952 & $1963;
       if ($or$cond14) {
        $1964 = HEAP32[$db>>2]|0;
        $1965 = $N46;
        $1966 = $1965;
        $1967 = HEAP32[$1966>>2]|0;
        $1968 = (($1965) + 4)|0;
        $1969 = $1968;
        $1970 = HEAP32[$1969>>2]|0;
        $1971 = $1967 & 2147483647;
        (_sqlite3_limit($1964,11,$1971)|0);
       }
      }
      $1972 = HEAP32[$v>>2]|0;
      $1973 = HEAP32[$db>>2]|0;
      $1974 = (_sqlite3_limit($1973,11,-1)|0);
      $1975 = ($1974|0)<(0);
      $1976 = $1975 << 31 >> 31;
      _returnSingleInt($1972,34877,$1974,$1976);
      break L21;
      break;
     }
     default: {
      $1922 = HEAP32[$zRight>>2]|0;
      $1923 = ($1922|0)!=(0|0);
      if ($1923) {
       $1924 = HEAP32[$db>>2]|0;
       $1925 = HEAP32[$zRight>>2]|0;
       $1926 = (_sqlite3Atoi($1925)|0);
       (_sqlite3_busy_timeout($1924,$1926)|0);
      }
      $1927 = HEAP32[$v>>2]|0;
      $1928 = HEAP32[$db>>2]|0;
      $1929 = ((($1928)) + 496|0);
      $1930 = HEAP32[$1929>>2]|0;
      $1931 = ($1930|0)<(0);
      $1932 = $1931 << 31 >> 31;
      _returnSingleInt($1927,34853,$1930,$1932);
      break L21;
     }
     }
    } while(0);
   }
  }
 } while(0);
 $1977 = HEAP32[$db>>2]|0;
 $1978 = HEAP32[$zLeft>>2]|0;
 _sqlite3DbFree($1977,$1978);
 $1979 = HEAP32[$db>>2]|0;
 $1980 = HEAP32[$zRight>>2]|0;
 _sqlite3DbFree($1979,$1980);
 STACKTOP = sp;return;
}
function _sqlite3FinishTrigger($pParse,$pStepList,$pAll) {
 $pParse = $pParse|0;
 $pStepList = $pStepList|0;
 $pAll = $pAll|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0;
 var $nameToken = 0, $or$cond = 0, $pHash = 0, $pLink = 0, $pTab = 0, $pTrig = 0, $sFix = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer6 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $z = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $pTrig = sp + 96|0;
 $zName = sp + 92|0;
 $db = sp + 88|0;
 $sFix = sp + 64|0;
 $iDb = sp + 56|0;
 $nameToken = sp + 48|0;
 $v = sp + 44|0;
 $z = sp + 40|0;
 $pLink = sp + 36|0;
 $pHash = sp + 32|0;
 $pTab = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStepList;
 HEAP32[$2>>2] = $pAll;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 492|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTrig>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 492|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 68|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 $14 = HEAP32[$pTrig>>2]|0;
 $15 = ($14|0)!=(0|0);
 $or$cond = $13 & $15;
 do {
  if ($or$cond) {
   $16 = HEAP32[$pTrig>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$zName>>2] = $17;
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$pTrig>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3SchemaToIndex($19,$22)|0);
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$pTrig>>2]|0;
   $26 = ((($25)) + 28|0);
   HEAP32[$26>>2] = $24;
   while(1) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ($27|0)!=(0|0);
    $29 = HEAP32[$pTrig>>2]|0;
    if (!($28)) {
     break;
    }
    $30 = HEAP32[$1>>2]|0;
    $31 = ((($30)) + 4|0);
    HEAP32[$31>>2] = $29;
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 28|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$1>>2] = $34;
   }
   $35 = HEAP32[$29>>2]|0;
   _sqlite3TokenInit($nameToken,$35);
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$iDb>>2]|0;
   _sqlite3FixInit($sFix,$36,$37,17794,$nameToken);
   $38 = HEAP32[$pTrig>>2]|0;
   $39 = ((($38)) + 28|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (_sqlite3FixTriggerStep($sFix,$40)|0);
   $42 = ($41|0)!=(0);
   if (!($42)) {
    $43 = HEAP32[$pTrig>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (_sqlite3FixExpr($sFix,$45)|0);
    $47 = ($46|0)!=(0);
    if (!($47)) {
     $48 = HEAP32[$db>>2]|0;
     $49 = ((($48)) + 140|0);
     $50 = ((($49)) + 5|0);
     $51 = HEAP8[$50>>0]|0;
     $52 = ($51<<24>>24)!=(0);
     if (!($52)) {
      $53 = HEAP32[$0>>2]|0;
      $54 = (_sqlite3GetVdbe($53)|0);
      HEAP32[$v>>2] = $54;
      $55 = HEAP32[$v>>2]|0;
      $56 = ($55|0)==(0|0);
      if ($56) {
       break;
      }
      $57 = HEAP32[$0>>2]|0;
      $58 = HEAP32[$iDb>>2]|0;
      _sqlite3BeginWriteOperation($57,0,$58);
      $59 = HEAP32[$db>>2]|0;
      $60 = HEAP32[$2>>2]|0;
      $61 = HEAP32[$60>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = ((($62)) + 4|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (_sqlite3DbStrNDup($59,$61,$64,0)|0);
      HEAP32[$z>>2] = $65;
      $66 = HEAP32[$0>>2]|0;
      $67 = HEAP32[$iDb>>2]|0;
      $68 = HEAP32[$db>>2]|0;
      $69 = ((($68)) + 16|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = (($70) + ($67<<4)|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = HEAP32[$iDb>>2]|0;
      $74 = ($73|0)==(1);
      $75 = $74 ? 21260 : 21279;
      $76 = HEAP32[$zName>>2]|0;
      $77 = HEAP32[$pTrig>>2]|0;
      $78 = ((($77)) + 4|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = HEAP32[$z>>2]|0;
      HEAP32[$vararg_buffer>>2] = $72;
      $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
      HEAP32[$vararg_ptr2>>2] = $75;
      $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
      HEAP32[$vararg_ptr3>>2] = $76;
      $vararg_ptr4 = ((($vararg_buffer)) + 12|0);
      HEAP32[$vararg_ptr4>>2] = $79;
      $vararg_ptr5 = ((($vararg_buffer)) + 16|0);
      HEAP32[$vararg_ptr5>>2] = $80;
      _sqlite3NestedParse($66,35785,$vararg_buffer);
      $81 = HEAP32[$db>>2]|0;
      $82 = HEAP32[$z>>2]|0;
      _sqlite3DbFree($81,$82);
      $83 = HEAP32[$0>>2]|0;
      $84 = HEAP32[$iDb>>2]|0;
      _sqlite3ChangeCookie($83,$84);
      $85 = HEAP32[$v>>2]|0;
      $86 = HEAP32[$iDb>>2]|0;
      $87 = HEAP32[$db>>2]|0;
      $88 = HEAP32[$zName>>2]|0;
      HEAP32[$vararg_buffer6>>2] = $88;
      $89 = (_sqlite3MPrintf($87,35849,$vararg_buffer6)|0);
      _sqlite3VdbeAddParseSchemaOp($85,$86,$89);
     }
     $90 = HEAP32[$db>>2]|0;
     $91 = ((($90)) + 140|0);
     $92 = ((($91)) + 5|0);
     $93 = HEAP8[$92>>0]|0;
     $94 = ($93<<24>>24)!=(0);
     if ($94) {
      $95 = HEAP32[$pTrig>>2]|0;
      HEAP32[$pLink>>2] = $95;
      $96 = HEAP32[$iDb>>2]|0;
      $97 = HEAP32[$db>>2]|0;
      $98 = ((($97)) + 16|0);
      $99 = HEAP32[$98>>2]|0;
      $100 = (($99) + ($96<<4)|0);
      $101 = ((($100)) + 12|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ((($102)) + 40|0);
      HEAP32[$pHash>>2] = $103;
      $104 = HEAP32[$pHash>>2]|0;
      $105 = HEAP32[$zName>>2]|0;
      $106 = HEAP32[$pTrig>>2]|0;
      $107 = (_sqlite3HashInsert($104,$105,$106)|0);
      HEAP32[$pTrig>>2] = $107;
      $108 = HEAP32[$pTrig>>2]|0;
      $109 = ($108|0)!=(0|0);
      if ($109) {
       $110 = HEAP32[$db>>2]|0;
       _sqlite3OomFault($110);
       break;
      }
      $111 = HEAP32[$pLink>>2]|0;
      $112 = ((($111)) + 20|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = HEAP32[$pLink>>2]|0;
      $115 = ((($114)) + 24|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($113|0)==($116|0);
      if ($117) {
       $118 = HEAP32[$pLink>>2]|0;
       $119 = ((($118)) + 24|0);
       $120 = HEAP32[$119>>2]|0;
       $121 = ((($120)) + 8|0);
       $122 = HEAP32[$pLink>>2]|0;
       $123 = ((($122)) + 4|0);
       $124 = HEAP32[$123>>2]|0;
       $125 = (_sqlite3HashFind($121,$124)|0);
       HEAP32[$pTab>>2] = $125;
       $126 = HEAP32[$pTab>>2]|0;
       $127 = ((($126)) + 60|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = HEAP32[$pLink>>2]|0;
       $130 = ((($129)) + 32|0);
       HEAP32[$130>>2] = $128;
       $131 = HEAP32[$pLink>>2]|0;
       $132 = HEAP32[$pTab>>2]|0;
       $133 = ((($132)) + 60|0);
       HEAP32[$133>>2] = $131;
      }
     }
    }
   }
  }
 } while(0);
 $134 = HEAP32[$db>>2]|0;
 $135 = HEAP32[$pTrig>>2]|0;
 _sqlite3DeleteTrigger($134,$135);
 $136 = HEAP32[$db>>2]|0;
 $137 = HEAP32[$1>>2]|0;
 _sqlite3DeleteTriggerStep($136,$137);
 STACKTOP = sp;return;
}
function _sqlite3BeginTrigger($pParse,$pName1,$pName2,$tr_tm,$op,$pColumns,$pTableName,$pWhen,$isTemp,$noErr) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $tr_tm = $tr_tm|0;
 $op = $op|0;
 $pColumns = $pColumns|0;
 $pTableName = $pTableName|0;
 $pWhen = $pWhen|0;
 $isTemp = $isTemp|0;
 $noErr = $noErr|0;
 var $$old = 0, $$old10 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $iTabDb = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pName = 0, $pTab = 0, $pTrigger = 0, $sFix = 0, $vararg_buffer = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0;
 var $vararg_buffer18 = 0, $vararg_buffer23 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $zDb = 0, $zDbTrig = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer23 = sp + 48|0;
 $vararg_buffer18 = sp + 32|0;
 $vararg_buffer16 = sp + 24|0;
 $vararg_buffer14 = sp + 16|0;
 $vararg_buffer12 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 156|0;
 $1 = sp + 152|0;
 $2 = sp + 148|0;
 $3 = sp + 144|0;
 $4 = sp + 140|0;
 $5 = sp + 136|0;
 $6 = sp + 132|0;
 $7 = sp + 128|0;
 $8 = sp + 124|0;
 $9 = sp + 120|0;
 $pTrigger = sp + 116|0;
 $pTab = sp + 112|0;
 $zName = sp + 108|0;
 $db = sp + 104|0;
 $iDb = sp + 100|0;
 $pName = sp + 96|0;
 $sFix = sp + 72|0;
 $iTabDb = sp + 68|0;
 $code = sp + 64|0;
 $zDb = sp + 60|0;
 $zDbTrig = sp + 56|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $tr_tm;
 HEAP32[$4>>2] = $op;
 HEAP32[$5>>2] = $pColumns;
 HEAP32[$6>>2] = $pTableName;
 HEAP32[$7>>2] = $pWhen;
 HEAP32[$8>>2] = $isTemp;
 HEAP32[$9>>2] = $noErr;
 HEAP32[$pTrigger>>2] = 0;
 HEAP32[$zName>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$8>>2]|0;
 $13 = ($12|0)!=(0);
 do {
  if ($13) {
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16>>>0)>(0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($18,35878,$vararg_buffer);
    break;
   } else {
    HEAP32[$iDb>>2] = 1;
    $19 = HEAP32[$1>>2]|0;
    HEAP32[$pName>>2] = $19;
    $$old = HEAP32[$6>>2]|0;
    $$old10 = ($$old|0)!=(0|0);
    if ($$old10) {
     label = 6;
     break;
    } else {
     break;
    }
   }
  } else {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = (_sqlite3TwoPartName($20,$21,$22,$pName)|0);
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$iDb>>2]|0;
   $25 = ($24|0)>=(0);
   $26 = HEAP32[$6>>2]|0;
   $27 = ($26|0)!=(0|0);
   $or$cond11 = $25 & $27;
   if ($or$cond11) {
    label = 6;
   }
  }
 } while(0);
 do {
  if ((label|0) == 6) {
   $28 = HEAP32[$db>>2]|0;
   $29 = ((($28)) + 65|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = ($31|0)!=(0);
   if (!($32)) {
    $33 = HEAP32[$db>>2]|0;
    $34 = ((($33)) + 140|0);
    $35 = ((($34)) + 5|0);
    $36 = HEAP8[$35>>0]|0;
    $37 = $36&255;
    $38 = ($37|0)!=(0);
    $39 = HEAP32[$iDb>>2]|0;
    $40 = ($39|0)!=(1);
    $or$cond = $38 & $40;
    if ($or$cond) {
     $41 = HEAP32[$db>>2]|0;
     $42 = HEAP32[$6>>2]|0;
     $43 = ((($42)) + 8|0);
     $44 = ((($43)) + 4|0);
     $45 = HEAP32[$44>>2]|0;
     _sqlite3DbFree($41,$45);
     $46 = HEAP32[$6>>2]|0;
     $47 = ((($46)) + 8|0);
     $48 = ((($47)) + 4|0);
     HEAP32[$48>>2] = 0;
    }
    $49 = HEAP32[$0>>2]|0;
    $50 = HEAP32[$6>>2]|0;
    $51 = (_sqlite3SrcListLookup($49,$50)|0);
    HEAP32[$pTab>>2] = $51;
    $52 = HEAP32[$db>>2]|0;
    $53 = ((($52)) + 140|0);
    $54 = ((($53)) + 5|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = ($56|0)==(0);
    if ($57) {
     $58 = HEAP32[$2>>2]|0;
     $59 = ((($58)) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($60|0)==(0);
     $62 = HEAP32[$pTab>>2]|0;
     $63 = ($62|0)!=(0|0);
     $or$cond3 = $61 & $63;
     if ($or$cond3) {
      $64 = HEAP32[$pTab>>2]|0;
      $65 = ((($64)) + 64|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$db>>2]|0;
      $68 = ((($67)) + 16|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = ((($69)) + 16|0);
      $71 = ((($70)) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ($66|0)==($72|0);
      if ($73) {
       HEAP32[$iDb>>2] = 1;
      }
     }
    }
    $74 = HEAP32[$db>>2]|0;
    $75 = ((($74)) + 65|0);
    $76 = HEAP8[$75>>0]|0;
    $77 = ($76<<24>>24)!=(0);
    if (!($77)) {
     $78 = HEAP32[$0>>2]|0;
     $79 = HEAP32[$iDb>>2]|0;
     $80 = HEAP32[$pName>>2]|0;
     _sqlite3FixInit($sFix,$78,$79,17794,$80);
     $81 = HEAP32[$6>>2]|0;
     $82 = (_sqlite3FixSrcList($sFix,$81)|0);
     $83 = ($82|0)!=(0);
     if (!($83)) {
      $84 = HEAP32[$0>>2]|0;
      $85 = HEAP32[$6>>2]|0;
      $86 = (_sqlite3SrcListLookup($84,$85)|0);
      HEAP32[$pTab>>2] = $86;
      $87 = HEAP32[$pTab>>2]|0;
      $88 = ($87|0)!=(0|0);
      if (!($88)) {
       $89 = HEAP32[$db>>2]|0;
       $90 = ((($89)) + 140|0);
       $91 = ((($90)) + 4|0);
       $92 = HEAP8[$91>>0]|0;
       $93 = $92&255;
       $94 = ($93|0)==(1);
       if (!($94)) {
        break;
       }
       $95 = HEAP32[$db>>2]|0;
       $96 = ((($95)) + 140|0);
       $97 = ((($96)) + 6|0);
       HEAP8[$97>>0] = 1;
       break;
      }
      $98 = HEAP32[$pTab>>2]|0;
      $99 = ((($98)) + 42|0);
      $100 = HEAP8[$99>>0]|0;
      $101 = $100&255;
      $102 = $101 & 16;
      $103 = ($102|0)!=(0);
      if ($103) {
       $104 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($104,35924,$vararg_buffer12);
       break;
      }
      $105 = HEAP32[$db>>2]|0;
      $106 = HEAP32[$pName>>2]|0;
      $107 = (_sqlite3NameFromToken($105,$106)|0);
      HEAP32[$zName>>2] = $107;
      $108 = HEAP32[$zName>>2]|0;
      $109 = ($108|0)!=(0|0);
      if ($109) {
       $110 = HEAP32[$0>>2]|0;
       $111 = HEAP32[$zName>>2]|0;
       $112 = (_sqlite3CheckObjectName($110,$111)|0);
       $113 = (0)!=($112|0);
       if (!($113)) {
        $114 = HEAP32[$iDb>>2]|0;
        $115 = HEAP32[$db>>2]|0;
        $116 = ((($115)) + 16|0);
        $117 = HEAP32[$116>>2]|0;
        $118 = (($117) + ($114<<4)|0);
        $119 = ((($118)) + 12|0);
        $120 = HEAP32[$119>>2]|0;
        $121 = ((($120)) + 40|0);
        $122 = HEAP32[$zName>>2]|0;
        $123 = (_sqlite3HashFind($121,$122)|0);
        $124 = ($123|0)!=(0|0);
        if ($124) {
         $125 = HEAP32[$9>>2]|0;
         $126 = ($125|0)!=(0);
         $127 = HEAP32[$0>>2]|0;
         if ($126) {
          $129 = HEAP32[$iDb>>2]|0;
          _sqlite3CodeVerifySchema($127,$129);
          break;
         } else {
          $128 = HEAP32[$pName>>2]|0;
          HEAP32[$vararg_buffer14>>2] = $128;
          _sqlite3ErrorMsg($127,35965,$vararg_buffer14);
          break;
         }
        }
        $130 = HEAP32[$pTab>>2]|0;
        $131 = HEAP32[$130>>2]|0;
        $132 = (_sqlite3_strnicmp($131,28844,7)|0);
        $133 = ($132|0)==(0);
        if ($133) {
         $134 = HEAP32[$0>>2]|0;
         _sqlite3ErrorMsg($134,35991,$vararg_buffer16);
         break;
        }
        $135 = HEAP32[$pTab>>2]|0;
        $136 = ((($135)) + 12|0);
        $137 = HEAP32[$136>>2]|0;
        $138 = ($137|0)!=(0|0);
        $139 = HEAP32[$3>>2]|0;
        $140 = ($139|0)!=(49);
        $or$cond5 = $138 & $140;
        if ($or$cond5) {
         $141 = HEAP32[$0>>2]|0;
         $142 = HEAP32[$3>>2]|0;
         $143 = ($142|0)==(35);
         $144 = $143 ? 36066 : 36073;
         $145 = HEAP32[$6>>2]|0;
         HEAP32[$vararg_buffer18>>2] = $144;
         $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
         HEAP32[$vararg_ptr21>>2] = $145;
         $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
         HEAP32[$vararg_ptr22>>2] = 0;
         _sqlite3ErrorMsg($141,36029,$vararg_buffer18);
         break;
        }
        $146 = HEAP32[$pTab>>2]|0;
        $147 = ((($146)) + 12|0);
        $148 = HEAP32[$147>>2]|0;
        $149 = ($148|0)==(0|0);
        $150 = HEAP32[$3>>2]|0;
        $151 = ($150|0)==(49);
        $or$cond7 = $149 & $151;
        if ($or$cond7) {
         $152 = HEAP32[$0>>2]|0;
         $153 = HEAP32[$6>>2]|0;
         HEAP32[$vararg_buffer23>>2] = $153;
         $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
         HEAP32[$vararg_ptr26>>2] = 0;
         _sqlite3ErrorMsg($152,36079,$vararg_buffer23);
         break;
        }
        $154 = HEAP32[$db>>2]|0;
        $155 = HEAP32[$pTab>>2]|0;
        $156 = ((($155)) + 64|0);
        $157 = HEAP32[$156>>2]|0;
        $158 = (_sqlite3SchemaToIndex($154,$157)|0);
        HEAP32[$iTabDb>>2] = $158;
        HEAP32[$code>>2] = 7;
        $159 = HEAP32[$iTabDb>>2]|0;
        $160 = HEAP32[$db>>2]|0;
        $161 = ((($160)) + 16|0);
        $162 = HEAP32[$161>>2]|0;
        $163 = (($162) + ($159<<4)|0);
        $164 = HEAP32[$163>>2]|0;
        HEAP32[$zDb>>2] = $164;
        $165 = HEAP32[$8>>2]|0;
        $166 = ($165|0)!=(0);
        if ($166) {
         $167 = HEAP32[$db>>2]|0;
         $168 = ((($167)) + 16|0);
         $169 = HEAP32[$168>>2]|0;
         $170 = ((($169)) + 16|0);
         $171 = HEAP32[$170>>2]|0;
         $173 = $171;
        } else {
         $172 = HEAP32[$zDb>>2]|0;
         $173 = $172;
        }
        HEAP32[$zDbTrig>>2] = $173;
        $174 = HEAP32[$iTabDb>>2]|0;
        $175 = ($174|0)==(1);
        $176 = HEAP32[$8>>2]|0;
        $177 = ($176|0)!=(0);
        $or$cond9 = $175 | $177;
        if ($or$cond9) {
         HEAP32[$code>>2] = 5;
        }
        $178 = HEAP32[$0>>2]|0;
        $179 = HEAP32[$code>>2]|0;
        $180 = HEAP32[$zName>>2]|0;
        $181 = HEAP32[$pTab>>2]|0;
        $182 = HEAP32[$181>>2]|0;
        $183 = HEAP32[$zDbTrig>>2]|0;
        $184 = (_sqlite3AuthCheck($178,$179,$180,$182,$183)|0);
        $185 = ($184|0)!=(0);
        if (!($185)) {
         $186 = HEAP32[$0>>2]|0;
         $187 = HEAP32[$iTabDb>>2]|0;
         $188 = ($187|0)==(1);
         $189 = $188 ? 21260 : 21279;
         $190 = HEAP32[$zDb>>2]|0;
         $191 = (_sqlite3AuthCheck($186,18,$189,0,$190)|0);
         $192 = ($191|0)!=(0);
         if (!($192)) {
          $193 = HEAP32[$3>>2]|0;
          $194 = ($193|0)==(49);
          if ($194) {
           HEAP32[$3>>2] = 35;
          }
          $195 = HEAP32[$db>>2]|0;
          $196 = (_sqlite3DbMallocZero($195,36,0)|0);
          HEAP32[$pTrigger>>2] = $196;
          $197 = HEAP32[$pTrigger>>2]|0;
          $198 = ($197|0)==(0|0);
          if ($198) {
           break;
          }
          $199 = HEAP32[$zName>>2]|0;
          $200 = HEAP32[$pTrigger>>2]|0;
          HEAP32[$200>>2] = $199;
          HEAP32[$zName>>2] = 0;
          $201 = HEAP32[$db>>2]|0;
          $202 = HEAP32[$6>>2]|0;
          $203 = ((($202)) + 8|0);
          $204 = ((($203)) + 8|0);
          $205 = HEAP32[$204>>2]|0;
          $206 = (_sqlite3DbStrDup($201,$205)|0);
          $207 = HEAP32[$pTrigger>>2]|0;
          $208 = ((($207)) + 4|0);
          HEAP32[$208>>2] = $206;
          $209 = HEAP32[$iDb>>2]|0;
          $210 = HEAP32[$db>>2]|0;
          $211 = ((($210)) + 16|0);
          $212 = HEAP32[$211>>2]|0;
          $213 = (($212) + ($209<<4)|0);
          $214 = ((($213)) + 12|0);
          $215 = HEAP32[$214>>2]|0;
          $216 = HEAP32[$pTrigger>>2]|0;
          $217 = ((($216)) + 20|0);
          HEAP32[$217>>2] = $215;
          $218 = HEAP32[$pTab>>2]|0;
          $219 = ((($218)) + 64|0);
          $220 = HEAP32[$219>>2]|0;
          $221 = HEAP32[$pTrigger>>2]|0;
          $222 = ((($221)) + 24|0);
          HEAP32[$222>>2] = $220;
          $223 = HEAP32[$4>>2]|0;
          $224 = $223&255;
          $225 = HEAP32[$pTrigger>>2]|0;
          $226 = ((($225)) + 8|0);
          HEAP8[$226>>0] = $224;
          $227 = HEAP32[$3>>2]|0;
          $228 = ($227|0)==(35);
          $229 = $228 ? 1 : 2;
          $230 = $229&255;
          $231 = HEAP32[$pTrigger>>2]|0;
          $232 = ((($231)) + 9|0);
          HEAP8[$232>>0] = $230;
          $233 = HEAP32[$db>>2]|0;
          $234 = HEAP32[$7>>2]|0;
          $235 = (_sqlite3ExprDup($233,$234,1)|0);
          $236 = HEAP32[$pTrigger>>2]|0;
          $237 = ((($236)) + 12|0);
          HEAP32[$237>>2] = $235;
          $238 = HEAP32[$db>>2]|0;
          $239 = HEAP32[$5>>2]|0;
          $240 = (_sqlite3IdListDup($238,$239)|0);
          $241 = HEAP32[$pTrigger>>2]|0;
          $242 = ((($241)) + 16|0);
          HEAP32[$242>>2] = $240;
          $243 = HEAP32[$pTrigger>>2]|0;
          $244 = HEAP32[$0>>2]|0;
          $245 = ((($244)) + 492|0);
          HEAP32[$245>>2] = $243;
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $246 = HEAP32[$db>>2]|0;
 $247 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($246,$247);
 $248 = HEAP32[$db>>2]|0;
 $249 = HEAP32[$6>>2]|0;
 _sqlite3SrcListDelete($248,$249);
 $250 = HEAP32[$db>>2]|0;
 $251 = HEAP32[$5>>2]|0;
 _sqlite3IdListDelete($250,$251);
 $252 = HEAP32[$db>>2]|0;
 $253 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($252,$253);
 $254 = HEAP32[$0>>2]|0;
 $255 = ((($254)) + 492|0);
 $256 = HEAP32[$255>>2]|0;
 $257 = ($256|0)!=(0|0);
 if ($257) {
  STACKTOP = sp;return;
 }
 $258 = HEAP32[$db>>2]|0;
 $259 = HEAP32[$pTrigger>>2]|0;
 _sqlite3DeleteTrigger($258,$259);
 STACKTOP = sp;return;
}
function _sqlite3TriggerUpdateStep($db,$pTableName,$pEList,$pWhere,$orconf) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pEList = $pEList|0;
 $pWhere = $pWhere|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 20|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pWhere;
 HEAP8[$4>>0] = $orconf;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_triggerStepAllocate($5,110,$6)|0);
 HEAP32[$pTriggerStep>>2] = $7;
 $8 = HEAP32[$pTriggerStep>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_sqlite3ExprListDup($10,$11,1)|0);
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 20|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_sqlite3ExprDup($15,$16,1)|0);
  $18 = HEAP32[$pTriggerStep>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP8[$4>>0]|0;
  $21 = HEAP32[$pTriggerStep>>2]|0;
  $22 = ((($21)) + 1|0);
  HEAP8[$22>>0] = $20;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 _sqlite3ExprListDelete($23,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($25,$26);
 $27 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3TriggerInsertStep($db,$pTableName,$pColumn,$pSelect,$orconf) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pColumn = $pColumn|0;
 $pSelect = $pSelect|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 20|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pColumn;
 HEAP32[$3>>2] = $pSelect;
 HEAP8[$4>>0] = $orconf;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_triggerStepAllocate($5,108,$6)|0);
 HEAP32[$pTriggerStep>>2] = $7;
 $8 = HEAP32[$pTriggerStep>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3SelectDup($10,$11,1)|0);
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 8|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$pTriggerStep>>2]|0;
  $17 = ((($16)) + 24|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP8[$4>>0]|0;
  $19 = HEAP32[$pTriggerStep>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP8[$20>>0] = $18;
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3SelectDelete($22,$23);
  $24 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $21 = HEAP32[$2>>2]|0;
  _sqlite3IdListDelete($10,$21);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3SelectDelete($22,$23);
  $24 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3TriggerDeleteStep($db,$pTableName,$pWhere) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pWhere;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_triggerStepAllocate($3,109,$4)|0);
 HEAP32[$pTriggerStep>>2] = $5;
 $6 = HEAP32[$pTriggerStep>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3ExprDup($8,$9,1)|0);
  $11 = HEAP32[$pTriggerStep>>2]|0;
  $12 = ((($11)) + 16|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP8[$14>>0] = 10;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($15,$16);
 $17 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3TriggerSelectStep($db,$pSelect) {
 $db = $db|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pTriggerStep = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3DbMallocZero($3,36,0)|0);
 HEAP32[$pTriggerStep>>2] = $4;
 $5 = HEAP32[$pTriggerStep>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  _sqlite3SelectDelete($7,$8);
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $9 = HEAP32[$pTriggerStep>>2]|0;
  HEAP8[$9>>0] = 119;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$pTriggerStep>>2]|0;
  $12 = ((($11)) + 8|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP8[$14>>0] = 10;
  $15 = HEAP32[$pTriggerStep>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _sqlite3DropTrigger($pParse,$pName,$noErr) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $j = 0, $pTrigger = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zDb = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pTrigger = sp + 28|0;
 $i = sp + 24|0;
 $zDb = sp + 20|0;
 $zName = sp + 16|0;
 $db = sp + 12|0;
 $j = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $noErr;
 HEAP32[$pTrigger>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 65|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3ReadSchema($9)|0);
 $11 = (0)!=($10|0);
 if ($11) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$zDb>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$zName>>2] = $19;
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$db>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = ($25|0)<(2);
  $27 = HEAP32[$i>>2]|0;
  $28 = $27 ^ 1;
  $29 = $26 ? $28 : $27;
  HEAP32[$j>>2] = $29;
  $30 = HEAP32[$zDb>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = HEAP32[$j>>2]|0;
   $33 = HEAP32[$db>>2]|0;
   $34 = ((($33)) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<4)|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$zDb>>2]|0;
   $39 = (_sqlite3_stricmp($37,$38)|0);
   $40 = ($39|0)!=(0);
   if (!($40)) {
    label = 7;
   }
  } else {
   label = 7;
  }
  if ((label|0) == 7) {
   label = 0;
   $41 = HEAP32[$j>>2]|0;
   $42 = HEAP32[$db>>2]|0;
   $43 = ((($42)) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($41<<4)|0);
   $46 = ((($45)) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ((($47)) + 40|0);
   $49 = HEAP32[$zName>>2]|0;
   $50 = (_sqlite3HashFind($48,$49)|0);
   HEAP32[$pTrigger>>2] = $50;
   $51 = HEAP32[$pTrigger>>2]|0;
   $52 = ($51|0)!=(0|0);
   if ($52) {
    break;
   }
  }
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$pTrigger>>2]|0;
 $56 = ($55|0)!=(0|0);
 if ($56) {
  $64 = HEAP32[$0>>2]|0;
  $65 = HEAP32[$pTrigger>>2]|0;
  _sqlite3DropTriggerPtr($64,$65);
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = ($57|0)!=(0);
 $59 = HEAP32[$0>>2]|0;
 if ($58) {
  $61 = HEAP32[$zDb>>2]|0;
  _sqlite3CodeVerifyNamedSchema($59,$61);
 } else {
  $60 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $60;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 0;
  _sqlite3ErrorMsg($59,36125,$vararg_buffer);
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 17|0);
 HEAP8[$63>>0] = 1;
 $66 = HEAP32[$db>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($66,$67);
 STACKTOP = sp;return;
}
function _sqlite3Attach($pParse,$p,$pDbname,$pKey) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDbname = $pDbname|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDbname;
 HEAP32[$3>>2] = $pKey;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 _codeAttach($4,24,3600,$5,$6,$7,$8);
 STACKTOP = sp;return;
}
function _sqlite3Detach($pParse,$pDbname) {
 $pParse = $pParse|0;
 $pDbname = $pDbname|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pDbname;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _codeAttach($2,25,3696,$3,0,0,$4);
 STACKTOP = sp;return;
}
function _sqlite3Reindex($pParse,$pName1,$pName2) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $pColl = 0, $pIndex = 0, $pObjName = 0;
 var $pTab = 0, $vararg_buffer = 0, $z = 0, $zColl = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $pColl = sp + 36|0;
 $z = sp + 32|0;
 $zDb = sp + 28|0;
 $pTab = sp + 24|0;
 $pIndex = sp + 20|0;
 $iDb = sp + 16|0;
 $db = sp + 12|0;
 $pObjName = sp + 8|0;
 $zColl = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _reindexDatabases($10,0);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  label = 6;
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0|0);
  if ($15) {
   label = 6;
  }
 }
 do {
  if ((label|0) == 6) {
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (_sqlite3NameFromToken($17,$18)|0);
   HEAP32[$zColl>>2] = $19;
   $20 = HEAP32[$zColl>>2]|0;
   $21 = ($20|0)!=(0|0);
   if (!($21)) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$db>>2]|0;
   $23 = HEAP32[$db>>2]|0;
   $24 = ((($23)) + 62|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = HEAP32[$zColl>>2]|0;
   $27 = (_sqlite3FindCollSeq($22,$25,$26,0)|0);
   HEAP32[$pColl>>2] = $27;
   $28 = HEAP32[$pColl>>2]|0;
   $29 = ($28|0)!=(0|0);
   if (!($29)) {
    $34 = HEAP32[$db>>2]|0;
    $35 = HEAP32[$zColl>>2]|0;
    _sqlite3DbFree($34,$35);
    break;
   }
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$zColl>>2]|0;
   _reindexDatabases($30,$31);
   $32 = HEAP32[$db>>2]|0;
   $33 = HEAP32[$zColl>>2]|0;
   _sqlite3DbFree($32,$33);
   STACKTOP = sp;return;
  }
 } while(0);
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = (_sqlite3TwoPartName($36,$37,$38,$pObjName)|0);
 HEAP32[$iDb>>2] = $39;
 $40 = HEAP32[$iDb>>2]|0;
 $41 = ($40|0)<(0);
 if ($41) {
  STACKTOP = sp;return;
 }
 $42 = HEAP32[$db>>2]|0;
 $43 = HEAP32[$pObjName>>2]|0;
 $44 = (_sqlite3NameFromToken($42,$43)|0);
 HEAP32[$z>>2] = $44;
 $45 = HEAP32[$z>>2]|0;
 $46 = ($45|0)==(0|0);
 if ($46) {
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$iDb>>2]|0;
 $48 = HEAP32[$db>>2]|0;
 $49 = ((($48)) + 16|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + ($47<<4)|0);
 $52 = HEAP32[$51>>2]|0;
 HEAP32[$zDb>>2] = $52;
 $53 = HEAP32[$db>>2]|0;
 $54 = HEAP32[$z>>2]|0;
 $55 = HEAP32[$zDb>>2]|0;
 $56 = (_sqlite3FindTable($53,$54,$55)|0);
 HEAP32[$pTab>>2] = $56;
 $57 = HEAP32[$pTab>>2]|0;
 $58 = ($57|0)!=(0|0);
 if ($58) {
  $59 = HEAP32[$0>>2]|0;
  $60 = HEAP32[$pTab>>2]|0;
  _reindexTable($59,$60,0);
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($61,$62);
  STACKTOP = sp;return;
 }
 $63 = HEAP32[$db>>2]|0;
 $64 = HEAP32[$z>>2]|0;
 $65 = HEAP32[$zDb>>2]|0;
 $66 = (_sqlite3FindIndex($63,$64,$65)|0);
 HEAP32[$pIndex>>2] = $66;
 $67 = HEAP32[$db>>2]|0;
 $68 = HEAP32[$z>>2]|0;
 _sqlite3DbFree($67,$68);
 $69 = HEAP32[$pIndex>>2]|0;
 $70 = ($69|0)!=(0|0);
 $71 = HEAP32[$0>>2]|0;
 if ($70) {
  $72 = HEAP32[$iDb>>2]|0;
  _sqlite3BeginWriteOperation($71,0,$72);
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$pIndex>>2]|0;
  _sqlite3RefillIndex($73,$74,-1);
  STACKTOP = sp;return;
 } else {
  _sqlite3ErrorMsg($71,36662,$vararg_buffer);
  STACKTOP = sp;return;
 }
}
function _sqlite3Analyze($pParse,$pName1,$pName2) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $db = 0, $i = 0, $iDb = 0, $pIdx = 0, $pTab = 0, $pTableName = 0, $v = 0;
 var $z = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $db = sp + 32|0;
 $iDb = sp + 28|0;
 $i = sp + 24|0;
 $z = sp + 20|0;
 $zDb = sp + 16|0;
 $pTab = sp + 12|0;
 $pIdx = sp + 8|0;
 $pTableName = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 L4: do {
  if ($9) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $10 = HEAP32[$i>>2]|0;
    $11 = HEAP32[$db>>2]|0;
    $12 = ((($11)) + 20|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = ($10|0)<($13|0);
    if (!($14)) {
     break L4;
    }
    $15 = HEAP32[$i>>2]|0;
    $16 = ($15|0)==(1);
    if (!($16)) {
     $17 = HEAP32[$0>>2]|0;
     $18 = HEAP32[$i>>2]|0;
     _analyzeDatabase($17,$18);
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = (($19) + 1)|0;
    HEAP32[$i>>2] = $20;
   }
  } else {
   $21 = HEAP32[$2>>2]|0;
   $22 = ((($21)) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)==(0);
   if (!($24)) {
    $53 = HEAP32[$0>>2]|0;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = (_sqlite3TwoPartName($53,$54,$55,$pTableName)|0);
    HEAP32[$iDb>>2] = $56;
    $57 = HEAP32[$iDb>>2]|0;
    $58 = ($57|0)>=(0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$iDb>>2]|0;
    $60 = HEAP32[$db>>2]|0;
    $61 = ((($60)) + 16|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($59<<4)|0);
    $64 = HEAP32[$63>>2]|0;
    HEAP32[$zDb>>2] = $64;
    $65 = HEAP32[$db>>2]|0;
    $66 = HEAP32[$pTableName>>2]|0;
    $67 = (_sqlite3NameFromToken($65,$66)|0);
    HEAP32[$z>>2] = $67;
    $68 = HEAP32[$z>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     break;
    }
    $70 = HEAP32[$db>>2]|0;
    $71 = HEAP32[$z>>2]|0;
    $72 = HEAP32[$zDb>>2]|0;
    $73 = (_sqlite3FindIndex($70,$71,$72)|0);
    HEAP32[$pIdx>>2] = $73;
    $74 = ($73|0)!=(0|0);
    $75 = HEAP32[$0>>2]|0;
    if ($74) {
     $76 = HEAP32[$pIdx>>2]|0;
     $77 = ((($76)) + 12|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$pIdx>>2]|0;
     _analyzeTable($75,$78,$79);
    } else {
     $80 = HEAP32[$z>>2]|0;
     $81 = HEAP32[$zDb>>2]|0;
     $82 = (_sqlite3LocateTable($75,0,$80,$81)|0);
     HEAP32[$pTab>>2] = $82;
     $83 = ($82|0)!=(0|0);
     if ($83) {
      $84 = HEAP32[$0>>2]|0;
      $85 = HEAP32[$pTab>>2]|0;
      _analyzeTable($84,$85,0);
     }
    }
    $86 = HEAP32[$db>>2]|0;
    $87 = HEAP32[$z>>2]|0;
    _sqlite3DbFree($86,$87);
    break;
   }
   $25 = HEAP32[$db>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = (_sqlite3FindDb($25,$26)|0);
   HEAP32[$iDb>>2] = $27;
   $28 = HEAP32[$iDb>>2]|0;
   $29 = ($28|0)>=(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$iDb>>2]|0;
    _analyzeDatabase($30,$31);
    break;
   }
   $32 = HEAP32[$db>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = (_sqlite3NameFromToken($32,$33)|0);
   HEAP32[$z>>2] = $34;
   $35 = HEAP32[$z>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    $38 = HEAP32[$z>>2]|0;
    $39 = (_sqlite3FindIndex($37,$38,0)|0);
    HEAP32[$pIdx>>2] = $39;
    $40 = ($39|0)!=(0|0);
    $41 = HEAP32[$0>>2]|0;
    if ($40) {
     $42 = HEAP32[$pIdx>>2]|0;
     $43 = ((($42)) + 12|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$pIdx>>2]|0;
     _analyzeTable($41,$44,$45);
    } else {
     $46 = HEAP32[$z>>2]|0;
     $47 = (_sqlite3LocateTable($41,0,$46,0)|0);
     HEAP32[$pTab>>2] = $47;
     $48 = ($47|0)!=(0|0);
     if ($48) {
      $49 = HEAP32[$0>>2]|0;
      $50 = HEAP32[$pTab>>2]|0;
      _analyzeTable($49,$50,0);
     }
    }
    $51 = HEAP32[$db>>2]|0;
    $52 = HEAP32[$z>>2]|0;
    _sqlite3DbFree($51,$52);
   }
  }
 } while(0);
 $88 = HEAP32[$0>>2]|0;
 $89 = (_sqlite3GetVdbe($88)|0);
 HEAP32[$v>>2] = $89;
 $90 = HEAP32[$v>>2]|0;
 $91 = ($90|0)!=(0|0);
 if (!($91)) {
  STACKTOP = sp;return;
 }
 $92 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp0($92,147)|0);
 STACKTOP = sp;return;
}
function _sqlite3AlterRenameTable($pParse,$pSrc,$pName) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $nTabName = 0, $p = 0, $pFrom = 0, $pTab = 0, $pVTab = 0, $savedDbFlags = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer22 = 0, $vararg_buffer27 = 0, $vararg_buffer4 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0;
 var $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr25 = 0, $vararg_ptr26 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zDb = 0, $zName = 0, $zTabName = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer27 = sp + 96|0;
 $vararg_buffer22 = sp + 80|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $iDb = sp + 160|0;
 $zDb = sp + 156|0;
 $pTab = sp + 152|0;
 $zName = sp + 148|0;
 $db = sp + 144|0;
 $nTabName = sp + 140|0;
 $zTabName = sp + 136|0;
 $v = sp + 132|0;
 $zWhere = sp + 128|0;
 $pVTab = sp + 124|0;
 $savedDbFlags = sp + 120|0;
 $i = sp + 116|0;
 $p = sp + 112|0;
 $pFrom = sp + 108|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pName;
 HEAP32[$zName>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 HEAP32[$zWhere>>2] = 0;
 HEAP32[$pVTab>>2] = 0;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$savedDbFlags>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 65|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 8|0);
   $15 = (_sqlite3LocateTableItem($12,0,$14)|0);
   HEAP32[$pTab>>2] = $15;
   $16 = HEAP32[$pTab>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$18>>2]|0;
    $20 = HEAP32[$pTab>>2]|0;
    $21 = ((($20)) + 64|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (_sqlite3SchemaToIndex($19,$22)|0);
    HEAP32[$iDb>>2] = $23;
    $24 = HEAP32[$iDb>>2]|0;
    $25 = HEAP32[$db>>2]|0;
    $26 = ((($25)) + 16|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (($27) + ($24<<4)|0);
    $29 = HEAP32[$28>>2]|0;
    HEAP32[$zDb>>2] = $29;
    $30 = HEAP32[$db>>2]|0;
    $31 = ((($30)) + 24|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 2097152;
    HEAP32[$31>>2] = $33;
    $34 = HEAP32[$db>>2]|0;
    $35 = HEAP32[$2>>2]|0;
    $36 = (_sqlite3NameFromToken($34,$35)|0);
    HEAP32[$zName>>2] = $36;
    $37 = HEAP32[$zName>>2]|0;
    $38 = ($37|0)!=(0|0);
    if ($38) {
     $39 = HEAP32[$db>>2]|0;
     $40 = HEAP32[$zName>>2]|0;
     $41 = HEAP32[$zDb>>2]|0;
     $42 = (_sqlite3FindTable($39,$40,$41)|0);
     $43 = ($42|0)!=(0|0);
     if (!($43)) {
      $44 = HEAP32[$db>>2]|0;
      $45 = HEAP32[$zName>>2]|0;
      $46 = HEAP32[$zDb>>2]|0;
      $47 = (_sqlite3FindIndex($44,$45,$46)|0);
      $48 = ($47|0)!=(0|0);
      if (!($48)) {
       $51 = HEAP32[$0>>2]|0;
       $52 = HEAP32[$pTab>>2]|0;
       $53 = HEAP32[$52>>2]|0;
       $54 = (_isSystemTable($51,$53)|0);
       $55 = (0)!=($54|0);
       if ($55) {
        break;
       }
       $56 = HEAP32[$0>>2]|0;
       $57 = HEAP32[$zName>>2]|0;
       $58 = (_sqlite3CheckObjectName($56,$57)|0);
       $59 = (0)!=($58|0);
       if ($59) {
        break;
       }
       $60 = HEAP32[$pTab>>2]|0;
       $61 = ((($60)) + 12|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = ($62|0)!=(0|0);
       $64 = HEAP32[$0>>2]|0;
       if ($63) {
        $65 = HEAP32[$pTab>>2]|0;
        $66 = HEAP32[$65>>2]|0;
        HEAP32[$vararg_buffer1>>2] = $66;
        _sqlite3ErrorMsg($64,36882,$vararg_buffer1);
        break;
       }
       $67 = HEAP32[$zDb>>2]|0;
       $68 = HEAP32[$pTab>>2]|0;
       $69 = HEAP32[$68>>2]|0;
       $70 = (_sqlite3AuthCheck($64,26,$67,$69,0)|0);
       $71 = ($70|0)!=(0);
       if ($71) {
        break;
       }
       $72 = HEAP32[$0>>2]|0;
       $73 = HEAP32[$pTab>>2]|0;
       $74 = (_sqlite3ViewGetColumnNames($72,$73)|0);
       $75 = ($74|0)!=(0);
       if ($75) {
        break;
       }
       $76 = HEAP32[$pTab>>2]|0;
       $77 = ((($76)) + 42|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78&255;
       $80 = $79 & 16;
       $81 = ($80|0)!=(0);
       if ($81) {
        $82 = HEAP32[$db>>2]|0;
        $83 = HEAP32[$pTab>>2]|0;
        $84 = (_sqlite3GetVTable($82,$83)|0);
        HEAP32[$pVTab>>2] = $84;
        $85 = HEAP32[$pVTab>>2]|0;
        $86 = ((($85)) + 8|0);
        $87 = HEAP32[$86>>2]|0;
        $88 = HEAP32[$87>>2]|0;
        $89 = ((($88)) + 76|0);
        $90 = HEAP32[$89>>2]|0;
        $91 = ($90|0)==(0|0);
        if ($91) {
         HEAP32[$pVTab>>2] = 0;
        }
       }
       $92 = HEAP32[$0>>2]|0;
       $93 = (_sqlite3GetVdbe($92)|0);
       HEAP32[$v>>2] = $93;
       $94 = HEAP32[$v>>2]|0;
       $95 = ($94|0)==(0|0);
       if ($95) {
        break;
       }
       $96 = HEAP32[$0>>2]|0;
       $97 = HEAP32[$pVTab>>2]|0;
       $98 = ($97|0)!=(0|0);
       $99 = $98&1;
       $100 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($96,$99,$100);
       $101 = HEAP32[$0>>2]|0;
       $102 = HEAP32[$iDb>>2]|0;
       _sqlite3ChangeCookie($101,$102);
       $103 = HEAP32[$pVTab>>2]|0;
       $104 = ($103|0)!=(0|0);
       if ($104) {
        $105 = HEAP32[$0>>2]|0;
        $106 = ((($105)) + 76|0);
        $107 = HEAP32[$106>>2]|0;
        $108 = (($107) + 1)|0;
        HEAP32[$106>>2] = $108;
        HEAP32[$i>>2] = $108;
        $109 = HEAP32[$v>>2]|0;
        $110 = HEAP32[$i>>2]|0;
        $111 = HEAP32[$zName>>2]|0;
        (_sqlite3VdbeLoadString($109,$110,$111)|0);
        $112 = HEAP32[$v>>2]|0;
        $113 = HEAP32[$i>>2]|0;
        $114 = HEAP32[$pVTab>>2]|0;
        (_sqlite3VdbeAddOp4($112,155,$113,0,0,$114,-10)|0);
        $115 = HEAP32[$0>>2]|0;
        _sqlite3MayAbort($115);
       }
       $116 = HEAP32[$pTab>>2]|0;
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$zTabName>>2] = $117;
       $118 = HEAP32[$zTabName>>2]|0;
       $119 = (_sqlite3Utf8CharLen($118,-1)|0);
       HEAP32[$nTabName>>2] = $119;
       $120 = HEAP32[$db>>2]|0;
       $121 = ((($120)) + 24|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = $122 & 524288;
       $124 = ($123|0)!=(0);
       if ($124) {
        $125 = HEAP32[$0>>2]|0;
        $126 = HEAP32[$pTab>>2]|0;
        $127 = (_whereForeignKeys($125,$126)|0);
        HEAP32[$zWhere>>2] = $127;
        $128 = ($127|0)!=(0|0);
        if ($128) {
         $129 = HEAP32[$0>>2]|0;
         $130 = HEAP32[$zDb>>2]|0;
         $131 = HEAP32[$iDb>>2]|0;
         $132 = ($131|0)==(1);
         $133 = $132 ? 21260 : 21279;
         $134 = HEAP32[$zTabName>>2]|0;
         $135 = HEAP32[$zName>>2]|0;
         $136 = HEAP32[$zWhere>>2]|0;
         HEAP32[$vararg_buffer4>>2] = $130;
         $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
         HEAP32[$vararg_ptr7>>2] = $133;
         $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
         HEAP32[$vararg_ptr8>>2] = $134;
         $vararg_ptr9 = ((($vararg_buffer4)) + 12|0);
         HEAP32[$vararg_ptr9>>2] = $135;
         $vararg_ptr10 = ((($vararg_buffer4)) + 16|0);
         HEAP32[$vararg_ptr10>>2] = $136;
         _sqlite3NestedParse($129,36909,$vararg_buffer4);
         $137 = HEAP32[$db>>2]|0;
         $138 = HEAP32[$zWhere>>2]|0;
         _sqlite3DbFree($137,$138);
        }
       }
       $139 = HEAP32[$0>>2]|0;
       $140 = HEAP32[$zDb>>2]|0;
       $141 = HEAP32[$iDb>>2]|0;
       $142 = ($141|0)==(1);
       $143 = $142 ? 21260 : 21279;
       $144 = HEAP32[$zName>>2]|0;
       $145 = HEAP32[$zName>>2]|0;
       $146 = HEAP32[$zName>>2]|0;
       $147 = HEAP32[$zName>>2]|0;
       $148 = HEAP32[$zName>>2]|0;
       $149 = HEAP32[$nTabName>>2]|0;
       $150 = HEAP32[$zTabName>>2]|0;
       HEAP32[$vararg_buffer11>>2] = $140;
       $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
       HEAP32[$vararg_ptr14>>2] = $143;
       $vararg_ptr15 = ((($vararg_buffer11)) + 8|0);
       HEAP32[$vararg_ptr15>>2] = $144;
       $vararg_ptr16 = ((($vararg_buffer11)) + 12|0);
       HEAP32[$vararg_ptr16>>2] = $145;
       $vararg_ptr17 = ((($vararg_buffer11)) + 16|0);
       HEAP32[$vararg_ptr17>>2] = $146;
       $vararg_ptr18 = ((($vararg_buffer11)) + 20|0);
       HEAP32[$vararg_ptr18>>2] = $147;
       $vararg_ptr19 = ((($vararg_buffer11)) + 24|0);
       HEAP32[$vararg_ptr19>>2] = $148;
       $vararg_ptr20 = ((($vararg_buffer11)) + 28|0);
       HEAP32[$vararg_ptr20>>2] = $149;
       $vararg_ptr21 = ((($vararg_buffer11)) + 32|0);
       HEAP32[$vararg_ptr21>>2] = $150;
       _sqlite3NestedParse($139,36978,$vararg_buffer11);
       $151 = HEAP32[$db>>2]|0;
       $152 = HEAP32[$zDb>>2]|0;
       $153 = (_sqlite3FindTable($151,28785,$152)|0);
       $154 = ($153|0)!=(0|0);
       if ($154) {
        $155 = HEAP32[$0>>2]|0;
        $156 = HEAP32[$zDb>>2]|0;
        $157 = HEAP32[$zName>>2]|0;
        $158 = HEAP32[$pTab>>2]|0;
        $159 = HEAP32[$158>>2]|0;
        HEAP32[$vararg_buffer22>>2] = $156;
        $vararg_ptr25 = ((($vararg_buffer22)) + 4|0);
        HEAP32[$vararg_ptr25>>2] = $157;
        $vararg_ptr26 = ((($vararg_buffer22)) + 8|0);
        HEAP32[$vararg_ptr26>>2] = $159;
        _sqlite3NestedParse($155,37362,$vararg_buffer22);
       }
       $160 = HEAP32[$0>>2]|0;
       $161 = HEAP32[$pTab>>2]|0;
       $162 = (_whereTempTriggers($160,$161)|0);
       HEAP32[$zWhere>>2] = $162;
       $163 = ($162|0)!=(0|0);
       if ($163) {
        $164 = HEAP32[$0>>2]|0;
        $165 = HEAP32[$zName>>2]|0;
        $166 = HEAP32[$zName>>2]|0;
        $167 = HEAP32[$zWhere>>2]|0;
        HEAP32[$vararg_buffer27>>2] = $165;
        $vararg_ptr30 = ((($vararg_buffer27)) + 4|0);
        HEAP32[$vararg_ptr30>>2] = $166;
        $vararg_ptr31 = ((($vararg_buffer27)) + 8|0);
        HEAP32[$vararg_ptr31>>2] = $167;
        _sqlite3NestedParse($164,37420,$vararg_buffer27);
        $168 = HEAP32[$db>>2]|0;
        $169 = HEAP32[$zWhere>>2]|0;
        _sqlite3DbFree($168,$169);
       }
       $170 = HEAP32[$db>>2]|0;
       $171 = ((($170)) + 24|0);
       $172 = HEAP32[$171>>2]|0;
       $173 = $172 & 524288;
       $174 = ($173|0)!=(0);
       L33: do {
        if ($174) {
         $175 = HEAP32[$pTab>>2]|0;
         $176 = (_sqlite3FkReferences($175)|0);
         HEAP32[$p>>2] = $176;
         while(1) {
          $177 = HEAP32[$p>>2]|0;
          $178 = ($177|0)!=(0|0);
          if (!($178)) {
           break L33;
          }
          $179 = HEAP32[$p>>2]|0;
          $180 = HEAP32[$179>>2]|0;
          HEAP32[$pFrom>>2] = $180;
          $181 = HEAP32[$pFrom>>2]|0;
          $182 = HEAP32[$pTab>>2]|0;
          $183 = ($181|0)!=($182|0);
          if ($183) {
           $184 = HEAP32[$0>>2]|0;
           $185 = HEAP32[$p>>2]|0;
           $186 = HEAP32[$185>>2]|0;
           $187 = HEAP32[$pFrom>>2]|0;
           $188 = HEAP32[$187>>2]|0;
           _reloadTableSchema($184,$186,$188);
          }
          $189 = HEAP32[$p>>2]|0;
          $190 = ((($189)) + 12|0);
          $191 = HEAP32[$190>>2]|0;
          HEAP32[$p>>2] = $191;
         }
        }
       } while(0);
       $192 = HEAP32[$0>>2]|0;
       $193 = HEAP32[$pTab>>2]|0;
       $194 = HEAP32[$zName>>2]|0;
       _reloadTableSchema($192,$193,$194);
       break;
      }
     }
     $49 = HEAP32[$0>>2]|0;
     $50 = HEAP32[$zName>>2]|0;
     HEAP32[$vararg_buffer>>2] = $50;
     _sqlite3ErrorMsg($49,36823,$vararg_buffer);
    }
   }
  }
 } while(0);
 $195 = HEAP32[$db>>2]|0;
 $196 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($195,$196);
 $197 = HEAP32[$db>>2]|0;
 $198 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($197,$198);
 $199 = HEAP32[$savedDbFlags>>2]|0;
 $200 = HEAP32[$db>>2]|0;
 $201 = ((($200)) + 24|0);
 HEAP32[$201>>2] = $199;
 STACKTOP = sp;return;
}
function _sqlite3AlterFinishAddColumn($pParse,$pColDef) {
 $pParse = $pParse|0;
 $pColDef = $pColDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $or$cond = 0, $or$cond3 = 0, $pCol = 0, $pDflt = 0, $pNew = 0, $pTab = 0, $pVal = 0, $rc = 0, $savedDbFlags = 0, $v = 0;
 var $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $zCol = 0, $zDb = 0, $zEnd = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer7 = sp + 16|0;
 $vararg_buffer5 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $pNew = sp + 116|0;
 $pTab = sp + 112|0;
 $iDb = sp + 108|0;
 $zDb = sp + 104|0;
 $zTab = sp + 100|0;
 $zCol = sp + 96|0;
 $pCol = sp + 92|0;
 $pDflt = sp + 88|0;
 $db = sp + 84|0;
 $v = sp + 80|0;
 $pVal = sp + 76|0;
 $rc = sp + 72|0;
 $zEnd = sp + 68|0;
 $savedDbFlags = sp + 64|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pColDef;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 65|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 488|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pNew>>2] = $18;
 $19 = HEAP32[$db>>2]|0;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3SchemaToIndex($19,$22)|0);
 HEAP32[$iDb>>2] = $23;
 $24 = HEAP32[$iDb>>2]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24<<4)|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$zDb>>2] = $29;
 $30 = HEAP32[$pNew>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ((($31)) + 16|0);
 HEAP32[$zTab>>2] = $32;
 $33 = HEAP32[$pNew>>2]|0;
 $34 = ((($33)) + 34|0);
 $35 = HEAP16[$34>>1]|0;
 $36 = $35 << 16 >> 16;
 $37 = (($36) - 1)|0;
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + (($37*24)|0)|0);
 HEAP32[$pCol>>2] = $41;
 $42 = HEAP32[$pCol>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pDflt>>2] = $44;
 $45 = HEAP32[$db>>2]|0;
 $46 = HEAP32[$zTab>>2]|0;
 $47 = HEAP32[$zDb>>2]|0;
 $48 = (_sqlite3FindTable($45,$46,$47)|0);
 HEAP32[$pTab>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$zDb>>2]|0;
 $51 = HEAP32[$pTab>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (_sqlite3AuthCheck($49,26,$50,$52,0)|0);
 $54 = ($53|0)!=(0);
 if ($54) {
  STACKTOP = sp;return;
 }
 $55 = HEAP32[$pDflt>>2]|0;
 $56 = ($55|0)!=(0|0);
 if ($56) {
  $57 = HEAP32[$pDflt>>2]|0;
  $58 = HEAP8[$57>>0]|0;
  $59 = $58&255;
  $60 = ($59|0)==(101);
  if ($60) {
   HEAP32[$pDflt>>2] = 0;
  }
 }
 $61 = HEAP32[$pCol>>2]|0;
 $62 = ((($61)) + 23|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = $63&255;
 $65 = $64 & 1;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($67,37598,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $68 = HEAP32[$pNew>>2]|0;
 $69 = ((($68)) + 8|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = ($70|0)!=(0|0);
 if ($71) {
  $72 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($72,37630,$vararg_buffer5);
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$db>>2]|0;
 $74 = ((($73)) + 24|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = $75 & 524288;
 $77 = ($76|0)!=(0);
 if ($77) {
  $78 = HEAP32[$pNew>>2]|0;
  $79 = ((($78)) + 16|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = ($80|0)!=(0|0);
  $82 = HEAP32[$pDflt>>2]|0;
  $83 = ($82|0)!=(0|0);
  $or$cond = $81 & $83;
  if ($or$cond) {
   $84 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($84,37657,$vararg_buffer7);
   STACKTOP = sp;return;
  }
 }
 $85 = HEAP32[$pCol>>2]|0;
 $86 = ((($85)) + 20|0);
 $87 = HEAP8[$86>>0]|0;
 $88 = $87&255;
 $89 = ($88|0)==(0);
 $90 = HEAP32[$pDflt>>2]|0;
 $91 = ($90|0)!=(0|0);
 $or$cond3 = $89 | $91;
 if (!($or$cond3)) {
  $92 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($92,37716,$vararg_buffer9);
  STACKTOP = sp;return;
 }
 $93 = HEAP32[$pDflt>>2]|0;
 $94 = ($93|0)!=(0|0);
 do {
  if ($94) {
   HEAP32[$pVal>>2] = 0;
   $95 = HEAP32[$db>>2]|0;
   $96 = HEAP32[$pDflt>>2]|0;
   $97 = (_sqlite3ValueFromExpr($95,$96,1,65,$pVal)|0);
   HEAP32[$rc>>2] = $97;
   $98 = HEAP32[$rc>>2]|0;
   $99 = ($98|0)!=(0);
   if ($99) {
    STACKTOP = sp;return;
   }
   $100 = HEAP32[$pVal>>2]|0;
   $101 = ($100|0)!=(0|0);
   if ($101) {
    $103 = HEAP32[$pVal>>2]|0;
    _sqlite3ValueFree($103);
    break;
   }
   $102 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($102,37769,$vararg_buffer11);
   STACKTOP = sp;return;
  }
 } while(0);
 $104 = HEAP32[$db>>2]|0;
 $105 = HEAP32[$1>>2]|0;
 $106 = HEAP32[$105>>2]|0;
 $107 = HEAP32[$1>>2]|0;
 $108 = ((($107)) + 4|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = (_sqlite3DbStrNDup($104,$106,$109,0)|0);
 HEAP32[$zCol>>2] = $110;
 $111 = HEAP32[$zCol>>2]|0;
 $112 = ($111|0)!=(0|0);
 if ($112) {
  $113 = HEAP32[$1>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) - 1)|0;
  $117 = HEAP32[$zCol>>2]|0;
  $118 = (($117) + ($116)|0);
  HEAP32[$zEnd>>2] = $118;
  $119 = HEAP32[$db>>2]|0;
  $120 = ((($119)) + 24|0);
  $121 = HEAP32[$120>>2]|0;
  HEAP32[$savedDbFlags>>2] = $121;
  while(1) {
   $122 = HEAP32[$zEnd>>2]|0;
   $123 = HEAP32[$zCol>>2]|0;
   $124 = ($122>>>0)>($123>>>0);
   if (!($124)) {
    break;
   }
   $125 = HEAP32[$zEnd>>2]|0;
   $126 = HEAP8[$125>>0]|0;
   $127 = $126 << 24 >> 24;
   $128 = ($127|0)==(59);
   if (!($128)) {
    $129 = HEAP32[$zEnd>>2]|0;
    $130 = HEAP8[$129>>0]|0;
    $131 = $130&255;
    $132 = (17533 + ($131)|0);
    $133 = HEAP8[$132>>0]|0;
    $134 = $133&255;
    $135 = $134 & 1;
    $136 = ($135|0)!=(0);
    if (!($136)) {
     break;
    }
   }
   $137 = HEAP32[$zEnd>>2]|0;
   $138 = ((($137)) + -1|0);
   HEAP32[$zEnd>>2] = $138;
   HEAP8[$137>>0] = 0;
  }
  $139 = HEAP32[$db>>2]|0;
  $140 = ((($139)) + 24|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = $141 | 2097152;
  HEAP32[$140>>2] = $142;
  $143 = HEAP32[$0>>2]|0;
  $144 = HEAP32[$zDb>>2]|0;
  $145 = HEAP32[$iDb>>2]|0;
  $146 = ($145|0)==(1);
  $147 = $146 ? 21260 : 21279;
  $148 = HEAP32[$pNew>>2]|0;
  $149 = ((($148)) + 44|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = HEAP32[$zCol>>2]|0;
  $152 = HEAP32[$pNew>>2]|0;
  $153 = ((($152)) + 44|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = (($154) + 1)|0;
  $156 = HEAP32[$zTab>>2]|0;
  HEAP32[$vararg_buffer13>>2] = $144;
  $vararg_ptr15 = ((($vararg_buffer13)) + 4|0);
  HEAP32[$vararg_ptr15>>2] = $147;
  $vararg_ptr16 = ((($vararg_buffer13)) + 8|0);
  HEAP32[$vararg_ptr16>>2] = $150;
  $vararg_ptr17 = ((($vararg_buffer13)) + 12|0);
  HEAP32[$vararg_ptr17>>2] = $151;
  $vararg_ptr18 = ((($vararg_buffer13)) + 16|0);
  HEAP32[$vararg_ptr18>>2] = $155;
  $vararg_ptr19 = ((($vararg_buffer13)) + 20|0);
  HEAP32[$vararg_ptr19>>2] = $156;
  _sqlite3NestedParse($143,37815,$vararg_buffer13);
  $157 = HEAP32[$db>>2]|0;
  $158 = HEAP32[$zCol>>2]|0;
  _sqlite3DbFree($157,$158);
  $159 = HEAP32[$savedDbFlags>>2]|0;
  $160 = HEAP32[$db>>2]|0;
  $161 = ((($160)) + 24|0);
  HEAP32[$161>>2] = $159;
 }
 $162 = HEAP32[$v>>2]|0;
 $163 = HEAP32[$iDb>>2]|0;
 (_sqlite3VdbeAddOp3($162,52,$163,2,4)|0);
 $164 = HEAP32[$0>>2]|0;
 $165 = HEAP32[$pTab>>2]|0;
 $166 = HEAP32[$pTab>>2]|0;
 $167 = HEAP32[$166>>2]|0;
 _reloadTableSchema($164,$165,$167);
 STACKTOP = sp;return;
}
function _sqlite3AlterBeginAddColumn($pParse,$pSrc) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0;
 var $iDb = 0, $nAlloc = 0, $pCol = 0, $pNew = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $pNew = sp + 48|0;
 $pTab = sp + 44|0;
 $v = sp + 40|0;
 $iDb = sp + 36|0;
 $i = sp + 32|0;
 $nAlloc = sp + 28|0;
 $db = sp + 24|0;
 $pCol = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 65|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 do {
  if (!($7)) {
   $8 = HEAP32[$0>>2]|0;
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 8|0);
   $11 = (_sqlite3LocateTableItem($8,0,$10)|0);
   HEAP32[$pTab>>2] = $11;
   $12 = HEAP32[$pTab>>2]|0;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $14 = HEAP32[$pTab>>2]|0;
    $15 = ((($14)) + 42|0);
    $16 = HEAP8[$15>>0]|0;
    $17 = $16&255;
    $18 = $17 & 16;
    $19 = ($18|0)!=(0);
    if ($19) {
     $20 = HEAP32[$0>>2]|0;
     _sqlite3ErrorMsg($20,37924,$vararg_buffer);
     break;
    }
    $21 = HEAP32[$pTab>>2]|0;
    $22 = ((($21)) + 12|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($23|0)!=(0|0);
    $25 = HEAP32[$0>>2]|0;
    if ($24) {
     _sqlite3ErrorMsg($25,37958,$vararg_buffer1);
     break;
    }
    $26 = HEAP32[$pTab>>2]|0;
    $27 = HEAP32[$26>>2]|0;
    $28 = (_isSystemTable($25,$27)|0);
    $29 = (0)!=($28|0);
    if (!($29)) {
     $30 = HEAP32[$db>>2]|0;
     $31 = HEAP32[$pTab>>2]|0;
     $32 = ((($31)) + 64|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = (_sqlite3SchemaToIndex($30,$33)|0);
     HEAP32[$iDb>>2] = $34;
     $35 = HEAP32[$db>>2]|0;
     $36 = (_sqlite3DbMallocZero($35,72,0)|0);
     HEAP32[$pNew>>2] = $36;
     $37 = HEAP32[$pNew>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$pNew>>2]|0;
      $40 = HEAP32[$0>>2]|0;
      $41 = ((($40)) + 488|0);
      HEAP32[$41>>2] = $39;
      $42 = HEAP32[$pNew>>2]|0;
      $43 = ((($42)) + 36|0);
      HEAP16[$43>>1] = 1;
      $44 = HEAP32[$pTab>>2]|0;
      $45 = ((($44)) + 34|0);
      $46 = HEAP16[$45>>1]|0;
      $47 = HEAP32[$pNew>>2]|0;
      $48 = ((($47)) + 34|0);
      HEAP16[$48>>1] = $46;
      $49 = HEAP32[$pNew>>2]|0;
      $50 = ((($49)) + 34|0);
      $51 = HEAP16[$50>>1]|0;
      $52 = $51 << 16 >> 16;
      $53 = (($52) - 1)|0;
      $54 = (($53|0) / 8)&-1;
      $55 = $54<<3;
      $56 = (($55) + 8)|0;
      HEAP32[$nAlloc>>2] = $56;
      $57 = HEAP32[$db>>2]|0;
      $58 = HEAP32[$nAlloc>>2]|0;
      $59 = ($58*24)|0;
      $60 = (_sqlite3DbMallocZero($57,$59,0)|0);
      $61 = HEAP32[$pNew>>2]|0;
      $62 = ((($61)) + 4|0);
      HEAP32[$62>>2] = $60;
      $63 = HEAP32[$db>>2]|0;
      $64 = HEAP32[$pTab>>2]|0;
      $65 = HEAP32[$64>>2]|0;
      HEAP32[$vararg_buffer3>>2] = $65;
      $66 = (_sqlite3MPrintf($63,37988,$vararg_buffer3)|0);
      $67 = HEAP32[$pNew>>2]|0;
      HEAP32[$67>>2] = $66;
      $68 = HEAP32[$pNew>>2]|0;
      $69 = ((($68)) + 4|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = ($70|0)!=(0|0);
      if ($71) {
       $72 = HEAP32[$pNew>>2]|0;
       $73 = HEAP32[$72>>2]|0;
       $74 = ($73|0)!=(0|0);
       if ($74) {
        $75 = HEAP32[$pNew>>2]|0;
        $76 = ((($75)) + 4|0);
        $77 = HEAP32[$76>>2]|0;
        $78 = HEAP32[$pTab>>2]|0;
        $79 = ((($78)) + 4|0);
        $80 = HEAP32[$79>>2]|0;
        $81 = HEAP32[$pNew>>2]|0;
        $82 = ((($81)) + 34|0);
        $83 = HEAP16[$82>>1]|0;
        $84 = $83 << 16 >> 16;
        $85 = ($84*24)|0;
        _memcpy(($77|0),($80|0),($85|0))|0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $86 = HEAP32[$i>>2]|0;
         $87 = HEAP32[$pNew>>2]|0;
         $88 = ((($87)) + 34|0);
         $89 = HEAP16[$88>>1]|0;
         $90 = $89 << 16 >> 16;
         $91 = ($86|0)<($90|0);
         if (!($91)) {
          break;
         }
         $92 = HEAP32[$i>>2]|0;
         $93 = HEAP32[$pNew>>2]|0;
         $94 = ((($93)) + 4|0);
         $95 = HEAP32[$94>>2]|0;
         $96 = (($95) + (($92*24)|0)|0);
         HEAP32[$pCol>>2] = $96;
         $97 = HEAP32[$db>>2]|0;
         $98 = HEAP32[$pCol>>2]|0;
         $99 = HEAP32[$98>>2]|0;
         $100 = (_sqlite3DbStrDup($97,$99)|0);
         $101 = HEAP32[$pCol>>2]|0;
         HEAP32[$101>>2] = $100;
         $102 = HEAP32[$pCol>>2]|0;
         $103 = ((($102)) + 16|0);
         HEAP32[$103>>2] = 0;
         $104 = HEAP32[$pCol>>2]|0;
         $105 = ((($104)) + 12|0);
         HEAP32[$105>>2] = 0;
         $106 = HEAP32[$pCol>>2]|0;
         $107 = ((($106)) + 4|0);
         HEAP32[$107>>2] = 0;
         $108 = HEAP32[$pCol>>2]|0;
         $109 = ((($108)) + 8|0);
         HEAP32[$109>>2] = 0;
         $110 = HEAP32[$i>>2]|0;
         $111 = (($110) + 1)|0;
         HEAP32[$i>>2] = $111;
        }
        $112 = HEAP32[$iDb>>2]|0;
        $113 = HEAP32[$db>>2]|0;
        $114 = ((($113)) + 16|0);
        $115 = HEAP32[$114>>2]|0;
        $116 = (($115) + ($112<<4)|0);
        $117 = ((($116)) + 12|0);
        $118 = HEAP32[$117>>2]|0;
        $119 = HEAP32[$pNew>>2]|0;
        $120 = ((($119)) + 64|0);
        HEAP32[$120>>2] = $118;
        $121 = HEAP32[$pTab>>2]|0;
        $122 = ((($121)) + 44|0);
        $123 = HEAP32[$122>>2]|0;
        $124 = HEAP32[$pNew>>2]|0;
        $125 = ((($124)) + 44|0);
        HEAP32[$125>>2] = $123;
        $126 = HEAP32[$pNew>>2]|0;
        $127 = ((($126)) + 36|0);
        HEAP16[$127>>1] = 1;
        $128 = HEAP32[$0>>2]|0;
        $129 = HEAP32[$iDb>>2]|0;
        _sqlite3BeginWriteOperation($128,0,$129);
        $130 = HEAP32[$0>>2]|0;
        $131 = (_sqlite3GetVdbe($130)|0);
        HEAP32[$v>>2] = $131;
        $132 = HEAP32[$v>>2]|0;
        $133 = ($132|0)!=(0|0);
        if ($133) {
         $134 = HEAP32[$0>>2]|0;
         $135 = HEAP32[$iDb>>2]|0;
         _sqlite3ChangeCookie($134,$135);
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $136 = HEAP32[$db>>2]|0;
 $137 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($136,$137);
 STACKTOP = sp;return;
}
function _sqlite3VtabFinishParse($pParse,$pEnd) {
 $pParse = $pParse|0;
 $pEnd = $pEnd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $iReg = 0, $pOld = 0, $pSchema = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer9 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0;
 var $zName = 0, $zStmt = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $pTab = sp + 72|0;
 $db = sp + 68|0;
 $zStmt = sp + 64|0;
 $zWhere = sp + 60|0;
 $iDb = sp + 56|0;
 $iReg = sp + 52|0;
 $v = sp + 48|0;
 $pOld = sp + 44|0;
 $pSchema = sp + 40|0;
 $zName = sp + 36|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEnd;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 488|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 _addArgumentToVtab($9);
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 516|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$pTab>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)<(1);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 140|0);
 $18 = ((($17)) + 5|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if (!($20)) {
  $21 = HEAP32[$1>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 500|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $24;
   $29 = $27;
   $30 = (($28) - ($29))|0;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($30) + ($33))|0;
   $35 = HEAP32[$0>>2]|0;
   $36 = ((($35)) + 500|0);
   $37 = ((($36)) + 4|0);
   HEAP32[$37>>2] = $34;
  }
  $38 = HEAP32[$db>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 500|0);
  HEAP32[$vararg_buffer>>2] = $40;
  $41 = (_sqlite3MPrintf($38,38007,$vararg_buffer)|0);
  HEAP32[$zStmt>>2] = $41;
  $42 = HEAP32[$db>>2]|0;
  $43 = HEAP32[$pTab>>2]|0;
  $44 = ((($43)) + 64|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3SchemaToIndex($42,$45)|0);
  HEAP32[$iDb>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$iDb>>2]|0;
  $49 = HEAP32[$db>>2]|0;
  $50 = ((($49)) + 16|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48<<4)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$iDb>>2]|0;
  $55 = ($54|0)==(1);
  $56 = $55 ? 21260 : 21279;
  $57 = HEAP32[$pTab>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$pTab>>2]|0;
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$zStmt>>2]|0;
  $62 = HEAP32[$0>>2]|0;
  $63 = ((($62)) + 392|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $53;
  $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $56;
  $vararg_ptr5 = ((($vararg_buffer1)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $58;
  $vararg_ptr6 = ((($vararg_buffer1)) + 12|0);
  HEAP32[$vararg_ptr6>>2] = $60;
  $vararg_ptr7 = ((($vararg_buffer1)) + 16|0);
  HEAP32[$vararg_ptr7>>2] = $61;
  $vararg_ptr8 = ((($vararg_buffer1)) + 20|0);
  HEAP32[$vararg_ptr8>>2] = $64;
  _sqlite3NestedParse($47,38031,$vararg_buffer1);
  $65 = HEAP32[$db>>2]|0;
  $66 = HEAP32[$zStmt>>2]|0;
  _sqlite3DbFree($65,$66);
  $67 = HEAP32[$0>>2]|0;
  $68 = (_sqlite3GetVdbe($67)|0);
  HEAP32[$v>>2] = $68;
  $69 = HEAP32[$0>>2]|0;
  $70 = HEAP32[$iDb>>2]|0;
  _sqlite3ChangeCookie($69,$70);
  $71 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp2($71,147,0,0)|0);
  $72 = HEAP32[$db>>2]|0;
  $73 = HEAP32[$pTab>>2]|0;
  $74 = HEAP32[$73>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $74;
  $75 = (_sqlite3MPrintf($72,38119,$vararg_buffer9)|0);
  HEAP32[$zWhere>>2] = $75;
  $76 = HEAP32[$v>>2]|0;
  $77 = HEAP32[$iDb>>2]|0;
  $78 = HEAP32[$zWhere>>2]|0;
  _sqlite3VdbeAddParseSchemaOp($76,$77,$78);
  $79 = HEAP32[$0>>2]|0;
  $80 = ((($79)) + 76|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$80>>2] = $82;
  HEAP32[$iReg>>2] = $82;
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$iReg>>2]|0;
  $85 = HEAP32[$pTab>>2]|0;
  $86 = HEAP32[$85>>2]|0;
  (_sqlite3VdbeLoadString($83,$84,$86)|0);
  $87 = HEAP32[$v>>2]|0;
  $88 = HEAP32[$iDb>>2]|0;
  $89 = HEAP32[$iReg>>2]|0;
  (_sqlite3VdbeAddOp2($87,150,$88,$89)|0);
  STACKTOP = sp;return;
 }
 $90 = HEAP32[$pTab>>2]|0;
 $91 = ((($90)) + 64|0);
 $92 = HEAP32[$91>>2]|0;
 HEAP32[$pSchema>>2] = $92;
 $93 = HEAP32[$pTab>>2]|0;
 $94 = HEAP32[$93>>2]|0;
 HEAP32[$zName>>2] = $94;
 $95 = HEAP32[$pSchema>>2]|0;
 $96 = ((($95)) + 8|0);
 $97 = HEAP32[$zName>>2]|0;
 $98 = HEAP32[$pTab>>2]|0;
 $99 = (_sqlite3HashInsert($96,$97,$98)|0);
 HEAP32[$pOld>>2] = $99;
 $100 = HEAP32[$pOld>>2]|0;
 $101 = ($100|0)!=(0|0);
 if ($101) {
  $102 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($102);
  STACKTOP = sp;return;
 } else {
  $103 = HEAP32[$0>>2]|0;
  $104 = ((($103)) + 488|0);
  HEAP32[$104>>2] = 0;
  STACKTOP = sp;return;
 }
}
function _sqlite3VtabBeginParse($pParse,$pName1,$pName2,$pModuleName,$ifNotExists) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pModuleName = $pModuleName|0;
 $ifNotExists = $ifNotExists|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $pTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $iDb = sp + 8|0;
 $pTable = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $pModuleName;
 HEAP32[$4>>2] = $ifNotExists;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 _sqlite3StartTable($5,$6,$7,0,0,1,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 488|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pTable>>2] = $11;
 $12 = HEAP32[$pTable>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$pTable>>2]|0;
 $18 = ((($17)) + 64|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$pTable>>2]|0;
 $22 = ((($21)) + 42|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $24 | 16;
 $26 = $25&255;
 HEAP8[$22>>0] = $26;
 $27 = HEAP32[$pTable>>2]|0;
 $28 = ((($27)) + 48|0);
 HEAP32[$28>>2] = 0;
 $29 = HEAP32[$db>>2]|0;
 $30 = HEAP32[$pTable>>2]|0;
 $31 = HEAP32[$db>>2]|0;
 $32 = HEAP32[$3>>2]|0;
 $33 = (_sqlite3NameFromToken($31,$32)|0);
 _addModuleArgument($29,$30,$33);
 $34 = HEAP32[$db>>2]|0;
 $35 = HEAP32[$pTable>>2]|0;
 _addModuleArgument($34,$35,0);
 $36 = HEAP32[$db>>2]|0;
 $37 = HEAP32[$pTable>>2]|0;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$pTable>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = (_sqlite3DbStrDup($38,$40)|0);
 _addModuleArgument($36,$37,$41);
 $42 = HEAP32[$3>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + ($44)|0);
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 500|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = $47;
 $52 = $50;
 $53 = (($51) - ($52))|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 500|0);
 $56 = ((($55)) + 4|0);
 HEAP32[$56>>2] = $53;
 $57 = HEAP32[$pTable>>2]|0;
 $58 = ((($57)) + 52|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0|0);
 if (!($60)) {
  STACKTOP = sp;return;
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$pTable>>2]|0;
 $63 = HEAP32[$62>>2]|0;
 $64 = HEAP32[$pTable>>2]|0;
 $65 = ((($64)) + 52|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$iDb>>2]|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 $71 = ((($70)) + 16|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = (($72) + ($68<<4)|0);
 $74 = HEAP32[$73>>2]|0;
 (_sqlite3AuthCheck($61,29,$63,$67,$74)|0);
 STACKTOP = sp;return;
}
function _sqlite3VtabArgInit($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 _addArgumentToVtab($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 516|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 516|0);
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VtabArgExtend($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 516|0);
 HEAP32[$pArg>>2] = $3;
 $4 = HEAP32[$pArg>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$pArg>>2]|0;
  HEAP32[$9>>2] = $8;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pArg>>2]|0;
  $14 = ((($13)) + 4|0);
  HEAP32[$14>>2] = $12;
  STACKTOP = sp;return;
 } else {
  $15 = ((($7)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($16)|0);
  $20 = HEAP32[$pArg>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = $19;
  $23 = $21;
  $24 = (($22) - ($23))|0;
  $25 = HEAP32[$pArg>>2]|0;
  $26 = ((($25)) + 4|0);
  HEAP32[$26>>2] = $24;
  STACKTOP = sp;return;
 }
}
function _sqlite3WithAdd($pParse,$pWith,$pName,$pArglist,$pQuery) {
 $pParse = $pParse|0;
 $pWith = $pWith|0;
 $pName = $pName|0;
 $pArglist = $pArglist|0;
 $pQuery = $pQuery|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $db = 0, $i = 0, $nByte = 0, $or$cond = 0, $pNew = 0, $vararg_buffer = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $db = sp + 20|0;
 $pNew = sp + 16|0;
 $zName = sp + 12|0;
 $i = sp + 8|0;
 $nByte = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pWith;
 HEAP32[$2>>2] = $pName;
 HEAP32[$3>>2] = $pArglist;
 HEAP32[$4>>2] = $pQuery;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_sqlite3NameFromToken($8,$9)|0);
 HEAP32[$zName>>2] = $10;
 $11 = HEAP32[$zName>>2]|0;
 $12 = ($11|0)!=(0|0);
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)!=(0|0);
 $or$cond = $12 & $14;
 L1: do {
  if ($or$cond) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $15 = HEAP32[$i>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    $17 = HEAP32[$16>>2]|0;
    $18 = ($15|0)<($17|0);
    if (!($18)) {
     break L1;
    }
    $19 = HEAP32[$zName>>2]|0;
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 8|0);
    $23 = (($22) + ($20<<4)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (_sqlite3_stricmp($19,$24)|0);
    $26 = ($25|0)==(0);
    if ($26) {
     $27 = HEAP32[$0>>2]|0;
     $28 = HEAP32[$zName>>2]|0;
     HEAP32[$vararg_buffer>>2] = $28;
     _sqlite3ErrorMsg($27,38146,$vararg_buffer);
    }
    $29 = HEAP32[$i>>2]|0;
    $30 = (($29) + 1)|0;
    HEAP32[$i>>2] = $30;
   }
  }
 } while(0);
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $34<<4;
  $36 = (24 + ($35))|0;
  HEAP32[$nByte>>2] = $36;
  $37 = HEAP32[$db>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$nByte>>2]|0;
  $40 = ($39|0)<(0);
  $41 = $40 << 31 >> 31;
  $42 = (_sqlite3DbRealloc($37,$38,$39,$41)|0);
  HEAP32[$pNew>>2] = $42;
 } else {
  $43 = HEAP32[$db>>2]|0;
  $44 = (_sqlite3DbMallocZero($43,24,0)|0);
  HEAP32[$pNew>>2] = $44;
 }
 $45 = HEAP32[$db>>2]|0;
 $46 = ((($45)) + 65|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = ($47<<24>>24)!=(0);
 if ($48) {
  $49 = HEAP32[$db>>2]|0;
  $50 = HEAP32[$3>>2]|0;
  _sqlite3ExprListDelete($49,$50);
  $51 = HEAP32[$db>>2]|0;
  $52 = HEAP32[$4>>2]|0;
  _sqlite3SelectDelete($51,$52);
  $53 = HEAP32[$db>>2]|0;
  $54 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($53,$54);
  $55 = HEAP32[$1>>2]|0;
  HEAP32[$pNew>>2] = $55;
  $85 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($85|0);
 } else {
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$pNew>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$pNew>>2]|0;
  $60 = ((($59)) + 8|0);
  $61 = (($60) + ($58<<4)|0);
  $62 = ((($61)) + 8|0);
  HEAP32[$62>>2] = $56;
  $63 = HEAP32[$3>>2]|0;
  $64 = HEAP32[$pNew>>2]|0;
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$pNew>>2]|0;
  $67 = ((($66)) + 8|0);
  $68 = (($67) + ($65<<4)|0);
  $69 = ((($68)) + 4|0);
  HEAP32[$69>>2] = $63;
  $70 = HEAP32[$zName>>2]|0;
  $71 = HEAP32[$pNew>>2]|0;
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$pNew>>2]|0;
  $74 = ((($73)) + 8|0);
  $75 = (($74) + ($72<<4)|0);
  HEAP32[$75>>2] = $70;
  $76 = HEAP32[$pNew>>2]|0;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$pNew>>2]|0;
  $79 = ((($78)) + 8|0);
  $80 = (($79) + ($77<<4)|0);
  $81 = ((($80)) + 12|0);
  HEAP32[$81>>2] = 0;
  $82 = HEAP32[$pNew>>2]|0;
  $83 = HEAP32[$82>>2]|0;
  $84 = (($83) + 1)|0;
  HEAP32[$82>>2] = $84;
  $85 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 return (0)|0;
}
function _yy_find_reduce_action($stateno,$iLookAhead) {
 $stateno = $stateno|0;
 $iLookAhead = $iLookAhead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $i = sp;
 HEAP32[$0>>2] = $stateno;
 HEAP8[$1>>0] = $iLookAhead;
 $2 = HEAP32[$0>>2]|0;
 $3 = (15048 + ($2<<1)|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 HEAP32[$i>>2] = $5;
 $6 = HEAP8[$1>>0]|0;
 $7 = $6&255;
 $8 = HEAP32[$i>>2]|0;
 $9 = (($8) + ($7))|0;
 HEAP32[$i>>2] = $9;
 $10 = HEAP32[$i>>2]|0;
 $11 = (12046 + ($10<<1)|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 STACKTOP = sp;return ($13|0);
}
function _yy_accept($yypParser) {
 $yypParser = $yypParser|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pParse = sp;
 HEAP32[$0>>2] = $yypParser;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pParse>>2] = $3;
 while(1) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)>=(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  _yy_pop_parser_stack($7);
 }
 $8 = HEAP32[$pParse>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 HEAP32[$10>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3GetVdbe($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (_sqlite3VdbeCreate($6)|0);
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 8|0);
  HEAP32[$9>>2] = $7;
  HEAP32[$v>>2] = $7;
  $10 = HEAP32[$v>>2]|0;
  $11 = ($10|0)!=(0|0);
  if ($11) {
   $12 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeAddOp0($12,158)|0);
  }
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 416|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)==(0|0);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = ((($18)) + 60|0);
   $20 = HEAP16[$19>>1]|0;
   $21 = $20&65535;
   $22 = $21 & 8;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$0>>2]|0;
    $25 = ((($24)) + 23|0);
    HEAP8[$25>>0] = 1;
   }
  }
 }
 $26 = HEAP32[$v>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3VdbeDeletePriorOpcode($p,$op) {
 $p = $p|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 HEAP8[$2>>0] = $op;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - 1)|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 96|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($6|0)>($11|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) - 1)|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + (($16*20)|0)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = HEAP8[$2>>0]|0;
  $24 = $23&255;
  $25 = ($22|0)==($24|0);
  if ($25) {
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) - 1)|0;
   $31 = (_sqlite3VdbeChangeToNoop($26,$30)|0);
   HEAP32[$0>>2] = $31;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3VdbeAddOp0($p,$op) {
 $p = $p|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeAddOp3($2,$3,0,0,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeJumpHere($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 96|0);
 HEAP32[$9>>2] = $5;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 32|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3VdbeChangeP2($10,$11,$14);
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOp4($p,$op,$p1,$p2,$p3,$zP4,$p4type) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $zP4 = $zP4|0;
 $p4type = $p4type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $zP4;
 HEAP32[$6>>2] = $p4type;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3VdbeAddOp3($7,$8,$9,$10,$11)|0);
 HEAP32[$addr>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$addr>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 _sqlite3VdbeChangeP4($13,$14,$15,$16);
 $17 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _codeTableLocks($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $p1 = 0, $pVdbe = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $pVdbe = sp + 8|0;
 $p = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$pVdbe>>2] = $2;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 404|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 408|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p1>>2] = $14;
  $15 = HEAP32[$pVdbe>>2]|0;
  $16 = HEAP32[$p1>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  (_sqlite3VdbeAddOp4($15,148,$16,$19,$23,$26,-2)|0);
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3AutoincrementBegin($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $aOp = 0;
 var $db = 0, $memId = 0, $p = 0, $pDb = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $p = sp + 20|0;
 $db = sp + 16|0;
 $pDb = sp + 12|0;
 $memId = sp + 8|0;
 $v = sp + 4|0;
 $aOp = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 412|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$db>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  HEAP32[$pDb>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$memId>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$pDb>>2]|0;
  $26 = ((($25)) + 12|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ((($27)) + 72|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3OpenTable($21,0,$24,$29,54);
  $30 = HEAP32[$v>>2]|0;
  $31 = HEAP32[$memId>>2]|0;
  $32 = (($31) - 1)|0;
  $33 = HEAP32[$p>>2]|0;
  $34 = ((($33)) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$35>>2]|0;
  (_sqlite3VdbeLoadString($30,$32,$36)|0);
  $37 = HEAP32[$v>>2]|0;
  $38 = (_sqlite3VdbeAddOpList($37,10,28357,0)|0);
  HEAP32[$aOp>>2] = $38;
  $39 = HEAP32[$aOp>>2]|0;
  $40 = ($39|0)==(0|0);
  if ($40) {
   label = 5;
   break;
  }
  $41 = HEAP32[$memId>>2]|0;
  $42 = HEAP32[$aOp>>2]|0;
  $43 = ((($42)) + 8|0);
  HEAP32[$43>>2] = $41;
  $44 = HEAP32[$memId>>2]|0;
  $45 = (($44) + 1)|0;
  $46 = HEAP32[$aOp>>2]|0;
  $47 = ((($46)) + 12|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP32[$memId>>2]|0;
  $49 = HEAP32[$aOp>>2]|0;
  $50 = ((($49)) + 40|0);
  $51 = ((($50)) + 12|0);
  HEAP32[$51>>2] = $48;
  $52 = HEAP32[$memId>>2]|0;
  $53 = (($52) - 1)|0;
  $54 = HEAP32[$aOp>>2]|0;
  $55 = ((($54)) + 60|0);
  $56 = ((($55)) + 4|0);
  HEAP32[$56>>2] = $53;
  $57 = HEAP32[$memId>>2]|0;
  $58 = HEAP32[$aOp>>2]|0;
  $59 = ((($58)) + 60|0);
  $60 = ((($59)) + 12|0);
  HEAP32[$60>>2] = $57;
  $61 = HEAP32[$aOp>>2]|0;
  $62 = ((($61)) + 60|0);
  $63 = ((($62)) + 3|0);
  HEAP8[$63>>0] = 16;
  $64 = HEAP32[$memId>>2]|0;
  $65 = (($64) + 1)|0;
  $66 = HEAP32[$aOp>>2]|0;
  $67 = ((($66)) + 80|0);
  $68 = ((($67)) + 8|0);
  HEAP32[$68>>2] = $65;
  $69 = HEAP32[$memId>>2]|0;
  $70 = HEAP32[$aOp>>2]|0;
  $71 = ((($70)) + 100|0);
  $72 = ((($71)) + 12|0);
  HEAP32[$72>>2] = $69;
  $73 = HEAP32[$memId>>2]|0;
  $74 = HEAP32[$aOp>>2]|0;
  $75 = ((($74)) + 160|0);
  $76 = ((($75)) + 8|0);
  HEAP32[$76>>2] = $73;
  $77 = HEAP32[$p>>2]|0;
  $78 = HEAP32[$77>>2]|0;
  HEAP32[$p>>2] = $78;
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _sqlite3ExprCode($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inReg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $inReg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)==(157);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 8|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 28|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($11,30,$14,$15)|0);
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = (_sqlite3ExprCodeTarget($16,$17,$18)|0);
 HEAP32[$inReg>>2] = $19;
 $20 = HEAP32[$inReg>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = ($20|0)!=($21|0);
 if (!($22)) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$inReg>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($29,31,$30,$31)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeGoto($p,$iDest) {
 $p = $p|0;
 $iDest = $iDest|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDest;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeAddOp3($2,13,0,$3,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeChangeToNoop($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $addr;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 65|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*20)|0)|0);
  HEAP32[$pOp>>2] = $12;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$pOp>>2]|0;
  $16 = ((($15)) + 1|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = HEAP32[$pOp>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  _freeP4($14,$18,$21);
  $22 = HEAP32[$pOp>>2]|0;
  $23 = ((($22)) + 1|0);
  HEAP8[$23>>0] = 0;
  $24 = HEAP32[$pOp>>2]|0;
  $25 = ((($24)) + 16|0);
  HEAP32[$25>>2] = 0;
  $26 = HEAP32[$pOp>>2]|0;
  HEAP8[$26>>0] = -96;
  HEAP32[$0>>2] = 1;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3VdbeChangeP2($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 $7 = ((($6)) + 8|0);
 HEAP32[$7>>2] = $3;
 STACKTOP = sp;return;
}
function _sqlite3VdbeGetOp($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $addr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)<(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 32|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) - 1)|0;
  HEAP32[$2>>2] = $8;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 65|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 2556;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($14*20)|0)|0);
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3OpenTable($pParse,$iCur,$iDb,$pTab,$opcode) {
 $pParse = $pParse|0;
 $iCur = $iCur|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 $opcode = $opcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPk = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $v = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$4>>2] = $opcode;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3GetVdbe($5)|0);
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 28|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)==(55);
 $14 = $13 ? 1 : 0;
 $15 = $14&255;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 _sqlite3TableLock($7,$8,$11,$15,$17);
 $18 = HEAP32[$3>>2]|0;
 $19 = ((($18)) + 42|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 & 32;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = ((($27)) + 28|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ((($31)) + 34|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33 << 16 >> 16;
  (_sqlite3VdbeAddOp4Int($24,$25,$26,$29,$30,$34)|0);
  STACKTOP = sp;return;
 } else {
  $35 = HEAP32[$3>>2]|0;
  $36 = (_sqlite3PrimaryKeyIndex($35)|0);
  HEAP32[$pPk>>2] = $36;
  $37 = HEAP32[$v>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$pPk>>2]|0;
  $41 = ((($40)) + 44|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeAddOp3($37,$38,$39,$42,$43)|0);
  $44 = HEAP32[$0>>2]|0;
  $45 = HEAP32[$pPk>>2]|0;
  _sqlite3VdbeSetP4KeyInfo($44,$45);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeLoadString($p,$iDest,$zStr) {
 $p = $p|0;
 $iDest = $iDest|0;
 $zStr = $zStr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDest;
 HEAP32[$2>>2] = $zStr;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeAddOp4($3,97,0,$4,0,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3TableLock($pParse,$iDb,$iTab,$isWriteLock,$zName) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $iTab = $iTab|0;
 $isWriteLock = $isWriteLock|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $nBytes = 0, $p = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 32|0;
 $4 = sp + 16|0;
 $pToplevel = sp + 12|0;
 $i = sp + 8|0;
 $nBytes = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iTab;
 HEAP8[$3>>0] = $isWriteLock;
 HEAP32[$4>>2] = $zName;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 416|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ((($9)) + 416|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = $9;
 }
 HEAP32[$pToplevel>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pToplevel>>2]|0;
  $15 = ((($14)) + 404|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$pToplevel>>2]|0;
  $20 = ((($19)) + 408|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<4)|0);
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ($24|0)==($25|0);
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$2>>2]|0;
   $31 = ($29|0)==($30|0);
   if ($31) {
    label = 7;
    break;
   }
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = ($35|0)!=(0);
  if ($36) {
   $41 = 1;
  } else {
   $37 = HEAP8[$3>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)!=(0);
   $41 = $39;
  }
  $40 = $41&1;
  $42 = $40&255;
  $43 = HEAP32[$p>>2]|0;
  $44 = ((($43)) + 8|0);
  HEAP8[$44>>0] = $42;
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$pToplevel>>2]|0;
 $48 = ((($47)) + 404|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 1)|0;
 $51 = $50<<4;
 HEAP32[$nBytes>>2] = $51;
 $52 = HEAP32[$pToplevel>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$pToplevel>>2]|0;
 $55 = ((($54)) + 408|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$nBytes>>2]|0;
 $58 = ($57|0)<(0);
 $59 = $58 << 31 >> 31;
 $60 = (_sqlite3DbReallocOrFree($53,$56,$57,$59)|0);
 $61 = HEAP32[$pToplevel>>2]|0;
 $62 = ((($61)) + 408|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$pToplevel>>2]|0;
 $64 = ((($63)) + 408|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)!=(0|0);
 $67 = HEAP32[$pToplevel>>2]|0;
 $68 = ((($67)) + 404|0);
 if ($66) {
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + 1)|0;
  HEAP32[$68>>2] = $70;
  $71 = HEAP32[$pToplevel>>2]|0;
  $72 = ((($71)) + 408|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($69<<4)|0);
  HEAP32[$p>>2] = $74;
  $75 = HEAP32[$1>>2]|0;
  $76 = HEAP32[$p>>2]|0;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$2>>2]|0;
  $78 = HEAP32[$p>>2]|0;
  $79 = ((($78)) + 4|0);
  HEAP32[$79>>2] = $77;
  $80 = HEAP8[$3>>0]|0;
  $81 = HEAP32[$p>>2]|0;
  $82 = ((($81)) + 8|0);
  HEAP8[$82>>0] = $80;
  $83 = HEAP32[$4>>2]|0;
  $84 = HEAP32[$p>>2]|0;
  $85 = ((($84)) + 12|0);
  HEAP32[$85>>2] = $83;
  STACKTOP = sp;return;
 } else {
  HEAP32[$68>>2] = 0;
  $86 = HEAP32[$pToplevel>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  _sqlite3OomFault($87);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeSetP4KeyInfo($pParse,$pIdx) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3KeyInfoOfIndex($6,$7)|0);
 _sqlite3VdbeChangeP4($5,-1,$8,-6);
 STACKTOP = sp;return;
}
function _sqlite3KeyInfoOfIndex($pParse,$pIdx) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $nCol = 0, $nKey = 0, $pKey = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $nCol = sp + 12|0;
 $nKey = sp + 8|0;
 $pKey = sp + 4|0;
 $zColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 HEAP32[$nCol>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 50|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 HEAP32[$nKey>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 55|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17&255) >>> 3;
 $19 = $18 & 1;
 $20 = $19&255;
 $21 = ($20|0)!=(0);
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 if ($21) {
  $24 = HEAP32[$nKey>>2]|0;
  $25 = HEAP32[$nCol>>2]|0;
  $26 = HEAP32[$nKey>>2]|0;
  $27 = (($25) - ($26))|0;
  $28 = (_sqlite3KeyInfoAlloc($23,$24,$27)|0);
  HEAP32[$pKey>>2] = $28;
 } else {
  $29 = HEAP32[$nCol>>2]|0;
  $30 = (_sqlite3KeyInfoAlloc($23,$29,0)|0);
  HEAP32[$pKey>>2] = $30;
 }
 $31 = HEAP32[$pKey>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$nCol>>2]|0;
   $35 = ($33|0)<($34|0);
   if (!($35)) {
    break;
   }
   $36 = HEAP32[$i>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 32|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($36<<2)|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$zColl>>2] = $41;
   $42 = HEAP32[$zColl>>2]|0;
   $43 = ($42|0)==(20716|0);
   if ($43) {
    $51 = 0;
   } else {
    $44 = HEAP32[$1>>2]|0;
    $45 = HEAP32[$zColl>>2]|0;
    $46 = (_sqlite3LocateCollSeq($44,$45)|0);
    $51 = $46;
   }
   $47 = HEAP32[$i>>2]|0;
   $48 = HEAP32[$pKey>>2]|0;
   $49 = ((($48)) + 20|0);
   $50 = (($49) + ($47<<2)|0);
   HEAP32[$50>>2] = $51;
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 28|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + ($52)|0);
   $57 = HEAP8[$56>>0]|0;
   $58 = HEAP32[$i>>2]|0;
   $59 = HEAP32[$pKey>>2]|0;
   $60 = ((($59)) + 16|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + ($58)|0);
   HEAP8[$62>>0] = $57;
   $63 = HEAP32[$i>>2]|0;
   $64 = (($63) + 1)|0;
   HEAP32[$i>>2] = $64;
  }
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 68|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)!=(0);
  if ($68) {
   $69 = HEAP32[$pKey>>2]|0;
   _sqlite3KeyInfoUnref($69);
   HEAP32[$pKey>>2] = 0;
  }
 }
 $70 = HEAP32[$pKey>>2]|0;
 HEAP32[$0>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($71|0);
}
function _sqlite3KeyInfoAlloc($db,$N,$X) {
 $db = $db|0;
 $N = $N|0;
 $X = $X|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $nExtra = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nExtra = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $X;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($3) + ($4))|0;
 $6 = ($5*5)|0;
 HEAP32[$nExtra>>2] = $6;
 $7 = HEAP32[$nExtra>>2]|0;
 $8 = (24 + ($7))|0;
 $9 = (_sqlite3Malloc($8,0)|0);
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (($12) + ($13))|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = (($16) + ($14<<2)|0);
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$1>>2]|0;
  $21 = $20&65535;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 6|0);
  HEAP16[$23>>1] = $21;
  $24 = HEAP32[$2>>2]|0;
  $25 = $24&65535;
  $26 = HEAP32[$p>>2]|0;
  $27 = ((($26)) + 8|0);
  HEAP16[$27>>1] = $25;
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 62|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = HEAP32[$p>>2]|0;
  $32 = ((($31)) + 4|0);
  HEAP8[$32>>0] = $30;
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 12|0);
  HEAP32[$35>>2] = $33;
  $36 = HEAP32[$p>>2]|0;
  HEAP32[$36>>2] = 1;
  $37 = HEAP32[$p>>2]|0;
  $38 = ((($37)) + 24|0);
  $39 = HEAP32[$nExtra>>2]|0;
  _memset(($38|0),0,($39|0))|0;
  $41 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $40 = HEAP32[$0>>2]|0;
  _sqlite3OomFault($40);
  $41 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _sqlite3LocateCollSeq($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $enc = 0, $initbusy = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $enc = sp + 17|0;
 $initbusy = sp + 16|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 62|0);
 $6 = HEAP8[$5>>0]|0;
 HEAP8[$enc>>0] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 140|0);
 $9 = ((($8)) + 5|0);
 $10 = HEAP8[$9>>0]|0;
 HEAP8[$initbusy>>0] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = HEAP8[$enc>>0]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP8[$initbusy>>0]|0;
 $15 = $14&255;
 $16 = (_sqlite3FindCollSeq($11,$12,$13,$15)|0);
 HEAP32[$pColl>>2] = $16;
 $17 = HEAP8[$initbusy>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if ($18) {
  $30 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $19 = HEAP32[$pColl>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = HEAP32[$pColl>>2]|0;
  $22 = ((($21)) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $30 = HEAP32[$pColl>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP8[$enc>>0]|0;
 $27 = HEAP32[$pColl>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = (_sqlite3GetCollSeq($25,$26,$27,$28)|0);
 HEAP32[$pColl>>2] = $29;
 $30 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3FindCollSeq($db,$enc,$zName,$create) {
 $db = $db|0;
 $enc = $enc|0;
 $zName = $zName|0;
 $create = $create|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 16|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $db;
 HEAP8[$1>>0] = $enc;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $create;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$3>>2]|0;
  $9 = (_findCollSeqEntry($6,$7,$8)|0);
  HEAP32[$pColl>>2] = $9;
 } else {
  $10 = ((($6)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pColl>>2] = $11;
 }
 $12 = HEAP32[$pColl>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $19 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $14 = HEAP8[$1>>0]|0;
 $15 = $14&255;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$pColl>>2]|0;
 $18 = (($17) + (($16*20)|0)|0);
 HEAP32[$pColl>>2] = $18;
 $19 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3GetCollSeq($pParse,$enc,$pColl,$zName) {
 $pParse = $pParse|0;
 $enc = $enc|0;
 $pColl = $pColl|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 24|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $p = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP8[$1>>0] = $enc;
 HEAP32[$2>>2] = $pColl;
 HEAP32[$3>>2] = $zName;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$db>>2]|0;
  $10 = HEAP8[$1>>0]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3FindCollSeq($9,$10,$11,0)|0);
  HEAP32[$p>>2] = $12;
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  if (!($18)) {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP8[$1>>0]|0;
  $21 = $20&255;
  $22 = HEAP32[$3>>2]|0;
  _callCollNeeded($19,$21,$22);
  $23 = HEAP32[$db>>2]|0;
  $24 = HEAP8[$1>>0]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = (_sqlite3FindCollSeq($23,$24,$25,0)|0);
  HEAP32[$p>>2] = $26;
 }
 $27 = HEAP32[$p>>2]|0;
 $28 = ($27|0)!=(0|0);
 if ($28) {
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   $33 = HEAP32[$db>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = (_synthCollSeq($33,$34)|0);
   $36 = ($35|0)!=(0);
   if ($36) {
    HEAP32[$p>>2] = 0;
   }
  }
 }
 $37 = HEAP32[$p>>2]|0;
 $38 = ($37|0)==(0|0);
 if (!($38)) {
  $41 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $40;
 _sqlite3ErrorMsg($39,28397,$vararg_buffer);
 $41 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _findCollSeqEntry($db,$zName,$create) {
 $db = $db|0;
 $zName = $zName|0;
 $create = $create|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $or$cond = 0, $pColl = 0, $pDel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pColl = sp + 8|0;
 $nName = sp + 4|0;
 $pDel = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $create;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 432|0);
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3HashFind($4,$5)|0);
 HEAP32[$pColl>>2] = $6;
 $7 = HEAP32[$pColl>>2]|0;
 $8 = (0|0)==($7|0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0);
 $or$cond = $8 & $10;
 if (!($or$cond)) {
  $58 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 HEAP32[$nName>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$nName>>2]|0;
 $15 = (60 + ($14))|0;
 $16 = (($15) + 1)|0;
 $17 = (_sqlite3DbMallocZero($13,$16,0)|0);
 HEAP32[$pColl>>2] = $17;
 $18 = HEAP32[$pColl>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $58 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 HEAP32[$pDel>>2] = 0;
 $20 = HEAP32[$pColl>>2]|0;
 $21 = ((($20)) + 60|0);
 $22 = HEAP32[$pColl>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$pColl>>2]|0;
 $24 = ((($23)) + 4|0);
 HEAP8[$24>>0] = 1;
 $25 = HEAP32[$pColl>>2]|0;
 $26 = ((($25)) + 60|0);
 $27 = HEAP32[$pColl>>2]|0;
 $28 = ((($27)) + 20|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pColl>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = ((($30)) + 4|0);
 HEAP8[$31>>0] = 2;
 $32 = HEAP32[$pColl>>2]|0;
 $33 = ((($32)) + 60|0);
 $34 = HEAP32[$pColl>>2]|0;
 $35 = ((($34)) + 40|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pColl>>2]|0;
 $37 = ((($36)) + 40|0);
 $38 = ((($37)) + 4|0);
 HEAP8[$38>>0] = 3;
 $39 = HEAP32[$pColl>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$nName>>2]|0;
 _memcpy(($40|0),($41|0),($42|0))|0;
 $43 = HEAP32[$nName>>2]|0;
 $44 = HEAP32[$pColl>>2]|0;
 $45 = HEAP32[$44>>2]|0;
 $46 = (($45) + ($43)|0);
 HEAP8[$46>>0] = 0;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 432|0);
 $49 = HEAP32[$pColl>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$pColl>>2]|0;
 $52 = (_sqlite3HashInsert($48,$50,$51)|0);
 HEAP32[$pDel>>2] = $52;
 $53 = HEAP32[$pDel>>2]|0;
 $54 = ($53|0)!=(0|0);
 if (!($54)) {
  $58 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $55 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($55);
 $56 = HEAP32[$0>>2]|0;
 $57 = HEAP32[$pDel>>2]|0;
 _sqlite3DbFree($56,$57);
 HEAP32[$pColl>>2] = 0;
 $58 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _callCollNeeded($db,$enc,$zName) {
 $db = $db|0;
 $enc = $enc|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, $zExternal = 0, $zExternal1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zExternal = sp + 8|0;
 $zExternal1 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $enc;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 224|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = HEAP32[$2>>2]|0;
   $9 = (_sqlite3DbStrDup($7,$8)|0);
   HEAP32[$zExternal>>2] = $9;
   $10 = HEAP32[$zExternal>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = ((($12)) + 224|0);
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$0>>2]|0;
    $16 = ((($15)) + 232|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$1>>2]|0;
    $20 = HEAP32[$zExternal>>2]|0;
    FUNCTION_TABLE_viiii[$14 & 255]($17,$18,$19,$20);
    $21 = HEAP32[$0>>2]|0;
    $22 = HEAP32[$zExternal>>2]|0;
    _sqlite3DbFree($21,$22);
    break;
   } else {
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 228|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (_sqlite3ValueNew($27)|0);
 HEAP32[$pTmp>>2] = $28;
 $29 = HEAP32[$pTmp>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 _sqlite3ValueSetStr($29,-1,$30,1,0);
 $31 = HEAP32[$pTmp>>2]|0;
 $32 = HEAP8[352>>0]|0;
 $33 = $32 << 24 >> 24;
 $34 = ($33|0)==(0);
 $35 = $34 ? 3 : 2;
 $36 = $35&255;
 $37 = (_sqlite3ValueText($31,$36)|0);
 HEAP32[$zExternal1>>2] = $37;
 $38 = HEAP32[$zExternal1>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 228|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 232|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 62|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = HEAP32[$zExternal1>>2]|0;
  FUNCTION_TABLE_viiii[$42 & 255]($45,$46,$50,$51);
 }
 $52 = HEAP32[$pTmp>>2]|0;
 _sqlite3ValueFree($52);
 STACKTOP = sp;return;
}
function _synthCollSeq($db,$pColl) {
 $db = $db|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pColl2 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pColl2 = sp + 8|0;
 $z = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pColl;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$z>>2] = $4;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = ($5|0)<(3);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$i>>2]|0;
  $9 = (28428 + ($8)|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = HEAP32[$z>>2]|0;
  $12 = (_sqlite3FindCollSeq($7,$10,$11,0)|0);
  HEAP32[$pColl2>>2] = $12;
  $13 = HEAP32[$pColl2>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   label = 4;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$pColl2>>2]|0;
  ;HEAP32[$17>>2]=HEAP32[$18>>2]|0;HEAP32[$17+4>>2]=HEAP32[$18+4>>2]|0;HEAP32[$17+8>>2]=HEAP32[$18+8>>2]|0;HEAP32[$17+12>>2]=HEAP32[$18+12>>2]|0;HEAP32[$17+16>>2]=HEAP32[$18+16>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 16|0);
  HEAP32[$20>>2] = 0;
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _sqlite3VdbeAddOp2($p,$op,$p1,$p2) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3VdbeAddOp3($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3ExprCodeTarget($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aListelem = 0, $addr = 0;
 var $constMask = 0, $db = 0, $destIfFalse = 0, $destIfNull = 0, $enc = 0, $endCoalesce = 0, $endLabel = 0, $exprOp = 0, $i = 0, $i2 = 0, $iTab = 0, $inReg = 0, $n = 0, $nExpr = 0, $nFarg = 0, $nId = 0, $nextCase = 0, $op = 0, $opCompare = 0, $or$cond = 0;
 var $or$cond3 = 0, $p1 = 0, $pAggInfo = 0, $pCol = 0, $pColl = 0, $pDef = 0, $pEList = 0, $pFarg = 0, $pInfo = 0, $pLItem = 0, $pLeft = 0, $pLeft1 = 0, $pRight = 0, $pTab = 0, $pTest = 0, $pX = 0, $r1 = 0, $r2 = 0, $r3 = 0, $r4 = 0;
 var $regFree1 = 0, $regFree2 = 0, $tempX = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr7 = 0, $z = 0, $zBlob = 0, $zId = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 320|0;
 $vararg_buffer8 = sp + 16|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 308|0;
 $1 = sp + 304|0;
 $2 = sp + 300|0;
 $3 = sp + 296|0;
 $v = sp + 292|0;
 $op = sp + 288|0;
 $inReg = sp + 284|0;
 $regFree1 = sp + 280|0;
 $regFree2 = sp + 276|0;
 $r1 = sp + 272|0;
 $r2 = sp + 268|0;
 $r3 = sp + 264|0;
 $r4 = sp + 260|0;
 $db = sp + 256|0;
 $tempX = sp + 208|0;
 $pAggInfo = sp + 200|0;
 $pCol = sp + 196|0;
 $iTab = sp + 192|0;
 $n = sp + 188|0;
 $z = sp + 184|0;
 $zBlob = sp + 180|0;
 $pLeft = sp + 176|0;
 $addr = sp + 172|0;
 $pInfo = sp + 168|0;
 $pFarg = sp + 164|0;
 $nFarg = sp + 160|0;
 $pDef = sp + 156|0;
 $nId = sp + 152|0;
 $zId = sp + 148|0;
 $constMask = sp + 144|0;
 $i = sp + 140|0;
 $enc = sp + 313|0;
 $pColl = sp + 136|0;
 $endCoalesce = sp + 132|0;
 $exprOp = sp + 312|0;
 $destIfFalse = sp + 128|0;
 $destIfNull = sp + 124|0;
 $pLeft1 = sp + 120|0;
 $pLItem = sp + 116|0;
 $pRight = sp + 112|0;
 $pTab = sp + 108|0;
 $p1 = sp + 104|0;
 $endLabel = sp + 100|0;
 $nextCase = sp + 96|0;
 $nExpr = sp + 92|0;
 $i2 = sp + 88|0;
 $pEList = sp + 84|0;
 $aListelem = sp + 80|0;
 $opCompare = sp + 32|0;
 $pX = sp + 24|0;
 $pTest = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $target;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$inReg>>2] = $7;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$v>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $737 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($737|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  HEAP32[$op>>2] = 101;
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  HEAP32[$op>>2] = $16;
 }
 $17 = HEAP32[$op>>2]|0;
 L9: do {
  switch ($17|0) {
  case 154:  {
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 40|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$pAggInfo>>2] = $20;
   $21 = HEAP32[$2>>2]|0;
   $22 = ((($21)) + 34|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23 << 16 >> 16;
   $25 = HEAP32[$pAggInfo>>2]|0;
   $26 = ((($25)) + 28|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + (($24*24)|0)|0);
   HEAP32[$pCol>>2] = $28;
   $29 = HEAP32[$pAggInfo>>2]|0;
   $30 = HEAP8[$29>>0]|0;
   $31 = ($30<<24>>24)!=(0);
   if (!($31)) {
    $32 = HEAP32[$pCol>>2]|0;
    $33 = ((($32)) + 16|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$inReg>>2] = $34;
    break L9;
   }
   $35 = HEAP32[$pAggInfo>>2]|0;
   $36 = ((($35)) + 1|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = ($37<<24>>24)!=(0);
   if ($38) {
    $39 = HEAP32[$v>>2]|0;
    $40 = HEAP32[$pAggInfo>>2]|0;
    $41 = ((($40)) + 8|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$pCol>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp3($39,47,$42,$45,$46)|0);
   } else {
    label = 11;
   }
   break;
  }
  case 152:  {
   label = 11;
   break;
  }
  case 132:  {
   $81 = HEAP32[$1>>2]|0;
   $82 = HEAP32[$2>>2]|0;
   $83 = HEAP32[$3>>2]|0;
   _codeInteger($81,$82,0,$83);
   break;
  }
  case 133:  {
   $84 = HEAP32[$v>>2]|0;
   $85 = HEAP32[$2>>2]|0;
   $86 = ((($85)) + 8|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$3>>2]|0;
   _codeReal($84,$87,0,$88);
   break;
  }
  case 97:  {
   $89 = HEAP32[$v>>2]|0;
   $90 = HEAP32[$3>>2]|0;
   $91 = HEAP32[$2>>2]|0;
   $92 = ((($91)) + 8|0);
   $93 = HEAP32[$92>>2]|0;
   (_sqlite3VdbeLoadString($89,$90,$93)|0);
   break;
  }
  case 101:  {
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($94,25,0,$95)|0);
   break;
  }
  case 134:  {
   $96 = HEAP32[$2>>2]|0;
   $97 = ((($96)) + 8|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ((($98)) + 2|0);
   HEAP32[$z>>2] = $99;
   $100 = HEAP32[$z>>2]|0;
   $101 = (_sqlite3Strlen30($100)|0);
   $102 = (($101) - 1)|0;
   HEAP32[$n>>2] = $102;
   $103 = HEAP32[$v>>2]|0;
   $104 = (_sqlite3VdbeDb($103)|0);
   $105 = HEAP32[$z>>2]|0;
   $106 = HEAP32[$n>>2]|0;
   $107 = (_sqlite3HexToBlob($104,$105,$106)|0);
   HEAP32[$zBlob>>2] = $107;
   $108 = HEAP32[$v>>2]|0;
   $109 = HEAP32[$n>>2]|0;
   $110 = (($109|0) / 2)&-1;
   $111 = HEAP32[$3>>2]|0;
   $112 = HEAP32[$zBlob>>2]|0;
   (_sqlite3VdbeAddOp4($108,27,$110,$111,0,$112,-1)|0);
   break;
  }
  case 135:  {
   $113 = HEAP32[$v>>2]|0;
   $114 = HEAP32[$2>>2]|0;
   $115 = ((($114)) + 32|0);
   $116 = HEAP16[$115>>1]|0;
   $117 = $116 << 16 >> 16;
   $118 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($113,28,$117,$118)|0);
   $119 = HEAP32[$2>>2]|0;
   $120 = ((($119)) + 8|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = ((($121)) + 1|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = $123 << 24 >> 24;
   $125 = ($124|0)!=(0);
   if ($125) {
    $126 = HEAP32[$v>>2]|0;
    $127 = HEAP32[$2>>2]|0;
    $128 = ((($127)) + 32|0);
    $129 = HEAP16[$128>>1]|0;
    $130 = $129 << 16 >> 16;
    $131 = (($130) - 1)|0;
    $132 = HEAP32[$1>>2]|0;
    $133 = ((($132)) + 476|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = (($134) + ($131<<2)|0);
    $136 = HEAP32[$135>>2]|0;
    _sqlite3VdbeChangeP4($126,-1,$136,-2);
   }
   break;
  }
  case 157:  {
   $137 = HEAP32[$2>>2]|0;
   $138 = ((($137)) + 28|0);
   $139 = HEAP32[$138>>2]|0;
   HEAP32[$inReg>>2] = $139;
   break;
  }
  case 38:  {
   $140 = HEAP32[$1>>2]|0;
   $141 = HEAP32[$2>>2]|0;
   $142 = ((($141)) + 12|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = HEAP32[$3>>2]|0;
   $145 = (_sqlite3ExprCodeTarget($140,$143,$144)|0);
   HEAP32[$inReg>>2] = $145;
   $146 = HEAP32[$inReg>>2]|0;
   $147 = HEAP32[$3>>2]|0;
   $148 = ($146|0)!=($147|0);
   if ($148) {
    $149 = HEAP32[$v>>2]|0;
    $150 = HEAP32[$inReg>>2]|0;
    $151 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($149,31,$150,$151)|0);
    $152 = HEAP32[$3>>2]|0;
    HEAP32[$inReg>>2] = $152;
   }
   $153 = HEAP32[$v>>2]|0;
   $154 = HEAP32[$3>>2]|0;
   $155 = HEAP32[$2>>2]|0;
   $156 = ((($155)) + 8|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (_sqlite3AffinityType($157,0)|0);
   $159 = $158 << 24 >> 24;
   (_sqlite3VdbeAddOp2($153,40,$154,$159)|0);
   $160 = HEAP32[$1>>2]|0;
   $161 = HEAP32[$inReg>>2]|0;
   _sqlite3ExprCacheAffinityChange($160,$161,1);
   break;
  }
  case 79: case 78: case 83: case 80: case 81: case 82:  {
   $162 = HEAP32[$1>>2]|0;
   $163 = HEAP32[$2>>2]|0;
   $164 = ((($163)) + 12|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = (_sqlite3ExprCodeTemp($162,$165,$regFree1)|0);
   HEAP32[$r1>>2] = $166;
   $167 = HEAP32[$1>>2]|0;
   $168 = HEAP32[$2>>2]|0;
   $169 = ((($168)) + 16|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = (_sqlite3ExprCodeTemp($167,$170,$regFree2)|0);
   HEAP32[$r2>>2] = $171;
   $172 = HEAP32[$1>>2]|0;
   $173 = HEAP32[$2>>2]|0;
   $174 = ((($173)) + 12|0);
   $175 = HEAP32[$174>>2]|0;
   $176 = HEAP32[$2>>2]|0;
   $177 = ((($176)) + 16|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = HEAP32[$op>>2]|0;
   $180 = HEAP32[$r1>>2]|0;
   $181 = HEAP32[$r2>>2]|0;
   $182 = HEAP32[$inReg>>2]|0;
   (_codeCompare($172,$175,$178,$179,$180,$181,$182,32)|0);
   break;
  }
  case 148: case 73:  {
   $183 = HEAP32[$1>>2]|0;
   $184 = HEAP32[$2>>2]|0;
   $185 = ((($184)) + 12|0);
   $186 = HEAP32[$185>>2]|0;
   $187 = (_sqlite3ExprCodeTemp($183,$186,$regFree1)|0);
   HEAP32[$r1>>2] = $187;
   $188 = HEAP32[$1>>2]|0;
   $189 = HEAP32[$2>>2]|0;
   $190 = ((($189)) + 16|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = (_sqlite3ExprCodeTemp($188,$191,$regFree2)|0);
   HEAP32[$r2>>2] = $192;
   $193 = HEAP32[$op>>2]|0;
   $194 = ($193|0)==(73);
   $195 = $194 ? 79 : 78;
   HEAP32[$op>>2] = $195;
   $196 = HEAP32[$1>>2]|0;
   $197 = HEAP32[$2>>2]|0;
   $198 = ((($197)) + 12|0);
   $199 = HEAP32[$198>>2]|0;
   $200 = HEAP32[$2>>2]|0;
   $201 = ((($200)) + 16|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = HEAP32[$op>>2]|0;
   $204 = HEAP32[$r1>>2]|0;
   $205 = HEAP32[$r2>>2]|0;
   $206 = HEAP32[$inReg>>2]|0;
   (_codeCompare($196,$199,$202,$203,$204,$205,$206,160)|0);
   break;
  }
  case 94: case 88: case 87: case 92: case 86: case 85: case 93: case 90: case 91: case 89: case 71: case 72:  {
   $207 = HEAP32[$1>>2]|0;
   $208 = HEAP32[$2>>2]|0;
   $209 = ((($208)) + 12|0);
   $210 = HEAP32[$209>>2]|0;
   $211 = (_sqlite3ExprCodeTemp($207,$210,$regFree1)|0);
   HEAP32[$r1>>2] = $211;
   $212 = HEAP32[$1>>2]|0;
   $213 = HEAP32[$2>>2]|0;
   $214 = ((($213)) + 16|0);
   $215 = HEAP32[$214>>2]|0;
   $216 = (_sqlite3ExprCodeTemp($212,$215,$regFree2)|0);
   HEAP32[$r2>>2] = $216;
   $217 = HEAP32[$v>>2]|0;
   $218 = HEAP32[$op>>2]|0;
   $219 = HEAP32[$r2>>2]|0;
   $220 = HEAP32[$r1>>2]|0;
   $221 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp3($217,$218,$219,$220,$221)|0);
   break;
  }
  case 155:  {
   $222 = HEAP32[$2>>2]|0;
   $223 = ((($222)) + 12|0);
   $224 = HEAP32[$223>>2]|0;
   HEAP32[$pLeft>>2] = $224;
   $225 = HEAP32[$pLeft>>2]|0;
   $226 = HEAP8[$225>>0]|0;
   $227 = $226&255;
   $228 = ($227|0)==(132);
   do {
    if ($228) {
     $229 = HEAP32[$1>>2]|0;
     $230 = HEAP32[$pLeft>>2]|0;
     $231 = HEAP32[$3>>2]|0;
     _codeInteger($229,$230,1,$231);
    } else {
     $232 = HEAP32[$pLeft>>2]|0;
     $233 = HEAP8[$232>>0]|0;
     $234 = $233&255;
     $235 = ($234|0)==(133);
     if ($235) {
      $236 = HEAP32[$v>>2]|0;
      $237 = HEAP32[$pLeft>>2]|0;
      $238 = ((($237)) + 8|0);
      $239 = HEAP32[$238>>2]|0;
      $240 = HEAP32[$3>>2]|0;
      _codeReal($236,$239,1,$240);
      break;
     } else {
      HEAP8[$tempX>>0] = -124;
      $241 = ((($tempX)) + 4|0);
      HEAP32[$241>>2] = 17408;
      $242 = ((($tempX)) + 8|0);
      HEAP32[$242>>2] = 0;
      $243 = HEAP32[$1>>2]|0;
      $244 = (_sqlite3ExprCodeTemp($243,$tempX,$regFree1)|0);
      HEAP32[$r1>>2] = $244;
      $245 = HEAP32[$1>>2]|0;
      $246 = HEAP32[$2>>2]|0;
      $247 = ((($246)) + 12|0);
      $248 = HEAP32[$247>>2]|0;
      $249 = (_sqlite3ExprCodeTemp($245,$248,$regFree2)|0);
      HEAP32[$r2>>2] = $249;
      $250 = HEAP32[$v>>2]|0;
      $251 = HEAP32[$r2>>2]|0;
      $252 = HEAP32[$r1>>2]|0;
      $253 = HEAP32[$3>>2]|0;
      (_sqlite3VdbeAddOp3($250,90,$251,$252,$253)|0);
      break;
     }
    }
   } while(0);
   $254 = HEAP32[$3>>2]|0;
   HEAP32[$inReg>>2] = $254;
   break;
  }
  case 19: case 96:  {
   $255 = HEAP32[$1>>2]|0;
   $256 = HEAP32[$2>>2]|0;
   $257 = ((($256)) + 12|0);
   $258 = HEAP32[$257>>2]|0;
   $259 = (_sqlite3ExprCodeTemp($255,$258,$regFree1)|0);
   HEAP32[$r1>>2] = $259;
   $260 = HEAP32[$3>>2]|0;
   HEAP32[$inReg>>2] = $260;
   $261 = HEAP32[$v>>2]|0;
   $262 = HEAP32[$op>>2]|0;
   $263 = HEAP32[$r1>>2]|0;
   $264 = HEAP32[$inReg>>2]|0;
   (_sqlite3VdbeAddOp2($261,$262,$263,$264)|0);
   break;
  }
  case 77: case 76:  {
   $265 = HEAP32[$v>>2]|0;
   $266 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($265,22,1,$266)|0);
   $267 = HEAP32[$1>>2]|0;
   $268 = HEAP32[$2>>2]|0;
   $269 = ((($268)) + 12|0);
   $270 = HEAP32[$269>>2]|0;
   $271 = (_sqlite3ExprCodeTemp($267,$270,$regFree1)|0);
   HEAP32[$r1>>2] = $271;
   $272 = HEAP32[$v>>2]|0;
   $273 = HEAP32[$op>>2]|0;
   $274 = HEAP32[$r1>>2]|0;
   $275 = (_sqlite3VdbeAddOp1($272,$273,$274)|0);
   HEAP32[$addr>>2] = $275;
   $276 = HEAP32[$v>>2]|0;
   $277 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($276,22,0,$277)|0);
   $278 = HEAP32[$v>>2]|0;
   $279 = HEAP32[$addr>>2]|0;
   _sqlite3VdbeJumpHere($278,$279);
   break;
  }
  case 153:  {
   $280 = HEAP32[$2>>2]|0;
   $281 = ((($280)) + 40|0);
   $282 = HEAP32[$281>>2]|0;
   HEAP32[$pInfo>>2] = $282;
   $283 = HEAP32[$pInfo>>2]|0;
   $284 = ($283|0)==(0|0);
   if ($284) {
    $285 = HEAP32[$1>>2]|0;
    $286 = HEAP32[$2>>2]|0;
    $287 = ((($286)) + 8|0);
    $288 = HEAP32[$287>>2]|0;
    HEAP32[$vararg_buffer>>2] = $288;
    _sqlite3ErrorMsg($285,28431,$vararg_buffer);
    break L9;
   } else {
    $289 = HEAP32[$2>>2]|0;
    $290 = ((($289)) + 34|0);
    $291 = HEAP16[$290>>1]|0;
    $292 = $291 << 16 >> 16;
    $293 = HEAP32[$pInfo>>2]|0;
    $294 = ((($293)) + 40|0);
    $295 = HEAP32[$294>>2]|0;
    $296 = (($295) + ($292<<4)|0);
    $297 = ((($296)) + 8|0);
    $298 = HEAP32[$297>>2]|0;
    HEAP32[$inReg>>2] = $298;
    break L9;
   }
   break;
  }
  case 151:  {
   HEAP32[$constMask>>2] = 0;
   $299 = HEAP32[$db>>2]|0;
   $300 = ((($299)) + 62|0);
   $301 = HEAP8[$300>>0]|0;
   HEAP8[$enc>>0] = $301;
   HEAP32[$pColl>>2] = 0;
   $302 = HEAP32[$2>>2]|0;
   $303 = ((($302)) + 4|0);
   $304 = HEAP32[$303>>2]|0;
   $305 = $304 & 16384;
   $306 = ($305|0)!=(0);
   if ($306) {
    HEAP32[$pFarg>>2] = 0;
   } else {
    $307 = HEAP32[$2>>2]|0;
    $308 = ((($307)) + 20|0);
    $309 = HEAP32[$308>>2]|0;
    HEAP32[$pFarg>>2] = $309;
   }
   $310 = HEAP32[$pFarg>>2]|0;
   $311 = ($310|0)!=(0|0);
   if ($311) {
    $312 = HEAP32[$pFarg>>2]|0;
    $313 = HEAP32[$312>>2]|0;
    $314 = $313;
   } else {
    $314 = 0;
   }
   HEAP32[$nFarg>>2] = $314;
   $315 = HEAP32[$2>>2]|0;
   $316 = ((($315)) + 8|0);
   $317 = HEAP32[$316>>2]|0;
   HEAP32[$zId>>2] = $317;
   $318 = HEAP32[$zId>>2]|0;
   $319 = (_sqlite3Strlen30($318)|0);
   HEAP32[$nId>>2] = $319;
   $320 = HEAP32[$db>>2]|0;
   $321 = HEAP32[$zId>>2]|0;
   $322 = HEAP32[$nId>>2]|0;
   $323 = HEAP32[$nFarg>>2]|0;
   $324 = HEAP8[$enc>>0]|0;
   $325 = (_sqlite3FindFunction($320,$321,$322,$323,$324,0)|0);
   HEAP32[$pDef>>2] = $325;
   $326 = HEAP32[$pDef>>2]|0;
   $327 = ($326|0)==(0|0);
   if (!($327)) {
    $328 = HEAP32[$pDef>>2]|0;
    $329 = ((($328)) + 16|0);
    $330 = HEAP32[$329>>2]|0;
    $331 = ($330|0)!=(0|0);
    if (!($331)) {
     $335 = HEAP32[$pDef>>2]|0;
     $336 = ((($335)) + 2|0);
     $337 = HEAP16[$336>>1]|0;
     $338 = $337&65535;
     $339 = $338 & 512;
     $340 = ($339|0)!=(0);
     if ($340) {
      $341 = HEAP32[$v>>2]|0;
      $342 = (_sqlite3VdbeMakeLabel($341)|0);
      HEAP32[$endCoalesce>>2] = $342;
      $343 = HEAP32[$1>>2]|0;
      $344 = HEAP32[$pFarg>>2]|0;
      $345 = ((($344)) + 4|0);
      $346 = HEAP32[$345>>2]|0;
      $347 = HEAP32[$346>>2]|0;
      $348 = HEAP32[$3>>2]|0;
      _sqlite3ExprCode($343,$347,$348);
      HEAP32[$i>>2] = 1;
      while(1) {
       $349 = HEAP32[$i>>2]|0;
       $350 = HEAP32[$nFarg>>2]|0;
       $351 = ($349|0)<($350|0);
       $352 = HEAP32[$v>>2]|0;
       if (!($351)) {
        break;
       }
       $353 = HEAP32[$3>>2]|0;
       $354 = HEAP32[$endCoalesce>>2]|0;
       (_sqlite3VdbeAddOp2($352,77,$353,$354)|0);
       $355 = HEAP32[$1>>2]|0;
       $356 = HEAP32[$3>>2]|0;
       _sqlite3ExprCacheRemove($355,$356,1);
       $357 = HEAP32[$1>>2]|0;
       _sqlite3ExprCachePush($357);
       $358 = HEAP32[$1>>2]|0;
       $359 = HEAP32[$i>>2]|0;
       $360 = HEAP32[$pFarg>>2]|0;
       $361 = ((($360)) + 4|0);
       $362 = HEAP32[$361>>2]|0;
       $363 = (($362) + (($359*20)|0)|0);
       $364 = HEAP32[$363>>2]|0;
       $365 = HEAP32[$3>>2]|0;
       _sqlite3ExprCode($358,$364,$365);
       $366 = HEAP32[$1>>2]|0;
       _sqlite3ExprCachePop($366);
       $367 = HEAP32[$i>>2]|0;
       $368 = (($367) + 1)|0;
       HEAP32[$i>>2] = $368;
      }
      $369 = HEAP32[$endCoalesce>>2]|0;
      _sqlite3VdbeResolveLabel($352,$369);
      break L9;
     }
     $370 = HEAP32[$pDef>>2]|0;
     $371 = ((($370)) + 2|0);
     $372 = HEAP16[$371>>1]|0;
     $373 = $372&65535;
     $374 = $373 & 1024;
     $375 = ($374|0)!=(0);
     if ($375) {
      $376 = HEAP32[$1>>2]|0;
      $377 = HEAP32[$pFarg>>2]|0;
      $378 = ((($377)) + 4|0);
      $379 = HEAP32[$378>>2]|0;
      $380 = HEAP32[$379>>2]|0;
      $381 = HEAP32[$3>>2]|0;
      $382 = (_sqlite3ExprCodeTarget($376,$380,$381)|0);
      HEAP32[$inReg>>2] = $382;
      break L9;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $383 = HEAP32[$i>>2]|0;
      $384 = HEAP32[$nFarg>>2]|0;
      $385 = ($383|0)<($384|0);
      if (!($385)) {
       break;
      }
      $386 = HEAP32[$i>>2]|0;
      $387 = ($386|0)<(32);
      if ($387) {
       $388 = HEAP32[$i>>2]|0;
       $389 = HEAP32[$pFarg>>2]|0;
       $390 = ((($389)) + 4|0);
       $391 = HEAP32[$390>>2]|0;
       $392 = (($391) + (($388*20)|0)|0);
       $393 = HEAP32[$392>>2]|0;
       $394 = (_sqlite3ExprIsConstant($393)|0);
       $395 = ($394|0)!=(0);
       if ($395) {
        $396 = HEAP32[$i>>2]|0;
        $397 = 1 << $396;
        $398 = HEAP32[$constMask>>2]|0;
        $399 = $398 | $397;
        HEAP32[$constMask>>2] = $399;
       }
      }
      $400 = HEAP32[$pDef>>2]|0;
      $401 = ((($400)) + 2|0);
      $402 = HEAP16[$401>>1]|0;
      $403 = $402&65535;
      $404 = $403 & 32;
      $405 = ($404|0)==(0);
      $406 = HEAP32[$pColl>>2]|0;
      $407 = ($406|0)!=(0|0);
      $or$cond = $405 | $407;
      if (!($or$cond)) {
       $408 = HEAP32[$1>>2]|0;
       $409 = HEAP32[$i>>2]|0;
       $410 = HEAP32[$pFarg>>2]|0;
       $411 = ((($410)) + 4|0);
       $412 = HEAP32[$411>>2]|0;
       $413 = (($412) + (($409*20)|0)|0);
       $414 = HEAP32[$413>>2]|0;
       $415 = (_sqlite3ExprCollSeq($408,$414)|0);
       HEAP32[$pColl>>2] = $415;
      }
      $416 = HEAP32[$i>>2]|0;
      $417 = (($416) + 1)|0;
      HEAP32[$i>>2] = $417;
     }
     $418 = HEAP32[$pFarg>>2]|0;
     $419 = ($418|0)!=(0|0);
     if ($419) {
      $420 = HEAP32[$constMask>>2]|0;
      $421 = ($420|0)!=(0);
      $422 = HEAP32[$1>>2]|0;
      if ($421) {
       $423 = ((($422)) + 76|0);
       $424 = HEAP32[$423>>2]|0;
       $425 = (($424) + 1)|0;
       HEAP32[$r1>>2] = $425;
       $426 = HEAP32[$nFarg>>2]|0;
       $427 = HEAP32[$1>>2]|0;
       $428 = ((($427)) + 76|0);
       $429 = HEAP32[$428>>2]|0;
       $430 = (($429) + ($426))|0;
       HEAP32[$428>>2] = $430;
      } else {
       $431 = HEAP32[$nFarg>>2]|0;
       $432 = (_sqlite3GetTempRange($422,$431)|0);
       HEAP32[$r1>>2] = $432;
      }
      $433 = HEAP32[$pDef>>2]|0;
      $434 = ((($433)) + 2|0);
      $435 = HEAP16[$434>>1]|0;
      $436 = $435&65535;
      $437 = $436 & 192;
      $438 = ($437|0)!=(0);
      do {
       if ($438) {
        $439 = HEAP32[$pFarg>>2]|0;
        $440 = ((($439)) + 4|0);
        $441 = HEAP32[$440>>2]|0;
        $442 = HEAP32[$441>>2]|0;
        $443 = HEAP8[$442>>0]|0;
        HEAP8[$exprOp>>0] = $443;
        $444 = HEAP8[$exprOp>>0]|0;
        $445 = $444&255;
        $446 = ($445|0)==(152);
        if (!($446)) {
         $447 = HEAP8[$exprOp>>0]|0;
         $448 = $447&255;
         $449 = ($448|0)==(154);
         if (!($449)) {
          break;
         }
        }
        $450 = HEAP32[$pDef>>2]|0;
        $451 = ((($450)) + 2|0);
        $452 = HEAP16[$451>>1]|0;
        $453 = $452&65535;
        $454 = $453 & 192;
        $455 = $454&255;
        $456 = HEAP32[$pFarg>>2]|0;
        $457 = ((($456)) + 4|0);
        $458 = HEAP32[$457>>2]|0;
        $459 = HEAP32[$458>>2]|0;
        $460 = ((($459)) + 38|0);
        HEAP8[$460>>0] = $455;
       }
      } while(0);
      $461 = HEAP32[$1>>2]|0;
      _sqlite3ExprCachePush($461);
      $462 = HEAP32[$1>>2]|0;
      $463 = HEAP32[$pFarg>>2]|0;
      $464 = HEAP32[$r1>>2]|0;
      (_sqlite3ExprCodeExprList($462,$463,$464,0,3)|0);
      $465 = HEAP32[$1>>2]|0;
      _sqlite3ExprCachePop($465);
     } else {
      HEAP32[$r1>>2] = 0;
     }
     $466 = HEAP32[$nFarg>>2]|0;
     $467 = ($466|0)>=(2);
     if ($467) {
      $468 = HEAP32[$2>>2]|0;
      $469 = ((($468)) + 4|0);
      $470 = HEAP32[$469>>2]|0;
      $471 = $470 & 128;
      $472 = ($471|0)!=(0);
      if ($472) {
       $473 = HEAP32[$db>>2]|0;
       $474 = HEAP32[$pDef>>2]|0;
       $475 = HEAP32[$nFarg>>2]|0;
       $476 = HEAP32[$pFarg>>2]|0;
       $477 = ((($476)) + 4|0);
       $478 = HEAP32[$477>>2]|0;
       $479 = ((($478)) + 20|0);
       $480 = HEAP32[$479>>2]|0;
       $481 = (_sqlite3VtabOverloadFunction($473,$474,$475,$480)|0);
       HEAP32[$pDef>>2] = $481;
      } else {
       label = 77;
      }
     } else {
      label = 77;
     }
     if ((label|0) == 77) {
      $482 = HEAP32[$nFarg>>2]|0;
      $483 = ($482|0)>(0);
      if ($483) {
       $484 = HEAP32[$db>>2]|0;
       $485 = HEAP32[$pDef>>2]|0;
       $486 = HEAP32[$nFarg>>2]|0;
       $487 = HEAP32[$pFarg>>2]|0;
       $488 = ((($487)) + 4|0);
       $489 = HEAP32[$488>>2]|0;
       $490 = HEAP32[$489>>2]|0;
       $491 = (_sqlite3VtabOverloadFunction($484,$485,$486,$490)|0);
       HEAP32[$pDef>>2] = $491;
      }
     }
     $492 = HEAP32[$pDef>>2]|0;
     $493 = ((($492)) + 2|0);
     $494 = HEAP16[$493>>1]|0;
     $495 = $494&65535;
     $496 = $495 & 32;
     $497 = ($496|0)!=(0);
     if ($497) {
      $498 = HEAP32[$pColl>>2]|0;
      $499 = ($498|0)!=(0|0);
      if (!($499)) {
       $500 = HEAP32[$db>>2]|0;
       $501 = ((($500)) + 8|0);
       $502 = HEAP32[$501>>2]|0;
       HEAP32[$pColl>>2] = $502;
      }
      $503 = HEAP32[$v>>2]|0;
      $504 = HEAP32[$pColl>>2]|0;
      (_sqlite3VdbeAddOp4($503,34,0,0,0,$504,-4)|0);
     }
     $505 = HEAP32[$v>>2]|0;
     $506 = HEAP32[$constMask>>2]|0;
     $507 = HEAP32[$r1>>2]|0;
     $508 = HEAP32[$3>>2]|0;
     $509 = HEAP32[$pDef>>2]|0;
     (_sqlite3VdbeAddOp4($505,35,$506,$507,$508,$509,-5)|0);
     $510 = HEAP32[$v>>2]|0;
     $511 = HEAP32[$nFarg>>2]|0;
     $512 = $511&255;
     _sqlite3VdbeChangeP5($510,$512);
     $513 = HEAP32[$nFarg>>2]|0;
     $514 = ($513|0)!=(0);
     $515 = HEAP32[$constMask>>2]|0;
     $516 = ($515|0)==(0);
     $or$cond3 = $514 & $516;
     if (!($or$cond3)) {
      break L9;
     }
     $517 = HEAP32[$1>>2]|0;
     $518 = HEAP32[$r1>>2]|0;
     $519 = HEAP32[$nFarg>>2]|0;
     _sqlite3ReleaseTempRange($517,$518,$519);
     break L9;
    }
   }
   $332 = HEAP32[$1>>2]|0;
   $333 = HEAP32[$nId>>2]|0;
   $334 = HEAP32[$zId>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $333;
   $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $334;
   _sqlite3ErrorMsg($332,28457,$vararg_buffer4);
   break;
  }
  case 119: case 20:  {
   $520 = HEAP32[$1>>2]|0;
   $521 = HEAP32[$2>>2]|0;
   $522 = (_sqlite3CodeSubselect($520,$521,0,0)|0);
   HEAP32[$inReg>>2] = $522;
   break;
  }
  case 75:  {
   $523 = HEAP32[$v>>2]|0;
   $524 = (_sqlite3VdbeMakeLabel($523)|0);
   HEAP32[$destIfFalse>>2] = $524;
   $525 = HEAP32[$v>>2]|0;
   $526 = (_sqlite3VdbeMakeLabel($525)|0);
   HEAP32[$destIfNull>>2] = $526;
   $527 = HEAP32[$v>>2]|0;
   $528 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($527,25,0,$528)|0);
   $529 = HEAP32[$1>>2]|0;
   $530 = HEAP32[$2>>2]|0;
   $531 = HEAP32[$destIfFalse>>2]|0;
   $532 = HEAP32[$destIfNull>>2]|0;
   _sqlite3ExprCodeIN($529,$530,$531,$532);
   $533 = HEAP32[$v>>2]|0;
   $534 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($533,22,1,$534)|0);
   $535 = HEAP32[$v>>2]|0;
   $536 = HEAP32[$destIfFalse>>2]|0;
   _sqlite3VdbeResolveLabel($535,$536);
   $537 = HEAP32[$v>>2]|0;
   $538 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($537,37,$538,0)|0);
   $539 = HEAP32[$v>>2]|0;
   $540 = HEAP32[$destIfNull>>2]|0;
   _sqlite3VdbeResolveLabel($539,$540);
   break;
  }
  case 74:  {
   $541 = HEAP32[$2>>2]|0;
   $542 = ((($541)) + 12|0);
   $543 = HEAP32[$542>>2]|0;
   HEAP32[$pLeft1>>2] = $543;
   $544 = HEAP32[$2>>2]|0;
   $545 = ((($544)) + 20|0);
   $546 = HEAP32[$545>>2]|0;
   $547 = ((($546)) + 4|0);
   $548 = HEAP32[$547>>2]|0;
   HEAP32[$pLItem>>2] = $548;
   $549 = HEAP32[$pLItem>>2]|0;
   $550 = HEAP32[$549>>2]|0;
   HEAP32[$pRight>>2] = $550;
   $551 = HEAP32[$1>>2]|0;
   $552 = HEAP32[$pLeft1>>2]|0;
   $553 = (_sqlite3ExprCodeTemp($551,$552,$regFree1)|0);
   HEAP32[$r1>>2] = $553;
   $554 = HEAP32[$1>>2]|0;
   $555 = HEAP32[$pRight>>2]|0;
   $556 = (_sqlite3ExprCodeTemp($554,$555,$regFree2)|0);
   HEAP32[$r2>>2] = $556;
   $557 = HEAP32[$1>>2]|0;
   $558 = (_sqlite3GetTempReg($557)|0);
   HEAP32[$r3>>2] = $558;
   $559 = HEAP32[$1>>2]|0;
   $560 = (_sqlite3GetTempReg($559)|0);
   HEAP32[$r4>>2] = $560;
   $561 = HEAP32[$1>>2]|0;
   $562 = HEAP32[$pLeft1>>2]|0;
   $563 = HEAP32[$pRight>>2]|0;
   $564 = HEAP32[$r1>>2]|0;
   $565 = HEAP32[$r2>>2]|0;
   $566 = HEAP32[$r3>>2]|0;
   (_codeCompare($561,$562,$563,83,$564,$565,$566,32)|0);
   $567 = HEAP32[$pLItem>>2]|0;
   $568 = ((($567)) + 20|0);
   HEAP32[$pLItem>>2] = $568;
   $569 = HEAP32[$pLItem>>2]|0;
   $570 = HEAP32[$569>>2]|0;
   HEAP32[$pRight>>2] = $570;
   $571 = HEAP32[$1>>2]|0;
   $572 = HEAP32[$regFree2>>2]|0;
   _sqlite3ReleaseTempReg($571,$572);
   $573 = HEAP32[$1>>2]|0;
   $574 = HEAP32[$pRight>>2]|0;
   $575 = (_sqlite3ExprCodeTemp($573,$574,$regFree2)|0);
   HEAP32[$r2>>2] = $575;
   $576 = HEAP32[$1>>2]|0;
   $577 = HEAP32[$pLeft1>>2]|0;
   $578 = HEAP32[$pRight>>2]|0;
   $579 = HEAP32[$r1>>2]|0;
   $580 = HEAP32[$r2>>2]|0;
   $581 = HEAP32[$r4>>2]|0;
   (_codeCompare($576,$577,$578,81,$579,$580,$581,32)|0);
   $582 = HEAP32[$v>>2]|0;
   $583 = HEAP32[$r3>>2]|0;
   $584 = HEAP32[$r4>>2]|0;
   $585 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp3($582,72,$583,$584,$585)|0);
   $586 = HEAP32[$1>>2]|0;
   $587 = HEAP32[$r3>>2]|0;
   _sqlite3ReleaseTempReg($586,$587);
   $588 = HEAP32[$1>>2]|0;
   $589 = HEAP32[$r4>>2]|0;
   _sqlite3ReleaseTempReg($588,$589);
   break;
  }
  case 156: case 95:  {
   $590 = HEAP32[$1>>2]|0;
   $591 = HEAP32[$2>>2]|0;
   $592 = ((($591)) + 12|0);
   $593 = HEAP32[$592>>2]|0;
   $594 = HEAP32[$3>>2]|0;
   $595 = (_sqlite3ExprCodeTarget($590,$593,$594)|0);
   HEAP32[$inReg>>2] = $595;
   break;
  }
  case 62:  {
   $596 = HEAP32[$2>>2]|0;
   $597 = ((($596)) + 44|0);
   $598 = HEAP32[$597>>2]|0;
   HEAP32[$pTab>>2] = $598;
   $599 = HEAP32[$2>>2]|0;
   $600 = ((($599)) + 28|0);
   $601 = HEAP32[$600>>2]|0;
   $602 = HEAP32[$pTab>>2]|0;
   $603 = ((($602)) + 34|0);
   $604 = HEAP16[$603>>1]|0;
   $605 = $604 << 16 >> 16;
   $606 = (($605) + 1)|0;
   $607 = Math_imul($601, $606)|0;
   $608 = (($607) + 1)|0;
   $609 = HEAP32[$2>>2]|0;
   $610 = ((($609)) + 32|0);
   $611 = HEAP16[$610>>1]|0;
   $612 = $611 << 16 >> 16;
   $613 = (($608) + ($612))|0;
   HEAP32[$p1>>2] = $613;
   $614 = HEAP32[$v>>2]|0;
   $615 = HEAP32[$p1>>2]|0;
   $616 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($614,134,$615,$616)|0);
   $617 = HEAP32[$2>>2]|0;
   $618 = ((($617)) + 32|0);
   $619 = HEAP16[$618>>1]|0;
   $620 = $619 << 16 >> 16;
   $621 = ($620|0)>=(0);
   if ($621) {
    $622 = HEAP32[$2>>2]|0;
    $623 = ((($622)) + 32|0);
    $624 = HEAP16[$623>>1]|0;
    $625 = $624 << 16 >> 16;
    $626 = HEAP32[$pTab>>2]|0;
    $627 = ((($626)) + 4|0);
    $628 = HEAP32[$627>>2]|0;
    $629 = (($628) + (($625*24)|0)|0);
    $630 = ((($629)) + 21|0);
    $631 = HEAP8[$630>>0]|0;
    $632 = $631 << 24 >> 24;
    $633 = ($632|0)==(69);
    if ($633) {
     $634 = HEAP32[$v>>2]|0;
     $635 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp1($634,39,$635)|0);
    }
   }
   break;
  }
  case 57:  {
   $704 = HEAP32[$1>>2]|0;
   $705 = ((($704)) + 420|0);
   $706 = HEAP32[$705>>2]|0;
   $707 = ($706|0)!=(0|0);
   if (!($707)) {
    $708 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($708,28482,$vararg_buffer8);
    HEAP32[$0>>2] = 0;
    $737 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($737|0);
   }
   $709 = HEAP32[$2>>2]|0;
   $710 = ((($709)) + 1|0);
   $711 = HEAP8[$710>>0]|0;
   $712 = $711 << 24 >> 24;
   $713 = ($712|0)==(2);
   if ($713) {
    $714 = HEAP32[$1>>2]|0;
    _sqlite3MayAbort($714);
   }
   $715 = HEAP32[$2>>2]|0;
   $716 = ((($715)) + 1|0);
   $717 = HEAP8[$716>>0]|0;
   $718 = $717 << 24 >> 24;
   $719 = ($718|0)==(4);
   if ($719) {
    $720 = HEAP32[$v>>2]|0;
    $721 = HEAP32[$2>>2]|0;
    $722 = ((($721)) + 8|0);
    $723 = HEAP32[$722>>2]|0;
    (_sqlite3VdbeAddOp4($720,21,0,4,0,$723,0)|0);
    break L9;
   } else {
    $724 = HEAP32[$1>>2]|0;
    $725 = HEAP32[$2>>2]|0;
    $726 = ((($725)) + 1|0);
    $727 = HEAP8[$726>>0]|0;
    $728 = $727 << 24 >> 24;
    $729 = HEAP32[$2>>2]|0;
    $730 = ((($729)) + 8|0);
    $731 = HEAP32[$730>>2]|0;
    _sqlite3HaltConstraint($724,1811,$728,$731,0,0);
    break L9;
   }
   break;
  }
  default: {
   HEAP32[$pTest>>2] = 0;
   $636 = HEAP32[$2>>2]|0;
   $637 = ((($636)) + 20|0);
   $638 = HEAP32[$637>>2]|0;
   HEAP32[$pEList>>2] = $638;
   $639 = HEAP32[$pEList>>2]|0;
   $640 = ((($639)) + 4|0);
   $641 = HEAP32[$640>>2]|0;
   HEAP32[$aListelem>>2] = $641;
   $642 = HEAP32[$pEList>>2]|0;
   $643 = HEAP32[$642>>2]|0;
   HEAP32[$nExpr>>2] = $643;
   $644 = HEAP32[$v>>2]|0;
   $645 = (_sqlite3VdbeMakeLabel($644)|0);
   HEAP32[$endLabel>>2] = $645;
   $646 = HEAP32[$2>>2]|0;
   $647 = ((($646)) + 12|0);
   $648 = HEAP32[$647>>2]|0;
   HEAP32[$pX>>2] = $648;
   $649 = ($648|0)!=(0|0);
   if ($649) {
    $650 = HEAP32[$pX>>2]|0;
    dest=$tempX; src=$650; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $651 = HEAP32[$1>>2]|0;
    $652 = HEAP32[$pX>>2]|0;
    $653 = (_sqlite3ExprCodeTemp($651,$652,$regFree1)|0);
    _exprToRegister($tempX,$653);
    HEAP8[$opCompare>>0] = 79;
    $654 = ((($opCompare)) + 12|0);
    HEAP32[$654>>2] = $tempX;
    HEAP32[$pTest>>2] = $opCompare;
    HEAP32[$regFree1>>2] = 0;
   }
   HEAP32[$i2>>2] = 0;
   while(1) {
    $655 = HEAP32[$i2>>2]|0;
    $656 = HEAP32[$nExpr>>2]|0;
    $657 = (($656) - 1)|0;
    $658 = ($655|0)<($657|0);
    if (!($658)) {
     break;
    }
    $659 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePush($659);
    $660 = HEAP32[$pX>>2]|0;
    $661 = ($660|0)!=(0|0);
    $662 = HEAP32[$i2>>2]|0;
    $663 = HEAP32[$aListelem>>2]|0;
    $664 = (($663) + (($662*20)|0)|0);
    $665 = HEAP32[$664>>2]|0;
    if ($661) {
     $666 = ((($opCompare)) + 16|0);
     HEAP32[$666>>2] = $665;
    } else {
     HEAP32[$pTest>>2] = $665;
    }
    $667 = HEAP32[$v>>2]|0;
    $668 = (_sqlite3VdbeMakeLabel($667)|0);
    HEAP32[$nextCase>>2] = $668;
    $669 = HEAP32[$1>>2]|0;
    $670 = HEAP32[$pTest>>2]|0;
    $671 = HEAP32[$nextCase>>2]|0;
    _sqlite3ExprIfFalse($669,$670,$671,16);
    $672 = HEAP32[$1>>2]|0;
    $673 = HEAP32[$i2>>2]|0;
    $674 = (($673) + 1)|0;
    $675 = HEAP32[$aListelem>>2]|0;
    $676 = (($675) + (($674*20)|0)|0);
    $677 = HEAP32[$676>>2]|0;
    $678 = HEAP32[$3>>2]|0;
    _sqlite3ExprCode($672,$677,$678);
    $679 = HEAP32[$v>>2]|0;
    $680 = HEAP32[$endLabel>>2]|0;
    (_sqlite3VdbeGoto($679,$680)|0);
    $681 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePop($681);
    $682 = HEAP32[$v>>2]|0;
    $683 = HEAP32[$nextCase>>2]|0;
    _sqlite3VdbeResolveLabel($682,$683);
    $684 = HEAP32[$i2>>2]|0;
    $685 = (($684) + 2)|0;
    HEAP32[$i2>>2] = $685;
   }
   $686 = HEAP32[$nExpr>>2]|0;
   $687 = $686 & 1;
   $688 = ($687|0)!=(0);
   if ($688) {
    $689 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePush($689);
    $690 = HEAP32[$1>>2]|0;
    $691 = HEAP32[$nExpr>>2]|0;
    $692 = (($691) - 1)|0;
    $693 = HEAP32[$pEList>>2]|0;
    $694 = ((($693)) + 4|0);
    $695 = HEAP32[$694>>2]|0;
    $696 = (($695) + (($692*20)|0)|0);
    $697 = HEAP32[$696>>2]|0;
    $698 = HEAP32[$3>>2]|0;
    _sqlite3ExprCode($690,$697,$698);
    $699 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePop($699);
   } else {
    $700 = HEAP32[$v>>2]|0;
    $701 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($700,25,0,$701)|0);
   }
   $702 = HEAP32[$v>>2]|0;
   $703 = HEAP32[$endLabel>>2]|0;
   _sqlite3VdbeResolveLabel($702,$703);
  }
  }
 } while(0);
 L139: do {
  if ((label|0) == 11) {
   $47 = HEAP32[$2>>2]|0;
   $48 = ((($47)) + 28|0);
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$iTab>>2] = $49;
   $50 = HEAP32[$iTab>>2]|0;
   $51 = ($50|0)<(0);
   do {
    if ($51) {
     $52 = HEAP32[$1>>2]|0;
     $53 = ((($52)) + 100|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)>(0);
     if ($55) {
      $56 = HEAP32[$2>>2]|0;
      $57 = ((($56)) + 32|0);
      $58 = HEAP16[$57>>1]|0;
      $59 = $58 << 16 >> 16;
      $60 = HEAP32[$1>>2]|0;
      $61 = ((($60)) + 100|0);
      $62 = HEAP32[$61>>2]|0;
      $63 = (($59) + ($62))|0;
      HEAP32[$inReg>>2] = $63;
      break L139;
     } else {
      $64 = HEAP32[$1>>2]|0;
      $65 = ((($64)) + 104|0);
      $66 = HEAP32[$65>>2]|0;
      HEAP32[$iTab>>2] = $66;
      break;
     }
    }
   } while(0);
   $67 = HEAP32[$1>>2]|0;
   $68 = HEAP32[$2>>2]|0;
   $69 = ((($68)) + 44|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$2>>2]|0;
   $72 = ((($71)) + 32|0);
   $73 = HEAP16[$72>>1]|0;
   $74 = $73 << 16 >> 16;
   $75 = HEAP32[$iTab>>2]|0;
   $76 = HEAP32[$3>>2]|0;
   $77 = HEAP32[$2>>2]|0;
   $78 = ((($77)) + 38|0);
   $79 = HEAP8[$78>>0]|0;
   $80 = (_sqlite3ExprCodeGetColumn($67,$70,$74,$75,$76,$79)|0);
   HEAP32[$inReg>>2] = $80;
  }
 } while(0);
 $732 = HEAP32[$1>>2]|0;
 $733 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($732,$733);
 $734 = HEAP32[$1>>2]|0;
 $735 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($734,$735);
 $736 = HEAP32[$inReg>>2]|0;
 HEAP32[$0>>2] = $736;
 $737 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($737|0);
}
function _sqlite3ExprCodeGetColumn($pParse,$pTab,$iColumn,$iTable,$iReg,$p5) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iColumn = $iColumn|0;
 $iTable = $iTable|0;
 $iReg = $iReg|0;
 $p5 = $p5|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $p = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 36|0;
 $v = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $iColumn;
 HEAP32[$4>>2] = $iTable;
 HEAP32[$5>>2] = $iReg;
 HEAP8[$6>>0] = $p5;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 HEAP32[$i>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 124|0);
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = ($12|0)<(10);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)>(0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = ($19|0)==($20|0);
   if ($21) {
    $22 = HEAP32[$p>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP16[$23>>1]|0;
    $25 = $24 << 16 >> 16;
    $26 = HEAP32[$3>>2]|0;
    $27 = ($25|0)==($26|0);
    if ($27) {
     label = 6;
     break;
    }
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  $44 = ((($43)) + 20|0);
  HEAP32[$p>>2] = $44;
 }
 if ((label|0) == 6) {
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 112|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$29>>2] = $31;
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 16|0);
  HEAP32[$33>>2] = $30;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3ExprCachePinRegister($34,$37);
  $38 = HEAP32[$p>>2]|0;
  $39 = ((($38)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$0>>2] = $40;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $45 = HEAP32[$v>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = HEAP32[$4>>2]|0;
 $48 = HEAP32[$3>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 _sqlite3ExprCodeGetColumnOfTable($45,$46,$47,$48,$49);
 $50 = HEAP8[$6>>0]|0;
 $51 = ($50<<24>>24)!=(0);
 if ($51) {
  $52 = HEAP32[$v>>2]|0;
  $53 = HEAP8[$6>>0]|0;
  _sqlite3VdbeChangeP5($52,$53);
 } else {
  $54 = HEAP32[$1>>2]|0;
  $55 = HEAP32[$4>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = HEAP32[$5>>2]|0;
  _sqlite3ExprCacheStore($54,$55,$56,$57);
 }
 $58 = HEAP32[$5>>2]|0;
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _codeInteger($pParse,$pExpr,$negFlag,$iMem) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $negFlag = $negFlag|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $c = 0, $i = 0, $or$cond = 0, $v = 0, $value = 0, $vararg_buffer = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $c = sp + 16|0;
 $value = sp;
 $z = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $negFlag;
 HEAP32[$3>>2] = $iMem;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 1024;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 8|0);
 if ($11) {
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = HEAP32[$i>>2]|0;
   $18 = (0 - ($17))|0;
   HEAP32[$i>>2] = $18;
  }
  $19 = HEAP32[$v>>2]|0;
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  (_sqlite3VdbeAddOp2($19,22,$20,$21)|0);
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$13>>2]|0;
 HEAP32[$z>>2] = $22;
 $23 = HEAP32[$z>>2]|0;
 $24 = (_sqlite3DecOrHexToI64($23,$value)|0);
 HEAP32[$c>>2] = $24;
 $25 = HEAP32[$c>>2]|0;
 $26 = ($25|0)==(0);
 if (!($26)) {
  $27 = HEAP32[$c>>2]|0;
  $28 = ($27|0)==(2);
  $29 = HEAP32[$2>>2]|0;
  $30 = ($29|0)!=(0);
  $or$cond = $28 & $30;
  if (!($or$cond)) {
   $51 = HEAP32[$z>>2]|0;
   $52 = (_sqlite3_strnicmp($51,28537,2)|0);
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP32[$0>>2]|0;
    $55 = HEAP32[$z>>2]|0;
    HEAP32[$vararg_buffer>>2] = $55;
    _sqlite3ErrorMsg($54,28540,$vararg_buffer);
    STACKTOP = sp;return;
   } else {
    $56 = HEAP32[$v>>2]|0;
    $57 = HEAP32[$z>>2]|0;
    $58 = HEAP32[$2>>2]|0;
    $59 = HEAP32[$3>>2]|0;
    _codeReal($56,$57,$58,$59);
    STACKTOP = sp;return;
   }
  }
 }
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$c>>2]|0;
  $34 = ($33|0)==(2);
  $35 = $value;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (_i64Subtract(0,0,($37|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $34 ? 0 : $41;
  $44 = $34 ? -2147483648 : $42;
  $45 = $value;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
 }
 $49 = HEAP32[$v>>2]|0;
 $50 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Dup8($49,23,0,$50,0,$value,-13)|0);
 STACKTOP = sp;return;
}
function _codeReal($v,$z,$negateFlag,$iMem) {
 $v = $v|0;
 $z = $z|0;
 $negateFlag = $negateFlag|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $value = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $negateFlag;
 HEAP32[$3>>2] = $iMem;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 (_sqlite3AtoF($6,$value,$8,1)|0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = +HEAPF64[$value>>3];
  $12 = -$11;
  HEAPF64[$value>>3] = $12;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Dup8($13,133,0,$14,0,$value,-12)|0);
 STACKTOP = sp;return;
}
function _sqlite3HexToBlob($db,$z,$n) {
 $db = $db|0;
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zBlob = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zBlob = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4|0) / 2)&-1;
 $6 = (($5) + 1)|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_sqlite3DbMallocRawNN($3,$6,$8)|0);
 HEAP32[$zBlob>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + -1)|0;
 HEAP32[$2>>2] = $11;
 $12 = HEAP32[$zBlob>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $44 = HEAP32[$zBlob>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($14|0)<($15|0);
  $17 = HEAP32[$i>>2]|0;
  if (!($16)) {
   break;
  }
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + ($17)|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = (_sqlite3HexToInt($21)|0);
  $23 = $22&255;
  $24 = $23 << 4;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + ($26)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = (_sqlite3HexToInt($30)|0);
  $32 = $31&255;
  $33 = $24 | $32;
  $34 = $33&255;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35|0) / 2)&-1;
  $37 = HEAP32[$zBlob>>2]|0;
  $38 = (($37) + ($36)|0);
  HEAP8[$38>>0] = $34;
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 2)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = (($17|0) / 2)&-1;
 $42 = HEAP32[$zBlob>>2]|0;
 $43 = (($42) + ($41)|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$zBlob>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3AffinityType($zIn,$pszEst) {
 $zIn = $zIn|0;
 $pszEst = $pszEst|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $aff = 0, $h = 0, $v = 0, $zChar = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 21|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $h = sp + 8|0;
 $aff = sp + 20|0;
 $zChar = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $zIn;
 HEAP32[$2>>2] = $pszEst;
 HEAP32[$h>>2] = 0;
 HEAP8[$aff>>0] = 67;
 HEAP32[$zChar>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP8[$aff>>0]|0;
  HEAP8[$0>>0] = $5;
  $91 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($91|0);
 }
 L4: while(1) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = ($7<<24>>24)!=(0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$h>>2]|0;
  $10 = $9 << 8;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = $13 & 255;
  $15 = (16974 + ($14)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($10) + ($17))|0;
  HEAP32[$h>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP32[$1>>2] = $20;
  $21 = HEAP32[$h>>2]|0;
  $22 = ($21|0)==(1667785074);
  if ($22) {
   HEAP8[$aff>>0] = 66;
   $23 = HEAP32[$1>>2]|0;
   HEAP32[$zChar>>2] = $23;
   continue;
  }
  $24 = HEAP32[$h>>2]|0;
  $25 = ($24|0)==(1668050786);
  if ($25) {
   HEAP8[$aff>>0] = 66;
   continue;
  }
  $26 = HEAP32[$h>>2]|0;
  $27 = ($26|0)==(1952807028);
  if ($27) {
   HEAP8[$aff>>0] = 66;
   continue;
  }
  $28 = HEAP32[$h>>2]|0;
  $29 = ($28|0)==(1651273570);
  do {
   if ($29) {
    $30 = HEAP8[$aff>>0]|0;
    $31 = $30 << 24 >> 24;
    $32 = ($31|0)==(67);
    if (!($32)) {
     $33 = HEAP8[$aff>>0]|0;
     $34 = $33 << 24 >> 24;
     $35 = ($34|0)==(69);
     if (!($35)) {
      break;
     }
    }
    HEAP8[$aff>>0] = 65;
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP8[$36>>0]|0;
    $38 = $37 << 24 >> 24;
    $39 = ($38|0)==(40);
    if (!($39)) {
     continue L4;
    }
    $40 = HEAP32[$1>>2]|0;
    HEAP32[$zChar>>2] = $40;
    continue L4;
   }
  } while(0);
  $41 = HEAP32[$h>>2]|0;
  $42 = ($41|0)==(1919246700);
  if ($42) {
   $43 = HEAP8[$aff>>0]|0;
   $44 = $43 << 24 >> 24;
   $45 = ($44|0)==(67);
   if ($45) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $46 = HEAP32[$h>>2]|0;
  $47 = ($46|0)==(1718382433);
  if ($47) {
   $48 = HEAP8[$aff>>0]|0;
   $49 = $48 << 24 >> 24;
   $50 = ($49|0)==(67);
   if ($50) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $51 = HEAP32[$h>>2]|0;
  $52 = ($51|0)==(1685026146);
  if ($52) {
   $53 = HEAP8[$aff>>0]|0;
   $54 = $53 << 24 >> 24;
   $55 = ($54|0)==(67);
   if ($55) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $56 = HEAP32[$h>>2]|0;
  $57 = $56 & 16777215;
  $58 = ($57|0)==(6909556);
  if ($58) {
   label = 25;
   break;
  }
 }
 if ((label|0) == 25) {
  HEAP8[$aff>>0] = 68;
 }
 $59 = HEAP32[$2>>2]|0;
 $60 = ($59|0)!=(0|0);
 L38: do {
  if ($60) {
   $61 = HEAP32[$2>>2]|0;
   HEAP8[$61>>0] = 1;
   $62 = HEAP8[$aff>>0]|0;
   $63 = $62 << 24 >> 24;
   $64 = ($63|0)<(67);
   if ($64) {
    $65 = HEAP32[$zChar>>2]|0;
    $66 = ($65|0)!=(0|0);
    if (!($66)) {
     $89 = HEAP32[$2>>2]|0;
     HEAP8[$89>>0] = 5;
     break;
    }
    while(1) {
     $67 = HEAP32[$zChar>>2]|0;
     $68 = HEAP8[$67>>0]|0;
     $69 = ($68<<24>>24)!=(0);
     if (!($69)) {
      break L38;
     }
     $70 = HEAP32[$zChar>>2]|0;
     $71 = HEAP8[$70>>0]|0;
     $72 = $71&255;
     $73 = (17533 + ($72)|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     $76 = $75 & 4;
     $77 = ($76|0)!=(0);
     if ($77) {
      break;
     }
     $87 = HEAP32[$zChar>>2]|0;
     $88 = ((($87)) + 1|0);
     HEAP32[$zChar>>2] = $88;
    }
    HEAP32[$v>>2] = 0;
    $78 = HEAP32[$zChar>>2]|0;
    (_sqlite3GetInt32($78,$v)|0);
    $79 = HEAP32[$v>>2]|0;
    $80 = (($79|0) / 4)&-1;
    $81 = (($80) + 1)|0;
    HEAP32[$v>>2] = $81;
    $82 = HEAP32[$v>>2]|0;
    $83 = ($82|0)>(255);
    $$ = $83 ? 255 : $81;
    HEAP32[$v>>2] = $$;
    $84 = HEAP32[$v>>2]|0;
    $85 = $84&255;
    $86 = HEAP32[$2>>2]|0;
    HEAP8[$86>>0] = $85;
   }
  }
 } while(0);
 $90 = HEAP8[$aff>>0]|0;
 HEAP8[$0>>0] = $90;
 $91 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($91|0);
}
function _sqlite3ExprCacheAffinityChange($pParse,$iStart,$iCount) {
 $pParse = $pParse|0;
 $iStart = $iStart|0;
 $iCount = $iCount|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iStart;
 HEAP32[$2>>2] = $iCount;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheRemove($3,$4,$5);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeTemp($pParse,$pExpr,$pReg) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pReg = $pReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pItem = 0, $r1 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $r2 = sp + 16|0;
 $p = sp + 12|0;
 $i = sp + 8|0;
 $pItem = sp + 4|0;
 $r1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pReg;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3ExprSkipCollate($4)|0);
 HEAP32[$2>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 23|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(157);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = (_sqlite3ExprIsConstantNotJoin($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 324|0);
    $20 = HEAP32[$19>>2]|0;
    HEAP32[$p>>2] = $20;
    $21 = HEAP32[$3>>2]|0;
    HEAP32[$21>>2] = 0;
    $22 = HEAP32[$p>>2]|0;
    $23 = ($22|0)!=(0|0);
    L5: do {
     if ($23) {
      $24 = HEAP32[$p>>2]|0;
      $25 = ((($24)) + 4|0);
      $26 = HEAP32[$25>>2]|0;
      HEAP32[$pItem>>2] = $26;
      $27 = HEAP32[$p>>2]|0;
      $28 = HEAP32[$27>>2]|0;
      HEAP32[$i>>2] = $28;
      while(1) {
       $29 = HEAP32[$i>>2]|0;
       $30 = ($29|0)>(0);
       if (!($30)) {
        break L5;
       }
       $31 = HEAP32[$pItem>>2]|0;
       $32 = ((($31)) + 13|0);
       $33 = HEAP8[$32>>0]|0;
       $34 = ($33&255) >>> 2;
       $35 = $34 & 1;
       $36 = $35&255;
       $37 = ($36|0)!=(0);
       if ($37) {
        $38 = HEAP32[$pItem>>2]|0;
        $39 = HEAP32[$38>>2]|0;
        $40 = HEAP32[$2>>2]|0;
        $41 = (_sqlite3ExprCompare($39,$40,-1)|0);
        $42 = ($41|0)==(0);
        if ($42) {
         break;
        }
       }
       $46 = HEAP32[$pItem>>2]|0;
       $47 = ((($46)) + 20|0);
       HEAP32[$pItem>>2] = $47;
       $48 = HEAP32[$i>>2]|0;
       $49 = (($48) + -1)|0;
       HEAP32[$i>>2] = $49;
      }
      $43 = HEAP32[$pItem>>2]|0;
      $44 = ((($43)) + 16|0);
      $45 = HEAP32[$44>>2]|0;
      HEAP32[$0>>2] = $45;
      $72 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($72|0);
     }
    } while(0);
    $50 = HEAP32[$1>>2]|0;
    $51 = ((($50)) + 76|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (($52) + 1)|0;
    HEAP32[$51>>2] = $53;
    HEAP32[$r2>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = HEAP32[$r2>>2]|0;
    _sqlite3ExprCodeAtInit($54,$55,$56,1);
   } else {
    label = 12;
   }
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 do {
  if ((label|0) == 12) {
   $57 = HEAP32[$1>>2]|0;
   $58 = (_sqlite3GetTempReg($57)|0);
   HEAP32[$r1>>2] = $58;
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = HEAP32[$r1>>2]|0;
   $62 = (_sqlite3ExprCodeTarget($59,$60,$61)|0);
   HEAP32[$r2>>2] = $62;
   $63 = HEAP32[$r2>>2]|0;
   $64 = HEAP32[$r1>>2]|0;
   $65 = ($63|0)==($64|0);
   if ($65) {
    $66 = HEAP32[$r1>>2]|0;
    $67 = HEAP32[$3>>2]|0;
    HEAP32[$67>>2] = $66;
    break;
   } else {
    $68 = HEAP32[$1>>2]|0;
    $69 = HEAP32[$r1>>2]|0;
    _sqlite3ReleaseTempReg($68,$69);
    $70 = HEAP32[$3>>2]|0;
    HEAP32[$70>>2] = 0;
    break;
   }
  }
 } while(0);
 $71 = HEAP32[$r2>>2]|0;
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _codeCompare($pParse,$pLeft,$pRight,$opcode,$in1,$in2,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 $opcode = $opcode|0;
 $in1 = $in1|0;
 $in2 = $in2|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, $p4 = 0, $p5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $7 = sp + 12|0;
 $p5 = sp + 8|0;
 $addr = sp + 4|0;
 $p4 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLeft;
 HEAP32[$2>>2] = $pRight;
 HEAP32[$3>>2] = $opcode;
 HEAP32[$4>>2] = $in1;
 HEAP32[$5>>2] = $in2;
 HEAP32[$6>>2] = $dest;
 HEAP32[$7>>2] = $jumpIfNull;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3BinaryCompareCollSeq($8,$9,$10)|0);
 HEAP32[$p4>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$7>>2]|0;
 $15 = (_binaryCompareP5($12,$13,$14)|0);
 $16 = $15&255;
 HEAP32[$p5>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$5>>2]|0;
 $22 = HEAP32[$6>>2]|0;
 $23 = HEAP32[$4>>2]|0;
 $24 = HEAP32[$p4>>2]|0;
 $25 = (_sqlite3VdbeAddOp4($19,$20,$21,$22,$23,$24,-4)|0);
 HEAP32[$addr>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 8|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$p5>>2]|0;
 $30 = $29&255;
 _sqlite3VdbeChangeP5($28,$30);
 $31 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _sqlite3VdbeAddOp1($p,$op,$p1) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeAddOp3($3,$4,$5,0,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeMakeLabel($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 116|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 HEAP32[$i>>2] = $6;
 $8 = HEAP32[$i>>2]|0;
 $9 = HEAP32[$i>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = $8 & $10;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 120|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$i>>2]|0;
  $19 = $18<<1;
  $20 = (($19) + 1)|0;
  $21 = $20<<2;
  $22 = (_sqlite3DbReallocOrFree($14,$17,$21,0)|0);
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 120|0);
  HEAP32[$24>>2] = $22;
 }
 $25 = HEAP32[$p>>2]|0;
 $26 = ((($25)) + 120|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $34 = HEAP32[$i>>2]|0;
  $35 = (-1 - ($34))|0;
  STACKTOP = sp;return ($35|0);
 }
 $29 = HEAP32[$i>>2]|0;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 120|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($29<<2)|0);
 HEAP32[$33>>2] = -1;
 $34 = HEAP32[$i>>2]|0;
 $35 = (-1 - ($34))|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3ExprCacheRemove($pParse,$iReg,$nReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iLast = 0, $p = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $iLast = sp + 8|0;
 $p = sp + 4|0;
 $r = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$2>>2] = $nReg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($3) + ($4))|0;
 $6 = (($5) - 1)|0;
 HEAP32[$iLast>>2] = $6;
 HEAP32[$i>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 124|0);
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(10);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$r>>2] = $13;
  $14 = HEAP32[$r>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ($14|0)>=($15|0);
  if ($16) {
   $17 = HEAP32[$r>>2]|0;
   $18 = HEAP32[$iLast>>2]|0;
   $19 = ($17|0)<=($18|0);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$p>>2]|0;
    _cacheEntryClear($20,$21);
    $22 = HEAP32[$p>>2]|0;
    $23 = ((($22)) + 12|0);
    HEAP32[$23>>2] = 0;
   }
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  $27 = ((($26)) + 20|0);
  HEAP32[$p>>2] = $27;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePush($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 108|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePop($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 108|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 124|0);
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = ($7|0)<(10);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$p>>2]|0;
   $14 = ((($13)) + 8|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$0>>2]|0;
   $17 = ((($16)) + 108|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($15|0)>($18|0);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$p>>2]|0;
    _cacheEntryClear($20,$21);
    $22 = HEAP32[$p>>2]|0;
    $23 = ((($22)) + 12|0);
    HEAP32[$23>>2] = 0;
   }
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  $27 = ((($26)) + 20|0);
  HEAP32[$p>>2] = $27;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeResolveLabel($v,$x) {
 $v = $v|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (-1 - ($5))|0;
 HEAP32[$j>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 120|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$j>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 120|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<2)|0);
  HEAP32[$18>>2] = $13;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 96|0);
 HEAP32[$24>>2] = $22;
 STACKTOP = sp;return;
}
function _sqlite3ExprIsConstant($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_exprIsConst($1,1,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ExprCollSeq($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $op = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $p = 0, $pColl = 0, $pNext = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $db = sp + 28|0;
 $pColl = sp + 24|0;
 $p = sp + 20|0;
 $op = sp + 16|0;
 $j = sp + 12|0;
 $zColl = sp + 8|0;
 $pNext = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$pColl>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$p>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$op>>2] = $9;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $12 & 512;
  $14 = ($13|0)!=(0);
  if ($14) {
   break;
  }
  $15 = HEAP32[$op>>2]|0;
  $16 = ($15|0)==(38);
  $17 = HEAP32[$op>>2]|0;
  $18 = ($17|0)==(156);
  $or$cond = $16 | $18;
  if ($or$cond) {
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$p>>2] = $21;
   continue;
  }
  $22 = HEAP32[$op>>2]|0;
  $23 = ($22|0)==(95);
  if ($23) {
   label = 9;
   break;
  }
  $24 = HEAP32[$op>>2]|0;
  $25 = ($24|0)==(157);
  if ($25) {
   $26 = HEAP32[$p>>2]|0;
   $27 = ((($26)) + 38|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = ($29|0)==(95);
   if ($30) {
    label = 9;
    break;
   }
  }
  $39 = HEAP32[$op>>2]|0;
  $40 = ($39|0)==(154);
  $41 = HEAP32[$op>>2]|0;
  $42 = ($41|0)==(152);
  $or$cond3 = $40 | $42;
  $43 = HEAP32[$op>>2]|0;
  $44 = ($43|0)==(157);
  $or$cond5 = $or$cond3 | $44;
  $45 = HEAP32[$op>>2]|0;
  $46 = ($45|0)==(62);
  $or$cond7 = $or$cond5 | $46;
  if ($or$cond7) {
   $47 = HEAP32[$p>>2]|0;
   $48 = ((($47)) + 44|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ($49|0)!=(0|0);
   if ($50) {
    label = 12;
    break;
   }
  }
  $72 = HEAP32[$p>>2]|0;
  $73 = ((($72)) + 4|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $74 & 256;
  $76 = ($75|0)!=(0);
  if (!($76)) {
   break;
  }
  $77 = HEAP32[$p>>2]|0;
  $78 = ((($77)) + 12|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0|0);
  if ($80) {
   $81 = HEAP32[$p>>2]|0;
   $82 = ((($81)) + 12|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = ((($83)) + 4|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $85 & 256;
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = HEAP32[$p>>2]|0;
    $89 = ((($88)) + 12|0);
    $90 = HEAP32[$89>>2]|0;
    HEAP32[$p>>2] = $90;
    continue;
   }
  }
  $91 = HEAP32[$p>>2]|0;
  $92 = ((($91)) + 16|0);
  $93 = HEAP32[$92>>2]|0;
  HEAP32[$pNext>>2] = $93;
  $94 = HEAP32[$p>>2]|0;
  $95 = ((($94)) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)!=(0|0);
  L20: do {
   if ($97) {
    $98 = HEAP32[$p>>2]|0;
    $99 = ((($98)) + 4|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $100 & 2048;
    $102 = ($101|0)!=(0);
    if (!($102)) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $103 = HEAP32[$i>>2]|0;
      $104 = HEAP32[$p>>2]|0;
      $105 = ((($104)) + 20|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = HEAP32[$106>>2]|0;
      $108 = ($103|0)<($107|0);
      if (!($108)) {
       break L20;
      }
      $109 = HEAP32[$i>>2]|0;
      $110 = HEAP32[$p>>2]|0;
      $111 = ((($110)) + 20|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = ((($112)) + 4|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = (($114) + (($109*20)|0)|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ((($116)) + 4|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = $118 & 256;
      $120 = ($119|0)!=(0);
      $121 = HEAP32[$i>>2]|0;
      if ($120) {
       break;
      }
      $129 = (($121) + 1)|0;
      HEAP32[$i>>2] = $129;
     }
     $122 = HEAP32[$p>>2]|0;
     $123 = ((($122)) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = ((($124)) + 4|0);
     $126 = HEAP32[$125>>2]|0;
     $127 = (($126) + (($121*20)|0)|0);
     $128 = HEAP32[$127>>2]|0;
     HEAP32[$pNext>>2] = $128;
    }
   }
  } while(0);
  $130 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $130;
 }
 if ((label|0) == 9) {
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$db>>2]|0;
  $33 = ((($32)) + 62|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3GetCollSeq($31,$34,0,$37)|0);
  HEAP32[$pColl>>2] = $38;
 }
 else if ((label|0) == 12) {
  $51 = HEAP32[$p>>2]|0;
  $52 = ((($51)) + 32|0);
  $53 = HEAP16[$52>>1]|0;
  $54 = $53 << 16 >> 16;
  HEAP32[$j>>2] = $54;
  $55 = HEAP32[$j>>2]|0;
  $56 = ($55|0)>=(0);
  if ($56) {
   $57 = HEAP32[$j>>2]|0;
   $58 = HEAP32[$p>>2]|0;
   $59 = ((($58)) + 44|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ((($60)) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + (($57*24)|0)|0);
   $64 = ((($63)) + 16|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$zColl>>2] = $65;
   $66 = HEAP32[$db>>2]|0;
   $67 = HEAP32[$db>>2]|0;
   $68 = ((($67)) + 62|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = HEAP32[$zColl>>2]|0;
   $71 = (_sqlite3FindCollSeq($66,$69,$70,0)|0);
   HEAP32[$pColl>>2] = $71;
  }
 }
 $131 = HEAP32[$0>>2]|0;
 $132 = HEAP32[$pColl>>2]|0;
 $133 = (_sqlite3CheckCollSeq($131,$132)|0);
 $134 = ($133|0)!=(0);
 if (!($134)) {
  $135 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 HEAP32[$pColl>>2] = 0;
 $135 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _sqlite3GetTempRange($pParse,$nReg) {
 $pParse = $pParse|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $nReg;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 60|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($8|0)<=($9|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 64|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11))|0;
  HEAP32[$13>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 60|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - ($16))|0;
  HEAP32[$18>>2] = $20;
  $30 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($30|0);
 } else {
  $21 = HEAP32[$0>>2]|0;
  $22 = ((($21)) + 76|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 76|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25))|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _sqlite3ExprCodeExprList($pParse,$pList,$target,$srcReg,$flags) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $target = $target|0;
 $srcReg = $srcReg|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $copyOp = 0, $i = 0, $inReg = 0, $j = 0, $n = 0, $pExpr = 0, $pItem = 0, $pOp = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 49|0;
 $pItem = sp + 28|0;
 $i = sp + 24|0;
 $j = sp + 20|0;
 $n = sp + 16|0;
 $copyOp = sp + 48|0;
 $v = sp + 12|0;
 $pExpr = sp + 8|0;
 $inReg = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $target;
 HEAP32[$3>>2] = $srcReg;
 HEAP8[$4>>0] = $flags;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 $9 = $8 ? 30 : 31;
 $10 = $9&255;
 HEAP8[$copyOp>>0] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 23|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 if (!($19)) {
  $20 = HEAP8[$4>>0]|0;
  $21 = $20&255;
  $22 = $21 & -3;
  $23 = $22&255;
  HEAP8[$4>>0] = $23;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pItem>>2] = $26;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$n>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$pItem>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pExpr>>2] = $31;
  $32 = HEAP8[$4>>0]|0;
  $33 = $32&255;
  $34 = $33 & 4;
  $35 = ($34|0)!=(0);
  if ($35) {
   $36 = HEAP32[$i>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + (($36*20)|0)|0);
   $41 = ((($40)) + 16|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = $42&65535;
   HEAP32[$j>>2] = $43;
   $44 = ($43|0)>(0);
   if ($44) {
    $45 = HEAP32[$v>>2]|0;
    $46 = HEAP8[$copyOp>>0]|0;
    $47 = $46&255;
    $48 = HEAP32[$j>>2]|0;
    $49 = HEAP32[$3>>2]|0;
    $50 = (($48) + ($49))|0;
    $51 = (($50) - 1)|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = HEAP32[$i>>2]|0;
    $54 = (($52) + ($53))|0;
    (_sqlite3VdbeAddOp2($45,$47,$51,$54)|0);
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  do {
   if ((label|0) == 8) {
    label = 0;
    $55 = HEAP8[$4>>0]|0;
    $56 = $55&255;
    $57 = $56 & 2;
    $58 = ($57|0)!=(0);
    if ($58) {
     $59 = HEAP32[$pExpr>>2]|0;
     $60 = (_sqlite3ExprIsConstant($59)|0);
     $61 = ($60|0)!=(0);
     if ($61) {
      $62 = HEAP32[$0>>2]|0;
      $63 = HEAP32[$pExpr>>2]|0;
      $64 = HEAP32[$2>>2]|0;
      $65 = HEAP32[$i>>2]|0;
      $66 = (($64) + ($65))|0;
      _sqlite3ExprCodeAtInit($62,$63,$66,0);
      break;
     }
    }
    $67 = HEAP32[$0>>2]|0;
    $68 = HEAP32[$pExpr>>2]|0;
    $69 = HEAP32[$2>>2]|0;
    $70 = HEAP32[$i>>2]|0;
    $71 = (($69) + ($70))|0;
    $72 = (_sqlite3ExprCodeTarget($67,$68,$71)|0);
    HEAP32[$inReg>>2] = $72;
    $73 = HEAP32[$inReg>>2]|0;
    $74 = HEAP32[$2>>2]|0;
    $75 = HEAP32[$i>>2]|0;
    $76 = (($74) + ($75))|0;
    $77 = ($73|0)!=($76|0);
    if ($77) {
     $78 = HEAP8[$copyOp>>0]|0;
     $79 = $78&255;
     $80 = ($79|0)==(30);
     if ($80) {
      $81 = HEAP32[$v>>2]|0;
      $82 = (_sqlite3VdbeGetOp($81,-1)|0);
      HEAP32[$pOp>>2] = $82;
      $83 = HEAP8[$82>>0]|0;
      $84 = $83&255;
      $85 = ($84|0)==(30);
      if ($85) {
       $86 = HEAP32[$pOp>>2]|0;
       $87 = ((($86)) + 4|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = HEAP32[$pOp>>2]|0;
       $90 = ((($89)) + 12|0);
       $91 = HEAP32[$90>>2]|0;
       $92 = (($88) + ($91))|0;
       $93 = (($92) + 1)|0;
       $94 = HEAP32[$inReg>>2]|0;
       $95 = ($93|0)==($94|0);
       if ($95) {
        $96 = HEAP32[$pOp>>2]|0;
        $97 = ((($96)) + 8|0);
        $98 = HEAP32[$97>>2]|0;
        $99 = HEAP32[$pOp>>2]|0;
        $100 = ((($99)) + 12|0);
        $101 = HEAP32[$100>>2]|0;
        $102 = (($98) + ($101))|0;
        $103 = (($102) + 1)|0;
        $104 = HEAP32[$2>>2]|0;
        $105 = HEAP32[$i>>2]|0;
        $106 = (($104) + ($105))|0;
        $107 = ($103|0)==($106|0);
        if ($107) {
         $108 = HEAP32[$pOp>>2]|0;
         $109 = ((($108)) + 12|0);
         $110 = HEAP32[$109>>2]|0;
         $111 = (($110) + 1)|0;
         HEAP32[$109>>2] = $111;
         break;
        }
       }
      }
     }
     $112 = HEAP32[$v>>2]|0;
     $113 = HEAP8[$copyOp>>0]|0;
     $114 = $113&255;
     $115 = HEAP32[$inReg>>2]|0;
     $116 = HEAP32[$2>>2]|0;
     $117 = HEAP32[$i>>2]|0;
     $118 = (($116) + ($117))|0;
     (_sqlite3VdbeAddOp2($112,$114,$115,$118)|0);
    }
   }
  } while(0);
  $119 = HEAP32[$i>>2]|0;
  $120 = (($119) + 1)|0;
  HEAP32[$i>>2] = $120;
  $121 = HEAP32[$pItem>>2]|0;
  $122 = ((($121)) + 20|0);
  HEAP32[$pItem>>2] = $122;
 }
 $123 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _sqlite3VtabOverloadFunction($db,$pDef,$nArg,$pExpr) {
 $db = $db|0;
 $pDef = $pDef|0;
 $nArg = $nArg|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $12 = 0, $13 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pArg = 0, $pMod = 0, $pNew = 0, $pTab = 0, $pVtab = 0, $rc = 0, $xSFunc = 0;
 var $z = 0, $zLowerName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $pTab = sp + 36|0;
 $pVtab = sp + 32|0;
 $pMod = sp + 28|0;
 $xSFunc = sp + 24|0;
 $pArg = sp + 20|0;
 $pNew = sp + 16|0;
 $rc = sp + 12|0;
 $zLowerName = sp + 8|0;
 $z = sp + 4|0;
 $5 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pDef;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $pExpr;
 HEAP32[$xSFunc>>2] = 0;
 HEAP32[$pArg>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $8;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $9 = HEAP32[$4>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(152);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $13;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $14 = HEAP32[$4>>2]|0;
 $15 = ((($14)) + 44|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pTab>>2] = $16;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $19;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $20 = HEAP32[$pTab>>2]|0;
 $21 = ((($20)) + 42|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 & 16;
 $25 = ($24|0)==(0);
 if ($25) {
  $26 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $26;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$pTab>>2]|0;
 $29 = (_sqlite3GetVTable($27,$28)|0);
 $30 = ((($29)) + 8|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pVtab>>2] = $31;
 $32 = HEAP32[$pVtab>>2]|0;
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$pMod>>2] = $33;
 $34 = HEAP32[$pMod>>2]|0;
 $35 = ((($34)) + 72|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)==(0|0);
 if ($37) {
  $38 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $38;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = ((($40)) + 20|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (_sqlite3DbStrDup($39,$42)|0);
 HEAP32[$zLowerName>>2] = $43;
 $44 = HEAP32[$zLowerName>>2]|0;
 $45 = ($44|0)!=(0|0);
 if ($45) {
  $46 = HEAP32[$zLowerName>>2]|0;
  HEAP32[$z>>2] = $46;
  while(1) {
   $47 = HEAP32[$z>>2]|0;
   $48 = HEAP8[$47>>0]|0;
   $49 = ($48<<24>>24)!=(0);
   if (!($49)) {
    break;
   }
   $50 = HEAP32[$z>>2]|0;
   $51 = HEAP8[$50>>0]|0;
   $52 = $51&255;
   $53 = (16974 + ($52)|0);
   $54 = HEAP8[$53>>0]|0;
   $55 = HEAP32[$z>>2]|0;
   HEAP8[$55>>0] = $54;
   $56 = HEAP32[$z>>2]|0;
   $57 = ((($56)) + 1|0);
   HEAP32[$z>>2] = $57;
  }
  $58 = HEAP32[$pMod>>2]|0;
  $59 = ((($58)) + 72|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$pVtab>>2]|0;
  $62 = HEAP32[$3>>2]|0;
  $63 = HEAP32[$zLowerName>>2]|0;
  HEAP32[$5>>2] = $xSFunc;
  $64 = HEAP32[$5>>2]|0;
  $65 = (FUNCTION_TABLE_iiiiii[$60 & 255]($61,$62,$63,$64,$pArg)|0);
  HEAP32[$rc>>2] = $65;
  $66 = HEAP32[$1>>2]|0;
  $67 = HEAP32[$zLowerName>>2]|0;
  _sqlite3DbFree($66,$67);
 }
 $68 = HEAP32[$rc>>2]|0;
 $69 = ($68|0)==(0);
 if ($69) {
  $70 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $70;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 $71 = HEAP32[$1>>2]|0;
 $72 = HEAP32[$2>>2]|0;
 $73 = ((($72)) + 20|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = (_sqlite3Strlen30($74)|0);
 $76 = (32 + ($75))|0;
 $77 = (($76) + 1)|0;
 $78 = (_sqlite3DbMallocZero($71,$77,0)|0);
 HEAP32[$pNew>>2] = $78;
 $79 = HEAP32[$pNew>>2]|0;
 $80 = ($79|0)==(0|0);
 if ($80) {
  $81 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $81;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 } else {
  $82 = HEAP32[$pNew>>2]|0;
  $83 = HEAP32[$2>>2]|0;
  ;HEAP32[$82>>2]=HEAP32[$83>>2]|0;HEAP32[$82+4>>2]=HEAP32[$83+4>>2]|0;HEAP32[$82+8>>2]=HEAP32[$83+8>>2]|0;HEAP32[$82+12>>2]=HEAP32[$83+12>>2]|0;HEAP32[$82+16>>2]=HEAP32[$83+16>>2]|0;HEAP32[$82+20>>2]=HEAP32[$83+20>>2]|0;HEAP32[$82+24>>2]=HEAP32[$83+24>>2]|0;HEAP32[$82+28>>2]=HEAP32[$83+28>>2]|0;
  $84 = HEAP32[$pNew>>2]|0;
  $85 = ((($84)) + 32|0);
  $86 = HEAP32[$pNew>>2]|0;
  $87 = ((($86)) + 20|0);
  HEAP32[$87>>2] = $85;
  $88 = HEAP32[$pNew>>2]|0;
  $89 = ((($88)) + 20|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = HEAP32[$2>>2]|0;
  $92 = ((($91)) + 20|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = ((($94)) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = (_sqlite3Strlen30($96)|0);
  $98 = (($97) + 1)|0;
  _memcpy(($90|0),($93|0),($98|0))|0;
  $99 = HEAP32[$xSFunc>>2]|0;
  $100 = HEAP32[$pNew>>2]|0;
  $101 = ((($100)) + 12|0);
  HEAP32[$101>>2] = $99;
  $102 = HEAP32[$pArg>>2]|0;
  $103 = HEAP32[$pNew>>2]|0;
  $104 = ((($103)) + 4|0);
  HEAP32[$104>>2] = $102;
  $105 = HEAP32[$pNew>>2]|0;
  $106 = ((($105)) + 2|0);
  $107 = HEAP16[$106>>1]|0;
  $108 = $107&65535;
  $109 = $108 | 16;
  $110 = $109&65535;
  HEAP16[$106>>1] = $110;
  $111 = HEAP32[$pNew>>2]|0;
  HEAP32[$0>>2] = $111;
  $112 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 return (0)|0;
}
function _sqlite3ReleaseTempRange($pParse,$iReg,$nReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$2>>2] = $nReg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheRemove($3,$4,$5);
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 60|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>($9|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 60|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 64|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _sqlite3CodeSubselect($pParse,$pExpr,$rHasNullFlag,$isRowid) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $rHasNullFlag = $rHasNullFlag|0;
 $isRowid = $isRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $affinity = 0, $cond = 0, $dest = 0, $dest1 = 0, $i = 0, $iValToIns = 0, $jmpIfDynamic = 0, $pE2 = 0, $pEList = 0, $pItem = 0, $pKeyInfo = 0;
 var $pLeft = 0, $pList = 0, $pSel = 0, $pSelect = 0, $r1 = 0, $r2 = 0, $r3 = 0, $rReg = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer = sp;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $jmpIfDynamic = sp + 120|0;
 $rReg = sp + 116|0;
 $v = sp + 112|0;
 $zMsg = sp + 108|0;
 $affinity = sp + 144|0;
 $addr = sp + 104|0;
 $pLeft = sp + 100|0;
 $pKeyInfo = sp + 96|0;
 $pSelect = sp + 92|0;
 $dest = sp + 72|0;
 $pEList = sp + 68|0;
 $i = sp + 64|0;
 $pList = sp + 60|0;
 $pItem = sp + 56|0;
 $r1 = sp + 52|0;
 $r2 = sp + 48|0;
 $r3 = sp + 44|0;
 $pE2 = sp + 40|0;
 $iValToIns = sp + 36|0;
 $pSel = sp + 32|0;
 $dest1 = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $rHasNullFlag;
 HEAP32[$4>>2] = $isRowid;
 HEAP32[$jmpIfDynamic>>2] = -1;
 HEAP32[$rReg>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3GetVdbe($5)|0);
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $247 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($247|0);
 }
 $9 = HEAP32[$1>>2]|0;
 _sqlite3ExprCachePush($9);
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $12 & 32;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3CodeOnce($15)|0);
  HEAP32[$jmpIfDynamic>>2] = $16;
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 453|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20|0)==(2);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$jmpIfDynamic>>2]|0;
  $25 = ($24|0)>=(0);
  $26 = $25 ? 17532 : 28589;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(75);
  $31 = $30 ? 28601 : 28606;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 472|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$vararg_buffer>>2] = $26;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $31;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $34;
  $35 = (_sqlite3MPrintf($23,28564,$vararg_buffer)|0);
  HEAP32[$zMsg>>2] = $35;
  $36 = HEAP32[$v>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 468|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$zMsg>>2]|0;
  (_sqlite3VdbeAddOp4($36,161,$39,0,0,$40,-1)|0);
 }
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $cond = ($43|0)==(75);
 $44 = HEAP32[$2>>2]|0;
 do {
  if ($cond) {
   $45 = ((($44)) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$pLeft>>2] = $46;
   HEAP32[$pKeyInfo>>2] = 0;
   $47 = HEAP32[$pLeft>>2]|0;
   $48 = (_sqlite3ExprAffinity($47)|0);
   HEAP8[$affinity>>0] = $48;
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 72|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (($51) + 1)|0;
   HEAP32[$50>>2] = $52;
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 28|0);
   HEAP32[$54>>2] = $51;
   $55 = HEAP32[$v>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   $57 = ((($56)) + 28|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$4>>2]|0;
   $60 = ($59|0)!=(0);
   $61 = $60 ^ 1;
   $62 = $61&1;
   $63 = (_sqlite3VdbeAddOp2($55,57,$58,$62)|0);
   HEAP32[$addr>>2] = $63;
   $64 = HEAP32[$4>>2]|0;
   $65 = ($64|0)!=(0);
   if ($65) {
    $69 = 0;
   } else {
    $66 = HEAP32[$1>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = (_sqlite3KeyInfoAlloc($67,1,1)|0);
    $69 = $68;
   }
   HEAP32[$pKeyInfo>>2] = $69;
   $70 = HEAP32[$2>>2]|0;
   $71 = ((($70)) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = $72 & 2048;
   $74 = ($73|0)!=(0);
   $75 = HEAP32[$2>>2]|0;
   $76 = ((($75)) + 20|0);
   do {
    if ($74) {
     $77 = HEAP32[$76>>2]|0;
     HEAP32[$pSelect>>2] = $77;
     $78 = HEAP32[$2>>2]|0;
     $79 = ((($78)) + 28|0);
     $80 = HEAP32[$79>>2]|0;
     _sqlite3SelectDestInit($dest,11,$80);
     $81 = HEAP8[$affinity>>0]|0;
     $82 = ((($dest)) + 1|0);
     HEAP8[$82>>0] = $81;
     $83 = HEAP32[$pSelect>>2]|0;
     $84 = ((($83)) + 8|0);
     HEAP32[$84>>2] = 0;
     $85 = HEAP32[$1>>2]|0;
     $86 = HEAP32[$pSelect>>2]|0;
     $87 = (_sqlite3Select($85,$86,$dest)|0);
     $88 = ($87|0)!=(0);
     if (!($88)) {
      $90 = HEAP32[$pSelect>>2]|0;
      $91 = HEAP32[$90>>2]|0;
      HEAP32[$pEList>>2] = $91;
      $92 = HEAP32[$1>>2]|0;
      $93 = HEAP32[$2>>2]|0;
      $94 = ((($93)) + 12|0);
      $95 = HEAP32[$94>>2]|0;
      $96 = HEAP32[$pEList>>2]|0;
      $97 = ((($96)) + 4|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = HEAP32[$98>>2]|0;
      $100 = (_sqlite3BinaryCompareCollSeq($92,$95,$99)|0);
      $101 = HEAP32[$pKeyInfo>>2]|0;
      $102 = ((($101)) + 20|0);
      HEAP32[$102>>2] = $100;
      break;
     }
     $89 = HEAP32[$pKeyInfo>>2]|0;
     _sqlite3KeyInfoUnref($89);
     HEAP32[$0>>2] = 0;
     $247 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($247|0);
    } else {
     $103 = HEAP32[$76>>2]|0;
     $104 = ($103|0)!=(0|0);
     if ($104) {
      $105 = HEAP32[$2>>2]|0;
      $106 = ((($105)) + 20|0);
      $107 = HEAP32[$106>>2]|0;
      HEAP32[$pList>>2] = $107;
      $108 = HEAP8[$affinity>>0]|0;
      $109 = ($108<<24>>24)!=(0);
      if (!($109)) {
       HEAP8[$affinity>>0] = 65;
      }
      $110 = HEAP32[$pKeyInfo>>2]|0;
      $111 = ($110|0)!=(0|0);
      if ($111) {
       $112 = HEAP32[$1>>2]|0;
       $113 = HEAP32[$2>>2]|0;
       $114 = ((($113)) + 12|0);
       $115 = HEAP32[$114>>2]|0;
       $116 = (_sqlite3ExprCollSeq($112,$115)|0);
       $117 = HEAP32[$pKeyInfo>>2]|0;
       $118 = ((($117)) + 20|0);
       HEAP32[$118>>2] = $116;
      }
      $119 = HEAP32[$1>>2]|0;
      $120 = (_sqlite3GetTempReg($119)|0);
      HEAP32[$r1>>2] = $120;
      $121 = HEAP32[$1>>2]|0;
      $122 = (_sqlite3GetTempReg($121)|0);
      HEAP32[$r2>>2] = $122;
      $123 = HEAP32[$4>>2]|0;
      $124 = ($123|0)!=(0);
      if ($124) {
       $125 = HEAP32[$v>>2]|0;
       $126 = HEAP32[$r2>>2]|0;
       (_sqlite3VdbeAddOp2($125,25,0,$126)|0);
      }
      $127 = HEAP32[$pList>>2]|0;
      $128 = HEAP32[$127>>2]|0;
      HEAP32[$i>>2] = $128;
      $129 = HEAP32[$pList>>2]|0;
      $130 = ((($129)) + 4|0);
      $131 = HEAP32[$130>>2]|0;
      HEAP32[$pItem>>2] = $131;
      while(1) {
       $132 = HEAP32[$i>>2]|0;
       $133 = ($132|0)>(0);
       if (!($133)) {
        break;
       }
       $134 = HEAP32[$pItem>>2]|0;
       $135 = HEAP32[$134>>2]|0;
       HEAP32[$pE2>>2] = $135;
       $136 = HEAP32[$jmpIfDynamic>>2]|0;
       $137 = ($136|0)>=(0);
       if ($137) {
        $138 = HEAP32[$pE2>>2]|0;
        $139 = (_sqlite3ExprIsConstant($138)|0);
        $140 = ($139|0)!=(0);
        if (!($140)) {
         $141 = HEAP32[$v>>2]|0;
         $142 = HEAP32[$jmpIfDynamic>>2]|0;
         (_sqlite3VdbeChangeToNoop($141,$142)|0);
         HEAP32[$jmpIfDynamic>>2] = -1;
        }
       }
       $143 = HEAP32[$4>>2]|0;
       $144 = ($143|0)!=(0);
       if ($144) {
        $145 = HEAP32[$pE2>>2]|0;
        $146 = (_sqlite3ExprIsInteger($145,$iValToIns)|0);
        $147 = ($146|0)!=(0);
        if ($147) {
         $148 = HEAP32[$v>>2]|0;
         $149 = HEAP32[$2>>2]|0;
         $150 = ((($149)) + 28|0);
         $151 = HEAP32[$150>>2]|0;
         $152 = HEAP32[$r2>>2]|0;
         $153 = HEAP32[$iValToIns>>2]|0;
         (_sqlite3VdbeAddOp3($148,84,$151,$152,$153)|0);
        } else {
         label = 29;
        }
       } else {
        label = 29;
       }
       do {
        if ((label|0) == 29) {
         label = 0;
         $154 = HEAP32[$1>>2]|0;
         $155 = HEAP32[$pE2>>2]|0;
         $156 = HEAP32[$r1>>2]|0;
         $157 = (_sqlite3ExprCodeTarget($154,$155,$156)|0);
         HEAP32[$r3>>2] = $157;
         $158 = HEAP32[$4>>2]|0;
         $159 = ($158|0)!=(0);
         $160 = HEAP32[$v>>2]|0;
         $161 = HEAP32[$r3>>2]|0;
         if ($159) {
          $162 = HEAP32[$v>>2]|0;
          $163 = (_sqlite3VdbeCurrentAddr($162)|0);
          $164 = (($163) + 2)|0;
          (_sqlite3VdbeAddOp2($160,38,$161,$164)|0);
          $165 = HEAP32[$v>>2]|0;
          $166 = HEAP32[$2>>2]|0;
          $167 = ((($166)) + 28|0);
          $168 = HEAP32[$167>>2]|0;
          $169 = HEAP32[$r2>>2]|0;
          $170 = HEAP32[$r3>>2]|0;
          (_sqlite3VdbeAddOp3($165,75,$168,$169,$170)|0);
          break;
         } else {
          $171 = HEAP32[$r2>>2]|0;
          (_sqlite3VdbeAddOp4($160,49,$161,1,$171,$affinity,1)|0);
          $172 = HEAP32[$1>>2]|0;
          $173 = HEAP32[$r3>>2]|0;
          _sqlite3ExprCacheAffinityChange($172,$173,1);
          $174 = HEAP32[$v>>2]|0;
          $175 = HEAP32[$2>>2]|0;
          $176 = ((($175)) + 28|0);
          $177 = HEAP32[$176>>2]|0;
          $178 = HEAP32[$r2>>2]|0;
          (_sqlite3VdbeAddOp2($174,110,$177,$178)|0);
          break;
         }
        }
       } while(0);
       $179 = HEAP32[$i>>2]|0;
       $180 = (($179) + -1)|0;
       HEAP32[$i>>2] = $180;
       $181 = HEAP32[$pItem>>2]|0;
       $182 = ((($181)) + 20|0);
       HEAP32[$pItem>>2] = $182;
      }
      $183 = HEAP32[$1>>2]|0;
      $184 = HEAP32[$r1>>2]|0;
      _sqlite3ReleaseTempReg($183,$184);
      $185 = HEAP32[$1>>2]|0;
      $186 = HEAP32[$r2>>2]|0;
      _sqlite3ReleaseTempReg($185,$186);
     }
    }
   } while(0);
   $187 = HEAP32[$pKeyInfo>>2]|0;
   $188 = ($187|0)!=(0|0);
   if ($188) {
    $189 = HEAP32[$v>>2]|0;
    $190 = HEAP32[$addr>>2]|0;
    $191 = HEAP32[$pKeyInfo>>2]|0;
    _sqlite3VdbeChangeP4($189,$190,$191,-6);
   }
  } else {
   $192 = ((($44)) + 20|0);
   $193 = HEAP32[$192>>2]|0;
   HEAP32[$pSel>>2] = $193;
   $194 = HEAP32[$1>>2]|0;
   $195 = ((($194)) + 76|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = (($196) + 1)|0;
   HEAP32[$195>>2] = $197;
   _sqlite3SelectDestInit($dest1,0,$197);
   $198 = HEAP32[$2>>2]|0;
   $199 = HEAP8[$198>>0]|0;
   $200 = $199&255;
   $201 = ($200|0)==(119);
   if ($201) {
    HEAP8[$dest1>>0] = 10;
    $202 = ((($dest1)) + 4|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = ((($dest1)) + 8|0);
    HEAP32[$204>>2] = $203;
    $205 = HEAP32[$v>>2]|0;
    $206 = ((($dest1)) + 4|0);
    $207 = HEAP32[$206>>2]|0;
    (_sqlite3VdbeAddOp2($205,25,0,$207)|0);
   } else {
    HEAP8[$dest1>>0] = 3;
    $208 = HEAP32[$v>>2]|0;
    $209 = ((($dest1)) + 4|0);
    $210 = HEAP32[$209>>2]|0;
    (_sqlite3VdbeAddOp2($208,22,0,$210)|0);
   }
   $211 = HEAP32[$1>>2]|0;
   $212 = HEAP32[$211>>2]|0;
   $213 = HEAP32[$pSel>>2]|0;
   $214 = ((($213)) + 60|0);
   $215 = HEAP32[$214>>2]|0;
   _sqlite3ExprDelete($212,$215);
   $216 = HEAP32[$1>>2]|0;
   $217 = (_sqlite3PExpr($216,132,0,0,(2536))|0);
   $218 = HEAP32[$pSel>>2]|0;
   $219 = ((($218)) + 60|0);
   HEAP32[$219>>2] = $217;
   $220 = HEAP32[$pSel>>2]|0;
   $221 = ((($220)) + 8|0);
   HEAP32[$221>>2] = 0;
   $222 = HEAP32[$pSel>>2]|0;
   $223 = ((($222)) + 6|0);
   $224 = HEAP16[$223>>1]|0;
   $225 = $224&65535;
   $226 = $225 & -513;
   $227 = $226&65535;
   HEAP16[$223>>1] = $227;
   $228 = HEAP32[$1>>2]|0;
   $229 = HEAP32[$pSel>>2]|0;
   $230 = (_sqlite3Select($228,$229,$dest1)|0);
   $231 = ($230|0)!=(0);
   if (!($231)) {
    $232 = ((($dest1)) + 4|0);
    $233 = HEAP32[$232>>2]|0;
    HEAP32[$rReg>>2] = $233;
    break;
   }
   HEAP32[$0>>2] = 0;
   $247 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($247|0);
  }
 } while(0);
 $234 = HEAP32[$3>>2]|0;
 $235 = ($234|0)!=(0);
 if ($235) {
  $236 = HEAP32[$v>>2]|0;
  $237 = HEAP32[$2>>2]|0;
  $238 = ((($237)) + 28|0);
  $239 = HEAP32[$238>>2]|0;
  $240 = HEAP32[$3>>2]|0;
  _sqlite3SetHasNullFlag($236,$239,$240);
 }
 $241 = HEAP32[$jmpIfDynamic>>2]|0;
 $242 = ($241|0)>=(0);
 if ($242) {
  $243 = HEAP32[$v>>2]|0;
  $244 = HEAP32[$jmpIfDynamic>>2]|0;
  _sqlite3VdbeJumpHere($243,$244);
 }
 $245 = HEAP32[$1>>2]|0;
 _sqlite3ExprCachePop($245);
 $246 = HEAP32[$rReg>>2]|0;
 HEAP32[$0>>2] = $246;
 $247 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($247|0);
}
function _sqlite3ExprCodeIN($pParse,$pExpr,$destIfFalse,$destIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $destIfFalse = $destIfFalse|0;
 $destIfNull = $destIfNull|0;
 var $$rRhsHasNull = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $addr1 = 0, $addr11 = 0, $affinity = 0, $eType = 0, $ii = 0, $labelOk = 0, $pColl = 0, $pList = 0, $r1 = 0, $r2 = 0, $rRhsHasNull = 0, $regCkNull = 0, $regToFree = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $rRhsHasNull = sp + 48|0;
 $affinity = sp + 68|0;
 $eType = sp + 44|0;
 $r1 = sp + 40|0;
 $v = sp + 36|0;
 $pList = sp + 32|0;
 $pColl = sp + 28|0;
 $labelOk = sp + 24|0;
 $r2 = sp + 20|0;
 $regToFree = sp + 16|0;
 $regCkNull = sp + 12|0;
 $ii = sp + 8|0;
 $addr1 = sp + 4|0;
 $addr11 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $destIfFalse;
 HEAP32[$3>>2] = $destIfNull;
 HEAP32[$rRhsHasNull>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = ($9|0)==($10|0);
 $$rRhsHasNull = $11 ? 0 : $rRhsHasNull;
 $12 = (_sqlite3FindInIndex($7,$8,3,$$rRhsHasNull)|0);
 HEAP32[$eType>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_comparisonAffinity($13)|0);
 HEAP8[$affinity>>0] = $14;
 $15 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePush($15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (_sqlite3GetTempReg($16)|0);
 HEAP32[$r1>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$r1>>2]|0;
 _sqlite3ExprCode($18,$21,$22);
 $23 = HEAP32[$eType>>2]|0;
 $24 = ($23|0)==(5);
 $25 = HEAP32[$1>>2]|0;
 if (!($24)) {
  $110 = ((($25)) + 12|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (_sqlite3ExprCanBeNull($111)|0);
  $113 = ($112|0)!=(0);
  do {
   if ($113) {
    $114 = HEAP32[$3>>2]|0;
    $115 = HEAP32[$2>>2]|0;
    $116 = ($114|0)==($115|0);
    $117 = HEAP32[$v>>2]|0;
    $118 = HEAP32[$r1>>2]|0;
    if ($116) {
     $119 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp2($117,76,$118,$119)|0);
     break;
    } else {
     $120 = (_sqlite3VdbeAddOp1($117,77,$118)|0);
     HEAP32[$addr1>>2] = $120;
     $121 = HEAP32[$v>>2]|0;
     $122 = HEAP32[$1>>2]|0;
     $123 = ((($122)) + 28|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeAddOp2($121,108,$124,$125)|0);
     $126 = HEAP32[$v>>2]|0;
     $127 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeGoto($126,$127)|0);
     $128 = HEAP32[$v>>2]|0;
     $129 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($128,$129);
     break;
    }
   }
  } while(0);
  $130 = HEAP32[$eType>>2]|0;
  $131 = ($130|0)==(1);
  $132 = HEAP32[$v>>2]|0;
  $133 = HEAP32[$r1>>2]|0;
  if ($131) {
   $134 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($132,38,$133,$134)|0);
   $135 = HEAP32[$v>>2]|0;
   $136 = HEAP32[$1>>2]|0;
   $137 = ((($136)) + 28|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = HEAP32[$2>>2]|0;
   $140 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp3($135,70,$138,$139,$140)|0);
   $158 = HEAP32[$0>>2]|0;
   $159 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($158,$159);
   $160 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($160);
   STACKTOP = sp;return;
  }
  (_sqlite3VdbeAddOp4($132,48,$133,1,0,$affinity,1)|0);
  $141 = HEAP32[$rRhsHasNull>>2]|0;
  $142 = ($141|0)==(0);
  $143 = HEAP32[$v>>2]|0;
  $144 = HEAP32[$1>>2]|0;
  $145 = ((($144)) + 28|0);
  $146 = HEAP32[$145>>2]|0;
  if ($142) {
   $147 = HEAP32[$2>>2]|0;
   $148 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp4Int($143,68,$146,$147,$148,1)|0);
   $158 = HEAP32[$0>>2]|0;
   $159 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($158,$159);
   $160 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($160);
   STACKTOP = sp;return;
  } else {
   $149 = HEAP32[$r1>>2]|0;
   $150 = (_sqlite3VdbeAddOp4Int($143,69,$146,0,$149,1)|0);
   HEAP32[$addr11>>2] = $150;
   $151 = HEAP32[$v>>2]|0;
   $152 = HEAP32[$rRhsHasNull>>2]|0;
   $153 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($151,76,$152,$153)|0);
   $154 = HEAP32[$v>>2]|0;
   $155 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeGoto($154,$155)|0);
   $156 = HEAP32[$v>>2]|0;
   $157 = HEAP32[$addr11>>2]|0;
   _sqlite3VdbeJumpHere($156,$157);
   $158 = HEAP32[$0>>2]|0;
   $159 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($158,$159);
   $160 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($160);
   STACKTOP = sp;return;
  }
 }
 $26 = ((($25)) + 20|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pList>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3ExprCollSeq($28,$31)|0);
 HEAP32[$pColl>>2] = $32;
 $33 = HEAP32[$v>>2]|0;
 $34 = (_sqlite3VdbeMakeLabel($33)|0);
 HEAP32[$labelOk>>2] = $34;
 HEAP32[$regCkNull>>2] = 0;
 $35 = HEAP32[$3>>2]|0;
 $36 = HEAP32[$2>>2]|0;
 $37 = ($35|0)!=($36|0);
 if ($37) {
  $38 = HEAP32[$0>>2]|0;
  $39 = (_sqlite3GetTempReg($38)|0);
  HEAP32[$regCkNull>>2] = $39;
  $40 = HEAP32[$v>>2]|0;
  $41 = HEAP32[$r1>>2]|0;
  $42 = HEAP32[$r1>>2]|0;
  $43 = HEAP32[$regCkNull>>2]|0;
  (_sqlite3VdbeAddOp3($40,85,$41,$42,$43)|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $44 = HEAP32[$ii>>2]|0;
  $45 = HEAP32[$pList>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = ($44|0)<($46|0);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$ii>>2]|0;
  $50 = HEAP32[$pList>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + (($49*20)|0)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3ExprCodeTemp($48,$54,$regToFree)|0);
  HEAP32[$r2>>2] = $55;
  $56 = HEAP32[$regCkNull>>2]|0;
  $57 = ($56|0)!=(0);
  if ($57) {
   $58 = HEAP32[$ii>>2]|0;
   $59 = HEAP32[$pList>>2]|0;
   $60 = ((($59)) + 4|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + (($58*20)|0)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (_sqlite3ExprCanBeNull($63)|0);
   $65 = ($64|0)!=(0);
   if ($65) {
    $66 = HEAP32[$v>>2]|0;
    $67 = HEAP32[$regCkNull>>2]|0;
    $68 = HEAP32[$r2>>2]|0;
    $69 = HEAP32[$regCkNull>>2]|0;
    (_sqlite3VdbeAddOp3($66,85,$67,$68,$69)|0);
   }
  }
  $70 = HEAP32[$ii>>2]|0;
  $71 = HEAP32[$pList>>2]|0;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) - 1)|0;
  $74 = ($70|0)<($73|0);
  if ($74) {
   label = 11;
  } else {
   $75 = HEAP32[$3>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ($75|0)!=($76|0);
   if ($77) {
    label = 11;
   } else {
    $85 = HEAP32[$v>>2]|0;
    $86 = HEAP32[$r1>>2]|0;
    $87 = HEAP32[$2>>2]|0;
    $88 = HEAP32[$r2>>2]|0;
    $89 = HEAP32[$pColl>>2]|0;
    (_sqlite3VdbeAddOp4($85,78,$86,$87,$88,$89,-4)|0);
    $90 = HEAP32[$v>>2]|0;
    $91 = HEAP8[$affinity>>0]|0;
    $92 = $91 << 24 >> 24;
    $93 = $92 | 16;
    $94 = $93&255;
    _sqlite3VdbeChangeP5($90,$94);
   }
  }
  if ((label|0) == 11) {
   label = 0;
   $78 = HEAP32[$v>>2]|0;
   $79 = HEAP32[$r1>>2]|0;
   $80 = HEAP32[$labelOk>>2]|0;
   $81 = HEAP32[$r2>>2]|0;
   $82 = HEAP32[$pColl>>2]|0;
   (_sqlite3VdbeAddOp4($78,79,$79,$80,$81,$82,-4)|0);
   $83 = HEAP32[$v>>2]|0;
   $84 = HEAP8[$affinity>>0]|0;
   _sqlite3VdbeChangeP5($83,$84);
  }
  $95 = HEAP32[$0>>2]|0;
  $96 = HEAP32[$regToFree>>2]|0;
  _sqlite3ReleaseTempReg($95,$96);
  $97 = HEAP32[$ii>>2]|0;
  $98 = (($97) + 1)|0;
  HEAP32[$ii>>2] = $98;
 }
 $99 = HEAP32[$regCkNull>>2]|0;
 $100 = ($99|0)!=(0);
 if ($100) {
  $101 = HEAP32[$v>>2]|0;
  $102 = HEAP32[$regCkNull>>2]|0;
  $103 = HEAP32[$3>>2]|0;
  (_sqlite3VdbeAddOp2($101,76,$102,$103)|0);
  $104 = HEAP32[$v>>2]|0;
  $105 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeGoto($104,$105)|0);
 }
 $106 = HEAP32[$v>>2]|0;
 $107 = HEAP32[$labelOk>>2]|0;
 _sqlite3VdbeResolveLabel($106,$107);
 $108 = HEAP32[$0>>2]|0;
 $109 = HEAP32[$regCkNull>>2]|0;
 _sqlite3ReleaseTempReg($108,$109);
 $158 = HEAP32[$0>>2]|0;
 $159 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($158,$159);
 $160 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePop($160);
 STACKTOP = sp;return;
}
function _sqlite3GetTempReg($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 19|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 76|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 1)|0;
  HEAP32[$8>>2] = $10;
  HEAP32[$0>>2] = $10;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $11 = ((($7)) + 19|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = (($12) + -1)<<24>>24;
  HEAP8[$11>>0] = $13;
  $14 = $13&255;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 28|0);
  $17 = (($16) + ($14<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3ReleaseTempReg($pParse,$iReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 19|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(8);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 124|0);
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)<(10);
  if (!($12)) {
   label = 8;
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   label = 6;
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 20|0);
  HEAP32[$p>>2] = $23;
 }
 if ((label|0) == 6) {
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 6|0);
  HEAP8[$19>>0] = 1;
  STACKTOP = sp;return;
 }
 else if ((label|0) == 8) {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 19|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = (($27) + 1)<<24>>24;
  HEAP8[$26>>0] = $28;
  $29 = $27&255;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 28|0);
  $32 = (($31) + ($29<<2)|0);
  HEAP32[$32>>2] = $24;
  STACKTOP = sp;return;
 }
}
function _exprToRegister($p,$iReg) {
 $p = $p|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iReg;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 38|0);
 HEAP8[$5>>0] = $3;
 $6 = HEAP32[$0>>2]|0;
 HEAP8[$6>>0] = -99;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $12 & -4097;
 HEAP32[$11>>2] = $13;
 STACKTOP = sp;return;
}
function _sqlite3ExprIfFalse($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d2 = 0, $destIfNull = 0, $op = 0, $or$cond = 0, $r1 = 0;
 var $r2 = 0, $regFree1 = 0, $regFree2 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $v = sp + 28|0;
 $op = sp + 24|0;
 $regFree1 = sp + 20|0;
 $regFree2 = sp + 16|0;
 $r1 = sp + 12|0;
 $r2 = sp + 8|0;
 $d2 = sp + 4|0;
 $destIfNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 HEAP32[$op>>2] = 0;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = (($13) + 0)|0;
 $15 = $14 ^ 1;
 $16 = (($15) - 0)|0;
 HEAP32[$op>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 L4: do {
  switch ($19|0) {
  case 72:  {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($20,$23,$24,$25);
   $26 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($26);
   $27 = HEAP32[$0>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 16|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($27,$30,$31,$32);
   $33 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($33);
   break;
  }
  case 71:  {
   $34 = HEAP32[$v>>2]|0;
   $35 = (_sqlite3VdbeMakeLabel($34)|0);
   HEAP32[$d2>>2] = $35;
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$d2>>2]|0;
   $41 = HEAP32[$3>>2]|0;
   $42 = $41 ^ 16;
   _sqlite3ExprIfTrue($36,$39,$40,$42);
   $43 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($43);
   $44 = HEAP32[$0>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 16|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($44,$47,$48,$49);
   $50 = HEAP32[$v>>2]|0;
   $51 = HEAP32[$d2>>2]|0;
   _sqlite3VdbeResolveLabel($50,$51);
   $52 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($52);
   break;
  }
  case 19:  {
   $53 = HEAP32[$0>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($53,$56,$57,$58);
   break;
  }
  case 79: case 78: case 83: case 80: case 81: case 82:  {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = ((($60)) + 12|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (_sqlite3ExprCodeTemp($59,$62,$regFree1)|0);
   HEAP32[$r1>>2] = $63;
   $64 = HEAP32[$0>>2]|0;
   $65 = HEAP32[$1>>2]|0;
   $66 = ((($65)) + 16|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (_sqlite3ExprCodeTemp($64,$67,$regFree2)|0);
   HEAP32[$r2>>2] = $68;
   $69 = HEAP32[$0>>2]|0;
   $70 = HEAP32[$1>>2]|0;
   $71 = ((($70)) + 12|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$1>>2]|0;
   $74 = ((($73)) + 16|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$op>>2]|0;
   $77 = HEAP32[$r1>>2]|0;
   $78 = HEAP32[$r2>>2]|0;
   $79 = HEAP32[$2>>2]|0;
   $80 = HEAP32[$3>>2]|0;
   (_codeCompare($69,$72,$75,$76,$77,$78,$79,$80)|0);
   break;
  }
  case 148: case 73:  {
   $81 = HEAP32[$0>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = ((($82)) + 12|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (_sqlite3ExprCodeTemp($81,$84,$regFree1)|0);
   HEAP32[$r1>>2] = $85;
   $86 = HEAP32[$0>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = ((($87)) + 16|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (_sqlite3ExprCodeTemp($86,$89,$regFree2)|0);
   HEAP32[$r2>>2] = $90;
   $91 = HEAP32[$1>>2]|0;
   $92 = HEAP8[$91>>0]|0;
   $93 = $92&255;
   $94 = ($93|0)==(73);
   $95 = $94 ? 78 : 79;
   HEAP32[$op>>2] = $95;
   $96 = HEAP32[$0>>2]|0;
   $97 = HEAP32[$1>>2]|0;
   $98 = ((($97)) + 12|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$1>>2]|0;
   $101 = ((($100)) + 16|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = HEAP32[$op>>2]|0;
   $104 = HEAP32[$r1>>2]|0;
   $105 = HEAP32[$r2>>2]|0;
   $106 = HEAP32[$2>>2]|0;
   (_codeCompare($96,$99,$102,$103,$104,$105,$106,128)|0);
   break;
  }
  case 77: case 76:  {
   $107 = HEAP32[$0>>2]|0;
   $108 = HEAP32[$1>>2]|0;
   $109 = ((($108)) + 12|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = (_sqlite3ExprCodeTemp($107,$110,$regFree1)|0);
   HEAP32[$r1>>2] = $111;
   $112 = HEAP32[$v>>2]|0;
   $113 = HEAP32[$op>>2]|0;
   $114 = HEAP32[$r1>>2]|0;
   $115 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($112,$113,$114,$115)|0);
   break;
  }
  case 74:  {
   $116 = HEAP32[$0>>2]|0;
   $117 = HEAP32[$1>>2]|0;
   $118 = HEAP32[$2>>2]|0;
   $119 = HEAP32[$3>>2]|0;
   _exprCodeBetween($116,$117,$118,0,$119);
   break;
  }
  case 75:  {
   $120 = HEAP32[$3>>2]|0;
   $121 = ($120|0)!=(0);
   if ($121) {
    $122 = HEAP32[$0>>2]|0;
    $123 = HEAP32[$1>>2]|0;
    $124 = HEAP32[$2>>2]|0;
    $125 = HEAP32[$2>>2]|0;
    _sqlite3ExprCodeIN($122,$123,$124,$125);
    break L4;
   } else {
    $126 = HEAP32[$v>>2]|0;
    $127 = (_sqlite3VdbeMakeLabel($126)|0);
    HEAP32[$destIfNull>>2] = $127;
    $128 = HEAP32[$0>>2]|0;
    $129 = HEAP32[$1>>2]|0;
    $130 = HEAP32[$2>>2]|0;
    $131 = HEAP32[$destIfNull>>2]|0;
    _sqlite3ExprCodeIN($128,$129,$130,$131);
    $132 = HEAP32[$v>>2]|0;
    $133 = HEAP32[$destIfNull>>2]|0;
    _sqlite3VdbeResolveLabel($132,$133);
    break L4;
   }
   break;
  }
  default: {
   $134 = HEAP32[$1>>2]|0;
   $135 = (_exprAlwaysFalse($134)|0);
   $136 = ($135|0)!=(0);
   if ($136) {
    $137 = HEAP32[$v>>2]|0;
    $138 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeGoto($137,$138)|0);
    break L4;
   }
   $139 = HEAP32[$1>>2]|0;
   $140 = (_exprAlwaysTrue($139)|0);
   $141 = ($140|0)!=(0);
   if (!($141)) {
    $142 = HEAP32[$0>>2]|0;
    $143 = HEAP32[$1>>2]|0;
    $144 = (_sqlite3ExprCodeTemp($142,$143,$regFree1)|0);
    HEAP32[$r1>>2] = $144;
    $145 = HEAP32[$v>>2]|0;
    $146 = HEAP32[$r1>>2]|0;
    $147 = HEAP32[$2>>2]|0;
    $148 = HEAP32[$3>>2]|0;
    $149 = ($148|0)!=(0);
    $150 = $149&1;
    (_sqlite3VdbeAddOp3($145,46,$146,$147,$150)|0);
   }
  }
  }
 } while(0);
 $151 = HEAP32[$0>>2]|0;
 $152 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($151,$152);
 $153 = HEAP32[$0>>2]|0;
 $154 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($153,$154);
 STACKTOP = sp;return;
}
function _sqlite3MayAbort($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 416|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ((($5)) + 416|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $7;
 } else {
  $8 = $5;
 }
 HEAP32[$pToplevel>>2] = $8;
 $9 = HEAP32[$pToplevel>>2]|0;
 $10 = ((($9)) + 21|0);
 HEAP8[$10>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3HaltConstraint($pParse,$errCode,$onError,$p4,$p4type,$p5Errmsg) {
 $pParse = $pParse|0;
 $errCode = $errCode|0;
 $onError = $onError|0;
 $p4 = $p4|0;
 $p4type = $p4type|0;
 $p5Errmsg = $p5Errmsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 21|0;
 $5 = sp + 20|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $errCode;
 HEAP32[$2>>2] = $onError;
 HEAP32[$3>>2] = $p4;
 HEAP8[$4>>0] = $p4type;
 HEAP8[$5>>0] = $p5Errmsg;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3GetVdbe($6)|0);
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _sqlite3MayAbort($10);
 }
 $11 = HEAP32[$v>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP8[$4>>0]|0;
 $16 = $15 << 24 >> 24;
 (_sqlite3VdbeAddOp4($11,21,$12,$13,0,$14,$16)|0);
 $17 = HEAP32[$v>>2]|0;
 $18 = HEAP8[$5>>0]|0;
 _sqlite3VdbeChangeP5($17,$18);
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePinRegister($pParse,$iReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$i>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 124|0);
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)<(10);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($8|0)==($9|0);
  if ($10) {
   $11 = HEAP32[$p>>2]|0;
   $12 = ((($11)) + 6|0);
   HEAP8[$12>>0] = 0;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 20|0);
  HEAP32[$p>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeGetColumnOfTable($v,$pTab,$iTabCur,$iCol,$regOut) {
 $v = $v|0;
 $pTab = $pTab|0;
 $iTabCur = $iTabCur|0;
 $iCol = $iCol|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $op = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $regOut;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  label = 3;
 } else {
  $7 = HEAP32[$3>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 32|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = ($7|0)==($11|0);
  if ($12) {
   label = 3;
  } else {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 42|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 16;
   $21 = ($20|0)!=(0);
   $22 = $21 ? 153 : 47;
   HEAP32[$op>>2] = $22;
   $23 = HEAP32[$3>>2]|0;
   HEAP32[$x>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 42|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = $27 & 32;
   $29 = ($28|0)==(0);
   if (!($29)) {
    $30 = HEAP32[$1>>2]|0;
    $31 = (_sqlite3PrimaryKeyIndex($30)|0);
    $32 = HEAP32[$3>>2]|0;
    $33 = $32&65535;
    $34 = (_sqlite3ColumnOfIndex($31,$33)|0);
    $35 = $34 << 16 >> 16;
    HEAP32[$x>>2] = $35;
   }
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$op>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$x>>2]|0;
   $40 = HEAP32[$4>>2]|0;
   (_sqlite3VdbeAddOp3($36,$37,$38,$39,$40)|0);
  }
 }
 if ((label|0) == 3) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  (_sqlite3VdbeAddOp2($13,103,$14,$15)|0);
 }
 $41 = HEAP32[$3>>2]|0;
 $42 = ($41|0)>=(0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$4>>2]|0;
 _sqlite3ColumnDefault($43,$44,$45,$46);
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheStore($pParse,$iTab,$iCol,$iReg) {
 $pParse = $pParse|0;
 $iTab = $iTab|0;
 $iCol = $iCol|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $i = 0, $idxLru = 0, $minLru = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $minLru = sp + 8|0;
 $idxLru = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$3>>2] = $iReg;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 60|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 2;
 $10 = ($9|0)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 124|0);
 HEAP32[$p>>2] = $12;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = ($13|0)<(10);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(0);
  if ($18) {
   label = 5;
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  $44 = ((($43)) + 20|0);
  HEAP32[$p>>2] = $44;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 108|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 8|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$p>>2]|0;
  HEAP32[$25>>2] = $24;
  $26 = HEAP32[$2>>2]|0;
  $27 = $26&65535;
  $28 = HEAP32[$p>>2]|0;
  $29 = ((($28)) + 4|0);
  HEAP16[$29>>1] = $27;
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$p>>2]|0;
  $32 = ((($31)) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$p>>2]|0;
  $34 = ((($33)) + 6|0);
  HEAP8[$34>>0] = 0;
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 112|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$36>>2] = $38;
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 16|0);
  HEAP32[$40>>2] = $37;
  STACKTOP = sp;return;
 }
 HEAP32[$minLru>>2] = 2147483647;
 HEAP32[$idxLru>>2] = -1;
 HEAP32[$i>>2] = 0;
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 124|0);
 HEAP32[$p>>2] = $46;
 while(1) {
  $47 = HEAP32[$i>>2]|0;
  $48 = ($47|0)<(10);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 16|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$minLru>>2]|0;
  $53 = ($51|0)<($52|0);
  if ($53) {
   $54 = HEAP32[$i>>2]|0;
   HEAP32[$idxLru>>2] = $54;
   $55 = HEAP32[$p>>2]|0;
   $56 = ((($55)) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$minLru>>2] = $57;
  }
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
  $60 = HEAP32[$p>>2]|0;
  $61 = ((($60)) + 20|0);
  HEAP32[$p>>2] = $61;
 }
 $62 = HEAP32[$idxLru>>2]|0;
 $63 = ($62|0)>=(0);
 if (!($63)) {
  STACKTOP = sp;return;
 }
 $64 = HEAP32[$idxLru>>2]|0;
 $65 = HEAP32[$0>>2]|0;
 $66 = ((($65)) + 124|0);
 $67 = (($66) + (($64*20)|0)|0);
 HEAP32[$p>>2] = $67;
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 108|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$p>>2]|0;
 $72 = ((($71)) + 8|0);
 HEAP32[$72>>2] = $70;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$p>>2]|0;
 HEAP32[$74>>2] = $73;
 $75 = HEAP32[$2>>2]|0;
 $76 = $75&65535;
 $77 = HEAP32[$p>>2]|0;
 $78 = ((($77)) + 4|0);
 HEAP16[$78>>1] = $76;
 $79 = HEAP32[$3>>2]|0;
 $80 = HEAP32[$p>>2]|0;
 $81 = ((($80)) + 12|0);
 HEAP32[$81>>2] = $79;
 $82 = HEAP32[$p>>2]|0;
 $83 = ((($82)) + 6|0);
 HEAP8[$83>>0] = 0;
 $84 = HEAP32[$0>>2]|0;
 $85 = ((($84)) + 112|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = (($86) + 1)|0;
 HEAP32[$85>>2] = $87;
 $88 = HEAP32[$p>>2]|0;
 $89 = ((($88)) + 16|0);
 HEAP32[$89>>2] = $86;
 STACKTOP = sp;return;
}
function _sqlite3ColumnOfIndex($pIdx,$iCol) {
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 10|0;
 $1 = sp + 4|0;
 $2 = sp + 8|0;
 $i = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP16[$2>>1] = $iCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 52|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP16[$2>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<1)|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = ($10|0)==($17|0);
  $19 = HEAP32[$i>>2]|0;
  if ($18) {
   label = 4;
   break;
  }
  $21 = (($19) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 if ((label|0) == 4) {
  $20 = $19&65535;
  HEAP16[$0>>1] = $20;
  $22 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($22|0);
 }
 else if ((label|0) == 6) {
  HEAP16[$0>>1] = -1;
  $22 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _sqlite3ColumnDefault($v,$pTab,$i,$iReg) {
 $v = $v|0;
 $pTab = $pTab|0;
 $i = $i|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $enc = 0, $pCol = 0, $pValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pValue = sp + 4|0;
 $enc = sp + 24|0;
 $pCol = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $i;
 HEAP32[$3>>2] = $iReg;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 HEAP32[$pValue>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3VdbeDb($8)|0);
 $10 = ((($9)) + 62|0);
 $11 = HEAP8[$10>>0]|0;
 HEAP8[$enc>>0] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + (($12*24)|0)|0);
 HEAP32[$pCol>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (_sqlite3VdbeDb($17)|0);
 $19 = HEAP32[$pCol>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP8[$enc>>0]|0;
 $23 = HEAP32[$pCol>>2]|0;
 $24 = ((($23)) + 21|0);
 $25 = HEAP8[$24>>0]|0;
 (_sqlite3ValueFromExpr($18,$21,$22,$25,$pValue)|0);
 $26 = HEAP32[$pValue>>2]|0;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  $29 = HEAP32[$pValue>>2]|0;
  _sqlite3VdbeChangeP4($28,-1,$29,-8);
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + (($30*24)|0)|0);
 $35 = ((($34)) + 21|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36 << 24 >> 24;
 $38 = ($37|0)==(69);
 if (!($38)) {
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp1($39,39,$40)|0);
 STACKTOP = sp;return;
}
function _sqlite3ValueFromExpr($db,$pExpr,$enc,$affinity,$ppVal) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $enc = $enc|0;
 $affinity = $affinity|0;
 $ppVal = $ppVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 13|0;
 $3 = sp + 12|0;
 $4 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $enc;
 HEAP8[$3>>0] = $affinity;
 HEAP32[$4>>2] = $ppVal;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP8[$2>>0]|0;
 $8 = HEAP8[$3>>0]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_valueFromExpr($5,$6,$7,$8,$9,0)|0);
 STACKTOP = sp;return ($10|0);
}
function _valueFromExpr($db,$pExpr,$enc,$affinity,$ppVal,$pCtx) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $enc = $enc|0;
 $affinity = $affinity|0;
 $ppVal = $ppVal|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0.0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aff = 0, $nVal = 0, $negInt = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pVal = 0, $rc = 0, $vararg_buffer = 0, $vararg_ptr8 = 0;
 var $zNeg = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 58|0;
 $4 = sp + 57|0;
 $5 = sp + 40|0;
 $6 = sp + 36|0;
 $op = sp + 32|0;
 $zVal = sp + 28|0;
 $pVal = sp + 24|0;
 $negInt = sp + 20|0;
 $zNeg = sp + 16|0;
 $rc = sp + 12|0;
 $aff = sp + 56|0;
 $nVal = sp + 8|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP8[$3>>0] = $enc;
 HEAP8[$4>>0] = $affinity;
 HEAP32[$5>>2] = $ppVal;
 HEAP32[$6>>2] = $pCtx;
 HEAP32[$zVal>>2] = 0;
 HEAP32[$pVal>>2] = 0;
 HEAP32[$negInt>>2] = 1;
 HEAP32[$zNeg>>2] = 17532;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$5>>2]|0;
  HEAP32[$9>>2] = 0;
  HEAP32[$0>>2] = 0;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 while(1) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  HEAP32[$op>>2] = $12;
  $13 = ($12|0)==(156);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$2>>2] = $16;
 }
 $17 = HEAP32[$op>>2]|0;
 $18 = ($17|0)==(157);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 38|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  HEAP32[$op>>2] = $22;
 }
 $23 = HEAP32[$op>>2]|0;
 $24 = ($23|0)==(38);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (_sqlite3AffinityType($27,0)|0);
  HEAP8[$aff>>0] = $28;
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP8[$3>>0]|0;
  $34 = HEAP8[$aff>>0]|0;
  $35 = HEAP32[$5>>2]|0;
  $36 = HEAP32[$6>>2]|0;
  $37 = (_valueFromExpr($29,$32,$33,$34,$35,$36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$5>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$5>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP8[$aff>>0]|0;
   _sqlite3VdbeMemCast($42,$43,1);
   $44 = HEAP32[$5>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP8[$4>>0]|0;
   _sqlite3ValueApplyAffinity($45,$46,1);
  }
  $47 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $47;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 $48 = HEAP32[$op>>2]|0;
 $49 = ($48|0)==(155);
 do {
  if ($49) {
   $50 = HEAP32[$2>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = ($54|0)==(132);
   if (!($55)) {
    $56 = HEAP32[$2>>2]|0;
    $57 = ((($56)) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = ($60|0)==(133);
    if (!($61)) {
     break;
    }
   }
   $62 = HEAP32[$2>>2]|0;
   $63 = ((($62)) + 12|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$2>>2] = $64;
   $65 = HEAP32[$2>>2]|0;
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   HEAP32[$op>>2] = $67;
   HEAP32[$negInt>>2] = -1;
   HEAP32[$zNeg>>2] = 20731;
  }
 } while(0);
 $68 = HEAP32[$op>>2]|0;
 $69 = ($68|0)==(97);
 $70 = HEAP32[$op>>2]|0;
 $71 = ($70|0)==(133);
 $or$cond = $69 | $71;
 $72 = HEAP32[$op>>2]|0;
 $73 = ($72|0)==(132);
 $or$cond3 = $or$cond | $73;
 do {
  if ($or$cond3) {
   $74 = HEAP32[$1>>2]|0;
   $75 = HEAP32[$6>>2]|0;
   $76 = (_valueNew($74,$75)|0);
   HEAP32[$pVal>>2] = $76;
   $77 = HEAP32[$pVal>>2]|0;
   $78 = ($77|0)==(0|0);
   if ($78) {
    label = 43;
   } else {
    $79 = HEAP32[$2>>2]|0;
    $80 = ((($79)) + 4|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = $81 & 1024;
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$pVal>>2]|0;
     $85 = HEAP32[$2>>2]|0;
     $86 = ((($85)) + 8|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = ($87|0)<(0);
     $89 = $88 << 31 >> 31;
     $90 = HEAP32[$negInt>>2]|0;
     $91 = ($90|0)<(0);
     $92 = $91 << 31 >> 31;
     $93 = (___muldi3(($87|0),($89|0),($90|0),($92|0))|0);
     $94 = tempRet0;
     _sqlite3VdbeMemSetInt64($84,$93,$94);
    } else {
     $95 = HEAP32[$1>>2]|0;
     $96 = HEAP32[$zNeg>>2]|0;
     $97 = HEAP32[$2>>2]|0;
     $98 = ((($97)) + 8|0);
     $99 = HEAP32[$98>>2]|0;
     HEAP32[$vararg_buffer>>2] = $96;
     $vararg_ptr8 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr8>>2] = $99;
     $100 = (_sqlite3MPrintf($95,28532,$vararg_buffer)|0);
     HEAP32[$zVal>>2] = $100;
     $101 = HEAP32[$zVal>>2]|0;
     $102 = ($101|0)==(0|0);
     if ($102) {
      label = 43;
      break;
     }
     $103 = HEAP32[$pVal>>2]|0;
     $104 = HEAP32[$zVal>>2]|0;
     _sqlite3ValueSetStr($103,-1,$104,1,169);
    }
    $105 = HEAP32[$op>>2]|0;
    $106 = ($105|0)==(132);
    $107 = HEAP32[$op>>2]|0;
    $108 = ($107|0)==(133);
    $or$cond5 = $106 | $108;
    if ($or$cond5) {
     $109 = HEAP8[$4>>0]|0;
     $110 = $109&255;
     $111 = ($110|0)==(65);
     if ($111) {
      $112 = HEAP32[$pVal>>2]|0;
      _sqlite3ValueApplyAffinity($112,67,1);
     } else {
      label = 24;
     }
    } else {
     label = 24;
    }
    if ((label|0) == 24) {
     $113 = HEAP32[$pVal>>2]|0;
     $114 = HEAP8[$4>>0]|0;
     _sqlite3ValueApplyAffinity($113,$114,1);
    }
    $115 = HEAP32[$pVal>>2]|0;
    $116 = ((($115)) + 8|0);
    $117 = HEAP16[$116>>1]|0;
    $118 = $117&65535;
    $119 = $118 & 12;
    $120 = ($119|0)!=(0);
    if ($120) {
     $121 = HEAP32[$pVal>>2]|0;
     $122 = ((($121)) + 8|0);
     $123 = HEAP16[$122>>1]|0;
     $124 = $123&65535;
     $125 = $124 & -3;
     $126 = $125&65535;
     HEAP16[$122>>1] = $126;
    }
    $127 = HEAP8[$3>>0]|0;
    $128 = $127&255;
    $129 = ($128|0)!=(1);
    if ($129) {
     $130 = HEAP32[$pVal>>2]|0;
     $131 = HEAP8[$3>>0]|0;
     $132 = $131&255;
     $133 = (_sqlite3VdbeChangeEncoding($130,$132)|0);
     HEAP32[$rc>>2] = $133;
     label = 42;
    } else {
     label = 42;
    }
   }
  } else {
   $134 = HEAP32[$op>>2]|0;
   $135 = ($134|0)==(155);
   if (!($135)) {
    $192 = HEAP32[$op>>2]|0;
    $193 = ($192|0)==(101);
    if ($193) {
     $194 = HEAP32[$1>>2]|0;
     $195 = HEAP32[$6>>2]|0;
     $196 = (_valueNew($194,$195)|0);
     HEAP32[$pVal>>2] = $196;
     $197 = HEAP32[$pVal>>2]|0;
     $198 = ($197|0)==(0|0);
     if ($198) {
      label = 43;
      break;
     } else {
      label = 42;
      break;
     }
    }
    $199 = HEAP32[$op>>2]|0;
    $200 = ($199|0)==(134);
    if (!($200)) {
     label = 42;
     break;
    }
    $201 = HEAP32[$1>>2]|0;
    $202 = HEAP32[$6>>2]|0;
    $203 = (_valueNew($201,$202)|0);
    HEAP32[$pVal>>2] = $203;
    $204 = HEAP32[$pVal>>2]|0;
    $205 = ($204|0)!=(0|0);
    if (!($205)) {
     label = 43;
     break;
    }
    $206 = HEAP32[$2>>2]|0;
    $207 = ((($206)) + 8|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = ((($208)) + 2|0);
    HEAP32[$zVal>>2] = $209;
    $210 = HEAP32[$zVal>>2]|0;
    $211 = (_sqlite3Strlen30($210)|0);
    $212 = (($211) - 1)|0;
    HEAP32[$nVal>>2] = $212;
    $213 = HEAP32[$pVal>>2]|0;
    $214 = HEAP32[$1>>2]|0;
    $215 = HEAP32[$zVal>>2]|0;
    $216 = HEAP32[$nVal>>2]|0;
    $217 = (_sqlite3HexToBlob($214,$215,$216)|0);
    $218 = HEAP32[$nVal>>2]|0;
    $219 = (($218|0) / 2)&-1;
    (_sqlite3VdbeMemSetStr($213,$217,$219,0,169)|0);
    label = 42;
    break;
   }
   $136 = HEAP32[$1>>2]|0;
   $137 = HEAP32[$2>>2]|0;
   $138 = ((($137)) + 12|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = HEAP8[$3>>0]|0;
   $141 = HEAP8[$4>>0]|0;
   $142 = (_sqlite3ValueFromExpr($136,$139,$140,$141,$pVal)|0);
   $143 = (0)==($142|0);
   $144 = HEAP32[$pVal>>2]|0;
   $145 = ($144|0)!=(0|0);
   $or$cond7 = $143 & $145;
   if ($or$cond7) {
    $146 = HEAP32[$pVal>>2]|0;
    (_sqlite3VdbeMemNumerify($146)|0);
    $147 = HEAP32[$pVal>>2]|0;
    $148 = ((($147)) + 8|0);
    $149 = HEAP16[$148>>1]|0;
    $150 = $149&65535;
    $151 = $150 & 8;
    $152 = ($151|0)!=(0);
    $153 = HEAP32[$pVal>>2]|0;
    do {
     if ($152) {
      $154 = +HEAPF64[$153>>3];
      $155 = -$154;
      $156 = HEAP32[$pVal>>2]|0;
      HEAPF64[$156>>3] = $155;
     } else {
      $157 = $153;
      $158 = $157;
      $159 = HEAP32[$158>>2]|0;
      $160 = (($157) + 4)|0;
      $161 = $160;
      $162 = HEAP32[$161>>2]|0;
      $163 = ($159|0)==(0);
      $164 = ($162|0)==(-2147483648);
      $165 = $163 & $164;
      $166 = HEAP32[$pVal>>2]|0;
      if ($165) {
       HEAPF64[$166>>3] = 9.2233720368547758E+18;
       $167 = HEAP32[$pVal>>2]|0;
       $168 = ((($167)) + 8|0);
       $169 = HEAP16[$168>>1]|0;
       $170 = $169&65535;
       $171 = $170 & -49664;
       $172 = $171 | 8;
       $173 = $172&65535;
       $174 = HEAP32[$pVal>>2]|0;
       $175 = ((($174)) + 8|0);
       HEAP16[$175>>1] = $173;
       break;
      } else {
       $176 = $166;
       $177 = $176;
       $178 = HEAP32[$177>>2]|0;
       $179 = (($176) + 4)|0;
       $180 = $179;
       $181 = HEAP32[$180>>2]|0;
       $182 = (_i64Subtract(0,0,($178|0),($181|0))|0);
       $183 = tempRet0;
       $184 = HEAP32[$pVal>>2]|0;
       $185 = $184;
       $186 = $185;
       HEAP32[$186>>2] = $182;
       $187 = (($185) + 4)|0;
       $188 = $187;
       HEAP32[$188>>2] = $183;
       break;
      }
     }
    } while(0);
    $189 = HEAP32[$pVal>>2]|0;
    $190 = HEAP8[$4>>0]|0;
    $191 = HEAP8[$3>>0]|0;
    _sqlite3ValueApplyAffinity($189,$190,$191);
    label = 42;
   } else {
    label = 42;
   }
  }
 } while(0);
 if ((label|0) == 42) {
  $220 = HEAP32[$pVal>>2]|0;
  $221 = HEAP32[$5>>2]|0;
  HEAP32[$221>>2] = $220;
  $222 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $222;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 else if ((label|0) == 43) {
  $223 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($223);
  $224 = HEAP32[$1>>2]|0;
  $225 = HEAP32[$zVal>>2]|0;
  _sqlite3DbFree($224,$225);
  $226 = HEAP32[$pVal>>2]|0;
  _sqlite3ValueFree($226);
  HEAP32[$0>>2] = 7;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 return (0)|0;
}
function _valueNew($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3ValueNew($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3VdbeAddOp4Dup8($p,$op,$p1,$p2,$p3,$zP4,$p4type) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $zP4 = $zP4|0;
 $p4type = $p4type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p4copy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $p4copy = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $zP4;
 HEAP32[$6>>2] = $p4type;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3VdbeDb($7)|0);
 $9 = (_sqlite3DbMallocRawNN($8,8,0)|0);
 HEAP32[$p4copy>>2] = $9;
 $10 = HEAP32[$p4copy>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$p4copy>>2]|0;
  $13 = HEAP32[$5>>2]|0;
  ;HEAP8[$12>>0]=HEAP8[$13>>0]|0;HEAP8[$12+1>>0]=HEAP8[$13+1>>0]|0;HEAP8[$12+2>>0]=HEAP8[$13+2>>0]|0;HEAP8[$12+3>>0]=HEAP8[$13+3>>0]|0;HEAP8[$12+4>>0]=HEAP8[$13+4>>0]|0;HEAP8[$12+5>>0]=HEAP8[$13+5>>0]|0;HEAP8[$12+6>>0]=HEAP8[$13+6>>0]|0;HEAP8[$12+7>>0]=HEAP8[$13+7>>0]|0;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$p4copy>>2]|0;
 $20 = HEAP32[$6>>2]|0;
 $21 = (_sqlite3VdbeAddOp4($14,$15,$16,$17,$18,$19,$20)|0);
 STACKTOP = sp;return ($21|0);
}
function _sqlite3ExprSkipCollate($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pExpr;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if ($2) {
   $3 = HEAP32[$0>>2]|0;
   $4 = ((($3)) + 4|0);
   $5 = HEAP32[$4>>2]|0;
   $6 = $5 & 4096;
   $7 = ($6|0)!=(0);
   $21 = $7;
  } else {
   $21 = 0;
  }
  $8 = HEAP32[$0>>2]|0;
  if (!($21)) {
   break;
  }
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 262144;
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$0>>2]|0;
  if ($12) {
   $14 = ((($13)) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$0>>2] = $18;
   continue;
  } else {
   $19 = ((($13)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$0>>2] = $20;
   continue;
  }
 }
 STACKTOP = sp;return ($8|0);
}
function _sqlite3ExprIsConstantNotJoin($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_exprIsConst($1,2,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ExprCompare($pA,$pB,$iTab) {
 $pA = $pA|0;
 $pB = $pB|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $combinedFlags = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $combinedFlags = sp;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $or$cond = $5 | $7;
 if ($or$cond) {
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($8|0)==($9|0);
  $11 = $10 ? 0 : 2;
  HEAP32[$0>>2] = $11;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $14 | $17;
 HEAP32[$combinedFlags>>2] = $18;
 $19 = HEAP32[$combinedFlags>>2]|0;
 $20 = $19 & 1024;
 $21 = ($20|0)!=(0);
 $22 = HEAP32[$1>>2]|0;
 if ($21) {
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $24 & $27;
  $29 = $28 & 1024;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = ((($34)) + 8|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($33|0)==($36|0);
   if ($37) {
    HEAP32[$0>>2] = 0;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
  }
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $38 = HEAP8[$22>>0]|0;
 $39 = $38&255;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($39|0)!=($42|0);
 $44 = HEAP32[$1>>2]|0;
 $45 = HEAP8[$44>>0]|0;
 $46 = $45&255;
 if ($43) {
  $47 = ($46|0)==(95);
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$2>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = (_sqlite3ExprCompare($50,$51,$52)|0);
   $54 = ($53|0)<(2);
   if ($54) {
    HEAP32[$0>>2] = 1;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
  }
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)==(95);
  if ($58) {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = ((($60)) + 12|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = (_sqlite3ExprCompare($59,$62,$63)|0);
   $65 = ($64|0)<(2);
   if ($65) {
    HEAP32[$0>>2] = 1;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
  }
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $66 = ($46|0)!=(152);
 do {
  if ($66) {
   $67 = HEAP32[$1>>2]|0;
   $68 = HEAP8[$67>>0]|0;
   $69 = $68&255;
   $70 = ($69|0)!=(154);
   if ($70) {
    $71 = HEAP32[$1>>2]|0;
    $72 = ((($71)) + 8|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = ($73|0)!=(0|0);
    if ($74) {
     $75 = HEAP32[$1>>2]|0;
     $76 = HEAP8[$75>>0]|0;
     $77 = $76&255;
     $78 = ($77|0)==(151);
     $79 = HEAP32[$1>>2]|0;
     $80 = ((($79)) + 8|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = HEAP32[$2>>2]|0;
     $83 = ((($82)) + 8|0);
     $84 = HEAP32[$83>>2]|0;
     if ($78) {
      $85 = (_sqlite3_stricmp($81,$84)|0);
      $86 = ($85|0)!=(0);
      if (!($86)) {
       break;
      }
      HEAP32[$0>>2] = 2;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     } else {
      $87 = (_strcmp($81,$84)|0);
      $88 = ($87|0)!=(0);
      if (!($88)) {
       break;
      }
      $89 = HEAP32[$1>>2]|0;
      $90 = HEAP8[$89>>0]|0;
      $91 = $90&255;
      $92 = ($91|0)==(95);
      $93 = $92 ? 1 : 2;
      HEAP32[$0>>2] = $93;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     }
    }
   }
  }
 } while(0);
 $94 = HEAP32[$1>>2]|0;
 $95 = ((($94)) + 4|0);
 $96 = HEAP32[$95>>2]|0;
 $97 = $96 & 16;
 $98 = HEAP32[$2>>2]|0;
 $99 = ((($98)) + 4|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = $100 & 16;
 $102 = ($97|0)!=($101|0);
 if ($102) {
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $103 = HEAP32[$combinedFlags>>2]|0;
 $104 = $103 & 16384;
 $105 = ($104|0)==(0);
 do {
  if ($105) {
   $106 = HEAP32[$combinedFlags>>2]|0;
   $107 = $106 & 2048;
   $108 = ($107|0)!=(0);
   if ($108) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $109 = HEAP32[$1>>2]|0;
   $110 = ((($109)) + 12|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = HEAP32[$2>>2]|0;
   $113 = ((($112)) + 12|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = HEAP32[$3>>2]|0;
   $116 = (_sqlite3ExprCompare($111,$114,$115)|0);
   $117 = ($116|0)!=(0);
   if ($117) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $118 = HEAP32[$1>>2]|0;
   $119 = ((($118)) + 16|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = HEAP32[$2>>2]|0;
   $122 = ((($121)) + 16|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = HEAP32[$3>>2]|0;
   $125 = (_sqlite3ExprCompare($120,$123,$124)|0);
   $126 = ($125|0)!=(0);
   if ($126) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $127 = HEAP32[$1>>2]|0;
   $128 = ((($127)) + 20|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = HEAP32[$2>>2]|0;
   $131 = ((($130)) + 20|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = HEAP32[$3>>2]|0;
   $134 = (_sqlite3ExprListCompare($129,$132,$133)|0);
   $135 = ($134|0)!=(0);
   if ($135) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $136 = HEAP32[$combinedFlags>>2]|0;
   $137 = $136 & 8192;
   $138 = ($137|0)==(0);
   if ($138) {
    $139 = HEAP32[$1>>2]|0;
    $140 = HEAP8[$139>>0]|0;
    $141 = $140&255;
    $142 = ($141|0)!=(97);
    if ($142) {
     $143 = HEAP32[$1>>2]|0;
     $144 = ((($143)) + 32|0);
     $145 = HEAP16[$144>>1]|0;
     $146 = $145 << 16 >> 16;
     $147 = HEAP32[$2>>2]|0;
     $148 = ((($147)) + 32|0);
     $149 = HEAP16[$148>>1]|0;
     $150 = $149 << 16 >> 16;
     $151 = ($146|0)!=($150|0);
     if ($151) {
      HEAP32[$0>>2] = 2;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     }
     $152 = HEAP32[$1>>2]|0;
     $153 = ((($152)) + 28|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = HEAP32[$2>>2]|0;
     $156 = ((($155)) + 28|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($154|0)!=($157|0);
     if ($158) {
      $159 = HEAP32[$1>>2]|0;
      $160 = ((($159)) + 28|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = HEAP32[$3>>2]|0;
      $163 = ($161|0)!=($162|0);
      if (!($163)) {
       $164 = HEAP32[$2>>2]|0;
       $165 = ((($164)) + 28|0);
       $166 = HEAP32[$165>>2]|0;
       $167 = ($166|0)>=(0);
       if (!($167)) {
        break;
       }
      }
      HEAP32[$0>>2] = 2;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     }
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $168 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($168|0);
}
function _sqlite3ExprCodeAtInit($pParse,$pExpr,$regDest,$reusable) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $regDest = $regDest|0;
 $reusable = $reusable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $p = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $regDest;
 HEAP8[$3>>0] = $reusable;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 324|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3ExprDup($8,$9,0)|0);
 HEAP32[$1>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$p>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_sqlite3ExprListAppend($11,$12,$13)|0);
 HEAP32[$p>>2] = $14;
 $15 = HEAP32[$p>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $37 = HEAP32[$p>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 324|0);
  HEAP32[$39>>2] = $37;
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$p>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) - 1)|0;
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + (($19*20)|0)|0);
 HEAP32[$pItem>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$pItem>>2]|0;
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP8[$3>>0]|0;
 $28 = $27&255;
 $29 = HEAP32[$pItem>>2]|0;
 $30 = ((($29)) + 13|0);
 $31 = $28&255;
 $32 = HEAP8[$30>>0]|0;
 $33 = $31 & 1;
 $34 = ($33 << 2)&255;
 $35 = $32 & -5;
 $36 = $35 | $34;
 HEAP8[$30>>0] = $36;
 $37 = HEAP32[$p>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 324|0);
 HEAP32[$39>>2] = $37;
 STACKTOP = sp;return;
}
function _exprIsConst($p,$initFlag,$iCur) {
 $p = $p|0;
 $initFlag = $initFlag|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $initFlag;
 HEAP32[$2>>2] = $iCur;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3&255;
 $5 = ((($w)) + 20|0);
 HEAP8[$5>>0] = $4;
 $6 = ((($w)) + 4|0);
 HEAP32[$6>>2] = 184;
 $7 = ((($w)) + 8|0);
 HEAP32[$7>>2] = 185;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($w)) + 24|0);
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$10)|0);
 $11 = ((($w)) + 20|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 STACKTOP = sp;return ($13|0);
}
function _exprNodeIsConstant($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 1;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 20|0);
   HEAP8[$14>>0] = 0;
   HEAP32[$0>>2] = 2;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 L4: do {
  if ((label|0) == 4) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   L6: do {
    switch ($17|0) {
    case 151:  {
     $18 = HEAP32[$1>>2]|0;
     $19 = ((($18)) + 20|0);
     $20 = HEAP8[$19>>0]|0;
     $21 = $20&255;
     $22 = ($21|0)>=(4);
     if (!($22)) {
      $23 = HEAP32[$2>>2]|0;
      $24 = ((($23)) + 4|0);
      $25 = HEAP32[$24>>2]|0;
      $26 = $25 & 524288;
      $27 = ($26|0)!=(0);
      if (!($27)) {
       $28 = HEAP32[$1>>2]|0;
       $29 = ((($28)) + 20|0);
       HEAP8[$29>>0] = 0;
       HEAP32[$0>>2] = 2;
       break L4;
      }
     }
     HEAP32[$0>>2] = 0;
     break L4;
     break;
    }
    case 154: case 153: case 152: case 27:  {
     $30 = HEAP32[$1>>2]|0;
     $31 = ((($30)) + 20|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = ($33|0)==(3);
     if ($34) {
      $35 = HEAP32[$2>>2]|0;
      $36 = ((($35)) + 28|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$1>>2]|0;
      $39 = ((($38)) + 24|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($37|0)==($40|0);
      if ($41) {
       HEAP32[$0>>2] = 0;
       break L4;
      }
     }
     $42 = HEAP32[$1>>2]|0;
     $43 = ((($42)) + 20|0);
     HEAP8[$43>>0] = 0;
     HEAP32[$0>>2] = 2;
     break L4;
     break;
    }
    case 135:  {
     $44 = HEAP32[$1>>2]|0;
     $45 = ((($44)) + 20|0);
     $46 = HEAP8[$45>>0]|0;
     $47 = $46&255;
     $48 = ($47|0)==(5);
     if ($48) {
      $49 = HEAP32[$2>>2]|0;
      HEAP8[$49>>0] = 101;
      break L6;
     }
     $50 = HEAP32[$1>>2]|0;
     $51 = ((($50)) + 20|0);
     $52 = HEAP8[$51>>0]|0;
     $53 = $52&255;
     $54 = ($53|0)==(4);
     if ($54) {
      $55 = HEAP32[$1>>2]|0;
      $56 = ((($55)) + 20|0);
      HEAP8[$56>>0] = 0;
      HEAP32[$0>>2] = 2;
      break L4;
     }
     break;
    }
    default: {
    }
    }
   } while(0);
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _selectNodeIsConstant($pWalker,$NotUsed) {
 $pWalker = $pWalker|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $NotUsed;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 20|0);
 HEAP8[$3>>0] = 0;
 STACKTOP = sp;return 2;
}
function _sqlite3WalkExpr($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_walkExpr($4,$5)|0);
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _walkExpr($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 do {
  if ($10) {
   $11 = HEAP32[$2>>2]|0;
   $12 = ((($11)) + 4|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = $13 & 16384;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = HEAP32[$2>>2]|0;
    $18 = ((($17)) + 12|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = (_sqlite3WalkExpr($16,$19)|0);
    $21 = ($20|0)!=(0);
    if ($21) {
     HEAP32[$0>>2] = 2;
     $44 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($44|0);
    }
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = ((($23)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (_sqlite3WalkExpr($22,$25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     HEAP32[$0>>2] = 2;
     $44 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($44|0);
    }
    $28 = HEAP32[$2>>2]|0;
    $29 = ((($28)) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = $30 & 2048;
    $32 = ($31|0)!=(0);
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = ((($34)) + 20|0);
    if ($32) {
     $36 = HEAP32[$35>>2]|0;
     $37 = (_sqlite3WalkSelect($33,$36)|0);
     $38 = ($37|0)!=(0);
     if (!($38)) {
      break;
     }
     HEAP32[$0>>2] = 2;
     $44 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($44|0);
    } else {
     $39 = HEAP32[$35>>2]|0;
     $40 = (_sqlite3WalkExprList($33,$39)|0);
     $41 = ($40|0)!=(0);
     if (!($41)) {
      break;
     }
     HEAP32[$0>>2] = 2;
     $44 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($44|0);
    }
   }
  }
 } while(0);
 $42 = HEAP32[$rc>>2]|0;
 $43 = $42 & 2;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3WalkSelect($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 do {
  if (!($4)) {
   $5 = HEAP32[$1>>2]|0;
   $6 = ((($5)) + 8|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $9 = HEAP32[$1>>2]|0;
    $10 = ((($9)) + 12|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($11|0)==(0|0);
    if ($12) {
     break;
    }
   }
   HEAP32[$rc>>2] = 0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + 1)|0;
   HEAP32[$14>>2] = $16;
   while(1) {
    $17 = HEAP32[$2>>2]|0;
    $18 = ($17|0)!=(0|0);
    if (!($18)) {
     label = 15;
     break;
    }
    $19 = HEAP32[$1>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)!=(0|0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = ((($23)) + 8|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = HEAP32[$1>>2]|0;
     $27 = HEAP32[$2>>2]|0;
     $28 = (FUNCTION_TABLE_iii[$25 & 255]($26,$27)|0);
     HEAP32[$rc>>2] = $28;
     $29 = HEAP32[$rc>>2]|0;
     $30 = ($29|0)!=(0);
     if ($30) {
      label = 15;
      break;
     }
    }
    $31 = HEAP32[$1>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = (_sqlite3WalkSelectExpr($31,$32)|0);
    $34 = ($33|0)!=(0);
    if ($34) {
     label = 11;
     break;
    }
    $35 = HEAP32[$1>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    $37 = (_sqlite3WalkSelectFrom($35,$36)|0);
    $38 = ($37|0)!=(0);
    if ($38) {
     label = 11;
     break;
    }
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$1>>2]|0;
     $48 = ((($47)) + 12|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = HEAP32[$2>>2]|0;
     FUNCTION_TABLE_vii[$49 & 255]($50,$51);
    }
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 52|0);
    $54 = HEAP32[$53>>2]|0;
    HEAP32[$2>>2] = $54;
   }
   if ((label|0) == 11) {
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 16|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = (($41) + -1)|0;
    HEAP32[$40>>2] = $42;
    HEAP32[$0>>2] = 2;
    $61 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($61|0);
   }
   else if ((label|0) == 15) {
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 16|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + -1)|0;
    HEAP32[$56>>2] = $58;
    $59 = HEAP32[$rc>>2]|0;
    $60 = $59 & 2;
    HEAP32[$0>>2] = $60;
    $61 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($61|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3WalkExprList($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   $5 = HEAP32[$2>>2]|0;
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$i>>2] = $6;
   $7 = HEAP32[$2>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$pItem>>2] = $9;
   while(1) {
    $10 = HEAP32[$i>>2]|0;
    $11 = ($10|0)>(0);
    if (!($11)) {
     break L1;
    }
    $12 = HEAP32[$1>>2]|0;
    $13 = HEAP32[$pItem>>2]|0;
    $14 = HEAP32[$13>>2]|0;
    $15 = (_sqlite3WalkExpr($12,$14)|0);
    $16 = ($15|0)!=(0);
    if ($16) {
     break;
    }
    $17 = HEAP32[$i>>2]|0;
    $18 = (($17) + -1)|0;
    HEAP32[$i>>2] = $18;
    $19 = HEAP32[$pItem>>2]|0;
    $20 = ((($19)) + 20|0);
    HEAP32[$pItem>>2] = $20;
   }
   HEAP32[$0>>2] = 2;
   $21 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($21|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3WalkSelectExpr($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3WalkExprList($3,$5)|0);
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   HEAP32[$0>>2] = 2;
  } else {
   $8 = HEAP32[$1>>2]|0;
   $9 = HEAP32[$2>>2]|0;
   $10 = ((($9)) + 36|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = (_sqlite3WalkExpr($8,$11)|0);
   $13 = ($12|0)!=(0);
   if ($13) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 40|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3WalkExprList($14,$17)|0);
   $19 = ($18|0)!=(0);
   if ($19) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = ((($21)) + 44|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3WalkExpr($20,$23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = ((($27)) + 48|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3WalkExprList($26,$29)|0);
   $31 = ($30|0)!=(0);
   if ($31) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $32 = HEAP32[$1>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = ((($33)) + 60|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3WalkExpr($32,$35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $38 = HEAP32[$1>>2]|0;
   $39 = HEAP32[$2>>2]|0;
   $40 = ((($39)) + 64|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (_sqlite3WalkExpr($38,$41)|0);
   $43 = ($42|0)!=(0);
   if ($43) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3WalkSelectFrom($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $pItem = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSrc = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ($6|0)!=(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$pSrc>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$i>>2] = $9;
   $10 = HEAP32[$pSrc>>2]|0;
   $11 = ((($10)) + 8|0);
   HEAP32[$pItem>>2] = $11;
   while(1) {
    $12 = HEAP32[$i>>2]|0;
    $13 = ($12|0)>(0);
    if (!($13)) {
     break L1;
    }
    $14 = HEAP32[$1>>2]|0;
    $15 = HEAP32[$pItem>>2]|0;
    $16 = ((($15)) + 20|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (_sqlite3WalkSelect($14,$17)|0);
    $19 = ($18|0)!=(0);
    if ($19) {
     label = 5;
     break;
    }
    $20 = HEAP32[$pItem>>2]|0;
    $21 = ((($20)) + 36|0);
    $22 = ((($21)) + 1|0);
    $23 = HEAP8[$22>>0]|0;
    $24 = ($23&255) >>> 2;
    $25 = $24 & 1;
    $26 = $25&255;
    $27 = ($26|0)!=(0);
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = HEAP32[$pItem>>2]|0;
     $30 = ((($29)) + 64|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = (_sqlite3WalkExprList($28,$31)|0);
     $33 = ($32|0)!=(0);
     if ($33) {
      label = 8;
      break;
     }
    }
    $34 = HEAP32[$i>>2]|0;
    $35 = (($34) + -1)|0;
    HEAP32[$i>>2] = $35;
    $36 = HEAP32[$pItem>>2]|0;
    $37 = ((($36)) + 72|0);
    HEAP32[$pItem>>2] = $37;
   }
   if ((label|0) == 5) {
    HEAP32[$0>>2] = 2;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
   else if ((label|0) == 8) {
    HEAP32[$0>>2] = 2;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3ExprListCompare($pA,$pB,$iTab) {
 $pA = $pA|0;
 $pB = $pB|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0;
 var $or$cond3 = 0, $pExprA = 0, $pExprB = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $pExprA = sp + 4|0;
 $pExprB = sp;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $or$cond = $5 & $7;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 $or$cond3 = $9 | $11;
 if ($or$cond3) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($13|0)!=($15|0);
 if ($16) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($17|0)<($19|0);
  if (!($20)) {
   label = 14;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + (($21*20)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pExprA>>2] = $26;
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + (($27*20)|0)|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$pExprB>>2] = $32;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*20)|0)|0);
  $38 = ((($37)) + 12|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($41*20)|0)|0);
  $46 = ((($45)) + 12|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = ($40|0)!=($48|0);
  if ($49) {
   label = 10;
   break;
  }
  $50 = HEAP32[$pExprA>>2]|0;
  $51 = HEAP32[$pExprB>>2]|0;
  $52 = HEAP32[$3>>2]|0;
  $53 = (_sqlite3ExprCompare($50,$51,$52)|0);
  $54 = ($53|0)!=(0);
  if ($54) {
   label = 12;
   break;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$i>>2] = $56;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _sqlite3ExprDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_exprDup($3,$4,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _exprDup($db,$p,$flags,$pzBuffer) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 $pzBuffer = $pzBuffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $isReduced = 0, $nNewSize = 0, $nSize = 0, $nStructSize = 0, $nToken = 0, $pNew = 0, $staticFlag = 0, $zAlloc = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $pNew = sp + 32|0;
 $isReduced = sp + 28|0;
 $zAlloc = sp + 24|0;
 $staticFlag = sp + 20|0;
 $nStructSize = sp + 16|0;
 $nNewSize = sp + 12|0;
 $nToken = sp + 8|0;
 $nSize = sp + 4|0;
 $zToken = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pzBuffer;
 HEAP32[$pNew>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $163 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($163|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = $6 & 1;
 HEAP32[$isReduced>>2] = $7;
 HEAP32[$staticFlag>>2] = 0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$3>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$zAlloc>>2] = $11;
  HEAP32[$staticFlag>>2] = 32768;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (_dupedExprSize($13,$14)|0);
  $16 = ($15|0)<(0);
  $17 = $16 << 31 >> 31;
  $18 = (_sqlite3DbMallocRawNN($12,$15,$17)|0);
  HEAP32[$zAlloc>>2] = $18;
 }
 $19 = HEAP32[$zAlloc>>2]|0;
 HEAP32[$pNew>>2] = $19;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ($20|0)!=(0|0);
 if (!($21)) {
  $163 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($163|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (_dupedExprStructSize($22,$23)|0);
 HEAP32[$nStructSize>>2] = $24;
 $25 = HEAP32[$nStructSize>>2]|0;
 $26 = $25 & 4095;
 HEAP32[$nNewSize>>2] = $26;
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $29 & 1024;
 $31 = ($30|0)!=(0);
 if ($31) {
  label = 9;
 } else {
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)!=(0|0);
  if ($35) {
   $36 = HEAP32[$1>>2]|0;
   $37 = ((($36)) + 8|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3Strlen30($38)|0);
   $40 = (($39) + 1)|0;
   HEAP32[$nToken>>2] = $40;
  } else {
   label = 9;
  }
 }
 if ((label|0) == 9) {
  HEAP32[$nToken>>2] = 0;
 }
 $41 = HEAP32[$isReduced>>2]|0;
 $42 = ($41|0)!=(0);
 if ($42) {
  $43 = HEAP32[$zAlloc>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = HEAP32[$nNewSize>>2]|0;
  _memcpy(($43|0),($44|0),($45|0))|0;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = (_exprStructSize($46)|0);
  HEAP32[$nSize>>2] = $47;
  $48 = HEAP32[$zAlloc>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$nSize>>2]|0;
  _memcpy(($48|0),($49|0),($50|0))|0;
  $51 = HEAP32[$nSize>>2]|0;
  $52 = ($51>>>0)<(48);
  if ($52) {
   $53 = HEAP32[$nSize>>2]|0;
   $54 = HEAP32[$zAlloc>>2]|0;
   $55 = (($54) + ($53)|0);
   $56 = HEAP32[$nSize>>2]|0;
   $57 = (48 - ($56))|0;
   _memset(($55|0),0,($57|0))|0;
  }
 }
 $58 = HEAP32[$pNew>>2]|0;
 $59 = ((($58)) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = $60 & -122881;
 HEAP32[$59>>2] = $61;
 $62 = HEAP32[$nStructSize>>2]|0;
 $63 = $62 & 24576;
 $64 = HEAP32[$pNew>>2]|0;
 $65 = ((($64)) + 4|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $66 | $63;
 HEAP32[$65>>2] = $67;
 $68 = HEAP32[$staticFlag>>2]|0;
 $69 = HEAP32[$pNew>>2]|0;
 $70 = ((($69)) + 4|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = $71 | $68;
 HEAP32[$70>>2] = $72;
 $73 = HEAP32[$nToken>>2]|0;
 $74 = ($73|0)!=(0);
 if ($74) {
  $75 = HEAP32[$nNewSize>>2]|0;
  $76 = HEAP32[$zAlloc>>2]|0;
  $77 = (($76) + ($75)|0);
  $78 = HEAP32[$pNew>>2]|0;
  $79 = ((($78)) + 8|0);
  HEAP32[$79>>2] = $77;
  HEAP32[$zToken>>2] = $77;
  $80 = HEAP32[$zToken>>2]|0;
  $81 = HEAP32[$1>>2]|0;
  $82 = ((($81)) + 8|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = HEAP32[$nToken>>2]|0;
  _memcpy(($80|0),($83|0),($84|0))|0;
 }
 $85 = HEAP32[$1>>2]|0;
 $86 = ((($85)) + 4|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$pNew>>2]|0;
 $89 = ((($88)) + 4|0);
 $90 = HEAP32[$89>>2]|0;
 $91 = $87 | $90;
 $92 = $91 & 16384;
 $93 = (0)==($92|0);
 do {
  if ($93) {
   $94 = HEAP32[$1>>2]|0;
   $95 = ((($94)) + 4|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = $96 & 2048;
   $98 = ($97|0)!=(0);
   $99 = HEAP32[$0>>2]|0;
   $100 = HEAP32[$1>>2]|0;
   $101 = ((($100)) + 20|0);
   if ($98) {
    $102 = HEAP32[$101>>2]|0;
    $103 = HEAP32[$isReduced>>2]|0;
    $104 = (_sqlite3SelectDup($99,$102,$103)|0);
    $105 = HEAP32[$pNew>>2]|0;
    $106 = ((($105)) + 20|0);
    HEAP32[$106>>2] = $104;
    break;
   } else {
    $107 = HEAP32[$101>>2]|0;
    $108 = HEAP32[$isReduced>>2]|0;
    $109 = (_sqlite3ExprListDup($99,$107,$108)|0);
    $110 = HEAP32[$pNew>>2]|0;
    $111 = ((($110)) + 20|0);
    HEAP32[$111>>2] = $109;
    break;
   }
  }
 } while(0);
 $112 = HEAP32[$pNew>>2]|0;
 $113 = ((($112)) + 4|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $114 & 24576;
 $116 = ($115|0)!=(0);
 $117 = HEAP32[$1>>2]|0;
 if (!($116)) {
  $145 = ((($117)) + 4|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = $146 & 16384;
  $148 = ($147|0)!=(0);
  if ($148) {
   $163 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($163|0);
  }
  $149 = HEAP32[$0>>2]|0;
  $150 = HEAP32[$1>>2]|0;
  $151 = ((($150)) + 12|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = (_sqlite3ExprDup($149,$152,0)|0);
  $154 = HEAP32[$pNew>>2]|0;
  $155 = ((($154)) + 12|0);
  HEAP32[$155>>2] = $153;
  $156 = HEAP32[$0>>2]|0;
  $157 = HEAP32[$1>>2]|0;
  $158 = ((($157)) + 16|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = (_sqlite3ExprDup($156,$159,0)|0);
  $161 = HEAP32[$pNew>>2]|0;
  $162 = ((($161)) + 16|0);
  HEAP32[$162>>2] = $160;
  $163 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($163|0);
 }
 $118 = HEAP32[$2>>2]|0;
 $119 = (_dupedExprNodeSize($117,$118)|0);
 $120 = HEAP32[$zAlloc>>2]|0;
 $121 = (($120) + ($119)|0);
 HEAP32[$zAlloc>>2] = $121;
 $122 = HEAP32[$pNew>>2]|0;
 $123 = ((($122)) + 4|0);
 $124 = HEAP32[$123>>2]|0;
 $125 = $124 & 8192;
 $126 = ($125|0)!=(0);
 if ($126) {
  $127 = HEAP32[$0>>2]|0;
  $128 = HEAP32[$1>>2]|0;
  $129 = ((($128)) + 12|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = (_exprDup($127,$130,1,$zAlloc)|0);
  $132 = HEAP32[$pNew>>2]|0;
  $133 = ((($132)) + 12|0);
  HEAP32[$133>>2] = $131;
  $134 = HEAP32[$0>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  $136 = ((($135)) + 16|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = (_exprDup($134,$137,1,$zAlloc)|0);
  $139 = HEAP32[$pNew>>2]|0;
  $140 = ((($139)) + 16|0);
  HEAP32[$140>>2] = $138;
 }
 $141 = HEAP32[$3>>2]|0;
 $142 = ($141|0)!=(0|0);
 if (!($142)) {
  $163 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($163|0);
 }
 $143 = HEAP32[$zAlloc>>2]|0;
 $144 = HEAP32[$3>>2]|0;
 HEAP32[$144>>2] = $143;
 $163 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($163|0);
}
function _dupedExprSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 HEAP32[$nByte>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $23 = HEAP32[$nByte>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_dupedExprNodeSize($4,$5)|0);
 HEAP32[$nByte>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = $7 & 1;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  $23 = HEAP32[$nByte>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_dupedExprSize($12,$13)|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (_dupedExprSize($17,$18)|0);
 $20 = (($14) + ($19))|0;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (($21) + ($20))|0;
 HEAP32[$nByte>>2] = $22;
 $23 = HEAP32[$nByte>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _dupedExprStructSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nSize = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = (0)==($3|0);
 do {
  if ($4) {
   HEAP32[$nSize>>2] = 48;
  } else {
   $5 = HEAP32[$0>>2]|0;
   $6 = ((($5)) + 12|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)!=(0|0);
   if (!($8)) {
    $9 = HEAP32[$0>>2]|0;
    $10 = ((($9)) + 20|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($11|0)!=(0|0);
    if (!($12)) {
     HEAP32[$nSize>>2] = 16396;
     break;
    }
   }
   HEAP32[$nSize>>2] = 8220;
  }
 } while(0);
 $13 = HEAP32[$nSize>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _exprStructSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 16384;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 12;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = $9 & 8192;
   $11 = ($10|0)!=(0);
   if ($11) {
    HEAP32[$0>>2] = 28;
    break;
   } else {
    HEAP32[$0>>2] = 48;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3SelectDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pNew = 0, $pPrior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pNew = sp + 4|0;
 $pPrior = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $131 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3DbMallocRawNN($6,72,0)|0);
 HEAP32[$pNew>>2] = $7;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $131 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_sqlite3ExprListDup($10,$12,$13)|0);
 $15 = HEAP32[$pNew>>2]|0;
 HEAP32[$15>>2] = $14;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (_sqlite3SrcListDup($16,$19,$20)|0);
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ((($22)) + 32|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$1>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 36|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (_sqlite3ExprDup($24,$27,$28)|0);
 $30 = HEAP32[$pNew>>2]|0;
 $31 = ((($30)) + 36|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 40|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (_sqlite3ExprListDup($32,$35,$36)|0);
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 40|0);
 HEAP32[$39>>2] = $37;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 44|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 $45 = (_sqlite3ExprDup($40,$43,$44)|0);
 $46 = HEAP32[$pNew>>2]|0;
 $47 = ((($46)) + 44|0);
 HEAP32[$47>>2] = $45;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = ((($49)) + 48|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 $53 = (_sqlite3ExprListDup($48,$51,$52)|0);
 $54 = HEAP32[$pNew>>2]|0;
 $55 = ((($54)) + 48|0);
 HEAP32[$55>>2] = $53;
 $56 = HEAP32[$2>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP8[$57>>0]|0;
 $59 = HEAP32[$pNew>>2]|0;
 $60 = ((($59)) + 4|0);
 HEAP8[$60>>0] = $58;
 $61 = HEAP32[$1>>2]|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = ((($62)) + 52|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$3>>2]|0;
 $66 = (_sqlite3SelectDup($61,$64,$65)|0);
 HEAP32[$pPrior>>2] = $66;
 $67 = HEAP32[$pNew>>2]|0;
 $68 = ((($67)) + 52|0);
 HEAP32[$68>>2] = $66;
 $69 = HEAP32[$pPrior>>2]|0;
 $70 = ($69|0)!=(0|0);
 if ($70) {
  $71 = HEAP32[$pNew>>2]|0;
  $72 = HEAP32[$pPrior>>2]|0;
  $73 = ((($72)) + 56|0);
  HEAP32[$73>>2] = $71;
 }
 $74 = HEAP32[$pNew>>2]|0;
 $75 = ((($74)) + 56|0);
 HEAP32[$75>>2] = 0;
 $76 = HEAP32[$1>>2]|0;
 $77 = HEAP32[$2>>2]|0;
 $78 = ((($77)) + 60|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = HEAP32[$3>>2]|0;
 $81 = (_sqlite3ExprDup($76,$79,$80)|0);
 $82 = HEAP32[$pNew>>2]|0;
 $83 = ((($82)) + 60|0);
 HEAP32[$83>>2] = $81;
 $84 = HEAP32[$1>>2]|0;
 $85 = HEAP32[$2>>2]|0;
 $86 = ((($85)) + 64|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$3>>2]|0;
 $89 = (_sqlite3ExprDup($84,$87,$88)|0);
 $90 = HEAP32[$pNew>>2]|0;
 $91 = ((($90)) + 64|0);
 HEAP32[$91>>2] = $89;
 $92 = HEAP32[$pNew>>2]|0;
 $93 = ((($92)) + 8|0);
 HEAP32[$93>>2] = 0;
 $94 = HEAP32[$pNew>>2]|0;
 $95 = ((($94)) + 12|0);
 HEAP32[$95>>2] = 0;
 $96 = HEAP32[$2>>2]|0;
 $97 = ((($96)) + 6|0);
 $98 = HEAP16[$97>>1]|0;
 $99 = $98&65535;
 $100 = $99 & -17;
 $101 = $100&65535;
 $102 = HEAP32[$pNew>>2]|0;
 $103 = ((($102)) + 6|0);
 HEAP16[$103>>1] = $101;
 $104 = HEAP32[$pNew>>2]|0;
 $105 = ((($104)) + 16|0);
 HEAP32[$105>>2] = -1;
 $106 = HEAP32[$pNew>>2]|0;
 $107 = ((($106)) + 16|0);
 $108 = ((($107)) + 4|0);
 HEAP32[$108>>2] = -1;
 $109 = HEAP32[$2>>2]|0;
 $110 = ((($109)) + 24|0);
 $111 = $110;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = HEAP32[$pNew>>2]|0;
 $118 = ((($117)) + 24|0);
 $119 = $118;
 $120 = $119;
 HEAP32[$120>>2] = $113;
 $121 = (($119) + 4)|0;
 $122 = $121;
 HEAP32[$122>>2] = $116;
 $123 = HEAP32[$1>>2]|0;
 $124 = HEAP32[$2>>2]|0;
 $125 = ((($124)) + 68|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = (_withDup($123,$126)|0);
 $128 = HEAP32[$pNew>>2]|0;
 $129 = ((($128)) + 68|0);
 HEAP32[$129>>2] = $127;
 $130 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $130;
 $131 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($131|0);
}
function _sqlite3ExprListDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $i = 0, $pItem = 0, $pNew = 0, $pOldExpr = 0;
 var $pOldItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pNew = sp + 16|0;
 $pItem = sp + 12|0;
 $pOldItem = sp + 8|0;
 $i = sp + 4|0;
 $pOldExpr = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3DbMallocRawNN($6,8,0)|0);
 HEAP32[$pNew>>2] = $7;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$i>>2] = $11;
 $12 = HEAP32[$pNew>>2]|0;
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = $13 & 1;
 $15 = ($14|0)==(0);
 L9: do {
  if ($15) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $16 = HEAP32[$i>>2]|0;
    $17 = HEAP32[$2>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = ($16|0)<($18|0);
    if (!($19)) {
     break L9;
    }
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$i>>2]|0;
    $22 = (($21) + ($20))|0;
    HEAP32[$i>>2] = $22;
   }
  }
 } while(0);
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$i>>2]|0;
 $25 = ($24*20)|0;
 $26 = (_sqlite3DbMallocRawNN($23,$25,0)|0);
 HEAP32[$pItem>>2] = $26;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = ((($27)) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pItem>>2]|0;
 $30 = ($29|0)==(0|0);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($31,$32);
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 4|0);
 $35 = HEAP32[$34>>2]|0;
 HEAP32[$pOldItem>>2] = $35;
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = ($36|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$pOldItem>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$pOldExpr>>2] = $41;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pOldExpr>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  $45 = (_sqlite3ExprDup($42,$43,$44)|0);
  $46 = HEAP32[$pItem>>2]|0;
  HEAP32[$46>>2] = $45;
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$pOldItem>>2]|0;
  $49 = ((($48)) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (_sqlite3DbStrDup($47,$50)|0);
  $52 = HEAP32[$pItem>>2]|0;
  $53 = ((($52)) + 4|0);
  HEAP32[$53>>2] = $51;
  $54 = HEAP32[$1>>2]|0;
  $55 = HEAP32[$pOldItem>>2]|0;
  $56 = ((($55)) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (_sqlite3DbStrDup($54,$57)|0);
  $59 = HEAP32[$pItem>>2]|0;
  $60 = ((($59)) + 8|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$pOldItem>>2]|0;
  $62 = ((($61)) + 12|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = HEAP32[$pItem>>2]|0;
  $65 = ((($64)) + 12|0);
  HEAP8[$65>>0] = $63;
  $66 = HEAP32[$pItem>>2]|0;
  $67 = ((($66)) + 13|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 & -2;
  HEAP8[$67>>0] = $69;
  $70 = HEAP32[$pOldItem>>2]|0;
  $71 = ((($70)) + 13|0);
  $72 = HEAP8[$71>>0]|0;
  $73 = ($72&255) >>> 1;
  $74 = $73 & 1;
  $75 = $74&255;
  $76 = HEAP32[$pItem>>2]|0;
  $77 = ((($76)) + 13|0);
  $78 = $75&255;
  $79 = HEAP8[$77>>0]|0;
  $80 = $78 & 1;
  $81 = ($80 << 1)&255;
  $82 = $79 & -3;
  $83 = $82 | $81;
  HEAP8[$77>>0] = $83;
  $84 = HEAP32[$pItem>>2]|0;
  $85 = ((($84)) + 16|0);
  $86 = HEAP32[$pOldItem>>2]|0;
  $87 = ((($86)) + 16|0);
  ;HEAP32[$85>>2]=HEAP32[$87>>2]|0;
  $88 = HEAP32[$i>>2]|0;
  $89 = (($88) + 1)|0;
  HEAP32[$i>>2] = $89;
  $90 = HEAP32[$pItem>>2]|0;
  $91 = ((($90)) + 20|0);
  HEAP32[$pItem>>2] = $91;
  $92 = HEAP32[$pOldItem>>2]|0;
  $93 = ((($92)) + 20|0);
  HEAP32[$pOldItem>>2] = $93;
 }
 $94 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $94;
 $95 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _dupedExprNodeSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_dupedExprStructSize($2,$3)|0);
 $5 = $4 & 4095;
 HEAP32[$nByte>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1024;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3Strlen30($17)|0);
   $19 = (($18) + 1)|0;
   $20 = HEAP32[$nByte>>2]|0;
   $21 = (($20) + ($19))|0;
   HEAP32[$nByte>>2] = $21;
  }
 }
 $22 = HEAP32[$nByte>>2]|0;
 $23 = (($22) + 7)|0;
 $24 = $23 & -8;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3SrcListDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $nByte = 0, $pNew = 0, $pNewItem = 0, $pOldItem = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $pNew = sp + 20|0;
 $i = sp + 16|0;
 $nByte = sp + 12|0;
 $pNewItem = sp + 8|0;
 $pOldItem = sp + 4|0;
 $pTab = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) - 1)|0;
  $12 = ($11*72)|0;
  $14 = $12;
 } else {
  $14 = 0;
 }
 $13 = (80 + ($14))|0;
 HEAP32[$nByte>>2] = $13;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$nByte>>2]|0;
 $17 = ($16|0)<(0);
 $18 = $17 << 31 >> 31;
 $19 = (_sqlite3DbMallocRawNN($15,$16,$18)|0);
 HEAP32[$pNew>>2] = $19;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ($20|0)==(0|0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$pNew>>2]|0;
 $25 = ((($24)) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$pNew>>2]|0;
 HEAP32[$26>>2] = $23;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = ($27|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = (($33) + (($31*72)|0)|0);
  HEAP32[$pNewItem>>2] = $34;
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = (($37) + (($35*72)|0)|0);
  HEAP32[$pOldItem>>2] = $38;
  $39 = HEAP32[$pOldItem>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pNewItem>>2]|0;
  HEAP32[$41>>2] = $40;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pOldItem>>2]|0;
  $44 = ((($43)) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3DbStrDup($42,$45)|0);
  $47 = HEAP32[$pNewItem>>2]|0;
  $48 = ((($47)) + 4|0);
  HEAP32[$48>>2] = $46;
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$pOldItem>>2]|0;
  $51 = ((($50)) + 8|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (_sqlite3DbStrDup($49,$52)|0);
  $54 = HEAP32[$pNewItem>>2]|0;
  $55 = ((($54)) + 8|0);
  HEAP32[$55>>2] = $53;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pOldItem>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (_sqlite3DbStrDup($56,$59)|0);
  $61 = HEAP32[$pNewItem>>2]|0;
  $62 = ((($61)) + 12|0);
  HEAP32[$62>>2] = $60;
  $63 = HEAP32[$pNewItem>>2]|0;
  $64 = ((($63)) + 36|0);
  $65 = HEAP32[$pOldItem>>2]|0;
  $66 = ((($65)) + 36|0);
  ;HEAP32[$64>>2]=HEAP32[$66>>2]|0;
  $67 = HEAP32[$pOldItem>>2]|0;
  $68 = ((($67)) + 44|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$pNewItem>>2]|0;
  $71 = ((($70)) + 44|0);
  HEAP32[$71>>2] = $69;
  $72 = HEAP32[$pOldItem>>2]|0;
  $73 = ((($72)) + 24|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = HEAP32[$pNewItem>>2]|0;
  $76 = ((($75)) + 24|0);
  HEAP32[$76>>2] = $74;
  $77 = HEAP32[$pOldItem>>2]|0;
  $78 = ((($77)) + 28|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = HEAP32[$pNewItem>>2]|0;
  $81 = ((($80)) + 28|0);
  HEAP32[$81>>2] = $79;
  $82 = HEAP32[$pNewItem>>2]|0;
  $83 = ((($82)) + 36|0);
  $84 = ((($83)) + 1|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = ($85&255) >>> 1;
  $87 = $86 & 1;
  $88 = $87&255;
  $89 = ($88|0)!=(0);
  if ($89) {
   $90 = HEAP32[$1>>2]|0;
   $91 = HEAP32[$pOldItem>>2]|0;
   $92 = ((($91)) + 64|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (_sqlite3DbStrDup($90,$93)|0);
   $95 = HEAP32[$pNewItem>>2]|0;
   $96 = ((($95)) + 64|0);
   HEAP32[$96>>2] = $94;
  }
  $97 = HEAP32[$pOldItem>>2]|0;
  $98 = ((($97)) + 68|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = HEAP32[$pNewItem>>2]|0;
  $101 = ((($100)) + 68|0);
  HEAP32[$101>>2] = $99;
  $102 = HEAP32[$pNewItem>>2]|0;
  $103 = ((($102)) + 36|0);
  $104 = ((($103)) + 1|0);
  $105 = HEAP8[$104>>0]|0;
  $106 = ($105&255) >>> 2;
  $107 = $106 & 1;
  $108 = $107&255;
  $109 = ($108|0)!=(0);
  if ($109) {
   $110 = HEAP32[$1>>2]|0;
   $111 = HEAP32[$pOldItem>>2]|0;
   $112 = ((($111)) + 64|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = HEAP32[$3>>2]|0;
   $115 = (_sqlite3ExprListDup($110,$113,$114)|0);
   $116 = HEAP32[$pNewItem>>2]|0;
   $117 = ((($116)) + 64|0);
   HEAP32[$117>>2] = $115;
  }
  $118 = HEAP32[$pOldItem>>2]|0;
  $119 = ((($118)) + 16|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = HEAP32[$pNewItem>>2]|0;
  $122 = ((($121)) + 16|0);
  HEAP32[$122>>2] = $120;
  HEAP32[$pTab>>2] = $120;
  $123 = HEAP32[$pTab>>2]|0;
  $124 = ($123|0)!=(0|0);
  if ($124) {
   $125 = HEAP32[$pTab>>2]|0;
   $126 = ((($125)) + 36|0);
   $127 = HEAP16[$126>>1]|0;
   $128 = (($127) + 1)<<16>>16;
   HEAP16[$126>>1] = $128;
  }
  $129 = HEAP32[$1>>2]|0;
  $130 = HEAP32[$pOldItem>>2]|0;
  $131 = ((($130)) + 20|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = HEAP32[$3>>2]|0;
  $134 = (_sqlite3SelectDup($129,$132,$133)|0);
  $135 = HEAP32[$pNewItem>>2]|0;
  $136 = ((($135)) + 20|0);
  HEAP32[$136>>2] = $134;
  $137 = HEAP32[$1>>2]|0;
  $138 = HEAP32[$pOldItem>>2]|0;
  $139 = ((($138)) + 48|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = HEAP32[$3>>2]|0;
  $142 = (_sqlite3ExprDup($137,$140,$141)|0);
  $143 = HEAP32[$pNewItem>>2]|0;
  $144 = ((($143)) + 48|0);
  HEAP32[$144>>2] = $142;
  $145 = HEAP32[$1>>2]|0;
  $146 = HEAP32[$pOldItem>>2]|0;
  $147 = ((($146)) + 52|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = (_sqlite3IdListDup($145,$148)|0);
  $150 = HEAP32[$pNewItem>>2]|0;
  $151 = ((($150)) + 52|0);
  HEAP32[$151>>2] = $149;
  $152 = HEAP32[$pOldItem>>2]|0;
  $153 = ((($152)) + 56|0);
  $154 = $153;
  $155 = $154;
  $156 = HEAP32[$155>>2]|0;
  $157 = (($154) + 4)|0;
  $158 = $157;
  $159 = HEAP32[$158>>2]|0;
  $160 = HEAP32[$pNewItem>>2]|0;
  $161 = ((($160)) + 56|0);
  $162 = $161;
  $163 = $162;
  HEAP32[$163>>2] = $156;
  $164 = (($162) + 4)|0;
  $165 = $164;
  HEAP32[$165>>2] = $159;
  $166 = HEAP32[$i>>2]|0;
  $167 = (($166) + 1)|0;
  HEAP32[$i>>2] = $167;
 }
 $168 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $168;
 $169 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _withDup($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $nByte = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRet = sp + 8|0;
 $nByte = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$pRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $62 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - 1)|0;
 $7 = $6<<4;
 $8 = (24 + ($7))|0;
 HEAP32[$nByte>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$nByte>>2]|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (_sqlite3DbMallocZero($9,$10,$12)|0);
 HEAP32[$pRet>>2] = $13;
 $14 = HEAP32[$pRet>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $62 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$pRet>>2]|0;
 HEAP32[$18>>2] = $17;
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ($19|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = (($26) + ($24<<4)|0);
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (_sqlite3SelectDup($23,$29,0)|0);
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pRet>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = (($33) + ($31<<4)|0);
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $30;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$i>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 8|0);
  $40 = (($39) + ($37<<4)|0);
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_sqlite3ExprListDup($36,$42,0)|0);
  $44 = HEAP32[$i>>2]|0;
  $45 = HEAP32[$pRet>>2]|0;
  $46 = ((($45)) + 8|0);
  $47 = (($46) + ($44<<4)|0);
  $48 = ((($47)) + 4|0);
  HEAP32[$48>>2] = $43;
  $49 = HEAP32[$0>>2]|0;
  $50 = HEAP32[$i>>2]|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = (($52) + ($50<<4)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3DbStrDup($49,$54)|0);
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$pRet>>2]|0;
  $58 = ((($57)) + 8|0);
  $59 = (($58) + ($56<<4)|0);
  HEAP32[$59>>2] = $55;
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3IdListDup($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pNew = 0, $pNewItem = 0, $pOldItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pNew = sp + 12|0;
 $i = sp + 8|0;
 $pNewItem = sp + 4|0;
 $pOldItem = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3DbMallocRawNN($5,8,0)|0);
 HEAP32[$pNew>>2] = $6;
 $7 = HEAP32[$pNew>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pNew>>2]|0;
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17<<3;
 $19 = (_sqlite3DbMallocRawNN($14,$18,0)|0);
 $20 = HEAP32[$pNew>>2]|0;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$pNew>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($24,$25);
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($31<<3)|0);
  HEAP32[$pNewItem>>2] = $34;
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($35<<3)|0);
  HEAP32[$pOldItem>>2] = $38;
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$pOldItem>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (_sqlite3DbStrDup($39,$41)|0);
  $43 = HEAP32[$pNewItem>>2]|0;
  HEAP32[$43>>2] = $42;
  $44 = HEAP32[$pOldItem>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$pNewItem>>2]|0;
  $48 = ((($47)) + 4|0);
  HEAP32[$48>>2] = $46;
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3BinaryCompareCollSeq($pParse,$pLeft,$pRight) {
 $pParse = $pParse|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLeft;
 HEAP32[$2>>2] = $pRight;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 256;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (_sqlite3ExprCollSeq($8,$9)|0);
  HEAP32[$pColl>>2] = $10;
  $29 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 256;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_sqlite3ExprCollSeq($18,$19)|0);
   HEAP32[$pColl>>2] = $20;
   $29 = HEAP32[$pColl>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (_sqlite3ExprCollSeq($21,$22)|0);
 HEAP32[$pColl>>2] = $23;
 $24 = HEAP32[$pColl>>2]|0;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $29 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (_sqlite3ExprCollSeq($26,$27)|0);
 HEAP32[$pColl>>2] = $28;
 $29 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _binaryCompareP5($pExpr1,$pExpr2,$jumpIfNull) {
 $pExpr1 = $pExpr1|0;
 $pExpr2 = $pExpr2|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 $aff = sp + 12|0;
 HEAP32[$0>>2] = $pExpr1;
 HEAP32[$1>>2] = $pExpr2;
 HEAP32[$2>>2] = $jumpIfNull;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP8[$aff>>0]|0;
 $7 = (_sqlite3CompareAffinity($5,$6)|0);
 $8 = $7&255;
 $9 = HEAP32[$2>>2]|0;
 $10 = $9&255;
 $11 = $10&255;
 $12 = $8 | $11;
 $13 = $12&255;
 HEAP8[$aff>>0] = $13;
 $14 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3ExprAffinity($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $op = 0, $or$cond = 0;
 var $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $op = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3ExprSkipCollate($2)|0);
 HEAP32[$1>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 512;
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP8[$0>>0] = 0;
  $56 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($56|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$op>>2] = $11;
 $12 = HEAP32[$op>>2]|0;
 $13 = ($12|0)==(119);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3ExprAffinity($20)|0);
  HEAP8[$0>>0] = $21;
  $56 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($56|0);
 }
 $22 = HEAP32[$op>>2]|0;
 $23 = ($22|0)==(38);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_sqlite3AffinityType($26,0)|0);
  HEAP8[$0>>0] = $27;
  $56 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($56|0);
 }
 $28 = HEAP32[$op>>2]|0;
 $29 = ($28|0)==(154);
 $30 = HEAP32[$op>>2]|0;
 $31 = ($30|0)==(152);
 $or$cond = $29 | $31;
 $32 = HEAP32[$op>>2]|0;
 $33 = ($32|0)==(157);
 $or$cond3 = $or$cond | $33;
 if ($or$cond3) {
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 44|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = ((($38)) + 32|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40 << 16 >> 16;
   HEAP32[$j>>2] = $41;
   $42 = HEAP32[$j>>2]|0;
   $43 = ($42|0)<(0);
   if ($43) {
    HEAP8[$0>>0] = 68;
    $56 = HEAP8[$0>>0]|0;
    STACKTOP = sp;return ($56|0);
   } else {
    $44 = HEAP32[$j>>2]|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = ((($45)) + 44|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = ((($47)) + 4|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49) + (($44*24)|0)|0);
    $51 = ((($50)) + 21|0);
    $52 = HEAP8[$51>>0]|0;
    HEAP8[$0>>0] = $52;
    $56 = HEAP8[$0>>0]|0;
    STACKTOP = sp;return ($56|0);
   }
  }
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = ((($53)) + 1|0);
 $55 = HEAP8[$54>>0]|0;
 HEAP8[$0>>0] = $55;
 $56 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($56|0);
}
function _sqlite3CompareAffinity($pExpr,$aff2) {
 $pExpr = $pExpr|0;
 $aff2 = $aff2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff1 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 6|0;
 $1 = sp;
 $2 = sp + 5|0;
 $aff1 = sp + 4|0;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $aff2;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff1>>0] = $4;
 $5 = HEAP8[$aff1>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP8[$2>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP8[$aff1>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = ($12|0)>=(67);
   if (!($13)) {
    $14 = HEAP8[$2>>0]|0;
    $15 = $14 << 24 >> 24;
    $16 = ($15|0)>=(67);
    if (!($16)) {
     HEAP8[$0>>0] = 65;
     $27 = HEAP8[$0>>0]|0;
     STACKTOP = sp;return ($27|0);
    }
   }
   HEAP8[$0>>0] = 67;
   $27 = HEAP8[$0>>0]|0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $17 = HEAP8[$aff1>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 $19 = HEAP8[$2>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 $or$cond = $18 | $20;
 if ($or$cond) {
  $21 = HEAP8[$aff1>>0]|0;
  $22 = $21 << 24 >> 24;
  $23 = HEAP8[$2>>0]|0;
  $24 = $23 << 24 >> 24;
  $25 = (($22) + ($24))|0;
  $26 = $25&255;
  HEAP8[$0>>0] = $26;
  $27 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  HEAP8[$0>>0] = 65;
  $27 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _cacheEntryClear($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 6|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 19|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)<(8);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 19|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = (($16) + 1)<<24>>24;
  HEAP8[$15>>0] = $17;
  $18 = $16&255;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 28|0);
  $21 = (($20) + ($18<<2)|0);
  HEAP32[$21>>2] = $13;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 6|0);
 HEAP8[$23>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3CheckCollSeq($pParse,$pColl) {
 $pParse = $pParse|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $p = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zName = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pColl;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$zName>>2] = $6;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$db>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$db>>2]|0;
  $11 = ((($10)) + 62|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$zName>>2]|0;
  $15 = (_sqlite3GetCollSeq($9,$12,$13,$14)|0);
  HEAP32[$p>>2] = $15;
  $16 = HEAP32[$p>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   HEAP32[$0>>2] = 1;
   $18 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($18|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $18 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3CodeOnce($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$v>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 84|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 $8 = (_sqlite3VdbeAddOp1($3,44,$6)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3SelectDestInit($pDest,$eDest,$iParm) {
 $pDest = $pDest|0;
 $eDest = $eDest|0;
 $iParm = $iParm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pDest;
 HEAP32[$1>>2] = $eDest;
 HEAP32[$2>>2] = $iParm;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3&255;
 $5 = HEAP32[$0>>2]|0;
 HEAP8[$5>>0] = $4;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 1|0);
 HEAP8[$10>>0] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 12|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ExprIsInteger($p,$pValue) {
 $p = $p|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pValue;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1024;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  HEAP32[$11>>2] = $10;
  HEAP32[$0>>2] = 1;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $12 = HEAP8[$8>>0]|0;
 $13 = $12&255;
 switch ($13|0) {
 case 156:  {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_sqlite3ExprIsInteger($16,$17)|0);
  HEAP32[$rc>>2] = $18;
  break;
 }
 case 155:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3ExprIsInteger($21,$v)|0);
  $23 = ($22|0)!=(0);
  if ($23) {
   $24 = HEAP32[$v>>2]|0;
   $25 = (0 - ($24))|0;
   $26 = HEAP32[$2>>2]|0;
   HEAP32[$26>>2] = $25;
   HEAP32[$rc>>2] = 1;
  }
  break;
 }
 default: {
 }
 }
 $27 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3VdbeCurrentAddr($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3SetHasNullFlag($v,$iCur,$regHasNull) {
 $v = $v|0;
 $iCur = $iCur|0;
 $regHasNull = $regHasNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $addr1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $regHasNull;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($3,22,0,$4)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3VdbeAddOp1($5,108,$6)|0);
 HEAP32[$addr1>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($8,47,$9,0,$10)|0);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP5($11,-128);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($12,$13);
 STACKTOP = sp;return;
}
function _sqlite3FindInIndex($pParse,$pX,$inFlags,$prRhsHasNull) {
 $pParse = $pParse|0;
 $pX = $pX|0;
 $inFlags = $inFlags|0;
 $prRhsHasNull = $prRhsHasNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $affinity_ok = 0, $db = 0, $eType = 0, $iAddr = 0, $iAddr1 = 0, $iCol = 0, $iDb = 0, $iTab = 0;
 var $mustBeUnique = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pExpr = 0, $pIdx = 0, $pReq = 0, $pTab = 0, $rMayHaveNull = 0, $savedNQueryLoop = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $p = sp + 56|0;
 $eType = sp + 52|0;
 $iTab = sp + 48|0;
 $mustBeUnique = sp + 44|0;
 $v = sp + 40|0;
 $db = sp + 36|0;
 $pTab = sp + 32|0;
 $pExpr = sp + 28|0;
 $iCol = sp + 78|0;
 $iDb = sp + 76|0;
 $iAddr = sp + 24|0;
 $pIdx = sp + 20|0;
 $pReq = sp + 16|0;
 $affinity_ok = sp + 12|0;
 $iAddr1 = sp + 8|0;
 $savedNQueryLoop = sp + 4|0;
 $rMayHaveNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pX;
 HEAP32[$2>>2] = $inFlags;
 HEAP32[$3>>2] = $prRhsHasNull;
 HEAP32[$eType>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 72|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 HEAP32[$iTab>>2] = $6;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3GetVdbe($8)|0);
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = $10 & 4;
 $12 = ($11|0)!=(0);
 $13 = $12&1;
 HEAP32[$mustBeUnique>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $16 & 2048;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 20|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21;
 } else {
  $22 = 0;
 }
 HEAP32[$p>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 68|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)==(0);
 L4: do {
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = (_isCandidateForInOpt($27)|0);
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$30>>2]|0;
    HEAP32[$db>>2] = $31;
    $32 = HEAP32[$p>>2]|0;
    $33 = ((($32)) + 32|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = ((($34)) + 8|0);
    $36 = ((($35)) + 16|0);
    $37 = HEAP32[$36>>2]|0;
    HEAP32[$pTab>>2] = $37;
    $38 = HEAP32[$p>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = ((($39)) + 4|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$pExpr>>2] = $42;
    $43 = HEAP32[$pExpr>>2]|0;
    $44 = ((($43)) + 32|0);
    $45 = HEAP16[$44>>1]|0;
    HEAP16[$iCol>>1] = $45;
    $46 = HEAP32[$db>>2]|0;
    $47 = HEAP32[$pTab>>2]|0;
    $48 = ((($47)) + 64|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (_sqlite3SchemaToIndex($46,$49)|0);
    $51 = $50&65535;
    HEAP16[$iDb>>1] = $51;
    $52 = HEAP32[$0>>2]|0;
    $53 = HEAP16[$iDb>>1]|0;
    $54 = $53 << 16 >> 16;
    _sqlite3CodeVerifySchema($52,$54);
    $55 = HEAP32[$0>>2]|0;
    $56 = HEAP16[$iDb>>1]|0;
    $57 = $56 << 16 >> 16;
    $58 = HEAP32[$pTab>>2]|0;
    $59 = ((($58)) + 28|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$pTab>>2]|0;
    $62 = HEAP32[$61>>2]|0;
    _sqlite3TableLock($55,$57,$60,0,$62);
    $63 = HEAP16[$iCol>>1]|0;
    $64 = $63 << 16 >> 16;
    $65 = ($64|0)<(0);
    $66 = HEAP32[$0>>2]|0;
    if ($65) {
     $67 = (_sqlite3CodeOnce($66)|0);
     HEAP32[$iAddr>>2] = $67;
     $68 = HEAP32[$0>>2]|0;
     $69 = HEAP32[$iTab>>2]|0;
     $70 = HEAP16[$iDb>>1]|0;
     $71 = $70 << 16 >> 16;
     $72 = HEAP32[$pTab>>2]|0;
     _sqlite3OpenTable($68,$69,$71,$72,54);
     HEAP32[$eType>>2] = 1;
     $73 = HEAP32[$v>>2]|0;
     $74 = HEAP32[$iAddr>>2]|0;
     _sqlite3VdbeJumpHere($73,$74);
     break;
    }
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 12|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$pExpr>>2]|0;
    $79 = (_sqlite3BinaryCompareCollSeq($66,$77,$78)|0);
    HEAP32[$pReq>>2] = $79;
    $80 = HEAP32[$1>>2]|0;
    $81 = HEAP16[$iCol>>1]|0;
    $82 = $81 << 16 >> 16;
    $83 = HEAP32[$pTab>>2]|0;
    $84 = ((($83)) + 4|0);
    $85 = HEAP32[$84>>2]|0;
    $86 = (($85) + (($82*24)|0)|0);
    $87 = ((($86)) + 21|0);
    $88 = HEAP8[$87>>0]|0;
    $89 = (_sqlite3IndexAffinityOk($80,$88)|0);
    HEAP32[$affinity_ok>>2] = $89;
    $90 = HEAP32[$pTab>>2]|0;
    $91 = ((($90)) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    HEAP32[$pIdx>>2] = $92;
    while(1) {
     $93 = HEAP32[$pIdx>>2]|0;
     $94 = ($93|0)!=(0|0);
     $95 = HEAP32[$eType>>2]|0;
     $96 = ($95|0)==(0);
     $or$cond = $94 & $96;
     $97 = HEAP32[$affinity_ok>>2]|0;
     $98 = ($97|0)!=(0);
     $or$cond3 = $or$cond & $98;
     if (!($or$cond3)) {
      break L4;
     }
     $99 = HEAP32[$pIdx>>2]|0;
     $100 = ((($99)) + 4|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = HEAP16[$101>>1]|0;
     $103 = $102 << 16 >> 16;
     $104 = HEAP16[$iCol>>1]|0;
     $105 = $104 << 16 >> 16;
     $106 = ($103|0)==($105|0);
     do {
      if ($106) {
       $107 = HEAP32[$db>>2]|0;
       $108 = HEAP32[$db>>2]|0;
       $109 = ((($108)) + 62|0);
       $110 = HEAP8[$109>>0]|0;
       $111 = HEAP32[$pIdx>>2]|0;
       $112 = ((($111)) + 32|0);
       $113 = HEAP32[$112>>2]|0;
       $114 = HEAP32[$113>>2]|0;
       $115 = (_sqlite3FindCollSeq($107,$110,$114,0)|0);
       $116 = HEAP32[$pReq>>2]|0;
       $117 = ($115|0)==($116|0);
       if ($117) {
        $118 = HEAP32[$mustBeUnique>>2]|0;
        $119 = ($118|0)!=(0);
        if ($119) {
         $120 = HEAP32[$pIdx>>2]|0;
         $121 = ((($120)) + 50|0);
         $122 = HEAP16[$121>>1]|0;
         $123 = $122&65535;
         $124 = ($123|0)==(1);
         if (!($124)) {
          break;
         }
         $125 = HEAP32[$pIdx>>2]|0;
         $126 = ((($125)) + 54|0);
         $127 = HEAP8[$126>>0]|0;
         $128 = $127&255;
         $129 = ($128|0)!=(0);
         if (!($129)) {
          break;
         }
        }
        $130 = HEAP32[$0>>2]|0;
        $131 = (_sqlite3CodeOnce($130)|0);
        HEAP32[$iAddr1>>2] = $131;
        $132 = HEAP32[$v>>2]|0;
        $133 = HEAP32[$iTab>>2]|0;
        $134 = HEAP32[$pIdx>>2]|0;
        $135 = ((($134)) + 44|0);
        $136 = HEAP32[$135>>2]|0;
        $137 = HEAP16[$iDb>>1]|0;
        $138 = $137 << 16 >> 16;
        (_sqlite3VdbeAddOp3($132,54,$133,$136,$138)|0);
        $139 = HEAP32[$0>>2]|0;
        $140 = HEAP32[$pIdx>>2]|0;
        _sqlite3VdbeSetP4KeyInfo($139,$140);
        $141 = HEAP32[$pIdx>>2]|0;
        $142 = ((($141)) + 28|0);
        $143 = HEAP32[$142>>2]|0;
        $144 = HEAP8[$143>>0]|0;
        $145 = $144&255;
        $146 = (3 + ($145))|0;
        HEAP32[$eType>>2] = $146;
        $147 = HEAP32[$3>>2]|0;
        $148 = ($147|0)!=(0|0);
        if ($148) {
         $149 = HEAP16[$iCol>>1]|0;
         $150 = $149 << 16 >> 16;
         $151 = HEAP32[$pTab>>2]|0;
         $152 = ((($151)) + 4|0);
         $153 = HEAP32[$152>>2]|0;
         $154 = (($153) + (($150*24)|0)|0);
         $155 = ((($154)) + 20|0);
         $156 = HEAP8[$155>>0]|0;
         $157 = ($156<<24>>24)!=(0);
         if (!($157)) {
          $158 = HEAP32[$0>>2]|0;
          $159 = ((($158)) + 76|0);
          $160 = HEAP32[$159>>2]|0;
          $161 = (($160) + 1)|0;
          HEAP32[$159>>2] = $161;
          $162 = HEAP32[$3>>2]|0;
          HEAP32[$162>>2] = $161;
          $163 = HEAP32[$v>>2]|0;
          $164 = HEAP32[$iTab>>2]|0;
          $165 = HEAP32[$3>>2]|0;
          $166 = HEAP32[$165>>2]|0;
          _sqlite3SetHasNullFlag($163,$164,$166);
         }
        }
        $167 = HEAP32[$v>>2]|0;
        $168 = HEAP32[$iAddr1>>2]|0;
        _sqlite3VdbeJumpHere($167,$168);
       }
      }
     } while(0);
     $169 = HEAP32[$pIdx>>2]|0;
     $170 = ((($169)) + 20|0);
     $171 = HEAP32[$170>>2]|0;
     HEAP32[$pIdx>>2] = $171;
    }
   }
  }
 } while(0);
 $172 = HEAP32[$eType>>2]|0;
 $173 = ($172|0)==(0);
 do {
  if ($173) {
   $174 = HEAP32[$2>>2]|0;
   $175 = $174 & 1;
   $176 = ($175|0)!=(0);
   if ($176) {
    $177 = HEAP32[$1>>2]|0;
    $178 = ((($177)) + 4|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = $179 & 2048;
    $181 = ($180|0)!=(0);
    if (!($181)) {
     $182 = HEAP32[$1>>2]|0;
     $183 = (_sqlite3InRhsIsConstant($182)|0);
     $184 = ($183|0)!=(0);
     if ($184) {
      $185 = HEAP32[$1>>2]|0;
      $186 = ((($185)) + 20|0);
      $187 = HEAP32[$186>>2]|0;
      $188 = HEAP32[$187>>2]|0;
      $189 = ($188|0)<=(2);
      if (!($189)) {
       break;
      }
     }
     HEAP32[$eType>>2] = 5;
    }
   }
  }
 } while(0);
 $190 = HEAP32[$eType>>2]|0;
 $191 = ($190|0)==(0);
 if (!($191)) {
  $228 = HEAP32[$iTab>>2]|0;
  $229 = HEAP32[$1>>2]|0;
  $230 = ((($229)) + 28|0);
  HEAP32[$230>>2] = $228;
  $231 = HEAP32[$eType>>2]|0;
  STACKTOP = sp;return ($231|0);
 }
 $192 = HEAP32[$0>>2]|0;
 $193 = ((($192)) + 428|0);
 $194 = HEAP32[$193>>2]|0;
 HEAP32[$savedNQueryLoop>>2] = $194;
 HEAP32[$rMayHaveNull>>2] = 0;
 HEAP32[$eType>>2] = 2;
 $195 = HEAP32[$2>>2]|0;
 $196 = $195 & 4;
 $197 = ($196|0)!=(0);
 if ($197) {
  $198 = HEAP32[$0>>2]|0;
  $199 = ((($198)) + 428|0);
  HEAP32[$199>>2] = 0;
  $200 = HEAP32[$1>>2]|0;
  $201 = ((($200)) + 12|0);
  $202 = HEAP32[$201>>2]|0;
  $203 = ((($202)) + 32|0);
  $204 = HEAP16[$203>>1]|0;
  $205 = $204 << 16 >> 16;
  $206 = ($205|0)<(0);
  if ($206) {
   $207 = HEAP32[$1>>2]|0;
   $208 = ((($207)) + 4|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = $209 & 2048;
   $211 = ($210|0)!=(0);
   if (!($211)) {
    HEAP32[$eType>>2] = 1;
   }
  }
 } else {
  $212 = HEAP32[$3>>2]|0;
  $213 = ($212|0)!=(0|0);
  if ($213) {
   $214 = HEAP32[$0>>2]|0;
   $215 = ((($214)) + 76|0);
   $216 = HEAP32[$215>>2]|0;
   $217 = (($216) + 1)|0;
   HEAP32[$215>>2] = $217;
   HEAP32[$rMayHaveNull>>2] = $217;
   $218 = HEAP32[$3>>2]|0;
   HEAP32[$218>>2] = $217;
  }
 }
 $219 = HEAP32[$0>>2]|0;
 $220 = HEAP32[$1>>2]|0;
 $221 = HEAP32[$rMayHaveNull>>2]|0;
 $222 = HEAP32[$eType>>2]|0;
 $223 = ($222|0)==(1);
 $224 = $223&1;
 (_sqlite3CodeSubselect($219,$220,$221,$224)|0);
 $225 = HEAP32[$savedNQueryLoop>>2]|0;
 $226 = HEAP32[$0>>2]|0;
 $227 = ((($226)) + 428|0);
 HEAP32[$227>>2] = $225;
 $231 = HEAP32[$eType>>2]|0;
 STACKTOP = sp;return ($231|0);
}
function _comparisonAffinity($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $aff = sp + 4|0;
 HEAP32[$0>>2] = $pExpr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP8[$aff>>0]|0;
  $13 = (_sqlite3CompareAffinity($11,$12)|0);
  HEAP8[$aff>>0] = $13;
  $29 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($29|0);
 }
 $14 = ((($9)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 & 2048;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP8[$aff>>0]|0;
  $26 = (_sqlite3CompareAffinity($24,$25)|0);
  HEAP8[$aff>>0] = $26;
  $29 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($29|0);
 }
 $27 = HEAP8[$aff>>0]|0;
 $28 = ($27<<24>>24)!=(0);
 if ($28) {
  $29 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($29|0);
 }
 HEAP8[$aff>>0] = 65;
 $29 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3ExprCanBeNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $p;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = HEAP8[$2>>0]|0;
  $4 = $3&255;
  $5 = ($4|0)==(156);
  if ($5) {
   $49 = 1;
  } else {
   $6 = HEAP32[$1>>2]|0;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7&255;
   $9 = ($8|0)==(155);
   $49 = $9;
  }
  $10 = HEAP32[$1>>2]|0;
  if (!($49)) {
   break;
  }
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$1>>2] = $12;
 }
 $13 = HEAP8[$10>>0]|0;
 HEAP8[$op>>0] = $13;
 $14 = HEAP8[$op>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(157);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 38|0);
  $19 = HEAP8[$18>>0]|0;
  HEAP8[$op>>0] = $19;
 }
 $20 = HEAP8[$op>>0]|0;
 $21 = $20&255;
 switch ($21|0) {
 case 134: case 133: case 97: case 132:  {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 152:  {
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 1048576;
  $26 = ($25|0)!=(0);
  if ($26) {
   $47 = 1;
  } else {
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = ($30|0)>=(0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 32|0);
    $34 = HEAP16[$33>>1]|0;
    $35 = $34 << 16 >> 16;
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 44|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ((($38)) + 4|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + (($35*24)|0)|0);
    $42 = ((($41)) + 20|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)==(0);
    $47 = $45;
   } else {
    $47 = 0;
   }
  }
  $46 = $47&1;
  HEAP32[$0>>2] = $46;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 default: {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 }
 return (0)|0;
}
function _isCandidateForInOpt($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pEList = 0, $pSrc = 0;
 var $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pSrc = sp + 8|0;
 $pEList = sp + 4|0;
 $pTab = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 do {
  if ($3) {
   HEAP32[$0>>2] = 0;
  } else {
   $4 = HEAP32[$1>>2]|0;
   $5 = ((($4)) + 52|0);
   $6 = HEAP32[$5>>2]|0;
   $7 = ($6|0)!=(0|0);
   if ($7) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 6|0);
   $10 = HEAP16[$9>>1]|0;
   $11 = $10&65535;
   $12 = $11 & 9;
   $13 = ($12|0)!=(0);
   if ($13) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 60|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 36|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 32|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$pSrc>>2] = $24;
   $25 = HEAP32[$pSrc>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(1);
   if ($27) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $28 = HEAP32[$pSrc>>2]|0;
   $29 = ((($28)) + 8|0);
   $30 = ((($29)) + 20|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $33 = HEAP32[$pSrc>>2]|0;
   $34 = ((($33)) + 8|0);
   $35 = ((($34)) + 16|0);
   $36 = HEAP32[$35>>2]|0;
   HEAP32[$pTab>>2] = $36;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = ($37|0)==(0|0);
   if ($38) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $39 = HEAP32[$pTab>>2]|0;
   $40 = ((($39)) + 42|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = $42 & 16;
   $44 = ($43|0)!=(0);
   if ($44) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$pEList>>2] = $46;
   $47 = HEAP32[$pEList>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = ($48|0)!=(1);
   if ($49) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $50 = HEAP32[$pEList>>2]|0;
   $51 = ((($50)) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54&255;
   $56 = ($55|0)!=(152);
   if ($56) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    HEAP32[$0>>2] = 1;
    break;
   }
  }
 } while(0);
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _sqlite3CodeVerifySchema($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pToplevel = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 416|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 416|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8;
 } else {
  $9 = $6;
 }
 HEAP32[$pToplevel>>2] = $9;
 $10 = HEAP32[$pToplevel>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$pToplevel>>2]|0;
 $13 = ((($12)) + 340|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = 1 << $15;
 $17 = $14 & $16;
 $18 = ($17|0)!=(0);
 $19 = $18&1;
 $20 = ($19|0)==(0);
 if (!($20)) {
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = 1 << $21;
 $23 = HEAP32[$pToplevel>>2]|0;
 $24 = ((($23)) + 340|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 | $22;
 HEAP32[$24>>2] = $26;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$db>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + ($27<<4)|0);
 $32 = ((($31)) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$pToplevel>>2]|0;
 $37 = ((($36)) + 344|0);
 $38 = (($37) + ($35<<2)|0);
 HEAP32[$38>>2] = $34;
 $39 = HEAP32[$1>>2]|0;
 $40 = ($39|0)==(1);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pToplevel>>2]|0;
 (_sqlite3OpenTempDatabase($41)|0);
 STACKTOP = sp;return;
}
function _sqlite3IndexAffinityOk($pExpr,$idx_affinity) {
 $pExpr = $pExpr|0;
 $idx_affinity = $idx_affinity|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 9|0;
 $aff = sp + 8|0;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $idx_affinity;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_comparisonAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP8[$aff>>0]|0;
 $6 = $5 << 24 >> 24;
 switch ($6|0) {
 case 65:  {
  HEAP32[$0>>2] = 1;
  break;
 }
 case 66:  {
  $7 = HEAP8[$2>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)==(66);
  $10 = $9&1;
  HEAP32[$0>>2] = $10;
  break;
 }
 default: {
  $11 = HEAP8[$2>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)>=(67);
  $14 = $13&1;
  HEAP32[$0>>2] = $14;
 }
 }
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3InRhsIsConstant($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pLHS = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pLHS = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pIn;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pLHS>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3ExprIsConstant($6)|0);
 HEAP32[$res>>2] = $7;
 $8 = HEAP32[$pLHS>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3ExprIfTrue($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d2 = 0, $destIfFalse = 0, $destIfNull = 0, $op = 0, $or$cond = 0, $r1 = 0, $r2 = 0, $regFree1 = 0, $regFree2 = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $v = sp + 32|0;
 $op = sp + 28|0;
 $regFree1 = sp + 24|0;
 $regFree2 = sp + 20|0;
 $r1 = sp + 16|0;
 $r2 = sp + 12|0;
 $d2 = sp + 8|0;
 $destIfFalse = sp + 4|0;
 $destIfNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 HEAP32[$op>>2] = 0;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$op>>2] = $13;
 $14 = HEAP32[$op>>2]|0;
 L4: do {
  switch ($14|0) {
  case 72:  {
   $15 = HEAP32[$v>>2]|0;
   $16 = (_sqlite3VdbeMakeLabel($15)|0);
   HEAP32[$d2>>2] = $16;
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$d2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = $22 ^ 16;
   _sqlite3ExprIfFalse($17,$20,$21,$23);
   $24 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($24);
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($25,$28,$29,$30);
   $31 = HEAP32[$v>>2]|0;
   $32 = HEAP32[$d2>>2]|0;
   _sqlite3VdbeResolveLabel($31,$32);
   $33 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($33);
   break;
  }
  case 71:  {
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($34,$37,$38,$39);
   $40 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($40);
   $41 = HEAP32[$0>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$2>>2]|0;
   $46 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($41,$44,$45,$46);
   $47 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($47);
   break;
  }
  case 19:  {
   $48 = HEAP32[$0>>2]|0;
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$2>>2]|0;
   $53 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($48,$51,$52,$53);
   break;
  }
  case 79: case 78: case 83: case 80: case 81: case 82:  {
   $54 = HEAP32[$0>>2]|0;
   $55 = HEAP32[$1>>2]|0;
   $56 = ((($55)) + 12|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (_sqlite3ExprCodeTemp($54,$57,$regFree1)|0);
   HEAP32[$r1>>2] = $58;
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (_sqlite3ExprCodeTemp($59,$62,$regFree2)|0);
   HEAP32[$r2>>2] = $63;
   $64 = HEAP32[$0>>2]|0;
   $65 = HEAP32[$1>>2]|0;
   $66 = ((($65)) + 12|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = HEAP32[$1>>2]|0;
   $69 = ((($68)) + 16|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$op>>2]|0;
   $72 = HEAP32[$r1>>2]|0;
   $73 = HEAP32[$r2>>2]|0;
   $74 = HEAP32[$2>>2]|0;
   $75 = HEAP32[$3>>2]|0;
   (_codeCompare($64,$67,$70,$71,$72,$73,$74,$75)|0);
   break;
  }
  case 148: case 73:  {
   $76 = HEAP32[$0>>2]|0;
   $77 = HEAP32[$1>>2]|0;
   $78 = ((($77)) + 12|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (_sqlite3ExprCodeTemp($76,$79,$regFree1)|0);
   HEAP32[$r1>>2] = $80;
   $81 = HEAP32[$0>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = ((($82)) + 16|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (_sqlite3ExprCodeTemp($81,$84,$regFree2)|0);
   HEAP32[$r2>>2] = $85;
   $86 = HEAP32[$op>>2]|0;
   $87 = ($86|0)==(73);
   $88 = $87 ? 79 : 78;
   HEAP32[$op>>2] = $88;
   $89 = HEAP32[$0>>2]|0;
   $90 = HEAP32[$1>>2]|0;
   $91 = ((($90)) + 12|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 16|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = HEAP32[$op>>2]|0;
   $97 = HEAP32[$r1>>2]|0;
   $98 = HEAP32[$r2>>2]|0;
   $99 = HEAP32[$2>>2]|0;
   (_codeCompare($89,$92,$95,$96,$97,$98,$99,128)|0);
   break;
  }
  case 77: case 76:  {
   $100 = HEAP32[$0>>2]|0;
   $101 = HEAP32[$1>>2]|0;
   $102 = ((($101)) + 12|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (_sqlite3ExprCodeTemp($100,$103,$regFree1)|0);
   HEAP32[$r1>>2] = $104;
   $105 = HEAP32[$v>>2]|0;
   $106 = HEAP32[$op>>2]|0;
   $107 = HEAP32[$r1>>2]|0;
   $108 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($105,$106,$107,$108)|0);
   break;
  }
  case 74:  {
   $109 = HEAP32[$0>>2]|0;
   $110 = HEAP32[$1>>2]|0;
   $111 = HEAP32[$2>>2]|0;
   $112 = HEAP32[$3>>2]|0;
   _exprCodeBetween($109,$110,$111,1,$112);
   break;
  }
  case 75:  {
   $113 = HEAP32[$v>>2]|0;
   $114 = (_sqlite3VdbeMakeLabel($113)|0);
   HEAP32[$destIfFalse>>2] = $114;
   $115 = HEAP32[$3>>2]|0;
   $116 = ($115|0)!=(0);
   $117 = HEAP32[$2>>2]|0;
   $118 = HEAP32[$destIfFalse>>2]|0;
   $119 = $116 ? $117 : $118;
   HEAP32[$destIfNull>>2] = $119;
   $120 = HEAP32[$0>>2]|0;
   $121 = HEAP32[$1>>2]|0;
   $122 = HEAP32[$destIfFalse>>2]|0;
   $123 = HEAP32[$destIfNull>>2]|0;
   _sqlite3ExprCodeIN($120,$121,$122,$123);
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeGoto($124,$125)|0);
   $126 = HEAP32[$v>>2]|0;
   $127 = HEAP32[$destIfFalse>>2]|0;
   _sqlite3VdbeResolveLabel($126,$127);
   break;
  }
  default: {
   $128 = HEAP32[$1>>2]|0;
   $129 = (_exprAlwaysTrue($128)|0);
   $130 = ($129|0)!=(0);
   if ($130) {
    $131 = HEAP32[$v>>2]|0;
    $132 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeGoto($131,$132)|0);
    break L4;
   }
   $133 = HEAP32[$1>>2]|0;
   $134 = (_exprAlwaysFalse($133)|0);
   $135 = ($134|0)!=(0);
   if (!($135)) {
    $136 = HEAP32[$0>>2]|0;
    $137 = HEAP32[$1>>2]|0;
    $138 = (_sqlite3ExprCodeTemp($136,$137,$regFree1)|0);
    HEAP32[$r1>>2] = $138;
    $139 = HEAP32[$v>>2]|0;
    $140 = HEAP32[$r1>>2]|0;
    $141 = HEAP32[$2>>2]|0;
    $142 = HEAP32[$3>>2]|0;
    $143 = ($142|0)!=(0);
    $144 = $143&1;
    (_sqlite3VdbeAddOp3($139,45,$140,$141,$144)|0);
   }
  }
  }
 } while(0);
 $145 = HEAP32[$0>>2]|0;
 $146 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($145,$146);
 $147 = HEAP32[$0>>2]|0;
 $148 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($147,$148);
 STACKTOP = sp;return;
}
function _exprCodeBetween($pParse,$pExpr,$dest,$jumpIfTrue,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfTrue = $jumpIfTrue|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $compLeft = 0, $compRight = 0, $exprAnd = 0, $exprX = 0;
 var $regFree1 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 216|0;
 $1 = sp + 212|0;
 $2 = sp + 208|0;
 $3 = sp + 204|0;
 $4 = sp + 200|0;
 $exprAnd = sp + 152|0;
 $compLeft = sp + 104|0;
 $compRight = sp + 56|0;
 $exprX = sp + 8|0;
 $regFree1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfTrue;
 HEAP32[$4>>2] = $jumpIfNull;
 HEAP32[$regFree1>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 dest=$exprX; src=$7; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 HEAP8[$exprAnd>>0] = 72;
 $8 = ((($exprAnd)) + 12|0);
 HEAP32[$8>>2] = $compLeft;
 $9 = ((($exprAnd)) + 16|0);
 HEAP32[$9>>2] = $compRight;
 HEAP8[$compLeft>>0] = 83;
 $10 = ((($compLeft)) + 12|0);
 HEAP32[$10>>2] = $exprX;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($compLeft)) + 16|0);
 HEAP32[$17>>2] = $16;
 HEAP8[$compRight>>0] = 81;
 $18 = ((($compRight)) + 12|0);
 HEAP32[$18>>2] = $exprX;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ((($compRight)) + 16|0);
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$0>>2]|0;
 $28 = (_sqlite3ExprCodeTemp($27,$exprX,$regFree1)|0);
 _exprToRegister($exprX,$28);
 $29 = HEAP32[$3>>2]|0;
 $30 = ($29|0)!=(0);
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$2>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 if ($30) {
  _sqlite3ExprIfTrue($31,$exprAnd,$32,$33);
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($34,$35);
  STACKTOP = sp;return;
 } else {
  _sqlite3ExprIfFalse($31,$exprAnd,$32,$33);
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($34,$35);
  STACKTOP = sp;return;
 }
}
function _exprAlwaysFalse($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$v>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 1;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 0;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3ExprIsInteger($7,$v)|0);
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$v>>2]|0;
    $11 = ($10|0)==(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _exprAlwaysTrue($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$v>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 1;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 0;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3ExprIsInteger($7,$v)|0);
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$v>>2]|0;
    $11 = ($10|0)!=(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3AuthCheck($pParse,$code,$zArg1,$zArg2,$zArg3) {
 $pParse = $pParse|0;
 $code = $code|0;
 $zArg1 = $zArg1|0;
 $zArg2 = $zArg2|0;
 $zArg3 = $zArg3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $db = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $code;
 HEAP32[$3>>2] = $zArg1;
 HEAP32[$4>>2] = $zArg2;
 HEAP32[$5>>2] = $zArg3;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 140|0);
 $10 = ((($9)) + 5|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 454|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   $19 = HEAP32[$db>>2]|0;
   $20 = ((($19)) + 288|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0|0);
   if ($22) {
    HEAP32[$0>>2] = 0;
    $48 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($48|0);
   }
   $23 = HEAP32[$db>>2]|0;
   $24 = ((($23)) + 288|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$db>>2]|0;
   $27 = ((($26)) + 292|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   $31 = HEAP32[$4>>2]|0;
   $32 = HEAP32[$5>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 496|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (FUNCTION_TABLE_iiiiiii[$25 & 255]($28,$29,$30,$31,$32,$35)|0);
   HEAP32[$rc>>2] = $36;
   $37 = HEAP32[$rc>>2]|0;
   $38 = ($37|0)==(1);
   if ($38) {
    $39 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($39,28619,$vararg_buffer);
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 12|0);
    HEAP32[$41>>2] = 23;
   } else {
    $42 = HEAP32[$rc>>2]|0;
    $43 = ($42|0)!=(0);
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)!=(2);
    $or$cond = $43 & $45;
    if ($or$cond) {
     HEAP32[$rc>>2] = 1;
     $46 = HEAP32[$1>>2]|0;
     _sqliteAuthBadReturnCode($46);
    }
   }
   $47 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $47;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _sqliteAuthBadReturnCode($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($1,28634,$vararg_buffer);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 12|0);
 HEAP32[$3>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3NameFromToken($db,$pName) {
 $db = $db|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3DbStrNDup($4,$6,$9,0)|0);
  HEAP32[$zName>>2] = $10;
  $11 = HEAP32[$zName>>2]|0;
  (_sqlite3Dequote($11)|0);
  $12 = HEAP32[$zName>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  HEAP32[$zName>>2] = 0;
  $12 = HEAP32[$zName>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _sqlite3Dequote($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $quote = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $quote = sp + 16|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = -1;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 HEAP8[$quote>>0] = $5;
 $6 = HEAP8[$quote>>0]|0;
 $7 = $6 << 24 >> 24;
 switch ($7|0) {
 case 96: case 34: case 39:  {
  break;
 }
 case 91:  {
  HEAP8[$quote>>0] = 93;
  break;
 }
 default: {
  HEAP32[$0>>2] = -1;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 }
 HEAP32[$i>>2] = 1;
 HEAP32[$j>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) + ($8)|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = HEAP8[$quote>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = ($12|0)==($14|0);
  $16 = HEAP32[$i>>2]|0;
  if ($15) {
   $17 = (($16) + 1)|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (($18) + ($17)|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = HEAP8[$quote>>0]|0;
   $23 = $22 << 24 >> 24;
   $24 = ($21|0)==($23|0);
   if (!($24)) {
    break;
   }
   $25 = HEAP8[$quote>>0]|0;
   $26 = HEAP32[$j>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$j>>2] = $27;
   $28 = HEAP32[$1>>2]|0;
   $29 = (($28) + ($26)|0);
   HEAP8[$29>>0] = $25;
   $30 = HEAP32[$i>>2]|0;
   $31 = (($30) + 1)|0;
   HEAP32[$i>>2] = $31;
  } else {
   $32 = HEAP32[$1>>2]|0;
   $33 = (($32) + ($16)|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = HEAP32[$j>>2]|0;
   $36 = (($35) + 1)|0;
   HEAP32[$j>>2] = $36;
   $37 = HEAP32[$1>>2]|0;
   $38 = (($37) + ($35)|0);
   HEAP8[$38>>0] = $34;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 1)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$j>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = (($42) + ($41)|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$j>>2]|0;
 HEAP32[$0>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3TwoPartName($pParse,$pName1,$pName2,$pUnqual) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pUnqual = $pUnqual|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $iDb = sp + 16|0;
 $db = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pName1;
 HEAP32[$3>>2] = $pName2;
 HEAP32[$4>>2] = $pUnqual;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9>>>0)>(0);
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 140|0);
 if ($10) {
  $13 = ((($12)) + 5|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = ($14<<24>>24)!=(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($16,28807,$vararg_buffer);
   HEAP32[$0>>2] = -1;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
  $17 = HEAP32[$3>>2]|0;
  $18 = HEAP32[$4>>2]|0;
  HEAP32[$18>>2] = $17;
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (_sqlite3FindDb($19,$20)|0);
  HEAP32[$iDb>>2] = $21;
  $22 = HEAP32[$iDb>>2]|0;
  $23 = ($22|0)<(0);
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $25;
   _sqlite3ErrorMsg($24,28824,$vararg_buffer1);
   HEAP32[$0>>2] = -1;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } else {
  $26 = ((($12)) + 4|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  HEAP32[$iDb>>2] = $28;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  HEAP32[$30>>2] = $29;
 }
 $31 = HEAP32[$iDb>>2]|0;
 HEAP32[$0>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3CheckObjectName($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 140|0);
 $6 = ((($5)) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 18|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = ((($15)) + 24|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $17 & 2048;
   $19 = ($18|0)==(0);
   if ($19) {
    $20 = HEAP32[$2>>2]|0;
    $21 = (_sqlite3_strnicmp($20,28844,7)|0);
    $22 = (0)==($21|0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer>>2] = $24;
     _sqlite3ErrorMsg($23,28852,$vararg_buffer);
     HEAP32[$0>>2] = 1;
     $25 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($25|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3BeginWriteOperation($pParse,$setStatement,$iDb) {
 $pParse = $pParse|0;
 $setStatement = $setStatement|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $setStatement;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 416|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = ((($7)) + 416|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9;
 } else {
  $10 = $7;
 }
 HEAP32[$pToplevel>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 _sqlite3CodeVerifySchema($11,$12);
 $13 = HEAP32[$2>>2]|0;
 $14 = 1 << $13;
 $15 = HEAP32[$pToplevel>>2]|0;
 $16 = ((($15)) + 336|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17 | $14;
 HEAP32[$16>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$pToplevel>>2]|0;
 $21 = ((($20)) + 20|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 | $19;
 $25 = $24&255;
 HEAP8[$21>>0] = $25;
 STACKTOP = sp;return;
}
function _sqlite3OpenMasterTable($p,$iDb) {
 $p = $p|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(1);
 $8 = $7 ? 21260 : 21279;
 _sqlite3TableLock($4,$5,1,1,$8);
 $9 = HEAP32[$v>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp4Int($9,55,0,1,$10,5)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 72|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 72|0);
 HEAP32[$16>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3FindDb($db,$pName) {
 $db = $db|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3NameFromToken($2,$3)|0);
 HEAP32[$zName>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$zName>>2]|0;
 $7 = (_sqlite3FindDbName($5,$6)|0);
 HEAP32[$i>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($8,$9);
 $10 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _convertToWithoutRowidTable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0;
 var $i = 0, $ipkToken = 0, $j = 0, $n = 0, $nPk = 0, $pIdx = 0, $pList = 0, $pPk = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pIdx = sp + 44|0;
 $pPk = sp + 40|0;
 $nPk = sp + 36|0;
 $i = sp + 32|0;
 $j = sp + 28|0;
 $db = sp + 24|0;
 $v = sp + 20|0;
 $pList = sp + 16|0;
 $ipkToken = sp + 8|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 424|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$v>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 424|0);
  $14 = HEAP32[$13>>2]|0;
  _sqlite3VdbeChangeOpcode($11,$14,121);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 32|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 << 16 >> 16;
 $19 = ($18|0)>=(0);
 $20 = HEAP32[$1>>2]|0;
 do {
  if ($19) {
   $21 = ((($20)) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 4|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + (($23*24)|0)|0);
   $28 = HEAP32[$27>>2]|0;
   _sqlite3TokenInit($ipkToken,$28);
   $29 = HEAP32[$0>>2]|0;
   $30 = HEAP32[$db>>2]|0;
   $31 = (_sqlite3ExprAlloc($30,27,$ipkToken,0)|0);
   $32 = (_sqlite3ExprListAppend($29,0,$31)|0);
   HEAP32[$pList>>2] = $32;
   $33 = HEAP32[$pList>>2]|0;
   $34 = ($33|0)==(0|0);
   if ($34) {
    STACKTOP = sp;return;
   }
   $35 = HEAP32[$0>>2]|0;
   $36 = ((($35)) + 452|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = HEAP32[$pList>>2]|0;
   $39 = ((($38)) + 4|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ((($40)) + 12|0);
   HEAP8[$41>>0] = $37;
   $42 = HEAP32[$0>>2]|0;
   $43 = HEAP32[$pList>>2]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = ((($44)) + 43|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = $46&255;
   $48 = (_sqlite3CreateIndex($42,0,0,0,$43,$47,0,0,0,0)|0);
   HEAP32[$pPk>>2] = $48;
   $49 = HEAP32[$pPk>>2]|0;
   $50 = ($49|0)==(0|0);
   if ($50) {
    STACKTOP = sp;return;
   } else {
    $51 = HEAP32[$pPk>>2]|0;
    $52 = ((($51)) + 55|0);
    $53 = HEAP8[$52>>0]|0;
    $54 = $53 & -4;
    $55 = $54 | 2;
    HEAP8[$52>>0] = $55;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 32|0);
    HEAP16[$57>>1] = -1;
    break;
   }
  } else {
   $58 = (_sqlite3PrimaryKeyIndex($20)|0);
   HEAP32[$pPk>>2] = $58;
   $59 = HEAP32[$v>>2]|0;
   $60 = ($59|0)!=(0|0);
   if ($60) {
    $61 = HEAP32[$v>>2]|0;
    $62 = HEAP32[$pPk>>2]|0;
    $63 = ((($62)) + 44|0);
    $64 = HEAP32[$63>>2]|0;
    _sqlite3VdbeChangeOpcode($61,$64,13);
   }
   HEAP32[$j>>2] = 1;
   HEAP32[$i>>2] = 1;
   while(1) {
    $65 = HEAP32[$i>>2]|0;
    $66 = HEAP32[$pPk>>2]|0;
    $67 = ((($66)) + 50|0);
    $68 = HEAP16[$67>>1]|0;
    $69 = $68&65535;
    $70 = ($65|0)<($69|0);
    if (!($70)) {
     break;
    }
    $71 = HEAP32[$pPk>>2]|0;
    $72 = ((($71)) + 4|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = HEAP32[$j>>2]|0;
    $75 = HEAP32[$i>>2]|0;
    $76 = HEAP32[$pPk>>2]|0;
    $77 = ((($76)) + 4|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (($78) + ($75<<1)|0);
    $80 = HEAP16[$79>>1]|0;
    $81 = $80 << 16 >> 16;
    $82 = (_hasColumn($73,$74,$81)|0);
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$pPk>>2]|0;
     $85 = ((($84)) + 52|0);
     $86 = HEAP16[$85>>1]|0;
     $87 = (($86) + -1)<<16>>16;
     HEAP16[$85>>1] = $87;
    } else {
     $88 = HEAP32[$i>>2]|0;
     $89 = HEAP32[$pPk>>2]|0;
     $90 = ((($89)) + 4|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = (($91) + ($88<<1)|0);
     $93 = HEAP16[$92>>1]|0;
     $94 = HEAP32[$j>>2]|0;
     $95 = (($94) + 1)|0;
     HEAP32[$j>>2] = $95;
     $96 = HEAP32[$pPk>>2]|0;
     $97 = ((($96)) + 4|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = (($98) + ($94<<1)|0);
     HEAP16[$99>>1] = $93;
    }
    $100 = HEAP32[$i>>2]|0;
    $101 = (($100) + 1)|0;
    HEAP32[$i>>2] = $101;
   }
   $102 = HEAP32[$j>>2]|0;
   $103 = $102&65535;
   $104 = HEAP32[$pPk>>2]|0;
   $105 = ((($104)) + 50|0);
   HEAP16[$105>>1] = $103;
  }
 } while(0);
 $106 = HEAP32[$pPk>>2]|0;
 $107 = ((($106)) + 55|0);
 $108 = HEAP8[$107>>0]|0;
 $109 = $108 & -33;
 $110 = $109 | 32;
 HEAP8[$107>>0] = $110;
 $111 = HEAP32[$pPk>>2]|0;
 $112 = ((($111)) + 50|0);
 $113 = HEAP16[$112>>1]|0;
 $114 = $113&65535;
 HEAP32[$nPk>>2] = $114;
 $115 = HEAP32[$db>>2]|0;
 $116 = ((($115)) + 140|0);
 $117 = ((($116)) + 7|0);
 $118 = HEAP8[$117>>0]|0;
 $119 = ($118<<24>>24)!=(0);
 if (!($119)) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $120 = HEAP32[$i>>2]|0;
   $121 = HEAP32[$nPk>>2]|0;
   $122 = ($120|0)<($121|0);
   if (!($122)) {
    break;
   }
   $123 = HEAP32[$i>>2]|0;
   $124 = HEAP32[$pPk>>2]|0;
   $125 = ((($124)) + 4|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = (($126) + ($123<<1)|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128 << 16 >> 16;
   $130 = HEAP32[$1>>2]|0;
   $131 = ((($130)) + 4|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = (($132) + (($129*24)|0)|0);
   $134 = ((($133)) + 20|0);
   HEAP8[$134>>0] = 2;
   $135 = HEAP32[$i>>2]|0;
   $136 = (($135) + 1)|0;
   HEAP32[$i>>2] = $136;
  }
  $137 = HEAP32[$pPk>>2]|0;
  $138 = ((($137)) + 55|0);
  $139 = HEAP8[$138>>0]|0;
  $140 = $139 & -9;
  $141 = $140 | 8;
  HEAP8[$138>>0] = $141;
 }
 $142 = HEAP32[$1>>2]|0;
 $143 = ((($142)) + 28|0);
 $144 = HEAP32[$143>>2]|0;
 $145 = HEAP32[$pPk>>2]|0;
 $146 = ((($145)) + 44|0);
 HEAP32[$146>>2] = $144;
 $147 = HEAP32[$1>>2]|0;
 $148 = ((($147)) + 8|0);
 $149 = HEAP32[$148>>2]|0;
 HEAP32[$pIdx>>2] = $149;
 L32: while(1) {
  $150 = HEAP32[$pIdx>>2]|0;
  $151 = ($150|0)!=(0|0);
  if (!($151)) {
   break;
  }
  $152 = HEAP32[$pIdx>>2]|0;
  $153 = ((($152)) + 55|0);
  $154 = HEAP8[$153>>0]|0;
  $155 = $154 & 3;
  $156 = $155&255;
  $157 = ($156|0)==(2);
  L35: do {
   if (!($157)) {
    HEAP32[$n>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $158 = HEAP32[$i>>2]|0;
     $159 = HEAP32[$nPk>>2]|0;
     $160 = ($158|0)<($159|0);
     if (!($160)) {
      break;
     }
     $161 = HEAP32[$pIdx>>2]|0;
     $162 = ((($161)) + 4|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = HEAP32[$pIdx>>2]|0;
     $165 = ((($164)) + 50|0);
     $166 = HEAP16[$165>>1]|0;
     $167 = $166&65535;
     $168 = HEAP32[$i>>2]|0;
     $169 = HEAP32[$pPk>>2]|0;
     $170 = ((($169)) + 4|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = (($171) + ($168<<1)|0);
     $173 = HEAP16[$172>>1]|0;
     $174 = $173 << 16 >> 16;
     $175 = (_hasColumn($163,$167,$174)|0);
     $176 = ($175|0)!=(0);
     if (!($176)) {
      $177 = HEAP32[$n>>2]|0;
      $178 = (($177) + 1)|0;
      HEAP32[$n>>2] = $178;
     }
     $179 = HEAP32[$i>>2]|0;
     $180 = (($179) + 1)|0;
     HEAP32[$i>>2] = $180;
    }
    $181 = HEAP32[$n>>2]|0;
    $182 = ($181|0)==(0);
    if ($182) {
     $183 = HEAP32[$pIdx>>2]|0;
     $184 = ((($183)) + 50|0);
     $185 = HEAP16[$184>>1]|0;
     $186 = HEAP32[$pIdx>>2]|0;
     $187 = ((($186)) + 52|0);
     HEAP16[$187>>1] = $185;
     break;
    }
    $188 = HEAP32[$db>>2]|0;
    $189 = HEAP32[$pIdx>>2]|0;
    $190 = HEAP32[$pIdx>>2]|0;
    $191 = ((($190)) + 50|0);
    $192 = HEAP16[$191>>1]|0;
    $193 = $192&65535;
    $194 = HEAP32[$n>>2]|0;
    $195 = (($193) + ($194))|0;
    $196 = (_resizeIndexObject($188,$189,$195)|0);
    $197 = ($196|0)!=(0);
    if ($197) {
     label = 46;
     break L32;
    }
    HEAP32[$i>>2] = 0;
    $198 = HEAP32[$pIdx>>2]|0;
    $199 = ((($198)) + 50|0);
    $200 = HEAP16[$199>>1]|0;
    $201 = $200&65535;
    HEAP32[$j>>2] = $201;
    while(1) {
     $202 = HEAP32[$i>>2]|0;
     $203 = HEAP32[$nPk>>2]|0;
     $204 = ($202|0)<($203|0);
     if (!($204)) {
      break L35;
     }
     $205 = HEAP32[$pIdx>>2]|0;
     $206 = ((($205)) + 4|0);
     $207 = HEAP32[$206>>2]|0;
     $208 = HEAP32[$pIdx>>2]|0;
     $209 = ((($208)) + 50|0);
     $210 = HEAP16[$209>>1]|0;
     $211 = $210&65535;
     $212 = HEAP32[$i>>2]|0;
     $213 = HEAP32[$pPk>>2]|0;
     $214 = ((($213)) + 4|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = (($215) + ($212<<1)|0);
     $217 = HEAP16[$216>>1]|0;
     $218 = $217 << 16 >> 16;
     $219 = (_hasColumn($207,$211,$218)|0);
     $220 = ($219|0)!=(0);
     if (!($220)) {
      $221 = HEAP32[$i>>2]|0;
      $222 = HEAP32[$pPk>>2]|0;
      $223 = ((($222)) + 4|0);
      $224 = HEAP32[$223>>2]|0;
      $225 = (($224) + ($221<<1)|0);
      $226 = HEAP16[$225>>1]|0;
      $227 = HEAP32[$j>>2]|0;
      $228 = HEAP32[$pIdx>>2]|0;
      $229 = ((($228)) + 4|0);
      $230 = HEAP32[$229>>2]|0;
      $231 = (($230) + ($227<<1)|0);
      HEAP16[$231>>1] = $226;
      $232 = HEAP32[$i>>2]|0;
      $233 = HEAP32[$pPk>>2]|0;
      $234 = ((($233)) + 32|0);
      $235 = HEAP32[$234>>2]|0;
      $236 = (($235) + ($232<<2)|0);
      $237 = HEAP32[$236>>2]|0;
      $238 = HEAP32[$j>>2]|0;
      $239 = HEAP32[$pIdx>>2]|0;
      $240 = ((($239)) + 32|0);
      $241 = HEAP32[$240>>2]|0;
      $242 = (($241) + ($238<<2)|0);
      HEAP32[$242>>2] = $237;
      $243 = HEAP32[$j>>2]|0;
      $244 = (($243) + 1)|0;
      HEAP32[$j>>2] = $244;
     }
     $245 = HEAP32[$i>>2]|0;
     $246 = (($245) + 1)|0;
     HEAP32[$i>>2] = $246;
    }
   }
  } while(0);
  $247 = HEAP32[$pIdx>>2]|0;
  $248 = ((($247)) + 20|0);
  $249 = HEAP32[$248>>2]|0;
  HEAP32[$pIdx>>2] = $249;
 }
 if ((label|0) == 46) {
  STACKTOP = sp;return;
 }
 $250 = HEAP32[$nPk>>2]|0;
 $251 = HEAP32[$1>>2]|0;
 $252 = ((($251)) + 34|0);
 $253 = HEAP16[$252>>1]|0;
 $254 = $253 << 16 >> 16;
 $255 = ($250|0)<($254|0);
 if (!($255)) {
  $294 = HEAP32[$1>>2]|0;
  $295 = ((($294)) + 34|0);
  $296 = HEAP16[$295>>1]|0;
  $297 = HEAP32[$pPk>>2]|0;
  $298 = ((($297)) + 52|0);
  HEAP16[$298>>1] = $296;
  STACKTOP = sp;return;
 }
 $256 = HEAP32[$db>>2]|0;
 $257 = HEAP32[$pPk>>2]|0;
 $258 = HEAP32[$1>>2]|0;
 $259 = ((($258)) + 34|0);
 $260 = HEAP16[$259>>1]|0;
 $261 = $260 << 16 >> 16;
 $262 = (_resizeIndexObject($256,$257,$261)|0);
 $263 = ($262|0)!=(0);
 if ($263) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $264 = HEAP32[$nPk>>2]|0;
 HEAP32[$j>>2] = $264;
 while(1) {
  $265 = HEAP32[$i>>2]|0;
  $266 = HEAP32[$1>>2]|0;
  $267 = ((($266)) + 34|0);
  $268 = HEAP16[$267>>1]|0;
  $269 = $268 << 16 >> 16;
  $270 = ($265|0)<($269|0);
  if (!($270)) {
   break;
  }
  $271 = HEAP32[$pPk>>2]|0;
  $272 = ((($271)) + 4|0);
  $273 = HEAP32[$272>>2]|0;
  $274 = HEAP32[$j>>2]|0;
  $275 = HEAP32[$i>>2]|0;
  $276 = (_hasColumn($273,$274,$275)|0);
  $277 = ($276|0)!=(0);
  if (!($277)) {
   $278 = HEAP32[$i>>2]|0;
   $279 = $278&65535;
   $280 = HEAP32[$j>>2]|0;
   $281 = HEAP32[$pPk>>2]|0;
   $282 = ((($281)) + 4|0);
   $283 = HEAP32[$282>>2]|0;
   $284 = (($283) + ($280<<1)|0);
   HEAP16[$284>>1] = $279;
   $285 = HEAP32[$j>>2]|0;
   $286 = HEAP32[$pPk>>2]|0;
   $287 = ((($286)) + 32|0);
   $288 = HEAP32[$287>>2]|0;
   $289 = (($288) + ($285<<2)|0);
   HEAP32[$289>>2] = 20716;
   $290 = HEAP32[$j>>2]|0;
   $291 = (($290) + 1)|0;
   HEAP32[$j>>2] = $291;
  }
  $292 = HEAP32[$i>>2]|0;
  $293 = (($292) + 1)|0;
  HEAP32[$i>>2] = $293;
 }
 STACKTOP = sp;return;
}
function _sqlite3ResolveSelfReference($pParse,$pTab,$type,$pExpr,$pList) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $type = $type|0;
 $pExpr = $pExpr|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $sNC = 0, $sSrc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 112|0;
 $sSrc = sp;
 $sNC = sp + 80|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $type;
 HEAP32[$3>>2] = $pExpr;
 HEAP32[$4>>2] = $pList;
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 dest=$sSrc; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$sSrc>>2] = 1;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($sSrc)) + 8|0);
 $8 = ((($7)) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($sSrc)) + 8|0);
 $11 = ((($10)) + 16|0);
 HEAP32[$11>>2] = $9;
 $12 = ((($sSrc)) + 8|0);
 $13 = ((($12)) + 44|0);
 HEAP32[$13>>2] = -1;
 $14 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $14;
 $15 = ((($sNC)) + 4|0);
 HEAP32[$15>>2] = $sSrc;
 $16 = HEAP32[$2>>2]|0;
 $17 = $16&65535;
 $18 = ((($sNC)) + 28|0);
 HEAP16[$18>>1] = $17;
 $19 = HEAP32[$3>>2]|0;
 $20 = (_sqlite3ResolveExprNames($sNC,$19)|0);
 $21 = ($20|0)==(0);
 $22 = HEAP32[$4>>2]|0;
 $23 = ($22|0)!=(0|0);
 $or$cond = $21 & $23;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$4>>2]|0;
 (_sqlite3ResolveExprListNames($sNC,$24)|0);
 STACKTOP = sp;return;
}
function _estimateTableWidth($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTabCol = 0, $wTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $wTable = sp + 8|0;
 $pTabCol = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$wTable>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 34|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTabCol>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)>(0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pTabCol>>2]|0;
  $11 = ((($10)) + 22|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = HEAP32[$wTable>>2]|0;
  $15 = (($14) + ($13))|0;
  HEAP32[$wTable>>2] = $15;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + -1)|0;
  HEAP32[$i>>2] = $17;
  $18 = HEAP32[$pTabCol>>2]|0;
  $19 = ((($18)) + 24|0);
  HEAP32[$pTabCol>>2] = $19;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 32|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)<(0);
 if (!($24)) {
  $27 = HEAP32[$wTable>>2]|0;
  $28 = $27<<2;
  $29 = (_sqlite3LogEst($28,0)|0);
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 40|0);
  HEAP16[$31>>1] = $29;
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$wTable>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$wTable>>2] = $26;
 $27 = HEAP32[$wTable>>2]|0;
 $28 = $27<<2;
 $29 = (_sqlite3LogEst($28,0)|0);
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 40|0);
 HEAP16[$31>>1] = $29;
 STACKTOP = sp;return;
}
function _estimateIndexWidth($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $aCol = 0, $i = 0, $wIndex = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $wIndex = sp + 8|0;
 $i = sp + 4|0;
 $aCol = sp;
 $x = sp + 16|0;
 HEAP32[$0>>2] = $pIdx;
 HEAP32[$wIndex>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$aCol>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 52|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9&65535;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  HEAP16[$x>>1] = $17;
  $18 = HEAP16[$x>>1]|0;
  $19 = $18 << 16 >> 16;
  $20 = ($19|0)<(0);
  if ($20) {
   $35 = 1;
  } else {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<1)|0);
   $26 = HEAP16[$25>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = HEAP32[$aCol>>2]|0;
   $29 = (($28) + (($27*24)|0)|0);
   $30 = ((($29)) + 22|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $35 = $32;
  }
  $33 = HEAP32[$wIndex>>2]|0;
  $34 = (($33) + ($35))|0;
  HEAP32[$wIndex>>2] = $34;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$wIndex>>2]|0;
 $39 = $38<<2;
 $40 = (_sqlite3LogEst($39,0)|0);
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 48|0);
 HEAP16[$42>>1] = $40;
 STACKTOP = sp;return;
}
function _sqlite3VdbeEndCoroutine($v,$regYield) {
 $v = $v|0;
 $regYield = $regYield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $regYield;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp1($2,17,$3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 19|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 60|0);
 HEAP32[$11>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ResultSetOfSelect($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $pTab = 0, $savedFlags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pTab = sp + 8|0;
 $db = sp + 4|0;
 $savedFlags = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$savedFlags>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & -5;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 | 64;
 HEAP32[$13>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 _sqlite3SelectPrep($16,$17,0);
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 68|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 while(1) {
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 52|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$2>>2]|0;
  $27 = ((($26)) + 52|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$2>>2] = $28;
 }
 $29 = HEAP32[$savedFlags>>2]|0;
 $30 = HEAP32[$db>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$db>>2]|0;
 $33 = (_sqlite3DbMallocZero($32,72,0)|0);
 HEAP32[$pTab>>2] = $33;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $36 = HEAP32[$pTab>>2]|0;
 $37 = ((($36)) + 36|0);
 HEAP16[$37>>1] = 1;
 $38 = HEAP32[$pTab>>2]|0;
 HEAP32[$38>>2] = 0;
 $39 = HEAP32[$pTab>>2]|0;
 $40 = ((($39)) + 38|0);
 HEAP16[$40>>1] = 200;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$pTab>>2]|0;
 $45 = ((($44)) + 34|0);
 $46 = HEAP32[$pTab>>2]|0;
 $47 = ((($46)) + 4|0);
 (_sqlite3ColumnsFromExprList($41,$43,$45,$47)|0);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$pTab>>2]|0;
 $50 = HEAP32[$2>>2]|0;
 _selectAddColumnTypeAndCollation($48,$49,$50);
 $51 = HEAP32[$pTab>>2]|0;
 $52 = ((($51)) + 32|0);
 HEAP16[$52>>1] = -1;
 $53 = HEAP32[$db>>2]|0;
 $54 = ((($53)) + 65|0);
 $55 = HEAP8[$54>>0]|0;
 $56 = ($55<<24>>24)!=(0);
 if ($56) {
  $57 = HEAP32[$db>>2]|0;
  $58 = HEAP32[$pTab>>2]|0;
  _sqlite3DeleteTable($57,$58);
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 } else {
  $59 = HEAP32[$pTab>>2]|0;
  HEAP32[$0>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return (0)|0;
}
function _sqlite3TableAffinity($v,$pTab,$iReg) {
 $v = $v|0;
 $pTab = $pTab|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $zColAff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $zColAff = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iReg;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$zColAff>>2] = $5;
 $6 = HEAP32[$zColAff>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (_sqlite3VdbeDb($8)|0);
  HEAP32[$db>>2] = $9;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 34|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = (($13) + 1)|0;
  $15 = ($14|0)<(0);
  $16 = $15 << 31 >> 31;
  $17 = (_sqlite3DbMallocRaw(0,$14,$16)|0);
  HEAP32[$zColAff>>2] = $17;
  $18 = HEAP32[$zColAff>>2]|0;
  $19 = ($18|0)!=(0|0);
  if (!($19)) {
   $20 = HEAP32[$db>>2]|0;
   _sqlite3OomFault($20);
   STACKTOP = sp;return;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 34|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24 << 16 >> 16;
   $26 = ($21|0)<($25|0);
   if (!($26)) {
    break;
   }
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + (($27*24)|0)|0);
   $32 = ((($31)) + 21|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = HEAP32[$i>>2]|0;
   $35 = HEAP32[$zColAff>>2]|0;
   $36 = (($35) + ($34)|0);
   HEAP8[$36>>0] = $33;
   $37 = HEAP32[$i>>2]|0;
   $38 = (($37) + 1)|0;
   HEAP32[$i>>2] = $38;
  }
  while(1) {
   $39 = HEAP32[$i>>2]|0;
   $40 = (($39) + -1)|0;
   HEAP32[$i>>2] = $40;
   $41 = HEAP32[$zColAff>>2]|0;
   $42 = (($41) + ($39)|0);
   HEAP8[$42>>0] = 0;
   $43 = HEAP32[$i>>2]|0;
   $44 = ($43|0)>=(0);
   if (!($44)) {
    break;
   }
   $45 = HEAP32[$i>>2]|0;
   $46 = HEAP32[$zColAff>>2]|0;
   $47 = (($46) + ($45)|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48 << 24 >> 24;
   $50 = ($49|0)==(65);
   if (!($50)) {
    break;
   }
  }
  $51 = HEAP32[$zColAff>>2]|0;
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 20|0);
  HEAP32[$53>>2] = $51;
 }
 $54 = HEAP32[$zColAff>>2]|0;
 $55 = (_sqlite3Strlen30($54)|0);
 HEAP32[$i>>2] = $55;
 $56 = HEAP32[$i>>2]|0;
 $57 = ($56|0)!=(0);
 if (!($57)) {
  STACKTOP = sp;return;
 }
 $58 = HEAP32[$2>>2]|0;
 $59 = ($58|0)!=(0);
 $60 = HEAP32[$0>>2]|0;
 if ($59) {
  $61 = HEAP32[$2>>2]|0;
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$zColAff>>2]|0;
  $64 = HEAP32[$i>>2]|0;
  (_sqlite3VdbeAddOp4($60,48,$61,$62,0,$63,$64)|0);
  STACKTOP = sp;return;
 } else {
  $65 = HEAP32[$zColAff>>2]|0;
  $66 = HEAP32[$i>>2]|0;
  _sqlite3VdbeChangeP4($60,-1,$65,$66);
  STACKTOP = sp;return;
 }
}
function _createTableStmt($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $k = 0, $len = 0, $n = 0, $pCol = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $zEnd = 0, $zSep = 0, $zSep2 = 0, $zStmt = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $i = sp + 56|0;
 $k = sp + 52|0;
 $n = sp + 48|0;
 $zStmt = sp + 44|0;
 $zSep = sp + 40|0;
 $zSep2 = sp + 36|0;
 $zEnd = sp + 32|0;
 $pCol = sp + 28|0;
 $len = sp + 24|0;
 $zType = sp + 20|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$n>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pCol>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = ((($7)) + 34|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pCol>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_identLength($13)|0);
  $15 = (($14) + 5)|0;
  $16 = HEAP32[$n>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$n>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
  $20 = HEAP32[$pCol>>2]|0;
  $21 = ((($20)) + 24|0);
  HEAP32[$pCol>>2] = $21;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (_identLength($23)|0);
 $25 = HEAP32[$n>>2]|0;
 $26 = (($25) + ($24))|0;
 HEAP32[$n>>2] = $26;
 $27 = HEAP32[$n>>2]|0;
 $28 = ($27|0)<(50);
 if ($28) {
  HEAP32[$zSep>>2] = 17532;
  HEAP32[$zSep2>>2] = 30773;
  HEAP32[$zEnd>>2] = 20733;
 } else {
  HEAP32[$zSep>>2] = 30775;
  HEAP32[$zSep2>>2] = 30779;
  HEAP32[$zEnd>>2] = 30784;
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = ((($29)) + 34|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31 << 16 >> 16;
 $33 = ($32*6)|0;
 $34 = (35 + ($33))|0;
 $35 = HEAP32[$n>>2]|0;
 $36 = (($35) + ($34))|0;
 HEAP32[$n>>2] = $36;
 $37 = HEAP32[$n>>2]|0;
 $38 = ($37|0)<(0);
 $39 = $38 << 31 >> 31;
 $40 = (_sqlite3DbMallocRaw(0,$37,$39)|0);
 HEAP32[$zStmt>>2] = $40;
 $41 = HEAP32[$zStmt>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  $43 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($43);
  HEAP32[$0>>2] = 0;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $44 = HEAP32[$n>>2]|0;
 $45 = HEAP32[$zStmt>>2]|0;
 (_sqlite3_snprintf($44,$45,30787,$vararg_buffer)|0);
 $46 = HEAP32[$zStmt>>2]|0;
 $47 = (_sqlite3Strlen30($46)|0);
 HEAP32[$k>>2] = $47;
 $48 = HEAP32[$zStmt>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 _identPut($48,$k,$50);
 $51 = HEAP32[$k>>2]|0;
 $52 = (($51) + 1)|0;
 HEAP32[$k>>2] = $52;
 $53 = HEAP32[$zStmt>>2]|0;
 $54 = (($53) + ($51)|0);
 HEAP8[$54>>0] = 40;
 $55 = HEAP32[$2>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = HEAP32[$56>>2]|0;
 HEAP32[$pCol>>2] = $57;
 HEAP32[$i>>2] = 0;
 while(1) {
  $58 = HEAP32[$i>>2]|0;
  $59 = HEAP32[$2>>2]|0;
  $60 = ((($59)) + 34|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61 << 16 >> 16;
  $63 = ($58|0)<($62|0);
  $64 = HEAP32[$n>>2]|0;
  $65 = HEAP32[$k>>2]|0;
  $66 = (($64) - ($65))|0;
  $67 = HEAP32[$k>>2]|0;
  $68 = HEAP32[$zStmt>>2]|0;
  $69 = (($68) + ($67)|0);
  if (!($63)) {
   break;
  }
  $70 = HEAP32[$zSep>>2]|0;
  (_sqlite3_snprintf($66,$69,$70,$vararg_buffer1)|0);
  $71 = HEAP32[$k>>2]|0;
  $72 = HEAP32[$zStmt>>2]|0;
  $73 = (($72) + ($71)|0);
  $74 = (_sqlite3Strlen30($73)|0);
  $75 = HEAP32[$k>>2]|0;
  $76 = (($75) + ($74))|0;
  HEAP32[$k>>2] = $76;
  $77 = HEAP32[$zSep2>>2]|0;
  HEAP32[$zSep>>2] = $77;
  $78 = HEAP32[$zStmt>>2]|0;
  $79 = HEAP32[$pCol>>2]|0;
  $80 = HEAP32[$79>>2]|0;
  _identPut($78,$k,$80);
  $81 = HEAP32[$pCol>>2]|0;
  $82 = ((($81)) + 21|0);
  $83 = HEAP8[$82>>0]|0;
  $84 = $83 << 24 >> 24;
  $85 = (($84) - 65)|0;
  $86 = (2588 + ($85<<2)|0);
  $87 = HEAP32[$86>>2]|0;
  HEAP32[$zType>>2] = $87;
  $88 = HEAP32[$zType>>2]|0;
  $89 = (_sqlite3Strlen30($88)|0);
  HEAP32[$len>>2] = $89;
  $90 = HEAP32[$k>>2]|0;
  $91 = HEAP32[$zStmt>>2]|0;
  $92 = (($91) + ($90)|0);
  $93 = HEAP32[$zType>>2]|0;
  $94 = HEAP32[$len>>2]|0;
  _memcpy(($92|0),($93|0),($94|0))|0;
  $95 = HEAP32[$len>>2]|0;
  $96 = HEAP32[$k>>2]|0;
  $97 = (($96) + ($95))|0;
  HEAP32[$k>>2] = $97;
  $98 = HEAP32[$i>>2]|0;
  $99 = (($98) + 1)|0;
  HEAP32[$i>>2] = $99;
  $100 = HEAP32[$pCol>>2]|0;
  $101 = ((($100)) + 24|0);
  HEAP32[$pCol>>2] = $101;
 }
 $102 = HEAP32[$zEnd>>2]|0;
 HEAP32[$vararg_buffer3>>2] = $102;
 (_sqlite3_snprintf($66,$69,17521,$vararg_buffer3)|0);
 $103 = HEAP32[$zStmt>>2]|0;
 HEAP32[$0>>2] = $103;
 $104 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _sqlite3NestedParse($pParse,$zFormat,$varargs) {
 $pParse = $pParse|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $db = 0, $saveBuf = 0, $zErrMsg = 0, $zSql = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $ap = sp + 16|0;
 $zSql = sp + 8|0;
 $zErrMsg = sp + 4|0;
 $db = sp;
 $saveBuf = sp + 40|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$zErrMsg>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3VMPrintf($8,$9,$ap)|0);
 HEAP32[$zSql>>2] = $10;
 $11 = HEAP32[$zSql>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 18|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = (($15) + 1)<<24>>24;
 HEAP8[$14>>0] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 444|0);
 dest=$saveBuf; src=$18; stop=dest+100|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 444|0);
 dest=$20; stop=dest+100|0; do { HEAP16[dest>>1]=0|0; dest=dest+2|0; } while ((dest|0) < (stop|0));
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$zSql>>2]|0;
 (_sqlite3RunParser($21,$22,$zErrMsg)|0);
 $23 = HEAP32[$db>>2]|0;
 $24 = HEAP32[$zErrMsg>>2]|0;
 _sqlite3DbFree($23,$24);
 $25 = HEAP32[$db>>2]|0;
 $26 = HEAP32[$zSql>>2]|0;
 _sqlite3DbFree($25,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 444|0);
 dest=$28; src=$saveBuf; stop=dest+100|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 18|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = (($31) + -1)<<24>>24;
 HEAP8[$30>>0] = $32;
 STACKTOP = sp;return;
}
function _sqlite3ChangeCookie($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<4)|0);
 $14 = ((($13)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 1)|0;
 (_sqlite3VdbeAddOp3($7,52,$8,1,$17)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddParseSchemaOp($p,$iDb,$zWhere) {
 $p = $p|0;
 $iDb = $iDb|0;
 $zWhere = $zWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zWhere;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp4($3,123,$4,0,0,$5,-1)|0);
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ((($8)) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$j>>2]|0;
  _sqlite3VdbeUsesBtree($12,$13);
  $14 = HEAP32[$j>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$j>>2] = $15;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeOpcode($p,$addr,$iNewOpcode) {
 $p = $p|0;
 $addr = $addr|0;
 $iNewOpcode = $iNewOpcode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP8[$2>>0] = $iNewOpcode;
 $3 = HEAP8[$2>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 HEAP8[$6>>0] = $3;
 STACKTOP = sp;return;
}
function _sqlite3TokenInit($p,$z) {
 $p = $p|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$3>>2] = $2;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $5;
 STACKTOP = sp;return;
}
function _sqlite3ExprAlloc($db,$op,$pToken,$dequote) {
 $db = $db|0;
 $op = $op|0;
 $pToken = $pToken|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $c = 0, $iValue = 0, $nExtra = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pNew = 0, dest = 0;
 var label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pNew = sp + 12|0;
 $nExtra = sp + 8|0;
 $iValue = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pToken;
 HEAP32[$3>>2] = $dequote;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$iValue>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ($6|0)!=(132);
   if (!($7)) {
    $8 = HEAP32[$2>>2]|0;
    $9 = HEAP32[$8>>2]|0;
    $10 = ($9|0)==(0|0);
    if (!($10)) {
     $11 = HEAP32[$2>>2]|0;
     $12 = HEAP32[$11>>2]|0;
     $13 = (_sqlite3GetInt32($12,$iValue)|0);
     $14 = ($13|0)==(0);
     if (!($14)) {
      break;
     }
    }
   }
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + 1)|0;
   HEAP32[$nExtra>>2] = $18;
  }
 } while(0);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$nExtra>>2]|0;
 $21 = (48 + ($20))|0;
 $22 = (_sqlite3DbMallocRawNN($19,$21,0)|0);
 HEAP32[$pNew>>2] = $22;
 $23 = HEAP32[$pNew>>2]|0;
 $24 = ($23|0)!=(0|0);
 if (!($24)) {
  $90 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $25 = HEAP32[$pNew>>2]|0;
 dest=$25; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $26 = HEAP32[$1>>2]|0;
 $27 = $26&255;
 $28 = HEAP32[$pNew>>2]|0;
 HEAP8[$28>>0] = $27;
 $29 = HEAP32[$pNew>>2]|0;
 $30 = ((($29)) + 34|0);
 HEAP16[$30>>1] = -1;
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0|0);
 do {
  if ($32) {
   $33 = HEAP32[$nExtra>>2]|0;
   $34 = ($33|0)==(0);
   $35 = HEAP32[$pNew>>2]|0;
   if ($34) {
    $36 = ((($35)) + 4|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = $37 | 1024;
    HEAP32[$36>>2] = $38;
    $39 = HEAP32[$iValue>>2]|0;
    $40 = HEAP32[$pNew>>2]|0;
    $41 = ((($40)) + 8|0);
    HEAP32[$41>>2] = $39;
    break;
   }
   $42 = ((($35)) + 48|0);
   $43 = HEAP32[$pNew>>2]|0;
   $44 = ((($43)) + 8|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$2>>2]|0;
   $46 = ((($45)) + 4|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ($47|0)!=(0);
   if ($48) {
    $49 = HEAP32[$pNew>>2]|0;
    $50 = ((($49)) + 8|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    $54 = HEAP32[$2>>2]|0;
    $55 = ((($54)) + 4|0);
    $56 = HEAP32[$55>>2]|0;
    _memcpy(($51|0),($53|0),($56|0))|0;
   }
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$pNew>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + ($59)|0);
   HEAP8[$63>>0] = 0;
   $64 = HEAP32[$3>>2]|0;
   $65 = ($64|0)!=(0);
   $66 = HEAP32[$nExtra>>2]|0;
   $67 = ($66|0)>=(3);
   $or$cond = $65 & $67;
   if ($or$cond) {
    $68 = HEAP32[$2>>2]|0;
    $69 = HEAP32[$68>>2]|0;
    $70 = HEAP8[$69>>0]|0;
    $71 = $70 << 24 >> 24;
    HEAP32[$c>>2] = $71;
    $72 = ($71|0)==(39);
    $73 = HEAP32[$c>>2]|0;
    $74 = ($73|0)==(34);
    $or$cond3 = $72 | $74;
    $75 = HEAP32[$c>>2]|0;
    $76 = ($75|0)==(91);
    $or$cond5 = $or$cond3 | $76;
    $77 = HEAP32[$c>>2]|0;
    $78 = ($77|0)==(96);
    $or$cond7 = $or$cond5 | $78;
    if ($or$cond7) {
     $79 = HEAP32[$pNew>>2]|0;
     $80 = ((($79)) + 8|0);
     $81 = HEAP32[$80>>2]|0;
     (_sqlite3Dequote($81)|0);
     $82 = HEAP32[$c>>2]|0;
     $83 = ($82|0)==(34);
     if ($83) {
      $84 = HEAP32[$pNew>>2]|0;
      $85 = ((($84)) + 4|0);
      $86 = HEAP32[$85>>2]|0;
      $87 = $86 | 64;
      HEAP32[$85>>2] = $87;
     }
    }
   }
  }
 } while(0);
 $88 = HEAP32[$pNew>>2]|0;
 $89 = ((($88)) + 24|0);
 HEAP32[$89>>2] = 1;
 $90 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($90|0);
}
function _hasColumn($aiCol,$nCol,$x) {
 $aiCol = $aiCol|0;
 $nCol = $nCol|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $aiCol;
 HEAP32[$2>>2] = $nCol;
 HEAP32[$3>>2] = $x;
 while(1) {
  $4 = HEAP32[$2>>2]|0;
  $5 = (($4) + -1)|0;
  HEAP32[$2>>2] = $5;
  $6 = ($4|0)>(0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$3>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 2|0);
  HEAP32[$1>>2] = $9;
  $10 = HEAP16[$8>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = ($7|0)==($11|0);
  if ($12) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 else if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _resizeIndexObject($db,$pIdx,$N) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $nByte = 0, $zExtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $zExtra = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pIdx;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10*7)|0;
 HEAP32[$nByte>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$nByte>>2]|0;
 $14 = ($13|0)<(0);
 $15 = $14 << 31 >> 31;
 $16 = (_sqlite3DbMallocZero($12,$13,$15)|0);
 HEAP32[$zExtra>>2] = $16;
 $17 = HEAP32[$zExtra>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$0>>2] = 7;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 } else {
  $19 = HEAP32[$zExtra>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 52|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26<<2;
  _memcpy(($19|0),($22|0),($27|0))|0;
  $28 = HEAP32[$zExtra>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 32|0);
  HEAP32[$30>>2] = $28;
  $31 = HEAP32[$3>>2]|0;
  $32 = $31<<2;
  $33 = HEAP32[$zExtra>>2]|0;
  $34 = (($33) + ($32)|0);
  HEAP32[$zExtra>>2] = $34;
  $35 = HEAP32[$zExtra>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = ((($39)) + 52|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42<<1;
  _memcpy(($35|0),($38|0),($43|0))|0;
  $44 = HEAP32[$zExtra>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 4|0);
  HEAP32[$46>>2] = $44;
  $47 = HEAP32[$3>>2]|0;
  $48 = $47<<1;
  $49 = HEAP32[$zExtra>>2]|0;
  $50 = (($49) + ($48)|0);
  HEAP32[$zExtra>>2] = $50;
  $51 = HEAP32[$zExtra>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = ((($52)) + 28|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$2>>2]|0;
  $56 = ((($55)) + 52|0);
  $57 = HEAP16[$56>>1]|0;
  $58 = $57&65535;
  _memcpy(($51|0),($54|0),($58|0))|0;
  $59 = HEAP32[$zExtra>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ((($60)) + 28|0);
  HEAP32[$61>>2] = $59;
  $62 = HEAP32[$3>>2]|0;
  $63 = $62&65535;
  $64 = HEAP32[$2>>2]|0;
  $65 = ((($64)) + 52|0);
  HEAP16[$65>>1] = $63;
  $66 = HEAP32[$2>>2]|0;
  $67 = ((($66)) + 55|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 & -17;
  $70 = $69 | 16;
  HEAP8[$67>>0] = $70;
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 return (0)|0;
}
function _sqlite3ResolveExprNames($pNC,$pExpr) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $pParse = 0, $savedHasAgg = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $savedHasAgg = sp + 44|0;
 $w = sp + 4|0;
 $pParse = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 464|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($10) + ($14))|0;
 $16 = (_sqlite3ExprCheckHeight($7,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pParse>>2]|0;
 $22 = ((($21)) + 464|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20))|0;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 28|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = $28 & 4098;
 $30 = $29&65535;
 HEAP16[$savedHasAgg>>1] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 28|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33&65535;
 $35 = $34 & -4099;
 $36 = $35&65535;
 HEAP16[$32>>1] = $36;
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 HEAP32[$w>>2] = $38;
 $39 = ((($w)) + 4|0);
 HEAP32[$39>>2] = 186;
 $40 = ((($w)) + 8|0);
 HEAP32[$40>>2] = 187;
 $41 = ((($w)) + 12|0);
 HEAP32[$41>>2] = 0;
 $42 = ((($w)) + 16|0);
 HEAP32[$42>>2] = 0;
 $43 = ((($w)) + 20|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($w)) + 24|0);
 HEAP32[$45>>2] = $44;
 $46 = HEAP32[$2>>2]|0;
 (_sqlite3WalkExpr($w,$46)|0);
 $47 = HEAP32[$2>>2]|0;
 $48 = ((($47)) + 24|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = ((($51)) + 464|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) - ($49))|0;
 HEAP32[$52>>2] = $54;
 $55 = HEAP32[$1>>2]|0;
 $56 = ((($55)) + 24|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)>(0);
 if ($58) {
  label = 7;
 } else {
  $59 = HEAP32[$w>>2]|0;
  $60 = ((($59)) + 68|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($61|0)>(0);
  if ($62) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $63 = HEAP32[$2>>2]|0;
  $64 = ((($63)) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = $65 | 8;
  HEAP32[$64>>2] = $66;
 }
 $67 = HEAP32[$1>>2]|0;
 $68 = ((($67)) + 28|0);
 $69 = HEAP16[$68>>1]|0;
 $70 = $69&65535;
 $71 = $70 & 2;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$2>>2]|0;
  $74 = ((($73)) + 4|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = $75 | 2;
  HEAP32[$74>>2] = $76;
 }
 $77 = HEAP16[$savedHasAgg>>1]|0;
 $78 = $77&65535;
 $79 = HEAP32[$1>>2]|0;
 $80 = ((($79)) + 28|0);
 $81 = HEAP16[$80>>1]|0;
 $82 = $81&65535;
 $83 = $82 | $78;
 $84 = $83&65535;
 HEAP16[$80>>1] = $84;
 $85 = HEAP32[$2>>2]|0;
 $86 = ((($85)) + 4|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $87 & 8;
 $89 = ($88|0)!=(0);
 $90 = $89&1;
 HEAP32[$0>>2] = $90;
 $91 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _sqlite3ResolveExprListNames($pNC,$pList) {
 $pNC = $pNC|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $5 = HEAP32[$i>>2]|0;
    $6 = HEAP32[$2>>2]|0;
    $7 = HEAP32[$6>>2]|0;
    $8 = ($5|0)<($7|0);
    if (!($8)) {
     break L1;
    }
    $9 = HEAP32[$1>>2]|0;
    $10 = HEAP32[$i>>2]|0;
    $11 = HEAP32[$2>>2]|0;
    $12 = ((($11)) + 4|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = (($13) + (($10*20)|0)|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = (_sqlite3ResolveExprNames($9,$15)|0);
    $17 = ($16|0)!=(0);
    if ($17) {
     break;
    }
    $18 = HEAP32[$i>>2]|0;
    $19 = (($18) + 1)|0;
    HEAP32[$i>>2] = $19;
   }
   HEAP32[$0>>2] = 2;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3ExprCheckHeight($pParse,$nHeight) {
 $pParse = $pParse|0;
 $nHeight = $nHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mxHeight = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $mxHeight = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $nHeight;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 88|0);
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$mxHeight>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$mxHeight>>2]|0;
 $9 = ($7|0)>($8|0);
 if (!($9)) {
  $12 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$mxHeight>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 _sqlite3ErrorMsg($10,29176,$vararg_buffer);
 HEAP32[$rc>>2] = 1;
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _resolveExprStep($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $auth = 0, $enc = 0, $is_agg = 0, $n = 0, $nId = 0, $nRef = 0, $no_such_func = 0, $pDef = 0, $pList = 0;
 var $pNC = 0, $pNC2 = 0, $pParse = 0, $pRight = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr6 = 0, $wrong_num_args = 0, $zColumn = 0, $zDb = 0, $zId = 0, $zTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $pNC = sp + 104|0;
 $pParse = sp + 100|0;
 $zColumn = sp + 96|0;
 $zTable = sp + 92|0;
 $zDb = sp + 88|0;
 $pRight = sp + 84|0;
 $pList = sp + 80|0;
 $n = sp + 76|0;
 $no_such_func = sp + 72|0;
 $wrong_num_args = sp + 68|0;
 $is_agg = sp + 64|0;
 $auth = sp + 60|0;
 $nId = sp + 56|0;
 $zId = sp + 52|0;
 $pDef = sp + 48|0;
 $enc = sp + 120|0;
 $pNC2 = sp + 44|0;
 $nRef = sp + 40|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pNC>>2] = $5;
 $6 = HEAP32[$pNC>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 4;
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$0>>2] = 1;
  $310 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($310|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 | 4;
 HEAP32[$14>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 switch ($19|0) {
 case 27:  {
  $20 = HEAP32[$pParse>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = ((($21)) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$pNC>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (_lookupName($20,0,0,$23,$24,$25)|0);
  HEAP32[$0>>2] = $26;
  $310 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($310|0);
  break;
 }
 case 122:  {
  $27 = HEAP32[$pParse>>2]|0;
  $28 = HEAP32[$pNC>>2]|0;
  _notValid($27,$28,29224,32);
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pRight>>2] = $31;
  $32 = HEAP32[$pRight>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = ($34|0)==(27);
  if ($35) {
   HEAP32[$zDb>>2] = 0;
   $36 = HEAP32[$2>>2]|0;
   $37 = ((($36)) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$zTable>>2] = $40;
   $41 = HEAP32[$pRight>>2]|0;
   $42 = ((($41)) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   HEAP32[$zColumn>>2] = $43;
  } else {
   $44 = HEAP32[$2>>2]|0;
   $45 = ((($44)) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   HEAP32[$zDb>>2] = $48;
   $49 = HEAP32[$pRight>>2]|0;
   $50 = ((($49)) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ((($51)) + 8|0);
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$zTable>>2] = $53;
   $54 = HEAP32[$pRight>>2]|0;
   $55 = ((($54)) + 16|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = ((($56)) + 8|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$zColumn>>2] = $58;
  }
  $59 = HEAP32[$pParse>>2]|0;
  $60 = HEAP32[$zDb>>2]|0;
  $61 = HEAP32[$zTable>>2]|0;
  $62 = HEAP32[$zColumn>>2]|0;
  $63 = HEAP32[$pNC>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = (_lookupName($59,$60,$61,$62,$63,$64)|0);
  HEAP32[$0>>2] = $65;
  $310 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($310|0);
  break;
 }
 case 151:  {
  $66 = HEAP32[$2>>2]|0;
  $67 = ((($66)) + 20|0);
  $68 = HEAP32[$67>>2]|0;
  HEAP32[$pList>>2] = $68;
  $69 = HEAP32[$pList>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$pList>>2]|0;
   $72 = HEAP32[$71>>2]|0;
   $73 = $72;
  } else {
   $73 = 0;
  }
  HEAP32[$n>>2] = $73;
  HEAP32[$no_such_func>>2] = 0;
  HEAP32[$wrong_num_args>>2] = 0;
  HEAP32[$is_agg>>2] = 0;
  $74 = HEAP32[$pParse>>2]|0;
  $75 = HEAP32[$74>>2]|0;
  $76 = ((($75)) + 62|0);
  $77 = HEAP8[$76>>0]|0;
  HEAP8[$enc>>0] = $77;
  $78 = HEAP32[$pParse>>2]|0;
  $79 = HEAP32[$pNC>>2]|0;
  _notValid($78,$79,29241,16);
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 8|0);
  $82 = HEAP32[$81>>2]|0;
  HEAP32[$zId>>2] = $82;
  $83 = HEAP32[$zId>>2]|0;
  $84 = (_sqlite3Strlen30($83)|0);
  HEAP32[$nId>>2] = $84;
  $85 = HEAP32[$pParse>>2]|0;
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$zId>>2]|0;
  $88 = HEAP32[$nId>>2]|0;
  $89 = HEAP32[$n>>2]|0;
  $90 = HEAP8[$enc>>0]|0;
  $91 = (_sqlite3FindFunction($86,$87,$88,$89,$90,0)|0);
  HEAP32[$pDef>>2] = $91;
  $92 = HEAP32[$pDef>>2]|0;
  $93 = ($92|0)==(0|0);
  do {
   if ($93) {
    $94 = HEAP32[$pParse>>2]|0;
    $95 = HEAP32[$94>>2]|0;
    $96 = HEAP32[$zId>>2]|0;
    $97 = HEAP32[$nId>>2]|0;
    $98 = HEAP8[$enc>>0]|0;
    $99 = (_sqlite3FindFunction($95,$96,$97,-2,$98,0)|0);
    HEAP32[$pDef>>2] = $99;
    $100 = HEAP32[$pDef>>2]|0;
    $101 = ($100|0)==(0|0);
    if ($101) {
     HEAP32[$no_such_func>>2] = 1;
     break;
    } else {
     HEAP32[$wrong_num_args>>2] = 1;
     break;
    }
   } else {
    $102 = HEAP32[$pDef>>2]|0;
    $103 = ((($102)) + 16|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = ($104|0)!=(0|0);
    $106 = $105&1;
    HEAP32[$is_agg>>2] = $106;
    $107 = HEAP32[$pDef>>2]|0;
    $108 = ((($107)) + 2|0);
    $109 = HEAP16[$108>>1]|0;
    $110 = $109&65535;
    $111 = $110 & 1024;
    $112 = ($111|0)!=(0);
    do {
     if ($112) {
      $113 = HEAP32[$2>>2]|0;
      $114 = ((($113)) + 4|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = $115 | 266240;
      HEAP32[$114>>2] = $116;
      $117 = HEAP32[$n>>2]|0;
      $118 = ($117|0)==(2);
      if (!($118)) {
       $136 = HEAP32[$pDef>>2]|0;
       $137 = ((($136)) + 20|0);
       $138 = HEAP32[$137>>2]|0;
       $139 = HEAP8[$138>>0]|0;
       $140 = $139 << 24 >> 24;
       $141 = ($140|0)==(117);
       $142 = $141 ? 8388608 : 125829120;
       $143 = HEAP32[$2>>2]|0;
       $144 = ((($143)) + 28|0);
       HEAP32[$144>>2] = $142;
       break;
      }
      $119 = HEAP32[$pList>>2]|0;
      $120 = ((($119)) + 4|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ((($121)) + 20|0);
      $123 = HEAP32[$122>>2]|0;
      $124 = (_exprProbability($123)|0);
      $125 = HEAP32[$2>>2]|0;
      $126 = ((($125)) + 28|0);
      HEAP32[$126>>2] = $124;
      $127 = HEAP32[$2>>2]|0;
      $128 = ((($127)) + 28|0);
      $129 = HEAP32[$128>>2]|0;
      $130 = ($129|0)<(0);
      if ($130) {
       $131 = HEAP32[$pParse>>2]|0;
       _sqlite3ErrorMsg($131,29251,$vararg_buffer);
       $132 = HEAP32[$pNC>>2]|0;
       $133 = ((($132)) + 24|0);
       $134 = HEAP32[$133>>2]|0;
       $135 = (($134) + 1)|0;
       HEAP32[$133>>2] = $135;
      }
     }
    } while(0);
    $145 = HEAP32[$pParse>>2]|0;
    $146 = HEAP32[$pDef>>2]|0;
    $147 = ((($146)) + 20|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = (_sqlite3AuthCheck($145,31,0,$148,0)|0);
    HEAP32[$auth>>2] = $149;
    $150 = HEAP32[$auth>>2]|0;
    $151 = ($150|0)!=(0);
    if ($151) {
     $152 = HEAP32[$auth>>2]|0;
     $153 = ($152|0)==(1);
     if ($153) {
      $154 = HEAP32[$pParse>>2]|0;
      $155 = HEAP32[$pDef>>2]|0;
      $156 = ((($155)) + 20|0);
      $157 = HEAP32[$156>>2]|0;
      HEAP32[$vararg_buffer1>>2] = $157;
      _sqlite3ErrorMsg($154,29322,$vararg_buffer1);
      $158 = HEAP32[$pNC>>2]|0;
      $159 = ((($158)) + 24|0);
      $160 = HEAP32[$159>>2]|0;
      $161 = (($160) + 1)|0;
      HEAP32[$159>>2] = $161;
     }
     $162 = HEAP32[$2>>2]|0;
     HEAP8[$162>>0] = 101;
     HEAP32[$0>>2] = 1;
     $310 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($310|0);
    } else {
     $163 = HEAP32[$pDef>>2]|0;
     $164 = ((($163)) + 2|0);
     $165 = HEAP16[$164>>1]|0;
     $166 = $165&65535;
     $167 = $166 & 10240;
     $168 = ($167|0)!=(0);
     if ($168) {
      $169 = HEAP32[$2>>2]|0;
      $170 = ((($169)) + 4|0);
      $171 = HEAP32[$170>>2]|0;
      $172 = $171 | 524288;
      HEAP32[$170>>2] = $172;
     }
     $173 = HEAP32[$pDef>>2]|0;
     $174 = ((($173)) + 2|0);
     $175 = HEAP16[$174>>1]|0;
     $176 = $175&65535;
     $177 = $176 & 2048;
     $178 = ($177|0)==(0);
     if (!($178)) {
      break;
     }
     $179 = HEAP32[$pParse>>2]|0;
     $180 = HEAP32[$pNC>>2]|0;
     _notValid($179,$180,29357,32);
     break;
    }
   }
  } while(0);
  $181 = HEAP32[$is_agg>>2]|0;
  $182 = ($181|0)!=(0);
  if ($182) {
   $183 = HEAP32[$pNC>>2]|0;
   $184 = ((($183)) + 28|0);
   $185 = HEAP16[$184>>1]|0;
   $186 = $185&65535;
   $187 = $186 & 1;
   $188 = ($187|0)==(0);
   if ($188) {
    $189 = HEAP32[$pParse>>2]|0;
    $190 = HEAP32[$nId>>2]|0;
    $191 = HEAP32[$zId>>2]|0;
    HEAP32[$vararg_buffer3>>2] = $190;
    $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
    HEAP32[$vararg_ptr6>>2] = $191;
    _sqlite3ErrorMsg($189,29385,$vararg_buffer3);
    $192 = HEAP32[$pNC>>2]|0;
    $193 = ((($192)) + 24|0);
    $194 = HEAP32[$193>>2]|0;
    $195 = (($194) + 1)|0;
    HEAP32[$193>>2] = $195;
    HEAP32[$is_agg>>2] = 0;
   } else {
    label = 31;
   }
  } else {
   label = 31;
  }
  do {
   if ((label|0) == 31) {
    $196 = HEAP32[$no_such_func>>2]|0;
    $197 = ($196|0)!=(0);
    if ($197) {
     $198 = HEAP32[$pParse>>2]|0;
     $199 = HEAP32[$198>>2]|0;
     $200 = ((($199)) + 140|0);
     $201 = ((($200)) + 5|0);
     $202 = HEAP8[$201>>0]|0;
     $203 = $202&255;
     $204 = ($203|0)==(0);
     if ($204) {
      $205 = HEAP32[$pParse>>2]|0;
      $206 = HEAP32[$nId>>2]|0;
      $207 = HEAP32[$zId>>2]|0;
      HEAP32[$vararg_buffer7>>2] = $206;
      $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
      HEAP32[$vararg_ptr10>>2] = $207;
      _sqlite3ErrorMsg($205,29421,$vararg_buffer7);
      $208 = HEAP32[$pNC>>2]|0;
      $209 = ((($208)) + 24|0);
      $210 = HEAP32[$209>>2]|0;
      $211 = (($210) + 1)|0;
      HEAP32[$209>>2] = $211;
      break;
     }
    }
    $212 = HEAP32[$wrong_num_args>>2]|0;
    $213 = ($212|0)!=(0);
    if ($213) {
     $214 = HEAP32[$pParse>>2]|0;
     $215 = HEAP32[$nId>>2]|0;
     $216 = HEAP32[$zId>>2]|0;
     HEAP32[$vararg_buffer11>>2] = $215;
     $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
     HEAP32[$vararg_ptr14>>2] = $216;
     _sqlite3ErrorMsg($214,29444,$vararg_buffer11);
     $217 = HEAP32[$pNC>>2]|0;
     $218 = ((($217)) + 24|0);
     $219 = HEAP32[$218>>2]|0;
     $220 = (($219) + 1)|0;
     HEAP32[$218>>2] = $220;
    }
   }
  } while(0);
  $221 = HEAP32[$is_agg>>2]|0;
  $222 = ($221|0)!=(0);
  if ($222) {
   $223 = HEAP32[$pNC>>2]|0;
   $224 = ((($223)) + 28|0);
   $225 = HEAP16[$224>>1]|0;
   $226 = $225&65535;
   $227 = $226 & -2;
   $228 = $227&65535;
   HEAP16[$224>>1] = $228;
  }
  $229 = HEAP32[$1>>2]|0;
  $230 = HEAP32[$pList>>2]|0;
  (_sqlite3WalkExprList($229,$230)|0);
  $231 = HEAP32[$is_agg>>2]|0;
  $232 = ($231|0)!=(0);
  if ($232) {
   $233 = HEAP32[$pNC>>2]|0;
   HEAP32[$pNC2>>2] = $233;
   $234 = HEAP32[$2>>2]|0;
   HEAP8[$234>>0] = -103;
   $235 = HEAP32[$2>>2]|0;
   $236 = ((($235)) + 38|0);
   HEAP8[$236>>0] = 0;
   while(1) {
    $237 = HEAP32[$pNC2>>2]|0;
    $238 = ($237|0)!=(0|0);
    if (!($238)) {
     break;
    }
    $239 = HEAP32[$2>>2]|0;
    $240 = HEAP32[$pNC2>>2]|0;
    $241 = ((($240)) + 4|0);
    $242 = HEAP32[$241>>2]|0;
    $243 = (_sqlite3FunctionUsesThisSrc($239,$242)|0);
    $244 = ($243|0)!=(0);
    $245 = $244 ^ 1;
    if (!($245)) {
     break;
    }
    $246 = HEAP32[$2>>2]|0;
    $247 = ((($246)) + 38|0);
    $248 = HEAP8[$247>>0]|0;
    $249 = (($248) + 1)<<24>>24;
    HEAP8[$247>>0] = $249;
    $250 = HEAP32[$pNC2>>2]|0;
    $251 = ((($250)) + 16|0);
    $252 = HEAP32[$251>>2]|0;
    HEAP32[$pNC2>>2] = $252;
   }
   $253 = HEAP32[$pNC2>>2]|0;
   $254 = ($253|0)!=(0|0);
   if ($254) {
    $255 = HEAP32[$pDef>>2]|0;
    $256 = ((($255)) + 2|0);
    $257 = HEAP16[$256>>1]|0;
    $258 = $257&65535;
    $259 = $258 & 4096;
    $260 = 2 | $259;
    $261 = HEAP32[$pNC2>>2]|0;
    $262 = ((($261)) + 28|0);
    $263 = HEAP16[$262>>1]|0;
    $264 = $263&65535;
    $265 = $264 | $260;
    $266 = $265&65535;
    HEAP16[$262>>1] = $266;
   }
   $267 = HEAP32[$pNC>>2]|0;
   $268 = ((($267)) + 28|0);
   $269 = HEAP16[$268>>1]|0;
   $270 = $269&65535;
   $271 = $270 | 1;
   $272 = $271&65535;
   HEAP16[$268>>1] = $272;
  }
  HEAP32[$0>>2] = 1;
  $310 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($310|0);
  break;
 }
 case 75: case 20: case 119:  {
  $273 = HEAP32[$2>>2]|0;
  $274 = ((($273)) + 4|0);
  $275 = HEAP32[$274>>2]|0;
  $276 = $275 & 2048;
  $277 = ($276|0)!=(0);
  if ($277) {
   $278 = HEAP32[$pNC>>2]|0;
   $279 = ((($278)) + 20|0);
   $280 = HEAP32[$279>>2]|0;
   HEAP32[$nRef>>2] = $280;
   $281 = HEAP32[$pParse>>2]|0;
   $282 = HEAP32[$pNC>>2]|0;
   _notValid($281,$282,29489,52);
   $283 = HEAP32[$1>>2]|0;
   $284 = HEAP32[$2>>2]|0;
   $285 = ((($284)) + 20|0);
   $286 = HEAP32[$285>>2]|0;
   (_sqlite3WalkSelect($283,$286)|0);
   $287 = HEAP32[$nRef>>2]|0;
   $288 = HEAP32[$pNC>>2]|0;
   $289 = ((($288)) + 20|0);
   $290 = HEAP32[$289>>2]|0;
   $291 = ($287|0)!=($290|0);
   if ($291) {
    $292 = HEAP32[$2>>2]|0;
    $293 = ((($292)) + 4|0);
    $294 = HEAP32[$293>>2]|0;
    $295 = $294 | 32;
    HEAP32[$293>>2] = $295;
   }
  }
  break;
 }
 case 135:  {
  $296 = HEAP32[$pParse>>2]|0;
  $297 = HEAP32[$pNC>>2]|0;
  _notValid($296,$297,29500,52);
  break;
 }
 default: {
 }
 }
 $298 = HEAP32[$pParse>>2]|0;
 $299 = ((($298)) + 68|0);
 $300 = HEAP32[$299>>2]|0;
 $301 = ($300|0)!=(0);
 if ($301) {
  $309 = 1;
 } else {
  $302 = HEAP32[$pParse>>2]|0;
  $303 = HEAP32[$302>>2]|0;
  $304 = ((($303)) + 65|0);
  $305 = HEAP8[$304>>0]|0;
  $306 = $305&255;
  $307 = ($306|0)!=(0);
  $309 = $307;
 }
 $308 = $309 ? 2 : 0;
 HEAP32[$0>>2] = $308;
 $310 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($310|0);
}
function _resolveSelectStep($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $db = 0, $i = 0, $isCompound = 0, $nCompound = 0, $nRef = 0, $or$cond = 0, $pGroupBy = 0, $pItem = 0, $pItem1 = 0, $pItem3 = 0, $pLeftmost = 0, $pNC = 0, $pOuterNC = 0, $pParse = 0, $pSub = 0, $pSub2 = 0, $sNC = 0, $vararg_buffer = 0, $vararg_buffer2 = 0;
 var $zSavedContext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $pOuterNC = sp + 104|0;
 $sNC = sp + 72|0;
 $isCompound = sp + 68|0;
 $nCompound = sp + 64|0;
 $pParse = sp + 60|0;
 $i = sp + 56|0;
 $pGroupBy = sp + 52|0;
 $pLeftmost = sp + 48|0;
 $db = sp + 44|0;
 $pSub = sp + 40|0;
 $pItem = sp + 36|0;
 $pNC = sp + 32|0;
 $nRef = sp + 28|0;
 $zSavedContext = sp + 24|0;
 $pItem1 = sp + 20|0;
 $pSub2 = sp + 16|0;
 $pItem3 = sp + 12|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 6|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 4;
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $340 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($340|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pOuterNC>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pParse>>2] = $13;
 $14 = HEAP32[$pParse>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 6|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = $19 & 32;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$pParse>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$pOuterNC>>2]|0;
  _sqlite3SelectPrep($22,$23,$24);
  $25 = HEAP32[$pParse>>2]|0;
  $26 = ((($25)) + 68|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $35 = 1;
  } else {
   $29 = HEAP32[$db>>2]|0;
   $30 = ((($29)) + 65|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $33 = ($32|0)!=(0);
   $35 = $33;
  }
  $34 = $35 ? 2 : 1;
  HEAP32[$0>>2] = $34;
  $340 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($340|0);
 }
 $36 = HEAP32[$2>>2]|0;
 $37 = ((($36)) + 52|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)!=(0|0);
 $40 = $39&1;
 HEAP32[$isCompound>>2] = $40;
 HEAP32[$nCompound>>2] = 0;
 $41 = HEAP32[$2>>2]|0;
 HEAP32[$pLeftmost>>2] = $41;
 L12: while(1) {
  $42 = HEAP32[$2>>2]|0;
  $43 = ($42|0)!=(0|0);
  if (!($43)) {
   label = 68;
   break;
  }
  $44 = HEAP32[$2>>2]|0;
  $45 = ((($44)) + 6|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46&65535;
  $48 = $47 | 4;
  $49 = $48&65535;
  HEAP16[$45>>1] = $49;
  ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
  $50 = HEAP32[$pParse>>2]|0;
  HEAP32[$sNC>>2] = $50;
  $51 = HEAP32[$2>>2]|0;
  $52 = ((($51)) + 60|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3ResolveExprNames($sNC,$53)|0);
  $55 = ($54|0)!=(0);
  if ($55) {
   label = 11;
   break;
  }
  $56 = HEAP32[$2>>2]|0;
  $57 = ((($56)) + 64|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (_sqlite3ResolveExprNames($sNC,$58)|0);
  $60 = ($59|0)!=(0);
  if ($60) {
   label = 11;
   break;
  }
  $61 = HEAP32[$2>>2]|0;
  $62 = ((($61)) + 6|0);
  $63 = HEAP16[$62>>1]|0;
  $64 = $63&65535;
  $65 = $64 & 16384;
  $66 = ($65|0)!=(0);
  if ($66) {
   $67 = HEAP32[$2>>2]|0;
   $68 = ((($67)) + 32|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ((($69)) + 8|0);
   $71 = ((($70)) + 20|0);
   $72 = HEAP32[$71>>2]|0;
   HEAP32[$pSub>>2] = $72;
   $73 = HEAP32[$2>>2]|0;
   $74 = ((($73)) + 48|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$pSub>>2]|0;
   $77 = ((($76)) + 48|0);
   HEAP32[$77>>2] = $75;
   $78 = HEAP32[$2>>2]|0;
   $79 = ((($78)) + 48|0);
   HEAP32[$79>>2] = 0;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $80 = HEAP32[$i>>2]|0;
   $81 = HEAP32[$2>>2]|0;
   $82 = ((($81)) + 32|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = HEAP32[$83>>2]|0;
   $85 = ($80|0)<($84|0);
   if (!($85)) {
    break;
   }
   $86 = HEAP32[$i>>2]|0;
   $87 = HEAP32[$2>>2]|0;
   $88 = ((($87)) + 32|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = ((($89)) + 8|0);
   $91 = (($90) + (($86*72)|0)|0);
   HEAP32[$pItem>>2] = $91;
   $92 = HEAP32[$pItem>>2]|0;
   $93 = ((($92)) + 20|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = ($94|0)!=(0|0);
   if ($95) {
    HEAP32[$nRef>>2] = 0;
    $96 = HEAP32[$pParse>>2]|0;
    $97 = ((($96)) + 496|0);
    $98 = HEAP32[$97>>2]|0;
    HEAP32[$zSavedContext>>2] = $98;
    $99 = HEAP32[$pOuterNC>>2]|0;
    HEAP32[$pNC>>2] = $99;
    while(1) {
     $100 = HEAP32[$pNC>>2]|0;
     $101 = ($100|0)!=(0|0);
     if (!($101)) {
      break;
     }
     $102 = HEAP32[$pNC>>2]|0;
     $103 = ((($102)) + 20|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = HEAP32[$nRef>>2]|0;
     $106 = (($105) + ($104))|0;
     HEAP32[$nRef>>2] = $106;
     $107 = HEAP32[$pNC>>2]|0;
     $108 = ((($107)) + 16|0);
     $109 = HEAP32[$108>>2]|0;
     HEAP32[$pNC>>2] = $109;
    }
    $110 = HEAP32[$pItem>>2]|0;
    $111 = ((($110)) + 8|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = ($112|0)!=(0|0);
    if ($113) {
     $114 = HEAP32[$pItem>>2]|0;
     $115 = ((($114)) + 8|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$pParse>>2]|0;
     $118 = ((($117)) + 496|0);
     HEAP32[$118>>2] = $116;
    }
    $119 = HEAP32[$pParse>>2]|0;
    $120 = HEAP32[$pItem>>2]|0;
    $121 = ((($120)) + 20|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = HEAP32[$pOuterNC>>2]|0;
    _sqlite3ResolveSelectNames($119,$122,$123);
    $124 = HEAP32[$zSavedContext>>2]|0;
    $125 = HEAP32[$pParse>>2]|0;
    $126 = ((($125)) + 496|0);
    HEAP32[$126>>2] = $124;
    $127 = HEAP32[$pParse>>2]|0;
    $128 = ((($127)) + 68|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = ($129|0)!=(0);
    if ($130) {
     label = 24;
     break L12;
    }
    $131 = HEAP32[$db>>2]|0;
    $132 = ((($131)) + 65|0);
    $133 = HEAP8[$132>>0]|0;
    $134 = $133&255;
    $135 = ($134|0)!=(0);
    if ($135) {
     label = 24;
     break L12;
    }
    $136 = HEAP32[$pOuterNC>>2]|0;
    HEAP32[$pNC>>2] = $136;
    while(1) {
     $137 = HEAP32[$pNC>>2]|0;
     $138 = ($137|0)!=(0|0);
     if (!($138)) {
      break;
     }
     $139 = HEAP32[$pNC>>2]|0;
     $140 = ((($139)) + 20|0);
     $141 = HEAP32[$140>>2]|0;
     $142 = HEAP32[$nRef>>2]|0;
     $143 = (($142) - ($141))|0;
     HEAP32[$nRef>>2] = $143;
     $144 = HEAP32[$pNC>>2]|0;
     $145 = ((($144)) + 16|0);
     $146 = HEAP32[$145>>2]|0;
     HEAP32[$pNC>>2] = $146;
    }
    $147 = HEAP32[$nRef>>2]|0;
    $148 = ($147|0)!=(0);
    $149 = $148&1;
    $150 = HEAP32[$pItem>>2]|0;
    $151 = ((($150)) + 36|0);
    $152 = ((($151)) + 1|0);
    $153 = $149&255;
    $154 = HEAP8[$152>>0]|0;
    $155 = $153 & 1;
    $156 = ($155 << 3)&255;
    $157 = $154 & -9;
    $158 = $157 | $156;
    HEAP8[$152>>0] = $158;
   }
   $159 = HEAP32[$i>>2]|0;
   $160 = (($159) + 1)|0;
   HEAP32[$i>>2] = $160;
  }
  $161 = ((($sNC)) + 28|0);
  HEAP16[$161>>1] = 1;
  $162 = HEAP32[$2>>2]|0;
  $163 = ((($162)) + 32|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = ((($sNC)) + 4|0);
  HEAP32[$165>>2] = $164;
  $166 = HEAP32[$pOuterNC>>2]|0;
  $167 = ((($sNC)) + 16|0);
  HEAP32[$167>>2] = $166;
  $168 = HEAP32[$2>>2]|0;
  $169 = HEAP32[$168>>2]|0;
  $170 = (_sqlite3ResolveExprListNames($sNC,$169)|0);
  $171 = ($170|0)!=(0);
  if ($171) {
   label = 31;
   break;
  }
  $172 = HEAP32[$2>>2]|0;
  $173 = ((($172)) + 40|0);
  $174 = HEAP32[$173>>2]|0;
  HEAP32[$pGroupBy>>2] = $174;
  $175 = HEAP32[$pGroupBy>>2]|0;
  $176 = ($175|0)!=(0|0);
  if ($176) {
   label = 34;
  } else {
   $177 = ((($sNC)) + 28|0);
   $178 = HEAP16[$177>>1]|0;
   $179 = $178&65535;
   $180 = $179 & 2;
   $181 = ($180|0)!=(0);
   if ($181) {
    label = 34;
   } else {
    $193 = ((($sNC)) + 28|0);
    $194 = HEAP16[$193>>1]|0;
    $195 = $194&65535;
    $196 = $195 & -2;
    $197 = $196&65535;
    HEAP16[$193>>1] = $197;
   }
  }
  if ((label|0) == 34) {
   label = 0;
   $182 = ((($sNC)) + 28|0);
   $183 = HEAP16[$182>>1]|0;
   $184 = $183&65535;
   $185 = $184 & 4096;
   $186 = 8 | $185;
   $187 = HEAP32[$2>>2]|0;
   $188 = ((($187)) + 6|0);
   $189 = HEAP16[$188>>1]|0;
   $190 = $189&65535;
   $191 = $190 | $186;
   $192 = $191&65535;
   HEAP16[$188>>1] = $192;
  }
  $198 = HEAP32[$2>>2]|0;
  $199 = ((($198)) + 44|0);
  $200 = HEAP32[$199>>2]|0;
  $201 = ($200|0)==(0|0);
  $202 = HEAP32[$pGroupBy>>2]|0;
  $203 = ($202|0)!=(0|0);
  $or$cond = $201 | $203;
  if (!($or$cond)) {
   label = 37;
   break;
  }
  $205 = HEAP32[$2>>2]|0;
  $206 = HEAP32[$205>>2]|0;
  $207 = ((($sNC)) + 8|0);
  HEAP32[$207>>2] = $206;
  $208 = HEAP32[$2>>2]|0;
  $209 = ((($208)) + 44|0);
  $210 = HEAP32[$209>>2]|0;
  $211 = (_sqlite3ResolveExprNames($sNC,$210)|0);
  $212 = ($211|0)!=(0);
  if ($212) {
   label = 39;
   break;
  }
  $213 = HEAP32[$2>>2]|0;
  $214 = ((($213)) + 36|0);
  $215 = HEAP32[$214>>2]|0;
  $216 = (_sqlite3ResolveExprNames($sNC,$215)|0);
  $217 = ($216|0)!=(0);
  if ($217) {
   label = 41;
   break;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $218 = HEAP32[$i>>2]|0;
   $219 = HEAP32[$2>>2]|0;
   $220 = ((($219)) + 32|0);
   $221 = HEAP32[$220>>2]|0;
   $222 = HEAP32[$221>>2]|0;
   $223 = ($218|0)<($222|0);
   if (!($223)) {
    break;
   }
   $224 = HEAP32[$i>>2]|0;
   $225 = HEAP32[$2>>2]|0;
   $226 = ((($225)) + 32|0);
   $227 = HEAP32[$226>>2]|0;
   $228 = ((($227)) + 8|0);
   $229 = (($228) + (($224*72)|0)|0);
   HEAP32[$pItem1>>2] = $229;
   $230 = HEAP32[$pItem1>>2]|0;
   $231 = ((($230)) + 36|0);
   $232 = ((($231)) + 1|0);
   $233 = HEAP8[$232>>0]|0;
   $234 = ($233&255) >>> 2;
   $235 = $234 & 1;
   $236 = $235&255;
   $237 = ($236|0)!=(0);
   if ($237) {
    $238 = HEAP32[$pItem1>>2]|0;
    $239 = ((($238)) + 64|0);
    $240 = HEAP32[$239>>2]|0;
    $241 = (_sqlite3ResolveExprListNames($sNC,$240)|0);
    $242 = ($241|0)!=(0);
    if ($242) {
     label = 46;
     break L12;
    }
   }
   $243 = HEAP32[$i>>2]|0;
   $244 = (($243) + 1)|0;
   HEAP32[$i>>2] = $244;
  }
  $245 = ((($sNC)) + 16|0);
  HEAP32[$245>>2] = 0;
  $246 = ((($sNC)) + 28|0);
  $247 = HEAP16[$246>>1]|0;
  $248 = $247&65535;
  $249 = $248 | 1;
  $250 = $249&65535;
  HEAP16[$246>>1] = $250;
  $251 = HEAP32[$2>>2]|0;
  $252 = ((($251)) + 6|0);
  $253 = HEAP16[$252>>1]|0;
  $254 = $253&65535;
  $255 = $254 & 16384;
  $256 = ($255|0)!=(0);
  if ($256) {
   $257 = HEAP32[$2>>2]|0;
   $258 = ((($257)) + 32|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = ((($259)) + 8|0);
   $261 = ((($260)) + 20|0);
   $262 = HEAP32[$261>>2]|0;
   HEAP32[$pSub2>>2] = $262;
   $263 = HEAP32[$pSub2>>2]|0;
   $264 = ((($263)) + 48|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = HEAP32[$2>>2]|0;
   $267 = ((($266)) + 48|0);
   HEAP32[$267>>2] = $265;
   $268 = HEAP32[$pSub2>>2]|0;
   $269 = ((($268)) + 48|0);
   HEAP32[$269>>2] = 0;
  }
  $270 = HEAP32[$isCompound>>2]|0;
  $271 = HEAP32[$nCompound>>2]|0;
  $272 = ($270|0)<=($271|0);
  if ($272) {
   $273 = HEAP32[$2>>2]|0;
   $274 = HEAP32[$2>>2]|0;
   $275 = ((($274)) + 48|0);
   $276 = HEAP32[$275>>2]|0;
   $277 = (_resolveOrderGroupBy($sNC,$273,$276,29797)|0);
   $278 = ($277|0)!=(0);
   if ($278) {
    label = 52;
    break;
   }
  }
  $279 = HEAP32[$db>>2]|0;
  $280 = ((($279)) + 65|0);
  $281 = HEAP8[$280>>0]|0;
  $282 = ($281<<24>>24)!=(0);
  if ($282) {
   label = 54;
   break;
  }
  $283 = HEAP32[$pGroupBy>>2]|0;
  $284 = ($283|0)!=(0|0);
  L64: do {
   if ($284) {
    $285 = HEAP32[$2>>2]|0;
    $286 = HEAP32[$pGroupBy>>2]|0;
    $287 = (_resolveOrderGroupBy($sNC,$285,$286,29803)|0);
    $288 = ($287|0)!=(0);
    if ($288) {
     label = 58;
     break L12;
    }
    $289 = HEAP32[$db>>2]|0;
    $290 = ((($289)) + 65|0);
    $291 = HEAP8[$290>>0]|0;
    $292 = $291&255;
    $293 = ($292|0)!=(0);
    if ($293) {
     label = 58;
     break L12;
    }
    HEAP32[$i>>2] = 0;
    $294 = HEAP32[$pGroupBy>>2]|0;
    $295 = ((($294)) + 4|0);
    $296 = HEAP32[$295>>2]|0;
    HEAP32[$pItem3>>2] = $296;
    while(1) {
     $297 = HEAP32[$i>>2]|0;
     $298 = HEAP32[$pGroupBy>>2]|0;
     $299 = HEAP32[$298>>2]|0;
     $300 = ($297|0)<($299|0);
     if (!($300)) {
      break L64;
     }
     $301 = HEAP32[$pItem3>>2]|0;
     $302 = HEAP32[$301>>2]|0;
     $303 = ((($302)) + 4|0);
     $304 = HEAP32[$303>>2]|0;
     $305 = $304 & 2;
     $306 = ($305|0)!=(0);
     if ($306) {
      label = 62;
      break L12;
     }
     $308 = HEAP32[$i>>2]|0;
     $309 = (($308) + 1)|0;
     HEAP32[$i>>2] = $309;
     $310 = HEAP32[$pItem3>>2]|0;
     $311 = ((($310)) + 20|0);
     HEAP32[$pItem3>>2] = $311;
    }
   }
  } while(0);
  $312 = HEAP32[$2>>2]|0;
  $313 = ((($312)) + 56|0);
  $314 = HEAP32[$313>>2]|0;
  $315 = ($314|0)!=(0|0);
  if ($315) {
   $316 = HEAP32[$2>>2]|0;
   $317 = HEAP32[$316>>2]|0;
   $318 = HEAP32[$317>>2]|0;
   $319 = HEAP32[$2>>2]|0;
   $320 = ((($319)) + 56|0);
   $321 = HEAP32[$320>>2]|0;
   $322 = HEAP32[$321>>2]|0;
   $323 = HEAP32[$322>>2]|0;
   $324 = ($318|0)!=($323|0);
   if ($324) {
    label = 66;
    break;
   }
  }
  $329 = HEAP32[$2>>2]|0;
  $330 = ((($329)) + 52|0);
  $331 = HEAP32[$330>>2]|0;
  HEAP32[$2>>2] = $331;
  $332 = HEAP32[$nCompound>>2]|0;
  $333 = (($332) + 1)|0;
  HEAP32[$nCompound>>2] = $333;
 }
 switch (label|0) {
  case 11: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 24: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 31: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 37: {
   $204 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($204,29753,$vararg_buffer);
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 39: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 41: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 46: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 52: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 54: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 58: {
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 62: {
   $307 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($307,29809,$vararg_buffer2);
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 66: {
   $325 = HEAP32[$pParse>>2]|0;
   $326 = HEAP32[$2>>2]|0;
   $327 = ((($326)) + 56|0);
   $328 = HEAP32[$327>>2]|0;
   _sqlite3SelectWrongNumTermsError($325,$328);
   HEAP32[$0>>2] = 2;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
  case 68: {
   $334 = HEAP32[$isCompound>>2]|0;
   $335 = ($334|0)!=(0);
   if ($335) {
    $336 = HEAP32[$pParse>>2]|0;
    $337 = HEAP32[$pLeftmost>>2]|0;
    $338 = (_resolveCompoundOrderBy($336,$337)|0);
    $339 = ($338|0)!=(0);
    if ($339) {
     HEAP32[$0>>2] = 2;
     $340 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($340|0);
    }
   }
   HEAP32[$0>>2] = 1;
   $340 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($340|0);
   break;
  }
 }
 return (0)|0;
}
function _lookupName($pParse,$zDb,$zTab,$zCol,$pNC,$pExpr) {
 $pParse = $pParse|0;
 $zDb = $zDb|0;
 $zTab = $zTab|0;
 $zCol = $zCol|0;
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $cnt = 0, $cntTab = 0, $db = 0, $hit = 0, $i = 0, $iCol = 0, $isTrigger = 0, $j = 0, $n = 0, $nSubquery = 0, $op = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0;
 var $or$cond9 = 0, $pCol = 0, $pEList = 0, $pItem = 0, $pMatch = 0, $pOrig = 0, $pSchema = 0, $pSrcList = 0, $pTab = 0, $pTopNC = 0, $vararg_buffer = 0, $vararg_buffer18 = 0, $vararg_buffer24 = 0, $vararg_buffer29 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr27 = 0, $vararg_ptr28 = 0, $vararg_ptr32 = 0;
 var $zAs = 0, $zErr = 0, $zTabName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer29 = sp + 40|0;
 $vararg_buffer24 = sp + 24|0;
 $vararg_buffer18 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 164|0;
 $1 = sp + 160|0;
 $2 = sp + 156|0;
 $3 = sp + 152|0;
 $4 = sp + 148|0;
 $5 = sp + 144|0;
 $6 = sp + 140|0;
 $i = sp + 136|0;
 $j = sp + 132|0;
 $cnt = sp + 128|0;
 $cntTab = sp + 124|0;
 $nSubquery = sp + 120|0;
 $db = sp + 116|0;
 $pItem = sp + 112|0;
 $pMatch = sp + 108|0;
 $pTopNC = sp + 104|0;
 $pSchema = sp + 100|0;
 $isTrigger = sp + 96|0;
 $pTab = sp + 92|0;
 $pCol = sp + 88|0;
 $pEList = sp + 84|0;
 $pSrcList = sp + 80|0;
 $hit = sp + 76|0;
 $zTabName = sp + 72|0;
 $op = sp + 68|0;
 $iCol = sp + 64|0;
 $zAs = sp + 60|0;
 $pOrig = sp + 56|0;
 $zErr = sp + 52|0;
 $n = sp + 48|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$4>>2] = $zCol;
 HEAP32[$5>>2] = $pNC;
 HEAP32[$6>>2] = $pExpr;
 HEAP32[$cnt>>2] = 0;
 HEAP32[$cntTab>>2] = 0;
 HEAP32[$nSubquery>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 HEAP32[$pMatch>>2] = 0;
 $9 = HEAP32[$5>>2]|0;
 HEAP32[$pTopNC>>2] = $9;
 HEAP32[$pSchema>>2] = 0;
 HEAP32[$isTrigger>>2] = 0;
 HEAP32[$pTab>>2] = 0;
 $10 = HEAP32[$6>>2]|0;
 $11 = ((($10)) + 28|0);
 HEAP32[$11>>2] = -1;
 $12 = HEAP32[$6>>2]|0;
 $13 = ((($12)) + 44|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)!=(0|0);
 L1: do {
  if ($15) {
   $16 = HEAP32[$5>>2]|0;
   $17 = ((($16)) + 28|0);
   $18 = HEAP16[$17>>1]|0;
   $19 = $18&65535;
   $20 = $19 & 20;
   $21 = ($20|0)!=(0);
   if ($21) {
    HEAP32[$2>>2] = 0;
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $22 = HEAP32[$i>>2]|0;
    $23 = HEAP32[$db>>2]|0;
    $24 = ((($23)) + 20|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)<($25|0);
    if (!($26)) {
     break L1;
    }
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$db>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = (_sqlite3_stricmp($32,$33)|0);
    $35 = ($34|0)==(0);
    $36 = HEAP32[$i>>2]|0;
    if ($35) {
     break;
    }
    $43 = (($36) + 1)|0;
    HEAP32[$i>>2] = $43;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = ((($37)) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($36<<4)|0);
   $41 = ((($40)) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   HEAP32[$pSchema>>2] = $42;
  }
 } while(0);
 L11: while(1) {
  $44 = HEAP32[$5>>2]|0;
  $45 = ($44|0)!=(0|0);
  $46 = HEAP32[$cnt>>2]|0;
  $47 = ($46|0)==(0);
  $48 = $45 ? $47 : 0;
  if (!($48)) {
   label = 89;
   break;
  }
  $49 = HEAP32[$5>>2]|0;
  $50 = ((($49)) + 4|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$pSrcList>>2] = $51;
  $52 = HEAP32[$pSrcList>>2]|0;
  $53 = ($52|0)!=(0|0);
  if ($53) {
   HEAP32[$i>>2] = 0;
   $54 = HEAP32[$pSrcList>>2]|0;
   $55 = ((($54)) + 8|0);
   HEAP32[$pItem>>2] = $55;
   while(1) {
    $56 = HEAP32[$i>>2]|0;
    $57 = HEAP32[$pSrcList>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    $59 = ($56|0)<($58|0);
    if (!($59)) {
     break;
    }
    $60 = HEAP32[$pItem>>2]|0;
    $61 = ((($60)) + 16|0);
    $62 = HEAP32[$61>>2]|0;
    HEAP32[$pTab>>2] = $62;
    $63 = HEAP32[$pItem>>2]|0;
    $64 = ((($63)) + 20|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = ($65|0)!=(0|0);
    if ($66) {
     $67 = HEAP32[$pItem>>2]|0;
     $68 = ((($67)) + 20|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ((($69)) + 6|0);
     $71 = HEAP16[$70>>1]|0;
     $72 = $71&65535;
     $73 = $72 & 1024;
     $74 = ($73|0)!=(0);
     if ($74) {
      HEAP32[$hit>>2] = 0;
      $75 = HEAP32[$pItem>>2]|0;
      $76 = ((($75)) + 20|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = HEAP32[$77>>2]|0;
      HEAP32[$pEList>>2] = $78;
      HEAP32[$j>>2] = 0;
      while(1) {
       $79 = HEAP32[$j>>2]|0;
       $80 = HEAP32[$pEList>>2]|0;
       $81 = HEAP32[$80>>2]|0;
       $82 = ($79|0)<($81|0);
       if (!($82)) {
        break;
       }
       $83 = HEAP32[$j>>2]|0;
       $84 = HEAP32[$pEList>>2]|0;
       $85 = ((($84)) + 4|0);
       $86 = HEAP32[$85>>2]|0;
       $87 = (($86) + (($83*20)|0)|0);
       $88 = ((($87)) + 8|0);
       $89 = HEAP32[$88>>2]|0;
       $90 = HEAP32[$4>>2]|0;
       $91 = HEAP32[$3>>2]|0;
       $92 = HEAP32[$2>>2]|0;
       $93 = (_sqlite3MatchSpanName($89,$90,$91,$92)|0);
       $94 = ($93|0)!=(0);
       if ($94) {
        $95 = HEAP32[$cnt>>2]|0;
        $96 = (($95) + 1)|0;
        HEAP32[$cnt>>2] = $96;
        HEAP32[$cntTab>>2] = 2;
        $97 = HEAP32[$pItem>>2]|0;
        HEAP32[$pMatch>>2] = $97;
        $98 = HEAP32[$j>>2]|0;
        $99 = $98&65535;
        $100 = HEAP32[$6>>2]|0;
        $101 = ((($100)) + 32|0);
        HEAP16[$101>>1] = $99;
        HEAP32[$hit>>2] = 1;
       }
       $102 = HEAP32[$j>>2]|0;
       $103 = (($102) + 1)|0;
       HEAP32[$j>>2] = $103;
      }
      $104 = HEAP32[$hit>>2]|0;
      $105 = ($104|0)!=(0);
      $106 = HEAP32[$3>>2]|0;
      $107 = ($106|0)==(0|0);
      $or$cond = $105 | $107;
      if (!($or$cond)) {
       label = 21;
      }
     } else {
      label = 21;
     }
    } else {
     label = 21;
    }
    L29: do {
     if ((label|0) == 21) {
      label = 0;
      $108 = HEAP32[$2>>2]|0;
      $109 = ($108|0)!=(0|0);
      if ($109) {
       $110 = HEAP32[$pTab>>2]|0;
       $111 = ((($110)) + 64|0);
       $112 = HEAP32[$111>>2]|0;
       $113 = HEAP32[$pSchema>>2]|0;
       $114 = ($112|0)!=($113|0);
       if ($114) {
        break;
       }
      }
      $115 = HEAP32[$3>>2]|0;
      $116 = ($115|0)!=(0|0);
      if ($116) {
       $117 = HEAP32[$pItem>>2]|0;
       $118 = ((($117)) + 12|0);
       $119 = HEAP32[$118>>2]|0;
       $120 = ($119|0)!=(0|0);
       if ($120) {
        $121 = HEAP32[$pItem>>2]|0;
        $122 = ((($121)) + 12|0);
        $123 = HEAP32[$122>>2]|0;
        $126 = $123;
       } else {
        $124 = HEAP32[$pTab>>2]|0;
        $125 = HEAP32[$124>>2]|0;
        $126 = $125;
       }
       HEAP32[$zTabName>>2] = $126;
       $127 = HEAP32[$zTabName>>2]|0;
       $128 = HEAP32[$3>>2]|0;
       $129 = (_sqlite3_stricmp($127,$128)|0);
       $130 = ($129|0)!=(0);
       if ($130) {
        break;
       }
      }
      $131 = HEAP32[$cntTab>>2]|0;
      $132 = (($131) + 1)|0;
      HEAP32[$cntTab>>2] = $132;
      $133 = (0)==($131|0);
      if ($133) {
       $134 = HEAP32[$pItem>>2]|0;
       HEAP32[$pMatch>>2] = $134;
      }
      HEAP32[$j>>2] = 0;
      $135 = HEAP32[$pTab>>2]|0;
      $136 = ((($135)) + 4|0);
      $137 = HEAP32[$136>>2]|0;
      HEAP32[$pCol>>2] = $137;
      while(1) {
       $138 = HEAP32[$j>>2]|0;
       $139 = HEAP32[$pTab>>2]|0;
       $140 = ((($139)) + 34|0);
       $141 = HEAP16[$140>>1]|0;
       $142 = $141 << 16 >> 16;
       $143 = ($138|0)<($142|0);
       if (!($143)) {
        break L29;
       }
       $144 = HEAP32[$pCol>>2]|0;
       $145 = HEAP32[$144>>2]|0;
       $146 = HEAP32[$4>>2]|0;
       $147 = (_sqlite3_stricmp($145,$146)|0);
       $148 = ($147|0)==(0);
       if ($148) {
        $149 = HEAP32[$cnt>>2]|0;
        $150 = ($149|0)==(1);
        if (!($150)) {
         break;
        }
        $151 = HEAP32[$pItem>>2]|0;
        $152 = ((($151)) + 36|0);
        $153 = HEAP8[$152>>0]|0;
        $154 = $153&255;
        $155 = $154 & 4;
        $156 = ($155|0)!=(0);
        if (!($156)) {
         $157 = HEAP32[$pItem>>2]|0;
         $158 = ((($157)) + 52|0);
         $159 = HEAP32[$158>>2]|0;
         $160 = HEAP32[$4>>2]|0;
         $161 = (_nameInUsingClause($159,$160)|0);
         $162 = ($161|0)!=(0);
         if (!($162)) {
          break;
         }
        }
       }
       $179 = HEAP32[$j>>2]|0;
       $180 = (($179) + 1)|0;
       HEAP32[$j>>2] = $180;
       $181 = HEAP32[$pCol>>2]|0;
       $182 = ((($181)) + 24|0);
       HEAP32[$pCol>>2] = $182;
      }
      $163 = HEAP32[$cnt>>2]|0;
      $164 = (($163) + 1)|0;
      HEAP32[$cnt>>2] = $164;
      $165 = HEAP32[$pItem>>2]|0;
      HEAP32[$pMatch>>2] = $165;
      $166 = HEAP32[$j>>2]|0;
      $167 = HEAP32[$pTab>>2]|0;
      $168 = ((($167)) + 32|0);
      $169 = HEAP16[$168>>1]|0;
      $170 = $169 << 16 >> 16;
      $171 = ($166|0)==($170|0);
      if ($171) {
       $176 = -1;
      } else {
       $172 = HEAP32[$j>>2]|0;
       $173 = $172&65535;
       $174 = $173 << 16 >> 16;
       $176 = $174;
      }
      $175 = $176&65535;
      $177 = HEAP32[$6>>2]|0;
      $178 = ((($177)) + 32|0);
      HEAP16[$178>>1] = $175;
     }
    } while(0);
    $183 = HEAP32[$i>>2]|0;
    $184 = (($183) + 1)|0;
    HEAP32[$i>>2] = $184;
    $185 = HEAP32[$pItem>>2]|0;
    $186 = ((($185)) + 72|0);
    HEAP32[$pItem>>2] = $186;
   }
   $187 = HEAP32[$pMatch>>2]|0;
   $188 = ($187|0)!=(0|0);
   if ($188) {
    $189 = HEAP32[$pMatch>>2]|0;
    $190 = ((($189)) + 44|0);
    $191 = HEAP32[$190>>2]|0;
    $192 = HEAP32[$6>>2]|0;
    $193 = ((($192)) + 28|0);
    HEAP32[$193>>2] = $191;
    $194 = HEAP32[$pMatch>>2]|0;
    $195 = ((($194)) + 16|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = HEAP32[$6>>2]|0;
    $198 = ((($197)) + 44|0);
    HEAP32[$198>>2] = $196;
    $199 = HEAP32[$pMatch>>2]|0;
    $200 = ((($199)) + 36|0);
    $201 = HEAP8[$200>>0]|0;
    $202 = $201&255;
    $203 = $202 & 8;
    $204 = ($203|0)!=(0);
    if ($204) {
     $205 = HEAP32[$6>>2]|0;
     $206 = ((($205)) + 4|0);
     $207 = HEAP32[$206>>2]|0;
     $208 = $207 | 1048576;
     HEAP32[$206>>2] = $208;
    }
    $209 = HEAP32[$6>>2]|0;
    $210 = ((($209)) + 44|0);
    $211 = HEAP32[$210>>2]|0;
    $212 = ((($211)) + 64|0);
    $213 = HEAP32[$212>>2]|0;
    HEAP32[$pSchema>>2] = $213;
   }
  }
  $214 = HEAP32[$2>>2]|0;
  $215 = ($214|0)==(0|0);
  $216 = HEAP32[$3>>2]|0;
  $217 = ($216|0)!=(0|0);
  $or$cond3 = $215 & $217;
  $218 = HEAP32[$cntTab>>2]|0;
  $219 = ($218|0)==(0);
  $or$cond5 = $or$cond3 & $219;
  if ($or$cond5) {
   $220 = HEAP32[$1>>2]|0;
   $221 = ((($220)) + 420|0);
   $222 = HEAP32[$221>>2]|0;
   $223 = ($222|0)!=(0|0);
   if ($223) {
    $224 = HEAP32[$1>>2]|0;
    $225 = ((($224)) + 440|0);
    $226 = HEAP8[$225>>0]|0;
    $227 = $226&255;
    HEAP32[$op>>2] = $227;
    $228 = HEAP32[$op>>2]|0;
    $229 = ($228|0)!=(109);
    if ($229) {
     $230 = HEAP32[$3>>2]|0;
     $231 = (_sqlite3_stricmp(29511,$230)|0);
     $232 = ($231|0)==(0);
     if ($232) {
      $233 = HEAP32[$6>>2]|0;
      $234 = ((($233)) + 28|0);
      HEAP32[$234>>2] = 1;
      $235 = HEAP32[$1>>2]|0;
      $236 = ((($235)) + 420|0);
      $237 = HEAP32[$236>>2]|0;
      HEAP32[$pTab>>2] = $237;
     } else {
      label = 50;
     }
    } else {
     label = 50;
    }
    do {
     if ((label|0) == 50) {
      label = 0;
      $238 = HEAP32[$op>>2]|0;
      $239 = ($238|0)!=(108);
      if ($239) {
       $240 = HEAP32[$3>>2]|0;
       $241 = (_sqlite3_stricmp(29515,$240)|0);
       $242 = ($241|0)==(0);
       if ($242) {
        $243 = HEAP32[$6>>2]|0;
        $244 = ((($243)) + 28|0);
        HEAP32[$244>>2] = 0;
        $245 = HEAP32[$1>>2]|0;
        $246 = ((($245)) + 420|0);
        $247 = HEAP32[$246>>2]|0;
        HEAP32[$pTab>>2] = $247;
        break;
       }
      }
      HEAP32[$pTab>>2] = 0;
     }
    } while(0);
    $248 = HEAP32[$pTab>>2]|0;
    $249 = ($248|0)!=(0|0);
    if ($249) {
     $250 = HEAP32[$pTab>>2]|0;
     $251 = ((($250)) + 64|0);
     $252 = HEAP32[$251>>2]|0;
     HEAP32[$pSchema>>2] = $252;
     $253 = HEAP32[$cntTab>>2]|0;
     $254 = (($253) + 1)|0;
     HEAP32[$cntTab>>2] = $254;
     HEAP32[$iCol>>2] = 0;
     $255 = HEAP32[$pTab>>2]|0;
     $256 = ((($255)) + 4|0);
     $257 = HEAP32[$256>>2]|0;
     HEAP32[$pCol>>2] = $257;
     while(1) {
      $258 = HEAP32[$iCol>>2]|0;
      $259 = HEAP32[$pTab>>2]|0;
      $260 = ((($259)) + 34|0);
      $261 = HEAP16[$260>>1]|0;
      $262 = $261 << 16 >> 16;
      $263 = ($258|0)<($262|0);
      if (!($263)) {
       break;
      }
      $264 = HEAP32[$pCol>>2]|0;
      $265 = HEAP32[$264>>2]|0;
      $266 = HEAP32[$4>>2]|0;
      $267 = (_sqlite3_stricmp($265,$266)|0);
      $268 = ($267|0)==(0);
      $269 = HEAP32[$iCol>>2]|0;
      if ($268) {
       label = 58;
       break;
      }
      $275 = (($269) + 1)|0;
      HEAP32[$iCol>>2] = $275;
      $276 = HEAP32[$pCol>>2]|0;
      $277 = ((($276)) + 24|0);
      HEAP32[$pCol>>2] = $277;
     }
     if ((label|0) == 58) {
      label = 0;
      $270 = HEAP32[$pTab>>2]|0;
      $271 = ((($270)) + 32|0);
      $272 = HEAP16[$271>>1]|0;
      $273 = $272 << 16 >> 16;
      $274 = ($269|0)==($273|0);
      if ($274) {
       HEAP32[$iCol>>2] = -1;
      }
     }
     $278 = HEAP32[$iCol>>2]|0;
     $279 = HEAP32[$pTab>>2]|0;
     $280 = ((($279)) + 34|0);
     $281 = HEAP16[$280>>1]|0;
     $282 = $281 << 16 >> 16;
     $283 = ($278|0)>=($282|0);
     if ($283) {
      $284 = HEAP32[$4>>2]|0;
      $285 = (_sqlite3IsRowid($284)|0);
      $286 = ($285|0)!=(0);
      if ($286) {
       $287 = HEAP32[$pTab>>2]|0;
       $288 = ((($287)) + 42|0);
       $289 = HEAP8[$288>>0]|0;
       $290 = $289&255;
       $291 = $290 & 64;
       $292 = ($291|0)==(0);
       if ($292) {
        HEAP32[$iCol>>2] = -1;
       }
      }
     }
     $293 = HEAP32[$iCol>>2]|0;
     $294 = HEAP32[$pTab>>2]|0;
     $295 = ((($294)) + 34|0);
     $296 = HEAP16[$295>>1]|0;
     $297 = $296 << 16 >> 16;
     $298 = ($293|0)<($297|0);
     if ($298) {
      $299 = HEAP32[$cnt>>2]|0;
      $300 = (($299) + 1)|0;
      HEAP32[$cnt>>2] = $300;
      $301 = HEAP32[$iCol>>2]|0;
      $302 = ($301|0)<(0);
      $303 = HEAP32[$6>>2]|0;
      do {
       if ($302) {
        $304 = ((($303)) + 1|0);
        HEAP8[$304>>0] = 68;
       } else {
        $305 = ((($303)) + 28|0);
        $306 = HEAP32[$305>>2]|0;
        $307 = ($306|0)==(0);
        $308 = HEAP32[$iCol>>2]|0;
        $309 = ($308|0)>=(32);
        $310 = HEAP32[$iCol>>2]|0;
        $311 = 1 << $310;
        $312 = $309 ? -1 : $311;
        $313 = HEAP32[$1>>2]|0;
        if ($307) {
         $314 = ((($313)) + 432|0);
         $315 = HEAP32[$314>>2]|0;
         $316 = $315 | $312;
         HEAP32[$314>>2] = $316;
         break;
        } else {
         $317 = ((($313)) + 436|0);
         $318 = HEAP32[$317>>2]|0;
         $319 = $318 | $312;
         HEAP32[$317>>2] = $319;
         break;
        }
       }
      } while(0);
      $320 = HEAP32[$iCol>>2]|0;
      $321 = $320&65535;
      $322 = HEAP32[$6>>2]|0;
      $323 = ((($322)) + 32|0);
      HEAP16[$323>>1] = $321;
      $324 = HEAP32[$pTab>>2]|0;
      $325 = HEAP32[$6>>2]|0;
      $326 = ((($325)) + 44|0);
      HEAP32[$326>>2] = $324;
      HEAP32[$isTrigger>>2] = 1;
     }
    }
   }
  }
  $327 = HEAP32[$cnt>>2]|0;
  $328 = ($327|0)==(0);
  $329 = HEAP32[$cntTab>>2]|0;
  $330 = ($329|0)==(1);
  $or$cond7 = $328 & $330;
  $331 = HEAP32[$pMatch>>2]|0;
  $332 = ($331|0)!=(0|0);
  $or$cond9 = $or$cond7 & $332;
  if ($or$cond9) {
   $333 = HEAP32[$5>>2]|0;
   $334 = ((($333)) + 28|0);
   $335 = HEAP16[$334>>1]|0;
   $336 = $335&65535;
   $337 = $336 & 32;
   $338 = ($337|0)==(0);
   if ($338) {
    $339 = HEAP32[$4>>2]|0;
    $340 = (_sqlite3IsRowid($339)|0);
    $341 = ($340|0)!=(0);
    if ($341) {
     $342 = HEAP32[$pMatch>>2]|0;
     $343 = ((($342)) + 16|0);
     $344 = HEAP32[$343>>2]|0;
     $345 = ((($344)) + 42|0);
     $346 = HEAP8[$345>>0]|0;
     $347 = $346&255;
     $348 = $347 & 64;
     $349 = ($348|0)==(0);
     if ($349) {
      HEAP32[$cnt>>2] = 1;
      $350 = HEAP32[$6>>2]|0;
      $351 = ((($350)) + 32|0);
      HEAP16[$351>>1] = -1;
      $352 = HEAP32[$6>>2]|0;
      $353 = ((($352)) + 1|0);
      HEAP8[$353>>0] = 68;
     }
    }
   }
  }
  $354 = HEAP32[$5>>2]|0;
  $355 = ((($354)) + 8|0);
  $356 = HEAP32[$355>>2]|0;
  HEAP32[$pEList>>2] = $356;
  $357 = ($356|0)!=(0|0);
  $358 = HEAP32[$3>>2]|0;
  $359 = ($358|0)==(0|0);
  $or$cond11 = $357 & $359;
  $360 = HEAP32[$cnt>>2]|0;
  $361 = ($360|0)==(0);
  $or$cond13 = $or$cond11 & $361;
  L105: do {
   if ($or$cond13) {
    HEAP32[$j>>2] = 0;
    while(1) {
     $362 = HEAP32[$j>>2]|0;
     $363 = HEAP32[$pEList>>2]|0;
     $364 = HEAP32[$363>>2]|0;
     $365 = ($362|0)<($364|0);
     if (!($365)) {
      break L105;
     }
     $366 = HEAP32[$j>>2]|0;
     $367 = HEAP32[$pEList>>2]|0;
     $368 = ((($367)) + 4|0);
     $369 = HEAP32[$368>>2]|0;
     $370 = (($369) + (($366*20)|0)|0);
     $371 = ((($370)) + 4|0);
     $372 = HEAP32[$371>>2]|0;
     HEAP32[$zAs>>2] = $372;
     $373 = HEAP32[$zAs>>2]|0;
     $374 = ($373|0)!=(0|0);
     if ($374) {
      $375 = HEAP32[$zAs>>2]|0;
      $376 = HEAP32[$4>>2]|0;
      $377 = (_sqlite3_stricmp($375,$376)|0);
      $378 = ($377|0)==(0);
      if ($378) {
       label = 82;
       break L11;
      }
     }
     $403 = HEAP32[$j>>2]|0;
     $404 = (($403) + 1)|0;
     HEAP32[$j>>2] = $404;
    }
   }
  } while(0);
  $405 = HEAP32[$cnt>>2]|0;
  $406 = ($405|0)==(0);
  if (!($406)) {
   continue;
  }
  $407 = HEAP32[$5>>2]|0;
  $408 = ((($407)) + 16|0);
  $409 = HEAP32[$408>>2]|0;
  HEAP32[$5>>2] = $409;
  $410 = HEAP32[$nSubquery>>2]|0;
  $411 = (($410) + 1)|0;
  HEAP32[$nSubquery>>2] = $411;
 }
 if ((label|0) == 82) {
  $379 = HEAP32[$j>>2]|0;
  $380 = HEAP32[$pEList>>2]|0;
  $381 = ((($380)) + 4|0);
  $382 = HEAP32[$381>>2]|0;
  $383 = (($382) + (($379*20)|0)|0);
  $384 = HEAP32[$383>>2]|0;
  HEAP32[$pOrig>>2] = $384;
  $385 = HEAP32[$5>>2]|0;
  $386 = ((($385)) + 28|0);
  $387 = HEAP16[$386>>1]|0;
  $388 = $387&65535;
  $389 = $388 & 1;
  $390 = ($389|0)==(0);
  if ($390) {
   $391 = HEAP32[$pOrig>>2]|0;
   $392 = ((($391)) + 4|0);
   $393 = HEAP32[$392>>2]|0;
   $394 = $393 & 2;
   $395 = ($394|0)!=(0);
   if ($395) {
    $396 = HEAP32[$1>>2]|0;
    $397 = HEAP32[$zAs>>2]|0;
    HEAP32[$vararg_buffer>>2] = $397;
    _sqlite3ErrorMsg($396,29519,$vararg_buffer);
    HEAP32[$0>>2] = 2;
    $519 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($519|0);
   }
  }
  $398 = HEAP32[$1>>2]|0;
  $399 = HEAP32[$pEList>>2]|0;
  $400 = HEAP32[$j>>2]|0;
  $401 = HEAP32[$6>>2]|0;
  $402 = HEAP32[$nSubquery>>2]|0;
  _resolveAlias($398,$399,$400,$401,17532,$402);
  HEAP32[$cnt>>2] = 1;
  HEAP32[$pMatch>>2] = 0;
 }
 else if ((label|0) == 89) {
  $412 = HEAP32[$cnt>>2]|0;
  $413 = ($412|0)==(0);
  $414 = HEAP32[$3>>2]|0;
  $415 = ($414|0)==(0|0);
  $or$cond15 = $413 & $415;
  if ($or$cond15) {
   $416 = HEAP32[$6>>2]|0;
   $417 = ((($416)) + 4|0);
   $418 = HEAP32[$417>>2]|0;
   $419 = $418 & 64;
   $420 = ($419|0)!=(0);
   if ($420) {
    $421 = HEAP32[$6>>2]|0;
    HEAP8[$421>>0] = 97;
    $422 = HEAP32[$6>>2]|0;
    $423 = ((($422)) + 44|0);
    HEAP32[$423>>2] = 0;
    HEAP32[$0>>2] = 1;
    $519 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($519|0);
   }
  }
  $424 = HEAP32[$cnt>>2]|0;
  $425 = ($424|0)!=(1);
  if ($425) {
   $426 = HEAP32[$cnt>>2]|0;
   $427 = ($426|0)==(0);
   $428 = $427 ? 29550 : 29565;
   HEAP32[$zErr>>2] = $428;
   $429 = HEAP32[$2>>2]|0;
   $430 = ($429|0)!=(0|0);
   do {
    if ($430) {
     $431 = HEAP32[$1>>2]|0;
     $432 = HEAP32[$zErr>>2]|0;
     $433 = HEAP32[$2>>2]|0;
     $434 = HEAP32[$3>>2]|0;
     $435 = HEAP32[$4>>2]|0;
     HEAP32[$vararg_buffer18>>2] = $432;
     $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
     HEAP32[$vararg_ptr21>>2] = $433;
     $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
     HEAP32[$vararg_ptr22>>2] = $434;
     $vararg_ptr23 = ((($vararg_buffer18)) + 12|0);
     HEAP32[$vararg_ptr23>>2] = $435;
     _sqlite3ErrorMsg($431,29587,$vararg_buffer18);
    } else {
     $436 = HEAP32[$3>>2]|0;
     $437 = ($436|0)!=(0|0);
     $438 = HEAP32[$1>>2]|0;
     $439 = HEAP32[$zErr>>2]|0;
     if ($437) {
      $440 = HEAP32[$3>>2]|0;
      $441 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer24>>2] = $439;
      $vararg_ptr27 = ((($vararg_buffer24)) + 4|0);
      HEAP32[$vararg_ptr27>>2] = $440;
      $vararg_ptr28 = ((($vararg_buffer24)) + 8|0);
      HEAP32[$vararg_ptr28>>2] = $441;
      _sqlite3ErrorMsg($438,25537,$vararg_buffer24);
      break;
     } else {
      $442 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer29>>2] = $439;
      $vararg_ptr32 = ((($vararg_buffer29)) + 4|0);
      HEAP32[$vararg_ptr32>>2] = $442;
      _sqlite3ErrorMsg($438,25547,$vararg_buffer29);
      break;
     }
    }
   } while(0);
   $443 = HEAP32[$1>>2]|0;
   $444 = ((($443)) + 17|0);
   HEAP8[$444>>0] = 1;
   $445 = HEAP32[$pTopNC>>2]|0;
   $446 = ((($445)) + 24|0);
   $447 = HEAP32[$446>>2]|0;
   $448 = (($447) + 1)|0;
   HEAP32[$446>>2] = $448;
  }
  $449 = HEAP32[$6>>2]|0;
  $450 = ((($449)) + 32|0);
  $451 = HEAP16[$450>>1]|0;
  $452 = $451 << 16 >> 16;
  $453 = ($452|0)>=(0);
  $454 = HEAP32[$pMatch>>2]|0;
  $455 = ($454|0)!=(0|0);
  $or$cond17 = $453 & $455;
  if ($or$cond17) {
   $456 = HEAP32[$6>>2]|0;
   $457 = ((($456)) + 32|0);
   $458 = HEAP16[$457>>1]|0;
   $459 = $458 << 16 >> 16;
   HEAP32[$n>>2] = $459;
   $460 = HEAP32[$n>>2]|0;
   $461 = ($460|0)>=(64);
   $$ = $461 ? 63 : $459;
   HEAP32[$n>>2] = $$;
   $462 = HEAP32[$n>>2]|0;
   $463 = (_bitshift64Shl(1,0,($462|0))|0);
   $464 = tempRet0;
   $465 = HEAP32[$pMatch>>2]|0;
   $466 = ((($465)) + 56|0);
   $467 = $466;
   $468 = $467;
   $469 = HEAP32[$468>>2]|0;
   $470 = (($467) + 4)|0;
   $471 = $470;
   $472 = HEAP32[$471>>2]|0;
   $473 = $469 | $463;
   $474 = $472 | $464;
   $475 = $466;
   $476 = $475;
   HEAP32[$476>>2] = $473;
   $477 = (($475) + 4)|0;
   $478 = $477;
   HEAP32[$478>>2] = $474;
  }
  $479 = HEAP32[$db>>2]|0;
  $480 = HEAP32[$6>>2]|0;
  $481 = ((($480)) + 12|0);
  $482 = HEAP32[$481>>2]|0;
  _sqlite3ExprDelete($479,$482);
  $483 = HEAP32[$6>>2]|0;
  $484 = ((($483)) + 12|0);
  HEAP32[$484>>2] = 0;
  $485 = HEAP32[$db>>2]|0;
  $486 = HEAP32[$6>>2]|0;
  $487 = ((($486)) + 16|0);
  $488 = HEAP32[$487>>2]|0;
  _sqlite3ExprDelete($485,$488);
  $489 = HEAP32[$6>>2]|0;
  $490 = ((($489)) + 16|0);
  HEAP32[$490>>2] = 0;
  $491 = HEAP32[$isTrigger>>2]|0;
  $492 = ($491|0)!=(0);
  $493 = $492 ? 62 : 152;
  $494 = $493&255;
  $495 = HEAP32[$6>>2]|0;
  HEAP8[$495>>0] = $494;
 }
 $496 = HEAP32[$cnt>>2]|0;
 $497 = ($496|0)==(1);
 if (!($497)) {
  HEAP32[$0>>2] = 2;
  $519 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $498 = HEAP32[$6>>2]|0;
 $499 = ((($498)) + 4|0);
 $500 = HEAP32[$499>>2]|0;
 $501 = $500 & 4194304;
 $502 = ($501|0)!=(0);
 if (!($502)) {
  $503 = HEAP32[$1>>2]|0;
  $504 = HEAP32[$6>>2]|0;
  $505 = HEAP32[$pSchema>>2]|0;
  $506 = HEAP32[$5>>2]|0;
  $507 = ((($506)) + 4|0);
  $508 = HEAP32[$507>>2]|0;
  _sqlite3AuthRead($503,$504,$505,$508);
 }
 while(1) {
  $509 = HEAP32[$pTopNC>>2]|0;
  $510 = ((($509)) + 20|0);
  $511 = HEAP32[$510>>2]|0;
  $512 = (($511) + 1)|0;
  HEAP32[$510>>2] = $512;
  $513 = HEAP32[$pTopNC>>2]|0;
  $514 = HEAP32[$5>>2]|0;
  $515 = ($513|0)==($514|0);
  if ($515) {
   break;
  }
  $516 = HEAP32[$pTopNC>>2]|0;
  $517 = ((($516)) + 16|0);
  $518 = HEAP32[$517>>2]|0;
  HEAP32[$pTopNC>>2] = $518;
 }
 HEAP32[$0>>2] = 1;
 $519 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($519|0);
}
function _notValid($pParse,$pNC,$zMsg,$validMask) {
 $pParse = $pParse|0;
 $pNC = $pNC|0;
 $zMsg = $zMsg|0;
 $validMask = $validMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zIn = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $zMsg;
 HEAP32[$3>>2] = $validMask;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = $7 & $8;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 HEAP32[$zIn>>2] = 29669;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 32;
 $16 = ($15|0)!=(0);
 if ($16) {
  HEAP32[$zIn>>2] = 29697;
 } else {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 28|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19&65535;
  $21 = $20 & 4;
  $22 = ($21|0)!=(0);
  if ($22) {
   HEAP32[$zIn>>2] = 29715;
  }
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$zIn>>2]|0;
 HEAP32[$vararg_buffer>>2] = $24;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $25;
 _sqlite3ErrorMsg($23,29733,$vararg_buffer);
 STACKTOP = sp;return;
}
function _exprProbability($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $r = sp;
 HEAP32[$1>>2] = $p;
 HEAPF64[$r>>3] = -1.0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(133);
 if ($5) {
  HEAP32[$0>>2] = -1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 (_sqlite3AtoF($8,$r,$12,1)|0);
 $13 = +HEAPF64[$r>>3];
 $14 = $13 > 1.0;
 if ($14) {
  HEAP32[$0>>2] = -1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $15 = +HEAPF64[$r>>3];
  $16 = $15 * 134217728.0;
  $17 = (~~(($16)));
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3FunctionUsesThisSrc($pExpr,$pSrcList) {
 $pExpr = $pExpr|0;
 $pSrcList = $pSrcList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $w = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $w = sp + 12|0;
 $cnt = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $pSrcList;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 4|0);
 HEAP32[$2>>2] = 188;
 $3 = ((($w)) + 24|0);
 HEAP32[$3>>2] = $cnt;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$cnt>>2] = $4;
 $5 = ((($cnt)) + 4|0);
 HEAP32[$5>>2] = 0;
 $6 = ((($cnt)) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 (_sqlite3WalkExprList($w,$9)|0);
 $10 = ((($cnt)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 $13 = ((($cnt)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)==(0);
 $16 = $12 ? 1 : $15;
 $17 = $16&1;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3MatchSpanName($zSpan,$zCol,$zTab,$zDb) {
 $zSpan = $zSpan|0;
 $zCol = $zCol|0;
 $zTab = $zTab|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $zSpan;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$4>>2] = $zDb;
 HEAP32[$n>>2] = 0;
 while(1) {
  $5 = HEAP32[$n>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + ($5)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$n>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)!=(46);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$n>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$n>>2] = $18;
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 do {
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$4>>2]|0;
   $23 = HEAP32[$n>>2]|0;
   $24 = (_sqlite3_strnicmp($21,$22,$23)|0);
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$n>>2]|0;
    $27 = HEAP32[$4>>2]|0;
    $28 = (($27) + ($26)|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29 << 24 >> 24;
    $31 = ($30|0)!=(0);
    if (!($31)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $32 = HEAP32[$n>>2]|0;
 $33 = (($32) + 1)|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + ($33)|0);
 HEAP32[$1>>2] = $35;
 HEAP32[$n>>2] = 0;
 while(1) {
  $36 = HEAP32[$n>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = (($37) + ($36)|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39 << 24 >> 24;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$n>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + ($42)|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = ($46|0)!=(46);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$n>>2]|0;
  $49 = (($48) + 1)|0;
  HEAP32[$n>>2] = $49;
 }
 $50 = HEAP32[$3>>2]|0;
 $51 = ($50|0)!=(0|0);
 do {
  if ($51) {
   $52 = HEAP32[$1>>2]|0;
   $53 = HEAP32[$3>>2]|0;
   $54 = HEAP32[$n>>2]|0;
   $55 = (_sqlite3_strnicmp($52,$53,$54)|0);
   $56 = ($55|0)!=(0);
   if (!($56)) {
    $57 = HEAP32[$n>>2]|0;
    $58 = HEAP32[$3>>2]|0;
    $59 = (($58) + ($57)|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60 << 24 >> 24;
    $62 = ($61|0)!=(0);
    if (!($62)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $63 = HEAP32[$n>>2]|0;
 $64 = (($63) + 1)|0;
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + ($64)|0);
 HEAP32[$1>>2] = $66;
 $67 = HEAP32[$2>>2]|0;
 $68 = ($67|0)!=(0|0);
 if ($68) {
  $69 = HEAP32[$1>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = (_sqlite3_stricmp($69,$70)|0);
  $72 = ($71|0)!=(0);
  if ($72) {
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $73 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _nameInUsingClause($pUsing,$zCol) {
 $pUsing = $pUsing|0;
 $zCol = $zCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $k = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $k = sp;
 HEAP32[$1>>2] = $pUsing;
 HEAP32[$2>>2] = $zCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$k>>2] = 0;
   while(1) {
    $5 = HEAP32[$k>>2]|0;
    $6 = HEAP32[$1>>2]|0;
    $7 = ((($6)) + 4|0);
    $8 = HEAP32[$7>>2]|0;
    $9 = ($5|0)<($8|0);
    if (!($9)) {
     break L1;
    }
    $10 = HEAP32[$k>>2]|0;
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = (($12) + ($10<<3)|0);
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$2>>2]|0;
    $16 = (_sqlite3_stricmp($14,$15)|0);
    $17 = ($16|0)==(0);
    if ($17) {
     break;
    }
    $18 = HEAP32[$k>>2]|0;
    $19 = (($18) + 1)|0;
    HEAP32[$k>>2] = $19;
   }
   HEAP32[$0>>2] = 1;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _resolveAlias($pParse,$pEList,$iCol,$pExpr,$zType,$nSubquery) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $iCol = $iCol|0;
 $pExpr = $pExpr|0;
 $zType = $zType|0;
 $nSubquery = $nSubquery|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pDup = 0, $pOrig = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $pOrig = sp + 8|0;
 $pDup = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$3>>2] = $pExpr;
 HEAP32[$4>>2] = $zType;
 HEAP32[$5>>2] = $nSubquery;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($6*20)|0)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pOrig>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$db>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pOrig>>2]|0;
 $16 = (_sqlite3ExprDup($14,$15,0)|0);
 HEAP32[$pDup>>2] = $16;
 $17 = HEAP32[$pDup>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP8[$19>>0]|0;
 $21 = $20 << 24 >> 24;
 $22 = ($21|0)!=(71);
 if ($22) {
  $23 = HEAP32[$pDup>>2]|0;
  $24 = HEAP32[$5>>2]|0;
  _incrAggFunctionDepth($23,$24);
 }
 $25 = HEAP32[$3>>2]|0;
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)==(95);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pDup>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_sqlite3ExprAddCollateString($29,$30,$33)|0);
  HEAP32[$pDup>>2] = $34;
 }
 $35 = HEAP32[$pDup>>2]|0;
 $36 = ((($35)) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 | 4194304;
 HEAP32[$36>>2] = $38;
 $39 = HEAP32[$3>>2]|0;
 $40 = ((($39)) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 | 32768;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$db>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($43,$44);
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$pDup>>2]|0;
 dest=$45; src=$46; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $47 = HEAP32[$3>>2]|0;
 $48 = ((($47)) + 4|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $49 & 1024;
 $51 = ($50|0)!=(0);
 if (!($51)) {
  $52 = HEAP32[$3>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0|0);
  if ($55) {
   $56 = HEAP32[$db>>2]|0;
   $57 = HEAP32[$3>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3DbStrDup($56,$59)|0);
   $61 = HEAP32[$3>>2]|0;
   $62 = ((($61)) + 8|0);
   HEAP32[$62>>2] = $60;
   $63 = HEAP32[$3>>2]|0;
   $64 = ((($63)) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 | 65536;
   HEAP32[$64>>2] = $66;
  }
 }
 $67 = HEAP32[$db>>2]|0;
 $68 = HEAP32[$pDup>>2]|0;
 _sqlite3DbFree($67,$68);
 STACKTOP = sp;return;
}
function _sqlite3AuthRead($pParse,$pExpr,$pSchema,$pTabList) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pSchema = $pSchema|0;
 $pTabList = $pTabList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $db = 0, $iCol = 0, $iDb = 0, $iSrc = 0, $pTab = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $db = sp + 20|0;
 $pTab = sp + 16|0;
 $zCol = sp + 12|0;
 $iSrc = sp + 8|0;
 $iDb = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pSchema;
 HEAP32[$3>>2] = $pTabList;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 HEAP32[$pTab>>2] = 0;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 288|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3SchemaToIndex($11,$12)|0);
 HEAP32[$iDb>>2] = $13;
 $14 = HEAP32[$iDb>>2]|0;
 $15 = ($14|0)<(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(62);
 L7: do {
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 420|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pTab>>2] = $22;
  } else {
   HEAP32[$iSrc>>2] = 0;
   while(1) {
    $23 = HEAP32[$iSrc>>2]|0;
    $24 = HEAP32[$3>>2]|0;
    $25 = HEAP32[$24>>2]|0;
    $26 = ($23|0)<($25|0);
    if (!($26)) {
     break L7;
    }
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 28|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$iSrc>>2]|0;
    $31 = HEAP32[$3>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = (($32) + (($30*72)|0)|0);
    $34 = ((($33)) + 44|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($29|0)==($35|0);
    $37 = HEAP32[$iSrc>>2]|0;
    if ($36) {
     break;
    }
    $43 = (($37) + 1)|0;
    HEAP32[$iSrc>>2] = $43;
   }
   $38 = HEAP32[$3>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = (($39) + (($37*72)|0)|0);
   $41 = ((($40)) + 16|0);
   $42 = HEAP32[$41>>2]|0;
   HEAP32[$pTab>>2] = $42;
  }
 } while(0);
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = HEAP16[$45>>1]|0;
 $47 = $46 << 16 >> 16;
 HEAP32[$iCol>>2] = $47;
 $48 = HEAP32[$pTab>>2]|0;
 $49 = ($48|0)==(0|0);
 if ($49) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$iCol>>2]|0;
 $51 = ($50|0)>=(0);
 do {
  if ($51) {
   $52 = HEAP32[$iCol>>2]|0;
   $53 = HEAP32[$pTab>>2]|0;
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + (($52*24)|0)|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$zCol>>2] = $57;
  } else {
   $58 = HEAP32[$pTab>>2]|0;
   $59 = ((($58)) + 32|0);
   $60 = HEAP16[$59>>1]|0;
   $61 = $60 << 16 >> 16;
   $62 = ($61|0)>=(0);
   if ($62) {
    $63 = HEAP32[$pTab>>2]|0;
    $64 = ((($63)) + 32|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = $65 << 16 >> 16;
    $67 = HEAP32[$pTab>>2]|0;
    $68 = ((($67)) + 4|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + (($66*24)|0)|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$zCol>>2] = $71;
    break;
   } else {
    HEAP32[$zCol>>2] = 29600;
    break;
   }
  }
 } while(0);
 $72 = HEAP32[$0>>2]|0;
 $73 = HEAP32[$pTab>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$zCol>>2]|0;
 $76 = HEAP32[$iDb>>2]|0;
 $77 = (_sqlite3AuthReadCol($72,$74,$75,$76)|0);
 $78 = (2)==($77|0);
 if (!($78)) {
  STACKTOP = sp;return;
 }
 $79 = HEAP32[$1>>2]|0;
 HEAP8[$79>>0] = 101;
 STACKTOP = sp;return;
}
function _incrAggFunctionDepth($pExpr,$N) {
 $pExpr = $pExpr|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $4 = ((($w)) + 4|0);
 HEAP32[$4>>2] = 189;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($w)) + 24|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$7)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprAddCollateString($pParse,$pExpr,$zC) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $zC = $zC|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $s = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $s = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $zC;
 $3 = HEAP32[$2>>2]|0;
 _sqlite3TokenInit($s,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3ExprAddCollateToken($4,$5,$s,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _incrAggDepth($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(153);
 if (!($5)) {
  STACKTOP = sp;return 0;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 38|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = (($12) + ($8))|0;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 STACKTOP = sp;return 0;
}
function _sqlite3AuthReadCol($pParse,$zTab,$zCol,$iDb) {
 $pParse = $pParse|0;
 $zTab = $zTab|0;
 $zCol = $zCol|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer6 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $db = sp + 32|0;
 $zDb = sp + 28|0;
 $rc = sp + 24|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zTab;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$3>>2] = $iDb;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$zDb>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 288|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$db>>2]|0;
 $16 = ((($15)) + 292|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$zDb>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 496|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (FUNCTION_TABLE_iiiiiii[$14 & 255]($17,20,$18,$19,$20,$23)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)==(1);
 if (!($26)) {
  $41 = HEAP32[$rc>>2]|0;
  $42 = ($41|0)!=(2);
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)!=(0);
  $or$cond3 = $42 & $44;
  if (!($or$cond3)) {
   $46 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
  $45 = HEAP32[$0>>2]|0;
  _sqliteAuthBadReturnCode($45);
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $27 = HEAP32[$db>>2]|0;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>(2);
 $31 = HEAP32[$3>>2]|0;
 $32 = ($31|0)!=(0);
 $or$cond = $30 | $32;
 $33 = HEAP32[$0>>2]|0;
 if ($or$cond) {
  $34 = HEAP32[$zDb>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $34;
  $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $35;
  $vararg_ptr5 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $36;
  _sqlite3ErrorMsg($33,29606,$vararg_buffer);
 } else {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer6>>2] = $37;
  $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
  HEAP32[$vararg_ptr9>>2] = $38;
  _sqlite3ErrorMsg($33,29639,$vararg_buffer6);
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 12|0);
 HEAP32[$40>>2] = 23;
 $46 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _exprSrcCount($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nSrc = 0, $p = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $i = sp + 12|0;
 $p = sp + 8|0;
 $pSrc = sp + 4|0;
 $nSrc = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(152);
 if (!($5)) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(154);
  if (!($9)) {
   STACKTOP = sp;return 0;
  }
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pSrc>>2] = $14;
 $15 = HEAP32[$pSrc>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$pSrc>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = $18;
 } else {
  $19 = 0;
 }
 HEAP32[$nSrc>>2] = $19;
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$nSrc>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 28|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$pSrc>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = (($28) + (($26*72)|0)|0);
  $30 = ((($29)) + 44|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($25|0)==($31|0);
  if ($32) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = HEAP32[$i>>2]|0;
 $36 = HEAP32[$nSrc>>2]|0;
 $37 = ($35|0)<($36|0);
 $38 = HEAP32[$p>>2]|0;
 if ($37) {
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$39>>2] = $41;
  STACKTOP = sp;return 0;
 } else {
  $42 = ((($38)) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$42>>2] = $44;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _sqlite3SelectPrep($pParse,$p,$pOuterNC) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pOuterNC = $pOuterNC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOuterNC;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 65|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 6|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 64;
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 _sqlite3SelectExpand($17,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 68|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$db>>2]|0;
 $24 = ((($23)) + 65|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)!=(0);
 if ($27) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 _sqlite3ResolveSelectNames($28,$29,$30);
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 68|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0);
 if ($34) {
  STACKTOP = sp;return;
 }
 $35 = HEAP32[$db>>2]|0;
 $36 = ((($35)) + 65|0);
 $37 = HEAP8[$36>>0]|0;
 $38 = $37&255;
 $39 = ($38|0)!=(0);
 if ($39) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 _sqlite3SelectAddTypeInfo($40,$41);
 STACKTOP = sp;return;
}
function _sqlite3ResolveSelectNames($pParse,$p,$pOuterNC) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pOuterNC = $pOuterNC|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOuterNC;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = ((($w)) + 4|0);
 HEAP32[$3>>2] = 186;
 $4 = ((($w)) + 8|0);
 HEAP32[$4>>2] = 187;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$w>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($w)) + 24|0);
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$8)|0);
 STACKTOP = sp;return;
}
function _resolveOrderGroupBy($pNC,$pSelect,$pOrderBy,$zType) {
 $pNC = $pNC|0;
 $pSelect = $pSelect|0;
 $pOrderBy = $pOrderBy|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $i = 0, $iCol = 0, $j = 0, $nResult = 0, $or$cond = 0, $pE = 0, $pE2 = 0, $pItem = 0;
 var $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $iCol = sp + 20|0;
 $pItem = sp + 16|0;
 $pParse = sp + 12|0;
 $nResult = sp + 8|0;
 $pE = sp + 4|0;
 $pE2 = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pOrderBy;
 HEAP32[$4>>2] = $zType;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nResult>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pParse>>2] = $11;
 HEAP32[$i>>2] = 0;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pItem>>2] = $14;
 L5: while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   label = 20;
   break;
  }
  $19 = HEAP32[$pItem>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pE>>2] = $20;
  $21 = HEAP32[$pE>>2]|0;
  $22 = (_sqlite3ExprSkipCollate($21)|0);
  HEAP32[$pE2>>2] = $22;
  $23 = HEAP32[$4>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = ($25|0)!=(71);
  if ($26) {
   $27 = HEAP32[$pParse>>2]|0;
   $28 = HEAP32[$2>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$pE2>>2]|0;
   $31 = (_resolveAsName($27,$29,$30)|0);
   HEAP32[$iCol>>2] = $31;
   $32 = HEAP32[$iCol>>2]|0;
   $33 = ($32|0)>(0);
   if ($33) {
    $34 = HEAP32[$iCol>>2]|0;
    $35 = $34&65535;
    $36 = HEAP32[$pItem>>2]|0;
    $37 = ((($36)) + 16|0);
    HEAP16[$37>>1] = $35;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  L11: do {
   if ((label|0) == 8) {
    label = 0;
    $38 = HEAP32[$pE2>>2]|0;
    $39 = (_sqlite3ExprIsInteger($38,$iCol)|0);
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$iCol>>2]|0;
     $42 = ($41|0)<(1);
     $43 = HEAP32[$iCol>>2]|0;
     $44 = ($43|0)>(65535);
     $or$cond = $42 | $44;
     if ($or$cond) {
      label = 10;
      break L5;
     }
     $50 = HEAP32[$iCol>>2]|0;
     $51 = $50&65535;
     $52 = HEAP32[$pItem>>2]|0;
     $53 = ((($52)) + 16|0);
     HEAP16[$53>>1] = $51;
     break;
    }
    $54 = HEAP32[$pItem>>2]|0;
    $55 = ((($54)) + 16|0);
    HEAP16[$55>>1] = 0;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$pE>>2]|0;
    $58 = (_sqlite3ResolveExprNames($56,$57)|0);
    $59 = ($58|0)!=(0);
    if ($59) {
     label = 13;
     break L5;
    }
    HEAP32[$j>>2] = 0;
    while(1) {
     $60 = HEAP32[$j>>2]|0;
     $61 = HEAP32[$2>>2]|0;
     $62 = HEAP32[$61>>2]|0;
     $63 = HEAP32[$62>>2]|0;
     $64 = ($60|0)<($63|0);
     if (!($64)) {
      break L11;
     }
     $65 = HEAP32[$pE>>2]|0;
     $66 = HEAP32[$j>>2]|0;
     $67 = HEAP32[$2>>2]|0;
     $68 = HEAP32[$67>>2]|0;
     $69 = ((($68)) + 4|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = (($70) + (($66*20)|0)|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = (_sqlite3ExprCompare($65,$72,-1)|0);
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$j>>2]|0;
      $76 = (($75) + 1)|0;
      $77 = $76&65535;
      $78 = HEAP32[$pItem>>2]|0;
      $79 = ((($78)) + 16|0);
      HEAP16[$79>>1] = $77;
     }
     $80 = HEAP32[$j>>2]|0;
     $81 = (($80) + 1)|0;
     HEAP32[$j>>2] = $81;
    }
   }
  } while(0);
  $82 = HEAP32[$i>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$i>>2] = $83;
  $84 = HEAP32[$pItem>>2]|0;
  $85 = ((($84)) + 20|0);
  HEAP32[$pItem>>2] = $85;
 }
 if ((label|0) == 10) {
  $45 = HEAP32[$pParse>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  $49 = HEAP32[$nResult>>2]|0;
  _resolveOutOfRangeError($45,$46,$48,$49);
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 20) {
  $86 = HEAP32[$pParse>>2]|0;
  $87 = HEAP32[$2>>2]|0;
  $88 = HEAP32[$3>>2]|0;
  $89 = HEAP32[$4>>2]|0;
  $90 = (_sqlite3ResolveOrderGroupBy($86,$87,$88,$89)|0);
  HEAP32[$0>>2] = $90;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 return (0)|0;
}
function _sqlite3SelectWrongNumTermsError($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 6|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 256;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  _sqlite3ErrorMsg($8,30517,$vararg_buffer);
  STACKTOP = sp;return;
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = (_selectOpName($12)|0);
  HEAP32[$vararg_buffer1>>2] = $13;
  _sqlite3ErrorMsg($8,30563,$vararg_buffer1);
  STACKTOP = sp;return;
 }
}
function _resolveCompoundOrderBy($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $moreToDo = 0, $pDup = 0, $pE = 0, $pEList = 0;
 var $pItem = 0, $pNew = 0, $pOrderBy = 0, $pParent = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $i = sp + 52|0;
 $pOrderBy = sp + 48|0;
 $pEList = sp + 44|0;
 $db = sp + 40|0;
 $moreToDo = sp + 36|0;
 $pItem = sp + 32|0;
 $iCol = sp + 28|0;
 $pE = sp + 24|0;
 $pDup = sp + 20|0;
 $pNew = sp + 16|0;
 $pParent = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$moreToDo>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pOrderBy>>2] = $5;
 $6 = HEAP32[$pOrderBy>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$pOrderBy>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 88|0);
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($11|0)>($15|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($17,30678,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$pOrderBy>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = ($18|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pOrderBy>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + (($22*20)|0)|0);
  $27 = ((($26)) + 13|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 & -2;
  HEAP8[$27>>0] = $29;
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 56|0);
 HEAP32[$33>>2] = 0;
 while(1) {
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 52|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = ((($39)) + 52|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ((($41)) + 56|0);
  HEAP32[$42>>2] = $38;
  $43 = HEAP32[$2>>2]|0;
  $44 = ((($43)) + 52|0);
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$2>>2] = $45;
 }
 L16: while(1) {
  $46 = HEAP32[$2>>2]|0;
  $47 = ($46|0)!=(0|0);
  $48 = HEAP32[$moreToDo>>2]|0;
  $49 = ($48|0)!=(0);
  $50 = $47 ? $49 : 0;
  if (!($50)) {
   label = 36;
   break;
  }
  HEAP32[$moreToDo>>2] = 0;
  $51 = HEAP32[$2>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  HEAP32[$pEList>>2] = $52;
  HEAP32[$i>>2] = 0;
  $53 = HEAP32[$pOrderBy>>2]|0;
  $54 = ((($53)) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  HEAP32[$pItem>>2] = $55;
  while(1) {
   $56 = HEAP32[$i>>2]|0;
   $57 = HEAP32[$pOrderBy>>2]|0;
   $58 = HEAP32[$57>>2]|0;
   $59 = ($56|0)<($58|0);
   if (!($59)) {
    break;
   }
   HEAP32[$iCol>>2] = -1;
   $60 = HEAP32[$pItem>>2]|0;
   $61 = ((($60)) + 13|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62 & 1;
   $64 = $63&255;
   $65 = ($64|0)!=(0);
   do {
    if (!($65)) {
     $66 = HEAP32[$pItem>>2]|0;
     $67 = HEAP32[$66>>2]|0;
     $68 = (_sqlite3ExprSkipCollate($67)|0);
     HEAP32[$pE>>2] = $68;
     $69 = HEAP32[$pE>>2]|0;
     $70 = (_sqlite3ExprIsInteger($69,$iCol)|0);
     $71 = ($70|0)!=(0);
     if ($71) {
      $72 = HEAP32[$iCol>>2]|0;
      $73 = ($72|0)<=(0);
      if ($73) {
       label = 18;
       break L16;
      }
      $74 = HEAP32[$iCol>>2]|0;
      $75 = HEAP32[$pEList>>2]|0;
      $76 = HEAP32[$75>>2]|0;
      $77 = ($74|0)>($76|0);
      if ($77) {
       label = 18;
       break L16;
      }
     } else {
      $83 = HEAP32[$1>>2]|0;
      $84 = HEAP32[$pEList>>2]|0;
      $85 = HEAP32[$pE>>2]|0;
      $86 = (_resolveAsName($83,$84,$85)|0);
      HEAP32[$iCol>>2] = $86;
      $87 = HEAP32[$iCol>>2]|0;
      $88 = ($87|0)==(0);
      if ($88) {
       $89 = HEAP32[$db>>2]|0;
       $90 = HEAP32[$pE>>2]|0;
       $91 = (_sqlite3ExprDup($89,$90,0)|0);
       HEAP32[$pDup>>2] = $91;
       $92 = HEAP32[$db>>2]|0;
       $93 = ((($92)) + 65|0);
       $94 = HEAP8[$93>>0]|0;
       $95 = ($94<<24>>24)!=(0);
       if (!($95)) {
        $96 = HEAP32[$1>>2]|0;
        $97 = HEAP32[$2>>2]|0;
        $98 = HEAP32[$pDup>>2]|0;
        $99 = (_resolveOrderByTermToExprList($96,$97,$98)|0);
        HEAP32[$iCol>>2] = $99;
       }
       $100 = HEAP32[$db>>2]|0;
       $101 = HEAP32[$pDup>>2]|0;
       _sqlite3ExprDelete($100,$101);
      }
     }
     $102 = HEAP32[$iCol>>2]|0;
     $103 = ($102|0)>(0);
     if (!($103)) {
      HEAP32[$moreToDo>>2] = 1;
      break;
     }
     $104 = HEAP32[$db>>2]|0;
     $105 = (_sqlite3Expr($104,132,0)|0);
     HEAP32[$pNew>>2] = $105;
     $106 = HEAP32[$pNew>>2]|0;
     $107 = ($106|0)==(0|0);
     if ($107) {
      label = 25;
      break L16;
     }
     $108 = HEAP32[$pNew>>2]|0;
     $109 = ((($108)) + 4|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = $110 | 1024;
     HEAP32[$109>>2] = $111;
     $112 = HEAP32[$iCol>>2]|0;
     $113 = HEAP32[$pNew>>2]|0;
     $114 = ((($113)) + 8|0);
     HEAP32[$114>>2] = $112;
     $115 = HEAP32[$pItem>>2]|0;
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$pE>>2]|0;
     $118 = ($116|0)==($117|0);
     if ($118) {
      $119 = HEAP32[$pNew>>2]|0;
      $120 = HEAP32[$pItem>>2]|0;
      HEAP32[$120>>2] = $119;
     } else {
      $121 = HEAP32[$pItem>>2]|0;
      $122 = HEAP32[$121>>2]|0;
      HEAP32[$pParent>>2] = $122;
      while(1) {
       $123 = HEAP32[$pParent>>2]|0;
       $124 = ((($123)) + 12|0);
       $125 = HEAP32[$124>>2]|0;
       $126 = HEAP8[$125>>0]|0;
       $127 = $126&255;
       $128 = ($127|0)==(95);
       if (!($128)) {
        break;
       }
       $129 = HEAP32[$pParent>>2]|0;
       $130 = ((($129)) + 12|0);
       $131 = HEAP32[$130>>2]|0;
       HEAP32[$pParent>>2] = $131;
      }
      $132 = HEAP32[$pNew>>2]|0;
      $133 = HEAP32[$pParent>>2]|0;
      $134 = ((($133)) + 12|0);
      HEAP32[$134>>2] = $132;
     }
     $135 = HEAP32[$db>>2]|0;
     $136 = HEAP32[$pE>>2]|0;
     _sqlite3ExprDelete($135,$136);
     $137 = HEAP32[$iCol>>2]|0;
     $138 = $137&65535;
     $139 = HEAP32[$pItem>>2]|0;
     $140 = ((($139)) + 16|0);
     HEAP16[$140>>1] = $138;
     $141 = HEAP32[$pItem>>2]|0;
     $142 = ((($141)) + 13|0);
     $143 = HEAP8[$142>>0]|0;
     $144 = $143 & -2;
     $145 = $144 | 1;
     HEAP8[$142>>0] = $145;
    }
   } while(0);
   $146 = HEAP32[$i>>2]|0;
   $147 = (($146) + 1)|0;
   HEAP32[$i>>2] = $147;
   $148 = HEAP32[$pItem>>2]|0;
   $149 = ((($148)) + 20|0);
   HEAP32[$pItem>>2] = $149;
  }
  $150 = HEAP32[$2>>2]|0;
  $151 = ((($150)) + 56|0);
  $152 = HEAP32[$151>>2]|0;
  HEAP32[$2>>2] = $152;
 }
 if ((label|0) == 18) {
  $78 = HEAP32[$1>>2]|0;
  $79 = HEAP32[$i>>2]|0;
  $80 = (($79) + 1)|0;
  $81 = HEAP32[$pEList>>2]|0;
  $82 = HEAP32[$81>>2]|0;
  _resolveOutOfRangeError($78,29797,$80,$82);
  HEAP32[$0>>2] = 1;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 else if ((label|0) == 25) {
  HEAP32[$0>>2] = 1;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 else if ((label|0) == 36) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $153 = HEAP32[$i>>2]|0;
   $154 = HEAP32[$pOrderBy>>2]|0;
   $155 = HEAP32[$154>>2]|0;
   $156 = ($153|0)<($155|0);
   if (!($156)) {
    label = 41;
    break;
   }
   $157 = HEAP32[$i>>2]|0;
   $158 = HEAP32[$pOrderBy>>2]|0;
   $159 = ((($158)) + 4|0);
   $160 = HEAP32[$159>>2]|0;
   $161 = (($160) + (($157*20)|0)|0);
   $162 = ((($161)) + 13|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163 & 1;
   $165 = $164&255;
   $166 = ($165|0)==(0);
   if ($166) {
    label = 39;
    break;
   }
   $170 = HEAP32[$i>>2]|0;
   $171 = (($170) + 1)|0;
   HEAP32[$i>>2] = $171;
  }
  if ((label|0) == 39) {
   $167 = HEAP32[$1>>2]|0;
   $168 = HEAP32[$i>>2]|0;
   $169 = (($168) + 1)|0;
   HEAP32[$vararg_buffer1>>2] = $169;
   _sqlite3ErrorMsg($167,30712,$vararg_buffer1);
   HEAP32[$0>>2] = 1;
   $172 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($172|0);
  }
  else if ((label|0) == 41) {
   HEAP32[$0>>2] = 0;
   $172 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($172|0);
  }
 }
 return (0)|0;
}
function _sqlite3SelectExpand($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSelect;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 4|0);
 HEAP32[$2>>2] = 190;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$w>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 22|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = ((($w)) + 8|0);
  HEAP32[$8>>2] = 191;
  $9 = HEAP32[$1>>2]|0;
  (_sqlite3WalkSelect($w,$9)|0);
 }
 $10 = ((($w)) + 8|0);
 HEAP32[$10>>2] = 192;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 6|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 512;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $18 = HEAP32[$1>>2]|0;
  (_sqlite3WalkSelect($w,$18)|0);
  STACKTOP = sp;return;
 }
 $17 = ((($w)) + 12|0);
 HEAP32[$17>>2] = 132;
 $18 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$18)|0);
 STACKTOP = sp;return;
}
function _sqlite3SelectAddTypeInfo($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSelect;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 12|0);
 HEAP32[$2>>2] = 133;
 $3 = ((($w)) + 4|0);
 HEAP32[$3>>2] = 190;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$w>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$5)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprWalkNoop($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _convertCompoundSelectToSubquery($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $12 = 0, $13 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $dummy = 0, $i = 0, $pNew = 0, $pNewSrc = 0;
 var $pParse = 0, $pX = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $i = sp + 32|0;
 $pNew = sp + 28|0;
 $pX = sp + 24|0;
 $db = sp + 20|0;
 $a = sp + 16|0;
 $pNewSrc = sp + 12|0;
 $pParse = sp + 8|0;
 $dummy = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 }
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$pX>>2] = $11;
 while(1) {
  $12 = HEAP32[$pX>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pX>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(116);
  if (!($18)) {
   $19 = HEAP32[$pX>>2]|0;
   $20 = ((($19)) + 4|0);
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = ($22|0)==(119);
   if (!($23)) {
    break;
   }
  }
  $24 = HEAP32[$pX>>2]|0;
  $25 = ((($24)) + 52|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pX>>2] = $26;
 }
 $27 = HEAP32[$pX>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = ((($29)) + 48|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ((($31)) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$a>>2] = $33;
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 48|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) - 1)|0;
 HEAP32[$i>>2] = $38;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = ($39|0)>=(0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$a>>2]|0;
  $43 = (($42) + (($41*20)|0)|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $46 & 256;
  $48 = ($47|0)!=(0);
  if ($48) {
   break;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + -1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$i>>2]|0;
 $52 = ($51|0)<(0);
 if ($52) {
  HEAP32[$0>>2] = 0;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 HEAP32[$pParse>>2] = $54;
 $55 = HEAP32[$pParse>>2]|0;
 $56 = HEAP32[$55>>2]|0;
 HEAP32[$db>>2] = $56;
 $57 = HEAP32[$db>>2]|0;
 $58 = (_sqlite3DbMallocZero($57,72,0)|0);
 HEAP32[$pNew>>2] = $58;
 $59 = HEAP32[$pNew>>2]|0;
 $60 = ($59|0)==(0|0);
 if ($60) {
  HEAP32[$0>>2] = 2;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 }
 ;HEAP32[$dummy>>2]=0|0;HEAP32[$dummy+4>>2]=0|0;
 $61 = HEAP32[$pParse>>2]|0;
 $62 = HEAP32[$pNew>>2]|0;
 $63 = (_sqlite3SrcListAppendFromTerm($61,0,0,0,$dummy,$62,0,0)|0);
 HEAP32[$pNewSrc>>2] = $63;
 $64 = HEAP32[$pNewSrc>>2]|0;
 $65 = ($64|0)==(0|0);
 if ($65) {
  HEAP32[$0>>2] = 2;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 } else {
  $66 = HEAP32[$pNew>>2]|0;
  $67 = HEAP32[$2>>2]|0;
  dest=$66; src=$67; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $68 = HEAP32[$pNewSrc>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = ((($69)) + 32|0);
  HEAP32[$70>>2] = $68;
  $71 = HEAP32[$pParse>>2]|0;
  $72 = HEAP32[$db>>2]|0;
  $73 = (_sqlite3Expr($72,158,0)|0);
  $74 = (_sqlite3ExprListAppend($71,0,$73)|0);
  $75 = HEAP32[$2>>2]|0;
  HEAP32[$75>>2] = $74;
  $76 = HEAP32[$2>>2]|0;
  $77 = ((($76)) + 4|0);
  HEAP8[$77>>0] = 119;
  $78 = HEAP32[$2>>2]|0;
  $79 = ((($78)) + 36|0);
  HEAP32[$79>>2] = 0;
  $80 = HEAP32[$pNew>>2]|0;
  $81 = ((($80)) + 40|0);
  HEAP32[$81>>2] = 0;
  $82 = HEAP32[$pNew>>2]|0;
  $83 = ((($82)) + 44|0);
  HEAP32[$83>>2] = 0;
  $84 = HEAP32[$pNew>>2]|0;
  $85 = ((($84)) + 48|0);
  HEAP32[$85>>2] = 0;
  $86 = HEAP32[$2>>2]|0;
  $87 = ((($86)) + 52|0);
  HEAP32[$87>>2] = 0;
  $88 = HEAP32[$2>>2]|0;
  $89 = ((($88)) + 56|0);
  HEAP32[$89>>2] = 0;
  $90 = HEAP32[$2>>2]|0;
  $91 = ((($90)) + 68|0);
  HEAP32[$91>>2] = 0;
  $92 = HEAP32[$2>>2]|0;
  $93 = ((($92)) + 6|0);
  $94 = HEAP16[$93>>1]|0;
  $95 = $94&65535;
  $96 = $95 & -129;
  $97 = $96&65535;
  HEAP16[$93>>1] = $97;
  $98 = HEAP32[$2>>2]|0;
  $99 = ((($98)) + 6|0);
  $100 = HEAP16[$99>>1]|0;
  $101 = $100&65535;
  $102 = $101 | 16384;
  $103 = $102&65535;
  HEAP16[$99>>1] = $103;
  $104 = HEAP32[$pNew>>2]|0;
  $105 = HEAP32[$pNew>>2]|0;
  $106 = ((($105)) + 52|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = ((($107)) + 56|0);
  HEAP32[$108>>2] = $104;
  $109 = HEAP32[$pNew>>2]|0;
  $110 = ((($109)) + 60|0);
  HEAP32[$110>>2] = 0;
  $111 = HEAP32[$pNew>>2]|0;
  $112 = ((($111)) + 64|0);
  HEAP32[$112>>2] = 0;
  HEAP32[$0>>2] = 0;
  $113 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($113|0);
 }
 return (0)|0;
}
function _selectExpander($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $flags = 0, $i = 0, $iDb = 0, $j = 0, $k = 0, $longNames = 0, $nCol = 0, $or$cond = 0, $or$cond3 = 0, $pE = 0, $pEList = 0, $pExpr = 0, $pFrom = 0;
 var $pLeft = 0, $pNew = 0, $pParse = 0, $pRight = 0, $pSel = 0, $pSub = 0, $pTab = 0, $pTab1 = 0, $pTabList = 0, $pX = 0, $sColname = 0, $selFlags = 0, $tableSeen = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer21 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0;
 var $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $zColname = 0, $zName = 0, $zSchemaName = 0, $zTName = 0, $zTabName = 0, $zToFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer21 = sp + 56|0;
 $vararg_buffer19 = sp + 48|0;
 $vararg_buffer16 = sp + 40|0;
 $vararg_buffer11 = sp + 24|0;
 $vararg_buffer7 = sp + 16|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 196|0;
 $1 = sp + 192|0;
 $2 = sp + 188|0;
 $pParse = sp + 184|0;
 $i = sp + 180|0;
 $j = sp + 176|0;
 $k = sp + 172|0;
 $pTabList = sp + 168|0;
 $pEList = sp + 164|0;
 $pFrom = sp + 160|0;
 $db = sp + 156|0;
 $pE = sp + 152|0;
 $pRight = sp + 148|0;
 $pExpr = sp + 144|0;
 $selFlags = sp + 202|0;
 $pTab = sp + 140|0;
 $pSel = sp + 136|0;
 $nCol = sp + 200|0;
 $a = sp + 132|0;
 $pNew = sp + 128|0;
 $flags = sp + 124|0;
 $longNames = sp + 120|0;
 $tableSeen = sp + 116|0;
 $zTName = sp + 112|0;
 $pTab1 = sp + 108|0;
 $pSub = sp + 104|0;
 $zTabName = sp + 100|0;
 $zSchemaName = sp + 96|0;
 $iDb = sp + 92|0;
 $zName = sp + 88|0;
 $zColname = sp + 84|0;
 $zToFree = sp + 80|0;
 $sColname = sp + 72|0;
 $pLeft = sp + 64|0;
 $pX = sp + 60|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 6|0);
 $9 = HEAP16[$8>>1]|0;
 HEAP16[$selFlags>>1] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 6|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 | 32;
 $15 = $14&65535;
 HEAP16[$11>>1] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 65|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 if ($19) {
  HEAP32[$0>>2] = 2;
  $544 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($544|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 32|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0|0);
 if (!($23)) {
  $24 = HEAP16[$selFlags>>1]|0;
  $25 = $24&65535;
  $26 = $25 & 32;
  $27 = ($26|0)!=(0);
  if (!($27)) {
   $28 = HEAP32[$2>>2]|0;
   $29 = ((($28)) + 32|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$pTabList>>2] = $30;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$31>>2]|0;
   HEAP32[$pEList>>2] = $32;
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(132|0);
   if ($36) {
    $37 = HEAP32[$pParse>>2]|0;
    $38 = HEAP32[$2>>2]|0;
    $39 = (_findRightmost($38)|0);
    $40 = ((($39)) + 68|0);
    $41 = HEAP32[$40>>2]|0;
    _sqlite3WithPush($37,$41,0);
   }
   $42 = HEAP32[$pParse>>2]|0;
   $43 = HEAP32[$pTabList>>2]|0;
   _sqlite3SrcListAssignCursors($42,$43);
   HEAP32[$i>>2] = 0;
   $44 = HEAP32[$pTabList>>2]|0;
   $45 = ((($44)) + 8|0);
   HEAP32[$pFrom>>2] = $45;
   L11: while(1) {
    $46 = HEAP32[$i>>2]|0;
    $47 = HEAP32[$pTabList>>2]|0;
    $48 = HEAP32[$47>>2]|0;
    $49 = ($46|0)<($48|0);
    if (!($49)) {
     label = 38;
     break;
    }
    $50 = HEAP32[$pFrom>>2]|0;
    $51 = ((($50)) + 36|0);
    $52 = ((($51)) + 1|0);
    $53 = HEAP8[$52>>0]|0;
    $54 = ($53&255) >>> 5;
    $55 = $54 & 1;
    $56 = $55&255;
    $57 = ($56|0)!=(0);
    if (!($57)) {
     $58 = HEAP32[$1>>2]|0;
     $59 = HEAP32[$pFrom>>2]|0;
     $60 = (_withExpand($58,$59)|0);
     $61 = ($60|0)!=(0);
     if ($61) {
      label = 12;
      break;
     }
     $62 = HEAP32[$pFrom>>2]|0;
     $63 = ((($62)) + 16|0);
     $64 = HEAP32[$63>>2]|0;
     $65 = ($64|0)!=(0|0);
     do {
      if (!($65)) {
       $66 = HEAP32[$pFrom>>2]|0;
       $67 = ((($66)) + 8|0);
       $68 = HEAP32[$67>>2]|0;
       $69 = ($68|0)==(0|0);
       if ($69) {
        $70 = HEAP32[$pFrom>>2]|0;
        $71 = ((($70)) + 20|0);
        $72 = HEAP32[$71>>2]|0;
        HEAP32[$pSel>>2] = $72;
        $73 = HEAP32[$1>>2]|0;
        $74 = HEAP32[$pSel>>2]|0;
        $75 = (_sqlite3WalkSelect($73,$74)|0);
        $76 = ($75|0)!=(0);
        if ($76) {
         label = 16;
         break L11;
        }
        $77 = HEAP32[$db>>2]|0;
        $78 = (_sqlite3DbMallocZero($77,72,0)|0);
        HEAP32[$pTab>>2] = $78;
        $79 = HEAP32[$pFrom>>2]|0;
        $80 = ((($79)) + 16|0);
        HEAP32[$80>>2] = $78;
        $81 = HEAP32[$pTab>>2]|0;
        $82 = ($81|0)==(0|0);
        if ($82) {
         label = 18;
         break L11;
        }
        $83 = HEAP32[$pTab>>2]|0;
        $84 = ((($83)) + 36|0);
        HEAP16[$84>>1] = 1;
        $85 = HEAP32[$db>>2]|0;
        $86 = HEAP32[$pTab>>2]|0;
        HEAP32[$vararg_buffer>>2] = $86;
        $87 = (_sqlite3MPrintf($85,29868,$vararg_buffer)|0);
        $88 = HEAP32[$pTab>>2]|0;
        HEAP32[$88>>2] = $87;
        while(1) {
         $89 = HEAP32[$pSel>>2]|0;
         $90 = ((($89)) + 52|0);
         $91 = HEAP32[$90>>2]|0;
         $92 = ($91|0)!=(0|0);
         if (!($92)) {
          break;
         }
         $93 = HEAP32[$pSel>>2]|0;
         $94 = ((($93)) + 52|0);
         $95 = HEAP32[$94>>2]|0;
         HEAP32[$pSel>>2] = $95;
        }
        $96 = HEAP32[$pParse>>2]|0;
        $97 = HEAP32[$pSel>>2]|0;
        $98 = HEAP32[$97>>2]|0;
        $99 = HEAP32[$pTab>>2]|0;
        $100 = ((($99)) + 34|0);
        $101 = HEAP32[$pTab>>2]|0;
        $102 = ((($101)) + 4|0);
        (_sqlite3ColumnsFromExprList($96,$98,$100,$102)|0);
        $103 = HEAP32[$pTab>>2]|0;
        $104 = ((($103)) + 32|0);
        HEAP16[$104>>1] = -1;
        $105 = HEAP32[$pTab>>2]|0;
        $106 = ((($105)) + 38|0);
        HEAP16[$106>>1] = 200;
        $107 = HEAP32[$pTab>>2]|0;
        $108 = ((($107)) + 42|0);
        $109 = HEAP8[$108>>0]|0;
        $110 = $109&255;
        $111 = $110 | 2;
        $112 = $111&255;
        HEAP8[$108>>0] = $112;
        break;
       }
       $113 = HEAP32[$pParse>>2]|0;
       $114 = HEAP32[$pFrom>>2]|0;
       $115 = (_sqlite3LocateTableItem($113,0,$114)|0);
       HEAP32[$pTab>>2] = $115;
       $116 = HEAP32[$pFrom>>2]|0;
       $117 = ((($116)) + 16|0);
       HEAP32[$117>>2] = $115;
       $118 = HEAP32[$pTab>>2]|0;
       $119 = ($118|0)==(0|0);
       if ($119) {
        label = 24;
        break L11;
       }
       $120 = HEAP32[$pTab>>2]|0;
       $121 = ((($120)) + 36|0);
       $122 = HEAP16[$121>>1]|0;
       $123 = $122&65535;
       $124 = ($123|0)==(65535);
       if ($124) {
        label = 26;
        break L11;
       }
       $130 = HEAP32[$pTab>>2]|0;
       $131 = ((($130)) + 36|0);
       $132 = HEAP16[$131>>1]|0;
       $133 = (($132) + 1)<<16>>16;
       HEAP16[$131>>1] = $133;
       $134 = HEAP32[$pTab>>2]|0;
       $135 = ((($134)) + 42|0);
       $136 = HEAP8[$135>>0]|0;
       $137 = $136&255;
       $138 = $137 & 16;
       $139 = ($138|0)!=(0);
       if (!($139)) {
        $140 = HEAP32[$pParse>>2]|0;
        $141 = HEAP32[$pFrom>>2]|0;
        $142 = (_cannotBeFunction($140,$141)|0);
        $143 = ($142|0)!=(0);
        if ($143) {
         label = 29;
         break L11;
        }
       }
       $144 = HEAP32[$pTab>>2]|0;
       $145 = ((($144)) + 42|0);
       $146 = HEAP8[$145>>0]|0;
       $147 = $146&255;
       $148 = $147 & 16;
       $149 = ($148|0)!=(0);
       if (!($149)) {
        $150 = HEAP32[$pTab>>2]|0;
        $151 = ((($150)) + 12|0);
        $152 = HEAP32[$151>>2]|0;
        $153 = ($152|0)!=(0|0);
        if (!($153)) {
         break;
        }
       }
       $154 = HEAP32[$pParse>>2]|0;
       $155 = HEAP32[$pTab>>2]|0;
       $156 = (_sqlite3ViewGetColumnNames($154,$155)|0);
       $157 = ($156|0)!=(0);
       if ($157) {
        label = 33;
        break L11;
       }
       $158 = HEAP32[$db>>2]|0;
       $159 = HEAP32[$pTab>>2]|0;
       $160 = ((($159)) + 12|0);
       $161 = HEAP32[$160>>2]|0;
       $162 = (_sqlite3SelectDup($158,$161,0)|0);
       $163 = HEAP32[$pFrom>>2]|0;
       $164 = ((($163)) + 20|0);
       HEAP32[$164>>2] = $162;
       $165 = HEAP32[$pTab>>2]|0;
       $166 = ((($165)) + 34|0);
       $167 = HEAP16[$166>>1]|0;
       HEAP16[$nCol>>1] = $167;
       $168 = HEAP32[$pTab>>2]|0;
       $169 = ((($168)) + 34|0);
       HEAP16[$169>>1] = -1;
       $170 = HEAP32[$1>>2]|0;
       $171 = HEAP32[$pFrom>>2]|0;
       $172 = ((($171)) + 20|0);
       $173 = HEAP32[$172>>2]|0;
       (_sqlite3WalkSelect($170,$173)|0);
       $174 = HEAP16[$nCol>>1]|0;
       $175 = HEAP32[$pTab>>2]|0;
       $176 = ((($175)) + 34|0);
       HEAP16[$176>>1] = $174;
      }
     } while(0);
     $177 = HEAP32[$pParse>>2]|0;
     $178 = HEAP32[$pFrom>>2]|0;
     $179 = (_sqlite3IndexedByLookup($177,$178)|0);
     $180 = ($179|0)!=(0);
     if ($180) {
      label = 36;
      break;
     }
    }
    $181 = HEAP32[$i>>2]|0;
    $182 = (($181) + 1)|0;
    HEAP32[$i>>2] = $182;
    $183 = HEAP32[$pFrom>>2]|0;
    $184 = ((($183)) + 72|0);
    HEAP32[$pFrom>>2] = $184;
   }
   if ((label|0) == 12) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 16) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 18) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 24) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 26) {
    $125 = HEAP32[$pParse>>2]|0;
    $126 = HEAP32[$pTab>>2]|0;
    $127 = HEAP32[$126>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $127;
    _sqlite3ErrorMsg($125,29881,$vararg_buffer4);
    $128 = HEAP32[$pFrom>>2]|0;
    $129 = ((($128)) + 16|0);
    HEAP32[$129>>2] = 0;
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 29) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 33) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 36) {
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
   else if ((label|0) == 38) {
    $185 = HEAP32[$db>>2]|0;
    $186 = ((($185)) + 65|0);
    $187 = HEAP8[$186>>0]|0;
    $188 = $187&255;
    $189 = ($188|0)!=(0);
    if (!($189)) {
     $190 = HEAP32[$pParse>>2]|0;
     $191 = HEAP32[$2>>2]|0;
     $192 = (_sqliteProcessJoin($190,$191)|0);
     $193 = ($192|0)!=(0);
     if (!($193)) {
      HEAP32[$k>>2] = 0;
      while(1) {
       $194 = HEAP32[$k>>2]|0;
       $195 = HEAP32[$pEList>>2]|0;
       $196 = HEAP32[$195>>2]|0;
       $197 = ($194|0)<($196|0);
       if (!($197)) {
        break;
       }
       $198 = HEAP32[$k>>2]|0;
       $199 = HEAP32[$pEList>>2]|0;
       $200 = ((($199)) + 4|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = (($201) + (($198*20)|0)|0);
       $203 = HEAP32[$202>>2]|0;
       HEAP32[$pE>>2] = $203;
       $204 = HEAP32[$pE>>2]|0;
       $205 = HEAP8[$204>>0]|0;
       $206 = $205&255;
       $207 = ($206|0)==(158);
       if ($207) {
        break;
       }
       $208 = HEAP32[$pE>>2]|0;
       $209 = HEAP8[$208>>0]|0;
       $210 = $209&255;
       $211 = ($210|0)==(122);
       if ($211) {
        $212 = HEAP32[$pE>>2]|0;
        $213 = ((($212)) + 16|0);
        $214 = HEAP32[$213>>2]|0;
        $215 = HEAP8[$214>>0]|0;
        $216 = $215&255;
        $217 = ($216|0)==(158);
        if ($217) {
         break;
        }
       }
       $218 = HEAP32[$k>>2]|0;
       $219 = (($218) + 1)|0;
       HEAP32[$k>>2] = $219;
      }
      $220 = HEAP32[$k>>2]|0;
      $221 = HEAP32[$pEList>>2]|0;
      $222 = HEAP32[$221>>2]|0;
      $223 = ($220|0)<($222|0);
      if ($223) {
       $224 = HEAP32[$pEList>>2]|0;
       $225 = ((($224)) + 4|0);
       $226 = HEAP32[$225>>2]|0;
       HEAP32[$a>>2] = $226;
       HEAP32[$pNew>>2] = 0;
       $227 = HEAP32[$pParse>>2]|0;
       $228 = HEAP32[$227>>2]|0;
       $229 = ((($228)) + 24|0);
       $230 = HEAP32[$229>>2]|0;
       HEAP32[$flags>>2] = $230;
       $231 = HEAP32[$flags>>2]|0;
       $232 = $231 & 4;
       $233 = ($232|0)!=(0);
       if ($233) {
        $234 = HEAP32[$flags>>2]|0;
        $235 = $234 & 64;
        $236 = ($235|0)==(0);
        $238 = $236;
       } else {
        $238 = 0;
       }
       $237 = $238&1;
       HEAP32[$longNames>>2] = $237;
       HEAP32[$k>>2] = 0;
       while(1) {
        $239 = HEAP32[$k>>2]|0;
        $240 = HEAP32[$pEList>>2]|0;
        $241 = HEAP32[$240>>2]|0;
        $242 = ($239|0)<($241|0);
        if (!($242)) {
         break;
        }
        $243 = HEAP32[$k>>2]|0;
        $244 = HEAP32[$a>>2]|0;
        $245 = (($244) + (($243*20)|0)|0);
        $246 = HEAP32[$245>>2]|0;
        HEAP32[$pE>>2] = $246;
        $247 = HEAP32[$pE>>2]|0;
        $248 = ((($247)) + 16|0);
        $249 = HEAP32[$248>>2]|0;
        HEAP32[$pRight>>2] = $249;
        $250 = HEAP32[$pE>>2]|0;
        $251 = HEAP8[$250>>0]|0;
        $252 = $251&255;
        $253 = ($252|0)!=(158);
        do {
         if ($253) {
          $254 = HEAP32[$pE>>2]|0;
          $255 = HEAP8[$254>>0]|0;
          $256 = $255&255;
          $257 = ($256|0)!=(122);
          if (!($257)) {
           $258 = HEAP32[$pRight>>2]|0;
           $259 = HEAP8[$258>>0]|0;
           $260 = $259&255;
           $261 = ($260|0)!=(158);
           if (!($261)) {
            label = 58;
            break;
           }
          }
          $262 = HEAP32[$pParse>>2]|0;
          $263 = HEAP32[$pNew>>2]|0;
          $264 = HEAP32[$k>>2]|0;
          $265 = HEAP32[$a>>2]|0;
          $266 = (($265) + (($264*20)|0)|0);
          $267 = HEAP32[$266>>2]|0;
          $268 = (_sqlite3ExprListAppend($262,$263,$267)|0);
          HEAP32[$pNew>>2] = $268;
          $269 = HEAP32[$pNew>>2]|0;
          $270 = ($269|0)!=(0|0);
          if ($270) {
           $271 = HEAP32[$k>>2]|0;
           $272 = HEAP32[$a>>2]|0;
           $273 = (($272) + (($271*20)|0)|0);
           $274 = ((($273)) + 4|0);
           $275 = HEAP32[$274>>2]|0;
           $276 = HEAP32[$pNew>>2]|0;
           $277 = HEAP32[$276>>2]|0;
           $278 = (($277) - 1)|0;
           $279 = HEAP32[$pNew>>2]|0;
           $280 = ((($279)) + 4|0);
           $281 = HEAP32[$280>>2]|0;
           $282 = (($281) + (($278*20)|0)|0);
           $283 = ((($282)) + 4|0);
           HEAP32[$283>>2] = $275;
           $284 = HEAP32[$k>>2]|0;
           $285 = HEAP32[$a>>2]|0;
           $286 = (($285) + (($284*20)|0)|0);
           $287 = ((($286)) + 8|0);
           $288 = HEAP32[$287>>2]|0;
           $289 = HEAP32[$pNew>>2]|0;
           $290 = HEAP32[$289>>2]|0;
           $291 = (($290) - 1)|0;
           $292 = HEAP32[$pNew>>2]|0;
           $293 = ((($292)) + 4|0);
           $294 = HEAP32[$293>>2]|0;
           $295 = (($294) + (($291*20)|0)|0);
           $296 = ((($295)) + 8|0);
           HEAP32[$296>>2] = $288;
           $297 = HEAP32[$k>>2]|0;
           $298 = HEAP32[$a>>2]|0;
           $299 = (($298) + (($297*20)|0)|0);
           $300 = ((($299)) + 4|0);
           HEAP32[$300>>2] = 0;
           $301 = HEAP32[$k>>2]|0;
           $302 = HEAP32[$a>>2]|0;
           $303 = (($302) + (($301*20)|0)|0);
           $304 = ((($303)) + 8|0);
           HEAP32[$304>>2] = 0;
          }
          $305 = HEAP32[$k>>2]|0;
          $306 = HEAP32[$a>>2]|0;
          $307 = (($306) + (($305*20)|0)|0);
          HEAP32[$307>>2] = 0;
         } else {
          label = 58;
         }
        } while(0);
        do {
         if ((label|0) == 58) {
          label = 0;
          HEAP32[$tableSeen>>2] = 0;
          HEAP32[$zTName>>2] = 0;
          $308 = HEAP32[$pE>>2]|0;
          $309 = HEAP8[$308>>0]|0;
          $310 = $309&255;
          $311 = ($310|0)==(122);
          if ($311) {
           $312 = HEAP32[$pE>>2]|0;
           $313 = ((($312)) + 12|0);
           $314 = HEAP32[$313>>2]|0;
           $315 = ((($314)) + 8|0);
           $316 = HEAP32[$315>>2]|0;
           HEAP32[$zTName>>2] = $316;
          }
          HEAP32[$i>>2] = 0;
          $317 = HEAP32[$pTabList>>2]|0;
          $318 = ((($317)) + 8|0);
          HEAP32[$pFrom>>2] = $318;
          while(1) {
           $319 = HEAP32[$i>>2]|0;
           $320 = HEAP32[$pTabList>>2]|0;
           $321 = HEAP32[$320>>2]|0;
           $322 = ($319|0)<($321|0);
           if (!($322)) {
            break;
           }
           $323 = HEAP32[$pFrom>>2]|0;
           $324 = ((($323)) + 16|0);
           $325 = HEAP32[$324>>2]|0;
           HEAP32[$pTab1>>2] = $325;
           $326 = HEAP32[$pFrom>>2]|0;
           $327 = ((($326)) + 20|0);
           $328 = HEAP32[$327>>2]|0;
           HEAP32[$pSub>>2] = $328;
           $329 = HEAP32[$pFrom>>2]|0;
           $330 = ((($329)) + 12|0);
           $331 = HEAP32[$330>>2]|0;
           HEAP32[$zTabName>>2] = $331;
           HEAP32[$zSchemaName>>2] = 0;
           $332 = HEAP32[$zTabName>>2]|0;
           $333 = ($332|0)==(0|0);
           if ($333) {
            $334 = HEAP32[$pTab1>>2]|0;
            $335 = HEAP32[$334>>2]|0;
            HEAP32[$zTabName>>2] = $335;
           }
           $336 = HEAP32[$db>>2]|0;
           $337 = ((($336)) + 65|0);
           $338 = HEAP8[$337>>0]|0;
           $339 = ($338<<24>>24)!=(0);
           if ($339) {
            break;
           }
           $340 = HEAP32[$pSub>>2]|0;
           $341 = ($340|0)==(0|0);
           if ($341) {
            label = 67;
           } else {
            $342 = HEAP32[$pSub>>2]|0;
            $343 = ((($342)) + 6|0);
            $344 = HEAP16[$343>>1]|0;
            $345 = $344&65535;
            $346 = $345 & 1024;
            $347 = ($346|0)==(0);
            if ($347) {
             label = 67;
            } else {
             label = 72;
            }
           }
           do {
            if ((label|0) == 67) {
             label = 0;
             HEAP32[$pSub>>2] = 0;
             $348 = HEAP32[$zTName>>2]|0;
             $349 = ($348|0)!=(0|0);
             if ($349) {
              $350 = HEAP32[$zTName>>2]|0;
              $351 = HEAP32[$zTabName>>2]|0;
              $352 = (_sqlite3_stricmp($350,$351)|0);
              $353 = ($352|0)!=(0);
              if ($353) {
               break;
              }
             }
             $354 = HEAP32[$db>>2]|0;
             $355 = HEAP32[$pTab1>>2]|0;
             $356 = ((($355)) + 64|0);
             $357 = HEAP32[$356>>2]|0;
             $358 = (_sqlite3SchemaToIndex($354,$357)|0);
             HEAP32[$iDb>>2] = $358;
             $359 = HEAP32[$iDb>>2]|0;
             $360 = ($359|0)>=(0);
             if ($360) {
              $361 = HEAP32[$iDb>>2]|0;
              $362 = HEAP32[$db>>2]|0;
              $363 = ((($362)) + 16|0);
              $364 = HEAP32[$363>>2]|0;
              $365 = (($364) + ($361<<4)|0);
              $366 = HEAP32[$365>>2]|0;
              $367 = $366;
             } else {
              $367 = 29920;
             }
             HEAP32[$zSchemaName>>2] = $367;
             label = 72;
            }
           } while(0);
           L90: do {
            if ((label|0) == 72) {
             label = 0;
             HEAP32[$j>>2] = 0;
             while(1) {
              $368 = HEAP32[$j>>2]|0;
              $369 = HEAP32[$pTab1>>2]|0;
              $370 = ((($369)) + 34|0);
              $371 = HEAP16[$370>>1]|0;
              $372 = $371 << 16 >> 16;
              $373 = ($368|0)<($372|0);
              if (!($373)) {
               break L90;
              }
              $374 = HEAP32[$j>>2]|0;
              $375 = HEAP32[$pTab1>>2]|0;
              $376 = ((($375)) + 4|0);
              $377 = HEAP32[$376>>2]|0;
              $378 = (($377) + (($374*24)|0)|0);
              $379 = HEAP32[$378>>2]|0;
              HEAP32[$zName>>2] = $379;
              $380 = HEAP32[$zTName>>2]|0;
              $381 = ($380|0)!=(0|0);
              $382 = HEAP32[$pSub>>2]|0;
              $383 = ($382|0)!=(0|0);
              $or$cond = $381 & $383;
              if ($or$cond) {
               $384 = HEAP32[$j>>2]|0;
               $385 = HEAP32[$pSub>>2]|0;
               $386 = HEAP32[$385>>2]|0;
               $387 = ((($386)) + 4|0);
               $388 = HEAP32[$387>>2]|0;
               $389 = (($388) + (($384*20)|0)|0);
               $390 = ((($389)) + 8|0);
               $391 = HEAP32[$390>>2]|0;
               $392 = HEAP32[$zTName>>2]|0;
               $393 = (_sqlite3MatchSpanName($391,0,$392,0)|0);
               $394 = ($393|0)==(0);
               if (!($394)) {
                label = 76;
               }
              } else {
               label = 76;
              }
              do {
               if ((label|0) == 76) {
                label = 0;
                $395 = HEAP32[$2>>2]|0;
                $396 = ((($395)) + 6|0);
                $397 = HEAP16[$396>>1]|0;
                $398 = $397&65535;
                $399 = $398 & 32768;
                $400 = ($399|0)==(0);
                if ($400) {
                 $401 = HEAP32[$j>>2]|0;
                 $402 = HEAP32[$pTab1>>2]|0;
                 $403 = ((($402)) + 4|0);
                 $404 = HEAP32[$403>>2]|0;
                 $405 = (($404) + (($401*24)|0)|0);
                 $406 = ((($405)) + 23|0);
                 $407 = HEAP8[$406>>0]|0;
                 $408 = $407&255;
                 $409 = $408 & 2;
                 $410 = ($409|0)!=(0);
                 if ($410) {
                  break;
                 }
                }
                HEAP32[$tableSeen>>2] = 1;
                $411 = HEAP32[$i>>2]|0;
                $412 = ($411|0)>(0);
                $413 = HEAP32[$zTName>>2]|0;
                $414 = ($413|0)==(0|0);
                $or$cond3 = $412 & $414;
                if ($or$cond3) {
                 $415 = HEAP32[$pFrom>>2]|0;
                 $416 = ((($415)) + 36|0);
                 $417 = HEAP8[$416>>0]|0;
                 $418 = $417&255;
                 $419 = $418 & 4;
                 $420 = ($419|0)!=(0);
                 if ($420) {
                  $421 = HEAP32[$pTabList>>2]|0;
                  $422 = HEAP32[$i>>2]|0;
                  $423 = HEAP32[$zName>>2]|0;
                  $424 = (_tableAndColumnIndex($421,$422,$423,0,0)|0);
                  $425 = ($424|0)!=(0);
                  if ($425) {
                   break;
                  }
                 }
                 $426 = HEAP32[$pFrom>>2]|0;
                 $427 = ((($426)) + 52|0);
                 $428 = HEAP32[$427>>2]|0;
                 $429 = HEAP32[$zName>>2]|0;
                 $430 = (_sqlite3IdListIndex($428,$429)|0);
                 $431 = ($430|0)>=(0);
                 if ($431) {
                  break;
                 }
                }
                $432 = HEAP32[$db>>2]|0;
                $433 = HEAP32[$zName>>2]|0;
                $434 = (_sqlite3Expr($432,27,$433)|0);
                HEAP32[$pRight>>2] = $434;
                $435 = HEAP32[$zName>>2]|0;
                HEAP32[$zColname>>2] = $435;
                HEAP32[$zToFree>>2] = 0;
                $436 = HEAP32[$longNames>>2]|0;
                $437 = ($436|0)!=(0);
                do {
                 if ($437) {
                  label = 84;
                 } else {
                  $438 = HEAP32[$pTabList>>2]|0;
                  $439 = HEAP32[$438>>2]|0;
                  $440 = ($439|0)>(1);
                  if ($440) {
                   label = 84;
                   break;
                  }
                  $464 = HEAP32[$pRight>>2]|0;
                  HEAP32[$pExpr>>2] = $464;
                 }
                } while(0);
                do {
                 if ((label|0) == 84) {
                  label = 0;
                  $441 = HEAP32[$db>>2]|0;
                  $442 = HEAP32[$zTabName>>2]|0;
                  $443 = (_sqlite3Expr($441,27,$442)|0);
                  HEAP32[$pLeft>>2] = $443;
                  $444 = HEAP32[$pParse>>2]|0;
                  $445 = HEAP32[$pLeft>>2]|0;
                  $446 = HEAP32[$pRight>>2]|0;
                  $447 = (_sqlite3PExpr($444,122,$445,$446,0)|0);
                  HEAP32[$pExpr>>2] = $447;
                  $448 = HEAP32[$zSchemaName>>2]|0;
                  $449 = ($448|0)!=(0|0);
                  if ($449) {
                   $450 = HEAP32[$db>>2]|0;
                   $451 = HEAP32[$zSchemaName>>2]|0;
                   $452 = (_sqlite3Expr($450,27,$451)|0);
                   HEAP32[$pLeft>>2] = $452;
                   $453 = HEAP32[$pParse>>2]|0;
                   $454 = HEAP32[$pLeft>>2]|0;
                   $455 = HEAP32[$pExpr>>2]|0;
                   $456 = (_sqlite3PExpr($453,122,$454,$455,0)|0);
                   HEAP32[$pExpr>>2] = $456;
                  }
                  $457 = HEAP32[$longNames>>2]|0;
                  $458 = ($457|0)!=(0);
                  if (!($458)) {
                   break;
                  }
                  $459 = HEAP32[$db>>2]|0;
                  $460 = HEAP32[$zTabName>>2]|0;
                  $461 = HEAP32[$zName>>2]|0;
                  HEAP32[$vararg_buffer7>>2] = $460;
                  $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
                  HEAP32[$vararg_ptr10>>2] = $461;
                  $462 = (_sqlite3MPrintf($459,29922,$vararg_buffer7)|0);
                  HEAP32[$zColname>>2] = $462;
                  $463 = HEAP32[$zColname>>2]|0;
                  HEAP32[$zToFree>>2] = $463;
                 }
                } while(0);
                $465 = HEAP32[$pParse>>2]|0;
                $466 = HEAP32[$pNew>>2]|0;
                $467 = HEAP32[$pExpr>>2]|0;
                $468 = (_sqlite3ExprListAppend($465,$466,$467)|0);
                HEAP32[$pNew>>2] = $468;
                $469 = HEAP32[$zColname>>2]|0;
                _sqlite3TokenInit($sColname,$469);
                $470 = HEAP32[$pParse>>2]|0;
                $471 = HEAP32[$pNew>>2]|0;
                _sqlite3ExprListSetName($470,$471,$sColname,0);
                $472 = HEAP32[$pNew>>2]|0;
                $473 = ($472|0)!=(0|0);
                do {
                 if ($473) {
                  $474 = HEAP32[$2>>2]|0;
                  $475 = ((($474)) + 6|0);
                  $476 = HEAP16[$475>>1]|0;
                  $477 = $476&65535;
                  $478 = $477 & 1024;
                  $479 = ($478|0)!=(0);
                  if (!($479)) {
                   break;
                  }
                  $480 = HEAP32[$pNew>>2]|0;
                  $481 = HEAP32[$480>>2]|0;
                  $482 = (($481) - 1)|0;
                  $483 = HEAP32[$pNew>>2]|0;
                  $484 = ((($483)) + 4|0);
                  $485 = HEAP32[$484>>2]|0;
                  $486 = (($485) + (($482*20)|0)|0);
                  HEAP32[$pX>>2] = $486;
                  $487 = HEAP32[$pSub>>2]|0;
                  $488 = ($487|0)!=(0|0);
                  $489 = HEAP32[$db>>2]|0;
                  if ($488) {
                   $490 = HEAP32[$j>>2]|0;
                   $491 = HEAP32[$pSub>>2]|0;
                   $492 = HEAP32[$491>>2]|0;
                   $493 = ((($492)) + 4|0);
                   $494 = HEAP32[$493>>2]|0;
                   $495 = (($494) + (($490*20)|0)|0);
                   $496 = ((($495)) + 8|0);
                   $497 = HEAP32[$496>>2]|0;
                   $498 = (_sqlite3DbStrDup($489,$497)|0);
                   $499 = HEAP32[$pX>>2]|0;
                   $500 = ((($499)) + 8|0);
                   HEAP32[$500>>2] = $498;
                  } else {
                   $501 = HEAP32[$zSchemaName>>2]|0;
                   $502 = HEAP32[$zTabName>>2]|0;
                   $503 = HEAP32[$zColname>>2]|0;
                   HEAP32[$vararg_buffer11>>2] = $501;
                   $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
                   HEAP32[$vararg_ptr14>>2] = $502;
                   $vararg_ptr15 = ((($vararg_buffer11)) + 8|0);
                   HEAP32[$vararg_ptr15>>2] = $503;
                   $504 = (_sqlite3MPrintf($489,29928,$vararg_buffer11)|0);
                   $505 = HEAP32[$pX>>2]|0;
                   $506 = ((($505)) + 8|0);
                   HEAP32[$506>>2] = $504;
                  }
                  $507 = HEAP32[$pX>>2]|0;
                  $508 = ((($507)) + 13|0);
                  $509 = HEAP8[$508>>0]|0;
                  $510 = $509 & -3;
                  $511 = $510 | 2;
                  HEAP8[$508>>0] = $511;
                 }
                } while(0);
                $512 = HEAP32[$db>>2]|0;
                $513 = HEAP32[$zToFree>>2]|0;
                _sqlite3DbFree($512,$513);
               }
              } while(0);
              $514 = HEAP32[$j>>2]|0;
              $515 = (($514) + 1)|0;
              HEAP32[$j>>2] = $515;
             }
            }
           } while(0);
           $516 = HEAP32[$i>>2]|0;
           $517 = (($516) + 1)|0;
           HEAP32[$i>>2] = $517;
           $518 = HEAP32[$pFrom>>2]|0;
           $519 = ((($518)) + 72|0);
           HEAP32[$pFrom>>2] = $519;
          }
          $520 = HEAP32[$tableSeen>>2]|0;
          $521 = ($520|0)!=(0);
          if (!($521)) {
           $522 = HEAP32[$zTName>>2]|0;
           $523 = ($522|0)!=(0|0);
           $524 = HEAP32[$pParse>>2]|0;
           if ($523) {
            $525 = HEAP32[$zTName>>2]|0;
            HEAP32[$vararg_buffer16>>2] = $525;
            _sqlite3ErrorMsg($524,29937,$vararg_buffer16);
            break;
           } else {
            _sqlite3ErrorMsg($524,29955,$vararg_buffer19);
            break;
           }
          }
         }
        } while(0);
        $526 = HEAP32[$k>>2]|0;
        $527 = (($526) + 1)|0;
        HEAP32[$k>>2] = $527;
       }
       $528 = HEAP32[$db>>2]|0;
       $529 = HEAP32[$pEList>>2]|0;
       _sqlite3ExprListDelete($528,$529);
       $530 = HEAP32[$pNew>>2]|0;
       $531 = HEAP32[$2>>2]|0;
       HEAP32[$531>>2] = $530;
      }
      $532 = HEAP32[$2>>2]|0;
      $533 = HEAP32[$532>>2]|0;
      $534 = ($533|0)!=(0|0);
      if ($534) {
       $535 = HEAP32[$2>>2]|0;
       $536 = HEAP32[$535>>2]|0;
       $537 = HEAP32[$536>>2]|0;
       $538 = HEAP32[$db>>2]|0;
       $539 = ((($538)) + 88|0);
       $540 = ((($539)) + 8|0);
       $541 = HEAP32[$540>>2]|0;
       $542 = ($537|0)>($541|0);
       if ($542) {
        $543 = HEAP32[$pParse>>2]|0;
        _sqlite3ErrorMsg($543,29975,$vararg_buffer21);
        HEAP32[$0>>2] = 2;
        $544 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($544|0);
       }
      }
      HEAP32[$0>>2] = 0;
      $544 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($544|0);
     }
    }
    HEAP32[$0>>2] = 2;
    $544 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($544|0);
   }
  }
 }
 HEAP32[$0>>2] = 1;
 $544 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($544|0);
}
function _selectPopWith($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, $pWith = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 $pWith = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pParse>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_findRightmost($4)|0);
 $6 = ((($5)) + 68|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pWith>>2] = $7;
 $8 = HEAP32[$pWith>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pWith>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pParse>>2]|0;
 $14 = ((($13)) + 536|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _findRightmost($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ((($1)) + 56|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  $5 = HEAP32[$0>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 56|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$0>>2] = $7;
 }
 STACKTOP = sp;return ($5|0);
}
function _sqlite3SrcListAssignCursors($pParse,$pList) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$pItem>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$pItem>>2]|0;
  $11 = ((($10)) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>=(0);
  if ($13) {
   label = 8;
   break;
  }
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 72|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$15>>2] = $17;
  $18 = HEAP32[$pItem>>2]|0;
  $19 = ((($18)) + 44|0);
  HEAP32[$19>>2] = $16;
  $20 = HEAP32[$pItem>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$pItem>>2]|0;
   $26 = ((($25)) + 20|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = HEAP32[$28>>2]|0;
   _sqlite3SrcListAssignCursors($24,$29);
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
  $32 = HEAP32[$pItem>>2]|0;
  $33 = ((($32)) + 72|0);
  HEAP32[$pItem>>2] = $33;
 }
 if ((label|0) == 8) {
  STACKTOP = sp;return;
 }
}
function _withExpand($pWalker,$pFrom) {
 $pWalker = $pWalker|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bMayRecursive = 0, $db = 0, $i = 0, $pCte = 0, $pEList = 0, $pItem = 0;
 var $pLeft = 0, $pParse = 0, $pSavedWith = 0, $pSel = 0, $pSrc = 0, $pTab = 0, $pWith = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $pParse = sp + 76|0;
 $db = sp + 72|0;
 $pCte = sp + 68|0;
 $pWith = sp + 64|0;
 $pTab = sp + 60|0;
 $pEList = sp + 56|0;
 $pSel = sp + 52|0;
 $pLeft = sp + 48|0;
 $bMayRecursive = sp + 44|0;
 $pSavedWith = sp + 40|0;
 $i = sp + 36|0;
 $pSrc = sp + 32|0;
 $pItem = sp + 28|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = ((($7)) + 536|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_searchWith($9,$10,$pWith)|0);
 HEAP32[$pCte>>2] = $11;
 $12 = HEAP32[$pCte>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$pCte>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  $18 = HEAP32[$pParse>>2]|0;
  if ($17) {
   $19 = HEAP32[$pCte>>2]|0;
   $20 = ((($19)) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$pCte>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$vararg_buffer>>2] = $23;
   _sqlite3ErrorMsg($18,$21,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $213 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($213|0);
  }
  $24 = HEAP32[$2>>2]|0;
  $25 = (_cannotBeFunction($18,$24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   HEAP32[$0>>2] = 1;
   $213 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($213|0);
  }
  $27 = HEAP32[$db>>2]|0;
  $28 = (_sqlite3DbMallocZero($27,72,0)|0);
  HEAP32[$pTab>>2] = $28;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  HEAP32[$30>>2] = $28;
  $31 = HEAP32[$pTab>>2]|0;
  $32 = ($31|0)==(0|0);
  if ($32) {
   HEAP32[$0>>2] = 2;
   $213 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($213|0);
  }
  $33 = HEAP32[$pTab>>2]|0;
  $34 = ((($33)) + 36|0);
  HEAP16[$34>>1] = 1;
  $35 = HEAP32[$db>>2]|0;
  $36 = HEAP32[$pCte>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3DbStrDup($35,$37)|0);
  $39 = HEAP32[$pTab>>2]|0;
  HEAP32[$39>>2] = $38;
  $40 = HEAP32[$pTab>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP16[$41>>1] = -1;
  $42 = HEAP32[$pTab>>2]|0;
  $43 = ((($42)) + 38|0);
  HEAP16[$43>>1] = 200;
  $44 = HEAP32[$pTab>>2]|0;
  $45 = ((($44)) + 42|0);
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = $47 | 66;
  $49 = $48&255;
  HEAP8[$45>>0] = $49;
  $50 = HEAP32[$db>>2]|0;
  $51 = HEAP32[$pCte>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3SelectDup($50,$53,0)|0);
  $55 = HEAP32[$2>>2]|0;
  $56 = ((($55)) + 20|0);
  HEAP32[$56>>2] = $54;
  $57 = HEAP32[$db>>2]|0;
  $58 = ((($57)) + 65|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = ($59<<24>>24)!=(0);
  if ($60) {
   HEAP32[$0>>2] = 7;
   $213 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($213|0);
  }
  $61 = HEAP32[$2>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[$pSel>>2] = $63;
  $64 = HEAP32[$pSel>>2]|0;
  $65 = ((($64)) + 4|0);
  $66 = HEAP8[$65>>0]|0;
  $67 = $66&255;
  $68 = ($67|0)==(116);
  if ($68) {
   $75 = 1;
  } else {
   $69 = HEAP32[$pSel>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = ($72|0)==(115);
   $75 = $73;
  }
  $74 = $75&1;
  HEAP32[$bMayRecursive>>2] = $74;
  $76 = HEAP32[$bMayRecursive>>2]|0;
  $77 = ($76|0)!=(0);
  L22: do {
   if ($77) {
    $78 = HEAP32[$2>>2]|0;
    $79 = ((($78)) + 20|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ((($80)) + 32|0);
    $82 = HEAP32[$81>>2]|0;
    HEAP32[$pSrc>>2] = $82;
    HEAP32[$i>>2] = 0;
    while(1) {
     $83 = HEAP32[$i>>2]|0;
     $84 = HEAP32[$pSrc>>2]|0;
     $85 = HEAP32[$84>>2]|0;
     $86 = ($83|0)<($85|0);
     if (!($86)) {
      break L22;
     }
     $87 = HEAP32[$i>>2]|0;
     $88 = HEAP32[$pSrc>>2]|0;
     $89 = ((($88)) + 8|0);
     $90 = (($89) + (($87*72)|0)|0);
     HEAP32[$pItem>>2] = $90;
     $91 = HEAP32[$pItem>>2]|0;
     $92 = ((($91)) + 4|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = ($93|0)==(0|0);
     if ($94) {
      $95 = HEAP32[$pItem>>2]|0;
      $96 = ((($95)) + 8|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ($97|0)!=(0|0);
      if ($98) {
       $99 = HEAP32[$pItem>>2]|0;
       $100 = ((($99)) + 8|0);
       $101 = HEAP32[$100>>2]|0;
       $102 = HEAP32[$pCte>>2]|0;
       $103 = HEAP32[$102>>2]|0;
       $104 = (_sqlite3_stricmp($101,$103)|0);
       $105 = (0)==($104|0);
       if ($105) {
        $106 = HEAP32[$pTab>>2]|0;
        $107 = HEAP32[$pItem>>2]|0;
        $108 = ((($107)) + 16|0);
        HEAP32[$108>>2] = $106;
        $109 = HEAP32[$pItem>>2]|0;
        $110 = ((($109)) + 36|0);
        $111 = ((($110)) + 1|0);
        $112 = HEAP8[$111>>0]|0;
        $113 = $112 & -33;
        $114 = $113 | 32;
        HEAP8[$111>>0] = $114;
        $115 = HEAP32[$pTab>>2]|0;
        $116 = ((($115)) + 36|0);
        $117 = HEAP16[$116>>1]|0;
        $118 = (($117) + 1)<<16>>16;
        HEAP16[$116>>1] = $118;
        $119 = HEAP32[$pSel>>2]|0;
        $120 = ((($119)) + 6|0);
        $121 = HEAP16[$120>>1]|0;
        $122 = $121&65535;
        $123 = $122 | 8192;
        $124 = $123&65535;
        HEAP16[$120>>1] = $124;
       }
      }
     }
     $125 = HEAP32[$i>>2]|0;
     $126 = (($125) + 1)|0;
     HEAP32[$i>>2] = $126;
    }
   }
  } while(0);
  $127 = HEAP32[$pTab>>2]|0;
  $128 = ((($127)) + 36|0);
  $129 = HEAP16[$128>>1]|0;
  $130 = $129&65535;
  $131 = ($130|0)>(2);
  if ($131) {
   $132 = HEAP32[$pParse>>2]|0;
   $133 = HEAP32[$pCte>>2]|0;
   $134 = HEAP32[$133>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $134;
   _sqlite3ErrorMsg($132,30006,$vararg_buffer1);
   HEAP32[$0>>2] = 1;
   $213 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($213|0);
  }
  $135 = HEAP32[$pCte>>2]|0;
  $136 = ((($135)) + 12|0);
  HEAP32[$136>>2] = 30049;
  $137 = HEAP32[$pParse>>2]|0;
  $138 = ((($137)) + 536|0);
  $139 = HEAP32[$138>>2]|0;
  HEAP32[$pSavedWith>>2] = $139;
  $140 = HEAP32[$pWith>>2]|0;
  $141 = HEAP32[$pParse>>2]|0;
  $142 = ((($141)) + 536|0);
  HEAP32[$142>>2] = $140;
  $143 = HEAP32[$1>>2]|0;
  $144 = HEAP32[$bMayRecursive>>2]|0;
  $145 = ($144|0)!=(0);
  $146 = HEAP32[$pSel>>2]|0;
  if ($145) {
   $147 = ((($146)) + 52|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = $148;
  } else {
   $149 = $146;
  }
  (_sqlite3WalkSelect($143,$149)|0);
  $150 = HEAP32[$pWith>>2]|0;
  $151 = HEAP32[$pParse>>2]|0;
  $152 = ((($151)) + 536|0);
  HEAP32[$152>>2] = $150;
  $153 = HEAP32[$pSel>>2]|0;
  HEAP32[$pLeft>>2] = $153;
  while(1) {
   $154 = HEAP32[$pLeft>>2]|0;
   $155 = ((($154)) + 52|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = ($156|0)!=(0|0);
   $158 = HEAP32[$pLeft>>2]|0;
   if (!($157)) {
    break;
   }
   $159 = ((($158)) + 52|0);
   $160 = HEAP32[$159>>2]|0;
   HEAP32[$pLeft>>2] = $160;
  }
  $161 = HEAP32[$158>>2]|0;
  HEAP32[$pEList>>2] = $161;
  $162 = HEAP32[$pCte>>2]|0;
  $163 = ((($162)) + 4|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = ($164|0)!=(0|0);
  if ($165) {
   $166 = HEAP32[$pEList>>2]|0;
   $167 = ($166|0)!=(0|0);
   if ($167) {
    $168 = HEAP32[$pEList>>2]|0;
    $169 = HEAP32[$168>>2]|0;
    $170 = HEAP32[$pCte>>2]|0;
    $171 = ((($170)) + 4|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$172>>2]|0;
    $174 = ($169|0)!=($173|0);
    if ($174) {
     $175 = HEAP32[$pParse>>2]|0;
     $176 = HEAP32[$pCte>>2]|0;
     $177 = HEAP32[$176>>2]|0;
     $178 = HEAP32[$pEList>>2]|0;
     $179 = HEAP32[$178>>2]|0;
     $180 = HEAP32[$pCte>>2]|0;
     $181 = ((($180)) + 4|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = HEAP32[$182>>2]|0;
     HEAP32[$vararg_buffer4>>2] = $177;
     $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $179;
     $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
     HEAP32[$vararg_ptr8>>2] = $183;
     _sqlite3ErrorMsg($175,30072,$vararg_buffer4);
     $184 = HEAP32[$pSavedWith>>2]|0;
     $185 = HEAP32[$pParse>>2]|0;
     $186 = ((($185)) + 536|0);
     HEAP32[$186>>2] = $184;
     HEAP32[$0>>2] = 1;
     $213 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($213|0);
    }
   }
   $187 = HEAP32[$pCte>>2]|0;
   $188 = ((($187)) + 4|0);
   $189 = HEAP32[$188>>2]|0;
   HEAP32[$pEList>>2] = $189;
  }
  $190 = HEAP32[$pParse>>2]|0;
  $191 = HEAP32[$pEList>>2]|0;
  $192 = HEAP32[$pTab>>2]|0;
  $193 = ((($192)) + 34|0);
  $194 = HEAP32[$pTab>>2]|0;
  $195 = ((($194)) + 4|0);
  (_sqlite3ColumnsFromExprList($190,$191,$193,$195)|0);
  $196 = HEAP32[$bMayRecursive>>2]|0;
  $197 = ($196|0)!=(0);
  if ($197) {
   $198 = HEAP32[$pSel>>2]|0;
   $199 = ((($198)) + 6|0);
   $200 = HEAP16[$199>>1]|0;
   $201 = $200&65535;
   $202 = $201 & 8192;
   $203 = ($202|0)!=(0);
   $204 = HEAP32[$pCte>>2]|0;
   $205 = ((($204)) + 12|0);
   if ($203) {
    HEAP32[$205>>2] = 30110;
   } else {
    HEAP32[$205>>2] = 30144;
   }
   $206 = HEAP32[$1>>2]|0;
   $207 = HEAP32[$pSel>>2]|0;
   (_sqlite3WalkSelect($206,$207)|0);
  }
  $208 = HEAP32[$pCte>>2]|0;
  $209 = ((($208)) + 12|0);
  HEAP32[$209>>2] = 0;
  $210 = HEAP32[$pSavedWith>>2]|0;
  $211 = HEAP32[$pParse>>2]|0;
  $212 = ((($211)) + 536|0);
  HEAP32[$212>>2] = $210;
 }
 HEAP32[$0>>2] = 0;
 $213 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($213|0);
}
function _sqlite3ColumnsFromExprList($pParse,$pEList,$pnCol,$paCol) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pnCol = $pnCol|0;
 $paCol = $paCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCol = 0, $cnt = 0, $db = 0, $ht = 0, $i = 0, $iCol = 0, $j = 0, $nCol = 0, $nName = 0, $p = 0;
 var $pCol = 0, $pColExpr = 0, $pTab = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $db = sp + 84|0;
 $i = sp + 80|0;
 $j = sp + 76|0;
 $cnt = sp + 72|0;
 $aCol = sp + 68|0;
 $pCol = sp + 64|0;
 $nCol = sp + 60|0;
 $p = sp + 56|0;
 $zName = sp + 52|0;
 $nName = sp + 48|0;
 $ht = sp + 32|0;
 $pColExpr = sp + 28|0;
 $pTab = sp + 24|0;
 $iCol = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pnCol;
 HEAP32[$4>>2] = $paCol;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 _sqlite3HashInit($ht);
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$nCol>>2] = $10;
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$nCol>>2]|0;
  $13 = ($12*24)|0;
  $14 = (_sqlite3DbMallocZero($11,$13,0)|0);
  HEAP32[$aCol>>2] = $14;
 } else {
  HEAP32[$nCol>>2] = 0;
  HEAP32[$aCol>>2] = 0;
 }
 $15 = HEAP32[$nCol>>2]|0;
 $16 = $15&65535;
 $17 = HEAP32[$3>>2]|0;
 HEAP16[$17>>1] = $16;
 $18 = HEAP32[$aCol>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 HEAP32[$19>>2] = $18;
 HEAP32[$i>>2] = 0;
 $20 = HEAP32[$aCol>>2]|0;
 HEAP32[$pCol>>2] = $20;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$nCol>>2]|0;
  $23 = ($21|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$db>>2]|0;
  $25 = ((($24)) + 65|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = ($26<<24>>24)!=(0);
  $28 = $27 ^ 1;
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + (($29*20)|0)|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (_sqlite3ExprSkipCollate($34)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = ((($37)) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + (($36*20)|0)|0);
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$zName>>2] = $42;
  $43 = ($42|0)!=(0|0);
  do {
   if (!($43)) {
    $44 = HEAP32[$p>>2]|0;
    HEAP32[$pColExpr>>2] = $44;
    while(1) {
     $45 = HEAP32[$pColExpr>>2]|0;
     $46 = HEAP8[$45>>0]|0;
     $47 = $46&255;
     $48 = ($47|0)==(122);
     $49 = HEAP32[$pColExpr>>2]|0;
     if (!($48)) {
      break;
     }
     $50 = ((($49)) + 16|0);
     $51 = HEAP32[$50>>2]|0;
     HEAP32[$pColExpr>>2] = $51;
    }
    $52 = HEAP8[$49>>0]|0;
    $53 = $52&255;
    $54 = ($53|0)==(152);
    if ($54) {
     $55 = HEAP32[$pColExpr>>2]|0;
     $56 = ((($55)) + 44|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pColExpr>>2]|0;
      $60 = ((($59)) + 32|0);
      $61 = HEAP16[$60>>1]|0;
      $62 = $61 << 16 >> 16;
      HEAP32[$iCol>>2] = $62;
      $63 = HEAP32[$pColExpr>>2]|0;
      $64 = ((($63)) + 44|0);
      $65 = HEAP32[$64>>2]|0;
      HEAP32[$pTab>>2] = $65;
      $66 = HEAP32[$iCol>>2]|0;
      $67 = ($66|0)<(0);
      if ($67) {
       $68 = HEAP32[$pTab>>2]|0;
       $69 = ((($68)) + 32|0);
       $70 = HEAP16[$69>>1]|0;
       $71 = $70 << 16 >> 16;
       HEAP32[$iCol>>2] = $71;
      }
      $72 = HEAP32[$iCol>>2]|0;
      $73 = ($72|0)>=(0);
      if ($73) {
       $74 = HEAP32[$iCol>>2]|0;
       $75 = HEAP32[$pTab>>2]|0;
       $76 = ((($75)) + 4|0);
       $77 = HEAP32[$76>>2]|0;
       $78 = (($77) + (($74*24)|0)|0);
       $79 = HEAP32[$78>>2]|0;
       $80 = $79;
      } else {
       $80 = 27337;
      }
      HEAP32[$zName>>2] = $80;
      break;
     }
    }
    $81 = HEAP32[$pColExpr>>2]|0;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = ($83|0)==(27);
    if ($84) {
     $85 = HEAP32[$pColExpr>>2]|0;
     $86 = ((($85)) + 8|0);
     $87 = HEAP32[$86>>2]|0;
     HEAP32[$zName>>2] = $87;
     break;
    } else {
     $88 = HEAP32[$i>>2]|0;
     $89 = HEAP32[$2>>2]|0;
     $90 = ((($89)) + 4|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = (($91) + (($88*20)|0)|0);
     $93 = ((($92)) + 8|0);
     $94 = HEAP32[$93>>2]|0;
     HEAP32[$zName>>2] = $94;
     break;
    }
   }
  } while(0);
  $95 = HEAP32[$db>>2]|0;
  $96 = HEAP32[$zName>>2]|0;
  HEAP32[$vararg_buffer>>2] = $96;
  $97 = (_sqlite3MPrintf($95,17521,$vararg_buffer)|0);
  HEAP32[$zName>>2] = $97;
  HEAP32[$cnt>>2] = 0;
  while(1) {
   $98 = HEAP32[$zName>>2]|0;
   $99 = ($98|0)!=(0|0);
   if ($99) {
    $100 = HEAP32[$zName>>2]|0;
    $101 = (_sqlite3HashFind($ht,$100)|0);
    $102 = ($101|0)!=(0|0);
    $169 = $102;
   } else {
    $169 = 0;
   }
   $103 = HEAP32[$zName>>2]|0;
   if (!($169)) {
    break;
   }
   $104 = (_sqlite3Strlen30($103)|0);
   HEAP32[$nName>>2] = $104;
   $105 = HEAP32[$nName>>2]|0;
   $106 = ($105|0)>(0);
   if ($106) {
    $107 = HEAP32[$nName>>2]|0;
    $108 = (($107) - 1)|0;
    HEAP32[$j>>2] = $108;
    while(1) {
     $109 = HEAP32[$j>>2]|0;
     $110 = ($109|0)>(0);
     if (!($110)) {
      break;
     }
     $111 = HEAP32[$j>>2]|0;
     $112 = HEAP32[$zName>>2]|0;
     $113 = (($112) + ($111)|0);
     $114 = HEAP8[$113>>0]|0;
     $115 = $114&255;
     $116 = (17533 + ($115)|0);
     $117 = HEAP8[$116>>0]|0;
     $118 = $117&255;
     $119 = $118 & 4;
     $120 = ($119|0)!=(0);
     if (!($120)) {
      break;
     }
     $121 = HEAP32[$j>>2]|0;
     $122 = (($121) + -1)|0;
     HEAP32[$j>>2] = $122;
    }
    $123 = HEAP32[$j>>2]|0;
    $124 = HEAP32[$zName>>2]|0;
    $125 = (($124) + ($123)|0);
    $126 = HEAP8[$125>>0]|0;
    $127 = $126 << 24 >> 24;
    $128 = ($127|0)==(58);
    if ($128) {
     $129 = HEAP32[$j>>2]|0;
     HEAP32[$nName>>2] = $129;
    }
   }
   $130 = HEAP32[$db>>2]|0;
   $131 = HEAP32[$nName>>2]|0;
   $132 = HEAP32[$zName>>2]|0;
   $133 = HEAP32[$cnt>>2]|0;
   $134 = (($133) + 1)|0;
   HEAP32[$cnt>>2] = $134;
   HEAP32[$vararg_buffer2>>2] = $131;
   $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $132;
   $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
   HEAP32[$vararg_ptr6>>2] = $134;
   $135 = (_sqlite3MPrintf($130,30182,$vararg_buffer2)|0);
   HEAP32[$zName>>2] = $135;
   $136 = HEAP32[$cnt>>2]|0;
   $137 = ($136>>>0)>(3);
   if (!($137)) {
    continue;
   }
   _sqlite3_randomness(4,$cnt);
  }
  $138 = HEAP32[$pCol>>2]|0;
  HEAP32[$138>>2] = $103;
  $139 = HEAP32[$zName>>2]|0;
  $140 = ($139|0)!=(0|0);
  if ($140) {
   $141 = HEAP32[$zName>>2]|0;
   $142 = HEAP32[$pCol>>2]|0;
   $143 = (_sqlite3HashInsert($ht,$141,$142)|0);
   $144 = HEAP32[$pCol>>2]|0;
   $145 = ($143|0)==($144|0);
   if ($145) {
    $146 = HEAP32[$db>>2]|0;
    _sqlite3OomFault($146);
   }
  }
  $147 = HEAP32[$i>>2]|0;
  $148 = (($147) + 1)|0;
  HEAP32[$i>>2] = $148;
  $149 = HEAP32[$pCol>>2]|0;
  $150 = ((($149)) + 24|0);
  HEAP32[$pCol>>2] = $150;
 }
 _sqlite3HashClear($ht);
 $151 = HEAP32[$db>>2]|0;
 $152 = ((($151)) + 65|0);
 $153 = HEAP8[$152>>0]|0;
 $154 = ($153<<24>>24)!=(0);
 if (!($154)) {
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $155 = HEAP32[$j>>2]|0;
  $156 = HEAP32[$i>>2]|0;
  $157 = ($155|0)<($156|0);
  $158 = HEAP32[$db>>2]|0;
  if (!($157)) {
   break;
  }
  $159 = HEAP32[$j>>2]|0;
  $160 = HEAP32[$aCol>>2]|0;
  $161 = (($160) + (($159*24)|0)|0);
  $162 = HEAP32[$161>>2]|0;
  _sqlite3DbFree($158,$162);
  $163 = HEAP32[$j>>2]|0;
  $164 = (($163) + 1)|0;
  HEAP32[$j>>2] = $164;
 }
 $165 = HEAP32[$aCol>>2]|0;
 _sqlite3DbFree($158,$165);
 $166 = HEAP32[$4>>2]|0;
 HEAP32[$166>>2] = 0;
 $167 = HEAP32[$3>>2]|0;
 HEAP16[$167>>1] = 0;
 HEAP32[$0>>2] = 7;
 $168 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($168|0);
}
function _sqlite3LocateTableItem($pParse,$isView,$p) {
 $pParse = $pParse|0;
 $isView = $isView|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zDb = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $isView;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3SchemaToIndex($7,$9)|0);
  HEAP32[$iDb>>2] = $10;
  $11 = HEAP32[$iDb>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ((($13)) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($11<<4)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$zDb>>2] = $17;
 } else {
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$zDb>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$zDb>>2]|0;
 $27 = (_sqlite3LocateTable($21,$22,$25,$26)|0);
 STACKTOP = sp;return ($27|0);
}
function _cannotBeFunction($pParse,$pFrom) {
 $pParse = $pParse|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = ((($4)) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$vararg_buffer>>2] = $14;
  _sqlite3ErrorMsg($11,30190,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  HEAP32[$0>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _sqlite3ViewGetColumnNames($pParse,$pTable) {
 $pParse = $pParse|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $db = 0, $n = 0, $nErr = 0, $pSel = 0, $pSelTab = 0, $vararg_buffer = 0, $xAuth = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pSelTab = sp + 24|0;
 $pSel = sp + 20|0;
 $nErr = sp + 16|0;
 $n = sp + 12|0;
 $db = sp + 8|0;
 $xAuth = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTable;
 HEAP32[$nErr>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3VtabCallConnect($5,$6)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 16;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 34|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 << 16 >> 16;
 $19 = ($18|0)>(0);
 if ($19) {
  HEAP32[$0>>2] = 0;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 34|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$vararg_buffer>>2] = $27;
  _sqlite3ErrorMsg($25,30213,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 24|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0|0);
 $32 = HEAP32[$db>>2]|0;
 do {
  if ($31) {
   $33 = ((($32)) + 248|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$33>>2] = $35;
   $36 = HEAP32[$1>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 24|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$2>>2]|0;
   $41 = ((($40)) + 34|0);
   $42 = HEAP32[$2>>2]|0;
   $43 = ((($42)) + 4|0);
   (_sqlite3ColumnsFromExprList($36,$39,$41,$43)|0);
   $44 = HEAP32[$db>>2]|0;
   $45 = ((($44)) + 248|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + -1)|0;
   HEAP32[$45>>2] = $47;
  } else {
   $48 = HEAP32[$2>>2]|0;
   $49 = ((($48)) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_sqlite3SelectDup($32,$50,0)|0);
   HEAP32[$pSel>>2] = $51;
   $52 = HEAP32[$pSel>>2]|0;
   $53 = ($52|0)!=(0|0);
   if (!($53)) {
    $109 = HEAP32[$nErr>>2]|0;
    $110 = (($109) + 1)|0;
    HEAP32[$nErr>>2] = $110;
    break;
   }
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 72|0);
   $56 = HEAP32[$55>>2]|0;
   HEAP32[$n>>2] = $56;
   $57 = HEAP32[$1>>2]|0;
   $58 = HEAP32[$pSel>>2]|0;
   $59 = ((($58)) + 32|0);
   $60 = HEAP32[$59>>2]|0;
   _sqlite3SrcListAssignCursors($57,$60);
   $61 = HEAP32[$2>>2]|0;
   $62 = ((($61)) + 34|0);
   HEAP16[$62>>1] = -1;
   $63 = HEAP32[$db>>2]|0;
   $64 = ((($63)) + 248|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($65) + 1)|0;
   HEAP32[$64>>2] = $66;
   $67 = HEAP32[$db>>2]|0;
   $68 = ((($67)) + 288|0);
   $69 = HEAP32[$68>>2]|0;
   HEAP32[$xAuth>>2] = $69;
   $70 = HEAP32[$db>>2]|0;
   $71 = ((($70)) + 288|0);
   HEAP32[$71>>2] = 0;
   $72 = HEAP32[$1>>2]|0;
   $73 = HEAP32[$pSel>>2]|0;
   $74 = (_sqlite3ResultSetOfSelect($72,$73)|0);
   HEAP32[$pSelTab>>2] = $74;
   $75 = HEAP32[$xAuth>>2]|0;
   $76 = HEAP32[$db>>2]|0;
   $77 = ((($76)) + 288|0);
   HEAP32[$77>>2] = $75;
   $78 = HEAP32[$db>>2]|0;
   $79 = ((($78)) + 248|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = (($80) + -1)|0;
   HEAP32[$79>>2] = $81;
   $82 = HEAP32[$n>>2]|0;
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 72|0);
   HEAP32[$84>>2] = $82;
   $85 = HEAP32[$pSelTab>>2]|0;
   $86 = ($85|0)!=(0|0);
   if ($86) {
    $87 = HEAP32[$pSelTab>>2]|0;
    $88 = ((($87)) + 34|0);
    $89 = HEAP16[$88>>1]|0;
    $90 = HEAP32[$2>>2]|0;
    $91 = ((($90)) + 34|0);
    HEAP16[$91>>1] = $89;
    $92 = HEAP32[$pSelTab>>2]|0;
    $93 = ((($92)) + 4|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = HEAP32[$2>>2]|0;
    $96 = ((($95)) + 4|0);
    HEAP32[$96>>2] = $94;
    $97 = HEAP32[$pSelTab>>2]|0;
    $98 = ((($97)) + 34|0);
    HEAP16[$98>>1] = 0;
    $99 = HEAP32[$pSelTab>>2]|0;
    $100 = ((($99)) + 4|0);
    HEAP32[$100>>2] = 0;
    $101 = HEAP32[$db>>2]|0;
    $102 = HEAP32[$pSelTab>>2]|0;
    _sqlite3DeleteTable($101,$102);
   } else {
    $103 = HEAP32[$2>>2]|0;
    $104 = ((($103)) + 34|0);
    HEAP16[$104>>1] = 0;
    $105 = HEAP32[$nErr>>2]|0;
    $106 = (($105) + 1)|0;
    HEAP32[$nErr>>2] = $106;
   }
   $107 = HEAP32[$db>>2]|0;
   $108 = HEAP32[$pSel>>2]|0;
   _sqlite3SelectDelete($107,$108);
  }
 } while(0);
 $111 = HEAP32[$2>>2]|0;
 $112 = ((($111)) + 64|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = ((($113)) + 78|0);
 $115 = HEAP16[$114>>1]|0;
 $116 = $115&65535;
 $117 = $116 | 2;
 $118 = $117&65535;
 HEAP16[$114>>1] = $118;
 $119 = HEAP32[$nErr>>2]|0;
 HEAP32[$0>>2] = $119;
 $120 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($120|0);
}
function _sqlite3IndexedByLookup($pParse,$pFrom) {
 $pParse = $pParse|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pIdx = 0, $pTab = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zIndexedBy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pTab = sp + 16|0;
 $zIndexedBy = sp + 12|0;
 $pIdx = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$2>>2]|0;
   $8 = ((($7)) + 36|0);
   $9 = ((($8)) + 1|0);
   $10 = HEAP8[$9>>0]|0;
   $11 = ($10&255) >>> 1;
   $12 = $11 & 1;
   $13 = $12&255;
   $14 = ($13|0)!=(0);
   if ($14) {
    $15 = HEAP32[$2>>2]|0;
    $16 = ((($15)) + 16|0);
    $17 = HEAP32[$16>>2]|0;
    HEAP32[$pTab>>2] = $17;
    $18 = HEAP32[$2>>2]|0;
    $19 = ((($18)) + 64|0);
    $20 = HEAP32[$19>>2]|0;
    HEAP32[$zIndexedBy>>2] = $20;
    $21 = HEAP32[$pTab>>2]|0;
    $22 = ((($21)) + 8|0);
    $23 = HEAP32[$22>>2]|0;
    HEAP32[$pIdx>>2] = $23;
    while(1) {
     $24 = HEAP32[$pIdx>>2]|0;
     $25 = ($24|0)!=(0|0);
     if (!($25)) {
      break;
     }
     $26 = HEAP32[$pIdx>>2]|0;
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$zIndexedBy>>2]|0;
     $29 = (_sqlite3_stricmp($27,$28)|0);
     $30 = ($29|0)!=(0);
     if (!($30)) {
      break;
     }
     $31 = HEAP32[$pIdx>>2]|0;
     $32 = ((($31)) + 20|0);
     $33 = HEAP32[$32>>2]|0;
     HEAP32[$pIdx>>2] = $33;
    }
    $34 = HEAP32[$pIdx>>2]|0;
    $35 = ($34|0)!=(0|0);
    if ($35) {
     $40 = HEAP32[$pIdx>>2]|0;
     $41 = HEAP32[$2>>2]|0;
     $42 = ((($41)) + 68|0);
     HEAP32[$42>>2] = $40;
     break;
    }
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP32[$zIndexedBy>>2]|0;
    HEAP32[$vararg_buffer>>2] = $37;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = 0;
    _sqlite3ErrorMsg($36,30243,$vararg_buffer);
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 17|0);
    HEAP8[$39>>0] = 1;
    HEAP32[$0>>2] = 1;
    $43 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($43|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _sqliteProcessJoin($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $iLeft = 0, $iLeft2 = 0, $iLeftCol = 0, $iLeftCol3 = 0, $iRightCol = 0, $isOuter = 0, $j = 0, $or$cond = 0, $pLeft = 0, $pLeftTab = 0, $pList = 0, $pRight = 0, $pRightTab = 0, $pSrc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0, $zName = 0, $zName1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $pSrc = sp + 80|0;
 $i = sp + 76|0;
 $j = sp + 72|0;
 $pLeft = sp + 68|0;
 $pRight = sp + 64|0;
 $pLeftTab = sp + 60|0;
 $pRightTab = sp + 56|0;
 $isOuter = sp + 52|0;
 $zName = sp + 48|0;
 $iLeft = sp + 44|0;
 $iLeftCol = sp + 40|0;
 $pList = sp + 36|0;
 $zName1 = sp + 32|0;
 $iLeft2 = sp + 28|0;
 $iLeftCol3 = sp + 24|0;
 $iRightCol = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ((($6)) + 8|0);
 HEAP32[$pLeft>>2] = $7;
 $8 = HEAP32[$pLeft>>2]|0;
 $9 = ((($8)) + 72|0);
 HEAP32[$pRight>>2] = $9;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pSrc>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) - 1)|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   label = 28;
   break;
  }
  $15 = HEAP32[$pLeft>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pLeftTab>>2] = $17;
  $18 = HEAP32[$pRight>>2]|0;
  $19 = ((($18)) + 16|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pRightTab>>2] = $20;
  $21 = HEAP32[$pLeftTab>>2]|0;
  $22 = ($21|0)==(0|0);
  $23 = HEAP32[$pRightTab>>2]|0;
  $24 = ($23|0)==(0|0);
  $or$cond = $22 | $24;
  L4: do {
   if (!($or$cond)) {
    $25 = HEAP32[$pRight>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = $28 & 32;
    $30 = ($29|0)!=(0);
    $31 = $30&1;
    HEAP32[$isOuter>>2] = $31;
    $32 = HEAP32[$pRight>>2]|0;
    $33 = ((($32)) + 36|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = $35 & 4;
    $37 = ($36|0)!=(0);
    L6: do {
     if ($37) {
      $38 = HEAP32[$pRight>>2]|0;
      $39 = ((($38)) + 48|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($40|0)!=(0|0);
      if ($41) {
       label = 7;
       break L1;
      }
      $42 = HEAP32[$pRight>>2]|0;
      $43 = ((($42)) + 52|0);
      $44 = HEAP32[$43>>2]|0;
      $45 = ($44|0)!=(0|0);
      if ($45) {
       label = 7;
       break L1;
      }
      HEAP32[$j>>2] = 0;
      while(1) {
       $47 = HEAP32[$j>>2]|0;
       $48 = HEAP32[$pRightTab>>2]|0;
       $49 = ((($48)) + 34|0);
       $50 = HEAP16[$49>>1]|0;
       $51 = $50 << 16 >> 16;
       $52 = ($47|0)<($51|0);
       if (!($52)) {
        break L6;
       }
       $53 = HEAP32[$j>>2]|0;
       $54 = HEAP32[$pRightTab>>2]|0;
       $55 = ((($54)) + 4|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = (($56) + (($53*24)|0)|0);
       $58 = HEAP32[$57>>2]|0;
       HEAP32[$zName>>2] = $58;
       $59 = HEAP32[$pSrc>>2]|0;
       $60 = HEAP32[$i>>2]|0;
       $61 = (($60) + 1)|0;
       $62 = HEAP32[$zName>>2]|0;
       $63 = (_tableAndColumnIndex($59,$61,$62,$iLeft,$iLeftCol)|0);
       $64 = ($63|0)!=(0);
       if ($64) {
        $65 = HEAP32[$1>>2]|0;
        $66 = HEAP32[$pSrc>>2]|0;
        $67 = HEAP32[$iLeft>>2]|0;
        $68 = HEAP32[$iLeftCol>>2]|0;
        $69 = HEAP32[$i>>2]|0;
        $70 = (($69) + 1)|0;
        $71 = HEAP32[$j>>2]|0;
        $72 = HEAP32[$isOuter>>2]|0;
        $73 = HEAP32[$2>>2]|0;
        $74 = ((($73)) + 36|0);
        _addWhereTerm($65,$66,$67,$68,$70,$71,$72,$74);
       }
       $75 = HEAP32[$j>>2]|0;
       $76 = (($75) + 1)|0;
       HEAP32[$j>>2] = $76;
      }
     }
    } while(0);
    $77 = HEAP32[$pRight>>2]|0;
    $78 = ((($77)) + 48|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)!=(0|0);
    if ($80) {
     $81 = HEAP32[$pRight>>2]|0;
     $82 = ((($81)) + 52|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)!=(0|0);
     if ($84) {
      label = 15;
      break L1;
     }
    }
    $86 = HEAP32[$pRight>>2]|0;
    $87 = ((($86)) + 48|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = ($88|0)!=(0|0);
    if ($89) {
     $90 = HEAP32[$isOuter>>2]|0;
     $91 = ($90|0)!=(0);
     if ($91) {
      $92 = HEAP32[$pRight>>2]|0;
      $93 = ((($92)) + 48|0);
      $94 = HEAP32[$93>>2]|0;
      $95 = HEAP32[$pRight>>2]|0;
      $96 = ((($95)) + 44|0);
      $97 = HEAP32[$96>>2]|0;
      _setJoinExpr($94,$97);
     }
     $98 = HEAP32[$1>>2]|0;
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$2>>2]|0;
     $101 = ((($100)) + 36|0);
     $102 = HEAP32[$101>>2]|0;
     $103 = HEAP32[$pRight>>2]|0;
     $104 = ((($103)) + 48|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (_sqlite3ExprAnd($99,$102,$105)|0);
     $107 = HEAP32[$2>>2]|0;
     $108 = ((($107)) + 36|0);
     HEAP32[$108>>2] = $106;
     $109 = HEAP32[$pRight>>2]|0;
     $110 = ((($109)) + 48|0);
     HEAP32[$110>>2] = 0;
    }
    $111 = HEAP32[$pRight>>2]|0;
    $112 = ((($111)) + 52|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = ($113|0)!=(0|0);
    if ($114) {
     $115 = HEAP32[$pRight>>2]|0;
     $116 = ((($115)) + 52|0);
     $117 = HEAP32[$116>>2]|0;
     HEAP32[$pList>>2] = $117;
     HEAP32[$j>>2] = 0;
     while(1) {
      $118 = HEAP32[$j>>2]|0;
      $119 = HEAP32[$pList>>2]|0;
      $120 = ((($119)) + 4|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ($118|0)<($121|0);
      if (!($122)) {
       break L4;
      }
      $123 = HEAP32[$j>>2]|0;
      $124 = HEAP32[$pList>>2]|0;
      $125 = HEAP32[$124>>2]|0;
      $126 = (($125) + ($123<<3)|0);
      $127 = HEAP32[$126>>2]|0;
      HEAP32[$zName1>>2] = $127;
      $128 = HEAP32[$pRightTab>>2]|0;
      $129 = HEAP32[$zName1>>2]|0;
      $130 = (_columnIndex($128,$129)|0);
      HEAP32[$iRightCol>>2] = $130;
      $131 = HEAP32[$iRightCol>>2]|0;
      $132 = ($131|0)<(0);
      if ($132) {
       label = 25;
       break L1;
      }
      $133 = HEAP32[$pSrc>>2]|0;
      $134 = HEAP32[$i>>2]|0;
      $135 = (($134) + 1)|0;
      $136 = HEAP32[$zName1>>2]|0;
      $137 = (_tableAndColumnIndex($133,$135,$136,$iLeft2,$iLeftCol3)|0);
      $138 = ($137|0)!=(0);
      if (!($138)) {
       label = 25;
       break L1;
      }
      $141 = HEAP32[$1>>2]|0;
      $142 = HEAP32[$pSrc>>2]|0;
      $143 = HEAP32[$iLeft2>>2]|0;
      $144 = HEAP32[$iLeftCol3>>2]|0;
      $145 = HEAP32[$i>>2]|0;
      $146 = (($145) + 1)|0;
      $147 = HEAP32[$iRightCol>>2]|0;
      $148 = HEAP32[$isOuter>>2]|0;
      $149 = HEAP32[$2>>2]|0;
      $150 = ((($149)) + 36|0);
      _addWhereTerm($141,$142,$143,$144,$146,$147,$148,$150);
      $151 = HEAP32[$j>>2]|0;
      $152 = (($151) + 1)|0;
      HEAP32[$j>>2] = $152;
     }
    }
   }
  } while(0);
  $153 = HEAP32[$i>>2]|0;
  $154 = (($153) + 1)|0;
  HEAP32[$i>>2] = $154;
  $155 = HEAP32[$pRight>>2]|0;
  $156 = ((($155)) + 72|0);
  HEAP32[$pRight>>2] = $156;
  $157 = HEAP32[$pLeft>>2]|0;
  $158 = ((($157)) + 72|0);
  HEAP32[$pLeft>>2] = $158;
 }
 if ((label|0) == 7) {
  $46 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = 0;
  _sqlite3ErrorMsg($46,30261,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 15) {
  $85 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($85,30311,$vararg_buffer2);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 25) {
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP32[$zName1>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $140;
  _sqlite3ErrorMsg($139,30366,$vararg_buffer4);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 28) {
  HEAP32[$0>>2] = 0;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 return (0)|0;
}
function _tableAndColumnIndex($pSrc,$N,$zCol,$piTab,$piCol) {
 $pSrc = $pSrc|0;
 $N = $N|0;
 $zCol = $zCol|0;
 $piTab = $piTab|0;
 $piCol = $piCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $i = sp + 4|0;
 $iCol = sp;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $N;
 HEAP32[$3>>2] = $zCol;
 HEAP32[$4>>2] = $piTab;
 HEAP32[$5>>2] = $piCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   label = 8;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = (($11) + (($9*72)|0)|0);
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (_columnIndex($14,$15)|0);
  HEAP32[$iCol>>2] = $16;
  $17 = HEAP32[$iCol>>2]|0;
  $18 = ($17|0)>=(0);
  if ($18) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$iCol>>2]|0;
  $24 = HEAP32[$5>>2]|0;
  HEAP32[$24>>2] = $23;
 }
 HEAP32[$0>>2] = 1;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3IdListIndex($pList,$zName) {
 $pList = $pList|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (_sqlite3_stricmp($14,$15)|0);
  $17 = ($16|0)==(0);
  $18 = HEAP32[$i>>2]|0;
  if ($17) {
   label = 6;
   break;
  }
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $18;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _searchWith($pWith,$pItem,$ppContext) {
 $pWith = $pWith|0;
 $pItem = $pItem|0;
 $ppContext = $ppContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $p = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zName = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pWith;
 HEAP32[$2>>2] = $pItem;
 HEAP32[$3>>2] = $ppContext;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$2>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$zName>>2] = $10;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$1>>2]|0;
    HEAP32[$p>>2] = $12;
    L4: while(1) {
     $13 = HEAP32[$p>>2]|0;
     $14 = ($13|0)!=(0|0);
     if (!($14)) {
      break L1;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $15 = HEAP32[$i>>2]|0;
      $16 = HEAP32[$p>>2]|0;
      $17 = HEAP32[$16>>2]|0;
      $18 = ($15|0)<($17|0);
      if (!($18)) {
       break;
      }
      $19 = HEAP32[$zName>>2]|0;
      $20 = HEAP32[$i>>2]|0;
      $21 = HEAP32[$p>>2]|0;
      $22 = ((($21)) + 8|0);
      $23 = (($22) + ($20<<4)|0);
      $24 = HEAP32[$23>>2]|0;
      $25 = (_sqlite3_stricmp($19,$24)|0);
      $26 = ($25|0)==(0);
      if ($26) {
       break L4;
      }
      $33 = HEAP32[$i>>2]|0;
      $34 = (($33) + 1)|0;
      HEAP32[$i>>2] = $34;
     }
     $35 = HEAP32[$p>>2]|0;
     $36 = ((($35)) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     HEAP32[$p>>2] = $37;
    }
    $27 = HEAP32[$p>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    HEAP32[$28>>2] = $27;
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$p>>2]|0;
    $31 = ((($30)) + 8|0);
    $32 = (($31) + ($29<<4)|0);
    HEAP32[$0>>2] = $32;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3VtabCallConnect($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pMod = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $zErr = 0, $zMod = 0, $zModule = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $db = sp + 32|0;
 $zMod = sp + 28|0;
 $pMod = sp + 24|0;
 $rc = sp + 20|0;
 $zModule = sp + 16|0;
 $zErr = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 42|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 & 16;
 $10 = ($9|0)==(0);
 if (!($10)) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_sqlite3GetVTable($11,$12)|0);
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 52|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$zMod>>2] = $18;
   $19 = HEAP32[$db>>2]|0;
   $20 = ((($19)) + 312|0);
   $21 = HEAP32[$zMod>>2]|0;
   $22 = (_sqlite3HashFind($20,$21)|0);
   HEAP32[$pMod>>2] = $22;
   $23 = HEAP32[$pMod>>2]|0;
   $24 = ($23|0)!=(0|0);
   if ($24) {
    HEAP32[$zErr>>2] = 0;
    $31 = HEAP32[$db>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = HEAP32[$pMod>>2]|0;
    $34 = HEAP32[$pMod>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    $36 = ((($35)) + 8|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (_vtabCallConstructor($31,$32,$33,$37,$zErr)|0);
    HEAP32[$rc>>2] = $38;
    $39 = HEAP32[$rc>>2]|0;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$zErr>>2]|0;
     HEAP32[$vararg_buffer1>>2] = $42;
     _sqlite3ErrorMsg($41,17521,$vararg_buffer1);
    }
    $43 = HEAP32[$db>>2]|0;
    $44 = HEAP32[$zErr>>2]|0;
    _sqlite3DbFree($43,$44);
   } else {
    $25 = HEAP32[$2>>2]|0;
    $26 = ((($25)) + 52|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$27>>2]|0;
    HEAP32[$zModule>>2] = $28;
    $29 = HEAP32[$1>>2]|0;
    $30 = HEAP32[$zModule>>2]|0;
    HEAP32[$vararg_buffer>>2] = $30;
    _sqlite3ErrorMsg($29,23747,$vararg_buffer);
    HEAP32[$rc>>2] = 1;
   }
   $45 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $45;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _addWhereTerm($pParse,$pSrc,$iLeft,$iColLeft,$iRight,$iColRight,$isOuterJoin,$ppWhere) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $iLeft = $iLeft|0;
 $iColLeft = $iColLeft|0;
 $iRight = $iRight|0;
 $iColRight = $iColRight|0;
 $isOuterJoin = $isOuterJoin|0;
 $ppWhere = $ppWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pE1 = 0, $pE2 = 0, $pEq = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $db = sp + 12|0;
 $pE1 = sp + 8|0;
 $pE2 = sp + 4|0;
 $pEq = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $iLeft;
 HEAP32[$3>>2] = $iColLeft;
 HEAP32[$4>>2] = $iRight;
 HEAP32[$5>>2] = $iColRight;
 HEAP32[$6>>2] = $isOuterJoin;
 HEAP32[$7>>2] = $ppWhere;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_sqlite3CreateColumnExpr($10,$11,$12,$13)|0);
 HEAP32[$pE1>>2] = $14;
 $15 = HEAP32[$db>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = (_sqlite3CreateColumnExpr($15,$16,$17,$18)|0);
 HEAP32[$pE2>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$pE1>>2]|0;
 $22 = HEAP32[$pE2>>2]|0;
 $23 = (_sqlite3PExpr($20,79,$21,$22,0)|0);
 HEAP32[$pEq>>2] = $23;
 $24 = HEAP32[$pEq>>2]|0;
 $25 = ($24|0)!=(0|0);
 $26 = HEAP32[$6>>2]|0;
 $27 = ($26|0)!=(0);
 $or$cond = $25 & $27;
 if (!($or$cond)) {
  $38 = HEAP32[$db>>2]|0;
  $39 = HEAP32[$7>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pEq>>2]|0;
  $42 = (_sqlite3ExprAnd($38,$40,$41)|0);
  $43 = HEAP32[$7>>2]|0;
  HEAP32[$43>>2] = $42;
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$pEq>>2]|0;
 $29 = ((($28)) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 | 1;
 HEAP32[$29>>2] = $31;
 $32 = HEAP32[$pE2>>2]|0;
 $33 = ((($32)) + 28|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $34&65535;
 $36 = HEAP32[$pEq>>2]|0;
 $37 = ((($36)) + 36|0);
 HEAP16[$37>>1] = $35;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$7>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pEq>>2]|0;
 $42 = (_sqlite3ExprAnd($38,$40,$41)|0);
 $43 = HEAP32[$7>>2]|0;
 HEAP32[$43>>2] = $42;
 STACKTOP = sp;return;
}
function _setJoinExpr($p,$iTable) {
 $p = $p|0;
 $iTable = $iTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = $6 | 1;
  HEAP32[$5>>2] = $7;
  $8 = HEAP32[$1>>2]|0;
  $9 = $8&65535;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 36|0);
  HEAP16[$11>>1] = $9;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)==(151);
  L4: do {
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = ((($16)) + 20|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($18|0)!=(0|0);
    if ($19) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $20 = HEAP32[$i>>2]|0;
      $21 = HEAP32[$0>>2]|0;
      $22 = ((($21)) + 20|0);
      $23 = HEAP32[$22>>2]|0;
      $24 = HEAP32[$23>>2]|0;
      $25 = ($20|0)<($24|0);
      if (!($25)) {
       break L4;
      }
      $26 = HEAP32[$i>>2]|0;
      $27 = HEAP32[$0>>2]|0;
      $28 = ((($27)) + 20|0);
      $29 = HEAP32[$28>>2]|0;
      $30 = ((($29)) + 4|0);
      $31 = HEAP32[$30>>2]|0;
      $32 = (($31) + (($26*20)|0)|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = HEAP32[$1>>2]|0;
      _setJoinExpr($33,$34);
      $35 = HEAP32[$i>>2]|0;
      $36 = (($35) + 1)|0;
      HEAP32[$i>>2] = $36;
     }
    }
   }
  } while(0);
  $37 = HEAP32[$0>>2]|0;
  $38 = ((($37)) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  _setJoinExpr($39,$40);
  $41 = HEAP32[$0>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  HEAP32[$0>>2] = $43;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprAnd($db,$pLeft,$pRight) {
 $db = $db|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$3>>2]|0;
 if ($5) {
  HEAP32[$0>>2] = $6;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = ($6|0)==(0|0);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  HEAP32[$0>>2] = $8;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $9 = (_exprAlwaysFalse($8)|0);
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_exprAlwaysFalse($11)|0);
  $13 = ($12|0)!=(0);
  if (!($13)) {
   $20 = HEAP32[$1>>2]|0;
   $21 = (_sqlite3ExprAlloc($20,72,0,0)|0);
   HEAP32[$pNew>>2] = $21;
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$pNew>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   _sqlite3ExprAttachSubtrees($22,$23,$24,$25);
   $26 = HEAP32[$pNew>>2]|0;
   HEAP32[$0>>2] = $26;
   $27 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($14,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($16,$17);
 $18 = HEAP32[$1>>2]|0;
 $19 = (_sqlite3ExprAlloc($18,132,2528,0)|0);
 HEAP32[$0>>2] = $19;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _columnIndex($pTab,$zCol) {
 $pTab = $pTab|0;
 $zCol = $zCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 34|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6 << 16 >> 16;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + (($9*24)|0)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (_sqlite3_stricmp($14,$15)|0);
  $17 = ($16|0)==(0);
  $18 = HEAP32[$i>>2]|0;
  if ($17) {
   label = 4;
   break;
  }
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = $18;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _sqlite3CreateColumnExpr($db,$pSrc,$iSrc,$iCol) {
 $db = $db|0;
 $pSrc = $pSrc|0;
 $iSrc = $iSrc|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $p = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $p = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $iSrc;
 HEAP32[$3>>2] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3ExprAlloc($4,152,0,0)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $60 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = (($10) + (($8*72)|0)|0);
 HEAP32[$pItem>>2] = $11;
 $12 = HEAP32[$pItem>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 44|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$pItem>>2]|0;
 $18 = ((($17)) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 28|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 44|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ((($24)) + 32|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26 << 16 >> 16;
 $28 = HEAP32[$3>>2]|0;
 $29 = ($27|0)==($28|0);
 if ($29) {
  $30 = HEAP32[$p>>2]|0;
  $31 = ((($30)) + 32|0);
  HEAP16[$31>>1] = -1;
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = $32&65535;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 32|0);
  HEAP16[$35>>1] = $33;
  $36 = HEAP32[$3>>2]|0;
  $37 = ($36|0)>=(64);
  $38 = HEAP32[$3>>2]|0;
  $39 = $37 ? 63 : $38;
  $40 = (_bitshift64Shl(1,0,($39|0))|0);
  $41 = tempRet0;
  $42 = HEAP32[$pItem>>2]|0;
  $43 = ((($42)) + 56|0);
  $44 = $43;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = $46 | $40;
  $51 = $49 | $41;
  $52 = $43;
  $53 = $52;
  HEAP32[$53>>2] = $50;
  $54 = (($52) + 4)|0;
  $55 = $54;
  HEAP32[$55>>2] = $51;
 }
 $56 = HEAP32[$p>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $58 | 4;
 HEAP32[$57>>2] = $59;
 $60 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _sqlite3ExprAttachSubtrees($db,$pRoot,$pLeft,$pRight) {
 $db = $db|0;
 $pRoot = $pRoot|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  _sqlite3ExprDelete($6,$7);
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($8,$9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 16|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$3>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = 2097408 & $17;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 | $18;
  HEAP32[$20>>2] = $22;
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 12|0);
  HEAP32[$27>>2] = $25;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = 2097408 & $30;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $34 | $31;
  HEAP32[$33>>2] = $35;
 }
 $36 = HEAP32[$1>>2]|0;
 _exprSetHeight($36);
 STACKTOP = sp;return;
}
function _exprSetHeight($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nHeight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nHeight = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nHeight>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 _heightOfExpr($3,$nHeight);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 _heightOfExpr($6,$nHeight);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 2048;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 20|0);
 if ($11) {
  $14 = HEAP32[$13>>2]|0;
  _heightOfSelect($14,$nHeight);
 } else {
  $15 = HEAP32[$13>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   _heightOfExprList($19,$nHeight);
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3ExprListFlags($22)|0);
   $24 = 2097408 & $23;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 4|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $27 | $24;
   HEAP32[$26>>2] = $28;
  }
 }
 $29 = HEAP32[$nHeight>>2]|0;
 $30 = (($29) + 1)|0;
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 24|0);
 HEAP32[$32>>2] = $30;
 STACKTOP = sp;return;
}
function _heightOfExpr($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($6|0)>($8|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 HEAP32[$13>>2] = $12;
 STACKTOP = sp;return;
}
function _heightOfSelect($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _heightOfExpr($6,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 44|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 _heightOfExpr($10,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 60|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 _heightOfExpr($14,$15);
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 _heightOfExpr($18,$19);
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 _heightOfExprList($21,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 40|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _heightOfExprList($25,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 48|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 _heightOfExprList($29,$30);
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 52|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 _heightOfSelect($33,$34);
 STACKTOP = sp;return;
}
function _heightOfExprList($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*20)|0)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  _heightOfExpr($13,$14);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprListFlags($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $m = 0, $pExpr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $m = sp + 4|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $pList;
 HEAP32[$m>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $22 = HEAP32[$m>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*20)|0)|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pExpr>>2] = $12;
  $13 = HEAP32[$pExpr>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$pExpr>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$m>>2]|0;
   $19 = $18 | $17;
   HEAP32[$m>>2] = $19;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 $22 = HEAP32[$m>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _selectAddSubqueryTypeInfo($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pFrom = 0, $pParse = 0, $pSel = 0, $pTab = 0, $pTabList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pParse = sp + 20|0;
 $i = sp + 16|0;
 $pTabList = sp + 12|0;
 $pFrom = sp + 8|0;
 $pTab = sp + 4|0;
 $pSel = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 6|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 | 64;
 $7 = $6&65535;
 HEAP16[$3>>1] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pParse>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 32|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pTabList>>2] = $12;
 HEAP32[$i>>2] = 0;
 $13 = HEAP32[$pTabList>>2]|0;
 $14 = ((($13)) + 8|0);
 HEAP32[$pFrom>>2] = $14;
 while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$pTabList>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$pFrom>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pTab>>2] = $21;
  $22 = HEAP32[$pTab>>2]|0;
  $23 = ((($22)) + 42|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = $25 & 2;
  $27 = ($26|0)!=(0);
  if ($27) {
   $28 = HEAP32[$pFrom>>2]|0;
   $29 = ((($28)) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$pSel>>2] = $30;
   $31 = HEAP32[$pSel>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    while(1) {
     $33 = HEAP32[$pSel>>2]|0;
     $34 = ((($33)) + 52|0);
     $35 = HEAP32[$34>>2]|0;
     $36 = ($35|0)!=(0|0);
     if (!($36)) {
      break;
     }
     $37 = HEAP32[$pSel>>2]|0;
     $38 = ((($37)) + 52|0);
     $39 = HEAP32[$38>>2]|0;
     HEAP32[$pSel>>2] = $39;
    }
    $40 = HEAP32[$pParse>>2]|0;
    $41 = HEAP32[$pTab>>2]|0;
    $42 = HEAP32[$pSel>>2]|0;
    _selectAddColumnTypeAndCollation($40,$41,$42);
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
  $45 = HEAP32[$pFrom>>2]|0;
  $46 = ((($45)) + 72|0);
  HEAP32[$pFrom>>2] = $46;
 }
 STACKTOP = sp;return;
}
function _selectAddColumnTypeAndCollation($pParse,$pTab,$pSelect) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $i = 0, $p = 0, $pCol = 0, $pColl = 0, $sNC = 0, $szAll = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $db = sp + 64|0;
 $sNC = sp + 32|0;
 $pCol = sp + 24|0;
 $pColl = sp + 20|0;
 $i = sp + 16|0;
 $p = sp + 12|0;
 $a = sp + 8|0;
 $szAll = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = $szAll;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 65|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 if ($12) {
  STACKTOP = sp;return;
 }
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 32|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($sNC)) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$a>>2] = $20;
 HEAP32[$i>>2] = 0;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pCol>>2] = $23;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 34|0);
  $27 = HEAP16[$26>>1]|0;
  $28 = $27 << 16 >> 16;
  $29 = ($24|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$a>>2]|0;
  $32 = (($31) + (($30*20)|0)|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$p>>2] = $33;
  $34 = HEAP32[$pCol>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)==(0|0);
  if ($37) {
   $38 = HEAP32[$db>>2]|0;
   $39 = HEAP32[$p>>2]|0;
   $40 = HEAP32[$pCol>>2]|0;
   $41 = ((($40)) + 22|0);
   $42 = (_columnTypeImpl($sNC,$39,$41)|0);
   $43 = (_sqlite3DbStrDup($38,$42)|0);
   $44 = HEAP32[$pCol>>2]|0;
   $45 = ((($44)) + 12|0);
   HEAP32[$45>>2] = $43;
  }
  $46 = HEAP32[$pCol>>2]|0;
  $47 = ((($46)) + 22|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = $szAll;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (_i64Add(($52|0),($55|0),($49|0),0)|0);
  $57 = tempRet0;
  $58 = $szAll;
  $59 = $58;
  HEAP32[$59>>2] = $56;
  $60 = (($58) + 4)|0;
  $61 = $60;
  HEAP32[$61>>2] = $57;
  $62 = HEAP32[$p>>2]|0;
  $63 = (_sqlite3ExprAffinity($62)|0);
  $64 = HEAP32[$pCol>>2]|0;
  $65 = ((($64)) + 21|0);
  HEAP8[$65>>0] = $63;
  $66 = HEAP32[$pCol>>2]|0;
  $67 = ((($66)) + 21|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 << 24 >> 24;
  $70 = ($69|0)==(0);
  if ($70) {
   $71 = HEAP32[$pCol>>2]|0;
   $72 = ((($71)) + 21|0);
   HEAP8[$72>>0] = 65;
  }
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$p>>2]|0;
  $75 = (_sqlite3ExprCollSeq($73,$74)|0);
  HEAP32[$pColl>>2] = $75;
  $76 = HEAP32[$pColl>>2]|0;
  $77 = ($76|0)!=(0|0);
  if ($77) {
   $78 = HEAP32[$pCol>>2]|0;
   $79 = ((($78)) + 16|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = ($80|0)==(0|0);
   if ($81) {
    $82 = HEAP32[$db>>2]|0;
    $83 = HEAP32[$pColl>>2]|0;
    $84 = HEAP32[$83>>2]|0;
    $85 = (_sqlite3DbStrDup($82,$84)|0);
    $86 = HEAP32[$pCol>>2]|0;
    $87 = ((($86)) + 16|0);
    HEAP32[$87>>2] = $85;
   }
  }
  $88 = HEAP32[$i>>2]|0;
  $89 = (($88) + 1)|0;
  HEAP32[$i>>2] = $89;
  $90 = HEAP32[$pCol>>2]|0;
  $91 = ((($90)) + 24|0);
  HEAP32[$pCol>>2] = $91;
 }
 $92 = $szAll;
 $93 = $92;
 $94 = HEAP32[$93>>2]|0;
 $95 = (($92) + 4)|0;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (___muldi3(($94|0),($97|0),4,0)|0);
 $99 = tempRet0;
 $100 = (_sqlite3LogEst($98,$99)|0);
 $101 = HEAP32[$1>>2]|0;
 $102 = ((($101)) + 40|0);
 HEAP16[$102>>1] = $100;
 STACKTOP = sp;return;
}
function _columnTypeImpl($pNC,$pExpr,$pEstWidth) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 $pEstWidth = $pEstWidth|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $estWidth = 0, $iCol = 0, $j = 0, $p = 0, $p3 = 0, $pS = 0, $pS2 = 0, $pTab = 0, $pTabList = 0, $sNC = 0;
 var $sNC1 = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $zType = sp + 100|0;
 $j = sp + 96|0;
 $estWidth = sp + 116|0;
 $pTab = sp + 92|0;
 $pS = sp + 88|0;
 $iCol = sp + 84|0;
 $pTabList = sp + 80|0;
 $sNC = sp + 48|0;
 $p = sp + 40|0;
 $sNC1 = sp + 8|0;
 $pS2 = sp + 4|0;
 $p3 = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pEstWidth;
 HEAP32[$zType>>2] = 0;
 HEAP8[$estWidth>>0] = 1;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 L1: do {
  switch ($5|0) {
  case 152: case 154:  {
   HEAP32[$pTab>>2] = 0;
   HEAP32[$pS>>2] = 0;
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 32|0);
   $8 = HEAP16[$7>>1]|0;
   $9 = $8 << 16 >> 16;
   HEAP32[$iCol>>2] = $9;
   while(1) {
    $10 = HEAP32[$0>>2]|0;
    $11 = ($10|0)!=(0|0);
    if (!($11)) {
     break;
    }
    $12 = HEAP32[$pTab>>2]|0;
    $13 = ($12|0)!=(0|0);
    $14 = $13 ^ 1;
    if (!($14)) {
     break;
    }
    $15 = HEAP32[$0>>2]|0;
    $16 = ((($15)) + 4|0);
    $17 = HEAP32[$16>>2]|0;
    HEAP32[$pTabList>>2] = $17;
    HEAP32[$j>>2] = 0;
    while(1) {
     $18 = HEAP32[$j>>2]|0;
     $19 = HEAP32[$pTabList>>2]|0;
     $20 = HEAP32[$19>>2]|0;
     $21 = ($18|0)<($20|0);
     if (!($21)) {
      break;
     }
     $22 = HEAP32[$j>>2]|0;
     $23 = HEAP32[$pTabList>>2]|0;
     $24 = ((($23)) + 8|0);
     $25 = (($24) + (($22*72)|0)|0);
     $26 = ((($25)) + 44|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 28|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($27|0)!=($30|0);
     if (!($31)) {
      break;
     }
     $32 = HEAP32[$j>>2]|0;
     $33 = (($32) + 1)|0;
     HEAP32[$j>>2] = $33;
    }
    $34 = HEAP32[$j>>2]|0;
    $35 = HEAP32[$pTabList>>2]|0;
    $36 = HEAP32[$35>>2]|0;
    $37 = ($34|0)<($36|0);
    if ($37) {
     $38 = HEAP32[$j>>2]|0;
     $39 = HEAP32[$pTabList>>2]|0;
     $40 = ((($39)) + 8|0);
     $41 = (($40) + (($38*72)|0)|0);
     $42 = ((($41)) + 16|0);
     $43 = HEAP32[$42>>2]|0;
     HEAP32[$pTab>>2] = $43;
     $44 = HEAP32[$j>>2]|0;
     $45 = HEAP32[$pTabList>>2]|0;
     $46 = ((($45)) + 8|0);
     $47 = (($46) + (($44*72)|0)|0);
     $48 = ((($47)) + 20|0);
     $49 = HEAP32[$48>>2]|0;
     HEAP32[$pS>>2] = $49;
     continue;
    } else {
     $50 = HEAP32[$0>>2]|0;
     $51 = ((($50)) + 16|0);
     $52 = HEAP32[$51>>2]|0;
     HEAP32[$0>>2] = $52;
     continue;
    }
   }
   $53 = HEAP32[$pTab>>2]|0;
   $54 = ($53|0)==(0|0);
   if (!($54)) {
    $55 = HEAP32[$pS>>2]|0;
    $56 = ($55|0)!=(0|0);
    if ($56) {
     $57 = HEAP32[$iCol>>2]|0;
     $58 = ($57|0)>=(0);
     if (!($58)) {
      break L1;
     }
     $59 = HEAP32[$iCol>>2]|0;
     $60 = HEAP32[$pS>>2]|0;
     $61 = HEAP32[$60>>2]|0;
     $62 = HEAP32[$61>>2]|0;
     $63 = ($59|0)<($62|0);
     if (!($63)) {
      break L1;
     }
     $64 = HEAP32[$iCol>>2]|0;
     $65 = HEAP32[$pS>>2]|0;
     $66 = HEAP32[$65>>2]|0;
     $67 = ((($66)) + 4|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + (($64*20)|0)|0);
     $70 = HEAP32[$69>>2]|0;
     HEAP32[$p>>2] = $70;
     $71 = HEAP32[$pS>>2]|0;
     $72 = ((($71)) + 32|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ((($sNC)) + 4|0);
     HEAP32[$74>>2] = $73;
     $75 = HEAP32[$0>>2]|0;
     $76 = ((($sNC)) + 16|0);
     HEAP32[$76>>2] = $75;
     $77 = HEAP32[$0>>2]|0;
     $78 = HEAP32[$77>>2]|0;
     HEAP32[$sNC>>2] = $78;
     $79 = HEAP32[$p>>2]|0;
     $80 = (_columnTypeImpl($sNC,$79,$estWidth)|0);
     HEAP32[$zType>>2] = $80;
     break L1;
    }
    $81 = HEAP32[$pTab>>2]|0;
    $82 = ((($81)) + 64|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ($83|0)!=(0|0);
    if ($84) {
     $85 = HEAP32[$iCol>>2]|0;
     $86 = ($85|0)<(0);
     if ($86) {
      $87 = HEAP32[$pTab>>2]|0;
      $88 = ((($87)) + 32|0);
      $89 = HEAP16[$88>>1]|0;
      $90 = $89 << 16 >> 16;
      HEAP32[$iCol>>2] = $90;
     }
     $91 = HEAP32[$iCol>>2]|0;
     $92 = ($91|0)<(0);
     if ($92) {
      HEAP32[$zType>>2] = 17889;
      break L1;
     } else {
      $93 = HEAP32[$iCol>>2]|0;
      $94 = HEAP32[$pTab>>2]|0;
      $95 = ((($94)) + 4|0);
      $96 = HEAP32[$95>>2]|0;
      $97 = (($96) + (($93*24)|0)|0);
      $98 = ((($97)) + 12|0);
      $99 = HEAP32[$98>>2]|0;
      HEAP32[$zType>>2] = $99;
      $100 = HEAP32[$iCol>>2]|0;
      $101 = HEAP32[$pTab>>2]|0;
      $102 = ((($101)) + 4|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = (($103) + (($100*24)|0)|0);
      $105 = ((($104)) + 22|0);
      $106 = HEAP8[$105>>0]|0;
      HEAP8[$estWidth>>0] = $106;
      break L1;
     }
    }
   }
   break;
  }
  case 119:  {
   $107 = HEAP32[$1>>2]|0;
   $108 = ((($107)) + 20|0);
   $109 = HEAP32[$108>>2]|0;
   HEAP32[$pS2>>2] = $109;
   $110 = HEAP32[$pS2>>2]|0;
   $111 = HEAP32[$110>>2]|0;
   $112 = ((($111)) + 4|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = HEAP32[$113>>2]|0;
   HEAP32[$p3>>2] = $114;
   $115 = HEAP32[$pS2>>2]|0;
   $116 = ((($115)) + 32|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = ((($sNC1)) + 4|0);
   HEAP32[$118>>2] = $117;
   $119 = HEAP32[$0>>2]|0;
   $120 = ((($sNC1)) + 16|0);
   HEAP32[$120>>2] = $119;
   $121 = HEAP32[$0>>2]|0;
   $122 = HEAP32[$121>>2]|0;
   HEAP32[$sNC1>>2] = $122;
   $123 = HEAP32[$p3>>2]|0;
   $124 = (_columnTypeImpl($sNC1,$123,$estWidth)|0);
   HEAP32[$zType>>2] = $124;
   break;
  }
  default: {
  }
  }
 } while(0);
 $125 = HEAP32[$2>>2]|0;
 $126 = ($125|0)!=(0|0);
 if (!($126)) {
  $129 = HEAP32[$zType>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 $127 = HEAP8[$estWidth>>0]|0;
 $128 = HEAP32[$2>>2]|0;
 HEAP8[$128>>0] = $127;
 $129 = HEAP32[$zType>>2]|0;
 STACKTOP = sp;return ($129|0);
}
function _resolveAsName($pParse,$pEList,$pE) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pE = $pE|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zAs = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $zCol = sp + 4|0;
 $zAs = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pE;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(27);
 L1: do {
  if ($7) {
   $8 = HEAP32[$3>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$zCol>>2] = $10;
   HEAP32[$i>>2] = 0;
   while(1) {
    $11 = HEAP32[$i>>2]|0;
    $12 = HEAP32[$2>>2]|0;
    $13 = HEAP32[$12>>2]|0;
    $14 = ($11|0)<($13|0);
    if (!($14)) {
     break L1;
    }
    $15 = HEAP32[$i>>2]|0;
    $16 = HEAP32[$2>>2]|0;
    $17 = ((($16)) + 4|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + (($15*20)|0)|0);
    $20 = ((($19)) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$zAs>>2] = $21;
    $22 = HEAP32[$zAs>>2]|0;
    $23 = ($22|0)!=(0|0);
    if ($23) {
     $24 = HEAP32[$zAs>>2]|0;
     $25 = HEAP32[$zCol>>2]|0;
     $26 = (_sqlite3_stricmp($24,$25)|0);
     $27 = ($26|0)==(0);
     if ($27) {
      break;
     }
    }
    $30 = HEAP32[$i>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$i>>2] = $31;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$0>>2] = $29;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _resolveOutOfRangeError($pParse,$zType,$i,$mx) {
 $pParse = $pParse|0;
 $zType = $zType|0;
 $i = $i|0;
 $mx = $mx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zType;
 HEAP32[$2>>2] = $i;
 HEAP32[$3>>2] = $mx;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $6;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $7;
 _sqlite3ErrorMsg($4,30430,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3ResolveOrderGroupBy($pParse,$pSelect,$pOrderBy,$zType) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 $pOrderBy = $pOrderBy|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pEList = 0, $pItem = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $i = sp + 16|0;
 $db = sp + 12|0;
 $pEList = sp + 8|0;
 $pItem = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pOrderBy;
 HEAP32[$4>>2] = $zType;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)==(0|0);
 if (!($8)) {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 65|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $15 = HEAP32[$3>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$db>>2]|0;
   $18 = ((($17)) + 88|0);
   $19 = ((($18)) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($16|0)>($20|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$4>>2]|0;
    HEAP32[$vararg_buffer>>2] = $23;
    _sqlite3ErrorMsg($22,30486,$vararg_buffer);
    HEAP32[$0>>2] = 1;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pEList>>2] = $25;
   HEAP32[$i>>2] = 0;
   $26 = HEAP32[$3>>2]|0;
   $27 = ((($26)) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   HEAP32[$pItem>>2] = $28;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$3>>2]|0;
    $31 = HEAP32[$30>>2]|0;
    $32 = ($29|0)<($31|0);
    if (!($32)) {
     label = 13;
     break;
    }
    $33 = HEAP32[$pItem>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP16[$34>>1]|0;
    $36 = ($35<<16>>16)!=(0);
    if ($36) {
     $37 = HEAP32[$pItem>>2]|0;
     $38 = ((($37)) + 16|0);
     $39 = HEAP16[$38>>1]|0;
     $40 = $39&65535;
     $41 = HEAP32[$pEList>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = ($40|0)>($42|0);
     $44 = HEAP32[$1>>2]|0;
     if ($43) {
      label = 10;
      break;
     }
     $50 = HEAP32[$pEList>>2]|0;
     $51 = HEAP32[$pItem>>2]|0;
     $52 = ((($51)) + 16|0);
     $53 = HEAP16[$52>>1]|0;
     $54 = $53&65535;
     $55 = (($54) - 1)|0;
     $56 = HEAP32[$pItem>>2]|0;
     $57 = HEAP32[$56>>2]|0;
     $58 = HEAP32[$4>>2]|0;
     _resolveAlias($44,$50,$55,$57,$58,0);
    }
    $59 = HEAP32[$i>>2]|0;
    $60 = (($59) + 1)|0;
    HEAP32[$i>>2] = $60;
    $61 = HEAP32[$pItem>>2]|0;
    $62 = ((($61)) + 20|0);
    HEAP32[$pItem>>2] = $62;
   }
   if ((label|0) == 10) {
    $45 = HEAP32[$4>>2]|0;
    $46 = HEAP32[$i>>2]|0;
    $47 = (($46) + 1)|0;
    $48 = HEAP32[$pEList>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    _resolveOutOfRangeError($44,$45,$47,$49);
    HEAP32[$0>>2] = 1;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
   else if ((label|0) == 13) {
    HEAP32[$0>>2] = 0;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $63 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _selectOpName($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 switch ($1|0) {
 case 116:  {
  HEAP32[$z>>2] = 30645;
  break;
 }
 case 118:  {
  HEAP32[$z>>2] = 30655;
  break;
 }
 case 117:  {
  HEAP32[$z>>2] = 30665;
  break;
 }
 default: {
  HEAP32[$z>>2] = 30672;
 }
 }
 $2 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _resolveOrderByTermToExprList($pParse,$pSelect,$pE) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 $pE = $pE|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $nc = 0, $pEList = 0, $rc = 0, $savedSuppErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $i = sp + 44|0;
 $pEList = sp + 40|0;
 $nc = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 $savedSuppErr = sp + 64|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pE;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pEList>>2] = $5;
 ;HEAP32[$nc>>2]=0|0;HEAP32[$nc+4>>2]=0|0;HEAP32[$nc+8>>2]=0|0;HEAP32[$nc+12>>2]=0|0;HEAP32[$nc+16>>2]=0|0;HEAP32[$nc+20>>2]=0|0;HEAP32[$nc+24>>2]=0|0;HEAP32[$nc+28>>2]=0|0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$nc>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($nc)) + 4|0);
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$pEList>>2]|0;
 $12 = ((($nc)) + 8|0);
 HEAP32[$12>>2] = $11;
 $13 = ((($nc)) + 28|0);
 HEAP16[$13>>1] = 1;
 $14 = ((($nc)) + 24|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$db>>2] = $16;
 $17 = HEAP32[$db>>2]|0;
 $18 = ((($17)) + 69|0);
 $19 = HEAP8[$18>>0]|0;
 HEAP8[$savedSuppErr>>0] = $19;
 $20 = HEAP32[$db>>2]|0;
 $21 = ((($20)) + 69|0);
 HEAP8[$21>>0] = 1;
 $22 = HEAP32[$3>>2]|0;
 $23 = (_sqlite3ResolveExprNames($nc,$22)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP8[$savedSuppErr>>0]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = ((($25)) + 69|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$pEList>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($29|0)<($31|0);
  if (!($32)) {
   label = 8;
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$pEList>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*20)|0)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$3>>2]|0;
  $40 = (_sqlite3ExprCompare($38,$39,-1)|0);
  $41 = ($40|0)<(2);
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  if ($41) {
   label = 6;
   break;
  }
  HEAP32[$i>>2] = $43;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $43;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 return (0)|0;
}
function _identLength($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$n>>2] = 0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = HEAP8[$1>>0]|0;
  $3 = ($2<<24>>24)!=(0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)==(34);
  if ($7) {
   $8 = HEAP32[$n>>2]|0;
   $9 = (($8) + 1)|0;
   HEAP32[$n>>2] = $9;
  }
  $10 = HEAP32[$n>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$n>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$0>>2] = $13;
 }
 $14 = HEAP32[$n>>2]|0;
 $15 = (($14) + 2)|0;
 STACKTOP = sp;return ($15|0);
}
function _identPut($z,$pIdx,$zSignedIdent) {
 $z = $z|0;
 $pIdx = $pIdx|0;
 $zSignedIdent = $zSignedIdent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $i = 0, $j = 0, $needQuote = 0, $zIdent = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zIdent = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $needQuote = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $zSignedIdent;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$zIdent>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$i>>2] = $5;
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$zIdent>>2]|0;
  $8 = (($7) + ($6)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = ($9<<24>>24)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$j>>2]|0;
  $12 = HEAP32[$zIdent>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (17533 + ($15)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = $18 & 6;
  $20 = ($19|0)!=(0);
  if (!($20)) {
   $21 = HEAP32[$j>>2]|0;
   $22 = HEAP32[$zIdent>>2]|0;
   $23 = (($22) + ($21)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = ($25|0)!=(95);
   if ($26) {
    break;
   }
  }
  $27 = HEAP32[$j>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$j>>2] = $28;
 }
 $29 = HEAP32[$zIdent>>2]|0;
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = (17533 + ($31)|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = $34 & 4;
 $36 = ($35|0)!=(0);
 if ($36) {
  $50 = 1;
 } else {
  $37 = HEAP32[$zIdent>>2]|0;
  $38 = HEAP32[$j>>2]|0;
  $39 = (_sqlite3KeywordCode($37,$38)|0);
  $40 = ($39|0)!=(27);
  if ($40) {
   $50 = 1;
  } else {
   $41 = HEAP32[$j>>2]|0;
   $42 = HEAP32[$zIdent>>2]|0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = ($45|0)!=(0);
   if ($46) {
    $50 = 1;
   } else {
    $47 = HEAP32[$j>>2]|0;
    $48 = ($47|0)==(0);
    $50 = $48;
   }
  }
 }
 $49 = $50&1;
 HEAP32[$needQuote>>2] = $49;
 $51 = HEAP32[$needQuote>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
  $55 = HEAP32[$0>>2]|0;
  $56 = (($55) + ($53)|0);
  HEAP8[$56>>0] = 34;
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $57 = HEAP32[$j>>2]|0;
  $58 = HEAP32[$zIdent>>2]|0;
  $59 = (($58) + ($57)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = ($60<<24>>24)!=(0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$j>>2]|0;
  $63 = HEAP32[$zIdent>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  HEAP32[$i>>2] = $67;
  $68 = HEAP32[$0>>2]|0;
  $69 = (($68) + ($66)|0);
  HEAP8[$69>>0] = $65;
  $70 = HEAP32[$j>>2]|0;
  $71 = HEAP32[$zIdent>>2]|0;
  $72 = (($71) + ($70)|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = ($74|0)==(34);
  if ($75) {
   $76 = HEAP32[$i>>2]|0;
   $77 = (($76) + 1)|0;
   HEAP32[$i>>2] = $77;
   $78 = HEAP32[$0>>2]|0;
   $79 = (($78) + ($76)|0);
   HEAP8[$79>>0] = 34;
  }
  $80 = HEAP32[$j>>2]|0;
  $81 = (($80) + 1)|0;
  HEAP32[$j>>2] = $81;
 }
 $82 = HEAP32[$needQuote>>2]|0;
 $83 = ($82|0)!=(0);
 if (!($83)) {
  $88 = HEAP32[$i>>2]|0;
  $89 = HEAP32[$0>>2]|0;
  $90 = (($89) + ($88)|0);
  HEAP8[$90>>0] = 0;
  $91 = HEAP32[$i>>2]|0;
  $92 = HEAP32[$1>>2]|0;
  HEAP32[$92>>2] = $91;
  STACKTOP = sp;return;
 }
 $84 = HEAP32[$i>>2]|0;
 $85 = (($84) + 1)|0;
 HEAP32[$i>>2] = $85;
 $86 = HEAP32[$0>>2]|0;
 $87 = (($86) + ($84)|0);
 HEAP8[$87>>0] = 34;
 $88 = HEAP32[$i>>2]|0;
 $89 = HEAP32[$0>>2]|0;
 $90 = (($89) + ($88)|0);
 HEAP8[$90>>0] = 0;
 $91 = HEAP32[$i>>2]|0;
 $92 = HEAP32[$1>>2]|0;
 HEAP32[$92>>2] = $91;
 STACKTOP = sp;return;
}
function _sqlite3ExprIsConstantOrFunction($p,$isInit) {
 $p = $p|0;
 $isInit = $isInit|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $isInit;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3&255;
 $5 = (4 + ($4))|0;
 $6 = (_exprIsConst($2,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3StringToId($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = ($3|0)==(97);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  HEAP8[$5>>0] = 27;
  STACKTOP = sp;return;
 }
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(95);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(97);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP8[$17>>0] = 27;
 STACKTOP = sp;return;
}
function _sqlite3SrcListLookup($pParse,$pSrc) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pItem = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 HEAP32[$pItem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$pItem>>2]|0;
 $6 = (_sqlite3LocateTableItem($4,0,$5)|0);
 HEAP32[$pTab>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$pItem>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DeleteTable($8,$11);
 $12 = HEAP32[$pTab>>2]|0;
 $13 = HEAP32[$pItem>>2]|0;
 $14 = ((($13)) + 16|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$pTab>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$pTab>>2]|0;
  $18 = ((($17)) + 36|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = (($19) + 1)<<16>>16;
  HEAP16[$18>>1] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$pItem>>2]|0;
 $23 = (_sqlite3IndexedByLookup($21,$22)|0);
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $25 = HEAP32[$pTab>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 HEAP32[$pTab>>2] = 0;
 $25 = HEAP32[$pTab>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3FixInit($pFix,$pParse,$iDb,$zType,$pName) {
 $pFix = $pFix|0;
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $zType = $zType|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pFix;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $zType;
 HEAP32[$4>>2] = $pName;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<4)|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17<<4)|0);
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$3>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 16|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$4>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 20|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$2>>2]|0;
 $33 = ($32|0)==(1);
 $34 = $33&1;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = $34;
 STACKTOP = sp;return;
}
function _sqlite3FixSrcList($pFix,$pList) {
 $pFix = $pFix|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $i = sp + 20|0;
 $zDb = sp + 16|0;
 $pItem = sp + 12|0;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$zDb>>2] = $7;
 HEAP32[$i>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$pItem>>2] = $9;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = ($10|0)<($12|0);
  if (!($13)) {
   label = 15;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$pItem>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$pItem>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$zDb>>2]|0;
    $26 = (_sqlite3_stricmp($24,$25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     label = 8;
     break;
    }
   }
   $39 = HEAP32[$1>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pItem>>2]|0;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   _sqlite3DbFree($41,$44);
   $45 = HEAP32[$pItem>>2]|0;
   $46 = ((($45)) + 4|0);
   HEAP32[$46>>2] = 0;
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$pItem>>2]|0;
   HEAP32[$50>>2] = $49;
  }
  $51 = HEAP32[$1>>2]|0;
  $52 = HEAP32[$pItem>>2]|0;
  $53 = ((($52)) + 20|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3FixSelect($51,$54)|0);
  $56 = ($55|0)!=(0);
  if ($56) {
   label = 11;
   break;
  }
  $57 = HEAP32[$1>>2]|0;
  $58 = HEAP32[$pItem>>2]|0;
  $59 = ((($58)) + 48|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (_sqlite3FixExpr($57,$60)|0);
  $62 = ($61|0)!=(0);
  if ($62) {
   label = 13;
   break;
  }
  $63 = HEAP32[$i>>2]|0;
  $64 = (($63) + 1)|0;
  HEAP32[$i>>2] = $64;
  $65 = HEAP32[$pItem>>2]|0;
  $66 = ((($65)) + 72|0);
  HEAP32[$pItem>>2] = $66;
 }
 if ((label|0) == 8) {
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 16|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$pItem>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$vararg_buffer>>2] = $32;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $35;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $38;
  _sqlite3ErrorMsg($29,31454,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 return (0)|0;
}
function _sqlite3AllocateIndexObject($db,$nCol,$nExtra,$ppExtra) {
 $db = $db|0;
 $nCol = $nCol|0;
 $nExtra = $nExtra|0;
 $ppExtra = $ppExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $nByte = 0, $p = 0, $pExtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 24|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $p = sp + 8|0;
 $nByte = sp + 4|0;
 $pExtra = sp;
 HEAP32[$0>>2] = $db;
 HEAP16[$1>>1] = $nCol;
 HEAP32[$2>>2] = $nExtra;
 HEAP32[$3>>2] = $ppExtra;
 $4 = HEAP16[$1>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = $5<<2;
 $7 = (($6) + 7)|0;
 $8 = $7 & -8;
 $9 = (56 + ($8))|0;
 $10 = HEAP16[$1>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (($11) + 1)|0;
 $13 = $12<<1;
 $14 = HEAP16[$1>>1]|0;
 $15 = $14 << 16 >> 16;
 $16 = $15<<1;
 $17 = (($13) + ($16))|0;
 $18 = HEAP16[$1>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = $19;
 $21 = (($17) + ($20))|0;
 $22 = (($21) + 7)|0;
 $23 = $22 & -8;
 $24 = (($9) + ($23))|0;
 HEAP32[$nByte>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$nByte>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($26) + ($27))|0;
 $29 = ($28|0)<(0);
 $30 = $29 << 31 >> 31;
 $31 = (_sqlite3DbMallocZero($25,$28,$30)|0);
 HEAP32[$p>>2] = $31;
 $32 = HEAP32[$p>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  $79 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 56|0);
 HEAP32[$pExtra>>2] = $35;
 $36 = HEAP32[$pExtra>>2]|0;
 $37 = HEAP32[$p>>2]|0;
 $38 = ((($37)) + 32|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP16[$1>>1]|0;
 $40 = $39 << 16 >> 16;
 $41 = $40<<2;
 $42 = (($41) + 7)|0;
 $43 = $42 & -8;
 $44 = HEAP32[$pExtra>>2]|0;
 $45 = (($44) + ($43)|0);
 HEAP32[$pExtra>>2] = $45;
 $46 = HEAP32[$pExtra>>2]|0;
 $47 = HEAP32[$p>>2]|0;
 $48 = ((($47)) + 8|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP16[$1>>1]|0;
 $50 = $49 << 16 >> 16;
 $51 = (($50) + 1)|0;
 $52 = $51<<1;
 $53 = HEAP32[$pExtra>>2]|0;
 $54 = (($53) + ($52)|0);
 HEAP32[$pExtra>>2] = $54;
 $55 = HEAP32[$pExtra>>2]|0;
 $56 = HEAP32[$p>>2]|0;
 $57 = ((($56)) + 4|0);
 HEAP32[$57>>2] = $55;
 $58 = HEAP16[$1>>1]|0;
 $59 = $58 << 16 >> 16;
 $60 = $59<<1;
 $61 = HEAP32[$pExtra>>2]|0;
 $62 = (($61) + ($60)|0);
 HEAP32[$pExtra>>2] = $62;
 $63 = HEAP32[$pExtra>>2]|0;
 $64 = HEAP32[$p>>2]|0;
 $65 = ((($64)) + 28|0);
 HEAP32[$65>>2] = $63;
 $66 = HEAP16[$1>>1]|0;
 $67 = HEAP32[$p>>2]|0;
 $68 = ((($67)) + 52|0);
 HEAP16[$68>>1] = $66;
 $69 = HEAP16[$1>>1]|0;
 $70 = $69 << 16 >> 16;
 $71 = (($70) - 1)|0;
 $72 = $71&65535;
 $73 = HEAP32[$p>>2]|0;
 $74 = ((($73)) + 50|0);
 HEAP16[$74>>1] = $72;
 $75 = HEAP32[$p>>2]|0;
 $76 = HEAP32[$nByte>>2]|0;
 $77 = (($75) + ($76)|0);
 $78 = HEAP32[$3>>2]|0;
 HEAP32[$78>>2] = $77;
 $79 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _sqlite3RefillIndex($pParse,$pIndex,$memRootPage) {
 $pParse = $pParse|0;
 $pIndex = $pIndex|0;
 $memRootPage = $memRootPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $addr2 = 0, $db = 0, $iDb = 0, $iIdx = 0, $iPartIdxLabel = 0;
 var $iSorter = 0, $iTab = 0, $j2 = 0, $or$cond = 0, $pKey = 0, $pTab = 0, $regRecord = 0, $tnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $pTab = sp + 52|0;
 $iTab = sp + 48|0;
 $iIdx = sp + 44|0;
 $iSorter = sp + 40|0;
 $addr1 = sp + 36|0;
 $addr2 = sp + 32|0;
 $tnum = sp + 28|0;
 $iPartIdxLabel = sp + 24|0;
 $v = sp + 20|0;
 $pKey = sp + 16|0;
 $regRecord = sp + 12|0;
 $db = sp + 8|0;
 $iDb = sp + 4|0;
 $j2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIndex;
 HEAP32[$2>>2] = $memRootPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 72|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$7>>2] = $9;
 HEAP32[$iTab>>2] = $8;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 72|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$11>>2] = $13;
 HEAP32[$iIdx>>2] = $12;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 24|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$iDb>>2]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24<<4)|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3AuthCheck($21,27,$23,0,$29)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$iDb>>2]|0;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ((($34)) + 28|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$pTab>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 _sqlite3TableLock($32,$33,$36,1,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = (_sqlite3GetVdbe($39)|0);
 HEAP32[$v>>2] = $40;
 $41 = HEAP32[$v>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$2>>2]|0;
 $44 = ($43|0)>=(0);
 if ($44) {
  $45 = HEAP32[$2>>2]|0;
  HEAP32[$tnum>>2] = $45;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 44|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$tnum>>2] = $48;
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = (_sqlite3KeyInfoOfIndex($49,$50)|0);
 HEAP32[$pKey>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 72|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 1)|0;
 HEAP32[$53>>2] = $55;
 HEAP32[$iSorter>>2] = $54;
 $56 = HEAP32[$v>>2]|0;
 $57 = HEAP32[$iSorter>>2]|0;
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 50|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 $62 = HEAP32[$pKey>>2]|0;
 $63 = (_sqlite3KeyInfoRef($62)|0);
 (_sqlite3VdbeAddOp4($56,58,$57,0,$61,$63,-6)|0);
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$iTab>>2]|0;
 $66 = HEAP32[$iDb>>2]|0;
 $67 = HEAP32[$pTab>>2]|0;
 _sqlite3OpenTable($64,$65,$66,$67,54);
 $68 = HEAP32[$v>>2]|0;
 $69 = HEAP32[$iTab>>2]|0;
 $70 = (_sqlite3VdbeAddOp2($68,108,$69,0)|0);
 HEAP32[$addr1>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 $72 = (_sqlite3GetTempReg($71)|0);
 HEAP32[$regRecord>>2] = $72;
 $73 = HEAP32[$0>>2]|0;
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$iTab>>2]|0;
 $76 = HEAP32[$regRecord>>2]|0;
 (_sqlite3GenerateIndexKey($73,$74,$75,$76,0,$iPartIdxLabel,0,0)|0);
 $77 = HEAP32[$v>>2]|0;
 $78 = HEAP32[$iSorter>>2]|0;
 $79 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($77,109,$78,$79)|0);
 $80 = HEAP32[$0>>2]|0;
 $81 = HEAP32[$iPartIdxLabel>>2]|0;
 _sqlite3ResolvePartIdxLabel($80,$81);
 $82 = HEAP32[$v>>2]|0;
 $83 = HEAP32[$iTab>>2]|0;
 $84 = HEAP32[$addr1>>2]|0;
 $85 = (($84) + 1)|0;
 (_sqlite3VdbeAddOp2($82,7,$83,$85)|0);
 $86 = HEAP32[$v>>2]|0;
 $87 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($86,$87);
 $88 = HEAP32[$2>>2]|0;
 $89 = ($88|0)<(0);
 if ($89) {
  $90 = HEAP32[$v>>2]|0;
  $91 = HEAP32[$tnum>>2]|0;
  $92 = HEAP32[$iDb>>2]|0;
  (_sqlite3VdbeAddOp2($90,119,$91,$92)|0);
 }
 $93 = HEAP32[$v>>2]|0;
 $94 = HEAP32[$iIdx>>2]|0;
 $95 = HEAP32[$tnum>>2]|0;
 $96 = HEAP32[$iDb>>2]|0;
 $97 = HEAP32[$pKey>>2]|0;
 (_sqlite3VdbeAddOp4($93,55,$94,$95,$96,$97,-6)|0);
 $98 = HEAP32[$v>>2]|0;
 $99 = HEAP32[$2>>2]|0;
 $100 = ($99|0)>=(0);
 $101 = $100 ? 16 : 0;
 $102 = 1 | $101;
 $103 = $102&255;
 _sqlite3VdbeChangeP5($98,$103);
 $104 = HEAP32[$v>>2]|0;
 $105 = HEAP32[$iSorter>>2]|0;
 $106 = (_sqlite3VdbeAddOp2($104,106,$105,0)|0);
 HEAP32[$addr1>>2] = $106;
 $107 = HEAP32[$1>>2]|0;
 $108 = ((($107)) + 54|0);
 $109 = HEAP8[$108>>0]|0;
 $110 = $109&255;
 $111 = ($110|0)!=(0);
 $112 = HEAP32[$pKey>>2]|0;
 $113 = ($112|0)!=(0|0);
 $or$cond = $111 & $113;
 $114 = HEAP32[$v>>2]|0;
 $115 = (_sqlite3VdbeCurrentAddr($114)|0);
 if ($or$cond) {
  $116 = (($115) + 3)|0;
  HEAP32[$j2>>2] = $116;
  $117 = HEAP32[$v>>2]|0;
  $118 = HEAP32[$j2>>2]|0;
  (_sqlite3VdbeGoto($117,$118)|0);
  $119 = HEAP32[$v>>2]|0;
  $120 = (_sqlite3VdbeCurrentAddr($119)|0);
  HEAP32[$addr2>>2] = $120;
  $121 = HEAP32[$v>>2]|0;
  $122 = HEAP32[$iSorter>>2]|0;
  $123 = HEAP32[$j2>>2]|0;
  $124 = HEAP32[$regRecord>>2]|0;
  $125 = HEAP32[$1>>2]|0;
  $126 = ((($125)) + 50|0);
  $127 = HEAP16[$126>>1]|0;
  $128 = $127&65535;
  (_sqlite3VdbeAddOp4Int($121,99,$122,$123,$124,$128)|0);
  $129 = HEAP32[$0>>2]|0;
  $130 = HEAP32[$1>>2]|0;
  _sqlite3UniqueConstraint($129,2,$130);
 } else {
  HEAP32[$addr2>>2] = $115;
 }
 $131 = HEAP32[$v>>2]|0;
 $132 = HEAP32[$iSorter>>2]|0;
 $133 = HEAP32[$regRecord>>2]|0;
 $134 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp3($131,100,$132,$133,$134)|0);
 $135 = HEAP32[$v>>2]|0;
 $136 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp3($135,105,$136,0,-1)|0);
 $137 = HEAP32[$v>>2]|0;
 $138 = HEAP32[$iIdx>>2]|0;
 $139 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp3($137,110,$138,$139,0)|0);
 $140 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($140,16);
 $141 = HEAP32[$0>>2]|0;
 $142 = HEAP32[$regRecord>>2]|0;
 _sqlite3ReleaseTempReg($141,$142);
 $143 = HEAP32[$v>>2]|0;
 $144 = HEAP32[$iSorter>>2]|0;
 $145 = HEAP32[$addr2>>2]|0;
 (_sqlite3VdbeAddOp2($143,3,$144,$145)|0);
 $146 = HEAP32[$v>>2]|0;
 $147 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($146,$147);
 $148 = HEAP32[$v>>2]|0;
 $149 = HEAP32[$iTab>>2]|0;
 (_sqlite3VdbeAddOp1($148,61,$149)|0);
 $150 = HEAP32[$v>>2]|0;
 $151 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp1($150,61,$151)|0);
 $152 = HEAP32[$v>>2]|0;
 $153 = HEAP32[$iSorter>>2]|0;
 (_sqlite3VdbeAddOp1($152,61,$153)|0);
 STACKTOP = sp;return;
}
function _sqlite3FixSelect($pFix,$pSelect) {
 $pFix = $pFix|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pSelect;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 20;
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3FixExprList($5,$7)|0);
  $9 = ($8|0)!=(0);
  if ($9) {
   label = 4;
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3FixSrcList($10,$13)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   label = 6;
   break;
  }
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 36|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3FixExpr($16,$19)|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   label = 8;
   break;
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 40|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3FixExprList($22,$25)|0);
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 10;
   break;
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 44|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3FixExpr($28,$31)|0);
  $33 = ($32|0)!=(0);
  if ($33) {
   label = 12;
   break;
  }
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 48|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3FixExprList($34,$37)|0);
  $39 = ($38|0)!=(0);
  if ($39) {
   label = 14;
   break;
  }
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 60|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (_sqlite3FixExpr($40,$43)|0);
  $45 = ($44|0)!=(0);
  if ($45) {
   label = 16;
   break;
  }
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 64|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_sqlite3FixExpr($46,$49)|0);
  $51 = ($50|0)!=(0);
  if ($51) {
   label = 18;
   break;
  }
  $52 = HEAP32[$2>>2]|0;
  $53 = ((($52)) + 52|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$2>>2] = $54;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 20) {
  HEAP32[$0>>2] = 0;
 }
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3FixExpr($pFix,$pExpr) {
 $pFix = $pFix|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pExpr;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 16;
   break;
  }
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)==(135);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   $12 = ((($11)) + 140|0);
   $13 = ((($12)) + 5|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = ($14<<24>>24)!=(0);
   if (!($15)) {
    label = 6;
    break;
   }
   $16 = HEAP32[$2>>2]|0;
   HEAP8[$16>>0] = 101;
  }
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 16384;
  $26 = ($25|0)!=(0);
  if ($26) {
   label = 16;
   break;
  }
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $29 & 2048;
  $31 = ($30|0)!=(0);
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = ((($33)) + 20|0);
  if ($31) {
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3FixSelect($32,$35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    label = 10;
    break;
   }
  } else {
   $38 = HEAP32[$34>>2]|0;
   $39 = (_sqlite3FixExprList($32,$38)|0);
   $40 = ($39|0)!=(0);
   if ($40) {
    label = 12;
    break;
   }
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (_sqlite3FixExpr($41,$44)|0);
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 14;
   break;
  }
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  HEAP32[$2>>2] = $49;
 }
 if ((label|0) == 6) {
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$vararg_buffer>>2] = $21;
  _sqlite3ErrorMsg($18,31500,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 return (0)|0;
}
function _sqlite3FixExprList($pFix,$pList) {
 $pFix = $pFix|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pItem>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   label = 8;
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$pItem>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3FixExpr($12,$14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 6;
   break;
  }
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = ((($19)) + 20|0);
  HEAP32[$pItem>>2] = $20;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3KeyInfoRef($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = HEAP32[$0>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[$3>>2] = $5;
 }
 $6 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3GenerateIndexKey($pParse,$pIdx,$iDataCur,$regOut,$prefixOnly,$piPartIdxLabel,$pPrior,$regPrior) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 $iDataCur = $iDataCur|0;
 $regOut = $regOut|0;
 $prefixOnly = $prefixOnly|0;
 $piPartIdxLabel = $piPartIdxLabel|0;
 $pPrior = $pPrior|0;
 $regPrior = $regPrior|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $j = 0, $nCol = 0, $regBase = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $v = sp + 12|0;
 $j = sp + 8|0;
 $regBase = sp + 4|0;
 $nCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $regOut;
 HEAP32[$4>>2] = $prefixOnly;
 HEAP32[$5>>2] = $piPartIdxLabel;
 HEAP32[$6>>2] = $pPrior;
 HEAP32[$7>>2] = $regPrior;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$5>>2]|0;
 $12 = ($11|0)!=(0|0);
 do {
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 36|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$v>>2]|0;
    $18 = (_sqlite3VdbeMakeLabel($17)|0);
    $19 = HEAP32[$5>>2]|0;
    HEAP32[$19>>2] = $18;
    $20 = HEAP32[$2>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 104|0);
    HEAP32[$22>>2] = $20;
    $23 = HEAP32[$0>>2]|0;
    _sqlite3ExprCachePush($23);
    $24 = HEAP32[$0>>2]|0;
    $25 = HEAP32[$1>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$5>>2]|0;
    $29 = HEAP32[$28>>2]|0;
    _sqlite3ExprIfFalseDup($24,$27,$29,16);
    break;
   } else {
    $30 = HEAP32[$5>>2]|0;
    HEAP32[$30>>2] = 0;
    break;
   }
  }
 } while(0);
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 55|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = ($35&255) >>> 3;
  $37 = $36 & 1;
  $38 = $37&255;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 50|0);
   $42 = HEAP16[$41>>1]|0;
   $$sink = $42;
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 52|0);
  $45 = HEAP16[$44>>1]|0;
  $$sink = $45;
 }
 $46 = $$sink&65535;
 HEAP32[$nCol>>2] = $46;
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$nCol>>2]|0;
 $49 = (_sqlite3GetTempRange($47,$48)|0);
 HEAP32[$regBase>>2] = $49;
 $50 = HEAP32[$6>>2]|0;
 $51 = ($50|0)!=(0|0);
 do {
  if ($51) {
   $52 = HEAP32[$regBase>>2]|0;
   $53 = HEAP32[$7>>2]|0;
   $54 = ($52|0)!=($53|0);
   if (!($54)) {
    $55 = HEAP32[$6>>2]|0;
    $56 = ((($55)) + 36|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0|0);
    if (!($58)) {
     break;
    }
   }
   HEAP32[$6>>2] = 0;
  }
 } while(0);
 HEAP32[$j>>2] = 0;
 while(1) {
  $59 = HEAP32[$j>>2]|0;
  $60 = HEAP32[$nCol>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$6>>2]|0;
  $63 = ($62|0)!=(0|0);
  if ($63) {
   $64 = HEAP32[$j>>2]|0;
   $65 = HEAP32[$6>>2]|0;
   $66 = ((($65)) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (($67) + ($64<<1)|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = $69 << 16 >> 16;
   $71 = HEAP32[$j>>2]|0;
   $72 = HEAP32[$1>>2]|0;
   $73 = ((($72)) + 4|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + ($71<<1)|0);
   $76 = HEAP16[$75>>1]|0;
   $77 = $76 << 16 >> 16;
   $78 = ($70|0)==($77|0);
   if ($78) {
    $79 = HEAP32[$j>>2]|0;
    $80 = HEAP32[$6>>2]|0;
    $81 = ((($80)) + 4|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (($82) + ($79<<1)|0);
    $84 = HEAP16[$83>>1]|0;
    $85 = $84 << 16 >> 16;
    $86 = ($85|0)!=(-2);
    if (!($86)) {
     label = 18;
    }
   } else {
    label = 18;
   }
  } else {
   label = 18;
  }
  if ((label|0) == 18) {
   label = 0;
   $87 = HEAP32[$0>>2]|0;
   $88 = HEAP32[$1>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = HEAP32[$j>>2]|0;
   $91 = HEAP32[$regBase>>2]|0;
   $92 = HEAP32[$j>>2]|0;
   $93 = (($91) + ($92))|0;
   _sqlite3ExprCodeLoadIndexColumn($87,$88,$89,$90,$93);
   $94 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeDeletePriorOpcode($94,39)|0);
  }
  $95 = HEAP32[$j>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$j>>2] = $96;
 }
 $97 = HEAP32[$3>>2]|0;
 $98 = ($97|0)!=(0);
 if (!($98)) {
  $103 = HEAP32[$0>>2]|0;
  $104 = HEAP32[$regBase>>2]|0;
  $105 = HEAP32[$nCol>>2]|0;
  _sqlite3ReleaseTempRange($103,$104,$105);
  $106 = HEAP32[$regBase>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $99 = HEAP32[$v>>2]|0;
 $100 = HEAP32[$regBase>>2]|0;
 $101 = HEAP32[$nCol>>2]|0;
 $102 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($99,49,$100,$101,$102)|0);
 $103 = HEAP32[$0>>2]|0;
 $104 = HEAP32[$regBase>>2]|0;
 $105 = HEAP32[$nCol>>2]|0;
 _sqlite3ReleaseTempRange($103,$104,$105);
 $106 = HEAP32[$regBase>>2]|0;
 STACKTOP = sp;return ($106|0);
}
function _sqlite3ResolvePartIdxLabel($pParse,$iLabel) {
 $pParse = $pParse|0;
 $iLabel = $iLabel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iLabel;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3VdbeResolveLabel($6,$7);
 $8 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePop($8);
 STACKTOP = sp;return;
}
function _sqlite3UniqueConstraint($pParse,$onError,$pIdx) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $errMsg = 0, $j = 0, $pTab = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $zCol = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $zErr = sp + 56|0;
 $j = sp + 52|0;
 $errMsg = sp + 24|0;
 $pTab = sp + 20|0;
 $zCol = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 _sqlite3StrAccumInit($errMsg,$7,0,0,200);
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  _sqlite3XPrintf($errMsg,31524,$vararg_buffer);
  $39 = (_sqlite3StrAccumFinish($errMsg)|0);
  HEAP32[$zErr>>2] = $39;
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 55|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43 & 3;
  $45 = $44&255;
  $46 = ($45|0)==(2);
  $47 = $46 ? 1555 : 2067;
  $48 = HEAP32[$1>>2]|0;
  $49 = HEAP32[$zErr>>2]|0;
  _sqlite3HaltConstraint($40,$47,$48,$49,-1,2);
  STACKTOP = sp;return;
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $14 = HEAP32[$j>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ((($15)) + 50|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = ($14|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$j>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<1)|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25 << 16 >> 16;
  $27 = HEAP32[$pTab>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + (($26*24)|0)|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$zCol>>2] = $31;
  $32 = HEAP32[$j>>2]|0;
  $33 = ($32|0)!=(0);
  if ($33) {
   _sqlite3StrAccumAppend($errMsg,31535,2);
  }
  $34 = HEAP32[$pTab>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$zCol>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $35;
  $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $36;
  _sqlite3XPrintf($errMsg,29922,$vararg_buffer1);
  $37 = HEAP32[$j>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$j>>2] = $38;
 }
 $39 = (_sqlite3StrAccumFinish($errMsg)|0);
 HEAP32[$zErr>>2] = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 55|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43 & 3;
 $45 = $44&255;
 $46 = ($45|0)==(2);
 $47 = $46 ? 1555 : 2067;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$zErr>>2]|0;
 _sqlite3HaltConstraint($40,$47,$48,$49,-1,2);
 STACKTOP = sp;return;
}
function _sqlite3ExprIfFalseDup($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, $pCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $db = sp + 4|0;
 $pCopy = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3ExprDup($6,$7,0)|0);
 HEAP32[$pCopy>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 65|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(0);
 if (!($13)) {
  $18 = HEAP32[$db>>2]|0;
  $19 = HEAP32[$pCopy>>2]|0;
  _sqlite3ExprDelete($18,$19);
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$pCopy>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 _sqlite3ExprIfFalse($14,$15,$16,$17);
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$pCopy>>2]|0;
 _sqlite3ExprDelete($18,$19);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeLoadIndexColumn($pParse,$pIdx,$iTabCur,$iIdxCol,$regOut) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 $iTabCur = $iTabCur|0;
 $iIdxCol = $iIdxCol|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iTabCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 $iTabCol = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iIdxCol;
 HEAP32[$4>>2] = $regOut;
 $5 = HEAP32[$3>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($5<<1)|0);
 $10 = HEAP16[$9>>1]|0;
 HEAP16[$iTabCol>>1] = $10;
 $11 = HEAP16[$iTabCol>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = ($12|0)==(-2);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 104|0);
  HEAP32[$16>>2] = $14;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 40|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($18*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  _sqlite3ExprCodeCopy($17,$25,$26);
  STACKTOP = sp;return;
 } else {
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP16[$iTabCol>>1]|0;
  $35 = $34 << 16 >> 16;
  $36 = HEAP32[$4>>2]|0;
  _sqlite3ExprCodeGetColumnOfTable($29,$32,$33,$35,$36);
  STACKTOP = sp;return;
 }
}
function _sqlite3ExprCodeCopy($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3ExprDup($5,$6,0)|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 65|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  $15 = HEAP32[$db>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  _sqlite3ExprDelete($15,$16);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($12,$13,$14);
 $15 = HEAP32[$db>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 _sqlite3ExprDelete($15,$16);
 STACKTOP = sp;return;
}
function _sqlite3CodeVerifyNamedSchema($pParse,$zDb) {
 $pParse = $pParse|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$db>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$db>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  HEAP32[$pDb>>2] = $13;
  $14 = HEAP32[$pDb>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  do {
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ($18|0)!=(0|0);
    if ($19) {
     $20 = HEAP32[$1>>2]|0;
     $21 = HEAP32[$pDb>>2]|0;
     $22 = HEAP32[$21>>2]|0;
     $23 = (_sqlite3_stricmp($20,$22)|0);
     $24 = (0)==($23|0);
     if (!($24)) {
      break;
     }
    }
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    _sqlite3CodeVerifySchema($25,$26);
   }
  } while(0);
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3ClearStatTables($pParse,$iDb,$zType,$zName) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $zType = $zType|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zDbName = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $i = sp + 28|0;
 $zDbName = sp + 24|0;
 $zTab = sp + 48|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zType;
 HEAP32[$3>>2] = $zName;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($4<<4)|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$zDbName>>2] = $10;
 HEAP32[$i>>2] = 1;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)<=(4);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  (_sqlite3_snprintf(24,$zTab,31851,$vararg_buffer)|0);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$zDbName>>2]|0;
  $17 = (_sqlite3FindTable($15,$zTab,$16)|0);
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$zDbName>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $20;
   $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $zTab;
   $vararg_ptr5 = ((($vararg_buffer1)) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $21;
   $vararg_ptr6 = ((($vararg_buffer1)) + 12|0);
   HEAP32[$vararg_ptr6>>2] = $22;
   _sqlite3NestedParse($19,31865,$vararg_buffer1);
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _sqlite3FkDropTable($pParse,$pName,$pTab) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $db = 0, $iSkip = 0, $p = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $db = sp + 12|0;
 $iSkip = sp + 8|0;
 $v = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 524288;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 42|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 16;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  STACKTOP = sp;return;
 }
 HEAP32[$iSkip>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3GetVdbe($20)|0);
 HEAP32[$v>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = (_sqlite3FkReferences($22)|0);
 $24 = ($23|0)==(0|0);
 do {
  if ($24) {
   $25 = HEAP32[$2>>2]|0;
   $26 = ((($25)) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$p>>2] = $27;
   while(1) {
    $28 = HEAP32[$p>>2]|0;
    $29 = ($28|0)!=(0|0);
    if (!($29)) {
     break;
    }
    $30 = HEAP32[$p>>2]|0;
    $31 = ((($30)) + 24|0);
    $32 = HEAP8[$31>>0]|0;
    $33 = $32&255;
    $34 = ($33|0)!=(0);
    if ($34) {
     break;
    }
    $35 = HEAP32[$db>>2]|0;
    $36 = ((($35)) + 24|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = $37 & 16777216;
    $39 = ($38|0)!=(0);
    if ($39) {
     break;
    }
    $40 = HEAP32[$p>>2]|0;
    $41 = ((($40)) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$p>>2] = $42;
   }
   $43 = HEAP32[$p>>2]|0;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $45 = HEAP32[$v>>2]|0;
    $46 = (_sqlite3VdbeMakeLabel($45)|0);
    HEAP32[$iSkip>>2] = $46;
    $47 = HEAP32[$v>>2]|0;
    $48 = HEAP32[$iSkip>>2]|0;
    (_sqlite3VdbeAddOp2($47,136,1,$48)|0);
    break;
   } else {
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 442|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$db>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = (_sqlite3SrcListDup($52,$53,0)|0);
 _sqlite3DeleteFrom($51,$54,0);
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 442|0);
 HEAP8[$56>>0] = 0;
 $57 = HEAP32[$db>>2]|0;
 $58 = ((($57)) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 & 16777216;
 $61 = ($60|0)==(0);
 if ($61) {
  $62 = HEAP32[$v>>2]|0;
  $63 = HEAP32[$v>>2]|0;
  $64 = (_sqlite3VdbeCurrentAddr($63)|0);
  $65 = (($64) + 2)|0;
  (_sqlite3VdbeAddOp2($62,136,0,$65)|0);
  $66 = HEAP32[$0>>2]|0;
  _sqlite3HaltConstraint($66,787,2,0,-2,4);
 }
 $67 = HEAP32[$iSkip>>2]|0;
 $68 = ($67|0)!=(0);
 if (!($68)) {
  STACKTOP = sp;return;
 }
 $69 = HEAP32[$v>>2]|0;
 $70 = HEAP32[$iSkip>>2]|0;
 _sqlite3VdbeResolveLabel($69,$70);
 STACKTOP = sp;return;
}
function _sqlite3CodeDropTable($pParse,$pTab,$iDb,$isView) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDb = $iDb|0;
 $isView = $isView|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pDb = 0, $pTrigger = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $v = sp + 32|0;
 $db = sp + 28|0;
 $pTrigger = sp + 24|0;
 $pDb = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $isView;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 HEAP32[$pDb>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3GetVdbe($11)|0);
 HEAP32[$v>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 _sqlite3BeginWriteOperation($13,1,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 42|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 & 16;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($21,149)|0);
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (_sqlite3TriggerList($22,$23)|0);
 HEAP32[$pTrigger>>2] = $24;
 while(1) {
  $25 = HEAP32[$pTrigger>>2]|0;
  $26 = ($25|0)!=(0|0);
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$pTrigger>>2]|0;
  _sqlite3DropTriggerPtr($27,$28);
  $29 = HEAP32[$pTrigger>>2]|0;
  $30 = ((($29)) + 32|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pTrigger>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 42|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = $35 & 8;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$pDb>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$vararg_buffer>>2] = $40;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $42;
  _sqlite3NestedParse($38,31895,$vararg_buffer);
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$pDb>>2]|0;
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = ($46|0)==(1);
 $48 = $47 ? 21260 : 21279;
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $45;
 $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $48;
 $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
 HEAP32[$vararg_ptr6>>2] = $50;
 _sqlite3NestedParse($43,31940,$vararg_buffer2);
 $51 = HEAP32[$3>>2]|0;
 $52 = ($51|0)!=(0);
 if (!($52)) {
  $53 = HEAP32[$1>>2]|0;
  $54 = ((($53)) + 42|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = $56 & 16;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   _destroyTable($59,$60);
  }
 }
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 42|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = $63&255;
 $65 = $64 & 16;
 $66 = ($65|0)!=(0);
 if (!($66)) {
  $71 = HEAP32[$v>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = HEAP32[$73>>2]|0;
  (_sqlite3VdbeAddOp4($71,125,$72,0,0,$74,0)|0);
  $75 = HEAP32[$0>>2]|0;
  $76 = HEAP32[$2>>2]|0;
  _sqlite3ChangeCookie($75,$76);
  $77 = HEAP32[$db>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  _sqliteViewResetAll($77,$78);
  STACKTOP = sp;return;
 }
 $67 = HEAP32[$v>>2]|0;
 $68 = HEAP32[$2>>2]|0;
 $69 = HEAP32[$1>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 (_sqlite3VdbeAddOp4($67,151,$68,0,0,$70,0)|0);
 $71 = HEAP32[$v>>2]|0;
 $72 = HEAP32[$2>>2]|0;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 (_sqlite3VdbeAddOp4($71,125,$72,0,0,$74,0)|0);
 $75 = HEAP32[$0>>2]|0;
 $76 = HEAP32[$2>>2]|0;
 _sqlite3ChangeCookie($75,$76);
 $77 = HEAP32[$db>>2]|0;
 $78 = HEAP32[$2>>2]|0;
 _sqliteViewResetAll($77,$78);
 STACKTOP = sp;return;
}
function _sqlite3FkReferences($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3HashFind($4,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3TriggerList($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pList = 0, $pTmpSchema = 0, $pTrig = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pTmpSchema = sp + 12|0;
 $pList = sp + 8|0;
 $p = sp + 4|0;
 $pTrig = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTmpSchema>>2] = $9;
 HEAP32[$pList>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 442|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $14 = HEAP32[$pTmpSchema>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14|0)!=($17|0);
 L5: do {
  if ($18) {
   $19 = HEAP32[$pTmpSchema>>2]|0;
   $20 = ((($19)) + 40|0);
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$p>>2] = $22;
   while(1) {
    $23 = HEAP32[$p>>2]|0;
    $24 = ($23|0)!=(0|0);
    if (!($24)) {
     break L5;
    }
    $25 = HEAP32[$p>>2]|0;
    $26 = ((($25)) + 8|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pTrig>>2] = $27;
    $28 = HEAP32[$pTrig>>2]|0;
    $29 = ((($28)) + 24|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = ((($31)) + 64|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30|0)==($33|0);
    if ($34) {
     $35 = HEAP32[$pTrig>>2]|0;
     $36 = ((($35)) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = HEAP32[$2>>2]|0;
     $39 = HEAP32[$38>>2]|0;
     $40 = (_sqlite3_stricmp($37,$39)|0);
     $41 = (0)==($40|0);
     if ($41) {
      $42 = HEAP32[$pList>>2]|0;
      $43 = ($42|0)!=(0|0);
      if ($43) {
       $44 = HEAP32[$pList>>2]|0;
       $50 = $44;
      } else {
       $45 = HEAP32[$2>>2]|0;
       $46 = ((($45)) + 60|0);
       $47 = HEAP32[$46>>2]|0;
       $50 = $47;
      }
      $48 = HEAP32[$pTrig>>2]|0;
      $49 = ((($48)) + 32|0);
      HEAP32[$49>>2] = $50;
      $51 = HEAP32[$pTrig>>2]|0;
      HEAP32[$pList>>2] = $51;
     }
    }
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    HEAP32[$p>>2] = $53;
   }
  }
 } while(0);
 $54 = HEAP32[$pList>>2]|0;
 $55 = ($54|0)!=(0|0);
 if ($55) {
  $56 = HEAP32[$pList>>2]|0;
  $60 = $56;
 } else {
  $57 = HEAP32[$2>>2]|0;
  $58 = ((($57)) + 60|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59;
 }
 HEAP32[$0>>2] = $60;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3DropTriggerPtr($pParse,$pTrigger) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $code = 0, $db = 0;
 var $iDb = 0, $pTable = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $pTable = sp + 36|0;
 $v = sp + 32|0;
 $db = sp + 28|0;
 $iDb = sp + 24|0;
 $code = sp + 20|0;
 $zDb = sp + 16|0;
 $zTab = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3SchemaToIndex($5,$8)|0);
 HEAP32[$iDb>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (_tableOfTrigger($10)|0);
 HEAP32[$pTable>>2] = $11;
 HEAP32[$code>>2] = 16;
 $12 = HEAP32[$iDb>>2]|0;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<4)|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$zDb>>2] = $17;
 $18 = HEAP32[$iDb>>2]|0;
 $19 = ($18|0)==(1);
 $20 = $19 ? 21260 : 21279;
 HEAP32[$zTab>>2] = $20;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = ($21|0)==(1);
 if ($22) {
  HEAP32[$code>>2] = 14;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$code>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$pTable>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$zDb>>2]|0;
 $30 = (_sqlite3AuthCheck($23,$24,$26,$28,$29)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$zTab>>2]|0;
 $34 = HEAP32[$zDb>>2]|0;
 $35 = (_sqlite3AuthCheck($32,9,$33,0,$34)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  STACKTOP = sp;return;
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = (_sqlite3GetVdbe($37)|0);
 HEAP32[$v>>2] = $38;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$iDb>>2]|0;
 $42 = HEAP32[$db>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<4)|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$iDb>>2]|0;
 $48 = ($47|0)==(1);
 $49 = $48 ? 21260 : 21279;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 HEAP32[$vararg_buffer>>2] = $46;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $49;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $51;
 _sqlite3NestedParse($40,31996,$vararg_buffer);
 $52 = HEAP32[$0>>2]|0;
 $53 = HEAP32[$iDb>>2]|0;
 _sqlite3ChangeCookie($52,$53);
 $54 = HEAP32[$v>>2]|0;
 $55 = HEAP32[$iDb>>2]|0;
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 (_sqlite3VdbeAddOp4($54,127,$55,0,0,$57,0)|0);
 STACKTOP = sp;return;
}
function _destroyTable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $iDestroyed = 0, $iIdx = 0, $iLargest = 0, $iTab = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $iTab = sp + 20|0;
 $iDestroyed = sp + 16|0;
 $pIdx = sp + 12|0;
 $iLargest = sp + 8|0;
 $iIdx = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$iTab>>2] = $4;
 HEAP32[$iDestroyed>>2] = 0;
 while(1) {
  HEAP32[$iLargest>>2] = 0;
  $5 = HEAP32[$iDestroyed>>2]|0;
  $6 = ($5|0)==(0);
  if ($6) {
   label = 4;
  } else {
   $7 = HEAP32[$iTab>>2]|0;
   $8 = HEAP32[$iDestroyed>>2]|0;
   $9 = ($7|0)<($8|0);
   if ($9) {
    label = 4;
   }
  }
  if ((label|0) == 4) {
   label = 0;
   $10 = HEAP32[$iTab>>2]|0;
   HEAP32[$iLargest>>2] = $10;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pIdx>>2] = $13;
  while(1) {
   $14 = HEAP32[$pIdx>>2]|0;
   $15 = ($14|0)!=(0|0);
   if (!($15)) {
    break;
   }
   $16 = HEAP32[$pIdx>>2]|0;
   $17 = ((($16)) + 44|0);
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$iIdx>>2] = $18;
   $19 = HEAP32[$iDestroyed>>2]|0;
   $20 = ($19|0)==(0);
   if ($20) {
    label = 9;
   } else {
    $21 = HEAP32[$iIdx>>2]|0;
    $22 = HEAP32[$iDestroyed>>2]|0;
    $23 = ($21|0)<($22|0);
    if ($23) {
     label = 9;
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $24 = HEAP32[$iIdx>>2]|0;
    $25 = HEAP32[$iLargest>>2]|0;
    $26 = ($24|0)>($25|0);
    if ($26) {
     $27 = HEAP32[$iIdx>>2]|0;
     HEAP32[$iLargest>>2] = $27;
    }
   }
   $28 = HEAP32[$pIdx>>2]|0;
   $29 = ((($28)) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$pIdx>>2] = $30;
  }
  $31 = HEAP32[$iLargest>>2]|0;
  $32 = ($31|0)==(0);
  if ($32) {
   break;
  }
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 64|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3SchemaToIndex($34,$37)|0);
  HEAP32[$iDb>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$iLargest>>2]|0;
  $41 = HEAP32[$iDb>>2]|0;
  _destroyRootPage($39,$40,$41);
  $42 = HEAP32[$iLargest>>2]|0;
  HEAP32[$iDestroyed>>2] = $42;
 }
 STACKTOP = sp;return;
}
function _sqliteViewResetAll($db,$idx) {
 $db = $db|0;
 $idx = $idx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $idx;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 78|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)==(2);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<4)|0);
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$i>>2] = $23;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTab>>2] = $28;
  $29 = HEAP32[$pTab>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = HEAP32[$0>>2]|0;
   $34 = HEAP32[$pTab>>2]|0;
   _sqlite3DeleteColumnNames($33,$34);
   $35 = HEAP32[$pTab>>2]|0;
   $36 = ((($35)) + 4|0);
   HEAP32[$36>>2] = 0;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = ((($37)) + 34|0);
   HEAP16[$38>>1] = 0;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<4)|0);
 $46 = ((($45)) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = ((($47)) + 78|0);
 $49 = HEAP16[$48>>1]|0;
 $50 = $49&65535;
 $51 = $50 & -3;
 $52 = $51&65535;
 HEAP16[$48>>1] = $52;
 STACKTOP = sp;return;
}
function _destroyRootPage($pParse,$iTable,$iDb) {
 $pParse = $pParse|0;
 $iTable = $iTable|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $v = sp + 24|0;
 $r1 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3GetTempReg($5)|0);
 HEAP32[$r1>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$r1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($7,118,$8,$9,$10)|0);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3MayAbort($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($13<<4)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ($20|0)==(1);
 $22 = $21 ? 21260 : 21279;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$r1>>2]|0;
 $25 = HEAP32[$r1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $19;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $22;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $23;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $24;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $25;
 _sqlite3NestedParse($12,32047,$vararg_buffer);
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($26,$27);
 STACKTOP = sp;return;
}
function _checkForMultiColumnSelectError($pParse,$pDest,$nExpr) {
 $pParse = $pParse|0;
 $pDest = $pDest|0;
 $nExpr = $nExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eDest = 0, $or$cond = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $eDest = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$3>>2] = $nExpr;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 HEAP32[$eDest>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)>(1);
 if ($8) {
  $9 = HEAP32[$eDest>>2]|0;
  $10 = ($9|0)==(10);
  $11 = HEAP32[$eDest>>2]|0;
  $12 = ($11|0)==(11);
  $or$cond = $10 | $12;
  if ($or$cond) {
   $13 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($13,32229,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $14 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($14|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _flattenSubquery($pParse,$p,$iFrom,$isAgg,$subqueryIsAgg) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iFrom = $iFrom|0;
 $isAgg = $isAgg|0;
 $subqueryIsAgg = $subqueryIsAgg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iParent = 0, $ii = 0, $jointype = 0, $nSubSrc = 0, $or$cond = 0;
 var $or$cond3 = 0, $pLimit = 0, $pList = 0, $pNew = 0, $pOffset = 0, $pOrderBy = 0, $pOrderBy1 = 0, $pParent = 0, $pPrior = 0, $pSrc = 0, $pSub = 0, $pSub1 = 0, $pSubSrc = 0, $pSubitem = 0, $pTabToDel = 0, $pToplevel = 0, $pWhere = 0, $zName = 0, $zSavedAuthContext = 0, dest = 0;
 var label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $zSavedAuthContext = sp + 88|0;
 $pParent = sp + 84|0;
 $pSub = sp + 80|0;
 $pSub1 = sp + 76|0;
 $pSrc = sp + 72|0;
 $pSubSrc = sp + 68|0;
 $pList = sp + 64|0;
 $iParent = sp + 60|0;
 $i = sp + 56|0;
 $pWhere = sp + 52|0;
 $pSubitem = sp + 48|0;
 $db = sp + 44|0;
 $ii = sp + 40|0;
 $pNew = sp + 36|0;
 $pOrderBy = sp + 32|0;
 $pLimit = sp + 28|0;
 $pOffset = sp + 24|0;
 $pPrior = sp + 20|0;
 $pTabToDel = sp + 16|0;
 $pToplevel = sp + 12|0;
 $nSubSrc = sp + 8|0;
 $jointype = sp + 116|0;
 $zName = sp + 4|0;
 $pOrderBy1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $iFrom;
 HEAP32[$4>>2] = $isAgg;
 HEAP32[$5>>2] = $subqueryIsAgg;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 496|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$zSavedAuthContext>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 60|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 1;
 $16 = ($15|0)!=(0);
 if ($16) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pSrc>>2] = $19;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$pSrc>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = (($22) + (($20*72)|0)|0);
 HEAP32[$pSubitem>>2] = $23;
 $24 = HEAP32[$pSubitem>>2]|0;
 $25 = ((($24)) + 44|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$iParent>>2] = $26;
 $27 = HEAP32[$pSubitem>>2]|0;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$pSub>>2] = $29;
 $30 = HEAP32[$5>>2]|0;
 $31 = ($30|0)!=(0);
 do {
  if ($31) {
   $32 = HEAP32[$4>>2]|0;
   $33 = ($32|0)!=(0);
   if ($33) {
    HEAP32[$0>>2] = 0;
    $572 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($572|0);
   }
   $34 = HEAP32[$pSrc>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)>(1);
   if ($36) {
    HEAP32[$0>>2] = 0;
    $572 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($572|0);
   }
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 36|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)!=(0|0);
   if ($40) {
    $41 = HEAP32[$2>>2]|0;
    $42 = ((($41)) + 36|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ((($43)) + 4|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = $45 & 2097152;
    $47 = ($46|0)!=(0);
    if (!($47)) {
     label = 10;
    }
   } else {
    label = 10;
   }
   if ((label|0) == 10) {
    $48 = HEAP32[$2>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    $50 = (_sqlite3ExprListFlags($49)|0);
    $51 = $50 & 2097152;
    $52 = ($51|0)!=(0);
    if (!($52)) {
     $53 = HEAP32[$2>>2]|0;
     $54 = ((($53)) + 48|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (_sqlite3ExprListFlags($55)|0);
     $57 = $56 & 2097152;
     $58 = ($57|0)!=(0);
     if (!($58)) {
      break;
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 } while(0);
 $59 = HEAP32[$pSub>>2]|0;
 $60 = ((($59)) + 32|0);
 $61 = HEAP32[$60>>2]|0;
 HEAP32[$pSubSrc>>2] = $61;
 $62 = HEAP32[$pSub>>2]|0;
 $63 = ((($62)) + 60|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = ($64|0)!=(0|0);
 if ($65) {
  $66 = HEAP32[$2>>2]|0;
  $67 = ((($66)) + 60|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = ($68|0)!=(0|0);
  if ($69) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $70 = HEAP32[$pSub>>2]|0;
 $71 = ((($70)) + 64|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)!=(0|0);
 if ($73) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $74 = HEAP32[$2>>2]|0;
 $75 = ((($74)) + 6|0);
 $76 = HEAP16[$75>>1]|0;
 $77 = $76&65535;
 $78 = $77 & 128;
 $79 = ($78|0)!=(0);
 if ($79) {
  $80 = HEAP32[$pSub>>2]|0;
  $81 = ((($80)) + 60|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($82|0)!=(0|0);
  if ($83) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $84 = HEAP32[$pSubSrc>>2]|0;
 $85 = HEAP32[$84>>2]|0;
 $86 = ($85|0)==(0);
 if ($86) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $87 = HEAP32[$pSub>>2]|0;
 $88 = ((($87)) + 6|0);
 $89 = HEAP16[$88>>1]|0;
 $90 = $89&65535;
 $91 = $90 & 1;
 $92 = ($91|0)!=(0);
 if ($92) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $93 = HEAP32[$pSub>>2]|0;
 $94 = ((($93)) + 60|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = ($95|0)!=(0|0);
 if ($96) {
  $97 = HEAP32[$pSrc>>2]|0;
  $98 = HEAP32[$97>>2]|0;
  $99 = ($98|0)>(1);
  $100 = HEAP32[$4>>2]|0;
  $101 = ($100|0)!=(0);
  $or$cond = $99 | $101;
  if ($or$cond) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $102 = HEAP32[$2>>2]|0;
 $103 = ((($102)) + 6|0);
 $104 = HEAP16[$103>>1]|0;
 $105 = $104&65535;
 $106 = $105 & 1;
 $107 = ($106|0)!=(0);
 $108 = HEAP32[$5>>2]|0;
 $109 = ($108|0)!=(0);
 $or$cond3 = $107 & $109;
 if ($or$cond3) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $110 = HEAP32[$2>>2]|0;
 $111 = ((($110)) + 48|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)!=(0|0);
 if ($113) {
  $114 = HEAP32[$pSub>>2]|0;
  $115 = ((($114)) + 48|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = ($116|0)!=(0|0);
  if ($117) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $118 = HEAP32[$4>>2]|0;
 $119 = ($118|0)!=(0);
 if ($119) {
  $120 = HEAP32[$pSub>>2]|0;
  $121 = ((($120)) + 48|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = ($122|0)!=(0|0);
  if ($123) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $124 = HEAP32[$pSub>>2]|0;
 $125 = ((($124)) + 60|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = ($126|0)!=(0|0);
 if ($127) {
  $128 = HEAP32[$2>>2]|0;
  $129 = ((($128)) + 36|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = ($130|0)!=(0|0);
  if ($131) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $132 = HEAP32[$pSub>>2]|0;
 $133 = ((($132)) + 60|0);
 $134 = HEAP32[$133>>2]|0;
 $135 = ($134|0)!=(0|0);
 if ($135) {
  $136 = HEAP32[$2>>2]|0;
  $137 = ((($136)) + 6|0);
  $138 = HEAP16[$137>>1]|0;
  $139 = $138&65535;
  $140 = $139 & 1;
  $141 = ($140|0)!=(0);
  if ($141) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $142 = HEAP32[$pSub>>2]|0;
 $143 = ((($142)) + 6|0);
 $144 = HEAP16[$143>>1]|0;
 $145 = $144&65535;
 $146 = $145 & 12288;
 $147 = ($146|0)!=(0);
 if ($147) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $148 = HEAP32[$2>>2]|0;
 $149 = ((($148)) + 6|0);
 $150 = HEAP16[$149>>1]|0;
 $151 = $150&65535;
 $152 = $151 & 8192;
 $153 = ($152|0)!=(0);
 if ($153) {
  $154 = HEAP32[$pSub>>2]|0;
  $155 = ((($154)) + 52|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = ($156|0)!=(0|0);
  if ($157) {
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 }
 $158 = HEAP32[$pSubitem>>2]|0;
 $159 = ((($158)) + 36|0);
 $160 = HEAP8[$159>>0]|0;
 $161 = $160&255;
 $162 = $161 & 32;
 $163 = ($162|0)!=(0);
 if ($163) {
  HEAP32[$0>>2] = 0;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $164 = HEAP32[$pSub>>2]|0;
 $165 = ((($164)) + 52|0);
 $166 = HEAP32[$165>>2]|0;
 $167 = ($166|0)!=(0|0);
 L87: do {
  if ($167) {
   $168 = HEAP32[$pSub>>2]|0;
   $169 = ((($168)) + 48|0);
   $170 = HEAP32[$169>>2]|0;
   $171 = ($170|0)!=(0|0);
   if ($171) {
    HEAP32[$0>>2] = 0;
    $572 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($572|0);
   }
   $172 = HEAP32[$4>>2]|0;
   $173 = ($172|0)!=(0);
   do {
    if (!($173)) {
     $174 = HEAP32[$2>>2]|0;
     $175 = ((($174)) + 6|0);
     $176 = HEAP16[$175>>1]|0;
     $177 = $176&65535;
     $178 = $177 & 1;
     $179 = ($178|0)!=(0);
     if ($179) {
      break;
     }
     $180 = HEAP32[$pSrc>>2]|0;
     $181 = HEAP32[$180>>2]|0;
     $182 = ($181|0)!=(1);
     if ($182) {
      break;
     }
     $183 = HEAP32[$pSub>>2]|0;
     HEAP32[$pSub1>>2] = $183;
     while(1) {
      $184 = HEAP32[$pSub1>>2]|0;
      $185 = ($184|0)!=(0|0);
      if (!($185)) {
       break;
      }
      $186 = HEAP32[$pSub1>>2]|0;
      $187 = ((($186)) + 6|0);
      $188 = HEAP16[$187>>1]|0;
      $189 = $188&65535;
      $190 = $189 & 9;
      $191 = ($190|0)!=(0);
      if ($191) {
       label = 62;
       break;
      }
      $192 = HEAP32[$pSub1>>2]|0;
      $193 = ((($192)) + 52|0);
      $194 = HEAP32[$193>>2]|0;
      $195 = ($194|0)!=(0|0);
      if ($195) {
       $196 = HEAP32[$pSub1>>2]|0;
       $197 = ((($196)) + 4|0);
       $198 = HEAP8[$197>>0]|0;
       $199 = $198&255;
       $200 = ($199|0)!=(116);
       if ($200) {
        label = 62;
        break;
       }
      }
      $201 = HEAP32[$pSub1>>2]|0;
      $202 = ((($201)) + 32|0);
      $203 = HEAP32[$202>>2]|0;
      $204 = HEAP32[$203>>2]|0;
      $205 = ($204|0)<(1);
      if ($205) {
       label = 62;
       break;
      }
      $206 = HEAP32[$pSub1>>2]|0;
      $207 = ((($206)) + 52|0);
      $208 = HEAP32[$207>>2]|0;
      HEAP32[$pSub1>>2] = $208;
     }
     if ((label|0) == 62) {
      HEAP32[$0>>2] = 0;
      $572 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($572|0);
     }
     $209 = HEAP32[$2>>2]|0;
     $210 = ((($209)) + 48|0);
     $211 = HEAP32[$210>>2]|0;
     $212 = ($211|0)!=(0|0);
     if (!($212)) {
      break L87;
     }
     HEAP32[$ii>>2] = 0;
     while(1) {
      $213 = HEAP32[$ii>>2]|0;
      $214 = HEAP32[$2>>2]|0;
      $215 = ((($214)) + 48|0);
      $216 = HEAP32[$215>>2]|0;
      $217 = HEAP32[$216>>2]|0;
      $218 = ($213|0)<($217|0);
      if (!($218)) {
       break L87;
      }
      $219 = HEAP32[$ii>>2]|0;
      $220 = HEAP32[$2>>2]|0;
      $221 = ((($220)) + 48|0);
      $222 = HEAP32[$221>>2]|0;
      $223 = ((($222)) + 4|0);
      $224 = HEAP32[$223>>2]|0;
      $225 = (($224) + (($219*20)|0)|0);
      $226 = ((($225)) + 16|0);
      $227 = HEAP16[$226>>1]|0;
      $228 = $227&65535;
      $229 = ($228|0)==(0);
      if ($229) {
       break;
      }
      $230 = HEAP32[$ii>>2]|0;
      $231 = (($230) + 1)|0;
      HEAP32[$ii>>2] = $231;
     }
     HEAP32[$0>>2] = 0;
     $572 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($572|0);
    }
   } while(0);
   HEAP32[$0>>2] = 0;
   $572 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($572|0);
  }
 } while(0);
 $232 = HEAP32[$pSubitem>>2]|0;
 $233 = ((($232)) + 8|0);
 $234 = HEAP32[$233>>2]|0;
 $235 = HEAP32[$1>>2]|0;
 $236 = ((($235)) + 496|0);
 HEAP32[$236>>2] = $234;
 $237 = HEAP32[$1>>2]|0;
 (_sqlite3AuthCheck($237,21,0,0,0)|0);
 $238 = HEAP32[$zSavedAuthContext>>2]|0;
 $239 = HEAP32[$1>>2]|0;
 $240 = ((($239)) + 496|0);
 HEAP32[$240>>2] = $238;
 $241 = HEAP32[$pSub>>2]|0;
 $242 = ((($241)) + 52|0);
 $243 = HEAP32[$242>>2]|0;
 HEAP32[$pSub>>2] = $243;
 while(1) {
  $244 = HEAP32[$pSub>>2]|0;
  $245 = ($244|0)!=(0|0);
  if (!($245)) {
   break;
  }
  $246 = HEAP32[$2>>2]|0;
  $247 = ((($246)) + 48|0);
  $248 = HEAP32[$247>>2]|0;
  HEAP32[$pOrderBy>>2] = $248;
  $249 = HEAP32[$2>>2]|0;
  $250 = ((($249)) + 60|0);
  $251 = HEAP32[$250>>2]|0;
  HEAP32[$pLimit>>2] = $251;
  $252 = HEAP32[$2>>2]|0;
  $253 = ((($252)) + 64|0);
  $254 = HEAP32[$253>>2]|0;
  HEAP32[$pOffset>>2] = $254;
  $255 = HEAP32[$2>>2]|0;
  $256 = ((($255)) + 52|0);
  $257 = HEAP32[$256>>2]|0;
  HEAP32[$pPrior>>2] = $257;
  $258 = HEAP32[$2>>2]|0;
  $259 = ((($258)) + 48|0);
  HEAP32[$259>>2] = 0;
  $260 = HEAP32[$2>>2]|0;
  $261 = ((($260)) + 32|0);
  HEAP32[$261>>2] = 0;
  $262 = HEAP32[$2>>2]|0;
  $263 = ((($262)) + 52|0);
  HEAP32[$263>>2] = 0;
  $264 = HEAP32[$2>>2]|0;
  $265 = ((($264)) + 60|0);
  HEAP32[$265>>2] = 0;
  $266 = HEAP32[$2>>2]|0;
  $267 = ((($266)) + 64|0);
  HEAP32[$267>>2] = 0;
  $268 = HEAP32[$db>>2]|0;
  $269 = HEAP32[$2>>2]|0;
  $270 = (_sqlite3SelectDup($268,$269,0)|0);
  HEAP32[$pNew>>2] = $270;
  $271 = HEAP32[$pOffset>>2]|0;
  $272 = HEAP32[$2>>2]|0;
  $273 = ((($272)) + 64|0);
  HEAP32[$273>>2] = $271;
  $274 = HEAP32[$pLimit>>2]|0;
  $275 = HEAP32[$2>>2]|0;
  $276 = ((($275)) + 60|0);
  HEAP32[$276>>2] = $274;
  $277 = HEAP32[$pOrderBy>>2]|0;
  $278 = HEAP32[$2>>2]|0;
  $279 = ((($278)) + 48|0);
  HEAP32[$279>>2] = $277;
  $280 = HEAP32[$pSrc>>2]|0;
  $281 = HEAP32[$2>>2]|0;
  $282 = ((($281)) + 32|0);
  HEAP32[$282>>2] = $280;
  $283 = HEAP32[$2>>2]|0;
  $284 = ((($283)) + 4|0);
  HEAP8[$284>>0] = 116;
  $285 = HEAP32[$pNew>>2]|0;
  $286 = ($285|0)==(0|0);
  $287 = HEAP32[$pPrior>>2]|0;
  if ($286) {
   $288 = HEAP32[$2>>2]|0;
   $289 = ((($288)) + 52|0);
   HEAP32[$289>>2] = $287;
  } else {
   $290 = HEAP32[$pNew>>2]|0;
   $291 = ((($290)) + 52|0);
   HEAP32[$291>>2] = $287;
   $292 = HEAP32[$pPrior>>2]|0;
   $293 = ($292|0)!=(0|0);
   if ($293) {
    $294 = HEAP32[$pNew>>2]|0;
    $295 = HEAP32[$pPrior>>2]|0;
    $296 = ((($295)) + 56|0);
    HEAP32[$296>>2] = $294;
   }
   $297 = HEAP32[$2>>2]|0;
   $298 = HEAP32[$pNew>>2]|0;
   $299 = ((($298)) + 56|0);
   HEAP32[$299>>2] = $297;
   $300 = HEAP32[$pNew>>2]|0;
   $301 = HEAP32[$2>>2]|0;
   $302 = ((($301)) + 52|0);
   HEAP32[$302>>2] = $300;
  }
  $303 = HEAP32[$db>>2]|0;
  $304 = ((($303)) + 65|0);
  $305 = HEAP8[$304>>0]|0;
  $306 = ($305<<24>>24)!=(0);
  if ($306) {
   label = 78;
   break;
  }
  $307 = HEAP32[$pSub>>2]|0;
  $308 = ((($307)) + 52|0);
  $309 = HEAP32[$308>>2]|0;
  HEAP32[$pSub>>2] = $309;
 }
 if ((label|0) == 78) {
  HEAP32[$0>>2] = 1;
  $572 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($572|0);
 }
 $310 = HEAP32[$pSubitem>>2]|0;
 $311 = ((($310)) + 20|0);
 $312 = HEAP32[$311>>2]|0;
 HEAP32[$pSub1>>2] = $312;
 HEAP32[$pSub>>2] = $312;
 $313 = HEAP32[$db>>2]|0;
 $314 = HEAP32[$pSubitem>>2]|0;
 $315 = ((($314)) + 4|0);
 $316 = HEAP32[$315>>2]|0;
 _sqlite3DbFree($313,$316);
 $317 = HEAP32[$db>>2]|0;
 $318 = HEAP32[$pSubitem>>2]|0;
 $319 = ((($318)) + 8|0);
 $320 = HEAP32[$319>>2]|0;
 _sqlite3DbFree($317,$320);
 $321 = HEAP32[$db>>2]|0;
 $322 = HEAP32[$pSubitem>>2]|0;
 $323 = ((($322)) + 12|0);
 $324 = HEAP32[$323>>2]|0;
 _sqlite3DbFree($321,$324);
 $325 = HEAP32[$pSubitem>>2]|0;
 $326 = ((($325)) + 4|0);
 HEAP32[$326>>2] = 0;
 $327 = HEAP32[$pSubitem>>2]|0;
 $328 = ((($327)) + 8|0);
 HEAP32[$328>>2] = 0;
 $329 = HEAP32[$pSubitem>>2]|0;
 $330 = ((($329)) + 12|0);
 HEAP32[$330>>2] = 0;
 $331 = HEAP32[$pSubitem>>2]|0;
 $332 = ((($331)) + 20|0);
 HEAP32[$332>>2] = 0;
 $333 = HEAP32[$pSubitem>>2]|0;
 $334 = ((($333)) + 16|0);
 $335 = HEAP32[$334>>2]|0;
 $336 = ($335|0)!=(0|0);
 if ($336) {
  $337 = HEAP32[$pSubitem>>2]|0;
  $338 = ((($337)) + 16|0);
  $339 = HEAP32[$338>>2]|0;
  HEAP32[$pTabToDel>>2] = $339;
  $340 = HEAP32[$pTabToDel>>2]|0;
  $341 = ((($340)) + 36|0);
  $342 = HEAP16[$341>>1]|0;
  $343 = $342&65535;
  $344 = ($343|0)==(1);
  if ($344) {
   $345 = HEAP32[$1>>2]|0;
   $346 = ((($345)) + 416|0);
   $347 = HEAP32[$346>>2]|0;
   $348 = ($347|0)!=(0|0);
   $349 = HEAP32[$1>>2]|0;
   if ($348) {
    $350 = ((($349)) + 416|0);
    $351 = HEAP32[$350>>2]|0;
    $352 = $351;
   } else {
    $352 = $349;
   }
   HEAP32[$pToplevel>>2] = $352;
   $353 = HEAP32[$pToplevel>>2]|0;
   $354 = ((($353)) + 528|0);
   $355 = HEAP32[$354>>2]|0;
   $356 = HEAP32[$pTabToDel>>2]|0;
   $357 = ((($356)) + 68|0);
   HEAP32[$357>>2] = $355;
   $358 = HEAP32[$pTabToDel>>2]|0;
   $359 = HEAP32[$pToplevel>>2]|0;
   $360 = ((($359)) + 528|0);
   HEAP32[$360>>2] = $358;
  } else {
   $361 = HEAP32[$pTabToDel>>2]|0;
   $362 = ((($361)) + 36|0);
   $363 = HEAP16[$362>>1]|0;
   $364 = (($363) + -1)<<16>>16;
   HEAP16[$362>>1] = $364;
  }
  $365 = HEAP32[$pSubitem>>2]|0;
  $366 = ((($365)) + 16|0);
  HEAP32[$366>>2] = 0;
 }
 $367 = HEAP32[$2>>2]|0;
 HEAP32[$pParent>>2] = $367;
 while(1) {
  $368 = HEAP32[$pParent>>2]|0;
  $369 = ($368|0)!=(0|0);
  if (!($369)) {
   break;
  }
  HEAP8[$jointype>>0] = 0;
  $370 = HEAP32[$pSub>>2]|0;
  $371 = ((($370)) + 32|0);
  $372 = HEAP32[$371>>2]|0;
  HEAP32[$pSubSrc>>2] = $372;
  $373 = HEAP32[$pSubSrc>>2]|0;
  $374 = HEAP32[$373>>2]|0;
  HEAP32[$nSubSrc>>2] = $374;
  $375 = HEAP32[$pParent>>2]|0;
  $376 = ((($375)) + 32|0);
  $377 = HEAP32[$376>>2]|0;
  HEAP32[$pSrc>>2] = $377;
  $378 = HEAP32[$pSrc>>2]|0;
  $379 = ($378|0)!=(0|0);
  if ($379) {
   $380 = HEAP32[$pSubitem>>2]|0;
   $381 = ((($380)) + 36|0);
   $382 = HEAP8[$381>>0]|0;
   HEAP8[$jointype>>0] = $382;
  } else {
   $383 = HEAP32[$db>>2]|0;
   $384 = (_sqlite3SrcListAppend($383,0,0,0)|0);
   $385 = HEAP32[$pParent>>2]|0;
   $386 = ((($385)) + 32|0);
   HEAP32[$386>>2] = $384;
   HEAP32[$pSrc>>2] = $384;
   $387 = HEAP32[$pSrc>>2]|0;
   $388 = ($387|0)==(0|0);
   if ($388) {
    break;
   }
  }
  $389 = HEAP32[$nSubSrc>>2]|0;
  $390 = ($389|0)>(1);
  if ($390) {
   $391 = HEAP32[$db>>2]|0;
   $392 = HEAP32[$pSrc>>2]|0;
   $393 = HEAP32[$nSubSrc>>2]|0;
   $394 = (($393) - 1)|0;
   $395 = HEAP32[$3>>2]|0;
   $396 = (($395) + 1)|0;
   $397 = (_sqlite3SrcListEnlarge($391,$392,$394,$396)|0);
   HEAP32[$pSrc>>2] = $397;
   $398 = HEAP32[$pParent>>2]|0;
   $399 = ((($398)) + 32|0);
   HEAP32[$399>>2] = $397;
   $400 = HEAP32[$db>>2]|0;
   $401 = ((($400)) + 65|0);
   $402 = HEAP8[$401>>0]|0;
   $403 = ($402<<24>>24)!=(0);
   if ($403) {
    break;
   }
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $404 = HEAP32[$i>>2]|0;
   $405 = HEAP32[$nSubSrc>>2]|0;
   $406 = ($404|0)<($405|0);
   if (!($406)) {
    break;
   }
   $407 = HEAP32[$db>>2]|0;
   $408 = HEAP32[$i>>2]|0;
   $409 = HEAP32[$3>>2]|0;
   $410 = (($408) + ($409))|0;
   $411 = HEAP32[$pSrc>>2]|0;
   $412 = ((($411)) + 8|0);
   $413 = (($412) + (($410*72)|0)|0);
   $414 = ((($413)) + 52|0);
   $415 = HEAP32[$414>>2]|0;
   _sqlite3IdListDelete($407,$415);
   $416 = HEAP32[$i>>2]|0;
   $417 = HEAP32[$3>>2]|0;
   $418 = (($416) + ($417))|0;
   $419 = HEAP32[$pSrc>>2]|0;
   $420 = ((($419)) + 8|0);
   $421 = (($420) + (($418*72)|0)|0);
   $422 = HEAP32[$i>>2]|0;
   $423 = HEAP32[$pSubSrc>>2]|0;
   $424 = ((($423)) + 8|0);
   $425 = (($424) + (($422*72)|0)|0);
   dest=$421; src=$425; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   $426 = HEAP32[$i>>2]|0;
   $427 = HEAP32[$pSubSrc>>2]|0;
   $428 = ((($427)) + 8|0);
   $429 = (($428) + (($426*72)|0)|0);
   dest=$429; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $430 = HEAP32[$i>>2]|0;
   $431 = (($430) + 1)|0;
   HEAP32[$i>>2] = $431;
  }
  $432 = HEAP8[$jointype>>0]|0;
  $433 = HEAP32[$3>>2]|0;
  $434 = HEAP32[$pSrc>>2]|0;
  $435 = ((($434)) + 8|0);
  $436 = (($435) + (($433*72)|0)|0);
  $437 = ((($436)) + 36|0);
  HEAP8[$437>>0] = $432;
  $438 = HEAP32[$pParent>>2]|0;
  $439 = HEAP32[$438>>2]|0;
  HEAP32[$pList>>2] = $439;
  HEAP32[$i>>2] = 0;
  while(1) {
   $440 = HEAP32[$i>>2]|0;
   $441 = HEAP32[$pList>>2]|0;
   $442 = HEAP32[$441>>2]|0;
   $443 = ($440|0)<($442|0);
   if (!($443)) {
    break;
   }
   $444 = HEAP32[$i>>2]|0;
   $445 = HEAP32[$pList>>2]|0;
   $446 = ((($445)) + 4|0);
   $447 = HEAP32[$446>>2]|0;
   $448 = (($447) + (($444*20)|0)|0);
   $449 = ((($448)) + 4|0);
   $450 = HEAP32[$449>>2]|0;
   $451 = ($450|0)==(0|0);
   if ($451) {
    $452 = HEAP32[$db>>2]|0;
    $453 = HEAP32[$i>>2]|0;
    $454 = HEAP32[$pList>>2]|0;
    $455 = ((($454)) + 4|0);
    $456 = HEAP32[$455>>2]|0;
    $457 = (($456) + (($453*20)|0)|0);
    $458 = ((($457)) + 8|0);
    $459 = HEAP32[$458>>2]|0;
    $460 = (_sqlite3DbStrDup($452,$459)|0);
    HEAP32[$zName>>2] = $460;
    $461 = HEAP32[$zName>>2]|0;
    (_sqlite3Dequote($461)|0);
    $462 = HEAP32[$zName>>2]|0;
    $463 = HEAP32[$i>>2]|0;
    $464 = HEAP32[$pList>>2]|0;
    $465 = ((($464)) + 4|0);
    $466 = HEAP32[$465>>2]|0;
    $467 = (($466) + (($463*20)|0)|0);
    $468 = ((($467)) + 4|0);
    HEAP32[$468>>2] = $462;
   }
   $469 = HEAP32[$i>>2]|0;
   $470 = (($469) + 1)|0;
   HEAP32[$i>>2] = $470;
  }
  $471 = HEAP32[$pSub>>2]|0;
  $472 = ((($471)) + 48|0);
  $473 = HEAP32[$472>>2]|0;
  $474 = ($473|0)!=(0|0);
  if ($474) {
   $475 = HEAP32[$pSub>>2]|0;
   $476 = ((($475)) + 48|0);
   $477 = HEAP32[$476>>2]|0;
   HEAP32[$pOrderBy1>>2] = $477;
   HEAP32[$i>>2] = 0;
   while(1) {
    $478 = HEAP32[$i>>2]|0;
    $479 = HEAP32[$pOrderBy1>>2]|0;
    $480 = HEAP32[$479>>2]|0;
    $481 = ($478|0)<($480|0);
    if (!($481)) {
     break;
    }
    $482 = HEAP32[$i>>2]|0;
    $483 = HEAP32[$pOrderBy1>>2]|0;
    $484 = ((($483)) + 4|0);
    $485 = HEAP32[$484>>2]|0;
    $486 = (($485) + (($482*20)|0)|0);
    $487 = ((($486)) + 16|0);
    HEAP16[$487>>1] = 0;
    $488 = HEAP32[$i>>2]|0;
    $489 = (($488) + 1)|0;
    HEAP32[$i>>2] = $489;
   }
   $490 = HEAP32[$pOrderBy1>>2]|0;
   $491 = HEAP32[$pParent>>2]|0;
   $492 = ((($491)) + 48|0);
   HEAP32[$492>>2] = $490;
   $493 = HEAP32[$pSub>>2]|0;
   $494 = ((($493)) + 48|0);
   HEAP32[$494>>2] = 0;
  }
  $495 = HEAP32[$db>>2]|0;
  $496 = HEAP32[$pSub>>2]|0;
  $497 = ((($496)) + 36|0);
  $498 = HEAP32[$497>>2]|0;
  $499 = (_sqlite3ExprDup($495,$498,0)|0);
  HEAP32[$pWhere>>2] = $499;
  $500 = HEAP32[$5>>2]|0;
  $501 = ($500|0)!=(0);
  if ($501) {
   $502 = HEAP32[$pParent>>2]|0;
   $503 = ((($502)) + 36|0);
   $504 = HEAP32[$503>>2]|0;
   $505 = HEAP32[$pParent>>2]|0;
   $506 = ((($505)) + 44|0);
   HEAP32[$506>>2] = $504;
   $507 = HEAP32[$pWhere>>2]|0;
   $508 = HEAP32[$pParent>>2]|0;
   $509 = ((($508)) + 36|0);
   HEAP32[$509>>2] = $507;
   $510 = HEAP32[$db>>2]|0;
   $511 = HEAP32[$pParent>>2]|0;
   $512 = ((($511)) + 44|0);
   $513 = HEAP32[$512>>2]|0;
   $514 = HEAP32[$db>>2]|0;
   $515 = HEAP32[$pSub>>2]|0;
   $516 = ((($515)) + 44|0);
   $517 = HEAP32[$516>>2]|0;
   $518 = (_sqlite3ExprDup($514,$517,0)|0);
   $519 = (_sqlite3ExprAnd($510,$513,$518)|0);
   $520 = HEAP32[$pParent>>2]|0;
   $521 = ((($520)) + 44|0);
   HEAP32[$521>>2] = $519;
   $522 = HEAP32[$db>>2]|0;
   $523 = HEAP32[$pSub>>2]|0;
   $524 = ((($523)) + 40|0);
   $525 = HEAP32[$524>>2]|0;
   $526 = (_sqlite3ExprListDup($522,$525,0)|0);
   $527 = HEAP32[$pParent>>2]|0;
   $528 = ((($527)) + 40|0);
   HEAP32[$528>>2] = $526;
  } else {
   $529 = HEAP32[$db>>2]|0;
   $530 = HEAP32[$pParent>>2]|0;
   $531 = ((($530)) + 36|0);
   $532 = HEAP32[$531>>2]|0;
   $533 = HEAP32[$pWhere>>2]|0;
   $534 = (_sqlite3ExprAnd($529,$532,$533)|0);
   $535 = HEAP32[$pParent>>2]|0;
   $536 = ((($535)) + 36|0);
   HEAP32[$536>>2] = $534;
  }
  $537 = HEAP32[$db>>2]|0;
  $538 = HEAP32[$pParent>>2]|0;
  $539 = HEAP32[$iParent>>2]|0;
  $540 = HEAP32[$pSub>>2]|0;
  $541 = HEAP32[$540>>2]|0;
  _substSelect($537,$538,$539,$541,0);
  $542 = HEAP32[$pSub>>2]|0;
  $543 = ((($542)) + 6|0);
  $544 = HEAP16[$543>>1]|0;
  $545 = $544&65535;
  $546 = $545 & 1;
  $547 = HEAP32[$pParent>>2]|0;
  $548 = ((($547)) + 6|0);
  $549 = HEAP16[$548>>1]|0;
  $550 = $549&65535;
  $551 = $550 | $546;
  $552 = $551&65535;
  HEAP16[$548>>1] = $552;
  $553 = HEAP32[$pSub>>2]|0;
  $554 = ((($553)) + 60|0);
  $555 = HEAP32[$554>>2]|0;
  $556 = ($555|0)!=(0|0);
  if ($556) {
   $557 = HEAP32[$pSub>>2]|0;
   $558 = ((($557)) + 60|0);
   $559 = HEAP32[$558>>2]|0;
   $560 = HEAP32[$pParent>>2]|0;
   $561 = ((($560)) + 60|0);
   HEAP32[$561>>2] = $559;
   $562 = HEAP32[$pSub>>2]|0;
   $563 = ((($562)) + 60|0);
   HEAP32[$563>>2] = 0;
  }
  $564 = HEAP32[$pParent>>2]|0;
  $565 = ((($564)) + 52|0);
  $566 = HEAP32[$565>>2]|0;
  HEAP32[$pParent>>2] = $566;
  $567 = HEAP32[$pSub>>2]|0;
  $568 = ((($567)) + 52|0);
  $569 = HEAP32[$568>>2]|0;
  HEAP32[$pSub>>2] = $569;
 }
 $570 = HEAP32[$db>>2]|0;
 $571 = HEAP32[$pSub1>>2]|0;
 _sqlite3SelectDelete($570,$571);
 HEAP32[$0>>2] = 1;
 $572 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($572|0);
}
function _multiSelect($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addr1 = 0, $addr7 = 0, $addr9 = 0, $apColl = 0, $db = 0, $dest = 0, $i = 0, $iBreak = 0, $iBreak3 = 0, $iCont = 0, $iCont2 = 0, $iStart = 0, $iStart4 = 0, $iSub1 = 0;
 var $iSub2 = 0, $intersectdest = 0, $nCol = 0, $nLimit = 0, $op = 0, $or$cond = 0, $pDelete = 0, $pFirst = 0, $pFirst8 = 0, $pKeyInfo = 0, $pLimit = 0, $pLimit5 = 0, $pLoop = 0, $pOffset = 0, $pOffset6 = 0, $pPrior = 0, $priorOp = 0, $r1 = 0, $rc = 0, $tab1 = 0;
 var $tab2 = 0, $unionTab = 0, $uniondest = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 220|0;
 $1 = sp + 216|0;
 $2 = sp + 212|0;
 $3 = sp + 208|0;
 $rc = sp + 204|0;
 $pPrior = sp + 200|0;
 $v = sp + 196|0;
 $dest = sp + 176|0;
 $pDelete = sp + 172|0;
 $db = sp + 168|0;
 $iSub1 = sp + 164|0;
 $iSub2 = sp + 160|0;
 $addr = sp + 156|0;
 $nLimit = sp + 152|0;
 $unionTab = sp + 148|0;
 $op = sp + 224|0;
 $priorOp = sp + 144|0;
 $pLimit = sp + 140|0;
 $pOffset = sp + 136|0;
 $addr1 = sp + 132|0;
 $uniondest = sp + 112|0;
 $iCont = sp + 108|0;
 $iBreak = sp + 104|0;
 $iStart = sp + 100|0;
 $pFirst = sp + 96|0;
 $tab1 = sp + 92|0;
 $tab2 = sp + 88|0;
 $iCont2 = sp + 84|0;
 $iBreak3 = sp + 80|0;
 $iStart4 = sp + 76|0;
 $pLimit5 = sp + 72|0;
 $pOffset6 = sp + 68|0;
 $addr7 = sp + 64|0;
 $intersectdest = sp + 44|0;
 $r1 = sp + 40|0;
 $pFirst8 = sp + 36|0;
 $i = sp + 32|0;
 $pKeyInfo = sp + 28|0;
 $pLoop = sp + 24|0;
 $apColl = sp + 20|0;
 $nCol = sp + 16|0;
 $addr9 = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDelete>>2] = 0;
 HEAP32[$iSub1>>2] = 0;
 HEAP32[$iSub2>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPrior>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 ;HEAP32[$dest>>2]=HEAP32[$9>>2]|0;HEAP32[$dest+4>>2]=HEAP32[$9+4>>2]|0;HEAP32[$dest+8>>2]=HEAP32[$9+8>>2]|0;HEAP32[$dest+12>>2]=HEAP32[$9+12>>2]|0;HEAP32[$dest+16>>2]=HEAP32[$9+16>>2]|0;
 $10 = HEAP32[$pPrior>>2]|0;
 $11 = ((($10)) + 48|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 L1: do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17&255;
   $19 = (_selectOpName($18)|0);
   HEAP32[$vararg_buffer>>2] = $19;
   _sqlite3ErrorMsg($14,32301,$vararg_buffer);
   HEAP32[$rc>>2] = 1;
  } else {
   $20 = HEAP32[$pPrior>>2]|0;
   $21 = ((($20)) + 60|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)!=(0|0);
   $24 = HEAP32[$1>>2]|0;
   if ($23) {
    $25 = HEAP32[$2>>2]|0;
    $26 = ((($25)) + 4|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = (_selectOpName($28)|0);
    HEAP32[$vararg_buffer2>>2] = $29;
    _sqlite3ErrorMsg($24,32349,$vararg_buffer2);
    HEAP32[$rc>>2] = 1;
    break;
   }
   $30 = (_sqlite3GetVdbe($24)|0);
   HEAP32[$v>>2] = $30;
   $31 = HEAP8[$dest>>0]|0;
   $32 = $31&255;
   $33 = ($32|0)==(12);
   if ($33) {
    $34 = HEAP32[$v>>2]|0;
    $35 = ((($dest)) + 4|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    (_sqlite3VdbeAddOp2($34,57,$36,$39)|0);
    $40 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($40,8);
    HEAP8[$dest>>0] = 14;
   }
   $41 = HEAP32[$2>>2]|0;
   $42 = ((($41)) + 6|0);
   $43 = HEAP16[$42>>1]|0;
   $44 = $43&65535;
   $45 = $44 & 512;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = HEAP32[$1>>2]|0;
    $48 = HEAP32[$2>>2]|0;
    $49 = (_multiSelectValues($47,$48,$dest)|0);
    HEAP32[$rc>>2] = $49;
    break;
   }
   $50 = HEAP32[$2>>2]|0;
   $51 = ((($50)) + 6|0);
   $52 = HEAP16[$51>>1]|0;
   $53 = $52&65535;
   $54 = $53 & 8192;
   $55 = ($54|0)!=(0);
   L13: do {
    if ($55) {
     $56 = HEAP32[$1>>2]|0;
     $57 = HEAP32[$2>>2]|0;
     _generateWithRecursiveQuery($56,$57,$dest);
    } else {
     $58 = HEAP32[$2>>2]|0;
     $59 = ((($58)) + 48|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($60|0)!=(0|0);
     if ($61) {
      $62 = HEAP32[$1>>2]|0;
      $63 = HEAP32[$2>>2]|0;
      $64 = HEAP32[$3>>2]|0;
      $65 = (_multiSelectOrderBy($62,$63,$64)|0);
      HEAP32[$0>>2] = $65;
      $624 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($624|0);
     }
     $66 = HEAP32[$2>>2]|0;
     $67 = ((($66)) + 4|0);
     $68 = HEAP8[$67>>0]|0;
     $69 = $68&255;
     switch ($69|0) {
     case 116:  {
      HEAP32[$addr>>2] = 0;
      $70 = HEAP32[$2>>2]|0;
      $71 = ((($70)) + 8|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = HEAP32[$pPrior>>2]|0;
      $74 = ((($73)) + 8|0);
      HEAP32[$74>>2] = $72;
      $75 = HEAP32[$2>>2]|0;
      $76 = ((($75)) + 12|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = HEAP32[$pPrior>>2]|0;
      $79 = ((($78)) + 12|0);
      HEAP32[$79>>2] = $77;
      $80 = HEAP32[$2>>2]|0;
      $81 = ((($80)) + 60|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = HEAP32[$pPrior>>2]|0;
      $84 = ((($83)) + 60|0);
      HEAP32[$84>>2] = $82;
      $85 = HEAP32[$2>>2]|0;
      $86 = ((($85)) + 64|0);
      $87 = HEAP32[$86>>2]|0;
      $88 = HEAP32[$pPrior>>2]|0;
      $89 = ((($88)) + 64|0);
      HEAP32[$89>>2] = $87;
      $90 = HEAP32[$1>>2]|0;
      $91 = ((($90)) + 472|0);
      $92 = HEAP32[$91>>2]|0;
      HEAP32[$iSub1>>2] = $92;
      $93 = HEAP32[$1>>2]|0;
      $94 = HEAP32[$pPrior>>2]|0;
      $95 = (_sqlite3Select($93,$94,$dest)|0);
      HEAP32[$rc>>2] = $95;
      $96 = HEAP32[$2>>2]|0;
      $97 = ((($96)) + 60|0);
      HEAP32[$97>>2] = 0;
      $98 = HEAP32[$2>>2]|0;
      $99 = ((($98)) + 64|0);
      HEAP32[$99>>2] = 0;
      $100 = HEAP32[$rc>>2]|0;
      $101 = ($100|0)!=(0);
      if ($101) {
       break L1;
      }
      $102 = HEAP32[$2>>2]|0;
      $103 = ((($102)) + 52|0);
      HEAP32[$103>>2] = 0;
      $104 = HEAP32[$pPrior>>2]|0;
      $105 = ((($104)) + 8|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = HEAP32[$2>>2]|0;
      $108 = ((($107)) + 8|0);
      HEAP32[$108>>2] = $106;
      $109 = HEAP32[$pPrior>>2]|0;
      $110 = ((($109)) + 12|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = HEAP32[$2>>2]|0;
      $113 = ((($112)) + 12|0);
      HEAP32[$113>>2] = $111;
      $114 = HEAP32[$2>>2]|0;
      $115 = ((($114)) + 8|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($116|0)!=(0);
      if ($117) {
       $118 = HEAP32[$v>>2]|0;
       $119 = HEAP32[$2>>2]|0;
       $120 = ((($119)) + 8|0);
       $121 = HEAP32[$120>>2]|0;
       $122 = (_sqlite3VdbeAddOp1($118,46,$121)|0);
       HEAP32[$addr>>2] = $122;
       $123 = HEAP32[$2>>2]|0;
       $124 = ((($123)) + 12|0);
       $125 = HEAP32[$124>>2]|0;
       $126 = ($125|0)!=(0);
       if ($126) {
        $127 = HEAP32[$v>>2]|0;
        $128 = HEAP32[$2>>2]|0;
        $129 = ((($128)) + 8|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = HEAP32[$2>>2]|0;
        $132 = ((($131)) + 12|0);
        $133 = HEAP32[$132>>2]|0;
        $134 = (($133) + 1)|0;
        $135 = HEAP32[$2>>2]|0;
        $136 = ((($135)) + 12|0);
        $137 = HEAP32[$136>>2]|0;
        (_sqlite3VdbeAddOp3($127,139,$130,$134,$137)|0);
       }
      }
      $138 = HEAP32[$1>>2]|0;
      $139 = ((($138)) + 472|0);
      $140 = HEAP32[$139>>2]|0;
      HEAP32[$iSub2>>2] = $140;
      $141 = HEAP32[$1>>2]|0;
      $142 = HEAP32[$2>>2]|0;
      $143 = (_sqlite3Select($141,$142,$dest)|0);
      HEAP32[$rc>>2] = $143;
      $144 = HEAP32[$2>>2]|0;
      $145 = ((($144)) + 52|0);
      $146 = HEAP32[$145>>2]|0;
      HEAP32[$pDelete>>2] = $146;
      $147 = HEAP32[$pPrior>>2]|0;
      $148 = HEAP32[$2>>2]|0;
      $149 = ((($148)) + 52|0);
      HEAP32[$149>>2] = $147;
      $150 = HEAP32[$pPrior>>2]|0;
      $151 = ((($150)) + 24|0);
      $152 = $151;
      $153 = $152;
      $154 = HEAP32[$153>>2]|0;
      $155 = (($152) + 4)|0;
      $156 = $155;
      $157 = HEAP32[$156>>2]|0;
      $158 = HEAP32[$2>>2]|0;
      $159 = ((($158)) + 24|0);
      $160 = $159;
      $161 = $160;
      $162 = HEAP32[$161>>2]|0;
      $163 = (($160) + 4)|0;
      $164 = $163;
      $165 = HEAP32[$164>>2]|0;
      $166 = (_i64Add(($162|0),($165|0),($154|0),($157|0))|0);
      $167 = tempRet0;
      $168 = $159;
      $169 = $168;
      HEAP32[$169>>2] = $166;
      $170 = (($168) + 4)|0;
      $171 = $170;
      HEAP32[$171>>2] = $167;
      $172 = HEAP32[$pPrior>>2]|0;
      $173 = ((($172)) + 60|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = ($174|0)!=(0|0);
      if ($175) {
       $176 = HEAP32[$pPrior>>2]|0;
       $177 = ((($176)) + 60|0);
       $178 = HEAP32[$177>>2]|0;
       $179 = (_sqlite3ExprIsInteger($178,$nLimit)|0);
       $180 = ($179|0)!=(0);
       $181 = HEAP32[$nLimit>>2]|0;
       $182 = ($181|0)>(0);
       $or$cond = $180 & $182;
       if ($or$cond) {
        $183 = HEAP32[$2>>2]|0;
        $184 = ((($183)) + 24|0);
        $185 = $184;
        $186 = $185;
        $187 = HEAP32[$186>>2]|0;
        $188 = (($185) + 4)|0;
        $189 = $188;
        $190 = HEAP32[$189>>2]|0;
        $191 = HEAP32[$nLimit>>2]|0;
        $192 = ($191|0)<(0);
        $193 = $192 << 31 >> 31;
        $194 = ($190>>>0)>($193>>>0);
        $195 = ($187>>>0)>($191>>>0);
        $196 = ($190|0)==($193|0);
        $197 = $196 & $195;
        $198 = $194 | $197;
        if ($198) {
         $199 = HEAP32[$nLimit>>2]|0;
         $200 = ($199|0)<(0);
         $201 = $200 << 31 >> 31;
         $202 = HEAP32[$2>>2]|0;
         $203 = ((($202)) + 24|0);
         $204 = $203;
         $205 = $204;
         HEAP32[$205>>2] = $199;
         $206 = (($204) + 4)|0;
         $207 = $206;
         HEAP32[$207>>2] = $201;
        }
       }
      }
      $208 = HEAP32[$addr>>2]|0;
      $209 = ($208|0)!=(0);
      if (!($209)) {
       break L13;
      }
      $210 = HEAP32[$v>>2]|0;
      $211 = HEAP32[$addr>>2]|0;
      _sqlite3VdbeJumpHere($210,$211);
      break L13;
      break;
     }
     case 115: case 117:  {
      HEAP8[$op>>0] = 0;
      HEAP32[$priorOp>>2] = 1;
      $212 = HEAP8[$dest>>0]|0;
      $213 = $212&255;
      $214 = HEAP32[$priorOp>>2]|0;
      $215 = ($213|0)==($214|0);
      if ($215) {
       $216 = ((($dest)) + 4|0);
       $217 = HEAP32[$216>>2]|0;
       HEAP32[$unionTab>>2] = $217;
      } else {
       $218 = HEAP32[$1>>2]|0;
       $219 = ((($218)) + 72|0);
       $220 = HEAP32[$219>>2]|0;
       $221 = (($220) + 1)|0;
       HEAP32[$219>>2] = $221;
       HEAP32[$unionTab>>2] = $220;
       $222 = HEAP32[$v>>2]|0;
       $223 = HEAP32[$unionTab>>2]|0;
       $224 = (_sqlite3VdbeAddOp2($222,57,$223,0)|0);
       HEAP32[$addr1>>2] = $224;
       $225 = HEAP32[$addr1>>2]|0;
       $226 = HEAP32[$2>>2]|0;
       $227 = ((($226)) + 16|0);
       HEAP32[$227>>2] = $225;
       $228 = HEAP32[$2>>2]|0;
       $229 = (_findRightmost($228)|0);
       $230 = ((($229)) + 6|0);
       $231 = HEAP16[$230>>1]|0;
       $232 = $231&65535;
       $233 = $232 | 16;
       $234 = $233&65535;
       HEAP16[$230>>1] = $234;
      }
      $235 = HEAP32[$priorOp>>2]|0;
      $236 = HEAP32[$unionTab>>2]|0;
      _sqlite3SelectDestInit($uniondest,$235,$236);
      $237 = HEAP32[$1>>2]|0;
      $238 = ((($237)) + 472|0);
      $239 = HEAP32[$238>>2]|0;
      HEAP32[$iSub1>>2] = $239;
      $240 = HEAP32[$1>>2]|0;
      $241 = HEAP32[$pPrior>>2]|0;
      $242 = (_sqlite3Select($240,$241,$uniondest)|0);
      HEAP32[$rc>>2] = $242;
      $243 = HEAP32[$rc>>2]|0;
      $244 = ($243|0)!=(0);
      if ($244) {
       break L1;
      }
      $245 = HEAP32[$2>>2]|0;
      $246 = ((($245)) + 4|0);
      $247 = HEAP8[$246>>0]|0;
      $248 = $247&255;
      $249 = ($248|0)==(117);
      if ($249) {
       HEAP8[$op>>0] = 2;
      } else {
       HEAP8[$op>>0] = 1;
      }
      $250 = HEAP32[$2>>2]|0;
      $251 = ((($250)) + 52|0);
      HEAP32[$251>>2] = 0;
      $252 = HEAP32[$2>>2]|0;
      $253 = ((($252)) + 60|0);
      $254 = HEAP32[$253>>2]|0;
      HEAP32[$pLimit>>2] = $254;
      $255 = HEAP32[$2>>2]|0;
      $256 = ((($255)) + 60|0);
      HEAP32[$256>>2] = 0;
      $257 = HEAP32[$2>>2]|0;
      $258 = ((($257)) + 64|0);
      $259 = HEAP32[$258>>2]|0;
      HEAP32[$pOffset>>2] = $259;
      $260 = HEAP32[$2>>2]|0;
      $261 = ((($260)) + 64|0);
      HEAP32[$261>>2] = 0;
      $262 = HEAP8[$op>>0]|0;
      HEAP8[$uniondest>>0] = $262;
      $263 = HEAP32[$1>>2]|0;
      $264 = ((($263)) + 472|0);
      $265 = HEAP32[$264>>2]|0;
      HEAP32[$iSub2>>2] = $265;
      $266 = HEAP32[$1>>2]|0;
      $267 = HEAP32[$2>>2]|0;
      $268 = (_sqlite3Select($266,$267,$uniondest)|0);
      HEAP32[$rc>>2] = $268;
      $269 = HEAP32[$db>>2]|0;
      $270 = HEAP32[$2>>2]|0;
      $271 = ((($270)) + 48|0);
      $272 = HEAP32[$271>>2]|0;
      _sqlite3ExprListDelete($269,$272);
      $273 = HEAP32[$2>>2]|0;
      $274 = ((($273)) + 52|0);
      $275 = HEAP32[$274>>2]|0;
      HEAP32[$pDelete>>2] = $275;
      $276 = HEAP32[$pPrior>>2]|0;
      $277 = HEAP32[$2>>2]|0;
      $278 = ((($277)) + 52|0);
      HEAP32[$278>>2] = $276;
      $279 = HEAP32[$2>>2]|0;
      $280 = ((($279)) + 48|0);
      HEAP32[$280>>2] = 0;
      $281 = HEAP32[$2>>2]|0;
      $282 = ((($281)) + 4|0);
      $283 = HEAP8[$282>>0]|0;
      $284 = $283&255;
      $285 = ($284|0)==(115);
      if ($285) {
       $286 = HEAP32[$pPrior>>2]|0;
       $287 = ((($286)) + 24|0);
       $288 = $287;
       $289 = $288;
       $290 = HEAP32[$289>>2]|0;
       $291 = (($288) + 4)|0;
       $292 = $291;
       $293 = HEAP32[$292>>2]|0;
       $294 = HEAP32[$2>>2]|0;
       $295 = ((($294)) + 24|0);
       $296 = $295;
       $297 = $296;
       $298 = HEAP32[$297>>2]|0;
       $299 = (($296) + 4)|0;
       $300 = $299;
       $301 = HEAP32[$300>>2]|0;
       $302 = (_i64Add(($298|0),($301|0),($290|0),($293|0))|0);
       $303 = tempRet0;
       $304 = $295;
       $305 = $304;
       HEAP32[$305>>2] = $302;
       $306 = (($304) + 4)|0;
       $307 = $306;
       HEAP32[$307>>2] = $303;
      }
      $308 = HEAP32[$db>>2]|0;
      $309 = HEAP32[$2>>2]|0;
      $310 = ((($309)) + 60|0);
      $311 = HEAP32[$310>>2]|0;
      _sqlite3ExprDelete($308,$311);
      $312 = HEAP32[$pLimit>>2]|0;
      $313 = HEAP32[$2>>2]|0;
      $314 = ((($313)) + 60|0);
      HEAP32[$314>>2] = $312;
      $315 = HEAP32[$pOffset>>2]|0;
      $316 = HEAP32[$2>>2]|0;
      $317 = ((($316)) + 64|0);
      HEAP32[$317>>2] = $315;
      $318 = HEAP32[$2>>2]|0;
      $319 = ((($318)) + 8|0);
      HEAP32[$319>>2] = 0;
      $320 = HEAP32[$2>>2]|0;
      $321 = ((($320)) + 12|0);
      HEAP32[$321>>2] = 0;
      $322 = HEAP8[$dest>>0]|0;
      $323 = $322&255;
      $324 = HEAP32[$priorOp>>2]|0;
      $325 = ($323|0)!=($324|0);
      if (!($325)) {
       break L13;
      }
      $326 = HEAP8[$dest>>0]|0;
      $327 = $326&255;
      $328 = ($327|0)==(9);
      if ($328) {
       $329 = HEAP32[$2>>2]|0;
       HEAP32[$pFirst>>2] = $329;
       while(1) {
        $330 = HEAP32[$pFirst>>2]|0;
        $331 = ((($330)) + 52|0);
        $332 = HEAP32[$331>>2]|0;
        $333 = ($332|0)!=(0|0);
        if (!($333)) {
         break;
        }
        $334 = HEAP32[$pFirst>>2]|0;
        $335 = ((($334)) + 52|0);
        $336 = HEAP32[$335>>2]|0;
        HEAP32[$pFirst>>2] = $336;
       }
       $337 = HEAP32[$1>>2]|0;
       $338 = HEAP32[$pFirst>>2]|0;
       $339 = ((($338)) + 32|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = HEAP32[$pFirst>>2]|0;
       $342 = HEAP32[$341>>2]|0;
       _generateColumnNames($337,$340,$342);
      }
      $343 = HEAP32[$v>>2]|0;
      $344 = (_sqlite3VdbeMakeLabel($343)|0);
      HEAP32[$iBreak>>2] = $344;
      $345 = HEAP32[$v>>2]|0;
      $346 = (_sqlite3VdbeMakeLabel($345)|0);
      HEAP32[$iCont>>2] = $346;
      $347 = HEAP32[$1>>2]|0;
      $348 = HEAP32[$2>>2]|0;
      $349 = HEAP32[$iBreak>>2]|0;
      _computeLimitRegisters($347,$348,$349);
      $350 = HEAP32[$v>>2]|0;
      $351 = HEAP32[$unionTab>>2]|0;
      $352 = HEAP32[$iBreak>>2]|0;
      (_sqlite3VdbeAddOp2($350,108,$351,$352)|0);
      $353 = HEAP32[$v>>2]|0;
      $354 = (_sqlite3VdbeCurrentAddr($353)|0);
      HEAP32[$iStart>>2] = $354;
      $355 = HEAP32[$1>>2]|0;
      $356 = HEAP32[$2>>2]|0;
      $357 = HEAP32[$2>>2]|0;
      $358 = HEAP32[$357>>2]|0;
      $359 = HEAP32[$unionTab>>2]|0;
      $360 = HEAP32[$iCont>>2]|0;
      $361 = HEAP32[$iBreak>>2]|0;
      _selectInnerLoop($355,$356,$358,$359,0,0,$dest,$360,$361);
      $362 = HEAP32[$v>>2]|0;
      $363 = HEAP32[$iCont>>2]|0;
      _sqlite3VdbeResolveLabel($362,$363);
      $364 = HEAP32[$v>>2]|0;
      $365 = HEAP32[$unionTab>>2]|0;
      $366 = HEAP32[$iStart>>2]|0;
      (_sqlite3VdbeAddOp2($364,7,$365,$366)|0);
      $367 = HEAP32[$v>>2]|0;
      $368 = HEAP32[$iBreak>>2]|0;
      _sqlite3VdbeResolveLabel($367,$368);
      $369 = HEAP32[$v>>2]|0;
      $370 = HEAP32[$unionTab>>2]|0;
      (_sqlite3VdbeAddOp2($369,61,$370,0)|0);
      break L13;
      break;
     }
     default: {
      $371 = HEAP32[$1>>2]|0;
      $372 = ((($371)) + 72|0);
      $373 = HEAP32[$372>>2]|0;
      $374 = (($373) + 1)|0;
      HEAP32[$372>>2] = $374;
      HEAP32[$tab1>>2] = $373;
      $375 = HEAP32[$1>>2]|0;
      $376 = ((($375)) + 72|0);
      $377 = HEAP32[$376>>2]|0;
      $378 = (($377) + 1)|0;
      HEAP32[$376>>2] = $378;
      HEAP32[$tab2>>2] = $377;
      $379 = HEAP32[$v>>2]|0;
      $380 = HEAP32[$tab1>>2]|0;
      $381 = (_sqlite3VdbeAddOp2($379,57,$380,0)|0);
      HEAP32[$addr7>>2] = $381;
      $382 = HEAP32[$addr7>>2]|0;
      $383 = HEAP32[$2>>2]|0;
      $384 = ((($383)) + 16|0);
      HEAP32[$384>>2] = $382;
      $385 = HEAP32[$2>>2]|0;
      $386 = (_findRightmost($385)|0);
      $387 = ((($386)) + 6|0);
      $388 = HEAP16[$387>>1]|0;
      $389 = $388&65535;
      $390 = $389 | 16;
      $391 = $390&65535;
      HEAP16[$387>>1] = $391;
      $392 = HEAP32[$tab1>>2]|0;
      _sqlite3SelectDestInit($intersectdest,1,$392);
      $393 = HEAP32[$1>>2]|0;
      $394 = ((($393)) + 472|0);
      $395 = HEAP32[$394>>2]|0;
      HEAP32[$iSub1>>2] = $395;
      $396 = HEAP32[$1>>2]|0;
      $397 = HEAP32[$pPrior>>2]|0;
      $398 = (_sqlite3Select($396,$397,$intersectdest)|0);
      HEAP32[$rc>>2] = $398;
      $399 = HEAP32[$rc>>2]|0;
      $400 = ($399|0)!=(0);
      if ($400) {
       break L1;
      }
      $401 = HEAP32[$v>>2]|0;
      $402 = HEAP32[$tab2>>2]|0;
      $403 = (_sqlite3VdbeAddOp2($401,57,$402,0)|0);
      HEAP32[$addr7>>2] = $403;
      $404 = HEAP32[$addr7>>2]|0;
      $405 = HEAP32[$2>>2]|0;
      $406 = ((($405)) + 16|0);
      $407 = ((($406)) + 4|0);
      HEAP32[$407>>2] = $404;
      $408 = HEAP32[$2>>2]|0;
      $409 = ((($408)) + 52|0);
      HEAP32[$409>>2] = 0;
      $410 = HEAP32[$2>>2]|0;
      $411 = ((($410)) + 60|0);
      $412 = HEAP32[$411>>2]|0;
      HEAP32[$pLimit5>>2] = $412;
      $413 = HEAP32[$2>>2]|0;
      $414 = ((($413)) + 60|0);
      HEAP32[$414>>2] = 0;
      $415 = HEAP32[$2>>2]|0;
      $416 = ((($415)) + 64|0);
      $417 = HEAP32[$416>>2]|0;
      HEAP32[$pOffset6>>2] = $417;
      $418 = HEAP32[$2>>2]|0;
      $419 = ((($418)) + 64|0);
      HEAP32[$419>>2] = 0;
      $420 = HEAP32[$tab2>>2]|0;
      $421 = ((($intersectdest)) + 4|0);
      HEAP32[$421>>2] = $420;
      $422 = HEAP32[$1>>2]|0;
      $423 = ((($422)) + 472|0);
      $424 = HEAP32[$423>>2]|0;
      HEAP32[$iSub2>>2] = $424;
      $425 = HEAP32[$1>>2]|0;
      $426 = HEAP32[$2>>2]|0;
      $427 = (_sqlite3Select($425,$426,$intersectdest)|0);
      HEAP32[$rc>>2] = $427;
      $428 = HEAP32[$2>>2]|0;
      $429 = ((($428)) + 52|0);
      $430 = HEAP32[$429>>2]|0;
      HEAP32[$pDelete>>2] = $430;
      $431 = HEAP32[$pPrior>>2]|0;
      $432 = HEAP32[$2>>2]|0;
      $433 = ((($432)) + 52|0);
      HEAP32[$433>>2] = $431;
      $434 = HEAP32[$2>>2]|0;
      $435 = ((($434)) + 24|0);
      $436 = $435;
      $437 = $436;
      $438 = HEAP32[$437>>2]|0;
      $439 = (($436) + 4)|0;
      $440 = $439;
      $441 = HEAP32[$440>>2]|0;
      $442 = HEAP32[$pPrior>>2]|0;
      $443 = ((($442)) + 24|0);
      $444 = $443;
      $445 = $444;
      $446 = HEAP32[$445>>2]|0;
      $447 = (($444) + 4)|0;
      $448 = $447;
      $449 = HEAP32[$448>>2]|0;
      $450 = ($441>>>0)>($449>>>0);
      $451 = ($438>>>0)>($446>>>0);
      $452 = ($441|0)==($449|0);
      $453 = $452 & $451;
      $454 = $450 | $453;
      if ($454) {
       $455 = HEAP32[$pPrior>>2]|0;
       $456 = ((($455)) + 24|0);
       $457 = $456;
       $458 = $457;
       $459 = HEAP32[$458>>2]|0;
       $460 = (($457) + 4)|0;
       $461 = $460;
       $462 = HEAP32[$461>>2]|0;
       $463 = HEAP32[$2>>2]|0;
       $464 = ((($463)) + 24|0);
       $465 = $464;
       $466 = $465;
       HEAP32[$466>>2] = $459;
       $467 = (($465) + 4)|0;
       $468 = $467;
       HEAP32[$468>>2] = $462;
      }
      $469 = HEAP32[$db>>2]|0;
      $470 = HEAP32[$2>>2]|0;
      $471 = ((($470)) + 60|0);
      $472 = HEAP32[$471>>2]|0;
      _sqlite3ExprDelete($469,$472);
      $473 = HEAP32[$pLimit5>>2]|0;
      $474 = HEAP32[$2>>2]|0;
      $475 = ((($474)) + 60|0);
      HEAP32[$475>>2] = $473;
      $476 = HEAP32[$pOffset6>>2]|0;
      $477 = HEAP32[$2>>2]|0;
      $478 = ((($477)) + 64|0);
      HEAP32[$478>>2] = $476;
      $479 = HEAP8[$dest>>0]|0;
      $480 = $479&255;
      $481 = ($480|0)==(9);
      if ($481) {
       $482 = HEAP32[$2>>2]|0;
       HEAP32[$pFirst8>>2] = $482;
       while(1) {
        $483 = HEAP32[$pFirst8>>2]|0;
        $484 = ((($483)) + 52|0);
        $485 = HEAP32[$484>>2]|0;
        $486 = ($485|0)!=(0|0);
        if (!($486)) {
         break;
        }
        $487 = HEAP32[$pFirst8>>2]|0;
        $488 = ((($487)) + 52|0);
        $489 = HEAP32[$488>>2]|0;
        HEAP32[$pFirst8>>2] = $489;
       }
       $490 = HEAP32[$1>>2]|0;
       $491 = HEAP32[$pFirst8>>2]|0;
       $492 = ((($491)) + 32|0);
       $493 = HEAP32[$492>>2]|0;
       $494 = HEAP32[$pFirst8>>2]|0;
       $495 = HEAP32[$494>>2]|0;
       _generateColumnNames($490,$493,$495);
      }
      $496 = HEAP32[$v>>2]|0;
      $497 = (_sqlite3VdbeMakeLabel($496)|0);
      HEAP32[$iBreak3>>2] = $497;
      $498 = HEAP32[$v>>2]|0;
      $499 = (_sqlite3VdbeMakeLabel($498)|0);
      HEAP32[$iCont2>>2] = $499;
      $500 = HEAP32[$1>>2]|0;
      $501 = HEAP32[$2>>2]|0;
      $502 = HEAP32[$iBreak3>>2]|0;
      _computeLimitRegisters($500,$501,$502);
      $503 = HEAP32[$v>>2]|0;
      $504 = HEAP32[$tab1>>2]|0;
      $505 = HEAP32[$iBreak3>>2]|0;
      (_sqlite3VdbeAddOp2($503,108,$504,$505)|0);
      $506 = HEAP32[$1>>2]|0;
      $507 = (_sqlite3GetTempReg($506)|0);
      HEAP32[$r1>>2] = $507;
      $508 = HEAP32[$v>>2]|0;
      $509 = HEAP32[$tab1>>2]|0;
      $510 = HEAP32[$r1>>2]|0;
      $511 = (_sqlite3VdbeAddOp2($508,101,$509,$510)|0);
      HEAP32[$iStart4>>2] = $511;
      $512 = HEAP32[$v>>2]|0;
      $513 = HEAP32[$tab2>>2]|0;
      $514 = HEAP32[$iCont2>>2]|0;
      $515 = HEAP32[$r1>>2]|0;
      (_sqlite3VdbeAddOp4Int($512,68,$513,$514,$515,0)|0);
      $516 = HEAP32[$1>>2]|0;
      $517 = HEAP32[$r1>>2]|0;
      _sqlite3ReleaseTempReg($516,$517);
      $518 = HEAP32[$1>>2]|0;
      $519 = HEAP32[$2>>2]|0;
      $520 = HEAP32[$2>>2]|0;
      $521 = HEAP32[$520>>2]|0;
      $522 = HEAP32[$tab1>>2]|0;
      $523 = HEAP32[$iCont2>>2]|0;
      $524 = HEAP32[$iBreak3>>2]|0;
      _selectInnerLoop($518,$519,$521,$522,0,0,$dest,$523,$524);
      $525 = HEAP32[$v>>2]|0;
      $526 = HEAP32[$iCont2>>2]|0;
      _sqlite3VdbeResolveLabel($525,$526);
      $527 = HEAP32[$v>>2]|0;
      $528 = HEAP32[$tab1>>2]|0;
      $529 = HEAP32[$iStart4>>2]|0;
      (_sqlite3VdbeAddOp2($527,7,$528,$529)|0);
      $530 = HEAP32[$v>>2]|0;
      $531 = HEAP32[$iBreak3>>2]|0;
      _sqlite3VdbeResolveLabel($530,$531);
      $532 = HEAP32[$v>>2]|0;
      $533 = HEAP32[$tab2>>2]|0;
      (_sqlite3VdbeAddOp2($532,61,$533,0)|0);
      $534 = HEAP32[$v>>2]|0;
      $535 = HEAP32[$tab1>>2]|0;
      (_sqlite3VdbeAddOp2($534,61,$535,0)|0);
      break L13;
     }
     }
    }
   } while(0);
   $536 = HEAP32[$1>>2]|0;
   $537 = HEAP32[$2>>2]|0;
   $538 = ((($537)) + 4|0);
   $539 = HEAP8[$538>>0]|0;
   $540 = $539&255;
   $541 = HEAP32[$iSub1>>2]|0;
   $542 = HEAP32[$iSub2>>2]|0;
   $543 = HEAP32[$2>>2]|0;
   $544 = ((($543)) + 4|0);
   $545 = HEAP8[$544>>0]|0;
   $546 = $545&255;
   $547 = ($546|0)!=(116);
   $548 = $547&1;
   _explainComposite($536,$540,$541,$542,$548);
   $549 = HEAP32[$2>>2]|0;
   $550 = ((($549)) + 6|0);
   $551 = HEAP16[$550>>1]|0;
   $552 = $551&65535;
   $553 = $552 & 16;
   $554 = ($553|0)!=(0);
   if ($554) {
    $555 = HEAP32[$2>>2]|0;
    $556 = HEAP32[$555>>2]|0;
    $557 = HEAP32[$556>>2]|0;
    HEAP32[$nCol>>2] = $557;
    $558 = HEAP32[$db>>2]|0;
    $559 = HEAP32[$nCol>>2]|0;
    $560 = (_sqlite3KeyInfoAlloc($558,$559,1)|0);
    HEAP32[$pKeyInfo>>2] = $560;
    $561 = HEAP32[$pKeyInfo>>2]|0;
    $562 = ($561|0)!=(0|0);
    if (!($562)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    HEAP32[$i>>2] = 0;
    $563 = HEAP32[$pKeyInfo>>2]|0;
    $564 = ((($563)) + 20|0);
    HEAP32[$apColl>>2] = $564;
    while(1) {
     $565 = HEAP32[$i>>2]|0;
     $566 = HEAP32[$nCol>>2]|0;
     $567 = ($565|0)<($566|0);
     if (!($567)) {
      break;
     }
     $568 = HEAP32[$1>>2]|0;
     $569 = HEAP32[$2>>2]|0;
     $570 = HEAP32[$i>>2]|0;
     $571 = (_multiSelectCollSeq($568,$569,$570)|0);
     $572 = HEAP32[$apColl>>2]|0;
     HEAP32[$572>>2] = $571;
     $573 = HEAP32[$apColl>>2]|0;
     $574 = HEAP32[$573>>2]|0;
     $575 = (0|0)==($574|0);
     if ($575) {
      $576 = HEAP32[$db>>2]|0;
      $577 = ((($576)) + 8|0);
      $578 = HEAP32[$577>>2]|0;
      $579 = HEAP32[$apColl>>2]|0;
      HEAP32[$579>>2] = $578;
     }
     $580 = HEAP32[$i>>2]|0;
     $581 = (($580) + 1)|0;
     HEAP32[$i>>2] = $581;
     $582 = HEAP32[$apColl>>2]|0;
     $583 = ((($582)) + 4|0);
     HEAP32[$apColl>>2] = $583;
    }
    $584 = HEAP32[$2>>2]|0;
    HEAP32[$pLoop>>2] = $584;
    while(1) {
     $585 = HEAP32[$pLoop>>2]|0;
     $586 = ($585|0)!=(0|0);
     if (!($586)) {
      break;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $587 = HEAP32[$i>>2]|0;
      $588 = ($587|0)<(2);
      if (!($588)) {
       break;
      }
      $589 = HEAP32[$i>>2]|0;
      $590 = HEAP32[$pLoop>>2]|0;
      $591 = ((($590)) + 16|0);
      $592 = (($591) + ($589<<2)|0);
      $593 = HEAP32[$592>>2]|0;
      HEAP32[$addr9>>2] = $593;
      $594 = HEAP32[$addr9>>2]|0;
      $595 = ($594|0)<(0);
      if ($595) {
       break;
      }
      $596 = HEAP32[$v>>2]|0;
      $597 = HEAP32[$addr9>>2]|0;
      $598 = HEAP32[$nCol>>2]|0;
      _sqlite3VdbeChangeP2($596,$597,$598);
      $599 = HEAP32[$v>>2]|0;
      $600 = HEAP32[$addr9>>2]|0;
      $601 = HEAP32[$pKeyInfo>>2]|0;
      $602 = (_sqlite3KeyInfoRef($601)|0);
      _sqlite3VdbeChangeP4($599,$600,$602,-6);
      $603 = HEAP32[$i>>2]|0;
      $604 = HEAP32[$pLoop>>2]|0;
      $605 = ((($604)) + 16|0);
      $606 = (($605) + ($603<<2)|0);
      HEAP32[$606>>2] = -1;
      $607 = HEAP32[$i>>2]|0;
      $608 = (($607) + 1)|0;
      HEAP32[$i>>2] = $608;
     }
     $609 = HEAP32[$pLoop>>2]|0;
     $610 = ((($609)) + 52|0);
     $611 = HEAP32[$610>>2]|0;
     HEAP32[$pLoop>>2] = $611;
    }
    $612 = HEAP32[$pKeyInfo>>2]|0;
    _sqlite3KeyInfoUnref($612);
   }
  }
 } while(0);
 $613 = ((($dest)) + 8|0);
 $614 = HEAP32[$613>>2]|0;
 $615 = HEAP32[$3>>2]|0;
 $616 = ((($615)) + 8|0);
 HEAP32[$616>>2] = $614;
 $617 = ((($dest)) + 12|0);
 $618 = HEAP32[$617>>2]|0;
 $619 = HEAP32[$3>>2]|0;
 $620 = ((($619)) + 12|0);
 HEAP32[$620>>2] = $618;
 $621 = HEAP32[$db>>2]|0;
 $622 = HEAP32[$pDelete>>2]|0;
 _sqlite3SelectDelete($621,$622);
 $623 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $623;
 $624 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($624|0);
}
function _sqlite3SelectExprHeight($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $nHeight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nHeight = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nHeight>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 _heightOfSelect($1,$nHeight);
 $2 = HEAP32[$nHeight>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _pushDownWhereTerms($db,$pSubq,$pWhere,$iCursor) {
 $db = $db|0;
 $pSubq = $pSubq|0;
 $pWhere = $pWhere|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $nChng = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pNew = sp + 4|0;
 $nChng = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSubq;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $iCursor;
 HEAP32[$nChng>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 6|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10 & 8200;
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 60|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 while(1) {
  $17 = HEAP32[$3>>2]|0;
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = ($19|0)==(72);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$1>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = (_pushDownWhereTerms($21,$22,$25,$26)|0);
  $28 = HEAP32[$nChng>>2]|0;
  $29 = (($28) + ($27))|0;
  HEAP32[$nChng>>2] = $29;
  $30 = HEAP32[$3>>2]|0;
  $31 = ((($30)) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$3>>2] = $32;
 }
 $33 = HEAP32[$3>>2]|0;
 $34 = ((($33)) + 4|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $35 & 1;
 $37 = ($36|0)!=(0);
 if ($37) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 $38 = HEAP32[$3>>2]|0;
 $39 = HEAP32[$4>>2]|0;
 $40 = (_sqlite3ExprIsTableConstant($38,$39)|0);
 $41 = ($40|0)!=(0);
 L20: do {
  if ($41) {
   $42 = HEAP32[$nChng>>2]|0;
   $43 = (($42) + 1)|0;
   HEAP32[$nChng>>2] = $43;
   while(1) {
    $44 = HEAP32[$2>>2]|0;
    $45 = ($44|0)!=(0|0);
    if (!($45)) {
     break L20;
    }
    $46 = HEAP32[$1>>2]|0;
    $47 = HEAP32[$3>>2]|0;
    $48 = (_sqlite3ExprDup($46,$47,0)|0);
    HEAP32[$pNew>>2] = $48;
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$pNew>>2]|0;
    $51 = HEAP32[$4>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    $54 = (_substExpr($49,$50,$51,$53)|0);
    HEAP32[$pNew>>2] = $54;
    $55 = HEAP32[$1>>2]|0;
    $56 = HEAP32[$2>>2]|0;
    $57 = ((($56)) + 36|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$pNew>>2]|0;
    $60 = (_sqlite3ExprAnd($55,$58,$59)|0);
    $61 = HEAP32[$2>>2]|0;
    $62 = ((($61)) + 36|0);
    HEAP32[$62>>2] = $60;
    $63 = HEAP32[$2>>2]|0;
    $64 = ((($63)) + 52|0);
    $65 = HEAP32[$64>>2]|0;
    HEAP32[$2>>2] = $65;
   }
  }
 } while(0);
 $66 = HEAP32[$nChng>>2]|0;
 HEAP32[$0>>2] = $66;
 $67 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($67|0);
}
function _sqlite3ClearTempRegCache($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 19|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 60|0);
 HEAP32[$4>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP1($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $3;
 STACKTOP = sp;return;
}
function _keyInfoFromExprList($pParse,$pList,$iStart,$nExtra) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $iStart = $iStart|0;
 $nExtra = $nExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $nExpr = 0;
 var $pColl = 0, $pInfo = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $nExpr = sp + 20|0;
 $pInfo = sp + 16|0;
 $pItem = sp + 12|0;
 $db = sp + 8|0;
 $i = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $iStart;
 HEAP32[$3>>2] = $nExtra;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nExpr>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$nExpr>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($9) - ($10))|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + 1)|0;
 $14 = (_sqlite3KeyInfoAlloc($8,$11,$13)|0);
 HEAP32[$pInfo>>2] = $14;
 $15 = HEAP32[$pInfo>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $56 = HEAP32[$pInfo>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$i>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($20) + (($21*20)|0)|0);
 HEAP32[$pItem>>2] = $22;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$nExpr>>2]|0;
  $25 = ($23|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (_sqlite3ExprCollSeq($26,$28)|0);
  HEAP32[$pColl>>2] = $29;
  $30 = HEAP32[$pColl>>2]|0;
  $31 = ($30|0)!=(0|0);
  if (!($31)) {
   $32 = HEAP32[$db>>2]|0;
   $33 = ((($32)) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pColl>>2] = $34;
  }
  $35 = HEAP32[$pColl>>2]|0;
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (($36) - ($37))|0;
  $39 = HEAP32[$pInfo>>2]|0;
  $40 = ((($39)) + 20|0);
  $41 = (($40) + ($38<<2)|0);
  HEAP32[$41>>2] = $35;
  $42 = HEAP32[$pItem>>2]|0;
  $43 = ((($42)) + 12|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = (($45) - ($46))|0;
  $48 = HEAP32[$pInfo>>2]|0;
  $49 = ((($48)) + 16|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + ($47)|0);
  HEAP8[$51>>0] = $44;
  $52 = HEAP32[$i>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$i>>2] = $53;
  $54 = HEAP32[$pItem>>2]|0;
  $55 = ((($54)) + 20|0);
  HEAP32[$pItem>>2] = $55;
 }
 $56 = HEAP32[$pInfo>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _computeLimitRegisters($pParse,$p,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $iLimit = 0, $iOffset = 0, $n = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $v = sp + 12|0;
 $iLimit = sp + 8|0;
 $iOffset = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iBreak;
 HEAP32[$v>>2] = 0;
 HEAP32[$iLimit>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($7);
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 60|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 76|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 1)|0;
 HEAP32[$13>>2] = $15;
 HEAP32[$iLimit>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = (_sqlite3GetVdbe($18)|0);
 HEAP32[$v>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 60|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3ExprIsInteger($22,$n)|0);
 $24 = ($23|0)!=(0);
 do {
  if ($24) {
   $25 = HEAP32[$v>>2]|0;
   $26 = HEAP32[$n>>2]|0;
   $27 = HEAP32[$iLimit>>2]|0;
   (_sqlite3VdbeAddOp2($25,22,$26,$27)|0);
   $28 = HEAP32[$n>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$v>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeGoto($30,$31)|0);
    break;
   }
   $32 = HEAP32[$n>>2]|0;
   $33 = ($32|0)>=(0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 24|0);
    $36 = $35;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = (($36) + 4)|0;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$n>>2]|0;
    $43 = ($42|0)<(0);
    $44 = $43 << 31 >> 31;
    $45 = ($41>>>0)>($44>>>0);
    $46 = ($38>>>0)>($42>>>0);
    $47 = ($41|0)==($44|0);
    $48 = $47 & $46;
    $49 = $45 | $48;
    if ($49) {
     $50 = HEAP32[$n>>2]|0;
     $51 = ($50|0)<(0);
     $52 = $51 << 31 >> 31;
     $53 = HEAP32[$1>>2]|0;
     $54 = ((($53)) + 24|0);
     $55 = $54;
     $56 = $55;
     HEAP32[$56>>2] = $50;
     $57 = (($55) + 4)|0;
     $58 = $57;
     HEAP32[$58>>2] = $52;
    }
   }
  } else {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = ((($60)) + 60|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$iLimit>>2]|0;
   _sqlite3ExprCode($59,$62,$63);
   $64 = HEAP32[$v>>2]|0;
   $65 = HEAP32[$iLimit>>2]|0;
   (_sqlite3VdbeAddOp1($64,38,$65)|0);
   $66 = HEAP32[$v>>2]|0;
   $67 = HEAP32[$iLimit>>2]|0;
   $68 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($66,46,$67,$68)|0);
  }
 } while(0);
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 64|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0|0);
 if (!($72)) {
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$0>>2]|0;
 $74 = ((($73)) + 76|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (($75) + 1)|0;
 HEAP32[$74>>2] = $76;
 HEAP32[$iOffset>>2] = $76;
 $77 = HEAP32[$1>>2]|0;
 $78 = ((($77)) + 12|0);
 HEAP32[$78>>2] = $76;
 $79 = HEAP32[$0>>2]|0;
 $80 = ((($79)) + 76|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = (($81) + 1)|0;
 HEAP32[$80>>2] = $82;
 $83 = HEAP32[$0>>2]|0;
 $84 = HEAP32[$1>>2]|0;
 $85 = ((($84)) + 64|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = HEAP32[$iOffset>>2]|0;
 _sqlite3ExprCode($83,$86,$87);
 $88 = HEAP32[$v>>2]|0;
 $89 = HEAP32[$iOffset>>2]|0;
 (_sqlite3VdbeAddOp1($88,38,$89)|0);
 $90 = HEAP32[$v>>2]|0;
 $91 = HEAP32[$iLimit>>2]|0;
 $92 = HEAP32[$iOffset>>2]|0;
 $93 = (($92) + 1)|0;
 $94 = HEAP32[$iOffset>>2]|0;
 (_sqlite3VdbeAddOp3($90,139,$91,$93,$94)|0);
 STACKTOP = sp;return;
}
function _sqlite3WhereBegin($pParse,$pTabList,$pWhere,$pOrderBy,$pResultSet,$wctrlFlags,$iIdxCur) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWhere = $pWhere|0;
 $pOrderBy = $pOrderBy|0;
 $pResultSet = $pResultSet|0;
 $wctrlFlags = $wctrlFlags|0;
 $iIdxCur = $iIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrExplain = 0, $b = 0, $bFordelete = 0, $bOnerow = 0, $db = 0, $iCur = 0, $iDb = 0;
 var $iIndexCur = 0, $ii = 0, $n = 0, $nByteWInfo = 0, $nTabList = 0, $notReady = 0, $op = 0, $op1 = 0, $or$cond = 0, $pEnd = 0, $pIx = 0, $pJ = 0, $pLevel = 0, $pLoop = 0, $pMaskSet = 0, $pTab = 0, $pTabItem = 0, $pTerm = 0, $pVTab = 0, $pWInfo = 0;
 var $rc = 0, $sWLB = 0, $tabUsed = 0, $v = 0, $vararg_buffer = 0, $wsFlags = 0, $wsFlags2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 180|0;
 $1 = sp + 176|0;
 $2 = sp + 172|0;
 $3 = sp + 168|0;
 $4 = sp + 164|0;
 $5 = sp + 160|0;
 $6 = sp + 184|0;
 $7 = sp + 156|0;
 $nByteWInfo = sp + 152|0;
 $nTabList = sp + 148|0;
 $pWInfo = sp + 144|0;
 $v = sp + 140|0;
 $notReady = sp + 16|0;
 $sWLB = sp + 120|0;
 $pMaskSet = sp + 116|0;
 $pLevel = sp + 112|0;
 $pLoop = sp + 108|0;
 $ii = sp + 104|0;
 $db = sp + 100|0;
 $rc = sp + 96|0;
 $bFordelete = sp + 186|0;
 $tabUsed = sp + 8|0;
 $pTerm = sp + 92|0;
 $pEnd = sp + 88|0;
 $wsFlags = sp + 84|0;
 $bOnerow = sp + 80|0;
 $pTab = sp + 76|0;
 $iDb = sp + 72|0;
 $pTabItem = sp + 68|0;
 $pVTab = sp + 64|0;
 $iCur = sp + 60|0;
 $op = sp + 56|0;
 $b = sp;
 $n = sp + 52|0;
 $pIx = sp + 48|0;
 $iIndexCur = sp + 44|0;
 $op1 = sp + 40|0;
 $pJ = sp + 36|0;
 $addrExplain = sp + 32|0;
 $wsFlags2 = sp + 28|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $pOrderBy;
 HEAP32[$5>>2] = $pResultSet;
 HEAP16[$6>>1] = $wctrlFlags;
 HEAP32[$7>>2] = $iIdxCur;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 HEAP8[$bFordelete>>0] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 ;HEAP32[$sWLB>>2]=0|0;HEAP32[$sWLB+4>>2]=0|0;HEAP32[$sWLB+8>>2]=0|0;HEAP32[$sWLB+12>>2]=0|0;HEAP32[$sWLB+16>>2]=0|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$4>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)>=(64);
  if ($17) {
   HEAP32[$4>>2] = 0;
  }
 }
 $18 = HEAP32[$4>>2]|0;
 $19 = ((($sWLB)) + 8|0);
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$db>>2]|0;
 $21 = ((($20)) + 60|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 32;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP16[$6>>1]|0;
  $27 = $26&65535;
  $28 = $27 & -1025;
  $29 = $28&65535;
  HEAP16[$6>>1] = $29;
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)>(64);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = 64;
  _sqlite3ErrorMsg($33,32492,$vararg_buffer);
  HEAP32[$0>>2] = 0;
  $832 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($832|0);
 }
 $34 = HEAP16[$6>>1]|0;
 $35 = $34&65535;
 $36 = $35 & 64;
 $37 = ($36|0)!=(0);
 if ($37) {
  $40 = 1;
 } else {
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = $39;
 }
 HEAP32[$nTabList>>2] = $40;
 $41 = HEAP32[$nTabList>>2]|0;
 $42 = (($41) - 1)|0;
 $43 = ($42*80)|0;
 $44 = (816 + ($43))|0;
 $45 = (($44) + 7)|0;
 $46 = $45 & -8;
 HEAP32[$nByteWInfo>>2] = $46;
 $47 = HEAP32[$db>>2]|0;
 $48 = HEAP32[$nByteWInfo>>2]|0;
 $49 = (($48) + 72)|0;
 $50 = (_sqlite3DbMallocZero($47,$49,0)|0);
 HEAP32[$pWInfo>>2] = $50;
 $51 = HEAP32[$db>>2]|0;
 $52 = ((($51)) + 65|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = ($53<<24>>24)!=(0);
 L15: do {
  if ($54) {
   $55 = HEAP32[$db>>2]|0;
   $56 = HEAP32[$pWInfo>>2]|0;
   _sqlite3DbFree($55,$56);
   HEAP32[$pWInfo>>2] = 0;
  } else {
   $57 = HEAP32[$pWInfo>>2]|0;
   $58 = ((($57)) + 60|0);
   $59 = ((($58)) + 4|0);
   HEAP32[$59>>2] = -1;
   $60 = HEAP32[$pWInfo>>2]|0;
   $61 = ((($60)) + 60|0);
   HEAP32[$61>>2] = -1;
   $62 = HEAP32[$nTabList>>2]|0;
   $63 = $62&255;
   $64 = HEAP32[$pWInfo>>2]|0;
   $65 = ((($64)) + 41|0);
   HEAP8[$65>>0] = $63;
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$pWInfo>>2]|0;
   HEAP32[$67>>2] = $66;
   $68 = HEAP32[$2>>2]|0;
   $69 = HEAP32[$pWInfo>>2]|0;
   $70 = ((($69)) + 4|0);
   HEAP32[$70>>2] = $68;
   $71 = HEAP32[$4>>2]|0;
   $72 = HEAP32[$pWInfo>>2]|0;
   $73 = ((($72)) + 8|0);
   HEAP32[$73>>2] = $71;
   $74 = HEAP32[$5>>2]|0;
   $75 = HEAP32[$pWInfo>>2]|0;
   $76 = ((($75)) + 12|0);
   HEAP32[$76>>2] = $74;
   $77 = HEAP32[$v>>2]|0;
   $78 = (_sqlite3VdbeMakeLabel($77)|0);
   $79 = HEAP32[$pWInfo>>2]|0;
   $80 = ((($79)) + 48|0);
   HEAP32[$80>>2] = $78;
   $81 = HEAP32[$pWInfo>>2]|0;
   $82 = ((($81)) + 52|0);
   HEAP32[$82>>2] = $78;
   $83 = HEAP16[$6>>1]|0;
   $84 = HEAP32[$pWInfo>>2]|0;
   $85 = ((($84)) + 34|0);
   HEAP16[$85>>1] = $83;
   $86 = HEAP32[$1>>2]|0;
   $87 = ((($86)) + 428|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = HEAP32[$pWInfo>>2]|0;
   $90 = ((($89)) + 56|0);
   HEAP32[$90>>2] = $88;
   $91 = HEAP32[$pWInfo>>2]|0;
   $92 = ((($91)) + 68|0);
   HEAP32[$pMaskSet>>2] = $92;
   $93 = HEAP32[$pWInfo>>2]|0;
   HEAP32[$sWLB>>2] = $93;
   $94 = HEAP32[$pWInfo>>2]|0;
   $95 = ((($94)) + 328|0);
   $96 = ((($sWLB)) + 4|0);
   HEAP32[$96>>2] = $95;
   $97 = HEAP32[$pWInfo>>2]|0;
   $98 = HEAP32[$nByteWInfo>>2]|0;
   $99 = (($97) + ($98)|0);
   $100 = ((($sWLB)) + 12|0);
   HEAP32[$100>>2] = $99;
   $101 = ((($sWLB)) + 12|0);
   $102 = HEAP32[$101>>2]|0;
   _whereLoopInit($102);
   $103 = HEAP32[$pMaskSet>>2]|0;
   HEAP32[$103>>2] = 0;
   $104 = HEAP32[$pWInfo>>2]|0;
   $105 = ((($104)) + 328|0);
   $106 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereClauseInit($105,$106);
   $107 = HEAP32[$pWInfo>>2]|0;
   $108 = ((($107)) + 328|0);
   $109 = HEAP32[$3>>2]|0;
   _sqlite3WhereSplit($108,$109,72);
   HEAP32[$ii>>2] = 0;
   while(1) {
    $110 = HEAP32[$ii>>2]|0;
    $111 = ((($sWLB)) + 4|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = ((($112)) + 12|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = ($110|0)<($114|0);
    $116 = HEAP32[$nTabList>>2]|0;
    $117 = ($116|0)==(0);
    if (!($115)) {
     break;
    }
    if ($117) {
     label = 16;
    } else {
     $118 = HEAP32[$ii>>2]|0;
     $119 = ((($sWLB)) + 4|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = ((($120)) + 20|0);
     $122 = HEAP32[$121>>2]|0;
     $123 = (($122) + (($118*48)|0)|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = (_sqlite3ExprIsConstantNotJoin($124)|0);
     $126 = ($125|0)!=(0);
     if ($126) {
      label = 16;
     }
    }
    if ((label|0) == 16) {
     label = 0;
     $127 = HEAP32[$1>>2]|0;
     $128 = HEAP32[$ii>>2]|0;
     $129 = ((($sWLB)) + 4|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = ((($130)) + 20|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (($132) + (($128*48)|0)|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = HEAP32[$pWInfo>>2]|0;
     $136 = ((($135)) + 52|0);
     $137 = HEAP32[$136>>2]|0;
     _sqlite3ExprIfFalse($127,$134,$137,16);
     $138 = HEAP32[$ii>>2]|0;
     $139 = ((($sWLB)) + 4|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = ((($140)) + 20|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = (($142) + (($138*48)|0)|0);
     $144 = ((($143)) + 20|0);
     $145 = HEAP16[$144>>1]|0;
     $146 = $145&65535;
     $147 = $146 | 4;
     $148 = $147&65535;
     HEAP16[$144>>1] = $148;
    }
    $149 = HEAP32[$ii>>2]|0;
    $150 = (($149) + 1)|0;
    HEAP32[$ii>>2] = $150;
   }
   if ($117) {
    $151 = HEAP32[$4>>2]|0;
    $152 = ($151|0)!=(0|0);
    if ($152) {
     $153 = HEAP32[$4>>2]|0;
     $154 = HEAP32[$153>>2]|0;
     $155 = $154&255;
     $156 = HEAP32[$pWInfo>>2]|0;
     $157 = ((($156)) + 36|0);
     HEAP8[$157>>0] = $155;
    }
    $158 = HEAP16[$6>>1]|0;
    $159 = $158&65535;
    $160 = $159 & 1024;
    $161 = ($160|0)!=(0);
    if ($161) {
     $162 = HEAP32[$pWInfo>>2]|0;
     $163 = ((($162)) + 40|0);
     HEAP8[$163>>0] = 1;
    }
   }
   HEAP32[$ii>>2] = 0;
   while(1) {
    $164 = HEAP32[$ii>>2]|0;
    $165 = HEAP32[$2>>2]|0;
    $166 = HEAP32[$165>>2]|0;
    $167 = ($164|0)<($166|0);
    if (!($167)) {
     break;
    }
    $168 = HEAP32[$pMaskSet>>2]|0;
    $169 = HEAP32[$ii>>2]|0;
    $170 = HEAP32[$2>>2]|0;
    $171 = ((($170)) + 8|0);
    $172 = (($171) + (($169*72)|0)|0);
    $173 = ((($172)) + 44|0);
    $174 = HEAP32[$173>>2]|0;
    _createMask($168,$174);
    $175 = HEAP32[$1>>2]|0;
    $176 = HEAP32[$ii>>2]|0;
    $177 = HEAP32[$2>>2]|0;
    $178 = ((($177)) + 8|0);
    $179 = (($178) + (($176*72)|0)|0);
    $180 = HEAP32[$pWInfo>>2]|0;
    $181 = ((($180)) + 328|0);
    _sqlite3WhereTabFuncArgs($175,$179,$181);
    $182 = HEAP32[$ii>>2]|0;
    $183 = (($182) + 1)|0;
    HEAP32[$ii>>2] = $183;
   }
   $184 = HEAP32[$2>>2]|0;
   $185 = HEAP32[$pWInfo>>2]|0;
   $186 = ((($185)) + 328|0);
   _sqlite3WhereExprAnalyze($184,$186);
   $187 = HEAP32[$db>>2]|0;
   $188 = ((($187)) + 65|0);
   $189 = HEAP8[$188>>0]|0;
   $190 = ($189<<24>>24)!=(0);
   if (!($190)) {
    $191 = HEAP16[$6>>1]|0;
    $192 = $191&65535;
    $193 = $192 & 1024;
    $194 = ($193|0)!=(0);
    do {
     if ($194) {
      $195 = HEAP32[$1>>2]|0;
      $196 = HEAP32[$2>>2]|0;
      $197 = HEAP32[$pWInfo>>2]|0;
      $198 = ((($197)) + 328|0);
      $199 = HEAP32[$5>>2]|0;
      $200 = (_isDistinctRedundant($195,$196,$198,$199)|0);
      $201 = ($200|0)!=(0);
      if ($201) {
       $202 = HEAP32[$pWInfo>>2]|0;
       $203 = ((($202)) + 40|0);
       HEAP8[$203>>0] = 1;
       break;
      }
      $204 = HEAP32[$4>>2]|0;
      $205 = ($204|0)==(0|0);
      if ($205) {
       $206 = HEAP32[$pWInfo>>2]|0;
       $207 = ((($206)) + 34|0);
       $208 = HEAP16[$207>>1]|0;
       $209 = $208&65535;
       $210 = $209 | 512;
       $211 = $210&65535;
       HEAP16[$207>>1] = $211;
       $212 = HEAP32[$5>>2]|0;
       $213 = HEAP32[$pWInfo>>2]|0;
       $214 = ((($213)) + 8|0);
       HEAP32[$214>>2] = $212;
      }
     }
    } while(0);
    $215 = HEAP32[$nTabList>>2]|0;
    $216 = ($215|0)!=(1);
    if ($216) {
     label = 34;
    } else {
     $217 = (_whereShortCut($sWLB)|0);
     $218 = ($217|0)==(0);
     if ($218) {
      label = 34;
     }
    }
    if ((label|0) == 34) {
     $219 = (_whereLoopAddAll($sWLB)|0);
     HEAP32[$rc>>2] = $219;
     $220 = HEAP32[$rc>>2]|0;
     $221 = ($220|0)!=(0);
     if ($221) {
      break;
     }
     $222 = HEAP32[$pWInfo>>2]|0;
     (_wherePathSolver($222,0)|0);
     $223 = HEAP32[$db>>2]|0;
     $224 = ((($223)) + 65|0);
     $225 = HEAP8[$224>>0]|0;
     $226 = ($225<<24>>24)!=(0);
     if ($226) {
      break;
     }
     $227 = HEAP32[$pWInfo>>2]|0;
     $228 = ((($227)) + 8|0);
     $229 = HEAP32[$228>>2]|0;
     $230 = ($229|0)!=(0|0);
     if ($230) {
      $231 = HEAP32[$pWInfo>>2]|0;
      $232 = HEAP32[$pWInfo>>2]|0;
      $233 = ((($232)) + 32|0);
      $234 = HEAP16[$233>>1]|0;
      $235 = $234 << 16 >> 16;
      $236 = (($235) + 1)|0;
      $237 = $236&65535;
      (_wherePathSolver($231,$237)|0);
      $238 = HEAP32[$db>>2]|0;
      $239 = ((($238)) + 65|0);
      $240 = HEAP8[$239>>0]|0;
      $241 = ($240<<24>>24)!=(0);
      if ($241) {
       break;
      }
     }
    }
    $242 = HEAP32[$pWInfo>>2]|0;
    $243 = ((($242)) + 8|0);
    $244 = HEAP32[$243>>2]|0;
    $245 = ($244|0)==(0|0);
    if ($245) {
     $246 = HEAP32[$db>>2]|0;
     $247 = ((($246)) + 24|0);
     $248 = HEAP32[$247>>2]|0;
     $249 = $248 & 131072;
     $250 = ($249|0)!=(0);
     if ($250) {
      $251 = HEAP32[$pWInfo>>2]|0;
      $252 = ((($251)) + 24|0);
      $253 = $252;
      $254 = $253;
      HEAP32[$254>>2] = -1;
      $255 = (($253) + 4)|0;
      $256 = $255;
      HEAP32[$256>>2] = -1;
     }
    }
    $257 = HEAP32[$1>>2]|0;
    $258 = ((($257)) + 68|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = ($259|0)!=(0);
    if (!($260)) {
     $261 = HEAP32[$db>>2]|0;
     $262 = ((($261)) + 65|0);
     $263 = HEAP8[$262>>0]|0;
     $264 = $263&255;
     $265 = ($264|0)!=(0);
     if (!($265)) {
      $266 = HEAP32[$pWInfo>>2]|0;
      $267 = ((($266)) + 41|0);
      $268 = HEAP8[$267>>0]|0;
      $269 = $268&255;
      $270 = ($269|0)>=(2);
      $271 = HEAP32[$5>>2]|0;
      $272 = ($271|0)!=(0|0);
      $or$cond = $270 & $272;
      L60: do {
       if ($or$cond) {
        $273 = HEAP32[$db>>2]|0;
        $274 = ((($273)) + 60|0);
        $275 = HEAP16[$274>>1]|0;
        $276 = $275&65535;
        $277 = $276 & 1024;
        $278 = ($277|0)==(0);
        if ($278) {
         $279 = HEAP32[$pMaskSet>>2]|0;
         $280 = HEAP32[$5>>2]|0;
         $281 = (_sqlite3WhereExprListUsage($279,$280)|0);
         $282 = tempRet0;
         $283 = $tabUsed;
         $284 = $283;
         HEAP32[$284>>2] = $281;
         $285 = (($283) + 4)|0;
         $286 = $285;
         HEAP32[$286>>2] = $282;
         $287 = ((($sWLB)) + 8|0);
         $288 = HEAP32[$287>>2]|0;
         $289 = ($288|0)!=(0|0);
         if ($289) {
          $290 = HEAP32[$pMaskSet>>2]|0;
          $291 = ((($sWLB)) + 8|0);
          $292 = HEAP32[$291>>2]|0;
          $293 = (_sqlite3WhereExprListUsage($290,$292)|0);
          $294 = tempRet0;
          $295 = $tabUsed;
          $296 = $295;
          $297 = HEAP32[$296>>2]|0;
          $298 = (($295) + 4)|0;
          $299 = $298;
          $300 = HEAP32[$299>>2]|0;
          $301 = $297 | $293;
          $302 = $300 | $294;
          $303 = $tabUsed;
          $304 = $303;
          HEAP32[$304>>2] = $301;
          $305 = (($303) + 4)|0;
          $306 = $305;
          HEAP32[$306>>2] = $302;
         }
         while(1) {
          $307 = HEAP32[$pWInfo>>2]|0;
          $308 = ((($307)) + 41|0);
          $309 = HEAP8[$308>>0]|0;
          $310 = $309&255;
          $311 = ($310|0)>=(2);
          if (!($311)) {
           break L60;
          }
          $312 = HEAP32[$pWInfo>>2]|0;
          $313 = ((($312)) + 41|0);
          $314 = HEAP8[$313>>0]|0;
          $315 = $314&255;
          $316 = (($315) - 1)|0;
          $317 = HEAP32[$pWInfo>>2]|0;
          $318 = ((($317)) + 736|0);
          $319 = (($318) + (($316*80)|0)|0);
          $320 = ((($319)) + 64|0);
          $321 = HEAP32[$320>>2]|0;
          HEAP32[$pLoop>>2] = $321;
          $322 = HEAP32[$pLoop>>2]|0;
          $323 = ((($322)) + 16|0);
          $324 = HEAP8[$323>>0]|0;
          $325 = $324&255;
          $326 = HEAP32[$pWInfo>>2]|0;
          $327 = ((($326)) + 4|0);
          $328 = HEAP32[$327>>2]|0;
          $329 = ((($328)) + 8|0);
          $330 = (($329) + (($325*72)|0)|0);
          $331 = ((($330)) + 36|0);
          $332 = HEAP8[$331>>0]|0;
          $333 = $332&255;
          $334 = $333 & 8;
          $335 = ($334|0)==(0);
          if ($335) {
           break L60;
          }
          $336 = HEAP16[$6>>1]|0;
          $337 = $336&65535;
          $338 = $337 & 1024;
          $339 = ($338|0)==(0);
          if ($339) {
           $340 = HEAP32[$pLoop>>2]|0;
           $341 = ((($340)) + 36|0);
           $342 = HEAP32[$341>>2]|0;
           $343 = $342 & 4096;
           $344 = ($343|0)==(0);
           if ($344) {
            break L60;
           }
          }
          $345 = $tabUsed;
          $346 = $345;
          $347 = HEAP32[$346>>2]|0;
          $348 = (($345) + 4)|0;
          $349 = $348;
          $350 = HEAP32[$349>>2]|0;
          $351 = HEAP32[$pLoop>>2]|0;
          $352 = ((($351)) + 8|0);
          $353 = $352;
          $354 = $353;
          $355 = HEAP32[$354>>2]|0;
          $356 = (($353) + 4)|0;
          $357 = $356;
          $358 = HEAP32[$357>>2]|0;
          $359 = $347 & $355;
          $360 = $350 & $358;
          $361 = ($359|0)!=(0);
          $362 = ($360|0)!=(0);
          $363 = $361 | $362;
          if ($363) {
           break L60;
          }
          $364 = ((($sWLB)) + 4|0);
          $365 = HEAP32[$364>>2]|0;
          $366 = ((($365)) + 20|0);
          $367 = HEAP32[$366>>2]|0;
          $368 = ((($sWLB)) + 4|0);
          $369 = HEAP32[$368>>2]|0;
          $370 = ((($369)) + 12|0);
          $371 = HEAP32[$370>>2]|0;
          $372 = (($367) + (($371*48)|0)|0);
          HEAP32[$pEnd>>2] = $372;
          $373 = ((($sWLB)) + 4|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = ((($374)) + 20|0);
          $376 = HEAP32[$375>>2]|0;
          HEAP32[$pTerm>>2] = $376;
          while(1) {
           $377 = HEAP32[$pTerm>>2]|0;
           $378 = HEAP32[$pEnd>>2]|0;
           $379 = ($377>>>0)<($378>>>0);
           if (!($379)) {
            break;
           }
           $380 = HEAP32[$pTerm>>2]|0;
           $381 = ((($380)) + 40|0);
           $382 = $381;
           $383 = $382;
           $384 = HEAP32[$383>>2]|0;
           $385 = (($382) + 4)|0;
           $386 = $385;
           $387 = HEAP32[$386>>2]|0;
           $388 = HEAP32[$pLoop>>2]|0;
           $389 = ((($388)) + 8|0);
           $390 = $389;
           $391 = $390;
           $392 = HEAP32[$391>>2]|0;
           $393 = (($390) + 4)|0;
           $394 = $393;
           $395 = HEAP32[$394>>2]|0;
           $396 = $384 & $392;
           $397 = $387 & $395;
           $398 = ($396|0)!=(0);
           $399 = ($397|0)!=(0);
           $400 = $398 | $399;
           if ($400) {
            $401 = HEAP32[$pTerm>>2]|0;
            $402 = HEAP32[$401>>2]|0;
            $403 = ((($402)) + 4|0);
            $404 = HEAP32[$403>>2]|0;
            $405 = $404 & 1;
            $406 = ($405|0)!=(0);
            if (!($406)) {
             break;
            }
           }
           $407 = HEAP32[$pTerm>>2]|0;
           $408 = ((($407)) + 48|0);
           HEAP32[$pTerm>>2] = $408;
          }
          $409 = HEAP32[$pTerm>>2]|0;
          $410 = HEAP32[$pEnd>>2]|0;
          $411 = ($409>>>0)<($410>>>0);
          if ($411) {
           break L60;
          }
          $412 = HEAP32[$pWInfo>>2]|0;
          $413 = ((($412)) + 41|0);
          $414 = HEAP8[$413>>0]|0;
          $415 = (($414) + -1)<<24>>24;
          HEAP8[$413>>0] = $415;
          $416 = HEAP32[$nTabList>>2]|0;
          $417 = (($416) + -1)|0;
          HEAP32[$nTabList>>2] = $417;
         }
        }
       }
      } while(0);
      $418 = HEAP32[$pWInfo>>2]|0;
      $419 = ((($418)) + 32|0);
      $420 = HEAP16[$419>>1]|0;
      $421 = $420 << 16 >> 16;
      $422 = HEAP32[$pWInfo>>2]|0;
      $423 = HEAP32[$422>>2]|0;
      $424 = ((($423)) + 428|0);
      $425 = HEAP32[$424>>2]|0;
      $426 = (($425) + ($421))|0;
      HEAP32[$424>>2] = $426;
      $427 = HEAP16[$6>>1]|0;
      $428 = $427&65535;
      $429 = $428 & 4;
      $430 = ($429|0)!=(0);
      do {
       if ($430) {
        $431 = HEAP32[$pWInfo>>2]|0;
        $432 = ((($431)) + 736|0);
        $433 = ((($432)) + 64|0);
        $434 = HEAP32[$433>>2]|0;
        $435 = ((($434)) + 36|0);
        $436 = HEAP32[$435>>2]|0;
        HEAP32[$wsFlags>>2] = $436;
        $437 = HEAP32[$wsFlags>>2]|0;
        $438 = $437 & 4096;
        $439 = ($438|0)!=(0);
        $440 = $439&1;
        HEAP32[$bOnerow>>2] = $440;
        $441 = HEAP32[$bOnerow>>2]|0;
        $442 = ($441|0)!=(0);
        if (!($442)) {
         $443 = HEAP16[$6>>1]|0;
         $444 = $443&65535;
         $445 = $444 & 8192;
         $446 = ($445|0)!=(0);
         if (!($446)) {
          break;
         }
         $447 = HEAP32[$wsFlags>>2]|0;
         $448 = $447 & 1024;
         $449 = (0)==($448|0);
         if (!($449)) {
          break;
         }
        }
        $450 = HEAP32[$bOnerow>>2]|0;
        $451 = ($450|0)!=(0);
        $452 = $451 ? 1 : 2;
        $453 = $452&255;
        $454 = HEAP32[$pWInfo>>2]|0;
        $455 = ((($454)) + 38|0);
        HEAP8[$455>>0] = $453;
        $456 = HEAP32[$2>>2]|0;
        $457 = ((($456)) + 8|0);
        $458 = ((($457)) + 16|0);
        $459 = HEAP32[$458>>2]|0;
        $460 = ((($459)) + 42|0);
        $461 = HEAP8[$460>>0]|0;
        $462 = $461&255;
        $463 = $462 & 32;
        $464 = ($463|0)==(0);
        if ($464) {
         $465 = HEAP32[$wsFlags>>2]|0;
         $466 = $465 & 64;
         $467 = ($466|0)!=(0);
         if ($467) {
          $468 = HEAP16[$6>>1]|0;
          $469 = $468&65535;
          $470 = $469 & 8192;
          $471 = ($470|0)!=(0);
          if ($471) {
           HEAP8[$bFordelete>>0] = 8;
          }
          $472 = HEAP32[$wsFlags>>2]|0;
          $473 = $472 & -65;
          $474 = HEAP32[$pWInfo>>2]|0;
          $475 = ((($474)) + 736|0);
          $476 = ((($475)) + 64|0);
          $477 = HEAP32[$476>>2]|0;
          $478 = ((($477)) + 36|0);
          HEAP32[$478>>2] = $473;
         }
        }
       }
      } while(0);
      HEAP32[$ii>>2] = 0;
      $479 = HEAP32[$pWInfo>>2]|0;
      $480 = ((($479)) + 736|0);
      HEAP32[$pLevel>>2] = $480;
      while(1) {
       $481 = HEAP32[$ii>>2]|0;
       $482 = HEAP32[$nTabList>>2]|0;
       $483 = ($481|0)<($482|0);
       if (!($483)) {
        break;
       }
       $484 = HEAP32[$pLevel>>2]|0;
       $485 = ((($484)) + 44|0);
       $486 = HEAP8[$485>>0]|0;
       $487 = $486&255;
       $488 = HEAP32[$2>>2]|0;
       $489 = ((($488)) + 8|0);
       $490 = (($489) + (($487*72)|0)|0);
       HEAP32[$pTabItem>>2] = $490;
       $491 = HEAP32[$pTabItem>>2]|0;
       $492 = ((($491)) + 16|0);
       $493 = HEAP32[$492>>2]|0;
       HEAP32[$pTab>>2] = $493;
       $494 = HEAP32[$db>>2]|0;
       $495 = HEAP32[$pTab>>2]|0;
       $496 = ((($495)) + 64|0);
       $497 = HEAP32[$496>>2]|0;
       $498 = (_sqlite3SchemaToIndex($494,$497)|0);
       HEAP32[$iDb>>2] = $498;
       $499 = HEAP32[$pLevel>>2]|0;
       $500 = ((($499)) + 64|0);
       $501 = HEAP32[$500>>2]|0;
       HEAP32[$pLoop>>2] = $501;
       $502 = HEAP32[$pTab>>2]|0;
       $503 = ((($502)) + 42|0);
       $504 = HEAP8[$503>>0]|0;
       $505 = $504&255;
       $506 = $505 & 2;
       $507 = ($506|0)!=(0);
       L97: do {
        if (!($507)) {
         $508 = HEAP32[$pTab>>2]|0;
         $509 = ((($508)) + 12|0);
         $510 = HEAP32[$509>>2]|0;
         $511 = ($510|0)!=(0|0);
         if (!($511)) {
          $512 = HEAP32[$pLoop>>2]|0;
          $513 = ((($512)) + 36|0);
          $514 = HEAP32[$513>>2]|0;
          $515 = $514 & 1024;
          $516 = ($515|0)!=(0);
          if ($516) {
           $517 = HEAP32[$db>>2]|0;
           $518 = HEAP32[$pTab>>2]|0;
           $519 = (_sqlite3GetVTable($517,$518)|0);
           HEAP32[$pVTab>>2] = $519;
           $520 = HEAP32[$pTabItem>>2]|0;
           $521 = ((($520)) + 44|0);
           $522 = HEAP32[$521>>2]|0;
           HEAP32[$iCur>>2] = $522;
           $523 = HEAP32[$v>>2]|0;
           $524 = HEAP32[$iCur>>2]|0;
           $525 = HEAP32[$pVTab>>2]|0;
           (_sqlite3VdbeAddOp4($523,152,$524,0,0,$525,-10)|0);
           break;
          }
          $526 = HEAP32[$pTab>>2]|0;
          $527 = ((($526)) + 42|0);
          $528 = HEAP8[$527>>0]|0;
          $529 = $528&255;
          $530 = $529 & 16;
          $531 = ($530|0)!=(0);
          if ($531) {
           break;
          }
          $532 = HEAP32[$pLoop>>2]|0;
          $533 = ((($532)) + 36|0);
          $534 = HEAP32[$533>>2]|0;
          $535 = $534 & 64;
          $536 = ($535|0)==(0);
          do {
           if ($536) {
            $537 = HEAP16[$6>>1]|0;
            $538 = $537&65535;
            $539 = $538 & 16;
            $540 = ($539|0)==(0);
            if (!($540)) {
             break;
            }
            HEAP32[$op>>2] = 54;
            $541 = HEAP32[$pWInfo>>2]|0;
            $542 = ((($541)) + 38|0);
            $543 = HEAP8[$542>>0]|0;
            $544 = $543&255;
            $545 = ($544|0)!=(0);
            if ($545) {
             HEAP32[$op>>2] = 55;
             $546 = HEAP32[$pTabItem>>2]|0;
             $547 = ((($546)) + 44|0);
             $548 = HEAP32[$547>>2]|0;
             $549 = HEAP32[$pWInfo>>2]|0;
             $550 = ((($549)) + 60|0);
             HEAP32[$550>>2] = $548;
            }
            $551 = HEAP32[$1>>2]|0;
            $552 = HEAP32[$pTabItem>>2]|0;
            $553 = ((($552)) + 44|0);
            $554 = HEAP32[$553>>2]|0;
            $555 = HEAP32[$iDb>>2]|0;
            $556 = HEAP32[$pTab>>2]|0;
            $557 = HEAP32[$op>>2]|0;
            _sqlite3OpenTable($551,$554,$555,$556,$557);
            $558 = HEAP32[$pWInfo>>2]|0;
            $559 = ((($558)) + 38|0);
            $560 = HEAP8[$559>>0]|0;
            $561 = $560&255;
            $562 = ($561|0)==(0);
            do {
             if ($562) {
              $563 = HEAP32[$pTab>>2]|0;
              $564 = ((($563)) + 34|0);
              $565 = HEAP16[$564>>1]|0;
              $566 = $565 << 16 >> 16;
              $567 = ($566|0)<(64);
              if (!($567)) {
               break;
              }
              $568 = HEAP32[$pTab>>2]|0;
              $569 = ((($568)) + 42|0);
              $570 = HEAP8[$569>>0]|0;
              $571 = $570&255;
              $572 = $571 & 32;
              $573 = ($572|0)==(0);
              if (!($573)) {
               break;
              }
              $574 = HEAP32[$pTabItem>>2]|0;
              $575 = ((($574)) + 56|0);
              $576 = $575;
              $577 = $576;
              $578 = HEAP32[$577>>2]|0;
              $579 = (($576) + 4)|0;
              $580 = $579;
              $581 = HEAP32[$580>>2]|0;
              $582 = $b;
              $583 = $582;
              HEAP32[$583>>2] = $578;
              $584 = (($582) + 4)|0;
              $585 = $584;
              HEAP32[$585>>2] = $581;
              HEAP32[$n>>2] = 0;
              while(1) {
               $586 = $b;
               $587 = $586;
               $588 = HEAP32[$587>>2]|0;
               $589 = (($586) + 4)|0;
               $590 = $589;
               $591 = HEAP32[$590>>2]|0;
               $592 = ($588|0)!=(0);
               $593 = ($591|0)!=(0);
               $594 = $592 | $593;
               if (!($594)) {
                break;
               }
               $595 = $b;
               $596 = $595;
               $597 = HEAP32[$596>>2]|0;
               $598 = (($595) + 4)|0;
               $599 = $598;
               $600 = HEAP32[$599>>2]|0;
               $601 = (_bitshift64Lshr(($597|0),($600|0),1)|0);
               $602 = tempRet0;
               $603 = $b;
               $604 = $603;
               HEAP32[$604>>2] = $601;
               $605 = (($603) + 4)|0;
               $606 = $605;
               HEAP32[$606>>2] = $602;
               $607 = HEAP32[$n>>2]|0;
               $608 = (($607) + 1)|0;
               HEAP32[$n>>2] = $608;
              }
              $609 = HEAP32[$v>>2]|0;
              $610 = HEAP32[$n>>2]|0;
              $611 = $610;
              _sqlite3VdbeChangeP4($609,-1,$611,-14);
             }
            } while(0);
            $612 = HEAP32[$v>>2]|0;
            $613 = HEAP8[$bFordelete>>0]|0;
            _sqlite3VdbeChangeP5($612,$613);
            break L97;
           }
          } while(0);
          $614 = HEAP32[$1>>2]|0;
          $615 = HEAP32[$iDb>>2]|0;
          $616 = HEAP32[$pTab>>2]|0;
          $617 = ((($616)) + 28|0);
          $618 = HEAP32[$617>>2]|0;
          $619 = HEAP32[$pTab>>2]|0;
          $620 = HEAP32[$619>>2]|0;
          _sqlite3TableLock($614,$615,$618,0,$620);
         }
        }
       } while(0);
       $621 = HEAP32[$pLoop>>2]|0;
       $622 = ((($621)) + 36|0);
       $623 = HEAP32[$622>>2]|0;
       $624 = $623 & 512;
       $625 = ($624|0)!=(0);
       do {
        if ($625) {
         $626 = HEAP32[$pLoop>>2]|0;
         $627 = ((($626)) + 24|0);
         $628 = ((($627)) + 4|0);
         $629 = HEAP32[$628>>2]|0;
         HEAP32[$pIx>>2] = $629;
         HEAP32[$op1>>2] = 54;
         $630 = HEAP32[$pTab>>2]|0;
         $631 = ((($630)) + 42|0);
         $632 = HEAP8[$631>>0]|0;
         $633 = $632&255;
         $634 = $633 & 32;
         $635 = ($634|0)==(0);
         do {
          if ($635) {
           label = 93;
          } else {
           $636 = HEAP32[$pIx>>2]|0;
           $637 = ((($636)) + 55|0);
           $638 = HEAP8[$637>>0]|0;
           $639 = $638 & 3;
           $640 = $639&255;
           $641 = ($640|0)==(2);
           if (!($641)) {
            label = 93;
            break;
           }
           $642 = HEAP16[$6>>1]|0;
           $643 = $642&65535;
           $644 = $643 & 64;
           $645 = ($644|0)!=(0);
           if (!($645)) {
            label = 93;
            break;
           }
           $646 = HEAP32[$pLevel>>2]|0;
           $647 = ((($646)) + 4|0);
           $648 = HEAP32[$647>>2]|0;
           HEAP32[$iIndexCur>>2] = $648;
           HEAP32[$op1>>2] = 0;
          }
         } while(0);
         L127: do {
          if ((label|0) == 93) {
           label = 0;
           $649 = HEAP32[$pWInfo>>2]|0;
           $650 = ((($649)) + 38|0);
           $651 = HEAP8[$650>>0]|0;
           $652 = $651&255;
           $653 = ($652|0)!=(0);
           if ($653) {
            $654 = HEAP32[$pTabItem>>2]|0;
            $655 = ((($654)) + 16|0);
            $656 = HEAP32[$655>>2]|0;
            $657 = ((($656)) + 8|0);
            $658 = HEAP32[$657>>2]|0;
            HEAP32[$pJ>>2] = $658;
            $659 = HEAP32[$7>>2]|0;
            HEAP32[$iIndexCur>>2] = $659;
            while(1) {
             $660 = HEAP32[$pJ>>2]|0;
             $661 = ($660|0)!=(0|0);
             if (!($661)) {
              break;
             }
             $662 = HEAP32[$pJ>>2]|0;
             $663 = HEAP32[$pIx>>2]|0;
             $664 = ($662|0)!=($663|0);
             if (!($664)) {
              break;
             }
             $665 = HEAP32[$iIndexCur>>2]|0;
             $666 = (($665) + 1)|0;
             HEAP32[$iIndexCur>>2] = $666;
             $667 = HEAP32[$pJ>>2]|0;
             $668 = ((($667)) + 20|0);
             $669 = HEAP32[$668>>2]|0;
             HEAP32[$pJ>>2] = $669;
            }
            HEAP32[$op1>>2] = 55;
            $670 = HEAP32[$iIndexCur>>2]|0;
            $671 = HEAP32[$pWInfo>>2]|0;
            $672 = ((($671)) + 60|0);
            $673 = ((($672)) + 4|0);
            HEAP32[$673>>2] = $670;
            break;
           } else {
            $674 = HEAP32[$7>>2]|0;
            $675 = ($674|0)!=(0);
            do {
             if ($675) {
              $676 = HEAP16[$6>>1]|0;
              $677 = $676&65535;
              $678 = $677 & 64;
              $679 = ($678|0)!=(0);
              if (!($679)) {
               break;
              }
              $680 = HEAP32[$7>>2]|0;
              HEAP32[$iIndexCur>>2] = $680;
              $681 = HEAP16[$6>>1]|0;
              $682 = $681&65535;
              $683 = $682 & 4096;
              $684 = ($683|0)!=(0);
              if (!($684)) {
               break L127;
              }
              HEAP32[$op1>>2] = 53;
              break L127;
             }
            } while(0);
            $685 = HEAP32[$1>>2]|0;
            $686 = ((($685)) + 72|0);
            $687 = HEAP32[$686>>2]|0;
            $688 = (($687) + 1)|0;
            HEAP32[$686>>2] = $688;
            HEAP32[$iIndexCur>>2] = $687;
            break;
           }
          }
         } while(0);
         $689 = HEAP32[$iIndexCur>>2]|0;
         $690 = HEAP32[$pLevel>>2]|0;
         $691 = ((($690)) + 8|0);
         HEAP32[$691>>2] = $689;
         $692 = HEAP32[$op1>>2]|0;
         $693 = ($692|0)!=(0);
         if (!($693)) {
          break;
         }
         $694 = HEAP32[$v>>2]|0;
         $695 = HEAP32[$op1>>2]|0;
         $696 = HEAP32[$iIndexCur>>2]|0;
         $697 = HEAP32[$pIx>>2]|0;
         $698 = ((($697)) + 44|0);
         $699 = HEAP32[$698>>2]|0;
         $700 = HEAP32[$iDb>>2]|0;
         (_sqlite3VdbeAddOp3($694,$695,$696,$699,$700)|0);
         $701 = HEAP32[$1>>2]|0;
         $702 = HEAP32[$pIx>>2]|0;
         _sqlite3VdbeSetP4KeyInfo($701,$702);
         $703 = HEAP32[$pLoop>>2]|0;
         $704 = ((($703)) + 36|0);
         $705 = HEAP32[$704>>2]|0;
         $706 = $705 & 15;
         $707 = ($706|0)!=(0);
         if (!($707)) {
          break;
         }
         $708 = HEAP32[$pLoop>>2]|0;
         $709 = ((($708)) + 36|0);
         $710 = HEAP32[$709>>2]|0;
         $711 = $710 & 32770;
         $712 = ($711|0)==(0);
         if (!($712)) {
          break;
         }
         $713 = HEAP32[$pWInfo>>2]|0;
         $714 = ((($713)) + 34|0);
         $715 = HEAP16[$714>>1]|0;
         $716 = $715&65535;
         $717 = $716 & 1;
         $718 = ($717|0)==(0);
         if (!($718)) {
          break;
         }
         $719 = HEAP32[$v>>2]|0;
         _sqlite3VdbeChangeP5($719,2);
        }
       } while(0);
       $720 = HEAP32[$iDb>>2]|0;
       $721 = ($720|0)>=(0);
       if ($721) {
        $722 = HEAP32[$1>>2]|0;
        $723 = HEAP32[$iDb>>2]|0;
        _sqlite3CodeVerifySchema($722,$723);
       }
       $724 = HEAP32[$ii>>2]|0;
       $725 = (($724) + 1)|0;
       HEAP32[$ii>>2] = $725;
       $726 = HEAP32[$pLevel>>2]|0;
       $727 = ((($726)) + 80|0);
       HEAP32[$pLevel>>2] = $727;
      }
      $728 = HEAP32[$v>>2]|0;
      $729 = (_sqlite3VdbeCurrentAddr($728)|0);
      $730 = HEAP32[$pWInfo>>2]|0;
      $731 = ((($730)) + 44|0);
      HEAP32[$731>>2] = $729;
      $732 = HEAP32[$db>>2]|0;
      $733 = ((($732)) + 65|0);
      $734 = HEAP8[$733>>0]|0;
      $735 = ($734<<24>>24)!=(0);
      if (!($735)) {
       $736 = $notReady;
       $737 = $736;
       HEAP32[$737>>2] = -1;
       $738 = (($736) + 4)|0;
       $739 = $738;
       HEAP32[$739>>2] = -1;
       HEAP32[$ii>>2] = 0;
       while(1) {
        $740 = HEAP32[$ii>>2]|0;
        $741 = HEAP32[$nTabList>>2]|0;
        $742 = ($740|0)<($741|0);
        if (!($742)) {
         break;
        }
        $743 = HEAP32[$ii>>2]|0;
        $744 = HEAP32[$pWInfo>>2]|0;
        $745 = ((($744)) + 736|0);
        $746 = (($745) + (($743*80)|0)|0);
        HEAP32[$pLevel>>2] = $746;
        $747 = HEAP32[$pLevel>>2]|0;
        $748 = ((($747)) + 64|0);
        $749 = HEAP32[$748>>2]|0;
        $750 = ((($749)) + 36|0);
        $751 = HEAP32[$750>>2]|0;
        HEAP32[$wsFlags2>>2] = $751;
        $752 = HEAP32[$pLevel>>2]|0;
        $753 = ((($752)) + 64|0);
        $754 = HEAP32[$753>>2]|0;
        $755 = ((($754)) + 36|0);
        $756 = HEAP32[$755>>2]|0;
        $757 = $756 & 16384;
        $758 = ($757|0)!=(0);
        if ($758) {
         $759 = HEAP32[$1>>2]|0;
         $760 = HEAP32[$pWInfo>>2]|0;
         $761 = ((($760)) + 328|0);
         $762 = HEAP32[$pLevel>>2]|0;
         $763 = ((($762)) + 44|0);
         $764 = HEAP8[$763>>0]|0;
         $765 = $764&255;
         $766 = HEAP32[$2>>2]|0;
         $767 = ((($766)) + 8|0);
         $768 = (($767) + (($765*72)|0)|0);
         $769 = $notReady;
         $770 = $769;
         $771 = HEAP32[$770>>2]|0;
         $772 = (($769) + 4)|0;
         $773 = $772;
         $774 = HEAP32[$773>>2]|0;
         $775 = HEAP32[$pLevel>>2]|0;
         _constructAutomaticIndex($759,$761,$768,$771,$774,$775);
         $776 = HEAP32[$db>>2]|0;
         $777 = ((($776)) + 65|0);
         $778 = HEAP8[$777>>0]|0;
         $779 = ($778<<24>>24)!=(0);
         if ($779) {
          break L15;
         }
        }
        $780 = HEAP32[$1>>2]|0;
        $781 = HEAP32[$2>>2]|0;
        $782 = HEAP32[$pLevel>>2]|0;
        $783 = HEAP32[$ii>>2]|0;
        $784 = HEAP32[$pLevel>>2]|0;
        $785 = ((($784)) + 44|0);
        $786 = HEAP8[$785>>0]|0;
        $787 = $786&255;
        $788 = HEAP16[$6>>1]|0;
        $789 = (_sqlite3WhereExplainOneScan($780,$781,$782,$783,$787,$788)|0);
        HEAP32[$addrExplain>>2] = $789;
        $790 = HEAP32[$v>>2]|0;
        $791 = (_sqlite3VdbeCurrentAddr($790)|0);
        $792 = HEAP32[$pLevel>>2]|0;
        $793 = ((($792)) + 32|0);
        HEAP32[$793>>2] = $791;
        $794 = HEAP32[$pWInfo>>2]|0;
        $795 = HEAP32[$ii>>2]|0;
        $796 = $notReady;
        $797 = $796;
        $798 = HEAP32[$797>>2]|0;
        $799 = (($796) + 4)|0;
        $800 = $799;
        $801 = HEAP32[$800>>2]|0;
        $802 = (_sqlite3WhereCodeOneLoopStart($794,$795,$798,$801)|0);
        $803 = tempRet0;
        $804 = $notReady;
        $805 = $804;
        HEAP32[$805>>2] = $802;
        $806 = (($804) + 4)|0;
        $807 = $806;
        HEAP32[$807>>2] = $803;
        $808 = HEAP32[$pLevel>>2]|0;
        $809 = ((($808)) + 24|0);
        $810 = HEAP32[$809>>2]|0;
        $811 = HEAP32[$pWInfo>>2]|0;
        $812 = ((($811)) + 48|0);
        HEAP32[$812>>2] = $810;
        $813 = HEAP32[$wsFlags2>>2]|0;
        $814 = $813 & 8192;
        $815 = ($814|0)==(0);
        do {
         if ($815) {
          $816 = HEAP16[$6>>1]|0;
          $817 = $816&65535;
          $818 = $817 & 64;
          $819 = ($818|0)==(0);
          if (!($819)) {
           break;
          }
         }
        } while(0);
        $820 = HEAP32[$ii>>2]|0;
        $821 = (($820) + 1)|0;
        HEAP32[$ii>>2] = $821;
       }
       $822 = HEAP32[$pWInfo>>2]|0;
       HEAP32[$0>>2] = $822;
       $832 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($832|0);
      }
     }
    }
   }
  }
 } while(0);
 $823 = HEAP32[$pWInfo>>2]|0;
 $824 = ($823|0)!=(0|0);
 if ($824) {
  $825 = HEAP32[$pWInfo>>2]|0;
  $826 = ((($825)) + 56|0);
  $827 = HEAP32[$826>>2]|0;
  $828 = HEAP32[$1>>2]|0;
  $829 = ((($828)) + 428|0);
  HEAP32[$829>>2] = $827;
  $830 = HEAP32[$db>>2]|0;
  $831 = HEAP32[$pWInfo>>2]|0;
  _whereInfoFree($830,$831);
 }
 HEAP32[$0>>2] = 0;
 $832 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($832|0);
}
function _sqlite3WhereOutputRowCount($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = (_sqlite3LogEstToInt($3)|0);
 $5 = tempRet0;
 tempRet0 = ($5);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereIsDistinct($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereIsOrdered($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 36|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 STACKTOP = sp;return ($4|0);
}
function _selectInnerLoop($pParse,$p,$pEList,$srcTab,$pSort,$pDistinct,$pDest,$iContinue,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pEList = $pEList|0;
 $srcTab = $srcTab|0;
 $pSort = $pSort|0;
 $pDistinct = $pDistinct|0;
 $pDest = $pDest|0;
 $iContinue = $iContinue|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addr = 0, $addrTest = 0, $eDest = 0, $ecelFlags = 0, $hasDistinct = 0, $i = 0, $iJump = 0, $iParm = 0, $nKey = 0, $nPrefixReg = 0, $nResultCol = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pColl = 0, $pOp = 0, $pSO = 0, $r1 = 0, $r11 = 0;
 var $r12 = 0, $r13 = 0, $r2 = 0, $r24 = 0, $r3 = 0, $regPrev = 0, $regResult = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $5 = sp + 104|0;
 $6 = sp + 100|0;
 $7 = sp + 96|0;
 $8 = sp + 92|0;
 $v = sp + 88|0;
 $i = sp + 84|0;
 $hasDistinct = sp + 80|0;
 $regResult = sp + 76|0;
 $eDest = sp + 72|0;
 $iParm = sp + 68|0;
 $nResultCol = sp + 64|0;
 $nPrefixReg = sp + 60|0;
 $ecelFlags = sp + 128|0;
 $pOp = sp + 56|0;
 $iJump = sp + 52|0;
 $regPrev = sp + 48|0;
 $pColl = sp + 44|0;
 $r1 = sp + 40|0;
 $r11 = sp + 36|0;
 $addr = sp + 32|0;
 $r2 = sp + 28|0;
 $r12 = sp + 24|0;
 $nKey = sp + 20|0;
 $r13 = sp + 16|0;
 $r24 = sp + 12|0;
 $r3 = sp + 8|0;
 $addrTest = sp + 4|0;
 $pSO = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $srcTab;
 HEAP32[$4>>2] = $pSort;
 HEAP32[$5>>2] = $pDistinct;
 HEAP32[$6>>2] = $pDest;
 HEAP32[$7>>2] = $iContinue;
 HEAP32[$8>>2] = $iBreak;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$6>>2]|0;
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 HEAP32[$eDest>>2] = $14;
 $15 = HEAP32[$6>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$iParm>>2] = $17;
 HEAP32[$nPrefixReg>>2] = 0;
 $18 = HEAP32[$5>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $20 = HEAP32[$5>>2]|0;
  $21 = ((($20)) + 1|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = $23;
 } else {
  $24 = 0;
 }
 HEAP32[$hasDistinct>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$4>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   HEAP32[$4>>2] = 0;
  }
 }
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0|0);
 $32 = HEAP32[$hasDistinct>>2]|0;
 $33 = ($32|0)!=(0);
 $or$cond = $31 | $33;
 if (!($or$cond)) {
  $34 = HEAP32[$v>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$7>>2]|0;
  _codeOffset($34,$37,$38);
 }
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$nResultCol>>2] = $40;
 $41 = HEAP32[$6>>2]|0;
 $42 = ((($41)) + 8|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$4>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $47 = HEAP32[$4>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$nPrefixReg>>2] = $49;
   $50 = HEAP32[$4>>2]|0;
   $51 = ((($50)) + 28|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = $53 & 1;
   $55 = ($54|0)!=(0);
   if (!($55)) {
    $56 = HEAP32[$nPrefixReg>>2]|0;
    $57 = (($56) + 1)|0;
    HEAP32[$nPrefixReg>>2] = $57;
   }
   $58 = HEAP32[$nPrefixReg>>2]|0;
   $59 = HEAP32[$0>>2]|0;
   $60 = ((($59)) + 76|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + ($58))|0;
   HEAP32[$60>>2] = $62;
  }
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 76|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 1)|0;
  $67 = HEAP32[$6>>2]|0;
  $68 = ((($67)) + 8|0);
  HEAP32[$68>>2] = $66;
  $69 = HEAP32[$nResultCol>>2]|0;
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 76|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + ($69))|0;
  HEAP32[$71>>2] = $73;
 } else {
  $74 = HEAP32[$6>>2]|0;
  $75 = ((($74)) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$nResultCol>>2]|0;
  $78 = (($76) + ($77))|0;
  $79 = HEAP32[$0>>2]|0;
  $80 = ((($79)) + 76|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = ($78|0)>($81|0);
  if ($82) {
   $83 = HEAP32[$nResultCol>>2]|0;
   $84 = HEAP32[$0>>2]|0;
   $85 = ((($84)) + 76|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($86) + ($83))|0;
   HEAP32[$85>>2] = $87;
  }
 }
 $88 = HEAP32[$nResultCol>>2]|0;
 $89 = HEAP32[$6>>2]|0;
 $90 = ((($89)) + 12|0);
 HEAP32[$90>>2] = $88;
 $91 = HEAP32[$6>>2]|0;
 $92 = ((($91)) + 8|0);
 $93 = HEAP32[$92>>2]|0;
 HEAP32[$regResult>>2] = $93;
 $94 = HEAP32[$3>>2]|0;
 $95 = ($94|0)>=(0);
 L22: do {
  if ($95) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $96 = HEAP32[$i>>2]|0;
    $97 = HEAP32[$nResultCol>>2]|0;
    $98 = ($96|0)<($97|0);
    if (!($98)) {
     break L22;
    }
    $99 = HEAP32[$v>>2]|0;
    $100 = HEAP32[$3>>2]|0;
    $101 = HEAP32[$i>>2]|0;
    $102 = HEAP32[$regResult>>2]|0;
    $103 = HEAP32[$i>>2]|0;
    $104 = (($102) + ($103))|0;
    (_sqlite3VdbeAddOp3($99,47,$100,$101,$104)|0);
    $105 = HEAP32[$i>>2]|0;
    $106 = (($105) + 1)|0;
    HEAP32[$i>>2] = $106;
   }
  } else {
   $107 = HEAP32[$eDest>>2]|0;
   $108 = ($107|0)!=(3);
   if ($108) {
    $109 = HEAP32[$eDest>>2]|0;
    $110 = ($109|0)==(10);
    $111 = HEAP32[$eDest>>2]|0;
    $112 = ($111|0)==(9);
    $or$cond3 = $110 | $112;
    $113 = HEAP32[$eDest>>2]|0;
    $114 = ($113|0)==(13);
    $or$cond5 = $or$cond3 | $114;
    if ($or$cond5) {
     HEAP8[$ecelFlags>>0] = 1;
    } else {
     HEAP8[$ecelFlags>>0] = 0;
    }
    $115 = HEAP32[$0>>2]|0;
    $116 = HEAP32[$2>>2]|0;
    $117 = HEAP32[$regResult>>2]|0;
    $118 = HEAP8[$ecelFlags>>0]|0;
    (_sqlite3ExprCodeExprList($115,$116,$117,0,$118)|0);
   }
  }
 } while(0);
 $119 = HEAP32[$hasDistinct>>2]|0;
 $120 = ($119|0)!=(0);
 if ($120) {
  $121 = HEAP32[$5>>2]|0;
  $122 = ((($121)) + 1|0);
  $123 = HEAP8[$122>>0]|0;
  $124 = $123&255;
  switch ($124|0) {
  case 2:  {
   $125 = HEAP32[$0>>2]|0;
   $126 = ((($125)) + 76|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) + 1)|0;
   HEAP32[$regPrev>>2] = $128;
   $129 = HEAP32[$nResultCol>>2]|0;
   $130 = HEAP32[$0>>2]|0;
   $131 = ((($130)) + 76|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = (($132) + ($129))|0;
   HEAP32[$131>>2] = $133;
   $134 = HEAP32[$v>>2]|0;
   $135 = HEAP32[$5>>2]|0;
   $136 = ((($135)) + 8|0);
   $137 = HEAP32[$136>>2]|0;
   (_sqlite3VdbeChangeToNoop($134,$137)|0);
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$5>>2]|0;
   $140 = ((($139)) + 8|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = (_sqlite3VdbeGetOp($138,$141)|0);
   HEAP32[$pOp>>2] = $142;
   $143 = HEAP32[$pOp>>2]|0;
   HEAP8[$143>>0] = 25;
   $144 = HEAP32[$pOp>>2]|0;
   $145 = ((($144)) + 4|0);
   HEAP32[$145>>2] = 1;
   $146 = HEAP32[$regPrev>>2]|0;
   $147 = HEAP32[$pOp>>2]|0;
   $148 = ((($147)) + 8|0);
   HEAP32[$148>>2] = $146;
   $149 = HEAP32[$v>>2]|0;
   $150 = (_sqlite3VdbeCurrentAddr($149)|0);
   $151 = HEAP32[$nResultCol>>2]|0;
   $152 = (($150) + ($151))|0;
   HEAP32[$iJump>>2] = $152;
   HEAP32[$i>>2] = 0;
   while(1) {
    $153 = HEAP32[$i>>2]|0;
    $154 = HEAP32[$nResultCol>>2]|0;
    $155 = ($153|0)<($154|0);
    if (!($155)) {
     break;
    }
    $156 = HEAP32[$0>>2]|0;
    $157 = HEAP32[$i>>2]|0;
    $158 = HEAP32[$2>>2]|0;
    $159 = ((($158)) + 4|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (($160) + (($157*20)|0)|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = (_sqlite3ExprCollSeq($156,$162)|0);
    HEAP32[$pColl>>2] = $163;
    $164 = HEAP32[$i>>2]|0;
    $165 = HEAP32[$nResultCol>>2]|0;
    $166 = (($165) - 1)|0;
    $167 = ($164|0)<($166|0);
    $168 = HEAP32[$v>>2]|0;
    $169 = HEAP32[$regResult>>2]|0;
    $170 = HEAP32[$i>>2]|0;
    $171 = (($169) + ($170))|0;
    if ($167) {
     $172 = HEAP32[$iJump>>2]|0;
     $173 = HEAP32[$regPrev>>2]|0;
     $174 = HEAP32[$i>>2]|0;
     $175 = (($173) + ($174))|0;
     (_sqlite3VdbeAddOp3($168,78,$171,$172,$175)|0);
    } else {
     $176 = HEAP32[$7>>2]|0;
     $177 = HEAP32[$regPrev>>2]|0;
     $178 = HEAP32[$i>>2]|0;
     $179 = (($177) + ($178))|0;
     (_sqlite3VdbeAddOp3($168,79,$171,$176,$179)|0);
    }
    $180 = HEAP32[$v>>2]|0;
    $181 = HEAP32[$pColl>>2]|0;
    _sqlite3VdbeChangeP4($180,-1,$181,-4);
    $182 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($182,-128);
    $183 = HEAP32[$i>>2]|0;
    $184 = (($183) + 1)|0;
    HEAP32[$i>>2] = $184;
   }
   $185 = HEAP32[$v>>2]|0;
   $186 = HEAP32[$regResult>>2]|0;
   $187 = HEAP32[$regPrev>>2]|0;
   $188 = HEAP32[$nResultCol>>2]|0;
   $189 = (($188) - 1)|0;
   (_sqlite3VdbeAddOp3($185,30,$186,$187,$189)|0);
   break;
  }
  case 1:  {
   $190 = HEAP32[$v>>2]|0;
   $191 = HEAP32[$5>>2]|0;
   $192 = ((($191)) + 8|0);
   $193 = HEAP32[$192>>2]|0;
   (_sqlite3VdbeChangeToNoop($190,$193)|0);
   break;
  }
  default: {
   $194 = HEAP32[$0>>2]|0;
   $195 = HEAP32[$5>>2]|0;
   $196 = ((($195)) + 4|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = HEAP32[$7>>2]|0;
   $199 = HEAP32[$nResultCol>>2]|0;
   $200 = HEAP32[$regResult>>2]|0;
   _codeDistinct($194,$197,$198,$199,$200);
  }
  }
  $201 = HEAP32[$4>>2]|0;
  $202 = ($201|0)==(0|0);
  if ($202) {
   $203 = HEAP32[$v>>2]|0;
   $204 = HEAP32[$1>>2]|0;
   $205 = ((($204)) + 12|0);
   $206 = HEAP32[$205>>2]|0;
   $207 = HEAP32[$7>>2]|0;
   _codeOffset($203,$206,$207);
  }
 }
 $208 = HEAP32[$eDest>>2]|0;
 L51: do {
  switch ($208|0) {
  case 1:  {
   $209 = HEAP32[$0>>2]|0;
   $210 = (_sqlite3GetTempReg($209)|0);
   HEAP32[$r1>>2] = $210;
   $211 = HEAP32[$v>>2]|0;
   $212 = HEAP32[$regResult>>2]|0;
   $213 = HEAP32[$nResultCol>>2]|0;
   $214 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp3($211,49,$212,$213,$214)|0);
   $215 = HEAP32[$v>>2]|0;
   $216 = HEAP32[$iParm>>2]|0;
   $217 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp2($215,110,$216,$217)|0);
   $218 = HEAP32[$0>>2]|0;
   $219 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($218,$219);
   break;
  }
  case 2:  {
   $220 = HEAP32[$v>>2]|0;
   $221 = HEAP32[$iParm>>2]|0;
   $222 = HEAP32[$regResult>>2]|0;
   $223 = HEAP32[$nResultCol>>2]|0;
   (_sqlite3VdbeAddOp3($220,111,$221,$222,$223)|0);
   break;
  }
  case 12: case 14: case 6: case 5:  {
   $224 = HEAP32[$0>>2]|0;
   $225 = HEAP32[$nPrefixReg>>2]|0;
   $226 = (($225) + 1)|0;
   $227 = (_sqlite3GetTempRange($224,$226)|0);
   HEAP32[$r11>>2] = $227;
   $228 = HEAP32[$v>>2]|0;
   $229 = HEAP32[$regResult>>2]|0;
   $230 = HEAP32[$nResultCol>>2]|0;
   $231 = HEAP32[$r11>>2]|0;
   $232 = HEAP32[$nPrefixReg>>2]|0;
   $233 = (($231) + ($232))|0;
   (_sqlite3VdbeAddOp3($228,49,$229,$230,$233)|0);
   $234 = HEAP32[$eDest>>2]|0;
   $235 = ($234|0)==(6);
   if ($235) {
    $236 = HEAP32[$v>>2]|0;
    $237 = (_sqlite3VdbeCurrentAddr($236)|0);
    $238 = (($237) + 4)|0;
    HEAP32[$addr>>2] = $238;
    $239 = HEAP32[$v>>2]|0;
    $240 = HEAP32[$iParm>>2]|0;
    $241 = (($240) + 1)|0;
    $242 = HEAP32[$addr>>2]|0;
    $243 = HEAP32[$r11>>2]|0;
    (_sqlite3VdbeAddOp4Int($239,69,$241,$242,$243,0)|0);
    $244 = HEAP32[$v>>2]|0;
    $245 = HEAP32[$iParm>>2]|0;
    $246 = (($245) + 1)|0;
    $247 = HEAP32[$r11>>2]|0;
    (_sqlite3VdbeAddOp2($244,110,$246,$247)|0);
   }
   $248 = HEAP32[$4>>2]|0;
   $249 = ($248|0)!=(0|0);
   $250 = HEAP32[$0>>2]|0;
   if ($249) {
    $251 = HEAP32[$4>>2]|0;
    $252 = HEAP32[$1>>2]|0;
    $253 = HEAP32[$r11>>2]|0;
    $254 = HEAP32[$nPrefixReg>>2]|0;
    $255 = (($253) + ($254))|0;
    $256 = HEAP32[$regResult>>2]|0;
    $257 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($250,$251,$252,$255,$256,1,$257);
   } else {
    $258 = (_sqlite3GetTempReg($250)|0);
    HEAP32[$r2>>2] = $258;
    $259 = HEAP32[$v>>2]|0;
    $260 = HEAP32[$iParm>>2]|0;
    $261 = HEAP32[$r2>>2]|0;
    (_sqlite3VdbeAddOp2($259,74,$260,$261)|0);
    $262 = HEAP32[$v>>2]|0;
    $263 = HEAP32[$iParm>>2]|0;
    $264 = HEAP32[$r11>>2]|0;
    $265 = HEAP32[$r2>>2]|0;
    (_sqlite3VdbeAddOp3($262,75,$263,$264,$265)|0);
    $266 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($266,8);
    $267 = HEAP32[$0>>2]|0;
    $268 = HEAP32[$r2>>2]|0;
    _sqlite3ReleaseTempReg($267,$268);
   }
   $269 = HEAP32[$0>>2]|0;
   $270 = HEAP32[$r11>>2]|0;
   $271 = HEAP32[$nPrefixReg>>2]|0;
   $272 = (($271) + 1)|0;
   _sqlite3ReleaseTempRange($269,$270,$272);
   break;
  }
  case 11:  {
   $273 = HEAP32[$2>>2]|0;
   $274 = ((($273)) + 4|0);
   $275 = HEAP32[$274>>2]|0;
   $276 = HEAP32[$275>>2]|0;
   $277 = HEAP32[$6>>2]|0;
   $278 = ((($277)) + 1|0);
   $279 = HEAP8[$278>>0]|0;
   $280 = (_sqlite3CompareAffinity($276,$279)|0);
   $281 = HEAP32[$6>>2]|0;
   $282 = ((($281)) + 1|0);
   HEAP8[$282>>0] = $280;
   $283 = HEAP32[$4>>2]|0;
   $284 = ($283|0)!=(0|0);
   $285 = HEAP32[$0>>2]|0;
   if ($284) {
    $286 = HEAP32[$4>>2]|0;
    $287 = HEAP32[$1>>2]|0;
    $288 = HEAP32[$regResult>>2]|0;
    $289 = HEAP32[$regResult>>2]|0;
    $290 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($285,$286,$287,$288,$289,1,$290);
    break L51;
   } else {
    $291 = (_sqlite3GetTempReg($285)|0);
    HEAP32[$r12>>2] = $291;
    $292 = HEAP32[$v>>2]|0;
    $293 = HEAP32[$regResult>>2]|0;
    $294 = HEAP32[$r12>>2]|0;
    $295 = HEAP32[$6>>2]|0;
    $296 = ((($295)) + 1|0);
    (_sqlite3VdbeAddOp4($292,49,$293,1,$294,$296,1)|0);
    $297 = HEAP32[$0>>2]|0;
    $298 = HEAP32[$regResult>>2]|0;
    _sqlite3ExprCacheAffinityChange($297,$298,1);
    $299 = HEAP32[$v>>2]|0;
    $300 = HEAP32[$iParm>>2]|0;
    $301 = HEAP32[$r12>>2]|0;
    (_sqlite3VdbeAddOp2($299,110,$300,$301)|0);
    $302 = HEAP32[$0>>2]|0;
    $303 = HEAP32[$r12>>2]|0;
    _sqlite3ReleaseTempReg($302,$303);
    break L51;
   }
   break;
  }
  case 3:  {
   $304 = HEAP32[$v>>2]|0;
   $305 = HEAP32[$iParm>>2]|0;
   (_sqlite3VdbeAddOp2($304,22,1,$305)|0);
   break;
  }
  case 10:  {
   $306 = HEAP32[$4>>2]|0;
   $307 = ($306|0)!=(0|0);
   if ($307) {
    $308 = HEAP32[$0>>2]|0;
    $309 = HEAP32[$4>>2]|0;
    $310 = HEAP32[$1>>2]|0;
    $311 = HEAP32[$regResult>>2]|0;
    $312 = HEAP32[$regResult>>2]|0;
    $313 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($308,$309,$310,$311,$312,1,$313);
   }
   break;
  }
  case 9: case 13:  {
   $314 = HEAP32[$4>>2]|0;
   $315 = ($314|0)!=(0|0);
   if ($315) {
    $316 = HEAP32[$0>>2]|0;
    $317 = HEAP32[$4>>2]|0;
    $318 = HEAP32[$1>>2]|0;
    $319 = HEAP32[$regResult>>2]|0;
    $320 = HEAP32[$regResult>>2]|0;
    $321 = HEAP32[$nResultCol>>2]|0;
    $322 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($316,$317,$318,$319,$320,$321,$322);
    break L51;
   }
   $323 = HEAP32[$eDest>>2]|0;
   $324 = ($323|0)==(13);
   $325 = HEAP32[$v>>2]|0;
   if ($324) {
    $326 = HEAP32[$6>>2]|0;
    $327 = ((($326)) + 4|0);
    $328 = HEAP32[$327>>2]|0;
    (_sqlite3VdbeAddOp1($325,18,$328)|0);
    break L51;
   } else {
    $329 = HEAP32[$regResult>>2]|0;
    $330 = HEAP32[$nResultCol>>2]|0;
    (_sqlite3VdbeAddOp2($325,33,$329,$330)|0);
    $331 = HEAP32[$0>>2]|0;
    $332 = HEAP32[$regResult>>2]|0;
    $333 = HEAP32[$nResultCol>>2]|0;
    _sqlite3ExprCacheAffinityChange($331,$332,$333);
    break L51;
   }
   break;
  }
  case 7: case 8:  {
   HEAP32[$addrTest>>2] = 0;
   $334 = HEAP32[$6>>2]|0;
   $335 = ((($334)) + 16|0);
   $336 = HEAP32[$335>>2]|0;
   HEAP32[$pSO>>2] = $336;
   $337 = HEAP32[$pSO>>2]|0;
   $338 = HEAP32[$337>>2]|0;
   HEAP32[$nKey>>2] = $338;
   $339 = HEAP32[$0>>2]|0;
   $340 = (_sqlite3GetTempReg($339)|0);
   HEAP32[$r13>>2] = $340;
   $341 = HEAP32[$0>>2]|0;
   $342 = HEAP32[$nKey>>2]|0;
   $343 = (($342) + 2)|0;
   $344 = (_sqlite3GetTempRange($341,$343)|0);
   HEAP32[$r24>>2] = $344;
   $345 = HEAP32[$r24>>2]|0;
   $346 = HEAP32[$nKey>>2]|0;
   $347 = (($345) + ($346))|0;
   $348 = (($347) + 1)|0;
   HEAP32[$r3>>2] = $348;
   $349 = HEAP32[$eDest>>2]|0;
   $350 = ($349|0)==(8);
   if ($350) {
    $351 = HEAP32[$v>>2]|0;
    $352 = HEAP32[$iParm>>2]|0;
    $353 = (($352) + 1)|0;
    $354 = HEAP32[$regResult>>2]|0;
    $355 = HEAP32[$nResultCol>>2]|0;
    $356 = (_sqlite3VdbeAddOp4Int($351,69,$353,0,$354,$355)|0);
    HEAP32[$addrTest>>2] = $356;
   }
   $357 = HEAP32[$v>>2]|0;
   $358 = HEAP32[$regResult>>2]|0;
   $359 = HEAP32[$nResultCol>>2]|0;
   $360 = HEAP32[$r3>>2]|0;
   (_sqlite3VdbeAddOp3($357,49,$358,$359,$360)|0);
   $361 = HEAP32[$eDest>>2]|0;
   $362 = ($361|0)==(8);
   if ($362) {
    $363 = HEAP32[$v>>2]|0;
    $364 = HEAP32[$iParm>>2]|0;
    $365 = (($364) + 1)|0;
    $366 = HEAP32[$r3>>2]|0;
    (_sqlite3VdbeAddOp2($363,110,$365,$366)|0);
    $367 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($367,16);
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $368 = HEAP32[$i>>2]|0;
    $369 = HEAP32[$nKey>>2]|0;
    $370 = ($368|0)<($369|0);
    $371 = HEAP32[$v>>2]|0;
    if (!($370)) {
     break;
    }
    $372 = HEAP32[$regResult>>2]|0;
    $373 = HEAP32[$i>>2]|0;
    $374 = HEAP32[$pSO>>2]|0;
    $375 = ((($374)) + 4|0);
    $376 = HEAP32[$375>>2]|0;
    $377 = (($376) + (($373*20)|0)|0);
    $378 = ((($377)) + 16|0);
    $379 = HEAP16[$378>>1]|0;
    $380 = $379&65535;
    $381 = (($372) + ($380))|0;
    $382 = (($381) - 1)|0;
    $383 = HEAP32[$r24>>2]|0;
    $384 = HEAP32[$i>>2]|0;
    $385 = (($383) + ($384))|0;
    (_sqlite3VdbeAddOp2($371,31,$382,$385)|0);
    $386 = HEAP32[$i>>2]|0;
    $387 = (($386) + 1)|0;
    HEAP32[$i>>2] = $387;
   }
   $388 = HEAP32[$iParm>>2]|0;
   $389 = HEAP32[$r24>>2]|0;
   $390 = HEAP32[$nKey>>2]|0;
   $391 = (($389) + ($390))|0;
   (_sqlite3VdbeAddOp2($371,73,$388,$391)|0);
   $392 = HEAP32[$v>>2]|0;
   $393 = HEAP32[$r24>>2]|0;
   $394 = HEAP32[$nKey>>2]|0;
   $395 = (($394) + 2)|0;
   $396 = HEAP32[$r13>>2]|0;
   (_sqlite3VdbeAddOp3($392,49,$393,$395,$396)|0);
   $397 = HEAP32[$v>>2]|0;
   $398 = HEAP32[$iParm>>2]|0;
   $399 = HEAP32[$r13>>2]|0;
   (_sqlite3VdbeAddOp2($397,110,$398,$399)|0);
   $400 = HEAP32[$addrTest>>2]|0;
   $401 = ($400|0)!=(0);
   if ($401) {
    $402 = HEAP32[$v>>2]|0;
    $403 = HEAP32[$addrTest>>2]|0;
    _sqlite3VdbeJumpHere($402,$403);
   }
   $404 = HEAP32[$0>>2]|0;
   $405 = HEAP32[$r13>>2]|0;
   _sqlite3ReleaseTempReg($404,$405);
   $406 = HEAP32[$0>>2]|0;
   $407 = HEAP32[$r24>>2]|0;
   $408 = HEAP32[$nKey>>2]|0;
   $409 = (($408) + 2)|0;
   _sqlite3ReleaseTempRange($406,$407,$409);
   break;
  }
  default: {
  }
  }
 } while(0);
 $410 = HEAP32[$4>>2]|0;
 $411 = ($410|0)==(0|0);
 if (!($411)) {
  STACKTOP = sp;return;
 }
 $412 = HEAP32[$1>>2]|0;
 $413 = ((($412)) + 8|0);
 $414 = HEAP32[$413>>2]|0;
 $415 = ($414|0)!=(0);
 if (!($415)) {
  STACKTOP = sp;return;
 }
 $416 = HEAP32[$v>>2]|0;
 $417 = HEAP32[$1>>2]|0;
 $418 = ((($417)) + 8|0);
 $419 = HEAP32[$418>>2]|0;
 $420 = HEAP32[$8>>2]|0;
 (_sqlite3VdbeAddOp2($416,141,$419,$420)|0);
 STACKTOP = sp;return;
}
function _sqlite3WhereContinueLabel($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3WhereBreakLabel($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3WhereEnd($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addr = 0, $db = 0, $i = 0, $j = 0, $k = 0, $last = 0, $op = 0, $pIdx = 0, $pIn = 0, $pLevel = 0, $pLoop = 0, $pOp = 0, $pParse = 0, $pPk = 0, $pTab = 0, $pTabItem = 0, $pTabList = 0, $v = 0, $ws = 0;
 var $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $pParse = sp + 76|0;
 $v = sp + 72|0;
 $i = sp + 68|0;
 $pLevel = sp + 64|0;
 $pLoop = sp + 60|0;
 $pTabList = sp + 56|0;
 $db = sp + 52|0;
 $addr = sp + 48|0;
 $pIn = sp + 44|0;
 $j = sp + 40|0;
 $op = sp + 36|0;
 $k = sp + 32|0;
 $last = sp + 28|0;
 $pOp = sp + 24|0;
 $pIdx = sp + 20|0;
 $pTabItem = sp + 16|0;
 $pTab = sp + 12|0;
 $ws = sp + 8|0;
 $x = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pParse>>2] = $2;
 $3 = HEAP32[$pParse>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTabList>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$pParse>>2]|0;
 _sqlite3ExprCacheClear($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 41|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = (($15) - 1)|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)>=(0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 736|0);
  $22 = (($21) + (($19*80)|0)|0);
  HEAP32[$pLevel>>2] = $22;
  $23 = HEAP32[$pLevel>>2]|0;
  $24 = ((($23)) + 64|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pLoop>>2] = $25;
  $26 = HEAP32[$v>>2]|0;
  $27 = HEAP32[$pLevel>>2]|0;
  $28 = ((($27)) + 24|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3VdbeResolveLabel($26,$29);
  $30 = HEAP32[$pLevel>>2]|0;
  $31 = ((($30)) + 45|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)!=(160);
  if ($34) {
   $35 = HEAP32[$v>>2]|0;
   $36 = HEAP32[$pLevel>>2]|0;
   $37 = ((($36)) + 45|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = HEAP32[$pLevel>>2]|0;
   $41 = ((($40)) + 48|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$pLevel>>2]|0;
   $44 = ((($43)) + 52|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$pLevel>>2]|0;
   $47 = ((($46)) + 46|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   (_sqlite3VdbeAddOp3($35,$39,$42,$45,$49)|0);
   $50 = HEAP32[$v>>2]|0;
   $51 = HEAP32[$pLevel>>2]|0;
   $52 = ((($51)) + 47|0);
   $53 = HEAP8[$52>>0]|0;
   _sqlite3VdbeChangeP5($50,$53);
  }
  $54 = HEAP32[$pLoop>>2]|0;
  $55 = ((($54)) + 36|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $56 & 2048;
  $58 = ($57|0)!=(0);
  L7: do {
   if ($58) {
    $59 = HEAP32[$pLevel>>2]|0;
    $60 = ((($59)) + 56|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($61|0)>(0);
    if ($62) {
     $63 = HEAP32[$v>>2]|0;
     $64 = HEAP32[$pLevel>>2]|0;
     $65 = ((($64)) + 16|0);
     $66 = HEAP32[$65>>2]|0;
     _sqlite3VdbeResolveLabel($63,$66);
     $67 = HEAP32[$pLevel>>2]|0;
     $68 = ((($67)) + 56|0);
     $69 = HEAP32[$68>>2]|0;
     HEAP32[$j>>2] = $69;
     $70 = HEAP32[$j>>2]|0;
     $71 = (($70) - 1)|0;
     $72 = HEAP32[$pLevel>>2]|0;
     $73 = ((($72)) + 56|0);
     $74 = ((($73)) + 4|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + (($71*12)|0)|0);
     HEAP32[$pIn>>2] = $76;
     while(1) {
      $77 = HEAP32[$j>>2]|0;
      $78 = ($77|0)>(0);
      if (!($78)) {
       break L7;
      }
      $79 = HEAP32[$v>>2]|0;
      $80 = HEAP32[$pIn>>2]|0;
      $81 = ((($80)) + 4|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = (($82) + 1)|0;
      _sqlite3VdbeJumpHere($79,$83);
      $84 = HEAP32[$v>>2]|0;
      $85 = HEAP32[$pIn>>2]|0;
      $86 = ((($85)) + 8|0);
      $87 = HEAP8[$86>>0]|0;
      $88 = $87&255;
      $89 = HEAP32[$pIn>>2]|0;
      $90 = HEAP32[$89>>2]|0;
      $91 = HEAP32[$pIn>>2]|0;
      $92 = ((($91)) + 4|0);
      $93 = HEAP32[$92>>2]|0;
      (_sqlite3VdbeAddOp2($84,$88,$90,$93)|0);
      $94 = HEAP32[$v>>2]|0;
      $95 = HEAP32[$pIn>>2]|0;
      $96 = ((($95)) + 4|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = (($97) - 1)|0;
      _sqlite3VdbeJumpHere($94,$98);
      $99 = HEAP32[$j>>2]|0;
      $100 = (($99) + -1)|0;
      HEAP32[$j>>2] = $100;
      $101 = HEAP32[$pIn>>2]|0;
      $102 = ((($101)) + -12|0);
      HEAP32[$pIn>>2] = $102;
     }
    }
   }
  } while(0);
  $103 = HEAP32[$v>>2]|0;
  $104 = HEAP32[$pLevel>>2]|0;
  $105 = ((($104)) + 12|0);
  $106 = HEAP32[$105>>2]|0;
  _sqlite3VdbeResolveLabel($103,$106);
  $107 = HEAP32[$pLevel>>2]|0;
  $108 = ((($107)) + 20|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = ($109|0)!=(0);
  if ($110) {
   $111 = HEAP32[$v>>2]|0;
   $112 = HEAP32[$pLevel>>2]|0;
   $113 = ((($112)) + 20|0);
   $114 = HEAP32[$113>>2]|0;
   (_sqlite3VdbeGoto($111,$114)|0);
   $115 = HEAP32[$v>>2]|0;
   $116 = HEAP32[$pLevel>>2]|0;
   $117 = ((($116)) + 20|0);
   $118 = HEAP32[$117>>2]|0;
   _sqlite3VdbeJumpHere($115,$118);
   $119 = HEAP32[$v>>2]|0;
   $120 = HEAP32[$pLevel>>2]|0;
   $121 = ((($120)) + 20|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = (($122) - 2)|0;
   _sqlite3VdbeJumpHere($119,$123);
  }
  $124 = HEAP32[$pLevel>>2]|0;
  $125 = ((($124)) + 40|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = ($126|0)!=(0);
  if ($127) {
   $128 = HEAP32[$v>>2]|0;
   $129 = HEAP32[$pLevel>>2]|0;
   $130 = ((($129)) + 40|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = (($131) - 1)|0;
   $133 = (_sqlite3VdbeGetOp($128,$132)|0);
   $134 = ((($133)) + 4|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = ($135|0)!=(0);
   if ($136) {
    HEAP32[$op>>2] = 141;
   } else {
    HEAP32[$op>>2] = 142;
   }
   $137 = HEAP32[$v>>2]|0;
   $138 = HEAP32[$op>>2]|0;
   $139 = HEAP32[$pLevel>>2]|0;
   $140 = ((($139)) + 36|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = HEAP32[$pLevel>>2]|0;
   $143 = ((($142)) + 40|0);
   $144 = HEAP32[$143>>2]|0;
   (_sqlite3VdbeAddOp2($137,$138,$141,$144)|0);
  }
  $145 = HEAP32[$pLevel>>2]|0;
  $146 = HEAP32[$145>>2]|0;
  $147 = ($146|0)!=(0);
  if ($147) {
   $148 = HEAP32[$v>>2]|0;
   $149 = HEAP32[$pLevel>>2]|0;
   $150 = HEAP32[$149>>2]|0;
   $151 = (_sqlite3VdbeAddOp1($148,138,$150)|0);
   HEAP32[$addr>>2] = $151;
   $152 = HEAP32[$pLoop>>2]|0;
   $153 = ((($152)) + 36|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = $154 & 64;
   $156 = ($155|0)==(0);
   if ($156) {
    $157 = HEAP32[$v>>2]|0;
    $158 = HEAP32[$i>>2]|0;
    $159 = HEAP32[$pTabList>>2]|0;
    $160 = ((($159)) + 8|0);
    $161 = (($160) + (($158*72)|0)|0);
    $162 = ((($161)) + 44|0);
    $163 = HEAP32[$162>>2]|0;
    (_sqlite3VdbeAddOp1($157,104,$163)|0);
   }
   $164 = HEAP32[$pLoop>>2]|0;
   $165 = ((($164)) + 36|0);
   $166 = HEAP32[$165>>2]|0;
   $167 = $166 & 512;
   $168 = ($167|0)!=(0);
   if ($168) {
    $169 = HEAP32[$v>>2]|0;
    $170 = HEAP32[$pLevel>>2]|0;
    $171 = ((($170)) + 8|0);
    $172 = HEAP32[$171>>2]|0;
    (_sqlite3VdbeAddOp1($169,104,$172)|0);
   }
   $173 = HEAP32[$pLevel>>2]|0;
   $174 = ((($173)) + 45|0);
   $175 = HEAP8[$174>>0]|0;
   $176 = $175&255;
   $177 = ($176|0)==(15);
   $178 = HEAP32[$v>>2]|0;
   $179 = HEAP32[$pLevel>>2]|0;
   if ($177) {
    $180 = ((($179)) + 48|0);
    $181 = HEAP32[$180>>2]|0;
    $182 = HEAP32[$pLevel>>2]|0;
    $183 = ((($182)) + 28|0);
    $184 = HEAP32[$183>>2]|0;
    (_sqlite3VdbeAddOp2($178,14,$181,$184)|0);
   } else {
    $185 = ((($179)) + 28|0);
    $186 = HEAP32[$185>>2]|0;
    (_sqlite3VdbeGoto($178,$186)|0);
   }
   $187 = HEAP32[$v>>2]|0;
   $188 = HEAP32[$addr>>2]|0;
   _sqlite3VdbeJumpHere($187,$188);
  }
  $189 = HEAP32[$i>>2]|0;
  $190 = (($189) + -1)|0;
  HEAP32[$i>>2] = $190;
 }
 $191 = HEAP32[$v>>2]|0;
 $192 = HEAP32[$0>>2]|0;
 $193 = ((($192)) + 52|0);
 $194 = HEAP32[$193>>2]|0;
 _sqlite3VdbeResolveLabel($191,$194);
 HEAP32[$i>>2] = 0;
 $195 = HEAP32[$0>>2]|0;
 $196 = ((($195)) + 736|0);
 HEAP32[$pLevel>>2] = $196;
 while(1) {
  $197 = HEAP32[$i>>2]|0;
  $198 = HEAP32[$0>>2]|0;
  $199 = ((($198)) + 41|0);
  $200 = HEAP8[$199>>0]|0;
  $201 = $200&255;
  $202 = ($197|0)<($201|0);
  if (!($202)) {
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  $203 = HEAP32[$pLevel>>2]|0;
  $204 = ((($203)) + 44|0);
  $205 = HEAP8[$204>>0]|0;
  $206 = $205&255;
  $207 = HEAP32[$pTabList>>2]|0;
  $208 = ((($207)) + 8|0);
  $209 = (($208) + (($206*72)|0)|0);
  HEAP32[$pTabItem>>2] = $209;
  $210 = HEAP32[$pTabItem>>2]|0;
  $211 = ((($210)) + 16|0);
  $212 = HEAP32[$211>>2]|0;
  HEAP32[$pTab>>2] = $212;
  $213 = HEAP32[$pLevel>>2]|0;
  $214 = ((($213)) + 64|0);
  $215 = HEAP32[$214>>2]|0;
  HEAP32[$pLoop>>2] = $215;
  $216 = HEAP32[$pTabItem>>2]|0;
  $217 = ((($216)) + 36|0);
  $218 = ((($217)) + 1|0);
  $219 = HEAP8[$218>>0]|0;
  $220 = ($219&255) >>> 4;
  $221 = $220 & 1;
  $222 = $221&255;
  $223 = ($222|0)!=(0);
  if ($223) {
   $224 = HEAP32[$db>>2]|0;
   $225 = ((($224)) + 65|0);
   $226 = HEAP8[$225>>0]|0;
   $227 = ($226<<24>>24)!=(0);
   if ($227) {
    label = 32;
   } else {
    $228 = HEAP32[$v>>2]|0;
    $229 = HEAP32[$pLevel>>2]|0;
    $230 = ((($229)) + 32|0);
    $231 = HEAP32[$230>>2]|0;
    $232 = HEAP32[$pLevel>>2]|0;
    $233 = ((($232)) + 4|0);
    $234 = HEAP32[$233>>2]|0;
    $235 = HEAP32[$pTabItem>>2]|0;
    $236 = ((($235)) + 32|0);
    $237 = HEAP32[$236>>2]|0;
    _translateColumnToCopy($228,$231,$234,$237,0);
   }
  } else {
   label = 32;
  }
  L44: do {
   if ((label|0) == 32) {
    label = 0;
    $238 = HEAP32[$pTab>>2]|0;
    $239 = ((($238)) + 42|0);
    $240 = HEAP8[$239>>0]|0;
    $241 = $240&255;
    $242 = $241 & 2;
    $243 = ($242|0)==(0);
    if ($243) {
     $244 = HEAP32[$pTab>>2]|0;
     $245 = ((($244)) + 12|0);
     $246 = HEAP32[$245>>2]|0;
     $247 = ($246|0)==(0|0);
     if ($247) {
      $248 = HEAP32[$0>>2]|0;
      $249 = ((($248)) + 34|0);
      $250 = HEAP16[$249>>1]|0;
      $251 = $250&65535;
      $252 = $251 & 16;
      $253 = ($252|0)==(0);
      if ($253) {
       $254 = HEAP32[$pLoop>>2]|0;
       $255 = ((($254)) + 36|0);
       $256 = HEAP32[$255>>2]|0;
       HEAP32[$ws>>2] = $256;
       $257 = HEAP32[$0>>2]|0;
       $258 = ((($257)) + 38|0);
       $259 = HEAP8[$258>>0]|0;
       $260 = $259&255;
       $261 = ($260|0)==(0);
       if ($261) {
        $262 = HEAP32[$ws>>2]|0;
        $263 = $262 & 64;
        $264 = ($263|0)==(0);
        if ($264) {
         $265 = HEAP32[$v>>2]|0;
         $266 = HEAP32[$pTabItem>>2]|0;
         $267 = ((($266)) + 44|0);
         $268 = HEAP32[$267>>2]|0;
         (_sqlite3VdbeAddOp1($265,61,$268)|0);
        }
       }
       $269 = HEAP32[$ws>>2]|0;
       $270 = $269 & 512;
       $271 = ($270|0)!=(0);
       if ($271) {
        $272 = HEAP32[$ws>>2]|0;
        $273 = $272 & 16640;
        $274 = ($273|0)==(0);
        if ($274) {
         $275 = HEAP32[$pLevel>>2]|0;
         $276 = ((($275)) + 8|0);
         $277 = HEAP32[$276>>2]|0;
         $278 = HEAP32[$0>>2]|0;
         $279 = ((($278)) + 60|0);
         $280 = ((($279)) + 4|0);
         $281 = HEAP32[$280>>2]|0;
         $282 = ($277|0)!=($281|0);
         if ($282) {
          $283 = HEAP32[$v>>2]|0;
          $284 = HEAP32[$pLevel>>2]|0;
          $285 = ((($284)) + 8|0);
          $286 = HEAP32[$285>>2]|0;
          (_sqlite3VdbeAddOp1($283,61,$286)|0);
         }
        }
       }
      }
     }
    }
    $287 = HEAP32[$pLoop>>2]|0;
    $288 = ((($287)) + 36|0);
    $289 = HEAP32[$288>>2]|0;
    $290 = $289 & 576;
    $291 = ($290|0)!=(0);
    $292 = HEAP32[$pLoop>>2]|0;
    if ($291) {
     $293 = ((($292)) + 24|0);
     $294 = ((($293)) + 4|0);
     $295 = HEAP32[$294>>2]|0;
     HEAP32[$pIdx>>2] = $295;
    } else {
     $296 = ((($292)) + 36|0);
     $297 = HEAP32[$296>>2]|0;
     $298 = $297 & 8192;
     $299 = ($298|0)!=(0);
     if ($299) {
      $300 = HEAP32[$pLevel>>2]|0;
      $301 = ((($300)) + 56|0);
      $302 = HEAP32[$301>>2]|0;
      HEAP32[$pIdx>>2] = $302;
     }
    }
    $303 = HEAP32[$pIdx>>2]|0;
    $304 = ($303|0)!=(0|0);
    if ($304) {
     $305 = HEAP32[$0>>2]|0;
     $306 = ((($305)) + 38|0);
     $307 = HEAP8[$306>>0]|0;
     $308 = $307&255;
     $309 = ($308|0)==(0);
     if (!($309)) {
      $310 = HEAP32[$pIdx>>2]|0;
      $311 = ((($310)) + 12|0);
      $312 = HEAP32[$311>>2]|0;
      $313 = ((($312)) + 42|0);
      $314 = HEAP8[$313>>0]|0;
      $315 = $314&255;
      $316 = $315 & 32;
      $317 = ($316|0)==(0);
      if ($317) {
       break;
      }
     }
     $318 = HEAP32[$db>>2]|0;
     $319 = ((($318)) + 65|0);
     $320 = HEAP8[$319>>0]|0;
     $321 = ($320<<24>>24)!=(0);
     if (!($321)) {
      $322 = HEAP32[$v>>2]|0;
      $323 = (_sqlite3VdbeCurrentAddr($322)|0);
      HEAP32[$last>>2] = $323;
      $324 = HEAP32[$pLevel>>2]|0;
      $325 = ((($324)) + 32|0);
      $326 = HEAP32[$325>>2]|0;
      HEAP32[$k>>2] = $326;
      $327 = HEAP32[$v>>2]|0;
      $328 = HEAP32[$k>>2]|0;
      $329 = (_sqlite3VdbeGetOp($327,$328)|0);
      HEAP32[$pOp>>2] = $329;
      while(1) {
       $330 = HEAP32[$k>>2]|0;
       $331 = HEAP32[$last>>2]|0;
       $332 = ($330|0)<($331|0);
       if (!($332)) {
        break L44;
       }
       $333 = HEAP32[$pOp>>2]|0;
       $334 = ((($333)) + 4|0);
       $335 = HEAP32[$334>>2]|0;
       $336 = HEAP32[$pLevel>>2]|0;
       $337 = ((($336)) + 4|0);
       $338 = HEAP32[$337>>2]|0;
       $339 = ($335|0)!=($338|0);
       do {
        if (!($339)) {
         $340 = HEAP32[$pOp>>2]|0;
         $341 = HEAP8[$340>>0]|0;
         $342 = $341&255;
         $343 = ($342|0)==(47);
         $344 = HEAP32[$pOp>>2]|0;
         if (!($343)) {
          $377 = HEAP8[$344>>0]|0;
          $378 = $377&255;
          $379 = ($378|0)==(103);
          if (!($379)) {
           break;
          }
          $380 = HEAP32[$pLevel>>2]|0;
          $381 = ((($380)) + 8|0);
          $382 = HEAP32[$381>>2]|0;
          $383 = HEAP32[$pOp>>2]|0;
          $384 = ((($383)) + 4|0);
          HEAP32[$384>>2] = $382;
          $385 = HEAP32[$pOp>>2]|0;
          HEAP8[$385>>0] = 113;
          break;
         }
         $345 = ((($344)) + 8|0);
         $346 = HEAP32[$345>>2]|0;
         HEAP32[$x>>2] = $346;
         $347 = HEAP32[$pTab>>2]|0;
         $348 = ((($347)) + 42|0);
         $349 = HEAP8[$348>>0]|0;
         $350 = $349&255;
         $351 = $350 & 32;
         $352 = ($351|0)==(0);
         if (!($352)) {
          $353 = HEAP32[$pTab>>2]|0;
          $354 = (_sqlite3PrimaryKeyIndex($353)|0);
          HEAP32[$pPk>>2] = $354;
          $355 = HEAP32[$x>>2]|0;
          $356 = HEAP32[$pPk>>2]|0;
          $357 = ((($356)) + 4|0);
          $358 = HEAP32[$357>>2]|0;
          $359 = (($358) + ($355<<1)|0);
          $360 = HEAP16[$359>>1]|0;
          $361 = $360 << 16 >> 16;
          HEAP32[$x>>2] = $361;
         }
         $362 = HEAP32[$pIdx>>2]|0;
         $363 = HEAP32[$x>>2]|0;
         $364 = $363&65535;
         $365 = (_sqlite3ColumnOfIndex($362,$364)|0);
         $366 = $365 << 16 >> 16;
         HEAP32[$x>>2] = $366;
         $367 = HEAP32[$x>>2]|0;
         $368 = ($367|0)>=(0);
         if ($368) {
          $369 = HEAP32[$x>>2]|0;
          $370 = HEAP32[$pOp>>2]|0;
          $371 = ((($370)) + 8|0);
          HEAP32[$371>>2] = $369;
          $372 = HEAP32[$pLevel>>2]|0;
          $373 = ((($372)) + 8|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = HEAP32[$pOp>>2]|0;
          $376 = ((($375)) + 4|0);
          HEAP32[$376>>2] = $374;
         }
        }
       } while(0);
       $386 = HEAP32[$k>>2]|0;
       $387 = (($386) + 1)|0;
       HEAP32[$k>>2] = $387;
       $388 = HEAP32[$pOp>>2]|0;
       $389 = ((($388)) + 20|0);
       HEAP32[$pOp>>2] = $389;
      }
     }
    }
   }
  } while(0);
  $390 = HEAP32[$i>>2]|0;
  $391 = (($390) + 1)|0;
  HEAP32[$i>>2] = $391;
  $392 = HEAP32[$pLevel>>2]|0;
  $393 = ((($392)) + 80|0);
  HEAP32[$pLevel>>2] = $393;
 }
 $394 = HEAP32[$0>>2]|0;
 $395 = ((($394)) + 56|0);
 $396 = HEAP32[$395>>2]|0;
 $397 = HEAP32[$pParse>>2]|0;
 $398 = ((($397)) + 428|0);
 HEAP32[$398>>2] = $396;
 $399 = HEAP32[$db>>2]|0;
 $400 = HEAP32[$0>>2]|0;
 _whereInfoFree($399,$400);
 STACKTOP = sp;return;
}
function _sqlite3ExprAnalyzeAggList($pNC,$pList) {
 $pNC = $pNC|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pItem = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pItem>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$pItem>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  _sqlite3ExprAnalyzeAggregates($11,$13);
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
  $16 = HEAP32[$pItem>>2]|0;
  $17 = ((($16)) + 20|0);
  HEAP32[$pItem>>2] = $17;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprAnalyzeAggregates($pNC,$pExpr) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pExpr;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 4|0);
 HEAP32[$2>>2] = 193;
 $3 = ((($w)) + 8|0);
 HEAP32[$3>>2] = 194;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($w)) + 24|0);
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$1>>2]|0;
 (_sqlite3WalkExpr($w,$6)|0);
 STACKTOP = sp;return;
}
function _explainTempTable($pParse,$zUsage) {
 $pParse = $pParse|0;
 $zUsage = $zUsage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0;
 var $vararg_buffer = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $v = sp + 8|0;
 $zMsg = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zUsage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 453|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(2);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $12;
 $13 = (_sqlite3MPrintf($11,33030,$vararg_buffer)|0);
 HEAP32[$zMsg>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 468|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($14,161,$17,0,0,$18,-1)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheClear($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$i>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 124|0);
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3|0)<(10);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 12|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$p>>2]|0;
   _cacheEntryClear($9,$10);
   $11 = HEAP32[$p>>2]|0;
   $12 = ((($11)) + 12|0);
   HEAP32[$12>>2] = 0;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 20|0);
  HEAP32[$p>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeGetColumnToReg($pParse,$pTab,$iColumn,$iTable,$iReg) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iColumn = $iColumn|0;
 $iTable = $iTable|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $r1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iColumn;
 HEAP32[$3>>2] = $iTable;
 HEAP32[$4>>2] = $iReg;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3ExprCodeGetColumn($5,$6,$7,$8,$9,0)|0);
 HEAP32[$r1>>2] = $10;
 $11 = HEAP32[$r1>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($11|0)!=($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$r1>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeAddOp2($16,31,$17,$18)|0);
 STACKTOP = sp;return;
}
function _sqlite3WhereIsSorted($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 37|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3ExprCodeMove($pParse,$iFrom,$iTo,$nReg) {
 $pParse = $pParse|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iFrom;
 HEAP32[$2>>2] = $iTo;
 HEAP32[$3>>2] = $nReg;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($6,29,$7,$8,$9)|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 _sqlite3ExprCacheRemove($10,$11,$12);
 STACKTOP = sp;return;
}
function _updateAccumulator($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $addrHitTest = 0, $addrNext = 0, $i = 0, $j = 0, $nArg = 0, $pC = 0, $pColl = 0, $pF = 0, $pItem = 0, $pList = 0, $regAgg = 0, $regHit = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $v = sp + 48|0;
 $i = sp + 44|0;
 $regHit = sp + 40|0;
 $addrHitTest = sp + 36|0;
 $pF = sp + 32|0;
 $pC = sp + 28|0;
 $nArg = sp + 24|0;
 $addrNext = sp + 20|0;
 $regAgg = sp + 16|0;
 $pList = sp + 12|0;
 $pColl = sp + 8|0;
 $pItem = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 HEAP32[$regHit>>2] = 0;
 HEAP32[$addrHitTest>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 HEAP8[$5>>0] = 1;
 HEAP32[$i>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pF>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  HEAP32[$addrNext>>2] = 0;
  $14 = HEAP32[$pF>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pList>>2] = $17;
  $18 = HEAP32[$pList>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pList>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$nArg>>2] = $21;
   $22 = HEAP32[$0>>2]|0;
   $23 = HEAP32[$nArg>>2]|0;
   $24 = (_sqlite3GetTempRange($22,$23)|0);
   HEAP32[$regAgg>>2] = $24;
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$pList>>2]|0;
   $27 = HEAP32[$regAgg>>2]|0;
   (_sqlite3ExprCodeExprList($25,$26,$27,0,1)|0);
  } else {
   HEAP32[$nArg>>2] = 0;
   HEAP32[$regAgg>>2] = 0;
  }
  $28 = HEAP32[$pF>>2]|0;
  $29 = ((($28)) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)>=(0);
  if ($31) {
   $32 = HEAP32[$v>>2]|0;
   $33 = (_sqlite3VdbeMakeLabel($32)|0);
   HEAP32[$addrNext>>2] = $33;
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$pF>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$addrNext>>2]|0;
   $39 = HEAP32[$regAgg>>2]|0;
   _codeDistinct($34,$37,$38,1,$39);
  }
  $40 = HEAP32[$pF>>2]|0;
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ((($42)) + 2|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 & 32;
  $47 = ($46|0)!=(0);
  if ($47) {
   HEAP32[$pColl>>2] = 0;
   HEAP32[$j>>2] = 0;
   $48 = HEAP32[$pList>>2]|0;
   $49 = ((($48)) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pItem>>2] = $50;
   while(1) {
    $51 = HEAP32[$pColl>>2]|0;
    $52 = ($51|0)!=(0|0);
    if ($52) {
     break;
    }
    $53 = HEAP32[$j>>2]|0;
    $54 = HEAP32[$nArg>>2]|0;
    $55 = ($53|0)<($54|0);
    if (!($55)) {
     break;
    }
    $56 = HEAP32[$0>>2]|0;
    $57 = HEAP32[$pItem>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    $59 = (_sqlite3ExprCollSeq($56,$58)|0);
    HEAP32[$pColl>>2] = $59;
    $60 = HEAP32[$j>>2]|0;
    $61 = (($60) + 1)|0;
    HEAP32[$j>>2] = $61;
    $62 = HEAP32[$pItem>>2]|0;
    $63 = ((($62)) + 20|0);
    HEAP32[$pItem>>2] = $63;
   }
   $64 = HEAP32[$pColl>>2]|0;
   $65 = ($64|0)!=(0|0);
   if (!($65)) {
    $66 = HEAP32[$0>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = ((($67)) + 8|0);
    $69 = HEAP32[$68>>2]|0;
    HEAP32[$pColl>>2] = $69;
   }
   $70 = HEAP32[$regHit>>2]|0;
   $71 = ($70|0)==(0);
   if ($71) {
    $72 = HEAP32[$1>>2]|0;
    $73 = ((($72)) + 36|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($74|0)!=(0);
    if ($75) {
     $76 = HEAP32[$0>>2]|0;
     $77 = ((($76)) + 76|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = (($78) + 1)|0;
     HEAP32[$77>>2] = $79;
     HEAP32[$regHit>>2] = $79;
    }
   }
   $80 = HEAP32[$v>>2]|0;
   $81 = HEAP32[$regHit>>2]|0;
   $82 = HEAP32[$pColl>>2]|0;
   (_sqlite3VdbeAddOp4($80,34,$81,0,0,$82,-4)|0);
  }
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$regAgg>>2]|0;
  $85 = HEAP32[$pF>>2]|0;
  $86 = ((($85)) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = HEAP32[$pF>>2]|0;
  $89 = ((($88)) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  (_sqlite3VdbeAddOp4($83,143,0,$84,$87,$90,-5)|0);
  $91 = HEAP32[$v>>2]|0;
  $92 = HEAP32[$nArg>>2]|0;
  $93 = $92&255;
  _sqlite3VdbeChangeP5($91,$93);
  $94 = HEAP32[$0>>2]|0;
  $95 = HEAP32[$regAgg>>2]|0;
  $96 = HEAP32[$nArg>>2]|0;
  _sqlite3ExprCacheAffinityChange($94,$95,$96);
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$regAgg>>2]|0;
  $99 = HEAP32[$nArg>>2]|0;
  _sqlite3ReleaseTempRange($97,$98,$99);
  $100 = HEAP32[$addrNext>>2]|0;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$v>>2]|0;
   $103 = HEAP32[$addrNext>>2]|0;
   _sqlite3VdbeResolveLabel($102,$103);
   $104 = HEAP32[$0>>2]|0;
   _sqlite3ExprCacheClear($104);
  }
  $105 = HEAP32[$i>>2]|0;
  $106 = (($105) + 1)|0;
  HEAP32[$i>>2] = $106;
  $107 = HEAP32[$pF>>2]|0;
  $108 = ((($107)) + 16|0);
  HEAP32[$pF>>2] = $108;
 }
 $109 = HEAP32[$regHit>>2]|0;
 $110 = ($109|0)!=(0);
 if ($110) {
  $111 = HEAP32[$v>>2]|0;
  $112 = HEAP32[$regHit>>2]|0;
  $113 = (_sqlite3VdbeAddOp1($111,45,$112)|0);
  HEAP32[$addrHitTest>>2] = $113;
 }
 $114 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($114);
 HEAP32[$i>>2] = 0;
 $115 = HEAP32[$1>>2]|0;
 $116 = ((($115)) + 28|0);
 $117 = HEAP32[$116>>2]|0;
 HEAP32[$pC>>2] = $117;
 while(1) {
  $118 = HEAP32[$i>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = ((($119)) + 36|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($118|0)<($121|0);
  if (!($122)) {
   break;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = HEAP32[$pC>>2]|0;
  $125 = ((($124)) + 20|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = HEAP32[$pC>>2]|0;
  $128 = ((($127)) + 16|0);
  $129 = HEAP32[$128>>2]|0;
  _sqlite3ExprCode($123,$126,$129);
  $130 = HEAP32[$i>>2]|0;
  $131 = (($130) + 1)|0;
  HEAP32[$i>>2] = $131;
  $132 = HEAP32[$pC>>2]|0;
  $133 = ((($132)) + 24|0);
  HEAP32[$pC>>2] = $133;
 }
 $134 = HEAP32[$1>>2]|0;
 HEAP8[$134>>0] = 0;
 $135 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($135);
 $136 = HEAP32[$addrHitTest>>2]|0;
 $137 = ($136|0)!=(0);
 if (!($137)) {
  STACKTOP = sp;return;
 }
 $138 = HEAP32[$v>>2]|0;
 $139 = HEAP32[$addrHitTest>>2]|0;
 _sqlite3VdbeJumpHere($138,$139);
 STACKTOP = sp;return;
}
function _finalizeAggFunctions($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pF = 0, $pList = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $v = sp + 12|0;
 $i = sp + 8|0;
 $pF = sp + 4|0;
 $pList = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pF>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pF>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pList>>2] = $16;
  $17 = HEAP32[$v>>2]|0;
  $18 = HEAP32[$pF>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pList>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$pList>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $28 = $24;
  } else {
   $28 = 0;
  }
  $25 = HEAP32[$pF>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3VdbeAddOp4($17,145,$20,$28,0,$27,-5)|0);
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
  $31 = HEAP32[$pF>>2]|0;
  $32 = ((($31)) + 16|0);
  HEAP32[$pF>>2] = $32;
 }
 STACKTOP = sp;return;
}
function _resetAccumulator($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $nReg = 0, $pE = 0, $pFunc = 0, $pKeyInfo = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $pFunc = sp + 16|0;
 $nReg = sp + 12|0;
 $pE = sp + 8|0;
 $pKeyInfo = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7) + ($10))|0;
 HEAP32[$nReg>>2] = $11;
 $12 = HEAP32[$nReg>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3VdbeAddOp3($14,25,0,$17,$20)|0);
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 40|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pFunc>>2] = $23;
 HEAP32[$i>>2] = 0;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 44|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($24|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$pFunc>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)>=(0);
  do {
   if ($32) {
    $33 = HEAP32[$pFunc>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$pE>>2] = $34;
    $35 = HEAP32[$pE>>2]|0;
    $36 = ((($35)) + 20|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0|0);
    if (!($38)) {
     $39 = HEAP32[$pE>>2]|0;
     $40 = ((($39)) + 20|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = ($42|0)!=(1);
     if (!($43)) {
      $47 = HEAP32[$0>>2]|0;
      $48 = HEAP32[$pE>>2]|0;
      $49 = ((($48)) + 20|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = (_keyInfoFromExprList($47,$50,0,0)|0);
      HEAP32[$pKeyInfo>>2] = $51;
      $52 = HEAP32[$v>>2]|0;
      $53 = HEAP32[$pFunc>>2]|0;
      $54 = ((($53)) + 12|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$pKeyInfo>>2]|0;
      (_sqlite3VdbeAddOp4($52,57,$55,0,0,$56,-6)|0);
      break;
     }
    }
    $44 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($44,33053,$vararg_buffer);
    $45 = HEAP32[$pFunc>>2]|0;
    $46 = ((($45)) + 12|0);
    HEAP32[$46>>2] = -1;
   }
  } while(0);
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$i>>2] = $58;
  $59 = HEAP32[$pFunc>>2]|0;
  $60 = ((($59)) + 16|0);
  HEAP32[$pFunc>>2] = $60;
 }
 STACKTOP = sp;return;
}
function _isSimpleCount($p,$pAggInfo) {
 $p = $p|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pExpr = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTab = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pAggInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(1);
  if (!($10)) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 32|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)!=(1);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 32|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ((($18)) + 8|0);
    $20 = ((($19)) + 20|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)!=(0|0);
    if (!($22)) {
     $23 = HEAP32[$1>>2]|0;
     $24 = ((($23)) + 32|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = ((($25)) + 8|0);
     $27 = ((($26)) + 16|0);
     $28 = HEAP32[$27>>2]|0;
     HEAP32[$pTab>>2] = $28;
     $29 = HEAP32[$1>>2]|0;
     $30 = HEAP32[$29>>2]|0;
     $31 = ((($30)) + 4|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$32>>2]|0;
     HEAP32[$pExpr>>2] = $33;
     $34 = HEAP32[$pTab>>2]|0;
     $35 = ((($34)) + 42|0);
     $36 = HEAP8[$35>>0]|0;
     $37 = $36&255;
     $38 = $37 & 16;
     $39 = ($38|0)!=(0);
     if ($39) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $40 = HEAP32[$pExpr>>2]|0;
     $41 = HEAP8[$40>>0]|0;
     $42 = $41&255;
     $43 = ($42|0)!=(153);
     if ($43) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $44 = HEAP32[$2>>2]|0;
     $45 = ((($44)) + 44|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)==(0);
     if ($47) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $48 = HEAP32[$2>>2]|0;
     $49 = ((($48)) + 40|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = ((($50)) + 4|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ((($52)) + 2|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = $55 & 256;
     $57 = ($56|0)==(0);
     if ($57) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $58 = HEAP32[$pExpr>>2]|0;
     $59 = ((($58)) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = $60 & 16;
     $62 = ($61|0)!=(0);
     if ($62) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     } else {
      $63 = HEAP32[$pTab>>2]|0;
      HEAP32[$0>>2] = $63;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _explainSimpleCount($pParse,$pTab,$pIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bCover = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zEqp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $bCover = sp + 16|0;
 $zEqp = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 453|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 42|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = $13 & 32;
  $15 = ($14|0)==(0);
  if ($15) {
   $24 = 1;
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 55|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 & 3;
   $20 = $19&255;
   $21 = ($20|0)==(2);
   $22 = $21 ^ 1;
   $24 = $22;
  }
 } else {
  $24 = 0;
 }
 $23 = $24&1;
 HEAP32[$bCover>>2] = $23;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$bCover>>2]|0;
 $30 = ($29|0)!=(0);
 $31 = $30 ? 33122 : 17532;
 $32 = HEAP32[$bCover>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$2>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = $35;
 } else {
  $36 = 17532;
 }
 HEAP32[$vararg_buffer>>2] = $28;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $31;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $36;
 $37 = (_sqlite3MPrintf($26,33104,$vararg_buffer)|0);
 HEAP32[$zEqp>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 468|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$zEqp>>2]|0;
 (_sqlite3VdbeAddOp4($40,161,$43,0,0,$44,-1)|0);
 STACKTOP = sp;return;
}
function _minMaxQuery($pAggInfo,$ppMinMax) {
 $pAggInfo = $pAggInfo|0;
 $ppMinMax = $ppMinMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $eRet = 0, $pEList = 0, $pExpr = 0, $zFunc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $eRet = sp + 12|0;
 $pExpr = sp + 8|0;
 $pEList = sp + 4|0;
 $zFunc = sp;
 HEAP32[$0>>2] = $pAggInfo;
 HEAP32[$1>>2] = $ppMinMax;
 HEAP32[$eRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(1);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = ((($7)) + 40|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$pExpr>>2] = $10;
   $11 = HEAP32[$pExpr>>2]|0;
   $12 = ((($11)) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pEList>>2] = $13;
   $14 = HEAP32[$pEList>>2]|0;
   $15 = ($14|0)!=(0|0);
   if ($15) {
    $16 = HEAP32[$pEList>>2]|0;
    $17 = HEAP32[$16>>2]|0;
    $18 = ($17|0)==(1);
    if ($18) {
     $19 = HEAP32[$pEList>>2]|0;
     $20 = ((($19)) + 4|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$21>>2]|0;
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)==(154);
     if ($25) {
      $26 = HEAP32[$pExpr>>2]|0;
      $27 = ((($26)) + 8|0);
      $28 = HEAP32[$27>>2]|0;
      HEAP32[$zFunc>>2] = $28;
      $29 = HEAP32[$zFunc>>2]|0;
      $30 = (_sqlite3_stricmp($29,33145)|0);
      $31 = ($30|0)==(0);
      if ($31) {
       HEAP32[$eRet>>2] = 1;
       $32 = HEAP32[$pEList>>2]|0;
       $33 = HEAP32[$1>>2]|0;
       HEAP32[$33>>2] = $32;
       break;
      }
      $34 = HEAP32[$zFunc>>2]|0;
      $35 = (_sqlite3_stricmp($34,33149)|0);
      $36 = ($35|0)==(0);
      if ($36) {
       HEAP32[$eRet>>2] = 2;
       $37 = HEAP32[$pEList>>2]|0;
       $38 = HEAP32[$1>>2]|0;
       HEAP32[$38>>2] = $37;
      }
     }
    }
   }
  }
 } while(0);
 $39 = HEAP32[$eRet>>2]|0;
 $40 = $39&255;
 STACKTOP = sp;return ($40|0);
}
function _generateSortTail($pParse,$p,$pSort,$nColumn,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pSort = $pSort|0;
 $nColumn = $nColumn|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addrBreak = 0, $addrContinue = 0, $addrOnce = 0, $bSeq = 0, $eDest = 0, $i = 0;
 var $iParm = 0, $iSortTab = 0, $iTab = 0, $nKey = 0, $nSortData = 0, $or$cond = 0, $pOrderBy = 0, $regRow = 0, $regRowid = 0, $regSortOut = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $v = sp + 64|0;
 $addrBreak = sp + 60|0;
 $addrContinue = sp + 56|0;
 $addr = sp + 52|0;
 $addrOnce = sp + 48|0;
 $iTab = sp + 44|0;
 $pOrderBy = sp + 40|0;
 $eDest = sp + 36|0;
 $iParm = sp + 32|0;
 $regRow = sp + 28|0;
 $regRowid = sp + 24|0;
 $nKey = sp + 20|0;
 $iSortTab = sp + 16|0;
 $nSortData = sp + 12|0;
 $i = sp + 8|0;
 $bSeq = sp + 4|0;
 $regSortOut = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pSort;
 HEAP32[$3>>2] = $nColumn;
 HEAP32[$4>>2] = $pDest;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$addrBreak>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = (_sqlite3VdbeMakeLabel($11)|0);
 HEAP32[$addrContinue>>2] = $12;
 HEAP32[$addrOnce>>2] = 0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pOrderBy>>2] = $14;
 $15 = HEAP32[$4>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 HEAP32[$eDest>>2] = $17;
 $18 = HEAP32[$4>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$iParm>>2] = $20;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$v>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  (_sqlite3VdbeAddOp2($25,14,$28,$31)|0);
  $32 = HEAP32[$v>>2]|0;
  $33 = HEAP32[$addrBreak>>2]|0;
  (_sqlite3VdbeGoto($32,$33)|0);
  $34 = HEAP32[$v>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3VdbeResolveLabel($34,$37);
 }
 $38 = HEAP32[$2>>2]|0;
 $39 = ((($38)) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$iTab>>2] = $40;
 $41 = HEAP32[$eDest>>2]|0;
 $42 = ($41|0)==(9);
 $43 = HEAP32[$eDest>>2]|0;
 $44 = ($43|0)==(13);
 $or$cond = $42 | $44;
 if ($or$cond) {
  HEAP32[$regRowid>>2] = 0;
  $45 = HEAP32[$4>>2]|0;
  $46 = ((($45)) + 8|0);
  $47 = HEAP32[$46>>2]|0;
  HEAP32[$regRow>>2] = $47;
  $48 = HEAP32[$3>>2]|0;
  HEAP32[$nSortData>>2] = $48;
 } else {
  $49 = HEAP32[$0>>2]|0;
  $50 = (_sqlite3GetTempReg($49)|0);
  HEAP32[$regRowid>>2] = $50;
  $51 = HEAP32[$0>>2]|0;
  $52 = (_sqlite3GetTempReg($51)|0);
  HEAP32[$regRow>>2] = $52;
  HEAP32[$nSortData>>2] = 1;
 }
 $53 = HEAP32[$pOrderBy>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 $55 = HEAP32[$2>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = (($54) - ($57))|0;
 HEAP32[$nKey>>2] = $58;
 $59 = HEAP32[$2>>2]|0;
 $60 = ((($59)) + 28|0);
 $61 = HEAP8[$60>>0]|0;
 $62 = $61&255;
 $63 = $62 & 1;
 $64 = ($63|0)!=(0);
 if ($64) {
  $65 = HEAP32[$0>>2]|0;
  $66 = ((($65)) + 76|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + 1)|0;
  HEAP32[$66>>2] = $68;
  HEAP32[$regSortOut>>2] = $68;
  $69 = HEAP32[$0>>2]|0;
  $70 = ((($69)) + 72|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$70>>2] = $72;
  HEAP32[$iSortTab>>2] = $71;
  $73 = HEAP32[$2>>2]|0;
  $74 = ((($73)) + 16|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = ($75|0)!=(0);
  if ($76) {
   $77 = HEAP32[$0>>2]|0;
   $78 = (_sqlite3CodeOnce($77)|0);
   HEAP32[$addrOnce>>2] = $78;
  }
  $79 = HEAP32[$v>>2]|0;
  $80 = HEAP32[$iSortTab>>2]|0;
  $81 = HEAP32[$regSortOut>>2]|0;
  $82 = HEAP32[$nKey>>2]|0;
  $83 = (($82) + 1)|0;
  $84 = HEAP32[$nSortData>>2]|0;
  $85 = (($83) + ($84))|0;
  (_sqlite3VdbeAddOp3($79,60,$80,$81,$85)|0);
  $86 = HEAP32[$addrOnce>>2]|0;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$v>>2]|0;
   $89 = HEAP32[$addrOnce>>2]|0;
   _sqlite3VdbeJumpHere($88,$89);
  }
  $90 = HEAP32[$v>>2]|0;
  $91 = HEAP32[$iTab>>2]|0;
  $92 = HEAP32[$addrBreak>>2]|0;
  $93 = (_sqlite3VdbeAddOp2($90,106,$91,$92)|0);
  $94 = (1 + ($93))|0;
  HEAP32[$addr>>2] = $94;
  $95 = HEAP32[$v>>2]|0;
  $96 = HEAP32[$1>>2]|0;
  $97 = ((($96)) + 12|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = HEAP32[$addrContinue>>2]|0;
  _codeOffset($95,$98,$99);
  $100 = HEAP32[$v>>2]|0;
  $101 = HEAP32[$iTab>>2]|0;
  $102 = HEAP32[$regSortOut>>2]|0;
  $103 = HEAP32[$iSortTab>>2]|0;
  (_sqlite3VdbeAddOp3($100,100,$101,$102,$103)|0);
  HEAP32[$bSeq>>2] = 0;
 } else {
  $104 = HEAP32[$v>>2]|0;
  $105 = HEAP32[$iTab>>2]|0;
  $106 = HEAP32[$addrBreak>>2]|0;
  $107 = (_sqlite3VdbeAddOp2($104,107,$105,$106)|0);
  $108 = (1 + ($107))|0;
  HEAP32[$addr>>2] = $108;
  $109 = HEAP32[$v>>2]|0;
  $110 = HEAP32[$1>>2]|0;
  $111 = ((($110)) + 12|0);
  $112 = HEAP32[$111>>2]|0;
  $113 = HEAP32[$addrContinue>>2]|0;
  _codeOffset($109,$112,$113);
  $114 = HEAP32[$iTab>>2]|0;
  HEAP32[$iSortTab>>2] = $114;
  HEAP32[$bSeq>>2] = 1;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $115 = HEAP32[$i>>2]|0;
  $116 = HEAP32[$nSortData>>2]|0;
  $117 = ($115|0)<($116|0);
  if (!($117)) {
   break;
  }
  $118 = HEAP32[$v>>2]|0;
  $119 = HEAP32[$iSortTab>>2]|0;
  $120 = HEAP32[$nKey>>2]|0;
  $121 = HEAP32[$bSeq>>2]|0;
  $122 = (($120) + ($121))|0;
  $123 = HEAP32[$i>>2]|0;
  $124 = (($122) + ($123))|0;
  $125 = HEAP32[$regRow>>2]|0;
  $126 = HEAP32[$i>>2]|0;
  $127 = (($125) + ($126))|0;
  (_sqlite3VdbeAddOp3($118,47,$119,$124,$127)|0);
  $128 = HEAP32[$i>>2]|0;
  $129 = (($128) + 1)|0;
  HEAP32[$i>>2] = $129;
 }
 $130 = HEAP32[$eDest>>2]|0;
 L22: do {
  switch ($130|0) {
  case 12:  {
   $131 = HEAP32[$v>>2]|0;
   $132 = HEAP32[$iParm>>2]|0;
   $133 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp2($131,74,$132,$133)|0);
   $134 = HEAP32[$v>>2]|0;
   $135 = HEAP32[$iParm>>2]|0;
   $136 = HEAP32[$regRow>>2]|0;
   $137 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp3($134,75,$135,$136,$137)|0);
   $138 = HEAP32[$v>>2]|0;
   _sqlite3VdbeChangeP5($138,8);
   break;
  }
  case 11:  {
   $139 = HEAP32[$v>>2]|0;
   $140 = HEAP32[$regRow>>2]|0;
   $141 = HEAP32[$regRowid>>2]|0;
   $142 = HEAP32[$4>>2]|0;
   $143 = ((($142)) + 1|0);
   (_sqlite3VdbeAddOp4($139,49,$140,1,$141,$143,1)|0);
   $144 = HEAP32[$0>>2]|0;
   $145 = HEAP32[$regRow>>2]|0;
   _sqlite3ExprCacheAffinityChange($144,$145,1);
   $146 = HEAP32[$v>>2]|0;
   $147 = HEAP32[$iParm>>2]|0;
   $148 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp2($146,110,$147,$148)|0);
   break;
  }
  case 10:  {
   $149 = HEAP32[$0>>2]|0;
   $150 = HEAP32[$regRow>>2]|0;
   $151 = HEAP32[$iParm>>2]|0;
   _sqlite3ExprCodeMove($149,$150,$151,1);
   break;
  }
  default: {
   $152 = HEAP32[$eDest>>2]|0;
   $153 = ($152|0)==(9);
   $154 = HEAP32[$v>>2]|0;
   $155 = HEAP32[$4>>2]|0;
   if ($153) {
    $156 = ((($155)) + 8|0);
    $157 = HEAP32[$156>>2]|0;
    $158 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($154,33,$157,$158)|0);
    $159 = HEAP32[$0>>2]|0;
    $160 = HEAP32[$4>>2]|0;
    $161 = ((($160)) + 8|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = HEAP32[$3>>2]|0;
    _sqlite3ExprCacheAffinityChange($159,$162,$163);
    break L22;
   } else {
    $164 = ((($155)) + 4|0);
    $165 = HEAP32[$164>>2]|0;
    (_sqlite3VdbeAddOp1($154,18,$165)|0);
    break L22;
   }
  }
  }
 } while(0);
 $166 = HEAP32[$regRowid>>2]|0;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = HEAP32[$0>>2]|0;
  $169 = HEAP32[$regRow>>2]|0;
  _sqlite3ReleaseTempReg($168,$169);
  $170 = HEAP32[$0>>2]|0;
  $171 = HEAP32[$regRowid>>2]|0;
  _sqlite3ReleaseTempReg($170,$171);
 }
 $172 = HEAP32[$v>>2]|0;
 $173 = HEAP32[$addrContinue>>2]|0;
 _sqlite3VdbeResolveLabel($172,$173);
 $174 = HEAP32[$2>>2]|0;
 $175 = ((($174)) + 28|0);
 $176 = HEAP8[$175>>0]|0;
 $177 = $176&255;
 $178 = $177 & 1;
 $179 = ($178|0)!=(0);
 $180 = HEAP32[$v>>2]|0;
 $181 = HEAP32[$iTab>>2]|0;
 $182 = HEAP32[$addr>>2]|0;
 if ($179) {
  (_sqlite3VdbeAddOp2($180,3,$181,$182)|0);
 } else {
  (_sqlite3VdbeAddOp2($180,7,$181,$182)|0);
 }
 $183 = HEAP32[$2>>2]|0;
 $184 = ((($183)) + 12|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = ($185|0)!=(0);
 if (!($186)) {
  $191 = HEAP32[$v>>2]|0;
  $192 = HEAP32[$addrBreak>>2]|0;
  _sqlite3VdbeResolveLabel($191,$192);
  STACKTOP = sp;return;
 }
 $187 = HEAP32[$v>>2]|0;
 $188 = HEAP32[$2>>2]|0;
 $189 = ((($188)) + 12|0);
 $190 = HEAP32[$189>>2]|0;
 (_sqlite3VdbeAddOp1($187,15,$190)|0);
 $191 = HEAP32[$v>>2]|0;
 $192 = HEAP32[$addrBreak>>2]|0;
 _sqlite3VdbeResolveLabel($191,$192);
 STACKTOP = sp;return;
}
function _generateColumnNames($pParse,$pTabList,$pEList) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $fullNames = 0, $i = 0, $iCol = 0, $j = 0, $or$cond = 0, $p = 0, $pTab = 0, $shortNames = 0, $v = 0, $vararg_buffer = 0;
 var $vararg_buffer3 = 0, $vararg_ptr2 = 0, $z = 0, $zCol = 0, $zName = 0, $zName1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $v = sp + 60|0;
 $i = sp + 56|0;
 $j = sp + 52|0;
 $db = sp + 48|0;
 $fullNames = sp + 44|0;
 $shortNames = sp + 40|0;
 $p = sp + 36|0;
 $zName = sp + 32|0;
 $pTab = sp + 28|0;
 $zCol = sp + 24|0;
 $iCol = sp + 20|0;
 $zName1 = sp + 16|0;
 $z = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 453|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = ((($17)) + 65|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20|0)!=(0);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 16|0);
 HEAP8[$23>>0] = 1;
 $24 = HEAP32[$db>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $26 & 4;
 $28 = ($27|0)!=(0);
 $29 = $28&1;
 HEAP32[$fullNames>>2] = $29;
 $30 = HEAP32[$db>>2]|0;
 $31 = ((($30)) + 24|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = $32 & 64;
 $34 = ($33|0)!=(0);
 $35 = $34&1;
 HEAP32[$shortNames>>2] = $35;
 $36 = HEAP32[$v>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 _sqlite3VdbeSetNumCols($36,$38);
 HEAP32[$i>>2] = 0;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = ($39|0)<($41|0);
  if (!($42)) {
   break;
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$2>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + (($43*20)|0)|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$p>>2] = $48;
  $49 = HEAP32[$p>>2]|0;
  $50 = ($49|0)==(0|0);
  do {
   if (!($50)) {
    $51 = HEAP32[$i>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 4|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + (($51*20)|0)|0);
    $56 = ((($55)) + 4|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0|0);
    if ($58) {
     $59 = HEAP32[$i>>2]|0;
     $60 = HEAP32[$2>>2]|0;
     $61 = ((($60)) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = (($62) + (($59*20)|0)|0);
     $64 = ((($63)) + 4|0);
     $65 = HEAP32[$64>>2]|0;
     HEAP32[$zName>>2] = $65;
     $66 = HEAP32[$v>>2]|0;
     $67 = HEAP32[$i>>2]|0;
     $68 = HEAP32[$zName>>2]|0;
     (_sqlite3VdbeSetColName($66,$67,0,$68,(-1))|0);
     break;
    }
    $69 = HEAP32[$p>>2]|0;
    $70 = HEAP8[$69>>0]|0;
    $71 = $70&255;
    $72 = ($71|0)==(152);
    if (!($72)) {
     $73 = HEAP32[$p>>2]|0;
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     $76 = ($75|0)==(154);
     if (!($76)) {
      $145 = HEAP32[$i>>2]|0;
      $146 = HEAP32[$2>>2]|0;
      $147 = ((($146)) + 4|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = (($148) + (($145*20)|0)|0);
      $150 = ((($149)) + 8|0);
      $151 = HEAP32[$150>>2]|0;
      HEAP32[$z>>2] = $151;
      $152 = HEAP32[$z>>2]|0;
      $153 = ($152|0)==(0|0);
      $154 = HEAP32[$db>>2]|0;
      if ($153) {
       $155 = HEAP32[$i>>2]|0;
       $156 = (($155) + 1)|0;
       HEAP32[$vararg_buffer3>>2] = $156;
       $157 = (_sqlite3MPrintf($154,33153,$vararg_buffer3)|0);
       $160 = $157;
      } else {
       $158 = HEAP32[$z>>2]|0;
       $159 = (_sqlite3DbStrDup($154,$158)|0);
       $160 = $159;
      }
      HEAP32[$z>>2] = $160;
      $161 = HEAP32[$v>>2]|0;
      $162 = HEAP32[$i>>2]|0;
      $163 = HEAP32[$z>>2]|0;
      (_sqlite3VdbeSetColName($161,$162,0,$163,169)|0);
      break;
     }
    }
    $77 = HEAP32[$p>>2]|0;
    $78 = ((($77)) + 32|0);
    $79 = HEAP16[$78>>1]|0;
    $80 = $79 << 16 >> 16;
    HEAP32[$iCol>>2] = $80;
    HEAP32[$j>>2] = 0;
    while(1) {
     $81 = HEAP32[$j>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = HEAP32[$82>>2]|0;
     $84 = ($81|0)<($83|0);
     if (!($84)) {
      break;
     }
     $85 = HEAP32[$j>>2]|0;
     $86 = HEAP32[$1>>2]|0;
     $87 = ((($86)) + 8|0);
     $88 = (($87) + (($85*72)|0)|0);
     $89 = ((($88)) + 44|0);
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP32[$p>>2]|0;
     $92 = ((($91)) + 28|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = ($90|0)==($93|0);
     if ($94) {
      break;
     }
     $95 = HEAP32[$j>>2]|0;
     $96 = (($95) + 1)|0;
     HEAP32[$j>>2] = $96;
    }
    $97 = HEAP32[$j>>2]|0;
    $98 = HEAP32[$1>>2]|0;
    $99 = ((($98)) + 8|0);
    $100 = (($99) + (($97*72)|0)|0);
    $101 = ((($100)) + 16|0);
    $102 = HEAP32[$101>>2]|0;
    HEAP32[$pTab>>2] = $102;
    $103 = HEAP32[$iCol>>2]|0;
    $104 = ($103|0)<(0);
    if ($104) {
     $105 = HEAP32[$pTab>>2]|0;
     $106 = ((($105)) + 32|0);
     $107 = HEAP16[$106>>1]|0;
     $108 = $107 << 16 >> 16;
     HEAP32[$iCol>>2] = $108;
    }
    $109 = HEAP32[$iCol>>2]|0;
    $110 = ($109|0)<(0);
    if ($110) {
     HEAP32[$zCol>>2] = 27337;
    } else {
     $111 = HEAP32[$iCol>>2]|0;
     $112 = HEAP32[$pTab>>2]|0;
     $113 = ((($112)) + 4|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + (($111*24)|0)|0);
     $116 = HEAP32[$115>>2]|0;
     HEAP32[$zCol>>2] = $116;
    }
    $117 = HEAP32[$shortNames>>2]|0;
    $118 = ($117|0)!=(0);
    $119 = HEAP32[$fullNames>>2]|0;
    $120 = ($119|0)!=(0);
    $or$cond = $118 | $120;
    if (!($or$cond)) {
     $121 = HEAP32[$v>>2]|0;
     $122 = HEAP32[$i>>2]|0;
     $123 = HEAP32[$db>>2]|0;
     $124 = HEAP32[$i>>2]|0;
     $125 = HEAP32[$2>>2]|0;
     $126 = ((($125)) + 4|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = (($127) + (($124*20)|0)|0);
     $129 = ((($128)) + 8|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = (_sqlite3DbStrDup($123,$130)|0);
     (_sqlite3VdbeSetColName($121,$122,0,$131,169)|0);
     break;
    }
    $132 = HEAP32[$fullNames>>2]|0;
    $133 = ($132|0)!=(0);
    if ($133) {
     HEAP32[$zName1>>2] = 0;
     $134 = HEAP32[$db>>2]|0;
     $135 = HEAP32[$pTab>>2]|0;
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$zCol>>2]|0;
     HEAP32[$vararg_buffer>>2] = $136;
     $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr2>>2] = $137;
     $138 = (_sqlite3MPrintf($134,29922,$vararg_buffer)|0);
     HEAP32[$zName1>>2] = $138;
     $139 = HEAP32[$v>>2]|0;
     $140 = HEAP32[$i>>2]|0;
     $141 = HEAP32[$zName1>>2]|0;
     (_sqlite3VdbeSetColName($139,$140,0,$141,169)|0);
     break;
    } else {
     $142 = HEAP32[$v>>2]|0;
     $143 = HEAP32[$i>>2]|0;
     $144 = HEAP32[$zCol>>2]|0;
     (_sqlite3VdbeSetColName($142,$143,0,$144,(-1))|0);
     break;
    }
   }
  } while(0);
  $164 = HEAP32[$i>>2]|0;
  $165 = (($164) + 1)|0;
  HEAP32[$i>>2] = $165;
 }
 $166 = HEAP32[$0>>2]|0;
 $167 = HEAP32[$1>>2]|0;
 $168 = HEAP32[$2>>2]|0;
 _generateColumnTypes($166,$167,$168);
 STACKTOP = sp;return;
}
function _sqlite3SrcListEnlarge($db,$pSrc,$nExtra,$iStart) {
 $db = $db|0;
 $pSrc = $pSrc|0;
 $nExtra = $nExtra|0;
 $iStart = $iStart|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $i = 0, $nAlloc = 0, $nGot = 0, $pNew = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $pNew = sp + 8|0;
 $nAlloc = sp + 4|0;
 $nGot = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $nExtra;
 HEAP32[$4>>2] = $iStart;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (($6) + ($7))|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 do {
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$3>>2]|0;
   $16 = (($14) + ($15))|0;
   HEAP32[$nAlloc>>2] = $16;
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$nAlloc>>2]|0;
   $20 = (($19) - 1)|0;
   $21 = ($20*72)|0;
   $22 = (80 + ($21))|0;
   $23 = (_sqlite3DbRealloc($17,$18,$22,0)|0);
   HEAP32[$pNew>>2] = $23;
   $24 = HEAP32[$pNew>>2]|0;
   $25 = ($24|0)==(0|0);
   if (!($25)) {
    $27 = HEAP32[$pNew>>2]|0;
    HEAP32[$2>>2] = $27;
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$pNew>>2]|0;
    $30 = (_sqlite3DbMallocSize($28,$29)|0);
    $31 = (($30) - 80)|0;
    $32 = (($31>>>0) / 72)&-1;
    $33 = (($32) + 1)|0;
    HEAP32[$nGot>>2] = $33;
    $34 = HEAP32[$nGot>>2]|0;
    $35 = HEAP32[$2>>2]|0;
    $36 = ((($35)) + 4|0);
    HEAP32[$36>>2] = $34;
    break;
   }
   $26 = HEAP32[$2>>2]|0;
   HEAP32[$0>>2] = $26;
   $79 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($79|0);
  }
 } while(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) - 1)|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$4>>2]|0;
  $42 = ($40|0)>=($41|0);
  if (!($42)) {
   break;
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  $45 = (($43) + ($44))|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ((($46)) + 8|0);
  $48 = (($47) + (($45*72)|0)|0);
  $49 = HEAP32[$i>>2]|0;
  $50 = HEAP32[$2>>2]|0;
  $51 = ((($50)) + 8|0);
  $52 = (($51) + (($49*72)|0)|0);
  dest=$48; src=$52; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + -1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$3>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + ($55))|0;
 HEAP32[$56>>2] = $58;
 $59 = HEAP32[$4>>2]|0;
 $60 = HEAP32[$2>>2]|0;
 $61 = ((($60)) + 8|0);
 $62 = (($61) + (($59*72)|0)|0);
 $63 = HEAP32[$3>>2]|0;
 $64 = ($63*72)|0;
 _memset(($62|0),0,($64|0))|0;
 $65 = HEAP32[$4>>2]|0;
 HEAP32[$i>>2] = $65;
 while(1) {
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$4>>2]|0;
  $68 = HEAP32[$3>>2]|0;
  $69 = (($67) + ($68))|0;
  $70 = ($66|0)<($69|0);
  if (!($70)) {
   break;
  }
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  $73 = ((($72)) + 8|0);
  $74 = (($73) + (($71*72)|0)|0);
  $75 = ((($74)) + 44|0);
  HEAP32[$75>>2] = -1;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 $78 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $78;
 $79 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _substSelect($db,$p,$iTable,$pEList,$doPrior) {
 $db = $db|0;
 $p = $p|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 $doPrior = $doPrior|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $i = 0, $pItem = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pSrc = sp + 8|0;
 $pItem = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $pEList;
 HEAP32[$4>>2] = $doPrior;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  _substExprList($7,$9,$10,$11);
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 40|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  _substExprList($12,$15,$16,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 48|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _substExprList($18,$21,$22,$23);
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 44|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = (_substExpr($24,$27,$28,$29)|0);
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 44|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = (_substExpr($33,$36,$37,$38)|0);
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 36|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$1>>2]|0;
  $43 = ((($42)) + 32|0);
  $44 = HEAP32[$43>>2]|0;
  HEAP32[$pSrc>>2] = $44;
  $45 = HEAP32[$pSrc>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$i>>2] = $46;
  $47 = HEAP32[$pSrc>>2]|0;
  $48 = ((($47)) + 8|0);
  HEAP32[$pItem>>2] = $48;
  while(1) {
   $49 = HEAP32[$i>>2]|0;
   $50 = ($49|0)>(0);
   if (!($50)) {
    break;
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = HEAP32[$pItem>>2]|0;
   $53 = ((($52)) + 20|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = HEAP32[$2>>2]|0;
   $56 = HEAP32[$3>>2]|0;
   _substSelect($51,$54,$55,$56,1);
   $57 = HEAP32[$pItem>>2]|0;
   $58 = ((($57)) + 36|0);
   $59 = ((($58)) + 1|0);
   $60 = HEAP8[$59>>0]|0;
   $61 = ($60&255) >>> 2;
   $62 = $61 & 1;
   $63 = $62&255;
   $64 = ($63|0)!=(0);
   if ($64) {
    $65 = HEAP32[$0>>2]|0;
    $66 = HEAP32[$pItem>>2]|0;
    $67 = ((($66)) + 64|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP32[$2>>2]|0;
    $70 = HEAP32[$3>>2]|0;
    _substExprList($65,$68,$69,$70);
   }
   $71 = HEAP32[$i>>2]|0;
   $72 = (($71) + -1)|0;
   HEAP32[$i>>2] = $72;
   $73 = HEAP32[$pItem>>2]|0;
   $74 = ((($73)) + 72|0);
   HEAP32[$pItem>>2] = $74;
  }
  $75 = HEAP32[$4>>2]|0;
  $76 = ($75|0)!=(0);
  if (!($76)) {
   label = 9;
   break;
  }
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 52|0);
  $79 = HEAP32[$78>>2]|0;
  HEAP32[$1>>2] = $79;
  $80 = ($79|0)!=(0|0);
  if (!($80)) {
   label = 9;
   break;
  }
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
}
function _substExprList($db,$pList,$iTable,$pEList) {
 $db = $db|0;
 $pList = $pList|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $pEList;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + (($11*20)|0)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = (_substExpr($10,$16,$17,$18)|0);
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  HEAP32[$24>>2] = $19;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _substExpr($db,$pExpr,$iTable,$pEList) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $iTable;
 HEAP32[$4>>2] = $pEList;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(152);
 do {
  if ($10) {
   $11 = HEAP32[$2>>2]|0;
   $12 = ((($11)) + 28|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = ($13|0)==($14|0);
   if ($15) {
    $16 = HEAP32[$2>>2]|0;
    $17 = ((($16)) + 32|0);
    $18 = HEAP16[$17>>1]|0;
    $19 = $18 << 16 >> 16;
    $20 = ($19|0)<(0);
    if ($20) {
     $21 = HEAP32[$2>>2]|0;
     HEAP8[$21>>0] = 101;
     break;
    } else {
     $22 = HEAP32[$1>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = ((($23)) + 32|0);
     $25 = HEAP16[$24>>1]|0;
     $26 = $25 << 16 >> 16;
     $27 = HEAP32[$4>>2]|0;
     $28 = ((($27)) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = (($29) + (($26*20)|0)|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = (_sqlite3ExprDup($22,$31,0)|0);
     HEAP32[$pNew>>2] = $32;
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     _sqlite3ExprDelete($33,$34);
     $35 = HEAP32[$pNew>>2]|0;
     HEAP32[$2>>2] = $35;
     break;
    }
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
 } while(0);
 do {
  if ((label|0) == 8) {
   $36 = HEAP32[$1>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$3>>2]|0;
   $41 = HEAP32[$4>>2]|0;
   $42 = (_substExpr($36,$39,$40,$41)|0);
   $43 = HEAP32[$2>>2]|0;
   $44 = ((($43)) + 12|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = ((($46)) + 16|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   $50 = HEAP32[$4>>2]|0;
   $51 = (_substExpr($45,$48,$49,$50)|0);
   $52 = HEAP32[$2>>2]|0;
   $53 = ((($52)) + 16|0);
   HEAP32[$53>>2] = $51;
   $54 = HEAP32[$2>>2]|0;
   $55 = ((($54)) + 4|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = $56 & 2048;
   $58 = ($57|0)!=(0);
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = ((($60)) + 20|0);
   if ($58) {
    $62 = HEAP32[$61>>2]|0;
    $63 = HEAP32[$3>>2]|0;
    $64 = HEAP32[$4>>2]|0;
    _substSelect($59,$62,$63,$64,1);
    break;
   } else {
    $65 = HEAP32[$61>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    $67 = HEAP32[$4>>2]|0;
    _substExprList($59,$65,$66,$67);
    break;
   }
  }
 } while(0);
 $68 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $68;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _multiSelectValues($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $nRow = 0, $pPrior = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pPrior = sp + 8|0;
 $nRow = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$nRow>>2] = 1;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = ((($3)) + 52|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)==(0|0);
  if ($6) {
   break;
  }
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 52|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$1>>2] = $9;
  $10 = HEAP32[$nRow>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$nRow>>2] = $11;
 }
 while(1) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   label = 7;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 52|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pPrior>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 52|0);
  HEAP32[$18>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = (_sqlite3Select($19,$20,$21)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$pPrior>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 52|0);
  HEAP32[$25>>2] = $23;
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 7;
   break;
  }
  $28 = HEAP32[$nRow>>2]|0;
  $29 = ($28|0)<(0);
  $30 = $29 << 31 >> 31;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 24|0);
  $33 = $32;
  $34 = $33;
  HEAP32[$34>>2] = $28;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = $30;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 56|0);
  $39 = HEAP32[$38>>2]|0;
  HEAP32[$1>>2] = $39;
 }
 if ((label|0) == 7) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 return (0)|0;
}
function _generateWithRecursiveQuery($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrBreak = 0, $addrCont = 0, $addrTop = 0, $destQueue = 0, $eDest = 0, $i = 0, $iCurrent = 0, $iDistinct = 0, $iQueue = 0, $nCol = 0, $pKeyInfo = 0, $pLimit = 0, $pOffset = 0, $pOrderBy = 0, $pSetup = 0;
 var $pSrc = 0, $rc = 0, $regCurrent = 0, $regLimit = 0, $regOffset = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $pSrc = sp + 100|0;
 $nCol = sp + 96|0;
 $v = sp + 92|0;
 $pSetup = sp + 88|0;
 $addrTop = sp + 84|0;
 $addrCont = sp + 80|0;
 $addrBreak = sp + 76|0;
 $iCurrent = sp + 72|0;
 $regCurrent = sp + 68|0;
 $iQueue = sp + 64|0;
 $iDistinct = sp + 60|0;
 $eDest = sp + 56|0;
 $destQueue = sp + 36|0;
 $i = sp + 32|0;
 $rc = sp + 28|0;
 $pOrderBy = sp + 24|0;
 $pLimit = sp + 20|0;
 $pOffset = sp + 16|0;
 $regLimit = sp + 12|0;
 $regOffset = sp + 8|0;
 $pKeyInfo = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDest;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nCol>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pSetup>>2] = $14;
 HEAP32[$iCurrent>>2] = 0;
 HEAP32[$iDistinct>>2] = 0;
 HEAP32[$eDest>>2] = 5;
 $15 = HEAP32[$0>>2]|0;
 $16 = (_sqlite3AuthCheck($15,33,0,0,0)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$v>>2]|0;
 $19 = (_sqlite3VdbeMakeLabel($18)|0);
 HEAP32[$addrBreak>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$addrBreak>>2]|0;
 _computeLimitRegisters($20,$21,$22);
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 60|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pLimit>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 64|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pOffset>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 8|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$regLimit>>2] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$regOffset>>2] = $34;
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 64|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 60|0);
 HEAP32[$38>>2] = 0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 12|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 8|0);
 HEAP32[$42>>2] = 0;
 $43 = HEAP32[$1>>2]|0;
 $44 = ((($43)) + 48|0);
 $45 = HEAP32[$44>>2]|0;
 HEAP32[$pOrderBy>>2] = $45;
 HEAP32[$i>>2] = 0;
 while(1) {
  $46 = HEAP32[$i>>2]|0;
  $47 = HEAP32[$pSrc>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($46|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[$i>>2]|0;
  $51 = HEAP32[$pSrc>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = (($52) + (($50*72)|0)|0);
  $54 = ((($53)) + 36|0);
  $55 = ((($54)) + 1|0);
  $56 = HEAP8[$55>>0]|0;
  $57 = ($56&255) >>> 5;
  $58 = $57 & 1;
  $59 = $58&255;
  $60 = ($59|0)!=(0);
  $61 = HEAP32[$i>>2]|0;
  if ($60) {
   label = 5;
   break;
  }
  $67 = (($61) + 1)|0;
  HEAP32[$i>>2] = $67;
 }
 if ((label|0) == 5) {
  $62 = HEAP32[$pSrc>>2]|0;
  $63 = ((($62)) + 8|0);
  $64 = (($63) + (($61*72)|0)|0);
  $65 = ((($64)) + 44|0);
  $66 = HEAP32[$65>>2]|0;
  HEAP32[$iCurrent>>2] = $66;
 }
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 72|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + 1)|0;
 HEAP32[$69>>2] = $71;
 HEAP32[$iQueue>>2] = $70;
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 4|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 $76 = ($75|0)==(115);
 $77 = HEAP32[$pOrderBy>>2]|0;
 $78 = ($77|0)!=(0|0);
 if ($76) {
  $79 = $78 ? 8 : 6;
  HEAP32[$eDest>>2] = $79;
  $80 = HEAP32[$0>>2]|0;
  $81 = ((($80)) + 72|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$81>>2] = $83;
  HEAP32[$iDistinct>>2] = $82;
 } else {
  $84 = $78 ? 7 : 5;
  HEAP32[$eDest>>2] = $84;
 }
 $85 = HEAP32[$eDest>>2]|0;
 $86 = HEAP32[$iQueue>>2]|0;
 _sqlite3SelectDestInit($destQueue,$85,$86);
 $87 = HEAP32[$0>>2]|0;
 $88 = ((($87)) + 76|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + 1)|0;
 HEAP32[$88>>2] = $90;
 HEAP32[$regCurrent>>2] = $90;
 $91 = HEAP32[$v>>2]|0;
 $92 = HEAP32[$iCurrent>>2]|0;
 $93 = HEAP32[$regCurrent>>2]|0;
 $94 = HEAP32[$nCol>>2]|0;
 (_sqlite3VdbeAddOp3($91,60,$92,$93,$94)|0);
 $95 = HEAP32[$pOrderBy>>2]|0;
 $96 = ($95|0)!=(0|0);
 if ($96) {
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = (_multiSelectOrderByKeyInfo($97,$98,1)|0);
  HEAP32[$pKeyInfo>>2] = $99;
  $100 = HEAP32[$v>>2]|0;
  $101 = HEAP32[$iQueue>>2]|0;
  $102 = HEAP32[$pOrderBy>>2]|0;
  $103 = HEAP32[$102>>2]|0;
  $104 = (($103) + 2)|0;
  $105 = HEAP32[$pKeyInfo>>2]|0;
  (_sqlite3VdbeAddOp4($100,57,$101,$104,0,$105,-6)|0);
  $106 = HEAP32[$pOrderBy>>2]|0;
  $107 = ((($destQueue)) + 16|0);
  HEAP32[$107>>2] = $106;
 } else {
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$iQueue>>2]|0;
  $110 = HEAP32[$nCol>>2]|0;
  (_sqlite3VdbeAddOp2($108,57,$109,$110)|0);
 }
 $111 = HEAP32[$iDistinct>>2]|0;
 $112 = ($111|0)!=(0);
 if ($112) {
  $113 = HEAP32[$v>>2]|0;
  $114 = HEAP32[$iDistinct>>2]|0;
  $115 = (_sqlite3VdbeAddOp2($113,57,$114,0)|0);
  $116 = HEAP32[$1>>2]|0;
  $117 = ((($116)) + 16|0);
  HEAP32[$117>>2] = $115;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 6|0);
  $120 = HEAP16[$119>>1]|0;
  $121 = $120&65535;
  $122 = $121 | 16;
  $123 = $122&65535;
  HEAP16[$119>>1] = $123;
 }
 $124 = HEAP32[$1>>2]|0;
 $125 = ((($124)) + 48|0);
 HEAP32[$125>>2] = 0;
 $126 = HEAP32[$pSetup>>2]|0;
 $127 = ((($126)) + 56|0);
 HEAP32[$127>>2] = 0;
 $128 = HEAP32[$0>>2]|0;
 $129 = HEAP32[$pSetup>>2]|0;
 $130 = (_sqlite3Select($128,$129,$destQueue)|0);
 HEAP32[$rc>>2] = $130;
 $131 = HEAP32[$1>>2]|0;
 $132 = HEAP32[$pSetup>>2]|0;
 $133 = ((($132)) + 56|0);
 HEAP32[$133>>2] = $131;
 $134 = HEAP32[$rc>>2]|0;
 $135 = ($134|0)!=(0);
 if (!($135)) {
  $136 = HEAP32[$v>>2]|0;
  $137 = HEAP32[$iQueue>>2]|0;
  $138 = HEAP32[$addrBreak>>2]|0;
  $139 = (_sqlite3VdbeAddOp2($136,108,$137,$138)|0);
  HEAP32[$addrTop>>2] = $139;
  $140 = HEAP32[$v>>2]|0;
  $141 = HEAP32[$iCurrent>>2]|0;
  (_sqlite3VdbeAddOp1($140,104,$141)|0);
  $142 = HEAP32[$pOrderBy>>2]|0;
  $143 = ($142|0)!=(0|0);
  $144 = HEAP32[$v>>2]|0;
  $145 = HEAP32[$iQueue>>2]|0;
  if ($143) {
   $146 = HEAP32[$pOrderBy>>2]|0;
   $147 = HEAP32[$146>>2]|0;
   $148 = (($147) + 1)|0;
   $149 = HEAP32[$regCurrent>>2]|0;
   (_sqlite3VdbeAddOp3($144,47,$145,$148,$149)|0);
  } else {
   $150 = HEAP32[$regCurrent>>2]|0;
   (_sqlite3VdbeAddOp2($144,102,$145,$150)|0);
  }
  $151 = HEAP32[$v>>2]|0;
  $152 = HEAP32[$iQueue>>2]|0;
  (_sqlite3VdbeAddOp1($151,95,$152)|0);
  $153 = HEAP32[$v>>2]|0;
  $154 = (_sqlite3VdbeMakeLabel($153)|0);
  HEAP32[$addrCont>>2] = $154;
  $155 = HEAP32[$v>>2]|0;
  $156 = HEAP32[$regOffset>>2]|0;
  $157 = HEAP32[$addrCont>>2]|0;
  _codeOffset($155,$156,$157);
  $158 = HEAP32[$0>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$160>>2]|0;
  $162 = HEAP32[$iCurrent>>2]|0;
  $163 = HEAP32[$2>>2]|0;
  $164 = HEAP32[$addrCont>>2]|0;
  $165 = HEAP32[$addrBreak>>2]|0;
  _selectInnerLoop($158,$159,$161,$162,0,0,$163,$164,$165);
  $166 = HEAP32[$regLimit>>2]|0;
  $167 = ($166|0)!=(0);
  if ($167) {
   $168 = HEAP32[$v>>2]|0;
   $169 = HEAP32[$regLimit>>2]|0;
   $170 = HEAP32[$addrBreak>>2]|0;
   (_sqlite3VdbeAddOp2($168,141,$169,$170)|0);
  }
  $171 = HEAP32[$v>>2]|0;
  $172 = HEAP32[$addrCont>>2]|0;
  _sqlite3VdbeResolveLabel($171,$172);
  $173 = HEAP32[$1>>2]|0;
  $174 = ((($173)) + 6|0);
  $175 = HEAP16[$174>>1]|0;
  $176 = $175&65535;
  $177 = $176 & 8;
  $178 = ($177|0)!=(0);
  if ($178) {
   $179 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($179,32394,$vararg_buffer);
  } else {
   $180 = HEAP32[$1>>2]|0;
   $181 = ((($180)) + 52|0);
   HEAP32[$181>>2] = 0;
   $182 = HEAP32[$0>>2]|0;
   $183 = HEAP32[$1>>2]|0;
   (_sqlite3Select($182,$183,$destQueue)|0);
   $184 = HEAP32[$pSetup>>2]|0;
   $185 = HEAP32[$1>>2]|0;
   $186 = ((($185)) + 52|0);
   HEAP32[$186>>2] = $184;
  }
  $187 = HEAP32[$v>>2]|0;
  $188 = HEAP32[$addrTop>>2]|0;
  (_sqlite3VdbeGoto($187,$188)|0);
  $189 = HEAP32[$v>>2]|0;
  $190 = HEAP32[$addrBreak>>2]|0;
  _sqlite3VdbeResolveLabel($189,$190);
 }
 $191 = HEAP32[$0>>2]|0;
 $192 = HEAP32[$191>>2]|0;
 $193 = HEAP32[$1>>2]|0;
 $194 = ((($193)) + 48|0);
 $195 = HEAP32[$194>>2]|0;
 _sqlite3ExprListDelete($192,$195);
 $196 = HEAP32[$pOrderBy>>2]|0;
 $197 = HEAP32[$1>>2]|0;
 $198 = ((($197)) + 48|0);
 HEAP32[$198>>2] = $196;
 $199 = HEAP32[$pLimit>>2]|0;
 $200 = HEAP32[$1>>2]|0;
 $201 = ((($200)) + 60|0);
 HEAP32[$201>>2] = $199;
 $202 = HEAP32[$pOffset>>2]|0;
 $203 = HEAP32[$1>>2]|0;
 $204 = ((($203)) + 64|0);
 HEAP32[$204>>2] = $202;
 STACKTOP = sp;return;
}
function _multiSelectOrderBy($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aPermute = 0, $addr1 = 0, $addrAeqB = 0, $addrAgtB = 0, $addrAltB = 0, $addrEofA = 0, $addrEofA_noB = 0, $addrEofB = 0, $addrOutA = 0, $addrOutB = 0, $addrSelectA = 0, $addrSelectB = 0, $db = 0;
 var $destA = 0, $destB = 0, $i = 0, $iSub1 = 0, $iSub2 = 0, $j = 0, $labelCmpr = 0, $labelEnd = 0, $nExpr = 0, $nOrderBy = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pFirst = 0, $pItem = 0, $pItem1 = 0, $pKeyDup = 0, $pKeyMerge = 0;
 var $pNew = 0, $pOrderBy = 0, $pPrior = 0, $regAddrA = 0, $regAddrB = 0, $regLimitA = 0, $regLimitB = 0, $regOutA = 0, $regOutB = 0, $regPrev = 0, $savedLimit = 0, $savedOffset = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 212|0;
 $1 = sp + 208|0;
 $2 = sp + 204|0;
 $3 = sp + 200|0;
 $i = sp + 196|0;
 $j = sp + 192|0;
 $pPrior = sp + 188|0;
 $v = sp + 184|0;
 $destA = sp + 164|0;
 $destB = sp + 144|0;
 $regAddrA = sp + 140|0;
 $regAddrB = sp + 136|0;
 $addrSelectA = sp + 132|0;
 $addrSelectB = sp + 128|0;
 $regOutA = sp + 124|0;
 $regOutB = sp + 120|0;
 $addrOutA = sp + 116|0;
 $addrOutB = sp + 112|0;
 $addrEofA = sp + 108|0;
 $addrEofA_noB = sp + 104|0;
 $addrEofB = sp + 100|0;
 $addrAltB = sp + 96|0;
 $addrAeqB = sp + 92|0;
 $addrAgtB = sp + 88|0;
 $regLimitA = sp + 84|0;
 $regLimitB = sp + 80|0;
 $regPrev = sp + 76|0;
 $savedLimit = sp + 72|0;
 $savedOffset = sp + 68|0;
 $labelCmpr = sp + 64|0;
 $labelEnd = sp + 60|0;
 $addr1 = sp + 56|0;
 $op = sp + 52|0;
 $pKeyDup = sp + 48|0;
 $pKeyMerge = sp + 44|0;
 $db = sp + 40|0;
 $pOrderBy = sp + 36|0;
 $nOrderBy = sp + 32|0;
 $aPermute = sp + 28|0;
 $iSub1 = sp + 24|0;
 $iSub2 = sp + 20|0;
 $pItem = sp + 16|0;
 $pNew = sp + 12|0;
 $pItem1 = sp + 8|0;
 $nExpr = sp + 4|0;
 $pFirst = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$addrOutB>>2] = 0;
 HEAP32[$pKeyDup>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = (_sqlite3VdbeMakeLabel($9)|0);
 HEAP32[$labelEnd>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = (_sqlite3VdbeMakeLabel($11)|0);
 HEAP32[$labelCmpr>>2] = $12;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 HEAP32[$op>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pPrior>>2] = $19;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pOrderBy>>2] = $22;
 $23 = HEAP32[$pOrderBy>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$nOrderBy>>2] = $24;
 $25 = HEAP32[$op>>2]|0;
 $26 = ($25|0)!=(116);
 L1: do {
  if ($26) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $27 = HEAP32[$db>>2]|0;
    $28 = ((($27)) + 65|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(0);
    if (!($31)) {
     break L1;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    $36 = ($32|0)<=($35|0);
    if (!($36)) {
     break L1;
    }
    HEAP32[$j>>2] = 0;
    $37 = HEAP32[$pOrderBy>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    HEAP32[$pItem>>2] = $39;
    while(1) {
     $40 = HEAP32[$j>>2]|0;
     $41 = HEAP32[$nOrderBy>>2]|0;
     $42 = ($40|0)<($41|0);
     if (!($42)) {
      break;
     }
     $43 = HEAP32[$pItem>>2]|0;
     $44 = ((($43)) + 16|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = $45&65535;
     $47 = HEAP32[$i>>2]|0;
     $48 = ($46|0)==($47|0);
     if ($48) {
      break;
     }
     $49 = HEAP32[$j>>2]|0;
     $50 = (($49) + 1)|0;
     HEAP32[$j>>2] = $50;
     $51 = HEAP32[$pItem>>2]|0;
     $52 = ((($51)) + 20|0);
     HEAP32[$pItem>>2] = $52;
    }
    $53 = HEAP32[$j>>2]|0;
    $54 = HEAP32[$nOrderBy>>2]|0;
    $55 = ($53|0)==($54|0);
    if ($55) {
     $56 = HEAP32[$db>>2]|0;
     $57 = (_sqlite3Expr($56,132,0)|0);
     HEAP32[$pNew>>2] = $57;
     $58 = HEAP32[$pNew>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      break;
     }
     $60 = HEAP32[$pNew>>2]|0;
     $61 = ((($60)) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = $62 | 1024;
     HEAP32[$61>>2] = $63;
     $64 = HEAP32[$i>>2]|0;
     $65 = HEAP32[$pNew>>2]|0;
     $66 = ((($65)) + 8|0);
     HEAP32[$66>>2] = $64;
     $67 = HEAP32[$1>>2]|0;
     $68 = HEAP32[$pOrderBy>>2]|0;
     $69 = HEAP32[$pNew>>2]|0;
     $70 = (_sqlite3ExprListAppend($67,$68,$69)|0);
     HEAP32[$pOrderBy>>2] = $70;
     $71 = HEAP32[$pOrderBy>>2]|0;
     $72 = ($71|0)!=(0|0);
     if ($72) {
      $73 = HEAP32[$i>>2]|0;
      $74 = $73&65535;
      $75 = HEAP32[$nOrderBy>>2]|0;
      $76 = (($75) + 1)|0;
      HEAP32[$nOrderBy>>2] = $76;
      $77 = HEAP32[$pOrderBy>>2]|0;
      $78 = ((($77)) + 4|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = (($79) + (($75*20)|0)|0);
      $81 = ((($80)) + 16|0);
      HEAP16[$81>>1] = $74;
     }
    }
    $82 = HEAP32[$i>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[$i>>2] = $83;
   }
   HEAP32[$0>>2] = 7;
   $506 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($506|0);
  }
 } while(0);
 $84 = HEAP32[$db>>2]|0;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = (($85) + 1)|0;
 $87 = $86<<2;
 $88 = (_sqlite3DbMallocRawNN($84,$87,0)|0);
 HEAP32[$aPermute>>2] = $88;
 $89 = HEAP32[$aPermute>>2]|0;
 $90 = ($89|0)!=(0|0);
 if ($90) {
  $91 = HEAP32[$nOrderBy>>2]|0;
  $92 = HEAP32[$aPermute>>2]|0;
  HEAP32[$92>>2] = $91;
  HEAP32[$i>>2] = 1;
  $93 = HEAP32[$pOrderBy>>2]|0;
  $94 = ((($93)) + 4|0);
  $95 = HEAP32[$94>>2]|0;
  HEAP32[$pItem1>>2] = $95;
  while(1) {
   $96 = HEAP32[$i>>2]|0;
   $97 = HEAP32[$nOrderBy>>2]|0;
   $98 = ($96|0)<=($97|0);
   if (!($98)) {
    break;
   }
   $99 = HEAP32[$pItem1>>2]|0;
   $100 = ((($99)) + 16|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = $101&65535;
   $103 = (($102) - 1)|0;
   $104 = HEAP32[$i>>2]|0;
   $105 = HEAP32[$aPermute>>2]|0;
   $106 = (($105) + ($104<<2)|0);
   HEAP32[$106>>2] = $103;
   $107 = HEAP32[$i>>2]|0;
   $108 = (($107) + 1)|0;
   HEAP32[$i>>2] = $108;
   $109 = HEAP32[$pItem1>>2]|0;
   $110 = ((($109)) + 20|0);
   HEAP32[$pItem1>>2] = $110;
  }
  $111 = HEAP32[$1>>2]|0;
  $112 = HEAP32[$2>>2]|0;
  $113 = (_multiSelectOrderByKeyInfo($111,$112,1)|0);
  HEAP32[$pKeyMerge>>2] = $113;
 } else {
  HEAP32[$pKeyMerge>>2] = 0;
 }
 $114 = HEAP32[$pOrderBy>>2]|0;
 $115 = HEAP32[$2>>2]|0;
 $116 = ((($115)) + 48|0);
 HEAP32[$116>>2] = $114;
 $117 = HEAP32[$1>>2]|0;
 $118 = HEAP32[$117>>2]|0;
 $119 = HEAP32[$pOrderBy>>2]|0;
 $120 = (_sqlite3ExprListDup($118,$119,0)|0);
 $121 = HEAP32[$pPrior>>2]|0;
 $122 = ((($121)) + 48|0);
 HEAP32[$122>>2] = $120;
 $123 = HEAP32[$op>>2]|0;
 $124 = ($123|0)==(116);
 L28: do {
  if ($124) {
   HEAP32[$regPrev>>2] = 0;
  } else {
   $125 = HEAP32[$2>>2]|0;
   $126 = HEAP32[$125>>2]|0;
   $127 = HEAP32[$126>>2]|0;
   HEAP32[$nExpr>>2] = $127;
   $128 = HEAP32[$1>>2]|0;
   $129 = ((($128)) + 76|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = (($130) + 1)|0;
   HEAP32[$regPrev>>2] = $131;
   $132 = HEAP32[$nExpr>>2]|0;
   $133 = (($132) + 1)|0;
   $134 = HEAP32[$1>>2]|0;
   $135 = ((($134)) + 76|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($136) + ($133))|0;
   HEAP32[$135>>2] = $137;
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$regPrev>>2]|0;
   (_sqlite3VdbeAddOp2($138,22,0,$139)|0);
   $140 = HEAP32[$db>>2]|0;
   $141 = HEAP32[$nExpr>>2]|0;
   $142 = (_sqlite3KeyInfoAlloc($140,$141,1)|0);
   HEAP32[$pKeyDup>>2] = $142;
   $143 = HEAP32[$pKeyDup>>2]|0;
   $144 = ($143|0)!=(0|0);
   if ($144) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $145 = HEAP32[$i>>2]|0;
     $146 = HEAP32[$nExpr>>2]|0;
     $147 = ($145|0)<($146|0);
     if (!($147)) {
      break L28;
     }
     $148 = HEAP32[$1>>2]|0;
     $149 = HEAP32[$2>>2]|0;
     $150 = HEAP32[$i>>2]|0;
     $151 = (_multiSelectCollSeq($148,$149,$150)|0);
     $152 = HEAP32[$i>>2]|0;
     $153 = HEAP32[$pKeyDup>>2]|0;
     $154 = ((($153)) + 20|0);
     $155 = (($154) + ($152<<2)|0);
     HEAP32[$155>>2] = $151;
     $156 = HEAP32[$i>>2]|0;
     $157 = HEAP32[$pKeyDup>>2]|0;
     $158 = ((($157)) + 16|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = (($159) + ($156)|0);
     HEAP8[$160>>0] = 0;
     $161 = HEAP32[$i>>2]|0;
     $162 = (($161) + 1)|0;
     HEAP32[$i>>2] = $162;
    }
   }
  }
 } while(0);
 $163 = HEAP32[$2>>2]|0;
 $164 = ((($163)) + 52|0);
 HEAP32[$164>>2] = 0;
 $165 = HEAP32[$pPrior>>2]|0;
 $166 = ((($165)) + 56|0);
 HEAP32[$166>>2] = 0;
 $167 = HEAP32[$1>>2]|0;
 $168 = HEAP32[$2>>2]|0;
 $169 = HEAP32[$2>>2]|0;
 $170 = ((($169)) + 48|0);
 $171 = HEAP32[$170>>2]|0;
 (_sqlite3ResolveOrderGroupBy($167,$168,$171,29797)|0);
 $172 = HEAP32[$pPrior>>2]|0;
 $173 = ((($172)) + 52|0);
 $174 = HEAP32[$173>>2]|0;
 $175 = ($174|0)==(0|0);
 if ($175) {
  $176 = HEAP32[$1>>2]|0;
  $177 = HEAP32[$pPrior>>2]|0;
  $178 = HEAP32[$pPrior>>2]|0;
  $179 = ((($178)) + 48|0);
  $180 = HEAP32[$179>>2]|0;
  (_sqlite3ResolveOrderGroupBy($176,$177,$180,29797)|0);
 }
 $181 = HEAP32[$1>>2]|0;
 $182 = HEAP32[$2>>2]|0;
 $183 = HEAP32[$labelEnd>>2]|0;
 _computeLimitRegisters($181,$182,$183);
 $184 = HEAP32[$2>>2]|0;
 $185 = ((($184)) + 8|0);
 $186 = HEAP32[$185>>2]|0;
 $187 = ($186|0)!=(0);
 $188 = HEAP32[$op>>2]|0;
 $189 = ($188|0)==(116);
 $or$cond = $187 & $189;
 if ($or$cond) {
  $190 = HEAP32[$1>>2]|0;
  $191 = ((($190)) + 76|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = (($192) + 1)|0;
  HEAP32[$191>>2] = $193;
  HEAP32[$regLimitA>>2] = $193;
  $194 = HEAP32[$1>>2]|0;
  $195 = ((($194)) + 76|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = (($196) + 1)|0;
  HEAP32[$195>>2] = $197;
  HEAP32[$regLimitB>>2] = $197;
  $198 = HEAP32[$v>>2]|0;
  $199 = HEAP32[$2>>2]|0;
  $200 = ((($199)) + 12|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = ($201|0)!=(0);
  $203 = HEAP32[$2>>2]|0;
  if ($202) {
   $204 = ((($203)) + 12|0);
   $205 = HEAP32[$204>>2]|0;
   $206 = (($205) + 1)|0;
   $210 = $206;
  } else {
   $207 = ((($203)) + 8|0);
   $208 = HEAP32[$207>>2]|0;
   $210 = $208;
  }
  $209 = HEAP32[$regLimitA>>2]|0;
  (_sqlite3VdbeAddOp2($198,30,$210,$209)|0);
  $211 = HEAP32[$v>>2]|0;
  $212 = HEAP32[$regLimitA>>2]|0;
  $213 = HEAP32[$regLimitB>>2]|0;
  (_sqlite3VdbeAddOp2($211,30,$212,$213)|0);
 } else {
  HEAP32[$regLimitB>>2] = 0;
  HEAP32[$regLimitA>>2] = 0;
 }
 $214 = HEAP32[$db>>2]|0;
 $215 = HEAP32[$2>>2]|0;
 $216 = ((($215)) + 60|0);
 $217 = HEAP32[$216>>2]|0;
 _sqlite3ExprDelete($214,$217);
 $218 = HEAP32[$2>>2]|0;
 $219 = ((($218)) + 60|0);
 HEAP32[$219>>2] = 0;
 $220 = HEAP32[$db>>2]|0;
 $221 = HEAP32[$2>>2]|0;
 $222 = ((($221)) + 64|0);
 $223 = HEAP32[$222>>2]|0;
 _sqlite3ExprDelete($220,$223);
 $224 = HEAP32[$2>>2]|0;
 $225 = ((($224)) + 64|0);
 HEAP32[$225>>2] = 0;
 $226 = HEAP32[$1>>2]|0;
 $227 = ((($226)) + 76|0);
 $228 = HEAP32[$227>>2]|0;
 $229 = (($228) + 1)|0;
 HEAP32[$227>>2] = $229;
 HEAP32[$regAddrA>>2] = $229;
 $230 = HEAP32[$1>>2]|0;
 $231 = ((($230)) + 76|0);
 $232 = HEAP32[$231>>2]|0;
 $233 = (($232) + 1)|0;
 HEAP32[$231>>2] = $233;
 HEAP32[$regAddrB>>2] = $233;
 $234 = HEAP32[$1>>2]|0;
 $235 = ((($234)) + 76|0);
 $236 = HEAP32[$235>>2]|0;
 $237 = (($236) + 1)|0;
 HEAP32[$235>>2] = $237;
 HEAP32[$regOutA>>2] = $237;
 $238 = HEAP32[$1>>2]|0;
 $239 = ((($238)) + 76|0);
 $240 = HEAP32[$239>>2]|0;
 $241 = (($240) + 1)|0;
 HEAP32[$239>>2] = $241;
 HEAP32[$regOutB>>2] = $241;
 $242 = HEAP32[$regAddrA>>2]|0;
 _sqlite3SelectDestInit($destA,13,$242);
 $243 = HEAP32[$regAddrB>>2]|0;
 _sqlite3SelectDestInit($destB,13,$243);
 $244 = HEAP32[$v>>2]|0;
 $245 = (_sqlite3VdbeCurrentAddr($244)|0);
 $246 = (($245) + 1)|0;
 HEAP32[$addrSelectA>>2] = $246;
 $247 = HEAP32[$v>>2]|0;
 $248 = HEAP32[$regAddrA>>2]|0;
 $249 = HEAP32[$addrSelectA>>2]|0;
 $250 = (_sqlite3VdbeAddOp3($247,16,$248,0,$249)|0);
 HEAP32[$addr1>>2] = $250;
 $251 = HEAP32[$regLimitA>>2]|0;
 $252 = HEAP32[$pPrior>>2]|0;
 $253 = ((($252)) + 8|0);
 HEAP32[$253>>2] = $251;
 $254 = HEAP32[$1>>2]|0;
 $255 = ((($254)) + 472|0);
 $256 = HEAP32[$255>>2]|0;
 HEAP32[$iSub1>>2] = $256;
 $257 = HEAP32[$1>>2]|0;
 $258 = HEAP32[$pPrior>>2]|0;
 (_sqlite3Select($257,$258,$destA)|0);
 $259 = HEAP32[$v>>2]|0;
 $260 = HEAP32[$regAddrA>>2]|0;
 _sqlite3VdbeEndCoroutine($259,$260);
 $261 = HEAP32[$v>>2]|0;
 $262 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($261,$262);
 $263 = HEAP32[$v>>2]|0;
 $264 = (_sqlite3VdbeCurrentAddr($263)|0);
 $265 = (($264) + 1)|0;
 HEAP32[$addrSelectB>>2] = $265;
 $266 = HEAP32[$v>>2]|0;
 $267 = HEAP32[$regAddrB>>2]|0;
 $268 = HEAP32[$addrSelectB>>2]|0;
 $269 = (_sqlite3VdbeAddOp3($266,16,$267,0,$268)|0);
 HEAP32[$addr1>>2] = $269;
 $270 = HEAP32[$2>>2]|0;
 $271 = ((($270)) + 8|0);
 $272 = HEAP32[$271>>2]|0;
 HEAP32[$savedLimit>>2] = $272;
 $273 = HEAP32[$2>>2]|0;
 $274 = ((($273)) + 12|0);
 $275 = HEAP32[$274>>2]|0;
 HEAP32[$savedOffset>>2] = $275;
 $276 = HEAP32[$regLimitB>>2]|0;
 $277 = HEAP32[$2>>2]|0;
 $278 = ((($277)) + 8|0);
 HEAP32[$278>>2] = $276;
 $279 = HEAP32[$2>>2]|0;
 $280 = ((($279)) + 12|0);
 HEAP32[$280>>2] = 0;
 $281 = HEAP32[$1>>2]|0;
 $282 = ((($281)) + 472|0);
 $283 = HEAP32[$282>>2]|0;
 HEAP32[$iSub2>>2] = $283;
 $284 = HEAP32[$1>>2]|0;
 $285 = HEAP32[$2>>2]|0;
 (_sqlite3Select($284,$285,$destB)|0);
 $286 = HEAP32[$savedLimit>>2]|0;
 $287 = HEAP32[$2>>2]|0;
 $288 = ((($287)) + 8|0);
 HEAP32[$288>>2] = $286;
 $289 = HEAP32[$savedOffset>>2]|0;
 $290 = HEAP32[$2>>2]|0;
 $291 = ((($290)) + 12|0);
 HEAP32[$291>>2] = $289;
 $292 = HEAP32[$v>>2]|0;
 $293 = HEAP32[$regAddrB>>2]|0;
 _sqlite3VdbeEndCoroutine($292,$293);
 $294 = HEAP32[$1>>2]|0;
 $295 = HEAP32[$2>>2]|0;
 $296 = HEAP32[$3>>2]|0;
 $297 = HEAP32[$regOutA>>2]|0;
 $298 = HEAP32[$regPrev>>2]|0;
 $299 = HEAP32[$pKeyDup>>2]|0;
 $300 = HEAP32[$labelEnd>>2]|0;
 $301 = (_generateOutputSubroutine($294,$295,$destA,$296,$297,$298,$299,$300)|0);
 HEAP32[$addrOutA>>2] = $301;
 $302 = HEAP32[$op>>2]|0;
 $303 = ($302|0)==(116);
 $304 = HEAP32[$op>>2]|0;
 $305 = ($304|0)==(115);
 $or$cond3 = $303 | $305;
 if ($or$cond3) {
  $306 = HEAP32[$1>>2]|0;
  $307 = HEAP32[$2>>2]|0;
  $308 = HEAP32[$3>>2]|0;
  $309 = HEAP32[$regOutB>>2]|0;
  $310 = HEAP32[$regPrev>>2]|0;
  $311 = HEAP32[$pKeyDup>>2]|0;
  $312 = HEAP32[$labelEnd>>2]|0;
  $313 = (_generateOutputSubroutine($306,$307,$destB,$308,$309,$310,$311,$312)|0);
  HEAP32[$addrOutB>>2] = $313;
 }
 $314 = HEAP32[$pKeyDup>>2]|0;
 _sqlite3KeyInfoUnref($314);
 $315 = HEAP32[$op>>2]|0;
 $316 = ($315|0)==(117);
 $317 = HEAP32[$op>>2]|0;
 $318 = ($317|0)==(118);
 $or$cond5 = $316 | $318;
 if ($or$cond5) {
  $319 = HEAP32[$labelEnd>>2]|0;
  HEAP32[$addrEofA>>2] = $319;
  HEAP32[$addrEofA_noB>>2] = $319;
 } else {
  $320 = HEAP32[$v>>2]|0;
  $321 = HEAP32[$regOutB>>2]|0;
  $322 = HEAP32[$addrOutB>>2]|0;
  $323 = (_sqlite3VdbeAddOp2($320,14,$321,$322)|0);
  HEAP32[$addrEofA>>2] = $323;
  $324 = HEAP32[$v>>2]|0;
  $325 = HEAP32[$regAddrB>>2]|0;
  $326 = HEAP32[$labelEnd>>2]|0;
  $327 = (_sqlite3VdbeAddOp2($324,18,$325,$326)|0);
  HEAP32[$addrEofA_noB>>2] = $327;
  $328 = HEAP32[$v>>2]|0;
  $329 = HEAP32[$addrEofA>>2]|0;
  (_sqlite3VdbeGoto($328,$329)|0);
  $330 = HEAP32[$pPrior>>2]|0;
  $331 = ((($330)) + 24|0);
  $332 = $331;
  $333 = $332;
  $334 = HEAP32[$333>>2]|0;
  $335 = (($332) + 4)|0;
  $336 = $335;
  $337 = HEAP32[$336>>2]|0;
  $338 = HEAP32[$2>>2]|0;
  $339 = ((($338)) + 24|0);
  $340 = $339;
  $341 = $340;
  $342 = HEAP32[$341>>2]|0;
  $343 = (($340) + 4)|0;
  $344 = $343;
  $345 = HEAP32[$344>>2]|0;
  $346 = (_i64Add(($342|0),($345|0),($334|0),($337|0))|0);
  $347 = tempRet0;
  $348 = $339;
  $349 = $348;
  HEAP32[$349>>2] = $346;
  $350 = (($348) + 4)|0;
  $351 = $350;
  HEAP32[$351>>2] = $347;
 }
 $352 = HEAP32[$op>>2]|0;
 $353 = ($352|0)==(118);
 if ($353) {
  $354 = HEAP32[$addrEofA>>2]|0;
  HEAP32[$addrEofB>>2] = $354;
  $355 = HEAP32[$2>>2]|0;
  $356 = ((($355)) + 24|0);
  $357 = $356;
  $358 = $357;
  $359 = HEAP32[$358>>2]|0;
  $360 = (($357) + 4)|0;
  $361 = $360;
  $362 = HEAP32[$361>>2]|0;
  $363 = HEAP32[$pPrior>>2]|0;
  $364 = ((($363)) + 24|0);
  $365 = $364;
  $366 = $365;
  $367 = HEAP32[$366>>2]|0;
  $368 = (($365) + 4)|0;
  $369 = $368;
  $370 = HEAP32[$369>>2]|0;
  $371 = ($362>>>0)>($370>>>0);
  $372 = ($359>>>0)>($367>>>0);
  $373 = ($362|0)==($370|0);
  $374 = $373 & $372;
  $375 = $371 | $374;
  if ($375) {
   $376 = HEAP32[$pPrior>>2]|0;
   $377 = ((($376)) + 24|0);
   $378 = $377;
   $379 = $378;
   $380 = HEAP32[$379>>2]|0;
   $381 = (($378) + 4)|0;
   $382 = $381;
   $383 = HEAP32[$382>>2]|0;
   $384 = HEAP32[$2>>2]|0;
   $385 = ((($384)) + 24|0);
   $386 = $385;
   $387 = $386;
   HEAP32[$387>>2] = $380;
   $388 = (($386) + 4)|0;
   $389 = $388;
   HEAP32[$389>>2] = $383;
  }
 } else {
  $390 = HEAP32[$v>>2]|0;
  $391 = HEAP32[$regOutA>>2]|0;
  $392 = HEAP32[$addrOutA>>2]|0;
  $393 = (_sqlite3VdbeAddOp2($390,14,$391,$392)|0);
  HEAP32[$addrEofB>>2] = $393;
  $394 = HEAP32[$v>>2]|0;
  $395 = HEAP32[$regAddrA>>2]|0;
  $396 = HEAP32[$labelEnd>>2]|0;
  (_sqlite3VdbeAddOp2($394,18,$395,$396)|0);
  $397 = HEAP32[$v>>2]|0;
  $398 = HEAP32[$addrEofB>>2]|0;
  (_sqlite3VdbeGoto($397,$398)|0);
 }
 $399 = HEAP32[$v>>2]|0;
 $400 = HEAP32[$regOutA>>2]|0;
 $401 = HEAP32[$addrOutA>>2]|0;
 $402 = (_sqlite3VdbeAddOp2($399,14,$400,$401)|0);
 HEAP32[$addrAltB>>2] = $402;
 $403 = HEAP32[$v>>2]|0;
 $404 = HEAP32[$regAddrA>>2]|0;
 $405 = HEAP32[$addrEofA>>2]|0;
 (_sqlite3VdbeAddOp2($403,18,$404,$405)|0);
 $406 = HEAP32[$v>>2]|0;
 $407 = HEAP32[$labelCmpr>>2]|0;
 (_sqlite3VdbeGoto($406,$407)|0);
 $408 = HEAP32[$op>>2]|0;
 $409 = ($408|0)==(116);
 do {
  if ($409) {
   $410 = HEAP32[$addrAltB>>2]|0;
   HEAP32[$addrAeqB>>2] = $410;
  } else {
   $411 = HEAP32[$op>>2]|0;
   $412 = ($411|0)==(118);
   if ($412) {
    $413 = HEAP32[$addrAltB>>2]|0;
    HEAP32[$addrAeqB>>2] = $413;
    $414 = HEAP32[$addrAltB>>2]|0;
    $415 = (($414) + 1)|0;
    HEAP32[$addrAltB>>2] = $415;
    break;
   } else {
    $416 = HEAP32[$v>>2]|0;
    $417 = HEAP32[$regAddrA>>2]|0;
    $418 = HEAP32[$addrEofA>>2]|0;
    $419 = (_sqlite3VdbeAddOp2($416,18,$417,$418)|0);
    HEAP32[$addrAeqB>>2] = $419;
    $420 = HEAP32[$v>>2]|0;
    $421 = HEAP32[$labelCmpr>>2]|0;
    (_sqlite3VdbeGoto($420,$421)|0);
    break;
   }
  }
 } while(0);
 $422 = HEAP32[$v>>2]|0;
 $423 = (_sqlite3VdbeCurrentAddr($422)|0);
 HEAP32[$addrAgtB>>2] = $423;
 $424 = HEAP32[$op>>2]|0;
 $425 = ($424|0)==(116);
 $426 = HEAP32[$op>>2]|0;
 $427 = ($426|0)==(115);
 $or$cond7 = $425 | $427;
 if ($or$cond7) {
  $428 = HEAP32[$v>>2]|0;
  $429 = HEAP32[$regOutB>>2]|0;
  $430 = HEAP32[$addrOutB>>2]|0;
  (_sqlite3VdbeAddOp2($428,14,$429,$430)|0);
 }
 $431 = HEAP32[$v>>2]|0;
 $432 = HEAP32[$regAddrB>>2]|0;
 $433 = HEAP32[$addrEofB>>2]|0;
 (_sqlite3VdbeAddOp2($431,18,$432,$433)|0);
 $434 = HEAP32[$v>>2]|0;
 $435 = HEAP32[$labelCmpr>>2]|0;
 (_sqlite3VdbeGoto($434,$435)|0);
 $436 = HEAP32[$v>>2]|0;
 $437 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($436,$437);
 $438 = HEAP32[$v>>2]|0;
 $439 = HEAP32[$regAddrA>>2]|0;
 $440 = HEAP32[$addrEofA_noB>>2]|0;
 (_sqlite3VdbeAddOp2($438,18,$439,$440)|0);
 $441 = HEAP32[$v>>2]|0;
 $442 = HEAP32[$regAddrB>>2]|0;
 $443 = HEAP32[$addrEofB>>2]|0;
 (_sqlite3VdbeAddOp2($441,18,$442,$443)|0);
 $444 = HEAP32[$v>>2]|0;
 $445 = HEAP32[$labelCmpr>>2]|0;
 _sqlite3VdbeResolveLabel($444,$445);
 $446 = HEAP32[$v>>2]|0;
 $447 = HEAP32[$aPermute>>2]|0;
 (_sqlite3VdbeAddOp4($446,41,0,0,0,$447,-15)|0);
 $448 = HEAP32[$v>>2]|0;
 $449 = ((($destA)) + 8|0);
 $450 = HEAP32[$449>>2]|0;
 $451 = ((($destB)) + 8|0);
 $452 = HEAP32[$451>>2]|0;
 $453 = HEAP32[$nOrderBy>>2]|0;
 $454 = HEAP32[$pKeyMerge>>2]|0;
 (_sqlite3VdbeAddOp4($448,42,$450,$452,$453,$454,-6)|0);
 $455 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($455,1);
 $456 = HEAP32[$v>>2]|0;
 $457 = HEAP32[$addrAltB>>2]|0;
 $458 = HEAP32[$addrAeqB>>2]|0;
 $459 = HEAP32[$addrAgtB>>2]|0;
 (_sqlite3VdbeAddOp3($456,43,$457,$458,$459)|0);
 $460 = HEAP32[$v>>2]|0;
 $461 = HEAP32[$labelEnd>>2]|0;
 _sqlite3VdbeResolveLabel($460,$461);
 $462 = HEAP32[$3>>2]|0;
 $463 = HEAP8[$462>>0]|0;
 $464 = $463&255;
 $465 = ($464|0)==(9);
 if ($465) {
  $466 = HEAP32[$pPrior>>2]|0;
  HEAP32[$pFirst>>2] = $466;
  while(1) {
   $467 = HEAP32[$pFirst>>2]|0;
   $468 = ((($467)) + 52|0);
   $469 = HEAP32[$468>>2]|0;
   $470 = ($469|0)!=(0|0);
   if (!($470)) {
    break;
   }
   $471 = HEAP32[$pFirst>>2]|0;
   $472 = ((($471)) + 52|0);
   $473 = HEAP32[$472>>2]|0;
   HEAP32[$pFirst>>2] = $473;
  }
  $474 = HEAP32[$1>>2]|0;
  $475 = HEAP32[$pFirst>>2]|0;
  $476 = ((($475)) + 32|0);
  $477 = HEAP32[$476>>2]|0;
  $478 = HEAP32[$pFirst>>2]|0;
  $479 = HEAP32[$478>>2]|0;
  _generateColumnNames($474,$477,$479);
 }
 $480 = HEAP32[$2>>2]|0;
 $481 = ((($480)) + 52|0);
 $482 = HEAP32[$481>>2]|0;
 $483 = ($482|0)!=(0|0);
 if ($483) {
  $484 = HEAP32[$db>>2]|0;
  $485 = HEAP32[$2>>2]|0;
  $486 = ((($485)) + 52|0);
  $487 = HEAP32[$486>>2]|0;
  _sqlite3SelectDelete($484,$487);
 }
 $488 = HEAP32[$pPrior>>2]|0;
 $489 = HEAP32[$2>>2]|0;
 $490 = ((($489)) + 52|0);
 HEAP32[$490>>2] = $488;
 $491 = HEAP32[$2>>2]|0;
 $492 = HEAP32[$pPrior>>2]|0;
 $493 = ((($492)) + 56|0);
 HEAP32[$493>>2] = $491;
 $494 = HEAP32[$1>>2]|0;
 $495 = HEAP32[$2>>2]|0;
 $496 = ((($495)) + 4|0);
 $497 = HEAP8[$496>>0]|0;
 $498 = $497&255;
 $499 = HEAP32[$iSub1>>2]|0;
 $500 = HEAP32[$iSub2>>2]|0;
 _explainComposite($494,$498,$499,$500,0);
 $501 = HEAP32[$1>>2]|0;
 $502 = ((($501)) + 68|0);
 $503 = HEAP32[$502>>2]|0;
 $504 = ($503|0)!=(0);
 $505 = $504&1;
 HEAP32[$0>>2] = $505;
 $506 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($506|0);
}
function _explainComposite($pParse,$op,$iSub1,$iSub2,$bUseTmp) {
 $pParse = $pParse|0;
 $op = $op|0;
 $iSub1 = $iSub1|0;
 $iSub2 = $iSub2|0;
 $bUseTmp = $bUseTmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $v = sp + 20|0;
 $zMsg = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSub1;
 HEAP32[$3>>2] = $iSub2;
 HEAP32[$4>>2] = $bUseTmp;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 453|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$v>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ($17|0)!=(0);
 $19 = $18 ? 32473 : 17532;
 $20 = HEAP32[$1>>2]|0;
 $21 = (_selectOpName($20)|0);
 HEAP32[$vararg_buffer>>2] = $15;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $16;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $19;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $21;
 $22 = (_sqlite3MPrintf($14,32436,$vararg_buffer)|0);
 HEAP32[$zMsg>>2] = $22;
 $23 = HEAP32[$v>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 468|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($23,161,$26,0,0,$27,-1)|0);
 STACKTOP = sp;return;
}
function _multiSelectCollSeq($pParse,$p,$iCol) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 52|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_multiSelectCollSeq($7,$10,$11)|0);
  HEAP32[$pRet>>2] = $12;
 } else {
  HEAP32[$pRet>>2] = 0;
 }
 $13 = HEAP32[$pRet>>2]|0;
 $14 = ($13|0)==(0|0);
 if (!($14)) {
  $29 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($15|0)<($18|0);
 if (!($19)) {
  $29 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + (($21*20)|0)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (_sqlite3ExprCollSeq($20,$27)|0);
 HEAP32[$pRet>>2] = $28;
 $29 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _multiSelectOrderByKeyInfo($pParse,$p,$nExtra) {
 $pParse = $pParse|0;
 $p = $p|0;
 $nExtra = $nExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $i = 0, $nOrderBy = 0, $pColl = 0, $pItem = 0, $pOrderBy = 0, $pRet = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pOrderBy = sp + 28|0;
 $nOrderBy = sp + 24|0;
 $db = sp + 20|0;
 $pRet = sp + 16|0;
 $i = sp + 12|0;
 $pItem = sp + 8|0;
 $pTerm = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nExtra;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pOrderBy>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 48|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nOrderBy>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$nOrderBy>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = (_sqlite3KeyInfoAlloc($12,$15,1)|0);
 HEAP32[$pRet>>2] = $16;
 $17 = HEAP32[$pRet>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  $78 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$nOrderBy>>2]|0;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pOrderBy>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + (($22*20)|0)|0);
  HEAP32[$pItem>>2] = $26;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTerm>>2] = $28;
  $29 = HEAP32[$pTerm>>2]|0;
  $30 = ((($29)) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $31 & 256;
  $33 = ($32|0)!=(0);
  $34 = HEAP32[$0>>2]|0;
  if ($33) {
   $35 = HEAP32[$pTerm>>2]|0;
   $36 = (_sqlite3ExprCollSeq($34,$35)|0);
   HEAP32[$pColl>>2] = $36;
  } else {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$pItem>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40&65535;
   $42 = (($41) - 1)|0;
   $43 = (_multiSelectCollSeq($34,$37,$42)|0);
   HEAP32[$pColl>>2] = $43;
   $44 = HEAP32[$pColl>>2]|0;
   $45 = ($44|0)==(0|0);
   if ($45) {
    $46 = HEAP32[$db>>2]|0;
    $47 = ((($46)) + 8|0);
    $48 = HEAP32[$47>>2]|0;
    HEAP32[$pColl>>2] = $48;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = HEAP32[$pTerm>>2]|0;
   $51 = HEAP32[$pColl>>2]|0;
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3ExprAddCollateString($49,$50,$52)|0);
   $54 = HEAP32[$i>>2]|0;
   $55 = HEAP32[$pOrderBy>>2]|0;
   $56 = ((($55)) + 4|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + (($54*20)|0)|0);
   HEAP32[$58>>2] = $53;
  }
  $59 = HEAP32[$pColl>>2]|0;
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$pRet>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = (($62) + ($60<<2)|0);
  HEAP32[$63>>2] = $59;
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$pOrderBy>>2]|0;
  $66 = ((($65)) + 4|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + (($64*20)|0)|0);
  $69 = ((($68)) + 12|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$pRet>>2]|0;
  $73 = ((($72)) + 16|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + ($71)|0);
  HEAP8[$75>>0] = $70;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 $78 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _codeOffset($v,$iOffset,$iContinue) {
 $v = $v|0;
 $iOffset = $iOffset|0;
 $iContinue = $iContinue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iOffset;
 HEAP32[$2>>2] = $iContinue;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($5,138,$6,$7,1)|0);
 STACKTOP = sp;return;
}
function _generateOutputSubroutine($pParse,$p,$pIn,$pDest,$regReturn,$regPrev,$pKeyInfo,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pIn = $pIn|0;
 $pDest = $pDest|0;
 $regReturn = $regReturn|0;
 $regPrev = $regPrev|0;
 $pKeyInfo = $pKeyInfo|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addr1 = 0, $addr2 = 0, $iContinue = 0, $r1 = 0, $r11 = 0, $r2 = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $6 = sp + 40|0;
 $7 = sp + 36|0;
 $8 = sp + 32|0;
 $v = sp + 28|0;
 $iContinue = sp + 24|0;
 $addr = sp + 20|0;
 $addr1 = sp + 16|0;
 $addr2 = sp + 12|0;
 $r1 = sp + 8|0;
 $r2 = sp + 4|0;
 $r11 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pIn;
 HEAP32[$4>>2] = $pDest;
 HEAP32[$5>>2] = $regReturn;
 HEAP32[$6>>2] = $regPrev;
 HEAP32[$7>>2] = $pKeyInfo;
 HEAP32[$8>>2] = $iBreak;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$v>>2]|0;
 $13 = (_sqlite3VdbeCurrentAddr($12)|0);
 HEAP32[$addr>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = (_sqlite3VdbeMakeLabel($14)|0);
 HEAP32[$iContinue>>2] = $15;
 $16 = HEAP32[$6>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$v>>2]|0;
  $19 = HEAP32[$6>>2]|0;
  $20 = (_sqlite3VdbeAddOp1($18,46,$19)|0);
  HEAP32[$addr1>>2] = $20;
  $21 = HEAP32[$v>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$6>>2]|0;
  $26 = (($25) + 1)|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$7>>2]|0;
  $31 = (_sqlite3KeyInfoRef($30)|0);
  $32 = (_sqlite3VdbeAddOp4($21,42,$24,$26,$29,$31,-6)|0);
  HEAP32[$addr2>>2] = $32;
  $33 = HEAP32[$v>>2]|0;
  $34 = HEAP32[$addr2>>2]|0;
  $35 = (($34) + 2)|0;
  $36 = HEAP32[$iContinue>>2]|0;
  $37 = HEAP32[$addr2>>2]|0;
  $38 = (($37) + 2)|0;
  (_sqlite3VdbeAddOp3($33,43,$35,$36,$38)|0);
  $39 = HEAP32[$v>>2]|0;
  $40 = HEAP32[$addr1>>2]|0;
  _sqlite3VdbeJumpHere($39,$40);
  $41 = HEAP32[$v>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$6>>2]|0;
  $46 = (($45) + 1)|0;
  $47 = HEAP32[$3>>2]|0;
  $48 = ((($47)) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) - 1)|0;
  (_sqlite3VdbeAddOp3($41,30,$44,$46,$50)|0);
  $51 = HEAP32[$v>>2]|0;
  $52 = HEAP32[$6>>2]|0;
  (_sqlite3VdbeAddOp2($51,22,1,$52)|0);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 $55 = ((($54)) + 65|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = ($56<<24>>24)!=(0);
 if ($57) {
  HEAP32[$0>>2] = 0;
  $190 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($190|0);
 }
 $58 = HEAP32[$v>>2]|0;
 $59 = HEAP32[$2>>2]|0;
 $60 = ((($59)) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$iContinue>>2]|0;
 _codeOffset($58,$61,$62);
 $63 = HEAP32[$4>>2]|0;
 $64 = HEAP8[$63>>0]|0;
 $65 = $64&255;
 switch ($65|0) {
 case 12:  {
  $66 = HEAP32[$1>>2]|0;
  $67 = (_sqlite3GetTempReg($66)|0);
  HEAP32[$r1>>2] = $67;
  $68 = HEAP32[$1>>2]|0;
  $69 = (_sqlite3GetTempReg($68)|0);
  HEAP32[$r2>>2] = $69;
  $70 = HEAP32[$v>>2]|0;
  $71 = HEAP32[$3>>2]|0;
  $72 = ((($71)) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$3>>2]|0;
  $75 = ((($74)) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$r1>>2]|0;
  (_sqlite3VdbeAddOp3($70,49,$73,$76,$77)|0);
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$4>>2]|0;
  $80 = ((($79)) + 4|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = HEAP32[$r2>>2]|0;
  (_sqlite3VdbeAddOp2($78,74,$81,$82)|0);
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$4>>2]|0;
  $85 = ((($84)) + 4|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$r1>>2]|0;
  $88 = HEAP32[$r2>>2]|0;
  (_sqlite3VdbeAddOp3($83,75,$86,$87,$88)|0);
  $89 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($89,8);
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$r2>>2]|0;
  _sqlite3ReleaseTempReg($90,$91);
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP32[$r1>>2]|0;
  _sqlite3ReleaseTempReg($92,$93);
  break;
 }
 case 11:  {
  $94 = HEAP32[$2>>2]|0;
  $95 = HEAP32[$94>>2]|0;
  $96 = ((($95)) + 4|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = HEAP32[$97>>2]|0;
  $99 = HEAP32[$4>>2]|0;
  $100 = ((($99)) + 1|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = (_sqlite3CompareAffinity($98,$101)|0);
  $103 = HEAP32[$4>>2]|0;
  $104 = ((($103)) + 1|0);
  HEAP8[$104>>0] = $102;
  $105 = HEAP32[$1>>2]|0;
  $106 = (_sqlite3GetTempReg($105)|0);
  HEAP32[$r11>>2] = $106;
  $107 = HEAP32[$v>>2]|0;
  $108 = HEAP32[$3>>2]|0;
  $109 = ((($108)) + 8|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$r11>>2]|0;
  $112 = HEAP32[$4>>2]|0;
  $113 = ((($112)) + 1|0);
  (_sqlite3VdbeAddOp4($107,49,$110,1,$111,$113,1)|0);
  $114 = HEAP32[$1>>2]|0;
  $115 = HEAP32[$3>>2]|0;
  $116 = ((($115)) + 8|0);
  $117 = HEAP32[$116>>2]|0;
  _sqlite3ExprCacheAffinityChange($114,$117,1);
  $118 = HEAP32[$v>>2]|0;
  $119 = HEAP32[$4>>2]|0;
  $120 = ((($119)) + 4|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = HEAP32[$r11>>2]|0;
  (_sqlite3VdbeAddOp2($118,110,$121,$122)|0);
  $123 = HEAP32[$1>>2]|0;
  $124 = HEAP32[$r11>>2]|0;
  _sqlite3ReleaseTempReg($123,$124);
  break;
 }
 case 10:  {
  $125 = HEAP32[$1>>2]|0;
  $126 = HEAP32[$3>>2]|0;
  $127 = ((($126)) + 8|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = HEAP32[$4>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  _sqlite3ExprCodeMove($125,$128,$131,1);
  break;
 }
 case 13:  {
  $132 = HEAP32[$4>>2]|0;
  $133 = ((($132)) + 8|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = ($134|0)==(0);
  if ($135) {
   $136 = HEAP32[$1>>2]|0;
   $137 = HEAP32[$3>>2]|0;
   $138 = ((($137)) + 12|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = (_sqlite3GetTempRange($136,$139)|0);
   $141 = HEAP32[$4>>2]|0;
   $142 = ((($141)) + 8|0);
   HEAP32[$142>>2] = $140;
   $143 = HEAP32[$3>>2]|0;
   $144 = ((($143)) + 12|0);
   $145 = HEAP32[$144>>2]|0;
   $146 = HEAP32[$4>>2]|0;
   $147 = ((($146)) + 12|0);
   HEAP32[$147>>2] = $145;
  }
  $148 = HEAP32[$1>>2]|0;
  $149 = HEAP32[$3>>2]|0;
  $150 = ((($149)) + 8|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$4>>2]|0;
  $153 = ((($152)) + 8|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = HEAP32[$3>>2]|0;
  $156 = ((($155)) + 12|0);
  $157 = HEAP32[$156>>2]|0;
  _sqlite3ExprCodeMove($148,$151,$154,$157);
  $158 = HEAP32[$v>>2]|0;
  $159 = HEAP32[$4>>2]|0;
  $160 = ((($159)) + 4|0);
  $161 = HEAP32[$160>>2]|0;
  (_sqlite3VdbeAddOp1($158,18,$161)|0);
  break;
 }
 default: {
  $162 = HEAP32[$v>>2]|0;
  $163 = HEAP32[$3>>2]|0;
  $164 = ((($163)) + 8|0);
  $165 = HEAP32[$164>>2]|0;
  $166 = HEAP32[$3>>2]|0;
  $167 = ((($166)) + 12|0);
  $168 = HEAP32[$167>>2]|0;
  (_sqlite3VdbeAddOp2($162,33,$165,$168)|0);
  $169 = HEAP32[$1>>2]|0;
  $170 = HEAP32[$3>>2]|0;
  $171 = ((($170)) + 8|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = HEAP32[$3>>2]|0;
  $174 = ((($173)) + 12|0);
  $175 = HEAP32[$174>>2]|0;
  _sqlite3ExprCacheAffinityChange($169,$172,$175);
 }
 }
 $176 = HEAP32[$2>>2]|0;
 $177 = ((($176)) + 8|0);
 $178 = HEAP32[$177>>2]|0;
 $179 = ($178|0)!=(0);
 if ($179) {
  $180 = HEAP32[$v>>2]|0;
  $181 = HEAP32[$2>>2]|0;
  $182 = ((($181)) + 8|0);
  $183 = HEAP32[$182>>2]|0;
  $184 = HEAP32[$8>>2]|0;
  (_sqlite3VdbeAddOp2($180,141,$183,$184)|0);
 }
 $185 = HEAP32[$v>>2]|0;
 $186 = HEAP32[$iContinue>>2]|0;
 _sqlite3VdbeResolveLabel($185,$186);
 $187 = HEAP32[$v>>2]|0;
 $188 = HEAP32[$5>>2]|0;
 (_sqlite3VdbeAddOp1($187,15,$188)|0);
 $189 = HEAP32[$addr>>2]|0;
 HEAP32[$0>>2] = $189;
 $190 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($190|0);
}
function _sqlite3ExprIsTableConstant($p,$iCur) {
 $p = $p|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCur;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_exprIsConst($2,3,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _whereLoopInit($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 56|0);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 48|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 40|0);
 HEAP16[$6>>1] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 HEAP16[$8>>1] = 3;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 36|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3WhereClauseInit($pWC,$pWInfo) {
 $pWC = $pWC|0;
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pWInfo;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$3>>2] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 4|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 16|0);
 HEAP32[$9>>2] = 8;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 20|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _sqlite3WhereSplit($pWC,$pExpr,$op) {
 $pWC = $pWC|0;
 $pExpr = $pExpr|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pE2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $pE2 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $op;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprSkipCollate($3)|0);
 HEAP32[$pE2>>2] = $4;
 $5 = HEAP8[$2>>0]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$pE2>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pE2>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = HEAP8[$2>>0]|0;
 $14 = $13&255;
 $15 = ($12|0)!=($14|0);
 $16 = HEAP32[$0>>2]|0;
 if ($15) {
  $17 = HEAP32[$1>>2]|0;
  (_whereClauseInsert($16,$17,0)|0);
  STACKTOP = sp;return;
 } else {
  $18 = HEAP32[$pE2>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP8[$2>>0]|0;
  _sqlite3WhereSplit($16,$20,$21);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$pE2>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP8[$2>>0]|0;
  _sqlite3WhereSplit($22,$25,$26);
  STACKTOP = sp;return;
 }
}
function _createMask($pMaskSet,$iCursor) {
 $pMaskSet = $pMaskSet|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $iCursor;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = (($7) + ($4<<2)|0);
 HEAP32[$8>>2] = $2;
 STACKTOP = sp;return;
}
function _sqlite3WhereTabFuncArgs($pParse,$pItem,$pWC) {
 $pParse = $pParse|0;
 $pItem = $pItem|0;
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $j = 0, $k = 0, $pArgs = 0, $pColRef = 0, $pTab = 0, $pTerm = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pTab = sp + 28|0;
 $j = sp + 24|0;
 $k = sp + 20|0;
 $pArgs = sp + 16|0;
 $pColRef = sp + 12|0;
 $pTerm = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pItem;
 HEAP32[$2>>2] = $pWC;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = ((($4)) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pTab>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 64|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pArgs>>2] = $16;
 $17 = HEAP32[$pArgs>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  STACKTOP = sp;return;
 }
 HEAP32[$k>>2] = 0;
 HEAP32[$j>>2] = 0;
 while(1) {
  $19 = HEAP32[$j>>2]|0;
  $20 = HEAP32[$pArgs>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ($19|0)<($21|0);
  if (!($22)) {
   label = 13;
   break;
  }
  while(1) {
   $23 = HEAP32[$k>>2]|0;
   $24 = HEAP32[$pTab>>2]|0;
   $25 = ((($24)) + 34|0);
   $26 = HEAP16[$25>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = ($23|0)<($27|0);
   if ($28) {
    $29 = HEAP32[$k>>2]|0;
    $30 = HEAP32[$pTab>>2]|0;
    $31 = ((($30)) + 4|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) + (($29*24)|0)|0);
    $34 = ((($33)) + 23|0);
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = $36 & 2;
    $38 = ($37|0)==(0);
    $82 = $38;
   } else {
    $82 = 0;
   }
   $39 = HEAP32[$k>>2]|0;
   if (!($82)) {
    break;
   }
   $40 = (($39) + 1)|0;
   HEAP32[$k>>2] = $40;
  }
  $41 = HEAP32[$pTab>>2]|0;
  $42 = ((($41)) + 34|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43 << 16 >> 16;
  $45 = ($39|0)>=($44|0);
  $46 = HEAP32[$0>>2]|0;
  if ($45) {
   label = 10;
   break;
  }
  $50 = (_sqlite3PExpr($46,152,0,0,0)|0);
  HEAP32[$pColRef>>2] = $50;
  $51 = HEAP32[$pColRef>>2]|0;
  $52 = ($51|0)==(0|0);
  if ($52) {
   label = 13;
   break;
  }
  $53 = HEAP32[$1>>2]|0;
  $54 = ((($53)) + 44|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$pColRef>>2]|0;
  $57 = ((($56)) + 28|0);
  HEAP32[$57>>2] = $55;
  $58 = HEAP32[$k>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$k>>2] = $59;
  $60 = $58&65535;
  $61 = HEAP32[$pColRef>>2]|0;
  $62 = ((($61)) + 32|0);
  HEAP16[$62>>1] = $60;
  $63 = HEAP32[$pTab>>2]|0;
  $64 = HEAP32[$pColRef>>2]|0;
  $65 = ((($64)) + 44|0);
  HEAP32[$65>>2] = $63;
  $66 = HEAP32[$0>>2]|0;
  $67 = HEAP32[$pColRef>>2]|0;
  $68 = HEAP32[$0>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$j>>2]|0;
  $71 = HEAP32[$pArgs>>2]|0;
  $72 = ((($71)) + 4|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + (($70*20)|0)|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (_sqlite3ExprDup($69,$75,0)|0);
  $77 = (_sqlite3PExpr($66,79,$67,$76,0)|0);
  HEAP32[$pTerm>>2] = $77;
  $78 = HEAP32[$2>>2]|0;
  $79 = HEAP32[$pTerm>>2]|0;
  (_whereClauseInsert($78,$79,1)|0);
  $80 = HEAP32[$j>>2]|0;
  $81 = (($80) + 1)|0;
  HEAP32[$j>>2] = $81;
 }
 if ((label|0) == 10) {
  $47 = HEAP32[$pTab>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$j>>2]|0;
  HEAP32[$vararg_buffer>>2] = $48;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $49;
  _sqlite3ErrorMsg($46,32520,$vararg_buffer);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 13) {
  STACKTOP = sp;return;
 }
}
function _sqlite3WhereExprAnalyze($pTabList,$pWC) {
 $pTabList = $pTabList|0;
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTabList;
 HEAP32[$1>>2] = $pWC;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 HEAP32[$i>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)>=(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  _exprAnalyze($8,$9,$10);
  $11 = HEAP32[$i>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$i>>2] = $12;
 }
 STACKTOP = sp;return;
}
function _isDistinctRedundant($pParse,$pTabList,$pWC,$pDistinct) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWC = $pWC|0;
 $pDistinct = $pDistinct|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $iBase = 0, $p = 0, $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pTab = sp + 16|0;
 $pIdx = sp + 12|0;
 $i = sp + 8|0;
 $iBase = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $pDistinct;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(1);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = ((($9)) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$iBase>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTab>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$4>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3ExprSkipCollate($25)|0);
  HEAP32[$p>>2] = $26;
  $27 = HEAP32[$p>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(152);
  if ($30) {
   $31 = HEAP32[$p>>2]|0;
   $32 = ((($31)) + 28|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$iBase>>2]|0;
   $35 = ($33|0)==($34|0);
   if ($35) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ((($36)) + 32|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38 << 16 >> 16;
    $40 = ($39|0)<(0);
    if ($40) {
     label = 8;
     break;
    }
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 $43 = HEAP32[$pTab>>2]|0;
 $44 = ((($43)) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 HEAP32[$pIdx>>2] = $45;
 while(1) {
  $46 = HEAP32[$pIdx>>2]|0;
  $47 = ($46|0)!=(0|0);
  if (!($47)) {
   label = 22;
   break;
  }
  $48 = HEAP32[$pIdx>>2]|0;
  $49 = ((($48)) + 54|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = ($51|0)!=(0);
  if ($52) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $53 = HEAP32[$i>>2]|0;
    $54 = HEAP32[$pIdx>>2]|0;
    $55 = ((($54)) + 50|0);
    $56 = HEAP16[$55>>1]|0;
    $57 = $56&65535;
    $58 = ($53|0)<($57|0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$3>>2]|0;
    $60 = HEAP32[$iBase>>2]|0;
    $61 = HEAP32[$i>>2]|0;
    $62 = HEAP32[$pIdx>>2]|0;
    $63 = (_sqlite3WhereFindTerm($59,$60,$61,-1,-1,2,$62)|0);
    $64 = (0|0)==($63|0);
    if ($64) {
     $65 = HEAP32[$1>>2]|0;
     $66 = HEAP32[$4>>2]|0;
     $67 = HEAP32[$iBase>>2]|0;
     $68 = HEAP32[$pIdx>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = (_findIndexCol($65,$66,$67,$68,$69)|0);
     $71 = ($70|0)<(0);
     if ($71) {
      break;
     }
     $72 = HEAP32[$pIdx>>2]|0;
     $73 = HEAP32[$i>>2]|0;
     $74 = (_indexColumnNotNull($72,$73)|0);
     $75 = ($74|0)==(0);
     if ($75) {
      break;
     }
    }
    $76 = HEAP32[$i>>2]|0;
    $77 = (($76) + 1)|0;
    HEAP32[$i>>2] = $77;
   }
   $78 = HEAP32[$i>>2]|0;
   $79 = HEAP32[$pIdx>>2]|0;
   $80 = ((($79)) + 50|0);
   $81 = HEAP16[$80>>1]|0;
   $82 = $81&65535;
   $83 = ($78|0)==($82|0);
   if ($83) {
    label = 20;
    break;
   }
  }
  $84 = HEAP32[$pIdx>>2]|0;
  $85 = ((($84)) + 20|0);
  $86 = HEAP32[$85>>2]|0;
  HEAP32[$pIdx>>2] = $86;
 }
 if ((label|0) == 20) {
  HEAP32[$0>>2] = 1;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 else if ((label|0) == 22) {
  HEAP32[$0>>2] = 0;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 return (0)|0;
}
function _whereShortCut($pBuilder) {
 $pBuilder = $pBuilder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCur = 0, $j = 0, $opMask = 0, $pIdx = 0, $pItem = 0, $pLoop = 0, $pTab = 0, $pTerm = 0, $pWC = 0, $pWInfo = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $pWInfo = sp + 36|0;
 $pItem = sp + 32|0;
 $pWC = sp + 28|0;
 $pTerm = sp + 24|0;
 $pLoop = sp + 20|0;
 $iCur = sp + 16|0;
 $j = sp + 12|0;
 $pTab = sp + 8|0;
 $pIdx = sp + 4|0;
 $opMask = sp;
 HEAP32[$1>>2] = $pBuilder;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pWInfo>>2] = $3;
 $4 = HEAP32[$pWInfo>>2]|0;
 $5 = ((($4)) + 34|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 32;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $10 = HEAP32[$pWInfo>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 8|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$pItem>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pTab>>2] = $16;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = ((($17)) + 42|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = $20 & 16;
 $22 = ($21|0)!=(0);
 if ($22) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $23 = HEAP32[$pItem>>2]|0;
 $24 = ((($23)) + 36|0);
 $25 = ((($24)) + 1|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = ($26&255) >>> 1;
 $28 = $27 & 1;
 $29 = $28&255;
 $30 = ($29|0)!=(0);
 if ($30) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $31 = HEAP32[$pItem>>2]|0;
 $32 = ((($31)) + 44|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$iCur>>2] = $33;
 $34 = HEAP32[$pWInfo>>2]|0;
 $35 = ((($34)) + 328|0);
 HEAP32[$pWC>>2] = $35;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 12|0);
 $38 = HEAP32[$37>>2]|0;
 HEAP32[$pLoop>>2] = $38;
 $39 = HEAP32[$pLoop>>2]|0;
 $40 = ((($39)) + 36|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$pLoop>>2]|0;
 $42 = ((($41)) + 42|0);
 HEAP16[$42>>1] = 0;
 $43 = HEAP32[$pWC>>2]|0;
 $44 = HEAP32[$iCur>>2]|0;
 $45 = (_sqlite3WhereFindTerm($43,$44,-1,0,0,130,0)|0);
 HEAP32[$pTerm>>2] = $45;
 $46 = HEAP32[$pTerm>>2]|0;
 $47 = ($46|0)!=(0|0);
 L13: do {
  if ($47) {
   $48 = HEAP32[$pLoop>>2]|0;
   $49 = ((($48)) + 36|0);
   HEAP32[$49>>2] = 4353;
   $50 = HEAP32[$pTerm>>2]|0;
   $51 = HEAP32[$pLoop>>2]|0;
   $52 = ((($51)) + 48|0);
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$53>>2] = $50;
   $54 = HEAP32[$pLoop>>2]|0;
   $55 = ((($54)) + 40|0);
   HEAP16[$55>>1] = 1;
   $56 = HEAP32[$pLoop>>2]|0;
   $57 = ((($56)) + 24|0);
   HEAP16[$57>>1] = 1;
   $58 = HEAP32[$pLoop>>2]|0;
   $59 = ((($58)) + 20|0);
   HEAP16[$59>>1] = 33;
  } else {
   $60 = HEAP32[$pTab>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$pIdx>>2] = $62;
   while(1) {
    $63 = HEAP32[$pIdx>>2]|0;
    $64 = ($63|0)!=(0|0);
    if (!($64)) {
     break L13;
    }
    $65 = HEAP32[$pIdx>>2]|0;
    $66 = ((($65)) + 54|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = ($68|0)!=(0);
    if ($69) {
     $70 = HEAP32[$pIdx>>2]|0;
     $71 = ((($70)) + 36|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = ($72|0)!=(0|0);
     if (!($73)) {
      $74 = HEAP32[$pIdx>>2]|0;
      $75 = ((($74)) + 50|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76&65535;
      $78 = ($77|0)>(3);
      if (!($78)) {
       $79 = HEAP32[$pIdx>>2]|0;
       $80 = ((($79)) + 55|0);
       $81 = HEAP8[$80>>0]|0;
       $82 = ($81&255) >>> 3;
       $83 = $82 & 1;
       $84 = $83&255;
       $85 = ($84|0)!=(0);
       $86 = $85 ? 130 : 2;
       HEAP32[$opMask>>2] = $86;
       HEAP32[$j>>2] = 0;
       while(1) {
        $87 = HEAP32[$j>>2]|0;
        $88 = HEAP32[$pIdx>>2]|0;
        $89 = ((($88)) + 50|0);
        $90 = HEAP16[$89>>1]|0;
        $91 = $90&65535;
        $92 = ($87|0)<($91|0);
        if (!($92)) {
         break;
        }
        $93 = HEAP32[$pWC>>2]|0;
        $94 = HEAP32[$iCur>>2]|0;
        $95 = HEAP32[$j>>2]|0;
        $96 = HEAP32[$opMask>>2]|0;
        $97 = HEAP32[$pIdx>>2]|0;
        $98 = (_sqlite3WhereFindTerm($93,$94,$95,0,0,$96,$97)|0);
        HEAP32[$pTerm>>2] = $98;
        $99 = HEAP32[$pTerm>>2]|0;
        $100 = ($99|0)==(0|0);
        if ($100) {
         break;
        }
        $101 = HEAP32[$pTerm>>2]|0;
        $102 = HEAP32[$j>>2]|0;
        $103 = HEAP32[$pLoop>>2]|0;
        $104 = ((($103)) + 48|0);
        $105 = HEAP32[$104>>2]|0;
        $106 = (($105) + ($102<<2)|0);
        HEAP32[$106>>2] = $101;
        $107 = HEAP32[$j>>2]|0;
        $108 = (($107) + 1)|0;
        HEAP32[$j>>2] = $108;
       }
       $109 = HEAP32[$j>>2]|0;
       $110 = HEAP32[$pIdx>>2]|0;
       $111 = ((($110)) + 50|0);
       $112 = HEAP16[$111>>1]|0;
       $113 = $112&65535;
       $114 = ($109|0)!=($113|0);
       if (!($114)) {
        break;
       }
      }
     }
    }
    $160 = HEAP32[$pIdx>>2]|0;
    $161 = ((($160)) + 20|0);
    $162 = HEAP32[$161>>2]|0;
    HEAP32[$pIdx>>2] = $162;
   }
   $115 = HEAP32[$pLoop>>2]|0;
   $116 = ((($115)) + 36|0);
   HEAP32[$116>>2] = 4609;
   $117 = HEAP32[$pIdx>>2]|0;
   $118 = ((($117)) + 55|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = ($119&255) >>> 5;
   $121 = $120 & 1;
   $122 = $121&255;
   $123 = ($122|0)!=(0);
   if ($123) {
    label = 21;
   } else {
    $124 = HEAP32[$pItem>>2]|0;
    $125 = ((($124)) + 56|0);
    $126 = $125;
    $127 = $126;
    $128 = HEAP32[$127>>2]|0;
    $129 = (($126) + 4)|0;
    $130 = $129;
    $131 = HEAP32[$130>>2]|0;
    $132 = HEAP32[$pIdx>>2]|0;
    $133 = (_columnsInIndex($132)|0);
    $134 = tempRet0;
    $135 = $133 ^ -1;
    $136 = $134 ^ -1;
    $137 = $128 & $135;
    $138 = $131 & $136;
    $139 = ($137|0)==(0);
    $140 = ($138|0)==(0);
    $141 = $139 & $140;
    if ($141) {
     label = 21;
    }
   }
   if ((label|0) == 21) {
    $142 = HEAP32[$pLoop>>2]|0;
    $143 = ((($142)) + 36|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = $144 | 64;
    HEAP32[$143>>2] = $145;
   }
   $146 = HEAP32[$j>>2]|0;
   $147 = $146&65535;
   $148 = HEAP32[$pLoop>>2]|0;
   $149 = ((($148)) + 40|0);
   HEAP16[$149>>1] = $147;
   $150 = HEAP32[$j>>2]|0;
   $151 = $150&65535;
   $152 = HEAP32[$pLoop>>2]|0;
   $153 = ((($152)) + 24|0);
   HEAP16[$153>>1] = $151;
   $154 = HEAP32[$pIdx>>2]|0;
   $155 = HEAP32[$pLoop>>2]|0;
   $156 = ((($155)) + 24|0);
   $157 = ((($156)) + 4|0);
   HEAP32[$157>>2] = $154;
   $158 = HEAP32[$pLoop>>2]|0;
   $159 = ((($158)) + 20|0);
   HEAP16[$159>>1] = 39;
  }
 } while(0);
 $163 = HEAP32[$pLoop>>2]|0;
 $164 = ((($163)) + 36|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = ($165|0)!=(0);
 if (!($166)) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $167 = HEAP32[$pLoop>>2]|0;
 $168 = ((($167)) + 22|0);
 HEAP16[$168>>1] = 1;
 $169 = HEAP32[$pLoop>>2]|0;
 $170 = HEAP32[$pWInfo>>2]|0;
 $171 = ((($170)) + 736|0);
 $172 = ((($171)) + 64|0);
 HEAP32[$172>>2] = $169;
 $173 = HEAP32[$pWInfo>>2]|0;
 $174 = ((($173)) + 68|0);
 $175 = HEAP32[$iCur>>2]|0;
 $176 = (_sqlite3WhereGetMask($174,$175)|0);
 $177 = tempRet0;
 $178 = HEAP32[$pLoop>>2]|0;
 $179 = ((($178)) + 8|0);
 $180 = $179;
 $181 = $180;
 HEAP32[$181>>2] = $176;
 $182 = (($180) + 4)|0;
 $183 = $182;
 HEAP32[$183>>2] = $177;
 $184 = HEAP32[$iCur>>2]|0;
 $185 = HEAP32[$pWInfo>>2]|0;
 $186 = ((($185)) + 736|0);
 $187 = ((($186)) + 4|0);
 HEAP32[$187>>2] = $184;
 $188 = HEAP32[$pWInfo>>2]|0;
 $189 = ((($188)) + 32|0);
 HEAP16[$189>>1] = 1;
 $190 = HEAP32[$pWInfo>>2]|0;
 $191 = ((($190)) + 8|0);
 $192 = HEAP32[$191>>2]|0;
 $193 = ($192|0)!=(0|0);
 if ($193) {
  $194 = HEAP32[$pWInfo>>2]|0;
  $195 = ((($194)) + 8|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = HEAP32[$196>>2]|0;
  $198 = $197&255;
  $199 = HEAP32[$pWInfo>>2]|0;
  $200 = ((($199)) + 36|0);
  HEAP8[$200>>0] = $198;
 }
 $201 = HEAP32[$pWInfo>>2]|0;
 $202 = ((($201)) + 34|0);
 $203 = HEAP16[$202>>1]|0;
 $204 = $203&65535;
 $205 = $204 & 1024;
 $206 = ($205|0)!=(0);
 if ($206) {
  $207 = HEAP32[$pWInfo>>2]|0;
  $208 = ((($207)) + 40|0);
  HEAP8[$208>>0] = 1;
 }
 HEAP32[$0>>2] = 1;
 $209 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($209|0);
}
function _whereLoopAddAll($pBuilder) {
 $pBuilder = $pBuilder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iTab = 0, $mExtra = 0;
 var $mPrior = 0, $mUnusable = 0, $p = 0, $pEnd = 0, $pItem = 0, $pNew = 0, $pTabList = 0, $pWInfo = 0, $priorJointype = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $pWInfo = sp + 56|0;
 $mExtra = sp + 16|0;
 $mPrior = sp + 8|0;
 $iTab = sp + 52|0;
 $pTabList = sp + 48|0;
 $pItem = sp + 44|0;
 $pEnd = sp + 40|0;
 $db = sp + 36|0;
 $rc = sp + 32|0;
 $pNew = sp + 28|0;
 $priorJointype = sp + 64|0;
 $mUnusable = sp;
 $p = sp + 24|0;
 HEAP32[$0>>2] = $pBuilder;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pWInfo>>2] = $2;
 $3 = $mExtra;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = $mPrior;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pWInfo>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pTabList>>2] = $13;
 $14 = HEAP32[$pWInfo>>2]|0;
 $15 = ((($14)) + 41|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = HEAP32[$pTabList>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = (($19) + (($17*72)|0)|0);
 HEAP32[$pEnd>>2] = $20;
 $21 = HEAP32[$pWInfo>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$db>>2] = $23;
 HEAP32[$rc>>2] = 0;
 HEAP8[$priorJointype>>0] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pNew>>2] = $26;
 $27 = HEAP32[$pNew>>2]|0;
 _whereLoopInit($27);
 HEAP32[$iTab>>2] = 0;
 $28 = HEAP32[$pTabList>>2]|0;
 $29 = ((($28)) + 8|0);
 HEAP32[$pItem>>2] = $29;
 while(1) {
  $30 = HEAP32[$pItem>>2]|0;
  $31 = HEAP32[$pEnd>>2]|0;
  $32 = ($30>>>0)<($31>>>0);
  if (!($32)) {
   label = 19;
   break;
  }
  $33 = $mUnusable;
  $34 = $33;
  HEAP32[$34>>2] = 0;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = 0;
  $37 = HEAP32[$iTab>>2]|0;
  $38 = $37&255;
  $39 = HEAP32[$pNew>>2]|0;
  $40 = ((($39)) + 16|0);
  HEAP8[$40>>0] = $38;
  $41 = HEAP32[$pWInfo>>2]|0;
  $42 = ((($41)) + 68|0);
  $43 = HEAP32[$pItem>>2]|0;
  $44 = ((($43)) + 44|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3WhereGetMask($42,$45)|0);
  $47 = tempRet0;
  $48 = HEAP32[$pNew>>2]|0;
  $49 = ((($48)) + 8|0);
  $50 = $49;
  $51 = $50;
  HEAP32[$51>>2] = $46;
  $52 = (($50) + 4)|0;
  $53 = $52;
  HEAP32[$53>>2] = $47;
  $54 = HEAP32[$pItem>>2]|0;
  $55 = ((($54)) + 36|0);
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = HEAP8[$priorJointype>>0]|0;
  $59 = $58&255;
  $60 = $57 | $59;
  $61 = $60 & 10;
  $62 = ($61|0)!=(0);
  if ($62) {
   $63 = $mPrior;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = (($63) + 4)|0;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = $mExtra;
   $70 = $69;
   HEAP32[$70>>2] = $65;
   $71 = (($69) + 4)|0;
   $72 = $71;
   HEAP32[$72>>2] = $68;
  }
  $73 = HEAP32[$pItem>>2]|0;
  $74 = ((($73)) + 36|0);
  $75 = HEAP8[$74>>0]|0;
  HEAP8[$priorJointype>>0] = $75;
  $76 = HEAP32[$pItem>>2]|0;
  $77 = ((($76)) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ((($78)) + 42|0);
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = $81 & 16;
  $83 = ($82|0)!=(0);
  if ($83) {
   $84 = HEAP32[$pItem>>2]|0;
   $85 = ((($84)) + 72|0);
   HEAP32[$p>>2] = $85;
   while(1) {
    $86 = HEAP32[$p>>2]|0;
    $87 = HEAP32[$pEnd>>2]|0;
    $88 = ($86>>>0)<($87>>>0);
    if (!($88)) {
     break;
    }
    $89 = $mUnusable;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    $92 = (($89) + 4)|0;
    $93 = $92;
    $94 = HEAP32[$93>>2]|0;
    $95 = ($91|0)!=(0);
    $96 = ($94|0)!=(0);
    $97 = $95 | $96;
    if ($97) {
     label = 10;
    } else {
     $98 = HEAP32[$p>>2]|0;
     $99 = ((($98)) + 36|0);
     $100 = HEAP8[$99>>0]|0;
     $101 = $100&255;
     $102 = $101 & 10;
     $103 = ($102|0)!=(0);
     if ($103) {
      label = 10;
     }
    }
    if ((label|0) == 10) {
     label = 0;
     $104 = HEAP32[$pWInfo>>2]|0;
     $105 = ((($104)) + 68|0);
     $106 = HEAP32[$p>>2]|0;
     $107 = ((($106)) + 44|0);
     $108 = HEAP32[$107>>2]|0;
     $109 = (_sqlite3WhereGetMask($105,$108)|0);
     $110 = tempRet0;
     $111 = $mUnusable;
     $112 = $111;
     $113 = HEAP32[$112>>2]|0;
     $114 = (($111) + 4)|0;
     $115 = $114;
     $116 = HEAP32[$115>>2]|0;
     $117 = $113 | $109;
     $118 = $116 | $110;
     $119 = $mUnusable;
     $120 = $119;
     HEAP32[$120>>2] = $117;
     $121 = (($119) + 4)|0;
     $122 = $121;
     HEAP32[$122>>2] = $118;
    }
    $123 = HEAP32[$p>>2]|0;
    $124 = ((($123)) + 72|0);
    HEAP32[$p>>2] = $124;
   }
   $125 = HEAP32[$0>>2]|0;
   $126 = $mExtra;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = (($126) + 4)|0;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = $mUnusable;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = (($132) + 4)|0;
   $136 = $135;
   $137 = HEAP32[$136>>2]|0;
   $138 = (_whereLoopAddVirtual($125,$128,$131,$134,$137)|0);
   HEAP32[$rc>>2] = $138;
  } else {
   $139 = HEAP32[$0>>2]|0;
   $140 = $mExtra;
   $141 = $140;
   $142 = HEAP32[$141>>2]|0;
   $143 = (($140) + 4)|0;
   $144 = $143;
   $145 = HEAP32[$144>>2]|0;
   $146 = (_whereLoopAddBtree($139,$142,$145)|0);
   HEAP32[$rc>>2] = $146;
  }
  $147 = HEAP32[$rc>>2]|0;
  $148 = ($147|0)==(0);
  if ($148) {
   $149 = HEAP32[$0>>2]|0;
   $150 = $mExtra;
   $151 = $150;
   $152 = HEAP32[$151>>2]|0;
   $153 = (($150) + 4)|0;
   $154 = $153;
   $155 = HEAP32[$154>>2]|0;
   $156 = $mUnusable;
   $157 = $156;
   $158 = HEAP32[$157>>2]|0;
   $159 = (($156) + 4)|0;
   $160 = $159;
   $161 = HEAP32[$160>>2]|0;
   $162 = (_whereLoopAddOr($149,$152,$155,$158,$161)|0);
   HEAP32[$rc>>2] = $162;
  }
  $163 = HEAP32[$pNew>>2]|0;
  $164 = ((($163)) + 8|0);
  $165 = $164;
  $166 = $165;
  $167 = HEAP32[$166>>2]|0;
  $168 = (($165) + 4)|0;
  $169 = $168;
  $170 = HEAP32[$169>>2]|0;
  $171 = $mPrior;
  $172 = $171;
  $173 = HEAP32[$172>>2]|0;
  $174 = (($171) + 4)|0;
  $175 = $174;
  $176 = HEAP32[$175>>2]|0;
  $177 = $173 | $167;
  $178 = $176 | $170;
  $179 = $mPrior;
  $180 = $179;
  HEAP32[$180>>2] = $177;
  $181 = (($179) + 4)|0;
  $182 = $181;
  HEAP32[$182>>2] = $178;
  $183 = HEAP32[$rc>>2]|0;
  $184 = ($183|0)!=(0);
  if ($184) {
   label = 19;
   break;
  }
  $185 = HEAP32[$db>>2]|0;
  $186 = ((($185)) + 65|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $187&255;
  $189 = ($188|0)!=(0);
  if ($189) {
   label = 19;
   break;
  }
  $190 = HEAP32[$iTab>>2]|0;
  $191 = (($190) + 1)|0;
  HEAP32[$iTab>>2] = $191;
  $192 = HEAP32[$pItem>>2]|0;
  $193 = ((($192)) + 72|0);
  HEAP32[$pItem>>2] = $193;
 }
 if ((label|0) == 19) {
  $194 = HEAP32[$db>>2]|0;
  $195 = HEAP32[$pNew>>2]|0;
  _whereLoopClear($194,$195);
  $196 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($196|0);
 }
 return (0)|0;
}
function _wherePathSolver($pWInfo,$nRowEst) {
 $pWInfo = $pWInfo|0;
 $nRowEst = $nRowEst|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aFrom = 0;
 var $aSortCost = 0, $aTo = 0, $db = 0, $iLoop = 0, $ii = 0, $isOrdered = 0, $jj = 0, $maskNew = 0, $mxChoice = 0, $mxCost = 0, $mxI = 0, $mxUnsorted = 0, $nFrom = 0, $nLoop = 0, $nOrder = 0, $nOrderBy = 0, $nOut = 0, $nSpace = 0, $nTo = 0, $notUsed = 0;
 var $or$cond = 0, $pFrom = 0, $pLevel = 0, $pParse = 0, $pSpace = 0, $pTo = 0, $pWLoop = 0, $pX = 0, $rCost = 0, $rUnsorted = 0, $rc = 0, $revMask = 0, $revMask1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer = sp + 32|0;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $2 = sp + 146|0;
 $mxChoice = sp + 124|0;
 $nLoop = sp + 120|0;
 $pParse = sp + 116|0;
 $db = sp + 112|0;
 $iLoop = sp + 108|0;
 $ii = sp + 104|0;
 $jj = sp + 100|0;
 $mxI = sp + 96|0;
 $nOrderBy = sp + 92|0;
 $mxCost = sp + 144|0;
 $mxUnsorted = sp + 142|0;
 $nTo = sp + 88|0;
 $nFrom = sp + 84|0;
 $aFrom = sp + 80|0;
 $aTo = sp + 76|0;
 $pFrom = sp + 72|0;
 $pTo = sp + 68|0;
 $pWLoop = sp + 64|0;
 $pX = sp + 60|0;
 $aSortCost = sp + 56|0;
 $pSpace = sp + 52|0;
 $nSpace = sp + 48|0;
 $nOut = sp + 140|0;
 $rCost = sp + 138|0;
 $rUnsorted = sp + 136|0;
 $isOrdered = sp + 148|0;
 $maskNew = sp + 24|0;
 $revMask = sp + 16|0;
 $pLevel = sp + 44|0;
 $notUsed = sp + 8|0;
 $rc = sp + 40|0;
 $revMask1 = sp;
 $nOrder = sp + 36|0;
 HEAP32[$1>>2] = $pWInfo;
 HEAP16[$2>>1] = $nRowEst;
 HEAP32[$mxI>>2] = 0;
 HEAP16[$mxCost>>1] = 0;
 HEAP16[$mxUnsorted>>1] = 0;
 HEAP32[$aSortCost>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 41|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$nLoop>>2] = $10;
 $11 = HEAP32[$nLoop>>2]|0;
 $12 = ($11|0)<=(1);
 if ($12) {
  $16 = 1;
 } else {
  $13 = HEAP32[$nLoop>>2]|0;
  $14 = ($13|0)==(2);
  $15 = $14 ? 5 : 10;
  $16 = $15;
 }
 HEAP32[$mxChoice>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  label = 5;
 } else {
  $21 = HEAP16[$2>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22|0)==(0);
  if ($23) {
   label = 5;
  } else {
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$nOrderBy>>2] = $27;
  }
 }
 if ((label|0) == 5) {
  HEAP32[$nOrderBy>>2] = 0;
 }
 $28 = HEAP32[$nLoop>>2]|0;
 $29 = $28<<2;
 $30 = (32 + ($29))|0;
 $31 = HEAP32[$mxChoice>>2]|0;
 $32 = Math_imul($30, $31)|0;
 $33 = $32<<1;
 HEAP32[$nSpace>>2] = $33;
 $34 = HEAP32[$nOrderBy>>2]|0;
 $35 = $34<<1;
 $36 = HEAP32[$nSpace>>2]|0;
 $37 = (($36) + ($35))|0;
 HEAP32[$nSpace>>2] = $37;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$nSpace>>2]|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = (_sqlite3DbMallocRawNN($38,$39,$41)|0);
 HEAP32[$pSpace>>2] = $42;
 $43 = HEAP32[$pSpace>>2]|0;
 $44 = ($43|0)==(0|0);
 if ($44) {
  HEAP32[$0>>2] = 7;
  $678 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($678|0);
 }
 $45 = HEAP32[$pSpace>>2]|0;
 HEAP32[$aTo>>2] = $45;
 $46 = HEAP32[$aTo>>2]|0;
 $47 = HEAP32[$mxChoice>>2]|0;
 $48 = (($46) + ($47<<5)|0);
 HEAP32[$aFrom>>2] = $48;
 $49 = HEAP32[$aFrom>>2]|0;
 ;HEAP32[$49>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;HEAP32[$49+20>>2]=0|0;HEAP32[$49+24>>2]=0|0;HEAP32[$49+28>>2]=0|0;
 $50 = HEAP32[$aFrom>>2]|0;
 $51 = HEAP32[$mxChoice>>2]|0;
 $52 = (($50) + ($51<<5)|0);
 HEAP32[$pX>>2] = $52;
 $53 = HEAP32[$mxChoice>>2]|0;
 $54 = $53<<1;
 HEAP32[$ii>>2] = $54;
 $55 = HEAP32[$aTo>>2]|0;
 HEAP32[$pFrom>>2] = $55;
 while(1) {
  $56 = HEAP32[$ii>>2]|0;
  $57 = ($56|0)>(0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$pX>>2]|0;
  $59 = HEAP32[$pFrom>>2]|0;
  $60 = ((($59)) + 24|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$ii>>2]|0;
  $62 = (($61) + -1)|0;
  HEAP32[$ii>>2] = $62;
  $63 = HEAP32[$pFrom>>2]|0;
  $64 = ((($63)) + 32|0);
  HEAP32[$pFrom>>2] = $64;
  $65 = HEAP32[$nLoop>>2]|0;
  $66 = HEAP32[$pX>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  HEAP32[$pX>>2] = $67;
 }
 $68 = HEAP32[$nOrderBy>>2]|0;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$pX>>2]|0;
  HEAP32[$aSortCost>>2] = $70;
  $71 = HEAP32[$aSortCost>>2]|0;
  $72 = HEAP32[$nOrderBy>>2]|0;
  $73 = $72<<1;
  _memset(($71|0),0,($73|0))|0;
 }
 $74 = HEAP32[$pParse>>2]|0;
 $75 = ((($74)) + 428|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = ($76>>>0)<(48);
 if ($77) {
  $78 = HEAP32[$pParse>>2]|0;
  $79 = ((($78)) + 428|0);
  $80 = HEAP32[$79>>2]|0;
  $82 = $80;
 } else {
  $82 = 48;
 }
 $81 = $82&65535;
 $83 = HEAP32[$aFrom>>2]|0;
 $84 = ((($83)) + 16|0);
 HEAP16[$84>>1] = $81;
 HEAP32[$nFrom>>2] = 1;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = ($85|0)!=(0);
 if ($86) {
  $87 = HEAP32[$nLoop>>2]|0;
  $88 = ($87|0)>(0);
  $89 = HEAP32[$nOrderBy>>2]|0;
  $90 = $88 ? -1 : $89;
  $91 = $90&255;
  $92 = HEAP32[$aFrom>>2]|0;
  $93 = ((($92)) + 22|0);
  HEAP8[$93>>0] = $91;
 }
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $94 = HEAP32[$iLoop>>2]|0;
  $95 = HEAP32[$nLoop>>2]|0;
  $96 = ($94|0)<($95|0);
  if (!($96)) {
   break;
  }
  HEAP32[$nTo>>2] = 0;
  HEAP32[$ii>>2] = 0;
  $97 = HEAP32[$aFrom>>2]|0;
  HEAP32[$pFrom>>2] = $97;
  while(1) {
   $98 = HEAP32[$ii>>2]|0;
   $99 = HEAP32[$nFrom>>2]|0;
   $100 = ($98|0)<($99|0);
   if (!($100)) {
    break;
   }
   $101 = HEAP32[$1>>2]|0;
   $102 = ((($101)) + 16|0);
   $103 = HEAP32[$102>>2]|0;
   HEAP32[$pWLoop>>2] = $103;
   while(1) {
    $104 = HEAP32[$pWLoop>>2]|0;
    $105 = ($104|0)!=(0|0);
    if (!($105)) {
     break;
    }
    $106 = HEAP32[$pFrom>>2]|0;
    $107 = ((($106)) + 22|0);
    $108 = HEAP8[$107>>0]|0;
    HEAP8[$isOrdered>>0] = $108;
    $109 = $revMask;
    $110 = $109;
    HEAP32[$110>>2] = 0;
    $111 = (($109) + 4)|0;
    $112 = $111;
    HEAP32[$112>>2] = 0;
    $113 = HEAP32[$pWLoop>>2]|0;
    $114 = $113;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP32[$pFrom>>2]|0;
    $121 = $120;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = (($121) + 4)|0;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = $123 ^ -1;
    $128 = $126 ^ -1;
    $129 = $116 & $127;
    $130 = $119 & $128;
    $131 = ($129|0)!=(0);
    $132 = ($130|0)!=(0);
    $133 = $131 | $132;
    L36: do {
     if (!($133)) {
      $134 = HEAP32[$pWLoop>>2]|0;
      $135 = ((($134)) + 8|0);
      $136 = $135;
      $137 = $136;
      $138 = HEAP32[$137>>2]|0;
      $139 = (($136) + 4)|0;
      $140 = $139;
      $141 = HEAP32[$140>>2]|0;
      $142 = HEAP32[$pFrom>>2]|0;
      $143 = $142;
      $144 = $143;
      $145 = HEAP32[$144>>2]|0;
      $146 = (($143) + 4)|0;
      $147 = $146;
      $148 = HEAP32[$147>>2]|0;
      $149 = $138 & $145;
      $150 = $141 & $148;
      $151 = ($149|0)!=(0);
      $152 = ($150|0)!=(0);
      $153 = $151 | $152;
      if (!($153)) {
       $154 = HEAP32[$pWLoop>>2]|0;
       $155 = ((($154)) + 18|0);
       $156 = HEAP16[$155>>1]|0;
       $157 = HEAP32[$pWLoop>>2]|0;
       $158 = ((($157)) + 20|0);
       $159 = HEAP16[$158>>1]|0;
       $160 = $159 << 16 >> 16;
       $161 = HEAP32[$pFrom>>2]|0;
       $162 = ((($161)) + 16|0);
       $163 = HEAP16[$162>>1]|0;
       $164 = $163 << 16 >> 16;
       $165 = (($160) + ($164))|0;
       $166 = $165&65535;
       $167 = (_sqlite3LogEstAdd($156,$166)|0);
       HEAP16[$rUnsorted>>1] = $167;
       $168 = HEAP16[$rUnsorted>>1]|0;
       $169 = HEAP32[$pFrom>>2]|0;
       $170 = ((($169)) + 20|0);
       $171 = HEAP16[$170>>1]|0;
       $172 = (_sqlite3LogEstAdd($168,$171)|0);
       HEAP16[$rUnsorted>>1] = $172;
       $173 = HEAP32[$pFrom>>2]|0;
       $174 = ((($173)) + 16|0);
       $175 = HEAP16[$174>>1]|0;
       $176 = $175 << 16 >> 16;
       $177 = HEAP32[$pWLoop>>2]|0;
       $178 = ((($177)) + 22|0);
       $179 = HEAP16[$178>>1]|0;
       $180 = $179 << 16 >> 16;
       $181 = (($176) + ($180))|0;
       $182 = $181&65535;
       HEAP16[$nOut>>1] = $182;
       $183 = HEAP32[$pFrom>>2]|0;
       $184 = $183;
       $185 = $184;
       $186 = HEAP32[$185>>2]|0;
       $187 = (($184) + 4)|0;
       $188 = $187;
       $189 = HEAP32[$188>>2]|0;
       $190 = HEAP32[$pWLoop>>2]|0;
       $191 = ((($190)) + 8|0);
       $192 = $191;
       $193 = $192;
       $194 = HEAP32[$193>>2]|0;
       $195 = (($192) + 4)|0;
       $196 = $195;
       $197 = HEAP32[$196>>2]|0;
       $198 = $186 | $194;
       $199 = $189 | $197;
       $200 = $maskNew;
       $201 = $200;
       HEAP32[$201>>2] = $198;
       $202 = (($200) + 4)|0;
       $203 = $202;
       HEAP32[$203>>2] = $199;
       $204 = HEAP8[$isOrdered>>0]|0;
       $205 = $204 << 24 >> 24;
       $206 = ($205|0)<(0);
       if ($206) {
        $207 = HEAP32[$1>>2]|0;
        $208 = HEAP32[$1>>2]|0;
        $209 = ((($208)) + 8|0);
        $210 = HEAP32[$209>>2]|0;
        $211 = HEAP32[$pFrom>>2]|0;
        $212 = HEAP32[$1>>2]|0;
        $213 = ((($212)) + 34|0);
        $214 = HEAP16[$213>>1]|0;
        $215 = HEAP32[$iLoop>>2]|0;
        $216 = $215&65535;
        $217 = HEAP32[$pWLoop>>2]|0;
        $218 = (_wherePathSatisfiesOrderBy($207,$210,$211,$214,$216,$217,$revMask)|0);
        HEAP8[$isOrdered>>0] = $218;
       } else {
        $219 = HEAP32[$pFrom>>2]|0;
        $220 = ((($219)) + 8|0);
        $221 = $220;
        $222 = $221;
        $223 = HEAP32[$222>>2]|0;
        $224 = (($221) + 4)|0;
        $225 = $224;
        $226 = HEAP32[$225>>2]|0;
        $227 = $revMask;
        $228 = $227;
        HEAP32[$228>>2] = $223;
        $229 = (($227) + 4)|0;
        $230 = $229;
        HEAP32[$230>>2] = $226;
       }
       $231 = HEAP8[$isOrdered>>0]|0;
       $232 = $231 << 24 >> 24;
       $233 = ($232|0)>=(0);
       if ($233) {
        $234 = HEAP8[$isOrdered>>0]|0;
        $235 = $234 << 24 >> 24;
        $236 = HEAP32[$nOrderBy>>2]|0;
        $237 = ($235|0)<($236|0);
        if ($237) {
         $238 = HEAP8[$isOrdered>>0]|0;
         $239 = $238 << 24 >> 24;
         $240 = HEAP32[$aSortCost>>2]|0;
         $241 = (($240) + ($239<<1)|0);
         $242 = HEAP16[$241>>1]|0;
         $243 = $242 << 16 >> 16;
         $244 = ($243|0)==(0);
         if ($244) {
          $245 = HEAP16[$2>>1]|0;
          $246 = HEAP32[$nOrderBy>>2]|0;
          $247 = HEAP8[$isOrdered>>0]|0;
          $248 = $247 << 24 >> 24;
          $249 = (_whereSortingCost($245,$246,$248)|0);
          $250 = HEAP8[$isOrdered>>0]|0;
          $251 = $250 << 24 >> 24;
          $252 = HEAP32[$aSortCost>>2]|0;
          $253 = (($252) + ($251<<1)|0);
          HEAP16[$253>>1] = $249;
         }
         $254 = HEAP16[$rUnsorted>>1]|0;
         $255 = HEAP8[$isOrdered>>0]|0;
         $256 = $255 << 24 >> 24;
         $257 = HEAP32[$aSortCost>>2]|0;
         $258 = (($257) + ($256<<1)|0);
         $259 = HEAP16[$258>>1]|0;
         $260 = (_sqlite3LogEstAdd($254,$259)|0);
         HEAP16[$rCost>>1] = $260;
        } else {
         label = 34;
        }
       } else {
        label = 34;
       }
       if ((label|0) == 34) {
        label = 0;
        $261 = HEAP16[$rUnsorted>>1]|0;
        HEAP16[$rCost>>1] = $261;
       }
       HEAP32[$jj>>2] = 0;
       $262 = HEAP32[$aTo>>2]|0;
       HEAP32[$pTo>>2] = $262;
       while(1) {
        $263 = HEAP32[$jj>>2]|0;
        $264 = HEAP32[$nTo>>2]|0;
        $265 = ($263|0)<($264|0);
        if (!($265)) {
         break;
        }
        $266 = HEAP32[$pTo>>2]|0;
        $267 = $266;
        $268 = $267;
        $269 = HEAP32[$268>>2]|0;
        $270 = (($267) + 4)|0;
        $271 = $270;
        $272 = HEAP32[$271>>2]|0;
        $273 = $maskNew;
        $274 = $273;
        $275 = HEAP32[$274>>2]|0;
        $276 = (($273) + 4)|0;
        $277 = $276;
        $278 = HEAP32[$277>>2]|0;
        $279 = ($269|0)==($275|0);
        $280 = ($272|0)==($278|0);
        $281 = $279 & $280;
        if ($281) {
         $282 = HEAP32[$pTo>>2]|0;
         $283 = ((($282)) + 22|0);
         $284 = HEAP8[$283>>0]|0;
         $285 = $284 << 24 >> 24;
         $286 = HEAP8[$isOrdered>>0]|0;
         $287 = $286 << 24 >> 24;
         $288 = $285 ^ $287;
         $289 = $288 & 128;
         $290 = ($289|0)==(0);
         if ($290) {
          break;
         }
        }
        $291 = HEAP32[$jj>>2]|0;
        $292 = (($291) + 1)|0;
        HEAP32[$jj>>2] = $292;
        $293 = HEAP32[$pTo>>2]|0;
        $294 = ((($293)) + 32|0);
        HEAP32[$pTo>>2] = $294;
       }
       $295 = HEAP32[$jj>>2]|0;
       $296 = HEAP32[$nTo>>2]|0;
       $297 = ($295|0)>=($296|0);
       do {
        if ($297) {
         $298 = HEAP32[$nTo>>2]|0;
         $299 = HEAP32[$mxChoice>>2]|0;
         $300 = ($298|0)>=($299|0);
         do {
          if ($300) {
           $301 = HEAP16[$rCost>>1]|0;
           $302 = $301 << 16 >> 16;
           $303 = HEAP16[$mxCost>>1]|0;
           $304 = $303 << 16 >> 16;
           $305 = ($302|0)>($304|0);
           if ($305) {
            break L36;
           }
           $306 = HEAP16[$rCost>>1]|0;
           $307 = $306 << 16 >> 16;
           $308 = HEAP16[$mxCost>>1]|0;
           $309 = $308 << 16 >> 16;
           $310 = ($307|0)==($309|0);
           if (!($310)) {
            break;
           }
           $311 = HEAP16[$rUnsorted>>1]|0;
           $312 = $311 << 16 >> 16;
           $313 = HEAP16[$mxUnsorted>>1]|0;
           $314 = $313 << 16 >> 16;
           $315 = ($312|0)>=($314|0);
           if ($315) {
            break L36;
           }
          }
         } while(0);
         $316 = HEAP32[$nTo>>2]|0;
         $317 = HEAP32[$mxChoice>>2]|0;
         $318 = ($316|0)<($317|0);
         if ($318) {
          $319 = HEAP32[$nTo>>2]|0;
          $320 = (($319) + 1)|0;
          HEAP32[$nTo>>2] = $320;
          HEAP32[$jj>>2] = $319;
         } else {
          $321 = HEAP32[$mxI>>2]|0;
          HEAP32[$jj>>2] = $321;
         }
         $322 = HEAP32[$jj>>2]|0;
         $323 = HEAP32[$aTo>>2]|0;
         $324 = (($323) + ($322<<5)|0);
         HEAP32[$pTo>>2] = $324;
        } else {
         $325 = HEAP32[$pTo>>2]|0;
         $326 = ((($325)) + 18|0);
         $327 = HEAP16[$326>>1]|0;
         $328 = $327 << 16 >> 16;
         $329 = HEAP16[$rCost>>1]|0;
         $330 = $329 << 16 >> 16;
         $331 = ($328|0)<($330|0);
         if ($331) {
          break L36;
         }
         $332 = HEAP32[$pTo>>2]|0;
         $333 = ((($332)) + 18|0);
         $334 = HEAP16[$333>>1]|0;
         $335 = $334 << 16 >> 16;
         $336 = HEAP16[$rCost>>1]|0;
         $337 = $336 << 16 >> 16;
         $338 = ($335|0)==($337|0);
         if (!($338)) {
          break;
         }
         $339 = HEAP32[$pTo>>2]|0;
         $340 = ((($339)) + 16|0);
         $341 = HEAP16[$340>>1]|0;
         $342 = $341 << 16 >> 16;
         $343 = HEAP16[$nOut>>1]|0;
         $344 = $343 << 16 >> 16;
         $345 = ($342|0)<=($344|0);
         if ($345) {
          break L36;
         }
        }
       } while(0);
       $346 = HEAP32[$pFrom>>2]|0;
       $347 = $346;
       $348 = $347;
       $349 = HEAP32[$348>>2]|0;
       $350 = (($347) + 4)|0;
       $351 = $350;
       $352 = HEAP32[$351>>2]|0;
       $353 = HEAP32[$pWLoop>>2]|0;
       $354 = ((($353)) + 8|0);
       $355 = $354;
       $356 = $355;
       $357 = HEAP32[$356>>2]|0;
       $358 = (($355) + 4)|0;
       $359 = $358;
       $360 = HEAP32[$359>>2]|0;
       $361 = $349 | $357;
       $362 = $352 | $360;
       $363 = HEAP32[$pTo>>2]|0;
       $364 = $363;
       $365 = $364;
       HEAP32[$365>>2] = $361;
       $366 = (($364) + 4)|0;
       $367 = $366;
       HEAP32[$367>>2] = $362;
       $368 = $revMask;
       $369 = $368;
       $370 = HEAP32[$369>>2]|0;
       $371 = (($368) + 4)|0;
       $372 = $371;
       $373 = HEAP32[$372>>2]|0;
       $374 = HEAP32[$pTo>>2]|0;
       $375 = ((($374)) + 8|0);
       $376 = $375;
       $377 = $376;
       HEAP32[$377>>2] = $370;
       $378 = (($376) + 4)|0;
       $379 = $378;
       HEAP32[$379>>2] = $373;
       $380 = HEAP16[$nOut>>1]|0;
       $381 = HEAP32[$pTo>>2]|0;
       $382 = ((($381)) + 16|0);
       HEAP16[$382>>1] = $380;
       $383 = HEAP16[$rCost>>1]|0;
       $384 = HEAP32[$pTo>>2]|0;
       $385 = ((($384)) + 18|0);
       HEAP16[$385>>1] = $383;
       $386 = HEAP16[$rUnsorted>>1]|0;
       $387 = HEAP32[$pTo>>2]|0;
       $388 = ((($387)) + 20|0);
       HEAP16[$388>>1] = $386;
       $389 = HEAP8[$isOrdered>>0]|0;
       $390 = HEAP32[$pTo>>2]|0;
       $391 = ((($390)) + 22|0);
       HEAP8[$391>>0] = $389;
       $392 = HEAP32[$pTo>>2]|0;
       $393 = ((($392)) + 24|0);
       $394 = HEAP32[$393>>2]|0;
       $395 = HEAP32[$pFrom>>2]|0;
       $396 = ((($395)) + 24|0);
       $397 = HEAP32[$396>>2]|0;
       $398 = HEAP32[$iLoop>>2]|0;
       $399 = $398<<2;
       _memcpy(($394|0),($397|0),($399|0))|0;
       $400 = HEAP32[$pWLoop>>2]|0;
       $401 = HEAP32[$iLoop>>2]|0;
       $402 = HEAP32[$pTo>>2]|0;
       $403 = ((($402)) + 24|0);
       $404 = HEAP32[$403>>2]|0;
       $405 = (($404) + ($401<<2)|0);
       HEAP32[$405>>2] = $400;
       $406 = HEAP32[$nTo>>2]|0;
       $407 = HEAP32[$mxChoice>>2]|0;
       $408 = ($406|0)>=($407|0);
       if ($408) {
        HEAP32[$mxI>>2] = 0;
        $409 = HEAP32[$aTo>>2]|0;
        $410 = ((($409)) + 18|0);
        $411 = HEAP16[$410>>1]|0;
        HEAP16[$mxCost>>1] = $411;
        $412 = HEAP32[$aTo>>2]|0;
        $413 = ((($412)) + 16|0);
        $414 = HEAP16[$413>>1]|0;
        HEAP16[$mxUnsorted>>1] = $414;
        HEAP32[$jj>>2] = 1;
        $415 = HEAP32[$aTo>>2]|0;
        $416 = ((($415)) + 32|0);
        HEAP32[$pTo>>2] = $416;
        while(1) {
         $417 = HEAP32[$jj>>2]|0;
         $418 = HEAP32[$mxChoice>>2]|0;
         $419 = ($417|0)<($418|0);
         if (!($419)) {
          break L36;
         }
         $420 = HEAP32[$pTo>>2]|0;
         $421 = ((($420)) + 18|0);
         $422 = HEAP16[$421>>1]|0;
         $423 = $422 << 16 >> 16;
         $424 = HEAP16[$mxCost>>1]|0;
         $425 = $424 << 16 >> 16;
         $426 = ($423|0)>($425|0);
         do {
          if ($426) {
           label = 58;
          } else {
           $427 = HEAP32[$pTo>>2]|0;
           $428 = ((($427)) + 18|0);
           $429 = HEAP16[$428>>1]|0;
           $430 = $429 << 16 >> 16;
           $431 = HEAP16[$mxCost>>1]|0;
           $432 = $431 << 16 >> 16;
           $433 = ($430|0)==($432|0);
           if (!($433)) {
            break;
           }
           $434 = HEAP32[$pTo>>2]|0;
           $435 = ((($434)) + 20|0);
           $436 = HEAP16[$435>>1]|0;
           $437 = $436 << 16 >> 16;
           $438 = HEAP16[$mxUnsorted>>1]|0;
           $439 = $438 << 16 >> 16;
           $440 = ($437|0)>($439|0);
           if ($440) {
            label = 58;
           }
          }
         } while(0);
         if ((label|0) == 58) {
          label = 0;
          $441 = HEAP32[$pTo>>2]|0;
          $442 = ((($441)) + 18|0);
          $443 = HEAP16[$442>>1]|0;
          HEAP16[$mxCost>>1] = $443;
          $444 = HEAP32[$pTo>>2]|0;
          $445 = ((($444)) + 20|0);
          $446 = HEAP16[$445>>1]|0;
          HEAP16[$mxUnsorted>>1] = $446;
          $447 = HEAP32[$jj>>2]|0;
          HEAP32[$mxI>>2] = $447;
         }
         $448 = HEAP32[$jj>>2]|0;
         $449 = (($448) + 1)|0;
         HEAP32[$jj>>2] = $449;
         $450 = HEAP32[$pTo>>2]|0;
         $451 = ((($450)) + 32|0);
         HEAP32[$pTo>>2] = $451;
        }
       }
      }
     }
    } while(0);
    $452 = HEAP32[$pWLoop>>2]|0;
    $453 = ((($452)) + 52|0);
    $454 = HEAP32[$453>>2]|0;
    HEAP32[$pWLoop>>2] = $454;
   }
   $455 = HEAP32[$ii>>2]|0;
   $456 = (($455) + 1)|0;
   HEAP32[$ii>>2] = $456;
   $457 = HEAP32[$pFrom>>2]|0;
   $458 = ((($457)) + 32|0);
   HEAP32[$pFrom>>2] = $458;
  }
  $459 = HEAP32[$aTo>>2]|0;
  HEAP32[$pFrom>>2] = $459;
  $460 = HEAP32[$aFrom>>2]|0;
  HEAP32[$aTo>>2] = $460;
  $461 = HEAP32[$pFrom>>2]|0;
  HEAP32[$aFrom>>2] = $461;
  $462 = HEAP32[$nTo>>2]|0;
  HEAP32[$nFrom>>2] = $462;
  $463 = HEAP32[$iLoop>>2]|0;
  $464 = (($463) + 1)|0;
  HEAP32[$iLoop>>2] = $464;
 }
 $465 = HEAP32[$nFrom>>2]|0;
 $466 = ($465|0)==(0);
 if ($466) {
  $467 = HEAP32[$pParse>>2]|0;
  _sqlite3ErrorMsg($467,32694,$vararg_buffer);
  $468 = HEAP32[$db>>2]|0;
  $469 = HEAP32[$pSpace>>2]|0;
  _sqlite3DbFree($468,$469);
  HEAP32[$0>>2] = 1;
  $678 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($678|0);
 }
 $470 = HEAP32[$aFrom>>2]|0;
 HEAP32[$pFrom>>2] = $470;
 HEAP32[$ii>>2] = 1;
 while(1) {
  $471 = HEAP32[$ii>>2]|0;
  $472 = HEAP32[$nFrom>>2]|0;
  $473 = ($471|0)<($472|0);
  if (!($473)) {
   break;
  }
  $474 = HEAP32[$pFrom>>2]|0;
  $475 = ((($474)) + 18|0);
  $476 = HEAP16[$475>>1]|0;
  $477 = $476 << 16 >> 16;
  $478 = HEAP32[$ii>>2]|0;
  $479 = HEAP32[$aFrom>>2]|0;
  $480 = (($479) + ($478<<5)|0);
  $481 = ((($480)) + 18|0);
  $482 = HEAP16[$481>>1]|0;
  $483 = $482 << 16 >> 16;
  $484 = ($477|0)>($483|0);
  if ($484) {
   $485 = HEAP32[$ii>>2]|0;
   $486 = HEAP32[$aFrom>>2]|0;
   $487 = (($486) + ($485<<5)|0);
   HEAP32[$pFrom>>2] = $487;
  }
  $488 = HEAP32[$ii>>2]|0;
  $489 = (($488) + 1)|0;
  HEAP32[$ii>>2] = $489;
 }
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $490 = HEAP32[$iLoop>>2]|0;
  $491 = HEAP32[$nLoop>>2]|0;
  $492 = ($490|0)<($491|0);
  $493 = HEAP32[$1>>2]|0;
  if (!($492)) {
   break;
  }
  $494 = ((($493)) + 736|0);
  $495 = HEAP32[$iLoop>>2]|0;
  $496 = (($494) + (($495*80)|0)|0);
  HEAP32[$pLevel>>2] = $496;
  $497 = HEAP32[$iLoop>>2]|0;
  $498 = HEAP32[$pFrom>>2]|0;
  $499 = ((($498)) + 24|0);
  $500 = HEAP32[$499>>2]|0;
  $501 = (($500) + ($497<<2)|0);
  $502 = HEAP32[$501>>2]|0;
  HEAP32[$pWLoop>>2] = $502;
  $503 = HEAP32[$pLevel>>2]|0;
  $504 = ((($503)) + 64|0);
  HEAP32[$504>>2] = $502;
  $505 = HEAP32[$pWLoop>>2]|0;
  $506 = ((($505)) + 16|0);
  $507 = HEAP8[$506>>0]|0;
  $508 = HEAP32[$pLevel>>2]|0;
  $509 = ((($508)) + 44|0);
  HEAP8[$509>>0] = $507;
  $510 = HEAP32[$pLevel>>2]|0;
  $511 = ((($510)) + 44|0);
  $512 = HEAP8[$511>>0]|0;
  $513 = $512&255;
  $514 = HEAP32[$1>>2]|0;
  $515 = ((($514)) + 4|0);
  $516 = HEAP32[$515>>2]|0;
  $517 = ((($516)) + 8|0);
  $518 = (($517) + (($513*72)|0)|0);
  $519 = ((($518)) + 44|0);
  $520 = HEAP32[$519>>2]|0;
  $521 = HEAP32[$pLevel>>2]|0;
  $522 = ((($521)) + 4|0);
  HEAP32[$522>>2] = $520;
  $523 = HEAP32[$iLoop>>2]|0;
  $524 = (($523) + 1)|0;
  HEAP32[$iLoop>>2] = $524;
 }
 $525 = ((($493)) + 34|0);
 $526 = HEAP16[$525>>1]|0;
 $527 = $526&65535;
 $528 = $527 & 1024;
 $529 = ($528|0)!=(0);
 if ($529) {
  $530 = HEAP32[$1>>2]|0;
  $531 = ((($530)) + 34|0);
  $532 = HEAP16[$531>>1]|0;
  $533 = $532&65535;
  $534 = $533 & 512;
  $535 = ($534|0)==(0);
  if ($535) {
   $536 = HEAP32[$1>>2]|0;
   $537 = ((($536)) + 40|0);
   $538 = HEAP8[$537>>0]|0;
   $539 = $538&255;
   $540 = ($539|0)==(0);
   if ($540) {
    $541 = HEAP16[$2>>1]|0;
    $542 = $541 << 16 >> 16;
    $543 = ($542|0)!=(0);
    if ($543) {
     $544 = HEAP32[$1>>2]|0;
     $545 = HEAP32[$1>>2]|0;
     $546 = ((($545)) + 12|0);
     $547 = HEAP32[$546>>2]|0;
     $548 = HEAP32[$pFrom>>2]|0;
     $549 = HEAP32[$nLoop>>2]|0;
     $550 = (($549) - 1)|0;
     $551 = $550&65535;
     $552 = HEAP32[$nLoop>>2]|0;
     $553 = (($552) - 1)|0;
     $554 = HEAP32[$pFrom>>2]|0;
     $555 = ((($554)) + 24|0);
     $556 = HEAP32[$555>>2]|0;
     $557 = (($556) + ($553<<2)|0);
     $558 = HEAP32[$557>>2]|0;
     $559 = (_wherePathSatisfiesOrderBy($544,$547,$548,512,$551,$558,$notUsed)|0);
     $560 = $559 << 24 >> 24;
     HEAP32[$rc>>2] = $560;
     $561 = HEAP32[$rc>>2]|0;
     $562 = HEAP32[$1>>2]|0;
     $563 = ((($562)) + 12|0);
     $564 = HEAP32[$563>>2]|0;
     $565 = HEAP32[$564>>2]|0;
     $566 = ($561|0)==($565|0);
     if ($566) {
      $567 = HEAP32[$1>>2]|0;
      $568 = ((($567)) + 40|0);
      HEAP8[$568>>0] = 2;
     }
    }
   }
  }
 }
 $569 = HEAP32[$1>>2]|0;
 $570 = ((($569)) + 8|0);
 $571 = HEAP32[$570>>2]|0;
 $572 = ($571|0)!=(0|0);
 if ($572) {
  $573 = HEAP32[$1>>2]|0;
  $574 = ((($573)) + 34|0);
  $575 = HEAP16[$574>>1]|0;
  $576 = $575&65535;
  $577 = $576 & 512;
  $578 = ($577|0)!=(0);
  $579 = HEAP32[$pFrom>>2]|0;
  $580 = ((($579)) + 22|0);
  $581 = HEAP8[$580>>0]|0;
  if ($578) {
   $582 = $581 << 24 >> 24;
   $583 = HEAP32[$1>>2]|0;
   $584 = ((($583)) + 8|0);
   $585 = HEAP32[$584>>2]|0;
   $586 = HEAP32[$585>>2]|0;
   $587 = ($582|0)==($586|0);
   if ($587) {
    $588 = HEAP32[$1>>2]|0;
    $589 = ((($588)) + 40|0);
    HEAP8[$589>>0] = 2;
   }
  } else {
   $590 = HEAP32[$1>>2]|0;
   $591 = ((($590)) + 36|0);
   HEAP8[$591>>0] = $581;
   $592 = HEAP32[$1>>2]|0;
   $593 = ((($592)) + 36|0);
   $594 = HEAP8[$593>>0]|0;
   $595 = $594 << 24 >> 24;
   $596 = ($595|0)<(0);
   if ($596) {
    $597 = HEAP32[$1>>2]|0;
    $598 = ((($597)) + 36|0);
    HEAP8[$598>>0] = 0;
   }
   $599 = HEAP32[$pFrom>>2]|0;
   $600 = ((($599)) + 8|0);
   $601 = $600;
   $602 = $601;
   $603 = HEAP32[$602>>2]|0;
   $604 = (($601) + 4)|0;
   $605 = $604;
   $606 = HEAP32[$605>>2]|0;
   $607 = HEAP32[$1>>2]|0;
   $608 = ((($607)) + 24|0);
   $609 = $608;
   $610 = $609;
   HEAP32[$610>>2] = $603;
   $611 = (($609) + 4)|0;
   $612 = $611;
   HEAP32[$612>>2] = $606;
  }
  $613 = HEAP32[$1>>2]|0;
  $614 = ((($613)) + 34|0);
  $615 = HEAP16[$614>>1]|0;
  $616 = $615&65535;
  $617 = $616 & 2048;
  $618 = ($617|0)!=(0);
  if ($618) {
   $619 = HEAP32[$1>>2]|0;
   $620 = ((($619)) + 36|0);
   $621 = HEAP8[$620>>0]|0;
   $622 = $621 << 24 >> 24;
   $623 = HEAP32[$1>>2]|0;
   $624 = ((($623)) + 8|0);
   $625 = HEAP32[$624>>2]|0;
   $626 = HEAP32[$625>>2]|0;
   $627 = ($622|0)==($626|0);
   $628 = HEAP32[$nLoop>>2]|0;
   $629 = ($628|0)>(0);
   $or$cond = $627 & $629;
   if ($or$cond) {
    $630 = $revMask1;
    $631 = $630;
    HEAP32[$631>>2] = 0;
    $632 = (($630) + 4)|0;
    $633 = $632;
    HEAP32[$633>>2] = 0;
    $634 = HEAP32[$1>>2]|0;
    $635 = HEAP32[$1>>2]|0;
    $636 = ((($635)) + 8|0);
    $637 = HEAP32[$636>>2]|0;
    $638 = HEAP32[$pFrom>>2]|0;
    $639 = HEAP32[$nLoop>>2]|0;
    $640 = (($639) - 1)|0;
    $641 = $640&65535;
    $642 = HEAP32[$nLoop>>2]|0;
    $643 = (($642) - 1)|0;
    $644 = HEAP32[$pFrom>>2]|0;
    $645 = ((($644)) + 24|0);
    $646 = HEAP32[$645>>2]|0;
    $647 = (($646) + ($643<<2)|0);
    $648 = HEAP32[$647>>2]|0;
    $649 = (_wherePathSatisfiesOrderBy($634,$637,$638,0,$641,$648,$revMask1)|0);
    $650 = $649 << 24 >> 24;
    HEAP32[$nOrder>>2] = $650;
    $651 = HEAP32[$nOrder>>2]|0;
    $652 = HEAP32[$1>>2]|0;
    $653 = ((($652)) + 8|0);
    $654 = HEAP32[$653>>2]|0;
    $655 = HEAP32[$654>>2]|0;
    $656 = ($651|0)==($655|0);
    if ($656) {
     $657 = HEAP32[$1>>2]|0;
     $658 = ((($657)) + 37|0);
     HEAP8[$658>>0] = 1;
     $659 = $revMask1;
     $660 = $659;
     $661 = HEAP32[$660>>2]|0;
     $662 = (($659) + 4)|0;
     $663 = $662;
     $664 = HEAP32[$663>>2]|0;
     $665 = HEAP32[$1>>2]|0;
     $666 = ((($665)) + 24|0);
     $667 = $666;
     $668 = $667;
     HEAP32[$668>>2] = $661;
     $669 = (($667) + 4)|0;
     $670 = $669;
     HEAP32[$670>>2] = $664;
    }
   }
  }
 }
 $671 = HEAP32[$pFrom>>2]|0;
 $672 = ((($671)) + 16|0);
 $673 = HEAP16[$672>>1]|0;
 $674 = HEAP32[$1>>2]|0;
 $675 = ((($674)) + 32|0);
 HEAP16[$675>>1] = $673;
 $676 = HEAP32[$db>>2]|0;
 $677 = HEAP32[$pSpace>>2]|0;
 _sqlite3DbFree($676,$677);
 HEAP32[$0>>2] = 0;
 $678 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($678|0);
}
function _sqlite3WhereExprListUsage($pMaskSet,$pList) {
 $pMaskSet = $pMaskSet|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $mask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $mask = sp;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $pList;
 $2 = $mask;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $35 = $mask;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  tempRet0 = ($40);
  STACKTOP = sp;return ($37|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*20)|0)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3WhereExprUsage($12,$18)|0);
  $20 = tempRet0;
  $21 = $mask;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = $23 | $19;
  $28 = $26 | $20;
  $29 = $mask;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = $mask;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 tempRet0 = ($40);
 STACKTOP = sp;return ($37|0);
}
function _constructAutomaticIndex($pParse,$pWC,$pSrc,$0,$1,$pLevel) {
 $pParse = $pParse|0;
 $pWC = $pWC|0;
 $pSrc = $pSrc|0;
 $0 = $0|0;
 $1 = $1|0;
 $pLevel = $pLevel|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0;
 var $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0;
 var $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0;
 var $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addrCounter = 0, $addrInit = 0, $addrTop = 0, $cMask = 0, $cMask2 = 0, $extraCols = 0, $i = 0, $iCol = 0, $iCol1 = 0, $iContinue = 0, $idxCols = 0, $mxBitCol = 0, $n = 0, $nKeyCol = 0, $pColl = 0, $pExpr = 0, $pIdx = 0, $pLoop = 0, $pPartial = 0;
 var $pTabItem = 0, $pTable = 0, $pTerm = 0, $pWCEnd = 0, $pX = 0, $regBase = 0, $regRecord = 0, $regYield = 0, $sentWarning = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zNotUsed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 40|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $4 = sp + 152|0;
 $5 = sp + 32|0;
 $6 = sp + 148|0;
 $nKeyCol = sp + 144|0;
 $pTerm = sp + 140|0;
 $pWCEnd = sp + 136|0;
 $pIdx = sp + 132|0;
 $v = sp + 128|0;
 $addrInit = sp + 124|0;
 $pTable = sp + 120|0;
 $addrTop = sp + 116|0;
 $regRecord = sp + 112|0;
 $n = sp + 108|0;
 $i = sp + 104|0;
 $mxBitCol = sp + 100|0;
 $pColl = sp + 96|0;
 $pLoop = sp + 92|0;
 $zNotUsed = sp + 88|0;
 $idxCols = sp + 24|0;
 $extraCols = sp + 16|0;
 $sentWarning = sp + 164|0;
 $pPartial = sp + 84|0;
 $iContinue = sp + 80|0;
 $pTabItem = sp + 76|0;
 $addrCounter = sp + 72|0;
 $regBase = sp + 68|0;
 $pExpr = sp + 64|0;
 $iCol = sp + 60|0;
 $cMask = sp + 8|0;
 $iCol1 = sp + 56|0;
 $cMask2 = sp;
 $pX = sp + 52|0;
 $regYield = sp + 48|0;
 HEAP32[$2>>2] = $pParse;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $pSrc;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$6>>2] = $pLevel;
 HEAP8[$sentWarning>>0] = 0;
 HEAP32[$pPartial>>2] = 0;
 HEAP32[$iContinue>>2] = 0;
 HEAP32[$addrCounter>>2] = 0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3CodeOnce($14)|0);
 HEAP32[$addrInit>>2] = $15;
 HEAP32[$nKeyCol>>2] = 0;
 $16 = HEAP32[$4>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pTable>>2] = $18;
 $19 = HEAP32[$3>>2]|0;
 $20 = ((($19)) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = ((($22)) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + (($21*48)|0)|0);
 HEAP32[$pWCEnd>>2] = $25;
 $26 = HEAP32[$6>>2]|0;
 $27 = ((($26)) + 64|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pLoop>>2] = $28;
 $29 = $idxCols;
 $30 = $29;
 HEAP32[$30>>2] = 0;
 $31 = (($29) + 4)|0;
 $32 = $31;
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$3>>2]|0;
 $34 = ((($33)) + 20|0);
 $35 = HEAP32[$34>>2]|0;
 HEAP32[$pTerm>>2] = $35;
 while(1) {
  $36 = HEAP32[$pTerm>>2]|0;
  $37 = HEAP32[$pWCEnd>>2]|0;
  $38 = ($36>>>0)<($37>>>0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$pTerm>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$pExpr>>2] = $40;
  $41 = HEAP32[$pLoop>>2]|0;
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = ($44|0)==(0);
  $49 = ($47|0)==(0);
  $50 = $48 & $49;
  if ($50) {
   $51 = HEAP32[$pTerm>>2]|0;
   $52 = ((($51)) + 20|0);
   $53 = HEAP16[$52>>1]|0;
   $54 = $53&65535;
   $55 = $54 & 2;
   $56 = ($55|0)==(0);
   if ($56) {
    $57 = HEAP32[$pExpr>>2]|0;
    $58 = ((($57)) + 4|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = $59 & 1;
    $61 = ($60|0)!=(0);
    if (!($61)) {
     $62 = HEAP32[$pExpr>>2]|0;
     $63 = HEAP32[$4>>2]|0;
     $64 = ((($63)) + 44|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = (_sqlite3ExprIsTableConstant($62,$65)|0);
     $67 = ($66|0)!=(0);
     if ($67) {
      $68 = HEAP32[$2>>2]|0;
      $69 = HEAP32[$68>>2]|0;
      $70 = HEAP32[$pPartial>>2]|0;
      $71 = HEAP32[$2>>2]|0;
      $72 = HEAP32[$71>>2]|0;
      $73 = HEAP32[$pExpr>>2]|0;
      $74 = (_sqlite3ExprDup($72,$73,0)|0);
      $75 = (_sqlite3ExprAnd($69,$70,$74)|0);
      HEAP32[$pPartial>>2] = $75;
     }
    }
   }
  }
  $76 = HEAP32[$pTerm>>2]|0;
  $77 = HEAP32[$4>>2]|0;
  $78 = $5;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = (_termCanDriveIndex($76,$77,$80,$83)|0);
  $85 = ($84|0)!=(0);
  if ($85) {
   $86 = HEAP32[$pTerm>>2]|0;
   $87 = ((($86)) + 12|0);
   $88 = HEAP32[$87>>2]|0;
   HEAP32[$iCol>>2] = $88;
   $89 = HEAP32[$iCol>>2]|0;
   $90 = ($89|0)>=(64);
   if ($90) {
    $96 = 0;$99 = -2147483648;
   } else {
    $91 = HEAP32[$iCol>>2]|0;
    $92 = (_bitshift64Shl(1,0,($91|0))|0);
    $93 = tempRet0;
    $96 = $92;$99 = $93;
   }
   $94 = $cMask;
   $95 = $94;
   HEAP32[$95>>2] = $96;
   $97 = (($94) + 4)|0;
   $98 = $97;
   HEAP32[$98>>2] = $99;
   $100 = HEAP8[$sentWarning>>0]|0;
   $101 = ($100<<24>>24)!=(0);
   if (!($101)) {
    $102 = HEAP32[$pTable>>2]|0;
    $103 = HEAP32[$102>>2]|0;
    $104 = HEAP32[$iCol>>2]|0;
    $105 = HEAP32[$pTable>>2]|0;
    $106 = ((($105)) + 4|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = (($107) + (($104*24)|0)|0);
    $109 = HEAP32[$108>>2]|0;
    HEAP32[$vararg_buffer>>2] = $103;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = $109;
    _sqlite3_log(284,32712,$vararg_buffer);
    HEAP8[$sentWarning>>0] = 1;
   }
   $110 = $idxCols;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($110) + 4)|0;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = $cMask;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($116) + 4)|0;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = $112 & $118;
   $123 = $115 & $121;
   $124 = ($122|0)==(0);
   $125 = ($123|0)==(0);
   $126 = $124 & $125;
   if ($126) {
    $127 = HEAP32[$2>>2]|0;
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$pLoop>>2]|0;
    $130 = HEAP32[$nKeyCol>>2]|0;
    $131 = (($130) + 1)|0;
    $132 = (_whereLoopResize($128,$129,$131)|0);
    $133 = ($132|0)!=(0);
    if ($133) {
     label = 57;
     break;
    }
    $134 = HEAP32[$pTerm>>2]|0;
    $135 = HEAP32[$nKeyCol>>2]|0;
    $136 = (($135) + 1)|0;
    HEAP32[$nKeyCol>>2] = $136;
    $137 = HEAP32[$pLoop>>2]|0;
    $138 = ((($137)) + 48|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = (($139) + ($135<<2)|0);
    HEAP32[$140>>2] = $134;
    $141 = $cMask;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = (($141) + 4)|0;
    $145 = $144;
    $146 = HEAP32[$145>>2]|0;
    $147 = $idxCols;
    $148 = $147;
    $149 = HEAP32[$148>>2]|0;
    $150 = (($147) + 4)|0;
    $151 = $150;
    $152 = HEAP32[$151>>2]|0;
    $153 = $149 | $143;
    $154 = $152 | $146;
    $155 = $idxCols;
    $156 = $155;
    HEAP32[$156>>2] = $153;
    $157 = (($155) + 4)|0;
    $158 = $157;
    HEAP32[$158>>2] = $154;
   }
  }
  $159 = HEAP32[$pTerm>>2]|0;
  $160 = ((($159)) + 48|0);
  HEAP32[$pTerm>>2] = $160;
 }
 if ((label|0) == 57) {
  $560 = HEAP32[$2>>2]|0;
  $561 = HEAP32[$560>>2]|0;
  $562 = HEAP32[$pPartial>>2]|0;
  _sqlite3ExprDelete($561,$562);
  STACKTOP = sp;return;
 }
 $161 = HEAP32[$nKeyCol>>2]|0;
 $162 = $161&65535;
 $163 = HEAP32[$pLoop>>2]|0;
 $164 = ((($163)) + 40|0);
 HEAP16[$164>>1] = $162;
 $165 = HEAP32[$pLoop>>2]|0;
 $166 = ((($165)) + 24|0);
 HEAP16[$166>>1] = $162;
 $167 = HEAP32[$pLoop>>2]|0;
 $168 = ((($167)) + 36|0);
 HEAP32[$168>>2] = 16961;
 $169 = HEAP32[$4>>2]|0;
 $170 = ((($169)) + 56|0);
 $171 = $170;
 $172 = $171;
 $173 = HEAP32[$172>>2]|0;
 $174 = (($171) + 4)|0;
 $175 = $174;
 $176 = HEAP32[$175>>2]|0;
 $177 = $idxCols;
 $178 = $177;
 $179 = HEAP32[$178>>2]|0;
 $180 = (($177) + 4)|0;
 $181 = $180;
 $182 = HEAP32[$181>>2]|0;
 $183 = $179 ^ -1;
 $184 = $182 ^ -1;
 $185 = $184 | -2147483648;
 $186 = $173 & $183;
 $187 = $176 & $185;
 $188 = $extraCols;
 $189 = $188;
 HEAP32[$189>>2] = $186;
 $190 = (($188) + 4)|0;
 $191 = $190;
 HEAP32[$191>>2] = $187;
 $192 = HEAP32[$pTable>>2]|0;
 $193 = ((($192)) + 34|0);
 $194 = HEAP16[$193>>1]|0;
 $195 = $194 << 16 >> 16;
 $196 = (63)<($195|0);
 if ($196) {
  $201 = 63;
 } else {
  $197 = HEAP32[$pTable>>2]|0;
  $198 = ((($197)) + 34|0);
  $199 = HEAP16[$198>>1]|0;
  $200 = $199 << 16 >> 16;
  $201 = $200;
 }
 HEAP32[$mxBitCol>>2] = $201;
 HEAP32[$i>>2] = 0;
 while(1) {
  $202 = HEAP32[$i>>2]|0;
  $203 = HEAP32[$mxBitCol>>2]|0;
  $204 = ($202|0)<($203|0);
  if (!($204)) {
   break;
  }
  $205 = $extraCols;
  $206 = $205;
  $207 = HEAP32[$206>>2]|0;
  $208 = (($205) + 4)|0;
  $209 = $208;
  $210 = HEAP32[$209>>2]|0;
  $211 = HEAP32[$i>>2]|0;
  $212 = (_bitshift64Shl(1,0,($211|0))|0);
  $213 = tempRet0;
  $214 = $207 & $212;
  $215 = $210 & $213;
  $216 = ($214|0)!=(0);
  $217 = ($215|0)!=(0);
  $218 = $216 | $217;
  if ($218) {
   $219 = HEAP32[$nKeyCol>>2]|0;
   $220 = (($219) + 1)|0;
   HEAP32[$nKeyCol>>2] = $220;
  }
  $221 = HEAP32[$i>>2]|0;
  $222 = (($221) + 1)|0;
  HEAP32[$i>>2] = $222;
 }
 $223 = HEAP32[$4>>2]|0;
 $224 = ((($223)) + 56|0);
 $225 = $224;
 $226 = $225;
 $227 = HEAP32[$226>>2]|0;
 $228 = (($225) + 4)|0;
 $229 = $228;
 $230 = HEAP32[$229>>2]|0;
 $231 = $230 & -2147483648;
 $232 = (0)!=(0);
 $233 = ($231|0)!=(0);
 $234 = $232 | $233;
 if ($234) {
  $235 = HEAP32[$pTable>>2]|0;
  $236 = ((($235)) + 34|0);
  $237 = HEAP16[$236>>1]|0;
  $238 = $237 << 16 >> 16;
  $239 = (($238) - 64)|0;
  $240 = (($239) + 1)|0;
  $241 = HEAP32[$nKeyCol>>2]|0;
  $242 = (($241) + ($240))|0;
  HEAP32[$nKeyCol>>2] = $242;
 }
 $243 = HEAP32[$2>>2]|0;
 $244 = HEAP32[$243>>2]|0;
 $245 = HEAP32[$nKeyCol>>2]|0;
 $246 = (($245) + 1)|0;
 $247 = $246&65535;
 $248 = (_sqlite3AllocateIndexObject($244,$247,0,$zNotUsed)|0);
 HEAP32[$pIdx>>2] = $248;
 $249 = HEAP32[$pIdx>>2]|0;
 $250 = ($249|0)==(0|0);
 if ($250) {
  $560 = HEAP32[$2>>2]|0;
  $561 = HEAP32[$560>>2]|0;
  $562 = HEAP32[$pPartial>>2]|0;
  _sqlite3ExprDelete($561,$562);
  STACKTOP = sp;return;
 }
 $251 = HEAP32[$pIdx>>2]|0;
 $252 = HEAP32[$pLoop>>2]|0;
 $253 = ((($252)) + 24|0);
 $254 = ((($253)) + 4|0);
 HEAP32[$254>>2] = $251;
 $255 = HEAP32[$pIdx>>2]|0;
 HEAP32[$255>>2] = 32738;
 $256 = HEAP32[$pTable>>2]|0;
 $257 = HEAP32[$pIdx>>2]|0;
 $258 = ((($257)) + 12|0);
 HEAP32[$258>>2] = $256;
 HEAP32[$n>>2] = 0;
 $259 = $idxCols;
 $260 = $259;
 HEAP32[$260>>2] = 0;
 $261 = (($259) + 4)|0;
 $262 = $261;
 HEAP32[$262>>2] = 0;
 $263 = HEAP32[$3>>2]|0;
 $264 = ((($263)) + 20|0);
 $265 = HEAP32[$264>>2]|0;
 HEAP32[$pTerm>>2] = $265;
 while(1) {
  $266 = HEAP32[$pTerm>>2]|0;
  $267 = HEAP32[$pWCEnd>>2]|0;
  $268 = ($266>>>0)<($267>>>0);
  if (!($268)) {
   break;
  }
  $269 = HEAP32[$pTerm>>2]|0;
  $270 = HEAP32[$4>>2]|0;
  $271 = $5;
  $272 = $271;
  $273 = HEAP32[$272>>2]|0;
  $274 = (($271) + 4)|0;
  $275 = $274;
  $276 = HEAP32[$275>>2]|0;
  $277 = (_termCanDriveIndex($269,$270,$273,$276)|0);
  $278 = ($277|0)!=(0);
  if ($278) {
   $279 = HEAP32[$pTerm>>2]|0;
   $280 = ((($279)) + 12|0);
   $281 = HEAP32[$280>>2]|0;
   HEAP32[$iCol1>>2] = $281;
   $282 = HEAP32[$iCol1>>2]|0;
   $283 = ($282|0)>=(64);
   if ($283) {
    $289 = 0;$292 = -2147483648;
   } else {
    $284 = HEAP32[$iCol1>>2]|0;
    $285 = (_bitshift64Shl(1,0,($284|0))|0);
    $286 = tempRet0;
    $289 = $285;$292 = $286;
   }
   $287 = $cMask2;
   $288 = $287;
   HEAP32[$288>>2] = $289;
   $290 = (($287) + 4)|0;
   $291 = $290;
   HEAP32[$291>>2] = $292;
   $293 = $idxCols;
   $294 = $293;
   $295 = HEAP32[$294>>2]|0;
   $296 = (($293) + 4)|0;
   $297 = $296;
   $298 = HEAP32[$297>>2]|0;
   $299 = $cMask2;
   $300 = $299;
   $301 = HEAP32[$300>>2]|0;
   $302 = (($299) + 4)|0;
   $303 = $302;
   $304 = HEAP32[$303>>2]|0;
   $305 = $295 & $301;
   $306 = $298 & $304;
   $307 = ($305|0)==(0);
   $308 = ($306|0)==(0);
   $309 = $307 & $308;
   if ($309) {
    $310 = HEAP32[$pTerm>>2]|0;
    $311 = HEAP32[$310>>2]|0;
    HEAP32[$pX>>2] = $311;
    $312 = $cMask2;
    $313 = $312;
    $314 = HEAP32[$313>>2]|0;
    $315 = (($312) + 4)|0;
    $316 = $315;
    $317 = HEAP32[$316>>2]|0;
    $318 = $idxCols;
    $319 = $318;
    $320 = HEAP32[$319>>2]|0;
    $321 = (($318) + 4)|0;
    $322 = $321;
    $323 = HEAP32[$322>>2]|0;
    $324 = $320 | $314;
    $325 = $323 | $317;
    $326 = $idxCols;
    $327 = $326;
    HEAP32[$327>>2] = $324;
    $328 = (($326) + 4)|0;
    $329 = $328;
    HEAP32[$329>>2] = $325;
    $330 = HEAP32[$pTerm>>2]|0;
    $331 = ((($330)) + 12|0);
    $332 = HEAP32[$331>>2]|0;
    $333 = $332&65535;
    $334 = HEAP32[$n>>2]|0;
    $335 = HEAP32[$pIdx>>2]|0;
    $336 = ((($335)) + 4|0);
    $337 = HEAP32[$336>>2]|0;
    $338 = (($337) + ($334<<1)|0);
    HEAP16[$338>>1] = $333;
    $339 = HEAP32[$2>>2]|0;
    $340 = HEAP32[$pX>>2]|0;
    $341 = ((($340)) + 12|0);
    $342 = HEAP32[$341>>2]|0;
    $343 = HEAP32[$pX>>2]|0;
    $344 = ((($343)) + 16|0);
    $345 = HEAP32[$344>>2]|0;
    $346 = (_sqlite3BinaryCompareCollSeq($339,$342,$345)|0);
    HEAP32[$pColl>>2] = $346;
    $347 = HEAP32[$pColl>>2]|0;
    $348 = ($347|0)!=(0|0);
    if ($348) {
     $349 = HEAP32[$pColl>>2]|0;
     $350 = HEAP32[$349>>2]|0;
     $356 = $350;
    } else {
     $356 = 20716;
    }
    $351 = HEAP32[$n>>2]|0;
    $352 = HEAP32[$pIdx>>2]|0;
    $353 = ((($352)) + 32|0);
    $354 = HEAP32[$353>>2]|0;
    $355 = (($354) + ($351<<2)|0);
    HEAP32[$355>>2] = $356;
    $357 = HEAP32[$n>>2]|0;
    $358 = (($357) + 1)|0;
    HEAP32[$n>>2] = $358;
   }
  }
  $359 = HEAP32[$pTerm>>2]|0;
  $360 = ((($359)) + 48|0);
  HEAP32[$pTerm>>2] = $360;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $361 = HEAP32[$i>>2]|0;
  $362 = HEAP32[$mxBitCol>>2]|0;
  $363 = ($361|0)<($362|0);
  if (!($363)) {
   break;
  }
  $364 = $extraCols;
  $365 = $364;
  $366 = HEAP32[$365>>2]|0;
  $367 = (($364) + 4)|0;
  $368 = $367;
  $369 = HEAP32[$368>>2]|0;
  $370 = HEAP32[$i>>2]|0;
  $371 = (_bitshift64Shl(1,0,($370|0))|0);
  $372 = tempRet0;
  $373 = $366 & $371;
  $374 = $369 & $372;
  $375 = ($373|0)!=(0);
  $376 = ($374|0)!=(0);
  $377 = $375 | $376;
  if ($377) {
   $378 = HEAP32[$i>>2]|0;
   $379 = $378&65535;
   $380 = HEAP32[$n>>2]|0;
   $381 = HEAP32[$pIdx>>2]|0;
   $382 = ((($381)) + 4|0);
   $383 = HEAP32[$382>>2]|0;
   $384 = (($383) + ($380<<1)|0);
   HEAP16[$384>>1] = $379;
   $385 = HEAP32[$n>>2]|0;
   $386 = HEAP32[$pIdx>>2]|0;
   $387 = ((($386)) + 32|0);
   $388 = HEAP32[$387>>2]|0;
   $389 = (($388) + ($385<<2)|0);
   HEAP32[$389>>2] = 20716;
   $390 = HEAP32[$n>>2]|0;
   $391 = (($390) + 1)|0;
   HEAP32[$n>>2] = $391;
  }
  $392 = HEAP32[$i>>2]|0;
  $393 = (($392) + 1)|0;
  HEAP32[$i>>2] = $393;
 }
 $394 = HEAP32[$4>>2]|0;
 $395 = ((($394)) + 56|0);
 $396 = $395;
 $397 = $396;
 $398 = HEAP32[$397>>2]|0;
 $399 = (($396) + 4)|0;
 $400 = $399;
 $401 = HEAP32[$400>>2]|0;
 $402 = $401 & -2147483648;
 $403 = (0)!=(0);
 $404 = ($402|0)!=(0);
 $405 = $403 | $404;
 L61: do {
  if ($405) {
   HEAP32[$i>>2] = 63;
   while(1) {
    $406 = HEAP32[$i>>2]|0;
    $407 = HEAP32[$pTable>>2]|0;
    $408 = ((($407)) + 34|0);
    $409 = HEAP16[$408>>1]|0;
    $410 = $409 << 16 >> 16;
    $411 = ($406|0)<($410|0);
    if (!($411)) {
     break L61;
    }
    $412 = HEAP32[$i>>2]|0;
    $413 = $412&65535;
    $414 = HEAP32[$n>>2]|0;
    $415 = HEAP32[$pIdx>>2]|0;
    $416 = ((($415)) + 4|0);
    $417 = HEAP32[$416>>2]|0;
    $418 = (($417) + ($414<<1)|0);
    HEAP16[$418>>1] = $413;
    $419 = HEAP32[$n>>2]|0;
    $420 = HEAP32[$pIdx>>2]|0;
    $421 = ((($420)) + 32|0);
    $422 = HEAP32[$421>>2]|0;
    $423 = (($422) + ($419<<2)|0);
    HEAP32[$423>>2] = 20716;
    $424 = HEAP32[$n>>2]|0;
    $425 = (($424) + 1)|0;
    HEAP32[$n>>2] = $425;
    $426 = HEAP32[$i>>2]|0;
    $427 = (($426) + 1)|0;
    HEAP32[$i>>2] = $427;
   }
  }
 } while(0);
 $428 = HEAP32[$n>>2]|0;
 $429 = HEAP32[$pIdx>>2]|0;
 $430 = ((($429)) + 4|0);
 $431 = HEAP32[$430>>2]|0;
 $432 = (($431) + ($428<<1)|0);
 HEAP16[$432>>1] = -1;
 $433 = HEAP32[$n>>2]|0;
 $434 = HEAP32[$pIdx>>2]|0;
 $435 = ((($434)) + 32|0);
 $436 = HEAP32[$435>>2]|0;
 $437 = (($436) + ($433<<2)|0);
 HEAP32[$437>>2] = 20716;
 $438 = HEAP32[$2>>2]|0;
 $439 = ((($438)) + 72|0);
 $440 = HEAP32[$439>>2]|0;
 $441 = (($440) + 1)|0;
 HEAP32[$439>>2] = $441;
 $442 = HEAP32[$6>>2]|0;
 $443 = ((($442)) + 8|0);
 HEAP32[$443>>2] = $440;
 $444 = HEAP32[$v>>2]|0;
 $445 = HEAP32[$6>>2]|0;
 $446 = ((($445)) + 8|0);
 $447 = HEAP32[$446>>2]|0;
 $448 = HEAP32[$nKeyCol>>2]|0;
 $449 = (($448) + 1)|0;
 (_sqlite3VdbeAddOp2($444,56,$447,$449)|0);
 $450 = HEAP32[$2>>2]|0;
 $451 = HEAP32[$pIdx>>2]|0;
 _sqlite3VdbeSetP4KeyInfo($450,$451);
 $452 = HEAP32[$2>>2]|0;
 _sqlite3ExprCachePush($452);
 $453 = HEAP32[$6>>2]|0;
 $454 = ((($453)) + 44|0);
 $455 = HEAP8[$454>>0]|0;
 $456 = $455&255;
 $457 = HEAP32[$3>>2]|0;
 $458 = HEAP32[$457>>2]|0;
 $459 = ((($458)) + 4|0);
 $460 = HEAP32[$459>>2]|0;
 $461 = ((($460)) + 8|0);
 $462 = (($461) + (($456*72)|0)|0);
 HEAP32[$pTabItem>>2] = $462;
 $463 = HEAP32[$pTabItem>>2]|0;
 $464 = ((($463)) + 36|0);
 $465 = ((($464)) + 1|0);
 $466 = HEAP8[$465>>0]|0;
 $467 = ($466&255) >>> 4;
 $468 = $467 & 1;
 $469 = $468&255;
 $470 = ($469|0)!=(0);
 if ($470) {
  $471 = HEAP32[$pTabItem>>2]|0;
  $472 = ((($471)) + 28|0);
  $473 = HEAP32[$472>>2]|0;
  HEAP32[$regYield>>2] = $473;
  $474 = HEAP32[$v>>2]|0;
  $475 = (_sqlite3VdbeAddOp2($474,22,0,0)|0);
  HEAP32[$addrCounter>>2] = $475;
  $476 = HEAP32[$v>>2]|0;
  $477 = HEAP32[$regYield>>2]|0;
  $478 = HEAP32[$pTabItem>>2]|0;
  $479 = ((($478)) + 24|0);
  $480 = HEAP32[$479>>2]|0;
  (_sqlite3VdbeAddOp3($476,16,$477,0,$480)|0);
  $481 = HEAP32[$v>>2]|0;
  $482 = HEAP32[$regYield>>2]|0;
  $483 = (_sqlite3VdbeAddOp1($481,18,$482)|0);
  HEAP32[$addrTop>>2] = $483;
 } else {
  $484 = HEAP32[$v>>2]|0;
  $485 = HEAP32[$6>>2]|0;
  $486 = ((($485)) + 4|0);
  $487 = HEAP32[$486>>2]|0;
  $488 = (_sqlite3VdbeAddOp1($484,108,$487)|0);
  HEAP32[$addrTop>>2] = $488;
 }
 $489 = HEAP32[$pPartial>>2]|0;
 $490 = ($489|0)!=(0|0);
 if ($490) {
  $491 = HEAP32[$v>>2]|0;
  $492 = (_sqlite3VdbeMakeLabel($491)|0);
  HEAP32[$iContinue>>2] = $492;
  $493 = HEAP32[$2>>2]|0;
  $494 = HEAP32[$pPartial>>2]|0;
  $495 = HEAP32[$iContinue>>2]|0;
  _sqlite3ExprIfFalse($493,$494,$495,16);
  $496 = HEAP32[$pLoop>>2]|0;
  $497 = ((($496)) + 36|0);
  $498 = HEAP32[$497>>2]|0;
  $499 = $498 | 131072;
  HEAP32[$497>>2] = $499;
 }
 $500 = HEAP32[$2>>2]|0;
 $501 = (_sqlite3GetTempReg($500)|0);
 HEAP32[$regRecord>>2] = $501;
 $502 = HEAP32[$2>>2]|0;
 $503 = HEAP32[$pIdx>>2]|0;
 $504 = HEAP32[$6>>2]|0;
 $505 = ((($504)) + 4|0);
 $506 = HEAP32[$505>>2]|0;
 $507 = HEAP32[$regRecord>>2]|0;
 $508 = (_sqlite3GenerateIndexKey($502,$503,$506,$507,0,0,0,0)|0);
 HEAP32[$regBase>>2] = $508;
 $509 = HEAP32[$v>>2]|0;
 $510 = HEAP32[$6>>2]|0;
 $511 = ((($510)) + 8|0);
 $512 = HEAP32[$511>>2]|0;
 $513 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($509,110,$512,$513)|0);
 $514 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($514,16);
 $515 = HEAP32[$pPartial>>2]|0;
 $516 = ($515|0)!=(0|0);
 if ($516) {
  $517 = HEAP32[$v>>2]|0;
  $518 = HEAP32[$iContinue>>2]|0;
  _sqlite3VdbeResolveLabel($517,$518);
 }
 $519 = HEAP32[$pTabItem>>2]|0;
 $520 = ((($519)) + 36|0);
 $521 = ((($520)) + 1|0);
 $522 = HEAP8[$521>>0]|0;
 $523 = ($522&255) >>> 4;
 $524 = $523 & 1;
 $525 = $524&255;
 $526 = ($525|0)!=(0);
 $527 = HEAP32[$v>>2]|0;
 if ($526) {
  $528 = HEAP32[$addrCounter>>2]|0;
  $529 = HEAP32[$regBase>>2]|0;
  $530 = HEAP32[$n>>2]|0;
  $531 = (($529) + ($530))|0;
  _sqlite3VdbeChangeP2($527,$528,$531);
  $532 = HEAP32[$v>>2]|0;
  $533 = HEAP32[$addrTop>>2]|0;
  $534 = HEAP32[$6>>2]|0;
  $535 = ((($534)) + 4|0);
  $536 = HEAP32[$535>>2]|0;
  $537 = HEAP32[$pTabItem>>2]|0;
  $538 = ((($537)) + 32|0);
  $539 = HEAP32[$538>>2]|0;
  _translateColumnToCopy($532,$533,$536,$539,1);
  $540 = HEAP32[$v>>2]|0;
  $541 = HEAP32[$addrTop>>2]|0;
  (_sqlite3VdbeGoto($540,$541)|0);
  $542 = HEAP32[$pTabItem>>2]|0;
  $543 = ((($542)) + 36|0);
  $544 = ((($543)) + 1|0);
  $545 = HEAP8[$544>>0]|0;
  $546 = $545 & -17;
  HEAP8[$544>>0] = $546;
 } else {
  $547 = HEAP32[$6>>2]|0;
  $548 = ((($547)) + 4|0);
  $549 = HEAP32[$548>>2]|0;
  $550 = HEAP32[$addrTop>>2]|0;
  $551 = (($550) + 1)|0;
  (_sqlite3VdbeAddOp2($527,7,$549,$551)|0);
 }
 $552 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($552,3);
 $553 = HEAP32[$v>>2]|0;
 $554 = HEAP32[$addrTop>>2]|0;
 _sqlite3VdbeJumpHere($553,$554);
 $555 = HEAP32[$2>>2]|0;
 $556 = HEAP32[$regRecord>>2]|0;
 _sqlite3ReleaseTempReg($555,$556);
 $557 = HEAP32[$2>>2]|0;
 _sqlite3ExprCachePop($557);
 $558 = HEAP32[$v>>2]|0;
 $559 = HEAP32[$addrInit>>2]|0;
 _sqlite3VdbeJumpHere($558,$559);
 $560 = HEAP32[$2>>2]|0;
 $561 = HEAP32[$560>>2]|0;
 $562 = HEAP32[$pPartial>>2]|0;
 _sqlite3ExprDelete($561,$562);
 STACKTOP = sp;return;
}
function _sqlite3WhereExplainOneScan($pParse,$pTabList,$pLevel,$iLevel,$iFrom,$wctrlFlags) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pLevel = $pLevel|0;
 $iLevel = $iLevel|0;
 $iFrom = $iFrom|0;
 $wctrlFlags = $wctrlFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $flags = 0, $iId = 0, $isSearch = 0, $pIdx = 0, $pItem = 0, $pLoop = 0, $ret = 0, $str = 0;
 var $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr16 = 0, $zBuf = 0, $zFmt = 0, $zMsg = 0, $zRangeOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 144|0;
 $1 = sp + 140|0;
 $2 = sp + 136|0;
 $3 = sp + 132|0;
 $4 = sp + 128|0;
 $5 = sp + 124|0;
 $6 = sp + 148|0;
 $ret = sp + 120|0;
 $pItem = sp + 116|0;
 $v = sp + 112|0;
 $db = sp + 108|0;
 $iId = sp + 104|0;
 $isSearch = sp + 100|0;
 $pLoop = sp + 96|0;
 $flags = sp + 92|0;
 $zMsg = sp + 88|0;
 $str = sp + 60|0;
 $zBuf = sp + 152|0;
 $zFmt = sp + 56|0;
 $pIdx = sp + 52|0;
 $zRangeOp = sp + 48|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pLevel;
 HEAP32[$4>>2] = $iLevel;
 HEAP32[$5>>2] = $iFrom;
 HEAP16[$6>>1] = $wctrlFlags;
 HEAP32[$ret>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 453|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)==(2);
 do {
  if ($11) {
   $12 = HEAP32[$3>>2]|0;
   $13 = ((($12)) + 44|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 8|0);
   $18 = (($17) + (($15*72)|0)|0);
   HEAP32[$pItem>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$v>>2] = $21;
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$db>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 468|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$iId>>2] = $26;
   $27 = HEAP32[$3>>2]|0;
   $28 = ((($27)) + 64|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$pLoop>>2] = $29;
   $30 = HEAP32[$pLoop>>2]|0;
   $31 = ((($30)) + 36|0);
   $32 = HEAP32[$31>>2]|0;
   HEAP32[$flags>>2] = $32;
   $33 = HEAP32[$flags>>2]|0;
   $34 = $33 & 8192;
   $35 = ($34|0)!=(0);
   if (!($35)) {
    $36 = HEAP16[$6>>1]|0;
    $37 = $36&65535;
    $38 = $37 & 64;
    $39 = ($38|0)!=(0);
    if (!($39)) {
     $40 = HEAP32[$flags>>2]|0;
     $41 = $40 & 48;
     $42 = ($41|0)!=(0);
     do {
      if ($42) {
       $56 = 1;
      } else {
       $43 = HEAP32[$flags>>2]|0;
       $44 = $43 & 1024;
       $45 = ($44|0)==(0);
       if ($45) {
        $46 = HEAP32[$pLoop>>2]|0;
        $47 = ((($46)) + 24|0);
        $48 = HEAP16[$47>>1]|0;
        $49 = $48&65535;
        $50 = ($49|0)>(0);
        if ($50) {
         $56 = 1;
         break;
        }
       }
       $51 = HEAP16[$6>>1]|0;
       $52 = $51&65535;
       $53 = $52 & 3;
       $54 = ($53|0)!=(0);
       $56 = $54;
      }
     } while(0);
     $55 = $56&1;
     HEAP32[$isSearch>>2] = $55;
     $57 = HEAP32[$db>>2]|0;
     _sqlite3StrAccumInit($str,$57,$zBuf,100,1000000000);
     $58 = HEAP32[$isSearch>>2]|0;
     $59 = ($58|0)!=(0);
     $60 = $59 ? 32749 : 32756;
     _sqlite3StrAccumAppendAll($str,$60);
     $61 = HEAP32[$pItem>>2]|0;
     $62 = ((($61)) + 20|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = ($63|0)!=(0|0);
     $65 = HEAP32[$pItem>>2]|0;
     if ($64) {
      $66 = ((($65)) + 40|0);
      $67 = HEAP8[$66>>0]|0;
      $68 = $67&255;
      HEAP32[$vararg_buffer>>2] = $68;
      _sqlite3XPrintf($str,32761,$vararg_buffer);
     } else {
      $69 = ((($65)) + 8|0);
      $70 = HEAP32[$69>>2]|0;
      HEAP32[$vararg_buffer1>>2] = $70;
      _sqlite3XPrintf($str,32774,$vararg_buffer1);
     }
     $71 = HEAP32[$pItem>>2]|0;
     $72 = ((($71)) + 12|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ($73|0)!=(0|0);
     if ($74) {
      $75 = HEAP32[$pItem>>2]|0;
      $76 = ((($75)) + 12|0);
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$vararg_buffer4>>2] = $77;
      _sqlite3XPrintf($str,32784,$vararg_buffer4);
     }
     $78 = HEAP32[$flags>>2]|0;
     $79 = $78 & 1280;
     $80 = ($79|0)==(0);
     do {
      if ($80) {
       HEAP32[$zFmt>>2] = 0;
       $81 = HEAP32[$pLoop>>2]|0;
       $82 = ((($81)) + 24|0);
       $83 = ((($82)) + 4|0);
       $84 = HEAP32[$83>>2]|0;
       HEAP32[$pIdx>>2] = $84;
       $85 = HEAP32[$pItem>>2]|0;
       $86 = ((($85)) + 16|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = ((($87)) + 42|0);
       $89 = HEAP8[$88>>0]|0;
       $90 = $89&255;
       $91 = $90 & 32;
       $92 = ($91|0)==(0);
       if ($92) {
        label = 19;
       } else {
        $93 = HEAP32[$pIdx>>2]|0;
        $94 = ((($93)) + 55|0);
        $95 = HEAP8[$94>>0]|0;
        $96 = $95 & 3;
        $97 = $96&255;
        $98 = ($97|0)==(2);
        if ($98) {
         $99 = HEAP32[$isSearch>>2]|0;
         $100 = ($99|0)!=(0);
         if ($100) {
          HEAP32[$zFmt>>2] = 32791;
         }
        } else {
         label = 19;
        }
       }
       do {
        if ((label|0) == 19) {
         $101 = HEAP32[$flags>>2]|0;
         $102 = $101 & 131072;
         $103 = ($102|0)!=(0);
         if ($103) {
          HEAP32[$zFmt>>2] = 32803;
          break;
         }
         $104 = HEAP32[$flags>>2]|0;
         $105 = $104 & 16384;
         $106 = ($105|0)!=(0);
         if ($106) {
          HEAP32[$zFmt>>2] = 32836;
          break;
         }
         $107 = HEAP32[$flags>>2]|0;
         $108 = $107 & 64;
         $109 = ($108|0)!=(0);
         if ($109) {
          HEAP32[$zFmt>>2] = 32861;
          break;
         } else {
          HEAP32[$zFmt>>2] = 32879;
          break;
         }
        }
       } while(0);
       $110 = HEAP32[$zFmt>>2]|0;
       $111 = ($110|0)!=(0|0);
       if ($111) {
        _sqlite3StrAccumAppend($str,32888,7);
        $112 = HEAP32[$zFmt>>2]|0;
        $113 = HEAP32[$pIdx>>2]|0;
        $114 = HEAP32[$113>>2]|0;
        HEAP32[$vararg_buffer7>>2] = $114;
        _sqlite3XPrintf($str,$112,$vararg_buffer7);
        $115 = HEAP32[$pLoop>>2]|0;
        _explainIndexRange($str,$115);
       }
      } else {
       $116 = HEAP32[$flags>>2]|0;
       $117 = $116 & 256;
       $118 = ($117|0)!=(0);
       if ($118) {
        $119 = HEAP32[$flags>>2]|0;
        $120 = $119 & 15;
        $121 = ($120|0)!=(0);
        if ($121) {
         $122 = HEAP32[$flags>>2]|0;
         $123 = $122 & 5;
         $124 = ($123|0)!=(0);
         do {
          if ($124) {
           HEAP32[$zRangeOp>>2] = 32896;
          } else {
           $125 = HEAP32[$flags>>2]|0;
           $126 = $125 & 48;
           $127 = ($126|0)==(48);
           if ($127) {
            HEAP32[$zRangeOp>>2] = 32898;
            break;
           }
           $128 = HEAP32[$flags>>2]|0;
           $129 = $128 & 32;
           $130 = ($129|0)!=(0);
           if ($130) {
            HEAP32[$zRangeOp>>2] = 32912;
            break;
           } else {
            HEAP32[$zRangeOp>>2] = 32914;
            break;
           }
          }
         } while(0);
         $131 = HEAP32[$zRangeOp>>2]|0;
         HEAP32[$vararg_buffer10>>2] = $131;
         _sqlite3XPrintf($str,32916,$vararg_buffer10);
         break;
        }
       }
       $132 = HEAP32[$flags>>2]|0;
       $133 = $132 & 1024;
       $134 = ($133|0)!=(0);
       if ($134) {
        $135 = HEAP32[$pLoop>>2]|0;
        $136 = ((($135)) + 24|0);
        $137 = HEAP32[$136>>2]|0;
        $138 = HEAP32[$pLoop>>2]|0;
        $139 = ((($138)) + 24|0);
        $140 = ((($139)) + 8|0);
        $141 = HEAP32[$140>>2]|0;
        HEAP32[$vararg_buffer13>>2] = $137;
        $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
        HEAP32[$vararg_ptr16>>2] = $141;
        _sqlite3XPrintf($str,32954,$vararg_buffer13);
       }
      }
     } while(0);
     $142 = (_sqlite3StrAccumFinish($str)|0);
     HEAP32[$zMsg>>2] = $142;
     $143 = HEAP32[$v>>2]|0;
     $144 = HEAP32[$iId>>2]|0;
     $145 = HEAP32[$4>>2]|0;
     $146 = HEAP32[$5>>2]|0;
     $147 = HEAP32[$zMsg>>2]|0;
     $148 = (_sqlite3VdbeAddOp4($143,161,$144,$145,$146,$147,-1)|0);
     HEAP32[$ret>>2] = $148;
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $150 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($150|0);
  }
 } while(0);
 $149 = HEAP32[$ret>>2]|0;
 HEAP32[$0>>2] = $149;
 $150 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($150|0);
}
function _sqlite3WhereCodeOneLoopStart($pWInfo,$iLevel,$0,$1) {
 $pWInfo = $pWInfo|0;
 $iLevel = $iLevel|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0;
 var $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0;
 var $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0;
 var $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0;
 var $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0, $1088 = 0, $1089 = 0;
 var $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0, $1105 = 0, $1106 = 0;
 var $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0, $1123 = 0, $1124 = 0;
 var $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0, $1141 = 0, $1142 = 0;
 var $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0, $116 = 0, $1160 = 0;
 var $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0, $1178 = 0, $1179 = 0;
 var $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0, $1196 = 0, $1197 = 0;
 var $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0, $1213 = 0, $1214 = 0;
 var $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0, $1231 = 0, $1232 = 0;
 var $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0, $125 = 0, $1250 = 0;
 var $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0, $1268 = 0, $1269 = 0;
 var $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0, $1286 = 0, $1287 = 0;
 var $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0, $1303 = 0, $1304 = 0;
 var $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0, $1321 = 0, $1322 = 0;
 var $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0, $134 = 0, $1340 = 0;
 var $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0, $1358 = 0, $1359 = 0;
 var $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0, $1376 = 0, $1377 = 0;
 var $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0, $1394 = 0, $1395 = 0;
 var $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0, $1411 = 0, $1412 = 0;
 var $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0, $143 = 0, $1430 = 0;
 var $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0, $1448 = 0, $1449 = 0;
 var $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0, $1466 = 0, $1467 = 0;
 var $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0, $1484 = 0, $1485 = 0;
 var $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0, $1501 = 0, $1502 = 0;
 var $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0, $152 = 0, $1520 = 0;
 var $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0, $1538 = 0, $1539 = 0;
 var $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0, $1556 = 0, $1557 = 0;
 var $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0, $1574 = 0, $1575 = 0;
 var $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0;
 var $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0;
 var $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0;
 var $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0;
 var $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0;
 var $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0;
 var $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0;
 var $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0;
 var $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0;
 var $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0;
 var $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0;
 var $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0;
 var $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0;
 var $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0;
 var $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0;
 var $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0;
 var $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0;
 var $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0;
 var $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0;
 var $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0;
 var $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0;
 var $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0;
 var $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0;
 var $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0;
 var $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0;
 var $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0;
 var $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0;
 var $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0;
 var $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0;
 var $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0;
 var $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0;
 var $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0;
 var $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0;
 var $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0;
 var $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0;
 var $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0;
 var $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0;
 var $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0;
 var $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0;
 var $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0;
 var $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0;
 var $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0;
 var $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0;
 var $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0;
 var $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0;
 var $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0;
 var $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0;
 var $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $addrBrk = 0, $addrCont = 0, $addrExplain = 0, $addrNotFound = 0, $addrNxt = 0, $bRev = 0, $bSeekPastNull = 0, $bStopAtNull = 0, $cEndAff = 0, $db = 0, $endEq = 0, $iCol = 0;
 var $iCovCur = 0, $iCur = 0, $iIdxCur = 0, $iLoopBody = 0, $iPk = 0, $iReg = 0, $iReleaseReg = 0, $iRetInit = 0, $iRowidReg = 0, $iSet = 0, $iTarget = 0, $iTerm = 0, $ii = 0, $j = 0, $jmp1 = 0, $k = 0, $memEndValue = 0, $nConstraint = 0, $nConstraint2 = 0, $nEq = 0;
 var $nExtraReg = 0, $nNotReady = 0, $nPk = 0, $omitTable = 0, $op = 0, $origSrc = 0, $pAlt = 0, $pAndExpr = 0, $pCov = 0, $pE = 0, $pE7 = 0, $pEAlt = 0, $pEnd = 0, $pExpr = 0, $pIdx = 0, $pLevel = 0, $pLoop = 0, $pOrExpr = 0, $pOrTab = 0, $pOrTerm = 0;
 var $pOrWc = 0, $pParse = 0, $pPk = 0, $pPk5 = 0, $pPk6 = 0, $pRangeEnd = 0, $pRangeStart = 0, $pRight = 0, $pRight4 = 0, $pStart = 0, $pSubLoop = 0, $pSubWInfo = 0, $pTab = 0, $pTabItem = 0, $pTerm = 0, $pWC = 0, $pX = 0, $pX1 = 0, $r = 0, $r1 = 0;
 var $rTemp = 0, $regBase = 0, $regReturn = 0, $regRowid = 0, $regRowset = 0, $regYield = 0, $skipLikeAddr = 0, $start = 0, $startEq = 0, $start_constraints = 0, $t = 0, $t3 = 0, $testOp = 0, $untestedTerms = 0, $v = 0, $wctrlFlags = 0, $zStartAff = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 368|0;
 $2 = sp + 8|0;
 $3 = sp + 352|0;
 $4 = sp + 348|0;
 $5 = sp;
 $j = sp + 344|0;
 $k = sp + 340|0;
 $iCur = sp + 336|0;
 $addrNxt = sp + 332|0;
 $omitTable = sp + 328|0;
 $bRev = sp + 324|0;
 $pLevel = sp + 320|0;
 $pLoop = sp + 316|0;
 $pWC = sp + 312|0;
 $pTerm = sp + 308|0;
 $pParse = sp + 304|0;
 $db = sp + 300|0;
 $v = sp + 296|0;
 $pTabItem = sp + 292|0;
 $addrBrk = sp + 288|0;
 $addrCont = sp + 284|0;
 $iRowidReg = sp + 280|0;
 $iReleaseReg = sp + 276|0;
 $regYield = sp + 272|0;
 $iReg = sp + 268|0;
 $addrNotFound = sp + 264|0;
 $nConstraint = sp + 260|0;
 $iTarget = sp + 256|0;
 $testOp = sp + 252|0;
 $start = sp + 248|0;
 $memEndValue = sp + 244|0;
 $pStart = sp + 240|0;
 $pEnd = sp + 236|0;
 $pX = sp + 232|0;
 $r1 = sp + 228|0;
 $rTemp = sp + 224|0;
 $pX1 = sp + 220|0;
 $nEq = sp + 358|0;
 $regBase = sp + 216|0;
 $pRangeStart = sp + 212|0;
 $pRangeEnd = sp + 208|0;
 $startEq = sp + 204|0;
 $endEq = sp + 200|0;
 $start_constraints = sp + 196|0;
 $nConstraint2 = sp + 192|0;
 $pIdx = sp + 188|0;
 $iIdxCur = sp + 184|0;
 $nExtraReg = sp + 180|0;
 $op = sp + 176|0;
 $zStartAff = sp + 172|0;
 $cEndAff = sp + 363|0;
 $bSeekPastNull = sp + 362|0;
 $bStopAtNull = sp + 361|0;
 $t = sp + 168|0;
 $t3 = sp + 360|0;
 $pRight = sp + 164|0;
 $pRight4 = sp + 160|0;
 $pPk = sp + 156|0;
 $pOrWc = sp + 152|0;
 $pOrTab = sp + 148|0;
 $pCov = sp + 144|0;
 $iCovCur = sp + 140|0;
 $regReturn = sp + 136|0;
 $regRowset = sp + 132|0;
 $regRowid = sp + 128|0;
 $iLoopBody = sp + 124|0;
 $iRetInit = sp + 120|0;
 $untestedTerms = sp + 116|0;
 $ii = sp + 112|0;
 $wctrlFlags = sp + 356|0;
 $pAndExpr = sp + 108|0;
 $pTab = sp + 104|0;
 $nNotReady = sp + 100|0;
 $origSrc = sp + 96|0;
 $pPk5 = sp + 92|0;
 $iTerm = sp + 88|0;
 $pExpr = sp + 84|0;
 $pOrTerm = sp + 80|0;
 $pSubWInfo = sp + 76|0;
 $pOrExpr = sp + 72|0;
 $jmp1 = sp + 68|0;
 $pSubLoop = sp + 64|0;
 $addrExplain = sp + 60|0;
 $r = sp + 56|0;
 $iSet = sp + 52|0;
 $pPk6 = sp + 48|0;
 $nPk = sp + 44|0;
 $iPk = sp + 40|0;
 $iCol = sp + 36|0;
 $pE = sp + 32|0;
 $skipLikeAddr = sp + 28|0;
 $pE7 = sp + 24|0;
 $pEAlt = sp + 20|0;
 $pAlt = sp + 16|0;
 HEAP32[$3>>2] = $pWInfo;
 HEAP32[$4>>2] = $iLevel;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$iRowidReg>>2] = 0;
 HEAP32[$iReleaseReg>>2] = 0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pParse>>2] = $11;
 $12 = HEAP32[$pParse>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$v>>2] = $14;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 328|0);
 HEAP32[$pWC>>2] = $16;
 $17 = HEAP32[$pParse>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$db>>2] = $18;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = ((($20)) + 736|0);
 $22 = (($21) + (($19*80)|0)|0);
 HEAP32[$pLevel>>2] = $22;
 $23 = HEAP32[$pLevel>>2]|0;
 $24 = ((($23)) + 64|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pLoop>>2] = $25;
 $26 = HEAP32[$pLevel>>2]|0;
 $27 = ((($26)) + 44|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$3>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = (($33) + (($29*72)|0)|0);
 HEAP32[$pTabItem>>2] = $34;
 $35 = HEAP32[$pTabItem>>2]|0;
 $36 = ((($35)) + 44|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$iCur>>2] = $37;
 $38 = $5;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 $45 = ((($44)) + 68|0);
 $46 = HEAP32[$iCur>>2]|0;
 $47 = (_sqlite3WhereGetMask($45,$46)|0);
 $48 = tempRet0;
 $49 = $47 ^ -1;
 $50 = $48 ^ -1;
 $51 = $40 & $49;
 $52 = $43 & $50;
 $53 = HEAP32[$pLevel>>2]|0;
 $54 = ((($53)) + 72|0);
 $55 = $54;
 $56 = $55;
 HEAP32[$56>>2] = $51;
 $57 = (($55) + 4)|0;
 $58 = $57;
 HEAP32[$58>>2] = $52;
 $59 = HEAP32[$3>>2]|0;
 $60 = ((($59)) + 24|0);
 $61 = $60;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$4>>2]|0;
 $68 = (_bitshift64Lshr(($63|0),($66|0),($67|0))|0);
 $69 = tempRet0;
 $70 = $68 & 1;
 HEAP32[$bRev>>2] = $70;
 $71 = HEAP32[$pLoop>>2]|0;
 $72 = ((($71)) + 36|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $73 & 64;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$3>>2]|0;
  $77 = ((($76)) + 34|0);
  $78 = HEAP16[$77>>1]|0;
  $79 = $78&65535;
  $80 = $79 & 32;
  $81 = ($80|0)==(0);
  $83 = $81;
 } else {
  $83 = 0;
 }
 $82 = $83&1;
 HEAP32[$omitTable>>2] = $82;
 $84 = HEAP32[$v>>2]|0;
 $85 = (_sqlite3VdbeMakeLabel($84)|0);
 $86 = HEAP32[$pLevel>>2]|0;
 $87 = ((($86)) + 16|0);
 HEAP32[$87>>2] = $85;
 $88 = HEAP32[$pLevel>>2]|0;
 $89 = ((($88)) + 12|0);
 HEAP32[$89>>2] = $85;
 HEAP32[$addrBrk>>2] = $85;
 $90 = HEAP32[$v>>2]|0;
 $91 = (_sqlite3VdbeMakeLabel($90)|0);
 $92 = HEAP32[$pLevel>>2]|0;
 $93 = ((($92)) + 24|0);
 HEAP32[$93>>2] = $91;
 HEAP32[$addrCont>>2] = $91;
 $94 = HEAP32[$pLevel>>2]|0;
 $95 = ((($94)) + 44|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = $96&255;
 $98 = ($97|0)>(0);
 if ($98) {
  $99 = HEAP32[$pTabItem>>2]|0;
  $100 = ((($99)) + 36|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = $101&255;
  $103 = $102 & 8;
  $104 = ($103|0)!=(0);
  if ($104) {
   $105 = HEAP32[$pParse>>2]|0;
   $106 = ((($105)) + 76|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + 1)|0;
   HEAP32[$106>>2] = $108;
   $109 = HEAP32[$pLevel>>2]|0;
   HEAP32[$109>>2] = $108;
   $110 = HEAP32[$v>>2]|0;
   $111 = HEAP32[$pLevel>>2]|0;
   $112 = HEAP32[$111>>2]|0;
   (_sqlite3VdbeAddOp2($110,22,0,$112)|0);
  }
 }
 $113 = HEAP32[$pTabItem>>2]|0;
 $114 = ((($113)) + 36|0);
 $115 = ((($114)) + 1|0);
 $116 = HEAP8[$115>>0]|0;
 $117 = ($116&255) >>> 4;
 $118 = $117 & 1;
 $119 = $118&255;
 $120 = ($119|0)!=(0);
 do {
  if ($120) {
   $121 = HEAP32[$pTabItem>>2]|0;
   $122 = ((($121)) + 28|0);
   $123 = HEAP32[$122>>2]|0;
   HEAP32[$regYield>>2] = $123;
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$regYield>>2]|0;
   $126 = HEAP32[$pTabItem>>2]|0;
   $127 = ((($126)) + 24|0);
   $128 = HEAP32[$127>>2]|0;
   (_sqlite3VdbeAddOp3($124,16,$125,0,$128)|0);
   $129 = HEAP32[$v>>2]|0;
   $130 = HEAP32[$regYield>>2]|0;
   $131 = HEAP32[$addrBrk>>2]|0;
   $132 = (_sqlite3VdbeAddOp2($129,18,$130,$131)|0);
   $133 = HEAP32[$pLevel>>2]|0;
   $134 = ((($133)) + 52|0);
   HEAP32[$134>>2] = $132;
   $135 = HEAP32[$pLevel>>2]|0;
   $136 = ((($135)) + 45|0);
   HEAP8[$136>>0] = 13;
  } else {
   $137 = HEAP32[$pLoop>>2]|0;
   $138 = ((($137)) + 36|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $139 & 1024;
   $141 = ($140|0)!=(0);
   $142 = HEAP32[$pLoop>>2]|0;
   if ($141) {
    $143 = ((($142)) + 40|0);
    $144 = HEAP16[$143>>1]|0;
    $145 = $144&65535;
    HEAP32[$nConstraint>>2] = $145;
    $146 = HEAP32[$pParse>>2]|0;
    _sqlite3ExprCachePush($146);
    $147 = HEAP32[$pParse>>2]|0;
    $148 = HEAP32[$nConstraint>>2]|0;
    $149 = (($148) + 2)|0;
    $150 = (_sqlite3GetTempRange($147,$149)|0);
    HEAP32[$iReg>>2] = $150;
    $151 = HEAP32[$pLevel>>2]|0;
    $152 = ((($151)) + 12|0);
    $153 = HEAP32[$152>>2]|0;
    HEAP32[$addrNotFound>>2] = $153;
    HEAP32[$j>>2] = 0;
    while(1) {
     $154 = HEAP32[$j>>2]|0;
     $155 = HEAP32[$nConstraint>>2]|0;
     $156 = ($154|0)<($155|0);
     if (!($156)) {
      break;
     }
     $157 = HEAP32[$iReg>>2]|0;
     $158 = HEAP32[$j>>2]|0;
     $159 = (($157) + ($158))|0;
     $160 = (($159) + 2)|0;
     HEAP32[$iTarget>>2] = $160;
     $161 = HEAP32[$j>>2]|0;
     $162 = HEAP32[$pLoop>>2]|0;
     $163 = ((($162)) + 48|0);
     $164 = HEAP32[$163>>2]|0;
     $165 = (($164) + ($161<<2)|0);
     $166 = HEAP32[$165>>2]|0;
     HEAP32[$pTerm>>2] = $166;
     $167 = HEAP32[$pTerm>>2]|0;
     $168 = ($167|0)==(0|0);
     do {
      if (!($168)) {
       $169 = HEAP32[$pTerm>>2]|0;
       $170 = ((($169)) + 18|0);
       $171 = HEAP16[$170>>1]|0;
       $172 = $171&65535;
       $173 = $172 & 1;
       $174 = ($173|0)!=(0);
       $175 = HEAP32[$pParse>>2]|0;
       $176 = HEAP32[$pTerm>>2]|0;
       if ($174) {
        $177 = HEAP32[$pLevel>>2]|0;
        $178 = HEAP32[$j>>2]|0;
        $179 = HEAP32[$bRev>>2]|0;
        $180 = HEAP32[$iTarget>>2]|0;
        (_codeEqualityTerm($175,$176,$177,$178,$179,$180)|0);
        $181 = HEAP32[$pLevel>>2]|0;
        $182 = ((($181)) + 16|0);
        $183 = HEAP32[$182>>2]|0;
        HEAP32[$addrNotFound>>2] = $183;
        break;
       } else {
        $184 = HEAP32[$176>>2]|0;
        $185 = ((($184)) + 16|0);
        $186 = HEAP32[$185>>2]|0;
        $187 = HEAP32[$iTarget>>2]|0;
        _sqlite3ExprCode($175,$186,$187);
        break;
       }
      }
     } while(0);
     $188 = HEAP32[$j>>2]|0;
     $189 = (($188) + 1)|0;
     HEAP32[$j>>2] = $189;
    }
    $190 = HEAP32[$v>>2]|0;
    $191 = HEAP32[$pLoop>>2]|0;
    $192 = ((($191)) + 24|0);
    $193 = HEAP32[$192>>2]|0;
    $194 = HEAP32[$iReg>>2]|0;
    (_sqlite3VdbeAddOp2($190,22,$193,$194)|0);
    $195 = HEAP32[$v>>2]|0;
    $196 = HEAP32[$nConstraint>>2]|0;
    $197 = HEAP32[$iReg>>2]|0;
    $198 = (($197) + 1)|0;
    (_sqlite3VdbeAddOp2($195,22,$196,$198)|0);
    $199 = HEAP32[$v>>2]|0;
    $200 = HEAP32[$iCur>>2]|0;
    $201 = HEAP32[$addrNotFound>>2]|0;
    $202 = HEAP32[$iReg>>2]|0;
    $203 = HEAP32[$pLoop>>2]|0;
    $204 = ((($203)) + 24|0);
    $205 = ((($204)) + 8|0);
    $206 = HEAP32[$205>>2]|0;
    $207 = HEAP32[$pLoop>>2]|0;
    $208 = ((($207)) + 24|0);
    $209 = ((($208)) + 4|0);
    $210 = HEAP8[$209>>0]|0;
    $211 = $210&255;
    $212 = ($211|0)!=(0);
    $213 = $212 ? -11 : -2;
    (_sqlite3VdbeAddOp4($199,11,$200,$201,$202,$206,$213)|0);
    $214 = HEAP32[$pLoop>>2]|0;
    $215 = ((($214)) + 24|0);
    $216 = ((($215)) + 4|0);
    HEAP8[$216>>0] = 0;
    HEAP32[$j>>2] = 0;
    while(1) {
     $217 = HEAP32[$j>>2]|0;
     $218 = HEAP32[$nConstraint>>2]|0;
     $219 = ($217|0)<($218|0);
     $220 = HEAP32[$j>>2]|0;
     $221 = ($220|0)<(16);
     $222 = $219 ? $221 : 0;
     if (!($222)) {
      break;
     }
     $223 = HEAP32[$pLoop>>2]|0;
     $224 = ((($223)) + 24|0);
     $225 = ((($224)) + 6|0);
     $226 = HEAP16[$225>>1]|0;
     $227 = $226&65535;
     $228 = HEAP32[$j>>2]|0;
     $229 = $227 >> $228;
     $230 = $229 & 1;
     $231 = ($230|0)!=(0);
     if ($231) {
      $232 = HEAP32[$pLevel>>2]|0;
      $233 = HEAP32[$j>>2]|0;
      $234 = HEAP32[$pLoop>>2]|0;
      $235 = ((($234)) + 48|0);
      $236 = HEAP32[$235>>2]|0;
      $237 = (($236) + ($233<<2)|0);
      $238 = HEAP32[$237>>2]|0;
      _disableTerm($232,$238);
     }
     $239 = HEAP32[$j>>2]|0;
     $240 = (($239) + 1)|0;
     HEAP32[$j>>2] = $240;
    }
    $241 = HEAP32[$iCur>>2]|0;
    $242 = HEAP32[$pLevel>>2]|0;
    $243 = ((($242)) + 48|0);
    HEAP32[$243>>2] = $241;
    $244 = HEAP32[$3>>2]|0;
    $245 = ((($244)) + 38|0);
    $246 = HEAP8[$245>>0]|0;
    $247 = $246&255;
    $248 = ($247|0)!=(0);
    $249 = $248 ? 160 : 154;
    $250 = $249&255;
    $251 = HEAP32[$pLevel>>2]|0;
    $252 = ((($251)) + 45|0);
    HEAP8[$252>>0] = $250;
    $253 = HEAP32[$v>>2]|0;
    $254 = (_sqlite3VdbeCurrentAddr($253)|0);
    $255 = HEAP32[$pLevel>>2]|0;
    $256 = ((($255)) + 52|0);
    HEAP32[$256>>2] = $254;
    $257 = HEAP32[$pParse>>2]|0;
    $258 = HEAP32[$iReg>>2]|0;
    $259 = HEAP32[$nConstraint>>2]|0;
    $260 = (($259) + 2)|0;
    _sqlite3ReleaseTempRange($257,$258,$260);
    $261 = HEAP32[$pParse>>2]|0;
    _sqlite3ExprCachePop($261);
    break;
   }
   $262 = ((($142)) + 36|0);
   $263 = HEAP32[$262>>2]|0;
   $264 = $263 & 256;
   $265 = ($264|0)!=(0);
   if ($265) {
    $266 = HEAP32[$pLoop>>2]|0;
    $267 = ((($266)) + 36|0);
    $268 = HEAP32[$267>>2]|0;
    $269 = $268 & 5;
    $270 = ($269|0)!=(0);
    if ($270) {
     $271 = HEAP32[$pLoop>>2]|0;
     $272 = ((($271)) + 48|0);
     $273 = HEAP32[$272>>2]|0;
     $274 = HEAP32[$273>>2]|0;
     HEAP32[$pTerm>>2] = $274;
     $275 = HEAP32[$pParse>>2]|0;
     $276 = ((($275)) + 76|0);
     $277 = HEAP32[$276>>2]|0;
     $278 = (($277) + 1)|0;
     HEAP32[$276>>2] = $278;
     HEAP32[$iReleaseReg>>2] = $278;
     $279 = HEAP32[$pParse>>2]|0;
     $280 = HEAP32[$pTerm>>2]|0;
     $281 = HEAP32[$pLevel>>2]|0;
     $282 = HEAP32[$bRev>>2]|0;
     $283 = HEAP32[$iReleaseReg>>2]|0;
     $284 = (_codeEqualityTerm($279,$280,$281,0,$282,$283)|0);
     HEAP32[$iRowidReg>>2] = $284;
     $285 = HEAP32[$iRowidReg>>2]|0;
     $286 = HEAP32[$iReleaseReg>>2]|0;
     $287 = ($285|0)!=($286|0);
     if ($287) {
      $288 = HEAP32[$pParse>>2]|0;
      $289 = HEAP32[$iReleaseReg>>2]|0;
      _sqlite3ReleaseTempReg($288,$289);
     }
     $290 = HEAP32[$pLevel>>2]|0;
     $291 = ((($290)) + 16|0);
     $292 = HEAP32[$291>>2]|0;
     HEAP32[$addrNxt>>2] = $292;
     $293 = HEAP32[$v>>2]|0;
     $294 = HEAP32[$iRowidReg>>2]|0;
     $295 = HEAP32[$addrNxt>>2]|0;
     (_sqlite3VdbeAddOp2($293,38,$294,$295)|0);
     $296 = HEAP32[$v>>2]|0;
     $297 = HEAP32[$iCur>>2]|0;
     $298 = HEAP32[$addrNxt>>2]|0;
     $299 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp3($296,70,$297,$298,$299)|0);
     $300 = HEAP32[$pParse>>2]|0;
     $301 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheAffinityChange($300,$301,1);
     $302 = HEAP32[$pParse>>2]|0;
     $303 = HEAP32[$iCur>>2]|0;
     $304 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheStore($302,$303,-1,$304);
     $305 = HEAP32[$pLevel>>2]|0;
     $306 = ((($305)) + 45|0);
     HEAP8[$306>>0] = -96;
     break;
    }
   }
   $307 = HEAP32[$pLoop>>2]|0;
   $308 = ((($307)) + 36|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = $309 & 256;
   $311 = ($310|0)!=(0);
   if ($311) {
    $312 = HEAP32[$pLoop>>2]|0;
    $313 = ((($312)) + 36|0);
    $314 = HEAP32[$313>>2]|0;
    $315 = $314 & 2;
    $316 = ($315|0)!=(0);
    if ($316) {
     HEAP32[$testOp>>2] = 160;
     HEAP32[$memEndValue>>2] = 0;
     HEAP32[$j>>2] = 0;
     HEAP32[$pEnd>>2] = 0;
     HEAP32[$pStart>>2] = 0;
     $317 = HEAP32[$pLoop>>2]|0;
     $318 = ((($317)) + 36|0);
     $319 = HEAP32[$318>>2]|0;
     $320 = $319 & 32;
     $321 = ($320|0)!=(0);
     if ($321) {
      $322 = HEAP32[$j>>2]|0;
      $323 = (($322) + 1)|0;
      HEAP32[$j>>2] = $323;
      $324 = HEAP32[$pLoop>>2]|0;
      $325 = ((($324)) + 48|0);
      $326 = HEAP32[$325>>2]|0;
      $327 = (($326) + ($322<<2)|0);
      $328 = HEAP32[$327>>2]|0;
      HEAP32[$pStart>>2] = $328;
     }
     $329 = HEAP32[$pLoop>>2]|0;
     $330 = ((($329)) + 36|0);
     $331 = HEAP32[$330>>2]|0;
     $332 = $331 & 16;
     $333 = ($332|0)!=(0);
     if ($333) {
      $334 = HEAP32[$j>>2]|0;
      $335 = (($334) + 1)|0;
      HEAP32[$j>>2] = $335;
      $336 = HEAP32[$pLoop>>2]|0;
      $337 = ((($336)) + 48|0);
      $338 = HEAP32[$337>>2]|0;
      $339 = (($338) + ($334<<2)|0);
      $340 = HEAP32[$339>>2]|0;
      HEAP32[$pEnd>>2] = $340;
     }
     $341 = HEAP32[$bRev>>2]|0;
     $342 = ($341|0)!=(0);
     if ($342) {
      $343 = HEAP32[$pStart>>2]|0;
      HEAP32[$pTerm>>2] = $343;
      $344 = HEAP32[$pEnd>>2]|0;
      HEAP32[$pStart>>2] = $344;
      $345 = HEAP32[$pTerm>>2]|0;
      HEAP32[$pEnd>>2] = $345;
     }
     $346 = HEAP32[$pStart>>2]|0;
     $347 = ($346|0)!=(0|0);
     if ($347) {
      $348 = HEAP32[$pStart>>2]|0;
      $349 = HEAP32[$348>>2]|0;
      HEAP32[$pX>>2] = $349;
      $350 = HEAP32[$pParse>>2]|0;
      $351 = HEAP32[$pX>>2]|0;
      $352 = ((($351)) + 16|0);
      $353 = HEAP32[$352>>2]|0;
      $354 = (_sqlite3ExprCodeTemp($350,$353,$rTemp)|0);
      HEAP32[$r1>>2] = $354;
      $355 = HEAP32[$v>>2]|0;
      $356 = HEAP32[$pX>>2]|0;
      $357 = HEAP8[$356>>0]|0;
      $358 = $357&255;
      $359 = (($358) - 80)|0;
      $360 = (33010 + ($359)|0);
      $361 = HEAP8[$360>>0]|0;
      $362 = $361&255;
      $363 = HEAP32[$iCur>>2]|0;
      $364 = HEAP32[$addrBrk>>2]|0;
      $365 = HEAP32[$r1>>2]|0;
      (_sqlite3VdbeAddOp3($355,$362,$363,$364,$365)|0);
      $366 = HEAP32[$pParse>>2]|0;
      $367 = HEAP32[$r1>>2]|0;
      _sqlite3ExprCacheAffinityChange($366,$367,1);
      $368 = HEAP32[$pParse>>2]|0;
      $369 = HEAP32[$rTemp>>2]|0;
      _sqlite3ReleaseTempReg($368,$369);
      $370 = HEAP32[$pLevel>>2]|0;
      $371 = HEAP32[$pStart>>2]|0;
      _disableTerm($370,$371);
     } else {
      $372 = HEAP32[$v>>2]|0;
      $373 = HEAP32[$bRev>>2]|0;
      $374 = ($373|0)!=(0);
      $375 = $374 ? 105 : 108;
      $376 = HEAP32[$iCur>>2]|0;
      $377 = HEAP32[$addrBrk>>2]|0;
      (_sqlite3VdbeAddOp2($372,$375,$376,$377)|0);
     }
     $378 = HEAP32[$pEnd>>2]|0;
     $379 = ($378|0)!=(0|0);
     if ($379) {
      $380 = HEAP32[$pEnd>>2]|0;
      $381 = HEAP32[$380>>2]|0;
      HEAP32[$pX1>>2] = $381;
      $382 = HEAP32[$pParse>>2]|0;
      $383 = ((($382)) + 76|0);
      $384 = HEAP32[$383>>2]|0;
      $385 = (($384) + 1)|0;
      HEAP32[$383>>2] = $385;
      HEAP32[$memEndValue>>2] = $385;
      $386 = HEAP32[$pParse>>2]|0;
      $387 = HEAP32[$pX1>>2]|0;
      $388 = ((($387)) + 16|0);
      $389 = HEAP32[$388>>2]|0;
      $390 = HEAP32[$memEndValue>>2]|0;
      _sqlite3ExprCode($386,$389,$390);
      $391 = HEAP32[$pX1>>2]|0;
      $392 = HEAP8[$391>>0]|0;
      $393 = $392&255;
      $394 = ($393|0)==(82);
      if ($394) {
       label = 41;
      } else {
       $395 = HEAP32[$pX1>>2]|0;
       $396 = HEAP8[$395>>0]|0;
       $397 = $396&255;
       $398 = ($397|0)==(80);
       if ($398) {
        label = 41;
       } else {
        $402 = HEAP32[$bRev>>2]|0;
        $403 = ($402|0)!=(0);
        $404 = $403 ? 82 : 80;
        HEAP32[$testOp>>2] = $404;
       }
      }
      if ((label|0) == 41) {
       $399 = HEAP32[$bRev>>2]|0;
       $400 = ($399|0)!=(0);
       $401 = $400 ? 81 : 83;
       HEAP32[$testOp>>2] = $401;
      }
      $405 = HEAP32[$pLevel>>2]|0;
      $406 = HEAP32[$pEnd>>2]|0;
      _disableTerm($405,$406);
     }
     $407 = HEAP32[$v>>2]|0;
     $408 = (_sqlite3VdbeCurrentAddr($407)|0);
     HEAP32[$start>>2] = $408;
     $409 = HEAP32[$bRev>>2]|0;
     $410 = ($409|0)!=(0);
     $411 = $410 ? 6 : 7;
     $412 = $411&255;
     $413 = HEAP32[$pLevel>>2]|0;
     $414 = ((($413)) + 45|0);
     HEAP8[$414>>0] = $412;
     $415 = HEAP32[$iCur>>2]|0;
     $416 = HEAP32[$pLevel>>2]|0;
     $417 = ((($416)) + 48|0);
     HEAP32[$417>>2] = $415;
     $418 = HEAP32[$start>>2]|0;
     $419 = HEAP32[$pLevel>>2]|0;
     $420 = ((($419)) + 52|0);
     HEAP32[$420>>2] = $418;
     $421 = HEAP32[$testOp>>2]|0;
     $422 = ($421|0)!=(160);
     if (!($422)) {
      break;
     }
     $423 = HEAP32[$pParse>>2]|0;
     $424 = ((($423)) + 76|0);
     $425 = HEAP32[$424>>2]|0;
     $426 = (($425) + 1)|0;
     HEAP32[$424>>2] = $426;
     HEAP32[$iRowidReg>>2] = $426;
     $427 = HEAP32[$v>>2]|0;
     $428 = HEAP32[$iCur>>2]|0;
     $429 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp2($427,103,$428,$429)|0);
     $430 = HEAP32[$pParse>>2]|0;
     $431 = HEAP32[$iCur>>2]|0;
     $432 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheStore($430,$431,-1,$432);
     $433 = HEAP32[$v>>2]|0;
     $434 = HEAP32[$testOp>>2]|0;
     $435 = HEAP32[$memEndValue>>2]|0;
     $436 = HEAP32[$addrBrk>>2]|0;
     $437 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp3($433,$434,$435,$436,$437)|0);
     $438 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($438,83);
     break;
    }
   }
   $439 = HEAP32[$pLoop>>2]|0;
   $440 = ((($439)) + 36|0);
   $441 = HEAP32[$440>>2]|0;
   $442 = $441 & 512;
   $443 = ($442|0)!=(0);
   $444 = HEAP32[$pLoop>>2]|0;
   if ($443) {
    $445 = ((($444)) + 24|0);
    $446 = HEAP16[$445>>1]|0;
    HEAP16[$nEq>>1] = $446;
    HEAP32[$pRangeStart>>2] = 0;
    HEAP32[$pRangeEnd>>2] = 0;
    HEAP32[$nExtraReg>>2] = 0;
    HEAP8[$cEndAff>>0] = 0;
    HEAP8[$bSeekPastNull>>0] = 0;
    HEAP8[$bStopAtNull>>0] = 0;
    $447 = HEAP32[$pLoop>>2]|0;
    $448 = ((($447)) + 24|0);
    $449 = ((($448)) + 4|0);
    $450 = HEAP32[$449>>2]|0;
    HEAP32[$pIdx>>2] = $450;
    $451 = HEAP32[$pLevel>>2]|0;
    $452 = ((($451)) + 8|0);
    $453 = HEAP32[$452>>2]|0;
    HEAP32[$iIdxCur>>2] = $453;
    $454 = HEAP32[$3>>2]|0;
    $455 = ((($454)) + 34|0);
    $456 = HEAP16[$455>>1]|0;
    $457 = $456&65535;
    $458 = $457 & 1;
    $459 = ($458|0)!=(0);
    if ($459) {
     $460 = HEAP32[$3>>2]|0;
     $461 = ((($460)) + 36|0);
     $462 = HEAP8[$461>>0]|0;
     $463 = $462 << 24 >> 24;
     $464 = ($463|0)>(0);
     if ($464) {
      $465 = HEAP32[$pIdx>>2]|0;
      $466 = ((($465)) + 50|0);
      $467 = HEAP16[$466>>1]|0;
      $468 = $467&65535;
      $469 = HEAP16[$nEq>>1]|0;
      $470 = $469&65535;
      $471 = ($468|0)>($470|0);
      if ($471) {
       HEAP8[$bSeekPastNull>>0] = 1;
       HEAP32[$nExtraReg>>2] = 1;
      }
     }
    }
    $472 = HEAP16[$nEq>>1]|0;
    $473 = $472&65535;
    HEAP32[$j>>2] = $473;
    $474 = HEAP32[$pLoop>>2]|0;
    $475 = ((($474)) + 36|0);
    $476 = HEAP32[$475>>2]|0;
    $477 = $476 & 32;
    $478 = ($477|0)!=(0);
    if ($478) {
     $479 = HEAP32[$j>>2]|0;
     $480 = (($479) + 1)|0;
     HEAP32[$j>>2] = $480;
     $481 = HEAP32[$pLoop>>2]|0;
     $482 = ((($481)) + 48|0);
     $483 = HEAP32[$482>>2]|0;
     $484 = (($483) + ($479<<2)|0);
     $485 = HEAP32[$484>>2]|0;
     HEAP32[$pRangeStart>>2] = $485;
     HEAP32[$nExtraReg>>2] = 1;
    }
    $486 = HEAP32[$pLoop>>2]|0;
    $487 = ((($486)) + 36|0);
    $488 = HEAP32[$487>>2]|0;
    $489 = $488 & 16;
    $490 = ($489|0)!=(0);
    if ($490) {
     $491 = HEAP32[$j>>2]|0;
     $492 = (($491) + 1)|0;
     HEAP32[$j>>2] = $492;
     $493 = HEAP32[$pLoop>>2]|0;
     $494 = ((($493)) + 48|0);
     $495 = HEAP32[$494>>2]|0;
     $496 = (($495) + ($491<<2)|0);
     $497 = HEAP32[$496>>2]|0;
     HEAP32[$pRangeEnd>>2] = $497;
     HEAP32[$nExtraReg>>2] = 1;
     $498 = HEAP32[$pRangeEnd>>2]|0;
     $499 = ((($498)) + 20|0);
     $500 = HEAP16[$499>>1]|0;
     $501 = $500&65535;
     $502 = $501 & 256;
     $503 = ($502|0)!=(0);
     if ($503) {
      $504 = HEAP32[$pParse>>2]|0;
      $505 = ((($504)) + 76|0);
      $506 = HEAP32[$505>>2]|0;
      $507 = (($506) + 1)|0;
      HEAP32[$505>>2] = $507;
      $508 = HEAP32[$pLevel>>2]|0;
      $509 = ((($508)) + 36|0);
      HEAP32[$509>>2] = $507;
      $510 = HEAP32[$v>>2]|0;
      $511 = HEAP32[$bRev>>2]|0;
      $512 = HEAP16[$nEq>>1]|0;
      $513 = $512&65535;
      $514 = HEAP32[$pIdx>>2]|0;
      $515 = ((($514)) + 28|0);
      $516 = HEAP32[$515>>2]|0;
      $517 = (($516) + ($513)|0);
      $518 = HEAP8[$517>>0]|0;
      $519 = $518&255;
      $520 = ($519|0)==(1);
      $521 = $520&1;
      $522 = $511 ^ $521;
      $523 = HEAP32[$pLevel>>2]|0;
      $524 = ((($523)) + 36|0);
      $525 = HEAP32[$524>>2]|0;
      (_sqlite3VdbeAddOp2($510,22,$522,$525)|0);
      $526 = HEAP32[$v>>2]|0;
      $527 = (_sqlite3VdbeCurrentAddr($526)|0);
      $528 = HEAP32[$pLevel>>2]|0;
      $529 = ((($528)) + 40|0);
      HEAP32[$529>>2] = $527;
     }
     $530 = HEAP32[$pRangeStart>>2]|0;
     $531 = ($530|0)==(0|0);
     if ($531) {
      $532 = HEAP16[$nEq>>1]|0;
      $533 = $532&65535;
      $534 = HEAP32[$pIdx>>2]|0;
      $535 = ((($534)) + 4|0);
      $536 = HEAP32[$535>>2]|0;
      $537 = (($536) + ($533<<1)|0);
      $538 = HEAP16[$537>>1]|0;
      $539 = $538 << 16 >> 16;
      HEAP32[$j>>2] = $539;
      $540 = ($539|0)>=(0);
      if ($540) {
       $541 = HEAP32[$j>>2]|0;
       $542 = HEAP32[$pIdx>>2]|0;
       $543 = ((($542)) + 12|0);
       $544 = HEAP32[$543>>2]|0;
       $545 = ((($544)) + 4|0);
       $546 = HEAP32[$545>>2]|0;
       $547 = (($546) + (($541*24)|0)|0);
       $548 = ((($547)) + 20|0);
       $549 = HEAP8[$548>>0]|0;
       $550 = $549&255;
       $551 = ($550|0)==(0);
       if ($551) {
        HEAP8[$bSeekPastNull>>0] = 1;
       }
      }
     }
    }
    $552 = HEAP16[$nEq>>1]|0;
    $553 = $552&65535;
    $554 = HEAP32[$pIdx>>2]|0;
    $555 = ((($554)) + 50|0);
    $556 = HEAP16[$555>>1]|0;
    $557 = $556&65535;
    $558 = ($553|0)<($557|0);
    if ($558) {
     $559 = HEAP32[$bRev>>2]|0;
     $560 = HEAP16[$nEq>>1]|0;
     $561 = $560&65535;
     $562 = HEAP32[$pIdx>>2]|0;
     $563 = ((($562)) + 28|0);
     $564 = HEAP32[$563>>2]|0;
     $565 = (($564) + ($561)|0);
     $566 = HEAP8[$565>>0]|0;
     $567 = $566&255;
     $568 = ($567|0)==(0);
     $569 = $568&1;
     $570 = ($559|0)==($569|0);
     if ($570) {
      label = 64;
     } else {
      label = 62;
     }
    } else {
     label = 62;
    }
    if ((label|0) == 62) {
     $571 = HEAP32[$bRev>>2]|0;
     $572 = ($571|0)!=(0);
     if ($572) {
      $573 = HEAP32[$pIdx>>2]|0;
      $574 = ((($573)) + 50|0);
      $575 = HEAP16[$574>>1]|0;
      $576 = $575&65535;
      $577 = HEAP16[$nEq>>1]|0;
      $578 = $577&65535;
      $579 = ($576|0)==($578|0);
      if ($579) {
       label = 64;
      }
     }
    }
    if ((label|0) == 64) {
     $580 = HEAP32[$pRangeEnd>>2]|0;
     HEAP32[$t>>2] = $580;
     $581 = HEAP32[$pRangeStart>>2]|0;
     HEAP32[$pRangeEnd>>2] = $581;
     $582 = HEAP32[$t>>2]|0;
     HEAP32[$pRangeStart>>2] = $582;
     $583 = HEAP8[$bSeekPastNull>>0]|0;
     HEAP8[$t3>>0] = $583;
     $584 = HEAP8[$bStopAtNull>>0]|0;
     HEAP8[$bSeekPastNull>>0] = $584;
     $585 = HEAP8[$t3>>0]|0;
     HEAP8[$bStopAtNull>>0] = $585;
    }
    $586 = HEAP32[$pParse>>2]|0;
    $587 = HEAP32[$pLevel>>2]|0;
    $588 = HEAP32[$bRev>>2]|0;
    $589 = HEAP32[$nExtraReg>>2]|0;
    $590 = (_codeAllEqualityTerms($586,$587,$588,$589,$zStartAff)|0);
    HEAP32[$regBase>>2] = $590;
    $591 = HEAP32[$zStartAff>>2]|0;
    $592 = ($591|0)!=(0|0);
    if ($592) {
     $593 = HEAP16[$nEq>>1]|0;
     $594 = $593&65535;
     $595 = HEAP32[$zStartAff>>2]|0;
     $596 = (($595) + ($594)|0);
     $597 = HEAP8[$596>>0]|0;
     HEAP8[$cEndAff>>0] = $597;
    }
    $598 = HEAP32[$pLevel>>2]|0;
    $599 = ((($598)) + 16|0);
    $600 = HEAP32[$599>>2]|0;
    HEAP32[$addrNxt>>2] = $600;
    $601 = HEAP32[$pRangeStart>>2]|0;
    $602 = ($601|0)!=(0|0);
    if ($602) {
     $603 = HEAP32[$pRangeStart>>2]|0;
     $604 = ((($603)) + 18|0);
     $605 = HEAP16[$604>>1]|0;
     $606 = $605&65535;
     $607 = $606 & 40;
     $608 = ($607|0)!=(0);
     $610 = $608;
    } else {
     $610 = 1;
    }
    $609 = $610&1;
    HEAP32[$startEq>>2] = $609;
    $611 = HEAP32[$pRangeEnd>>2]|0;
    $612 = ($611|0)!=(0|0);
    if ($612) {
     $613 = HEAP32[$pRangeEnd>>2]|0;
     $614 = ((($613)) + 18|0);
     $615 = HEAP16[$614>>1]|0;
     $616 = $615&65535;
     $617 = $616 & 40;
     $618 = ($617|0)!=(0);
     $620 = $618;
    } else {
     $620 = 1;
    }
    $619 = $620&1;
    HEAP32[$endEq>>2] = $619;
    $621 = HEAP32[$pRangeStart>>2]|0;
    $622 = ($621|0)!=(0|0);
    if ($622) {
     $627 = 1;
    } else {
     $623 = HEAP16[$nEq>>1]|0;
     $624 = $623&65535;
     $625 = ($624|0)>(0);
     $627 = $625;
    }
    $626 = $627&1;
    HEAP32[$start_constraints>>2] = $626;
    $628 = HEAP16[$nEq>>1]|0;
    $629 = $628&65535;
    HEAP32[$nConstraint2>>2] = $629;
    $630 = HEAP32[$pRangeStart>>2]|0;
    $631 = ($630|0)!=(0|0);
    if ($631) {
     $632 = HEAP32[$pRangeStart>>2]|0;
     $633 = HEAP32[$632>>2]|0;
     $634 = ((($633)) + 16|0);
     $635 = HEAP32[$634>>2]|0;
     HEAP32[$pRight>>2] = $635;
     $636 = HEAP32[$pParse>>2]|0;
     $637 = HEAP32[$pRight>>2]|0;
     $638 = HEAP32[$regBase>>2]|0;
     $639 = HEAP16[$nEq>>1]|0;
     $640 = $639&65535;
     $641 = (($638) + ($640))|0;
     _sqlite3ExprCode($636,$637,$641);
     $642 = HEAP32[$v>>2]|0;
     $643 = HEAP32[$pLevel>>2]|0;
     $644 = HEAP32[$pRangeStart>>2]|0;
     _whereLikeOptimizationStringFixup($642,$643,$644);
     $645 = HEAP32[$pRangeStart>>2]|0;
     $646 = ((($645)) + 20|0);
     $647 = HEAP16[$646>>1]|0;
     $648 = $647&65535;
     $649 = $648 & 0;
     $650 = ($649|0)==(0);
     do {
      if ($650) {
       $651 = HEAP32[$pRight>>2]|0;
       $652 = (_sqlite3ExprCanBeNull($651)|0);
       $653 = ($652|0)!=(0);
       if (!($653)) {
        break;
       }
       $654 = HEAP32[$v>>2]|0;
       $655 = HEAP32[$regBase>>2]|0;
       $656 = HEAP16[$nEq>>1]|0;
       $657 = $656&65535;
       $658 = (($655) + ($657))|0;
       $659 = HEAP32[$addrNxt>>2]|0;
       (_sqlite3VdbeAddOp2($654,76,$658,$659)|0);
      }
     } while(0);
     $660 = HEAP32[$zStartAff>>2]|0;
     $661 = ($660|0)!=(0|0);
     do {
      if ($661) {
       $662 = HEAP32[$pRight>>2]|0;
       $663 = HEAP16[$nEq>>1]|0;
       $664 = $663&65535;
       $665 = HEAP32[$zStartAff>>2]|0;
       $666 = (($665) + ($664)|0);
       $667 = HEAP8[$666>>0]|0;
       $668 = (_sqlite3CompareAffinity($662,$667)|0);
       $669 = $668 << 24 >> 24;
       $670 = ($669|0)==(65);
       if ($670) {
        $671 = HEAP16[$nEq>>1]|0;
        $672 = $671&65535;
        $673 = HEAP32[$zStartAff>>2]|0;
        $674 = (($673) + ($672)|0);
        HEAP8[$674>>0] = 65;
       }
       $675 = HEAP32[$pRight>>2]|0;
       $676 = HEAP16[$nEq>>1]|0;
       $677 = $676&65535;
       $678 = HEAP32[$zStartAff>>2]|0;
       $679 = (($678) + ($677)|0);
       $680 = HEAP8[$679>>0]|0;
       $681 = (_sqlite3ExprNeedsNoAffinityChange($675,$680)|0);
       $682 = ($681|0)!=(0);
       if (!($682)) {
        break;
       }
       $683 = HEAP16[$nEq>>1]|0;
       $684 = $683&65535;
       $685 = HEAP32[$zStartAff>>2]|0;
       $686 = (($685) + ($684)|0);
       HEAP8[$686>>0] = 65;
      }
     } while(0);
     $687 = HEAP32[$nConstraint2>>2]|0;
     $688 = (($687) + 1)|0;
     HEAP32[$nConstraint2>>2] = $688;
    } else {
     $689 = HEAP8[$bSeekPastNull>>0]|0;
     $690 = ($689<<24>>24)!=(0);
     if ($690) {
      $691 = HEAP32[$v>>2]|0;
      $692 = HEAP32[$regBase>>2]|0;
      $693 = HEAP16[$nEq>>1]|0;
      $694 = $693&65535;
      $695 = (($692) + ($694))|0;
      (_sqlite3VdbeAddOp2($691,25,0,$695)|0);
      $696 = HEAP32[$nConstraint2>>2]|0;
      $697 = (($696) + 1)|0;
      HEAP32[$nConstraint2>>2] = $697;
      HEAP32[$startEq>>2] = 0;
      HEAP32[$start_constraints>>2] = 1;
     }
    }
    $698 = HEAP32[$pParse>>2]|0;
    $699 = HEAP32[$regBase>>2]|0;
    $700 = HEAP32[$nConstraint2>>2]|0;
    $701 = HEAP8[$bSeekPastNull>>0]|0;
    $702 = $701&255;
    $703 = (($700) - ($702))|0;
    $704 = HEAP32[$zStartAff>>2]|0;
    _codeApplyAffinity($698,$699,$703,$704);
    $705 = HEAP32[$start_constraints>>2]|0;
    $706 = $705 << 2;
    $707 = HEAP32[$startEq>>2]|0;
    $708 = $707 << 1;
    $709 = (($706) + ($708))|0;
    $710 = HEAP32[$bRev>>2]|0;
    $711 = (($709) + ($710))|0;
    $712 = (33014 + ($711)|0);
    $713 = HEAP8[$712>>0]|0;
    $714 = $713&255;
    HEAP32[$op>>2] = $714;
    $715 = HEAP32[$v>>2]|0;
    $716 = HEAP32[$op>>2]|0;
    $717 = HEAP32[$iIdxCur>>2]|0;
    $718 = HEAP32[$addrNxt>>2]|0;
    $719 = HEAP32[$regBase>>2]|0;
    $720 = HEAP32[$nConstraint2>>2]|0;
    (_sqlite3VdbeAddOp4Int($715,$716,$717,$718,$719,$720)|0);
    $721 = HEAP16[$nEq>>1]|0;
    $722 = $721&65535;
    HEAP32[$nConstraint2>>2] = $722;
    $723 = HEAP32[$pRangeEnd>>2]|0;
    $724 = ($723|0)!=(0|0);
    do {
     if ($724) {
      $725 = HEAP32[$pRangeEnd>>2]|0;
      $726 = HEAP32[$725>>2]|0;
      $727 = ((($726)) + 16|0);
      $728 = HEAP32[$727>>2]|0;
      HEAP32[$pRight4>>2] = $728;
      $729 = HEAP32[$pParse>>2]|0;
      $730 = HEAP32[$regBase>>2]|0;
      $731 = HEAP16[$nEq>>1]|0;
      $732 = $731&65535;
      $733 = (($730) + ($732))|0;
      _sqlite3ExprCacheRemove($729,$733,1);
      $734 = HEAP32[$pParse>>2]|0;
      $735 = HEAP32[$pRight4>>2]|0;
      $736 = HEAP32[$regBase>>2]|0;
      $737 = HEAP16[$nEq>>1]|0;
      $738 = $737&65535;
      $739 = (($736) + ($738))|0;
      _sqlite3ExprCode($734,$735,$739);
      $740 = HEAP32[$v>>2]|0;
      $741 = HEAP32[$pLevel>>2]|0;
      $742 = HEAP32[$pRangeEnd>>2]|0;
      _whereLikeOptimizationStringFixup($740,$741,$742);
      $743 = HEAP32[$pRangeEnd>>2]|0;
      $744 = ((($743)) + 20|0);
      $745 = HEAP16[$744>>1]|0;
      $746 = $745&65535;
      $747 = $746 & 0;
      $748 = ($747|0)==(0);
      do {
       if ($748) {
        $749 = HEAP32[$pRight4>>2]|0;
        $750 = (_sqlite3ExprCanBeNull($749)|0);
        $751 = ($750|0)!=(0);
        if (!($751)) {
         break;
        }
        $752 = HEAP32[$v>>2]|0;
        $753 = HEAP32[$regBase>>2]|0;
        $754 = HEAP16[$nEq>>1]|0;
        $755 = $754&65535;
        $756 = (($753) + ($755))|0;
        $757 = HEAP32[$addrNxt>>2]|0;
        (_sqlite3VdbeAddOp2($752,76,$756,$757)|0);
       }
      } while(0);
      $758 = HEAP32[$pRight4>>2]|0;
      $759 = HEAP8[$cEndAff>>0]|0;
      $760 = (_sqlite3CompareAffinity($758,$759)|0);
      $761 = $760 << 24 >> 24;
      $762 = ($761|0)!=(65);
      do {
       if ($762) {
        $763 = HEAP32[$pRight4>>2]|0;
        $764 = HEAP8[$cEndAff>>0]|0;
        $765 = (_sqlite3ExprNeedsNoAffinityChange($763,$764)|0);
        $766 = ($765|0)!=(0);
        if ($766) {
         break;
        }
        $767 = HEAP32[$pParse>>2]|0;
        $768 = HEAP32[$regBase>>2]|0;
        $769 = HEAP16[$nEq>>1]|0;
        $770 = $769&65535;
        $771 = (($768) + ($770))|0;
        _codeApplyAffinity($767,$771,1,$cEndAff);
       }
      } while(0);
      $772 = HEAP32[$nConstraint2>>2]|0;
      $773 = (($772) + 1)|0;
      HEAP32[$nConstraint2>>2] = $773;
     } else {
      $774 = HEAP8[$bStopAtNull>>0]|0;
      $775 = ($774<<24>>24)!=(0);
      if (!($775)) {
       break;
      }
      $776 = HEAP32[$v>>2]|0;
      $777 = HEAP32[$regBase>>2]|0;
      $778 = HEAP16[$nEq>>1]|0;
      $779 = $778&65535;
      $780 = (($777) + ($779))|0;
      (_sqlite3VdbeAddOp2($776,25,0,$780)|0);
      HEAP32[$endEq>>2] = 0;
      $781 = HEAP32[$nConstraint2>>2]|0;
      $782 = (($781) + 1)|0;
      HEAP32[$nConstraint2>>2] = $782;
     }
    } while(0);
    $783 = HEAP32[$db>>2]|0;
    $784 = HEAP32[$zStartAff>>2]|0;
    _sqlite3DbFree($783,$784);
    $785 = HEAP32[$v>>2]|0;
    $786 = (_sqlite3VdbeCurrentAddr($785)|0);
    $787 = HEAP32[$pLevel>>2]|0;
    $788 = ((($787)) + 52|0);
    HEAP32[$788>>2] = $786;
    $789 = HEAP32[$nConstraint2>>2]|0;
    $790 = ($789|0)!=(0);
    if ($790) {
     $791 = HEAP32[$bRev>>2]|0;
     $792 = $791<<1;
     $793 = HEAP32[$endEq>>2]|0;
     $794 = (($792) + ($793))|0;
     $795 = (33022 + ($794)|0);
     $796 = HEAP8[$795>>0]|0;
     $797 = $796&255;
     HEAP32[$op>>2] = $797;
     $798 = HEAP32[$v>>2]|0;
     $799 = HEAP32[$op>>2]|0;
     $800 = HEAP32[$iIdxCur>>2]|0;
     $801 = HEAP32[$addrNxt>>2]|0;
     $802 = HEAP32[$regBase>>2]|0;
     $803 = HEAP32[$nConstraint2>>2]|0;
     (_sqlite3VdbeAddOp4Int($798,$799,$800,$801,$802,$803)|0);
    }
    $804 = HEAP32[$pLevel>>2]|0;
    $805 = HEAP32[$pRangeStart>>2]|0;
    _disableTerm($804,$805);
    $806 = HEAP32[$pLevel>>2]|0;
    $807 = HEAP32[$pRangeEnd>>2]|0;
    _disableTerm($806,$807);
    $808 = HEAP32[$omitTable>>2]|0;
    $809 = ($808|0)!=(0);
    do {
     if (!($809)) {
      $810 = HEAP32[$pIdx>>2]|0;
      $811 = ((($810)) + 12|0);
      $812 = HEAP32[$811>>2]|0;
      $813 = ((($812)) + 42|0);
      $814 = HEAP8[$813>>0]|0;
      $815 = $814&255;
      $816 = $815 & 32;
      $817 = ($816|0)==(0);
      if ($817) {
       $818 = HEAP32[$3>>2]|0;
       $819 = ((($818)) + 38|0);
       $820 = HEAP8[$819>>0]|0;
       $821 = $820&255;
       $822 = ($821|0)!=(0);
       if ($822) {
        $823 = HEAP32[$pParse>>2]|0;
        $824 = ((($823)) + 76|0);
        $825 = HEAP32[$824>>2]|0;
        $826 = (($825) + 1)|0;
        HEAP32[$824>>2] = $826;
        HEAP32[$iRowidReg>>2] = $826;
        $827 = HEAP32[$v>>2]|0;
        $828 = HEAP32[$iIdxCur>>2]|0;
        $829 = HEAP32[$iRowidReg>>2]|0;
        (_sqlite3VdbeAddOp2($827,113,$828,$829)|0);
        $830 = HEAP32[$pParse>>2]|0;
        $831 = HEAP32[$iCur>>2]|0;
        $832 = HEAP32[$iRowidReg>>2]|0;
        _sqlite3ExprCacheStore($830,$831,-1,$832);
        $833 = HEAP32[$v>>2]|0;
        $834 = HEAP32[$iCur>>2]|0;
        $835 = HEAP32[$iRowidReg>>2]|0;
        (_sqlite3VdbeAddOp3($833,70,$834,0,$835)|0);
        break;
       } else {
        $836 = HEAP32[$3>>2]|0;
        $837 = HEAP32[$pIdx>>2]|0;
        $838 = HEAP32[$iCur>>2]|0;
        $839 = HEAP32[$iIdxCur>>2]|0;
        _codeDeferredSeek($836,$837,$838,$839);
        break;
       }
      }
      $840 = HEAP32[$iCur>>2]|0;
      $841 = HEAP32[$iIdxCur>>2]|0;
      $842 = ($840|0)!=($841|0);
      if (!($842)) {
       break;
      }
      $843 = HEAP32[$pIdx>>2]|0;
      $844 = ((($843)) + 12|0);
      $845 = HEAP32[$844>>2]|0;
      $846 = (_sqlite3PrimaryKeyIndex($845)|0);
      HEAP32[$pPk>>2] = $846;
      $847 = HEAP32[$pParse>>2]|0;
      $848 = HEAP32[$pPk>>2]|0;
      $849 = ((($848)) + 50|0);
      $850 = HEAP16[$849>>1]|0;
      $851 = $850&65535;
      $852 = (_sqlite3GetTempRange($847,$851)|0);
      HEAP32[$iRowidReg>>2] = $852;
      HEAP32[$j>>2] = 0;
      while(1) {
       $853 = HEAP32[$j>>2]|0;
       $854 = HEAP32[$pPk>>2]|0;
       $855 = ((($854)) + 50|0);
       $856 = HEAP16[$855>>1]|0;
       $857 = $856&65535;
       $858 = ($853|0)<($857|0);
       if (!($858)) {
        break;
       }
       $859 = HEAP32[$pIdx>>2]|0;
       $860 = HEAP32[$j>>2]|0;
       $861 = HEAP32[$pPk>>2]|0;
       $862 = ((($861)) + 4|0);
       $863 = HEAP32[$862>>2]|0;
       $864 = (($863) + ($860<<1)|0);
       $865 = HEAP16[$864>>1]|0;
       $866 = (_sqlite3ColumnOfIndex($859,$865)|0);
       $867 = $866 << 16 >> 16;
       HEAP32[$k>>2] = $867;
       $868 = HEAP32[$v>>2]|0;
       $869 = HEAP32[$iIdxCur>>2]|0;
       $870 = HEAP32[$k>>2]|0;
       $871 = HEAP32[$iRowidReg>>2]|0;
       $872 = HEAP32[$j>>2]|0;
       $873 = (($871) + ($872))|0;
       (_sqlite3VdbeAddOp3($868,47,$869,$870,$873)|0);
       $874 = HEAP32[$j>>2]|0;
       $875 = (($874) + 1)|0;
       HEAP32[$j>>2] = $875;
      }
      $876 = HEAP32[$v>>2]|0;
      $877 = HEAP32[$iCur>>2]|0;
      $878 = HEAP32[$addrCont>>2]|0;
      $879 = HEAP32[$iRowidReg>>2]|0;
      $880 = HEAP32[$pPk>>2]|0;
      $881 = ((($880)) + 50|0);
      $882 = HEAP16[$881>>1]|0;
      $883 = $882&65535;
      (_sqlite3VdbeAddOp4Int($876,68,$877,$878,$879,$883)|0);
     }
    } while(0);
    $884 = HEAP32[$pLoop>>2]|0;
    $885 = ((($884)) + 36|0);
    $886 = HEAP32[$885>>2]|0;
    $887 = $886 & 4096;
    $888 = ($887|0)!=(0);
    do {
     if ($888) {
      $889 = HEAP32[$pLevel>>2]|0;
      $890 = ((($889)) + 45|0);
      HEAP8[$890>>0] = -96;
     } else {
      $891 = HEAP32[$bRev>>2]|0;
      $892 = ($891|0)!=(0);
      $893 = HEAP32[$pLevel>>2]|0;
      $894 = ((($893)) + 45|0);
      if ($892) {
       HEAP8[$894>>0] = 6;
       break;
      } else {
       HEAP8[$894>>0] = 7;
       break;
      }
     }
    } while(0);
    $895 = HEAP32[$iIdxCur>>2]|0;
    $896 = HEAP32[$pLevel>>2]|0;
    $897 = ((($896)) + 48|0);
    HEAP32[$897>>2] = $895;
    $898 = HEAP32[$pLoop>>2]|0;
    $899 = ((($898)) + 36|0);
    $900 = HEAP32[$899>>2]|0;
    $901 = $900 & 65536;
    $902 = ($901|0)!=(0);
    $903 = $902 ? 1 : 0;
    $904 = $903&255;
    $905 = HEAP32[$pLevel>>2]|0;
    $906 = ((($905)) + 46|0);
    HEAP8[$906>>0] = $904;
    $907 = HEAP32[$pLoop>>2]|0;
    $908 = ((($907)) + 36|0);
    $909 = HEAP32[$908>>2]|0;
    $910 = $909 & 15;
    $911 = ($910|0)==(0);
    if (!($911)) {
     break;
    }
    $912 = HEAP32[$pLevel>>2]|0;
    $913 = ((($912)) + 47|0);
    HEAP8[$913>>0] = 1;
    break;
   }
   $914 = ((($444)) + 36|0);
   $915 = HEAP32[$914>>2]|0;
   $916 = $915 & 8192;
   $917 = ($916|0)!=(0);
   if (!($917)) {
    $1329 = HEAP32[$pTabItem>>2]|0;
    $1330 = ((($1329)) + 36|0);
    $1331 = ((($1330)) + 1|0);
    $1332 = HEAP8[$1331>>0]|0;
    $1333 = ($1332&255) >>> 5;
    $1334 = $1333 & 1;
    $1335 = $1334&255;
    $1336 = ($1335|0)!=(0);
    if ($1336) {
     $1337 = HEAP32[$pLevel>>2]|0;
     $1338 = ((($1337)) + 45|0);
     HEAP8[$1338>>0] = -96;
     break;
    } else {
     $1339 = HEAP32[$bRev>>2]|0;
     $1340 = (33026 + ($1339)|0);
     $1341 = HEAP8[$1340>>0]|0;
     $1342 = HEAP32[$pLevel>>2]|0;
     $1343 = ((($1342)) + 45|0);
     HEAP8[$1343>>0] = $1341;
     $1344 = HEAP32[$iCur>>2]|0;
     $1345 = HEAP32[$pLevel>>2]|0;
     $1346 = ((($1345)) + 48|0);
     HEAP32[$1346>>2] = $1344;
     $1347 = HEAP32[$v>>2]|0;
     $1348 = HEAP32[$bRev>>2]|0;
     $1349 = (33028 + ($1348)|0);
     $1350 = HEAP8[$1349>>0]|0;
     $1351 = $1350&255;
     $1352 = HEAP32[$iCur>>2]|0;
     $1353 = HEAP32[$addrBrk>>2]|0;
     $1354 = (_sqlite3VdbeAddOp2($1347,$1351,$1352,$1353)|0);
     $1355 = (1 + ($1354))|0;
     $1356 = HEAP32[$pLevel>>2]|0;
     $1357 = ((($1356)) + 52|0);
     HEAP32[$1357>>2] = $1355;
     $1358 = HEAP32[$pLevel>>2]|0;
     $1359 = ((($1358)) + 47|0);
     HEAP8[$1359>>0] = 1;
     break;
    }
   }
   HEAP32[$pCov>>2] = 0;
   $918 = HEAP32[$pParse>>2]|0;
   $919 = ((($918)) + 72|0);
   $920 = HEAP32[$919>>2]|0;
   $921 = (($920) + 1)|0;
   HEAP32[$919>>2] = $921;
   HEAP32[$iCovCur>>2] = $920;
   $922 = HEAP32[$pParse>>2]|0;
   $923 = ((($922)) + 76|0);
   $924 = HEAP32[$923>>2]|0;
   $925 = (($924) + 1)|0;
   HEAP32[$923>>2] = $925;
   HEAP32[$regReturn>>2] = $925;
   HEAP32[$regRowset>>2] = 0;
   HEAP32[$regRowid>>2] = 0;
   $926 = HEAP32[$v>>2]|0;
   $927 = (_sqlite3VdbeMakeLabel($926)|0);
   HEAP32[$iLoopBody>>2] = $927;
   HEAP32[$untestedTerms>>2] = 0;
   HEAP32[$pAndExpr>>2] = 0;
   $928 = HEAP32[$pTabItem>>2]|0;
   $929 = ((($928)) + 16|0);
   $930 = HEAP32[$929>>2]|0;
   HEAP32[$pTab>>2] = $930;
   $931 = HEAP32[$pLoop>>2]|0;
   $932 = ((($931)) + 48|0);
   $933 = HEAP32[$932>>2]|0;
   $934 = HEAP32[$933>>2]|0;
   HEAP32[$pTerm>>2] = $934;
   $935 = HEAP32[$pTerm>>2]|0;
   $936 = ((($935)) + 12|0);
   $937 = HEAP32[$936>>2]|0;
   HEAP32[$pOrWc>>2] = $937;
   $938 = HEAP32[$pLevel>>2]|0;
   $939 = ((($938)) + 45|0);
   HEAP8[$939>>0] = 15;
   $940 = HEAP32[$regReturn>>2]|0;
   $941 = HEAP32[$pLevel>>2]|0;
   $942 = ((($941)) + 48|0);
   HEAP32[$942>>2] = $940;
   $943 = HEAP32[$3>>2]|0;
   $944 = ((($943)) + 41|0);
   $945 = HEAP8[$944>>0]|0;
   $946 = $945&255;
   $947 = ($946|0)>(1);
   $948 = HEAP32[$3>>2]|0;
   L165: do {
    if ($947) {
     $949 = ((($948)) + 41|0);
     $950 = HEAP8[$949>>0]|0;
     $951 = $950&255;
     $952 = HEAP32[$4>>2]|0;
     $953 = (($951) - ($952))|0;
     $954 = (($953) - 1)|0;
     HEAP32[$nNotReady>>2] = $954;
     $955 = HEAP32[$db>>2]|0;
     $956 = HEAP32[$nNotReady>>2]|0;
     $957 = ($956*72)|0;
     $958 = (80 + ($957))|0;
     $959 = (_sqlite3DbMallocRaw($955,$958,0)|0);
     HEAP32[$pOrTab>>2] = $959;
     $960 = HEAP32[$pOrTab>>2]|0;
     $961 = ($960|0)==(0|0);
     if ($961) {
      $962 = $5;
      $963 = $962;
      $964 = HEAP32[$963>>2]|0;
      $965 = (($962) + 4)|0;
      $966 = $965;
      $967 = HEAP32[$966>>2]|0;
      $968 = $2;
      $969 = $968;
      HEAP32[$969>>2] = $964;
      $970 = (($968) + 4)|0;
      $971 = $970;
      HEAP32[$971>>2] = $967;
      $1584 = $2;
      $1585 = $1584;
      $1586 = HEAP32[$1585>>2]|0;
      $1587 = (($1584) + 4)|0;
      $1588 = $1587;
      $1589 = HEAP32[$1588>>2]|0;
      tempRet0 = ($1589);
      STACKTOP = sp;return ($1586|0);
     }
     $972 = HEAP32[$nNotReady>>2]|0;
     $973 = (($972) + 1)|0;
     $974 = $973&255;
     $975 = $974&255;
     $976 = HEAP32[$pOrTab>>2]|0;
     $977 = ((($976)) + 4|0);
     HEAP32[$977>>2] = $975;
     $978 = HEAP32[$pOrTab>>2]|0;
     $979 = ((($978)) + 4|0);
     $980 = HEAP32[$979>>2]|0;
     $981 = HEAP32[$pOrTab>>2]|0;
     HEAP32[$981>>2] = $980;
     $982 = HEAP32[$pOrTab>>2]|0;
     $983 = ((($982)) + 8|0);
     $984 = HEAP32[$pTabItem>>2]|0;
     dest=$983; src=$984; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
     $985 = HEAP32[$3>>2]|0;
     $986 = ((($985)) + 4|0);
     $987 = HEAP32[$986>>2]|0;
     $988 = ((($987)) + 8|0);
     HEAP32[$origSrc>>2] = $988;
     HEAP32[$k>>2] = 1;
     while(1) {
      $989 = HEAP32[$k>>2]|0;
      $990 = HEAP32[$nNotReady>>2]|0;
      $991 = ($989|0)<=($990|0);
      if (!($991)) {
       break L165;
      }
      $992 = HEAP32[$k>>2]|0;
      $993 = HEAP32[$pOrTab>>2]|0;
      $994 = ((($993)) + 8|0);
      $995 = (($994) + (($992*72)|0)|0);
      $996 = HEAP32[$k>>2]|0;
      $997 = HEAP32[$pLevel>>2]|0;
      $998 = (($997) + (($996*80)|0)|0);
      $999 = ((($998)) + 44|0);
      $1000 = HEAP8[$999>>0]|0;
      $1001 = $1000&255;
      $1002 = HEAP32[$origSrc>>2]|0;
      $1003 = (($1002) + (($1001*72)|0)|0);
      dest=$995; src=$1003; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
      $1004 = HEAP32[$k>>2]|0;
      $1005 = (($1004) + 1)|0;
      HEAP32[$k>>2] = $1005;
     }
    } else {
     $1006 = ((($948)) + 4|0);
     $1007 = HEAP32[$1006>>2]|0;
     HEAP32[$pOrTab>>2] = $1007;
    }
   } while(0);
   $1008 = HEAP32[$3>>2]|0;
   $1009 = ((($1008)) + 34|0);
   $1010 = HEAP16[$1009>>1]|0;
   $1011 = $1010&65535;
   $1012 = $1011 & 8;
   $1013 = ($1012|0)==(0);
   if ($1013) {
    $1014 = HEAP32[$pTab>>2]|0;
    $1015 = ((($1014)) + 42|0);
    $1016 = HEAP8[$1015>>0]|0;
    $1017 = $1016&255;
    $1018 = $1017 & 32;
    $1019 = ($1018|0)==(0);
    if ($1019) {
     $1020 = HEAP32[$pParse>>2]|0;
     $1021 = ((($1020)) + 76|0);
     $1022 = HEAP32[$1021>>2]|0;
     $1023 = (($1022) + 1)|0;
     HEAP32[$1021>>2] = $1023;
     HEAP32[$regRowset>>2] = $1023;
     $1024 = HEAP32[$v>>2]|0;
     $1025 = HEAP32[$regRowset>>2]|0;
     (_sqlite3VdbeAddOp2($1024,25,0,$1025)|0);
    } else {
     $1026 = HEAP32[$pTab>>2]|0;
     $1027 = (_sqlite3PrimaryKeyIndex($1026)|0);
     HEAP32[$pPk5>>2] = $1027;
     $1028 = HEAP32[$pParse>>2]|0;
     $1029 = ((($1028)) + 72|0);
     $1030 = HEAP32[$1029>>2]|0;
     $1031 = (($1030) + 1)|0;
     HEAP32[$1029>>2] = $1031;
     HEAP32[$regRowset>>2] = $1030;
     $1032 = HEAP32[$v>>2]|0;
     $1033 = HEAP32[$regRowset>>2]|0;
     $1034 = HEAP32[$pPk5>>2]|0;
     $1035 = ((($1034)) + 50|0);
     $1036 = HEAP16[$1035>>1]|0;
     $1037 = $1036&65535;
     (_sqlite3VdbeAddOp2($1032,57,$1033,$1037)|0);
     $1038 = HEAP32[$pParse>>2]|0;
     $1039 = HEAP32[$pPk5>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($1038,$1039);
    }
    $1040 = HEAP32[$pParse>>2]|0;
    $1041 = ((($1040)) + 76|0);
    $1042 = HEAP32[$1041>>2]|0;
    $1043 = (($1042) + 1)|0;
    HEAP32[$1041>>2] = $1043;
    HEAP32[$regRowid>>2] = $1043;
   }
   $1044 = HEAP32[$v>>2]|0;
   $1045 = HEAP32[$regReturn>>2]|0;
   $1046 = (_sqlite3VdbeAddOp2($1044,22,0,$1045)|0);
   HEAP32[$iRetInit>>2] = $1046;
   $1047 = HEAP32[$pWC>>2]|0;
   $1048 = ((($1047)) + 12|0);
   $1049 = HEAP32[$1048>>2]|0;
   $1050 = ($1049|0)>(1);
   if ($1050) {
    HEAP32[$iTerm>>2] = 0;
    while(1) {
     $1051 = HEAP32[$iTerm>>2]|0;
     $1052 = HEAP32[$pWC>>2]|0;
     $1053 = ((($1052)) + 12|0);
     $1054 = HEAP32[$1053>>2]|0;
     $1055 = ($1051|0)<($1054|0);
     if (!($1055)) {
      break;
     }
     $1056 = HEAP32[$iTerm>>2]|0;
     $1057 = HEAP32[$pWC>>2]|0;
     $1058 = ((($1057)) + 20|0);
     $1059 = HEAP32[$1058>>2]|0;
     $1060 = (($1059) + (($1056*48)|0)|0);
     $1061 = HEAP32[$1060>>2]|0;
     HEAP32[$pExpr>>2] = $1061;
     $1062 = HEAP32[$iTerm>>2]|0;
     $1063 = HEAP32[$pWC>>2]|0;
     $1064 = ((($1063)) + 20|0);
     $1065 = HEAP32[$1064>>2]|0;
     $1066 = (($1065) + (($1062*48)|0)|0);
     $1067 = HEAP32[$pTerm>>2]|0;
     $1068 = ($1066|0)==($1067|0);
     if (!($1068)) {
      $1069 = HEAP32[$pExpr>>2]|0;
      $1070 = ((($1069)) + 4|0);
      $1071 = HEAP32[$1070>>2]|0;
      $1072 = $1071 & 1;
      $1073 = ($1072|0)!=(0);
      if (!($1073)) {
       $1074 = HEAP32[$iTerm>>2]|0;
       $1075 = HEAP32[$pWC>>2]|0;
       $1076 = ((($1075)) + 20|0);
       $1077 = HEAP32[$1076>>2]|0;
       $1078 = (($1077) + (($1074*48)|0)|0);
       $1079 = ((($1078)) + 20|0);
       $1080 = HEAP16[$1079>>1]|0;
       $1081 = $1080&65535;
       $1082 = $1081 & 6;
       $1083 = ($1082|0)!=(0);
       if (!($1083)) {
        $1084 = HEAP32[$iTerm>>2]|0;
        $1085 = HEAP32[$pWC>>2]|0;
        $1086 = ((($1085)) + 20|0);
        $1087 = HEAP32[$1086>>2]|0;
        $1088 = (($1087) + (($1084*48)|0)|0);
        $1089 = ((($1088)) + 18|0);
        $1090 = HEAP16[$1089>>1]|0;
        $1091 = $1090&65535;
        $1092 = $1091 & 8191;
        $1093 = ($1092|0)==(0);
        if (!($1093)) {
         $1094 = HEAP32[$db>>2]|0;
         $1095 = HEAP32[$pExpr>>2]|0;
         $1096 = (_sqlite3ExprDup($1094,$1095,0)|0);
         HEAP32[$pExpr>>2] = $1096;
         $1097 = HEAP32[$db>>2]|0;
         $1098 = HEAP32[$pAndExpr>>2]|0;
         $1099 = HEAP32[$pExpr>>2]|0;
         $1100 = (_sqlite3ExprAnd($1097,$1098,$1099)|0);
         HEAP32[$pAndExpr>>2] = $1100;
        }
       }
      }
     }
     $1101 = HEAP32[$iTerm>>2]|0;
     $1102 = (($1101) + 1)|0;
     HEAP32[$iTerm>>2] = $1102;
    }
    $1103 = HEAP32[$pAndExpr>>2]|0;
    $1104 = ($1103|0)!=(0|0);
    if ($1104) {
     $1105 = HEAP32[$pParse>>2]|0;
     $1106 = HEAP32[$pAndExpr>>2]|0;
     $1107 = (_sqlite3PExpr($1105,328,0,$1106,0)|0);
     HEAP32[$pAndExpr>>2] = $1107;
    }
   }
   HEAP16[$wctrlFlags>>1] = 240;
   HEAP32[$ii>>2] = 0;
   while(1) {
    $1108 = HEAP32[$ii>>2]|0;
    $1109 = HEAP32[$pOrWc>>2]|0;
    $1110 = ((($1109)) + 12|0);
    $1111 = HEAP32[$1110>>2]|0;
    $1112 = ($1108|0)<($1111|0);
    if (!($1112)) {
     break;
    }
    $1113 = HEAP32[$ii>>2]|0;
    $1114 = HEAP32[$pOrWc>>2]|0;
    $1115 = ((($1114)) + 20|0);
    $1116 = HEAP32[$1115>>2]|0;
    $1117 = (($1116) + (($1113*48)|0)|0);
    HEAP32[$pOrTerm>>2] = $1117;
    $1118 = HEAP32[$pOrTerm>>2]|0;
    $1119 = ((($1118)) + 8|0);
    $1120 = HEAP32[$1119>>2]|0;
    $1121 = HEAP32[$iCur>>2]|0;
    $1122 = ($1120|0)==($1121|0);
    if ($1122) {
     label = 142;
    } else {
     $1123 = HEAP32[$pOrTerm>>2]|0;
     $1124 = ((($1123)) + 18|0);
     $1125 = HEAP16[$1124>>1]|0;
     $1126 = $1125&65535;
     $1127 = $1126 & 1024;
     $1128 = ($1127|0)!=(0);
     if ($1128) {
      label = 142;
     }
    }
    if ((label|0) == 142) {
     label = 0;
     $1129 = HEAP32[$pOrTerm>>2]|0;
     $1130 = HEAP32[$1129>>2]|0;
     HEAP32[$pOrExpr>>2] = $1130;
     HEAP32[$jmp1>>2] = 0;
     $1131 = HEAP32[$pAndExpr>>2]|0;
     $1132 = ($1131|0)!=(0|0);
     if ($1132) {
      $1133 = HEAP32[$pOrExpr>>2]|0;
      $1134 = ((($1133)) + 4|0);
      $1135 = HEAP32[$1134>>2]|0;
      $1136 = $1135 & 1;
      $1137 = ($1136|0)!=(0);
      if (!($1137)) {
       $1138 = HEAP32[$pOrExpr>>2]|0;
       $1139 = HEAP32[$pAndExpr>>2]|0;
       $1140 = ((($1139)) + 12|0);
       HEAP32[$1140>>2] = $1138;
       $1141 = HEAP32[$pAndExpr>>2]|0;
       HEAP32[$pOrExpr>>2] = $1141;
      }
     }
     $1142 = HEAP32[$pParse>>2]|0;
     $1143 = HEAP32[$pOrTab>>2]|0;
     $1144 = HEAP32[$pOrExpr>>2]|0;
     $1145 = HEAP16[$wctrlFlags>>1]|0;
     $1146 = HEAP32[$iCovCur>>2]|0;
     $1147 = (_sqlite3WhereBegin($1142,$1143,$1144,0,0,$1145,$1146)|0);
     HEAP32[$pSubWInfo>>2] = $1147;
     $1148 = HEAP32[$pSubWInfo>>2]|0;
     $1149 = ($1148|0)!=(0|0);
     if ($1149) {
      $1150 = HEAP32[$pParse>>2]|0;
      $1151 = HEAP32[$pOrTab>>2]|0;
      $1152 = HEAP32[$pSubWInfo>>2]|0;
      $1153 = ((($1152)) + 736|0);
      $1154 = HEAP32[$4>>2]|0;
      $1155 = HEAP32[$pLevel>>2]|0;
      $1156 = ((($1155)) + 44|0);
      $1157 = HEAP8[$1156>>0]|0;
      $1158 = $1157&255;
      $1159 = (_sqlite3WhereExplainOneScan($1150,$1151,$1153,$1154,$1158,0)|0);
      HEAP32[$addrExplain>>2] = $1159;
      $1160 = HEAP32[$3>>2]|0;
      $1161 = ((($1160)) + 34|0);
      $1162 = HEAP16[$1161>>1]|0;
      $1163 = $1162&65535;
      $1164 = $1163 & 8;
      $1165 = ($1164|0)==(0);
      do {
       if ($1165) {
        $1166 = HEAP32[$ii>>2]|0;
        $1167 = HEAP32[$pOrWc>>2]|0;
        $1168 = ((($1167)) + 12|0);
        $1169 = HEAP32[$1168>>2]|0;
        $1170 = (($1169) - 1)|0;
        $1171 = ($1166|0)==($1170|0);
        $1172 = HEAP32[$ii>>2]|0;
        $1173 = $1171 ? -1 : $1172;
        HEAP32[$iSet>>2] = $1173;
        $1174 = HEAP32[$pTab>>2]|0;
        $1175 = ((($1174)) + 42|0);
        $1176 = HEAP8[$1175>>0]|0;
        $1177 = $1176&255;
        $1178 = $1177 & 32;
        $1179 = ($1178|0)==(0);
        if ($1179) {
         $1180 = HEAP32[$pParse>>2]|0;
         $1181 = HEAP32[$pTab>>2]|0;
         $1182 = HEAP32[$iCur>>2]|0;
         $1183 = HEAP32[$regRowid>>2]|0;
         $1184 = (_sqlite3ExprCodeGetColumn($1180,$1181,-1,$1182,$1183,0)|0);
         HEAP32[$r>>2] = $1184;
         $1185 = HEAP32[$v>>2]|0;
         $1186 = HEAP32[$regRowset>>2]|0;
         $1187 = HEAP32[$r>>2]|0;
         $1188 = HEAP32[$iSet>>2]|0;
         $1189 = (_sqlite3VdbeAddOp4Int($1185,131,$1186,0,$1187,$1188)|0);
         HEAP32[$jmp1>>2] = $1189;
         break;
        }
        $1190 = HEAP32[$pTab>>2]|0;
        $1191 = (_sqlite3PrimaryKeyIndex($1190)|0);
        HEAP32[$pPk6>>2] = $1191;
        $1192 = HEAP32[$pPk6>>2]|0;
        $1193 = ((($1192)) + 50|0);
        $1194 = HEAP16[$1193>>1]|0;
        $1195 = $1194&65535;
        HEAP32[$nPk>>2] = $1195;
        $1196 = HEAP32[$pParse>>2]|0;
        $1197 = HEAP32[$nPk>>2]|0;
        $1198 = (_sqlite3GetTempRange($1196,$1197)|0);
        HEAP32[$r>>2] = $1198;
        HEAP32[$iPk>>2] = 0;
        while(1) {
         $1199 = HEAP32[$iPk>>2]|0;
         $1200 = HEAP32[$nPk>>2]|0;
         $1201 = ($1199|0)<($1200|0);
         if (!($1201)) {
          break;
         }
         $1202 = HEAP32[$iPk>>2]|0;
         $1203 = HEAP32[$pPk6>>2]|0;
         $1204 = ((($1203)) + 4|0);
         $1205 = HEAP32[$1204>>2]|0;
         $1206 = (($1205) + ($1202<<1)|0);
         $1207 = HEAP16[$1206>>1]|0;
         $1208 = $1207 << 16 >> 16;
         HEAP32[$iCol>>2] = $1208;
         $1209 = HEAP32[$pParse>>2]|0;
         $1210 = HEAP32[$pTab>>2]|0;
         $1211 = HEAP32[$iCol>>2]|0;
         $1212 = HEAP32[$iCur>>2]|0;
         $1213 = HEAP32[$r>>2]|0;
         $1214 = HEAP32[$iPk>>2]|0;
         $1215 = (($1213) + ($1214))|0;
         _sqlite3ExprCodeGetColumnToReg($1209,$1210,$1211,$1212,$1215);
         $1216 = HEAP32[$iPk>>2]|0;
         $1217 = (($1216) + 1)|0;
         HEAP32[$iPk>>2] = $1217;
        }
        $1218 = HEAP32[$iSet>>2]|0;
        $1219 = ($1218|0)!=(0);
        if ($1219) {
         $1220 = HEAP32[$v>>2]|0;
         $1221 = HEAP32[$regRowset>>2]|0;
         $1222 = HEAP32[$r>>2]|0;
         $1223 = HEAP32[$nPk>>2]|0;
         $1224 = (_sqlite3VdbeAddOp4Int($1220,69,$1221,0,$1222,$1223)|0);
         HEAP32[$jmp1>>2] = $1224;
        }
        $1225 = HEAP32[$iSet>>2]|0;
        $1226 = ($1225|0)>=(0);
        do {
         if ($1226) {
          $1227 = HEAP32[$v>>2]|0;
          $1228 = HEAP32[$r>>2]|0;
          $1229 = HEAP32[$nPk>>2]|0;
          $1230 = HEAP32[$regRowid>>2]|0;
          (_sqlite3VdbeAddOp3($1227,49,$1228,$1229,$1230)|0);
          $1231 = HEAP32[$v>>2]|0;
          $1232 = HEAP32[$regRowset>>2]|0;
          $1233 = HEAP32[$regRowid>>2]|0;
          (_sqlite3VdbeAddOp3($1231,110,$1232,$1233,0)|0);
          $1234 = HEAP32[$iSet>>2]|0;
          $1235 = ($1234|0)!=(0);
          if (!($1235)) {
           break;
          }
          $1236 = HEAP32[$v>>2]|0;
          _sqlite3VdbeChangeP5($1236,16);
         }
        } while(0);
        $1237 = HEAP32[$pParse>>2]|0;
        $1238 = HEAP32[$r>>2]|0;
        $1239 = HEAP32[$nPk>>2]|0;
        _sqlite3ReleaseTempRange($1237,$1238,$1239);
       }
      } while(0);
      $1240 = HEAP32[$v>>2]|0;
      $1241 = HEAP32[$regReturn>>2]|0;
      $1242 = HEAP32[$iLoopBody>>2]|0;
      (_sqlite3VdbeAddOp2($1240,14,$1241,$1242)|0);
      $1243 = HEAP32[$jmp1>>2]|0;
      $1244 = ($1243|0)!=(0);
      if ($1244) {
       $1245 = HEAP32[$v>>2]|0;
       $1246 = HEAP32[$jmp1>>2]|0;
       _sqlite3VdbeJumpHere($1245,$1246);
      }
      $1247 = HEAP32[$pSubWInfo>>2]|0;
      $1248 = ((($1247)) + 39|0);
      $1249 = HEAP8[$1248>>0]|0;
      $1250 = ($1249<<24>>24)!=(0);
      if ($1250) {
       HEAP32[$untestedTerms>>2] = 1;
      }
      $1251 = HEAP32[$pSubWInfo>>2]|0;
      $1252 = ((($1251)) + 736|0);
      $1253 = ((($1252)) + 64|0);
      $1254 = HEAP32[$1253>>2]|0;
      HEAP32[$pSubLoop>>2] = $1254;
      $1255 = HEAP32[$pSubLoop>>2]|0;
      $1256 = ((($1255)) + 36|0);
      $1257 = HEAP32[$1256>>2]|0;
      $1258 = $1257 & 512;
      $1259 = ($1258|0)!=(0);
      do {
       if ($1259) {
        $1260 = HEAP32[$ii>>2]|0;
        $1261 = ($1260|0)==(0);
        if (!($1261)) {
         $1262 = HEAP32[$pSubLoop>>2]|0;
         $1263 = ((($1262)) + 24|0);
         $1264 = ((($1263)) + 4|0);
         $1265 = HEAP32[$1264>>2]|0;
         $1266 = HEAP32[$pCov>>2]|0;
         $1267 = ($1265|0)==($1266|0);
         if (!($1267)) {
          label = 168;
          break;
         }
        }
        $1268 = HEAP32[$pTab>>2]|0;
        $1269 = ((($1268)) + 42|0);
        $1270 = HEAP8[$1269>>0]|0;
        $1271 = $1270&255;
        $1272 = $1271 & 32;
        $1273 = ($1272|0)==(0);
        if (!($1273)) {
         $1274 = HEAP32[$pSubLoop>>2]|0;
         $1275 = ((($1274)) + 24|0);
         $1276 = ((($1275)) + 4|0);
         $1277 = HEAP32[$1276>>2]|0;
         $1278 = ((($1277)) + 55|0);
         $1279 = HEAP8[$1278>>0]|0;
         $1280 = $1279 & 3;
         $1281 = $1280&255;
         $1282 = ($1281|0)==(2);
         if ($1282) {
          label = 168;
          break;
         }
        }
        $1283 = HEAP32[$pSubLoop>>2]|0;
        $1284 = ((($1283)) + 24|0);
        $1285 = ((($1284)) + 4|0);
        $1286 = HEAP32[$1285>>2]|0;
        HEAP32[$pCov>>2] = $1286;
        $1287 = HEAP16[$wctrlFlags>>1]|0;
        $1288 = $1287&65535;
        $1289 = $1288 | 4096;
        $1290 = $1289&65535;
        HEAP16[$wctrlFlags>>1] = $1290;
       } else {
        label = 168;
       }
      } while(0);
      if ((label|0) == 168) {
       label = 0;
       HEAP32[$pCov>>2] = 0;
      }
      $1291 = HEAP32[$pSubWInfo>>2]|0;
      _sqlite3WhereEnd($1291);
     }
    }
    $1292 = HEAP32[$ii>>2]|0;
    $1293 = (($1292) + 1)|0;
    HEAP32[$ii>>2] = $1293;
   }
   $1294 = HEAP32[$pCov>>2]|0;
   $1295 = HEAP32[$pLevel>>2]|0;
   $1296 = ((($1295)) + 56|0);
   HEAP32[$1296>>2] = $1294;
   $1297 = HEAP32[$pCov>>2]|0;
   $1298 = ($1297|0)!=(0|0);
   if ($1298) {
    $1299 = HEAP32[$iCovCur>>2]|0;
    $1300 = HEAP32[$pLevel>>2]|0;
    $1301 = ((($1300)) + 8|0);
    HEAP32[$1301>>2] = $1299;
   }
   $1302 = HEAP32[$pAndExpr>>2]|0;
   $1303 = ($1302|0)!=(0|0);
   if ($1303) {
    $1304 = HEAP32[$pAndExpr>>2]|0;
    $1305 = ((($1304)) + 12|0);
    HEAP32[$1305>>2] = 0;
    $1306 = HEAP32[$db>>2]|0;
    $1307 = HEAP32[$pAndExpr>>2]|0;
    _sqlite3ExprDelete($1306,$1307);
   }
   $1308 = HEAP32[$v>>2]|0;
   $1309 = HEAP32[$iRetInit>>2]|0;
   $1310 = HEAP32[$v>>2]|0;
   $1311 = (_sqlite3VdbeCurrentAddr($1310)|0);
   _sqlite3VdbeChangeP1($1308,$1309,$1311);
   $1312 = HEAP32[$v>>2]|0;
   $1313 = HEAP32[$pLevel>>2]|0;
   $1314 = ((($1313)) + 12|0);
   $1315 = HEAP32[$1314>>2]|0;
   (_sqlite3VdbeGoto($1312,$1315)|0);
   $1316 = HEAP32[$v>>2]|0;
   $1317 = HEAP32[$iLoopBody>>2]|0;
   _sqlite3VdbeResolveLabel($1316,$1317);
   $1318 = HEAP32[$3>>2]|0;
   $1319 = ((($1318)) + 41|0);
   $1320 = HEAP8[$1319>>0]|0;
   $1321 = $1320&255;
   $1322 = ($1321|0)>(1);
   if ($1322) {
    $1323 = HEAP32[$db>>2]|0;
    $1324 = HEAP32[$pOrTab>>2]|0;
    _sqlite3DbFree($1323,$1324);
   }
   $1325 = HEAP32[$untestedTerms>>2]|0;
   $1326 = ($1325|0)!=(0);
   if (!($1326)) {
    $1327 = HEAP32[$pLevel>>2]|0;
    $1328 = HEAP32[$pTerm>>2]|0;
    _disableTerm($1327,$1328);
   }
  }
 } while(0);
 $1360 = HEAP32[$pWC>>2]|0;
 $1361 = ((($1360)) + 20|0);
 $1362 = HEAP32[$1361>>2]|0;
 HEAP32[$pTerm>>2] = $1362;
 $1363 = HEAP32[$pWC>>2]|0;
 $1364 = ((($1363)) + 12|0);
 $1365 = HEAP32[$1364>>2]|0;
 HEAP32[$j>>2] = $1365;
 while(1) {
  $1366 = HEAP32[$j>>2]|0;
  $1367 = ($1366|0)>(0);
  if (!($1367)) {
   break;
  }
  HEAP32[$skipLikeAddr>>2] = 0;
  $1368 = HEAP32[$pTerm>>2]|0;
  $1369 = ((($1368)) + 20|0);
  $1370 = HEAP16[$1369>>1]|0;
  $1371 = $1370&65535;
  $1372 = $1371 & 6;
  $1373 = ($1372|0)!=(0);
  do {
   if (!($1373)) {
    $1374 = HEAP32[$pTerm>>2]|0;
    $1375 = ((($1374)) + 40|0);
    $1376 = $1375;
    $1377 = $1376;
    $1378 = HEAP32[$1377>>2]|0;
    $1379 = (($1376) + 4)|0;
    $1380 = $1379;
    $1381 = HEAP32[$1380>>2]|0;
    $1382 = HEAP32[$pLevel>>2]|0;
    $1383 = ((($1382)) + 72|0);
    $1384 = $1383;
    $1385 = $1384;
    $1386 = HEAP32[$1385>>2]|0;
    $1387 = (($1384) + 4)|0;
    $1388 = $1387;
    $1389 = HEAP32[$1388>>2]|0;
    $1390 = $1378 & $1386;
    $1391 = $1381 & $1389;
    $1392 = ($1390|0)!=(0);
    $1393 = ($1391|0)!=(0);
    $1394 = $1392 | $1393;
    if ($1394) {
     $1395 = HEAP32[$3>>2]|0;
     $1396 = ((($1395)) + 39|0);
     HEAP8[$1396>>0] = 1;
     break;
    }
    $1397 = HEAP32[$pTerm>>2]|0;
    $1398 = HEAP32[$1397>>2]|0;
    HEAP32[$pE>>2] = $1398;
    $1399 = HEAP32[$pLevel>>2]|0;
    $1400 = HEAP32[$1399>>2]|0;
    $1401 = ($1400|0)!=(0);
    if ($1401) {
     $1402 = HEAP32[$pE>>2]|0;
     $1403 = ((($1402)) + 4|0);
     $1404 = HEAP32[$1403>>2]|0;
     $1405 = $1404 & 1;
     $1406 = ($1405|0)!=(0);
     if (!($1406)) {
      break;
     }
    }
    $1407 = HEAP32[$pTerm>>2]|0;
    $1408 = ((($1407)) + 20|0);
    $1409 = HEAP16[$1408>>1]|0;
    $1410 = $1409&65535;
    $1411 = $1410 & 512;
    $1412 = ($1411|0)!=(0);
    if ($1412) {
     $1413 = HEAP32[$v>>2]|0;
     $1414 = HEAP32[$pLevel>>2]|0;
     $1415 = ((($1414)) + 36|0);
     $1416 = HEAP32[$1415>>2]|0;
     $1417 = (_sqlite3VdbeAddOp1($1413,46,$1416)|0);
     HEAP32[$skipLikeAddr>>2] = $1417;
    }
    $1418 = HEAP32[$pParse>>2]|0;
    $1419 = HEAP32[$pE>>2]|0;
    $1420 = HEAP32[$addrCont>>2]|0;
    _sqlite3ExprIfFalse($1418,$1419,$1420,16);
    $1421 = HEAP32[$skipLikeAddr>>2]|0;
    $1422 = ($1421|0)!=(0);
    if ($1422) {
     $1423 = HEAP32[$v>>2]|0;
     $1424 = HEAP32[$skipLikeAddr>>2]|0;
     _sqlite3VdbeJumpHere($1423,$1424);
    }
    $1425 = HEAP32[$pTerm>>2]|0;
    $1426 = ((($1425)) + 20|0);
    $1427 = HEAP16[$1426>>1]|0;
    $1428 = $1427&65535;
    $1429 = $1428 | 4;
    $1430 = $1429&65535;
    HEAP16[$1426>>1] = $1430;
   }
  } while(0);
  $1431 = HEAP32[$j>>2]|0;
  $1432 = (($1431) + -1)|0;
  HEAP32[$j>>2] = $1432;
  $1433 = HEAP32[$pTerm>>2]|0;
  $1434 = ((($1433)) + 48|0);
  HEAP32[$pTerm>>2] = $1434;
 }
 $1435 = HEAP32[$pWC>>2]|0;
 $1436 = ((($1435)) + 20|0);
 $1437 = HEAP32[$1436>>2]|0;
 HEAP32[$pTerm>>2] = $1437;
 $1438 = HEAP32[$pWC>>2]|0;
 $1439 = ((($1438)) + 12|0);
 $1440 = HEAP32[$1439>>2]|0;
 HEAP32[$j>>2] = $1440;
 while(1) {
  $1441 = HEAP32[$j>>2]|0;
  $1442 = ($1441|0)>(0);
  if (!($1442)) {
   break;
  }
  $1443 = HEAP32[$pTerm>>2]|0;
  $1444 = ((($1443)) + 20|0);
  $1445 = HEAP16[$1444>>1]|0;
  $1446 = $1445&65535;
  $1447 = $1446 & 6;
  $1448 = ($1447|0)!=(0);
  if (!($1448)) {
   $1449 = HEAP32[$pTerm>>2]|0;
   $1450 = ((($1449)) + 18|0);
   $1451 = HEAP16[$1450>>1]|0;
   $1452 = $1451&65535;
   $1453 = $1452 & 130;
   $1454 = ($1453|0)==(0);
   if (!($1454)) {
    $1455 = HEAP32[$pTerm>>2]|0;
    $1456 = ((($1455)) + 18|0);
    $1457 = HEAP16[$1456>>1]|0;
    $1458 = $1457&65535;
    $1459 = $1458 & 2048;
    $1460 = ($1459|0)==(0);
    if (!($1460)) {
     $1461 = HEAP32[$pTerm>>2]|0;
     $1462 = ((($1461)) + 8|0);
     $1463 = HEAP32[$1462>>2]|0;
     $1464 = HEAP32[$iCur>>2]|0;
     $1465 = ($1463|0)!=($1464|0);
     if (!($1465)) {
      $1466 = HEAP32[$pLevel>>2]|0;
      $1467 = HEAP32[$1466>>2]|0;
      $1468 = ($1467|0)!=(0);
      if (!($1468)) {
       $1469 = HEAP32[$pTerm>>2]|0;
       $1470 = HEAP32[$1469>>2]|0;
       HEAP32[$pE7>>2] = $1470;
       $1471 = HEAP32[$pWC>>2]|0;
       $1472 = HEAP32[$iCur>>2]|0;
       $1473 = HEAP32[$pTerm>>2]|0;
       $1474 = ((($1473)) + 12|0);
       $1475 = HEAP32[$1474>>2]|0;
       $1476 = $5;
       $1477 = $1476;
       $1478 = HEAP32[$1477>>2]|0;
       $1479 = (($1476) + 4)|0;
       $1480 = $1479;
       $1481 = HEAP32[$1480>>2]|0;
       $1482 = (_sqlite3WhereFindTerm($1471,$1472,$1475,$1478,$1481,131,0)|0);
       HEAP32[$pAlt>>2] = $1482;
       $1483 = HEAP32[$pAlt>>2]|0;
       $1484 = ($1483|0)==(0|0);
       if (!($1484)) {
        $1485 = HEAP32[$pAlt>>2]|0;
        $1486 = ((($1485)) + 20|0);
        $1487 = HEAP16[$1486>>1]|0;
        $1488 = $1487&65535;
        $1489 = $1488 & 4;
        $1490 = ($1489|0)!=(0);
        if (!($1490)) {
         $1491 = HEAP32[$db>>2]|0;
         $1492 = (_sqlite3DbMallocRaw($1491,48,0)|0);
         HEAP32[$pEAlt>>2] = $1492;
         $1493 = HEAP32[$pEAlt>>2]|0;
         $1494 = ($1493|0)!=(0|0);
         if ($1494) {
          $1495 = HEAP32[$pEAlt>>2]|0;
          $1496 = HEAP32[$pAlt>>2]|0;
          $1497 = HEAP32[$1496>>2]|0;
          dest=$1495; src=$1497; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
          $1498 = HEAP32[$pE7>>2]|0;
          $1499 = ((($1498)) + 12|0);
          $1500 = HEAP32[$1499>>2]|0;
          $1501 = HEAP32[$pEAlt>>2]|0;
          $1502 = ((($1501)) + 12|0);
          HEAP32[$1502>>2] = $1500;
          $1503 = HEAP32[$pParse>>2]|0;
          $1504 = HEAP32[$pEAlt>>2]|0;
          $1505 = HEAP32[$addrCont>>2]|0;
          _sqlite3ExprIfFalse($1503,$1504,$1505,16);
          $1506 = HEAP32[$db>>2]|0;
          $1507 = HEAP32[$pEAlt>>2]|0;
          _sqlite3DbFree($1506,$1507);
         }
        }
       }
      }
     }
    }
   }
  }
  $1508 = HEAP32[$j>>2]|0;
  $1509 = (($1508) + -1)|0;
  HEAP32[$j>>2] = $1509;
  $1510 = HEAP32[$pTerm>>2]|0;
  $1511 = ((($1510)) + 48|0);
  HEAP32[$pTerm>>2] = $1511;
 }
 $1512 = HEAP32[$pLevel>>2]|0;
 $1513 = HEAP32[$1512>>2]|0;
 $1514 = ($1513|0)!=(0);
 L289: do {
  if ($1514) {
   $1515 = HEAP32[$v>>2]|0;
   $1516 = (_sqlite3VdbeCurrentAddr($1515)|0);
   $1517 = HEAP32[$pLevel>>2]|0;
   $1518 = ((($1517)) + 28|0);
   HEAP32[$1518>>2] = $1516;
   $1519 = HEAP32[$v>>2]|0;
   $1520 = HEAP32[$pLevel>>2]|0;
   $1521 = HEAP32[$1520>>2]|0;
   (_sqlite3VdbeAddOp2($1519,22,1,$1521)|0);
   $1522 = HEAP32[$pParse>>2]|0;
   _sqlite3ExprCacheClear($1522);
   $1523 = HEAP32[$pWC>>2]|0;
   $1524 = ((($1523)) + 20|0);
   $1525 = HEAP32[$1524>>2]|0;
   HEAP32[$pTerm>>2] = $1525;
   HEAP32[$j>>2] = 0;
   while(1) {
    $1526 = HEAP32[$j>>2]|0;
    $1527 = HEAP32[$pWC>>2]|0;
    $1528 = ((($1527)) + 12|0);
    $1529 = HEAP32[$1528>>2]|0;
    $1530 = ($1526|0)<($1529|0);
    if (!($1530)) {
     break L289;
    }
    $1531 = HEAP32[$pTerm>>2]|0;
    $1532 = ((($1531)) + 20|0);
    $1533 = HEAP16[$1532>>1]|0;
    $1534 = $1533&65535;
    $1535 = $1534 & 6;
    $1536 = ($1535|0)!=(0);
    if (!($1536)) {
     $1537 = HEAP32[$pTerm>>2]|0;
     $1538 = ((($1537)) + 40|0);
     $1539 = $1538;
     $1540 = $1539;
     $1541 = HEAP32[$1540>>2]|0;
     $1542 = (($1539) + 4)|0;
     $1543 = $1542;
     $1544 = HEAP32[$1543>>2]|0;
     $1545 = HEAP32[$pLevel>>2]|0;
     $1546 = ((($1545)) + 72|0);
     $1547 = $1546;
     $1548 = $1547;
     $1549 = HEAP32[$1548>>2]|0;
     $1550 = (($1547) + 4)|0;
     $1551 = $1550;
     $1552 = HEAP32[$1551>>2]|0;
     $1553 = $1541 & $1549;
     $1554 = $1544 & $1552;
     $1555 = ($1553|0)!=(0);
     $1556 = ($1554|0)!=(0);
     $1557 = $1555 | $1556;
     if (!($1557)) {
      $1558 = HEAP32[$pParse>>2]|0;
      $1559 = HEAP32[$pTerm>>2]|0;
      $1560 = HEAP32[$1559>>2]|0;
      $1561 = HEAP32[$addrCont>>2]|0;
      _sqlite3ExprIfFalse($1558,$1560,$1561,16);
      $1562 = HEAP32[$pTerm>>2]|0;
      $1563 = ((($1562)) + 20|0);
      $1564 = HEAP16[$1563>>1]|0;
      $1565 = $1564&65535;
      $1566 = $1565 | 4;
      $1567 = $1566&65535;
      HEAP16[$1563>>1] = $1567;
     }
    }
    $1568 = HEAP32[$j>>2]|0;
    $1569 = (($1568) + 1)|0;
    HEAP32[$j>>2] = $1569;
    $1570 = HEAP32[$pTerm>>2]|0;
    $1571 = ((($1570)) + 48|0);
    HEAP32[$pTerm>>2] = $1571;
   }
  }
 } while(0);
 $1572 = HEAP32[$pLevel>>2]|0;
 $1573 = ((($1572)) + 72|0);
 $1574 = $1573;
 $1575 = $1574;
 $1576 = HEAP32[$1575>>2]|0;
 $1577 = (($1574) + 4)|0;
 $1578 = $1577;
 $1579 = HEAP32[$1578>>2]|0;
 $1580 = $2;
 $1581 = $1580;
 HEAP32[$1581>>2] = $1576;
 $1582 = (($1580) + 4)|0;
 $1583 = $1582;
 HEAP32[$1583>>2] = $1579;
 $1584 = $2;
 $1585 = $1584;
 $1586 = HEAP32[$1585>>2]|0;
 $1587 = (($1584) + 4)|0;
 $1588 = $1587;
 $1589 = HEAP32[$1588>>2]|0;
 tempRet0 = ($1589);
 STACKTOP = sp;return ($1586|0);
}
function _whereInfoFree($db,$pWInfo) {
 $db = $db|0;
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pLevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $pLevel = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pWInfo;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 41|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 736|0);
  $13 = (($12) + (($10*80)|0)|0);
  HEAP32[$pLevel>>2] = $13;
  $14 = HEAP32[$pLevel>>2]|0;
  $15 = ((($14)) + 64|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$pLevel>>2]|0;
   $19 = ((($18)) + 64|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ((($20)) + 36|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = $22 & 2048;
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$pLevel>>2]|0;
    $27 = ((($26)) + 56|0);
    $28 = ((($27)) + 4|0);
    $29 = HEAP32[$28>>2]|0;
    _sqlite3DbFree($25,$29);
   }
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 328|0);
 _sqlite3WhereClauseClear($33);
 while(1) {
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 16|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 16|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$p>>2] = $40;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 52|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 16|0);
  HEAP32[$45>>2] = $43;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$p>>2]|0;
  _whereLoopDelete($46,$47);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($48,$49);
 STACKTOP = sp;return;
}
function _whereClauseInsert($pWC,$p,$wtFlags) {
 $pWC = $pWC|0;
 $p = $p|0;
 $wtFlags = $wtFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $idx = 0;
 var $pOld = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 28|0;
 $pTerm = sp + 12|0;
 $idx = sp + 8|0;
 $pOld = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $p;
 HEAP16[$3>>1] = $wtFlags;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>=($9|0);
 do {
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pOld>>2] = $13;
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$db>>2] = $17;
   $18 = HEAP32[$db>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21*48)|0;
   $23 = $22<<1;
   $24 = (_sqlite3DbMallocRawNN($18,$23,0)|0);
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 20|0);
   HEAP32[$26>>2] = $24;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 20|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0|0);
   if (!($30)) {
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 20|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$pOld>>2]|0;
    $44 = HEAP32[$1>>2]|0;
    $45 = ((($44)) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = ($46*48)|0;
    _memcpy(($42|0),($43|0),($47|0))|0;
    $48 = HEAP32[$pOld>>2]|0;
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 24|0);
    $51 = ($48|0)!=($50|0);
    if ($51) {
     $52 = HEAP32[$db>>2]|0;
     $53 = HEAP32[$pOld>>2]|0;
     _sqlite3DbFree($52,$53);
    }
    $54 = HEAP32[$db>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 20|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (_sqlite3DbMallocSize($54,$57)|0);
    $59 = (($58>>>0) / 48)&-1;
    $60 = HEAP32[$1>>2]|0;
    $61 = ((($60)) + 16|0);
    HEAP32[$61>>2] = $59;
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 12|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = HEAP32[$1>>2]|0;
    $66 = ((($65)) + 20|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67) + (($64*48)|0)|0);
    $69 = HEAP32[$1>>2]|0;
    $70 = ((($69)) + 16|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = HEAP32[$1>>2]|0;
    $73 = ((($72)) + 12|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = (($71) - ($74))|0;
    $76 = ($75*48)|0;
    _memset(($68|0),0,($76|0))|0;
    break;
   }
   $31 = HEAP16[$3>>1]|0;
   $32 = $31&65535;
   $33 = $32 & 1;
   $34 = ($33|0)!=(0);
   if ($34) {
    $35 = HEAP32[$db>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    _sqlite3ExprDelete($35,$36);
   }
   $37 = HEAP32[$pOld>>2]|0;
   $38 = HEAP32[$1>>2]|0;
   $39 = ((($38)) + 20|0);
   HEAP32[$39>>2] = $37;
   HEAP32[$0>>2] = 0;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 } while(0);
 $77 = HEAP32[$1>>2]|0;
 $78 = ((($77)) + 12|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = (($79) + 1)|0;
 HEAP32[$78>>2] = $80;
 HEAP32[$idx>>2] = $79;
 $81 = HEAP32[$1>>2]|0;
 $82 = ((($81)) + 20|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (($83) + (($79*48)|0)|0);
 HEAP32[$pTerm>>2] = $84;
 $85 = HEAP32[$2>>2]|0;
 $86 = ($85|0)!=(0|0);
 if ($86) {
  $87 = HEAP32[$2>>2]|0;
  $88 = ((($87)) + 4|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = $89 & 262144;
  $91 = ($90|0)!=(0);
  if ($91) {
   $92 = HEAP32[$2>>2]|0;
   $93 = ((($92)) + 28|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = ($94|0)<(0);
   $96 = $95 << 31 >> 31;
   $97 = (_sqlite3LogEst($94,$96)|0);
   $98 = $97 << 16 >> 16;
   $99 = (($98) - 270)|0;
   $100 = $99&65535;
   $101 = HEAP32[$pTerm>>2]|0;
   $102 = ((($101)) + 16|0);
   HEAP16[$102>>1] = $100;
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 if ((label|0) == 12) {
  $103 = HEAP32[$pTerm>>2]|0;
  $104 = ((($103)) + 16|0);
  HEAP16[$104>>1] = 1;
 }
 $105 = HEAP32[$2>>2]|0;
 $106 = (_sqlite3ExprSkipCollate($105)|0);
 $107 = HEAP32[$pTerm>>2]|0;
 HEAP32[$107>>2] = $106;
 $108 = HEAP16[$3>>1]|0;
 $109 = HEAP32[$pTerm>>2]|0;
 $110 = ((($109)) + 20|0);
 HEAP16[$110>>1] = $108;
 $111 = HEAP32[$1>>2]|0;
 $112 = HEAP32[$pTerm>>2]|0;
 $113 = ((($112)) + 24|0);
 HEAP32[$113>>2] = $111;
 $114 = HEAP32[$pTerm>>2]|0;
 $115 = ((($114)) + 4|0);
 HEAP32[$115>>2] = -1;
 $116 = HEAP32[$idx>>2]|0;
 HEAP32[$0>>2] = $116;
 $117 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($117|0);
}
function _exprAnalyze($pSrc,$pWC,$idxTerm) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $idxTerm = $idxTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c5 = 0, $db = 0;
 var $eExtraOp = 0, $eOp2 = 0, $extraRight = 0, $i = 0, $i4 = 0, $iColumn = 0, $iCur = 0, $idxNew = 0, $idxNew1 = 0, $idxNew13 = 0, $idxNew2 = 0, $idxNew6 = 0, $isComplete = 0, $noCase = 0, $op = 0, $opMask = 0, $pC = 0, $pDup = 0, $pExpr = 0, $pLeft = 0;
 var $pLeft2 = 0, $pLeft8 = 0, $pList = 0, $pMaskSet = 0, $pNew = 0, $pNewExpr = 0, $pNewExpr1 = 0, $pNewExpr2 = 0, $pNewExpr9 = 0, $pNewTerm = 0, $pParse = 0, $pRight = 0, $pRight7 = 0, $pStr1 = 0, $pStr2 = 0, $pTerm = 0, $pWInfo = 0, $prereqAll = 0, $prereqColumn = 0, $prereqExpr = 0;
 var $prereqLeft = 0, $wtFlags = 0, $x = 0, $zCollSeqName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 196|0;
 $1 = sp + 192|0;
 $2 = sp + 188|0;
 $pWInfo = sp + 184|0;
 $pTerm = sp + 180|0;
 $pMaskSet = sp + 176|0;
 $pExpr = sp + 172|0;
 $prereqLeft = sp + 40|0;
 $prereqAll = sp + 32|0;
 $extraRight = sp + 24|0;
 $pStr1 = sp + 168|0;
 $isComplete = sp + 164|0;
 $noCase = sp + 160|0;
 $op = sp + 156|0;
 $pParse = sp + 152|0;
 $db = sp + 148|0;
 $eOp2 = sp + 208|0;
 $x = sp + 16|0;
 $iCur = sp + 144|0;
 $iColumn = sp + 140|0;
 $pLeft = sp + 136|0;
 $pRight = sp + 132|0;
 $opMask = sp + 204|0;
 $pNew = sp + 128|0;
 $pDup = sp + 124|0;
 $eExtraOp = sp + 202|0;
 $idxNew = sp + 120|0;
 $pList = sp + 116|0;
 $i = sp + 112|0;
 $pNewExpr = sp + 108|0;
 $idxNew1 = sp + 104|0;
 $pLeft2 = sp + 100|0;
 $pStr2 = sp + 96|0;
 $pNewExpr1 = sp + 92|0;
 $pNewExpr2 = sp + 88|0;
 $idxNew13 = sp + 84|0;
 $idxNew2 = sp + 80|0;
 $zCollSeqName = sp + 76|0;
 $wtFlags = sp + 200|0;
 $i4 = sp + 72|0;
 $c = sp + 207|0;
 $c5 = sp + 206|0;
 $pC = sp + 68|0;
 $idxNew6 = sp + 64|0;
 $pRight7 = sp + 60|0;
 $pLeft8 = sp + 56|0;
 $pNewTerm = sp + 52|0;
 $prereqColumn = sp + 8|0;
 $prereqExpr = sp;
 $pNewExpr9 = sp + 48|0;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $idxTerm;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = $extraRight;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAP32[$pStr1>>2] = 0;
 HEAP32[$isComplete>>2] = 0;
 HEAP32[$noCase>>2] = 0;
 $9 = HEAP32[$pWInfo>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pParse>>2] = $10;
 $11 = HEAP32[$pParse>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 65|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = ($15<<24>>24)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + (($17*48)|0)|0);
 HEAP32[$pTerm>>2] = $21;
 $22 = HEAP32[$pWInfo>>2]|0;
 $23 = ((($22)) + 68|0);
 HEAP32[$pMaskSet>>2] = $23;
 $24 = HEAP32[$pTerm>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pExpr>>2] = $25;
 $26 = HEAP32[$pMaskSet>>2]|0;
 $27 = HEAP32[$pExpr>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3WhereExprUsage($26,$29)|0);
 $31 = tempRet0;
 $32 = $prereqLeft;
 $33 = $32;
 HEAP32[$33>>2] = $30;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = HEAP32[$pExpr>>2]|0;
 $37 = HEAP8[$36>>0]|0;
 $38 = $37&255;
 HEAP32[$op>>2] = $38;
 $39 = HEAP32[$op>>2]|0;
 $40 = ($39|0)==(75);
 do {
  if ($40) {
   $41 = HEAP32[$pExpr>>2]|0;
   $42 = ((($41)) + 4|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = $43 & 2048;
   $45 = ($44|0)!=(0);
   $46 = HEAP32[$pMaskSet>>2]|0;
   $47 = HEAP32[$pExpr>>2]|0;
   $48 = ((($47)) + 20|0);
   if ($45) {
    $49 = HEAP32[$48>>2]|0;
    $50 = (_exprSelectUsage($46,$49)|0);
    $51 = tempRet0;
    $52 = HEAP32[$pTerm>>2]|0;
    $53 = ((($52)) + 32|0);
    $54 = $53;
    $55 = $54;
    HEAP32[$55>>2] = $50;
    $56 = (($54) + 4)|0;
    $57 = $56;
    HEAP32[$57>>2] = $51;
    break;
   } else {
    $58 = HEAP32[$48>>2]|0;
    $59 = (_sqlite3WhereExprListUsage($46,$58)|0);
    $60 = tempRet0;
    $61 = HEAP32[$pTerm>>2]|0;
    $62 = ((($61)) + 32|0);
    $63 = $62;
    $64 = $63;
    HEAP32[$64>>2] = $59;
    $65 = (($63) + 4)|0;
    $66 = $65;
    HEAP32[$66>>2] = $60;
    break;
   }
  } else {
   $67 = HEAP32[$op>>2]|0;
   $68 = ($67|0)==(76);
   if ($68) {
    $69 = HEAP32[$pTerm>>2]|0;
    $70 = ((($69)) + 32|0);
    $71 = $70;
    $72 = $71;
    HEAP32[$72>>2] = 0;
    $73 = (($71) + 4)|0;
    $74 = $73;
    HEAP32[$74>>2] = 0;
    break;
   } else {
    $75 = HEAP32[$pMaskSet>>2]|0;
    $76 = HEAP32[$pExpr>>2]|0;
    $77 = ((($76)) + 16|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = (_sqlite3WhereExprUsage($75,$78)|0);
    $80 = tempRet0;
    $81 = HEAP32[$pTerm>>2]|0;
    $82 = ((($81)) + 32|0);
    $83 = $82;
    $84 = $83;
    HEAP32[$84>>2] = $79;
    $85 = (($83) + 4)|0;
    $86 = $85;
    HEAP32[$86>>2] = $80;
    break;
   }
  }
 } while(0);
 $87 = HEAP32[$pMaskSet>>2]|0;
 $88 = HEAP32[$pExpr>>2]|0;
 $89 = (_sqlite3WhereExprUsage($87,$88)|0);
 $90 = tempRet0;
 $91 = $prereqAll;
 $92 = $91;
 HEAP32[$92>>2] = $89;
 $93 = (($91) + 4)|0;
 $94 = $93;
 HEAP32[$94>>2] = $90;
 $95 = HEAP32[$pExpr>>2]|0;
 $96 = ((($95)) + 4|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = $97 & 1;
 $99 = ($98|0)!=(0);
 if ($99) {
  $100 = HEAP32[$pMaskSet>>2]|0;
  $101 = HEAP32[$pExpr>>2]|0;
  $102 = ((($101)) + 36|0);
  $103 = HEAP16[$102>>1]|0;
  $104 = $103 << 16 >> 16;
  $105 = (_sqlite3WhereGetMask($100,$104)|0);
  $106 = tempRet0;
  $107 = $x;
  $108 = $107;
  HEAP32[$108>>2] = $105;
  $109 = (($107) + 4)|0;
  $110 = $109;
  HEAP32[$110>>2] = $106;
  $111 = $x;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  $117 = $prereqAll;
  $118 = $117;
  $119 = HEAP32[$118>>2]|0;
  $120 = (($117) + 4)|0;
  $121 = $120;
  $122 = HEAP32[$121>>2]|0;
  $123 = $119 | $113;
  $124 = $122 | $116;
  $125 = $prereqAll;
  $126 = $125;
  HEAP32[$126>>2] = $123;
  $127 = (($125) + 4)|0;
  $128 = $127;
  HEAP32[$128>>2] = $124;
  $129 = $x;
  $130 = $129;
  $131 = HEAP32[$130>>2]|0;
  $132 = (($129) + 4)|0;
  $133 = $132;
  $134 = HEAP32[$133>>2]|0;
  $135 = (_i64Subtract(($131|0),($134|0),1,0)|0);
  $136 = tempRet0;
  $137 = $extraRight;
  $138 = $137;
  HEAP32[$138>>2] = $135;
  $139 = (($137) + 4)|0;
  $140 = $139;
  HEAP32[$140>>2] = $136;
 }
 $141 = $prereqAll;
 $142 = $141;
 $143 = HEAP32[$142>>2]|0;
 $144 = (($141) + 4)|0;
 $145 = $144;
 $146 = HEAP32[$145>>2]|0;
 $147 = HEAP32[$pTerm>>2]|0;
 $148 = ((($147)) + 40|0);
 $149 = $148;
 $150 = $149;
 HEAP32[$150>>2] = $143;
 $151 = (($149) + 4)|0;
 $152 = $151;
 HEAP32[$152>>2] = $146;
 $153 = HEAP32[$pTerm>>2]|0;
 $154 = ((($153)) + 8|0);
 HEAP32[$154>>2] = -1;
 $155 = HEAP32[$pTerm>>2]|0;
 $156 = ((($155)) + 4|0);
 HEAP32[$156>>2] = -1;
 $157 = HEAP32[$pTerm>>2]|0;
 $158 = ((($157)) + 18|0);
 HEAP16[$158>>1] = 0;
 $159 = HEAP32[$op>>2]|0;
 $160 = (_allowedOp($159)|0);
 $161 = ($160|0)!=(0);
 $162 = HEAP32[$pExpr>>2]|0;
 L17: do {
  if ($161) {
   $163 = ((($162)) + 12|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = (_sqlite3ExprSkipCollate($164)|0);
   HEAP32[$pLeft>>2] = $165;
   $166 = HEAP32[$pExpr>>2]|0;
   $167 = ((($166)) + 16|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (_sqlite3ExprSkipCollate($168)|0);
   HEAP32[$pRight>>2] = $169;
   $170 = HEAP32[$pTerm>>2]|0;
   $171 = ((($170)) + 32|0);
   $172 = $171;
   $173 = $172;
   $174 = HEAP32[$173>>2]|0;
   $175 = (($172) + 4)|0;
   $176 = $175;
   $177 = HEAP32[$176>>2]|0;
   $178 = $prereqLeft;
   $179 = $178;
   $180 = HEAP32[$179>>2]|0;
   $181 = (($178) + 4)|0;
   $182 = $181;
   $183 = HEAP32[$182>>2]|0;
   $184 = $174 & $180;
   $185 = $177 & $183;
   $186 = ($184|0)==(0);
   $187 = ($185|0)==(0);
   $188 = $186 & $187;
   $189 = $188 ? 8191 : 2048;
   $190 = $189&65535;
   HEAP16[$opMask>>1] = $190;
   $191 = HEAP32[$0>>2]|0;
   $192 = $prereqLeft;
   $193 = $192;
   $194 = HEAP32[$193>>2]|0;
   $195 = (($192) + 4)|0;
   $196 = $195;
   $197 = HEAP32[$196>>2]|0;
   $198 = HEAP32[$pLeft>>2]|0;
   $199 = (_exprMightBeIndexed($191,$194,$197,$198,$iCur,$iColumn)|0);
   $200 = ($199|0)!=(0);
   if ($200) {
    $201 = HEAP32[$iCur>>2]|0;
    $202 = HEAP32[$pTerm>>2]|0;
    $203 = ((($202)) + 8|0);
    HEAP32[$203>>2] = $201;
    $204 = HEAP32[$iColumn>>2]|0;
    $205 = HEAP32[$pTerm>>2]|0;
    $206 = ((($205)) + 12|0);
    HEAP32[$206>>2] = $204;
    $207 = HEAP32[$op>>2]|0;
    $208 = (_operatorMask($207)|0);
    $209 = $208&65535;
    $210 = HEAP16[$opMask>>1]|0;
    $211 = $210&65535;
    $212 = $209 & $211;
    $213 = $212&65535;
    $214 = HEAP32[$pTerm>>2]|0;
    $215 = ((($214)) + 18|0);
    HEAP16[$215>>1] = $213;
   }
   $216 = HEAP32[$op>>2]|0;
   $217 = ($216|0)==(73);
   if ($217) {
    $218 = HEAP32[$pTerm>>2]|0;
    $219 = ((($218)) + 20|0);
    $220 = HEAP16[$219>>1]|0;
    $221 = $220&65535;
    $222 = $221 | 2048;
    $223 = $222&65535;
    HEAP16[$219>>1] = $223;
   }
   $224 = HEAP32[$pRight>>2]|0;
   $225 = ($224|0)!=(0|0);
   if ($225) {
    $226 = HEAP32[$0>>2]|0;
    $227 = HEAP32[$pTerm>>2]|0;
    $228 = ((($227)) + 32|0);
    $229 = $228;
    $230 = $229;
    $231 = HEAP32[$230>>2]|0;
    $232 = (($229) + 4)|0;
    $233 = $232;
    $234 = HEAP32[$233>>2]|0;
    $235 = HEAP32[$pRight>>2]|0;
    $236 = (_exprMightBeIndexed($226,$231,$234,$235,$iCur,$iColumn)|0);
    $237 = ($236|0)!=(0);
    if ($237) {
     HEAP16[$eExtraOp>>1] = 0;
     $238 = HEAP32[$pTerm>>2]|0;
     $239 = ((($238)) + 8|0);
     $240 = HEAP32[$239>>2]|0;
     $241 = ($240|0)>=(0);
     if ($241) {
      $242 = HEAP32[$db>>2]|0;
      $243 = HEAP32[$pExpr>>2]|0;
      $244 = (_sqlite3ExprDup($242,$243,0)|0);
      HEAP32[$pDup>>2] = $244;
      $245 = HEAP32[$db>>2]|0;
      $246 = ((($245)) + 65|0);
      $247 = HEAP8[$246>>0]|0;
      $248 = ($247<<24>>24)!=(0);
      if ($248) {
       $249 = HEAP32[$db>>2]|0;
       $250 = HEAP32[$pDup>>2]|0;
       _sqlite3ExprDelete($249,$250);
       STACKTOP = sp;return;
      }
      $251 = HEAP32[$1>>2]|0;
      $252 = HEAP32[$pDup>>2]|0;
      $253 = (_whereClauseInsert($251,$252,3)|0);
      HEAP32[$idxNew>>2] = $253;
      $254 = HEAP32[$idxNew>>2]|0;
      $255 = ($254|0)==(0);
      if ($255) {
       STACKTOP = sp;return;
      }
      $256 = HEAP32[$idxNew>>2]|0;
      $257 = HEAP32[$1>>2]|0;
      $258 = ((($257)) + 20|0);
      $259 = HEAP32[$258>>2]|0;
      $260 = (($259) + (($256*48)|0)|0);
      HEAP32[$pNew>>2] = $260;
      $261 = HEAP32[$1>>2]|0;
      $262 = HEAP32[$idxNew>>2]|0;
      $263 = HEAP32[$2>>2]|0;
      _markTermAsChild($261,$262,$263);
      $264 = HEAP32[$op>>2]|0;
      $265 = ($264|0)==(73);
      if ($265) {
       $266 = HEAP32[$pNew>>2]|0;
       $267 = ((($266)) + 20|0);
       $268 = HEAP16[$267>>1]|0;
       $269 = $268&65535;
       $270 = $269 | 2048;
       $271 = $270&65535;
       HEAP16[$267>>1] = $271;
      }
      $272 = HEAP32[$2>>2]|0;
      $273 = HEAP32[$1>>2]|0;
      $274 = ((($273)) + 20|0);
      $275 = HEAP32[$274>>2]|0;
      $276 = (($275) + (($272*48)|0)|0);
      HEAP32[$pTerm>>2] = $276;
      $277 = HEAP32[$pTerm>>2]|0;
      $278 = ((($277)) + 20|0);
      $279 = HEAP16[$278>>1]|0;
      $280 = $279&65535;
      $281 = $280 | 8;
      $282 = $281&65535;
      HEAP16[$278>>1] = $282;
      $283 = HEAP32[$pParse>>2]|0;
      $284 = HEAP32[$pDup>>2]|0;
      $285 = (_termIsEquivalence($283,$284)|0);
      $286 = ($285|0)!=(0);
      if ($286) {
       $287 = HEAP32[$pTerm>>2]|0;
       $288 = ((($287)) + 18|0);
       $289 = HEAP16[$288>>1]|0;
       $290 = $289&65535;
       $291 = $290 | 2048;
       $292 = $291&65535;
       HEAP16[$288>>1] = $292;
       HEAP16[$eExtraOp>>1] = 2048;
      }
     } else {
      $293 = HEAP32[$pExpr>>2]|0;
      HEAP32[$pDup>>2] = $293;
      $294 = HEAP32[$pTerm>>2]|0;
      HEAP32[$pNew>>2] = $294;
     }
     $295 = HEAP32[$pParse>>2]|0;
     $296 = HEAP32[$pDup>>2]|0;
     _exprCommute($295,$296);
     $297 = HEAP32[$iCur>>2]|0;
     $298 = HEAP32[$pNew>>2]|0;
     $299 = ((($298)) + 8|0);
     HEAP32[$299>>2] = $297;
     $300 = HEAP32[$iColumn>>2]|0;
     $301 = HEAP32[$pNew>>2]|0;
     $302 = ((($301)) + 12|0);
     HEAP32[$302>>2] = $300;
     $303 = $prereqLeft;
     $304 = $303;
     $305 = HEAP32[$304>>2]|0;
     $306 = (($303) + 4)|0;
     $307 = $306;
     $308 = HEAP32[$307>>2]|0;
     $309 = $extraRight;
     $310 = $309;
     $311 = HEAP32[$310>>2]|0;
     $312 = (($309) + 4)|0;
     $313 = $312;
     $314 = HEAP32[$313>>2]|0;
     $315 = $305 | $311;
     $316 = $308 | $314;
     $317 = HEAP32[$pNew>>2]|0;
     $318 = ((($317)) + 32|0);
     $319 = $318;
     $320 = $319;
     HEAP32[$320>>2] = $315;
     $321 = (($319) + 4)|0;
     $322 = $321;
     HEAP32[$322>>2] = $316;
     $323 = $prereqAll;
     $324 = $323;
     $325 = HEAP32[$324>>2]|0;
     $326 = (($323) + 4)|0;
     $327 = $326;
     $328 = HEAP32[$327>>2]|0;
     $329 = HEAP32[$pNew>>2]|0;
     $330 = ((($329)) + 40|0);
     $331 = $330;
     $332 = $331;
     HEAP32[$332>>2] = $325;
     $333 = (($331) + 4)|0;
     $334 = $333;
     HEAP32[$334>>2] = $328;
     $335 = HEAP32[$pDup>>2]|0;
     $336 = HEAP8[$335>>0]|0;
     $337 = $336&255;
     $338 = (_operatorMask($337)|0);
     $339 = $338&65535;
     $340 = HEAP16[$eExtraOp>>1]|0;
     $341 = $340&65535;
     $342 = (($339) + ($341))|0;
     $343 = HEAP16[$opMask>>1]|0;
     $344 = $343&65535;
     $345 = $342 & $344;
     $346 = $345&65535;
     $347 = HEAP32[$pNew>>2]|0;
     $348 = ((($347)) + 18|0);
     HEAP16[$348>>1] = $346;
    }
   }
  } else {
   $349 = HEAP8[$162>>0]|0;
   $350 = $349&255;
   $351 = ($350|0)==(74);
   if ($351) {
    $352 = HEAP32[$1>>2]|0;
    $353 = ((($352)) + 8|0);
    $354 = HEAP8[$353>>0]|0;
    $355 = $354&255;
    $356 = ($355|0)==(72);
    if ($356) {
     $357 = HEAP32[$pExpr>>2]|0;
     $358 = ((($357)) + 20|0);
     $359 = HEAP32[$358>>2]|0;
     HEAP32[$pList>>2] = $359;
     HEAP32[$i>>2] = 0;
     while(1) {
      $360 = HEAP32[$i>>2]|0;
      $361 = ($360|0)<(2);
      if (!($361)) {
       break L17;
      }
      $362 = HEAP32[$pParse>>2]|0;
      $363 = HEAP32[$i>>2]|0;
      $364 = (32556 + ($363)|0);
      $365 = HEAP8[$364>>0]|0;
      $366 = $365&255;
      $367 = HEAP32[$db>>2]|0;
      $368 = HEAP32[$pExpr>>2]|0;
      $369 = ((($368)) + 12|0);
      $370 = HEAP32[$369>>2]|0;
      $371 = (_sqlite3ExprDup($367,$370,0)|0);
      $372 = HEAP32[$db>>2]|0;
      $373 = HEAP32[$i>>2]|0;
      $374 = HEAP32[$pList>>2]|0;
      $375 = ((($374)) + 4|0);
      $376 = HEAP32[$375>>2]|0;
      $377 = (($376) + (($373*20)|0)|0);
      $378 = HEAP32[$377>>2]|0;
      $379 = (_sqlite3ExprDup($372,$378,0)|0);
      $380 = (_sqlite3PExpr($362,$366,$371,$379,0)|0);
      HEAP32[$pNewExpr>>2] = $380;
      $381 = HEAP32[$pNewExpr>>2]|0;
      $382 = HEAP32[$pExpr>>2]|0;
      _transferJoinMarkings($381,$382);
      $383 = HEAP32[$1>>2]|0;
      $384 = HEAP32[$pNewExpr>>2]|0;
      $385 = (_whereClauseInsert($383,$384,3)|0);
      HEAP32[$idxNew1>>2] = $385;
      $386 = HEAP32[$0>>2]|0;
      $387 = HEAP32[$1>>2]|0;
      $388 = HEAP32[$idxNew1>>2]|0;
      _exprAnalyze($386,$387,$388);
      $389 = HEAP32[$2>>2]|0;
      $390 = HEAP32[$1>>2]|0;
      $391 = ((($390)) + 20|0);
      $392 = HEAP32[$391>>2]|0;
      $393 = (($392) + (($389*48)|0)|0);
      HEAP32[$pTerm>>2] = $393;
      $394 = HEAP32[$1>>2]|0;
      $395 = HEAP32[$idxNew1>>2]|0;
      $396 = HEAP32[$2>>2]|0;
      _markTermAsChild($394,$395,$396);
      $397 = HEAP32[$i>>2]|0;
      $398 = (($397) + 1)|0;
      HEAP32[$i>>2] = $398;
     }
    }
   }
   $399 = HEAP32[$pExpr>>2]|0;
   $400 = HEAP8[$399>>0]|0;
   $401 = $400&255;
   $402 = ($401|0)==(71);
   if ($402) {
    $403 = HEAP32[$0>>2]|0;
    $404 = HEAP32[$1>>2]|0;
    $405 = HEAP32[$2>>2]|0;
    _exprAnalyzeOrTerm($403,$404,$405);
    $406 = HEAP32[$2>>2]|0;
    $407 = HEAP32[$1>>2]|0;
    $408 = ((($407)) + 20|0);
    $409 = HEAP32[$408>>2]|0;
    $410 = (($409) + (($406*48)|0)|0);
    HEAP32[$pTerm>>2] = $410;
   }
  }
 } while(0);
 $411 = HEAP32[$1>>2]|0;
 $412 = ((($411)) + 8|0);
 $413 = HEAP8[$412>>0]|0;
 $414 = $413&255;
 $415 = ($414|0)==(72);
 if ($415) {
  $416 = HEAP32[$pParse>>2]|0;
  $417 = HEAP32[$pExpr>>2]|0;
  $418 = (_isLikeOrGlob($416,$417,$pStr1,$isComplete,$noCase)|0);
  $419 = ($418|0)!=(0);
  if ($419) {
   HEAP16[$wtFlags>>1] = 259;
   $420 = HEAP32[$pExpr>>2]|0;
   $421 = ((($420)) + 20|0);
   $422 = HEAP32[$421>>2]|0;
   $423 = ((($422)) + 4|0);
   $424 = HEAP32[$423>>2]|0;
   $425 = ((($424)) + 20|0);
   $426 = HEAP32[$425>>2]|0;
   HEAP32[$pLeft2>>2] = $426;
   $427 = HEAP32[$db>>2]|0;
   $428 = HEAP32[$pStr1>>2]|0;
   $429 = (_sqlite3ExprDup($427,$428,0)|0);
   HEAP32[$pStr2>>2] = $429;
   $430 = HEAP32[$noCase>>2]|0;
   $431 = ($430|0)!=(0);
   L55: do {
    if ($431) {
     $432 = HEAP32[$pParse>>2]|0;
     $433 = HEAP32[$432>>2]|0;
     $434 = ((($433)) + 65|0);
     $435 = HEAP8[$434>>0]|0;
     $436 = ($435<<24>>24)!=(0);
     if (!($436)) {
      $437 = HEAP32[$pTerm>>2]|0;
      $438 = ((($437)) + 20|0);
      $439 = HEAP16[$438>>1]|0;
      $440 = $439&65535;
      $441 = $440 | 1024;
      $442 = $441&65535;
      HEAP16[$438>>1] = $442;
      HEAP32[$i4>>2] = 0;
      while(1) {
       $443 = HEAP32[$i4>>2]|0;
       $444 = HEAP32[$pStr1>>2]|0;
       $445 = ((($444)) + 8|0);
       $446 = HEAP32[$445>>2]|0;
       $447 = (($446) + ($443)|0);
       $448 = HEAP8[$447>>0]|0;
       HEAP8[$c>>0] = $448;
       $449 = $448 << 24 >> 24;
       $450 = ($449|0)!=(0);
       if (!($450)) {
        break L55;
       }
       $451 = HEAP8[$c>>0]|0;
       $452 = $451 << 24 >> 24;
       $453 = HEAP8[$c>>0]|0;
       $454 = $453&255;
       $455 = (17533 + ($454)|0);
       $456 = HEAP8[$455>>0]|0;
       $457 = $456&255;
       $458 = $457 & 32;
       $459 = $458 ^ -1;
       $460 = $452 & $459;
       $461 = $460&255;
       $462 = HEAP32[$i4>>2]|0;
       $463 = HEAP32[$pStr1>>2]|0;
       $464 = ((($463)) + 8|0);
       $465 = HEAP32[$464>>2]|0;
       $466 = (($465) + ($462)|0);
       HEAP8[$466>>0] = $461;
       $467 = HEAP8[$c>>0]|0;
       $468 = $467&255;
       $469 = (16974 + ($468)|0);
       $470 = HEAP8[$469>>0]|0;
       $471 = HEAP32[$i4>>2]|0;
       $472 = HEAP32[$pStr2>>2]|0;
       $473 = ((($472)) + 8|0);
       $474 = HEAP32[$473>>2]|0;
       $475 = (($474) + ($471)|0);
       HEAP8[$475>>0] = $470;
       $476 = HEAP32[$i4>>2]|0;
       $477 = (($476) + 1)|0;
       HEAP32[$i4>>2] = $477;
      }
     }
    }
   } while(0);
   $478 = HEAP32[$db>>2]|0;
   $479 = ((($478)) + 65|0);
   $480 = HEAP8[$479>>0]|0;
   $481 = ($480<<24>>24)!=(0);
   if (!($481)) {
    $482 = HEAP32[$pStr2>>2]|0;
    $483 = ((($482)) + 8|0);
    $484 = HEAP32[$483>>2]|0;
    $485 = (_sqlite3Strlen30($484)|0);
    $486 = (($485) - 1)|0;
    $487 = HEAP32[$pStr2>>2]|0;
    $488 = ((($487)) + 8|0);
    $489 = HEAP32[$488>>2]|0;
    $490 = (($489) + ($486)|0);
    HEAP32[$pC>>2] = $490;
    $491 = HEAP32[$pC>>2]|0;
    $492 = HEAP8[$491>>0]|0;
    HEAP8[$c5>>0] = $492;
    $493 = HEAP32[$noCase>>2]|0;
    $494 = ($493|0)!=(0);
    if ($494) {
     $495 = HEAP8[$c5>>0]|0;
     $496 = $495&255;
     $497 = ($496|0)==(64);
     if ($497) {
      HEAP32[$isComplete>>2] = 0;
     }
     $498 = HEAP8[$c5>>0]|0;
     $499 = $498&255;
     $500 = (16974 + ($499)|0);
     $501 = HEAP8[$500>>0]|0;
     HEAP8[$c5>>0] = $501;
    }
    $502 = HEAP8[$c5>>0]|0;
    $503 = $502&255;
    $504 = (($503) + 1)|0;
    $505 = $504&255;
    $506 = HEAP32[$pC>>2]|0;
    HEAP8[$506>>0] = $505;
   }
   $507 = HEAP32[$noCase>>2]|0;
   $508 = ($507|0)!=(0);
   $509 = $508 ? 32558 : 20716;
   HEAP32[$zCollSeqName>>2] = $509;
   $510 = HEAP32[$db>>2]|0;
   $511 = HEAP32[$pLeft2>>2]|0;
   $512 = (_sqlite3ExprDup($510,$511,0)|0);
   HEAP32[$pNewExpr1>>2] = $512;
   $513 = HEAP32[$pParse>>2]|0;
   $514 = HEAP32[$pParse>>2]|0;
   $515 = HEAP32[$pNewExpr1>>2]|0;
   $516 = HEAP32[$zCollSeqName>>2]|0;
   $517 = (_sqlite3ExprAddCollateString($514,$515,$516)|0);
   $518 = HEAP32[$pStr1>>2]|0;
   $519 = (_sqlite3PExpr($513,83,$517,$518,0)|0);
   HEAP32[$pNewExpr1>>2] = $519;
   $520 = HEAP32[$pNewExpr1>>2]|0;
   $521 = HEAP32[$pExpr>>2]|0;
   _transferJoinMarkings($520,$521);
   $522 = HEAP32[$1>>2]|0;
   $523 = HEAP32[$pNewExpr1>>2]|0;
   $524 = (_whereClauseInsert($522,$523,259)|0);
   HEAP32[$idxNew13>>2] = $524;
   $525 = HEAP32[$0>>2]|0;
   $526 = HEAP32[$1>>2]|0;
   $527 = HEAP32[$idxNew13>>2]|0;
   _exprAnalyze($525,$526,$527);
   $528 = HEAP32[$db>>2]|0;
   $529 = HEAP32[$pLeft2>>2]|0;
   $530 = (_sqlite3ExprDup($528,$529,0)|0);
   HEAP32[$pNewExpr2>>2] = $530;
   $531 = HEAP32[$pParse>>2]|0;
   $532 = HEAP32[$pParse>>2]|0;
   $533 = HEAP32[$pNewExpr2>>2]|0;
   $534 = HEAP32[$zCollSeqName>>2]|0;
   $535 = (_sqlite3ExprAddCollateString($532,$533,$534)|0);
   $536 = HEAP32[$pStr2>>2]|0;
   $537 = (_sqlite3PExpr($531,82,$535,$536,0)|0);
   HEAP32[$pNewExpr2>>2] = $537;
   $538 = HEAP32[$pNewExpr2>>2]|0;
   $539 = HEAP32[$pExpr>>2]|0;
   _transferJoinMarkings($538,$539);
   $540 = HEAP32[$1>>2]|0;
   $541 = HEAP32[$pNewExpr2>>2]|0;
   $542 = (_whereClauseInsert($540,$541,259)|0);
   HEAP32[$idxNew2>>2] = $542;
   $543 = HEAP32[$0>>2]|0;
   $544 = HEAP32[$1>>2]|0;
   $545 = HEAP32[$idxNew2>>2]|0;
   _exprAnalyze($543,$544,$545);
   $546 = HEAP32[$2>>2]|0;
   $547 = HEAP32[$1>>2]|0;
   $548 = ((($547)) + 20|0);
   $549 = HEAP32[$548>>2]|0;
   $550 = (($549) + (($546*48)|0)|0);
   HEAP32[$pTerm>>2] = $550;
   $551 = HEAP32[$isComplete>>2]|0;
   $552 = ($551|0)!=(0);
   if ($552) {
    $553 = HEAP32[$1>>2]|0;
    $554 = HEAP32[$idxNew13>>2]|0;
    $555 = HEAP32[$2>>2]|0;
    _markTermAsChild($553,$554,$555);
    $556 = HEAP32[$1>>2]|0;
    $557 = HEAP32[$idxNew2>>2]|0;
    $558 = HEAP32[$2>>2]|0;
    _markTermAsChild($556,$557,$558);
   }
  }
 }
 $559 = HEAP32[$pExpr>>2]|0;
 $560 = (_isMatchOfColumn($559,$eOp2)|0);
 $561 = ($560|0)!=(0);
 if ($561) {
  $562 = HEAP32[$pExpr>>2]|0;
  $563 = ((($562)) + 20|0);
  $564 = HEAP32[$563>>2]|0;
  $565 = ((($564)) + 4|0);
  $566 = HEAP32[$565>>2]|0;
  $567 = HEAP32[$566>>2]|0;
  HEAP32[$pRight7>>2] = $567;
  $568 = HEAP32[$pExpr>>2]|0;
  $569 = ((($568)) + 20|0);
  $570 = HEAP32[$569>>2]|0;
  $571 = ((($570)) + 4|0);
  $572 = HEAP32[$571>>2]|0;
  $573 = ((($572)) + 20|0);
  $574 = HEAP32[$573>>2]|0;
  HEAP32[$pLeft8>>2] = $574;
  $575 = HEAP32[$pMaskSet>>2]|0;
  $576 = HEAP32[$pRight7>>2]|0;
  $577 = (_sqlite3WhereExprUsage($575,$576)|0);
  $578 = tempRet0;
  $579 = $prereqExpr;
  $580 = $579;
  HEAP32[$580>>2] = $577;
  $581 = (($579) + 4)|0;
  $582 = $581;
  HEAP32[$582>>2] = $578;
  $583 = HEAP32[$pMaskSet>>2]|0;
  $584 = HEAP32[$pLeft8>>2]|0;
  $585 = (_sqlite3WhereExprUsage($583,$584)|0);
  $586 = tempRet0;
  $587 = $prereqColumn;
  $588 = $587;
  HEAP32[$588>>2] = $585;
  $589 = (($587) + 4)|0;
  $590 = $589;
  HEAP32[$590>>2] = $586;
  $591 = $prereqExpr;
  $592 = $591;
  $593 = HEAP32[$592>>2]|0;
  $594 = (($591) + 4)|0;
  $595 = $594;
  $596 = HEAP32[$595>>2]|0;
  $597 = $prereqColumn;
  $598 = $597;
  $599 = HEAP32[$598>>2]|0;
  $600 = (($597) + 4)|0;
  $601 = $600;
  $602 = HEAP32[$601>>2]|0;
  $603 = $593 & $599;
  $604 = $596 & $602;
  $605 = ($603|0)==(0);
  $606 = ($604|0)==(0);
  $607 = $605 & $606;
  if ($607) {
   $608 = HEAP32[$pParse>>2]|0;
   $609 = HEAP32[$db>>2]|0;
   $610 = HEAP32[$pRight7>>2]|0;
   $611 = (_sqlite3ExprDup($609,$610,0)|0);
   $612 = (_sqlite3PExpr($608,51,0,$611,0)|0);
   HEAP32[$pNewExpr9>>2] = $612;
   $613 = HEAP32[$1>>2]|0;
   $614 = HEAP32[$pNewExpr9>>2]|0;
   $615 = (_whereClauseInsert($613,$614,3)|0);
   HEAP32[$idxNew6>>2] = $615;
   $616 = HEAP32[$idxNew6>>2]|0;
   $617 = HEAP32[$1>>2]|0;
   $618 = ((($617)) + 20|0);
   $619 = HEAP32[$618>>2]|0;
   $620 = (($619) + (($616*48)|0)|0);
   HEAP32[$pNewTerm>>2] = $620;
   $621 = $prereqExpr;
   $622 = $621;
   $623 = HEAP32[$622>>2]|0;
   $624 = (($621) + 4)|0;
   $625 = $624;
   $626 = HEAP32[$625>>2]|0;
   $627 = HEAP32[$pNewTerm>>2]|0;
   $628 = ((($627)) + 32|0);
   $629 = $628;
   $630 = $629;
   HEAP32[$630>>2] = $623;
   $631 = (($629) + 4)|0;
   $632 = $631;
   HEAP32[$632>>2] = $626;
   $633 = HEAP32[$pLeft8>>2]|0;
   $634 = ((($633)) + 28|0);
   $635 = HEAP32[$634>>2]|0;
   $636 = HEAP32[$pNewTerm>>2]|0;
   $637 = ((($636)) + 8|0);
   HEAP32[$637>>2] = $635;
   $638 = HEAP32[$pLeft8>>2]|0;
   $639 = ((($638)) + 32|0);
   $640 = HEAP16[$639>>1]|0;
   $641 = $640 << 16 >> 16;
   $642 = HEAP32[$pNewTerm>>2]|0;
   $643 = ((($642)) + 12|0);
   HEAP32[$643>>2] = $641;
   $644 = HEAP32[$pNewTerm>>2]|0;
   $645 = ((($644)) + 18|0);
   HEAP16[$645>>1] = 64;
   $646 = HEAP8[$eOp2>>0]|0;
   $647 = HEAP32[$pNewTerm>>2]|0;
   $648 = ((($647)) + 23|0);
   HEAP8[$648>>0] = $646;
   $649 = HEAP32[$1>>2]|0;
   $650 = HEAP32[$idxNew6>>2]|0;
   $651 = HEAP32[$2>>2]|0;
   _markTermAsChild($649,$650,$651);
   $652 = HEAP32[$2>>2]|0;
   $653 = HEAP32[$1>>2]|0;
   $654 = ((($653)) + 20|0);
   $655 = HEAP32[$654>>2]|0;
   $656 = (($655) + (($652*48)|0)|0);
   HEAP32[$pTerm>>2] = $656;
   $657 = HEAP32[$pTerm>>2]|0;
   $658 = ((($657)) + 20|0);
   $659 = HEAP16[$658>>1]|0;
   $660 = $659&65535;
   $661 = $660 | 8;
   $662 = $661&65535;
   HEAP16[$658>>1] = $662;
   $663 = HEAP32[$pTerm>>2]|0;
   $664 = ((($663)) + 40|0);
   $665 = $664;
   $666 = $665;
   $667 = HEAP32[$666>>2]|0;
   $668 = (($665) + 4)|0;
   $669 = $668;
   $670 = HEAP32[$669>>2]|0;
   $671 = HEAP32[$pNewTerm>>2]|0;
   $672 = ((($671)) + 40|0);
   $673 = $672;
   $674 = $673;
   HEAP32[$674>>2] = $667;
   $675 = (($673) + 4)|0;
   $676 = $675;
   HEAP32[$676>>2] = $670;
  }
 }
 $677 = $extraRight;
 $678 = $677;
 $679 = HEAP32[$678>>2]|0;
 $680 = (($677) + 4)|0;
 $681 = $680;
 $682 = HEAP32[$681>>2]|0;
 $683 = HEAP32[$pTerm>>2]|0;
 $684 = ((($683)) + 32|0);
 $685 = $684;
 $686 = $685;
 $687 = HEAP32[$686>>2]|0;
 $688 = (($685) + 4)|0;
 $689 = $688;
 $690 = HEAP32[$689>>2]|0;
 $691 = $687 | $679;
 $692 = $690 | $682;
 $693 = $684;
 $694 = $693;
 HEAP32[$694>>2] = $691;
 $695 = (($693) + 4)|0;
 $696 = $695;
 HEAP32[$696>>2] = $692;
 STACKTOP = sp;return;
}
function _sqlite3WhereExprUsage($pMaskSet,$p) {
 $pMaskSet = $pMaskSet|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $mask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $mask = sp;
 HEAP32[$1>>2] = $pMaskSet;
 HEAP32[$2>>2] = $p;
 $3 = $mask;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = $0;
  $10 = $9;
  HEAP32[$10>>2] = 0;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = 0;
  $111 = $0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  tempRet0 = ($116);
  STACKTOP = sp;return ($113|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(152);
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 if ($16) {
  $19 = ((($18)) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3WhereGetMask($17,$20)|0);
  $22 = tempRet0;
  $23 = $mask;
  $24 = $23;
  HEAP32[$24>>2] = $21;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $mask;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = $0;
  $34 = $33;
  HEAP32[$34>>2] = $29;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = $32;
  $111 = $0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (($111) + 4)|0;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  tempRet0 = ($116);
  STACKTOP = sp;return ($113|0);
 }
 $37 = ((($18)) + 16|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (_sqlite3WhereExprUsage($17,$38)|0);
 $40 = tempRet0;
 $41 = $mask;
 $42 = $41;
 HEAP32[$42>>2] = $39;
 $43 = (($41) + 4)|0;
 $44 = $43;
 HEAP32[$44>>2] = $40;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (_sqlite3WhereExprUsage($45,$48)|0);
 $50 = tempRet0;
 $51 = $mask;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = $53 | $49;
 $58 = $56 | $50;
 $59 = $mask;
 $60 = $59;
 HEAP32[$60>>2] = $57;
 $61 = (($59) + 4)|0;
 $62 = $61;
 HEAP32[$62>>2] = $58;
 $63 = HEAP32[$2>>2]|0;
 $64 = ((($63)) + 4|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = $65 & 2048;
 $67 = ($66|0)!=(0);
 $68 = HEAP32[$1>>2]|0;
 $69 = HEAP32[$2>>2]|0;
 $70 = ((($69)) + 20|0);
 if ($67) {
  $71 = HEAP32[$70>>2]|0;
  $72 = (_exprSelectUsage($68,$71)|0);
  $73 = tempRet0;
  $74 = $mask;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($74) + 4)|0;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = $76 | $72;
  $81 = $79 | $73;
  $82 = $mask;
  $83 = $82;
  HEAP32[$83>>2] = $80;
  $84 = (($82) + 4)|0;
  $85 = $84;
  HEAP32[$85>>2] = $81;
 } else {
  $86 = HEAP32[$70>>2]|0;
  $87 = (_sqlite3WhereExprListUsage($68,$86)|0);
  $88 = tempRet0;
  $89 = $mask;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = $91 | $87;
  $96 = $94 | $88;
  $97 = $mask;
  $98 = $97;
  HEAP32[$98>>2] = $95;
  $99 = (($97) + 4)|0;
  $100 = $99;
  HEAP32[$100>>2] = $96;
 }
 $101 = $mask;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = (($101) + 4)|0;
 $105 = $104;
 $106 = HEAP32[$105>>2]|0;
 $107 = $0;
 $108 = $107;
 HEAP32[$108>>2] = $103;
 $109 = (($107) + 4)|0;
 $110 = $109;
 HEAP32[$110>>2] = $106;
 $111 = $0;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 tempRet0 = ($116);
 STACKTOP = sp;return ($113|0);
}
function _exprSelectUsage($pMaskSet,$pS) {
 $pMaskSet = $pMaskSet|0;
 $pS = $pS|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $mask = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $mask = sp;
 $pSrc = sp + 12|0;
 $i = sp + 8|0;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $pS;
 $2 = $mask;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 while(1) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 32|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pSrc>>2] = $10;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3WhereExprListUsage($11,$13)|0);
  $15 = tempRet0;
  $16 = $mask;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = $18 | $14;
  $23 = $21 | $15;
  $24 = $mask;
  $25 = $24;
  HEAP32[$25>>2] = $22;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  $28 = HEAP32[$0>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 40|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3WhereExprListUsage($28,$31)|0);
  $33 = tempRet0;
  $34 = $mask;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = $36 | $32;
  $41 = $39 | $33;
  $42 = $mask;
  $43 = $42;
  HEAP32[$43>>2] = $40;
  $44 = (($42) + 4)|0;
  $45 = $44;
  HEAP32[$45>>2] = $41;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 48|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_sqlite3WhereExprListUsage($46,$49)|0);
  $51 = tempRet0;
  $52 = $mask;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = $54 | $50;
  $59 = $57 | $51;
  $60 = $mask;
  $61 = $60;
  HEAP32[$61>>2] = $58;
  $62 = (($60) + 4)|0;
  $63 = $62;
  HEAP32[$63>>2] = $59;
  $64 = HEAP32[$0>>2]|0;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 36|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (_sqlite3WhereExprUsage($64,$67)|0);
  $69 = tempRet0;
  $70 = $mask;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $72 | $68;
  $77 = $75 | $69;
  $78 = $mask;
  $79 = $78;
  HEAP32[$79>>2] = $76;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = HEAP32[$0>>2]|0;
  $83 = HEAP32[$1>>2]|0;
  $84 = ((($83)) + 44|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (_sqlite3WhereExprUsage($82,$85)|0);
  $87 = tempRet0;
  $88 = $mask;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = $90 | $86;
  $95 = $93 | $87;
  $96 = $mask;
  $97 = $96;
  HEAP32[$97>>2] = $94;
  $98 = (($96) + 4)|0;
  $99 = $98;
  HEAP32[$99>>2] = $95;
  $100 = HEAP32[$pSrc>>2]|0;
  $101 = ($100|0)!=(0|0);
  L4: do {
   if ($101) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $102 = HEAP32[$i>>2]|0;
     $103 = HEAP32[$pSrc>>2]|0;
     $104 = HEAP32[$103>>2]|0;
     $105 = ($102|0)<($104|0);
     if (!($105)) {
      break L4;
     }
     $106 = HEAP32[$0>>2]|0;
     $107 = HEAP32[$i>>2]|0;
     $108 = HEAP32[$pSrc>>2]|0;
     $109 = ((($108)) + 8|0);
     $110 = (($109) + (($107*72)|0)|0);
     $111 = ((($110)) + 20|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (_exprSelectUsage($106,$112)|0);
     $114 = tempRet0;
     $115 = $mask;
     $116 = $115;
     $117 = HEAP32[$116>>2]|0;
     $118 = (($115) + 4)|0;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = $117 | $113;
     $122 = $120 | $114;
     $123 = $mask;
     $124 = $123;
     HEAP32[$124>>2] = $121;
     $125 = (($123) + 4)|0;
     $126 = $125;
     HEAP32[$126>>2] = $122;
     $127 = HEAP32[$0>>2]|0;
     $128 = HEAP32[$i>>2]|0;
     $129 = HEAP32[$pSrc>>2]|0;
     $130 = ((($129)) + 8|0);
     $131 = (($130) + (($128*72)|0)|0);
     $132 = ((($131)) + 48|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = (_sqlite3WhereExprUsage($127,$133)|0);
     $135 = tempRet0;
     $136 = $mask;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = $138 | $134;
     $143 = $141 | $135;
     $144 = $mask;
     $145 = $144;
     HEAP32[$145>>2] = $142;
     $146 = (($144) + 4)|0;
     $147 = $146;
     HEAP32[$147>>2] = $143;
     $148 = HEAP32[$i>>2]|0;
     $149 = (($148) + 1)|0;
     HEAP32[$i>>2] = $149;
    }
   }
  } while(0);
  $150 = HEAP32[$1>>2]|0;
  $151 = ((($150)) + 52|0);
  $152 = HEAP32[$151>>2]|0;
  HEAP32[$1>>2] = $152;
 }
 $153 = $mask;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = (($153) + 4)|0;
 $157 = $156;
 $158 = HEAP32[$157>>2]|0;
 tempRet0 = ($158);
 STACKTOP = sp;return ($155|0);
}
function _sqlite3WhereGetMask($pMaskSet,$iCursor) {
 $pMaskSet = $pMaskSet|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 HEAP32[$1>>2] = $pMaskSet;
 HEAP32[$2>>2] = $iCursor;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ($11|0)==($12|0);
  $14 = HEAP32[$i>>2]|0;
  if ($13) {
   label = 4;
   break;
  }
  $21 = (($14) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 if ((label|0) == 4) {
  $15 = (_bitshift64Shl(1,0,($14|0))|0);
  $16 = tempRet0;
  $17 = $0;
  $18 = $17;
  HEAP32[$18>>2] = $15;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $26 = $0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  tempRet0 = ($31);
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 6) {
  $22 = $0;
  $23 = $22;
  HEAP32[$23>>2] = 0;
  $24 = (($22) + 4)|0;
  $25 = $24;
  HEAP32[$25>>2] = 0;
  $26 = $0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  tempRet0 = ($31);
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _allowedOp($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(75);
 if ($2) {
  $12 = 1;
 } else {
  $3 = HEAP32[$0>>2]|0;
  $4 = ($3|0)>=(79);
  $5 = HEAP32[$0>>2]|0;
  $6 = ($5|0)<=(83);
  $or$cond = $4 & $6;
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)==(76);
  $or$cond3 = $or$cond | $8;
  if ($or$cond3) {
   $12 = 1;
  } else {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)==(73);
   $12 = $10;
  }
 }
 $11 = $12&1;
 STACKTOP = sp;return ($11|0);
}
function _exprMightBeIndexed($pFrom,$0,$1,$pExpr,$piCur,$piColumn) {
 $pFrom = $pFrom|0;
 $0 = $0|0;
 $1 = $1|0;
 $pExpr = $pExpr|0;
 $piCur = $piCur|0;
 $piColumn = $piColumn|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCur = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $pIdx = sp + 16|0;
 $i = sp + 12|0;
 $iCur = sp + 8|0;
 HEAP32[$3>>2] = $pFrom;
 $8 = $4;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$5>>2] = $pExpr;
 HEAP32[$6>>2] = $piCur;
 HEAP32[$7>>2] = $piColumn;
 $12 = HEAP32[$5>>2]|0;
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(152);
 if ($15) {
  $16 = HEAP32[$5>>2]|0;
  $17 = ((($16)) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$6>>2]|0;
  HEAP32[$19>>2] = $18;
  $20 = HEAP32[$5>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22 << 16 >> 16;
  $24 = HEAP32[$7>>2]|0;
  HEAP32[$24>>2] = $23;
  HEAP32[$2>>2] = 1;
  $131 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $25 = $4;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($27|0)==(0);
 $32 = ($30|0)==(0);
 $33 = $31 & $32;
 if ($33) {
  HEAP32[$2>>2] = 0;
  $131 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 $34 = $4;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = $4;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + 4)|0;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 $46 = (_i64Subtract(($42|0),($45|0),1,0)|0);
 $47 = tempRet0;
 $48 = $36 & $46;
 $49 = $39 & $47;
 $50 = ($48|0)!=(0);
 $51 = ($49|0)!=(0);
 $52 = $50 | $51;
 if ($52) {
  HEAP32[$2>>2] = 0;
  $131 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $53 = $4;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($53) + 4)|0;
  $57 = $56;
  $58 = HEAP32[$57>>2]|0;
  $59 = ($58>>>0)>(0);
  $60 = ($55>>>0)>(1);
  $61 = ($58|0)==(0);
  $62 = $61 & $60;
  $63 = $59 | $62;
  $64 = HEAP32[$i>>2]|0;
  if (!($63)) {
   break;
  }
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
  $66 = $4;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($66) + 4)|0;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (_bitshift64Lshr(($68|0),($71|0),1)|0);
  $73 = tempRet0;
  $74 = $4;
  $75 = $74;
  HEAP32[$75>>2] = $72;
  $76 = (($74) + 4)|0;
  $77 = $76;
  HEAP32[$77>>2] = $73;
 }
 $78 = HEAP32[$3>>2]|0;
 $79 = ((($78)) + 8|0);
 $80 = (($79) + (($64*72)|0)|0);
 $81 = ((($80)) + 44|0);
 $82 = HEAP32[$81>>2]|0;
 HEAP32[$iCur>>2] = $82;
 $83 = HEAP32[$i>>2]|0;
 $84 = HEAP32[$3>>2]|0;
 $85 = ((($84)) + 8|0);
 $86 = (($85) + (($83*72)|0)|0);
 $87 = ((($86)) + 16|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = ((($88)) + 8|0);
 $90 = HEAP32[$89>>2]|0;
 HEAP32[$pIdx>>2] = $90;
 L17: while(1) {
  $91 = HEAP32[$pIdx>>2]|0;
  $92 = ($91|0)!=(0|0);
  if (!($92)) {
   label = 20;
   break;
  }
  $93 = HEAP32[$pIdx>>2]|0;
  $94 = ((($93)) + 40|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = ($95|0)==(0|0);
  L20: do {
   if (!($96)) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $97 = HEAP32[$i>>2]|0;
     $98 = HEAP32[$pIdx>>2]|0;
     $99 = ((($98)) + 50|0);
     $100 = HEAP16[$99>>1]|0;
     $101 = $100&65535;
     $102 = ($97|0)<($101|0);
     if (!($102)) {
      break L20;
     }
     $103 = HEAP32[$i>>2]|0;
     $104 = HEAP32[$pIdx>>2]|0;
     $105 = ((($104)) + 4|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = (($106) + ($103<<1)|0);
     $108 = HEAP16[$107>>1]|0;
     $109 = $108 << 16 >> 16;
     $110 = ($109|0)!=(-2);
     if (!($110)) {
      $111 = HEAP32[$5>>2]|0;
      $112 = HEAP32[$i>>2]|0;
      $113 = HEAP32[$pIdx>>2]|0;
      $114 = ((($113)) + 40|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = ((($115)) + 4|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = (($117) + (($112*20)|0)|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = HEAP32[$iCur>>2]|0;
      $121 = (_sqlite3ExprCompare($111,$119,$120)|0);
      $122 = ($121|0)==(0);
      if ($122) {
       label = 17;
       break L17;
      }
     }
     $126 = HEAP32[$i>>2]|0;
     $127 = (($126) + 1)|0;
     HEAP32[$i>>2] = $127;
    }
   }
  } while(0);
  $128 = HEAP32[$pIdx>>2]|0;
  $129 = ((($128)) + 20|0);
  $130 = HEAP32[$129>>2]|0;
  HEAP32[$pIdx>>2] = $130;
 }
 if ((label|0) == 17) {
  $123 = HEAP32[$iCur>>2]|0;
  $124 = HEAP32[$6>>2]|0;
  HEAP32[$124>>2] = $123;
  $125 = HEAP32[$7>>2]|0;
  HEAP32[$125>>2] = -2;
  HEAP32[$2>>2] = 1;
  $131 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 else if ((label|0) == 20) {
  HEAP32[$2>>2] = 0;
  $131 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($131|0);
 }
 return (0)|0;
}
function _operatorMask($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $c = sp + 4|0;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(75);
 do {
  if ($2) {
   HEAP16[$c>>1] = 1;
  } else {
   $3 = HEAP32[$0>>2]|0;
   $4 = ($3|0)==(76);
   if ($4) {
    HEAP16[$c>>1] = 256;
    break;
   }
   $5 = HEAP32[$0>>2]|0;
   $6 = ($5|0)==(73);
   if ($6) {
    HEAP16[$c>>1] = 128;
    break;
   } else {
    $7 = HEAP32[$0>>2]|0;
    $8 = (($7) - 79)|0;
    $9 = 2 << $8;
    $10 = $9&65535;
    HEAP16[$c>>1] = $10;
    break;
   }
  }
 } while(0);
 $11 = HEAP16[$c>>1]|0;
 STACKTOP = sp;return ($11|0);
}
function _markTermAsChild($pWC,$iChild,$iParent) {
 $pWC = $pWC|0;
 $iChild = $iChild|0;
 $iParent = $iParent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $iChild;
 HEAP32[$2>>2] = $iParent;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + (($4*48)|0)|0);
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = $3;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*48)|0)|0);
 $15 = ((($14)) + 16|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + (($17*48)|0)|0);
 $22 = ((($21)) + 16|0);
 HEAP16[$22>>1] = $16;
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 20|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + (($23*48)|0)|0);
 $28 = ((($27)) + 22|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = (($29) + 1)<<24>>24;
 HEAP8[$28>>0] = $30;
 STACKTOP = sp;return;
}
function _termIsEquivalence($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aff1 = 0, $aff2 = 0, $pColl = 0, $zColl1 = 0, $zColl2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $aff1 = sp + 25|0;
 $aff2 = sp + 24|0;
 $pColl = sp + 8|0;
 $zColl1 = sp + 4|0;
 $zColl2 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 60|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 512;
 $9 = ($8|0)==(0);
 if (!($9)) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(79);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(73);
  if ($17) {
   HEAP32[$0>>2] = 0;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 & 1;
 $22 = ($21|0)!=(0);
 if ($22) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3ExprAffinity($25)|0);
 HEAP8[$aff1>>0] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3ExprAffinity($29)|0);
 HEAP8[$aff2>>0] = $30;
 $31 = HEAP8[$aff1>>0]|0;
 $32 = $31 << 24 >> 24;
 $33 = HEAP8[$aff2>>0]|0;
 $34 = $33 << 24 >> 24;
 $35 = ($32|0)!=($34|0);
 do {
  if ($35) {
   $36 = HEAP8[$aff1>>0]|0;
   $37 = $36 << 24 >> 24;
   $38 = ($37|0)>=(67);
   if ($38) {
    $39 = HEAP8[$aff2>>0]|0;
    $40 = $39 << 24 >> 24;
    $41 = ($40|0)>=(67);
    if ($41) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 } while(0);
 $42 = HEAP32[$1>>2]|0;
 $43 = HEAP32[$2>>2]|0;
 $44 = ((($43)) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 16|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (_sqlite3BinaryCompareCollSeq($42,$45,$48)|0);
 HEAP32[$pColl>>2] = $49;
 $50 = HEAP32[$pColl>>2]|0;
 $51 = ($50|0)==(0|0);
 if (!($51)) {
  $52 = HEAP32[$pColl>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3_stricmp($53,20716)|0);
  $55 = ($54|0)==(0);
  if (!($55)) {
   $56 = HEAP32[$1>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3ExprCollSeq($56,$59)|0);
   HEAP32[$pColl>>2] = $60;
   $61 = HEAP32[$pColl>>2]|0;
   $62 = ($61|0)!=(0|0);
   if ($62) {
    $63 = HEAP32[$pColl>>2]|0;
    $64 = HEAP32[$63>>2]|0;
    $65 = $64;
   } else {
    $65 = 0;
   }
   HEAP32[$zColl1>>2] = $65;
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$2>>2]|0;
   $68 = ((($67)) + 16|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = (_sqlite3ExprCollSeq($66,$69)|0);
   HEAP32[$pColl>>2] = $70;
   $71 = HEAP32[$pColl>>2]|0;
   $72 = ($71|0)!=(0|0);
   if ($72) {
    $73 = HEAP32[$pColl>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = $74;
   } else {
    $75 = 0;
   }
   HEAP32[$zColl2>>2] = $75;
   $76 = HEAP32[$zColl1>>2]|0;
   $77 = HEAP32[$zColl2>>2]|0;
   $78 = (_sqlite3_stricmp($76,$77)|0);
   $79 = ($78|0)==(0);
   $80 = $79&1;
   HEAP32[$0>>2] = $80;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $81 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _exprCommute($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $expLeft = 0, $expRight = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $expRight = sp + 14|0;
 $expLeft = sp + 12|0;
 $t = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 256;
 $8 = $7&65535;
 HEAP16[$expRight>>1] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 256;
 $15 = $14&65535;
 HEAP16[$expLeft>>1] = $15;
 $16 = HEAP16[$expRight>>1]|0;
 $17 = $16&65535;
 $18 = HEAP16[$expLeft>>1]|0;
 $19 = $18&65535;
 $20 = ($17|0)==($19|0);
 do {
  if ($20) {
   $21 = HEAP16[$expRight>>1]|0;
   $22 = ($21<<16>>16)!=(0);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ((($25)) + 4|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = $27 & -257;
    HEAP32[$26>>2] = $28;
    break;
   }
   $29 = HEAP32[$0>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3ExprCollSeq($29,$32)|0);
   $34 = ($33|0)!=(0|0);
   if ($34) {
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = $39 | 256;
    HEAP32[$38>>2] = $40;
   }
  }
 } while(0);
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 16|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$t>>2] = $43;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 16|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP32[$t>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 12|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$1>>2]|0;
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = ($54|0)>=(80);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP8[$56>>0]|0;
 $58 = $57&255;
 $59 = (($58) - 80)|0;
 $60 = $59 ^ 2;
 $61 = (($60) + 80)|0;
 $62 = $61&255;
 $63 = HEAP32[$1>>2]|0;
 HEAP8[$63>>0] = $62;
 STACKTOP = sp;return;
}
function _transferJoinMarkings($pDerived,$pBase) {
 $pDerived = $pDerived|0;
 $pBase = $pBase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pDerived;
 HEAP32[$1>>2] = $pBase;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 1;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 | $7;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 36|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 36|0);
 HEAP16[$16>>1] = $14;
 STACKTOP = sp;return;
}
function _exprAnalyzeOrTerm($pSrc,$pWC,$idxTerm) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $idxTerm = $idxTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $affLeft = 0, $affRight = 0, $b = 0, $b1 = 0, $chngToIN = 0, $db = 0, $i = 0, $iColumn = 0, $iCursor = 0, $iOne = 0, $iTwo = 0, $idxNew = 0, $indexable = 0, $j = 0, $j2 = 0, $okToChngToIN = 0, $pAndInfo = 0, $pAndTerm = 0;
 var $pAndWC = 0, $pDup = 0, $pExpr = 0, $pLeft = 0, $pList = 0, $pNew = 0, $pOne = 0, $pOrInfo = 0, $pOrTerm = 0, $pOrWc = 0, $pOther = 0, $pParse = 0, $pTerm = 0, $pTwo = 0, $pWInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 156|0;
 $1 = sp + 152|0;
 $2 = sp + 148|0;
 $pWInfo = sp + 144|0;
 $pParse = sp + 140|0;
 $db = sp + 136|0;
 $pTerm = sp + 132|0;
 $pExpr = sp + 128|0;
 $i = sp + 124|0;
 $pOrWc = sp + 120|0;
 $pOrTerm = sp + 116|0;
 $pOrInfo = sp + 112|0;
 $chngToIN = sp + 24|0;
 $indexable = sp + 16|0;
 $pAndInfo = sp + 108|0;
 $pAndWC = sp + 104|0;
 $pAndTerm = sp + 100|0;
 $j = sp + 96|0;
 $b = sp + 8|0;
 $b1 = sp;
 $pOther = sp + 92|0;
 $iOne = sp + 88|0;
 $pOne = sp + 84|0;
 $iTwo = sp + 80|0;
 $pTwo = sp + 76|0;
 $okToChngToIN = sp + 72|0;
 $iColumn = sp + 68|0;
 $iCursor = sp + 64|0;
 $j2 = sp + 60|0;
 $affLeft = sp + 56|0;
 $affRight = sp + 52|0;
 $pDup = sp + 48|0;
 $pList = sp + 44|0;
 $pLeft = sp + 40|0;
 $pNew = sp + 36|0;
 $idxNew = sp + 32|0;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $idxTerm;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = HEAP32[$pWInfo>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*48)|0)|0);
 HEAP32[$pTerm>>2] = $13;
 $14 = HEAP32[$pTerm>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pExpr>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = (_sqlite3DbMallocZero($16,416,0)|0);
 HEAP32[$pOrInfo>>2] = $17;
 $18 = HEAP32[$pTerm>>2]|0;
 $19 = ((($18)) + 12|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$pOrInfo>>2]|0;
 $21 = ($20|0)==(0|0);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$pTerm>>2]|0;
 $23 = ((($22)) + 20|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 | 16;
 $27 = $26&65535;
 HEAP16[$23>>1] = $27;
 $28 = HEAP32[$pOrInfo>>2]|0;
 HEAP32[$pOrWc>>2] = $28;
 $29 = HEAP32[$pOrWc>>2]|0;
 $30 = HEAP32[$pWInfo>>2]|0;
 _sqlite3WhereClauseInit($29,$30);
 $31 = HEAP32[$pOrWc>>2]|0;
 $32 = HEAP32[$pExpr>>2]|0;
 _sqlite3WhereSplit($31,$32,71);
 $33 = HEAP32[$0>>2]|0;
 $34 = HEAP32[$pOrWc>>2]|0;
 _sqlite3WhereExprAnalyze($33,$34);
 $35 = HEAP32[$db>>2]|0;
 $36 = ((($35)) + 65|0);
 $37 = HEAP8[$36>>0]|0;
 $38 = ($37<<24>>24)!=(0);
 if ($38) {
  STACKTOP = sp;return;
 }
 $39 = $indexable;
 $40 = $39;
 HEAP32[$40>>2] = -1;
 $41 = (($39) + 4)|0;
 $42 = $41;
 HEAP32[$42>>2] = -1;
 $43 = $chngToIN;
 $44 = $43;
 HEAP32[$44>>2] = -1;
 $45 = (($43) + 4)|0;
 $46 = $45;
 HEAP32[$46>>2] = -1;
 $47 = HEAP32[$pOrWc>>2]|0;
 $48 = ((($47)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) - 1)|0;
 HEAP32[$i>>2] = $50;
 $51 = HEAP32[$pOrWc>>2]|0;
 $52 = ((($51)) + 20|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$pOrTerm>>2] = $53;
 while(1) {
  $54 = HEAP32[$i>>2]|0;
  $55 = ($54|0)>=(0);
  $56 = $indexable;
  $57 = $56;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($56) + 4)|0;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = ($58|0)!=(0);
  $63 = ($61|0)!=(0);
  $64 = $62 | $63;
  $65 = $55 ? $64 : 0;
  if (!($65)) {
   break;
  }
  $66 = HEAP32[$pOrTerm>>2]|0;
  $67 = ((($66)) + 18|0);
  $68 = HEAP16[$67>>1]|0;
  $69 = $68&65535;
  $70 = $69 & 511;
  $71 = ($70|0)==(0);
  do {
   if ($71) {
    $72 = $chngToIN;
    $73 = $72;
    HEAP32[$73>>2] = 0;
    $74 = (($72) + 4)|0;
    $75 = $74;
    HEAP32[$75>>2] = 0;
    $76 = HEAP32[$db>>2]|0;
    $77 = (_sqlite3DbMallocRawNN($76,408,0)|0);
    HEAP32[$pAndInfo>>2] = $77;
    $78 = HEAP32[$pAndInfo>>2]|0;
    $79 = ($78|0)!=(0|0);
    if ($79) {
     $80 = $b;
     $81 = $80;
     HEAP32[$81>>2] = 0;
     $82 = (($80) + 4)|0;
     $83 = $82;
     HEAP32[$83>>2] = 0;
     $84 = HEAP32[$pAndInfo>>2]|0;
     $85 = HEAP32[$pOrTerm>>2]|0;
     $86 = ((($85)) + 12|0);
     HEAP32[$86>>2] = $84;
     $87 = HEAP32[$pOrTerm>>2]|0;
     $88 = ((($87)) + 20|0);
     $89 = HEAP16[$88>>1]|0;
     $90 = $89&65535;
     $91 = $90 | 32;
     $92 = $91&65535;
     HEAP16[$88>>1] = $92;
     $93 = HEAP32[$pOrTerm>>2]|0;
     $94 = ((($93)) + 18|0);
     HEAP16[$94>>1] = 1024;
     $95 = HEAP32[$pAndInfo>>2]|0;
     HEAP32[$pAndWC>>2] = $95;
     $96 = HEAP32[$pAndWC>>2]|0;
     $97 = HEAP32[$1>>2]|0;
     $98 = HEAP32[$97>>2]|0;
     _sqlite3WhereClauseInit($96,$98);
     $99 = HEAP32[$pAndWC>>2]|0;
     $100 = HEAP32[$pOrTerm>>2]|0;
     $101 = HEAP32[$100>>2]|0;
     _sqlite3WhereSplit($99,$101,72);
     $102 = HEAP32[$0>>2]|0;
     $103 = HEAP32[$pAndWC>>2]|0;
     _sqlite3WhereExprAnalyze($102,$103);
     $104 = HEAP32[$1>>2]|0;
     $105 = HEAP32[$pAndWC>>2]|0;
     $106 = ((($105)) + 4|0);
     HEAP32[$106>>2] = $104;
     $107 = HEAP32[$db>>2]|0;
     $108 = ((($107)) + 65|0);
     $109 = HEAP8[$108>>0]|0;
     $110 = ($109<<24>>24)!=(0);
     L13: do {
      if (!($110)) {
       HEAP32[$j>>2] = 0;
       $111 = HEAP32[$pAndWC>>2]|0;
       $112 = ((($111)) + 20|0);
       $113 = HEAP32[$112>>2]|0;
       HEAP32[$pAndTerm>>2] = $113;
       while(1) {
        $114 = HEAP32[$j>>2]|0;
        $115 = HEAP32[$pAndWC>>2]|0;
        $116 = ((($115)) + 12|0);
        $117 = HEAP32[$116>>2]|0;
        $118 = ($114|0)<($117|0);
        if (!($118)) {
         break L13;
        }
        $119 = HEAP32[$pAndTerm>>2]|0;
        $120 = HEAP32[$119>>2]|0;
        $121 = HEAP8[$120>>0]|0;
        $122 = $121&255;
        $123 = (_allowedOp($122)|0);
        $124 = ($123|0)!=(0);
        if ($124) {
         $125 = HEAP32[$pWInfo>>2]|0;
         $126 = ((($125)) + 68|0);
         $127 = HEAP32[$pAndTerm>>2]|0;
         $128 = ((($127)) + 8|0);
         $129 = HEAP32[$128>>2]|0;
         $130 = (_sqlite3WhereGetMask($126,$129)|0);
         $131 = tempRet0;
         $132 = $b;
         $133 = $132;
         $134 = HEAP32[$133>>2]|0;
         $135 = (($132) + 4)|0;
         $136 = $135;
         $137 = HEAP32[$136>>2]|0;
         $138 = $134 | $130;
         $139 = $137 | $131;
         $140 = $b;
         $141 = $140;
         HEAP32[$141>>2] = $138;
         $142 = (($140) + 4)|0;
         $143 = $142;
         HEAP32[$143>>2] = $139;
        }
        $144 = HEAP32[$j>>2]|0;
        $145 = (($144) + 1)|0;
        HEAP32[$j>>2] = $145;
        $146 = HEAP32[$pAndTerm>>2]|0;
        $147 = ((($146)) + 48|0);
        HEAP32[$pAndTerm>>2] = $147;
       }
      }
     } while(0);
     $148 = $b;
     $149 = $148;
     $150 = HEAP32[$149>>2]|0;
     $151 = (($148) + 4)|0;
     $152 = $151;
     $153 = HEAP32[$152>>2]|0;
     $154 = $indexable;
     $155 = $154;
     $156 = HEAP32[$155>>2]|0;
     $157 = (($154) + 4)|0;
     $158 = $157;
     $159 = HEAP32[$158>>2]|0;
     $160 = $156 & $150;
     $161 = $159 & $153;
     $162 = $indexable;
     $163 = $162;
     HEAP32[$163>>2] = $160;
     $164 = (($162) + 4)|0;
     $165 = $164;
     HEAP32[$165>>2] = $161;
    }
   } else {
    $166 = HEAP32[$pOrTerm>>2]|0;
    $167 = ((($166)) + 20|0);
    $168 = HEAP16[$167>>1]|0;
    $169 = $168&65535;
    $170 = $169 & 8;
    $171 = ($170|0)!=(0);
    if (!($171)) {
     $172 = HEAP32[$pWInfo>>2]|0;
     $173 = ((($172)) + 68|0);
     $174 = HEAP32[$pOrTerm>>2]|0;
     $175 = ((($174)) + 8|0);
     $176 = HEAP32[$175>>2]|0;
     $177 = (_sqlite3WhereGetMask($173,$176)|0);
     $178 = tempRet0;
     $179 = $b1;
     $180 = $179;
     HEAP32[$180>>2] = $177;
     $181 = (($179) + 4)|0;
     $182 = $181;
     HEAP32[$182>>2] = $178;
     $183 = HEAP32[$pOrTerm>>2]|0;
     $184 = ((($183)) + 20|0);
     $185 = HEAP16[$184>>1]|0;
     $186 = $185&65535;
     $187 = $186 & 2;
     $188 = ($187|0)!=(0);
     if ($188) {
      $189 = HEAP32[$pOrTerm>>2]|0;
      $190 = ((($189)) + 4|0);
      $191 = HEAP32[$190>>2]|0;
      $192 = HEAP32[$pOrWc>>2]|0;
      $193 = ((($192)) + 20|0);
      $194 = HEAP32[$193>>2]|0;
      $195 = (($194) + (($191*48)|0)|0);
      HEAP32[$pOther>>2] = $195;
      $196 = HEAP32[$pWInfo>>2]|0;
      $197 = ((($196)) + 68|0);
      $198 = HEAP32[$pOther>>2]|0;
      $199 = ((($198)) + 8|0);
      $200 = HEAP32[$199>>2]|0;
      $201 = (_sqlite3WhereGetMask($197,$200)|0);
      $202 = tempRet0;
      $203 = $b1;
      $204 = $203;
      $205 = HEAP32[$204>>2]|0;
      $206 = (($203) + 4)|0;
      $207 = $206;
      $208 = HEAP32[$207>>2]|0;
      $209 = $205 | $201;
      $210 = $208 | $202;
      $211 = $b1;
      $212 = $211;
      HEAP32[$212>>2] = $209;
      $213 = (($211) + 4)|0;
      $214 = $213;
      HEAP32[$214>>2] = $210;
     }
     $215 = $b1;
     $216 = $215;
     $217 = HEAP32[$216>>2]|0;
     $218 = (($215) + 4)|0;
     $219 = $218;
     $220 = HEAP32[$219>>2]|0;
     $221 = $indexable;
     $222 = $221;
     $223 = HEAP32[$222>>2]|0;
     $224 = (($221) + 4)|0;
     $225 = $224;
     $226 = HEAP32[$225>>2]|0;
     $227 = $223 & $217;
     $228 = $226 & $220;
     $229 = $indexable;
     $230 = $229;
     HEAP32[$230>>2] = $227;
     $231 = (($229) + 4)|0;
     $232 = $231;
     HEAP32[$232>>2] = $228;
     $233 = HEAP32[$pOrTerm>>2]|0;
     $234 = ((($233)) + 18|0);
     $235 = HEAP16[$234>>1]|0;
     $236 = $235&65535;
     $237 = $236 & 2;
     $238 = ($237|0)==(0);
     if ($238) {
      $239 = $chngToIN;
      $240 = $239;
      HEAP32[$240>>2] = 0;
      $241 = (($239) + 4)|0;
      $242 = $241;
      HEAP32[$242>>2] = 0;
      break;
     } else {
      $243 = $b1;
      $244 = $243;
      $245 = HEAP32[$244>>2]|0;
      $246 = (($243) + 4)|0;
      $247 = $246;
      $248 = HEAP32[$247>>2]|0;
      $249 = $chngToIN;
      $250 = $249;
      $251 = HEAP32[$250>>2]|0;
      $252 = (($249) + 4)|0;
      $253 = $252;
      $254 = HEAP32[$253>>2]|0;
      $255 = $251 & $245;
      $256 = $254 & $248;
      $257 = $chngToIN;
      $258 = $257;
      HEAP32[$258>>2] = $255;
      $259 = (($257) + 4)|0;
      $260 = $259;
      HEAP32[$260>>2] = $256;
      break;
     }
    }
   }
  } while(0);
  $261 = HEAP32[$i>>2]|0;
  $262 = (($261) + -1)|0;
  HEAP32[$i>>2] = $262;
  $263 = HEAP32[$pOrTerm>>2]|0;
  $264 = ((($263)) + 48|0);
  HEAP32[$pOrTerm>>2] = $264;
 }
 $265 = $indexable;
 $266 = $265;
 $267 = HEAP32[$266>>2]|0;
 $268 = (($265) + 4)|0;
 $269 = $268;
 $270 = HEAP32[$269>>2]|0;
 $271 = HEAP32[$pOrInfo>>2]|0;
 $272 = ((($271)) + 408|0);
 $273 = $272;
 $274 = $273;
 HEAP32[$274>>2] = $267;
 $275 = (($273) + 4)|0;
 $276 = $275;
 HEAP32[$276>>2] = $270;
 $277 = $indexable;
 $278 = $277;
 $279 = HEAP32[$278>>2]|0;
 $280 = (($277) + 4)|0;
 $281 = $280;
 $282 = HEAP32[$281>>2]|0;
 $283 = ($279|0)==(0);
 $284 = ($282|0)==(0);
 $285 = $283 & $284;
 $286 = $285 ? 0 : 512;
 $287 = $286&65535;
 $288 = HEAP32[$pTerm>>2]|0;
 $289 = ((($288)) + 18|0);
 HEAP16[$289>>1] = $287;
 $290 = $indexable;
 $291 = $290;
 $292 = HEAP32[$291>>2]|0;
 $293 = (($290) + 4)|0;
 $294 = $293;
 $295 = HEAP32[$294>>2]|0;
 $296 = ($292|0)!=(0);
 $297 = ($295|0)!=(0);
 $298 = $296 | $297;
 L32: do {
  if ($298) {
   $299 = HEAP32[$pOrWc>>2]|0;
   $300 = ((($299)) + 12|0);
   $301 = HEAP32[$300>>2]|0;
   $302 = ($301|0)==(2);
   if ($302) {
    HEAP32[$iOne>>2] = 0;
    L35: while(1) {
     $303 = HEAP32[$pOrWc>>2]|0;
     $304 = ((($303)) + 20|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = HEAP32[$iOne>>2]|0;
     $307 = (($306) + 1)|0;
     HEAP32[$iOne>>2] = $307;
     $308 = (_whereNthSubterm($305,$306)|0);
     HEAP32[$pOne>>2] = $308;
     $309 = ($308|0)!=(0|0);
     if (!($309)) {
      break L32;
     }
     HEAP32[$iTwo>>2] = 0;
     while(1) {
      $310 = HEAP32[$pOrWc>>2]|0;
      $311 = ((($310)) + 20|0);
      $312 = HEAP32[$311>>2]|0;
      $313 = ((($312)) + 48|0);
      $314 = HEAP32[$iTwo>>2]|0;
      $315 = (($314) + 1)|0;
      HEAP32[$iTwo>>2] = $315;
      $316 = (_whereNthSubterm($313,$314)|0);
      HEAP32[$pTwo>>2] = $316;
      $317 = ($316|0)!=(0|0);
      if (!($317)) {
       continue L35;
      }
      $318 = HEAP32[$0>>2]|0;
      $319 = HEAP32[$1>>2]|0;
      $320 = HEAP32[$pOne>>2]|0;
      $321 = HEAP32[$pTwo>>2]|0;
      _whereCombineDisjuncts($318,$319,$320,$321);
     }
    }
   }
  }
 } while(0);
 $322 = $chngToIN;
 $323 = $322;
 $324 = HEAP32[$323>>2]|0;
 $325 = (($322) + 4)|0;
 $326 = $325;
 $327 = HEAP32[$326>>2]|0;
 $328 = ($324|0)!=(0);
 $329 = ($327|0)!=(0);
 $330 = $328 | $329;
 if (!($330)) {
  STACKTOP = sp;return;
 }
 HEAP32[$okToChngToIN>>2] = 0;
 HEAP32[$iColumn>>2] = -1;
 HEAP32[$iCursor>>2] = -1;
 HEAP32[$j2>>2] = 0;
 HEAP32[$j2>>2] = 0;
 while(1) {
  $331 = HEAP32[$j2>>2]|0;
  $332 = ($331|0)<(2);
  if (!($332)) {
   break;
  }
  $333 = HEAP32[$okToChngToIN>>2]|0;
  $334 = ($333|0)!=(0);
  $335 = $334 ^ 1;
  if (!($335)) {
   break;
  }
  $336 = HEAP32[$pOrWc>>2]|0;
  $337 = ((($336)) + 20|0);
  $338 = HEAP32[$337>>2]|0;
  HEAP32[$pOrTerm>>2] = $338;
  $339 = HEAP32[$pOrWc>>2]|0;
  $340 = ((($339)) + 12|0);
  $341 = HEAP32[$340>>2]|0;
  $342 = (($341) - 1)|0;
  HEAP32[$i>>2] = $342;
  while(1) {
   $343 = HEAP32[$i>>2]|0;
   $344 = ($343|0)>=(0);
   if (!($344)) {
    break;
   }
   $345 = HEAP32[$pOrTerm>>2]|0;
   $346 = ((($345)) + 20|0);
   $347 = HEAP16[$346>>1]|0;
   $348 = $347&65535;
   $349 = $348 & -65;
   $350 = $349&65535;
   HEAP16[$346>>1] = $350;
   $351 = HEAP32[$pOrTerm>>2]|0;
   $352 = ((($351)) + 8|0);
   $353 = HEAP32[$352>>2]|0;
   $354 = HEAP32[$iCursor>>2]|0;
   $355 = ($353|0)==($354|0);
   if (!($355)) {
    $356 = $chngToIN;
    $357 = $356;
    $358 = HEAP32[$357>>2]|0;
    $359 = (($356) + 4)|0;
    $360 = $359;
    $361 = HEAP32[$360>>2]|0;
    $362 = HEAP32[$pWInfo>>2]|0;
    $363 = ((($362)) + 68|0);
    $364 = HEAP32[$pOrTerm>>2]|0;
    $365 = ((($364)) + 8|0);
    $366 = HEAP32[$365>>2]|0;
    $367 = (_sqlite3WhereGetMask($363,$366)|0);
    $368 = tempRet0;
    $369 = $358 & $367;
    $370 = $361 & $368;
    $371 = ($369|0)==(0);
    $372 = ($370|0)==(0);
    $373 = $371 & $372;
    if (!($373)) {
     label = 36;
     break;
    }
   }
   $380 = HEAP32[$i>>2]|0;
   $381 = (($380) + -1)|0;
   HEAP32[$i>>2] = $381;
   $382 = HEAP32[$pOrTerm>>2]|0;
   $383 = ((($382)) + 48|0);
   HEAP32[$pOrTerm>>2] = $383;
  }
  if ((label|0) == 36) {
   label = 0;
   $374 = HEAP32[$pOrTerm>>2]|0;
   $375 = ((($374)) + 12|0);
   $376 = HEAP32[$375>>2]|0;
   HEAP32[$iColumn>>2] = $376;
   $377 = HEAP32[$pOrTerm>>2]|0;
   $378 = ((($377)) + 8|0);
   $379 = HEAP32[$378>>2]|0;
   HEAP32[$iCursor>>2] = $379;
  }
  $384 = HEAP32[$i>>2]|0;
  $385 = ($384|0)<(0);
  if ($385) {
   break;
  }
  HEAP32[$okToChngToIN>>2] = 1;
  while(1) {
   $386 = HEAP32[$i>>2]|0;
   $387 = ($386|0)>=(0);
   $388 = HEAP32[$okToChngToIN>>2]|0;
   $389 = ($388|0)!=(0);
   $390 = $387 ? $389 : 0;
   if (!($390)) {
    break;
   }
   $391 = HEAP32[$pOrTerm>>2]|0;
   $392 = ((($391)) + 8|0);
   $393 = HEAP32[$392>>2]|0;
   $394 = HEAP32[$iCursor>>2]|0;
   $395 = ($393|0)!=($394|0);
   $396 = HEAP32[$pOrTerm>>2]|0;
   do {
    if ($395) {
     $397 = ((($396)) + 20|0);
     $398 = HEAP16[$397>>1]|0;
     $399 = $398&65535;
     $400 = $399 & -65;
     $401 = $400&65535;
     HEAP16[$397>>1] = $401;
    } else {
     $402 = ((($396)) + 12|0);
     $403 = HEAP32[$402>>2]|0;
     $404 = HEAP32[$iColumn>>2]|0;
     $405 = ($403|0)!=($404|0);
     if ($405) {
      HEAP32[$okToChngToIN>>2] = 0;
      break;
     }
     $406 = HEAP32[$pOrTerm>>2]|0;
     $407 = HEAP32[$406>>2]|0;
     $408 = ((($407)) + 16|0);
     $409 = HEAP32[$408>>2]|0;
     $410 = (_sqlite3ExprAffinity($409)|0);
     $411 = $410 << 24 >> 24;
     HEAP32[$affRight>>2] = $411;
     $412 = HEAP32[$pOrTerm>>2]|0;
     $413 = HEAP32[$412>>2]|0;
     $414 = ((($413)) + 12|0);
     $415 = HEAP32[$414>>2]|0;
     $416 = (_sqlite3ExprAffinity($415)|0);
     $417 = $416 << 24 >> 24;
     HEAP32[$affLeft>>2] = $417;
     $418 = HEAP32[$affRight>>2]|0;
     $419 = ($418|0)!=(0);
     if ($419) {
      $420 = HEAP32[$affRight>>2]|0;
      $421 = HEAP32[$affLeft>>2]|0;
      $422 = ($420|0)!=($421|0);
      if ($422) {
       HEAP32[$okToChngToIN>>2] = 0;
       break;
      }
     }
     $423 = HEAP32[$pOrTerm>>2]|0;
     $424 = ((($423)) + 20|0);
     $425 = HEAP16[$424>>1]|0;
     $426 = $425&65535;
     $427 = $426 | 64;
     $428 = $427&65535;
     HEAP16[$424>>1] = $428;
    }
   } while(0);
   $429 = HEAP32[$i>>2]|0;
   $430 = (($429) + -1)|0;
   HEAP32[$i>>2] = $430;
   $431 = HEAP32[$pOrTerm>>2]|0;
   $432 = ((($431)) + 48|0);
   HEAP32[$pOrTerm>>2] = $432;
  }
  $433 = HEAP32[$j2>>2]|0;
  $434 = (($433) + 1)|0;
  HEAP32[$j2>>2] = $434;
 }
 $435 = HEAP32[$okToChngToIN>>2]|0;
 $436 = ($435|0)!=(0);
 if (!($436)) {
  STACKTOP = sp;return;
 }
 HEAP32[$pList>>2] = 0;
 HEAP32[$pLeft>>2] = 0;
 $437 = HEAP32[$pOrWc>>2]|0;
 $438 = ((($437)) + 12|0);
 $439 = HEAP32[$438>>2]|0;
 $440 = (($439) - 1)|0;
 HEAP32[$i>>2] = $440;
 $441 = HEAP32[$pOrWc>>2]|0;
 $442 = ((($441)) + 20|0);
 $443 = HEAP32[$442>>2]|0;
 HEAP32[$pOrTerm>>2] = $443;
 while(1) {
  $444 = HEAP32[$i>>2]|0;
  $445 = ($444|0)>=(0);
  if (!($445)) {
   break;
  }
  $446 = HEAP32[$pOrTerm>>2]|0;
  $447 = ((($446)) + 20|0);
  $448 = HEAP16[$447>>1]|0;
  $449 = $448&65535;
  $450 = $449 & 64;
  $451 = ($450|0)==(0);
  if (!($451)) {
   $452 = HEAP32[$db>>2]|0;
   $453 = HEAP32[$pOrTerm>>2]|0;
   $454 = HEAP32[$453>>2]|0;
   $455 = ((($454)) + 16|0);
   $456 = HEAP32[$455>>2]|0;
   $457 = (_sqlite3ExprDup($452,$456,0)|0);
   HEAP32[$pDup>>2] = $457;
   $458 = HEAP32[$pWInfo>>2]|0;
   $459 = HEAP32[$458>>2]|0;
   $460 = HEAP32[$pList>>2]|0;
   $461 = HEAP32[$pDup>>2]|0;
   $462 = (_sqlite3ExprListAppend($459,$460,$461)|0);
   HEAP32[$pList>>2] = $462;
   $463 = HEAP32[$pOrTerm>>2]|0;
   $464 = HEAP32[$463>>2]|0;
   $465 = ((($464)) + 12|0);
   $466 = HEAP32[$465>>2]|0;
   HEAP32[$pLeft>>2] = $466;
  }
  $467 = HEAP32[$i>>2]|0;
  $468 = (($467) + -1)|0;
  HEAP32[$i>>2] = $468;
  $469 = HEAP32[$pOrTerm>>2]|0;
  $470 = ((($469)) + 48|0);
  HEAP32[$pOrTerm>>2] = $470;
 }
 $471 = HEAP32[$db>>2]|0;
 $472 = HEAP32[$pLeft>>2]|0;
 $473 = (_sqlite3ExprDup($471,$472,0)|0);
 HEAP32[$pDup>>2] = $473;
 $474 = HEAP32[$pParse>>2]|0;
 $475 = HEAP32[$pDup>>2]|0;
 $476 = (_sqlite3PExpr($474,75,$475,0,0)|0);
 HEAP32[$pNew>>2] = $476;
 $477 = HEAP32[$pNew>>2]|0;
 $478 = ($477|0)!=(0|0);
 if ($478) {
  $479 = HEAP32[$pNew>>2]|0;
  $480 = HEAP32[$pExpr>>2]|0;
  _transferJoinMarkings($479,$480);
  $481 = HEAP32[$pList>>2]|0;
  $482 = HEAP32[$pNew>>2]|0;
  $483 = ((($482)) + 20|0);
  HEAP32[$483>>2] = $481;
  $484 = HEAP32[$1>>2]|0;
  $485 = HEAP32[$pNew>>2]|0;
  $486 = (_whereClauseInsert($484,$485,3)|0);
  HEAP32[$idxNew>>2] = $486;
  $487 = HEAP32[$0>>2]|0;
  $488 = HEAP32[$1>>2]|0;
  $489 = HEAP32[$idxNew>>2]|0;
  _exprAnalyze($487,$488,$489);
  $490 = HEAP32[$2>>2]|0;
  $491 = HEAP32[$1>>2]|0;
  $492 = ((($491)) + 20|0);
  $493 = HEAP32[$492>>2]|0;
  $494 = (($493) + (($490*48)|0)|0);
  HEAP32[$pTerm>>2] = $494;
  $495 = HEAP32[$1>>2]|0;
  $496 = HEAP32[$idxNew>>2]|0;
  $497 = HEAP32[$2>>2]|0;
  _markTermAsChild($495,$496,$497);
 } else {
  $498 = HEAP32[$db>>2]|0;
  $499 = HEAP32[$pList>>2]|0;
  _sqlite3ExprListDelete($498,$499);
 }
 $500 = HEAP32[$pTerm>>2]|0;
 $501 = ((($500)) + 18|0);
 HEAP16[$501>>1] = 4096;
 STACKTOP = sp;return;
}
function _isLikeOrGlob($pParse,$pExpr,$ppPrefix,$pisComplete,$pnoCase) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $ppPrefix = $ppPrefix|0;
 $pisComplete = $pisComplete|0;
 $pnoCase = $pnoCase|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $cnt = 0, $db = 0, $iCol = 0, $op = 0, $pLeft = 0, $pList = 0, $pPrefix = 0, $pReprepare = 0, $pRight = 0, $pVal = 0;
 var $r1 = 0, $rc = 0, $v = 0, $wc = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $z = sp + 56|0;
 $pRight = sp + 52|0;
 $pLeft = sp + 48|0;
 $pList = sp + 44|0;
 $c = sp + 40|0;
 $cnt = sp + 36|0;
 $wc = sp + 84|0;
 $db = sp + 32|0;
 $pVal = sp + 28|0;
 $op = sp + 24|0;
 $rc = sp + 20|0;
 $pReprepare = sp + 16|0;
 $iCol = sp + 12|0;
 $pPrefix = sp + 8|0;
 $v = sp + 4|0;
 $r1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $ppPrefix;
 HEAP32[$4>>2] = $pisComplete;
 HEAP32[$5>>2] = $pnoCase;
 HEAP32[$z>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$pVal>>2] = 0;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$5>>2]|0;
 $11 = (_sqlite3IsLikeFunction($8,$9,$10,$wc)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  HEAP32[$0>>2] = 0;
  $167 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($167|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pList>>2] = $15;
 $16 = HEAP32[$pList>>2]|0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pLeft>>2] = $20;
 $21 = HEAP32[$pLeft>>2]|0;
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(152);
 if (!($24)) {
  $25 = HEAP32[$pLeft>>2]|0;
  $26 = (_sqlite3ExprAffinity($25)|0);
  $27 = $26 << 24 >> 24;
  $28 = ($27|0)!=(66);
  if (!($28)) {
   $29 = HEAP32[$pLeft>>2]|0;
   $30 = ((($29)) + 44|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ((($31)) + 42|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = $34 & 16;
   $36 = ($35|0)!=(0);
   if (!($36)) {
    $37 = HEAP32[$pList>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$39>>2]|0;
    $41 = (_sqlite3ExprSkipCollate($40)|0);
    HEAP32[$pRight>>2] = $41;
    $42 = HEAP32[$pRight>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    HEAP32[$op>>2] = $44;
    $45 = HEAP32[$op>>2]|0;
    $46 = ($45|0)==(135);
    if ($46) {
     $47 = HEAP32[$1>>2]|0;
     $48 = ((($47)) + 480|0);
     $49 = HEAP32[$48>>2]|0;
     HEAP32[$pReprepare>>2] = $49;
     $50 = HEAP32[$pRight>>2]|0;
     $51 = ((($50)) + 32|0);
     $52 = HEAP16[$51>>1]|0;
     $53 = $52 << 16 >> 16;
     HEAP32[$iCol>>2] = $53;
     $54 = HEAP32[$pReprepare>>2]|0;
     $55 = HEAP32[$iCol>>2]|0;
     $56 = (_sqlite3VdbeGetBoundValue($54,$55,65)|0);
     HEAP32[$pVal>>2] = $56;
     $57 = HEAP32[$pVal>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pVal>>2]|0;
      $60 = (_sqlite3_value_type($59)|0);
      $61 = ($60|0)==(3);
      if ($61) {
       $62 = HEAP32[$pVal>>2]|0;
       $63 = (_sqlite3_value_text($62)|0);
       HEAP32[$z>>2] = $63;
      }
     }
     $64 = HEAP32[$1>>2]|0;
     $65 = ((($64)) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$iCol>>2]|0;
     _sqlite3VdbeSetVarmask($66,$67);
    } else {
     $68 = HEAP32[$op>>2]|0;
     $69 = ($68|0)==(97);
     if ($69) {
      $70 = HEAP32[$pRight>>2]|0;
      $71 = ((($70)) + 8|0);
      $72 = HEAP32[$71>>2]|0;
      HEAP32[$z>>2] = $72;
     }
    }
    $73 = HEAP32[$z>>2]|0;
    $74 = ($73|0)!=(0|0);
    do {
     if ($74) {
      HEAP32[$cnt>>2] = 0;
      while(1) {
       $75 = HEAP32[$cnt>>2]|0;
       $76 = HEAP32[$z>>2]|0;
       $77 = (($76) + ($75)|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78 << 24 >> 24;
       HEAP32[$c>>2] = $79;
       $80 = ($79|0)!=(0);
       if ($80) {
        $81 = HEAP32[$c>>2]|0;
        $82 = HEAP8[$wc>>0]|0;
        $83 = $82 << 24 >> 24;
        $84 = ($81|0)!=($83|0);
        if ($84) {
         $85 = HEAP32[$c>>2]|0;
         $86 = ((($wc)) + 1|0);
         $87 = HEAP8[$86>>0]|0;
         $88 = $87 << 24 >> 24;
         $89 = ($85|0)!=($88|0);
         if ($89) {
          $90 = HEAP32[$c>>2]|0;
          $91 = ((($wc)) + 2|0);
          $92 = HEAP8[$91>>0]|0;
          $93 = $92 << 24 >> 24;
          $94 = ($90|0)!=($93|0);
          $168 = $94;
         } else {
          $168 = 0;
         }
        } else {
         $168 = 0;
        }
       } else {
        $168 = 0;
       }
       $95 = HEAP32[$cnt>>2]|0;
       if (!($168)) {
        break;
       }
       $96 = (($95) + 1)|0;
       HEAP32[$cnt>>2] = $96;
      }
      $97 = ($95|0)!=(0);
      if ($97) {
       $98 = HEAP32[$cnt>>2]|0;
       $99 = (($98) - 1)|0;
       $100 = HEAP32[$z>>2]|0;
       $101 = (($100) + ($99)|0);
       $102 = HEAP8[$101>>0]|0;
       $103 = $102&255;
       $104 = (255)!=($103|0);
       if ($104) {
        $105 = HEAP32[$c>>2]|0;
        $106 = HEAP8[$wc>>0]|0;
        $107 = $106 << 24 >> 24;
        $108 = ($105|0)==($107|0);
        if ($108) {
         $109 = HEAP32[$cnt>>2]|0;
         $110 = (($109) + 1)|0;
         $111 = HEAP32[$z>>2]|0;
         $112 = (($111) + ($110)|0);
         $113 = HEAP8[$112>>0]|0;
         $114 = $113 << 24 >> 24;
         $115 = ($114|0)==(0);
         $117 = $115;
        } else {
         $117 = 0;
        }
        $116 = $117&1;
        $118 = HEAP32[$4>>2]|0;
        HEAP32[$118>>2] = $116;
        $119 = HEAP32[$db>>2]|0;
        $120 = HEAP32[$z>>2]|0;
        $121 = (_sqlite3Expr($119,97,$120)|0);
        HEAP32[$pPrefix>>2] = $121;
        $122 = HEAP32[$pPrefix>>2]|0;
        $123 = ($122|0)!=(0|0);
        if ($123) {
         $124 = HEAP32[$cnt>>2]|0;
         $125 = HEAP32[$pPrefix>>2]|0;
         $126 = ((($125)) + 8|0);
         $127 = HEAP32[$126>>2]|0;
         $128 = (($127) + ($124)|0);
         HEAP8[$128>>0] = 0;
        }
        $129 = HEAP32[$pPrefix>>2]|0;
        $130 = HEAP32[$3>>2]|0;
        HEAP32[$130>>2] = $129;
        $131 = HEAP32[$op>>2]|0;
        $132 = ($131|0)==(135);
        if (!($132)) {
         break;
        }
        $133 = HEAP32[$1>>2]|0;
        $134 = ((($133)) + 8|0);
        $135 = HEAP32[$134>>2]|0;
        HEAP32[$v>>2] = $135;
        $136 = HEAP32[$v>>2]|0;
        $137 = HEAP32[$pRight>>2]|0;
        $138 = ((($137)) + 32|0);
        $139 = HEAP16[$138>>1]|0;
        $140 = $139 << 16 >> 16;
        _sqlite3VdbeSetVarmask($136,$140);
        $141 = HEAP32[$4>>2]|0;
        $142 = HEAP32[$141>>2]|0;
        $143 = ($142|0)!=(0);
        if (!($143)) {
         break;
        }
        $144 = HEAP32[$pRight>>2]|0;
        $145 = ((($144)) + 8|0);
        $146 = HEAP32[$145>>2]|0;
        $147 = ((($146)) + 1|0);
        $148 = HEAP8[$147>>0]|0;
        $149 = $148 << 24 >> 24;
        $150 = ($149|0)!=(0);
        if (!($150)) {
         break;
        }
        $151 = HEAP32[$1>>2]|0;
        $152 = (_sqlite3GetTempReg($151)|0);
        HEAP32[$r1>>2] = $152;
        $153 = HEAP32[$1>>2]|0;
        $154 = HEAP32[$pRight>>2]|0;
        $155 = HEAP32[$r1>>2]|0;
        (_sqlite3ExprCodeTarget($153,$154,$155)|0);
        $156 = HEAP32[$v>>2]|0;
        $157 = HEAP32[$v>>2]|0;
        $158 = (_sqlite3VdbeCurrentAddr($157)|0);
        $159 = (($158) - 1)|0;
        _sqlite3VdbeChangeP3($156,$159,0);
        $160 = HEAP32[$1>>2]|0;
        $161 = HEAP32[$r1>>2]|0;
        _sqlite3ReleaseTempReg($160,$161);
        break;
       }
      }
      HEAP32[$z>>2] = 0;
     }
    } while(0);
    $162 = HEAP32[$z>>2]|0;
    $163 = ($162|0)!=(0|0);
    $164 = $163&1;
    HEAP32[$rc>>2] = $164;
    $165 = HEAP32[$pVal>>2]|0;
    _sqlite3ValueFree($165);
    $166 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $166;
    $167 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($167|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $167 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($167|0);
}
function _isMatchOfColumn($pExpr,$peOp2) {
 $pExpr = $pExpr|0;
 $peOp2 = $peOp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aOp = 0, $i = 0, $pCol = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $aOp = sp + 16|0;
 $pList = sp + 8|0;
 $pCol = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $peOp2;
 ;HEAP32[$aOp>>2]=HEAP32[2608>>2]|0;HEAP32[$aOp+4>>2]=HEAP32[2608+4>>2]|0;HEAP32[$aOp+8>>2]=HEAP32[2608+8>>2]|0;HEAP32[$aOp+12>>2]=HEAP32[2608+12>>2]|0;HEAP32[$aOp+16>>2]=HEAP32[2608+16>>2]|0;HEAP32[$aOp+20>>2]=HEAP32[2608+20>>2]|0;HEAP32[$aOp+24>>2]=HEAP32[2608+24>>2]|0;HEAP32[$aOp+28>>2]=HEAP32[2608+28>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)!=(151);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pList>>2] = $9;
 $10 = HEAP32[$pList>>2]|0;
 $11 = ($10|0)==(0|0);
 if (!($11)) {
  $12 = HEAP32[$pList>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(2);
  if (!($14)) {
   $15 = HEAP32[$pList>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$pCol>>2] = $19;
   $20 = HEAP32[$pCol>>2]|0;
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = ($22|0)!=(152);
   if (!($23)) {
    $24 = HEAP32[$pCol>>2]|0;
    $25 = ((($24)) + 44|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ((($26)) + 42|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = $28&255;
    $30 = $29 & 16;
    $31 = ($30|0)!=(0);
    if ($31) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $32 = HEAP32[$i>>2]|0;
      $33 = ($32|0)<(4);
      if (!($33)) {
       label = 14;
       break;
      }
      $34 = HEAP32[$1>>2]|0;
      $35 = ((($34)) + 8|0);
      $36 = HEAP32[$35>>2]|0;
      $37 = HEAP32[$i>>2]|0;
      $38 = (($aOp) + ($37<<3)|0);
      $39 = HEAP32[$38>>2]|0;
      $40 = (_sqlite3_stricmp($36,$39)|0);
      $41 = ($40|0)==(0);
      $42 = HEAP32[$i>>2]|0;
      if ($41) {
       label = 12;
       break;
      }
      $47 = (($42) + 1)|0;
      HEAP32[$i>>2] = $47;
     }
     if ((label|0) == 12) {
      $43 = (($aOp) + ($42<<3)|0);
      $44 = ((($43)) + 4|0);
      $45 = HEAP8[$44>>0]|0;
      $46 = HEAP32[$2>>2]|0;
      HEAP8[$46>>0] = $45;
      HEAP32[$0>>2] = 1;
      $48 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($48|0);
     }
     else if ((label|0) == 14) {
      HEAP32[$0>>2] = 0;
      $48 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($48|0);
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _whereNthSubterm($pTerm,$N) {
 $pTerm = $pTerm|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pTerm;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 18|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = ($6|0)!=(1024);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  $9 = ($8|0)==(0);
  $10 = HEAP32[$1>>2]|0;
  $11 = $9 ? $10 : 0;
  HEAP32[$0>>2] = $11;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($8|0)<($16|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($18*48)|0)|0);
  HEAP32[$0>>2] = $24;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _whereCombineDisjuncts($pSrc,$pWC,$pOne,$pTwo) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $pOne = $pOne|0;
 $pTwo = $pTwo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $db = 0, $eOp = 0, $idxNew = 0, $op = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $eOp = sp + 32|0;
 $db = sp + 12|0;
 $pNew = sp + 8|0;
 $op = sp + 4|0;
 $idxNew = sp;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $pOne;
 HEAP32[$3>>2] = $pTwo;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 18|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 18|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $7 | $11;
 $13 = $12&65535;
 HEAP16[$eOp>>1] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 18|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 & 62;
 $19 = ($18|0)==(0);
 if ($19) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$3>>2]|0;
 $21 = ((($20)) + 18|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 62;
 $25 = ($24|0)==(0);
 if ($25) {
  STACKTOP = sp;return;
 }
 $26 = HEAP16[$eOp>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 26;
 $29 = HEAP16[$eOp>>1]|0;
 $30 = $29&65535;
 $31 = ($28|0)!=($30|0);
 if ($31) {
  $32 = HEAP16[$eOp>>1]|0;
  $33 = $32&65535;
  $34 = $33 & 38;
  $35 = HEAP16[$eOp>>1]|0;
  $36 = $35&65535;
  $37 = ($34|0)!=($36|0);
  if ($37) {
   STACKTOP = sp;return;
  }
 }
 $38 = HEAP32[$2>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = ((($39)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = ((($43)) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = (_sqlite3ExprCompare($41,$45,-1)|0);
 $47 = ($46|0)!=(0);
 if ($47) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$2>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = ((($53)) + 16|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (_sqlite3ExprCompare($51,$55,-1)|0);
 $57 = ($56|0)!=(0);
 if ($57) {
  STACKTOP = sp;return;
 }
 $58 = HEAP16[$eOp>>1]|0;
 $59 = $58&65535;
 $60 = HEAP16[$eOp>>1]|0;
 $61 = $60&65535;
 $62 = (($61) - 1)|0;
 $63 = $59 & $62;
 $64 = ($63|0)!=(0);
 do {
  if ($64) {
   $65 = HEAP16[$eOp>>1]|0;
   $66 = $65&65535;
   $67 = $66 & 24;
   $68 = ($67|0)!=(0);
   if ($68) {
    HEAP16[$eOp>>1] = 8;
    break;
   } else {
    HEAP16[$eOp>>1] = 32;
    break;
   }
  }
 } while(0);
 $69 = HEAP32[$1>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$71>>2]|0;
 HEAP32[$db>>2] = $72;
 $73 = HEAP32[$db>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = (_sqlite3ExprDup($73,$75,0)|0);
 HEAP32[$pNew>>2] = $76;
 $77 = HEAP32[$pNew>>2]|0;
 $78 = ($77|0)==(0|0);
 if ($78) {
  STACKTOP = sp;return;
 }
 HEAP32[$op>>2] = 79;
 while(1) {
  $79 = HEAP16[$eOp>>1]|0;
  $80 = $79&65535;
  $81 = HEAP32[$op>>2]|0;
  $82 = (($81) - 79)|0;
  $83 = 2 << $82;
  $84 = ($80|0)!=($83|0);
  $85 = HEAP32[$op>>2]|0;
  if (!($84)) {
   break;
  }
  $86 = (($85) + 1)|0;
  HEAP32[$op>>2] = $86;
 }
 $87 = $85&255;
 $88 = HEAP32[$pNew>>2]|0;
 HEAP8[$88>>0] = $87;
 $89 = HEAP32[$1>>2]|0;
 $90 = HEAP32[$pNew>>2]|0;
 $91 = (_whereClauseInsert($89,$90,3)|0);
 HEAP32[$idxNew>>2] = $91;
 $92 = HEAP32[$0>>2]|0;
 $93 = HEAP32[$1>>2]|0;
 $94 = HEAP32[$idxNew>>2]|0;
 _exprAnalyze($92,$93,$94);
 STACKTOP = sp;return;
}
function _sqlite3IsLikeFunction($db,$pExpr,$pIsNocase,$aWc) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $pIsNocase = $pIsNocase|0;
 $aWc = $aWc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pDef = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pIsNocase;
 HEAP32[$4>>2] = $aWc;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)!=(151);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = ((($13)) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(2);
   if (!($17)) {
    $18 = HEAP32[$1>>2]|0;
    $19 = HEAP32[$2>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = HEAP32[$2>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (_sqlite3Strlen30($24)|0);
    $26 = (_sqlite3FindFunction($18,$21,$25,2,1,0)|0);
    HEAP32[$pDef>>2] = $26;
    $27 = HEAP32[$pDef>>2]|0;
    $28 = ($27|0)==(0|0);
    if (!($28)) {
     $29 = HEAP32[$pDef>>2]|0;
     $30 = ((($29)) + 2|0);
     $31 = HEAP16[$30>>1]|0;
     $32 = $31&65535;
     $33 = $32 & 4;
     $34 = ($33|0)==(0);
     if (!($34)) {
      $35 = HEAP32[$4>>2]|0;
      $36 = HEAP32[$pDef>>2]|0;
      $37 = ((($36)) + 4|0);
      $38 = HEAP32[$37>>2]|0;
      ;HEAP8[$35>>0]=HEAP8[$38>>0]|0;HEAP8[$35+1>>0]=HEAP8[$38+1>>0]|0;HEAP8[$35+2>>0]=HEAP8[$38+2>>0]|0;
      $39 = HEAP32[$pDef>>2]|0;
      $40 = ((($39)) + 2|0);
      $41 = HEAP16[$40>>1]|0;
      $42 = $41&65535;
      $43 = $42 & 8;
      $44 = ($43|0)==(0);
      $45 = $44&1;
      $46 = HEAP32[$3>>2]|0;
      HEAP32[$46>>2] = $45;
      HEAP32[$0>>2] = 1;
      $47 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($47|0);
     }
    }
    HEAP32[$0>>2] = 0;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3VdbeGetBoundValue($v,$iVar,$aff) {
 $v = $v|0;
 $iVar = $iVar|0;
 $aff = $aff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMem = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $pMem = sp + 4|0;
 $pRet = sp;
 HEAP32[$1>>2] = $v;
 HEAP32[$2>>2] = $iVar;
 HEAP8[$3>>0] = $aff;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 60|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*40)|0)|0);
  HEAP32[$pMem>>2] = $11;
  $12 = HEAP32[$pMem>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = $15 & 1;
  $17 = (0)==($16|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3ValueNew($19)|0);
   HEAP32[$pRet>>2] = $20;
   $21 = HEAP32[$pRet>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$pRet>>2]|0;
    $24 = HEAP32[$pMem>>2]|0;
    (_sqlite3VdbeMemCopy($23,$24)|0);
    $25 = HEAP32[$pRet>>2]|0;
    $26 = HEAP8[$3>>0]|0;
    _sqlite3ValueApplyAffinity($25,$26,1);
   }
   $27 = HEAP32[$pRet>>2]|0;
   HEAP32[$0>>2] = $27;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3VdbeSetVarmask($v,$iVar) {
 $v = $v|0;
 $iVar = $iVar|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iVar;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(32);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 188|0);
  HEAP32[$5>>2] = -1;
  STACKTOP = sp;return;
 } else {
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = 1 << $7;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 188|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 | $8;
  HEAP32[$10>>2] = $12;
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeChangeP3($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = $3;
 STACKTOP = sp;return;
}
function _sqlite3WhereFindTerm($pWC,$iCur,$iColumn,$0,$1,$op,$pIdx) {
 $pWC = $pWC|0;
 $iCur = $iCur|0;
 $iColumn = $iColumn|0;
 $0 = $0|0;
 $1 = $1|0;
 $op = $op|0;
 $pIdx = $pIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pResult = 0, $scan = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 120|0;
 $6 = sp;
 $7 = sp + 116|0;
 $8 = sp + 112|0;
 $pResult = sp + 108|0;
 $p = sp + 104|0;
 $scan = sp + 8|0;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $iCur;
 HEAP32[$5>>2] = $iColumn;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $op;
 HEAP32[$8>>2] = $pIdx;
 HEAP32[$pResult>>2] = 0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$7>>2]|0;
 $17 = HEAP32[$8>>2]|0;
 $18 = (_whereScanInit($scan,$13,$14,$15,$16,$17)|0);
 HEAP32[$p>>2] = $18;
 $19 = HEAP32[$7>>2]|0;
 $20 = $19 & 130;
 HEAP32[$7>>2] = $20;
 while(1) {
  $21 = HEAP32[$p>>2]|0;
  $22 = ($21|0)!=(0|0);
  if (!($22)) {
   label = 10;
   break;
  }
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 32|0);
  $25 = $24;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = $6;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = $27 & $33;
  $38 = $30 & $36;
  $39 = ($37|0)==(0);
  $40 = ($38|0)==(0);
  $41 = $39 & $40;
  if ($41) {
   $42 = HEAP32[$p>>2]|0;
   $43 = ((($42)) + 32|0);
   $44 = $43;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = (($44) + 4)|0;
   $48 = $47;
   $49 = HEAP32[$48>>2]|0;
   $50 = ($46|0)==(0);
   $51 = ($49|0)==(0);
   $52 = $50 & $51;
   if ($52) {
    $53 = HEAP32[$p>>2]|0;
    $54 = ((($53)) + 18|0);
    $55 = HEAP16[$54>>1]|0;
    $56 = $55&65535;
    $57 = HEAP32[$7>>2]|0;
    $58 = $56 & $57;
    $59 = ($58|0)!=(0);
    if ($59) {
     label = 6;
     break;
    }
   }
   $61 = HEAP32[$pResult>>2]|0;
   $62 = ($61|0)==(0|0);
   if ($62) {
    $63 = HEAP32[$p>>2]|0;
    HEAP32[$pResult>>2] = $63;
   }
  }
  $64 = (_whereScanNext($scan)|0);
  HEAP32[$p>>2] = $64;
 }
 if ((label|0) == 6) {
  $60 = HEAP32[$p>>2]|0;
  HEAP32[$2>>2] = $60;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 10) {
  $65 = HEAP32[$pResult>>2]|0;
  HEAP32[$2>>2] = $65;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return (0)|0;
}
function _findIndexCol($pParse,$pList,$iBase,$pIdx,$iCol) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $iBase = $iBase|0;
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pColl = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $i = sp + 12|0;
 $zColl = sp + 8|0;
 $p = sp + 4|0;
 $pColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $iBase;
 HEAP32[$4>>2] = $pIdx;
 HEAP32[$5>>2] = $iCol;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = ((($7)) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<2)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$zColl>>2] = $11;
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($12|0)<($14|0);
  if (!($15)) {
   label = 10;
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + (($16*20)|0)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3ExprSkipCollate($21)|0);
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)==(152);
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = HEAP32[$5>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<1)|0);
   $36 = HEAP16[$35>>1]|0;
   $37 = $36 << 16 >> 16;
   $38 = ($30|0)==($37|0);
   if ($38) {
    $39 = HEAP32[$p>>2]|0;
    $40 = ((($39)) + 28|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$3>>2]|0;
    $43 = ($41|0)==($42|0);
    if ($43) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$i>>2]|0;
     $46 = HEAP32[$2>>2]|0;
     $47 = ((($46)) + 4|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (($48) + (($45*20)|0)|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (_sqlite3ExprCollSeq($44,$50)|0);
     HEAP32[$pColl>>2] = $51;
     $52 = HEAP32[$pColl>>2]|0;
     $53 = ($52|0)!=(0|0);
     if ($53) {
      $54 = HEAP32[$pColl>>2]|0;
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$zColl>>2]|0;
      $57 = (_sqlite3_stricmp($55,$56)|0);
      $58 = (0)==($57|0);
      if ($58) {
       label = 8;
       break;
      }
     }
    }
   }
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 if ((label|0) == 8) {
  $59 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $59;
  $62 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = -1;
  $62 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 return (0)|0;
}
function _indexColumnNotNull($pIdx,$iCol) {
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<1)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 HEAP32[$j>>2] = $9;
 $10 = HEAP32[$j>>2]|0;
 $11 = ($10|0)>=(0);
 $12 = HEAP32[$j>>2]|0;
 if ($11) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($12*24)|0)|0);
  $19 = ((($18)) + 20|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  HEAP32[$0>>2] = $21;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $22 = ($12|0)==(-1);
 if ($22) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _whereScanInit($pScan,$pWC,$iCur,$iColumn,$opMask,$pIdx) {
 $pScan = $pScan|0;
 $pWC = $pWC|0;
 $iCur = $iCur|0;
 $iColumn = $iColumn|0;
 $opMask = $opMask|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $j = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $pScan;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $iCur;
 HEAP32[$3>>2] = $iColumn;
 HEAP32[$4>>2] = $opMask;
 HEAP32[$5>>2] = $pIdx;
 HEAP32[$j>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$5>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  HEAP32[$j>>2] = $15;
  $16 = HEAP32[$j>>2]|0;
  $17 = HEAP32[$5>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<1)|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21 << 16 >> 16;
  HEAP32[$3>>2] = $22;
  $23 = HEAP32[$3>>2]|0;
  $24 = ($23|0)==(-2);
  if ($24) {
   $25 = HEAP32[$j>>2]|0;
   $26 = HEAP32[$5>>2]|0;
   $27 = ((($26)) + 40|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + (($25*20)|0)|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$0>>2]|0;
   $34 = ((($33)) + 12|0);
   HEAP32[$34>>2] = $32;
  }
 }
 $35 = HEAP32[$5>>2]|0;
 $36 = ($35|0)!=(0|0);
 $37 = HEAP32[$3>>2]|0;
 $38 = ($37|0)>=(0);
 $or$cond = $36 & $38;
 if ($or$cond) {
  $39 = HEAP32[$3>>2]|0;
  $40 = HEAP32[$5>>2]|0;
  $41 = ((($40)) + 12|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ((($42)) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($39*24)|0)|0);
  $46 = ((($45)) + 21|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = HEAP32[$0>>2]|0;
  $49 = ((($48)) + 16|0);
  HEAP8[$49>>0] = $47;
  $50 = HEAP32[$j>>2]|0;
  $51 = HEAP32[$5>>2]|0;
  $52 = ((($51)) + 32|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($50<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 8|0);
  HEAP32[$57>>2] = $55;
 } else {
  $58 = HEAP32[$0>>2]|0;
  $59 = ((($58)) + 16|0);
  HEAP8[$59>>0] = 0;
  $60 = HEAP32[$0>>2]|0;
  $61 = ((($60)) + 8|0);
  HEAP32[$61>>2] = 0;
 }
 $62 = HEAP32[$4>>2]|0;
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 20|0);
 HEAP32[$64>>2] = $62;
 $65 = HEAP32[$0>>2]|0;
 $66 = ((($65)) + 24|0);
 HEAP32[$66>>2] = 0;
 $67 = HEAP32[$2>>2]|0;
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 28|0);
 HEAP32[$69>>2] = $67;
 $70 = HEAP32[$3>>2]|0;
 $71 = $70&65535;
 $72 = HEAP32[$0>>2]|0;
 $73 = ((($72)) + 72|0);
 HEAP16[$73>>1] = $71;
 $74 = HEAP32[$0>>2]|0;
 $75 = ((($74)) + 17|0);
 HEAP8[$75>>0] = 1;
 $76 = HEAP32[$0>>2]|0;
 $77 = ((($76)) + 18|0);
 HEAP8[$77>>0] = 1;
 $78 = HEAP32[$0>>2]|0;
 $79 = (_whereScanNext($78)|0);
 STACKTOP = sp;return ($79|0);
}
function _whereScanNext($pScan) {
 $pScan = $pScan|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iColumn = 0, $iCur = 0, $j = 0, $k = 0, $pColl = 0, $pParse = 0, $pTerm = 0, $pWC = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $iCur = sp + 28|0;
 $iColumn = sp + 40|0;
 $pX = sp + 24|0;
 $pWC = sp + 20|0;
 $pTerm = sp + 16|0;
 $k = sp + 12|0;
 $j = sp + 8|0;
 $pColl = sp + 4|0;
 $pParse = sp;
 HEAP32[$1>>2] = $pScan;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$k>>2] = $4;
 L1: while(1) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 18|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 17|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($8|0)<=($12|0);
  if (!($13)) {
   label = 40;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 18|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($17) - 1)|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 28|0);
  $21 = (($20) + ($18<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$iCur>>2] = $22;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 18|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = (($26) - 1)|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 72|0);
  $30 = (($29) + ($27<<1)|0);
  $31 = HEAP16[$30>>1]|0;
  HEAP16[$iColumn>>1] = $31;
  $32 = HEAP16[$iColumn>>1]|0;
  $33 = $32 << 16 >> 16;
  $34 = ($33|0)==(-2);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = ($37|0)==(0|0);
   if ($38) {
    label = 5;
    break;
   }
  }
  while(1) {
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$pWC>>2] = $41;
   $42 = ($41|0)!=(0|0);
   if (!($42)) {
    break;
   }
   $43 = HEAP32[$pWC>>2]|0;
   $44 = ((($43)) + 20|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$k>>2]|0;
   $47 = (($45) + (($46*48)|0)|0);
   HEAP32[$pTerm>>2] = $47;
   while(1) {
    $48 = HEAP32[$k>>2]|0;
    $49 = HEAP32[$pWC>>2]|0;
    $50 = ((($49)) + 12|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ($48|0)<($51|0);
    if (!($52)) {
     break;
    }
    $53 = HEAP32[$pTerm>>2]|0;
    $54 = ((($53)) + 8|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = HEAP32[$iCur>>2]|0;
    $57 = ($55|0)==($56|0);
    do {
     if ($57) {
      $58 = HEAP32[$pTerm>>2]|0;
      $59 = ((($58)) + 12|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = HEAP16[$iColumn>>1]|0;
      $62 = $61 << 16 >> 16;
      $63 = ($60|0)==($62|0);
      if ($63) {
       $64 = HEAP16[$iColumn>>1]|0;
       $65 = $64 << 16 >> 16;
       $66 = ($65|0)!=(-2);
       if (!($66)) {
        $67 = HEAP32[$pTerm>>2]|0;
        $68 = HEAP32[$67>>2]|0;
        $69 = ((($68)) + 12|0);
        $70 = HEAP32[$69>>2]|0;
        $71 = HEAP32[$1>>2]|0;
        $72 = ((($71)) + 12|0);
        $73 = HEAP32[$72>>2]|0;
        $74 = HEAP32[$iCur>>2]|0;
        $75 = (_sqlite3ExprCompare($70,$73,$74)|0);
        $76 = ($75|0)==(0);
        if (!($76)) {
         break;
        }
       }
       $77 = HEAP32[$1>>2]|0;
       $78 = ((($77)) + 18|0);
       $79 = HEAP8[$78>>0]|0;
       $80 = $79&255;
       $81 = ($80|0)<=(1);
       if (!($81)) {
        $82 = HEAP32[$pTerm>>2]|0;
        $83 = HEAP32[$82>>2]|0;
        $84 = ((($83)) + 4|0);
        $85 = HEAP32[$84>>2]|0;
        $86 = $85 & 1;
        $87 = ($86|0)!=(0);
        if ($87) {
         break;
        }
       }
       $88 = HEAP32[$pTerm>>2]|0;
       $89 = ((($88)) + 18|0);
       $90 = HEAP16[$89>>1]|0;
       $91 = $90&65535;
       $92 = $91 & 2048;
       $93 = ($92|0)!=(0);
       if ($93) {
        $94 = HEAP32[$1>>2]|0;
        $95 = ((($94)) + 17|0);
        $96 = HEAP8[$95>>0]|0;
        $97 = $96&255;
        $98 = ($97|0)<(11);
        if ($98) {
         $99 = HEAP32[$pTerm>>2]|0;
         $100 = HEAP32[$99>>2]|0;
         $101 = ((($100)) + 16|0);
         $102 = HEAP32[$101>>2]|0;
         $103 = (_sqlite3ExprSkipCollate($102)|0);
         HEAP32[$pX>>2] = $103;
         $104 = HEAP8[$103>>0]|0;
         $105 = $104&255;
         $106 = ($105|0)==(152);
         if ($106) {
          HEAP32[$j>>2] = 0;
          while(1) {
           $107 = HEAP32[$j>>2]|0;
           $108 = HEAP32[$1>>2]|0;
           $109 = ((($108)) + 17|0);
           $110 = HEAP8[$109>>0]|0;
           $111 = $110&255;
           $112 = ($107|0)<($111|0);
           if (!($112)) {
            break;
           }
           $113 = HEAP32[$j>>2]|0;
           $114 = HEAP32[$1>>2]|0;
           $115 = ((($114)) + 28|0);
           $116 = (($115) + ($113<<2)|0);
           $117 = HEAP32[$116>>2]|0;
           $118 = HEAP32[$pX>>2]|0;
           $119 = ((($118)) + 28|0);
           $120 = HEAP32[$119>>2]|0;
           $121 = ($117|0)==($120|0);
           if ($121) {
            $122 = HEAP32[$j>>2]|0;
            $123 = HEAP32[$1>>2]|0;
            $124 = ((($123)) + 72|0);
            $125 = (($124) + ($122<<1)|0);
            $126 = HEAP16[$125>>1]|0;
            $127 = $126 << 16 >> 16;
            $128 = HEAP32[$pX>>2]|0;
            $129 = ((($128)) + 32|0);
            $130 = HEAP16[$129>>1]|0;
            $131 = $130 << 16 >> 16;
            $132 = ($127|0)==($131|0);
            if ($132) {
             break;
            }
           }
           $133 = HEAP32[$j>>2]|0;
           $134 = (($133) + 1)|0;
           HEAP32[$j>>2] = $134;
          }
          $135 = HEAP32[$j>>2]|0;
          $136 = HEAP32[$1>>2]|0;
          $137 = ((($136)) + 17|0);
          $138 = HEAP8[$137>>0]|0;
          $139 = $138&255;
          $140 = ($135|0)==($139|0);
          if ($140) {
           $141 = HEAP32[$pX>>2]|0;
           $142 = ((($141)) + 28|0);
           $143 = HEAP32[$142>>2]|0;
           $144 = HEAP32[$j>>2]|0;
           $145 = HEAP32[$1>>2]|0;
           $146 = ((($145)) + 28|0);
           $147 = (($146) + ($144<<2)|0);
           HEAP32[$147>>2] = $143;
           $148 = HEAP32[$pX>>2]|0;
           $149 = ((($148)) + 32|0);
           $150 = HEAP16[$149>>1]|0;
           $151 = HEAP32[$j>>2]|0;
           $152 = HEAP32[$1>>2]|0;
           $153 = ((($152)) + 72|0);
           $154 = (($153) + ($151<<1)|0);
           HEAP16[$154>>1] = $150;
           $155 = HEAP32[$1>>2]|0;
           $156 = ((($155)) + 17|0);
           $157 = HEAP8[$156>>0]|0;
           $158 = (($157) + 1)<<24>>24;
           HEAP8[$156>>0] = $158;
          }
         }
        }
       }
       $159 = HEAP32[$pTerm>>2]|0;
       $160 = ((($159)) + 18|0);
       $161 = HEAP16[$160>>1]|0;
       $162 = $161&65535;
       $163 = HEAP32[$1>>2]|0;
       $164 = ((($163)) + 20|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = $162 & $165;
       $167 = ($166|0)!=(0);
       if ($167) {
        $168 = HEAP32[$1>>2]|0;
        $169 = ((($168)) + 8|0);
        $170 = HEAP32[$169>>2]|0;
        $171 = ($170|0)!=(0|0);
        if ($171) {
         $172 = HEAP32[$pTerm>>2]|0;
         $173 = ((($172)) + 18|0);
         $174 = HEAP16[$173>>1]|0;
         $175 = $174&65535;
         $176 = $175 & 256;
         $177 = ($176|0)==(0);
         if ($177) {
          $178 = HEAP32[$pWC>>2]|0;
          $179 = HEAP32[$178>>2]|0;
          $180 = HEAP32[$179>>2]|0;
          HEAP32[$pParse>>2] = $180;
          $181 = HEAP32[$pTerm>>2]|0;
          $182 = HEAP32[$181>>2]|0;
          HEAP32[$pX>>2] = $182;
          $183 = HEAP32[$pX>>2]|0;
          $184 = HEAP32[$1>>2]|0;
          $185 = ((($184)) + 16|0);
          $186 = HEAP8[$185>>0]|0;
          $187 = (_sqlite3IndexAffinityOk($183,$186)|0);
          $188 = ($187|0)!=(0);
          if (!($188)) {
           break;
          }
          $189 = HEAP32[$pParse>>2]|0;
          $190 = HEAP32[$pX>>2]|0;
          $191 = ((($190)) + 12|0);
          $192 = HEAP32[$191>>2]|0;
          $193 = HEAP32[$pX>>2]|0;
          $194 = ((($193)) + 16|0);
          $195 = HEAP32[$194>>2]|0;
          $196 = (_sqlite3BinaryCompareCollSeq($189,$192,$195)|0);
          HEAP32[$pColl>>2] = $196;
          $197 = HEAP32[$pColl>>2]|0;
          $198 = ($197|0)==(0|0);
          if ($198) {
           $199 = HEAP32[$pParse>>2]|0;
           $200 = HEAP32[$199>>2]|0;
           $201 = ((($200)) + 8|0);
           $202 = HEAP32[$201>>2]|0;
           HEAP32[$pColl>>2] = $202;
          }
          $203 = HEAP32[$pColl>>2]|0;
          $204 = HEAP32[$203>>2]|0;
          $205 = HEAP32[$1>>2]|0;
          $206 = ((($205)) + 8|0);
          $207 = HEAP32[$206>>2]|0;
          $208 = (_sqlite3_stricmp($204,$207)|0);
          $209 = ($208|0)!=(0);
          if ($209) {
           break;
          }
         }
        }
        $210 = HEAP32[$pTerm>>2]|0;
        $211 = ((($210)) + 18|0);
        $212 = HEAP16[$211>>1]|0;
        $213 = $212&65535;
        $214 = $213 & 130;
        $215 = ($214|0)!=(0);
        if (!($215)) {
         label = 36;
         break L1;
        }
        $216 = HEAP32[$pTerm>>2]|0;
        $217 = HEAP32[$216>>2]|0;
        $218 = ((($217)) + 16|0);
        $219 = HEAP32[$218>>2]|0;
        HEAP32[$pX>>2] = $219;
        $220 = HEAP8[$219>>0]|0;
        $221 = $220&255;
        $222 = ($221|0)==(152);
        if (!($222)) {
         label = 36;
         break L1;
        }
        $223 = HEAP32[$pX>>2]|0;
        $224 = ((($223)) + 28|0);
        $225 = HEAP32[$224>>2]|0;
        $226 = HEAP32[$1>>2]|0;
        $227 = ((($226)) + 28|0);
        $228 = HEAP32[$227>>2]|0;
        $229 = ($225|0)==($228|0);
        if (!($229)) {
         label = 36;
         break L1;
        }
        $230 = HEAP32[$pX>>2]|0;
        $231 = ((($230)) + 32|0);
        $232 = HEAP16[$231>>1]|0;
        $233 = $232 << 16 >> 16;
        $234 = HEAP32[$1>>2]|0;
        $235 = ((($234)) + 72|0);
        $236 = HEAP16[$235>>1]|0;
        $237 = $236 << 16 >> 16;
        $238 = ($233|0)==($237|0);
        if (!($238)) {
         label = 36;
         break L1;
        }
       }
      }
     }
    } while(0);
    $244 = HEAP32[$k>>2]|0;
    $245 = (($244) + 1)|0;
    HEAP32[$k>>2] = $245;
    $246 = HEAP32[$pTerm>>2]|0;
    $247 = ((($246)) + 48|0);
    HEAP32[$pTerm>>2] = $247;
   }
   $248 = HEAP32[$1>>2]|0;
   $249 = ((($248)) + 4|0);
   $250 = HEAP32[$249>>2]|0;
   $251 = ((($250)) + 4|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = HEAP32[$1>>2]|0;
   $254 = ((($253)) + 4|0);
   HEAP32[$254>>2] = $252;
   HEAP32[$k>>2] = 0;
  }
  $255 = HEAP32[$1>>2]|0;
  $256 = HEAP32[$255>>2]|0;
  $257 = HEAP32[$1>>2]|0;
  $258 = ((($257)) + 4|0);
  HEAP32[$258>>2] = $256;
  HEAP32[$k>>2] = 0;
  $259 = HEAP32[$1>>2]|0;
  $260 = ((($259)) + 18|0);
  $261 = HEAP8[$260>>0]|0;
  $262 = (($261) + 1)<<24>>24;
  HEAP8[$260>>0] = $262;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 else if ((label|0) == 36) {
  $239 = HEAP32[$k>>2]|0;
  $240 = (($239) + 1)|0;
  $241 = HEAP32[$1>>2]|0;
  $242 = ((($241)) + 24|0);
  HEAP32[$242>>2] = $240;
  $243 = HEAP32[$pTerm>>2]|0;
  HEAP32[$0>>2] = $243;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 else if ((label|0) == 40) {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 return (0)|0;
}
function _columnsInIndex($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $m = 0, $or$cond = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $m = sp;
 $j = sp + 12|0;
 $x = sp + 8|0;
 HEAP32[$0>>2] = $pIdx;
 $1 = $m;
 $2 = $1;
 HEAP32[$2>>2] = 0;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = (($8) - 1)|0;
 HEAP32[$j>>2] = $9;
 while(1) {
  $10 = HEAP32[$j>>2]|0;
  $11 = ($10|0)>=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$j>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  HEAP32[$x>>2] = $18;
  $19 = HEAP32[$x>>2]|0;
  $20 = ($19|0)>=(0);
  $21 = HEAP32[$x>>2]|0;
  $22 = ($21|0)<(63);
  $or$cond = $20 & $22;
  if ($or$cond) {
   $23 = HEAP32[$x>>2]|0;
   $24 = (_bitshift64Shl(1,0,($23|0))|0);
   $25 = tempRet0;
   $26 = $m;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = $28 | $24;
   $33 = $31 | $25;
   $34 = $m;
   $35 = $34;
   HEAP32[$35>>2] = $32;
   $36 = (($34) + 4)|0;
   $37 = $36;
   HEAP32[$37>>2] = $33;
  }
  $38 = HEAP32[$j>>2]|0;
  $39 = (($38) + -1)|0;
  HEAP32[$j>>2] = $39;
 }
 $40 = $m;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + 4)|0;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 tempRet0 = ($45);
 STACKTOP = sp;return ($42|0);
}
function _whereLoopAddVirtual($pBuilder,$0,$1,$2,$3) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0;
 var $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0;
 var $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0;
 var $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0.0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0;
 var $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0;
 var $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0;
 var $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iPhase = 0, $iTerm = 0, $j = 0, $mxTerm = 0, $nConstraint = 0, $or$cond = 0, $or$cond3 = 0, $pIdxCons = 0, $pIdxInfo = 0, $pNew = 0, $pParse = 0;
 var $pSrc = 0, $pTab = 0, $pTerm = 0, $pUsage = 0, $pWC = 0, $pWInfo = 0, $rc = 0, $seenIn = 0, $seenVar = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp + 16|0;
 $4 = sp + 104|0;
 $5 = sp + 100|0;
 $6 = sp + 8|0;
 $7 = sp;
 $pWInfo = sp + 96|0;
 $pParse = sp + 92|0;
 $pWC = sp + 88|0;
 $pSrc = sp + 84|0;
 $pTab = sp + 80|0;
 $db = sp + 76|0;
 $pIdxInfo = sp + 72|0;
 $pIdxCons = sp + 68|0;
 $pUsage = sp + 64|0;
 $pTerm = sp + 60|0;
 $i = sp + 56|0;
 $j = sp + 52|0;
 $iTerm = sp + 48|0;
 $mxTerm = sp + 44|0;
 $nConstraint = sp + 40|0;
 $seenIn = sp + 36|0;
 $seenVar = sp + 32|0;
 $iPhase = sp + 28|0;
 $pNew = sp + 24|0;
 $rc = sp + 20|0;
 HEAP32[$5>>2] = $pBuilder;
 $8 = $6;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 $12 = $7;
 $13 = $12;
 HEAP32[$13>>2] = $2;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $3;
 HEAP32[$seenIn>>2] = 0;
 HEAP32[$seenVar>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pWInfo>>2] = $17;
 $18 = HEAP32[$pWInfo>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pParse>>2] = $19;
 $20 = HEAP32[$pParse>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$db>>2] = $21;
 $22 = HEAP32[$5>>2]|0;
 $23 = ((($22)) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$pWC>>2] = $24;
 $25 = HEAP32[$5>>2]|0;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pNew>>2] = $27;
 $28 = HEAP32[$pNew>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = HEAP32[$pWInfo>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ((($34)) + 8|0);
 $36 = (($35) + (($31*72)|0)|0);
 HEAP32[$pSrc>>2] = $36;
 $37 = HEAP32[$pSrc>>2]|0;
 $38 = ((($37)) + 16|0);
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$pTab>>2] = $39;
 $40 = HEAP32[$pParse>>2]|0;
 $41 = HEAP32[$pWC>>2]|0;
 $42 = $7;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$pSrc>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 $50 = ((($49)) + 8|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (_allocateIndexInfo($40,$41,$44,$47,$48,$51)|0);
 HEAP32[$pIdxInfo>>2] = $52;
 $53 = HEAP32[$pIdxInfo>>2]|0;
 $54 = ($53|0)==(0|0);
 if ($54) {
  HEAP32[$4>>2] = 7;
  $470 = HEAP32[$4>>2]|0;
  STACKTOP = sp;return ($470|0);
 }
 $55 = HEAP32[$pNew>>2]|0;
 $56 = $55;
 $57 = $56;
 HEAP32[$57>>2] = 0;
 $58 = (($56) + 4)|0;
 $59 = $58;
 HEAP32[$59>>2] = 0;
 $60 = HEAP32[$pNew>>2]|0;
 $61 = ((($60)) + 18|0);
 HEAP16[$61>>1] = 0;
 $62 = HEAP32[$pNew>>2]|0;
 $63 = ((($62)) + 36|0);
 HEAP32[$63>>2] = 1024;
 $64 = HEAP32[$pNew>>2]|0;
 $65 = ((($64)) + 40|0);
 HEAP16[$65>>1] = 0;
 $66 = HEAP32[$pNew>>2]|0;
 $67 = ((($66)) + 24|0);
 $68 = ((($67)) + 4|0);
 HEAP8[$68>>0] = 0;
 $69 = HEAP32[$pIdxInfo>>2]|0;
 $70 = ((($69)) + 16|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$pUsage>>2] = $71;
 $72 = HEAP32[$pIdxInfo>>2]|0;
 $73 = HEAP32[$72>>2]|0;
 HEAP32[$nConstraint>>2] = $73;
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$pNew>>2]|0;
 $76 = HEAP32[$nConstraint>>2]|0;
 $77 = (_whereLoopResize($74,$75,$76)|0);
 $78 = ($77|0)!=(0);
 if ($78) {
  $79 = HEAP32[$db>>2]|0;
  $80 = HEAP32[$pIdxInfo>>2]|0;
  _sqlite3DbFree($79,$80);
  HEAP32[$4>>2] = 7;
  $470 = HEAP32[$4>>2]|0;
  STACKTOP = sp;return ($470|0);
 }
 HEAP32[$iPhase>>2] = 0;
 L9: while(1) {
  $81 = HEAP32[$iPhase>>2]|0;
  $82 = ($81|0)<=(3);
  if (!($82)) {
   break;
  }
  $83 = HEAP32[$seenIn>>2]|0;
  $84 = ($83|0)!=(0);
  if (!($84)) {
   $85 = HEAP32[$iPhase>>2]|0;
   $86 = $85 & 1;
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = HEAP32[$iPhase>>2]|0;
    $89 = (($88) + 1)|0;
    HEAP32[$iPhase>>2] = $89;
    $90 = HEAP32[$iPhase>>2]|0;
    $91 = ($90|0)>(3);
    if ($91) {
     break;
    }
   }
  }
  $92 = HEAP32[$seenVar>>2]|0;
  $93 = ($92|0)==(0);
  $94 = HEAP32[$iPhase>>2]|0;
  $95 = ($94|0)>(1);
  $or$cond = $93 & $95;
  if ($or$cond) {
   break;
  }
  $96 = HEAP32[$pIdxInfo>>2]|0;
  $97 = ((($96)) + 4|0);
  $98 = HEAP32[$97>>2]|0;
  HEAP32[$pIdxCons>>2] = $98;
  HEAP32[$i>>2] = 0;
  while(1) {
   $99 = HEAP32[$i>>2]|0;
   $100 = HEAP32[$pIdxInfo>>2]|0;
   $101 = HEAP32[$100>>2]|0;
   $102 = ($99|0)<($101|0);
   if (!($102)) {
    break;
   }
   $103 = HEAP32[$pIdxCons>>2]|0;
   $104 = ((($103)) + 8|0);
   $105 = HEAP32[$104>>2]|0;
   HEAP32[$j>>2] = $105;
   $106 = HEAP32[$j>>2]|0;
   $107 = HEAP32[$pWC>>2]|0;
   $108 = ((($107)) + 20|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + (($106*48)|0)|0);
   HEAP32[$pTerm>>2] = $110;
   $111 = HEAP32[$iPhase>>2]|0;
   L20: do {
    switch ($111|0) {
    case 0:  {
     $112 = HEAP32[$pIdxCons>>2]|0;
     $113 = ((($112)) + 5|0);
     HEAP8[$113>>0] = 0;
     $114 = HEAP32[$pTerm>>2]|0;
     $115 = ((($114)) + 18|0);
     $116 = HEAP16[$115>>1]|0;
     $117 = $116&65535;
     $118 = $117 & 1;
     $119 = ($118|0)!=(0);
     if ($119) {
      HEAP32[$seenIn>>2] = 1;
     }
     $120 = HEAP32[$pTerm>>2]|0;
     $121 = ((($120)) + 32|0);
     $122 = $121;
     $123 = $122;
     $124 = HEAP32[$123>>2]|0;
     $125 = (($122) + 4)|0;
     $126 = $125;
     $127 = HEAP32[$126>>2]|0;
     $128 = $6;
     $129 = $128;
     $130 = HEAP32[$129>>2]|0;
     $131 = (($128) + 4)|0;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = $130 ^ -1;
     $135 = $133 ^ -1;
     $136 = $124 & $134;
     $137 = $127 & $135;
     $138 = ($136|0)!=(0);
     $139 = ($137|0)!=(0);
     $140 = $138 | $139;
     if ($140) {
      HEAP32[$seenVar>>2] = 1;
      break L20;
     }
     $141 = HEAP32[$pTerm>>2]|0;
     $142 = ((($141)) + 18|0);
     $143 = HEAP16[$142>>1]|0;
     $144 = $143&65535;
     $145 = $144 & 1;
     $146 = ($145|0)==(0);
     if ($146) {
      $147 = HEAP32[$pIdxCons>>2]|0;
      $148 = ((($147)) + 5|0);
      HEAP8[$148>>0] = 1;
     }
     break;
    }
    case 1:  {
     $149 = HEAP32[$pTerm>>2]|0;
     $150 = ((($149)) + 32|0);
     $151 = $150;
     $152 = $151;
     $153 = HEAP32[$152>>2]|0;
     $154 = (($151) + 4)|0;
     $155 = $154;
     $156 = HEAP32[$155>>2]|0;
     $157 = $6;
     $158 = $157;
     $159 = HEAP32[$158>>2]|0;
     $160 = (($157) + 4)|0;
     $161 = $160;
     $162 = HEAP32[$161>>2]|0;
     $163 = $159 ^ -1;
     $164 = $162 ^ -1;
     $165 = $153 & $163;
     $166 = $156 & $164;
     $167 = ($165|0)==(0);
     $168 = ($166|0)==(0);
     $169 = $167 & $168;
     $170 = $169&1;
     $171 = $170&255;
     $172 = HEAP32[$pIdxCons>>2]|0;
     $173 = ((($172)) + 5|0);
     HEAP8[$173>>0] = $171;
     break;
    }
    case 2:  {
     $174 = HEAP32[$pTerm>>2]|0;
     $175 = ((($174)) + 18|0);
     $176 = HEAP16[$175>>1]|0;
     $177 = $176&65535;
     $178 = $177 & 1;
     $179 = ($178|0)==(0);
     $180 = $179&1;
     $181 = $180&255;
     $182 = HEAP32[$pIdxCons>>2]|0;
     $183 = ((($182)) + 5|0);
     HEAP8[$183>>0] = $181;
     break;
    }
    default: {
     $184 = HEAP32[$pIdxCons>>2]|0;
     $185 = ((($184)) + 5|0);
     HEAP8[$185>>0] = 1;
    }
    }
   } while(0);
   $186 = HEAP32[$i>>2]|0;
   $187 = (($186) + 1)|0;
   HEAP32[$i>>2] = $187;
   $188 = HEAP32[$pIdxCons>>2]|0;
   $189 = ((($188)) + 12|0);
   HEAP32[$pIdxCons>>2] = $189;
  }
  $190 = HEAP32[$pUsage>>2]|0;
  $191 = HEAP32[$pIdxInfo>>2]|0;
  $192 = HEAP32[$191>>2]|0;
  $193 = $192<<3;
  _memset(($190|0),0,($193|0))|0;
  $194 = HEAP32[$pIdxInfo>>2]|0;
  $195 = ((($194)) + 28|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = ($196|0)!=(0);
  if ($197) {
   $198 = HEAP32[$pIdxInfo>>2]|0;
   $199 = ((($198)) + 24|0);
   $200 = HEAP32[$199>>2]|0;
   _sqlite3_free($200);
  }
  $201 = HEAP32[$pIdxInfo>>2]|0;
  $202 = ((($201)) + 24|0);
  HEAP32[$202>>2] = 0;
  $203 = HEAP32[$pIdxInfo>>2]|0;
  $204 = ((($203)) + 20|0);
  HEAP32[$204>>2] = 0;
  $205 = HEAP32[$pIdxInfo>>2]|0;
  $206 = ((($205)) + 28|0);
  HEAP32[$206>>2] = 0;
  $207 = HEAP32[$pIdxInfo>>2]|0;
  $208 = ((($207)) + 32|0);
  HEAP32[$208>>2] = 0;
  $209 = HEAP32[$pIdxInfo>>2]|0;
  $210 = ((($209)) + 40|0);
  HEAPF64[$210>>3] = 4.9999999999999998E+98;
  $211 = HEAP32[$pIdxInfo>>2]|0;
  $212 = ((($211)) + 48|0);
  $213 = $212;
  $214 = $213;
  HEAP32[$214>>2] = 25;
  $215 = (($213) + 4)|0;
  $216 = $215;
  HEAP32[$216>>2] = 0;
  $217 = HEAP32[$pIdxInfo>>2]|0;
  $218 = ((($217)) + 56|0);
  HEAP32[$218>>2] = 0;
  $219 = HEAP32[$pSrc>>2]|0;
  $220 = ((($219)) + 56|0);
  $221 = $220;
  $222 = $221;
  $223 = HEAP32[$222>>2]|0;
  $224 = (($221) + 4)|0;
  $225 = $224;
  $226 = HEAP32[$225>>2]|0;
  $227 = HEAP32[$pIdxInfo>>2]|0;
  $228 = ((($227)) + 64|0);
  $229 = $228;
  $230 = $229;
  HEAP32[$230>>2] = $223;
  $231 = (($229) + 4)|0;
  $232 = $231;
  HEAP32[$232>>2] = $226;
  $233 = HEAP32[$pParse>>2]|0;
  $234 = HEAP32[$pTab>>2]|0;
  $235 = HEAP32[$pIdxInfo>>2]|0;
  $236 = (_vtabBestIndex($233,$234,$235)|0);
  HEAP32[$rc>>2] = $236;
  $237 = HEAP32[$rc>>2]|0;
  $238 = ($237|0)!=(0);
  if ($238) {
   break;
  }
  $239 = HEAP32[$pIdxInfo>>2]|0;
  $240 = ((($239)) + 4|0);
  $241 = HEAP32[$240>>2]|0;
  HEAP32[$pIdxCons>>2] = $241;
  $242 = $6;
  $243 = $242;
  $244 = HEAP32[$243>>2]|0;
  $245 = (($242) + 4)|0;
  $246 = $245;
  $247 = HEAP32[$246>>2]|0;
  $248 = HEAP32[$pNew>>2]|0;
  $249 = $248;
  $250 = $249;
  HEAP32[$250>>2] = $244;
  $251 = (($249) + 4)|0;
  $252 = $251;
  HEAP32[$252>>2] = $247;
  HEAP32[$mxTerm>>2] = -1;
  HEAP32[$i>>2] = 0;
  while(1) {
   $253 = HEAP32[$i>>2]|0;
   $254 = HEAP32[$nConstraint>>2]|0;
   $255 = ($253|0)<($254|0);
   if (!($255)) {
    break;
   }
   $256 = HEAP32[$i>>2]|0;
   $257 = HEAP32[$pNew>>2]|0;
   $258 = ((($257)) + 48|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = (($259) + ($256<<2)|0);
   HEAP32[$260>>2] = 0;
   $261 = HEAP32[$i>>2]|0;
   $262 = (($261) + 1)|0;
   HEAP32[$i>>2] = $262;
  }
  $263 = HEAP32[$pNew>>2]|0;
  $264 = ((($263)) + 24|0);
  $265 = ((($264)) + 6|0);
  HEAP16[$265>>1] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $266 = HEAP32[$i>>2]|0;
   $267 = HEAP32[$nConstraint>>2]|0;
   $268 = ($266|0)<($267|0);
   if (!($268)) {
    break;
   }
   $269 = HEAP32[$i>>2]|0;
   $270 = HEAP32[$pUsage>>2]|0;
   $271 = (($270) + ($269<<3)|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = (($272) - 1)|0;
   HEAP32[$iTerm>>2] = $273;
   $274 = ($273|0)>=(0);
   if ($274) {
    $275 = HEAP32[$pIdxCons>>2]|0;
    $276 = ((($275)) + 8|0);
    $277 = HEAP32[$276>>2]|0;
    HEAP32[$j>>2] = $277;
    $278 = HEAP32[$iTerm>>2]|0;
    $279 = HEAP32[$nConstraint>>2]|0;
    $280 = ($278|0)>=($279|0);
    $281 = HEAP32[$j>>2]|0;
    $282 = ($281|0)<(0);
    $or$cond3 = $280 | $282;
    if ($or$cond3) {
     label = 36;
     break L9;
    }
    $283 = HEAP32[$j>>2]|0;
    $284 = HEAP32[$pWC>>2]|0;
    $285 = ((($284)) + 12|0);
    $286 = HEAP32[$285>>2]|0;
    $287 = ($283|0)>=($286|0);
    if ($287) {
     label = 36;
     break L9;
    }
    $288 = HEAP32[$iTerm>>2]|0;
    $289 = HEAP32[$pNew>>2]|0;
    $290 = ((($289)) + 48|0);
    $291 = HEAP32[$290>>2]|0;
    $292 = (($291) + ($288<<2)|0);
    $293 = HEAP32[$292>>2]|0;
    $294 = ($293|0)!=(0|0);
    if ($294) {
     label = 36;
     break L9;
    }
    $298 = HEAP32[$j>>2]|0;
    $299 = HEAP32[$pWC>>2]|0;
    $300 = ((($299)) + 20|0);
    $301 = HEAP32[$300>>2]|0;
    $302 = (($301) + (($298*48)|0)|0);
    HEAP32[$pTerm>>2] = $302;
    $303 = HEAP32[$pTerm>>2]|0;
    $304 = ((($303)) + 32|0);
    $305 = $304;
    $306 = $305;
    $307 = HEAP32[$306>>2]|0;
    $308 = (($305) + 4)|0;
    $309 = $308;
    $310 = HEAP32[$309>>2]|0;
    $311 = HEAP32[$pNew>>2]|0;
    $312 = $311;
    $313 = $312;
    $314 = HEAP32[$313>>2]|0;
    $315 = (($312) + 4)|0;
    $316 = $315;
    $317 = HEAP32[$316>>2]|0;
    $318 = $314 | $307;
    $319 = $317 | $310;
    $320 = $311;
    $321 = $320;
    HEAP32[$321>>2] = $318;
    $322 = (($320) + 4)|0;
    $323 = $322;
    HEAP32[$323>>2] = $319;
    $324 = HEAP32[$pTerm>>2]|0;
    $325 = HEAP32[$iTerm>>2]|0;
    $326 = HEAP32[$pNew>>2]|0;
    $327 = ((($326)) + 48|0);
    $328 = HEAP32[$327>>2]|0;
    $329 = (($328) + ($325<<2)|0);
    HEAP32[$329>>2] = $324;
    $330 = HEAP32[$iTerm>>2]|0;
    $331 = HEAP32[$mxTerm>>2]|0;
    $332 = ($330|0)>($331|0);
    if ($332) {
     $333 = HEAP32[$iTerm>>2]|0;
     HEAP32[$mxTerm>>2] = $333;
    }
    $334 = HEAP32[$iTerm>>2]|0;
    $335 = ($334|0)<(16);
    if ($335) {
     $336 = HEAP32[$i>>2]|0;
     $337 = HEAP32[$pUsage>>2]|0;
     $338 = (($337) + ($336<<3)|0);
     $339 = ((($338)) + 4|0);
     $340 = HEAP8[$339>>0]|0;
     $341 = $340&255;
     $342 = ($341|0)!=(0);
     if ($342) {
      $343 = HEAP32[$iTerm>>2]|0;
      $344 = 1 << $343;
      $345 = HEAP32[$pNew>>2]|0;
      $346 = ((($345)) + 24|0);
      $347 = ((($346)) + 6|0);
      $348 = HEAP16[$347>>1]|0;
      $349 = $348&65535;
      $350 = $349 | $344;
      $351 = $350&65535;
      HEAP16[$347>>1] = $351;
     }
    }
    $352 = HEAP32[$pTerm>>2]|0;
    $353 = ((($352)) + 18|0);
    $354 = HEAP16[$353>>1]|0;
    $355 = $354&65535;
    $356 = $355 & 1;
    $357 = ($356|0)!=(0);
    if ($357) {
     $358 = HEAP32[$i>>2]|0;
     $359 = HEAP32[$pUsage>>2]|0;
     $360 = (($359) + ($358<<3)|0);
     $361 = ((($360)) + 4|0);
     $362 = HEAP8[$361>>0]|0;
     $363 = $362&255;
     $364 = ($363|0)==(0);
     if ($364) {
      break;
     }
     $365 = HEAP32[$pIdxInfo>>2]|0;
     $366 = ((($365)) + 32|0);
     HEAP32[$366>>2] = 0;
     $367 = HEAP32[$pIdxInfo>>2]|0;
     $368 = ((($367)) + 56|0);
     $369 = HEAP32[$368>>2]|0;
     $370 = $369 & -2;
     HEAP32[$368>>2] = $370;
    }
   }
   $371 = HEAP32[$i>>2]|0;
   $372 = (($371) + 1)|0;
   HEAP32[$i>>2] = $372;
   $373 = HEAP32[$pIdxCons>>2]|0;
   $374 = ((($373)) + 12|0);
   HEAP32[$pIdxCons>>2] = $374;
  }
  $375 = HEAP32[$i>>2]|0;
  $376 = HEAP32[$nConstraint>>2]|0;
  $377 = ($375|0)>=($376|0);
  if ($377) {
   $378 = HEAP32[$mxTerm>>2]|0;
   $379 = (($378) + 1)|0;
   $380 = $379&65535;
   $381 = HEAP32[$pNew>>2]|0;
   $382 = ((($381)) + 40|0);
   HEAP16[$382>>1] = $380;
   $383 = HEAP32[$pIdxInfo>>2]|0;
   $384 = ((($383)) + 20|0);
   $385 = HEAP32[$384>>2]|0;
   $386 = HEAP32[$pNew>>2]|0;
   $387 = ((($386)) + 24|0);
   HEAP32[$387>>2] = $385;
   $388 = HEAP32[$pIdxInfo>>2]|0;
   $389 = ((($388)) + 28|0);
   $390 = HEAP32[$389>>2]|0;
   $391 = $390&255;
   $392 = HEAP32[$pNew>>2]|0;
   $393 = ((($392)) + 24|0);
   $394 = ((($393)) + 4|0);
   HEAP8[$394>>0] = $391;
   $395 = HEAP32[$pIdxInfo>>2]|0;
   $396 = ((($395)) + 28|0);
   HEAP32[$396>>2] = 0;
   $397 = HEAP32[$pIdxInfo>>2]|0;
   $398 = ((($397)) + 24|0);
   $399 = HEAP32[$398>>2]|0;
   $400 = HEAP32[$pNew>>2]|0;
   $401 = ((($400)) + 24|0);
   $402 = ((($401)) + 8|0);
   HEAP32[$402>>2] = $399;
   $403 = HEAP32[$pIdxInfo>>2]|0;
   $404 = ((($403)) + 32|0);
   $405 = HEAP32[$404>>2]|0;
   $406 = ($405|0)!=(0);
   if ($406) {
    $407 = HEAP32[$pIdxInfo>>2]|0;
    $408 = ((($407)) + 8|0);
    $409 = HEAP32[$408>>2]|0;
    $411 = $409;
   } else {
    $411 = 0;
   }
   $410 = $411&255;
   $412 = HEAP32[$pNew>>2]|0;
   $413 = ((($412)) + 24|0);
   $414 = ((($413)) + 5|0);
   HEAP8[$414>>0] = $410;
   $415 = HEAP32[$pNew>>2]|0;
   $416 = ((($415)) + 18|0);
   HEAP16[$416>>1] = 0;
   $417 = HEAP32[$pIdxInfo>>2]|0;
   $418 = ((($417)) + 40|0);
   $419 = +HEAPF64[$418>>3];
   $420 = (_sqlite3LogEstFromDouble($419)|0);
   $421 = HEAP32[$pNew>>2]|0;
   $422 = ((($421)) + 20|0);
   HEAP16[$422>>1] = $420;
   $423 = HEAP32[$pIdxInfo>>2]|0;
   $424 = ((($423)) + 48|0);
   $425 = $424;
   $426 = $425;
   $427 = HEAP32[$426>>2]|0;
   $428 = (($425) + 4)|0;
   $429 = $428;
   $430 = HEAP32[$429>>2]|0;
   $431 = (_sqlite3LogEst($427,$430)|0);
   $432 = HEAP32[$pNew>>2]|0;
   $433 = ((($432)) + 22|0);
   HEAP16[$433>>1] = $431;
   $434 = HEAP32[$pIdxInfo>>2]|0;
   $435 = ((($434)) + 56|0);
   $436 = HEAP32[$435>>2]|0;
   $437 = $436 & 1;
   $438 = ($437|0)!=(0);
   $439 = HEAP32[$pNew>>2]|0;
   $440 = ((($439)) + 36|0);
   $441 = HEAP32[$440>>2]|0;
   if ($438) {
    $442 = $441 | 4096;
    HEAP32[$440>>2] = $442;
   } else {
    $443 = $441 & -4097;
    HEAP32[$440>>2] = $443;
   }
   $444 = HEAP32[$5>>2]|0;
   $445 = HEAP32[$pNew>>2]|0;
   (_whereLoopInsert($444,$445)|0);
   $446 = HEAP32[$pNew>>2]|0;
   $447 = ((($446)) + 24|0);
   $448 = ((($447)) + 4|0);
   $449 = HEAP8[$448>>0]|0;
   $450 = ($449<<24>>24)!=(0);
   if ($450) {
    $451 = HEAP32[$pNew>>2]|0;
    $452 = ((($451)) + 24|0);
    $453 = ((($452)) + 8|0);
    $454 = HEAP32[$453>>2]|0;
    _sqlite3_free($454);
    $455 = HEAP32[$pNew>>2]|0;
    $456 = ((($455)) + 24|0);
    $457 = ((($456)) + 4|0);
    HEAP8[$457>>0] = 0;
   }
  }
  $458 = HEAP32[$iPhase>>2]|0;
  $459 = (($458) + 1)|0;
  HEAP32[$iPhase>>2] = $459;
 }
 if ((label|0) == 36) {
  HEAP32[$rc>>2] = 1;
  $295 = HEAP32[$pParse>>2]|0;
  $296 = HEAP32[$pTab>>2]|0;
  $297 = HEAP32[$296>>2]|0;
  HEAP32[$vararg_buffer>>2] = $297;
  _sqlite3ErrorMsg($295,32588,$vararg_buffer);
 }
 $460 = HEAP32[$pIdxInfo>>2]|0;
 $461 = ((($460)) + 28|0);
 $462 = HEAP32[$461>>2]|0;
 $463 = ($462|0)!=(0);
 if ($463) {
  $464 = HEAP32[$pIdxInfo>>2]|0;
  $465 = ((($464)) + 24|0);
  $466 = HEAP32[$465>>2]|0;
  _sqlite3_free($466);
 }
 $467 = HEAP32[$db>>2]|0;
 $468 = HEAP32[$pIdxInfo>>2]|0;
 _sqlite3DbFree($467,$468);
 $469 = HEAP32[$rc>>2]|0;
 HEAP32[$4>>2] = $469;
 $470 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($470|0);
}
function _whereLoopAddBtree($pBuilder,$0,$1) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiColumnPk = 0, $aiRowEstPk = 0, $b = 0, $iSortIdx = 0, $m = 0, $or$cond = 0, $pFirst = 0, $pNew = 0, $pProbe = 0, $pSrc = 0, $pTab = 0;
 var $pTabList = 0, $pTerm = 0, $pWC = 0, $pWCEnd = 0, $pWInfo = 0, $rLogSize = 0, $rSize = 0, $rc = 0, $sPk = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $2 = sp + 128|0;
 $3 = sp + 8|0;
 $pWInfo = sp + 124|0;
 $pProbe = sp + 120|0;
 $sPk = sp + 64|0;
 $aiRowEstPk = sp + 140|0;
 $aiColumnPk = sp + 136|0;
 $pTabList = sp + 56|0;
 $pSrc = sp + 52|0;
 $pNew = sp + 48|0;
 $rc = sp + 44|0;
 $iSortIdx = sp + 40|0;
 $b = sp + 36|0;
 $rSize = sp + 134|0;
 $rLogSize = sp + 132|0;
 $pWC = sp + 32|0;
 $pTab = sp + 28|0;
 $pFirst = sp + 24|0;
 $pTerm = sp + 20|0;
 $pWCEnd = sp + 16|0;
 $m = sp;
 HEAP32[$2>>2] = $pBuilder;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP16[$aiColumnPk>>1] = -1;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iSortIdx>>2] = 1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pNew>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pWInfo>>2] = $12;
 $13 = HEAP32[$pWInfo>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTabList>>2] = $15;
 $16 = HEAP32[$pTabList>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = (($17) + (($21*72)|0)|0);
 HEAP32[$pSrc>>2] = $22;
 $23 = HEAP32[$pSrc>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pTab>>2] = $25;
 $26 = HEAP32[$2>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pWC>>2] = $28;
 $29 = HEAP32[$pSrc>>2]|0;
 $30 = ((($29)) + 68|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)!=(0|0);
 do {
  if ($32) {
   $33 = HEAP32[$pSrc>>2]|0;
   $34 = ((($33)) + 68|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pProbe>>2] = $35;
  } else {
   $36 = HEAP32[$pTab>>2]|0;
   $37 = ((($36)) + 42|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = $39 & 32;
   $41 = ($40|0)==(0);
   if (!($41)) {
    $42 = HEAP32[$pTab>>2]|0;
    $43 = ((($42)) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    HEAP32[$pProbe>>2] = $44;
    break;
   }
   dest=$sPk; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $45 = ((($sPk)) + 50|0);
   HEAP16[$45>>1] = 1;
   $46 = ((($sPk)) + 52|0);
   HEAP16[$46>>1] = 1;
   $47 = ((($sPk)) + 4|0);
   HEAP32[$47>>2] = $aiColumnPk;
   $48 = ((($sPk)) + 8|0);
   HEAP32[$48>>2] = $aiRowEstPk;
   $49 = ((($sPk)) + 54|0);
   HEAP8[$49>>0] = 5;
   $50 = HEAP32[$pTab>>2]|0;
   $51 = ((($sPk)) + 12|0);
   HEAP32[$51>>2] = $50;
   $52 = HEAP32[$pTab>>2]|0;
   $53 = ((($52)) + 40|0);
   $54 = HEAP16[$53>>1]|0;
   $55 = ((($sPk)) + 48|0);
   HEAP16[$55>>1] = $54;
   $56 = HEAP32[$pTab>>2]|0;
   $57 = ((($56)) + 38|0);
   $58 = HEAP16[$57>>1]|0;
   HEAP16[$aiRowEstPk>>1] = $58;
   $59 = ((($aiRowEstPk)) + 2|0);
   HEAP16[$59>>1] = 0;
   $60 = HEAP32[$pSrc>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = ((($62)) + 8|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$pFirst>>2] = $64;
   $65 = HEAP32[$pSrc>>2]|0;
   $66 = ((($65)) + 36|0);
   $67 = ((($66)) + 1|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = $68 & 1;
   $70 = $69&255;
   $71 = ($70|0)==(0);
   if ($71) {
    $72 = HEAP32[$pFirst>>2]|0;
    $73 = ((($sPk)) + 20|0);
    HEAP32[$73>>2] = $72;
   }
   HEAP32[$pProbe>>2] = $sPk;
  }
 } while(0);
 $74 = HEAP32[$pTab>>2]|0;
 $75 = ((($74)) + 38|0);
 $76 = HEAP16[$75>>1]|0;
 HEAP16[$rSize>>1] = $76;
 $77 = HEAP16[$rSize>>1]|0;
 $78 = (_estLog($77)|0);
 HEAP16[$rLogSize>>1] = $78;
 $79 = HEAP32[$2>>2]|0;
 $80 = ((($79)) + 16|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = ($81|0)!=(0|0);
 L11: do {
  if (!($82)) {
   $83 = HEAP32[$pWInfo>>2]|0;
   $84 = ((($83)) + 34|0);
   $85 = HEAP16[$84>>1]|0;
   $86 = $85&65535;
   $87 = $86 & 128;
   $88 = ($87|0)==(0);
   if ($88) {
    $89 = HEAP32[$pWInfo>>2]|0;
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = ((($91)) + 24|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = $93 & 1048576;
    $95 = ($94|0)!=(0);
    if ($95) {
     $96 = HEAP32[$pSrc>>2]|0;
     $97 = ((($96)) + 68|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = ($98|0)==(0|0);
     if ($99) {
      $100 = HEAP32[$pSrc>>2]|0;
      $101 = ((($100)) + 36|0);
      $102 = ((($101)) + 1|0);
      $103 = HEAP8[$102>>0]|0;
      $104 = $103 & 1;
      $105 = $104&255;
      $106 = ($105|0)!=(0);
      if (!($106)) {
       $107 = HEAP32[$pTab>>2]|0;
       $108 = ((($107)) + 42|0);
       $109 = HEAP8[$108>>0]|0;
       $110 = $109&255;
       $111 = $110 & 32;
       $112 = ($111|0)==(0);
       if ($112) {
        $113 = HEAP32[$pSrc>>2]|0;
        $114 = ((($113)) + 36|0);
        $115 = ((($114)) + 1|0);
        $116 = HEAP8[$115>>0]|0;
        $117 = ($116&255) >>> 3;
        $118 = $117 & 1;
        $119 = $118&255;
        $120 = ($119|0)!=(0);
        if (!($120)) {
         $121 = HEAP32[$pSrc>>2]|0;
         $122 = ((($121)) + 36|0);
         $123 = ((($122)) + 1|0);
         $124 = HEAP8[$123>>0]|0;
         $125 = ($124&255) >>> 5;
         $126 = $125 & 1;
         $127 = $126&255;
         $128 = ($127|0)!=(0);
         if (!($128)) {
          $129 = HEAP32[$pWC>>2]|0;
          $130 = ((($129)) + 20|0);
          $131 = HEAP32[$130>>2]|0;
          $132 = HEAP32[$pWC>>2]|0;
          $133 = ((($132)) + 12|0);
          $134 = HEAP32[$133>>2]|0;
          $135 = (($131) + (($134*48)|0)|0);
          HEAP32[$pWCEnd>>2] = $135;
          $136 = HEAP32[$pWC>>2]|0;
          $137 = ((($136)) + 20|0);
          $138 = HEAP32[$137>>2]|0;
          HEAP32[$pTerm>>2] = $138;
          while(1) {
           $139 = HEAP32[$rc>>2]|0;
           $140 = ($139|0)==(0);
           if (!($140)) {
            break L11;
           }
           $141 = HEAP32[$pTerm>>2]|0;
           $142 = HEAP32[$pWCEnd>>2]|0;
           $143 = ($141>>>0)<($142>>>0);
           if (!($143)) {
            break L11;
           }
           $144 = HEAP32[$pTerm>>2]|0;
           $145 = ((($144)) + 32|0);
           $146 = $145;
           $147 = $146;
           $148 = HEAP32[$147>>2]|0;
           $149 = (($146) + 4)|0;
           $150 = $149;
           $151 = HEAP32[$150>>2]|0;
           $152 = HEAP32[$pNew>>2]|0;
           $153 = ((($152)) + 8|0);
           $154 = $153;
           $155 = $154;
           $156 = HEAP32[$155>>2]|0;
           $157 = (($154) + 4)|0;
           $158 = $157;
           $159 = HEAP32[$158>>2]|0;
           $160 = $148 & $156;
           $161 = $151 & $159;
           $162 = ($160|0)!=(0);
           $163 = ($161|0)!=(0);
           $164 = $162 | $163;
           if (!($164)) {
            $165 = HEAP32[$pTerm>>2]|0;
            $166 = HEAP32[$pSrc>>2]|0;
            $167 = (_termCanDriveIndex($165,$166,0,0)|0);
            $168 = ($167|0)!=(0);
            if ($168) {
             $169 = HEAP32[$pNew>>2]|0;
             $170 = ((($169)) + 24|0);
             HEAP16[$170>>1] = 1;
             $171 = HEAP32[$pNew>>2]|0;
             $172 = ((($171)) + 42|0);
             HEAP16[$172>>1] = 0;
             $173 = HEAP32[$pNew>>2]|0;
             $174 = ((($173)) + 24|0);
             $175 = ((($174)) + 4|0);
             HEAP32[$175>>2] = 0;
             $176 = HEAP32[$pNew>>2]|0;
             $177 = ((($176)) + 40|0);
             HEAP16[$177>>1] = 1;
             $178 = HEAP32[$pTerm>>2]|0;
             $179 = HEAP32[$pNew>>2]|0;
             $180 = ((($179)) + 48|0);
             $181 = HEAP32[$180>>2]|0;
             HEAP32[$181>>2] = $178;
             $182 = HEAP16[$rLogSize>>1]|0;
             $183 = $182 << 16 >> 16;
             $184 = HEAP16[$rSize>>1]|0;
             $185 = $184 << 16 >> 16;
             $186 = (($183) + ($185))|0;
             $187 = (($186) + 4)|0;
             $188 = $187&65535;
             $189 = HEAP32[$pNew>>2]|0;
             $190 = ((($189)) + 18|0);
             HEAP16[$190>>1] = $188;
             $191 = HEAP32[$pTab>>2]|0;
             $192 = ((($191)) + 12|0);
             $193 = HEAP32[$192>>2]|0;
             $194 = ($193|0)==(0|0);
             if ($194) {
              $195 = HEAP32[$pTab>>2]|0;
              $196 = ((($195)) + 42|0);
              $197 = HEAP8[$196>>0]|0;
              $198 = $197&255;
              $199 = $198 & 2;
              $200 = ($199|0)==(0);
              if ($200) {
               $201 = HEAP32[$pNew>>2]|0;
               $202 = ((($201)) + 18|0);
               $203 = HEAP16[$202>>1]|0;
               $204 = $203 << 16 >> 16;
               $205 = (($204) + 24)|0;
               $206 = $205&65535;
               HEAP16[$202>>1] = $206;
              }
             }
             $207 = HEAP32[$pNew>>2]|0;
             $208 = ((($207)) + 22|0);
             HEAP16[$208>>1] = 43;
             $209 = HEAP16[$rLogSize>>1]|0;
             $210 = HEAP32[$pNew>>2]|0;
             $211 = ((($210)) + 22|0);
             $212 = HEAP16[$211>>1]|0;
             $213 = (_sqlite3LogEstAdd($209,$212)|0);
             $214 = HEAP32[$pNew>>2]|0;
             $215 = ((($214)) + 20|0);
             HEAP16[$215>>1] = $213;
             $216 = HEAP32[$pNew>>2]|0;
             $217 = ((($216)) + 36|0);
             HEAP32[$217>>2] = 16384;
             $218 = $3;
             $219 = $218;
             $220 = HEAP32[$219>>2]|0;
             $221 = (($218) + 4)|0;
             $222 = $221;
             $223 = HEAP32[$222>>2]|0;
             $224 = HEAP32[$pTerm>>2]|0;
             $225 = ((($224)) + 32|0);
             $226 = $225;
             $227 = $226;
             $228 = HEAP32[$227>>2]|0;
             $229 = (($226) + 4)|0;
             $230 = $229;
             $231 = HEAP32[$230>>2]|0;
             $232 = $220 | $228;
             $233 = $223 | $231;
             $234 = HEAP32[$pNew>>2]|0;
             $235 = $234;
             $236 = $235;
             HEAP32[$236>>2] = $232;
             $237 = (($235) + 4)|0;
             $238 = $237;
             HEAP32[$238>>2] = $233;
             $239 = HEAP32[$2>>2]|0;
             $240 = HEAP32[$pNew>>2]|0;
             $241 = (_whereLoopInsert($239,$240)|0);
             HEAP32[$rc>>2] = $241;
            }
           }
           $242 = HEAP32[$pTerm>>2]|0;
           $243 = ((($242)) + 48|0);
           HEAP32[$pTerm>>2] = $243;
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 L32: while(1) {
  $244 = HEAP32[$rc>>2]|0;
  $245 = ($244|0)==(0);
  $246 = HEAP32[$pProbe>>2]|0;
  $247 = ($246|0)!=(0|0);
  $248 = $245 ? $247 : 0;
  if (!($248)) {
   label = 46;
   break;
  }
  $249 = HEAP32[$pProbe>>2]|0;
  $250 = ((($249)) + 36|0);
  $251 = HEAP32[$250>>2]|0;
  $252 = ($251|0)!=(0|0);
  if ($252) {
   $253 = HEAP32[$pSrc>>2]|0;
   $254 = ((($253)) + 44|0);
   $255 = HEAP32[$254>>2]|0;
   $256 = HEAP32[$pWC>>2]|0;
   $257 = HEAP32[$pProbe>>2]|0;
   $258 = ((($257)) + 36|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = (_whereUsablePartialIndex($255,$256,$259)|0);
   $261 = ($260|0)!=(0);
   if ($261) {
    label = 29;
   }
  } else {
   label = 29;
  }
  if ((label|0) == 29) {
   label = 0;
   $262 = HEAP32[$pProbe>>2]|0;
   $263 = ((($262)) + 8|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = HEAP16[$264>>1]|0;
   HEAP16[$rSize>>1] = $265;
   $266 = HEAP32[$pNew>>2]|0;
   $267 = ((($266)) + 24|0);
   HEAP16[$267>>1] = 0;
   $268 = HEAP32[$pNew>>2]|0;
   $269 = ((($268)) + 42|0);
   HEAP16[$269>>1] = 0;
   $270 = HEAP32[$pNew>>2]|0;
   $271 = ((($270)) + 40|0);
   HEAP16[$271>>1] = 0;
   $272 = HEAP32[$pNew>>2]|0;
   $273 = ((($272)) + 17|0);
   HEAP8[$273>>0] = 0;
   $274 = HEAP32[$pNew>>2]|0;
   $275 = ((($274)) + 18|0);
   HEAP16[$275>>1] = 0;
   $276 = $3;
   $277 = $276;
   $278 = HEAP32[$277>>2]|0;
   $279 = (($276) + 4)|0;
   $280 = $279;
   $281 = HEAP32[$280>>2]|0;
   $282 = HEAP32[$pNew>>2]|0;
   $283 = $282;
   $284 = $283;
   HEAP32[$284>>2] = $278;
   $285 = (($283) + 4)|0;
   $286 = $285;
   HEAP32[$286>>2] = $281;
   $287 = HEAP16[$rSize>>1]|0;
   $288 = HEAP32[$pNew>>2]|0;
   $289 = ((($288)) + 22|0);
   HEAP16[$289>>1] = $287;
   $290 = HEAP32[$pProbe>>2]|0;
   $291 = HEAP32[$pNew>>2]|0;
   $292 = ((($291)) + 24|0);
   $293 = ((($292)) + 4|0);
   HEAP32[$293>>2] = $290;
   $294 = HEAP32[$2>>2]|0;
   $295 = HEAP32[$pProbe>>2]|0;
   $296 = HEAP32[$pSrc>>2]|0;
   $297 = ((($296)) + 44|0);
   $298 = HEAP32[$297>>2]|0;
   $299 = (_indexMightHelpWithOrderBy($294,$295,$298)|0);
   HEAP32[$b>>2] = $299;
   $300 = HEAP32[$pProbe>>2]|0;
   $301 = ((($300)) + 44|0);
   $302 = HEAP32[$301>>2]|0;
   $303 = ($302|0)<=(0);
   do {
    if ($303) {
     $304 = HEAP32[$pNew>>2]|0;
     $305 = ((($304)) + 36|0);
     HEAP32[$305>>2] = 256;
     $306 = HEAP32[$b>>2]|0;
     $307 = ($306|0)!=(0);
     $308 = HEAP32[$iSortIdx>>2]|0;
     $309 = $307 ? $308 : 0;
     $310 = $309&255;
     $311 = HEAP32[$pNew>>2]|0;
     $312 = ((($311)) + 17|0);
     HEAP8[$312>>0] = $310;
     $313 = HEAP16[$rSize>>1]|0;
     $314 = $313 << 16 >> 16;
     $315 = (($314) + 16)|0;
     $316 = $315&65535;
     $317 = HEAP32[$pNew>>2]|0;
     $318 = ((($317)) + 20|0);
     HEAP16[$318>>1] = $316;
     $319 = HEAP32[$pWC>>2]|0;
     $320 = HEAP32[$pNew>>2]|0;
     $321 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($319,$320,$321);
     $322 = HEAP32[$2>>2]|0;
     $323 = HEAP32[$pNew>>2]|0;
     $324 = (_whereLoopInsert($322,$323)|0);
     HEAP32[$rc>>2] = $324;
     $325 = HEAP16[$rSize>>1]|0;
     $326 = HEAP32[$pNew>>2]|0;
     $327 = ((($326)) + 22|0);
     HEAP16[$327>>1] = $325;
     $328 = HEAP32[$rc>>2]|0;
     $329 = ($328|0)!=(0);
     if ($329) {
      label = 46;
      break L32;
     }
    } else {
     $330 = HEAP32[$pProbe>>2]|0;
     $331 = ((($330)) + 55|0);
     $332 = HEAP8[$331>>0]|0;
     $333 = ($332&255) >>> 5;
     $334 = $333 & 1;
     $335 = $334&255;
     $336 = ($335|0)!=(0);
     if ($336) {
      $337 = HEAP32[$pNew>>2]|0;
      $338 = ((($337)) + 36|0);
      HEAP32[$338>>2] = 576;
      $339 = $m;
      $340 = $339;
      HEAP32[$340>>2] = 0;
      $341 = (($339) + 4)|0;
      $342 = $341;
      HEAP32[$342>>2] = 0;
     } else {
      $343 = HEAP32[$pSrc>>2]|0;
      $344 = ((($343)) + 56|0);
      $345 = $344;
      $346 = $345;
      $347 = HEAP32[$346>>2]|0;
      $348 = (($345) + 4)|0;
      $349 = $348;
      $350 = HEAP32[$349>>2]|0;
      $351 = HEAP32[$pProbe>>2]|0;
      $352 = (_columnsInIndex($351)|0);
      $353 = tempRet0;
      $354 = $352 ^ -1;
      $355 = $353 ^ -1;
      $356 = $347 & $354;
      $357 = $350 & $355;
      $358 = $m;
      $359 = $358;
      HEAP32[$359>>2] = $356;
      $360 = (($358) + 4)|0;
      $361 = $360;
      HEAP32[$361>>2] = $357;
      $362 = $m;
      $363 = $362;
      $364 = HEAP32[$363>>2]|0;
      $365 = (($362) + 4)|0;
      $366 = $365;
      $367 = HEAP32[$366>>2]|0;
      $368 = ($364|0)==(0);
      $369 = ($367|0)==(0);
      $370 = $368 & $369;
      $371 = $370 ? 576 : 512;
      $372 = HEAP32[$pNew>>2]|0;
      $373 = ((($372)) + 36|0);
      HEAP32[$373>>2] = $371;
     }
     $374 = HEAP32[$b>>2]|0;
     $375 = ($374|0)!=(0);
     if (!($375)) {
      $376 = HEAP32[$pTab>>2]|0;
      $377 = ((($376)) + 42|0);
      $378 = HEAP8[$377>>0]|0;
      $379 = $378&255;
      $380 = $379 & 32;
      $381 = ($380|0)==(0);
      if ($381) {
       $382 = $m;
       $383 = $382;
       $384 = HEAP32[$383>>2]|0;
       $385 = (($382) + 4)|0;
       $386 = $385;
       $387 = HEAP32[$386>>2]|0;
       $388 = ($384|0)==(0);
       $389 = ($387|0)==(0);
       $390 = $388 & $389;
       if (!($390)) {
        break;
       }
       $391 = HEAP32[$pProbe>>2]|0;
       $392 = ((($391)) + 55|0);
       $393 = HEAP8[$392>>0]|0;
       $394 = ($393&255) >>> 2;
       $395 = $394 & 1;
       $396 = $395&255;
       $397 = ($396|0)==(0);
       if (!($397)) {
        break;
       }
       $398 = HEAP32[$pProbe>>2]|0;
       $399 = ((($398)) + 48|0);
       $400 = HEAP16[$399>>1]|0;
       $401 = $400 << 16 >> 16;
       $402 = HEAP32[$pTab>>2]|0;
       $403 = ((($402)) + 40|0);
       $404 = HEAP16[$403>>1]|0;
       $405 = $404 << 16 >> 16;
       $406 = ($401|0)<($405|0);
       if (!($406)) {
        break;
       }
       $407 = HEAP32[$pWInfo>>2]|0;
       $408 = ((($407)) + 34|0);
       $409 = HEAP16[$408>>1]|0;
       $410 = $409&65535;
       $411 = $410 & 4;
       $412 = ($411|0)==(0);
       $413 = HEAP32[(24)>>2]|0;
       $414 = ($413|0)!=(0);
       $or$cond = $412 & $414;
       if (!($or$cond)) {
        break;
       }
       $415 = HEAP32[$pWInfo>>2]|0;
       $416 = HEAP32[$415>>2]|0;
       $417 = HEAP32[$416>>2]|0;
       $418 = ((($417)) + 60|0);
       $419 = HEAP16[$418>>1]|0;
       $420 = $419&65535;
       $421 = $420 & 64;
       $422 = ($421|0)==(0);
       if (!($422)) {
        break;
       }
      }
     }
     $423 = HEAP32[$b>>2]|0;
     $424 = ($423|0)!=(0);
     $425 = HEAP32[$iSortIdx>>2]|0;
     $426 = $424 ? $425 : 0;
     $427 = $426&255;
     $428 = HEAP32[$pNew>>2]|0;
     $429 = ((($428)) + 17|0);
     HEAP8[$429>>0] = $427;
     $430 = HEAP16[$rSize>>1]|0;
     $431 = $430 << 16 >> 16;
     $432 = (($431) + 1)|0;
     $433 = HEAP32[$pProbe>>2]|0;
     $434 = ((($433)) + 48|0);
     $435 = HEAP16[$434>>1]|0;
     $436 = $435 << 16 >> 16;
     $437 = ($436*15)|0;
     $438 = HEAP32[$pTab>>2]|0;
     $439 = ((($438)) + 40|0);
     $440 = HEAP16[$439>>1]|0;
     $441 = $440 << 16 >> 16;
     $442 = (($437|0) / ($441|0))&-1;
     $443 = (($432) + ($442))|0;
     $444 = $443&65535;
     $445 = HEAP32[$pNew>>2]|0;
     $446 = ((($445)) + 20|0);
     HEAP16[$446>>1] = $444;
     $447 = $m;
     $448 = $447;
     $449 = HEAP32[$448>>2]|0;
     $450 = (($447) + 4)|0;
     $451 = $450;
     $452 = HEAP32[$451>>2]|0;
     $453 = ($449|0)!=(0);
     $454 = ($452|0)!=(0);
     $455 = $453 | $454;
     if ($455) {
      $456 = HEAP32[$pNew>>2]|0;
      $457 = ((($456)) + 20|0);
      $458 = HEAP16[$457>>1]|0;
      $459 = HEAP16[$rSize>>1]|0;
      $460 = $459 << 16 >> 16;
      $461 = (($460) + 16)|0;
      $462 = $461&65535;
      $463 = (_sqlite3LogEstAdd($458,$462)|0);
      $464 = HEAP32[$pNew>>2]|0;
      $465 = ((($464)) + 20|0);
      HEAP16[$465>>1] = $463;
     }
     $466 = HEAP32[$pWC>>2]|0;
     $467 = HEAP32[$pNew>>2]|0;
     $468 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($466,$467,$468);
     $469 = HEAP32[$2>>2]|0;
     $470 = HEAP32[$pNew>>2]|0;
     $471 = (_whereLoopInsert($469,$470)|0);
     HEAP32[$rc>>2] = $471;
     $472 = HEAP16[$rSize>>1]|0;
     $473 = HEAP32[$pNew>>2]|0;
     $474 = ((($473)) + 22|0);
     HEAP16[$474>>1] = $472;
     $475 = HEAP32[$rc>>2]|0;
     $476 = ($475|0)!=(0);
     if ($476) {
      label = 46;
      break L32;
     }
    }
   } while(0);
   $477 = HEAP32[$2>>2]|0;
   $478 = HEAP32[$pSrc>>2]|0;
   $479 = HEAP32[$pProbe>>2]|0;
   $480 = (_whereLoopAddBtreeIndex($477,$478,$479,0)|0);
   HEAP32[$rc>>2] = $480;
   $481 = HEAP32[$pSrc>>2]|0;
   $482 = ((($481)) + 68|0);
   $483 = HEAP32[$482>>2]|0;
   $484 = ($483|0)!=(0|0);
   if ($484) {
    label = 46;
    break;
   }
  }
  $485 = HEAP32[$pProbe>>2]|0;
  $486 = ((($485)) + 20|0);
  $487 = HEAP32[$486>>2]|0;
  HEAP32[$pProbe>>2] = $487;
  $488 = HEAP32[$iSortIdx>>2]|0;
  $489 = (($488) + 1)|0;
  HEAP32[$iSortIdx>>2] = $489;
 }
 if ((label|0) == 46) {
  $490 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($490|0);
 }
 return (0)|0;
}
function _whereLoopAddOr($pBuilder,$0,$1,$2,$3) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCur = 0, $j = 0, $once = 0, $pItem = 0, $pNew = 0;
 var $pOrTerm = 0, $pOrWC = 0, $pOrWCEnd = 0, $pTerm = 0, $pWC = 0, $pWCEnd = 0, $pWInfo = 0, $rc = 0, $sCur = 0, $sPrev = 0, $sSubBuild = 0, $sSum = 0, $tempWC = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 672|0;
 $4 = sp + 668|0;
 $5 = sp + 584|0;
 $6 = sp + 576|0;
 $pWInfo = sp + 664|0;
 $pWC = sp + 660|0;
 $pNew = sp + 656|0;
 $pTerm = sp + 652|0;
 $pWCEnd = sp + 648|0;
 $rc = sp + 644|0;
 $iCur = sp + 640|0;
 $tempWC = sp + 168|0;
 $sSubBuild = sp + 620|0;
 $sSum = sp + 112|0;
 $sCur = sp + 56|0;
 $pItem = sp + 616|0;
 $pOrWC = sp + 612|0;
 $pOrWCEnd = sp + 608|0;
 $pOrTerm = sp + 604|0;
 $once = sp + 600|0;
 $i = sp + 596|0;
 $j = sp + 592|0;
 $sPrev = sp;
 HEAP32[$4>>2] = $pBuilder;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 HEAP32[$12>>2] = $2;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $3;
 $15 = HEAP32[$4>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pWInfo>>2] = $16;
 HEAP32[$rc>>2] = 0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pWC>>2] = $19;
 $20 = HEAP32[$pWC>>2]|0;
 $21 = ((($20)) + 20|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pWC>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($22) + (($25*48)|0)|0);
 HEAP32[$pWCEnd>>2] = $26;
 $27 = HEAP32[$4>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$pNew>>2] = $29;
 dest=$sSum; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $30 = HEAP32[$pWInfo>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = HEAP32[$pNew>>2]|0;
 $35 = ((($34)) + 16|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = (($33) + (($37*72)|0)|0);
 HEAP32[$pItem>>2] = $38;
 $39 = HEAP32[$pItem>>2]|0;
 $40 = ((($39)) + 44|0);
 $41 = HEAP32[$40>>2]|0;
 HEAP32[$iCur>>2] = $41;
 $42 = HEAP32[$pWC>>2]|0;
 $43 = ((($42)) + 20|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pTerm>>2] = $44;
 while(1) {
  $45 = HEAP32[$pTerm>>2]|0;
  $46 = HEAP32[$pWCEnd>>2]|0;
  $47 = ($45>>>0)<($46>>>0);
  $48 = HEAP32[$rc>>2]|0;
  $49 = ($48|0)==(0);
  $50 = $47 ? $49 : 0;
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$pTerm>>2]|0;
  $52 = ((($51)) + 18|0);
  $53 = HEAP16[$52>>1]|0;
  $54 = $53&65535;
  $55 = $54 & 512;
  $56 = ($55|0)!=(0);
  L4: do {
   if ($56) {
    $57 = HEAP32[$pTerm>>2]|0;
    $58 = ((($57)) + 12|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ((($59)) + 408|0);
    $61 = $60;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (($61) + 4)|0;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[$pNew>>2]|0;
    $68 = ((($67)) + 8|0);
    $69 = $68;
    $70 = $69;
    $71 = HEAP32[$70>>2]|0;
    $72 = (($69) + 4)|0;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = $63 & $71;
    $76 = $66 & $74;
    $77 = ($75|0)!=(0);
    $78 = ($76|0)!=(0);
    $79 = $77 | $78;
    if ($79) {
     $80 = HEAP32[$pTerm>>2]|0;
     $81 = ((($80)) + 12|0);
     $82 = HEAP32[$81>>2]|0;
     HEAP32[$pOrWC>>2] = $82;
     $83 = HEAP32[$pOrWC>>2]|0;
     $84 = ((($83)) + 12|0);
     $85 = HEAP32[$84>>2]|0;
     $86 = HEAP32[$pOrWC>>2]|0;
     $87 = ((($86)) + 20|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = (($88) + (($85*48)|0)|0);
     HEAP32[$pOrWCEnd>>2] = $89;
     HEAP32[$once>>2] = 1;
     $90 = HEAP32[$4>>2]|0;
     ;HEAP32[$sSubBuild>>2]=HEAP32[$90>>2]|0;HEAP32[$sSubBuild+4>>2]=HEAP32[$90+4>>2]|0;HEAP32[$sSubBuild+8>>2]=HEAP32[$90+8>>2]|0;HEAP32[$sSubBuild+12>>2]=HEAP32[$90+12>>2]|0;HEAP32[$sSubBuild+16>>2]=HEAP32[$90+16>>2]|0;
     $91 = ((($sSubBuild)) + 8|0);
     HEAP32[$91>>2] = 0;
     $92 = ((($sSubBuild)) + 16|0);
     HEAP32[$92>>2] = $sCur;
     $93 = HEAP32[$pOrWC>>2]|0;
     $94 = ((($93)) + 20|0);
     $95 = HEAP32[$94>>2]|0;
     HEAP32[$pOrTerm>>2] = $95;
     L7: while(1) {
      $96 = HEAP32[$pOrTerm>>2]|0;
      $97 = HEAP32[$pOrWCEnd>>2]|0;
      $98 = ($96>>>0)<($97>>>0);
      if (!($98)) {
       break;
      }
      $99 = HEAP32[$pOrTerm>>2]|0;
      $100 = ((($99)) + 18|0);
      $101 = HEAP16[$100>>1]|0;
      $102 = $101&65535;
      $103 = $102 & 1024;
      $104 = ($103|0)!=(0);
      $105 = HEAP32[$pOrTerm>>2]|0;
      if ($104) {
       $106 = ((($105)) + 12|0);
       $107 = HEAP32[$106>>2]|0;
       $108 = ((($sSubBuild)) + 4|0);
       HEAP32[$108>>2] = $107;
       label = 11;
      } else {
       $109 = ((($105)) + 8|0);
       $110 = HEAP32[$109>>2]|0;
       $111 = HEAP32[$iCur>>2]|0;
       $112 = ($110|0)==($111|0);
       if ($112) {
        $113 = HEAP32[$pWC>>2]|0;
        $114 = HEAP32[$113>>2]|0;
        HEAP32[$tempWC>>2] = $114;
        $115 = HEAP32[$pWC>>2]|0;
        $116 = ((($tempWC)) + 4|0);
        HEAP32[$116>>2] = $115;
        $117 = ((($tempWC)) + 8|0);
        HEAP8[$117>>0] = 72;
        $118 = ((($tempWC)) + 12|0);
        HEAP32[$118>>2] = 1;
        $119 = HEAP32[$pOrTerm>>2]|0;
        $120 = ((($tempWC)) + 20|0);
        HEAP32[$120>>2] = $119;
        $121 = ((($sSubBuild)) + 4|0);
        HEAP32[$121>>2] = $tempWC;
        label = 11;
       }
      }
      L14: do {
       if ((label|0) == 11) {
        label = 0;
        HEAP16[$sCur>>1] = 0;
        $122 = HEAP32[$pItem>>2]|0;
        $123 = ((($122)) + 16|0);
        $124 = HEAP32[$123>>2]|0;
        $125 = ((($124)) + 42|0);
        $126 = HEAP8[$125>>0]|0;
        $127 = $126&255;
        $128 = $127 & 16;
        $129 = ($128|0)!=(0);
        $130 = $5;
        $131 = $130;
        $132 = HEAP32[$131>>2]|0;
        $133 = (($130) + 4)|0;
        $134 = $133;
        $135 = HEAP32[$134>>2]|0;
        if ($129) {
         $136 = $6;
         $137 = $136;
         $138 = HEAP32[$137>>2]|0;
         $139 = (($136) + 4)|0;
         $140 = $139;
         $141 = HEAP32[$140>>2]|0;
         $142 = (_whereLoopAddVirtual($sSubBuild,$132,$135,$138,$141)|0);
         HEAP32[$rc>>2] = $142;
        } else {
         $143 = (_whereLoopAddBtree($sSubBuild,$132,$135)|0);
         HEAP32[$rc>>2] = $143;
        }
        $144 = HEAP32[$rc>>2]|0;
        $145 = ($144|0)==(0);
        if ($145) {
         $146 = $5;
         $147 = $146;
         $148 = HEAP32[$147>>2]|0;
         $149 = (($146) + 4)|0;
         $150 = $149;
         $151 = HEAP32[$150>>2]|0;
         $152 = $6;
         $153 = $152;
         $154 = HEAP32[$153>>2]|0;
         $155 = (($152) + 4)|0;
         $156 = $155;
         $157 = HEAP32[$156>>2]|0;
         $158 = (_whereLoopAddOr($sSubBuild,$148,$151,$154,$157)|0);
         HEAP32[$rc>>2] = $158;
        }
        $159 = HEAP16[$sCur>>1]|0;
        $160 = $159&65535;
        $161 = ($160|0)==(0);
        if ($161) {
         label = 17;
         break L7;
        }
        $162 = HEAP32[$once>>2]|0;
        $163 = ($162|0)!=(0);
        if ($163) {
         _whereOrMove($sSum,$sCur);
         HEAP32[$once>>2] = 0;
         break;
        }
        _whereOrMove($sPrev,$sSum);
        HEAP16[$sSum>>1] = 0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $164 = HEAP32[$i>>2]|0;
         $165 = HEAP16[$sPrev>>1]|0;
         $166 = $165&65535;
         $167 = ($164|0)<($166|0);
         if (!($167)) {
          break L14;
         }
         HEAP32[$j>>2] = 0;
         while(1) {
          $168 = HEAP32[$j>>2]|0;
          $169 = HEAP16[$sCur>>1]|0;
          $170 = $169&65535;
          $171 = ($168|0)<($170|0);
          $172 = HEAP32[$i>>2]|0;
          if (!($171)) {
           break;
          }
          $173 = ((($sPrev)) + 8|0);
          $174 = (($173) + ($172<<4)|0);
          $175 = $174;
          $176 = $175;
          $177 = HEAP32[$176>>2]|0;
          $178 = (($175) + 4)|0;
          $179 = $178;
          $180 = HEAP32[$179>>2]|0;
          $181 = HEAP32[$j>>2]|0;
          $182 = ((($sCur)) + 8|0);
          $183 = (($182) + ($181<<4)|0);
          $184 = $183;
          $185 = $184;
          $186 = HEAP32[$185>>2]|0;
          $187 = (($184) + 4)|0;
          $188 = $187;
          $189 = HEAP32[$188>>2]|0;
          $190 = $177 | $186;
          $191 = $180 | $189;
          $192 = HEAP32[$i>>2]|0;
          $193 = ((($sPrev)) + 8|0);
          $194 = (($193) + ($192<<4)|0);
          $195 = ((($194)) + 8|0);
          $196 = HEAP16[$195>>1]|0;
          $197 = HEAP32[$j>>2]|0;
          $198 = ((($sCur)) + 8|0);
          $199 = (($198) + ($197<<4)|0);
          $200 = ((($199)) + 8|0);
          $201 = HEAP16[$200>>1]|0;
          $202 = (_sqlite3LogEstAdd($196,$201)|0);
          $203 = HEAP32[$i>>2]|0;
          $204 = ((($sPrev)) + 8|0);
          $205 = (($204) + ($203<<4)|0);
          $206 = ((($205)) + 10|0);
          $207 = HEAP16[$206>>1]|0;
          $208 = HEAP32[$j>>2]|0;
          $209 = ((($sCur)) + 8|0);
          $210 = (($209) + ($208<<4)|0);
          $211 = ((($210)) + 10|0);
          $212 = HEAP16[$211>>1]|0;
          $213 = (_sqlite3LogEstAdd($207,$212)|0);
          (_whereOrInsert($sSum,$190,$191,$202,$213)|0);
          $214 = HEAP32[$j>>2]|0;
          $215 = (($214) + 1)|0;
          HEAP32[$j>>2] = $215;
         }
         $216 = (($172) + 1)|0;
         HEAP32[$i>>2] = $216;
        }
       }
      } while(0);
      $217 = HEAP32[$pOrTerm>>2]|0;
      $218 = ((($217)) + 48|0);
      HEAP32[$pOrTerm>>2] = $218;
     }
     if ((label|0) == 17) {
      label = 0;
      HEAP16[$sSum>>1] = 0;
     }
     $219 = HEAP32[$pNew>>2]|0;
     $220 = ((($219)) + 40|0);
     HEAP16[$220>>1] = 1;
     $221 = HEAP32[$pTerm>>2]|0;
     $222 = HEAP32[$pNew>>2]|0;
     $223 = ((($222)) + 48|0);
     $224 = HEAP32[$223>>2]|0;
     HEAP32[$224>>2] = $221;
     $225 = HEAP32[$pNew>>2]|0;
     $226 = ((($225)) + 36|0);
     HEAP32[$226>>2] = 8192;
     $227 = HEAP32[$pNew>>2]|0;
     $228 = ((($227)) + 18|0);
     HEAP16[$228>>1] = 0;
     $229 = HEAP32[$pNew>>2]|0;
     $230 = ((($229)) + 17|0);
     HEAP8[$230>>0] = 0;
     $231 = HEAP32[$pNew>>2]|0;
     $232 = ((($231)) + 24|0);
     ;HEAP32[$232>>2]=0|0;HEAP32[$232+4>>2]=0|0;HEAP32[$232+8>>2]=0|0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $233 = HEAP32[$rc>>2]|0;
      $234 = ($233|0)==(0);
      if (!($234)) {
       break L4;
      }
      $235 = HEAP32[$i>>2]|0;
      $236 = HEAP16[$sSum>>1]|0;
      $237 = $236&65535;
      $238 = ($235|0)<($237|0);
      if (!($238)) {
       break L4;
      }
      $239 = HEAP32[$i>>2]|0;
      $240 = ((($sSum)) + 8|0);
      $241 = (($240) + ($239<<4)|0);
      $242 = ((($241)) + 8|0);
      $243 = HEAP16[$242>>1]|0;
      $244 = $243 << 16 >> 16;
      $245 = (($244) + 1)|0;
      $246 = $245&65535;
      $247 = HEAP32[$pNew>>2]|0;
      $248 = ((($247)) + 20|0);
      HEAP16[$248>>1] = $246;
      $249 = HEAP32[$i>>2]|0;
      $250 = ((($sSum)) + 8|0);
      $251 = (($250) + ($249<<4)|0);
      $252 = ((($251)) + 10|0);
      $253 = HEAP16[$252>>1]|0;
      $254 = HEAP32[$pNew>>2]|0;
      $255 = ((($254)) + 22|0);
      HEAP16[$255>>1] = $253;
      $256 = HEAP32[$i>>2]|0;
      $257 = ((($sSum)) + 8|0);
      $258 = (($257) + ($256<<4)|0);
      $259 = $258;
      $260 = $259;
      $261 = HEAP32[$260>>2]|0;
      $262 = (($259) + 4)|0;
      $263 = $262;
      $264 = HEAP32[$263>>2]|0;
      $265 = HEAP32[$pNew>>2]|0;
      $266 = $265;
      $267 = $266;
      HEAP32[$267>>2] = $261;
      $268 = (($266) + 4)|0;
      $269 = $268;
      HEAP32[$269>>2] = $264;
      $270 = HEAP32[$4>>2]|0;
      $271 = HEAP32[$pNew>>2]|0;
      $272 = (_whereLoopInsert($270,$271)|0);
      HEAP32[$rc>>2] = $272;
      $273 = HEAP32[$i>>2]|0;
      $274 = (($273) + 1)|0;
      HEAP32[$i>>2] = $274;
     }
    }
   }
  } while(0);
  $275 = HEAP32[$pTerm>>2]|0;
  $276 = ((($275)) + 48|0);
  HEAP32[$pTerm>>2] = $276;
 }
 $277 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($277|0);
}
function _whereLoopClear($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = ($4|0)!=($6|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 48|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3DbFree($8,$11);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 _whereLoopClearUnion($12,$13);
 $14 = HEAP32[$1>>2]|0;
 _whereLoopInit($14);
 STACKTOP = sp;return;
}
function _allocateIndexInfo($pParse,$pWC,$0,$1,$pSrc,$pOrderBy) {
 $pParse = $pParse|0;
 $pWC = $pWC|0;
 $0 = $0|0;
 $1 = $1|0;
 $pSrc = $pSrc|0;
 $pOrderBy = $pOrderBy|0;
 var $$ = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0;
 var $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0;
 var $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0;
 var $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0;
 var $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0;
 var $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0;
 var $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0;
 var $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $j = 0, $n = 0, $nOrderBy = 0, $nTerm = 0, $op = 0, $pExpr = 0, $pExpr1 = 0, $pIdxCons = 0, $pIdxInfo = 0, $pIdxOrderBy = 0, $pTerm = 0, $pUsage = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $i = sp + 56|0;
 $j = sp + 52|0;
 $nTerm = sp + 48|0;
 $pIdxCons = sp + 44|0;
 $pIdxOrderBy = sp + 40|0;
 $pUsage = sp + 36|0;
 $pTerm = sp + 32|0;
 $nOrderBy = sp + 28|0;
 $pIdxInfo = sp + 24|0;
 $n = sp + 20|0;
 $pExpr = sp + 16|0;
 $op = sp + 80|0;
 $pExpr1 = sp + 12|0;
 HEAP32[$3>>2] = $pParse;
 HEAP32[$4>>2] = $pWC;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $pSrc;
 HEAP32[$7>>2] = $pOrderBy;
 HEAP32[$nTerm>>2] = 0;
 HEAP32[$i>>2] = 0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ((($12)) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pTerm>>2] = $14;
 while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$4>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$pTerm>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$6>>2]|0;
  $24 = ((($23)) + 44|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($22|0)!=($25|0);
  if (!($26)) {
   $27 = HEAP32[$pTerm>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = $28;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($29) + 4)|0;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = $5;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = $31 & $37;
   $42 = $34 & $40;
   $43 = ($41|0)!=(0);
   $44 = ($42|0)!=(0);
   $45 = $43 | $44;
   if (!($45)) {
    $46 = HEAP32[$pTerm>>2]|0;
    $47 = ((($46)) + 18|0);
    $48 = HEAP16[$47>>1]|0;
    $49 = $48&65535;
    $50 = $49 & -2433;
    $51 = ($50|0)==(0);
    if (!($51)) {
     $52 = HEAP32[$pTerm>>2]|0;
     $53 = ((($52)) + 20|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = $55 & 0;
     $57 = ($56|0)!=(0);
     if (!($57)) {
      $58 = HEAP32[$nTerm>>2]|0;
      $59 = (($58) + 1)|0;
      HEAP32[$nTerm>>2] = $59;
     }
    }
   }
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
  $62 = HEAP32[$pTerm>>2]|0;
  $63 = ((($62)) + 48|0);
  HEAP32[$pTerm>>2] = $63;
 }
 HEAP32[$nOrderBy>>2] = 0;
 $64 = HEAP32[$7>>2]|0;
 $65 = ($64|0)!=(0|0);
 if ($65) {
  $66 = HEAP32[$7>>2]|0;
  $67 = HEAP32[$66>>2]|0;
  HEAP32[$n>>2] = $67;
  HEAP32[$i>>2] = 0;
  while(1) {
   $68 = HEAP32[$i>>2]|0;
   $69 = HEAP32[$n>>2]|0;
   $70 = ($68|0)<($69|0);
   if (!($70)) {
    break;
   }
   $71 = HEAP32[$i>>2]|0;
   $72 = HEAP32[$7>>2]|0;
   $73 = ((($72)) + 4|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + (($71*20)|0)|0);
   $76 = HEAP32[$75>>2]|0;
   HEAP32[$pExpr>>2] = $76;
   $77 = HEAP32[$pExpr>>2]|0;
   $78 = HEAP8[$77>>0]|0;
   $79 = $78&255;
   $80 = ($79|0)!=(152);
   if ($80) {
    break;
   }
   $81 = HEAP32[$pExpr>>2]|0;
   $82 = ((($81)) + 28|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = HEAP32[$6>>2]|0;
   $85 = ((($84)) + 44|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($83|0)!=($86|0);
   if ($87) {
    break;
   }
   $88 = HEAP32[$i>>2]|0;
   $89 = (($88) + 1)|0;
   HEAP32[$i>>2] = $89;
  }
  $90 = HEAP32[$i>>2]|0;
  $91 = HEAP32[$n>>2]|0;
  $92 = ($90|0)==($91|0);
  if ($92) {
   $93 = HEAP32[$n>>2]|0;
   HEAP32[$nOrderBy>>2] = $93;
  }
 }
 $94 = HEAP32[$3>>2]|0;
 $95 = HEAP32[$94>>2]|0;
 $96 = HEAP32[$nTerm>>2]|0;
 $97 = ($96*20)|0;
 $98 = (72 + ($97))|0;
 $99 = HEAP32[$nOrderBy>>2]|0;
 $100 = $99<<3;
 $101 = (($98) + ($100))|0;
 $102 = (_sqlite3DbMallocZero($95,$101,0)|0);
 HEAP32[$pIdxInfo>>2] = $102;
 $103 = HEAP32[$pIdxInfo>>2]|0;
 $104 = ($103|0)==(0|0);
 if ($104) {
  $105 = HEAP32[$3>>2]|0;
  _sqlite3ErrorMsg($105,18773,$vararg_buffer);
  HEAP32[$2>>2] = 0;
  $242 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($242|0);
 }
 $106 = HEAP32[$pIdxInfo>>2]|0;
 $107 = ((($106)) + 72|0);
 HEAP32[$pIdxCons>>2] = $107;
 $108 = HEAP32[$nTerm>>2]|0;
 $109 = HEAP32[$pIdxCons>>2]|0;
 $110 = (($109) + (($108*12)|0)|0);
 HEAP32[$pIdxOrderBy>>2] = $110;
 $111 = HEAP32[$nOrderBy>>2]|0;
 $112 = HEAP32[$pIdxOrderBy>>2]|0;
 $113 = (($112) + ($111<<3)|0);
 HEAP32[$pUsage>>2] = $113;
 $114 = HEAP32[$nTerm>>2]|0;
 $115 = HEAP32[$pIdxInfo>>2]|0;
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$nOrderBy>>2]|0;
 $117 = HEAP32[$pIdxInfo>>2]|0;
 $118 = ((($117)) + 8|0);
 HEAP32[$118>>2] = $116;
 $119 = HEAP32[$pIdxCons>>2]|0;
 $120 = HEAP32[$pIdxInfo>>2]|0;
 $121 = ((($120)) + 4|0);
 HEAP32[$121>>2] = $119;
 $122 = HEAP32[$pIdxOrderBy>>2]|0;
 $123 = HEAP32[$pIdxInfo>>2]|0;
 $124 = ((($123)) + 12|0);
 HEAP32[$124>>2] = $122;
 $125 = HEAP32[$pUsage>>2]|0;
 $126 = HEAP32[$pIdxInfo>>2]|0;
 $127 = ((($126)) + 16|0);
 HEAP32[$127>>2] = $125;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 $128 = HEAP32[$4>>2]|0;
 $129 = ((($128)) + 20|0);
 $130 = HEAP32[$129>>2]|0;
 HEAP32[$pTerm>>2] = $130;
 while(1) {
  $131 = HEAP32[$i>>2]|0;
  $132 = HEAP32[$4>>2]|0;
  $133 = ((($132)) + 12|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = ($131|0)<($134|0);
  if (!($135)) {
   break;
  }
  $136 = HEAP32[$pTerm>>2]|0;
  $137 = ((($136)) + 8|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = HEAP32[$6>>2]|0;
  $140 = ((($139)) + 44|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($138|0)!=($141|0);
  if (!($142)) {
   $143 = HEAP32[$pTerm>>2]|0;
   $144 = ((($143)) + 32|0);
   $145 = $144;
   $146 = $145;
   $147 = HEAP32[$146>>2]|0;
   $148 = (($145) + 4)|0;
   $149 = $148;
   $150 = HEAP32[$149>>2]|0;
   $151 = $5;
   $152 = $151;
   $153 = HEAP32[$152>>2]|0;
   $154 = (($151) + 4)|0;
   $155 = $154;
   $156 = HEAP32[$155>>2]|0;
   $157 = $147 & $153;
   $158 = $150 & $156;
   $159 = ($157|0)!=(0);
   $160 = ($158|0)!=(0);
   $161 = $159 | $160;
   if (!($161)) {
    $162 = HEAP32[$pTerm>>2]|0;
    $163 = ((($162)) + 18|0);
    $164 = HEAP16[$163>>1]|0;
    $165 = $164&65535;
    $166 = $165 & -2433;
    $167 = ($166|0)==(0);
    if (!($167)) {
     $168 = HEAP32[$pTerm>>2]|0;
     $169 = ((($168)) + 20|0);
     $170 = HEAP16[$169>>1]|0;
     $171 = $170&65535;
     $172 = $171 & 0;
     $173 = ($172|0)!=(0);
     if (!($173)) {
      $174 = HEAP32[$pTerm>>2]|0;
      $175 = ((($174)) + 12|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = HEAP32[$j>>2]|0;
      $178 = HEAP32[$pIdxCons>>2]|0;
      $179 = (($178) + (($177*12)|0)|0);
      HEAP32[$179>>2] = $176;
      $180 = HEAP32[$i>>2]|0;
      $181 = HEAP32[$j>>2]|0;
      $182 = HEAP32[$pIdxCons>>2]|0;
      $183 = (($182) + (($181*12)|0)|0);
      $184 = ((($183)) + 8|0);
      HEAP32[$184>>2] = $180;
      $185 = HEAP32[$pTerm>>2]|0;
      $186 = ((($185)) + 18|0);
      $187 = HEAP16[$186>>1]|0;
      $188 = $187&255;
      $189 = $188&255;
      $190 = $189 & 8191;
      $191 = $190&255;
      HEAP8[$op>>0] = $191;
      $192 = HEAP8[$op>>0]|0;
      $193 = $192&255;
      $194 = ($193|0)==(1);
      $$ = $194 ? 2 : $191;
      HEAP8[$op>>0] = $$;
      $195 = HEAP8[$op>>0]|0;
      $196 = $195&255;
      $197 = ($196|0)==(64);
      if ($197) {
       $198 = HEAP32[$pTerm>>2]|0;
       $199 = ((($198)) + 23|0);
       $200 = HEAP8[$199>>0]|0;
       HEAP8[$op>>0] = $200;
      }
      $201 = HEAP8[$op>>0]|0;
      $202 = HEAP32[$j>>2]|0;
      $203 = HEAP32[$pIdxCons>>2]|0;
      $204 = (($203) + (($202*12)|0)|0);
      $205 = ((($204)) + 4|0);
      HEAP8[$205>>0] = $201;
      $206 = HEAP32[$j>>2]|0;
      $207 = (($206) + 1)|0;
      HEAP32[$j>>2] = $207;
     }
    }
   }
  }
  $208 = HEAP32[$i>>2]|0;
  $209 = (($208) + 1)|0;
  HEAP32[$i>>2] = $209;
  $210 = HEAP32[$pTerm>>2]|0;
  $211 = ((($210)) + 48|0);
  HEAP32[$pTerm>>2] = $211;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $212 = HEAP32[$i>>2]|0;
  $213 = HEAP32[$nOrderBy>>2]|0;
  $214 = ($212|0)<($213|0);
  if (!($214)) {
   break;
  }
  $215 = HEAP32[$i>>2]|0;
  $216 = HEAP32[$7>>2]|0;
  $217 = ((($216)) + 4|0);
  $218 = HEAP32[$217>>2]|0;
  $219 = (($218) + (($215*20)|0)|0);
  $220 = HEAP32[$219>>2]|0;
  HEAP32[$pExpr1>>2] = $220;
  $221 = HEAP32[$pExpr1>>2]|0;
  $222 = ((($221)) + 32|0);
  $223 = HEAP16[$222>>1]|0;
  $224 = $223 << 16 >> 16;
  $225 = HEAP32[$i>>2]|0;
  $226 = HEAP32[$pIdxOrderBy>>2]|0;
  $227 = (($226) + ($225<<3)|0);
  HEAP32[$227>>2] = $224;
  $228 = HEAP32[$i>>2]|0;
  $229 = HEAP32[$7>>2]|0;
  $230 = ((($229)) + 4|0);
  $231 = HEAP32[$230>>2]|0;
  $232 = (($231) + (($228*20)|0)|0);
  $233 = ((($232)) + 12|0);
  $234 = HEAP8[$233>>0]|0;
  $235 = HEAP32[$i>>2]|0;
  $236 = HEAP32[$pIdxOrderBy>>2]|0;
  $237 = (($236) + ($235<<3)|0);
  $238 = ((($237)) + 4|0);
  HEAP8[$238>>0] = $234;
  $239 = HEAP32[$i>>2]|0;
  $240 = (($239) + 1)|0;
  HEAP32[$i>>2] = $240;
 }
 $241 = HEAP32[$pIdxInfo>>2]|0;
 HEAP32[$2>>2] = $241;
 $242 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($242|0);
}
function _whereLoopResize($db,$p,$n) {
 $db = $db|0;
 $p = $p|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $paNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $paNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = (($10) + 7)|0;
 $12 = $11 & -8;
 HEAP32[$3>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = $14<<2;
 $16 = (_sqlite3DbMallocRawNN($13,$15,0)|0);
 HEAP32[$paNew>>2] = $16;
 $17 = HEAP32[$paNew>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$0>>2] = 7;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $19 = HEAP32[$paNew>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26<<2;
 _memcpy(($19|0),($22|0),($27|0))|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 48|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 $32 = ((($31)) + 56|0);
 $33 = ($30|0)!=($32|0);
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 48|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3DbFree($34,$37);
 }
 $38 = HEAP32[$paNew>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = ((($39)) + 48|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$3>>2]|0;
 $42 = $41&65535;
 $43 = HEAP32[$2>>2]|0;
 $44 = ((($43)) + 44|0);
 HEAP16[$44>>1] = $42;
 HEAP32[$0>>2] = 0;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _vtabBestIndex($pParse,$pTab,$p) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pVtab = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pVtab = sp + 28|0;
 $i = sp + 24|0;
 $rc = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3GetVTable($4,$5)|0);
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pVtab>>2] = $8;
 $9 = HEAP32[$pVtab>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pVtab>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (FUNCTION_TABLE_iii[$12 & 255]($13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 do {
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   $19 = ($18|0)==(7);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$20>>2]|0;
    _sqlite3OomFault($21);
    break;
   }
   $22 = HEAP32[$pVtab>>2]|0;
   $23 = ((($22)) + 8|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0|0);
   $26 = HEAP32[$0>>2]|0;
   if ($25) {
    $29 = HEAP32[$pVtab>>2]|0;
    $30 = ((($29)) + 8|0);
    $31 = HEAP32[$30>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $31;
    _sqlite3ErrorMsg($26,17521,$vararg_buffer1);
    break;
   } else {
    $27 = HEAP32[$rc>>2]|0;
    $28 = (_sqlite3ErrStr($27)|0);
    HEAP32[$vararg_buffer>>2] = $28;
    _sqlite3ErrorMsg($26,17521,$vararg_buffer);
    break;
   }
  }
 } while(0);
 $32 = HEAP32[$pVtab>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 _sqlite3_free($34);
 $35 = HEAP32[$pVtab>>2]|0;
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $37 = HEAP32[$i>>2]|0;
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = ($37|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($41*12)|0)|0);
  $46 = ((($45)) + 5|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = ($47<<24>>24)!=(0);
  if (!($48)) {
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$2>>2]|0;
   $51 = ((($50)) + 16|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + ($49<<3)|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = ($54|0)>(0);
   if ($55) {
    $56 = HEAP32[$0>>2]|0;
    $57 = HEAP32[$1>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $58;
    _sqlite3ErrorMsg($56,32616,$vararg_buffer4);
   }
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$i>>2] = $60;
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 68|0);
 $63 = HEAP32[$62>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _sqlite3LogEstFromDouble($x) {
 $x = +$x;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0.0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $e = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 18|0;
 $1 = sp + 8|0;
 $a = sp;
 $e = sp + 16|0;
 HEAPF64[$1>>3] = $x;
 $2 = +HEAPF64[$1>>3];
 $3 = $2 <= 1.0;
 if ($3) {
  HEAP16[$0>>1] = 0;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 }
 $4 = +HEAPF64[$1>>3];
 $5 = $4 <= 2.0E+9;
 if ($5) {
  $6 = +HEAPF64[$1>>3];
  $7 = (~~$6)>>>0;
  $8 = +Math_abs($6) >= 1.0 ? $6 > 0.0 ? (~~+Math_min(+Math_floor($6 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($6 - +(~~$6 >>> 0)) / 4294967296.0) >>> 0 : 0;
  $9 = (_sqlite3LogEst($7,$8)|0);
  HEAP16[$0>>1] = $9;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  ;HEAP32[$a>>2]=HEAP32[$1>>2]|0;HEAP32[$a+4>>2]=HEAP32[$1+4>>2]|0;
  $10 = $a;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (_bitshift64Lshr(($12|0),($15|0),52)|0);
  $17 = tempRet0;
  $18 = (_i64Subtract(($16|0),($17|0),1022,0)|0);
  $19 = tempRet0;
  $20 = $18&65535;
  HEAP16[$e>>1] = $20;
  $21 = HEAP16[$e>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22*10)|0;
  $24 = $23&65535;
  HEAP16[$0>>1] = $24;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _whereLoopInsert($pBuilder,$pTemplate) {
 $pBuilder = $pBuilder|0;
 $pTemplate = $pTemplate|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $db = 0, $p = 0, $pIndex = 0, $pToDel = 0;
 var $pWInfo = 0, $ppPrev = 0, $ppTail = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $ppPrev = sp + 24|0;
 $p = sp + 20|0;
 $pWInfo = sp + 16|0;
 $db = sp + 12|0;
 $ppTail = sp + 8|0;
 $pToDel = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pTemplate;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = HEAP32[$pWInfo>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 40|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = ($14<<16>>16)!=(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = $19;
   $21 = $20;
   $22 = HEAP32[$21>>2]|0;
   $23 = (($20) + 4)|0;
   $24 = $23;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = ((($26)) + 20|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = ((($29)) + 22|0);
   $31 = HEAP16[$30>>1]|0;
   (_whereOrInsert($18,$22,$25,$28,$31)|0);
  }
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $32 = HEAP32[$pWInfo>>2]|0;
 $33 = ((($32)) + 16|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 _whereLoopAdjustCost($34,$35);
 $36 = HEAP32[$pWInfo>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = HEAP32[$2>>2]|0;
 $39 = (_whereLoopFindLesser($37,$38)|0);
 HEAP32[$ppPrev>>2] = $39;
 $40 = HEAP32[$ppPrev>>2]|0;
 $41 = ($40|0)==(0|0);
 if ($41) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $42 = HEAP32[$ppPrev>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$p>>2] = $43;
 $44 = HEAP32[$p>>2]|0;
 $45 = ($44|0)==(0|0);
 L12: do {
  if ($45) {
   $46 = HEAP32[$db>>2]|0;
   $47 = (_sqlite3DbMallocRawNN($46,72,0)|0);
   HEAP32[$p>>2] = $47;
   $48 = HEAP32[$ppPrev>>2]|0;
   HEAP32[$48>>2] = $47;
   $49 = HEAP32[$p>>2]|0;
   $50 = ($49|0)==(0|0);
   if (!($50)) {
    $51 = HEAP32[$p>>2]|0;
    _whereLoopInit($51);
    $52 = HEAP32[$p>>2]|0;
    $53 = ((($52)) + 52|0);
    HEAP32[$53>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $95 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($95|0);
  } else {
   $54 = HEAP32[$p>>2]|0;
   $55 = ((($54)) + 52|0);
   HEAP32[$ppTail>>2] = $55;
   while(1) {
    $56 = HEAP32[$ppTail>>2]|0;
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0|0);
    if (!($58)) {
     break L12;
    }
    $59 = HEAP32[$ppTail>>2]|0;
    $60 = HEAP32[$2>>2]|0;
    $61 = (_whereLoopFindLesser($59,$60)|0);
    HEAP32[$ppTail>>2] = $61;
    $62 = HEAP32[$ppTail>>2]|0;
    $63 = ($62|0)==(0|0);
    if ($63) {
     break L12;
    }
    $64 = HEAP32[$ppTail>>2]|0;
    $65 = HEAP32[$64>>2]|0;
    HEAP32[$pToDel>>2] = $65;
    $66 = HEAP32[$pToDel>>2]|0;
    $67 = ($66|0)==(0|0);
    if ($67) {
     break L12;
    }
    $68 = HEAP32[$pToDel>>2]|0;
    $69 = ((($68)) + 52|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = HEAP32[$ppTail>>2]|0;
    HEAP32[$71>>2] = $70;
    $72 = HEAP32[$db>>2]|0;
    $73 = HEAP32[$pToDel>>2]|0;
    _whereLoopDelete($72,$73);
   }
  }
 } while(0);
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$p>>2]|0;
 $76 = HEAP32[$2>>2]|0;
 (_whereLoopXfer($74,$75,$76)|0);
 $77 = HEAP32[$p>>2]|0;
 $78 = ((($77)) + 36|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = $79 & 1024;
 $81 = ($80|0)==(0);
 if ($81) {
  $82 = HEAP32[$p>>2]|0;
  $83 = ((($82)) + 24|0);
  $84 = ((($83)) + 4|0);
  $85 = HEAP32[$84>>2]|0;
  HEAP32[$pIndex>>2] = $85;
  $86 = HEAP32[$pIndex>>2]|0;
  $87 = ($86|0)!=(0|0);
  if ($87) {
   $88 = HEAP32[$pIndex>>2]|0;
   $89 = ((($88)) + 44|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = ($90|0)==(0);
   if ($91) {
    $92 = HEAP32[$p>>2]|0;
    $93 = ((($92)) + 24|0);
    $94 = ((($93)) + 4|0);
    HEAP32[$94>>2] = 0;
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $95 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _whereOrInsert($pSet,$0,$1,$rRun,$nOut) {
 $pSet = $pSet|0;
 $0 = $0|0;
 $1 = $1|0;
 $rRun = $rRun|0;
 $nOut = $nOut|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 24|0;
 $6 = sp + 22|0;
 $i = sp + 20|0;
 $p = sp + 8|0;
 HEAP32[$3>>2] = $pSet;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP16[$5>>1] = $rRun;
 HEAP16[$6>>1] = $nOut;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP16[$11>>1]|0;
 HEAP16[$i>>1] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ((($13)) + 8|0);
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP16[$i>>1]|0;
  $16 = $15&65535;
  $17 = ($16|0)>(0);
  if (!($17)) {
   label = 9;
   break;
  }
  $18 = HEAP16[$5>>1]|0;
  $19 = $18 << 16 >> 16;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22 << 16 >> 16;
  $24 = ($19|0)<=($23|0);
  if ($24) {
   $25 = $4;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (($25) + 4)|0;
   $29 = $28;
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$p>>2]|0;
   $32 = $31;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = (($32) + 4)|0;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = $27 & $34;
   $39 = $30 & $37;
   $40 = $4;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = (($40) + 4)|0;
   $44 = $43;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($38|0)==($42|0);
   $47 = ($39|0)==($45|0);
   $48 = $46 & $47;
   if ($48) {
    break;
   }
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 8|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51 << 16 >> 16;
  $53 = HEAP16[$5>>1]|0;
  $54 = $53 << 16 >> 16;
  $55 = ($52|0)<=($54|0);
  if ($55) {
   $56 = HEAP32[$p>>2]|0;
   $57 = $56;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (($57) + 4)|0;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = $4;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = (($63) + 4)|0;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = $59 & $65;
   $70 = $62 & $68;
   $71 = HEAP32[$p>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = ($69|0)==($74|0);
   $79 = ($70|0)==($77|0);
   $80 = $78 & $79;
   if ($80) {
    label = 7;
    break;
   }
  }
  $81 = HEAP16[$i>>1]|0;
  $82 = (($81) + -1)<<16>>16;
  HEAP16[$i>>1] = $82;
  $83 = HEAP32[$p>>2]|0;
  $84 = ((($83)) + 16|0);
  HEAP32[$p>>2] = $84;
 }
 if ((label|0) == 7) {
  HEAP32[$2>>2] = 0;
  $153 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($153|0);
 }
 do {
  if ((label|0) == 9) {
   $85 = HEAP32[$3>>2]|0;
   $86 = HEAP16[$85>>1]|0;
   $87 = $86&65535;
   $88 = ($87|0)<(3);
   $89 = HEAP32[$3>>2]|0;
   if ($88) {
    $90 = HEAP16[$89>>1]|0;
    $91 = (($90) + 1)<<16>>16;
    HEAP16[$89>>1] = $91;
    $92 = $90&65535;
    $93 = HEAP32[$3>>2]|0;
    $94 = ((($93)) + 8|0);
    $95 = (($94) + ($92<<4)|0);
    HEAP32[$p>>2] = $95;
    $96 = HEAP16[$6>>1]|0;
    $97 = HEAP32[$p>>2]|0;
    $98 = ((($97)) + 10|0);
    HEAP16[$98>>1] = $96;
    break;
   }
   $99 = ((($89)) + 8|0);
   HEAP32[$p>>2] = $99;
   HEAP16[$i>>1] = 1;
   while(1) {
    $100 = HEAP16[$i>>1]|0;
    $101 = $100&65535;
    $102 = HEAP32[$3>>2]|0;
    $103 = HEAP16[$102>>1]|0;
    $104 = $103&65535;
    $105 = ($101|0)<($104|0);
    $106 = HEAP32[$p>>2]|0;
    $107 = ((($106)) + 8|0);
    $108 = HEAP16[$107>>1]|0;
    $109 = $108 << 16 >> 16;
    if (!($105)) {
     break;
    }
    $110 = HEAP16[$i>>1]|0;
    $111 = $110&65535;
    $112 = HEAP32[$3>>2]|0;
    $113 = ((($112)) + 8|0);
    $114 = (($113) + ($111<<4)|0);
    $115 = ((($114)) + 8|0);
    $116 = HEAP16[$115>>1]|0;
    $117 = $116 << 16 >> 16;
    $118 = ($109|0)>($117|0);
    if ($118) {
     $119 = HEAP32[$3>>2]|0;
     $120 = ((($119)) + 8|0);
     $121 = HEAP16[$i>>1]|0;
     $122 = $121&65535;
     $123 = (($120) + ($122<<4)|0);
     HEAP32[$p>>2] = $123;
    }
    $124 = HEAP16[$i>>1]|0;
    $125 = (($124) + 1)<<16>>16;
    HEAP16[$i>>1] = $125;
   }
   $126 = HEAP16[$5>>1]|0;
   $127 = $126 << 16 >> 16;
   $128 = ($109|0)<=($127|0);
   if ($128) {
    HEAP32[$2>>2] = 0;
    $153 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($153|0);
   }
  }
 } while(0);
 $129 = $4;
 $130 = $129;
 $131 = HEAP32[$130>>2]|0;
 $132 = (($129) + 4)|0;
 $133 = $132;
 $134 = HEAP32[$133>>2]|0;
 $135 = HEAP32[$p>>2]|0;
 $136 = $135;
 $137 = $136;
 HEAP32[$137>>2] = $131;
 $138 = (($136) + 4)|0;
 $139 = $138;
 HEAP32[$139>>2] = $134;
 $140 = HEAP16[$5>>1]|0;
 $141 = HEAP32[$p>>2]|0;
 $142 = ((($141)) + 8|0);
 HEAP16[$142>>1] = $140;
 $143 = HEAP32[$p>>2]|0;
 $144 = ((($143)) + 10|0);
 $145 = HEAP16[$144>>1]|0;
 $146 = $145 << 16 >> 16;
 $147 = HEAP16[$6>>1]|0;
 $148 = $147 << 16 >> 16;
 $149 = ($146|0)>($148|0);
 if ($149) {
  $150 = HEAP16[$6>>1]|0;
  $151 = HEAP32[$p>>2]|0;
  $152 = ((($151)) + 10|0);
  HEAP16[$152>>1] = $150;
 }
 HEAP32[$2>>2] = 1;
 $153 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _whereLoopAdjustCost($p,$pTemplate) {
 $p = $p|0;
 $pTemplate = $pTemplate|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pTemplate;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 512;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 $8 = ($7|0)!=(0|0);
 $or$cond = $6 & $8;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 16|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($12|0)!=($16|0);
  do {
   if (!($17)) {
    $18 = HEAP32[$0>>2]|0;
    $19 = ((($18)) + 36|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $20 & 512;
    $22 = ($21|0)==(0);
    if (!($22)) {
     $23 = HEAP32[$0>>2]|0;
     $24 = HEAP32[$1>>2]|0;
     $25 = (_whereLoopCheaperProperSubset($23,$24)|0);
     $26 = ($25|0)!=(0);
     if ($26) {
      $27 = HEAP32[$0>>2]|0;
      $28 = ((($27)) + 20|0);
      $29 = HEAP16[$28>>1]|0;
      $30 = HEAP32[$1>>2]|0;
      $31 = ((($30)) + 20|0);
      HEAP16[$31>>1] = $29;
      $32 = HEAP32[$0>>2]|0;
      $33 = ((($32)) + 22|0);
      $34 = HEAP16[$33>>1]|0;
      $35 = $34 << 16 >> 16;
      $36 = (($35) - 1)|0;
      $37 = $36&65535;
      $38 = HEAP32[$1>>2]|0;
      $39 = ((($38)) + 22|0);
      HEAP16[$39>>1] = $37;
      break;
     }
     $40 = HEAP32[$1>>2]|0;
     $41 = HEAP32[$0>>2]|0;
     $42 = (_whereLoopCheaperProperSubset($40,$41)|0);
     $43 = ($42|0)!=(0);
     if ($43) {
      $44 = HEAP32[$0>>2]|0;
      $45 = ((($44)) + 20|0);
      $46 = HEAP16[$45>>1]|0;
      $47 = HEAP32[$1>>2]|0;
      $48 = ((($47)) + 20|0);
      HEAP16[$48>>1] = $46;
      $49 = HEAP32[$0>>2]|0;
      $50 = ((($49)) + 22|0);
      $51 = HEAP16[$50>>1]|0;
      $52 = $51 << 16 >> 16;
      $53 = (($52) + 1)|0;
      $54 = $53&65535;
      $55 = HEAP32[$1>>2]|0;
      $56 = ((($55)) + 22|0);
      HEAP16[$56>>1] = $54;
     }
    }
   }
  } while(0);
  $57 = HEAP32[$0>>2]|0;
  $58 = ((($57)) + 52|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$0>>2] = $59;
  $$old = HEAP32[$0>>2]|0;
  $$old1 = ($$old|0)!=(0|0);
  if (!($$old1)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _whereLoopFindLesser($ppPrev,$pTemplate) {
 $ppPrev = $ppPrev|0;
 $pTemplate = $pTemplate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $ppPrev;
 HEAP32[$2>>2] = $pTemplate;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 19;
   break;
  }
  $7 = HEAP32[$p>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 16|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($10|0)!=($14|0);
  if (!($15)) {
   $16 = HEAP32[$p>>2]|0;
   $17 = ((($16)) + 17|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 17|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($19|0)!=($23|0);
   if (!($24)) {
    $25 = HEAP32[$p>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = $27 & 16384;
    $29 = ($28|0)!=(0);
    if ($29) {
     $30 = HEAP32[$2>>2]|0;
     $31 = ((($30)) + 42|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32&65535;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = HEAP32[$2>>2]|0;
      $36 = ((($35)) + 36|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = $37 & 512;
      $39 = ($38|0)!=(0);
      if ($39) {
       $40 = HEAP32[$2>>2]|0;
       $41 = ((($40)) + 36|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = $42 & 1;
       $44 = ($43|0)!=(0);
       if ($44) {
        $45 = HEAP32[$p>>2]|0;
        $46 = $45;
        $47 = $46;
        $48 = HEAP32[$47>>2]|0;
        $49 = (($46) + 4)|0;
        $50 = $49;
        $51 = HEAP32[$50>>2]|0;
        $52 = HEAP32[$2>>2]|0;
        $53 = $52;
        $54 = $53;
        $55 = HEAP32[$54>>2]|0;
        $56 = (($53) + 4)|0;
        $57 = $56;
        $58 = HEAP32[$57>>2]|0;
        $59 = $48 & $55;
        $60 = $51 & $58;
        $61 = HEAP32[$2>>2]|0;
        $62 = $61;
        $63 = $62;
        $64 = HEAP32[$63>>2]|0;
        $65 = (($62) + 4)|0;
        $66 = $65;
        $67 = HEAP32[$66>>2]|0;
        $68 = ($59|0)==($64|0);
        $69 = ($60|0)==($67|0);
        $70 = $68 & $69;
        if ($70) {
         label = 19;
         break;
        }
       }
      }
     }
    }
    $71 = HEAP32[$p>>2]|0;
    $72 = $71;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$2>>2]|0;
    $79 = $78;
    $80 = $79;
    $81 = HEAP32[$80>>2]|0;
    $82 = (($79) + 4)|0;
    $83 = $82;
    $84 = HEAP32[$83>>2]|0;
    $85 = $74 & $81;
    $86 = $77 & $84;
    $87 = HEAP32[$p>>2]|0;
    $88 = $87;
    $89 = $88;
    $90 = HEAP32[$89>>2]|0;
    $91 = (($88) + 4)|0;
    $92 = $91;
    $93 = HEAP32[$92>>2]|0;
    $94 = ($85|0)==($90|0);
    $95 = ($86|0)==($93|0);
    $96 = $94 & $95;
    if ($96) {
     $97 = HEAP32[$p>>2]|0;
     $98 = ((($97)) + 18|0);
     $99 = HEAP16[$98>>1]|0;
     $100 = $99 << 16 >> 16;
     $101 = HEAP32[$2>>2]|0;
     $102 = ((($101)) + 18|0);
     $103 = HEAP16[$102>>1]|0;
     $104 = $103 << 16 >> 16;
     $105 = ($100|0)<=($104|0);
     if ($105) {
      $106 = HEAP32[$p>>2]|0;
      $107 = ((($106)) + 20|0);
      $108 = HEAP16[$107>>1]|0;
      $109 = $108 << 16 >> 16;
      $110 = HEAP32[$2>>2]|0;
      $111 = ((($110)) + 20|0);
      $112 = HEAP16[$111>>1]|0;
      $113 = $112 << 16 >> 16;
      $114 = ($109|0)<=($113|0);
      if ($114) {
       $115 = HEAP32[$p>>2]|0;
       $116 = ((($115)) + 22|0);
       $117 = HEAP16[$116>>1]|0;
       $118 = $117 << 16 >> 16;
       $119 = HEAP32[$2>>2]|0;
       $120 = ((($119)) + 22|0);
       $121 = HEAP16[$120>>1]|0;
       $122 = $121 << 16 >> 16;
       $123 = ($118|0)<=($122|0);
       if ($123) {
        label = 14;
        break;
       }
      }
     }
    }
    $124 = HEAP32[$p>>2]|0;
    $125 = $124;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (($125) + 4)|0;
    $129 = $128;
    $130 = HEAP32[$129>>2]|0;
    $131 = HEAP32[$2>>2]|0;
    $132 = $131;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (($132) + 4)|0;
    $136 = $135;
    $137 = HEAP32[$136>>2]|0;
    $138 = $127 & $134;
    $139 = $130 & $137;
    $140 = HEAP32[$2>>2]|0;
    $141 = $140;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = (($141) + 4)|0;
    $145 = $144;
    $146 = HEAP32[$145>>2]|0;
    $147 = ($138|0)==($143|0);
    $148 = ($139|0)==($146|0);
    $149 = $147 & $148;
    if ($149) {
     $150 = HEAP32[$p>>2]|0;
     $151 = ((($150)) + 20|0);
     $152 = HEAP16[$151>>1]|0;
     $153 = $152 << 16 >> 16;
     $154 = HEAP32[$2>>2]|0;
     $155 = ((($154)) + 20|0);
     $156 = HEAP16[$155>>1]|0;
     $157 = $156 << 16 >> 16;
     $158 = ($153|0)>=($157|0);
     if ($158) {
      $159 = HEAP32[$p>>2]|0;
      $160 = ((($159)) + 22|0);
      $161 = HEAP16[$160>>1]|0;
      $162 = $161 << 16 >> 16;
      $163 = HEAP32[$2>>2]|0;
      $164 = ((($163)) + 22|0);
      $165 = HEAP16[$164>>1]|0;
      $166 = $165 << 16 >> 16;
      $167 = ($162|0)>=($166|0);
      if ($167) {
       label = 19;
       break;
      }
     }
    }
   }
  }
  $168 = HEAP32[$p>>2]|0;
  $169 = ((($168)) + 52|0);
  HEAP32[$1>>2] = $169;
  $170 = HEAP32[$1>>2]|0;
  $171 = HEAP32[$170>>2]|0;
  HEAP32[$p>>2] = $171;
 }
 if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $173 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($173|0);
 }
 else if ((label|0) == 19) {
  $172 = HEAP32[$1>>2]|0;
  HEAP32[$0>>2] = $172;
  $173 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($173|0);
 }
 return (0)|0;
}
function _whereLoopDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _whereLoopClear($2,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($4,$5);
 STACKTOP = sp;return;
}
function _whereLoopXfer($db,$pTo,$pFrom) {
 $db = $db|0;
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pTo;
 HEAP32[$3>>2] = $pFrom;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _whereLoopClearUnion($4,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = (_whereLoopResize($6,$7,$11)|0);
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$2>>2]|0;
 if ($13) {
  $15 = ((($14)) + 24|0);
  ;HEAP32[$15>>2]=0|0;HEAP32[$15+4>>2]=0|0;HEAP32[$15+8>>2]=0|0;
  HEAP32[$0>>2] = 7;
  $43 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $16 = HEAP32[$3>>2]|0;
 dest=$14; src=$16; stop=dest+44|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 40|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26<<2;
 _memcpy(($19|0),($22|0),($27|0))|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = ((($28)) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 & 1024;
 $32 = ($31|0)!=(0);
 $33 = HEAP32[$3>>2]|0;
 if ($32) {
  $34 = ((($33)) + 24|0);
  $35 = ((($34)) + 4|0);
  HEAP8[$35>>0] = 0;
 } else {
  $36 = ((($33)) + 36|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $37 & 16384;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$3>>2]|0;
   $41 = ((($40)) + 24|0);
   $42 = ((($41)) + 4|0);
   HEAP32[$42>>2] = 0;
  }
 }
 HEAP32[$0>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _whereLoopCheaperProperSubset($pX,$pY) {
 $pX = $pX|0;
 $pY = $pY|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $i = 0;
 var $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pX;
 HEAP32[$2>>2] = $pY;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 42|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = (($6) - ($10))|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 40|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 42|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($15) - ($19))|0;
 $21 = ($11|0)>=($20|0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = ((($22)) + 42|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 42|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = ($25|0)>($29|0);
 if ($30) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 20|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33 << 16 >> 16;
 $35 = HEAP32[$2>>2]|0;
 $36 = ((($35)) + 20|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37 << 16 >> 16;
 $39 = ($34|0)>=($38|0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 20|0);
  $42 = HEAP16[$41>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = HEAP32[$2>>2]|0;
  $45 = ((($44)) + 20|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46 << 16 >> 16;
  $48 = ($43|0)>($47|0);
  if ($48) {
   HEAP32[$0>>2] = 0;
   $98 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($98|0);
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 22|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51 << 16 >> 16;
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 22|0);
  $55 = HEAP16[$54>>1]|0;
  $56 = $55 << 16 >> 16;
  $57 = ($52|0)>($56|0);
  if ($57) {
   HEAP32[$0>>2] = 0;
   $98 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($98|0);
  }
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 40|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 $62 = (($61) - 1)|0;
 HEAP32[$i>>2] = $62;
 while(1) {
  $63 = HEAP32[$i>>2]|0;
  $64 = ($63|0)>=(0);
  if (!($64)) {
   label = 20;
   break;
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$1>>2]|0;
  $67 = ((($66)) + 48|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + ($65<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = ($70|0)==(0|0);
  if (!($71)) {
   $72 = HEAP32[$2>>2]|0;
   $73 = ((($72)) + 40|0);
   $74 = HEAP16[$73>>1]|0;
   $75 = $74&65535;
   $76 = (($75) - 1)|0;
   HEAP32[$j>>2] = $76;
   while(1) {
    $77 = HEAP32[$j>>2]|0;
    $78 = ($77|0)>=(0);
    if (!($78)) {
     break;
    }
    $79 = HEAP32[$j>>2]|0;
    $80 = HEAP32[$2>>2]|0;
    $81 = ((($80)) + 48|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (($82) + ($79<<2)|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = HEAP32[$i>>2]|0;
    $86 = HEAP32[$1>>2]|0;
    $87 = ((($86)) + 48|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = (($88) + ($85<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($84|0)==($90|0);
    if ($91) {
     break;
    }
    $92 = HEAP32[$j>>2]|0;
    $93 = (($92) + -1)|0;
    HEAP32[$j>>2] = $93;
   }
   $94 = HEAP32[$j>>2]|0;
   $95 = ($94|0)<(0);
   if ($95) {
    label = 18;
    break;
   }
  }
  $96 = HEAP32[$i>>2]|0;
  $97 = (($96) + -1)|0;
  HEAP32[$i>>2] = $97;
 }
 if ((label|0) == 18) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 else if ((label|0) == 20) {
  HEAP32[$0>>2] = 1;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 return (0)|0;
}
function _whereLoopClearUnion($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 17408;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 1024;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 24|0);
  $14 = ((($13)) + 4|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 24|0);
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3_free($21);
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 24|0);
   $24 = ((($23)) + 4|0);
   HEAP8[$24>>0] = 0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 24|0);
   $27 = ((($26)) + 8|0);
   HEAP32[$27>>2] = 0;
   STACKTOP = sp;return;
  }
 }
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 & 16384;
 $32 = ($31|0)!=(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 24|0);
 $35 = ((($34)) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)!=(0|0);
 if (!($37)) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 24|0);
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 _sqlite3DbFree($38,$44);
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$1>>2]|0;
 $47 = ((($46)) + 24|0);
 $48 = ((($47)) + 4|0);
 $49 = HEAP32[$48>>2]|0;
 _sqlite3DbFree($45,$49);
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = ((($51)) + 4|0);
 HEAP32[$52>>2] = 0;
 STACKTOP = sp;return;
}
function _estLog($N) {
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP16[$0>>1] = $N;
 $1 = HEAP16[$0>>1]|0;
 $2 = $1 << 16 >> 16;
 $3 = ($2|0)<=(10);
 if ($3) {
  $12 = 0;
  $11 = $12&65535;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP16[$0>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3LogEst($5,$7)|0);
 $9 = $8 << 16 >> 16;
 $10 = (($9) - 33)|0;
 $12 = $10;
 $11 = $12&65535;
 STACKTOP = sp;return ($11|0);
}
function _termCanDriveIndex($pTerm,$pSrc,$0,$1) {
 $pTerm = $pTerm|0;
 $pSrc = $pSrc|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 $aff = sp + 20|0;
 HEAP32[$3>>2] = $pTerm;
 HEAP32[$4>>2] = $pSrc;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$3>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($12|0)!=($15|0);
 if ($16) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 18|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 130;
 $22 = ($21|0)==(0);
 if ($22) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $23 = HEAP32[$3>>2]|0;
 $24 = ((($23)) + 32|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = $5;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = $27 & $33;
 $38 = $30 & $36;
 $39 = ($37|0)!=(0);
 $40 = ($38|0)!=(0);
 $41 = $39 | $40;
 if ($41) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $42 = HEAP32[$3>>2]|0;
 $43 = ((($42)) + 12|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)<(0);
 if ($45) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $46 = HEAP32[$3>>2]|0;
 $47 = ((($46)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$4>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ((($51)) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) + (($48*24)|0)|0);
 $55 = ((($54)) + 21|0);
 $56 = HEAP8[$55>>0]|0;
 HEAP8[$aff>>0] = $56;
 $57 = HEAP32[$3>>2]|0;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP8[$aff>>0]|0;
 $60 = (_sqlite3IndexAffinityOk($58,$59)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  HEAP32[$2>>2] = 1;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 } else {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 return (0)|0;
}
function _sqlite3LogEstAdd($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 2|0;
 $2 = sp;
 HEAP16[$1>>1] = $a;
 HEAP16[$2>>1] = $b;
 $3 = HEAP16[$1>>1]|0;
 $4 = $3 << 16 >> 16;
 $5 = HEAP16[$2>>1]|0;
 $6 = $5 << 16 >> 16;
 $7 = ($4|0)>=($6|0);
 if ($7) {
  $8 = HEAP16[$1>>1]|0;
  $9 = $8 << 16 >> 16;
  $10 = HEAP16[$2>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = (($11) + 49)|0;
  $13 = ($9|0)>($12|0);
  $14 = HEAP16[$1>>1]|0;
  if ($13) {
   HEAP16[$0>>1] = $14;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
  $15 = $14 << 16 >> 16;
  $16 = HEAP16[$2>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = (($17) + 31)|0;
  $19 = ($15|0)>($18|0);
  $20 = HEAP16[$1>>1]|0;
  $21 = $20 << 16 >> 16;
  if ($19) {
   $22 = (($21) + 1)|0;
   $23 = $22&65535;
   HEAP16[$0>>1] = $23;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  } else {
   $24 = HEAP16[$1>>1]|0;
   $25 = $24 << 16 >> 16;
   $26 = HEAP16[$2>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = (($25) - ($27))|0;
   $29 = (32662 + ($28)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = (($21) + ($31))|0;
   $33 = $32&65535;
   HEAP16[$0>>1] = $33;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
 } else {
  $34 = HEAP16[$2>>1]|0;
  $35 = $34 << 16 >> 16;
  $36 = HEAP16[$1>>1]|0;
  $37 = $36 << 16 >> 16;
  $38 = (($37) + 49)|0;
  $39 = ($35|0)>($38|0);
  $40 = HEAP16[$2>>1]|0;
  if ($39) {
   HEAP16[$0>>1] = $40;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
  $41 = $40 << 16 >> 16;
  $42 = HEAP16[$1>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = (($43) + 31)|0;
  $45 = ($41|0)>($44|0);
  $46 = HEAP16[$2>>1]|0;
  $47 = $46 << 16 >> 16;
  if ($45) {
   $48 = (($47) + 1)|0;
   $49 = $48&65535;
   HEAP16[$0>>1] = $49;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  } else {
   $50 = HEAP16[$2>>1]|0;
   $51 = $50 << 16 >> 16;
   $52 = HEAP16[$1>>1]|0;
   $53 = $52 << 16 >> 16;
   $54 = (($51) - ($53))|0;
   $55 = (32662 + ($54)|0);
   $56 = HEAP8[$55>>0]|0;
   $57 = $56&255;
   $58 = (($47) + ($57))|0;
   $59 = $58&65535;
   HEAP16[$0>>1] = $59;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
 }
 return (0)|0;
}
function _whereUsablePartialIndex($iTab,$pWC,$pWhere) {
 $iTab = $iTab|0;
 $pWC = $pWC|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pExpr = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $pTerm = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $pWC;
 HEAP32[$3>>2] = $pWhere;
 while(1) {
  $4 = HEAP32[$3>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = ($6|0)==(72);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$3>>2]|0;
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_whereUsablePartialIndex($8,$9,$12)|0);
  $14 = ($13|0)!=(0);
  if (!($14)) {
   label = 4;
   break;
  }
  $15 = HEAP32[$3>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$3>>2] = $17;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 HEAP32[$i>>2] = 0;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pTerm>>2] = $20;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($21|0)<($24|0);
  if (!($25)) {
   label = 13;
   break;
  }
  $26 = HEAP32[$pTerm>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pExpr>>2] = $27;
  $28 = HEAP32[$pExpr>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = (_sqlite3ExprImpliesExpr($28,$29,$30)|0);
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = HEAP32[$pExpr>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $35 & 1;
   $37 = ($36|0)!=(0);
   if (!($37)) {
    label = 11;
    break;
   }
   $38 = HEAP32[$pExpr>>2]|0;
   $39 = ((($38)) + 36|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40 << 16 >> 16;
   $42 = HEAP32[$1>>2]|0;
   $43 = ($41|0)==($42|0);
   if ($43) {
    label = 11;
    break;
   }
  }
  $44 = HEAP32[$i>>2]|0;
  $45 = (($44) + 1)|0;
  HEAP32[$i>>2] = $45;
  $46 = HEAP32[$pTerm>>2]|0;
  $47 = ((($46)) + 48|0);
  HEAP32[$pTerm>>2] = $47;
 }
 if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 return (0)|0;
}
function _indexMightHelpWithOrderBy($pBuilder,$pIndex,$iCursor) {
 $pBuilder = $pBuilder|0;
 $pIndex = $pIndex|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aColExpr = 0, $ii = 0, $jj = 0, $pExpr = 0, $pOB = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pOB = sp + 16|0;
 $aColExpr = sp + 12|0;
 $ii = sp + 8|0;
 $jj = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pIndex;
 HEAP32[$3>>2] = $iCursor;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 55|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pOB>>2] = $14;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 HEAP32[$ii>>2] = 0;
 L9: while(1) {
  $16 = HEAP32[$ii>>2]|0;
  $17 = HEAP32[$pOB>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   label = 24;
   break;
  }
  $20 = HEAP32[$ii>>2]|0;
  $21 = HEAP32[$pOB>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3ExprSkipCollate($25)|0);
  HEAP32[$pExpr>>2] = $26;
  $27 = HEAP32[$pExpr>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(152);
  L12: do {
   if ($30) {
    $31 = HEAP32[$pExpr>>2]|0;
    $32 = ((($31)) + 28|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = HEAP32[$3>>2]|0;
    $35 = ($33|0)==($34|0);
    if ($35) {
     $36 = HEAP32[$pExpr>>2]|0;
     $37 = ((($36)) + 32|0);
     $38 = HEAP16[$37>>1]|0;
     $39 = $38 << 16 >> 16;
     $40 = ($39|0)<(0);
     if ($40) {
      label = 10;
      break L9;
     }
     HEAP32[$jj>>2] = 0;
     while(1) {
      $41 = HEAP32[$jj>>2]|0;
      $42 = HEAP32[$2>>2]|0;
      $43 = ((($42)) + 50|0);
      $44 = HEAP16[$43>>1]|0;
      $45 = $44&65535;
      $46 = ($41|0)<($45|0);
      if (!($46)) {
       break L12;
      }
      $47 = HEAP32[$pExpr>>2]|0;
      $48 = ((($47)) + 32|0);
      $49 = HEAP16[$48>>1]|0;
      $50 = $49 << 16 >> 16;
      $51 = HEAP32[$jj>>2]|0;
      $52 = HEAP32[$2>>2]|0;
      $53 = ((($52)) + 4|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = (($54) + ($51<<1)|0);
      $56 = HEAP16[$55>>1]|0;
      $57 = $56 << 16 >> 16;
      $58 = ($50|0)==($57|0);
      if ($58) {
       label = 14;
       break L9;
      }
      $59 = HEAP32[$jj>>2]|0;
      $60 = (($59) + 1)|0;
      HEAP32[$jj>>2] = $60;
     }
    } else {
     label = 16;
    }
   } else {
    label = 16;
   }
  } while(0);
  L20: do {
   if ((label|0) == 16) {
    label = 0;
    $61 = HEAP32[$2>>2]|0;
    $62 = ((($61)) + 40|0);
    $63 = HEAP32[$62>>2]|0;
    HEAP32[$aColExpr>>2] = $63;
    $64 = ($63|0)!=(0|0);
    if ($64) {
     HEAP32[$jj>>2] = 0;
     while(1) {
      $65 = HEAP32[$jj>>2]|0;
      $66 = HEAP32[$2>>2]|0;
      $67 = ((($66)) + 50|0);
      $68 = HEAP16[$67>>1]|0;
      $69 = $68&65535;
      $70 = ($65|0)<($69|0);
      if (!($70)) {
       break L20;
      }
      $71 = HEAP32[$jj>>2]|0;
      $72 = HEAP32[$2>>2]|0;
      $73 = ((($72)) + 4|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + ($71<<1)|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76 << 16 >> 16;
      $78 = ($77|0)!=(-2);
      if (!($78)) {
       $79 = HEAP32[$pExpr>>2]|0;
       $80 = HEAP32[$jj>>2]|0;
       $81 = HEAP32[$aColExpr>>2]|0;
       $82 = ((($81)) + 4|0);
       $83 = HEAP32[$82>>2]|0;
       $84 = (($83) + (($80*20)|0)|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = HEAP32[$3>>2]|0;
       $87 = (_sqlite3ExprCompare($79,$85,$86)|0);
       $88 = ($87|0)==(0);
       if ($88) {
        label = 21;
        break L9;
       }
      }
      $89 = HEAP32[$jj>>2]|0;
      $90 = (($89) + 1)|0;
      HEAP32[$jj>>2] = $90;
     }
    }
   }
  } while(0);
  $91 = HEAP32[$ii>>2]|0;
  $92 = (($91) + 1)|0;
  HEAP32[$ii>>2] = $92;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 else if ((label|0) == 21) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 else if ((label|0) == 24) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 return (0)|0;
}
function _whereLoopOutputAdjust($pWC,$pLoop,$nRow) {
 $pWC = $pWC|0;
 $pLoop = $pLoop|0;
 $nRow = $nRow|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $iReduce = 0, $j = 0, $k = 0, $notAllowed = 0, $or$cond = 0, $or$cond3 = 0, $pRight = 0, $pTerm = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 42|0;
 $pTerm = sp + 28|0;
 $pX = sp + 24|0;
 $notAllowed = sp;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $k = sp + 12|0;
 $iReduce = sp + 40|0;
 $pRight = sp + 8|0;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pLoop;
 HEAP16[$2>>1] = $nRow;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = $6 | $14;
 $19 = $9 | $17;
 $20 = $18 ^ -1;
 $21 = $19 ^ -1;
 $22 = $notAllowed;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 HEAP16[$iReduce>>1] = 0;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$i>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pTerm>>2] = $31;
 while(1) {
  $32 = HEAP32[$i>>2]|0;
  $33 = ($32|0)>(0);
  if (!($33)) {
   break;
  }
  $34 = HEAP32[$pTerm>>2]|0;
  $35 = ((($34)) + 20|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 & 2;
  $39 = ($38|0)!=(0);
  if ($39) {
   break;
  }
  $40 = HEAP32[$pTerm>>2]|0;
  $41 = ((($40)) + 40|0);
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = ((($48)) + 8|0);
  $50 = $49;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = $44 & $52;
  $57 = $47 & $55;
  $58 = ($56|0)==(0);
  $59 = ($57|0)==(0);
  $60 = $58 & $59;
  do {
   if (!($60)) {
    $61 = HEAP32[$pTerm>>2]|0;
    $62 = ((($61)) + 40|0);
    $63 = $62;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (($63) + 4)|0;
    $67 = $66;
    $68 = HEAP32[$67>>2]|0;
    $69 = $notAllowed;
    $70 = $69;
    $71 = HEAP32[$70>>2]|0;
    $72 = (($69) + 4)|0;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = $65 & $71;
    $76 = $68 & $74;
    $77 = ($75|0)!=(0);
    $78 = ($76|0)!=(0);
    $79 = $77 | $78;
    if (!($79)) {
     $80 = HEAP32[$1>>2]|0;
     $81 = ((($80)) + 40|0);
     $82 = HEAP16[$81>>1]|0;
     $83 = $82&65535;
     $84 = (($83) - 1)|0;
     HEAP32[$j>>2] = $84;
     while(1) {
      $85 = HEAP32[$j>>2]|0;
      $86 = ($85|0)>=(0);
      if (!($86)) {
       break;
      }
      $87 = HEAP32[$j>>2]|0;
      $88 = HEAP32[$1>>2]|0;
      $89 = ((($88)) + 48|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (($90) + ($87<<2)|0);
      $92 = HEAP32[$91>>2]|0;
      HEAP32[$pX>>2] = $92;
      $93 = HEAP32[$pX>>2]|0;
      $94 = ($93|0)==(0|0);
      if (!($94)) {
       $95 = HEAP32[$pX>>2]|0;
       $96 = HEAP32[$pTerm>>2]|0;
       $97 = ($95|0)==($96|0);
       if ($97) {
        break;
       }
       $98 = HEAP32[$pX>>2]|0;
       $99 = ((($98)) + 4|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = ($100|0)>=(0);
       if ($101) {
        $102 = HEAP32[$pX>>2]|0;
        $103 = ((($102)) + 4|0);
        $104 = HEAP32[$103>>2]|0;
        $105 = HEAP32[$0>>2]|0;
        $106 = ((($105)) + 20|0);
        $107 = HEAP32[$106>>2]|0;
        $108 = (($107) + (($104*48)|0)|0);
        $109 = HEAP32[$pTerm>>2]|0;
        $110 = ($108|0)==($109|0);
        if ($110) {
         break;
        }
       }
      }
      $111 = HEAP32[$j>>2]|0;
      $112 = (($111) + -1)|0;
      HEAP32[$j>>2] = $112;
     }
     $113 = HEAP32[$j>>2]|0;
     $114 = ($113|0)<(0);
     if ($114) {
      $115 = HEAP32[$pTerm>>2]|0;
      $116 = ((($115)) + 16|0);
      $117 = HEAP16[$116>>1]|0;
      $118 = $117 << 16 >> 16;
      $119 = ($118|0)<=(0);
      if ($119) {
       $120 = HEAP32[$pTerm>>2]|0;
       $121 = ((($120)) + 16|0);
       $122 = HEAP16[$121>>1]|0;
       $123 = $122 << 16 >> 16;
       $124 = HEAP32[$1>>2]|0;
       $125 = ((($124)) + 22|0);
       $126 = HEAP16[$125>>1]|0;
       $127 = $126 << 16 >> 16;
       $128 = (($127) + ($123))|0;
       $129 = $128&65535;
       HEAP16[$125>>1] = $129;
       break;
      }
      $130 = HEAP32[$1>>2]|0;
      $131 = ((($130)) + 22|0);
      $132 = HEAP16[$131>>1]|0;
      $133 = (($132) + -1)<<16>>16;
      HEAP16[$131>>1] = $133;
      $134 = HEAP32[$pTerm>>2]|0;
      $135 = ((($134)) + 18|0);
      $136 = HEAP16[$135>>1]|0;
      $137 = $136&65535;
      $138 = $137 & 130;
      $139 = ($138|0)!=(0);
      if ($139) {
       $140 = HEAP32[$pTerm>>2]|0;
       $141 = HEAP32[$140>>2]|0;
       $142 = ((($141)) + 16|0);
       $143 = HEAP32[$142>>2]|0;
       HEAP32[$pRight>>2] = $143;
       $144 = HEAP32[$pRight>>2]|0;
       $145 = (_sqlite3ExprIsInteger($144,$k)|0);
       $146 = ($145|0)!=(0);
       $147 = HEAP32[$k>>2]|0;
       $148 = ($147|0)>=(-1);
       $or$cond = $146 & $148;
       $149 = HEAP32[$k>>2]|0;
       $150 = ($149|0)<=(1);
       $or$cond3 = $or$cond & $150;
       if ($or$cond3) {
        HEAP32[$k>>2] = 10;
       } else {
        HEAP32[$k>>2] = 20;
       }
       $151 = HEAP16[$iReduce>>1]|0;
       $152 = $151 << 16 >> 16;
       $153 = HEAP32[$k>>2]|0;
       $154 = ($152|0)<($153|0);
       if ($154) {
        $155 = HEAP32[$k>>2]|0;
        $156 = $155&65535;
        HEAP16[$iReduce>>1] = $156;
       }
      }
     }
    }
   }
  } while(0);
  $157 = HEAP32[$i>>2]|0;
  $158 = (($157) + -1)|0;
  HEAP32[$i>>2] = $158;
  $159 = HEAP32[$pTerm>>2]|0;
  $160 = ((($159)) + 48|0);
  HEAP32[$pTerm>>2] = $160;
 }
 $161 = HEAP32[$1>>2]|0;
 $162 = ((($161)) + 22|0);
 $163 = HEAP16[$162>>1]|0;
 $164 = $163 << 16 >> 16;
 $165 = HEAP16[$2>>1]|0;
 $166 = $165 << 16 >> 16;
 $167 = HEAP16[$iReduce>>1]|0;
 $168 = $167 << 16 >> 16;
 $169 = (($166) - ($168))|0;
 $170 = ($164|0)>($169|0);
 if (!($170)) {
  STACKTOP = sp;return;
 }
 $171 = HEAP16[$2>>1]|0;
 $172 = $171 << 16 >> 16;
 $173 = HEAP16[$iReduce>>1]|0;
 $174 = $173 << 16 >> 16;
 $175 = (($172) - ($174))|0;
 $176 = $175&65535;
 $177 = HEAP32[$1>>2]|0;
 $178 = ((($177)) + 22|0);
 HEAP16[$178>>1] = $176;
 STACKTOP = sp;return;
}
function _whereLoopAddBtreeIndex($pBuilder,$pSrc,$pProbe,$nInMul) {
 $pBuilder = $pBuilder|0;
 $pSrc = $pSrc|0;
 $pProbe = $pProbe|0;
 $nInMul = $nInMul|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $eOp = 0, $iCol = 0, $nEq = 0, $nIn = 0, $nIter = 0, $nOutUnadjusted = 0, $opMask = 0, $pBtm = 0;
 var $pExpr = 0, $pNew = 0, $pParse = 0, $pTerm = 0, $pTop = 0, $pWInfo = 0, $rCostIdx = 0, $rLogSize = 0, $rSize = 0, $rc = 0, $saved_nEq = 0, $saved_nLTerm = 0, $saved_nOut = 0, $saved_nSkip = 0, $saved_prereq = 0, $saved_wsFlags = 0, $scan = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $3 = sp + 160|0;
 $4 = sp + 196|0;
 $pWInfo = sp + 156|0;
 $pParse = sp + 152|0;
 $db = sp + 148|0;
 $pNew = sp + 144|0;
 $pTerm = sp + 140|0;
 $opMask = sp + 136|0;
 $scan = sp + 40|0;
 $saved_prereq = sp;
 $saved_nLTerm = sp + 194|0;
 $saved_nEq = sp + 192|0;
 $saved_nSkip = sp + 190|0;
 $saved_wsFlags = sp + 36|0;
 $saved_nOut = sp + 188|0;
 $rc = sp + 32|0;
 $rSize = sp + 186|0;
 $rLogSize = sp + 184|0;
 $pTop = sp + 28|0;
 $pBtm = sp + 24|0;
 $eOp = sp + 182|0;
 $rCostIdx = sp + 180|0;
 $nOutUnadjusted = sp + 178|0;
 $nIn = sp + 20|0;
 $pExpr = sp + 16|0;
 $iCol = sp + 12|0;
 $nEq = sp + 8|0;
 $nIter = sp + 176|0;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $pProbe;
 HEAP16[$4>>1] = $nInMul;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pWInfo>>2] = $6;
 $7 = HEAP32[$pWInfo>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pParse>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pTop>>2] = 0;
 HEAP32[$pBtm>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pNew>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = ((($14)) + 65|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 7;
  $670 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($670|0);
 }
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 36|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 & 32;
 $22 = ($21|0)!=(0);
 do {
  if ($22) {
   HEAP32[$opMask>>2] = 24;
  } else {
   $23 = HEAP32[$2>>2]|0;
   $24 = ((($23)) + 36|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = $26 & 8;
   $28 = ($27|0)!=(0);
   if ($28) {
    HEAP32[$opMask>>2] = 63;
    break;
   } else {
    HEAP32[$opMask>>2] = 447;
    break;
   }
  }
 } while(0);
 $29 = HEAP32[$3>>2]|0;
 $30 = ((($29)) + 55|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = ($31&255) >>> 2;
 $33 = $32 & 1;
 $34 = $33&255;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = HEAP32[$opMask>>2]|0;
  $37 = $36 & -61;
  HEAP32[$opMask>>2] = $37;
 }
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 24|0);
 $40 = HEAP16[$39>>1]|0;
 HEAP16[$saved_nEq>>1] = $40;
 $41 = HEAP32[$pNew>>2]|0;
 $42 = ((($41)) + 42|0);
 $43 = HEAP16[$42>>1]|0;
 HEAP16[$saved_nSkip>>1] = $43;
 $44 = HEAP32[$pNew>>2]|0;
 $45 = ((($44)) + 40|0);
 $46 = HEAP16[$45>>1]|0;
 HEAP16[$saved_nLTerm>>1] = $46;
 $47 = HEAP32[$pNew>>2]|0;
 $48 = ((($47)) + 36|0);
 $49 = HEAP32[$48>>2]|0;
 HEAP32[$saved_wsFlags>>2] = $49;
 $50 = HEAP32[$pNew>>2]|0;
 $51 = $50;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = $saved_prereq;
 $58 = $57;
 HEAP32[$58>>2] = $53;
 $59 = (($57) + 4)|0;
 $60 = $59;
 HEAP32[$60>>2] = $56;
 $61 = HEAP32[$pNew>>2]|0;
 $62 = ((($61)) + 22|0);
 $63 = HEAP16[$62>>1]|0;
 HEAP16[$saved_nOut>>1] = $63;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 4|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$2>>2]|0;
 $68 = ((($67)) + 44|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP16[$saved_nEq>>1]|0;
 $71 = $70&65535;
 $72 = HEAP32[$opMask>>2]|0;
 $73 = HEAP32[$3>>2]|0;
 $74 = (_whereScanInit($scan,$66,$69,$71,$72,$73)|0);
 HEAP32[$pTerm>>2] = $74;
 $75 = HEAP32[$pNew>>2]|0;
 $76 = ((($75)) + 18|0);
 HEAP16[$76>>1] = 0;
 $77 = HEAP32[$3>>2]|0;
 $78 = ((($77)) + 8|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = HEAP16[$79>>1]|0;
 HEAP16[$rSize>>1] = $80;
 $81 = HEAP16[$rSize>>1]|0;
 $82 = (_estLog($81)|0);
 HEAP16[$rLogSize>>1] = $82;
 L15: while(1) {
  $83 = HEAP32[$rc>>2]|0;
  $84 = ($83|0)==(0);
  $85 = HEAP32[$pTerm>>2]|0;
  $86 = ($85|0)!=(0|0);
  $87 = $84 ? $86 : 0;
  if (!($87)) {
   break;
  }
  $88 = HEAP32[$pTerm>>2]|0;
  $89 = ((($88)) + 18|0);
  $90 = HEAP16[$89>>1]|0;
  HEAP16[$eOp>>1] = $90;
  HEAP32[$nIn>>2] = 0;
  $91 = HEAP16[$eOp>>1]|0;
  $92 = $91&65535;
  $93 = ($92|0)==(256);
  if ($93) {
   label = 14;
  } else {
   $94 = HEAP32[$pTerm>>2]|0;
   $95 = ((($94)) + 20|0);
   $96 = HEAP16[$95>>1]|0;
   $97 = $96&65535;
   $98 = $97 & 0;
   $99 = ($98|0)!=(0);
   if ($99) {
    label = 14;
   } else {
    label = 15;
   }
  }
  if ((label|0) == 14) {
   label = 0;
   $100 = HEAP32[$3>>2]|0;
   $101 = HEAP16[$saved_nEq>>1]|0;
   $102 = $101&65535;
   $103 = (_indexColumnNotNull($100,$102)|0);
   $104 = ($103|0)!=(0);
   if (!($104)) {
    label = 15;
   }
  }
  do {
   if ((label|0) == 15) {
    label = 0;
    $105 = HEAP32[$pTerm>>2]|0;
    $106 = ((($105)) + 32|0);
    $107 = $106;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$pNew>>2]|0;
    $114 = ((($113)) + 8|0);
    $115 = $114;
    $116 = $115;
    $117 = HEAP32[$116>>2]|0;
    $118 = (($115) + 4)|0;
    $119 = $118;
    $120 = HEAP32[$119>>2]|0;
    $121 = $109 & $117;
    $122 = $112 & $120;
    $123 = ($121|0)!=(0);
    $124 = ($122|0)!=(0);
    $125 = $123 | $124;
    if (!($125)) {
     $126 = HEAP32[$pTerm>>2]|0;
     $127 = ((($126)) + 20|0);
     $128 = HEAP16[$127>>1]|0;
     $129 = $128&65535;
     $130 = $129 & 256;
     $131 = ($130|0)!=(0);
     if ($131) {
      $132 = HEAP32[$pTerm>>2]|0;
      $133 = ((($132)) + 18|0);
      $134 = HEAP16[$133>>1]|0;
      $135 = $134&65535;
      $136 = ($135|0)==(16);
      if ($136) {
       break;
      }
     }
     $137 = HEAP32[$saved_wsFlags>>2]|0;
     $138 = HEAP32[$pNew>>2]|0;
     $139 = ((($138)) + 36|0);
     HEAP32[$139>>2] = $137;
     $140 = HEAP16[$saved_nEq>>1]|0;
     $141 = HEAP32[$pNew>>2]|0;
     $142 = ((($141)) + 24|0);
     HEAP16[$142>>1] = $140;
     $143 = HEAP16[$saved_nLTerm>>1]|0;
     $144 = HEAP32[$pNew>>2]|0;
     $145 = ((($144)) + 40|0);
     HEAP16[$145>>1] = $143;
     $146 = HEAP32[$db>>2]|0;
     $147 = HEAP32[$pNew>>2]|0;
     $148 = HEAP32[$pNew>>2]|0;
     $149 = ((($148)) + 40|0);
     $150 = HEAP16[$149>>1]|0;
     $151 = $150&65535;
     $152 = (($151) + 1)|0;
     $153 = (_whereLoopResize($146,$147,$152)|0);
     $154 = ($153|0)!=(0);
     if ($154) {
      break L15;
     }
     $155 = HEAP32[$pTerm>>2]|0;
     $156 = HEAP32[$pNew>>2]|0;
     $157 = ((($156)) + 40|0);
     $158 = HEAP16[$157>>1]|0;
     $159 = (($158) + 1)<<16>>16;
     HEAP16[$157>>1] = $159;
     $160 = $158&65535;
     $161 = HEAP32[$pNew>>2]|0;
     $162 = ((($161)) + 48|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = (($163) + ($160<<2)|0);
     HEAP32[$164>>2] = $155;
     $165 = $saved_prereq;
     $166 = $165;
     $167 = HEAP32[$166>>2]|0;
     $168 = (($165) + 4)|0;
     $169 = $168;
     $170 = HEAP32[$169>>2]|0;
     $171 = HEAP32[$pTerm>>2]|0;
     $172 = ((($171)) + 32|0);
     $173 = $172;
     $174 = $173;
     $175 = HEAP32[$174>>2]|0;
     $176 = (($173) + 4)|0;
     $177 = $176;
     $178 = HEAP32[$177>>2]|0;
     $179 = $167 | $175;
     $180 = $170 | $178;
     $181 = HEAP32[$pNew>>2]|0;
     $182 = ((($181)) + 8|0);
     $183 = $182;
     $184 = $183;
     $185 = HEAP32[$184>>2]|0;
     $186 = (($183) + 4)|0;
     $187 = $186;
     $188 = HEAP32[$187>>2]|0;
     $189 = $185 ^ -1;
     $190 = $188 ^ -1;
     $191 = $179 & $189;
     $192 = $180 & $190;
     $193 = HEAP32[$pNew>>2]|0;
     $194 = $193;
     $195 = $194;
     HEAP32[$195>>2] = $191;
     $196 = (($194) + 4)|0;
     $197 = $196;
     HEAP32[$197>>2] = $192;
     $198 = HEAP16[$eOp>>1]|0;
     $199 = $198&65535;
     $200 = $199 & 1;
     $201 = ($200|0)!=(0);
     do {
      if ($201) {
       $202 = HEAP32[$pTerm>>2]|0;
       $203 = HEAP32[$202>>2]|0;
       HEAP32[$pExpr>>2] = $203;
       $204 = HEAP32[$pNew>>2]|0;
       $205 = ((($204)) + 36|0);
       $206 = HEAP32[$205>>2]|0;
       $207 = $206 | 4;
       HEAP32[$205>>2] = $207;
       $208 = HEAP32[$pExpr>>2]|0;
       $209 = ((($208)) + 4|0);
       $210 = HEAP32[$209>>2]|0;
       $211 = $210 & 2048;
       $212 = ($211|0)!=(0);
       if ($212) {
        HEAP32[$nIn>>2] = 46;
        break;
       }
       $213 = HEAP32[$pExpr>>2]|0;
       $214 = ((($213)) + 20|0);
       $215 = HEAP32[$214>>2]|0;
       $216 = ($215|0)!=(0|0);
       if ($216) {
        $217 = HEAP32[$pExpr>>2]|0;
        $218 = ((($217)) + 20|0);
        $219 = HEAP32[$218>>2]|0;
        $220 = HEAP32[$219>>2]|0;
        $221 = ($220|0)!=(0);
        if ($221) {
         $222 = HEAP32[$pExpr>>2]|0;
         $223 = ((($222)) + 20|0);
         $224 = HEAP32[$223>>2]|0;
         $225 = HEAP32[$224>>2]|0;
         $226 = ($225|0)<(0);
         $227 = $226 << 31 >> 31;
         $228 = (_sqlite3LogEst($225,$227)|0);
         $229 = $228 << 16 >> 16;
         HEAP32[$nIn>>2] = $229;
        }
       }
      } else {
       $230 = HEAP16[$eOp>>1]|0;
       $231 = $230&65535;
       $232 = $231 & 130;
       $233 = ($232|0)!=(0);
       if ($233) {
        $234 = HEAP16[$saved_nEq>>1]|0;
        $235 = $234&65535;
        $236 = HEAP32[$3>>2]|0;
        $237 = ((($236)) + 4|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = (($238) + ($235<<1)|0);
        $240 = HEAP16[$239>>1]|0;
        $241 = $240 << 16 >> 16;
        HEAP32[$iCol>>2] = $241;
        $242 = HEAP32[$pNew>>2]|0;
        $243 = ((($242)) + 36|0);
        $244 = HEAP32[$243>>2]|0;
        $245 = $244 | 1;
        HEAP32[$243>>2] = $245;
        $246 = HEAP32[$iCol>>2]|0;
        $247 = ($246|0)==(-1);
        if (!($247)) {
         $248 = HEAP32[$iCol>>2]|0;
         $249 = ($248|0)>(0);
         if (!($249)) {
          break;
         }
         $250 = HEAP16[$4>>1]|0;
         $251 = $250 << 16 >> 16;
         $252 = ($251|0)==(0);
         if (!($252)) {
          break;
         }
         $253 = HEAP16[$saved_nEq>>1]|0;
         $254 = $253&65535;
         $255 = HEAP32[$3>>2]|0;
         $256 = ((($255)) + 50|0);
         $257 = HEAP16[$256>>1]|0;
         $258 = $257&65535;
         $259 = (($258) - 1)|0;
         $260 = ($254|0)==($259|0);
         if (!($260)) {
          break;
         }
        }
        $261 = HEAP32[$iCol>>2]|0;
        $262 = ($261|0)>=(0);
        if ($262) {
         $263 = HEAP32[$3>>2]|0;
         $264 = ((($263)) + 55|0);
         $265 = HEAP8[$264>>0]|0;
         $266 = ($265&255) >>> 3;
         $267 = $266 & 1;
         $268 = $267&255;
         $269 = ($268|0)==(0);
         if ($269) {
          $270 = HEAP32[$pNew>>2]|0;
          $271 = ((($270)) + 36|0);
          $272 = HEAP32[$271>>2]|0;
          $273 = $272 | 65536;
          HEAP32[$271>>2] = $273;
          break;
         }
        }
        $274 = HEAP32[$pNew>>2]|0;
        $275 = ((($274)) + 36|0);
        $276 = HEAP32[$275>>2]|0;
        $277 = $276 | 4096;
        HEAP32[$275>>2] = $277;
        break;
       }
       $278 = HEAP16[$eOp>>1]|0;
       $279 = $278&65535;
       $280 = $279 & 256;
       $281 = ($280|0)!=(0);
       if ($281) {
        $282 = HEAP32[$pNew>>2]|0;
        $283 = ((($282)) + 36|0);
        $284 = HEAP32[$283>>2]|0;
        $285 = $284 | 8;
        HEAP32[$283>>2] = $285;
        break;
       }
       $286 = HEAP16[$eOp>>1]|0;
       $287 = $286&65535;
       $288 = $287 & 36;
       $289 = ($288|0)!=(0);
       $290 = HEAP32[$pNew>>2]|0;
       $291 = ((($290)) + 36|0);
       $292 = HEAP32[$291>>2]|0;
       if ($289) {
        $293 = $292 | 34;
        HEAP32[$291>>2] = $293;
        $294 = HEAP32[$pTerm>>2]|0;
        HEAP32[$pBtm>>2] = $294;
        HEAP32[$pTop>>2] = 0;
        $295 = HEAP32[$pTerm>>2]|0;
        $296 = ((($295)) + 20|0);
        $297 = HEAP16[$296>>1]|0;
        $298 = $297&65535;
        $299 = $298 & 256;
        $300 = ($299|0)!=(0);
        if (!($300)) {
         break;
        }
        $301 = HEAP32[$pTerm>>2]|0;
        $302 = ((($301)) + 48|0);
        HEAP32[$pTop>>2] = $302;
        $303 = HEAP32[$db>>2]|0;
        $304 = HEAP32[$pNew>>2]|0;
        $305 = HEAP32[$pNew>>2]|0;
        $306 = ((($305)) + 40|0);
        $307 = HEAP16[$306>>1]|0;
        $308 = $307&65535;
        $309 = (($308) + 1)|0;
        $310 = (_whereLoopResize($303,$304,$309)|0);
        $311 = ($310|0)!=(0);
        if ($311) {
         break L15;
        }
        $312 = HEAP32[$pTop>>2]|0;
        $313 = HEAP32[$pNew>>2]|0;
        $314 = ((($313)) + 40|0);
        $315 = HEAP16[$314>>1]|0;
        $316 = (($315) + 1)<<16>>16;
        HEAP16[$314>>1] = $316;
        $317 = $315&65535;
        $318 = HEAP32[$pNew>>2]|0;
        $319 = ((($318)) + 48|0);
        $320 = HEAP32[$319>>2]|0;
        $321 = (($320) + ($317<<2)|0);
        HEAP32[$321>>2] = $312;
        $322 = HEAP32[$pNew>>2]|0;
        $323 = ((($322)) + 36|0);
        $324 = HEAP32[$323>>2]|0;
        $325 = $324 | 16;
        HEAP32[$323>>2] = $325;
        break;
       } else {
        $326 = $292 | 18;
        HEAP32[$291>>2] = $326;
        $327 = HEAP32[$pTerm>>2]|0;
        HEAP32[$pTop>>2] = $327;
        $328 = HEAP32[$pNew>>2]|0;
        $329 = ((($328)) + 36|0);
        $330 = HEAP32[$329>>2]|0;
        $331 = $330 & 32;
        $332 = ($331|0)!=(0);
        if ($332) {
         $333 = HEAP32[$pNew>>2]|0;
         $334 = ((($333)) + 40|0);
         $335 = HEAP16[$334>>1]|0;
         $336 = $335&65535;
         $337 = (($336) - 2)|0;
         $338 = HEAP32[$pNew>>2]|0;
         $339 = ((($338)) + 48|0);
         $340 = HEAP32[$339>>2]|0;
         $341 = (($340) + ($337<<2)|0);
         $342 = HEAP32[$341>>2]|0;
         $343 = $342;
        } else {
         $343 = 0;
        }
        HEAP32[$pBtm>>2] = $343;
        break;
       }
      }
     } while(0);
     $344 = HEAP32[$pNew>>2]|0;
     $345 = ((($344)) + 36|0);
     $346 = HEAP32[$345>>2]|0;
     $347 = $346 & 2;
     $348 = ($347|0)!=(0);
     do {
      if ($348) {
       $349 = HEAP32[$pParse>>2]|0;
       $350 = HEAP32[$1>>2]|0;
       $351 = HEAP32[$pBtm>>2]|0;
       $352 = HEAP32[$pTop>>2]|0;
       $353 = HEAP32[$pNew>>2]|0;
       (_whereRangeScanEst($349,$350,$351,$352,$353)|0);
      } else {
       $354 = HEAP32[$pNew>>2]|0;
       $355 = ((($354)) + 24|0);
       $356 = HEAP16[$355>>1]|0;
       $357 = (($356) + 1)<<16>>16;
       HEAP16[$355>>1] = $357;
       $358 = $357&65535;
       HEAP32[$nEq>>2] = $358;
       $359 = HEAP32[$pTerm>>2]|0;
       $360 = ((($359)) + 16|0);
       $361 = HEAP16[$360>>1]|0;
       $362 = $361 << 16 >> 16;
       $363 = ($362|0)<=(0);
       if ($363) {
        $364 = HEAP16[$saved_nEq>>1]|0;
        $365 = $364&65535;
        $366 = HEAP32[$3>>2]|0;
        $367 = ((($366)) + 4|0);
        $368 = HEAP32[$367>>2]|0;
        $369 = (($368) + ($365<<1)|0);
        $370 = HEAP16[$369>>1]|0;
        $371 = $370 << 16 >> 16;
        $372 = ($371|0)>=(0);
        if ($372) {
         $373 = HEAP32[$pTerm>>2]|0;
         $374 = ((($373)) + 16|0);
         $375 = HEAP16[$374>>1]|0;
         $376 = $375 << 16 >> 16;
         $377 = HEAP32[$pNew>>2]|0;
         $378 = ((($377)) + 22|0);
         $379 = HEAP16[$378>>1]|0;
         $380 = $379 << 16 >> 16;
         $381 = (($380) + ($376))|0;
         $382 = $381&65535;
         HEAP16[$378>>1] = $382;
         $383 = HEAP32[$nIn>>2]|0;
         $384 = HEAP32[$pNew>>2]|0;
         $385 = ((($384)) + 22|0);
         $386 = HEAP16[$385>>1]|0;
         $387 = $386 << 16 >> 16;
         $388 = (($387) - ($383))|0;
         $389 = $388&65535;
         HEAP16[$385>>1] = $389;
         break;
        }
       }
       $390 = HEAP32[$nEq>>2]|0;
       $391 = HEAP32[$3>>2]|0;
       $392 = ((($391)) + 8|0);
       $393 = HEAP32[$392>>2]|0;
       $394 = (($393) + ($390<<1)|0);
       $395 = HEAP16[$394>>1]|0;
       $396 = $395 << 16 >> 16;
       $397 = HEAP32[$nEq>>2]|0;
       $398 = (($397) - 1)|0;
       $399 = HEAP32[$3>>2]|0;
       $400 = ((($399)) + 8|0);
       $401 = HEAP32[$400>>2]|0;
       $402 = (($401) + ($398<<1)|0);
       $403 = HEAP16[$402>>1]|0;
       $404 = $403 << 16 >> 16;
       $405 = (($396) - ($404))|0;
       $406 = HEAP32[$pNew>>2]|0;
       $407 = ((($406)) + 22|0);
       $408 = HEAP16[$407>>1]|0;
       $409 = $408 << 16 >> 16;
       $410 = (($409) + ($405))|0;
       $411 = $410&65535;
       HEAP16[$407>>1] = $411;
       $412 = HEAP16[$eOp>>1]|0;
       $413 = $412&65535;
       $414 = $413 & 256;
       $415 = ($414|0)!=(0);
       if ($415) {
        $416 = HEAP32[$pNew>>2]|0;
        $417 = ((($416)) + 22|0);
        $418 = HEAP16[$417>>1]|0;
        $419 = $418 << 16 >> 16;
        $420 = (($419) + 10)|0;
        $421 = $420&65535;
        HEAP16[$417>>1] = $421;
       }
      }
     } while(0);
     $422 = HEAP32[$pNew>>2]|0;
     $423 = ((($422)) + 22|0);
     $424 = HEAP16[$423>>1]|0;
     $425 = $424 << 16 >> 16;
     $426 = (($425) + 1)|0;
     $427 = HEAP32[$3>>2]|0;
     $428 = ((($427)) + 48|0);
     $429 = HEAP16[$428>>1]|0;
     $430 = $429 << 16 >> 16;
     $431 = ($430*15)|0;
     $432 = HEAP32[$2>>2]|0;
     $433 = ((($432)) + 16|0);
     $434 = HEAP32[$433>>2]|0;
     $435 = ((($434)) + 40|0);
     $436 = HEAP16[$435>>1]|0;
     $437 = $436 << 16 >> 16;
     $438 = (($431|0) / ($437|0))&-1;
     $439 = (($426) + ($438))|0;
     $440 = $439&65535;
     HEAP16[$rCostIdx>>1] = $440;
     $441 = HEAP16[$rLogSize>>1]|0;
     $442 = HEAP16[$rCostIdx>>1]|0;
     $443 = (_sqlite3LogEstAdd($441,$442)|0);
     $444 = HEAP32[$pNew>>2]|0;
     $445 = ((($444)) + 20|0);
     HEAP16[$445>>1] = $443;
     $446 = HEAP32[$pNew>>2]|0;
     $447 = ((($446)) + 36|0);
     $448 = HEAP32[$447>>2]|0;
     $449 = $448 & 320;
     $450 = ($449|0)==(0);
     if ($450) {
      $451 = HEAP32[$pNew>>2]|0;
      $452 = ((($451)) + 20|0);
      $453 = HEAP16[$452>>1]|0;
      $454 = HEAP32[$pNew>>2]|0;
      $455 = ((($454)) + 22|0);
      $456 = HEAP16[$455>>1]|0;
      $457 = $456 << 16 >> 16;
      $458 = (($457) + 16)|0;
      $459 = $458&65535;
      $460 = (_sqlite3LogEstAdd($453,$459)|0);
      $461 = HEAP32[$pNew>>2]|0;
      $462 = ((($461)) + 20|0);
      HEAP16[$462>>1] = $460;
     }
     $463 = HEAP32[$pNew>>2]|0;
     $464 = ((($463)) + 22|0);
     $465 = HEAP16[$464>>1]|0;
     HEAP16[$nOutUnadjusted>>1] = $465;
     $466 = HEAP16[$4>>1]|0;
     $467 = $466 << 16 >> 16;
     $468 = HEAP32[$nIn>>2]|0;
     $469 = (($467) + ($468))|0;
     $470 = HEAP32[$pNew>>2]|0;
     $471 = ((($470)) + 20|0);
     $472 = HEAP16[$471>>1]|0;
     $473 = $472 << 16 >> 16;
     $474 = (($473) + ($469))|0;
     $475 = $474&65535;
     HEAP16[$471>>1] = $475;
     $476 = HEAP16[$4>>1]|0;
     $477 = $476 << 16 >> 16;
     $478 = HEAP32[$nIn>>2]|0;
     $479 = (($477) + ($478))|0;
     $480 = HEAP32[$pNew>>2]|0;
     $481 = ((($480)) + 22|0);
     $482 = HEAP16[$481>>1]|0;
     $483 = $482 << 16 >> 16;
     $484 = (($483) + ($479))|0;
     $485 = $484&65535;
     HEAP16[$481>>1] = $485;
     $486 = HEAP32[$1>>2]|0;
     $487 = ((($486)) + 4|0);
     $488 = HEAP32[$487>>2]|0;
     $489 = HEAP32[$pNew>>2]|0;
     $490 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($488,$489,$490);
     $491 = HEAP32[$1>>2]|0;
     $492 = HEAP32[$pNew>>2]|0;
     $493 = (_whereLoopInsert($491,$492)|0);
     HEAP32[$rc>>2] = $493;
     $494 = HEAP32[$pNew>>2]|0;
     $495 = ((($494)) + 36|0);
     $496 = HEAP32[$495>>2]|0;
     $497 = $496 & 2;
     $498 = ($497|0)!=(0);
     if ($498) {
      $499 = HEAP16[$saved_nOut>>1]|0;
      $500 = HEAP32[$pNew>>2]|0;
      $501 = ((($500)) + 22|0);
      HEAP16[$501>>1] = $499;
     } else {
      $502 = HEAP16[$nOutUnadjusted>>1]|0;
      $503 = HEAP32[$pNew>>2]|0;
      $504 = ((($503)) + 22|0);
      HEAP16[$504>>1] = $502;
     }
     $505 = HEAP32[$pNew>>2]|0;
     $506 = ((($505)) + 36|0);
     $507 = HEAP32[$506>>2]|0;
     $508 = $507 & 16;
     $509 = ($508|0)==(0);
     if ($509) {
      $510 = HEAP32[$pNew>>2]|0;
      $511 = ((($510)) + 24|0);
      $512 = HEAP16[$511>>1]|0;
      $513 = $512&65535;
      $514 = HEAP32[$3>>2]|0;
      $515 = ((($514)) + 52|0);
      $516 = HEAP16[$515>>1]|0;
      $517 = $516&65535;
      $518 = ($513|0)<($517|0);
      if ($518) {
       $519 = HEAP32[$1>>2]|0;
       $520 = HEAP32[$2>>2]|0;
       $521 = HEAP32[$3>>2]|0;
       $522 = HEAP16[$4>>1]|0;
       $523 = $522 << 16 >> 16;
       $524 = HEAP32[$nIn>>2]|0;
       $525 = (($523) + ($524))|0;
       $526 = $525&65535;
       (_whereLoopAddBtreeIndex($519,$520,$521,$526)|0);
      }
     }
     $527 = HEAP16[$saved_nOut>>1]|0;
     $528 = HEAP32[$pNew>>2]|0;
     $529 = ((($528)) + 22|0);
     HEAP16[$529>>1] = $527;
    }
   }
  } while(0);
  $530 = (_whereScanNext($scan)|0);
  HEAP32[$pTerm>>2] = $530;
 }
 $531 = $saved_prereq;
 $532 = $531;
 $533 = HEAP32[$532>>2]|0;
 $534 = (($531) + 4)|0;
 $535 = $534;
 $536 = HEAP32[$535>>2]|0;
 $537 = HEAP32[$pNew>>2]|0;
 $538 = $537;
 $539 = $538;
 HEAP32[$539>>2] = $533;
 $540 = (($538) + 4)|0;
 $541 = $540;
 HEAP32[$541>>2] = $536;
 $542 = HEAP16[$saved_nEq>>1]|0;
 $543 = HEAP32[$pNew>>2]|0;
 $544 = ((($543)) + 24|0);
 HEAP16[$544>>1] = $542;
 $545 = HEAP16[$saved_nSkip>>1]|0;
 $546 = HEAP32[$pNew>>2]|0;
 $547 = ((($546)) + 42|0);
 HEAP16[$547>>1] = $545;
 $548 = HEAP32[$saved_wsFlags>>2]|0;
 $549 = HEAP32[$pNew>>2]|0;
 $550 = ((($549)) + 36|0);
 HEAP32[$550>>2] = $548;
 $551 = HEAP16[$saved_nOut>>1]|0;
 $552 = HEAP32[$pNew>>2]|0;
 $553 = ((($552)) + 22|0);
 HEAP16[$553>>1] = $551;
 $554 = HEAP16[$saved_nLTerm>>1]|0;
 $555 = HEAP32[$pNew>>2]|0;
 $556 = ((($555)) + 40|0);
 HEAP16[$556>>1] = $554;
 $557 = HEAP16[$saved_nEq>>1]|0;
 $558 = $557&65535;
 $559 = HEAP16[$saved_nSkip>>1]|0;
 $560 = $559&65535;
 $561 = ($558|0)==($560|0);
 if ($561) {
  $562 = HEAP16[$saved_nEq>>1]|0;
  $563 = $562&65535;
  $564 = (($563) + 1)|0;
  $565 = HEAP32[$3>>2]|0;
  $566 = ((($565)) + 50|0);
  $567 = HEAP16[$566>>1]|0;
  $568 = $567&65535;
  $569 = ($564|0)<($568|0);
  if ($569) {
   $570 = HEAP32[$3>>2]|0;
   $571 = ((($570)) + 55|0);
   $572 = HEAP8[$571>>0]|0;
   $573 = ($572&255) >>> 6;
   $574 = $573 & 1;
   $575 = $574&255;
   $576 = ($575|0)==(0);
   if ($576) {
    $577 = HEAP16[$saved_nEq>>1]|0;
    $578 = $577&65535;
    $579 = (($578) + 1)|0;
    $580 = HEAP32[$3>>2]|0;
    $581 = ((($580)) + 8|0);
    $582 = HEAP32[$581>>2]|0;
    $583 = (($582) + ($579<<1)|0);
    $584 = HEAP16[$583>>1]|0;
    $585 = $584 << 16 >> 16;
    $586 = ($585|0)>=(42);
    if ($586) {
     $587 = HEAP32[$db>>2]|0;
     $588 = HEAP32[$pNew>>2]|0;
     $589 = HEAP32[$pNew>>2]|0;
     $590 = ((($589)) + 40|0);
     $591 = HEAP16[$590>>1]|0;
     $592 = $591&65535;
     $593 = (($592) + 1)|0;
     $594 = (_whereLoopResize($587,$588,$593)|0);
     HEAP32[$rc>>2] = $594;
     $595 = ($594|0)==(0);
     if ($595) {
      $596 = HEAP32[$pNew>>2]|0;
      $597 = ((($596)) + 24|0);
      $598 = HEAP16[$597>>1]|0;
      $599 = (($598) + 1)<<16>>16;
      HEAP16[$597>>1] = $599;
      $600 = HEAP32[$pNew>>2]|0;
      $601 = ((($600)) + 42|0);
      $602 = HEAP16[$601>>1]|0;
      $603 = (($602) + 1)<<16>>16;
      HEAP16[$601>>1] = $603;
      $604 = HEAP32[$pNew>>2]|0;
      $605 = ((($604)) + 40|0);
      $606 = HEAP16[$605>>1]|0;
      $607 = (($606) + 1)<<16>>16;
      HEAP16[$605>>1] = $607;
      $608 = $606&65535;
      $609 = HEAP32[$pNew>>2]|0;
      $610 = ((($609)) + 48|0);
      $611 = HEAP32[$610>>2]|0;
      $612 = (($611) + ($608<<2)|0);
      HEAP32[$612>>2] = 0;
      $613 = HEAP32[$pNew>>2]|0;
      $614 = ((($613)) + 36|0);
      $615 = HEAP32[$614>>2]|0;
      $616 = $615 | 32768;
      HEAP32[$614>>2] = $616;
      $617 = HEAP16[$saved_nEq>>1]|0;
      $618 = $617&65535;
      $619 = HEAP32[$3>>2]|0;
      $620 = ((($619)) + 8|0);
      $621 = HEAP32[$620>>2]|0;
      $622 = (($621) + ($618<<1)|0);
      $623 = HEAP16[$622>>1]|0;
      $624 = $623 << 16 >> 16;
      $625 = HEAP16[$saved_nEq>>1]|0;
      $626 = $625&65535;
      $627 = (($626) + 1)|0;
      $628 = HEAP32[$3>>2]|0;
      $629 = ((($628)) + 8|0);
      $630 = HEAP32[$629>>2]|0;
      $631 = (($630) + ($627<<1)|0);
      $632 = HEAP16[$631>>1]|0;
      $633 = $632 << 16 >> 16;
      $634 = (($624) - ($633))|0;
      $635 = $634&65535;
      HEAP16[$nIter>>1] = $635;
      $636 = HEAP16[$nIter>>1]|0;
      $637 = $636 << 16 >> 16;
      $638 = HEAP32[$pNew>>2]|0;
      $639 = ((($638)) + 22|0);
      $640 = HEAP16[$639>>1]|0;
      $641 = $640 << 16 >> 16;
      $642 = (($641) - ($637))|0;
      $643 = $642&65535;
      HEAP16[$639>>1] = $643;
      $644 = HEAP16[$nIter>>1]|0;
      $645 = $644 << 16 >> 16;
      $646 = (($645) + 5)|0;
      $647 = $646&65535;
      HEAP16[$nIter>>1] = $647;
      $648 = HEAP32[$1>>2]|0;
      $649 = HEAP32[$2>>2]|0;
      $650 = HEAP32[$3>>2]|0;
      $651 = HEAP16[$nIter>>1]|0;
      $652 = $651 << 16 >> 16;
      $653 = HEAP16[$4>>1]|0;
      $654 = $653 << 16 >> 16;
      $655 = (($652) + ($654))|0;
      $656 = $655&65535;
      (_whereLoopAddBtreeIndex($648,$649,$650,$656)|0);
      $657 = HEAP16[$saved_nOut>>1]|0;
      $658 = HEAP32[$pNew>>2]|0;
      $659 = ((($658)) + 22|0);
      HEAP16[$659>>1] = $657;
      $660 = HEAP16[$saved_nEq>>1]|0;
      $661 = HEAP32[$pNew>>2]|0;
      $662 = ((($661)) + 24|0);
      HEAP16[$662>>1] = $660;
      $663 = HEAP16[$saved_nSkip>>1]|0;
      $664 = HEAP32[$pNew>>2]|0;
      $665 = ((($664)) + 42|0);
      HEAP16[$665>>1] = $663;
      $666 = HEAP32[$saved_wsFlags>>2]|0;
      $667 = HEAP32[$pNew>>2]|0;
      $668 = ((($667)) + 36|0);
      HEAP32[$668>>2] = $666;
     }
    }
   }
  }
 }
 $669 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $669;
 $670 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($670|0);
}
function _sqlite3ExprImpliesExpr($pE1,$pE2,$iTab) {
 $pE1 = $pE1|0;
 $pE2 = $pE2|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $pE1;
 HEAP32[$2>>2] = $pE2;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_sqlite3ExprCompare($4,$5,$6)|0);
 $8 = ($7|0)==(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(71);
 do {
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$3>>2]|0;
   $18 = (_sqlite3ExprImpliesExpr($13,$16,$17)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = HEAP32[$2>>2]|0;
    $22 = ((($21)) + 16|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$3>>2]|0;
    $25 = (_sqlite3ExprImpliesExpr($20,$23,$24)|0);
    $26 = ($25|0)!=(0);
    if (!($26)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 } while(0);
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)==(77);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$3>>2]|0;
  $38 = (_sqlite3ExprCompare($33,$36,$37)|0);
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = ($42|0)!=(76);
   if ($43) {
    $44 = HEAP32[$1>>2]|0;
    $45 = HEAP8[$44>>0]|0;
    $46 = $45&255;
    $47 = ($46|0)!=(73);
    if ($47) {
     HEAP32[$0>>2] = 1;
     $48 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($48|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _whereRangeScanEst($pParse,$pBuilder,$pLower,$pUpper,$pLoop) {
 $pParse = $pParse|0;
 $pBuilder = $pBuilder|0;
 $pLower = $pLower|0;
 $pUpper = $pUpper|0;
 $pLoop = $pLoop|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nNew = 0, $nOut = 0, $or$cond = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $nOut = sp;
 $nNew = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pLower;
 HEAP32[$3>>2] = $pUpper;
 HEAP32[$4>>2] = $pLoop;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 22|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7 << 16 >> 16;
 HEAP32[$nOut>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$nOut>>2]|0;
 $11 = $10&65535;
 $12 = (_whereRangeAdjust($9,$11)|0);
 HEAP16[$nNew>>1] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP16[$nNew>>1]|0;
 $15 = (_whereRangeAdjust($13,$14)|0);
 HEAP16[$nNew>>1] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 16|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = ($21|0)>(0);
  $23 = HEAP32[$3>>2]|0;
  $24 = ($23|0)!=(0|0);
  $or$cond = $22 & $24;
  if ($or$cond) {
   $25 = HEAP32[$3>>2]|0;
   $26 = ((($25)) + 16|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27 << 16 >> 16;
   $29 = ($28|0)>(0);
   if ($29) {
    $30 = HEAP16[$nNew>>1]|0;
    $31 = $30 << 16 >> 16;
    $32 = (($31) - 20)|0;
    $33 = $32&65535;
    HEAP16[$nNew>>1] = $33;
   }
  }
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ($34|0)!=(0|0);
 $36 = $35&1;
 $37 = HEAP32[$3>>2]|0;
 $38 = ($37|0)!=(0|0);
 $39 = $38&1;
 $40 = (($36) + ($39))|0;
 $41 = HEAP32[$nOut>>2]|0;
 $42 = (($41) - ($40))|0;
 HEAP32[$nOut>>2] = $42;
 $43 = HEAP16[$nNew>>1]|0;
 $44 = $43 << 16 >> 16;
 $45 = ($44|0)<(10);
 if ($45) {
  HEAP16[$nNew>>1] = 10;
 }
 $46 = HEAP16[$nNew>>1]|0;
 $47 = $46 << 16 >> 16;
 $48 = HEAP32[$nOut>>2]|0;
 $49 = ($47|0)<($48|0);
 if (!($49)) {
  $52 = HEAP32[$nOut>>2]|0;
  $53 = $52&65535;
  $54 = HEAP32[$4>>2]|0;
  $55 = ((($54)) + 22|0);
  HEAP16[$55>>1] = $53;
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $50 = HEAP16[$nNew>>1]|0;
 $51 = $50 << 16 >> 16;
 HEAP32[$nOut>>2] = $51;
 $52 = HEAP32[$nOut>>2]|0;
 $53 = $52&65535;
 $54 = HEAP32[$4>>2]|0;
 $55 = ((($54)) + 22|0);
 HEAP16[$55>>1] = $53;
 $56 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _whereRangeAdjust($pTerm,$nNew) {
 $pTerm = $pTerm|0;
 $nNew = $nNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 6|0;
 $nRet = sp + 4|0;
 HEAP32[$0>>2] = $pTerm;
 HEAP16[$1>>1] = $nNew;
 $2 = HEAP16[$1>>1]|0;
 HEAP16[$nRet>>1] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$0>>2]|0;
   $6 = ((($5)) + 16|0);
   $7 = HEAP16[$6>>1]|0;
   $8 = $7 << 16 >> 16;
   $9 = ($8|0)<=(0);
   $10 = HEAP32[$0>>2]|0;
   if ($9) {
    $11 = ((($10)) + 16|0);
    $12 = HEAP16[$11>>1]|0;
    $13 = $12 << 16 >> 16;
    $14 = HEAP16[$nRet>>1]|0;
    $15 = $14 << 16 >> 16;
    $16 = (($15) + ($13))|0;
    $17 = $16&65535;
    HEAP16[$nRet>>1] = $17;
    break;
   }
   $18 = ((($10)) + 20|0);
   $19 = HEAP16[$18>>1]|0;
   $20 = $19&65535;
   $21 = $20 & 0;
   $22 = ($21|0)==(0);
   if ($22) {
    $23 = HEAP16[$nRet>>1]|0;
    $24 = $23 << 16 >> 16;
    $25 = (($24) - 20)|0;
    $26 = $25&65535;
    HEAP16[$nRet>>1] = $26;
   }
  }
 } while(0);
 $27 = HEAP16[$nRet>>1]|0;
 STACKTOP = sp;return ($27|0);
}
function _whereOrMove($pDest,$pSrc) {
 $pDest = $pDest|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pDest;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 HEAP16[$4>>1] = $3;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11<<4;
 _memcpy(($6|0),($8|0),($12|0))|0;
 STACKTOP = sp;return;
}
function _wherePathSatisfiesOrderBy($pWInfo,$pOrderBy,$pPath,$wctrlFlags,$nLoop,$pLast,$pRevMask) {
 $pWInfo = $pWInfo|0;
 $pOrderBy = $pOrderBy|0;
 $pPath = $pPath|0;
 $wctrlFlags = $wctrlFlags|0;
 $nLoop = $nLoop|0;
 $pLast = $pLast|0;
 $pRevMask = $pRevMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bOnce = 0, $db = 0, $distinctColumns = 0, $i = 0, $iColumn = 0, $iCur = 0, $iLoop = 0, $isMatch = 0;
 var $isOrderDistinct = 0, $j = 0, $m = 0, $mTerm = 0, $nColumn = 0, $nKeyCol = 0, $nOrderBy = 0, $obDone = 0, $obSat = 0, $or$cond = 0, $orderDistinctMask = 0, $p = 0, $pColl = 0, $pIndex = 0, $pLoop = 0, $pOBExpr = 0, $pTerm = 0, $ready = 0, $rev = 0, $revIdx = 0;
 var $revSet = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 141|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 132|0;
 $5 = sp + 130|0;
 $6 = sp + 108|0;
 $7 = sp + 104|0;
 $revSet = sp + 140|0;
 $rev = sp + 139|0;
 $revIdx = sp + 138|0;
 $isOrderDistinct = sp + 137|0;
 $distinctColumns = sp + 136|0;
 $isMatch = sp + 135|0;
 $nKeyCol = sp + 128|0;
 $nColumn = sp + 126|0;
 $nOrderBy = sp + 124|0;
 $iLoop = sp + 100|0;
 $i = sp + 96|0;
 $j = sp + 92|0;
 $iCur = sp + 88|0;
 $iColumn = sp + 84|0;
 $pLoop = sp + 80|0;
 $pTerm = sp + 76|0;
 $pOBExpr = sp + 72|0;
 $pColl = sp + 68|0;
 $pIndex = sp + 64|0;
 $db = sp + 60|0;
 $obSat = sp + 40|0;
 $obDone = sp + 32|0;
 $orderDistinctMask = sp + 24|0;
 $ready = sp + 16|0;
 $z1 = sp + 56|0;
 $z2 = sp + 52|0;
 $bOnce = sp + 134|0;
 $p = sp + 48|0;
 $mTerm = sp + 8|0;
 $m = sp;
 HEAP32[$1>>2] = $pWInfo;
 HEAP32[$2>>2] = $pOrderBy;
 HEAP32[$3>>2] = $pPath;
 HEAP16[$4>>1] = $wctrlFlags;
 HEAP16[$5>>1] = $nLoop;
 HEAP32[$6>>2] = $pLast;
 HEAP32[$7>>2] = $pRevMask;
 HEAP32[$pLoop>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = $obSat;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = HEAP16[$5>>1]|0;
 $16 = $15&65535;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 60|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20&65535;
  $22 = $21 & 128;
  $23 = ($22|0)!=(0);
  if ($23) {
   HEAP8[$0>>0] = 0;
   $691 = HEAP8[$0>>0]|0;
   STACKTOP = sp;return ($691|0);
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = $25&65535;
 HEAP16[$nOrderBy>>1] = $26;
 $27 = HEAP16[$nOrderBy>>1]|0;
 $28 = $27&65535;
 $29 = ($28|0)>(63);
 if ($29) {
  HEAP8[$0>>0] = 0;
  $691 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($691|0);
 }
 HEAP8[$isOrderDistinct>>0] = 1;
 $30 = HEAP16[$nOrderBy>>1]|0;
 $31 = $30&65535;
 $32 = (_bitshift64Shl(1,0,($31|0))|0);
 $33 = tempRet0;
 $34 = (_i64Subtract(($32|0),($33|0),1,0)|0);
 $35 = tempRet0;
 $36 = $obDone;
 $37 = $36;
 HEAP32[$37>>2] = $34;
 $38 = (($36) + 4)|0;
 $39 = $38;
 HEAP32[$39>>2] = $35;
 $40 = $orderDistinctMask;
 $41 = $40;
 HEAP32[$41>>2] = 0;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = 0;
 $44 = $ready;
 $45 = $44;
 HEAP32[$45>>2] = 0;
 $46 = (($44) + 4)|0;
 $47 = $46;
 HEAP32[$47>>2] = 0;
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $48 = HEAP8[$isOrderDistinct>>0]|0;
  $49 = $48&255;
  $50 = ($49|0)!=(0);
  if (!($50)) {
   break;
  }
  $51 = $obSat;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($51) + 4)|0;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = $obDone;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = ($56>>>0)<($62>>>0);
  $64 = ($53>>>0)<($59>>>0);
  $65 = ($56|0)==($62|0);
  $66 = $65 & $64;
  $67 = $63 | $66;
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$iLoop>>2]|0;
  $69 = HEAP16[$5>>1]|0;
  $70 = $69&65535;
  $71 = ($68|0)<=($70|0);
  if (!($71)) {
   break;
  }
  $72 = HEAP32[$iLoop>>2]|0;
  $73 = ($72|0)>(0);
  if ($73) {
   $74 = HEAP32[$pLoop>>2]|0;
   $75 = ((($74)) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = $ready;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($82) + 4)|0;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = $84 | $78;
   $89 = $87 | $81;
   $90 = $ready;
   $91 = $90;
   HEAP32[$91>>2] = $88;
   $92 = (($90) + 4)|0;
   $93 = $92;
   HEAP32[$93>>2] = $89;
  }
  $94 = HEAP32[$iLoop>>2]|0;
  $95 = HEAP16[$5>>1]|0;
  $96 = $95&65535;
  $97 = ($94|0)<($96|0);
  if ($97) {
   $98 = HEAP32[$iLoop>>2]|0;
   $99 = HEAP32[$3>>2]|0;
   $100 = ((($99)) + 24|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = (($101) + ($98<<2)|0);
   $103 = HEAP32[$102>>2]|0;
   $105 = $103;
  } else {
   $104 = HEAP32[$6>>2]|0;
   $105 = $104;
  }
  HEAP32[$pLoop>>2] = $105;
  $106 = HEAP32[$pLoop>>2]|0;
  $107 = ((($106)) + 36|0);
  $108 = HEAP32[$107>>2]|0;
  $109 = $108 & 1024;
  $110 = ($109|0)!=(0);
  $111 = HEAP32[$pLoop>>2]|0;
  if ($110) {
   label = 16;
   break;
  }
  $126 = ((($111)) + 16|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = HEAP32[$1>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = ((($131)) + 8|0);
  $133 = (($132) + (($128*72)|0)|0);
  $134 = ((($133)) + 44|0);
  $135 = HEAP32[$134>>2]|0;
  HEAP32[$iCur>>2] = $135;
  HEAP32[$i>>2] = 0;
  while(1) {
   $136 = HEAP32[$i>>2]|0;
   $137 = HEAP16[$nOrderBy>>1]|0;
   $138 = $137&65535;
   $139 = ($136|0)<($138|0);
   if (!($139)) {
    break;
   }
   $140 = HEAP32[$i>>2]|0;
   $141 = (_bitshift64Shl(1,0,($140|0))|0);
   $142 = tempRet0;
   $143 = $obSat;
   $144 = $143;
   $145 = HEAP32[$144>>2]|0;
   $146 = (($143) + 4)|0;
   $147 = $146;
   $148 = HEAP32[$147>>2]|0;
   $149 = $141 & $145;
   $150 = $142 & $148;
   $151 = ($149|0)!=(0);
   $152 = ($150|0)!=(0);
   $153 = $151 | $152;
   do {
    if (!($153)) {
     $154 = HEAP32[$i>>2]|0;
     $155 = HEAP32[$2>>2]|0;
     $156 = ((($155)) + 4|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = (($157) + (($154*20)|0)|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = (_sqlite3ExprSkipCollate($159)|0);
     HEAP32[$pOBExpr>>2] = $160;
     $161 = HEAP32[$pOBExpr>>2]|0;
     $162 = HEAP8[$161>>0]|0;
     $163 = $162&255;
     $164 = ($163|0)!=(152);
     if (!($164)) {
      $165 = HEAP32[$pOBExpr>>2]|0;
      $166 = ((($165)) + 28|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = HEAP32[$iCur>>2]|0;
      $169 = ($167|0)!=($168|0);
      if (!($169)) {
       $170 = HEAP32[$1>>2]|0;
       $171 = ((($170)) + 328|0);
       $172 = HEAP32[$iCur>>2]|0;
       $173 = HEAP32[$pOBExpr>>2]|0;
       $174 = ((($173)) + 32|0);
       $175 = HEAP16[$174>>1]|0;
       $176 = $175 << 16 >> 16;
       $177 = $ready;
       $178 = $177;
       $179 = HEAP32[$178>>2]|0;
       $180 = (($177) + 4)|0;
       $181 = $180;
       $182 = HEAP32[$181>>2]|0;
       $183 = $179 ^ -1;
       $184 = $182 ^ -1;
       $185 = (_sqlite3WhereFindTerm($171,$172,$176,$183,$184,386,0)|0);
       HEAP32[$pTerm>>2] = $185;
       $186 = HEAP32[$pTerm>>2]|0;
       $187 = ($186|0)==(0|0);
       if (!($187)) {
        $188 = HEAP32[$pTerm>>2]|0;
        $189 = ((($188)) + 18|0);
        $190 = HEAP16[$189>>1]|0;
        $191 = $190&65535;
        $192 = $191 & 130;
        $193 = ($192|0)!=(0);
        if ($193) {
         $194 = HEAP32[$pOBExpr>>2]|0;
         $195 = ((($194)) + 32|0);
         $196 = HEAP16[$195>>1]|0;
         $197 = $196 << 16 >> 16;
         $198 = ($197|0)>=(0);
         if ($198) {
          $199 = HEAP32[$1>>2]|0;
          $200 = HEAP32[$199>>2]|0;
          $201 = HEAP32[$i>>2]|0;
          $202 = HEAP32[$2>>2]|0;
          $203 = ((($202)) + 4|0);
          $204 = HEAP32[$203>>2]|0;
          $205 = (($204) + (($201*20)|0)|0);
          $206 = HEAP32[$205>>2]|0;
          $207 = (_sqlite3ExprCollSeq($200,$206)|0);
          HEAP32[$pColl>>2] = $207;
          $208 = HEAP32[$pColl>>2]|0;
          $209 = ($208|0)!=(0|0);
          if (!($209)) {
           $210 = HEAP32[$db>>2]|0;
           $211 = ((($210)) + 8|0);
           $212 = HEAP32[$211>>2]|0;
           HEAP32[$pColl>>2] = $212;
          }
          $213 = HEAP32[$pColl>>2]|0;
          $214 = HEAP32[$213>>2]|0;
          HEAP32[$z1>>2] = $214;
          $215 = HEAP32[$1>>2]|0;
          $216 = HEAP32[$215>>2]|0;
          $217 = HEAP32[$pTerm>>2]|0;
          $218 = HEAP32[$217>>2]|0;
          $219 = (_sqlite3ExprCollSeq($216,$218)|0);
          HEAP32[$pColl>>2] = $219;
          $220 = HEAP32[$pColl>>2]|0;
          $221 = ($220|0)!=(0|0);
          if (!($221)) {
           $222 = HEAP32[$db>>2]|0;
           $223 = ((($222)) + 8|0);
           $224 = HEAP32[$223>>2]|0;
           HEAP32[$pColl>>2] = $224;
          }
          $225 = HEAP32[$pColl>>2]|0;
          $226 = HEAP32[$225>>2]|0;
          HEAP32[$z2>>2] = $226;
          $227 = HEAP32[$z1>>2]|0;
          $228 = HEAP32[$z2>>2]|0;
          $229 = (_sqlite3_stricmp($227,$228)|0);
          $230 = ($229|0)!=(0);
          if ($230) {
           break;
          }
         }
        }
        $231 = HEAP32[$i>>2]|0;
        $232 = (_bitshift64Shl(1,0,($231|0))|0);
        $233 = tempRet0;
        $234 = $obSat;
        $235 = $234;
        $236 = HEAP32[$235>>2]|0;
        $237 = (($234) + 4)|0;
        $238 = $237;
        $239 = HEAP32[$238>>2]|0;
        $240 = $236 | $232;
        $241 = $239 | $233;
        $242 = $obSat;
        $243 = $242;
        HEAP32[$243>>2] = $240;
        $244 = (($242) + 4)|0;
        $245 = $244;
        HEAP32[$245>>2] = $241;
       }
      }
     }
    }
   } while(0);
   $246 = HEAP32[$i>>2]|0;
   $247 = (($246) + 1)|0;
   HEAP32[$i>>2] = $247;
  }
  $248 = HEAP32[$pLoop>>2]|0;
  $249 = ((($248)) + 36|0);
  $250 = HEAP32[$249>>2]|0;
  $251 = $250 & 4096;
  $252 = ($251|0)==(0);
  if ($252) {
   $253 = HEAP32[$pLoop>>2]|0;
   $254 = ((($253)) + 36|0);
   $255 = HEAP32[$254>>2]|0;
   $256 = $255 & 256;
   $257 = ($256|0)!=(0);
   if ($257) {
    HEAP32[$pIndex>>2] = 0;
    HEAP16[$nKeyCol>>1] = 0;
    HEAP16[$nColumn>>1] = 1;
   } else {
    $258 = HEAP32[$pLoop>>2]|0;
    $259 = ((($258)) + 24|0);
    $260 = ((($259)) + 4|0);
    $261 = HEAP32[$260>>2]|0;
    HEAP32[$pIndex>>2] = $261;
    $262 = ($261|0)==(0|0);
    if ($262) {
     label = 38;
     break;
    }
    $263 = HEAP32[$pIndex>>2]|0;
    $264 = ((($263)) + 55|0);
    $265 = HEAP8[$264>>0]|0;
    $266 = ($265&255) >>> 2;
    $267 = $266 & 1;
    $268 = $267&255;
    $269 = ($268|0)!=(0);
    if ($269) {
     label = 38;
     break;
    }
    $270 = HEAP32[$pIndex>>2]|0;
    $271 = ((($270)) + 50|0);
    $272 = HEAP16[$271>>1]|0;
    HEAP16[$nKeyCol>>1] = $272;
    $273 = HEAP32[$pIndex>>2]|0;
    $274 = ((($273)) + 52|0);
    $275 = HEAP16[$274>>1]|0;
    HEAP16[$nColumn>>1] = $275;
    $276 = HEAP32[$pIndex>>2]|0;
    $277 = ((($276)) + 54|0);
    $278 = HEAP8[$277>>0]|0;
    $279 = $278&255;
    $280 = ($279|0)!=(0);
    $281 = $280&1;
    $282 = $281&255;
    HEAP8[$isOrderDistinct>>0] = $282;
   }
   HEAP8[$revSet>>0] = 0;
   HEAP8[$rev>>0] = 0;
   HEAP8[$distinctColumns>>0] = 0;
   HEAP32[$j>>2] = 0;
   while(1) {
    $283 = HEAP32[$j>>2]|0;
    $284 = HEAP16[$nColumn>>1]|0;
    $285 = $284&65535;
    $286 = ($283|0)<($285|0);
    if (!($286)) {
     break;
    }
    $287 = HEAP32[$j>>2]|0;
    $288 = HEAP32[$pLoop>>2]|0;
    $289 = ((($288)) + 24|0);
    $290 = HEAP16[$289>>1]|0;
    $291 = $290&65535;
    $292 = ($287|0)<($291|0);
    if ($292) {
     $293 = HEAP32[$pLoop>>2]|0;
     $294 = ((($293)) + 42|0);
     $295 = HEAP16[$294>>1]|0;
     $296 = $295&65535;
     $297 = ($296|0)==(0);
     if ($297) {
      $298 = HEAP32[$j>>2]|0;
      $299 = HEAP32[$pLoop>>2]|0;
      $300 = ((($299)) + 48|0);
      $301 = HEAP32[$300>>2]|0;
      $302 = (($301) + ($298<<2)|0);
      $303 = HEAP32[$302>>2]|0;
      $304 = ((($303)) + 18|0);
      $305 = HEAP16[$304>>1]|0;
      $306 = $305&65535;
      HEAP32[$i>>2] = $306;
      $307 = $306 & 386;
      $308 = ($307|0)!=(0);
      if ($308) {
       $309 = HEAP32[$i>>2]|0;
       $310 = $309 & 256;
       $311 = ($310|0)!=(0);
       if ($311) {
        HEAP8[$isOrderDistinct>>0] = 0;
       }
      } else {
       label = 47;
      }
     } else {
      label = 47;
     }
    } else {
     label = 47;
    }
    if ((label|0) == 47) {
     label = 0;
     $312 = HEAP32[$pIndex>>2]|0;
     $313 = ($312|0)!=(0|0);
     if ($313) {
      $314 = HEAP32[$j>>2]|0;
      $315 = HEAP32[$pIndex>>2]|0;
      $316 = ((($315)) + 4|0);
      $317 = HEAP32[$316>>2]|0;
      $318 = (($317) + ($314<<1)|0);
      $319 = HEAP16[$318>>1]|0;
      $320 = $319 << 16 >> 16;
      HEAP32[$iColumn>>2] = $320;
      $321 = HEAP32[$j>>2]|0;
      $322 = HEAP32[$pIndex>>2]|0;
      $323 = ((($322)) + 28|0);
      $324 = HEAP32[$323>>2]|0;
      $325 = (($324) + ($321)|0);
      $326 = HEAP8[$325>>0]|0;
      HEAP8[$revIdx>>0] = $326;
      $327 = HEAP32[$iColumn>>2]|0;
      $328 = HEAP32[$pIndex>>2]|0;
      $329 = ((($328)) + 12|0);
      $330 = HEAP32[$329>>2]|0;
      $331 = ((($330)) + 32|0);
      $332 = HEAP16[$331>>1]|0;
      $333 = $332 << 16 >> 16;
      $334 = ($327|0)==($333|0);
      if ($334) {
       HEAP32[$iColumn>>2] = -1;
      }
     } else {
      HEAP32[$iColumn>>2] = -1;
      HEAP8[$revIdx>>0] = 0;
     }
     $335 = HEAP8[$isOrderDistinct>>0]|0;
     $336 = $335&255;
     $337 = ($336|0)!=(0);
     $338 = HEAP32[$iColumn>>2]|0;
     $339 = ($338|0)>=(0);
     $or$cond = $337 & $339;
     do {
      if ($or$cond) {
       $340 = HEAP32[$j>>2]|0;
       $341 = HEAP32[$pLoop>>2]|0;
       $342 = ((($341)) + 24|0);
       $343 = HEAP16[$342>>1]|0;
       $344 = $343&65535;
       $345 = ($340|0)>=($344|0);
       if ($345) {
        $346 = HEAP32[$iColumn>>2]|0;
        $347 = HEAP32[$pIndex>>2]|0;
        $348 = ((($347)) + 12|0);
        $349 = HEAP32[$348>>2]|0;
        $350 = ((($349)) + 4|0);
        $351 = HEAP32[$350>>2]|0;
        $352 = (($351) + (($346*24)|0)|0);
        $353 = ((($352)) + 20|0);
        $354 = HEAP8[$353>>0]|0;
        $355 = $354&255;
        $356 = ($355|0)==(0);
        if (!($356)) {
         break;
        }
        HEAP8[$isOrderDistinct>>0] = 0;
       }
      }
     } while(0);
     HEAP8[$bOnce>>0] = 1;
     HEAP8[$isMatch>>0] = 0;
     HEAP32[$i>>2] = 0;
     L71: while(1) {
      $357 = HEAP8[$bOnce>>0]|0;
      $358 = $357&255;
      $359 = ($358|0)!=(0);
      if (!($359)) {
       break;
      }
      $360 = HEAP32[$i>>2]|0;
      $361 = HEAP16[$nOrderBy>>1]|0;
      $362 = $361&65535;
      $363 = ($360|0)<($362|0);
      if (!($363)) {
       break;
      }
      $364 = HEAP32[$i>>2]|0;
      $365 = (_bitshift64Shl(1,0,($364|0))|0);
      $366 = tempRet0;
      $367 = $obSat;
      $368 = $367;
      $369 = HEAP32[$368>>2]|0;
      $370 = (($367) + 4)|0;
      $371 = $370;
      $372 = HEAP32[$371>>2]|0;
      $373 = $365 & $369;
      $374 = $366 & $372;
      $375 = ($373|0)!=(0);
      $376 = ($374|0)!=(0);
      $377 = $375 | $376;
      do {
       if (!($377)) {
        $378 = HEAP32[$i>>2]|0;
        $379 = HEAP32[$2>>2]|0;
        $380 = ((($379)) + 4|0);
        $381 = HEAP32[$380>>2]|0;
        $382 = (($381) + (($378*20)|0)|0);
        $383 = HEAP32[$382>>2]|0;
        $384 = (_sqlite3ExprSkipCollate($383)|0);
        HEAP32[$pOBExpr>>2] = $384;
        $385 = HEAP16[$4>>1]|0;
        $386 = $385&65535;
        $387 = $386 & 768;
        $388 = ($387|0)==(0);
        if ($388) {
         HEAP8[$bOnce>>0] = 0;
        }
        $389 = HEAP32[$iColumn>>2]|0;
        $390 = ($389|0)>=(-1);
        $391 = HEAP32[$pOBExpr>>2]|0;
        if ($390) {
         $392 = HEAP8[$391>>0]|0;
         $393 = $392&255;
         $394 = ($393|0)!=(152);
         if ($394) {
          break;
         }
         $395 = HEAP32[$pOBExpr>>2]|0;
         $396 = ((($395)) + 28|0);
         $397 = HEAP32[$396>>2]|0;
         $398 = HEAP32[$iCur>>2]|0;
         $399 = ($397|0)!=($398|0);
         if ($399) {
          break;
         }
         $400 = HEAP32[$pOBExpr>>2]|0;
         $401 = ((($400)) + 32|0);
         $402 = HEAP16[$401>>1]|0;
         $403 = $402 << 16 >> 16;
         $404 = HEAP32[$iColumn>>2]|0;
         $405 = ($403|0)!=($404|0);
         if ($405) {
          break;
         }
        } else {
         $406 = HEAP32[$j>>2]|0;
         $407 = HEAP32[$pIndex>>2]|0;
         $408 = ((($407)) + 40|0);
         $409 = HEAP32[$408>>2]|0;
         $410 = ((($409)) + 4|0);
         $411 = HEAP32[$410>>2]|0;
         $412 = (($411) + (($406*20)|0)|0);
         $413 = HEAP32[$412>>2]|0;
         $414 = HEAP32[$iCur>>2]|0;
         $415 = (_sqlite3ExprCompare($391,$413,$414)|0);
         $416 = ($415|0)!=(0);
         if ($416) {
          break;
         }
        }
        $417 = HEAP32[$iColumn>>2]|0;
        $418 = ($417|0)>=(0);
        if (!($418)) {
         label = 70;
         break L71;
        }
        $419 = HEAP32[$1>>2]|0;
        $420 = HEAP32[$419>>2]|0;
        $421 = HEAP32[$i>>2]|0;
        $422 = HEAP32[$2>>2]|0;
        $423 = ((($422)) + 4|0);
        $424 = HEAP32[$423>>2]|0;
        $425 = (($424) + (($421*20)|0)|0);
        $426 = HEAP32[$425>>2]|0;
        $427 = (_sqlite3ExprCollSeq($420,$426)|0);
        HEAP32[$pColl>>2] = $427;
        $428 = HEAP32[$pColl>>2]|0;
        $429 = ($428|0)!=(0|0);
        if (!($429)) {
         $430 = HEAP32[$db>>2]|0;
         $431 = ((($430)) + 8|0);
         $432 = HEAP32[$431>>2]|0;
         HEAP32[$pColl>>2] = $432;
        }
        $433 = HEAP32[$pColl>>2]|0;
        $434 = HEAP32[$433>>2]|0;
        $435 = HEAP32[$j>>2]|0;
        $436 = HEAP32[$pIndex>>2]|0;
        $437 = ((($436)) + 32|0);
        $438 = HEAP32[$437>>2]|0;
        $439 = (($438) + ($435<<2)|0);
        $440 = HEAP32[$439>>2]|0;
        $441 = (_sqlite3_stricmp($434,$440)|0);
        $442 = ($441|0)!=(0);
        if (!($442)) {
         label = 70;
         break L71;
        }
       }
      } while(0);
      $443 = HEAP32[$i>>2]|0;
      $444 = (($443) + 1)|0;
      HEAP32[$i>>2] = $444;
     }
     if ((label|0) == 70) {
      label = 0;
      HEAP8[$isMatch>>0] = 1;
     }
     $445 = HEAP8[$isMatch>>0]|0;
     $446 = $445&255;
     $447 = ($446|0)!=(0);
     do {
      if ($447) {
       $448 = HEAP16[$4>>1]|0;
       $449 = $448&65535;
       $450 = $449 & 256;
       $451 = ($450|0)==(0);
       if (!($451)) {
        break;
       }
       $452 = HEAP8[$revSet>>0]|0;
       $453 = ($452<<24>>24)!=(0);
       if ($453) {
        $454 = HEAP8[$rev>>0]|0;
        $455 = $454&255;
        $456 = HEAP8[$revIdx>>0]|0;
        $457 = $456&255;
        $458 = $455 ^ $457;
        $459 = HEAP32[$i>>2]|0;
        $460 = HEAP32[$2>>2]|0;
        $461 = ((($460)) + 4|0);
        $462 = HEAP32[$461>>2]|0;
        $463 = (($462) + (($459*20)|0)|0);
        $464 = ((($463)) + 12|0);
        $465 = HEAP8[$464>>0]|0;
        $466 = $465&255;
        $467 = ($458|0)!=($466|0);
        if (!($467)) {
         break;
        }
        HEAP8[$isMatch>>0] = 0;
        break;
       }
       $468 = HEAP8[$revIdx>>0]|0;
       $469 = $468&255;
       $470 = HEAP32[$i>>2]|0;
       $471 = HEAP32[$2>>2]|0;
       $472 = ((($471)) + 4|0);
       $473 = HEAP32[$472>>2]|0;
       $474 = (($473) + (($470*20)|0)|0);
       $475 = ((($474)) + 12|0);
       $476 = HEAP8[$475>>0]|0;
       $477 = $476&255;
       $478 = $469 ^ $477;
       $479 = $478&255;
       HEAP8[$rev>>0] = $479;
       $480 = HEAP8[$rev>>0]|0;
       $481 = ($480<<24>>24)!=(0);
       if ($481) {
        $482 = HEAP32[$iLoop>>2]|0;
        $483 = (_bitshift64Shl(1,0,($482|0))|0);
        $484 = tempRet0;
        $485 = HEAP32[$7>>2]|0;
        $486 = $485;
        $487 = $486;
        $488 = HEAP32[$487>>2]|0;
        $489 = (($486) + 4)|0;
        $490 = $489;
        $491 = HEAP32[$490>>2]|0;
        $492 = $488 | $483;
        $493 = $491 | $484;
        $494 = $485;
        $495 = $494;
        HEAP32[$495>>2] = $492;
        $496 = (($494) + 4)|0;
        $497 = $496;
        HEAP32[$497>>2] = $493;
       }
       HEAP8[$revSet>>0] = 1;
      }
     } while(0);
     $498 = HEAP8[$isMatch>>0]|0;
     $499 = ($498<<24>>24)!=(0);
     if (!($499)) {
      label = 84;
      break;
     }
     $500 = HEAP32[$iColumn>>2]|0;
     $501 = ($500|0)<(0);
     if ($501) {
      HEAP8[$distinctColumns>>0] = 1;
     }
     $502 = HEAP32[$i>>2]|0;
     $503 = (_bitshift64Shl(1,0,($502|0))|0);
     $504 = tempRet0;
     $505 = $obSat;
     $506 = $505;
     $507 = HEAP32[$506>>2]|0;
     $508 = (($505) + 4)|0;
     $509 = $508;
     $510 = HEAP32[$509>>2]|0;
     $511 = $507 | $503;
     $512 = $510 | $504;
     $513 = $obSat;
     $514 = $513;
     HEAP32[$514>>2] = $511;
     $515 = (($513) + 4)|0;
     $516 = $515;
     HEAP32[$516>>2] = $512;
    }
    $523 = HEAP32[$j>>2]|0;
    $524 = (($523) + 1)|0;
    HEAP32[$j>>2] = $524;
   }
   do {
    if ((label|0) == 84) {
     label = 0;
     $517 = HEAP32[$j>>2]|0;
     $518 = ($517|0)==(0);
     if (!($518)) {
      $519 = HEAP32[$j>>2]|0;
      $520 = HEAP16[$nKeyCol>>1]|0;
      $521 = $520&65535;
      $522 = ($519|0)<($521|0);
      if (!($522)) {
       break;
      }
     }
     HEAP8[$isOrderDistinct>>0] = 0;
    }
   } while(0);
   $525 = HEAP8[$distinctColumns>>0]|0;
   $526 = ($525<<24>>24)!=(0);
   if ($526) {
    HEAP8[$isOrderDistinct>>0] = 1;
   }
  }
  $527 = HEAP8[$isOrderDistinct>>0]|0;
  $528 = ($527<<24>>24)!=(0);
  L118: do {
   if ($528) {
    $529 = HEAP32[$pLoop>>2]|0;
    $530 = ((($529)) + 8|0);
    $531 = $530;
    $532 = $531;
    $533 = HEAP32[$532>>2]|0;
    $534 = (($531) + 4)|0;
    $535 = $534;
    $536 = HEAP32[$535>>2]|0;
    $537 = $orderDistinctMask;
    $538 = $537;
    $539 = HEAP32[$538>>2]|0;
    $540 = (($537) + 4)|0;
    $541 = $540;
    $542 = HEAP32[$541>>2]|0;
    $543 = $539 | $533;
    $544 = $542 | $536;
    $545 = $orderDistinctMask;
    $546 = $545;
    HEAP32[$546>>2] = $543;
    $547 = (($545) + 4)|0;
    $548 = $547;
    HEAP32[$548>>2] = $544;
    HEAP32[$i>>2] = 0;
    while(1) {
     $549 = HEAP32[$i>>2]|0;
     $550 = HEAP16[$nOrderBy>>1]|0;
     $551 = $550&65535;
     $552 = ($549|0)<($551|0);
     if (!($552)) {
      break L118;
     }
     $553 = HEAP32[$i>>2]|0;
     $554 = (_bitshift64Shl(1,0,($553|0))|0);
     $555 = tempRet0;
     $556 = $obSat;
     $557 = $556;
     $558 = HEAP32[$557>>2]|0;
     $559 = (($556) + 4)|0;
     $560 = $559;
     $561 = HEAP32[$560>>2]|0;
     $562 = $554 & $558;
     $563 = $555 & $561;
     $564 = ($562|0)!=(0);
     $565 = ($563|0)!=(0);
     $566 = $564 | $565;
     do {
      if (!($566)) {
       $567 = HEAP32[$i>>2]|0;
       $568 = HEAP32[$2>>2]|0;
       $569 = ((($568)) + 4|0);
       $570 = HEAP32[$569>>2]|0;
       $571 = (($570) + (($567*20)|0)|0);
       $572 = HEAP32[$571>>2]|0;
       HEAP32[$p>>2] = $572;
       $573 = HEAP32[$1>>2]|0;
       $574 = ((($573)) + 68|0);
       $575 = HEAP32[$p>>2]|0;
       $576 = (_sqlite3WhereExprUsage($574,$575)|0);
       $577 = tempRet0;
       $578 = $mTerm;
       $579 = $578;
       HEAP32[$579>>2] = $576;
       $580 = (($578) + 4)|0;
       $581 = $580;
       HEAP32[$581>>2] = $577;
       $582 = $mTerm;
       $583 = $582;
       $584 = HEAP32[$583>>2]|0;
       $585 = (($582) + 4)|0;
       $586 = $585;
       $587 = HEAP32[$586>>2]|0;
       $588 = ($584|0)==(0);
       $589 = ($587|0)==(0);
       $590 = $588 & $589;
       if ($590) {
        $591 = HEAP32[$p>>2]|0;
        $592 = (_sqlite3ExprIsConstant($591)|0);
        $593 = ($592|0)!=(0);
        if (!($593)) {
         break;
        }
       }
       $594 = $mTerm;
       $595 = $594;
       $596 = HEAP32[$595>>2]|0;
       $597 = (($594) + 4)|0;
       $598 = $597;
       $599 = HEAP32[$598>>2]|0;
       $600 = $orderDistinctMask;
       $601 = $600;
       $602 = HEAP32[$601>>2]|0;
       $603 = (($600) + 4)|0;
       $604 = $603;
       $605 = HEAP32[$604>>2]|0;
       $606 = $602 ^ -1;
       $607 = $605 ^ -1;
       $608 = $596 & $606;
       $609 = $599 & $607;
       $610 = ($608|0)==(0);
       $611 = ($609|0)==(0);
       $612 = $610 & $611;
       if ($612) {
        $613 = HEAP32[$i>>2]|0;
        $614 = (_bitshift64Shl(1,0,($613|0))|0);
        $615 = tempRet0;
        $616 = $obSat;
        $617 = $616;
        $618 = HEAP32[$617>>2]|0;
        $619 = (($616) + 4)|0;
        $620 = $619;
        $621 = HEAP32[$620>>2]|0;
        $622 = $618 | $614;
        $623 = $621 | $615;
        $624 = $obSat;
        $625 = $624;
        HEAP32[$625>>2] = $622;
        $626 = (($624) + 4)|0;
        $627 = $626;
        HEAP32[$627>>2] = $623;
       }
      }
     } while(0);
     $628 = HEAP32[$i>>2]|0;
     $629 = (($628) + 1)|0;
     HEAP32[$i>>2] = $629;
    }
   }
  } while(0);
  $630 = HEAP32[$iLoop>>2]|0;
  $631 = (($630) + 1)|0;
  HEAP32[$iLoop>>2] = $631;
 }
 if ((label|0) == 16) {
  $112 = ((($111)) + 24|0);
  $113 = ((($112)) + 5|0);
  $114 = HEAP8[$113>>0]|0;
  $115 = ($114<<24>>24)!=(0);
  if ($115) {
   $116 = $obDone;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($116) + 4)|0;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = $obSat;
   $123 = $122;
   HEAP32[$123>>2] = $118;
   $124 = (($122) + 4)|0;
   $125 = $124;
   HEAP32[$125>>2] = $121;
  }
 }
 else if ((label|0) == 38) {
  HEAP8[$0>>0] = 0;
  $691 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($691|0);
 }
 $632 = $obSat;
 $633 = $632;
 $634 = HEAP32[$633>>2]|0;
 $635 = (($632) + 4)|0;
 $636 = $635;
 $637 = HEAP32[$636>>2]|0;
 $638 = $obDone;
 $639 = $638;
 $640 = HEAP32[$639>>2]|0;
 $641 = (($638) + 4)|0;
 $642 = $641;
 $643 = HEAP32[$642>>2]|0;
 $644 = ($634|0)==($640|0);
 $645 = ($637|0)==($643|0);
 $646 = $644 & $645;
 if ($646) {
  $647 = HEAP16[$nOrderBy>>1]|0;
  $648 = $647&255;
  HEAP8[$0>>0] = $648;
  $691 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($691|0);
 }
 $649 = HEAP8[$isOrderDistinct>>0]|0;
 $650 = ($649<<24>>24)!=(0);
 if ($650) {
  HEAP8[$0>>0] = -1;
  $691 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($691|0);
 }
 $651 = HEAP16[$nOrderBy>>1]|0;
 $652 = $651&65535;
 $653 = (($652) - 1)|0;
 HEAP32[$i>>2] = $653;
 while(1) {
  $654 = HEAP32[$i>>2]|0;
  $655 = ($654|0)>(0);
  if (!($655)) {
   label = 108;
   break;
  }
  $656 = HEAP32[$i>>2]|0;
  $657 = (_bitshift64Shl(1,0,($656|0))|0);
  $658 = tempRet0;
  $659 = (_i64Subtract(($657|0),($658|0),1,0)|0);
  $660 = tempRet0;
  $661 = $m;
  $662 = $661;
  HEAP32[$662>>2] = $659;
  $663 = (($661) + 4)|0;
  $664 = $663;
  HEAP32[$664>>2] = $660;
  $665 = $obSat;
  $666 = $665;
  $667 = HEAP32[$666>>2]|0;
  $668 = (($665) + 4)|0;
  $669 = $668;
  $670 = HEAP32[$669>>2]|0;
  $671 = $m;
  $672 = $671;
  $673 = HEAP32[$672>>2]|0;
  $674 = (($671) + 4)|0;
  $675 = $674;
  $676 = HEAP32[$675>>2]|0;
  $677 = $667 & $673;
  $678 = $670 & $676;
  $679 = $m;
  $680 = $679;
  $681 = HEAP32[$680>>2]|0;
  $682 = (($679) + 4)|0;
  $683 = $682;
  $684 = HEAP32[$683>>2]|0;
  $685 = ($677|0)==($681|0);
  $686 = ($678|0)==($684|0);
  $687 = $685 & $686;
  $688 = HEAP32[$i>>2]|0;
  if ($687) {
   label = 106;
   break;
  }
  $690 = (($688) + -1)|0;
  HEAP32[$i>>2] = $690;
 }
 if ((label|0) == 106) {
  $689 = $688&255;
  HEAP8[$0>>0] = $689;
  $691 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($691|0);
 }
 else if ((label|0) == 108) {
  HEAP8[$0>>0] = 0;
  $691 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($691|0);
 }
 return (0)|0;
}
function _whereSortingCost($nRow,$nOrderBy,$nSorted) {
 $nRow = $nRow|0;
 $nOrderBy = $nOrderBy|0;
 $nSorted = $nSorted|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rScale = 0, $rSortCost = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 4|0;
 $2 = sp;
 $rScale = sp + 10|0;
 $rSortCost = sp + 8|0;
 HEAP16[$0>>1] = $nRow;
 HEAP32[$1>>2] = $nOrderBy;
 HEAP32[$2>>2] = $nSorted;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($3) - ($4))|0;
 $6 = ($5*100)|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($6|0) / ($7|0))&-1;
 $9 = ($8|0)<(0);
 $10 = $9 << 31 >> 31;
 $11 = (_sqlite3LogEst($8,$10)|0);
 $12 = $11 << 16 >> 16;
 $13 = (($12) - 66)|0;
 $14 = $13&65535;
 HEAP16[$rScale>>1] = $14;
 $15 = HEAP16[$0>>1]|0;
 $16 = $15 << 16 >> 16;
 $17 = HEAP16[$0>>1]|0;
 $18 = (_estLog($17)|0);
 $19 = $18 << 16 >> 16;
 $20 = (($16) + ($19))|0;
 $21 = HEAP16[$rScale>>1]|0;
 $22 = $21 << 16 >> 16;
 $23 = (($20) + ($22))|0;
 $24 = (($23) + 16)|0;
 $25 = $24&65535;
 HEAP16[$rSortCost>>1] = $25;
 $26 = HEAP16[$rSortCost>>1]|0;
 STACKTOP = sp;return ($26|0);
}
function _translateColumnToCopy($v,$iStart,$iTabCur,$iRegister,$bIncrRowid) {
 $v = $v|0;
 $iStart = $iStart|0;
 $iTabCur = $iTabCur|0;
 $iRegister = $iRegister|0;
 $bIncrRowid = $bIncrRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iEnd = 0, $pOp = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pOp = sp + 4|0;
 $iEnd = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iStart;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iRegister;
 HEAP32[$4>>2] = $bIncrRowid;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3VdbeGetOp($5,$6)|0);
 HEAP32[$pOp>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3VdbeCurrentAddr($8)|0);
 HEAP32[$iEnd>>2] = $9;
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$iEnd>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pOp>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($15|0)!=($16|0);
  do {
   if (!($17)) {
    $18 = HEAP32[$pOp>>2]|0;
    $19 = HEAP8[$18>>0]|0;
    $20 = $19&255;
    $21 = ($20|0)==(47);
    $22 = HEAP32[$pOp>>2]|0;
    if ($21) {
     HEAP8[$22>>0] = 30;
     $23 = HEAP32[$pOp>>2]|0;
     $24 = ((($23)) + 8|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = HEAP32[$3>>2]|0;
     $27 = (($25) + ($26))|0;
     $28 = HEAP32[$pOp>>2]|0;
     $29 = ((($28)) + 4|0);
     HEAP32[$29>>2] = $27;
     $30 = HEAP32[$pOp>>2]|0;
     $31 = ((($30)) + 12|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$pOp>>2]|0;
     $34 = ((($33)) + 8|0);
     HEAP32[$34>>2] = $32;
     $35 = HEAP32[$pOp>>2]|0;
     $36 = ((($35)) + 12|0);
     HEAP32[$36>>2] = 0;
     break;
    }
    $37 = HEAP8[$22>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)==(103);
    if ($39) {
     $40 = HEAP32[$4>>2]|0;
     $41 = ($40|0)!=(0);
     $42 = HEAP32[$pOp>>2]|0;
     if ($41) {
      HEAP8[$42>>0] = 37;
      $43 = HEAP32[$pOp>>2]|0;
      $44 = ((($43)) + 8|0);
      $45 = HEAP32[$44>>2]|0;
      $46 = HEAP32[$pOp>>2]|0;
      $47 = ((($46)) + 4|0);
      HEAP32[$47>>2] = $45;
      $48 = HEAP32[$pOp>>2]|0;
      $49 = ((($48)) + 8|0);
      HEAP32[$49>>2] = 1;
      break;
     } else {
      HEAP8[$42>>0] = 25;
      $50 = HEAP32[$pOp>>2]|0;
      $51 = ((($50)) + 4|0);
      HEAP32[$51>>2] = 0;
      $52 = HEAP32[$pOp>>2]|0;
      $53 = ((($52)) + 12|0);
      HEAP32[$53>>2] = 0;
      break;
     }
    }
   }
  } while(0);
  $54 = HEAP32[$1>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$1>>2] = $55;
  $56 = HEAP32[$pOp>>2]|0;
  $57 = ((($56)) + 20|0);
  HEAP32[$pOp>>2] = $57;
 }
 STACKTOP = sp;return;
}
function _explainIndexRange($pStr,$pLoop) {
 $pStr = $pStr|0;
 $pLoop = $pLoop|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $nEq = 0, $nSkip = 0, $pIndex = 0, $vararg_buffer = 0, $z = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pIndex = sp + 24|0;
 $nEq = sp + 38|0;
 $nSkip = sp + 36|0;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $z = sp + 12|0;
 $z1 = sp + 8|0;
 $z2 = sp + 4|0;
 HEAP32[$0>>2] = $pStr;
 HEAP32[$1>>2] = $pLoop;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pIndex>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP16[$7>>1]|0;
 HEAP16[$nEq>>1] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP16[$10>>1]|0;
 HEAP16[$nSkip>>1] = $11;
 $12 = HEAP16[$nEq>>1]|0;
 $13 = $12&65535;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 36|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $17 & 48;
  $19 = ($18|0)==(0);
  if ($19) {
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($20,32981,2);
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP16[$nEq>>1]|0;
  $23 = $22&65535;
  $24 = ($21|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$pIndex>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = (_explainIndexColumnName($25,$26)|0);
  HEAP32[$z>>2] = $27;
  $28 = HEAP32[$i>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   _sqlite3StrAccumAppend($30,32984,5);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP16[$nSkip>>1]|0;
  $34 = $33&65535;
  $35 = ($32|0)>=($34|0);
  $36 = $35 ? 32990 : 32995;
  $37 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer>>2] = $37;
  _sqlite3XPrintf($31,$36,$vararg_buffer);
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 $40 = HEAP32[$i>>2]|0;
 HEAP32[$j>>2] = $40;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 36|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $43 & 32;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$pIndex>>2]|0;
  $47 = HEAP32[$i>>2]|0;
  $48 = (_explainIndexColumnName($46,$47)|0);
  HEAP32[$z1>>2] = $48;
  $49 = HEAP32[$0>>2]|0;
  $50 = HEAP32[$i>>2]|0;
  $51 = (($50) + 1)|0;
  HEAP32[$i>>2] = $51;
  $52 = HEAP32[$z1>>2]|0;
  _explainAppendTerm($49,$50,$52,32912);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = ((($53)) + 36|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = $55 & 16;
 $57 = ($56|0)!=(0);
 if ($57) {
  $58 = HEAP32[$pIndex>>2]|0;
  $59 = HEAP32[$j>>2]|0;
  $60 = (_explainIndexColumnName($58,$59)|0);
  HEAP32[$z2>>2] = $60;
  $61 = HEAP32[$0>>2]|0;
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$z2>>2]|0;
  _explainAppendTerm($61,$62,$63,32914);
 }
 $64 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($64,20733,1);
 STACKTOP = sp;return;
}
function _explainIndexColumnName($pIdx,$i) {
 $pIdx = $pIdx|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<1)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 HEAP32[$2>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(-2);
 if ($11) {
  HEAP32[$0>>2] = 33003;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(-1);
 if ($13) {
  HEAP32[$0>>2] = 27337;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + (($14*24)|0)|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$0>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _explainAppendTerm($pStr,$iTerm,$zColumn,$zOp) {
 $pStr = $pStr|0;
 $iTerm = $iTerm|0;
 $zColumn = $zColumn|0;
 $zOp = $zOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pStr;
 HEAP32[$1>>2] = $iTerm;
 HEAP32[$2>>2] = $zColumn;
 HEAP32[$3>>2] = $zOp;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($6,32984,5);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3StrAccumAppendAll($7,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 _sqlite3StrAccumAppend($9,$10,1);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($11,21679,1);
 STACKTOP = sp;return;
}
function _codeEqualityTerm($pParse,$pTerm,$pLevel,$iEq,$bRev,$iTarget) {
 $pParse = $pParse|0;
 $pTerm = $pTerm|0;
 $pLevel = $pLevel|0;
 $iEq = $iEq|0;
 $bRev = $bRev|0;
 $iTarget = $iTarget|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eType = 0, $iReg = 0, $iTab = 0, $pIn = 0, $pLoop = 0, $pX = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $pX = sp + 24|0;
 $v = sp + 20|0;
 $iReg = sp + 16|0;
 $eType = sp + 12|0;
 $iTab = sp + 8|0;
 $pIn = sp + 4|0;
 $pLoop = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTerm;
 HEAP32[$2>>2] = $pLevel;
 HEAP32[$3>>2] = $iEq;
 HEAP32[$4>>2] = $bRev;
 HEAP32[$5>>2] = $iTarget;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pX>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$pX>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(79);
 if (!($14)) {
  $15 = HEAP32[$pX>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(73);
  if (!($18)) {
   $25 = HEAP32[$pX>>2]|0;
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = ($27|0)==(76);
   if ($28) {
    $29 = HEAP32[$5>>2]|0;
    HEAP32[$iReg>>2] = $29;
    $30 = HEAP32[$v>>2]|0;
    $31 = HEAP32[$iReg>>2]|0;
    (_sqlite3VdbeAddOp2($30,25,0,$31)|0);
    $141 = HEAP32[$2>>2]|0;
    $142 = HEAP32[$1>>2]|0;
    _disableTerm($141,$142);
    $143 = HEAP32[$iReg>>2]|0;
    STACKTOP = sp;return ($143|0);
   }
   $32 = HEAP32[$2>>2]|0;
   $33 = ((($32)) + 64|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pLoop>>2] = $34;
   $35 = HEAP32[$pLoop>>2]|0;
   $36 = ((($35)) + 36|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = $37 & 1024;
   $39 = ($38|0)==(0);
   if ($39) {
    $40 = HEAP32[$pLoop>>2]|0;
    $41 = ((($40)) + 24|0);
    $42 = ((($41)) + 4|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)!=(0|0);
    if ($44) {
     $45 = HEAP32[$3>>2]|0;
     $46 = HEAP32[$pLoop>>2]|0;
     $47 = ((($46)) + 24|0);
     $48 = ((($47)) + 4|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ((($49)) + 28|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + ($45)|0);
     $53 = HEAP8[$52>>0]|0;
     $54 = $53&255;
     $55 = ($54|0)!=(0);
     if ($55) {
      $56 = HEAP32[$4>>2]|0;
      $57 = ($56|0)!=(0);
      $58 = $57 ^ 1;
      $59 = $58&1;
      HEAP32[$4>>2] = $59;
     }
    }
   }
   $60 = HEAP32[$5>>2]|0;
   HEAP32[$iReg>>2] = $60;
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$pX>>2]|0;
   $63 = (_sqlite3FindInIndex($61,$62,4,0)|0);
   HEAP32[$eType>>2] = $63;
   $64 = HEAP32[$eType>>2]|0;
   $65 = ($64|0)==(4);
   if ($65) {
    $66 = HEAP32[$4>>2]|0;
    $67 = ($66|0)!=(0);
    $68 = $67 ^ 1;
    $69 = $68&1;
    HEAP32[$4>>2] = $69;
   }
   $70 = HEAP32[$pX>>2]|0;
   $71 = ((($70)) + 28|0);
   $72 = HEAP32[$71>>2]|0;
   HEAP32[$iTab>>2] = $72;
   $73 = HEAP32[$v>>2]|0;
   $74 = HEAP32[$4>>2]|0;
   $75 = ($74|0)!=(0);
   $76 = $75 ? 105 : 108;
   $77 = HEAP32[$iTab>>2]|0;
   (_sqlite3VdbeAddOp2($73,$76,$77,0)|0);
   $78 = HEAP32[$pLoop>>2]|0;
   $79 = ((($78)) + 36|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = $80 | 2048;
   HEAP32[$79>>2] = $81;
   $82 = HEAP32[$2>>2]|0;
   $83 = ((($82)) + 56|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)==(0);
   if ($85) {
    $86 = HEAP32[$v>>2]|0;
    $87 = (_sqlite3VdbeMakeLabel($86)|0);
    $88 = HEAP32[$2>>2]|0;
    $89 = ((($88)) + 16|0);
    HEAP32[$89>>2] = $87;
   }
   $90 = HEAP32[$2>>2]|0;
   $91 = ((($90)) + 56|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = (($92) + 1)|0;
   HEAP32[$91>>2] = $93;
   $94 = HEAP32[$0>>2]|0;
   $95 = HEAP32[$94>>2]|0;
   $96 = HEAP32[$2>>2]|0;
   $97 = ((($96)) + 56|0);
   $98 = ((($97)) + 4|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$2>>2]|0;
   $101 = ((($100)) + 56|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = ($102*12)|0;
   $104 = (_sqlite3DbReallocOrFree($95,$99,$103,0)|0);
   $105 = HEAP32[$2>>2]|0;
   $106 = ((($105)) + 56|0);
   $107 = ((($106)) + 4|0);
   HEAP32[$107>>2] = $104;
   $108 = HEAP32[$2>>2]|0;
   $109 = ((($108)) + 56|0);
   $110 = ((($109)) + 4|0);
   $111 = HEAP32[$110>>2]|0;
   HEAP32[$pIn>>2] = $111;
   $112 = HEAP32[$pIn>>2]|0;
   $113 = ($112|0)!=(0|0);
   $114 = HEAP32[$2>>2]|0;
   $115 = ((($114)) + 56|0);
   if (!($113)) {
    HEAP32[$115>>2] = 0;
    $141 = HEAP32[$2>>2]|0;
    $142 = HEAP32[$1>>2]|0;
    _disableTerm($141,$142);
    $143 = HEAP32[$iReg>>2]|0;
    STACKTOP = sp;return ($143|0);
   }
   $116 = HEAP32[$115>>2]|0;
   $117 = (($116) - 1)|0;
   $118 = HEAP32[$pIn>>2]|0;
   $119 = (($118) + (($117*12)|0)|0);
   HEAP32[$pIn>>2] = $119;
   $120 = HEAP32[$iTab>>2]|0;
   $121 = HEAP32[$pIn>>2]|0;
   HEAP32[$121>>2] = $120;
   $122 = HEAP32[$eType>>2]|0;
   $123 = ($122|0)==(1);
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$iTab>>2]|0;
   $126 = HEAP32[$iReg>>2]|0;
   if ($123) {
    $127 = (_sqlite3VdbeAddOp2($124,103,$125,$126)|0);
    $128 = HEAP32[$pIn>>2]|0;
    $129 = ((($128)) + 4|0);
    HEAP32[$129>>2] = $127;
   } else {
    $130 = (_sqlite3VdbeAddOp3($124,47,$125,0,$126)|0);
    $131 = HEAP32[$pIn>>2]|0;
    $132 = ((($131)) + 4|0);
    HEAP32[$132>>2] = $130;
   }
   $133 = HEAP32[$4>>2]|0;
   $134 = ($133|0)!=(0);
   $135 = $134 ? 4 : 5;
   $136 = $135&255;
   $137 = HEAP32[$pIn>>2]|0;
   $138 = ((($137)) + 8|0);
   HEAP8[$138>>0] = $136;
   $139 = HEAP32[$v>>2]|0;
   $140 = HEAP32[$iReg>>2]|0;
   (_sqlite3VdbeAddOp1($139,76,$140)|0);
   $141 = HEAP32[$2>>2]|0;
   $142 = HEAP32[$1>>2]|0;
   _disableTerm($141,$142);
   $143 = HEAP32[$iReg>>2]|0;
   STACKTOP = sp;return ($143|0);
  }
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$pX>>2]|0;
 $21 = ((($20)) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$5>>2]|0;
 $24 = (_sqlite3ExprCodeTarget($19,$22,$23)|0);
 HEAP32[$iReg>>2] = $24;
 $141 = HEAP32[$2>>2]|0;
 $142 = HEAP32[$1>>2]|0;
 _disableTerm($141,$142);
 $143 = HEAP32[$iReg>>2]|0;
 STACKTOP = sp;return ($143|0);
}
function _disableTerm($pLevel,$pTerm) {
 $pLevel = $pLevel|0;
 $pTerm = $pTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $nLoop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nLoop = sp;
 HEAP32[$0>>2] = $pLevel;
 HEAP32[$1>>2] = $pTerm;
 HEAP32[$nLoop>>2] = 0;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   label = 14;
   break;
  }
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = $7 & 4;
  $9 = ($8|0)==(0);
  if (!($9)) {
   label = 14;
   break;
  }
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)==(0);
  if (!($12)) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $16 & 1;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    label = 14;
    break;
   }
  }
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 72|0);
  $21 = $20;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 40|0);
  $29 = $28;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = $23 & $31;
  $36 = $26 & $34;
  $37 = ($35|0)==(0);
  $38 = ($36|0)==(0);
  $39 = $37 & $38;
  if (!($39)) {
   label = 14;
   break;
  }
  $40 = HEAP32[$nLoop>>2]|0;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 20|0);
   $44 = HEAP16[$43>>1]|0;
   $45 = $44&65535;
   $46 = $45 & 1024;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 20|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = $51 | 512;
    $53 = $52&65535;
    HEAP16[$49>>1] = $53;
   } else {
    label = 10;
   }
  } else {
   label = 10;
  }
  if ((label|0) == 10) {
   label = 0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 20|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   $58 = $57 | 4;
   $59 = $58&65535;
   HEAP16[$55>>1] = $59;
  }
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($62|0)<(0);
  if ($63) {
   label = 14;
   break;
  }
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 4|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  $68 = ((($67)) + 24|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = ((($69)) + 20|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + (($66*48)|0)|0);
  HEAP32[$1>>2] = $72;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 22|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = (($75) + -1)<<24>>24;
  HEAP8[$74>>0] = $76;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 22|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = ($80|0)!=(0);
  if ($81) {
   label = 14;
   break;
  }
  $82 = HEAP32[$nLoop>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$nLoop>>2] = $83;
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
}
function _codeAllEqualityTerms($pParse,$pLevel,$bRev,$nExtraReg,$pzAff) {
 $pParse = $pParse|0;
 $pLevel = $pLevel|0;
 $bRev = $bRev|0;
 $nExtraReg = $nExtraReg|0;
 $pzAff = $pzAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iIdxCur = 0, $j = 0, $nEq = 0, $nReg = 0, $nSkip = 0, $pIdx = 0, $pLoop = 0, $pRight = 0, $pTerm = 0, $r1 = 0;
 var $regBase = 0, $v = 0, $zAff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $nEq = sp + 66|0;
 $nSkip = sp + 64|0;
 $v = sp + 40|0;
 $pIdx = sp + 36|0;
 $pTerm = sp + 32|0;
 $pLoop = sp + 28|0;
 $j = sp + 24|0;
 $regBase = sp + 20|0;
 $nReg = sp + 16|0;
 $zAff = sp + 12|0;
 $iIdxCur = sp + 8|0;
 $r1 = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLevel;
 HEAP32[$2>>2] = $bRev;
 HEAP32[$3>>2] = $nExtraReg;
 HEAP32[$4>>2] = $pzAff;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 64|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pLoop>>2] = $10;
 $11 = HEAP32[$pLoop>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP16[$12>>1]|0;
 HEAP16[$nEq>>1] = $13;
 $14 = HEAP32[$pLoop>>2]|0;
 $15 = ((($14)) + 42|0);
 $16 = HEAP16[$15>>1]|0;
 HEAP16[$nSkip>>1] = $16;
 $17 = HEAP32[$pLoop>>2]|0;
 $18 = ((($17)) + 24|0);
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pIdx>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 76|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 1)|0;
 HEAP32[$regBase>>2] = $24;
 $25 = HEAP32[$pLoop>>2]|0;
 $26 = ((($25)) + 24|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = HEAP32[$3>>2]|0;
 $30 = (($28) + ($29))|0;
 HEAP32[$nReg>>2] = $30;
 $31 = HEAP32[$nReg>>2]|0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 76|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31))|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$pIdx>>2]|0;
 $41 = (_sqlite3IndexAffinityStr($39,$40)|0);
 $42 = (_sqlite3DbStrDup($37,$41)|0);
 HEAP32[$zAff>>2] = $42;
 $43 = HEAP16[$nSkip>>1]|0;
 $44 = ($43<<16>>16)!=(0);
 L1: do {
  if ($44) {
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 8|0);
   $47 = HEAP32[$46>>2]|0;
   HEAP32[$iIdxCur>>2] = $47;
   $48 = HEAP32[$v>>2]|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = ($49|0)!=(0);
   $51 = $50 ? 105 : 108;
   $52 = HEAP32[$iIdxCur>>2]|0;
   (_sqlite3VdbeAddOp1($48,$51,$52)|0);
   $53 = HEAP32[$v>>2]|0;
   $54 = (_sqlite3VdbeAddOp0($53,13)|0);
   HEAP32[$j>>2] = $54;
   $55 = HEAP32[$v>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   $57 = ($56|0)!=(0);
   $58 = $57 ? 63 : 66;
   $59 = HEAP32[$iIdxCur>>2]|0;
   $60 = HEAP32[$regBase>>2]|0;
   $61 = HEAP16[$nSkip>>1]|0;
   $62 = $61&65535;
   $63 = (_sqlite3VdbeAddOp4Int($55,$58,$59,0,$60,$62)|0);
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 20|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP32[$v>>2]|0;
   $67 = HEAP32[$j>>2]|0;
   _sqlite3VdbeJumpHere($66,$67);
   HEAP32[$j>>2] = 0;
   while(1) {
    $68 = HEAP32[$j>>2]|0;
    $69 = HEAP16[$nSkip>>1]|0;
    $70 = $69&65535;
    $71 = ($68|0)<($70|0);
    if (!($71)) {
     break L1;
    }
    $72 = HEAP32[$v>>2]|0;
    $73 = HEAP32[$iIdxCur>>2]|0;
    $74 = HEAP32[$j>>2]|0;
    $75 = HEAP32[$regBase>>2]|0;
    $76 = HEAP32[$j>>2]|0;
    $77 = (($75) + ($76))|0;
    (_sqlite3VdbeAddOp3($72,47,$73,$74,$77)|0);
    $78 = HEAP32[$j>>2]|0;
    $79 = (($78) + 1)|0;
    HEAP32[$j>>2] = $79;
   }
  }
 } while(0);
 $80 = HEAP16[$nSkip>>1]|0;
 $81 = $80&65535;
 HEAP32[$j>>2] = $81;
 while(1) {
  $82 = HEAP32[$j>>2]|0;
  $83 = HEAP16[$nEq>>1]|0;
  $84 = $83&65535;
  $85 = ($82|0)<($84|0);
  if (!($85)) {
   break;
  }
  $86 = HEAP32[$j>>2]|0;
  $87 = HEAP32[$pLoop>>2]|0;
  $88 = ((($87)) + 48|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = (($89) + ($86<<2)|0);
  $91 = HEAP32[$90>>2]|0;
  HEAP32[$pTerm>>2] = $91;
  $92 = HEAP32[$0>>2]|0;
  $93 = HEAP32[$pTerm>>2]|0;
  $94 = HEAP32[$1>>2]|0;
  $95 = HEAP32[$j>>2]|0;
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$regBase>>2]|0;
  $98 = HEAP32[$j>>2]|0;
  $99 = (($97) + ($98))|0;
  $100 = (_codeEqualityTerm($92,$93,$94,$95,$96,$99)|0);
  HEAP32[$r1>>2] = $100;
  $101 = HEAP32[$r1>>2]|0;
  $102 = HEAP32[$regBase>>2]|0;
  $103 = HEAP32[$j>>2]|0;
  $104 = (($102) + ($103))|0;
  $105 = ($101|0)!=($104|0);
  do {
   if ($105) {
    $106 = HEAP32[$nReg>>2]|0;
    $107 = ($106|0)==(1);
    if ($107) {
     $108 = HEAP32[$0>>2]|0;
     $109 = HEAP32[$regBase>>2]|0;
     _sqlite3ReleaseTempReg($108,$109);
     $110 = HEAP32[$r1>>2]|0;
     HEAP32[$regBase>>2] = $110;
     break;
    } else {
     $111 = HEAP32[$v>>2]|0;
     $112 = HEAP32[$r1>>2]|0;
     $113 = HEAP32[$regBase>>2]|0;
     $114 = HEAP32[$j>>2]|0;
     $115 = (($113) + ($114))|0;
     (_sqlite3VdbeAddOp2($111,31,$112,$115)|0);
     break;
    }
   }
  } while(0);
  $116 = HEAP32[$pTerm>>2]|0;
  $117 = ((($116)) + 18|0);
  $118 = HEAP16[$117>>1]|0;
  $119 = $118&65535;
  $120 = $119 & 257;
  $121 = ($120|0)==(0);
  if ($121) {
   $122 = HEAP32[$pTerm>>2]|0;
   $123 = HEAP32[$122>>2]|0;
   $124 = ((($123)) + 16|0);
   $125 = HEAP32[$124>>2]|0;
   HEAP32[$pRight>>2] = $125;
   $126 = HEAP32[$pTerm>>2]|0;
   $127 = ((($126)) + 20|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128&65535;
   $130 = $129 & 2048;
   $131 = ($130|0)==(0);
   if ($131) {
    $132 = HEAP32[$pRight>>2]|0;
    $133 = (_sqlite3ExprCanBeNull($132)|0);
    $134 = ($133|0)!=(0);
    if ($134) {
     $135 = HEAP32[$v>>2]|0;
     $136 = HEAP32[$regBase>>2]|0;
     $137 = HEAP32[$j>>2]|0;
     $138 = (($136) + ($137))|0;
     $139 = HEAP32[$1>>2]|0;
     $140 = ((($139)) + 12|0);
     $141 = HEAP32[$140>>2]|0;
     (_sqlite3VdbeAddOp2($135,76,$138,$141)|0);
    }
   }
   $142 = HEAP32[$zAff>>2]|0;
   $143 = ($142|0)!=(0|0);
   if ($143) {
    $144 = HEAP32[$pRight>>2]|0;
    $145 = HEAP32[$j>>2]|0;
    $146 = HEAP32[$zAff>>2]|0;
    $147 = (($146) + ($145)|0);
    $148 = HEAP8[$147>>0]|0;
    $149 = (_sqlite3CompareAffinity($144,$148)|0);
    $150 = $149 << 24 >> 24;
    $151 = ($150|0)==(65);
    if ($151) {
     $152 = HEAP32[$j>>2]|0;
     $153 = HEAP32[$zAff>>2]|0;
     $154 = (($153) + ($152)|0);
     HEAP8[$154>>0] = 65;
    }
    $155 = HEAP32[$pRight>>2]|0;
    $156 = HEAP32[$j>>2]|0;
    $157 = HEAP32[$zAff>>2]|0;
    $158 = (($157) + ($156)|0);
    $159 = HEAP8[$158>>0]|0;
    $160 = (_sqlite3ExprNeedsNoAffinityChange($155,$159)|0);
    $161 = ($160|0)!=(0);
    if ($161) {
     $162 = HEAP32[$j>>2]|0;
     $163 = HEAP32[$zAff>>2]|0;
     $164 = (($163) + ($162)|0);
     HEAP8[$164>>0] = 65;
    }
   }
  }
  $165 = HEAP32[$j>>2]|0;
  $166 = (($165) + 1)|0;
  HEAP32[$j>>2] = $166;
 }
 $167 = HEAP32[$zAff>>2]|0;
 $168 = HEAP32[$4>>2]|0;
 HEAP32[$168>>2] = $167;
 $169 = HEAP32[$regBase>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _whereLikeOptimizationStringFixup($v,$pLevel,$pTerm) {
 $v = $v|0;
 $pLevel = $pLevel|0;
 $pTerm = $pTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pLevel;
 HEAP32[$2>>2] = $pTerm;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 256;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3VdbeGetOp($9,-1)|0);
 HEAP32[$pOp>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pOp>>2]|0;
 $15 = ((($14)) + 12|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$pOp>>2]|0;
 $17 = ((($16)) + 3|0);
 HEAP8[$17>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3ExprNeedsNoAffinityChange($p,$aff) {
 $p = $p|0;
 $aff = $aff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 9|0;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $p;
 HEAP8[$2>>0] = $aff;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(65);
 L1: do {
  if ($5) {
   HEAP32[$0>>2] = 1;
  } else {
   while(1) {
    $6 = HEAP32[$1>>2]|0;
    $7 = HEAP8[$6>>0]|0;
    $8 = $7&255;
    $9 = ($8|0)==(156);
    if ($9) {
     $60 = 1;
    } else {
     $10 = HEAP32[$1>>2]|0;
     $11 = HEAP8[$10>>0]|0;
     $12 = $11&255;
     $13 = ($12|0)==(155);
     $60 = $13;
    }
    $14 = HEAP32[$1>>2]|0;
    if (!($60)) {
     break;
    }
    $15 = ((($14)) + 12|0);
    $16 = HEAP32[$15>>2]|0;
    HEAP32[$1>>2] = $16;
   }
   $17 = HEAP8[$14>>0]|0;
   HEAP8[$op>>0] = $17;
   $18 = HEAP8[$op>>0]|0;
   $19 = $18&255;
   $20 = ($19|0)==(157);
   if ($20) {
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 38|0);
    $23 = HEAP8[$22>>0]|0;
    HEAP8[$op>>0] = $23;
   }
   $24 = HEAP8[$op>>0]|0;
   $25 = $24&255;
   switch ($25|0) {
   case 132:  {
    $26 = HEAP8[$2>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)==(68);
    if ($28) {
     $33 = 1;
    } else {
     $29 = HEAP8[$2>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(67);
     $33 = $31;
    }
    $32 = $33&1;
    HEAP32[$0>>2] = $32;
    break L1;
    break;
   }
   case 133:  {
    $34 = HEAP8[$2>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(69);
    if ($36) {
     $41 = 1;
    } else {
     $37 = HEAP8[$2>>0]|0;
     $38 = $37 << 24 >> 24;
     $39 = ($38|0)==(67);
     $41 = $39;
    }
    $40 = $41&1;
    HEAP32[$0>>2] = $40;
    break L1;
    break;
   }
   case 97:  {
    $42 = HEAP8[$2>>0]|0;
    $43 = $42 << 24 >> 24;
    $44 = ($43|0)==(66);
    $45 = $44&1;
    HEAP32[$0>>2] = $45;
    break L1;
    break;
   }
   case 134:  {
    HEAP32[$0>>2] = 1;
    break L1;
    break;
   }
   case 152:  {
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 32|0);
    $48 = HEAP16[$47>>1]|0;
    $49 = $48 << 16 >> 16;
    $50 = ($49|0)<(0);
    if ($50) {
     $51 = HEAP8[$2>>0]|0;
     $52 = $51 << 24 >> 24;
     $53 = ($52|0)==(68);
     if ($53) {
      $58 = 1;
     } else {
      $54 = HEAP8[$2>>0]|0;
      $55 = $54 << 24 >> 24;
      $56 = ($55|0)==(67);
      $58 = $56;
     }
    } else {
     $58 = 0;
    }
    $57 = $58&1;
    HEAP32[$0>>2] = $57;
    break L1;
    break;
   }
   default: {
    HEAP32[$0>>2] = 0;
    break L1;
   }
   }
  }
 } while(0);
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _codeApplyAffinity($pParse,$base,$n,$zAff) {
 $pParse = $pParse|0;
 $base = $base|0;
 $n = $n|0;
 $zAff = $zAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $base;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $zAff;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 while(1) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ($9|0)>(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = ($13|0)==(65);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = (($15) + -1)|0;
  HEAP32[$2>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$1>>2] = $18;
  $19 = HEAP32[$3>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP32[$3>>2] = $20;
 }
 while(1) {
  $21 = HEAP32[$2>>2]|0;
  $22 = ($21|0)>(1);
  if ($22) {
   $23 = HEAP32[$2>>2]|0;
   $24 = (($23) - 1)|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = (($25) + ($24)|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27 << 24 >> 24;
   $29 = ($28|0)==(65);
   $41 = $29;
  } else {
   $41 = 0;
  }
  $30 = HEAP32[$2>>2]|0;
  if (!($41)) {
   break;
  }
  $31 = (($30) + -1)|0;
  HEAP32[$2>>2] = $31;
 }
 $32 = ($30|0)>(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$v>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp4($33,48,$34,$35,0,$36,$37)|0);
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheAffinityChange($38,$39,$40);
 STACKTOP = sp;return;
}
function _codeDeferredSeek($pWInfo,$pIdx,$iCur,$iIdxCur) {
 $pWInfo = $pWInfo|0;
 $pIdx = $pIdx|0;
 $iCur = $iCur|0;
 $iIdxCur = $iIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $ai = 0;
 var $i = 0, $pParse = 0, $pTab = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pParse = sp + 16|0;
 $v = sp + 12|0;
 $i = sp + 8|0;
 $pTab = sp + 4|0;
 $ai = sp;
 HEAP32[$0>>2] = $pWInfo;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iCur;
 HEAP32[$3>>2] = $iIdxCur;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP32[$pParse>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($9,112,$10,0,$11)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 34|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 32;
 $17 = ($16|0)!=(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$pParse>>2]|0;
 $19 = ((($18)) + 416|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0|0);
 $22 = HEAP32[$pParse>>2]|0;
 if ($21) {
  $23 = ((($22)) + 416|0);
  $24 = HEAP32[$23>>2]|0;
  $26 = $24;
 } else {
  $26 = $22;
 }
 $25 = ((($26)) + 336|0);
 $27 = HEAP32[$25>>2]|0;
 $28 = ($27|0)==(0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pTab>>2] = $31;
 $32 = HEAP32[$pParse>>2]|0;
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ((($34)) + 34|0);
 $36 = HEAP16[$35>>1]|0;
 $37 = $36 << 16 >> 16;
 $38 = (($37) + 1)|0;
 $39 = $38<<2;
 $40 = (_sqlite3DbMallocZero($33,$39,0)|0);
 HEAP32[$ai>>2] = $40;
 $41 = HEAP32[$ai>>2]|0;
 $42 = ($41|0)!=(0|0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$pTab>>2]|0;
 $44 = ((($43)) + 34|0);
 $45 = HEAP16[$44>>1]|0;
 $46 = $45 << 16 >> 16;
 $47 = HEAP32[$ai>>2]|0;
 HEAP32[$47>>2] = $46;
 HEAP32[$i>>2] = 0;
 while(1) {
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 52|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51&65535;
  $53 = (($52) - 1)|0;
  $54 = ($48|0)<($53|0);
  if (!($54)) {
   break;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 4|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($55<<1)|0);
  $60 = HEAP16[$59>>1]|0;
  $61 = $60 << 16 >> 16;
  $62 = ($61|0)>=(0);
  if ($62) {
   $63 = HEAP32[$i>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = HEAP32[$i>>2]|0;
   $66 = HEAP32[$1>>2]|0;
   $67 = ((($66)) + 4|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($68) + ($65<<1)|0);
   $70 = HEAP16[$69>>1]|0;
   $71 = $70 << 16 >> 16;
   $72 = (($71) + 1)|0;
   $73 = HEAP32[$ai>>2]|0;
   $74 = (($73) + ($72<<2)|0);
   HEAP32[$74>>2] = $64;
  }
  $75 = HEAP32[$i>>2]|0;
  $76 = (($75) + 1)|0;
  HEAP32[$i>>2] = $76;
 }
 $77 = HEAP32[$v>>2]|0;
 $78 = HEAP32[$ai>>2]|0;
 _sqlite3VdbeChangeP4($77,-1,$78,-15);
 STACKTOP = sp;return;
}
function _sqlite3IndexAffinityStr($db,$pIdx) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $9 = 0, $aff = 0, $n = 0, $pTab = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $n = sp + 4|0;
 $pTab = sp;
 $x = sp + 20|0;
 $aff = sp + 22|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$2>>2]|0;
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pTab>>2] = $9;
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 52|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12&65535;
  $14 = (($13) + 1)|0;
  $15 = ($14|0)<(0);
  $16 = $15 << 31 >> 31;
  $17 = (_sqlite3DbMallocRaw(0,$14,$16)|0);
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if (!($23)) {
   $24 = HEAP32[$1>>2]|0;
   _sqlite3OomFault($24);
   HEAP32[$0>>2] = 0;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
  HEAP32[$n>>2] = 0;
  while(1) {
   $25 = HEAP32[$n>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = ((($26)) + 52|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = $28&65535;
   $30 = ($25|0)<($29|0);
   $31 = HEAP32[$n>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   if (!($30)) {
    break;
   }
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<1)|0);
   $36 = HEAP16[$35>>1]|0;
   HEAP16[$x>>1] = $36;
   $37 = HEAP16[$x>>1]|0;
   $38 = $37 << 16 >> 16;
   $39 = ($38|0)>=(0);
   $40 = HEAP16[$x>>1]|0;
   $41 = $40 << 16 >> 16;
   do {
    if ($39) {
     $42 = HEAP32[$pTab>>2]|0;
     $43 = ((($42)) + 4|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = (($44) + (($41*24)|0)|0);
     $46 = ((($45)) + 21|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = HEAP32[$n>>2]|0;
     $49 = HEAP32[$2>>2]|0;
     $50 = ((($49)) + 16|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + ($48)|0);
     HEAP8[$52>>0] = $47;
    } else {
     $53 = ($41|0)==(-1);
     $54 = HEAP32[$n>>2]|0;
     $55 = HEAP32[$2>>2]|0;
     if ($53) {
      $56 = ((($55)) + 16|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = (($57) + ($54)|0);
      HEAP8[$58>>0] = 68;
      break;
     } else {
      $59 = ((($55)) + 40|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ((($60)) + 4|0);
      $62 = HEAP32[$61>>2]|0;
      $63 = (($62) + (($54*20)|0)|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (_sqlite3ExprAffinity($64)|0);
      HEAP8[$aff>>0] = $65;
      $66 = HEAP8[$aff>>0]|0;
      $67 = $66 << 24 >> 24;
      $68 = ($67|0)==(0);
      $$ = $68 ? 65 : $65;
      HEAP8[$aff>>0] = $$;
      $69 = HEAP8[$aff>>0]|0;
      $70 = HEAP32[$n>>2]|0;
      $71 = HEAP32[$2>>2]|0;
      $72 = ((($71)) + 16|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = (($73) + ($70)|0);
      HEAP8[$74>>0] = $69;
      break;
     }
    }
   } while(0);
   $75 = HEAP32[$n>>2]|0;
   $76 = (($75) + 1)|0;
   HEAP32[$n>>2] = $76;
  }
  $77 = ((($32)) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (($78) + ($31)|0);
  HEAP8[$79>>0] = 0;
 }
 $80 = HEAP32[$2>>2]|0;
 $81 = ((($80)) + 16|0);
 $82 = HEAP32[$81>>2]|0;
 HEAP32[$0>>2] = $82;
 $83 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($83|0);
}
function _sqlite3WhereClauseClear($pWC) {
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $db = 0, $i = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $a = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pWC;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$a>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = ($12|0)>=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$a>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = $17 & 1;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = HEAP32[$db>>2]|0;
   $21 = HEAP32[$a>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   _sqlite3ExprDelete($20,$22);
  }
  $23 = HEAP32[$a>>2]|0;
  $24 = ((($23)) + 20|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26 & 16;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$a>>2]|0;
   $31 = ((($30)) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   _whereOrInfoDelete($29,$32);
  } else {
   $33 = HEAP32[$a>>2]|0;
   $34 = ((($33)) + 20|0);
   $35 = HEAP16[$34>>1]|0;
   $36 = $35&65535;
   $37 = $36 & 32;
   $38 = ($37|0)!=(0);
   if ($38) {
    $39 = HEAP32[$db>>2]|0;
    $40 = HEAP32[$a>>2]|0;
    $41 = ((($40)) + 12|0);
    $42 = HEAP32[$41>>2]|0;
    _whereAndInfoDelete($39,$42);
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + -1)|0;
  HEAP32[$i>>2] = $44;
  $45 = HEAP32[$a>>2]|0;
  $46 = ((($45)) + 48|0);
  HEAP32[$a>>2] = $46;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 20|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = ($49|0)!=($51|0);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$db>>2]|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 20|0);
 $56 = HEAP32[$55>>2]|0;
 _sqlite3DbFree($53,$56);
 STACKTOP = sp;return;
}
function _whereOrInfoDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3WhereClauseClear($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($3,$4);
 STACKTOP = sp;return;
}
function _whereAndInfoDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3WhereClauseClear($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($3,$4);
 STACKTOP = sp;return;
}
function _sqlite3LogEstToInt($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $n = sp;
 HEAP16[$1>>1] = $x;
 $2 = HEAP16[$1>>1]|0;
 $3 = $2 << 16 >> 16;
 $4 = ($3|0)<(10);
 if ($4) {
  $5 = $0;
  $6 = $5;
  HEAP32[$6>>2] = 1;
  $7 = (($5) + 4)|0;
  $8 = $7;
  HEAP32[$8>>2] = 0;
  $104 = $0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  tempRet0 = ($109);
  STACKTOP = sp;return ($106|0);
 }
 $9 = HEAP16[$1>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = (($10|0) % 10)&-1;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = $n;
 $15 = $14;
 HEAP32[$15>>2] = $11;
 $16 = (($14) + 4)|0;
 $17 = $16;
 HEAP32[$17>>2] = $13;
 $18 = HEAP16[$1>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = (($19|0) / 10)&-1;
 $21 = $20&65535;
 HEAP16[$1>>1] = $21;
 $22 = $n;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27>>>0)>(0);
 $29 = ($24>>>0)>=(5);
 $30 = ($27|0)==(0);
 $31 = $30 & $29;
 $32 = $28 | $31;
 $33 = $n;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 if ($32) {
  $39 = (_i64Subtract(($35|0),($38|0),2,0)|0);
  $40 = tempRet0;
  $41 = $n;
  $42 = $41;
  HEAP32[$42>>2] = $39;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $40;
 } else {
  $45 = ($38>>>0)>(0);
  $46 = ($35>>>0)>=(1);
  $47 = ($38|0)==(0);
  $48 = $47 & $46;
  $49 = $45 | $48;
  if ($49) {
   $50 = $n;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = (_i64Subtract(($52|0),($55|0),1,0)|0);
   $57 = tempRet0;
   $58 = $n;
   $59 = $58;
   HEAP32[$59>>2] = $56;
   $60 = (($58) + 4)|0;
   $61 = $60;
   HEAP32[$61>>2] = $57;
  }
 }
 $62 = HEAP16[$1>>1]|0;
 $63 = $62 << 16 >> 16;
 $64 = ($63|0)>=(3);
 if (!($64)) {
  $87 = $n;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  $93 = (_i64Add(($89|0),($92|0),8,0)|0);
  $94 = tempRet0;
  $95 = HEAP16[$1>>1]|0;
  $96 = $95 << 16 >> 16;
  $97 = (3 - ($96))|0;
  $98 = (_bitshift64Lshr(($93|0),($94|0),($97|0))|0);
  $99 = tempRet0;
  $100 = $0;
  $101 = $100;
  HEAP32[$101>>2] = $98;
  $102 = (($100) + 4)|0;
  $103 = $102;
  HEAP32[$103>>2] = $99;
  $104 = $0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  tempRet0 = ($109);
  STACKTOP = sp;return ($106|0);
 }
 $65 = HEAP16[$1>>1]|0;
 $66 = $65 << 16 >> 16;
 $67 = ($66|0)>(60);
 if ($67) {
  $83 = -1;$86 = 2147483647;
 } else {
  $68 = $n;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = (_i64Add(($70|0),($73|0),8,0)|0);
  $75 = tempRet0;
  $76 = HEAP16[$1>>1]|0;
  $77 = $76 << 16 >> 16;
  $78 = (($77) - 3)|0;
  $79 = (_bitshift64Shl(($74|0),($75|0),($78|0))|0);
  $80 = tempRet0;
  $83 = $79;$86 = $80;
 }
 $81 = $0;
 $82 = $81;
 HEAP32[$82>>2] = $83;
 $84 = (($81) + 4)|0;
 $85 = $84;
 HEAP32[$85>>2] = $86;
 $104 = $0;
 $105 = $104;
 $106 = HEAP32[$105>>2]|0;
 $107 = (($104) + 4)|0;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 tempRet0 = ($109);
 STACKTOP = sp;return ($106|0);
}
function _codeDistinct($pParse,$iTab,$addrRepeat,$N,$iMem) {
 $pParse = $pParse|0;
 $iTab = $iTab|0;
 $addrRepeat = $addrRepeat|0;
 $N = $N|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $v = sp + 4|0;
 $r1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $addrRepeat;
 HEAP32[$3>>2] = $N;
 HEAP32[$4>>2] = $iMem;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3GetTempReg($8)|0);
 HEAP32[$r1>>2] = $9;
 $10 = HEAP32[$v>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Int($10,69,$11,$12,$13,$14)|0);
 $15 = HEAP32[$v>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp3($15,49,$16,$17,$18)|0);
 $19 = HEAP32[$v>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp2($19,110,$20,$21)|0);
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($22,$23);
 STACKTOP = sp;return;
}
function _pushOntoSorter($pParse,$pSort,$pSelect,$regData,$regOrigData,$nData,$nPrefixReg) {
 $pParse = $pParse|0;
 $pSort = $pSort|0;
 $pSelect = $pSelect|0;
 $regData = $regData|0;
 $regOrigData = $regOrigData|0;
 $nData = $nData|0;
 $nPrefixReg = $nPrefixReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addrFirst = 0, $addrJmp = 0, $bSeq = 0, $iLimit = 0, $nBase = 0, $nExpr = 0, $nKey = 0, $nOBSat = 0, $op = 0, $pKI = 0, $pOp = 0, $regBase = 0, $regPrevKey = 0, $regRecord = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $v = sp + 60|0;
 $bSeq = sp + 56|0;
 $nExpr = sp + 52|0;
 $nBase = sp + 48|0;
 $regBase = sp + 44|0;
 $regRecord = sp + 40|0;
 $nOBSat = sp + 36|0;
 $op = sp + 32|0;
 $iLimit = sp + 28|0;
 $regPrevKey = sp + 24|0;
 $addrFirst = sp + 20|0;
 $addrJmp = sp + 16|0;
 $pOp = sp + 12|0;
 $nKey = sp + 8|0;
 $pKI = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSort;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $regData;
 HEAP32[$4>>2] = $regOrigData;
 HEAP32[$5>>2] = $nData;
 HEAP32[$6>>2] = $nPrefixReg;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 28|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 1;
 $15 = ($14|0)==(0);
 $16 = $15&1;
 HEAP32[$bSeq>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$nExpr>>2] = $19;
 $20 = HEAP32[$nExpr>>2]|0;
 $21 = HEAP32[$bSeq>>2]|0;
 $22 = (($20) + ($21))|0;
 $23 = HEAP32[$5>>2]|0;
 $24 = (($22) + ($23))|0;
 HEAP32[$nBase>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 76|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$26>>2] = $28;
 HEAP32[$regRecord>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$nOBSat>>2] = $31;
 $32 = HEAP32[$6>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$3>>2]|0;
  $35 = HEAP32[$nExpr>>2]|0;
  $36 = (($34) - ($35))|0;
  $37 = HEAP32[$bSeq>>2]|0;
  $38 = (($36) - ($37))|0;
  HEAP32[$regBase>>2] = $38;
 } else {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 76|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$regBase>>2] = $42;
  $43 = HEAP32[$nBase>>2]|0;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 76|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + ($43))|0;
  HEAP32[$45>>2] = $47;
 }
 $48 = HEAP32[$2>>2]|0;
 $49 = ((($48)) + 12|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0);
 $52 = HEAP32[$2>>2]|0;
 if ($51) {
  $53 = ((($52)) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 1)|0;
  $58 = $55;
 } else {
  $56 = ((($52)) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $57;
 }
 HEAP32[$iLimit>>2] = $58;
 $59 = HEAP32[$v>>2]|0;
 $60 = (_sqlite3VdbeMakeLabel($59)|0);
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 24|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$0>>2]|0;
 $64 = HEAP32[$1>>2]|0;
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$regBase>>2]|0;
 $67 = HEAP32[$4>>2]|0;
 (_sqlite3ExprCodeExprList($63,$65,$66,$67,5)|0);
 $68 = HEAP32[$bSeq>>2]|0;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$v>>2]|0;
  $71 = HEAP32[$1>>2]|0;
  $72 = ((($71)) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$regBase>>2]|0;
  $75 = HEAP32[$nExpr>>2]|0;
  $76 = (($74) + ($75))|0;
  (_sqlite3VdbeAddOp2($70,73,$73,$76)|0);
 }
 $77 = HEAP32[$6>>2]|0;
 $78 = ($77|0)==(0);
 if ($78) {
  $79 = HEAP32[$0>>2]|0;
  $80 = HEAP32[$3>>2]|0;
  $81 = HEAP32[$regBase>>2]|0;
  $82 = HEAP32[$nExpr>>2]|0;
  $83 = (($81) + ($82))|0;
  $84 = HEAP32[$bSeq>>2]|0;
  $85 = (($83) + ($84))|0;
  $86 = HEAP32[$5>>2]|0;
  _sqlite3ExprCodeMove($79,$80,$85,$86);
 }
 $87 = HEAP32[$v>>2]|0;
 $88 = HEAP32[$regBase>>2]|0;
 $89 = HEAP32[$nOBSat>>2]|0;
 $90 = (($88) + ($89))|0;
 $91 = HEAP32[$nBase>>2]|0;
 $92 = HEAP32[$nOBSat>>2]|0;
 $93 = (($91) - ($92))|0;
 $94 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp3($87,49,$90,$93,$94)|0);
 $95 = HEAP32[$nOBSat>>2]|0;
 $96 = ($95|0)>(0);
 if ($96) {
  $97 = HEAP32[$0>>2]|0;
  $98 = ((($97)) + 76|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = (($99) + 1)|0;
  HEAP32[$regPrevKey>>2] = $100;
  $101 = HEAP32[$1>>2]|0;
  $102 = ((($101)) + 4|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = HEAP32[$0>>2]|0;
  $105 = ((($104)) + 76|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (($106) + ($103))|0;
  HEAP32[$105>>2] = $107;
  $108 = HEAP32[$nExpr>>2]|0;
  $109 = HEAP32[$1>>2]|0;
  $110 = ((($109)) + 4|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (($108) - ($111))|0;
  $113 = HEAP32[$bSeq>>2]|0;
  $114 = (($112) + ($113))|0;
  HEAP32[$nKey>>2] = $114;
  $115 = HEAP32[$bSeq>>2]|0;
  $116 = ($115|0)!=(0);
  $117 = HEAP32[$v>>2]|0;
  if ($116) {
   $118 = HEAP32[$regBase>>2]|0;
   $119 = HEAP32[$nExpr>>2]|0;
   $120 = (($118) + ($119))|0;
   $121 = (_sqlite3VdbeAddOp1($117,46,$120)|0);
   HEAP32[$addrFirst>>2] = $121;
  } else {
   $122 = HEAP32[$1>>2]|0;
   $123 = ((($122)) + 8|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = (_sqlite3VdbeAddOp1($117,59,$124)|0);
   HEAP32[$addrFirst>>2] = $125;
  }
  $126 = HEAP32[$v>>2]|0;
  $127 = HEAP32[$regPrevKey>>2]|0;
  $128 = HEAP32[$regBase>>2]|0;
  $129 = HEAP32[$1>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  (_sqlite3VdbeAddOp3($126,42,$127,$128,$131)|0);
  $132 = HEAP32[$v>>2]|0;
  $133 = HEAP32[$1>>2]|0;
  $134 = ((($133)) + 20|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (_sqlite3VdbeGetOp($132,$135)|0);
  HEAP32[$pOp>>2] = $136;
  $137 = HEAP32[$0>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = ((($138)) + 65|0);
  $140 = HEAP8[$139>>0]|0;
  $141 = ($140<<24>>24)!=(0);
  if ($141) {
   STACKTOP = sp;return;
  }
  $142 = HEAP32[$nKey>>2]|0;
  $143 = HEAP32[$5>>2]|0;
  $144 = (($142) + ($143))|0;
  $145 = HEAP32[$pOp>>2]|0;
  $146 = ((($145)) + 8|0);
  HEAP32[$146>>2] = $144;
  $147 = HEAP32[$pOp>>2]|0;
  $148 = ((($147)) + 16|0);
  $149 = HEAP32[$148>>2]|0;
  HEAP32[$pKI>>2] = $149;
  $150 = HEAP32[$pKI>>2]|0;
  $151 = ((($150)) + 16|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = HEAP32[$pKI>>2]|0;
  $154 = ((($153)) + 6|0);
  $155 = HEAP16[$154>>1]|0;
  $156 = $155&65535;
  _memset(($152|0),0,($156|0))|0;
  $157 = HEAP32[$v>>2]|0;
  $158 = HEAP32[$pKI>>2]|0;
  _sqlite3VdbeChangeP4($157,-1,$158,-6);
  $159 = HEAP32[$0>>2]|0;
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$160>>2]|0;
  $162 = HEAP32[$nOBSat>>2]|0;
  $163 = HEAP32[$pKI>>2]|0;
  $164 = ((($163)) + 8|0);
  $165 = HEAP16[$164>>1]|0;
  $166 = $165&65535;
  $167 = (($166) - 1)|0;
  $168 = (_keyInfoFromExprList($159,$161,$162,$167)|0);
  $169 = HEAP32[$pOp>>2]|0;
  $170 = ((($169)) + 16|0);
  HEAP32[$170>>2] = $168;
  $171 = HEAP32[$v>>2]|0;
  $172 = (_sqlite3VdbeCurrentAddr($171)|0);
  HEAP32[$addrJmp>>2] = $172;
  $173 = HEAP32[$v>>2]|0;
  $174 = HEAP32[$addrJmp>>2]|0;
  $175 = (($174) + 1)|0;
  $176 = HEAP32[$addrJmp>>2]|0;
  $177 = (($176) + 1)|0;
  (_sqlite3VdbeAddOp3($173,43,$175,0,$177)|0);
  $178 = HEAP32[$v>>2]|0;
  $179 = (_sqlite3VdbeMakeLabel($178)|0);
  $180 = HEAP32[$1>>2]|0;
  $181 = ((($180)) + 16|0);
  HEAP32[$181>>2] = $179;
  $182 = HEAP32[$0>>2]|0;
  $183 = ((($182)) + 76|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = (($184) + 1)|0;
  HEAP32[$183>>2] = $185;
  $186 = HEAP32[$1>>2]|0;
  $187 = ((($186)) + 12|0);
  HEAP32[$187>>2] = $185;
  $188 = HEAP32[$v>>2]|0;
  $189 = HEAP32[$1>>2]|0;
  $190 = ((($189)) + 12|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = HEAP32[$1>>2]|0;
  $193 = ((($192)) + 16|0);
  $194 = HEAP32[$193>>2]|0;
  (_sqlite3VdbeAddOp2($188,14,$191,$194)|0);
  $195 = HEAP32[$v>>2]|0;
  $196 = HEAP32[$1>>2]|0;
  $197 = ((($196)) + 8|0);
  $198 = HEAP32[$197>>2]|0;
  (_sqlite3VdbeAddOp1($195,120,$198)|0);
  $199 = HEAP32[$iLimit>>2]|0;
  $200 = ($199|0)!=(0);
  if ($200) {
   $201 = HEAP32[$v>>2]|0;
   $202 = HEAP32[$iLimit>>2]|0;
   $203 = HEAP32[$1>>2]|0;
   $204 = ((($203)) + 24|0);
   $205 = HEAP32[$204>>2]|0;
   (_sqlite3VdbeAddOp2($201,46,$202,$205)|0);
  }
  $206 = HEAP32[$v>>2]|0;
  $207 = HEAP32[$addrFirst>>2]|0;
  _sqlite3VdbeJumpHere($206,$207);
  $208 = HEAP32[$0>>2]|0;
  $209 = HEAP32[$regBase>>2]|0;
  $210 = HEAP32[$regPrevKey>>2]|0;
  $211 = HEAP32[$1>>2]|0;
  $212 = ((($211)) + 4|0);
  $213 = HEAP32[$212>>2]|0;
  _sqlite3ExprCodeMove($208,$209,$210,$213);
  $214 = HEAP32[$v>>2]|0;
  $215 = HEAP32[$addrJmp>>2]|0;
  _sqlite3VdbeJumpHere($214,$215);
 }
 $216 = HEAP32[$1>>2]|0;
 $217 = ((($216)) + 28|0);
 $218 = HEAP8[$217>>0]|0;
 $219 = $218&255;
 $220 = $219 & 1;
 $221 = ($220|0)!=(0);
 if ($221) {
  HEAP32[$op>>2] = 109;
 } else {
  HEAP32[$op>>2] = 110;
 }
 $222 = HEAP32[$v>>2]|0;
 $223 = HEAP32[$op>>2]|0;
 $224 = HEAP32[$1>>2]|0;
 $225 = ((($224)) + 8|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($222,$223,$226,$227)|0);
 $228 = HEAP32[$iLimit>>2]|0;
 $229 = ($228|0)!=(0);
 if (!($229)) {
  STACKTOP = sp;return;
 }
 $230 = HEAP32[$v>>2]|0;
 $231 = HEAP32[$iLimit>>2]|0;
 $232 = (_sqlite3VdbeAddOp3($230,140,$231,0,1)|0);
 HEAP32[$addr>>2] = $232;
 $233 = HEAP32[$v>>2]|0;
 $234 = HEAP32[$1>>2]|0;
 $235 = ((($234)) + 8|0);
 $236 = HEAP32[$235>>2]|0;
 (_sqlite3VdbeAddOp1($233,105,$236)|0);
 $237 = HEAP32[$v>>2]|0;
 $238 = HEAP32[$1>>2]|0;
 $239 = ((($238)) + 8|0);
 $240 = HEAP32[$239>>2]|0;
 (_sqlite3VdbeAddOp1($237,95,$240)|0);
 $241 = HEAP32[$v>>2]|0;
 $242 = HEAP32[$addr>>2]|0;
 _sqlite3VdbeJumpHere($241,$242);
 STACKTOP = sp;return;
}
function _analyzeAggregate($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $enc = 0, $i = 0, $j = 0, $k = 0, $n = 0, $pAggInfo = 0, $pCol = 0, $pE = 0, $pGB = 0, $pItem = 0, $pItem1 = 0, $pNC = 0;
 var $pParse = 0, $pSrcList = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $i = sp + 52|0;
 $pNC = sp + 48|0;
 $pParse = sp + 44|0;
 $pSrcList = sp + 40|0;
 $pAggInfo = sp + 36|0;
 $pItem = sp + 32|0;
 $pCol = sp + 28|0;
 $k = sp + 24|0;
 $j = sp + 20|0;
 $n = sp + 16|0;
 $pGB = sp + 12|0;
 $pTerm = sp + 8|0;
 $pE = sp + 4|0;
 $pItem1 = sp;
 $enc = sp + 68|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pNC>>2] = $5;
 $6 = HEAP32[$pNC>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$pNC>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pSrcList>>2] = $10;
 $11 = HEAP32[$pNC>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pAggInfo>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 switch ($16|0) {
 case 152: case 154:  {
  $17 = HEAP32[$pSrcList>>2]|0;
  $18 = ($17|0)!=(0|0);
  L5: do {
   if ($18) {
    $19 = HEAP32[$pSrcList>>2]|0;
    $20 = ((($19)) + 8|0);
    HEAP32[$pItem>>2] = $20;
    HEAP32[$i>>2] = 0;
    while(1) {
     $21 = HEAP32[$i>>2]|0;
     $22 = HEAP32[$pSrcList>>2]|0;
     $23 = HEAP32[$22>>2]|0;
     $24 = ($21|0)<($23|0);
     if (!($24)) {
      break L5;
     }
     $25 = HEAP32[$2>>2]|0;
     $26 = ((($25)) + 28|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$pItem>>2]|0;
     $29 = ((($28)) + 44|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($27|0)==($30|0);
     if ($31) {
      break;
     }
     $162 = HEAP32[$i>>2]|0;
     $163 = (($162) + 1)|0;
     HEAP32[$i>>2] = $163;
     $164 = HEAP32[$pItem>>2]|0;
     $165 = ((($164)) + 72|0);
     HEAP32[$pItem>>2] = $165;
    }
    $32 = HEAP32[$pAggInfo>>2]|0;
    $33 = ((($32)) + 28|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$pCol>>2] = $34;
    HEAP32[$k>>2] = 0;
    while(1) {
     $35 = HEAP32[$k>>2]|0;
     $36 = HEAP32[$pAggInfo>>2]|0;
     $37 = ((($36)) + 32|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = ($35|0)<($38|0);
     if (!($39)) {
      break;
     }
     $40 = HEAP32[$pCol>>2]|0;
     $41 = ((($40)) + 4|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = HEAP32[$2>>2]|0;
     $44 = ((($43)) + 28|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($42|0)==($45|0);
     if ($46) {
      $47 = HEAP32[$pCol>>2]|0;
      $48 = ((($47)) + 8|0);
      $49 = HEAP32[$48>>2]|0;
      $50 = HEAP32[$2>>2]|0;
      $51 = ((($50)) + 32|0);
      $52 = HEAP16[$51>>1]|0;
      $53 = $52 << 16 >> 16;
      $54 = ($49|0)==($53|0);
      if ($54) {
       break;
      }
     }
     $55 = HEAP32[$k>>2]|0;
     $56 = (($55) + 1)|0;
     HEAP32[$k>>2] = $56;
     $57 = HEAP32[$pCol>>2]|0;
     $58 = ((($57)) + 24|0);
     HEAP32[$pCol>>2] = $58;
    }
    $59 = HEAP32[$k>>2]|0;
    $60 = HEAP32[$pAggInfo>>2]|0;
    $61 = ((($60)) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($59|0)>=($62|0);
    if ($63) {
     $64 = HEAP32[$pParse>>2]|0;
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$pAggInfo>>2]|0;
     $67 = (_addAggInfoColumn($65,$66)|0);
     HEAP32[$k>>2] = $67;
     $68 = ($67|0)>=(0);
     if ($68) {
      $69 = HEAP32[$k>>2]|0;
      $70 = HEAP32[$pAggInfo>>2]|0;
      $71 = ((($70)) + 28|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = (($72) + (($69*24)|0)|0);
      HEAP32[$pCol>>2] = $73;
      $74 = HEAP32[$2>>2]|0;
      $75 = ((($74)) + 44|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = HEAP32[$pCol>>2]|0;
      HEAP32[$77>>2] = $76;
      $78 = HEAP32[$2>>2]|0;
      $79 = ((($78)) + 28|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = HEAP32[$pCol>>2]|0;
      $82 = ((($81)) + 4|0);
      HEAP32[$82>>2] = $80;
      $83 = HEAP32[$2>>2]|0;
      $84 = ((($83)) + 32|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85 << 16 >> 16;
      $87 = HEAP32[$pCol>>2]|0;
      $88 = ((($87)) + 8|0);
      HEAP32[$88>>2] = $86;
      $89 = HEAP32[$pParse>>2]|0;
      $90 = ((($89)) + 76|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (($91) + 1)|0;
      HEAP32[$90>>2] = $92;
      $93 = HEAP32[$pCol>>2]|0;
      $94 = ((($93)) + 16|0);
      HEAP32[$94>>2] = $92;
      $95 = HEAP32[$pCol>>2]|0;
      $96 = ((($95)) + 12|0);
      HEAP32[$96>>2] = -1;
      $97 = HEAP32[$2>>2]|0;
      $98 = HEAP32[$pCol>>2]|0;
      $99 = ((($98)) + 20|0);
      HEAP32[$99>>2] = $97;
      $100 = HEAP32[$pAggInfo>>2]|0;
      $101 = ((($100)) + 24|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ($102|0)!=(0|0);
      L22: do {
       if ($103) {
        $104 = HEAP32[$pAggInfo>>2]|0;
        $105 = ((($104)) + 24|0);
        $106 = HEAP32[$105>>2]|0;
        HEAP32[$pGB>>2] = $106;
        $107 = HEAP32[$pGB>>2]|0;
        $108 = ((($107)) + 4|0);
        $109 = HEAP32[$108>>2]|0;
        HEAP32[$pTerm>>2] = $109;
        $110 = HEAP32[$pGB>>2]|0;
        $111 = HEAP32[$110>>2]|0;
        HEAP32[$n>>2] = $111;
        HEAP32[$j>>2] = 0;
        while(1) {
         $112 = HEAP32[$j>>2]|0;
         $113 = HEAP32[$n>>2]|0;
         $114 = ($112|0)<($113|0);
         if (!($114)) {
          break L22;
         }
         $115 = HEAP32[$pTerm>>2]|0;
         $116 = HEAP32[$115>>2]|0;
         HEAP32[$pE>>2] = $116;
         $117 = HEAP32[$pE>>2]|0;
         $118 = HEAP8[$117>>0]|0;
         $119 = $118&255;
         $120 = ($119|0)==(152);
         if ($120) {
          $121 = HEAP32[$pE>>2]|0;
          $122 = ((($121)) + 28|0);
          $123 = HEAP32[$122>>2]|0;
          $124 = HEAP32[$2>>2]|0;
          $125 = ((($124)) + 28|0);
          $126 = HEAP32[$125>>2]|0;
          $127 = ($123|0)==($126|0);
          if ($127) {
           $128 = HEAP32[$pE>>2]|0;
           $129 = ((($128)) + 32|0);
           $130 = HEAP16[$129>>1]|0;
           $131 = $130 << 16 >> 16;
           $132 = HEAP32[$2>>2]|0;
           $133 = ((($132)) + 32|0);
           $134 = HEAP16[$133>>1]|0;
           $135 = $134 << 16 >> 16;
           $136 = ($131|0)==($135|0);
           if ($136) {
            break;
           }
          }
         }
         $140 = HEAP32[$j>>2]|0;
         $141 = (($140) + 1)|0;
         HEAP32[$j>>2] = $141;
         $142 = HEAP32[$pTerm>>2]|0;
         $143 = ((($142)) + 20|0);
         HEAP32[$pTerm>>2] = $143;
        }
        $137 = HEAP32[$j>>2]|0;
        $138 = HEAP32[$pCol>>2]|0;
        $139 = ((($138)) + 12|0);
        HEAP32[$139>>2] = $137;
       }
      } while(0);
      $144 = HEAP32[$pCol>>2]|0;
      $145 = ((($144)) + 12|0);
      $146 = HEAP32[$145>>2]|0;
      $147 = ($146|0)<(0);
      if ($147) {
       $148 = HEAP32[$pAggInfo>>2]|0;
       $149 = ((($148)) + 12|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = (($150) + 1)|0;
       HEAP32[$149>>2] = $151;
       $152 = HEAP32[$pCol>>2]|0;
       $153 = ((($152)) + 12|0);
       HEAP32[$153>>2] = $150;
      }
     }
    }
    $154 = HEAP32[$pAggInfo>>2]|0;
    $155 = HEAP32[$2>>2]|0;
    $156 = ((($155)) + 40|0);
    HEAP32[$156>>2] = $154;
    $157 = HEAP32[$2>>2]|0;
    HEAP8[$157>>0] = -102;
    $158 = HEAP32[$k>>2]|0;
    $159 = $158&65535;
    $160 = HEAP32[$2>>2]|0;
    $161 = ((($160)) + 34|0);
    HEAP16[$161>>1] = $159;
   }
  } while(0);
  HEAP32[$0>>2] = 1;
  $267 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($267|0);
  break;
 }
 case 153:  {
  $166 = HEAP32[$pNC>>2]|0;
  $167 = ((($166)) + 28|0);
  $168 = HEAP16[$167>>1]|0;
  $169 = $168&65535;
  $170 = $169 & 8;
  $171 = ($170|0)==(0);
  if ($171) {
   $172 = HEAP32[$1>>2]|0;
   $173 = ((($172)) + 16|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = HEAP32[$2>>2]|0;
   $176 = ((($175)) + 38|0);
   $177 = HEAP8[$176>>0]|0;
   $178 = $177&255;
   $179 = ($174|0)==($178|0);
   if ($179) {
    $180 = HEAP32[$pAggInfo>>2]|0;
    $181 = ((($180)) + 40|0);
    $182 = HEAP32[$181>>2]|0;
    HEAP32[$pItem1>>2] = $182;
    HEAP32[$i>>2] = 0;
    while(1) {
     $183 = HEAP32[$i>>2]|0;
     $184 = HEAP32[$pAggInfo>>2]|0;
     $185 = ((($184)) + 44|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = ($183|0)<($186|0);
     if (!($187)) {
      break;
     }
     $188 = HEAP32[$pItem1>>2]|0;
     $189 = HEAP32[$188>>2]|0;
     $190 = HEAP32[$2>>2]|0;
     $191 = (_sqlite3ExprCompare($189,$190,-1)|0);
     $192 = ($191|0)==(0);
     if ($192) {
      break;
     }
     $193 = HEAP32[$i>>2]|0;
     $194 = (($193) + 1)|0;
     HEAP32[$i>>2] = $194;
     $195 = HEAP32[$pItem1>>2]|0;
     $196 = ((($195)) + 16|0);
     HEAP32[$pItem1>>2] = $196;
    }
    $197 = HEAP32[$i>>2]|0;
    $198 = HEAP32[$pAggInfo>>2]|0;
    $199 = ((($198)) + 44|0);
    $200 = HEAP32[$199>>2]|0;
    $201 = ($197|0)>=($200|0);
    do {
     if ($201) {
      $202 = HEAP32[$pParse>>2]|0;
      $203 = HEAP32[$202>>2]|0;
      $204 = ((($203)) + 62|0);
      $205 = HEAP8[$204>>0]|0;
      HEAP8[$enc>>0] = $205;
      $206 = HEAP32[$pParse>>2]|0;
      $207 = HEAP32[$206>>2]|0;
      $208 = HEAP32[$pAggInfo>>2]|0;
      $209 = (_addAggInfoFunc($207,$208)|0);
      HEAP32[$i>>2] = $209;
      $210 = HEAP32[$i>>2]|0;
      $211 = ($210|0)>=(0);
      if ($211) {
       $212 = HEAP32[$i>>2]|0;
       $213 = HEAP32[$pAggInfo>>2]|0;
       $214 = ((($213)) + 40|0);
       $215 = HEAP32[$214>>2]|0;
       $216 = (($215) + ($212<<4)|0);
       HEAP32[$pItem1>>2] = $216;
       $217 = HEAP32[$2>>2]|0;
       $218 = HEAP32[$pItem1>>2]|0;
       HEAP32[$218>>2] = $217;
       $219 = HEAP32[$pParse>>2]|0;
       $220 = ((($219)) + 76|0);
       $221 = HEAP32[$220>>2]|0;
       $222 = (($221) + 1)|0;
       HEAP32[$220>>2] = $222;
       $223 = HEAP32[$pItem1>>2]|0;
       $224 = ((($223)) + 8|0);
       HEAP32[$224>>2] = $222;
       $225 = HEAP32[$pParse>>2]|0;
       $226 = HEAP32[$225>>2]|0;
       $227 = HEAP32[$2>>2]|0;
       $228 = ((($227)) + 8|0);
       $229 = HEAP32[$228>>2]|0;
       $230 = HEAP32[$2>>2]|0;
       $231 = ((($230)) + 8|0);
       $232 = HEAP32[$231>>2]|0;
       $233 = (_sqlite3Strlen30($232)|0);
       $234 = HEAP32[$2>>2]|0;
       $235 = ((($234)) + 20|0);
       $236 = HEAP32[$235>>2]|0;
       $237 = ($236|0)!=(0|0);
       if ($237) {
        $238 = HEAP32[$2>>2]|0;
        $239 = ((($238)) + 20|0);
        $240 = HEAP32[$239>>2]|0;
        $241 = HEAP32[$240>>2]|0;
        $243 = $241;
       } else {
        $243 = 0;
       }
       $242 = HEAP8[$enc>>0]|0;
       $244 = (_sqlite3FindFunction($226,$229,$233,$243,$242,0)|0);
       $245 = HEAP32[$pItem1>>2]|0;
       $246 = ((($245)) + 4|0);
       HEAP32[$246>>2] = $244;
       $247 = HEAP32[$2>>2]|0;
       $248 = ((($247)) + 4|0);
       $249 = HEAP32[$248>>2]|0;
       $250 = $249 & 16;
       $251 = ($250|0)!=(0);
       if ($251) {
        $252 = HEAP32[$pParse>>2]|0;
        $253 = ((($252)) + 72|0);
        $254 = HEAP32[$253>>2]|0;
        $255 = (($254) + 1)|0;
        HEAP32[$253>>2] = $255;
        $256 = HEAP32[$pItem1>>2]|0;
        $257 = ((($256)) + 12|0);
        HEAP32[$257>>2] = $254;
        break;
       } else {
        $258 = HEAP32[$pItem1>>2]|0;
        $259 = ((($258)) + 12|0);
        HEAP32[$259>>2] = -1;
        break;
       }
      }
     }
    } while(0);
    $260 = HEAP32[$i>>2]|0;
    $261 = $260&65535;
    $262 = HEAP32[$2>>2]|0;
    $263 = ((($262)) + 34|0);
    HEAP16[$263>>1] = $261;
    $264 = HEAP32[$pAggInfo>>2]|0;
    $265 = HEAP32[$2>>2]|0;
    $266 = ((($265)) + 40|0);
    HEAP32[$266>>2] = $264;
    HEAP32[$0>>2] = 1;
    $267 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($267|0);
   }
  }
  HEAP32[$0>>2] = 0;
  $267 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($267|0);
  break;
 }
 default: {
  HEAP32[$0>>2] = 0;
  $267 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($267|0);
 }
 }
 return (0)|0;
}
function _analyzeAggregatesInSelect($pWalker,$pSelect) {
 $pWalker = $pWalker|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pSelect;
 STACKTOP = sp;return 0;
}
function _addAggInfoColumn($db,$pInfo) {
 $db = $db|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = (_sqlite3ArrayAllocate($2,$5,24,$7,$i)|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 28|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _addAggInfoFunc($db,$pInfo) {
 $db = $db|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = (_sqlite3ArrayAllocate($2,$5,16,$7,$i)|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 40|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3ArrayAllocate($db,$pArray,$szEntry,$pnEntry,$pIdx) {
 $db = $db|0;
 $pArray = $pArray|0;
 $szEntry = $szEntry|0;
 $pnEntry = $pnEntry|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pNew = 0, $sz = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $z = sp + 12|0;
 $n = sp + 8|0;
 $sz = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pArray;
 HEAP32[$3>>2] = $szEntry;
 HEAP32[$4>>2] = $pnEntry;
 HEAP32[$5>>2] = $pIdx;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$n>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = $8 & $10;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$n>>2]|0;
   $14 = ($13|0)==(0);
   $15 = HEAP32[$n>>2]|0;
   $16 = $15<<1;
   $17 = $14 ? 1 : $16;
   HEAP32[$sz>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = HEAP32[$sz>>2]|0;
   $21 = HEAP32[$3>>2]|0;
   $22 = Math_imul($20, $21)|0;
   $23 = ($22|0)<(0);
   $24 = $23 << 31 >> 31;
   $25 = (_sqlite3DbRealloc($18,$19,$22,$24)|0);
   HEAP32[$pNew>>2] = $25;
   $26 = HEAP32[$pNew>>2]|0;
   $27 = ($26|0)==(0|0);
   if (!($27)) {
    $30 = HEAP32[$pNew>>2]|0;
    HEAP32[$2>>2] = $30;
    break;
   }
   $28 = HEAP32[$5>>2]|0;
   HEAP32[$28>>2] = -1;
   $29 = HEAP32[$2>>2]|0;
   HEAP32[$0>>2] = $29;
   $44 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($44|0);
  }
 } while(0);
 $31 = HEAP32[$2>>2]|0;
 HEAP32[$z>>2] = $31;
 $32 = HEAP32[$n>>2]|0;
 $33 = HEAP32[$3>>2]|0;
 $34 = Math_imul($32, $33)|0;
 $35 = HEAP32[$z>>2]|0;
 $36 = (($35) + ($34)|0);
 $37 = HEAP32[$3>>2]|0;
 _memset(($36|0),0,($37|0))|0;
 $38 = HEAP32[$n>>2]|0;
 $39 = HEAP32[$5>>2]|0;
 HEAP32[$39>>2] = $38;
 $40 = HEAP32[$4>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 1)|0;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _generateColumnTypes($pParse,$pTabList,$pEList) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $sNC = 0, $v = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $v = sp + 44|0;
 $i = sp + 40|0;
 $sNC = sp + 8|0;
 $p = sp + 4|0;
 $zType = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($sNC)) + 4|0);
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($9|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*20)|0)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  $20 = (_columnTypeImpl($sNC,$19,0)|0);
  HEAP32[$zType>>2] = $20;
  $21 = HEAP32[$v>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$zType>>2]|0;
  (_sqlite3VdbeSetColName($21,$22,1,$23,(-1))|0);
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
 }
 STACKTOP = sp;return;
}
function _sqlite3TriggersExist($pParse,$pTab,$op,$pChanges,$pMask) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $op = $op|0;
 $pChanges = $pChanges|0;
 $pMask = $pMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mask = 0, $p = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $mask = sp + 8|0;
 $pList = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $pMask;
 HEAP32[$mask>>2] = 0;
 HEAP32[$pList>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 8388608;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (_sqlite3TriggerList($11,$12)|0);
  HEAP32[$pList>>2] = $13;
 }
 $14 = HEAP32[$pList>>2]|0;
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP32[$p>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = HEAP32[$2>>2]|0;
  $22 = ($20|0)==($21|0);
  if ($22) {
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = (_checkColumnOverlap($25,$26)|0);
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$p>>2]|0;
    $30 = ((($29)) + 9|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = $31&255;
    $33 = HEAP32[$mask>>2]|0;
    $34 = $33 | $32;
    HEAP32[$mask>>2] = $34;
   }
  }
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 32|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$p>>2] = $37;
 }
 $38 = HEAP32[$4>>2]|0;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  $42 = HEAP32[$mask>>2]|0;
  $43 = ($42|0)!=(0);
  $44 = HEAP32[$pList>>2]|0;
  $45 = $43 ? $44 : 0;
  STACKTOP = sp;return ($45|0);
 }
 $40 = HEAP32[$mask>>2]|0;
 $41 = HEAP32[$4>>2]|0;
 HEAP32[$41>>2] = $40;
 $42 = HEAP32[$mask>>2]|0;
 $43 = ($42|0)!=(0);
 $44 = HEAP32[$pList>>2]|0;
 $45 = $43 ? $44 : 0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3FkRequired($pParse,$pTab,$aChange,$chngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $aChange = $aChange|0;
 $chngRowid = $chngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $chngRowid;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 524288;
 $10 = ($9|0)!=(0);
 L1: do {
  if ($10) {
   $11 = HEAP32[$3>>2]|0;
   $12 = ($11|0)!=(0|0);
   $13 = HEAP32[$2>>2]|0;
   if (!($12)) {
    $14 = (_sqlite3FkReferences($13)|0);
    $15 = ($14|0)!=(0|0);
    if ($15) {
     $21 = 1;
    } else {
     $16 = HEAP32[$2>>2]|0;
     $17 = ((($16)) + 16|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = ($18|0)!=(0|0);
     $21 = $19;
    }
    $20 = $21&1;
    HEAP32[$0>>2] = $20;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
   $22 = ((($13)) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$p>>2] = $23;
   while(1) {
    $24 = HEAP32[$p>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$2>>2]|0;
    if (!($25)) {
     break;
    }
    $27 = HEAP32[$p>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = HEAP32[$4>>2]|0;
    $30 = (_fkChildIsModified($26,$27,$28,$29)|0);
    $31 = ($30|0)!=(0);
    if ($31) {
     label = 9;
     break;
    }
    $32 = HEAP32[$p>>2]|0;
    $33 = ((($32)) + 4|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$p>>2] = $34;
   }
   if ((label|0) == 9) {
    HEAP32[$0>>2] = 1;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
   $35 = (_sqlite3FkReferences($26)|0);
   HEAP32[$p>>2] = $35;
   while(1) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ($36|0)!=(0|0);
    if (!($37)) {
     break L1;
    }
    $38 = HEAP32[$2>>2]|0;
    $39 = HEAP32[$p>>2]|0;
    $40 = HEAP32[$3>>2]|0;
    $41 = HEAP32[$4>>2]|0;
    $42 = (_fkParentIsModified($38,$39,$40,$41)|0);
    $43 = ($42|0)!=(0);
    if ($43) {
     break;
    }
    $44 = HEAP32[$p>>2]|0;
    $45 = ((($44)) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$p>>2] = $46;
   }
   HEAP32[$0>>2] = 1;
   $47 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($47|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3IsReadOnly($pParse,$pTab,$viewOk) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $viewOk = $viewOk|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $viewOk;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 42|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 16;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_sqlite3GetVTable($11,$12)|0);
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 52|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)==(0|0);
  if (!($19)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 42|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 & 1;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = HEAP32[$1>>2]|0;
    $27 = HEAP32[$26>>2]|0;
    $28 = ((($27)) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = $29 & 2048;
    $31 = ($30|0)==(0);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = ((($32)) + 18|0);
     $34 = HEAP8[$33>>0]|0;
     $35 = $34&255;
     $36 = ($35|0)==(0);
     if ($36) {
      break;
     }
    }
   }
   $40 = HEAP32[$3>>2]|0;
   $41 = ($40|0)!=(0);
   if (!($41)) {
    $42 = HEAP32[$2>>2]|0;
    $43 = ((($42)) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = ($44|0)!=(0|0);
    if ($45) {
     $46 = HEAP32[$1>>2]|0;
     $47 = HEAP32[$2>>2]|0;
     $48 = HEAP32[$47>>2]|0;
     HEAP32[$vararg_buffer1>>2] = $48;
     _sqlite3ErrorMsg($46,33439,$vararg_buffer1);
     HEAP32[$0>>2] = 1;
     $49 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($49|0);
    }
   }
   HEAP32[$0>>2] = 0;
   $49 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 } while(0);
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$vararg_buffer>>2] = $39;
 _sqlite3ErrorMsg($37,33410,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3AuthContextPush($pParse,$pContext,$zContext) {
 $pParse = $pParse|0;
 $pContext = $pContext|0;
 $zContext = $zContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $zContext;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 496|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 496|0);
 HEAP32[$12>>2] = $10;
 STACKTOP = sp;return;
}
function _sqlite3VdbeCountChanges($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 88|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -5;
 $5 = $4 | 4;
 HEAP16[$2>>1] = $5;
 STACKTOP = sp;return;
}
function _sqlite3MaterializeView($pParse,$pView,$pWhere,$iCur) {
 $pParse = $pParse|0;
 $pView = $pView|0;
 $pWhere = $pWhere|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $dest = 0, $iDb = 0, $pFrom = 0, $pSel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $dest = sp + 16|0;
 $pSel = sp + 12|0;
 $pFrom = sp + 8|0;
 $db = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pView;
 HEAP32[$2>>2] = $pWhere;
 HEAP32[$3>>2] = $iCur;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3SchemaToIndex($6,$9)|0);
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3ExprDup($11,$12,0)|0);
 HEAP32[$2>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = (_sqlite3SrcListAppend($14,0,0,0)|0);
 HEAP32[$pFrom>>2] = $15;
 $16 = HEAP32[$pFrom>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3DbStrDup($18,$20)|0);
  $22 = HEAP32[$pFrom>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = ((($23)) + 8|0);
  HEAP32[$24>>2] = $21;
  $25 = HEAP32[$db>>2]|0;
  $26 = HEAP32[$iDb>>2]|0;
  $27 = HEAP32[$db>>2]|0;
  $28 = ((($27)) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<4)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3DbStrDup($25,$31)|0);
  $33 = HEAP32[$pFrom>>2]|0;
  $34 = ((($33)) + 8|0);
  $35 = ((($34)) + 4|0);
  HEAP32[$35>>2] = $32;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pFrom>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = (_sqlite3SelectNew($36,0,$37,$38,0,0,0,-32768,0,0)|0);
 HEAP32[$pSel>>2] = $39;
 $40 = HEAP32[$3>>2]|0;
 _sqlite3SelectDestInit($dest,12,$40);
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$pSel>>2]|0;
 (_sqlite3Select($41,$42,$dest)|0);
 $43 = HEAP32[$db>>2]|0;
 $44 = HEAP32[$pSel>>2]|0;
 _sqlite3SelectDelete($43,$44);
 STACKTOP = sp;return;
}
function _sqlite3WhereOkOnePass($pWInfo,$aiCur) {
 $pWInfo = $pWInfo|0;
 $aiCur = $aiCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWInfo;
 HEAP32[$1>>2] = $aiCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 60|0);
 ;HEAP32[$2>>2]=HEAP32[$4>>2]|0;HEAP32[$2+4>>2]=HEAP32[$4+4>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 38|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3OpenTableAndIndices($pParse,$pTab,$op,$p5,$iBase,$aToOpen,$piDataCur,$piIdxCur) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $op = $op|0;
 $p5 = $p5|0;
 $iBase = $iBase|0;
 $aToOpen = $aToOpen|0;
 $piDataCur = $piDataCur|0;
 $piIdxCur = $piIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $pIdx = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 56|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $8 = sp + 24|0;
 $i = sp + 20|0;
 $iDb = sp + 16|0;
 $iDataCur = sp + 12|0;
 $pIdx = sp + 8|0;
 $v = sp + 4|0;
 $iIdxCur = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $op;
 HEAP8[$4>>0] = $p5;
 HEAP32[$5>>2] = $iBase;
 HEAP32[$6>>2] = $aToOpen;
 HEAP32[$7>>2] = $piDataCur;
 HEAP32[$8>>2] = $piIdxCur;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 16;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 64|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = (_sqlite3GetVdbe($21)|0);
 HEAP32[$v>>2] = $22;
 $23 = HEAP32[$5>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 72|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$5>>2] = $27;
 }
 $28 = HEAP32[$5>>2]|0;
 $29 = (($28) + 1)|0;
 HEAP32[$5>>2] = $29;
 HEAP32[$iDataCur>>2] = $28;
 $30 = HEAP32[$7>>2]|0;
 $31 = ($30|0)!=(0|0);
 if ($31) {
  $32 = HEAP32[$iDataCur>>2]|0;
  $33 = HEAP32[$7>>2]|0;
  HEAP32[$33>>2] = $32;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 42|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $37 & 32;
 $39 = ($38|0)==(0);
 do {
  if ($39) {
   $40 = HEAP32[$6>>2]|0;
   $41 = ($40|0)==(0|0);
   if (!($41)) {
    $42 = HEAP32[$6>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)!=(0);
    if (!($45)) {
     label = 11;
     break;
    }
   }
   $46 = HEAP32[$1>>2]|0;
   $47 = HEAP32[$iDataCur>>2]|0;
   $48 = HEAP32[$iDb>>2]|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = HEAP32[$3>>2]|0;
   _sqlite3OpenTable($46,$47,$48,$49,$50);
  } else {
   label = 11;
  }
 } while(0);
 if ((label|0) == 11) {
  $51 = HEAP32[$1>>2]|0;
  $52 = HEAP32[$iDb>>2]|0;
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 28|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = ($56|0)==(55);
  $58 = $57&1;
  $59 = $58&255;
  $60 = HEAP32[$2>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  _sqlite3TableLock($51,$52,$55,$59,$61);
 }
 $62 = HEAP32[$8>>2]|0;
 $63 = ($62|0)!=(0|0);
 if ($63) {
  $64 = HEAP32[$5>>2]|0;
  $65 = HEAP32[$8>>2]|0;
  HEAP32[$65>>2] = $64;
 }
 HEAP32[$i>>2] = 0;
 $66 = HEAP32[$2>>2]|0;
 $67 = ((($66)) + 8|0);
 $68 = HEAP32[$67>>2]|0;
 HEAP32[$pIdx>>2] = $68;
 while(1) {
  $69 = HEAP32[$pIdx>>2]|0;
  $70 = ($69|0)!=(0|0);
  $71 = HEAP32[$5>>2]|0;
  if (!($70)) {
   break;
  }
  $72 = (($71) + 1)|0;
  HEAP32[$5>>2] = $72;
  HEAP32[$iIdxCur>>2] = $71;
  $73 = HEAP32[$6>>2]|0;
  $74 = ($73|0)==(0|0);
  if ($74) {
   label = 18;
  } else {
   $75 = HEAP32[$i>>2]|0;
   $76 = (($75) + 1)|0;
   $77 = HEAP32[$6>>2]|0;
   $78 = (($77) + ($76)|0);
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = ($80|0)!=(0);
   if ($81) {
    label = 18;
   }
  }
  if ((label|0) == 18) {
   label = 0;
   $82 = HEAP32[$v>>2]|0;
   $83 = HEAP32[$3>>2]|0;
   $84 = HEAP32[$iIdxCur>>2]|0;
   $85 = HEAP32[$pIdx>>2]|0;
   $86 = ((($85)) + 44|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$iDb>>2]|0;
   (_sqlite3VdbeAddOp3($82,$83,$84,$87,$88)|0);
   $89 = HEAP32[$1>>2]|0;
   $90 = HEAP32[$pIdx>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($89,$90);
  }
  $91 = HEAP32[$pIdx>>2]|0;
  $92 = ((($91)) + 55|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93 & 3;
  $95 = $94&255;
  $96 = ($95|0)==(2);
  if ($96) {
   $97 = HEAP32[$2>>2]|0;
   $98 = ((($97)) + 42|0);
   $99 = HEAP8[$98>>0]|0;
   $100 = $99&255;
   $101 = $100 & 32;
   $102 = ($101|0)==(0);
   if ($102) {
    label = 23;
   } else {
    $103 = HEAP32[$7>>2]|0;
    $104 = ($103|0)!=(0|0);
    if ($104) {
     $105 = HEAP32[$iIdxCur>>2]|0;
     $106 = HEAP32[$7>>2]|0;
     HEAP32[$106>>2] = $105;
    }
   }
  } else {
   label = 23;
  }
  if ((label|0) == 23) {
   label = 0;
   $107 = HEAP32[$v>>2]|0;
   $108 = HEAP8[$4>>0]|0;
   _sqlite3VdbeChangeP5($107,$108);
  }
  $109 = HEAP32[$pIdx>>2]|0;
  $110 = ((($109)) + 20|0);
  $111 = HEAP32[$110>>2]|0;
  HEAP32[$pIdx>>2] = $111;
  $112 = HEAP32[$i>>2]|0;
  $113 = (($112) + 1)|0;
  HEAP32[$i>>2] = $113;
 }
 $114 = HEAP32[$1>>2]|0;
 $115 = ((($114)) + 72|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = ($71|0)>($116|0);
 if ($117) {
  $118 = HEAP32[$5>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = ((($119)) + 72|0);
  HEAP32[$120>>2] = $118;
 }
 $121 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $121;
 $122 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($122|0);
}
function _sqlite3VtabMakeWritable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $apVtabLock = 0, $i = 0, $n = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pToplevel = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $apVtabLock = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 416|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 416|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8;
 } else {
  $9 = $6;
 }
 HEAP32[$pToplevel>>2] = $9;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pToplevel>>2]|0;
  $12 = ((($11)) + 456|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$pToplevel>>2]|0;
  $18 = ((($17)) + 524|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($15|0)==($21|0);
  if ($22) {
   label = 10;
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 if ((label|0) == 10) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$pToplevel>>2]|0;
 $26 = ((($25)) + 456|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 $29 = $28<<2;
 HEAP32[$n>>2] = $29;
 $30 = HEAP32[$pToplevel>>2]|0;
 $31 = ((($30)) + 524|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$n>>2]|0;
 $34 = ($33|0)<(0);
 $35 = $34 << 31 >> 31;
 $36 = (_sqlite3_realloc64($32,$33,$35)|0);
 HEAP32[$apVtabLock>>2] = $36;
 $37 = HEAP32[$apVtabLock>>2]|0;
 $38 = ($37|0)!=(0|0);
 if ($38) {
  $39 = HEAP32[$apVtabLock>>2]|0;
  $40 = HEAP32[$pToplevel>>2]|0;
  $41 = ((($40)) + 524|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pToplevel>>2]|0;
  $44 = ((($43)) + 456|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$44>>2] = $46;
  $47 = HEAP32[$pToplevel>>2]|0;
  $48 = ((($47)) + 524|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($45<<2)|0);
  HEAP32[$50>>2] = $42;
  STACKTOP = sp;return;
 } else {
  $51 = HEAP32[$pToplevel>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  _sqlite3OomFault($52);
  STACKTOP = sp;return;
 }
}
function _sqlite3GenerateRowDelete($pParse,$pTab,$pTrigger,$iDataCur,$iIdxCur,$iPk,$nPk,$count,$onconf,$eMode,$iIdxNoSeek) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pTrigger = $pTrigger|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $iPk = $iPk|0;
 $nPk = $nPk|0;
 $count = $count|0;
 $onconf = $onconf|0;
 $eMode = $eMode|0;
 $iIdxNoSeek = $iIdxNoSeek|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addrStart = 0, $iCol = 0, $iLabel = 0, $iOld = 0, $mask = 0, $opSeek = 0, $or$cond = 0, $p5 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 52|0;
 $7 = sp + 58|0;
 $8 = sp + 57|0;
 $9 = sp + 56|0;
 $10 = sp + 24|0;
 $v = sp + 20|0;
 $iOld = sp + 16|0;
 $iLabel = sp + 12|0;
 $opSeek = sp + 55|0;
 $mask = sp + 8|0;
 $iCol = sp + 4|0;
 $addrStart = sp;
 $p5 = sp + 54|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pTrigger;
 HEAP32[$3>>2] = $iDataCur;
 HEAP32[$4>>2] = $iIdxCur;
 HEAP32[$5>>2] = $iPk;
 HEAP16[$6>>1] = $nPk;
 HEAP8[$7>>0] = $count;
 HEAP8[$8>>0] = $onconf;
 HEAP8[$9>>0] = $eMode;
 HEAP32[$10>>2] = $iIdxNoSeek;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 HEAP32[$iOld>>2] = 0;
 $14 = HEAP32[$v>>2]|0;
 $15 = (_sqlite3VdbeMakeLabel($14)|0);
 HEAP32[$iLabel>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 42|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = $19 & 32;
 $21 = ($20|0)==(0);
 $22 = $21 ? 70 : 68;
 $23 = $22&255;
 HEAP8[$opSeek>>0] = $23;
 $24 = HEAP8[$9>>0]|0;
 $25 = $24&255;
 $26 = ($25|0)==(0);
 if ($26) {
  $27 = HEAP32[$v>>2]|0;
  $28 = HEAP8[$opSeek>>0]|0;
  $29 = $28&255;
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$iLabel>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  $33 = HEAP16[$6>>1]|0;
  $34 = $33 << 16 >> 16;
  (_sqlite3VdbeAddOp4Int($27,$29,$30,$31,$32,$34)|0);
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = (_sqlite3FkRequired($35,$36,0,0)|0);
 $38 = ($37|0)!=(0);
 $39 = HEAP32[$2>>2]|0;
 $40 = ($39|0)!=(0|0);
 $or$cond = $38 | $40;
 if ($or$cond) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP8[$8>>0]|0;
  $45 = $44&255;
  $46 = (_sqlite3TriggerColmask($41,$42,0,0,3,$43,$45)|0);
  HEAP32[$mask>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = (_sqlite3FkOldmask($47,$48)|0);
  $50 = HEAP32[$mask>>2]|0;
  $51 = $50 | $49;
  HEAP32[$mask>>2] = $51;
  $52 = HEAP32[$0>>2]|0;
  $53 = ((($52)) + 76|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$iOld>>2] = $55;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 34|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58 << 16 >> 16;
  $60 = (1 + ($59))|0;
  $61 = HEAP32[$0>>2]|0;
  $62 = ((($61)) + 76|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + ($60))|0;
  HEAP32[$62>>2] = $64;
  $65 = HEAP32[$v>>2]|0;
  $66 = HEAP32[$5>>2]|0;
  $67 = HEAP32[$iOld>>2]|0;
  (_sqlite3VdbeAddOp2($65,30,$66,$67)|0);
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $68 = HEAP32[$iCol>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 34|0);
   $71 = HEAP16[$70>>1]|0;
   $72 = $71 << 16 >> 16;
   $73 = ($68|0)<($72|0);
   if (!($73)) {
    break;
   }
   $74 = HEAP32[$mask>>2]|0;
   $75 = ($74|0)==(-1);
   if ($75) {
    label = 9;
   } else {
    $76 = HEAP32[$iCol>>2]|0;
    $77 = ($76|0)<=(31);
    if ($77) {
     $78 = HEAP32[$mask>>2]|0;
     $79 = HEAP32[$iCol>>2]|0;
     $80 = 1 << $79;
     $81 = $78 & $80;
     $82 = ($81|0)!=(0);
     if ($82) {
      label = 9;
     }
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $83 = HEAP32[$v>>2]|0;
    $84 = HEAP32[$1>>2]|0;
    $85 = HEAP32[$3>>2]|0;
    $86 = HEAP32[$iCol>>2]|0;
    $87 = HEAP32[$iOld>>2]|0;
    $88 = HEAP32[$iCol>>2]|0;
    $89 = (($87) + ($88))|0;
    $90 = (($89) + 1)|0;
    _sqlite3ExprCodeGetColumnOfTable($83,$84,$85,$86,$90);
   }
   $91 = HEAP32[$iCol>>2]|0;
   $92 = (($91) + 1)|0;
   HEAP32[$iCol>>2] = $92;
  }
  $93 = HEAP32[$v>>2]|0;
  $94 = (_sqlite3VdbeCurrentAddr($93)|0);
  HEAP32[$addrStart>>2] = $94;
  $95 = HEAP32[$0>>2]|0;
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  $98 = HEAP32[$iOld>>2]|0;
  $99 = HEAP8[$8>>0]|0;
  $100 = $99&255;
  $101 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($95,$96,109,0,1,$97,$98,$100,$101);
  $102 = HEAP32[$addrStart>>2]|0;
  $103 = HEAP32[$v>>2]|0;
  $104 = (_sqlite3VdbeCurrentAddr($103)|0);
  $105 = ($102|0)<($104|0);
  if ($105) {
   $106 = HEAP32[$v>>2]|0;
   $107 = HEAP8[$opSeek>>0]|0;
   $108 = $107&255;
   $109 = HEAP32[$3>>2]|0;
   $110 = HEAP32[$iLabel>>2]|0;
   $111 = HEAP32[$5>>2]|0;
   $112 = HEAP16[$6>>1]|0;
   $113 = $112 << 16 >> 16;
   (_sqlite3VdbeAddOp4Int($106,$108,$109,$110,$111,$113)|0);
  }
  $114 = HEAP32[$0>>2]|0;
  $115 = HEAP32[$1>>2]|0;
  $116 = HEAP32[$iOld>>2]|0;
  _sqlite3FkCheck($114,$115,$116,0,0,0);
 }
 $117 = HEAP32[$1>>2]|0;
 $118 = ((($117)) + 12|0);
 $119 = HEAP32[$118>>2]|0;
 $120 = ($119|0)==(0|0);
 if (!($120)) {
  $154 = HEAP32[$0>>2]|0;
  $155 = HEAP32[$1>>2]|0;
  $156 = HEAP32[$iOld>>2]|0;
  _sqlite3FkActions($154,$155,0,$156,0,0);
  $157 = HEAP32[$0>>2]|0;
  $158 = HEAP32[$2>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  $160 = HEAP32[$iOld>>2]|0;
  $161 = HEAP8[$8>>0]|0;
  $162 = $161&255;
  $163 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($157,$158,109,0,2,$159,$160,$162,$163);
  $164 = HEAP32[$v>>2]|0;
  $165 = HEAP32[$iLabel>>2]|0;
  _sqlite3VdbeResolveLabel($164,$165);
  STACKTOP = sp;return;
 }
 HEAP8[$p5>>0] = 0;
 $121 = HEAP32[$0>>2]|0;
 $122 = HEAP32[$1>>2]|0;
 $123 = HEAP32[$3>>2]|0;
 $124 = HEAP32[$4>>2]|0;
 $125 = HEAP32[$10>>2]|0;
 _sqlite3GenerateRowIndexDelete($121,$122,$123,$124,0,$125);
 $126 = HEAP32[$v>>2]|0;
 $127 = HEAP32[$3>>2]|0;
 $128 = HEAP8[$7>>0]|0;
 $129 = $128&255;
 $130 = ($129|0)!=(0);
 $131 = $130 ? 1 : 0;
 (_sqlite3VdbeAddOp2($126,95,$127,$131)|0);
 $132 = HEAP8[$7>>0]|0;
 $133 = ($132<<24>>24)!=(0);
 if ($133) {
  $134 = HEAP32[$v>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  $136 = HEAP32[$135>>2]|0;
  _sqlite3VdbeChangeP4($134,-1,$136,0);
 }
 $137 = HEAP8[$9>>0]|0;
 $138 = $137&255;
 $139 = ($138|0)!=(0);
 if ($139) {
  $140 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($140,4);
 }
 $141 = HEAP32[$10>>2]|0;
 $142 = ($141|0)>=(0);
 if ($142) {
  $143 = HEAP32[$v>>2]|0;
  $144 = HEAP32[$10>>2]|0;
  (_sqlite3VdbeAddOp1($143,95,$144)|0);
 }
 $145 = HEAP8[$9>>0]|0;
 $146 = $145&255;
 $147 = ($146|0)==(2);
 if ($147) {
  $148 = HEAP8[$p5>>0]|0;
  $149 = $148&255;
  $150 = $149 | 2;
  $151 = $150&255;
  HEAP8[$p5>>0] = $151;
 }
 $152 = HEAP32[$v>>2]|0;
 $153 = HEAP8[$p5>>0]|0;
 _sqlite3VdbeChangeP5($152,$153);
 $154 = HEAP32[$0>>2]|0;
 $155 = HEAP32[$1>>2]|0;
 $156 = HEAP32[$iOld>>2]|0;
 _sqlite3FkActions($154,$155,0,$156,0,0);
 $157 = HEAP32[$0>>2]|0;
 $158 = HEAP32[$2>>2]|0;
 $159 = HEAP32[$1>>2]|0;
 $160 = HEAP32[$iOld>>2]|0;
 $161 = HEAP8[$8>>0]|0;
 $162 = $161&255;
 $163 = HEAP32[$iLabel>>2]|0;
 _sqlite3CodeRowTrigger($157,$158,109,0,2,$159,$160,$162,$163);
 $164 = HEAP32[$v>>2]|0;
 $165 = HEAP32[$iLabel>>2]|0;
 _sqlite3VdbeResolveLabel($164,$165);
 STACKTOP = sp;return;
}
function _sqlite3AutoincrementEnd($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 412|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _autoIncrementEnd($5);
 STACKTOP = sp;return;
}
function _sqlite3AuthContextPop($pContext) {
 $pContext = $pContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pContext;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 496|0);
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = 0;
 STACKTOP = sp;return;
}
function _checkColumnOverlap($pIdList,$pEList) {
 $pIdList = $pIdList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $e = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $e = sp;
 HEAP32[$1>>2] = $pIdList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 $or$cond = $4 | $6;
 if ($or$cond) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 HEAP32[$e>>2] = 0;
 while(1) {
  $7 = HEAP32[$e>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$e>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*20)|0)|0);
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3IdListIndex($11,$18)|0);
  $20 = ($19|0)>=(0);
  if ($20) {
   label = 6;
   break;
  }
  $21 = HEAP32[$e>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$e>>2] = $22;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _fkChildIsModified($pTab,$p,$aChange,$bChngRowid) {
 $pTab = $pTab|0;
 $p = $p|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iChildKey = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $i = sp + 4|0;
 $iChildKey = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $bChngRowid;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 36|0);
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$iChildKey>>2] = $14;
  $15 = HEAP32[$iChildKey>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)>=(0);
  if ($19) {
   label = 4;
   break;
  }
  $20 = HEAP32[$iChildKey>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 32|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23 << 16 >> 16;
  $25 = ($20|0)==($24|0);
  $26 = HEAP32[$4>>2]|0;
  $27 = ($26|0)!=(0);
  $or$cond = $25 & $27;
  if ($or$cond) {
   label = 6;
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$i>>2] = $29;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _fkParentIsModified($pTab,$p,$aChange,$bChngRowid) {
 $pTab = $pTab|0;
 $p = $p|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iKey = 0, $or$cond = 0;
 var $pCol = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $zKey = sp + 8|0;
 $iKey = sp + 4|0;
 $pCol = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $bChngRowid;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 14;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 36|0);
  $13 = (($12) + ($10<<3)|0);
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$zKey>>2] = $15;
  HEAP32[$iKey>>2] = 0;
  while(1) {
   $16 = HEAP32[$iKey>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 34|0);
   $19 = HEAP16[$18>>1]|0;
   $20 = $19 << 16 >> 16;
   $21 = ($16|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$iKey>>2]|0;
   $23 = HEAP32[$3>>2]|0;
   $24 = (($23) + ($22<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)>=(0);
   if ($26) {
    label = 7;
   } else {
    $27 = HEAP32[$iKey>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 32|0);
    $30 = HEAP16[$29>>1]|0;
    $31 = $30 << 16 >> 16;
    $32 = ($27|0)==($31|0);
    $33 = HEAP32[$4>>2]|0;
    $34 = ($33|0)!=(0);
    $or$cond = $32 & $34;
    if ($or$cond) {
     label = 7;
    }
   }
   do {
    if ((label|0) == 7) {
     label = 0;
     $35 = HEAP32[$iKey>>2]|0;
     $36 = HEAP32[$1>>2]|0;
     $37 = ((($36)) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = (($38) + (($35*24)|0)|0);
     HEAP32[$pCol>>2] = $39;
     $40 = HEAP32[$zKey>>2]|0;
     $41 = ($40|0)!=(0|0);
     $42 = HEAP32[$pCol>>2]|0;
     if ($41) {
      $43 = HEAP32[$42>>2]|0;
      $44 = HEAP32[$zKey>>2]|0;
      $45 = (_sqlite3_stricmp($43,$44)|0);
      $46 = (0)==($45|0);
      if ($46) {
       label = 9;
       break L1;
      } else {
       break;
      }
     } else {
      $47 = ((($42)) + 23|0);
      $48 = HEAP8[$47>>0]|0;
      $49 = $48&255;
      $50 = $49 & 1;
      $51 = ($50|0)!=(0);
      if ($51) {
       label = 11;
       break L1;
      } else {
       break;
      }
     }
    }
   } while(0);
   $52 = HEAP32[$iKey>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$iKey>>2] = $53;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 9) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return (0)|0;
}
function _sqlite3TriggerColmask($pParse,$pTrigger,$pChanges,$isNew,$tr_tm,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pChanges = $pChanges|0;
 $isNew = $isNew|0;
 $tr_tm = $tr_tm|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mask = 0, $op = 0, $p = 0, $pPrg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $op = sp + 12|0;
 $mask = sp + 8|0;
 $p = sp + 4|0;
 $pPrg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $isNew;
 HEAP32[$4>>2] = $tr_tm;
 HEAP32[$5>>2] = $pTab;
 HEAP32[$6>>2] = $orconf;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8 ? 110 : 109;
 HEAP32[$op>>2] = $9;
 HEAP32[$mask>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 8|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = HEAP32[$op>>2]|0;
  $18 = ($16|0)==($17|0);
  if ($18) {
   $19 = HEAP32[$4>>2]|0;
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 9|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $19 & $23;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = HEAP32[$p>>2]|0;
    $27 = ((($26)) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$2>>2]|0;
    $30 = (_checkColumnOverlap($28,$29)|0);
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = HEAP32[$0>>2]|0;
     $33 = HEAP32[$p>>2]|0;
     $34 = HEAP32[$5>>2]|0;
     $35 = HEAP32[$6>>2]|0;
     $36 = (_getRowTrigger($32,$33,$34,$35)|0);
     HEAP32[$pPrg>>2] = $36;
     $37 = HEAP32[$pPrg>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$3>>2]|0;
      $40 = HEAP32[$pPrg>>2]|0;
      $41 = ((($40)) + 16|0);
      $42 = (($41) + ($39<<2)|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = HEAP32[$mask>>2]|0;
      $45 = $44 | $43;
      HEAP32[$mask>>2] = $45;
     }
    }
   }
  }
  $46 = HEAP32[$p>>2]|0;
  $47 = ((($46)) + 32|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$p>>2] = $48;
 }
 $49 = HEAP32[$mask>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3FkOldmask($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $mask = 0;
 var $p = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $mask = sp + 12|0;
 $p = sp + 8|0;
 $i = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$mask>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 524288;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $77 = HEAP32[$mask>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   break;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $13 = HEAP32[$i>>2]|0;
   $14 = HEAP32[$p>>2]|0;
   $15 = ((($14)) + 20|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($13|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 36|0);
   $21 = (($20) + ($18<<3)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)>(31);
   if ($23) {
    $32 = -1;
   } else {
    $24 = HEAP32[$i>>2]|0;
    $25 = HEAP32[$p>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = (($26) + ($24<<3)|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = 1 << $28;
    $32 = $29;
   }
   $30 = HEAP32[$mask>>2]|0;
   $31 = $30 | $32;
   HEAP32[$mask>>2] = $31;
   $33 = HEAP32[$i>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$i>>2] = $34;
  }
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$p>>2] = $37;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = (_sqlite3FkReferences($38)|0);
 HEAP32[$p>>2] = $39;
 while(1) {
  $40 = HEAP32[$p>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$p>>2]|0;
  (_sqlite3FkLocateIndex($42,$43,$44,$pIdx,0)|0);
  $45 = HEAP32[$pIdx>>2]|0;
  $46 = ($45|0)!=(0|0);
  L18: do {
   if ($46) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $47 = HEAP32[$i>>2]|0;
     $48 = HEAP32[$pIdx>>2]|0;
     $49 = ((($48)) + 50|0);
     $50 = HEAP16[$49>>1]|0;
     $51 = $50&65535;
     $52 = ($47|0)<($51|0);
     if (!($52)) {
      break L18;
     }
     $53 = HEAP32[$i>>2]|0;
     $54 = HEAP32[$pIdx>>2]|0;
     $55 = ((($54)) + 4|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (($56) + ($53<<1)|0);
     $58 = HEAP16[$57>>1]|0;
     $59 = $58 << 16 >> 16;
     $60 = ($59|0)>(31);
     if ($60) {
      $71 = -1;
     } else {
      $61 = HEAP32[$i>>2]|0;
      $62 = HEAP32[$pIdx>>2]|0;
      $63 = ((($62)) + 4|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (($64) + ($61<<1)|0);
      $66 = HEAP16[$65>>1]|0;
      $67 = $66 << 16 >> 16;
      $68 = 1 << $67;
      $71 = $68;
     }
     $69 = HEAP32[$mask>>2]|0;
     $70 = $69 | $71;
     HEAP32[$mask>>2] = $70;
     $72 = HEAP32[$i>>2]|0;
     $73 = (($72) + 1)|0;
     HEAP32[$i>>2] = $73;
    }
   }
  } while(0);
  $74 = HEAP32[$p>>2]|0;
  $75 = ((($74)) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  HEAP32[$p>>2] = $76;
 }
 $77 = HEAP32[$mask>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _sqlite3CodeRowTrigger($pParse,$pTrigger,$op,$pChanges,$tr_tm,$pTab,$reg,$orconf,$ignoreJump) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $op = $op|0;
 $pChanges = $pChanges|0;
 $tr_tm = $tr_tm|0;
 $pTab = $pTab|0;
 $reg = $reg|0;
 $orconf = $orconf|0;
 $ignoreJump = $ignoreJump|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 12|0;
 $7 = sp + 8|0;
 $8 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $tr_tm;
 HEAP32[$5>>2] = $pTab;
 HEAP32[$6>>2] = $reg;
 HEAP32[$7>>2] = $orconf;
 HEAP32[$8>>2] = $ignoreJump;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $9;
 while(1) {
  $10 = HEAP32[$p>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   $19 = ((($18)) + 9|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = HEAP32[$4>>2]|0;
   $23 = ($21|0)==($22|0);
   if ($23) {
    $24 = HEAP32[$p>>2]|0;
    $25 = ((($24)) + 16|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = HEAP32[$3>>2]|0;
    $28 = (_checkColumnOverlap($26,$27)|0);
    $29 = ($28|0)!=(0);
    if ($29) {
     $30 = HEAP32[$0>>2]|0;
     $31 = HEAP32[$p>>2]|0;
     $32 = HEAP32[$5>>2]|0;
     $33 = HEAP32[$6>>2]|0;
     $34 = HEAP32[$7>>2]|0;
     $35 = HEAP32[$8>>2]|0;
     _sqlite3CodeRowTriggerDirect($30,$31,$32,$33,$34,$35);
    }
   }
  }
  $36 = HEAP32[$p>>2]|0;
  $37 = ((($36)) + 32|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$p>>2] = $38;
 }
 STACKTOP = sp;return;
}
function _sqlite3FkCheck($pParse,$pTab,$regOld,$regNew,$aChange,$bChngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $regOld = $regOld|0;
 $regNew = $regNew|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aiCol = 0, $aiCol2 = 0, $aiFree = 0, $bIgnore = 0, $db = 0, $eAction = 0, $i = 0, $iCol = 0, $iDb = 0, $iJump = 0, $iReg = 0, $isIgnoreErrors = 0, $or$cond = 0, $or$cond3 = 0, $pFKey = 0, $pIdx = 0, $pIdx1 = 0, $pItem = 0, $pSrc = 0, $pTo = 0;
 var $rcauth = 0, $v = 0, $zCol = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $db = sp + 84|0;
 $pFKey = sp + 80|0;
 $iDb = sp + 76|0;
 $zDb = sp + 72|0;
 $isIgnoreErrors = sp + 68|0;
 $pTo = sp + 64|0;
 $pIdx = sp + 60|0;
 $aiFree = sp + 56|0;
 $aiCol = sp + 52|0;
 $iCol = sp + 48|0;
 $i = sp + 44|0;
 $bIgnore = sp + 40|0;
 $v = sp + 36|0;
 $iJump = sp + 32|0;
 $iReg = sp + 28|0;
 $rcauth = sp + 24|0;
 $zCol = sp + 20|0;
 $pIdx1 = sp + 16|0;
 $pSrc = sp + 12|0;
 $aiCol2 = sp + 8|0;
 $pItem = sp + 4|0;
 $eAction = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $regOld;
 HEAP32[$3>>2] = $regNew;
 HEAP32[$4>>2] = $aChange;
 HEAP32[$5>>2] = $bChngRowid;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 442|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$isIgnoreErrors>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 & 524288;
 $16 = ($15|0)==(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 64|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3SchemaToIndex($17,$20)|0);
 HEAP32[$iDb>>2] = $21;
 $22 = HEAP32[$iDb>>2]|0;
 $23 = HEAP32[$db>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<4)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$zDb>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$pFKey>>2] = $30;
 L4: while(1) {
  $31 = HEAP32[$pFKey>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   label = 39;
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  HEAP32[$aiFree>>2] = 0;
  HEAP32[$bIgnore>>2] = 0;
  $33 = HEAP32[$4>>2]|0;
  $34 = ($33|0)!=(0|0);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pFKey>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3_stricmp($36,$39)|0);
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP32[$pFKey>>2]|0;
    $44 = HEAP32[$4>>2]|0;
    $45 = HEAP32[$5>>2]|0;
    $46 = (_fkChildIsModified($42,$43,$44,$45)|0);
    $47 = ($46|0)==(0);
    if (!($47)) {
     label = 7;
    }
   } else {
    label = 7;
   }
  } else {
   label = 7;
  }
  do {
   if ((label|0) == 7) {
    label = 0;
    $48 = HEAP32[$0>>2]|0;
    $49 = ((($48)) + 442|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = ($50<<24>>24)!=(0);
    if ($51) {
     $52 = HEAP32[$db>>2]|0;
     $53 = HEAP32[$pFKey>>2]|0;
     $54 = ((($53)) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = HEAP32[$zDb>>2]|0;
     $57 = (_sqlite3FindTable($52,$55,$56)|0);
     HEAP32[$pTo>>2] = $57;
    } else {
     $58 = HEAP32[$0>>2]|0;
     $59 = HEAP32[$pFKey>>2]|0;
     $60 = ((($59)) + 8|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = HEAP32[$zDb>>2]|0;
     $63 = (_sqlite3LocateTable($58,0,$61,$62)|0);
     HEAP32[$pTo>>2] = $63;
    }
    $64 = HEAP32[$pTo>>2]|0;
    $65 = ($64|0)!=(0|0);
    if ($65) {
     $66 = HEAP32[$0>>2]|0;
     $67 = HEAP32[$pTo>>2]|0;
     $68 = HEAP32[$pFKey>>2]|0;
     $69 = (_sqlite3FkLocateIndex($66,$67,$68,$pIdx,$aiFree)|0);
     $70 = ($69|0)!=(0);
     if (!($70)) {
      $112 = HEAP32[$aiFree>>2]|0;
      $113 = ($112|0)!=(0|0);
      if ($113) {
       $114 = HEAP32[$aiFree>>2]|0;
       HEAP32[$aiCol>>2] = $114;
      } else {
       $115 = HEAP32[$pFKey>>2]|0;
       $116 = ((($115)) + 36|0);
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$iCol>>2] = $117;
       HEAP32[$aiCol>>2] = $iCol;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $118 = HEAP32[$i>>2]|0;
       $119 = HEAP32[$pFKey>>2]|0;
       $120 = ((($119)) + 20|0);
       $121 = HEAP32[$120>>2]|0;
       $122 = ($118|0)<($121|0);
       if (!($122)) {
        break;
       }
       $123 = HEAP32[$i>>2]|0;
       $124 = HEAP32[$aiCol>>2]|0;
       $125 = (($124) + ($123<<2)|0);
       $126 = HEAP32[$125>>2]|0;
       $127 = HEAP32[$1>>2]|0;
       $128 = ((($127)) + 32|0);
       $129 = HEAP16[$128>>1]|0;
       $130 = $129 << 16 >> 16;
       $131 = ($126|0)==($130|0);
       if ($131) {
        $132 = HEAP32[$i>>2]|0;
        $133 = HEAP32[$aiCol>>2]|0;
        $134 = (($133) + ($132<<2)|0);
        HEAP32[$134>>2] = -1;
       }
       $135 = HEAP32[$db>>2]|0;
       $136 = ((($135)) + 288|0);
       $137 = HEAP32[$136>>2]|0;
       $138 = ($137|0)!=(0|0);
       if ($138) {
        $139 = HEAP32[$pIdx>>2]|0;
        $140 = ($139|0)!=(0|0);
        if ($140) {
         $141 = HEAP32[$i>>2]|0;
         $142 = HEAP32[$pIdx>>2]|0;
         $143 = ((($142)) + 4|0);
         $144 = HEAP32[$143>>2]|0;
         $145 = (($144) + ($141<<1)|0);
         $146 = HEAP16[$145>>1]|0;
         $$sink = $146;
        } else {
         $147 = HEAP32[$pTo>>2]|0;
         $148 = ((($147)) + 32|0);
         $149 = HEAP16[$148>>1]|0;
         $$sink = $149;
        }
        $150 = $$sink << 16 >> 16;
        $151 = HEAP32[$pTo>>2]|0;
        $152 = ((($151)) + 4|0);
        $153 = HEAP32[$152>>2]|0;
        $154 = (($153) + (($150*24)|0)|0);
        $155 = HEAP32[$154>>2]|0;
        HEAP32[$zCol>>2] = $155;
        $156 = HEAP32[$0>>2]|0;
        $157 = HEAP32[$pTo>>2]|0;
        $158 = HEAP32[$157>>2]|0;
        $159 = HEAP32[$zCol>>2]|0;
        $160 = HEAP32[$iDb>>2]|0;
        $161 = (_sqlite3AuthReadCol($156,$158,$159,$160)|0);
        HEAP32[$rcauth>>2] = $161;
        $162 = HEAP32[$rcauth>>2]|0;
        $163 = ($162|0)==(2);
        $164 = $163&1;
        HEAP32[$bIgnore>>2] = $164;
       }
       $165 = HEAP32[$i>>2]|0;
       $166 = (($165) + 1)|0;
       HEAP32[$i>>2] = $166;
      }
      $167 = HEAP32[$0>>2]|0;
      $168 = HEAP32[$iDb>>2]|0;
      $169 = HEAP32[$pTo>>2]|0;
      $170 = ((($169)) + 28|0);
      $171 = HEAP32[$170>>2]|0;
      $172 = HEAP32[$pTo>>2]|0;
      $173 = HEAP32[$172>>2]|0;
      _sqlite3TableLock($167,$168,$171,0,$173);
      $174 = HEAP32[$0>>2]|0;
      $175 = ((($174)) + 72|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = (($176) + 1)|0;
      HEAP32[$175>>2] = $177;
      $178 = HEAP32[$2>>2]|0;
      $179 = ($178|0)!=(0);
      if ($179) {
       $180 = HEAP32[$0>>2]|0;
       $181 = HEAP32[$iDb>>2]|0;
       $182 = HEAP32[$pTo>>2]|0;
       $183 = HEAP32[$pIdx>>2]|0;
       $184 = HEAP32[$pFKey>>2]|0;
       $185 = HEAP32[$aiCol>>2]|0;
       $186 = HEAP32[$2>>2]|0;
       $187 = HEAP32[$bIgnore>>2]|0;
       _fkLookupParent($180,$181,$182,$183,$184,$185,$186,-1,$187);
      }
      $188 = HEAP32[$3>>2]|0;
      $189 = ($188|0)!=(0);
      if ($189) {
       $190 = HEAP32[$0>>2]|0;
       $191 = HEAP32[$pFKey>>2]|0;
       $192 = (_isSetNullAction($190,$191)|0);
       $193 = ($192|0)!=(0);
       if (!($193)) {
        $194 = HEAP32[$0>>2]|0;
        $195 = HEAP32[$iDb>>2]|0;
        $196 = HEAP32[$pTo>>2]|0;
        $197 = HEAP32[$pIdx>>2]|0;
        $198 = HEAP32[$pFKey>>2]|0;
        $199 = HEAP32[$aiCol>>2]|0;
        $200 = HEAP32[$3>>2]|0;
        $201 = HEAP32[$bIgnore>>2]|0;
        _fkLookupParent($194,$195,$196,$197,$198,$199,$200,1,$201);
       }
      }
      $202 = HEAP32[$db>>2]|0;
      $203 = HEAP32[$aiFree>>2]|0;
      _sqlite3DbFree($202,$203);
      break;
     }
    }
    $71 = HEAP32[$isIgnoreErrors>>2]|0;
    $72 = ($71|0)!=(0);
    if (!($72)) {
     label = 59;
     break L4;
    }
    $73 = HEAP32[$db>>2]|0;
    $74 = ((($73)) + 65|0);
    $75 = HEAP8[$74>>0]|0;
    $76 = $75&255;
    $77 = ($76|0)!=(0);
    if ($77) {
     label = 59;
     break L4;
    }
    $78 = HEAP32[$pTo>>2]|0;
    $79 = ($78|0)==(0|0);
    if ($79) {
     $80 = HEAP32[$0>>2]|0;
     $81 = (_sqlite3GetVdbe($80)|0);
     HEAP32[$v>>2] = $81;
     $82 = HEAP32[$v>>2]|0;
     $83 = (_sqlite3VdbeCurrentAddr($82)|0);
     $84 = HEAP32[$pFKey>>2]|0;
     $85 = ((($84)) + 20|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = (($83) + ($86))|0;
     $88 = (($87) + 1)|0;
     HEAP32[$iJump>>2] = $88;
     HEAP32[$i>>2] = 0;
     while(1) {
      $89 = HEAP32[$i>>2]|0;
      $90 = HEAP32[$pFKey>>2]|0;
      $91 = ((($90)) + 20|0);
      $92 = HEAP32[$91>>2]|0;
      $93 = ($89|0)<($92|0);
      if (!($93)) {
       break;
      }
      $94 = HEAP32[$i>>2]|0;
      $95 = HEAP32[$pFKey>>2]|0;
      $96 = ((($95)) + 36|0);
      $97 = (($96) + ($94<<3)|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = HEAP32[$2>>2]|0;
      $100 = (($98) + ($99))|0;
      $101 = (($100) + 1)|0;
      HEAP32[$iReg>>2] = $101;
      $102 = HEAP32[$v>>2]|0;
      $103 = HEAP32[$iReg>>2]|0;
      $104 = HEAP32[$iJump>>2]|0;
      (_sqlite3VdbeAddOp2($102,76,$103,$104)|0);
      $105 = HEAP32[$i>>2]|0;
      $106 = (($105) + 1)|0;
      HEAP32[$i>>2] = $106;
     }
     $107 = HEAP32[$v>>2]|0;
     $108 = HEAP32[$pFKey>>2]|0;
     $109 = ((($108)) + 24|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     (_sqlite3VdbeAddOp2($107,135,$111,-1)|0);
    }
   }
  } while(0);
  $204 = HEAP32[$pFKey>>2]|0;
  $205 = ((($204)) + 4|0);
  $206 = HEAP32[$205>>2]|0;
  HEAP32[$pFKey>>2] = $206;
 }
 if ((label|0) == 39) {
  $207 = HEAP32[$1>>2]|0;
  $208 = (_sqlite3FkReferences($207)|0);
  HEAP32[$pFKey>>2] = $208;
  L55: while(1) {
   $209 = HEAP32[$pFKey>>2]|0;
   $210 = ($209|0)!=(0|0);
   if (!($210)) {
    label = 59;
    break;
   }
   HEAP32[$pIdx1>>2] = 0;
   HEAP32[$aiCol2>>2] = 0;
   $211 = HEAP32[$4>>2]|0;
   $212 = ($211|0)!=(0|0);
   if ($212) {
    $213 = HEAP32[$1>>2]|0;
    $214 = HEAP32[$pFKey>>2]|0;
    $215 = HEAP32[$4>>2]|0;
    $216 = HEAP32[$5>>2]|0;
    $217 = (_fkParentIsModified($213,$214,$215,$216)|0);
    $218 = ($217|0)==(0);
    if (!($218)) {
     label = 43;
    }
   } else {
    label = 43;
   }
   do {
    if ((label|0) == 43) {
     label = 0;
     $219 = HEAP32[$pFKey>>2]|0;
     $220 = ((($219)) + 24|0);
     $221 = HEAP8[$220>>0]|0;
     $222 = ($221<<24>>24)!=(0);
     if (!($222)) {
      $223 = HEAP32[$db>>2]|0;
      $224 = ((($223)) + 24|0);
      $225 = HEAP32[$224>>2]|0;
      $226 = $225 & 16777216;
      $227 = ($226|0)!=(0);
      if (!($227)) {
       $228 = HEAP32[$0>>2]|0;
       $229 = ((($228)) + 416|0);
       $230 = HEAP32[$229>>2]|0;
       $231 = ($230|0)!=(0|0);
       if (!($231)) {
        $232 = HEAP32[$0>>2]|0;
        $233 = ((($232)) + 20|0);
        $234 = HEAP8[$233>>0]|0;
        $235 = ($234<<24>>24)!=(0);
        if (!($235)) {
         break;
        }
       }
      }
     }
     $236 = HEAP32[$0>>2]|0;
     $237 = HEAP32[$1>>2]|0;
     $238 = HEAP32[$pFKey>>2]|0;
     $239 = (_sqlite3FkLocateIndex($236,$237,$238,$pIdx1,$aiCol2)|0);
     $240 = ($239|0)!=(0);
     if ($240) {
      $241 = HEAP32[$isIgnoreErrors>>2]|0;
      $242 = ($241|0)!=(0);
      if (!($242)) {
       label = 59;
       break L55;
      }
      $243 = HEAP32[$db>>2]|0;
      $244 = ((($243)) + 65|0);
      $245 = HEAP8[$244>>0]|0;
      $246 = $245&255;
      $247 = ($246|0)!=(0);
      if ($247) {
       label = 59;
       break L55;
      } else {
       break;
      }
     }
     $248 = HEAP32[$db>>2]|0;
     $249 = (_sqlite3SrcListAppend($248,0,0,0)|0);
     HEAP32[$pSrc>>2] = $249;
     $250 = HEAP32[$pSrc>>2]|0;
     $251 = ($250|0)!=(0|0);
     if ($251) {
      $252 = HEAP32[$pSrc>>2]|0;
      $253 = ((($252)) + 8|0);
      HEAP32[$pItem>>2] = $253;
      $254 = HEAP32[$pFKey>>2]|0;
      $255 = HEAP32[$254>>2]|0;
      $256 = HEAP32[$pItem>>2]|0;
      $257 = ((($256)) + 16|0);
      HEAP32[$257>>2] = $255;
      $258 = HEAP32[$pFKey>>2]|0;
      $259 = HEAP32[$258>>2]|0;
      $260 = HEAP32[$259>>2]|0;
      $261 = HEAP32[$pItem>>2]|0;
      $262 = ((($261)) + 8|0);
      HEAP32[$262>>2] = $260;
      $263 = HEAP32[$pItem>>2]|0;
      $264 = ((($263)) + 16|0);
      $265 = HEAP32[$264>>2]|0;
      $266 = ((($265)) + 36|0);
      $267 = HEAP16[$266>>1]|0;
      $268 = (($267) + 1)<<16>>16;
      HEAP16[$266>>1] = $268;
      $269 = HEAP32[$0>>2]|0;
      $270 = ((($269)) + 72|0);
      $271 = HEAP32[$270>>2]|0;
      $272 = (($271) + 1)|0;
      HEAP32[$270>>2] = $272;
      $273 = HEAP32[$pItem>>2]|0;
      $274 = ((($273)) + 44|0);
      HEAP32[$274>>2] = $271;
      $275 = HEAP32[$3>>2]|0;
      $276 = ($275|0)!=(0);
      if ($276) {
       $277 = HEAP32[$0>>2]|0;
       $278 = HEAP32[$pSrc>>2]|0;
       $279 = HEAP32[$1>>2]|0;
       $280 = HEAP32[$pIdx1>>2]|0;
       $281 = HEAP32[$pFKey>>2]|0;
       $282 = HEAP32[$aiCol2>>2]|0;
       $283 = HEAP32[$3>>2]|0;
       _fkScanChildren($277,$278,$279,$280,$281,$282,$283,-1);
      }
      $284 = HEAP32[$2>>2]|0;
      $285 = ($284|0)!=(0);
      if ($285) {
       $286 = HEAP32[$4>>2]|0;
       $287 = ($286|0)!=(0|0);
       $288 = $287&1;
       $289 = HEAP32[$pFKey>>2]|0;
       $290 = ((($289)) + 25|0);
       $291 = (($290) + ($288)|0);
       $292 = HEAP8[$291>>0]|0;
       $293 = $292&255;
       HEAP32[$eAction>>2] = $293;
       $294 = HEAP32[$0>>2]|0;
       $295 = HEAP32[$pSrc>>2]|0;
       $296 = HEAP32[$1>>2]|0;
       $297 = HEAP32[$pIdx1>>2]|0;
       $298 = HEAP32[$pFKey>>2]|0;
       $299 = HEAP32[$aiCol2>>2]|0;
       $300 = HEAP32[$2>>2]|0;
       _fkScanChildren($294,$295,$296,$297,$298,$299,$300,1);
       $301 = HEAP32[$pFKey>>2]|0;
       $302 = ((($301)) + 24|0);
       $303 = HEAP8[$302>>0]|0;
       $304 = ($303<<24>>24)==(0);
       $305 = HEAP32[$eAction>>2]|0;
       $306 = ($305|0)!=(9);
       $or$cond = $304 & $306;
       $307 = HEAP32[$eAction>>2]|0;
       $308 = ($307|0)!=(7);
       $or$cond3 = $or$cond & $308;
       if ($or$cond3) {
        $309 = HEAP32[$0>>2]|0;
        _sqlite3MayAbort($309);
       }
      }
      $310 = HEAP32[$pItem>>2]|0;
      $311 = ((($310)) + 8|0);
      HEAP32[$311>>2] = 0;
      $312 = HEAP32[$db>>2]|0;
      $313 = HEAP32[$pSrc>>2]|0;
      _sqlite3SrcListDelete($312,$313);
     }
     $314 = HEAP32[$db>>2]|0;
     $315 = HEAP32[$aiCol2>>2]|0;
     _sqlite3DbFree($314,$315);
    }
   } while(0);
   $316 = HEAP32[$pFKey>>2]|0;
   $317 = ((($316)) + 12|0);
   $318 = HEAP32[$317>>2]|0;
   HEAP32[$pFKey>>2] = $318;
  }
  if ((label|0) == 59) {
   STACKTOP = sp;return;
  }
 }
 else if ((label|0) == 59) {
  STACKTOP = sp;return;
 }
}
function _sqlite3GenerateRowIndexDelete($pParse,$pTab,$iDataCur,$iIdxCur,$aRegIdx,$iIdxNoSeek) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $aRegIdx = $aRegIdx|0;
 $iIdxNoSeek = $iIdxNoSeek|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iPartIdxLabel = 0, $pIdx = 0, $pPk = 0, $pPrior = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $i = sp + 24|0;
 $r1 = sp + 20|0;
 $iPartIdxLabel = sp + 16|0;
 $pIdx = sp + 12|0;
 $pPrior = sp + 8|0;
 $v = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $iIdxCur;
 HEAP32[$4>>2] = $aRegIdx;
 HEAP32[$5>>2] = $iIdxNoSeek;
 HEAP32[$r1>>2] = -1;
 HEAP32[$pPrior>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 32;
 $14 = ($13|0)==(0);
 if ($14) {
  $17 = 0;
 } else {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3PrimaryKeyIndex($15)|0);
  $17 = $16;
 }
 HEAP32[$pPk>>2] = $17;
 HEAP32[$i>>2] = 0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pIdx>>2] = $20;
 while(1) {
  $21 = HEAP32[$pIdx>>2]|0;
  $22 = ($21|0)!=(0|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$4>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$i>>2]|0;
   $26 = HEAP32[$4>>2]|0;
   $27 = (($26) + ($25<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)==(0);
   if (!($29)) {
    label = 7;
   }
  } else {
   label = 7;
  }
  if ((label|0) == 7) {
   label = 0;
   $30 = HEAP32[$pIdx>>2]|0;
   $31 = HEAP32[$pPk>>2]|0;
   $32 = ($30|0)==($31|0);
   if (!($32)) {
    $33 = HEAP32[$3>>2]|0;
    $34 = HEAP32[$i>>2]|0;
    $35 = (($33) + ($34))|0;
    $36 = HEAP32[$5>>2]|0;
    $37 = ($35|0)==($36|0);
    if (!($37)) {
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$pIdx>>2]|0;
     $40 = HEAP32[$2>>2]|0;
     $41 = HEAP32[$pPrior>>2]|0;
     $42 = HEAP32[$r1>>2]|0;
     $43 = (_sqlite3GenerateIndexKey($38,$39,$40,0,1,$iPartIdxLabel,$41,$42)|0);
     HEAP32[$r1>>2] = $43;
     $44 = HEAP32[$v>>2]|0;
     $45 = HEAP32[$3>>2]|0;
     $46 = HEAP32[$i>>2]|0;
     $47 = (($45) + ($46))|0;
     $48 = HEAP32[$r1>>2]|0;
     $49 = HEAP32[$pIdx>>2]|0;
     $50 = ((($49)) + 55|0);
     $51 = HEAP8[$50>>0]|0;
     $52 = ($51&255) >>> 3;
     $53 = $52 & 1;
     $54 = $53&255;
     $55 = ($54|0)!=(0);
     $56 = HEAP32[$pIdx>>2]|0;
     if ($55) {
      $57 = ((($56)) + 50|0);
      $58 = HEAP16[$57>>1]|0;
      $$sink = $58;
     } else {
      $59 = ((($56)) + 52|0);
      $60 = HEAP16[$59>>1]|0;
      $$sink = $60;
     }
     $61 = $$sink&65535;
     (_sqlite3VdbeAddOp3($44,111,$47,$48,$61)|0);
     $62 = HEAP32[$0>>2]|0;
     $63 = HEAP32[$iPartIdxLabel>>2]|0;
     _sqlite3ResolvePartIdxLabel($62,$63);
     $64 = HEAP32[$pIdx>>2]|0;
     HEAP32[$pPrior>>2] = $64;
    }
   }
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = (($65) + 1)|0;
  HEAP32[$i>>2] = $66;
  $67 = HEAP32[$pIdx>>2]|0;
  $68 = ((($67)) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$pIdx>>2] = $69;
 }
 STACKTOP = sp;return;
}
function _sqlite3FkActions($pParse,$pTab,$pChanges,$regOld,$aChange,$bChngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pChanges = $pChanges|0;
 $regOld = $regOld|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAct = 0, $pFKey = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pFKey = sp + 4|0;
 $pAct = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $regOld;
 HEAP32[$4>>2] = $aChange;
 HEAP32[$5>>2] = $bChngRowid;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 524288;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (_sqlite3FkReferences($12)|0);
 HEAP32[$pFKey>>2] = $13;
 while(1) {
  $14 = HEAP32[$pFKey>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$4>>2]|0;
  $17 = ($16|0)==(0|0);
  if ($17) {
   label = 6;
  } else {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$pFKey>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = HEAP32[$5>>2]|0;
   $22 = (_fkParentIsModified($18,$19,$20,$21)|0);
   $23 = ($22|0)!=(0);
   if ($23) {
    label = 6;
   }
  }
  if ((label|0) == 6) {
   label = 0;
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$pFKey>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = (_fkActionTrigger($24,$25,$26,$27)|0);
   HEAP32[$pAct>>2] = $28;
   $29 = HEAP32[$pAct>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    $31 = HEAP32[$0>>2]|0;
    $32 = HEAP32[$pAct>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$3>>2]|0;
    _sqlite3CodeRowTriggerDirect($31,$32,$33,$34,2,0);
   }
  }
  $35 = HEAP32[$pFKey>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$pFKey>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _getRowTrigger($pParse,$pTrigger,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPrg = 0, $pRoot = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pRoot = sp + 4|0;
 $pPrg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $orconf;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 416|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  $9 = ((($8)) + 416|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10;
 } else {
  $11 = $8;
 }
 HEAP32[$pRoot>>2] = $11;
 $12 = HEAP32[$pRoot>>2]|0;
 $13 = ((($12)) + 532|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pPrg>>2] = $14;
 while(1) {
  $15 = HEAP32[$pPrg>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pPrg>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ($18|0)!=($19|0);
  if (!($20)) {
   $21 = HEAP32[$pPrg>>2]|0;
   $22 = ((($21)) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$3>>2]|0;
   $25 = ($23|0)!=($24|0);
   if (!($25)) {
    break;
   }
  }
  $26 = HEAP32[$pPrg>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pPrg>>2] = $28;
 }
 $29 = HEAP32[$pPrg>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $36 = HEAP32[$pPrg>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = (_codeRowTrigger($31,$32,$33,$34)|0);
 HEAP32[$pPrg>>2] = $35;
 $36 = HEAP32[$pPrg>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _codeRowTrigger($pParse,$pTrigger,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iEndTrigger = 0, $pPrg = 0, $pProgram = 0, $pSubParse = 0, $pTop = 0, $pWhen = 0, $sNC = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $pTop = sp + 64|0;
 $db = sp + 60|0;
 $pPrg = sp + 56|0;
 $pWhen = sp + 52|0;
 $v = sp + 48|0;
 $sNC = sp + 16|0;
 $pProgram = sp + 12|0;
 $pSubParse = sp + 8|0;
 $iEndTrigger = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTrigger;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$4>>2] = $orconf;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 416|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 if ($8) {
  $10 = ((($9)) + 416|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = $9;
 }
 HEAP32[$pTop>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 HEAP32[$pWhen>>2] = 0;
 HEAP32[$pProgram>>2] = 0;
 HEAP32[$iEndTrigger>>2] = 0;
 $15 = HEAP32[$db>>2]|0;
 $16 = (_sqlite3DbMallocZero($15,24,0)|0);
 HEAP32[$pPrg>>2] = $16;
 $17 = HEAP32[$pPrg>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  HEAP32[$0>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($165|0);
 }
 $19 = HEAP32[$pTop>>2]|0;
 $20 = ((($19)) + 532|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$pPrg>>2]|0;
 $23 = ((($22)) + 4|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$pPrg>>2]|0;
 $25 = HEAP32[$pTop>>2]|0;
 $26 = ((($25)) + 532|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$db>>2]|0;
 $28 = (_sqlite3DbMallocZero($27,28,0)|0);
 HEAP32[$pProgram>>2] = $28;
 $29 = HEAP32[$pPrg>>2]|0;
 $30 = ((($29)) + 8|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pProgram>>2]|0;
 $32 = ($31|0)!=(0|0);
 if (!($32)) {
  HEAP32[$0>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($165|0);
 }
 $33 = HEAP32[$pTop>>2]|0;
 $34 = ((($33)) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$pProgram>>2]|0;
 _sqlite3VdbeLinkSubProgram($35,$36);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$pPrg>>2]|0;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$4>>2]|0;
 $40 = HEAP32[$pPrg>>2]|0;
 $41 = ((($40)) + 12|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pPrg>>2]|0;
 $43 = ((($42)) + 16|0);
 HEAP32[$43>>2] = -1;
 $44 = HEAP32[$pPrg>>2]|0;
 $45 = ((($44)) + 16|0);
 $46 = ((($45)) + 4|0);
 HEAP32[$46>>2] = -1;
 $47 = HEAP32[$db>>2]|0;
 $48 = (_sqlite3DbMallocZero($47,544,0)|0);
 HEAP32[$pSubParse>>2] = $48;
 $49 = HEAP32[$pSubParse>>2]|0;
 $50 = ($49|0)!=(0|0);
 if (!($50)) {
  HEAP32[$0>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($165|0);
 }
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $51 = HEAP32[$pSubParse>>2]|0;
 HEAP32[$sNC>>2] = $51;
 $52 = HEAP32[$db>>2]|0;
 $53 = HEAP32[$pSubParse>>2]|0;
 HEAP32[$53>>2] = $52;
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$pSubParse>>2]|0;
 $56 = ((($55)) + 420|0);
 HEAP32[$56>>2] = $54;
 $57 = HEAP32[$pTop>>2]|0;
 $58 = HEAP32[$pSubParse>>2]|0;
 $59 = ((($58)) + 416|0);
 HEAP32[$59>>2] = $57;
 $60 = HEAP32[$2>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$pSubParse>>2]|0;
 $63 = ((($62)) + 496|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$2>>2]|0;
 $65 = ((($64)) + 8|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = HEAP32[$pSubParse>>2]|0;
 $68 = ((($67)) + 440|0);
 HEAP8[$68>>0] = $66;
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 428|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$pSubParse>>2]|0;
 $73 = ((($72)) + 428|0);
 HEAP32[$73>>2] = $71;
 $74 = HEAP32[$pSubParse>>2]|0;
 $75 = (_sqlite3GetVdbe($74)|0);
 HEAP32[$v>>2] = $75;
 $76 = HEAP32[$v>>2]|0;
 $77 = ($76|0)!=(0|0);
 if ($77) {
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$db>>2]|0;
  $80 = HEAP32[$2>>2]|0;
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$vararg_buffer>>2] = $81;
  $82 = (_sqlite3MPrintf($79,33477,$vararg_buffer)|0);
  _sqlite3VdbeChangeP4($78,-1,$82,-1);
  $83 = HEAP32[$2>>2]|0;
  $84 = ((($83)) + 12|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($85|0)!=(0|0);
  if ($86) {
   $87 = HEAP32[$db>>2]|0;
   $88 = HEAP32[$2>>2]|0;
   $89 = ((($88)) + 12|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (_sqlite3ExprDup($87,$90,0)|0);
   HEAP32[$pWhen>>2] = $91;
   $92 = HEAP32[$pWhen>>2]|0;
   $93 = (_sqlite3ResolveExprNames($sNC,$92)|0);
   $94 = (0)==($93|0);
   if ($94) {
    $95 = HEAP32[$db>>2]|0;
    $96 = ((($95)) + 65|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $97&255;
    $99 = ($98|0)==(0);
    if ($99) {
     $100 = HEAP32[$v>>2]|0;
     $101 = (_sqlite3VdbeMakeLabel($100)|0);
     HEAP32[$iEndTrigger>>2] = $101;
     $102 = HEAP32[$pSubParse>>2]|0;
     $103 = HEAP32[$pWhen>>2]|0;
     $104 = HEAP32[$iEndTrigger>>2]|0;
     _sqlite3ExprIfFalse($102,$103,$104,16);
    }
   }
   $105 = HEAP32[$db>>2]|0;
   $106 = HEAP32[$pWhen>>2]|0;
   _sqlite3ExprDelete($105,$106);
  }
  $107 = HEAP32[$pSubParse>>2]|0;
  $108 = HEAP32[$2>>2]|0;
  $109 = ((($108)) + 28|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$4>>2]|0;
  (_codeTriggerProgram($107,$110,$111)|0);
  $112 = HEAP32[$iEndTrigger>>2]|0;
  $113 = ($112|0)!=(0);
  if ($113) {
   $114 = HEAP32[$v>>2]|0;
   $115 = HEAP32[$iEndTrigger>>2]|0;
   _sqlite3VdbeResolveLabel($114,$115);
  }
  $116 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($116,21)|0);
  $117 = HEAP32[$1>>2]|0;
  $118 = HEAP32[$pSubParse>>2]|0;
  _transferParseError($117,$118);
  $119 = HEAP32[$db>>2]|0;
  $120 = ((($119)) + 65|0);
  $121 = HEAP8[$120>>0]|0;
  $122 = $121&255;
  $123 = ($122|0)==(0);
  if ($123) {
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$pProgram>>2]|0;
   $126 = ((($125)) + 4|0);
   $127 = HEAP32[$pTop>>2]|0;
   $128 = ((($127)) + 400|0);
   $129 = (_sqlite3VdbeTakeOpArray($124,$126,$128)|0);
   $130 = HEAP32[$pProgram>>2]|0;
   HEAP32[$130>>2] = $129;
  }
  $131 = HEAP32[$pSubParse>>2]|0;
  $132 = ((($131)) + 76|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$pProgram>>2]|0;
  $135 = ((($134)) + 8|0);
  HEAP32[$135>>2] = $133;
  $136 = HEAP32[$pSubParse>>2]|0;
  $137 = ((($136)) + 72|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = HEAP32[$pProgram>>2]|0;
  $140 = ((($139)) + 12|0);
  HEAP32[$140>>2] = $138;
  $141 = HEAP32[$pSubParse>>2]|0;
  $142 = ((($141)) + 84|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = HEAP32[$pProgram>>2]|0;
  $145 = ((($144)) + 16|0);
  HEAP32[$145>>2] = $143;
  $146 = HEAP32[$2>>2]|0;
  $147 = HEAP32[$pProgram>>2]|0;
  $148 = ((($147)) + 20|0);
  HEAP32[$148>>2] = $146;
  $149 = HEAP32[$pSubParse>>2]|0;
  $150 = ((($149)) + 432|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$pPrg>>2]|0;
  $153 = ((($152)) + 16|0);
  HEAP32[$153>>2] = $151;
  $154 = HEAP32[$pSubParse>>2]|0;
  $155 = ((($154)) + 436|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = HEAP32[$pPrg>>2]|0;
  $158 = ((($157)) + 16|0);
  $159 = ((($158)) + 4|0);
  HEAP32[$159>>2] = $156;
  $160 = HEAP32[$v>>2]|0;
  _sqlite3VdbeDelete($160);
 }
 $161 = HEAP32[$pSubParse>>2]|0;
 _sqlite3ParserReset($161);
 $162 = HEAP32[$db>>2]|0;
 $163 = HEAP32[$pSubParse>>2]|0;
 _sqlite3DbFree($162,$163);
 $164 = HEAP32[$pPrg>>2]|0;
 HEAP32[$0>>2] = $164;
 $165 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($165|0);
}
function _sqlite3VdbeLinkSubProgram($pVdbe,$p) {
 $pVdbe = $pVdbe|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVdbe;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 192|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 24|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 192|0);
 HEAP32[$9>>2] = $7;
 STACKTOP = sp;return;
}
function _codeTriggerProgram($pParse,$pStepList,$orconf) {
 $pParse = $pParse|0;
 $pStepList = $pStepList|0;
 $orconf = $orconf|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $db = 0, $pSelect = 0, $pStep = 0, $sDest = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pStep = sp + 32|0;
 $v = sp + 28|0;
 $db = sp + 24|0;
 $sDest = sp + 4|0;
 $pSelect = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStepList;
 HEAP32[$2>>2] = $orconf;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 HEAP32[$pStep>>2] = $8;
 while(1) {
  $9 = HEAP32[$pStep>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$2>>2]|0;
  $12 = ($11|0)==(10);
  if ($12) {
   $13 = HEAP32[$pStep>>2]|0;
   $14 = ((($13)) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $$sink = $15;
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = $16&255;
   $$sink = $17;
  }
  $18 = $$sink&255;
  $19 = $18&255;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 441|0);
  HEAP8[$21>>0] = $19;
  $22 = HEAP32[$pStep>>2]|0;
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  switch ($24|0) {
  case 110:  {
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$0>>2]|0;
   $27 = HEAP32[$pStep>>2]|0;
   $28 = (_targetSrcList($26,$27)|0);
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$pStep>>2]|0;
   $31 = ((($30)) + 20|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3ExprListDup($29,$32,0)|0);
   $34 = HEAP32[$db>>2]|0;
   $35 = HEAP32[$pStep>>2]|0;
   $36 = ((($35)) + 16|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (_sqlite3ExprDup($34,$37,0)|0);
   $39 = HEAP32[$0>>2]|0;
   $40 = ((($39)) + 441|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   _sqlite3Update($25,$28,$33,$38,$42);
   break;
  }
  case 108:  {
   $43 = HEAP32[$0>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = HEAP32[$pStep>>2]|0;
   $46 = (_targetSrcList($44,$45)|0);
   $47 = HEAP32[$db>>2]|0;
   $48 = HEAP32[$pStep>>2]|0;
   $49 = ((($48)) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_sqlite3SelectDup($47,$50,0)|0);
   $52 = HEAP32[$db>>2]|0;
   $53 = HEAP32[$pStep>>2]|0;
   $54 = ((($53)) + 24|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (_sqlite3IdListDup($52,$55)|0);
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 441|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = $59&255;
   _sqlite3Insert($43,$46,$51,$56,$60);
   break;
  }
  case 109:  {
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$0>>2]|0;
   $63 = HEAP32[$pStep>>2]|0;
   $64 = (_targetSrcList($62,$63)|0);
   $65 = HEAP32[$db>>2]|0;
   $66 = HEAP32[$pStep>>2]|0;
   $67 = ((($66)) + 16|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (_sqlite3ExprDup($65,$68,0)|0);
   _sqlite3DeleteFrom($61,$64,$69);
   break;
  }
  default: {
   $70 = HEAP32[$db>>2]|0;
   $71 = HEAP32[$pStep>>2]|0;
   $72 = ((($71)) + 8|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (_sqlite3SelectDup($70,$73,0)|0);
   HEAP32[$pSelect>>2] = $74;
   _sqlite3SelectDestInit($sDest,4,0);
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$pSelect>>2]|0;
   (_sqlite3Select($75,$76,$sDest)|0);
   $77 = HEAP32[$db>>2]|0;
   $78 = HEAP32[$pSelect>>2]|0;
   _sqlite3SelectDelete($77,$78);
  }
  }
  $79 = HEAP32[$pStep>>2]|0;
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = ($81|0)!=(119);
  if ($82) {
   $83 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeAddOp0($83,98)|0);
  }
  $84 = HEAP32[$pStep>>2]|0;
  $85 = ((($84)) + 28|0);
  $86 = HEAP32[$85>>2]|0;
  HEAP32[$pStep>>2] = $86;
 }
 STACKTOP = sp;return 0;
}
function _transferParseError($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 68|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 4|0);
  HEAP32[$10>>2] = $8;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 68|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 68|0);
  HEAP32[$15>>2] = $13;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 12|0);
  HEAP32[$20>>2] = $18;
  STACKTOP = sp;return;
 } else {
  $21 = HEAP32[$6>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3DbFree($21,$24);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeTakeOpArray($p,$pnOp,$pnMaxArg) {
 $p = $p|0;
 $pnOp = $pnOp|0;
 $pnMaxArg = $pnMaxArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aOp = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnOp;
 HEAP32[$2>>2] = $pnMaxArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$aOp>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 _resolveP2Values($6,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$aOp>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _targetSrcList($pParse,$pStep) {
 $pParse = $pParse|0;
 $pStep = $pStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $or$cond = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $iDb = sp + 4|0;
 $pSrc = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStep;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (_sqlite3SrcListAppend($4,0,0,0)|0);
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $46 = HEAP32[$pSrc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3DbStrDup($8,$11)|0);
 $13 = HEAP32[$pSrc>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) - 1)|0;
 $16 = HEAP32[$pSrc>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = (($17) + (($15*72)|0)|0);
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = $12;
 $20 = HEAP32[$db>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3SchemaToIndex($20,$25)|0);
 HEAP32[$iDb>>2] = $26;
 $27 = HEAP32[$iDb>>2]|0;
 $28 = ($27|0)==(0);
 $29 = HEAP32[$iDb>>2]|0;
 $30 = ($29|0)>=(2);
 $or$cond = $28 | $30;
 if (!($or$cond)) {
  $46 = HEAP32[$pSrc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $31 = HEAP32[$db>>2]|0;
 $32 = HEAP32[$iDb>>2]|0;
 $33 = HEAP32[$db>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + ($32<<4)|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (_sqlite3DbStrDup($31,$37)|0);
 $39 = HEAP32[$pSrc>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) - 1)|0;
 $42 = HEAP32[$pSrc>>2]|0;
 $43 = ((($42)) + 8|0);
 $44 = (($43) + (($41*72)|0)|0);
 $45 = ((($44)) + 4|0);
 HEAP32[$45>>2] = $38;
 $46 = HEAP32[$pSrc>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3FkLocateIndex($pParse,$pParent,$pFKey,$ppIdx,$paiCol) {
 $pParse = $pParse|0;
 $pParent = $pParent|0;
 $pFKey = $pFKey|0;
 $ppIdx = $ppIdx|0;
 $paiCol = $paiCol|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiCol = 0, $i = 0, $i1 = 0;
 var $iCol = 0, $j = 0, $nCol = 0, $pIdx = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zDfltColl = 0, $zIdxCol = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $pIdx = sp + 40|0;
 $aiCol = sp + 36|0;
 $nCol = sp + 32|0;
 $zKey = sp + 28|0;
 $i = sp + 24|0;
 $i1 = sp + 20|0;
 $j = sp + 16|0;
 $iCol = sp + 68|0;
 $zDfltColl = sp + 12|0;
 $zIdxCol = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pParent;
 HEAP32[$3>>2] = $pFKey;
 HEAP32[$4>>2] = $ppIdx;
 HEAP32[$5>>2] = $paiCol;
 HEAP32[$pIdx>>2] = 0;
 HEAP32[$aiCol>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nCol>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 36|0);
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$zKey>>2] = $12;
 $13 = HEAP32[$nCol>>2]|0;
 $14 = ($13|0)==(1);
 do {
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 32|0);
   $17 = HEAP16[$16>>1]|0;
   $18 = $17 << 16 >> 16;
   $19 = ($18|0)>=(0);
   if ($19) {
    $20 = HEAP32[$zKey>>2]|0;
    $21 = ($20|0)!=(0|0);
    if (!($21)) {
     HEAP32[$0>>2] = 0;
     $175 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($175|0);
    }
    $22 = HEAP32[$2>>2]|0;
    $23 = ((($22)) + 32|0);
    $24 = HEAP16[$23>>1]|0;
    $25 = $24 << 16 >> 16;
    $26 = HEAP32[$2>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (($28) + (($25*24)|0)|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$zKey>>2]|0;
    $32 = (_sqlite3_stricmp($30,$31)|0);
    $33 = ($32|0)!=(0);
    if (!($33)) {
     HEAP32[$0>>2] = 0;
     $175 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($175|0);
    }
   }
  } else {
   $34 = HEAP32[$5>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$nCol>>2]|0;
    $39 = $38<<2;
    $40 = (_sqlite3DbMallocRawNN($37,$39,0)|0);
    HEAP32[$aiCol>>2] = $40;
    $41 = HEAP32[$aiCol>>2]|0;
    $42 = ($41|0)!=(0|0);
    if ($42) {
     $43 = HEAP32[$aiCol>>2]|0;
     $44 = HEAP32[$5>>2]|0;
     HEAP32[$44>>2] = $43;
     break;
    }
    HEAP32[$0>>2] = 1;
    $175 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($175|0);
   }
  }
 } while(0);
 $45 = HEAP32[$2>>2]|0;
 $46 = ((($45)) + 8|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pIdx>>2] = $47;
 L17: while(1) {
  $48 = HEAP32[$pIdx>>2]|0;
  $49 = ($48|0)!=(0|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[$pIdx>>2]|0;
  $51 = ((($50)) + 50|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = HEAP32[$nCol>>2]|0;
  $55 = ($53|0)==($54|0);
  do {
   if ($55) {
    $56 = HEAP32[$pIdx>>2]|0;
    $57 = ((($56)) + 54|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    $60 = ($59|0)!=(0);
    if ($60) {
     $61 = HEAP32[$zKey>>2]|0;
     $62 = ($61|0)==(0|0);
     if ($62) {
      $63 = HEAP32[$pIdx>>2]|0;
      $64 = ((($63)) + 55|0);
      $65 = HEAP8[$64>>0]|0;
      $66 = $65 & 3;
      $67 = $66&255;
      $68 = ($67|0)==(2);
      if ($68) {
       label = 17;
       break L17;
      } else {
       break;
      }
     }
     HEAP32[$i1>>2] = 0;
     while(1) {
      $84 = HEAP32[$i1>>2]|0;
      $85 = HEAP32[$nCol>>2]|0;
      $86 = ($84|0)<($85|0);
      if (!($86)) {
       break;
      }
      $87 = HEAP32[$i1>>2]|0;
      $88 = HEAP32[$pIdx>>2]|0;
      $89 = ((($88)) + 4|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (($90) + ($87<<1)|0);
      $92 = HEAP16[$91>>1]|0;
      HEAP16[$iCol>>1] = $92;
      $93 = HEAP16[$iCol>>1]|0;
      $94 = $93 << 16 >> 16;
      $95 = ($94|0)<(0);
      if ($95) {
       break;
      }
      $96 = HEAP16[$iCol>>1]|0;
      $97 = $96 << 16 >> 16;
      $98 = HEAP32[$2>>2]|0;
      $99 = ((($98)) + 4|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = (($100) + (($97*24)|0)|0);
      $102 = ((($101)) + 16|0);
      $103 = HEAP32[$102>>2]|0;
      HEAP32[$zDfltColl>>2] = $103;
      $104 = HEAP32[$zDfltColl>>2]|0;
      $105 = ($104|0)!=(0|0);
      $$ = $105 ? $103 : 20716;
      HEAP32[$zDfltColl>>2] = $$;
      $106 = HEAP32[$i1>>2]|0;
      $107 = HEAP32[$pIdx>>2]|0;
      $108 = ((($107)) + 32|0);
      $109 = HEAP32[$108>>2]|0;
      $110 = (($109) + ($106<<2)|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = HEAP32[$zDfltColl>>2]|0;
      $113 = (_sqlite3_stricmp($111,$112)|0);
      $114 = ($113|0)!=(0);
      if ($114) {
       break;
      }
      $115 = HEAP16[$iCol>>1]|0;
      $116 = $115 << 16 >> 16;
      $117 = HEAP32[$2>>2]|0;
      $118 = ((($117)) + 4|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = (($119) + (($116*24)|0)|0);
      $121 = HEAP32[$120>>2]|0;
      HEAP32[$zIdxCol>>2] = $121;
      HEAP32[$j>>2] = 0;
      while(1) {
       $122 = HEAP32[$j>>2]|0;
       $123 = HEAP32[$nCol>>2]|0;
       $124 = ($122|0)<($123|0);
       if (!($124)) {
        break;
       }
       $125 = HEAP32[$j>>2]|0;
       $126 = HEAP32[$3>>2]|0;
       $127 = ((($126)) + 36|0);
       $128 = (($127) + ($125<<3)|0);
       $129 = ((($128)) + 4|0);
       $130 = HEAP32[$129>>2]|0;
       $131 = HEAP32[$zIdxCol>>2]|0;
       $132 = (_sqlite3_stricmp($130,$131)|0);
       $133 = ($132|0)==(0);
       if ($133) {
        label = 28;
        break;
       }
       $144 = HEAP32[$j>>2]|0;
       $145 = (($144) + 1)|0;
       HEAP32[$j>>2] = $145;
      }
      if ((label|0) == 28) {
       label = 0;
       $134 = HEAP32[$aiCol>>2]|0;
       $135 = ($134|0)!=(0|0);
       if ($135) {
        $136 = HEAP32[$j>>2]|0;
        $137 = HEAP32[$3>>2]|0;
        $138 = ((($137)) + 36|0);
        $139 = (($138) + ($136<<3)|0);
        $140 = HEAP32[$139>>2]|0;
        $141 = HEAP32[$i1>>2]|0;
        $142 = HEAP32[$aiCol>>2]|0;
        $143 = (($142) + ($141<<2)|0);
        HEAP32[$143>>2] = $140;
       }
      }
      $146 = HEAP32[$j>>2]|0;
      $147 = HEAP32[$nCol>>2]|0;
      $148 = ($146|0)==($147|0);
      if ($148) {
       break;
      }
      $149 = HEAP32[$i1>>2]|0;
      $150 = (($149) + 1)|0;
      HEAP32[$i1>>2] = $150;
     }
     $151 = HEAP32[$i1>>2]|0;
     $152 = HEAP32[$nCol>>2]|0;
     $153 = ($151|0)==($152|0);
     if ($153) {
      break L17;
     }
    }
   }
  } while(0);
  $154 = HEAP32[$pIdx>>2]|0;
  $155 = ((($154)) + 20|0);
  $156 = HEAP32[$155>>2]|0;
  HEAP32[$pIdx>>2] = $156;
 }
 L42: do {
  if ((label|0) == 17) {
   $69 = HEAP32[$aiCol>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $71 = HEAP32[$i>>2]|0;
     $72 = HEAP32[$nCol>>2]|0;
     $73 = ($71|0)<($72|0);
     if (!($73)) {
      break L42;
     }
     $74 = HEAP32[$i>>2]|0;
     $75 = HEAP32[$3>>2]|0;
     $76 = ((($75)) + 36|0);
     $77 = (($76) + ($74<<3)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$i>>2]|0;
     $80 = HEAP32[$aiCol>>2]|0;
     $81 = (($80) + ($79<<2)|0);
     HEAP32[$81>>2] = $78;
     $82 = HEAP32[$i>>2]|0;
     $83 = (($82) + 1)|0;
     HEAP32[$i>>2] = $83;
    }
   }
  }
 } while(0);
 $157 = HEAP32[$pIdx>>2]|0;
 $158 = ($157|0)!=(0|0);
 if ($158) {
  $173 = HEAP32[$pIdx>>2]|0;
  $174 = HEAP32[$4>>2]|0;
  HEAP32[$174>>2] = $173;
  HEAP32[$0>>2] = 0;
  $175 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($175|0);
 }
 $159 = HEAP32[$1>>2]|0;
 $160 = ((($159)) + 442|0);
 $161 = HEAP8[$160>>0]|0;
 $162 = ($161<<24>>24)!=(0);
 if (!($162)) {
  $163 = HEAP32[$1>>2]|0;
  $164 = HEAP32[$3>>2]|0;
  $165 = HEAP32[$164>>2]|0;
  $166 = HEAP32[$165>>2]|0;
  $167 = HEAP32[$3>>2]|0;
  $168 = ((($167)) + 8|0);
  $169 = HEAP32[$168>>2]|0;
  HEAP32[$vararg_buffer>>2] = $166;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $169;
  _sqlite3ErrorMsg($163,33491,$vararg_buffer);
 }
 $170 = HEAP32[$1>>2]|0;
 $171 = HEAP32[$170>>2]|0;
 $172 = HEAP32[$aiCol>>2]|0;
 _sqlite3DbFree($171,$172);
 HEAP32[$0>>2] = 1;
 $175 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($175|0);
}
function _sqlite3CodeRowTriggerDirect($pParse,$p,$pTab,$reg,$orconf,$ignoreJump) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pTab = $pTab|0;
 $reg = $reg|0;
 $orconf = $orconf|0;
 $ignoreJump = $ignoreJump|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bRecursive = 0;
 var $pPrg = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $v = sp + 8|0;
 $pPrg = sp + 4|0;
 $bRecursive = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $reg;
 HEAP32[$4>>2] = $orconf;
 HEAP32[$5>>2] = $ignoreJump;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3GetVdbe($6)|0);
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_getRowTrigger($8,$9,$10,$11)|0);
 HEAP32[$pPrg>>2] = $12;
 $13 = HEAP32[$pPrg>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ((($19)) + 24|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 & 262144;
  $23 = (0)==($22|0);
  $25 = $23;
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 HEAP32[$bRecursive>>2] = $24;
 $26 = HEAP32[$v>>2]|0;
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$5>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 76|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 1)|0;
 HEAP32[$30>>2] = $32;
 $33 = HEAP32[$pPrg>>2]|0;
 $34 = ((($33)) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 (_sqlite3VdbeAddOp4($26,132,$27,$28,$32,$35,-18)|0);
 $36 = HEAP32[$v>>2]|0;
 $37 = HEAP32[$bRecursive>>2]|0;
 $38 = $37&255;
 _sqlite3VdbeChangeP5($36,$38);
 STACKTOP = sp;return;
}
function _fkLookupParent($pParse,$iDb,$pTab,$pIdx,$pFKey,$aiCol,$regData,$nIncr,$isIgnore) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 $pFKey = $pFKey|0;
 $aiCol = $aiCol|0;
 $regData = $regData|0;
 $nIncr = $nIncr|0;
 $isIgnore = $isIgnore|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iChild = 0, $iCur = 0, $iJump = 0, $iMustBeInt = 0, $iOk = 0, $iParent = 0, $iReg = 0, $nCol = 0, $or$cond = 0, $or$cond3 = 0, $regRec = 0, $regTemp = 0, $regTemp1 = 0;
 var $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $8 = sp + 52|0;
 $i = sp + 48|0;
 $v = sp + 44|0;
 $iCur = sp + 40|0;
 $iOk = sp + 36|0;
 $iReg = sp + 32|0;
 $iMustBeInt = sp + 28|0;
 $regTemp = sp + 24|0;
 $nCol = sp + 20|0;
 $regTemp1 = sp + 16|0;
 $regRec = sp + 12|0;
 $iJump = sp + 8|0;
 $iChild = sp + 4|0;
 $iParent = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pIdx;
 HEAP32[$4>>2] = $pFKey;
 HEAP32[$5>>2] = $aiCol;
 HEAP32[$6>>2] = $regData;
 HEAP32[$7>>2] = $nIncr;
 HEAP32[$8>>2] = $isIgnore;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 72|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) - 1)|0;
 HEAP32[$iCur>>2] = $14;
 $15 = HEAP32[$v>>2]|0;
 $16 = (_sqlite3VdbeMakeLabel($15)|0);
 HEAP32[$iOk>>2] = $16;
 $17 = HEAP32[$7>>2]|0;
 $18 = ($17|0)<(0);
 if ($18) {
  $19 = HEAP32[$v>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = ((($20)) + 24|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$iOk>>2]|0;
  (_sqlite3VdbeAddOp2($19,136,$23,$24)|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$6>>2]|0;
  $35 = (($33) + ($34))|0;
  $36 = (($35) + 1)|0;
  HEAP32[$iReg>>2] = $36;
  $37 = HEAP32[$v>>2]|0;
  $38 = HEAP32[$iReg>>2]|0;
  $39 = HEAP32[$iOk>>2]|0;
  (_sqlite3VdbeAddOp2($37,76,$38,$39)|0);
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$8>>2]|0;
 $43 = ($42|0)==(0);
 do {
  if ($43) {
   $44 = HEAP32[$3>>2]|0;
   $45 = ($44|0)==(0|0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    $47 = (_sqlite3GetTempReg($46)|0);
    HEAP32[$regTemp>>2] = $47;
    $48 = HEAP32[$v>>2]|0;
    $49 = HEAP32[$5>>2]|0;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + 1)|0;
    $52 = HEAP32[$6>>2]|0;
    $53 = (($51) + ($52))|0;
    $54 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp2($48,31,$53,$54)|0);
    $55 = HEAP32[$v>>2]|0;
    $56 = HEAP32[$regTemp>>2]|0;
    $57 = (_sqlite3VdbeAddOp2($55,38,$56,0)|0);
    HEAP32[$iMustBeInt>>2] = $57;
    $58 = HEAP32[$2>>2]|0;
    $59 = HEAP32[$4>>2]|0;
    $60 = HEAP32[$59>>2]|0;
    $61 = ($58|0)==($60|0);
    $62 = HEAP32[$7>>2]|0;
    $63 = ($62|0)==(1);
    $or$cond = $61 & $63;
    if ($or$cond) {
     $64 = HEAP32[$v>>2]|0;
     $65 = HEAP32[$6>>2]|0;
     $66 = HEAP32[$iOk>>2]|0;
     $67 = HEAP32[$regTemp>>2]|0;
     (_sqlite3VdbeAddOp3($64,79,$65,$66,$67)|0);
     $68 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($68,-112);
    }
    $69 = HEAP32[$0>>2]|0;
    $70 = HEAP32[$iCur>>2]|0;
    $71 = HEAP32[$1>>2]|0;
    $72 = HEAP32[$2>>2]|0;
    _sqlite3OpenTable($69,$70,$71,$72,54);
    $73 = HEAP32[$v>>2]|0;
    $74 = HEAP32[$iCur>>2]|0;
    $75 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp3($73,70,$74,0,$75)|0);
    $76 = HEAP32[$v>>2]|0;
    $77 = HEAP32[$iOk>>2]|0;
    (_sqlite3VdbeGoto($76,$77)|0);
    $78 = HEAP32[$v>>2]|0;
    $79 = HEAP32[$v>>2]|0;
    $80 = (_sqlite3VdbeCurrentAddr($79)|0);
    $81 = (($80) - 2)|0;
    _sqlite3VdbeJumpHere($78,$81);
    $82 = HEAP32[$v>>2]|0;
    $83 = HEAP32[$iMustBeInt>>2]|0;
    _sqlite3VdbeJumpHere($82,$83);
    $84 = HEAP32[$0>>2]|0;
    $85 = HEAP32[$regTemp>>2]|0;
    _sqlite3ReleaseTempReg($84,$85);
    break;
   }
   $86 = HEAP32[$4>>2]|0;
   $87 = ((($86)) + 20|0);
   $88 = HEAP32[$87>>2]|0;
   HEAP32[$nCol>>2] = $88;
   $89 = HEAP32[$0>>2]|0;
   $90 = HEAP32[$nCol>>2]|0;
   $91 = (_sqlite3GetTempRange($89,$90)|0);
   HEAP32[$regTemp1>>2] = $91;
   $92 = HEAP32[$0>>2]|0;
   $93 = (_sqlite3GetTempReg($92)|0);
   HEAP32[$regRec>>2] = $93;
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$iCur>>2]|0;
   $96 = HEAP32[$3>>2]|0;
   $97 = ((($96)) + 44|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$1>>2]|0;
   (_sqlite3VdbeAddOp3($94,54,$95,$98,$99)|0);
   $100 = HEAP32[$0>>2]|0;
   $101 = HEAP32[$3>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($100,$101);
   HEAP32[$i>>2] = 0;
   while(1) {
    $102 = HEAP32[$i>>2]|0;
    $103 = HEAP32[$nCol>>2]|0;
    $104 = ($102|0)<($103|0);
    if (!($104)) {
     break;
    }
    $105 = HEAP32[$v>>2]|0;
    $106 = HEAP32[$i>>2]|0;
    $107 = HEAP32[$5>>2]|0;
    $108 = (($107) + ($106<<2)|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 1)|0;
    $111 = HEAP32[$6>>2]|0;
    $112 = (($110) + ($111))|0;
    $113 = HEAP32[$regTemp1>>2]|0;
    $114 = HEAP32[$i>>2]|0;
    $115 = (($113) + ($114))|0;
    (_sqlite3VdbeAddOp2($105,30,$112,$115)|0);
    $116 = HEAP32[$i>>2]|0;
    $117 = (($116) + 1)|0;
    HEAP32[$i>>2] = $117;
   }
   $118 = HEAP32[$2>>2]|0;
   $119 = HEAP32[$4>>2]|0;
   $120 = HEAP32[$119>>2]|0;
   $121 = ($118|0)==($120|0);
   $122 = HEAP32[$7>>2]|0;
   $123 = ($122|0)==(1);
   $or$cond3 = $121 & $123;
   if ($or$cond3) {
    $124 = HEAP32[$v>>2]|0;
    $125 = (_sqlite3VdbeCurrentAddr($124)|0);
    $126 = HEAP32[$nCol>>2]|0;
    $127 = (($125) + ($126))|0;
    $128 = (($127) + 1)|0;
    HEAP32[$iJump>>2] = $128;
    HEAP32[$i>>2] = 0;
    while(1) {
     $129 = HEAP32[$i>>2]|0;
     $130 = HEAP32[$nCol>>2]|0;
     $131 = ($129|0)<($130|0);
     if (!($131)) {
      break;
     }
     $132 = HEAP32[$i>>2]|0;
     $133 = HEAP32[$5>>2]|0;
     $134 = (($133) + ($132<<2)|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = (($135) + 1)|0;
     $137 = HEAP32[$6>>2]|0;
     $138 = (($136) + ($137))|0;
     HEAP32[$iChild>>2] = $138;
     $139 = HEAP32[$i>>2]|0;
     $140 = HEAP32[$3>>2]|0;
     $141 = ((($140)) + 4|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = (($142) + ($139<<1)|0);
     $144 = HEAP16[$143>>1]|0;
     $145 = $144 << 16 >> 16;
     $146 = (($145) + 1)|0;
     $147 = HEAP32[$6>>2]|0;
     $148 = (($146) + ($147))|0;
     HEAP32[$iParent>>2] = $148;
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$3>>2]|0;
     $151 = ((($150)) + 4|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = (($152) + ($149<<1)|0);
     $154 = HEAP16[$153>>1]|0;
     $155 = $154 << 16 >> 16;
     $156 = HEAP32[$2>>2]|0;
     $157 = ((($156)) + 32|0);
     $158 = HEAP16[$157>>1]|0;
     $159 = $158 << 16 >> 16;
     $160 = ($155|0)==($159|0);
     if ($160) {
      $161 = HEAP32[$6>>2]|0;
      HEAP32[$iParent>>2] = $161;
     }
     $162 = HEAP32[$v>>2]|0;
     $163 = HEAP32[$iChild>>2]|0;
     $164 = HEAP32[$iJump>>2]|0;
     $165 = HEAP32[$iParent>>2]|0;
     (_sqlite3VdbeAddOp3($162,78,$163,$164,$165)|0);
     $166 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($166,16);
     $167 = HEAP32[$i>>2]|0;
     $168 = (($167) + 1)|0;
     HEAP32[$i>>2] = $168;
    }
    $169 = HEAP32[$v>>2]|0;
    $170 = HEAP32[$iOk>>2]|0;
    (_sqlite3VdbeGoto($169,$170)|0);
   }
   $171 = HEAP32[$v>>2]|0;
   $172 = HEAP32[$regTemp1>>2]|0;
   $173 = HEAP32[$nCol>>2]|0;
   $174 = HEAP32[$regRec>>2]|0;
   $175 = HEAP32[$0>>2]|0;
   $176 = HEAP32[$175>>2]|0;
   $177 = HEAP32[$3>>2]|0;
   $178 = (_sqlite3IndexAffinityStr($176,$177)|0);
   $179 = HEAP32[$nCol>>2]|0;
   (_sqlite3VdbeAddOp4($171,49,$172,$173,$174,$178,$179)|0);
   $180 = HEAP32[$v>>2]|0;
   $181 = HEAP32[$iCur>>2]|0;
   $182 = HEAP32[$iOk>>2]|0;
   $183 = HEAP32[$regRec>>2]|0;
   (_sqlite3VdbeAddOp4Int($180,69,$181,$182,$183,0)|0);
   $184 = HEAP32[$0>>2]|0;
   $185 = HEAP32[$regRec>>2]|0;
   _sqlite3ReleaseTempReg($184,$185);
   $186 = HEAP32[$0>>2]|0;
   $187 = HEAP32[$regTemp1>>2]|0;
   $188 = HEAP32[$nCol>>2]|0;
   _sqlite3ReleaseTempRange($186,$187,$188);
  }
 } while(0);
 $189 = HEAP32[$4>>2]|0;
 $190 = ((($189)) + 24|0);
 $191 = HEAP8[$190>>0]|0;
 $192 = ($191<<24>>24)!=(0);
 if (!($192)) {
  $193 = HEAP32[$0>>2]|0;
  $194 = HEAP32[$193>>2]|0;
  $195 = ((($194)) + 24|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = $196 & 16777216;
  $198 = ($197|0)!=(0);
  if (!($198)) {
   $199 = HEAP32[$0>>2]|0;
   $200 = ((($199)) + 416|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = ($201|0)!=(0|0);
   if (!($202)) {
    $203 = HEAP32[$0>>2]|0;
    $204 = ((($203)) + 20|0);
    $205 = HEAP8[$204>>0]|0;
    $206 = ($205<<24>>24)!=(0);
    if (!($206)) {
     $207 = HEAP32[$0>>2]|0;
     _sqlite3HaltConstraint($207,787,2,0,-2,4);
     $222 = HEAP32[$v>>2]|0;
     $223 = HEAP32[$iOk>>2]|0;
     _sqlite3VdbeResolveLabel($222,$223);
     $224 = HEAP32[$v>>2]|0;
     $225 = HEAP32[$iCur>>2]|0;
     (_sqlite3VdbeAddOp1($224,61,$225)|0);
     STACKTOP = sp;return;
    }
   }
  }
 }
 $208 = HEAP32[$7>>2]|0;
 $209 = ($208|0)>(0);
 if ($209) {
  $210 = HEAP32[$4>>2]|0;
  $211 = ((($210)) + 24|0);
  $212 = HEAP8[$211>>0]|0;
  $213 = $212&255;
  $214 = ($213|0)==(0);
  if ($214) {
   $215 = HEAP32[$0>>2]|0;
   _sqlite3MayAbort($215);
  }
 }
 $216 = HEAP32[$v>>2]|0;
 $217 = HEAP32[$4>>2]|0;
 $218 = ((($217)) + 24|0);
 $219 = HEAP8[$218>>0]|0;
 $220 = $219&255;
 $221 = HEAP32[$7>>2]|0;
 (_sqlite3VdbeAddOp2($216,135,$220,$221)|0);
 $222 = HEAP32[$v>>2]|0;
 $223 = HEAP32[$iOk>>2]|0;
 _sqlite3VdbeResolveLabel($222,$223);
 $224 = HEAP32[$v>>2]|0;
 $225 = HEAP32[$iCur>>2]|0;
 (_sqlite3VdbeAddOp1($224,61,$225)|0);
 STACKTOP = sp;return;
}
function _isSetNullAction($pParse,$pFKey) {
 $pParse = $pParse|0;
 $pFKey = $pFKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pTop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTop = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFKey;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 416|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 416|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9;
 } else {
  $10 = $7;
 }
 HEAP32[$pTop>>2] = $10;
 $11 = HEAP32[$pTop>>2]|0;
 $12 = ((($11)) + 532|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 do {
  if ($14) {
   $15 = HEAP32[$pTop>>2]|0;
   $16 = ((($15)) + 532|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$p>>2] = $18;
   $19 = HEAP32[$p>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 28|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)==($22|0);
   if ($23) {
    $24 = HEAP32[$2>>2]|0;
    $25 = ((($24)) + 25|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)==(7);
    if (!($28)) {
     label = 6;
    }
   } else {
    label = 6;
   }
   if ((label|0) == 6) {
    $29 = HEAP32[$p>>2]|0;
    $30 = HEAP32[$2>>2]|0;
    $31 = ((($30)) + 28|0);
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($29|0)==($33|0);
    if (!($34)) {
     break;
    }
    $35 = HEAP32[$2>>2]|0;
    $36 = ((($35)) + 25|0);
    $37 = ((($36)) + 1|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = ($39|0)==(7);
    if (!($40)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $41 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _fkScanChildren($pParse,$pSrc,$pTab,$pIdx,$pFKey,$aiCol,$regData,$nIncr) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 $pFKey = $pFKey|0;
 $aiCol = $aiCol|0;
 $regData = $regData|0;
 $nIncr = $nIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $iCol4 = 0, $iFkIfZero = 0, $or$cond = 0, $pAll = 0, $pEq = 0, $pEq3 = 0, $pLeft = 0, $pLeft1 = 0, $pNe = 0, $pPk = 0;
 var $pRight = 0, $pRight2 = 0, $pWInfo = 0, $pWhere = 0, $sNameContext = 0, $v = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 112|0;
 $5 = sp + 108|0;
 $6 = sp + 104|0;
 $7 = sp + 100|0;
 $db = sp + 96|0;
 $i = sp + 92|0;
 $pWhere = sp + 88|0;
 $sNameContext = sp + 56|0;
 $pWInfo = sp + 48|0;
 $iFkIfZero = sp + 44|0;
 $v = sp + 40|0;
 $pLeft = sp + 36|0;
 $pRight = sp + 32|0;
 $pEq = sp + 28|0;
 $iCol = sp + 134|0;
 $zCol = sp + 24|0;
 $pNe = sp + 20|0;
 $pLeft1 = sp + 16|0;
 $pRight2 = sp + 12|0;
 $pEq3 = sp + 8|0;
 $pAll = sp + 4|0;
 $pPk = sp;
 $iCol4 = sp + 132|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pIdx;
 HEAP32[$4>>2] = $pFKey;
 HEAP32[$5>>2] = $aiCol;
 HEAP32[$6>>2] = $regData;
 HEAP32[$7>>2] = $nIncr;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 HEAP32[$pWhere>>2] = 0;
 HEAP32[$iFkIfZero>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3GetVdbe($10)|0);
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$7>>2]|0;
 $13 = ($12|0)<(0);
 if ($13) {
  $14 = HEAP32[$v>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  $16 = ((($15)) + 24|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = (_sqlite3VdbeAddOp2($14,136,$18,0)|0);
  HEAP32[$iFkIfZero>>2] = $19;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$3>>2]|0;
  $26 = ($25|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$3>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + ($27<<1)|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32 << 16 >> 16;
   $35 = $33;
  } else {
   $35 = -1;
  }
  $34 = $35&65535;
  HEAP16[$iCol>>1] = $34;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$6>>2]|0;
  $39 = HEAP16[$iCol>>1]|0;
  $40 = (_exprTableRegister($36,$37,$38,$39)|0);
  HEAP32[$pLeft>>2] = $40;
  $41 = HEAP32[$5>>2]|0;
  $42 = ($41|0)!=(0|0);
  if ($42) {
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$5>>2]|0;
   $45 = (($44) + ($43<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $51 = $46;
  } else {
   $47 = HEAP32[$4>>2]|0;
   $48 = ((($47)) + 36|0);
   $49 = HEAP32[$48>>2]|0;
   $51 = $49;
  }
  $50 = $51&65535;
  HEAP16[$iCol>>1] = $50;
  $52 = HEAP16[$iCol>>1]|0;
  $53 = $52 << 16 >> 16;
  $54 = HEAP32[$4>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = ((($55)) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + (($53*24)|0)|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$zCol>>2] = $59;
  $60 = HEAP32[$db>>2]|0;
  $61 = HEAP32[$zCol>>2]|0;
  $62 = (_sqlite3Expr($60,27,$61)|0);
  HEAP32[$pRight>>2] = $62;
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pLeft>>2]|0;
  $65 = HEAP32[$pRight>>2]|0;
  $66 = (_sqlite3PExpr($63,79,$64,$65,0)|0);
  HEAP32[$pEq>>2] = $66;
  $67 = HEAP32[$db>>2]|0;
  $68 = HEAP32[$pWhere>>2]|0;
  $69 = HEAP32[$pEq>>2]|0;
  $70 = (_sqlite3ExprAnd($67,$68,$69)|0);
  HEAP32[$pWhere>>2] = $70;
  $71 = HEAP32[$i>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$i>>2] = $72;
 }
 $73 = HEAP32[$2>>2]|0;
 $74 = HEAP32[$4>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = ($73|0)==($75|0);
 $77 = HEAP32[$7>>2]|0;
 $78 = ($77|0)>(0);
 $or$cond = $76 & $78;
 if ($or$cond) {
  $79 = HEAP32[$2>>2]|0;
  $80 = ((($79)) + 42|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = $81&255;
  $83 = $82 & 32;
  $84 = ($83|0)==(0);
  if ($84) {
   $85 = HEAP32[$0>>2]|0;
   $86 = HEAP32[$2>>2]|0;
   $87 = HEAP32[$6>>2]|0;
   $88 = (_exprTableRegister($85,$86,$87,-1)|0);
   HEAP32[$pLeft1>>2] = $88;
   $89 = HEAP32[$db>>2]|0;
   $90 = HEAP32[$2>>2]|0;
   $91 = HEAP32[$1>>2]|0;
   $92 = ((($91)) + 8|0);
   $93 = ((($92)) + 44|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (_exprTableColumn($89,$90,$94,-1)|0);
   HEAP32[$pRight2>>2] = $95;
   $96 = HEAP32[$0>>2]|0;
   $97 = HEAP32[$pLeft1>>2]|0;
   $98 = HEAP32[$pRight2>>2]|0;
   $99 = (_sqlite3PExpr($96,78,$97,$98,0)|0);
   HEAP32[$pNe>>2] = $99;
  } else {
   HEAP32[$pAll>>2] = 0;
   $100 = HEAP32[$2>>2]|0;
   $101 = (_sqlite3PrimaryKeyIndex($100)|0);
   HEAP32[$pPk>>2] = $101;
   HEAP32[$i>>2] = 0;
   while(1) {
    $102 = HEAP32[$i>>2]|0;
    $103 = HEAP32[$pPk>>2]|0;
    $104 = ((($103)) + 50|0);
    $105 = HEAP16[$104>>1]|0;
    $106 = $105&65535;
    $107 = ($102|0)<($106|0);
    if (!($107)) {
     break;
    }
    $108 = HEAP32[$i>>2]|0;
    $109 = HEAP32[$3>>2]|0;
    $110 = ((($109)) + 4|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = (($111) + ($108<<1)|0);
    $113 = HEAP16[$112>>1]|0;
    HEAP16[$iCol4>>1] = $113;
    $114 = HEAP32[$0>>2]|0;
    $115 = HEAP32[$2>>2]|0;
    $116 = HEAP32[$6>>2]|0;
    $117 = HEAP16[$iCol4>>1]|0;
    $118 = (_exprTableRegister($114,$115,$116,$117)|0);
    HEAP32[$pLeft1>>2] = $118;
    $119 = HEAP32[$db>>2]|0;
    $120 = HEAP32[$2>>2]|0;
    $121 = HEAP32[$1>>2]|0;
    $122 = ((($121)) + 8|0);
    $123 = ((($122)) + 44|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP16[$iCol4>>1]|0;
    $126 = (_exprTableColumn($119,$120,$124,$125)|0);
    HEAP32[$pRight2>>2] = $126;
    $127 = HEAP32[$0>>2]|0;
    $128 = HEAP32[$pLeft1>>2]|0;
    $129 = HEAP32[$pRight2>>2]|0;
    $130 = (_sqlite3PExpr($127,79,$128,$129,0)|0);
    HEAP32[$pEq3>>2] = $130;
    $131 = HEAP32[$db>>2]|0;
    $132 = HEAP32[$pAll>>2]|0;
    $133 = HEAP32[$pEq3>>2]|0;
    $134 = (_sqlite3ExprAnd($131,$132,$133)|0);
    HEAP32[$pAll>>2] = $134;
    $135 = HEAP32[$i>>2]|0;
    $136 = (($135) + 1)|0;
    HEAP32[$i>>2] = $136;
   }
   $137 = HEAP32[$0>>2]|0;
   $138 = HEAP32[$pAll>>2]|0;
   $139 = (_sqlite3PExpr($137,19,$138,0,0)|0);
   HEAP32[$pNe>>2] = $139;
  }
  $140 = HEAP32[$db>>2]|0;
  $141 = HEAP32[$pWhere>>2]|0;
  $142 = HEAP32[$pNe>>2]|0;
  $143 = (_sqlite3ExprAnd($140,$141,$142)|0);
  HEAP32[$pWhere>>2] = $143;
 }
 ;HEAP32[$sNameContext>>2]=0|0;HEAP32[$sNameContext+4>>2]=0|0;HEAP32[$sNameContext+8>>2]=0|0;HEAP32[$sNameContext+12>>2]=0|0;HEAP32[$sNameContext+16>>2]=0|0;HEAP32[$sNameContext+20>>2]=0|0;HEAP32[$sNameContext+24>>2]=0|0;HEAP32[$sNameContext+28>>2]=0|0;
 $144 = HEAP32[$1>>2]|0;
 $145 = ((($sNameContext)) + 4|0);
 HEAP32[$145>>2] = $144;
 $146 = HEAP32[$0>>2]|0;
 HEAP32[$sNameContext>>2] = $146;
 $147 = HEAP32[$pWhere>>2]|0;
 (_sqlite3ResolveExprNames($sNameContext,$147)|0);
 $148 = HEAP32[$0>>2]|0;
 $149 = HEAP32[$1>>2]|0;
 $150 = HEAP32[$pWhere>>2]|0;
 $151 = (_sqlite3WhereBegin($148,$149,$150,0,0,0,0)|0);
 HEAP32[$pWInfo>>2] = $151;
 $152 = HEAP32[$v>>2]|0;
 $153 = HEAP32[$4>>2]|0;
 $154 = ((($153)) + 24|0);
 $155 = HEAP8[$154>>0]|0;
 $156 = $155&255;
 $157 = HEAP32[$7>>2]|0;
 (_sqlite3VdbeAddOp2($152,135,$156,$157)|0);
 $158 = HEAP32[$pWInfo>>2]|0;
 $159 = ($158|0)!=(0|0);
 if ($159) {
  $160 = HEAP32[$pWInfo>>2]|0;
  _sqlite3WhereEnd($160);
 }
 $161 = HEAP32[$db>>2]|0;
 $162 = HEAP32[$pWhere>>2]|0;
 _sqlite3ExprDelete($161,$162);
 $163 = HEAP32[$iFkIfZero>>2]|0;
 $164 = ($163|0)!=(0);
 if (!($164)) {
  STACKTOP = sp;return;
 }
 $165 = HEAP32[$v>>2]|0;
 $166 = HEAP32[$iFkIfZero>>2]|0;
 _sqlite3VdbeJumpHere($165,$166);
 STACKTOP = sp;return;
}
function _exprTableRegister($pParse,$pTab,$regBase,$iCol) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $regBase = $regBase|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pCol = 0, $pExpr = 0;
 var $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 28|0;
 $pExpr = sp + 12|0;
 $pCol = sp + 8|0;
 $zColl = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $regBase;
 HEAP16[$3>>1] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3Expr($6,157,0)|0);
 HEAP32[$pExpr>>2] = $7;
 $8 = HEAP32[$pExpr>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $56 = HEAP32[$pExpr>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $10 = HEAP16[$3>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = ($11|0)>=(0);
 if ($12) {
  $13 = HEAP16[$3>>1]|0;
  $14 = $13 << 16 >> 16;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 32|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($14|0)!=($18|0);
  if ($19) {
   $20 = HEAP16[$3>>1]|0;
   $21 = $20 << 16 >> 16;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + (($21*24)|0)|0);
   HEAP32[$pCol>>2] = $25;
   $26 = HEAP32[$2>>2]|0;
   $27 = HEAP16[$3>>1]|0;
   $28 = $27 << 16 >> 16;
   $29 = (($26) + ($28))|0;
   $30 = (($29) + 1)|0;
   $31 = HEAP32[$pExpr>>2]|0;
   $32 = ((($31)) + 28|0);
   HEAP32[$32>>2] = $30;
   $33 = HEAP32[$pCol>>2]|0;
   $34 = ((($33)) + 21|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = HEAP32[$pExpr>>2]|0;
   $37 = ((($36)) + 1|0);
   HEAP8[$37>>0] = $35;
   $38 = HEAP32[$pCol>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$zColl>>2] = $40;
   $41 = HEAP32[$zColl>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    $43 = HEAP32[$db>>2]|0;
    $44 = ((($43)) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$zColl>>2] = $46;
   }
   $47 = HEAP32[$0>>2]|0;
   $48 = HEAP32[$pExpr>>2]|0;
   $49 = HEAP32[$zColl>>2]|0;
   $50 = (_sqlite3ExprAddCollateString($47,$48,$49)|0);
   HEAP32[$pExpr>>2] = $50;
   $56 = HEAP32[$pExpr>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 }
 $51 = HEAP32[$2>>2]|0;
 $52 = HEAP32[$pExpr>>2]|0;
 $53 = ((($52)) + 28|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$pExpr>>2]|0;
 $55 = ((($54)) + 1|0);
 HEAP8[$55>>0] = 68;
 $56 = HEAP32[$pExpr>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _exprTableColumn($db,$pTab,$iCursor,$iCol) {
 $db = $db|0;
 $pTab = $pTab|0;
 $iCursor = $iCursor|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pExpr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iCursor;
 HEAP16[$3>>1] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3Expr($4,152,0)|0);
 HEAP32[$pExpr>>2] = $5;
 $6 = HEAP32[$pExpr>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $17 = HEAP32[$pExpr>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$pExpr>>2]|0;
 $10 = ((($9)) + 44|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$pExpr>>2]|0;
 $13 = ((($12)) + 28|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP16[$3>>1]|0;
 $15 = HEAP32[$pExpr>>2]|0;
 $16 = ((($15)) + 32|0);
 HEAP16[$16>>1] = $14;
 $17 = HEAP32[$pExpr>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _fkActionTrigger($pParse,$pTab,$pFKey,$pChanges) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pFKey = $pFKey|0;
 $pChanges = $pChanges|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $action = 0, $aiCol = 0, $db = 0, $i = 0, $iAction = 0, $iFromCol = 0, $nFrom = 0, $or$cond = 0, $or$cond3 = 0;
 var $pDflt = 0, $pEq = 0, $pIdx = 0, $pList = 0, $pNew = 0, $pRaise = 0, $pSelect = 0, $pStep = 0, $pTrigger = 0, $pWhen = 0, $pWhere = 0, $tFrom = 0, $tFromCol = 0, $tNew = 0, $tOld = 0, $tToCol = 0, $zFrom = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $1 = sp + 132|0;
 $2 = sp + 128|0;
 $3 = sp + 124|0;
 $4 = sp + 120|0;
 $db = sp + 116|0;
 $action = sp + 112|0;
 $pTrigger = sp + 108|0;
 $iAction = sp + 104|0;
 $zFrom = sp + 100|0;
 $nFrom = sp + 96|0;
 $pIdx = sp + 92|0;
 $aiCol = sp + 88|0;
 $pStep = sp + 84|0;
 $pWhere = sp + 80|0;
 $pList = sp + 76|0;
 $pSelect = sp + 72|0;
 $i = sp + 68|0;
 $pWhen = sp + 64|0;
 $tOld = sp + 56|0;
 $tNew = sp + 48|0;
 $tFromCol = sp + 40|0;
 $tToCol = sp + 32|0;
 $iFromCol = sp + 28|0;
 $pEq = sp + 24|0;
 $pNew = sp + 20|0;
 $pDflt = sp + 16|0;
 $tFrom = sp + 8|0;
 $pRaise = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pFKey;
 HEAP32[$4>>2] = $pChanges;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8&1;
 HEAP32[$iAction>>2] = $9;
 $10 = HEAP32[$iAction>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 25|0);
 $13 = (($12) + ($10)|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 HEAP32[$action>>2] = $15;
 $16 = HEAP32[$iAction>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 28|0);
 $19 = (($18) + ($16<<2)|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pTrigger>>2] = $20;
 $21 = HEAP32[$action>>2]|0;
 $22 = ($21|0)==(0);
 $23 = HEAP32[$pTrigger>>2]|0;
 $24 = ($23|0)!=(0|0);
 $or$cond = $22 | $24;
 if (!($or$cond)) {
  HEAP32[$pIdx>>2] = 0;
  HEAP32[$aiCol>>2] = 0;
  HEAP32[$pStep>>2] = 0;
  HEAP32[$pWhere>>2] = 0;
  HEAP32[$pList>>2] = 0;
  HEAP32[$pSelect>>2] = 0;
  HEAP32[$pWhen>>2] = 0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = (_sqlite3FkLocateIndex($25,$26,$27,$pIdx,$aiCol)|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   HEAP32[$0>>2] = 0;
   $269 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($269|0);
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $30 = HEAP32[$i>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = ((($31)) + 20|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($30|0)<($33|0);
   if (!($34)) {
    break;
   }
   ;HEAP32[$tOld>>2]=HEAP32[2640>>2]|0;HEAP32[$tOld+4>>2]=HEAP32[2640+4>>2]|0;
   ;HEAP32[$tNew>>2]=HEAP32[2648>>2]|0;HEAP32[$tNew+4>>2]=HEAP32[2648+4>>2]|0;
   $35 = HEAP32[$aiCol>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$i>>2]|0;
    $38 = HEAP32[$aiCol>>2]|0;
    $39 = (($38) + ($37<<2)|0);
    $40 = HEAP32[$39>>2]|0;
    $44 = $40;
   } else {
    $41 = HEAP32[$3>>2]|0;
    $42 = ((($41)) + 36|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = $43;
   }
   HEAP32[$iFromCol>>2] = $44;
   $45 = HEAP32[$pIdx>>2]|0;
   $46 = ($45|0)!=(0|0);
   if ($46) {
    $47 = HEAP32[$i>>2]|0;
    $48 = HEAP32[$pIdx>>2]|0;
    $49 = ((($48)) + 4|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + ($47<<1)|0);
    $52 = HEAP16[$51>>1]|0;
    $$sink = $52;
   } else {
    $53 = HEAP32[$2>>2]|0;
    $54 = ((($53)) + 32|0);
    $55 = HEAP16[$54>>1]|0;
    $$sink = $55;
   }
   $56 = $$sink << 16 >> 16;
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + (($56*24)|0)|0);
   $61 = HEAP32[$60>>2]|0;
   _sqlite3TokenInit($tToCol,$61);
   $62 = HEAP32[$iFromCol>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = HEAP32[$63>>2]|0;
   $65 = ((($64)) + 4|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($66) + (($62*24)|0)|0);
   $68 = HEAP32[$67>>2]|0;
   _sqlite3TokenInit($tFromCol,$68);
   $69 = HEAP32[$1>>2]|0;
   $70 = HEAP32[$1>>2]|0;
   $71 = HEAP32[$db>>2]|0;
   $72 = (_sqlite3ExprAlloc($71,27,$tOld,0)|0);
   $73 = HEAP32[$db>>2]|0;
   $74 = (_sqlite3ExprAlloc($73,27,$tToCol,0)|0);
   $75 = (_sqlite3PExpr($70,122,$72,$74,0)|0);
   $76 = HEAP32[$db>>2]|0;
   $77 = (_sqlite3ExprAlloc($76,27,$tFromCol,0)|0);
   $78 = (_sqlite3PExpr($69,79,$75,$77,0)|0);
   HEAP32[$pEq>>2] = $78;
   $79 = HEAP32[$db>>2]|0;
   $80 = HEAP32[$pWhere>>2]|0;
   $81 = HEAP32[$pEq>>2]|0;
   $82 = (_sqlite3ExprAnd($79,$80,$81)|0);
   HEAP32[$pWhere>>2] = $82;
   $83 = HEAP32[$4>>2]|0;
   $84 = ($83|0)!=(0|0);
   if ($84) {
    $85 = HEAP32[$1>>2]|0;
    $86 = HEAP32[$1>>2]|0;
    $87 = HEAP32[$db>>2]|0;
    $88 = (_sqlite3ExprAlloc($87,27,$tOld,0)|0);
    $89 = HEAP32[$db>>2]|0;
    $90 = (_sqlite3ExprAlloc($89,27,$tToCol,0)|0);
    $91 = (_sqlite3PExpr($86,122,$88,$90,0)|0);
    $92 = HEAP32[$1>>2]|0;
    $93 = HEAP32[$db>>2]|0;
    $94 = (_sqlite3ExprAlloc($93,27,$tNew,0)|0);
    $95 = HEAP32[$db>>2]|0;
    $96 = (_sqlite3ExprAlloc($95,27,$tToCol,0)|0);
    $97 = (_sqlite3PExpr($92,122,$94,$96,0)|0);
    $98 = (_sqlite3PExpr($85,73,$91,$97,0)|0);
    HEAP32[$pEq>>2] = $98;
    $99 = HEAP32[$db>>2]|0;
    $100 = HEAP32[$pWhen>>2]|0;
    $101 = HEAP32[$pEq>>2]|0;
    $102 = (_sqlite3ExprAnd($99,$100,$101)|0);
    HEAP32[$pWhen>>2] = $102;
   }
   $103 = HEAP32[$action>>2]|0;
   $104 = ($103|0)!=(6);
   if ($104) {
    $105 = HEAP32[$action>>2]|0;
    $106 = ($105|0)!=(9);
    $107 = HEAP32[$4>>2]|0;
    $108 = ($107|0)!=(0|0);
    $or$cond3 = $106 | $108;
    if ($or$cond3) {
     $109 = HEAP32[$action>>2]|0;
     $110 = ($109|0)==(9);
     do {
      if ($110) {
       $111 = HEAP32[$1>>2]|0;
       $112 = HEAP32[$db>>2]|0;
       $113 = (_sqlite3ExprAlloc($112,27,$tNew,0)|0);
       $114 = HEAP32[$db>>2]|0;
       $115 = (_sqlite3ExprAlloc($114,27,$tToCol,0)|0);
       $116 = (_sqlite3PExpr($111,122,$113,$115,0)|0);
       HEAP32[$pNew>>2] = $116;
      } else {
       $117 = HEAP32[$action>>2]|0;
       $118 = ($117|0)==(8);
       if (!($118)) {
        $134 = HEAP32[$1>>2]|0;
        $135 = (_sqlite3PExpr($134,101,0,0,0)|0);
        HEAP32[$pNew>>2] = $135;
        break;
       }
       $119 = HEAP32[$iFromCol>>2]|0;
       $120 = HEAP32[$3>>2]|0;
       $121 = HEAP32[$120>>2]|0;
       $122 = ((($121)) + 4|0);
       $123 = HEAP32[$122>>2]|0;
       $124 = (($123) + (($119*24)|0)|0);
       $125 = ((($124)) + 4|0);
       $126 = HEAP32[$125>>2]|0;
       HEAP32[$pDflt>>2] = $126;
       $127 = HEAP32[$pDflt>>2]|0;
       $128 = ($127|0)!=(0|0);
       if ($128) {
        $129 = HEAP32[$db>>2]|0;
        $130 = HEAP32[$pDflt>>2]|0;
        $131 = (_sqlite3ExprDup($129,$130,0)|0);
        HEAP32[$pNew>>2] = $131;
        break;
       } else {
        $132 = HEAP32[$1>>2]|0;
        $133 = (_sqlite3PExpr($132,101,0,0,0)|0);
        HEAP32[$pNew>>2] = $133;
        break;
       }
      }
     } while(0);
     $136 = HEAP32[$1>>2]|0;
     $137 = HEAP32[$pList>>2]|0;
     $138 = HEAP32[$pNew>>2]|0;
     $139 = (_sqlite3ExprListAppend($136,$137,$138)|0);
     HEAP32[$pList>>2] = $139;
     $140 = HEAP32[$1>>2]|0;
     $141 = HEAP32[$pList>>2]|0;
     _sqlite3ExprListSetName($140,$141,$tFromCol,0);
    }
   }
   $142 = HEAP32[$i>>2]|0;
   $143 = (($142) + 1)|0;
   HEAP32[$i>>2] = $143;
  }
  $144 = HEAP32[$db>>2]|0;
  $145 = HEAP32[$aiCol>>2]|0;
  _sqlite3DbFree($144,$145);
  $146 = HEAP32[$3>>2]|0;
  $147 = HEAP32[$146>>2]|0;
  $148 = HEAP32[$147>>2]|0;
  HEAP32[$zFrom>>2] = $148;
  $149 = HEAP32[$zFrom>>2]|0;
  $150 = (_sqlite3Strlen30($149)|0);
  HEAP32[$nFrom>>2] = $150;
  $151 = HEAP32[$action>>2]|0;
  $152 = ($151|0)==(6);
  if ($152) {
   $153 = HEAP32[$zFrom>>2]|0;
   HEAP32[$tFrom>>2] = $153;
   $154 = HEAP32[$nFrom>>2]|0;
   $155 = ((($tFrom)) + 4|0);
   HEAP32[$155>>2] = $154;
   $156 = HEAP32[$db>>2]|0;
   $157 = (_sqlite3Expr($156,57,19133)|0);
   HEAP32[$pRaise>>2] = $157;
   $158 = HEAP32[$pRaise>>2]|0;
   $159 = ($158|0)!=(0|0);
   if ($159) {
    $160 = HEAP32[$pRaise>>2]|0;
    $161 = ((($160)) + 1|0);
    HEAP8[$161>>0] = 2;
   }
   $162 = HEAP32[$1>>2]|0;
   $163 = HEAP32[$1>>2]|0;
   $164 = HEAP32[$pRaise>>2]|0;
   $165 = (_sqlite3ExprListAppend($163,0,$164)|0);
   $166 = HEAP32[$db>>2]|0;
   $167 = (_sqlite3SrcListAppend($166,0,$tFrom,0)|0);
   $168 = HEAP32[$pWhere>>2]|0;
   $169 = (_sqlite3SelectNew($162,$165,$167,$168,0,0,0,0,0,0)|0);
   HEAP32[$pSelect>>2] = $169;
   HEAP32[$pWhere>>2] = 0;
  }
  $170 = HEAP32[$db>>2]|0;
  $171 = ((($170)) + 248|0);
  $172 = HEAP32[$171>>2]|0;
  $173 = (($172) + 1)|0;
  HEAP32[$171>>2] = $173;
  $174 = HEAP32[$db>>2]|0;
  $175 = HEAP32[$nFrom>>2]|0;
  $176 = (72 + ($175))|0;
  $177 = (($176) + 1)|0;
  $178 = (_sqlite3DbMallocZero($174,$177,0)|0);
  HEAP32[$pTrigger>>2] = $178;
  $179 = HEAP32[$pTrigger>>2]|0;
  $180 = ($179|0)!=(0|0);
  if ($180) {
   $181 = HEAP32[$pTrigger>>2]|0;
   $182 = ((($181)) + 36|0);
   $183 = HEAP32[$pTrigger>>2]|0;
   $184 = ((($183)) + 28|0);
   HEAP32[$184>>2] = $182;
   HEAP32[$pStep>>2] = $182;
   $185 = HEAP32[$pStep>>2]|0;
   $186 = ((($185)) + 36|0);
   $187 = HEAP32[$pStep>>2]|0;
   $188 = ((($187)) + 12|0);
   HEAP32[$188>>2] = $186;
   $189 = HEAP32[$pStep>>2]|0;
   $190 = ((($189)) + 12|0);
   $191 = HEAP32[$190>>2]|0;
   $192 = HEAP32[$zFrom>>2]|0;
   $193 = HEAP32[$nFrom>>2]|0;
   _memcpy(($191|0),($192|0),($193|0))|0;
   $194 = HEAP32[$db>>2]|0;
   $195 = HEAP32[$pWhere>>2]|0;
   $196 = (_sqlite3ExprDup($194,$195,1)|0);
   $197 = HEAP32[$pStep>>2]|0;
   $198 = ((($197)) + 16|0);
   HEAP32[$198>>2] = $196;
   $199 = HEAP32[$db>>2]|0;
   $200 = HEAP32[$pList>>2]|0;
   $201 = (_sqlite3ExprListDup($199,$200,1)|0);
   $202 = HEAP32[$pStep>>2]|0;
   $203 = ((($202)) + 20|0);
   HEAP32[$203>>2] = $201;
   $204 = HEAP32[$db>>2]|0;
   $205 = HEAP32[$pSelect>>2]|0;
   $206 = (_sqlite3SelectDup($204,$205,1)|0);
   $207 = HEAP32[$pStep>>2]|0;
   $208 = ((($207)) + 8|0);
   HEAP32[$208>>2] = $206;
   $209 = HEAP32[$pWhen>>2]|0;
   $210 = ($209|0)!=(0|0);
   if ($210) {
    $211 = HEAP32[$1>>2]|0;
    $212 = HEAP32[$pWhen>>2]|0;
    $213 = (_sqlite3PExpr($211,19,$212,0,0)|0);
    HEAP32[$pWhen>>2] = $213;
    $214 = HEAP32[$db>>2]|0;
    $215 = HEAP32[$pWhen>>2]|0;
    $216 = (_sqlite3ExprDup($214,$215,1)|0);
    $217 = HEAP32[$pTrigger>>2]|0;
    $218 = ((($217)) + 12|0);
    HEAP32[$218>>2] = $216;
   }
  }
  $219 = HEAP32[$db>>2]|0;
  $220 = ((($219)) + 248|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = (($221) + -1)|0;
  HEAP32[$220>>2] = $222;
  $223 = HEAP32[$db>>2]|0;
  $224 = HEAP32[$pWhere>>2]|0;
  _sqlite3ExprDelete($223,$224);
  $225 = HEAP32[$db>>2]|0;
  $226 = HEAP32[$pWhen>>2]|0;
  _sqlite3ExprDelete($225,$226);
  $227 = HEAP32[$db>>2]|0;
  $228 = HEAP32[$pList>>2]|0;
  _sqlite3ExprListDelete($227,$228);
  $229 = HEAP32[$db>>2]|0;
  $230 = HEAP32[$pSelect>>2]|0;
  _sqlite3SelectDelete($229,$230);
  $231 = HEAP32[$db>>2]|0;
  $232 = ((($231)) + 65|0);
  $233 = HEAP8[$232>>0]|0;
  $234 = $233&255;
  $235 = ($234|0)==(1);
  if ($235) {
   $236 = HEAP32[$db>>2]|0;
   $237 = HEAP32[$pTrigger>>2]|0;
   _fkTriggerDelete($236,$237);
   HEAP32[$0>>2] = 0;
   $269 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($269|0);
  }
  $238 = HEAP32[$action>>2]|0;
  switch ($238|0) {
  case 6:  {
   $239 = HEAP32[$pStep>>2]|0;
   HEAP8[$239>>0] = 119;
   break;
  }
  case 9:  {
   $240 = HEAP32[$4>>2]|0;
   $241 = ($240|0)!=(0|0);
   if ($241) {
    label = 38;
   } else {
    $242 = HEAP32[$pStep>>2]|0;
    HEAP8[$242>>0] = 109;
   }
   break;
  }
  default: {
   label = 38;
  }
  }
  if ((label|0) == 38) {
   $243 = HEAP32[$pStep>>2]|0;
   HEAP8[$243>>0] = 110;
  }
  $244 = HEAP32[$pTrigger>>2]|0;
  $245 = HEAP32[$pStep>>2]|0;
  $246 = ((($245)) + 4|0);
  HEAP32[$246>>2] = $244;
  $247 = HEAP32[$2>>2]|0;
  $248 = ((($247)) + 64|0);
  $249 = HEAP32[$248>>2]|0;
  $250 = HEAP32[$pTrigger>>2]|0;
  $251 = ((($250)) + 20|0);
  HEAP32[$251>>2] = $249;
  $252 = HEAP32[$2>>2]|0;
  $253 = ((($252)) + 64|0);
  $254 = HEAP32[$253>>2]|0;
  $255 = HEAP32[$pTrigger>>2]|0;
  $256 = ((($255)) + 24|0);
  HEAP32[$256>>2] = $254;
  $257 = HEAP32[$pTrigger>>2]|0;
  $258 = HEAP32[$iAction>>2]|0;
  $259 = HEAP32[$3>>2]|0;
  $260 = ((($259)) + 28|0);
  $261 = (($260) + ($258<<2)|0);
  HEAP32[$261>>2] = $257;
  $262 = HEAP32[$4>>2]|0;
  $263 = ($262|0)!=(0|0);
  $264 = $263 ? 110 : 109;
  $265 = $264&255;
  $266 = HEAP32[$pTrigger>>2]|0;
  $267 = ((($266)) + 8|0);
  HEAP8[$267>>0] = $265;
 }
 $268 = HEAP32[$pTrigger>>2]|0;
 HEAP32[$0>>2] = $268;
 $269 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($269|0);
}
function _autoIncrementEnd($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $aOp = 0, $db = 0, $iRec = 0, $memId = 0, $p = 0, $pDb = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $p = sp + 24|0;
 $v = sp + 20|0;
 $db = sp + 16|0;
 $aOp = sp + 12|0;
 $pDb = sp + 8|0;
 $iRec = sp + 4|0;
 $memId = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 412|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$db>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  HEAP32[$pDb>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$memId>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3GetTempReg($21)|0);
  HEAP32[$iRec>>2] = $22;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$pDb>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 72|0);
  $31 = HEAP32[$30>>2]|0;
  _sqlite3OpenTable($23,0,$26,$31,55);
  $32 = HEAP32[$v>>2]|0;
  $33 = (_sqlite3VdbeAddOpList($32,5,33536,0)|0);
  HEAP32[$aOp>>2] = $33;
  $34 = HEAP32[$aOp>>2]|0;
  $35 = ($34|0)==(0|0);
  if ($35) {
   label = 5;
   break;
  }
  $36 = HEAP32[$memId>>2]|0;
  $37 = (($36) + 1)|0;
  $38 = HEAP32[$aOp>>2]|0;
  $39 = ((($38)) + 4|0);
  HEAP32[$39>>2] = $37;
  $40 = HEAP32[$memId>>2]|0;
  $41 = (($40) + 1)|0;
  $42 = HEAP32[$aOp>>2]|0;
  $43 = ((($42)) + 20|0);
  $44 = ((($43)) + 8|0);
  HEAP32[$44>>2] = $41;
  $45 = HEAP32[$memId>>2]|0;
  $46 = (($45) - 1)|0;
  $47 = HEAP32[$aOp>>2]|0;
  $48 = ((($47)) + 40|0);
  $49 = ((($48)) + 4|0);
  HEAP32[$49>>2] = $46;
  $50 = HEAP32[$iRec>>2]|0;
  $51 = HEAP32[$aOp>>2]|0;
  $52 = ((($51)) + 40|0);
  $53 = ((($52)) + 12|0);
  HEAP32[$53>>2] = $50;
  $54 = HEAP32[$iRec>>2]|0;
  $55 = HEAP32[$aOp>>2]|0;
  $56 = ((($55)) + 60|0);
  $57 = ((($56)) + 8|0);
  HEAP32[$57>>2] = $54;
  $58 = HEAP32[$memId>>2]|0;
  $59 = (($58) + 1)|0;
  $60 = HEAP32[$aOp>>2]|0;
  $61 = ((($60)) + 60|0);
  $62 = ((($61)) + 12|0);
  HEAP32[$62>>2] = $59;
  $63 = HEAP32[$aOp>>2]|0;
  $64 = ((($63)) + 60|0);
  $65 = ((($64)) + 3|0);
  HEAP8[$65>>0] = 8;
  $66 = HEAP32[$0>>2]|0;
  $67 = HEAP32[$iRec>>2]|0;
  _sqlite3ReleaseTempReg($66,$67);
  $68 = HEAP32[$p>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$p>>2] = $69;
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _updateVirtualTable($pParse,$pSrc,$pTab,$pChanges,$pRowid,$aXRef,$pWhere,$onError) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pTab = $pTab|0;
 $pChanges = $pChanges|0;
 $pRowid = $pRowid|0;
 $aXRef = $aXRef|0;
 $pWhere = $pWhere|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aDummy = 0, $addr = 0, $bOnePass = 0, $db = 0, $ephemTab = 0, $i = 0, $iCsr = 0, $nArg = 0, $pVTab = 0, $pWInfo = 0, $regArg = 0, $regRec = 0, $regRowid = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $v = sp + 56|0;
 $ephemTab = sp + 52|0;
 $i = sp + 48|0;
 $db = sp + 44|0;
 $pVTab = sp + 40|0;
 $pWInfo = sp + 36|0;
 $nArg = sp + 32|0;
 $regArg = sp + 28|0;
 $regRec = sp + 24|0;
 $regRowid = sp + 20|0;
 $iCsr = sp + 16|0;
 $aDummy = sp + 8|0;
 $bOnePass = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $pRowid;
 HEAP32[$5>>2] = $aXRef;
 HEAP32[$6>>2] = $pWhere;
 HEAP32[$7>>2] = $onError;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3GetVTable($13,$14)|0);
 HEAP32[$pVTab>>2] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 34|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = (2 + ($19))|0;
 HEAP32[$nArg>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = ((($22)) + 44|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$iCsr>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 72|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$26>>2] = $28;
 HEAP32[$ephemTab>>2] = $27;
 $29 = HEAP32[$v>>2]|0;
 $30 = HEAP32[$ephemTab>>2]|0;
 $31 = HEAP32[$nArg>>2]|0;
 $32 = (_sqlite3VdbeAddOp2($29,57,$30,$31)|0);
 HEAP32[$addr>>2] = $32;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 76|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + 1)|0;
 HEAP32[$regArg>>2] = $36;
 $37 = HEAP32[$nArg>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 76|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + ($37))|0;
 HEAP32[$39>>2] = $41;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 76|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + 1)|0;
 HEAP32[$43>>2] = $45;
 HEAP32[$regRec>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 76|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + 1)|0;
 HEAP32[$47>>2] = $49;
 HEAP32[$regRowid>>2] = $49;
 $50 = HEAP32[$0>>2]|0;
 $51 = HEAP32[$1>>2]|0;
 $52 = HEAP32[$6>>2]|0;
 $53 = (_sqlite3WhereBegin($50,$51,$52,0,0,4,0)|0);
 HEAP32[$pWInfo>>2] = $53;
 $54 = HEAP32[$pWInfo>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$v>>2]|0;
 $57 = HEAP32[$iCsr>>2]|0;
 $58 = HEAP32[$regArg>>2]|0;
 (_sqlite3VdbeAddOp2($56,103,$57,$58)|0);
 $59 = HEAP32[$4>>2]|0;
 $60 = ($59|0)!=(0|0);
 if ($60) {
  $61 = HEAP32[$0>>2]|0;
  $62 = HEAP32[$4>>2]|0;
  $63 = HEAP32[$regArg>>2]|0;
  $64 = (($63) + 1)|0;
  _sqlite3ExprCode($61,$62,$64);
 } else {
  $65 = HEAP32[$v>>2]|0;
  $66 = HEAP32[$iCsr>>2]|0;
  $67 = HEAP32[$regArg>>2]|0;
  $68 = (($67) + 1)|0;
  (_sqlite3VdbeAddOp2($65,103,$66,$68)|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $69 = HEAP32[$i>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = ((($70)) + 34|0);
  $72 = HEAP16[$71>>1]|0;
  $73 = $72 << 16 >> 16;
  $74 = ($69|0)<($73|0);
  if (!($74)) {
   break;
  }
  $75 = HEAP32[$i>>2]|0;
  $76 = HEAP32[$5>>2]|0;
  $77 = (($76) + ($75<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)>=(0);
  if ($79) {
   $80 = HEAP32[$0>>2]|0;
   $81 = HEAP32[$i>>2]|0;
   $82 = HEAP32[$5>>2]|0;
   $83 = (($82) + ($81<<2)|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$3>>2]|0;
   $86 = ((($85)) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (($87) + (($84*20)|0)|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = HEAP32[$regArg>>2]|0;
   $91 = (($90) + 2)|0;
   $92 = HEAP32[$i>>2]|0;
   $93 = (($91) + ($92))|0;
   _sqlite3ExprCode($80,$89,$93);
  } else {
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$iCsr>>2]|0;
   $96 = HEAP32[$i>>2]|0;
   $97 = HEAP32[$regArg>>2]|0;
   $98 = (($97) + 2)|0;
   $99 = HEAP32[$i>>2]|0;
   $100 = (($98) + ($99))|0;
   (_sqlite3VdbeAddOp3($94,153,$95,$96,$100)|0);
  }
  $101 = HEAP32[$i>>2]|0;
  $102 = (($101) + 1)|0;
  HEAP32[$i>>2] = $102;
 }
 $103 = HEAP32[$pWInfo>>2]|0;
 $104 = (_sqlite3WhereOkOnePass($103,$aDummy)|0);
 HEAP32[$bOnePass>>2] = $104;
 $105 = HEAP32[$bOnePass>>2]|0;
 $106 = ($105|0)!=(0);
 $107 = HEAP32[$v>>2]|0;
 if ($106) {
  $108 = HEAP32[$addr>>2]|0;
  (_sqlite3VdbeChangeToNoop($107,$108)|0);
  $109 = HEAP32[$0>>2]|0;
  $110 = ((($109)) + 416|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ($111|0)==(0|0);
  if ($112) {
   $113 = HEAP32[$0>>2]|0;
   $114 = ((($113)) + 20|0);
   HEAP8[$114>>0] = 0;
  }
 } else {
  $115 = HEAP32[$regArg>>2]|0;
  $116 = HEAP32[$nArg>>2]|0;
  $117 = HEAP32[$regRec>>2]|0;
  (_sqlite3VdbeAddOp3($107,49,$115,$116,$117)|0);
  $118 = HEAP32[$v>>2]|0;
  $119 = HEAP32[$ephemTab>>2]|0;
  $120 = HEAP32[$regRowid>>2]|0;
  (_sqlite3VdbeAddOp2($118,74,$119,$120)|0);
  $121 = HEAP32[$v>>2]|0;
  $122 = HEAP32[$ephemTab>>2]|0;
  $123 = HEAP32[$regRec>>2]|0;
  $124 = HEAP32[$regRowid>>2]|0;
  (_sqlite3VdbeAddOp3($121,75,$122,$123,$124)|0);
 }
 $125 = HEAP32[$bOnePass>>2]|0;
 $126 = ($125|0)==(0);
 L21: do {
  if ($126) {
   $127 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($127);
   $128 = HEAP32[$v>>2]|0;
   $129 = HEAP32[$ephemTab>>2]|0;
   $130 = (_sqlite3VdbeAddOp1($128,108,$129)|0);
   HEAP32[$addr>>2] = $130;
   HEAP32[$i>>2] = 0;
   while(1) {
    $131 = HEAP32[$i>>2]|0;
    $132 = HEAP32[$nArg>>2]|0;
    $133 = ($131|0)<($132|0);
    if (!($133)) {
     break L21;
    }
    $134 = HEAP32[$v>>2]|0;
    $135 = HEAP32[$ephemTab>>2]|0;
    $136 = HEAP32[$i>>2]|0;
    $137 = HEAP32[$regArg>>2]|0;
    $138 = HEAP32[$i>>2]|0;
    $139 = (($137) + ($138))|0;
    (_sqlite3VdbeAddOp3($134,47,$135,$136,$139)|0);
    $140 = HEAP32[$i>>2]|0;
    $141 = (($140) + 1)|0;
    HEAP32[$i>>2] = $141;
   }
  }
 } while(0);
 $142 = HEAP32[$0>>2]|0;
 $143 = HEAP32[$2>>2]|0;
 _sqlite3VtabMakeWritable($142,$143);
 $144 = HEAP32[$v>>2]|0;
 $145 = HEAP32[$nArg>>2]|0;
 $146 = HEAP32[$regArg>>2]|0;
 $147 = HEAP32[$pVTab>>2]|0;
 (_sqlite3VdbeAddOp4($144,12,0,$145,$146,$147,-10)|0);
 $148 = HEAP32[$v>>2]|0;
 $149 = HEAP32[$7>>2]|0;
 $150 = ($149|0)==(10);
 $151 = HEAP32[$7>>2]|0;
 $152 = $150 ? 2 : $151;
 $153 = $152&255;
 _sqlite3VdbeChangeP5($148,$153);
 $154 = HEAP32[$0>>2]|0;
 _sqlite3MayAbort($154);
 $155 = HEAP32[$bOnePass>>2]|0;
 $156 = ($155|0)==(0);
 if ($156) {
  $157 = HEAP32[$v>>2]|0;
  $158 = HEAP32[$ephemTab>>2]|0;
  $159 = HEAP32[$addr>>2]|0;
  $160 = (($159) + 1)|0;
  (_sqlite3VdbeAddOp2($157,7,$158,$160)|0);
  $161 = HEAP32[$v>>2]|0;
  $162 = HEAP32[$addr>>2]|0;
  _sqlite3VdbeJumpHere($161,$162);
  $163 = HEAP32[$v>>2]|0;
  $164 = HEAP32[$ephemTab>>2]|0;
  (_sqlite3VdbeAddOp2($163,61,$164,0)|0);
  STACKTOP = sp;return;
 } else {
  $165 = HEAP32[$pWInfo>>2]|0;
  _sqlite3WhereEnd($165);
  STACKTOP = sp;return;
 }
}
function _sqlite3GenerateConstraintChecks($pParse,$pTab,$aRegIdx,$iDataCur,$iIdxCur,$regNewData,$regOldData,$pkChng,$overrideError,$ignoreDest,$pbMayReplace,$aiChng) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $aRegIdx = $aRegIdx|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $regNewData = $regNewData|0;
 $regOldData = $regOldData|0;
 $pkChng = $pkChng|0;
 $overrideError = $overrideError|0;
 $ignoreDest = $ignoreDest|0;
 $pbMayReplace = $pbMayReplace|0;
 $aiChng = $aiChng|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0;
 var $addrJump = 0, $addrRowidOk = 0, $addrUniqueOk = 0, $allOk = 0, $bAffinityDone = 0, $db = 0, $i = 0, $iField = 0, $iThisCur = 0, $ipkBottom = 0, $ipkTop = 0, $isUpdate = 0, $ix = 0, $nCol = 0, $nPkField = 0, $onError = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $p4 = 0;
 var $pCheck = 0, $pExpr = 0, $pIdx = 0, $pPk = 0, $pTrigger = 0, $pTrigger2 = 0, $regCmp = 0, $regIdx = 0, $regR = 0, $regRowid = 0, $seenReplace = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $x = 0, $x1 = 0, $zMsg = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $3 = sp + 164|0;
 $4 = sp + 160|0;
 $5 = sp + 156|0;
 $6 = sp + 152|0;
 $7 = sp + 183|0;
 $8 = sp + 182|0;
 $9 = sp + 148|0;
 $10 = sp + 144|0;
 $11 = sp + 140|0;
 $v = sp + 136|0;
 $pIdx = sp + 132|0;
 $pPk = sp + 128|0;
 $db = sp + 124|0;
 $i = sp + 120|0;
 $ix = sp + 116|0;
 $nCol = sp + 112|0;
 $onError = sp + 108|0;
 $addr1 = sp + 104|0;
 $seenReplace = sp + 100|0;
 $nPkField = sp + 96|0;
 $ipkTop = sp + 92|0;
 $ipkBottom = sp + 88|0;
 $isUpdate = sp + 181|0;
 $bAffinityDone = sp + 180|0;
 $regRowid = sp + 84|0;
 $zMsg = sp + 80|0;
 $pCheck = sp + 76|0;
 $allOk = sp + 72|0;
 $pExpr = sp + 68|0;
 $zName = sp + 64|0;
 $addrRowidOk = sp + 60|0;
 $pTrigger = sp + 56|0;
 $regIdx = sp + 52|0;
 $regR = sp + 48|0;
 $iThisCur = sp + 44|0;
 $addrUniqueOk = sp + 40|0;
 $iField = sp + 36|0;
 $x = sp + 32|0;
 $x1 = sp + 28|0;
 $addrJump = sp + 24|0;
 $op = sp + 20|0;
 $regCmp = sp + 16|0;
 $p4 = sp + 12|0;
 $pTrigger2 = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $aRegIdx;
 HEAP32[$3>>2] = $iDataCur;
 HEAP32[$4>>2] = $iIdxCur;
 HEAP32[$5>>2] = $regNewData;
 HEAP32[$6>>2] = $regOldData;
 HEAP8[$7>>0] = $pkChng;
 HEAP8[$8>>0] = $overrideError;
 HEAP32[$9>>2] = $ignoreDest;
 HEAP32[$10>>2] = $pbMayReplace;
 HEAP32[$11>>2] = $aiChng;
 HEAP32[$pPk>>2] = 0;
 HEAP32[$seenReplace>>2] = 0;
 HEAP32[$ipkTop>>2] = 0;
 HEAP32[$ipkBottom>>2] = 0;
 HEAP8[$bAffinityDone>>0] = 0;
 HEAP32[$regRowid>>2] = -1;
 $12 = HEAP32[$6>>2]|0;
 $13 = ($12|0)!=(0);
 $14 = $13&1;
 $15 = $14&255;
 HEAP8[$isUpdate>>0] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$db>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = (_sqlite3GetVdbe($18)|0);
 HEAP32[$v>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 34|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 HEAP32[$nCol>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 42|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = $27 & 32;
 $29 = ($28|0)==(0);
 if ($29) {
  HEAP32[$pPk>>2] = 0;
  HEAP32[$nPkField>>2] = 1;
 } else {
  $30 = HEAP32[$1>>2]|0;
  $31 = (_sqlite3PrimaryKeyIndex($30)|0);
  HEAP32[$pPk>>2] = $31;
  $32 = HEAP32[$pPk>>2]|0;
  $33 = ((($32)) + 50|0);
  $34 = HEAP16[$33>>1]|0;
  $35 = $34&65535;
  HEAP32[$nPkField>>2] = $35;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$nCol>>2]|0;
  $38 = ($36|0)<($37|0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 32|0);
  $42 = HEAP16[$41>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = ($39|0)==($43|0);
  L8: do {
   if (!($44)) {
    $45 = HEAP32[$11>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$i>>2]|0;
     $48 = HEAP32[$11>>2]|0;
     $49 = (($48) + ($47<<2)|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = ($50|0)<(0);
     if ($51) {
      break;
     }
    }
    $52 = HEAP32[$i>>2]|0;
    $53 = HEAP32[$1>>2]|0;
    $54 = ((($53)) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + (($52*24)|0)|0);
    $57 = ((($56)) + 20|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    HEAP32[$onError>>2] = $59;
    $60 = HEAP32[$onError>>2]|0;
    $61 = ($60|0)==(0);
    if (!($61)) {
     $62 = HEAP8[$8>>0]|0;
     $63 = $62&255;
     $64 = ($63|0)!=(10);
     if ($64) {
      $65 = HEAP8[$8>>0]|0;
      $66 = $65&255;
      HEAP32[$onError>>2] = $66;
     } else {
      $67 = HEAP32[$onError>>2]|0;
      $68 = ($67|0)==(10);
      if ($68) {
       HEAP32[$onError>>2] = 2;
      }
     }
     $69 = HEAP32[$onError>>2]|0;
     $70 = ($69|0)==(5);
     if ($70) {
      $71 = HEAP32[$i>>2]|0;
      $72 = HEAP32[$1>>2]|0;
      $73 = ((($72)) + 4|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + (($71*24)|0)|0);
      $76 = ((($75)) + 4|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = ($77|0)==(0|0);
      if ($78) {
       HEAP32[$onError>>2] = 2;
      }
     }
     $79 = HEAP32[$onError>>2]|0;
     switch ($79|0) {
     case 2:  {
      $80 = HEAP32[$0>>2]|0;
      _sqlite3MayAbort($80);
      break;
     }
     case 3: case 1:  {
      break;
     }
     case 4:  {
      $99 = HEAP32[$v>>2]|0;
      $100 = HEAP32[$5>>2]|0;
      $101 = (($100) + 1)|0;
      $102 = HEAP32[$i>>2]|0;
      $103 = (($101) + ($102))|0;
      $104 = HEAP32[$9>>2]|0;
      (_sqlite3VdbeAddOp2($99,76,$103,$104)|0);
      break L8;
      break;
     }
     default: {
      $105 = HEAP32[$v>>2]|0;
      $106 = HEAP32[$5>>2]|0;
      $107 = (($106) + 1)|0;
      $108 = HEAP32[$i>>2]|0;
      $109 = (($107) + ($108))|0;
      $110 = (_sqlite3VdbeAddOp1($105,77,$109)|0);
      HEAP32[$addr1>>2] = $110;
      $111 = HEAP32[$0>>2]|0;
      $112 = HEAP32[$i>>2]|0;
      $113 = HEAP32[$1>>2]|0;
      $114 = ((($113)) + 4|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = (($115) + (($112*24)|0)|0);
      $117 = ((($116)) + 4|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = HEAP32[$5>>2]|0;
      $120 = (($119) + 1)|0;
      $121 = HEAP32[$i>>2]|0;
      $122 = (($120) + ($121))|0;
      _sqlite3ExprCode($111,$118,$122);
      $123 = HEAP32[$v>>2]|0;
      $124 = HEAP32[$addr1>>2]|0;
      _sqlite3VdbeJumpHere($123,$124);
      break L8;
     }
     }
     $81 = HEAP32[$db>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = HEAP32[$82>>2]|0;
     $84 = HEAP32[$i>>2]|0;
     $85 = HEAP32[$1>>2]|0;
     $86 = ((($85)) + 4|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = (($87) + (($84*24)|0)|0);
     $89 = HEAP32[$88>>2]|0;
     HEAP32[$vararg_buffer>>2] = $83;
     $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr4>>2] = $89;
     $90 = (_sqlite3MPrintf($81,29922,$vararg_buffer)|0);
     HEAP32[$zMsg>>2] = $90;
     $91 = HEAP32[$v>>2]|0;
     $92 = HEAP32[$onError>>2]|0;
     $93 = HEAP32[$5>>2]|0;
     $94 = (($93) + 1)|0;
     $95 = HEAP32[$i>>2]|0;
     $96 = (($94) + ($95))|0;
     $97 = HEAP32[$zMsg>>2]|0;
     (_sqlite3VdbeAddOp4($91,20,1299,$92,$96,$97,-1)|0);
     $98 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($98,1);
    }
   }
  } while(0);
  $125 = HEAP32[$i>>2]|0;
  $126 = (($125) + 1)|0;
  HEAP32[$i>>2] = $126;
 }
 $127 = HEAP32[$1>>2]|0;
 $128 = ((($127)) + 24|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = ($129|0)!=(0|0);
 L30: do {
  if ($130) {
   $131 = HEAP32[$db>>2]|0;
   $132 = ((($131)) + 24|0);
   $133 = HEAP32[$132>>2]|0;
   $134 = $133 & 8192;
   $135 = ($134|0)==(0);
   if ($135) {
    $136 = HEAP32[$1>>2]|0;
    $137 = ((($136)) + 24|0);
    $138 = HEAP32[$137>>2]|0;
    HEAP32[$pCheck>>2] = $138;
    $139 = HEAP32[$5>>2]|0;
    $140 = (($139) + 1)|0;
    $141 = HEAP32[$0>>2]|0;
    $142 = ((($141)) + 100|0);
    HEAP32[$142>>2] = $140;
    $143 = HEAP8[$8>>0]|0;
    $144 = $143&255;
    $145 = ($144|0)!=(10);
    $146 = HEAP8[$8>>0]|0;
    $147 = $146&255;
    $148 = $145 ? $147 : 2;
    HEAP32[$onError>>2] = $148;
    HEAP32[$i>>2] = 0;
    while(1) {
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$pCheck>>2]|0;
     $151 = HEAP32[$150>>2]|0;
     $152 = ($149|0)<($151|0);
     if (!($152)) {
      break L30;
     }
     $153 = HEAP32[$i>>2]|0;
     $154 = HEAP32[$pCheck>>2]|0;
     $155 = ((($154)) + 4|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = (($156) + (($153*20)|0)|0);
     $158 = HEAP32[$157>>2]|0;
     HEAP32[$pExpr>>2] = $158;
     $159 = HEAP32[$11>>2]|0;
     $160 = ($159|0)!=(0|0);
     if ($160) {
      $161 = HEAP32[$pExpr>>2]|0;
      $162 = HEAP32[$11>>2]|0;
      $163 = HEAP8[$7>>0]|0;
      $164 = $163&255;
      $165 = (_checkConstraintUnchanged($161,$162,$164)|0);
      $166 = ($165|0)!=(0);
      if (!($166)) {
       label = 29;
      }
     } else {
      label = 29;
     }
     if ((label|0) == 29) {
      label = 0;
      $167 = HEAP32[$v>>2]|0;
      $168 = (_sqlite3VdbeMakeLabel($167)|0);
      HEAP32[$allOk>>2] = $168;
      $169 = HEAP32[$0>>2]|0;
      $170 = HEAP32[$pExpr>>2]|0;
      $171 = HEAP32[$allOk>>2]|0;
      _sqlite3ExprIfTrue($169,$170,$171,16);
      $172 = HEAP32[$onError>>2]|0;
      $173 = ($172|0)==(4);
      if ($173) {
       $174 = HEAP32[$v>>2]|0;
       $175 = HEAP32[$9>>2]|0;
       (_sqlite3VdbeGoto($174,$175)|0);
      } else {
       $176 = HEAP32[$i>>2]|0;
       $177 = HEAP32[$pCheck>>2]|0;
       $178 = ((($177)) + 4|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = (($179) + (($176*20)|0)|0);
       $181 = ((($180)) + 4|0);
       $182 = HEAP32[$181>>2]|0;
       HEAP32[$zName>>2] = $182;
       $183 = HEAP32[$zName>>2]|0;
       $184 = ($183|0)==(0|0);
       if ($184) {
        $185 = HEAP32[$1>>2]|0;
        $186 = HEAP32[$185>>2]|0;
        HEAP32[$zName>>2] = $186;
       }
       $187 = HEAP32[$onError>>2]|0;
       $188 = ($187|0)==(5);
       if ($188) {
        HEAP32[$onError>>2] = 2;
       }
       $189 = HEAP32[$0>>2]|0;
       $190 = HEAP32[$onError>>2]|0;
       $191 = HEAP32[$zName>>2]|0;
       _sqlite3HaltConstraint($189,275,$190,$191,0,3);
      }
      $192 = HEAP32[$v>>2]|0;
      $193 = HEAP32[$allOk>>2]|0;
      _sqlite3VdbeResolveLabel($192,$193);
     }
     $194 = HEAP32[$i>>2]|0;
     $195 = (($194) + 1)|0;
     HEAP32[$i>>2] = $195;
    }
   }
  }
 } while(0);
 $196 = HEAP8[$7>>0]|0;
 $197 = $196&255;
 $198 = ($197|0)!=(0);
 $199 = HEAP32[$pPk>>2]|0;
 $200 = ($199|0)==(0|0);
 $or$cond = $198 & $200;
 if ($or$cond) {
  $201 = HEAP32[$v>>2]|0;
  $202 = (_sqlite3VdbeMakeLabel($201)|0);
  HEAP32[$addrRowidOk>>2] = $202;
  $203 = HEAP32[$1>>2]|0;
  $204 = ((($203)) + 43|0);
  $205 = HEAP8[$204>>0]|0;
  $206 = $205&255;
  HEAP32[$onError>>2] = $206;
  $207 = HEAP8[$8>>0]|0;
  $208 = $207&255;
  $209 = ($208|0)!=(10);
  if ($209) {
   $210 = HEAP8[$8>>0]|0;
   $211 = $210&255;
   HEAP32[$onError>>2] = $211;
  } else {
   $212 = HEAP32[$onError>>2]|0;
   $213 = ($212|0)==(10);
   if ($213) {
    HEAP32[$onError>>2] = 2;
   }
  }
  $214 = HEAP8[$isUpdate>>0]|0;
  $215 = ($214<<24>>24)!=(0);
  if ($215) {
   $216 = HEAP32[$v>>2]|0;
   $217 = HEAP32[$5>>2]|0;
   $218 = HEAP32[$addrRowidOk>>2]|0;
   $219 = HEAP32[$6>>2]|0;
   (_sqlite3VdbeAddOp3($216,79,$217,$218,$219)|0);
   $220 = HEAP32[$v>>2]|0;
   _sqlite3VdbeChangeP5($220,-112);
  }
  $221 = HEAP32[$onError>>2]|0;
  $222 = ($221|0)==(5);
  L62: do {
   if ($222) {
    $223 = HEAP8[$8>>0]|0;
    $224 = $223&255;
    $225 = ($224|0)!=(5);
    if ($225) {
     $226 = HEAP32[$1>>2]|0;
     $227 = ((($226)) + 8|0);
     $228 = HEAP32[$227>>2]|0;
     HEAP32[$pIdx>>2] = $228;
     while(1) {
      $229 = HEAP32[$pIdx>>2]|0;
      $230 = ($229|0)!=(0|0);
      if (!($230)) {
       break L62;
      }
      $231 = HEAP32[$pIdx>>2]|0;
      $232 = ((($231)) + 54|0);
      $233 = HEAP8[$232>>0]|0;
      $234 = $233&255;
      $235 = ($234|0)==(4);
      if ($235) {
       break;
      }
      $236 = HEAP32[$pIdx>>2]|0;
      $237 = ((($236)) + 54|0);
      $238 = HEAP8[$237>>0]|0;
      $239 = $238&255;
      $240 = ($239|0)==(3);
      if ($240) {
       break;
      }
      $243 = HEAP32[$pIdx>>2]|0;
      $244 = ((($243)) + 20|0);
      $245 = HEAP32[$244>>2]|0;
      HEAP32[$pIdx>>2] = $245;
     }
     $241 = HEAP32[$v>>2]|0;
     $242 = (_sqlite3VdbeAddOp0($241,13)|0);
     HEAP32[$ipkTop>>2] = $242;
    }
   }
  } while(0);
  $246 = HEAP32[$v>>2]|0;
  $247 = HEAP32[$3>>2]|0;
  $248 = HEAP32[$addrRowidOk>>2]|0;
  $249 = HEAP32[$5>>2]|0;
  (_sqlite3VdbeAddOp3($246,70,$247,$248,$249)|0);
  $250 = HEAP32[$onError>>2]|0;
  switch ($250|0) {
  case 3: case 2: case 1:  {
   label = 55;
   break;
  }
  case 5:  {
   HEAP32[$pTrigger>>2] = 0;
   $254 = HEAP32[$db>>2]|0;
   $255 = ((($254)) + 24|0);
   $256 = HEAP32[$255>>2]|0;
   $257 = $256 & 262144;
   $258 = ($257|0)!=(0);
   if ($258) {
    $259 = HEAP32[$0>>2]|0;
    $260 = HEAP32[$1>>2]|0;
    $261 = (_sqlite3TriggersExist($259,$260,109,0,0)|0);
    HEAP32[$pTrigger>>2] = $261;
   }
   $262 = HEAP32[$pTrigger>>2]|0;
   $263 = ($262|0)!=(0|0);
   if ($263) {
    label = 60;
   } else {
    $264 = HEAP32[$0>>2]|0;
    $265 = HEAP32[$1>>2]|0;
    $266 = (_sqlite3FkRequired($264,$265,0,0)|0);
    $267 = ($266|0)!=(0);
    if ($267) {
     label = 60;
    } else {
     $275 = HEAP32[$1>>2]|0;
     $276 = ((($275)) + 8|0);
     $277 = HEAP32[$276>>2]|0;
     $278 = ($277|0)!=(0|0);
     if ($278) {
      $279 = HEAP32[$0>>2]|0;
      _sqlite3MultiWrite($279);
      $280 = HEAP32[$0>>2]|0;
      $281 = HEAP32[$1>>2]|0;
      $282 = HEAP32[$3>>2]|0;
      $283 = HEAP32[$4>>2]|0;
      _sqlite3GenerateRowIndexDelete($280,$281,$282,$283,0,-1);
     }
    }
   }
   if ((label|0) == 60) {
    $268 = HEAP32[$0>>2]|0;
    _sqlite3MultiWrite($268);
    $269 = HEAP32[$0>>2]|0;
    $270 = HEAP32[$1>>2]|0;
    $271 = HEAP32[$pTrigger>>2]|0;
    $272 = HEAP32[$3>>2]|0;
    $273 = HEAP32[$4>>2]|0;
    $274 = HEAP32[$5>>2]|0;
    _sqlite3GenerateRowDelete($269,$270,$271,$272,$273,$274,1,0,5,1,-1);
   }
   HEAP32[$seenReplace>>2] = 1;
   break;
  }
  case 4:  {
   $284 = HEAP32[$v>>2]|0;
   $285 = HEAP32[$9>>2]|0;
   (_sqlite3VdbeGoto($284,$285)|0);
   break;
  }
  default: {
   HEAP32[$onError>>2] = 2;
   label = 55;
  }
  }
  if ((label|0) == 55) {
   $251 = HEAP32[$0>>2]|0;
   $252 = HEAP32[$onError>>2]|0;
   $253 = HEAP32[$1>>2]|0;
   _sqlite3RowidConstraint($251,$252,$253);
  }
  $286 = HEAP32[$v>>2]|0;
  $287 = HEAP32[$addrRowidOk>>2]|0;
  _sqlite3VdbeResolveLabel($286,$287);
  $288 = HEAP32[$ipkTop>>2]|0;
  $289 = ($288|0)!=(0);
  if ($289) {
   $290 = HEAP32[$v>>2]|0;
   $291 = (_sqlite3VdbeAddOp0($290,13)|0);
   HEAP32[$ipkBottom>>2] = $291;
   $292 = HEAP32[$v>>2]|0;
   $293 = HEAP32[$ipkTop>>2]|0;
   _sqlite3VdbeJumpHere($292,$293);
  }
 }
 HEAP32[$ix>>2] = 0;
 $294 = HEAP32[$1>>2]|0;
 $295 = ((($294)) + 8|0);
 $296 = HEAP32[$295>>2]|0;
 HEAP32[$pIdx>>2] = $296;
 while(1) {
  $297 = HEAP32[$pIdx>>2]|0;
  $298 = ($297|0)!=(0|0);
  if (!($298)) {
   break;
  }
  $299 = HEAP32[$ix>>2]|0;
  $300 = HEAP32[$2>>2]|0;
  $301 = (($300) + ($299<<2)|0);
  $302 = HEAP32[$301>>2]|0;
  $303 = ($302|0)==(0);
  do {
   if (!($303)) {
    $304 = HEAP8[$bAffinityDone>>0]|0;
    $305 = $304&255;
    $306 = ($305|0)==(0);
    if ($306) {
     $307 = HEAP32[$v>>2]|0;
     $308 = HEAP32[$1>>2]|0;
     $309 = HEAP32[$5>>2]|0;
     $310 = (($309) + 1)|0;
     _sqlite3TableAffinity($307,$308,$310);
     HEAP8[$bAffinityDone>>0] = 1;
    }
    $311 = HEAP32[$4>>2]|0;
    $312 = HEAP32[$ix>>2]|0;
    $313 = (($311) + ($312))|0;
    HEAP32[$iThisCur>>2] = $313;
    $314 = HEAP32[$v>>2]|0;
    $315 = (_sqlite3VdbeMakeLabel($314)|0);
    HEAP32[$addrUniqueOk>>2] = $315;
    $316 = HEAP32[$pIdx>>2]|0;
    $317 = ((($316)) + 36|0);
    $318 = HEAP32[$317>>2]|0;
    $319 = ($318|0)!=(0|0);
    if ($319) {
     $320 = HEAP32[$v>>2]|0;
     $321 = HEAP32[$ix>>2]|0;
     $322 = HEAP32[$2>>2]|0;
     $323 = (($322) + ($321<<2)|0);
     $324 = HEAP32[$323>>2]|0;
     (_sqlite3VdbeAddOp2($320,25,0,$324)|0);
     $325 = HEAP32[$5>>2]|0;
     $326 = (($325) + 1)|0;
     $327 = HEAP32[$0>>2]|0;
     $328 = ((($327)) + 100|0);
     HEAP32[$328>>2] = $326;
     $329 = HEAP32[$0>>2]|0;
     $330 = HEAP32[$pIdx>>2]|0;
     $331 = ((($330)) + 36|0);
     $332 = HEAP32[$331>>2]|0;
     $333 = HEAP32[$addrUniqueOk>>2]|0;
     _sqlite3ExprIfFalseDup($329,$332,$333,16);
     $334 = HEAP32[$0>>2]|0;
     $335 = ((($334)) + 100|0);
     HEAP32[$335>>2] = 0;
    }
    $336 = HEAP32[$0>>2]|0;
    $337 = HEAP32[$pIdx>>2]|0;
    $338 = ((($337)) + 52|0);
    $339 = HEAP16[$338>>1]|0;
    $340 = $339&65535;
    $341 = (_sqlite3GetTempRange($336,$340)|0);
    HEAP32[$regIdx>>2] = $341;
    HEAP32[$i>>2] = 0;
    while(1) {
     $342 = HEAP32[$i>>2]|0;
     $343 = HEAP32[$pIdx>>2]|0;
     $344 = ((($343)) + 52|0);
     $345 = HEAP16[$344>>1]|0;
     $346 = $345&65535;
     $347 = ($342|0)<($346|0);
     if (!($347)) {
      break;
     }
     $348 = HEAP32[$i>>2]|0;
     $349 = HEAP32[$pIdx>>2]|0;
     $350 = ((($349)) + 4|0);
     $351 = HEAP32[$350>>2]|0;
     $352 = (($351) + ($348<<1)|0);
     $353 = HEAP16[$352>>1]|0;
     $354 = $353 << 16 >> 16;
     HEAP32[$iField>>2] = $354;
     $355 = HEAP32[$iField>>2]|0;
     $356 = ($355|0)==(-2);
     do {
      if ($356) {
       $357 = HEAP32[$5>>2]|0;
       $358 = (($357) + 1)|0;
       $359 = HEAP32[$0>>2]|0;
       $360 = ((($359)) + 100|0);
       HEAP32[$360>>2] = $358;
       $361 = HEAP32[$0>>2]|0;
       $362 = HEAP32[$i>>2]|0;
       $363 = HEAP32[$pIdx>>2]|0;
       $364 = ((($363)) + 40|0);
       $365 = HEAP32[$364>>2]|0;
       $366 = ((($365)) + 4|0);
       $367 = HEAP32[$366>>2]|0;
       $368 = (($367) + (($362*20)|0)|0);
       $369 = HEAP32[$368>>2]|0;
       $370 = HEAP32[$regIdx>>2]|0;
       $371 = HEAP32[$i>>2]|0;
       $372 = (($370) + ($371))|0;
       _sqlite3ExprCodeCopy($361,$369,$372);
       $373 = HEAP32[$0>>2]|0;
       $374 = ((($373)) + 100|0);
       HEAP32[$374>>2] = 0;
      } else {
       $375 = HEAP32[$iField>>2]|0;
       $376 = ($375|0)==(-1);
       if ($376) {
        label = 80;
       } else {
        $377 = HEAP32[$iField>>2]|0;
        $378 = HEAP32[$1>>2]|0;
        $379 = ((($378)) + 32|0);
        $380 = HEAP16[$379>>1]|0;
        $381 = $380 << 16 >> 16;
        $382 = ($377|0)==($381|0);
        if ($382) {
         label = 80;
        } else {
         $397 = HEAP32[$iField>>2]|0;
         $398 = HEAP32[$5>>2]|0;
         $399 = (($397) + ($398))|0;
         $400 = (($399) + 1)|0;
         HEAP32[$x>>2] = $400;
        }
       }
       if ((label|0) == 80) {
        label = 0;
        $383 = HEAP32[$regRowid>>2]|0;
        $384 = HEAP32[$regIdx>>2]|0;
        $385 = HEAP32[$i>>2]|0;
        $386 = (($384) + ($385))|0;
        $387 = ($383|0)==($386|0);
        if ($387) {
         break;
        }
        $388 = HEAP32[$5>>2]|0;
        HEAP32[$x>>2] = $388;
        $389 = HEAP32[$pIdx>>2]|0;
        $390 = ((($389)) + 36|0);
        $391 = HEAP32[$390>>2]|0;
        $392 = ($391|0)!=(0|0);
        if ($392) {
         $396 = -1;
        } else {
         $393 = HEAP32[$regIdx>>2]|0;
         $394 = HEAP32[$i>>2]|0;
         $395 = (($393) + ($394))|0;
         $396 = $395;
        }
        HEAP32[$regRowid>>2] = $396;
       }
       $401 = HEAP32[$v>>2]|0;
       $402 = HEAP32[$iField>>2]|0;
       $403 = ($402|0)<(0);
       $404 = $403 ? 32 : 31;
       $405 = HEAP32[$x>>2]|0;
       $406 = HEAP32[$regIdx>>2]|0;
       $407 = HEAP32[$i>>2]|0;
       $408 = (($406) + ($407))|0;
       (_sqlite3VdbeAddOp2($401,$404,$405,$408)|0);
      }
     } while(0);
     $409 = HEAP32[$i>>2]|0;
     $410 = (($409) + 1)|0;
     HEAP32[$i>>2] = $410;
    }
    $411 = HEAP32[$v>>2]|0;
    $412 = HEAP32[$regIdx>>2]|0;
    $413 = HEAP32[$pIdx>>2]|0;
    $414 = ((($413)) + 52|0);
    $415 = HEAP16[$414>>1]|0;
    $416 = $415&65535;
    $417 = HEAP32[$ix>>2]|0;
    $418 = HEAP32[$2>>2]|0;
    $419 = (($418) + ($417<<2)|0);
    $420 = HEAP32[$419>>2]|0;
    (_sqlite3VdbeAddOp3($411,49,$412,$416,$420)|0);
    $421 = HEAP32[$0>>2]|0;
    $422 = HEAP32[$regIdx>>2]|0;
    $423 = HEAP32[$pIdx>>2]|0;
    $424 = ((($423)) + 52|0);
    $425 = HEAP16[$424>>1]|0;
    $426 = $425&65535;
    _sqlite3ExprCacheAffinityChange($421,$422,$426);
    $427 = HEAP8[$isUpdate>>0]|0;
    $428 = $427&255;
    $429 = ($428|0)!=(0);
    if ($429) {
     $430 = HEAP32[$pPk>>2]|0;
     $431 = HEAP32[$pIdx>>2]|0;
     $432 = ($430|0)==($431|0);
     if ($432) {
      $433 = HEAP8[$7>>0]|0;
      $434 = $433&255;
      $435 = ($434|0)==(0);
      if ($435) {
       $436 = HEAP32[$v>>2]|0;
       $437 = HEAP32[$addrUniqueOk>>2]|0;
       _sqlite3VdbeResolveLabel($436,$437);
       break;
      }
     }
    }
    $438 = HEAP32[$pIdx>>2]|0;
    $439 = ((($438)) + 54|0);
    $440 = HEAP8[$439>>0]|0;
    $441 = $440&255;
    HEAP32[$onError>>2] = $441;
    $442 = HEAP32[$onError>>2]|0;
    $443 = ($442|0)==(0);
    if ($443) {
     $444 = HEAP32[$0>>2]|0;
     $445 = HEAP32[$regIdx>>2]|0;
     $446 = HEAP32[$pIdx>>2]|0;
     $447 = ((($446)) + 52|0);
     $448 = HEAP16[$447>>1]|0;
     $449 = $448&65535;
     _sqlite3ReleaseTempRange($444,$445,$449);
     $450 = HEAP32[$v>>2]|0;
     $451 = HEAP32[$addrUniqueOk>>2]|0;
     _sqlite3VdbeResolveLabel($450,$451);
     break;
    }
    $452 = HEAP8[$8>>0]|0;
    $453 = $452&255;
    $454 = ($453|0)!=(10);
    if ($454) {
     $455 = HEAP8[$8>>0]|0;
     $456 = $455&255;
     HEAP32[$onError>>2] = $456;
    } else {
     $457 = HEAP32[$onError>>2]|0;
     $458 = ($457|0)==(10);
     if ($458) {
      HEAP32[$onError>>2] = 2;
     }
    }
    $459 = HEAP32[$v>>2]|0;
    $460 = HEAP32[$iThisCur>>2]|0;
    $461 = HEAP32[$addrUniqueOk>>2]|0;
    $462 = HEAP32[$regIdx>>2]|0;
    $463 = HEAP32[$pIdx>>2]|0;
    $464 = ((($463)) + 50|0);
    $465 = HEAP16[$464>>1]|0;
    $466 = $465&65535;
    (_sqlite3VdbeAddOp4Int($459,67,$460,$461,$462,$466)|0);
    $467 = HEAP32[$pIdx>>2]|0;
    $468 = HEAP32[$pPk>>2]|0;
    $469 = ($467|0)==($468|0);
    if ($469) {
     $470 = HEAP32[$regIdx>>2]|0;
     $474 = $470;
    } else {
     $471 = HEAP32[$0>>2]|0;
     $472 = HEAP32[$nPkField>>2]|0;
     $473 = (_sqlite3GetTempRange($471,$472)|0);
     $474 = $473;
    }
    HEAP32[$regR>>2] = $474;
    $475 = HEAP8[$isUpdate>>0]|0;
    $476 = $475&255;
    $477 = ($476|0)!=(0);
    $478 = HEAP32[$onError>>2]|0;
    $479 = ($478|0)==(5);
    $or$cond3 = $477 | $479;
    L137: do {
     if ($or$cond3) {
      $480 = HEAP32[$1>>2]|0;
      $481 = ((($480)) + 42|0);
      $482 = HEAP8[$481>>0]|0;
      $483 = $482&255;
      $484 = $483 & 32;
      $485 = ($484|0)==(0);
      if ($485) {
       $486 = HEAP32[$v>>2]|0;
       $487 = HEAP32[$iThisCur>>2]|0;
       $488 = HEAP32[$regR>>2]|0;
       (_sqlite3VdbeAddOp2($486,113,$487,$488)|0);
       $489 = HEAP8[$isUpdate>>0]|0;
       $490 = ($489<<24>>24)!=(0);
       if (!($490)) {
        break;
       }
       $491 = HEAP32[$v>>2]|0;
       $492 = HEAP32[$regR>>2]|0;
       $493 = HEAP32[$addrUniqueOk>>2]|0;
       $494 = HEAP32[$6>>2]|0;
       (_sqlite3VdbeAddOp3($491,79,$492,$493,$494)|0);
       $495 = HEAP32[$v>>2]|0;
       _sqlite3VdbeChangeP5($495,-112);
       break;
      }
      $496 = HEAP32[$pIdx>>2]|0;
      $497 = HEAP32[$pPk>>2]|0;
      $498 = ($496|0)!=($497|0);
      L143: do {
       if ($498) {
        HEAP32[$i>>2] = 0;
        while(1) {
         $499 = HEAP32[$i>>2]|0;
         $500 = HEAP32[$pPk>>2]|0;
         $501 = ((($500)) + 50|0);
         $502 = HEAP16[$501>>1]|0;
         $503 = $502&65535;
         $504 = ($499|0)<($503|0);
         if (!($504)) {
          break L143;
         }
         $505 = HEAP32[$pIdx>>2]|0;
         $506 = HEAP32[$i>>2]|0;
         $507 = HEAP32[$pPk>>2]|0;
         $508 = ((($507)) + 4|0);
         $509 = HEAP32[$508>>2]|0;
         $510 = (($509) + ($506<<1)|0);
         $511 = HEAP16[$510>>1]|0;
         $512 = (_sqlite3ColumnOfIndex($505,$511)|0);
         $513 = $512 << 16 >> 16;
         HEAP32[$x1>>2] = $513;
         $514 = HEAP32[$v>>2]|0;
         $515 = HEAP32[$iThisCur>>2]|0;
         $516 = HEAP32[$x1>>2]|0;
         $517 = HEAP32[$regR>>2]|0;
         $518 = HEAP32[$i>>2]|0;
         $519 = (($517) + ($518))|0;
         (_sqlite3VdbeAddOp3($514,47,$515,$516,$519)|0);
         $520 = HEAP32[$i>>2]|0;
         $521 = (($520) + 1)|0;
         HEAP32[$i>>2] = $521;
        }
       }
      } while(0);
      $522 = HEAP8[$isUpdate>>0]|0;
      $523 = ($522<<24>>24)!=(0);
      if ($523) {
       $524 = HEAP32[$v>>2]|0;
       $525 = (_sqlite3VdbeCurrentAddr($524)|0);
       $526 = HEAP32[$pPk>>2]|0;
       $527 = ((($526)) + 50|0);
       $528 = HEAP16[$527>>1]|0;
       $529 = $528&65535;
       $530 = (($525) + ($529))|0;
       HEAP32[$addrJump>>2] = $530;
       HEAP32[$op>>2] = 78;
       $531 = HEAP32[$pIdx>>2]|0;
       $532 = ((($531)) + 55|0);
       $533 = HEAP8[$532>>0]|0;
       $534 = $533 & 3;
       $535 = $534&255;
       $536 = ($535|0)==(2);
       $537 = HEAP32[$regIdx>>2]|0;
       $538 = HEAP32[$regR>>2]|0;
       $539 = $536 ? $537 : $538;
       HEAP32[$regCmp>>2] = $539;
       HEAP32[$i>>2] = 0;
       while(1) {
        $540 = HEAP32[$i>>2]|0;
        $541 = HEAP32[$pPk>>2]|0;
        $542 = ((($541)) + 50|0);
        $543 = HEAP16[$542>>1]|0;
        $544 = $543&65535;
        $545 = ($540|0)<($544|0);
        if (!($545)) {
         break L137;
        }
        $546 = HEAP32[$0>>2]|0;
        $547 = HEAP32[$i>>2]|0;
        $548 = HEAP32[$pPk>>2]|0;
        $549 = ((($548)) + 32|0);
        $550 = HEAP32[$549>>2]|0;
        $551 = (($550) + ($547<<2)|0);
        $552 = HEAP32[$551>>2]|0;
        $553 = (_sqlite3LocateCollSeq($546,$552)|0);
        HEAP32[$p4>>2] = $553;
        $554 = HEAP32[$i>>2]|0;
        $555 = HEAP32[$pPk>>2]|0;
        $556 = ((($555)) + 4|0);
        $557 = HEAP32[$556>>2]|0;
        $558 = (($557) + ($554<<1)|0);
        $559 = HEAP16[$558>>1]|0;
        $560 = $559 << 16 >> 16;
        HEAP32[$x1>>2] = $560;
        $561 = HEAP32[$i>>2]|0;
        $562 = HEAP32[$pPk>>2]|0;
        $563 = ((($562)) + 50|0);
        $564 = HEAP16[$563>>1]|0;
        $565 = $564&65535;
        $566 = (($565) - 1)|0;
        $567 = ($561|0)==($566|0);
        if ($567) {
         $568 = HEAP32[$addrUniqueOk>>2]|0;
         HEAP32[$addrJump>>2] = $568;
         HEAP32[$op>>2] = 79;
        }
        $569 = HEAP32[$v>>2]|0;
        $570 = HEAP32[$op>>2]|0;
        $571 = HEAP32[$6>>2]|0;
        $572 = (($571) + 1)|0;
        $573 = HEAP32[$x1>>2]|0;
        $574 = (($572) + ($573))|0;
        $575 = HEAP32[$addrJump>>2]|0;
        $576 = HEAP32[$regCmp>>2]|0;
        $577 = HEAP32[$i>>2]|0;
        $578 = (($576) + ($577))|0;
        $579 = HEAP32[$p4>>2]|0;
        (_sqlite3VdbeAddOp4($569,$570,$574,$575,$578,$579,-4)|0);
        $580 = HEAP32[$v>>2]|0;
        _sqlite3VdbeChangeP5($580,-112);
        $581 = HEAP32[$i>>2]|0;
        $582 = (($581) + 1)|0;
        HEAP32[$i>>2] = $582;
       }
      }
     }
    } while(0);
    $583 = HEAP32[$onError>>2]|0;
    switch ($583|0) {
    case 3: case 2: case 1:  {
     $584 = HEAP32[$0>>2]|0;
     $585 = HEAP32[$onError>>2]|0;
     $586 = HEAP32[$pIdx>>2]|0;
     _sqlite3UniqueConstraint($584,$585,$586);
     break;
    }
    case 4:  {
     $587 = HEAP32[$v>>2]|0;
     $588 = HEAP32[$9>>2]|0;
     (_sqlite3VdbeGoto($587,$588)|0);
     break;
    }
    default: {
     HEAP32[$pTrigger2>>2] = 0;
     $589 = HEAP32[$0>>2]|0;
     _sqlite3MultiWrite($589);
     $590 = HEAP32[$db>>2]|0;
     $591 = ((($590)) + 24|0);
     $592 = HEAP32[$591>>2]|0;
     $593 = $592 & 262144;
     $594 = ($593|0)!=(0);
     if ($594) {
      $595 = HEAP32[$0>>2]|0;
      $596 = HEAP32[$1>>2]|0;
      $597 = (_sqlite3TriggersExist($595,$596,109,0,0)|0);
      HEAP32[$pTrigger2>>2] = $597;
     }
     $598 = HEAP32[$0>>2]|0;
     $599 = HEAP32[$1>>2]|0;
     $600 = HEAP32[$pTrigger2>>2]|0;
     $601 = HEAP32[$3>>2]|0;
     $602 = HEAP32[$4>>2]|0;
     $603 = HEAP32[$regR>>2]|0;
     $604 = HEAP32[$nPkField>>2]|0;
     $605 = $604&65535;
     $606 = HEAP32[$pIdx>>2]|0;
     $607 = HEAP32[$pPk>>2]|0;
     $608 = ($606|0)==($607|0);
     $609 = $608 ? 1 : 0;
     $610 = $609&255;
     _sqlite3GenerateRowDelete($598,$599,$600,$601,$602,$603,$605,0,5,$610,-1);
     HEAP32[$seenReplace>>2] = 1;
    }
    }
    $611 = HEAP32[$v>>2]|0;
    $612 = HEAP32[$addrUniqueOk>>2]|0;
    _sqlite3VdbeResolveLabel($611,$612);
    $613 = HEAP32[$0>>2]|0;
    $614 = HEAP32[$regIdx>>2]|0;
    $615 = HEAP32[$pIdx>>2]|0;
    $616 = ((($615)) + 52|0);
    $617 = HEAP16[$616>>1]|0;
    $618 = $617&65535;
    _sqlite3ReleaseTempRange($613,$614,$618);
    $619 = HEAP32[$regR>>2]|0;
    $620 = HEAP32[$regIdx>>2]|0;
    $621 = ($619|0)!=($620|0);
    if ($621) {
     $622 = HEAP32[$0>>2]|0;
     $623 = HEAP32[$regR>>2]|0;
     $624 = HEAP32[$nPkField>>2]|0;
     _sqlite3ReleaseTempRange($622,$623,$624);
    }
   }
  } while(0);
  $625 = HEAP32[$pIdx>>2]|0;
  $626 = ((($625)) + 20|0);
  $627 = HEAP32[$626>>2]|0;
  HEAP32[$pIdx>>2] = $627;
  $628 = HEAP32[$ix>>2]|0;
  $629 = (($628) + 1)|0;
  HEAP32[$ix>>2] = $629;
 }
 $630 = HEAP32[$ipkTop>>2]|0;
 $631 = ($630|0)!=(0);
 if (!($631)) {
  $637 = HEAP32[$seenReplace>>2]|0;
  $638 = HEAP32[$10>>2]|0;
  HEAP32[$638>>2] = $637;
  STACKTOP = sp;return;
 }
 $632 = HEAP32[$v>>2]|0;
 $633 = HEAP32[$ipkTop>>2]|0;
 $634 = (($633) + 1)|0;
 (_sqlite3VdbeGoto($632,$634)|0);
 $635 = HEAP32[$v>>2]|0;
 $636 = HEAP32[$ipkBottom>>2]|0;
 _sqlite3VdbeJumpHere($635,$636);
 $637 = HEAP32[$seenReplace>>2]|0;
 $638 = HEAP32[$10>>2]|0;
 HEAP32[$638>>2] = $637;
 STACKTOP = sp;return;
}
function _sqlite3CompleteInsertion($pParse,$pTab,$iDataCur,$iIdxCur,$regNewData,$aRegIdx,$isUpdate,$appendBias,$useSeekResult) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $regNewData = $regNewData|0;
 $aRegIdx = $aRegIdx|0;
 $isUpdate = $isUpdate|0;
 $appendBias = $appendBias|0;
 $useSeekResult = $useSeekResult|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bAffinityDone = 0, $i = 0, $pIdx = 0, $pik_flags = 0, $regData = 0, $regRec = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $v = sp + 16|0;
 $pIdx = sp + 12|0;
 $pik_flags = sp + 57|0;
 $regData = sp + 8|0;
 $regRec = sp + 4|0;
 $i = sp;
 $bAffinityDone = sp + 56|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $iIdxCur;
 HEAP32[$4>>2] = $regNewData;
 HEAP32[$5>>2] = $aRegIdx;
 HEAP32[$6>>2] = $isUpdate;
 HEAP32[$7>>2] = $appendBias;
 HEAP32[$8>>2] = $useSeekResult;
 HEAP8[$bAffinityDone>>0] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 HEAP32[$i>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pIdx>>2] = $13;
 while(1) {
  $14 = HEAP32[$pIdx>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$5>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)==(0);
  if (!($20)) {
   HEAP8[$bAffinityDone>>0] = 1;
   $21 = HEAP32[$pIdx>>2]|0;
   $22 = ((($21)) + 36|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)!=(0|0);
   if ($24) {
    $25 = HEAP32[$v>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    $27 = HEAP32[$5>>2]|0;
    $28 = (($27) + ($26<<2)|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$v>>2]|0;
    $31 = (_sqlite3VdbeCurrentAddr($30)|0);
    $32 = (($31) + 2)|0;
    (_sqlite3VdbeAddOp2($25,76,$29,$32)|0);
   }
   $33 = HEAP32[$v>>2]|0;
   $34 = HEAP32[$3>>2]|0;
   $35 = HEAP32[$i>>2]|0;
   $36 = (($34) + ($35))|0;
   $37 = HEAP32[$i>>2]|0;
   $38 = HEAP32[$5>>2]|0;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   (_sqlite3VdbeAddOp2($33,110,$36,$40)|0);
   HEAP8[$pik_flags>>0] = 0;
   $41 = HEAP32[$8>>2]|0;
   $42 = ($41|0)!=(0);
   $$ = $42 ? 16 : 0;
   HEAP8[$pik_flags>>0] = $$;
   $43 = HEAP32[$pIdx>>2]|0;
   $44 = ((($43)) + 55|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = $45 & 3;
   $47 = $46&255;
   $48 = ($47|0)==(2);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 42|0);
    $51 = HEAP8[$50>>0]|0;
    $52 = $51&255;
    $53 = $52 & 32;
    $54 = ($53|0)==(0);
    if (!($54)) {
     $55 = HEAP8[$pik_flags>>0]|0;
     $56 = $55&255;
     $57 = $56 | 1;
     $58 = $57&255;
     HEAP8[$pik_flags>>0] = $58;
    }
   }
   $59 = HEAP32[$v>>2]|0;
   $60 = HEAP8[$pik_flags>>0]|0;
   _sqlite3VdbeChangeP5($59,$60);
  }
  $61 = HEAP32[$pIdx>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[$pIdx>>2] = $63;
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 42|0);
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = $69 & 32;
 $71 = ($70|0)==(0);
 if (!($71)) {
  STACKTOP = sp;return;
 }
 $72 = HEAP32[$4>>2]|0;
 $73 = (($72) + 1)|0;
 HEAP32[$regData>>2] = $73;
 $74 = HEAP32[$0>>2]|0;
 $75 = (_sqlite3GetTempReg($74)|0);
 HEAP32[$regRec>>2] = $75;
 $76 = HEAP32[$v>>2]|0;
 $77 = HEAP32[$regData>>2]|0;
 $78 = HEAP32[$1>>2]|0;
 $79 = ((($78)) + 34|0);
 $80 = HEAP16[$79>>1]|0;
 $81 = $80 << 16 >> 16;
 $82 = HEAP32[$regRec>>2]|0;
 (_sqlite3VdbeAddOp3($76,49,$77,$81,$82)|0);
 $83 = HEAP8[$bAffinityDone>>0]|0;
 $84 = ($83<<24>>24)!=(0);
 if (!($84)) {
  $85 = HEAP32[$v>>2]|0;
  $86 = HEAP32[$1>>2]|0;
  _sqlite3TableAffinity($85,$86,0);
 }
 $87 = HEAP32[$0>>2]|0;
 $88 = HEAP32[$regData>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 34|0);
 $91 = HEAP16[$90>>1]|0;
 $92 = $91 << 16 >> 16;
 _sqlite3ExprCacheAffinityChange($87,$88,$92);
 $93 = HEAP32[$0>>2]|0;
 $94 = ((($93)) + 18|0);
 $95 = HEAP8[$94>>0]|0;
 $96 = ($95<<24>>24)!=(0);
 if ($96) {
  HEAP8[$pik_flags>>0] = 0;
 } else {
  HEAP8[$pik_flags>>0] = 1;
  $97 = HEAP32[$6>>2]|0;
  $98 = ($97|0)!=(0);
  $99 = $98 ? 4 : 2;
  $100 = HEAP8[$pik_flags>>0]|0;
  $101 = $100&255;
  $102 = $101 | $99;
  $103 = $102&255;
  HEAP8[$pik_flags>>0] = $103;
 }
 $104 = HEAP32[$7>>2]|0;
 $105 = ($104|0)!=(0);
 if ($105) {
  $106 = HEAP8[$pik_flags>>0]|0;
  $107 = $106&255;
  $108 = $107 | 8;
  $109 = $108&255;
  HEAP8[$pik_flags>>0] = $109;
 }
 $110 = HEAP32[$8>>2]|0;
 $111 = ($110|0)!=(0);
 if ($111) {
  $112 = HEAP8[$pik_flags>>0]|0;
  $113 = $112&255;
  $114 = $113 | 16;
  $115 = $114&255;
  HEAP8[$pik_flags>>0] = $115;
 }
 $116 = HEAP32[$v>>2]|0;
 $117 = HEAP32[$2>>2]|0;
 $118 = HEAP32[$regRec>>2]|0;
 $119 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeAddOp3($116,75,$117,$118,$119)|0);
 $120 = HEAP32[$0>>2]|0;
 $121 = ((($120)) + 18|0);
 $122 = HEAP8[$121>>0]|0;
 $123 = ($122<<24>>24)!=(0);
 if (!($123)) {
  $124 = HEAP32[$v>>2]|0;
  $125 = HEAP32[$1>>2]|0;
  $126 = HEAP32[$125>>2]|0;
  _sqlite3VdbeChangeP4($124,-1,$126,0);
 }
 $127 = HEAP32[$v>>2]|0;
 $128 = HEAP8[$pik_flags>>0]|0;
 _sqlite3VdbeChangeP5($127,$128);
 STACKTOP = sp;return;
}
function _checkConstraintUnchanged($pExpr,$aiChng,$chngRowid) {
 $pExpr = $pExpr|0;
 $aiChng = $aiChng|0;
 $chngRowid = $chngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $aiChng;
 HEAP32[$2>>2] = $chngRowid;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = ((($w)) + 20|0);
 HEAP8[$3>>0] = 0;
 $4 = ((($w)) + 4|0);
 HEAP32[$4>>2] = 195;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($w)) + 24|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$7)|0);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $15 = ((($w)) + 20|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  $18 = $17 ^ 1;
  $19 = $18&1;
  STACKTOP = sp;return ($19|0);
 }
 $10 = ((($w)) + 20|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & -3;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 $15 = ((($w)) + 20|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)!=(0);
 $18 = $17 ^ 1;
 $19 = $18&1;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3RowidConstraint($pParse,$onError,$pTab) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $zMsg = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5 << 16 >> 16;
 $7 = ($6|0)>=(0);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 if ($7) {
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 32|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14 << 16 >> 16;
  $16 = HEAP32[$2>>2]|0;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + (($15*24)|0)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$vararg_buffer>>2] = $11;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $20;
  $21 = (_sqlite3MPrintf($9,29922,$vararg_buffer)|0);
  HEAP32[$zMsg>>2] = $21;
  HEAP32[$rc>>2] = 1555;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMsg>>2]|0;
  _sqlite3HaltConstraint($23,$24,$25,$26,-1,2);
  STACKTOP = sp;return;
 } else {
  HEAP32[$vararg_buffer2>>2] = $11;
  $22 = (_sqlite3MPrintf($9,33611,$vararg_buffer2)|0);
  HEAP32[$zMsg>>2] = $22;
  HEAP32[$rc>>2] = 2579;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMsg>>2]|0;
  _sqlite3HaltConstraint($23,$24,$25,$26,-1,2);
  STACKTOP = sp;return;
 }
}
function _sqlite3MultiWrite($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 416|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ((($5)) + 416|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $7;
 } else {
  $8 = $5;
 }
 HEAP32[$pToplevel>>2] = $8;
 $9 = HEAP32[$pToplevel>>2]|0;
 $10 = ((($9)) + 20|0);
 HEAP8[$10>>0] = 1;
 STACKTOP = sp;return;
}
function _checkConstraintExprNode($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(152);
 if (!($5)) {
  STACKTOP = sp;return 0;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 $10 = ($9|0)>=(0);
 if (!($10)) {
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 20|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29&255;
  $31 = $30 | 2;
  $32 = $31&255;
  HEAP8[$28>>0] = $32;
  STACKTOP = sp;return 0;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 32|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<2)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)>=(0);
 if (!($20)) {
  STACKTOP = sp;return 0;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 20|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $24 | 1;
 $26 = $25&255;
 HEAP8[$22>>0] = $26;
 STACKTOP = sp;return 0;
}
function _xferOptimization($pParse,$pDest,$pSelect,$onError,$iDbDest) {
 $pParse = $pParse|0;
 $pDest = $pDest|0;
 $pSelect = $pSelect|0;
 $onError = $onError|0;
 $iDbDest = $iDbDest|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0;
 var $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0;
 var $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0;
 var $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0;
 var $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0;
 var $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0;
 var $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0;
 var $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0;
 var $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0;
 var $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addr1 = 0, $addr2 = 0, $db = 0, $destHasUniqueIdx = 0, $emptyDestTest = 0, $emptySrcTest = 0, $i = 0, $iDbSrc = 0, $iDest = 0, $iSrc = 0, $idxInsFlags = 0, $or$cond = 0, $or$cond4 = 0, $pDestCol = 0, $pDestIdx = 0, $pEList = 0, $pItem = 0, $pSrc = 0, $pSrcCol = 0;
 var $pSrcIdx = 0, $regAutoinc = 0, $regData = 0, $regRowid = 0, $v = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $db = sp + 84|0;
 $pEList = sp + 80|0;
 $pSrc = sp + 76|0;
 $pSrcIdx = sp + 72|0;
 $pDestIdx = sp + 68|0;
 $pItem = sp + 64|0;
 $i = sp + 60|0;
 $iDbSrc = sp + 56|0;
 $iSrc = sp + 52|0;
 $iDest = sp + 48|0;
 $addr1 = sp + 44|0;
 $addr2 = sp + 40|0;
 $emptyDestTest = sp + 36|0;
 $emptySrcTest = sp + 32|0;
 $v = sp + 28|0;
 $regAutoinc = sp + 24|0;
 $destHasUniqueIdx = sp + 20|0;
 $regData = sp + 16|0;
 $regRowid = sp + 12|0;
 $pDestCol = sp + 8|0;
 $pSrcCol = sp + 4|0;
 $idxInsFlags = sp + 112|0;
 $zColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$3>>2] = $pSelect;
 HEAP32[$4>>2] = $onError;
 HEAP32[$5>>2] = $iDbDest;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$emptyDestTest>>2] = 0;
 HEAP32[$emptySrcTest>>2] = 0;
 HEAP32[$destHasUniqueIdx>>2] = 0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $518 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($518|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 536|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 68|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_sqlite3TriggerList($18,$19)|0);
   $21 = ($20|0)!=(0|0);
   if ($21) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $22 = HEAP32[$2>>2]|0;
   $23 = ((($22)) + 42|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = $25 & 16;
   $27 = ($26|0)!=(0);
   if ($27) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $28 = HEAP32[$4>>2]|0;
   $29 = ($28|0)==(10);
   if ($29) {
    $30 = HEAP32[$2>>2]|0;
    $31 = ((($30)) + 32|0);
    $32 = HEAP16[$31>>1]|0;
    $33 = $32 << 16 >> 16;
    $34 = ($33|0)>=(0);
    if ($34) {
     $35 = HEAP32[$2>>2]|0;
     $36 = ((($35)) + 43|0);
     $37 = HEAP8[$36>>0]|0;
     $38 = $37&255;
     HEAP32[$4>>2] = $38;
    }
    $39 = HEAP32[$4>>2]|0;
    $40 = ($39|0)==(10);
    if ($40) {
     HEAP32[$4>>2] = 2;
    }
   }
   $41 = HEAP32[$3>>2]|0;
   $42 = ((($41)) + 32|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(1);
   if ($45) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $46 = HEAP32[$3>>2]|0;
   $47 = ((($46)) + 32|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ((($48)) + 8|0);
   $50 = ((($49)) + 20|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($51|0)!=(0|0);
   if ($52) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $53 = HEAP32[$3>>2]|0;
   $54 = ((($53)) + 36|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)!=(0|0);
   if ($56) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $57 = HEAP32[$3>>2]|0;
   $58 = ((($57)) + 48|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)!=(0|0);
   if ($60) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $61 = HEAP32[$3>>2]|0;
   $62 = ((($61)) + 40|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($63|0)!=(0|0);
   if ($64) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $65 = HEAP32[$3>>2]|0;
   $66 = ((($65)) + 60|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = ($67|0)!=(0|0);
   if ($68) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $69 = HEAP32[$3>>2]|0;
   $70 = ((($69)) + 52|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)!=(0|0);
   if ($72) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $73 = HEAP32[$3>>2]|0;
   $74 = ((($73)) + 6|0);
   $75 = HEAP16[$74>>1]|0;
   $76 = $75&65535;
   $77 = $76 & 1;
   $78 = ($77|0)!=(0);
   if ($78) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $79 = HEAP32[$3>>2]|0;
   $80 = HEAP32[$79>>2]|0;
   HEAP32[$pEList>>2] = $80;
   $81 = HEAP32[$pEList>>2]|0;
   $82 = HEAP32[$81>>2]|0;
   $83 = ($82|0)!=(1);
   if ($83) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $84 = HEAP32[$pEList>>2]|0;
   $85 = ((($84)) + 4|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP8[$87>>0]|0;
   $89 = $88&255;
   $90 = ($89|0)!=(158);
   if ($90) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $91 = HEAP32[$3>>2]|0;
   $92 = ((($91)) + 32|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = ((($93)) + 8|0);
   HEAP32[$pItem>>2] = $94;
   $95 = HEAP32[$1>>2]|0;
   $96 = HEAP32[$pItem>>2]|0;
   $97 = (_sqlite3LocateTableItem($95,0,$96)|0);
   HEAP32[$pSrc>>2] = $97;
   $98 = HEAP32[$pSrc>>2]|0;
   $99 = ($98|0)==(0|0);
   if ($99) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $100 = HEAP32[$pSrc>>2]|0;
   $101 = HEAP32[$2>>2]|0;
   $102 = ($100|0)==($101|0);
   if ($102) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $103 = HEAP32[$2>>2]|0;
   $104 = ((($103)) + 42|0);
   $105 = HEAP8[$104>>0]|0;
   $106 = $105&255;
   $107 = $106 & 32;
   $108 = ($107|0)==(0);
   $109 = $108&1;
   $110 = HEAP32[$pSrc>>2]|0;
   $111 = ((($110)) + 42|0);
   $112 = HEAP8[$111>>0]|0;
   $113 = $112&255;
   $114 = $113 & 32;
   $115 = ($114|0)==(0);
   $116 = $115&1;
   $117 = ($109|0)!=($116|0);
   if ($117) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $118 = HEAP32[$pSrc>>2]|0;
   $119 = ((($118)) + 42|0);
   $120 = HEAP8[$119>>0]|0;
   $121 = $120&255;
   $122 = $121 & 16;
   $123 = ($122|0)!=(0);
   if ($123) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $124 = HEAP32[$pSrc>>2]|0;
   $125 = ((($124)) + 12|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = ($126|0)!=(0|0);
   if ($127) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $128 = HEAP32[$2>>2]|0;
   $129 = ((($128)) + 34|0);
   $130 = HEAP16[$129>>1]|0;
   $131 = $130 << 16 >> 16;
   $132 = HEAP32[$pSrc>>2]|0;
   $133 = ((($132)) + 34|0);
   $134 = HEAP16[$133>>1]|0;
   $135 = $134 << 16 >> 16;
   $136 = ($131|0)!=($135|0);
   if ($136) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   $137 = HEAP32[$2>>2]|0;
   $138 = ((($137)) + 32|0);
   $139 = HEAP16[$138>>1]|0;
   $140 = $139 << 16 >> 16;
   $141 = HEAP32[$pSrc>>2]|0;
   $142 = ((($141)) + 32|0);
   $143 = HEAP16[$142>>1]|0;
   $144 = $143 << 16 >> 16;
   $145 = ($140|0)!=($144|0);
   if ($145) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   HEAP32[$i>>2] = 0;
   L91: while(1) {
    $146 = HEAP32[$i>>2]|0;
    $147 = HEAP32[$2>>2]|0;
    $148 = ((($147)) + 34|0);
    $149 = HEAP16[$148>>1]|0;
    $150 = $149 << 16 >> 16;
    $151 = ($146|0)<($150|0);
    if (!($151)) {
     label = 64;
     break;
    }
    $152 = HEAP32[$i>>2]|0;
    $153 = HEAP32[$2>>2]|0;
    $154 = ((($153)) + 4|0);
    $155 = HEAP32[$154>>2]|0;
    $156 = (($155) + (($152*24)|0)|0);
    HEAP32[$pDestCol>>2] = $156;
    $157 = HEAP32[$i>>2]|0;
    $158 = HEAP32[$pSrc>>2]|0;
    $159 = ((($158)) + 4|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (($160) + (($157*24)|0)|0);
    HEAP32[$pSrcCol>>2] = $161;
    $162 = HEAP32[$pDestCol>>2]|0;
    $163 = ((($162)) + 21|0);
    $164 = HEAP8[$163>>0]|0;
    $165 = $164 << 24 >> 24;
    $166 = HEAP32[$pSrcCol>>2]|0;
    $167 = ((($166)) + 21|0);
    $168 = HEAP8[$167>>0]|0;
    $169 = $168 << 24 >> 24;
    $170 = ($165|0)!=($169|0);
    if ($170) {
     label = 52;
     break;
    }
    $171 = HEAP32[$pDestCol>>2]|0;
    $172 = ((($171)) + 16|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = HEAP32[$pSrcCol>>2]|0;
    $175 = ((($174)) + 16|0);
    $176 = HEAP32[$175>>2]|0;
    $177 = (_sqlite3_stricmp($173,$176)|0);
    $178 = ($177|0)!=(0);
    if ($178) {
     label = 54;
     break;
    }
    $179 = HEAP32[$pDestCol>>2]|0;
    $180 = ((($179)) + 20|0);
    $181 = HEAP8[$180>>0]|0;
    $182 = $181&255;
    $183 = ($182|0)!=(0);
    if ($183) {
     $184 = HEAP32[$pSrcCol>>2]|0;
     $185 = ((($184)) + 20|0);
     $186 = HEAP8[$185>>0]|0;
     $187 = ($186<<24>>24)!=(0);
     if (!($187)) {
      label = 57;
      break;
     }
    }
    $188 = HEAP32[$i>>2]|0;
    $189 = ($188|0)>(0);
    do {
     if ($189) {
      $190 = HEAP32[$pDestCol>>2]|0;
      $191 = ((($190)) + 8|0);
      $192 = HEAP32[$191>>2]|0;
      $193 = ($192|0)==(0|0);
      $194 = $193&1;
      $195 = HEAP32[$pSrcCol>>2]|0;
      $196 = ((($195)) + 8|0);
      $197 = HEAP32[$196>>2]|0;
      $198 = ($197|0)==(0|0);
      $199 = $198&1;
      $200 = ($194|0)!=($199|0);
      if ($200) {
       label = 62;
       break L91;
      }
      $201 = HEAP32[$pDestCol>>2]|0;
      $202 = ((($201)) + 8|0);
      $203 = HEAP32[$202>>2]|0;
      $204 = ($203|0)!=(0|0);
      if (!($204)) {
       break;
      }
      $205 = HEAP32[$pDestCol>>2]|0;
      $206 = ((($205)) + 8|0);
      $207 = HEAP32[$206>>2]|0;
      $208 = HEAP32[$pSrcCol>>2]|0;
      $209 = ((($208)) + 8|0);
      $210 = HEAP32[$209>>2]|0;
      $211 = (_strcmp($207,$210)|0);
      $212 = ($211|0)!=(0);
      if ($212) {
       label = 62;
       break L91;
      }
     }
    } while(0);
    $213 = HEAP32[$i>>2]|0;
    $214 = (($213) + 1)|0;
    HEAP32[$i>>2] = $214;
   }
   if ((label|0) == 52) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   else if ((label|0) == 54) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   else if ((label|0) == 57) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   else if ((label|0) == 62) {
    HEAP32[$0>>2] = 0;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
   else if ((label|0) == 64) {
    $215 = HEAP32[$2>>2]|0;
    $216 = ((($215)) + 8|0);
    $217 = HEAP32[$216>>2]|0;
    HEAP32[$pDestIdx>>2] = $217;
    while(1) {
     $218 = HEAP32[$pDestIdx>>2]|0;
     $219 = ($218|0)!=(0|0);
     if (!($219)) {
      break;
     }
     $220 = HEAP32[$pDestIdx>>2]|0;
     $221 = ((($220)) + 54|0);
     $222 = HEAP8[$221>>0]|0;
     $223 = $222&255;
     $224 = ($223|0)!=(0);
     if ($224) {
      HEAP32[$destHasUniqueIdx>>2] = 1;
     }
     $225 = HEAP32[$pSrc>>2]|0;
     $226 = ((($225)) + 8|0);
     $227 = HEAP32[$226>>2]|0;
     HEAP32[$pSrcIdx>>2] = $227;
     while(1) {
      $228 = HEAP32[$pSrcIdx>>2]|0;
      $229 = ($228|0)!=(0|0);
      if (!($229)) {
       break;
      }
      $230 = HEAP32[$pDestIdx>>2]|0;
      $231 = HEAP32[$pSrcIdx>>2]|0;
      $232 = (_xferCompatibleIndex($230,$231)|0);
      $233 = ($232|0)!=(0);
      if ($233) {
       break;
      }
      $234 = HEAP32[$pSrcIdx>>2]|0;
      $235 = ((($234)) + 20|0);
      $236 = HEAP32[$235>>2]|0;
      HEAP32[$pSrcIdx>>2] = $236;
     }
     $237 = HEAP32[$pSrcIdx>>2]|0;
     $238 = ($237|0)==(0|0);
     if ($238) {
      label = 73;
      break;
     }
     $239 = HEAP32[$pDestIdx>>2]|0;
     $240 = ((($239)) + 20|0);
     $241 = HEAP32[$240>>2]|0;
     HEAP32[$pDestIdx>>2] = $241;
    }
    if ((label|0) == 73) {
     HEAP32[$0>>2] = 0;
     $518 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($518|0);
    }
    $242 = HEAP32[$2>>2]|0;
    $243 = ((($242)) + 24|0);
    $244 = HEAP32[$243>>2]|0;
    $245 = ($244|0)!=(0|0);
    do {
     if ($245) {
      $246 = HEAP32[$pSrc>>2]|0;
      $247 = ((($246)) + 24|0);
      $248 = HEAP32[$247>>2]|0;
      $249 = HEAP32[$2>>2]|0;
      $250 = ((($249)) + 24|0);
      $251 = HEAP32[$250>>2]|0;
      $252 = (_sqlite3ExprListCompare($248,$251,-1)|0);
      $253 = ($252|0)!=(0);
      if (!($253)) {
       break;
      }
      HEAP32[$0>>2] = 0;
      $518 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($518|0);
     }
    } while(0);
    $254 = HEAP32[$db>>2]|0;
    $255 = ((($254)) + 24|0);
    $256 = HEAP32[$255>>2]|0;
    $257 = $256 & 524288;
    $258 = ($257|0)!=(0);
    do {
     if ($258) {
      $259 = HEAP32[$2>>2]|0;
      $260 = ((($259)) + 16|0);
      $261 = HEAP32[$260>>2]|0;
      $262 = ($261|0)!=(0|0);
      if (!($262)) {
       break;
      }
      HEAP32[$0>>2] = 0;
      $518 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($518|0);
     }
    } while(0);
    $263 = HEAP32[$db>>2]|0;
    $264 = ((($263)) + 24|0);
    $265 = HEAP32[$264>>2]|0;
    $266 = $265 & 128;
    $267 = ($266|0)!=(0);
    if ($267) {
     HEAP32[$0>>2] = 0;
     $518 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($518|0);
    }
    $268 = HEAP32[$db>>2]|0;
    $269 = HEAP32[$pSrc>>2]|0;
    $270 = ((($269)) + 64|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = (_sqlite3SchemaToIndex($268,$271)|0);
    HEAP32[$iDbSrc>>2] = $272;
    $273 = HEAP32[$1>>2]|0;
    $274 = (_sqlite3GetVdbe($273)|0);
    HEAP32[$v>>2] = $274;
    $275 = HEAP32[$1>>2]|0;
    $276 = HEAP32[$iDbSrc>>2]|0;
    _sqlite3CodeVerifySchema($275,$276);
    $277 = HEAP32[$1>>2]|0;
    $278 = ((($277)) + 72|0);
    $279 = HEAP32[$278>>2]|0;
    $280 = (($279) + 1)|0;
    HEAP32[$278>>2] = $280;
    HEAP32[$iSrc>>2] = $279;
    $281 = HEAP32[$1>>2]|0;
    $282 = ((($281)) + 72|0);
    $283 = HEAP32[$282>>2]|0;
    $284 = (($283) + 1)|0;
    HEAP32[$282>>2] = $284;
    HEAP32[$iDest>>2] = $283;
    $285 = HEAP32[$1>>2]|0;
    $286 = HEAP32[$5>>2]|0;
    $287 = HEAP32[$2>>2]|0;
    $288 = (_autoIncBegin($285,$286,$287)|0);
    HEAP32[$regAutoinc>>2] = $288;
    $289 = HEAP32[$1>>2]|0;
    $290 = (_sqlite3GetTempReg($289)|0);
    HEAP32[$regData>>2] = $290;
    $291 = HEAP32[$1>>2]|0;
    $292 = (_sqlite3GetTempReg($291)|0);
    HEAP32[$regRowid>>2] = $292;
    $293 = HEAP32[$1>>2]|0;
    $294 = HEAP32[$iDest>>2]|0;
    $295 = HEAP32[$5>>2]|0;
    $296 = HEAP32[$2>>2]|0;
    _sqlite3OpenTable($293,$294,$295,$296,55);
    $297 = HEAP32[$db>>2]|0;
    $298 = ((($297)) + 24|0);
    $299 = HEAP32[$298>>2]|0;
    $300 = $299 & 134217728;
    $301 = ($300|0)==(0);
    do {
     if ($301) {
      $302 = HEAP32[$2>>2]|0;
      $303 = ((($302)) + 32|0);
      $304 = HEAP16[$303>>1]|0;
      $305 = $304 << 16 >> 16;
      $306 = ($305|0)<(0);
      if ($306) {
       $307 = HEAP32[$2>>2]|0;
       $308 = ((($307)) + 8|0);
       $309 = HEAP32[$308>>2]|0;
       $310 = ($309|0)!=(0|0);
       $311 = HEAP32[$destHasUniqueIdx>>2]|0;
       $312 = ($311|0)!=(0);
       $or$cond = $310 | $312;
       if (!($or$cond)) {
        label = 87;
       }
      } else {
       $$old = HEAP32[$destHasUniqueIdx>>2]|0;
       $$old1 = ($$old|0)!=(0);
       if (!($$old1)) {
        label = 87;
       }
      }
      if ((label|0) == 87) {
       $313 = HEAP32[$4>>2]|0;
       $314 = ($313|0)!=(2);
       $315 = HEAP32[$4>>2]|0;
       $316 = ($315|0)!=(1);
       $or$cond4 = $314 & $316;
       if (!($or$cond4)) {
        break;
       }
      }
      $317 = HEAP32[$v>>2]|0;
      $318 = HEAP32[$iDest>>2]|0;
      $319 = (_sqlite3VdbeAddOp2($317,108,$318,0)|0);
      HEAP32[$addr1>>2] = $319;
      $320 = HEAP32[$v>>2]|0;
      $321 = (_sqlite3VdbeAddOp0($320,13)|0);
      HEAP32[$emptyDestTest>>2] = $321;
      $322 = HEAP32[$v>>2]|0;
      $323 = HEAP32[$addr1>>2]|0;
      _sqlite3VdbeJumpHere($322,$323);
     }
    } while(0);
    $324 = HEAP32[$pSrc>>2]|0;
    $325 = ((($324)) + 42|0);
    $326 = HEAP8[$325>>0]|0;
    $327 = $326&255;
    $328 = $327 & 32;
    $329 = ($328|0)==(0);
    $330 = HEAP32[$1>>2]|0;
    if ($329) {
     $331 = HEAP32[$iSrc>>2]|0;
     $332 = HEAP32[$iDbSrc>>2]|0;
     $333 = HEAP32[$pSrc>>2]|0;
     _sqlite3OpenTable($330,$331,$332,$333,54);
     $334 = HEAP32[$v>>2]|0;
     $335 = HEAP32[$iSrc>>2]|0;
     $336 = (_sqlite3VdbeAddOp2($334,108,$335,0)|0);
     HEAP32[$emptySrcTest>>2] = $336;
     $337 = HEAP32[$2>>2]|0;
     $338 = ((($337)) + 32|0);
     $339 = HEAP16[$338>>1]|0;
     $340 = $339 << 16 >> 16;
     $341 = ($340|0)>=(0);
     do {
      if ($341) {
       $342 = HEAP32[$v>>2]|0;
       $343 = HEAP32[$iSrc>>2]|0;
       $344 = HEAP32[$regRowid>>2]|0;
       $345 = (_sqlite3VdbeAddOp2($342,103,$343,$344)|0);
       HEAP32[$addr1>>2] = $345;
       $346 = HEAP32[$v>>2]|0;
       $347 = HEAP32[$iDest>>2]|0;
       $348 = HEAP32[$regRowid>>2]|0;
       $349 = (_sqlite3VdbeAddOp3($346,70,$347,0,$348)|0);
       HEAP32[$addr2>>2] = $349;
       $350 = HEAP32[$1>>2]|0;
       $351 = HEAP32[$4>>2]|0;
       $352 = HEAP32[$2>>2]|0;
       _sqlite3RowidConstraint($350,$351,$352);
       $353 = HEAP32[$v>>2]|0;
       $354 = HEAP32[$addr2>>2]|0;
       _sqlite3VdbeJumpHere($353,$354);
       $355 = HEAP32[$1>>2]|0;
       $356 = HEAP32[$regAutoinc>>2]|0;
       $357 = HEAP32[$regRowid>>2]|0;
       _autoIncStep($355,$356,$357);
      } else {
       $358 = HEAP32[$2>>2]|0;
       $359 = ((($358)) + 8|0);
       $360 = HEAP32[$359>>2]|0;
       $361 = ($360|0)==(0|0);
       $362 = HEAP32[$v>>2]|0;
       if ($361) {
        $363 = HEAP32[$iDest>>2]|0;
        $364 = HEAP32[$regRowid>>2]|0;
        $365 = (_sqlite3VdbeAddOp2($362,74,$363,$364)|0);
        HEAP32[$addr1>>2] = $365;
        break;
       } else {
        $366 = HEAP32[$iSrc>>2]|0;
        $367 = HEAP32[$regRowid>>2]|0;
        $368 = (_sqlite3VdbeAddOp2($362,103,$366,$367)|0);
        HEAP32[$addr1>>2] = $368;
        break;
       }
      }
     } while(0);
     $369 = HEAP32[$v>>2]|0;
     $370 = HEAP32[$iSrc>>2]|0;
     $371 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp2($369,102,$370,$371)|0);
     $372 = HEAP32[$v>>2]|0;
     $373 = HEAP32[$iDest>>2]|0;
     $374 = HEAP32[$regData>>2]|0;
     $375 = HEAP32[$regRowid>>2]|0;
     $376 = HEAP32[$2>>2]|0;
     $377 = HEAP32[$376>>2]|0;
     (_sqlite3VdbeAddOp4($372,75,$373,$374,$375,$377,0)|0);
     $378 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($378,11);
     $379 = HEAP32[$v>>2]|0;
     $380 = HEAP32[$iSrc>>2]|0;
     $381 = HEAP32[$addr1>>2]|0;
     (_sqlite3VdbeAddOp2($379,7,$380,$381)|0);
     $382 = HEAP32[$v>>2]|0;
     $383 = HEAP32[$iSrc>>2]|0;
     (_sqlite3VdbeAddOp2($382,61,$383,0)|0);
     $384 = HEAP32[$v>>2]|0;
     $385 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($384,61,$385,0)|0);
    } else {
     $386 = HEAP32[$5>>2]|0;
     $387 = HEAP32[$2>>2]|0;
     $388 = ((($387)) + 28|0);
     $389 = HEAP32[$388>>2]|0;
     $390 = HEAP32[$2>>2]|0;
     $391 = HEAP32[$390>>2]|0;
     _sqlite3TableLock($330,$386,$389,1,$391);
     $392 = HEAP32[$1>>2]|0;
     $393 = HEAP32[$iDbSrc>>2]|0;
     $394 = HEAP32[$pSrc>>2]|0;
     $395 = ((($394)) + 28|0);
     $396 = HEAP32[$395>>2]|0;
     $397 = HEAP32[$pSrc>>2]|0;
     $398 = HEAP32[$397>>2]|0;
     _sqlite3TableLock($392,$393,$396,0,$398);
    }
    $399 = HEAP32[$2>>2]|0;
    $400 = ((($399)) + 8|0);
    $401 = HEAP32[$400>>2]|0;
    HEAP32[$pDestIdx>>2] = $401;
    while(1) {
     $402 = HEAP32[$pDestIdx>>2]|0;
     $403 = ($402|0)!=(0|0);
     if (!($403)) {
      break;
     }
     HEAP8[$idxInsFlags>>0] = 0;
     $404 = HEAP32[$pSrc>>2]|0;
     $405 = ((($404)) + 8|0);
     $406 = HEAP32[$405>>2]|0;
     HEAP32[$pSrcIdx>>2] = $406;
     while(1) {
      $407 = HEAP32[$pSrcIdx>>2]|0;
      $408 = ($407|0)!=(0|0);
      if (!($408)) {
       break;
      }
      $409 = HEAP32[$pDestIdx>>2]|0;
      $410 = HEAP32[$pSrcIdx>>2]|0;
      $411 = (_xferCompatibleIndex($409,$410)|0);
      $412 = ($411|0)!=(0);
      if ($412) {
       break;
      }
      $413 = HEAP32[$pSrcIdx>>2]|0;
      $414 = ((($413)) + 20|0);
      $415 = HEAP32[$414>>2]|0;
      HEAP32[$pSrcIdx>>2] = $415;
     }
     $416 = HEAP32[$v>>2]|0;
     $417 = HEAP32[$iSrc>>2]|0;
     $418 = HEAP32[$pSrcIdx>>2]|0;
     $419 = ((($418)) + 44|0);
     $420 = HEAP32[$419>>2]|0;
     $421 = HEAP32[$iDbSrc>>2]|0;
     (_sqlite3VdbeAddOp3($416,54,$417,$420,$421)|0);
     $422 = HEAP32[$1>>2]|0;
     $423 = HEAP32[$pSrcIdx>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($422,$423);
     $424 = HEAP32[$v>>2]|0;
     $425 = HEAP32[$iDest>>2]|0;
     $426 = HEAP32[$pDestIdx>>2]|0;
     $427 = ((($426)) + 44|0);
     $428 = HEAP32[$427>>2]|0;
     $429 = HEAP32[$5>>2]|0;
     (_sqlite3VdbeAddOp3($424,55,$425,$428,$429)|0);
     $430 = HEAP32[$1>>2]|0;
     $431 = HEAP32[$pDestIdx>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($430,$431);
     $432 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($432,1);
     $433 = HEAP32[$v>>2]|0;
     $434 = HEAP32[$iSrc>>2]|0;
     $435 = (_sqlite3VdbeAddOp2($433,108,$434,0)|0);
     HEAP32[$addr1>>2] = $435;
     $436 = HEAP32[$v>>2]|0;
     $437 = HEAP32[$iSrc>>2]|0;
     $438 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp2($436,101,$437,$438)|0);
     $439 = HEAP32[$db>>2]|0;
     $440 = ((($439)) + 24|0);
     $441 = HEAP32[$440>>2]|0;
     $442 = $441 & 134217728;
     $443 = ($442|0)!=(0);
     do {
      if ($443) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $444 = HEAP32[$i>>2]|0;
        $445 = HEAP32[$pSrcIdx>>2]|0;
        $446 = ((($445)) + 52|0);
        $447 = HEAP16[$446>>1]|0;
        $448 = $447&65535;
        $449 = ($444|0)<($448|0);
        if (!($449)) {
         break;
        }
        $450 = HEAP32[$i>>2]|0;
        $451 = HEAP32[$pSrcIdx>>2]|0;
        $452 = ((($451)) + 32|0);
        $453 = HEAP32[$452>>2]|0;
        $454 = (($453) + ($450<<2)|0);
        $455 = HEAP32[$454>>2]|0;
        HEAP32[$zColl>>2] = $455;
        $456 = HEAP32[$zColl>>2]|0;
        $457 = (_sqlite3_stricmp(20716,$456)|0);
        $458 = ($457|0)!=(0);
        if ($458) {
         break;
        }
        $459 = HEAP32[$i>>2]|0;
        $460 = (($459) + 1)|0;
        HEAP32[$i>>2] = $460;
       }
       $461 = HEAP32[$i>>2]|0;
       $462 = HEAP32[$pSrcIdx>>2]|0;
       $463 = ((($462)) + 52|0);
       $464 = HEAP16[$463>>1]|0;
       $465 = $464&65535;
       $466 = ($461|0)==($465|0);
       if (!($466)) {
        break;
       }
       HEAP8[$idxInsFlags>>0] = 16;
       $467 = HEAP32[$v>>2]|0;
       $468 = HEAP32[$iDest>>2]|0;
       (_sqlite3VdbeAddOp3($467,105,$468,0,-1)|0);
      }
     } while(0);
     $469 = HEAP32[$pSrc>>2]|0;
     $470 = ((($469)) + 42|0);
     $471 = HEAP8[$470>>0]|0;
     $472 = $471&255;
     $473 = $472 & 32;
     $474 = ($473|0)==(0);
     do {
      if (!($474)) {
       $475 = HEAP32[$pDestIdx>>2]|0;
       $476 = ((($475)) + 55|0);
       $477 = HEAP8[$476>>0]|0;
       $478 = $477 & 3;
       $479 = $478&255;
       $480 = ($479|0)==(2);
       if (!($480)) {
        break;
       }
       $481 = HEAP8[$idxInsFlags>>0]|0;
       $482 = $481&255;
       $483 = $482 | 1;
       $484 = $483&255;
       HEAP8[$idxInsFlags>>0] = $484;
      }
     } while(0);
     $485 = HEAP32[$v>>2]|0;
     $486 = HEAP32[$iDest>>2]|0;
     $487 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp3($485,110,$486,$487,1)|0);
     $488 = HEAP32[$v>>2]|0;
     $489 = HEAP8[$idxInsFlags>>0]|0;
     _sqlite3VdbeChangeP5($488,$489);
     $490 = HEAP32[$v>>2]|0;
     $491 = HEAP32[$iSrc>>2]|0;
     $492 = HEAP32[$addr1>>2]|0;
     $493 = (($492) + 1)|0;
     (_sqlite3VdbeAddOp2($490,7,$491,$493)|0);
     $494 = HEAP32[$v>>2]|0;
     $495 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($494,$495);
     $496 = HEAP32[$v>>2]|0;
     $497 = HEAP32[$iSrc>>2]|0;
     (_sqlite3VdbeAddOp2($496,61,$497,0)|0);
     $498 = HEAP32[$v>>2]|0;
     $499 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($498,61,$499,0)|0);
     $500 = HEAP32[$pDestIdx>>2]|0;
     $501 = ((($500)) + 20|0);
     $502 = HEAP32[$501>>2]|0;
     HEAP32[$pDestIdx>>2] = $502;
    }
    $503 = HEAP32[$emptySrcTest>>2]|0;
    $504 = ($503|0)!=(0);
    if ($504) {
     $505 = HEAP32[$v>>2]|0;
     $506 = HEAP32[$emptySrcTest>>2]|0;
     _sqlite3VdbeJumpHere($505,$506);
    }
    $507 = HEAP32[$1>>2]|0;
    $508 = HEAP32[$regRowid>>2]|0;
    _sqlite3ReleaseTempReg($507,$508);
    $509 = HEAP32[$1>>2]|0;
    $510 = HEAP32[$regData>>2]|0;
    _sqlite3ReleaseTempReg($509,$510);
    $511 = HEAP32[$emptyDestTest>>2]|0;
    $512 = ($511|0)!=(0);
    if ($512) {
     $513 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp2($513,21,0,0)|0);
     $514 = HEAP32[$v>>2]|0;
     $515 = HEAP32[$emptyDestTest>>2]|0;
     _sqlite3VdbeJumpHere($514,$515);
     $516 = HEAP32[$v>>2]|0;
     $517 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($516,61,$517,0)|0);
     HEAP32[$0>>2] = 0;
     $518 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($518|0);
    } else {
     HEAP32[$0>>2] = 1;
     $518 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($518|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $518 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($518|0);
}
function _autoIncBegin($pParse,$iDb,$pTab) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $memId = 0, $pInfo = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $memId = sp + 8|0;
 $pToplevel = sp + 4|0;
 $pInfo = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$memId>>2] = 0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 42|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 8;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 416|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0|0);
  $14 = HEAP32[$1>>2]|0;
  if ($13) {
   $15 = ((($14)) + 416|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $16;
  } else {
   $17 = $14;
  }
  HEAP32[$pToplevel>>2] = $17;
  $18 = HEAP32[$pToplevel>>2]|0;
  $19 = ((($18)) + 412|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pInfo>>2] = $20;
  while(1) {
   $21 = HEAP32[$pInfo>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$pInfo>>2]|0;
    $24 = ((($23)) + 4|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$3>>2]|0;
    $27 = ($25|0)!=($26|0);
    $68 = $27;
   } else {
    $68 = 0;
   }
   $28 = HEAP32[$pInfo>>2]|0;
   if (!($68)) {
    break;
   }
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$pInfo>>2] = $29;
  }
  $30 = ($28|0)==(0|0);
  do {
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    $33 = (_sqlite3DbMallocRawNN($32,16,0)|0);
    HEAP32[$pInfo>>2] = $33;
    $34 = HEAP32[$pInfo>>2]|0;
    $35 = ($34|0)==(0|0);
    if (!($35)) {
     $36 = HEAP32[$pToplevel>>2]|0;
     $37 = ((($36)) + 412|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = HEAP32[$pInfo>>2]|0;
     HEAP32[$39>>2] = $38;
     $40 = HEAP32[$pInfo>>2]|0;
     $41 = HEAP32[$pToplevel>>2]|0;
     $42 = ((($41)) + 412|0);
     HEAP32[$42>>2] = $40;
     $43 = HEAP32[$3>>2]|0;
     $44 = HEAP32[$pInfo>>2]|0;
     $45 = ((($44)) + 4|0);
     HEAP32[$45>>2] = $43;
     $46 = HEAP32[$2>>2]|0;
     $47 = HEAP32[$pInfo>>2]|0;
     $48 = ((($47)) + 8|0);
     HEAP32[$48>>2] = $46;
     $49 = HEAP32[$pToplevel>>2]|0;
     $50 = ((($49)) + 76|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + 1)|0;
     HEAP32[$50>>2] = $52;
     $53 = HEAP32[$pToplevel>>2]|0;
     $54 = ((($53)) + 76|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (($55) + 1)|0;
     HEAP32[$54>>2] = $56;
     $57 = HEAP32[$pInfo>>2]|0;
     $58 = ((($57)) + 12|0);
     HEAP32[$58>>2] = $56;
     $59 = HEAP32[$pToplevel>>2]|0;
     $60 = ((($59)) + 76|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = (($61) + 1)|0;
     HEAP32[$60>>2] = $62;
     break;
    }
    HEAP32[$0>>2] = 0;
    $67 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($67|0);
   }
  } while(0);
  $63 = HEAP32[$pInfo>>2]|0;
  $64 = ((($63)) + 12|0);
  $65 = HEAP32[$64>>2]|0;
  HEAP32[$memId>>2] = $65;
 }
 $66 = HEAP32[$memId>>2]|0;
 HEAP32[$0>>2] = $66;
 $67 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($67|0);
}
function _readsTable($p,$iDb,$pTab) {
 $p = $p|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iEnd = 0, $pIndex = 0, $pOp = 0, $pVTab = 0, $tnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $iEnd = sp + 16|0;
 $pVTab = sp + 12|0;
 $pOp = sp + 8|0;
 $pIndex = sp + 4|0;
 $tnum = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = (_sqlite3VdbeCurrentAddr($6)|0);
 HEAP32[$iEnd>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 42|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = $11 & 16;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_sqlite3GetVTable($15,$16)|0);
  $18 = $17;
 } else {
  $18 = 0;
 }
 HEAP32[$pVTab>>2] = $18;
 HEAP32[$i>>2] = 1;
 L4: while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$iEnd>>2]|0;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   label = 18;
   break;
  }
  $22 = HEAP32[$v>>2]|0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (_sqlite3VdbeGetOp($22,$23)|0);
  HEAP32[$pOp>>2] = $24;
  $25 = HEAP32[$pOp>>2]|0;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = ($27|0)==(54);
  L7: do {
   if ($28) {
    $29 = HEAP32[$pOp>>2]|0;
    $30 = ((($29)) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = ($31|0)==($32|0);
    if ($33) {
     $34 = HEAP32[$pOp>>2]|0;
     $35 = ((($34)) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     HEAP32[$tnum>>2] = $36;
     $37 = HEAP32[$tnum>>2]|0;
     $38 = HEAP32[$3>>2]|0;
     $39 = ((($38)) + 28|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($37|0)==($40|0);
     if ($41) {
      label = 8;
      break L4;
     }
     $42 = HEAP32[$3>>2]|0;
     $43 = ((($42)) + 8|0);
     $44 = HEAP32[$43>>2]|0;
     HEAP32[$pIndex>>2] = $44;
     while(1) {
      $45 = HEAP32[$pIndex>>2]|0;
      $46 = ($45|0)!=(0|0);
      if (!($46)) {
       break L7;
      }
      $47 = HEAP32[$tnum>>2]|0;
      $48 = HEAP32[$pIndex>>2]|0;
      $49 = ((($48)) + 44|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = ($47|0)==($50|0);
      if ($51) {
       label = 12;
       break L4;
      }
      $52 = HEAP32[$pIndex>>2]|0;
      $53 = ((($52)) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      HEAP32[$pIndex>>2] = $54;
     }
    }
   }
  } while(0);
  $55 = HEAP32[$pOp>>2]|0;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)==(152);
  if ($58) {
   $59 = HEAP32[$pOp>>2]|0;
   $60 = ((($59)) + 16|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$pVTab>>2]|0;
   $63 = ($61|0)==($62|0);
   if ($63) {
    label = 16;
    break;
   }
  }
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 0;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return (0)|0;
}
function _sqlite3ExprCodeAndCache($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iMem = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $v = sp + 4|0;
 $iMem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($6,$7,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 76|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$10>>2] = $12;
 HEAP32[$iMem>>2] = $12;
 $13 = HEAP32[$v>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$iMem>>2]|0;
 (_sqlite3VdbeAddOp2($13,30,$14,$15)|0);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$iMem>>2]|0;
 _exprToRegister($16,$17);
 STACKTOP = sp;return;
}
function _autoIncStep($pParse,$memId,$regRowid) {
 $pParse = $pParse|0;
 $memId = $memId|0;
 $regRowid = $regRowid|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $memId;
 HEAP32[$2>>2] = $regRowid;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($7,137,$8,$9)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeFactorable($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 23|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_sqlite3ExprIsConstant($8)|0);
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   _sqlite3ExprCodeAtInit($11,$12,$13,0);
   STACKTOP = sp;return;
  }
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($14,$15,$16);
 STACKTOP = sp;return;
}
function _xferCompatibleIndex($pDest,$pSrc) {
 $pDest = $pDest|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pDest;
 HEAP32[$2>>2] = $pSrc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 50|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 50|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($6|0)!=($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 54|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 54|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($15|0)!=($19|0);
 if ($20) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 50|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24&65535;
  $26 = ($21|0)<($25|0);
  if (!($26)) {
   label = 17;
   break;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<1)|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32 << 16 >> 16;
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<1)|0);
  $39 = HEAP16[$38>>1]|0;
  $40 = $39 << 16 >> 16;
  $41 = ($33|0)!=($40|0);
  if ($41) {
   label = 8;
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = ((($43)) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<1)|0);
  $47 = HEAP16[$46>>1]|0;
  $48 = $47 << 16 >> 16;
  $49 = ($48|0)==(-2);
  if ($49) {
   $50 = HEAP32[$i>>2]|0;
   $51 = HEAP32[$2>>2]|0;
   $52 = ((($51)) + 40|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + (($50*20)|0)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$i>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 40|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ((($61)) + 4|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (($63) + (($58*20)|0)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (_sqlite3ExprCompare($57,$65,-1)|0);
   $67 = ($66|0)!=(0);
   if ($67) {
    label = 11;
    break;
   }
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = ((($69)) + 28|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + ($68)|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = HEAP32[$i>>2]|0;
  $76 = HEAP32[$1>>2]|0;
  $77 = ((($76)) + 28|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (($78) + ($75)|0);
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = ($74|0)!=($81|0);
  if ($82) {
   label = 13;
   break;
  }
  $83 = HEAP32[$i>>2]|0;
  $84 = HEAP32[$2>>2]|0;
  $85 = ((($84)) + 32|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + ($83<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = HEAP32[$i>>2]|0;
  $90 = HEAP32[$1>>2]|0;
  $91 = ((($90)) + 32|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + ($89<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (_sqlite3_stricmp($88,$94)|0);
  $96 = ($95|0)!=(0);
  if ($96) {
   label = 15;
   break;
  }
  $97 = HEAP32[$i>>2]|0;
  $98 = (($97) + 1)|0;
  HEAP32[$i>>2] = $98;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 17) {
  $99 = HEAP32[$2>>2]|0;
  $100 = ((($99)) + 36|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = HEAP32[$1>>2]|0;
  $103 = ((($102)) + 36|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = (_sqlite3ExprCompare($101,$104,-1)|0);
  $106 = ($105|0)!=(0);
  if ($106) {
   HEAP32[$0>>2] = 0;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  } else {
   HEAP32[$0>>2] = 1;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  }
 }
 return (0)|0;
}
function _sqlite3VdbeRunOnlyOnce($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 88|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -17;
 $5 = $4 | 16;
 HEAP16[$2>>1] = $5;
 STACKTOP = sp;return;
}
function _returnSingleText($v,$zLabel,$zValue) {
 $v = $v|0;
 $zLabel = $zLabel|0;
 $zValue = $zValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $zLabel;
 HEAP32[$2>>2] = $zValue;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeLoadString($5,1,$6)|0);
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _setOneColumnName($7,$8);
 $9 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeAddOp2($9,33,1,1)|0);
 STACKTOP = sp;return;
}
function _setOneColumnName($v,$z) {
 $v = $v|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$0>>2]|0;
 _setAllColumnNames($2,1,$1);
 STACKTOP = sp;return;
}
function _sqlite3BtreeSetCacheSize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3PagerSetCachesize($7,$8);
 STACKTOP = sp;return 0;
}
function _returnSingleInt($v,$zLabel,$0,$1) {
 $v = $v|0;
 $zLabel = $zLabel|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$2>>2] = $v;
 HEAP32[$3>>2] = $zLabel;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp4Dup8($9,23,0,1,0,$4,-13)|0);
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 _setOneColumnName($10,$11);
 $12 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($12,33,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3BtreeSecureDelete($p,$newFlag) {
 $p = $p|0;
 $newFlag = $newFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $b = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $newFlag;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $5 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 22|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12&65535;
  $14 = $13 & -5;
  $15 = $14&65535;
  HEAP16[$11>>1] = $15;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ((($20)) + 22|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22&65535;
   $24 = $23 | 4;
   $25 = $24&65535;
   HEAP16[$21>>1] = $25;
  }
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($28)) + 22|0);
 $30 = HEAP16[$29>>1]|0;
 $31 = $30&65535;
 $32 = $31 & 4;
 $33 = ($32|0)!=(0);
 $34 = $33&1;
 HEAP32[$b>>2] = $34;
 $35 = HEAP32[$b>>2]|0;
 HEAP32[$0>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _getLockingMode($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = (_sqlite3_stricmp($4,34084)|0);
   $6 = (0)==($5|0);
   if ($6) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3_stricmp($7,34077)|0);
   $9 = (0)==($8|0);
   if ($9) {
    HEAP32[$0>>2] = 0;
   } else {
    label = 6;
   }
  } else {
   label = 6;
  }
 } while(0);
 if ((label|0) == 6) {
  HEAP32[$0>>2] = -1;
 }
 $10 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3PagerLockingMode($pPager,$eMode) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 13|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = ($6<<24>>24)!=(0);
  if (!($7)) {
   $8 = HEAP32[$0>>2]|0;
   $9 = ((($8)) + 216|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3WalHeapMemory($10)|0);
   $12 = ($11|0)!=(0);
   if (!($12)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = $13&255;
    $15 = HEAP32[$0>>2]|0;
    $16 = ((($15)) + 4|0);
    HEAP8[$16>>0] = $14;
   }
  }
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3PagerJournalSizeLimit($pPager,$0,$1) {
 $pPager = $pPager|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pPager;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)>(-1);
 $15 = ($10>>>0)>=(4294967295);
 $16 = ($13|0)==(-1);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if ($18) {
  $19 = $3;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 168|0);
  $27 = $26;
  $28 = $27;
  HEAP32[$28>>2] = $21;
  $29 = (($27) + 4)|0;
  $30 = $29;
  HEAP32[$30>>2] = $24;
  $31 = HEAP32[$2>>2]|0;
  $32 = ((($31)) + 216|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $3;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  _sqlite3WalLimit($33,$36,$39);
 }
 $40 = HEAP32[$2>>2]|0;
 $41 = ((($40)) + 168|0);
 $42 = $41;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 tempRet0 = ($47);
 STACKTOP = sp;return ($44|0);
}
function _getAutoVacuum($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_stricmp($2,35527)|0);
 $4 = (0)==($3|0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 0;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = (_sqlite3_stricmp($5,34821)|0);
   $7 = (0)==($6|0);
   if ($7) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (_sqlite3_stricmp($8,35532)|0);
   $10 = (0)==($9|0);
   if ($10) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_sqlite3Atoi($11)|0);
    HEAP32[$i>>2] = $12;
    $13 = HEAP32[$i>>2]|0;
    $14 = ($13|0)>=(0);
    $15 = HEAP32[$i>>2]|0;
    $16 = ($15|0)<=(2);
    $or$cond = $14 & $16;
    $17 = HEAP32[$i>>2]|0;
    $18 = $or$cond ? $17 : 0;
    $19 = $18&255;
    $20 = $19&255;
    HEAP32[$0>>2] = $20;
    break;
   }
  }
 } while(0);
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3BtreeSetSpillSize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pBt = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3PagerSetSpillsize($7,$8)|0);
 HEAP32[$res>>2] = $9;
 $10 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _setAllPagerFlags($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pDb = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 63|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pDb>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$n>>2] = $10;
 while(1) {
  $11 = HEAP32[$n>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$n>>2] = $12;
  $13 = ($11|0)>(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pDb>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$pDb>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$pDb>>2]|0;
   $22 = ((($21)) + 8|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 24|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $27 & 56;
   $29 = $24 | $28;
   (_sqlite3BtreeSetPagerFlags($20,$29)|0);
  }
  $30 = HEAP32[$pDb>>2]|0;
  $31 = ((($30)) + 16|0);
  HEAP32[$pDb>>2] = $31;
 }
 STACKTOP = sp;return;
}
function _changeTempStorage($pParse,$zStorageType) {
 $pParse = $pParse|0;
 $zStorageType = $zStorageType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $ts = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $ts = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zStorageType;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_getTempStore($3)|0);
 HEAP32[$ts>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 64|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$ts>>2]|0;
 $12 = ($10|0)==($11|0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = (_invalidateTempStorage($13)|0);
 $15 = ($14|0)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $16 = HEAP32[$ts>>2]|0;
  $17 = $16&255;
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 64|0);
  HEAP8[$19>>0] = $17;
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _invalidateTempStorage($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$db>>2]|0;
   $12 = ((($11)) + 63|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = ($13<<24>>24)!=(0);
   if ($14) {
    $15 = HEAP32[$db>>2]|0;
    $16 = ((($15)) + 16|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ((($17)) + 16|0);
    $19 = ((($18)) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (_sqlite3BtreeIsInReadTrans($20)|0);
    $22 = ($21|0)!=(0);
    if (!($22)) {
     $24 = HEAP32[$db>>2]|0;
     $25 = ((($24)) + 16|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ((($26)) + 16|0);
     $28 = ((($27)) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     (_sqlite3BtreeClose($29)|0);
     $30 = HEAP32[$db>>2]|0;
     $31 = ((($30)) + 16|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = ((($32)) + 16|0);
     $34 = ((($33)) + 4|0);
     HEAP32[$34>>2] = 0;
     $35 = HEAP32[$db>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($35);
     break;
    }
   }
   $23 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($23,35544,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _getSafetyLevel($z,$omitFull,$dflt) {
 $z = $z|0;
 $omitFull = $omitFull|0;
 $dflt = $dflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 17|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 16|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $omitFull;
 HEAP8[$3>>0] = $dflt;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (17533 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & 4;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$1>>2]|0;
 if ($11) {
  $13 = (_sqlite3Atoi($12)|0);
  $14 = $13&255;
  HEAP8[$0>>0] = $14;
  $46 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($46|0);
 }
 $15 = (_sqlite3Strlen30($12)|0);
 HEAP32[$n>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)<(8);
  if (!($17)) {
   label = 11;
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (35639 + ($18)|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = HEAP32[$n>>2]|0;
  $23 = ($21|0)==($22|0);
  if ($23) {
   $24 = HEAP32[$i>>2]|0;
   $25 = (35631 + ($24)|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = (35606 + ($27)|0);
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$n>>2]|0;
   $31 = (_sqlite3_strnicmp($28,$29,$30)|0);
   $32 = ($31|0)==(0);
   if ($32) {
    $33 = HEAP32[$2>>2]|0;
    $34 = ($33|0)!=(0);
    if (!($34)) {
     label = 9;
     break;
    }
    $35 = HEAP32[$i>>2]|0;
    $36 = (35647 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)<=(1);
    if ($39) {
     label = 9;
     break;
    }
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 if ((label|0) == 9) {
  $40 = HEAP32[$i>>2]|0;
  $41 = (35647 + ($40)|0);
  $42 = HEAP8[$41>>0]|0;
  HEAP8[$0>>0] = $42;
  $46 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($46|0);
 }
 else if ((label|0) == 11) {
  $45 = HEAP8[$3>>0]|0;
  HEAP8[$0>>0] = $45;
  $46 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($46|0);
 }
 return (0)|0;
}
function _setAllColumnNames($v,$N,$azCol) {
 $v = $v|0;
 $N = $N|0;
 $azCol = $azCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $azCol;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3VdbeSetNumCols($3,$4);
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + ($10<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3VdbeSetColName($8,$9,0,$13,0)|0);
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeMultiLoad($p,$iDest,$zTypes,$varargs) {
 $p = $p|0;
 $iDest = $iDest|0;
 $zTypes = $zTypes|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $arglist_current = 0;
 var $arglist_current2 = 0, $arglist_next = 0, $arglist_next3 = 0, $c = 0, $expanded = 0, $expanded10 = 0, $expanded12 = 0, $expanded13 = 0, $expanded14 = 0, $expanded3 = 0, $expanded5 = 0, $expanded6 = 0, $expanded7 = 0, $expanded9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $ap = sp + 8|0;
 $i = sp + 4|0;
 $c = sp + 36|0;
 $z = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDest;
 HEAP32[$2>>2] = $zTypes;
 HEAP32[$ap>>2] = $varargs;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = (($4) + ($3)|0);
  $6 = HEAP8[$5>>0]|0;
  HEAP8[$c>>0] = $6;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)!=(0);
  if (!($8)) {
   break;
  }
  $9 = HEAP8[$c>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = ($10|0)==(115);
  if ($11) {
   $arglist_current = HEAP32[$ap>>2]|0;
   $12 = $arglist_current;
   $13 = ((0) + 4|0);
   $expanded3 = $13;
   $expanded = (($expanded3) - 1)|0;
   $14 = (($12) + ($expanded))|0;
   $15 = ((0) + 4|0);
   $expanded7 = $15;
   $expanded6 = (($expanded7) - 1)|0;
   $expanded5 = $expanded6 ^ -1;
   $16 = $14 & $expanded5;
   $17 = $16;
   $18 = HEAP32[$17>>2]|0;
   $arglist_next = ((($17)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next;
   HEAP32[$z>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$z>>2]|0;
   $21 = ($20|0)==(0|0);
   $22 = $21 ? 25 : 97;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$1>>2] = $24;
   $25 = HEAP32[$z>>2]|0;
   (_sqlite3VdbeAddOp4($19,$22,0,$23,0,$25,0)|0);
  } else {
   $26 = HEAP32[$0>>2]|0;
   $arglist_current2 = HEAP32[$ap>>2]|0;
   $27 = $arglist_current2;
   $28 = ((0) + 4|0);
   $expanded10 = $28;
   $expanded9 = (($expanded10) - 1)|0;
   $29 = (($27) + ($expanded9))|0;
   $30 = ((0) + 4|0);
   $expanded14 = $30;
   $expanded13 = (($expanded14) - 1)|0;
   $expanded12 = $expanded13 ^ -1;
   $31 = $29 & $expanded12;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $arglist_next3 = ((($32)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next3;
   $34 = HEAP32[$1>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$1>>2] = $35;
   (_sqlite3VdbeAddOp2($26,22,$33,$34)|0);
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _actionName($action) {
 $action = $action|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $zName = sp;
 HEAP8[$0>>0] = $action;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 switch ($2|0) {
 case 7:  {
  HEAP32[$zName>>2] = 35655;
  break;
 }
 case 8:  {
  HEAP32[$zName>>2] = 35664;
  break;
 }
 case 9:  {
  HEAP32[$zName>>2] = 35676;
  break;
 }
 case 6:  {
  HEAP32[$zName>>2] = 35684;
  break;
 }
 default: {
  HEAP32[$zName>>2] = 35693;
 }
 }
 $3 = HEAP32[$zName>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3RegisterLikeFunctions($db,$caseSensitive) {
 $db = $db|0;
 $caseSensitive = $caseSensitive|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pInfo = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $caseSensitive;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$pInfo>>2] = 35703;
 } else {
  HEAP32[$pInfo>>2] = 17528;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$pInfo>>2]|0;
 (_sqlite3CreateFunc($4,32576,2,1,$5,167,0,0,0)|0);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$pInfo>>2]|0;
 (_sqlite3CreateFunc($6,32576,3,1,$7,167,0,0,0)|0);
 $8 = HEAP32[$0>>2]|0;
 (_sqlite3CreateFunc($8,32571,2,1,17524,167,0,0,0)|0);
 $9 = HEAP32[$0>>2]|0;
 _setLikeOptFlag($9,32571,12);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = $12 ? 12 : 4;
 $14 = $13&255;
 _setLikeOptFlag($10,32576,$14);
 STACKTOP = sp;return;
}
function _sqlite3WalHeapMemory($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3WalLimit($pWal,$0,$1) {
 $pWal = $pWal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pWal;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $12;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $15;
 STACKTOP = sp;return;
}
function _sqlite3PagerSetSpillsize($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 212|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3PcacheSetSpillsize($4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3PcacheSetSpillsize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ($4|0)<(0);
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ($6|0)<(0);
   $8 = $7 << 31 >> 31;
   $9 = (___muldi3(-1024,-1,($6|0),($8|0))|0);
   $10 = tempRet0;
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 28|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($13) + ($16))|0;
   $18 = ($17|0)<(0);
   $19 = $18 << 31 >> 31;
   $20 = (___divdi3(($9|0),($10|0),($17|0),($19|0))|0);
   $21 = tempRet0;
   HEAP32[$1>>2] = $20;
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 20|0);
  HEAP32[$24>>2] = $22;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = (_numberOfCachePages($25)|0);
 HEAP32[$res>>2] = $26;
 $27 = HEAP32[$res>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 20|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($27|0)<($30|0);
 if (!($31)) {
  $35 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 20|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$res>>2] = $34;
 $35 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3BtreeSetPagerFlags($p,$pgFlags) {
 $p = $p|0;
 $pgFlags = $pgFlags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pgFlags;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3PagerSetFlags($7,$8);
 STACKTOP = sp;return 0;
}
function _sqlite3PagerSetFlags($pPager,$pgFlags) {
 $pPager = $pPager|0;
 $pgFlags = $pgFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $level = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $level = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgFlags;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 7;
 HEAP32[$level>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 13|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 7|0);
  HEAP8[$9>>0] = 1;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 8|0);
  HEAP8[$11>>0] = 0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 9|0);
  HEAP8[$13>>0] = 0;
 } else {
  $14 = HEAP32[$level>>2]|0;
  $15 = ($14|0)==(1);
  $16 = $15 ? 1 : 0;
  $17 = $16&255;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 7|0);
  HEAP8[$19>>0] = $17;
  $20 = HEAP32[$level>>2]|0;
  $21 = ($20>>>0)>=(3);
  $22 = $21 ? 1 : 0;
  $23 = $22&255;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 8|0);
  HEAP8[$25>>0] = $23;
  $26 = HEAP32[$level>>2]|0;
  $27 = ($26|0)==(4);
  $28 = $27 ? 1 : 0;
  $29 = $28&255;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 9|0);
  HEAP8[$31>>0] = $29;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 7|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = ($34<<24>>24)!=(0);
 do {
  if ($35) {
   $36 = HEAP32[$0>>2]|0;
   $37 = ((($36)) + 12|0);
   HEAP8[$37>>0] = 0;
   $38 = HEAP32[$0>>2]|0;
   $39 = ((($38)) + 10|0);
   HEAP8[$39>>0] = 0;
  } else {
   $40 = HEAP32[$1>>2]|0;
   $41 = $40 & 8;
   $42 = ($41|0)!=(0);
   if ($42) {
    $43 = HEAP32[$0>>2]|0;
    $44 = ((($43)) + 12|0);
    HEAP8[$44>>0] = 3;
    $45 = HEAP32[$0>>2]|0;
    $46 = ((($45)) + 10|0);
    HEAP8[$46>>0] = 3;
    break;
   }
   $47 = HEAP32[$1>>2]|0;
   $48 = $47 & 16;
   $49 = ($48|0)!=(0);
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 12|0);
   HEAP8[$51>>0] = 2;
   $52 = HEAP32[$0>>2]|0;
   $53 = ((($52)) + 10|0);
   if ($49) {
    HEAP8[$53>>0] = 3;
    break;
   } else {
    HEAP8[$53>>0] = 2;
    break;
   }
  }
 } while(0);
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 12|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 11|0);
 HEAP8[$58>>0] = $56;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 8|0);
 $61 = HEAP8[$60>>0]|0;
 $62 = ($61<<24>>24)!=(0);
 if ($62) {
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 11|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = $66 | 32;
  $68 = $67&255;
  HEAP8[$64>>0] = $68;
 }
 $69 = HEAP32[$1>>2]|0;
 $70 = $69 & 32;
 $71 = ($70|0)!=(0);
 $72 = HEAP32[$0>>2]|0;
 $73 = ((($72)) + 21|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 if ($71) {
  $76 = $75 & -2;
  $77 = $76&255;
  HEAP8[$73>>0] = $77;
  STACKTOP = sp;return;
 } else {
  $78 = $75 | 1;
  $79 = $78&255;
  HEAP8[$73>>0] = $79;
  STACKTOP = sp;return;
 }
}
function _getTempStore($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)>=(48);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)<=(50);
  if ($9) {
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP8[$10>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = (($12) - 48)|0;
   HEAP32[$0>>2] = $13;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 do {
  if ((label|0) == 4) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (_sqlite3_stricmp($14,34444)|0);
   $16 = ($15|0)==(0);
   if ($16) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3_stricmp($17,22529)|0);
   $19 = ($18|0)==(0);
   if ($19) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _likeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $escape = 0, $nPat = 0, $or$cond = 0, $pInfo = 0, $zA = 0, $zB = 0, $zEsc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $zA = sp + 24|0;
 $zB = sp + 20|0;
 $escape = sp + 16|0;
 $nPat = sp + 12|0;
 $db = sp + 8|0;
 $pInfo = sp + 4|0;
 $zEsc = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3_user_data($5)|0);
 HEAP32[$pInfo>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zB>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3_value_text($12)|0);
 HEAP32[$zA>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_bytes($15)|0);
 HEAP32[$nPat>>2] = $16;
 $17 = HEAP32[$nPat>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 88|0);
 $20 = ((($19)) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($17|0)>($21|0);
 if ($22) {
  $23 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($23,35707,-1);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ($24|0)==(3);
 do {
  if ($25) {
   $26 = HEAP32[$2>>2]|0;
   $27 = ((($26)) + 8|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_sqlite3_value_text($28)|0);
   HEAP32[$zEsc>>2] = $29;
   $30 = HEAP32[$zEsc>>2]|0;
   $31 = ($30|0)==(0|0);
   if ($31) {
    STACKTOP = sp;return;
   }
   $32 = HEAP32[$zEsc>>2]|0;
   $33 = (_sqlite3Utf8CharLen($32,-1)|0);
   $34 = ($33|0)!=(1);
   if (!($34)) {
    $36 = (_sqlite3Utf8Read($zEsc)|0);
    HEAP32[$escape>>2] = $36;
    break;
   }
   $35 = HEAP32[$0>>2]|0;
   _sqlite3_result_error($35,35740,-1);
   STACKTOP = sp;return;
  } else {
   $37 = HEAP32[$pInfo>>2]|0;
   $38 = ((($37)) + 2|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   HEAP32[$escape>>2] = $40;
  }
 } while(0);
 $41 = HEAP32[$zA>>2]|0;
 $42 = ($41|0)!=(0|0);
 $43 = HEAP32[$zB>>2]|0;
 $44 = ($43|0)!=(0|0);
 $or$cond = $42 & $44;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$zB>>2]|0;
 $47 = HEAP32[$zA>>2]|0;
 $48 = HEAP32[$pInfo>>2]|0;
 $49 = HEAP32[$escape>>2]|0;
 $50 = (_patternCompare($46,$47,$48,$49)|0);
 _sqlite3_result_int($45,$50);
 STACKTOP = sp;return;
}
function _setLikeOptFlag($db,$zName,$flagVal) {
 $db = $db|0;
 $zName = $zName|0;
 $flagVal = $flagVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $pDef = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP8[$2>>0] = $flagVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 $7 = (_sqlite3FindFunction($3,$4,$6,2,1,0)|0);
 HEAP32[$pDef>>2] = $7;
 $8 = HEAP32[$pDef>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP8[$2>>0]|0;
 $11 = $10&255;
 $12 = HEAP32[$pDef>>2]|0;
 $13 = ((($12)) + 2|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 | $11;
 $17 = $16&65535;
 HEAP16[$13>>1] = $17;
 STACKTOP = sp;return;
}
function _sqlite3FixTriggerStep($pFix,$pStep) {
 $pFix = $pFix|0;
 $pStep = $pStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pStep;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 10;
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = ((($6)) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3FixSelect($5,$8)|0);
  $10 = ($9|0)!=(0);
  if ($10) {
   label = 4;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3FixExpr($11,$14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 6;
   break;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3FixExprList($17,$20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   label = 8;
   break;
  }
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 28|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$2>>2] = $25;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 0;
 }
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _triggerStepAllocate($db,$op,$pName) {
 $db = $db|0;
 $op = $op|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 16|0;
 $2 = sp + 8|0;
 $pTriggerStep = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP8[$1>>0] = $op;
 HEAP32[$2>>2] = $pName;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (36 + ($6))|0;
 $8 = (($7) + 1)|0;
 $9 = (_sqlite3DbMallocZero($3,$8,0)|0);
 HEAP32[$pTriggerStep>>2] = $9;
 $10 = HEAP32[$pTriggerStep>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $26 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $12 = HEAP32[$pTriggerStep>>2]|0;
 $13 = ((($12)) + 36|0);
 HEAP32[$z>>2] = $13;
 $14 = HEAP32[$z>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 _memcpy(($14|0),($16|0),($19|0))|0;
 $20 = HEAP32[$z>>2]|0;
 (_sqlite3Dequote($20)|0);
 $21 = HEAP32[$z>>2]|0;
 $22 = HEAP32[$pTriggerStep>>2]|0;
 $23 = ((($22)) + 12|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP8[$1>>0]|0;
 $25 = HEAP32[$pTriggerStep>>2]|0;
 HEAP8[$25>>0] = $24;
 $26 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _attachFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $db = 0, $flags = 0, $i = 0, $iDb = 0, $or$cond = 0, $pPager = 0, $pVfs = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0;
 var $vararg_buffer7 = 0, $vararg_buffer9 = 0, $z = 0, $zErr = 0, $zErrDyn = 0, $zFile = 0, $zName = 0, $zPath = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $i = sp + 104|0;
 $rc = sp + 100|0;
 $db = sp + 96|0;
 $zName = sp + 92|0;
 $zFile = sp + 88|0;
 $zPath = sp + 84|0;
 $zErr = sp + 80|0;
 $flags = sp + 76|0;
 $aNew = sp + 72|0;
 $zErrDyn = sp + 68|0;
 $pVfs = sp + 64|0;
 $z = sp + 60|0;
 $pPager = sp + 56|0;
 $iDb = sp + 52|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 HEAP32[$zPath>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 HEAP32[$zErrDyn>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zFile>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zName>>2] = $11;
 $12 = HEAP32[$zFile>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  HEAP32[$zFile>>2] = 17532;
 }
 $14 = HEAP32[$zName>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$zName>>2] = 17532;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$db>>2]|0;
 $20 = ((($19)) + 88|0);
 $21 = ((($20)) + 28|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 2)|0;
 $24 = ($18|0)>=($23|0);
 $25 = HEAP32[$db>>2]|0;
 do {
  if ($24) {
   $26 = HEAP32[$db>>2]|0;
   $27 = ((($26)) + 88|0);
   $28 = ((($27)) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$vararg_buffer>>2] = $29;
   $30 = (_sqlite3MPrintf($25,36159,$vararg_buffer)|0);
   HEAP32[$zErrDyn>>2] = $30;
  } else {
   $31 = ((($25)) + 63|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = ($32<<24>>24)!=(0);
   if (!($33)) {
    $34 = HEAP32[$db>>2]|0;
    $35 = (_sqlite3MPrintf($34,36196,$vararg_buffer2)|0);
    HEAP32[$zErrDyn>>2] = $35;
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $36 = HEAP32[$i>>2]|0;
    $37 = HEAP32[$db>>2]|0;
    $38 = ((($37)) + 20|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)<($39|0);
    if (!($40)) {
     break;
    }
    $41 = HEAP32[$i>>2]|0;
    $42 = HEAP32[$db>>2]|0;
    $43 = ((($42)) + 16|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + ($41<<4)|0);
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$z>>2] = $46;
    $47 = HEAP32[$z>>2]|0;
    $48 = HEAP32[$zName>>2]|0;
    $49 = (_sqlite3_stricmp($47,$48)|0);
    $50 = ($49|0)==(0);
    if ($50) {
     label = 12;
     break;
    }
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
   if ((label|0) == 12) {
    $51 = HEAP32[$db>>2]|0;
    $52 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $52;
    $53 = (_sqlite3MPrintf($51,36238,$vararg_buffer4)|0);
    HEAP32[$zErrDyn>>2] = $53;
    break;
   }
   $56 = HEAP32[$db>>2]|0;
   $57 = ((($56)) + 16|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 460|0);
   $61 = ($58|0)==($60|0);
   $62 = HEAP32[$db>>2]|0;
   do {
    if ($61) {
     $63 = (_sqlite3DbMallocRawNN($62,48,0)|0);
     HEAP32[$aNew>>2] = $63;
     $64 = HEAP32[$aNew>>2]|0;
     $65 = ($64|0)==(0|0);
     if ($65) {
      STACKTOP = sp;return;
     } else {
      $66 = HEAP32[$aNew>>2]|0;
      $67 = HEAP32[$db>>2]|0;
      $68 = ((($67)) + 16|0);
      $69 = HEAP32[$68>>2]|0;
      ;HEAP32[$66>>2]=HEAP32[$69>>2]|0;HEAP32[$66+4>>2]=HEAP32[$69+4>>2]|0;HEAP32[$66+8>>2]=HEAP32[$69+8>>2]|0;HEAP32[$66+12>>2]=HEAP32[$69+12>>2]|0;HEAP32[$66+16>>2]=HEAP32[$69+16>>2]|0;HEAP32[$66+20>>2]=HEAP32[$69+20>>2]|0;HEAP32[$66+24>>2]=HEAP32[$69+24>>2]|0;HEAP32[$66+28>>2]=HEAP32[$69+28>>2]|0;
      break;
     }
    } else {
     $70 = HEAP32[$db>>2]|0;
     $71 = ((($70)) + 16|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = HEAP32[$db>>2]|0;
     $74 = ((($73)) + 20|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + 1)|0;
     $77 = $76<<4;
     $78 = (_sqlite3DbRealloc($62,$72,$77,0)|0);
     HEAP32[$aNew>>2] = $78;
     $79 = HEAP32[$aNew>>2]|0;
     $80 = ($79|0)==(0|0);
     if ($80) {
      STACKTOP = sp;return;
     }
    }
   } while(0);
   $81 = HEAP32[$aNew>>2]|0;
   $82 = HEAP32[$db>>2]|0;
   $83 = ((($82)) + 16|0);
   HEAP32[$83>>2] = $81;
   $84 = HEAP32[$db>>2]|0;
   $85 = ((($84)) + 20|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP32[$db>>2]|0;
   $88 = ((($87)) + 16|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + ($86<<4)|0);
   HEAP32[$aNew>>2] = $90;
   $91 = HEAP32[$aNew>>2]|0;
   ;HEAP32[$91>>2]=0|0;HEAP32[$91+4>>2]=0|0;HEAP32[$91+8>>2]=0|0;HEAP32[$91+12>>2]=0|0;
   $92 = HEAP32[$db>>2]|0;
   $93 = ((($92)) + 48|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$flags>>2] = $94;
   $95 = HEAP32[$db>>2]|0;
   $96 = HEAP32[$95>>2]|0;
   $97 = ((($96)) + 16|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$zFile>>2]|0;
   $100 = (_sqlite3ParseUri($98,$99,$flags,$pVfs,$zPath,$zErr)|0);
   HEAP32[$rc>>2] = $100;
   $101 = HEAP32[$rc>>2]|0;
   $102 = ($101|0)!=(0);
   if ($102) {
    $103 = HEAP32[$rc>>2]|0;
    $104 = ($103|0)==(7);
    if ($104) {
     $105 = HEAP32[$db>>2]|0;
     _sqlite3OomFault($105);
    }
    $106 = HEAP32[$0>>2]|0;
    $107 = HEAP32[$zErr>>2]|0;
    _sqlite3_result_error($106,$107,-1);
    $108 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($108);
    STACKTOP = sp;return;
   }
   $109 = HEAP32[$flags>>2]|0;
   $110 = $109 | 256;
   HEAP32[$flags>>2] = $110;
   $111 = HEAP32[$pVfs>>2]|0;
   $112 = HEAP32[$zPath>>2]|0;
   $113 = HEAP32[$db>>2]|0;
   $114 = HEAP32[$aNew>>2]|0;
   $115 = ((($114)) + 4|0);
   $116 = HEAP32[$flags>>2]|0;
   $117 = (_sqlite3BtreeOpen($111,$112,$113,$115,0,$116)|0);
   HEAP32[$rc>>2] = $117;
   $118 = HEAP32[$zPath>>2]|0;
   _sqlite3_free($118);
   $119 = HEAP32[$db>>2]|0;
   $120 = ((($119)) + 20|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = (($121) + 1)|0;
   HEAP32[$120>>2] = $122;
   $123 = HEAP32[$rc>>2]|0;
   $124 = ($123|0)==(19);
   if ($124) {
    HEAP32[$rc>>2] = 1;
    $125 = HEAP32[$db>>2]|0;
    $126 = (_sqlite3MPrintf($125,36268,$vararg_buffer7)|0);
    HEAP32[$zErrDyn>>2] = $126;
   } else {
    $127 = HEAP32[$rc>>2]|0;
    $128 = ($127|0)==(0);
    if ($128) {
     $129 = HEAP32[$db>>2]|0;
     $130 = HEAP32[$aNew>>2]|0;
     $131 = ((($130)) + 4|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (_sqlite3SchemaGet($129,$132)|0);
     $134 = HEAP32[$aNew>>2]|0;
     $135 = ((($134)) + 12|0);
     HEAP32[$135>>2] = $133;
     $136 = HEAP32[$aNew>>2]|0;
     $137 = ((($136)) + 12|0);
     $138 = HEAP32[$137>>2]|0;
     $139 = ($138|0)!=(0|0);
     if ($139) {
      $140 = HEAP32[$aNew>>2]|0;
      $141 = ((($140)) + 12|0);
      $142 = HEAP32[$141>>2]|0;
      $143 = ((($142)) + 76|0);
      $144 = HEAP8[$143>>0]|0;
      $145 = $144&255;
      $146 = ($145|0)!=(0);
      if ($146) {
       $147 = HEAP32[$aNew>>2]|0;
       $148 = ((($147)) + 12|0);
       $149 = HEAP32[$148>>2]|0;
       $150 = ((($149)) + 77|0);
       $151 = HEAP8[$150>>0]|0;
       $152 = $151&255;
       $153 = HEAP32[$db>>2]|0;
       $154 = ((($153)) + 62|0);
       $155 = HEAP8[$154>>0]|0;
       $156 = $155&255;
       $157 = ($152|0)!=($156|0);
       if ($157) {
        $158 = HEAP32[$db>>2]|0;
        $159 = (_sqlite3MPrintf($158,36297,$vararg_buffer9)|0);
        HEAP32[$zErrDyn>>2] = $159;
        HEAP32[$rc>>2] = 1;
       }
      }
     } else {
      HEAP32[$rc>>2] = 7;
     }
     $160 = HEAP32[$aNew>>2]|0;
     $161 = ((($160)) + 4|0);
     $162 = HEAP32[$161>>2]|0;
     _sqlite3BtreeEnter($162);
     $163 = HEAP32[$aNew>>2]|0;
     $164 = ((($163)) + 4|0);
     $165 = HEAP32[$164>>2]|0;
     $166 = (_sqlite3BtreePager($165)|0);
     HEAP32[$pPager>>2] = $166;
     $167 = HEAP32[$pPager>>2]|0;
     $168 = HEAP32[$db>>2]|0;
     $169 = ((($168)) + 67|0);
     $170 = HEAP8[$169>>0]|0;
     $171 = $170&255;
     (_sqlite3PagerLockingMode($167,$171)|0);
     $172 = HEAP32[$aNew>>2]|0;
     $173 = ((($172)) + 4|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = HEAP32[$db>>2]|0;
     $176 = ((($175)) + 16|0);
     $177 = HEAP32[$176>>2]|0;
     $178 = ((($177)) + 4|0);
     $179 = HEAP32[$178>>2]|0;
     $180 = (_sqlite3BtreeSecureDelete($179,-1)|0);
     (_sqlite3BtreeSecureDelete($174,$180)|0);
     $181 = HEAP32[$aNew>>2]|0;
     $182 = ((($181)) + 4|0);
     $183 = HEAP32[$182>>2]|0;
     $184 = HEAP32[$db>>2]|0;
     $185 = ((($184)) + 24|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = $186 & 56;
     $188 = 3 | $187;
     (_sqlite3BtreeSetPagerFlags($183,$188)|0);
    }
   }
   $189 = HEAP32[$aNew>>2]|0;
   $190 = ((($189)) + 8|0);
   HEAP8[$190>>0] = 3;
   $191 = HEAP32[$db>>2]|0;
   $192 = HEAP32[$zName>>2]|0;
   $193 = (_sqlite3DbStrDup($191,$192)|0);
   $194 = HEAP32[$aNew>>2]|0;
   HEAP32[$194>>2] = $193;
   $195 = HEAP32[$rc>>2]|0;
   $196 = ($195|0)==(0);
   if ($196) {
    $197 = HEAP32[$aNew>>2]|0;
    $198 = HEAP32[$197>>2]|0;
    $199 = ($198|0)==(0|0);
    if ($199) {
     HEAP32[$rc>>2] = 7;
    }
   }
   $200 = HEAP32[$rc>>2]|0;
   $201 = ($200|0)==(0);
   if ($201) {
    $202 = HEAP32[$db>>2]|0;
    _sqlite3BtreeEnterAll($202);
    $203 = HEAP32[$db>>2]|0;
    $204 = (_sqlite3Init($203,$zErrDyn)|0);
    HEAP32[$rc>>2] = $204;
   }
   $205 = HEAP32[$rc>>2]|0;
   $206 = ($205|0)!=(0);
   if (!($206)) {
    STACKTOP = sp;return;
   }
   $207 = HEAP32[$db>>2]|0;
   $208 = ((($207)) + 20|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = (($209) - 1)|0;
   HEAP32[$iDb>>2] = $210;
   $211 = HEAP32[$iDb>>2]|0;
   $212 = HEAP32[$db>>2]|0;
   $213 = ((($212)) + 16|0);
   $214 = HEAP32[$213>>2]|0;
   $215 = (($214) + ($211<<4)|0);
   $216 = ((($215)) + 4|0);
   $217 = HEAP32[$216>>2]|0;
   $218 = ($217|0)!=(0|0);
   if ($218) {
    $219 = HEAP32[$iDb>>2]|0;
    $220 = HEAP32[$db>>2]|0;
    $221 = ((($220)) + 16|0);
    $222 = HEAP32[$221>>2]|0;
    $223 = (($222) + ($219<<4)|0);
    $224 = ((($223)) + 4|0);
    $225 = HEAP32[$224>>2]|0;
    (_sqlite3BtreeClose($225)|0);
    $226 = HEAP32[$iDb>>2]|0;
    $227 = HEAP32[$db>>2]|0;
    $228 = ((($227)) + 16|0);
    $229 = HEAP32[$228>>2]|0;
    $230 = (($229) + ($226<<4)|0);
    $231 = ((($230)) + 4|0);
    HEAP32[$231>>2] = 0;
    $232 = HEAP32[$iDb>>2]|0;
    $233 = HEAP32[$db>>2]|0;
    $234 = ((($233)) + 16|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = (($235) + ($232<<4)|0);
    $237 = ((($236)) + 12|0);
    HEAP32[$237>>2] = 0;
   }
   $238 = HEAP32[$db>>2]|0;
   _sqlite3ResetAllSchemasOfConnection($238);
   $239 = HEAP32[$iDb>>2]|0;
   $240 = HEAP32[$db>>2]|0;
   $241 = ((($240)) + 20|0);
   HEAP32[$241>>2] = $239;
   $242 = HEAP32[$rc>>2]|0;
   $243 = ($242|0)==(7);
   $244 = HEAP32[$rc>>2]|0;
   $245 = ($244|0)==(3082);
   $or$cond = $243 | $245;
   if ($or$cond) {
    $246 = HEAP32[$db>>2]|0;
    _sqlite3OomFault($246);
    $247 = HEAP32[$db>>2]|0;
    $248 = HEAP32[$zErrDyn>>2]|0;
    _sqlite3DbFree($247,$248);
    $249 = HEAP32[$db>>2]|0;
    $250 = (_sqlite3MPrintf($249,18773,$vararg_buffer11)|0);
    HEAP32[$zErrDyn>>2] = $250;
    break;
   }
   $251 = HEAP32[$zErrDyn>>2]|0;
   $252 = ($251|0)==(0|0);
   if ($252) {
    $253 = HEAP32[$db>>2]|0;
    $254 = HEAP32[$zFile>>2]|0;
    HEAP32[$vararg_buffer13>>2] = $254;
    $255 = (_sqlite3MPrintf($253,36365,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $255;
   }
  }
 } while(0);
 $256 = HEAP32[$zErrDyn>>2]|0;
 $257 = ($256|0)!=(0|0);
 if ($257) {
  $258 = HEAP32[$0>>2]|0;
  $259 = HEAP32[$zErrDyn>>2]|0;
  _sqlite3_result_error($258,$259,-1);
  $260 = HEAP32[$db>>2]|0;
  $261 = HEAP32[$zErrDyn>>2]|0;
  _sqlite3DbFree($260,$261);
 }
 $262 = HEAP32[$rc>>2]|0;
 $263 = ($262|0)!=(0);
 if (!($263)) {
  STACKTOP = sp;return;
 }
 $264 = HEAP32[$0>>2]|0;
 $265 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($264,$265);
 STACKTOP = sp;return;
}
function _codeAttach($pParse,$type,$pFunc,$pAuthArg,$pFilename,$pDbname,$pKey) {
 $pParse = $pParse|0;
 $type = $type|0;
 $pFunc = $pFunc|0;
 $pAuthArg = $pAuthArg|0;
 $pFilename = $pFilename|0;
 $pDbname = $pDbname|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, $regArgs = 0, $sName = 0, $v = 0;
 var $zAuthArg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $rc = sp + 48|0;
 $sName = sp + 16|0;
 $v = sp + 12|0;
 $db = sp + 8|0;
 $regArgs = sp + 4|0;
 $zAuthArg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $type;
 HEAP32[$2>>2] = $pFunc;
 HEAP32[$3>>2] = $pAuthArg;
 HEAP32[$4>>2] = $pFilename;
 HEAP32[$5>>2] = $pDbname;
 HEAP32[$6>>2] = $pKey;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 ;HEAP32[$sName>>2]=0|0;HEAP32[$sName+4>>2]=0|0;HEAP32[$sName+8>>2]=0|0;HEAP32[$sName+12>>2]=0|0;HEAP32[$sName+16>>2]=0|0;HEAP32[$sName+20>>2]=0|0;HEAP32[$sName+24>>2]=0|0;HEAP32[$sName+28>>2]=0|0;
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$sName>>2] = $9;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_resolveAttachExpr($sName,$10)|0);
 HEAP32[$rc>>2] = $11;
 $12 = (0)!=($11|0);
 do {
  if (!($12)) {
   $13 = HEAP32[$5>>2]|0;
   $14 = (_resolveAttachExpr($sName,$13)|0);
   HEAP32[$rc>>2] = $14;
   $15 = (0)!=($14|0);
   if (!($15)) {
    $16 = HEAP32[$6>>2]|0;
    $17 = (_resolveAttachExpr($sName,$16)|0);
    HEAP32[$rc>>2] = $17;
    $18 = (0)!=($17|0);
    if (!($18)) {
     $19 = HEAP32[$3>>2]|0;
     $20 = ($19|0)!=(0|0);
     if ($20) {
      $21 = HEAP32[$3>>2]|0;
      $22 = HEAP8[$21>>0]|0;
      $23 = $22&255;
      $24 = ($23|0)==(97);
      if ($24) {
       $25 = HEAP32[$3>>2]|0;
       $26 = ((($25)) + 8|0);
       $27 = HEAP32[$26>>2]|0;
       HEAP32[$zAuthArg>>2] = $27;
      } else {
       HEAP32[$zAuthArg>>2] = 0;
      }
      $28 = HEAP32[$0>>2]|0;
      $29 = HEAP32[$1>>2]|0;
      $30 = HEAP32[$zAuthArg>>2]|0;
      $31 = (_sqlite3AuthCheck($28,$29,$30,0,0)|0);
      HEAP32[$rc>>2] = $31;
      $32 = HEAP32[$rc>>2]|0;
      $33 = ($32|0)!=(0);
      if ($33) {
       break;
      }
     }
     $34 = HEAP32[$0>>2]|0;
     $35 = (_sqlite3GetVdbe($34)|0);
     HEAP32[$v>>2] = $35;
     $36 = HEAP32[$0>>2]|0;
     $37 = (_sqlite3GetTempRange($36,4)|0);
     HEAP32[$regArgs>>2] = $37;
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$4>>2]|0;
     $40 = HEAP32[$regArgs>>2]|0;
     _sqlite3ExprCode($38,$39,$40);
     $41 = HEAP32[$0>>2]|0;
     $42 = HEAP32[$5>>2]|0;
     $43 = HEAP32[$regArgs>>2]|0;
     $44 = (($43) + 1)|0;
     _sqlite3ExprCode($41,$42,$44);
     $45 = HEAP32[$0>>2]|0;
     $46 = HEAP32[$6>>2]|0;
     $47 = HEAP32[$regArgs>>2]|0;
     $48 = (($47) + 2)|0;
     _sqlite3ExprCode($45,$46,$48);
     $49 = HEAP32[$v>>2]|0;
     $50 = ($49|0)!=(0|0);
     if ($50) {
      $51 = HEAP32[$v>>2]|0;
      $52 = HEAP32[$regArgs>>2]|0;
      $53 = (($52) + 3)|0;
      $54 = HEAP32[$2>>2]|0;
      $55 = HEAP16[$54>>1]|0;
      $56 = $55 << 16 >> 16;
      $57 = (($53) - ($56))|0;
      $58 = HEAP32[$regArgs>>2]|0;
      $59 = (($58) + 3)|0;
      $60 = HEAP32[$2>>2]|0;
      (_sqlite3VdbeAddOp4($51,35,0,$57,$59,$60,-5)|0);
      $61 = HEAP32[$v>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = HEAP16[$62>>1]|0;
      $64 = $63&255;
      _sqlite3VdbeChangeP5($61,$64);
      $65 = HEAP32[$v>>2]|0;
      $66 = HEAP32[$1>>2]|0;
      $67 = ($66|0)==(24);
      $68 = $67&1;
      (_sqlite3VdbeAddOp1($65,147,$68)|0);
     }
    }
   }
  }
 } while(0);
 $69 = HEAP32[$db>>2]|0;
 $70 = HEAP32[$4>>2]|0;
 _sqlite3ExprDelete($69,$70);
 $71 = HEAP32[$db>>2]|0;
 $72 = HEAP32[$5>>2]|0;
 _sqlite3ExprDelete($71,$72);
 $73 = HEAP32[$db>>2]|0;
 $74 = HEAP32[$6>>2]|0;
 _sqlite3ExprDelete($73,$74);
 STACKTOP = sp;return;
}
function _sqlite3ParseUri($zDefaultVfs,$zUri,$pFlags,$ppVfs,$pzFile,$pzErrMsg) {
 $zDefaultVfs = $zDefaultVfs|0;
 $zUri = $zUri|0;
 $pFlags = $pFlags|0;
 $ppVfs = $ppVfs|0;
 $pzFile = $pzFile|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMode = 0, $c = 0, $eState = 0, $flags = 0, $i = 0, $iIn = 0, $iOut = 0, $limit = 0, $mask = 0, $mode = 0, $nByte = 0, $nOpt = 0;
 var $nUri = 0, $nVal = 0, $octet = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer17 = 0, $vararg_buffer21 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr20 = 0, $z = 0, $zFile = 0, $zModeType = 0, $zOpt = 0, $zVal = 0, $zVfs = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer21 = sp + 32|0;
 $vararg_buffer17 = sp + 24|0;
 $vararg_buffer13 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 120|0;
 $6 = sp + 116|0;
 $rc = sp + 112|0;
 $flags = sp + 108|0;
 $zVfs = sp + 104|0;
 $zFile = sp + 100|0;
 $c = sp + 144|0;
 $nUri = sp + 96|0;
 $zOpt = sp + 92|0;
 $eState = sp + 88|0;
 $iIn = sp + 84|0;
 $iOut = sp + 80|0;
 $nByte = sp;
 $octet = sp + 76|0;
 $nOpt = sp + 72|0;
 $zVal = sp + 68|0;
 $nVal = sp + 64|0;
 $aMode = sp + 60|0;
 $zModeType = sp + 56|0;
 $mask = sp + 52|0;
 $limit = sp + 48|0;
 $i = sp + 44|0;
 $mode = sp + 40|0;
 $z = sp + 36|0;
 HEAP32[$1>>2] = $zDefaultVfs;
 HEAP32[$2>>2] = $zUri;
 HEAP32[$3>>2] = $pFlags;
 HEAP32[$4>>2] = $ppVfs;
 HEAP32[$5>>2] = $pzFile;
 HEAP32[$6>>2] = $pzErrMsg;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$flags>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$zVfs>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3Strlen30($10)|0);
 HEAP32[$nUri>>2] = $11;
 $12 = HEAP32[$flags>>2]|0;
 $13 = $12 & 64;
 $14 = ($13|0)!=(0);
 $15 = HEAP32[(20)>>2]|0;
 $16 = ($15|0)!=(0);
 $or$cond = $14 | $16;
 $17 = HEAP32[$nUri>>2]|0;
 $18 = ($17|0)>=(5);
 $or$cond3 = $or$cond & $18;
 L1: do {
  if ($or$cond3) {
   $19 = HEAP32[$2>>2]|0;
   $20 = (_memcmp($19,36393,5)|0);
   $21 = ($20|0)==(0);
   if ($21) {
    HEAP32[$iOut>>2] = 0;
    $22 = HEAP32[$nUri>>2]|0;
    $23 = (($22) + 2)|0;
    $24 = ($23|0)<(0);
    $25 = $24 << 31 >> 31;
    $26 = $nByte;
    $27 = $26;
    HEAP32[$27>>2] = $23;
    $28 = (($26) + 4)|0;
    $29 = $28;
    HEAP32[$29>>2] = $25;
    $30 = HEAP32[$flags>>2]|0;
    $31 = $30 | 64;
    HEAP32[$flags>>2] = $31;
    HEAP32[$iIn>>2] = 0;
    while(1) {
     $32 = HEAP32[$iIn>>2]|0;
     $33 = HEAP32[$nUri>>2]|0;
     $34 = ($32|0)<($33|0);
     if (!($34)) {
      break;
     }
     $35 = HEAP32[$iIn>>2]|0;
     $36 = HEAP32[$2>>2]|0;
     $37 = (($36) + ($35)|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38 << 24 >> 24;
     $40 = ($39|0)==(38);
     $41 = $40&1;
     $42 = ($41|0)<(0);
     $43 = $42 << 31 >> 31;
     $44 = $nByte;
     $45 = $44;
     $46 = HEAP32[$45>>2]|0;
     $47 = (($44) + 4)|0;
     $48 = $47;
     $49 = HEAP32[$48>>2]|0;
     $50 = (_i64Add(($46|0),($49|0),($41|0),($43|0))|0);
     $51 = tempRet0;
     $52 = $nByte;
     $53 = $52;
     HEAP32[$53>>2] = $50;
     $54 = (($52) + 4)|0;
     $55 = $54;
     HEAP32[$55>>2] = $51;
     $56 = HEAP32[$iIn>>2]|0;
     $57 = (($56) + 1)|0;
     HEAP32[$iIn>>2] = $57;
    }
    $58 = $nByte;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (_sqlite3_malloc64($60,$63)|0);
    HEAP32[$zFile>>2] = $64;
    $65 = HEAP32[$zFile>>2]|0;
    $66 = ($65|0)!=(0|0);
    if (!($66)) {
     HEAP32[$0>>2] = 7;
     $386 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($386|0);
    }
    HEAP32[$iIn>>2] = 5;
    $67 = HEAP32[$2>>2]|0;
    $68 = ((($67)) + 5|0);
    $69 = HEAP8[$68>>0]|0;
    $70 = $69 << 24 >> 24;
    $71 = ($70|0)==(47);
    do {
     if ($71) {
      $72 = HEAP32[$2>>2]|0;
      $73 = ((($72)) + 6|0);
      $74 = HEAP8[$73>>0]|0;
      $75 = $74 << 24 >> 24;
      $76 = ($75|0)==(47);
      if ($76) {
       HEAP32[$iIn>>2] = 7;
       while(1) {
        $77 = HEAP32[$iIn>>2]|0;
        $78 = HEAP32[$2>>2]|0;
        $79 = (($78) + ($77)|0);
        $80 = HEAP8[$79>>0]|0;
        $81 = $80 << 24 >> 24;
        $82 = ($81|0)!=(0);
        if ($82) {
         $83 = HEAP32[$iIn>>2]|0;
         $84 = HEAP32[$2>>2]|0;
         $85 = (($84) + ($83)|0);
         $86 = HEAP8[$85>>0]|0;
         $87 = $86 << 24 >> 24;
         $88 = ($87|0)!=(47);
         $387 = $88;
        } else {
         $387 = 0;
        }
        $89 = HEAP32[$iIn>>2]|0;
        if (!($387)) {
         break;
        }
        $90 = (($89) + 1)|0;
        HEAP32[$iIn>>2] = $90;
       }
       $91 = ($89|0)!=(7);
       if ($91) {
        $92 = HEAP32[$iIn>>2]|0;
        $93 = ($92|0)!=(16);
        if (!($93)) {
         $94 = HEAP32[$2>>2]|0;
         $95 = ((($94)) + 7|0);
         $96 = (_memcmp(36399,$95,9)|0);
         $97 = ($96|0)!=(0);
         if (!($97)) {
          break;
         }
        }
        $98 = HEAP32[$iIn>>2]|0;
        $99 = (($98) - 7)|0;
        $100 = HEAP32[$2>>2]|0;
        $101 = ((($100)) + 7|0);
        HEAP32[$vararg_buffer>>2] = $99;
        $vararg_ptr12 = ((($vararg_buffer)) + 4|0);
        HEAP32[$vararg_ptr12>>2] = $101;
        $102 = (_sqlite3_mprintf(36409,$vararg_buffer)|0);
        $103 = HEAP32[$6>>2]|0;
        HEAP32[$103>>2] = $102;
        HEAP32[$rc>>2] = 1;
        break L1;
       }
      }
     }
    } while(0);
    HEAP32[$eState>>2] = 0;
    L27: while(1) {
     $104 = HEAP32[$iIn>>2]|0;
     $105 = HEAP32[$2>>2]|0;
     $106 = (($105) + ($104)|0);
     $107 = HEAP8[$106>>0]|0;
     HEAP8[$c>>0] = $107;
     $108 = $107 << 24 >> 24;
     $109 = ($108|0)!=(0);
     if (!($109)) {
      break;
     }
     $110 = HEAP8[$c>>0]|0;
     $111 = $110 << 24 >> 24;
     $112 = ($111|0)!=(35);
     if (!($112)) {
      break;
     }
     $113 = HEAP32[$iIn>>2]|0;
     $114 = (($113) + 1)|0;
     HEAP32[$iIn>>2] = $114;
     $115 = HEAP8[$c>>0]|0;
     $116 = $115 << 24 >> 24;
     $117 = ($116|0)==(37);
     do {
      if ($117) {
       $118 = HEAP32[$iIn>>2]|0;
       $119 = HEAP32[$2>>2]|0;
       $120 = (($119) + ($118)|0);
       $121 = HEAP8[$120>>0]|0;
       $122 = $121&255;
       $123 = (17533 + ($122)|0);
       $124 = HEAP8[$123>>0]|0;
       $125 = $124&255;
       $126 = $125 & 8;
       $127 = ($126|0)!=(0);
       if ($127) {
        $128 = HEAP32[$iIn>>2]|0;
        $129 = (($128) + 1)|0;
        $130 = HEAP32[$2>>2]|0;
        $131 = (($130) + ($129)|0);
        $132 = HEAP8[$131>>0]|0;
        $133 = $132&255;
        $134 = (17533 + ($133)|0);
        $135 = HEAP8[$134>>0]|0;
        $136 = $135&255;
        $137 = $136 & 8;
        $138 = ($137|0)!=(0);
        if ($138) {
         $139 = HEAP32[$iIn>>2]|0;
         $140 = (($139) + 1)|0;
         HEAP32[$iIn>>2] = $140;
         $141 = HEAP32[$2>>2]|0;
         $142 = (($141) + ($139)|0);
         $143 = HEAP8[$142>>0]|0;
         $144 = $143 << 24 >> 24;
         $145 = (_sqlite3HexToInt($144)|0);
         $146 = $145&255;
         $147 = $146 << 4;
         HEAP32[$octet>>2] = $147;
         $148 = HEAP32[$iIn>>2]|0;
         $149 = (($148) + 1)|0;
         HEAP32[$iIn>>2] = $149;
         $150 = HEAP32[$2>>2]|0;
         $151 = (($150) + ($148)|0);
         $152 = HEAP8[$151>>0]|0;
         $153 = $152 << 24 >> 24;
         $154 = (_sqlite3HexToInt($153)|0);
         $155 = $154&255;
         $156 = HEAP32[$octet>>2]|0;
         $157 = (($156) + ($155))|0;
         HEAP32[$octet>>2] = $157;
         $158 = HEAP32[$octet>>2]|0;
         $159 = ($158|0)==(0);
         if (!($159)) {
          $189 = HEAP32[$octet>>2]|0;
          $190 = $189&255;
          HEAP8[$c>>0] = $190;
          break;
         }
         while(1) {
          $160 = HEAP32[$iIn>>2]|0;
          $161 = HEAP32[$2>>2]|0;
          $162 = (($161) + ($160)|0);
          $163 = HEAP8[$162>>0]|0;
          HEAP8[$c>>0] = $163;
          $164 = $163 << 24 >> 24;
          $165 = ($164|0)!=(0);
          if (!($165)) {
           continue L27;
          }
          $166 = HEAP8[$c>>0]|0;
          $167 = $166 << 24 >> 24;
          $168 = ($167|0)!=(35);
          if (!($168)) {
           continue L27;
          }
          $169 = HEAP32[$eState>>2]|0;
          $170 = ($169|0)!=(0);
          if (!($170)) {
           $171 = HEAP8[$c>>0]|0;
           $172 = $171 << 24 >> 24;
           $173 = ($172|0)!=(63);
           if (!($173)) {
            continue L27;
           }
          }
          $174 = HEAP32[$eState>>2]|0;
          $175 = ($174|0)!=(1);
          if (!($175)) {
           $176 = HEAP8[$c>>0]|0;
           $177 = $176 << 24 >> 24;
           $178 = ($177|0)!=(61);
           if (!($178)) {
            continue L27;
           }
           $179 = HEAP8[$c>>0]|0;
           $180 = $179 << 24 >> 24;
           $181 = ($180|0)!=(38);
           if (!($181)) {
            continue L27;
           }
          }
          $182 = HEAP32[$eState>>2]|0;
          $183 = ($182|0)!=(2);
          if (!($183)) {
           $184 = HEAP8[$c>>0]|0;
           $185 = $184 << 24 >> 24;
           $186 = ($185|0)!=(38);
           if (!($186)) {
            continue L27;
           }
          }
          $187 = HEAP32[$iIn>>2]|0;
          $188 = (($187) + 1)|0;
          HEAP32[$iIn>>2] = $188;
         }
        } else {
         label = 37;
        }
       } else {
        label = 37;
       }
      } else {
       label = 37;
      }
     } while(0);
     L51: do {
      if ((label|0) == 37) {
       label = 0;
       $191 = HEAP32[$eState>>2]|0;
       $192 = ($191|0)==(1);
       do {
        if ($192) {
         $193 = HEAP8[$c>>0]|0;
         $194 = $193 << 24 >> 24;
         $195 = ($194|0)==(38);
         if (!($195)) {
          $196 = HEAP8[$c>>0]|0;
          $197 = $196 << 24 >> 24;
          $198 = ($197|0)==(61);
          if (!($198)) {
           break;
          }
         }
         $199 = HEAP32[$iOut>>2]|0;
         $200 = (($199) - 1)|0;
         $201 = HEAP32[$zFile>>2]|0;
         $202 = (($201) + ($200)|0);
         $203 = HEAP8[$202>>0]|0;
         $204 = $203 << 24 >> 24;
         $205 = ($204|0)==(0);
         if ($205) {
          while(1) {
           $206 = HEAP32[$iIn>>2]|0;
           $207 = HEAP32[$2>>2]|0;
           $208 = (($207) + ($206)|0);
           $209 = HEAP8[$208>>0]|0;
           $210 = $209 << 24 >> 24;
           $211 = ($210|0)!=(0);
           if (!($211)) {
            continue L27;
           }
           $212 = HEAP32[$iIn>>2]|0;
           $213 = HEAP32[$2>>2]|0;
           $214 = (($213) + ($212)|0);
           $215 = HEAP8[$214>>0]|0;
           $216 = $215 << 24 >> 24;
           $217 = ($216|0)!=(35);
           if (!($217)) {
            continue L27;
           }
           $218 = HEAP32[$iIn>>2]|0;
           $219 = (($218) - 1)|0;
           $220 = HEAP32[$2>>2]|0;
           $221 = (($220) + ($219)|0);
           $222 = HEAP8[$221>>0]|0;
           $223 = $222 << 24 >> 24;
           $224 = ($223|0)!=(38);
           if (!($224)) {
            continue L27;
           }
           $225 = HEAP32[$iIn>>2]|0;
           $226 = (($225) + 1)|0;
           HEAP32[$iIn>>2] = $226;
          }
         } else {
          $227 = HEAP8[$c>>0]|0;
          $228 = $227 << 24 >> 24;
          $229 = ($228|0)==(38);
          if ($229) {
           $230 = HEAP32[$iOut>>2]|0;
           $231 = (($230) + 1)|0;
           HEAP32[$iOut>>2] = $231;
           $232 = HEAP32[$zFile>>2]|0;
           $233 = (($232) + ($230)|0);
           HEAP8[$233>>0] = 0;
          } else {
           HEAP32[$eState>>2] = 2;
          }
          HEAP8[$c>>0] = 0;
          break L51;
         }
        }
       } while(0);
       $234 = HEAP32[$eState>>2]|0;
       $235 = ($234|0)==(0);
       if ($235) {
        $236 = HEAP8[$c>>0]|0;
        $237 = $236 << 24 >> 24;
        $238 = ($237|0)==(63);
        if (!($238)) {
         label = 51;
        }
       } else {
        label = 51;
       }
       if ((label|0) == 51) {
        label = 0;
        $239 = HEAP32[$eState>>2]|0;
        $240 = ($239|0)==(2);
        if (!($240)) {
         break;
        }
        $241 = HEAP8[$c>>0]|0;
        $242 = $241 << 24 >> 24;
        $243 = ($242|0)==(38);
        if (!($243)) {
         break;
        }
       }
       HEAP8[$c>>0] = 0;
       HEAP32[$eState>>2] = 1;
      }
     } while(0);
     $244 = HEAP8[$c>>0]|0;
     $245 = HEAP32[$iOut>>2]|0;
     $246 = (($245) + 1)|0;
     HEAP32[$iOut>>2] = $246;
     $247 = HEAP32[$zFile>>2]|0;
     $248 = (($247) + ($245)|0);
     HEAP8[$248>>0] = $244;
    }
    $249 = HEAP32[$eState>>2]|0;
    $250 = ($249|0)==(1);
    if ($250) {
     $251 = HEAP32[$iOut>>2]|0;
     $252 = (($251) + 1)|0;
     HEAP32[$iOut>>2] = $252;
     $253 = HEAP32[$zFile>>2]|0;
     $254 = (($253) + ($251)|0);
     HEAP8[$254>>0] = 0;
    }
    $255 = HEAP32[$iOut>>2]|0;
    $256 = (($255) + 1)|0;
    HEAP32[$iOut>>2] = $256;
    $257 = HEAP32[$zFile>>2]|0;
    $258 = (($257) + ($255)|0);
    HEAP8[$258>>0] = 0;
    $259 = HEAP32[$iOut>>2]|0;
    $260 = (($259) + 1)|0;
    HEAP32[$iOut>>2] = $260;
    $261 = HEAP32[$zFile>>2]|0;
    $262 = (($261) + ($259)|0);
    HEAP8[$262>>0] = 0;
    $263 = HEAP32[$zFile>>2]|0;
    $264 = (_sqlite3Strlen30($263)|0);
    $265 = (($264) + 1)|0;
    $266 = HEAP32[$zFile>>2]|0;
    $267 = (($266) + ($265)|0);
    HEAP32[$zOpt>>2] = $267;
    while(1) {
     $268 = HEAP32[$zOpt>>2]|0;
     $269 = HEAP8[$268>>0]|0;
     $270 = ($269<<24>>24)!=(0);
     if (!($270)) {
      label = 84;
      break L1;
     }
     $271 = HEAP32[$zOpt>>2]|0;
     $272 = (_sqlite3Strlen30($271)|0);
     HEAP32[$nOpt>>2] = $272;
     $273 = HEAP32[$nOpt>>2]|0;
     $274 = (($273) + 1)|0;
     $275 = HEAP32[$zOpt>>2]|0;
     $276 = (($275) + ($274)|0);
     HEAP32[$zVal>>2] = $276;
     $277 = HEAP32[$zVal>>2]|0;
     $278 = (_sqlite3Strlen30($277)|0);
     HEAP32[$nVal>>2] = $278;
     $279 = HEAP32[$nOpt>>2]|0;
     $280 = ($279|0)==(3);
     if ($280) {
      $281 = HEAP32[$zOpt>>2]|0;
      $282 = (_memcmp(36437,$281,3)|0);
      $283 = ($282|0)==(0);
      if ($283) {
       $284 = HEAP32[$zVal>>2]|0;
       HEAP32[$zVfs>>2] = $284;
      } else {
       label = 62;
      }
     } else {
      label = 62;
     }
     if ((label|0) == 62) {
      label = 0;
      HEAP32[$aMode>>2] = 0;
      HEAP32[$zModeType>>2] = 0;
      HEAP32[$mask>>2] = 0;
      HEAP32[$limit>>2] = 0;
      $285 = HEAP32[$nOpt>>2]|0;
      $286 = ($285|0)==(5);
      if ($286) {
       $287 = HEAP32[$zOpt>>2]|0;
       $288 = (_memcmp(36441,$287,5)|0);
       $289 = ($288|0)==(0);
       if ($289) {
        HEAP32[$mask>>2] = 393216;
        HEAP32[$aMode>>2] = 3632;
        $290 = HEAP32[$mask>>2]|0;
        HEAP32[$limit>>2] = $290;
        HEAP32[$zModeType>>2] = 36441;
       }
      }
      $291 = HEAP32[$nOpt>>2]|0;
      $292 = ($291|0)==(4);
      if ($292) {
       $293 = HEAP32[$zOpt>>2]|0;
       $294 = (_memcmp(36462,$293,4)|0);
       $295 = ($294|0)==(0);
       if ($295) {
        HEAP32[$mask>>2] = 135;
        HEAP32[$aMode>>2] = 3656;
        $296 = HEAP32[$mask>>2]|0;
        $297 = HEAP32[$flags>>2]|0;
        $298 = $296 & $297;
        HEAP32[$limit>>2] = $298;
        HEAP32[$zModeType>>2] = 18333;
       }
      }
      $299 = HEAP32[$aMode>>2]|0;
      $300 = ($299|0)!=(0|0);
      if ($300) {
       HEAP32[$mode>>2] = 0;
       HEAP32[$i>>2] = 0;
       while(1) {
        $301 = HEAP32[$i>>2]|0;
        $302 = HEAP32[$aMode>>2]|0;
        $303 = (($302) + ($301<<3)|0);
        $304 = HEAP32[$303>>2]|0;
        $305 = ($304|0)!=(0|0);
        if (!($305)) {
         break;
        }
        $306 = HEAP32[$i>>2]|0;
        $307 = HEAP32[$aMode>>2]|0;
        $308 = (($307) + ($306<<3)|0);
        $309 = HEAP32[$308>>2]|0;
        HEAP32[$z>>2] = $309;
        $310 = HEAP32[$nVal>>2]|0;
        $311 = HEAP32[$z>>2]|0;
        $312 = (_sqlite3Strlen30($311)|0);
        $313 = ($310|0)==($312|0);
        if ($313) {
         $314 = HEAP32[$zVal>>2]|0;
         $315 = HEAP32[$z>>2]|0;
         $316 = HEAP32[$nVal>>2]|0;
         $317 = (_memcmp($314,$315,$316)|0);
         $318 = (0)==($317|0);
         if ($318) {
          label = 73;
          break;
         }
        }
        $324 = HEAP32[$i>>2]|0;
        $325 = (($324) + 1)|0;
        HEAP32[$i>>2] = $325;
       }
       if ((label|0) == 73) {
        label = 0;
        $319 = HEAP32[$i>>2]|0;
        $320 = HEAP32[$aMode>>2]|0;
        $321 = (($320) + ($319<<3)|0);
        $322 = ((($321)) + 4|0);
        $323 = HEAP32[$322>>2]|0;
        HEAP32[$mode>>2] = $323;
       }
       $326 = HEAP32[$mode>>2]|0;
       $327 = ($326|0)==(0);
       if ($327) {
        label = 76;
        break;
       }
       $332 = HEAP32[$mode>>2]|0;
       $333 = $332 & -129;
       $334 = HEAP32[$limit>>2]|0;
       $335 = ($333|0)>($334|0);
       if ($335) {
        label = 78;
        break;
       }
       $340 = HEAP32[$flags>>2]|0;
       $341 = HEAP32[$mask>>2]|0;
       $342 = $341 ^ -1;
       $343 = $340 & $342;
       $344 = HEAP32[$mode>>2]|0;
       $345 = $343 | $344;
       HEAP32[$flags>>2] = $345;
      }
     }
     $346 = HEAP32[$nVal>>2]|0;
     $347 = (($346) + 1)|0;
     $348 = HEAP32[$zVal>>2]|0;
     $349 = (($348) + ($347)|0);
     HEAP32[$zOpt>>2] = $349;
    }
    if ((label|0) == 76) {
     $328 = HEAP32[$zModeType>>2]|0;
     $329 = HEAP32[$zVal>>2]|0;
     HEAP32[$vararg_buffer13>>2] = $328;
     $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
     HEAP32[$vararg_ptr16>>2] = $329;
     $330 = (_sqlite3_mprintf(36477,$vararg_buffer13)|0);
     $331 = HEAP32[$6>>2]|0;
     HEAP32[$331>>2] = $330;
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 78) {
     $336 = HEAP32[$zModeType>>2]|0;
     $337 = HEAP32[$zVal>>2]|0;
     HEAP32[$vararg_buffer17>>2] = $336;
     $vararg_ptr20 = ((($vararg_buffer17)) + 4|0);
     HEAP32[$vararg_ptr20>>2] = $337;
     $338 = (_sqlite3_mprintf(36497,$vararg_buffer17)|0);
     $339 = HEAP32[$6>>2]|0;
     HEAP32[$339>>2] = $338;
     HEAP32[$rc>>2] = 3;
     break;
    }
   } else {
    label = 81;
   }
  } else {
   label = 81;
  }
 } while(0);
 do {
  if ((label|0) == 81) {
   $350 = HEAP32[$nUri>>2]|0;
   $351 = (($350) + 2)|0;
   $352 = ($351|0)<(0);
   $353 = $352 << 31 >> 31;
   $354 = (_sqlite3_malloc64($351,$353)|0);
   HEAP32[$zFile>>2] = $354;
   $355 = HEAP32[$zFile>>2]|0;
   $356 = ($355|0)!=(0|0);
   if ($356) {
    $357 = HEAP32[$zFile>>2]|0;
    $358 = HEAP32[$2>>2]|0;
    $359 = HEAP32[$nUri>>2]|0;
    _memcpy(($357|0),($358|0),($359|0))|0;
    $360 = HEAP32[$nUri>>2]|0;
    $361 = HEAP32[$zFile>>2]|0;
    $362 = (($361) + ($360)|0);
    HEAP8[$362>>0] = 0;
    $363 = HEAP32[$nUri>>2]|0;
    $364 = (($363) + 1)|0;
    $365 = HEAP32[$zFile>>2]|0;
    $366 = (($365) + ($364)|0);
    HEAP8[$366>>0] = 0;
    $367 = HEAP32[$flags>>2]|0;
    $368 = $367 & -65;
    HEAP32[$flags>>2] = $368;
    label = 84;
    break;
   }
   HEAP32[$0>>2] = 7;
   $386 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($386|0);
  }
 } while(0);
 if ((label|0) == 84) {
  $369 = HEAP32[$zVfs>>2]|0;
  $370 = (_sqlite3_vfs_find($369)|0);
  $371 = HEAP32[$4>>2]|0;
  HEAP32[$371>>2] = $370;
  $372 = HEAP32[$4>>2]|0;
  $373 = HEAP32[$372>>2]|0;
  $374 = ($373|0)==(0|0);
  if ($374) {
   $375 = HEAP32[$zVfs>>2]|0;
   HEAP32[$vararg_buffer21>>2] = $375;
   $376 = (_sqlite3_mprintf(36521,$vararg_buffer21)|0);
   $377 = HEAP32[$6>>2]|0;
   HEAP32[$377>>2] = $376;
   HEAP32[$rc>>2] = 1;
  }
 }
 $378 = HEAP32[$rc>>2]|0;
 $379 = ($378|0)!=(0);
 if ($379) {
  $380 = HEAP32[$zFile>>2]|0;
  _sqlite3_free($380);
  HEAP32[$zFile>>2] = 0;
 }
 $381 = HEAP32[$flags>>2]|0;
 $382 = HEAP32[$3>>2]|0;
 HEAP32[$382>>2] = $381;
 $383 = HEAP32[$zFile>>2]|0;
 $384 = HEAP32[$5>>2]|0;
 HEAP32[$384>>2] = $383;
 $385 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $385;
 $386 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($386|0);
}
function _sqlite3SchemaGet($db,$pBt) {
 $db = $db|0;
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pBt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3BtreeSchema($4,84,144)|0);
  HEAP32[$p>>2] = $5;
 } else {
  $6 = (_sqlite3DbMallocZero(0,84,0)|0);
  HEAP32[$p>>2] = $6;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3OomFault($9);
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 76|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = (0)==($13|0);
 if (!($14)) {
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 8|0);
 _sqlite3HashInit($16);
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 24|0);
 _sqlite3HashInit($18);
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 40|0);
 _sqlite3HashInit($20);
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 56|0);
 _sqlite3HashInit($22);
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 77|0);
 HEAP8[$24>>0] = 1;
 $25 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _resolveAttachExpr($pName,$pExpr) {
 $pName = $pName|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pName;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = HEAP8[$4>>0]|0;
   $6 = $5&255;
   $7 = ($6|0)!=(27);
   if ($7) {
    $8 = HEAP32[$0>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = (_sqlite3ResolveExprNames($8,$9)|0);
    HEAP32[$rc>>2] = $10;
    break;
   } else {
    $11 = HEAP32[$1>>2]|0;
    HEAP8[$11>>0] = 97;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _detachFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pDb = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer6 = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $zName = sp + 40|0;
 $db = sp + 36|0;
 $i = sp + 32|0;
 $pDb = sp + 28|0;
 $zErr = sp + 56|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zName>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3_context_db_handle($6)|0);
 HEAP32[$db>>2] = $7;
 HEAP32[$pDb>>2] = 0;
 $8 = HEAP32[$zName>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$zName>>2] = 17532;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$db>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$db>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  HEAP32[$pDb>>2] = $19;
  $20 = HEAP32[$pDb>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)==(0|0);
  if (!($23)) {
   $24 = HEAP32[$pDb>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$zName>>2]|0;
   $27 = (_sqlite3_stricmp($25,$26)|0);
   $28 = ($27|0)==(0);
   if ($28) {
    break;
   }
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$db>>2]|0;
 $33 = ((($32)) + 20|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($31|0)>=($34|0);
 do {
  if ($35) {
   $36 = HEAP32[$zName>>2]|0;
   HEAP32[$vararg_buffer>>2] = $36;
   (_sqlite3_snprintf(128,$zErr,36551,$vararg_buffer)|0);
  } else {
   $37 = HEAP32[$i>>2]|0;
   $38 = ($37|0)<(2);
   if ($38) {
    $39 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $39;
    (_sqlite3_snprintf(128,$zErr,36572,$vararg_buffer1)|0);
    break;
   }
   $40 = HEAP32[$db>>2]|0;
   $41 = ((($40)) + 63|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = ($42<<24>>24)!=(0);
   if (!($43)) {
    (_sqlite3_snprintf(128,$zErr,36598,$vararg_buffer4)|0);
    break;
   }
   $44 = HEAP32[$pDb>>2]|0;
   $45 = ((($44)) + 4|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (_sqlite3BtreeIsInReadTrans($46)|0);
   $48 = ($47|0)!=(0);
   if (!($48)) {
    $49 = HEAP32[$pDb>>2]|0;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (_sqlite3BtreeIsInBackup($51)|0);
    $53 = ($52|0)!=(0);
    if (!($53)) {
     $55 = HEAP32[$pDb>>2]|0;
     $56 = ((($55)) + 4|0);
     $57 = HEAP32[$56>>2]|0;
     (_sqlite3BtreeClose($57)|0);
     $58 = HEAP32[$pDb>>2]|0;
     $59 = ((($58)) + 4|0);
     HEAP32[$59>>2] = 0;
     $60 = HEAP32[$pDb>>2]|0;
     $61 = ((($60)) + 12|0);
     HEAP32[$61>>2] = 0;
     $62 = HEAP32[$db>>2]|0;
     _sqlite3CollapseDatabaseArray($62);
     STACKTOP = sp;return;
    }
   }
   $54 = HEAP32[$zName>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $54;
   (_sqlite3_snprintf(128,$zErr,36640,$vararg_buffer6)|0);
  }
 } while(0);
 $63 = HEAP32[$0>>2]|0;
 _sqlite3_result_error($63,$zErr,-1);
 STACKTOP = sp;return;
}
function _sqlite3BtreeIsInBackup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 $5 = $4&1;
 STACKTOP = sp;return ($5|0);
}
function _reindexDatabases($pParse,$zColl) {
 $pParse = $pParse|0;
 $zColl = $zColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $k = 0, $pDb = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pDb = sp + 16|0;
 $iDb = sp + 12|0;
 $db = sp + 8|0;
 $k = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zColl;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$iDb>>2] = 0;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pDb>>2] = $6;
 while(1) {
  $7 = HEAP32[$iDb>>2]|0;
  $8 = HEAP32[$db>>2]|0;
  $9 = ((($8)) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pDb>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$k>>2] = $17;
  while(1) {
   $18 = HEAP32[$k>>2]|0;
   $19 = ($18|0)!=(0|0);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$k>>2]|0;
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pTab>>2] = $22;
   $23 = HEAP32[$0>>2]|0;
   $24 = HEAP32[$pTab>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   _reindexTable($23,$24,$25);
   $26 = HEAP32[$k>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$k>>2] = $27;
  }
  $28 = HEAP32[$iDb>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$iDb>>2] = $29;
  $30 = HEAP32[$pDb>>2]|0;
  $31 = ((($30)) + 16|0);
  HEAP32[$pDb>>2] = $31;
 }
 STACKTOP = sp;return;
}
function _reindexTable($pParse,$pTab,$zColl) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $zColl = $zColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $pIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pIndex = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zColl;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pIndex>>2] = $5;
 while(1) {
  $6 = HEAP32[$pIndex>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 5;
  } else {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$pIndex>>2]|0;
   $12 = (_collationMatch($10,$11)|0);
   $13 = ($12|0)!=(0);
   if ($13) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 64|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3SchemaToIndex($15,$18)|0);
   HEAP32[$iDb>>2] = $19;
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$iDb>>2]|0;
   _sqlite3BeginWriteOperation($20,0,$21);
   $22 = HEAP32[$0>>2]|0;
   $23 = HEAP32[$pIndex>>2]|0;
   _sqlite3RefillIndex($22,$23,-1);
  }
  $24 = HEAP32[$pIndex>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pIndex>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _collationMatch($zColl,$pIndex) {
 $zColl = $zColl|0;
 $pIndex = $pIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $zColl;
 HEAP32[$2>>2] = $pIndex;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = ((($4)) + 52|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 7;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$z>>2] = $14;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<1)|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = ($21|0)>=(0);
  if ($22) {
   $23 = HEAP32[$z>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_sqlite3_stricmp($23,$24)|0);
   $26 = (0)==($25|0);
   if ($26) {
    label = 5;
    break;
   }
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _analyzeDatabase($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iMem = 0, $iStatCur = 0, $iTab = 0, $k = 0, $pSchema = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $db = sp + 24|0;
 $pSchema = sp + 20|0;
 $k = sp + 16|0;
 $iStatCur = sp + 12|0;
 $iMem = sp + 8|0;
 $iTab = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pSchema>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 _sqlite3BeginWriteOperation($11,0,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 72|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$iStatCur>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 72|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 3)|0;
 HEAP32[$17>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$iStatCur>>2]|0;
 _openStatTable($20,$21,$22,0,0);
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 76|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$iMem>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 72|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$iTab>>2] = $29;
 $30 = HEAP32[$pSchema>>2]|0;
 $31 = ((($30)) + 8|0);
 $32 = ((($31)) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$k>>2] = $33;
 while(1) {
  $34 = HEAP32[$k>>2]|0;
  $35 = ($34|0)!=(0|0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$k>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$pTab>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$pTab>>2]|0;
  $41 = HEAP32[$iStatCur>>2]|0;
  $42 = HEAP32[$iMem>>2]|0;
  $43 = HEAP32[$iTab>>2]|0;
  _analyzeOneTable($39,$40,0,$41,$42,$43);
  $44 = HEAP32[$k>>2]|0;
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$k>>2] = $45;
 }
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 _loadAnalysis($46,$47);
 STACKTOP = sp;return;
}
function _analyzeTable($pParse,$pTab,$pOnlyIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pOnlyIdx = $pOnlyIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iDb = 0, $iStatCur = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $iDb = sp + 4|0;
 $iStatCur = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pOnlyIdx;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3SchemaToIndex($4,$7)|0);
 HEAP32[$iDb>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($9,0,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 72|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$iStatCur>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 72|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 3)|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = ($18|0)!=(0|0);
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = HEAP32[$iStatCur>>2]|0;
 if ($19) {
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  _openStatTable($20,$21,$22,$24,33991);
 } else {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  _openStatTable($20,$21,$22,$26,31847);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$iStatCur>>2]|0;
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 76|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + 1)|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 72|0);
 $37 = HEAP32[$36>>2]|0;
 _analyzeOneTable($27,$28,$29,$30,$34,$37);
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$iDb>>2]|0;
 _loadAnalysis($38,$39);
 STACKTOP = sp;return;
}
function _openStatTable($pParse,$iDb,$iStatCur,$zWhere,$zWhereType) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $iStatCur = $iStatCur|0;
 $zWhere = $zWhere|0;
 $zWhereType = $zWhereType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aCreateTbl = 0, $aRoot = 0, $db = 0, $i = 0, $pDb = 0, $pStat = 0;
 var $v = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $i = sp + 64|0;
 $db = sp + 60|0;
 $pDb = sp + 56|0;
 $v = sp + 52|0;
 $aRoot = sp + 40|0;
 $aCreateTbl = sp + 88|0;
 $zTab = sp + 36|0;
 $pStat = sp + 32|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iStatCur;
 HEAP32[$3>>2] = $zWhere;
 HEAP32[$4>>2] = $zWhereType;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<4)|0);
 HEAP32[$pDb>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)<(3);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (3728 + ($18<<3)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$zTab>>2] = $20;
  $21 = HEAP32[$db>>2]|0;
  $22 = HEAP32[$zTab>>2]|0;
  $23 = HEAP32[$pDb>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3FindTable($21,$22,$24)|0);
  HEAP32[$pStat>>2] = $25;
  $26 = ($25|0)==(0|0);
  do {
   if ($26) {
    $27 = HEAP32[$i>>2]|0;
    $28 = (3728 + ($27<<3)|0);
    $29 = ((($28)) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)!=(0|0);
    if ($31) {
     $32 = HEAP32[$0>>2]|0;
     $33 = HEAP32[$pDb>>2]|0;
     $34 = HEAP32[$33>>2]|0;
     $35 = HEAP32[$zTab>>2]|0;
     $36 = HEAP32[$i>>2]|0;
     $37 = (3728 + ($36<<3)|0);
     $38 = ((($37)) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     HEAP32[$vararg_buffer>>2] = $34;
     $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr1>>2] = $35;
     $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
     HEAP32[$vararg_ptr2>>2] = $39;
     _sqlite3NestedParse($32,36747,$vararg_buffer);
     $40 = HEAP32[$0>>2]|0;
     $41 = ((($40)) + 396|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = HEAP32[$i>>2]|0;
     $44 = (($aRoot) + ($43<<2)|0);
     HEAP32[$44>>2] = $42;
     $45 = HEAP32[$i>>2]|0;
     $46 = (($aCreateTbl) + ($45)|0);
     HEAP8[$46>>0] = 16;
    }
   } else {
    $47 = HEAP32[$pStat>>2]|0;
    $48 = ((($47)) + 28|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = HEAP32[$i>>2]|0;
    $51 = (($aRoot) + ($50<<2)|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$i>>2]|0;
    $53 = (($aCreateTbl) + ($52)|0);
    HEAP8[$53>>0] = 0;
    $54 = HEAP32[$0>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = HEAP32[$i>>2]|0;
    $57 = (($aRoot) + ($56<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$zTab>>2]|0;
    _sqlite3TableLock($54,$55,$58,1,$59);
    $60 = HEAP32[$3>>2]|0;
    $61 = ($60|0)!=(0|0);
    if ($61) {
     $62 = HEAP32[$0>>2]|0;
     $63 = HEAP32[$pDb>>2]|0;
     $64 = HEAP32[$63>>2]|0;
     $65 = HEAP32[$zTab>>2]|0;
     $66 = HEAP32[$4>>2]|0;
     $67 = HEAP32[$3>>2]|0;
     HEAP32[$vararg_buffer3>>2] = $64;
     $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
     HEAP32[$vararg_ptr6>>2] = $65;
     $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
     HEAP32[$vararg_ptr7>>2] = $66;
     $vararg_ptr8 = ((($vararg_buffer3)) + 12|0);
     HEAP32[$vararg_ptr8>>2] = $67;
     _sqlite3NestedParse($62,31865,$vararg_buffer3);
     break;
    } else {
     $68 = HEAP32[$v>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = (($aRoot) + ($69<<2)|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = HEAP32[$1>>2]|0;
     (_sqlite3VdbeAddOp2($68,119,$71,$72)|0);
     break;
    }
   }
  } while(0);
  $73 = HEAP32[$i>>2]|0;
  $74 = (($73) + 1)|0;
  HEAP32[$i>>2] = $74;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $75 = HEAP32[$i>>2]|0;
  $76 = (3728 + ($75<<3)|0);
  $77 = ((($76)) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(0|0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$v>>2]|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = HEAP32[$i>>2]|0;
  $83 = (($81) + ($82))|0;
  $84 = HEAP32[$i>>2]|0;
  $85 = (($aRoot) + ($84<<2)|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeAddOp4Int($80,55,$83,$86,$87,3)|0);
  $88 = HEAP32[$v>>2]|0;
  $89 = HEAP32[$i>>2]|0;
  $90 = (($aCreateTbl) + ($89)|0);
  $91 = HEAP8[$90>>0]|0;
  _sqlite3VdbeChangeP5($88,$91);
  $92 = HEAP32[$i>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$i>>2] = $93;
 }
 STACKTOP = sp;return;
}
function _analyzeOneTable($pParse,$pTab,$pOnlyIdx,$iStatCur,$iMem,$iTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pOnlyIdx = $pOnlyIdx|0;
 $iStatCur = $iStatCur|0;
 $iMem = $iMem|0;
 $iTab = $iTab|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aGotoChng = 0, $addrNextRow = 0, $addrRewind = 0;
 var $db = 0, $endDistinctTest = 0, $i = 0, $iDb = 0, $iIdxCur = 0, $iTabCur = 0, $jZeroRows = 0, $nCol = 0, $nColTest = 0, $needTableCnt = 0, $or$cond = 0, $pColl = 0, $pIdx = 0, $regChng = 0, $regIdxname = 0, $regNewRowid = 0, $regPrev = 0, $regStat1 = 0, $regStat4 = 0, $regTabname = 0;
 var $regTemp = 0, $v = 0, $zIdxName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $db = sp + 92|0;
 $pIdx = sp + 88|0;
 $iIdxCur = sp + 84|0;
 $iTabCur = sp + 80|0;
 $v = sp + 76|0;
 $i = sp + 72|0;
 $jZeroRows = sp + 68|0;
 $iDb = sp + 64|0;
 $needTableCnt = sp + 120|0;
 $regNewRowid = sp + 60|0;
 $regStat4 = sp + 56|0;
 $regChng = sp + 52|0;
 $regTemp = sp + 48|0;
 $regTabname = sp + 44|0;
 $regIdxname = sp + 40|0;
 $regStat1 = sp + 36|0;
 $regPrev = sp + 32|0;
 $nCol = sp + 28|0;
 $addrRewind = sp + 24|0;
 $addrNextRow = sp + 20|0;
 $zIdxName = sp + 16|0;
 $nColTest = sp + 12|0;
 $endDistinctTest = sp + 8|0;
 $aGotoChng = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pOnlyIdx;
 HEAP32[$3>>2] = $iStatCur;
 HEAP32[$4>>2] = $iMem;
 HEAP32[$5>>2] = $iTab;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$jZeroRows>>2] = -1;
 HEAP8[$needTableCnt>>0] = 1;
 $8 = HEAP32[$4>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$4>>2] = $9;
 HEAP32[$regNewRowid>>2] = $8;
 $10 = HEAP32[$4>>2]|0;
 $11 = (($10) + 1)|0;
 HEAP32[$4>>2] = $11;
 HEAP32[$regStat4>>2] = $10;
 $12 = HEAP32[$4>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$4>>2] = $13;
 HEAP32[$regChng>>2] = $12;
 $14 = HEAP32[$4>>2]|0;
 $15 = (($14) + 1)|0;
 HEAP32[$4>>2] = $15;
 HEAP32[$regTemp>>2] = $14;
 $16 = HEAP32[$4>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$4>>2] = $17;
 HEAP32[$regTabname>>2] = $16;
 $18 = HEAP32[$4>>2]|0;
 $19 = (($18) + 1)|0;
 HEAP32[$4>>2] = $19;
 HEAP32[$regIdxname>>2] = $18;
 $20 = HEAP32[$4>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$4>>2] = $21;
 HEAP32[$regStat1>>2] = $20;
 $22 = HEAP32[$4>>2]|0;
 HEAP32[$regPrev>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 76|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 $27 = ($25|0)>($26|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 76|0);
  $30 = HEAP32[$29>>2]|0;
  $34 = $30;
 } else {
  $31 = HEAP32[$4>>2]|0;
  $34 = $31;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 76|0);
 HEAP32[$33>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 $36 = (_sqlite3GetVdbe($35)|0);
 HEAP32[$v>>2] = $36;
 $37 = HEAP32[$v>>2]|0;
 $38 = ($37|0)==(0|0);
 $39 = HEAP32[$1>>2]|0;
 $40 = ($39|0)==(0|0);
 $or$cond = $38 | $40;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 28|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3_strlike(36770,$46,0)|0);
 $48 = ($47|0)==(0);
 if ($48) {
  STACKTOP = sp;return;
 }
 $49 = HEAP32[$db>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 64|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (_sqlite3SchemaToIndex($49,$52)|0);
 HEAP32[$iDb>>2] = $53;
 $54 = HEAP32[$0>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$iDb>>2]|0;
 $58 = HEAP32[$db>>2]|0;
 $59 = ((($58)) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + ($57<<4)|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3AuthCheck($54,28,$56,0,$62)|0);
 $64 = ($63|0)!=(0);
 if ($64) {
  STACKTOP = sp;return;
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$iDb>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = ((($67)) + 28|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$1>>2]|0;
 $71 = HEAP32[$70>>2]|0;
 _sqlite3TableLock($65,$66,$69,0,$71);
 $72 = HEAP32[$5>>2]|0;
 $73 = (($72) + 1)|0;
 HEAP32[$5>>2] = $73;
 HEAP32[$iTabCur>>2] = $72;
 $74 = HEAP32[$5>>2]|0;
 $75 = (($74) + 1)|0;
 HEAP32[$5>>2] = $75;
 HEAP32[$iIdxCur>>2] = $74;
 $76 = HEAP32[$0>>2]|0;
 $77 = ((($76)) + 72|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = ($78|0)>($79|0);
 if ($80) {
  $81 = HEAP32[$0>>2]|0;
  $82 = ((($81)) + 72|0);
  $83 = HEAP32[$82>>2]|0;
  $87 = $83;
 } else {
  $84 = HEAP32[$5>>2]|0;
  $87 = $84;
 }
 $85 = HEAP32[$0>>2]|0;
 $86 = ((($85)) + 72|0);
 HEAP32[$86>>2] = $87;
 $88 = HEAP32[$0>>2]|0;
 $89 = HEAP32[$iTabCur>>2]|0;
 $90 = HEAP32[$iDb>>2]|0;
 $91 = HEAP32[$1>>2]|0;
 _sqlite3OpenTable($88,$89,$90,$91,54);
 $92 = HEAP32[$v>>2]|0;
 $93 = HEAP32[$regTabname>>2]|0;
 $94 = HEAP32[$1>>2]|0;
 $95 = HEAP32[$94>>2]|0;
 (_sqlite3VdbeLoadString($92,$93,$95)|0);
 $96 = HEAP32[$1>>2]|0;
 $97 = ((($96)) + 8|0);
 $98 = HEAP32[$97>>2]|0;
 HEAP32[$pIdx>>2] = $98;
 while(1) {
  $99 = HEAP32[$pIdx>>2]|0;
  $100 = ($99|0)!=(0|0);
  $101 = HEAP32[$2>>2]|0;
  if (!($100)) {
   break;
  }
  $102 = ($101|0)!=(0|0);
  if ($102) {
   $103 = HEAP32[$2>>2]|0;
   $104 = HEAP32[$pIdx>>2]|0;
   $105 = ($103|0)!=($104|0);
   if (!($105)) {
    label = 15;
   }
  } else {
   label = 15;
  }
  do {
   if ((label|0) == 15) {
    label = 0;
    $106 = HEAP32[$pIdx>>2]|0;
    $107 = ((($106)) + 36|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = ($108|0)==(0|0);
    if ($109) {
     HEAP8[$needTableCnt>>0] = 0;
    }
    $110 = HEAP32[$1>>2]|0;
    $111 = ((($110)) + 42|0);
    $112 = HEAP8[$111>>0]|0;
    $113 = $112&255;
    $114 = $113 & 32;
    $115 = ($114|0)==(0);
    if ($115) {
     label = 20;
    } else {
     $116 = HEAP32[$pIdx>>2]|0;
     $117 = ((($116)) + 55|0);
     $118 = HEAP8[$117>>0]|0;
     $119 = $118 & 3;
     $120 = $119&255;
     $121 = ($120|0)==(2);
     if ($121) {
      $122 = HEAP32[$pIdx>>2]|0;
      $123 = ((($122)) + 50|0);
      $124 = HEAP16[$123>>1]|0;
      $125 = $124&65535;
      HEAP32[$nCol>>2] = $125;
      $126 = HEAP32[$1>>2]|0;
      $127 = HEAP32[$126>>2]|0;
      HEAP32[$zIdxName>>2] = $127;
      $128 = HEAP32[$nCol>>2]|0;
      $129 = (($128) - 1)|0;
      HEAP32[$nColTest>>2] = $129;
     } else {
      label = 20;
     }
    }
    if ((label|0) == 20) {
     label = 0;
     $130 = HEAP32[$pIdx>>2]|0;
     $131 = ((($130)) + 52|0);
     $132 = HEAP16[$131>>1]|0;
     $133 = $132&65535;
     HEAP32[$nCol>>2] = $133;
     $134 = HEAP32[$pIdx>>2]|0;
     $135 = HEAP32[$134>>2]|0;
     HEAP32[$zIdxName>>2] = $135;
     $136 = HEAP32[$pIdx>>2]|0;
     $137 = ((($136)) + 55|0);
     $138 = HEAP8[$137>>0]|0;
     $139 = ($138&255) >>> 3;
     $140 = $139 & 1;
     $141 = $140&255;
     $142 = ($141|0)!=(0);
     if ($142) {
      $143 = HEAP32[$pIdx>>2]|0;
      $144 = ((($143)) + 50|0);
      $145 = HEAP16[$144>>1]|0;
      $146 = $145&65535;
      $$sink = $146;
     } else {
      $147 = HEAP32[$nCol>>2]|0;
      $$sink = $147;
     }
     $148 = (($$sink) - 1)|0;
     HEAP32[$nColTest>>2] = $148;
    }
    $149 = HEAP32[$v>>2]|0;
    $150 = HEAP32[$regIdxname>>2]|0;
    $151 = HEAP32[$zIdxName>>2]|0;
    (_sqlite3VdbeLoadString($149,$150,$151)|0);
    $152 = HEAP32[$0>>2]|0;
    $153 = ((($152)) + 76|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = HEAP32[$regPrev>>2]|0;
    $156 = HEAP32[$nColTest>>2]|0;
    $157 = (($155) + ($156))|0;
    $158 = ($154|0)>($157|0);
    if ($158) {
     $159 = HEAP32[$0>>2]|0;
     $160 = ((($159)) + 76|0);
     $161 = HEAP32[$160>>2]|0;
     $167 = $161;
    } else {
     $162 = HEAP32[$regPrev>>2]|0;
     $163 = HEAP32[$nColTest>>2]|0;
     $164 = (($162) + ($163))|0;
     $167 = $164;
    }
    $165 = HEAP32[$0>>2]|0;
    $166 = ((($165)) + 76|0);
    HEAP32[$166>>2] = $167;
    $168 = HEAP32[$v>>2]|0;
    $169 = HEAP32[$iIdxCur>>2]|0;
    $170 = HEAP32[$pIdx>>2]|0;
    $171 = ((($170)) + 44|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$iDb>>2]|0;
    (_sqlite3VdbeAddOp3($168,54,$169,$172,$173)|0);
    $174 = HEAP32[$0>>2]|0;
    $175 = HEAP32[$pIdx>>2]|0;
    _sqlite3VdbeSetP4KeyInfo($174,$175);
    $176 = HEAP32[$v>>2]|0;
    $177 = HEAP32[$nCol>>2]|0;
    $178 = HEAP32[$regStat4>>2]|0;
    $179 = (($178) + 1)|0;
    (_sqlite3VdbeAddOp2($176,22,$177,$179)|0);
    $180 = HEAP32[$v>>2]|0;
    $181 = HEAP32[$pIdx>>2]|0;
    $182 = ((($181)) + 50|0);
    $183 = HEAP16[$182>>1]|0;
    $184 = $183&65535;
    $185 = HEAP32[$regStat4>>2]|0;
    $186 = (($185) + 2)|0;
    (_sqlite3VdbeAddOp2($180,22,$184,$186)|0);
    $187 = HEAP32[$v>>2]|0;
    $188 = HEAP32[$regStat4>>2]|0;
    $189 = (($188) + 1)|0;
    $190 = HEAP32[$regStat4>>2]|0;
    (_sqlite3VdbeAddOp4($187,35,0,$189,$190,3752,-5)|0);
    $191 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($191,2);
    $192 = HEAP32[$v>>2]|0;
    $193 = HEAP32[$iIdxCur>>2]|0;
    $194 = (_sqlite3VdbeAddOp1($192,108,$193)|0);
    HEAP32[$addrRewind>>2] = $194;
    $195 = HEAP32[$v>>2]|0;
    $196 = HEAP32[$regChng>>2]|0;
    (_sqlite3VdbeAddOp2($195,22,0,$196)|0);
    $197 = HEAP32[$v>>2]|0;
    $198 = (_sqlite3VdbeCurrentAddr($197)|0);
    HEAP32[$addrNextRow>>2] = $198;
    $199 = HEAP32[$nColTest>>2]|0;
    $200 = ($199|0)>(0);
    if ($200) {
     $201 = HEAP32[$v>>2]|0;
     $202 = (_sqlite3VdbeMakeLabel($201)|0);
     HEAP32[$endDistinctTest>>2] = $202;
     $203 = HEAP32[$db>>2]|0;
     $204 = HEAP32[$nColTest>>2]|0;
     $205 = $204<<2;
     $206 = (_sqlite3DbMallocRawNN($203,$205,0)|0);
     HEAP32[$aGotoChng>>2] = $206;
     $207 = HEAP32[$aGotoChng>>2]|0;
     $208 = ($207|0)==(0|0);
     if ($208) {
      break;
     }
     $209 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($209,13)|0);
     $210 = HEAP32[$v>>2]|0;
     $211 = (_sqlite3VdbeCurrentAddr($210)|0);
     HEAP32[$addrNextRow>>2] = $211;
     $212 = HEAP32[$nColTest>>2]|0;
     $213 = ($212|0)==(1);
     if ($213) {
      $214 = HEAP32[$pIdx>>2]|0;
      $215 = ((($214)) + 50|0);
      $216 = HEAP16[$215>>1]|0;
      $217 = $216&65535;
      $218 = ($217|0)==(1);
      if ($218) {
       $219 = HEAP32[$pIdx>>2]|0;
       $220 = ((($219)) + 54|0);
       $221 = HEAP8[$220>>0]|0;
       $222 = $221&255;
       $223 = ($222|0)!=(0);
       if ($223) {
        $224 = HEAP32[$v>>2]|0;
        $225 = HEAP32[$regPrev>>2]|0;
        $226 = HEAP32[$endDistinctTest>>2]|0;
        (_sqlite3VdbeAddOp2($224,77,$225,$226)|0);
       }
      }
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $227 = HEAP32[$i>>2]|0;
      $228 = HEAP32[$nColTest>>2]|0;
      $229 = ($227|0)<($228|0);
      if (!($229)) {
       break;
      }
      $230 = HEAP32[$0>>2]|0;
      $231 = HEAP32[$i>>2]|0;
      $232 = HEAP32[$pIdx>>2]|0;
      $233 = ((($232)) + 32|0);
      $234 = HEAP32[$233>>2]|0;
      $235 = (($234) + ($231<<2)|0);
      $236 = HEAP32[$235>>2]|0;
      $237 = (_sqlite3LocateCollSeq($230,$236)|0);
      HEAP32[$pColl>>2] = $237;
      $238 = HEAP32[$v>>2]|0;
      $239 = HEAP32[$i>>2]|0;
      $240 = HEAP32[$regChng>>2]|0;
      (_sqlite3VdbeAddOp2($238,22,$239,$240)|0);
      $241 = HEAP32[$v>>2]|0;
      $242 = HEAP32[$iIdxCur>>2]|0;
      $243 = HEAP32[$i>>2]|0;
      $244 = HEAP32[$regTemp>>2]|0;
      (_sqlite3VdbeAddOp3($241,47,$242,$243,$244)|0);
      $245 = HEAP32[$v>>2]|0;
      $246 = HEAP32[$regTemp>>2]|0;
      $247 = HEAP32[$regPrev>>2]|0;
      $248 = HEAP32[$i>>2]|0;
      $249 = (($247) + ($248))|0;
      $250 = HEAP32[$pColl>>2]|0;
      $251 = (_sqlite3VdbeAddOp4($245,78,$246,0,$249,$250,-4)|0);
      $252 = HEAP32[$i>>2]|0;
      $253 = HEAP32[$aGotoChng>>2]|0;
      $254 = (($253) + ($252<<2)|0);
      HEAP32[$254>>2] = $251;
      $255 = HEAP32[$v>>2]|0;
      _sqlite3VdbeChangeP5($255,-128);
      $256 = HEAP32[$i>>2]|0;
      $257 = (($256) + 1)|0;
      HEAP32[$i>>2] = $257;
     }
     $258 = HEAP32[$v>>2]|0;
     $259 = HEAP32[$nColTest>>2]|0;
     $260 = HEAP32[$regChng>>2]|0;
     (_sqlite3VdbeAddOp2($258,22,$259,$260)|0);
     $261 = HEAP32[$v>>2]|0;
     $262 = HEAP32[$endDistinctTest>>2]|0;
     (_sqlite3VdbeGoto($261,$262)|0);
     $263 = HEAP32[$v>>2]|0;
     $264 = HEAP32[$addrNextRow>>2]|0;
     $265 = (($264) - 1)|0;
     _sqlite3VdbeJumpHere($263,$265);
     HEAP32[$i>>2] = 0;
     while(1) {
      $266 = HEAP32[$i>>2]|0;
      $267 = HEAP32[$nColTest>>2]|0;
      $268 = ($266|0)<($267|0);
      $269 = HEAP32[$v>>2]|0;
      if (!($268)) {
       break;
      }
      $270 = HEAP32[$i>>2]|0;
      $271 = HEAP32[$aGotoChng>>2]|0;
      $272 = (($271) + ($270<<2)|0);
      $273 = HEAP32[$272>>2]|0;
      _sqlite3VdbeJumpHere($269,$273);
      $274 = HEAP32[$v>>2]|0;
      $275 = HEAP32[$iIdxCur>>2]|0;
      $276 = HEAP32[$i>>2]|0;
      $277 = HEAP32[$regPrev>>2]|0;
      $278 = HEAP32[$i>>2]|0;
      $279 = (($277) + ($278))|0;
      (_sqlite3VdbeAddOp3($274,47,$275,$276,$279)|0);
      $280 = HEAP32[$i>>2]|0;
      $281 = (($280) + 1)|0;
      HEAP32[$i>>2] = $281;
     }
     $282 = HEAP32[$endDistinctTest>>2]|0;
     _sqlite3VdbeResolveLabel($269,$282);
     $283 = HEAP32[$db>>2]|0;
     $284 = HEAP32[$aGotoChng>>2]|0;
     _sqlite3DbFree($283,$284);
    }
    $285 = HEAP32[$v>>2]|0;
    $286 = HEAP32[$regStat4>>2]|0;
    $287 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp4($285,35,1,$286,$287,3784,-5)|0);
    $288 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($288,2);
    $289 = HEAP32[$v>>2]|0;
    $290 = HEAP32[$iIdxCur>>2]|0;
    $291 = HEAP32[$addrNextRow>>2]|0;
    (_sqlite3VdbeAddOp2($289,7,$290,$291)|0);
    $292 = HEAP32[$v>>2]|0;
    $293 = HEAP32[$regStat4>>2]|0;
    $294 = HEAP32[$regStat1>>2]|0;
    _callStatGet($292,$293,0,$294);
    $295 = HEAP32[$v>>2]|0;
    $296 = HEAP32[$regTabname>>2]|0;
    $297 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp4($295,49,$296,3,$297,36779,0)|0);
    $298 = HEAP32[$v>>2]|0;
    $299 = HEAP32[$3>>2]|0;
    $300 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp2($298,74,$299,$300)|0);
    $301 = HEAP32[$v>>2]|0;
    $302 = HEAP32[$3>>2]|0;
    $303 = HEAP32[$regTemp>>2]|0;
    $304 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp3($301,75,$302,$303,$304)|0);
    $305 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($305,8);
    $306 = HEAP32[$v>>2]|0;
    $307 = HEAP32[$addrRewind>>2]|0;
    _sqlite3VdbeJumpHere($306,$307);
   }
  } while(0);
  $308 = HEAP32[$pIdx>>2]|0;
  $309 = ((($308)) + 20|0);
  $310 = HEAP32[$309>>2]|0;
  HEAP32[$pIdx>>2] = $310;
 }
 $311 = ($101|0)==(0|0);
 if (!($311)) {
  STACKTOP = sp;return;
 }
 $312 = HEAP8[$needTableCnt>>0]|0;
 $313 = $312&255;
 $314 = ($313|0)!=(0);
 if (!($314)) {
  STACKTOP = sp;return;
 }
 $315 = HEAP32[$v>>2]|0;
 $316 = HEAP32[$iTabCur>>2]|0;
 $317 = HEAP32[$regStat1>>2]|0;
 (_sqlite3VdbeAddOp2($315,50,$316,$317)|0);
 $318 = HEAP32[$v>>2]|0;
 $319 = HEAP32[$regStat1>>2]|0;
 $320 = (_sqlite3VdbeAddOp1($318,46,$319)|0);
 HEAP32[$jZeroRows>>2] = $320;
 $321 = HEAP32[$v>>2]|0;
 $322 = HEAP32[$regIdxname>>2]|0;
 (_sqlite3VdbeAddOp2($321,25,0,$322)|0);
 $323 = HEAP32[$v>>2]|0;
 $324 = HEAP32[$regTabname>>2]|0;
 $325 = HEAP32[$regTemp>>2]|0;
 (_sqlite3VdbeAddOp4($323,49,$324,3,$325,36779,0)|0);
 $326 = HEAP32[$v>>2]|0;
 $327 = HEAP32[$3>>2]|0;
 $328 = HEAP32[$regNewRowid>>2]|0;
 (_sqlite3VdbeAddOp2($326,74,$327,$328)|0);
 $329 = HEAP32[$v>>2]|0;
 $330 = HEAP32[$3>>2]|0;
 $331 = HEAP32[$regTemp>>2]|0;
 $332 = HEAP32[$regNewRowid>>2]|0;
 (_sqlite3VdbeAddOp3($329,75,$330,$331,$332)|0);
 $333 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($333,8);
 $334 = HEAP32[$v>>2]|0;
 $335 = HEAP32[$jZeroRows>>2]|0;
 _sqlite3VdbeJumpHere($334,$335);
 STACKTOP = sp;return;
}
function _loadAnalysis($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$v>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp1($6,124,$7)|0);
 STACKTOP = sp;return;
}
function _callStatGet($v,$regStat4,$iParam,$regOut) {
 $v = $v|0;
 $regStat4 = $regStat4|0;
 $iParam = $iParam|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $regStat4;
 HEAP32[$2>>2] = $iParam;
 HEAP32[$3>>2] = $regOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4($4,35,0,$5,$6,3816,-5)|0);
 $7 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP5($7,1);
 STACKTOP = sp;return;
}
function _statInit($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $nCol = 0, $nColUp = 0, $nKeyCol = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $p = sp + 20|0;
 $nCol = sp + 16|0;
 $nKeyCol = sp + 12|0;
 $nColUp = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$nCol>>2] = $5;
 $6 = HEAP32[$nCol>>2]|0;
 $7 = (($6) + 1)|0;
 $8 = $7 & -2;
 HEAP32[$nColUp>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_int($11)|0);
 HEAP32[$nKeyCol>>2] = $12;
 $13 = HEAP32[$nColUp>>2]|0;
 $14 = $13<<2;
 $15 = (56 + ($14))|0;
 $16 = HEAP32[$nColUp>>2]|0;
 $17 = $16<<2;
 $18 = (($15) + ($17))|0;
 HEAP32[$n>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3_context_db_handle($19)|0);
 HEAP32[$db>>2] = $20;
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$n>>2]|0;
 $23 = ($22|0)<(0);
 $24 = $23 << 31 >> 31;
 $25 = (_sqlite3DbMallocZero($21,$22,$24)|0);
 HEAP32[$p>>2] = $25;
 $26 = HEAP32[$p>>2]|0;
 $27 = ($26|0)==(0|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($28);
  STACKTOP = sp;return;
 } else {
  $29 = HEAP32[$db>>2]|0;
  $30 = HEAP32[$p>>2]|0;
  $31 = ((($30)) + 52|0);
  HEAP32[$31>>2] = $29;
  $32 = HEAP32[$p>>2]|0;
  HEAP32[$32>>2] = 0;
  $33 = HEAP32[$nCol>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $33;
  $36 = HEAP32[$nKeyCol>>2]|0;
  $37 = HEAP32[$p>>2]|0;
  $38 = ((($37)) + 12|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 56|0);
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 20|0);
  $43 = ((($42)) + 4|0);
  HEAP32[$43>>2] = $40;
  $44 = HEAP32[$nColUp>>2]|0;
  $45 = HEAP32[$p>>2]|0;
  $46 = ((($45)) + 20|0);
  $47 = ((($46)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($44<<2)|0);
  $50 = HEAP32[$p>>2]|0;
  $51 = ((($50)) + 20|0);
  HEAP32[$51>>2] = $49;
  $52 = HEAP32[$0>>2]|0;
  $53 = HEAP32[$p>>2]|0;
  _sqlite3_result_blob($52,$53,56,145);
  STACKTOP = sp;return;
 }
}
function _stat4Destructor($pOld) {
 $pOld = $pOld|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pOld;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 _sqlite3DbFree($4,$5);
 STACKTOP = sp;return;
}
function _statPush($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $iChng = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $p = sp + 4|0;
 $iChng = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_int($8)|0);
 HEAP32[$iChng>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $13 = HEAP32[$i>>2]|0;
   $14 = HEAP32[$p>>2]|0;
   $15 = ((($14)) + 8|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($13|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18<<2)|0);
   HEAP32[$22>>2] = 1;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$i>>2] = $24;
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 1)|0;
  HEAP32[$60>>2] = $62;
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$p>>2]|0;
 $26 = HEAP32[$iChng>>2]|0;
 _samplePushPrevious($25,$26);
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$iChng>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$p>>2]|0;
  $32 = ((($31)) + 20|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$34>>2] = $36;
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 $39 = HEAP32[$iChng>>2]|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$p>>2]|0;
  $47 = ((($46)) + 20|0);
  $48 = ((($47)) + 4|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($45<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$50>>2] = $52;
  $53 = HEAP32[$i>>2]|0;
  $54 = HEAP32[$p>>2]|0;
  $55 = ((($54)) + 20|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + ($53<<2)|0);
  HEAP32[$57>>2] = 1;
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 $60 = HEAP32[$p>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = (($61) + 1)|0;
 HEAP32[$60>>2] = $62;
 STACKTOP = sp;return;
}
function _samplePushPrevious($p,$iChng) {
 $p = $p|0;
 $iChng = $iChng|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iChng;
 STACKTOP = sp;return;
}
function _statGet($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $iVal = 0, $nDistinct = 0, $p = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $z = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 24|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $p = sp + 44|0;
 $z = sp + 40|0;
 $i = sp + 36|0;
 $zRet = sp + 32|0;
 $nDistinct = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 $10 = ($9*25)|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (_sqlite3MallocZero($10,$12)|0);
 HEAP32[$zRet>>2] = $13;
 $14 = HEAP32[$zRet>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($16);
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$zRet>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = $vararg_buffer;
 $21 = $20;
 HEAP32[$21>>2] = $19;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = 0;
 (_sqlite3_snprintf(24,$17,36812,$vararg_buffer)|0);
 $24 = HEAP32[$zRet>>2]|0;
 $25 = HEAP32[$zRet>>2]|0;
 $26 = (_sqlite3Strlen30($25)|0);
 $27 = (($24) + ($26)|0);
 HEAP32[$z>>2] = $27;
 HEAP32[$i>>2] = 0;
 while(1) {
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 20|0);
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($33<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 1)|0;
  $41 = $nDistinct;
  $42 = $41;
  HEAP32[$42>>2] = $40;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = 0;
  $45 = HEAP32[$p>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $nDistinct;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (_i64Add(($46|0),0,($49|0),($52|0))|0);
  $54 = tempRet0;
  $55 = (_i64Subtract(($53|0),($54|0),1,0)|0);
  $56 = tempRet0;
  $57 = $nDistinct;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (___udivdi3(($55|0),($56|0),($59|0),($62|0))|0);
  $64 = tempRet0;
  $65 = $iVal;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
  $69 = HEAP32[$z>>2]|0;
  $70 = $iVal;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $vararg_buffer1;
  $77 = $76;
  HEAP32[$77>>2] = $72;
  $78 = (($76) + 4)|0;
  $79 = $78;
  HEAP32[$79>>2] = $75;
  (_sqlite3_snprintf(24,$69,36817,$vararg_buffer1)|0);
  $80 = HEAP32[$z>>2]|0;
  $81 = (_sqlite3Strlen30($80)|0);
  $82 = HEAP32[$z>>2]|0;
  $83 = (($82) + ($81)|0);
  HEAP32[$z>>2] = $83;
  $84 = HEAP32[$i>>2]|0;
  $85 = (($84) + 1)|0;
  HEAP32[$i>>2] = $85;
 }
 $86 = HEAP32[$0>>2]|0;
 $87 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($86,$87,-1,142);
 STACKTOP = sp;return;
}
function _isSystemTable($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3Strlen30($3)|0);
 $5 = ($4|0)>(6);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_sqlite3_strnicmp($6,28844,7)|0);
  $8 = (0)==($7|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer>>2] = $10;
   _sqlite3ErrorMsg($9,37512,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $11 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($11|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _whereForeignKeys($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $zWhere = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $zWhere = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$zWhere>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3FkReferences($2)|0);
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$zWhere>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (_whereOrName($7,$8,$11)|0);
  HEAP32[$zWhere>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$p>>2] = $15;
 }
 $16 = HEAP32[$zWhere>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _whereTempTriggers($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pTempSchema = 0, $pTrig = 0, $vararg_buffer = 0, $zNew = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTrig = sp + 20|0;
 $zWhere = sp + 16|0;
 $pTempSchema = sp + 12|0;
 $db = sp + 8|0;
 $zNew = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$zWhere>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTempSchema>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 64|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pTempSchema>>2]|0;
 $13 = ($11|0)!=($12|0);
 L1: do {
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$db>>2] = $15;
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3TriggerList($16,$17)|0);
   HEAP32[$pTrig>>2] = $18;
   while(1) {
    $19 = HEAP32[$pTrig>>2]|0;
    $20 = ($19|0)!=(0|0);
    if (!($20)) {
     break L1;
    }
    $21 = HEAP32[$pTrig>>2]|0;
    $22 = ((($21)) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$pTempSchema>>2]|0;
    $25 = ($23|0)==($24|0);
    if ($25) {
     $26 = HEAP32[$db>>2]|0;
     $27 = HEAP32[$zWhere>>2]|0;
     $28 = HEAP32[$pTrig>>2]|0;
     $29 = HEAP32[$28>>2]|0;
     $30 = (_whereOrName($26,$27,$29)|0);
     HEAP32[$zWhere>>2] = $30;
    }
    $31 = HEAP32[$pTrig>>2]|0;
    $32 = ((($31)) + 32|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$pTrig>>2] = $33;
   }
  }
 } while(0);
 $34 = HEAP32[$zWhere>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  $44 = HEAP32[$zWhere>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$zWhere>>2]|0;
 HEAP32[$vararg_buffer>>2] = $38;
 $39 = (_sqlite3MPrintf($37,37562,$vararg_buffer)|0);
 HEAP32[$zNew>>2] = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$zWhere>>2]|0;
 _sqlite3DbFree($41,$42);
 $43 = HEAP32[$zNew>>2]|0;
 HEAP32[$zWhere>>2] = $43;
 $44 = HEAP32[$zWhere>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _reloadTableSchema($pParse,$pTab,$zName) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $iTrigDb = 0, $pTrig = 0, $v = 0, $vararg_buffer = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $v = sp + 20|0;
 $zWhere = sp + 16|0;
 $iDb = sp + 12|0;
 $pTrig = sp + 8|0;
 $iTrigDb = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 64|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3SchemaToIndex($8,$11)|0);
 HEAP32[$iDb>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3TriggerList($13,$14)|0);
 HEAP32[$pTrig>>2] = $15;
 while(1) {
  $16 = HEAP32[$pTrig>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pTrig>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (_sqlite3SchemaToIndex($19,$22)|0);
  HEAP32[$iTrigDb>>2] = $23;
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$iTrigDb>>2]|0;
  $26 = HEAP32[$pTrig>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3VdbeAddOp4($24,127,$25,0,0,$27,0)|0);
  $28 = HEAP32[$pTrig>>2]|0;
  $29 = ((($28)) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pTrig>>2] = $30;
 }
 $31 = HEAP32[$v>>2]|0;
 $32 = HEAP32[$iDb>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = HEAP32[$33>>2]|0;
 (_sqlite3VdbeAddOp4($31,125,$32,0,0,$34,0)|0);
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $37;
 $38 = (_sqlite3MPrintf($36,37586,$vararg_buffer)|0);
 HEAP32[$zWhere>>2] = $38;
 $39 = HEAP32[$zWhere>>2]|0;
 $40 = ($39|0)!=(0|0);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$v>>2]|0;
 $42 = HEAP32[$iDb>>2]|0;
 $43 = HEAP32[$zWhere>>2]|0;
 _sqlite3VdbeAddParseSchemaOp($41,$42,$43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = (_whereTempTriggers($44,$45)|0);
 HEAP32[$zWhere>>2] = $46;
 $47 = ($46|0)!=(0|0);
 if (!($47)) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$v>>2]|0;
 $49 = HEAP32[$zWhere>>2]|0;
 _sqlite3VdbeAddParseSchemaOp($48,1,$49);
 STACKTOP = sp;return;
}
function _whereOrName($db,$zWhere,$zConstant) {
 $db = $db|0;
 $zWhere = $zWhere|0;
 $zConstant = $zConstant|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $zNew = sp + 16|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zWhere;
 HEAP32[$2>>2] = $zConstant;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $8;
  $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $9;
  $10 = (_sqlite3MPrintf($5,37548,$vararg_buffer1)|0);
  HEAP32[$zNew>>2] = $10;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($11,$12);
  $13 = HEAP32[$zNew>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $6 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $6;
  $7 = (_sqlite3MPrintf($5,37540,$vararg_buffer)|0);
  HEAP32[$zNew>>2] = $7;
  $13 = HEAP32[$zNew>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _addArgumentToVtab($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $z = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 516|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 488|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 516|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$z>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 516|0);
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$db>>2] = $17;
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 488|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$z>>2]|0;
 $24 = HEAP32[$n>>2]|0;
 $25 = ($24|0)<(0);
 $26 = $25 << 31 >> 31;
 $27 = (_sqlite3DbStrNDup($22,$23,$24,$26)|0);
 _addModuleArgument($18,$21,$27);
 STACKTOP = sp;return;
}
function _trimFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$old = 0, $$old1 = 0, $$old2 = 0, $$old3 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0;
 var $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0;
 var $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0;
 var $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0;
 var $167 = 0, $168 = 0, $169 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aLen = 0, $azChar = 0, $flags = 0, $i = 0, $len = 0, $len1 = 0;
 var $nChar = 0, $nIn = 0, $or$cond = 0, $or$cond4 = 0, $z = 0, $zCharSet = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $zIn = sp + 40|0;
 $zCharSet = sp + 36|0;
 $nIn = sp + 32|0;
 $flags = sp + 28|0;
 $i = sp + 24|0;
 $aLen = sp + 20|0;
 $azChar = sp + 16|0;
 $nChar = sp + 12|0;
 $z = sp + 8|0;
 $len = sp + 4|0;
 $len1 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$aLen>>2] = 0;
 HEAP32[$azChar>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 $6 = ($5|0)==(5);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zIn>>2] = $9;
 $10 = HEAP32[$zIn>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_bytes($13)|0);
 HEAP32[$nIn>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = ($15|0)==(1);
 L7: do {
  if ($16) {
   HEAP32[$nChar>>2] = 1;
   HEAP32[$aLen>>2] = 38524;
   HEAP32[$azChar>>2] = 5672;
   HEAP32[$zCharSet>>2] = 0;
  } else {
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3_value_text($19)|0);
   HEAP32[$zCharSet>>2] = $20;
   $21 = ($20|0)==(0|0);
   if ($21) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$zCharSet>>2]|0;
   HEAP32[$z>>2] = $22;
   HEAP32[$nChar>>2] = 0;
   while(1) {
    $23 = HEAP32[$z>>2]|0;
    $24 = HEAP8[$23>>0]|0;
    $25 = ($24<<24>>24)!=(0);
    if (!($25)) {
     break;
    }
    $26 = HEAP32[$z>>2]|0;
    $27 = ((($26)) + 1|0);
    HEAP32[$z>>2] = $27;
    $28 = HEAP8[$26>>0]|0;
    $29 = $28&255;
    $30 = ($29|0)>=(192);
    L16: do {
     if ($30) {
      while(1) {
       $31 = HEAP32[$z>>2]|0;
       $32 = HEAP8[$31>>0]|0;
       $33 = $32&255;
       $34 = $33 & 192;
       $35 = ($34|0)==(128);
       if (!($35)) {
        break L16;
       }
       $36 = HEAP32[$z>>2]|0;
       $37 = ((($36)) + 1|0);
       HEAP32[$z>>2] = $37;
      }
     }
    } while(0);
    $38 = HEAP32[$nChar>>2]|0;
    $39 = (($38) + 1)|0;
    HEAP32[$nChar>>2] = $39;
   }
   $40 = HEAP32[$nChar>>2]|0;
   $41 = ($40|0)>(0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = HEAP32[$nChar>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (___muldi3(($43|0),($45|0),5,0)|0);
    $47 = tempRet0;
    $48 = (_contextMalloc($42,$46,$47)|0);
    HEAP32[$azChar>>2] = $48;
    $49 = HEAP32[$azChar>>2]|0;
    $50 = ($49|0)==(0|0);
    if ($50) {
     STACKTOP = sp;return;
    }
    $51 = HEAP32[$nChar>>2]|0;
    $52 = HEAP32[$azChar>>2]|0;
    $53 = (($52) + ($51<<2)|0);
    HEAP32[$aLen>>2] = $53;
    $54 = HEAP32[$zCharSet>>2]|0;
    HEAP32[$z>>2] = $54;
    HEAP32[$nChar>>2] = 0;
    while(1) {
     $55 = HEAP32[$z>>2]|0;
     $56 = HEAP8[$55>>0]|0;
     $57 = ($56<<24>>24)!=(0);
     if (!($57)) {
      break L7;
     }
     $58 = HEAP32[$z>>2]|0;
     $59 = HEAP32[$nChar>>2]|0;
     $60 = HEAP32[$azChar>>2]|0;
     $61 = (($60) + ($59<<2)|0);
     HEAP32[$61>>2] = $58;
     $62 = HEAP32[$z>>2]|0;
     $63 = ((($62)) + 1|0);
     HEAP32[$z>>2] = $63;
     $64 = HEAP8[$62>>0]|0;
     $65 = $64&255;
     $66 = ($65|0)>=(192);
     L29: do {
      if ($66) {
       while(1) {
        $67 = HEAP32[$z>>2]|0;
        $68 = HEAP8[$67>>0]|0;
        $69 = $68&255;
        $70 = $69 & 192;
        $71 = ($70|0)==(128);
        if (!($71)) {
         break L29;
        }
        $72 = HEAP32[$z>>2]|0;
        $73 = ((($72)) + 1|0);
        HEAP32[$z>>2] = $73;
       }
      }
     } while(0);
     $74 = HEAP32[$z>>2]|0;
     $75 = HEAP32[$nChar>>2]|0;
     $76 = HEAP32[$azChar>>2]|0;
     $77 = (($76) + ($75<<2)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = $74;
     $80 = $78;
     $81 = (($79) - ($80))|0;
     $82 = $81&255;
     $83 = HEAP32[$nChar>>2]|0;
     $84 = HEAP32[$aLen>>2]|0;
     $85 = (($84) + ($83)|0);
     HEAP8[$85>>0] = $82;
     $86 = HEAP32[$nChar>>2]|0;
     $87 = (($86) + 1)|0;
     HEAP32[$nChar>>2] = $87;
    }
   }
  }
 } while(0);
 $88 = HEAP32[$nChar>>2]|0;
 $89 = ($88|0)>(0);
 if ($89) {
  $90 = HEAP32[$0>>2]|0;
  $91 = (_sqlite3_user_data($90)|0);
  $92 = $91;
  HEAP32[$flags>>2] = $92;
  $93 = HEAP32[$flags>>2]|0;
  $94 = $93 & 1;
  $95 = ($94|0)!=(0);
  $96 = HEAP32[$nIn>>2]|0;
  $97 = ($96|0)>(0);
  $or$cond = $95 & $97;
  L37: do {
   if ($or$cond) {
    while(1) {
     HEAP32[$len>>2] = 0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $98 = HEAP32[$i>>2]|0;
      $99 = HEAP32[$nChar>>2]|0;
      $100 = ($98|0)<($99|0);
      if (!($100)) {
       break;
      }
      $101 = HEAP32[$i>>2]|0;
      $102 = HEAP32[$aLen>>2]|0;
      $103 = (($102) + ($101)|0);
      $104 = HEAP8[$103>>0]|0;
      $105 = $104&255;
      HEAP32[$len>>2] = $105;
      $106 = HEAP32[$len>>2]|0;
      $107 = HEAP32[$nIn>>2]|0;
      $108 = ($106|0)<=($107|0);
      if ($108) {
       $109 = HEAP32[$zIn>>2]|0;
       $110 = HEAP32[$i>>2]|0;
       $111 = HEAP32[$azChar>>2]|0;
       $112 = (($111) + ($110<<2)|0);
       $113 = HEAP32[$112>>2]|0;
       $114 = HEAP32[$len>>2]|0;
       $115 = (_memcmp($109,$113,$114)|0);
       $116 = ($115|0)==(0);
       if ($116) {
        break;
       }
      }
      $117 = HEAP32[$i>>2]|0;
      $118 = (($117) + 1)|0;
      HEAP32[$i>>2] = $118;
     }
     $119 = HEAP32[$i>>2]|0;
     $120 = HEAP32[$nChar>>2]|0;
     $121 = ($119|0)>=($120|0);
     if ($121) {
      break L37;
     }
     $122 = HEAP32[$len>>2]|0;
     $123 = HEAP32[$zIn>>2]|0;
     $124 = (($123) + ($122)|0);
     HEAP32[$zIn>>2] = $124;
     $125 = HEAP32[$len>>2]|0;
     $126 = HEAP32[$nIn>>2]|0;
     $127 = (($126) - ($125))|0;
     HEAP32[$nIn>>2] = $127;
     $$old = HEAP32[$nIn>>2]|0;
     $$old1 = ($$old|0)>(0);
     if (!($$old1)) {
      break;
     }
    }
   }
  } while(0);
  $128 = HEAP32[$flags>>2]|0;
  $129 = $128 & 2;
  $130 = ($129|0)!=(0);
  $131 = HEAP32[$nIn>>2]|0;
  $132 = ($131|0)>(0);
  $or$cond4 = $130 & $132;
  L49: do {
   if ($or$cond4) {
    while(1) {
     HEAP32[$len1>>2] = 0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $133 = HEAP32[$i>>2]|0;
      $134 = HEAP32[$nChar>>2]|0;
      $135 = ($133|0)<($134|0);
      if (!($135)) {
       break;
      }
      $136 = HEAP32[$i>>2]|0;
      $137 = HEAP32[$aLen>>2]|0;
      $138 = (($137) + ($136)|0);
      $139 = HEAP8[$138>>0]|0;
      $140 = $139&255;
      HEAP32[$len1>>2] = $140;
      $141 = HEAP32[$len1>>2]|0;
      $142 = HEAP32[$nIn>>2]|0;
      $143 = ($141|0)<=($142|0);
      if ($143) {
       $144 = HEAP32[$nIn>>2]|0;
       $145 = HEAP32[$len1>>2]|0;
       $146 = (($144) - ($145))|0;
       $147 = HEAP32[$zIn>>2]|0;
       $148 = (($147) + ($146)|0);
       $149 = HEAP32[$i>>2]|0;
       $150 = HEAP32[$azChar>>2]|0;
       $151 = (($150) + ($149<<2)|0);
       $152 = HEAP32[$151>>2]|0;
       $153 = HEAP32[$len1>>2]|0;
       $154 = (_memcmp($148,$152,$153)|0);
       $155 = ($154|0)==(0);
       if ($155) {
        break;
       }
      }
      $156 = HEAP32[$i>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$i>>2] = $157;
     }
     $158 = HEAP32[$i>>2]|0;
     $159 = HEAP32[$nChar>>2]|0;
     $160 = ($158|0)>=($159|0);
     if ($160) {
      break L49;
     }
     $161 = HEAP32[$len1>>2]|0;
     $162 = HEAP32[$nIn>>2]|0;
     $163 = (($162) - ($161))|0;
     HEAP32[$nIn>>2] = $163;
     $$old2 = HEAP32[$nIn>>2]|0;
     $$old3 = ($$old2|0)>(0);
     if (!($$old3)) {
      break;
     }
    }
   }
  } while(0);
  $164 = HEAP32[$zCharSet>>2]|0;
  $165 = ($164|0)!=(0|0);
  if ($165) {
   $166 = HEAP32[$azChar>>2]|0;
   _sqlite3_free($166);
  }
 }
 $167 = HEAP32[$0>>2]|0;
 $168 = HEAP32[$zIn>>2]|0;
 $169 = HEAP32[$nIn>>2]|0;
 _sqlite3_result_text($167,$168,$169,(-1));
 STACKTOP = sp;return;
}
function _minmaxFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $iBest = 0, $mask = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $mask = sp + 8|0;
 $iBest = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_user_data($3)|0);
 $5 = ($4|0)==(0|0);
 $6 = $5 ? 0 : -1;
 HEAP32[$mask>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetFuncCollSeq($7)|0);
 HEAP32[$pColl>>2] = $8;
 HEAP32[$iBest>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = ($11|0)==(5);
 if ($12) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3_value_type($19)|0);
  $21 = ($20|0)==(5);
  if ($21) {
   label = 9;
   break;
  }
  $22 = HEAP32[$iBest>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + ($26<<2)|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pColl>>2]|0;
  $31 = (_sqlite3MemCompare($25,$29,$30)|0);
  $32 = HEAP32[$mask>>2]|0;
  $33 = $31 ^ $32;
  $34 = ($33|0)>=(0);
  if ($34) {
   $35 = HEAP32[$i>>2]|0;
   HEAP32[$iBest>>2] = $35;
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$iBest>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 _sqlite3_result_value($38,$42);
 STACKTOP = sp;return;
}
function _minmaxStep($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cmp = 0, $max = 0, $or$cond = 0, $or$cond3 = 0, $pArg = 0, $pBest = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pArg = sp + 16|0;
 $pBest = sp + 12|0;
 $max = sp + 8|0;
 $cmp = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pArg>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3_aggregate_context($5,40)|0);
 HEAP32[$pBest>>2] = $6;
 $7 = HEAP32[$pBest>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = ($11|0)==(5);
 $13 = HEAP32[$pBest>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = ($15<<16>>16)!=(0);
 if ($12) {
  if (!($16)) {
   STACKTOP = sp;return;
  }
  $17 = HEAP32[$0>>2]|0;
  _sqlite3SkipAccumulatorLoad($17);
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 if (!($16)) {
  $39 = (_sqlite3_context_db_handle($18)|0);
  $40 = HEAP32[$pBest>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pBest>>2]|0;
  $43 = HEAP32[$pArg>>2]|0;
  (_sqlite3VdbeMemCopy($42,$43)|0);
  STACKTOP = sp;return;
 }
 $19 = (_sqlite3GetFuncCollSeq($18)|0);
 HEAP32[$pColl>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3_user_data($20)|0);
 $22 = ($21|0)!=(0|0);
 $23 = $22&1;
 HEAP32[$max>>2] = $23;
 $24 = HEAP32[$pBest>>2]|0;
 $25 = HEAP32[$pArg>>2]|0;
 $26 = HEAP32[$pColl>>2]|0;
 $27 = (_sqlite3MemCompare($24,$25,$26)|0);
 HEAP32[$cmp>>2] = $27;
 $28 = HEAP32[$max>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$cmp>>2]|0;
 $31 = ($30|0)<(0);
 $or$cond = $29 & $31;
 if (!($or$cond)) {
  $32 = HEAP32[$max>>2]|0;
  $33 = ($32|0)==(0);
  $34 = HEAP32[$cmp>>2]|0;
  $35 = ($34|0)>(0);
  $or$cond3 = $33 & $35;
  if (!($or$cond3)) {
   $38 = HEAP32[$0>>2]|0;
   _sqlite3SkipAccumulatorLoad($38);
   STACKTOP = sp;return;
  }
 }
 $36 = HEAP32[$pBest>>2]|0;
 $37 = HEAP32[$pArg>>2]|0;
 (_sqlite3VdbeMemCopy($36,$37)|0);
 STACKTOP = sp;return;
}
function _minMaxFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pRes = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$pRes>>2] = $2;
 $3 = HEAP32[$pRes>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$pRes>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = ($7<<16>>16)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$pRes>>2]|0;
  _sqlite3_result_value($9,$10);
 }
 $11 = HEAP32[$pRes>>2]|0;
 _sqlite3VdbeMemRelease($11);
 STACKTOP = sp;return;
}
function _typeofFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 HEAP32[$z>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 1:  {
  HEAP32[$z>>2] = 25482;
  break;
 }
 case 3:  {
  HEAP32[$z>>2] = 38525;
  break;
 }
 case 2:  {
  HEAP32[$z>>2] = 25477;
  break;
 }
 case 4:  {
  HEAP32[$z>>2] = 38530;
  break;
 }
 default: {
  HEAP32[$z>>2] = 25472;
 }
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$z>>2]|0;
 _sqlite3_result_text($6,$7,-1,0);
 STACKTOP = sp;return;
}
function _lengthFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $len = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $len = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 2: case 1: case 4:  {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_bytes($8)|0);
  _sqlite3_result_int($6,$9);
  STACKTOP = sp;return;
  break;
 }
 case 3:  {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_text($11)|0);
  HEAP32[$z>>2] = $12;
  $13 = HEAP32[$z>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   STACKTOP = sp;return;
  }
  HEAP32[$len>>2] = 0;
  L10: while(1) {
   $15 = HEAP32[$z>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$len>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$len>>2] = $19;
   $20 = HEAP32[$z>>2]|0;
   $21 = ((($20)) + 1|0);
   HEAP32[$z>>2] = $21;
   $22 = HEAP8[$20>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)>=(192);
   if (!($24)) {
    continue;
   }
   while(1) {
    $25 = HEAP32[$z>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = $27 & 192;
    $29 = ($28|0)==(128);
    if (!($29)) {
     continue L10;
    }
    $30 = HEAP32[$z>>2]|0;
    $31 = ((($30)) + 1|0);
    HEAP32[$z>>2] = $31;
   }
  }
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$len>>2]|0;
  _sqlite3_result_int($32,$33);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $34 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($34);
  STACKTOP = sp;return;
 }
 }
}
function _instrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $N = 0, $isText = 0, $nHaystack = 0, $nNeedle = 0, $or$cond = 0, $or$cond3 = 0, $typeHaystack = 0, $typeNeedle = 0, $zHaystack = 0, $zNeedle = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $zHaystack = sp + 28|0;
 $zNeedle = sp + 24|0;
 $nHaystack = sp + 20|0;
 $nNeedle = sp + 16|0;
 $typeHaystack = sp + 12|0;
 $typeNeedle = sp + 8|0;
 $N = sp + 4|0;
 $isText = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$N>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 HEAP32[$typeHaystack>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_type($8)|0);
 HEAP32[$typeNeedle>>2] = $9;
 $10 = HEAP32[$typeHaystack>>2]|0;
 $11 = ($10|0)==(5);
 $12 = HEAP32[$typeNeedle>>2]|0;
 $13 = ($12|0)==(5);
 $or$cond = $11 | $13;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_bytes($15)|0);
 HEAP32[$nHaystack>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3_value_bytes($19)|0);
 HEAP32[$nNeedle>>2] = $20;
 $21 = HEAP32[$typeHaystack>>2]|0;
 $22 = ($21|0)==(4);
 $23 = HEAP32[$typeNeedle>>2]|0;
 $24 = ($23|0)==(4);
 $or$cond3 = $22 & $24;
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 if ($or$cond3) {
  $27 = (_sqlite3_value_blob($26)|0);
  HEAP32[$zHaystack>>2] = $27;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3_value_blob($30)|0);
  HEAP32[$zNeedle>>2] = $31;
  HEAP32[$isText>>2] = 0;
 } else {
  $32 = (_sqlite3_value_text($26)|0);
  HEAP32[$zHaystack>>2] = $32;
  $33 = HEAP32[$2>>2]|0;
  $34 = ((($33)) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3_value_text($35)|0);
  HEAP32[$zNeedle>>2] = $36;
  HEAP32[$isText>>2] = 1;
 }
 L7: while(1) {
  $37 = HEAP32[$nNeedle>>2]|0;
  $38 = HEAP32[$nHaystack>>2]|0;
  $39 = ($37|0)<=($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$zHaystack>>2]|0;
  $41 = HEAP32[$zNeedle>>2]|0;
  $42 = HEAP32[$nNeedle>>2]|0;
  $43 = (_memcmp($40,$41,$42)|0);
  $44 = ($43|0)!=(0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$N>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$N>>2] = $46;
  while(1) {
   $47 = HEAP32[$nHaystack>>2]|0;
   $48 = (($47) + -1)|0;
   HEAP32[$nHaystack>>2] = $48;
   $49 = HEAP32[$zHaystack>>2]|0;
   $50 = ((($49)) + 1|0);
   HEAP32[$zHaystack>>2] = $50;
   $51 = HEAP32[$isText>>2]|0;
   $52 = ($51|0)!=(0);
   if (!($52)) {
    continue L7;
   }
   $53 = HEAP32[$zHaystack>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54&255;
   $56 = $55 & 192;
   $57 = ($56|0)==(128);
   if (!($57)) {
    continue L7;
   }
  }
 }
 $58 = HEAP32[$nNeedle>>2]|0;
 $59 = HEAP32[$nHaystack>>2]|0;
 $60 = ($58|0)>($59|0);
 if ($60) {
  HEAP32[$N>>2] = 0;
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$N>>2]|0;
 _sqlite3_result_int($61,$62);
 STACKTOP = sp;return;
}
function _substrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $len = 0, $negP2 = 0;
 var $p0type = 0, $p1 = 0, $p2 = 0, $z = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $z = sp + 32|0;
 $z2 = sp + 28|0;
 $len = sp + 24|0;
 $p0type = sp + 20|0;
 $p1 = sp + 8|0;
 $p2 = sp;
 $negP2 = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$negP2>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3_value_type($5)|0);
 $7 = ($6|0)==(5);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(3);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_sqlite3_value_type($12)|0);
  $14 = ($13|0)==(5);
  if ($14) {
   STACKTOP = sp;return;
  }
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3_value_type($16)|0);
 HEAP32[$p0type>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3_value_int($20)|0);
 $22 = ($21|0)<(0);
 $23 = $22 << 31 >> 31;
 $24 = $p1;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $23;
 $28 = HEAP32[$p0type>>2]|0;
 $29 = ($28|0)==(4);
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 L8: do {
  if ($29) {
   $32 = (_sqlite3_value_bytes($31)|0);
   HEAP32[$len>>2] = $32;
   $33 = HEAP32[$2>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3_value_blob($34)|0);
   HEAP32[$z>>2] = $35;
   $36 = HEAP32[$z>>2]|0;
   $37 = ($36|0)==(0|0);
   if ($37) {
    STACKTOP = sp;return;
   }
  } else {
   $38 = (_sqlite3_value_text($31)|0);
   HEAP32[$z>>2] = $38;
   $39 = HEAP32[$z>>2]|0;
   $40 = ($39|0)==(0|0);
   if ($40) {
    STACKTOP = sp;return;
   }
   HEAP32[$len>>2] = 0;
   $41 = $p1;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($41) + 4)|0;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)<(0);
   if ($47) {
    $48 = HEAP32[$z>>2]|0;
    HEAP32[$z2>>2] = $48;
    while(1) {
     $49 = HEAP32[$z2>>2]|0;
     $50 = HEAP8[$49>>0]|0;
     $51 = ($50<<24>>24)!=(0);
     if (!($51)) {
      break L8;
     }
     $52 = HEAP32[$z2>>2]|0;
     $53 = ((($52)) + 1|0);
     HEAP32[$z2>>2] = $53;
     $54 = HEAP8[$52>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)>=(192);
     L19: do {
      if ($56) {
       while(1) {
        $57 = HEAP32[$z2>>2]|0;
        $58 = HEAP8[$57>>0]|0;
        $59 = $58&255;
        $60 = $59 & 192;
        $61 = ($60|0)==(128);
        if (!($61)) {
         break L19;
        }
        $62 = HEAP32[$z2>>2]|0;
        $63 = ((($62)) + 1|0);
        HEAP32[$z2>>2] = $63;
       }
      }
     } while(0);
     $64 = HEAP32[$len>>2]|0;
     $65 = (($64) + 1)|0;
     HEAP32[$len>>2] = $65;
    }
   }
  }
 } while(0);
 $66 = HEAP32[$1>>2]|0;
 $67 = ($66|0)==(3);
 if ($67) {
  $68 = HEAP32[$2>>2]|0;
  $69 = ((($68)) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (_sqlite3_value_int($70)|0);
  $72 = ($71|0)<(0);
  $73 = $72 << 31 >> 31;
  $74 = $p2;
  $75 = $74;
  HEAP32[$75>>2] = $71;
  $76 = (($74) + 4)|0;
  $77 = $76;
  HEAP32[$77>>2] = $73;
  $78 = $p2;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = ($83|0)<(0);
  if ($84) {
   $85 = $p2;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = (($85) + 4)|0;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (_i64Subtract(0,0,($87|0),($90|0))|0);
   $92 = tempRet0;
   $93 = $p2;
   $94 = $93;
   HEAP32[$94>>2] = $91;
   $95 = (($93) + 4)|0;
   $96 = $95;
   HEAP32[$96>>2] = $92;
   HEAP32[$negP2>>2] = 1;
  }
 } else {
  $97 = HEAP32[$0>>2]|0;
  $98 = (_sqlite3_context_db_handle($97)|0);
  $99 = ((($98)) + 88|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($100|0)<(0);
  $102 = $101 << 31 >> 31;
  $103 = $p2;
  $104 = $103;
  HEAP32[$104>>2] = $100;
  $105 = (($103) + 4)|0;
  $106 = $105;
  HEAP32[$106>>2] = $102;
 }
 $107 = $p1;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($107) + 4)|0;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)<(0);
 do {
  if ($113) {
   $114 = HEAP32[$len>>2]|0;
   $115 = ($114|0)<(0);
   $116 = $115 << 31 >> 31;
   $117 = $p1;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $120 = (($117) + 4)|0;
   $121 = $120;
   $122 = HEAP32[$121>>2]|0;
   $123 = (_i64Add(($119|0),($122|0),($114|0),($116|0))|0);
   $124 = tempRet0;
   $125 = $p1;
   $126 = $125;
   HEAP32[$126>>2] = $123;
   $127 = (($125) + 4)|0;
   $128 = $127;
   HEAP32[$128>>2] = $124;
   $129 = $p1;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) + 4)|0;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = ($134|0)<(0);
   if ($135) {
    $136 = $p1;
    $137 = $136;
    $138 = HEAP32[$137>>2]|0;
    $139 = (($136) + 4)|0;
    $140 = $139;
    $141 = HEAP32[$140>>2]|0;
    $142 = $p2;
    $143 = $142;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($142) + 4)|0;
    $146 = $145;
    $147 = HEAP32[$146>>2]|0;
    $148 = (_i64Add(($144|0),($147|0),($138|0),($141|0))|0);
    $149 = tempRet0;
    $150 = $p2;
    $151 = $150;
    HEAP32[$151>>2] = $148;
    $152 = (($150) + 4)|0;
    $153 = $152;
    HEAP32[$153>>2] = $149;
    $154 = $p2;
    $155 = $154;
    $156 = HEAP32[$155>>2]|0;
    $157 = (($154) + 4)|0;
    $158 = $157;
    $159 = HEAP32[$158>>2]|0;
    $160 = ($159|0)<(0);
    $161 = $160 ? 0 : $148;
    $162 = $160 ? 0 : $149;
    $163 = $p2;
    $164 = $163;
    HEAP32[$164>>2] = $161;
    $165 = (($163) + 4)|0;
    $166 = $165;
    HEAP32[$166>>2] = $162;
    $167 = $p1;
    $168 = $167;
    HEAP32[$168>>2] = 0;
    $169 = (($167) + 4)|0;
    $170 = $169;
    HEAP32[$170>>2] = 0;
   }
  } else {
   $171 = $p1;
   $172 = $171;
   $173 = HEAP32[$172>>2]|0;
   $174 = (($171) + 4)|0;
   $175 = $174;
   $176 = HEAP32[$175>>2]|0;
   $177 = ($176|0)>(0);
   $178 = ($173>>>0)>(0);
   $179 = ($176|0)==(0);
   $180 = $179 & $178;
   $181 = $177 | $180;
   if ($181) {
    $182 = $p1;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = (($182) + 4)|0;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = (_i64Add(($184|0),($187|0),-1,-1)|0);
    $189 = tempRet0;
    $190 = $p1;
    $191 = $190;
    HEAP32[$191>>2] = $188;
    $192 = (($190) + 4)|0;
    $193 = $192;
    HEAP32[$193>>2] = $189;
    break;
   }
   $194 = $p2;
   $195 = $194;
   $196 = HEAP32[$195>>2]|0;
   $197 = (($194) + 4)|0;
   $198 = $197;
   $199 = HEAP32[$198>>2]|0;
   $200 = ($199|0)>(0);
   $201 = ($196>>>0)>(0);
   $202 = ($199|0)==(0);
   $203 = $202 & $201;
   $204 = $200 | $203;
   if ($204) {
    $205 = $p2;
    $206 = $205;
    $207 = HEAP32[$206>>2]|0;
    $208 = (($205) + 4)|0;
    $209 = $208;
    $210 = HEAP32[$209>>2]|0;
    $211 = (_i64Add(($207|0),($210|0),-1,-1)|0);
    $212 = tempRet0;
    $213 = $p2;
    $214 = $213;
    HEAP32[$214>>2] = $211;
    $215 = (($213) + 4)|0;
    $216 = $215;
    HEAP32[$216>>2] = $212;
   }
  }
 } while(0);
 $217 = HEAP32[$negP2>>2]|0;
 $218 = ($217|0)!=(0);
 if ($218) {
  $219 = $p2;
  $220 = $219;
  $221 = HEAP32[$220>>2]|0;
  $222 = (($219) + 4)|0;
  $223 = $222;
  $224 = HEAP32[$223>>2]|0;
  $225 = $p1;
  $226 = $225;
  $227 = HEAP32[$226>>2]|0;
  $228 = (($225) + 4)|0;
  $229 = $228;
  $230 = HEAP32[$229>>2]|0;
  $231 = (_i64Subtract(($227|0),($230|0),($221|0),($224|0))|0);
  $232 = tempRet0;
  $233 = $p1;
  $234 = $233;
  HEAP32[$234>>2] = $231;
  $235 = (($233) + 4)|0;
  $236 = $235;
  HEAP32[$236>>2] = $232;
  $237 = $p1;
  $238 = $237;
  $239 = HEAP32[$238>>2]|0;
  $240 = (($237) + 4)|0;
  $241 = $240;
  $242 = HEAP32[$241>>2]|0;
  $243 = ($242|0)<(0);
  if ($243) {
   $244 = $p1;
   $245 = $244;
   $246 = HEAP32[$245>>2]|0;
   $247 = (($244) + 4)|0;
   $248 = $247;
   $249 = HEAP32[$248>>2]|0;
   $250 = $p2;
   $251 = $250;
   $252 = HEAP32[$251>>2]|0;
   $253 = (($250) + 4)|0;
   $254 = $253;
   $255 = HEAP32[$254>>2]|0;
   $256 = (_i64Add(($252|0),($255|0),($246|0),($249|0))|0);
   $257 = tempRet0;
   $258 = $p2;
   $259 = $258;
   HEAP32[$259>>2] = $256;
   $260 = (($258) + 4)|0;
   $261 = $260;
   HEAP32[$261>>2] = $257;
   $262 = $p1;
   $263 = $262;
   HEAP32[$263>>2] = 0;
   $264 = (($262) + 4)|0;
   $265 = $264;
   HEAP32[$265>>2] = 0;
  }
 }
 $266 = HEAP32[$p0type>>2]|0;
 $267 = ($266|0)!=(4);
 if (!($267)) {
  $353 = $p1;
  $354 = $353;
  $355 = HEAP32[$354>>2]|0;
  $356 = (($353) + 4)|0;
  $357 = $356;
  $358 = HEAP32[$357>>2]|0;
  $359 = $p2;
  $360 = $359;
  $361 = HEAP32[$360>>2]|0;
  $362 = (($359) + 4)|0;
  $363 = $362;
  $364 = HEAP32[$363>>2]|0;
  $365 = (_i64Add(($355|0),($358|0),($361|0),($364|0))|0);
  $366 = tempRet0;
  $367 = HEAP32[$len>>2]|0;
  $368 = ($367|0)<(0);
  $369 = $368 << 31 >> 31;
  $370 = ($366|0)>($369|0);
  $371 = ($365>>>0)>($367>>>0);
  $372 = ($366|0)==($369|0);
  $373 = $372 & $371;
  $374 = $370 | $373;
  if ($374) {
   $375 = HEAP32[$len>>2]|0;
   $376 = ($375|0)<(0);
   $377 = $376 << 31 >> 31;
   $378 = $p1;
   $379 = $378;
   $380 = HEAP32[$379>>2]|0;
   $381 = (($378) + 4)|0;
   $382 = $381;
   $383 = HEAP32[$382>>2]|0;
   $384 = (_i64Subtract(($375|0),($377|0),($380|0),($383|0))|0);
   $385 = tempRet0;
   $386 = $p2;
   $387 = $386;
   HEAP32[$387>>2] = $384;
   $388 = (($386) + 4)|0;
   $389 = $388;
   HEAP32[$389>>2] = $385;
   $390 = $p2;
   $391 = $390;
   $392 = HEAP32[$391>>2]|0;
   $393 = (($390) + 4)|0;
   $394 = $393;
   $395 = HEAP32[$394>>2]|0;
   $396 = ($395|0)<(0);
   $397 = $396 ? 0 : $384;
   $398 = $396 ? 0 : $385;
   $399 = $p2;
   $400 = $399;
   HEAP32[$400>>2] = $397;
   $401 = (($399) + 4)|0;
   $402 = $401;
   HEAP32[$402>>2] = $398;
  }
  $403 = HEAP32[$0>>2]|0;
  $404 = $p1;
  $405 = $404;
  $406 = HEAP32[$405>>2]|0;
  $407 = (($404) + 4)|0;
  $408 = $407;
  $409 = HEAP32[$408>>2]|0;
  $410 = HEAP32[$z>>2]|0;
  $411 = (($410) + ($406)|0);
  $412 = $p2;
  $413 = $412;
  $414 = HEAP32[$413>>2]|0;
  $415 = (($412) + 4)|0;
  $416 = $415;
  $417 = HEAP32[$416>>2]|0;
  _sqlite3_result_blob64($403,$411,$414,$417,(-1));
  STACKTOP = sp;return;
 }
 while(1) {
  $268 = HEAP32[$z>>2]|0;
  $269 = HEAP8[$268>>0]|0;
  $270 = $269&255;
  $271 = ($270|0)!=(0);
  $272 = $p1;
  $273 = $272;
  $274 = HEAP32[$273>>2]|0;
  $275 = (($272) + 4)|0;
  $276 = $275;
  $277 = HEAP32[$276>>2]|0;
  $278 = ($274|0)!=(0);
  $279 = ($277|0)!=(0);
  $280 = $278 | $279;
  $281 = $271 ? $280 : 0;
  $282 = HEAP32[$z>>2]|0;
  if (!($281)) {
   break;
  }
  $283 = ((($282)) + 1|0);
  HEAP32[$z>>2] = $283;
  $284 = HEAP8[$282>>0]|0;
  $285 = $284&255;
  $286 = ($285|0)>=(192);
  L52: do {
   if ($286) {
    while(1) {
     $287 = HEAP32[$z>>2]|0;
     $288 = HEAP8[$287>>0]|0;
     $289 = $288&255;
     $290 = $289 & 192;
     $291 = ($290|0)==(128);
     if (!($291)) {
      break L52;
     }
     $292 = HEAP32[$z>>2]|0;
     $293 = ((($292)) + 1|0);
     HEAP32[$z>>2] = $293;
    }
   }
  } while(0);
  $294 = $p1;
  $295 = $294;
  $296 = HEAP32[$295>>2]|0;
  $297 = (($294) + 4)|0;
  $298 = $297;
  $299 = HEAP32[$298>>2]|0;
  $300 = (_i64Add(($296|0),($299|0),-1,-1)|0);
  $301 = tempRet0;
  $302 = $p1;
  $303 = $302;
  HEAP32[$303>>2] = $300;
  $304 = (($302) + 4)|0;
  $305 = $304;
  HEAP32[$305>>2] = $301;
 }
 HEAP32[$z2>>2] = $282;
 while(1) {
  $306 = HEAP32[$z2>>2]|0;
  $307 = HEAP8[$306>>0]|0;
  $308 = $307&255;
  $309 = ($308|0)!=(0);
  $310 = $p2;
  $311 = $310;
  $312 = HEAP32[$311>>2]|0;
  $313 = (($310) + 4)|0;
  $314 = $313;
  $315 = HEAP32[$314>>2]|0;
  $316 = ($312|0)!=(0);
  $317 = ($315|0)!=(0);
  $318 = $316 | $317;
  $319 = $309 ? $318 : 0;
  if (!($319)) {
   break;
  }
  $320 = HEAP32[$z2>>2]|0;
  $321 = ((($320)) + 1|0);
  HEAP32[$z2>>2] = $321;
  $322 = HEAP8[$320>>0]|0;
  $323 = $322&255;
  $324 = ($323|0)>=(192);
  L61: do {
   if ($324) {
    while(1) {
     $325 = HEAP32[$z2>>2]|0;
     $326 = HEAP8[$325>>0]|0;
     $327 = $326&255;
     $328 = $327 & 192;
     $329 = ($328|0)==(128);
     if (!($329)) {
      break L61;
     }
     $330 = HEAP32[$z2>>2]|0;
     $331 = ((($330)) + 1|0);
     HEAP32[$z2>>2] = $331;
    }
   }
  } while(0);
  $332 = $p2;
  $333 = $332;
  $334 = HEAP32[$333>>2]|0;
  $335 = (($332) + 4)|0;
  $336 = $335;
  $337 = HEAP32[$336>>2]|0;
  $338 = (_i64Add(($334|0),($337|0),-1,-1)|0);
  $339 = tempRet0;
  $340 = $p2;
  $341 = $340;
  HEAP32[$341>>2] = $338;
  $342 = (($340) + 4)|0;
  $343 = $342;
  HEAP32[$343>>2] = $339;
 }
 $344 = HEAP32[$0>>2]|0;
 $345 = HEAP32[$z>>2]|0;
 $346 = HEAP32[$z2>>2]|0;
 $347 = HEAP32[$z>>2]|0;
 $348 = $346;
 $349 = $347;
 $350 = (($348) - ($349))|0;
 $351 = ($350|0)<(0);
 $352 = $351 << 31 >> 31;
 _sqlite3_result_text64($344,$345,$350,$352,(-1),1);
 STACKTOP = sp;return;
}
function _printfFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $str = 0, $vararg_buffer = 0, $x = 0, $zFormat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $x = sp + 44|0;
 $str = sp + 16|0;
 $zFormat = sp + 12|0;
 $n = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)>=(1);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zFormat>>2] = $9;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) - 1)|0;
 HEAP32[$x>>2] = $12;
 $13 = ((($x)) + 4|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = ((($x)) + 8|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 88|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3StrAccumInit($str,$17,0,0,$20);
 $21 = ((($str)) + 25|0);
 HEAP8[$21>>0] = 2;
 $22 = HEAP32[$zFormat>>2]|0;
 HEAP32[$vararg_buffer>>2] = $x;
 _sqlite3XPrintf($str,$22,$vararg_buffer);
 $23 = ((($str)) + 12|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$n>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = (_sqlite3StrAccumFinish($str)|0);
 $27 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($25,$26,$27,169);
 STACKTOP = sp;return;
}
function _unicodeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$z>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$z>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (_sqlite3Utf8Read($z)|0);
 _sqlite3_result_int($12,$13);
 STACKTOP = sp;return;
}
function _charFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $or$cond = 0, $x = 0, $z = 0, $zOut = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $z = sp + 20|0;
 $zOut = sp + 16|0;
 $i = sp + 12|0;
 $x = sp;
 $c = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3<<2;
 $5 = (($4) + 1)|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3_malloc64($5,$7)|0);
 HEAP32[$z>>2] = $8;
 HEAP32[$zOut>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($11);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3_value_int64($18)|0);
  $20 = tempRet0;
  $21 = $x;
  $22 = $21;
  HEAP32[$22>>2] = $19;
  $23 = (($21) + 4)|0;
  $24 = $23;
  HEAP32[$24>>2] = $20;
  $25 = $x;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)<(0);
  $32 = $x;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)>(0);
  $39 = ($34>>>0)>(1114111);
  $40 = ($37|0)==(0);
  $41 = $40 & $39;
  $42 = $38 | $41;
  $or$cond = $31 | $42;
  $43 = $or$cond ? 65533 : $19;
  $44 = $or$cond ? 0 : $20;
  $45 = $x;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  $49 = $x;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = $51 & 2097151;
  HEAP32[$c>>2] = $55;
  $56 = HEAP32[$c>>2]|0;
  $57 = ($56>>>0)<(128);
  $58 = HEAP32[$c>>2]|0;
  do {
   if ($57) {
    $59 = $58 & 255;
    $60 = $59&255;
    $61 = HEAP32[$zOut>>2]|0;
    $62 = ((($61)) + 1|0);
    HEAP32[$zOut>>2] = $62;
    HEAP8[$61>>0] = $60;
   } else {
    $63 = ($58>>>0)<(2048);
    $64 = HEAP32[$c>>2]|0;
    if ($63) {
     $65 = $64 >>> 6;
     $66 = $65 & 31;
     $67 = $66&255;
     $68 = $67&255;
     $69 = (192 + ($68))|0;
     $70 = $69&255;
     $71 = HEAP32[$zOut>>2]|0;
     $72 = ((($71)) + 1|0);
     HEAP32[$zOut>>2] = $72;
     HEAP8[$71>>0] = $70;
     $73 = HEAP32[$c>>2]|0;
     $74 = $73 & 63;
     $75 = $74&255;
     $76 = $75&255;
     $77 = (128 + ($76))|0;
     $78 = $77&255;
     $79 = HEAP32[$zOut>>2]|0;
     $80 = ((($79)) + 1|0);
     HEAP32[$zOut>>2] = $80;
     HEAP8[$79>>0] = $78;
     break;
    }
    $81 = ($64>>>0)<(65536);
    $82 = HEAP32[$c>>2]|0;
    if ($81) {
     $83 = $82 >>> 12;
     $84 = $83 & 15;
     $85 = $84&255;
     $86 = $85&255;
     $87 = (224 + ($86))|0;
     $88 = $87&255;
     $89 = HEAP32[$zOut>>2]|0;
     $90 = ((($89)) + 1|0);
     HEAP32[$zOut>>2] = $90;
     HEAP8[$89>>0] = $88;
     $91 = HEAP32[$c>>2]|0;
     $92 = $91 >>> 6;
     $93 = $92 & 63;
     $94 = $93&255;
     $95 = $94&255;
     $96 = (128 + ($95))|0;
     $97 = $96&255;
     $98 = HEAP32[$zOut>>2]|0;
     $99 = ((($98)) + 1|0);
     HEAP32[$zOut>>2] = $99;
     HEAP8[$98>>0] = $97;
     $100 = HEAP32[$c>>2]|0;
     $101 = $100 & 63;
     $102 = $101&255;
     $103 = $102&255;
     $104 = (128 + ($103))|0;
     $105 = $104&255;
     $106 = HEAP32[$zOut>>2]|0;
     $107 = ((($106)) + 1|0);
     HEAP32[$zOut>>2] = $107;
     HEAP8[$106>>0] = $105;
     break;
    } else {
     $108 = $82 >>> 18;
     $109 = $108 & 7;
     $110 = $109&255;
     $111 = $110&255;
     $112 = (240 + ($111))|0;
     $113 = $112&255;
     $114 = HEAP32[$zOut>>2]|0;
     $115 = ((($114)) + 1|0);
     HEAP32[$zOut>>2] = $115;
     HEAP8[$114>>0] = $113;
     $116 = HEAP32[$c>>2]|0;
     $117 = $116 >>> 12;
     $118 = $117 & 63;
     $119 = $118&255;
     $120 = $119&255;
     $121 = (128 + ($120))|0;
     $122 = $121&255;
     $123 = HEAP32[$zOut>>2]|0;
     $124 = ((($123)) + 1|0);
     HEAP32[$zOut>>2] = $124;
     HEAP8[$123>>0] = $122;
     $125 = HEAP32[$c>>2]|0;
     $126 = $125 >>> 6;
     $127 = $126 & 63;
     $128 = $127&255;
     $129 = $128&255;
     $130 = (128 + ($129))|0;
     $131 = $130&255;
     $132 = HEAP32[$zOut>>2]|0;
     $133 = ((($132)) + 1|0);
     HEAP32[$zOut>>2] = $133;
     HEAP8[$132>>0] = $131;
     $134 = HEAP32[$c>>2]|0;
     $135 = $134 & 63;
     $136 = $135&255;
     $137 = $136&255;
     $138 = (128 + ($137))|0;
     $139 = $138&255;
     $140 = HEAP32[$zOut>>2]|0;
     $141 = ((($140)) + 1|0);
     HEAP32[$zOut>>2] = $141;
     HEAP8[$140>>0] = $139;
     break;
    }
   }
  } while(0);
  $142 = HEAP32[$i>>2]|0;
  $143 = (($142) + 1)|0;
  HEAP32[$i>>2] = $143;
 }
 $144 = HEAP32[$0>>2]|0;
 $145 = HEAP32[$z>>2]|0;
 $146 = HEAP32[$zOut>>2]|0;
 $147 = HEAP32[$z>>2]|0;
 $148 = $146;
 $149 = $147;
 $150 = (($148) - ($149))|0;
 $151 = ($150|0)<(0);
 $152 = $151 << 31 >> 31;
 _sqlite3_result_text64($144,$145,$150,$152,142,1);
 STACKTOP = sp;return;
}
function _absFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iVal = 0, $rVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iVal = sp + 8|0;
 $rVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 1:  {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3_value_int64($7)|0);
  $9 = tempRet0;
  $10 = $iVal;
  $11 = $10;
  HEAP32[$11>>2] = $8;
  $12 = (($10) + 4)|0;
  $13 = $12;
  HEAP32[$13>>2] = $9;
  $14 = $iVal;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)<(0);
  do {
   if ($20) {
    $21 = $iVal;
    $22 = $21;
    $23 = HEAP32[$22>>2]|0;
    $24 = (($21) + 4)|0;
    $25 = $24;
    $26 = HEAP32[$25>>2]|0;
    $27 = ($23|0)==(0);
    $28 = ($26|0)==(-2147483648);
    $29 = $27 & $28;
    if (!($29)) {
     $31 = $iVal;
     $32 = $31;
     $33 = HEAP32[$32>>2]|0;
     $34 = (($31) + 4)|0;
     $35 = $34;
     $36 = HEAP32[$35>>2]|0;
     $37 = (_i64Subtract(0,0,($33|0),($36|0))|0);
     $38 = tempRet0;
     $39 = $iVal;
     $40 = $39;
     HEAP32[$40>>2] = $37;
     $41 = (($39) + 4)|0;
     $42 = $41;
     HEAP32[$42>>2] = $38;
     break;
    }
    $30 = HEAP32[$0>>2]|0;
    _sqlite3_result_error($30,38535,-1);
    STACKTOP = sp;return;
   }
  } while(0);
  $43 = HEAP32[$0>>2]|0;
  $44 = $iVal;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  _sqlite3_result_int64($43,$46,$49);
  STACKTOP = sp;return;
  break;
 }
 case 5:  {
  $50 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($50);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $51 = HEAP32[$2>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $53 = (+_sqlite3_value_double($52));
  HEAPF64[$rVal>>3] = $53;
  $54 = +HEAPF64[$rVal>>3];
  $55 = $54 < 0.0;
  if ($55) {
   $56 = +HEAPF64[$rVal>>3];
   $57 = -$56;
   HEAPF64[$rVal>>3] = $57;
  }
  $58 = HEAP32[$0>>2]|0;
  $59 = +HEAPF64[$rVal>>3];
  _sqlite3_result_double($58,$59);
  STACKTOP = sp;return;
 }
 }
}
function _roundFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $$$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0;
 var $25 = 0, $26 = 0, $27 = 0.0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0;
 var $43 = 0.0, $44 = 0.0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0.0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $r = 0, $vararg_buffer = 0, $vararg_ptr6 = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $n = sp + 28|0;
 $r = sp;
 $zBuf = sp + 24|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$n>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 do {
  if ($4) {
   $5 = HEAP32[$2>>2]|0;
   $6 = ((($5)) + 4|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = (_sqlite3_value_type($7)|0);
   $9 = (5)==($8|0);
   if ($9) {
    STACKTOP = sp;return;
   } else {
    $10 = HEAP32[$2>>2]|0;
    $11 = ((($10)) + 4|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = (_sqlite3_value_int($12)|0);
    HEAP32[$n>>2] = $13;
    $14 = HEAP32[$n>>2]|0;
    $15 = ($14|0)>(30);
    $$ = $15 ? 30 : $13;
    HEAP32[$n>>2] = $$;
    $16 = HEAP32[$n>>2]|0;
    $17 = ($16|0)<(0);
    $$$ = $17 ? 0 : $$;
    HEAP32[$n>>2] = $$$;
    break;
   }
  }
 } while(0);
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3_value_type($19)|0);
 $21 = ($20|0)==(5);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (+_sqlite3_value_double($23));
 HEAPF64[$r>>3] = $24;
 $25 = HEAP32[$n>>2]|0;
 $26 = ($25|0)==(0);
 $27 = +HEAPF64[$r>>3];
 $28 = $27 >= 0.0;
 $or$cond = $26 & $28;
 $29 = +HEAPF64[$r>>3];
 $30 = $29 < 9.2233720368547758E+18;
 $or$cond3 = $or$cond & $30;
 do {
  if ($or$cond3) {
   $31 = +HEAPF64[$r>>3];
   $32 = $31 + 0.5;
   $33 = (~~$32)>>>0;
   $34 = +Math_abs($32) >= 1.0 ? $32 > 0.0 ? (~~+Math_min(+Math_floor($32 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($32 - +(~~$32 >>> 0)) / 4294967296.0) >>> 0 : 0;
   $35 = (+($33>>>0)) + (4294967296.0*(+($34|0)));
   HEAPF64[$r>>3] = $35;
  } else {
   $36 = HEAP32[$n>>2]|0;
   $37 = ($36|0)==(0);
   $38 = +HEAPF64[$r>>3];
   $39 = $38 < 0.0;
   $or$cond5 = $37 & $39;
   if ($or$cond5) {
    $40 = +HEAPF64[$r>>3];
    $41 = -$40;
    $42 = $41 < 9.2233720368547758E+18;
    if ($42) {
     $43 = +HEAPF64[$r>>3];
     $44 = -$43;
     $45 = $44 + 0.5;
     $46 = (~~$45)>>>0;
     $47 = +Math_abs($45) >= 1.0 ? $45 > 0.0 ? (~~+Math_min(+Math_floor($45 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($45 - +(~~$45 >>> 0)) / 4294967296.0) >>> 0 : 0;
     $48 = (+($46>>>0)) + (4294967296.0*(+($47|0)));
     $49 = -$48;
     HEAPF64[$r>>3] = $49;
     break;
    }
   }
   $50 = HEAP32[$n>>2]|0;
   $51 = +HEAPF64[$r>>3];
   HEAP32[$vararg_buffer>>2] = $50;
   $vararg_ptr6 = ((($vararg_buffer)) + 8|0);
   HEAPF64[$vararg_ptr6>>3] = $51;
   $52 = (_sqlite3_mprintf(38552,$vararg_buffer)|0);
   HEAP32[$zBuf>>2] = $52;
   $53 = HEAP32[$zBuf>>2]|0;
   $54 = ($53|0)==(0|0);
   if (!($54)) {
    $56 = HEAP32[$zBuf>>2]|0;
    $57 = HEAP32[$zBuf>>2]|0;
    $58 = (_sqlite3Strlen30($57)|0);
    (_sqlite3AtoF($56,$r,$58,1)|0);
    $59 = HEAP32[$zBuf>>2]|0;
    _sqlite3_free($59);
    break;
   }
   $55 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_nomem($55);
   STACKTOP = sp;return;
  }
 } while(0);
 $60 = HEAP32[$0>>2]|0;
 $61 = +HEAPF64[$r>>3];
 _sqlite3_result_double($60,$61);
 STACKTOP = sp;return;
}
function _upperFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$z2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$n>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_i64Add(($12|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($11,$15,$16)|0);
 HEAP32[$z1>>2] = $17;
 $18 = HEAP32[$z1>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$z2>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26 << 24 >> 24;
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$z2>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = (17533 + ($32)|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = $35 & 32;
  $37 = $36 ^ -1;
  $38 = $27 & $37;
  $39 = $38&255;
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$z1>>2]|0;
  $42 = (($41) + ($40)|0);
  HEAP8[$42>>0] = $39;
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$z1>>2]|0;
 $47 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($45,$46,$47,142);
 STACKTOP = sp;return;
}
function _lowerFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0;
 var $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$z2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$n>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_i64Add(($12|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($11,$15,$16)|0);
 HEAP32[$z1>>2] = $17;
 $18 = HEAP32[$z1>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$z2>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = (16974 + ($27)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$z1>>2]|0;
  $32 = (($31) + ($30)|0);
  HEAP8[$32>>0] = $29;
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$z1>>2]|0;
 $37 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($35,$36,$37,142);
 STACKTOP = sp;return;
}
function _versionFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_libversion()|0);
 _sqlite3_result_text($3,$4,-1,0);
 STACKTOP = sp;return;
}
function _hexFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $n = 0, $pBlob = 0, $z = 0, $zHex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $n = sp + 12|0;
 $pBlob = sp + 8|0;
 $zHex = sp + 4|0;
 $z = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$pBlob>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$n>>2]|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (___muldi3(($10|0),($12|0),2,0)|0);
 $14 = tempRet0;
 $15 = (_i64Add(($13|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($9,$15,$16)|0);
 HEAP32[$zHex>>2] = $17;
 HEAP32[$z>>2] = $17;
 $18 = HEAP32[$zHex>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$pBlob>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  HEAP8[$c>>0] = $24;
  $25 = HEAP8[$c>>0]|0;
  $26 = $25&255;
  $27 = $26 >> 4;
  $28 = $27 & 15;
  $29 = (38557 + ($28)|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = HEAP32[$z>>2]|0;
  $32 = ((($31)) + 1|0);
  HEAP32[$z>>2] = $32;
  HEAP8[$31>>0] = $30;
  $33 = HEAP8[$c>>0]|0;
  $34 = $33&255;
  $35 = $34 & 15;
  $36 = (38557 + ($35)|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = HEAP32[$z>>2]|0;
  $39 = ((($38)) + 1|0);
  HEAP32[$z>>2] = $39;
  HEAP8[$38>>0] = $37;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
  $42 = HEAP32[$pBlob>>2]|0;
  $43 = ((($42)) + 1|0);
  HEAP32[$pBlob>>2] = $43;
 }
 $44 = HEAP32[$z>>2]|0;
 HEAP8[$44>>0] = 0;
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$zHex>>2]|0;
 $47 = HEAP32[$n>>2]|0;
 $48 = $47<<1;
 _sqlite3_result_text($45,$46,$48,142);
 STACKTOP = sp;return;
}
function _randomFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $r = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 _sqlite3_randomness(8,$r);
 $3 = $r;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = $r;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 2147483647;
  $17 = (_i64Subtract(0,0,($12|0),($16|0))|0);
  $18 = tempRet0;
  $19 = $r;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = $r;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 _sqlite3_result_int64($23,$26,$29);
 STACKTOP = sp;return;
}
function _randomBlob($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $n = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$n>>2]|0;
 $7 = ($6|0)<(1);
 $$ = $7 ? 1 : $5;
 HEAP32[$n>>2] = $$;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_contextMalloc($8,$9,$11)|0);
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$n>>2]|0;
 $16 = HEAP32[$p>>2]|0;
 _sqlite3_randomness($15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$n>>2]|0;
 _sqlite3_result_blob($17,$18,$19,142);
 STACKTOP = sp;return;
}
function _nullifFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetFuncCollSeq($3)|0);
 HEAP32[$pColl>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$pColl>>2]|0;
 $11 = (_sqlite3MemCompare($6,$9,$10)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 _sqlite3_result_value($13,$15);
 STACKTOP = sp;return;
}
function _sourceidFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_sourceid()|0);
 _sqlite3_result_text($3,$4,-1,0);
 STACKTOP = sp;return;
}
function _errlogFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$vararg_buffer>>2] = $9;
 _sqlite3_log($5,17521,$vararg_buffer);
 STACKTOP = sp;return;
}
function _compileoptionusedFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $zOptName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $zOptName = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zOptName>>2] = $5;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$zOptName>>2]|0;
 $9 = (_sqlite3_compileoption_used($8)|0);
 _sqlite3_result_int($7,$9);
 STACKTOP = sp;return;
}
function _compileoptiongetFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$n>>2]|0;
 $8 = (_sqlite3_compileoption_get($7)|0);
 _sqlite3_result_text($6,$8,-1,0);
 STACKTOP = sp;return;
}
function _quoteFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0.0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $i1 = 0, $j = 0, $n = 0, $nBlob = 0, $r1 = 0, $r2 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $z = 0, $zArg = 0, $zBlob = 0, $zBuf = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer1 = sp + 32|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $r1 = sp + 16|0;
 $r2 = sp + 8|0;
 $zBuf = sp + 84|0;
 $zText = sp + 68|0;
 $zBlob = sp + 64|0;
 $nBlob = sp + 60|0;
 $i = sp + 56|0;
 $i1 = sp + 52|0;
 $j = sp + 48|0;
 $n = sp;
 $zArg = sp + 44|0;
 $z = sp + 40|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 2:  {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (+_sqlite3_value_double($7));
  HEAPF64[$r1>>3] = $8;
  $9 = +HEAPF64[$r1>>3];
  HEAPF64[$vararg_buffer>>3] = $9;
  (_sqlite3_snprintf(50,$zBuf,19290,$vararg_buffer)|0);
  (_sqlite3AtoF($zBuf,$r2,20,1)|0);
  $10 = +HEAPF64[$r1>>3];
  $11 = +HEAPF64[$r2>>3];
  $12 = $10 != $11;
  if ($12) {
   $13 = +HEAPF64[$r1>>3];
   HEAPF64[$vararg_buffer1>>3] = $13;
   (_sqlite3_snprintf(50,$zBuf,38573,$vararg_buffer1)|0);
  }
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($14,$zBuf,-1,(-1));
  STACKTOP = sp;return;
  break;
 }
 case 1:  {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_result_value($15,$17);
  STACKTOP = sp;return;
  break;
 }
 case 4:  {
  HEAP32[$zText>>2] = 0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3_value_blob($19)|0);
  HEAP32[$zBlob>>2] = $20;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (_sqlite3_value_bytes($22)|0);
  HEAP32[$nBlob>>2] = $23;
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$nBlob>>2]|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (___muldi3(2,0,($25|0),($27|0))|0);
  $29 = tempRet0;
  $30 = (_i64Add(($28|0),($29|0),4,0)|0);
  $31 = tempRet0;
  $32 = (_contextMalloc($24,$30,$31)|0);
  HEAP32[$zText>>2] = $32;
  $33 = HEAP32[$zText>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   STACKTOP = sp;return;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$nBlob>>2]|0;
   $37 = ($35|0)<($36|0);
   if (!($37)) {
    break;
   }
   $38 = HEAP32[$i>>2]|0;
   $39 = HEAP32[$zBlob>>2]|0;
   $40 = (($39) + ($38)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41 << 24 >> 24;
   $43 = $42 >> 4;
   $44 = $43 & 15;
   $45 = (38557 + ($44)|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = HEAP32[$i>>2]|0;
   $48 = $47<<1;
   $49 = (($48) + 2)|0;
   $50 = HEAP32[$zText>>2]|0;
   $51 = (($50) + ($49)|0);
   HEAP8[$51>>0] = $46;
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$zBlob>>2]|0;
   $54 = (($53) + ($52)|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = $56 & 15;
   $58 = (38557 + ($57)|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = HEAP32[$i>>2]|0;
   $61 = $60<<1;
   $62 = (($61) + 3)|0;
   $63 = HEAP32[$zText>>2]|0;
   $64 = (($63) + ($62)|0);
   HEAP8[$64>>0] = $59;
   $65 = HEAP32[$i>>2]|0;
   $66 = (($65) + 1)|0;
   HEAP32[$i>>2] = $66;
  }
  $67 = HEAP32[$nBlob>>2]|0;
  $68 = $67<<1;
  $69 = (($68) + 2)|0;
  $70 = HEAP32[$zText>>2]|0;
  $71 = (($70) + ($69)|0);
  HEAP8[$71>>0] = 39;
  $72 = HEAP32[$nBlob>>2]|0;
  $73 = $72<<1;
  $74 = (($73) + 3)|0;
  $75 = HEAP32[$zText>>2]|0;
  $76 = (($75) + ($74)|0);
  HEAP8[$76>>0] = 0;
  $77 = HEAP32[$zText>>2]|0;
  HEAP8[$77>>0] = 88;
  $78 = HEAP32[$zText>>2]|0;
  $79 = ((($78)) + 1|0);
  HEAP8[$79>>0] = 39;
  $80 = HEAP32[$0>>2]|0;
  $81 = HEAP32[$zText>>2]|0;
  _sqlite3_result_text($80,$81,-1,(-1));
  $82 = HEAP32[$zText>>2]|0;
  _sqlite3_free($82);
  STACKTOP = sp;return;
  break;
 }
 case 3:  {
  $83 = HEAP32[$2>>2]|0;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3_value_text($84)|0);
  HEAP32[$zArg>>2] = $85;
  $86 = HEAP32[$zArg>>2]|0;
  $87 = ($86|0)==(0|0);
  if ($87) {
   STACKTOP = sp;return;
  }
  HEAP32[$i1>>2] = 0;
  $88 = $n;
  $89 = $88;
  HEAP32[$89>>2] = 0;
  $90 = (($88) + 4)|0;
  $91 = $90;
  HEAP32[$91>>2] = 0;
  while(1) {
   $92 = HEAP32[$i1>>2]|0;
   $93 = HEAP32[$zArg>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = ($95<<24>>24)!=(0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$i1>>2]|0;
   $98 = HEAP32[$zArg>>2]|0;
   $99 = (($98) + ($97)|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = ($101|0)==(39);
   if ($102) {
    $103 = $n;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($103) + 4)|0;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = (_i64Add(($105|0),($108|0),1,0)|0);
    $110 = tempRet0;
    $111 = $n;
    $112 = $111;
    HEAP32[$112>>2] = $109;
    $113 = (($111) + 4)|0;
    $114 = $113;
    HEAP32[$114>>2] = $110;
   }
   $115 = HEAP32[$i1>>2]|0;
   $116 = (($115) + 1)|0;
   HEAP32[$i1>>2] = $116;
  }
  $117 = HEAP32[$0>>2]|0;
  $118 = HEAP32[$i1>>2]|0;
  $119 = ($118|0)<(0);
  $120 = $119 << 31 >> 31;
  $121 = $n;
  $122 = $121;
  $123 = HEAP32[$122>>2]|0;
  $124 = (($121) + 4)|0;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (_i64Add(($118|0),($120|0),($123|0),($126|0))|0);
  $128 = tempRet0;
  $129 = (_i64Add(($127|0),($128|0),3,0)|0);
  $130 = tempRet0;
  $131 = (_contextMalloc($117,$129,$130)|0);
  HEAP32[$z>>2] = $131;
  $132 = HEAP32[$z>>2]|0;
  $133 = ($132|0)!=(0|0);
  if (!($133)) {
   STACKTOP = sp;return;
  }
  $134 = HEAP32[$z>>2]|0;
  HEAP8[$134>>0] = 39;
  HEAP32[$i1>>2] = 0;
  HEAP32[$j>>2] = 1;
  while(1) {
   $135 = HEAP32[$i1>>2]|0;
   $136 = HEAP32[$zArg>>2]|0;
   $137 = (($136) + ($135)|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = ($138<<24>>24)!=(0);
   if (!($139)) {
    break;
   }
   $140 = HEAP32[$i1>>2]|0;
   $141 = HEAP32[$zArg>>2]|0;
   $142 = (($141) + ($140)|0);
   $143 = HEAP8[$142>>0]|0;
   $144 = HEAP32[$j>>2]|0;
   $145 = (($144) + 1)|0;
   HEAP32[$j>>2] = $145;
   $146 = HEAP32[$z>>2]|0;
   $147 = (($146) + ($144)|0);
   HEAP8[$147>>0] = $143;
   $148 = HEAP32[$i1>>2]|0;
   $149 = HEAP32[$zArg>>2]|0;
   $150 = (($149) + ($148)|0);
   $151 = HEAP8[$150>>0]|0;
   $152 = $151&255;
   $153 = ($152|0)==(39);
   if ($153) {
    $154 = HEAP32[$j>>2]|0;
    $155 = (($154) + 1)|0;
    HEAP32[$j>>2] = $155;
    $156 = HEAP32[$z>>2]|0;
    $157 = (($156) + ($154)|0);
    HEAP8[$157>>0] = 39;
   }
   $158 = HEAP32[$i1>>2]|0;
   $159 = (($158) + 1)|0;
   HEAP32[$i1>>2] = $159;
  }
  $160 = HEAP32[$j>>2]|0;
  $161 = (($160) + 1)|0;
  HEAP32[$j>>2] = $161;
  $162 = HEAP32[$z>>2]|0;
  $163 = (($162) + ($160)|0);
  HEAP8[$163>>0] = 39;
  $164 = HEAP32[$j>>2]|0;
  $165 = HEAP32[$z>>2]|0;
  $166 = (($165) + ($164)|0);
  HEAP8[$166>>0] = 0;
  $167 = HEAP32[$0>>2]|0;
  $168 = HEAP32[$z>>2]|0;
  $169 = HEAP32[$j>>2]|0;
  _sqlite3_result_text($167,$168,$169,142);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $170 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($170,18200,4,0);
  STACKTOP = sp;return;
 }
 }
}
function _last_insert_rowid($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_last_insert_rowid($6)|0);
 $8 = tempRet0;
 _sqlite3_result_int64($5,$7,$8);
 STACKTOP = sp;return;
}
function _changes($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_changes($6)|0);
 _sqlite3_result_int($5,$7);
 STACKTOP = sp;return;
}
function _total_changes($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_total_changes($6)|0);
 _sqlite3_result_int($5,$7);
 STACKTOP = sp;return;
}
function _replaceFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $loopLimit = 0, $nOut = 0, $nPattern = 0;
 var $nRep = 0, $nStr = 0, $zOld = 0, $zOut = 0, $zPattern = 0, $zRep = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $zStr = sp + 52|0;
 $zPattern = sp + 48|0;
 $zRep = sp + 44|0;
 $zOut = sp + 40|0;
 $nStr = sp + 36|0;
 $nPattern = sp + 32|0;
 $nRep = sp + 28|0;
 $nOut = sp;
 $loopLimit = sp + 24|0;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $zOld = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zStr>>2] = $5;
 $6 = HEAP32[$zStr>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3_value_bytes($9)|0);
 HEAP32[$nStr>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_text($13)|0);
 HEAP32[$zPattern>>2] = $14;
 $15 = HEAP32[$zPattern>>2]|0;
 $16 = ($15|0)==(0|0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$zPattern>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  _sqlite3_result_value($21,$23);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3_value_bytes($26)|0);
 HEAP32[$nPattern>>2] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_value_text($30)|0);
 HEAP32[$zRep>>2] = $31;
 $32 = HEAP32[$zRep>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (_sqlite3_value_bytes($36)|0);
 HEAP32[$nRep>>2] = $37;
 $38 = HEAP32[$nStr>>2]|0;
 $39 = (($38) + 1)|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = $nOut;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = (($42) + 4)|0;
 $45 = $44;
 HEAP32[$45>>2] = $41;
 $46 = HEAP32[$0>>2]|0;
 $47 = $nOut;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) + 4)|0;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (_contextMalloc($46,$49,$52)|0);
 HEAP32[$zOut>>2] = $53;
 $54 = HEAP32[$zOut>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$nStr>>2]|0;
 $57 = HEAP32[$nPattern>>2]|0;
 $58 = (($56) - ($57))|0;
 HEAP32[$loopLimit>>2] = $58;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$loopLimit>>2]|0;
  $61 = ($59|0)<=($60|0);
  if (!($61)) {
   label = 18;
   break;
  }
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$zStr>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = HEAP32[$zPattern>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = ($66|0)!=($69|0);
  if ($70) {
   label = 11;
  } else {
   $71 = HEAP32[$i>>2]|0;
   $72 = HEAP32[$zStr>>2]|0;
   $73 = (($72) + ($71)|0);
   $74 = HEAP32[$zPattern>>2]|0;
   $75 = HEAP32[$nPattern>>2]|0;
   $76 = (_memcmp($73,$74,$75)|0);
   $77 = ($76|0)!=(0);
   if ($77) {
    label = 11;
   } else {
    $86 = HEAP32[$0>>2]|0;
    $87 = (_sqlite3_context_db_handle($86)|0);
    HEAP32[$db>>2] = $87;
    $88 = HEAP32[$nRep>>2]|0;
    $89 = HEAP32[$nPattern>>2]|0;
    $90 = (($88) - ($89))|0;
    $91 = ($90|0)<(0);
    $92 = $91 << 31 >> 31;
    $93 = $nOut;
    $94 = $93;
    $95 = HEAP32[$94>>2]|0;
    $96 = (($93) + 4)|0;
    $97 = $96;
    $98 = HEAP32[$97>>2]|0;
    $99 = (_i64Add(($95|0),($98|0),($90|0),($92|0))|0);
    $100 = tempRet0;
    $101 = $nOut;
    $102 = $101;
    HEAP32[$102>>2] = $99;
    $103 = (($101) + 4)|0;
    $104 = $103;
    HEAP32[$104>>2] = $100;
    $105 = $nOut;
    $106 = $105;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($105) + 4)|0;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (_i64Subtract(($107|0),($110|0),1,0)|0);
    $112 = tempRet0;
    $113 = HEAP32[$db>>2]|0;
    $114 = ((($113)) + 88|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = ($115|0)<(0);
    $117 = $116 << 31 >> 31;
    $118 = ($112|0)>($117|0);
    $119 = ($111>>>0)>($115>>>0);
    $120 = ($112|0)==($117|0);
    $121 = $120 & $119;
    $122 = $118 | $121;
    if ($122) {
     label = 13;
     break;
    }
    $125 = HEAP32[$zOut>>2]|0;
    HEAP32[$zOld>>2] = $125;
    $126 = HEAP32[$zOut>>2]|0;
    $127 = $nOut;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = ($129|0)<(0);
    $134 = $133 << 31 >> 31;
    $135 = (_sqlite3_realloc64($126,$129,$134)|0);
    HEAP32[$zOut>>2] = $135;
    $136 = HEAP32[$zOut>>2]|0;
    $137 = ($136|0)==(0|0);
    if ($137) {
     label = 15;
     break;
    }
    $140 = HEAP32[$j>>2]|0;
    $141 = HEAP32[$zOut>>2]|0;
    $142 = (($141) + ($140)|0);
    $143 = HEAP32[$zRep>>2]|0;
    $144 = HEAP32[$nRep>>2]|0;
    _memcpy(($142|0),($143|0),($144|0))|0;
    $145 = HEAP32[$nRep>>2]|0;
    $146 = HEAP32[$j>>2]|0;
    $147 = (($146) + ($145))|0;
    HEAP32[$j>>2] = $147;
    $148 = HEAP32[$nPattern>>2]|0;
    $149 = (($148) - 1)|0;
    $150 = HEAP32[$i>>2]|0;
    $151 = (($150) + ($149))|0;
    HEAP32[$i>>2] = $151;
   }
  }
  if ((label|0) == 11) {
   label = 0;
   $78 = HEAP32[$i>>2]|0;
   $79 = HEAP32[$zStr>>2]|0;
   $80 = (($79) + ($78)|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = HEAP32[$j>>2]|0;
   $83 = (($82) + 1)|0;
   HEAP32[$j>>2] = $83;
   $84 = HEAP32[$zOut>>2]|0;
   $85 = (($84) + ($82)|0);
   HEAP8[$85>>0] = $81;
  }
  $152 = HEAP32[$i>>2]|0;
  $153 = (($152) + 1)|0;
  HEAP32[$i>>2] = $153;
 }
 if ((label|0) == 13) {
  $123 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_toobig($123);
  $124 = HEAP32[$zOut>>2]|0;
  _sqlite3_free($124);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 15) {
  $138 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($138);
  $139 = HEAP32[$zOld>>2]|0;
  _sqlite3_free($139);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 18) {
  $154 = HEAP32[$j>>2]|0;
  $155 = HEAP32[$zOut>>2]|0;
  $156 = (($155) + ($154)|0);
  $157 = HEAP32[$i>>2]|0;
  $158 = HEAP32[$zStr>>2]|0;
  $159 = (($158) + ($157)|0);
  $160 = HEAP32[$nStr>>2]|0;
  $161 = HEAP32[$i>>2]|0;
  $162 = (($160) - ($161))|0;
  _memcpy(($156|0),($159|0),($162|0))|0;
  $163 = HEAP32[$nStr>>2]|0;
  $164 = HEAP32[$i>>2]|0;
  $165 = (($163) - ($164))|0;
  $166 = HEAP32[$j>>2]|0;
  $167 = (($166) + ($165))|0;
  HEAP32[$j>>2] = $167;
  $168 = HEAP32[$j>>2]|0;
  $169 = HEAP32[$zOut>>2]|0;
  $170 = (($169) + ($168)|0);
  HEAP8[$170>>0] = 0;
  $171 = HEAP32[$0>>2]|0;
  $172 = HEAP32[$zOut>>2]|0;
  $173 = HEAP32[$j>>2]|0;
  _sqlite3_result_text($171,$172,$173,142);
  STACKTOP = sp;return;
 }
}
function _zeroblobFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $n = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int64($4)|0);
 $6 = tempRet0;
 $7 = $n;
 $8 = $7;
 HEAP32[$8>>2] = $5;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $6;
 $11 = $n;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)<(0);
 $18 = $17 ? 0 : $5;
 $19 = $17 ? 0 : $6;
 $20 = $n;
 $21 = $20;
 HEAP32[$21>>2] = $18;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $19;
 $24 = HEAP32[$0>>2]|0;
 $25 = $n;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_result_zeroblob64($24,$27,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$0>>2]|0;
 $35 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($34,$35);
 STACKTOP = sp;return;
}
function _sumStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0.0;
 var $45 = 0.0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $type = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $type = sp + 8|0;
 $v = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_aggregate_context($3,32)|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_numeric_type($6)|0);
 HEAP32[$type>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$type>>2]|0;
 $11 = ($10|0)!=(5);
 $or$cond = $9 & $11;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 $22 = $13;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = HEAP32[$type>>2]|0;
 $27 = ($26|0)==(1);
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 if (!($27)) {
  $68 = (+_sqlite3_value_double($29));
  $69 = HEAP32[$p>>2]|0;
  $70 = +HEAPF64[$69>>3];
  $71 = $70 + $68;
  HEAPF64[$69>>3] = $71;
  $72 = HEAP32[$p>>2]|0;
  $73 = ((($72)) + 25|0);
  HEAP8[$73>>0] = 1;
  STACKTOP = sp;return;
 }
 $30 = (_sqlite3_value_int64($29)|0);
 $31 = tempRet0;
 $32 = $v;
 $33 = $32;
 HEAP32[$33>>2] = $30;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = $v;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (+($38>>>0)) + (4294967296.0*(+($41|0)));
 $43 = HEAP32[$p>>2]|0;
 $44 = +HEAPF64[$43>>3];
 $45 = $44 + $42;
 HEAPF64[$43>>3] = $45;
 $46 = HEAP32[$p>>2]|0;
 $47 = ((($46)) + 25|0);
 $48 = HEAP8[$47>>0]|0;
 $49 = $48&255;
 $50 = HEAP32[$p>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = $52&255;
 $54 = $49 | $53;
 $55 = ($54|0)==(0);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$p>>2]|0;
 $57 = ((($56)) + 8|0);
 $58 = $v;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (_sqlite3AddInt64($57,$60,$63)|0);
 $65 = ($64|0)!=(0);
 if (!($65)) {
  STACKTOP = sp;return;
 }
 $66 = HEAP32[$p>>2]|0;
 $67 = ((($66)) + 24|0);
 HEAP8[$67>>0] = 1;
 STACKTOP = sp;return;
}
function _sumFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 $14 = ($9>>>0)>(0);
 $15 = ($12|0)==(0);
 $16 = $15 & $14;
 $17 = $13 | $16;
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = ($20<<24>>24)!=(0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($22,38535,-1);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 25|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25<<24>>24)!=(0);
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$p>>2]|0;
 if ($26) {
  $29 = +HEAPF64[$28>>3];
  _sqlite3_result_double($27,$29);
  STACKTOP = sp;return;
 } else {
  $30 = ((($28)) + 8|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3_result_int64($27,$33,$36);
  STACKTOP = sp;return;
 }
}
function _totalFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0.0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$p>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $8 = 0.0;
  _sqlite3_result_double($3,$8);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = +HEAPF64[$6>>3];
 $8 = $7;
 _sqlite3_result_double($3,$8);
 STACKTOP = sp;return;
}
function _avgFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 $14 = ($9>>>0)>(0);
 $15 = ($12|0)==(0);
 $16 = $15 & $14;
 $17 = $13 | $16;
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = +HEAPF64[$19>>3];
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (+($25>>>0)) + (4294967296.0*(+($28|0)));
 $30 = $20 / $29;
 _sqlite3_result_double($18,$30);
 STACKTOP = sp;return;
}
function _countStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_aggregate_context($3,8)|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $$old = HEAP32[$p>>2]|0;
  $$old1 = ($$old|0)!=(0|0);
  if (!($$old1)) {
   STACKTOP = sp;return;
  }
 } else {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_type($8)|0);
  $10 = (5)!=($9|0);
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   STACKTOP = sp;return;
  }
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 $22 = $13;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 STACKTOP = sp;return;
}
function _countFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$p>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $13 = 0;$14 = 0;
  _sqlite3_result_int64($3,$13,$14);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $9;$14 = $12;
 _sqlite3_result_int64($3,$13,$14);
 STACKTOP = sp;return;
}
function _groupConcatStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $firstTerm = 0, $nSep = 0, $nVal = 0, $pAccum = 0, $zSep = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $zVal = sp + 24|0;
 $pAccum = sp + 20|0;
 $zSep = sp + 16|0;
 $nVal = sp + 12|0;
 $nSep = sp + 8|0;
 $db = sp + 4|0;
 $firstTerm = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 $6 = ($5|0)==(5);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_aggregate_context($7,28)|0);
 HEAP32[$pAccum>>2] = $8;
 $9 = HEAP32[$pAccum>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$pAccum>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0);
 $17 = $16&1;
 HEAP32[$firstTerm>>2] = $17;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 88|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pAccum>>2]|0;
 $22 = ((($21)) + 20|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$firstTerm>>2]|0;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $25 = HEAP32[$1>>2]|0;
  $26 = ($25|0)==(2);
  if ($26) {
   $27 = HEAP32[$2>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3_value_text($29)|0);
   HEAP32[$zSep>>2] = $30;
   $31 = HEAP32[$2>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (_sqlite3_value_bytes($33)|0);
   HEAP32[$nSep>>2] = $34;
  } else {
   HEAP32[$zSep>>2] = 30773;
   HEAP32[$nSep>>2] = 1;
  }
  $35 = HEAP32[$nSep>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$pAccum>>2]|0;
   $38 = HEAP32[$zSep>>2]|0;
   $39 = HEAP32[$nSep>>2]|0;
   _sqlite3StrAccumAppend($37,$38,$39);
  }
 }
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (_sqlite3_value_text($41)|0);
 HEAP32[$zVal>>2] = $42;
 $43 = HEAP32[$2>>2]|0;
 $44 = HEAP32[$43>>2]|0;
 $45 = (_sqlite3_value_bytes($44)|0);
 HEAP32[$nVal>>2] = $45;
 $46 = HEAP32[$zVal>>2]|0;
 $47 = ($46|0)!=(0|0);
 if (!($47)) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$pAccum>>2]|0;
 $49 = HEAP32[$zVal>>2]|0;
 $50 = HEAP32[$nVal>>2]|0;
 _sqlite3StrAccumAppend($48,$49,$50);
 STACKTOP = sp;return;
}
function _groupConcatFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAccum = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pAccum = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$pAccum>>2] = $2;
 $3 = HEAP32[$pAccum>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$pAccum>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_toobig($10);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$pAccum>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(1);
 $16 = HEAP32[$0>>2]|0;
 if ($15) {
  _sqlite3_result_error_nomem($16);
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$pAccum>>2]|0;
  $18 = (_sqlite3StrAccumFinish($17)|0);
  _sqlite3_result_text($16,$18,-1,142);
  STACKTOP = sp;return;
 }
}
function _sqlite3FuncDefInsert($pHash,$pDef) {
 $pHash = $pHash|0;
 $pDef = $pDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c1 = 0, $h = 0, $nName = 0, $pOther = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pOther = sp + 8|0;
 $nName = sp + 4|0;
 $c1 = sp + 20|0;
 $h = sp;
 HEAP32[$0>>2] = $pHash;
 HEAP32[$1>>2] = $pDef;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 HEAP32[$nName>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 HEAP8[$c1>>0] = $9;
 $10 = HEAP8[$c1>>0]|0;
 $11 = $10&255;
 $12 = (16974 + ($11)|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = HEAP32[$nName>>2]|0;
 $16 = (($14) + ($15))|0;
 $17 = (($16|0) % 23)&-1;
 HEAP32[$h>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$h>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 20|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$nName>>2]|0;
 $24 = (_functionSearch($18,$19,$22,$23)|0);
 HEAP32[$pOther>>2] = $24;
 $25 = HEAP32[$pOther>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$pOther>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 8|0);
  HEAP32[$31>>2] = $29;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$pOther>>2]|0;
  $34 = ((($33)) + 8|0);
  HEAP32[$34>>2] = $32;
  STACKTOP = sp;return;
 } else {
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP32[$36>>2] = 0;
  $37 = HEAP32[$h>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = (($38) + ($37<<2)|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = ((($41)) + 24|0);
  HEAP32[$42>>2] = $40;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$h>>2]|0;
  $45 = HEAP32[$0>>2]|0;
  $46 = (($45) + ($44<<2)|0);
  HEAP32[$46>>2] = $43;
  STACKTOP = sp;return;
 }
}
function _sqlite3RegisterDateTimeFunctions() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aFunc = 0, $i = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 8|0;
 $pHash = sp + 4|0;
 $aFunc = sp;
 HEAP32[$pHash>>2] = 848;
 HEAP32[$aFunc>>2] = 5676;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0|0)<(8);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$pHash>>2]|0;
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$aFunc>>2]|0;
  $5 = (($4) + ($3<<5)|0);
  _sqlite3FuncDefInsert($2,$5);
  $6 = HEAP32[$i>>2]|0;
  $7 = (($6) + 1)|0;
  HEAP32[$i>>2] = $7;
 }
 STACKTOP = sp;return;
}
function _sqlite3AlterFunctions() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aFunc = 0, $i = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp + 8|0;
 $pHash = sp + 4|0;
 $aFunc = sp;
 HEAP32[$pHash>>2] = 848;
 HEAP32[$aFunc>>2] = 5932;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0|0)<(3);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$pHash>>2]|0;
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$aFunc>>2]|0;
  $5 = (($4) + ($3<<5)|0);
  _sqlite3FuncDefInsert($2,$5);
  $6 = HEAP32[$i>>2]|0;
  $7 = (($6) + 1)|0;
  HEAP32[$i>>2] = $7;
 }
 STACKTOP = sp;return;
}
function _contextMalloc($context,$0,$1) {
 $context = $context|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $z = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$2>>2] = $context;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3_context_db_handle($8)|0);
 HEAP32[$db>>2] = $9;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 88|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = ($15|0)>($20|0);
 $22 = ($12>>>0)>($18>>>0);
 $23 = ($15|0)==($20|0);
 $24 = $23 & $22;
 $25 = $21 | $24;
 if ($25) {
  $26 = HEAP32[$2>>2]|0;
  _sqlite3_result_error_toobig($26);
  HEAP32[$z>>2] = 0;
  $37 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $27 = $3;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (_sqlite3Malloc($29,$32)|0);
 HEAP32[$z>>2] = $33;
 $34 = HEAP32[$z>>2]|0;
 $35 = ($34|0)!=(0|0);
 if ($35) {
  $37 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $36 = HEAP32[$2>>2]|0;
 _sqlite3_result_error_nomem($36);
 $37 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3GetFuncCollSeq($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) - 1)|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($4*20)|0)|0);
 HEAP32[$pOp>>2] = $10;
 $11 = HEAP32[$pOp>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3SkipAccumulatorLoad($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 HEAP8[$2>>0] = 1;
 STACKTOP = sp;return;
}
function _functionSearch($pHash,$h,$zFunc,$nFunc) {
 $pHash = $pHash|0;
 $h = $h|0;
 $zFunc = $zFunc|0;
 $nFunc = $nFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $h;
 HEAP32[$3>>2] = $zFunc;
 HEAP32[$4>>2] = $nFunc;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + ($5<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   label = 7;
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$3>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  $16 = (_sqlite3_strnicmp($13,$14,$15)|0);
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$4>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18)|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23 << 24 >> 24;
   $25 = ($24|0)==(0);
   if ($25) {
    label = 5;
    break;
   }
  }
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 24|0);
  $29 = HEAP32[$28>>2]|0;
  HEAP32[$p>>2] = $29;
 }
 if ((label|0) == 5) {
  $26 = HEAP32[$p>>2]|0;
  HEAP32[$0>>2] = $26;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _juliandayFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $x = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeJD($x);
 $8 = HEAP32[$0>>2]|0;
 $9 = $x;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (+($11>>>0)) + (4294967296.0*(+($14|0)));
 $16 = $15 / 8.64E+7;
 _sqlite3_result_double($8,$16);
 STACKTOP = sp;return;
}
function _dateFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $x = 0, $zBuf = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $x = sp;
 $zBuf = sp + 72|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeYMD($x);
 $8 = ((($x)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($x)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($x)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 (_sqlite3_snprintf(100,$zBuf,38798,$vararg_buffer)|0);
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($14,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _timeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $x = 0;
 var $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $x = sp;
 $zBuf = sp + 72|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeHMS($x);
 $8 = ((($x)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($x)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($x)) + 32|0);
 $13 = +HEAPF64[$12>>3];
 $14 = (~~(($13)));
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $14;
 (_sqlite3_snprintf(100,$zBuf,38813,$vararg_buffer)|0);
 $15 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($15,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _datetimeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $x = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $x = sp;
 $zBuf = sp + 84|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeYMD_HMS($x);
 $8 = ((($x)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($x)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($x)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($x)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($x)) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($x)) + 32|0);
 $19 = +HEAPF64[$18>>3];
 $20 = (~~(($19)));
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $15;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $17;
 $vararg_ptr5 = ((($vararg_buffer)) + 20|0);
 HEAP32[$vararg_ptr5>>2] = $20;
 (_sqlite3_snprintf(100,$zBuf,38828,$vararg_buffer)|0);
 $21 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($21,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _strftimeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0.0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0.0, $276 = 0.0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $n = 0, $nDay = 0, $s = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $wd = 0, $x = 0, $y = 0, $z = 0, $zBuf = 0, $zFmt = 0, dest = 0, label = 0, sp = 0, src = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0;
 $vararg_buffer28 = sp + 192|0;
 $vararg_buffer25 = sp + 184|0;
 $vararg_buffer22 = sp + 176|0;
 $vararg_buffer19 = sp + 168|0;
 $vararg_buffer16 = sp + 160|0;
 $vararg_buffer13 = sp + 152|0;
 $vararg_buffer10 = sp + 144|0;
 $vararg_buffer7 = sp + 136|0;
 $vararg_buffer4 = sp + 128|0;
 $vararg_buffer1 = sp + 120|0;
 $vararg_buffer = sp + 112|0;
 $0 = sp + 232|0;
 $1 = sp + 228|0;
 $2 = sp + 224|0;
 $x = sp + 64|0;
 $n = sp + 56|0;
 $i = sp + 220|0;
 $j = sp + 216|0;
 $z = sp + 212|0;
 $db = sp + 208|0;
 $zFmt = sp + 204|0;
 $zBuf = sp + 236|0;
 $s = sp + 48|0;
 $nDay = sp + 200|0;
 $y = sp;
 $wd = sp + 196|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zFmt>>2] = $7;
 $8 = HEAP32[$zFmt>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) - 1)|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = (_isDate($10,$12,$14,$x)|0);
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = (_sqlite3_context_db_handle($17)|0);
 HEAP32[$db>>2] = $18;
 HEAP32[$i>>2] = 0;
 $19 = $n;
 $20 = $19;
 HEAP32[$20>>2] = 1;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 L10: while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$zFmt>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = ($26<<24>>24)!=(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$zFmt>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31 << 24 >> 24;
  $33 = ($32|0)==(37);
  if ($33) {
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + 1)|0;
   $36 = HEAP32[$zFmt>>2]|0;
   $37 = (($36) + ($35)|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38 << 24 >> 24;
   switch ($39|0) {
   case 87: case 83: case 77: case 109: case 72: case 100:  {
    $40 = $n;
    $41 = $40;
    $42 = HEAP32[$41>>2]|0;
    $43 = (($40) + 4)|0;
    $44 = $43;
    $45 = HEAP32[$44>>2]|0;
    $46 = (_i64Add(($42|0),($45|0),1,0)|0);
    $47 = tempRet0;
    $48 = $n;
    $49 = $48;
    HEAP32[$49>>2] = $46;
    $50 = (($48) + 4)|0;
    $51 = $50;
    HEAP32[$51>>2] = $47;
    break;
   }
   case 37: case 119:  {
    break;
   }
   case 102:  {
    $52 = $n;
    $53 = $52;
    $54 = HEAP32[$53>>2]|0;
    $55 = (($52) + 4)|0;
    $56 = $55;
    $57 = HEAP32[$56>>2]|0;
    $58 = (_i64Add(($54|0),($57|0),8,0)|0);
    $59 = tempRet0;
    $60 = $n;
    $61 = $60;
    HEAP32[$61>>2] = $58;
    $62 = (($60) + 4)|0;
    $63 = $62;
    HEAP32[$63>>2] = $59;
    break;
   }
   case 106:  {
    $64 = $n;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = (($64) + 4)|0;
    $68 = $67;
    $69 = HEAP32[$68>>2]|0;
    $70 = (_i64Add(($66|0),($69|0),3,0)|0);
    $71 = tempRet0;
    $72 = $n;
    $73 = $72;
    HEAP32[$73>>2] = $70;
    $74 = (($72) + 4)|0;
    $75 = $74;
    HEAP32[$75>>2] = $71;
    break;
   }
   case 89:  {
    $76 = $n;
    $77 = $76;
    $78 = HEAP32[$77>>2]|0;
    $79 = (($76) + 4)|0;
    $80 = $79;
    $81 = HEAP32[$80>>2]|0;
    $82 = (_i64Add(($78|0),($81|0),8,0)|0);
    $83 = tempRet0;
    $84 = $n;
    $85 = $84;
    HEAP32[$85>>2] = $82;
    $86 = (($84) + 4)|0;
    $87 = $86;
    HEAP32[$87>>2] = $83;
    break;
   }
   case 74: case 115:  {
    $88 = $n;
    $89 = $88;
    $90 = HEAP32[$89>>2]|0;
    $91 = (($88) + 4)|0;
    $92 = $91;
    $93 = HEAP32[$92>>2]|0;
    $94 = (_i64Add(($90|0),($93|0),50,0)|0);
    $95 = tempRet0;
    $96 = $n;
    $97 = $96;
    HEAP32[$97>>2] = $94;
    $98 = (($96) + 4)|0;
    $99 = $98;
    HEAP32[$99>>2] = $95;
    break;
   }
   default: {
    label = 44;
    break L10;
   }
   }
   $100 = HEAP32[$i>>2]|0;
   $101 = (($100) + 1)|0;
   HEAP32[$i>>2] = $101;
  }
  $102 = HEAP32[$i>>2]|0;
  $103 = (($102) + 1)|0;
  HEAP32[$i>>2] = $103;
  $104 = $n;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = (_i64Add(($106|0),($109|0),1,0)|0);
  $111 = tempRet0;
  $112 = $n;
  $113 = $112;
  HEAP32[$113>>2] = $110;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = $111;
 }
 if ((label|0) == 44) {
  STACKTOP = sp;return;
 }
 $116 = $n;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = (($116) + 4)|0;
 $120 = $119;
 $121 = HEAP32[$120>>2]|0;
 $122 = ($121>>>0)<(0);
 $123 = ($118>>>0)<(100);
 $124 = ($121|0)==(0);
 $125 = $124 & $123;
 $126 = $122 | $125;
 if ($126) {
  HEAP32[$z>>2] = $zBuf;
 } else {
  $127 = $n;
  $128 = $127;
  $129 = HEAP32[$128>>2]|0;
  $130 = (($127) + 4)|0;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = HEAP32[$db>>2]|0;
  $134 = ((($133)) + 88|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = ($135|0)<(0);
  $137 = $136 << 31 >> 31;
  $138 = ($132>>>0)>($137>>>0);
  $139 = ($129>>>0)>($135>>>0);
  $140 = ($132|0)==($137|0);
  $141 = $140 & $139;
  $142 = $138 | $141;
  if ($142) {
   $143 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_toobig($143);
   STACKTOP = sp;return;
  }
  $144 = HEAP32[$db>>2]|0;
  $145 = $n;
  $146 = $145;
  $147 = HEAP32[$146>>2]|0;
  $148 = (($145) + 4)|0;
  $149 = $148;
  $150 = HEAP32[$149>>2]|0;
  $151 = ($147|0)<(0);
  $152 = $151 << 31 >> 31;
  $153 = (_sqlite3DbMallocRawNN($144,$147,$152)|0);
  HEAP32[$z>>2] = $153;
  $154 = HEAP32[$z>>2]|0;
  $155 = ($154|0)==(0|0);
  if ($155) {
   $156 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_nomem($156);
   STACKTOP = sp;return;
  }
 }
 _computeJD($x);
 _computeYMD_HMS($x);
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $157 = HEAP32[$i>>2]|0;
  $158 = HEAP32[$zFmt>>2]|0;
  $159 = (($158) + ($157)|0);
  $160 = HEAP8[$159>>0]|0;
  $161 = ($160<<24>>24)!=(0);
  if (!($161)) {
   break;
  }
  $162 = HEAP32[$i>>2]|0;
  $163 = HEAP32[$zFmt>>2]|0;
  $164 = (($163) + ($162)|0);
  $165 = HEAP8[$164>>0]|0;
  $166 = $165 << 24 >> 24;
  $167 = ($166|0)!=(37);
  $168 = HEAP32[$i>>2]|0;
  L39: do {
   if ($167) {
    $169 = HEAP32[$zFmt>>2]|0;
    $170 = (($169) + ($168)|0);
    $171 = HEAP8[$170>>0]|0;
    $172 = HEAP32[$j>>2]|0;
    $173 = (($172) + 1)|0;
    HEAP32[$j>>2] = $173;
    $174 = HEAP32[$z>>2]|0;
    $175 = (($174) + ($172)|0);
    HEAP8[$175>>0] = $171;
   } else {
    $176 = (($168) + 1)|0;
    HEAP32[$i>>2] = $176;
    $177 = HEAP32[$i>>2]|0;
    $178 = HEAP32[$zFmt>>2]|0;
    $179 = (($178) + ($177)|0);
    $180 = HEAP8[$179>>0]|0;
    $181 = $180 << 24 >> 24;
    do {
     switch ($181|0) {
     case 100:  {
      $182 = HEAP32[$j>>2]|0;
      $183 = HEAP32[$z>>2]|0;
      $184 = (($183) + ($182)|0);
      $185 = ((($x)) + 16|0);
      $186 = HEAP32[$185>>2]|0;
      HEAP32[$vararg_buffer>>2] = $186;
      (_sqlite3_snprintf(3,$184,38858,$vararg_buffer)|0);
      $187 = HEAP32[$j>>2]|0;
      $188 = (($187) + 2)|0;
      HEAP32[$j>>2] = $188;
      break L39;
      break;
     }
     case 102:  {
      $189 = ((($x)) + 32|0);
      $190 = +HEAPF64[$189>>3];
      HEAPF64[$s>>3] = $190;
      $191 = +HEAPF64[$s>>3];
      $192 = $191 > 59.999000000000002;
      if ($192) {
       HEAPF64[$s>>3] = 59.999000000000002;
      }
      $193 = HEAP32[$j>>2]|0;
      $194 = HEAP32[$z>>2]|0;
      $195 = (($194) + ($193)|0);
      $196 = +HEAPF64[$s>>3];
      HEAPF64[$vararg_buffer1>>3] = $196;
      (_sqlite3_snprintf(7,$195,38863,$vararg_buffer1)|0);
      $197 = HEAP32[$j>>2]|0;
      $198 = HEAP32[$z>>2]|0;
      $199 = (($198) + ($197)|0);
      $200 = (_sqlite3Strlen30($199)|0);
      $201 = HEAP32[$j>>2]|0;
      $202 = (($201) + ($200))|0;
      HEAP32[$j>>2] = $202;
      break L39;
      break;
     }
     case 72:  {
      $203 = HEAP32[$j>>2]|0;
      $204 = HEAP32[$z>>2]|0;
      $205 = (($204) + ($203)|0);
      $206 = ((($x)) + 20|0);
      $207 = HEAP32[$206>>2]|0;
      HEAP32[$vararg_buffer4>>2] = $207;
      (_sqlite3_snprintf(3,$205,38858,$vararg_buffer4)|0);
      $208 = HEAP32[$j>>2]|0;
      $209 = (($208) + 2)|0;
      HEAP32[$j>>2] = $209;
      break L39;
      break;
     }
     case 106: case 87:  {
      dest=$y; src=$x; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
      $210 = ((($y)) + 42|0);
      HEAP8[$210>>0] = 0;
      $211 = ((($y)) + 12|0);
      HEAP32[$211>>2] = 1;
      $212 = ((($y)) + 16|0);
      HEAP32[$212>>2] = 1;
      _computeJD($y);
      $213 = $x;
      $214 = $213;
      $215 = HEAP32[$214>>2]|0;
      $216 = (($213) + 4)|0;
      $217 = $216;
      $218 = HEAP32[$217>>2]|0;
      $219 = $y;
      $220 = $219;
      $221 = HEAP32[$220>>2]|0;
      $222 = (($219) + 4)|0;
      $223 = $222;
      $224 = HEAP32[$223>>2]|0;
      $225 = (_i64Subtract(($215|0),($218|0),($221|0),($224|0))|0);
      $226 = tempRet0;
      $227 = (_i64Add(($225|0),($226|0),43200000,0)|0);
      $228 = tempRet0;
      $229 = (___divdi3(($227|0),($228|0),86400000,0)|0);
      $230 = tempRet0;
      HEAP32[$nDay>>2] = $229;
      $231 = HEAP32[$i>>2]|0;
      $232 = HEAP32[$zFmt>>2]|0;
      $233 = (($232) + ($231)|0);
      $234 = HEAP8[$233>>0]|0;
      $235 = $234 << 24 >> 24;
      $236 = ($235|0)==(87);
      if ($236) {
       $237 = $x;
       $238 = $237;
       $239 = HEAP32[$238>>2]|0;
       $240 = (($237) + 4)|0;
       $241 = $240;
       $242 = HEAP32[$241>>2]|0;
       $243 = (_i64Add(($239|0),($242|0),43200000,0)|0);
       $244 = tempRet0;
       $245 = (___divdi3(($243|0),($244|0),86400000,0)|0);
       $246 = tempRet0;
       $247 = (___remdi3(($245|0),($246|0),7,0)|0);
       $248 = tempRet0;
       HEAP32[$wd>>2] = $247;
       $249 = HEAP32[$j>>2]|0;
       $250 = HEAP32[$z>>2]|0;
       $251 = (($250) + ($249)|0);
       $252 = HEAP32[$nDay>>2]|0;
       $253 = (($252) + 7)|0;
       $254 = HEAP32[$wd>>2]|0;
       $255 = (($253) - ($254))|0;
       $256 = (($255|0) / 7)&-1;
       HEAP32[$vararg_buffer7>>2] = $256;
       (_sqlite3_snprintf(3,$251,38858,$vararg_buffer7)|0);
       $257 = HEAP32[$j>>2]|0;
       $258 = (($257) + 2)|0;
       HEAP32[$j>>2] = $258;
       break L39;
      } else {
       $259 = HEAP32[$j>>2]|0;
       $260 = HEAP32[$z>>2]|0;
       $261 = (($260) + ($259)|0);
       $262 = HEAP32[$nDay>>2]|0;
       $263 = (($262) + 1)|0;
       HEAP32[$vararg_buffer10>>2] = $263;
       (_sqlite3_snprintf(4,$261,38870,$vararg_buffer10)|0);
       $264 = HEAP32[$j>>2]|0;
       $265 = (($264) + 3)|0;
       HEAP32[$j>>2] = $265;
       break L39;
      }
      break;
     }
     case 74:  {
      $266 = HEAP32[$j>>2]|0;
      $267 = HEAP32[$z>>2]|0;
      $268 = (($267) + ($266)|0);
      $269 = $x;
      $270 = $269;
      $271 = HEAP32[$270>>2]|0;
      $272 = (($269) + 4)|0;
      $273 = $272;
      $274 = HEAP32[$273>>2]|0;
      $275 = (+($271>>>0)) + (4294967296.0*(+($274|0)));
      $276 = $275 / 8.64E+7;
      HEAPF64[$vararg_buffer13>>3] = $276;
      (_sqlite3_snprintf(20,$268,20753,$vararg_buffer13)|0);
      $277 = HEAP32[$j>>2]|0;
      $278 = HEAP32[$z>>2]|0;
      $279 = (($278) + ($277)|0);
      $280 = (_sqlite3Strlen30($279)|0);
      $281 = HEAP32[$j>>2]|0;
      $282 = (($281) + ($280))|0;
      HEAP32[$j>>2] = $282;
      break L39;
      break;
     }
     case 109:  {
      $283 = HEAP32[$j>>2]|0;
      $284 = HEAP32[$z>>2]|0;
      $285 = (($284) + ($283)|0);
      $286 = ((($x)) + 12|0);
      $287 = HEAP32[$286>>2]|0;
      HEAP32[$vararg_buffer16>>2] = $287;
      (_sqlite3_snprintf(3,$285,38858,$vararg_buffer16)|0);
      $288 = HEAP32[$j>>2]|0;
      $289 = (($288) + 2)|0;
      HEAP32[$j>>2] = $289;
      break L39;
      break;
     }
     case 77:  {
      $290 = HEAP32[$j>>2]|0;
      $291 = HEAP32[$z>>2]|0;
      $292 = (($291) + ($290)|0);
      $293 = ((($x)) + 24|0);
      $294 = HEAP32[$293>>2]|0;
      HEAP32[$vararg_buffer19>>2] = $294;
      (_sqlite3_snprintf(3,$292,38858,$vararg_buffer19)|0);
      $295 = HEAP32[$j>>2]|0;
      $296 = (($295) + 2)|0;
      HEAP32[$j>>2] = $296;
      break L39;
      break;
     }
     case 115:  {
      $297 = HEAP32[$j>>2]|0;
      $298 = HEAP32[$z>>2]|0;
      $299 = (($298) + ($297)|0);
      $300 = $x;
      $301 = $300;
      $302 = HEAP32[$301>>2]|0;
      $303 = (($300) + 4)|0;
      $304 = $303;
      $305 = HEAP32[$304>>2]|0;
      $306 = (___divdi3(($302|0),($305|0),1000,0)|0);
      $307 = tempRet0;
      $308 = (_i64Subtract(($306|0),($307|0),413362496,49)|0);
      $309 = tempRet0;
      $310 = $vararg_buffer22;
      $311 = $310;
      HEAP32[$311>>2] = $308;
      $312 = (($310) + 4)|0;
      $313 = $312;
      HEAP32[$313>>2] = $309;
      (_sqlite3_snprintf(30,$299,19285,$vararg_buffer22)|0);
      $314 = HEAP32[$j>>2]|0;
      $315 = HEAP32[$z>>2]|0;
      $316 = (($315) + ($314)|0);
      $317 = (_sqlite3Strlen30($316)|0);
      $318 = HEAP32[$j>>2]|0;
      $319 = (($318) + ($317))|0;
      HEAP32[$j>>2] = $319;
      break L39;
      break;
     }
     case 83:  {
      $320 = HEAP32[$j>>2]|0;
      $321 = HEAP32[$z>>2]|0;
      $322 = (($321) + ($320)|0);
      $323 = ((($x)) + 32|0);
      $324 = +HEAPF64[$323>>3];
      $325 = (~~(($324)));
      HEAP32[$vararg_buffer25>>2] = $325;
      (_sqlite3_snprintf(3,$322,38858,$vararg_buffer25)|0);
      $326 = HEAP32[$j>>2]|0;
      $327 = (($326) + 2)|0;
      HEAP32[$j>>2] = $327;
      break L39;
      break;
     }
     case 119:  {
      $328 = $x;
      $329 = $328;
      $330 = HEAP32[$329>>2]|0;
      $331 = (($328) + 4)|0;
      $332 = $331;
      $333 = HEAP32[$332>>2]|0;
      $334 = (_i64Add(($330|0),($333|0),129600000,0)|0);
      $335 = tempRet0;
      $336 = (___divdi3(($334|0),($335|0),86400000,0)|0);
      $337 = tempRet0;
      $338 = (___remdi3(($336|0),($337|0),7,0)|0);
      $339 = tempRet0;
      $340 = $338&255;
      $341 = $340 << 24 >> 24;
      $342 = (($341) + 48)|0;
      $343 = $342&255;
      $344 = HEAP32[$j>>2]|0;
      $345 = (($344) + 1)|0;
      HEAP32[$j>>2] = $345;
      $346 = HEAP32[$z>>2]|0;
      $347 = (($346) + ($344)|0);
      HEAP8[$347>>0] = $343;
      break L39;
      break;
     }
     case 89:  {
      $348 = HEAP32[$j>>2]|0;
      $349 = HEAP32[$z>>2]|0;
      $350 = (($349) + ($348)|0);
      $351 = ((($x)) + 8|0);
      $352 = HEAP32[$351>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $352;
      (_sqlite3_snprintf(5,$350,38875,$vararg_buffer28)|0);
      $353 = HEAP32[$j>>2]|0;
      $354 = HEAP32[$z>>2]|0;
      $355 = (($354) + ($353)|0);
      $356 = (_sqlite3Strlen30($355)|0);
      $357 = HEAP32[$j>>2]|0;
      $358 = (($357) + ($356))|0;
      HEAP32[$j>>2] = $358;
      break L39;
      break;
     }
     default: {
      $359 = HEAP32[$j>>2]|0;
      $360 = (($359) + 1)|0;
      HEAP32[$j>>2] = $360;
      $361 = HEAP32[$z>>2]|0;
      $362 = (($361) + ($359)|0);
      HEAP8[$362>>0] = 37;
      break L39;
     }
     }
    } while(0);
   }
  } while(0);
  $363 = HEAP32[$i>>2]|0;
  $364 = (($363) + 1)|0;
  HEAP32[$i>>2] = $364;
 }
 $365 = HEAP32[$j>>2]|0;
 $366 = HEAP32[$z>>2]|0;
 $367 = (($366) + ($365)|0);
 HEAP8[$367>>0] = 0;
 $368 = HEAP32[$0>>2]|0;
 $369 = HEAP32[$z>>2]|0;
 $370 = HEAP32[$z>>2]|0;
 $371 = ($370|0)==($zBuf|0);
 $372 = $371 ? (-1) : 169;
 _sqlite3_result_text($368,$369,-1,$372);
 STACKTOP = sp;return;
}
function _ctimeFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _timeFunc($3,0,0);
 STACKTOP = sp;return;
}
function _ctimestampFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _datetimeFunc($3,0,0);
 STACKTOP = sp;return;
}
function _cdateFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _dateFunc($3,0,0);
 STACKTOP = sp;return;
}
function _isDate($context,$argc,$argv,$p) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $i = 0, $or$cond = 0;
 var $z = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $i = sp + 8|0;
 $z = sp + 4|0;
 $eType = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $p;
 $5 = HEAP32[$4>>2]|0;
 dest=$5; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$4>>2]|0;
  $10 = (_setDateTimeToCurrent($8,$9)|0);
  HEAP32[$0>>2] = $10;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3_value_type($12)|0);
 HEAP32[$eType>>2] = $13;
 $14 = ($13|0)==(2);
 $15 = HEAP32[$eType>>2]|0;
 $16 = ($15|0)==(1);
 $or$cond = $14 | $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 do {
  if ($or$cond) {
   $19 = (+_sqlite3_value_double($18));
   $20 = $19 * 8.64E+7;
   $21 = $20 + 0.5;
   $22 = (~~$21)>>>0;
   $23 = +Math_abs($21) >= 1.0 ? $21 > 0.0 ? (~~+Math_min(+Math_floor($21 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($21 - +(~~$21 >>> 0)) / 4294967296.0) >>> 0 : 0;
   $24 = HEAP32[$4>>2]|0;
   $25 = $24;
   $26 = $25;
   HEAP32[$26>>2] = $22;
   $27 = (($25) + 4)|0;
   $28 = $27;
   HEAP32[$28>>2] = $23;
   $29 = HEAP32[$4>>2]|0;
   $30 = ((($29)) + 42|0);
   HEAP8[$30>>0] = 1;
  } else {
   $31 = (_sqlite3_value_text($18)|0);
   HEAP32[$z>>2] = $31;
   $32 = HEAP32[$z>>2]|0;
   $33 = ($32|0)!=(0|0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = HEAP32[$z>>2]|0;
    $36 = HEAP32[$4>>2]|0;
    $37 = (_parseDateOrTime($34,$35,$36)|0);
    $38 = ($37|0)!=(0);
    if (!($38)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 } while(0);
 HEAP32[$i>>2] = 1;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = ($39|0)<($40|0);
  if (!($41)) {
   label = 14;
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$3>>2]|0;
  $44 = (($43) + ($42<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3_value_text($45)|0);
  HEAP32[$z>>2] = $46;
  $47 = HEAP32[$z>>2]|0;
  $48 = ($47|0)==(0|0);
  if ($48) {
   label = 12;
   break;
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$z>>2]|0;
  $51 = HEAP32[$4>>2]|0;
  $52 = (_parseModifier($49,$50,$51)|0);
  $53 = ($52|0)!=(0);
  if ($53) {
   label = 12;
   break;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return (0)|0;
}
function _computeJD($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $A = 0, $B = 0, $D = 0, $M = 0, $X1 = 0, $X2 = 0, $Y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $Y = sp + 24|0;
 $M = sp + 20|0;
 $D = sp + 16|0;
 $A = sp + 12|0;
 $B = sp + 8|0;
 $X1 = sp + 4|0;
 $X2 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 42|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$Y>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$M>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$D>>2] = $17;
 } else {
  HEAP32[$Y>>2] = 2000;
  HEAP32[$M>>2] = 1;
  HEAP32[$D>>2] = 1;
 }
 $18 = HEAP32[$M>>2]|0;
 $19 = ($18|0)<=(2);
 if ($19) {
  $20 = HEAP32[$Y>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$Y>>2] = $21;
  $22 = HEAP32[$M>>2]|0;
  $23 = (($22) + 12)|0;
  HEAP32[$M>>2] = $23;
 }
 $24 = HEAP32[$Y>>2]|0;
 $25 = (($24|0) / 100)&-1;
 HEAP32[$A>>2] = $25;
 $26 = HEAP32[$A>>2]|0;
 $27 = (2 - ($26))|0;
 $28 = HEAP32[$A>>2]|0;
 $29 = (($28|0) / 4)&-1;
 $30 = (($27) + ($29))|0;
 HEAP32[$B>>2] = $30;
 $31 = HEAP32[$Y>>2]|0;
 $32 = (($31) + 4716)|0;
 $33 = ($32*36525)|0;
 $34 = (($33|0) / 100)&-1;
 HEAP32[$X1>>2] = $34;
 $35 = HEAP32[$M>>2]|0;
 $36 = (($35) + 1)|0;
 $37 = ($36*306001)|0;
 $38 = (($37|0) / 10000)&-1;
 HEAP32[$X2>>2] = $38;
 $39 = HEAP32[$X1>>2]|0;
 $40 = HEAP32[$X2>>2]|0;
 $41 = (($39) + ($40))|0;
 $42 = HEAP32[$D>>2]|0;
 $43 = (($41) + ($42))|0;
 $44 = HEAP32[$B>>2]|0;
 $45 = (($43) + ($44))|0;
 $46 = (+($45|0));
 $47 = $46 - 1524.5;
 $48 = $47 * 8.64E+7;
 $49 = (~~$48)>>>0;
 $50 = +Math_abs($48) >= 1.0 ? $48 > 0.0 ? (~~+Math_min(+Math_floor($48 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($48 - +(~~$48 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $51 = HEAP32[$0>>2]|0;
 $52 = $51;
 $53 = $52;
 HEAP32[$53>>2] = $49;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $50;
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 42|0);
 HEAP8[$57>>0] = 1;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 41|0);
 $60 = HEAP8[$59>>0]|0;
 $61 = ($60<<24>>24)!=(0);
 if (!($61)) {
  STACKTOP = sp;return;
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 20|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = Math_imul($64, 3600000)|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = ((($66)) + 24|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($68*60000)|0;
 $70 = (($65) + ($69))|0;
 $71 = ($70|0)<(0);
 $72 = $71 << 31 >> 31;
 $73 = HEAP32[$0>>2]|0;
 $74 = ((($73)) + 32|0);
 $75 = +HEAPF64[$74>>3];
 $76 = $75 * 1000.0;
 $77 = (~~$76)>>>0;
 $78 = +Math_abs($76) >= 1.0 ? $76 > 0.0 ? (~~+Math_min(+Math_floor($76 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($76 - +(~~$76 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $79 = (_i64Add(($70|0),($72|0),($77|0),($78|0))|0);
 $80 = tempRet0;
 $81 = HEAP32[$0>>2]|0;
 $82 = $81;
 $83 = $82;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($82) + 4)|0;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (_i64Add(($84|0),($87|0),($79|0),($80|0))|0);
 $89 = tempRet0;
 $90 = $81;
 $91 = $90;
 HEAP32[$91>>2] = $88;
 $92 = (($90) + 4)|0;
 $93 = $92;
 HEAP32[$93>>2] = $89;
 $94 = HEAP32[$0>>2]|0;
 $95 = ((($94)) + 43|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = ($96<<24>>24)!=(0);
 if (!($97)) {
  STACKTOP = sp;return;
 }
 $98 = HEAP32[$0>>2]|0;
 $99 = ((($98)) + 28|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100*60000)|0;
 $102 = ($101|0)<(0);
 $103 = $102 << 31 >> 31;
 $104 = HEAP32[$0>>2]|0;
 $105 = $104;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = (($105) + 4)|0;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = (_i64Subtract(($107|0),($110|0),($101|0),($103|0))|0);
 $112 = tempRet0;
 $113 = $104;
 $114 = $113;
 HEAP32[$114>>2] = $111;
 $115 = (($113) + 4)|0;
 $116 = $115;
 HEAP32[$116>>2] = $112;
 $117 = HEAP32[$0>>2]|0;
 $118 = ((($117)) + 40|0);
 HEAP8[$118>>0] = 0;
 $119 = HEAP32[$0>>2]|0;
 $120 = ((($119)) + 41|0);
 HEAP8[$120>>0] = 0;
 $121 = HEAP32[$0>>2]|0;
 $122 = ((($121)) + 43|0);
 HEAP8[$122>>0] = 0;
 STACKTOP = sp;return;
}
function _setDateTimeToCurrent($context,$p) {
 $context = $context|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3StmtCurrentTime($3)|0);
 $5 = tempRet0;
 $6 = HEAP32[$2>>2]|0;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = HEAP32[$2>>2]|0;
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)>(0);
 $19 = ($14>>>0)>(0);
 $20 = ($17|0)==(0);
 $21 = $20 & $19;
 $22 = $18 | $21;
 if ($22) {
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 42|0);
  HEAP8[$24>>0] = 1;
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _parseDateOrTime($context,$zDate,$p) {
 $context = $context|0;
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $r = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $zDate;
 HEAP32[$3>>2] = $p;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_parseYyyyMmDd($4,$5)|0);
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = (_parseHhMmSs($8,$9)|0);
 $11 = ($10|0)==(0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3_stricmp($12,38662)|0);
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_setDateTimeToCurrent($15,$16)|0);
  HEAP32[$0>>2] = $17;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3Strlen30($19)|0);
 $21 = (_sqlite3AtoF($18,$r,$20,1)|0);
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = +HEAPF64[$r>>3];
  $24 = $23 * 8.64E+7;
  $25 = $24 + 0.5;
  $26 = (~~$25)>>>0;
  $27 = +Math_abs($25) >= 1.0 ? $25 > 0.0 ? (~~+Math_min(+Math_floor($25 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($25 - +(~~$25 >>> 0)) / 4294967296.0) >>> 0 : 0;
  $28 = HEAP32[$3>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $27;
  $33 = HEAP32[$3>>2]|0;
  $34 = ((($33)) + 42|0);
  HEAP8[$34>>0] = 1;
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  HEAP32[$0>>2] = 1;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _parseModifier($pCtx,$zMod,$p) {
 $pCtx = $pCtx|0;
 $zMod = $zMod|0;
 $p = $p|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0.0, $154 = 0, $155 = 0.0, $156 = 0.0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0.0, $435 = 0, $436 = 0.0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0.0, $443 = 0.0, $444 = 0.0, $445 = 0.0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0.0, $467 = 0.0, $468 = 0.0, $469 = 0.0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0.0, $493 = 0.0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0.0, $515 = 0.0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0.0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0.0, $57 = 0, $570 = 0, $571 = 0, $572 = 0.0, $573 = 0.0, $574 = 0, $575 = 0.0, $576 = 0, $577 = 0.0, $578 = 0.0, $579 = 0.0, $58 = 0, $580 = 0.0, $581 = 0.0, $582 = 0.0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0.0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0.0, $616 = 0.0, $617 = 0, $618 = 0.0, $619 = 0;
 var $62 = 0, $620 = 0.0, $621 = 0.0, $622 = 0.0, $623 = 0.0, $624 = 0.0, $625 = 0.0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Z = 0, $c1 = 0, $day = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $r = 0, $rRounder = 0, $rc = 0, $tx = 0, $x = 0, $y = 0, $y1 = 0, $z = 0;
 var $z2 = 0, $zBuf = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $rc = sp + 112|0;
 $n = sp + 108|0;
 $r = sp + 80|0;
 $z = sp + 104|0;
 $zBuf = sp + 128|0;
 $c1 = sp + 72|0;
 $Z = sp + 64|0;
 $rRounder = sp + 56|0;
 $z2 = sp + 100|0;
 $tx = sp + 8|0;
 $day = sp;
 $x = sp + 96|0;
 $y = sp + 92|0;
 $y1 = sp + 88|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $zMod;
 HEAP32[$2>>2] = $p;
 HEAP32[$rc>>2] = 1;
 HEAP32[$z>>2] = $zBuf;
 HEAP32[$n>>2] = 0;
 while(1) {
  $3 = HEAP32[$n>>2]|0;
  $4 = ($3|0)<(29);
  if ($4) {
   $5 = HEAP32[$n>>2]|0;
   $6 = HEAP32[$1>>2]|0;
   $7 = (($6) + ($5)|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8 << 24 >> 24;
   $10 = ($9|0)!=(0);
   $643 = $10;
  } else {
   $643 = 0;
  }
  $11 = HEAP32[$n>>2]|0;
  if (!($643)) {
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (16974 + ($15)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = HEAP32[$n>>2]|0;
  $19 = HEAP32[$z>>2]|0;
  $20 = (($19) + ($18)|0);
  HEAP8[$20>>0] = $17;
  $21 = HEAP32[$n>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$n>>2] = $22;
 }
 $23 = HEAP32[$z>>2]|0;
 $24 = (($23) + ($11)|0);
 HEAP8[$24>>0] = 0;
 $25 = HEAP32[$z>>2]|0;
 $26 = HEAP8[$25>>0]|0;
 $27 = $26 << 24 >> 24;
 switch ($27|0) {
 case 108:  {
  $28 = HEAP32[$z>>2]|0;
  $29 = (_strcmp($28,38698)|0);
  $30 = ($29|0)==(0);
  if (!($30)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $31 = HEAP32[$2>>2]|0;
  _computeJD($31);
  $32 = HEAP32[$2>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = (_localtimeOffset($32,$33,$rc)|0);
  $35 = tempRet0;
  $36 = HEAP32[$2>>2]|0;
  $37 = $36;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),($34|0),($35|0))|0);
  $44 = tempRet0;
  $45 = $36;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  $49 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($49);
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 117:  {
  $50 = HEAP32[$z>>2]|0;
  $51 = (_strcmp($50,38708)|0);
  $52 = ($51|0)==(0);
  if ($52) {
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 42|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = ($56|0)!=(0);
   if ($57) {
    $58 = HEAP32[$2>>2]|0;
    $59 = $58;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59) + 4)|0;
    $63 = $62;
    $64 = HEAP32[$63>>2]|0;
    $65 = (_i64Add(($61|0),($64|0),43200,0)|0);
    $66 = tempRet0;
    $67 = (___divdi3(($65|0),($66|0),86400,0)|0);
    $68 = tempRet0;
    $69 = (_i64Add(($67|0),($68|0),1045635584,49096)|0);
    $70 = tempRet0;
    $71 = HEAP32[$2>>2]|0;
    $72 = $71;
    $73 = $72;
    HEAP32[$73>>2] = $69;
    $74 = (($72) + 4)|0;
    $75 = $74;
    HEAP32[$75>>2] = $70;
    $76 = HEAP32[$2>>2]|0;
    _clearYMD_HMS_TZ($76);
    HEAP32[$rc>>2] = 0;
    $642 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($642|0);
   }
  }
  $77 = HEAP32[$z>>2]|0;
  $78 = (_strcmp($77,38718)|0);
  $79 = ($78|0)==(0);
  if (!($79)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 44|0);
  $82 = HEAP8[$81>>0]|0;
  $83 = $82 << 24 >> 24;
  $84 = ($83|0)==(0);
  if (!($84)) {
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $85 = HEAP32[$2>>2]|0;
  _computeJD($85);
  $86 = HEAP32[$2>>2]|0;
  $87 = HEAP32[$0>>2]|0;
  $88 = (_localtimeOffset($86,$87,$rc)|0);
  $89 = tempRet0;
  $90 = $c1;
  $91 = $90;
  HEAP32[$91>>2] = $88;
  $92 = (($90) + 4)|0;
  $93 = $92;
  HEAP32[$93>>2] = $89;
  $94 = HEAP32[$rc>>2]|0;
  $95 = ($94|0)==(0);
  if ($95) {
   $96 = $c1;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($96) + 4)|0;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = HEAP32[$2>>2]|0;
   $103 = $102;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = (($103) + 4)|0;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (_i64Subtract(($105|0),($108|0),($98|0),($101|0))|0);
   $110 = tempRet0;
   $111 = $102;
   $112 = $111;
   HEAP32[$112>>2] = $109;
   $113 = (($111) + 4)|0;
   $114 = $113;
   HEAP32[$114>>2] = $110;
   $115 = HEAP32[$2>>2]|0;
   _clearYMD_HMS_TZ($115);
   $116 = $c1;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($116) + 4)|0;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = HEAP32[$2>>2]|0;
   $123 = HEAP32[$0>>2]|0;
   $124 = (_localtimeOffset($122,$123,$rc)|0);
   $125 = tempRet0;
   $126 = (_i64Subtract(($118|0),($121|0),($124|0),($125|0))|0);
   $127 = tempRet0;
   $128 = HEAP32[$2>>2]|0;
   $129 = $128;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) + 4)|0;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = (_i64Add(($131|0),($134|0),($126|0),($127|0))|0);
   $136 = tempRet0;
   $137 = $128;
   $138 = $137;
   HEAP32[$138>>2] = $135;
   $139 = (($137) + 4)|0;
   $140 = $139;
   HEAP32[$140>>2] = $136;
  }
  $141 = HEAP32[$2>>2]|0;
  $142 = ((($141)) + 44|0);
  HEAP8[$142>>0] = 1;
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 119:  {
  $143 = HEAP32[$z>>2]|0;
  $144 = (_strncmp($143,38722,8)|0);
  $145 = ($144|0)==(0);
  if (!($145)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $146 = HEAP32[$z>>2]|0;
  $147 = ((($146)) + 8|0);
  $148 = HEAP32[$z>>2]|0;
  $149 = ((($148)) + 8|0);
  $150 = (_sqlite3Strlen30($149)|0);
  $151 = (_sqlite3AtoF($147,$r,$150,1)|0);
  $152 = ($151|0)!=(0);
  if (!($152)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $153 = +HEAPF64[$r>>3];
  $154 = (~~(($153)));
  HEAP32[$n>>2] = $154;
  $155 = (+($154|0));
  $156 = +HEAPF64[$r>>3];
  $157 = $155 == $156;
  $158 = HEAP32[$n>>2]|0;
  $159 = ($158|0)>=(0);
  $or$cond = $157 & $159;
  $160 = +HEAPF64[$r>>3];
  $161 = $160 < 7.0;
  $or$cond3 = $or$cond & $161;
  if (!($or$cond3)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $162 = HEAP32[$2>>2]|0;
  _computeYMD_HMS($162);
  $163 = HEAP32[$2>>2]|0;
  $164 = ((($163)) + 43|0);
  HEAP8[$164>>0] = 0;
  $165 = HEAP32[$2>>2]|0;
  $166 = ((($165)) + 42|0);
  HEAP8[$166>>0] = 0;
  $167 = HEAP32[$2>>2]|0;
  _computeJD($167);
  $168 = HEAP32[$2>>2]|0;
  $169 = $168;
  $170 = $169;
  $171 = HEAP32[$170>>2]|0;
  $172 = (($169) + 4)|0;
  $173 = $172;
  $174 = HEAP32[$173>>2]|0;
  $175 = (_i64Add(($171|0),($174|0),129600000,0)|0);
  $176 = tempRet0;
  $177 = (___divdi3(($175|0),($176|0),86400000,0)|0);
  $178 = tempRet0;
  $179 = (___remdi3(($177|0),($178|0),7,0)|0);
  $180 = tempRet0;
  $181 = $Z;
  $182 = $181;
  HEAP32[$182>>2] = $179;
  $183 = (($181) + 4)|0;
  $184 = $183;
  HEAP32[$184>>2] = $180;
  $185 = $Z;
  $186 = $185;
  $187 = HEAP32[$186>>2]|0;
  $188 = (($185) + 4)|0;
  $189 = $188;
  $190 = HEAP32[$189>>2]|0;
  $191 = HEAP32[$n>>2]|0;
  $192 = ($191|0)<(0);
  $193 = $192 << 31 >> 31;
  $194 = ($190|0)>($193|0);
  $195 = ($187>>>0)>($191>>>0);
  $196 = ($190|0)==($193|0);
  $197 = $196 & $195;
  $198 = $194 | $197;
  if ($198) {
   $199 = $Z;
   $200 = $199;
   $201 = HEAP32[$200>>2]|0;
   $202 = (($199) + 4)|0;
   $203 = $202;
   $204 = HEAP32[$203>>2]|0;
   $205 = (_i64Subtract(($201|0),($204|0),7,0)|0);
   $206 = tempRet0;
   $207 = $Z;
   $208 = $207;
   HEAP32[$208>>2] = $205;
   $209 = (($207) + 4)|0;
   $210 = $209;
   HEAP32[$210>>2] = $206;
  }
  $211 = HEAP32[$n>>2]|0;
  $212 = ($211|0)<(0);
  $213 = $212 << 31 >> 31;
  $214 = $Z;
  $215 = $214;
  $216 = HEAP32[$215>>2]|0;
  $217 = (($214) + 4)|0;
  $218 = $217;
  $219 = HEAP32[$218>>2]|0;
  $220 = (_i64Subtract(($211|0),($213|0),($216|0),($219|0))|0);
  $221 = tempRet0;
  $222 = (___muldi3(($220|0),($221|0),86400000,0)|0);
  $223 = tempRet0;
  $224 = HEAP32[$2>>2]|0;
  $225 = $224;
  $226 = $225;
  $227 = HEAP32[$226>>2]|0;
  $228 = (($225) + 4)|0;
  $229 = $228;
  $230 = HEAP32[$229>>2]|0;
  $231 = (_i64Add(($227|0),($230|0),($222|0),($223|0))|0);
  $232 = tempRet0;
  $233 = $224;
  $234 = $233;
  HEAP32[$234>>2] = $231;
  $235 = (($233) + 4)|0;
  $236 = $235;
  HEAP32[$236>>2] = $232;
  $237 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($237);
  HEAP32[$rc>>2] = 0;
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 115:  {
  $238 = HEAP32[$z>>2]|0;
  $239 = (_strncmp($238,38731,9)|0);
  $240 = ($239|0)!=(0);
  if ($240) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $241 = HEAP32[$z>>2]|0;
  $242 = ((($241)) + 9|0);
  HEAP32[$z>>2] = $242;
  $243 = HEAP32[$2>>2]|0;
  _computeYMD($243);
  $244 = HEAP32[$2>>2]|0;
  $245 = ((($244)) + 41|0);
  HEAP8[$245>>0] = 1;
  $246 = HEAP32[$2>>2]|0;
  $247 = ((($246)) + 24|0);
  HEAP32[$247>>2] = 0;
  $248 = HEAP32[$2>>2]|0;
  $249 = ((($248)) + 20|0);
  HEAP32[$249>>2] = 0;
  $250 = HEAP32[$2>>2]|0;
  $251 = ((($250)) + 32|0);
  HEAPF64[$251>>3] = 0.0;
  $252 = HEAP32[$2>>2]|0;
  $253 = ((($252)) + 43|0);
  HEAP8[$253>>0] = 0;
  $254 = HEAP32[$2>>2]|0;
  $255 = ((($254)) + 42|0);
  HEAP8[$255>>0] = 0;
  $256 = HEAP32[$z>>2]|0;
  $257 = (_strcmp($256,38741)|0);
  $258 = ($257|0)==(0);
  if ($258) {
   $259 = HEAP32[$2>>2]|0;
   $260 = ((($259)) + 16|0);
   HEAP32[$260>>2] = 1;
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $261 = HEAP32[$z>>2]|0;
  $262 = (_strcmp($261,38747)|0);
  $263 = ($262|0)==(0);
  if ($263) {
   $264 = HEAP32[$2>>2]|0;
   _computeYMD($264);
   $265 = HEAP32[$2>>2]|0;
   $266 = ((($265)) + 12|0);
   HEAP32[$266>>2] = 1;
   $267 = HEAP32[$2>>2]|0;
   $268 = ((($267)) + 16|0);
   HEAP32[$268>>2] = 1;
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $269 = HEAP32[$z>>2]|0;
  $270 = (_strcmp($269,38752)|0);
  $271 = ($270|0)==(0);
  if (!($271)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  HEAP32[$rc>>2] = 0;
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 57: case 56: case 55: case 54: case 53: case 52: case 51: case 50: case 49: case 48: case 45: case 43:  {
  HEAP32[$n>>2] = 1;
  while(1) {
   $272 = HEAP32[$n>>2]|0;
   $273 = HEAP32[$z>>2]|0;
   $274 = (($273) + ($272)|0);
   $275 = HEAP8[$274>>0]|0;
   $276 = $275 << 24 >> 24;
   $277 = ($276|0)!=(0);
   if (!($277)) {
    break;
   }
   $278 = HEAP32[$n>>2]|0;
   $279 = HEAP32[$z>>2]|0;
   $280 = (($279) + ($278)|0);
   $281 = HEAP8[$280>>0]|0;
   $282 = $281 << 24 >> 24;
   $283 = ($282|0)!=(58);
   if (!($283)) {
    break;
   }
   $284 = HEAP32[$n>>2]|0;
   $285 = HEAP32[$z>>2]|0;
   $286 = (($285) + ($284)|0);
   $287 = HEAP8[$286>>0]|0;
   $288 = $287&255;
   $289 = (17533 + ($288)|0);
   $290 = HEAP8[$289>>0]|0;
   $291 = $290&255;
   $292 = $291 & 1;
   $293 = ($292|0)!=(0);
   $294 = $293 ^ 1;
   if (!($294)) {
    break;
   }
   $295 = HEAP32[$n>>2]|0;
   $296 = (($295) + 1)|0;
   HEAP32[$n>>2] = $296;
  }
  $297 = HEAP32[$z>>2]|0;
  $298 = HEAP32[$n>>2]|0;
  $299 = (_sqlite3AtoF($297,$r,$298,1)|0);
  $300 = ($299|0)!=(0);
  if (!($300)) {
   HEAP32[$rc>>2] = 1;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $301 = HEAP32[$n>>2]|0;
  $302 = HEAP32[$z>>2]|0;
  $303 = (($302) + ($301)|0);
  $304 = HEAP8[$303>>0]|0;
  $305 = $304 << 24 >> 24;
  $306 = ($305|0)==(58);
  if ($306) {
   $307 = HEAP32[$z>>2]|0;
   HEAP32[$z2>>2] = $307;
   $308 = HEAP32[$z2>>2]|0;
   $309 = HEAP8[$308>>0]|0;
   $310 = $309&255;
   $311 = (17533 + ($310)|0);
   $312 = HEAP8[$311>>0]|0;
   $313 = $312&255;
   $314 = $313 & 4;
   $315 = ($314|0)!=(0);
   if (!($315)) {
    $316 = HEAP32[$z2>>2]|0;
    $317 = ((($316)) + 1|0);
    HEAP32[$z2>>2] = $317;
   }
   dest=$tx; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $318 = HEAP32[$z2>>2]|0;
   $319 = (_parseHhMmSs($318,$tx)|0);
   $320 = ($319|0)!=(0);
   if ($320) {
    $642 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($642|0);
   }
   _computeJD($tx);
   $321 = $tx;
   $322 = $321;
   $323 = HEAP32[$322>>2]|0;
   $324 = (($321) + 4)|0;
   $325 = $324;
   $326 = HEAP32[$325>>2]|0;
   $327 = (_i64Subtract(($323|0),($326|0),43200000,0)|0);
   $328 = tempRet0;
   $329 = $tx;
   $330 = $329;
   HEAP32[$330>>2] = $327;
   $331 = (($329) + 4)|0;
   $332 = $331;
   HEAP32[$332>>2] = $328;
   $333 = $tx;
   $334 = $333;
   $335 = HEAP32[$334>>2]|0;
   $336 = (($333) + 4)|0;
   $337 = $336;
   $338 = HEAP32[$337>>2]|0;
   $339 = (___divdi3(($335|0),($338|0),86400000,0)|0);
   $340 = tempRet0;
   $341 = $day;
   $342 = $341;
   HEAP32[$342>>2] = $339;
   $343 = (($341) + 4)|0;
   $344 = $343;
   HEAP32[$344>>2] = $340;
   $345 = $day;
   $346 = $345;
   $347 = HEAP32[$346>>2]|0;
   $348 = (($345) + 4)|0;
   $349 = $348;
   $350 = HEAP32[$349>>2]|0;
   $351 = (___muldi3(($347|0),($350|0),86400000,0)|0);
   $352 = tempRet0;
   $353 = $tx;
   $354 = $353;
   $355 = HEAP32[$354>>2]|0;
   $356 = (($353) + 4)|0;
   $357 = $356;
   $358 = HEAP32[$357>>2]|0;
   $359 = (_i64Subtract(($355|0),($358|0),($351|0),($352|0))|0);
   $360 = tempRet0;
   $361 = $tx;
   $362 = $361;
   HEAP32[$362>>2] = $359;
   $363 = (($361) + 4)|0;
   $364 = $363;
   HEAP32[$364>>2] = $360;
   $365 = HEAP32[$z>>2]|0;
   $366 = HEAP8[$365>>0]|0;
   $367 = $366 << 24 >> 24;
   $368 = ($367|0)==(45);
   if ($368) {
    $369 = $tx;
    $370 = $369;
    $371 = HEAP32[$370>>2]|0;
    $372 = (($369) + 4)|0;
    $373 = $372;
    $374 = HEAP32[$373>>2]|0;
    $375 = (_i64Subtract(0,0,($371|0),($374|0))|0);
    $376 = tempRet0;
    $377 = $tx;
    $378 = $377;
    HEAP32[$378>>2] = $375;
    $379 = (($377) + 4)|0;
    $380 = $379;
    HEAP32[$380>>2] = $376;
   }
   $381 = HEAP32[$2>>2]|0;
   _computeJD($381);
   $382 = HEAP32[$2>>2]|0;
   _clearYMD_HMS_TZ($382);
   $383 = $tx;
   $384 = $383;
   $385 = HEAP32[$384>>2]|0;
   $386 = (($383) + 4)|0;
   $387 = $386;
   $388 = HEAP32[$387>>2]|0;
   $389 = HEAP32[$2>>2]|0;
   $390 = $389;
   $391 = $390;
   $392 = HEAP32[$391>>2]|0;
   $393 = (($390) + 4)|0;
   $394 = $393;
   $395 = HEAP32[$394>>2]|0;
   $396 = (_i64Add(($392|0),($395|0),($385|0),($388|0))|0);
   $397 = tempRet0;
   $398 = $389;
   $399 = $398;
   HEAP32[$399>>2] = $396;
   $400 = (($398) + 4)|0;
   $401 = $400;
   HEAP32[$401>>2] = $397;
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $402 = HEAP32[$n>>2]|0;
  $403 = HEAP32[$z>>2]|0;
  $404 = (($403) + ($402)|0);
  HEAP32[$z>>2] = $404;
  while(1) {
   $405 = HEAP32[$z>>2]|0;
   $406 = HEAP8[$405>>0]|0;
   $407 = $406&255;
   $408 = (17533 + ($407)|0);
   $409 = HEAP8[$408>>0]|0;
   $410 = $409&255;
   $411 = $410 & 1;
   $412 = ($411|0)!=(0);
   $413 = HEAP32[$z>>2]|0;
   if (!($412)) {
    break;
   }
   $414 = ((($413)) + 1|0);
   HEAP32[$z>>2] = $414;
  }
  $415 = (_sqlite3Strlen30($413)|0);
  HEAP32[$n>>2] = $415;
  $416 = HEAP32[$n>>2]|0;
  $417 = ($416|0)>(10);
  $418 = HEAP32[$n>>2]|0;
  $419 = ($418|0)<(3);
  $or$cond6 = $417 | $419;
  if ($or$cond6) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $420 = HEAP32[$n>>2]|0;
  $421 = (($420) - 1)|0;
  $422 = HEAP32[$z>>2]|0;
  $423 = (($422) + ($421)|0);
  $424 = HEAP8[$423>>0]|0;
  $425 = $424 << 24 >> 24;
  $426 = ($425|0)==(115);
  if ($426) {
   $427 = HEAP32[$n>>2]|0;
   $428 = (($427) - 1)|0;
   $429 = HEAP32[$z>>2]|0;
   $430 = (($429) + ($428)|0);
   HEAP8[$430>>0] = 0;
   $431 = HEAP32[$n>>2]|0;
   $432 = (($431) + -1)|0;
   HEAP32[$n>>2] = $432;
  }
  $433 = HEAP32[$2>>2]|0;
  _computeJD($433);
  HEAP32[$rc>>2] = 0;
  $434 = +HEAPF64[$r>>3];
  $435 = $434 < 0.0;
  $436 = $435 ? -0.5 : 0.5;
  HEAPF64[$rRounder>>3] = $436;
  $437 = HEAP32[$n>>2]|0;
  $438 = ($437|0)==(3);
  if ($438) {
   $439 = HEAP32[$z>>2]|0;
   $440 = (_strcmp($439,38752)|0);
   $441 = ($440|0)==(0);
   if ($441) {
    $442 = +HEAPF64[$r>>3];
    $443 = $442 * 8.64E+7;
    $444 = +HEAPF64[$rRounder>>3];
    $445 = $443 + $444;
    $446 = (~~$445)>>>0;
    $447 = +Math_abs($445) >= 1.0 ? $445 > 0.0 ? (~~+Math_min(+Math_floor($445 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($445 - +(~~$445 >>> 0)) / 4294967296.0) >>> 0 : 0;
    $448 = HEAP32[$2>>2]|0;
    $449 = $448;
    $450 = $449;
    $451 = HEAP32[$450>>2]|0;
    $452 = (($449) + 4)|0;
    $453 = $452;
    $454 = HEAP32[$453>>2]|0;
    $455 = (_i64Add(($451|0),($454|0),($446|0),($447|0))|0);
    $456 = tempRet0;
    $457 = $448;
    $458 = $457;
    HEAP32[$458>>2] = $455;
    $459 = (($457) + 4)|0;
    $460 = $459;
    HEAP32[$460>>2] = $456;
   } else {
    label = 54;
   }
  } else {
   label = 54;
  }
  do {
   if ((label|0) == 54) {
    $461 = HEAP32[$n>>2]|0;
    $462 = ($461|0)==(4);
    if ($462) {
     $463 = HEAP32[$z>>2]|0;
     $464 = (_strcmp($463,38756)|0);
     $465 = ($464|0)==(0);
     if ($465) {
      $466 = +HEAPF64[$r>>3];
      $467 = $466 * 3.6E+6;
      $468 = +HEAPF64[$rRounder>>3];
      $469 = $467 + $468;
      $470 = (~~$469)>>>0;
      $471 = +Math_abs($469) >= 1.0 ? $469 > 0.0 ? (~~+Math_min(+Math_floor($469 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($469 - +(~~$469 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $472 = HEAP32[$2>>2]|0;
      $473 = $472;
      $474 = $473;
      $475 = HEAP32[$474>>2]|0;
      $476 = (($473) + 4)|0;
      $477 = $476;
      $478 = HEAP32[$477>>2]|0;
      $479 = (_i64Add(($475|0),($478|0),($470|0),($471|0))|0);
      $480 = tempRet0;
      $481 = $472;
      $482 = $481;
      HEAP32[$482>>2] = $479;
      $483 = (($481) + 4)|0;
      $484 = $483;
      HEAP32[$484>>2] = $480;
      break;
     }
    }
    $485 = HEAP32[$n>>2]|0;
    $486 = ($485|0)==(6);
    if ($486) {
     $487 = HEAP32[$z>>2]|0;
     $488 = (_strcmp($487,38761)|0);
     $489 = ($488|0)==(0);
     if ($489) {
      $490 = +HEAPF64[$r>>3];
      $491 = $490 * 6.0E+4;
      $492 = +HEAPF64[$rRounder>>3];
      $493 = $491 + $492;
      $494 = (~~$493)>>>0;
      $495 = +Math_abs($493) >= 1.0 ? $493 > 0.0 ? (~~+Math_min(+Math_floor($493 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($493 - +(~~$493 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $496 = HEAP32[$2>>2]|0;
      $497 = $496;
      $498 = $497;
      $499 = HEAP32[$498>>2]|0;
      $500 = (($497) + 4)|0;
      $501 = $500;
      $502 = HEAP32[$501>>2]|0;
      $503 = (_i64Add(($499|0),($502|0),($494|0),($495|0))|0);
      $504 = tempRet0;
      $505 = $496;
      $506 = $505;
      HEAP32[$506>>2] = $503;
      $507 = (($505) + 4)|0;
      $508 = $507;
      HEAP32[$508>>2] = $504;
      break;
     }
    }
    $509 = HEAP32[$n>>2]|0;
    $510 = ($509|0)==(6);
    if ($510) {
     $511 = HEAP32[$z>>2]|0;
     $512 = (_strcmp($511,38768)|0);
     $513 = ($512|0)==(0);
     if ($513) {
      $514 = +HEAPF64[$r>>3];
      $515 = $514 * 1000.0;
      $516 = +HEAPF64[$rRounder>>3];
      $517 = $515 + $516;
      $518 = (~~$517)>>>0;
      $519 = +Math_abs($517) >= 1.0 ? $517 > 0.0 ? (~~+Math_min(+Math_floor($517 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($517 - +(~~$517 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $520 = HEAP32[$2>>2]|0;
      $521 = $520;
      $522 = $521;
      $523 = HEAP32[$522>>2]|0;
      $524 = (($521) + 4)|0;
      $525 = $524;
      $526 = HEAP32[$525>>2]|0;
      $527 = (_i64Add(($523|0),($526|0),($518|0),($519|0))|0);
      $528 = tempRet0;
      $529 = $520;
      $530 = $529;
      HEAP32[$530>>2] = $527;
      $531 = (($529) + 4)|0;
      $532 = $531;
      HEAP32[$532>>2] = $528;
      break;
     }
    }
    $533 = HEAP32[$n>>2]|0;
    $534 = ($533|0)==(5);
    if ($534) {
     $535 = HEAP32[$z>>2]|0;
     $536 = (_strcmp($535,38741)|0);
     $537 = ($536|0)==(0);
     if ($537) {
      $538 = HEAP32[$2>>2]|0;
      _computeYMD_HMS($538);
      $539 = +HEAPF64[$r>>3];
      $540 = (~~(($539)));
      $541 = HEAP32[$2>>2]|0;
      $542 = ((($541)) + 12|0);
      $543 = HEAP32[$542>>2]|0;
      $544 = (($543) + ($540))|0;
      HEAP32[$542>>2] = $544;
      $545 = HEAP32[$2>>2]|0;
      $546 = ((($545)) + 12|0);
      $547 = HEAP32[$546>>2]|0;
      $548 = ($547|0)>(0);
      $549 = HEAP32[$2>>2]|0;
      $550 = ((($549)) + 12|0);
      $551 = HEAP32[$550>>2]|0;
      $552 = (($551) - 1)|0;
      $553 = (($551) - 12)|0;
      $$sink = $548 ? $552 : $553;
      $554 = (($$sink|0) / 12)&-1;
      HEAP32[$x>>2] = $554;
      $555 = HEAP32[$x>>2]|0;
      $556 = HEAP32[$2>>2]|0;
      $557 = ((($556)) + 8|0);
      $558 = HEAP32[$557>>2]|0;
      $559 = (($558) + ($555))|0;
      HEAP32[$557>>2] = $559;
      $560 = HEAP32[$x>>2]|0;
      $561 = ($560*12)|0;
      $562 = HEAP32[$2>>2]|0;
      $563 = ((($562)) + 12|0);
      $564 = HEAP32[$563>>2]|0;
      $565 = (($564) - ($561))|0;
      HEAP32[$563>>2] = $565;
      $566 = HEAP32[$2>>2]|0;
      $567 = ((($566)) + 42|0);
      HEAP8[$567>>0] = 0;
      $568 = HEAP32[$2>>2]|0;
      _computeJD($568);
      $569 = +HEAPF64[$r>>3];
      $570 = (~~(($569)));
      HEAP32[$y>>2] = $570;
      $571 = HEAP32[$y>>2]|0;
      $572 = (+($571|0));
      $573 = +HEAPF64[$r>>3];
      $574 = $572 != $573;
      if (!($574)) {
       break;
      }
      $575 = +HEAPF64[$r>>3];
      $576 = HEAP32[$y>>2]|0;
      $577 = (+($576|0));
      $578 = $575 - $577;
      $579 = $578 * 30.0;
      $580 = $579 * 8.64E+7;
      $581 = +HEAPF64[$rRounder>>3];
      $582 = $580 + $581;
      $583 = (~~$582)>>>0;
      $584 = +Math_abs($582) >= 1.0 ? $582 > 0.0 ? (~~+Math_min(+Math_floor($582 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($582 - +(~~$582 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $585 = HEAP32[$2>>2]|0;
      $586 = $585;
      $587 = $586;
      $588 = HEAP32[$587>>2]|0;
      $589 = (($586) + 4)|0;
      $590 = $589;
      $591 = HEAP32[$590>>2]|0;
      $592 = (_i64Add(($588|0),($591|0),($583|0),($584|0))|0);
      $593 = tempRet0;
      $594 = $585;
      $595 = $594;
      HEAP32[$595>>2] = $592;
      $596 = (($594) + 4)|0;
      $597 = $596;
      HEAP32[$597>>2] = $593;
      break;
     }
    }
    $598 = HEAP32[$n>>2]|0;
    $599 = ($598|0)==(4);
    if ($599) {
     $600 = HEAP32[$z>>2]|0;
     $601 = (_strcmp($600,38747)|0);
     $602 = ($601|0)==(0);
     if ($602) {
      $603 = +HEAPF64[$r>>3];
      $604 = (~~(($603)));
      HEAP32[$y1>>2] = $604;
      $605 = HEAP32[$2>>2]|0;
      _computeYMD_HMS($605);
      $606 = HEAP32[$y1>>2]|0;
      $607 = HEAP32[$2>>2]|0;
      $608 = ((($607)) + 8|0);
      $609 = HEAP32[$608>>2]|0;
      $610 = (($609) + ($606))|0;
      HEAP32[$608>>2] = $610;
      $611 = HEAP32[$2>>2]|0;
      $612 = ((($611)) + 42|0);
      HEAP8[$612>>0] = 0;
      $613 = HEAP32[$2>>2]|0;
      _computeJD($613);
      $614 = HEAP32[$y1>>2]|0;
      $615 = (+($614|0));
      $616 = +HEAPF64[$r>>3];
      $617 = $615 != $616;
      if (!($617)) {
       break;
      }
      $618 = +HEAPF64[$r>>3];
      $619 = HEAP32[$y1>>2]|0;
      $620 = (+($619|0));
      $621 = $618 - $620;
      $622 = $621 * 365.0;
      $623 = $622 * 8.64E+7;
      $624 = +HEAPF64[$rRounder>>3];
      $625 = $623 + $624;
      $626 = (~~$625)>>>0;
      $627 = +Math_abs($625) >= 1.0 ? $625 > 0.0 ? (~~+Math_min(+Math_floor($625 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($625 - +(~~$625 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $628 = HEAP32[$2>>2]|0;
      $629 = $628;
      $630 = $629;
      $631 = HEAP32[$630>>2]|0;
      $632 = (($629) + 4)|0;
      $633 = $632;
      $634 = HEAP32[$633>>2]|0;
      $635 = (_i64Add(($631|0),($634|0),($626|0),($627|0))|0);
      $636 = tempRet0;
      $637 = $628;
      $638 = $637;
      HEAP32[$638>>2] = $635;
      $639 = (($637) + 4)|0;
      $640 = $639;
      HEAP32[$640>>2] = $636;
      break;
     }
    }
    HEAP32[$rc>>2] = 1;
   }
  } while(0);
  $641 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($641);
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 default: {
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
 }
 }
 return (0)|0;
}
function _sqlite3StmtCurrentTime($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $piTime = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $piTime = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 136|0);
 HEAP32[$piTime>>2] = $4;
 $5 = HEAP32[$piTime>>2]|0;
 $6 = $5;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($6) + 4)|0;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)==(0);
 $13 = ($11|0)==(0);
 $14 = $12 & $13;
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$piTime>>2]|0;
  $21 = (_sqlite3OsCurrentTimeInt64($19,$20)|0);
  HEAP32[$rc>>2] = $21;
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)!=(0);
  if ($23) {
   $24 = HEAP32[$piTime>>2]|0;
   $25 = $24;
   $26 = $25;
   HEAP32[$26>>2] = 0;
   $27 = (($25) + 4)|0;
   $28 = $27;
   HEAP32[$28>>2] = 0;
  }
 }
 $29 = HEAP32[$piTime>>2]|0;
 $30 = $29;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 tempRet0 = ($35);
 STACKTOP = sp;return ($32|0);
}
function _parseYyyyMmDd($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $D = 0, $M = 0, $Y = 0, $neg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $Y = sp + 24|0;
 $M = sp + 20|0;
 $D = sp + 16|0;
 $neg = sp + 12|0;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(45);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 1|0);
  HEAP32[$1>>2] = $8;
  HEAP32[$neg>>2] = 1;
 } else {
  HEAP32[$neg>>2] = 0;
 }
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $Y;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $M;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $D;
 $10 = (_getDigits($9,38666,$vararg_buffer)|0);
 $11 = ($10|0)!=(3);
 if ($11) {
  HEAP32[$0>>2] = 1;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 10|0);
 HEAP32[$1>>2] = $13;
 while(1) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = (17533 + ($16)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = $19 & 1;
  $21 = ($20|0)!=(0);
  if ($21) {
   $60 = 1;
  } else {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   $25 = (84)==($24|0);
   $60 = $25;
  }
  $26 = HEAP32[$1>>2]|0;
  if (!($60)) {
   break;
  }
  $27 = ((($26)) + 1|0);
  HEAP32[$1>>2] = $27;
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = (_parseHhMmSs($26,$28)|0);
 $30 = ($29|0)==(0);
 do {
  if (!($30)) {
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP8[$31>>0]|0;
   $33 = $32 << 24 >> 24;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$2>>2]|0;
    $36 = ((($35)) + 41|0);
    HEAP8[$36>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 1;
   $59 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($59|0);
  }
 } while(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = ((($37)) + 42|0);
 HEAP8[$38>>0] = 0;
 $39 = HEAP32[$2>>2]|0;
 $40 = ((($39)) + 40|0);
 HEAP8[$40>>0] = 1;
 $41 = HEAP32[$neg>>2]|0;
 $42 = ($41|0)!=(0);
 $43 = HEAP32[$Y>>2]|0;
 $44 = (0 - ($43))|0;
 $45 = $42 ? $44 : $43;
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 8|0);
 HEAP32[$47>>2] = $45;
 $48 = HEAP32[$M>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = ((($49)) + 12|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$D>>2]|0;
 $52 = HEAP32[$2>>2]|0;
 $53 = ((($52)) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$2>>2]|0;
 $55 = ((($54)) + 43|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = ($56<<24>>24)!=(0);
 if ($57) {
  $58 = HEAP32[$2>>2]|0;
  _computeJD($58);
 }
 HEAP32[$0>>2] = 0;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _parseHhMmSs($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $h = 0, $m = 0, $ms = 0, $rScale = 0, $s = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 24|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $h = sp + 36|0;
 $m = sp + 32|0;
 $s = sp + 28|0;
 $ms = sp + 8|0;
 $rScale = sp;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 HEAPF64[$ms>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $h;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $m;
 $4 = (_getDigits($3,38678,$vararg_buffer)|0);
 $5 = ($4|0)!=(2);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 5|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($10|0)==(58);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$1>>2] = $13;
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $s;
  $15 = (_getDigits($14,38686,$vararg_buffer2)|0);
  $16 = ($15|0)!=(1);
  if ($16) {
   HEAP32[$0>>2] = 1;
   $85 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($85|0);
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 2|0);
  HEAP32[$1>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($21|0)==(46);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 1|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = (17533 + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = $29 & 4;
   $31 = ($30|0)!=(0);
   if ($31) {
    HEAPF64[$rScale>>3] = 1.0;
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 1|0);
    HEAP32[$1>>2] = $33;
    while(1) {
     $34 = HEAP32[$1>>2]|0;
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     $37 = (17533 + ($36)|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38&255;
     $40 = $39 & 4;
     $41 = ($40|0)!=(0);
     if (!($41)) {
      break;
     }
     $42 = +HEAPF64[$ms>>3];
     $43 = $42 * 10.0;
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP8[$44>>0]|0;
     $46 = $45 << 24 >> 24;
     $47 = (+($46|0));
     $48 = $43 + $47;
     $49 = $48 - 48.0;
     HEAPF64[$ms>>3] = $49;
     $50 = +HEAPF64[$rScale>>3];
     $51 = $50 * 10.0;
     HEAPF64[$rScale>>3] = $51;
     $52 = HEAP32[$1>>2]|0;
     $53 = ((($52)) + 1|0);
     HEAP32[$1>>2] = $53;
    }
    $54 = +HEAPF64[$rScale>>3];
    $55 = +HEAPF64[$ms>>3];
    $56 = $55 / $54;
    HEAPF64[$ms>>3] = $56;
   }
  }
 } else {
  HEAP32[$s>>2] = 0;
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = ((($57)) + 42|0);
 HEAP8[$58>>0] = 0;
 $59 = HEAP32[$2>>2]|0;
 $60 = ((($59)) + 41|0);
 HEAP8[$60>>0] = 1;
 $61 = HEAP32[$h>>2]|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = ((($62)) + 20|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$m>>2]|0;
 $65 = HEAP32[$2>>2]|0;
 $66 = ((($65)) + 24|0);
 HEAP32[$66>>2] = $64;
 $67 = HEAP32[$s>>2]|0;
 $68 = (+($67|0));
 $69 = +HEAPF64[$ms>>3];
 $70 = $68 + $69;
 $71 = HEAP32[$2>>2]|0;
 $72 = ((($71)) + 32|0);
 HEAPF64[$72>>3] = $70;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 $75 = (_parseTimezone($73,$74)|0);
 $76 = ($75|0)!=(0);
 if ($76) {
  HEAP32[$0>>2] = 1;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 } else {
  $77 = HEAP32[$2>>2]|0;
  $78 = ((($77)) + 28|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0);
  $81 = $80 ? 1 : 0;
  $82 = $81&255;
  $83 = HEAP32[$2>>2]|0;
  $84 = ((($83)) + 43|0);
  HEAP8[$84>>0] = $82;
  HEAP32[$0>>2] = 0;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 return (0)|0;
}
function _getDigits($zDate,$zFormat,$varargs) {
 $zDate = $zDate|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $N = 0, $ap = 0, $arglist_current = 0;
 var $arglist_next = 0, $cnt = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $max = 0, $min = 0, $nextC = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $cnt = sp + 4|0;
 $nextC = sp + 36|0;
 $N = sp + 35|0;
 $min = sp + 34|0;
 $val = sp;
 $max = sp + 32|0;
 HEAP32[$0>>2] = $zDate;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$cnt>>2] = 0;
 HEAP32[$ap>>2] = $varargs;
 L1: while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = HEAP8[$2>>0]|0;
  $4 = $3 << 24 >> 24;
  $5 = (($4) - 48)|0;
  $6 = $5&255;
  HEAP8[$N>>0] = $6;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 1|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = (($10) - 48)|0;
  $12 = $11&255;
  HEAP8[$min>>0] = $12;
  HEAP32[$val>>2] = 0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 2|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = (($16) - 97)|0;
  $18 = (15670 + ($17<<1)|0);
  $19 = HEAP16[$18>>1]|0;
  HEAP16[$max>>1] = $19;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 3|0);
  $22 = HEAP8[$21>>0]|0;
  HEAP8[$nextC>>0] = $22;
  HEAP32[$val>>2] = 0;
  while(1) {
   $23 = HEAP8[$N>>0]|0;
   $24 = (($23) + -1)<<24>>24;
   HEAP8[$N>>0] = $24;
   $25 = ($23<<24>>24)!=(0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$0>>2]|0;
   $27 = HEAP8[$26>>0]|0;
   $28 = $27&255;
   $29 = (17533 + ($28)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = $31 & 4;
   $33 = ($32|0)!=(0);
   if (!($33)) {
    label = 11;
    break L1;
   }
   $34 = HEAP32[$val>>2]|0;
   $35 = ($34*10)|0;
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP8[$36>>0]|0;
   $38 = $37 << 24 >> 24;
   $39 = (($35) + ($38))|0;
   $40 = (($39) - 48)|0;
   HEAP32[$val>>2] = $40;
   $41 = HEAP32[$0>>2]|0;
   $42 = ((($41)) + 1|0);
   HEAP32[$0>>2] = $42;
  }
  $43 = HEAP32[$val>>2]|0;
  $44 = HEAP8[$min>>0]|0;
  $45 = $44 << 24 >> 24;
  $46 = ($43|0)<($45|0);
  if ($46) {
   label = 11;
   break;
  }
  $47 = HEAP32[$val>>2]|0;
  $48 = HEAP16[$max>>1]|0;
  $49 = $48&65535;
  $50 = ($47|0)>($49|0);
  if ($50) {
   label = 11;
   break;
  }
  $51 = HEAP8[$nextC>>0]|0;
  $52 = $51 << 24 >> 24;
  $53 = ($52|0)!=(0);
  if ($53) {
   $54 = HEAP8[$nextC>>0]|0;
   $55 = $54 << 24 >> 24;
   $56 = HEAP32[$0>>2]|0;
   $57 = HEAP8[$56>>0]|0;
   $58 = $57 << 24 >> 24;
   $59 = ($55|0)!=($58|0);
   if ($59) {
    label = 11;
    break;
   }
  }
  $60 = HEAP32[$val>>2]|0;
  $arglist_current = HEAP32[$ap>>2]|0;
  $61 = $arglist_current;
  $62 = ((0) + 4|0);
  $expanded2 = $62;
  $expanded = (($expanded2) - 1)|0;
  $63 = (($61) + ($expanded))|0;
  $64 = ((0) + 4|0);
  $expanded6 = $64;
  $expanded5 = (($expanded6) - 1)|0;
  $expanded4 = $expanded5 ^ -1;
  $65 = $63 & $expanded4;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $arglist_next = ((($66)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next;
  HEAP32[$67>>2] = $60;
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 1|0);
  HEAP32[$0>>2] = $69;
  $70 = HEAP32[$cnt>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$cnt>>2] = $71;
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 4|0);
  HEAP32[$1>>2] = $73;
  $74 = HEAP8[$nextC>>0]|0;
  $75 = ($74<<24>>24)!=(0);
  if (!($75)) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  $76 = HEAP32[$cnt>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 return (0)|0;
}
function _parseTimezone($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $nHr = 0, $nMn = 0, $or$cond = 0, $sgn = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $sgn = sp + 20|0;
 $nHr = sp + 16|0;
 $nMn = sp + 12|0;
 $c = sp + 8|0;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 HEAP32[$sgn>>2] = 0;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4&255;
  $6 = (17533 + ($5)|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = $8 & 1;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 1|0);
  HEAP32[$1>>2] = $12;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 28|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16 << 24 >> 24;
 HEAP32[$c>>2] = $17;
 $18 = HEAP32[$c>>2]|0;
 $19 = ($18|0)==(45);
 do {
  if ($19) {
   HEAP32[$sgn>>2] = -1;
   label = 11;
  } else {
   $20 = HEAP32[$c>>2]|0;
   $21 = ($20|0)==(43);
   if ($21) {
    HEAP32[$sgn>>2] = 1;
    label = 11;
    break;
   }
   $22 = HEAP32[$c>>2]|0;
   $23 = ($22|0)==(90);
   $24 = HEAP32[$c>>2]|0;
   $25 = ($24|0)==(122);
   $or$cond = $23 | $25;
   if ($or$cond) {
    $26 = HEAP32[$1>>2]|0;
    $27 = ((($26)) + 1|0);
    HEAP32[$1>>2] = $27;
    break;
   }
   $28 = HEAP32[$c>>2]|0;
   $29 = ($28|0)!=(0);
   $30 = $29&1;
   HEAP32[$0>>2] = $30;
   $63 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } while(0);
 do {
  if ((label|0) == 11) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 1|0);
   HEAP32[$1>>2] = $32;
   $33 = HEAP32[$1>>2]|0;
   HEAP32[$vararg_buffer>>2] = $nHr;
   $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr2>>2] = $nMn;
   $34 = (_getDigits($33,38690,$vararg_buffer)|0);
   $35 = ($34|0)!=(2);
   if (!($35)) {
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 5|0);
    HEAP32[$1>>2] = $37;
    $38 = HEAP32[$sgn>>2]|0;
    $39 = HEAP32[$nMn>>2]|0;
    $40 = HEAP32[$nHr>>2]|0;
    $41 = ($40*60)|0;
    $42 = (($39) + ($41))|0;
    $43 = Math_imul($38, $42)|0;
    $44 = HEAP32[$2>>2]|0;
    $45 = ((($44)) + 28|0);
    HEAP32[$45>>2] = $43;
    break;
   }
   HEAP32[$0>>2] = 1;
   $63 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } while(0);
 while(1) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = (17533 + ($48)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 & 1;
  $53 = ($52|0)!=(0);
  if (!($53)) {
   break;
  }
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 1|0);
  HEAP32[$1>>2] = $55;
 }
 $56 = HEAP32[$2>>2]|0;
 $57 = ((($56)) + 44|0);
 HEAP8[$57>>0] = 1;
 $58 = HEAP32[$1>>2]|0;
 $59 = HEAP8[$58>>0]|0;
 $60 = $59 << 24 >> 24;
 $61 = ($60|0)!=(0);
 $62 = $61&1;
 HEAP32[$0>>2] = $62;
 $63 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _localtimeOffset($p,$pCtx,$pRc) {
 $p = $p|0;
 $pCtx = $pCtx|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $s = 0, $sLocal = 0, $t = 0, $x = 0, $y = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 96|0;
 $1 = sp + 164|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $x = sp + 48|0;
 $y = sp;
 $t = sp + 152|0;
 $sLocal = sp + 108|0;
 $s = sp + 104|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $pRc;
 dest=$sLocal; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $4 = HEAP32[$1>>2]|0;
 dest=$x; src=$4; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 _computeYMD_HMS($x);
 $5 = ((($x)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)<(1971);
 if ($7) {
  label = 3;
 } else {
  $8 = ((($x)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)>=(2038);
  if ($10) {
   label = 3;
  } else {
   $17 = ((($x)) + 32|0);
   $18 = +HEAPF64[$17>>3];
   $19 = $18 + 0.5;
   $20 = (~~(($19)));
   HEAP32[$s>>2] = $20;
   $21 = HEAP32[$s>>2]|0;
   $22 = (+($21|0));
   $23 = ((($x)) + 32|0);
   HEAPF64[$23>>3] = $22;
  }
 }
 if ((label|0) == 3) {
  $11 = ((($x)) + 8|0);
  HEAP32[$11>>2] = 2000;
  $12 = ((($x)) + 12|0);
  HEAP32[$12>>2] = 1;
  $13 = ((($x)) + 16|0);
  HEAP32[$13>>2] = 1;
  $14 = ((($x)) + 20|0);
  HEAP32[$14>>2] = 0;
  $15 = ((($x)) + 24|0);
  HEAP32[$15>>2] = 0;
  $16 = ((($x)) + 32|0);
  HEAPF64[$16>>3] = 0.0;
 }
 $24 = ((($x)) + 28|0);
 HEAP32[$24>>2] = 0;
 $25 = ((($x)) + 42|0);
 HEAP8[$25>>0] = 0;
 _computeJD($x);
 $26 = $x;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (___divdi3(($28|0),($31|0),1000,0)|0);
 $33 = tempRet0;
 $34 = (_i64Subtract(($32|0),($33|0),413362496,49)|0);
 $35 = tempRet0;
 HEAP32[$t>>2] = $34;
 $36 = (_osLocaltime($t,$sLocal)|0);
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$2>>2]|0;
  _sqlite3_result_error($38,38775,-1);
  $39 = HEAP32[$3>>2]|0;
  HEAP32[$39>>2] = 1;
  $40 = $0;
  $41 = $40;
  HEAP32[$41>>2] = 0;
  $42 = (($40) + 4)|0;
  $43 = $42;
  HEAP32[$43>>2] = 0;
  $87 = $0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  tempRet0 = ($92);
  STACKTOP = sp;return ($89|0);
 } else {
  $44 = ((($sLocal)) + 20|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1900)|0;
  $47 = ((($y)) + 8|0);
  HEAP32[$47>>2] = $46;
  $48 = ((($sLocal)) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 1)|0;
  $51 = ((($y)) + 12|0);
  HEAP32[$51>>2] = $50;
  $52 = ((($sLocal)) + 12|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ((($y)) + 16|0);
  HEAP32[$54>>2] = $53;
  $55 = ((($sLocal)) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ((($y)) + 20|0);
  HEAP32[$57>>2] = $56;
  $58 = ((($sLocal)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ((($y)) + 24|0);
  HEAP32[$60>>2] = $59;
  $61 = HEAP32[$sLocal>>2]|0;
  $62 = (+($61|0));
  $63 = ((($y)) + 32|0);
  HEAPF64[$63>>3] = $62;
  $64 = ((($y)) + 40|0);
  HEAP8[$64>>0] = 1;
  $65 = ((($y)) + 41|0);
  HEAP8[$65>>0] = 1;
  $66 = ((($y)) + 42|0);
  HEAP8[$66>>0] = 0;
  $67 = ((($y)) + 43|0);
  HEAP8[$67>>0] = 0;
  _computeJD($y);
  $68 = HEAP32[$3>>2]|0;
  HEAP32[$68>>2] = 0;
  $69 = $y;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = $x;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (_i64Subtract(($71|0),($74|0),($77|0),($80|0))|0);
  $82 = tempRet0;
  $83 = $0;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
  $87 = $0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  tempRet0 = ($92);
  STACKTOP = sp;return ($89|0);
 }
 return (0)|0;
}
function _clearYMD_HMS_TZ($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 41|0);
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 43|0);
 HEAP8[$6>>0] = 0;
 STACKTOP = sp;return;
}
function _computeYMD_HMS($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _computeYMD($1);
 $2 = HEAP32[$0>>2]|0;
 _computeHMS($2);
 STACKTOP = sp;return;
}
function _computeYMD($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $A = 0, $B = 0, $C = 0, $D = 0, $E = 0, $X1 = 0, $Z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $Z = sp + 24|0;
 $A = sp + 20|0;
 $B = sp + 16|0;
 $C = sp + 12|0;
 $D = sp + 8|0;
 $E = sp + 4|0;
 $X1 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 42|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $15 = $9;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_i64Add(($17|0),($20|0),43200000,0)|0);
  $22 = tempRet0;
  $23 = (___divdi3(($21|0),($22|0),86400000,0)|0);
  $24 = tempRet0;
  HEAP32[$Z>>2] = $23;
  $25 = HEAP32[$Z>>2]|0;
  $26 = (+($25|0));
  $27 = $26 - 1867216.25;
  $28 = $27 / 36524.25;
  $29 = (~~(($28)));
  HEAP32[$A>>2] = $29;
  $30 = HEAP32[$Z>>2]|0;
  $31 = (($30) + 1)|0;
  $32 = HEAP32[$A>>2]|0;
  $33 = (($31) + ($32))|0;
  $34 = HEAP32[$A>>2]|0;
  $35 = (($34|0) / 4)&-1;
  $36 = (($33) - ($35))|0;
  HEAP32[$A>>2] = $36;
  $37 = HEAP32[$A>>2]|0;
  $38 = (($37) + 1524)|0;
  HEAP32[$B>>2] = $38;
  $39 = HEAP32[$B>>2]|0;
  $40 = (+($39|0));
  $41 = $40 - 122.09999999999999;
  $42 = $41 / 365.25;
  $43 = (~~(($42)));
  HEAP32[$C>>2] = $43;
  $44 = HEAP32[$C>>2]|0;
  $45 = $44 & 32767;
  $46 = ($45*36525)|0;
  $47 = (($46|0) / 100)&-1;
  HEAP32[$D>>2] = $47;
  $48 = HEAP32[$B>>2]|0;
  $49 = HEAP32[$D>>2]|0;
  $50 = (($48) - ($49))|0;
  $51 = (+($50|0));
  $52 = $51 / 30.600100000000001;
  $53 = (~~(($52)));
  HEAP32[$E>>2] = $53;
  $54 = HEAP32[$E>>2]|0;
  $55 = (+($54|0));
  $56 = 30.600100000000001 * $55;
  $57 = (~~(($56)));
  HEAP32[$X1>>2] = $57;
  $58 = HEAP32[$B>>2]|0;
  $59 = HEAP32[$D>>2]|0;
  $60 = (($58) - ($59))|0;
  $61 = HEAP32[$X1>>2]|0;
  $62 = (($60) - ($61))|0;
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 16|0);
  HEAP32[$64>>2] = $62;
  $65 = HEAP32[$E>>2]|0;
  $66 = ($65|0)<(14);
  $67 = HEAP32[$E>>2]|0;
  $68 = (($67) - 1)|0;
  $69 = (($67) - 13)|0;
  $70 = $66 ? $68 : $69;
  $71 = HEAP32[$0>>2]|0;
  $72 = ((($71)) + 12|0);
  HEAP32[$72>>2] = $70;
  $73 = HEAP32[$0>>2]|0;
  $74 = ((($73)) + 12|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = ($75|0)>(2);
  $77 = HEAP32[$C>>2]|0;
  $78 = (($77) - 4716)|0;
  $79 = (($77) - 4715)|0;
  $80 = $76 ? $78 : $79;
  $81 = HEAP32[$0>>2]|0;
  $82 = ((($81)) + 8|0);
  HEAP32[$82>>2] = $80;
 } else {
  $10 = ((($9)) + 8|0);
  HEAP32[$10>>2] = 2000;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 12|0);
  HEAP32[$12>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 16|0);
  HEAP32[$14>>2] = 1;
 }
 $83 = HEAP32[$0>>2]|0;
 $84 = ((($83)) + 40|0);
 HEAP8[$84>>0] = 1;
 STACKTOP = sp;return;
}
function _osLocaltime($t,$pTm) {
 $t = $t|0;
 $pTm = $pTm|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pX = 0, $rc = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pX = sp;
 HEAP32[$0>>2] = $t;
 HEAP32[$1>>2] = $pTm;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_localtime(($2|0))|0);
 HEAP32[$pX>>2] = $3;
 $4 = HEAP32[(276)>>2]|0;
 $5 = ($4|0)!=(0);
 $$ = $5 ? 0 : $3;
 HEAP32[$pX>>2] = $$;
 $6 = HEAP32[$pX>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$pX>>2]|0;
  dest=$8; src=$9; stop=dest+44|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 }
 $10 = HEAP32[$pX>>2]|0;
 $11 = ($10|0)==(0|0);
 $12 = $11&1;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _computeHMS($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $s = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $s = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 41|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _computeJD($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_i64Add(($9|0),($12|0),43200000,0)|0);
 $14 = tempRet0;
 $15 = (___remdi3(($13|0),($14|0),86400000,0)|0);
 $16 = tempRet0;
 HEAP32[$s>>2] = $15;
 $17 = HEAP32[$s>>2]|0;
 $18 = (+($17|0));
 $19 = $18 / 1000.0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 32|0);
 HEAPF64[$21>>3] = $19;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 32|0);
 $24 = +HEAPF64[$23>>3];
 $25 = (~~(($24)));
 HEAP32[$s>>2] = $25;
 $26 = HEAP32[$s>>2]|0;
 $27 = (+($26|0));
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 32|0);
 $30 = +HEAPF64[$29>>3];
 $31 = $30 - $27;
 HEAPF64[$29>>3] = $31;
 $32 = HEAP32[$s>>2]|0;
 $33 = (($32|0) / 3600)&-1;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 20|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 20|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38*3600)|0;
 $40 = HEAP32[$s>>2]|0;
 $41 = (($40) - ($39))|0;
 HEAP32[$s>>2] = $41;
 $42 = HEAP32[$s>>2]|0;
 $43 = (($42|0) / 60)&-1;
 $44 = HEAP32[$0>>2]|0;
 $45 = ((($44)) + 24|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$s>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 24|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($49*60)|0;
 $51 = (($46) - ($50))|0;
 $52 = (+($51|0));
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 32|0);
 $55 = +HEAPF64[$54>>3];
 $56 = $55 + $52;
 HEAPF64[$54>>3] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 41|0);
 HEAP8[$58>>0] = 1;
 STACKTOP = sp;return;
}
function _renameTableFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $len = 0, $tname = 0, $token = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zCsr = 0, $zRet = 0, $zSql = 0, $zTableName = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $zSql = sp + 48|0;
 $zTableName = sp + 44|0;
 $token = sp + 40|0;
 $tname = sp + 32|0;
 $zCsr = sp + 28|0;
 $len = sp + 24|0;
 $zRet = sp + 20|0;
 $db = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zSql>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zTableName>>2] = $9;
 $10 = HEAP32[$zSql>>2]|0;
 HEAP32[$zCsr>>2] = $10;
 HEAP32[$len>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$zSql>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $15 = HEAP32[$zCsr>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   label = 7;
   break;
  }
  $18 = HEAP32[$zCsr>>2]|0;
  HEAP32[$tname>>2] = $18;
  $19 = HEAP32[$len>>2]|0;
  $20 = ((($tname)) + 4|0);
  HEAP32[$20>>2] = $19;
  while(1) {
   $21 = HEAP32[$len>>2]|0;
   $22 = HEAP32[$zCsr>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zCsr>>2] = $23;
   $24 = HEAP32[$zCsr>>2]|0;
   $25 = (_sqlite3GetToken($24,$token)|0);
   HEAP32[$len>>2] = $25;
   $26 = HEAP32[$token>>2]|0;
   $27 = ($26|0)==(159);
   if (!($27)) {
    break;
   }
  }
  $28 = HEAP32[$token>>2]|0;
  $29 = ($28|0)!=(22);
  $30 = HEAP32[$token>>2]|0;
  $31 = ($30|0)!=(125);
  $32 = $29 ? $31 : 0;
  if (!($32)) {
   break;
  }
 }
 if ((label|0) == 7) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$db>>2]|0;
 $34 = HEAP32[$tname>>2]|0;
 $35 = HEAP32[$zSql>>2]|0;
 $36 = $34;
 $37 = $35;
 $38 = (($36) - ($37))|0;
 $39 = HEAP32[$zSql>>2]|0;
 $40 = HEAP32[$zTableName>>2]|0;
 $41 = HEAP32[$tname>>2]|0;
 $42 = ((($tname)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + ($43)|0);
 HEAP32[$vararg_buffer>>2] = $38;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $39;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $40;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $44;
 $45 = (_sqlite3MPrintf($33,38943,$vararg_buffer)|0);
 HEAP32[$zRet>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($46,$47,-1,169);
 STACKTOP = sp;return;
}
function _renameTriggerFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $dist = 0;
 var $len = 0, $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $tname = 0, $token = 0, $vararg_buffer = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zCsr = 0, $zRet = 0, $zSql = 0, $zTableName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $zSql = sp + 56|0;
 $zTableName = sp + 52|0;
 $token = sp + 48|0;
 $tname = sp + 40|0;
 $dist = sp + 32|0;
 $zCsr = sp + 28|0;
 $len = sp + 24|0;
 $zRet = sp + 20|0;
 $db = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zSql>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zTableName>>2] = $9;
 HEAP32[$dist>>2] = 3;
 $10 = HEAP32[$zSql>>2]|0;
 HEAP32[$zCsr>>2] = $10;
 HEAP32[$len>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$zSql>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $15 = HEAP32[$zCsr>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   label = 8;
   break;
  }
  $18 = HEAP32[$zCsr>>2]|0;
  HEAP32[$tname>>2] = $18;
  $19 = HEAP32[$len>>2]|0;
  $20 = ((($tname)) + 4|0);
  HEAP32[$20>>2] = $19;
  while(1) {
   $21 = HEAP32[$len>>2]|0;
   $22 = HEAP32[$zCsr>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zCsr>>2] = $23;
   $24 = HEAP32[$zCsr>>2]|0;
   $25 = (_sqlite3GetToken($24,$token)|0);
   HEAP32[$len>>2] = $25;
   $26 = HEAP32[$token>>2]|0;
   $27 = ($26|0)==(159);
   if (!($27)) {
    break;
   }
  }
  $28 = HEAP32[$dist>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$dist>>2] = $29;
  $30 = HEAP32[$token>>2]|0;
  $31 = ($30|0)==(122);
  $32 = HEAP32[$token>>2]|0;
  $33 = ($32|0)==(107);
  $or$cond = $31 | $33;
  $$ = $or$cond ? 0 : $29;
  HEAP32[$dist>>2] = $$;
  $34 = HEAP32[$dist>>2]|0;
  $35 = ($34|0)!=(2);
  if ($35) {
   continue;
  }
  $36 = HEAP32[$token>>2]|0;
  $37 = ($36|0)!=(137);
  $38 = HEAP32[$token>>2]|0;
  $39 = ($38|0)!=(46);
  $or$cond3 = $37 & $39;
  $40 = HEAP32[$token>>2]|0;
  $41 = ($40|0)!=(5);
  $or$cond6 = $or$cond3 & $41;
  if (!($or$cond6)) {
   break;
  }
 }
 if ((label|0) == 8) {
  STACKTOP = sp;return;
 }
 $42 = HEAP32[$db>>2]|0;
 $43 = HEAP32[$tname>>2]|0;
 $44 = HEAP32[$zSql>>2]|0;
 $45 = $43;
 $46 = $44;
 $47 = (($45) - ($46))|0;
 $48 = HEAP32[$zSql>>2]|0;
 $49 = HEAP32[$zTableName>>2]|0;
 $50 = HEAP32[$tname>>2]|0;
 $51 = ((($tname)) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + ($52)|0);
 HEAP32[$vararg_buffer>>2] = $47;
 $vararg_ptr7 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr7>>2] = $48;
 $vararg_ptr8 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr8>>2] = $49;
 $vararg_ptr9 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr9>>2] = $53;
 $54 = (_sqlite3MPrintf($42,38943,$vararg_buffer)|0);
 HEAP32[$zRet>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($55,$56,-1,169);
 STACKTOP = sp;return;
}
function _renameParentFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $db = 0, $n = 0, $or$cond = 0, $token = 0, $vararg_buffer = 0, $vararg_buffer5 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr8 = 0, $z = 0, $zInput = 0, $zNew = 0, $zOld = 0, $zOut = 0;
 var $zOutput = 0, $zParent = 0, $zResult = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $db = sp + 64|0;
 $zOutput = sp + 60|0;
 $zResult = sp + 56|0;
 $zInput = sp + 52|0;
 $zOld = sp + 48|0;
 $zNew = sp + 44|0;
 $z = sp + 40|0;
 $n = sp + 36|0;
 $token = sp + 32|0;
 $zParent = sp + 28|0;
 $zOut = sp + 24|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 HEAP32[$zOutput>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zInput>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zOld>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_value_text($14)|0);
 HEAP32[$zNew>>2] = $15;
 $16 = HEAP32[$zInput>>2]|0;
 $17 = ($16|0)==(0|0);
 $18 = HEAP32[$zOld>>2]|0;
 $19 = ($18|0)==(0|0);
 $or$cond = $17 | $19;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$zInput>>2]|0;
 HEAP32[$z>>2] = $20;
 while(1) {
  $21 = HEAP32[$z>>2]|0;
  $22 = HEAP8[$21>>0]|0;
  $23 = ($22<<24>>24)!=(0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$z>>2]|0;
  $25 = (_sqlite3GetToken($24,$token)|0);
  HEAP32[$n>>2] = $25;
  $26 = HEAP32[$token>>2]|0;
  $27 = ($26|0)==(105);
  if ($27) {
   while(1) {
    $28 = HEAP32[$n>>2]|0;
    $29 = HEAP32[$z>>2]|0;
    $30 = (($29) + ($28)|0);
    HEAP32[$z>>2] = $30;
    $31 = HEAP32[$z>>2]|0;
    $32 = (_sqlite3GetToken($31,$token)|0);
    HEAP32[$n>>2] = $32;
    $33 = HEAP32[$token>>2]|0;
    $34 = ($33|0)==(159);
    if (!($34)) {
     break;
    }
   }
   $35 = HEAP32[$token>>2]|0;
   $36 = ($35|0)==(160);
   if ($36) {
    break;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = HEAP32[$z>>2]|0;
   $39 = HEAP32[$n>>2]|0;
   $40 = ($39|0)<(0);
   $41 = $40 << 31 >> 31;
   $42 = (_sqlite3DbStrNDup($37,$38,$39,$41)|0);
   HEAP32[$zParent>>2] = $42;
   $43 = HEAP32[$zParent>>2]|0;
   $44 = ($43|0)==(0|0);
   if ($44) {
    break;
   }
   $45 = HEAP32[$zParent>>2]|0;
   (_sqlite3Dequote($45)|0);
   $46 = HEAP32[$zOld>>2]|0;
   $47 = HEAP32[$zParent>>2]|0;
   $48 = (_sqlite3_stricmp($46,$47)|0);
   $49 = (0)==($48|0);
   if ($49) {
    $50 = HEAP32[$db>>2]|0;
    $51 = HEAP32[$zOutput>>2]|0;
    $52 = ($51|0)!=(0|0);
    $53 = HEAP32[$zOutput>>2]|0;
    $54 = $52 ? $53 : 17532;
    $55 = HEAP32[$z>>2]|0;
    $56 = HEAP32[$zInput>>2]|0;
    $57 = $55;
    $58 = $56;
    $59 = (($57) - ($58))|0;
    $60 = HEAP32[$zInput>>2]|0;
    $61 = HEAP32[$zNew>>2]|0;
    HEAP32[$vararg_buffer>>2] = $54;
    $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr2>>2] = $59;
    $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
    HEAP32[$vararg_ptr3>>2] = $60;
    $vararg_ptr4 = ((($vararg_buffer)) + 12|0);
    HEAP32[$vararg_ptr4>>2] = $61;
    $62 = (_sqlite3MPrintf($50,38954,$vararg_buffer)|0);
    HEAP32[$zOut>>2] = $62;
    $63 = HEAP32[$db>>2]|0;
    $64 = HEAP32[$zOutput>>2]|0;
    _sqlite3DbFree($63,$64);
    $65 = HEAP32[$zOut>>2]|0;
    HEAP32[$zOutput>>2] = $65;
    $66 = HEAP32[$n>>2]|0;
    $67 = HEAP32[$z>>2]|0;
    $68 = (($67) + ($66)|0);
    HEAP32[$zInput>>2] = $68;
   }
   $69 = HEAP32[$db>>2]|0;
   $70 = HEAP32[$zParent>>2]|0;
   _sqlite3DbFree($69,$70);
  }
  $71 = HEAP32[$z>>2]|0;
  $72 = HEAP32[$n>>2]|0;
  $73 = (($71) + ($72)|0);
  HEAP32[$z>>2] = $73;
 }
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$zOutput>>2]|0;
 $76 = ($75|0)!=(0|0);
 $77 = HEAP32[$zOutput>>2]|0;
 $78 = $76 ? $77 : 17532;
 $79 = HEAP32[$zInput>>2]|0;
 HEAP32[$vararg_buffer5>>2] = $78;
 $vararg_ptr8 = ((($vararg_buffer5)) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $79;
 $80 = (_sqlite3MPrintf($74,28532,$vararg_buffer5)|0);
 HEAP32[$zResult>>2] = $80;
 $81 = HEAP32[$0>>2]|0;
 $82 = HEAP32[$zResult>>2]|0;
 _sqlite3_result_text($81,$82,-1,169);
 $83 = HEAP32[$db>>2]|0;
 $84 = HEAP32[$zOutput>>2]|0;
 _sqlite3DbFree($83,$84);
 STACKTOP = sp;return;
}
function _sqlite3MemMalloc($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$0>>2] = $nByte;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7)|0;
 $3 = $2 & -8;
 HEAP32[$0>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8)|0;
 $6 = (_malloc($5)|0);
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ($9|0)<(0);
  $11 = $10 << 31 >> 31;
  $12 = HEAP32[$p>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $9;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 8|0);
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  HEAP32[$vararg_buffer>>2] = $9;
  _sqlite3_log(7,38965,$vararg_buffer);
  $19 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3MemFree($pPrior) {
 $pPrior = $pPrior|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPrior;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + -8|0);
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 _free($4);
 STACKTOP = sp;return;
}
function _sqlite3MemRealloc($pPrior,$nByte) {
 $pPrior = $pPrior|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pPrior;
 HEAP32[$1>>2] = $nByte;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + -8|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8)|0;
 $8 = (_realloc($5,$7)|0);
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = HEAP32[$p>>2]|0;
  $15 = $14;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = $13;
  $19 = HEAP32[$p>>2]|0;
  $20 = ((($19)) + 8|0);
  HEAP32[$p>>2] = $20;
  $24 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3MemSize($21)|0);
  $23 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $23;
  _sqlite3_log(7,39003,$vararg_buffer);
  $24 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3MemSize($pPrior) {
 $pPrior = $pPrior|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPrior;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + -8|0);
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3MemRoundup($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7)|0;
 $3 = $2 & -8;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3MemInit($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return 0;
}
function _sqlite3MemShutdown($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return;
}
function _pcache1Init($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 dest=1584; stop=dest+100|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $1 = HEAP32[(212)>>2]|0;
 $2 = ($1|0)==(0|0);
 $3 = $2&1;
 HEAP32[(1640)>>2] = $3;
 $4 = HEAP32[(1640)>>2]|0;
 $5 = ($4|0)!=(0);
 $6 = HEAP32[(220)>>2]|0;
 $7 = ($6|0)!=(0);
 $or$cond = $5 & $7;
 $8 = HEAP32[(212)>>2]|0;
 $9 = ($8|0)==(0|0);
 $or$cond3 = $or$cond & $9;
 if ($or$cond3) {
  $10 = HEAP32[(220)>>2]|0;
  HEAP32[(1644)>>2] = $10;
  HEAP32[(1596)>>2] = 10;
  HEAP32[(1636)>>2] = 1;
  STACKTOP = sp;return 0;
 } else {
  HEAP32[(1644)>>2] = 0;
  HEAP32[(1596)>>2] = 10;
  HEAP32[(1636)>>2] = 1;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _pcache1Shutdown($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 dest=1584; stop=dest+100|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _pcache1Create($szPage,$szExtra,$bPurgeable) {
 $szPage = $szPage|0;
 $szExtra = $szExtra|0;
 $bPurgeable = $bPurgeable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pCache = sp + 8|0;
 $pGroup = sp + 4|0;
 $sz = sp;
 HEAP32[$0>>2] = $szPage;
 HEAP32[$1>>2] = $szExtra;
 HEAP32[$2>>2] = $bPurgeable;
 $3 = HEAP32[(1640)>>2]|0;
 $4 = ($3*52)|0;
 $5 = (60 + ($4))|0;
 HEAP32[$sz>>2] = $5;
 $6 = HEAP32[$sz>>2]|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_sqlite3MallocZero($6,$8)|0);
 HEAP32[$pCache>>2] = $9;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $81 = HEAP32[$pCache>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $12 = HEAP32[(1640)>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$pCache>>2]|0;
  $15 = ((($14)) + 60|0);
  HEAP32[$pGroup>>2] = $15;
  $16 = HEAP32[$pGroup>>2]|0;
  $17 = ((($16)) + 12|0);
  HEAP32[$17>>2] = 10;
 } else {
  HEAP32[$pGroup>>2] = 1584;
 }
 $18 = HEAP32[$pGroup>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = ((($19)) + 14|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$pGroup>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = ((($25)) + 14|0);
  HEAP8[$26>>0] = 1;
  $27 = HEAP32[$pGroup>>2]|0;
  $28 = ((($27)) + 20|0);
  $29 = HEAP32[$pGroup>>2]|0;
  $30 = ((($29)) + 20|0);
  $31 = ((($30)) + 24|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$pGroup>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = ((($33)) + 28|0);
  HEAP32[$34>>2] = $28;
 }
 $35 = HEAP32[$pGroup>>2]|0;
 $36 = HEAP32[$pCache>>2]|0;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$0>>2]|0;
 $38 = HEAP32[$pCache>>2]|0;
 $39 = ((($38)) + 4|0);
 HEAP32[$39>>2] = $37;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$pCache>>2]|0;
 $42 = ((($41)) + 8|0);
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = (($43) + ($44))|0;
 $46 = (($45) + 32)|0;
 $47 = HEAP32[$pCache>>2]|0;
 $48 = ((($47)) + 12|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP32[$2>>2]|0;
 $50 = ($49|0)!=(0);
 $51 = $50 ? 1 : 0;
 $52 = HEAP32[$pCache>>2]|0;
 $53 = ((($52)) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$pCache>>2]|0;
 _pcache1ResizeHash($54);
 $55 = HEAP32[$2>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$pCache>>2]|0;
  $58 = ((($57)) + 20|0);
  HEAP32[$58>>2] = 10;
  $59 = HEAP32[$pCache>>2]|0;
  $60 = ((($59)) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$pGroup>>2]|0;
  $63 = ((($62)) + 8|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($64) + ($61))|0;
  HEAP32[$63>>2] = $65;
  $66 = HEAP32[$pGroup>>2]|0;
  $67 = ((($66)) + 4|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + 10)|0;
  $70 = HEAP32[$pGroup>>2]|0;
  $71 = ((($70)) + 8|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($69) - ($72))|0;
  $74 = HEAP32[$pGroup>>2]|0;
  $75 = ((($74)) + 12|0);
  HEAP32[$75>>2] = $73;
 }
 $76 = HEAP32[$pCache>>2]|0;
 $77 = ((($76)) + 44|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = ($78|0)==(0);
 if (!($79)) {
  $81 = HEAP32[$pCache>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $80 = HEAP32[$pCache>>2]|0;
 _pcache1Destroy($80);
 HEAP32[$pCache>>2] = 0;
 $81 = HEAP32[$pCache>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _pcache1Cachesize($p,$nMax) {
 $p = $p|0;
 $nMax = $nMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0;
 var $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nMax;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $2;
 $3 = HEAP32[$pCache>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pCache>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pGroup>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($9) - ($12))|0;
 $14 = HEAP32[$pGroup>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13))|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$pGroup>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 10)|0;
 $22 = HEAP32[$pGroup>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($21) - ($24))|0;
 $26 = HEAP32[$pGroup>>2]|0;
 $27 = ((($26)) + 12|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$pCache>>2]|0;
 $30 = ((($29)) + 24|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pCache>>2]|0;
 $32 = ((($31)) + 24|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33*9)|0;
 $35 = (($34>>>0) / 10)&-1;
 $36 = HEAP32[$pCache>>2]|0;
 $37 = ((($36)) + 28|0);
 HEAP32[$37>>2] = $35;
 $38 = HEAP32[$pCache>>2]|0;
 _pcache1EnforceMaxPage($38);
 STACKTOP = sp;return;
}
function _pcache1Pagecount($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $n = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $n = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$n>>2] = $4;
 $5 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _pcache1Fetch($p,$iKey,$createFlag) {
 $p = $p|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iKey;
 HEAP32[$2>>2] = $createFlag;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_pcache1FetchNoMutex($3,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _pcache1Unpin($p,$pPg,$reuseUnlikely) {
 $p = $p|0;
 $pPg = $pPg|0;
 $reuseUnlikely = $reuseUnlikely|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0;
 var $pPage = 0, $ppFirst = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pCache = sp + 12|0;
 $pPage = sp + 8|0;
 $pGroup = sp + 4|0;
 $ppFirst = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $reuseUnlikely;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pPage>>2] = $4;
 $5 = HEAP32[$pCache>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pGroup>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = HEAP32[$pGroup>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$pGroup>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11>>>0)>($14>>>0);
  if (!($15)) {
   $17 = HEAP32[$pGroup>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = ((($18)) + 24|0);
   HEAP32[$ppFirst>>2] = $19;
   $20 = HEAP32[$pGroup>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$pPage>>2]|0;
   $23 = ((($22)) + 28|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$pPage>>2]|0;
   $25 = HEAP32[$ppFirst>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pPage>>2]|0;
   $28 = ((($27)) + 24|0);
   HEAP32[$28>>2] = $26;
   $29 = ((($26)) + 28|0);
   HEAP32[$29>>2] = $24;
   $30 = HEAP32[$pPage>>2]|0;
   $31 = HEAP32[$ppFirst>>2]|0;
   HEAP32[$31>>2] = $30;
   $32 = HEAP32[$pCache>>2]|0;
   $33 = ((($32)) + 36|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$33>>2] = $35;
   $36 = HEAP32[$pPage>>2]|0;
   $37 = ((($36)) + 12|0);
   HEAP8[$37>>0] = 0;
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$pPage>>2]|0;
 _pcache1RemoveFromHash($16,1);
 STACKTOP = sp;return;
}
function _pcache1Rekey($p,$pPg,$iOld,$iNew) {
 $p = $p|0;
 $pPg = $pPg|0;
 $iOld = $iOld|0;
 $iNew = $iNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pCache = 0, $pPage = 0;
 var $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pCache = sp + 12|0;
 $pPage = sp + 8|0;
 $pp = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iOld;
 HEAP32[$3>>2] = $iNew;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pPage>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$pCache>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) % ($9>>>0))&-1;
 HEAP32[$h>>2] = $10;
 $11 = HEAP32[$h>>2]|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<2)|0);
 HEAP32[$pp>>2] = $15;
 while(1) {
  $16 = HEAP32[$pp>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pPage>>2]|0;
  $19 = ($17|0)!=($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$pp>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 16|0);
  HEAP32[$pp>>2] = $22;
 }
 $23 = HEAP32[$pPage>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$pp>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$pCache>>2]|0;
 $29 = ((($28)) + 44|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($27>>>0) % ($30>>>0))&-1;
 HEAP32[$h>>2] = $31;
 $32 = HEAP32[$3>>2]|0;
 $33 = HEAP32[$pPage>>2]|0;
 $34 = ((($33)) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$h>>2]|0;
 $36 = HEAP32[$pCache>>2]|0;
 $37 = ((($36)) + 48|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35<<2)|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pPage>>2]|0;
 $42 = ((($41)) + 16|0);
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$pPage>>2]|0;
 $44 = HEAP32[$h>>2]|0;
 $45 = HEAP32[$pCache>>2]|0;
 $46 = ((($45)) + 48|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44<<2)|0);
 HEAP32[$48>>2] = $43;
 $49 = HEAP32[$3>>2]|0;
 $50 = HEAP32[$pCache>>2]|0;
 $51 = ((($50)) + 32|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($49>>>0)>($52>>>0);
 if (!($53)) {
  STACKTOP = sp;return;
 }
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$pCache>>2]|0;
 $56 = ((($55)) + 32|0);
 HEAP32[$56>>2] = $54;
 STACKTOP = sp;return;
}
function _pcache1Truncate($p,$iLimit) {
 $p = $p|0;
 $iLimit = $iLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLimit;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$pCache>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3>>>0)<=($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pCache>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _pcache1TruncateUnsafe($8,$9);
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = ((($12)) + 32|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _pcache1Destroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pGroup>>2] = $3;
 $4 = HEAP32[$pCache>>2]|0;
 _pcache1TruncateUnsafe($4,0);
 $5 = HEAP32[$pCache>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$pGroup>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) - ($7))|0;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = ((($12)) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$pGroup>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) - ($14))|0;
 HEAP32[$16>>2] = $18;
 $19 = HEAP32[$pGroup>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 10)|0;
 $23 = HEAP32[$pGroup>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($22) - ($25))|0;
 $27 = HEAP32[$pGroup>>2]|0;
 $28 = ((($27)) + 12|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pCache>>2]|0;
 _pcache1EnforceMaxPage($29);
 $30 = HEAP32[$pCache>>2]|0;
 $31 = ((($30)) + 56|0);
 $32 = HEAP32[$31>>2]|0;
 _sqlite3_free($32);
 $33 = HEAP32[$pCache>>2]|0;
 $34 = ((($33)) + 48|0);
 $35 = HEAP32[$34>>2]|0;
 _sqlite3_free($35);
 $36 = HEAP32[$pCache>>2]|0;
 _sqlite3_free($36);
 STACKTOP = sp;return;
}
function _pcache1Shrink($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $savedMaxPage = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pCache = sp + 8|0;
 $pGroup = sp + 4|0;
 $savedMaxPage = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pCache>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pGroup>>2] = $7;
 $8 = HEAP32[$pGroup>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$savedMaxPage>>2] = $10;
 $11 = HEAP32[$pGroup>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$pCache>>2]|0;
 _pcache1EnforceMaxPage($13);
 $14 = HEAP32[$savedMaxPage>>2]|0;
 $15 = HEAP32[$pGroup>>2]|0;
 $16 = ((($15)) + 4|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _pcache1ResizeHash($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $apNew = 0, $h = 0, $i = 0, $nNew = 0, $pNext = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $apNew = sp + 20|0;
 $nNew = sp + 16|0;
 $i = sp + 12|0;
 $pPage = sp + 8|0;
 $pNext = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<1;
 HEAP32[$nNew>>2] = $4;
 $5 = HEAP32[$nNew>>2]|0;
 $6 = ($5>>>0)<(256);
 $$ = $6 ? 256 : $4;
 HEAP32[$nNew>>2] = $$;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  _sqlite3BeginBenignMalloc();
 }
 $11 = HEAP32[$nNew>>2]|0;
 $12 = $11<<2;
 $13 = (_sqlite3MallocZero($12,0)|0);
 HEAP32[$apNew>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 44|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  _sqlite3EndBenignMalloc();
 }
 $18 = HEAP32[$apNew>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = ((($21)) + 44|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20>>>0)<($23>>>0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pNext>>2] = $30;
  while(1) {
   $31 = HEAP32[$pNext>>2]|0;
   HEAP32[$pPage>>2] = $31;
   $32 = ($31|0)!=(0|0);
   if (!($32)) {
    break;
   }
   $33 = HEAP32[$pPage>>2]|0;
   $34 = ((($33)) + 8|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$nNew>>2]|0;
   $37 = (($35>>>0) % ($36>>>0))&-1;
   HEAP32[$h>>2] = $37;
   $38 = HEAP32[$pPage>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$pNext>>2] = $40;
   $41 = HEAP32[$h>>2]|0;
   $42 = HEAP32[$apNew>>2]|0;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pPage>>2]|0;
   $46 = ((($45)) + 16|0);
   HEAP32[$46>>2] = $44;
   $47 = HEAP32[$pPage>>2]|0;
   $48 = HEAP32[$h>>2]|0;
   $49 = HEAP32[$apNew>>2]|0;
   $50 = (($49) + ($48<<2)|0);
   HEAP32[$50>>2] = $47;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$i>>2] = $52;
 }
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 48|0);
 $55 = HEAP32[$54>>2]|0;
 _sqlite3_free($55);
 $56 = HEAP32[$apNew>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 48|0);
 HEAP32[$58>>2] = $56;
 $59 = HEAP32[$nNew>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 44|0);
 HEAP32[$61>>2] = $59;
 STACKTOP = sp;return;
}
function _pcache1EnforceMaxPage($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pGroup = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pGroup>>2] = $2;
 while(1) {
  $3 = HEAP32[$pGroup>>2]|0;
  $4 = ((($3)) + 16|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = HEAP32[$pGroup>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5>>>0)>($8>>>0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pGroup>>2]|0;
  $11 = ((($10)) + 20|0);
  $12 = ((($11)) + 28|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
  $14 = ((($13)) + 14|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)==(0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$p>>2]|0;
  (_pcache1PinPage($18)|0);
  $19 = HEAP32[$p>>2]|0;
  _pcache1RemoveFromHash($19,1);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 40|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 56|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($26|0)!=(0|0);
 if (!($27)) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 56|0);
 $30 = HEAP32[$29>>2]|0;
 _sqlite3_free($30);
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 52|0);
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 56|0);
 HEAP32[$34>>2] = 0;
 STACKTOP = sp;return;
}
function _pcache1PinPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 28|0);
 HEAP32[$17>>2] = $13;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 28|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 12|0);
 HEAP8[$23>>0] = 1;
 $24 = HEAP32[$pCache>>2]|0;
 $25 = ((($24)) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + -1)|0;
 HEAP32[$25>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _pcache1RemoveFromHash($pPage,$freeFlag) {
 $pPage = $pPage|0;
 $freeFlag = $freeFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pCache = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $h = sp + 8|0;
 $pCache = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $freeFlag;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$pCache>>2]|0;
 $9 = ((($8)) + 44|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7>>>0) % ($10>>>0))&-1;
 HEAP32[$h>>2] = $11;
 $12 = HEAP32[$h>>2]|0;
 $13 = HEAP32[$pCache>>2]|0;
 $14 = ((($13)) + 48|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<2)|0);
 HEAP32[$pp>>2] = $16;
 while(1) {
  $17 = HEAP32[$pp>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ($18|0)!=($19|0);
  $21 = HEAP32[$pp>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 16|0);
  if (!($20)) {
   break;
  }
  HEAP32[$pp>>2] = $23;
 }
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$pp>>2]|0;
 HEAP32[$25>>2] = $24;
 $26 = HEAP32[$pCache>>2]|0;
 $27 = ((($26)) + 40|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + -1)|0;
 HEAP32[$27>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($30|0)!=(0);
 if (!($31)) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 _pcache1FreePage($32);
 STACKTOP = sp;return;
}
function _pcache1FreePage($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 13|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$pCache>>2]|0;
  $9 = ((($8)) + 52|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 16|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$pCache>>2]|0;
  $15 = ((($14)) + 52|0);
  HEAP32[$15>>2] = $13;
 } else {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  _pcache1Free($17);
 }
 $18 = HEAP32[$pCache>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$pCache>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + -1)|0;
 HEAP32[$24>>2] = $26;
 STACKTOP = sp;return;
}
function _pcache1FetchNoMutex($p,$iKey,$createFlag) {
 $p = $p|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pCache = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pCache = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iKey;
 HEAP32[$3>>2] = $createFlag;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pCache>>2] = $4;
 HEAP32[$pPage>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$pCache>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5>>>0) % ($8>>>0))&-1;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = ((($10)) + 48|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<2)|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pPage>>2] = $14;
 while(1) {
  $15 = HEAP32[$pPage>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   $17 = HEAP32[$pPage>>2]|0;
   $18 = ((($17)) + 8|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ($19|0)!=($20|0);
   $39 = $21;
  } else {
   $39 = 0;
  }
  $22 = HEAP32[$pPage>>2]|0;
  if (!($39)) {
   break;
  }
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pPage>>2] = $24;
 }
 $25 = ($22|0)!=(0|0);
 if ($25) {
  $26 = HEAP32[$pPage>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = ($28<<24>>24)!=(0);
  $30 = HEAP32[$pPage>>2]|0;
  if ($29) {
   HEAP32[$0>>2] = $30;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  } else {
   $31 = (_pcache1PinPage($30)|0);
   HEAP32[$0>>2] = $31;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = ($32|0)!=(0);
  if ($33) {
   $34 = HEAP32[$pCache>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = (_pcache1FetchStage2($34,$35,$36)|0);
   HEAP32[$0>>2] = $37;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  } else {
   HEAP32[$0>>2] = 0;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
 }
 return (0)|0;
}
function _pcache1FetchStage2($pCache,$iKey,$createFlag) {
 $pCache = $pCache|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $h = 0, $nPinned = 0, $pGroup = 0, $pOther = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $nPinned = sp + 16|0;
 $pGroup = sp + 12|0;
 $pPage = sp + 8|0;
 $pOther = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $iKey;
 HEAP32[$3>>2] = $createFlag;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pGroup>>2] = $5;
 HEAP32[$pPage>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 36|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($8) - ($11))|0;
 HEAP32[$nPinned>>2] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)==(1);
 do {
  if ($14) {
   $15 = HEAP32[$nPinned>>2]|0;
   $16 = HEAP32[$pGroup>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($15>>>0)>=($18>>>0);
   if (!($19)) {
    $20 = HEAP32[$nPinned>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 28|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($20>>>0)>=($23>>>0);
    if (!($24)) {
     $25 = HEAP32[$1>>2]|0;
     $26 = (_pcache1UnderMemoryPressure($25)|0);
     $27 = ($26|0)!=(0);
     if (!($27)) {
      break;
     }
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 36|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$nPinned>>2]|0;
     $32 = ($30>>>0)<($31>>>0);
     if (!($32)) {
      break;
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $147 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($147|0);
  }
 } while(0);
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 40|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 44|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($35>>>0)>=($38>>>0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  _pcache1ResizeHash($40);
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 16|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0);
 do {
  if ($44) {
   $45 = HEAP32[$pGroup>>2]|0;
   $46 = ((($45)) + 20|0);
   $47 = ((($46)) + 28|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ((($48)) + 14|0);
   $50 = HEAP8[$49>>0]|0;
   $51 = ($50<<24>>24)!=(0);
   if (!($51)) {
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 40|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + 1)|0;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 24|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = ($55>>>0)>=($58>>>0);
    if (!($59)) {
     $60 = HEAP32[$1>>2]|0;
     $61 = (_pcache1UnderMemoryPressure($60)|0);
     $62 = ($61|0)!=(0);
     if (!($62)) {
      break;
     }
    }
    $63 = HEAP32[$pGroup>>2]|0;
    $64 = ((($63)) + 20|0);
    $65 = ((($64)) + 28|0);
    $66 = HEAP32[$65>>2]|0;
    HEAP32[$pPage>>2] = $66;
    $67 = HEAP32[$pPage>>2]|0;
    _pcache1RemoveFromHash($67,0);
    $68 = HEAP32[$pPage>>2]|0;
    (_pcache1PinPage($68)|0);
    $69 = HEAP32[$pPage>>2]|0;
    $70 = ((($69)) + 20|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$pOther>>2] = $71;
    $72 = HEAP32[$pOther>>2]|0;
    $73 = ((($72)) + 12|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 12|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = ($74|0)!=($77|0);
    if ($78) {
     $79 = HEAP32[$pPage>>2]|0;
     _pcache1FreePage($79);
     HEAP32[$pPage>>2] = 0;
     break;
    } else {
     $80 = HEAP32[$pOther>>2]|0;
     $81 = ((($80)) + 16|0);
     $82 = HEAP32[$81>>2]|0;
     $83 = HEAP32[$1>>2]|0;
     $84 = ((($83)) + 16|0);
     $85 = HEAP32[$84>>2]|0;
     $86 = (($82) - ($85))|0;
     $87 = HEAP32[$pGroup>>2]|0;
     $88 = ((($87)) + 16|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = (($89) - ($86))|0;
     HEAP32[$88>>2] = $90;
     break;
    }
   }
  }
 } while(0);
 $91 = HEAP32[$pPage>>2]|0;
 $92 = ($91|0)!=(0|0);
 if (!($92)) {
  $93 = HEAP32[$1>>2]|0;
  $94 = HEAP32[$3>>2]|0;
  $95 = ($94|0)==(1);
  $96 = $95&1;
  $97 = (_pcache1AllocPage($93,$96)|0);
  HEAP32[$pPage>>2] = $97;
 }
 $98 = HEAP32[$pPage>>2]|0;
 $99 = ($98|0)!=(0|0);
 if ($99) {
  $100 = HEAP32[$2>>2]|0;
  $101 = HEAP32[$1>>2]|0;
  $102 = ((($101)) + 44|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = (($100>>>0) % ($103>>>0))&-1;
  HEAP32[$h>>2] = $104;
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 40|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($107) + 1)|0;
  HEAP32[$106>>2] = $108;
  $109 = HEAP32[$2>>2]|0;
  $110 = HEAP32[$pPage>>2]|0;
  $111 = ((($110)) + 8|0);
  HEAP32[$111>>2] = $109;
  $112 = HEAP32[$h>>2]|0;
  $113 = HEAP32[$1>>2]|0;
  $114 = ((($113)) + 48|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) + ($112<<2)|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = HEAP32[$pPage>>2]|0;
  $119 = ((($118)) + 16|0);
  HEAP32[$119>>2] = $117;
  $120 = HEAP32[$1>>2]|0;
  $121 = HEAP32[$pPage>>2]|0;
  $122 = ((($121)) + 20|0);
  HEAP32[$122>>2] = $120;
  $123 = HEAP32[$pPage>>2]|0;
  $124 = ((($123)) + 28|0);
  HEAP32[$124>>2] = 0;
  $125 = HEAP32[$pPage>>2]|0;
  $126 = ((($125)) + 24|0);
  HEAP32[$126>>2] = 0;
  $127 = HEAP32[$pPage>>2]|0;
  $128 = ((($127)) + 12|0);
  HEAP8[$128>>0] = 1;
  $129 = HEAP32[$pPage>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  HEAP32[$131>>2] = 0;
  $132 = HEAP32[$pPage>>2]|0;
  $133 = HEAP32[$h>>2]|0;
  $134 = HEAP32[$1>>2]|0;
  $135 = ((($134)) + 48|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = (($136) + ($133<<2)|0);
  HEAP32[$137>>2] = $132;
  $138 = HEAP32[$2>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  $140 = ((($139)) + 32|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($138>>>0)>($141>>>0);
  if ($142) {
   $143 = HEAP32[$2>>2]|0;
   $144 = HEAP32[$1>>2]|0;
   $145 = ((($144)) + 32|0);
   HEAP32[$145>>2] = $143;
  }
 }
 $146 = HEAP32[$pPage>>2]|0;
 HEAP32[$0>>2] = $146;
 $147 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($147|0);
}
function _pcache1UnderMemoryPressure($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pCache;
 $2 = HEAP32[(1652)>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($6) + ($9))|0;
  $11 = HEAP32[(1648)>>2]|0;
  $12 = ($10|0)<=($11|0);
  if ($12) {
   $13 = HEAP32[(1680)>>2]|0;
   HEAP32[$0>>2] = $13;
   $15 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($15|0);
  }
 }
 $14 = (_sqlite3HeapNearlyFull()|0);
 HEAP32[$0>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _pcache1AllocPage($pCache,$benignMalloc) {
 $pCache = $pCache|0;
 $benignMalloc = $benignMalloc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pPg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pPg = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $benignMalloc;
 HEAP32[$p>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   label = 4;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 40|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)==(0);
   if ($10) {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_pcache1InitBulk($11)|0);
    $13 = ($12|0)!=(0);
    if ($13) {
     label = 4;
     break;
    }
   }
   $24 = HEAP32[$2>>2]|0;
   $25 = ($24|0)!=(0);
   if ($25) {
    _sqlite3BeginBenignMalloc();
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 12|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_pcache1Alloc($28)|0);
   HEAP32[$pPg>>2] = $29;
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 4|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$pPg>>2]|0;
   $34 = (($33) + ($32)|0);
   HEAP32[$p>>2] = $34;
   $35 = HEAP32[$2>>2]|0;
   $36 = ($35|0)!=(0);
   if ($36) {
    _sqlite3EndBenignMalloc();
   }
   $37 = HEAP32[$pPg>>2]|0;
   $38 = ($37|0)==(0|0);
   if (!($38)) {
    $39 = HEAP32[$pPg>>2]|0;
    $40 = HEAP32[$p>>2]|0;
    HEAP32[$40>>2] = $39;
    $41 = HEAP32[$p>>2]|0;
    $42 = ((($41)) + 32|0);
    $43 = HEAP32[$p>>2]|0;
    $44 = ((($43)) + 4|0);
    HEAP32[$44>>2] = $42;
    $45 = HEAP32[$p>>2]|0;
    $46 = ((($45)) + 13|0);
    HEAP8[$46>>0] = 0;
    $47 = HEAP32[$p>>2]|0;
    $48 = ((($47)) + 14|0);
    HEAP8[$48>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 0;
   $59 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($59|0);
  }
 } while(0);
 if ((label|0) == 4) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 52|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$p>>2] = $16;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 52|0);
  HEAP32[$21>>2] = $19;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 16|0);
  HEAP32[$23>>2] = 0;
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$53>>2]|0;
  $55 = ((($54)) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + 1)|0;
  HEAP32[$55>>2] = $57;
 }
 $58 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _pcache1InitBulk($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $nBulk = 0, $pX = 0, $szBulk = 0, $zBulk = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $szBulk = sp;
 $zBulk = sp + 20|0;
 $nBulk = sp + 16|0;
 $i = sp + 12|0;
 $pX = sp + 8|0;
 HEAP32[$1>>2] = $pCache;
 $2 = HEAP32[(1644)>>2]|0;
 $3 = ($2|0)==(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6>>>0)<(3);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[(1644)>>2]|0;
 $9 = ($8|0)>(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)<(0);
  $14 = $13 << 31 >> 31;
  $15 = HEAP32[(1644)>>2]|0;
  $16 = ($15|0)<(0);
  $17 = $16 << 31 >> 31;
  $18 = (___muldi3(($12|0),($14|0),($15|0),($17|0))|0);
  $19 = tempRet0;
  $20 = $szBulk;
  $21 = $20;
  HEAP32[$21>>2] = $18;
  $22 = (($20) + 4)|0;
  $23 = $22;
  HEAP32[$23>>2] = $19;
 } else {
  $24 = HEAP32[(1644)>>2]|0;
  $25 = ($24|0)<(0);
  $26 = $25 << 31 >> 31;
  $27 = (___muldi3(-1024,-1,($24|0),($26|0))|0);
  $28 = tempRet0;
  $29 = $szBulk;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
 }
 $33 = $szBulk;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)<(0);
 $43 = $42 << 31 >> 31;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 24|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (___muldi3(($41|0),($43|0),($46|0),0)|0);
 $48 = tempRet0;
 $49 = ($38|0)>($48|0);
 $50 = ($35>>>0)>($47>>>0);
 $51 = ($38|0)==($48|0);
 $52 = $51 & $50;
 $53 = $49 | $52;
 if ($53) {
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 24|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = Math_imul($56, $59)|0;
  $61 = $szBulk;
  $62 = $61;
  HEAP32[$62>>2] = $60;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = 0;
 }
 $65 = $szBulk;
 $66 = $65;
 $67 = HEAP32[$66>>2]|0;
 $68 = (($65) + 4)|0;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (_sqlite3Malloc($67,$70)|0);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 56|0);
 HEAP32[$73>>2] = $71;
 HEAP32[$zBulk>>2] = $71;
 _sqlite3EndBenignMalloc();
 $74 = HEAP32[$zBulk>>2]|0;
 $75 = ($74|0)!=(0|0);
 L16: do {
  if ($75) {
   $76 = HEAP32[$zBulk>>2]|0;
   $77 = (_sqlite3MallocSize($76)|0);
   $78 = HEAP32[$1>>2]|0;
   $79 = ((($78)) + 12|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = (($77|0) / ($80|0))&-1;
   HEAP32[$nBulk>>2] = $81;
   HEAP32[$i>>2] = 0;
   while(1) {
    $82 = HEAP32[$i>>2]|0;
    $83 = HEAP32[$nBulk>>2]|0;
    $84 = ($82|0)<($83|0);
    if (!($84)) {
     break L16;
    }
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 4|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = HEAP32[$zBulk>>2]|0;
    $89 = (($88) + ($87)|0);
    HEAP32[$pX>>2] = $89;
    $90 = HEAP32[$zBulk>>2]|0;
    $91 = HEAP32[$pX>>2]|0;
    HEAP32[$91>>2] = $90;
    $92 = HEAP32[$pX>>2]|0;
    $93 = ((($92)) + 32|0);
    $94 = HEAP32[$pX>>2]|0;
    $95 = ((($94)) + 4|0);
    HEAP32[$95>>2] = $93;
    $96 = HEAP32[$pX>>2]|0;
    $97 = ((($96)) + 13|0);
    HEAP8[$97>>0] = 1;
    $98 = HEAP32[$pX>>2]|0;
    $99 = ((($98)) + 14|0);
    HEAP8[$99>>0] = 0;
    $100 = HEAP32[$1>>2]|0;
    $101 = ((($100)) + 52|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = HEAP32[$pX>>2]|0;
    $104 = ((($103)) + 16|0);
    HEAP32[$104>>2] = $102;
    $105 = HEAP32[$pX>>2]|0;
    $106 = HEAP32[$1>>2]|0;
    $107 = ((($106)) + 52|0);
    HEAP32[$107>>2] = $105;
    $108 = HEAP32[$1>>2]|0;
    $109 = ((($108)) + 12|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$zBulk>>2]|0;
    $112 = (($111) + ($110)|0);
    HEAP32[$zBulk>>2] = $112;
    $113 = HEAP32[$i>>2]|0;
    $114 = (($113) + 1)|0;
    HEAP32[$i>>2] = $114;
   }
  }
 } while(0);
 $115 = HEAP32[$1>>2]|0;
 $116 = ((($115)) + 52|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = ($117|0)!=(0|0);
 $119 = $118&1;
 HEAP32[$0>>2] = $119;
 $120 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($120|0);
}
function _pcache1TruncateUnsafe($pCache,$iLimit) {
 $pCache = $pCache|0;
 $iLimit = $iLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pPage = 0;
 var $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $h = sp + 8|0;
 $pp = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $iLimit;
 HEAP32[$h>>2] = 0;
 while(1) {
  $2 = HEAP32[$h>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 44|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2>>>0)<($5>>>0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$h>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 48|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<2)|0);
  HEAP32[$pp>>2] = $11;
  while(1) {
   $12 = HEAP32[$pp>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pPage>>2] = $13;
   $14 = ($13|0)!=(0|0);
   if (!($14)) {
    break;
   }
   $15 = HEAP32[$pPage>>2]|0;
   $16 = ((($15)) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ($17>>>0)>=($18>>>0);
   if (!($19)) {
    $34 = HEAP32[$pPage>>2]|0;
    $35 = ((($34)) + 16|0);
    HEAP32[$pp>>2] = $35;
    continue;
   }
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 40|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + -1)|0;
   HEAP32[$21>>2] = $23;
   $24 = HEAP32[$pPage>>2]|0;
   $25 = ((($24)) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pp>>2]|0;
   HEAP32[$27>>2] = $26;
   $28 = HEAP32[$pPage>>2]|0;
   $29 = ((($28)) + 12|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = ($30<<24>>24)!=(0);
   if (!($31)) {
    $32 = HEAP32[$pPage>>2]|0;
    (_pcache1PinPage($32)|0);
   }
   $33 = HEAP32[$pPage>>2]|0;
   _pcache1FreePage($33);
  }
  $36 = HEAP32[$h>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$h>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheShrink($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[(160)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$1 & 255]($4);
 STACKTOP = sp;return;
}
function _disconnectAllVtab($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pMod = 0, $pSchema = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $i = sp + 16|0;
 $p = sp + 12|0;
 $pSchema = sp + 8|0;
 $pTab = sp + 4|0;
 $pMod = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pSchema>>2] = $13;
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<4)|0);
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)!=(0|0);
  L4: do {
   if ($21) {
    $22 = HEAP32[$pSchema>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = ((($23)) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    HEAP32[$p>>2] = $25;
    while(1) {
     $26 = HEAP32[$p>>2]|0;
     $27 = ($26|0)!=(0|0);
     if (!($27)) {
      break L4;
     }
     $28 = HEAP32[$p>>2]|0;
     $29 = ((($28)) + 8|0);
     $30 = HEAP32[$29>>2]|0;
     HEAP32[$pTab>>2] = $30;
     $31 = HEAP32[$pTab>>2]|0;
     $32 = ((($31)) + 42|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33&255;
     $35 = $34 & 16;
     $36 = ($35|0)!=(0);
     if ($36) {
      $37 = HEAP32[$0>>2]|0;
      $38 = HEAP32[$pTab>>2]|0;
      _sqlite3VtabDisconnect($37,$38);
     }
     $39 = HEAP32[$p>>2]|0;
     $40 = HEAP32[$39>>2]|0;
     HEAP32[$p>>2] = $40;
    }
   }
  } while(0);
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = ((($43)) + 312|0);
 $45 = ((($44)) + 8|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$p>>2] = $46;
 while(1) {
  $47 = HEAP32[$p>>2]|0;
  $48 = ($47|0)!=(0|0);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$pMod>>2] = $51;
  $52 = HEAP32[$pMod>>2]|0;
  $53 = ((($52)) + 16|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0|0);
  if ($55) {
   $56 = HEAP32[$0>>2]|0;
   $57 = HEAP32[$pMod>>2]|0;
   $58 = ((($57)) + 16|0);
   $59 = HEAP32[$58>>2]|0;
   _sqlite3VtabDisconnect($56,$59);
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  HEAP32[$p>>2] = $61;
 }
 $62 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($62);
 STACKTOP = sp;return;
}
function _connectionIsBusy($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $j = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = HEAP32[$j>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<4)|0);
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pBt>>2] = $17;
  $18 = HEAP32[$pBt>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pBt>>2]|0;
   $21 = (_sqlite3BtreeIsInBackup($20)|0);
   $22 = ($21|0)!=(0);
   if ($22) {
    label = 7;
    break;
   }
  }
  $23 = HEAP32[$j>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$j>>2] = $24;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _sqlite3VtabDisconnect($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pVTab = 0, $ppVTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $ppVTab = sp + 4|0;
 $pVTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 HEAP32[$ppVTab>>2] = $3;
 while(1) {
  $4 = HEAP32[$ppVTab>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$ppVTab>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ($9|0)==($10|0);
  $12 = HEAP32[$ppVTab>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  if ($11) {
   break;
  }
  $19 = ((($13)) + 24|0);
  HEAP32[$ppVTab>>2] = $19;
 }
 if ((label|0) == 6) {
  STACKTOP = sp;return;
 }
 HEAP32[$pVTab>>2] = $13;
 $14 = HEAP32[$pVTab>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$ppVTab>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$pVTab>>2]|0;
 _sqlite3VtabUnlock($18);
 STACKTOP = sp;return;
}
function _functionDestroy($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pDestructor = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pDestructor = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pDestructor>>2] = $4;
 $5 = HEAP32[$pDestructor>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pDestructor>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$pDestructor>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$pDestructor>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$pDestructor>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 FUNCTION_TABLE_vi[$15 & 255]($18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$pDestructor>>2]|0;
 _sqlite3DbFree($19,$20);
 STACKTOP = sp;return;
}
function _matchQuality($p,$nArg,$enc) {
 $p = $p|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $match = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $match = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nArg;
 HEAP8[$3>>0] = $enc;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(-2);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  $10 = $9 ? 0 : 6;
  HEAP32[$0>>2] = $10;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $11 = HEAP16[$6>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = HEAP32[$2>>2]|0;
 $14 = ($12|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = ($17|0)>=(0);
  if ($18) {
   HEAP32[$0>>2] = 0;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP16[$19>>1]|0;
 $21 = $20 << 16 >> 16;
 $22 = HEAP32[$2>>2]|0;
 $23 = ($21|0)==($22|0);
 if ($23) {
  HEAP32[$match>>2] = 4;
 } else {
  HEAP32[$match>>2] = 1;
 }
 $24 = HEAP8[$3>>0]|0;
 $25 = $24&255;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 2|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = $29 & 3;
 $31 = ($25|0)==($30|0);
 if ($31) {
  $32 = HEAP32[$match>>2]|0;
  $33 = (($32) + 2)|0;
  HEAP32[$match>>2] = $33;
 } else {
  $34 = HEAP8[$3>>0]|0;
  $35 = $34&255;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 2|0);
  $38 = HEAP16[$37>>1]|0;
  $39 = $38&65535;
  $40 = $35 & $39;
  $41 = $40 & 2;
  $42 = ($41|0)!=(0);
  if ($42) {
   $43 = HEAP32[$match>>2]|0;
   $44 = (($43) + 1)|0;
   HEAP32[$match>>2] = $44;
  }
 }
 $45 = HEAP32[$match>>2]|0;
 HEAP32[$0>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3BtreeCheckpoint($p,$eMode,$pnLog,$pnCkpt) {
 $p = $p|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$2>>2] = $pnLog;
 HEAP32[$3>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBt>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$rc>>2] = 6;
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $15 = HEAP32[$pBt>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$3>>2]|0;
  $20 = (_sqlite3PagerCheckpoint($16,$17,$18,$19)|0);
  HEAP32[$rc>>2] = $20;
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3PagerCheckpoint($pPager,$eMode,$pnLog,$pnCkpt) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$2>>2] = $pnLog;
 HEAP32[$3>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 216|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $32 = 0;
 } else {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 184|0);
  $16 = HEAP32[$15>>2]|0;
  $32 = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 188|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 10|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 160|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 208|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$3>>2]|0;
 $33 = (_sqlite3WalCheckpoint($10,$11,$32,$19,$23,$26,$29,$30,$31)|0);
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _binCollFunc($padFlag,$nKey1,$pKey1,$nKey2,$pKey2) {
 $padFlag = $padFlag|0;
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $nKey2 = $nKey2|0;
 $pKey2 = $pKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $padFlag;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey2;
 HEAP32[$4>>2] = $pKey2;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($5|0)<($6|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = $7 ? $8 : $9;
 HEAP32[$n>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$n>>2]|0;
 $14 = (_memcmp($11,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$n>>2]|0;
  $21 = (($19) + ($20)|0);
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$n>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = (_allSpaces($21,$24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   $27 = HEAP32[$4>>2]|0;
   $28 = HEAP32[$n>>2]|0;
   $29 = (($27) + ($28)|0);
   $30 = HEAP32[$3>>2]|0;
   $31 = HEAP32[$n>>2]|0;
   $32 = (($30) - ($31))|0;
   $33 = (_allSpaces($29,$32)|0);
   $34 = ($33|0)!=(0);
   if ($34) {
    $38 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (($35) - ($36))|0;
 HEAP32[$rc>>2] = $37;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _nocaseCollatingFunc($NotUsed,$nKey1,$pKey1,$nKey2,$pKey2) {
 $NotUsed = $NotUsed|0;
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $nKey2 = $nKey2|0;
 $pKey2 = $pKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $r = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey2;
 HEAP32[$4>>2] = $pKey2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)<($8|0);
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = $9 ? $10 : $11;
 $13 = (_sqlite3_strnicmp($5,$6,$12)|0);
 HEAP32[$r>>2] = $13;
 $14 = HEAP32[$r>>2]|0;
 $15 = (0)==($14|0);
 if (!($15)) {
  $19 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($16) - ($17))|0;
 HEAP32[$r>>2] = $18;
 $19 = HEAP32[$r>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3RegisterBuiltinFunctions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_overload_function($1,39866,2)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(7);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($5);
 STACKTOP = sp;return;
}
function _sqlite3AutoLoadExtensions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $go = 0, $i = 0, $rc = 0, $vararg_buffer = 0, $xInit = 0, $zErrmsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $i = sp + 20|0;
 $go = sp + 16|0;
 $rc = sp + 12|0;
 $xInit = sp + 8|0;
 $zErrmsg = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$go>>2] = 1;
 $1 = HEAP32[836>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$go>>2]|0;
  $4 = ($3|0)!=(0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[836>>2]|0;
  $7 = ($5>>>0)>=($6>>>0);
  if ($7) {
   HEAP32[$xInit>>2] = 0;
   HEAP32[$go>>2] = 0;
  } else {
   $8 = HEAP32[$i>>2]|0;
   $9 = HEAP32[(840)>>2]|0;
   $10 = (($9) + ($8<<2)|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$xInit>>2] = $11;
  }
  HEAP32[$zErrmsg>>2] = 0;
  $12 = HEAP32[$xInit>>2]|0;
  $13 = ($12|0)!=(0|0);
  if ($13) {
   $14 = HEAP32[$xInit>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = (FUNCTION_TABLE_iiii[$14 & 255]($15,$zErrmsg,6220)|0);
   HEAP32[$rc>>2] = $16;
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$rc>>2]|0;
    $20 = HEAP32[$zErrmsg>>2]|0;
    HEAP32[$vararg_buffer>>2] = $20;
    _sqlite3ErrorWithMsg($18,$19,39872,$vararg_buffer);
    HEAP32[$go>>2] = 0;
   }
  }
  $21 = HEAP32[$zErrmsg>>2]|0;
  _sqlite3_free($21);
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 STACKTOP = sp;return;
}
function _sqlite3Fts3Init($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $pHash = 0, $pPorter = 0, $pSimple = 0, $pUnicode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $pHash = sp + 12|0;
 $pSimple = sp + 8|0;
 $pPorter = sp + 4|0;
 $pUnicode = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pHash>>2] = 0;
 HEAP32[$pSimple>>2] = 0;
 HEAP32[$pPorter>>2] = 0;
 HEAP32[$pUnicode>>2] = 0;
 _sqlite3Fts3UnicodeTokenizer($pUnicode);
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3Fts3InitAux($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $6;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 _sqlite3Fts3SimpleTokenizerModule($pSimple);
 _sqlite3Fts3PorterTokenizerModule($pPorter);
 $7 = (_sqlite3_malloc(20)|0);
 HEAP32[$pHash>>2] = $7;
 $8 = HEAP32[$pHash>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$pHash>>2]|0;
  _sqlite3Fts3HashInit($10,1,1);
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$pHash>>2]|0;
   $14 = HEAP32[$pSimple>>2]|0;
   $15 = (_sqlite3Fts3HashInsert($13,39911,7,$14)|0);
   $16 = ($15|0)!=(0|0);
   if (!($16)) {
    $17 = HEAP32[$pHash>>2]|0;
    $18 = HEAP32[$pPorter>>2]|0;
    $19 = (_sqlite3Fts3HashInsert($17,39918,7,$18)|0);
    $20 = ($19|0)!=(0|0);
    if (!($20)) {
     $21 = HEAP32[$pHash>>2]|0;
     $22 = HEAP32[$pUnicode>>2]|0;
     $23 = (_sqlite3Fts3HashInsert($21,39925,10,$22)|0);
     $24 = ($23|0)!=(0|0);
     if (!($24)) {
      break;
     }
    }
   }
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $25 = HEAP32[$rc>>2]|0;
 $26 = (0)==($25|0);
 if ($26) {
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$pHash>>2]|0;
  $29 = (_sqlite3Fts3InitHashTable($27,$28,39935)|0);
  HEAP32[$rc>>2] = $29;
  $30 = (0)==($29|0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = (_sqlite3_overload_function($31,39950,-1)|0);
   HEAP32[$rc>>2] = $32;
   $33 = (0)==($32|0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = (_sqlite3_overload_function($34,39958,1)|0);
    HEAP32[$rc>>2] = $35;
    $36 = (0)==($35|0);
    if ($36) {
     $37 = HEAP32[$1>>2]|0;
     $38 = (_sqlite3_overload_function($37,39966,1)|0);
     HEAP32[$rc>>2] = $38;
     $39 = (0)==($38|0);
     if ($39) {
      $40 = HEAP32[$1>>2]|0;
      $41 = (_sqlite3_overload_function($40,39966,2)|0);
      HEAP32[$rc>>2] = $41;
      $42 = (0)==($41|0);
      if ($42) {
       $43 = HEAP32[$1>>2]|0;
       $44 = (_sqlite3_overload_function($43,39976,1)|0);
       HEAP32[$rc>>2] = $44;
       $45 = (0)==($44|0);
       if ($45) {
        $46 = HEAP32[$1>>2]|0;
        $47 = HEAP32[$pHash>>2]|0;
        $48 = (_sqlite3_create_module_v2($46,39985,7072,$47,146)|0);
        HEAP32[$rc>>2] = $48;
        $49 = HEAP32[$rc>>2]|0;
        $50 = ($49|0)==(0);
        if ($50) {
         $51 = HEAP32[$1>>2]|0;
         $52 = HEAP32[$pHash>>2]|0;
         $53 = (_sqlite3_create_module_v2($51,39990,7072,$52,0)|0);
         HEAP32[$rc>>2] = $53;
        }
        $54 = HEAP32[$rc>>2]|0;
        $55 = ($54|0)==(0);
        if ($55) {
         $56 = HEAP32[$1>>2]|0;
         $57 = HEAP32[$pHash>>2]|0;
         $58 = (_sqlite3Fts3InitTok($56,$57)|0);
         HEAP32[$rc>>2] = $58;
        }
        $59 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $59;
        $65 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($65|0);
       }
      }
     }
    }
   }
  }
 }
 $60 = HEAP32[$pHash>>2]|0;
 $61 = ($60|0)!=(0|0);
 if ($61) {
  $62 = HEAP32[$pHash>>2]|0;
  _sqlite3Fts3HashClear($62);
  $63 = HEAP32[$pHash>>2]|0;
  _sqlite3_free($63);
 }
 $64 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $64;
 $65 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _allSpaces($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)>(0);
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = (($4) - 1)|0;
   $6 = HEAP32[$0>>2]|0;
   $7 = (($6) + ($5)|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8 << 24 >> 24;
   $10 = ($9|0)==(32);
   $15 = $10;
  } else {
   $15 = 0;
  }
  $11 = HEAP32[$1>>2]|0;
  if (!($15)) {
   break;
  }
  $12 = (($11) + -1)|0;
  HEAP32[$1>>2] = $12;
 }
 $13 = ($11|0)==(0);
 $14 = $13&1;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3Fts3UnicodeTokenizer($ppModule) {
 $ppModule = $ppModule|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $ppModule;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$1>>2] = 7164;
 STACKTOP = sp;return;
}
function _sqlite3Fts3InitAux($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_create_module($1,40160,8832,0)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3Fts3SimpleTokenizerModule($ppModule) {
 $ppModule = $ppModule|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $ppModule;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$1>>2] = 9084;
 STACKTOP = sp;return;
}
function _sqlite3Fts3PorterTokenizerModule($ppModule) {
 $ppModule = $ppModule|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $ppModule;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$1>>2] = 9112;
 STACKTOP = sp;return;
}
function _sqlite3Fts3HashInit($pNew,$keyClass,$copyKey) {
 $pNew = $pNew|0;
 $keyClass = $keyClass|0;
 $copyKey = $copyKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pNew;
 HEAP8[$1>>0] = $keyClass;
 HEAP8[$2>>0] = $copyKey;
 $3 = HEAP8[$1>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 HEAP8[$4>>0] = $3;
 $5 = HEAP8[$2>>0]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 1|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 4|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP32[$15>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3HashInsert($pH,$pKey,$nKey,$data) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 $data = $data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $elem = 0, $h = 0, $hraw = 0, $new_elem = 0;
 var $old_data = 0, $or$cond = 0, $xHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $hraw = sp + 20|0;
 $h = sp + 16|0;
 $elem = sp + 12|0;
 $new_elem = sp + 8|0;
 $xHash = sp + 4|0;
 $old_data = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $nKey;
 HEAP32[$4>>2] = $data;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6 << 24 >> 24;
 $8 = (_ftsHashFunction($7)|0);
 HEAP32[$xHash>>2] = $8;
 $9 = HEAP32[$xHash>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iii[$9 & 255]($10,$11)|0);
 HEAP32[$hraw>>2] = $12;
 $13 = HEAP32[$hraw>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) - 1)|0;
 $18 = $13 & $17;
 HEAP32[$h>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$h>>2]|0;
 $23 = (_fts3FindElementByHash($19,$20,$21,$22)|0);
 HEAP32[$elem>>2] = $23;
 $24 = HEAP32[$elem>>2]|0;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $26 = HEAP32[$elem>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$old_data>>2] = $28;
  $29 = HEAP32[$4>>2]|0;
  $30 = ($29|0)==(0|0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP32[$elem>>2]|0;
   $33 = HEAP32[$h>>2]|0;
   _fts3RemoveElementByHash($31,$32,$33);
  } else {
   $34 = HEAP32[$4>>2]|0;
   $35 = HEAP32[$elem>>2]|0;
   $36 = ((($35)) + 8|0);
   HEAP32[$36>>2] = $34;
  }
  $37 = HEAP32[$old_data>>2]|0;
  HEAP32[$0>>2] = $37;
  $115 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($115|0);
 }
 $38 = HEAP32[$4>>2]|0;
 $39 = ($38|0)==(0|0);
 if ($39) {
  HEAP32[$0>>2] = 0;
  $115 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($115|0);
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 12|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$1>>2]|0;
  $45 = (_fts3Rehash($44,8)|0);
  $46 = ($45|0)!=(0);
  if (!($46)) {
   label = 10;
  }
 } else {
  label = 10;
 }
 do {
  if ((label|0) == 10) {
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$1>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($49|0)>=($52|0);
   if ($53) {
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 12|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = $57<<1;
    $59 = (_fts3Rehash($54,$58)|0);
    $60 = ($59|0)!=(0);
    if ($60) {
     break;
    }
   }
   $64 = (_fts3HashMalloc(20)|0);
   HEAP32[$new_elem>>2] = $64;
   $65 = HEAP32[$new_elem>>2]|0;
   $66 = ($65|0)==(0|0);
   if ($66) {
    $67 = HEAP32[$4>>2]|0;
    HEAP32[$0>>2] = $67;
    $115 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($115|0);
   }
   $68 = HEAP32[$1>>2]|0;
   $69 = ((($68)) + 1|0);
   $70 = HEAP8[$69>>0]|0;
   $71 = $70 << 24 >> 24;
   $72 = ($71|0)!=(0);
   $73 = HEAP32[$2>>2]|0;
   $74 = ($73|0)!=(0|0);
   $or$cond = $72 & $74;
   do {
    if ($or$cond) {
     $75 = HEAP32[$3>>2]|0;
     $76 = (_fts3HashMalloc($75)|0);
     $77 = HEAP32[$new_elem>>2]|0;
     $78 = ((($77)) + 12|0);
     HEAP32[$78>>2] = $76;
     $79 = HEAP32[$new_elem>>2]|0;
     $80 = ((($79)) + 12|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = ($81|0)==(0|0);
     $83 = HEAP32[$new_elem>>2]|0;
     if (!($82)) {
      $85 = ((($83)) + 12|0);
      $86 = HEAP32[$85>>2]|0;
      $87 = HEAP32[$2>>2]|0;
      $88 = HEAP32[$3>>2]|0;
      _memcpy(($86|0),($87|0),($88|0))|0;
      break;
     }
     _fts3HashFree($83);
     $84 = HEAP32[$4>>2]|0;
     HEAP32[$0>>2] = $84;
     $115 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($115|0);
    } else {
     $89 = HEAP32[$2>>2]|0;
     $90 = HEAP32[$new_elem>>2]|0;
     $91 = ((($90)) + 12|0);
     HEAP32[$91>>2] = $89;
    }
   } while(0);
   $92 = HEAP32[$3>>2]|0;
   $93 = HEAP32[$new_elem>>2]|0;
   $94 = ((($93)) + 16|0);
   HEAP32[$94>>2] = $92;
   $95 = HEAP32[$1>>2]|0;
   $96 = ((($95)) + 4|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + 1)|0;
   HEAP32[$96>>2] = $98;
   $99 = HEAP32[$hraw>>2]|0;
   $100 = HEAP32[$1>>2]|0;
   $101 = ((($100)) + 12|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) - 1)|0;
   $104 = $99 & $103;
   HEAP32[$h>>2] = $104;
   $105 = HEAP32[$1>>2]|0;
   $106 = HEAP32[$h>>2]|0;
   $107 = HEAP32[$1>>2]|0;
   $108 = ((($107)) + 16|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + ($106<<3)|0);
   $111 = HEAP32[$new_elem>>2]|0;
   _fts3HashInsertElement($105,$110,$111);
   $112 = HEAP32[$4>>2]|0;
   $113 = HEAP32[$new_elem>>2]|0;
   $114 = ((($113)) + 8|0);
   HEAP32[$114>>2] = $112;
   HEAP32[$0>>2] = 0;
   $115 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($115|0);
  }
 } while(0);
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 4|0);
 HEAP32[$62>>2] = 0;
 $63 = HEAP32[$4>>2]|0;
 HEAP32[$0>>2] = $63;
 $115 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($115|0);
}
function _sqlite3Fts3InitHashTable($db,$pHash,$zName) {
 $db = $db|0;
 $pHash = $pHash|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $any = 0, $p = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $p = sp + 4|0;
 $any = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $zName;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 HEAP32[$any>>2] = 5;
 $4 = HEAP32[$rc>>2]|0;
 $5 = (0)==($4|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$p>>2]|0;
  $9 = (_sqlite3_create_function($6,$7,1,5,$8,188,0,0)|0);
  HEAP32[$rc>>2] = $9;
 }
 $10 = HEAP32[$rc>>2]|0;
 $11 = (0)==($10|0);
 if (!($11)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = (_sqlite3_create_function($12,$13,2,5,$14,188,0,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _hashDestroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pHash = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pHash>>2] = $1;
 $2 = HEAP32[$pHash>>2]|0;
 _sqlite3Fts3HashClear($2);
 $3 = HEAP32[$pHash>>2]|0;
 _sqlite3_free($3);
 STACKTOP = sp;return;
}
function _sqlite3Fts3InitTok($db,$pHash) {
 $db = $db|0;
 $pHash = $pHash|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pHash;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_create_module($2,45077,9268,$3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3Fts3HashClear($pH) {
 $pH = $pH|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $elem = sp + 4|0;
 $next_elem = sp;
 HEAP32[$0>>2] = $pH;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$elem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 _fts3HashFree($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 16|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = 0;
 while(1) {
  $13 = HEAP32[$elem>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$elem>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$next_elem>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 1|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$elem>>2]|0;
   $23 = ((($22)) + 12|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0|0);
   if ($25) {
    $26 = HEAP32[$elem>>2]|0;
    $27 = ((($26)) + 12|0);
    $28 = HEAP32[$27>>2]|0;
    _fts3HashFree($28);
   }
  }
  $29 = HEAP32[$elem>>2]|0;
  _fts3HashFree($29);
  $30 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $30;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 4|0);
 HEAP32[$32>>2] = 0;
 STACKTOP = sp;return;
}
function _unicodeCreate($nArg,$azArg,$pp) {
 $nArg = $nArg|0;
 $azArg = $azArg|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $pNew = 0, $rc = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pNew = sp + 16|0;
 $i = sp + 12|0;
 $rc = sp + 8|0;
 $z = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $nArg;
 HEAP32[$2>>2] = $azArg;
 HEAP32[$3>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $4 = (_sqlite3_malloc(16)|0);
 HEAP32[$pNew>>2] = $4;
 $5 = HEAP32[$pNew>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 7;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $7 = HEAP32[$pNew>>2]|0;
 ;HEAP32[$7>>2]=0|0;HEAP32[$7+4>>2]=0|0;HEAP32[$7+8>>2]=0|0;HEAP32[$7+12>>2]=0|0;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$z>>2] = $18;
  $19 = HEAP32[$z>>2]|0;
  $20 = (_strlen($19)|0);
  HEAP32[$n>>2] = $20;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($21|0)==(19);
  if ($22) {
   $23 = HEAP32[$z>>2]|0;
   $24 = (_memcmp(39995,$23,19)|0);
   $25 = ($24|0)==(0);
   if ($25) {
    $26 = HEAP32[$pNew>>2]|0;
    $27 = ((($26)) + 4|0);
    HEAP32[$27>>2] = 1;
   } else {
    label = 9;
   }
  } else {
   label = 9;
  }
  do {
   if ((label|0) == 9) {
    label = 0;
    $28 = HEAP32[$n>>2]|0;
    $29 = ($28|0)==(19);
    if ($29) {
     $30 = HEAP32[$z>>2]|0;
     $31 = (_memcmp(40015,$30,19)|0);
     $32 = ($31|0)==(0);
     if ($32) {
      $33 = HEAP32[$pNew>>2]|0;
      $34 = ((($33)) + 4|0);
      HEAP32[$34>>2] = 0;
      break;
     }
    }
    $35 = HEAP32[$n>>2]|0;
    $36 = ($35|0)>=(11);
    if ($36) {
     $37 = HEAP32[$z>>2]|0;
     $38 = (_memcmp(40035,$37,11)|0);
     $39 = ($38|0)==(0);
     if ($39) {
      $40 = HEAP32[$pNew>>2]|0;
      $41 = HEAP32[$z>>2]|0;
      $42 = ((($41)) + 11|0);
      $43 = HEAP32[$n>>2]|0;
      $44 = (($43) - 11)|0;
      $45 = (_unicodeAddExceptions($40,1,$42,$44)|0);
      HEAP32[$rc>>2] = $45;
      break;
     }
    }
    $46 = HEAP32[$n>>2]|0;
    $47 = ($46|0)>=(11);
    if ($47) {
     $48 = HEAP32[$z>>2]|0;
     $49 = (_memcmp(40047,$48,11)|0);
     $50 = ($49|0)==(0);
     if ($50) {
      $51 = HEAP32[$pNew>>2]|0;
      $52 = HEAP32[$z>>2]|0;
      $53 = ((($52)) + 11|0);
      $54 = HEAP32[$n>>2]|0;
      $55 = (($54) - 11)|0;
      $56 = (_unicodeAddExceptions($51,0,$53,$55)|0);
      HEAP32[$rc>>2] = $56;
      break;
     }
    }
    HEAP32[$rc>>2] = 1;
   }
  } while(0);
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$i>>2] = $58;
 }
 $59 = HEAP32[$rc>>2]|0;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = HEAP32[$pNew>>2]|0;
  (_unicodeDestroy($61)|0);
  HEAP32[$pNew>>2] = 0;
 }
 $62 = HEAP32[$pNew>>2]|0;
 $63 = HEAP32[$3>>2]|0;
 HEAP32[$63>>2] = $62;
 $64 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $64;
 $65 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _unicodeDestroy($pTokenizer) {
 $pTokenizer = $pTokenizer|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pTokenizer;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return 0;
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$p>>2]|0;
 _sqlite3_free($7);
 STACKTOP = sp;return 0;
}
function _unicodeOpen($p,$aInput,$nInput,$pp) {
 $p = $p|0;
 $aInput = $aInput|0;
 $nInput = $nInput|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aInput;
 HEAP32[$3>>2] = $nInput;
 HEAP32[$4>>2] = $pp;
 $5 = (_sqlite3_malloc(28)|0);
 HEAP32[$pCsr>>2] = $5;
 $6 = HEAP32[$pCsr>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 7;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $8 = HEAP32[$pCsr>>2]|0;
 ;HEAP32[$8>>2]=0|0;HEAP32[$8+4>>2]=0|0;HEAP32[$8+8>>2]=0|0;HEAP32[$8+12>>2]=0|0;HEAP32[$8+16>>2]=0|0;HEAP32[$8+20>>2]=0|0;HEAP32[$8+24>>2]=0|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$pCsr>>2]|0;
 $11 = ((($10)) + 4|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0|0);
 do {
  if ($13) {
   $14 = HEAP32[$pCsr>>2]|0;
   $15 = ((($14)) + 8|0);
   HEAP32[$15>>2] = 0;
  } else {
   $16 = HEAP32[$3>>2]|0;
   $17 = ($16|0)<(0);
   if ($17) {
    $18 = HEAP32[$2>>2]|0;
    $19 = (_strlen($18)|0);
    $20 = HEAP32[$pCsr>>2]|0;
    $21 = ((($20)) + 8|0);
    HEAP32[$21>>2] = $19;
    break;
   } else {
    $22 = HEAP32[$3>>2]|0;
    $23 = HEAP32[$pCsr>>2]|0;
    $24 = ((($23)) + 8|0);
    HEAP32[$24>>2] = $22;
    break;
   }
  }
 } while(0);
 $25 = HEAP32[$pCsr>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 HEAP32[$0>>2] = 0;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _unicodeClose($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($5);
 STACKTOP = sp;return 0;
}
function _unicodeNext($pC,$paToken,$pnToken,$piStart,$piEnd,$piPos) {
 $pC = $pC|0;
 $paToken = $paToken|0;
 $pnToken = $pnToken|0;
 $piStart = $piStart|0;
 $piEnd = $piEnd|0;
 $piPos = $piPos|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCode = 0, $iOut = 0, $p = 0, $pCsr = 0, $z = 0, $zEnd = 0;
 var $zNew = 0, $zOut = 0, $zStart = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $6 = sp + 40|0;
 $pCsr = sp + 36|0;
 $p = sp + 32|0;
 $iCode = sp + 28|0;
 $zOut = sp + 24|0;
 $z = sp + 20|0;
 $zStart = sp + 16|0;
 $zEnd = sp + 12|0;
 $zTerm = sp + 8|0;
 $iOut = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $pC;
 HEAP32[$2>>2] = $paToken;
 HEAP32[$3>>2] = $pnToken;
 HEAP32[$4>>2] = $piStart;
 HEAP32[$5>>2] = $piEnd;
 HEAP32[$6>>2] = $piPos;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $7;
 $8 = HEAP32[$pCsr>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 HEAP32[$iCode>>2] = 0;
 $10 = HEAP32[$pCsr>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pCsr>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12)|0);
 HEAP32[$z>>2] = $16;
 $17 = HEAP32[$z>>2]|0;
 HEAP32[$zStart>>2] = $17;
 $18 = HEAP32[$pCsr>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pCsr>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20)|0);
 HEAP32[$zTerm>>2] = $24;
 while(1) {
  $25 = HEAP32[$z>>2]|0;
  $26 = HEAP32[$zTerm>>2]|0;
  $27 = ($25>>>0)<($26>>>0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$z>>2]|0;
  $29 = ((($28)) + 1|0);
  HEAP32[$z>>2] = $29;
  $30 = HEAP8[$28>>0]|0;
  $31 = $30&255;
  HEAP32[$iCode>>2] = $31;
  $32 = HEAP32[$iCode>>2]|0;
  $33 = ($32|0)>=(192);
  do {
   if ($33) {
    $34 = HEAP32[$iCode>>2]|0;
    $35 = (($34) - 192)|0;
    $36 = (19221 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    HEAP32[$iCode>>2] = $38;
    while(1) {
     $39 = HEAP32[$z>>2]|0;
     $40 = HEAP32[$zTerm>>2]|0;
     $41 = ($39|0)!=($40|0);
     if ($41) {
      $42 = HEAP32[$z>>2]|0;
      $43 = HEAP8[$42>>0]|0;
      $44 = $43&255;
      $45 = $44 & 192;
      $46 = ($45|0)==(128);
      $292 = $46;
     } else {
      $292 = 0;
     }
     $47 = HEAP32[$iCode>>2]|0;
     if (!($292)) {
      break;
     }
     $48 = $47 << 6;
     $49 = HEAP32[$z>>2]|0;
     $50 = ((($49)) + 1|0);
     HEAP32[$z>>2] = $50;
     $51 = HEAP8[$49>>0]|0;
     $52 = $51&255;
     $53 = 63 & $52;
     $54 = (($48) + ($53))|0;
     HEAP32[$iCode>>2] = $54;
    }
    $55 = ($47|0)<(128);
    if (!($55)) {
     $56 = HEAP32[$iCode>>2]|0;
     $57 = $56 & -2048;
     $58 = ($57|0)==(55296);
     if (!($58)) {
      $59 = HEAP32[$iCode>>2]|0;
      $60 = $59 & -2;
      $61 = ($60|0)==(65534);
      if (!($61)) {
       break;
      }
     }
    }
    HEAP32[$iCode>>2] = 65533;
   }
  } while(0);
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$iCode>>2]|0;
  $64 = (_unicodeIsAlnum($62,$63)|0);
  $65 = ($64|0)!=(0);
  if ($65) {
   break;
  }
  $66 = HEAP32[$z>>2]|0;
  HEAP32[$zStart>>2] = $66;
 }
 $67 = HEAP32[$zStart>>2]|0;
 $68 = HEAP32[$zTerm>>2]|0;
 $69 = ($67>>>0)>=($68>>>0);
 if ($69) {
  HEAP32[$0>>2] = 101;
  $291 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($291|0);
 }
 $70 = HEAP32[$pCsr>>2]|0;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 HEAP32[$zOut>>2] = $72;
 while(1) {
  $73 = HEAP32[$zOut>>2]|0;
  $74 = HEAP32[$pCsr>>2]|0;
  $75 = ((($74)) + 20|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $73;
  $78 = $76;
  $79 = (($77) - ($78))|0;
  $80 = HEAP32[$pCsr>>2]|0;
  $81 = ((($80)) + 24|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) - 4)|0;
  $84 = ($79|0)>=($83|0);
  if ($84) {
   $85 = HEAP32[$pCsr>>2]|0;
   $86 = ((($85)) + 20|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$pCsr>>2]|0;
   $89 = ((($88)) + 24|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (($90) + 64)|0;
   $92 = (_sqlite3_realloc($87,$91)|0);
   HEAP32[$zNew>>2] = $92;
   $93 = HEAP32[$zNew>>2]|0;
   $94 = ($93|0)!=(0|0);
   if (!($94)) {
    label = 20;
    break;
   }
   $95 = HEAP32[$zOut>>2]|0;
   $96 = HEAP32[$pCsr>>2]|0;
   $97 = ((($96)) + 20|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = $95;
   $100 = $98;
   $101 = (($99) - ($100))|0;
   $102 = HEAP32[$zNew>>2]|0;
   $103 = (($102) + ($101)|0);
   HEAP32[$zOut>>2] = $103;
   $104 = HEAP32[$zNew>>2]|0;
   $105 = HEAP32[$pCsr>>2]|0;
   $106 = ((($105)) + 20|0);
   HEAP32[$106>>2] = $104;
   $107 = HEAP32[$pCsr>>2]|0;
   $108 = ((($107)) + 24|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + 64)|0;
   HEAP32[$108>>2] = $110;
  }
  $111 = HEAP32[$z>>2]|0;
  HEAP32[$zEnd>>2] = $111;
  $112 = HEAP32[$iCode>>2]|0;
  $113 = HEAP32[$p>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (_sqlite3FtsUnicodeFold($112,$115)|0);
  HEAP32[$iOut>>2] = $116;
  $117 = HEAP32[$iOut>>2]|0;
  $118 = ($117|0)!=(0);
  do {
   if ($118) {
    $119 = HEAP32[$iOut>>2]|0;
    $120 = ($119|0)<(128);
    $121 = HEAP32[$iOut>>2]|0;
    if ($120) {
     $122 = $121 & 255;
     $123 = $122&255;
     $124 = HEAP32[$zOut>>2]|0;
     $125 = ((($124)) + 1|0);
     HEAP32[$zOut>>2] = $125;
     HEAP8[$124>>0] = $123;
     break;
    }
    $126 = ($121|0)<(2048);
    $127 = HEAP32[$iOut>>2]|0;
    if ($126) {
     $128 = $127 >> 6;
     $129 = $128 & 31;
     $130 = $129&255;
     $131 = $130&255;
     $132 = (192 + ($131))|0;
     $133 = $132&255;
     $134 = HEAP32[$zOut>>2]|0;
     $135 = ((($134)) + 1|0);
     HEAP32[$zOut>>2] = $135;
     HEAP8[$134>>0] = $133;
     $136 = HEAP32[$iOut>>2]|0;
     $137 = $136 & 63;
     $138 = $137&255;
     $139 = $138&255;
     $140 = (128 + ($139))|0;
     $141 = $140&255;
     $142 = HEAP32[$zOut>>2]|0;
     $143 = ((($142)) + 1|0);
     HEAP32[$zOut>>2] = $143;
     HEAP8[$142>>0] = $141;
     break;
    }
    $144 = ($127|0)<(65536);
    $145 = HEAP32[$iOut>>2]|0;
    if ($144) {
     $146 = $145 >> 12;
     $147 = $146 & 15;
     $148 = $147&255;
     $149 = $148&255;
     $150 = (224 + ($149))|0;
     $151 = $150&255;
     $152 = HEAP32[$zOut>>2]|0;
     $153 = ((($152)) + 1|0);
     HEAP32[$zOut>>2] = $153;
     HEAP8[$152>>0] = $151;
     $154 = HEAP32[$iOut>>2]|0;
     $155 = $154 >> 6;
     $156 = $155 & 63;
     $157 = $156&255;
     $158 = $157&255;
     $159 = (128 + ($158))|0;
     $160 = $159&255;
     $161 = HEAP32[$zOut>>2]|0;
     $162 = ((($161)) + 1|0);
     HEAP32[$zOut>>2] = $162;
     HEAP8[$161>>0] = $160;
     $163 = HEAP32[$iOut>>2]|0;
     $164 = $163 & 63;
     $165 = $164&255;
     $166 = $165&255;
     $167 = (128 + ($166))|0;
     $168 = $167&255;
     $169 = HEAP32[$zOut>>2]|0;
     $170 = ((($169)) + 1|0);
     HEAP32[$zOut>>2] = $170;
     HEAP8[$169>>0] = $168;
     break;
    } else {
     $171 = $145 >> 18;
     $172 = $171 & 7;
     $173 = $172&255;
     $174 = $173&255;
     $175 = (240 + ($174))|0;
     $176 = $175&255;
     $177 = HEAP32[$zOut>>2]|0;
     $178 = ((($177)) + 1|0);
     HEAP32[$zOut>>2] = $178;
     HEAP8[$177>>0] = $176;
     $179 = HEAP32[$iOut>>2]|0;
     $180 = $179 >> 12;
     $181 = $180 & 63;
     $182 = $181&255;
     $183 = $182&255;
     $184 = (128 + ($183))|0;
     $185 = $184&255;
     $186 = HEAP32[$zOut>>2]|0;
     $187 = ((($186)) + 1|0);
     HEAP32[$zOut>>2] = $187;
     HEAP8[$186>>0] = $185;
     $188 = HEAP32[$iOut>>2]|0;
     $189 = $188 >> 6;
     $190 = $189 & 63;
     $191 = $190&255;
     $192 = $191&255;
     $193 = (128 + ($192))|0;
     $194 = $193&255;
     $195 = HEAP32[$zOut>>2]|0;
     $196 = ((($195)) + 1|0);
     HEAP32[$zOut>>2] = $196;
     HEAP8[$195>>0] = $194;
     $197 = HEAP32[$iOut>>2]|0;
     $198 = $197 & 63;
     $199 = $198&255;
     $200 = $199&255;
     $201 = (128 + ($200))|0;
     $202 = $201&255;
     $203 = HEAP32[$zOut>>2]|0;
     $204 = ((($203)) + 1|0);
     HEAP32[$zOut>>2] = $204;
     HEAP8[$203>>0] = $202;
     break;
    }
   }
  } while(0);
  $205 = HEAP32[$z>>2]|0;
  $206 = HEAP32[$zTerm>>2]|0;
  $207 = ($205>>>0)>=($206>>>0);
  if ($207) {
   label = 43;
   break;
  }
  $208 = HEAP32[$z>>2]|0;
  $209 = ((($208)) + 1|0);
  HEAP32[$z>>2] = $209;
  $210 = HEAP8[$208>>0]|0;
  $211 = $210&255;
  HEAP32[$iCode>>2] = $211;
  $212 = HEAP32[$iCode>>2]|0;
  $213 = ($212|0)>=(192);
  do {
   if ($213) {
    $214 = HEAP32[$iCode>>2]|0;
    $215 = (($214) - 192)|0;
    $216 = (19221 + ($215)|0);
    $217 = HEAP8[$216>>0]|0;
    $218 = $217&255;
    HEAP32[$iCode>>2] = $218;
    while(1) {
     $219 = HEAP32[$z>>2]|0;
     $220 = HEAP32[$zTerm>>2]|0;
     $221 = ($219|0)!=($220|0);
     if ($221) {
      $222 = HEAP32[$z>>2]|0;
      $223 = HEAP8[$222>>0]|0;
      $224 = $223&255;
      $225 = $224 & 192;
      $226 = ($225|0)==(128);
      $293 = $226;
     } else {
      $293 = 0;
     }
     $227 = HEAP32[$iCode>>2]|0;
     if (!($293)) {
      break;
     }
     $228 = $227 << 6;
     $229 = HEAP32[$z>>2]|0;
     $230 = ((($229)) + 1|0);
     HEAP32[$z>>2] = $230;
     $231 = HEAP8[$229>>0]|0;
     $232 = $231&255;
     $233 = 63 & $232;
     $234 = (($228) + ($233))|0;
     HEAP32[$iCode>>2] = $234;
    }
    $235 = ($227|0)<(128);
    if (!($235)) {
     $236 = HEAP32[$iCode>>2]|0;
     $237 = $236 & -2048;
     $238 = ($237|0)==(55296);
     if (!($238)) {
      $239 = HEAP32[$iCode>>2]|0;
      $240 = $239 & -2;
      $241 = ($240|0)==(65534);
      if (!($241)) {
       break;
      }
     }
    }
    HEAP32[$iCode>>2] = 65533;
   }
  } while(0);
  $242 = HEAP32[$p>>2]|0;
  $243 = HEAP32[$iCode>>2]|0;
  $244 = (_unicodeIsAlnum($242,$243)|0);
  $245 = ($244|0)!=(0);
  if ($245) {
   continue;
  }
  $246 = HEAP32[$iCode>>2]|0;
  $247 = (_sqlite3FtsUnicodeIsdiacritic($246)|0);
  $248 = ($247|0)!=(0);
  if (!($248)) {
   label = 43;
   break;
  }
 }
 if ((label|0) == 20) {
  HEAP32[$0>>2] = 7;
  $291 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($291|0);
 }
 else if ((label|0) == 43) {
  $249 = HEAP32[$z>>2]|0;
  $250 = HEAP32[$pCsr>>2]|0;
  $251 = ((($250)) + 4|0);
  $252 = HEAP32[$251>>2]|0;
  $253 = $249;
  $254 = $252;
  $255 = (($253) - ($254))|0;
  $256 = HEAP32[$pCsr>>2]|0;
  $257 = ((($256)) + 12|0);
  HEAP32[$257>>2] = $255;
  $258 = HEAP32[$pCsr>>2]|0;
  $259 = ((($258)) + 20|0);
  $260 = HEAP32[$259>>2]|0;
  $261 = HEAP32[$2>>2]|0;
  HEAP32[$261>>2] = $260;
  $262 = HEAP32[$zOut>>2]|0;
  $263 = HEAP32[$pCsr>>2]|0;
  $264 = ((($263)) + 20|0);
  $265 = HEAP32[$264>>2]|0;
  $266 = $262;
  $267 = $265;
  $268 = (($266) - ($267))|0;
  $269 = HEAP32[$3>>2]|0;
  HEAP32[$269>>2] = $268;
  $270 = HEAP32[$zStart>>2]|0;
  $271 = HEAP32[$pCsr>>2]|0;
  $272 = ((($271)) + 4|0);
  $273 = HEAP32[$272>>2]|0;
  $274 = $270;
  $275 = $273;
  $276 = (($274) - ($275))|0;
  $277 = HEAP32[$4>>2]|0;
  HEAP32[$277>>2] = $276;
  $278 = HEAP32[$zEnd>>2]|0;
  $279 = HEAP32[$pCsr>>2]|0;
  $280 = ((($279)) + 4|0);
  $281 = HEAP32[$280>>2]|0;
  $282 = $278;
  $283 = $281;
  $284 = (($282) - ($283))|0;
  $285 = HEAP32[$5>>2]|0;
  HEAP32[$285>>2] = $284;
  $286 = HEAP32[$pCsr>>2]|0;
  $287 = ((($286)) + 16|0);
  $288 = HEAP32[$287>>2]|0;
  $289 = (($288) + 1)|0;
  HEAP32[$287>>2] = $289;
  $290 = HEAP32[$6>>2]|0;
  HEAP32[$290>>2] = $288;
  HEAP32[$0>>2] = 0;
  $291 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($291|0);
 }
 return (0)|0;
}
function _unicodeAddExceptions($p,$bAlnum,$zIn,$nIn) {
 $p = $p|0;
 $bAlnum = $bAlnum|0;
 $zIn = $zIn|0;
 $nIn = $nIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $i = 0, $iCode = 0;
 var $j = 0, $nEntry = 0, $nNew = 0, $z = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $z = sp + 28|0;
 $zTerm = sp + 24|0;
 $iCode = sp + 20|0;
 $nEntry = sp + 16|0;
 $aNew = sp + 12|0;
 $nNew = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bAlnum;
 HEAP32[$3>>2] = $zIn;
 HEAP32[$4>>2] = $nIn;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$z>>2]|0;
 $8 = (($7) + ($6)|0);
 HEAP32[$zTerm>>2] = $8;
 HEAP32[$nEntry>>2] = 0;
 while(1) {
  $9 = HEAP32[$z>>2]|0;
  $10 = HEAP32[$zTerm>>2]|0;
  $11 = ($9>>>0)<($10>>>0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$z>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$z>>2] = $13;
  $14 = HEAP8[$12>>0]|0;
  $15 = $14&255;
  HEAP32[$iCode>>2] = $15;
  $16 = HEAP32[$iCode>>2]|0;
  $17 = ($16|0)>=(192);
  do {
   if ($17) {
    $18 = HEAP32[$iCode>>2]|0;
    $19 = (($18) - 192)|0;
    $20 = (19221 + ($19)|0);
    $21 = HEAP8[$20>>0]|0;
    $22 = $21&255;
    HEAP32[$iCode>>2] = $22;
    while(1) {
     $23 = HEAP32[$z>>2]|0;
     $24 = HEAP32[$zTerm>>2]|0;
     $25 = ($23|0)!=($24|0);
     if ($25) {
      $26 = HEAP32[$z>>2]|0;
      $27 = HEAP8[$26>>0]|0;
      $28 = $27&255;
      $29 = $28 & 192;
      $30 = ($29|0)==(128);
      $155 = $30;
     } else {
      $155 = 0;
     }
     $31 = HEAP32[$iCode>>2]|0;
     if (!($155)) {
      break;
     }
     $32 = $31 << 6;
     $33 = HEAP32[$z>>2]|0;
     $34 = ((($33)) + 1|0);
     HEAP32[$z>>2] = $34;
     $35 = HEAP8[$33>>0]|0;
     $36 = $35&255;
     $37 = 63 & $36;
     $38 = (($32) + ($37))|0;
     HEAP32[$iCode>>2] = $38;
    }
    $39 = ($31|0)<(128);
    if (!($39)) {
     $40 = HEAP32[$iCode>>2]|0;
     $41 = $40 & -2048;
     $42 = ($41|0)==(55296);
     if (!($42)) {
      $43 = HEAP32[$iCode>>2]|0;
      $44 = $43 & -2;
      $45 = ($44|0)==(65534);
      if (!($45)) {
       break;
      }
     }
    }
    HEAP32[$iCode>>2] = 65533;
   }
  } while(0);
  $46 = HEAP32[$iCode>>2]|0;
  $47 = (_sqlite3FtsUnicodeIsalnum($46)|0);
  $48 = HEAP32[$2>>2]|0;
  $49 = ($47|0)!=($48|0);
  if (!($49)) {
   continue;
  }
  $50 = HEAP32[$iCode>>2]|0;
  $51 = (_sqlite3FtsUnicodeIsdiacritic($50)|0);
  $52 = ($51|0)==(0);
  if (!($52)) {
   continue;
  }
  $53 = HEAP32[$nEntry>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$nEntry>>2] = $54;
 }
 $55 = HEAP32[$nEntry>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 8|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$nEntry>>2]|0;
  $64 = (($62) + ($63))|0;
  $65 = $64<<2;
  $66 = (_sqlite3_realloc($59,$65)|0);
  HEAP32[$aNew>>2] = $66;
  $67 = HEAP32[$aNew>>2]|0;
  $68 = ($67|0)==(0|0);
  if ($68) {
   HEAP32[$0>>2] = 7;
   $154 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($154|0);
  }
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 8|0);
  $71 = HEAP32[$70>>2]|0;
  HEAP32[$nNew>>2] = $71;
  $72 = HEAP32[$3>>2]|0;
  HEAP32[$z>>2] = $72;
  while(1) {
   $73 = HEAP32[$z>>2]|0;
   $74 = HEAP32[$zTerm>>2]|0;
   $75 = ($73>>>0)<($74>>>0);
   if (!($75)) {
    break;
   }
   $76 = HEAP32[$z>>2]|0;
   $77 = ((($76)) + 1|0);
   HEAP32[$z>>2] = $77;
   $78 = HEAP8[$76>>0]|0;
   $79 = $78&255;
   HEAP32[$iCode>>2] = $79;
   $80 = HEAP32[$iCode>>2]|0;
   $81 = ($80|0)>=(192);
   do {
    if ($81) {
     $82 = HEAP32[$iCode>>2]|0;
     $83 = (($82) - 192)|0;
     $84 = (19221 + ($83)|0);
     $85 = HEAP8[$84>>0]|0;
     $86 = $85&255;
     HEAP32[$iCode>>2] = $86;
     while(1) {
      $87 = HEAP32[$z>>2]|0;
      $88 = HEAP32[$zTerm>>2]|0;
      $89 = ($87|0)!=($88|0);
      if ($89) {
       $90 = HEAP32[$z>>2]|0;
       $91 = HEAP8[$90>>0]|0;
       $92 = $91&255;
       $93 = $92 & 192;
       $94 = ($93|0)==(128);
       $156 = $94;
      } else {
       $156 = 0;
      }
      $95 = HEAP32[$iCode>>2]|0;
      if (!($156)) {
       break;
      }
      $96 = $95 << 6;
      $97 = HEAP32[$z>>2]|0;
      $98 = ((($97)) + 1|0);
      HEAP32[$z>>2] = $98;
      $99 = HEAP8[$97>>0]|0;
      $100 = $99&255;
      $101 = 63 & $100;
      $102 = (($96) + ($101))|0;
      HEAP32[$iCode>>2] = $102;
     }
     $103 = ($95|0)<(128);
     if (!($103)) {
      $104 = HEAP32[$iCode>>2]|0;
      $105 = $104 & -2048;
      $106 = ($105|0)==(55296);
      if (!($106)) {
       $107 = HEAP32[$iCode>>2]|0;
       $108 = $107 & -2;
       $109 = ($108|0)==(65534);
       if (!($109)) {
        break;
       }
      }
     }
     HEAP32[$iCode>>2] = 65533;
    }
   } while(0);
   $110 = HEAP32[$iCode>>2]|0;
   $111 = (_sqlite3FtsUnicodeIsalnum($110)|0);
   $112 = HEAP32[$2>>2]|0;
   $113 = ($111|0)!=($112|0);
   if (!($113)) {
    continue;
   }
   $114 = HEAP32[$iCode>>2]|0;
   $115 = (_sqlite3FtsUnicodeIsdiacritic($114)|0);
   $116 = ($115|0)==(0);
   if (!($116)) {
    continue;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $117 = HEAP32[$i>>2]|0;
    $118 = HEAP32[$nNew>>2]|0;
    $119 = ($117|0)<($118|0);
    if (!($119)) {
     break;
    }
    $120 = HEAP32[$i>>2]|0;
    $121 = HEAP32[$aNew>>2]|0;
    $122 = (($121) + ($120<<2)|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = HEAP32[$iCode>>2]|0;
    $125 = ($123|0)<($124|0);
    if (!($125)) {
     break;
    }
    $126 = HEAP32[$i>>2]|0;
    $127 = (($126) + 1)|0;
    HEAP32[$i>>2] = $127;
   }
   $128 = HEAP32[$nNew>>2]|0;
   HEAP32[$j>>2] = $128;
   while(1) {
    $129 = HEAP32[$j>>2]|0;
    $130 = HEAP32[$i>>2]|0;
    $131 = ($129|0)>($130|0);
    if (!($131)) {
     break;
    }
    $132 = HEAP32[$j>>2]|0;
    $133 = (($132) - 1)|0;
    $134 = HEAP32[$aNew>>2]|0;
    $135 = (($134) + ($133<<2)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = HEAP32[$j>>2]|0;
    $138 = HEAP32[$aNew>>2]|0;
    $139 = (($138) + ($137<<2)|0);
    HEAP32[$139>>2] = $136;
    $140 = HEAP32[$j>>2]|0;
    $141 = (($140) + -1)|0;
    HEAP32[$j>>2] = $141;
   }
   $142 = HEAP32[$iCode>>2]|0;
   $143 = HEAP32[$i>>2]|0;
   $144 = HEAP32[$aNew>>2]|0;
   $145 = (($144) + ($143<<2)|0);
   HEAP32[$145>>2] = $142;
   $146 = HEAP32[$nNew>>2]|0;
   $147 = (($146) + 1)|0;
   HEAP32[$nNew>>2] = $147;
  }
  $148 = HEAP32[$aNew>>2]|0;
  $149 = HEAP32[$1>>2]|0;
  $150 = ((($149)) + 12|0);
  HEAP32[$150>>2] = $148;
  $151 = HEAP32[$nNew>>2]|0;
  $152 = HEAP32[$1>>2]|0;
  $153 = ((($152)) + 8|0);
  HEAP32[$153>>2] = $151;
 }
 HEAP32[$0>>2] = 0;
 $154 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($154|0);
}
function _sqlite3FtsUnicodeIsalnum($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iHi = 0, $iLo = 0, $iRes = 0, $iTest = 0, $key = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $key = sp + 16|0;
 $iRes = sp + 12|0;
 $iHi = sp + 8|0;
 $iLo = sp + 4|0;
 $iTest = sp;
 HEAP32[$1>>2] = $c;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)<(128);
 $4 = HEAP32[$1>>2]|0;
 if ($3) {
  $5 = $4 >> 5;
  $6 = (8816 + ($5<<2)|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = $8 & 31;
  $10 = 1 << $9;
  $11 = $7 & $10;
  $12 = ($11|0)==(0);
  $13 = $12&1;
  HEAP32[$0>>2] = $13;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $14 = ($4|0)<(4194304);
 if (!($14)) {
  HEAP32[$0>>2] = 1;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = $15 << 10;
 $17 = $16 | 1023;
 HEAP32[$key>>2] = $17;
 HEAP32[$iRes>>2] = 0;
 HEAP32[$iHi>>2] = 405;
 HEAP32[$iLo>>2] = 0;
 while(1) {
  $18 = HEAP32[$iHi>>2]|0;
  $19 = HEAP32[$iLo>>2]|0;
  $20 = ($18|0)>=($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$iHi>>2]|0;
  $22 = HEAP32[$iLo>>2]|0;
  $23 = (($21) + ($22))|0;
  $24 = (($23|0) / 2)&-1;
  HEAP32[$iTest>>2] = $24;
  $25 = HEAP32[$key>>2]|0;
  $26 = HEAP32[$iTest>>2]|0;
  $27 = (7192 + ($26<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25>>>0)>=($28>>>0);
  $30 = HEAP32[$iTest>>2]|0;
  if ($29) {
   HEAP32[$iRes>>2] = $30;
   $31 = HEAP32[$iTest>>2]|0;
   $32 = (($31) + 1)|0;
   HEAP32[$iLo>>2] = $32;
   continue;
  } else {
   $33 = (($30) - 1)|0;
   HEAP32[$iHi>>2] = $33;
   continue;
  }
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = HEAP32[$iRes>>2]|0;
 $36 = (7192 + ($35<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 >>> 10;
 $39 = HEAP32[$iRes>>2]|0;
 $40 = (7192 + ($39<<2)|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 & 1023;
 $43 = (($38) + ($42))|0;
 $44 = ($34>>>0)>=($43>>>0);
 $45 = $44&1;
 HEAP32[$0>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3FtsUnicodeIsdiacritic($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $mask0 = 0, $mask1 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $mask0 = sp + 4|0;
 $mask1 = sp;
 HEAP32[$1>>2] = $c;
 HEAP32[$mask0>>2] = 134389727;
 HEAP32[$mask1>>2] = 221688;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)<(768);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)>(817);
 $or$cond = $3 | $5;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)<(800);
 if ($7) {
  $8 = HEAP32[$mask0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) - 768)|0;
  $11 = 1 << $10;
  $12 = $8 & $11;
  $19 = $12;
 } else {
  $13 = HEAP32[$mask1>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) - 768)|0;
  $16 = (($15) - 32)|0;
  $17 = 1 << $16;
  $18 = $13 & $17;
  $19 = $18;
 }
 HEAP32[$0>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _unicodeIsAlnum($p,$iCode) {
 $p = $p|0;
 $iCode = $iCode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCode;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3FtsUnicodeIsalnum($2)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_unicodeIsException($4,$5)|0);
 $7 = $3 ^ $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3FtsUnicodeFold($c,$bRemoveDiacritic) {
 $c = $c|0;
 $bRemoveDiacritic = $bRemoveDiacritic|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $cmp = 0;
 var $iHi = 0, $iLo = 0, $iRes = 0, $iTest = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $ret = sp + 24|0;
 $iHi = sp + 20|0;
 $iLo = sp + 16|0;
 $iRes = sp + 12|0;
 $iTest = sp + 8|0;
 $cmp = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $c;
 HEAP32[$1>>2] = $bRemoveDiacritic;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$ret>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)<(128);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ($5|0)>=(65);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)<=(90);
  $or$cond = $6 & $8;
  if (!($or$cond)) {
   $78 = HEAP32[$ret>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 32)|0;
  HEAP32[$ret>>2] = $10;
  $78 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = ($5|0)<(65536);
 if (!($11)) {
  $72 = HEAP32[$0>>2]|0;
  $73 = ($72|0)>=(66560);
  $74 = HEAP32[$0>>2]|0;
  $75 = ($74|0)<(66600);
  $or$cond3 = $73 & $75;
  if (!($or$cond3)) {
   $78 = HEAP32[$ret>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
  $76 = HEAP32[$0>>2]|0;
  $77 = (($76) + 40)|0;
  HEAP32[$ret>>2] = $77;
  $78 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 HEAP32[$iHi>>2] = 162;
 HEAP32[$iLo>>2] = 0;
 HEAP32[$iRes>>2] = -1;
 while(1) {
  $12 = HEAP32[$iHi>>2]|0;
  $13 = HEAP32[$iLo>>2]|0;
  $14 = ($12|0)>=($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$iHi>>2]|0;
  $16 = HEAP32[$iLo>>2]|0;
  $17 = (($15) + ($16))|0;
  $18 = (($17|0) / 2)&-1;
  HEAP32[$iTest>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$iTest>>2]|0;
  $21 = (15682 + ($20<<2)|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22&65535;
  $24 = (($19) - ($23))|0;
  HEAP32[$cmp>>2] = $24;
  $25 = HEAP32[$cmp>>2]|0;
  $26 = ($25|0)>=(0);
  $27 = HEAP32[$iTest>>2]|0;
  if ($26) {
   HEAP32[$iRes>>2] = $27;
   $28 = HEAP32[$iTest>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$iLo>>2] = $29;
   continue;
  } else {
   $30 = (($27) - 1)|0;
   HEAP32[$iHi>>2] = $30;
   continue;
  }
 }
 $31 = HEAP32[$iRes>>2]|0;
 $32 = ($31|0)>=(0);
 if ($32) {
  $33 = HEAP32[$iRes>>2]|0;
  $34 = (15682 + ($33<<2)|0);
  HEAP32[$p>>2] = $34;
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$p>>2]|0;
  $37 = HEAP16[$36>>1]|0;
  $38 = $37&65535;
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 3|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41&255;
  $43 = (($38) + ($42))|0;
  $44 = ($35|0)<($43|0);
  if ($44) {
   $45 = HEAP32[$p>>2]|0;
   $46 = ((($45)) + 2|0);
   $47 = HEAP8[$46>>0]|0;
   $48 = $47&255;
   $49 = 1 & $48;
   $50 = HEAP32[$p>>2]|0;
   $51 = HEAP16[$50>>1]|0;
   $52 = $51&65535;
   $53 = HEAP32[$0>>2]|0;
   $54 = $52 ^ $53;
   $55 = $49 & $54;
   $56 = (0)==($55|0);
   if ($56) {
    $57 = HEAP32[$0>>2]|0;
    $58 = HEAP32[$p>>2]|0;
    $59 = ((($58)) + 2|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60&255;
    $62 = $61 >> 1;
    $63 = (16334 + ($62<<1)|0);
    $64 = HEAP16[$63>>1]|0;
    $65 = $64&65535;
    $66 = (($57) + ($65))|0;
    $67 = $66 & 65535;
    HEAP32[$ret>>2] = $67;
   }
  }
 }
 $68 = HEAP32[$1>>2]|0;
 $69 = ($68|0)!=(0);
 if (!($69)) {
  $78 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $70 = HEAP32[$ret>>2]|0;
 $71 = (_remove_diacritic($70)|0);
 HEAP32[$ret>>2] = $71;
 $78 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _unicodeIsException($p,$iCode) {
 $p = $p|0;
 $iCode = $iCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $iHi = 0, $iLo = 0;
 var $iTest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $a = sp + 12|0;
 $iLo = sp + 8|0;
 $iHi = sp + 4|0;
 $iTest = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iCode;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 L1: do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 12|0);
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$a>>2] = $9;
   HEAP32[$iLo>>2] = 0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) - 1)|0;
   HEAP32[$iHi>>2] = $13;
   while(1) {
    $14 = HEAP32[$iHi>>2]|0;
    $15 = HEAP32[$iLo>>2]|0;
    $16 = ($14|0)>=($15|0);
    if (!($16)) {
     break L1;
    }
    $17 = HEAP32[$iHi>>2]|0;
    $18 = HEAP32[$iLo>>2]|0;
    $19 = (($17) + ($18))|0;
    $20 = (($19|0) / 2)&-1;
    HEAP32[$iTest>>2] = $20;
    $21 = HEAP32[$2>>2]|0;
    $22 = HEAP32[$iTest>>2]|0;
    $23 = HEAP32[$a>>2]|0;
    $24 = (($23) + ($22<<2)|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($21|0)==($25|0);
    if ($26) {
     break;
    }
    $27 = HEAP32[$2>>2]|0;
    $28 = HEAP32[$iTest>>2]|0;
    $29 = HEAP32[$a>>2]|0;
    $30 = (($29) + ($28<<2)|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = ($27|0)>($31|0);
    $33 = HEAP32[$iTest>>2]|0;
    if ($32) {
     $34 = (($33) + 1)|0;
     HEAP32[$iLo>>2] = $34;
     continue;
    } else {
     $35 = (($33) - 1)|0;
     HEAP32[$iHi>>2] = $35;
     continue;
    }
   }
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _remove_diacritic($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $aChar = 0, $aDia = 0, $iHi = 0, $iLo = 0, $iRes = 0, $iTest = 0, $key = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0;
 $0 = sp + 20|0;
 $aDia = sp + 24|0;
 $aChar = sp + 226|0;
 $key = sp + 16|0;
 $iRes = sp + 12|0;
 $iHi = sp + 8|0;
 $iLo = sp + 4|0;
 $iTest = sp;
 HEAP32[$0>>2] = $c;
 _memcpy(($aDia|0),(16488|0),202)|0;
 dest=$aChar; src=40059; stop=dest+101|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 << 3;
 $3 = $2 | 7;
 HEAP32[$key>>2] = $3;
 HEAP32[$iRes>>2] = 0;
 HEAP32[$iHi>>2] = 100;
 HEAP32[$iLo>>2] = 0;
 while(1) {
  $4 = HEAP32[$iHi>>2]|0;
  $5 = HEAP32[$iLo>>2]|0;
  $6 = ($4|0)>=($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$iHi>>2]|0;
  $8 = HEAP32[$iLo>>2]|0;
  $9 = (($7) + ($8))|0;
  $10 = (($9|0) / 2)&-1;
  HEAP32[$iTest>>2] = $10;
  $11 = HEAP32[$key>>2]|0;
  $12 = HEAP32[$iTest>>2]|0;
  $13 = (($aDia) + ($12<<1)|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = ($11>>>0)>=($15>>>0);
  $17 = HEAP32[$iTest>>2]|0;
  if ($16) {
   HEAP32[$iRes>>2] = $17;
   $18 = HEAP32[$iTest>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$iLo>>2] = $19;
   continue;
  } else {
   $20 = (($17) - 1)|0;
   HEAP32[$iHi>>2] = $20;
   continue;
  }
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$iRes>>2]|0;
 $23 = (($aDia) + ($22<<1)|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 >> 3;
 $27 = HEAP32[$iRes>>2]|0;
 $28 = (($aDia) + ($27<<1)|0);
 $29 = HEAP16[$28>>1]|0;
 $30 = $29&65535;
 $31 = $30 & 7;
 $32 = (($26) + ($31))|0;
 $33 = ($21|0)>($32|0);
 if ($33) {
  $34 = HEAP32[$0>>2]|0;
  $39 = $34;
  STACKTOP = sp;return ($39|0);
 } else {
  $35 = HEAP32[$iRes>>2]|0;
  $36 = (($aChar) + ($35)|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = $37 << 24 >> 24;
  $39 = $38;
  STACKTOP = sp;return ($39|0);
 }
 return (0)|0;
}
function _fts3auxConnectMethod($db,$pUnused,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pUnused = $pUnused|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nByte = 0, $nDb = 0, $nFts3 = 0, $or$cond = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $zDb = 0, $zFts3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $zDb = sp + 28|0;
 $zFts3 = sp + 24|0;
 $nDb = sp + 20|0;
 $nFts3 = sp + 16|0;
 $nByte = sp + 12|0;
 $rc = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pUnused;
 HEAP32[$3>>2] = $argc;
 HEAP32[$4>>2] = $argv;
 HEAP32[$5>>2] = $ppVtab;
 HEAP32[$6>>2] = $pzErr;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)!=(4);
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(5);
 $or$cond = $8 & $10;
 do {
  if (!($or$cond)) {
   $11 = HEAP32[$4>>2]|0;
   $12 = ((($11)) + 4|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$zDb>>2] = $13;
   $14 = HEAP32[$zDb>>2]|0;
   $15 = (_strlen($14)|0);
   HEAP32[$nDb>>2] = $15;
   $16 = HEAP32[$3>>2]|0;
   $17 = ($16|0)==(5);
   if ($17) {
    $18 = HEAP32[$nDb>>2]|0;
    $19 = ($18|0)==(4);
    if (!($19)) {
     break;
    }
    $20 = HEAP32[$zDb>>2]|0;
    $21 = (_sqlite3_strnicmp(17802,$20,4)|0);
    $22 = (0)==($21|0);
    if (!($22)) {
     break;
    }
    $23 = HEAP32[$4>>2]|0;
    $24 = ((($23)) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    HEAP32[$zDb>>2] = $25;
    $26 = HEAP32[$zDb>>2]|0;
    $27 = (_strlen($26)|0);
    HEAP32[$nDb>>2] = $27;
    $28 = HEAP32[$4>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    HEAP32[$zFts3>>2] = $30;
   } else {
    $31 = HEAP32[$4>>2]|0;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$zFts3>>2] = $33;
   }
   $34 = HEAP32[$zFts3>>2]|0;
   $35 = (_strlen($34)|0);
   HEAP32[$nFts3>>2] = $35;
   $36 = HEAP32[$1>>2]|0;
   $37 = (_sqlite3_declare_vtab($36,40168)|0);
   HEAP32[$rc>>2] = $37;
   $38 = HEAP32[$rc>>2]|0;
   $39 = ($38|0)!=(0);
   if ($39) {
    $40 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $40;
    $107 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($107|0);
   }
   $41 = HEAP32[$nDb>>2]|0;
   $42 = (296 + ($41))|0;
   $43 = HEAP32[$nFts3>>2]|0;
   $44 = (($42) + ($43))|0;
   $45 = (($44) + 2)|0;
   HEAP32[$nByte>>2] = $45;
   $46 = HEAP32[$nByte>>2]|0;
   $47 = (_sqlite3_malloc($46)|0);
   HEAP32[$p>>2] = $47;
   $48 = HEAP32[$p>>2]|0;
   $49 = ($48|0)!=(0|0);
   if ($49) {
    $50 = HEAP32[$p>>2]|0;
    $51 = HEAP32[$nByte>>2]|0;
    _memset(($50|0),0,($51|0))|0;
    $52 = HEAP32[$p>>2]|0;
    $53 = ((($52)) + 16|0);
    $54 = HEAP32[$p>>2]|0;
    $55 = ((($54)) + 12|0);
    HEAP32[$55>>2] = $53;
    $56 = HEAP32[$p>>2]|0;
    $57 = ((($56)) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = ((($58)) + 280|0);
    $60 = HEAP32[$p>>2]|0;
    $61 = ((($60)) + 12|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ((($62)) + 16|0);
    HEAP32[$63>>2] = $59;
    $64 = HEAP32[$nDb>>2]|0;
    $65 = (($64) + 1)|0;
    $66 = HEAP32[$p>>2]|0;
    $67 = ((($66)) + 12|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ((($68)) + 16|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (($70) + ($65)|0);
    $72 = HEAP32[$p>>2]|0;
    $73 = ((($72)) + 12|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ((($74)) + 20|0);
    HEAP32[$75>>2] = $71;
    $76 = HEAP32[$1>>2]|0;
    $77 = HEAP32[$p>>2]|0;
    $78 = ((($77)) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ((($79)) + 12|0);
    HEAP32[$80>>2] = $76;
    $81 = HEAP32[$p>>2]|0;
    $82 = ((($81)) + 12|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ((($83)) + 248|0);
    HEAP32[$84>>2] = 1;
    $85 = HEAP32[$p>>2]|0;
    $86 = ((($85)) + 12|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ((($87)) + 16|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$zDb>>2]|0;
    $91 = HEAP32[$nDb>>2]|0;
    _memcpy(($89|0),($90|0),($91|0))|0;
    $92 = HEAP32[$p>>2]|0;
    $93 = ((($92)) + 12|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ((($94)) + 20|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$zFts3>>2]|0;
    $98 = HEAP32[$nFts3>>2]|0;
    _memcpy(($96|0),($97|0),($98|0))|0;
    $99 = HEAP32[$p>>2]|0;
    $100 = ((($99)) + 12|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = ((($101)) + 20|0);
    $103 = HEAP32[$102>>2]|0;
    _sqlite3Fts3Dequote($103);
    $104 = HEAP32[$p>>2]|0;
    $105 = HEAP32[$5>>2]|0;
    HEAP32[$105>>2] = $104;
    HEAP32[$0>>2] = 0;
    $107 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($107|0);
   } else {
    HEAP32[$0>>2] = 7;
    $107 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($107|0);
   }
  }
 } while(0);
 $106 = HEAP32[$6>>2]|0;
 _sqlite3Fts3ErrMsg($106,40237,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $107 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($107|0);
}
function _fts3auxBestIndexMethod($pVTab,$pInfo) {
 $pVTab = $pVTab|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCol = 0, $iEq = 0, $iGe = 0, $iLangid = 0, $iLe = 0, $iNext = 0, $op = 0;
 var $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $i = sp + 28|0;
 $iEq = sp + 24|0;
 $iGe = sp + 20|0;
 $iLe = sp + 16|0;
 $iLangid = sp + 12|0;
 $iNext = sp + 8|0;
 $op = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pVTab;
 HEAP32[$1>>2] = $pInfo;
 HEAP32[$iEq>>2] = -1;
 HEAP32[$iGe>>2] = -1;
 HEAP32[$iLe>>2] = -1;
 HEAP32[$iLangid>>2] = -1;
 HEAP32[$iNext>>2] = 1;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(1);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ((($13)) + 4|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)==(0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 32|0);
    HEAP32[$19>>2] = 1;
   }
  }
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = ($20|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($24*12)|0)|0);
  $29 = ((($28)) + 5|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = ($30<<24>>24)!=(0);
  if ($31) {
   $32 = HEAP32[$i>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + (($32*12)|0)|0);
   $37 = ((($36)) + 4|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   HEAP32[$op>>2] = $39;
   $40 = HEAP32[$i>>2]|0;
   $41 = HEAP32[$1>>2]|0;
   $42 = ((($41)) + 4|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (($43) + (($40*12)|0)|0);
   $45 = HEAP32[$44>>2]|0;
   HEAP32[$iCol>>2] = $45;
   $46 = HEAP32[$iCol>>2]|0;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$op>>2]|0;
    $49 = ($48|0)==(2);
    if ($49) {
     $50 = HEAP32[$i>>2]|0;
     HEAP32[$iEq>>2] = $50;
    }
    $51 = HEAP32[$op>>2]|0;
    $52 = ($51|0)==(16);
    if ($52) {
     $53 = HEAP32[$i>>2]|0;
     HEAP32[$iLe>>2] = $53;
    }
    $54 = HEAP32[$op>>2]|0;
    $55 = ($54|0)==(8);
    if ($55) {
     $56 = HEAP32[$i>>2]|0;
     HEAP32[$iLe>>2] = $56;
    }
    $57 = HEAP32[$op>>2]|0;
    $58 = ($57|0)==(4);
    if ($58) {
     $59 = HEAP32[$i>>2]|0;
     HEAP32[$iGe>>2] = $59;
    }
    $60 = HEAP32[$op>>2]|0;
    $61 = ($60|0)==(32);
    if ($61) {
     $62 = HEAP32[$i>>2]|0;
     HEAP32[$iGe>>2] = $62;
    }
   }
   $63 = HEAP32[$iCol>>2]|0;
   $64 = ($63|0)==(4);
   $65 = HEAP32[$op>>2]|0;
   $66 = ($65|0)==(2);
   $or$cond = $64 & $66;
   if ($or$cond) {
    $67 = HEAP32[$i>>2]|0;
    HEAP32[$iLangid>>2] = $67;
   }
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = (($68) + 1)|0;
  HEAP32[$i>>2] = $69;
 }
 $70 = HEAP32[$iEq>>2]|0;
 $71 = ($70|0)>=(0);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 20|0);
 if ($71) {
  HEAP32[$73>>2] = 1;
  $74 = HEAP32[$iNext>>2]|0;
  $75 = (($74) + 1)|0;
  HEAP32[$iNext>>2] = $75;
  $76 = HEAP32[$iEq>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 16|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + ($76<<3)|0);
  HEAP32[$80>>2] = $74;
  $81 = HEAP32[$1>>2]|0;
  $82 = ((($81)) + 40|0);
  HEAPF64[$82>>3] = 5.0;
 } else {
  HEAP32[$73>>2] = 0;
  $83 = HEAP32[$1>>2]|0;
  $84 = ((($83)) + 40|0);
  HEAPF64[$84>>3] = 2.0E+4;
  $85 = HEAP32[$iGe>>2]|0;
  $86 = ($85|0)>=(0);
  if ($86) {
   $87 = HEAP32[$1>>2]|0;
   $88 = ((($87)) + 20|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 2)|0;
   HEAP32[$88>>2] = $90;
   $91 = HEAP32[$iNext>>2]|0;
   $92 = (($91) + 1)|0;
   HEAP32[$iNext>>2] = $92;
   $93 = HEAP32[$iGe>>2]|0;
   $94 = HEAP32[$1>>2]|0;
   $95 = ((($94)) + 16|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = (($96) + ($93<<3)|0);
   HEAP32[$97>>2] = $91;
   $98 = HEAP32[$1>>2]|0;
   $99 = ((($98)) + 40|0);
   $100 = +HEAPF64[$99>>3];
   $101 = $100 / 2.0;
   HEAPF64[$99>>3] = $101;
  }
  $102 = HEAP32[$iLe>>2]|0;
  $103 = ($102|0)>=(0);
  if ($103) {
   $104 = HEAP32[$1>>2]|0;
   $105 = ((($104)) + 20|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($106) + 4)|0;
   HEAP32[$105>>2] = $107;
   $108 = HEAP32[$iNext>>2]|0;
   $109 = (($108) + 1)|0;
   HEAP32[$iNext>>2] = $109;
   $110 = HEAP32[$iLe>>2]|0;
   $111 = HEAP32[$1>>2]|0;
   $112 = ((($111)) + 16|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = (($113) + ($110<<3)|0);
   HEAP32[$114>>2] = $108;
   $115 = HEAP32[$1>>2]|0;
   $116 = ((($115)) + 40|0);
   $117 = +HEAPF64[$116>>3];
   $118 = $117 / 2.0;
   HEAPF64[$116>>3] = $118;
  }
 }
 $119 = HEAP32[$iLangid>>2]|0;
 $120 = ($119|0)>=(0);
 if (!($120)) {
  STACKTOP = sp;return 0;
 }
 $121 = HEAP32[$iNext>>2]|0;
 $122 = (($121) + 1)|0;
 HEAP32[$iNext>>2] = $122;
 $123 = HEAP32[$iLangid>>2]|0;
 $124 = HEAP32[$1>>2]|0;
 $125 = ((($124)) + 16|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = (($126) + ($123<<3)|0);
 HEAP32[$127>>2] = $121;
 $128 = HEAP32[$1>>2]|0;
 $129 = ((($128)) + 40|0);
 $130 = +HEAPF64[$129>>3];
 $131 = $130 + -1.0;
 HEAPF64[$129>>3] = $131;
 STACKTOP = sp;return 0;
}
function _fts3auxDisconnectMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0;
 var $pFts3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $pFts3 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFts3>>2] = $4;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = ($5|0)<(40);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$pFts3>>2]|0;
  $9 = ((($8)) + 56|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  (_sqlite3_finalize($11)|0);
  $12 = HEAP32[$i>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$i>>2] = $13;
 }
 $14 = HEAP32[$pFts3>>2]|0;
 $15 = ((($14)) + 240|0);
 $16 = HEAP32[$15>>2]|0;
 _sqlite3_free($16);
 $17 = HEAP32[$p>>2]|0;
 _sqlite3_free($17);
 STACKTOP = sp;return 0;
}
function _fts3auxOpenMethod($pVTab,$ppCsr) {
 $pVTab = $pVTab|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $ppCsr;
 $3 = (_sqlite3_malloc(120)|0);
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$pCsr>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$pCsr>>2]|0;
  dest=$6; stop=dest+120|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $7 = HEAP32[$pCsr>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  HEAP32[$8>>2] = $7;
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 7;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _fts3auxCloseMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0;
 var $pFts3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pFts3 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFts3>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $5;
 $6 = HEAP32[$pFts3>>2]|0;
 _sqlite3Fts3SegmentsClose($6);
 $7 = HEAP32[$pCsr>>2]|0;
 $8 = ((($7)) + 4|0);
 _sqlite3Fts3SegReaderFinish($8);
 $9 = HEAP32[$pCsr>>2]|0;
 $10 = ((($9)) + 60|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3_free($11);
 $12 = HEAP32[$pCsr>>2]|0;
 $13 = ((($12)) + 76|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3_free($14);
 $15 = HEAP32[$pCsr>>2]|0;
 $16 = ((($15)) + 112|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3_free($17);
 $18 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($18);
 STACKTOP = sp;return 0;
}
function _fts3auxFilterMethod($pCursor,$idxNum,$idxStr,$nVal,$apVal) {
 $pCursor = $pCursor|0;
 $idxNum = $idxNum|0;
 $idxStr = $idxStr|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $iEq = 0, $iGe = 0, $iLangVal = 0, $iLangid = 0, $iLe = 0, $iNext = 0, $isScan = 0, $or$cond = 0, $pCsr = 0, $pFts3 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $pCsr = sp + 52|0;
 $pFts3 = sp + 48|0;
 $rc = sp + 44|0;
 $isScan = sp + 40|0;
 $iLangVal = sp + 36|0;
 $iEq = sp + 32|0;
 $iGe = sp + 28|0;
 $iLe = sp + 24|0;
 $iLangid = sp + 20|0;
 $iNext = sp + 16|0;
 $zStr = sp + 12|0;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $idxNum;
 HEAP32[$3>>2] = $idxStr;
 HEAP32[$4>>2] = $nVal;
 HEAP32[$5>>2] = $apVal;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pFts3>>2] = $10;
 HEAP32[$isScan>>2] = 0;
 HEAP32[$iLangVal>>2] = 0;
 HEAP32[$iEq>>2] = -1;
 HEAP32[$iGe>>2] = -1;
 HEAP32[$iLe>>2] = -1;
 HEAP32[$iLangid>>2] = -1;
 HEAP32[$iNext>>2] = 0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(1);
 if ($12) {
  $13 = HEAP32[$iNext>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$iNext>>2] = $14;
  HEAP32[$iEq>>2] = $13;
 } else {
  HEAP32[$isScan>>2] = 1;
  $15 = HEAP32[$2>>2]|0;
  $16 = $15 & 2;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$iNext>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$iNext>>2] = $19;
   HEAP32[$iGe>>2] = $18;
  }
  $20 = HEAP32[$2>>2]|0;
  $21 = $20 & 4;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$iNext>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$iNext>>2] = $24;
   HEAP32[$iLe>>2] = $23;
  }
 }
 $25 = HEAP32[$iNext>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 $27 = ($25|0)<($26|0);
 if ($27) {
  $28 = HEAP32[$iNext>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$iNext>>2] = $29;
  HEAP32[$iLangid>>2] = $28;
 }
 $30 = HEAP32[$pCsr>>2]|0;
 $31 = ((($30)) + 4|0);
 _sqlite3Fts3SegReaderFinish($31);
 $32 = HEAP32[$pCsr>>2]|0;
 $33 = ((($32)) + 60|0);
 $34 = HEAP32[$33>>2]|0;
 _sqlite3_free($34);
 $35 = HEAP32[$pCsr>>2]|0;
 $36 = ((($35)) + 112|0);
 $37 = HEAP32[$36>>2]|0;
 _sqlite3_free($37);
 $38 = HEAP32[$pCsr>>2]|0;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$pCsr>>2]|0;
 $41 = ((($40)) + 120|0);
 $42 = HEAP32[$pCsr>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = $41;
 $45 = $43;
 $46 = (($44) - ($45))|0;
 _memset(($39|0),0,($46|0))|0;
 $47 = HEAP32[$pCsr>>2]|0;
 $48 = ((($47)) + 60|0);
 $49 = ((($48)) + 12|0);
 HEAP32[$49>>2] = 3;
 $50 = HEAP32[$isScan>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = HEAP32[$pCsr>>2]|0;
  $53 = ((($52)) + 60|0);
  $54 = ((($53)) + 12|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = $55 | 16;
  HEAP32[$54>>2] = $56;
 }
 $57 = HEAP32[$iEq>>2]|0;
 $58 = ($57|0)>=(0);
 $59 = HEAP32[$iGe>>2]|0;
 $60 = ($59|0)>=(0);
 $or$cond = $58 | $60;
 if ($or$cond) {
  $61 = HEAP32[$5>>2]|0;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_sqlite3_value_text($62)|0);
  HEAP32[$zStr>>2] = $63;
  $64 = HEAP32[$zStr>>2]|0;
  $65 = ($64|0)!=(0|0);
  if ($65) {
   $66 = HEAP32[$zStr>>2]|0;
   HEAP32[$vararg_buffer>>2] = $66;
   $67 = (_sqlite3_mprintf(17521,$vararg_buffer)|0);
   $68 = HEAP32[$pCsr>>2]|0;
   $69 = ((($68)) + 60|0);
   HEAP32[$69>>2] = $67;
   $70 = HEAP32[$5>>2]|0;
   $71 = HEAP32[$70>>2]|0;
   $72 = (_sqlite3_value_bytes($71)|0);
   $73 = HEAP32[$pCsr>>2]|0;
   $74 = ((($73)) + 60|0);
   $75 = ((($74)) + 4|0);
   HEAP32[$75>>2] = $72;
   $76 = HEAP32[$pCsr>>2]|0;
   $77 = ((($76)) + 60|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)==(0|0);
   if ($79) {
    HEAP32[$0>>2] = 7;
    $139 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($139|0);
   }
  }
 }
 $80 = HEAP32[$iLe>>2]|0;
 $81 = ($80|0)>=(0);
 if ($81) {
  $82 = HEAP32[$iLe>>2]|0;
  $83 = HEAP32[$5>>2]|0;
  $84 = (($83) + ($82<<2)|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (_sqlite3_value_text($85)|0);
  HEAP32[$vararg_buffer2>>2] = $86;
  $87 = (_sqlite3_mprintf(17521,$vararg_buffer2)|0);
  $88 = HEAP32[$pCsr>>2]|0;
  $89 = ((($88)) + 76|0);
  HEAP32[$89>>2] = $87;
  $90 = HEAP32[$iLe>>2]|0;
  $91 = HEAP32[$5>>2]|0;
  $92 = (($91) + ($90<<2)|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (_sqlite3_value_bytes($93)|0);
  $95 = HEAP32[$pCsr>>2]|0;
  $96 = ((($95)) + 80|0);
  HEAP32[$96>>2] = $94;
  $97 = HEAP32[$pCsr>>2]|0;
  $98 = ((($97)) + 76|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($99|0)==(0|0);
  if ($100) {
   HEAP32[$0>>2] = 7;
   $139 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($139|0);
  }
 }
 $101 = HEAP32[$iLangid>>2]|0;
 $102 = ($101|0)>=(0);
 if ($102) {
  $103 = HEAP32[$iLangid>>2]|0;
  $104 = HEAP32[$5>>2]|0;
  $105 = (($104) + ($103<<2)|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (_sqlite3_value_int($106)|0);
  HEAP32[$iLangVal>>2] = $107;
  $108 = HEAP32[$iLangVal>>2]|0;
  $109 = ($108|0)<(0);
  $$ = $109 ? 0 : $107;
  HEAP32[$iLangVal>>2] = $$;
 }
 $110 = HEAP32[$iLangVal>>2]|0;
 $111 = HEAP32[$pCsr>>2]|0;
 $112 = ((($111)) + 84|0);
 HEAP32[$112>>2] = $110;
 $113 = HEAP32[$pFts3>>2]|0;
 $114 = HEAP32[$iLangVal>>2]|0;
 $115 = HEAP32[$pCsr>>2]|0;
 $116 = ((($115)) + 60|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = HEAP32[$pCsr>>2]|0;
 $119 = ((($118)) + 60|0);
 $120 = ((($119)) + 4|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$isScan>>2]|0;
 $123 = HEAP32[$pCsr>>2]|0;
 $124 = ((($123)) + 4|0);
 $125 = (_sqlite3Fts3SegReaderCursor($113,$114,0,-2,$117,$121,0,$122,$124)|0);
 HEAP32[$rc>>2] = $125;
 $126 = HEAP32[$rc>>2]|0;
 $127 = ($126|0)==(0);
 if ($127) {
  $128 = HEAP32[$pFts3>>2]|0;
  $129 = HEAP32[$pCsr>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$pCsr>>2]|0;
  $132 = ((($131)) + 60|0);
  $133 = (_sqlite3Fts3SegReaderStart($128,$130,$132)|0);
  HEAP32[$rc>>2] = $133;
 }
 $134 = HEAP32[$rc>>2]|0;
 $135 = ($134|0)==(0);
 if ($135) {
  $136 = HEAP32[$1>>2]|0;
  $137 = (_fts3auxNextMethod($136)|0);
  HEAP32[$rc>>2] = $137;
 }
 $138 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $138;
 $139 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($139|0);
}
function _fts3auxNextMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aDoclist = 0, $eState = 0, $i = 0, $iCol = 0, $mc = 0, $n = 0, $nDoclist = 0, $pCsr = 0, $pFts3 = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pCsr = sp + 44|0;
 $pFts3 = sp + 40|0;
 $rc = sp + 36|0;
 $i = sp + 32|0;
 $nDoclist = sp + 28|0;
 $aDoclist = sp + 24|0;
 $iCol = sp + 20|0;
 $eState = sp + 16|0;
 $n = sp + 12|0;
 $mc = sp + 8|0;
 $v = sp;
 HEAP32[$1>>2] = $pCursor;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pFts3>>2] = $6;
 $7 = HEAP32[$pCsr>>2]|0;
 $8 = ((($7)) + 96|0);
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_i64Add(($11|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = $8;
 $18 = $17;
 HEAP32[$18>>2] = $15;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $16;
 $21 = HEAP32[$pCsr>>2]|0;
 $22 = ((($21)) + 104|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 1)|0;
 HEAP32[$22>>2] = $24;
 while(1) {
  $25 = HEAP32[$pCsr>>2]|0;
  $26 = ((($25)) + 104|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$pCsr>>2]|0;
  $29 = ((($28)) + 108|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = HEAP32[$pCsr>>2]|0;
  $33 = ((($32)) + 104|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pCsr>>2]|0;
  $36 = ((($35)) + 112|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<4)|0);
  $39 = $38;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($39) + 4)|0;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)>(0);
  $46 = ($41>>>0)>(0);
  $47 = ($44|0)==(0);
  $48 = $47 & $46;
  $49 = $45 | $48;
  if ($49) {
   label = 4;
   break;
  }
  $50 = HEAP32[$pCsr>>2]|0;
  $51 = ((($50)) + 104|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$51>>2] = $53;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 $54 = HEAP32[$pFts3>>2]|0;
 $55 = HEAP32[$pCsr>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = (_sqlite3Fts3SegReaderStep($54,$56)|0);
 HEAP32[$rc>>2] = $57;
 $58 = HEAP32[$rc>>2]|0;
 $59 = ($58|0)==(100);
 do {
  if ($59) {
   HEAP32[$i>>2] = 0;
   $60 = HEAP32[$pCsr>>2]|0;
   $61 = ((($60)) + 4|0);
   $62 = ((($61)) + 52|0);
   $63 = HEAP32[$62>>2]|0;
   HEAP32[$nDoclist>>2] = $63;
   $64 = HEAP32[$pCsr>>2]|0;
   $65 = ((($64)) + 4|0);
   $66 = ((($65)) + 48|0);
   $67 = HEAP32[$66>>2]|0;
   HEAP32[$aDoclist>>2] = $67;
   HEAP32[$eState>>2] = 0;
   $68 = HEAP32[$pCsr>>2]|0;
   $69 = ((($68)) + 76|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ($70|0)!=(0|0);
   do {
    if ($71) {
     $72 = HEAP32[$pCsr>>2]|0;
     $73 = ((($72)) + 80|0);
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$pCsr>>2]|0;
     $76 = ((($75)) + 4|0);
     $77 = ((($76)) + 44|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = ($74|0)<($78|0);
     $80 = HEAP32[$pCsr>>2]|0;
     if ($79) {
      $81 = ((($80)) + 80|0);
      $82 = HEAP32[$81>>2]|0;
      $86 = $82;
     } else {
      $83 = ((($80)) + 4|0);
      $84 = ((($83)) + 44|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = $85;
     }
     HEAP32[$n>>2] = $86;
     $87 = HEAP32[$pCsr>>2]|0;
     $88 = ((($87)) + 76|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = HEAP32[$pCsr>>2]|0;
     $91 = ((($90)) + 4|0);
     $92 = ((($91)) + 40|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = HEAP32[$n>>2]|0;
     $95 = (_memcmp($89,$93,$94)|0);
     HEAP32[$mc>>2] = $95;
     $96 = HEAP32[$mc>>2]|0;
     $97 = ($96|0)<(0);
     if (!($97)) {
      $98 = HEAP32[$mc>>2]|0;
      $99 = ($98|0)==(0);
      if (!($99)) {
       break;
      }
      $100 = HEAP32[$pCsr>>2]|0;
      $101 = ((($100)) + 4|0);
      $102 = ((($101)) + 44|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = HEAP32[$pCsr>>2]|0;
      $105 = ((($104)) + 80|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = ($103|0)>($106|0);
      if (!($107)) {
       break;
      }
     }
     $108 = HEAP32[$pCsr>>2]|0;
     $109 = ((($108)) + 88|0);
     HEAP32[$109>>2] = 1;
     HEAP32[$0>>2] = 0;
     $263 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($263|0);
    }
   } while(0);
   $110 = HEAP32[$pCsr>>2]|0;
   $111 = (_fts3auxGrowStatArray($110,2)|0);
   $112 = ($111|0)!=(0);
   if ($112) {
    HEAP32[$0>>2] = 7;
    $263 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($263|0);
   }
   $113 = HEAP32[$pCsr>>2]|0;
   $114 = ((($113)) + 112|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = HEAP32[$pCsr>>2]|0;
   $117 = ((($116)) + 108|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = $118<<4;
   _memset(($115|0),0,($119|0))|0;
   HEAP32[$iCol>>2] = 0;
   L27: while(1) {
    $120 = HEAP32[$i>>2]|0;
    $121 = HEAP32[$nDoclist>>2]|0;
    $122 = ($120|0)<($121|0);
    if (!($122)) {
     label = 32;
     break;
    }
    $123 = $v;
    $124 = $123;
    HEAP32[$124>>2] = 0;
    $125 = (($123) + 4)|0;
    $126 = $125;
    HEAP32[$126>>2] = 0;
    $127 = HEAP32[$i>>2]|0;
    $128 = HEAP32[$aDoclist>>2]|0;
    $129 = (($128) + ($127)|0);
    $130 = (_sqlite3Fts3GetVarint($129,$v)|0);
    $131 = HEAP32[$i>>2]|0;
    $132 = (($131) + ($130))|0;
    HEAP32[$i>>2] = $132;
    $133 = HEAP32[$eState>>2]|0;
    switch ($133|0) {
    case 0:  {
     $134 = HEAP32[$pCsr>>2]|0;
     $135 = ((($134)) + 112|0);
     $136 = HEAP32[$135>>2]|0;
     $137 = $136;
     $138 = $137;
     $139 = HEAP32[$138>>2]|0;
     $140 = (($137) + 4)|0;
     $141 = $140;
     $142 = HEAP32[$141>>2]|0;
     $143 = (_i64Add(($139|0),($142|0),1,0)|0);
     $144 = tempRet0;
     $145 = $136;
     $146 = $145;
     HEAP32[$146>>2] = $143;
     $147 = (($145) + 4)|0;
     $148 = $147;
     HEAP32[$148>>2] = $144;
     HEAP32[$eState>>2] = 1;
     HEAP32[$iCol>>2] = 0;
     continue L27;
     break;
    }
    case 1:  {
     $149 = $v;
     $150 = $149;
     $151 = HEAP32[$150>>2]|0;
     $152 = (($149) + 4)|0;
     $153 = $152;
     $154 = HEAP32[$153>>2]|0;
     $155 = ($154|0)>(0);
     $156 = ($151>>>0)>(1);
     $157 = ($154|0)==(0);
     $158 = $157 & $156;
     $159 = $155 | $158;
     if ($159) {
      $160 = HEAP32[$pCsr>>2]|0;
      $161 = ((($160)) + 112|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ((($162)) + 16|0);
      $164 = $163;
      $165 = $164;
      $166 = HEAP32[$165>>2]|0;
      $167 = (($164) + 4)|0;
      $168 = $167;
      $169 = HEAP32[$168>>2]|0;
      $170 = (_i64Add(($166|0),($169|0),1,0)|0);
      $171 = tempRet0;
      $172 = $163;
      $173 = $172;
      HEAP32[$173>>2] = $170;
      $174 = (($172) + 4)|0;
      $175 = $174;
      HEAP32[$175>>2] = $171;
     }
     HEAP32[$eState>>2] = 2;
     break;
    }
    case 2:  {
     break;
    }
    default: {
     $229 = $v;
     $230 = $229;
     $231 = HEAP32[$230>>2]|0;
     $232 = (($229) + 4)|0;
     $233 = $232;
     $234 = HEAP32[$233>>2]|0;
     HEAP32[$iCol>>2] = $231;
     $235 = HEAP32[$pCsr>>2]|0;
     $236 = HEAP32[$iCol>>2]|0;
     $237 = (($236) + 2)|0;
     $238 = (_fts3auxGrowStatArray($235,$237)|0);
     $239 = ($238|0)!=(0);
     if ($239) {
      break L27;
     }
     $240 = HEAP32[$iCol>>2]|0;
     $241 = (($240) + 1)|0;
     $242 = HEAP32[$pCsr>>2]|0;
     $243 = ((($242)) + 112|0);
     $244 = HEAP32[$243>>2]|0;
     $245 = (($244) + ($241<<4)|0);
     $246 = $245;
     $247 = $246;
     $248 = HEAP32[$247>>2]|0;
     $249 = (($246) + 4)|0;
     $250 = $249;
     $251 = HEAP32[$250>>2]|0;
     $252 = (_i64Add(($248|0),($251|0),1,0)|0);
     $253 = tempRet0;
     $254 = $245;
     $255 = $254;
     HEAP32[$255>>2] = $252;
     $256 = (($254) + 4)|0;
     $257 = $256;
     HEAP32[$257>>2] = $253;
     HEAP32[$eState>>2] = 2;
     continue L27;
    }
    }
    $176 = $v;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (($176) + 4)|0;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = ($178|0)==(0);
    $183 = ($181|0)==(0);
    $184 = $182 & $183;
    if ($184) {
     HEAP32[$eState>>2] = 0;
     continue;
    }
    $185 = $v;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = (($185) + 4)|0;
    $189 = $188;
    $190 = HEAP32[$189>>2]|0;
    $191 = ($187|0)==(1);
    $192 = ($190|0)==(0);
    $193 = $191 & $192;
    if ($193) {
     HEAP32[$eState>>2] = 3;
     continue;
    } else {
     $194 = HEAP32[$iCol>>2]|0;
     $195 = (($194) + 1)|0;
     $196 = HEAP32[$pCsr>>2]|0;
     $197 = ((($196)) + 112|0);
     $198 = HEAP32[$197>>2]|0;
     $199 = (($198) + ($195<<4)|0);
     $200 = ((($199)) + 8|0);
     $201 = $200;
     $202 = $201;
     $203 = HEAP32[$202>>2]|0;
     $204 = (($201) + 4)|0;
     $205 = $204;
     $206 = HEAP32[$205>>2]|0;
     $207 = (_i64Add(($203|0),($206|0),1,0)|0);
     $208 = tempRet0;
     $209 = $200;
     $210 = $209;
     HEAP32[$210>>2] = $207;
     $211 = (($209) + 4)|0;
     $212 = $211;
     HEAP32[$212>>2] = $208;
     $213 = HEAP32[$pCsr>>2]|0;
     $214 = ((($213)) + 112|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = ((($215)) + 8|0);
     $217 = $216;
     $218 = $217;
     $219 = HEAP32[$218>>2]|0;
     $220 = (($217) + 4)|0;
     $221 = $220;
     $222 = HEAP32[$221>>2]|0;
     $223 = (_i64Add(($219|0),($222|0),1,0)|0);
     $224 = tempRet0;
     $225 = $216;
     $226 = $225;
     HEAP32[$226>>2] = $223;
     $227 = (($225) + 4)|0;
     $228 = $227;
     HEAP32[$228>>2] = $224;
     continue;
    }
   }
   if ((label|0) == 32) {
    $258 = HEAP32[$pCsr>>2]|0;
    $259 = ((($258)) + 104|0);
    HEAP32[$259>>2] = 0;
    HEAP32[$rc>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $263 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($263|0);
  } else {
   $260 = HEAP32[$pCsr>>2]|0;
   $261 = ((($260)) + 88|0);
   HEAP32[$261>>2] = 1;
  }
 } while(0);
 $262 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $262;
 $263 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($263|0);
}
function _fts3auxEofMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 88|0);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3auxColumnMethod($pCursor,$pCtx,$iCol) {
 $pCursor = $pCursor|0;
 $pCtx = $pCtx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pCtx;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 0:  {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = ((($7)) + 40|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = ((($11)) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  _sqlite3_result_text($5,$9,$13,(-1));
  STACKTOP = sp;return 0;
  break;
 }
 case 1:  {
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 104|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0);
  $18 = HEAP32[$1>>2]|0;
  if ($17) {
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 104|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) - 1)|0;
   _sqlite3_result_int($18,$22);
   STACKTOP = sp;return 0;
  } else {
   _sqlite3_result_text($18,29920,-1,0);
   STACKTOP = sp;return 0;
  }
  break;
 }
 case 2:  {
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 104|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 112|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<4)|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3_result_int64($23,$33,$36);
  STACKTOP = sp;return 0;
  break;
 }
 case 3:  {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP32[$p>>2]|0;
  $39 = ((($38)) + 104|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 112|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<4)|0);
  $45 = ((($44)) + 8|0);
  $46 = $45;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  _sqlite3_result_int64($37,$48,$51);
  STACKTOP = sp;return 0;
  break;
 }
 default: {
  $52 = HEAP32[$1>>2]|0;
  $53 = HEAP32[$p>>2]|0;
  $54 = ((($53)) + 84|0);
  $55 = HEAP32[$54>>2]|0;
  _sqlite3_result_int($52,$55);
  STACKTOP = sp;return 0;
 }
 }
 return (0)|0;
}
function _fts3auxRowidMethod($pCursor,$pRowid) {
 $pCursor = $pCursor|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pRowid;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$pCsr>>2]|0;
 $4 = ((($3)) + 96|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3Fts3Dequote($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iIn = 0, $iOut = 0, $quote = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $quote = sp + 12|0;
 $iIn = sp + 4|0;
 $iOut = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 HEAP8[$quote>>0] = $2;
 $3 = HEAP8[$quote>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(91);
 if (!($5)) {
  $6 = HEAP8[$quote>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)==(39);
  if (!($8)) {
   $9 = HEAP8[$quote>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)==(34);
   if (!($11)) {
    $12 = HEAP8[$quote>>0]|0;
    $13 = $12 << 24 >> 24;
    $14 = ($13|0)==(96);
    if (!($14)) {
     STACKTOP = sp;return;
    }
   }
  }
 }
 HEAP32[$iIn>>2] = 1;
 HEAP32[$iOut>>2] = 0;
 $15 = HEAP8[$quote>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(91);
 if ($17) {
  HEAP8[$quote>>0] = 93;
 }
 while(1) {
  $18 = HEAP32[$iIn>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = ($21<<24>>24)!=(0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$iIn>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26 << 24 >> 24;
  $28 = HEAP8[$quote>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($27|0)==($29|0);
  $31 = HEAP32[$iIn>>2]|0;
  $32 = (($31) + 1)|0;
  if (!($30)) {
   HEAP32[$iIn>>2] = $32;
   $47 = HEAP32[$0>>2]|0;
   $48 = (($47) + ($31)|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = HEAP32[$iOut>>2]|0;
   $51 = (($50) + 1)|0;
   HEAP32[$iOut>>2] = $51;
   $52 = HEAP32[$0>>2]|0;
   $53 = (($52) + ($50)|0);
   HEAP8[$53>>0] = $49;
   continue;
  }
  $33 = HEAP32[$0>>2]|0;
  $34 = (($33) + ($32)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35 << 24 >> 24;
  $37 = HEAP8[$quote>>0]|0;
  $38 = $37 << 24 >> 24;
  $39 = ($36|0)!=($38|0);
  if ($39) {
   break;
  }
  $40 = HEAP8[$quote>>0]|0;
  $41 = HEAP32[$iOut>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$iOut>>2] = $42;
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + ($41)|0);
  HEAP8[$44>>0] = $40;
  $45 = HEAP32[$iIn>>2]|0;
  $46 = (($45) + 2)|0;
  HEAP32[$iIn>>2] = $46;
 }
 $54 = HEAP32[$iOut>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + ($54)|0);
 HEAP8[$56>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3ErrMsg($pzErr,$zFormat,$varargs) {
 $pzErr = $pzErr|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $pzErr;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3_vmprintf($4,$ap)|0);
 $6 = HEAP32[$0>>2]|0;
 HEAP32[$6>>2] = $5;
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegmentsClose($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 244|0);
 $3 = HEAP32[$2>>2]|0;
 (_sqlite3_blob_close($3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 244|0);
 HEAP32[$5>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegReaderFinish($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($8<<2)|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3Fts3SegReaderFree($12);
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 _sqlite3_free($16);
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 _sqlite3_free($19);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 HEAP32[$22>>2] = 0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 16|0);
 HEAP32[$24>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegReaderFree($pReader) {
 $pReader = $pReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pReader;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $21 = HEAP32[$0>>2]|0;
  _sqlite3_free($21);
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 64|0);
  $9 = HEAP32[$8>>2]|0;
  _sqlite3_free($9);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 5|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 40|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_free($17);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 52|0);
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3_blob_close($20)|0);
 $21 = HEAP32[$0>>2]|0;
 _sqlite3_free($21);
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegReaderCursor($p,$iLangid,$iIndex,$iLevel,$zTerm,$nTerm,$isPrefix,$isScan,$pCsr) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $isPrefix = $isPrefix|0;
 $isScan = $isScan|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 8|0;
 $7 = sp + 4|0;
 $8 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $isPrefix;
 HEAP32[$7>>2] = $isScan;
 HEAP32[$8>>2] = $pCsr;
 $9 = HEAP32[$8>>2]|0;
 dest=$9; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 $17 = HEAP32[$7>>2]|0;
 $18 = HEAP32[$8>>2]|0;
 $19 = (_fts3SegReaderCursor($10,$11,$12,$13,$14,$15,$16,$17,$18)|0);
 STACKTOP = sp;return ($19|0);
}
function _sqlite3Fts3SegReaderStart($p,$pCsr,$pFilter) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 $pFilter = $pFilter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pFilter;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_fts3SegReaderStart($6,$7,$9,$12)|0);
 STACKTOP = sp;return ($13|0);
}
function _fts3SegReaderCursor($p,$iLangid,$iIndex,$iLevel,$zTerm,$nTerm,$isPrefix,$isScan,$pCsr) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $isPrefix = $isPrefix|0;
 $isScan = $isScan|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iEndBlock = 0, $iLeavesEndBlock = 0, $iLeavesEndBlock$ = 0, $iStartBlock = 0, $nRoot = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pSeg = 0, $pSeg1 = 0;
 var $pStmt = 0, $pi = 0, $rc = 0, $rc2 = 0, $zRoot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $8 = sp + 56|0;
 $rc = sp + 52|0;
 $pStmt = sp + 48|0;
 $rc2 = sp + 44|0;
 $pSeg = sp + 40|0;
 $pSeg1 = sp + 36|0;
 $iStartBlock = sp + 16|0;
 $iLeavesEndBlock = sp + 8|0;
 $iEndBlock = sp;
 $nRoot = sp + 32|0;
 $zRoot = sp + 28|0;
 $pi = sp + 24|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $isPrefix;
 HEAP32[$7>>2] = $isScan;
 HEAP32[$8>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)<(0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 252|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   HEAP32[$pSeg>>2] = 0;
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$4>>2]|0;
   $18 = HEAP32[$5>>2]|0;
   $19 = HEAP32[$6>>2]|0;
   $20 = ($19|0)!=(0);
   $21 = HEAP32[$7>>2]|0;
   $22 = ($21|0)!=(0);
   $23 = $20 ? 1 : $22;
   $24 = $23&1;
   $25 = (_sqlite3Fts3SegReaderPending($15,$16,$17,$18,$24,$pSeg)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)==(0);
   $28 = HEAP32[$pSeg>>2]|0;
   $29 = ($28|0)!=(0|0);
   $or$cond = $27 & $29;
   if ($or$cond) {
    $30 = HEAP32[$8>>2]|0;
    $31 = HEAP32[$pSeg>>2]|0;
    $32 = (_fts3SegReaderCursorAppend($30,$31)|0);
    HEAP32[$rc>>2] = $32;
   }
  }
 }
 $33 = HEAP32[$3>>2]|0;
 $34 = ($33|0)!=(-1);
 L6: do {
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$0>>2]|0;
    $38 = HEAP32[$1>>2]|0;
    $39 = HEAP32[$2>>2]|0;
    $40 = HEAP32[$3>>2]|0;
    $41 = (_sqlite3Fts3AllSegdirs($37,$38,$39,$40,$pStmt)|0);
    HEAP32[$rc>>2] = $41;
   }
   while(1) {
    $42 = HEAP32[$rc>>2]|0;
    $43 = ($42|0)==(0);
    if (!($43)) {
     break L6;
    }
    $44 = HEAP32[$pStmt>>2]|0;
    $45 = (_sqlite3_step($44)|0);
    HEAP32[$rc>>2] = $45;
    $46 = (100)==($45|0);
    if (!($46)) {
     break L6;
    }
    HEAP32[$pSeg1>>2] = 0;
    $47 = HEAP32[$pStmt>>2]|0;
    $48 = (_sqlite3_column_int64($47,1)|0);
    $49 = tempRet0;
    $50 = $iStartBlock;
    $51 = $50;
    HEAP32[$51>>2] = $48;
    $52 = (($50) + 4)|0;
    $53 = $52;
    HEAP32[$53>>2] = $49;
    $54 = HEAP32[$pStmt>>2]|0;
    $55 = (_sqlite3_column_int64($54,2)|0);
    $56 = tempRet0;
    $57 = $iLeavesEndBlock;
    $58 = $57;
    HEAP32[$58>>2] = $55;
    $59 = (($57) + 4)|0;
    $60 = $59;
    HEAP32[$60>>2] = $56;
    $61 = HEAP32[$pStmt>>2]|0;
    $62 = (_sqlite3_column_int64($61,3)|0);
    $63 = tempRet0;
    $64 = $iEndBlock;
    $65 = $64;
    HEAP32[$65>>2] = $62;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = $63;
    $68 = HEAP32[$pStmt>>2]|0;
    $69 = (_sqlite3_column_bytes($68,4)|0);
    HEAP32[$nRoot>>2] = $69;
    $70 = HEAP32[$pStmt>>2]|0;
    $71 = (_sqlite3_column_blob($70,4)|0);
    HEAP32[$zRoot>>2] = $71;
    $72 = $iStartBlock;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = ($74|0)!=(0);
    $79 = ($77|0)!=(0);
    $80 = $78 | $79;
    $81 = HEAP32[$4>>2]|0;
    $82 = ($81|0)!=(0|0);
    $or$cond3 = $80 & $82;
    if ($or$cond3) {
     $83 = HEAP32[$6>>2]|0;
     $84 = ($83|0)!=(0);
     $iLeavesEndBlock$ = $84 ? $iLeavesEndBlock : 0;
     HEAP32[$pi>>2] = $iLeavesEndBlock$;
     $85 = HEAP32[$0>>2]|0;
     $86 = HEAP32[$4>>2]|0;
     $87 = HEAP32[$5>>2]|0;
     $88 = HEAP32[$zRoot>>2]|0;
     $89 = HEAP32[$nRoot>>2]|0;
     $90 = HEAP32[$pi>>2]|0;
     $91 = (_fts3SelectLeaf($85,$86,$87,$88,$89,$iStartBlock,$90)|0);
     HEAP32[$rc>>2] = $91;
     $92 = HEAP32[$rc>>2]|0;
     $93 = ($92|0)!=(0);
     if ($93) {
      break L6;
     }
     $94 = HEAP32[$6>>2]|0;
     $95 = ($94|0)==(0);
     $96 = HEAP32[$7>>2]|0;
     $97 = ($96|0)==(0);
     $or$cond5 = $95 & $97;
     if ($or$cond5) {
      $98 = $iStartBlock;
      $99 = $98;
      $100 = HEAP32[$99>>2]|0;
      $101 = (($98) + 4)|0;
      $102 = $101;
      $103 = HEAP32[$102>>2]|0;
      $104 = $iLeavesEndBlock;
      $105 = $104;
      HEAP32[$105>>2] = $100;
      $106 = (($104) + 4)|0;
      $107 = $106;
      HEAP32[$107>>2] = $103;
     }
    }
    $108 = HEAP32[$8>>2]|0;
    $109 = ((($108)) + 4|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = (($110) + 1)|0;
    $112 = HEAP32[$6>>2]|0;
    $113 = ($112|0)==(0);
    $114 = HEAP32[$7>>2]|0;
    $115 = ($114|0)==(0);
    $116 = $113 ? $115 : 0;
    $117 = $116&1;
    $118 = $iStartBlock;
    $119 = $118;
    $120 = HEAP32[$119>>2]|0;
    $121 = (($118) + 4)|0;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = $iLeavesEndBlock;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = (($124) + 4)|0;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = $iEndBlock;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($130) + 4)|0;
    $134 = $133;
    $135 = HEAP32[$134>>2]|0;
    $136 = HEAP32[$zRoot>>2]|0;
    $137 = HEAP32[$nRoot>>2]|0;
    $138 = (_sqlite3Fts3SegReaderNew($111,$117,$120,$123,$126,$129,$132,$135,$136,$137,$pSeg1)|0);
    HEAP32[$rc>>2] = $138;
    $139 = HEAP32[$rc>>2]|0;
    $140 = ($139|0)!=(0);
    if ($140) {
     break L6;
    }
    $141 = HEAP32[$8>>2]|0;
    $142 = HEAP32[$pSeg1>>2]|0;
    $143 = (_fts3SegReaderCursorAppend($141,$142)|0);
    HEAP32[$rc>>2] = $143;
   }
  }
 } while(0);
 $144 = HEAP32[$pStmt>>2]|0;
 $145 = (_sqlite3_reset($144)|0);
 HEAP32[$rc2>>2] = $145;
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(101);
 if (!($147)) {
  $149 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($149|0);
 }
 $148 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $148;
 $149 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($149|0);
}
function _sqlite3Fts3SegReaderPending($p,$iIndex,$zTerm,$nTerm,$bPrefix,$ppReader) {
 $p = $p|0;
 $iIndex = $iIndex|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $bPrefix = $bPrefix|0;
 $ppReader = $ppReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $aElem = 0, $aElem2 = 0, $nAlloc = 0, $nByte = 0, $nElem = 0, $nKey = 0, $pE = 0;
 var $pHash = 0, $pReader = 0, $rc = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $pReader = sp + 40|0;
 $pE = sp + 36|0;
 $aElem = sp + 32|0;
 $nElem = sp + 28|0;
 $rc = sp + 24|0;
 $pHash = sp + 20|0;
 $nAlloc = sp + 16|0;
 $zKey = sp + 12|0;
 $nKey = sp + 8|0;
 $aElem2 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iIndex;
 HEAP32[$2>>2] = $zTerm;
 HEAP32[$3>>2] = $nTerm;
 HEAP32[$4>>2] = $bPrefix;
 HEAP32[$5>>2] = $ppReader;
 HEAP32[$pReader>>2] = 0;
 HEAP32[$aElem>>2] = 0;
 HEAP32[$nElem>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 252|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($6*24)|0)|0);
 $11 = ((($10)) + 4|0);
 HEAP32[$pHash>>2] = $11;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  HEAP32[$nAlloc>>2] = 0;
  $14 = HEAP32[$pHash>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pE>>2] = $16;
  while(1) {
   $17 = HEAP32[$pE>>2]|0;
   $18 = ($17|0)!=(0|0);
   if (!($18)) {
    break;
   }
   $19 = HEAP32[$pE>>2]|0;
   $20 = ((($19)) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$zKey>>2] = $21;
   $22 = HEAP32[$pE>>2]|0;
   $23 = ((($22)) + 16|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$nKey>>2] = $24;
   $25 = HEAP32[$3>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    label = 7;
   } else {
    $27 = HEAP32[$nKey>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = ($27|0)>=($28|0);
    if ($29) {
     $30 = HEAP32[$zKey>>2]|0;
     $31 = HEAP32[$2>>2]|0;
     $32 = HEAP32[$3>>2]|0;
     $33 = (_memcmp($30,$31,$32)|0);
     $34 = (0)==($33|0);
     if ($34) {
      label = 7;
     }
    }
   }
   if ((label|0) == 7) {
    label = 0;
    $35 = HEAP32[$nElem>>2]|0;
    $36 = HEAP32[$nAlloc>>2]|0;
    $37 = ($35|0)==($36|0);
    if ($37) {
     $38 = HEAP32[$nAlloc>>2]|0;
     $39 = (($38) + 16)|0;
     HEAP32[$nAlloc>>2] = $39;
     $40 = HEAP32[$aElem>>2]|0;
     $41 = HEAP32[$nAlloc>>2]|0;
     $42 = $41<<2;
     $43 = (_sqlite3_realloc($40,$42)|0);
     HEAP32[$aElem2>>2] = $43;
     $44 = HEAP32[$aElem2>>2]|0;
     $45 = ($44|0)!=(0|0);
     if (!($45)) {
      label = 9;
      break;
     }
     $46 = HEAP32[$aElem2>>2]|0;
     HEAP32[$aElem>>2] = $46;
    }
    $47 = HEAP32[$pE>>2]|0;
    $48 = HEAP32[$nElem>>2]|0;
    $49 = (($48) + 1)|0;
    HEAP32[$nElem>>2] = $49;
    $50 = HEAP32[$aElem>>2]|0;
    $51 = (($50) + ($48<<2)|0);
    HEAP32[$51>>2] = $47;
   }
   $52 = HEAP32[$pE>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$pE>>2] = $53;
  }
  if ((label|0) == 9) {
   HEAP32[$rc>>2] = 7;
   HEAP32[$nElem>>2] = 0;
  }
  $54 = HEAP32[$nElem>>2]|0;
  $55 = ($54|0)>(1);
  if ($55) {
   $56 = HEAP32[$aElem>>2]|0;
   $57 = HEAP32[$nElem>>2]|0;
   _qsort($56,$57,4,196);
  }
 } else {
  $58 = HEAP32[$pHash>>2]|0;
  $59 = HEAP32[$2>>2]|0;
  $60 = HEAP32[$3>>2]|0;
  $61 = (_sqlite3Fts3HashFindElem($58,$59,$60)|0);
  HEAP32[$pE>>2] = $61;
  $62 = HEAP32[$pE>>2]|0;
  $63 = ($62|0)!=(0|0);
  if ($63) {
   HEAP32[$aElem>>2] = $pE;
   HEAP32[$nElem>>2] = 1;
  }
 }
 $64 = HEAP32[$nElem>>2]|0;
 $65 = ($64|0)>(0);
 do {
  if ($65) {
   $66 = HEAP32[$nElem>>2]|0;
   $67 = (($66) + 1)|0;
   $68 = $67<<2;
   $69 = (96 + ($68))|0;
   HEAP32[$nByte>>2] = $69;
   $70 = HEAP32[$nByte>>2]|0;
   $71 = (_sqlite3_malloc($70)|0);
   HEAP32[$pReader>>2] = $71;
   $72 = HEAP32[$pReader>>2]|0;
   $73 = ($72|0)!=(0|0);
   if ($73) {
    $74 = HEAP32[$pReader>>2]|0;
    $75 = HEAP32[$nByte>>2]|0;
    _memset(($74|0),0,($75|0))|0;
    $76 = HEAP32[$pReader>>2]|0;
    HEAP32[$76>>2] = 2147483647;
    $77 = HEAP32[$pReader>>2]|0;
    $78 = ((($77)) + 96|0);
    $79 = HEAP32[$pReader>>2]|0;
    $80 = ((($79)) + 56|0);
    HEAP32[$80>>2] = $78;
    $81 = HEAP32[$pReader>>2]|0;
    $82 = ((($81)) + 56|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = HEAP32[$aElem>>2]|0;
    $85 = HEAP32[$nElem>>2]|0;
    $86 = $85<<2;
    _memcpy(($83|0),($84|0),($86|0))|0;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $87 = HEAP32[$4>>2]|0;
 $88 = ($87|0)!=(0);
 if (!($88)) {
  $90 = HEAP32[$pReader>>2]|0;
  $91 = HEAP32[$5>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($92|0);
 }
 $89 = HEAP32[$aElem>>2]|0;
 _sqlite3_free($89);
 $90 = HEAP32[$pReader>>2]|0;
 $91 = HEAP32[$5>>2]|0;
 HEAP32[$91>>2] = $90;
 $92 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($92|0);
}
function _fts3SegReaderCursorAppend($pCsr,$pNew) {
 $pCsr = $pCsr|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $apNew = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $apNew = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5|0) % 16)&-1;
 $7 = ($6|0)==(0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (($10) + 16)|0;
   $12 = $11<<2;
   HEAP32[$nByte>>2] = $12;
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$nByte>>2]|0;
   $16 = (_sqlite3_realloc($14,$15)|0);
   HEAP32[$apNew>>2] = $16;
   $17 = HEAP32[$apNew>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $20 = HEAP32[$apNew>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    HEAP32[$21>>2] = $20;
    break;
   }
   $19 = HEAP32[$2>>2]|0;
   _sqlite3Fts3SegReaderFree($19);
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$24>>2] = $26;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25<<2)|0);
 HEAP32[$29>>2] = $22;
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3Fts3AllSegdirs($p,$iLangid,$iIndex,$iLevel,$ppStmt) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $ppStmt = $ppStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $ppStmt;
 HEAP32[$pStmt>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)<(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = (_fts3SqlStmt($7,13,$pStmt,0)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$rc>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$pStmt>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = (_getAbsoluteLevel($12,$13,$14,0)|0);
   $16 = tempRet0;
   (_sqlite3_bind_int64($11,1,$15,$16)|0);
   $17 = HEAP32[$pStmt>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = (_getAbsoluteLevel($18,$19,$20,1023)|0);
   $22 = tempRet0;
   (_sqlite3_bind_int64($17,2,$21,$22)|0);
  }
 } else {
  $23 = (_fts3SqlStmt($7,12,$pStmt,0)|0);
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(0);
  if ($25) {
   $26 = HEAP32[$pStmt>>2]|0;
   $27 = HEAP32[$0>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   $31 = (_getAbsoluteLevel($27,$28,$29,$30)|0);
   $32 = tempRet0;
   (_sqlite3_bind_int64($26,1,$31,$32)|0);
  }
 }
 $33 = HEAP32[$pStmt>>2]|0;
 $34 = HEAP32[$4>>2]|0;
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _fts3SelectLeaf($p,$zTerm,$nTerm,$zNode,$nNode,$piLeaf,$piLeaf2) {
 $p = $p|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $zNode = $zNode|0;
 $nNode = $nNode|0;
 $piLeaf = $piLeaf|0;
 $piLeaf2 = $piLeaf2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $iHeight = 0, $nBlob = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $zBlob = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $rc = sp + 12|0;
 $iHeight = sp + 8|0;
 $zBlob = sp + 4|0;
 $nBlob = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $zNode;
 HEAP32[$4>>2] = $nNode;
 HEAP32[$5>>2] = $piLeaf;
 HEAP32[$6>>2] = $piLeaf2;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & 128;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$3>>2]|0;
 if ($11) {
  (_sqlite3Fts3GetVarint32($12,$iHeight)|0);
 } else {
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  HEAP32[$iHeight>>2] = $14;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$6>>2]|0;
 $21 = (_fts3ScanInteriorNode($15,$16,$17,$18,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)==(0);
 $24 = HEAP32[$iHeight>>2]|0;
 $25 = ($24|0)>(1);
 $or$cond = $23 & $25;
 if (!($or$cond)) {
  $99 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($99|0);
 }
 HEAP32[$zBlob>>2] = 0;
 HEAP32[$nBlob>>2] = 0;
 $26 = HEAP32[$5>>2]|0;
 $27 = ($26|0)!=(0|0);
 $28 = HEAP32[$6>>2]|0;
 $29 = ($28|0)!=(0|0);
 $or$cond3 = $27 & $29;
 if ($or$cond3) {
  $30 = HEAP32[$5>>2]|0;
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$6>>2]|0;
  $38 = $37;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($38) + 4)|0;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = ($33|0)!=($40|0);
  $45 = ($36|0)!=($43|0);
  $46 = $44 | $45;
  if ($46) {
   $47 = HEAP32[$0>>2]|0;
   $48 = HEAP32[$5>>2]|0;
   $49 = $48;
   $50 = $49;
   $51 = HEAP32[$50>>2]|0;
   $52 = (($49) + 4)|0;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (_sqlite3Fts3ReadBlock($47,$51,$54,$zBlob,$nBlob,0)|0);
   HEAP32[$rc>>2] = $55;
   $56 = HEAP32[$rc>>2]|0;
   $57 = ($56|0)==(0);
   if ($57) {
    $58 = HEAP32[$0>>2]|0;
    $59 = HEAP32[$1>>2]|0;
    $60 = HEAP32[$2>>2]|0;
    $61 = HEAP32[$zBlob>>2]|0;
    $62 = HEAP32[$nBlob>>2]|0;
    $63 = HEAP32[$5>>2]|0;
    $64 = (_fts3SelectLeaf($58,$59,$60,$61,$62,$63,0)|0);
    HEAP32[$rc>>2] = $64;
   }
   $65 = HEAP32[$zBlob>>2]|0;
   _sqlite3_free($65);
   HEAP32[$5>>2] = 0;
   HEAP32[$zBlob>>2] = 0;
  }
 }
 $66 = HEAP32[$rc>>2]|0;
 $67 = ($66|0)==(0);
 if ($67) {
  $68 = HEAP32[$0>>2]|0;
  $69 = HEAP32[$5>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$5>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $85 = $74;$86 = $77;
  } else {
   $78 = HEAP32[$6>>2]|0;
   $79 = $78;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (($79) + 4)|0;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = $81;$86 = $84;
  }
  $87 = (_sqlite3Fts3ReadBlock($68,$85,$86,$zBlob,$nBlob,0)|0);
  HEAP32[$rc>>2] = $87;
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = ($88|0)==(0);
 if ($89) {
  $90 = HEAP32[$0>>2]|0;
  $91 = HEAP32[$1>>2]|0;
  $92 = HEAP32[$2>>2]|0;
  $93 = HEAP32[$zBlob>>2]|0;
  $94 = HEAP32[$nBlob>>2]|0;
  $95 = HEAP32[$5>>2]|0;
  $96 = HEAP32[$6>>2]|0;
  $97 = (_fts3SelectLeaf($90,$91,$92,$93,$94,$95,$96)|0);
  HEAP32[$rc>>2] = $97;
 }
 $98 = HEAP32[$zBlob>>2]|0;
 _sqlite3_free($98);
 $99 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($99|0);
}
function _sqlite3Fts3SegReaderNew($iAge,$bLookup,$0,$1,$2,$3,$4,$5,$zRoot,$nRoot,$ppReader) {
 $iAge = $iAge|0;
 $bLookup = $bLookup|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $4 = $4|0;
 $5 = $5|0;
 $zRoot = $zRoot|0;
 $nRoot = $nRoot|0;
 $ppReader = $ppReader|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $nExtra = 0, $pReader = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $6 = sp + 52|0;
 $7 = sp + 48|0;
 $8 = sp + 44|0;
 $9 = sp + 16|0;
 $10 = sp + 8|0;
 $11 = sp;
 $12 = sp + 40|0;
 $13 = sp + 36|0;
 $14 = sp + 32|0;
 $pReader = sp + 28|0;
 $nExtra = sp + 24|0;
 HEAP32[$7>>2] = $iAge;
 HEAP32[$8>>2] = $bLookup;
 $15 = $9;
 $16 = $15;
 HEAP32[$16>>2] = $0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = $1;
 $19 = $10;
 $20 = $19;
 HEAP32[$20>>2] = $2;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $3;
 $23 = $11;
 $24 = $23;
 HEAP32[$24>>2] = $4;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $5;
 HEAP32[$12>>2] = $zRoot;
 HEAP32[$13>>2] = $nRoot;
 HEAP32[$14>>2] = $ppReader;
 HEAP32[$nExtra>>2] = 0;
 $27 = $9;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($29|0)==(0);
 $34 = ($32|0)==(0);
 $35 = $33 & $34;
 if ($35) {
  $36 = HEAP32[$13>>2]|0;
  $37 = (($36) + 20)|0;
  HEAP32[$nExtra>>2] = $37;
 }
 $38 = HEAP32[$nExtra>>2]|0;
 $39 = (96 + ($38))|0;
 $40 = (_sqlite3_malloc($39)|0);
 HEAP32[$pReader>>2] = $40;
 $41 = HEAP32[$pReader>>2]|0;
 $42 = ($41|0)!=(0|0);
 if (!($42)) {
  HEAP32[$6>>2] = 7;
  $125 = HEAP32[$6>>2]|0;
  STACKTOP = sp;return ($125|0);
 }
 $43 = HEAP32[$pReader>>2]|0;
 dest=$43; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $44 = HEAP32[$7>>2]|0;
 $45 = HEAP32[$pReader>>2]|0;
 HEAP32[$45>>2] = $44;
 $46 = HEAP32[$8>>2]|0;
 $47 = ($46|0)!=(0);
 $48 = $47&1;
 $49 = $48&255;
 $50 = HEAP32[$pReader>>2]|0;
 $51 = ((($50)) + 4|0);
 HEAP8[$51>>0] = $49;
 $52 = $9;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$pReader>>2]|0;
 $59 = ((($58)) + 8|0);
 $60 = $59;
 $61 = $60;
 HEAP32[$61>>2] = $54;
 $62 = (($60) + 4)|0;
 $63 = $62;
 HEAP32[$63>>2] = $57;
 $64 = $10;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = (($64) + 4)|0;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$pReader>>2]|0;
 $71 = ((($70)) + 16|0);
 $72 = $71;
 $73 = $72;
 HEAP32[$73>>2] = $66;
 $74 = (($72) + 4)|0;
 $75 = $74;
 HEAP32[$75>>2] = $69;
 $76 = $11;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($76) + 4)|0;
 $80 = $79;
 $81 = HEAP32[$80>>2]|0;
 $82 = HEAP32[$pReader>>2]|0;
 $83 = ((($82)) + 24|0);
 $84 = $83;
 $85 = $84;
 HEAP32[$85>>2] = $78;
 $86 = (($84) + 4)|0;
 $87 = $86;
 HEAP32[$87>>2] = $81;
 $88 = HEAP32[$nExtra>>2]|0;
 $89 = ($88|0)!=(0);
 if ($89) {
  $90 = HEAP32[$pReader>>2]|0;
  $91 = ((($90)) + 96|0);
  $92 = HEAP32[$pReader>>2]|0;
  $93 = ((($92)) + 40|0);
  HEAP32[$93>>2] = $91;
  $94 = HEAP32[$pReader>>2]|0;
  $95 = ((($94)) + 5|0);
  HEAP8[$95>>0] = 1;
  $96 = HEAP32[$13>>2]|0;
  $97 = HEAP32[$pReader>>2]|0;
  $98 = ((($97)) + 44|0);
  HEAP32[$98>>2] = $96;
  $99 = HEAP32[$pReader>>2]|0;
  $100 = ((($99)) + 40|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = HEAP32[$12>>2]|0;
  $103 = HEAP32[$13>>2]|0;
  _memcpy(($101|0),($102|0),($103|0))|0;
  $104 = HEAP32[$13>>2]|0;
  $105 = HEAP32[$pReader>>2]|0;
  $106 = ((($105)) + 40|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($107) + ($104)|0);
  dest=$108; stop=dest+20|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 } else {
  $109 = $9;
  $110 = $109;
  $111 = HEAP32[$110>>2]|0;
  $112 = (($109) + 4)|0;
  $113 = $112;
  $114 = HEAP32[$113>>2]|0;
  $115 = (_i64Subtract(($111|0),($114|0),1,0)|0);
  $116 = tempRet0;
  $117 = HEAP32[$pReader>>2]|0;
  $118 = ((($117)) + 32|0);
  $119 = $118;
  $120 = $119;
  HEAP32[$120>>2] = $115;
  $121 = (($119) + 4)|0;
  $122 = $121;
  HEAP32[$122>>2] = $116;
 }
 $123 = HEAP32[$pReader>>2]|0;
 $124 = HEAP32[$14>>2]|0;
 HEAP32[$124>>2] = $123;
 HEAP32[$6>>2] = 0;
 $125 = HEAP32[$6>>2]|0;
 STACKTOP = sp;return ($125|0);
}
function _fts3CompareElemByTerm($lhs,$rhs) {
 $lhs = $lhs|0;
 $rhs = $rhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $n = 0, $n1 = 0, $n2 = 0, $z1 = 0, $z2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $z1 = sp + 20|0;
 $z2 = sp + 16|0;
 $n1 = sp + 12|0;
 $n2 = sp + 8|0;
 $n = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $lhs;
 HEAP32[$1>>2] = $rhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$z1>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$z2>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$n1>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$n2>>2] = $17;
 $18 = HEAP32[$n1>>2]|0;
 $19 = HEAP32[$n2>>2]|0;
 $20 = ($18|0)<($19|0);
 $21 = HEAP32[$n1>>2]|0;
 $22 = HEAP32[$n2>>2]|0;
 $23 = $20 ? $21 : $22;
 HEAP32[$n>>2] = $23;
 $24 = HEAP32[$z1>>2]|0;
 $25 = HEAP32[$z2>>2]|0;
 $26 = HEAP32[$n>>2]|0;
 $27 = (_memcmp($24,$25,$26)|0);
 HEAP32[$c>>2] = $27;
 $28 = HEAP32[$c>>2]|0;
 $29 = ($28|0)==(0);
 if (!($29)) {
  $33 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $30 = HEAP32[$n1>>2]|0;
 $31 = HEAP32[$n2>>2]|0;
 $32 = (($30) - ($31))|0;
 HEAP32[$c>>2] = $32;
 $33 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3Fts3HashFindElem($pH,$pKey,$nKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $xHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $h = sp + 4|0;
 $xHash = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $nKey;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 16|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if (!($9)) {
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP8[$10>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = (_ftsHashFunction($12)|0);
   HEAP32[$xHash>>2] = $13;
   $14 = HEAP32[$xHash>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$3>>2]|0;
   $17 = (FUNCTION_TABLE_iii[$14 & 255]($15,$16)|0);
   HEAP32[$h>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = HEAP32[$h>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 12|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) - 1)|0;
   $26 = $21 & $25;
   $27 = (_fts3FindElementByHash($18,$19,$20,$26)|0);
   HEAP32[$0>>2] = $27;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _ftsHashFunction($keyClass) {
 $keyClass = $keyClass|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $keyClass;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(1);
 if ($3) {
  HEAP32[$0>>2] = 197;
 } else {
  HEAP32[$0>>2] = 198;
 }
 $4 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3FindElementByHash($pH,$pKey,$nKey,$h) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $count = 0, $elem = 0, $pEntry = 0, $xCompare = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $elem = sp + 12|0;
 $count = sp + 8|0;
 $xCompare = sp + 4|0;
 $pEntry = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $nKey;
 HEAP32[$4>>2] = $h;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 L1: do {
  if ($8) {
   $9 = HEAP32[$4>>2]|0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 16|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + ($9<<3)|0);
   HEAP32[$pEntry>>2] = $13;
   $14 = HEAP32[$pEntry>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$elem>>2] = $16;
   $17 = HEAP32[$pEntry>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$count>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP8[$19>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = (_ftsCompareFunction($21)|0);
   HEAP32[$xCompare>>2] = $22;
   while(1) {
    $23 = HEAP32[$count>>2]|0;
    $24 = (($23) + -1)|0;
    HEAP32[$count>>2] = $24;
    $25 = ($23|0)!=(0);
    $26 = HEAP32[$elem>>2]|0;
    $27 = ($26|0)!=(0|0);
    $28 = $25 ? $27 : 0;
    if (!($28)) {
     break L1;
    }
    $29 = HEAP32[$xCompare>>2]|0;
    $30 = HEAP32[$elem>>2]|0;
    $31 = ((($30)) + 12|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$elem>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    $37 = HEAP32[$3>>2]|0;
    $38 = (FUNCTION_TABLE_iiiii[$29 & 255]($32,$35,$36,$37)|0);
    $39 = ($38|0)==(0);
    $40 = HEAP32[$elem>>2]|0;
    if ($39) {
     break;
    }
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$elem>>2] = $41;
   }
   HEAP32[$0>>2] = $40;
   $42 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($42|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $42 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _fts3StrHash($pKey,$nKey) {
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $h = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $z = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $pKey;
 HEAP32[$1>>2] = $nKey;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 HEAP32[$h>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)<=(0);
 if ($4) {
  $5 = HEAP32[$z>>2]|0;
  $6 = (_strlen($5)|0);
  HEAP32[$1>>2] = $6;
 }
 while(1) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ($7|0)>(0);
  $9 = HEAP32[$h>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = $9 << 3;
  $11 = HEAP32[$h>>2]|0;
  $12 = $10 ^ $11;
  $13 = HEAP32[$z>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP32[$z>>2] = $14;
  $15 = HEAP8[$13>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = $12 ^ $16;
  HEAP32[$h>>2] = $17;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + -1)|0;
  HEAP32[$1>>2] = $19;
 }
 $20 = $9 & 2147483647;
 STACKTOP = sp;return ($20|0);
}
function _fts3BinHash($pKey,$nKey) {
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $h = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $pKey;
 HEAP32[$1>>2] = $nKey;
 HEAP32[$h>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = (($3) + -1)|0;
  HEAP32[$1>>2] = $4;
  $5 = ($3|0)>(0);
  $6 = HEAP32[$h>>2]|0;
  if (!($5)) {
   break;
  }
  $7 = $6 << 3;
  $8 = HEAP32[$h>>2]|0;
  $9 = $7 ^ $8;
  $10 = HEAP32[$z>>2]|0;
  $11 = ((($10)) + 1|0);
  HEAP32[$z>>2] = $11;
  $12 = HEAP8[$10>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = $9 ^ $13;
  HEAP32[$h>>2] = $14;
 }
 $15 = $6 & 2147483647;
 STACKTOP = sp;return ($15|0);
}
function _ftsCompareFunction($keyClass) {
 $keyClass = $keyClass|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $keyClass;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(1);
 if ($3) {
  HEAP32[$0>>2] = 142;
 } else {
  HEAP32[$0>>2] = 143;
 }
 $4 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3StrCompare($pKey1,$n1,$pKey2,$n2) {
 $pKey1 = $pKey1|0;
 $n1 = $n1|0;
 $pKey2 = $pKey2|0;
 $n2 = $n2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $n1;
 HEAP32[$3>>2] = $pKey2;
 HEAP32[$4>>2] = $n2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($5|0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_strncmp($8,$9,$10)|0);
  HEAP32[$0>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _fts3BinCompare($pKey1,$n1,$pKey2,$n2) {
 $pKey1 = $pKey1|0;
 $n1 = $n1|0;
 $pKey2 = $pKey2|0;
 $n2 = $n2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $n1;
 HEAP32[$3>>2] = $pKey2;
 HEAP32[$4>>2] = $n2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($5|0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_memcmp($8,$9,$10)|0);
  HEAP32[$0>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _fts3SqlStmt($p,$eStmt,$pp,$apVal) {
 $p = $p|0;
 $eStmt = $eStmt|0;
 $pp = $pp|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $azSql = 0, $i = 0, $nParam = 0, $pStmt = 0, $rc = 0;
 var $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr9 = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 228|0;
 $1 = sp + 224|0;
 $2 = sp + 220|0;
 $3 = sp + 216|0;
 $azSql = sp + 56|0;
 $rc = sp + 48|0;
 $pStmt = sp + 44|0;
 $zSql = sp + 40|0;
 $i = sp + 36|0;
 $nParam = sp + 32|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eStmt;
 HEAP32[$2>>2] = $pp;
 HEAP32[$3>>2] = $apVal;
 _memcpy(($azSql|0),(8924|0),160)|0;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = (($6) + ($4<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pStmt>>2] = $8;
 $9 = HEAP32[$pStmt>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if (!($10)) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ($11|0)==(18);
   $13 = HEAP32[$1>>2]|0;
   do {
    if ($12) {
     $14 = (($azSql) + ($13<<2)|0);
     $15 = HEAP32[$14>>2]|0;
     $16 = HEAP32[$0>>2]|0;
     $17 = ((($16)) + 16|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = HEAP32[$0>>2]|0;
     $20 = ((($19)) + 20|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$0>>2]|0;
     $23 = ((($22)) + 220|0);
     $24 = HEAP32[$23>>2]|0;
     HEAP32[$vararg_buffer>>2] = $18;
     $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr1>>2] = $21;
     $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
     HEAP32[$vararg_ptr2>>2] = $24;
     $25 = (_sqlite3_mprintf($15,$vararg_buffer)|0);
     HEAP32[$zSql>>2] = $25;
    } else {
     $26 = ($13|0)==(7);
     $27 = HEAP32[$1>>2]|0;
     $28 = (($azSql) + ($27<<2)|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$0>>2]|0;
     if ($26) {
      $31 = ((($30)) + 216|0);
      $32 = HEAP32[$31>>2]|0;
      HEAP32[$vararg_buffer3>>2] = $32;
      $33 = (_sqlite3_mprintf($29,$vararg_buffer3)|0);
      HEAP32[$zSql>>2] = $33;
      break;
     } else {
      $34 = ((($30)) + 16|0);
      $35 = HEAP32[$34>>2]|0;
      $36 = HEAP32[$0>>2]|0;
      $37 = ((($36)) + 20|0);
      $38 = HEAP32[$37>>2]|0;
      HEAP32[$vararg_buffer6>>2] = $35;
      $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
      HEAP32[$vararg_ptr9>>2] = $38;
      $39 = (_sqlite3_mprintf($29,$vararg_buffer6)|0);
      HEAP32[$zSql>>2] = $39;
      break;
     }
    }
   } while(0);
   $40 = HEAP32[$zSql>>2]|0;
   $41 = ($40|0)!=(0|0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = HEAP32[$zSql>>2]|0;
    $46 = (_sqlite3_prepare_v2($44,$45,-1,$pStmt,0)|0);
    HEAP32[$rc>>2] = $46;
    $47 = HEAP32[$zSql>>2]|0;
    _sqlite3_free($47);
    $48 = HEAP32[$pStmt>>2]|0;
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$0>>2]|0;
    $51 = ((($50)) + 56|0);
    $52 = (($51) + ($49<<2)|0);
    HEAP32[$52>>2] = $48;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $53 = HEAP32[$3>>2]|0;
 $54 = ($53|0)!=(0|0);
 if (!($54)) {
  $72 = HEAP32[$pStmt>>2]|0;
  $73 = HEAP32[$2>>2]|0;
  HEAP32[$73>>2] = $72;
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 $55 = HEAP32[$pStmt>>2]|0;
 $56 = (_sqlite3_bind_parameter_count($55)|0);
 HEAP32[$nParam>>2] = $56;
 HEAP32[$i>>2] = 0;
 while(1) {
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  if (!($58)) {
   label = 15;
   break;
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$nParam>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   label = 15;
   break;
  }
  $62 = HEAP32[$pStmt>>2]|0;
  $63 = HEAP32[$i>>2]|0;
  $64 = (($63) + 1)|0;
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$3>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (_sqlite3_bind_value($62,$64,$68)|0);
  HEAP32[$rc>>2] = $69;
  $70 = HEAP32[$i>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$i>>2] = $71;
 }
 if ((label|0) == 15) {
  $72 = HEAP32[$pStmt>>2]|0;
  $73 = HEAP32[$2>>2]|0;
  HEAP32[$73>>2] = $72;
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 return (0)|0;
}
function _getAbsoluteLevel($p,$iLangid,$iIndex,$iLevel) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iBase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $iBase = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $6 = $5 << 31 >> 31;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 248|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (___muldi3(($4|0),($6|0),($9|0),($11|0))|0);
 $13 = tempRet0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)<(0);
 $16 = $15 << 31 >> 31;
 $17 = (_i64Add(($12|0),($13|0),($14|0),($16|0))|0);
 $18 = tempRet0;
 $19 = (___muldi3(($17|0),($18|0),1024,0)|0);
 $20 = tempRet0;
 $21 = $iBase;
 $22 = $21;
 HEAP32[$22>>2] = $19;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = $iBase;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$3>>2]|0;
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = (_i64Add(($27|0),($30|0),($31|0),($33|0))|0);
 $35 = tempRet0;
 tempRet0 = ($35);
 STACKTOP = sp;return ($34|0);
}
function _sqlite3Fts3GetVarint32($p,$pi) {
 $p = $p|0;
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $a = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pi;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 1|0);
 HEAP32[$1>>2] = $4;
 $5 = HEAP8[$3>>0]|0;
 $6 = $5 << 24 >> 24;
 HEAP32[$a>>2] = $6;
 $7 = HEAP32[$a>>2]|0;
 $8 = $7 & 127;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 1|0);
 HEAP32[$1>>2] = $10;
 $11 = HEAP8[$9>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = $12 << 7;
 $14 = $8 | $13;
 HEAP32[$a>>2] = $14;
 $15 = HEAP32[$a>>2]|0;
 $16 = $15 & 16384;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$a>>2]|0;
 if ($17) {
  $19 = HEAP32[$2>>2]|0;
  HEAP32[$19>>2] = $18;
  HEAP32[$0>>2] = 2;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $20 = $18 & 16383;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 1|0);
 HEAP32[$1>>2] = $22;
 $23 = HEAP8[$21>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = $24 << 14;
 $26 = $20 | $25;
 HEAP32[$a>>2] = $26;
 $27 = HEAP32[$a>>2]|0;
 $28 = $27 & 2097152;
 $29 = ($28|0)==(0);
 $30 = HEAP32[$a>>2]|0;
 if ($29) {
  $31 = HEAP32[$2>>2]|0;
  HEAP32[$31>>2] = $30;
  HEAP32[$0>>2] = 3;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $32 = $30 & 2097151;
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 1|0);
 HEAP32[$1>>2] = $34;
 $35 = HEAP8[$33>>0]|0;
 $36 = $35 << 24 >> 24;
 $37 = $36 << 21;
 $38 = $32 | $37;
 HEAP32[$a>>2] = $38;
 $39 = HEAP32[$a>>2]|0;
 $40 = $39 & 268435456;
 $41 = ($40|0)==(0);
 $42 = HEAP32[$a>>2]|0;
 if ($41) {
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  HEAP32[$0>>2] = 4;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 } else {
  $44 = $42 & 268435455;
  HEAP32[$a>>2] = $44;
  $45 = HEAP32[$a>>2]|0;
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47 << 24 >> 24;
  $49 = $48 & 15;
  $50 = $49 << 28;
  $51 = $45 | $50;
  $52 = HEAP32[$2>>2]|0;
  HEAP32[$52>>2] = $51;
  HEAP32[$0>>2] = 5;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 return (0)|0;
}
function _fts3ScanInteriorNode($zTerm,$nTerm,$zNode,$nNode,$piFirst,$piLast) {
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $zNode = $zNode|0;
 $nNode = $nNode|0;
 $piFirst = $piFirst|0;
 $piLast = $piLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cmp = 0, $iChild = 0, $isFirstTerm = 0, $nAlloc = 0;
 var $nBuffer = 0, $nPrefix = 0, $nSuffix = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $zBuffer = 0, $zCsr = 0, $zEnd = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $rc = sp + 48|0;
 $zCsr = sp + 44|0;
 $zEnd = sp + 40|0;
 $zBuffer = sp + 36|0;
 $nAlloc = sp + 32|0;
 $isFirstTerm = sp + 28|0;
 $iChild = sp;
 $cmp = sp + 24|0;
 $nSuffix = sp + 20|0;
 $nPrefix = sp + 16|0;
 $nBuffer = sp + 12|0;
 $zNew = sp + 8|0;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $zNode;
 HEAP32[$4>>2] = $nNode;
 HEAP32[$5>>2] = $piFirst;
 HEAP32[$6>>2] = $piLast;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$zCsr>>2] = $7;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$zCsr>>2]|0;
 $10 = (($9) + ($8)|0);
 HEAP32[$zEnd>>2] = $10;
 HEAP32[$zBuffer>>2] = 0;
 HEAP32[$nAlloc>>2] = 0;
 HEAP32[$isFirstTerm>>2] = 1;
 $11 = HEAP32[$zCsr>>2]|0;
 $12 = (_sqlite3Fts3GetVarint($11,$iChild)|0);
 $13 = HEAP32[$zCsr>>2]|0;
 $14 = (($13) + ($12)|0);
 HEAP32[$zCsr>>2] = $14;
 $15 = HEAP32[$zCsr>>2]|0;
 $16 = (_sqlite3Fts3GetVarint($15,$iChild)|0);
 $17 = HEAP32[$zCsr>>2]|0;
 $18 = (($17) + ($16)|0);
 HEAP32[$zCsr>>2] = $18;
 $19 = HEAP32[$zCsr>>2]|0;
 $20 = HEAP32[$zEnd>>2]|0;
 $21 = ($19>>>0)>($20>>>0);
 if ($21) {
  HEAP32[$0>>2] = 267;
  $175 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($175|0);
 }
 while(1) {
  $22 = HEAP32[$zCsr>>2]|0;
  $23 = HEAP32[$zEnd>>2]|0;
  $24 = ($22>>>0)<($23>>>0);
  if (!($24)) {
   label = 28;
   break;
  }
  $25 = HEAP32[$5>>2]|0;
  $26 = ($25|0)!=(0|0);
  $27 = HEAP32[$6>>2]|0;
  $28 = ($27|0)!=(0|0);
  $29 = $26 ? 1 : $28;
  if (!($29)) {
   label = 28;
   break;
  }
  HEAP32[$nPrefix>>2] = 0;
  $30 = HEAP32[$isFirstTerm>>2]|0;
  $31 = ($30|0)!=(0);
  if (!($31)) {
   $32 = HEAP32[$zCsr>>2]|0;
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = $34 & 128;
   $36 = ($35|0)!=(0);
   $37 = HEAP32[$zCsr>>2]|0;
   if ($36) {
    $38 = (_sqlite3Fts3GetVarint32($37,$nPrefix)|0);
    $43 = $38;
   } else {
    $39 = HEAP8[$37>>0]|0;
    $40 = $39&255;
    HEAP32[$nPrefix>>2] = $40;
    $43 = 1;
   }
   $41 = HEAP32[$zCsr>>2]|0;
   $42 = (($41) + ($43)|0);
   HEAP32[$zCsr>>2] = $42;
  }
  HEAP32[$isFirstTerm>>2] = 0;
  $44 = HEAP32[$zCsr>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46 & 128;
  $48 = ($47|0)!=(0);
  $49 = HEAP32[$zCsr>>2]|0;
  if ($48) {
   $50 = (_sqlite3Fts3GetVarint32($49,$nSuffix)|0);
   $55 = $50;
  } else {
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   HEAP32[$nSuffix>>2] = $52;
   $55 = 1;
  }
  $53 = HEAP32[$zCsr>>2]|0;
  $54 = (($53) + ($55)|0);
  HEAP32[$zCsr>>2] = $54;
  $56 = HEAP32[$nPrefix>>2]|0;
  $57 = ($56|0)<(0);
  $58 = HEAP32[$nSuffix>>2]|0;
  $59 = ($58|0)<(0);
  $or$cond = $57 | $59;
  if ($or$cond) {
   label = 15;
   break;
  }
  $60 = HEAP32[$nSuffix>>2]|0;
  $61 = HEAP32[$zCsr>>2]|0;
  $62 = (($61) + ($60)|0);
  $63 = HEAP32[$zEnd>>2]|0;
  $64 = ($62>>>0)>($63>>>0);
  if ($64) {
   label = 15;
   break;
  }
  $65 = HEAP32[$nPrefix>>2]|0;
  $66 = HEAP32[$nSuffix>>2]|0;
  $67 = (($65) + ($66))|0;
  $68 = HEAP32[$nAlloc>>2]|0;
  $69 = ($67|0)>($68|0);
  if ($69) {
   $70 = HEAP32[$nPrefix>>2]|0;
   $71 = HEAP32[$nSuffix>>2]|0;
   $72 = (($70) + ($71))|0;
   $73 = $72<<1;
   HEAP32[$nAlloc>>2] = $73;
   $74 = HEAP32[$zBuffer>>2]|0;
   $75 = HEAP32[$nAlloc>>2]|0;
   $76 = (_sqlite3_realloc($74,$75)|0);
   HEAP32[$zNew>>2] = $76;
   $77 = HEAP32[$zNew>>2]|0;
   $78 = ($77|0)!=(0|0);
   if (!($78)) {
    label = 18;
    break;
   }
   $79 = HEAP32[$zNew>>2]|0;
   HEAP32[$zBuffer>>2] = $79;
  }
  $80 = HEAP32[$nPrefix>>2]|0;
  $81 = HEAP32[$zBuffer>>2]|0;
  $82 = (($81) + ($80)|0);
  $83 = HEAP32[$zCsr>>2]|0;
  $84 = HEAP32[$nSuffix>>2]|0;
  _memcpy(($82|0),($83|0),($84|0))|0;
  $85 = HEAP32[$nPrefix>>2]|0;
  $86 = HEAP32[$nSuffix>>2]|0;
  $87 = (($85) + ($86))|0;
  HEAP32[$nBuffer>>2] = $87;
  $88 = HEAP32[$nSuffix>>2]|0;
  $89 = HEAP32[$zCsr>>2]|0;
  $90 = (($89) + ($88)|0);
  HEAP32[$zCsr>>2] = $90;
  $91 = HEAP32[$1>>2]|0;
  $92 = HEAP32[$zBuffer>>2]|0;
  $93 = HEAP32[$nBuffer>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = ($93|0)>($94|0);
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$nBuffer>>2]|0;
  $98 = $95 ? $96 : $97;
  $99 = (_memcmp($91,$92,$98)|0);
  HEAP32[$cmp>>2] = $99;
  $100 = HEAP32[$5>>2]|0;
  $101 = ($100|0)!=(0|0);
  do {
   if ($101) {
    $102 = HEAP32[$cmp>>2]|0;
    $103 = ($102|0)<(0);
    if (!($103)) {
     $104 = HEAP32[$cmp>>2]|0;
     $105 = ($104|0)==(0);
     if (!($105)) {
      break;
     }
     $106 = HEAP32[$nBuffer>>2]|0;
     $107 = HEAP32[$2>>2]|0;
     $108 = ($106|0)>($107|0);
     if (!($108)) {
      break;
     }
    }
    $109 = $iChild;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = HEAP32[$5>>2]|0;
    $116 = $115;
    $117 = $116;
    HEAP32[$117>>2] = $111;
    $118 = (($116) + 4)|0;
    $119 = $118;
    HEAP32[$119>>2] = $114;
    HEAP32[$5>>2] = 0;
   }
  } while(0);
  $120 = HEAP32[$6>>2]|0;
  $121 = ($120|0)!=(0|0);
  $122 = HEAP32[$cmp>>2]|0;
  $123 = ($122|0)<(0);
  $or$cond3 = $121 & $123;
  if ($or$cond3) {
   $124 = $iChild;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = HEAP32[$6>>2]|0;
   $131 = $130;
   $132 = $131;
   HEAP32[$132>>2] = $126;
   $133 = (($131) + 4)|0;
   $134 = $133;
   HEAP32[$134>>2] = $129;
   HEAP32[$6>>2] = 0;
  }
  $135 = $iChild;
  $136 = $135;
  $137 = HEAP32[$136>>2]|0;
  $138 = (($135) + 4)|0;
  $139 = $138;
  $140 = HEAP32[$139>>2]|0;
  $141 = (_i64Add(($137|0),($140|0),1,0)|0);
  $142 = tempRet0;
  $143 = $iChild;
  $144 = $143;
  HEAP32[$144>>2] = $141;
  $145 = (($143) + 4)|0;
  $146 = $145;
  HEAP32[$146>>2] = $142;
 }
 if ((label|0) == 15) {
  HEAP32[$rc>>2] = 267;
 }
 else if ((label|0) == 18) {
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 28) {
  $147 = HEAP32[$5>>2]|0;
  $148 = ($147|0)!=(0|0);
  if ($148) {
   $149 = $iChild;
   $150 = $149;
   $151 = HEAP32[$150>>2]|0;
   $152 = (($149) + 4)|0;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = HEAP32[$5>>2]|0;
   $156 = $155;
   $157 = $156;
   HEAP32[$157>>2] = $151;
   $158 = (($156) + 4)|0;
   $159 = $158;
   HEAP32[$159>>2] = $154;
  }
  $160 = HEAP32[$6>>2]|0;
  $161 = ($160|0)!=(0|0);
  if ($161) {
   $162 = $iChild;
   $163 = $162;
   $164 = HEAP32[$163>>2]|0;
   $165 = (($162) + 4)|0;
   $166 = $165;
   $167 = HEAP32[$166>>2]|0;
   $168 = HEAP32[$6>>2]|0;
   $169 = $168;
   $170 = $169;
   HEAP32[$170>>2] = $164;
   $171 = (($169) + 4)|0;
   $172 = $171;
   HEAP32[$172>>2] = $167;
  }
 }
 $173 = HEAP32[$zBuffer>>2]|0;
 _sqlite3_free($173);
 $174 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $174;
 $175 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($175|0);
}
function _sqlite3Fts3ReadBlock($p,$0,$1,$paBlob,$pnBlob,$pnLoad) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $paBlob = $paBlob|0;
 $pnBlob = $pnBlob|0;
 $pnLoad = $pnLoad|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aByte = 0, $nByte = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $rc = sp + 20|0;
 $nByte = sp + 16|0;
 $aByte = sp + 12|0;
 HEAP32[$3>>2] = $p;
 $8 = $4;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$5>>2] = $paBlob;
 HEAP32[$6>>2] = $pnBlob;
 HEAP32[$7>>2] = $pnLoad;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 244|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0|0);
 $16 = HEAP32[$3>>2]|0;
 if ($15) {
  $17 = ((($16)) + 244|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3_blob_reopen($18,$21,$24)|0);
  HEAP32[$rc>>2] = $25;
 } else {
  $26 = ((($16)) + 240|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (0|0)==($27|0);
  if ($28) {
   $29 = HEAP32[$3>>2]|0;
   $30 = ((($29)) + 20|0);
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$vararg_buffer>>2] = $31;
   $32 = (_sqlite3_mprintf(42541,$vararg_buffer)|0);
   $33 = HEAP32[$3>>2]|0;
   $34 = ((($33)) + 240|0);
   HEAP32[$34>>2] = $32;
   $35 = HEAP32[$3>>2]|0;
   $36 = ((($35)) + 240|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (0|0)==($37|0);
   if ($38) {
    HEAP32[$2>>2] = 7;
    $93 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($93|0);
   }
  }
  $39 = HEAP32[$3>>2]|0;
  $40 = ((($39)) + 12|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$3>>2]|0;
  $46 = ((($45)) + 240|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $4;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($48) + 4)|0;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  $55 = ((($54)) + 244|0);
  $56 = (_sqlite3_blob_open($41,$44,$47,42553,$50,$53,0,$55)|0);
  HEAP32[$rc>>2] = $56;
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$3>>2]|0;
  $60 = ((($59)) + 244|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (_sqlite3_blob_bytes($61)|0);
  HEAP32[$nByte>>2] = $62;
  $63 = HEAP32[$nByte>>2]|0;
  $64 = HEAP32[$6>>2]|0;
  HEAP32[$64>>2] = $63;
  $65 = HEAP32[$5>>2]|0;
  $66 = ($65|0)!=(0|0);
  if ($66) {
   $67 = HEAP32[$nByte>>2]|0;
   $68 = (($67) + 20)|0;
   $69 = (_sqlite3_malloc($68)|0);
   HEAP32[$aByte>>2] = $69;
   $70 = HEAP32[$aByte>>2]|0;
   $71 = ($70|0)!=(0|0);
   if ($71) {
    $72 = HEAP32[$7>>2]|0;
    $73 = ($72|0)!=(0|0);
    $74 = HEAP32[$nByte>>2]|0;
    $75 = ($74|0)>(16384);
    $or$cond = $73 & $75;
    if ($or$cond) {
     HEAP32[$nByte>>2] = 4096;
     $76 = HEAP32[$nByte>>2]|0;
     $77 = HEAP32[$7>>2]|0;
     HEAP32[$77>>2] = $76;
    }
    $78 = HEAP32[$3>>2]|0;
    $79 = ((($78)) + 244|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = HEAP32[$aByte>>2]|0;
    $82 = HEAP32[$nByte>>2]|0;
    $83 = (_sqlite3_blob_read($80,$81,$82,0)|0);
    HEAP32[$rc>>2] = $83;
    $84 = HEAP32[$nByte>>2]|0;
    $85 = HEAP32[$aByte>>2]|0;
    $86 = (($85) + ($84)|0);
    dest=$86; stop=dest+20|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
    $87 = HEAP32[$rc>>2]|0;
    $88 = ($87|0)!=(0);
    if ($88) {
     $89 = HEAP32[$aByte>>2]|0;
     _sqlite3_free($89);
     HEAP32[$aByte>>2] = 0;
    }
   } else {
    HEAP32[$rc>>2] = 7;
   }
   $90 = HEAP32[$aByte>>2]|0;
   $91 = HEAP32[$5>>2]|0;
   HEAP32[$91>>2] = $90;
  }
 }
 $92 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $92;
 $93 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3Fts3GetVarint($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $c = 0, $pStart = 0, $shift = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pStart = sp + 24|0;
 $a = sp + 20|0;
 $b = sp + 8|0;
 $shift = sp + 16|0;
 $c = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pStart>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 1|0);
 HEAP32[$1>>2] = $5;
 $6 = HEAP8[$4>>0]|0;
 $7 = $6 << 24 >> 24;
 HEAP32[$a>>2] = $7;
 $8 = HEAP32[$a>>2]|0;
 $9 = $8 & 128;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$a>>2]|0;
 if ($10) {
  $12 = HEAP32[$2>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $11;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = 0;
  HEAP32[$0>>2] = 1;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $17 = $11 & 127;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 1|0);
 HEAP32[$1>>2] = $19;
 $20 = HEAP8[$18>>0]|0;
 $21 = $20 << 24 >> 24;
 $22 = $21 << 7;
 $23 = $17 | $22;
 HEAP32[$a>>2] = $23;
 $24 = HEAP32[$a>>2]|0;
 $25 = $24 & 16384;
 $26 = ($25|0)==(0);
 $27 = HEAP32[$a>>2]|0;
 if ($26) {
  $28 = HEAP32[$2>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = 0;
  HEAP32[$0>>2] = 2;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $33 = $27 & 16383;
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 1|0);
 HEAP32[$1>>2] = $35;
 $36 = HEAP8[$34>>0]|0;
 $37 = $36 << 24 >> 24;
 $38 = $37 << 14;
 $39 = $33 | $38;
 HEAP32[$a>>2] = $39;
 $40 = HEAP32[$a>>2]|0;
 $41 = $40 & 2097152;
 $42 = ($41|0)==(0);
 $43 = HEAP32[$a>>2]|0;
 if ($42) {
  $44 = HEAP32[$2>>2]|0;
  $45 = $44;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = 0;
  HEAP32[$0>>2] = 3;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $49 = $43 & 2097151;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 1|0);
 HEAP32[$1>>2] = $51;
 $52 = HEAP8[$50>>0]|0;
 $53 = $52 << 24 >> 24;
 $54 = $53 << 21;
 $55 = $49 | $54;
 HEAP32[$a>>2] = $55;
 $56 = HEAP32[$a>>2]|0;
 $57 = $56 & 268435456;
 $58 = ($57|0)==(0);
 $59 = HEAP32[$a>>2]|0;
 if ($58) {
  $60 = HEAP32[$2>>2]|0;
  $61 = $60;
  $62 = $61;
  HEAP32[$62>>2] = $59;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = 0;
  HEAP32[$0>>2] = 4;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $65 = $59 & 268435455;
 $66 = $b;
 $67 = $66;
 HEAP32[$67>>2] = $65;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = 0;
 HEAP32[$shift>>2] = 28;
 while(1) {
  $70 = HEAP32[$shift>>2]|0;
  $71 = ($70|0)<=(63);
  if (!($71)) {
   break;
  }
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 1|0);
  HEAP32[$1>>2] = $73;
  $74 = HEAP8[$72>>0]|0;
  $75 = $74 << 24 >> 24;
  $76 = ($75|0)<(0);
  $77 = $76 << 31 >> 31;
  $78 = $c;
  $79 = $78;
  HEAP32[$79>>2] = $75;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = $c;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = (($82) + 4)|0;
  $86 = $85;
  $87 = HEAP32[$86>>2]|0;
  $88 = $84 & 127;
  $89 = HEAP32[$shift>>2]|0;
  $90 = (_bitshift64Shl(($88|0),0,($89|0))|0);
  $91 = tempRet0;
  $92 = $b;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = (_i64Add(($94|0),($97|0),($90|0),($91|0))|0);
  $99 = tempRet0;
  $100 = $b;
  $101 = $100;
  HEAP32[$101>>2] = $98;
  $102 = (($100) + 4)|0;
  $103 = $102;
  HEAP32[$103>>2] = $99;
  $104 = $c;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = $106 & 128;
  $111 = ($110|0)==(0);
  $112 = (0)==(0);
  $113 = $111 & $112;
  if ($113) {
   break;
  }
  $114 = HEAP32[$shift>>2]|0;
  $115 = (($114) + 7)|0;
  HEAP32[$shift>>2] = $115;
 }
 $116 = $b;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = (($116) + 4)|0;
 $120 = $119;
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$2>>2]|0;
 $123 = $122;
 $124 = $123;
 HEAP32[$124>>2] = $118;
 $125 = (($123) + 4)|0;
 $126 = $125;
 HEAP32[$126>>2] = $121;
 $127 = HEAP32[$1>>2]|0;
 $128 = HEAP32[$pStart>>2]|0;
 $129 = $127;
 $130 = $128;
 $131 = (($129) - ($130))|0;
 HEAP32[$0>>2] = $131;
 $132 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _fts3SegReaderStart($p,$pCsr,$zTerm,$nTerm) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nSeg = 0, $or$cond = 0, $pSeg = 0, $rc = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $i = sp + 16|0;
 $nSeg = sp + 12|0;
 $res = sp + 8|0;
 $pSeg = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$3>>2] = $zTerm;
 HEAP32[$4>>2] = $nTerm;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nSeg>>2] = $7;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $8 = HEAP32[$2>>2]|0;
  $9 = ((($8)) + 28|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)==(0);
  if (!($11)) {
   label = 12;
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 12;
   break;
  }
  HEAP32[$res>>2] = 0;
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($17<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pSeg>>2] = $21;
  while(1) {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$pSeg>>2]|0;
   $24 = (_fts3SegReaderNext($22,$23,0)|0);
   HEAP32[$rc>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)!=(0);
   if ($26) {
    label = 6;
    break L1;
   }
   $28 = HEAP32[$3>>2]|0;
   $29 = ($28|0)!=(0|0);
   if (!($29)) {
    break;
   }
   $30 = HEAP32[$pSeg>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = (_fts3SegReaderTermCmp($30,$31,$32)|0);
   HEAP32[$res>>2] = $33;
   $34 = ($33|0)<(0);
   if (!($34)) {
    break;
   }
  }
  $35 = HEAP32[$pSeg>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = $37&255;
  $39 = ($38|0)!=(0);
  $40 = HEAP32[$res>>2]|0;
  $41 = ($40|0)!=(0);
  $or$cond = $39 & $41;
  if ($or$cond) {
   $42 = HEAP32[$pSeg>>2]|0;
   _fts3SegReaderSetEof($42);
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 if ((label|0) == 6) {
  $27 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $27;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 else if ((label|0) == 12) {
  $45 = HEAP32[$2>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$nSeg>>2]|0;
  $48 = HEAP32[$nSeg>>2]|0;
  _fts3SegReaderSort($46,$47,$48,199);
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 return (0)|0;
}
function _fts3SegReaderNext($p,$pReader,$bIncr) {
 $p = $p|0;
 $pReader = $pReader|0;
 $bIncr = $bIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCopy = 0, $nCopy = 0, $nNew = 0, $nPrefix = 0, $nSuffix = 0, $or$cond = 0;
 var $pElem = 0, $pList = 0, $pNext = 0, $rc = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $rc = sp + 36|0;
 $pNext = sp + 32|0;
 $nPrefix = sp + 28|0;
 $nSuffix = sp + 24|0;
 $pElem = sp + 20|0;
 $aCopy = sp + 16|0;
 $pList = sp + 12|0;
 $nCopy = sp + 8|0;
 $nNew = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pReader;
 HEAP32[$3>>2] = $bIncr;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 72|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  $11 = ((($8)) + 76|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12)|0);
  HEAP32[$pNext>>2] = $16;
 } else {
  $9 = ((($8)) + 40|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pNext>>2] = $10;
 }
 $17 = HEAP32[$pNext>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = HEAP32[$pNext>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 44|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 40|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($22)|0);
  $27 = ($19>>>0)>=($26>>>0);
  if ($27) {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 56|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  $32 = HEAP32[$2>>2]|0;
  if ($31) {
   $33 = ((($32)) + 56|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pElem>>2] = $35;
   $36 = HEAP32[$2>>2]|0;
   $37 = ((($36)) + 40|0);
   $38 = HEAP32[$37>>2]|0;
   _sqlite3_free($38);
   $39 = HEAP32[$2>>2]|0;
   $40 = ((($39)) + 40|0);
   HEAP32[$40>>2] = 0;
   $41 = HEAP32[$pElem>>2]|0;
   $42 = ($41|0)!=(0|0);
   do {
    if ($42) {
     $43 = HEAP32[$pElem>>2]|0;
     $44 = ((($43)) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     HEAP32[$pList>>2] = $45;
     $46 = HEAP32[$pList>>2]|0;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($47) + 1)|0;
     HEAP32[$nCopy>>2] = $48;
     $49 = HEAP32[$pElem>>2]|0;
     $50 = ((($49)) + 12|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = HEAP32[$2>>2]|0;
     $53 = ((($52)) + 64|0);
     HEAP32[$53>>2] = $51;
     $54 = HEAP32[$pElem>>2]|0;
     $55 = ((($54)) + 16|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = HEAP32[$2>>2]|0;
     $58 = ((($57)) + 60|0);
     HEAP32[$58>>2] = $56;
     $59 = HEAP32[$nCopy>>2]|0;
     $60 = (_sqlite3_malloc($59)|0);
     HEAP32[$aCopy>>2] = $60;
     $61 = HEAP32[$aCopy>>2]|0;
     $62 = ($61|0)!=(0|0);
     if ($62) {
      $63 = HEAP32[$aCopy>>2]|0;
      $64 = HEAP32[$pList>>2]|0;
      $65 = ((($64)) + 4|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$nCopy>>2]|0;
      _memcpy(($63|0),($66|0),($67|0))|0;
      $68 = HEAP32[$nCopy>>2]|0;
      $69 = HEAP32[$2>>2]|0;
      $70 = ((($69)) + 76|0);
      HEAP32[$70>>2] = $68;
      $71 = HEAP32[$2>>2]|0;
      $72 = ((($71)) + 44|0);
      HEAP32[$72>>2] = $68;
      $73 = HEAP32[$aCopy>>2]|0;
      $74 = HEAP32[$2>>2]|0;
      $75 = ((($74)) + 72|0);
      HEAP32[$75>>2] = $73;
      $76 = HEAP32[$2>>2]|0;
      $77 = ((($76)) + 40|0);
      HEAP32[$77>>2] = $73;
      $78 = HEAP32[$2>>2]|0;
      $79 = ((($78)) + 56|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = ((($80)) + 4|0);
      HEAP32[$79>>2] = $81;
      break;
     }
     HEAP32[$0>>2] = 7;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
   } while(0);
   HEAP32[$0>>2] = 0;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  _fts3SegReaderSetEof($32);
  $82 = HEAP32[$2>>2]|0;
  $83 = ((($82)) + 32|0);
  $84 = $83;
  $85 = $84;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($84) + 4)|0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$2>>2]|0;
  $91 = ((($90)) + 16|0);
  $92 = $91;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($89|0)>($97|0);
  $99 = ($86>>>0)>=($94>>>0);
  $100 = ($89|0)==($97|0);
  $101 = $100 & $99;
  $102 = $98 | $101;
  if ($102) {
   HEAP32[$0>>2] = 0;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $103 = HEAP32[$1>>2]|0;
  $104 = HEAP32[$2>>2]|0;
  $105 = ((($104)) + 32|0);
  $106 = $105;
  $107 = $106;
  $108 = HEAP32[$107>>2]|0;
  $109 = (($106) + 4)|0;
  $110 = $109;
  $111 = HEAP32[$110>>2]|0;
  $112 = (_i64Add(($108|0),($111|0),1,0)|0);
  $113 = tempRet0;
  $114 = $105;
  $115 = $114;
  HEAP32[$115>>2] = $112;
  $116 = (($114) + 4)|0;
  $117 = $116;
  HEAP32[$117>>2] = $113;
  $118 = HEAP32[$2>>2]|0;
  $119 = ((($118)) + 40|0);
  $120 = HEAP32[$2>>2]|0;
  $121 = ((($120)) + 44|0);
  $122 = HEAP32[$3>>2]|0;
  $123 = ($122|0)!=(0);
  $124 = HEAP32[$2>>2]|0;
  $125 = ((($124)) + 48|0);
  $126 = $123 ? $125 : 0;
  $127 = (_sqlite3Fts3ReadBlock($103,$112,$113,$119,$121,$126)|0);
  HEAP32[$rc>>2] = $127;
  $128 = HEAP32[$rc>>2]|0;
  $129 = ($128|0)!=(0);
  if ($129) {
   $130 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $130;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $131 = HEAP32[$3>>2]|0;
  $132 = ($131|0)!=(0);
  if ($132) {
   $133 = HEAP32[$2>>2]|0;
   $134 = ((($133)) + 48|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = HEAP32[$2>>2]|0;
   $137 = ((($136)) + 44|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = ($135|0)<($138|0);
   if ($139) {
    $140 = HEAP32[$1>>2]|0;
    $141 = ((($140)) + 244|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = HEAP32[$2>>2]|0;
    $144 = ((($143)) + 52|0);
    HEAP32[$144>>2] = $142;
    $145 = HEAP32[$1>>2]|0;
    $146 = ((($145)) + 244|0);
    HEAP32[$146>>2] = 0;
   }
  }
  $147 = HEAP32[$2>>2]|0;
  $148 = ((($147)) + 40|0);
  $149 = HEAP32[$148>>2]|0;
  HEAP32[$pNext>>2] = $149;
 }
 $150 = HEAP32[$2>>2]|0;
 $151 = HEAP32[$pNext>>2]|0;
 $152 = (_fts3SegReaderRequire($150,$151,20)|0);
 HEAP32[$rc>>2] = $152;
 $153 = HEAP32[$rc>>2]|0;
 $154 = ($153|0)!=(0);
 if ($154) {
  $155 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $155;
  $293 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($293|0);
 }
 $156 = HEAP32[$pNext>>2]|0;
 $157 = HEAP8[$156>>0]|0;
 $158 = $157&255;
 $159 = $158 & 128;
 $160 = ($159|0)!=(0);
 $161 = HEAP32[$pNext>>2]|0;
 if ($160) {
  $162 = (_sqlite3Fts3GetVarint32($161,$nPrefix)|0);
  $167 = $162;
 } else {
  $163 = HEAP8[$161>>0]|0;
  $164 = $163&255;
  HEAP32[$nPrefix>>2] = $164;
  $167 = 1;
 }
 $165 = HEAP32[$pNext>>2]|0;
 $166 = (($165) + ($167)|0);
 HEAP32[$pNext>>2] = $166;
 $168 = HEAP32[$pNext>>2]|0;
 $169 = HEAP8[$168>>0]|0;
 $170 = $169&255;
 $171 = $170 & 128;
 $172 = ($171|0)!=(0);
 $173 = HEAP32[$pNext>>2]|0;
 if ($172) {
  $174 = (_sqlite3Fts3GetVarint32($173,$nSuffix)|0);
  $179 = $174;
 } else {
  $175 = HEAP8[$173>>0]|0;
  $176 = $175&255;
  HEAP32[$nSuffix>>2] = $176;
  $179 = 1;
 }
 $177 = HEAP32[$pNext>>2]|0;
 $178 = (($177) + ($179)|0);
 HEAP32[$pNext>>2] = $178;
 $180 = HEAP32[$nPrefix>>2]|0;
 $181 = ($180|0)<(0);
 $182 = HEAP32[$nSuffix>>2]|0;
 $183 = ($182|0)<=(0);
 $or$cond = $181 | $183;
 if (!($or$cond)) {
  $184 = HEAP32[$nSuffix>>2]|0;
  $185 = HEAP32[$pNext>>2]|0;
  $186 = (($185) + ($184)|0);
  $187 = HEAP32[$2>>2]|0;
  $188 = ((($187)) + 44|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = HEAP32[$2>>2]|0;
  $191 = ((($190)) + 40|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = (($192) + ($189)|0);
  $194 = ($186>>>0)>($193>>>0);
  if (!($194)) {
   $195 = HEAP32[$nPrefix>>2]|0;
   $196 = HEAP32[$nSuffix>>2]|0;
   $197 = (($195) + ($196))|0;
   $198 = HEAP32[$2>>2]|0;
   $199 = ((($198)) + 68|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = ($197|0)>($200|0);
   do {
    if ($201) {
     $202 = HEAP32[$nPrefix>>2]|0;
     $203 = HEAP32[$nSuffix>>2]|0;
     $204 = (($202) + ($203))|0;
     $205 = $204<<1;
     HEAP32[$nNew>>2] = $205;
     $206 = HEAP32[$2>>2]|0;
     $207 = ((($206)) + 64|0);
     $208 = HEAP32[$207>>2]|0;
     $209 = HEAP32[$nNew>>2]|0;
     $210 = (_sqlite3_realloc($208,$209)|0);
     HEAP32[$zNew>>2] = $210;
     $211 = HEAP32[$zNew>>2]|0;
     $212 = ($211|0)!=(0|0);
     if ($212) {
      $213 = HEAP32[$zNew>>2]|0;
      $214 = HEAP32[$2>>2]|0;
      $215 = ((($214)) + 64|0);
      HEAP32[$215>>2] = $213;
      $216 = HEAP32[$nNew>>2]|0;
      $217 = HEAP32[$2>>2]|0;
      $218 = ((($217)) + 68|0);
      HEAP32[$218>>2] = $216;
      break;
     }
     HEAP32[$0>>2] = 7;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
   } while(0);
   $219 = HEAP32[$2>>2]|0;
   $220 = HEAP32[$pNext>>2]|0;
   $221 = HEAP32[$nSuffix>>2]|0;
   $222 = (($221) + 10)|0;
   $223 = (_fts3SegReaderRequire($219,$220,$222)|0);
   HEAP32[$rc>>2] = $223;
   $224 = HEAP32[$rc>>2]|0;
   $225 = ($224|0)!=(0);
   if ($225) {
    $226 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $226;
    $293 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($293|0);
   }
   $227 = HEAP32[$nPrefix>>2]|0;
   $228 = HEAP32[$2>>2]|0;
   $229 = ((($228)) + 64|0);
   $230 = HEAP32[$229>>2]|0;
   $231 = (($230) + ($227)|0);
   $232 = HEAP32[$pNext>>2]|0;
   $233 = HEAP32[$nSuffix>>2]|0;
   _memcpy(($231|0),($232|0),($233|0))|0;
   $234 = HEAP32[$nPrefix>>2]|0;
   $235 = HEAP32[$nSuffix>>2]|0;
   $236 = (($234) + ($235))|0;
   $237 = HEAP32[$2>>2]|0;
   $238 = ((($237)) + 60|0);
   HEAP32[$238>>2] = $236;
   $239 = HEAP32[$nSuffix>>2]|0;
   $240 = HEAP32[$pNext>>2]|0;
   $241 = (($240) + ($239)|0);
   HEAP32[$pNext>>2] = $241;
   $242 = HEAP32[$pNext>>2]|0;
   $243 = HEAP8[$242>>0]|0;
   $244 = $243&255;
   $245 = $244 & 128;
   $246 = ($245|0)!=(0);
   $247 = HEAP32[$pNext>>2]|0;
   if ($246) {
    $248 = HEAP32[$2>>2]|0;
    $249 = ((($248)) + 76|0);
    $250 = (_sqlite3Fts3GetVarint32($247,$249)|0);
    $257 = $250;
   } else {
    $251 = HEAP8[$247>>0]|0;
    $252 = $251&255;
    $253 = HEAP32[$2>>2]|0;
    $254 = ((($253)) + 76|0);
    HEAP32[$254>>2] = $252;
    $257 = 1;
   }
   $255 = HEAP32[$pNext>>2]|0;
   $256 = (($255) + ($257)|0);
   HEAP32[$pNext>>2] = $256;
   $258 = HEAP32[$pNext>>2]|0;
   $259 = HEAP32[$2>>2]|0;
   $260 = ((($259)) + 72|0);
   HEAP32[$260>>2] = $258;
   $261 = HEAP32[$2>>2]|0;
   $262 = ((($261)) + 80|0);
   HEAP32[$262>>2] = 0;
   $263 = HEAP32[$2>>2]|0;
   $264 = ((($263)) + 76|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = HEAP32[$2>>2]|0;
   $267 = ((($266)) + 72|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = (($268) + ($265)|0);
   $270 = HEAP32[$2>>2]|0;
   $271 = ((($270)) + 44|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = HEAP32[$2>>2]|0;
   $274 = ((($273)) + 40|0);
   $275 = HEAP32[$274>>2]|0;
   $276 = (($275) + ($272)|0);
   $277 = ($269>>>0)>($276>>>0);
   do {
    if (!($277)) {
     $278 = HEAP32[$2>>2]|0;
     $279 = ((($278)) + 48|0);
     $280 = HEAP32[$279>>2]|0;
     $281 = ($280|0)==(0);
     if ($281) {
      $282 = HEAP32[$2>>2]|0;
      $283 = ((($282)) + 76|0);
      $284 = HEAP32[$283>>2]|0;
      $285 = (($284) - 1)|0;
      $286 = HEAP32[$2>>2]|0;
      $287 = ((($286)) + 72|0);
      $288 = HEAP32[$287>>2]|0;
      $289 = (($288) + ($285)|0);
      $290 = HEAP8[$289>>0]|0;
      $291 = $290 << 24 >> 24;
      $292 = ($291|0)!=(0);
      if ($292) {
       break;
      }
     }
     HEAP32[$0>>2] = 0;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
   } while(0);
   HEAP32[$0>>2] = 267;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
 }
 HEAP32[$0>>2] = 267;
 $293 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($293|0);
}
function _fts3SegReaderTermCmp($pSeg,$zTerm,$nTerm) {
 $pSeg = $pSeg|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pSeg;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$res>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $29 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 60|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($9|0)>($10|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 64|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 if ($11) {
  $16 = HEAP32[$2>>2]|0;
  $17 = (_memcmp($14,$15,$16)|0);
  HEAP32[$res>>2] = $17;
 } else {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 60|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_memcmp($14,$15,$20)|0);
  HEAP32[$res>>2] = $21;
 }
 $22 = HEAP32[$res>>2]|0;
 $23 = ($22|0)==(0);
 if (!($23)) {
  $29 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($26) - ($27))|0;
 HEAP32[$res>>2] = $28;
 $29 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _fts3SegReaderSetEof($pSeg) {
 $pSeg = $pSeg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSeg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 5|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if ($5) {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 40|0);
  HEAP32[$15>>2] = 0;
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 (_sqlite3_blob_close($11)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 52|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 40|0);
 HEAP32[$15>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3SegReaderSort($apSegment,$nSegment,$nSuspect,$xCmp) {
 $apSegment = $apSegment|0;
 $nSegment = $nSegment|0;
 $nSuspect = $nSuspect|0;
 $xCmp = $xCmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $apSegment;
 HEAP32[$1>>2] = $nSegment;
 HEAP32[$2>>2] = $nSuspect;
 HEAP32[$3>>2] = $xCmp;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($4|0)==($5|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  $8 = (($7) + -1)|0;
  HEAP32[$2>>2] = $8;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) - 1)|0;
 HEAP32[$i>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)>=(0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  HEAP32[$j>>2] = $13;
  while(1) {
   $14 = HEAP32[$j>>2]|0;
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) - 1)|0;
   $17 = ($14|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = (($20) + ($19<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = (($23) + 1)|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = (($25) + ($24<<2)|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (FUNCTION_TABLE_iii[$18 & 255]($22,$27)|0);
   $29 = ($28|0)<(0);
   if ($29) {
    break;
   }
   $30 = HEAP32[$j>>2]|0;
   $31 = (($30) + 1)|0;
   $32 = HEAP32[$0>>2]|0;
   $33 = (($32) + ($31<<2)|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pTmp>>2] = $34;
   $35 = HEAP32[$j>>2]|0;
   $36 = HEAP32[$0>>2]|0;
   $37 = (($36) + ($35<<2)|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$j>>2]|0;
   $40 = (($39) + 1)|0;
   $41 = HEAP32[$0>>2]|0;
   $42 = (($41) + ($40<<2)|0);
   HEAP32[$42>>2] = $38;
   $43 = HEAP32[$pTmp>>2]|0;
   $44 = HEAP32[$j>>2]|0;
   $45 = HEAP32[$0>>2]|0;
   $46 = (($45) + ($44<<2)|0);
   HEAP32[$46>>2] = $43;
   $47 = HEAP32[$j>>2]|0;
   $48 = (($47) + 1)|0;
   HEAP32[$j>>2] = $48;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + -1)|0;
  HEAP32[$i>>2] = $50;
 }
 STACKTOP = sp;return;
}
function _fts3SegReaderCmp($pLhs,$pRhs) {
 $pLhs = $pLhs|0;
 $pRhs = $pRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $pLhs;
 HEAP32[$1>>2] = $pRhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 40|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  if ($9) {
   $10 = HEAP32[$0>>2]|0;
   $11 = ((($10)) + 60|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 60|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($12) - ($15))|0;
   HEAP32[$rc2>>2] = $16;
   $17 = HEAP32[$rc2>>2]|0;
   $18 = ($17|0)<(0);
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 64|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 64|0);
   $24 = HEAP32[$23>>2]|0;
   if ($18) {
    $25 = HEAP32[$0>>2]|0;
    $26 = ((($25)) + 60|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (_memcmp($21,$24,$27)|0);
    HEAP32[$rc>>2] = $28;
   } else {
    $29 = HEAP32[$1>>2]|0;
    $30 = ((($29)) + 60|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (_memcmp($21,$24,$31)|0);
    HEAP32[$rc>>2] = $32;
   }
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $35;
   }
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $36 = HEAP32[$0>>2]|0;
  $37 = ((($36)) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)==(0|0);
  $40 = $39&1;
  $41 = HEAP32[$1>>2]|0;
  $42 = ((($41)) + 40|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($43|0)==(0|0);
  $45 = $44&1;
  $46 = (($40) - ($45))|0;
  HEAP32[$rc>>2] = $46;
 }
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if (!($48)) {
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) - ($52))|0;
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _fts3SegReaderRequire($pReader,$pFrom,$nByte) {
 $pReader = $pReader|0;
 $pFrom = $pFrom|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pReader;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 52|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)!=(0|0);
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  $or$cond = $6 & $8;
  if (!($or$cond)) {
   label = 5;
   break;
  }
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 40|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $9;
  $14 = $12;
  $15 = (($13) - ($14))|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($15) + ($16))|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 48|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)>($20|0);
  if (!($21)) {
   label = 5;
   break;
  }
  $22 = HEAP32[$0>>2]|0;
  $23 = (_fts3SegReaderIncrRead($22)|0);
  HEAP32[$rc>>2] = $23;
 }
 if ((label|0) == 5) {
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _fts3SegReaderIncrRead($pReader) {
 $pReader = $pReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $nRead = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $nRead = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pReader;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 48|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($3) - ($6))|0;
 $8 = ($7|0)<(4096);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 48|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11) - ($14))|0;
  $16 = $15;
 } else {
  $16 = 4096;
 }
 HEAP32[$nRead>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 40|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22)|0);
 $27 = HEAP32[$nRead>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 48|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_blob_read($19,$26,$27,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)==(0);
 if (!($33)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $34 = HEAP32[$nRead>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 48|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + ($34))|0;
 HEAP32[$36>>2] = $38;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 48|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 40|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41)|0);
 dest=$45; stop=dest+20|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 48|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 44|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($48|0)==($51|0);
 if (!($52)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 52|0);
 $55 = HEAP32[$54>>2]|0;
 (_sqlite3_blob_close($55)|0);
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 52|0);
 HEAP32[$57>>2] = 0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 48|0);
 HEAP32[$59>>2] = 0;
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _sqlite3Fts3SegReaderStep($p,$pCsr) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $aNew = 0, $apSegment = 0;
 var $i = 0, $iDelta = 0, $iDocid = 0, $iPrev = 0, $isColFilter = 0, $isFirst = 0, $isIgnoreEmpty = 0, $isPrefix = 0, $isRequirePos = 0, $isScan = 0, $j = 0, $nByte = 0, $nDoclist = 0, $nList = 0, $nMerge = 0, $nSegment = 0, $nWrite = 0, $or$cond = 0, $or$cond10 = 0, $or$cond3 = 0;
 var $or$cond5 = 0, $or$cond8 = 0, $pFilter = 0, $pList = 0, $pSeg = 0, $rc = 0, $xCmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $rc = sp + 108|0;
 $isIgnoreEmpty = sp + 104|0;
 $isRequirePos = sp + 100|0;
 $isColFilter = sp + 96|0;
 $isPrefix = sp + 92|0;
 $isScan = sp + 88|0;
 $isFirst = sp + 84|0;
 $apSegment = sp + 80|0;
 $nSegment = sp + 76|0;
 $pFilter = sp + 72|0;
 $xCmp = sp + 68|0;
 $nMerge = sp + 64|0;
 $i = sp + 60|0;
 $pSeg = sp + 56|0;
 $nDoclist = sp + 52|0;
 $iPrev = sp + 16|0;
 $j = sp + 48|0;
 $pList = sp + 44|0;
 $nList = sp + 40|0;
 $nByte = sp + 36|0;
 $iDocid = sp + 8|0;
 $iDelta = sp;
 $aNew = sp + 32|0;
 $a = sp + 28|0;
 $nWrite = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 2;
 HEAP32[$isIgnoreEmpty>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 1;
 HEAP32[$isRequirePos>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 & 4;
 HEAP32[$isColFilter>>2] = $20;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 & 8;
 HEAP32[$isPrefix>>2] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = $31 & 16;
 HEAP32[$isScan>>2] = $32;
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ((($35)) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 & 32;
 HEAP32[$isFirst>>2] = $38;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$apSegment>>2] = $40;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$nSegment>>2] = $43;
 $44 = HEAP32[$2>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$pFilter>>2] = $46;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 231|0);
 $49 = HEAP8[$48>>0]|0;
 $50 = $49&255;
 $51 = ($50|0)!=(0);
 $52 = $51 ? 200 : 201;
 HEAP32[$xCmp>>2] = $52;
 $53 = HEAP32[$2>>2]|0;
 $54 = ((($53)) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = ($55|0)==(0);
 if ($56) {
  HEAP32[$0>>2] = 0;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 L4: while(1) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $57 = HEAP32[$i>>2]|0;
   $58 = HEAP32[$2>>2]|0;
   $59 = ((($58)) + 8|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ($57|0)<($60|0);
   if (!($61)) {
    break;
   }
   $62 = HEAP32[$i>>2]|0;
   $63 = HEAP32[$apSegment>>2]|0;
   $64 = (($63) + ($62<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$pSeg>>2] = $65;
   $66 = HEAP32[$pSeg>>2]|0;
   $67 = ((($66)) + 4|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = ($68<<24>>24)!=(0);
   if ($69) {
    $70 = HEAP32[$pSeg>>2]|0;
    _fts3SegReaderSetEof($70);
   } else {
    $71 = HEAP32[$1>>2]|0;
    $72 = HEAP32[$pSeg>>2]|0;
    $73 = (_fts3SegReaderNext($71,$72,0)|0);
    HEAP32[$rc>>2] = $73;
   }
   $74 = HEAP32[$rc>>2]|0;
   $75 = ($74|0)!=(0);
   if ($75) {
    label = 9;
    break L4;
   }
   $77 = HEAP32[$i>>2]|0;
   $78 = (($77) + 1)|0;
   HEAP32[$i>>2] = $78;
  }
  $79 = HEAP32[$apSegment>>2]|0;
  $80 = HEAP32[$nSegment>>2]|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = ((($81)) + 8|0);
  $83 = HEAP32[$82>>2]|0;
  _fts3SegReaderSort($79,$80,$83,199);
  $84 = HEAP32[$2>>2]|0;
  $85 = ((($84)) + 8|0);
  HEAP32[$85>>2] = 0;
  $86 = HEAP32[$apSegment>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  $88 = ((($87)) + 40|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = ($89|0)==(0|0);
  if ($90) {
   label = 61;
   break;
  }
  $91 = HEAP32[$apSegment>>2]|0;
  $92 = HEAP32[$91>>2]|0;
  $93 = ((($92)) + 60|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = HEAP32[$2>>2]|0;
  $96 = ((($95)) + 44|0);
  HEAP32[$96>>2] = $94;
  $97 = HEAP32[$apSegment>>2]|0;
  $98 = HEAP32[$97>>2]|0;
  $99 = ((($98)) + 64|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = ((($101)) + 40|0);
  HEAP32[$102>>2] = $100;
  $103 = HEAP32[$pFilter>>2]|0;
  $104 = HEAP32[$103>>2]|0;
  $105 = ($104|0)==(0|0);
  $106 = HEAP32[$isScan>>2]|0;
  $107 = ($106|0)!=(0);
  $or$cond = $105 | $107;
  if (!($or$cond)) {
   $108 = HEAP32[$2>>2]|0;
   $109 = ((($108)) + 44|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = HEAP32[$pFilter>>2]|0;
   $112 = ((($111)) + 4|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = ($110|0)<($113|0);
   if ($114) {
    label = 61;
    break;
   }
   $115 = HEAP32[$isPrefix>>2]|0;
   $116 = ($115|0)!=(0);
   if (!($116)) {
    $117 = HEAP32[$2>>2]|0;
    $118 = ((($117)) + 44|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP32[$pFilter>>2]|0;
    $121 = ((($120)) + 4|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = ($119|0)>($122|0);
    if ($123) {
     label = 61;
     break;
    }
   }
   $124 = HEAP32[$2>>2]|0;
   $125 = ((($124)) + 40|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = HEAP32[$pFilter>>2]|0;
   $128 = HEAP32[$127>>2]|0;
   $129 = HEAP32[$pFilter>>2]|0;
   $130 = ((($129)) + 4|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = (_memcmp($126,$128,$131)|0);
   $133 = ($132|0)!=(0);
   if ($133) {
    label = 61;
    break;
   }
  }
  HEAP32[$nMerge>>2] = 1;
  while(1) {
   $134 = HEAP32[$nMerge>>2]|0;
   $135 = HEAP32[$nSegment>>2]|0;
   $136 = ($134|0)<($135|0);
   if ($136) {
    $137 = HEAP32[$nMerge>>2]|0;
    $138 = HEAP32[$apSegment>>2]|0;
    $139 = (($138) + ($137<<2)|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = ((($140)) + 40|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ($142|0)!=(0|0);
    if ($143) {
     $144 = HEAP32[$nMerge>>2]|0;
     $145 = HEAP32[$apSegment>>2]|0;
     $146 = (($145) + ($144<<2)|0);
     $147 = HEAP32[$146>>2]|0;
     $148 = ((($147)) + 60|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = HEAP32[$2>>2]|0;
     $151 = ((($150)) + 44|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = ($149|0)==($152|0);
     if ($153) {
      $154 = HEAP32[$2>>2]|0;
      $155 = ((($154)) + 40|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = HEAP32[$nMerge>>2]|0;
      $158 = HEAP32[$apSegment>>2]|0;
      $159 = (($158) + ($157<<2)|0);
      $160 = HEAP32[$159>>2]|0;
      $161 = ((($160)) + 64|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = HEAP32[$2>>2]|0;
      $164 = ((($163)) + 44|0);
      $165 = HEAP32[$164>>2]|0;
      $166 = (_memcmp($156,$162,$165)|0);
      $167 = (0)==($166|0);
      $476 = $167;
     } else {
      $476 = 0;
     }
    } else {
     $476 = 0;
    }
   } else {
    $476 = 0;
   }
   $168 = HEAP32[$nMerge>>2]|0;
   if (!($476)) {
    break;
   }
   $169 = (($168) + 1)|0;
   HEAP32[$nMerge>>2] = $169;
  }
  $170 = ($168|0)!=(1);
  $171 = HEAP32[$isIgnoreEmpty>>2]|0;
  $172 = ($171|0)!=(0);
  $or$cond3 = $170 | $172;
  $173 = HEAP32[$isFirst>>2]|0;
  $174 = ($173|0)!=(0);
  $or$cond5 = $or$cond3 | $174;
  do {
   if ($or$cond5) {
    label = 32;
   } else {
    $175 = HEAP32[$1>>2]|0;
    $176 = ((($175)) + 231|0);
    $177 = HEAP8[$176>>0]|0;
    $178 = $177&255;
    $179 = ($178|0)==(0);
    if (!($179)) {
     $180 = HEAP32[$apSegment>>2]|0;
     $181 = HEAP32[$180>>2]|0;
     $182 = ((($181)) + 56|0);
     $183 = HEAP32[$182>>2]|0;
     $184 = ($183|0)!=(0|0);
     $185 = $184&1;
     $186 = ($185|0)==(0);
     if (!($186)) {
      label = 32;
      break;
     }
    }
    $187 = HEAP32[$apSegment>>2]|0;
    $188 = HEAP32[$187>>2]|0;
    $189 = ((($188)) + 76|0);
    $190 = HEAP32[$189>>2]|0;
    $191 = HEAP32[$2>>2]|0;
    $192 = ((($191)) + 52|0);
    HEAP32[$192>>2] = $190;
    $193 = HEAP32[$apSegment>>2]|0;
    $194 = HEAP32[$193>>2]|0;
    $195 = ((($194)) + 56|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = ($196|0)!=(0|0);
    if ($197) {
     $198 = HEAP32[$2>>2]|0;
     $199 = HEAP32[$apSegment>>2]|0;
     $200 = HEAP32[$199>>2]|0;
     $201 = ((($200)) + 72|0);
     $202 = HEAP32[$201>>2]|0;
     $203 = HEAP32[$2>>2]|0;
     $204 = ((($203)) + 52|0);
     $205 = HEAP32[$204>>2]|0;
     $206 = (_fts3MsrBufferData($198,$202,$205)|0);
     HEAP32[$rc>>2] = $206;
     $207 = HEAP32[$2>>2]|0;
     $208 = ((($207)) + 16|0);
     $209 = HEAP32[$208>>2]|0;
     $210 = HEAP32[$2>>2]|0;
     $211 = ((($210)) + 48|0);
     HEAP32[$211>>2] = $209;
    } else {
     $212 = HEAP32[$apSegment>>2]|0;
     $213 = HEAP32[$212>>2]|0;
     $214 = ((($213)) + 72|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = HEAP32[$2>>2]|0;
     $217 = ((($216)) + 48|0);
     HEAP32[$217>>2] = $215;
    }
    $218 = HEAP32[$rc>>2]|0;
    $219 = ($218|0)==(0);
    if ($219) {
     HEAP32[$rc>>2] = 100;
    }
   }
  } while(0);
  if ((label|0) == 32) {
   label = 0;
   HEAP32[$nDoclist>>2] = 0;
   $220 = $iPrev;
   $221 = $220;
   HEAP32[$221>>2] = 0;
   $222 = (($220) + 4)|0;
   $223 = $222;
   HEAP32[$223>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $224 = HEAP32[$i>>2]|0;
    $225 = HEAP32[$nMerge>>2]|0;
    $226 = ($224|0)<($225|0);
    if (!($226)) {
     break;
    }
    $227 = HEAP32[$1>>2]|0;
    $228 = HEAP32[$i>>2]|0;
    $229 = HEAP32[$apSegment>>2]|0;
    $230 = (($229) + ($228<<2)|0);
    $231 = HEAP32[$230>>2]|0;
    (_fts3SegReaderFirstDocid($227,$231)|0);
    $232 = HEAP32[$i>>2]|0;
    $233 = (($232) + 1)|0;
    HEAP32[$i>>2] = $233;
   }
   $234 = HEAP32[$apSegment>>2]|0;
   $235 = HEAP32[$nMerge>>2]|0;
   $236 = HEAP32[$nMerge>>2]|0;
   $237 = HEAP32[$xCmp>>2]|0;
   _fts3SegReaderSort($234,$235,$236,$237);
   while(1) {
    $238 = HEAP32[$apSegment>>2]|0;
    $239 = HEAP32[$238>>2]|0;
    $240 = ((($239)) + 80|0);
    $241 = HEAP32[$240>>2]|0;
    $242 = ($241|0)!=(0|0);
    if (!($242)) {
     break;
    }
    HEAP32[$pList>>2] = 0;
    HEAP32[$nList>>2] = 0;
    $243 = HEAP32[$apSegment>>2]|0;
    $244 = HEAP32[$243>>2]|0;
    $245 = ((($244)) + 88|0);
    $246 = $245;
    $247 = $246;
    $248 = HEAP32[$247>>2]|0;
    $249 = (($246) + 4)|0;
    $250 = $249;
    $251 = HEAP32[$250>>2]|0;
    $252 = $iDocid;
    $253 = $252;
    HEAP32[$253>>2] = $248;
    $254 = (($252) + 4)|0;
    $255 = $254;
    HEAP32[$255>>2] = $251;
    $256 = HEAP32[$1>>2]|0;
    $257 = HEAP32[$apSegment>>2]|0;
    $258 = HEAP32[$257>>2]|0;
    (_fts3SegReaderNextDocid($256,$258,$pList,$nList)|0);
    HEAP32[$j>>2] = 1;
    while(1) {
     $259 = HEAP32[$j>>2]|0;
     $260 = HEAP32[$nMerge>>2]|0;
     $261 = ($259|0)<($260|0);
     if (!($261)) {
      break;
     }
     $262 = HEAP32[$j>>2]|0;
     $263 = HEAP32[$apSegment>>2]|0;
     $264 = (($263) + ($262<<2)|0);
     $265 = HEAP32[$264>>2]|0;
     $266 = ((($265)) + 80|0);
     $267 = HEAP32[$266>>2]|0;
     $268 = ($267|0)!=(0|0);
     if (!($268)) {
      break;
     }
     $269 = HEAP32[$j>>2]|0;
     $270 = HEAP32[$apSegment>>2]|0;
     $271 = (($270) + ($269<<2)|0);
     $272 = HEAP32[$271>>2]|0;
     $273 = ((($272)) + 88|0);
     $274 = $273;
     $275 = $274;
     $276 = HEAP32[$275>>2]|0;
     $277 = (($274) + 4)|0;
     $278 = $277;
     $279 = HEAP32[$278>>2]|0;
     $280 = $iDocid;
     $281 = $280;
     $282 = HEAP32[$281>>2]|0;
     $283 = (($280) + 4)|0;
     $284 = $283;
     $285 = HEAP32[$284>>2]|0;
     $286 = ($276|0)==($282|0);
     $287 = ($279|0)==($285|0);
     $288 = $286 & $287;
     if (!($288)) {
      break;
     }
     $289 = HEAP32[$1>>2]|0;
     $290 = HEAP32[$j>>2]|0;
     $291 = HEAP32[$apSegment>>2]|0;
     $292 = (($291) + ($290<<2)|0);
     $293 = HEAP32[$292>>2]|0;
     (_fts3SegReaderNextDocid($289,$293,0,0)|0);
     $294 = HEAP32[$j>>2]|0;
     $295 = (($294) + 1)|0;
     HEAP32[$j>>2] = $295;
    }
    $296 = HEAP32[$isColFilter>>2]|0;
    $297 = ($296|0)!=(0);
    if ($297) {
     $298 = HEAP32[$pFilter>>2]|0;
     $299 = ((($298)) + 8|0);
     $300 = HEAP32[$299>>2]|0;
     _fts3ColumnFilter($300,0,$pList,$nList);
    }
    $301 = HEAP32[$isIgnoreEmpty>>2]|0;
    $302 = ($301|0)==(0);
    $303 = HEAP32[$nList>>2]|0;
    $304 = ($303|0)>(0);
    $or$cond8 = $302 | $304;
    do {
     if ($or$cond8) {
      $305 = HEAP32[$1>>2]|0;
      $306 = ((($305)) + 231|0);
      $307 = HEAP8[$306>>0]|0;
      $308 = $307&255;
      $309 = ($308|0)!=(0);
      $310 = HEAP32[$nDoclist>>2]|0;
      $311 = ($310|0)>(0);
      $or$cond10 = $309 & $311;
      if ($or$cond10) {
       $312 = $iPrev;
       $313 = $312;
       $314 = HEAP32[$313>>2]|0;
       $315 = (($312) + 4)|0;
       $316 = $315;
       $317 = HEAP32[$316>>2]|0;
       $318 = $iDocid;
       $319 = $318;
       $320 = HEAP32[$319>>2]|0;
       $321 = (($318) + 4)|0;
       $322 = $321;
       $323 = HEAP32[$322>>2]|0;
       $324 = (_i64Subtract(($314|0),($317|0),($320|0),($323|0))|0);
       $325 = tempRet0;
       $326 = $iDelta;
       $327 = $326;
       HEAP32[$327>>2] = $324;
       $328 = (($326) + 4)|0;
       $329 = $328;
       HEAP32[$329>>2] = $325;
      } else {
       $330 = $iDocid;
       $331 = $330;
       $332 = HEAP32[$331>>2]|0;
       $333 = (($330) + 4)|0;
       $334 = $333;
       $335 = HEAP32[$334>>2]|0;
       $336 = $iPrev;
       $337 = $336;
       $338 = HEAP32[$337>>2]|0;
       $339 = (($336) + 4)|0;
       $340 = $339;
       $341 = HEAP32[$340>>2]|0;
       $342 = (_i64Subtract(($332|0),($335|0),($338|0),($341|0))|0);
       $343 = tempRet0;
       $344 = $iDelta;
       $345 = $344;
       HEAP32[$345>>2] = $342;
       $346 = (($344) + 4)|0;
       $347 = $346;
       HEAP32[$347>>2] = $343;
      }
      $348 = $iDelta;
      $349 = $348;
      $350 = HEAP32[$349>>2]|0;
      $351 = (($348) + 4)|0;
      $352 = $351;
      $353 = HEAP32[$352>>2]|0;
      $354 = (_sqlite3Fts3VarintLen($350,$353)|0);
      $355 = HEAP32[$isRequirePos>>2]|0;
      $356 = ($355|0)!=(0);
      $357 = HEAP32[$nList>>2]|0;
      $358 = (($357) + 1)|0;
      $359 = $356 ? $358 : 0;
      $360 = (($354) + ($359))|0;
      HEAP32[$nByte>>2] = $360;
      $361 = HEAP32[$nDoclist>>2]|0;
      $362 = HEAP32[$nByte>>2]|0;
      $363 = (($361) + ($362))|0;
      $364 = HEAP32[$2>>2]|0;
      $365 = ((($364)) + 20|0);
      $366 = HEAP32[$365>>2]|0;
      $367 = ($363|0)>($366|0);
      if ($367) {
       $368 = HEAP32[$nDoclist>>2]|0;
       $369 = HEAP32[$nByte>>2]|0;
       $370 = (($368) + ($369))|0;
       $371 = $370<<1;
       $372 = HEAP32[$2>>2]|0;
       $373 = ((($372)) + 20|0);
       HEAP32[$373>>2] = $371;
       $374 = HEAP32[$2>>2]|0;
       $375 = ((($374)) + 16|0);
       $376 = HEAP32[$375>>2]|0;
       $377 = HEAP32[$2>>2]|0;
       $378 = ((($377)) + 20|0);
       $379 = HEAP32[$378>>2]|0;
       $380 = (_sqlite3_realloc($376,$379)|0);
       HEAP32[$aNew>>2] = $380;
       $381 = HEAP32[$aNew>>2]|0;
       $382 = ($381|0)!=(0|0);
       if (!($382)) {
        label = 50;
        break L4;
       }
       $383 = HEAP32[$aNew>>2]|0;
       $384 = HEAP32[$2>>2]|0;
       $385 = ((($384)) + 16|0);
       HEAP32[$385>>2] = $383;
      }
      $386 = HEAP32[$isFirst>>2]|0;
      $387 = ($386|0)!=(0);
      $388 = HEAP32[$nDoclist>>2]|0;
      $389 = HEAP32[$2>>2]|0;
      $390 = ((($389)) + 16|0);
      $391 = HEAP32[$390>>2]|0;
      $392 = (($391) + ($388)|0);
      if ($387) {
       HEAP32[$a>>2] = $392;
       $393 = $iDelta;
       $394 = $393;
       $395 = HEAP32[$394>>2]|0;
       $396 = (($393) + 4)|0;
       $397 = $396;
       $398 = HEAP32[$397>>2]|0;
       $399 = HEAP32[$pList>>2]|0;
       $400 = HEAP32[$nList>>2]|0;
       $401 = HEAP32[$a>>2]|0;
       $402 = (_sqlite3Fts3FirstFilter($395,$398,$399,$400,$401)|0);
       HEAP32[$nWrite>>2] = $402;
       $403 = HEAP32[$nWrite>>2]|0;
       $404 = ($403|0)!=(0);
       if (!($404)) {
        break;
       }
       $405 = $iDocid;
       $406 = $405;
       $407 = HEAP32[$406>>2]|0;
       $408 = (($405) + 4)|0;
       $409 = $408;
       $410 = HEAP32[$409>>2]|0;
       $411 = $iPrev;
       $412 = $411;
       HEAP32[$412>>2] = $407;
       $413 = (($411) + 4)|0;
       $414 = $413;
       HEAP32[$414>>2] = $410;
       $415 = HEAP32[$nWrite>>2]|0;
       $416 = HEAP32[$nDoclist>>2]|0;
       $417 = (($416) + ($415))|0;
       HEAP32[$nDoclist>>2] = $417;
       break;
      } else {
       $418 = $iDelta;
       $419 = $418;
       $420 = HEAP32[$419>>2]|0;
       $421 = (($418) + 4)|0;
       $422 = $421;
       $423 = HEAP32[$422>>2]|0;
       $424 = (_sqlite3Fts3PutVarint($392,$420,$423)|0);
       $425 = HEAP32[$nDoclist>>2]|0;
       $426 = (($425) + ($424))|0;
       HEAP32[$nDoclist>>2] = $426;
       $427 = $iDocid;
       $428 = $427;
       $429 = HEAP32[$428>>2]|0;
       $430 = (($427) + 4)|0;
       $431 = $430;
       $432 = HEAP32[$431>>2]|0;
       $433 = $iPrev;
       $434 = $433;
       HEAP32[$434>>2] = $429;
       $435 = (($433) + 4)|0;
       $436 = $435;
       HEAP32[$436>>2] = $432;
       $437 = HEAP32[$isRequirePos>>2]|0;
       $438 = ($437|0)!=(0);
       if (!($438)) {
        break;
       }
       $439 = HEAP32[$nDoclist>>2]|0;
       $440 = HEAP32[$2>>2]|0;
       $441 = ((($440)) + 16|0);
       $442 = HEAP32[$441>>2]|0;
       $443 = (($442) + ($439)|0);
       $444 = HEAP32[$pList>>2]|0;
       $445 = HEAP32[$nList>>2]|0;
       _memcpy(($443|0),($444|0),($445|0))|0;
       $446 = HEAP32[$nList>>2]|0;
       $447 = HEAP32[$nDoclist>>2]|0;
       $448 = (($447) + ($446))|0;
       HEAP32[$nDoclist>>2] = $448;
       $449 = HEAP32[$nDoclist>>2]|0;
       $450 = (($449) + 1)|0;
       HEAP32[$nDoclist>>2] = $450;
       $451 = HEAP32[$2>>2]|0;
       $452 = ((($451)) + 16|0);
       $453 = HEAP32[$452>>2]|0;
       $454 = (($453) + ($449)|0);
       HEAP8[$454>>0] = 0;
       break;
      }
     }
    } while(0);
    $455 = HEAP32[$apSegment>>2]|0;
    $456 = HEAP32[$nMerge>>2]|0;
    $457 = HEAP32[$j>>2]|0;
    $458 = HEAP32[$xCmp>>2]|0;
    _fts3SegReaderSort($455,$456,$457,$458);
   }
   $459 = HEAP32[$nDoclist>>2]|0;
   $460 = ($459|0)>(0);
   if ($460) {
    $461 = HEAP32[$2>>2]|0;
    $462 = ((($461)) + 16|0);
    $463 = HEAP32[$462>>2]|0;
    $464 = HEAP32[$2>>2]|0;
    $465 = ((($464)) + 48|0);
    HEAP32[$465>>2] = $463;
    $466 = HEAP32[$nDoclist>>2]|0;
    $467 = HEAP32[$2>>2]|0;
    $468 = ((($467)) + 52|0);
    HEAP32[$468>>2] = $466;
    HEAP32[$rc>>2] = 100;
   }
  }
  $469 = HEAP32[$nMerge>>2]|0;
  $470 = HEAP32[$2>>2]|0;
  $471 = ((($470)) + 8|0);
  HEAP32[$471>>2] = $469;
  $472 = HEAP32[$rc>>2]|0;
  $473 = ($472|0)==(0);
  if (!($473)) {
   label = 61;
   break;
  }
 }
 if ((label|0) == 9) {
  $76 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $76;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 else if ((label|0) == 50) {
  HEAP32[$0>>2] = 7;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 else if ((label|0) == 61) {
  $474 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $474;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 return (0)|0;
}
function _fts3auxGrowStatArray($pCsr,$nSize) {
 $pCsr = $pCsr|0;
 $nSize = $nSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aNew = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $nSize;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)>($6|0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 112|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   $12 = $11<<4;
   $13 = (_sqlite3_realloc($10,$12)|0);
   HEAP32[$aNew>>2] = $13;
   $14 = HEAP32[$aNew>>2]|0;
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 108|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$aNew>>2]|0;
    $20 = (($19) + ($18<<4)|0);
    $21 = HEAP32[$2>>2]|0;
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 108|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($21) - ($24))|0;
    $26 = $25<<4;
    _memset(($20|0),0,($26|0))|0;
    $27 = HEAP32[$aNew>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 112|0);
    HEAP32[$29>>2] = $27;
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 108|0);
    HEAP32[$32>>2] = $30;
    break;
   }
   HEAP32[$0>>2] = 7;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _fts3SegReaderDoclistCmpRev($pLhs,$pRhs) {
 $pLhs = $pLhs|0;
 $pRhs = $pRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pLhs;
 HEAP32[$1>>2] = $pRhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 80|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = $5&1;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 80|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $10&1;
 $12 = (($6) - ($11))|0;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 88|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 88|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($19|0)==($27|0);
 $32 = ($22|0)==($30|0);
 $33 = $31 & $32;
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) - ($37))|0;
  HEAP32[$rc>>2] = $38;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 88|0);
  $41 = $40;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 88|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = ($46|0)<($54|0);
  $56 = ($43>>>0)<($51>>>0);
  $57 = ($46|0)==($54|0);
  $58 = $57 & $56;
  $59 = $55 | $58;
  $60 = $59 ? 1 : -1;
  HEAP32[$rc>>2] = $60;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _fts3SegReaderDoclistCmp($pLhs,$pRhs) {
 $pLhs = $pLhs|0;
 $pRhs = $pRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pLhs;
 HEAP32[$1>>2] = $pRhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 80|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = $5&1;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 80|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $10&1;
 $12 = (($6) - ($11))|0;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 88|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 88|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($19|0)==($27|0);
 $32 = ($22|0)==($30|0);
 $33 = $31 & $32;
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) - ($37))|0;
  HEAP32[$rc>>2] = $38;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 88|0);
  $41 = $40;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 88|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = ($46|0)>($54|0);
  $56 = ($43>>>0)>($51>>>0);
  $57 = ($46|0)==($54|0);
  $58 = $57 & $56;
  $59 = $55 | $58;
  $60 = $59 ? 1 : -1;
  HEAP32[$rc>>2] = $60;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _fts3MsrBufferData($pMsr,$pList,$nList) {
 $pMsr = $pMsr|0;
 $pList = $pList|0;
 $nList = $nList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pMsr;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $nList;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($4|0)>($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$3>>2]|0;
   $10 = $9<<1;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 20|0);
   HEAP32[$12>>2] = $10;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 20|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3_realloc($15,$18)|0);
   HEAP32[$pNew>>2] = $19;
   $20 = HEAP32[$pNew>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$pNew>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 16|0);
    HEAP32[$24>>2] = $22;
    break;
   }
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 _memcpy(($27|0),($28|0),($29|0))|0;
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _fts3SegReaderFirstDocid($pTab,$pReader) {
 $pTab = $pTab|0;
 $pReader = $pReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bEof = 0, $n = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $bEof = sp + 16|0;
 $n = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $pReader;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 231|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 56|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0|0);
  if ($10) {
   HEAP8[$bEof>>0] = 0;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 88|0);
   $13 = $12;
   $14 = $13;
   HEAP32[$14>>2] = 0;
   $15 = (($13) + 4)|0;
   $16 = $15;
   HEAP32[$16>>2] = 0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 84|0);
   HEAP32[$18>>2] = 0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 72|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 76|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 80|0);
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 88|0);
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 84|0);
   _sqlite3Fts3DoclistPrev(0,$21,$24,$26,$28,$30,$bEof);
   $51 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($51|0);
  }
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 72|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (_fts3SegReaderRequire($31,$34,10)|0);
 HEAP32[$rc>>2] = $35;
 $36 = HEAP32[$rc>>2]|0;
 $37 = ($36|0)==(0);
 if (!($37)) {
  $51 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 72|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 88|0);
 $43 = (_sqlite3Fts3GetVarint($40,$42)|0);
 HEAP32[$n>>2] = $43;
 $44 = HEAP32[$n>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 72|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44)|0);
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 80|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _fts3SegReaderNextDocid($pTab,$pReader,$ppOffsetList,$pnOffsetList) {
 $pTab = $pTab|0;
 $pReader = $pReader|0;
 $ppOffsetList = $ppOffsetList|0;
 $pnOffsetList = $pnOffsetList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bEof = 0, $c = 0, $iDelta = 0, $p = 0, $pEnd = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $rc = sp + 16|0;
 $p = sp + 12|0;
 $c = sp + 41|0;
 $bEof = sp + 40|0;
 $pEnd = sp + 8|0;
 $iDelta = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pReader;
 HEAP32[$3>>2] = $ppOffsetList;
 HEAP32[$4>>2] = $pnOffsetList;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 HEAP8[$c>>0] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 231|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(0);
 do {
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = ((($13)) + 56|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    HEAP8[$bEof>>0] = 0;
    $17 = HEAP32[$3>>2]|0;
    $18 = ($17|0)!=(0|0);
    if ($18) {
     $19 = HEAP32[$2>>2]|0;
     $20 = ((($19)) + 80|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$3>>2]|0;
     HEAP32[$22>>2] = $21;
     $23 = HEAP32[$2>>2]|0;
     $24 = ((($23)) + 84|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = (($25) - 1)|0;
     $27 = HEAP32[$4>>2]|0;
     HEAP32[$27>>2] = $26;
    }
    $28 = HEAP32[$2>>2]|0;
    $29 = ((($28)) + 72|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = ((($31)) + 76|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = ((($34)) + 88|0);
    $36 = HEAP32[$2>>2]|0;
    $37 = ((($36)) + 84|0);
    _sqlite3Fts3DoclistPrev(0,$30,$33,$p,$35,$37,$bEof);
    $38 = HEAP8[$bEof>>0]|0;
    $39 = ($38<<24>>24)!=(0);
    if ($39) {
     $40 = HEAP32[$2>>2]|0;
     $41 = ((($40)) + 80|0);
     HEAP32[$41>>2] = 0;
     break;
    } else {
     $42 = HEAP32[$p>>2]|0;
     $43 = HEAP32[$2>>2]|0;
     $44 = ((($43)) + 80|0);
     HEAP32[$44>>2] = $42;
     break;
    }
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
 } while(0);
 do {
  if ((label|0) == 8) {
   $45 = HEAP32[$2>>2]|0;
   $46 = ((($45)) + 76|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = ((($48)) + 72|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + ($47)|0);
   HEAP32[$pEnd>>2] = $51;
   while(1) {
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP8[$52>>0]|0;
    $54 = $53 << 24 >> 24;
    $55 = HEAP8[$c>>0]|0;
    $56 = $55 << 24 >> 24;
    $57 = $54 | $56;
    $58 = ($57|0)!=(0);
    if ($58) {
     $59 = HEAP32[$p>>2]|0;
     $60 = ((($59)) + 1|0);
     HEAP32[$p>>2] = $60;
     $61 = HEAP8[$59>>0]|0;
     $62 = $61 << 24 >> 24;
     $63 = $62 & 128;
     $64 = $63&255;
     HEAP8[$c>>0] = $64;
     continue;
    }
    $65 = HEAP32[$2>>2]|0;
    $66 = ((($65)) + 52|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = ($67|0)==(0|0);
    if ($68) {
     break;
    }
    $69 = HEAP32[$p>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = ((($70)) + 48|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = HEAP32[$2>>2]|0;
    $74 = ((($73)) + 40|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + ($72)|0);
    $77 = ($69>>>0)<($76>>>0);
    if ($77) {
     break;
    }
    $78 = HEAP32[$2>>2]|0;
    $79 = (_fts3SegReaderIncrRead($78)|0);
    HEAP32[$rc>>2] = $79;
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     label = 14;
     break;
    }
   }
   if ((label|0) == 14) {
    $82 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $82;
    $153 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($153|0);
   }
   $83 = HEAP32[$p>>2]|0;
   $84 = ((($83)) + 1|0);
   HEAP32[$p>>2] = $84;
   $85 = HEAP32[$3>>2]|0;
   $86 = ($85|0)!=(0|0);
   if ($86) {
    $87 = HEAP32[$2>>2]|0;
    $88 = ((($87)) + 80|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$3>>2]|0;
    HEAP32[$90>>2] = $89;
    $91 = HEAP32[$p>>2]|0;
    $92 = HEAP32[$2>>2]|0;
    $93 = ((($92)) + 80|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = $91;
    $96 = $94;
    $97 = (($95) - ($96))|0;
    $98 = (($97) - 1)|0;
    $99 = HEAP32[$4>>2]|0;
    HEAP32[$99>>2] = $98;
   }
   while(1) {
    $100 = HEAP32[$p>>2]|0;
    $101 = HEAP32[$pEnd>>2]|0;
    $102 = ($100>>>0)<($101>>>0);
    if ($102) {
     $103 = HEAP32[$p>>2]|0;
     $104 = HEAP8[$103>>0]|0;
     $105 = $104 << 24 >> 24;
     $106 = ($105|0)==(0);
     $154 = $106;
    } else {
     $154 = 0;
    }
    $107 = HEAP32[$p>>2]|0;
    if (!($154)) {
     break;
    }
    $108 = ((($107)) + 1|0);
    HEAP32[$p>>2] = $108;
   }
   $109 = HEAP32[$pEnd>>2]|0;
   $110 = ($107>>>0)>=($109>>>0);
   $111 = HEAP32[$2>>2]|0;
   if ($110) {
    $112 = ((($111)) + 80|0);
    HEAP32[$112>>2] = 0;
    break;
   }
   $113 = HEAP32[$p>>2]|0;
   $114 = (_fts3SegReaderRequire($111,$113,10)|0);
   HEAP32[$rc>>2] = $114;
   $115 = HEAP32[$rc>>2]|0;
   $116 = ($115|0)==(0);
   if ($116) {
    $117 = HEAP32[$p>>2]|0;
    $118 = HEAP32[$p>>2]|0;
    $119 = (_sqlite3Fts3GetVarint($118,$iDelta)|0);
    $120 = (($117) + ($119)|0);
    $121 = HEAP32[$2>>2]|0;
    $122 = ((($121)) + 80|0);
    HEAP32[$122>>2] = $120;
    $123 = HEAP32[$1>>2]|0;
    $124 = ((($123)) + 231|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = ($125<<24>>24)!=(0);
    $127 = $iDelta;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = HEAP32[$2>>2]|0;
    $134 = ((($133)) + 88|0);
    $135 = $134;
    $136 = $135;
    $137 = HEAP32[$136>>2]|0;
    $138 = (($135) + 4)|0;
    $139 = $138;
    $140 = HEAP32[$139>>2]|0;
    if ($126) {
     $141 = (_i64Subtract(($137|0),($140|0),($129|0),($132|0))|0);
     $142 = tempRet0;
     $143 = $134;
     $144 = $143;
     HEAP32[$144>>2] = $141;
     $145 = (($143) + 4)|0;
     $146 = $145;
     HEAP32[$146>>2] = $142;
     break;
    } else {
     $147 = (_i64Add(($137|0),($140|0),($129|0),($132|0))|0);
     $148 = tempRet0;
     $149 = $134;
     $150 = $149;
     HEAP32[$150>>2] = $147;
     $151 = (($149) + 4)|0;
     $152 = $151;
     HEAP32[$152>>2] = $148;
     break;
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $153 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _fts3ColumnFilter($iCol,$bZero,$ppList,$pnList) {
 $iCol = $iCol|0;
 $bZero = $bZero|0;
 $ppList = $ppList|0;
 $pnList = $pnList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $c = 0, $iCurrent = 0, $nList = 0;
 var $p = 0, $pEnd = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pList = sp + 16|0;
 $nList = sp + 12|0;
 $pEnd = sp + 8|0;
 $iCurrent = sp + 4|0;
 $p = sp;
 $c = sp + 36|0;
 HEAP32[$0>>2] = $iCol;
 HEAP32[$1>>2] = $bZero;
 HEAP32[$2>>2] = $ppList;
 HEAP32[$3>>2] = $pnList;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pList>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nList>>2] = $7;
 $8 = HEAP32[$nList>>2]|0;
 $9 = HEAP32[$pList>>2]|0;
 $10 = (($9) + ($8)|0);
 HEAP32[$pEnd>>2] = $10;
 HEAP32[$iCurrent>>2] = 0;
 $11 = HEAP32[$pList>>2]|0;
 HEAP32[$p>>2] = $11;
 while(1) {
  HEAP8[$c>>0] = 0;
  while(1) {
   $12 = HEAP32[$p>>2]|0;
   $13 = HEAP32[$pEnd>>2]|0;
   $14 = ($12>>>0)<($13>>>0);
   if (!($14)) {
    break;
   }
   $15 = HEAP8[$c>>0]|0;
   $16 = $15 << 24 >> 24;
   $17 = HEAP32[$p>>2]|0;
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   $20 = $16 | $19;
   $21 = $20 & 254;
   $22 = ($21|0)!=(0);
   if (!($22)) {
    break;
   }
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 1|0);
   HEAP32[$p>>2] = $24;
   $25 = HEAP8[$23>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = $26 & 128;
   $28 = $27&255;
   HEAP8[$c>>0] = $28;
  }
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$iCurrent>>2]|0;
  $31 = ($29|0)==($30|0);
  $32 = HEAP32[$p>>2]|0;
  $33 = HEAP32[$pList>>2]|0;
  $34 = $32;
  $35 = $33;
  $36 = (($34) - ($35))|0;
  if ($31) {
   label = 7;
   break;
  }
  $37 = HEAP32[$nList>>2]|0;
  $38 = (($37) - ($36))|0;
  HEAP32[$nList>>2] = $38;
  $39 = HEAP32[$p>>2]|0;
  HEAP32[$pList>>2] = $39;
  $40 = HEAP32[$nList>>2]|0;
  $41 = ($40|0)==(0);
  if ($41) {
   break;
  }
  $42 = HEAP32[$pList>>2]|0;
  $43 = ((($42)) + 1|0);
  HEAP32[$p>>2] = $43;
  $44 = HEAP32[$p>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46 & 128;
  $48 = ($47|0)!=(0);
  $49 = HEAP32[$p>>2]|0;
  if ($48) {
   $50 = (_sqlite3Fts3GetVarint32($49,$iCurrent)|0);
   $55 = $50;
  } else {
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   HEAP32[$iCurrent>>2] = $52;
   $55 = 1;
  }
  $53 = HEAP32[$p>>2]|0;
  $54 = (($53) + ($55)|0);
  HEAP32[$p>>2] = $54;
 }
 if ((label|0) == 7) {
  HEAP32[$nList>>2] = $36;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = ($56|0)!=(0);
 if (!($57)) {
  $73 = HEAP32[$pList>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  $75 = HEAP32[$nList>>2]|0;
  $76 = HEAP32[$3>>2]|0;
  HEAP32[$76>>2] = $75;
  STACKTOP = sp;return;
 }
 $58 = HEAP32[$nList>>2]|0;
 $59 = HEAP32[$pList>>2]|0;
 $60 = (($59) + ($58)|0);
 $61 = HEAP32[$pEnd>>2]|0;
 $62 = ($60|0)!=($61|0);
 if (!($62)) {
  $73 = HEAP32[$pList>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  $75 = HEAP32[$nList>>2]|0;
  $76 = HEAP32[$3>>2]|0;
  HEAP32[$76>>2] = $75;
  STACKTOP = sp;return;
 }
 $63 = HEAP32[$nList>>2]|0;
 $64 = HEAP32[$pList>>2]|0;
 $65 = (($64) + ($63)|0);
 $66 = HEAP32[$pEnd>>2]|0;
 $67 = HEAP32[$nList>>2]|0;
 $68 = HEAP32[$pList>>2]|0;
 $69 = (($68) + ($67)|0);
 $70 = $66;
 $71 = $69;
 $72 = (($70) - ($71))|0;
 _memset(($65|0),0,($72|0))|0;
 $73 = HEAP32[$pList>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 HEAP32[$74>>2] = $73;
 $75 = HEAP32[$nList>>2]|0;
 $76 = HEAP32[$3>>2]|0;
 HEAP32[$76>>2] = $75;
 STACKTOP = sp;return;
}
function _sqlite3Fts3VarintLen($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $i = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$i>>2] = $8;
  $9 = $2;
  $10 = $9;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9) + 4)|0;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (_bitshift64Lshr(($11|0),($14|0),7)|0);
  $16 = tempRet0;
  $17 = $2;
  $18 = $17;
  HEAP32[$18>>2] = $15;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $21 = $2;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)!=(0);
  $28 = ($26|0)!=(0);
  $29 = $27 | $28;
  if (!($29)) {
   break;
  }
 }
 $30 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3Fts3FirstFilter($0,$1,$pList,$nList,$pOut) {
 $0 = $0|0;
 $1 = $1|0;
 $pList = $pList|0;
 $nList = $nList|0;
 $pOut = $pOut|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $bWritten = 0, $iCol = 0, $nOut = 0, $p = 0, $pEnd = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 8|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $nOut = sp + 28|0;
 $bWritten = sp + 24|0;
 $p = sp + 20|0;
 $pEnd = sp + 16|0;
 $iCol = sp;
 $6 = $2;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nList;
 HEAP32[$5>>2] = $pOut;
 HEAP32[$nOut>>2] = 0;
 HEAP32[$bWritten>>2] = 0;
 $10 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + ($11)|0);
 HEAP32[$pEnd>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)!=(1);
 if ($17) {
  $18 = HEAP32[$p>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)==(2);
  if ($21) {
   $22 = HEAP32[$nOut>>2]|0;
   $23 = HEAP32[$5>>2]|0;
   $24 = (($23) + ($22)|0);
   $25 = $2;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (($25) + 4)|0;
   $29 = $28;
   $30 = HEAP32[$29>>2]|0;
   $31 = (_sqlite3Fts3PutVarint($24,$27,$30)|0);
   $32 = HEAP32[$nOut>>2]|0;
   $33 = (($32) + ($31))|0;
   HEAP32[$nOut>>2] = $33;
   $34 = HEAP32[$nOut>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$nOut>>2] = $35;
   $36 = HEAP32[$5>>2]|0;
   $37 = (($36) + ($34)|0);
   HEAP8[$37>>0] = 2;
   HEAP32[$bWritten>>2] = 1;
  }
  _fts3ColumnlistCopy(0,$p);
 }
 while(1) {
  $38 = HEAP32[$p>>2]|0;
  $39 = HEAP32[$pEnd>>2]|0;
  $40 = ($38>>>0)<($39>>>0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$p>>2]|0;
  $42 = HEAP8[$41>>0]|0;
  $43 = $42 << 24 >> 24;
  $44 = ($43|0)==(1);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$p>>2]|0;
  $46 = ((($45)) + 1|0);
  HEAP32[$p>>2] = $46;
  $47 = HEAP32[$p>>2]|0;
  $48 = (_sqlite3Fts3GetVarint($47,$iCol)|0);
  $49 = HEAP32[$p>>2]|0;
  $50 = (($49) + ($48)|0);
  HEAP32[$p>>2] = $50;
  $51 = HEAP32[$p>>2]|0;
  $52 = HEAP8[$51>>0]|0;
  $53 = $52 << 24 >> 24;
  $54 = ($53|0)==(2);
  if ($54) {
   $55 = HEAP32[$bWritten>>2]|0;
   $56 = ($55|0)==(0);
   if ($56) {
    $57 = HEAP32[$nOut>>2]|0;
    $58 = HEAP32[$5>>2]|0;
    $59 = (($58) + ($57)|0);
    $60 = $2;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($60) + 4)|0;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (_sqlite3Fts3PutVarint($59,$62,$65)|0);
    $67 = HEAP32[$nOut>>2]|0;
    $68 = (($67) + ($66))|0;
    HEAP32[$nOut>>2] = $68;
    HEAP32[$bWritten>>2] = 1;
   }
   $69 = HEAP32[$nOut>>2]|0;
   $70 = (($69) + 1)|0;
   HEAP32[$nOut>>2] = $70;
   $71 = HEAP32[$5>>2]|0;
   $72 = (($71) + ($69)|0);
   HEAP8[$72>>0] = 1;
   $73 = HEAP32[$nOut>>2]|0;
   $74 = HEAP32[$5>>2]|0;
   $75 = (($74) + ($73)|0);
   $76 = $iCol;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (_sqlite3Fts3PutVarint($75,$78,$81)|0);
   $83 = HEAP32[$nOut>>2]|0;
   $84 = (($83) + ($82))|0;
   HEAP32[$nOut>>2] = $84;
   $85 = HEAP32[$nOut>>2]|0;
   $86 = (($85) + 1)|0;
   HEAP32[$nOut>>2] = $86;
   $87 = HEAP32[$5>>2]|0;
   $88 = (($87) + ($85)|0);
   HEAP8[$88>>0] = 2;
  }
  _fts3ColumnlistCopy(0,$p);
 }
 $89 = HEAP32[$bWritten>>2]|0;
 $90 = ($89|0)!=(0);
 if (!($90)) {
  $95 = HEAP32[$nOut>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $91 = HEAP32[$nOut>>2]|0;
 $92 = (($91) + 1)|0;
 HEAP32[$nOut>>2] = $92;
 $93 = HEAP32[$5>>2]|0;
 $94 = (($93) + ($91)|0);
 HEAP8[$94>>0] = 0;
 $95 = HEAP32[$nOut>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _sqlite3Fts3PutVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $q = 0, $vu = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 8|0;
 $q = sp + 16|0;
 $vu = sp;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$q>>2] = $8;
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $vu;
 $16 = $15;
 HEAP32[$16>>2] = $11;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 while(1) {
  $19 = $vu;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = $21 & 127;
  $26 = $25 | 128;
  $27 = $26&255;
  $28 = HEAP32[$q>>2]|0;
  $29 = ((($28)) + 1|0);
  HEAP32[$q>>2] = $29;
  HEAP8[$28>>0] = $27;
  $30 = $vu;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_bitshift64Lshr(($32|0),($35|0),7)|0);
  $37 = tempRet0;
  $38 = $vu;
  $39 = $38;
  HEAP32[$39>>2] = $36;
  $40 = (($38) + 4)|0;
  $41 = $40;
  HEAP32[$41>>2] = $37;
  $42 = $vu;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = ($44|0)!=(0);
  $49 = ($47|0)!=(0);
  $50 = $48 | $49;
  if (!($50)) {
   break;
  }
 }
 $51 = HEAP32[$q>>2]|0;
 $52 = ((($51)) + -1|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = $54 & 127;
 $56 = $55&255;
 HEAP8[$52>>0] = $56;
 $57 = HEAP32[$q>>2]|0;
 $58 = HEAP32[$2>>2]|0;
 $59 = $57;
 $60 = $58;
 $61 = (($59) - ($60))|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3Fts3DoclistPrev($bDescIdx,$aDoclist,$nDoclist,$ppIter,$piDocid,$pnList,$pbEof) {
 $bDescIdx = $bDescIdx|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 $ppIter = $ppIter|0;
 $piDocid = $piDocid|0;
 $pnList = $pnList|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iDelta = 0, $iDelta2 = 0, $iDocid = 0, $iMul = 0, $iMul1 = 0, $p = 0, $pDocid = 0, $pEnd = 0, $pNext = 0, $pSave = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $p = sp + 48|0;
 $iDocid = sp + 16|0;
 $pNext = sp + 44|0;
 $pDocid = sp + 40|0;
 $pEnd = sp + 36|0;
 $iMul = sp + 32|0;
 $iDelta = sp + 8|0;
 $iMul1 = sp + 28|0;
 $iDelta2 = sp;
 $pSave = sp + 24|0;
 HEAP32[$0>>2] = $bDescIdx;
 HEAP32[$1>>2] = $aDoclist;
 HEAP32[$2>>2] = $nDoclist;
 HEAP32[$3>>2] = $ppIter;
 HEAP32[$4>>2] = $piDocid;
 HEAP32[$5>>2] = $pnList;
 HEAP32[$6>>2] = $pbEof;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ($9|0)==(0|0);
 if (!($10)) {
  $81 = HEAP32[$0>>2]|0;
  $82 = ($81|0)!=(0);
  $83 = $82 ? -1 : 1;
  HEAP32[$iMul1>>2] = $83;
  $84 = HEAP32[$1>>2]|0;
  _fts3GetReverseVarint($p,$84,$iDelta2);
  $85 = HEAP32[$iMul1>>2]|0;
  $86 = ($85|0)<(0);
  $87 = $86 << 31 >> 31;
  $88 = $iDelta2;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (___muldi3(($85|0),($87|0),($90|0),($93|0))|0);
  $95 = tempRet0;
  $96 = HEAP32[$4>>2]|0;
  $97 = $96;
  $98 = $97;
  $99 = HEAP32[$98>>2]|0;
  $100 = (($97) + 4)|0;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (_i64Subtract(($99|0),($102|0),($94|0),($95|0))|0);
  $104 = tempRet0;
  $105 = $96;
  $106 = $105;
  HEAP32[$106>>2] = $103;
  $107 = (($105) + 4)|0;
  $108 = $107;
  HEAP32[$108>>2] = $104;
  $109 = HEAP32[$p>>2]|0;
  $110 = HEAP32[$1>>2]|0;
  $111 = ($109|0)==($110|0);
  if ($111) {
   $112 = HEAP32[$6>>2]|0;
   HEAP8[$112>>0] = 1;
  } else {
   $113 = HEAP32[$p>>2]|0;
   HEAP32[$pSave>>2] = $113;
   $114 = HEAP32[$1>>2]|0;
   _fts3ReversePoslist($114,$p);
   $115 = HEAP32[$pSave>>2]|0;
   $116 = HEAP32[$p>>2]|0;
   $117 = $115;
   $118 = $116;
   $119 = (($117) - ($118))|0;
   $120 = HEAP32[$5>>2]|0;
   HEAP32[$120>>2] = $119;
  }
  $121 = HEAP32[$p>>2]|0;
  $122 = HEAP32[$3>>2]|0;
  HEAP32[$122>>2] = $121;
  STACKTOP = sp;return;
 }
 $11 = $iDocid;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 HEAP32[$pNext>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 HEAP32[$pDocid>>2] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16)|0);
 HEAP32[$pEnd>>2] = $18;
 HEAP32[$iMul>>2] = 1;
 while(1) {
  $19 = HEAP32[$pDocid>>2]|0;
  $20 = HEAP32[$pEnd>>2]|0;
  $21 = ($19>>>0)<($20>>>0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pDocid>>2]|0;
  $23 = (_sqlite3Fts3GetVarint($22,$iDelta)|0);
  $24 = HEAP32[$pDocid>>2]|0;
  $25 = (($24) + ($23)|0);
  HEAP32[$pDocid>>2] = $25;
  $26 = HEAP32[$iMul>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = $iDelta;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (___muldi3(($26|0),($28|0),($31|0),($34|0))|0);
  $36 = tempRet0;
  $37 = $iDocid;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),($35|0),($36|0))|0);
  $44 = tempRet0;
  $45 = $iDocid;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  $49 = HEAP32[$pDocid>>2]|0;
  HEAP32[$pNext>>2] = $49;
  _fts3PoslistCopy(0,$pDocid);
  while(1) {
   $50 = HEAP32[$pDocid>>2]|0;
   $51 = HEAP32[$pEnd>>2]|0;
   $52 = ($50>>>0)<($51>>>0);
   if (!($52)) {
    break;
   }
   $53 = HEAP32[$pDocid>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54 << 24 >> 24;
   $56 = ($55|0)==(0);
   if (!($56)) {
    break;
   }
   $57 = HEAP32[$pDocid>>2]|0;
   $58 = ((($57)) + 1|0);
   HEAP32[$pDocid>>2] = $58;
  }
  $59 = HEAP32[$0>>2]|0;
  $60 = ($59|0)!=(0);
  $61 = $60 ? -1 : 1;
  HEAP32[$iMul>>2] = $61;
 }
 $62 = HEAP32[$pEnd>>2]|0;
 $63 = HEAP32[$pNext>>2]|0;
 $64 = $62;
 $65 = $63;
 $66 = (($64) - ($65))|0;
 $67 = HEAP32[$5>>2]|0;
 HEAP32[$67>>2] = $66;
 $68 = HEAP32[$pNext>>2]|0;
 $69 = HEAP32[$3>>2]|0;
 HEAP32[$69>>2] = $68;
 $70 = $iDocid;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = (($70) + 4)|0;
 $74 = $73;
 $75 = HEAP32[$74>>2]|0;
 $76 = HEAP32[$4>>2]|0;
 $77 = $76;
 $78 = $77;
 HEAP32[$78>>2] = $72;
 $79 = (($77) + 4)|0;
 $80 = $79;
 HEAP32[$80>>2] = $75;
 STACKTOP = sp;return;
}
function _fts3PoslistCopy($pp,$ppPoslist) {
 $pp = $pp|0;
 $ppPoslist = $ppPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $n = 0;
 var $p = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pEnd = sp + 8|0;
 $c = sp + 20|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $ppPoslist;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pEnd>>2] = $3;
 HEAP8[$c>>0] = 0;
 while(1) {
  $4 = HEAP32[$pEnd>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = HEAP8[$c>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = $6 | $8;
  $10 = ($9|0)!=(0);
  $11 = HEAP32[$pEnd>>2]|0;
  $12 = ((($11)) + 1|0);
  HEAP32[$pEnd>>2] = $12;
  if (!($10)) {
   break;
  }
  $13 = HEAP8[$11>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = $14 & 128;
  $16 = $15&255;
  HEAP8[$c>>0] = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  $36 = HEAP32[$pEnd>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  HEAP32[$37>>2] = $36;
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$pEnd>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = $19;
 $23 = $21;
 $24 = (($22) - ($23))|0;
 HEAP32[$n>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$p>>2] = $26;
 $27 = HEAP32[$p>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$n>>2]|0;
 _memcpy(($27|0),($29|0),($30|0))|0;
 $31 = HEAP32[$n>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = (($32) + ($31)|0);
 HEAP32[$p>>2] = $33;
 $34 = HEAP32[$p>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 HEAP32[$35>>2] = $34;
 $36 = HEAP32[$pEnd>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 HEAP32[$37>>2] = $36;
 STACKTOP = sp;return;
}
function _fts3GetReverseVarint($pp,$pStart,$pVal) {
 $pp = $pp|0;
 $pStart = $pStart|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $iVal = sp;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pStart;
 HEAP32[$2>>2] = $pVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + -2|0);
 HEAP32[$p>>2] = $5;
 while(1) {
  $6 = HEAP32[$p>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ($6>>>0)>=($7>>>0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = $11 & 128;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + -1|0);
  HEAP32[$p>>2] = $15;
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 1|0);
 HEAP32[$p>>2] = $17;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$p>>2]|0;
 (_sqlite3Fts3GetVarint($20,$iVal)|0);
 $21 = $iVal;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = $27;
 $29 = $28;
 HEAP32[$29>>2] = $23;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $26;
 STACKTOP = sp;return;
}
function _fts3ReversePoslist($pStart,$ppPoslist) {
 $pStart = $pStart|0;
 $ppPoslist = $ppPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 $c = sp + 12|0;
 HEAP32[$0>>2] = $pStart;
 HEAP32[$1>>2] = $ppPoslist;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + -2|0);
 HEAP32[$p>>2] = $4;
 HEAP8[$c>>0] = 0;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ($5>>>0)>($6>>>0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$p>>2]|0;
  $9 = ((($8)) + -1|0);
  HEAP32[$p>>2] = $9;
  $10 = HEAP8[$8>>0]|0;
  HEAP8[$c>>0] = $10;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  if (!($12)) {
   break;
  }
 }
 while(1) {
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ($13>>>0)>($14>>>0);
  if ($15) {
   $16 = HEAP32[$p>>2]|0;
   $17 = HEAP8[$16>>0]|0;
   $18 = $17 << 24 >> 24;
   $19 = $18 & 128;
   $20 = HEAP8[$c>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = $19 | $21;
   $23 = ($22|0)!=(0);
   $47 = $23;
  } else {
   $47 = 0;
  }
  $24 = HEAP32[$p>>2]|0;
  if (!($47)) {
   break;
  }
  $25 = ((($24)) + -1|0);
  HEAP32[$p>>2] = $25;
  $26 = HEAP8[$24>>0]|0;
  HEAP8[$c>>0] = $26;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ($24>>>0)>($27>>>0);
 if ($28) {
  label = 11;
 } else {
  $29 = HEAP8[$c>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = ($30|0)==(0);
  if ($31) {
   $32 = HEAP32[$1>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = ((($34)) + 2|0);
   $36 = ($33>>>0)>($35>>>0);
   if ($36) {
    label = 11;
   }
  }
 }
 if ((label|0) == 11) {
  $37 = HEAP32[$p>>2]|0;
  $38 = ((($37)) + 2|0);
  HEAP32[$p>>2] = $38;
 }
 while(1) {
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 1|0);
  HEAP32[$p>>2] = $40;
  $41 = HEAP8[$39>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = $42 & 128;
  $44 = ($43|0)!=(0);
  if (!($44)) {
   break;
  }
 }
 $45 = HEAP32[$p>>2]|0;
 $46 = HEAP32[$1>>2]|0;
 HEAP32[$46>>2] = $45;
 STACKTOP = sp;return;
}
function _fts3ColumnlistCopy($pp,$ppPoslist) {
 $pp = $pp|0;
 $ppPoslist = $ppPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0;
 var $n = 0, $p = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pEnd = sp + 8|0;
 $c = sp + 20|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $ppPoslist;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pEnd>>2] = $3;
 HEAP8[$c>>0] = 0;
 while(1) {
  $4 = HEAP32[$pEnd>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = HEAP8[$c>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = $6 | $8;
  $10 = 254 & $9;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pEnd>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$pEnd>>2] = $13;
  $14 = HEAP8[$12>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = $15 & 128;
  $17 = $16&255;
  HEAP8[$c>>0] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $37 = HEAP32[$pEnd>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  HEAP32[$38>>2] = $37;
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$pEnd>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = $20;
 $24 = $22;
 $25 = (($23) - ($24))|0;
 HEAP32[$n>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$p>>2] = $27;
 $28 = HEAP32[$p>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$n>>2]|0;
 _memcpy(($28|0),($30|0),($31|0))|0;
 $32 = HEAP32[$n>>2]|0;
 $33 = HEAP32[$p>>2]|0;
 $34 = (($33) + ($32)|0);
 HEAP32[$p>>2] = $34;
 $35 = HEAP32[$p>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$pEnd>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 HEAP32[$38>>2] = $37;
 STACKTOP = sp;return;
}
function _simpleCreate($argc,$argv,$ppTokenizer) {
 $argc = $argc|0;
 $argv = $argv|0;
 $ppTokenizer = $ppTokenizer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0, $i = 0, $i1 = 0, $n = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $t = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $ch = sp + 32|0;
 $i1 = sp;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$3>>2] = $ppTokenizer;
 $4 = (_sqlite3_malloc(132)|0);
 HEAP32[$t>>2] = $4;
 $5 = HEAP32[$t>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 7;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 $7 = HEAP32[$t>>2]|0;
 _memset(($7|0),0,132)|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)>(1);
 L5: do {
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   $11 = ((($10)) + 4|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (_strlen($12)|0);
   HEAP32[$n>>2] = $13;
   HEAP32[$i>>2] = 0;
   while(1) {
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$n>>2]|0;
    $16 = ($14|0)<($15|0);
    if (!($16)) {
     break L5;
    }
    $17 = HEAP32[$i>>2]|0;
    $18 = HEAP32[$2>>2]|0;
    $19 = ((($18)) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + ($17)|0);
    $22 = HEAP8[$21>>0]|0;
    HEAP8[$ch>>0] = $22;
    $23 = HEAP8[$ch>>0]|0;
    $24 = $23&255;
    $25 = ($24|0)>=(128);
    if ($25) {
     break;
    }
    $27 = HEAP8[$ch>>0]|0;
    $28 = $27&255;
    $29 = HEAP32[$t>>2]|0;
    $30 = ((($29)) + 4|0);
    $31 = (($30) + ($28)|0);
    HEAP8[$31>>0] = 1;
    $32 = HEAP32[$i>>2]|0;
    $33 = (($32) + 1)|0;
    HEAP32[$i>>2] = $33;
   }
   $26 = HEAP32[$t>>2]|0;
   _sqlite3_free($26);
   HEAP32[$0>>2] = 1;
   $50 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($50|0);
  } else {
   HEAP32[$i1>>2] = 1;
   while(1) {
    $34 = HEAP32[$i1>>2]|0;
    $35 = ($34|0)<(128);
    if (!($35)) {
     break L5;
    }
    $36 = HEAP32[$i1>>2]|0;
    $37 = (_fts3_isalnum($36)|0);
    $38 = ($37|0)!=(0);
    $39 = $38 ^ 1;
    $40 = $39 ? -1 : 0;
    $41 = $40&255;
    $42 = HEAP32[$i1>>2]|0;
    $43 = HEAP32[$t>>2]|0;
    $44 = ((($43)) + 4|0);
    $45 = (($44) + ($42)|0);
    HEAP8[$45>>0] = $41;
    $46 = HEAP32[$i1>>2]|0;
    $47 = (($46) + 1)|0;
    HEAP32[$i1>>2] = $47;
   }
  }
 } while(0);
 $48 = HEAP32[$t>>2]|0;
 $49 = HEAP32[$3>>2]|0;
 HEAP32[$49>>2] = $48;
 HEAP32[$0>>2] = 0;
 $50 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _simpleDestroy($pTokenizer) {
 $pTokenizer = $pTokenizer|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTokenizer;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return 0;
}
function _simpleOpen($pTokenizer,$pInput,$nBytes,$ppCursor) {
 $pTokenizer = $pTokenizer|0;
 $pInput = $pInput|0;
 $nBytes = $nBytes|0;
 $ppCursor = $ppCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $c = sp;
 HEAP32[$1>>2] = $pTokenizer;
 HEAP32[$2>>2] = $pInput;
 HEAP32[$3>>2] = $nBytes;
 HEAP32[$4>>2] = $ppCursor;
 $5 = (_sqlite3_malloc(28)|0);
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 7;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$c>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 do {
  if ($12) {
   $13 = HEAP32[$c>>2]|0;
   $14 = ((($13)) + 8|0);
   HEAP32[$14>>2] = 0;
  } else {
   $15 = HEAP32[$3>>2]|0;
   $16 = ($15|0)<(0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    $18 = (_strlen($17)|0);
    $19 = HEAP32[$c>>2]|0;
    $20 = ((($19)) + 8|0);
    HEAP32[$20>>2] = $18;
    break;
   } else {
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$c>>2]|0;
    $23 = ((($22)) + 8|0);
    HEAP32[$23>>2] = $21;
    break;
   }
  }
 } while(0);
 $24 = HEAP32[$c>>2]|0;
 $25 = ((($24)) + 12|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$c>>2]|0;
 $27 = ((($26)) + 16|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$c>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$c>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$c>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 HEAP32[$33>>2] = $32;
 HEAP32[$0>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _simpleClose($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$c>>2] = $1;
 $2 = HEAP32[$c>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$c>>2]|0;
 _sqlite3_free($5);
 STACKTOP = sp;return 0;
}
function _simpleNext($pCursor,$ppToken,$pnBytes,$piStartOffset,$piEndOffset,$piPosition) {
 $pCursor = $pCursor|0;
 $ppToken = $ppToken|0;
 $pnBytes = $pnBytes|0;
 $piStartOffset = $piStartOffset|0;
 $piEndOffset = $piEndOffset|0;
 $piPosition = $piPosition|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0;
 var $ch = 0, $i = 0, $iStartOffset = 0, $n = 0, $p = 0, $pNew = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $c = sp + 24|0;
 $t = sp + 20|0;
 $p = sp + 16|0;
 $iStartOffset = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $pNew = sp;
 $ch = sp + 56|0;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $ppToken;
 HEAP32[$3>>2] = $pnBytes;
 HEAP32[$4>>2] = $piStartOffset;
 HEAP32[$5>>2] = $piEndOffset;
 HEAP32[$6>>2] = $piPosition;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$c>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$t>>2] = $9;
 $10 = HEAP32[$c>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p>>2] = $12;
 while(1) {
  $13 = HEAP32[$c>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$c>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   label = 25;
   break;
  }
  while(1) {
   $20 = HEAP32[$c>>2]|0;
   $21 = ((($20)) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$c>>2]|0;
   $24 = ((($23)) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($22|0)<($25|0);
   if ($26) {
    $27 = HEAP32[$t>>2]|0;
    $28 = HEAP32[$c>>2]|0;
    $29 = ((($28)) + 12|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$p>>2]|0;
    $32 = (($31) + ($30)|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = (_simpleDelim($27,$33)|0);
    $35 = ($34|0)!=(0);
    $137 = $35;
   } else {
    $137 = 0;
   }
   $36 = HEAP32[$c>>2]|0;
   $37 = ((($36)) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   if (!($137)) {
    break;
   }
   $39 = (($38) + 1)|0;
   HEAP32[$37>>2] = $39;
  }
  HEAP32[$iStartOffset>>2] = $38;
  while(1) {
   $40 = HEAP32[$c>>2]|0;
   $41 = ((($40)) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$c>>2]|0;
   $44 = ((($43)) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($42|0)<($45|0);
   if ($46) {
    $47 = HEAP32[$t>>2]|0;
    $48 = HEAP32[$c>>2]|0;
    $49 = ((($48)) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$p>>2]|0;
    $52 = (($51) + ($50)|0);
    $53 = HEAP8[$52>>0]|0;
    $54 = (_simpleDelim($47,$53)|0);
    $55 = ($54|0)!=(0);
    $56 = $55 ^ 1;
    $138 = $56;
   } else {
    $138 = 0;
   }
   $57 = HEAP32[$c>>2]|0;
   $58 = ((($57)) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   if (!($138)) {
    break;
   }
   $60 = (($59) + 1)|0;
   HEAP32[$58>>2] = $60;
  }
  $61 = HEAP32[$iStartOffset>>2]|0;
  $62 = ($59|0)>($61|0);
  if ($62) {
   break;
  }
 }
 if ((label|0) == 25) {
  HEAP32[$0>>2] = 101;
  $136 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($136|0);
 }
 $63 = HEAP32[$c>>2]|0;
 $64 = ((($63)) + 12|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$iStartOffset>>2]|0;
 $67 = (($65) - ($66))|0;
 HEAP32[$n>>2] = $67;
 $68 = HEAP32[$n>>2]|0;
 $69 = HEAP32[$c>>2]|0;
 $70 = ((($69)) + 24|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($68|0)>($71|0);
 do {
  if ($72) {
   $73 = HEAP32[$n>>2]|0;
   $74 = (($73) + 20)|0;
   $75 = HEAP32[$c>>2]|0;
   $76 = ((($75)) + 24|0);
   HEAP32[$76>>2] = $74;
   $77 = HEAP32[$c>>2]|0;
   $78 = ((($77)) + 20|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = HEAP32[$c>>2]|0;
   $81 = ((($80)) + 24|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (_sqlite3_realloc($79,$82)|0);
   HEAP32[$pNew>>2] = $83;
   $84 = HEAP32[$pNew>>2]|0;
   $85 = ($84|0)!=(0|0);
   if ($85) {
    $86 = HEAP32[$pNew>>2]|0;
    $87 = HEAP32[$c>>2]|0;
    $88 = ((($87)) + 20|0);
    HEAP32[$88>>2] = $86;
    break;
   }
   HEAP32[$0>>2] = 7;
   $136 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($136|0);
  }
 } while(0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $89 = HEAP32[$i>>2]|0;
  $90 = HEAP32[$n>>2]|0;
  $91 = ($89|0)<($90|0);
  if (!($91)) {
   break;
  }
  $92 = HEAP32[$iStartOffset>>2]|0;
  $93 = HEAP32[$i>>2]|0;
  $94 = (($92) + ($93))|0;
  $95 = HEAP32[$p>>2]|0;
  $96 = (($95) + ($94)|0);
  $97 = HEAP8[$96>>0]|0;
  HEAP8[$ch>>0] = $97;
  $98 = HEAP8[$ch>>0]|0;
  $99 = $98&255;
  $100 = ($99|0)>=(65);
  if ($100) {
   $101 = HEAP8[$ch>>0]|0;
   $102 = $101&255;
   $103 = ($102|0)<=(90);
   if ($103) {
    $104 = HEAP8[$ch>>0]|0;
    $105 = $104&255;
    $106 = (($105) - 65)|0;
    $107 = (($106) + 97)|0;
    $111 = $107;
   } else {
    label = 22;
   }
  } else {
   label = 22;
  }
  if ((label|0) == 22) {
   label = 0;
   $108 = HEAP8[$ch>>0]|0;
   $109 = $108&255;
   $111 = $109;
  }
  $110 = $111&255;
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$c>>2]|0;
  $114 = ((($113)) + 20|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) + ($112)|0);
  HEAP8[$116>>0] = $110;
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + 1)|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = HEAP32[$c>>2]|0;
 $120 = ((($119)) + 20|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$2>>2]|0;
 HEAP32[$122>>2] = $121;
 $123 = HEAP32[$n>>2]|0;
 $124 = HEAP32[$3>>2]|0;
 HEAP32[$124>>2] = $123;
 $125 = HEAP32[$iStartOffset>>2]|0;
 $126 = HEAP32[$4>>2]|0;
 HEAP32[$126>>2] = $125;
 $127 = HEAP32[$c>>2]|0;
 $128 = ((($127)) + 12|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = HEAP32[$5>>2]|0;
 HEAP32[$130>>2] = $129;
 $131 = HEAP32[$c>>2]|0;
 $132 = ((($131)) + 16|0);
 $133 = HEAP32[$132>>2]|0;
 $134 = (($133) + 1)|0;
 HEAP32[$132>>2] = $134;
 $135 = HEAP32[$6>>2]|0;
 HEAP32[$135>>2] = $133;
 HEAP32[$0>>2] = 0;
 $136 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($136|0);
}
function _fts3_isalnum($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $x;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>=(48);
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)<=(57);
 $or$cond = $2 & $4;
 if ($or$cond) {
  $15 = 1;
 } else {
  $5 = HEAP32[$0>>2]|0;
  $6 = ($5|0)>=(65);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)<=(90);
  $or$cond3 = $6 & $8;
  if ($or$cond3) {
   $15 = 1;
  } else {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)>=(97);
   $11 = HEAP32[$0>>2]|0;
   $12 = ($11|0)<=(122);
   $13 = $10 ? $12 : 0;
   $15 = $13;
  }
 }
 $14 = $15&1;
 STACKTOP = sp;return ($14|0);
}
function _simpleDelim($t,$c) {
 $t = $t|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $t;
 HEAP8[$1>>0] = $c;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = ($3|0)<(128);
 if (!($4)) {
  $14 = 0;
  $13 = $14&1;
  STACKTOP = sp;return ($13|0);
 }
 $5 = HEAP8[$1>>0]|0;
 $6 = $5&255;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = (($8) + ($6)|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10 << 24 >> 24;
 $12 = ($11|0)!=(0);
 $14 = $12;
 $13 = $14&1;
 STACKTOP = sp;return ($13|0);
}
function _porterCreate($argc,$argv,$ppTokenizer) {
 $argc = $argc|0;
 $argv = $argv|0;
 $ppTokenizer = $ppTokenizer|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $t = sp;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$3>>2] = $ppTokenizer;
 $4 = (_sqlite3_malloc(4)|0);
 HEAP32[$t>>2] = $4;
 $5 = HEAP32[$t>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 7;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $7 = HEAP32[$t>>2]|0;
  ;HEAP32[$7>>2]=0|0;
  $8 = HEAP32[$t>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  HEAP32[$9>>2] = $8;
  HEAP32[$0>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _porterDestroy($pTokenizer) {
 $pTokenizer = $pTokenizer|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTokenizer;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return 0;
}
function _porterOpen($pTokenizer,$zInput,$nInput,$ppCursor) {
 $pTokenizer = $pTokenizer|0;
 $zInput = $zInput|0;
 $nInput = $nInput|0;
 $ppCursor = $ppCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $c = sp;
 HEAP32[$1>>2] = $pTokenizer;
 HEAP32[$2>>2] = $zInput;
 HEAP32[$3>>2] = $nInput;
 HEAP32[$4>>2] = $ppCursor;
 $5 = (_sqlite3_malloc(28)|0);
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 7;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$c>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 do {
  if ($12) {
   $13 = HEAP32[$c>>2]|0;
   $14 = ((($13)) + 8|0);
   HEAP32[$14>>2] = 0;
  } else {
   $15 = HEAP32[$3>>2]|0;
   $16 = ($15|0)<(0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    $18 = (_strlen($17)|0);
    $19 = HEAP32[$c>>2]|0;
    $20 = ((($19)) + 8|0);
    HEAP32[$20>>2] = $18;
    break;
   } else {
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$c>>2]|0;
    $23 = ((($22)) + 8|0);
    HEAP32[$23>>2] = $21;
    break;
   }
  }
 } while(0);
 $24 = HEAP32[$c>>2]|0;
 $25 = ((($24)) + 12|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$c>>2]|0;
 $27 = ((($26)) + 16|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$c>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$c>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$c>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 HEAP32[$33>>2] = $32;
 HEAP32[$0>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _porterClose($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$c>>2] = $1;
 $2 = HEAP32[$c>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$c>>2]|0;
 _sqlite3_free($5);
 STACKTOP = sp;return 0;
}
function _porterNext($pCursor,$pzToken,$pnBytes,$piStartOffset,$piEndOffset,$piPosition) {
 $pCursor = $pCursor|0;
 $pzToken = $pzToken|0;
 $pnBytes = $pnBytes|0;
 $piStartOffset = $piStartOffset|0;
 $piEndOffset = $piEndOffset|0;
 $piPosition = $piPosition|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $ch = 0, $iStartOffset = 0, $n = 0, $pNew = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $c = sp + 20|0;
 $z = sp + 16|0;
 $iStartOffset = sp + 12|0;
 $ch = sp + 8|0;
 $n = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $pzToken;
 HEAP32[$3>>2] = $pnBytes;
 HEAP32[$4>>2] = $piStartOffset;
 HEAP32[$5>>2] = $piEndOffset;
 HEAP32[$6>>2] = $piPosition;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$c>>2] = $7;
 $8 = HEAP32[$c>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$z>>2] = $10;
 while(1) {
  $11 = HEAP32[$c>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$c>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   label = 23;
   break;
  }
  while(1) {
   $18 = HEAP32[$c>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$c>>2]|0;
   $22 = ((($21)) + 8|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($20|0)<($23|0);
   if ($24) {
    $25 = HEAP32[$c>>2]|0;
    $26 = ((($25)) + 12|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$z>>2]|0;
    $29 = (($28) + ($27)|0);
    $30 = HEAP8[$29>>0]|0;
    $31 = $30 << 24 >> 24;
    HEAP32[$ch>>2] = $31;
    $32 = $31 & 128;
    $33 = ($32|0)==(0);
    if ($33) {
     $34 = HEAP32[$ch>>2]|0;
     $35 = ($34|0)<(48);
     if ($35) {
      $128 = 1;
     } else {
      $36 = HEAP32[$ch>>2]|0;
      $37 = (($36) - 48)|0;
      $38 = (42559 + ($37)|0);
      $39 = HEAP8[$38>>0]|0;
      $40 = ($39<<24>>24)!=(0);
      $41 = $40 ^ 1;
      $128 = $41;
     }
    } else {
     $128 = 0;
    }
   } else {
    $128 = 0;
   }
   $42 = HEAP32[$c>>2]|0;
   $43 = ((($42)) + 12|0);
   $44 = HEAP32[$43>>2]|0;
   if (!($128)) {
    break;
   }
   $45 = (($44) + 1)|0;
   HEAP32[$43>>2] = $45;
  }
  HEAP32[$iStartOffset>>2] = $44;
  while(1) {
   $46 = HEAP32[$c>>2]|0;
   $47 = ((($46)) + 12|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$c>>2]|0;
   $50 = ((($49)) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($48|0)<($51|0);
   if ($52) {
    $53 = HEAP32[$c>>2]|0;
    $54 = ((($53)) + 12|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = HEAP32[$z>>2]|0;
    $57 = (($56) + ($55)|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58 << 24 >> 24;
    HEAP32[$ch>>2] = $59;
    $60 = $59 & 128;
    $61 = ($60|0)==(0);
    if ($61) {
     $62 = HEAP32[$ch>>2]|0;
     $63 = ($62|0)<(48);
     if ($63) {
      $71 = 1;
     } else {
      $64 = HEAP32[$ch>>2]|0;
      $65 = (($64) - 48)|0;
      $66 = (42559 + ($65)|0);
      $67 = HEAP8[$66>>0]|0;
      $68 = ($67<<24>>24)!=(0);
      $69 = $68 ^ 1;
      $71 = $69;
     }
    } else {
     $71 = 0;
    }
    $70 = $71 ^ 1;
    $129 = $70;
   } else {
    $129 = 0;
   }
   $72 = HEAP32[$c>>2]|0;
   $73 = ((($72)) + 12|0);
   $74 = HEAP32[$73>>2]|0;
   if (!($129)) {
    break;
   }
   $75 = (($74) + 1)|0;
   HEAP32[$73>>2] = $75;
  }
  $76 = HEAP32[$iStartOffset>>2]|0;
  $77 = ($74|0)>($76|0);
  if ($77) {
   break;
  }
 }
 if ((label|0) == 23) {
  HEAP32[$0>>2] = 101;
  $127 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $78 = HEAP32[$c>>2]|0;
 $79 = ((($78)) + 12|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = HEAP32[$iStartOffset>>2]|0;
 $82 = (($80) - ($81))|0;
 HEAP32[$n>>2] = $82;
 $83 = HEAP32[$n>>2]|0;
 $84 = HEAP32[$c>>2]|0;
 $85 = ((($84)) + 24|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = ($83|0)>($86|0);
 do {
  if ($87) {
   $88 = HEAP32[$n>>2]|0;
   $89 = (($88) + 20)|0;
   $90 = HEAP32[$c>>2]|0;
   $91 = ((($90)) + 24|0);
   HEAP32[$91>>2] = $89;
   $92 = HEAP32[$c>>2]|0;
   $93 = ((($92)) + 20|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = HEAP32[$c>>2]|0;
   $96 = ((($95)) + 24|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = (_sqlite3_realloc($94,$97)|0);
   HEAP32[$pNew>>2] = $98;
   $99 = HEAP32[$pNew>>2]|0;
   $100 = ($99|0)!=(0|0);
   if ($100) {
    $101 = HEAP32[$pNew>>2]|0;
    $102 = HEAP32[$c>>2]|0;
    $103 = ((($102)) + 20|0);
    HEAP32[$103>>2] = $101;
    break;
   }
   HEAP32[$0>>2] = 7;
   $127 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($127|0);
  }
 } while(0);
 $104 = HEAP32[$iStartOffset>>2]|0;
 $105 = HEAP32[$z>>2]|0;
 $106 = (($105) + ($104)|0);
 $107 = HEAP32[$n>>2]|0;
 $108 = HEAP32[$c>>2]|0;
 $109 = ((($108)) + 20|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$3>>2]|0;
 _porter_stemmer($106,$107,$110,$111);
 $112 = HEAP32[$c>>2]|0;
 $113 = ((($112)) + 20|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = HEAP32[$2>>2]|0;
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$iStartOffset>>2]|0;
 $117 = HEAP32[$4>>2]|0;
 HEAP32[$117>>2] = $116;
 $118 = HEAP32[$c>>2]|0;
 $119 = ((($118)) + 12|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = HEAP32[$5>>2]|0;
 HEAP32[$121>>2] = $120;
 $122 = HEAP32[$c>>2]|0;
 $123 = ((($122)) + 16|0);
 $124 = HEAP32[$123>>2]|0;
 $125 = (($124) + 1)|0;
 HEAP32[$123>>2] = $125;
 $126 = HEAP32[$6>>2]|0;
 HEAP32[$126>>2] = $124;
 HEAP32[$0>>2] = 0;
 $127 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($127|0);
}
function _porter_stemmer($zIn,$nIn,$zOut,$pnOut) {
 $zIn = $zIn|0;
 $nIn = $nIn|0;
 $zOut = $zOut|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $j = 0, $or$cond = 0, $z = 0;
 var $z2 = 0, $zReverse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $zReverse = sp + 36|0;
 $z = sp + 4|0;
 $z2 = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nIn;
 HEAP32[$2>>2] = $zOut;
 HEAP32[$3>>2] = $pnOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(3);
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)>=(21);
 $or$cond = $5 | $7;
 if ($or$cond) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  _copy_stemmer($8,$9,$10,$11);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 HEAP32[$j>>2] = 22;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + ($15)|0);
  $18 = HEAP8[$17>>0]|0;
  HEAP8[$c>>0] = $18;
  $19 = HEAP8[$c>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)>=(65);
  if ($21) {
   $22 = HEAP8[$c>>0]|0;
   $23 = $22 << 24 >> 24;
   $24 = ($23|0)<=(90);
   if ($24) {
    $25 = HEAP8[$c>>0]|0;
    $26 = $25 << 24 >> 24;
    $27 = (($26) + 97)|0;
    $28 = (($27) - 65)|0;
    $29 = $28&255;
    $30 = HEAP32[$j>>2]|0;
    $31 = (($zReverse) + ($30)|0);
    HEAP8[$31>>0] = $29;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  if ((label|0) == 8) {
   label = 0;
   $32 = HEAP8[$c>>0]|0;
   $33 = $32 << 24 >> 24;
   $34 = ($33|0)>=(97);
   if (!($34)) {
    label = 11;
    break;
   }
   $35 = HEAP8[$c>>0]|0;
   $36 = $35 << 24 >> 24;
   $37 = ($36|0)<=(122);
   if (!($37)) {
    label = 11;
    break;
   }
   $38 = HEAP8[$c>>0]|0;
   $39 = HEAP32[$j>>2]|0;
   $40 = (($zReverse) + ($39)|0);
   HEAP8[$40>>0] = $38;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
  $47 = HEAP32[$j>>2]|0;
  $48 = (($47) + -1)|0;
  HEAP32[$j>>2] = $48;
 }
 if ((label|0) == 11) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  _copy_stemmer($41,$42,$43,$44);
  STACKTOP = sp;return;
 }
 $49 = ((($zReverse)) + 23|0);
 ;HEAP8[$49>>0]=0|0;HEAP8[$49+1>>0]=0|0;HEAP8[$49+2>>0]=0|0;HEAP8[$49+3>>0]=0|0;HEAP8[$49+4>>0]=0|0;
 $50 = HEAP32[$j>>2]|0;
 $51 = (($50) + 1)|0;
 $52 = (($zReverse) + ($51)|0);
 HEAP32[$z>>2] = $52;
 $53 = HEAP32[$z>>2]|0;
 $54 = HEAP8[$53>>0]|0;
 $55 = $54 << 24 >> 24;
 $56 = ($55|0)==(115);
 if ($56) {
  $57 = (_stem($z,42639,42644,0)|0);
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = (_stem($z,42647,42651,0)|0);
   $60 = ($59|0)!=(0);
   if (!($60)) {
    $61 = (_stem($z,42644,42644,0)|0);
    $62 = ($61|0)!=(0);
    if (!($62)) {
     $63 = HEAP32[$z>>2]|0;
     $64 = ((($63)) + 1|0);
     HEAP32[$z>>2] = $64;
    }
   }
  }
 }
 $65 = HEAP32[$z>>2]|0;
 HEAP32[$z2>>2] = $65;
 $66 = (_stem($z,42653,42657,172)|0);
 $67 = ($66|0)!=(0);
 do {
  if (!($67)) {
   $68 = (_stem($z,42660,17532,173)|0);
   $69 = ($68|0)!=(0);
   if (!($69)) {
    $70 = (_stem($z,42664,17532,173)|0);
    $71 = ($70|0)!=(0);
    if (!($71)) {
     break;
    }
   }
   $72 = HEAP32[$z>>2]|0;
   $73 = HEAP32[$z2>>2]|0;
   $74 = ($72|0)!=($73|0);
   if ($74) {
    $75 = (_stem($z,42667,42670,0)|0);
    $76 = ($75|0)!=(0);
    if (!($76)) {
     $77 = (_stem($z,42674,42677,0)|0);
     $78 = ($77|0)!=(0);
     if (!($78)) {
      $79 = (_stem($z,42681,42684,0)|0);
      $80 = ($79|0)!=(0);
      if (!($80)) {
       $81 = HEAP32[$z>>2]|0;
       $82 = (_doubleConsonant($81)|0);
       $83 = ($82|0)!=(0);
       if ($83) {
        $84 = HEAP32[$z>>2]|0;
        $85 = HEAP8[$84>>0]|0;
        $86 = $85 << 24 >> 24;
        $87 = ($86|0)!=(108);
        if ($87) {
         $88 = HEAP32[$z>>2]|0;
         $89 = HEAP8[$88>>0]|0;
         $90 = $89 << 24 >> 24;
         $91 = ($90|0)!=(115);
         if ($91) {
          $92 = HEAP32[$z>>2]|0;
          $93 = HEAP8[$92>>0]|0;
          $94 = $93 << 24 >> 24;
          $95 = ($94|0)!=(122);
          if ($95) {
           $96 = HEAP32[$z>>2]|0;
           $97 = ((($96)) + 1|0);
           HEAP32[$z>>2] = $97;
           break;
          }
         }
        }
       }
       $98 = HEAP32[$z>>2]|0;
       $99 = (_m_eq_1($98)|0);
       $100 = ($99|0)!=(0);
       if ($100) {
        $101 = HEAP32[$z>>2]|0;
        $102 = (_star_oh($101)|0);
        $103 = ($102|0)!=(0);
        if ($103) {
         $104 = HEAP32[$z>>2]|0;
         $105 = ((($104)) + -1|0);
         HEAP32[$z>>2] = $105;
         HEAP8[$105>>0] = 101;
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $106 = HEAP32[$z>>2]|0;
 $107 = HEAP8[$106>>0]|0;
 $108 = $107 << 24 >> 24;
 $109 = ($108|0)==(121);
 if ($109) {
  $110 = HEAP32[$z>>2]|0;
  $111 = ((($110)) + 1|0);
  $112 = (_hasVowel($111)|0);
  $113 = ($112|0)!=(0);
  if ($113) {
   $114 = HEAP32[$z>>2]|0;
   HEAP8[$114>>0] = 105;
  }
 }
 $115 = HEAP32[$z>>2]|0;
 $116 = ((($115)) + 1|0);
 $117 = HEAP8[$116>>0]|0;
 $118 = $117 << 24 >> 24;
 switch ($118|0) {
 case 97:  {
  $119 = (_stem($z,42688,42670,172)|0);
  $120 = ($119|0)!=(0);
  if (!($120)) {
   (_stem($z,42696,42703,172)|0);
  }
  break;
 }
 case 99:  {
  $121 = (_stem($z,42708,42713,172)|0);
  $122 = ($121|0)!=(0);
  if (!($122)) {
   (_stem($z,42718,42723,172)|0);
  }
  break;
 }
 case 101:  {
  (_stem($z,42728,42684,172)|0);
  break;
 }
 case 103:  {
  (_stem($z,42733,34804,172)|0);
  break;
 }
 case 108:  {
  $123 = (_stem($z,42738,42677,172)|0);
  $124 = ($123|0)!=(0);
  if (!($124)) {
   $125 = (_stem($z,42742,42747,172)|0);
   $126 = ($125|0)!=(0);
   if (!($126)) {
    $127 = (_stem($z,42750,42756,172)|0);
    $128 = ($127|0)!=(0);
    if (!($128)) {
     $129 = (_stem($z,42760,42764,172)|0);
     $130 = ($129|0)!=(0);
     if (!($130)) {
      (_stem($z,42766,42772,172)|0);
     }
    }
   }
  }
  break;
 }
 case 111:  {
  $131 = (_stem($z,42776,42684,172)|0);
  $132 = ($131|0)!=(0);
  if (!($132)) {
   $133 = (_stem($z,42784,42670,172)|0);
   $134 = ($133|0)!=(0);
   if (!($134)) {
    (_stem($z,42790,42670,172)|0);
   }
  }
  break;
 }
 case 115:  {
  $135 = (_stem($z,42795,42747,172)|0);
  $136 = ($135|0)!=(0);
  if (!($136)) {
   $137 = (_stem($z,42801,42809,172)|0);
   $138 = ($137|0)!=(0);
   if (!($138)) {
    $139 = (_stem($z,42813,42821,172)|0);
    $140 = ($139|0)!=(0);
    if (!($140)) {
     (_stem($z,42825,42772,172)|0);
    }
   }
  }
  break;
 }
 case 116:  {
  $141 = (_stem($z,42833,42747,172)|0);
  $142 = ($141|0)!=(0);
  if (!($142)) {
   $143 = (_stem($z,42839,42809,172)|0);
   $144 = ($143|0)!=(0);
   if (!($144)) {
    (_stem($z,42845,42677,172)|0);
   }
  }
  break;
 }
 default: {
 }
 }
 $145 = HEAP32[$z>>2]|0;
 $146 = HEAP8[$145>>0]|0;
 $147 = $146 << 24 >> 24;
 switch ($147|0) {
 case 101:  {
  $148 = (_stem($z,42852,42858,172)|0);
  $149 = ($148|0)!=(0);
  if (!($149)) {
   $150 = (_stem($z,42861,17532,172)|0);
   $151 = ($150|0)!=(0);
   if (!($151)) {
    (_stem($z,42867,42747,172)|0);
   }
  }
  break;
 }
 case 105:  {
  (_stem($z,42873,42858,172)|0);
  break;
 }
 case 108:  {
  $152 = (_stem($z,42879,42858,172)|0);
  $153 = ($152|0)!=(0);
  if (!($153)) {
   (_stem($z,42884,17532,172)|0);
  }
  break;
 }
 case 115:  {
  (_stem($z,42888,17532,172)|0);
  break;
 }
 default: {
 }
 }
 $154 = HEAP32[$z>>2]|0;
 $155 = ((($154)) + 1|0);
 $156 = HEAP8[$155>>0]|0;
 $157 = $156 << 24 >> 24;
 L80: do {
  switch ($157|0) {
  case 97:  {
   $158 = HEAP32[$z>>2]|0;
   $159 = HEAP8[$158>>0]|0;
   $160 = $159 << 24 >> 24;
   $161 = ($160|0)==(108);
   if ($161) {
    $162 = HEAP32[$z>>2]|0;
    $163 = ((($162)) + 2|0);
    $164 = (_m_gt_1($163)|0);
    $165 = ($164|0)!=(0);
    if ($165) {
     $166 = HEAP32[$z>>2]|0;
     $167 = ((($166)) + 2|0);
     HEAP32[$z>>2] = $167;
    }
   }
   break;
  }
  case 99:  {
   $168 = HEAP32[$z>>2]|0;
   $169 = HEAP8[$168>>0]|0;
   $170 = $169 << 24 >> 24;
   $171 = ($170|0)==(101);
   if ($171) {
    $172 = HEAP32[$z>>2]|0;
    $173 = ((($172)) + 2|0);
    $174 = HEAP8[$173>>0]|0;
    $175 = $174 << 24 >> 24;
    $176 = ($175|0)==(110);
    if ($176) {
     $177 = HEAP32[$z>>2]|0;
     $178 = ((($177)) + 3|0);
     $179 = HEAP8[$178>>0]|0;
     $180 = $179 << 24 >> 24;
     $181 = ($180|0)==(97);
     if (!($181)) {
      $182 = HEAP32[$z>>2]|0;
      $183 = ((($182)) + 3|0);
      $184 = HEAP8[$183>>0]|0;
      $185 = $184 << 24 >> 24;
      $186 = ($185|0)==(101);
      if (!($186)) {
       break L80;
      }
     }
     $187 = HEAP32[$z>>2]|0;
     $188 = ((($187)) + 4|0);
     $189 = (_m_gt_1($188)|0);
     $190 = ($189|0)!=(0);
     if ($190) {
      $191 = HEAP32[$z>>2]|0;
      $192 = ((($191)) + 4|0);
      HEAP32[$z>>2] = $192;
     }
    }
   }
   break;
  }
  case 101:  {
   $193 = HEAP32[$z>>2]|0;
   $194 = HEAP8[$193>>0]|0;
   $195 = $194 << 24 >> 24;
   $196 = ($195|0)==(114);
   if ($196) {
    $197 = HEAP32[$z>>2]|0;
    $198 = ((($197)) + 2|0);
    $199 = (_m_gt_1($198)|0);
    $200 = ($199|0)!=(0);
    if ($200) {
     $201 = HEAP32[$z>>2]|0;
     $202 = ((($201)) + 2|0);
     HEAP32[$z>>2] = $202;
    }
   }
   break;
  }
  case 105:  {
   $203 = HEAP32[$z>>2]|0;
   $204 = HEAP8[$203>>0]|0;
   $205 = $204 << 24 >> 24;
   $206 = ($205|0)==(99);
   if ($206) {
    $207 = HEAP32[$z>>2]|0;
    $208 = ((($207)) + 2|0);
    $209 = (_m_gt_1($208)|0);
    $210 = ($209|0)!=(0);
    if ($210) {
     $211 = HEAP32[$z>>2]|0;
     $212 = ((($211)) + 2|0);
     HEAP32[$z>>2] = $212;
    }
   }
   break;
  }
  case 108:  {
   $213 = HEAP32[$z>>2]|0;
   $214 = HEAP8[$213>>0]|0;
   $215 = $214 << 24 >> 24;
   $216 = ($215|0)==(101);
   if ($216) {
    $217 = HEAP32[$z>>2]|0;
    $218 = ((($217)) + 2|0);
    $219 = HEAP8[$218>>0]|0;
    $220 = $219 << 24 >> 24;
    $221 = ($220|0)==(98);
    if ($221) {
     $222 = HEAP32[$z>>2]|0;
     $223 = ((($222)) + 3|0);
     $224 = HEAP8[$223>>0]|0;
     $225 = $224 << 24 >> 24;
     $226 = ($225|0)==(97);
     if (!($226)) {
      $227 = HEAP32[$z>>2]|0;
      $228 = ((($227)) + 3|0);
      $229 = HEAP8[$228>>0]|0;
      $230 = $229 << 24 >> 24;
      $231 = ($230|0)==(105);
      if (!($231)) {
       break L80;
      }
     }
     $232 = HEAP32[$z>>2]|0;
     $233 = ((($232)) + 4|0);
     $234 = (_m_gt_1($233)|0);
     $235 = ($234|0)!=(0);
     if ($235) {
      $236 = HEAP32[$z>>2]|0;
      $237 = ((($236)) + 4|0);
      HEAP32[$z>>2] = $237;
     }
    }
   }
   break;
  }
  case 110:  {
   $238 = HEAP32[$z>>2]|0;
   $239 = HEAP8[$238>>0]|0;
   $240 = $239 << 24 >> 24;
   $241 = ($240|0)==(116);
   if ($241) {
    $242 = HEAP32[$z>>2]|0;
    $243 = ((($242)) + 2|0);
    $244 = HEAP8[$243>>0]|0;
    $245 = $244 << 24 >> 24;
    $246 = ($245|0)==(97);
    $247 = HEAP32[$z>>2]|0;
    if ($246) {
     $248 = ((($247)) + 3|0);
     $249 = (_m_gt_1($248)|0);
     $250 = ($249|0)!=(0);
     if (!($250)) {
      break L80;
     }
     $251 = HEAP32[$z>>2]|0;
     $252 = ((($251)) + 3|0);
     HEAP32[$z>>2] = $252;
     break L80;
    }
    $253 = ((($247)) + 2|0);
    $254 = HEAP8[$253>>0]|0;
    $255 = $254 << 24 >> 24;
    $256 = ($255|0)==(101);
    if ($256) {
     $257 = (_stem($z,42893,17532,174)|0);
     $258 = ($257|0)!=(0);
     if (!($258)) {
      $259 = (_stem($z,42899,17532,174)|0);
      $260 = ($259|0)!=(0);
      if (!($260)) {
       (_stem($z,42904,17532,174)|0);
      }
     }
    }
   }
   break;
  }
  case 111:  {
   $261 = HEAP32[$z>>2]|0;
   $262 = HEAP8[$261>>0]|0;
   $263 = $262 << 24 >> 24;
   $264 = ($263|0)==(117);
   $265 = HEAP32[$z>>2]|0;
   if ($264) {
    $266 = ((($265)) + 2|0);
    $267 = (_m_gt_1($266)|0);
    $268 = ($267|0)!=(0);
    if (!($268)) {
     break L80;
    }
    $269 = HEAP32[$z>>2]|0;
    $270 = ((($269)) + 2|0);
    HEAP32[$z>>2] = $270;
    break L80;
   }
   $271 = ((($265)) + 3|0);
   $272 = HEAP8[$271>>0]|0;
   $273 = $272 << 24 >> 24;
   $274 = ($273|0)==(115);
   if (!($274)) {
    $275 = HEAP32[$z>>2]|0;
    $276 = ((($275)) + 3|0);
    $277 = HEAP8[$276>>0]|0;
    $278 = $277 << 24 >> 24;
    $279 = ($278|0)==(116);
    if (!($279)) {
     break L80;
    }
   }
   (_stem($z,42908,17532,174)|0);
   break;
  }
  case 115:  {
   $280 = HEAP32[$z>>2]|0;
   $281 = HEAP8[$280>>0]|0;
   $282 = $281 << 24 >> 24;
   $283 = ($282|0)==(109);
   if ($283) {
    $284 = HEAP32[$z>>2]|0;
    $285 = ((($284)) + 2|0);
    $286 = HEAP8[$285>>0]|0;
    $287 = $286 << 24 >> 24;
    $288 = ($287|0)==(105);
    if ($288) {
     $289 = HEAP32[$z>>2]|0;
     $290 = ((($289)) + 3|0);
     $291 = (_m_gt_1($290)|0);
     $292 = ($291|0)!=(0);
     if ($292) {
      $293 = HEAP32[$z>>2]|0;
      $294 = ((($293)) + 3|0);
      HEAP32[$z>>2] = $294;
     }
    }
   }
   break;
  }
  case 116:  {
   $295 = (_stem($z,42912,17532,174)|0);
   $296 = ($295|0)!=(0);
   if (!($296)) {
    (_stem($z,42916,17532,174)|0);
   }
   break;
  }
  case 117:  {
   $297 = HEAP32[$z>>2]|0;
   $298 = HEAP8[$297>>0]|0;
   $299 = $298 << 24 >> 24;
   $300 = ($299|0)==(115);
   if ($300) {
    $301 = HEAP32[$z>>2]|0;
    $302 = ((($301)) + 2|0);
    $303 = HEAP8[$302>>0]|0;
    $304 = $303 << 24 >> 24;
    $305 = ($304|0)==(111);
    if ($305) {
     $306 = HEAP32[$z>>2]|0;
     $307 = ((($306)) + 3|0);
     $308 = (_m_gt_1($307)|0);
     $309 = ($308|0)!=(0);
     if ($309) {
      $310 = HEAP32[$z>>2]|0;
      $311 = ((($310)) + 3|0);
      HEAP32[$z>>2] = $311;
     }
    }
   }
   break;
  }
  case 122: case 118:  {
   $312 = HEAP32[$z>>2]|0;
   $313 = HEAP8[$312>>0]|0;
   $314 = $313 << 24 >> 24;
   $315 = ($314|0)==(101);
   if ($315) {
    $316 = HEAP32[$z>>2]|0;
    $317 = ((($316)) + 2|0);
    $318 = HEAP8[$317>>0]|0;
    $319 = $318 << 24 >> 24;
    $320 = ($319|0)==(105);
    if ($320) {
     $321 = HEAP32[$z>>2]|0;
     $322 = ((($321)) + 3|0);
     $323 = (_m_gt_1($322)|0);
     $324 = ($323|0)!=(0);
     if ($324) {
      $325 = HEAP32[$z>>2]|0;
      $326 = ((($325)) + 3|0);
      HEAP32[$z>>2] = $326;
     }
    }
   }
   break;
  }
  default: {
  }
  }
 } while(0);
 $327 = HEAP32[$z>>2]|0;
 $328 = HEAP8[$327>>0]|0;
 $329 = $328 << 24 >> 24;
 $330 = ($329|0)==(101);
 do {
  if ($330) {
   $331 = HEAP32[$z>>2]|0;
   $332 = ((($331)) + 1|0);
   $333 = (_m_gt_1($332)|0);
   $334 = ($333|0)!=(0);
   $335 = HEAP32[$z>>2]|0;
   $336 = ((($335)) + 1|0);
   if ($334) {
    HEAP32[$z>>2] = $336;
    break;
   }
   $337 = (_m_eq_1($336)|0);
   $338 = ($337|0)!=(0);
   if ($338) {
    $339 = HEAP32[$z>>2]|0;
    $340 = ((($339)) + 1|0);
    $341 = (_star_oh($340)|0);
    $342 = ($341|0)!=(0);
    if (!($342)) {
     $343 = HEAP32[$z>>2]|0;
     $344 = ((($343)) + 1|0);
     HEAP32[$z>>2] = $344;
    }
   }
  }
 } while(0);
 $345 = HEAP32[$z>>2]|0;
 $346 = (_m_gt_1($345)|0);
 $347 = ($346|0)!=(0);
 if ($347) {
  $348 = HEAP32[$z>>2]|0;
  $349 = HEAP8[$348>>0]|0;
  $350 = $349 << 24 >> 24;
  $351 = ($350|0)==(108);
  if ($351) {
   $352 = HEAP32[$z>>2]|0;
   $353 = ((($352)) + 1|0);
   $354 = HEAP8[$353>>0]|0;
   $355 = $354 << 24 >> 24;
   $356 = ($355|0)==(108);
   if ($356) {
    $357 = HEAP32[$z>>2]|0;
    $358 = ((($357)) + 1|0);
    HEAP32[$z>>2] = $358;
   }
  }
 }
 $359 = HEAP32[$z>>2]|0;
 $360 = (_strlen($359)|0);
 HEAP32[$i>>2] = $360;
 $361 = HEAP32[$3>>2]|0;
 HEAP32[$361>>2] = $360;
 $362 = HEAP32[$i>>2]|0;
 $363 = HEAP32[$2>>2]|0;
 $364 = (($363) + ($362)|0);
 HEAP8[$364>>0] = 0;
 while(1) {
  $365 = HEAP32[$z>>2]|0;
  $366 = HEAP8[$365>>0]|0;
  $367 = ($366<<24>>24)!=(0);
  if (!($367)) {
   break;
  }
  $368 = HEAP32[$z>>2]|0;
  $369 = ((($368)) + 1|0);
  HEAP32[$z>>2] = $369;
  $370 = HEAP8[$368>>0]|0;
  $371 = HEAP32[$i>>2]|0;
  $372 = (($371) + -1)|0;
  HEAP32[$i>>2] = $372;
  $373 = HEAP32[$2>>2]|0;
  $374 = (($373) + ($372)|0);
  HEAP8[$374>>0] = $370;
 }
 STACKTOP = sp;return;
}
function _copy_stemmer($zIn,$nIn,$zOut,$pnOut) {
 $zIn = $zIn|0;
 $nIn = $nIn|0;
 $zOut = $zOut|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $hasDigit = 0, $i = 0, $j = 0, $mx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $mx = sp + 8|0;
 $j = sp + 4|0;
 $hasDigit = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nIn;
 HEAP32[$2>>2] = $zOut;
 HEAP32[$3>>2] = $pnOut;
 HEAP32[$hasDigit>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = HEAP8[$9>>0]|0;
  HEAP8[$c>>0] = $10;
  $11 = HEAP8[$c>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)>=(65);
  if ($13) {
   $14 = HEAP8[$c>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)<=(90);
   if ($16) {
    $17 = HEAP8[$c>>0]|0;
    $18 = $17 << 24 >> 24;
    $19 = (($18) - 65)|0;
    $20 = (($19) + 97)|0;
    $21 = $20&255;
    $22 = HEAP32[$i>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = (($23) + ($22)|0);
    HEAP8[$24>>0] = $21;
   } else {
    label = 6;
   }
  } else {
   label = 6;
  }
  if ((label|0) == 6) {
   label = 0;
   $25 = HEAP8[$c>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = ($26|0)>=(48);
   if ($27) {
    $28 = HEAP8[$c>>0]|0;
    $29 = $28 << 24 >> 24;
    $30 = ($29|0)<=(57);
    if ($30) {
     HEAP32[$hasDigit>>2] = 1;
    }
   }
   $31 = HEAP8[$c>>0]|0;
   $32 = HEAP32[$i>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = (($33) + ($32)|0);
   HEAP8[$34>>0] = $31;
  }
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$i>>2] = $36;
 }
 $37 = HEAP32[$hasDigit>>2]|0;
 $38 = ($37|0)!=(0);
 $39 = $38 ? 3 : 10;
 HEAP32[$mx>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$mx>>2]|0;
 $42 = $41<<1;
 $43 = ($40|0)>($42|0);
 if (!($43)) {
  $63 = HEAP32[$i>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = (($64) + ($63)|0);
  HEAP8[$65>>0] = 0;
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$3>>2]|0;
  HEAP32[$67>>2] = $66;
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$mx>>2]|0;
 HEAP32[$j>>2] = $44;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$mx>>2]|0;
 $47 = (($45) - ($46))|0;
 HEAP32[$i>>2] = $47;
 while(1) {
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = ($48|0)<($49|0);
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + ($51)|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = HEAP32[$j>>2]|0;
  $56 = HEAP32[$2>>2]|0;
  $57 = (($56) + ($55)|0);
  HEAP8[$57>>0] = $54;
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
  $60 = HEAP32[$j>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$j>>2] = $61;
 }
 $62 = HEAP32[$j>>2]|0;
 HEAP32[$i>>2] = $62;
 $63 = HEAP32[$i>>2]|0;
 $64 = HEAP32[$2>>2]|0;
 $65 = (($64) + ($63)|0);
 HEAP8[$65>>0] = 0;
 $66 = HEAP32[$i>>2]|0;
 $67 = HEAP32[$3>>2]|0;
 HEAP32[$67>>2] = $66;
 STACKTOP = sp;return;
}
function _stem($pz,$zFrom,$zTo,$xCond) {
 $pz = $pz|0;
 $zFrom = $zFrom|0;
 $zTo = $zTo|0;
 $xCond = $xCond|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $pz;
 HEAP32[$2>>2] = $zFrom;
 HEAP32[$3>>2] = $zTo;
 HEAP32[$4>>2] = $xCond;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$z>>2] = $6;
 while(1) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = HEAP32[$z>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = ($13|0)==($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$z>>2]|0;
  $19 = ((($18)) + 1|0);
  HEAP32[$z>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 1|0);
  HEAP32[$2>>2] = $21;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP8[$22>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)!=(0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $26 = HEAP32[$4>>2]|0;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$4>>2]|0;
  $29 = HEAP32[$z>>2]|0;
  $30 = (FUNCTION_TABLE_ii[$28 & 255]($29)|0);
  $31 = ($30|0)!=(0);
  if (!($31)) {
   HEAP32[$0>>2] = 1;
   $42 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($42|0);
  }
 }
 while(1) {
  $32 = HEAP32[$3>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = ($33<<24>>24)!=(0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 1|0);
  HEAP32[$3>>2] = $36;
  $37 = HEAP8[$35>>0]|0;
  $38 = HEAP32[$z>>2]|0;
  $39 = ((($38)) + -1|0);
  HEAP32[$z>>2] = $39;
  HEAP8[$39>>0] = $37;
 }
 $40 = HEAP32[$z>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 HEAP32[$41>>2] = $40;
 HEAP32[$0>>2] = 1;
 $42 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _m_gt_0($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (_isVowel($2)|0);
  $4 = ($3|0)!=(0);
  $5 = HEAP32[$1>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 1|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP8[$5>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_isConsonant($10)|0);
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   break;
  }
  $14 = ((($13)) + 1|0);
  HEAP32[$1>>2] = $14;
 }
 $15 = HEAP8[$13>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)!=(0);
 $18 = $17&1;
 HEAP32[$0>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _hasVowel($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $z;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = (_isConsonant($1)|0);
  $3 = ($2|0)!=(0);
  $4 = HEAP32[$0>>2]|0;
  if (!($3)) {
   break;
  }
  $5 = ((($4)) + 1|0);
  HEAP32[$0>>2] = $5;
 }
 $6 = HEAP8[$4>>0]|0;
 $7 = $6 << 24 >> 24;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _doubleConsonant($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_isConsonant($1)|0);
 $3 = ($2|0)!=(0);
 if (!($3)) {
  $13 = 0;
  $12 = $13&1;
  STACKTOP = sp;return ($12|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 1|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($6|0)==($10|0);
 $13 = $11;
 $12 = $13&1;
 STACKTOP = sp;return ($12|0);
}
function _m_eq_1($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (_isVowel($2)|0);
  $4 = ($3|0)!=(0);
  $5 = HEAP32[$1>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 1|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP8[$5>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_isConsonant($10)|0);
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   break;
  }
  $14 = ((($13)) + 1|0);
  HEAP32[$1>>2] = $14;
 }
 $15 = HEAP8[$13>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (_isVowel($18)|0);
  $20 = ($19|0)!=(0);
  $21 = HEAP32[$1>>2]|0;
  if (!($20)) {
   break;
  }
  $22 = ((($21)) + 1|0);
  HEAP32[$1>>2] = $22;
 }
 $23 = HEAP8[$21>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP32[$0>>2] = 1;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (_isConsonant($26)|0);
  $28 = ($27|0)!=(0);
  $29 = HEAP32[$1>>2]|0;
  if (!($28)) {
   break;
  }
  $30 = ((($29)) + 1|0);
  HEAP32[$1>>2] = $30;
 }
 $31 = HEAP8[$29>>0]|0;
 $32 = $31 << 24 >> 24;
 $33 = ($32|0)==(0);
 $34 = $33&1;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _star_oh($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_isConsonant($1)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)!=(119);
  if ($7) {
   $8 = HEAP32[$0>>2]|0;
   $9 = HEAP8[$8>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)!=(120);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = HEAP8[$12>>0]|0;
    $14 = $13 << 24 >> 24;
    $15 = ($14|0)!=(121);
    if ($15) {
     $16 = HEAP32[$0>>2]|0;
     $17 = ((($16)) + 1|0);
     $18 = (_isVowel($17)|0);
     $19 = ($18|0)!=(0);
     if ($19) {
      $20 = HEAP32[$0>>2]|0;
      $21 = ((($20)) + 2|0);
      $22 = (_isConsonant($21)|0);
      $23 = ($22|0)!=(0);
      $25 = $23;
     } else {
      $25 = 0;
     }
    } else {
     $25 = 0;
    }
   } else {
    $25 = 0;
   }
  } else {
   $25 = 0;
  }
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 STACKTOP = sp;return ($24|0);
}
function _m_gt_1($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (_isVowel($2)|0);
  $4 = ($3|0)!=(0);
  $5 = HEAP32[$1>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 1|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP8[$5>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_isConsonant($10)|0);
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   break;
  }
  $14 = ((($13)) + 1|0);
  HEAP32[$1>>2] = $14;
 }
 $15 = HEAP8[$13>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (_isVowel($18)|0);
  $20 = ($19|0)!=(0);
  $21 = HEAP32[$1>>2]|0;
  if (!($20)) {
   break;
  }
  $22 = ((($21)) + 1|0);
  HEAP32[$1>>2] = $22;
 }
 $23 = HEAP8[$21>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (_isConsonant($26)|0);
  $28 = ($27|0)!=(0);
  $29 = HEAP32[$1>>2]|0;
  if (!($28)) {
   break;
  }
  $30 = ((($29)) + 1|0);
  HEAP32[$1>>2] = $30;
 }
 $31 = HEAP8[$29>>0]|0;
 $32 = $31 << 24 >> 24;
 $33 = ($32|0)!=(0);
 $34 = $33&1;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _isVowel($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $j = sp;
 $x = sp + 12|0;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 HEAP8[$x>>0] = $3;
 $4 = HEAP8[$x>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $7 = HEAP8[$x>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = (($8) - 97)|0;
 $10 = (42920 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 HEAP32[$j>>2] = $12;
 $13 = HEAP32[$j>>2]|0;
 $14 = ($13|0)<(2);
 if ($14) {
  $15 = HEAP32[$j>>2]|0;
  $16 = (1 - ($15))|0;
  HEAP32[$0>>2] = $16;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 1|0);
  $19 = (_isConsonant($18)|0);
  HEAP32[$0>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _isConsonant($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $j = sp;
 $x = sp + 12|0;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 HEAP8[$x>>0] = $3;
 $4 = HEAP8[$x>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = HEAP8[$x>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = (($8) - 97)|0;
 $10 = (42920 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 HEAP32[$j>>2] = $12;
 $13 = HEAP32[$j>>2]|0;
 $14 = ($13|0)<(2);
 if ($14) {
  $15 = HEAP32[$j>>2]|0;
  HEAP32[$0>>2] = $15;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 1|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18 << 24 >> 24;
 $20 = ($19|0)==(0);
 if ($20) {
  $26 = 1;
 } else {
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 1|0);
  $23 = (_isVowel($22)|0);
  $24 = ($23|0)!=(0);
  $26 = $24;
 }
 $25 = $26&1;
 HEAP32[$0>>2] = $25;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _fts3RemoveElementByHash($pH,$elem,$h) {
 $pH = $pH|0;
 $elem = $elem|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pEntry = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $elem;
 HEAP32[$2>>2] = $h;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 if ($6) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$11>>2] = $8;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  HEAP32[$13>>2] = $8;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  HEAP32[$22>>2] = $19;
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($23<<3)|0);
 HEAP32[$pEntry>>2] = $27;
 $28 = HEAP32[$pEntry>>2]|0;
 $29 = ((($28)) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = ($30|0)==($31|0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pEntry>>2]|0;
  $36 = ((($35)) + 4|0);
  HEAP32[$36>>2] = $34;
 }
 $37 = HEAP32[$pEntry>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + -1)|0;
 HEAP32[$37>>2] = $39;
 $40 = HEAP32[$pEntry>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)<=(0);
 if ($42) {
  $43 = HEAP32[$pEntry>>2]|0;
  $44 = ((($43)) + 4|0);
  HEAP32[$44>>2] = 0;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 1|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47 << 24 >> 24;
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 12|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($52|0)!=(0|0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   _fts3HashFree($56);
  }
 }
 $57 = HEAP32[$1>>2]|0;
 _fts3HashFree($57);
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + -1)|0;
 HEAP32[$59>>2] = $61;
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 4|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = ($64|0)<=(0);
 if (!($65)) {
  STACKTOP = sp;return;
 }
 $66 = HEAP32[$0>>2]|0;
 _sqlite3Fts3HashClear($66);
 STACKTOP = sp;return;
}
function _fts3Rehash($pH,$new_size) {
 $pH = $pH|0;
 $new_size = $new_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_ht = 0, $next_elem = 0, $xHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $new_ht = sp + 16|0;
 $elem = sp + 12|0;
 $next_elem = sp + 8|0;
 $xHash = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $new_size;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<3;
 $5 = (_fts3HashMalloc($4)|0);
 HEAP32[$new_ht>>2] = $5;
 $6 = HEAP32[$new_ht>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 _fts3HashFree($10);
 $11 = HEAP32[$new_ht>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 16|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18 << 24 >> 24;
 $20 = (_ftsHashFunction($19)|0);
 HEAP32[$xHash>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$elem>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 8|0);
 HEAP32[$25>>2] = 0;
 while(1) {
  $26 = HEAP32[$elem>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$xHash>>2]|0;
  $29 = HEAP32[$elem>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$elem>>2]|0;
  $33 = ((($32)) + 16|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (FUNCTION_TABLE_iii[$28 & 255]($31,$34)|0);
  $36 = HEAP32[$2>>2]|0;
  $37 = (($36) - 1)|0;
  $38 = $35 & $37;
  HEAP32[$h>>2] = $38;
  $39 = HEAP32[$elem>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$next_elem>>2] = $40;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$h>>2]|0;
  $43 = HEAP32[$new_ht>>2]|0;
  $44 = (($43) + ($42<<3)|0);
  $45 = HEAP32[$elem>>2]|0;
  _fts3HashInsertElement($41,$44,$45);
  $46 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $46;
 }
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _fts3HashMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_malloc($1)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $7 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _memset(($5|0),0,($6|0))|0;
 $7 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _fts3HashFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return;
}
function _fts3HashInsertElement($pH,$pEntry,$pNew) {
 $pH = $pH|0;
 $pEntry = $pEntry|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pHead = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pHead = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pEntry;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pHead>>2] = $5;
 $6 = HEAP32[$pHead>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  HEAP32[$31>>2] = $30;
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)!=(0|0);
  if ($35) {
   $36 = HEAP32[$2>>2]|0;
   $37 = HEAP32[$0>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 4|0);
   HEAP32[$40>>2] = $36;
  }
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 4|0);
  HEAP32[$42>>2] = 0;
  $43 = HEAP32[$2>>2]|0;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  HEAP32[$45>>2] = $43;
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$46>>2] = $48;
  $49 = HEAP32[$2>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 4|0);
  HEAP32[$51>>2] = $49;
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pHead>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$pHead>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$pHead>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$2>>2]|0;
 if ($18) {
  $20 = HEAP32[$pHead>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$22>>2] = $19;
 } else {
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 8|0);
  HEAP32[$24>>2] = $19;
 }
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$pHead>>2]|0;
 $27 = ((($26)) + 4|0);
 HEAP32[$27>>2] = $25;
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$2>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 4|0);
 HEAP32[$51>>2] = $49;
 STACKTOP = sp;return;
}
function _scalarFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0, $pHash = 0, $pPtr = 0, $vararg_buffer = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pHash = sp + 20|0;
 $pPtr = sp + 16|0;
 $zName = sp + 12|0;
 $nName = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$pPtr>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_user_data($3)|0);
 HEAP32[$pHash>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zName>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3_value_bytes($9)|0);
 $11 = (($10) + 1)|0;
 HEAP32[$nName>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(2);
 if ($13) {
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($14,42946,-1);
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$zName>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$pHash>>2]|0;
  $18 = HEAP32[$zName>>2]|0;
  $19 = HEAP32[$nName>>2]|0;
  $20 = (_sqlite3Fts3HashFind($17,$18,$19)|0);
  HEAP32[$pPtr>>2] = $20;
 }
 $21 = HEAP32[$pPtr>>2]|0;
 $22 = ($21|0)!=(0|0);
 if ($22) {
  $28 = HEAP32[$0>>2]|0;
  _sqlite3_result_blob($28,$pPtr,4,(-1));
  STACKTOP = sp;return;
 } else {
  $23 = HEAP32[$zName>>2]|0;
  HEAP32[$vararg_buffer>>2] = $23;
  $24 = (_sqlite3_mprintf(43015,$vararg_buffer)|0);
  HEAP32[$zErr>>2] = $24;
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$zErr>>2]|0;
  _sqlite3_result_error($25,$26,-1);
  $27 = HEAP32[$zErr>>2]|0;
  _sqlite3_free($27);
  STACKTOP = sp;return;
 }
}
function _sqlite3Fts3HashFind($pH,$pKey,$nKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pElem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pElem = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pKey;
 HEAP32[$2>>2] = $nKey;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3Fts3HashFindElem($3,$4,$5)|0);
 HEAP32[$pElem>>2] = $6;
 $7 = HEAP32[$pElem>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $12 = 0;
  STACKTOP = sp;return ($12|0);
 }
 $9 = HEAP32[$pElem>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11;
 STACKTOP = sp;return ($12|0);
}
function _fts3CreateMethod($db,$pAux,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pAux = $pAux|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pAux;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $ppVtab;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$5>>2]|0;
 $12 = (_fts3InitVtab(1,$6,$7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _fts3ConnectMethod($db,$pAux,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pAux = $pAux|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pAux;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $ppVtab;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$5>>2]|0;
 $12 = (_fts3InitVtab(0,$6,$7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _fts3BestIndexMethod($pVTab,$pInfo) {
 $pVTab = $pVTab|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDocid = 0, $i = 0, $iCons = 0, $iDocidGe = 0, $iDocidLe = 0, $iIdx = 0, $iLangidCons = 0, $or$cond = 0, $p = 0, $pCons = 0, $pOrder = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $p = sp + 36|0;
 $i = sp + 32|0;
 $iCons = sp + 28|0;
 $iLangidCons = sp + 24|0;
 $iDocidGe = sp + 20|0;
 $iDocidLe = sp + 16|0;
 $iIdx = sp + 12|0;
 $bDocid = sp + 8|0;
 $pCons = sp + 4|0;
 $pOrder = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 HEAP32[$iCons>>2] = -1;
 HEAP32[$iLangidCons>>2] = -1;
 HEAP32[$iDocidGe>>2] = -1;
 HEAP32[$iDocidLe>>2] = -1;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 20|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 40|0);
 HEAPF64[$7>>3] = 5.0E+6;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*12)|0)|0);
  HEAP32[$pCons>>2] = $16;
  $17 = HEAP32[$pCons>>2]|0;
  $18 = ((($17)) + 5|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)==(0);
  $22 = HEAP32[$pCons>>2]|0;
  L4: do {
   if ($21) {
    $23 = ((($22)) + 4|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24&255;
    $26 = ($25|0)==(64);
    if ($26) {
     label = 5;
     break L1;
    }
   } else {
    $32 = HEAP32[$22>>2]|0;
    $33 = ($32|0)<(0);
    if ($33) {
     $42 = 1;
    } else {
     $34 = HEAP32[$pCons>>2]|0;
     $35 = HEAP32[$34>>2]|0;
     $36 = HEAP32[$p>>2]|0;
     $37 = ((($36)) + 24|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = (($38) + 1)|0;
     $40 = ($35|0)==($39|0);
     $42 = $40;
    }
    $41 = $42&1;
    HEAP32[$bDocid>>2] = $41;
    $43 = HEAP32[$iCons>>2]|0;
    $44 = ($43|0)<(0);
    if ($44) {
     $45 = HEAP32[$pCons>>2]|0;
     $46 = ((($45)) + 4|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = $47&255;
     $49 = ($48|0)==(2);
     $50 = HEAP32[$bDocid>>2]|0;
     $51 = ($50|0)!=(0);
     $or$cond = $49 & $51;
     if ($or$cond) {
      $52 = HEAP32[$2>>2]|0;
      $53 = ((($52)) + 20|0);
      HEAP32[$53>>2] = 1;
      $54 = HEAP32[$2>>2]|0;
      $55 = ((($54)) + 40|0);
      HEAPF64[$55>>3] = 1.0;
      $56 = HEAP32[$i>>2]|0;
      HEAP32[$iCons>>2] = $56;
     }
    }
    $57 = HEAP32[$pCons>>2]|0;
    $58 = ((($57)) + 4|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = ($60|0)==(64);
    if ($61) {
     $62 = HEAP32[$pCons>>2]|0;
     $63 = HEAP32[$62>>2]|0;
     $64 = ($63|0)>=(0);
     if ($64) {
      $65 = HEAP32[$pCons>>2]|0;
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$p>>2]|0;
      $68 = ((($67)) + 24|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = ($66|0)<=($69|0);
      if ($70) {
       $71 = HEAP32[$pCons>>2]|0;
       $72 = HEAP32[$71>>2]|0;
       $73 = (2 + ($72))|0;
       $74 = HEAP32[$2>>2]|0;
       $75 = ((($74)) + 20|0);
       HEAP32[$75>>2] = $73;
       $76 = HEAP32[$2>>2]|0;
       $77 = ((($76)) + 40|0);
       HEAPF64[$77>>3] = 2.0;
       $78 = HEAP32[$i>>2]|0;
       HEAP32[$iCons>>2] = $78;
      }
     }
    }
    $79 = HEAP32[$pCons>>2]|0;
    $80 = ((($79)) + 4|0);
    $81 = HEAP8[$80>>0]|0;
    $82 = $81&255;
    $83 = ($82|0)==(2);
    if ($83) {
     $84 = HEAP32[$pCons>>2]|0;
     $85 = HEAP32[$84>>2]|0;
     $86 = HEAP32[$p>>2]|0;
     $87 = ((($86)) + 24|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = (($88) + 2)|0;
     $90 = ($85|0)==($89|0);
     if ($90) {
      $91 = HEAP32[$i>>2]|0;
      HEAP32[$iLangidCons>>2] = $91;
     }
    }
    $92 = HEAP32[$bDocid>>2]|0;
    $93 = ($92|0)!=(0);
    if ($93) {
     $94 = HEAP32[$pCons>>2]|0;
     $95 = ((($94)) + 4|0);
     $96 = HEAP8[$95>>0]|0;
     $97 = $96&255;
     switch ($97|0) {
     case 4: case 32:  {
      $98 = HEAP32[$i>>2]|0;
      HEAP32[$iDocidGe>>2] = $98;
      break L4;
      break;
     }
     case 16: case 8:  {
      $99 = HEAP32[$i>>2]|0;
      HEAP32[$iDocidLe>>2] = $99;
      break L4;
      break;
     }
     default: {
      break L4;
     }
     }
    }
   }
  } while(0);
  $100 = HEAP32[$i>>2]|0;
  $101 = (($100) + 1)|0;
  HEAP32[$i>>2] = $101;
 }
 if ((label|0) == 5) {
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 20|0);
  HEAP32[$28>>2] = 0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 40|0);
  HEAPF64[$30>>3] = 1.0000000000000001E+50;
  $31 = HEAP32[$2>>2]|0;
  _fts3SetEstimatedRows($31,0,262144);
  HEAP32[$0>>2] = 0;
  $186 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($186|0);
 }
 $102 = HEAP32[$2>>2]|0;
 $103 = ((($102)) + 20|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = ($104|0)==(1);
 if ($105) {
  $106 = HEAP32[$2>>2]|0;
  _fts3SetUniqueFlag($106);
 }
 HEAP32[$iIdx>>2] = 1;
 $107 = HEAP32[$iCons>>2]|0;
 $108 = ($107|0)>=(0);
 if ($108) {
  $109 = HEAP32[$iIdx>>2]|0;
  $110 = (($109) + 1)|0;
  HEAP32[$iIdx>>2] = $110;
  $111 = HEAP32[$iCons>>2]|0;
  $112 = HEAP32[$2>>2]|0;
  $113 = ((($112)) + 16|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = (($114) + ($111<<3)|0);
  HEAP32[$115>>2] = $109;
  $116 = HEAP32[$iCons>>2]|0;
  $117 = HEAP32[$2>>2]|0;
  $118 = ((($117)) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = (($119) + ($116<<3)|0);
  $121 = ((($120)) + 4|0);
  HEAP8[$121>>0] = 1;
 }
 $122 = HEAP32[$iLangidCons>>2]|0;
 $123 = ($122|0)>=(0);
 if ($123) {
  $124 = HEAP32[$2>>2]|0;
  $125 = ((($124)) + 20|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = $126 | 65536;
  HEAP32[$125>>2] = $127;
  $128 = HEAP32[$iIdx>>2]|0;
  $129 = (($128) + 1)|0;
  HEAP32[$iIdx>>2] = $129;
  $130 = HEAP32[$iLangidCons>>2]|0;
  $131 = HEAP32[$2>>2]|0;
  $132 = ((($131)) + 16|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = (($133) + ($130<<3)|0);
  HEAP32[$134>>2] = $128;
 }
 $135 = HEAP32[$iDocidGe>>2]|0;
 $136 = ($135|0)>=(0);
 if ($136) {
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 20|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = $139 | 131072;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$iIdx>>2]|0;
  $142 = (($141) + 1)|0;
  HEAP32[$iIdx>>2] = $142;
  $143 = HEAP32[$iDocidGe>>2]|0;
  $144 = HEAP32[$2>>2]|0;
  $145 = ((($144)) + 16|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = (($146) + ($143<<3)|0);
  HEAP32[$147>>2] = $141;
 }
 $148 = HEAP32[$iDocidLe>>2]|0;
 $149 = ($148|0)>=(0);
 if ($149) {
  $150 = HEAP32[$2>>2]|0;
  $151 = ((($150)) + 20|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = $152 | 262144;
  HEAP32[$151>>2] = $153;
  $154 = HEAP32[$iIdx>>2]|0;
  $155 = (($154) + 1)|0;
  HEAP32[$iIdx>>2] = $155;
  $156 = HEAP32[$iDocidLe>>2]|0;
  $157 = HEAP32[$2>>2]|0;
  $158 = ((($157)) + 16|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = (($159) + ($156<<3)|0);
  HEAP32[$160>>2] = $154;
 }
 $161 = HEAP32[$2>>2]|0;
 $162 = ((($161)) + 8|0);
 $163 = HEAP32[$162>>2]|0;
 $164 = ($163|0)==(1);
 do {
  if ($164) {
   $165 = HEAP32[$2>>2]|0;
   $166 = ((($165)) + 12|0);
   $167 = HEAP32[$166>>2]|0;
   HEAP32[$pOrder>>2] = $167;
   $168 = HEAP32[$pOrder>>2]|0;
   $169 = HEAP32[$168>>2]|0;
   $170 = ($169|0)<(0);
   if (!($170)) {
    $171 = HEAP32[$pOrder>>2]|0;
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$p>>2]|0;
    $174 = ((($173)) + 24|0);
    $175 = HEAP32[$174>>2]|0;
    $176 = (($175) + 1)|0;
    $177 = ($172|0)==($176|0);
    if (!($177)) {
     break;
    }
   }
   $178 = HEAP32[$pOrder>>2]|0;
   $179 = ((($178)) + 4|0);
   $180 = HEAP8[$179>>0]|0;
   $181 = ($180<<24>>24)!=(0);
   $182 = HEAP32[$2>>2]|0;
   $183 = ((($182)) + 24|0);
   if ($181) {
    HEAP32[$183>>2] = 44060;
   } else {
    HEAP32[$183>>2] = 44065;
   }
   $184 = HEAP32[$2>>2]|0;
   $185 = ((($184)) + 32|0);
   HEAP32[$185>>2] = 1;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $186 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($186|0);
}
function _fts3DisconnectMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = ($2|0)<(40);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 56|0);
  $7 = (($6) + ($4<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  (_sqlite3_finalize($8)|0);
  $9 = HEAP32[$i>>2]|0;
  $10 = (($9) + 1)|0;
  HEAP32[$i>>2] = $10;
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 240|0);
 $13 = HEAP32[$12>>2]|0;
 _sqlite3_free($13);
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 216|0);
 $16 = HEAP32[$15>>2]|0;
 _sqlite3_free($16);
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 220|0);
 $19 = HEAP32[$18>>2]|0;
 _sqlite3_free($19);
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 40|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3_free($22);
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP32[$24>>2]|0;
 _sqlite3_free($25);
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 36|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ((($29)) + 8|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 36|0);
 $34 = HEAP32[$33>>2]|0;
 (FUNCTION_TABLE_ii[$31 & 255]($34)|0);
 $35 = HEAP32[$p>>2]|0;
 _sqlite3_free($35);
 STACKTOP = sp;return 0;
}
function _fts3DestroyMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer14 = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer14 = sp + 32|0;
 $vararg_buffer10 = sp + 24|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $p = sp + 52|0;
 $rc = sp + 48|0;
 $zDb = sp + 44|0;
 $db = sp + 40|0;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$zDb>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$zDb>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $16;
  _fts3DbExec($rc,$12,44069,$vararg_buffer);
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$zDb>>2]|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $18;
 $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $21;
 _fts3DbExec($rc,$17,44106,$vararg_buffer2);
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$zDb>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 20|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$vararg_buffer6>>2] = $23;
 $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
 HEAP32[$vararg_ptr9>>2] = $26;
 _fts3DbExec($rc,$22,44144,$vararg_buffer6);
 $27 = HEAP32[$db>>2]|0;
 $28 = HEAP32[$zDb>>2]|0;
 $29 = HEAP32[$p>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$vararg_buffer10>>2] = $28;
 $vararg_ptr13 = ((($vararg_buffer10)) + 4|0);
 HEAP32[$vararg_ptr13>>2] = $31;
 _fts3DbExec($rc,$27,44180,$vararg_buffer10);
 $32 = HEAP32[$db>>2]|0;
 $33 = HEAP32[$zDb>>2]|0;
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 20|0);
 $36 = HEAP32[$35>>2]|0;
 HEAP32[$vararg_buffer14>>2] = $33;
 $vararg_ptr17 = ((($vararg_buffer14)) + 4|0);
 HEAP32[$vararg_ptr17>>2] = $36;
 _fts3DbExec($rc,$32,44217,$vararg_buffer14);
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  $40 = (_fts3DisconnectMethod($39)|0);
  $42 = $40;
  STACKTOP = sp;return ($42|0);
 } else {
  $41 = HEAP32[$rc>>2]|0;
  $42 = $41;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _fts3OpenMethod($pVTab,$ppCsr) {
 $pVTab = $pVTab|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pCsr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $ppCsr;
 $3 = (_sqlite3_malloc(96)|0);
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$4>>2] = $3;
 $5 = HEAP32[$pCsr>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$pCsr>>2]|0;
  dest=$7; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  HEAP32[$0>>2] = 0;
  $8 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($8|0);
 } else {
  HEAP32[$0>>2] = 7;
  $8 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 return (0)|0;
}
function _fts3CloseMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 (_sqlite3_finalize($4)|0);
 $5 = HEAP32[$pCsr>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3Fts3ExprFree($7);
 $8 = HEAP32[$pCsr>>2]|0;
 _sqlite3Fts3FreeDeferredTokens($8);
 $9 = HEAP32[$pCsr>>2]|0;
 $10 = ((($9)) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3_free($11);
 $12 = HEAP32[$pCsr>>2]|0;
 $13 = ((($12)) + 92|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3Fts3MIBufferFree($14);
 $15 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($15);
 STACKTOP = sp;return 0;
}
function _fts3FilterMethod($pCursor,$idxNum,$idxStr,$nVal,$apVal) {
 $pCursor = $pCursor|0;
 $idxNum = $idxNum|0;
 $idxStr = $idxStr|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eSearch = 0, $iCol = 0, $iIdx = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pCons = 0, $pCsr = 0;
 var $pDocidGe = 0, $pDocidLe = 0, $pLangid = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zQuery = 0, $zSql = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $rc = sp + 84|0;
 $zSql = sp + 80|0;
 $eSearch = sp + 76|0;
 $p = sp + 72|0;
 $pCsr = sp + 68|0;
 $pCons = sp + 64|0;
 $pLangid = sp + 60|0;
 $pDocidGe = sp + 56|0;
 $pDocidLe = sp + 52|0;
 $iIdx = sp + 48|0;
 $iCol = sp + 44|0;
 $zQuery = sp + 40|0;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $idxNum;
 HEAP32[$3>>2] = $idxStr;
 HEAP32[$4>>2] = $nVal;
 HEAP32[$5>>2] = $apVal;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $8;
 HEAP32[$pCons>>2] = 0;
 HEAP32[$pLangid>>2] = 0;
 HEAP32[$pDocidGe>>2] = 0;
 HEAP32[$pDocidLe>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = $9 & 65535;
 HEAP32[$eSearch>>2] = $10;
 HEAP32[$iIdx>>2] = 0;
 $11 = HEAP32[$eSearch>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$iIdx>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$iIdx>>2] = $14;
  $15 = HEAP32[$5>>2]|0;
  $16 = (($15) + ($13<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pCons>>2] = $17;
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = $18 & 65536;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$iIdx>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$iIdx>>2] = $22;
  $23 = HEAP32[$5>>2]|0;
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pLangid>>2] = $25;
 }
 $26 = HEAP32[$2>>2]|0;
 $27 = $26 & 131072;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = HEAP32[$iIdx>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$iIdx>>2] = $30;
  $31 = HEAP32[$5>>2]|0;
  $32 = (($31) + ($29<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$pDocidGe>>2] = $33;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = $34 & 262144;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$iIdx>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$iIdx>>2] = $38;
  $39 = HEAP32[$5>>2]|0;
  $40 = (($39) + ($37<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$pDocidLe>>2] = $41;
 }
 $42 = HEAP32[$pCsr>>2]|0;
 $43 = ((($42)) + 8|0);
 $44 = HEAP32[$43>>2]|0;
 (_sqlite3_finalize($44)|0);
 $45 = HEAP32[$pCsr>>2]|0;
 $46 = ((($45)) + 44|0);
 $47 = HEAP32[$46>>2]|0;
 _sqlite3_free($47);
 $48 = HEAP32[$pCsr>>2]|0;
 $49 = ((($48)) + 92|0);
 $50 = HEAP32[$49>>2]|0;
 _sqlite3Fts3MIBufferFree($50);
 $51 = HEAP32[$pCsr>>2]|0;
 $52 = ((($51)) + 12|0);
 $53 = HEAP32[$52>>2]|0;
 _sqlite3Fts3ExprFree($53);
 $54 = HEAP32[$1>>2]|0;
 $55 = ((($54)) + 4|0);
 dest=$55; stop=dest+92|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $56 = HEAP32[$pDocidGe>>2]|0;
 $57 = (_fts3DocidRange($56,0,-2147483648)|0);
 $58 = tempRet0;
 $59 = HEAP32[$pCsr>>2]|0;
 $60 = ((($59)) + 72|0);
 $61 = $60;
 $62 = $61;
 HEAP32[$62>>2] = $57;
 $63 = (($61) + 4)|0;
 $64 = $63;
 HEAP32[$64>>2] = $58;
 $65 = HEAP32[$pDocidLe>>2]|0;
 $66 = (_fts3DocidRange($65,-1,2147483647)|0);
 $67 = tempRet0;
 $68 = HEAP32[$pCsr>>2]|0;
 $69 = ((($68)) + 80|0);
 $70 = $69;
 $71 = $70;
 HEAP32[$71>>2] = $66;
 $72 = (($70) + 4)|0;
 $73 = $72;
 HEAP32[$73>>2] = $67;
 $74 = HEAP32[$3>>2]|0;
 $75 = ($74|0)!=(0|0);
 if ($75) {
  $76 = HEAP32[$3>>2]|0;
  $77 = HEAP8[$76>>0]|0;
  $78 = $77 << 24 >> 24;
  $79 = ($78|0)==(68);
  $80 = $79&1;
  $81 = $80&255;
  $82 = HEAP32[$pCsr>>2]|0;
  $83 = ((($82)) + 52|0);
  HEAP8[$83>>0] = $81;
 } else {
  $84 = HEAP32[$p>>2]|0;
  $85 = ((($84)) + 231|0);
  $86 = HEAP8[$85>>0]|0;
  $87 = HEAP32[$pCsr>>2]|0;
  $88 = ((($87)) + 52|0);
  HEAP8[$88>>0] = $86;
 }
 $89 = HEAP32[$eSearch>>2]|0;
 $90 = $89&65535;
 $91 = HEAP32[$pCsr>>2]|0;
 $92 = ((($91)) + 4|0);
 HEAP16[$92>>1] = $90;
 $93 = HEAP32[$eSearch>>2]|0;
 $94 = ($93|0)!=(1);
 $95 = HEAP32[$eSearch>>2]|0;
 $96 = ($95|0)!=(0);
 $or$cond = $94 & $96;
 do {
  if ($or$cond) {
   $97 = HEAP32[$eSearch>>2]|0;
   $98 = (($97) - 2)|0;
   HEAP32[$iCol>>2] = $98;
   $99 = HEAP32[$pCons>>2]|0;
   $100 = (_sqlite3_value_text($99)|0);
   HEAP32[$zQuery>>2] = $100;
   $101 = HEAP32[$zQuery>>2]|0;
   $102 = ($101|0)==(0|0);
   if ($102) {
    $103 = HEAP32[$pCons>>2]|0;
    $104 = (_sqlite3_value_type($103)|0);
    $105 = ($104|0)!=(5);
    if ($105) {
     HEAP32[$0>>2] = 7;
     $231 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($231|0);
    }
   }
   $106 = HEAP32[$pCsr>>2]|0;
   $107 = ((($106)) + 16|0);
   HEAP32[$107>>2] = 0;
   $108 = HEAP32[$pLangid>>2]|0;
   $109 = ($108|0)!=(0|0);
   if ($109) {
    $110 = HEAP32[$pLangid>>2]|0;
    $111 = (_sqlite3_value_int($110)|0);
    $112 = HEAP32[$pCsr>>2]|0;
    $113 = ((($112)) + 16|0);
    HEAP32[$113>>2] = $111;
   }
   $114 = HEAP32[$p>>2]|0;
   $115 = ((($114)) + 36|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = HEAP32[$pCsr>>2]|0;
   $118 = ((($117)) + 16|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = HEAP32[$p>>2]|0;
   $121 = ((($120)) + 28|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = HEAP32[$p>>2]|0;
   $124 = ((($123)) + 228|0);
   $125 = HEAP8[$124>>0]|0;
   $126 = $125&255;
   $127 = HEAP32[$p>>2]|0;
   $128 = ((($127)) + 24|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = HEAP32[$iCol>>2]|0;
   $131 = HEAP32[$zQuery>>2]|0;
   $132 = HEAP32[$pCsr>>2]|0;
   $133 = ((($132)) + 12|0);
   $134 = HEAP32[$p>>2]|0;
   $135 = ((($134)) + 8|0);
   $136 = (_sqlite3Fts3ExprParse($116,$119,$122,$126,$129,$130,$131,-1,$133,$135)|0);
   HEAP32[$rc>>2] = $136;
   $137 = HEAP32[$rc>>2]|0;
   $138 = ($137|0)!=(0);
   if ($138) {
    $139 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $139;
    $231 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($231|0);
   }
   $140 = HEAP32[$pCsr>>2]|0;
   $141 = (_fts3EvalStart($140)|0);
   HEAP32[$rc>>2] = $141;
   $142 = HEAP32[$p>>2]|0;
   _sqlite3Fts3SegmentsClose($142);
   $143 = HEAP32[$rc>>2]|0;
   $144 = ($143|0)!=(0);
   if (!($144)) {
    $146 = HEAP32[$pCsr>>2]|0;
    $147 = ((($146)) + 44|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = HEAP32[$pCsr>>2]|0;
    $150 = ((($149)) + 40|0);
    HEAP32[$150>>2] = $148;
    $151 = HEAP32[$pCsr>>2]|0;
    $152 = ((($151)) + 32|0);
    $153 = $152;
    $154 = $153;
    HEAP32[$154>>2] = 0;
    $155 = (($153) + 4)|0;
    $156 = $155;
    HEAP32[$156>>2] = 0;
    break;
   }
   $145 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $145;
   $231 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($231|0);
  }
 } while(0);
 $157 = HEAP32[$eSearch>>2]|0;
 $158 = ($157|0)==(0);
 do {
  if ($158) {
   $159 = HEAP32[$pDocidGe>>2]|0;
   $160 = ($159|0)!=(0|0);
   $161 = HEAP32[$pDocidLe>>2]|0;
   $162 = ($161|0)!=(0|0);
   $or$cond3 = $160 | $162;
   $163 = HEAP32[$p>>2]|0;
   $164 = ((($163)) + 216|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = HEAP32[$pCsr>>2]|0;
   if ($or$cond3) {
    $167 = ((($166)) + 72|0);
    $168 = $167;
    $169 = $168;
    $170 = HEAP32[$169>>2]|0;
    $171 = (($168) + 4)|0;
    $172 = $171;
    $173 = HEAP32[$172>>2]|0;
    $174 = HEAP32[$pCsr>>2]|0;
    $175 = ((($174)) + 80|0);
    $176 = $175;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (($176) + 4)|0;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = HEAP32[$pCsr>>2]|0;
    $183 = ((($182)) + 52|0);
    $184 = HEAP8[$183>>0]|0;
    $185 = $184&255;
    $186 = ($185|0)!=(0);
    $187 = $186 ? 44060 : 44065;
    HEAP32[$vararg_buffer>>2] = $165;
    $vararg_ptr4 = ((($vararg_buffer)) + 8|0);
    $188 = $vararg_ptr4;
    $189 = $188;
    HEAP32[$189>>2] = $170;
    $190 = (($188) + 4)|0;
    $191 = $190;
    HEAP32[$191>>2] = $173;
    $vararg_ptr5 = ((($vararg_buffer)) + 16|0);
    $192 = $vararg_ptr5;
    $193 = $192;
    HEAP32[$193>>2] = $178;
    $194 = (($192) + 4)|0;
    $195 = $194;
    HEAP32[$195>>2] = $181;
    $vararg_ptr6 = ((($vararg_buffer)) + 24|0);
    HEAP32[$vararg_ptr6>>2] = $187;
    $196 = (_sqlite3_mprintf(44251,$vararg_buffer)|0);
    HEAP32[$zSql>>2] = $196;
   } else {
    $197 = ((($166)) + 52|0);
    $198 = HEAP8[$197>>0]|0;
    $199 = $198&255;
    $200 = ($199|0)!=(0);
    $201 = $200 ? 44060 : 44065;
    HEAP32[$vararg_buffer7>>2] = $165;
    $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
    HEAP32[$vararg_ptr10>>2] = $201;
    $202 = (_sqlite3_mprintf(44313,$vararg_buffer7)|0);
    HEAP32[$zSql>>2] = $202;
   }
   $203 = HEAP32[$zSql>>2]|0;
   $204 = ($203|0)!=(0|0);
   if ($204) {
    $205 = HEAP32[$p>>2]|0;
    $206 = ((($205)) + 12|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = HEAP32[$zSql>>2]|0;
    $209 = HEAP32[$pCsr>>2]|0;
    $210 = ((($209)) + 8|0);
    $211 = (_sqlite3_prepare_v2($207,$208,-1,$210,0)|0);
    HEAP32[$rc>>2] = $211;
    $212 = HEAP32[$zSql>>2]|0;
    _sqlite3_free($212);
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    break;
   }
  } else {
   $213 = HEAP32[$eSearch>>2]|0;
   $214 = ($213|0)==(1);
   if ($214) {
    $215 = HEAP32[$pCsr>>2]|0;
    $216 = HEAP32[$pCsr>>2]|0;
    $217 = ((($216)) + 8|0);
    $218 = (_fts3CursorSeekStmt($215,$217)|0);
    HEAP32[$rc>>2] = $218;
    $219 = HEAP32[$rc>>2]|0;
    $220 = ($219|0)==(0);
    if ($220) {
     $221 = HEAP32[$pCsr>>2]|0;
     $222 = ((($221)) + 8|0);
     $223 = HEAP32[$222>>2]|0;
     $224 = HEAP32[$pCons>>2]|0;
     $225 = (_sqlite3_bind_value($223,1,$224)|0);
     HEAP32[$rc>>2] = $225;
    }
   }
  }
 } while(0);
 $226 = HEAP32[$rc>>2]|0;
 $227 = ($226|0)!=(0);
 if ($227) {
  $228 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $228;
  $231 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($231|0);
 } else {
  $229 = HEAP32[$1>>2]|0;
  $230 = (_fts3NextMethod($229)|0);
  HEAP32[$0>>2] = $230;
  $231 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($231|0);
 }
 return (0)|0;
}
function _fts3NextMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)==(1);
 if (!($6)) {
  $7 = HEAP32[$pCsr>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($10|0)==(0);
  if (!($11)) {
   $33 = HEAP32[$0>>2]|0;
   $34 = (_fts3EvalNext($33)|0);
   HEAP32[$rc>>2] = $34;
   $35 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($35|0);
  }
 }
 $12 = HEAP32[$pCsr>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_step($14)|0);
 $16 = (100)!=($15|0);
 $17 = HEAP32[$pCsr>>2]|0;
 if ($16) {
  $18 = ((($17)) + 6|0);
  HEAP8[$18>>0] = 1;
  $19 = HEAP32[$pCsr>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3_reset($21)|0);
  HEAP32[$rc>>2] = $22;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  $23 = ((($17)) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3_column_int64($24,0)|0);
  $26 = tempRet0;
  $27 = HEAP32[$pCsr>>2]|0;
  $28 = ((($27)) + 32|0);
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $25;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $26;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _fts3EofMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 6|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _fts3ColumnMethod($pCursor,$pCtx,$iCol) {
 $pCursor = $pCursor|0;
 $pCtx = $pCtx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $iLangid = 0, $p = 0, $pCsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pCsr = sp + 8|0;
 $p = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pCtx;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 $11 = ($6|0)==($10|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$pCsr>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = $14;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  _sqlite3_result_int64($12,$17,$20);
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 24|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($21|0)==($24|0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  _sqlite3_result_blob($26,$pCsr,4,(-1));
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$p>>2]|0;
 $29 = ((($28)) + 24|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 2)|0;
 $32 = ($27|0)==($31|0);
 if ($32) {
  $33 = HEAP32[$pCsr>>2]|0;
  $34 = ((($33)) + 12|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0|0);
  if ($36) {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$pCsr>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)<(0);
   $42 = $41 << 31 >> 31;
   _sqlite3_result_int64($37,$40,$42);
   $81 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 $43 = HEAP32[$pCsr>>2]|0;
 $44 = (_fts3CursorSeek(0,$43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(0);
 if (!($46)) {
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $47 = HEAP32[$2>>2]|0;
 $48 = HEAP32[$p>>2]|0;
 $49 = ((($48)) + 24|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + 2)|0;
 $52 = ($47|0)==($51|0);
 if ($52) {
  HEAP32[$iLangid>>2] = 0;
  $53 = HEAP32[$p>>2]|0;
  $54 = ((($53)) + 44|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0|0);
  if ($56) {
   $57 = HEAP32[$pCsr>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$p>>2]|0;
   $61 = ((($60)) + 24|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + 1)|0;
   $64 = (_sqlite3_column_int($59,$63)|0);
   HEAP32[$iLangid>>2] = $64;
  }
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$iLangid>>2]|0;
  _sqlite3_result_int($65,$66);
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 } else {
  $67 = HEAP32[$pCsr>>2]|0;
  $68 = ((($67)) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (_sqlite3_data_count($69)|0);
  $71 = HEAP32[$2>>2]|0;
  $72 = (($71) + 1)|0;
  $73 = ($70|0)>($72|0);
  if (!($73)) {
   $81 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
  $74 = HEAP32[$1>>2]|0;
  $75 = HEAP32[$pCsr>>2]|0;
  $76 = ((($75)) + 8|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  $79 = (($78) + 1)|0;
  $80 = (_sqlite3_column_value($77,$79)|0);
  _sqlite3_result_value($74,$80);
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 return (0)|0;
}
function _fts3RowidMethod($pCursor,$pRowid) {
 $pCursor = $pCursor|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pRowid;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$pCsr>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _fts3UpdateMethod($pVtab,$nArg,$apVal,$pRowid) {
 $pVtab = $pVtab|0;
 $nArg = $nArg|0;
 $apVal = $apVal|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $nArg;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$3>>2] = $pRowid;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3Fts3UpdateMethod($4,$5,$6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _fts3BeginMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 52|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$p>>2]|0;
 $5 = (_fts3SetHasStat($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _fts3SyncMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $A = 0;
 var $mxLevel = 0, $nMinMerge = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $nMinMerge = sp + 16|0;
 $p = sp + 12|0;
 $rc = sp + 8|0;
 $mxLevel = sp + 4|0;
 $A = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$nMinMerge>>2] = 64;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (_sqlite3Fts3PendingTermsFlush($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 52|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8>>>0)>(4);
  if ($9) {
   $10 = HEAP32[$p>>2]|0;
   $11 = ((($10)) + 48|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)!=(0);
   if ($13) {
    $14 = HEAP32[$p>>2]|0;
    $15 = ((($14)) + 48|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($16|0)!=(255);
    if ($17) {
     HEAP32[$mxLevel>>2] = 0;
     $18 = HEAP32[$p>>2]|0;
     $19 = (_sqlite3Fts3MaxLevel($18,$mxLevel)|0);
     HEAP32[$rc>>2] = $19;
     $20 = HEAP32[$p>>2]|0;
     $21 = ((($20)) + 52|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = HEAP32[$mxLevel>>2]|0;
     $24 = Math_imul($22, $23)|0;
     HEAP32[$A>>2] = $24;
     $25 = HEAP32[$A>>2]|0;
     $26 = (($25|0) / 2)&-1;
     $27 = HEAP32[$A>>2]|0;
     $28 = (($27) + ($26))|0;
     HEAP32[$A>>2] = $28;
     $29 = HEAP32[$A>>2]|0;
     $30 = ($29|0)>(64);
     if ($30) {
      $31 = HEAP32[$p>>2]|0;
      $32 = HEAP32[$A>>2]|0;
      $33 = HEAP32[$p>>2]|0;
      $34 = ((($33)) + 48|0);
      $35 = HEAP32[$34>>2]|0;
      $36 = (_sqlite3Fts3Incrmerge($31,$32,$35)|0);
      HEAP32[$rc>>2] = $36;
     }
    }
   }
  }
 }
 $37 = HEAP32[$p>>2]|0;
 _sqlite3Fts3SegmentsClose($37);
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3CommitMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVtab;
 STACKTOP = sp;return 0;
}
function _fts3RollbackMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $2 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 _sqlite3Fts3PendingTermsClear($2);
 STACKTOP = sp;return 0;
}
function _fts3FindFunctionMethod($pVtab,$nArg,$zName,$pxFunc$coerce,$ppArg) {
 $pVtab = $pVtab|0;
 $nArg = $nArg|0;
 $zName = $zName|0;
 $pxFunc$coerce = $pxFunc$coerce|0;
 $ppArg = $ppArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aOverload = 0, $i = 0, $pxFunc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $6 = sp + 40|0;
 $aOverload = sp + 8|0;
 $i = sp;
 HEAP32[$1>>2] = $pxFunc$coerce;
 $pxFunc = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = $pVtab;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $zName;
 HEAP32[$5>>2] = $pxFunc;
 HEAP32[$6>>2] = $ppArg;
 ;HEAP32[$aOverload>>2]=HEAP32[9236>>2]|0;HEAP32[$aOverload+4>>2]=HEAP32[9236+4>>2]|0;HEAP32[$aOverload+8>>2]=HEAP32[9236+8>>2]|0;HEAP32[$aOverload+12>>2]=HEAP32[9236+12>>2]|0;HEAP32[$aOverload+16>>2]=HEAP32[9236+16>>2]|0;HEAP32[$aOverload+20>>2]=HEAP32[9236+20>>2]|0;HEAP32[$aOverload+24>>2]=HEAP32[9236+24>>2]|0;HEAP32[$aOverload+28>>2]=HEAP32[9236+28>>2]|0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = ($7|0)<(4);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$4>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  $11 = (($aOverload) + ($10<<3)|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_strcmp($9,$12)|0);
  $14 = ($13|0)==(0);
  $15 = HEAP32[$i>>2]|0;
  if ($14) {
   label = 4;
   break;
  }
  $20 = (($15) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 if ((label|0) == 4) {
  $16 = (($aOverload) + ($15<<3)|0);
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$5>>2]|0;
  HEAP32[$19>>2] = $18;
  HEAP32[$0>>2] = 1;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _fts3RenameMethod($pVtab,$zName) {
 $pVtab = $pVtab|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer18 = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0;
 var $vararg_ptr22 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $p = sp + 84|0;
 $db = sp + 80|0;
 $rc = sp + 76|0;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = (_fts3SetHasStat($6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = HEAP32[$p>>2]|0;
  $11 = (_sqlite3Fts3PendingTermsFlush($10)|0);
  HEAP32[$rc>>2] = $11;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 40|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[$db>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $19;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $22;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $23;
  _fts3DbExec($rc,$16,44818,$vararg_buffer);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 230|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = ($26<<24>>24)!=(0);
 if ($27) {
  $28 = HEAP32[$db>>2]|0;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $31;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $34;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = $35;
  _fts3DbExec($rc,$28,44871,$vararg_buffer3);
 }
 $36 = HEAP32[$p>>2]|0;
 $37 = ((($36)) + 229|0);
 $38 = HEAP8[$37>>0]|0;
 $39 = ($38<<24>>24)!=(0);
 if ($39) {
  $40 = HEAP32[$db>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$p>>2]|0;
  $45 = ((($44)) + 20|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer8>>2] = $43;
  $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
  HEAP32[$vararg_ptr11>>2] = $46;
  $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
  HEAP32[$vararg_ptr12>>2] = $47;
  _fts3DbExec($rc,$40,44924,$vararg_buffer8);
 }
 $48 = HEAP32[$db>>2]|0;
 $49 = HEAP32[$p>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$p>>2]|0;
 $53 = ((($52)) + 20|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer13>>2] = $51;
 $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
 HEAP32[$vararg_ptr16>>2] = $54;
 $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
 HEAP32[$vararg_ptr17>>2] = $55;
 _fts3DbExec($rc,$48,44971,$vararg_buffer13);
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$p>>2]|0;
 $58 = ((($57)) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = HEAP32[$p>>2]|0;
 $61 = ((($60)) + 20|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer18>>2] = $59;
 $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
 HEAP32[$vararg_ptr21>>2] = $62;
 $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
 HEAP32[$vararg_ptr22>>2] = $63;
 _fts3DbExec($rc,$56,45025,$vararg_buffer18);
 $64 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _fts3SavepointMethod($pVtab,$iSavepoint) {
 $pVtab = $pVtab|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 232|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 if (!($6)) {
  $9 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_fts3SyncMethod($7)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _fts3ReleaseMethod($pVtab,$iSavepoint) {
 $pVtab = $pVtab|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $iSavepoint;
 STACKTOP = sp;return 0;
}
function _fts3RollbackToMethod($pVtab,$iSavepoint) {
 $pVtab = $pVtab|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $iSavepoint;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 _sqlite3Fts3PendingTermsClear($3);
 STACKTOP = sp;return 0;
}
function _fts3InitVtab($isCreate,$db,$pAux,$argc,$argv,$ppVTab,$pzErr) {
 $isCreate = $isCreate|0;
 $db = $db|0;
 $pAux = $pAux|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVTab = $ppVTab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aCol = 0, $aFts4Opt = 0, $aIndex = 0, $azNotindexed = 0, $bDescIdx = 0, $bNoDocsize = 0, $i = 0, $iCol = 0, $iOpt = 0, $isFts4 = 0, $j = 0, $k = 0, $n = 0, $n2 = 0, $nByte = 0, $nCol = 0, $nDb = 0, $nIndex = 0, $nKey = 0, $nName = 0;
 var $nNotindexed = 0, $nString = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $p = 0, $pHash = 0, $pOp = 0, $pTokenizer = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer17 = 0, $vararg_buffer20 = 0, $vararg_buffer8 = 0, $z = 0, $z1 = 0, $zCompress = 0;
 var $zContent = 0, $zCsr = 0, $zLanguageid = 0, $zMiss = 0, $zNot = 0, $zPrefix = 0, $zUncompress = 0, $zVal = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $vararg_buffer20 = sp + 40|0;
 $vararg_buffer17 = sp + 32|0;
 $vararg_buffer14 = sp + 24|0;
 $vararg_buffer11 = sp + 16|0;
 $vararg_buffer8 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 280|0;
 $1 = sp + 276|0;
 $2 = sp + 272|0;
 $3 = sp + 268|0;
 $4 = sp + 264|0;
 $5 = sp + 260|0;
 $6 = sp + 256|0;
 $pHash = sp + 252|0;
 $p = sp + 248|0;
 $rc = sp + 244|0;
 $i = sp + 240|0;
 $nByte = sp + 236|0;
 $iCol = sp + 232|0;
 $nString = sp + 228|0;
 $nCol = sp + 224|0;
 $zCsr = sp + 220|0;
 $nDb = sp + 216|0;
 $nName = sp + 212|0;
 $isFts4 = sp + 208|0;
 $aCol = sp + 204|0;
 $pTokenizer = sp + 200|0;
 $nIndex = sp + 196|0;
 $aIndex = sp + 192|0;
 $bNoDocsize = sp + 188|0;
 $bDescIdx = sp + 184|0;
 $zPrefix = sp + 180|0;
 $zCompress = sp + 176|0;
 $zUncompress = sp + 172|0;
 $zContent = sp + 168|0;
 $zLanguageid = sp + 164|0;
 $azNotindexed = sp + 160|0;
 $nNotindexed = sp + 156|0;
 $z = sp + 152|0;
 $nKey = sp + 148|0;
 $zVal = sp + 144|0;
 $aFts4Opt = sp + 80|0;
 $iOpt = sp + 76|0;
 $pOp = sp + 72|0;
 $j = sp + 68|0;
 $k = sp + 64|0;
 $z1 = sp + 60|0;
 $n = sp + 56|0;
 $n2 = sp + 52|0;
 $zNot = sp + 48|0;
 $zMiss = sp + 44|0;
 HEAP32[$0>>2] = $isCreate;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pAux;
 HEAP32[$3>>2] = $argc;
 HEAP32[$4>>2] = $argv;
 HEAP32[$5>>2] = $ppVTab;
 HEAP32[$6>>2] = $pzErr;
 $7 = HEAP32[$2>>2]|0;
 HEAP32[$pHash>>2] = $7;
 HEAP32[$p>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nString>>2] = 0;
 HEAP32[$nCol>>2] = 0;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 3|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = ($12|0)==(52);
 $14 = $13&1;
 HEAP32[$isFts4>>2] = $14;
 HEAP32[$pTokenizer>>2] = 0;
 HEAP32[$nIndex>>2] = 0;
 HEAP32[$aIndex>>2] = 0;
 HEAP32[$bNoDocsize>>2] = 0;
 HEAP32[$bDescIdx>>2] = 0;
 HEAP32[$zPrefix>>2] = 0;
 HEAP32[$zCompress>>2] = 0;
 HEAP32[$zUncompress>>2] = 0;
 HEAP32[$zContent>>2] = 0;
 HEAP32[$zLanguageid>>2] = 0;
 HEAP32[$azNotindexed>>2] = 0;
 HEAP32[$nNotindexed>>2] = 0;
 $15 = HEAP32[$4>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_strlen($17)|0);
 $19 = (($18) + 1)|0;
 HEAP32[$nDb>>2] = $19;
 $20 = HEAP32[$4>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_strlen($22)|0);
 $24 = (($23) + 1)|0;
 HEAP32[$nName>>2] = $24;
 $25 = HEAP32[$3>>2]|0;
 $26 = (($25) - 2)|0;
 $27 = $26<<2;
 HEAP32[$nByte>>2] = $27;
 $28 = HEAP32[$nByte>>2]|0;
 $29 = (_sqlite3_malloc($28)|0);
 HEAP32[$aCol>>2] = $29;
 $30 = HEAP32[$aCol>>2]|0;
 $31 = ($30|0)!=(0|0);
 if ($31) {
  $32 = HEAP32[$aCol>>2]|0;
  $33 = HEAP32[$nByte>>2]|0;
  _memset(($32|0),0,($33|0))|0;
  $34 = HEAP32[$nByte>>2]|0;
  $35 = (_sqlite3_malloc($34)|0);
  HEAP32[$azNotindexed>>2] = $35;
 }
 $36 = HEAP32[$azNotindexed>>2]|0;
 $37 = ($36|0)!=(0|0);
 if ($37) {
  $38 = HEAP32[$azNotindexed>>2]|0;
  $39 = HEAP32[$nByte>>2]|0;
  _memset(($38|0),0,($39|0))|0;
 }
 $40 = HEAP32[$aCol>>2]|0;
 $41 = ($40|0)!=(0|0);
 $42 = HEAP32[$azNotindexed>>2]|0;
 $43 = ($42|0)!=(0|0);
 $or$cond = $41 & $43;
 do {
  if ($or$cond) {
   HEAP32[$i>>2] = 3;
   while(1) {
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)==(0);
    if (!($45)) {
     break;
    }
    $46 = HEAP32[$i>>2]|0;
    $47 = HEAP32[$3>>2]|0;
    $48 = ($46|0)<($47|0);
    if (!($48)) {
     break;
    }
    $49 = HEAP32[$i>>2]|0;
    $50 = HEAP32[$4>>2]|0;
    $51 = (($50) + ($49<<2)|0);
    $52 = HEAP32[$51>>2]|0;
    HEAP32[$z>>2] = $52;
    $53 = HEAP32[$pTokenizer>>2]|0;
    $54 = ($53|0)!=(0|0);
    if ($54) {
     label = 15;
    } else {
     $55 = HEAP32[$z>>2]|0;
     $56 = (_strlen($55)|0);
     $57 = ($56>>>0)>(8);
     if ($57) {
      $58 = HEAP32[$z>>2]|0;
      $59 = (_sqlite3_strnicmp($58,43037,8)|0);
      $60 = (0)==($59|0);
      if ($60) {
       $61 = HEAP32[$z>>2]|0;
       $62 = ((($61)) + 8|0);
       $63 = HEAP8[$62>>0]|0;
       $64 = (_sqlite3Fts3IsIdChar($63)|0);
       $65 = (0)==($64|0);
       if ($65) {
        $66 = HEAP32[$pHash>>2]|0;
        $67 = HEAP32[$z>>2]|0;
        $68 = ((($67)) + 9|0);
        $69 = HEAP32[$6>>2]|0;
        $70 = (_sqlite3Fts3InitTokenizer($66,$68,$pTokenizer,$69)|0);
        HEAP32[$rc>>2] = $70;
       } else {
        label = 15;
       }
      } else {
       label = 15;
      }
     } else {
      label = 15;
     }
    }
    do {
     if ((label|0) == 15) {
      label = 0;
      $71 = HEAP32[$isFts4>>2]|0;
      $72 = ($71|0)!=(0);
      if ($72) {
       $73 = HEAP32[$z>>2]|0;
       $74 = (_fts3IsSpecialColumn($73,$nKey,$zVal)|0);
       $75 = ($74|0)!=(0);
       if ($75) {
        dest=$aFts4Opt; src=9140; stop=dest+64|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
        $76 = HEAP32[$zVal>>2]|0;
        $77 = ($76|0)!=(0|0);
        if (!($77)) {
         HEAP32[$rc>>2] = 7;
         break;
        }
        HEAP32[$iOpt>>2] = 0;
        while(1) {
         $78 = HEAP32[$iOpt>>2]|0;
         $79 = ($78|0)<(8);
         if (!($79)) {
          break;
         }
         $80 = HEAP32[$iOpt>>2]|0;
         $81 = (($aFts4Opt) + ($80<<3)|0);
         HEAP32[$pOp>>2] = $81;
         $82 = HEAP32[$nKey>>2]|0;
         $83 = HEAP32[$pOp>>2]|0;
         $84 = ((($83)) + 4|0);
         $85 = HEAP32[$84>>2]|0;
         $86 = ($82|0)==($85|0);
         if ($86) {
          $87 = HEAP32[$z>>2]|0;
          $88 = HEAP32[$pOp>>2]|0;
          $89 = HEAP32[$88>>2]|0;
          $90 = HEAP32[$pOp>>2]|0;
          $91 = ((($90)) + 4|0);
          $92 = HEAP32[$91>>2]|0;
          $93 = (_sqlite3_strnicmp($87,$89,$92)|0);
          $94 = ($93|0)!=(0);
          if (!($94)) {
           break;
          }
         }
         $95 = HEAP32[$iOpt>>2]|0;
         $96 = (($95) + 1)|0;
         HEAP32[$iOpt>>2] = $96;
        }
        $97 = HEAP32[$iOpt>>2]|0;
        $98 = ($97|0)==(8);
        L33: do {
         if ($98) {
          $99 = HEAP32[$6>>2]|0;
          $100 = HEAP32[$z>>2]|0;
          HEAP32[$vararg_buffer>>2] = $100;
          _sqlite3Fts3ErrMsg($99,43103,$vararg_buffer);
          HEAP32[$rc>>2] = 1;
         } else {
          $101 = HEAP32[$iOpt>>2]|0;
          switch ($101|0) {
          case 0:  {
           $102 = HEAP32[$zVal>>2]|0;
           $103 = (_strlen($102)|0);
           $104 = ($103|0)!=(4);
           if ($104) {
            label = 29;
           } else {
            $105 = HEAP32[$zVal>>2]|0;
            $106 = (_sqlite3_strnicmp($105,39985,4)|0);
            $107 = ($106|0)!=(0);
            if ($107) {
             label = 29;
            }
           }
           if ((label|0) == 29) {
            label = 0;
            $108 = HEAP32[$6>>2]|0;
            $109 = HEAP32[$zVal>>2]|0;
            HEAP32[$vararg_buffer8>>2] = $109;
            _sqlite3Fts3ErrMsg($108,43130,$vararg_buffer8);
            HEAP32[$rc>>2] = 1;
           }
           HEAP32[$bNoDocsize>>2] = 1;
           break L33;
           break;
          }
          case 1:  {
           $110 = HEAP32[$zPrefix>>2]|0;
           _sqlite3_free($110);
           $111 = HEAP32[$zVal>>2]|0;
           HEAP32[$zPrefix>>2] = $111;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 2:  {
           $112 = HEAP32[$zCompress>>2]|0;
           _sqlite3_free($112);
           $113 = HEAP32[$zVal>>2]|0;
           HEAP32[$zCompress>>2] = $113;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 3:  {
           $114 = HEAP32[$zUncompress>>2]|0;
           _sqlite3_free($114);
           $115 = HEAP32[$zVal>>2]|0;
           HEAP32[$zUncompress>>2] = $115;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 4:  {
           $116 = HEAP32[$zVal>>2]|0;
           $117 = (_strlen($116)|0);
           $118 = ($117|0)!=(3);
           if ($118) {
            label = 36;
           } else {
            $119 = HEAP32[$zVal>>2]|0;
            $120 = (_sqlite3_strnicmp($119,43157,3)|0);
            $121 = ($120|0)!=(0);
            if ($121) {
             label = 36;
            }
           }
           do {
            if ((label|0) == 36) {
             label = 0;
             $122 = HEAP32[$zVal>>2]|0;
             $123 = (_strlen($122)|0);
             $124 = ($123|0)!=(4);
             if (!($124)) {
              $125 = HEAP32[$zVal>>2]|0;
              $126 = (_sqlite3_strnicmp($125,34386,4)|0);
              $127 = ($126|0)!=(0);
              if (!($127)) {
               break;
              }
             }
             $128 = HEAP32[$6>>2]|0;
             $129 = HEAP32[$zVal>>2]|0;
             HEAP32[$vararg_buffer11>>2] = $129;
             _sqlite3Fts3ErrMsg($128,43161,$vararg_buffer11);
             HEAP32[$rc>>2] = 1;
            }
           } while(0);
           $130 = HEAP32[$zVal>>2]|0;
           $131 = HEAP8[$130>>0]|0;
           $132 = $131 << 24 >> 24;
           $133 = ($132|0)==(100);
           if ($133) {
            $139 = 1;
           } else {
            $134 = HEAP32[$zVal>>2]|0;
            $135 = HEAP8[$134>>0]|0;
            $136 = $135 << 24 >> 24;
            $137 = ($136|0)==(68);
            $139 = $137;
           }
           $138 = $139&1;
           HEAP32[$bDescIdx>>2] = $138;
           break L33;
           break;
          }
          case 5:  {
           $140 = HEAP32[$zContent>>2]|0;
           _sqlite3_free($140);
           $141 = HEAP32[$zVal>>2]|0;
           HEAP32[$zContent>>2] = $141;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 6:  {
           $142 = HEAP32[$zLanguageid>>2]|0;
           _sqlite3_free($142);
           $143 = HEAP32[$zVal>>2]|0;
           HEAP32[$zLanguageid>>2] = $143;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 7:  {
           $144 = HEAP32[$zVal>>2]|0;
           $145 = HEAP32[$nNotindexed>>2]|0;
           $146 = (($145) + 1)|0;
           HEAP32[$nNotindexed>>2] = $146;
           $147 = HEAP32[$azNotindexed>>2]|0;
           $148 = (($147) + ($145<<2)|0);
           HEAP32[$148>>2] = $144;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          default: {
           break L33;
          }
          }
         }
        } while(0);
        $149 = HEAP32[$zVal>>2]|0;
        _sqlite3_free($149);
        break;
       }
      }
      $150 = HEAP32[$z>>2]|0;
      $151 = (_strlen($150)|0);
      $152 = (($151) + 1)|0;
      $153 = HEAP32[$nString>>2]|0;
      $154 = (($153) + ($152))|0;
      HEAP32[$nString>>2] = $154;
      $155 = HEAP32[$z>>2]|0;
      $156 = HEAP32[$nCol>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$nCol>>2] = $157;
      $158 = HEAP32[$aCol>>2]|0;
      $159 = (($158) + ($156<<2)|0);
      HEAP32[$159>>2] = $155;
     }
    } while(0);
    $160 = HEAP32[$i>>2]|0;
    $161 = (($160) + 1)|0;
    HEAP32[$i>>2] = $161;
   }
   $162 = HEAP32[$rc>>2]|0;
   $163 = ($162|0)==(0);
   $164 = HEAP32[$zContent>>2]|0;
   $165 = ($164|0)!=(0|0);
   $or$cond3 = $163 & $165;
   L65: do {
    if ($or$cond3) {
     $166 = HEAP32[$zCompress>>2]|0;
     _sqlite3_free($166);
     $167 = HEAP32[$zUncompress>>2]|0;
     _sqlite3_free($167);
     HEAP32[$zCompress>>2] = 0;
     HEAP32[$zUncompress>>2] = 0;
     $168 = HEAP32[$nCol>>2]|0;
     $169 = ($168|0)==(0);
     if ($169) {
      $170 = HEAP32[$aCol>>2]|0;
      _sqlite3_free($170);
      HEAP32[$aCol>>2] = 0;
      $171 = HEAP32[$1>>2]|0;
      $172 = HEAP32[$4>>2]|0;
      $173 = ((($172)) + 4|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = HEAP32[$zContent>>2]|0;
      $176 = HEAP32[$6>>2]|0;
      $177 = (_fts3ContentColumns($171,$174,$175,$aCol,$nCol,$nString,$176)|0);
      HEAP32[$rc>>2] = $177;
      $178 = HEAP32[$rc>>2]|0;
      $179 = ($178|0)==(0);
      $180 = HEAP32[$zLanguageid>>2]|0;
      $181 = ($180|0)!=(0|0);
      $or$cond5 = $179 & $181;
      if ($or$cond5) {
       HEAP32[$j>>2] = 0;
       while(1) {
        $182 = HEAP32[$j>>2]|0;
        $183 = HEAP32[$nCol>>2]|0;
        $184 = ($182|0)<($183|0);
        if (!($184)) {
         break L65;
        }
        $185 = HEAP32[$zLanguageid>>2]|0;
        $186 = HEAP32[$j>>2]|0;
        $187 = HEAP32[$aCol>>2]|0;
        $188 = (($187) + ($186<<2)|0);
        $189 = HEAP32[$188>>2]|0;
        $190 = (_sqlite3_stricmp($185,$189)|0);
        $191 = ($190|0)==(0);
        $192 = HEAP32[$j>>2]|0;
        if ($191) {
         break;
        }
        $208 = (($192) + 1)|0;
        HEAP32[$j>>2] = $208;
       }
       HEAP32[$k>>2] = $192;
       while(1) {
        $193 = HEAP32[$k>>2]|0;
        $194 = HEAP32[$nCol>>2]|0;
        $195 = ($193|0)<($194|0);
        if (!($195)) {
         break;
        }
        $196 = HEAP32[$k>>2]|0;
        $197 = (($196) + 1)|0;
        $198 = HEAP32[$aCol>>2]|0;
        $199 = (($198) + ($197<<2)|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = HEAP32[$k>>2]|0;
        $202 = HEAP32[$aCol>>2]|0;
        $203 = (($202) + ($201<<2)|0);
        HEAP32[$203>>2] = $200;
        $204 = HEAP32[$k>>2]|0;
        $205 = (($204) + 1)|0;
        HEAP32[$k>>2] = $205;
       }
       $206 = HEAP32[$nCol>>2]|0;
       $207 = (($206) + -1)|0;
       HEAP32[$nCol>>2] = $207;
      }
     }
    }
   } while(0);
   $209 = HEAP32[$rc>>2]|0;
   $210 = ($209|0)!=(0);
   if (!($210)) {
    $211 = HEAP32[$nCol>>2]|0;
    $212 = ($211|0)==(0);
    if ($212) {
     $213 = HEAP32[$aCol>>2]|0;
     HEAP32[$213>>2] = 43073;
     HEAP32[$nString>>2] = 8;
     HEAP32[$nCol>>2] = 1;
    }
    $214 = HEAP32[$pTokenizer>>2]|0;
    $215 = ($214|0)==(0|0);
    if ($215) {
     $216 = HEAP32[$pHash>>2]|0;
     $217 = HEAP32[$6>>2]|0;
     $218 = (_sqlite3Fts3InitTokenizer($216,39911,$pTokenizer,$217)|0);
     HEAP32[$rc>>2] = $218;
     $219 = HEAP32[$rc>>2]|0;
     $220 = ($219|0)!=(0);
     if ($220) {
      break;
     }
    }
    $221 = HEAP32[$zPrefix>>2]|0;
    $222 = (_fts3PrefixParameter($221,$nIndex,$aIndex)|0);
    HEAP32[$rc>>2] = $222;
    $223 = HEAP32[$rc>>2]|0;
    $224 = ($223|0)==(1);
    if ($224) {
     $225 = HEAP32[$6>>2]|0;
     $226 = HEAP32[$zPrefix>>2]|0;
     HEAP32[$vararg_buffer14>>2] = $226;
     _sqlite3Fts3ErrMsg($225,43184,$vararg_buffer14);
    }
    $227 = HEAP32[$rc>>2]|0;
    $228 = ($227|0)!=(0);
    if (!($228)) {
     $229 = HEAP32[$nCol>>2]|0;
     $230 = $229<<2;
     $231 = (280 + ($230))|0;
     $232 = HEAP32[$nIndex>>2]|0;
     $233 = ($232*24)|0;
     $234 = (($231) + ($233))|0;
     $235 = HEAP32[$nCol>>2]|0;
     $236 = $235;
     $237 = (($234) + ($236))|0;
     $238 = HEAP32[$nName>>2]|0;
     $239 = (($237) + ($238))|0;
     $240 = HEAP32[$nDb>>2]|0;
     $241 = (($239) + ($240))|0;
     $242 = HEAP32[$nString>>2]|0;
     $243 = (($241) + ($242))|0;
     HEAP32[$nByte>>2] = $243;
     $244 = HEAP32[$nByte>>2]|0;
     $245 = (_sqlite3_malloc($244)|0);
     HEAP32[$p>>2] = $245;
     $246 = HEAP32[$p>>2]|0;
     $247 = ($246|0)==(0|0);
     if ($247) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $248 = HEAP32[$p>>2]|0;
     $249 = HEAP32[$nByte>>2]|0;
     _memset(($248|0),0,($249|0))|0;
     $250 = HEAP32[$1>>2]|0;
     $251 = HEAP32[$p>>2]|0;
     $252 = ((($251)) + 12|0);
     HEAP32[$252>>2] = $250;
     $253 = HEAP32[$nCol>>2]|0;
     $254 = HEAP32[$p>>2]|0;
     $255 = ((($254)) + 24|0);
     HEAP32[$255>>2] = $253;
     $256 = HEAP32[$p>>2]|0;
     $257 = ((($256)) + 260|0);
     HEAP32[$257>>2] = 0;
     $258 = HEAP32[$p>>2]|0;
     $259 = ((($258)) + 280|0);
     $260 = HEAP32[$p>>2]|0;
     $261 = ((($260)) + 28|0);
     HEAP32[$261>>2] = $259;
     $262 = HEAP32[$pTokenizer>>2]|0;
     $263 = HEAP32[$p>>2]|0;
     $264 = ((($263)) + 36|0);
     HEAP32[$264>>2] = $262;
     $265 = HEAP32[$p>>2]|0;
     $266 = ((($265)) + 256|0);
     HEAP32[$266>>2] = 1048576;
     $267 = HEAP32[$isFts4>>2]|0;
     $268 = ($267|0)!=(0);
     $269 = HEAP32[$bNoDocsize>>2]|0;
     $270 = ($269|0)==(0);
     $271 = $268 ? $270 : 0;
     $272 = $271&1;
     $273 = $272&255;
     $274 = HEAP32[$p>>2]|0;
     $275 = ((($274)) + 230|0);
     HEAP8[$275>>0] = $273;
     $276 = HEAP32[$isFts4>>2]|0;
     $277 = $276&255;
     $278 = HEAP32[$p>>2]|0;
     $279 = ((($278)) + 229|0);
     HEAP8[$279>>0] = $277;
     $280 = HEAP32[$isFts4>>2]|0;
     $281 = $280&255;
     $282 = HEAP32[$p>>2]|0;
     $283 = ((($282)) + 228|0);
     HEAP8[$283>>0] = $281;
     $284 = HEAP32[$bDescIdx>>2]|0;
     $285 = $284&255;
     $286 = HEAP32[$p>>2]|0;
     $287 = ((($286)) + 231|0);
     HEAP8[$287>>0] = $285;
     $288 = HEAP32[$p>>2]|0;
     $289 = ((($288)) + 48|0);
     HEAP32[$289>>2] = 255;
     $290 = HEAP32[$zContent>>2]|0;
     $291 = HEAP32[$p>>2]|0;
     $292 = ((($291)) + 40|0);
     HEAP32[$292>>2] = $290;
     $293 = HEAP32[$zLanguageid>>2]|0;
     $294 = HEAP32[$p>>2]|0;
     $295 = ((($294)) + 44|0);
     HEAP32[$295>>2] = $293;
     HEAP32[$zContent>>2] = 0;
     HEAP32[$zLanguageid>>2] = 0;
     $296 = HEAP32[$nCol>>2]|0;
     $297 = HEAP32[$p>>2]|0;
     $298 = ((($297)) + 28|0);
     $299 = HEAP32[$298>>2]|0;
     $300 = (($299) + ($296<<2)|0);
     $301 = HEAP32[$p>>2]|0;
     $302 = ((($301)) + 252|0);
     HEAP32[$302>>2] = $300;
     $303 = HEAP32[$p>>2]|0;
     $304 = ((($303)) + 252|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = HEAP32[$aIndex>>2]|0;
     $307 = HEAP32[$nIndex>>2]|0;
     $308 = ($307*24)|0;
     _memcpy(($305|0),($306|0),($308|0))|0;
     $309 = HEAP32[$nIndex>>2]|0;
     $310 = HEAP32[$p>>2]|0;
     $311 = ((($310)) + 248|0);
     HEAP32[$311>>2] = $309;
     HEAP32[$i>>2] = 0;
     while(1) {
      $312 = HEAP32[$i>>2]|0;
      $313 = HEAP32[$nIndex>>2]|0;
      $314 = ($312|0)<($313|0);
      if (!($314)) {
       break;
      }
      $315 = HEAP32[$i>>2]|0;
      $316 = HEAP32[$p>>2]|0;
      $317 = ((($316)) + 252|0);
      $318 = HEAP32[$317>>2]|0;
      $319 = (($318) + (($315*24)|0)|0);
      $320 = ((($319)) + 4|0);
      _sqlite3Fts3HashInit($320,1,1);
      $321 = HEAP32[$i>>2]|0;
      $322 = (($321) + 1)|0;
      HEAP32[$i>>2] = $322;
     }
     $323 = HEAP32[$nIndex>>2]|0;
     $324 = HEAP32[$p>>2]|0;
     $325 = ((($324)) + 252|0);
     $326 = HEAP32[$325>>2]|0;
     $327 = (($326) + (($323*24)|0)|0);
     $328 = HEAP32[$p>>2]|0;
     $329 = ((($328)) + 32|0);
     HEAP32[$329>>2] = $327;
     $330 = HEAP32[$nCol>>2]|0;
     $331 = HEAP32[$p>>2]|0;
     $332 = ((($331)) + 32|0);
     $333 = HEAP32[$332>>2]|0;
     $334 = (($333) + ($330)|0);
     HEAP32[$zCsr>>2] = $334;
     $335 = HEAP32[$zCsr>>2]|0;
     $336 = HEAP32[$p>>2]|0;
     $337 = ((($336)) + 20|0);
     HEAP32[$337>>2] = $335;
     $338 = HEAP32[$zCsr>>2]|0;
     $339 = HEAP32[$4>>2]|0;
     $340 = ((($339)) + 8|0);
     $341 = HEAP32[$340>>2]|0;
     $342 = HEAP32[$nName>>2]|0;
     _memcpy(($338|0),($341|0),($342|0))|0;
     $343 = HEAP32[$nName>>2]|0;
     $344 = HEAP32[$zCsr>>2]|0;
     $345 = (($344) + ($343)|0);
     HEAP32[$zCsr>>2] = $345;
     $346 = HEAP32[$zCsr>>2]|0;
     $347 = HEAP32[$p>>2]|0;
     $348 = ((($347)) + 16|0);
     HEAP32[$348>>2] = $346;
     $349 = HEAP32[$zCsr>>2]|0;
     $350 = HEAP32[$4>>2]|0;
     $351 = ((($350)) + 4|0);
     $352 = HEAP32[$351>>2]|0;
     $353 = HEAP32[$nDb>>2]|0;
     _memcpy(($349|0),($352|0),($353|0))|0;
     $354 = HEAP32[$nDb>>2]|0;
     $355 = HEAP32[$zCsr>>2]|0;
     $356 = (($355) + ($354)|0);
     HEAP32[$zCsr>>2] = $356;
     HEAP32[$iCol>>2] = 0;
     while(1) {
      $357 = HEAP32[$iCol>>2]|0;
      $358 = HEAP32[$nCol>>2]|0;
      $359 = ($357|0)<($358|0);
      if (!($359)) {
       break;
      }
      HEAP32[$n>>2] = 0;
      $360 = HEAP32[$iCol>>2]|0;
      $361 = HEAP32[$aCol>>2]|0;
      $362 = (($361) + ($360<<2)|0);
      $363 = HEAP32[$362>>2]|0;
      $364 = (_sqlite3Fts3NextToken($363,$n)|0);
      HEAP32[$z1>>2] = $364;
      $365 = HEAP32[$zCsr>>2]|0;
      $366 = HEAP32[$z1>>2]|0;
      $367 = HEAP32[$n>>2]|0;
      _memcpy(($365|0),($366|0),($367|0))|0;
      $368 = HEAP32[$n>>2]|0;
      $369 = HEAP32[$zCsr>>2]|0;
      $370 = (($369) + ($368)|0);
      HEAP8[$370>>0] = 0;
      $371 = HEAP32[$zCsr>>2]|0;
      _sqlite3Fts3Dequote($371);
      $372 = HEAP32[$zCsr>>2]|0;
      $373 = HEAP32[$iCol>>2]|0;
      $374 = HEAP32[$p>>2]|0;
      $375 = ((($374)) + 28|0);
      $376 = HEAP32[$375>>2]|0;
      $377 = (($376) + ($373<<2)|0);
      HEAP32[$377>>2] = $372;
      $378 = HEAP32[$n>>2]|0;
      $379 = (($378) + 1)|0;
      $380 = HEAP32[$zCsr>>2]|0;
      $381 = (($380) + ($379)|0);
      HEAP32[$zCsr>>2] = $381;
      $382 = HEAP32[$iCol>>2]|0;
      $383 = (($382) + 1)|0;
      HEAP32[$iCol>>2] = $383;
     }
     HEAP32[$iCol>>2] = 0;
     while(1) {
      $384 = HEAP32[$iCol>>2]|0;
      $385 = HEAP32[$nCol>>2]|0;
      $386 = ($384|0)<($385|0);
      if (!($386)) {
       break;
      }
      $387 = HEAP32[$iCol>>2]|0;
      $388 = HEAP32[$p>>2]|0;
      $389 = ((($388)) + 28|0);
      $390 = HEAP32[$389>>2]|0;
      $391 = (($390) + ($387<<2)|0);
      $392 = HEAP32[$391>>2]|0;
      $393 = (_strlen($392)|0);
      HEAP32[$n2>>2] = $393;
      HEAP32[$i>>2] = 0;
      while(1) {
       $394 = HEAP32[$i>>2]|0;
       $395 = HEAP32[$nNotindexed>>2]|0;
       $396 = ($394|0)<($395|0);
       if (!($396)) {
        break;
       }
       $397 = HEAP32[$i>>2]|0;
       $398 = HEAP32[$azNotindexed>>2]|0;
       $399 = (($398) + ($397<<2)|0);
       $400 = HEAP32[$399>>2]|0;
       HEAP32[$zNot>>2] = $400;
       $401 = HEAP32[$zNot>>2]|0;
       $402 = ($401|0)!=(0|0);
       if ($402) {
        $403 = HEAP32[$n2>>2]|0;
        $404 = HEAP32[$zNot>>2]|0;
        $405 = (_strlen($404)|0);
        $406 = ($403|0)==($405|0);
        if ($406) {
         $407 = HEAP32[$iCol>>2]|0;
         $408 = HEAP32[$p>>2]|0;
         $409 = ((($408)) + 28|0);
         $410 = HEAP32[$409>>2]|0;
         $411 = (($410) + ($407<<2)|0);
         $412 = HEAP32[$411>>2]|0;
         $413 = HEAP32[$zNot>>2]|0;
         $414 = HEAP32[$n2>>2]|0;
         $415 = (_sqlite3_strnicmp($412,$413,$414)|0);
         $416 = (0)==($415|0);
         if ($416) {
          $417 = HEAP32[$iCol>>2]|0;
          $418 = HEAP32[$p>>2]|0;
          $419 = ((($418)) + 32|0);
          $420 = HEAP32[$419>>2]|0;
          $421 = (($420) + ($417)|0);
          HEAP8[$421>>0] = 1;
          $422 = HEAP32[$zNot>>2]|0;
          _sqlite3_free($422);
          $423 = HEAP32[$i>>2]|0;
          $424 = HEAP32[$azNotindexed>>2]|0;
          $425 = (($424) + ($423<<2)|0);
          HEAP32[$425>>2] = 0;
         }
        }
       }
       $426 = HEAP32[$i>>2]|0;
       $427 = (($426) + 1)|0;
       HEAP32[$i>>2] = $427;
      }
      $428 = HEAP32[$iCol>>2]|0;
      $429 = (($428) + 1)|0;
      HEAP32[$iCol>>2] = $429;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $430 = HEAP32[$i>>2]|0;
      $431 = HEAP32[$nNotindexed>>2]|0;
      $432 = ($430|0)<($431|0);
      if (!($432)) {
       break;
      }
      $433 = HEAP32[$i>>2]|0;
      $434 = HEAP32[$azNotindexed>>2]|0;
      $435 = (($434) + ($433<<2)|0);
      $436 = HEAP32[$435>>2]|0;
      $437 = ($436|0)!=(0|0);
      if ($437) {
       $438 = HEAP32[$6>>2]|0;
       $439 = HEAP32[$i>>2]|0;
       $440 = HEAP32[$azNotindexed>>2]|0;
       $441 = (($440) + ($439<<2)|0);
       $442 = HEAP32[$441>>2]|0;
       HEAP32[$vararg_buffer17>>2] = $442;
       _sqlite3Fts3ErrMsg($438,33579,$vararg_buffer17);
       HEAP32[$rc>>2] = 1;
      }
      $443 = HEAP32[$i>>2]|0;
      $444 = (($443) + 1)|0;
      HEAP32[$i>>2] = $444;
     }
     $445 = HEAP32[$rc>>2]|0;
     $446 = ($445|0)==(0);
     if ($446) {
      $447 = HEAP32[$zCompress>>2]|0;
      $448 = ($447|0)==(0|0);
      $449 = $448&1;
      $450 = HEAP32[$zUncompress>>2]|0;
      $451 = ($450|0)==(0|0);
      $452 = $451&1;
      $453 = ($449|0)!=($452|0);
      if ($453) {
       $454 = HEAP32[$zCompress>>2]|0;
       $455 = ($454|0)==(0|0);
       $456 = $455 ? 43053 : 43062;
       HEAP32[$zMiss>>2] = $456;
       HEAP32[$rc>>2] = 1;
       $457 = HEAP32[$6>>2]|0;
       $458 = HEAP32[$zMiss>>2]|0;
       HEAP32[$vararg_buffer20>>2] = $458;
       _sqlite3Fts3ErrMsg($457,43219,$vararg_buffer20);
      }
     }
     $459 = HEAP32[$p>>2]|0;
     $460 = HEAP32[$zUncompress>>2]|0;
     $461 = (_fts3ReadExprList($459,$460,$rc)|0);
     $462 = HEAP32[$p>>2]|0;
     $463 = ((($462)) + 216|0);
     HEAP32[$463>>2] = $461;
     $464 = HEAP32[$p>>2]|0;
     $465 = HEAP32[$zCompress>>2]|0;
     $466 = (_fts3WriteExprList($464,$465,$rc)|0);
     $467 = HEAP32[$p>>2]|0;
     $468 = ((($467)) + 220|0);
     HEAP32[$468>>2] = $466;
     $469 = HEAP32[$rc>>2]|0;
     $470 = ($469|0)!=(0);
     if (!($470)) {
      $471 = HEAP32[$0>>2]|0;
      $472 = ($471|0)!=(0);
      if ($472) {
       $473 = HEAP32[$p>>2]|0;
       $474 = (_fts3CreateTables($473)|0);
       HEAP32[$rc>>2] = $474;
      }
      $475 = HEAP32[$isFts4>>2]|0;
      $476 = ($475|0)!=(0);
      $477 = HEAP32[$0>>2]|0;
      $478 = ($477|0)!=(0);
      $or$cond7 = $476 | $478;
      if (!($or$cond7)) {
       $479 = HEAP32[$p>>2]|0;
       $480 = ((($479)) + 229|0);
       HEAP8[$480>>0] = 2;
      }
      $481 = HEAP32[$p>>2]|0;
      _fts3DatabasePageSize($rc,$481);
      $482 = HEAP32[$p>>2]|0;
      $483 = ((($482)) + 236|0);
      $484 = HEAP32[$483>>2]|0;
      $485 = (($484) - 35)|0;
      $486 = HEAP32[$p>>2]|0;
      $487 = ((($486)) + 224|0);
      HEAP32[$487>>2] = $485;
      $488 = HEAP32[$p>>2]|0;
      _fts3DeclareVtab($rc,$488);
     }
    }
   }
  } else {
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $489 = HEAP32[$zPrefix>>2]|0;
 _sqlite3_free($489);
 $490 = HEAP32[$aIndex>>2]|0;
 _sqlite3_free($490);
 $491 = HEAP32[$zCompress>>2]|0;
 _sqlite3_free($491);
 $492 = HEAP32[$zUncompress>>2]|0;
 _sqlite3_free($492);
 $493 = HEAP32[$zContent>>2]|0;
 _sqlite3_free($493);
 $494 = HEAP32[$zLanguageid>>2]|0;
 _sqlite3_free($494);
 HEAP32[$i>>2] = 0;
 while(1) {
  $495 = HEAP32[$i>>2]|0;
  $496 = HEAP32[$nNotindexed>>2]|0;
  $497 = ($495|0)<($496|0);
  if (!($497)) {
   break;
  }
  $498 = HEAP32[$i>>2]|0;
  $499 = HEAP32[$azNotindexed>>2]|0;
  $500 = (($499) + ($498<<2)|0);
  $501 = HEAP32[$500>>2]|0;
  _sqlite3_free($501);
  $502 = HEAP32[$i>>2]|0;
  $503 = (($502) + 1)|0;
  HEAP32[$i>>2] = $503;
 }
 $504 = HEAP32[$aCol>>2]|0;
 _sqlite3_free($504);
 $505 = HEAP32[$azNotindexed>>2]|0;
 _sqlite3_free($505);
 $506 = HEAP32[$rc>>2]|0;
 $507 = ($506|0)!=(0);
 $508 = HEAP32[$p>>2]|0;
 if (!($507)) {
  $518 = HEAP32[$5>>2]|0;
  HEAP32[$518>>2] = $508;
  $519 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $509 = ($508|0)!=(0|0);
 if ($509) {
  $510 = HEAP32[$p>>2]|0;
  (_fts3DisconnectMethod($510)|0);
  $519 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $511 = HEAP32[$pTokenizer>>2]|0;
 $512 = ($511|0)!=(0|0);
 if (!($512)) {
  $519 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $513 = HEAP32[$pTokenizer>>2]|0;
 $514 = HEAP32[$513>>2]|0;
 $515 = ((($514)) + 8|0);
 $516 = HEAP32[$515>>2]|0;
 $517 = HEAP32[$pTokenizer>>2]|0;
 (FUNCTION_TABLE_ii[$516 & 255]($517)|0);
 $519 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($519|0);
}
function _sqlite3Fts3IsIdChar($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = $2 & 128;
 $4 = ($3|0)!=(0);
 if ($4) {
  $12 = 1;
  $11 = $12&1;
  STACKTOP = sp;return ($11|0);
 }
 $5 = HEAP8[$0>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = (43260 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)!=(0);
 $12 = $10;
 $11 = $12&1;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3Fts3InitTokenizer($pHash,$zArg,$ppTok,$pzErr) {
 $pHash = $pHash|0;
 $zArg = $zArg|0;
 $ppTok = $ppTok|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $aArg = 0, $aNew = 0, $iArg = 0, $m = 0, $n = 0, $nNew = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $z = 0, $zCopy = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $rc = sp + 56|0;
 $z = sp + 52|0;
 $n = sp + 48|0;
 $zCopy = sp + 44|0;
 $zEnd = sp + 40|0;
 $m = sp + 36|0;
 $aArg = sp + 32|0;
 $iArg = sp + 28|0;
 $nNew = sp + 24|0;
 $aNew = sp + 20|0;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $zArg;
 HEAP32[$3>>2] = $ppTok;
 HEAP32[$4>>2] = $pzErr;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$z>>2] = $5;
 HEAP32[$n>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $6;
 $7 = (_sqlite3_mprintf(17521,$vararg_buffer)|0);
 HEAP32[$zCopy>>2] = $7;
 $8 = HEAP32[$zCopy>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  HEAP32[$0>>2] = 7;
  $83 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($83|0);
 }
 $10 = HEAP32[$zCopy>>2]|0;
 $11 = (_strlen($10)|0);
 $12 = HEAP32[$zCopy>>2]|0;
 $13 = (($12) + ($11)|0);
 HEAP32[$zEnd>>2] = $13;
 $14 = HEAP32[$zCopy>>2]|0;
 $15 = (_sqlite3Fts3NextToken($14,$n)|0);
 HEAP32[$z>>2] = $15;
 $16 = HEAP32[$z>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$zCopy>>2]|0;
  HEAP32[$z>>2] = $18;
 }
 $19 = HEAP32[$n>>2]|0;
 $20 = HEAP32[$z>>2]|0;
 $21 = (($20) + ($19)|0);
 HEAP8[$21>>0] = 0;
 $22 = HEAP32[$z>>2]|0;
 _sqlite3Fts3Dequote($22);
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$z>>2]|0;
 $25 = HEAP32[$z>>2]|0;
 $26 = (_strlen($25)|0);
 $27 = (($26) + 1)|0;
 $28 = (_sqlite3Fts3HashFind($23,$24,$27)|0);
 HEAP32[$m>>2] = $28;
 $29 = HEAP32[$m>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  HEAP32[$aArg>>2] = 0;
  HEAP32[$iArg>>2] = 0;
  $33 = HEAP32[$n>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = HEAP32[$z>>2]|0;
  $36 = (($35) + ($34)|0);
  HEAP32[$z>>2] = $36;
  while(1) {
   $37 = HEAP32[$z>>2]|0;
   $38 = HEAP32[$zEnd>>2]|0;
   $39 = ($37>>>0)<($38>>>0);
   if (!($39)) {
    break;
   }
   $40 = HEAP32[$z>>2]|0;
   $41 = (_sqlite3Fts3NextToken($40,$n)|0);
   HEAP32[$z>>2] = $41;
   $42 = (0|0)!=($41|0);
   if (!($42)) {
    break;
   }
   $43 = HEAP32[$iArg>>2]|0;
   $44 = (($43) + 1)|0;
   $45 = $44<<2;
   HEAP32[$nNew>>2] = $45;
   $46 = HEAP32[$aArg>>2]|0;
   $47 = HEAP32[$nNew>>2]|0;
   $48 = (_sqlite3_realloc($46,$47)|0);
   HEAP32[$aNew>>2] = $48;
   $49 = HEAP32[$aNew>>2]|0;
   $50 = ($49|0)!=(0|0);
   if (!($50)) {
    label = 11;
    break;
   }
   $53 = HEAP32[$aNew>>2]|0;
   HEAP32[$aArg>>2] = $53;
   $54 = HEAP32[$z>>2]|0;
   $55 = HEAP32[$iArg>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$iArg>>2] = $56;
   $57 = HEAP32[$aArg>>2]|0;
   $58 = (($57) + ($55<<2)|0);
   HEAP32[$58>>2] = $54;
   $59 = HEAP32[$n>>2]|0;
   $60 = HEAP32[$z>>2]|0;
   $61 = (($60) + ($59)|0);
   HEAP8[$61>>0] = 0;
   $62 = HEAP32[$z>>2]|0;
   _sqlite3Fts3Dequote($62);
   $63 = HEAP32[$n>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = HEAP32[$z>>2]|0;
   $66 = (($65) + ($64)|0);
   HEAP32[$z>>2] = $66;
  }
  if ((label|0) == 11) {
   $51 = HEAP32[$zCopy>>2]|0;
   _sqlite3_free($51);
   $52 = HEAP32[$aArg>>2]|0;
   _sqlite3_free($52);
   HEAP32[$0>>2] = 7;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
  $67 = HEAP32[$m>>2]|0;
  $68 = ((($67)) + 4|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$iArg>>2]|0;
  $71 = HEAP32[$aArg>>2]|0;
  $72 = HEAP32[$3>>2]|0;
  $73 = (FUNCTION_TABLE_iiii[$69 & 255]($70,$71,$72)|0);
  HEAP32[$rc>>2] = $73;
  $74 = HEAP32[$rc>>2]|0;
  $75 = ($74|0)!=(0);
  if ($75) {
   $76 = HEAP32[$4>>2]|0;
   _sqlite3Fts3ErrMsg($76,43388,$vararg_buffer4);
  } else {
   $77 = HEAP32[$m>>2]|0;
   $78 = HEAP32[$3>>2]|0;
   $79 = HEAP32[$78>>2]|0;
   HEAP32[$79>>2] = $77;
  }
  $80 = HEAP32[$aArg>>2]|0;
  _sqlite3_free($80);
 } else {
  $31 = HEAP32[$4>>2]|0;
  $32 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $32;
  _sqlite3Fts3ErrMsg($31,43015,$vararg_buffer1);
  HEAP32[$rc>>2] = 1;
 }
 $81 = HEAP32[$zCopy>>2]|0;
 _sqlite3_free($81);
 $82 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $82;
 $83 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($83|0);
}
function _fts3IsSpecialColumn($z,$pnKey,$pzValue) {
 $z = $z|0;
 $pnKey = $pnKey|0;
 $pzValue = $pzValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $zCsr = 0, $zValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zValue = sp + 8|0;
 $zCsr = sp + 4|0;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pnKey;
 HEAP32[$3>>2] = $pzValue;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$zCsr>>2] = $4;
 while(1) {
  $5 = HEAP32[$zCsr>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)!=(61);
  $9 = HEAP32[$zCsr>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  if ($12) {
   label = 4;
   break;
  }
  $13 = HEAP32[$zCsr>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP32[$zCsr>>2] = $14;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = $9;
 $17 = $15;
 $18 = (($16) - ($17))|0;
 $19 = HEAP32[$2>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$zCsr>>2]|0;
 $21 = ((($20)) + 1|0);
 HEAP32[$vararg_buffer>>2] = $21;
 $22 = (_sqlite3_mprintf(17521,$vararg_buffer)|0);
 HEAP32[$zValue>>2] = $22;
 $23 = HEAP32[$zValue>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$zValue>>2]|0;
  _sqlite3Fts3Dequote($25);
 }
 $26 = HEAP32[$zValue>>2]|0;
 $27 = HEAP32[$3>>2]|0;
 HEAP32[$27>>2] = $26;
 HEAP32[$0>>2] = 1;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fts3ContentColumns($db,$zDb,$zTbl,$pazCol,$pnCol,$pnStr,$pzErr) {
 $db = $db|0;
 $zDb = $zDb|0;
 $zTbl = $zTbl|0;
 $pazCol = $pazCol|0;
 $pnCol = $pnCol|0;
 $pnStr = $pnStr|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $azCol = 0, $i = 0, $n = 0;
 var $nCol = 0, $nStr = 0, $p = 0, $pStmt = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $zCol = 0, $zCol1 = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $6 = sp + 56|0;
 $rc = sp + 52|0;
 $zSql = sp + 48|0;
 $pStmt = sp + 44|0;
 $azCol = sp + 40|0;
 $nStr = sp + 36|0;
 $nCol = sp + 32|0;
 $i = sp + 28|0;
 $zCol = sp + 24|0;
 $p = sp + 20|0;
 $zCol1 = sp + 16|0;
 $n = sp + 12|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDb;
 HEAP32[$2>>2] = $zTbl;
 HEAP32[$3>>2] = $pazCol;
 HEAP32[$4>>2] = $pnCol;
 HEAP32[$5>>2] = $pnStr;
 HEAP32[$6>>2] = $pzErr;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $7;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $8;
 $9 = (_sqlite3_mprintf(43406,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $9;
 $10 = HEAP32[$zSql>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$zSql>>2]|0;
  $14 = (_sqlite3_prepare($12,$13,-1,$pStmt,0)|0);
  HEAP32[$rc>>2] = $14;
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = HEAP32[$6>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = (_sqlite3_errmsg($18)|0);
   HEAP32[$vararg_buffer2>>2] = $19;
   _sqlite3Fts3ErrMsg($17,17521,$vararg_buffer2);
  }
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $20 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($20);
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if (!($22)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 HEAP32[$nStr>>2] = 0;
 $23 = HEAP32[$pStmt>>2]|0;
 $24 = (_sqlite3_column_count($23)|0);
 HEAP32[$nCol>>2] = $24;
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$nCol>>2]|0;
  $27 = ($25|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$pStmt>>2]|0;
  $29 = HEAP32[$i>>2]|0;
  $30 = (_sqlite3_column_name($28,$29)|0);
  HEAP32[$zCol>>2] = $30;
  $31 = HEAP32[$zCol>>2]|0;
  $32 = (_strlen($31)|0);
  $33 = (($32) + 1)|0;
  $34 = HEAP32[$nStr>>2]|0;
  $35 = (($34) + ($33))|0;
  HEAP32[$nStr>>2] = $35;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$nCol>>2]|0;
 $39 = $38<<2;
 $40 = HEAP32[$nStr>>2]|0;
 $41 = (($39) + ($40))|0;
 $42 = (_sqlite3_malloc($41)|0);
 HEAP32[$azCol>>2] = $42;
 $43 = HEAP32[$azCol>>2]|0;
 $44 = ($43|0)==(0|0);
 L13: do {
  if ($44) {
   HEAP32[$rc>>2] = 7;
  } else {
   $45 = HEAP32[$nCol>>2]|0;
   $46 = HEAP32[$azCol>>2]|0;
   $47 = (($46) + ($45<<2)|0);
   HEAP32[$p>>2] = $47;
   HEAP32[$i>>2] = 0;
   while(1) {
    $48 = HEAP32[$i>>2]|0;
    $49 = HEAP32[$nCol>>2]|0;
    $50 = ($48|0)<($49|0);
    if (!($50)) {
     break L13;
    }
    $51 = HEAP32[$pStmt>>2]|0;
    $52 = HEAP32[$i>>2]|0;
    $53 = (_sqlite3_column_name($51,$52)|0);
    HEAP32[$zCol1>>2] = $53;
    $54 = HEAP32[$zCol1>>2]|0;
    $55 = (_strlen($54)|0);
    $56 = (($55) + 1)|0;
    HEAP32[$n>>2] = $56;
    $57 = HEAP32[$p>>2]|0;
    $58 = HEAP32[$zCol1>>2]|0;
    $59 = HEAP32[$n>>2]|0;
    _memcpy(($57|0),($58|0),($59|0))|0;
    $60 = HEAP32[$p>>2]|0;
    $61 = HEAP32[$i>>2]|0;
    $62 = HEAP32[$azCol>>2]|0;
    $63 = (($62) + ($61<<2)|0);
    HEAP32[$63>>2] = $60;
    $64 = HEAP32[$n>>2]|0;
    $65 = HEAP32[$p>>2]|0;
    $66 = (($65) + ($64)|0);
    HEAP32[$p>>2] = $66;
    $67 = HEAP32[$i>>2]|0;
    $68 = (($67) + 1)|0;
    HEAP32[$i>>2] = $68;
   }
  }
 } while(0);
 $69 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_finalize($69)|0);
 $70 = HEAP32[$nCol>>2]|0;
 $71 = HEAP32[$4>>2]|0;
 HEAP32[$71>>2] = $70;
 $72 = HEAP32[$nStr>>2]|0;
 $73 = HEAP32[$5>>2]|0;
 HEAP32[$73>>2] = $72;
 $74 = HEAP32[$azCol>>2]|0;
 $75 = HEAP32[$3>>2]|0;
 HEAP32[$75>>2] = $74;
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _fts3PrefixParameter($zParam,$pnIndex,$apIndex) {
 $zParam = $zParam|0;
 $pnIndex = $pnIndex|0;
 $apIndex = $apIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aIndex = 0;
 var $i = 0, $nIndex = 0, $nPrefix = 0, $p = 0, $p1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aIndex = sp + 20|0;
 $nIndex = sp + 16|0;
 $p = sp + 12|0;
 $p1 = sp + 8|0;
 $i = sp + 4|0;
 $nPrefix = sp;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$2>>2] = $pnIndex;
 HEAP32[$3>>2] = $apIndex;
 HEAP32[$nIndex>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 L1: do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$nIndex>>2]|0;
    $11 = (($10) + 1)|0;
    HEAP32[$nIndex>>2] = $11;
    $12 = HEAP32[$1>>2]|0;
    HEAP32[$p>>2] = $12;
    while(1) {
     $13 = HEAP32[$p>>2]|0;
     $14 = HEAP8[$13>>0]|0;
     $15 = ($14<<24>>24)!=(0);
     if (!($15)) {
      break L1;
     }
     $16 = HEAP32[$p>>2]|0;
     $17 = HEAP8[$16>>0]|0;
     $18 = $17 << 24 >> 24;
     $19 = ($18|0)==(44);
     if ($19) {
      $20 = HEAP32[$nIndex>>2]|0;
      $21 = (($20) + 1)|0;
      HEAP32[$nIndex>>2] = $21;
     }
     $22 = HEAP32[$p>>2]|0;
     $23 = ((($22)) + 1|0);
     HEAP32[$p>>2] = $23;
    }
   }
  }
 } while(0);
 $24 = HEAP32[$nIndex>>2]|0;
 $25 = ($24*24)|0;
 $26 = (_sqlite3_malloc($25)|0);
 HEAP32[$aIndex>>2] = $26;
 $27 = HEAP32[$aIndex>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 HEAP32[$28>>2] = $27;
 $29 = HEAP32[$aIndex>>2]|0;
 $30 = ($29|0)!=(0|0);
 if (!($30)) {
  HEAP32[$0>>2] = 7;
  $58 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $31 = HEAP32[$aIndex>>2]|0;
 $32 = HEAP32[$nIndex>>2]|0;
 $33 = ($32*24)|0;
 _memset(($31|0),0,($33|0))|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = ($34|0)!=(0|0);
 L15: do {
  if ($35) {
   $36 = HEAP32[$1>>2]|0;
   HEAP32[$p1>>2] = $36;
   HEAP32[$i>>2] = 1;
   while(1) {
    $37 = HEAP32[$i>>2]|0;
    $38 = HEAP32[$nIndex>>2]|0;
    $39 = ($37|0)<($38|0);
    if (!($39)) {
     break L15;
    }
    HEAP32[$nPrefix>>2] = 0;
    $40 = (_fts3GobbleInt($p1,$nPrefix)|0);
    $41 = ($40|0)!=(0);
    if ($41) {
     break;
    }
    $42 = HEAP32[$nPrefix>>2]|0;
    $43 = ($42|0)==(0);
    if ($43) {
     $44 = HEAP32[$nIndex>>2]|0;
     $45 = (($44) + -1)|0;
     HEAP32[$nIndex>>2] = $45;
     $46 = HEAP32[$i>>2]|0;
     $47 = (($46) + -1)|0;
     HEAP32[$i>>2] = $47;
    } else {
     $48 = HEAP32[$nPrefix>>2]|0;
     $49 = HEAP32[$i>>2]|0;
     $50 = HEAP32[$aIndex>>2]|0;
     $51 = (($50) + (($49*24)|0)|0);
     HEAP32[$51>>2] = $48;
    }
    $52 = HEAP32[$p1>>2]|0;
    $53 = ((($52)) + 1|0);
    HEAP32[$p1>>2] = $53;
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
   HEAP32[$0>>2] = 1;
   $58 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($58|0);
  }
 } while(0);
 $56 = HEAP32[$nIndex>>2]|0;
 $57 = HEAP32[$2>>2]|0;
 HEAP32[$57>>2] = $56;
 HEAP32[$0>>2] = 0;
 $58 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _sqlite3Fts3NextToken($zStr,$pn) {
 $zStr = $zStr|0;
 $pn = $pn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $z1 = sp + 4|0;
 $z2 = sp;
 $c = sp + 20|0;
 HEAP32[$1>>2] = $zStr;
 HEAP32[$2>>2] = $pn;
 HEAP32[$z2>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$z1>>2] = $3;
 L1: while(1) {
  $4 = HEAP32[$z2>>2]|0;
  $5 = ($4|0)==(0|0);
  if (!($5)) {
   label = 21;
   break;
  }
  $6 = HEAP32[$z1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  HEAP8[$c>>0] = $7;
  $8 = HEAP8[$c>>0]|0;
  $9 = $8 << 24 >> 24;
  switch ($9|0) {
  case 0:  {
   label = 4;
   break L1;
   break;
  }
  case 96: case 34: case 39:  {
   $10 = HEAP32[$z1>>2]|0;
   HEAP32[$z2>>2] = $10;
   while(1) {
    $11 = HEAP32[$z2>>2]|0;
    $12 = ((($11)) + 1|0);
    HEAP32[$z2>>2] = $12;
    $13 = HEAP8[$12>>0]|0;
    $14 = $13 << 24 >> 24;
    $15 = ($14|0)!=(0);
    if (!($15)) {
     continue L1;
    }
    $16 = HEAP32[$z2>>2]|0;
    $17 = HEAP8[$16>>0]|0;
    $18 = $17 << 24 >> 24;
    $19 = HEAP8[$c>>0]|0;
    $20 = $19 << 24 >> 24;
    $21 = ($18|0)!=($20|0);
    if ($21) {
     continue;
    }
    $22 = HEAP32[$z2>>2]|0;
    $23 = ((($22)) + 1|0);
    HEAP32[$z2>>2] = $23;
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = HEAP8[$c>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($25|0)==($27|0);
    if (!($28)) {
     continue L1;
    }
   }
   break;
  }
  case 91:  {
   $29 = HEAP32[$z1>>2]|0;
   $30 = ((($29)) + 1|0);
   HEAP32[$z2>>2] = $30;
   while(1) {
    $31 = HEAP32[$z2>>2]|0;
    $32 = HEAP8[$31>>0]|0;
    $33 = $32 << 24 >> 24;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$z2>>2]|0;
     $36 = HEAP8[$35>>0]|0;
     $37 = $36 << 24 >> 24;
     $38 = ($37|0)!=(93);
     $65 = $38;
    } else {
     $65 = 0;
    }
    $39 = HEAP32[$z2>>2]|0;
    if (!($65)) {
     break;
    }
    $40 = ((($39)) + 1|0);
    HEAP32[$z2>>2] = $40;
   }
   $41 = HEAP8[$39>>0]|0;
   $42 = ($41<<24>>24)!=(0);
   if (!($42)) {
    continue L1;
   }
   $43 = HEAP32[$z2>>2]|0;
   $44 = ((($43)) + 1|0);
   HEAP32[$z2>>2] = $44;
   continue L1;
   break;
  }
  default: {
   $45 = HEAP32[$z1>>2]|0;
   $46 = HEAP8[$45>>0]|0;
   $47 = (_sqlite3Fts3IsIdChar($46)|0);
   $48 = ($47|0)!=(0);
   $49 = HEAP32[$z1>>2]|0;
   $50 = ((($49)) + 1|0);
   if (!($48)) {
    HEAP32[$z1>>2] = $50;
    continue L1;
   }
   HEAP32[$z2>>2] = $50;
   while(1) {
    $51 = HEAP32[$z2>>2]|0;
    $52 = HEAP8[$51>>0]|0;
    $53 = (_sqlite3Fts3IsIdChar($52)|0);
    $54 = ($53|0)!=(0);
    if (!($54)) {
     continue L1;
    }
    $55 = HEAP32[$z2>>2]|0;
    $56 = ((($55)) + 1|0);
    HEAP32[$z2>>2] = $56;
   }
  }
  }
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 else if ((label|0) == 21) {
  $57 = HEAP32[$z2>>2]|0;
  $58 = HEAP32[$z1>>2]|0;
  $59 = $57;
  $60 = $58;
  $61 = (($59) - ($60))|0;
  $62 = HEAP32[$2>>2]|0;
  HEAP32[$62>>2] = $61;
  $63 = HEAP32[$z1>>2]|0;
  HEAP32[$0>>2] = $63;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _fts3ReadExprList($p,$zFunc,$pRc) {
 $p = $p|0;
 $zFunc = $zFunc|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $zFree = 0, $zFunction = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $zRet = sp + 80|0;
 $zFree = sp + 76|0;
 $zFunction = sp + 72|0;
 $i = sp + 68|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $pRc;
 HEAP32[$zRet>>2] = 0;
 HEAP32[$zFree>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (_fts3QuoteId($9)|0);
   HEAP32[$zFunction>>2] = $10;
   HEAP32[$zFree>>2] = $10;
  } else {
   HEAP32[$zFunction>>2] = 17532;
  }
  $11 = HEAP32[$2>>2]|0;
  _fts3Appendf($11,$zRet,43426,$vararg_buffer);
  HEAP32[$i>>2] = 0;
  while(1) {
   $12 = HEAP32[$i>>2]|0;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 24|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($12|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$2>>2]|0;
   $18 = HEAP32[$zFunction>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$i>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 28|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + ($20<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $18;
   $vararg_ptr3 = ((($vararg_buffer1)) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $19;
   $vararg_ptr4 = ((($vararg_buffer1)) + 8|0);
   HEAP32[$vararg_ptr4>>2] = $25;
   _fts3Appendf($17,$zRet,43432,$vararg_buffer1);
   $26 = HEAP32[$i>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$i>>2] = $27;
  }
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 44|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer5>>2] = 43454;
   _fts3Appendf($32,$zRet,43447,$vararg_buffer5);
  }
  $33 = HEAP32[$zFree>>2]|0;
  _sqlite3_free($33);
 } else {
  $34 = HEAP32[$2>>2]|0;
  _fts3Appendf($34,$zRet,27337,$vararg_buffer8);
  HEAP32[$i>>2] = 0;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$0>>2]|0;
   $37 = ((($36)) + 24|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($35|0)<($38|0);
   if (!($39)) {
    break;
   }
   $40 = HEAP32[$2>>2]|0;
   $41 = HEAP32[$i>>2]|0;
   $42 = HEAP32[$0>>2]|0;
   $43 = ((($42)) + 28|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($41<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$vararg_buffer10>>2] = $46;
   _fts3Appendf($40,$zRet,43461,$vararg_buffer10);
   $47 = HEAP32[$i>>2]|0;
   $48 = (($47) + 1)|0;
   HEAP32[$i>>2] = $48;
  }
  $49 = HEAP32[$0>>2]|0;
  $50 = ((($49)) + 44|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ($51|0)!=(0|0);
  if ($52) {
   $53 = HEAP32[$2>>2]|0;
   $54 = HEAP32[$0>>2]|0;
   $55 = ((($54)) + 44|0);
   $56 = HEAP32[$55>>2]|0;
   HEAP32[$vararg_buffer13>>2] = $56;
   _fts3Appendf($53,$zRet,43447,$vararg_buffer13);
  }
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 40|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)!=(0|0);
 $65 = HEAP32[$0>>2]|0;
 if ($64) {
  $66 = ((($65)) + 40|0);
  $67 = HEAP32[$66>>2]|0;
  $75 = $67;
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 40|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)!=(0|0);
  $74 = $73 ? 17532 : 43493;
  HEAP32[$vararg_buffer16>>2] = $60;
  $vararg_ptr19 = ((($vararg_buffer16)) + 4|0);
  HEAP32[$vararg_ptr19>>2] = $75;
  $vararg_ptr20 = ((($vararg_buffer16)) + 8|0);
  HEAP32[$vararg_ptr20>>2] = $74;
  _fts3Appendf($57,$zRet,43470,$vararg_buffer16);
  $76 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($76|0);
 } else {
  $68 = ((($65)) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  $75 = $69;
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 40|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)!=(0|0);
  $74 = $73 ? 17532 : 43493;
  HEAP32[$vararg_buffer16>>2] = $60;
  $vararg_ptr19 = ((($vararg_buffer16)) + 4|0);
  HEAP32[$vararg_ptr19>>2] = $75;
  $vararg_ptr20 = ((($vararg_buffer16)) + 8|0);
  HEAP32[$vararg_ptr20>>2] = $74;
  _fts3Appendf($57,$zRet,43470,$vararg_buffer16);
  $76 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 return (0)|0;
}
function _fts3WriteExprList($p,$zFunc,$pRc) {
 $p = $p|0;
 $zFunc = $zFunc|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $zFree = 0, $zFunction = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $zRet = sp + 32|0;
 $zFree = sp + 28|0;
 $zFunction = sp + 24|0;
 $i = sp + 20|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $pRc;
 HEAP32[$zRet>>2] = 0;
 HEAP32[$zFree>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (_fts3QuoteId($5)|0);
  HEAP32[$zFunction>>2] = $6;
  HEAP32[$zFree>>2] = $6;
 } else {
  HEAP32[$zFunction>>2] = 17532;
 }
 $7 = HEAP32[$2>>2]|0;
 _fts3Appendf($7,$zRet,21679,$vararg_buffer);
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$zFunction>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $14;
  _fts3Appendf($13,$zRet,43502,$vararg_buffer1);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  $22 = HEAP32[$zFree>>2]|0;
  _sqlite3_free($22);
  $23 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $21 = HEAP32[$2>>2]|0;
 _fts3Appendf($21,$zRet,43509,$vararg_buffer3);
 $22 = HEAP32[$zFree>>2]|0;
 _sqlite3_free($22);
 $23 = HEAP32[$zRet>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _fts3CreateTables($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $db = 0, $i = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer15 = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer23 = 0, $vararg_buffer6 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr18 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $z = 0;
 var $zContentCols = 0, $zLanguageid = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer23 = sp + 64|0;
 $vararg_buffer19 = sp + 56|0;
 $vararg_buffer15 = sp + 48|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 96|0;
 $rc = sp + 92|0;
 $i = sp + 88|0;
 $db = sp + 84|0;
 $zLanguageid = sp + 80|0;
 $zContentCols = sp + 76|0;
 $z = sp + 72|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$zLanguageid>>2] = $10;
  $11 = (_sqlite3_mprintf(43513,$vararg_buffer)|0);
  HEAP32[$zContentCols>>2] = $11;
  HEAP32[$i>>2] = 0;
  while(1) {
   $12 = HEAP32[$zContentCols>>2]|0;
   $13 = ($12|0)!=(0|0);
   if (!($13)) {
    break;
   }
   $14 = HEAP32[$i>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 24|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ($14|0)<($17|0);
   if (!($18)) {
    break;
   }
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 28|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + ($19<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$z>>2] = $24;
   $25 = HEAP32[$zContentCols>>2]|0;
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$z>>2]|0;
   HEAP32[$vararg_buffer2>>2] = $25;
   $vararg_ptr4 = ((($vararg_buffer2)) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $26;
   $vararg_ptr5 = ((($vararg_buffer2)) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $27;
   $28 = (_sqlite3_mprintf(43539,$vararg_buffer2)|0);
   HEAP32[$zContentCols>>2] = $28;
   $29 = HEAP32[$i>>2]|0;
   $30 = (($29) + 1)|0;
   HEAP32[$i>>2] = $30;
  }
  $31 = HEAP32[$zLanguageid>>2]|0;
  $32 = ($31|0)!=(0|0);
  $33 = HEAP32[$zContentCols>>2]|0;
  $34 = ($33|0)!=(0|0);
  $or$cond = $32 & $34;
  if ($or$cond) {
   $35 = HEAP32[$zContentCols>>2]|0;
   $36 = HEAP32[$zLanguageid>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $35;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $36;
   $37 = (_sqlite3_mprintf(43551,$vararg_buffer6)|0);
   HEAP32[$zContentCols>>2] = $37;
  }
  $38 = HEAP32[$zContentCols>>2]|0;
  $39 = ($38|0)==(0|0);
  if ($39) {
   HEAP32[$rc>>2] = 7;
  }
  $40 = HEAP32[$db>>2]|0;
  $41 = HEAP32[$0>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 20|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$zContentCols>>2]|0;
  HEAP32[$vararg_buffer10>>2] = $43;
  $vararg_ptr13 = ((($vararg_buffer10)) + 4|0);
  HEAP32[$vararg_ptr13>>2] = $46;
  $vararg_ptr14 = ((($vararg_buffer10)) + 8|0);
  HEAP32[$vararg_ptr14>>2] = $47;
  _fts3DbExec($rc,$40,43562,$vararg_buffer10);
  $48 = HEAP32[$zContentCols>>2]|0;
  _sqlite3_free($48);
 }
 $49 = HEAP32[$db>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 16|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 20|0);
 $55 = HEAP32[$54>>2]|0;
 HEAP32[$vararg_buffer15>>2] = $52;
 $vararg_ptr18 = ((($vararg_buffer15)) + 4|0);
 HEAP32[$vararg_ptr18>>2] = $55;
 _fts3DbExec($rc,$49,43595,$vararg_buffer15);
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 20|0);
 $62 = HEAP32[$61>>2]|0;
 HEAP32[$vararg_buffer19>>2] = $59;
 $vararg_ptr22 = ((($vararg_buffer19)) + 4|0);
 HEAP32[$vararg_ptr22>>2] = $62;
 _fts3DbExec($rc,$56,43667,$vararg_buffer19);
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 230|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = ($65<<24>>24)!=(0);
 if ($66) {
  $67 = HEAP32[$db>>2]|0;
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 16|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = HEAP32[$0>>2]|0;
  $72 = ((($71)) + 20|0);
  $73 = HEAP32[$72>>2]|0;
  HEAP32[$vararg_buffer23>>2] = $70;
  $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
  HEAP32[$vararg_ptr26>>2] = $73;
  _fts3DbExec($rc,$67,43820,$vararg_buffer23);
 }
 $74 = HEAP32[$0>>2]|0;
 $75 = ((($74)) + 229|0);
 $76 = HEAP8[$75>>0]|0;
 $77 = ($76<<24>>24)!=(0);
 if (!($77)) {
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $78 = HEAP32[$0>>2]|0;
 _sqlite3Fts3CreateStatTable($rc,$78);
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _fts3DatabasePageSize($pRc,$p) {
 $pRc = $pRc|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, $vararg_buffer = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $zSql = sp + 8|0;
 $pStmt = sp + 4|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$vararg_buffer>>2] = $7;
 $8 = (_sqlite3_mprintf(43965,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $8;
 $9 = HEAP32[$zSql>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$zSql>>2]|0;
   $15 = (_sqlite3_prepare($13,$14,-1,$pStmt,0)|0);
   HEAP32[$rc>>2] = $15;
   $16 = HEAP32[$rc>>2]|0;
   $17 = ($16|0)==(0);
   if ($17) {
    $18 = HEAP32[$pStmt>>2]|0;
    (_sqlite3_step($18)|0);
    $19 = HEAP32[$pStmt>>2]|0;
    $20 = (_sqlite3_column_int($19,0)|0);
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 236|0);
    HEAP32[$22>>2] = $20;
    $23 = HEAP32[$pStmt>>2]|0;
    $24 = (_sqlite3_finalize($23)|0);
    HEAP32[$rc>>2] = $24;
    break;
   }
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(23);
   if ($26) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 236|0);
    HEAP32[$28>>2] = 1024;
    HEAP32[$rc>>2] = 0;
   }
  } else {
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $29 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($29);
 $30 = HEAP32[$rc>>2]|0;
 $31 = HEAP32[$0>>2]|0;
 HEAP32[$31>>2] = $30;
 STACKTOP = sp;return;
}
function _fts3DeclareVtab($pRc,$p) {
 $pRc = $pRc|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0, $rc = 0;
 var $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer9 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr8 = 0, $zCols = 0, $zLanguageid = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $i = sp + 52|0;
 $rc = sp + 48|0;
 $zSql = sp + 44|0;
 $zCols = sp + 40|0;
 $zLanguageid = sp + 36|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = 43985;
 }
 HEAP32[$zLanguageid>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$vararg_buffer>>2] = 1;
 (_sqlite3_vtab_config($15,1,$vararg_buffer)|0);
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 28|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $19;
 $20 = (_sqlite3_mprintf(43994,$vararg_buffer2)|0);
 HEAP32[$zCols>>2] = $20;
 HEAP32[$i>>2] = 1;
 while(1) {
  $21 = HEAP32[$zCols>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$i>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 24|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($23|0)<($26|0);
   $56 = $27;
  } else {
   $56 = 0;
  }
  $28 = HEAP32[$zCols>>2]|0;
  if (!($56)) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 28|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29<<2)|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$vararg_buffer5>>2] = $28;
  $vararg_ptr8 = ((($vararg_buffer5)) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $34;
  $35 = (_sqlite3_mprintf(43999,$vararg_buffer5)|0);
  HEAP32[$zCols>>2] = $35;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 20|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$zLanguageid>>2]|0;
 HEAP32[$vararg_buffer9>>2] = $28;
 $vararg_ptr12 = ((($vararg_buffer9)) + 4|0);
 HEAP32[$vararg_ptr12>>2] = $40;
 $vararg_ptr13 = ((($vararg_buffer9)) + 8|0);
 HEAP32[$vararg_ptr13>>2] = $41;
 $42 = (_sqlite3_mprintf(44006,$vararg_buffer9)|0);
 HEAP32[$zSql>>2] = $42;
 $43 = HEAP32[$zCols>>2]|0;
 $44 = ($43|0)!=(0|0);
 $45 = HEAP32[$zSql>>2]|0;
 $46 = ($45|0)!=(0|0);
 $or$cond = $44 & $46;
 if ($or$cond) {
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$zSql>>2]|0;
  $51 = (_sqlite3_declare_vtab($49,$50)|0);
  HEAP32[$rc>>2] = $51;
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $52 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($52);
 $53 = HEAP32[$zCols>>2]|0;
 _sqlite3_free($53);
 $54 = HEAP32[$rc>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 HEAP32[$55>>2] = $54;
 STACKTOP = sp;return;
}
function _fts3GobbleInt($pp,$pnOut) {
 $pp = $pp|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $MAX_NPREFIX = 0, $nInt = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $MAX_NPREFIX = sp + 8|0;
 $p = sp + 4|0;
 $nInt = sp;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $pnOut;
 HEAP32[$MAX_NPREFIX>>2] = 10000000;
 HEAP32[$nInt>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)>=(48);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)<=(57);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$nInt>>2]|0;
  $14 = ($13*10)|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = (($17) - 48)|0;
  $19 = (($14) + ($18))|0;
  HEAP32[$nInt>>2] = $19;
  $20 = HEAP32[$nInt>>2]|0;
  $21 = ($20|0)>(10000000);
  if ($21) {
   label = 5;
   break;
  }
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 1|0);
  HEAP32[$p>>2] = $23;
 }
 if ((label|0) == 5) {
  HEAP32[$nInt>>2] = 0;
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = ($24|0)==($26|0);
 if ($27) {
  HEAP32[$0>>2] = 1;
  $32 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($32|0);
 } else {
  $28 = HEAP32[$nInt>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$p>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  HEAP32[$31>>2] = $30;
  HEAP32[$0>>2] = 0;
  $32 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 return (0)|0;
}
function _fts3QuoteId($zInput) {
 $zInput = $zInput|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $nRet = 0, $z = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $nRet = sp + 12|0;
 $zRet = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $zInput;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_strlen($1)|0);
 $3 = $2<<1;
 $4 = (2 + ($3))|0;
 $5 = (($4) + 1)|0;
 HEAP32[$nRet>>2] = $5;
 $6 = HEAP32[$nRet>>2]|0;
 $7 = (_sqlite3_malloc($6)|0);
 HEAP32[$zRet>>2] = $7;
 $8 = HEAP32[$zRet>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $38 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $10 = HEAP32[$zRet>>2]|0;
 HEAP32[$z>>2] = $10;
 $11 = HEAP32[$z>>2]|0;
 $12 = ((($11)) + 1|0);
 HEAP32[$z>>2] = $12;
 HEAP8[$11>>0] = 34;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21 << 24 >> 24;
  $23 = ($22|0)==(34);
  if ($23) {
   $24 = HEAP32[$z>>2]|0;
   $25 = ((($24)) + 1|0);
   HEAP32[$z>>2] = $25;
   HEAP8[$24>>0] = 34;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + ($26)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = HEAP32[$z>>2]|0;
  $31 = ((($30)) + 1|0);
  HEAP32[$z>>2] = $31;
  HEAP8[$30>>0] = $29;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 $34 = HEAP32[$z>>2]|0;
 $35 = ((($34)) + 1|0);
 HEAP32[$z>>2] = $35;
 HEAP8[$34>>0] = 34;
 $36 = HEAP32[$z>>2]|0;
 $37 = ((($36)) + 1|0);
 HEAP32[$z>>2] = $37;
 HEAP8[$36>>0] = 0;
 $38 = HEAP32[$zRet>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3Appendf($pRc,$pz,$zFormat,$varargs) {
 $pRc = $pRc|0;
 $pz = $pz|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $z = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $ap = sp + 16|0;
 $z = sp + 12|0;
 $z2 = sp + 8|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $pz;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3_vmprintf($6,$ap)|0);
 HEAP32[$z>>2] = $7;
 $8 = HEAP32[$z>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$z>>2]|0;
   HEAP32[$vararg_buffer>>2] = $14;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $15;
   $16 = (_sqlite3_mprintf(28532,$vararg_buffer)|0);
   HEAP32[$z2>>2] = $16;
   $17 = HEAP32[$z>>2]|0;
   _sqlite3_free($17);
   $18 = HEAP32[$z2>>2]|0;
   HEAP32[$z>>2] = $18;
  }
 }
 $19 = HEAP32[$z>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  HEAP32[$21>>2] = 7;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 _sqlite3_free($23);
 $24 = HEAP32[$z>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 HEAP32[$25>>2] = $24;
 STACKTOP = sp;return;
}
function _fts3DbExec($pRc,$db,$zFormat,$varargs) {
 $pRc = $pRc|0;
 $db = $db|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $ap = sp + 8|0;
 $zSql = sp;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3_vmprintf($6,$ap)|0);
 HEAP32[$zSql>>2] = $7;
 $8 = HEAP32[$zSql>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  HEAP32[$10>>2] = 7;
  STACKTOP = sp;return;
 } else {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$zSql>>2]|0;
  $13 = (_sqlite3_exec($11,$12,0,0,0)|0);
  $14 = HEAP32[$0>>2]|0;
  HEAP32[$14>>2] = $13;
  $15 = HEAP32[$zSql>>2]|0;
  _sqlite3_free($15);
  STACKTOP = sp;return;
 }
}
function _sqlite3Fts3CreateStatTable($pRc,$p) {
 $pRc = $pRc|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$vararg_buffer>>2] = $8;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 _fts3DbExec($2,$5,43888,$vararg_buffer);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 229|0);
 HEAP8[$16>>0] = 1;
 STACKTOP = sp;return;
}
function _fts3SetEstimatedRows($pIdxInfo,$0,$1) {
 $pIdxInfo = $pIdxInfo|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pIdxInfo;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_libversion_number()|0);
 $9 = ($8|0)>=(3008002);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 48|0);
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $12;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $15;
 STACKTOP = sp;return;
}
function _fts3SetUniqueFlag($pIdxInfo) {
 $pIdxInfo = $pIdxInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pIdxInfo;
 $1 = (_sqlite3_libversion_number()|0);
 $2 = ($1|0)>=(3008012);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 | 1;
 HEAP32[$4>>2] = $6;
 STACKTOP = sp;return;
}
function _sqlite3Fts3ExprFree($pDel) {
 $pDel = $pDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $p = 0, $pParent = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pParent = sp;
 HEAP32[$0>>2] = $pDel;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 while(1) {
  $2 = HEAP32[$p>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 12|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   $8 = HEAP32[$p>>2]|0;
   $9 = ((($8)) + 16|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if (!($11)) {
    break;
   }
  }
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  $16 = HEAP32[$p>>2]|0;
  if ($15) {
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $21 = $18;
  } else {
   $19 = ((($16)) + 16|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $20;
  }
  HEAP32[$p>>2] = $21;
 }
 L11: while(1) {
  $22 = HEAP32[$p>>2]|0;
  $23 = ($22|0)!=(0|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pParent>>2] = $26;
  $27 = HEAP32[$p>>2]|0;
  _fts3FreeExprNode($27);
  $28 = HEAP32[$pParent>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = HEAP32[$pParent>>2]|0;
   $32 = ((($31)) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($30|0)==($33|0);
   if ($34) {
    $35 = HEAP32[$pParent>>2]|0;
    $36 = ((($35)) + 16|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)!=(0|0);
    if ($38) {
     $39 = HEAP32[$pParent>>2]|0;
     $40 = ((($39)) + 16|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$p>>2] = $41;
     while(1) {
      $42 = HEAP32[$p>>2]|0;
      $43 = ($42|0)!=(0|0);
      if (!($43)) {
       continue L11;
      }
      $44 = HEAP32[$p>>2]|0;
      $45 = ((($44)) + 12|0);
      $46 = HEAP32[$45>>2]|0;
      $47 = ($46|0)!=(0|0);
      if (!($47)) {
       $48 = HEAP32[$p>>2]|0;
       $49 = ((($48)) + 16|0);
       $50 = HEAP32[$49>>2]|0;
       $51 = ($50|0)!=(0|0);
       if (!($51)) {
        continue L11;
       }
      }
      $52 = HEAP32[$p>>2]|0;
      $53 = ((($52)) + 12|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($54|0)!=(0|0);
      $56 = HEAP32[$p>>2]|0;
      if ($55) {
       $57 = ((($56)) + 12|0);
       $58 = HEAP32[$57>>2]|0;
       $61 = $58;
      } else {
       $59 = ((($56)) + 16|0);
       $60 = HEAP32[$59>>2]|0;
       $61 = $60;
      }
      HEAP32[$p>>2] = $61;
     }
    }
   }
  }
  $62 = HEAP32[$pParent>>2]|0;
  HEAP32[$p>>2] = $62;
 }
 STACKTOP = sp;return;
}
function _sqlite3Fts3FreeDeferredTokens($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pDef = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pDef>>2] = $3;
 while(1) {
  $4 = HEAP32[$pDef>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$pDef>>2]|0;
  $7 = ((($6)) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$pNext>>2] = $8;
  $9 = HEAP32[$pDef>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  _fts3PendingListDelete($11);
  $12 = HEAP32[$pDef>>2]|0;
  _sqlite3_free($12);
  $13 = HEAP32[$pNext>>2]|0;
  HEAP32[$pDef>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 HEAP32[$15>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3MIBufferFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP8[$3>>0] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 2|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)==(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 _sqlite3_free($18);
 STACKTOP = sp;return;
}
function _fts3FreeExprNode($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3Fts3EvalPhraseCleanup($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$0>>2]|0;
 _sqlite3_free($7);
 STACKTOP = sp;return;
}
function _sqlite3Fts3EvalPhraseCleanup($pPhrase) {
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pPhrase;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$0>>2]|0;
 _fts3EvalInvalidatePoslist($5);
 $6 = HEAP32[$0>>2]|0;
 dest=$6; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 72|0);
  $15 = (($14) + (($12*24)|0)|0);
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  _fts3SegReaderCursorFree($17);
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 72|0);
  $21 = (($20) + (($18*24)|0)|0);
  $22 = ((($21)) + 20|0);
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _fts3EvalInvalidatePoslist($pPhrase) {
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPhrase;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 28|0);
  $7 = HEAP32[$6>>2]|0;
  _sqlite3_free($7);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 32|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 24|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3SegReaderCursorFree($pSegcsr) {
 $pSegcsr = $pSegcsr|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSegcsr;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegReaderFinish($1);
 $2 = HEAP32[$0>>2]|0;
 _sqlite3_free($2);
 STACKTOP = sp;return;
}
function _fts3PendingListDelete($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pList;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return;
}
function _fts3DocidRange($pVal,$0,$1) {
 $pVal = $pVal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $4 = sp;
 $eType = sp + 16|0;
 HEAP32[$3>>2] = $pVal;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3_value_numeric_type($11)|0);
  HEAP32[$eType>>2] = $12;
  $13 = HEAP32[$eType>>2]|0;
  $14 = ($13|0)==(1);
  if ($14) {
   $15 = HEAP32[$3>>2]|0;
   $16 = (_sqlite3_value_int64($15)|0);
   $17 = tempRet0;
   $18 = $2;
   $19 = $18;
   HEAP32[$19>>2] = $16;
   $20 = (($18) + 4)|0;
   $21 = $20;
   HEAP32[$21>>2] = $17;
   $32 = $2;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = (($32) + 4)|0;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   tempRet0 = ($37);
   STACKTOP = sp;return ($34|0);
  }
 }
 $22 = $4;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = $2;
 $29 = $28;
 HEAP32[$29>>2] = $24;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 $32 = $2;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 tempRet0 = ($37);
 STACKTOP = sp;return ($34|0);
}
function _sqlite3Fts3ExprParse($pTokenizer,$iLangid,$azCol,$bFts4,$nCol,$iDefaultCol,$z,$n,$ppExpr,$pzErr) {
 $pTokenizer = $pTokenizer|0;
 $iLangid = $iLangid|0;
 $azCol = $azCol|0;
 $bFts4 = $bFts4|0;
 $nCol = $nCol|0;
 $iDefaultCol = $iDefaultCol|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $9 = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$0>>2] = $pTokenizer;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $azCol;
 HEAP32[$3>>2] = $bFts4;
 HEAP32[$4>>2] = $nCol;
 HEAP32[$5>>2] = $iDefaultCol;
 HEAP32[$6>>2] = $z;
 HEAP32[$7>>2] = $n;
 HEAP32[$8>>2] = $ppExpr;
 HEAP32[$9>>2] = $pzErr;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 $17 = HEAP32[$7>>2]|0;
 $18 = HEAP32[$8>>2]|0;
 $19 = (_fts3ExprParseUnbalanced($10,$11,$12,$13,$14,$15,$16,$17,$18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$8>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$8>>2]|0;
   $26 = (_fts3ExprBalance($25,12)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$rc>>2]|0;
   $28 = ($27|0)==(0);
   if ($28) {
    $29 = HEAP32[$8>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = (_fts3ExprCheckDepth($30,12)|0);
    HEAP32[$rc>>2] = $31;
   }
  }
 }
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  $44 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $34 = HEAP32[$8>>2]|0;
 $35 = HEAP32[$34>>2]|0;
 _sqlite3Fts3ExprFree($35);
 $36 = HEAP32[$8>>2]|0;
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(18);
 if ($38) {
  $39 = HEAP32[$9>>2]|0;
  HEAP32[$vararg_buffer>>2] = 12;
  _sqlite3Fts3ErrMsg($39,44341,$vararg_buffer);
  HEAP32[$rc>>2] = 1;
  $44 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(1);
 if (!($41)) {
  $44 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $42 = HEAP32[$9>>2]|0;
 $43 = HEAP32[$6>>2]|0;
 HEAP32[$vararg_buffer1>>2] = $43;
 _sqlite3Fts3ErrMsg($42,44393,$vararg_buffer1);
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _fts3EvalStart($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $aTC = 0, $apOr = 0, $ii = 0, $nOr = 0, $nToken = 0, $or$cond = 0, $pTC = 0;
 var $pTab = 0, $ppOr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $pTab = sp + 32|0;
 $rc = sp + 28|0;
 $nToken = sp + 24|0;
 $nOr = sp + 20|0;
 $aTC = sp + 16|0;
 $apOr = sp + 12|0;
 $ii = sp + 8|0;
 $pTC = sp + 4|0;
 $ppOr = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pTab>>2] = $2;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nToken>>2] = 0;
 HEAP32[$nOr>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 _fts3EvalAllocateReaders($3,$6,$nToken,$nOr,$rc);
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 $9 = HEAP32[$nToken>>2]|0;
 $10 = ($9|0)>(1);
 $or$cond = $8 & $10;
 do {
  if ($or$cond) {
   $11 = HEAP32[$pTab>>2]|0;
   $12 = ((($11)) + 228|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = HEAP32[$nToken>>2]|0;
    $17 = ($16*24)|0;
    $18 = HEAP32[$nOr>>2]|0;
    $19 = $18<<2;
    $20 = $19<<1;
    $21 = (($17) + ($20))|0;
    $22 = (_sqlite3_malloc($21)|0);
    HEAP32[$aTC>>2] = $22;
    $23 = HEAP32[$nToken>>2]|0;
    $24 = HEAP32[$aTC>>2]|0;
    $25 = (($24) + (($23*24)|0)|0);
    HEAP32[$apOr>>2] = $25;
    $26 = HEAP32[$aTC>>2]|0;
    $27 = ($26|0)!=(0|0);
    if (!($27)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    $28 = HEAP32[$aTC>>2]|0;
    HEAP32[$pTC>>2] = $28;
    $29 = HEAP32[$apOr>>2]|0;
    HEAP32[$ppOr>>2] = $29;
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    _fts3EvalTokenCosts($30,0,$33,$pTC,$ppOr,$rc);
    $34 = HEAP32[$pTC>>2]|0;
    $35 = HEAP32[$aTC>>2]|0;
    $36 = $34;
    $37 = $35;
    $38 = (($36) - ($37))|0;
    $39 = (($38|0) / 24)&-1;
    HEAP32[$nToken>>2] = $39;
    $40 = HEAP32[$ppOr>>2]|0;
    $41 = HEAP32[$apOr>>2]|0;
    $42 = $40;
    $43 = $41;
    $44 = (($42) - ($43))|0;
    $45 = (($44|0) / 4)&-1;
    HEAP32[$nOr>>2] = $45;
    $46 = HEAP32[$rc>>2]|0;
    $47 = ($46|0)==(0);
    L7: do {
     if ($47) {
      $48 = HEAP32[$0>>2]|0;
      $49 = HEAP32[$aTC>>2]|0;
      $50 = HEAP32[$nToken>>2]|0;
      $51 = (_fts3EvalSelectDeferred($48,0,$49,$50)|0);
      HEAP32[$rc>>2] = $51;
      HEAP32[$ii>>2] = 0;
      while(1) {
       $52 = HEAP32[$rc>>2]|0;
       $53 = ($52|0)==(0);
       if (!($53)) {
        break L7;
       }
       $54 = HEAP32[$ii>>2]|0;
       $55 = HEAP32[$nOr>>2]|0;
       $56 = ($54|0)<($55|0);
       if (!($56)) {
        break L7;
       }
       $57 = HEAP32[$0>>2]|0;
       $58 = HEAP32[$ii>>2]|0;
       $59 = HEAP32[$apOr>>2]|0;
       $60 = (($59) + ($58<<2)|0);
       $61 = HEAP32[$60>>2]|0;
       $62 = HEAP32[$aTC>>2]|0;
       $63 = HEAP32[$nToken>>2]|0;
       $64 = (_fts3EvalSelectDeferred($57,$61,$62,$63)|0);
       HEAP32[$rc>>2] = $64;
       $65 = HEAP32[$ii>>2]|0;
       $66 = (($65) + 1)|0;
       HEAP32[$ii>>2] = $66;
      }
     }
    } while(0);
    $67 = HEAP32[$aTC>>2]|0;
    _sqlite3_free($67);
   }
  }
 } while(0);
 $68 = HEAP32[$0>>2]|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 12|0);
 $71 = HEAP32[$70>>2]|0;
 _fts3EvalStartReaders($68,$71,$rc);
 $72 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _fts3CursorSeekStmt($pCsr,$ppStmt) {
 $pCsr = $pCsr|0;
 $ppStmt = $ppStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 $zSql = sp + 4|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $ppStmt;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = HEAP32[$7>>2]|0;
   HEAP32[$p>>2] = $8;
   $9 = HEAP32[$p>>2]|0;
   $10 = ((($9)) + 216|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer>>2] = $11;
   $12 = (_sqlite3_mprintf(44442,$vararg_buffer)|0);
   HEAP32[$zSql>>2] = $12;
   $13 = HEAP32[$zSql>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $15 = HEAP32[$p>>2]|0;
    $16 = ((($15)) + 12|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$zSql>>2]|0;
    $19 = HEAP32[$1>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = (_sqlite3_prepare_v2($17,$18,-1,$20,0)|0);
    HEAP32[$rc>>2] = $21;
    $22 = HEAP32[$zSql>>2]|0;
    _sqlite3_free($22);
    break;
   }
   HEAP32[$0>>2] = 7;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 } while(0);
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$2>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fts3ExprParseUnbalanced($pTokenizer,$iLangid,$azCol,$bFts4,$nCol,$iDefaultCol,$z,$n,$ppExpr) {
 $pTokenizer = $pTokenizer|0;
 $iLangid = $iLangid|0;
 $azCol = $azCol|0;
 $bFts4 = $bFts4|0;
 $nCol = $nCol|0;
 $iDefaultCol = $iDefaultCol|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nParsed = 0;
 var $rc = 0, $sParse = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $6 = sp + 56|0;
 $7 = sp + 52|0;
 $8 = sp + 48|0;
 $9 = sp + 44|0;
 $nParsed = sp + 40|0;
 $rc = sp + 36|0;
 $sParse = sp;
 HEAP32[$1>>2] = $pTokenizer;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $azCol;
 HEAP32[$4>>2] = $bFts4;
 HEAP32[$5>>2] = $nCol;
 HEAP32[$6>>2] = $iDefaultCol;
 HEAP32[$7>>2] = $z;
 HEAP32[$8>>2] = $n;
 HEAP32[$9>>2] = $ppExpr;
 dest=$sParse; stop=dest+36|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$sParse>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($sParse)) + 4|0);
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = ((($sParse)) + 8|0);
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$5>>2]|0;
 $16 = ((($sParse)) + 16|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$6>>2]|0;
 $18 = ((($sParse)) + 20|0);
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$4>>2]|0;
 $20 = ((($sParse)) + 12|0);
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$7>>2]|0;
 $22 = ($21|0)==(0|0);
 if ($22) {
  $23 = HEAP32[$9>>2]|0;
  HEAP32[$23>>2] = 0;
  HEAP32[$0>>2] = 0;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $24 = HEAP32[$8>>2]|0;
 $25 = ($24|0)<(0);
 if ($25) {
  $26 = HEAP32[$7>>2]|0;
  $27 = (_strlen($26)|0);
  HEAP32[$8>>2] = $27;
 }
 $28 = HEAP32[$7>>2]|0;
 $29 = HEAP32[$8>>2]|0;
 $30 = HEAP32[$9>>2]|0;
 $31 = (_fts3ExprParse($sParse,$28,$29,$30,$nParsed)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)==(0);
 if ($33) {
  $34 = ((($sParse)) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   HEAP32[$rc>>2] = 1;
  }
 }
 $37 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3ExprBalance($pp,$nMaxDepth) {
 $pp = $pp|0;
 $nMaxDepth = $nMaxDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apLeaf = 0, $eType = 0, $i = 0, $iLvl = 0, $or$cond = 0;
 var $p = 0, $pDel = 0, $pFree = 0, $pLeft = 0, $pParent = 0, $pRight = 0, $pRoot = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $rc = sp + 44|0;
 $pRoot = sp + 40|0;
 $pFree = sp + 36|0;
 $eType = sp + 32|0;
 $apLeaf = sp + 28|0;
 $i = sp + 24|0;
 $p = sp + 20|0;
 $iLvl = sp + 16|0;
 $pParent = sp + 12|0;
 $pDel = sp + 8|0;
 $pLeft = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $nMaxDepth;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pRoot>>2] = $3;
 HEAP32[$pFree>>2] = 0;
 $4 = HEAP32[$pRoot>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$eType>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$rc>>2] = 1;
 }
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 do {
  if ($9) {
   $10 = HEAP32[$eType>>2]|0;
   $11 = ($10|0)==(3);
   $12 = HEAP32[$eType>>2]|0;
   $13 = ($12|0)==(4);
   $or$cond = $11 | $13;
   if (!($or$cond)) {
    $189 = HEAP32[$eType>>2]|0;
    $190 = ($189|0)==(2);
    if (!($190)) {
     break;
    }
    $191 = HEAP32[$pRoot>>2]|0;
    $192 = ((($191)) + 12|0);
    $193 = HEAP32[$192>>2]|0;
    HEAP32[$pLeft>>2] = $193;
    $194 = HEAP32[$pRoot>>2]|0;
    $195 = ((($194)) + 16|0);
    $196 = HEAP32[$195>>2]|0;
    HEAP32[$pRight>>2] = $196;
    $197 = HEAP32[$pRoot>>2]|0;
    $198 = ((($197)) + 12|0);
    HEAP32[$198>>2] = 0;
    $199 = HEAP32[$pRoot>>2]|0;
    $200 = ((($199)) + 16|0);
    HEAP32[$200>>2] = 0;
    $201 = HEAP32[$pLeft>>2]|0;
    $202 = ((($201)) + 8|0);
    HEAP32[$202>>2] = 0;
    $203 = HEAP32[$pRight>>2]|0;
    $204 = ((($203)) + 8|0);
    HEAP32[$204>>2] = 0;
    $205 = HEAP32[$1>>2]|0;
    $206 = (($205) - 1)|0;
    $207 = (_fts3ExprBalance($pLeft,$206)|0);
    HEAP32[$rc>>2] = $207;
    $208 = HEAP32[$rc>>2]|0;
    $209 = ($208|0)==(0);
    if ($209) {
     $210 = HEAP32[$1>>2]|0;
     $211 = (($210) - 1)|0;
     $212 = (_fts3ExprBalance($pRight,$211)|0);
     HEAP32[$rc>>2] = $212;
    }
    $213 = HEAP32[$rc>>2]|0;
    $214 = ($213|0)!=(0);
    if ($214) {
     $215 = HEAP32[$pRight>>2]|0;
     _sqlite3Fts3ExprFree($215);
     $216 = HEAP32[$pLeft>>2]|0;
     _sqlite3Fts3ExprFree($216);
     break;
    } else {
     $217 = HEAP32[$pLeft>>2]|0;
     $218 = HEAP32[$pRoot>>2]|0;
     $219 = ((($218)) + 12|0);
     HEAP32[$219>>2] = $217;
     $220 = HEAP32[$pRoot>>2]|0;
     $221 = HEAP32[$pLeft>>2]|0;
     $222 = ((($221)) + 8|0);
     HEAP32[$222>>2] = $220;
     $223 = HEAP32[$pRight>>2]|0;
     $224 = HEAP32[$pRoot>>2]|0;
     $225 = ((($224)) + 16|0);
     HEAP32[$225>>2] = $223;
     $226 = HEAP32[$pRoot>>2]|0;
     $227 = HEAP32[$pRight>>2]|0;
     $228 = ((($227)) + 8|0);
     HEAP32[$228>>2] = $226;
     break;
    }
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = $14<<2;
   $16 = (_sqlite3_malloc($15)|0);
   HEAP32[$apLeaf>>2] = $16;
   $17 = HEAP32[$apLeaf>>2]|0;
   $18 = (0|0)==($17|0);
   if ($18) {
    HEAP32[$rc>>2] = 7;
   } else {
    $19 = HEAP32[$apLeaf>>2]|0;
    $20 = HEAP32[$1>>2]|0;
    $21 = $20<<2;
    _memset(($19|0),0,($21|0))|0;
   }
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$pRoot>>2]|0;
    HEAP32[$p>>2] = $24;
    while(1) {
     $25 = HEAP32[$p>>2]|0;
     $26 = HEAP32[$25>>2]|0;
     $27 = HEAP32[$eType>>2]|0;
     $28 = ($26|0)==($27|0);
     if (!($28)) {
      break;
     }
     $29 = HEAP32[$p>>2]|0;
     $30 = ((($29)) + 12|0);
     $31 = HEAP32[$30>>2]|0;
     HEAP32[$p>>2] = $31;
    }
    while(1) {
     $32 = HEAP32[$p>>2]|0;
     $33 = ((($32)) + 8|0);
     $34 = HEAP32[$33>>2]|0;
     HEAP32[$pParent>>2] = $34;
     $35 = HEAP32[$p>>2]|0;
     $36 = ((($35)) + 8|0);
     HEAP32[$36>>2] = 0;
     $37 = HEAP32[$pParent>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$pParent>>2]|0;
      $40 = ((($39)) + 12|0);
      HEAP32[$40>>2] = 0;
     } else {
      HEAP32[$pRoot>>2] = 0;
     }
     $41 = HEAP32[$1>>2]|0;
     $42 = (($41) - 1)|0;
     $43 = (_fts3ExprBalance($p,$42)|0);
     HEAP32[$rc>>2] = $43;
     $44 = HEAP32[$rc>>2]|0;
     $45 = ($44|0)!=(0);
     if ($45) {
      break;
     }
     HEAP32[$iLvl>>2] = 0;
     while(1) {
      $46 = HEAP32[$p>>2]|0;
      $47 = ($46|0)!=(0|0);
      if (!($47)) {
       break;
      }
      $48 = HEAP32[$iLvl>>2]|0;
      $49 = HEAP32[$1>>2]|0;
      $50 = ($48|0)<($49|0);
      if (!($50)) {
       break;
      }
      $51 = HEAP32[$iLvl>>2]|0;
      $52 = HEAP32[$apLeaf>>2]|0;
      $53 = (($52) + ($51<<2)|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($54|0)==(0|0);
      if ($55) {
       $56 = HEAP32[$p>>2]|0;
       $57 = HEAP32[$iLvl>>2]|0;
       $58 = HEAP32[$apLeaf>>2]|0;
       $59 = (($58) + ($57<<2)|0);
       HEAP32[$59>>2] = $56;
       HEAP32[$p>>2] = 0;
      } else {
       $60 = HEAP32[$iLvl>>2]|0;
       $61 = HEAP32[$apLeaf>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       $63 = HEAP32[$62>>2]|0;
       $64 = HEAP32[$pFree>>2]|0;
       $65 = ((($64)) + 12|0);
       HEAP32[$65>>2] = $63;
       $66 = HEAP32[$p>>2]|0;
       $67 = HEAP32[$pFree>>2]|0;
       $68 = ((($67)) + 16|0);
       HEAP32[$68>>2] = $66;
       $69 = HEAP32[$pFree>>2]|0;
       $70 = HEAP32[$pFree>>2]|0;
       $71 = ((($70)) + 12|0);
       $72 = HEAP32[$71>>2]|0;
       $73 = ((($72)) + 8|0);
       HEAP32[$73>>2] = $69;
       $74 = HEAP32[$pFree>>2]|0;
       $75 = HEAP32[$pFree>>2]|0;
       $76 = ((($75)) + 16|0);
       $77 = HEAP32[$76>>2]|0;
       $78 = ((($77)) + 8|0);
       HEAP32[$78>>2] = $74;
       $79 = HEAP32[$pFree>>2]|0;
       HEAP32[$p>>2] = $79;
       $80 = HEAP32[$pFree>>2]|0;
       $81 = ((($80)) + 8|0);
       $82 = HEAP32[$81>>2]|0;
       HEAP32[$pFree>>2] = $82;
       $83 = HEAP32[$p>>2]|0;
       $84 = ((($83)) + 8|0);
       HEAP32[$84>>2] = 0;
       $85 = HEAP32[$iLvl>>2]|0;
       $86 = HEAP32[$apLeaf>>2]|0;
       $87 = (($86) + ($85<<2)|0);
       HEAP32[$87>>2] = 0;
      }
      $88 = HEAP32[$iLvl>>2]|0;
      $89 = (($88) + 1)|0;
      HEAP32[$iLvl>>2] = $89;
     }
     $90 = HEAP32[$p>>2]|0;
     $91 = ($90|0)!=(0|0);
     if ($91) {
      label = 24;
      break;
     }
     $93 = HEAP32[$pParent>>2]|0;
     $94 = ($93|0)==(0|0);
     if ($94) {
      break;
     }
     $95 = HEAP32[$pParent>>2]|0;
     $96 = ((($95)) + 16|0);
     $97 = HEAP32[$96>>2]|0;
     HEAP32[$p>>2] = $97;
     while(1) {
      $98 = HEAP32[$p>>2]|0;
      $99 = HEAP32[$98>>2]|0;
      $100 = HEAP32[$eType>>2]|0;
      $101 = ($99|0)==($100|0);
      if (!($101)) {
       break;
      }
      $102 = HEAP32[$p>>2]|0;
      $103 = ((($102)) + 12|0);
      $104 = HEAP32[$103>>2]|0;
      HEAP32[$p>>2] = $104;
     }
     $105 = HEAP32[$pParent>>2]|0;
     $106 = ((($105)) + 8|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = HEAP32[$pParent>>2]|0;
     $109 = ((($108)) + 16|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ((($110)) + 8|0);
     HEAP32[$111>>2] = $107;
     $112 = HEAP32[$pParent>>2]|0;
     $113 = ((($112)) + 8|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = ($114|0)!=(0|0);
     $116 = HEAP32[$pParent>>2]|0;
     $117 = ((($116)) + 16|0);
     $118 = HEAP32[$117>>2]|0;
     if ($115) {
      $119 = HEAP32[$pParent>>2]|0;
      $120 = ((($119)) + 8|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ((($121)) + 12|0);
      HEAP32[$122>>2] = $118;
     } else {
      HEAP32[$pRoot>>2] = $118;
     }
     $123 = HEAP32[$pFree>>2]|0;
     $124 = HEAP32[$pParent>>2]|0;
     $125 = ((($124)) + 8|0);
     HEAP32[$125>>2] = $123;
     $126 = HEAP32[$pParent>>2]|0;
     HEAP32[$pFree>>2] = $126;
    }
    if ((label|0) == 24) {
     $92 = HEAP32[$p>>2]|0;
     _sqlite3Fts3ExprFree($92);
     HEAP32[$rc>>2] = 18;
    }
    $127 = HEAP32[$rc>>2]|0;
    $128 = ($127|0)==(0);
    L53: do {
     if ($128) {
      HEAP32[$p>>2] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $129 = HEAP32[$i>>2]|0;
       $130 = HEAP32[$1>>2]|0;
       $131 = ($129|0)<($130|0);
       if (!($131)) {
        break;
       }
       $132 = HEAP32[$i>>2]|0;
       $133 = HEAP32[$apLeaf>>2]|0;
       $134 = (($133) + ($132<<2)|0);
       $135 = HEAP32[$134>>2]|0;
       $136 = ($135|0)!=(0|0);
       do {
        if ($136) {
         $137 = HEAP32[$p>>2]|0;
         $138 = ($137|0)==(0|0);
         if ($138) {
          $139 = HEAP32[$i>>2]|0;
          $140 = HEAP32[$apLeaf>>2]|0;
          $141 = (($140) + ($139<<2)|0);
          $142 = HEAP32[$141>>2]|0;
          HEAP32[$p>>2] = $142;
          $143 = HEAP32[$p>>2]|0;
          $144 = ((($143)) + 8|0);
          HEAP32[$144>>2] = 0;
          break;
         } else {
          $145 = HEAP32[$p>>2]|0;
          $146 = HEAP32[$pFree>>2]|0;
          $147 = ((($146)) + 16|0);
          HEAP32[$147>>2] = $145;
          $148 = HEAP32[$i>>2]|0;
          $149 = HEAP32[$apLeaf>>2]|0;
          $150 = (($149) + ($148<<2)|0);
          $151 = HEAP32[$150>>2]|0;
          $152 = HEAP32[$pFree>>2]|0;
          $153 = ((($152)) + 12|0);
          HEAP32[$153>>2] = $151;
          $154 = HEAP32[$pFree>>2]|0;
          $155 = HEAP32[$pFree>>2]|0;
          $156 = ((($155)) + 12|0);
          $157 = HEAP32[$156>>2]|0;
          $158 = ((($157)) + 8|0);
          HEAP32[$158>>2] = $154;
          $159 = HEAP32[$pFree>>2]|0;
          $160 = HEAP32[$pFree>>2]|0;
          $161 = ((($160)) + 16|0);
          $162 = HEAP32[$161>>2]|0;
          $163 = ((($162)) + 8|0);
          HEAP32[$163>>2] = $159;
          $164 = HEAP32[$pFree>>2]|0;
          HEAP32[$p>>2] = $164;
          $165 = HEAP32[$pFree>>2]|0;
          $166 = ((($165)) + 8|0);
          $167 = HEAP32[$166>>2]|0;
          HEAP32[$pFree>>2] = $167;
          $168 = HEAP32[$p>>2]|0;
          $169 = ((($168)) + 8|0);
          HEAP32[$169>>2] = 0;
          break;
         }
        }
       } while(0);
       $170 = HEAP32[$i>>2]|0;
       $171 = (($170) + 1)|0;
       HEAP32[$i>>2] = $171;
      }
      $172 = HEAP32[$p>>2]|0;
      HEAP32[$pRoot>>2] = $172;
     } else {
      HEAP32[$i>>2] = 0;
      while(1) {
       $173 = HEAP32[$i>>2]|0;
       $174 = HEAP32[$1>>2]|0;
       $175 = ($173|0)<($174|0);
       if (!($175)) {
        break;
       }
       $176 = HEAP32[$i>>2]|0;
       $177 = HEAP32[$apLeaf>>2]|0;
       $178 = (($177) + ($176<<2)|0);
       $179 = HEAP32[$178>>2]|0;
       _sqlite3Fts3ExprFree($179);
       $180 = HEAP32[$i>>2]|0;
       $181 = (($180) + 1)|0;
       HEAP32[$i>>2] = $181;
      }
      while(1) {
       $182 = HEAP32[$pFree>>2]|0;
       HEAP32[$pDel>>2] = $182;
       $183 = ($182|0)!=(0|0);
       if (!($183)) {
        break L53;
       }
       $184 = HEAP32[$pDel>>2]|0;
       $185 = ((($184)) + 8|0);
       $186 = HEAP32[$185>>2]|0;
       HEAP32[$pFree>>2] = $186;
       $187 = HEAP32[$pDel>>2]|0;
       _sqlite3_free($187);
      }
     }
    } while(0);
    $188 = HEAP32[$apLeaf>>2]|0;
    _sqlite3_free($188);
   }
  }
 } while(0);
 $229 = HEAP32[$rc>>2]|0;
 $230 = ($229|0)!=(0);
 if (!($230)) {
  $232 = HEAP32[$pRoot>>2]|0;
  $233 = HEAP32[$0>>2]|0;
  HEAP32[$233>>2] = $232;
  $234 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($234|0);
 }
 $231 = HEAP32[$pRoot>>2]|0;
 _sqlite3Fts3ExprFree($231);
 HEAP32[$pRoot>>2] = 0;
 $232 = HEAP32[$pRoot>>2]|0;
 $233 = HEAP32[$0>>2]|0;
 HEAP32[$233>>2] = $232;
 $234 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($234|0);
}
function _fts3ExprCheckDepth($p,$nMaxDepth) {
 $p = $p|0;
 $nMaxDepth = $nMaxDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nMaxDepth;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = ($4|0)<(0);
   if ($5) {
    HEAP32[$rc>>2] = 18;
    break;
   }
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 12|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = HEAP32[$1>>2]|0;
   $10 = (($9) - 1)|0;
   $11 = (_fts3ExprCheckDepth($8,$10)|0);
   HEAP32[$rc>>2] = $11;
   $12 = HEAP32[$rc>>2]|0;
   $13 = ($12|0)==(0);
   if ($13) {
    $14 = HEAP32[$0>>2]|0;
    $15 = ((($14)) + 16|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = HEAP32[$1>>2]|0;
    $18 = (($17) - 1)|0;
    $19 = (_fts3ExprCheckDepth($16,$18)|0);
    HEAP32[$rc>>2] = $19;
   }
  }
 } while(0);
 $20 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _fts3ExprParse($pParse,$z,$n,$ppExpr,$pnConsumed) {
 $pParse = $pParse|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pnConsumed = $pnConsumed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $eType = 0, $isPhrase = 0, $isRequirePhrase = 0, $nByte = 0, $nIn = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $p = 0, $pAnd = 0, $pNotBranch = 0, $pPrev = 0, $pRet = 0;
 var $rc = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $pRet = sp + 44|0;
 $pPrev = sp + 40|0;
 $pNotBranch = sp + 36|0;
 $nIn = sp + 32|0;
 $zIn = sp + 28|0;
 $rc = sp + 24|0;
 $isRequirePhrase = sp + 20|0;
 $p = sp + 16|0;
 $nByte = sp + 12|0;
 $isPhrase = sp + 8|0;
 $eType = sp + 4|0;
 $pAnd = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $ppExpr;
 HEAP32[$4>>2] = $pnConsumed;
 HEAP32[$pRet>>2] = 0;
 HEAP32[$pPrev>>2] = 0;
 HEAP32[$pNotBranch>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$nIn>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$zIn>>2] = $6;
 HEAP32[$rc>>2] = 0;
 HEAP32[$isRequirePhrase>>2] = 1;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 25;
   break;
  }
  HEAP32[$p>>2] = 0;
  HEAP32[$nByte>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$zIn>>2]|0;
  $11 = HEAP32[$nIn>>2]|0;
  $12 = (_getNextNode($9,$10,$11,$p,$nByte)|0);
  HEAP32[$rc>>2] = $12;
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(0);
  $15 = HEAP32[$p>>2]|0;
  $16 = ($15|0)!=(0|0);
  $or$cond13 = $14 & $16;
  if ($or$cond13) {
   $17 = HEAP32[$p>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$eType>>2] = $18;
   $19 = HEAP32[$eType>>2]|0;
   $20 = ($19|0)==(5);
   if ($20) {
    $26 = 1;
   } else {
    $21 = HEAP32[$p>>2]|0;
    $22 = ((($21)) + 12|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($23|0)!=(0|0);
    $26 = $24;
   }
   $25 = $26&1;
   HEAP32[$isPhrase>>2] = $25;
   $27 = HEAP32[$isPhrase>>2]|0;
   $28 = ($27|0)==(0);
   $29 = HEAP32[$isRequirePhrase>>2]|0;
   $30 = ($29|0)!=(0);
   $or$cond = $28 & $30;
   if ($or$cond) {
    label = 7;
    break;
   }
   $32 = HEAP32[$isPhrase>>2]|0;
   $33 = ($32|0)==(0);
   $34 = HEAP32[$isRequirePhrase>>2]|0;
   $35 = ($34|0)!=(0);
   $or$cond3 = $33 | $35;
   if (!($or$cond3)) {
    $36 = (_fts3MallocZero(48)|0);
    HEAP32[$pAnd>>2] = $36;
    $37 = HEAP32[$pAnd>>2]|0;
    $38 = ($37|0)!=(0|0);
    if (!($38)) {
     label = 10;
     break;
    }
    $40 = HEAP32[$pAnd>>2]|0;
    HEAP32[$40>>2] = 3;
    $41 = HEAP32[$pPrev>>2]|0;
    $42 = HEAP32[$pAnd>>2]|0;
    _insertBinaryOperator($pRet,$41,$42);
    $43 = HEAP32[$pAnd>>2]|0;
    HEAP32[$pPrev>>2] = $43;
   }
   $44 = HEAP32[$pPrev>>2]|0;
   $45 = ($44|0)!=(0|0);
   if ($45) {
    $46 = HEAP32[$eType>>2]|0;
    $47 = ($46|0)!=(1);
    $48 = HEAP32[$isPhrase>>2]|0;
    $49 = ($48|0)!=(0);
    $or$cond5 = $47 | $49;
    if (!($or$cond5)) {
     $50 = HEAP32[$pPrev>>2]|0;
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)!=(5);
     if ($52) {
      label = 17;
      break;
     }
    }
    $53 = HEAP32[$eType>>2]|0;
    $54 = ($53|0)!=(5);
    $55 = HEAP32[$isPhrase>>2]|0;
    $56 = ($55|0)!=(0);
    $or$cond7 = $54 & $56;
    if ($or$cond7) {
     $57 = HEAP32[$pPrev>>2]|0;
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(1);
     if ($59) {
      label = 17;
      break;
     }
    }
   }
   $61 = HEAP32[$isPhrase>>2]|0;
   $62 = ($61|0)!=(0);
   do {
    if ($62) {
     $63 = HEAP32[$pRet>>2]|0;
     $64 = ($63|0)!=(0|0);
     $65 = HEAP32[$p>>2]|0;
     if ($64) {
      $66 = HEAP32[$pPrev>>2]|0;
      $67 = ((($66)) + 16|0);
      HEAP32[$67>>2] = $65;
      $68 = HEAP32[$pPrev>>2]|0;
      $69 = HEAP32[$p>>2]|0;
      $70 = ((($69)) + 8|0);
      HEAP32[$70>>2] = $68;
      break;
     } else {
      HEAP32[$pRet>>2] = $65;
      break;
     }
    } else {
     $71 = HEAP32[$pPrev>>2]|0;
     $72 = HEAP32[$p>>2]|0;
     _insertBinaryOperator($pRet,$71,$72);
    }
   } while(0);
   $73 = HEAP32[$isPhrase>>2]|0;
   $74 = ($73|0)!=(0);
   $75 = $74 ^ 1;
   $76 = $75&1;
   HEAP32[$isRequirePhrase>>2] = $76;
   $77 = HEAP32[$p>>2]|0;
   HEAP32[$pPrev>>2] = $77;
  }
  $78 = HEAP32[$nByte>>2]|0;
  $79 = HEAP32[$nIn>>2]|0;
  $80 = (($79) - ($78))|0;
  HEAP32[$nIn>>2] = $80;
  $81 = HEAP32[$nByte>>2]|0;
  $82 = HEAP32[$zIn>>2]|0;
  $83 = (($82) + ($81)|0);
  HEAP32[$zIn>>2] = $83;
 }
 if ((label|0) == 7) {
  $31 = HEAP32[$p>>2]|0;
  _sqlite3Fts3ExprFree($31);
  HEAP32[$rc>>2] = 1;
 }
 else if ((label|0) == 10) {
  $39 = HEAP32[$p>>2]|0;
  _sqlite3Fts3ExprFree($39);
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 17) {
  $60 = HEAP32[$p>>2]|0;
  _sqlite3Fts3ExprFree($60);
  HEAP32[$rc>>2] = 1;
 }
 else if ((label|0) == 25) {
  $84 = HEAP32[$rc>>2]|0;
  $85 = ($84|0)==(101);
  $86 = HEAP32[$pRet>>2]|0;
  $87 = ($86|0)!=(0|0);
  $or$cond9 = $85 & $87;
  $88 = HEAP32[$isRequirePhrase>>2]|0;
  $89 = ($88|0)!=(0);
  $or$cond11 = $or$cond9 & $89;
  if ($or$cond11) {
   HEAP32[$rc>>2] = 1;
  }
  $90 = HEAP32[$rc>>2]|0;
  $91 = ($90|0)==(101);
  if ($91) {
   HEAP32[$rc>>2] = 0;
  }
  $92 = HEAP32[$2>>2]|0;
  $93 = HEAP32[$nIn>>2]|0;
  $94 = (($92) - ($93))|0;
  $95 = HEAP32[$4>>2]|0;
  HEAP32[$95>>2] = $94;
 }
 $96 = HEAP32[$rc>>2]|0;
 $97 = ($96|0)!=(0);
 if (!($97)) {
  $100 = HEAP32[$pRet>>2]|0;
  $101 = HEAP32[$3>>2]|0;
  HEAP32[$101>>2] = $100;
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $98 = HEAP32[$pRet>>2]|0;
 _sqlite3Fts3ExprFree($98);
 $99 = HEAP32[$pNotBranch>>2]|0;
 _sqlite3Fts3ExprFree($99);
 HEAP32[$pRet>>2] = 0;
 $100 = HEAP32[$pRet>>2]|0;
 $101 = HEAP32[$3>>2]|0;
 HEAP32[$101>>2] = $100;
 $102 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _getNextNode($pParse,$z,$n,$ppExpr,$pnConsumed) {
 $pParse = $pParse|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pnConsumed = $pnConsumed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cNext = 0, $iCol = 0, $iColLen = 0, $ii = 0, $nConsumed = 0, $nInput = 0, $nKey = 0;
 var $nNear = 0, $nStr = 0, $pKey = 0, $pRet = 0, $rc = 0, $zInput = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $5 = sp + 52|0;
 $ii = sp + 48|0;
 $iCol = sp + 44|0;
 $iColLen = sp + 40|0;
 $rc = sp + 36|0;
 $pRet = sp + 32|0;
 $zInput = sp + 28|0;
 $nInput = sp + 24|0;
 $pKey = sp + 20|0;
 $nNear = sp + 16|0;
 $nKey = sp + 12|0;
 $cNext = sp + 76|0;
 $nConsumed = sp + 8|0;
 $zStr = sp + 4|0;
 $nStr = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $ppExpr;
 HEAP32[$5>>2] = $pnConsumed;
 HEAP32[$pRet>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$zInput>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$nInput>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 24|0);
 HEAP32[$9>>2] = 0;
 while(1) {
  $10 = HEAP32[$nInput>>2]|0;
  $11 = ($10|0)>(0);
  if ($11) {
   $12 = HEAP32[$zInput>>2]|0;
   $13 = HEAP8[$12>>0]|0;
   $14 = (_fts3isspace($13)|0);
   $15 = ($14|0)!=(0);
   $272 = $15;
  } else {
   $272 = 0;
  }
  $16 = HEAP32[$nInput>>2]|0;
  if (!($272)) {
   break;
  }
  $17 = (($16) + -1)|0;
  HEAP32[$nInput>>2] = $17;
  $18 = HEAP32[$zInput>>2]|0;
  $19 = ((($18)) + 1|0);
  HEAP32[$zInput>>2] = $19;
 }
 $20 = ($16|0)==(0);
 if ($20) {
  HEAP32[$0>>2] = 101;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $21 = HEAP32[$ii>>2]|0;
  $22 = ($21|0)<(4);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$ii>>2]|0;
  $24 = (9204 + ($23<<3)|0);
  HEAP32[$pKey>>2] = $24;
  $25 = HEAP32[$pKey>>2]|0;
  $26 = ((($25)) + 5|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  $29 = $28 & -2;
  $30 = ($29|0)!=(0);
  if (!($30)) {
   $31 = HEAP32[$nInput>>2]|0;
   $32 = HEAP32[$pKey>>2]|0;
   $33 = ((($32)) + 4|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = $34&255;
   $36 = ($31|0)>=($35|0);
   if ($36) {
    $37 = HEAP32[$zInput>>2]|0;
    $38 = HEAP32[$pKey>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pKey>>2]|0;
    $41 = ((($40)) + 4|0);
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = (_memcmp($37,$39,$43)|0);
    $45 = (0)==($44|0);
    if ($45) {
     HEAP32[$nNear>>2] = 10;
     $46 = HEAP32[$pKey>>2]|0;
     $47 = ((($46)) + 4|0);
     $48 = HEAP8[$47>>0]|0;
     $49 = $48&255;
     HEAP32[$nKey>>2] = $49;
     $50 = HEAP32[$pKey>>2]|0;
     $51 = ((($50)) + 6|0);
     $52 = HEAP8[$51>>0]|0;
     $53 = $52&255;
     $54 = ($53|0)==(1);
     L19: do {
      if ($54) {
       $55 = HEAP32[$zInput>>2]|0;
       $56 = ((($55)) + 4|0);
       $57 = HEAP8[$56>>0]|0;
       $58 = $57 << 24 >> 24;
       $59 = ($58|0)==(47);
       if ($59) {
        $60 = HEAP32[$zInput>>2]|0;
        $61 = ((($60)) + 5|0);
        $62 = HEAP8[$61>>0]|0;
        $63 = $62 << 24 >> 24;
        $64 = ($63|0)>=(48);
        if ($64) {
         $65 = HEAP32[$zInput>>2]|0;
         $66 = ((($65)) + 5|0);
         $67 = HEAP8[$66>>0]|0;
         $68 = $67 << 24 >> 24;
         $69 = ($68|0)<=(57);
         if ($69) {
          HEAP32[$nNear>>2] = 0;
          HEAP32[$nKey>>2] = 5;
          while(1) {
           $70 = HEAP32[$nKey>>2]|0;
           $71 = HEAP32[$zInput>>2]|0;
           $72 = (($71) + ($70)|0);
           $73 = HEAP8[$72>>0]|0;
           $74 = $73 << 24 >> 24;
           $75 = ($74|0)>=(48);
           if (!($75)) {
            break L19;
           }
           $76 = HEAP32[$nKey>>2]|0;
           $77 = HEAP32[$zInput>>2]|0;
           $78 = (($77) + ($76)|0);
           $79 = HEAP8[$78>>0]|0;
           $80 = $79 << 24 >> 24;
           $81 = ($80|0)<=(57);
           if (!($81)) {
            break L19;
           }
           $82 = HEAP32[$nNear>>2]|0;
           $83 = ($82*10)|0;
           $84 = HEAP32[$nKey>>2]|0;
           $85 = HEAP32[$zInput>>2]|0;
           $86 = (($85) + ($84)|0);
           $87 = HEAP8[$86>>0]|0;
           $88 = $87 << 24 >> 24;
           $89 = (($88) - 48)|0;
           $90 = (($83) + ($89))|0;
           HEAP32[$nNear>>2] = $90;
           $91 = HEAP32[$nKey>>2]|0;
           $92 = (($91) + 1)|0;
           HEAP32[$nKey>>2] = $92;
          }
         }
        }
       }
      }
     } while(0);
     $93 = HEAP32[$nKey>>2]|0;
     $94 = HEAP32[$zInput>>2]|0;
     $95 = (($94) + ($93)|0);
     $96 = HEAP8[$95>>0]|0;
     HEAP8[$cNext>>0] = $96;
     $97 = HEAP8[$cNext>>0]|0;
     $98 = (_fts3isspace($97)|0);
     $99 = ($98|0)!=(0);
     if ($99) {
      label = 26;
      break;
     }
     $100 = HEAP8[$cNext>>0]|0;
     $101 = $100 << 24 >> 24;
     $102 = ($101|0)==(34);
     if ($102) {
      label = 26;
      break;
     }
     $103 = HEAP8[$cNext>>0]|0;
     $104 = $103 << 24 >> 24;
     $105 = ($104|0)==(40);
     if ($105) {
      label = 26;
      break;
     }
     $106 = HEAP8[$cNext>>0]|0;
     $107 = $106 << 24 >> 24;
     $108 = ($107|0)==(41);
     if ($108) {
      label = 26;
      break;
     }
     $109 = HEAP8[$cNext>>0]|0;
     $110 = $109 << 24 >> 24;
     $111 = ($110|0)==(0);
     if ($111) {
      label = 26;
      break;
     }
    }
   }
  }
  $133 = HEAP32[$ii>>2]|0;
  $134 = (($133) + 1)|0;
  HEAP32[$ii>>2] = $134;
 }
 if ((label|0) == 26) {
  $112 = (_fts3MallocZero(48)|0);
  HEAP32[$pRet>>2] = $112;
  $113 = HEAP32[$pRet>>2]|0;
  $114 = ($113|0)!=(0|0);
  if ($114) {
   $115 = HEAP32[$pKey>>2]|0;
   $116 = ((($115)) + 6|0);
   $117 = HEAP8[$116>>0]|0;
   $118 = $117&255;
   $119 = HEAP32[$pRet>>2]|0;
   HEAP32[$119>>2] = $118;
   $120 = HEAP32[$nNear>>2]|0;
   $121 = HEAP32[$pRet>>2]|0;
   $122 = ((($121)) + 4|0);
   HEAP32[$122>>2] = $120;
   $123 = HEAP32[$pRet>>2]|0;
   $124 = HEAP32[$4>>2]|0;
   HEAP32[$124>>2] = $123;
   $125 = HEAP32[$zInput>>2]|0;
   $126 = HEAP32[$2>>2]|0;
   $127 = $125;
   $128 = $126;
   $129 = (($127) - ($128))|0;
   $130 = HEAP32[$nKey>>2]|0;
   $131 = (($129) + ($130))|0;
   $132 = HEAP32[$5>>2]|0;
   HEAP32[$132>>2] = $131;
   HEAP32[$0>>2] = 0;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  } else {
   HEAP32[$0>>2] = 7;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  }
 }
 $135 = HEAP32[$zInput>>2]|0;
 $136 = HEAP8[$135>>0]|0;
 $137 = $136 << 24 >> 24;
 $138 = ($137|0)==(34);
 if ($138) {
  HEAP32[$ii>>2] = 1;
  while(1) {
   $139 = HEAP32[$ii>>2]|0;
   $140 = HEAP32[$nInput>>2]|0;
   $141 = ($139|0)<($140|0);
   if (!($141)) {
    break;
   }
   $142 = HEAP32[$ii>>2]|0;
   $143 = HEAP32[$zInput>>2]|0;
   $144 = (($143) + ($142)|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = $145 << 24 >> 24;
   $147 = ($146|0)!=(34);
   if (!($147)) {
    break;
   }
   $148 = HEAP32[$ii>>2]|0;
   $149 = (($148) + 1)|0;
   HEAP32[$ii>>2] = $149;
  }
  $150 = HEAP32[$zInput>>2]|0;
  $151 = HEAP32[$2>>2]|0;
  $152 = $150;
  $153 = $151;
  $154 = (($152) - ($153))|0;
  $155 = HEAP32[$ii>>2]|0;
  $156 = (($154) + ($155))|0;
  $157 = (($156) + 1)|0;
  $158 = HEAP32[$5>>2]|0;
  HEAP32[$158>>2] = $157;
  $159 = HEAP32[$ii>>2]|0;
  $160 = HEAP32[$nInput>>2]|0;
  $161 = ($159|0)==($160|0);
  if ($161) {
   HEAP32[$0>>2] = 1;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  } else {
   $162 = HEAP32[$1>>2]|0;
   $163 = HEAP32[$zInput>>2]|0;
   $164 = ((($163)) + 1|0);
   $165 = HEAP32[$ii>>2]|0;
   $166 = (($165) - 1)|0;
   $167 = HEAP32[$4>>2]|0;
   $168 = (_getNextString($162,$164,$166,$167)|0);
   HEAP32[$0>>2] = $168;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  }
 }
 $169 = HEAP32[$zInput>>2]|0;
 $170 = HEAP8[$169>>0]|0;
 $171 = $170 << 24 >> 24;
 $172 = ($171|0)==(40);
 if ($172) {
  HEAP32[$nConsumed>>2] = 0;
  $173 = HEAP32[$1>>2]|0;
  $174 = ((($173)) + 32|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = (($175) + 1)|0;
  HEAP32[$174>>2] = $176;
  $177 = HEAP32[$1>>2]|0;
  $178 = HEAP32[$zInput>>2]|0;
  $179 = ((($178)) + 1|0);
  $180 = HEAP32[$nInput>>2]|0;
  $181 = (($180) - 1)|0;
  $182 = HEAP32[$4>>2]|0;
  $183 = (_fts3ExprParse($177,$179,$181,$182,$nConsumed)|0);
  HEAP32[$rc>>2] = $183;
  $184 = HEAP32[$rc>>2]|0;
  $185 = ($184|0)==(0);
  if ($185) {
   $186 = HEAP32[$4>>2]|0;
   $187 = HEAP32[$186>>2]|0;
   $188 = ($187|0)!=(0|0);
   if (!($188)) {
    HEAP32[$rc>>2] = 101;
   }
  }
  $189 = HEAP32[$zInput>>2]|0;
  $190 = HEAP32[$2>>2]|0;
  $191 = $189;
  $192 = $190;
  $193 = (($191) - ($192))|0;
  $194 = (($193) + 1)|0;
  $195 = HEAP32[$nConsumed>>2]|0;
  $196 = (($194) + ($195))|0;
  $197 = HEAP32[$5>>2]|0;
  HEAP32[$197>>2] = $196;
  $198 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $198;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 $199 = HEAP32[$zInput>>2]|0;
 $200 = HEAP8[$199>>0]|0;
 $201 = $200 << 24 >> 24;
 $202 = ($201|0)==(41);
 $203 = HEAP32[$1>>2]|0;
 if ($202) {
  $204 = ((($203)) + 32|0);
  $205 = HEAP32[$204>>2]|0;
  $206 = (($205) + -1)|0;
  HEAP32[$204>>2] = $206;
  $207 = HEAP32[$zInput>>2]|0;
  $208 = HEAP32[$2>>2]|0;
  $209 = $207;
  $210 = $208;
  $211 = (($209) - ($210))|0;
  $212 = (($211) + 1)|0;
  $213 = HEAP32[$5>>2]|0;
  HEAP32[$213>>2] = $212;
  $214 = HEAP32[$4>>2]|0;
  HEAP32[$214>>2] = 0;
  HEAP32[$0>>2] = 101;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 $215 = ((($203)) + 20|0);
 $216 = HEAP32[$215>>2]|0;
 HEAP32[$iCol>>2] = $216;
 HEAP32[$iColLen>>2] = 0;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $217 = HEAP32[$ii>>2]|0;
  $218 = HEAP32[$1>>2]|0;
  $219 = ((($218)) + 16|0);
  $220 = HEAP32[$219>>2]|0;
  $221 = ($217|0)<($220|0);
  if (!($221)) {
   break;
  }
  $222 = HEAP32[$ii>>2]|0;
  $223 = HEAP32[$1>>2]|0;
  $224 = ((($223)) + 8|0);
  $225 = HEAP32[$224>>2]|0;
  $226 = (($225) + ($222<<2)|0);
  $227 = HEAP32[$226>>2]|0;
  HEAP32[$zStr>>2] = $227;
  $228 = HEAP32[$zStr>>2]|0;
  $229 = (_strlen($228)|0);
  HEAP32[$nStr>>2] = $229;
  $230 = HEAP32[$nInput>>2]|0;
  $231 = HEAP32[$nStr>>2]|0;
  $232 = ($230|0)>($231|0);
  if ($232) {
   $233 = HEAP32[$nStr>>2]|0;
   $234 = HEAP32[$zInput>>2]|0;
   $235 = (($234) + ($233)|0);
   $236 = HEAP8[$235>>0]|0;
   $237 = $236 << 24 >> 24;
   $238 = ($237|0)==(58);
   if ($238) {
    $239 = HEAP32[$zStr>>2]|0;
    $240 = HEAP32[$zInput>>2]|0;
    $241 = HEAP32[$nStr>>2]|0;
    $242 = (_sqlite3_strnicmp($239,$240,$241)|0);
    $243 = ($242|0)==(0);
    if ($243) {
     label = 50;
     break;
    }
   }
  }
  $253 = HEAP32[$ii>>2]|0;
  $254 = (($253) + 1)|0;
  HEAP32[$ii>>2] = $254;
 }
 if ((label|0) == 50) {
  $244 = HEAP32[$ii>>2]|0;
  HEAP32[$iCol>>2] = $244;
  $245 = HEAP32[$zInput>>2]|0;
  $246 = HEAP32[$2>>2]|0;
  $247 = $245;
  $248 = $246;
  $249 = (($247) - ($248))|0;
  $250 = HEAP32[$nStr>>2]|0;
  $251 = (($249) + ($250))|0;
  $252 = (($251) + 1)|0;
  HEAP32[$iColLen>>2] = $252;
 }
 $255 = HEAP32[$1>>2]|0;
 $256 = HEAP32[$iCol>>2]|0;
 $257 = HEAP32[$iColLen>>2]|0;
 $258 = HEAP32[$2>>2]|0;
 $259 = (($258) + ($257)|0);
 $260 = HEAP32[$3>>2]|0;
 $261 = HEAP32[$iColLen>>2]|0;
 $262 = (($260) - ($261))|0;
 $263 = HEAP32[$4>>2]|0;
 $264 = HEAP32[$5>>2]|0;
 $265 = (_getNextToken($255,$256,$259,$262,$263,$264)|0);
 HEAP32[$rc>>2] = $265;
 $266 = HEAP32[$iColLen>>2]|0;
 $267 = HEAP32[$5>>2]|0;
 $268 = HEAP32[$267>>2]|0;
 $269 = (($268) + ($266))|0;
 HEAP32[$267>>2] = $269;
 $270 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $270;
 $271 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($271|0);
}
function _fts3MallocZero($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $nByte;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_malloc($1)|0);
 HEAP32[$pRet>>2] = $2;
 $3 = HEAP32[$pRet>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $7 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$pRet>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _memset(($5|0),0,($6|0))|0;
 $7 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _insertBinaryOperator($ppHead,$pPrev,$pNew) {
 $ppHead = $ppHead|0;
 $pPrev = $pPrev|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSplit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pSplit = sp;
 HEAP32[$0>>2] = $ppHead;
 HEAP32[$1>>2] = $pPrev;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pSplit>>2] = $3;
 while(1) {
  $4 = HEAP32[$pSplit>>2]|0;
  $5 = ((($4)) + 8|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0|0);
  if ($7) {
   $8 = HEAP32[$pSplit>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_opPrecedence($10)|0);
   $12 = HEAP32[$2>>2]|0;
   $13 = (_opPrecedence($12)|0);
   $14 = ($11|0)<=($13|0);
   $36 = $14;
  } else {
   $36 = 0;
  }
  $15 = HEAP32[$pSplit>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  if (!($36)) {
   break;
  }
  HEAP32[$pSplit>>2] = $17;
 }
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$2>>2]|0;
 if ($18) {
  $20 = HEAP32[$pSplit>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 16|0);
  HEAP32[$23>>2] = $19;
  $24 = HEAP32[$pSplit>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 8|0);
  HEAP32[$28>>2] = $26;
  $30 = HEAP32[$pSplit>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = ((($31)) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$pSplit>>2]|0;
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $33;
  STACKTOP = sp;return;
 } else {
  $29 = HEAP32[$0>>2]|0;
  HEAP32[$29>>2] = $19;
  $30 = HEAP32[$pSplit>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = ((($31)) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$pSplit>>2]|0;
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $33;
  STACKTOP = sp;return;
 }
}
function _fts3isspace($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = ($2|0)==(32);
 if ($3) {
  $20 = 1;
 } else {
  $4 = HEAP8[$0>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($5|0)==(9);
  if ($6) {
   $20 = 1;
  } else {
   $7 = HEAP8[$0>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)==(10);
   if ($9) {
    $20 = 1;
   } else {
    $10 = HEAP8[$0>>0]|0;
    $11 = $10 << 24 >> 24;
    $12 = ($11|0)==(13);
    if ($12) {
     $20 = 1;
    } else {
     $13 = HEAP8[$0>>0]|0;
     $14 = $13 << 24 >> 24;
     $15 = ($14|0)==(11);
     if ($15) {
      $20 = 1;
     } else {
      $16 = HEAP8[$0>>0]|0;
      $17 = $16 << 24 >> 24;
      $18 = ($17|0)==(12);
      $20 = $18;
     }
    }
   }
  }
 }
 $19 = $20&1;
 STACKTOP = sp;return ($19|0);
}
function _getNextString($pParse,$zInput,$nInput,$ppExpr) {
 $pParse = $pParse|0;
 $zInput = $zInput|0;
 $nInput = $nInput|0;
 $ppExpr = $ppExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iBegin = 0;
 var $iEnd = 0, $iPos = 0, $ii = 0, $jj = 0, $nByte = 0, $nSpace = 0, $nTemp = 0, $nToken = 0, $p = 0, $pCursor = 0, $pModule = 0, $pToken = 0, $pTokenizer = 0, $rc = 0, $zBuf = 0, $zByte = 0, $zTemp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $pTokenizer = sp + 68|0;
 $pModule = sp + 64|0;
 $rc = sp + 60|0;
 $p = sp + 56|0;
 $pCursor = sp + 52|0;
 $zTemp = sp + 48|0;
 $nTemp = sp + 44|0;
 $nSpace = sp + 40|0;
 $nToken = sp + 36|0;
 $ii = sp + 32|0;
 $zByte = sp + 28|0;
 $nByte = sp + 24|0;
 $iBegin = sp + 20|0;
 $iEnd = sp + 16|0;
 $iPos = sp + 12|0;
 $pToken = sp + 8|0;
 $jj = sp + 4|0;
 $zBuf = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zInput;
 HEAP32[$3>>2] = $nInput;
 HEAP32[$4>>2] = $ppExpr;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTokenizer>>2] = $6;
 $7 = HEAP32[$pTokenizer>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pModule>>2] = $8;
 HEAP32[$p>>2] = 0;
 HEAP32[$pCursor>>2] = 0;
 HEAP32[$zTemp>>2] = 0;
 HEAP32[$nTemp>>2] = 0;
 HEAP32[$nSpace>>2] = 144;
 HEAP32[$nToken>>2] = 0;
 $9 = HEAP32[$pTokenizer>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = (_sqlite3Fts3OpenTokenizer($9,$12,$13,$14,$pCursor)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 L1: do {
  if ($17) {
   HEAP32[$ii>>2] = 0;
   while(1) {
    $18 = HEAP32[$rc>>2]|0;
    $19 = ($18|0)==(0);
    if (!($19)) {
     break;
    }
    HEAP32[$nByte>>2] = 0;
    HEAP32[$iBegin>>2] = 0;
    HEAP32[$iEnd>>2] = 0;
    HEAP32[$iPos>>2] = 0;
    $20 = HEAP32[$pModule>>2]|0;
    $21 = ((($20)) + 20|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$pCursor>>2]|0;
    $24 = (FUNCTION_TABLE_iiiiiii[$22 & 255]($23,$zByte,$nByte,$iBegin,$iEnd,$iPos)|0);
    HEAP32[$rc>>2] = $24;
    $25 = HEAP32[$rc>>2]|0;
    $26 = ($25|0)==(0);
    if ($26) {
     $27 = HEAP32[$p>>2]|0;
     $28 = HEAP32[$ii>>2]|0;
     $29 = ($28*24)|0;
     $30 = (144 + ($29))|0;
     $31 = (_fts3ReallocOrFree($27,$30)|0);
     HEAP32[$p>>2] = $31;
     $32 = HEAP32[$p>>2]|0;
     $33 = ($32|0)!=(0|0);
     if (!($33)) {
      break L1;
     }
     $34 = HEAP32[$zTemp>>2]|0;
     $35 = HEAP32[$nTemp>>2]|0;
     $36 = HEAP32[$nByte>>2]|0;
     $37 = (($35) + ($36))|0;
     $38 = (_fts3ReallocOrFree($34,$37)|0);
     HEAP32[$zTemp>>2] = $38;
     $39 = HEAP32[$zTemp>>2]|0;
     $40 = ($39|0)!=(0|0);
     if (!($40)) {
      break L1;
     }
     $41 = HEAP32[$ii>>2]|0;
     $42 = HEAP32[$p>>2]|0;
     $43 = ((($42)) + 48|0);
     $44 = ((($43)) + 72|0);
     $45 = (($44) + (($41*24)|0)|0);
     HEAP32[$pToken>>2] = $45;
     $46 = HEAP32[$pToken>>2]|0;
     ;HEAP32[$46>>2]=0|0;HEAP32[$46+4>>2]=0|0;HEAP32[$46+8>>2]=0|0;HEAP32[$46+12>>2]=0|0;HEAP32[$46+16>>2]=0|0;HEAP32[$46+20>>2]=0|0;
     $47 = HEAP32[$nTemp>>2]|0;
     $48 = HEAP32[$zTemp>>2]|0;
     $49 = (($48) + ($47)|0);
     $50 = HEAP32[$zByte>>2]|0;
     $51 = HEAP32[$nByte>>2]|0;
     _memcpy(($49|0),($50|0),($51|0))|0;
     $52 = HEAP32[$nByte>>2]|0;
     $53 = HEAP32[$nTemp>>2]|0;
     $54 = (($53) + ($52))|0;
     HEAP32[$nTemp>>2] = $54;
     $55 = HEAP32[$nByte>>2]|0;
     $56 = HEAP32[$pToken>>2]|0;
     $57 = ((($56)) + 4|0);
     HEAP32[$57>>2] = $55;
     $58 = HEAP32[$iEnd>>2]|0;
     $59 = HEAP32[$3>>2]|0;
     $60 = ($58|0)<($59|0);
     if ($60) {
      $61 = HEAP32[$iEnd>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = (($62) + ($61)|0);
      $64 = HEAP8[$63>>0]|0;
      $65 = $64 << 24 >> 24;
      $66 = ($65|0)==(42);
      $68 = $66;
     } else {
      $68 = 0;
     }
     $67 = $68&1;
     $69 = HEAP32[$pToken>>2]|0;
     $70 = ((($69)) + 8|0);
     HEAP32[$70>>2] = $67;
     $71 = HEAP32[$iBegin>>2]|0;
     $72 = ($71|0)>(0);
     if ($72) {
      $73 = HEAP32[$iBegin>>2]|0;
      $74 = (($73) - 1)|0;
      $75 = HEAP32[$2>>2]|0;
      $76 = (($75) + ($74)|0);
      $77 = HEAP8[$76>>0]|0;
      $78 = $77 << 24 >> 24;
      $79 = ($78|0)==(94);
      $81 = $79;
     } else {
      $81 = 0;
     }
     $80 = $81&1;
     $82 = HEAP32[$pToken>>2]|0;
     $83 = ((($82)) + 12|0);
     HEAP32[$83>>2] = $80;
     $84 = HEAP32[$ii>>2]|0;
     $85 = (($84) + 1)|0;
     HEAP32[$nToken>>2] = $85;
    }
    $86 = HEAP32[$ii>>2]|0;
    $87 = (($86) + 1)|0;
    HEAP32[$ii>>2] = $87;
   }
   $88 = HEAP32[$pModule>>2]|0;
   $89 = ((($88)) + 16|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$pCursor>>2]|0;
   (FUNCTION_TABLE_ii[$90 & 255]($91)|0);
   HEAP32[$pCursor>>2] = 0;
   label = 14;
  } else {
   label = 14;
  }
 } while(0);
 do {
  if ((label|0) == 14) {
   $92 = HEAP32[$rc>>2]|0;
   $93 = ($92|0)==(101);
   if ($93) {
    HEAP32[$zBuf>>2] = 0;
    $94 = HEAP32[$p>>2]|0;
    $95 = HEAP32[$nToken>>2]|0;
    $96 = ($95*24)|0;
    $97 = (144 + ($96))|0;
    $98 = HEAP32[$nTemp>>2]|0;
    $99 = (($97) + ($98))|0;
    $100 = (_fts3ReallocOrFree($94,$99)|0);
    HEAP32[$p>>2] = $100;
    $101 = HEAP32[$p>>2]|0;
    $102 = ($101|0)!=(0|0);
    if (!($102)) {
     break;
    }
    $103 = HEAP32[$p>>2]|0;
    $104 = HEAP32[$p>>2]|0;
    $105 = ((($104)) + 48|0);
    $106 = ((($105)) + 72|0);
    $107 = HEAP32[$p>>2]|0;
    $108 = $106;
    $109 = $107;
    $110 = (($108) - ($109))|0;
    _memset(($103|0),0,($110|0))|0;
    $111 = HEAP32[$p>>2]|0;
    HEAP32[$111>>2] = 5;
    $112 = HEAP32[$p>>2]|0;
    $113 = ((($112)) + 48|0);
    $114 = HEAP32[$p>>2]|0;
    $115 = ((($114)) + 20|0);
    HEAP32[$115>>2] = $113;
    $116 = HEAP32[$1>>2]|0;
    $117 = ((($116)) + 20|0);
    $118 = HEAP32[$117>>2]|0;
    $119 = HEAP32[$p>>2]|0;
    $120 = ((($119)) + 20|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = ((($121)) + 68|0);
    HEAP32[$122>>2] = $118;
    $123 = HEAP32[$nToken>>2]|0;
    $124 = HEAP32[$p>>2]|0;
    $125 = ((($124)) + 20|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ((($126)) + 64|0);
    HEAP32[$127>>2] = $123;
    $128 = HEAP32[$nToken>>2]|0;
    $129 = HEAP32[$p>>2]|0;
    $130 = ((($129)) + 20|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = ((($131)) + 72|0);
    $133 = (($132) + (($128*24)|0)|0);
    HEAP32[$zBuf>>2] = $133;
    $134 = HEAP32[$zTemp>>2]|0;
    $135 = ($134|0)!=(0|0);
    if ($135) {
     $136 = HEAP32[$zBuf>>2]|0;
     $137 = HEAP32[$zTemp>>2]|0;
     $138 = HEAP32[$nTemp>>2]|0;
     _memcpy(($136|0),($137|0),($138|0))|0;
     $139 = HEAP32[$zTemp>>2]|0;
     _sqlite3_free($139);
    }
    HEAP32[$jj>>2] = 0;
    while(1) {
     $140 = HEAP32[$jj>>2]|0;
     $141 = HEAP32[$p>>2]|0;
     $142 = ((($141)) + 20|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = ((($143)) + 64|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = ($140|0)<($145|0);
     if (!($146)) {
      break;
     }
     $147 = HEAP32[$zBuf>>2]|0;
     $148 = HEAP32[$jj>>2]|0;
     $149 = HEAP32[$p>>2]|0;
     $150 = ((($149)) + 20|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = ((($151)) + 72|0);
     $153 = (($152) + (($148*24)|0)|0);
     HEAP32[$153>>2] = $147;
     $154 = HEAP32[$jj>>2]|0;
     $155 = HEAP32[$p>>2]|0;
     $156 = ((($155)) + 20|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ((($157)) + 72|0);
     $159 = (($158) + (($154*24)|0)|0);
     $160 = ((($159)) + 4|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$zBuf>>2]|0;
     $163 = (($162) + ($161)|0);
     HEAP32[$zBuf>>2] = $163;
     $164 = HEAP32[$jj>>2]|0;
     $165 = (($164) + 1)|0;
     HEAP32[$jj>>2] = $165;
    }
    HEAP32[$rc>>2] = 0;
   }
   $166 = HEAP32[$p>>2]|0;
   $167 = HEAP32[$4>>2]|0;
   HEAP32[$167>>2] = $166;
   $168 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $168;
   $178 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($178|0);
  }
 } while(0);
 $169 = HEAP32[$pCursor>>2]|0;
 $170 = ($169|0)!=(0|0);
 if ($170) {
  $171 = HEAP32[$pModule>>2]|0;
  $172 = ((($171)) + 16|0);
  $173 = HEAP32[$172>>2]|0;
  $174 = HEAP32[$pCursor>>2]|0;
  (FUNCTION_TABLE_ii[$173 & 255]($174)|0);
 }
 $175 = HEAP32[$zTemp>>2]|0;
 _sqlite3_free($175);
 $176 = HEAP32[$p>>2]|0;
 _sqlite3_free($176);
 $177 = HEAP32[$4>>2]|0;
 HEAP32[$177>>2] = 0;
 HEAP32[$0>>2] = 7;
 $178 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($178|0);
}
function _getNextToken($pParse,$iCol,$z,$n,$ppExpr,$pnConsumed) {
 $pParse = $pParse|0;
 $iCol = $iCol|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pnConsumed = $pnConsumed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $iEnd = 0, $iPosition = 0, $iStart = 0, $nByte = 0, $nToken = 0, $or$cond = 0, $or$cond3 = 0, $pCursor = 0, $pModule = 0, $pRet = 0, $pTokenizer = 0, $rc = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $pTokenizer = sp + 44|0;
 $pModule = sp + 40|0;
 $rc = sp + 36|0;
 $pCursor = sp + 32|0;
 $pRet = sp + 28|0;
 $i = sp + 24|0;
 $zToken = sp + 20|0;
 $nToken = sp + 16|0;
 $iStart = sp + 12|0;
 $iEnd = sp + 8|0;
 $iPosition = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $ppExpr;
 HEAP32[$5>>2] = $pnConsumed;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTokenizer>>2] = $7;
 $8 = HEAP32[$pTokenizer>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pModule>>2] = $9;
 HEAP32[$pRet>>2] = 0;
 HEAP32[$i>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)==(40);
  if ($18) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (($20) + ($19)|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22 << 24 >> 24;
  $24 = ($23|0)==(41);
  if ($24) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + ($25)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)==(34);
  if ($30) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
 }
 $33 = HEAP32[$i>>2]|0;
 $34 = HEAP32[$5>>2]|0;
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$pTokenizer>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$i>>2]|0;
 $41 = (_sqlite3Fts3OpenTokenizer($35,$38,$39,$40,$pCursor)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if (!($43)) {
  $138 = HEAP32[$pRet>>2]|0;
  $139 = HEAP32[$4>>2]|0;
  HEAP32[$139>>2] = $138;
  $140 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($140|0);
 }
 HEAP32[$nToken>>2] = 0;
 HEAP32[$iStart>>2] = 0;
 HEAP32[$iEnd>>2] = 0;
 HEAP32[$iPosition>>2] = 0;
 $44 = HEAP32[$pModule>>2]|0;
 $45 = ((($44)) + 20|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$pCursor>>2]|0;
 $48 = (FUNCTION_TABLE_iiiiiii[$46 & 255]($47,$zToken,$nToken,$iStart,$iEnd,$iPosition)|0);
 HEAP32[$rc>>2] = $48;
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$nToken>>2]|0;
  $52 = (144 + ($51))|0;
  HEAP32[$nByte>>2] = $52;
  $53 = HEAP32[$nByte>>2]|0;
  $54 = (_fts3MallocZero($53)|0);
  HEAP32[$pRet>>2] = $54;
  $55 = HEAP32[$pRet>>2]|0;
  $56 = ($55|0)!=(0|0);
  L13: do {
   if ($56) {
    $57 = HEAP32[$pRet>>2]|0;
    HEAP32[$57>>2] = 5;
    $58 = HEAP32[$pRet>>2]|0;
    $59 = ((($58)) + 48|0);
    $60 = HEAP32[$pRet>>2]|0;
    $61 = ((($60)) + 20|0);
    HEAP32[$61>>2] = $59;
    $62 = HEAP32[$pRet>>2]|0;
    $63 = ((($62)) + 20|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ((($64)) + 64|0);
    HEAP32[$65>>2] = 1;
    $66 = HEAP32[$1>>2]|0;
    $67 = HEAP32[$pRet>>2]|0;
    $68 = ((($67)) + 20|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = ((($69)) + 68|0);
    HEAP32[$70>>2] = $66;
    $71 = HEAP32[$nToken>>2]|0;
    $72 = HEAP32[$pRet>>2]|0;
    $73 = ((($72)) + 20|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ((($74)) + 72|0);
    $76 = ((($75)) + 4|0);
    HEAP32[$76>>2] = $71;
    $77 = HEAP32[$pRet>>2]|0;
    $78 = ((($77)) + 20|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ((($79)) + 96|0);
    $81 = HEAP32[$pRet>>2]|0;
    $82 = ((($81)) + 20|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ((($83)) + 72|0);
    HEAP32[$84>>2] = $80;
    $85 = HEAP32[$pRet>>2]|0;
    $86 = ((($85)) + 20|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ((($87)) + 72|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$zToken>>2]|0;
    $91 = HEAP32[$nToken>>2]|0;
    _memcpy(($89|0),($90|0),($91|0))|0;
    $92 = HEAP32[$iEnd>>2]|0;
    $93 = HEAP32[$3>>2]|0;
    $94 = ($92|0)<($93|0);
    if ($94) {
     $95 = HEAP32[$iEnd>>2]|0;
     $96 = HEAP32[$2>>2]|0;
     $97 = (($96) + ($95)|0);
     $98 = HEAP8[$97>>0]|0;
     $99 = $98 << 24 >> 24;
     $100 = ($99|0)==(42);
     if ($100) {
      $101 = HEAP32[$pRet>>2]|0;
      $102 = ((($101)) + 20|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = ((($103)) + 72|0);
      $105 = ((($104)) + 8|0);
      HEAP32[$105>>2] = 1;
      $106 = HEAP32[$iEnd>>2]|0;
      $107 = (($106) + 1)|0;
      HEAP32[$iEnd>>2] = $107;
     }
    }
    while(1) {
     $108 = HEAP32[$0>>2]|0;
     $109 = ((($108)) + 12|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ($110|0)!=(0);
     $112 = HEAP32[$iStart>>2]|0;
     $113 = ($112|0)>(0);
     $or$cond = $111 & $113;
     if (!($or$cond)) {
      break L13;
     }
     $114 = HEAP32[$iStart>>2]|0;
     $115 = (($114) - 1)|0;
     $116 = HEAP32[$2>>2]|0;
     $117 = (($116) + ($115)|0);
     $118 = HEAP8[$117>>0]|0;
     $119 = $118 << 24 >> 24;
     $120 = ($119|0)==(94);
     if (!($120)) {
      break L13;
     }
     $121 = HEAP32[$pRet>>2]|0;
     $122 = ((($121)) + 20|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = ((($123)) + 72|0);
     $125 = ((($124)) + 12|0);
     HEAP32[$125>>2] = 1;
     $126 = HEAP32[$iStart>>2]|0;
     $127 = (($126) + -1)|0;
     HEAP32[$iStart>>2] = $127;
    }
   } else {
    HEAP32[$rc>>2] = 7;
   }
  } while(0);
  $128 = HEAP32[$iEnd>>2]|0;
  $129 = HEAP32[$5>>2]|0;
  HEAP32[$129>>2] = $128;
 } else {
  $130 = HEAP32[$i>>2]|0;
  $131 = ($130|0)!=(0);
  $132 = HEAP32[$rc>>2]|0;
  $133 = ($132|0)==(101);
  $or$cond3 = $131 & $133;
  if ($or$cond3) {
   HEAP32[$rc>>2] = 0;
  }
 }
 $134 = HEAP32[$pModule>>2]|0;
 $135 = ((($134)) + 16|0);
 $136 = HEAP32[$135>>2]|0;
 $137 = HEAP32[$pCursor>>2]|0;
 (FUNCTION_TABLE_ii[$136 & 255]($137)|0);
 $138 = HEAP32[$pRet>>2]|0;
 $139 = HEAP32[$4>>2]|0;
 HEAP32[$139>>2] = $138;
 $140 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($140|0);
}
function _sqlite3Fts3OpenTokenizer($pTokenizer,$iLangid,$z,$n,$ppCsr) {
 $pTokenizer = $pTokenizer|0;
 $iLangid = $iLangid|0;
 $z = $z|0;
 $n = $n|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $pModule = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pModule = sp + 8|0;
 $pCsr = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTokenizer;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $ppCsr;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pModule>>2] = $6;
 HEAP32[$pCsr>>2] = 0;
 $7 = HEAP32[$pModule>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (FUNCTION_TABLE_iiiii[$9 & 255]($10,$11,$12,$pCsr)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(0);
 if ($15) {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pCsr>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$pModule>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)>=(1);
  if ($20) {
   $21 = HEAP32[$pModule>>2]|0;
   $22 = ((($21)) + 24|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$pCsr>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = (FUNCTION_TABLE_iii[$23 & 255]($24,$25)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$rc>>2]|0;
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$pModule>>2]|0;
    $30 = ((($29)) + 16|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$pCsr>>2]|0;
    (FUNCTION_TABLE_ii[$31 & 255]($32)|0);
    HEAP32[$pCsr>>2] = 0;
   }
  }
 }
 $33 = HEAP32[$pCsr>>2]|0;
 $34 = HEAP32[$4>>2]|0;
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _fts3ReallocOrFree($pOrig,$nNew) {
 $pOrig = $pOrig|0;
 $nNew = $nNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $pOrig;
 HEAP32[$1>>2] = $nNew;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_realloc($2,$3)|0);
 HEAP32[$pRet>>2] = $4;
 $5 = HEAP32[$pRet>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $8 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $7 = HEAP32[$0>>2]|0;
 _sqlite3_free($7);
 $8 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _opPrecedence($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _fts3EvalAllocateReaders($pCsr,$pExpr,$pnToken,$pnOr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pnToken = $pnToken|0;
 $pnOr = $pnOr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $nToken = 0, $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $nToken = sp + 8|0;
 $pToken = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pnToken;
 HEAP32[$3>>2] = $pnOr;
 HEAP32[$4>>2] = $pRc;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (0)==($8|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(5);
 $13 = HEAP32[$1>>2]|0;
 if (!($12)) {
  $53 = HEAP32[$13>>2]|0;
  $54 = ($53|0)==(4);
  $55 = $54&1;
  $56 = HEAP32[$3>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($55))|0;
  HEAP32[$56>>2] = $58;
  $59 = HEAP32[$0>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 12|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$2>>2]|0;
  $64 = HEAP32[$3>>2]|0;
  $65 = HEAP32[$4>>2]|0;
  _fts3EvalAllocateReaders($59,$62,$63,$64,$65);
  $66 = HEAP32[$0>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  $68 = ((($67)) + 16|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = HEAP32[$3>>2]|0;
  $72 = HEAP32[$4>>2]|0;
  _fts3EvalAllocateReaders($66,$69,$70,$71,$72);
  STACKTOP = sp;return;
 }
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$nToken>>2] = $17;
 $18 = HEAP32[$nToken>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18))|0;
 HEAP32[$19>>2] = $21;
 HEAP32[$i>>2] = 0;
 while(1) {
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$nToken>>2]|0;
  $24 = ($22|0)<($23|0);
  if (!($24)) {
   label = 9;
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 72|0);
  $30 = (($29) + (($25*24)|0)|0);
  HEAP32[$pToken>>2] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$pToken>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$pToken>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$pToken>>2]|0;
  $38 = ((($37)) + 8|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$pToken>>2]|0;
  $41 = ((($40)) + 20|0);
  $42 = (_fts3TermSegReaderCursor($31,$33,$36,$39,$41)|0);
  HEAP32[$rc>>2] = $42;
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 7;
   break;
  }
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$i>>2] = $48;
 }
 if ((label|0) == 7) {
  $45 = HEAP32[$rc>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  HEAP32[$46>>2] = $45;
  STACKTOP = sp;return;
 }
 else if ((label|0) == 9) {
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 20|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ((($51)) + 44|0);
  HEAP32[$52>>2] = -1;
  STACKTOP = sp;return;
 }
}
function _fts3EvalTokenCosts($pCsr,$pRoot,$pExpr,$ppTC,$ppOr,$pRc) {
 $pCsr = $pCsr|0;
 $pRoot = $pRoot|0;
 $pExpr = $pExpr|0;
 $ppTC = $ppTC|0;
 $ppOr = $ppOr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $i = 0;
 var $pPhrase = 0, $pTC = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $pPhrase = sp + 8|0;
 $i = sp + 4|0;
 $pTC = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $ppTC;
 HEAP32[$4>>2] = $ppOr;
 HEAP32[$5>>2] = $pRc;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(5);
 $12 = HEAP32[$2>>2]|0;
 if ($11) {
  $13 = ((($12)) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pPhrase>>2] = $14;
  HEAP32[$i>>2] = 0;
  while(1) {
   $15 = HEAP32[$5>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)==(0);
   if (!($17)) {
    label = 13;
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$pPhrase>>2]|0;
   $20 = ((($19)) + 64|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($18|0)<($21|0);
   if (!($22)) {
    label = 13;
    break;
   }
   $23 = HEAP32[$3>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = ((($24)) + 24|0);
   HEAP32[$23>>2] = $25;
   HEAP32[$pTC>>2] = $24;
   $26 = HEAP32[$pPhrase>>2]|0;
   $27 = HEAP32[$pTC>>2]|0;
   HEAP32[$27>>2] = $26;
   $28 = HEAP32[$i>>2]|0;
   $29 = HEAP32[$pTC>>2]|0;
   $30 = ((($29)) + 4|0);
   HEAP32[$30>>2] = $28;
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP32[$pTC>>2]|0;
   $33 = ((($32)) + 12|0);
   HEAP32[$33>>2] = $31;
   $34 = HEAP32[$i>>2]|0;
   $35 = HEAP32[$pPhrase>>2]|0;
   $36 = ((($35)) + 72|0);
   $37 = (($36) + (($34*24)|0)|0);
   $38 = HEAP32[$pTC>>2]|0;
   $39 = ((($38)) + 8|0);
   HEAP32[$39>>2] = $37;
   $40 = HEAP32[$pPhrase>>2]|0;
   $41 = ((($40)) + 68|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$pTC>>2]|0;
   $44 = ((($43)) + 20|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$0>>2]|0;
   $46 = HEAP32[$pTC>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ((($48)) + 20|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$pTC>>2]|0;
   $52 = ((($51)) + 16|0);
   $53 = (_sqlite3Fts3MsrOvfl($45,$50,$52)|0);
   $54 = HEAP32[$5>>2]|0;
   HEAP32[$54>>2] = $53;
   $55 = HEAP32[$i>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$i>>2] = $56;
  }
  if ((label|0) == 13) {
   STACKTOP = sp;return;
  }
 }
 $57 = HEAP32[$12>>2]|0;
 $58 = ($57|0)!=(2);
 if (!($58)) {
  STACKTOP = sp;return;
 }
 $59 = HEAP32[$2>>2]|0;
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)==(4);
 if ($61) {
  $62 = HEAP32[$2>>2]|0;
  $63 = ((($62)) + 12|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$1>>2] = $64;
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$4>>2]|0;
  $67 = HEAP32[$66>>2]|0;
  HEAP32[$67>>2] = $65;
  $68 = HEAP32[$4>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = ((($69)) + 4|0);
  HEAP32[$68>>2] = $70;
 }
 $71 = HEAP32[$0>>2]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = HEAP32[$2>>2]|0;
 $74 = ((($73)) + 12|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = HEAP32[$3>>2]|0;
 $77 = HEAP32[$4>>2]|0;
 $78 = HEAP32[$5>>2]|0;
 _fts3EvalTokenCosts($71,$72,$75,$76,$77,$78);
 $79 = HEAP32[$2>>2]|0;
 $80 = HEAP32[$79>>2]|0;
 $81 = ($80|0)==(4);
 if ($81) {
  $82 = HEAP32[$2>>2]|0;
  $83 = ((($82)) + 16|0);
  $84 = HEAP32[$83>>2]|0;
  HEAP32[$1>>2] = $84;
  $85 = HEAP32[$1>>2]|0;
  $86 = HEAP32[$4>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  HEAP32[$87>>2] = $85;
  $88 = HEAP32[$4>>2]|0;
  $89 = HEAP32[$88>>2]|0;
  $90 = ((($89)) + 4|0);
  HEAP32[$88>>2] = $90;
 }
 $91 = HEAP32[$0>>2]|0;
 $92 = HEAP32[$1>>2]|0;
 $93 = HEAP32[$2>>2]|0;
 $94 = ((($93)) + 16|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = HEAP32[$3>>2]|0;
 $97 = HEAP32[$4>>2]|0;
 $98 = HEAP32[$5>>2]|0;
 _fts3EvalTokenCosts($91,$92,$95,$96,$97,$98);
 STACKTOP = sp;return;
}
function _fts3EvalSelectDeferred($pCsr,$pRoot,$aTC,$nTC) {
 $pCsr = $pCsr|0;
 $pRoot = $pRoot|0;
 $aTC = $aTC|0;
 $nTC = $nTC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iTC = 0, $ii = 0, $nCount = 0, $nDocSize = 0, $nList = 0, $nLoad4 = 0, $nMinEst = 0, $nOvfl = 0, $nToken = 0, $or$cond = 0, $pList = 0, $pTC = 0, $pTab = 0, $pToken = 0, $pToken1 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $pTab = sp + 56|0;
 $nDocSize = sp + 52|0;
 $rc = sp + 48|0;
 $ii = sp + 44|0;
 $nOvfl = sp + 40|0;
 $nToken = sp + 36|0;
 $nMinEst = sp + 32|0;
 $nLoad4 = sp + 28|0;
 $iTC = sp + 24|0;
 $pTC = sp + 20|0;
 $pToken = sp + 16|0;
 $pToken1 = sp + 12|0;
 $nList = sp + 8|0;
 $pList = sp + 4|0;
 $nCount = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pRoot;
 HEAP32[$3>>2] = $aTC;
 HEAP32[$4>>2] = $nTC;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTab>>2] = $6;
 HEAP32[$nDocSize>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nOvfl>>2] = 0;
 HEAP32[$nToken>>2] = 0;
 HEAP32[$nMinEst>>2] = 0;
 HEAP32[$nLoad4>>2] = 1;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ((($7)) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $11 = HEAP32[$ii>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$ii>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + (($14*24)|0)|0);
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = ($18|0)==($19|0);
  if ($20) {
   $21 = HEAP32[$ii>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (($22) + (($21*24)|0)|0);
   $24 = ((($23)) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$nOvfl>>2]|0;
   $27 = (($26) + ($25))|0;
   HEAP32[$nOvfl>>2] = $27;
   $28 = HEAP32[$nToken>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$nToken>>2] = $29;
  }
  $30 = HEAP32[$ii>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$ii>>2] = $31;
 }
 $32 = HEAP32[$nOvfl>>2]|0;
 $33 = ($32|0)==(0);
 $34 = HEAP32[$nToken>>2]|0;
 $35 = ($34|0)<(2);
 $or$cond = $33 | $35;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = (_fts3EvalAverageDocsize($36,$nDocSize)|0);
 HEAP32[$rc>>2] = $37;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $38 = HEAP32[$ii>>2]|0;
  $39 = HEAP32[$nToken>>2]|0;
  $40 = ($38|0)<($39|0);
  $41 = HEAP32[$rc>>2]|0;
  $42 = ($41|0)==(0);
  $43 = $40 ? $42 : 0;
  if (!($43)) {
   break;
  }
  HEAP32[$pTC>>2] = 0;
  HEAP32[$iTC>>2] = 0;
  while(1) {
   $44 = HEAP32[$iTC>>2]|0;
   $45 = HEAP32[$4>>2]|0;
   $46 = ($44|0)<($45|0);
   if (!($46)) {
    break;
   }
   $47 = HEAP32[$iTC>>2]|0;
   $48 = HEAP32[$3>>2]|0;
   $49 = (($48) + (($47*24)|0)|0);
   $50 = ((($49)) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($51|0)!=(0|0);
   do {
    if ($52) {
     $53 = HEAP32[$iTC>>2]|0;
     $54 = HEAP32[$3>>2]|0;
     $55 = (($54) + (($53*24)|0)|0);
     $56 = ((($55)) + 12|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = HEAP32[$2>>2]|0;
     $59 = ($57|0)==($58|0);
     if ($59) {
      $60 = HEAP32[$pTC>>2]|0;
      $61 = ($60|0)!=(0|0);
      if ($61) {
       $62 = HEAP32[$iTC>>2]|0;
       $63 = HEAP32[$3>>2]|0;
       $64 = (($63) + (($62*24)|0)|0);
       $65 = ((($64)) + 16|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = HEAP32[$pTC>>2]|0;
       $68 = ((($67)) + 16|0);
       $69 = HEAP32[$68>>2]|0;
       $70 = ($66|0)<($69|0);
       if (!($70)) {
        break;
       }
      }
      $71 = HEAP32[$iTC>>2]|0;
      $72 = HEAP32[$3>>2]|0;
      $73 = (($72) + (($71*24)|0)|0);
      HEAP32[$pTC>>2] = $73;
     }
    }
   } while(0);
   $74 = HEAP32[$iTC>>2]|0;
   $75 = (($74) + 1)|0;
   HEAP32[$iTC>>2] = $75;
  }
  $76 = HEAP32[$ii>>2]|0;
  $77 = ($76|0)!=(0);
  if ($77) {
   $78 = HEAP32[$pTC>>2]|0;
   $79 = ((($78)) + 16|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = HEAP32[$nMinEst>>2]|0;
   $82 = HEAP32[$nLoad4>>2]|0;
   $83 = (($82|0) / 4)&-1;
   $84 = (($81) + ($83))|0;
   $85 = (($84) - 1)|0;
   $86 = HEAP32[$nLoad4>>2]|0;
   $87 = (($86|0) / 4)&-1;
   $88 = (($85|0) / ($87|0))&-1;
   $89 = HEAP32[$nDocSize>>2]|0;
   $90 = Math_imul($88, $89)|0;
   $91 = ($80|0)>=($90|0);
   if ($91) {
    $92 = HEAP32[$pTC>>2]|0;
    $93 = ((($92)) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    HEAP32[$pToken>>2] = $94;
    $95 = HEAP32[$1>>2]|0;
    $96 = HEAP32[$pToken>>2]|0;
    $97 = HEAP32[$pTC>>2]|0;
    $98 = ((($97)) + 20|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = (_sqlite3Fts3DeferToken($95,$96,$99)|0);
    HEAP32[$rc>>2] = $100;
    $101 = HEAP32[$pToken>>2]|0;
    $102 = ((($101)) + 20|0);
    $103 = HEAP32[$102>>2]|0;
    _fts3SegReaderCursorFree($103);
    $104 = HEAP32[$pToken>>2]|0;
    $105 = ((($104)) + 20|0);
    HEAP32[$105>>2] = 0;
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  do {
   if ((label|0) == 23) {
    label = 0;
    $106 = HEAP32[$ii>>2]|0;
    $107 = ($106|0)<(12);
    if ($107) {
     $108 = HEAP32[$nLoad4>>2]|0;
     $109 = $108<<2;
     HEAP32[$nLoad4>>2] = $109;
    }
    $110 = HEAP32[$ii>>2]|0;
    $111 = ($110|0)==(0);
    if (!($111)) {
     $112 = HEAP32[$pTC>>2]|0;
     $113 = HEAP32[$112>>2]|0;
     $114 = ((($113)) + 64|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = ($115|0)>(1);
     if (!($116)) {
      break;
     }
     $117 = HEAP32[$ii>>2]|0;
     $118 = HEAP32[$nToken>>2]|0;
     $119 = (($118) - 1)|0;
     $120 = ($117|0)!=($119|0);
     if (!($120)) {
      break;
     }
    }
    $121 = HEAP32[$pTC>>2]|0;
    $122 = ((($121)) + 8|0);
    $123 = HEAP32[$122>>2]|0;
    HEAP32[$pToken1>>2] = $123;
    HEAP32[$nList>>2] = 0;
    HEAP32[$pList>>2] = 0;
    $124 = HEAP32[$pTab>>2]|0;
    $125 = HEAP32[$pToken1>>2]|0;
    $126 = HEAP32[$pTC>>2]|0;
    $127 = ((($126)) + 20|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = (_fts3TermSelect($124,$125,$128,$nList,$pList)|0);
    HEAP32[$rc>>2] = $129;
    $130 = HEAP32[$rc>>2]|0;
    $131 = ($130|0)==(0);
    if ($131) {
     $132 = HEAP32[$pTab>>2]|0;
     $133 = HEAP32[$pTC>>2]|0;
     $134 = HEAP32[$133>>2]|0;
     $135 = HEAP32[$pTC>>2]|0;
     $136 = ((($135)) + 4|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = HEAP32[$pList>>2]|0;
     $139 = HEAP32[$nList>>2]|0;
     $140 = (_fts3EvalPhraseMergeToken($132,$134,$137,$138,$139)|0);
     HEAP32[$rc>>2] = $140;
    }
    $141 = HEAP32[$rc>>2]|0;
    $142 = ($141|0)==(0);
    if ($142) {
     $143 = HEAP32[$pTC>>2]|0;
     $144 = HEAP32[$143>>2]|0;
     $145 = HEAP32[$144>>2]|0;
     $146 = HEAP32[$pTC>>2]|0;
     $147 = HEAP32[$146>>2]|0;
     $148 = ((($147)) + 4|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = (_fts3DoclistCountDocids($145,$149)|0);
     HEAP32[$nCount>>2] = $150;
     $151 = HEAP32[$ii>>2]|0;
     $152 = ($151|0)==(0);
     if (!($152)) {
      $153 = HEAP32[$nCount>>2]|0;
      $154 = HEAP32[$nMinEst>>2]|0;
      $155 = ($153|0)<($154|0);
      if (!($155)) {
       break;
      }
     }
     $156 = HEAP32[$nCount>>2]|0;
     HEAP32[$nMinEst>>2] = $156;
    }
   }
  } while(0);
  $157 = HEAP32[$pTC>>2]|0;
  $158 = ((($157)) + 8|0);
  HEAP32[$158>>2] = 0;
  $159 = HEAP32[$ii>>2]|0;
  $160 = (($159) + 1)|0;
  HEAP32[$ii>>2] = $160;
 }
 $161 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $161;
 $162 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _fts3EvalStartReaders($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $i = 0, $nToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nToken = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (0)==($6|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(5);
 if (!($10)) {
  $45 = HEAP32[$0>>2]|0;
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 12|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$2>>2]|0;
  _fts3EvalStartReaders($45,$48,$49);
  $50 = HEAP32[$0>>2]|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  _fts3EvalStartReaders($50,$53,$54);
  $55 = HEAP32[$1>>2]|0;
  $56 = ((($55)) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ((($57)) + 34|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = $59&255;
  $61 = ($60|0)!=(0);
  if ($61) {
   $62 = HEAP32[$1>>2]|0;
   $63 = ((($62)) + 16|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = ((($64)) + 34|0);
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   $68 = ($67|0)!=(0);
   $70 = $68;
  } else {
   $70 = 0;
  }
  $69 = $70&1;
  $71 = $69&255;
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 34|0);
  HEAP8[$73>>0] = $71;
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 64|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$nToken>>2] = $15;
 $16 = HEAP32[$nToken>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$nToken>>2]|0;
   $20 = ($18|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 20|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ((($24)) + 72|0);
   $26 = (($25) + (($21*24)|0)|0);
   $27 = ((($26)) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)==(0|0);
   if ($29) {
    break;
   }
   $30 = HEAP32[$i>>2]|0;
   $31 = (($30) + 1)|0;
   HEAP32[$i>>2] = $31;
  }
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$nToken>>2]|0;
  $34 = ($32|0)==($33|0);
  $35 = $34&1;
  $36 = $35&255;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 34|0);
  HEAP8[$38>>0] = $36;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 20|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (_fts3EvalPhraseStart($39,1,$42)|0);
 $44 = HEAP32[$2>>2]|0;
 HEAP32[$44>>2] = $43;
 STACKTOP = sp;return;
}
function _fts3TermSegReaderCursor($pCsr,$zTerm,$nTerm,$isPrefix,$ppSegcsr) {
 $pCsr = $pCsr|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $isPrefix = $isPrefix|0;
 $ppSegcsr = $ppSegcsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $bFound = 0, $i = 0, $p = 0;
 var $pSegcsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pSegcsr = sp + 16|0;
 $rc = sp + 12|0;
 $i = sp + 8|0;
 $bFound = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $isPrefix;
 HEAP32[$4>>2] = $ppSegcsr;
 HEAP32[$rc>>2] = 7;
 $5 = (_sqlite3_malloc(56)|0);
 HEAP32[$pSegcsr>>2] = $5;
 $6 = HEAP32[$pSegcsr>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $94 = HEAP32[$pSegcsr>>2]|0;
  $95 = HEAP32[$4>>2]|0;
  HEAP32[$95>>2] = $94;
  $96 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 HEAP32[$bFound>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0);
 L4: do {
  if ($11) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $12 = HEAP32[$bFound>>2]|0;
    $13 = ($12|0)==(0);
    if (!($13)) {
     break;
    }
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$p>>2]|0;
    $16 = ((($15)) + 248|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ($14|0)<($17|0);
    if (!($18)) {
     break;
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = HEAP32[$p>>2]|0;
    $21 = ((($20)) + 252|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + (($19*24)|0)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$2>>2]|0;
    $26 = ($24|0)==($25|0);
    if ($26) {
     HEAP32[$bFound>>2] = 1;
     $27 = HEAP32[$p>>2]|0;
     $28 = HEAP32[$0>>2]|0;
     $29 = ((($28)) + 16|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$i>>2]|0;
     $32 = HEAP32[$1>>2]|0;
     $33 = HEAP32[$2>>2]|0;
     $34 = HEAP32[$pSegcsr>>2]|0;
     $35 = (_sqlite3Fts3SegReaderCursor($27,$30,$31,-2,$32,$33,0,0,$34)|0);
     HEAP32[$rc>>2] = $35;
     $36 = HEAP32[$pSegcsr>>2]|0;
     $37 = ((($36)) + 36|0);
     HEAP32[$37>>2] = 1;
    }
    $38 = HEAP32[$i>>2]|0;
    $39 = (($38) + 1)|0;
    HEAP32[$i>>2] = $39;
   }
   HEAP32[$i>>2] = 1;
   while(1) {
    $40 = HEAP32[$bFound>>2]|0;
    $41 = ($40|0)==(0);
    if (!($41)) {
     break L4;
    }
    $42 = HEAP32[$i>>2]|0;
    $43 = HEAP32[$p>>2]|0;
    $44 = ((($43)) + 248|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($42|0)<($45|0);
    if (!($46)) {
     break L4;
    }
    $47 = HEAP32[$i>>2]|0;
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 252|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + (($47*24)|0)|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (($53) + 1)|0;
    $55 = ($52|0)==($54|0);
    if ($55) {
     HEAP32[$bFound>>2] = 1;
     $56 = HEAP32[$p>>2]|0;
     $57 = HEAP32[$0>>2]|0;
     $58 = ((($57)) + 16|0);
     $59 = HEAP32[$58>>2]|0;
     $60 = HEAP32[$i>>2]|0;
     $61 = HEAP32[$1>>2]|0;
     $62 = HEAP32[$2>>2]|0;
     $63 = HEAP32[$pSegcsr>>2]|0;
     $64 = (_sqlite3Fts3SegReaderCursor($56,$59,$60,-2,$61,$62,1,0,$63)|0);
     HEAP32[$rc>>2] = $64;
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)==(0);
     if ($66) {
      $67 = HEAP32[$p>>2]|0;
      $68 = HEAP32[$0>>2]|0;
      $69 = ((($68)) + 16|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = HEAP32[$1>>2]|0;
      $72 = HEAP32[$2>>2]|0;
      $73 = HEAP32[$pSegcsr>>2]|0;
      $74 = (_fts3SegReaderCursorAddZero($67,$70,$71,$72,$73)|0);
      HEAP32[$rc>>2] = $74;
     }
    }
    $75 = HEAP32[$i>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$i>>2] = $76;
   }
  }
 } while(0);
 $77 = HEAP32[$bFound>>2]|0;
 $78 = ($77|0)==(0);
 if (!($78)) {
  $94 = HEAP32[$pSegcsr>>2]|0;
  $95 = HEAP32[$4>>2]|0;
  HEAP32[$95>>2] = $94;
  $96 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 $79 = HEAP32[$p>>2]|0;
 $80 = HEAP32[$0>>2]|0;
 $81 = ((($80)) + 16|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = HEAP32[$1>>2]|0;
 $84 = HEAP32[$2>>2]|0;
 $85 = HEAP32[$3>>2]|0;
 $86 = HEAP32[$pSegcsr>>2]|0;
 $87 = (_sqlite3Fts3SegReaderCursor($79,$82,0,-2,$83,$84,$85,0,$86)|0);
 HEAP32[$rc>>2] = $87;
 $88 = HEAP32[$3>>2]|0;
 $89 = ($88|0)!=(0);
 $90 = $89 ^ 1;
 $91 = $90&1;
 $92 = HEAP32[$pSegcsr>>2]|0;
 $93 = ((($92)) + 36|0);
 HEAP32[$93>>2] = $91;
 $94 = HEAP32[$pSegcsr>>2]|0;
 $95 = HEAP32[$4>>2]|0;
 HEAP32[$95>>2] = $94;
 $96 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($96|0);
}
function _fts3SegReaderCursorAddZero($p,$iLangid,$zTerm,$nTerm,$pCsr) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $zTerm;
 HEAP32[$3>>2] = $nTerm;
 HEAP32[$4>>2] = $pCsr;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_fts3SegReaderCursor($5,$6,0,-2,$7,$8,0,0,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3Fts3MsrOvfl($pCsr,$pMsr,$pnOvfl) {
 $pCsr = $pCsr|0;
 $pMsr = $pMsr|0;
 $pnOvfl = $pnOvfl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $ii = 0, $jj = 0, $nBlob = 0;
 var $nOvfl = 0, $p = 0, $pReader = 0, $pgsz = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $p = sp + 32|0;
 $nOvfl = sp + 28|0;
 $ii = sp + 24|0;
 $rc = sp + 20|0;
 $pgsz = sp + 16|0;
 $pReader = sp + 12|0;
 $jj = sp;
 $nBlob = sp + 8|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pMsr;
 HEAP32[$2>>2] = $pnOvfl;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 HEAP32[$nOvfl>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 236|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgsz>>2] = $7;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $8 = HEAP32[$rc>>2]|0;
  $9 = ($8|0)==(0);
  if (!($9)) {
   label = 13;
   break;
  }
  $10 = HEAP32[$ii>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   label = 13;
   break;
  }
  $15 = HEAP32[$ii>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($15<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pReader>>2] = $19;
  $20 = HEAP32[$pReader>>2]|0;
  $21 = ((($20)) + 56|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  L5: do {
   if (!($23)) {
    $24 = HEAP32[$pReader>>2]|0;
    $25 = ((($24)) + 5|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)!=(0);
    if (!($28)) {
     $29 = HEAP32[$pReader>>2]|0;
     $30 = ((($29)) + 8|0);
     $31 = $30;
     $32 = $31;
     $33 = HEAP32[$32>>2]|0;
     $34 = (($31) + 4)|0;
     $35 = $34;
     $36 = HEAP32[$35>>2]|0;
     $37 = $jj;
     $38 = $37;
     HEAP32[$38>>2] = $33;
     $39 = (($37) + 4)|0;
     $40 = $39;
     HEAP32[$40>>2] = $36;
     while(1) {
      $41 = $jj;
      $42 = $41;
      $43 = HEAP32[$42>>2]|0;
      $44 = (($41) + 4)|0;
      $45 = $44;
      $46 = HEAP32[$45>>2]|0;
      $47 = HEAP32[$pReader>>2]|0;
      $48 = ((($47)) + 16|0);
      $49 = $48;
      $50 = $49;
      $51 = HEAP32[$50>>2]|0;
      $52 = (($49) + 4)|0;
      $53 = $52;
      $54 = HEAP32[$53>>2]|0;
      $55 = ($46|0)<($54|0);
      $56 = ($43>>>0)<=($51>>>0);
      $57 = ($46|0)==($54|0);
      $58 = $57 & $56;
      $59 = $55 | $58;
      if (!($59)) {
       break L5;
      }
      $60 = HEAP32[$p>>2]|0;
      $61 = $jj;
      $62 = $61;
      $63 = HEAP32[$62>>2]|0;
      $64 = (($61) + 4)|0;
      $65 = $64;
      $66 = HEAP32[$65>>2]|0;
      $67 = (_sqlite3Fts3ReadBlock($60,$63,$66,0,$nBlob,0)|0);
      HEAP32[$rc>>2] = $67;
      $68 = HEAP32[$rc>>2]|0;
      $69 = ($68|0)!=(0);
      if ($69) {
       break L5;
      }
      $70 = HEAP32[$nBlob>>2]|0;
      $71 = (($70) + 35)|0;
      $72 = HEAP32[$pgsz>>2]|0;
      $73 = ($71|0)>($72|0);
      if ($73) {
       $74 = HEAP32[$nBlob>>2]|0;
       $75 = (($74) + 34)|0;
       $76 = HEAP32[$pgsz>>2]|0;
       $77 = (($75|0) / ($76|0))&-1;
       $78 = HEAP32[$nOvfl>>2]|0;
       $79 = (($78) + ($77))|0;
       HEAP32[$nOvfl>>2] = $79;
      }
      $80 = $jj;
      $81 = $80;
      $82 = HEAP32[$81>>2]|0;
      $83 = (($80) + 4)|0;
      $84 = $83;
      $85 = HEAP32[$84>>2]|0;
      $86 = (_i64Add(($82|0),($85|0),1,0)|0);
      $87 = tempRet0;
      $88 = $jj;
      $89 = $88;
      HEAP32[$89>>2] = $86;
      $90 = (($88) + 4)|0;
      $91 = $90;
      HEAP32[$91>>2] = $87;
     }
    }
   }
  } while(0);
  $92 = HEAP32[$ii>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$ii>>2] = $93;
 }
 if ((label|0) == 13) {
  $94 = HEAP32[$nOvfl>>2]|0;
  $95 = HEAP32[$2>>2]|0;
  HEAP32[$95>>2] = $94;
  $96 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 return (0)|0;
}
function _fts3EvalAverageDocsize($pCsr,$pnPage) {
 $pCsr = $pCsr|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $nByte = 0, $nDoc = 0, $or$cond = 0, $p = 0, $pEnd = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $p = sp + 28|0;
 $pStmt = sp + 24|0;
 $nDoc = sp + 8|0;
 $nByte = sp;
 $pEnd = sp + 20|0;
 $a = sp + 16|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pnPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$p>>2] = $8;
  $9 = $nDoc;
  $10 = $9;
  HEAP32[$10>>2] = 0;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = 0;
  $13 = $nByte;
  $14 = $13;
  HEAP32[$14>>2] = 0;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = 0;
  $17 = HEAP32[$p>>2]|0;
  $18 = (_sqlite3Fts3SelectDoctotal($17,$pStmt)|0);
  HEAP32[$rc>>2] = $18;
  $19 = HEAP32[$rc>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $21;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
  $22 = HEAP32[$pStmt>>2]|0;
  $23 = (_sqlite3_column_blob($22,0)|0);
  HEAP32[$a>>2] = $23;
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = (_sqlite3_column_bytes($24,0)|0);
  $26 = HEAP32[$a>>2]|0;
  $27 = (($26) + ($25)|0);
  HEAP32[$pEnd>>2] = $27;
  $28 = HEAP32[$a>>2]|0;
  $29 = (_sqlite3Fts3GetVarint($28,$nDoc)|0);
  $30 = HEAP32[$a>>2]|0;
  $31 = (($30) + ($29)|0);
  HEAP32[$a>>2] = $31;
  while(1) {
   $32 = HEAP32[$a>>2]|0;
   $33 = HEAP32[$pEnd>>2]|0;
   $34 = ($32>>>0)<($33>>>0);
   if (!($34)) {
    break;
   }
   $35 = HEAP32[$a>>2]|0;
   $36 = (_sqlite3Fts3GetVarint($35,$nByte)|0);
   $37 = HEAP32[$a>>2]|0;
   $38 = (($37) + ($36)|0);
   HEAP32[$a>>2] = $38;
  }
  $39 = $nDoc;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($39) + 4)|0;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($41|0)==(0);
  $46 = ($44|0)==(0);
  $47 = $45 & $46;
  $48 = $nByte;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($48) + 4)|0;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = ($50|0)==(0);
  $55 = ($53|0)==(0);
  $56 = $54 & $55;
  $or$cond = $47 | $56;
  if ($or$cond) {
   $57 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_reset($57)|0);
   HEAP32[$0>>2] = 267;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
  $58 = $nDoc;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 64|0);
  $66 = $65;
  $67 = $66;
  HEAP32[$67>>2] = $60;
  $68 = (($66) + 4)|0;
  $69 = $68;
  HEAP32[$69>>2] = $63;
  $70 = $nByte;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $nDoc;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  $79 = (($76) + 4)|0;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (___divdi3(($72|0),($75|0),($78|0),($81|0))|0);
  $83 = tempRet0;
  $84 = HEAP32[$p>>2]|0;
  $85 = ((($84)) + 236|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = ($86|0)<(0);
  $88 = $87 << 31 >> 31;
  $89 = (_i64Add(($82|0),($83|0),($86|0),($88|0))|0);
  $90 = tempRet0;
  $91 = HEAP32[$p>>2]|0;
  $92 = ((($91)) + 236|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = ($93|0)<(0);
  $95 = $94 << 31 >> 31;
  $96 = (___divdi3(($89|0),($90|0),($93|0),($95|0))|0);
  $97 = tempRet0;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 60|0);
  HEAP32[$99>>2] = $96;
  $100 = HEAP32[$pStmt>>2]|0;
  $101 = (_sqlite3_reset($100)|0);
  HEAP32[$rc>>2] = $101;
  $102 = HEAP32[$rc>>2]|0;
  $103 = ($102|0)!=(0);
  if ($103) {
   $104 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $104;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
 }
 $105 = HEAP32[$1>>2]|0;
 $106 = ((($105)) + 60|0);
 $107 = HEAP32[$106>>2]|0;
 $108 = HEAP32[$2>>2]|0;
 HEAP32[$108>>2] = $107;
 HEAP32[$0>>2] = 0;
 $109 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($109|0);
}
function _sqlite3Fts3DeferToken($pCsr,$pToken,$iCol) {
 $pCsr = $pCsr|0;
 $pToken = $pToken|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDeferred = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pDeferred = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pToken;
 HEAP32[$3>>2] = $iCol;
 $4 = (_sqlite3_malloc(16)|0);
 HEAP32[$pDeferred>>2] = $4;
 $5 = HEAP32[$pDeferred>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$pDeferred>>2]|0;
  ;HEAP32[$7>>2]=0|0;HEAP32[$7+4>>2]=0|0;HEAP32[$7+8>>2]=0|0;HEAP32[$7+12>>2]=0|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$pDeferred>>2]|0;
  HEAP32[$9>>2] = $8;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pDeferred>>2]|0;
  $14 = ((($13)) + 8|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$3>>2]|0;
  $16 = HEAP32[$pDeferred>>2]|0;
  $17 = ((($16)) + 4|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP32[$pDeferred>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 24|0);
  HEAP32[$20>>2] = $18;
  $21 = HEAP32[$pDeferred>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 16|0);
  HEAP32[$23>>2] = $21;
  HEAP32[$0>>2] = 0;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  HEAP32[$0>>2] = 7;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _fts3TermSelect($p,$pTok,$iColumn,$pnOut,$ppOut) {
 $p = $p|0;
 $pTok = $pTok|0;
 $iColumn = $iColumn|0;
 $pnOut = $pnOut|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $filter = 0, $i = 0, $pSegcsr = 0, $rc = 0, $tsc = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $3 = sp + 164|0;
 $4 = sp + 160|0;
 $rc = sp + 156|0;
 $pSegcsr = sp + 152|0;
 $tsc = sp + 24|0;
 $filter = sp + 8|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pTok;
 HEAP32[$2>>2] = $iColumn;
 HEAP32[$3>>2] = $pnOut;
 HEAP32[$4>>2] = $ppOut;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pSegcsr>>2] = $7;
 dest=$tsc; stop=dest+128|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 $12 = $11 ? 8 : 0;
 $13 = 3 | $12;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = $17 ? 32 : 0;
 $19 = $13 | $18;
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 24|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)<($23|0);
 $25 = $24 ? 4 : 0;
 $26 = $19 | $25;
 $27 = ((($filter)) + 12|0);
 HEAP32[$27>>2] = $26;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($filter)) + 8|0);
 HEAP32[$29>>2] = $28;
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$filter>>2] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ((($filter)) + 4|0);
 HEAP32[$35>>2] = $34;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pSegcsr>>2]|0;
 $38 = (_sqlite3Fts3SegReaderStart($36,$37,$filter)|0);
 HEAP32[$rc>>2] = $38;
 while(1) {
  $39 = HEAP32[$rc>>2]|0;
  $40 = (0)==($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$pSegcsr>>2]|0;
  $43 = (_sqlite3Fts3SegReaderStep($41,$42)|0);
  HEAP32[$rc>>2] = $43;
  $44 = (100)==($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$0>>2]|0;
  $46 = HEAP32[$pSegcsr>>2]|0;
  $47 = ((($46)) + 48|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$pSegcsr>>2]|0;
  $50 = ((($49)) + 52|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (_fts3TermSelectMerge($45,$tsc,$48,$51)|0);
  HEAP32[$rc>>2] = $52;
 }
 $53 = HEAP32[$rc>>2]|0;
 $54 = ($53|0)==(0);
 if ($54) {
  $55 = HEAP32[$0>>2]|0;
  $56 = (_fts3TermSelectFinishMerge($55,$tsc)|0);
  HEAP32[$rc>>2] = $56;
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$tsc>>2]|0;
  $60 = HEAP32[$4>>2]|0;
  HEAP32[$60>>2] = $59;
  $61 = ((($tsc)) + 64|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  $71 = HEAP32[$pSegcsr>>2]|0;
  _fts3SegReaderCursorFree($71);
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 20|0);
  HEAP32[$73>>2] = 0;
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $64 = HEAP32[$i>>2]|0;
  $65 = ($64|0)<(16);
  if (!($65)) {
   break;
  }
  $66 = HEAP32[$i>>2]|0;
  $67 = (($tsc) + ($66<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  _sqlite3_free($68);
  $69 = HEAP32[$i>>2]|0;
  $70 = (($69) + 1)|0;
  HEAP32[$i>>2] = $70;
 }
 $71 = HEAP32[$pSegcsr>>2]|0;
 _fts3SegReaderCursorFree($71);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 20|0);
 HEAP32[$73>>2] = 0;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _fts3EvalPhraseMergeToken($pTab,$p,$iToken,$pList,$nList) {
 $pTab = $pTab|0;
 $p = $p|0;
 $iToken = $iToken|0;
 $pList = $pList|0;
 $nList = $nList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $nDiff = 0, $nLeft = 0, $nRight = 0, $pLeft = 0, $pRight = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $rc = sp + 20|0;
 $pLeft = sp + 16|0;
 $pRight = sp + 12|0;
 $nLeft = sp + 8|0;
 $nRight = sp + 4|0;
 $nDiff = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iToken;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nList;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 $7 = HEAP32[$1>>2]|0;
 do {
  if ($6) {
   $8 = HEAP32[$7>>2]|0;
   _sqlite3_free($8);
   $9 = HEAP32[$1>>2]|0;
   HEAP32[$9>>2] = 0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 4|0);
   HEAP32[$11>>2] = 0;
  } else {
   $12 = ((($7)) + 44|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)<(0);
   if ($14) {
    $15 = HEAP32[$3>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    HEAP32[$16>>2] = $15;
    $17 = HEAP32[$4>>2]|0;
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 4|0);
    HEAP32[$19>>2] = $17;
    break;
   }
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0|0);
   if ($22) {
    $23 = HEAP32[$3>>2]|0;
    _sqlite3_free($23);
    break;
   }
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 44|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = ($26|0)<($27|0);
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   if ($28) {
    HEAP32[$pLeft>>2] = $30;
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$nLeft>>2] = $33;
    $34 = HEAP32[$3>>2]|0;
    HEAP32[$pRight>>2] = $34;
    $35 = HEAP32[$4>>2]|0;
    HEAP32[$nRight>>2] = $35;
    $36 = HEAP32[$2>>2]|0;
    $37 = HEAP32[$1>>2]|0;
    $38 = ((($37)) + 44|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($36) - ($39))|0;
    HEAP32[$nDiff>>2] = $40;
   } else {
    HEAP32[$pRight>>2] = $30;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 4|0);
    $43 = HEAP32[$42>>2]|0;
    HEAP32[$nRight>>2] = $43;
    $44 = HEAP32[$3>>2]|0;
    HEAP32[$pLeft>>2] = $44;
    $45 = HEAP32[$4>>2]|0;
    HEAP32[$nLeft>>2] = $45;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 44|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = HEAP32[$2>>2]|0;
    $50 = (($48) - ($49))|0;
    HEAP32[$nDiff>>2] = $50;
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = ((($51)) + 231|0);
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = HEAP32[$nDiff>>2]|0;
   $56 = HEAP32[$pLeft>>2]|0;
   $57 = HEAP32[$nLeft>>2]|0;
   $58 = (_fts3DoclistPhraseMerge($54,$55,$56,$57,$pRight,$nRight)|0);
   HEAP32[$rc>>2] = $58;
   $59 = HEAP32[$pLeft>>2]|0;
   _sqlite3_free($59);
   $60 = HEAP32[$pRight>>2]|0;
   $61 = HEAP32[$1>>2]|0;
   HEAP32[$61>>2] = $60;
   $62 = HEAP32[$nRight>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 4|0);
   HEAP32[$64>>2] = $62;
  }
 } while(0);
 $65 = HEAP32[$2>>2]|0;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 44|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($65|0)>($68|0);
 if (!($69)) {
  $73 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $70 = HEAP32[$2>>2]|0;
 $71 = HEAP32[$1>>2]|0;
 $72 = ((($71)) + 44|0);
 HEAP32[$72>>2] = $70;
 $73 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _fts3DoclistCountDocids($aList,$nList) {
 $aList = $aList|0;
 $nList = $nList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $aEnd = 0, $nDoc = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $nDoc = sp + 8|0;
 $aEnd = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $aList;
 HEAP32[$1>>2] = $nList;
 HEAP32[$nDoc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $19 = HEAP32[$nDoc>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + ($4)|0);
 HEAP32[$aEnd>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $7;
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = HEAP32[$aEnd>>2]|0;
  $10 = ($8>>>0)<($9>>>0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$nDoc>>2]|0;
  $12 = (($11) + 1)|0;
  HEAP32[$nDoc>>2] = $12;
  while(1) {
   $13 = HEAP32[$p>>2]|0;
   $14 = ((($13)) + 1|0);
   HEAP32[$p>>2] = $14;
   $15 = HEAP8[$13>>0]|0;
   $16 = $15 << 24 >> 24;
   $17 = $16 & 128;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    break;
   }
  }
  _fts3PoslistCopy(0,$p);
 }
 $19 = HEAP32[$nDoc>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3Fts3SelectDoctotal($pTab,$ppStmt) {
 $pTab = $pTab|0;
 $ppStmt = $ppStmt|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $ppStmt;
 HEAP32[$pStmt>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,22,$pStmt,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_bind_int($6,1,0)|0);
   $7 = HEAP32[$pStmt>>2]|0;
   $8 = (_sqlite3_step($7)|0);
   $9 = ($8|0)!=(100);
   if (!($9)) {
    $10 = HEAP32[$pStmt>>2]|0;
    $11 = (_sqlite3_column_type($10,0)|0);
    $12 = ($11|0)!=(4);
    if (!($12)) {
     break;
    }
   }
   $13 = HEAP32[$pStmt>>2]|0;
   $14 = (_sqlite3_reset($13)|0);
   HEAP32[$rc>>2] = $14;
   $15 = HEAP32[$rc>>2]|0;
   $16 = ($15|0)==(0);
   $$ = $16 ? 267 : $14;
   HEAP32[$rc>>2] = $$;
   HEAP32[$pStmt>>2] = 0;
  }
 } while(0);
 $17 = HEAP32[$pStmt>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _fts3TermSelectMerge($p,$pTS,$aDoclist,$nDoclist) {
 $p = $p|0;
 $pTS = $pTS|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $aMerge = 0, $aNew = 0, $iOut = 0, $nMerge = 0, $nNew = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $aMerge = sp + 20|0;
 $nMerge = sp + 16|0;
 $iOut = sp + 12|0;
 $aNew = sp + 8|0;
 $nNew = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTS;
 HEAP32[$3>>2] = $aDoclist;
 HEAP32[$4>>2] = $nDoclist;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$4>>2]|0;
   $9 = (($8) + 10)|0;
   $10 = (($9) + 1)|0;
   $11 = (_sqlite3_malloc($10)|0);
   $12 = HEAP32[$2>>2]|0;
   HEAP32[$12>>2] = $11;
   $13 = HEAP32[$4>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 64|0);
   HEAP32[$15>>2] = $13;
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $19 = HEAP32[$2>>2]|0;
    $20 = HEAP32[$19>>2]|0;
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$4>>2]|0;
    _memcpy(($20|0),($21|0),($22|0))|0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $88 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($88|0);
  } else {
   $23 = HEAP32[$3>>2]|0;
   HEAP32[$aMerge>>2] = $23;
   $24 = HEAP32[$4>>2]|0;
   HEAP32[$nMerge>>2] = $24;
   HEAP32[$iOut>>2] = 0;
   while(1) {
    $25 = HEAP32[$iOut>>2]|0;
    $26 = ($25|0)<(16);
    if (!($26)) {
     break L1;
    }
    $27 = HEAP32[$iOut>>2]|0;
    $28 = HEAP32[$2>>2]|0;
    $29 = (($28) + ($27<<2)|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     label = 8;
     break;
    }
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 231|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = HEAP32[$aMerge>>2]|0;
    $46 = HEAP32[$nMerge>>2]|0;
    $47 = HEAP32[$iOut>>2]|0;
    $48 = HEAP32[$2>>2]|0;
    $49 = (($48) + ($47<<2)|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$iOut>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 64|0);
    $54 = (($53) + ($51<<2)|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (_fts3DoclistOrMerge($44,$45,$46,$50,$55,$aNew,$nNew)|0);
    HEAP32[$rc>>2] = $56;
    $57 = HEAP32[$rc>>2]|0;
    $58 = ($57|0)!=(0);
    $59 = HEAP32[$aMerge>>2]|0;
    $60 = HEAP32[$3>>2]|0;
    $61 = ($59|0)!=($60|0);
    if ($58) {
     break;
    }
    if ($61) {
     $64 = HEAP32[$aMerge>>2]|0;
     _sqlite3_free($64);
    }
    $65 = HEAP32[$iOut>>2]|0;
    $66 = HEAP32[$2>>2]|0;
    $67 = (($66) + ($65<<2)|0);
    $68 = HEAP32[$67>>2]|0;
    _sqlite3_free($68);
    $69 = HEAP32[$iOut>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = (($70) + ($69<<2)|0);
    HEAP32[$71>>2] = 0;
    $72 = HEAP32[$aNew>>2]|0;
    HEAP32[$aMerge>>2] = $72;
    $73 = HEAP32[$nNew>>2]|0;
    HEAP32[$nMerge>>2] = $73;
    $74 = HEAP32[$iOut>>2]|0;
    $75 = (($74) + 1)|0;
    $76 = ($75|0)==(16);
    if ($76) {
     $77 = HEAP32[$aMerge>>2]|0;
     $78 = HEAP32[$iOut>>2]|0;
     $79 = HEAP32[$2>>2]|0;
     $80 = (($79) + ($78<<2)|0);
     HEAP32[$80>>2] = $77;
     $81 = HEAP32[$nMerge>>2]|0;
     $82 = HEAP32[$iOut>>2]|0;
     $83 = HEAP32[$2>>2]|0;
     $84 = ((($83)) + 64|0);
     $85 = (($84) + ($82<<2)|0);
     HEAP32[$85>>2] = $81;
    }
    $86 = HEAP32[$iOut>>2]|0;
    $87 = (($86) + 1)|0;
    HEAP32[$iOut>>2] = $87;
   }
   if ((label|0) == 8) {
    $32 = HEAP32[$aMerge>>2]|0;
    $33 = HEAP32[$iOut>>2]|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = (($34) + ($33<<2)|0);
    HEAP32[$35>>2] = $32;
    $36 = HEAP32[$nMerge>>2]|0;
    $37 = HEAP32[$iOut>>2]|0;
    $38 = HEAP32[$2>>2]|0;
    $39 = ((($38)) + 64|0);
    $40 = (($39) + ($37<<2)|0);
    HEAP32[$40>>2] = $36;
    break;
   }
   if ($61) {
    $62 = HEAP32[$aMerge>>2]|0;
    _sqlite3_free($62);
   }
   $63 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $63;
   $88 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($88|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $88 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _fts3TermSelectFinishMerge($p,$pTS) {
 $p = $p|0;
 $pTS = $pTS|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aNew = 0, $aOut = 0, $i = 0, $nNew = 0, $nOut = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $aOut = sp + 20|0;
 $nOut = sp + 16|0;
 $i = sp + 12|0;
 $nNew = sp + 8|0;
 $aNew = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTS;
 HEAP32[$aOut>>2] = 0;
 HEAP32[$nOut>>2] = 0;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3|0)<(16);
  if (!($4)) {
   label = 10;
   break;
  }
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + ($5<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  do {
   if ($9) {
    $10 = HEAP32[$aOut>>2]|0;
    $11 = ($10|0)!=(0|0);
    if (!($11)) {
     $12 = HEAP32[$i>>2]|0;
     $13 = HEAP32[$2>>2]|0;
     $14 = (($13) + ($12<<2)|0);
     $15 = HEAP32[$14>>2]|0;
     HEAP32[$aOut>>2] = $15;
     $16 = HEAP32[$i>>2]|0;
     $17 = HEAP32[$2>>2]|0;
     $18 = ((($17)) + 64|0);
     $19 = (($18) + ($16<<2)|0);
     $20 = HEAP32[$19>>2]|0;
     HEAP32[$nOut>>2] = $20;
     $21 = HEAP32[$i>>2]|0;
     $22 = HEAP32[$2>>2]|0;
     $23 = (($22) + ($21<<2)|0);
     HEAP32[$23>>2] = 0;
     break;
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 231|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = HEAP32[$i>>2]|0;
    $29 = HEAP32[$2>>2]|0;
    $30 = (($29) + ($28<<2)|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = (($34) + ($32<<2)|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$aOut>>2]|0;
    $38 = HEAP32[$nOut>>2]|0;
    $39 = (_fts3DoclistOrMerge($27,$31,$36,$37,$38,$aNew,$nNew)|0);
    HEAP32[$rc>>2] = $39;
    $40 = HEAP32[$rc>>2]|0;
    $41 = ($40|0)!=(0);
    if ($41) {
     label = 7;
     break L1;
    }
    $44 = HEAP32[$i>>2]|0;
    $45 = HEAP32[$2>>2]|0;
    $46 = (($45) + ($44<<2)|0);
    $47 = HEAP32[$46>>2]|0;
    _sqlite3_free($47);
    $48 = HEAP32[$aOut>>2]|0;
    _sqlite3_free($48);
    $49 = HEAP32[$i>>2]|0;
    $50 = HEAP32[$2>>2]|0;
    $51 = (($50) + ($49<<2)|0);
    HEAP32[$51>>2] = 0;
    $52 = HEAP32[$aNew>>2]|0;
    HEAP32[$aOut>>2] = $52;
    $53 = HEAP32[$nNew>>2]|0;
    HEAP32[$nOut>>2] = $53;
   }
  } while(0);
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 7) {
  $42 = HEAP32[$aOut>>2]|0;
  _sqlite3_free($42);
  $43 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $43;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 else if ((label|0) == 10) {
  $56 = HEAP32[$aOut>>2]|0;
  $57 = HEAP32[$2>>2]|0;
  HEAP32[$57>>2] = $56;
  $58 = HEAP32[$nOut>>2]|0;
  $59 = HEAP32[$2>>2]|0;
  $60 = ((($59)) + 64|0);
  HEAP32[$60>>2] = $58;
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _fts3DoclistOrMerge($bDescDoclist,$a1,$n1,$a2,$n2,$paOut,$pnOut) {
 $bDescDoclist = $bDescDoclist|0;
 $a1 = $a1|0;
 $n1 = $n1|0;
 $a2 = $a2|0;
 $n2 = $n2|0;
 $paOut = $paOut|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOut = 0, $bFirstOut = 0, $i1 = 0, $i2 = 0, $iDiff = 0, $iPrev = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $p = 0, $p1 = 0, $p2 = 0, $pEnd1 = 0, $pEnd2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $i1 = sp + 24|0;
 $i2 = sp + 16|0;
 $iPrev = sp + 8|0;
 $pEnd1 = sp + 56|0;
 $pEnd2 = sp + 52|0;
 $p1 = sp + 48|0;
 $p2 = sp + 44|0;
 $p = sp + 40|0;
 $aOut = sp + 36|0;
 $bFirstOut = sp + 32|0;
 $iDiff = sp;
 HEAP32[$1>>2] = $bDescDoclist;
 HEAP32[$2>>2] = $a1;
 HEAP32[$3>>2] = $n1;
 HEAP32[$4>>2] = $a2;
 HEAP32[$5>>2] = $n2;
 HEAP32[$6>>2] = $paOut;
 HEAP32[$7>>2] = $pnOut;
 $8 = $i1;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 $12 = $i2;
 $13 = $12;
 HEAP32[$13>>2] = 0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = 0;
 $16 = $iPrev;
 $17 = $16;
 HEAP32[$17>>2] = 0;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($21) + ($20)|0);
 HEAP32[$pEnd1>>2] = $22;
 $23 = HEAP32[$5>>2]|0;
 $24 = HEAP32[$4>>2]|0;
 $25 = (($24) + ($23)|0);
 HEAP32[$pEnd2>>2] = $25;
 $26 = HEAP32[$2>>2]|0;
 HEAP32[$p1>>2] = $26;
 $27 = HEAP32[$4>>2]|0;
 HEAP32[$p2>>2] = $27;
 HEAP32[$bFirstOut>>2] = 0;
 $28 = HEAP32[$6>>2]|0;
 HEAP32[$28>>2] = 0;
 $29 = HEAP32[$7>>2]|0;
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP32[$5>>2]|0;
 $32 = (($30) + ($31))|0;
 $33 = (($32) + 10)|0;
 $34 = (($33) - 1)|0;
 $35 = (_sqlite3_malloc($34)|0);
 HEAP32[$aOut>>2] = $35;
 $36 = HEAP32[$aOut>>2]|0;
 $37 = ($36|0)!=(0|0);
 if (!($37)) {
  HEAP32[$0>>2] = 7;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $38 = HEAP32[$aOut>>2]|0;
 HEAP32[$p>>2] = $38;
 $39 = HEAP32[$pEnd1>>2]|0;
 _fts3GetDeltaVarint3($p1,$39,0,$i1);
 $40 = HEAP32[$pEnd2>>2]|0;
 _fts3GetDeltaVarint3($p2,$40,0,$i2);
 while(1) {
  $41 = HEAP32[$p1>>2]|0;
  $42 = ($41|0)!=(0|0);
  $43 = HEAP32[$p2>>2]|0;
  $44 = ($43|0)!=(0|0);
  $45 = $42 ? 1 : $44;
  if (!($45)) {
   break;
  }
  $46 = HEAP32[$1>>2]|0;
  $47 = ($46|0)!=(0);
  $48 = $47 ? -1 : 1;
  $49 = ($48|0)<(0);
  $50 = $49 << 31 >> 31;
  $51 = $i1;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($51) + 4)|0;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = $i2;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_i64Subtract(($53|0),($56|0),($59|0),($62|0))|0);
  $64 = tempRet0;
  $65 = (___muldi3(($48|0),($50|0),($63|0),($64|0))|0);
  $66 = tempRet0;
  $67 = $iDiff;
  $68 = $67;
  HEAP32[$68>>2] = $65;
  $69 = (($67) + 4)|0;
  $70 = $69;
  HEAP32[$70>>2] = $66;
  $71 = HEAP32[$p2>>2]|0;
  $72 = ($71|0)!=(0|0);
  $73 = HEAP32[$p1>>2]|0;
  $74 = ($73|0)!=(0|0);
  $or$cond = $72 & $74;
  $75 = $iDiff;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = ($77|0)==(0);
  $82 = ($80|0)==(0);
  $83 = $81 & $82;
  $or$cond3 = $or$cond & $83;
  if ($or$cond3) {
   $84 = HEAP32[$1>>2]|0;
   $85 = $i1;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = (($85) + 4)|0;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   _fts3PutDeltaVarint3($p,$84,$iPrev,$bFirstOut,$87,$90);
   _fts3PoslistMerge($p,$p1,$p2);
   $91 = HEAP32[$pEnd1>>2]|0;
   $92 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p1,$91,$92,$i1);
   $93 = HEAP32[$pEnd2>>2]|0;
   $94 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p2,$93,$94,$i2);
   continue;
  }
  $95 = HEAP32[$p2>>2]|0;
  $96 = ($95|0)!=(0|0);
  if ($96) {
   $97 = HEAP32[$p1>>2]|0;
   $98 = ($97|0)!=(0|0);
   $99 = $iDiff;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = (($99) + 4)|0;
   $103 = $102;
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)<(0);
   $or$cond5 = $98 & $105;
   if (!($or$cond5)) {
    $115 = HEAP32[$1>>2]|0;
    $116 = $i2;
    $117 = $116;
    $118 = HEAP32[$117>>2]|0;
    $119 = (($116) + 4)|0;
    $120 = $119;
    $121 = HEAP32[$120>>2]|0;
    _fts3PutDeltaVarint3($p,$115,$iPrev,$bFirstOut,$118,$121);
    _fts3PoslistCopy($p,$p2);
    $122 = HEAP32[$pEnd2>>2]|0;
    $123 = HEAP32[$1>>2]|0;
    _fts3GetDeltaVarint3($p2,$122,$123,$i2);
    continue;
   }
  }
  $106 = HEAP32[$1>>2]|0;
  $107 = $i1;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = (($107) + 4)|0;
  $111 = $110;
  $112 = HEAP32[$111>>2]|0;
  _fts3PutDeltaVarint3($p,$106,$iPrev,$bFirstOut,$109,$112);
  _fts3PoslistCopy($p,$p1);
  $113 = HEAP32[$pEnd1>>2]|0;
  $114 = HEAP32[$1>>2]|0;
  _fts3GetDeltaVarint3($p1,$113,$114,$i1);
 }
 $124 = HEAP32[$aOut>>2]|0;
 $125 = HEAP32[$6>>2]|0;
 HEAP32[$125>>2] = $124;
 $126 = HEAP32[$p>>2]|0;
 $127 = HEAP32[$aOut>>2]|0;
 $128 = $126;
 $129 = $127;
 $130 = (($128) - ($129))|0;
 $131 = HEAP32[$7>>2]|0;
 HEAP32[$131>>2] = $130;
 HEAP32[$0>>2] = 0;
 $132 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _fts3GetDeltaVarint3($pp,$pEnd,$bDescIdx,$pVal) {
 $pp = $pp|0;
 $pEnd = $pEnd|0;
 $bDescIdx = $bDescIdx|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pEnd;
 HEAP32[$2>>2] = $bDescIdx;
 HEAP32[$3>>2] = $pVal;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5>>>0)>=($6>>>0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  HEAP32[$8>>2] = 0;
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3Fts3GetVarint($9,$iVal)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($10)|0);
 HEAP32[$11>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)!=(0);
 $16 = $iVal;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 if ($15) {
  $29 = (_i64Subtract(($25|0),($28|0),($18|0),($21|0))|0);
  $30 = tempRet0;
  $31 = $22;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
  STACKTOP = sp;return;
 } else {
  $35 = (_i64Add(($25|0),($28|0),($18|0),($21|0))|0);
  $36 = tempRet0;
  $37 = $22;
  $38 = $37;
  HEAP32[$38>>2] = $35;
  $39 = (($37) + 4)|0;
  $40 = $39;
  HEAP32[$40>>2] = $36;
  STACKTOP = sp;return;
 }
}
function _fts3PutDeltaVarint3($pp,$bDescIdx,$piPrev,$pbFirst,$0,$1) {
 $pp = $pp|0;
 $bDescIdx = $bDescIdx|0;
 $piPrev = $piPrev|0;
 $pbFirst = $pbFirst|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $iWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 8|0;
 $iWrite = sp;
 HEAP32[$2>>2] = $pp;
 HEAP32[$3>>2] = $bDescIdx;
 HEAP32[$4>>2] = $piPrev;
 HEAP32[$5>>2] = $pbFirst;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  label = 3;
 } else {
  $13 = HEAP32[$5>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0);
  if ($15) {
   label = 3;
  } else {
   $35 = HEAP32[$4>>2]|0;
   $36 = $35;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($36) + 4)|0;
   $40 = $39;
   $41 = HEAP32[$40>>2]|0;
   $42 = $6;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_i64Subtract(($38|0),($41|0),($44|0),($47|0))|0);
   $49 = tempRet0;
   $50 = $iWrite;
   $51 = $50;
   HEAP32[$51>>2] = $48;
   $52 = (($50) + 4)|0;
   $53 = $52;
   HEAP32[$53>>2] = $49;
  }
 }
 if ((label|0) == 3) {
  $16 = $6;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  $23 = $22;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (_i64Subtract(($18|0),($21|0),($25|0),($28|0))|0);
  $30 = tempRet0;
  $31 = $iWrite;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
 }
 $54 = HEAP32[$2>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = $iWrite;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($56) + 4)|0;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = (_sqlite3Fts3PutVarint($55,$58,$61)|0);
 $63 = HEAP32[$2>>2]|0;
 $64 = HEAP32[$63>>2]|0;
 $65 = (($64) + ($62)|0);
 HEAP32[$63>>2] = $65;
 $66 = $6;
 $67 = $66;
 $68 = HEAP32[$67>>2]|0;
 $69 = (($66) + 4)|0;
 $70 = $69;
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$4>>2]|0;
 $73 = $72;
 $74 = $73;
 HEAP32[$74>>2] = $68;
 $75 = (($73) + 4)|0;
 $76 = $75;
 HEAP32[$76>>2] = $71;
 $77 = HEAP32[$5>>2]|0;
 HEAP32[$77>>2] = 1;
 STACKTOP = sp;return;
}
function _fts3PoslistMerge($pp,$pp1,$pp2) {
 $pp = $pp|0;
 $pp1 = $pp1|0;
 $pp2 = $pp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i1 = 0, $i2 = 0, $iCol1 = 0, $iCol2 = 0, $iPrev = 0, $n = 0, $p = 0, $p1 = 0, $p2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $p = sp + 44|0;
 $p1 = sp + 40|0;
 $p2 = sp + 36|0;
 $iCol1 = sp + 32|0;
 $iCol2 = sp + 28|0;
 $i1 = sp + 16|0;
 $i2 = sp + 8|0;
 $iPrev = sp;
 $n = sp + 24|0;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pp1;
 HEAP32[$2>>2] = $pp2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p1>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p2>>2] = $8;
 L1: while(1) {
  $9 = HEAP32[$p1>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   $13 = HEAP32[$p2>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    break;
   }
  }
  $17 = HEAP32[$p1>>2]|0;
  $18 = HEAP8[$17>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = ($19|0)==(1);
  $21 = HEAP32[$p1>>2]|0;
  do {
   if ($20) {
    $22 = ((($21)) + 1|0);
    $23 = HEAP8[$22>>0]|0;
    $24 = $23&255;
    $25 = $24 & 128;
    $26 = ($25|0)!=(0);
    $27 = HEAP32[$p1>>2]|0;
    $28 = ((($27)) + 1|0);
    if ($26) {
     (_sqlite3Fts3GetVarint32($28,$iCol1)|0);
     break;
    } else {
     $29 = HEAP8[$28>>0]|0;
     $30 = $29&255;
     HEAP32[$iCol1>>2] = $30;
     break;
    }
   } else {
    $31 = HEAP8[$21>>0]|0;
    $32 = $31 << 24 >> 24;
    $33 = ($32|0)==(0);
    if ($33) {
     HEAP32[$iCol1>>2] = 2147483647;
     break;
    } else {
     HEAP32[$iCol1>>2] = 0;
     break;
    }
   }
  } while(0);
  $34 = HEAP32[$p2>>2]|0;
  $35 = HEAP8[$34>>0]|0;
  $36 = $35 << 24 >> 24;
  $37 = ($36|0)==(1);
  $38 = HEAP32[$p2>>2]|0;
  do {
   if ($37) {
    $39 = ((($38)) + 1|0);
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    $42 = $41 & 128;
    $43 = ($42|0)!=(0);
    $44 = HEAP32[$p2>>2]|0;
    $45 = ((($44)) + 1|0);
    if ($43) {
     (_sqlite3Fts3GetVarint32($45,$iCol2)|0);
     break;
    } else {
     $46 = HEAP8[$45>>0]|0;
     $47 = $46&255;
     HEAP32[$iCol2>>2] = $47;
     break;
    }
   } else {
    $48 = HEAP8[$38>>0]|0;
    $49 = $48 << 24 >> 24;
    $50 = ($49|0)==(0);
    if ($50) {
     HEAP32[$iCol2>>2] = 2147483647;
     break;
    } else {
     HEAP32[$iCol2>>2] = 0;
     break;
    }
   }
  } while(0);
  $51 = HEAP32[$iCol1>>2]|0;
  $52 = HEAP32[$iCol2>>2]|0;
  $53 = ($51|0)==($52|0);
  if (!($53)) {
   $168 = HEAP32[$iCol1>>2]|0;
   $169 = HEAP32[$iCol2>>2]|0;
   $170 = ($168|0)<($169|0);
   if ($170) {
    $171 = HEAP32[$iCol1>>2]|0;
    $172 = (_fts3PutColNumber($p,$171)|0);
    $173 = HEAP32[$p1>>2]|0;
    $174 = (($173) + ($172)|0);
    HEAP32[$p1>>2] = $174;
    _fts3ColumnlistCopy($p,$p1);
    continue;
   } else {
    $175 = HEAP32[$iCol2>>2]|0;
    $176 = (_fts3PutColNumber($p,$175)|0);
    $177 = HEAP32[$p2>>2]|0;
    $178 = (($177) + ($176)|0);
    HEAP32[$p2>>2] = $178;
    _fts3ColumnlistCopy($p,$p2);
    continue;
   }
  }
  $54 = $i1;
  $55 = $54;
  HEAP32[$55>>2] = 0;
  $56 = (($54) + 4)|0;
  $57 = $56;
  HEAP32[$57>>2] = 0;
  $58 = $i2;
  $59 = $58;
  HEAP32[$59>>2] = 0;
  $60 = (($58) + 4)|0;
  $61 = $60;
  HEAP32[$61>>2] = 0;
  $62 = $iPrev;
  $63 = $62;
  HEAP32[$63>>2] = 0;
  $64 = (($62) + 4)|0;
  $65 = $64;
  HEAP32[$65>>2] = 0;
  $66 = HEAP32[$iCol1>>2]|0;
  $67 = (_fts3PutColNumber($p,$66)|0);
  HEAP32[$n>>2] = $67;
  $68 = HEAP32[$n>>2]|0;
  $69 = HEAP32[$p1>>2]|0;
  $70 = (($69) + ($68)|0);
  HEAP32[$p1>>2] = $70;
  $71 = HEAP32[$n>>2]|0;
  $72 = HEAP32[$p2>>2]|0;
  $73 = (($72) + ($71)|0);
  HEAP32[$p2>>2] = $73;
  _fts3GetDeltaVarint($p1,$i1);
  _fts3GetDeltaVarint($p2,$i2);
  while(1) {
   $74 = $i1;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($74) + 4)|0;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = $i2;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (($80) + 4)|0;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = ($79|0)<($85|0);
   $87 = ($76>>>0)<($82>>>0);
   $88 = ($79|0)==($85|0);
   $89 = $88 & $87;
   $90 = $86 | $89;
   $91 = $i1;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = (($91) + 4)|0;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = $i2;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = (($97) + 4)|0;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = $90 ? $93 : $99;
   $104 = $90 ? $96 : $102;
   _fts3PutDeltaVarint($p,$iPrev,$103,$104);
   $105 = $iPrev;
   $106 = $105;
   $107 = HEAP32[$106>>2]|0;
   $108 = (($105) + 4)|0;
   $109 = $108;
   $110 = HEAP32[$109>>2]|0;
   $111 = (_i64Subtract(($107|0),($110|0),2,0)|0);
   $112 = tempRet0;
   $113 = $iPrev;
   $114 = $113;
   HEAP32[$114>>2] = $111;
   $115 = (($113) + 4)|0;
   $116 = $115;
   HEAP32[$116>>2] = $112;
   $117 = $i1;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $120 = (($117) + 4)|0;
   $121 = $120;
   $122 = HEAP32[$121>>2]|0;
   $123 = $i2;
   $124 = $123;
   $125 = HEAP32[$124>>2]|0;
   $126 = (($123) + 4)|0;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = ($119|0)==($125|0);
   $130 = ($122|0)==($128|0);
   $131 = $129 & $130;
   do {
    if ($131) {
     _fts3ReadNextPos($p1,$i1);
     _fts3ReadNextPos($p2,$i2);
    } else {
     $132 = $i1;
     $133 = $132;
     $134 = HEAP32[$133>>2]|0;
     $135 = (($132) + 4)|0;
     $136 = $135;
     $137 = HEAP32[$136>>2]|0;
     $138 = $i2;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (($138) + 4)|0;
     $142 = $141;
     $143 = HEAP32[$142>>2]|0;
     $144 = ($137|0)<($143|0);
     $145 = ($134>>>0)<($140>>>0);
     $146 = ($137|0)==($143|0);
     $147 = $146 & $145;
     $148 = $144 | $147;
     if ($148) {
      _fts3ReadNextPos($p1,$i1);
      break;
     } else {
      _fts3ReadNextPos($p2,$i2);
      break;
     }
    }
   } while(0);
   $149 = $i1;
   $150 = $149;
   $151 = HEAP32[$150>>2]|0;
   $152 = (($149) + 4)|0;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = ($151|0)!=(2147483647);
   $156 = ($154|0)!=(0);
   $157 = $155 | $156;
   $158 = $i2;
   $159 = $158;
   $160 = HEAP32[$159>>2]|0;
   $161 = (($158) + 4)|0;
   $162 = $161;
   $163 = HEAP32[$162>>2]|0;
   $164 = ($160|0)!=(2147483647);
   $165 = ($163|0)!=(0);
   $166 = $164 | $165;
   $167 = $157 ? 1 : $166;
   if (!($167)) {
    continue L1;
   }
  }
 }
 $179 = HEAP32[$p>>2]|0;
 $180 = ((($179)) + 1|0);
 HEAP32[$p>>2] = $180;
 HEAP8[$179>>0] = 0;
 $181 = HEAP32[$p>>2]|0;
 $182 = HEAP32[$0>>2]|0;
 HEAP32[$182>>2] = $181;
 $183 = HEAP32[$p1>>2]|0;
 $184 = ((($183)) + 1|0);
 $185 = HEAP32[$1>>2]|0;
 HEAP32[$185>>2] = $184;
 $186 = HEAP32[$p2>>2]|0;
 $187 = ((($186)) + 1|0);
 $188 = HEAP32[$2>>2]|0;
 HEAP32[$188>>2] = $187;
 STACKTOP = sp;return;
}
function _fts3PutColNumber($pp,$iCol) {
 $pp = $pp|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$n>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  $18 = HEAP32[$n>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 1|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)<(0);
 $10 = $9 << 31 >> 31;
 $11 = (_sqlite3Fts3PutVarint($7,$8,$10)|0);
 $12 = (1 + ($11))|0;
 HEAP32[$n>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 HEAP8[$13>>0] = 1;
 $14 = HEAP32[$n>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 $16 = (($15) + ($14)|0);
 $17 = HEAP32[$0>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3GetDeltaVarint($pp,$pVal) {
 $pp = $pp|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3Fts3GetVarint($3,$iVal)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($4)|0);
 HEAP32[$5>>2] = $7;
 $8 = $iVal;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (_i64Add(($17|0),($20|0),($10|0),($13|0))|0);
 $22 = tempRet0;
 $23 = $14;
 $24 = $23;
 HEAP32[$24>>2] = $21;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 STACKTOP = sp;return;
}
function _fts3PutDeltaVarint($pp,$piPrev,$0,$1) {
 $pp = $pp|0;
 $piPrev = $piPrev|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$2>>2] = $pp;
 HEAP32[$3>>2] = $piPrev;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = $17;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (_i64Subtract(($13|0),($16|0),($20|0),($23|0))|0);
 $25 = tempRet0;
 $26 = (_sqlite3Fts3PutVarint($10,$24,$25)|0);
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($26)|0);
 HEAP32[$27>>2] = $29;
 $30 = $4;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = $36;
 $38 = $37;
 HEAP32[$38>>2] = $32;
 $39 = (($37) + 4)|0;
 $40 = $39;
 HEAP32[$40>>2] = $35;
 STACKTOP = sp;return;
}
function _fts3ReadNextPos($pp,$pi) {
 $pp = $pp|0;
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pi;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = $5 & 254;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  _fts3GetDeltaVarint($8,$9);
  $10 = HEAP32[$1>>2]|0;
  $11 = $10;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($11) + 4)|0;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (_i64Subtract(($13|0),($16|0),2,0)|0);
  $18 = tempRet0;
  $19 = $10;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
  STACKTOP = sp;return;
 } else {
  $23 = HEAP32[$1>>2]|0;
  $24 = $23;
  $25 = $24;
  HEAP32[$25>>2] = 2147483647;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = 0;
  STACKTOP = sp;return;
 }
}
function _fts3DoclistPhraseMerge($bDescDoclist,$nDist,$aLeft,$nLeft,$paRight,$pnRight) {
 $bDescDoclist = $bDescDoclist|0;
 $nDist = $nDist|0;
 $aLeft = $aLeft|0;
 $nLeft = $nLeft|0;
 $paRight = $paRight|0;
 $pnRight = $pnRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $aOut = 0, $aRight = 0, $bFirstOut = 0, $bFirstOutSave = 0, $i1 = 0, $i2 = 0, $iDiff = 0, $iPrev = 0, $iPrevSave = 0, $p = 0, $p1 = 0, $p2 = 0, $pEnd1 = 0, $pEnd2 = 0, $pSave = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 80|0;
 $i1 = sp + 32|0;
 $i2 = sp + 24|0;
 $iPrev = sp + 16|0;
 $aRight = sp + 76|0;
 $pEnd1 = sp + 72|0;
 $pEnd2 = sp + 68|0;
 $p1 = sp + 64|0;
 $p2 = sp + 60|0;
 $p = sp + 56|0;
 $bFirstOut = sp + 52|0;
 $aOut = sp + 48|0;
 $iDiff = sp + 8|0;
 $pSave = sp + 44|0;
 $iPrevSave = sp;
 $bFirstOutSave = sp + 40|0;
 HEAP32[$1>>2] = $bDescDoclist;
 HEAP32[$2>>2] = $nDist;
 HEAP32[$3>>2] = $aLeft;
 HEAP32[$4>>2] = $nLeft;
 HEAP32[$5>>2] = $paRight;
 HEAP32[$6>>2] = $pnRight;
 $7 = $i1;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = $i2;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = $iPrev;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$aRight>>2] = $20;
 $21 = HEAP32[$4>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = (($22) + ($21)|0);
 HEAP32[$pEnd1>>2] = $23;
 $24 = HEAP32[$6>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$aRight>>2]|0;
 $27 = (($26) + ($25)|0);
 HEAP32[$pEnd2>>2] = $27;
 $28 = HEAP32[$3>>2]|0;
 HEAP32[$p1>>2] = $28;
 $29 = HEAP32[$aRight>>2]|0;
 HEAP32[$p2>>2] = $29;
 HEAP32[$bFirstOut>>2] = 0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$6>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + 10)|0;
  $35 = (_sqlite3_malloc($34)|0);
  HEAP32[$aOut>>2] = $35;
  $36 = HEAP32[$aOut>>2]|0;
  $37 = ($36|0)==(0|0);
  if ($37) {
   HEAP32[$0>>2] = 7;
   $141 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($141|0);
  }
 } else {
  $38 = HEAP32[$aRight>>2]|0;
  HEAP32[$aOut>>2] = $38;
 }
 $39 = HEAP32[$aOut>>2]|0;
 HEAP32[$p>>2] = $39;
 $40 = HEAP32[$pEnd1>>2]|0;
 _fts3GetDeltaVarint3($p1,$40,0,$i1);
 $41 = HEAP32[$pEnd2>>2]|0;
 _fts3GetDeltaVarint3($p2,$41,0,$i2);
 while(1) {
  $42 = HEAP32[$p1>>2]|0;
  $43 = ($42|0)!=(0|0);
  $44 = HEAP32[$p2>>2]|0;
  $45 = ($44|0)!=(0|0);
  $46 = $43 ? $45 : 0;
  if (!($46)) {
   break;
  }
  $47 = HEAP32[$1>>2]|0;
  $48 = ($47|0)!=(0);
  $49 = $48 ? -1 : 1;
  $50 = ($49|0)<(0);
  $51 = $50 << 31 >> 31;
  $52 = $i1;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = $i2;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (_i64Subtract(($54|0),($57|0),($60|0),($63|0))|0);
  $65 = tempRet0;
  $66 = (___muldi3(($49|0),($51|0),($64|0),($65|0))|0);
  $67 = tempRet0;
  $68 = $iDiff;
  $69 = $68;
  HEAP32[$69>>2] = $66;
  $70 = (($68) + 4)|0;
  $71 = $70;
  HEAP32[$71>>2] = $67;
  $72 = $iDiff;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = ($74|0)==(0);
  $79 = ($77|0)==(0);
  $80 = $78 & $79;
  if ($80) {
   $81 = HEAP32[$p>>2]|0;
   HEAP32[$pSave>>2] = $81;
   $82 = $iPrev;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($82) + 4)|0;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = $iPrevSave;
   $89 = $88;
   HEAP32[$89>>2] = $84;
   $90 = (($88) + 4)|0;
   $91 = $90;
   HEAP32[$91>>2] = $87;
   $92 = HEAP32[$bFirstOut>>2]|0;
   HEAP32[$bFirstOutSave>>2] = $92;
   $93 = HEAP32[$1>>2]|0;
   $94 = $i1;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   _fts3PutDeltaVarint3($p,$93,$iPrev,$bFirstOut,$96,$99);
   $100 = HEAP32[$2>>2]|0;
   $101 = (_fts3PoslistPhraseMerge($p,$100,0,1,$p1,$p2)|0);
   $102 = (0)==($101|0);
   if ($102) {
    $103 = HEAP32[$pSave>>2]|0;
    HEAP32[$p>>2] = $103;
    $104 = $iPrevSave;
    $105 = $104;
    $106 = HEAP32[$105>>2]|0;
    $107 = (($104) + 4)|0;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = $iPrev;
    $111 = $110;
    HEAP32[$111>>2] = $106;
    $112 = (($110) + 4)|0;
    $113 = $112;
    HEAP32[$113>>2] = $109;
    $114 = HEAP32[$bFirstOutSave>>2]|0;
    HEAP32[$bFirstOut>>2] = $114;
   }
   $115 = HEAP32[$pEnd1>>2]|0;
   $116 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p1,$115,$116,$i1);
   $117 = HEAP32[$pEnd2>>2]|0;
   $118 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p2,$117,$118,$i2);
   continue;
  } else {
   $119 = $iDiff;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = (($119) + 4)|0;
   $123 = $122;
   $124 = HEAP32[$123>>2]|0;
   $125 = ($124|0)<(0);
   if ($125) {
    _fts3PoslistCopy(0,$p1);
    $126 = HEAP32[$pEnd1>>2]|0;
    $127 = HEAP32[$1>>2]|0;
    _fts3GetDeltaVarint3($p1,$126,$127,$i1);
    continue;
   } else {
    _fts3PoslistCopy(0,$p2);
    $128 = HEAP32[$pEnd2>>2]|0;
    $129 = HEAP32[$1>>2]|0;
    _fts3GetDeltaVarint3($p2,$128,$129,$i2);
    continue;
   }
  }
 }
 $130 = HEAP32[$p>>2]|0;
 $131 = HEAP32[$aOut>>2]|0;
 $132 = $130;
 $133 = $131;
 $134 = (($132) - ($133))|0;
 $135 = HEAP32[$6>>2]|0;
 HEAP32[$135>>2] = $134;
 $136 = HEAP32[$1>>2]|0;
 $137 = ($136|0)!=(0);
 if ($137) {
  $138 = HEAP32[$aRight>>2]|0;
  _sqlite3_free($138);
  $139 = HEAP32[$aOut>>2]|0;
  $140 = HEAP32[$5>>2]|0;
  HEAP32[$140>>2] = $139;
 }
 HEAP32[$0>>2] = 0;
 $141 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($141|0);
}
function _fts3PoslistPhraseMerge($pp,$nToken,$isSaveLeft,$isExact,$pp1,$pp2) {
 $pp = $pp|0;
 $nToken = $nToken|0;
 $isSaveLeft = $isSaveLeft|0;
 $isExact = $isExact|0;
 $pp1 = $pp1|0;
 $pp2 = $pp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCol1 = 0, $iCol2 = 0, $iPos1 = 0, $iPos2 = 0, $iPrev = 0, $iSave = 0, $p = 0, $p1 = 0, $p2 = 0, $pSave = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $6 = sp + 56|0;
 $p = sp + 52|0;
 $p1 = sp + 48|0;
 $p2 = sp + 44|0;
 $iCol1 = sp + 40|0;
 $iCol2 = sp + 36|0;
 $pSave = sp + 32|0;
 $iPrev = sp + 24|0;
 $iPos1 = sp + 16|0;
 $iPos2 = sp + 8|0;
 $iSave = sp;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $nToken;
 HEAP32[$3>>2] = $isSaveLeft;
 HEAP32[$4>>2] = $isExact;
 HEAP32[$5>>2] = $pp1;
 HEAP32[$6>>2] = $pp2;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$5>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p1>>2] = $10;
 $11 = HEAP32[$6>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p2>>2] = $12;
 HEAP32[$iCol1>>2] = 0;
 HEAP32[$iCol2>>2] = 0;
 $13 = HEAP32[$p1>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14 << 24 >> 24;
 $16 = ($15|0)==(1);
 if ($16) {
  $17 = HEAP32[$p1>>2]|0;
  $18 = ((($17)) + 1|0);
  HEAP32[$p1>>2] = $18;
  $19 = HEAP32[$p1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = $21 & 128;
  $23 = ($22|0)!=(0);
  $24 = HEAP32[$p1>>2]|0;
  if ($23) {
   $25 = (_sqlite3Fts3GetVarint32($24,$iCol1)|0);
   $30 = $25;
  } else {
   $26 = HEAP8[$24>>0]|0;
   $27 = $26&255;
   HEAP32[$iCol1>>2] = $27;
   $30 = 1;
  }
  $28 = HEAP32[$p1>>2]|0;
  $29 = (($28) + ($30)|0);
  HEAP32[$p1>>2] = $29;
 }
 $31 = HEAP32[$p2>>2]|0;
 $32 = HEAP8[$31>>0]|0;
 $33 = $32 << 24 >> 24;
 $34 = ($33|0)==(1);
 if ($34) {
  $35 = HEAP32[$p2>>2]|0;
  $36 = ((($35)) + 1|0);
  HEAP32[$p2>>2] = $36;
  $37 = HEAP32[$p2>>2]|0;
  $38 = HEAP8[$37>>0]|0;
  $39 = $38&255;
  $40 = $39 & 128;
  $41 = ($40|0)!=(0);
  $42 = HEAP32[$p2>>2]|0;
  if ($41) {
   $43 = (_sqlite3Fts3GetVarint32($42,$iCol2)|0);
   $48 = $43;
  } else {
   $44 = HEAP8[$42>>0]|0;
   $45 = $44&255;
   HEAP32[$iCol2>>2] = $45;
   $48 = 1;
  }
  $46 = HEAP32[$p2>>2]|0;
  $47 = (($46) + ($48)|0);
  HEAP32[$p2>>2] = $47;
 }
 while(1) {
  $49 = HEAP32[$iCol1>>2]|0;
  $50 = HEAP32[$iCol2>>2]|0;
  $51 = ($49|0)==($50|0);
  if (!($51)) {
   $315 = HEAP32[$iCol1>>2]|0;
   $316 = HEAP32[$iCol2>>2]|0;
   $317 = ($315|0)<($316|0);
   if ($317) {
    _fts3ColumnlistCopy(0,$p1);
    $318 = HEAP32[$p1>>2]|0;
    $319 = HEAP8[$318>>0]|0;
    $320 = $319 << 24 >> 24;
    $321 = (0)==($320|0);
    if ($321) {
     break;
    }
    $322 = HEAP32[$p1>>2]|0;
    $323 = ((($322)) + 1|0);
    HEAP32[$p1>>2] = $323;
    $324 = HEAP32[$p1>>2]|0;
    $325 = HEAP8[$324>>0]|0;
    $326 = $325&255;
    $327 = $326 & 128;
    $328 = ($327|0)!=(0);
    $329 = HEAP32[$p1>>2]|0;
    if ($328) {
     $330 = (_sqlite3Fts3GetVarint32($329,$iCol1)|0);
     $335 = $330;
    } else {
     $331 = HEAP8[$329>>0]|0;
     $332 = $331&255;
     HEAP32[$iCol1>>2] = $332;
     $335 = 1;
    }
    $333 = HEAP32[$p1>>2]|0;
    $334 = (($333) + ($335)|0);
    HEAP32[$p1>>2] = $334;
    continue;
   } else {
    _fts3ColumnlistCopy(0,$p2);
    $336 = HEAP32[$p2>>2]|0;
    $337 = HEAP8[$336>>0]|0;
    $338 = $337 << 24 >> 24;
    $339 = (0)==($338|0);
    if ($339) {
     break;
    }
    $340 = HEAP32[$p2>>2]|0;
    $341 = ((($340)) + 1|0);
    HEAP32[$p2>>2] = $341;
    $342 = HEAP32[$p2>>2]|0;
    $343 = HEAP8[$342>>0]|0;
    $344 = $343&255;
    $345 = $344 & 128;
    $346 = ($345|0)!=(0);
    $347 = HEAP32[$p2>>2]|0;
    if ($346) {
     $348 = (_sqlite3Fts3GetVarint32($347,$iCol2)|0);
     $353 = $348;
    } else {
     $349 = HEAP8[$347>>0]|0;
     $350 = $349&255;
     HEAP32[$iCol2>>2] = $350;
     $353 = 1;
    }
    $351 = HEAP32[$p2>>2]|0;
    $352 = (($351) + ($353)|0);
    HEAP32[$p2>>2] = $352;
    continue;
   }
  }
  $52 = HEAP32[$p>>2]|0;
  HEAP32[$pSave>>2] = $52;
  $53 = $iPrev;
  $54 = $53;
  HEAP32[$54>>2] = 0;
  $55 = (($53) + 4)|0;
  $56 = $55;
  HEAP32[$56>>2] = 0;
  $57 = $iPos1;
  $58 = $57;
  HEAP32[$58>>2] = 0;
  $59 = (($57) + 4)|0;
  $60 = $59;
  HEAP32[$60>>2] = 0;
  $61 = $iPos2;
  $62 = $61;
  HEAP32[$62>>2] = 0;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = 0;
  $65 = HEAP32[$iCol1>>2]|0;
  $66 = ($65|0)!=(0);
  if ($66) {
   $67 = HEAP32[$p>>2]|0;
   $68 = ((($67)) + 1|0);
   HEAP32[$p>>2] = $68;
   HEAP8[$67>>0] = 1;
   $69 = HEAP32[$p>>2]|0;
   $70 = HEAP32[$iCol1>>2]|0;
   $71 = ($70|0)<(0);
   $72 = $71 << 31 >> 31;
   $73 = (_sqlite3Fts3PutVarint($69,$70,$72)|0);
   $74 = HEAP32[$p>>2]|0;
   $75 = (($74) + ($73)|0);
   HEAP32[$p>>2] = $75;
  }
  _fts3GetDeltaVarint($p1,$iPos1);
  $76 = $iPos1;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  $79 = (($76) + 4)|0;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (_i64Subtract(($78|0),($81|0),2,0)|0);
  $83 = tempRet0;
  $84 = $iPos1;
  $85 = $84;
  HEAP32[$85>>2] = $82;
  $86 = (($84) + 4)|0;
  $87 = $86;
  HEAP32[$87>>2] = $83;
  _fts3GetDeltaVarint($p2,$iPos2);
  $88 = $iPos2;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (_i64Subtract(($90|0),($93|0),2,0)|0);
  $95 = tempRet0;
  $96 = $iPos2;
  $97 = $96;
  HEAP32[$97>>2] = $94;
  $98 = (($96) + 4)|0;
  $99 = $98;
  HEAP32[$99>>2] = $95;
  while(1) {
   $100 = $iPos2;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($100) + 4)|0;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = $iPos1;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (($106) + 4)|0;
   $110 = $109;
   $111 = HEAP32[$110>>2]|0;
   $112 = HEAP32[$2>>2]|0;
   $113 = ($112|0)<(0);
   $114 = $113 << 31 >> 31;
   $115 = (_i64Add(($108|0),($111|0),($112|0),($114|0))|0);
   $116 = tempRet0;
   $117 = ($102|0)==($115|0);
   $118 = ($105|0)==($116|0);
   $119 = $117 & $118;
   if ($119) {
    label = 19;
   } else {
    $120 = HEAP32[$4>>2]|0;
    $121 = ($120|0)==(0);
    if ($121) {
     $122 = $iPos2;
     $123 = $122;
     $124 = HEAP32[$123>>2]|0;
     $125 = (($122) + 4)|0;
     $126 = $125;
     $127 = HEAP32[$126>>2]|0;
     $128 = $iPos1;
     $129 = $128;
     $130 = HEAP32[$129>>2]|0;
     $131 = (($128) + 4)|0;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = ($127|0)>($133|0);
     $135 = ($124>>>0)>($130>>>0);
     $136 = ($127|0)==($133|0);
     $137 = $136 & $135;
     $138 = $134 | $137;
     if ($138) {
      $139 = $iPos2;
      $140 = $139;
      $141 = HEAP32[$140>>2]|0;
      $142 = (($139) + 4)|0;
      $143 = $142;
      $144 = HEAP32[$143>>2]|0;
      $145 = $iPos1;
      $146 = $145;
      $147 = HEAP32[$146>>2]|0;
      $148 = (($145) + 4)|0;
      $149 = $148;
      $150 = HEAP32[$149>>2]|0;
      $151 = HEAP32[$2>>2]|0;
      $152 = ($151|0)<(0);
      $153 = $152 << 31 >> 31;
      $154 = (_i64Add(($147|0),($150|0),($151|0),($153|0))|0);
      $155 = tempRet0;
      $156 = ($144|0)<($155|0);
      $157 = ($141>>>0)<=($154>>>0);
      $158 = ($144|0)==($155|0);
      $159 = $158 & $157;
      $160 = $156 | $159;
      if ($160) {
       label = 19;
      }
     }
    }
   }
   if ((label|0) == 19) {
    label = 0;
    $161 = HEAP32[$3>>2]|0;
    $162 = ($161|0)!=(0);
    $163 = $iPos1;
    $164 = $163;
    $165 = HEAP32[$164>>2]|0;
    $166 = (($163) + 4)|0;
    $167 = $166;
    $168 = HEAP32[$167>>2]|0;
    $169 = $iPos2;
    $170 = $169;
    $171 = HEAP32[$170>>2]|0;
    $172 = (($169) + 4)|0;
    $173 = $172;
    $174 = HEAP32[$173>>2]|0;
    $175 = $162 ? $165 : $171;
    $176 = $162 ? $168 : $174;
    $177 = $iSave;
    $178 = $177;
    HEAP32[$178>>2] = $175;
    $179 = (($177) + 4)|0;
    $180 = $179;
    HEAP32[$180>>2] = $176;
    $181 = $iSave;
    $182 = $181;
    $183 = HEAP32[$182>>2]|0;
    $184 = (($181) + 4)|0;
    $185 = $184;
    $186 = HEAP32[$185>>2]|0;
    $187 = (_i64Add(($183|0),($186|0),2,0)|0);
    $188 = tempRet0;
    _fts3PutDeltaVarint($p,$iPrev,$187,$188);
    $189 = $iPrev;
    $190 = $189;
    $191 = HEAP32[$190>>2]|0;
    $192 = (($189) + 4)|0;
    $193 = $192;
    $194 = HEAP32[$193>>2]|0;
    $195 = (_i64Subtract(($191|0),($194|0),2,0)|0);
    $196 = tempRet0;
    $197 = $iPrev;
    $198 = $197;
    HEAP32[$198>>2] = $195;
    $199 = (($197) + 4)|0;
    $200 = $199;
    HEAP32[$200>>2] = $196;
    HEAP32[$pSave>>2] = 0;
   }
   $201 = HEAP32[$3>>2]|0;
   $202 = ($201|0)!=(0);
   if ($202) {
    label = 22;
   } else {
    $203 = $iPos2;
    $204 = $203;
    $205 = HEAP32[$204>>2]|0;
    $206 = (($203) + 4)|0;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = $iPos1;
    $210 = $209;
    $211 = HEAP32[$210>>2]|0;
    $212 = (($209) + 4)|0;
    $213 = $212;
    $214 = HEAP32[$213>>2]|0;
    $215 = HEAP32[$2>>2]|0;
    $216 = ($215|0)<(0);
    $217 = $216 << 31 >> 31;
    $218 = (_i64Add(($211|0),($214|0),($215|0),($217|0))|0);
    $219 = tempRet0;
    $220 = ($208|0)<($219|0);
    $221 = ($205>>>0)<=($218>>>0);
    $222 = ($208|0)==($219|0);
    $223 = $222 & $221;
    $224 = $220 | $223;
    if (!($224)) {
     label = 22;
    }
   }
   if ((label|0) == 22) {
    label = 0;
    $225 = $iPos2;
    $226 = $225;
    $227 = HEAP32[$226>>2]|0;
    $228 = (($225) + 4)|0;
    $229 = $228;
    $230 = HEAP32[$229>>2]|0;
    $231 = $iPos1;
    $232 = $231;
    $233 = HEAP32[$232>>2]|0;
    $234 = (($231) + 4)|0;
    $235 = $234;
    $236 = HEAP32[$235>>2]|0;
    $237 = ($230|0)<($236|0);
    $238 = ($227>>>0)<=($233>>>0);
    $239 = ($230|0)==($236|0);
    $240 = $239 & $238;
    $241 = $237 | $240;
    if (!($241)) {
     $259 = HEAP32[$p1>>2]|0;
     $260 = HEAP8[$259>>0]|0;
     $261 = $260 << 24 >> 24;
     $262 = $261 & 254;
     $263 = ($262|0)==(0);
     if ($263) {
      break;
     }
     _fts3GetDeltaVarint($p1,$iPos1);
     $264 = $iPos1;
     $265 = $264;
     $266 = HEAP32[$265>>2]|0;
     $267 = (($264) + 4)|0;
     $268 = $267;
     $269 = HEAP32[$268>>2]|0;
     $270 = (_i64Subtract(($266|0),($269|0),2,0)|0);
     $271 = tempRet0;
     $272 = $iPos1;
     $273 = $272;
     HEAP32[$273>>2] = $270;
     $274 = (($272) + 4)|0;
     $275 = $274;
     HEAP32[$275>>2] = $271;
     continue;
    }
   }
   $242 = HEAP32[$p2>>2]|0;
   $243 = HEAP8[$242>>0]|0;
   $244 = $243 << 24 >> 24;
   $245 = $244 & 254;
   $246 = ($245|0)==(0);
   if ($246) {
    break;
   }
   _fts3GetDeltaVarint($p2,$iPos2);
   $247 = $iPos2;
   $248 = $247;
   $249 = HEAP32[$248>>2]|0;
   $250 = (($247) + 4)|0;
   $251 = $250;
   $252 = HEAP32[$251>>2]|0;
   $253 = (_i64Subtract(($249|0),($252|0),2,0)|0);
   $254 = tempRet0;
   $255 = $iPos2;
   $256 = $255;
   HEAP32[$256>>2] = $253;
   $257 = (($255) + 4)|0;
   $258 = $257;
   HEAP32[$258>>2] = $254;
  }
  $276 = HEAP32[$pSave>>2]|0;
  $277 = ($276|0)!=(0|0);
  if ($277) {
   $278 = HEAP32[$pSave>>2]|0;
   HEAP32[$p>>2] = $278;
  }
  _fts3ColumnlistCopy(0,$p1);
  _fts3ColumnlistCopy(0,$p2);
  $279 = HEAP32[$p1>>2]|0;
  $280 = HEAP8[$279>>0]|0;
  $281 = $280 << 24 >> 24;
  $282 = (0)==($281|0);
  if ($282) {
   break;
  }
  $283 = HEAP32[$p2>>2]|0;
  $284 = HEAP8[$283>>0]|0;
  $285 = $284 << 24 >> 24;
  $286 = (0)==($285|0);
  if ($286) {
   break;
  }
  $287 = HEAP32[$p1>>2]|0;
  $288 = ((($287)) + 1|0);
  HEAP32[$p1>>2] = $288;
  $289 = HEAP32[$p1>>2]|0;
  $290 = HEAP8[$289>>0]|0;
  $291 = $290&255;
  $292 = $291 & 128;
  $293 = ($292|0)!=(0);
  $294 = HEAP32[$p1>>2]|0;
  if ($293) {
   $295 = (_sqlite3Fts3GetVarint32($294,$iCol1)|0);
   $300 = $295;
  } else {
   $296 = HEAP8[$294>>0]|0;
   $297 = $296&255;
   HEAP32[$iCol1>>2] = $297;
   $300 = 1;
  }
  $298 = HEAP32[$p1>>2]|0;
  $299 = (($298) + ($300)|0);
  HEAP32[$p1>>2] = $299;
  $301 = HEAP32[$p2>>2]|0;
  $302 = ((($301)) + 1|0);
  HEAP32[$p2>>2] = $302;
  $303 = HEAP32[$p2>>2]|0;
  $304 = HEAP8[$303>>0]|0;
  $305 = $304&255;
  $306 = $305 & 128;
  $307 = ($306|0)!=(0);
  $308 = HEAP32[$p2>>2]|0;
  if ($307) {
   $309 = (_sqlite3Fts3GetVarint32($308,$iCol2)|0);
   $314 = $309;
  } else {
   $310 = HEAP8[$308>>0]|0;
   $311 = $310&255;
   HEAP32[$iCol2>>2] = $311;
   $314 = 1;
  }
  $312 = HEAP32[$p2>>2]|0;
  $313 = (($312) + ($314)|0);
  HEAP32[$p2>>2] = $313;
 }
 _fts3PoslistCopy(0,$p2);
 _fts3PoslistCopy(0,$p1);
 $354 = HEAP32[$p1>>2]|0;
 $355 = HEAP32[$5>>2]|0;
 HEAP32[$355>>2] = $354;
 $356 = HEAP32[$p2>>2]|0;
 $357 = HEAP32[$6>>2]|0;
 HEAP32[$357>>2] = $356;
 $358 = HEAP32[$1>>2]|0;
 $359 = HEAP32[$358>>2]|0;
 $360 = HEAP32[$p>>2]|0;
 $361 = ($359|0)==($360|0);
 if ($361) {
  HEAP32[$0>>2] = 0;
  $366 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($366|0);
 } else {
  $362 = HEAP32[$p>>2]|0;
  $363 = ((($362)) + 1|0);
  HEAP32[$p>>2] = $363;
  HEAP8[$362>>0] = 0;
  $364 = HEAP32[$p>>2]|0;
  $365 = HEAP32[$1>>2]|0;
  HEAP32[$365>>2] = $364;
  HEAP32[$0>>2] = 1;
  $366 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($366|0);
 }
 return (0)|0;
}
function _fts3EvalPhraseStart($pCsr,$bOptOk,$p) {
 $pCsr = $pCsr|0;
 $bOptOk = $bOptOk|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bHaveIncr = 0, $bIncrOk = 0, $i = 0, $iCol = 0, $or$cond = 0, $pSegcsr = 0, $pTab = 0, $pToken = 0, $pToken1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pTab = sp + 32|0;
 $rc = sp + 28|0;
 $i = sp + 24|0;
 $bHaveIncr = sp + 20|0;
 $bIncrOk = sp + 16|0;
 $pToken = sp + 12|0;
 $iCol = sp + 8|0;
 $pToken1 = sp + 4|0;
 $pSegcsr = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $bOptOk;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bHaveIncr>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 52|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = HEAP32[$pTab>>2]|0;
  $12 = ((($11)) + 231|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($10|0)==($14|0);
  if ($15) {
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 64|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)<=(4);
   if ($19) {
    $20 = HEAP32[$2>>2]|0;
    $21 = ((($20)) + 64|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ($22|0)>(0);
    $25 = $23;
   } else {
    $25 = 0;
   }
  } else {
   $25 = 0;
  }
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 HEAP32[$bIncrOk>>2] = $24;
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$bIncrOk>>2]|0;
  $27 = ($26|0)==(1);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 64|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 72|0);
  $36 = (($35) + (($33*24)|0)|0);
  HEAP32[$pToken>>2] = $36;
  $37 = HEAP32[$pToken>>2]|0;
  $38 = ((($37)) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0);
  if ($40) {
   label = 11;
  } else {
   $41 = HEAP32[$pToken>>2]|0;
   $42 = ((($41)) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $45 = HEAP32[$pToken>>2]|0;
    $46 = ((($45)) + 20|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = ((($47)) + 36|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ($49|0)!=(0);
    if (!($50)) {
     label = 11;
    }
   }
  }
  if ((label|0) == 11) {
   label = 0;
   HEAP32[$bIncrOk>>2] = 0;
  }
  $51 = HEAP32[$pToken>>2]|0;
  $52 = ((($51)) + 20|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($53|0)!=(0|0);
  if ($54) {
   HEAP32[$bHaveIncr>>2] = 1;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$i>>2] = $56;
 }
 $57 = HEAP32[$bIncrOk>>2]|0;
 $58 = ($57|0)!=(0);
 $59 = HEAP32[$bHaveIncr>>2]|0;
 $60 = ($59|0)!=(0);
 $or$cond = $58 & $60;
 if (!($or$cond)) {
  $101 = HEAP32[$0>>2]|0;
  $102 = HEAP32[$2>>2]|0;
  $103 = (_fts3EvalPhraseLoad($101,$102)|0);
  HEAP32[$rc>>2] = $103;
  $104 = HEAP32[$2>>2]|0;
  $105 = ((($104)) + 40|0);
  HEAP32[$105>>2] = 0;
  $106 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $61 = HEAP32[$2>>2]|0;
 $62 = ((($61)) + 68|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = HEAP32[$pTab>>2]|0;
 $65 = ((($64)) + 24|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($63|0)>=($66|0);
 if ($67) {
  $71 = -1;
 } else {
  $68 = HEAP32[$2>>2]|0;
  $69 = ((($68)) + 68|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $70;
 }
 HEAP32[$iCol>>2] = $71;
 HEAP32[$i>>2] = 0;
 while(1) {
  $72 = HEAP32[$rc>>2]|0;
  $73 = ($72|0)==(0);
  if (!($73)) {
   break;
  }
  $74 = HEAP32[$i>>2]|0;
  $75 = HEAP32[$2>>2]|0;
  $76 = ((($75)) + 64|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($74|0)<($77|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$i>>2]|0;
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 72|0);
  $82 = (($81) + (($79*24)|0)|0);
  HEAP32[$pToken1>>2] = $82;
  $83 = HEAP32[$pToken1>>2]|0;
  $84 = ((($83)) + 20|0);
  $85 = HEAP32[$84>>2]|0;
  HEAP32[$pSegcsr>>2] = $85;
  $86 = HEAP32[$pSegcsr>>2]|0;
  $87 = ($86|0)!=(0|0);
  if ($87) {
   $88 = HEAP32[$pTab>>2]|0;
   $89 = HEAP32[$pSegcsr>>2]|0;
   $90 = HEAP32[$iCol>>2]|0;
   $91 = HEAP32[$pToken1>>2]|0;
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$pToken1>>2]|0;
   $94 = ((($93)) + 4|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (_sqlite3Fts3MsrIncrStart($88,$89,$90,$92,$95)|0);
   HEAP32[$rc>>2] = $96;
  }
  $97 = HEAP32[$i>>2]|0;
  $98 = (($97) + 1)|0;
  HEAP32[$i>>2] = $98;
 }
 $99 = HEAP32[$2>>2]|0;
 $100 = ((($99)) + 40|0);
 HEAP32[$100>>2] = 1;
 $106 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($106|0);
}
function _sqlite3Fts3MsrIncrStart($p,$pCsr,$iCol,$zTerm,$nTerm) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 $iCol = $iCol|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $i = 0, $nSegment = 0, $pSeg = 0, $rc = 0, $xCmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $nSegment = sp + 8|0;
 $xCmp = sp + 4|0;
 $pSeg = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nSegment>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 231|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(0);
 $14 = $13 ? 200 : 201;
 HEAP32[$xCmp>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = (_fts3SegReaderStart($15,$16,$17,$18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $22;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$nSegment>>2]|0;
  $25 = ($23|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($26<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pSeg>>2] = $30;
  $31 = HEAP32[$pSeg>>2]|0;
  $32 = ((($31)) + 40|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$pSeg>>2]|0;
  $36 = HEAP32[$4>>2]|0;
  $37 = HEAP32[$5>>2]|0;
  $38 = (_fts3SegReaderTermCmp($35,$36,$37)|0);
  $39 = ($38|0)!=(0);
  if ($39) {
   break;
  }
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$i>>2]|0;
 $43 = HEAP32[$2>>2]|0;
 $44 = ((($43)) + 8|0);
 HEAP32[$44>>2] = $42;
 HEAP32[$i>>2] = 0;
 while(1) {
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ((($46)) + 8|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($45|0)<($48|0);
  if (!($49)) {
   label = 13;
   break;
  }
  $50 = HEAP32[$1>>2]|0;
  $51 = HEAP32[$i>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($51<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = (_fts3SegReaderFirstDocid($50,$55)|0);
  HEAP32[$rc>>2] = $56;
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)!=(0);
  if ($58) {
   label = 11;
   break;
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 if ((label|0) == 11) {
  $59 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $59;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 else if ((label|0) == 13) {
  $62 = HEAP32[$2>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$xCmp>>2]|0;
  _fts3SegReaderSort($63,$64,$65,$66);
  $67 = HEAP32[$3>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  $69 = ((($68)) + 24|0);
  HEAP32[$69>>2] = $67;
  HEAP32[$0>>2] = 0;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 return (0)|0;
}
function _fts3EvalPhraseLoad($pCsr,$p) {
 $pCsr = $pCsr|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iToken = 0, $nThis = 0, $pTab = 0, $pThis = 0;
 var $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTab = sp + 20|0;
 $iToken = sp + 16|0;
 $rc = sp + 12|0;
 $pToken = sp + 8|0;
 $nThis = sp + 4|0;
 $pThis = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTab>>2] = $3;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iToken>>2] = 0;
 while(1) {
  $4 = HEAP32[$rc>>2]|0;
  $5 = ($4|0)==(0);
  if (!($5)) {
   label = 8;
   break;
  }
  $6 = HEAP32[$iToken>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 64|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$iToken>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 72|0);
  $14 = (($13) + (($11*24)|0)|0);
  HEAP32[$pToken>>2] = $14;
  $15 = HEAP32[$pToken>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   HEAP32[$nThis>>2] = 0;
   HEAP32[$pThis>>2] = 0;
   $19 = HEAP32[$pTab>>2]|0;
   $20 = HEAP32[$pToken>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 68|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_fts3TermSelect($19,$20,$23,$nThis,$pThis)|0);
   HEAP32[$rc>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    $27 = HEAP32[$pTab>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$iToken>>2]|0;
    $30 = HEAP32[$pThis>>2]|0;
    $31 = HEAP32[$nThis>>2]|0;
    $32 = (_fts3EvalPhraseMergeToken($27,$28,$29,$30,$31)|0);
    HEAP32[$rc>>2] = $32;
   }
  }
  $33 = HEAP32[$iToken>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$iToken>>2] = $34;
 }
 if ((label|0) == 8) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _fts3EvalNext($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pExpr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pExpr>>2] = $3;
 $4 = HEAP32[$pExpr>>2]|0;
 $5 = ($4|0)==(0|0);
 L1: do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 6|0);
   HEAP8[$7>>0] = 1;
  } else {
   while(1) {
    $8 = HEAP32[$0>>2]|0;
    $9 = ((($8)) + 7|0);
    $10 = HEAP8[$9>>0]|0;
    $11 = $10&255;
    $12 = ($11|0)==(0);
    if ($12) {
     $13 = HEAP32[$0>>2]|0;
     $14 = ((($13)) + 8|0);
     $15 = HEAP32[$14>>2]|0;
     (_sqlite3_reset($15)|0);
    }
    $16 = HEAP32[$0>>2]|0;
    $17 = HEAP32[$pExpr>>2]|0;
    _fts3EvalNextRow($16,$17,$rc);
    $18 = HEAP32[$pExpr>>2]|0;
    $19 = ((($18)) + 32|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 6|0);
    HEAP8[$22>>0] = $20;
    $23 = HEAP32[$0>>2]|0;
    $24 = ((($23)) + 7|0);
    HEAP8[$24>>0] = 1;
    $25 = HEAP32[$0>>2]|0;
    $26 = ((($25)) + 88|0);
    HEAP32[$26>>2] = 1;
    $27 = HEAP32[$pExpr>>2]|0;
    $28 = ((($27)) + 24|0);
    $29 = $28;
    $30 = $29;
    $31 = HEAP32[$30>>2]|0;
    $32 = (($29) + 4)|0;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 32|0);
    $37 = $36;
    $38 = $37;
    HEAP32[$38>>2] = $31;
    $39 = (($37) + 4)|0;
    $40 = $39;
    HEAP32[$40>>2] = $34;
    $41 = HEAP32[$0>>2]|0;
    $42 = ((($41)) + 6|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)==(0);
    if (!($45)) {
     break L1;
    }
    $46 = HEAP32[$0>>2]|0;
    $47 = (_sqlite3Fts3EvalTestDeferred($46,$rc)|0);
    $48 = ($47|0)!=(0);
    if (!($48)) {
     break;
    }
   }
  }
 } while(0);
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if (!($50)) {
  $105 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($105|0);
 }
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 52|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = ($54|0)==(0);
 if ($55) {
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 32|0);
  $58 = $57;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$0>>2]|0;
  $65 = ((($64)) + 80|0);
  $66 = $65;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($66) + 4)|0;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = ($63|0)>($71|0);
  $73 = ($60>>>0)>($68>>>0);
  $74 = ($63|0)==($71|0);
  $75 = $74 & $73;
  $76 = $72 | $75;
  if (!($76)) {
   label = 10;
  }
 } else {
  label = 10;
 }
 if ((label|0) == 10) {
  $77 = HEAP32[$0>>2]|0;
  $78 = ((($77)) + 52|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = ($80|0)!=(0);
  if (!($81)) {
   $105 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($105|0);
  }
  $82 = HEAP32[$0>>2]|0;
  $83 = ((($82)) + 32|0);
  $84 = $83;
  $85 = $84;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($84) + 4)|0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$0>>2]|0;
  $91 = ((($90)) + 72|0);
  $92 = $91;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($89|0)<($97|0);
  $99 = ($86>>>0)<($94>>>0);
  $100 = ($89|0)==($97|0);
  $101 = $100 & $99;
  $102 = $98 | $101;
  if (!($102)) {
   $105 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($105|0);
  }
 }
 $103 = HEAP32[$0>>2]|0;
 $104 = ((($103)) + 6|0);
 HEAP8[$104>>0] = 1;
 $105 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($105|0);
}
function _fts3EvalNextRow($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDescDoclist = 0, $iCmp = 0, $iDiff = 0, $or$cond = 0, $or$cond6 = 0, $or$cond8 = 0, $pDl = 0, $pDl1 = 0, $pLeft = 0, $pLeft2 = 0;
 var $pLeft4 = 0, $pPhrase = 0, $pRight = 0, $pRight3 = 0, $pRight5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $bDescDoclist = sp + 52|0;
 $pLeft = sp + 48|0;
 $pRight = sp + 44|0;
 $iDiff = sp + 8|0;
 $pDl = sp + 40|0;
 $pDl1 = sp + 36|0;
 $pLeft2 = sp + 32|0;
 $pRight3 = sp + 28|0;
 $iCmp = sp;
 $pLeft4 = sp + 24|0;
 $pRight5 = sp + 20|0;
 $pPhrase = sp + 16|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 HEAP32[$bDescDoclist>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 33|0);
 HEAP8[$11>>0] = 1;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 switch ($13|0) {
 case 3: case 1:  {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pLeft>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pRight>>2] = $19;
  $20 = HEAP32[$pLeft>>2]|0;
  $21 = ((($20)) + 34|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = ($22<<24>>24)!=(0);
  if ($23) {
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$pRight>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($24,$25,$26);
   $27 = HEAP32[$pRight>>2]|0;
   $28 = ((($27)) + 24|0);
   $29 = $28;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($29) + 4)|0;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 24|0);
   $37 = $36;
   $38 = $37;
   HEAP32[$38>>2] = $31;
   $39 = (($37) + 4)|0;
   $40 = $39;
   HEAP32[$40>>2] = $34;
   $41 = HEAP32[$pRight>>2]|0;
   $42 = ((($41)) + 32|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = ((($44)) + 32|0);
   HEAP8[$45>>0] = $43;
   STACKTOP = sp;return;
  }
  $46 = HEAP32[$pRight>>2]|0;
  $47 = ((($46)) + 34|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = ($48<<24>>24)!=(0);
  $50 = HEAP32[$0>>2]|0;
  $51 = HEAP32[$pLeft>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  _fts3EvalNextRow($50,$51,$52);
  if ($49) {
   $53 = HEAP32[$pLeft>>2]|0;
   $54 = ((($53)) + 24|0);
   $55 = $54;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($55) + 4)|0;
   $59 = $58;
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$1>>2]|0;
   $62 = ((($61)) + 24|0);
   $63 = $62;
   $64 = $63;
   HEAP32[$64>>2] = $57;
   $65 = (($63) + 4)|0;
   $66 = $65;
   HEAP32[$66>>2] = $60;
   $67 = HEAP32[$pLeft>>2]|0;
   $68 = ((($67)) + 32|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = HEAP32[$1>>2]|0;
   $71 = ((($70)) + 32|0);
   HEAP8[$71>>0] = $69;
   STACKTOP = sp;return;
  }
  $72 = HEAP32[$0>>2]|0;
  $73 = HEAP32[$pRight>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  _fts3EvalNextRow($72,$73,$74);
  while(1) {
   $75 = HEAP32[$pLeft>>2]|0;
   $76 = ((($75)) + 32|0);
   $77 = HEAP8[$76>>0]|0;
   $78 = ($77<<24>>24)!=(0);
   if ($78) {
    break;
   }
   $79 = HEAP32[$pRight>>2]|0;
   $80 = ((($79)) + 32|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = ($81<<24>>24)!=(0);
   if ($82) {
    break;
   }
   $83 = HEAP32[$2>>2]|0;
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)==(0);
   if (!($85)) {
    break;
   }
   $86 = HEAP32[$bDescDoclist>>2]|0;
   $87 = ($86|0)!=(0);
   $88 = $87 ? -1 : 1;
   $89 = ($88|0)<(0);
   $90 = $89 << 31 >> 31;
   $91 = HEAP32[$pLeft>>2]|0;
   $92 = ((($91)) + 24|0);
   $93 = $92;
   $94 = $93;
   $95 = HEAP32[$94>>2]|0;
   $96 = (($93) + 4)|0;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$pRight>>2]|0;
   $100 = ((($99)) + 24|0);
   $101 = $100;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = (($101) + 4)|0;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (_i64Subtract(($95|0),($98|0),($103|0),($106|0))|0);
   $108 = tempRet0;
   $109 = (___muldi3(($88|0),($90|0),($107|0),($108|0))|0);
   $110 = tempRet0;
   $111 = $iDiff;
   $112 = $111;
   HEAP32[$112>>2] = $109;
   $113 = (($111) + 4)|0;
   $114 = $113;
   HEAP32[$114>>2] = $110;
   $115 = $iDiff;
   $116 = $115;
   $117 = HEAP32[$116>>2]|0;
   $118 = (($115) + 4)|0;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = ($117|0)==(0);
   $122 = ($120|0)==(0);
   $123 = $121 & $122;
   if ($123) {
    break;
   }
   $124 = $iDiff;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = ($129|0)<(0);
   $131 = HEAP32[$0>>2]|0;
   if ($130) {
    $132 = HEAP32[$pLeft>>2]|0;
    $133 = HEAP32[$2>>2]|0;
    _fts3EvalNextRow($131,$132,$133);
    continue;
   } else {
    $134 = HEAP32[$pRight>>2]|0;
    $135 = HEAP32[$2>>2]|0;
    _fts3EvalNextRow($131,$134,$135);
    continue;
   }
  }
  $136 = HEAP32[$pLeft>>2]|0;
  $137 = ((($136)) + 24|0);
  $138 = $137;
  $139 = $138;
  $140 = HEAP32[$139>>2]|0;
  $141 = (($138) + 4)|0;
  $142 = $141;
  $143 = HEAP32[$142>>2]|0;
  $144 = HEAP32[$1>>2]|0;
  $145 = ((($144)) + 24|0);
  $146 = $145;
  $147 = $146;
  HEAP32[$147>>2] = $140;
  $148 = (($146) + 4)|0;
  $149 = $148;
  HEAP32[$149>>2] = $143;
  $150 = HEAP32[$pLeft>>2]|0;
  $151 = ((($150)) + 32|0);
  $152 = HEAP8[$151>>0]|0;
  $153 = $152&255;
  $154 = ($153|0)!=(0);
  if ($154) {
   $161 = 1;
  } else {
   $155 = HEAP32[$pRight>>2]|0;
   $156 = ((($155)) + 32|0);
   $157 = HEAP8[$156>>0]|0;
   $158 = $157&255;
   $159 = ($158|0)!=(0);
   $161 = $159;
  }
  $160 = $161&1;
  $162 = $160&255;
  $163 = HEAP32[$1>>2]|0;
  $164 = ((($163)) + 32|0);
  HEAP8[$164>>0] = $162;
  $165 = HEAP32[$1>>2]|0;
  $166 = HEAP32[$165>>2]|0;
  $167 = ($166|0)==(1);
  if (!($167)) {
   STACKTOP = sp;return;
  }
  $168 = HEAP32[$1>>2]|0;
  $169 = ((($168)) + 32|0);
  $170 = HEAP8[$169>>0]|0;
  $171 = $170&255;
  $172 = ($171|0)!=(0);
  if (!($172)) {
   STACKTOP = sp;return;
  }
  $173 = HEAP32[$pRight>>2]|0;
  $174 = ((($173)) + 20|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = ($175|0)!=(0|0);
  L35: do {
   if ($176) {
    $177 = HEAP32[$pRight>>2]|0;
    $178 = ((($177)) + 20|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = HEAP32[$179>>2]|0;
    $181 = ($180|0)!=(0|0);
    if ($181) {
     $182 = HEAP32[$pRight>>2]|0;
     $183 = ((($182)) + 20|0);
     $184 = HEAP32[$183>>2]|0;
     HEAP32[$pDl>>2] = $184;
     while(1) {
      $185 = HEAP32[$2>>2]|0;
      $186 = HEAP32[$185>>2]|0;
      $187 = ($186|0)==(0);
      if (!($187)) {
       break L35;
      }
      $188 = HEAP32[$pRight>>2]|0;
      $189 = ((($188)) + 32|0);
      $190 = HEAP8[$189>>0]|0;
      $191 = $190&255;
      $192 = ($191|0)==(0);
      if (!($192)) {
       break L35;
      }
      $193 = HEAP32[$pDl>>2]|0;
      $194 = ((($193)) + 28|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = HEAP32[$pDl>>2]|0;
      $197 = ((($196)) + 32|0);
      $198 = HEAP32[$197>>2]|0;
      _memset(($195|0),0,($198|0))|0;
      $199 = HEAP32[$0>>2]|0;
      $200 = HEAP32[$pRight>>2]|0;
      $201 = HEAP32[$2>>2]|0;
      _fts3EvalNextRow($199,$200,$201);
     }
    }
   }
  } while(0);
  $202 = HEAP32[$pLeft>>2]|0;
  $203 = ((($202)) + 20|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = ($204|0)!=(0|0);
  if (!($205)) {
   STACKTOP = sp;return;
  }
  $206 = HEAP32[$pLeft>>2]|0;
  $207 = ((($206)) + 20|0);
  $208 = HEAP32[$207>>2]|0;
  $209 = HEAP32[$208>>2]|0;
  $210 = ($209|0)!=(0|0);
  if (!($210)) {
   STACKTOP = sp;return;
  }
  $211 = HEAP32[$pLeft>>2]|0;
  $212 = ((($211)) + 20|0);
  $213 = HEAP32[$212>>2]|0;
  HEAP32[$pDl1>>2] = $213;
  while(1) {
   $214 = HEAP32[$2>>2]|0;
   $215 = HEAP32[$214>>2]|0;
   $216 = ($215|0)==(0);
   if (!($216)) {
    label = 53;
    break;
   }
   $217 = HEAP32[$pLeft>>2]|0;
   $218 = ((($217)) + 32|0);
   $219 = HEAP8[$218>>0]|0;
   $220 = $219&255;
   $221 = ($220|0)==(0);
   if (!($221)) {
    label = 53;
    break;
   }
   $222 = HEAP32[$pDl1>>2]|0;
   $223 = ((($222)) + 28|0);
   $224 = HEAP32[$223>>2]|0;
   $225 = HEAP32[$pDl1>>2]|0;
   $226 = ((($225)) + 32|0);
   $227 = HEAP32[$226>>2]|0;
   _memset(($224|0),0,($227|0))|0;
   $228 = HEAP32[$0>>2]|0;
   $229 = HEAP32[$pLeft>>2]|0;
   $230 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($228,$229,$230);
  }
  if ((label|0) == 53) {
   STACKTOP = sp;return;
  }
  break;
 }
 case 4:  {
  $231 = HEAP32[$1>>2]|0;
  $232 = ((($231)) + 12|0);
  $233 = HEAP32[$232>>2]|0;
  HEAP32[$pLeft2>>2] = $233;
  $234 = HEAP32[$1>>2]|0;
  $235 = ((($234)) + 16|0);
  $236 = HEAP32[$235>>2]|0;
  HEAP32[$pRight3>>2] = $236;
  $237 = HEAP32[$bDescDoclist>>2]|0;
  $238 = ($237|0)!=(0);
  $239 = $238 ? -1 : 1;
  $240 = ($239|0)<(0);
  $241 = $240 << 31 >> 31;
  $242 = HEAP32[$pLeft2>>2]|0;
  $243 = ((($242)) + 24|0);
  $244 = $243;
  $245 = $244;
  $246 = HEAP32[$245>>2]|0;
  $247 = (($244) + 4)|0;
  $248 = $247;
  $249 = HEAP32[$248>>2]|0;
  $250 = HEAP32[$pRight3>>2]|0;
  $251 = ((($250)) + 24|0);
  $252 = $251;
  $253 = $252;
  $254 = HEAP32[$253>>2]|0;
  $255 = (($252) + 4)|0;
  $256 = $255;
  $257 = HEAP32[$256>>2]|0;
  $258 = (_i64Subtract(($246|0),($249|0),($254|0),($257|0))|0);
  $259 = tempRet0;
  $260 = (___muldi3(($239|0),($241|0),($258|0),($259|0))|0);
  $261 = tempRet0;
  $262 = $iCmp;
  $263 = $262;
  HEAP32[$263>>2] = $260;
  $264 = (($262) + 4)|0;
  $265 = $264;
  HEAP32[$265>>2] = $261;
  $266 = HEAP32[$pRight3>>2]|0;
  $267 = ((($266)) + 32|0);
  $268 = HEAP8[$267>>0]|0;
  $269 = $268&255;
  $270 = ($269|0)!=(0);
  do {
   if ($270) {
    label = 33;
   } else {
    $271 = HEAP32[$pLeft2>>2]|0;
    $272 = ((($271)) + 32|0);
    $273 = HEAP8[$272>>0]|0;
    $274 = $273&255;
    $275 = ($274|0)==(0);
    $276 = $iCmp;
    $277 = $276;
    $278 = HEAP32[$277>>2]|0;
    $279 = (($276) + 4)|0;
    $280 = $279;
    $281 = HEAP32[$280>>2]|0;
    $282 = ($281|0)<(0);
    $or$cond = $275 & $282;
    if ($or$cond) {
     label = 33;
    } else {
     $286 = HEAP32[$pLeft2>>2]|0;
     $287 = ((($286)) + 32|0);
     $288 = HEAP8[$287>>0]|0;
     $289 = $288&255;
     $290 = ($289|0)!=(0);
     if (!($290)) {
      $291 = HEAP32[$pRight3>>2]|0;
      $292 = ((($291)) + 32|0);
      $293 = HEAP8[$292>>0]|0;
      $294 = $293&255;
      $295 = ($294|0)==(0);
      $296 = $iCmp;
      $297 = $296;
      $298 = HEAP32[$297>>2]|0;
      $299 = (($296) + 4)|0;
      $300 = $299;
      $301 = HEAP32[$300>>2]|0;
      $302 = ($301|0)>(0);
      $303 = ($298>>>0)>(0);
      $304 = ($301|0)==(0);
      $305 = $304 & $303;
      $306 = $302 | $305;
      $or$cond6 = $295 & $306;
      if (!($or$cond6)) {
       $310 = HEAP32[$0>>2]|0;
       $311 = HEAP32[$pLeft2>>2]|0;
       $312 = HEAP32[$2>>2]|0;
       _fts3EvalNextRow($310,$311,$312);
       $313 = HEAP32[$0>>2]|0;
       $314 = HEAP32[$pRight3>>2]|0;
       $315 = HEAP32[$2>>2]|0;
       _fts3EvalNextRow($313,$314,$315);
       break;
      }
     }
     $307 = HEAP32[$0>>2]|0;
     $308 = HEAP32[$pRight3>>2]|0;
     $309 = HEAP32[$2>>2]|0;
     _fts3EvalNextRow($307,$308,$309);
    }
   }
  } while(0);
  if ((label|0) == 33) {
   $283 = HEAP32[$0>>2]|0;
   $284 = HEAP32[$pLeft2>>2]|0;
   $285 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($283,$284,$285);
  }
  $316 = HEAP32[$pLeft2>>2]|0;
  $317 = ((($316)) + 32|0);
  $318 = HEAP8[$317>>0]|0;
  $319 = $318&255;
  $320 = ($319|0)!=(0);
  if ($320) {
   $321 = HEAP32[$pRight3>>2]|0;
   $322 = ((($321)) + 32|0);
   $323 = HEAP8[$322>>0]|0;
   $324 = $323&255;
   $325 = ($324|0)!=(0);
   $327 = $325;
  } else {
   $327 = 0;
  }
  $326 = $327&1;
  $328 = $326&255;
  $329 = HEAP32[$1>>2]|0;
  $330 = ((($329)) + 32|0);
  HEAP8[$330>>0] = $328;
  $331 = HEAP32[$bDescDoclist>>2]|0;
  $332 = ($331|0)!=(0);
  $333 = $332 ? -1 : 1;
  $334 = ($333|0)<(0);
  $335 = $334 << 31 >> 31;
  $336 = HEAP32[$pLeft2>>2]|0;
  $337 = ((($336)) + 24|0);
  $338 = $337;
  $339 = $338;
  $340 = HEAP32[$339>>2]|0;
  $341 = (($338) + 4)|0;
  $342 = $341;
  $343 = HEAP32[$342>>2]|0;
  $344 = HEAP32[$pRight3>>2]|0;
  $345 = ((($344)) + 24|0);
  $346 = $345;
  $347 = $346;
  $348 = HEAP32[$347>>2]|0;
  $349 = (($346) + 4)|0;
  $350 = $349;
  $351 = HEAP32[$350>>2]|0;
  $352 = (_i64Subtract(($340|0),($343|0),($348|0),($351|0))|0);
  $353 = tempRet0;
  $354 = (___muldi3(($333|0),($335|0),($352|0),($353|0))|0);
  $355 = tempRet0;
  $356 = $iCmp;
  $357 = $356;
  HEAP32[$357>>2] = $354;
  $358 = (($356) + 4)|0;
  $359 = $358;
  HEAP32[$359>>2] = $355;
  $360 = HEAP32[$pRight3>>2]|0;
  $361 = ((($360)) + 32|0);
  $362 = HEAP8[$361>>0]|0;
  $363 = $362&255;
  $364 = ($363|0)!=(0);
  if (!($364)) {
   $365 = HEAP32[$pLeft2>>2]|0;
   $366 = ((($365)) + 32|0);
   $367 = HEAP8[$366>>0]|0;
   $368 = $367&255;
   $369 = ($368|0)==(0);
   $370 = $iCmp;
   $371 = $370;
   $372 = HEAP32[$371>>2]|0;
   $373 = (($370) + 4)|0;
   $374 = $373;
   $375 = HEAP32[$374>>2]|0;
   $376 = ($375|0)<(0);
   $or$cond8 = $369 & $376;
   if (!($or$cond8)) {
    $391 = HEAP32[$pRight3>>2]|0;
    $392 = ((($391)) + 24|0);
    $393 = $392;
    $394 = $393;
    $395 = HEAP32[$394>>2]|0;
    $396 = (($393) + 4)|0;
    $397 = $396;
    $398 = HEAP32[$397>>2]|0;
    $399 = HEAP32[$1>>2]|0;
    $400 = ((($399)) + 24|0);
    $401 = $400;
    $402 = $401;
    HEAP32[$402>>2] = $395;
    $403 = (($401) + 4)|0;
    $404 = $403;
    HEAP32[$404>>2] = $398;
    STACKTOP = sp;return;
   }
  }
  $377 = HEAP32[$pLeft2>>2]|0;
  $378 = ((($377)) + 24|0);
  $379 = $378;
  $380 = $379;
  $381 = HEAP32[$380>>2]|0;
  $382 = (($379) + 4)|0;
  $383 = $382;
  $384 = HEAP32[$383>>2]|0;
  $385 = HEAP32[$1>>2]|0;
  $386 = ((($385)) + 24|0);
  $387 = $386;
  $388 = $387;
  HEAP32[$388>>2] = $381;
  $389 = (($387) + 4)|0;
  $390 = $389;
  HEAP32[$390>>2] = $384;
  STACKTOP = sp;return;
  break;
 }
 case 2:  {
  $405 = HEAP32[$1>>2]|0;
  $406 = ((($405)) + 12|0);
  $407 = HEAP32[$406>>2]|0;
  HEAP32[$pLeft4>>2] = $407;
  $408 = HEAP32[$1>>2]|0;
  $409 = ((($408)) + 16|0);
  $410 = HEAP32[$409>>2]|0;
  HEAP32[$pRight5>>2] = $410;
  $411 = HEAP32[$pRight5>>2]|0;
  $412 = ((($411)) + 33|0);
  $413 = HEAP8[$412>>0]|0;
  $414 = $413&255;
  $415 = ($414|0)==(0);
  if ($415) {
   $416 = HEAP32[$0>>2]|0;
   $417 = HEAP32[$pRight5>>2]|0;
   $418 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($416,$417,$418);
  }
  $419 = HEAP32[$0>>2]|0;
  $420 = HEAP32[$pLeft4>>2]|0;
  $421 = HEAP32[$2>>2]|0;
  _fts3EvalNextRow($419,$420,$421);
  $422 = HEAP32[$pLeft4>>2]|0;
  $423 = ((($422)) + 32|0);
  $424 = HEAP8[$423>>0]|0;
  $425 = $424&255;
  $426 = ($425|0)==(0);
  L80: do {
   if ($426) {
    while(1) {
     $427 = HEAP32[$2>>2]|0;
     $428 = HEAP32[$427>>2]|0;
     $429 = ($428|0)!=(0);
     if ($429) {
      break L80;
     }
     $430 = HEAP32[$pRight5>>2]|0;
     $431 = ((($430)) + 32|0);
     $432 = HEAP8[$431>>0]|0;
     $433 = ($432<<24>>24)!=(0);
     if ($433) {
      break L80;
     }
     $434 = HEAP32[$bDescDoclist>>2]|0;
     $435 = ($434|0)!=(0);
     $436 = $435 ? -1 : 1;
     $437 = ($436|0)<(0);
     $438 = $437 << 31 >> 31;
     $439 = HEAP32[$pLeft4>>2]|0;
     $440 = ((($439)) + 24|0);
     $441 = $440;
     $442 = $441;
     $443 = HEAP32[$442>>2]|0;
     $444 = (($441) + 4)|0;
     $445 = $444;
     $446 = HEAP32[$445>>2]|0;
     $447 = HEAP32[$pRight5>>2]|0;
     $448 = ((($447)) + 24|0);
     $449 = $448;
     $450 = $449;
     $451 = HEAP32[$450>>2]|0;
     $452 = (($449) + 4)|0;
     $453 = $452;
     $454 = HEAP32[$453>>2]|0;
     $455 = (_i64Subtract(($443|0),($446|0),($451|0),($454|0))|0);
     $456 = tempRet0;
     $457 = (___muldi3(($436|0),($438|0),($455|0),($456|0))|0);
     $458 = tempRet0;
     $459 = ($458|0)>(0);
     $460 = ($457>>>0)>(0);
     $461 = ($458|0)==(0);
     $462 = $461 & $460;
     $463 = $459 | $462;
     if (!($463)) {
      break L80;
     }
     $464 = HEAP32[$0>>2]|0;
     $465 = HEAP32[$pRight5>>2]|0;
     $466 = HEAP32[$2>>2]|0;
     _fts3EvalNextRow($464,$465,$466);
    }
   }
  } while(0);
  $467 = HEAP32[$pLeft4>>2]|0;
  $468 = ((($467)) + 24|0);
  $469 = $468;
  $470 = $469;
  $471 = HEAP32[$470>>2]|0;
  $472 = (($469) + 4)|0;
  $473 = $472;
  $474 = HEAP32[$473>>2]|0;
  $475 = HEAP32[$1>>2]|0;
  $476 = ((($475)) + 24|0);
  $477 = $476;
  $478 = $477;
  HEAP32[$478>>2] = $471;
  $479 = (($477) + 4)|0;
  $480 = $479;
  HEAP32[$480>>2] = $474;
  $481 = HEAP32[$pLeft4>>2]|0;
  $482 = ((($481)) + 32|0);
  $483 = HEAP8[$482>>0]|0;
  $484 = HEAP32[$1>>2]|0;
  $485 = ((($484)) + 32|0);
  HEAP8[$485>>0] = $483;
  STACKTOP = sp;return;
  break;
 }
 default: {
  $486 = HEAP32[$1>>2]|0;
  $487 = ((($486)) + 20|0);
  $488 = HEAP32[$487>>2]|0;
  HEAP32[$pPhrase>>2] = $488;
  $489 = HEAP32[$pPhrase>>2]|0;
  _fts3EvalInvalidatePoslist($489);
  $490 = HEAP32[$0>>2]|0;
  $491 = HEAP32[$pPhrase>>2]|0;
  $492 = HEAP32[$1>>2]|0;
  $493 = ((($492)) + 32|0);
  $494 = (_fts3EvalPhraseNext($490,$491,$493)|0);
  $495 = HEAP32[$2>>2]|0;
  HEAP32[$495>>2] = $494;
  $496 = HEAP32[$pPhrase>>2]|0;
  $497 = ((($496)) + 16|0);
  $498 = $497;
  $499 = $498;
  $500 = HEAP32[$499>>2]|0;
  $501 = (($498) + 4)|0;
  $502 = $501;
  $503 = HEAP32[$502>>2]|0;
  $504 = HEAP32[$1>>2]|0;
  $505 = ((($504)) + 24|0);
  $506 = $505;
  $507 = $506;
  HEAP32[$507>>2] = $500;
  $508 = (($506) + 4)|0;
  $509 = $508;
  HEAP32[$509>>2] = $503;
  STACKTOP = sp;return;
 }
 }
}
function _sqlite3Fts3EvalTestDeferred($pCsr,$pRc) {
 $pCsr = $pCsr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bMiss = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $bMiss = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pRc;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$rc>>2] = $3;
 HEAP32[$bMiss>>2] = 0;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)==(0);
  $28 = HEAP32[$bMiss>>2]|0;
  $29 = ($28|0)!=(0);
  $30 = $27 ? $29 : 0;
  $31 = $30&1;
  STACKTOP = sp;return ($31|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = (_fts3CursorSeek(0,$10)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (_sqlite3Fts3CacheDeferredDoclists($14)|0);
   HEAP32[$rc>>2] = $15;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_fts3EvalTestExpr($16,$19,$rc)|0);
 $21 = (0)==($20|0);
 $22 = $21&1;
 HEAP32[$bMiss>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 _sqlite3Fts3FreeDeferredDoclists($23);
 $24 = HEAP32[$rc>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 HEAP32[$25>>2] = $24;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)==(0);
 $28 = HEAP32[$bMiss>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = $27 ? $29 : 0;
 $31 = $30&1;
 STACKTOP = sp;return ($31|0);
}
function _fts3EvalPhraseNext($pCsr,$p,$pbEof) {
 $pCsr = $pCsr|0;
 $p = $p|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDL = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pDL = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pDL>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_fts3EvalIncrPhraseNext($10,$11,$12)|0);
  HEAP32[$rc>>2] = $13;
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 $14 = ((($10)) + 52|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = ((($17)) + 231|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($16|0)!=($20|0);
 if ($21) {
  $22 = HEAP32[$pDL>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   $26 = HEAP32[$pTab>>2]|0;
   $27 = ((($26)) + 231|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = HEAP32[$pDL>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$pDL>>2]|0;
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$pDL>>2]|0;
   $36 = ((($35)) + 8|0);
   $37 = HEAP32[$pDL>>2]|0;
   $38 = ((($37)) + 16|0);
   $39 = HEAP32[$pDL>>2]|0;
   $40 = ((($39)) + 32|0);
   $41 = HEAP32[$2>>2]|0;
   _sqlite3Fts3DoclistPrev($29,$31,$34,$36,$38,$40,$41);
   $42 = HEAP32[$pDL>>2]|0;
   $43 = ((($42)) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pDL>>2]|0;
   $46 = ((($45)) + 28|0);
   HEAP32[$46>>2] = $44;
   $50 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 }
 $47 = HEAP32[$pTab>>2]|0;
 $48 = HEAP32[$pDL>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 _fts3EvalDlPhraseNext($47,$48,$49);
 $50 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _fts3EvalIncrPhraseNext($pCsr,$p,$pbEof) {
 $pCsr = $pCsr|0;
 $p = $p|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $a = 0, $aDoclist = 0, $bDescDoclist = 0, $bEof = 0, $bMaxSet = 0, $i = 0, $iMax = 0, $nByte = 0, $nDist = 0, $nList = 0, $pDL = 0, $pL = 0, $pOut = 0, $pR = 0, $pTab = 0, $rc = 0, $res = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $3 = sp + 160|0;
 $rc = sp + 156|0;
 $pDL = sp + 152|0;
 $pTab = sp + 148|0;
 $bEof = sp + 176|0;
 $bDescDoclist = sp + 144|0;
 $a = sp + 8|0;
 $bMaxSet = sp + 140|0;
 $iMax = sp;
 $i = sp + 136|0;
 $nList = sp + 132|0;
 $nByte = sp + 128|0;
 $aDoclist = sp + 124|0;
 $pL = sp + 120|0;
 $pR = sp + 116|0;
 $pOut = sp + 112|0;
 $nDist = sp + 108|0;
 $res = sp + 104|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$pDL>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTab>>2] = $6;
 HEAP8[$bEof>>0] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(1);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 40|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$pTab>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 72|0);
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$pDL>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$pDL>>2]|0;
   $23 = ((($22)) + 28|0);
   $24 = HEAP32[$pDL>>2]|0;
   $25 = ((($24)) + 32|0);
   $26 = (_sqlite3Fts3MsrIncrNext($15,$19,$21,$23,$25)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$pDL>>2]|0;
   $28 = ((($27)) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0|0);
   if ($30) {
    HEAP8[$bEof>>0] = 1;
   }
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 L5: do {
  if ((label|0) == 5) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 52|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   HEAP32[$bDescDoclist>>2] = $34;
   dest=$a; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   while(1) {
    $35 = HEAP8[$bEof>>0]|0;
    $36 = $35&255;
    $37 = ($36|0)==(0);
    if (!($37)) {
     break L5;
    }
    HEAP32[$bMaxSet>>2] = 0;
    $38 = $iMax;
    $39 = $38;
    HEAP32[$39>>2] = 0;
    $40 = (($38) + 4)|0;
    $41 = $40;
    HEAP32[$41>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $42 = HEAP32[$rc>>2]|0;
     $43 = ($42|0)==(0);
     if (!($43)) {
      break;
     }
     $44 = HEAP32[$i>>2]|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = ((($45)) + 64|0);
     $47 = HEAP32[$46>>2]|0;
     $48 = ($44|0)<($47|0);
     if (!($48)) {
      break;
     }
     $49 = HEAP8[$bEof>>0]|0;
     $50 = $49&255;
     $51 = ($50|0)==(0);
     if (!($51)) {
      break;
     }
     $52 = HEAP32[$pTab>>2]|0;
     $53 = HEAP32[$2>>2]|0;
     $54 = HEAP32[$i>>2]|0;
     $55 = HEAP32[$i>>2]|0;
     $56 = (($a) + (($55*24)|0)|0);
     $57 = (_incrPhraseTokenNext($52,$53,$54,$56,$bEof)|0);
     HEAP32[$rc>>2] = $57;
     $58 = HEAP32[$i>>2]|0;
     $59 = (($a) + (($58*24)|0)|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($60|0)==(0);
     do {
      if ($61) {
       $62 = HEAP32[$bMaxSet>>2]|0;
       $63 = ($62|0)==(0);
       if (!($63)) {
        $64 = HEAP32[$bDescDoclist>>2]|0;
        $65 = ($64|0)!=(0);
        $66 = $65 ? -1 : 1;
        $67 = ($66|0)<(0);
        $68 = $67 << 31 >> 31;
        $69 = $iMax;
        $70 = $69;
        $71 = HEAP32[$70>>2]|0;
        $72 = (($69) + 4)|0;
        $73 = $72;
        $74 = HEAP32[$73>>2]|0;
        $75 = HEAP32[$i>>2]|0;
        $76 = (($a) + (($75*24)|0)|0);
        $77 = ((($76)) + 8|0);
        $78 = $77;
        $79 = $78;
        $80 = HEAP32[$79>>2]|0;
        $81 = (($78) + 4)|0;
        $82 = $81;
        $83 = HEAP32[$82>>2]|0;
        $84 = (_i64Subtract(($71|0),($74|0),($80|0),($83|0))|0);
        $85 = tempRet0;
        (___muldi3(($66|0),($68|0),($84|0),($85|0))|0);
        $86 = tempRet0;
        $87 = ($86|0)<(0);
        if (!($87)) {
         break;
        }
       }
       $88 = HEAP32[$i>>2]|0;
       $89 = (($a) + (($88*24)|0)|0);
       $90 = ((($89)) + 8|0);
       $91 = $90;
       $92 = $91;
       $93 = HEAP32[$92>>2]|0;
       $94 = (($91) + 4)|0;
       $95 = $94;
       $96 = HEAP32[$95>>2]|0;
       $97 = $iMax;
       $98 = $97;
       HEAP32[$98>>2] = $93;
       $99 = (($97) + 4)|0;
       $100 = $99;
       HEAP32[$100>>2] = $96;
       HEAP32[$bMaxSet>>2] = 1;
      }
     } while(0);
     $101 = HEAP32[$i>>2]|0;
     $102 = (($101) + 1)|0;
     HEAP32[$i>>2] = $102;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $103 = HEAP32[$i>>2]|0;
     $104 = HEAP32[$2>>2]|0;
     $105 = ((($104)) + 64|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = ($103|0)<($106|0);
     if (!($107)) {
      break;
     }
     while(1) {
      $108 = HEAP32[$rc>>2]|0;
      $109 = ($108|0)==(0);
      if (!($109)) {
       break;
      }
      $110 = HEAP8[$bEof>>0]|0;
      $111 = $110&255;
      $112 = ($111|0)==(0);
      if (!($112)) {
       break;
      }
      $113 = HEAP32[$i>>2]|0;
      $114 = (($a) + (($113*24)|0)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = ($115|0)==(0);
      if (!($116)) {
       break;
      }
      $117 = HEAP32[$bDescDoclist>>2]|0;
      $118 = ($117|0)!=(0);
      $119 = $118 ? -1 : 1;
      $120 = ($119|0)<(0);
      $121 = $120 << 31 >> 31;
      $122 = HEAP32[$i>>2]|0;
      $123 = (($a) + (($122*24)|0)|0);
      $124 = ((($123)) + 8|0);
      $125 = $124;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = (($125) + 4)|0;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = $iMax;
      $132 = $131;
      $133 = HEAP32[$132>>2]|0;
      $134 = (($131) + 4)|0;
      $135 = $134;
      $136 = HEAP32[$135>>2]|0;
      $137 = (_i64Subtract(($127|0),($130|0),($133|0),($136|0))|0);
      $138 = tempRet0;
      (___muldi3(($119|0),($121|0),($137|0),($138|0))|0);
      $139 = tempRet0;
      $140 = ($139|0)<(0);
      if (!($140)) {
       break;
      }
      $141 = HEAP32[$pTab>>2]|0;
      $142 = HEAP32[$2>>2]|0;
      $143 = HEAP32[$i>>2]|0;
      $144 = HEAP32[$i>>2]|0;
      $145 = (($a) + (($144*24)|0)|0);
      $146 = (_incrPhraseTokenNext($141,$142,$143,$145,$bEof)|0);
      HEAP32[$rc>>2] = $146;
      $147 = HEAP32[$bDescDoclist>>2]|0;
      $148 = ($147|0)!=(0);
      $149 = $148 ? -1 : 1;
      $150 = ($149|0)<(0);
      $151 = $150 << 31 >> 31;
      $152 = HEAP32[$i>>2]|0;
      $153 = (($a) + (($152*24)|0)|0);
      $154 = ((($153)) + 8|0);
      $155 = $154;
      $156 = $155;
      $157 = HEAP32[$156>>2]|0;
      $158 = (($155) + 4)|0;
      $159 = $158;
      $160 = HEAP32[$159>>2]|0;
      $161 = $iMax;
      $162 = $161;
      $163 = HEAP32[$162>>2]|0;
      $164 = (($161) + 4)|0;
      $165 = $164;
      $166 = HEAP32[$165>>2]|0;
      $167 = (_i64Subtract(($157|0),($160|0),($163|0),($166|0))|0);
      $168 = tempRet0;
      $169 = (___muldi3(($149|0),($151|0),($167|0),($168|0))|0);
      $170 = tempRet0;
      $171 = ($170|0)>(0);
      $172 = ($169>>>0)>(0);
      $173 = ($170|0)==(0);
      $174 = $173 & $172;
      $175 = $171 | $174;
      if (!($175)) {
       continue;
      }
      $176 = HEAP32[$i>>2]|0;
      $177 = (($a) + (($176*24)|0)|0);
      $178 = ((($177)) + 8|0);
      $179 = $178;
      $180 = $179;
      $181 = HEAP32[$180>>2]|0;
      $182 = (($179) + 4)|0;
      $183 = $182;
      $184 = HEAP32[$183>>2]|0;
      $185 = $iMax;
      $186 = $185;
      HEAP32[$186>>2] = $181;
      $187 = (($185) + 4)|0;
      $188 = $187;
      HEAP32[$188>>2] = $184;
      HEAP32[$i>>2] = 0;
     }
     $189 = HEAP32[$i>>2]|0;
     $190 = (($189) + 1)|0;
     HEAP32[$i>>2] = $190;
    }
    $191 = HEAP8[$bEof>>0]|0;
    $192 = $191&255;
    $193 = ($192|0)==(0);
    if (!($193)) {
     continue;
    }
    HEAP32[$nList>>2] = 0;
    $194 = HEAP32[$2>>2]|0;
    $195 = ((($194)) + 64|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = (($196) - 1)|0;
    $198 = (($a) + (($197*24)|0)|0);
    $199 = ((($198)) + 20|0);
    $200 = HEAP32[$199>>2]|0;
    HEAP32[$nByte>>2] = $200;
    $201 = HEAP32[$nByte>>2]|0;
    $202 = (($201) + 1)|0;
    $203 = (_sqlite3_malloc($202)|0);
    HEAP32[$aDoclist>>2] = $203;
    $204 = HEAP32[$aDoclist>>2]|0;
    $205 = ($204|0)!=(0|0);
    if (!($205)) {
     break;
    }
    $206 = HEAP32[$aDoclist>>2]|0;
    $207 = HEAP32[$2>>2]|0;
    $208 = ((($207)) + 64|0);
    $209 = HEAP32[$208>>2]|0;
    $210 = (($209) - 1)|0;
    $211 = (($a) + (($210*24)|0)|0);
    $212 = ((($211)) + 16|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = HEAP32[$nByte>>2]|0;
    $215 = (($214) + 1)|0;
    _memcpy(($206|0),($213|0),($215|0))|0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $216 = HEAP32[$i>>2]|0;
     $217 = HEAP32[$2>>2]|0;
     $218 = ((($217)) + 64|0);
     $219 = HEAP32[$218>>2]|0;
     $220 = (($219) - 1)|0;
     $221 = ($216|0)<($220|0);
     if (!($221)) {
      break;
     }
     $222 = HEAP32[$i>>2]|0;
     $223 = (($a) + (($222*24)|0)|0);
     $224 = HEAP32[$223>>2]|0;
     $225 = ($224|0)==(0);
     if ($225) {
      $226 = HEAP32[$i>>2]|0;
      $227 = (($a) + (($226*24)|0)|0);
      $228 = ((($227)) + 16|0);
      $229 = HEAP32[$228>>2]|0;
      HEAP32[$pL>>2] = $229;
      $230 = HEAP32[$aDoclist>>2]|0;
      HEAP32[$pR>>2] = $230;
      $231 = HEAP32[$aDoclist>>2]|0;
      HEAP32[$pOut>>2] = $231;
      $232 = HEAP32[$2>>2]|0;
      $233 = ((($232)) + 64|0);
      $234 = HEAP32[$233>>2]|0;
      $235 = (($234) - 1)|0;
      $236 = HEAP32[$i>>2]|0;
      $237 = (($235) - ($236))|0;
      HEAP32[$nDist>>2] = $237;
      $238 = HEAP32[$nDist>>2]|0;
      $239 = (_fts3PoslistPhraseMerge($pOut,$238,0,1,$pL,$pR)|0);
      HEAP32[$res>>2] = $239;
      $240 = HEAP32[$res>>2]|0;
      $241 = ($240|0)==(0);
      if ($241) {
       break;
      }
      $242 = HEAP32[$pOut>>2]|0;
      $243 = HEAP32[$aDoclist>>2]|0;
      $244 = $242;
      $245 = $243;
      $246 = (($244) - ($245))|0;
      HEAP32[$nList>>2] = $246;
     }
     $247 = HEAP32[$i>>2]|0;
     $248 = (($247) + 1)|0;
     HEAP32[$i>>2] = $248;
    }
    $249 = HEAP32[$i>>2]|0;
    $250 = HEAP32[$2>>2]|0;
    $251 = ((($250)) + 64|0);
    $252 = HEAP32[$251>>2]|0;
    $253 = (($252) - 1)|0;
    $254 = ($249|0)==($253|0);
    if ($254) {
     label = 35;
     break;
    }
    $275 = HEAP32[$aDoclist>>2]|0;
    _sqlite3_free($275);
   }
   if ((label|0) == 35) {
    $255 = $iMax;
    $256 = $255;
    $257 = HEAP32[$256>>2]|0;
    $258 = (($255) + 4)|0;
    $259 = $258;
    $260 = HEAP32[$259>>2]|0;
    $261 = HEAP32[$pDL>>2]|0;
    $262 = ((($261)) + 16|0);
    $263 = $262;
    $264 = $263;
    HEAP32[$264>>2] = $257;
    $265 = (($263) + 4)|0;
    $266 = $265;
    HEAP32[$266>>2] = $260;
    $267 = HEAP32[$aDoclist>>2]|0;
    $268 = HEAP32[$pDL>>2]|0;
    $269 = ((($268)) + 28|0);
    HEAP32[$269>>2] = $267;
    $270 = HEAP32[$nList>>2]|0;
    $271 = HEAP32[$pDL>>2]|0;
    $272 = ((($271)) + 32|0);
    HEAP32[$272>>2] = $270;
    $273 = HEAP32[$pDL>>2]|0;
    $274 = ((($273)) + 24|0);
    HEAP32[$274>>2] = 1;
    break;
   }
   HEAP32[$0>>2] = 7;
   $279 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($279|0);
  }
 } while(0);
 $276 = HEAP8[$bEof>>0]|0;
 $277 = HEAP32[$3>>2]|0;
 HEAP8[$277>>0] = $276;
 $278 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $278;
 $279 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($279|0);
}
function _fts3EvalDlPhraseNext($pTab,$pDL,$pbEof) {
 $pTab = $pTab|0;
 $pDL = $pDL|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $iDelta = 0;
 var $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pIter = sp + 12|0;
 $pEnd = sp + 8|0;
 $iDelta = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $pDL;
 HEAP32[$2>>2] = $pbEof;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($5)|0);
 HEAP32[$pEnd>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 $13 = HEAP32[$1>>2]|0;
 if ($12) {
  $14 = ((($13)) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$pIter>>2] = $15;
 } else {
  $16 = HEAP32[$13>>2]|0;
  HEAP32[$pIter>>2] = $16;
 }
 $17 = HEAP32[$pIter>>2]|0;
 $18 = HEAP32[$pEnd>>2]|0;
 $19 = ($17>>>0)>=($18>>>0);
 if ($19) {
  $20 = HEAP32[$2>>2]|0;
  HEAP8[$20>>0] = 1;
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$pIter>>2]|0;
 $22 = (_sqlite3Fts3GetVarint($21,$iDelta)|0);
 $23 = HEAP32[$pIter>>2]|0;
 $24 = (($23) + ($22)|0);
 HEAP32[$pIter>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 231|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = ($28|0)==(0);
 if ($29) {
  label = 8;
 } else {
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)==(0|0);
  if ($33) {
   label = 8;
  } else {
   $54 = $iDelta;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($54) + 4)|0;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = $61;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Subtract(($64|0),($67|0),($56|0),($59|0))|0);
   $69 = tempRet0;
   $70 = $61;
   $71 = $70;
   HEAP32[$71>>2] = $68;
   $72 = (($70) + 4)|0;
   $73 = $72;
   HEAP32[$73>>2] = $69;
  }
 }
 if ((label|0) == 8) {
  $34 = $iDelta;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 16|0);
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (_i64Add(($44|0),($47|0),($36|0),($39|0))|0);
  $49 = tempRet0;
  $50 = $41;
  $51 = $50;
  HEAP32[$51>>2] = $48;
  $52 = (($50) + 4)|0;
  $53 = $52;
  HEAP32[$53>>2] = $49;
 }
 $74 = HEAP32[$pIter>>2]|0;
 $75 = HEAP32[$1>>2]|0;
 $76 = ((($75)) + 28|0);
 HEAP32[$76>>2] = $74;
 _fts3PoslistCopy(0,$pIter);
 $77 = HEAP32[$pIter>>2]|0;
 $78 = HEAP32[$1>>2]|0;
 $79 = ((($78)) + 28|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $77;
 $82 = $80;
 $83 = (($81) - ($82))|0;
 $84 = HEAP32[$1>>2]|0;
 $85 = ((($84)) + 32|0);
 HEAP32[$85>>2] = $83;
 while(1) {
  $86 = HEAP32[$pIter>>2]|0;
  $87 = HEAP32[$pEnd>>2]|0;
  $88 = ($86>>>0)<($87>>>0);
  if ($88) {
   $89 = HEAP32[$pIter>>2]|0;
   $90 = HEAP8[$89>>0]|0;
   $91 = $90 << 24 >> 24;
   $92 = ($91|0)==(0);
   $98 = $92;
  } else {
   $98 = 0;
  }
  $93 = HEAP32[$pIter>>2]|0;
  if (!($98)) {
   break;
  }
  $94 = ((($93)) + 1|0);
  HEAP32[$pIter>>2] = $94;
 }
 $95 = HEAP32[$1>>2]|0;
 $96 = ((($95)) + 8|0);
 HEAP32[$96>>2] = $93;
 $97 = HEAP32[$2>>2]|0;
 HEAP8[$97>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3MsrIncrNext($p,$pMsr,$piDocid,$paPoslist,$pnPoslist) {
 $p = $p|0;
 $pMsr = $pMsr|0;
 $piDocid = $piDocid|0;
 $paPoslist = $paPoslist|0;
 $pnPoslist = $pnPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apSegment = 0, $iDocid = 0, $j = 0, $nList = 0;
 var $nMerge = 0, $pList = 0, $pSeg = 0, $rc = 0, $xCmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $nMerge = sp + 36|0;
 $apSegment = sp + 32|0;
 $xCmp = sp + 28|0;
 $pSeg = sp + 24|0;
 $rc = sp + 20|0;
 $pList = sp + 16|0;
 $nList = sp + 12|0;
 $j = sp + 8|0;
 $iDocid = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pMsr;
 HEAP32[$3>>2] = $piDocid;
 HEAP32[$4>>2] = $paPoslist;
 HEAP32[$5>>2] = $pnPoslist;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nMerge>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$apSegment>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 231|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 $16 = $15 ? 200 : 201;
 HEAP32[$xCmp>>2] = $16;
 $17 = HEAP32[$nMerge>>2]|0;
 $18 = ($17|0)==(0);
 if ($18) {
  $19 = HEAP32[$4>>2]|0;
  HEAP32[$19>>2] = 0;
  HEAP32[$0>>2] = 0;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 while(1) {
  $20 = HEAP32[$2>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$pSeg>>2] = $22;
  $23 = HEAP32[$pSeg>>2]|0;
  $24 = ((($23)) + 80|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)==(0|0);
  if ($26) {
   label = 4;
   break;
  }
  $28 = HEAP32[$apSegment>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 88|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = $iDocid;
  $38 = $37;
  HEAP32[$38>>2] = $33;
  $39 = (($37) + 4)|0;
  $40 = $39;
  HEAP32[$40>>2] = $36;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$apSegment>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = (_fts3SegReaderNextDocid($41,$43,$pList,$nList)|0);
  HEAP32[$rc>>2] = $44;
  HEAP32[$j>>2] = 1;
  while(1) {
   $45 = HEAP32[$rc>>2]|0;
   $46 = ($45|0)==(0);
   if (!($46)) {
    break;
   }
   $47 = HEAP32[$j>>2]|0;
   $48 = HEAP32[$nMerge>>2]|0;
   $49 = ($47|0)<($48|0);
   if (!($49)) {
    break;
   }
   $50 = HEAP32[$j>>2]|0;
   $51 = HEAP32[$apSegment>>2]|0;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ((($53)) + 80|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)!=(0|0);
   if (!($56)) {
    break;
   }
   $57 = HEAP32[$j>>2]|0;
   $58 = HEAP32[$apSegment>>2]|0;
   $59 = (($58) + ($57<<2)|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ((($60)) + 88|0);
   $62 = $61;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = $iDocid;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = (($68) + 4)|0;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = ($64|0)==($70|0);
   $75 = ($67|0)==($73|0);
   $76 = $74 & $75;
   if (!($76)) {
    break;
   }
   $77 = HEAP32[$1>>2]|0;
   $78 = HEAP32[$j>>2]|0;
   $79 = HEAP32[$apSegment>>2]|0;
   $80 = (($79) + ($78<<2)|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = (_fts3SegReaderNextDocid($77,$81,0,0)|0);
   HEAP32[$rc>>2] = $82;
   $83 = HEAP32[$j>>2]|0;
   $84 = (($83) + 1)|0;
   HEAP32[$j>>2] = $84;
  }
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)!=(0);
  if ($86) {
   label = 12;
   break;
  }
  $88 = HEAP32[$2>>2]|0;
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$nMerge>>2]|0;
  $91 = HEAP32[$j>>2]|0;
  $92 = HEAP32[$xCmp>>2]|0;
  _fts3SegReaderSort($89,$90,$91,$92);
  $93 = HEAP32[$nList>>2]|0;
  $94 = ($93|0)>(0);
  if ($94) {
   $95 = HEAP32[$apSegment>>2]|0;
   $96 = HEAP32[$95>>2]|0;
   $97 = ((($96)) + 56|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($98|0)!=(0|0);
   if ($99) {
    $100 = HEAP32[$2>>2]|0;
    $101 = HEAP32[$pList>>2]|0;
    $102 = HEAP32[$nList>>2]|0;
    $103 = (($102) + 1)|0;
    $104 = (_fts3MsrBufferData($100,$101,$103)|0);
    HEAP32[$rc>>2] = $104;
    $105 = HEAP32[$rc>>2]|0;
    $106 = ($105|0)!=(0);
    if ($106) {
     label = 16;
     break;
    }
    $108 = HEAP32[$2>>2]|0;
    $109 = ((($108)) + 16|0);
    $110 = HEAP32[$109>>2]|0;
    HEAP32[$pList>>2] = $110;
   }
  }
  $111 = HEAP32[$2>>2]|0;
  $112 = ((($111)) + 24|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($113|0)>=(0);
  if ($114) {
   $115 = HEAP32[$2>>2]|0;
   $116 = ((($115)) + 24|0);
   $117 = HEAP32[$116>>2]|0;
   _fts3ColumnFilter($117,1,$pList,$nList);
  }
  $118 = HEAP32[$nList>>2]|0;
  $119 = ($118|0)>(0);
  if ($119) {
   label = 21;
   break;
  }
 }
 if ((label|0) == 4) {
  $27 = HEAP32[$4>>2]|0;
  HEAP32[$27>>2] = 0;
 }
 else if ((label|0) == 12) {
  $87 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $87;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 else if ((label|0) == 16) {
  $107 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $107;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 else if ((label|0) == 21) {
  $120 = HEAP32[$pList>>2]|0;
  $121 = HEAP32[$4>>2]|0;
  HEAP32[$121>>2] = $120;
  $122 = $iDocid;
  $123 = $122;
  $124 = HEAP32[$123>>2]|0;
  $125 = (($122) + 4)|0;
  $126 = $125;
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$3>>2]|0;
  $129 = $128;
  $130 = $129;
  HEAP32[$130>>2] = $124;
  $131 = (($129) + 4)|0;
  $132 = $131;
  HEAP32[$132>>2] = $127;
  $133 = HEAP32[$nList>>2]|0;
  $134 = HEAP32[$5>>2]|0;
  HEAP32[$134>>2] = $133;
 }
 HEAP32[$0>>2] = 0;
 $135 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _incrPhraseTokenNext($pTab,$pPhrase,$iToken,$p,$pbEof) {
 $pTab = $pTab|0;
 $pPhrase = $pPhrase|0;
 $iToken = $iToken|0;
 $p = $p|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pToken = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $pPhrase;
 HEAP32[$2>>2] = $iToken;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($7|0)==($8|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  _fts3EvalDlPhraseNext($10,$11,$12);
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 28|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = ((($16)) + 16|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 32|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = ((($21)) + 20|0);
  HEAP32[$22>>2] = $20;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = $24;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = $32;
  $34 = $33;
  HEAP32[$34>>2] = $27;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = $30;
  $62 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 72|0);
 $40 = (($39) + (($37*24)|0)|0);
 HEAP32[$pToken>>2] = $40;
 $41 = HEAP32[$pToken>>2]|0;
 $42 = ((($41)) + 20|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0|0);
 if (!($44)) {
  $61 = HEAP32[$3>>2]|0;
  HEAP32[$61>>2] = 1;
  $62 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$pToken>>2]|0;
 $47 = ((($46)) + 20|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$3>>2]|0;
 $50 = ((($49)) + 8|0);
 $51 = HEAP32[$3>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$3>>2]|0;
 $54 = ((($53)) + 20|0);
 $55 = (_sqlite3Fts3MsrIncrNext($45,$48,$50,$52,$54)|0);
 HEAP32[$rc>>2] = $55;
 $56 = HEAP32[$3>>2]|0;
 $57 = ((($56)) + 16|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = ($58|0)==(0|0);
 if (!($59)) {
  $62 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $60 = HEAP32[$4>>2]|0;
 HEAP8[$60>>0] = 1;
 $62 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _fts3CursorSeek($pContext,$pCsr) {
 $pContext = $pContext|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 7|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$pStmt>>2] = 0;
  $7 = HEAP32[$2>>2]|0;
  $8 = (_fts3CursorSeekStmt($7,$pStmt)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$rc>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$2>>2]|0;
   $12 = ((($11)) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 32|0);
   $16 = $15;
   $17 = $16;
   $18 = HEAP32[$17>>2]|0;
   $19 = (($16) + 4)|0;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   (_sqlite3_bind_int64($13,1,$18,$21)|0);
   $22 = HEAP32[$2>>2]|0;
   $23 = ((($22)) + 7|0);
   HEAP8[$23>>0] = 0;
   $24 = HEAP32[$2>>2]|0;
   $25 = ((($24)) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (_sqlite3_step($26)|0);
   $28 = (100)==($27|0);
   if ($28) {
    HEAP32[$0>>2] = 0;
    $49 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($49|0);
   }
   $29 = HEAP32[$2>>2]|0;
   $30 = ((($29)) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (_sqlite3_reset($31)|0);
   HEAP32[$rc>>2] = $32;
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$2>>2]|0;
    $36 = HEAP32[$35>>2]|0;
    $37 = ((($36)) + 40|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ($38|0)==(0|0);
    if ($39) {
     HEAP32[$rc>>2] = 267;
     $40 = HEAP32[$2>>2]|0;
     $41 = ((($40)) + 6|0);
     HEAP8[$41>>0] = 1;
    }
   }
  }
 }
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)!=(0);
 $44 = HEAP32[$1>>2]|0;
 $45 = ($44|0)!=(0|0);
 $or$cond = $43 & $45;
 if ($or$cond) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($46,$47);
 }
 $48 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3Fts3CacheDeferredDoclists($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDocid = 0, $iDum1 = 0, $iDum2 = 0, $iPos = 0, $nToken = 0, $or$cond = 0, $p = 0;
 var $pDef = 0, $pModule = 0, $pPT = 0, $pT = 0, $pTC = 0, $rc = 0, $zText = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $rc = sp + 60|0;
 $i = sp + 56|0;
 $iDocid = sp;
 $pDef = sp + 52|0;
 $p = sp + 48|0;
 $pT = sp + 44|0;
 $pModule = sp + 40|0;
 $zText = sp + 36|0;
 $pTC = sp + 32|0;
 $zToken = sp + 28|0;
 $nToken = sp + 24|0;
 $iDum1 = sp + 20|0;
 $iDum2 = sp + 16|0;
 $iPos = sp + 12|0;
 $pPT = sp + 8|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $151 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($151|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pT>>2] = $9;
 $10 = HEAP32[$pT>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pModule>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_column_int64($14,0)|0);
 $16 = tempRet0;
 $17 = $iDocid;
 $18 = $17;
 HEAP32[$18>>2] = $15;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $16;
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($21|0)<($24|0);
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)==(0);
  $28 = $25 ? $27 : 0;
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$p>>2]|0;
  $31 = ((($30)) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29)|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$0>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$i>>2]|0;
   $41 = (($40) + 1)|0;
   $42 = (_sqlite3_column_text($39,$41)|0);
   HEAP32[$zText>>2] = $42;
   HEAP32[$pTC>>2] = 0;
   $43 = HEAP32[$pT>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = ((($44)) + 16|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$zText>>2]|0;
   $48 = (_sqlite3Fts3OpenTokenizer($43,$46,$47,-1,$pTC)|0);
   HEAP32[$rc>>2] = $48;
   L9: while(1) {
    $49 = HEAP32[$rc>>2]|0;
    $50 = ($49|0)==(0);
    if (!($50)) {
     break;
    }
    HEAP32[$nToken>>2] = 0;
    HEAP32[$iDum1>>2] = 0;
    HEAP32[$iDum2>>2] = 0;
    HEAP32[$iPos>>2] = 0;
    $51 = HEAP32[$pModule>>2]|0;
    $52 = ((($51)) + 20|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = HEAP32[$pTC>>2]|0;
    $55 = (FUNCTION_TABLE_iiiiiii[$53 & 255]($54,$zToken,$nToken,$iDum1,$iDum2,$iPos)|0);
    HEAP32[$rc>>2] = $55;
    $56 = HEAP32[$0>>2]|0;
    $57 = ((($56)) + 24|0);
    $58 = HEAP32[$57>>2]|0;
    HEAP32[$pDef>>2] = $58;
    while(1) {
     $59 = HEAP32[$pDef>>2]|0;
     $60 = ($59|0)!=(0|0);
     $61 = HEAP32[$rc>>2]|0;
     $62 = ($61|0)==(0);
     $63 = $60 ? $62 : 0;
     if (!($63)) {
      continue L9;
     }
     $64 = HEAP32[$pDef>>2]|0;
     $65 = HEAP32[$64>>2]|0;
     HEAP32[$pPT>>2] = $65;
     $66 = HEAP32[$pDef>>2]|0;
     $67 = ((($66)) + 4|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = HEAP32[$p>>2]|0;
     $70 = ((($69)) + 24|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = ($68|0)>=($71|0);
     if ($72) {
      label = 11;
     } else {
      $73 = HEAP32[$pDef>>2]|0;
      $74 = ((($73)) + 4|0);
      $75 = HEAP32[$74>>2]|0;
      $76 = HEAP32[$i>>2]|0;
      $77 = ($75|0)==($76|0);
      if ($77) {
       label = 11;
      }
     }
     do {
      if ((label|0) == 11) {
       label = 0;
       $78 = HEAP32[$pPT>>2]|0;
       $79 = ((($78)) + 12|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = ($80|0)==(0);
       $82 = HEAP32[$iPos>>2]|0;
       $83 = ($82|0)==(0);
       $or$cond = $81 | $83;
       if ($or$cond) {
        $84 = HEAP32[$pPT>>2]|0;
        $85 = ((($84)) + 4|0);
        $86 = HEAP32[$85>>2]|0;
        $87 = HEAP32[$nToken>>2]|0;
        $88 = ($86|0)==($87|0);
        if (!($88)) {
         $89 = HEAP32[$pPT>>2]|0;
         $90 = ((($89)) + 8|0);
         $91 = HEAP32[$90>>2]|0;
         $92 = ($91|0)!=(0);
         if (!($92)) {
          break;
         }
         $93 = HEAP32[$pPT>>2]|0;
         $94 = ((($93)) + 4|0);
         $95 = HEAP32[$94>>2]|0;
         $96 = HEAP32[$nToken>>2]|0;
         $97 = ($95|0)<($96|0);
         if (!($97)) {
          break;
         }
        }
        $98 = HEAP32[$zToken>>2]|0;
        $99 = HEAP32[$pPT>>2]|0;
        $100 = HEAP32[$99>>2]|0;
        $101 = HEAP32[$pPT>>2]|0;
        $102 = ((($101)) + 4|0);
        $103 = HEAP32[$102>>2]|0;
        $104 = (_memcmp($98,$100,$103)|0);
        $105 = (0)==($104|0);
        if ($105) {
         $106 = HEAP32[$pDef>>2]|0;
         $107 = ((($106)) + 12|0);
         $108 = $iDocid;
         $109 = $108;
         $110 = HEAP32[$109>>2]|0;
         $111 = (($108) + 4)|0;
         $112 = $111;
         $113 = HEAP32[$112>>2]|0;
         $114 = HEAP32[$i>>2]|0;
         $115 = ($114|0)<(0);
         $116 = $115 << 31 >> 31;
         $117 = HEAP32[$iPos>>2]|0;
         $118 = ($117|0)<(0);
         $119 = $118 << 31 >> 31;
         (_fts3PendingListAppend($107,$110,$113,$114,$116,$117,$119,$rc)|0);
        }
       }
      }
     } while(0);
     $120 = HEAP32[$pDef>>2]|0;
     $121 = ((($120)) + 8|0);
     $122 = HEAP32[$121>>2]|0;
     HEAP32[$pDef>>2] = $122;
    }
   }
   $123 = HEAP32[$pTC>>2]|0;
   $124 = ($123|0)!=(0|0);
   if ($124) {
    $125 = HEAP32[$pModule>>2]|0;
    $126 = ((($125)) + 16|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = HEAP32[$pTC>>2]|0;
    (FUNCTION_TABLE_ii[$127 & 255]($128)|0);
   }
   $129 = HEAP32[$rc>>2]|0;
   $130 = ($129|0)==(101);
   if ($130) {
    HEAP32[$rc>>2] = 0;
   }
  }
  $131 = HEAP32[$i>>2]|0;
  $132 = (($131) + 1)|0;
  HEAP32[$i>>2] = $132;
 }
 $133 = HEAP32[$0>>2]|0;
 $134 = ((($133)) + 24|0);
 $135 = HEAP32[$134>>2]|0;
 HEAP32[$pDef>>2] = $135;
 while(1) {
  $136 = HEAP32[$pDef>>2]|0;
  $137 = ($136|0)!=(0|0);
  $138 = HEAP32[$rc>>2]|0;
  $139 = ($138|0)==(0);
  $140 = $137 ? $139 : 0;
  if (!($140)) {
   break;
  }
  $141 = HEAP32[$pDef>>2]|0;
  $142 = ((($141)) + 12|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = ($143|0)!=(0|0);
  if ($144) {
   $145 = HEAP32[$pDef>>2]|0;
   $146 = ((($145)) + 12|0);
   $147 = (_fts3PendingListAppendVarint($146,0,0)|0);
   HEAP32[$rc>>2] = $147;
  }
  $148 = HEAP32[$pDef>>2]|0;
  $149 = ((($148)) + 8|0);
  $150 = HEAP32[$149>>2]|0;
  HEAP32[$pDef>>2] = $150;
 }
 $151 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($151|0);
}
function _fts3EvalTestExpr($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bHit = 0, $bHit1 = 0;
 var $bHit2 = 0, $p = 0, $pPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $bHit = sp + 16|0;
 $p = sp + 12|0;
 $bHit1 = sp + 8|0;
 $bHit2 = sp + 4|0;
 $pPhrase = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 HEAP32[$bHit>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 switch ($7|0) {
 case 3: case 1:  {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_fts3EvalTestExpr($8,$11,$12)|0);
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_fts3EvalTestExpr($15,$18,$19)|0);
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = (_fts3EvalNearTest($22,$23)|0);
    $25 = ($24|0)!=(0);
    $27 = $25;
   } else {
    $27 = 0;
   }
  } else {
   $27 = 0;
  }
  $26 = $27&1;
  HEAP32[$bHit>>2] = $26;
  $28 = HEAP32[$bHit>>2]|0;
  $29 = ($28|0)==(0);
  if (!($29)) {
   $217 = HEAP32[$bHit>>2]|0;
   STACKTOP = sp;return ($217|0);
  }
  $30 = HEAP32[$1>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)==(1);
  if (!($32)) {
   $217 = HEAP32[$bHit>>2]|0;
   STACKTOP = sp;return ($217|0);
  }
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 8|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)==(0|0);
  if (!($36)) {
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)!=(1);
   if (!($41)) {
    $217 = HEAP32[$bHit>>2]|0;
    STACKTOP = sp;return ($217|0);
   }
  }
  $42 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $42;
  while(1) {
   $43 = HEAP32[$p>>2]|0;
   $44 = ((($43)) + 20|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)==(0|0);
   $47 = HEAP32[$p>>2]|0;
   if (!($46)) {
    break;
   }
   $48 = ((($47)) + 16|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ((($49)) + 24|0);
   $51 = $50;
   $52 = $51;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($51) + 4)|0;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 32|0);
   $59 = $58;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = (($59) + 4)|0;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = ($53|0)==($61|0);
   $66 = ($56|0)==($64|0);
   $67 = $65 & $66;
   if ($67) {
    $68 = HEAP32[$p>>2]|0;
    $69 = ((($68)) + 16|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ((($70)) + 20|0);
    $72 = HEAP32[$71>>2]|0;
    _fts3EvalInvalidatePoslist($72);
   }
   $73 = HEAP32[$p>>2]|0;
   $74 = ((($73)) + 12|0);
   $75 = HEAP32[$74>>2]|0;
   HEAP32[$p>>2] = $75;
  }
  $76 = ((($47)) + 24|0);
  $77 = $76;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = (($77) + 4)|0;
  $81 = $80;
  $82 = HEAP32[$81>>2]|0;
  $83 = HEAP32[$0>>2]|0;
  $84 = ((($83)) + 32|0);
  $85 = $84;
  $86 = $85;
  $87 = HEAP32[$86>>2]|0;
  $88 = (($85) + 4)|0;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = ($79|0)==($87|0);
  $92 = ($82|0)==($90|0);
  $93 = $91 & $92;
  if (!($93)) {
   $217 = HEAP32[$bHit>>2]|0;
   STACKTOP = sp;return ($217|0);
  }
  $94 = HEAP32[$p>>2]|0;
  $95 = ((($94)) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  _fts3EvalInvalidatePoslist($96);
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
  break;
 }
 case 4:  {
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 12|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = (_fts3EvalTestExpr($97,$100,$101)|0);
  HEAP32[$bHit1>>2] = $102;
  $103 = HEAP32[$0>>2]|0;
  $104 = HEAP32[$1>>2]|0;
  $105 = ((($104)) + 16|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = HEAP32[$2>>2]|0;
  $108 = (_fts3EvalTestExpr($103,$106,$107)|0);
  HEAP32[$bHit2>>2] = $108;
  $109 = HEAP32[$bHit1>>2]|0;
  $110 = ($109|0)!=(0);
  $111 = HEAP32[$bHit2>>2]|0;
  $112 = ($111|0)!=(0);
  $113 = $110 ? 1 : $112;
  $114 = $113&1;
  HEAP32[$bHit>>2] = $114;
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
  break;
 }
 case 2:  {
  $115 = HEAP32[$0>>2]|0;
  $116 = HEAP32[$1>>2]|0;
  $117 = ((($116)) + 12|0);
  $118 = HEAP32[$117>>2]|0;
  $119 = HEAP32[$2>>2]|0;
  $120 = (_fts3EvalTestExpr($115,$118,$119)|0);
  $121 = ($120|0)!=(0);
  if ($121) {
   $122 = HEAP32[$0>>2]|0;
   $123 = HEAP32[$1>>2]|0;
   $124 = ((($123)) + 16|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = HEAP32[$2>>2]|0;
   $127 = (_fts3EvalTestExpr($122,$125,$126)|0);
   $128 = ($127|0)!=(0);
   $129 = $128 ^ 1;
   $131 = $129;
  } else {
   $131 = 0;
  }
  $130 = $131&1;
  HEAP32[$bHit>>2] = $130;
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
  break;
 }
 default: {
  $132 = HEAP32[$0>>2]|0;
  $133 = ((($132)) + 24|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = ($134|0)!=(0|0);
  do {
   if ($135) {
    $136 = HEAP32[$1>>2]|0;
    $137 = ((($136)) + 24|0);
    $138 = $137;
    $139 = $138;
    $140 = HEAP32[$139>>2]|0;
    $141 = (($138) + 4)|0;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = HEAP32[$0>>2]|0;
    $145 = ((($144)) + 32|0);
    $146 = $145;
    $147 = $146;
    $148 = HEAP32[$147>>2]|0;
    $149 = (($146) + 4)|0;
    $150 = $149;
    $151 = HEAP32[$150>>2]|0;
    $152 = ($140|0)==($148|0);
    $153 = ($143|0)==($151|0);
    $154 = $152 & $153;
    if (!($154)) {
     $155 = HEAP32[$1>>2]|0;
     $156 = ((($155)) + 34|0);
     $157 = HEAP8[$156>>0]|0;
     $158 = $157&255;
     $159 = ($158|0)!=(0);
     if (!($159)) {
      break;
     }
    }
    $160 = HEAP32[$1>>2]|0;
    $161 = ((($160)) + 20|0);
    $162 = HEAP32[$161>>2]|0;
    HEAP32[$pPhrase>>2] = $162;
    $163 = HEAP32[$1>>2]|0;
    $164 = ((($163)) + 34|0);
    $165 = HEAP8[$164>>0]|0;
    $166 = ($165<<24>>24)!=(0);
    if ($166) {
     $167 = HEAP32[$pPhrase>>2]|0;
     _fts3EvalInvalidatePoslist($167);
    }
    $168 = HEAP32[$0>>2]|0;
    $169 = HEAP32[$pPhrase>>2]|0;
    $170 = (_fts3EvalDeferredPhrase($168,$169)|0);
    $171 = HEAP32[$2>>2]|0;
    HEAP32[$171>>2] = $170;
    $172 = HEAP32[$pPhrase>>2]|0;
    $173 = ((($172)) + 28|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = ($174|0)!=(0|0);
    $176 = $175&1;
    HEAP32[$bHit>>2] = $176;
    $177 = HEAP32[$0>>2]|0;
    $178 = ((($177)) + 32|0);
    $179 = $178;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = (($179) + 4)|0;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = HEAP32[$1>>2]|0;
    $186 = ((($185)) + 24|0);
    $187 = $186;
    $188 = $187;
    HEAP32[$188>>2] = $181;
    $189 = (($187) + 4)|0;
    $190 = $189;
    HEAP32[$190>>2] = $184;
    $217 = HEAP32[$bHit>>2]|0;
    STACKTOP = sp;return ($217|0);
   }
  } while(0);
  $191 = HEAP32[$1>>2]|0;
  $192 = ((($191)) + 32|0);
  $193 = HEAP8[$192>>0]|0;
  $194 = $193&255;
  $195 = ($194|0)==(0);
  if ($195) {
   $196 = HEAP32[$1>>2]|0;
   $197 = ((($196)) + 24|0);
   $198 = $197;
   $199 = $198;
   $200 = HEAP32[$199>>2]|0;
   $201 = (($198) + 4)|0;
   $202 = $201;
   $203 = HEAP32[$202>>2]|0;
   $204 = HEAP32[$0>>2]|0;
   $205 = ((($204)) + 32|0);
   $206 = $205;
   $207 = $206;
   $208 = HEAP32[$207>>2]|0;
   $209 = (($206) + 4)|0;
   $210 = $209;
   $211 = HEAP32[$210>>2]|0;
   $212 = ($200|0)==($208|0);
   $213 = ($203|0)==($211|0);
   $214 = $212 & $213;
   $216 = $214;
  } else {
   $216 = 0;
  }
  $215 = $216&1;
  HEAP32[$bHit>>2] = $215;
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
 }
 }
 return (0)|0;
}
function _sqlite3Fts3FreeDeferredDoclists($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pDef = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pDef>>2] = $3;
 while(1) {
  $4 = HEAP32[$pDef>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$pDef>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  _fts3PendingListDelete($8);
  $9 = HEAP32[$pDef>>2]|0;
  $10 = ((($9)) + 12|0);
  HEAP32[$10>>2] = 0;
  $11 = HEAP32[$pDef>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pDef>>2] = $13;
 }
 STACKTOP = sp;return;
}
function _fts3PendingListAppend($pp,$0,$1,$2,$3,$4,$5,$pRc) {
 $pp = $pp|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $4 = $4|0;
 $5 = $5|0;
 $pRc = $pRc|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iDelta = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $6 = sp + 48|0;
 $7 = sp + 44|0;
 $8 = sp + 24|0;
 $9 = sp + 16|0;
 $10 = sp + 8|0;
 $11 = sp + 40|0;
 $p = sp + 36|0;
 $rc = sp + 32|0;
 $iDelta = sp;
 HEAP32[$7>>2] = $pp;
 $12 = $8;
 $13 = $12;
 HEAP32[$13>>2] = $0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $1;
 $16 = $9;
 $17 = $16;
 HEAP32[$17>>2] = $2;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $3;
 $20 = $10;
 $21 = $20;
 HEAP32[$21>>2] = $4;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $5;
 HEAP32[$11>>2] = $pRc;
 $24 = HEAP32[$7>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$p>>2] = $25;
 HEAP32[$rc>>2] = 0;
 $26 = HEAP32[$p>>2]|0;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$p>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = $29;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = $8;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = ($32|0)!=($38|0);
  $43 = ($35|0)!=($41|0);
  $44 = $42 | $43;
  if ($44) {
   label = 3;
  } else {
   label = 9;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $45 = $8;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($45) + 4)|0;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$p>>2]|0;
  $52 = ($51|0)!=(0|0);
  if ($52) {
   $53 = HEAP32[$p>>2]|0;
   $54 = ((($53)) + 16|0);
   $55 = $54;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($55) + 4)|0;
   $59 = $58;
   $60 = HEAP32[$59>>2]|0;
   $61 = $57;$62 = $60;
  } else {
   $61 = 0;$62 = 0;
  }
  $63 = (_i64Subtract(($47|0),($50|0),($61|0),($62|0))|0);
  $64 = tempRet0;
  $65 = $iDelta;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
  $69 = HEAP32[$p>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$p>>2]|0;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($72) + 1)|0;
   HEAP32[$71>>2] = $73;
  }
  $74 = $iDelta;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($74) + 4)|0;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = (_fts3PendingListAppendVarint($p,$76,$79)|0);
  HEAP32[$rc>>2] = $80;
  $81 = (0)!=($80|0);
  if (!($81)) {
   $82 = HEAP32[$p>>2]|0;
   $83 = ((($82)) + 24|0);
   $84 = $83;
   $85 = $84;
   HEAP32[$85>>2] = -1;
   $86 = (($84) + 4)|0;
   $87 = $86;
   HEAP32[$87>>2] = -1;
   $88 = HEAP32[$p>>2]|0;
   $89 = ((($88)) + 32|0);
   $90 = $89;
   $91 = $90;
   HEAP32[$91>>2] = 0;
   $92 = (($90) + 4)|0;
   $93 = $92;
   HEAP32[$93>>2] = 0;
   $94 = $8;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$p>>2]|0;
   $101 = ((($100)) + 16|0);
   $102 = $101;
   $103 = $102;
   HEAP32[$103>>2] = $96;
   $104 = (($102) + 4)|0;
   $105 = $104;
   HEAP32[$105>>2] = $99;
   label = 9;
  }
 }
 do {
  if ((label|0) == 9) {
   $106 = $9;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (($106) + 4)|0;
   $110 = $109;
   $111 = HEAP32[$110>>2]|0;
   $112 = ($111|0)>(0);
   $113 = ($108>>>0)>(0);
   $114 = ($111|0)==(0);
   $115 = $114 & $113;
   $116 = $112 | $115;
   if ($116) {
    $117 = HEAP32[$p>>2]|0;
    $118 = ((($117)) + 24|0);
    $119 = $118;
    $120 = $119;
    $121 = HEAP32[$120>>2]|0;
    $122 = (($119) + 4)|0;
    $123 = $122;
    $124 = HEAP32[$123>>2]|0;
    $125 = $9;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (($125) + 4)|0;
    $129 = $128;
    $130 = HEAP32[$129>>2]|0;
    $131 = ($121|0)!=($127|0);
    $132 = ($124|0)!=($130|0);
    $133 = $131 | $132;
    if ($133) {
     $134 = (_fts3PendingListAppendVarint($p,1,0)|0);
     HEAP32[$rc>>2] = $134;
     $135 = (0)!=($134|0);
     if ($135) {
      break;
     }
     $136 = $9;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = (_fts3PendingListAppendVarint($p,$138,$141)|0);
     HEAP32[$rc>>2] = $142;
     $143 = (0)!=($142|0);
     if ($143) {
      break;
     }
     $144 = $9;
     $145 = $144;
     $146 = HEAP32[$145>>2]|0;
     $147 = (($144) + 4)|0;
     $148 = $147;
     $149 = HEAP32[$148>>2]|0;
     $150 = HEAP32[$p>>2]|0;
     $151 = ((($150)) + 24|0);
     $152 = $151;
     $153 = $152;
     HEAP32[$153>>2] = $146;
     $154 = (($152) + 4)|0;
     $155 = $154;
     HEAP32[$155>>2] = $149;
     $156 = HEAP32[$p>>2]|0;
     $157 = ((($156)) + 32|0);
     $158 = $157;
     $159 = $158;
     HEAP32[$159>>2] = 0;
     $160 = (($158) + 4)|0;
     $161 = $160;
     HEAP32[$161>>2] = 0;
    }
   }
   $162 = $9;
   $163 = $162;
   $164 = HEAP32[$163>>2]|0;
   $165 = (($162) + 4)|0;
   $166 = $165;
   $167 = HEAP32[$166>>2]|0;
   $168 = ($167|0)>(0);
   $169 = ($164>>>0)>=(0);
   $170 = ($167|0)==(0);
   $171 = $170 & $169;
   $172 = $168 | $171;
   if ($172) {
    $173 = $10;
    $174 = $173;
    $175 = HEAP32[$174>>2]|0;
    $176 = (($173) + 4)|0;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (_i64Add(2,0,($175|0),($178|0))|0);
    $180 = tempRet0;
    $181 = HEAP32[$p>>2]|0;
    $182 = ((($181)) + 32|0);
    $183 = $182;
    $184 = $183;
    $185 = HEAP32[$184>>2]|0;
    $186 = (($183) + 4)|0;
    $187 = $186;
    $188 = HEAP32[$187>>2]|0;
    $189 = (_i64Subtract(($179|0),($180|0),($185|0),($188|0))|0);
    $190 = tempRet0;
    $191 = (_fts3PendingListAppendVarint($p,$189,$190)|0);
    HEAP32[$rc>>2] = $191;
    $192 = HEAP32[$rc>>2]|0;
    $193 = ($192|0)==(0);
    if ($193) {
     $194 = $10;
     $195 = $194;
     $196 = HEAP32[$195>>2]|0;
     $197 = (($194) + 4)|0;
     $198 = $197;
     $199 = HEAP32[$198>>2]|0;
     $200 = HEAP32[$p>>2]|0;
     $201 = ((($200)) + 32|0);
     $202 = $201;
     $203 = $202;
     HEAP32[$203>>2] = $196;
     $204 = (($202) + 4)|0;
     $205 = $204;
     HEAP32[$205>>2] = $199;
    }
   }
  }
 } while(0);
 $206 = HEAP32[$rc>>2]|0;
 $207 = HEAP32[$11>>2]|0;
 HEAP32[$207>>2] = $206;
 $208 = HEAP32[$p>>2]|0;
 $209 = HEAP32[$7>>2]|0;
 $210 = HEAP32[$209>>2]|0;
 $211 = ($208|0)!=($210|0);
 if ($211) {
  $212 = HEAP32[$p>>2]|0;
  $213 = HEAP32[$7>>2]|0;
  HEAP32[$213>>2] = $212;
  HEAP32[$6>>2] = 1;
  $214 = HEAP32[$6>>2]|0;
  STACKTOP = sp;return ($214|0);
 } else {
  HEAP32[$6>>2] = 0;
  $214 = HEAP32[$6>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 return (0)|0;
}
function _fts3PendingListAppendVarint($pp,$0,$1) {
 $pp = $pp|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $nNew = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $p = sp + 12|0;
 $nNew = sp + 8|0;
 HEAP32[$3>>2] = $pp;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$p>>2]|0;
 $12 = ($11|0)!=(0|0);
 do {
  if ($12) {
   $23 = HEAP32[$p>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + 10)|0;
   $26 = (($25) + 1)|0;
   $27 = HEAP32[$p>>2]|0;
   $28 = ((($27)) + 8|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($26|0)>($29|0);
   if ($30) {
    $31 = HEAP32[$p>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = $33<<1;
    HEAP32[$nNew>>2] = $34;
    $35 = HEAP32[$p>>2]|0;
    $36 = HEAP32[$nNew>>2]|0;
    $37 = (40 + ($36))|0;
    $38 = (_sqlite3_realloc($35,$37)|0);
    HEAP32[$p>>2] = $38;
    $39 = HEAP32[$p>>2]|0;
    $40 = ($39|0)!=(0|0);
    if ($40) {
     $44 = HEAP32[$nNew>>2]|0;
     $45 = HEAP32[$p>>2]|0;
     $46 = ((($45)) + 8|0);
     HEAP32[$46>>2] = $44;
     $47 = HEAP32[$p>>2]|0;
     $48 = ((($47)) + 40|0);
     $49 = HEAP32[$p>>2]|0;
     $50 = ((($49)) + 4|0);
     HEAP32[$50>>2] = $48;
     break;
    }
    $41 = HEAP32[$3>>2]|0;
    $42 = HEAP32[$41>>2]|0;
    _sqlite3_free($42);
    $43 = HEAP32[$3>>2]|0;
    HEAP32[$43>>2] = 0;
    HEAP32[$2>>2] = 7;
    $75 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($75|0);
   }
  } else {
   $13 = (_sqlite3_malloc(140)|0);
   HEAP32[$p>>2] = $13;
   $14 = HEAP32[$p>>2]|0;
   $15 = ($14|0)!=(0|0);
   if ($15) {
    $16 = HEAP32[$p>>2]|0;
    $17 = ((($16)) + 8|0);
    HEAP32[$17>>2] = 100;
    $18 = HEAP32[$p>>2]|0;
    $19 = ((($18)) + 40|0);
    $20 = HEAP32[$p>>2]|0;
    $21 = ((($20)) + 4|0);
    HEAP32[$21>>2] = $19;
    $22 = HEAP32[$p>>2]|0;
    HEAP32[$22>>2] = 0;
    break;
   }
   HEAP32[$2>>2] = 7;
   $75 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($75|0);
  }
 } while(0);
 $51 = HEAP32[$p>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$p>>2]|0;
 $54 = ((($53)) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + ($52)|0);
 $57 = $4;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3Fts3PutVarint($56,$59,$62)|0);
 $64 = HEAP32[$p>>2]|0;
 $65 = HEAP32[$64>>2]|0;
 $66 = (($65) + ($63))|0;
 HEAP32[$64>>2] = $66;
 $67 = HEAP32[$p>>2]|0;
 $68 = HEAP32[$67>>2]|0;
 $69 = HEAP32[$p>>2]|0;
 $70 = ((($69)) + 4|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + ($68)|0);
 HEAP8[$72>>0] = 0;
 $73 = HEAP32[$p>>2]|0;
 $74 = HEAP32[$3>>2]|0;
 HEAP32[$74>>2] = $73;
 HEAP32[$2>>2] = 0;
 $75 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _fts3EvalNearTest($pExpr,$pRc) {
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aPoslist = 0, $aTmp = 0, $nNear = 0, $nNear1 = 0, $nTmp = 0;
 var $nToken = 0, $or$cond = 0, $p = 0, $pPhrase = 0, $pPhrase2 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $res = sp + 36|0;
 $p = sp + 32|0;
 $nTmp = sp + 28|0;
 $aTmp = sp + 24|0;
 $aPoslist = sp + 20|0;
 $nToken = sp + 16|0;
 $pPhrase = sp + 12|0;
 $nNear = sp + 8|0;
 $nNear1 = sp + 4|0;
 $pPhrase2 = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $pRc;
 HEAP32[$res>>2] = 1;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(1);
 if (!($7)) {
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0|0);
 if (!($16)) {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)!=(1);
  if (!($21)) {
   $134 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($134|0);
  }
 }
 HEAP32[$nTmp>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $22;
 while(1) {
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 12|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)!=(0|0);
  $27 = HEAP32[$p>>2]|0;
  if (!($26)) {
   break;
  }
  $28 = ((($27)) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 20|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ((($31)) + 32|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$nTmp>>2]|0;
  $35 = (($34) + ($33))|0;
  HEAP32[$nTmp>>2] = $35;
  $36 = HEAP32[$p>>2]|0;
  $37 = ((($36)) + 12|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$p>>2] = $38;
 }
 $39 = ((($27)) + 20|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ((($40)) + 32|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$nTmp>>2]|0;
 $44 = (($43) + ($42))|0;
 HEAP32[$nTmp>>2] = $44;
 $45 = HEAP32[$nTmp>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  HEAP32[$res>>2] = 0;
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $47 = HEAP32[$nTmp>>2]|0;
 $48 = $47<<1;
 $49 = (_sqlite3_malloc($48)|0);
 HEAP32[$aTmp>>2] = $49;
 $50 = HEAP32[$aTmp>>2]|0;
 $51 = ($50|0)!=(0|0);
 L22: do {
  if ($51) {
   $53 = HEAP32[$p>>2]|0;
   $54 = ((($53)) + 20|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ((($55)) + 28|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$aPoslist>>2] = $57;
   $58 = HEAP32[$p>>2]|0;
   $59 = ((($58)) + 20|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ((($60)) + 64|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$nToken>>2] = $62;
   $63 = HEAP32[$p>>2]|0;
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$p>>2] = $65;
   while(1) {
    $66 = HEAP32[$res>>2]|0;
    $67 = ($66|0)!=(0);
    $68 = HEAP32[$p>>2]|0;
    $69 = ($68|0)!=(0|0);
    $or$cond = $67 & $69;
    if (!($or$cond)) {
     break;
    }
    $70 = HEAP32[$p>>2]|0;
    $71 = HEAP32[$70>>2]|0;
    $72 = ($71|0)==(1);
    if (!($72)) {
     break;
    }
    $73 = HEAP32[$p>>2]|0;
    $74 = ((($73)) + 16|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ((($75)) + 20|0);
    $77 = HEAP32[$76>>2]|0;
    HEAP32[$pPhrase>>2] = $77;
    $78 = HEAP32[$p>>2]|0;
    $79 = ((($78)) + 4|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$nNear>>2] = $80;
    $81 = HEAP32[$nNear>>2]|0;
    $82 = HEAP32[$aTmp>>2]|0;
    $83 = HEAP32[$pPhrase>>2]|0;
    $84 = (_fts3EvalNearTrim($81,$82,$aPoslist,$nToken,$83)|0);
    HEAP32[$res>>2] = $84;
    $85 = HEAP32[$p>>2]|0;
    $86 = ((($85)) + 8|0);
    $87 = HEAP32[$86>>2]|0;
    HEAP32[$p>>2] = $87;
   }
   $88 = HEAP32[$0>>2]|0;
   $89 = ((($88)) + 16|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = ((($90)) + 20|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ((($92)) + 28|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$aPoslist>>2] = $94;
   $95 = HEAP32[$0>>2]|0;
   $96 = ((($95)) + 16|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = ((($97)) + 20|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = ((($99)) + 64|0);
   $101 = HEAP32[$100>>2]|0;
   HEAP32[$nToken>>2] = $101;
   $102 = HEAP32[$0>>2]|0;
   $103 = ((($102)) + 12|0);
   $104 = HEAP32[$103>>2]|0;
   HEAP32[$p>>2] = $104;
   while(1) {
    $105 = HEAP32[$p>>2]|0;
    $106 = ($105|0)!=(0|0);
    $107 = HEAP32[$res>>2]|0;
    $108 = ($107|0)!=(0);
    $109 = $106 ? $108 : 0;
    if (!($109)) {
     break L22;
    }
    $110 = HEAP32[$p>>2]|0;
    $111 = ((($110)) + 8|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = ((($112)) + 4|0);
    $114 = HEAP32[$113>>2]|0;
    HEAP32[$nNear1>>2] = $114;
    $115 = HEAP32[$p>>2]|0;
    $116 = HEAP32[$115>>2]|0;
    $117 = ($116|0)==(1);
    $118 = HEAP32[$p>>2]|0;
    if ($117) {
     $119 = ((($118)) + 16|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = ((($120)) + 20|0);
     $122 = HEAP32[$121>>2]|0;
     $125 = $122;
    } else {
     $123 = ((($118)) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = $124;
    }
    HEAP32[$pPhrase2>>2] = $125;
    $126 = HEAP32[$nNear1>>2]|0;
    $127 = HEAP32[$aTmp>>2]|0;
    $128 = HEAP32[$pPhrase2>>2]|0;
    $129 = (_fts3EvalNearTrim($126,$127,$aPoslist,$nToken,$128)|0);
    HEAP32[$res>>2] = $129;
    $130 = HEAP32[$p>>2]|0;
    $131 = ((($130)) + 12|0);
    $132 = HEAP32[$131>>2]|0;
    HEAP32[$p>>2] = $132;
   }
  } else {
   $52 = HEAP32[$1>>2]|0;
   HEAP32[$52>>2] = 7;
   HEAP32[$res>>2] = 0;
  }
 } while(0);
 $133 = HEAP32[$aTmp>>2]|0;
 _sqlite3_free($133);
 $134 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($134|0);
}
function _fts3EvalDeferredPhrase($pCsr,$pPhrase) {
 $pCsr = $pCsr|0;
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOut = 0, $aOut3 = 0, $aPoslist = 0, $iPrev = 0, $iToken = 0, $nDistance = 0, $nList = 0, $nMaxUndeferred = 0, $nPoslist = 0, $p1 = 0;
 var $p11 = 0, $p2 = 0, $p22 = 0, $pDeferred = 0, $pList = 0, $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $iToken = sp + 64|0;
 $aPoslist = sp + 60|0;
 $nPoslist = sp + 56|0;
 $iPrev = sp + 52|0;
 $pToken = sp + 48|0;
 $pDeferred = sp + 44|0;
 $pList = sp + 40|0;
 $nList = sp + 36|0;
 $rc = sp + 32|0;
 $aOut = sp + 28|0;
 $p1 = sp + 24|0;
 $p2 = sp + 20|0;
 $nMaxUndeferred = sp + 16|0;
 $nDistance = sp + 12|0;
 $p11 = sp + 8|0;
 $p22 = sp + 4|0;
 $aOut3 = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pPhrase;
 HEAP32[$aPoslist>>2] = 0;
 HEAP32[$nPoslist>>2] = 0;
 HEAP32[$iPrev>>2] = -1;
 HEAP32[$iToken>>2] = 0;
 while(1) {
  $3 = HEAP32[$iToken>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = ((($4)) + 64|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 14;
   break;
  }
  $8 = HEAP32[$iToken>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 72|0);
  $11 = (($10) + (($8*24)|0)|0);
  HEAP32[$pToken>>2] = $11;
  $12 = HEAP32[$pToken>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pDeferred>>2] = $14;
  $15 = HEAP32[$pDeferred>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   $17 = HEAP32[$pDeferred>>2]|0;
   $18 = (_sqlite3Fts3DeferredTokenList($17,$pList,$nList)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)!=(0);
   if ($20) {
    label = 5;
    break;
   }
   $22 = HEAP32[$pList>>2]|0;
   $23 = ($22|0)==(0|0);
   $24 = HEAP32[$aPoslist>>2]|0;
   if ($23) {
    label = 7;
    break;
   }
   $29 = ($24|0)==(0|0);
   $30 = HEAP32[$pList>>2]|0;
   if ($29) {
    HEAP32[$aPoslist>>2] = $30;
    $31 = HEAP32[$nList>>2]|0;
    HEAP32[$nPoslist>>2] = $31;
   } else {
    HEAP32[$aOut>>2] = $30;
    $32 = HEAP32[$aPoslist>>2]|0;
    HEAP32[$p1>>2] = $32;
    $33 = HEAP32[$aOut>>2]|0;
    HEAP32[$p2>>2] = $33;
    $34 = HEAP32[$iToken>>2]|0;
    $35 = HEAP32[$iPrev>>2]|0;
    $36 = (($34) - ($35))|0;
    (_fts3PoslistPhraseMerge($aOut,$36,0,1,$p1,$p2)|0);
    $37 = HEAP32[$aPoslist>>2]|0;
    _sqlite3_free($37);
    $38 = HEAP32[$pList>>2]|0;
    HEAP32[$aPoslist>>2] = $38;
    $39 = HEAP32[$aOut>>2]|0;
    $40 = HEAP32[$aPoslist>>2]|0;
    $41 = $39;
    $42 = $40;
    $43 = (($41) - ($42))|0;
    HEAP32[$nPoslist>>2] = $43;
    $44 = HEAP32[$nPoslist>>2]|0;
    $45 = ($44|0)==(0);
    if ($45) {
     label = 11;
     break;
    }
   }
   $51 = HEAP32[$iToken>>2]|0;
   HEAP32[$iPrev>>2] = $51;
  }
  $52 = HEAP32[$iToken>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$iToken>>2] = $53;
 }
 if ((label|0) == 5) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 else if ((label|0) == 7) {
  _sqlite3_free($24);
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 28|0);
  HEAP32[$26>>2] = 0;
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 32|0);
  HEAP32[$28>>2] = 0;
  HEAP32[$0>>2] = 0;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 else if ((label|0) == 11) {
  $46 = HEAP32[$aPoslist>>2]|0;
  _sqlite3_free($46);
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 28|0);
  HEAP32[$48>>2] = 0;
  $49 = HEAP32[$2>>2]|0;
  $50 = ((($49)) + 32|0);
  HEAP32[$50>>2] = 0;
  HEAP32[$0>>2] = 0;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 else if ((label|0) == 14) {
  $54 = HEAP32[$iPrev>>2]|0;
  $55 = ($54|0)>=(0);
  do {
   if ($55) {
    $56 = HEAP32[$2>>2]|0;
    $57 = ((($56)) + 44|0);
    $58 = HEAP32[$57>>2]|0;
    HEAP32[$nMaxUndeferred>>2] = $58;
    $59 = HEAP32[$nMaxUndeferred>>2]|0;
    $60 = ($59|0)<(0);
    if ($60) {
     $61 = HEAP32[$aPoslist>>2]|0;
     $62 = HEAP32[$2>>2]|0;
     $63 = ((($62)) + 28|0);
     HEAP32[$63>>2] = $61;
     $64 = HEAP32[$nPoslist>>2]|0;
     $65 = HEAP32[$2>>2]|0;
     $66 = ((($65)) + 32|0);
     HEAP32[$66>>2] = $64;
     $67 = HEAP32[$1>>2]|0;
     $68 = ((($67)) + 32|0);
     $69 = $68;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = (($69) + 4)|0;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$2>>2]|0;
     $76 = ((($75)) + 16|0);
     $77 = $76;
     $78 = $77;
     HEAP32[$78>>2] = $71;
     $79 = (($77) + 4)|0;
     $80 = $79;
     HEAP32[$80>>2] = $74;
     $81 = HEAP32[$2>>2]|0;
     $82 = ((($81)) + 24|0);
     HEAP32[$82>>2] = 1;
     break;
    }
    $83 = HEAP32[$nMaxUndeferred>>2]|0;
    $84 = HEAP32[$iPrev>>2]|0;
    $85 = ($83|0)>($84|0);
    if ($85) {
     $86 = HEAP32[$aPoslist>>2]|0;
     HEAP32[$p11>>2] = $86;
     $87 = HEAP32[$2>>2]|0;
     $88 = ((($87)) + 28|0);
     $89 = HEAP32[$88>>2]|0;
     HEAP32[$p22>>2] = $89;
     $90 = HEAP32[$nMaxUndeferred>>2]|0;
     $91 = HEAP32[$iPrev>>2]|0;
     $92 = (($90) - ($91))|0;
     HEAP32[$nDistance>>2] = $92;
    } else {
     $93 = HEAP32[$2>>2]|0;
     $94 = ((($93)) + 28|0);
     $95 = HEAP32[$94>>2]|0;
     HEAP32[$p11>>2] = $95;
     $96 = HEAP32[$aPoslist>>2]|0;
     HEAP32[$p22>>2] = $96;
     $97 = HEAP32[$iPrev>>2]|0;
     $98 = HEAP32[$nMaxUndeferred>>2]|0;
     $99 = (($97) - ($98))|0;
     HEAP32[$nDistance>>2] = $99;
    }
    $100 = HEAP32[$nPoslist>>2]|0;
    $101 = (($100) + 8)|0;
    $102 = (_sqlite3_malloc($101)|0);
    HEAP32[$aOut3>>2] = $102;
    $103 = HEAP32[$aOut3>>2]|0;
    $104 = ($103|0)!=(0|0);
    if (!($104)) {
     $105 = HEAP32[$aPoslist>>2]|0;
     _sqlite3_free($105);
     HEAP32[$0>>2] = 7;
     $129 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($129|0);
    }
    $106 = HEAP32[$aOut3>>2]|0;
    $107 = HEAP32[$2>>2]|0;
    $108 = ((($107)) + 28|0);
    HEAP32[$108>>2] = $106;
    $109 = HEAP32[$nDistance>>2]|0;
    $110 = (_fts3PoslistPhraseMerge($aOut3,$109,0,1,$p11,$p22)|0);
    $111 = ($110|0)!=(0);
    if ($111) {
     $112 = HEAP32[$2>>2]|0;
     $113 = ((($112)) + 24|0);
     HEAP32[$113>>2] = 1;
     $114 = HEAP32[$aOut3>>2]|0;
     $115 = HEAP32[$2>>2]|0;
     $116 = ((($115)) + 28|0);
     $117 = HEAP32[$116>>2]|0;
     $118 = $114;
     $119 = $117;
     $120 = (($118) - ($119))|0;
     $121 = HEAP32[$2>>2]|0;
     $122 = ((($121)) + 32|0);
     HEAP32[$122>>2] = $120;
    } else {
     $123 = HEAP32[$aOut3>>2]|0;
     _sqlite3_free($123);
     $124 = HEAP32[$2>>2]|0;
     $125 = ((($124)) + 28|0);
     HEAP32[$125>>2] = 0;
     $126 = HEAP32[$2>>2]|0;
     $127 = ((($126)) + 32|0);
     HEAP32[$127>>2] = 0;
    }
    $128 = HEAP32[$aPoslist>>2]|0;
    _sqlite3_free($128);
   }
  } while(0);
  HEAP32[$0>>2] = 0;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 return (0)|0;
}
function _fts3EvalNearTrim($nNear,$aTmp,$paPoslist,$pnToken,$pPhrase) {
 $nNear = $nNear|0;
 $aTmp = $aTmp|0;
 $paPoslist = $paPoslist|0;
 $pnToken = $pnToken|0;
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nNew = 0, $nParam1 = 0, $nParam2 = 0, $p2 = 0, $pOut = 0, $res = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $nParam1 = sp + 20|0;
 $nParam2 = sp + 16|0;
 $nNew = sp + 12|0;
 $p2 = sp + 8|0;
 $pOut = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $nNear;
 HEAP32[$1>>2] = $aTmp;
 HEAP32[$2>>2] = $paPoslist;
 HEAP32[$3>>2] = $pnToken;
 HEAP32[$4>>2] = $pPhrase;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ((($6)) + 64|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5) + ($8))|0;
 HEAP32[$nParam1>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + ($12))|0;
 HEAP32[$nParam2>>2] = $13;
 $14 = HEAP32[$4>>2]|0;
 $15 = ((($14)) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pOut>>2] = $16;
 HEAP32[$p2>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$nParam1>>2]|0;
 $19 = HEAP32[$nParam2>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = (_fts3PoslistNearMerge($pOut,$17,$18,$19,$20,$p2)|0);
 HEAP32[$res>>2] = $21;
 $22 = HEAP32[$res>>2]|0;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  $53 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $24 = HEAP32[$pOut>>2]|0;
 $25 = HEAP32[$4>>2]|0;
 $26 = ((($25)) + 28|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $24;
 $29 = $27;
 $30 = (($28) - ($29))|0;
 $31 = (($30) - 1)|0;
 HEAP32[$nNew>>2] = $31;
 $32 = HEAP32[$nNew>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 $34 = ((($33)) + 28|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + ($32)|0);
 $37 = HEAP32[$4>>2]|0;
 $38 = ((($37)) + 32|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$nNew>>2]|0;
 $41 = (($39) - ($40))|0;
 _memset(($36|0),0,($41|0))|0;
 $42 = HEAP32[$nNew>>2]|0;
 $43 = HEAP32[$4>>2]|0;
 $44 = ((($43)) + 32|0);
 HEAP32[$44>>2] = $42;
 $45 = HEAP32[$4>>2]|0;
 $46 = ((($45)) + 28|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$2>>2]|0;
 HEAP32[$48>>2] = $47;
 $49 = HEAP32[$4>>2]|0;
 $50 = ((($49)) + 64|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 HEAP32[$52>>2] = $51;
 $53 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _fts3PoslistNearMerge($pp,$aTmp,$nRight,$nLeft,$pp1,$pp2) {
 $pp = $pp|0;
 $aTmp = $aTmp|0;
 $nRight = $nRight|0;
 $nLeft = $nLeft|0;
 $pp1 = $pp1|0;
 $pp2 = $pp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aTmp2 = 0, $p1 = 0;
 var $p2 = 0, $pTmp1 = 0, $pTmp2 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $p1 = sp + 20|0;
 $p2 = sp + 16|0;
 $pTmp1 = sp + 12|0;
 $pTmp2 = sp + 8|0;
 $aTmp2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $aTmp;
 HEAP32[$2>>2] = $nRight;
 HEAP32[$3>>2] = $nLeft;
 HEAP32[$4>>2] = $pp1;
 HEAP32[$5>>2] = $pp2;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p1>>2] = $7;
 $8 = HEAP32[$5>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p2>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$pTmp1>>2] = $10;
 HEAP32[$res>>2] = 1;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$5>>2]|0;
 (_fts3PoslistPhraseMerge($pTmp1,$11,0,0,$12,$13)|0);
 $14 = HEAP32[$pTmp1>>2]|0;
 HEAP32[$pTmp2>>2] = $14;
 HEAP32[$aTmp2>>2] = $14;
 $15 = HEAP32[$p1>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$p2>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$3>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$4>>2]|0;
 (_fts3PoslistPhraseMerge($pTmp2,$19,1,0,$20,$21)|0);
 $22 = HEAP32[$pTmp1>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ($22|0)!=($23|0);
 if ($24) {
  $25 = HEAP32[$pTmp2>>2]|0;
  $26 = HEAP32[$aTmp2>>2]|0;
  $27 = ($25|0)!=($26|0);
  if ($27) {
   $28 = HEAP32[$0>>2]|0;
   _fts3PoslistMerge($28,$1,$aTmp2);
   $37 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $29 = HEAP32[$pTmp1>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($29|0)!=($30|0);
 if ($31) {
  $32 = HEAP32[$0>>2]|0;
  _fts3PoslistCopy($32,$1);
  $37 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $33 = HEAP32[$pTmp2>>2]|0;
 $34 = HEAP32[$aTmp2>>2]|0;
 $35 = ($33|0)!=($34|0);
 if ($35) {
  $36 = HEAP32[$0>>2]|0;
  _fts3PoslistCopy($36,$aTmp2);
  $37 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($37|0);
 } else {
  HEAP32[$res>>2] = 0;
  $37 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3Fts3DeferredTokenList($p,$ppData,$pnData) {
 $p = $p|0;
 $ppData = $ppData|0;
 $pnData = $pnData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $dummy = 0, $nSkip = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pRet = sp + 12|0;
 $nSkip = sp + 8|0;
 $dummy = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $ppData;
 HEAP32[$3>>2] = $pnData;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_malloc($13)|0);
 HEAP32[$pRet>>2] = $14;
 $15 = HEAP32[$pRet>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3Fts3GetVarint($21,$dummy)|0);
  HEAP32[$nSkip>>2] = $22;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 12|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$nSkip>>2]|0;
  $28 = (($26) - ($27))|0;
  $29 = HEAP32[$3>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$pRet>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  HEAP32[$31>>2] = $30;
  $32 = HEAP32[$pRet>>2]|0;
  $33 = HEAP32[$nSkip>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + ($33)|0);
  $40 = HEAP32[$3>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  _memcpy(($32|0),($39|0),($41|0))|0;
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 } else {
  HEAP32[$0>>2] = 7;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _sqlite3Fts3UpdateMethod($pVtab,$nArg,$apVal,$pRowid) {
 $pVtab = $pVtab|0;
 $nArg = $nArg|0;
 $apVal = $apVal|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aSzDel = 0, $aSzIns = 0, $bInsertDone = 0, $iLangid = 0, $isRemove = 0, $nChng = 0, $or$cond = 0, $p = 0, $pNewRowid = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $p = sp + 32|0;
 $rc = sp + 28|0;
 $isRemove = sp + 24|0;
 $aSzIns = sp + 20|0;
 $aSzDel = sp + 16|0;
 $nChng = sp + 12|0;
 $bInsertDone = sp + 8|0;
 $pNewRowid = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $nArg;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$3>>2] = $pRowid;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $4;
 HEAP32[$rc>>2] = 0;
 HEAP32[$isRemove>>2] = 0;
 HEAP32[$aSzIns>>2] = 0;
 HEAP32[$aSzDel>>2] = 0;
 HEAP32[$nChng>>2] = 0;
 HEAP32[$bInsertDone>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)>(1);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_type($8)|0);
  $10 = ($9|0)==(5);
  if ($10) {
   $11 = HEAP32[$p>>2]|0;
   $12 = ((($11)) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (($13) + 2)|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + ($14<<2)|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3_value_type($17)|0);
   $19 = ($18|0)!=(5);
   if ($19) {
    $20 = HEAP32[$p>>2]|0;
    $21 = HEAP32[$p>>2]|0;
    $22 = ((($21)) + 24|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = (($23) + 2)|0;
    $25 = HEAP32[$2>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (_fts3SpecialInsert($20,$27)|0);
    HEAP32[$rc>>2] = $28;
    $206 = HEAP32[$aSzDel>>2]|0;
    _sqlite3_free($206);
    $207 = HEAP32[$p>>2]|0;
    _sqlite3Fts3SegmentsClose($207);
    $208 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($208|0);
   }
  }
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = ($29|0)>(1);
 if ($30) {
  $31 = HEAP32[$p>>2]|0;
  $32 = ((($31)) + 24|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (2 + ($33))|0;
  $35 = (($34) + 2)|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = (($36) + ($35<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (_sqlite3_value_int($38)|0);
  $40 = ($39|0)<(0);
  if ($40) {
   HEAP32[$rc>>2] = 19;
   $206 = HEAP32[$aSzDel>>2]|0;
   _sqlite3_free($206);
   $207 = HEAP32[$p>>2]|0;
   _sqlite3Fts3SegmentsClose($207);
   $208 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($208|0);
  }
 }
 $41 = HEAP32[$p>>2]|0;
 $42 = ((($41)) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 1)|0;
 $45 = $44<<2;
 $46 = $45<<1;
 $47 = (_sqlite3_malloc($46)|0);
 HEAP32[$aSzDel>>2] = $47;
 $48 = HEAP32[$aSzDel>>2]|0;
 $49 = ($48|0)==(0|0);
 if ($49) {
  HEAP32[$rc>>2] = 7;
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $50 = HEAP32[$p>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + 1)|0;
 $54 = HEAP32[$aSzDel>>2]|0;
 $55 = (($54) + ($53<<2)|0);
 HEAP32[$aSzIns>>2] = $55;
 $56 = HEAP32[$aSzDel>>2]|0;
 $57 = HEAP32[$p>>2]|0;
 $58 = ((($57)) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + 1)|0;
 $61 = $60<<2;
 $62 = $61<<1;
 _memset(($56|0),0,($62|0))|0;
 $63 = HEAP32[$p>>2]|0;
 $64 = (_fts3Writelock($63)|0);
 HEAP32[$rc>>2] = $64;
 $65 = HEAP32[$rc>>2]|0;
 $66 = ($65|0)!=(0);
 if ($66) {
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $67 = HEAP32[$1>>2]|0;
 $68 = ($67|0)>(1);
 do {
  if ($68) {
   $69 = HEAP32[$p>>2]|0;
   $70 = ((($69)) + 40|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)==(0|0);
   if ($72) {
    $73 = HEAP32[$p>>2]|0;
    $74 = ((($73)) + 24|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (3 + ($75))|0;
    $77 = HEAP32[$2>>2]|0;
    $78 = (($77) + ($76<<2)|0);
    $79 = HEAP32[$78>>2]|0;
    HEAP32[$pNewRowid>>2] = $79;
    $80 = HEAP32[$pNewRowid>>2]|0;
    $81 = (_sqlite3_value_type($80)|0);
    $82 = ($81|0)==(5);
    if ($82) {
     $83 = HEAP32[$2>>2]|0;
     $84 = ((($83)) + 4|0);
     $85 = HEAP32[$84>>2]|0;
     HEAP32[$pNewRowid>>2] = $85;
    }
    $86 = HEAP32[$pNewRowid>>2]|0;
    $87 = (_sqlite3_value_type($86)|0);
    $88 = ($87|0)!=(5);
    if ($88) {
     $89 = HEAP32[$2>>2]|0;
     $90 = HEAP32[$89>>2]|0;
     $91 = (_sqlite3_value_type($90)|0);
     $92 = ($91|0)==(5);
     if (!($92)) {
      $93 = HEAP32[$2>>2]|0;
      $94 = HEAP32[$93>>2]|0;
      $95 = (_sqlite3_value_int64($94)|0);
      $96 = tempRet0;
      $97 = HEAP32[$pNewRowid>>2]|0;
      $98 = (_sqlite3_value_int64($97)|0);
      $99 = tempRet0;
      $100 = ($95|0)!=($98|0);
      $101 = ($96|0)!=($99|0);
      $102 = $100 | $101;
      if (!($102)) {
       break;
      }
     }
     $103 = HEAP32[$p>>2]|0;
     $104 = ((($103)) + 12|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (_sqlite3_vtab_on_conflict($105)|0);
     $107 = ($106|0)==(5);
     $108 = HEAP32[$p>>2]|0;
     if ($107) {
      $109 = HEAP32[$pNewRowid>>2]|0;
      $110 = HEAP32[$aSzDel>>2]|0;
      $111 = (_fts3DeleteByRowid($108,$109,$nChng,$110)|0);
      HEAP32[$rc>>2] = $111;
      break;
     } else {
      $112 = HEAP32[$2>>2]|0;
      $113 = HEAP32[$3>>2]|0;
      $114 = (_fts3InsertData($108,$112,$113)|0);
      HEAP32[$rc>>2] = $114;
      HEAP32[$bInsertDone>>2] = 1;
      break;
     }
    }
   }
  }
 } while(0);
 $115 = HEAP32[$rc>>2]|0;
 $116 = ($115|0)!=(0);
 if ($116) {
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $117 = HEAP32[$2>>2]|0;
 $118 = HEAP32[$117>>2]|0;
 $119 = (_sqlite3_value_type($118)|0);
 $120 = ($119|0)!=(5);
 if ($120) {
  $121 = HEAP32[$p>>2]|0;
  $122 = HEAP32[$2>>2]|0;
  $123 = HEAP32[$122>>2]|0;
  $124 = HEAP32[$aSzDel>>2]|0;
  $125 = (_fts3DeleteByRowid($121,$123,$nChng,$124)|0);
  HEAP32[$rc>>2] = $125;
  HEAP32[$isRemove>>2] = 1;
 }
 $126 = HEAP32[$1>>2]|0;
 $127 = ($126|0)>(1);
 $128 = HEAP32[$rc>>2]|0;
 $129 = ($128|0)==(0);
 $or$cond = $127 & $129;
 if ($or$cond) {
  $130 = HEAP32[$p>>2]|0;
  $131 = ((($130)) + 24|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = (2 + ($132))|0;
  $134 = (($133) + 2)|0;
  $135 = HEAP32[$2>>2]|0;
  $136 = (($135) + ($134<<2)|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = (_sqlite3_value_int($137)|0);
  HEAP32[$iLangid>>2] = $138;
  $139 = HEAP32[$bInsertDone>>2]|0;
  $140 = ($139|0)==(0);
  if ($140) {
   $141 = HEAP32[$p>>2]|0;
   $142 = HEAP32[$2>>2]|0;
   $143 = HEAP32[$3>>2]|0;
   $144 = (_fts3InsertData($141,$142,$143)|0);
   HEAP32[$rc>>2] = $144;
   $145 = HEAP32[$rc>>2]|0;
   $146 = ($145|0)==(19);
   if ($146) {
    $147 = HEAP32[$p>>2]|0;
    $148 = ((($147)) + 40|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = ($149|0)==(0|0);
    if ($150) {
     HEAP32[$rc>>2] = 267;
    }
   }
  }
  $151 = HEAP32[$rc>>2]|0;
  $152 = ($151|0)==(0);
  do {
   if ($152) {
    $153 = HEAP32[$isRemove>>2]|0;
    $154 = ($153|0)!=(0);
    if ($154) {
     $155 = HEAP32[$3>>2]|0;
     $156 = $155;
     $157 = $156;
     $158 = HEAP32[$157>>2]|0;
     $159 = (($156) + 4)|0;
     $160 = $159;
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$p>>2]|0;
     $163 = ((($162)) + 264|0);
     $164 = $163;
     $165 = $164;
     $166 = HEAP32[$165>>2]|0;
     $167 = (($164) + 4)|0;
     $168 = $167;
     $169 = HEAP32[$168>>2]|0;
     $170 = ($158|0)!=($166|0);
     $171 = ($161|0)!=($169|0);
     $172 = $170 | $171;
     if (!($172)) {
      break;
     }
    }
    $173 = HEAP32[$p>>2]|0;
    $174 = HEAP32[$iLangid>>2]|0;
    $175 = HEAP32[$3>>2]|0;
    $176 = $175;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (($176) + 4)|0;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = (_fts3PendingTermsDocid($173,0,$174,$178,$181)|0);
    HEAP32[$rc>>2] = $182;
   }
  } while(0);
  $183 = HEAP32[$rc>>2]|0;
  $184 = ($183|0)==(0);
  if ($184) {
   $185 = HEAP32[$p>>2]|0;
   $186 = HEAP32[$iLangid>>2]|0;
   $187 = HEAP32[$2>>2]|0;
   $188 = HEAP32[$aSzIns>>2]|0;
   $189 = (_fts3InsertTerms($185,$186,$187,$188)|0);
   HEAP32[$rc>>2] = $189;
  }
  $190 = HEAP32[$p>>2]|0;
  $191 = ((($190)) + 230|0);
  $192 = HEAP8[$191>>0]|0;
  $193 = ($192<<24>>24)!=(0);
  if ($193) {
   $194 = HEAP32[$p>>2]|0;
   $195 = HEAP32[$aSzIns>>2]|0;
   _fts3InsertDocsize($rc,$194,$195);
  }
  $196 = HEAP32[$nChng>>2]|0;
  $197 = (($196) + 1)|0;
  HEAP32[$nChng>>2] = $197;
 }
 $198 = HEAP32[$p>>2]|0;
 $199 = ((($198)) + 228|0);
 $200 = HEAP8[$199>>0]|0;
 $201 = ($200<<24>>24)!=(0);
 if (!($201)) {
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $202 = HEAP32[$p>>2]|0;
 $203 = HEAP32[$aSzIns>>2]|0;
 $204 = HEAP32[$aSzDel>>2]|0;
 $205 = HEAP32[$nChng>>2]|0;
 _fts3UpdateDocTotals($rc,$202,$203,$204,$205);
 $206 = HEAP32[$aSzDel>>2]|0;
 _sqlite3_free($206);
 $207 = HEAP32[$p>>2]|0;
 _sqlite3Fts3SegmentsClose($207);
 $208 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($208|0);
}
function _fts3SpecialInsert($p,$pVal) {
 $p = $p|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nVal = 0, $rc = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $zVal = sp + 4|0;
 $nVal = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pVal;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3_value_text($3)|0);
 HEAP32[$zVal>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3_value_bytes($5)|0);
 HEAP32[$nVal>>2] = $6;
 $7 = HEAP32[$zVal>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  HEAP32[$0>>2] = 7;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $9 = HEAP32[$nVal>>2]|0;
 $10 = ($9|0)==(8);
 if ($10) {
  $11 = HEAP32[$zVal>>2]|0;
  $12 = (_sqlite3_strnicmp($11,39976,8)|0);
  $13 = (0)==($12|0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (_fts3DoOptimize($14,0)|0);
   HEAP32[$rc>>2] = $15;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 do {
  if ((label|0) == 6) {
   $16 = HEAP32[$nVal>>2]|0;
   $17 = ($16|0)==(7);
   if ($17) {
    $18 = HEAP32[$zVal>>2]|0;
    $19 = (_sqlite3_strnicmp($18,44468,7)|0);
    $20 = (0)==($19|0);
    if ($20) {
     $21 = HEAP32[$1>>2]|0;
     $22 = (_fts3DoRebuild($21)|0);
     HEAP32[$rc>>2] = $22;
     break;
    }
   }
   $23 = HEAP32[$nVal>>2]|0;
   $24 = ($23|0)==(15);
   if ($24) {
    $25 = HEAP32[$zVal>>2]|0;
    $26 = (_sqlite3_strnicmp($25,44476,15)|0);
    $27 = (0)==($26|0);
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = (_fts3DoIntegrityCheck($28)|0);
     HEAP32[$rc>>2] = $29;
     break;
    }
   }
   $30 = HEAP32[$nVal>>2]|0;
   $31 = ($30|0)>(6);
   if ($31) {
    $32 = HEAP32[$zVal>>2]|0;
    $33 = (_sqlite3_strnicmp($32,44492,6)|0);
    $34 = (0)==($33|0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     $36 = HEAP32[$zVal>>2]|0;
     $37 = ((($36)) + 6|0);
     $38 = (_fts3DoIncrmerge($35,$37)|0);
     HEAP32[$rc>>2] = $38;
     break;
    }
   }
   $39 = HEAP32[$nVal>>2]|0;
   $40 = ($39|0)>(10);
   if ($40) {
    $41 = HEAP32[$zVal>>2]|0;
    $42 = (_sqlite3_strnicmp($41,44499,10)|0);
    $43 = (0)==($42|0);
    if ($43) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$zVal>>2]|0;
     $46 = ((($45)) + 10|0);
     $47 = (_fts3DoAutoincrmerge($44,$46)|0);
     HEAP32[$rc>>2] = $47;
     break;
    }
   }
   HEAP32[$rc>>2] = 1;
  }
 } while(0);
 $48 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _fts3Writelock($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 260|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_fts3SqlStmt($5,16,$pStmt,0)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if ($8) {
   $9 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_bind_null($9,1)|0);
   $10 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_step($10)|0);
   $11 = HEAP32[$pStmt>>2]|0;
   $12 = (_sqlite3_reset($11)|0);
   HEAP32[$rc>>2] = $12;
  }
 }
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _fts3DeleteByRowid($p,$pRowid,$pnChng,$aSzDel) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 $pnChng = $pnChng|0;
 $aSzDel = $aSzDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $bFound = 0, $isEmpty = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $bFound = sp + 4|0;
 $isEmpty = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pRowid;
 HEAP32[$2>>2] = $pnChng;
 HEAP32[$3>>2] = $aSzDel;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bFound>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 _fts3DeleteTerms($rc,$4,$5,$6,$bFound);
 $7 = HEAP32[$bFound>>2]|0;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 $or$cond = $8 & $10;
 if (!($or$cond)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 HEAP32[$isEmpty>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (_fts3IsEmpty($11,$12,$isEmpty)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(0);
 if (!($15)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $16 = HEAP32[$isEmpty>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (_fts3DeleteAll($18,1)|0);
  HEAP32[$rc>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  HEAP32[$20>>2] = 0;
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 1)|0;
  $26 = $25<<2;
  $27 = $26<<1;
  _memset(($21|0),0,($27|0))|0;
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) - 1)|0;
 $31 = HEAP32[$2>>2]|0;
 HEAP32[$31>>2] = $30;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 40|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  $36 = HEAP32[$0>>2]|0;
  _fts3SqlExec($rc,$36,0,$1);
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 230|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = ($39<<24>>24)!=(0);
 if (!($40)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $41 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$41,19,$1);
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _fts3InsertData($p,$apVal,$piDocid) {
 $p = $p|0;
 $apVal = $apVal|0;
 $piDocid = $piDocid|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $pContentInsert = 0, $pRowid = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $pContentInsert = sp + 4|0;
 $pRowid = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$3>>2] = $piDocid;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 3)|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pRowid>>2] = $14;
  $15 = HEAP32[$pRowid>>2]|0;
  $16 = (_sqlite3_value_type($15)|0);
  $17 = ($16|0)==(5);
  if ($17) {
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$pRowid>>2] = $20;
  }
  $21 = HEAP32[$pRowid>>2]|0;
  $22 = (_sqlite3_value_type($21)|0);
  $23 = ($22|0)!=(1);
  if ($23) {
   HEAP32[$0>>2] = 19;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  } else {
   $24 = HEAP32[$pRowid>>2]|0;
   $25 = (_sqlite3_value_int64($24)|0);
   $26 = tempRet0;
   $27 = HEAP32[$3>>2]|0;
   $28 = $27;
   $29 = $28;
   HEAP32[$29>>2] = $25;
   $30 = (($28) + 4)|0;
   $31 = $30;
   HEAP32[$31>>2] = $26;
   HEAP32[$0>>2] = 0;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  }
 }
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = (_fts3SqlStmt($8,18,$pContentInsert,$33)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 44|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$pContentInsert>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 24|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + 2)|0;
   $46 = HEAP32[$1>>2]|0;
   $47 = ((($46)) + 24|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + 4)|0;
   $50 = HEAP32[$2>>2]|0;
   $51 = (($50) + ($49<<2)|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3_value_int($52)|0);
   $54 = (_sqlite3_bind_int($41,$45,$53)|0);
   HEAP32[$rc>>2] = $54;
  }
 }
 $55 = HEAP32[$rc>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $57;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 24|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (3 + ($60))|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = (($62) + ($61<<2)|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = (_sqlite3_value_type($64)|0);
 $66 = (5)!=($65|0);
 if ($66) {
  $67 = HEAP32[$2>>2]|0;
  $68 = HEAP32[$67>>2]|0;
  $69 = (_sqlite3_value_type($68)|0);
  $70 = (5)==($69|0);
  if ($70) {
   $71 = HEAP32[$2>>2]|0;
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (_sqlite3_value_type($73)|0);
   $75 = (5)!=($74|0);
   if ($75) {
    HEAP32[$0>>2] = 1;
    $102 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($102|0);
   }
  }
  $76 = HEAP32[$pContentInsert>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 24|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (3 + ($79))|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = (($81) + ($80<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = (_sqlite3_bind_value($76,1,$83)|0);
  HEAP32[$rc>>2] = $84;
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)!=(0);
  if ($86) {
   $87 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $87;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  }
 }
 $88 = HEAP32[$pContentInsert>>2]|0;
 (_sqlite3_step($88)|0);
 $89 = HEAP32[$pContentInsert>>2]|0;
 $90 = (_sqlite3_reset($89)|0);
 HEAP32[$rc>>2] = $90;
 $91 = HEAP32[$1>>2]|0;
 $92 = ((($91)) + 12|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = (_sqlite3_last_insert_rowid($93)|0);
 $95 = tempRet0;
 $96 = HEAP32[$3>>2]|0;
 $97 = $96;
 $98 = $97;
 HEAP32[$98>>2] = $94;
 $99 = (($97) + 4)|0;
 $100 = $99;
 HEAP32[$100>>2] = $95;
 $101 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $101;
 $102 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _fts3PendingTermsDocid($p,$bDelete,$iLangid,$0,$1) {
 $p = $p|0;
 $bDelete = $bDelete|0;
 $iLangid = $iLangid|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $bDelete;
 HEAP32[$5>>2] = $iLangid;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 264|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($16|0)<($24|0);
 $26 = ($13>>>0)<($21>>>0);
 $27 = ($16|0)==($24|0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 do {
  if ($29) {
   label = 6;
  } else {
   $30 = $6;
   $31 = $30;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($30) + 4)|0;
   $34 = $33;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = ((($36)) + 264|0);
   $38 = $37;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = (($38) + 4)|0;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = ($32|0)==($40|0);
   $45 = ($35|0)==($43|0);
   $46 = $44 & $45;
   if ($46) {
    $47 = HEAP32[$3>>2]|0;
    $48 = ((($47)) + 276|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ($49|0)==(0);
    if ($50) {
     label = 6;
     break;
    }
   }
   $51 = HEAP32[$3>>2]|0;
   $52 = ((($51)) + 272|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$5>>2]|0;
   $55 = ($53|0)!=($54|0);
   if ($55) {
    label = 6;
   } else {
    $56 = HEAP32[$3>>2]|0;
    $57 = ((($56)) + 260|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$3>>2]|0;
    $60 = ((($59)) + 256|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($58|0)>($61|0);
    if ($62) {
     label = 6;
    }
   }
  }
 } while(0);
 if ((label|0) == 6) {
  $63 = HEAP32[$3>>2]|0;
  $64 = (_sqlite3Fts3PendingTermsFlush($63)|0);
  HEAP32[$rc>>2] = $64;
  $65 = HEAP32[$rc>>2]|0;
  $66 = ($65|0)!=(0);
  if ($66) {
   $67 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $67;
   $86 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($86|0);
  }
 }
 $68 = $6;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($68) + 4)|0;
 $72 = $71;
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP32[$3>>2]|0;
 $75 = ((($74)) + 264|0);
 $76 = $75;
 $77 = $76;
 HEAP32[$77>>2] = $70;
 $78 = (($76) + 4)|0;
 $79 = $78;
 HEAP32[$79>>2] = $73;
 $80 = HEAP32[$5>>2]|0;
 $81 = HEAP32[$3>>2]|0;
 $82 = ((($81)) + 272|0);
 HEAP32[$82>>2] = $80;
 $83 = HEAP32[$4>>2]|0;
 $84 = HEAP32[$3>>2]|0;
 $85 = ((($84)) + 276|0);
 HEAP32[$85>>2] = $83;
 HEAP32[$2>>2] = 0;
 $86 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _fts3InsertTerms($p,$iLangid,$apVal,$aSz) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $apVal = $apVal|0;
 $aSz = $aSz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iCol = 0, $rc = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $iCol = sp + 8|0;
 $zText = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $apVal;
 HEAP32[$4>>2] = $aSz;
 HEAP32[$i>>2] = 2;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 2)|0;
  $10 = ($5|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = (($11) - 2)|0;
  HEAP32[$iCol>>2] = $12;
  $13 = HEAP32[$iCol>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 32|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = ($19|0)==(0);
  if ($20) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (($22) + ($21<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (_sqlite3_value_text($24)|0);
   HEAP32[$zText>>2] = $25;
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = HEAP32[$zText>>2]|0;
   $29 = HEAP32[$iCol>>2]|0;
   $30 = HEAP32[$iCol>>2]|0;
   $31 = HEAP32[$4>>2]|0;
   $32 = (($31) + ($30<<2)|0);
   $33 = (_fts3PendingTermsAdd($26,$27,$28,$29,$32)|0);
   HEAP32[$rc>>2] = $33;
   $34 = HEAP32[$rc>>2]|0;
   $35 = ($34|0)!=(0);
   if ($35) {
    label = 5;
    break;
   }
   $37 = HEAP32[$i>>2]|0;
   $38 = HEAP32[$3>>2]|0;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (_sqlite3_value_bytes($40)|0);
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 24|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$4>>2]|0;
   $46 = (($45) + ($44<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($41))|0;
   HEAP32[$46>>2] = $48;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 if ((label|0) == 5) {
  $36 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $36;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 return (0)|0;
}
function _fts3InsertDocsize($pRC,$p,$aSz) {
 $pRC = $pRC|0;
 $p = $p|0;
 $aSz = $aSz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $nBlob = 0, $pBlob = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pBlob = sp + 12|0;
 $nBlob = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aSz;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8*10)|0;
 $10 = (_sqlite3_malloc($9)|0);
 HEAP32[$pBlob>>2] = $10;
 $11 = HEAP32[$pBlob>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  HEAP32[$13>>2] = 7;
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$pBlob>>2]|0;
 _fts3EncodeIntArray($16,$17,$18,$nBlob);
 $19 = HEAP32[$1>>2]|0;
 $20 = (_fts3SqlStmt($19,20,$pStmt,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$pBlob>>2]|0;
  _sqlite3_free($23);
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  HEAP32[$25>>2] = $24;
  STACKTOP = sp;return;
 } else {
  $26 = HEAP32[$pStmt>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 264|0);
  $29 = $28;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  (_sqlite3_bind_int64($26,1,$31,$34)|0);
  $35 = HEAP32[$pStmt>>2]|0;
  $36 = HEAP32[$pBlob>>2]|0;
  $37 = HEAP32[$nBlob>>2]|0;
  (_sqlite3_bind_blob($35,2,$36,$37,142)|0);
  $38 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($38)|0);
  $39 = HEAP32[$pStmt>>2]|0;
  $40 = (_sqlite3_reset($39)|0);
  $41 = HEAP32[$0>>2]|0;
  HEAP32[$41>>2] = $40;
  STACKTOP = sp;return;
 }
}
function _fts3UpdateDocTotals($pRC,$p,$aSzIns,$aSzDel,$nChng) {
 $pRC = $pRC|0;
 $p = $p|0;
 $aSzIns = $aSzIns|0;
 $aSzDel = $aSzDel|0;
 $nChng = $nChng|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0;
 var $i = 0, $nBlob = 0, $nStat = 0, $pBlob = 0, $pStmt = 0, $rc = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $pBlob = sp + 28|0;
 $nBlob = sp + 24|0;
 $a = sp + 20|0;
 $pStmt = sp + 16|0;
 $i = sp + 12|0;
 $rc = sp + 8|0;
 $nStat = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aSzIns;
 HEAP32[$3>>2] = $aSzDel;
 HEAP32[$4>>2] = $nChng;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 2)|0;
 HEAP32[$nStat>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$nStat>>2]|0;
 $13 = ($12*14)|0;
 $14 = (_sqlite3_malloc($13)|0);
 HEAP32[$a>>2] = $14;
 $15 = HEAP32[$a>>2]|0;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  HEAP32[$17>>2] = 7;
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$nStat>>2]|0;
 $19 = HEAP32[$a>>2]|0;
 $20 = (($19) + ($18<<2)|0);
 HEAP32[$pBlob>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = (_fts3SqlStmt($21,22,$pStmt,0)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$a>>2]|0;
  _sqlite3_free($25);
  $26 = HEAP32[$rc>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  HEAP32[$27>>2] = $26;
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_bind_int($28,1,0)|0);
 $29 = HEAP32[$pStmt>>2]|0;
 $30 = (_sqlite3_step($29)|0);
 $31 = ($30|0)==(100);
 if ($31) {
  $32 = HEAP32[$nStat>>2]|0;
  $33 = HEAP32[$a>>2]|0;
  $34 = HEAP32[$pStmt>>2]|0;
  $35 = (_sqlite3_column_blob($34,0)|0);
  $36 = HEAP32[$pStmt>>2]|0;
  $37 = (_sqlite3_column_bytes($36,0)|0);
  _fts3DecodeIntArray($32,$33,$35,$37);
 } else {
  $38 = HEAP32[$a>>2]|0;
  $39 = HEAP32[$nStat>>2]|0;
  $40 = $39<<2;
  _memset(($38|0),0,($40|0))|0;
 }
 $41 = HEAP32[$pStmt>>2]|0;
 $42 = (_sqlite3_reset($41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$rc>>2]|0;
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = HEAP32[$a>>2]|0;
  _sqlite3_free($45);
  $46 = HEAP32[$rc>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  HEAP32[$47>>2] = $46;
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$4>>2]|0;
 $49 = ($48|0)<(0);
 if ($49) {
  $50 = HEAP32[$a>>2]|0;
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$4>>2]|0;
  $53 = (0 - ($52))|0;
  $54 = ($51>>>0)<($53>>>0);
  if ($54) {
   $55 = HEAP32[$a>>2]|0;
   HEAP32[$55>>2] = 0;
  } else {
   label = 14;
  }
 } else {
  label = 14;
 }
 if ((label|0) == 14) {
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$a>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($56))|0;
  HEAP32[$57>>2] = $59;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = ((($61)) + 24|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + 1)|0;
  $65 = ($60|0)<($64|0);
  if (!($65)) {
   break;
  }
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  $68 = HEAP32[$a>>2]|0;
  $69 = (($68) + ($67<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$x>>2] = $70;
  $71 = HEAP32[$x>>2]|0;
  $72 = HEAP32[$i>>2]|0;
  $73 = HEAP32[$2>>2]|0;
  $74 = (($73) + ($72<<2)|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($71) + ($75))|0;
  $77 = HEAP32[$i>>2]|0;
  $78 = HEAP32[$3>>2]|0;
  $79 = (($78) + ($77<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = ($76>>>0)<($80>>>0);
  if ($81) {
   HEAP32[$x>>2] = 0;
  } else {
   $82 = HEAP32[$x>>2]|0;
   $83 = HEAP32[$i>>2]|0;
   $84 = HEAP32[$2>>2]|0;
   $85 = (($84) + ($83<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($82) + ($86))|0;
   $88 = HEAP32[$i>>2]|0;
   $89 = HEAP32[$3>>2]|0;
   $90 = (($89) + ($88<<2)|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($87) - ($91))|0;
   HEAP32[$x>>2] = $92;
  }
  $93 = HEAP32[$x>>2]|0;
  $94 = HEAP32[$i>>2]|0;
  $95 = (($94) + 1)|0;
  $96 = HEAP32[$a>>2]|0;
  $97 = (($96) + ($95<<2)|0);
  HEAP32[$97>>2] = $93;
  $98 = HEAP32[$i>>2]|0;
  $99 = (($98) + 1)|0;
  HEAP32[$i>>2] = $99;
 }
 $100 = HEAP32[$nStat>>2]|0;
 $101 = HEAP32[$a>>2]|0;
 $102 = HEAP32[$pBlob>>2]|0;
 _fts3EncodeIntArray($100,$101,$102,$nBlob);
 $103 = HEAP32[$1>>2]|0;
 $104 = (_fts3SqlStmt($103,23,$pStmt,0)|0);
 HEAP32[$rc>>2] = $104;
 $105 = HEAP32[$rc>>2]|0;
 $106 = ($105|0)!=(0);
 if ($106) {
  $107 = HEAP32[$a>>2]|0;
  _sqlite3_free($107);
  $108 = HEAP32[$rc>>2]|0;
  $109 = HEAP32[$0>>2]|0;
  HEAP32[$109>>2] = $108;
  STACKTOP = sp;return;
 } else {
  $110 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_bind_int($110,1,0)|0);
  $111 = HEAP32[$pStmt>>2]|0;
  $112 = HEAP32[$pBlob>>2]|0;
  $113 = HEAP32[$nBlob>>2]|0;
  (_sqlite3_bind_blob($111,2,$112,$113,0)|0);
  $114 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($114)|0);
  $115 = HEAP32[$pStmt>>2]|0;
  $116 = (_sqlite3_reset($115)|0);
  $117 = HEAP32[$0>>2]|0;
  HEAP32[$117>>2] = $116;
  $118 = HEAP32[$a>>2]|0;
  _sqlite3_free($118);
  STACKTOP = sp;return;
 }
}
function _fts3DoOptimize($p,$bReturnDone) {
 $p = $p|0;
 $bReturnDone = $bReturnDone|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bSeenDone = 0, $i = 0, $iLangid = 0, $or$cond = 0, $or$cond3 = 0, $pAllLangid = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $bSeenDone = sp + 20|0;
 $rc = sp + 16|0;
 $pAllLangid = sp + 12|0;
 $rc2 = sp + 8|0;
 $i = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $bReturnDone;
 HEAP32[$bSeenDone>>2] = 0;
 HEAP32[$pAllLangid>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,27,$pAllLangid,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[$pAllLangid>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 272|0);
  $9 = HEAP32[$8>>2]|0;
  (_sqlite3_bind_int($6,1,$9)|0);
  $10 = HEAP32[$pAllLangid>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 248|0);
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3_bind_int($10,2,$13)|0);
  L3: while(1) {
   $14 = HEAP32[$pAllLangid>>2]|0;
   $15 = (_sqlite3_step($14)|0);
   $16 = ($15|0)==(100);
   $17 = HEAP32[$pAllLangid>>2]|0;
   if (!($16)) {
    break;
   }
   $18 = (_sqlite3_column_int($17,0)|0);
   HEAP32[$iLangid>>2] = $18;
   HEAP32[$i>>2] = 0;
   while(1) {
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)==(0);
    if (!($20)) {
     continue L3;
    }
    $21 = HEAP32[$i>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 248|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($21|0)<($24|0);
    if (!($25)) {
     continue L3;
    }
    $26 = HEAP32[$0>>2]|0;
    $27 = HEAP32[$iLangid>>2]|0;
    $28 = HEAP32[$i>>2]|0;
    $29 = (_fts3SegmentMerge($26,$27,$28,-2)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$rc>>2]|0;
    $31 = ($30|0)==(101);
    if ($31) {
     HEAP32[$bSeenDone>>2] = 1;
     HEAP32[$rc>>2] = 0;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = (($32) + 1)|0;
    HEAP32[$i>>2] = $33;
   }
  }
  $34 = (_sqlite3_reset($17)|0);
  HEAP32[$rc2>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $37;
  }
 }
 $38 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegmentsClose($38);
 $39 = HEAP32[$0>>2]|0;
 _sqlite3Fts3PendingTermsClear($39);
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(0);
 $42 = HEAP32[$1>>2]|0;
 $43 = ($42|0)!=(0);
 $or$cond = $41 & $43;
 $44 = HEAP32[$bSeenDone>>2]|0;
 $45 = ($44|0)!=(0);
 $or$cond3 = $or$cond & $45;
 $46 = HEAP32[$rc>>2]|0;
 $47 = $or$cond3 ? 101 : $46;
 STACKTOP = sp;return ($47|0);
}
function _fts3DoRebuild($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aSz = 0, $aSzDel = 0, $aSzIns = 0, $iCol = 0, $iLangid = 0, $nByte = 0, $nEntry = 0, $pStmt = 0, $rc = 0, $rc2 = 0, $vararg_buffer = 0, $z = 0, $zSql = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $rc = sp + 48|0;
 $aSz = sp + 44|0;
 $aSzIns = sp + 40|0;
 $aSzDel = sp + 36|0;
 $pStmt = sp + 32|0;
 $nEntry = sp + 28|0;
 $zSql = sp + 24|0;
 $nByte = sp + 20|0;
 $iCol = sp + 16|0;
 $iLangid = sp + 12|0;
 $z = sp + 8|0;
 $rc2 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_fts3DeleteAll($1,0)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  $145 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($145|0);
 }
 HEAP32[$aSz>>2] = 0;
 HEAP32[$aSzIns>>2] = 0;
 HEAP32[$aSzDel>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 HEAP32[$nEntry>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 216|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$vararg_buffer>>2] = $7;
 $8 = (_sqlite3_mprintf(44520,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $8;
 $9 = HEAP32[$zSql>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$zSql>>2]|0;
  $15 = (_sqlite3_prepare_v2($13,$14,-1,$pStmt,0)|0);
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$zSql>>2]|0;
  _sqlite3_free($16);
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 do {
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 24|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + 1)|0;
   $23 = $22<<2;
   $24 = ($23*3)|0;
   HEAP32[$nByte>>2] = $24;
   $25 = HEAP32[$nByte>>2]|0;
   $26 = (_sqlite3_malloc($25)|0);
   HEAP32[$aSz>>2] = $26;
   $27 = HEAP32[$aSz>>2]|0;
   $28 = ($27|0)==(0|0);
   if ($28) {
    HEAP32[$rc>>2] = 7;
    break;
   } else {
    $29 = HEAP32[$aSz>>2]|0;
    $30 = HEAP32[$nByte>>2]|0;
    _memset(($29|0),0,($30|0))|0;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 24|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 1)|0;
    $35 = HEAP32[$aSz>>2]|0;
    $36 = (($35) + ($34<<2)|0);
    HEAP32[$aSzIns>>2] = $36;
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 24|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + 1)|0;
    $41 = HEAP32[$aSzIns>>2]|0;
    $42 = (($41) + ($40<<2)|0);
    HEAP32[$aSzDel>>2] = $42;
    break;
   }
  }
 } while(0);
 L13: while(1) {
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)==(0);
  if ($44) {
   $45 = HEAP32[$pStmt>>2]|0;
   $46 = (_sqlite3_step($45)|0);
   $47 = (100)==($46|0);
   $146 = $47;
  } else {
   $146 = 0;
  }
  $48 = HEAP32[$0>>2]|0;
  if (!($146)) {
   break;
  }
  $49 = HEAP32[$pStmt>>2]|0;
  $50 = (_langidFromSelect($48,$49)|0);
  HEAP32[$iLangid>>2] = $50;
  $51 = HEAP32[$0>>2]|0;
  $52 = HEAP32[$iLangid>>2]|0;
  $53 = HEAP32[$pStmt>>2]|0;
  $54 = (_sqlite3_column_int64($53,0)|0);
  $55 = tempRet0;
  $56 = (_fts3PendingTermsDocid($51,0,$52,$54,$55)|0);
  HEAP32[$rc>>2] = $56;
  $57 = HEAP32[$aSz>>2]|0;
  $58 = HEAP32[$0>>2]|0;
  $59 = ((($58)) + 24|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + 1)|0;
  $62 = $61<<2;
  _memset(($57|0),0,($62|0))|0;
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)==(0);
   if (!($64)) {
    break;
   }
   $65 = HEAP32[$iCol>>2]|0;
   $66 = HEAP32[$0>>2]|0;
   $67 = ((($66)) + 24|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = ($65|0)<($68|0);
   if (!($69)) {
    break;
   }
   $70 = HEAP32[$iCol>>2]|0;
   $71 = HEAP32[$0>>2]|0;
   $72 = ((($71)) + 32|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($73) + ($70)|0);
   $75 = HEAP8[$74>>0]|0;
   $76 = $75&255;
   $77 = ($76|0)==(0);
   if ($77) {
    $78 = HEAP32[$pStmt>>2]|0;
    $79 = HEAP32[$iCol>>2]|0;
    $80 = (($79) + 1)|0;
    $81 = (_sqlite3_column_text($78,$80)|0);
    HEAP32[$z>>2] = $81;
    $82 = HEAP32[$0>>2]|0;
    $83 = HEAP32[$iLangid>>2]|0;
    $84 = HEAP32[$z>>2]|0;
    $85 = HEAP32[$iCol>>2]|0;
    $86 = HEAP32[$iCol>>2]|0;
    $87 = HEAP32[$aSz>>2]|0;
    $88 = (($87) + ($86<<2)|0);
    $89 = (_fts3PendingTermsAdd($82,$83,$84,$85,$88)|0);
    HEAP32[$rc>>2] = $89;
    $90 = HEAP32[$pStmt>>2]|0;
    $91 = HEAP32[$iCol>>2]|0;
    $92 = (($91) + 1)|0;
    $93 = (_sqlite3_column_bytes($90,$92)|0);
    $94 = HEAP32[$0>>2]|0;
    $95 = ((($94)) + 24|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$aSz>>2]|0;
    $98 = (($97) + ($96<<2)|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = (($99) + ($93))|0;
    HEAP32[$98>>2] = $100;
   }
   $101 = HEAP32[$iCol>>2]|0;
   $102 = (($101) + 1)|0;
   HEAP32[$iCol>>2] = $102;
  }
  $103 = HEAP32[$0>>2]|0;
  $104 = ((($103)) + 230|0);
  $105 = HEAP8[$104>>0]|0;
  $106 = ($105<<24>>24)!=(0);
  if ($106) {
   $107 = HEAP32[$0>>2]|0;
   $108 = HEAP32[$aSz>>2]|0;
   _fts3InsertDocsize($rc,$107,$108);
  }
  $109 = HEAP32[$rc>>2]|0;
  $110 = ($109|0)!=(0);
  if ($110) {
   $111 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_finalize($111)|0);
   HEAP32[$pStmt>>2] = 0;
   continue;
  }
  $112 = HEAP32[$nEntry>>2]|0;
  $113 = (($112) + 1)|0;
  HEAP32[$nEntry>>2] = $113;
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $114 = HEAP32[$iCol>>2]|0;
   $115 = HEAP32[$0>>2]|0;
   $116 = ((($115)) + 24|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = ($114|0)<=($117|0);
   if (!($118)) {
    continue L13;
   }
   $119 = HEAP32[$iCol>>2]|0;
   $120 = HEAP32[$aSz>>2]|0;
   $121 = (($120) + ($119<<2)|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = HEAP32[$iCol>>2]|0;
   $124 = HEAP32[$aSzIns>>2]|0;
   $125 = (($124) + ($123<<2)|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = (($126) + ($122))|0;
   HEAP32[$125>>2] = $127;
   $128 = HEAP32[$iCol>>2]|0;
   $129 = (($128) + 1)|0;
   HEAP32[$iCol>>2] = $129;
  }
 }
 $130 = ((($48)) + 228|0);
 $131 = HEAP8[$130>>0]|0;
 $132 = ($131<<24>>24)!=(0);
 if ($132) {
  $133 = HEAP32[$0>>2]|0;
  $134 = HEAP32[$aSzIns>>2]|0;
  $135 = HEAP32[$aSzDel>>2]|0;
  $136 = HEAP32[$nEntry>>2]|0;
  _fts3UpdateDocTotals($rc,$133,$134,$135,$136);
 }
 $137 = HEAP32[$aSz>>2]|0;
 _sqlite3_free($137);
 $138 = HEAP32[$pStmt>>2]|0;
 $139 = ($138|0)!=(0|0);
 if (!($139)) {
  $145 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($145|0);
 }
 $140 = HEAP32[$pStmt>>2]|0;
 $141 = (_sqlite3_finalize($140)|0);
 HEAP32[$rc2>>2] = $141;
 $142 = HEAP32[$rc>>2]|0;
 $143 = ($142|0)==(0);
 if (!($143)) {
  $145 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($145|0);
 }
 $144 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $144;
 $145 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($145|0);
}
function _fts3DoIntegrityCheck($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $bOk = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $bOk = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$bOk>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_fts3IntegrityCheck($1,$bOk)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(0);
 $5 = HEAP32[$bOk>>2]|0;
 $6 = ($5|0)==(0);
 $or$cond = $4 & $6;
 $$ = $or$cond ? 267 : $2;
 HEAP32[$rc>>2] = $$;
 $7 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _fts3DoIncrmerge($p,$zParam) {
 $p = $p|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nMerge = 0, $nMin = 0, $or$cond = 0, $rc = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $nMin = sp + 8|0;
 $nMerge = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$nMin>>2] = 8;
 HEAP32[$nMerge>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$z>>2] = $2;
 $3 = (_fts3Getint($z)|0);
 HEAP32[$nMerge>>2] = $3;
 $4 = HEAP32[$z>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = ($6|0)==(44);
 if ($7) {
  $8 = HEAP32[$z>>2]|0;
  $9 = ((($8)) + 1|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$z>>2]|0;
   $14 = ((($13)) + 1|0);
   HEAP32[$z>>2] = $14;
   $15 = (_fts3Getint($z)|0);
   HEAP32[$nMin>>2] = $15;
  }
 }
 $16 = HEAP32[$z>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17 << 24 >> 24;
 $19 = ($18|0)!=(0);
 $20 = HEAP32[$nMin>>2]|0;
 $21 = ($20|0)<(2);
 $or$cond = $19 | $21;
 if ($or$cond) {
  HEAP32[$rc>>2] = 1;
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 HEAP32[$rc>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 229|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = ($24<<24>>24)!=(0);
 if (!($25)) {
  $26 = HEAP32[$0>>2]|0;
  _sqlite3Fts3CreateStatTable($rc,$26);
 }
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)==(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$nMerge>>2]|0;
  $31 = HEAP32[$nMin>>2]|0;
  $32 = (_sqlite3Fts3Incrmerge($29,$30,$31)|0);
  HEAP32[$rc>>2] = $32;
 }
 $33 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegmentsClose($33);
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _fts3DoAutoincrmerge($p,$zParam) {
 $p = $p|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zParam;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $3 = (_fts3Getint($2)|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 48|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 48|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(1);
 if ($9) {
  label = 3;
 } else {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 48|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>(16);
  if ($13) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 48|0);
  HEAP32[$15>>2] = 8;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 229|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 if (!($19)) {
  $20 = HEAP32[$1>>2]|0;
  _sqlite3Fts3CreateStatTable($rc,$20);
  $21 = HEAP32[$rc>>2]|0;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $23;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = (_fts3SqlStmt($24,23,$pStmt,0)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 } else {
  $29 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_bind_int($29,1,2)|0);
  $30 = HEAP32[$pStmt>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 48|0);
  $33 = HEAP32[$32>>2]|0;
  (_sqlite3_bind_int($30,2,$33)|0);
  $34 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($34)|0);
  $35 = HEAP32[$pStmt>>2]|0;
  $36 = (_sqlite3_reset($35)|0);
  HEAP32[$rc>>2] = $36;
  $37 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 return (0)|0;
}
function _fts3SegmentMerge($p,$iLangid,$iIndex,$iLevel) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bIgnoreEmpty = 0, $csr = 0, $filter = 0;
 var $iIdx = 0, $iMaxLevel = 0, $iNewLevel = 0, $or$cond = 0, $pWriter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $rc = sp + 104|0;
 $iIdx = sp + 100|0;
 $iNewLevel = sp + 8|0;
 $pWriter = sp + 96|0;
 $filter = sp + 80|0;
 $csr = sp + 24|0;
 $bIgnoreEmpty = sp + 16|0;
 $iMaxLevel = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$iIdx>>2] = 0;
 $4 = $iNewLevel;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 HEAP32[$pWriter>>2] = 0;
 HEAP32[$bIgnoreEmpty>>2] = 0;
 $8 = $iMaxLevel;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = (_sqlite3Fts3SegReaderCursor($12,$13,$14,$15,0,0,1,0,$csr)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)!=(0);
 L1: do {
  if (!($18)) {
   $19 = ((($csr)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)==(0);
   if (!($21)) {
    $22 = HEAP32[$3>>2]|0;
    $23 = ($22|0)!=(-1);
    if ($23) {
     $24 = HEAP32[$0>>2]|0;
     $25 = HEAP32[$1>>2]|0;
     $26 = HEAP32[$2>>2]|0;
     $27 = (_fts3SegmentMaxLevel($24,$25,$26,$iMaxLevel)|0);
     HEAP32[$rc>>2] = $27;
     $28 = HEAP32[$rc>>2]|0;
     $29 = ($28|0)!=(0);
     if ($29) {
      break;
     }
    }
    $30 = HEAP32[$3>>2]|0;
    $31 = ($30|0)==(-2);
    do {
     if ($31) {
      $32 = ((($csr)) + 4|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = ($33|0)==(1);
      if ($34) {
       HEAP32[$rc>>2] = 101;
       break L1;
      } else {
       $35 = $iMaxLevel;
       $36 = $35;
       $37 = HEAP32[$36>>2]|0;
       $38 = (($35) + 4)|0;
       $39 = $38;
       $40 = HEAP32[$39>>2]|0;
       $41 = $iNewLevel;
       $42 = $41;
       HEAP32[$42>>2] = $37;
       $43 = (($41) + 4)|0;
       $44 = $43;
       HEAP32[$44>>2] = $40;
       HEAP32[$bIgnoreEmpty>>2] = 1;
       break;
      }
     } else {
      $45 = HEAP32[$0>>2]|0;
      $46 = HEAP32[$1>>2]|0;
      $47 = HEAP32[$2>>2]|0;
      $48 = HEAP32[$3>>2]|0;
      $49 = (($48) + 1)|0;
      $50 = (_getAbsoluteLevel($45,$46,$47,$49)|0);
      $51 = tempRet0;
      $52 = $iNewLevel;
      $53 = $52;
      HEAP32[$53>>2] = $50;
      $54 = (($52) + 4)|0;
      $55 = $54;
      HEAP32[$55>>2] = $51;
      $56 = HEAP32[$0>>2]|0;
      $57 = HEAP32[$1>>2]|0;
      $58 = HEAP32[$2>>2]|0;
      $59 = HEAP32[$3>>2]|0;
      $60 = (($59) + 1)|0;
      $61 = (_fts3AllocateSegdirIdx($56,$57,$58,$60,$iIdx)|0);
      HEAP32[$rc>>2] = $61;
      $62 = HEAP32[$3>>2]|0;
      $63 = ($62|0)!=(-1);
      if ($63) {
       $64 = $iNewLevel;
       $65 = $64;
       $66 = HEAP32[$65>>2]|0;
       $67 = (($64) + 4)|0;
       $68 = $67;
       $69 = HEAP32[$68>>2]|0;
       $70 = $iMaxLevel;
       $71 = $70;
       $72 = HEAP32[$71>>2]|0;
       $73 = (($70) + 4)|0;
       $74 = $73;
       $75 = HEAP32[$74>>2]|0;
       $76 = ($69|0)>($75|0);
       $77 = ($66>>>0)>($72>>>0);
       $78 = ($69|0)==($75|0);
       $79 = $78 & $77;
       $80 = $76 | $79;
       $82 = $80;
      } else {
       $82 = 0;
      }
      $81 = $82&1;
      HEAP32[$bIgnoreEmpty>>2] = $81;
     }
    } while(0);
    $83 = HEAP32[$rc>>2]|0;
    $84 = ($83|0)!=(0);
    if (!($84)) {
     ;HEAP32[$filter>>2]=0|0;HEAP32[$filter+4>>2]=0|0;HEAP32[$filter+8>>2]=0|0;HEAP32[$filter+12>>2]=0|0;
     $85 = ((($filter)) + 12|0);
     HEAP32[$85>>2] = 1;
     $86 = HEAP32[$bIgnoreEmpty>>2]|0;
     $87 = ($86|0)!=(0);
     $88 = $87 ? 2 : 0;
     $89 = ((($filter)) + 12|0);
     $90 = HEAP32[$89>>2]|0;
     $91 = $90 | $88;
     HEAP32[$89>>2] = $91;
     $92 = HEAP32[$0>>2]|0;
     $93 = (_sqlite3Fts3SegReaderStart($92,$csr,$filter)|0);
     HEAP32[$rc>>2] = $93;
     while(1) {
      $94 = HEAP32[$rc>>2]|0;
      $95 = (0)==($94|0);
      if (!($95)) {
       break;
      }
      $96 = HEAP32[$0>>2]|0;
      $97 = (_sqlite3Fts3SegReaderStep($96,$csr)|0);
      HEAP32[$rc>>2] = $97;
      $98 = HEAP32[$rc>>2]|0;
      $99 = ($98|0)!=(100);
      if ($99) {
       break;
      }
      $100 = HEAP32[$0>>2]|0;
      $101 = ((($csr)) + 40|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ((($csr)) + 44|0);
      $104 = HEAP32[$103>>2]|0;
      $105 = ((($csr)) + 48|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = ((($csr)) + 52|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = (_fts3SegWriterAdd($100,$pWriter,1,$102,$104,$106,$108)|0);
      HEAP32[$rc>>2] = $109;
     }
     $110 = HEAP32[$rc>>2]|0;
     $111 = ($110|0)!=(0);
     if (!($111)) {
      $112 = HEAP32[$3>>2]|0;
      $113 = ($112|0)!=(-1);
      if ($113) {
       $114 = HEAP32[$0>>2]|0;
       $115 = HEAP32[$1>>2]|0;
       $116 = HEAP32[$2>>2]|0;
       $117 = HEAP32[$3>>2]|0;
       $118 = HEAP32[$csr>>2]|0;
       $119 = ((($csr)) + 4|0);
       $120 = HEAP32[$119>>2]|0;
       $121 = (_fts3DeleteSegdir($114,$115,$116,$117,$118,$120)|0);
       HEAP32[$rc>>2] = $121;
       $122 = HEAP32[$rc>>2]|0;
       $123 = ($122|0)==(0);
       $124 = HEAP32[$pWriter>>2]|0;
       $125 = ($124|0)!=(0|0);
       $or$cond = $123 & $125;
       if (!($or$cond)) {
        break;
       }
      } else {
       $$old = HEAP32[$pWriter>>2]|0;
       $$old1 = ($$old|0)!=(0|0);
       if (!($$old1)) {
        break;
       }
      }
      $126 = HEAP32[$0>>2]|0;
      $127 = HEAP32[$pWriter>>2]|0;
      $128 = $iNewLevel;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = (($128) + 4)|0;
      $132 = $131;
      $133 = HEAP32[$132>>2]|0;
      $134 = HEAP32[$iIdx>>2]|0;
      $135 = (_fts3SegWriterFlush($126,$127,$130,$133,$134)|0);
      HEAP32[$rc>>2] = $135;
      $136 = HEAP32[$rc>>2]|0;
      $137 = ($136|0)==(0);
      if ($137) {
       $138 = HEAP32[$3>>2]|0;
       $139 = ($138|0)==(-1);
       if (!($139)) {
        $140 = $iNewLevel;
        $141 = $140;
        $142 = HEAP32[$141>>2]|0;
        $143 = (($140) + 4)|0;
        $144 = $143;
        $145 = HEAP32[$144>>2]|0;
        $146 = $iMaxLevel;
        $147 = $146;
        $148 = HEAP32[$147>>2]|0;
        $149 = (($146) + 4)|0;
        $150 = $149;
        $151 = HEAP32[$150>>2]|0;
        $152 = ($145|0)<($151|0);
        $153 = ($142>>>0)<($148>>>0);
        $154 = ($145|0)==($151|0);
        $155 = $154 & $153;
        $156 = $152 | $155;
        if (!($156)) {
         break;
        }
       }
       $157 = HEAP32[$0>>2]|0;
       $158 = $iNewLevel;
       $159 = $158;
       $160 = HEAP32[$159>>2]|0;
       $161 = (($158) + 4)|0;
       $162 = $161;
       $163 = HEAP32[$162>>2]|0;
       $164 = HEAP32[$pWriter>>2]|0;
       $165 = ((($164)) + 56|0);
       $166 = $165;
       $167 = $166;
       $168 = HEAP32[$167>>2]|0;
       $169 = (($166) + 4)|0;
       $170 = $169;
       $171 = HEAP32[$170>>2]|0;
       $172 = (_fts3PromoteSegments($157,$160,$163,$168,$171)|0);
       HEAP32[$rc>>2] = $172;
      }
     }
    }
   }
  }
 } while(0);
 $173 = HEAP32[$pWriter>>2]|0;
 _fts3SegWriterFree($173);
 _sqlite3Fts3SegReaderFinish($csr);
 $174 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($174|0);
}
function _sqlite3Fts3PendingTermsClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pElem = 0, $pHash = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $pElem = sp + 8|0;
 $pHash = sp + 4|0;
 $pList = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 248|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 252|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + (($6*24)|0)|0);
  $11 = ((($10)) + 4|0);
  HEAP32[$pHash>>2] = $11;
  $12 = HEAP32[$pHash>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pElem>>2] = $14;
  while(1) {
   $15 = HEAP32[$pElem>>2]|0;
   $16 = ($15|0)!=(0|0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$pElem>>2]|0;
   $18 = ((($17)) + 8|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$pList>>2] = $19;
   $20 = HEAP32[$pList>>2]|0;
   _fts3PendingListDelete($20);
   $21 = HEAP32[$pElem>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pElem>>2] = $22;
  }
  $23 = HEAP32[$pHash>>2]|0;
  _sqlite3Fts3HashClear($23);
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 260|0);
 HEAP32[$27>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3SegmentMaxLevel($p,$iLangid,$iIndex,$pnMax) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $pnMax = $pnMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $iIndex;
 HEAP32[$4>>2] = $pnMax;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_fts3SqlStmt($5,15,$pStmt,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $10 = HEAP32[$pStmt>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_getAbsoluteLevel($11,$12,$13,0)|0);
 $15 = tempRet0;
 (_sqlite3_bind_int64($10,1,$14,$15)|0);
 $16 = HEAP32[$pStmt>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$3>>2]|0;
 $20 = (_getAbsoluteLevel($17,$18,$19,1023)|0);
 $21 = tempRet0;
 (_sqlite3_bind_int64($16,2,$20,$21)|0);
 $22 = HEAP32[$pStmt>>2]|0;
 $23 = (_sqlite3_step($22)|0);
 $24 = (100)==($23|0);
 if ($24) {
  $25 = HEAP32[$pStmt>>2]|0;
  $26 = (_sqlite3_column_int64($25,0)|0);
  $27 = tempRet0;
  $28 = HEAP32[$4>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $27;
 }
 $33 = HEAP32[$pStmt>>2]|0;
 $34 = (_sqlite3_reset($33)|0);
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _fts3AllocateSegdirIdx($p,$iLangid,$iIndex,$iLevel,$piIdx) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $piIdx = $piIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iNext = 0, $pNextIdx = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $rc = sp + 8|0;
 $pNextIdx = sp + 4|0;
 $iNext = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $piIdx;
 HEAP32[$iNext>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_fts3SqlStmt($5,8,$pNextIdx,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$pNextIdx>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$3>>2]|0;
  $14 = (_getAbsoluteLevel($10,$11,$12,$13)|0);
  $15 = tempRet0;
  (_sqlite3_bind_int64($9,1,$14,$15)|0);
  $16 = HEAP32[$pNextIdx>>2]|0;
  $17 = (_sqlite3_step($16)|0);
  $18 = (100)==($17|0);
  if ($18) {
   $19 = HEAP32[$pNextIdx>>2]|0;
   $20 = (_sqlite3_column_int($19,0)|0);
   HEAP32[$iNext>>2] = $20;
  }
  $21 = HEAP32[$pNextIdx>>2]|0;
  $22 = (_sqlite3_reset($21)|0);
  HEAP32[$rc>>2] = $22;
 }
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)==(0);
 if (!($24)) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $25 = HEAP32[$iNext>>2]|0;
 $26 = ($25|0)>=(16);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = (_fts3SegmentMerge($27,$28,$29,$30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$4>>2]|0;
  HEAP32[$32>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  $33 = HEAP32[$iNext>>2]|0;
  $34 = HEAP32[$4>>2]|0;
  HEAP32[$34>>2] = $33;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _fts3SegWriterAdd($p,$ppWriter,$isCopyTerm,$zTerm,$nTerm,$aDoclist,$nDoclist) {
 $p = $p|0;
 $ppWriter = $ppWriter|0;
 $isCopyTerm = $isCopyTerm|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $nData = 0, $nPrefix = 0, $nReq = 0, $nSuffix = 0, $pStmt = 0;
 var $pWriter = 0, $rc = 0, $rc1 = 0, $zNew = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $6 = sp + 44|0;
 $7 = sp + 40|0;
 $nPrefix = sp + 36|0;
 $nSuffix = sp + 32|0;
 $nReq = sp + 28|0;
 $nData = sp + 24|0;
 $pWriter = sp + 20|0;
 $rc = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc1 = sp + 8|0;
 $aNew = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $ppWriter;
 HEAP32[$3>>2] = $isCopyTerm;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $aDoclist;
 HEAP32[$7>>2] = $nDoclist;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pWriter>>2] = $9;
 $10 = HEAP32[$pWriter>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $12 = (_sqlite3_malloc(64)|0);
  HEAP32[$pWriter>>2] = $12;
  $13 = HEAP32[$pWriter>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   HEAP32[$0>>2] = 7;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $15 = HEAP32[$pWriter>>2]|0;
  dest=$15; stop=dest+64|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $16 = HEAP32[$pWriter>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 224|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3_malloc($20)|0);
  $22 = HEAP32[$pWriter>>2]|0;
  $23 = ((($22)) + 48|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$pWriter>>2]|0;
  $25 = ((($24)) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   HEAP32[$0>>2] = 7;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 224|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$pWriter>>2]|0;
  $32 = ((($31)) + 40|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$1>>2]|0;
  $34 = (_fts3SqlStmt($33,10,$pStmt,0)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $37;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $38 = HEAP32[$pStmt>>2]|0;
  $39 = (_sqlite3_step($38)|0);
  $40 = (100)==($39|0);
  if ($40) {
   $41 = HEAP32[$pStmt>>2]|0;
   $42 = (_sqlite3_column_int64($41,0)|0);
   $43 = tempRet0;
   $44 = HEAP32[$pWriter>>2]|0;
   $45 = ((($44)) + 16|0);
   $46 = $45;
   $47 = $46;
   HEAP32[$47>>2] = $42;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $43;
   $50 = HEAP32[$pWriter>>2]|0;
   $51 = ((($50)) + 16|0);
   $52 = $51;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($52) + 4)|0;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$pWriter>>2]|0;
   $59 = ((($58)) + 8|0);
   $60 = $59;
   $61 = $60;
   HEAP32[$61>>2] = $54;
   $62 = (($60) + 4)|0;
   $63 = $62;
   HEAP32[$63>>2] = $57;
  }
  $64 = HEAP32[$pStmt>>2]|0;
  $65 = (_sqlite3_reset($64)|0);
  HEAP32[$rc>>2] = $65;
  $66 = HEAP32[$rc>>2]|0;
  $67 = ($66|0)!=(0);
  if ($67) {
   $68 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $68;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
 }
 $69 = HEAP32[$pWriter>>2]|0;
 $70 = ((($69)) + 44|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$nData>>2] = $71;
 $72 = HEAP32[$pWriter>>2]|0;
 $73 = ((($72)) + 24|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$pWriter>>2]|0;
 $76 = ((($75)) + 28|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = HEAP32[$4>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = (_fts3PrefixCompress($74,$77,$78,$79)|0);
 HEAP32[$nPrefix>>2] = $80;
 $81 = HEAP32[$5>>2]|0;
 $82 = HEAP32[$nPrefix>>2]|0;
 $83 = (($81) - ($82))|0;
 HEAP32[$nSuffix>>2] = $83;
 $84 = HEAP32[$nPrefix>>2]|0;
 $85 = ($84|0)<(0);
 $86 = $85 << 31 >> 31;
 $87 = (_sqlite3Fts3VarintLen($84,$86)|0);
 $88 = HEAP32[$nSuffix>>2]|0;
 $89 = ($88|0)<(0);
 $90 = $89 << 31 >> 31;
 $91 = (_sqlite3Fts3VarintLen($88,$90)|0);
 $92 = (($87) + ($91))|0;
 $93 = HEAP32[$nSuffix>>2]|0;
 $94 = (($92) + ($93))|0;
 $95 = HEAP32[$7>>2]|0;
 $96 = ($95|0)<(0);
 $97 = $96 << 31 >> 31;
 $98 = (_sqlite3Fts3VarintLen($95,$97)|0);
 $99 = (($94) + ($98))|0;
 $100 = HEAP32[$7>>2]|0;
 $101 = (($99) + ($100))|0;
 HEAP32[$nReq>>2] = $101;
 $102 = HEAP32[$nData>>2]|0;
 $103 = ($102|0)>(0);
 do {
  if ($103) {
   $104 = HEAP32[$nData>>2]|0;
   $105 = HEAP32[$nReq>>2]|0;
   $106 = (($104) + ($105))|0;
   $107 = HEAP32[$1>>2]|0;
   $108 = ((($107)) + 224|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = ($106|0)>($109|0);
   if ($110) {
    $111 = HEAP32[$1>>2]|0;
    $112 = HEAP32[$pWriter>>2]|0;
    $113 = ((($112)) + 16|0);
    $114 = $113;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = (_i64Add(($116|0),($119|0),1,0)|0);
    $121 = tempRet0;
    $122 = $113;
    $123 = $122;
    HEAP32[$123>>2] = $120;
    $124 = (($122) + 4)|0;
    $125 = $124;
    HEAP32[$125>>2] = $121;
    $126 = HEAP32[$pWriter>>2]|0;
    $127 = ((($126)) + 48|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$nData>>2]|0;
    $130 = (_fts3WriteSegment($111,$116,$119,$128,$129)|0);
    HEAP32[$rc1>>2] = $130;
    $131 = HEAP32[$rc1>>2]|0;
    $132 = ($131|0)!=(0);
    if ($132) {
     $133 = HEAP32[$rc1>>2]|0;
     HEAP32[$0>>2] = $133;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
    $134 = HEAP32[$1>>2]|0;
    $135 = ((($134)) + 52|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = (($136) + 1)|0;
    HEAP32[$135>>2] = $137;
    $138 = HEAP32[$1>>2]|0;
    $139 = HEAP32[$pWriter>>2]|0;
    $140 = HEAP32[$3>>2]|0;
    $141 = HEAP32[$4>>2]|0;
    $142 = HEAP32[$nPrefix>>2]|0;
    $143 = (($142) + 1)|0;
    $144 = (_fts3NodeAddTerm($138,$139,$140,$141,$143)|0);
    HEAP32[$rc1>>2] = $144;
    $145 = HEAP32[$rc1>>2]|0;
    $146 = ($145|0)!=(0);
    if (!($146)) {
     HEAP32[$nData>>2] = 0;
     $148 = HEAP32[$pWriter>>2]|0;
     $149 = ((($148)) + 28|0);
     HEAP32[$149>>2] = 0;
     HEAP32[$nPrefix>>2] = 0;
     $150 = HEAP32[$5>>2]|0;
     HEAP32[$nSuffix>>2] = $150;
     $151 = HEAP32[$5>>2]|0;
     $152 = ($151|0)<(0);
     $153 = $152 << 31 >> 31;
     $154 = (_sqlite3Fts3VarintLen($151,$153)|0);
     $155 = (1 + ($154))|0;
     $156 = HEAP32[$5>>2]|0;
     $157 = (($155) + ($156))|0;
     $158 = HEAP32[$7>>2]|0;
     $159 = ($158|0)<(0);
     $160 = $159 << 31 >> 31;
     $161 = (_sqlite3Fts3VarintLen($158,$160)|0);
     $162 = (($157) + ($161))|0;
     $163 = HEAP32[$7>>2]|0;
     $164 = (($162) + ($163))|0;
     HEAP32[$nReq>>2] = $164;
     break;
    }
    $147 = HEAP32[$rc1>>2]|0;
    HEAP32[$0>>2] = $147;
    $293 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($293|0);
   }
  }
 } while(0);
 $165 = HEAP32[$nReq>>2]|0;
 $166 = ($165|0)<(0);
 $167 = $166 << 31 >> 31;
 $168 = HEAP32[$pWriter>>2]|0;
 $169 = ((($168)) + 56|0);
 $170 = $169;
 $171 = $170;
 $172 = HEAP32[$171>>2]|0;
 $173 = (($170) + 4)|0;
 $174 = $173;
 $175 = HEAP32[$174>>2]|0;
 $176 = (_i64Add(($172|0),($175|0),($165|0),($167|0))|0);
 $177 = tempRet0;
 $178 = $169;
 $179 = $178;
 HEAP32[$179>>2] = $176;
 $180 = (($178) + 4)|0;
 $181 = $180;
 HEAP32[$181>>2] = $177;
 $182 = HEAP32[$nReq>>2]|0;
 $183 = HEAP32[$pWriter>>2]|0;
 $184 = ((($183)) + 40|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = ($182|0)>($185|0);
 do {
  if ($186) {
   $187 = HEAP32[$pWriter>>2]|0;
   $188 = ((($187)) + 48|0);
   $189 = HEAP32[$188>>2]|0;
   $190 = HEAP32[$nReq>>2]|0;
   $191 = (_sqlite3_realloc($189,$190)|0);
   HEAP32[$aNew>>2] = $191;
   $192 = HEAP32[$aNew>>2]|0;
   $193 = ($192|0)!=(0|0);
   if ($193) {
    $194 = HEAP32[$aNew>>2]|0;
    $195 = HEAP32[$pWriter>>2]|0;
    $196 = ((($195)) + 48|0);
    HEAP32[$196>>2] = $194;
    $197 = HEAP32[$nReq>>2]|0;
    $198 = HEAP32[$pWriter>>2]|0;
    $199 = ((($198)) + 40|0);
    HEAP32[$199>>2] = $197;
    break;
   }
   HEAP32[$0>>2] = 7;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
 } while(0);
 $200 = HEAP32[$nData>>2]|0;
 $201 = HEAP32[$pWriter>>2]|0;
 $202 = ((($201)) + 48|0);
 $203 = HEAP32[$202>>2]|0;
 $204 = (($203) + ($200)|0);
 $205 = HEAP32[$nPrefix>>2]|0;
 $206 = ($205|0)<(0);
 $207 = $206 << 31 >> 31;
 $208 = (_sqlite3Fts3PutVarint($204,$205,$207)|0);
 $209 = HEAP32[$nData>>2]|0;
 $210 = (($209) + ($208))|0;
 HEAP32[$nData>>2] = $210;
 $211 = HEAP32[$nData>>2]|0;
 $212 = HEAP32[$pWriter>>2]|0;
 $213 = ((($212)) + 48|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = (($214) + ($211)|0);
 $216 = HEAP32[$nSuffix>>2]|0;
 $217 = ($216|0)<(0);
 $218 = $217 << 31 >> 31;
 $219 = (_sqlite3Fts3PutVarint($215,$216,$218)|0);
 $220 = HEAP32[$nData>>2]|0;
 $221 = (($220) + ($219))|0;
 HEAP32[$nData>>2] = $221;
 $222 = HEAP32[$nData>>2]|0;
 $223 = HEAP32[$pWriter>>2]|0;
 $224 = ((($223)) + 48|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + ($222)|0);
 $227 = HEAP32[$nPrefix>>2]|0;
 $228 = HEAP32[$4>>2]|0;
 $229 = (($228) + ($227)|0);
 $230 = HEAP32[$nSuffix>>2]|0;
 _memcpy(($226|0),($229|0),($230|0))|0;
 $231 = HEAP32[$nSuffix>>2]|0;
 $232 = HEAP32[$nData>>2]|0;
 $233 = (($232) + ($231))|0;
 HEAP32[$nData>>2] = $233;
 $234 = HEAP32[$nData>>2]|0;
 $235 = HEAP32[$pWriter>>2]|0;
 $236 = ((($235)) + 48|0);
 $237 = HEAP32[$236>>2]|0;
 $238 = (($237) + ($234)|0);
 $239 = HEAP32[$7>>2]|0;
 $240 = ($239|0)<(0);
 $241 = $240 << 31 >> 31;
 $242 = (_sqlite3Fts3PutVarint($238,$239,$241)|0);
 $243 = HEAP32[$nData>>2]|0;
 $244 = (($243) + ($242))|0;
 HEAP32[$nData>>2] = $244;
 $245 = HEAP32[$nData>>2]|0;
 $246 = HEAP32[$pWriter>>2]|0;
 $247 = ((($246)) + 48|0);
 $248 = HEAP32[$247>>2]|0;
 $249 = (($248) + ($245)|0);
 $250 = HEAP32[$6>>2]|0;
 $251 = HEAP32[$7>>2]|0;
 _memcpy(($249|0),($250|0),($251|0))|0;
 $252 = HEAP32[$nData>>2]|0;
 $253 = HEAP32[$7>>2]|0;
 $254 = (($252) + ($253))|0;
 $255 = HEAP32[$pWriter>>2]|0;
 $256 = ((($255)) + 44|0);
 HEAP32[$256>>2] = $254;
 $257 = HEAP32[$3>>2]|0;
 $258 = ($257|0)!=(0);
 if ($258) {
  $259 = HEAP32[$5>>2]|0;
  $260 = HEAP32[$pWriter>>2]|0;
  $261 = ((($260)) + 32|0);
  $262 = HEAP32[$261>>2]|0;
  $263 = ($259|0)>($262|0);
  do {
   if ($263) {
    $264 = HEAP32[$pWriter>>2]|0;
    $265 = ((($264)) + 36|0);
    $266 = HEAP32[$265>>2]|0;
    $267 = HEAP32[$5>>2]|0;
    $268 = $267<<1;
    $269 = (_sqlite3_realloc($266,$268)|0);
    HEAP32[$zNew>>2] = $269;
    $270 = HEAP32[$zNew>>2]|0;
    $271 = ($270|0)!=(0|0);
    if ($271) {
     $272 = HEAP32[$5>>2]|0;
     $273 = $272<<1;
     $274 = HEAP32[$pWriter>>2]|0;
     $275 = ((($274)) + 32|0);
     HEAP32[$275>>2] = $273;
     $276 = HEAP32[$zNew>>2]|0;
     $277 = HEAP32[$pWriter>>2]|0;
     $278 = ((($277)) + 36|0);
     HEAP32[$278>>2] = $276;
     $279 = HEAP32[$zNew>>2]|0;
     $280 = HEAP32[$pWriter>>2]|0;
     $281 = ((($280)) + 24|0);
     HEAP32[$281>>2] = $279;
     break;
    }
    HEAP32[$0>>2] = 7;
    $293 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($293|0);
   }
  } while(0);
  $282 = HEAP32[$pWriter>>2]|0;
  $283 = ((($282)) + 24|0);
  $284 = HEAP32[$283>>2]|0;
  $285 = HEAP32[$4>>2]|0;
  $286 = HEAP32[$5>>2]|0;
  _memcpy(($284|0),($285|0),($286|0))|0;
 } else {
  $287 = HEAP32[$4>>2]|0;
  $288 = HEAP32[$pWriter>>2]|0;
  $289 = ((($288)) + 24|0);
  HEAP32[$289>>2] = $287;
 }
 $290 = HEAP32[$5>>2]|0;
 $291 = HEAP32[$pWriter>>2]|0;
 $292 = ((($291)) + 28|0);
 HEAP32[$292>>2] = $290;
 HEAP32[$0>>2] = 0;
 $293 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($293|0);
}
function _fts3DeleteSegdir($p,$iLangid,$iIndex,$iLevel,$apSegment,$nReader) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $apSegment = $apSegment|0;
 $nReader = $nReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pDelete = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $pDelete = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $iIndex;
 HEAP32[$4>>2] = $iLevel;
 HEAP32[$5>>2] = $apSegment;
 HEAP32[$6>>2] = $nReader;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDelete>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$6>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$5>>2]|0;
  $15 = (($14) + ($13<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_fts3DeleteSegment($12,$16)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $22;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $23 = HEAP32[$4>>2]|0;
 $24 = ($23|0)==(-2);
 $25 = HEAP32[$1>>2]|0;
 if ($24) {
  $26 = (_fts3SqlStmt($25,26,$pDelete,0)|0);
  HEAP32[$rc>>2] = $26;
  $27 = HEAP32[$rc>>2]|0;
  $28 = ($27|0)==(0);
  if ($28) {
   $29 = HEAP32[$pDelete>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   $33 = (_getAbsoluteLevel($30,$31,$32,0)|0);
   $34 = tempRet0;
   (_sqlite3_bind_int64($29,1,$33,$34)|0);
   $35 = HEAP32[$pDelete>>2]|0;
   $36 = HEAP32[$1>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = HEAP32[$3>>2]|0;
   $39 = (_getAbsoluteLevel($36,$37,$38,1023)|0);
   $40 = tempRet0;
   (_sqlite3_bind_int64($35,2,$39,$40)|0);
  }
 } else {
  $41 = (_fts3SqlStmt($25,16,$pDelete,0)|0);
  HEAP32[$rc>>2] = $41;
  $42 = HEAP32[$rc>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$pDelete>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = HEAP32[$3>>2]|0;
   $48 = HEAP32[$4>>2]|0;
   $49 = (_getAbsoluteLevel($45,$46,$47,$48)|0);
   $50 = tempRet0;
   (_sqlite3_bind_int64($44,1,$49,$50)|0);
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$pDelete>>2]|0;
  (_sqlite3_step($53)|0);
  $54 = HEAP32[$pDelete>>2]|0;
  $55 = (_sqlite3_reset($54)|0);
  HEAP32[$rc>>2] = $55;
 }
 $56 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $56;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _fts3SegWriterFlush($p,$pWriter,$0,$1,$iIdx) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $iLast = 0, $iLastLeaf = 0, $nRoot = 0, $rc = 0, $zRoot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 16|0;
 $5 = sp + 36|0;
 $rc = sp + 32|0;
 $iLast = sp + 8|0;
 $iLastLeaf = sp;
 $zRoot = sp + 28|0;
 $nRoot = sp + 24|0;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pWriter;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $iIdx;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $114 = HEAP32[$2>>2]|0;
  $115 = $4;
  $116 = $115;
  $117 = HEAP32[$116>>2]|0;
  $118 = (($115) + 4)|0;
  $119 = $118;
  $120 = HEAP32[$119>>2]|0;
  $121 = HEAP32[$5>>2]|0;
  $122 = HEAP32[$3>>2]|0;
  $123 = ((($122)) + 56|0);
  $124 = $123;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (($124) + 4)|0;
  $128 = $127;
  $129 = HEAP32[$128>>2]|0;
  $130 = HEAP32[$3>>2]|0;
  $131 = ((($130)) + 48|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = HEAP32[$3>>2]|0;
  $134 = ((($133)) + 44|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (_fts3WriteSegdir($114,$117,$120,$121,0,0,0,0,0,0,$126,$129,$132,$135)|0);
  HEAP32[$rc>>2] = $136;
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 52|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($139) + 1)|0;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 $13 = $iLast;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 HEAP32[$zRoot>>2] = 0;
 HEAP32[$nRoot>>2] = 0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = $iLastLeaf;
 $26 = $25;
 HEAP32[$26>>2] = $21;
 $27 = (($25) + 4)|0;
 $28 = $27;
 HEAP32[$28>>2] = $24;
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 $31 = ((($30)) + 16|0);
 $32 = $31;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (_i64Add(($34|0),($37|0),1,0)|0);
 $39 = tempRet0;
 $40 = $31;
 $41 = $40;
 HEAP32[$41>>2] = $38;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = HEAP32[$3>>2]|0;
 $45 = ((($44)) + 48|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$3>>2]|0;
 $48 = ((($47)) + 44|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (_fts3WriteSegment($29,$34,$37,$46,$49)|0);
 HEAP32[$rc>>2] = $50;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$2>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = $57;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$3>>2]|0;
  $65 = ((($64)) + 16|0);
  $66 = $65;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($66) + 4)|0;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (_fts3NodeWrite($53,$55,1,$60,$63,$68,$71,$iLast,$zRoot,$nRoot)|0);
  HEAP32[$rc>>2] = $72;
 }
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)==(0);
 if (!($74)) {
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 52|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($139) + 1)|0;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 $75 = HEAP32[$2>>2]|0;
 $76 = $4;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($76) + 4)|0;
 $80 = $79;
 $81 = HEAP32[$80>>2]|0;
 $82 = HEAP32[$5>>2]|0;
 $83 = HEAP32[$3>>2]|0;
 $84 = ((($83)) + 8|0);
 $85 = $84;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = $iLastLeaf;
 $92 = $91;
 $93 = HEAP32[$92>>2]|0;
 $94 = (($91) + 4)|0;
 $95 = $94;
 $96 = HEAP32[$95>>2]|0;
 $97 = $iLast;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (($97) + 4)|0;
 $101 = $100;
 $102 = HEAP32[$101>>2]|0;
 $103 = HEAP32[$3>>2]|0;
 $104 = ((($103)) + 56|0);
 $105 = $104;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = (($105) + 4)|0;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$zRoot>>2]|0;
 $112 = HEAP32[$nRoot>>2]|0;
 $113 = (_fts3WriteSegdir($75,$78,$81,$82,$87,$90,$93,$96,$99,$102,$107,$110,$111,$112)|0);
 HEAP32[$rc>>2] = $113;
 $137 = HEAP32[$2>>2]|0;
 $138 = ((($137)) + 52|0);
 $139 = HEAP32[$138>>2]|0;
 $140 = (($139) + 1)|0;
 HEAP32[$138>>2] = $140;
 $141 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($141|0);
}
function _fts3PromoteSegments($p,$0,$1,$2,$3) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bOk = 0, $dummy = 0, $iIdx = 0, $iLast = 0, $nLimit = 0;
 var $nSize = 0, $pRange = 0, $pUpdate1 = 0, $pUpdate2 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $4 = sp + 72|0;
 $5 = sp + 40|0;
 $6 = sp + 32|0;
 $rc = sp + 68|0;
 $pRange = sp + 64|0;
 $bOk = sp + 60|0;
 $iLast = sp + 24|0;
 $nLimit = sp + 16|0;
 $nSize = sp + 8|0;
 $dummy = sp;
 $iIdx = sp + 56|0;
 $pUpdate1 = sp + 52|0;
 $pUpdate2 = sp + 48|0;
 HEAP32[$4>>2] = $p;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 HEAP32[$12>>2] = $2;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$4>>2]|0;
 $16 = (_fts3SqlStmt($15,37,$pRange,0)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  $158 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 HEAP32[$bOk>>2] = 0;
 $19 = $5;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (___divdi3(($21|0),($24|0),1024,0)|0);
 $26 = tempRet0;
 $27 = (_i64Add(($25|0),($26|0),1,0)|0);
 $28 = tempRet0;
 $29 = (___muldi3(($27|0),($28|0),1024,0)|0);
 $30 = tempRet0;
 $31 = (_i64Subtract(($29|0),($30|0),1,0)|0);
 $32 = tempRet0;
 $33 = $iLast;
 $34 = $33;
 HEAP32[$34>>2] = $31;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = $32;
 $37 = $6;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (___muldi3(($39|0),($42|0),3,0)|0);
 $44 = tempRet0;
 $45 = (___divdi3(($43|0),($44|0),2,0)|0);
 $46 = tempRet0;
 $47 = $nLimit;
 $48 = $47;
 HEAP32[$48>>2] = $45;
 $49 = (($47) + 4)|0;
 $50 = $49;
 HEAP32[$50>>2] = $46;
 $51 = HEAP32[$pRange>>2]|0;
 $52 = $5;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = (_i64Add(($54|0),($57|0),1,0)|0);
 $59 = tempRet0;
 (_sqlite3_bind_int64($51,1,$58,$59)|0);
 $60 = HEAP32[$pRange>>2]|0;
 $61 = $iLast;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 (_sqlite3_bind_int64($60,2,$63,$66)|0);
 while(1) {
  $67 = HEAP32[$pRange>>2]|0;
  $68 = (_sqlite3_step($67)|0);
  $69 = (100)==($68|0);
  if (!($69)) {
   break;
  }
  $70 = $nSize;
  $71 = $70;
  HEAP32[$71>>2] = 0;
  $72 = (($70) + 4)|0;
  $73 = $72;
  HEAP32[$73>>2] = 0;
  $74 = HEAP32[$pRange>>2]|0;
  _fts3ReadEndBlockField($74,2,$dummy,$nSize);
  $75 = $nSize;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = ($80|0)<(0);
  $82 = ($77>>>0)<=(0);
  $83 = ($80|0)==(0);
  $84 = $83 & $82;
  $85 = $81 | $84;
  if ($85) {
   label = 6;
   break;
  }
  $86 = $nSize;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (($86) + 4)|0;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = $nLimit;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($91|0)>($97|0);
  $99 = ($88>>>0)>($94>>>0);
  $100 = ($91|0)==($97|0);
  $101 = $100 & $99;
  $102 = $98 | $101;
  if ($102) {
   label = 6;
   break;
  }
  HEAP32[$bOk>>2] = 1;
 }
 if ((label|0) == 6) {
  HEAP32[$bOk>>2] = 0;
 }
 $103 = HEAP32[$pRange>>2]|0;
 $104 = (_sqlite3_reset($103)|0);
 HEAP32[$rc>>2] = $104;
 $105 = HEAP32[$bOk>>2]|0;
 $106 = ($105|0)!=(0);
 if (!($106)) {
  $158 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 HEAP32[$iIdx>>2] = 0;
 HEAP32[$pUpdate1>>2] = 0;
 HEAP32[$pUpdate2>>2] = 0;
 $107 = HEAP32[$rc>>2]|0;
 $108 = ($107|0)==(0);
 if ($108) {
  $109 = HEAP32[$4>>2]|0;
  $110 = (_fts3SqlStmt($109,38,$pUpdate1,0)|0);
  HEAP32[$rc>>2] = $110;
 }
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 if ($112) {
  $113 = HEAP32[$4>>2]|0;
  $114 = (_fts3SqlStmt($113,39,$pUpdate2,0)|0);
  HEAP32[$rc>>2] = $114;
 }
 $115 = HEAP32[$rc>>2]|0;
 $116 = ($115|0)==(0);
 L21: do {
  if ($116) {
   $117 = HEAP32[$pRange>>2]|0;
   $118 = $5;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   (_sqlite3_bind_int64($117,1,$120,$123)|0);
   while(1) {
    $124 = HEAP32[$pRange>>2]|0;
    $125 = (_sqlite3_step($124)|0);
    $126 = (100)==($125|0);
    if (!($126)) {
     break L21;
    }
    $127 = HEAP32[$pUpdate1>>2]|0;
    $128 = HEAP32[$iIdx>>2]|0;
    $129 = (($128) + 1)|0;
    HEAP32[$iIdx>>2] = $129;
    (_sqlite3_bind_int($127,1,$128)|0);
    $130 = HEAP32[$pUpdate1>>2]|0;
    $131 = HEAP32[$pRange>>2]|0;
    $132 = (_sqlite3_column_int($131,0)|0);
    (_sqlite3_bind_int($130,2,$132)|0);
    $133 = HEAP32[$pUpdate1>>2]|0;
    $134 = HEAP32[$pRange>>2]|0;
    $135 = (_sqlite3_column_int($134,1)|0);
    (_sqlite3_bind_int($133,3,$135)|0);
    $136 = HEAP32[$pUpdate1>>2]|0;
    (_sqlite3_step($136)|0);
    $137 = HEAP32[$pUpdate1>>2]|0;
    $138 = (_sqlite3_reset($137)|0);
    HEAP32[$rc>>2] = $138;
    $139 = HEAP32[$rc>>2]|0;
    $140 = ($139|0)!=(0);
    if ($140) {
     break;
    }
   }
   $141 = HEAP32[$pRange>>2]|0;
   (_sqlite3_reset($141)|0);
  }
 } while(0);
 $142 = HEAP32[$rc>>2]|0;
 $143 = ($142|0)==(0);
 if ($143) {
  $144 = HEAP32[$pRange>>2]|0;
  $145 = (_sqlite3_reset($144)|0);
  HEAP32[$rc>>2] = $145;
 }
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(0);
 if (!($147)) {
  $158 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $148 = HEAP32[$pUpdate2>>2]|0;
 $149 = $5;
 $150 = $149;
 $151 = HEAP32[$150>>2]|0;
 $152 = (($149) + 4)|0;
 $153 = $152;
 $154 = HEAP32[$153>>2]|0;
 (_sqlite3_bind_int64($148,1,$151,$154)|0);
 $155 = HEAP32[$pUpdate2>>2]|0;
 (_sqlite3_step($155)|0);
 $156 = HEAP32[$pUpdate2>>2]|0;
 $157 = (_sqlite3_reset($156)|0);
 HEAP32[$rc>>2] = $157;
 $158 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($158|0);
}
function _fts3SegWriterFree($pWriter) {
 $pWriter = $pWriter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWriter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3_free($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 36|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 _fts3NodeFree($10);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3_free($11);
 STACKTOP = sp;return;
}
function _fts3PrefixCompress($zPrev,$nPrev,$zNext,$nNext) {
 $zPrev = $zPrev|0;
 $nPrev = $nPrev|0;
 $zNext = $zNext|0;
 $nNext = $nNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $zPrev;
 HEAP32[$1>>2] = $nPrev;
 HEAP32[$2>>2] = $zNext;
 HEAP32[$3>>2] = $nNext;
 HEAP32[$n>>2] = 0;
 while(1) {
  $4 = HEAP32[$n>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$n>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = HEAP32[$n>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + ($12)|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = ($11|0)==($16|0);
  if (!($17)) {
   label = 5;
   break;
  }
  $18 = HEAP32[$n>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$n>>2] = $19;
 }
 if ((label|0) == 5) {
  $20 = HEAP32[$n>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _fts3WriteSegment($p,$0,$1,$z,$n) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $z = $z|0;
 $n = $n|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $z;
 HEAP32[$5>>2] = $n;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_fts3SqlStmt($10,9,$pStmt,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $14 = HEAP32[$pStmt>>2]|0;
 $15 = $3;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3_bind_int64($14,1,$17,$20)|0);
 $21 = HEAP32[$pStmt>>2]|0;
 $22 = HEAP32[$4>>2]|0;
 $23 = HEAP32[$5>>2]|0;
 (_sqlite3_bind_blob($21,2,$22,$23,0)|0);
 $24 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_step($24)|0);
 $25 = HEAP32[$pStmt>>2]|0;
 $26 = (_sqlite3_reset($25)|0);
 HEAP32[$rc>>2] = $26;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _fts3NodeAddTerm($p,$ppTree,$isCopyTerm,$zTerm,$nTerm) {
 $p = $p|0;
 $ppTree = $ppTree|0;
 $isCopyTerm = $isCopyTerm|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nData = 0, $nPrefix = 0, $nReq = 0, $nSuffix = 0, $pNew = 0, $pParent = 0, $pTree = 0, $rc = 0, $zNew = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $pTree = sp + 32|0;
 $rc = sp + 28|0;
 $pNew = sp + 24|0;
 $nData = sp + 20|0;
 $nReq = sp + 16|0;
 $nPrefix = sp + 12|0;
 $nSuffix = sp + 8|0;
 $zNew = sp + 4|0;
 $pParent = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $ppTree;
 HEAP32[$3>>2] = $isCopyTerm;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTree>>2] = $7;
 $8 = HEAP32[$pTree>>2]|0;
 $9 = ($8|0)!=(0|0);
 do {
  if ($9) {
   $10 = HEAP32[$pTree>>2]|0;
   $11 = ((($10)) + 32|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$nData>>2] = $12;
   $13 = HEAP32[$nData>>2]|0;
   HEAP32[$nReq>>2] = $13;
   $14 = HEAP32[$pTree>>2]|0;
   $15 = ((($14)) + 16|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pTree>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = HEAP32[$5>>2]|0;
   $22 = (_fts3PrefixCompress($16,$19,$20,$21)|0);
   HEAP32[$nPrefix>>2] = $22;
   $23 = HEAP32[$5>>2]|0;
   $24 = HEAP32[$nPrefix>>2]|0;
   $25 = (($23) - ($24))|0;
   HEAP32[$nSuffix>>2] = $25;
   $26 = HEAP32[$nPrefix>>2]|0;
   $27 = ($26|0)<(0);
   $28 = $27 << 31 >> 31;
   $29 = (_sqlite3Fts3VarintLen($26,$28)|0);
   $30 = HEAP32[$nSuffix>>2]|0;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = (_sqlite3Fts3VarintLen($30,$32)|0);
   $34 = (($29) + ($33))|0;
   $35 = HEAP32[$nSuffix>>2]|0;
   $36 = (($34) + ($35))|0;
   $37 = HEAP32[$nReq>>2]|0;
   $38 = (($37) + ($36))|0;
   HEAP32[$nReq>>2] = $38;
   $39 = HEAP32[$nReq>>2]|0;
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 224|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($39|0)<=($42|0);
   if (!($43)) {
    $44 = HEAP32[$pTree>>2]|0;
    $45 = ((($44)) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = ($46|0)!=(0|0);
    if ($47) {
     break;
    }
   }
   $48 = HEAP32[$nReq>>2]|0;
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 224|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($48|0)>($51|0);
   if ($52) {
    $53 = HEAP32[$nReq>>2]|0;
    $54 = (_sqlite3_malloc($53)|0);
    $55 = HEAP32[$pTree>>2]|0;
    $56 = ((($55)) + 36|0);
    HEAP32[$56>>2] = $54;
    $57 = HEAP32[$pTree>>2]|0;
    $58 = ((($57)) + 36|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ($59|0)!=(0|0);
    if (!($60)) {
     HEAP32[$0>>2] = 7;
     $207 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($207|0);
    }
   }
   $61 = HEAP32[$pTree>>2]|0;
   $62 = ((($61)) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($63|0)!=(0|0);
   if ($64) {
    $65 = HEAP32[$nData>>2]|0;
    $66 = HEAP32[$pTree>>2]|0;
    $67 = ((($66)) + 36|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + ($65)|0);
    $70 = HEAP32[$nPrefix>>2]|0;
    $71 = ($70|0)<(0);
    $72 = $71 << 31 >> 31;
    $73 = (_sqlite3Fts3PutVarint($69,$70,$72)|0);
    $74 = HEAP32[$nData>>2]|0;
    $75 = (($74) + ($73))|0;
    HEAP32[$nData>>2] = $75;
   }
   $76 = HEAP32[$nData>>2]|0;
   $77 = HEAP32[$pTree>>2]|0;
   $78 = ((($77)) + 36|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76)|0);
   $81 = HEAP32[$nSuffix>>2]|0;
   $82 = ($81|0)<(0);
   $83 = $82 << 31 >> 31;
   $84 = (_sqlite3Fts3PutVarint($80,$81,$83)|0);
   $85 = HEAP32[$nData>>2]|0;
   $86 = (($85) + ($84))|0;
   HEAP32[$nData>>2] = $86;
   $87 = HEAP32[$nData>>2]|0;
   $88 = HEAP32[$pTree>>2]|0;
   $89 = ((($88)) + 36|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (($90) + ($87)|0);
   $92 = HEAP32[$nPrefix>>2]|0;
   $93 = HEAP32[$4>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP32[$nSuffix>>2]|0;
   _memcpy(($91|0),($94|0),($95|0))|0;
   $96 = HEAP32[$nData>>2]|0;
   $97 = HEAP32[$nSuffix>>2]|0;
   $98 = (($96) + ($97))|0;
   $99 = HEAP32[$pTree>>2]|0;
   $100 = ((($99)) + 32|0);
   HEAP32[$100>>2] = $98;
   $101 = HEAP32[$pTree>>2]|0;
   $102 = ((($101)) + 12|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (($103) + 1)|0;
   HEAP32[$102>>2] = $104;
   $105 = HEAP32[$3>>2]|0;
   $106 = ($105|0)!=(0);
   if ($106) {
    $107 = HEAP32[$pTree>>2]|0;
    $108 = ((($107)) + 24|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = HEAP32[$5>>2]|0;
    $111 = ($109|0)<($110|0);
    do {
     if ($111) {
      $112 = HEAP32[$pTree>>2]|0;
      $113 = ((($112)) + 28|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = HEAP32[$5>>2]|0;
      $116 = $115<<1;
      $117 = (_sqlite3_realloc($114,$116)|0);
      HEAP32[$zNew>>2] = $117;
      $118 = HEAP32[$zNew>>2]|0;
      $119 = ($118|0)!=(0|0);
      if ($119) {
       $120 = HEAP32[$5>>2]|0;
       $121 = $120<<1;
       $122 = HEAP32[$pTree>>2]|0;
       $123 = ((($122)) + 24|0);
       HEAP32[$123>>2] = $121;
       $124 = HEAP32[$zNew>>2]|0;
       $125 = HEAP32[$pTree>>2]|0;
       $126 = ((($125)) + 28|0);
       HEAP32[$126>>2] = $124;
       break;
      }
      HEAP32[$0>>2] = 7;
      $207 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($207|0);
     }
    } while(0);
    $127 = HEAP32[$pTree>>2]|0;
    $128 = ((($127)) + 28|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = HEAP32[$pTree>>2]|0;
    $131 = ((($130)) + 16|0);
    HEAP32[$131>>2] = $129;
    $132 = HEAP32[$pTree>>2]|0;
    $133 = ((($132)) + 16|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = HEAP32[$4>>2]|0;
    $136 = HEAP32[$5>>2]|0;
    _memcpy(($134|0),($135|0),($136|0))|0;
    $137 = HEAP32[$5>>2]|0;
    $138 = HEAP32[$pTree>>2]|0;
    $139 = ((($138)) + 20|0);
    HEAP32[$139>>2] = $137;
   } else {
    $140 = HEAP32[$4>>2]|0;
    $141 = HEAP32[$pTree>>2]|0;
    $142 = ((($141)) + 16|0);
    HEAP32[$142>>2] = $140;
    $143 = HEAP32[$5>>2]|0;
    $144 = HEAP32[$pTree>>2]|0;
    $145 = ((($144)) + 20|0);
    HEAP32[$145>>2] = $143;
   }
   HEAP32[$0>>2] = 0;
   $207 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($207|0);
  }
 } while(0);
 $146 = HEAP32[$1>>2]|0;
 $147 = ((($146)) + 224|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = (40 + ($148))|0;
 $150 = (_sqlite3_malloc($149)|0);
 HEAP32[$pNew>>2] = $150;
 $151 = HEAP32[$pNew>>2]|0;
 $152 = ($151|0)!=(0|0);
 if (!($152)) {
  HEAP32[$0>>2] = 7;
  $207 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($207|0);
 }
 $153 = HEAP32[$pNew>>2]|0;
 dest=$153; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $154 = HEAP32[$pNew>>2]|0;
 $155 = ((($154)) + 32|0);
 HEAP32[$155>>2] = 11;
 $156 = HEAP32[$pNew>>2]|0;
 $157 = ((($156)) + 40|0);
 $158 = HEAP32[$pNew>>2]|0;
 $159 = ((($158)) + 36|0);
 HEAP32[$159>>2] = $157;
 $160 = HEAP32[$pTree>>2]|0;
 $161 = ($160|0)!=(0|0);
 if ($161) {
  $162 = HEAP32[$pTree>>2]|0;
  $163 = HEAP32[$162>>2]|0;
  HEAP32[$pParent>>2] = $163;
  $164 = HEAP32[$1>>2]|0;
  $165 = HEAP32[$3>>2]|0;
  $166 = HEAP32[$4>>2]|0;
  $167 = HEAP32[$5>>2]|0;
  $168 = (_fts3NodeAddTerm($164,$pParent,$165,$166,$167)|0);
  HEAP32[$rc>>2] = $168;
  $169 = HEAP32[$pTree>>2]|0;
  $170 = HEAP32[$169>>2]|0;
  $171 = ($170|0)==(0|0);
  if ($171) {
   $172 = HEAP32[$pParent>>2]|0;
   $173 = HEAP32[$pTree>>2]|0;
   HEAP32[$173>>2] = $172;
  }
  $174 = HEAP32[$pNew>>2]|0;
  $175 = HEAP32[$pTree>>2]|0;
  $176 = ((($175)) + 4|0);
  HEAP32[$176>>2] = $174;
  $177 = HEAP32[$pTree>>2]|0;
  $178 = ((($177)) + 8|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = HEAP32[$pNew>>2]|0;
  $181 = ((($180)) + 8|0);
  HEAP32[$181>>2] = $179;
  $182 = HEAP32[$pParent>>2]|0;
  $183 = HEAP32[$pNew>>2]|0;
  HEAP32[$183>>2] = $182;
  $184 = HEAP32[$pTree>>2]|0;
  $185 = ((($184)) + 28|0);
  $186 = HEAP32[$185>>2]|0;
  $187 = HEAP32[$pNew>>2]|0;
  $188 = ((($187)) + 28|0);
  HEAP32[$188>>2] = $186;
  $189 = HEAP32[$pTree>>2]|0;
  $190 = ((($189)) + 24|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = HEAP32[$pNew>>2]|0;
  $193 = ((($192)) + 24|0);
  HEAP32[$193>>2] = $191;
  $194 = HEAP32[$pTree>>2]|0;
  $195 = ((($194)) + 28|0);
  HEAP32[$195>>2] = 0;
 } else {
  $196 = HEAP32[$pNew>>2]|0;
  $197 = HEAP32[$pNew>>2]|0;
  $198 = ((($197)) + 8|0);
  HEAP32[$198>>2] = $196;
  $199 = HEAP32[$1>>2]|0;
  $200 = HEAP32[$3>>2]|0;
  $201 = HEAP32[$4>>2]|0;
  $202 = HEAP32[$5>>2]|0;
  $203 = (_fts3NodeAddTerm($199,$pNew,$200,$201,$202)|0);
  HEAP32[$rc>>2] = $203;
 }
 $204 = HEAP32[$pNew>>2]|0;
 $205 = HEAP32[$2>>2]|0;
 HEAP32[$205>>2] = $204;
 $206 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $206;
 $207 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($207|0);
}
function _fts3DeleteSegment($p,$pSeg) {
 $p = $p|0;
 $pSeg = $pSeg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDelete = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pDelete = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pSeg;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)!=(0);
 $11 = ($9|0)!=(0);
 $12 = $10 | $11;
 if (!($12)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (_fts3SqlStmt($13,17,$pDelete,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[$pDelete>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = $19;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 (_sqlite3_bind_int64($17,1,$22,$25)|0);
 $26 = HEAP32[$pDelete>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 24|0);
 $29 = $28;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (($29) + 4)|0;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 (_sqlite3_bind_int64($26,2,$31,$34)|0);
 $35 = HEAP32[$pDelete>>2]|0;
 (_sqlite3_step($35)|0);
 $36 = HEAP32[$pDelete>>2]|0;
 $37 = (_sqlite3_reset($36)|0);
 HEAP32[$rc>>2] = $37;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3NodeWrite($p,$pTree,$iHeight,$0,$1,$2,$3,$piLast,$paRoot,$pnRoot) {
 $p = $p|0;
 $pTree = $pTree|0;
 $iHeight = $iHeight|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $piLast = $piLast|0;
 $paRoot = $paRoot|0;
 $pnRoot = $pnRoot|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iNextFree = 0, $iNextLeaf = 0, $nStart = 0, $nStart1 = 0, $nWrite = 0, $pIter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 24|0;
 $8 = sp + 16|0;
 $9 = sp + 60|0;
 $10 = sp + 56|0;
 $11 = sp + 52|0;
 $rc = sp + 48|0;
 $nStart = sp + 44|0;
 $pIter = sp + 40|0;
 $iNextFree = sp + 8|0;
 $iNextLeaf = sp;
 $nStart1 = sp + 36|0;
 $nWrite = sp + 32|0;
 HEAP32[$4>>2] = $p;
 HEAP32[$5>>2] = $pTree;
 HEAP32[$6>>2] = $iHeight;
 $12 = $7;
 $13 = $12;
 HEAP32[$13>>2] = $0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $1;
 $16 = $8;
 $17 = $16;
 HEAP32[$17>>2] = $2;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $3;
 HEAP32[$9>>2] = $piLast;
 HEAP32[$10>>2] = $paRoot;
 HEAP32[$11>>2] = $pnRoot;
 HEAP32[$rc>>2] = 0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  $23 = HEAP32[$5>>2]|0;
  $24 = HEAP32[$6>>2]|0;
  $25 = $7;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (_fts3TreeFinishNode($23,$24,$27,$30)|0);
  HEAP32[$nStart>>2] = $31;
  $32 = $8;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (_i64Subtract(($34|0),($37|0),1,0)|0);
  $39 = tempRet0;
  $40 = HEAP32[$9>>2]|0;
  $41 = $40;
  $42 = $41;
  HEAP32[$42>>2] = $38;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $39;
  $45 = HEAP32[$5>>2]|0;
  $46 = ((($45)) + 32|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$nStart>>2]|0;
  $49 = (($47) - ($48))|0;
  $50 = HEAP32[$11>>2]|0;
  HEAP32[$50>>2] = $49;
  $51 = HEAP32[$nStart>>2]|0;
  $52 = HEAP32[$5>>2]|0;
  $53 = ((($52)) + 36|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51)|0);
  $56 = HEAP32[$10>>2]|0;
  HEAP32[$56>>2] = $55;
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $57 = $8;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = $iNextFree;
 $64 = $63;
 HEAP32[$64>>2] = $59;
 $65 = (($63) + 4)|0;
 $66 = $65;
 HEAP32[$66>>2] = $62;
 $67 = $7;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = $iNextLeaf;
 $74 = $73;
 HEAP32[$74>>2] = $69;
 $75 = (($73) + 4)|0;
 $76 = $75;
 HEAP32[$76>>2] = $72;
 $77 = HEAP32[$5>>2]|0;
 $78 = ((($77)) + 8|0);
 $79 = HEAP32[$78>>2]|0;
 HEAP32[$pIter>>2] = $79;
 while(1) {
  $80 = HEAP32[$pIter>>2]|0;
  $81 = ($80|0)!=(0|0);
  $82 = HEAP32[$rc>>2]|0;
  $83 = ($82|0)==(0);
  $84 = $81 ? $83 : 0;
  if (!($84)) {
   break;
  }
  $85 = HEAP32[$pIter>>2]|0;
  $86 = HEAP32[$6>>2]|0;
  $87 = $iNextLeaf;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  $93 = (_fts3TreeFinishNode($85,$86,$89,$92)|0);
  HEAP32[$nStart1>>2] = $93;
  $94 = HEAP32[$pIter>>2]|0;
  $95 = ((($94)) + 32|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = HEAP32[$nStart1>>2]|0;
  $98 = (($96) - ($97))|0;
  HEAP32[$nWrite>>2] = $98;
  $99 = HEAP32[$4>>2]|0;
  $100 = $iNextFree;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($100) + 4)|0;
  $104 = $103;
  $105 = HEAP32[$104>>2]|0;
  $106 = HEAP32[$nStart1>>2]|0;
  $107 = HEAP32[$pIter>>2]|0;
  $108 = ((($107)) + 36|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = (($109) + ($106)|0);
  $111 = HEAP32[$nWrite>>2]|0;
  $112 = (_fts3WriteSegment($99,$102,$105,$110,$111)|0);
  HEAP32[$rc>>2] = $112;
  $113 = $iNextFree;
  $114 = $113;
  $115 = HEAP32[$114>>2]|0;
  $116 = (($113) + 4)|0;
  $117 = $116;
  $118 = HEAP32[$117>>2]|0;
  $119 = (_i64Add(($115|0),($118|0),1,0)|0);
  $120 = tempRet0;
  $121 = $iNextFree;
  $122 = $121;
  HEAP32[$122>>2] = $119;
  $123 = (($121) + 4)|0;
  $124 = $123;
  HEAP32[$124>>2] = $120;
  $125 = HEAP32[$pIter>>2]|0;
  $126 = ((($125)) + 12|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = (($127) + 1)|0;
  $129 = ($128|0)<(0);
  $130 = $129 << 31 >> 31;
  $131 = $iNextLeaf;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = (($131) + 4)|0;
  $135 = $134;
  $136 = HEAP32[$135>>2]|0;
  $137 = (_i64Add(($133|0),($136|0),($128|0),($130|0))|0);
  $138 = tempRet0;
  $139 = $iNextLeaf;
  $140 = $139;
  HEAP32[$140>>2] = $137;
  $141 = (($139) + 4)|0;
  $142 = $141;
  HEAP32[$142>>2] = $138;
  $143 = HEAP32[$pIter>>2]|0;
  $144 = ((($143)) + 4|0);
  $145 = HEAP32[$144>>2]|0;
  HEAP32[$pIter>>2] = $145;
 }
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(0);
 if (!($147)) {
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $148 = HEAP32[$4>>2]|0;
 $149 = HEAP32[$5>>2]|0;
 $150 = HEAP32[$149>>2]|0;
 $151 = HEAP32[$6>>2]|0;
 $152 = (($151) + 1)|0;
 $153 = $8;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = (($153) + 4)|0;
 $157 = $156;
 $158 = HEAP32[$157>>2]|0;
 $159 = $iNextFree;
 $160 = $159;
 $161 = HEAP32[$160>>2]|0;
 $162 = (($159) + 4)|0;
 $163 = $162;
 $164 = HEAP32[$163>>2]|0;
 $165 = HEAP32[$9>>2]|0;
 $166 = HEAP32[$10>>2]|0;
 $167 = HEAP32[$11>>2]|0;
 $168 = (_fts3NodeWrite($148,$150,$152,$155,$158,$161,$164,$165,$166,$167)|0);
 HEAP32[$rc>>2] = $168;
 $169 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _fts3WriteSegdir($p,$0,$1,$iIdx,$2,$3,$4,$5,$6,$7,$8,$9,$zRoot,$nRoot) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $2 = $2|0;
 $3 = $3|0;
 $4 = $4|0;
 $5 = $5|0;
 $6 = $6|0;
 $7 = $7|0;
 $8 = $8|0;
 $9 = $9|0;
 $zRoot = $zRoot|0;
 $nRoot = $nRoot|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $12 = 0, $13 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pStmt = 0, $rc = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp + 40|0;
 $10 = sp + 84|0;
 $11 = sp + 80|0;
 $12 = sp + 32|0;
 $13 = sp + 76|0;
 $14 = sp + 24|0;
 $15 = sp + 16|0;
 $16 = sp + 8|0;
 $17 = sp;
 $18 = sp + 72|0;
 $19 = sp + 68|0;
 $pStmt = sp + 64|0;
 $rc = sp + 60|0;
 $zEnd = sp + 56|0;
 HEAP32[$11>>2] = $p;
 $20 = $12;
 $21 = $20;
 HEAP32[$21>>2] = $0;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $1;
 HEAP32[$13>>2] = $iIdx;
 $24 = $14;
 $25 = $24;
 HEAP32[$25>>2] = $2;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $3;
 $28 = $15;
 $29 = $28;
 HEAP32[$29>>2] = $4;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $5;
 $32 = $16;
 $33 = $32;
 HEAP32[$33>>2] = $6;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $7;
 $36 = $17;
 $37 = $36;
 HEAP32[$37>>2] = $8;
 $38 = (($36) + 4)|0;
 $39 = $38;
 HEAP32[$39>>2] = $9;
 HEAP32[$18>>2] = $zRoot;
 HEAP32[$19>>2] = $nRoot;
 $40 = HEAP32[$11>>2]|0;
 $41 = (_fts3SqlStmt($40,11,$pStmt,0)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$pStmt>>2]|0;
  $45 = $12;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($45) + 4)|0;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  (_sqlite3_bind_int64($44,1,$47,$50)|0);
  $51 = HEAP32[$pStmt>>2]|0;
  $52 = HEAP32[$13>>2]|0;
  (_sqlite3_bind_int($51,2,$52)|0);
  $53 = HEAP32[$pStmt>>2]|0;
  $54 = $14;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  (_sqlite3_bind_int64($53,3,$56,$59)|0);
  $60 = HEAP32[$pStmt>>2]|0;
  $61 = $15;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($61) + 4)|0;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  (_sqlite3_bind_int64($60,4,$63,$66)|0);
  $67 = $17;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($69|0)==(0);
  $74 = ($72|0)==(0);
  $75 = $73 & $74;
  do {
   if ($75) {
    $76 = HEAP32[$pStmt>>2]|0;
    $77 = $16;
    $78 = $77;
    $79 = HEAP32[$78>>2]|0;
    $80 = (($77) + 4)|0;
    $81 = $80;
    $82 = HEAP32[$81>>2]|0;
    (_sqlite3_bind_int64($76,5,$79,$82)|0);
   } else {
    $83 = $16;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $86 = (($83) + 4)|0;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = $17;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    $92 = (($89) + 4)|0;
    $93 = $92;
    $94 = HEAP32[$93>>2]|0;
    $95 = $vararg_buffer;
    $96 = $95;
    HEAP32[$96>>2] = $85;
    $97 = (($95) + 4)|0;
    $98 = $97;
    HEAP32[$98>>2] = $88;
    $vararg_ptr1 = ((($vararg_buffer)) + 8|0);
    $99 = $vararg_ptr1;
    $100 = $99;
    HEAP32[$100>>2] = $91;
    $101 = (($99) + 4)|0;
    $102 = $101;
    HEAP32[$102>>2] = $94;
    $103 = (_sqlite3_mprintf(44510,$vararg_buffer)|0);
    HEAP32[$zEnd>>2] = $103;
    $104 = HEAP32[$zEnd>>2]|0;
    $105 = ($104|0)!=(0|0);
    if ($105) {
     $106 = HEAP32[$pStmt>>2]|0;
     $107 = HEAP32[$zEnd>>2]|0;
     (_sqlite3_bind_text($106,5,$107,-1,142)|0);
     break;
    }
    HEAP32[$10>>2] = 7;
    $115 = HEAP32[$10>>2]|0;
    STACKTOP = sp;return ($115|0);
   }
  } while(0);
  $108 = HEAP32[$pStmt>>2]|0;
  $109 = HEAP32[$18>>2]|0;
  $110 = HEAP32[$19>>2]|0;
  (_sqlite3_bind_blob($108,6,$109,$110,0)|0);
  $111 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($111)|0);
  $112 = HEAP32[$pStmt>>2]|0;
  $113 = (_sqlite3_reset($112)|0);
  HEAP32[$rc>>2] = $113;
 }
 $114 = HEAP32[$rc>>2]|0;
 HEAP32[$10>>2] = $114;
 $115 = HEAP32[$10>>2]|0;
 STACKTOP = sp;return ($115|0);
}
function _fts3TreeFinishNode($pTree,$iHeight,$0,$1) {
 $pTree = $pTree|0;
 $iHeight = $iHeight|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $nStart = sp + 8|0;
 HEAP32[$2>>2] = $pTree;
 HEAP32[$3>>2] = $iHeight;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3Fts3VarintLen($11,$14)|0);
 $16 = (10 - ($15))|0;
 HEAP32[$nStart>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = $17&255;
 $19 = HEAP32[$nStart>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + ($19)|0);
 HEAP8[$23>>0] = $18;
 $24 = HEAP32[$nStart>>2]|0;
 $25 = (($24) + 1)|0;
 $26 = HEAP32[$2>>2]|0;
 $27 = ((($26)) + 36|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25)|0);
 $30 = $4;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 (_sqlite3Fts3PutVarint($29,$32,$35)|0);
 $36 = HEAP32[$nStart>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _fts3ReadEndBlockField($pStmt,$iCol,$piEndBlock,$pnByte) {
 $pStmt = $pStmt|0;
 $iCol = $iCol|0;
 $piEndBlock = $piEndBlock|0;
 $pnByte = $pnByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $iMul = 0, $iVal = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $zText = sp + 16|0;
 $i = sp + 12|0;
 $iMul = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$2>>2] = $piEndBlock;
 HEAP32[$3>>2] = $pnByte;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3_column_text($4,$5)|0);
 HEAP32[$zText>>2] = $6;
 $7 = HEAP32[$zText>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 HEAP32[$iMul>>2] = 1;
 $9 = $iVal;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$zText>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)>=(48);
  if ($18) {
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$zText>>2]|0;
   $21 = (($20) + ($19)|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)<=(57);
   $120 = $24;
  } else {
   $120 = 0;
  }
  $25 = $iVal;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  if (!($120)) {
   break;
  }
  $31 = (___muldi3(($27|0),($30|0),10,0)|0);
  $32 = tempRet0;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$zText>>2]|0;
  $35 = (($34) + ($33)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = (($37) - 48)|0;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = (_i64Add(($31|0),($32|0),($38|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $iVal;
  $44 = $43;
  HEAP32[$44>>2] = $41;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$i>>2] = $48;
 }
 $49 = HEAP32[$2>>2]|0;
 $50 = $49;
 $51 = $50;
 HEAP32[$51>>2] = $27;
 $52 = (($50) + 4)|0;
 $53 = $52;
 HEAP32[$53>>2] = $30;
 while(1) {
  $54 = HEAP32[$i>>2]|0;
  $55 = HEAP32[$zText>>2]|0;
  $56 = (($55) + ($54)|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = $57&255;
  $59 = ($58|0)==(32);
  if (!($59)) {
   break;
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = $iVal;
 $63 = $62;
 HEAP32[$63>>2] = 0;
 $64 = (($62) + 4)|0;
 $65 = $64;
 HEAP32[$65>>2] = 0;
 $66 = HEAP32[$i>>2]|0;
 $67 = HEAP32[$zText>>2]|0;
 $68 = (($67) + ($66)|0);
 $69 = HEAP8[$68>>0]|0;
 $70 = $69&255;
 $71 = ($70|0)==(45);
 if ($71) {
  $72 = HEAP32[$i>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$i>>2] = $73;
  HEAP32[$iMul>>2] = -1;
 }
 while(1) {
  $74 = HEAP32[$i>>2]|0;
  $75 = HEAP32[$zText>>2]|0;
  $76 = (($75) + ($74)|0);
  $77 = HEAP8[$76>>0]|0;
  $78 = $77&255;
  $79 = ($78|0)>=(48);
  if ($79) {
   $80 = HEAP32[$i>>2]|0;
   $81 = HEAP32[$zText>>2]|0;
   $82 = (($81) + ($80)|0);
   $83 = HEAP8[$82>>0]|0;
   $84 = $83&255;
   $85 = ($84|0)<=(57);
   $121 = $85;
  } else {
   $121 = 0;
  }
  $86 = $iVal;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (($86) + 4)|0;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  if (!($121)) {
   break;
  }
  $92 = (___muldi3(($88|0),($91|0),10,0)|0);
  $93 = tempRet0;
  $94 = HEAP32[$i>>2]|0;
  $95 = HEAP32[$zText>>2]|0;
  $96 = (($95) + ($94)|0);
  $97 = HEAP8[$96>>0]|0;
  $98 = $97&255;
  $99 = (($98) - 48)|0;
  $100 = ($99|0)<(0);
  $101 = $100 << 31 >> 31;
  $102 = (_i64Add(($92|0),($93|0),($99|0),($101|0))|0);
  $103 = tempRet0;
  $104 = $iVal;
  $105 = $104;
  HEAP32[$105>>2] = $102;
  $106 = (($104) + 4)|0;
  $107 = $106;
  HEAP32[$107>>2] = $103;
  $108 = HEAP32[$i>>2]|0;
  $109 = (($108) + 1)|0;
  HEAP32[$i>>2] = $109;
 }
 $110 = HEAP32[$iMul>>2]|0;
 $111 = ($110|0)<(0);
 $112 = $111 << 31 >> 31;
 $113 = (___muldi3(($88|0),($91|0),($110|0),($112|0))|0);
 $114 = tempRet0;
 $115 = HEAP32[$3>>2]|0;
 $116 = $115;
 $117 = $116;
 HEAP32[$117>>2] = $113;
 $118 = (($116) + 4)|0;
 $119 = $118;
 HEAP32[$119>>2] = $114;
 STACKTOP = sp;return;
}
function _fts3NodeFree($pTree) {
 $pTree = $pTree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pTree;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 _fts3NodeFree($7);
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pRight>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 36|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$p>>2]|0;
  $17 = ((($16)) + 40|0);
  $18 = ($15|0)!=($17|0);
  if ($18) {
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 36|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3_free($21);
  }
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 28|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_free($24);
  $25 = HEAP32[$p>>2]|0;
  _sqlite3_free($25);
  $26 = HEAP32[$pRight>>2]|0;
  HEAP32[$p>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _fts3DeleteAll($p,$bContent) {
 $p = $p|0;
 $bContent = $bContent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $bContent;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3Fts3PendingTermsClear($2);
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  _fts3SqlExec($rc,$5,2,0);
 }
 $6 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$6,3,0);
 $7 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$7,4,0);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 230|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  _fts3SqlExec($rc,$12,5,0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 229|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = ($15<<24>>24)!=(0);
 if (!($16)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $17 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$17,6,0);
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _langidFromSelect($p,$pSelect) {
 $p = $p|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iLangid = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pSelect;
 HEAP32[$iLangid>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $12 = HEAP32[$iLangid>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 $11 = (_sqlite3_column_int($6,$10)|0);
 HEAP32[$iLangid>>2] = $11;
 $12 = HEAP32[$iLangid>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _fts3PendingTermsAdd($p,$iLangid,$zText,$iCol,$pnWord) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $zText = $zText|0;
 $iCol = $iCol|0;
 $pnWord = $pnWord|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $i = 0, $iEnd = 0, $iPos = 0, $iStart = 0, $nToken = 0, $nWord = 0, $or$cond = 0, $or$cond$not = 0, $or$cond3 = 0;
 var $pCsr = 0, $pIndex = 0, $pModule = 0, $pTokenizer = 0, $rc = 0, $xNext = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $5 = sp + 52|0;
 $rc = sp + 48|0;
 $iStart = sp + 44|0;
 $iEnd = sp + 40|0;
 $iPos = sp + 36|0;
 $nWord = sp + 32|0;
 $zToken = sp + 28|0;
 $nToken = sp + 24|0;
 $pTokenizer = sp + 20|0;
 $pModule = sp + 16|0;
 $pCsr = sp + 12|0;
 $xNext = sp + 8|0;
 $i = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $zText;
 HEAP32[$4>>2] = $iCol;
 HEAP32[$5>>2] = $pnWord;
 HEAP32[$iStart>>2] = 0;
 HEAP32[$iEnd>>2] = 0;
 HEAP32[$iPos>>2] = 0;
 HEAP32[$nWord>>2] = 0;
 HEAP32[$nToken>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 36|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTokenizer>>2] = $8;
 $9 = HEAP32[$pTokenizer>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pModule>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[$5>>2]|0;
  HEAP32[$13>>2] = 0;
  HEAP32[$0>>2] = 0;
  $90 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $14 = HEAP32[$pTokenizer>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = (_sqlite3Fts3OpenTokenizer($14,$15,$16,-1,$pCsr)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $90 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $21 = HEAP32[$pModule>>2]|0;
 $22 = ((($21)) + 20|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$xNext>>2] = $23;
 L9: while(1) {
  $24 = HEAP32[$rc>>2]|0;
  $25 = (0)==($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$xNext>>2]|0;
  $27 = HEAP32[$pCsr>>2]|0;
  $28 = (FUNCTION_TABLE_iiiiiii[$26 & 255]($27,$zToken,$nToken,$iStart,$iEnd,$iPos)|0);
  HEAP32[$rc>>2] = $28;
  $29 = (0)==($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$iPos>>2]|0;
  $31 = HEAP32[$nWord>>2]|0;
  $32 = ($30|0)>=($31|0);
  if ($32) {
   $33 = HEAP32[$iPos>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$nWord>>2] = $34;
  }
  $35 = HEAP32[$iPos>>2]|0;
  $36 = ($35|0)>=(0);
  $37 = HEAP32[$zToken>>2]|0;
  $38 = ($37|0)!=(0|0);
  $or$cond = $36 & $38;
  $or$cond$not = $or$cond ^ 1;
  $39 = HEAP32[$nToken>>2]|0;
  $40 = ($39|0)<=(0);
  $or$cond3 = $or$cond$not | $40;
  if ($or$cond3) {
   label = 11;
   break;
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$4>>2]|0;
  $43 = HEAP32[$iPos>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 252|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ((($46)) + 4|0);
  $48 = HEAP32[$zToken>>2]|0;
  $49 = HEAP32[$nToken>>2]|0;
  $50 = (_fts3PendingTermsAddOne($41,$42,$43,$47,$48,$49)|0);
  HEAP32[$rc>>2] = $50;
  HEAP32[$i>>2] = 1;
  while(1) {
   $51 = HEAP32[$rc>>2]|0;
   $52 = ($51|0)==(0);
   if (!($52)) {
    continue L9;
   }
   $53 = HEAP32[$i>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 248|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = ($53|0)<($56|0);
   if (!($57)) {
    continue L9;
   }
   $58 = HEAP32[$i>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 252|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + (($58*24)|0)|0);
   HEAP32[$pIndex>>2] = $62;
   $63 = HEAP32[$nToken>>2]|0;
   $64 = HEAP32[$pIndex>>2]|0;
   $65 = HEAP32[$64>>2]|0;
   $66 = ($63|0)<($65|0);
   if (!($66)) {
    $67 = HEAP32[$1>>2]|0;
    $68 = HEAP32[$4>>2]|0;
    $69 = HEAP32[$iPos>>2]|0;
    $70 = HEAP32[$pIndex>>2]|0;
    $71 = ((($70)) + 4|0);
    $72 = HEAP32[$zToken>>2]|0;
    $73 = HEAP32[$pIndex>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = (_fts3PendingTermsAddOne($67,$68,$69,$71,$72,$74)|0);
    HEAP32[$rc>>2] = $75;
   }
   $76 = HEAP32[$i>>2]|0;
   $77 = (($76) + 1)|0;
   HEAP32[$i>>2] = $77;
  }
 }
 if ((label|0) == 11) {
  HEAP32[$rc>>2] = 1;
 }
 $78 = HEAP32[$pModule>>2]|0;
 $79 = ((($78)) + 16|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = HEAP32[$pCsr>>2]|0;
 (FUNCTION_TABLE_ii[$80 & 255]($81)|0);
 $82 = HEAP32[$nWord>>2]|0;
 $83 = HEAP32[$5>>2]|0;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($84) + ($82))|0;
 HEAP32[$83>>2] = $85;
 $86 = HEAP32[$rc>>2]|0;
 $87 = ($86|0)==(101);
 $88 = HEAP32[$rc>>2]|0;
 $89 = $87 ? 0 : $88;
 HEAP32[$0>>2] = $89;
 $90 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($90|0);
}
function _fts3SqlExec($pRC,$p,$eStmt,$apVal) {
 $pRC = $pRC|0;
 $p = $p|0;
 $eStmt = $eStmt|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $eStmt;
 HEAP32[$3>>2] = $apVal;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = (_fts3SqlStmt($7,$8,$pStmt,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($13)|0);
  $14 = HEAP32[$pStmt>>2]|0;
  $15 = (_sqlite3_reset($14)|0);
  HEAP32[$rc>>2] = $15;
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 HEAP32[$17>>2] = $16;
 STACKTOP = sp;return;
}
function _fts3PendingTermsAddOne($p,$iCol,$iPos,$pHash,$zToken,$nToken) {
 $p = $p|0;
 $iCol = $iCol|0;
 $iPos = $iPos|0;
 $pHash = $pHash|0;
 $zToken = $zToken|0;
 $nToken = $nToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pList = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$2>>2] = $iPos;
 HEAP32[$3>>2] = $pHash;
 HEAP32[$4>>2] = $zToken;
 HEAP32[$5>>2] = $nToken;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$5>>2]|0;
 $9 = (_sqlite3Fts3HashFind($6,$7,$8)|0);
 HEAP32[$pList>>2] = $9;
 $10 = HEAP32[$pList>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$pList>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$5>>2]|0;
  $15 = (($13) + ($14))|0;
  $16 = (($15) + 20)|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 260|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - ($16))|0;
  HEAP32[$18>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 264|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = ($29|0)<(0);
 $31 = $30 << 31 >> 31;
 $32 = HEAP32[$2>>2]|0;
 $33 = ($32|0)<(0);
 $34 = $33 << 31 >> 31;
 $35 = (_fts3PendingListAppend($pList,$25,$28,$29,$31,$32,$34,$rc)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$pList>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = HEAP32[$4>>2]|0;
  $40 = HEAP32[$5>>2]|0;
  $41 = HEAP32[$pList>>2]|0;
  $42 = (_sqlite3Fts3HashInsert($38,$39,$40,$41)|0);
  $43 = ($37|0)==($42|0);
  if ($43) {
   $44 = HEAP32[$pList>>2]|0;
   _sqlite3_free($44);
   HEAP32[$rc>>2] = 7;
  }
 }
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(0);
 if (!($46)) {
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $47 = HEAP32[$pList>>2]|0;
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 $50 = (($48) + ($49))|0;
 $51 = (($50) + 20)|0;
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 260|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + ($51))|0;
 HEAP32[$53>>2] = $55;
 $56 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _fts3IntegrityCheck($p,$pbOk) {
 $p = $p|0;
 $pbOk = $pbOk|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum1 = 0, $cksum2 = 0, $i = 0, $i1 = 0, $iCol = 0, $iDocid = 0, $iDum1 = 0, $iDum2 = 0, $iLang = 0, $iLangid = 0;
 var $iPos = 0, $nText = 0, $nToken = 0, $pAllLangid = 0, $pModule = 0, $pStmt = 0, $pT = 0, $rc = 0, $rc2 = 0, $vararg_buffer = 0, $zSql = 0, $zText = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $rc = sp + 100|0;
 $cksum1 = sp + 16|0;
 $cksum2 = sp + 8|0;
 $pAllLangid = sp + 96|0;
 $rc2 = sp + 92|0;
 $iLangid = sp + 88|0;
 $i = sp + 84|0;
 $pModule = sp + 80|0;
 $pStmt = sp + 76|0;
 $zSql = sp + 72|0;
 $iDocid = sp;
 $iLang = sp + 68|0;
 $iCol = sp + 64|0;
 $zText = sp + 60|0;
 $nText = sp + 56|0;
 $pT = sp + 52|0;
 $zToken = sp + 48|0;
 $nToken = sp + 44|0;
 $iDum1 = sp + 40|0;
 $iDum2 = sp + 36|0;
 $iPos = sp + 32|0;
 $i1 = sp + 28|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pbOk;
 HEAP32[$rc>>2] = 0;
 $2 = $cksum1;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = $cksum2;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 HEAP32[$pAllLangid>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (_fts3SqlStmt($10,27,$pAllLangid,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$pAllLangid>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 272|0);
  $17 = HEAP32[$16>>2]|0;
  (_sqlite3_bind_int($14,1,$17)|0);
  $18 = HEAP32[$pAllLangid>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 248|0);
  $21 = HEAP32[$20>>2]|0;
  (_sqlite3_bind_int($18,2,$21)|0);
  L3: while(1) {
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$pAllLangid>>2]|0;
    $25 = (_sqlite3_step($24)|0);
    $26 = ($25|0)==(100);
    $228 = $26;
   } else {
    $228 = 0;
   }
   $27 = HEAP32[$pAllLangid>>2]|0;
   if (!($228)) {
    break;
   }
   $28 = (_sqlite3_column_int($27,0)|0);
   HEAP32[$iLangid>>2] = $28;
   HEAP32[$i>>2] = 0;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 248|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($29|0)<($32|0);
    if (!($33)) {
     continue L3;
    }
    $34 = $cksum1;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$0>>2]|0;
    $41 = HEAP32[$iLangid>>2]|0;
    $42 = HEAP32[$i>>2]|0;
    $43 = (_fts3ChecksumIndex($40,$41,$42,$rc)|0);
    $44 = tempRet0;
    $45 = $36 ^ $43;
    $46 = $39 ^ $44;
    $47 = $cksum1;
    $48 = $47;
    HEAP32[$48>>2] = $45;
    $49 = (($47) + 4)|0;
    $50 = $49;
    HEAP32[$50>>2] = $46;
    $51 = HEAP32[$i>>2]|0;
    $52 = (($51) + 1)|0;
    HEAP32[$i>>2] = $52;
   }
  }
  $53 = (_sqlite3_reset($27)|0);
  HEAP32[$rc2>>2] = $53;
  $54 = HEAP32[$rc>>2]|0;
  $55 = ($54|0)==(0);
  if ($55) {
   $56 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $56;
  }
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if (!($58)) {
  $210 = $cksum1;
  $211 = $210;
  $212 = HEAP32[$211>>2]|0;
  $213 = (($210) + 4)|0;
  $214 = $213;
  $215 = HEAP32[$214>>2]|0;
  $216 = $cksum2;
  $217 = $216;
  $218 = HEAP32[$217>>2]|0;
  $219 = (($216) + 4)|0;
  $220 = $219;
  $221 = HEAP32[$220>>2]|0;
  $222 = ($212|0)==($218|0);
  $223 = ($215|0)==($221|0);
  $224 = $222 & $223;
  $225 = $224&1;
  $226 = HEAP32[$1>>2]|0;
  HEAP32[$226>>2] = $225;
  $227 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 36|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$61>>2]|0;
 HEAP32[$pModule>>2] = $62;
 HEAP32[$pStmt>>2] = 0;
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 216|0);
 $65 = HEAP32[$64>>2]|0;
 HEAP32[$vararg_buffer>>2] = $65;
 $66 = (_sqlite3_mprintf(44520,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $66;
 $67 = HEAP32[$zSql>>2]|0;
 $68 = ($67|0)!=(0|0);
 if ($68) {
  $69 = HEAP32[$0>>2]|0;
  $70 = ((($69)) + 12|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = HEAP32[$zSql>>2]|0;
  $73 = (_sqlite3_prepare_v2($71,$72,-1,$pStmt,0)|0);
  HEAP32[$rc>>2] = $73;
  $74 = HEAP32[$zSql>>2]|0;
  _sqlite3_free($74);
 } else {
  HEAP32[$rc>>2] = 7;
 }
 L21: while(1) {
  $75 = HEAP32[$rc>>2]|0;
  $76 = ($75|0)==(0);
  if ($76) {
   $77 = HEAP32[$pStmt>>2]|0;
   $78 = (_sqlite3_step($77)|0);
   $79 = (100)==($78|0);
   $229 = $79;
  } else {
   $229 = 0;
  }
  $80 = HEAP32[$pStmt>>2]|0;
  if (!($229)) {
   break;
  }
  $81 = (_sqlite3_column_int64($80,0)|0);
  $82 = tempRet0;
  $83 = $iDocid;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
  $87 = HEAP32[$0>>2]|0;
  $88 = HEAP32[$pStmt>>2]|0;
  $89 = (_langidFromSelect($87,$88)|0);
  HEAP32[$iLang>>2] = $89;
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $90 = HEAP32[$rc>>2]|0;
   $91 = ($90|0)==(0);
   if (!($91)) {
    continue L21;
   }
   $92 = HEAP32[$iCol>>2]|0;
   $93 = HEAP32[$0>>2]|0;
   $94 = ((($93)) + 24|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = ($92|0)<($95|0);
   if (!($96)) {
    continue L21;
   }
   $97 = HEAP32[$iCol>>2]|0;
   $98 = HEAP32[$0>>2]|0;
   $99 = ((($98)) + 32|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (($100) + ($97)|0);
   $102 = HEAP8[$101>>0]|0;
   $103 = $102&255;
   $104 = ($103|0)==(0);
   if ($104) {
    $105 = HEAP32[$pStmt>>2]|0;
    $106 = HEAP32[$iCol>>2]|0;
    $107 = (($106) + 1)|0;
    $108 = (_sqlite3_column_text($105,$107)|0);
    HEAP32[$zText>>2] = $108;
    $109 = HEAP32[$pStmt>>2]|0;
    $110 = HEAP32[$iCol>>2]|0;
    $111 = (($110) + 1)|0;
    $112 = (_sqlite3_column_bytes($109,$111)|0);
    HEAP32[$nText>>2] = $112;
    HEAP32[$pT>>2] = 0;
    $113 = HEAP32[$0>>2]|0;
    $114 = ((($113)) + 36|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = HEAP32[$iLang>>2]|0;
    $117 = HEAP32[$zText>>2]|0;
    $118 = HEAP32[$nText>>2]|0;
    $119 = (_sqlite3Fts3OpenTokenizer($115,$116,$117,$118,$pT)|0);
    HEAP32[$rc>>2] = $119;
    L33: while(1) {
     $120 = HEAP32[$rc>>2]|0;
     $121 = ($120|0)==(0);
     if (!($121)) {
      break;
     }
     HEAP32[$nToken>>2] = 0;
     HEAP32[$iDum1>>2] = 0;
     HEAP32[$iDum2>>2] = 0;
     HEAP32[$iPos>>2] = 0;
     $122 = HEAP32[$pModule>>2]|0;
     $123 = ((($122)) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = HEAP32[$pT>>2]|0;
     $126 = (FUNCTION_TABLE_iiiiiii[$124 & 255]($125,$zToken,$nToken,$iDum1,$iDum2,$iPos)|0);
     HEAP32[$rc>>2] = $126;
     $127 = HEAP32[$rc>>2]|0;
     $128 = ($127|0)==(0);
     if (!($128)) {
      continue;
     }
     $129 = $cksum2;
     $130 = $129;
     $131 = HEAP32[$130>>2]|0;
     $132 = (($129) + 4)|0;
     $133 = $132;
     $134 = HEAP32[$133>>2]|0;
     $135 = HEAP32[$zToken>>2]|0;
     $136 = HEAP32[$nToken>>2]|0;
     $137 = HEAP32[$iLang>>2]|0;
     $138 = $iDocid;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (($138) + 4)|0;
     $142 = $141;
     $143 = HEAP32[$142>>2]|0;
     $144 = HEAP32[$iCol>>2]|0;
     $145 = HEAP32[$iPos>>2]|0;
     $146 = (_fts3ChecksumEntry($135,$136,$137,0,$140,$143,$144,$145)|0);
     $147 = tempRet0;
     $148 = $131 ^ $146;
     $149 = $134 ^ $147;
     $150 = $cksum2;
     $151 = $150;
     HEAP32[$151>>2] = $148;
     $152 = (($150) + 4)|0;
     $153 = $152;
     HEAP32[$153>>2] = $149;
     HEAP32[$i1>>2] = 1;
     while(1) {
      $154 = HEAP32[$i1>>2]|0;
      $155 = HEAP32[$0>>2]|0;
      $156 = ((($155)) + 248|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = ($154|0)<($157|0);
      if (!($158)) {
       continue L33;
      }
      $159 = HEAP32[$i1>>2]|0;
      $160 = HEAP32[$0>>2]|0;
      $161 = ((($160)) + 252|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = (($162) + (($159*24)|0)|0);
      $164 = HEAP32[$163>>2]|0;
      $165 = HEAP32[$nToken>>2]|0;
      $166 = ($164|0)<=($165|0);
      if ($166) {
       $167 = $cksum2;
       $168 = $167;
       $169 = HEAP32[$168>>2]|0;
       $170 = (($167) + 4)|0;
       $171 = $170;
       $172 = HEAP32[$171>>2]|0;
       $173 = HEAP32[$zToken>>2]|0;
       $174 = HEAP32[$i1>>2]|0;
       $175 = HEAP32[$0>>2]|0;
       $176 = ((($175)) + 252|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = (($177) + (($174*24)|0)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = HEAP32[$iLang>>2]|0;
       $181 = HEAP32[$i1>>2]|0;
       $182 = $iDocid;
       $183 = $182;
       $184 = HEAP32[$183>>2]|0;
       $185 = (($182) + 4)|0;
       $186 = $185;
       $187 = HEAP32[$186>>2]|0;
       $188 = HEAP32[$iCol>>2]|0;
       $189 = HEAP32[$iPos>>2]|0;
       $190 = (_fts3ChecksumEntry($173,$179,$180,$181,$184,$187,$188,$189)|0);
       $191 = tempRet0;
       $192 = $169 ^ $190;
       $193 = $172 ^ $191;
       $194 = $cksum2;
       $195 = $194;
       HEAP32[$195>>2] = $192;
       $196 = (($194) + 4)|0;
       $197 = $196;
       HEAP32[$197>>2] = $193;
      }
      $198 = HEAP32[$i1>>2]|0;
      $199 = (($198) + 1)|0;
      HEAP32[$i1>>2] = $199;
     }
    }
    $200 = HEAP32[$pT>>2]|0;
    $201 = ($200|0)!=(0|0);
    if ($201) {
     $202 = HEAP32[$pModule>>2]|0;
     $203 = ((($202)) + 16|0);
     $204 = HEAP32[$203>>2]|0;
     $205 = HEAP32[$pT>>2]|0;
     (FUNCTION_TABLE_ii[$204 & 255]($205)|0);
    }
    $206 = HEAP32[$rc>>2]|0;
    $207 = ($206|0)==(101);
    if ($207) {
     HEAP32[$rc>>2] = 0;
    }
   }
   $208 = HEAP32[$iCol>>2]|0;
   $209 = (($208) + 1)|0;
   HEAP32[$iCol>>2] = $209;
  }
 }
 (_sqlite3_finalize($80)|0);
 $210 = $cksum1;
 $211 = $210;
 $212 = HEAP32[$211>>2]|0;
 $213 = (($210) + 4)|0;
 $214 = $213;
 $215 = HEAP32[$214>>2]|0;
 $216 = $cksum2;
 $217 = $216;
 $218 = HEAP32[$217>>2]|0;
 $219 = (($216) + 4)|0;
 $220 = $219;
 $221 = HEAP32[$220>>2]|0;
 $222 = ($212|0)==($218|0);
 $223 = ($215|0)==($221|0);
 $224 = $222 & $223;
 $225 = $224&1;
 $226 = HEAP32[$1>>2]|0;
 HEAP32[$226>>2] = $225;
 $227 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($227|0);
}
function _fts3ChecksumIndex($p,$iLangid,$iIndex,$pRc) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $csr = 0, $filter = 0, $iCol = 0, $iDocid = 0, $iPos = 0, $iVal = 0, $or$cond = 0, $pCsr = 0, $pEnd = 0, $rc = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $filter = sp + 112|0;
 $csr = sp + 56|0;
 $rc = sp + 48|0;
 $cksum = sp + 32|0;
 $pCsr = sp + 44|0;
 $pEnd = sp + 40|0;
 $iDocid = sp + 24|0;
 $iCol = sp + 16|0;
 $iPos = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $pRc;
 $4 = $cksum;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 ;HEAP32[$filter>>2]=0|0;HEAP32[$filter+4>>2]=0|0;HEAP32[$filter+8>>2]=0|0;HEAP32[$filter+12>>2]=0|0;
 dest=$csr; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = ((($filter)) + 12|0);
 HEAP32[$8>>2] = 3;
 $9 = ((($filter)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 | 16;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3Fts3SegReaderCursor($12,$13,$14,-2,0,0,0,1,$csr)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (_sqlite3Fts3SegReaderStart($18,$csr,$filter)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if (!($21)) {
  _sqlite3Fts3SegReaderFinish($csr);
  $179 = HEAP32[$rc>>2]|0;
  $180 = HEAP32[$3>>2]|0;
  HEAP32[$180>>2] = $179;
  $181 = $cksum;
  $182 = $181;
  $183 = HEAP32[$182>>2]|0;
  $184 = (($181) + 4)|0;
  $185 = $184;
  $186 = HEAP32[$185>>2]|0;
  tempRet0 = ($186);
  STACKTOP = sp;return ($183|0);
 }
 L6: while(1) {
  $22 = HEAP32[$0>>2]|0;
  $23 = (_sqlite3Fts3SegReaderStep($22,$csr)|0);
  HEAP32[$rc>>2] = $23;
  $24 = (100)==($23|0);
  if (!($24)) {
   break;
  }
  $25 = ((($csr)) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pCsr>>2] = $26;
  $27 = ((($csr)) + 52|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$pCsr>>2]|0;
  $30 = (($29) + ($28)|0);
  HEAP32[$pEnd>>2] = $30;
  $31 = $iDocid;
  $32 = $31;
  HEAP32[$32>>2] = 0;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = 0;
  $35 = $iCol;
  $36 = $35;
  HEAP32[$36>>2] = 0;
  $37 = (($35) + 4)|0;
  $38 = $37;
  HEAP32[$38>>2] = 0;
  $39 = $iPos;
  $40 = $39;
  HEAP32[$40>>2] = 0;
  $41 = (($39) + 4)|0;
  $42 = $41;
  HEAP32[$42>>2] = 0;
  $43 = HEAP32[$pCsr>>2]|0;
  $44 = (_sqlite3Fts3GetVarint($43,$iDocid)|0);
  $45 = HEAP32[$pCsr>>2]|0;
  $46 = (($45) + ($44)|0);
  HEAP32[$pCsr>>2] = $46;
  while(1) {
   $47 = HEAP32[$pCsr>>2]|0;
   $48 = HEAP32[$pEnd>>2]|0;
   $49 = ($47>>>0)<($48>>>0);
   if (!($49)) {
    continue L6;
   }
   $50 = $iVal;
   $51 = $50;
   HEAP32[$51>>2] = 0;
   $52 = (($50) + 4)|0;
   $53 = $52;
   HEAP32[$53>>2] = 0;
   $54 = HEAP32[$pCsr>>2]|0;
   $55 = (_sqlite3Fts3GetVarint($54,$iVal)|0);
   $56 = HEAP32[$pCsr>>2]|0;
   $57 = (($56) + ($55)|0);
   HEAP32[$pCsr>>2] = $57;
   $58 = HEAP32[$pCsr>>2]|0;
   $59 = HEAP32[$pEnd>>2]|0;
   $60 = ($58>>>0)<($59>>>0);
   if (!($60)) {
    continue;
   }
   $61 = $iVal;
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $64 = (($61) + 4)|0;
   $65 = $64;
   $66 = HEAP32[$65>>2]|0;
   $67 = ($63|0)==(0);
   $68 = ($66|0)==(0);
   $69 = $67 & $68;
   $70 = $iVal;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($70) + 4)|0;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = ($72|0)==(1);
   $77 = ($75|0)==(0);
   $78 = $76 & $77;
   $or$cond = $69 | $78;
   if (!($or$cond)) {
    $121 = $iVal;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = (($121) + 4)|0;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = (_i64Subtract(($123|0),($126|0),2,0)|0);
    $128 = tempRet0;
    $129 = $iPos;
    $130 = $129;
    $131 = HEAP32[$130>>2]|0;
    $132 = (($129) + 4)|0;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (_i64Add(($131|0),($134|0),($127|0),($128|0))|0);
    $136 = tempRet0;
    $137 = $iPos;
    $138 = $137;
    HEAP32[$138>>2] = $135;
    $139 = (($137) + 4)|0;
    $140 = $139;
    HEAP32[$140>>2] = $136;
    $141 = $cksum;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = (($141) + 4)|0;
    $145 = $144;
    $146 = HEAP32[$145>>2]|0;
    $147 = ((($csr)) + 40|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = ((($csr)) + 44|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = HEAP32[$1>>2]|0;
    $152 = HEAP32[$2>>2]|0;
    $153 = $iDocid;
    $154 = $153;
    $155 = HEAP32[$154>>2]|0;
    $156 = (($153) + 4)|0;
    $157 = $156;
    $158 = HEAP32[$157>>2]|0;
    $159 = $iCol;
    $160 = $159;
    $161 = HEAP32[$160>>2]|0;
    $162 = (($159) + 4)|0;
    $163 = $162;
    $164 = HEAP32[$163>>2]|0;
    $165 = $iPos;
    $166 = $165;
    $167 = HEAP32[$166>>2]|0;
    $168 = (($165) + 4)|0;
    $169 = $168;
    $170 = HEAP32[$169>>2]|0;
    $171 = (_fts3ChecksumEntry($148,$150,$151,$152,$155,$158,$161,$167)|0);
    $172 = tempRet0;
    $173 = $143 ^ $171;
    $174 = $146 ^ $172;
    $175 = $cksum;
    $176 = $175;
    HEAP32[$176>>2] = $173;
    $177 = (($175) + 4)|0;
    $178 = $177;
    HEAP32[$178>>2] = $174;
    continue;
   }
   $79 = $iCol;
   $80 = $79;
   HEAP32[$80>>2] = 0;
   $81 = (($79) + 4)|0;
   $82 = $81;
   HEAP32[$82>>2] = 0;
   $83 = $iPos;
   $84 = $83;
   HEAP32[$84>>2] = 0;
   $85 = (($83) + 4)|0;
   $86 = $85;
   HEAP32[$86>>2] = 0;
   $87 = $iVal;
   $88 = $87;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($87) + 4)|0;
   $91 = $90;
   $92 = HEAP32[$91>>2]|0;
   $93 = ($89|0)!=(0);
   $94 = ($92|0)!=(0);
   $95 = $93 | $94;
   $96 = HEAP32[$pCsr>>2]|0;
   if ($95) {
    $97 = (_sqlite3Fts3GetVarint($96,$iCol)|0);
    $98 = HEAP32[$pCsr>>2]|0;
    $99 = (($98) + ($97)|0);
    HEAP32[$pCsr>>2] = $99;
    continue;
   } else {
    $100 = (_sqlite3Fts3GetVarint($96,$iVal)|0);
    $101 = HEAP32[$pCsr>>2]|0;
    $102 = (($101) + ($100)|0);
    HEAP32[$pCsr>>2] = $102;
    $103 = $iVal;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($103) + 4)|0;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = $iDocid;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = (_i64Add(($111|0),($114|0),($105|0),($108|0))|0);
    $116 = tempRet0;
    $117 = $iDocid;
    $118 = $117;
    HEAP32[$118>>2] = $115;
    $119 = (($117) + 4)|0;
    $120 = $119;
    HEAP32[$120>>2] = $116;
    continue;
   }
  }
 }
 _sqlite3Fts3SegReaderFinish($csr);
 $179 = HEAP32[$rc>>2]|0;
 $180 = HEAP32[$3>>2]|0;
 HEAP32[$180>>2] = $179;
 $181 = $cksum;
 $182 = $181;
 $183 = HEAP32[$182>>2]|0;
 $184 = (($181) + 4)|0;
 $185 = $184;
 $186 = HEAP32[$185>>2]|0;
 tempRet0 = ($186);
 STACKTOP = sp;return ($183|0);
}
function _fts3ChecksumEntry($zTerm,$nTerm,$iLangid,$iIndex,$0,$1,$iCol,$iPos) {
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $0 = $0|0;
 $1 = $1|0;
 $iCol = $iCol|0;
 $iPos = $iPos|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 8|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $i = sp + 16|0;
 $ret = sp;
 HEAP32[$2>>2] = $zTerm;
 HEAP32[$3>>2] = $nTerm;
 HEAP32[$4>>2] = $iLangid;
 HEAP32[$5>>2] = $iIndex;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $iCol;
 HEAP32[$8>>2] = $iPos;
 $13 = $6;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = $ret;
 $20 = $19;
 HEAP32[$20>>2] = $15;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $18;
 $23 = $ret;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (_bitshift64Shl(($25|0),($28|0),3)|0);
 $30 = tempRet0;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = (_i64Add(($29|0),($30|0),($31|0),($33|0))|0);
 $35 = tempRet0;
 $36 = $ret;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (_i64Add(($38|0),($41|0),($34|0),($35|0))|0);
 $43 = tempRet0;
 $44 = $ret;
 $45 = $44;
 HEAP32[$45>>2] = $42;
 $46 = (($44) + 4)|0;
 $47 = $46;
 HEAP32[$47>>2] = $43;
 $48 = $ret;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (_bitshift64Shl(($50|0),($53|0),3)|0);
 $55 = tempRet0;
 $56 = HEAP32[$5>>2]|0;
 $57 = ($56|0)<(0);
 $58 = $57 << 31 >> 31;
 $59 = (_i64Add(($54|0),($55|0),($56|0),($58|0))|0);
 $60 = tempRet0;
 $61 = $ret;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = (_i64Add(($63|0),($66|0),($59|0),($60|0))|0);
 $68 = tempRet0;
 $69 = $ret;
 $70 = $69;
 HEAP32[$70>>2] = $67;
 $71 = (($69) + 4)|0;
 $72 = $71;
 HEAP32[$72>>2] = $68;
 $73 = $ret;
 $74 = $73;
 $75 = HEAP32[$74>>2]|0;
 $76 = (($73) + 4)|0;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (_bitshift64Shl(($75|0),($78|0),3)|0);
 $80 = tempRet0;
 $81 = HEAP32[$7>>2]|0;
 $82 = ($81|0)<(0);
 $83 = $82 << 31 >> 31;
 $84 = (_i64Add(($79|0),($80|0),($81|0),($83|0))|0);
 $85 = tempRet0;
 $86 = $ret;
 $87 = $86;
 $88 = HEAP32[$87>>2]|0;
 $89 = (($86) + 4)|0;
 $90 = $89;
 $91 = HEAP32[$90>>2]|0;
 $92 = (_i64Add(($88|0),($91|0),($84|0),($85|0))|0);
 $93 = tempRet0;
 $94 = $ret;
 $95 = $94;
 HEAP32[$95>>2] = $92;
 $96 = (($94) + 4)|0;
 $97 = $96;
 HEAP32[$97>>2] = $93;
 $98 = $ret;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (($98) + 4)|0;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = (_bitshift64Shl(($100|0),($103|0),3)|0);
 $105 = tempRet0;
 $106 = HEAP32[$8>>2]|0;
 $107 = ($106|0)<(0);
 $108 = $107 << 31 >> 31;
 $109 = (_i64Add(($104|0),($105|0),($106|0),($108|0))|0);
 $110 = tempRet0;
 $111 = $ret;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = (_i64Add(($113|0),($116|0),($109|0),($110|0))|0);
 $118 = tempRet0;
 $119 = $ret;
 $120 = $119;
 HEAP32[$120>>2] = $117;
 $121 = (($119) + 4)|0;
 $122 = $121;
 HEAP32[$122>>2] = $118;
 HEAP32[$i>>2] = 0;
 while(1) {
  $123 = HEAP32[$i>>2]|0;
  $124 = HEAP32[$3>>2]|0;
  $125 = ($123|0)<($124|0);
  $126 = $ret;
  $127 = $126;
  $128 = HEAP32[$127>>2]|0;
  $129 = (($126) + 4)|0;
  $130 = $129;
  $131 = HEAP32[$130>>2]|0;
  if (!($125)) {
   break;
  }
  $132 = (_bitshift64Shl(($128|0),($131|0),3)|0);
  $133 = tempRet0;
  $134 = HEAP32[$i>>2]|0;
  $135 = HEAP32[$2>>2]|0;
  $136 = (($135) + ($134)|0);
  $137 = HEAP8[$136>>0]|0;
  $138 = $137 << 24 >> 24;
  $139 = ($138|0)<(0);
  $140 = $139 << 31 >> 31;
  $141 = (_i64Add(($132|0),($133|0),($138|0),($140|0))|0);
  $142 = tempRet0;
  $143 = $ret;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (($143) + 4)|0;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (_i64Add(($145|0),($148|0),($141|0),($142|0))|0);
  $150 = tempRet0;
  $151 = $ret;
  $152 = $151;
  HEAP32[$152>>2] = $149;
  $153 = (($151) + 4)|0;
  $154 = $153;
  HEAP32[$154>>2] = $150;
  $155 = HEAP32[$i>>2]|0;
  $156 = (($155) + 1)|0;
  HEAP32[$i>>2] = $156;
 }
 tempRet0 = ($131);
 STACKTOP = sp;return ($128|0);
}
function _fts3Getint($pz) {
 $pz = $pz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $z = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pz;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$z>>2] = $2;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$z>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($5|0)>=(48);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$z>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)<=(57);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11*10)|0;
  $13 = HEAP32[$z>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP32[$z>>2] = $14;
  $15 = HEAP8[$13>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = (($12) + ($16))|0;
  $18 = (($17) - 48)|0;
  HEAP32[$i>>2] = $18;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$z>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  HEAP32[$20>>2] = $19;
  $21 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3Fts3Incrmerge($p,$nMerge,$nMin) {
 $p = $p|0;
 $nMerge = $nMerge|0;
 $nMin = $nMin|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDirtyHint = 0, $bIgnore = 0, $bUseHint = 0, $hint = 0, $iAbsLevel = 0, $iHintAbsLevel = 0, $iIdx = 0;
 var $nAlloc = 0, $nHint = 0, $nHintSeg = 0, $nKey = 0, $nMod = 0, $nRem = 0, $nSeg = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pCsr = 0, $pFilter = 0, $pFindLevel = 0, $pWriter = 0, $rc = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $rc = sp + 96|0;
 $nRem = sp + 92|0;
 $pCsr = sp + 88|0;
 $pFilter = sp + 84|0;
 $pWriter = sp + 80|0;
 $nSeg = sp + 76|0;
 $iAbsLevel = sp + 16|0;
 $hint = sp + 64|0;
 $bDirtyHint = sp + 60|0;
 $nAlloc = sp + 56|0;
 $nMod = sp + 8|0;
 $pFindLevel = sp + 52|0;
 $bUseHint = sp + 48|0;
 $iIdx = sp + 44|0;
 $nHint = sp + 40|0;
 $iHintAbsLevel = sp;
 $nHintSeg = sp + 36|0;
 $bIgnore = sp + 32|0;
 $zKey = sp + 28|0;
 $nKey = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nMerge;
 HEAP32[$3>>2] = $nMin;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$nRem>>2] = $4;
 HEAP32[$nSeg>>2] = 0;
 $5 = $iAbsLevel;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 ;HEAP32[$hint>>2]=0|0;HEAP32[$hint+4>>2]=0|0;HEAP32[$hint+8>>2]=0|0;
 HEAP32[$bDirtyHint>>2] = 0;
 HEAP32[$nAlloc>>2] = 640;
 $9 = (_sqlite3_malloc(640)|0);
 HEAP32[$pWriter>>2] = $9;
 $10 = HEAP32[$pWriter>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  HEAP32[$0>>2] = 7;
  $312 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($312|0);
 }
 $12 = HEAP32[$pWriter>>2]|0;
 $13 = ((($12)) + 568|0);
 HEAP32[$pFilter>>2] = $13;
 $14 = HEAP32[$pFilter>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP32[$pCsr>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_fts3IncrmergeHintLoad($16,$hint)|0);
 HEAP32[$rc>>2] = $17;
 while(1) {
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)==(0);
  $20 = HEAP32[$nRem>>2]|0;
  $21 = ($20|0)>(0);
  $22 = $19 ? $21 : 0;
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 248|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $25<<10;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = $nMod;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
  HEAP32[$pFindLevel>>2] = 0;
  HEAP32[$bUseHint>>2] = 0;
  HEAP32[$iIdx>>2] = 0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (_fts3SqlStmt($33,28,$pFindLevel,0)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$pFindLevel>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  (_sqlite3_bind_int($35,1,$36)|0);
  $37 = HEAP32[$pFindLevel>>2]|0;
  $38 = (_sqlite3_step($37)|0);
  $39 = ($38|0)==(100);
  if ($39) {
   $40 = HEAP32[$pFindLevel>>2]|0;
   $41 = (_sqlite3_column_int64($40,0)|0);
   $42 = tempRet0;
   $43 = $iAbsLevel;
   $44 = $43;
   HEAP32[$44>>2] = $41;
   $45 = (($43) + 4)|0;
   $46 = $45;
   HEAP32[$46>>2] = $42;
   $47 = HEAP32[$3>>2]|0;
   HEAP32[$nSeg>>2] = $47;
  } else {
   HEAP32[$nSeg>>2] = -1;
  }
  $48 = HEAP32[$pFindLevel>>2]|0;
  $49 = (_sqlite3_reset($48)|0);
  HEAP32[$rc>>2] = $49;
  $50 = HEAP32[$rc>>2]|0;
  $51 = ($50|0)==(0);
  do {
   if ($51) {
    $52 = ((($hint)) + 4|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = ($53|0)!=(0);
    if ($54) {
     $55 = ((($hint)) + 4|0);
     $56 = HEAP32[$55>>2]|0;
     HEAP32[$nHint>>2] = $56;
     $57 = $iHintAbsLevel;
     $58 = $57;
     HEAP32[$58>>2] = 0;
     $59 = (($57) + 4)|0;
     $60 = $59;
     HEAP32[$60>>2] = 0;
     HEAP32[$nHintSeg>>2] = 0;
     $61 = (_fts3IncrmergeHintPop($hint,$iHintAbsLevel,$nHintSeg)|0);
     HEAP32[$rc>>2] = $61;
     $62 = HEAP32[$nSeg>>2]|0;
     $63 = ($62|0)<(0);
     if (!($63)) {
      $64 = $iAbsLevel;
      $65 = $64;
      $66 = HEAP32[$65>>2]|0;
      $67 = (($64) + 4)|0;
      $68 = $67;
      $69 = HEAP32[$68>>2]|0;
      $70 = $nMod;
      $71 = $70;
      $72 = HEAP32[$71>>2]|0;
      $73 = (($70) + 4)|0;
      $74 = $73;
      $75 = HEAP32[$74>>2]|0;
      $76 = (___remdi3(($66|0),($69|0),($72|0),($75|0))|0);
      $77 = tempRet0;
      $78 = $iHintAbsLevel;
      $79 = $78;
      $80 = HEAP32[$79>>2]|0;
      $81 = (($78) + 4)|0;
      $82 = $81;
      $83 = HEAP32[$82>>2]|0;
      $84 = $nMod;
      $85 = $84;
      $86 = HEAP32[$85>>2]|0;
      $87 = (($84) + 4)|0;
      $88 = $87;
      $89 = HEAP32[$88>>2]|0;
      $90 = (___remdi3(($80|0),($83|0),($86|0),($89|0))|0);
      $91 = tempRet0;
      $92 = ($77|0)>($91|0);
      $93 = ($76>>>0)>=($90>>>0);
      $94 = ($77|0)==($91|0);
      $95 = $94 & $93;
      $96 = $92 | $95;
      if (!($96)) {
       $108 = HEAP32[$nHint>>2]|0;
       $109 = ((($hint)) + 4|0);
       HEAP32[$109>>2] = $108;
       break;
      }
     }
     $97 = $iHintAbsLevel;
     $98 = $97;
     $99 = HEAP32[$98>>2]|0;
     $100 = (($97) + 4)|0;
     $101 = $100;
     $102 = HEAP32[$101>>2]|0;
     $103 = $iAbsLevel;
     $104 = $103;
     HEAP32[$104>>2] = $99;
     $105 = (($103) + 4)|0;
     $106 = $105;
     HEAP32[$106>>2] = $102;
     $107 = HEAP32[$nHintSeg>>2]|0;
     HEAP32[$nSeg>>2] = $107;
     HEAP32[$bUseHint>>2] = 1;
     HEAP32[$bDirtyHint>>2] = 1;
    }
   }
  } while(0);
  $110 = HEAP32[$nSeg>>2]|0;
  $111 = ($110|0)<(0);
  if ($111) {
   break;
  }
  $112 = HEAP32[$pWriter>>2]|0;
  _memset(($112|0),0,640)|0;
  $113 = HEAP32[$pFilter>>2]|0;
  $114 = ((($113)) + 12|0);
  HEAP32[$114>>2] = 1;
  $115 = HEAP32[$rc>>2]|0;
  $116 = ($115|0)==(0);
  do {
   if ($116) {
    $117 = HEAP32[$1>>2]|0;
    $118 = $iAbsLevel;
    $119 = $118;
    $120 = HEAP32[$119>>2]|0;
    $121 = (($118) + 4)|0;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = (_fts3IncrmergeOutputIdx($117,$120,$123,$iIdx)|0);
    HEAP32[$rc>>2] = $124;
    $125 = HEAP32[$iIdx>>2]|0;
    $126 = ($125|0)==(0);
    if (!($126)) {
     $127 = HEAP32[$bUseHint>>2]|0;
     $128 = ($127|0)!=(0);
     $129 = HEAP32[$iIdx>>2]|0;
     $130 = ($129|0)==(1);
     $or$cond = $128 & $130;
     if (!($or$cond)) {
      break;
     }
    }
    HEAP32[$bIgnore>>2] = 0;
    $131 = HEAP32[$1>>2]|0;
    $132 = $iAbsLevel;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (($132) + 4)|0;
    $136 = $135;
    $137 = HEAP32[$136>>2]|0;
    $138 = (_i64Add(($134|0),($137|0),1,0)|0);
    $139 = tempRet0;
    $140 = (_fts3SegmentIsMaxLevel($131,$138,$139,$bIgnore)|0);
    HEAP32[$rc>>2] = $140;
    $141 = HEAP32[$bIgnore>>2]|0;
    $142 = ($141|0)!=(0);
    if ($142) {
     $143 = HEAP32[$pFilter>>2]|0;
     $144 = ((($143)) + 12|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = $145 | 2;
     HEAP32[$144>>2] = $146;
    }
   }
  } while(0);
  $147 = HEAP32[$rc>>2]|0;
  $148 = ($147|0)==(0);
  if ($148) {
   $149 = HEAP32[$1>>2]|0;
   $150 = $iAbsLevel;
   $151 = $150;
   $152 = HEAP32[$151>>2]|0;
   $153 = (($150) + 4)|0;
   $154 = $153;
   $155 = HEAP32[$154>>2]|0;
   $156 = HEAP32[$nSeg>>2]|0;
   $157 = HEAP32[$pCsr>>2]|0;
   $158 = (_fts3IncrmergeCsr($149,$152,$155,$156,$157)|0);
   HEAP32[$rc>>2] = $158;
  }
  $159 = HEAP32[$rc>>2]|0;
  $160 = (0)==($159|0);
  if ($160) {
   $161 = HEAP32[$pCsr>>2]|0;
   $162 = ((($161)) + 4|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = HEAP32[$nSeg>>2]|0;
   $165 = ($163|0)==($164|0);
   if ($165) {
    $166 = HEAP32[$1>>2]|0;
    $167 = HEAP32[$pCsr>>2]|0;
    $168 = HEAP32[$pFilter>>2]|0;
    $169 = (_sqlite3Fts3SegReaderStart($166,$167,$168)|0);
    HEAP32[$rc>>2] = $169;
    $170 = (0)==($169|0);
    if ($170) {
     $171 = HEAP32[$1>>2]|0;
     $172 = HEAP32[$pCsr>>2]|0;
     $173 = (_sqlite3Fts3SegReaderStep($171,$172)|0);
     HEAP32[$rc>>2] = $173;
     $174 = (100)==($173|0);
     if ($174) {
      $175 = HEAP32[$bUseHint>>2]|0;
      $176 = ($175|0)!=(0);
      $177 = HEAP32[$iIdx>>2]|0;
      $178 = ($177|0)>(0);
      $or$cond3 = $176 & $178;
      if ($or$cond3) {
       $179 = HEAP32[$pCsr>>2]|0;
       $180 = ((($179)) + 40|0);
       $181 = HEAP32[$180>>2]|0;
       HEAP32[$zKey>>2] = $181;
       $182 = HEAP32[$pCsr>>2]|0;
       $183 = ((($182)) + 44|0);
       $184 = HEAP32[$183>>2]|0;
       HEAP32[$nKey>>2] = $184;
       $185 = HEAP32[$1>>2]|0;
       $186 = $iAbsLevel;
       $187 = $186;
       $188 = HEAP32[$187>>2]|0;
       $189 = (($186) + 4)|0;
       $190 = $189;
       $191 = HEAP32[$190>>2]|0;
       $192 = HEAP32[$iIdx>>2]|0;
       $193 = (($192) - 1)|0;
       $194 = HEAP32[$zKey>>2]|0;
       $195 = HEAP32[$nKey>>2]|0;
       $196 = HEAP32[$pWriter>>2]|0;
       $197 = (_fts3IncrmergeLoad($185,$188,$191,$193,$194,$195,$196)|0);
       HEAP32[$rc>>2] = $197;
      } else {
       $198 = HEAP32[$1>>2]|0;
       $199 = $iAbsLevel;
       $200 = $199;
       $201 = HEAP32[$200>>2]|0;
       $202 = (($199) + 4)|0;
       $203 = $202;
       $204 = HEAP32[$203>>2]|0;
       $205 = HEAP32[$iIdx>>2]|0;
       $206 = HEAP32[$pCsr>>2]|0;
       $207 = HEAP32[$pWriter>>2]|0;
       $208 = (_fts3IncrmergeWriter($198,$201,$204,$205,$206,$207)|0);
       HEAP32[$rc>>2] = $208;
      }
      $209 = HEAP32[$rc>>2]|0;
      $210 = ($209|0)==(0);
      if ($210) {
       $211 = HEAP32[$pWriter>>2]|0;
       $212 = HEAP32[$211>>2]|0;
       $213 = ($212|0)!=(0);
       if ($213) {
        while(1) {
         $214 = HEAP32[$1>>2]|0;
         $215 = HEAP32[$pWriter>>2]|0;
         $216 = HEAP32[$pCsr>>2]|0;
         $217 = (_fts3IncrmergeAppend($214,$215,$216)|0);
         HEAP32[$rc>>2] = $217;
         $218 = HEAP32[$rc>>2]|0;
         $219 = ($218|0)==(0);
         if ($219) {
          $220 = HEAP32[$1>>2]|0;
          $221 = HEAP32[$pCsr>>2]|0;
          $222 = (_sqlite3Fts3SegReaderStep($220,$221)|0);
          HEAP32[$rc>>2] = $222;
         }
         $223 = HEAP32[$pWriter>>2]|0;
         $224 = ((($223)) + 4|0);
         $225 = HEAP32[$224>>2]|0;
         $226 = HEAP32[$nRem>>2]|0;
         $227 = ($225|0)>=($226|0);
         $228 = HEAP32[$rc>>2]|0;
         $229 = ($228|0)==(100);
         $or$cond5 = $227 & $229;
         if ($or$cond5) {
          HEAP32[$rc>>2] = 0;
         }
         $230 = HEAP32[$rc>>2]|0;
         $231 = ($230|0)==(100);
         if (!($231)) {
          break;
         }
        }
        $232 = HEAP32[$rc>>2]|0;
        $233 = ($232|0)==(0);
        if ($233) {
         $234 = HEAP32[$pWriter>>2]|0;
         $235 = ((($234)) + 4|0);
         $236 = HEAP32[$235>>2]|0;
         $237 = (1 + ($236))|0;
         $238 = HEAP32[$nRem>>2]|0;
         $239 = (($238) - ($237))|0;
         HEAP32[$nRem>>2] = $239;
         $240 = HEAP32[$1>>2]|0;
         $241 = $iAbsLevel;
         $242 = $241;
         $243 = HEAP32[$242>>2]|0;
         $244 = (($241) + 4)|0;
         $245 = $244;
         $246 = HEAP32[$245>>2]|0;
         $247 = HEAP32[$pCsr>>2]|0;
         $248 = (_fts3IncrmergeChomp($240,$243,$246,$247,$nSeg)|0);
         HEAP32[$rc>>2] = $248;
         $249 = HEAP32[$nSeg>>2]|0;
         $250 = ($249|0)!=(0);
         if ($250) {
          HEAP32[$bDirtyHint>>2] = 1;
          $251 = $iAbsLevel;
          $252 = $251;
          $253 = HEAP32[$252>>2]|0;
          $254 = (($251) + 4)|0;
          $255 = $254;
          $256 = HEAP32[$255>>2]|0;
          $257 = HEAP32[$nSeg>>2]|0;
          _fts3IncrmergeHintPush($hint,$253,$256,$257,$rc);
         }
        }
       }
      }
      $258 = HEAP32[$nSeg>>2]|0;
      $259 = ($258|0)!=(0);
      if ($259) {
       $260 = HEAP32[$pWriter>>2]|0;
       $261 = ((($260)) + 40|0);
       $262 = $261;
       $263 = $262;
       $264 = HEAP32[$263>>2]|0;
       $265 = (($262) + 4)|0;
       $266 = $265;
       $267 = HEAP32[$266>>2]|0;
       $268 = (___muldi3(($264|0),($267|0),-1,-1)|0);
       $269 = tempRet0;
       $270 = HEAP32[$pWriter>>2]|0;
       $271 = ((($270)) + 40|0);
       $272 = $271;
       $273 = $272;
       HEAP32[$273>>2] = $268;
       $274 = (($272) + 4)|0;
       $275 = $274;
       HEAP32[$275>>2] = $269;
      }
      $276 = HEAP32[$1>>2]|0;
      $277 = HEAP32[$pWriter>>2]|0;
      _fts3IncrmergeRelease($276,$277,$rc);
      $278 = HEAP32[$nSeg>>2]|0;
      $279 = ($278|0)==(0);
      if ($279) {
       $280 = HEAP32[$pWriter>>2]|0;
       $281 = ((($280)) + 48|0);
       $282 = HEAP8[$281>>0]|0;
       $283 = $282&255;
       $284 = ($283|0)==(0);
       if ($284) {
        $285 = HEAP32[$1>>2]|0;
        $286 = $iAbsLevel;
        $287 = $286;
        $288 = HEAP32[$287>>2]|0;
        $289 = (($286) + 4)|0;
        $290 = $289;
        $291 = HEAP32[$290>>2]|0;
        $292 = (_i64Add(($288|0),($291|0),1,0)|0);
        $293 = tempRet0;
        $294 = HEAP32[$pWriter>>2]|0;
        $295 = ((($294)) + 40|0);
        $296 = $295;
        $297 = $296;
        $298 = HEAP32[$297>>2]|0;
        $299 = (($296) + 4)|0;
        $300 = $299;
        $301 = HEAP32[$300>>2]|0;
        (_fts3PromoteSegments($285,$292,$293,$298,$301)|0);
       }
      }
     }
    }
   }
  }
  $302 = HEAP32[$pCsr>>2]|0;
  _sqlite3Fts3SegReaderFinish($302);
 }
 $303 = HEAP32[$bDirtyHint>>2]|0;
 $304 = ($303|0)!=(0);
 $305 = HEAP32[$rc>>2]|0;
 $306 = ($305|0)==(0);
 $or$cond7 = $304 & $306;
 if ($or$cond7) {
  $307 = HEAP32[$1>>2]|0;
  $308 = (_fts3IncrmergeHintStore($307,$hint)|0);
  HEAP32[$rc>>2] = $308;
 }
 $309 = HEAP32[$pWriter>>2]|0;
 _sqlite3_free($309);
 $310 = HEAP32[$hint>>2]|0;
 _sqlite3_free($310);
 $311 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $311;
 $312 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($312|0);
}
function _fts3IncrmergeHintLoad($p,$pHint) {
 $p = $p|0;
 $pHint = $pHint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHint = 0, $nHint = 0, $pSelect = 0, $rc = 0, $rc2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pSelect = sp + 16|0;
 $rc = sp + 12|0;
 $rc2 = sp + 8|0;
 $aHint = sp + 4|0;
 $nHint = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pHint;
 HEAP32[$pSelect>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_fts3SqlStmt($4,22,$pSelect,0)|0);
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$pSelect>>2]|0;
 (_sqlite3_bind_int($8,1,1)|0);
 $9 = HEAP32[$pSelect>>2]|0;
 $10 = (_sqlite3_step($9)|0);
 $11 = (100)==($10|0);
 if ($11) {
  $12 = HEAP32[$pSelect>>2]|0;
  $13 = (_sqlite3_column_blob($12,0)|0);
  HEAP32[$aHint>>2] = $13;
  $14 = HEAP32[$pSelect>>2]|0;
  $15 = (_sqlite3_column_bytes($14,0)|0);
  HEAP32[$nHint>>2] = $15;
  $16 = HEAP32[$aHint>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$nHint>>2]|0;
   _blobGrowBuffer($18,$19,$rc);
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)==(0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$aHint>>2]|0;
    $25 = HEAP32[$nHint>>2]|0;
    _memcpy(($23|0),($24|0),($25|0))|0;
    $26 = HEAP32[$nHint>>2]|0;
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 4|0);
    HEAP32[$28>>2] = $26;
   }
  }
 }
 $29 = HEAP32[$pSelect>>2]|0;
 $30 = (_sqlite3_reset($29)|0);
 HEAP32[$rc2>>2] = $30;
 $31 = HEAP32[$rc>>2]|0;
 $32 = ($31|0)==(0);
 if (!($32)) {
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $33 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _fts3IncrmergeHintPop($pHint,$piAbsLevel,$pnInput) {
 $pHint = $pHint|0;
 $piAbsLevel = $piAbsLevel|0;
 $pnInput = $pnInput|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $nHint = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $nHint = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pHint;
 HEAP32[$2>>2] = $piAbsLevel;
 HEAP32[$3>>2] = $pnInput;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$nHint>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 2)|0;
 HEAP32[$i>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)>(0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) - 1)|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($14)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = $19 & 128;
  $21 = ($20|0)!=(0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + -1)|0;
  HEAP32[$i>>2] = $23;
 }
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = ($24|0)>(0);
  if ($25) {
   $26 = HEAP32[$i>>2]|0;
   $27 = (($26) - 1)|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + ($27)|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31 << 24 >> 24;
   $33 = $32 & 128;
   $34 = ($33|0)!=(0);
   $71 = $34;
  } else {
   $71 = 0;
  }
  $35 = HEAP32[$i>>2]|0;
  if (!($71)) {
   break;
  }
  $36 = (($35) + -1)|0;
  HEAP32[$i>>2] = $36;
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 4|0);
 HEAP32[$38>>2] = $35;
 $39 = HEAP32[$i>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + ($39)|0);
 $43 = HEAP32[$2>>2]|0;
 $44 = (_sqlite3Fts3GetVarint($42,$43)|0);
 $45 = HEAP32[$i>>2]|0;
 $46 = (($45) + ($44))|0;
 HEAP32[$i>>2] = $46;
 $47 = HEAP32[$i>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + ($47)|0);
 $51 = HEAP8[$50>>0]|0;
 $52 = $51&255;
 $53 = $52 & 128;
 $54 = ($53|0)!=(0);
 $55 = HEAP32[$i>>2]|0;
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + ($55)|0);
 if ($54) {
  $59 = HEAP32[$3>>2]|0;
  $60 = (_sqlite3Fts3GetVarint32($58,$59)|0);
  $66 = $60;
 } else {
  $61 = HEAP8[$58>>0]|0;
  $62 = $61&255;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  $66 = 1;
 }
 $64 = HEAP32[$i>>2]|0;
 $65 = (($64) + ($66))|0;
 HEAP32[$i>>2] = $65;
 $67 = HEAP32[$i>>2]|0;
 $68 = HEAP32[$nHint>>2]|0;
 $69 = ($67|0)!=($68|0);
 if ($69) {
  HEAP32[$0>>2] = 267;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 } else {
  HEAP32[$0>>2] = 0;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 return (0)|0;
}
function _fts3IncrmergeOutputIdx($p,$0,$1,$piIdx) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $piIdx = $piIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOutputIdx = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $pOutputIdx = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $piIdx;
 HEAP32[$pOutputIdx>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,8,$pOutputIdx,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $28 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $13 = HEAP32[$pOutputIdx>>2]|0;
 $14 = $3;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 (_sqlite3_bind_int64($13,1,$20,$21)|0);
 $22 = HEAP32[$pOutputIdx>>2]|0;
 (_sqlite3_step($22)|0);
 $23 = HEAP32[$pOutputIdx>>2]|0;
 $24 = (_sqlite3_column_int($23,0)|0);
 $25 = HEAP32[$4>>2]|0;
 HEAP32[$25>>2] = $24;
 $26 = HEAP32[$pOutputIdx>>2]|0;
 $27 = (_sqlite3_reset($26)|0);
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fts3SegmentIsMaxLevel($p,$0,$1,$pbMax) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pbMax = $pbMax|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp;
 $5 = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $p;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $pbMax;
 $10 = HEAP32[$3>>2]|0;
 $11 = (_fts3SqlStmt($10,15,$pStmt,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $14;
  $48 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $15 = HEAP32[$pStmt>>2]|0;
 $16 = $4;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (_i64Add(($18|0),($21|0),1,0)|0);
 $23 = tempRet0;
 (_sqlite3_bind_int64($15,1,$22,$23)|0);
 $24 = HEAP32[$pStmt>>2]|0;
 $25 = $4;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (___divdi3(($27|0),($30|0),1024,0)|0);
 $32 = tempRet0;
 $33 = (_i64Add(($31|0),($32|0),1,0)|0);
 $34 = tempRet0;
 $35 = (___muldi3(($33|0),($34|0),1024,0)|0);
 $36 = tempRet0;
 (_sqlite3_bind_int64($24,2,$35,$36)|0);
 $37 = HEAP32[$5>>2]|0;
 HEAP32[$37>>2] = 0;
 $38 = HEAP32[$pStmt>>2]|0;
 $39 = (_sqlite3_step($38)|0);
 $40 = (100)==($39|0);
 if ($40) {
  $41 = HEAP32[$pStmt>>2]|0;
  $42 = (_sqlite3_column_type($41,0)|0);
  $43 = ($42|0)==(5);
  $44 = $43&1;
  $45 = HEAP32[$5>>2]|0;
  HEAP32[$45>>2] = $44;
 }
 $46 = HEAP32[$pStmt>>2]|0;
 $47 = (_sqlite3_reset($46)|0);
 HEAP32[$2>>2] = $47;
 $48 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _fts3IncrmergeCsr($p,$0,$1,$nSeg,$pCsr) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $nSeg = $nSeg|0;
 $pCsr = $pCsr|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $nByte = 0, $pStmt = 0, $rc = 0, $rc2 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $rc = sp + 24|0;
 $pStmt = sp + 20|0;
 $nByte = sp + 16|0;
 $i = sp + 12|0;
 $rc2 = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nSeg;
 HEAP32[$5>>2] = $pCsr;
 HEAP32[$pStmt>>2] = 0;
 $10 = HEAP32[$5>>2]|0;
 dest=$10; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $11 = HEAP32[$4>>2]|0;
 $12 = $11<<2;
 HEAP32[$nByte>>2] = $12;
 $13 = HEAP32[$nByte>>2]|0;
 $14 = (_sqlite3_malloc($13)|0);
 $15 = HEAP32[$5>>2]|0;
 HEAP32[$15>>2] = $14;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$rc>>2] = 7;
 } else {
  $19 = HEAP32[$5>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$nByte>>2]|0;
  _memset(($20|0),0,($21|0))|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = (_fts3SqlStmt($22,12,$pStmt,0)|0);
  HEAP32[$rc>>2] = $23;
 }
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if (!($25)) {
  $71 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $26 = HEAP32[$pStmt>>2]|0;
 $27 = $3;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3_bind_int64($26,1,$29,$32)|0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)==(0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$pStmt>>2]|0;
  $36 = (_sqlite3_step($35)|0);
  $37 = ($36|0)==(100);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$4>>2]|0;
  $40 = ($38|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$pStmt>>2]|0;
  $43 = (_sqlite3_column_int64($42,1)|0);
  $44 = tempRet0;
  $45 = HEAP32[$pStmt>>2]|0;
  $46 = (_sqlite3_column_int64($45,2)|0);
  $47 = tempRet0;
  $48 = HEAP32[$pStmt>>2]|0;
  $49 = (_sqlite3_column_int64($48,3)|0);
  $50 = tempRet0;
  $51 = HEAP32[$pStmt>>2]|0;
  $52 = (_sqlite3_column_blob($51,4)|0);
  $53 = HEAP32[$pStmt>>2]|0;
  $54 = (_sqlite3_column_bytes($53,4)|0);
  $55 = HEAP32[$i>>2]|0;
  $56 = HEAP32[$5>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($55<<2)|0);
  $59 = (_sqlite3Fts3SegReaderNew($41,0,$43,$44,$46,$47,$49,$50,$52,$54,$58)|0);
  HEAP32[$rc>>2] = $59;
  $60 = HEAP32[$5>>2]|0;
  $61 = ((($60)) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) + 1)|0;
  HEAP32[$61>>2] = $63;
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 $66 = HEAP32[$pStmt>>2]|0;
 $67 = (_sqlite3_reset($66)|0);
 HEAP32[$rc2>>2] = $67;
 $68 = HEAP32[$rc>>2]|0;
 $69 = ($68|0)==(0);
 if (!($69)) {
  $71 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $70 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $70;
 $71 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($71|0);
}
function _fts3IncrmergeLoad($p,$0,$1,$iIdx,$zKey,$nKey,$pWriter) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $zKey = $zKey|0;
 $nKey = $nKey|0;
 $pWriter = $pWriter|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBlock = 0, $aLeaf = 0, $aRoot = 0, $bAppendable = 0, $i = 0, $iEnd = 0, $iLeafEnd = 0, $iStart = 0;
 var $nBlock = 0, $nHeight = 0, $nLeaf = 0, $nRoot = 0, $or$cond = 0, $or$cond3 = 0, $pNode = 0, $pSelect = 0, $rc = 0, $rc2 = 0, $reader = 0, $reader1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $2 = sp + 200|0;
 $3 = sp + 196|0;
 $4 = sp + 120|0;
 $5 = sp + 192|0;
 $6 = sp + 188|0;
 $7 = sp + 184|0;
 $8 = sp + 180|0;
 $rc = sp + 176|0;
 $pSelect = sp + 172|0;
 $iStart = sp + 112|0;
 $iLeafEnd = sp + 104|0;
 $iEnd = sp + 96|0;
 $aRoot = sp + 168|0;
 $nRoot = sp + 164|0;
 $rc2 = sp + 160|0;
 $bAppendable = sp + 156|0;
 $aLeaf = sp + 152|0;
 $nLeaf = sp + 148|0;
 $reader = sp + 48|0;
 $i = sp + 144|0;
 $nHeight = sp + 140|0;
 $pNode = sp + 136|0;
 $reader1 = sp;
 $aBlock = sp + 132|0;
 $nBlock = sp + 128|0;
 HEAP32[$3>>2] = $p;
 $9 = $4;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$5>>2] = $iIdx;
 HEAP32[$6>>2] = $zKey;
 HEAP32[$7>>2] = $nKey;
 HEAP32[$8>>2] = $pWriter;
 HEAP32[$pSelect>>2] = 0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_fts3SqlStmt($13,32,$pSelect,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = $iStart;
  $18 = $17;
  HEAP32[$18>>2] = 0;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = 0;
  $21 = $iLeafEnd;
  $22 = $21;
  HEAP32[$22>>2] = 0;
  $23 = (($21) + 4)|0;
  $24 = $23;
  HEAP32[$24>>2] = 0;
  $25 = $iEnd;
  $26 = $25;
  HEAP32[$26>>2] = 0;
  $27 = (($25) + 4)|0;
  $28 = $27;
  HEAP32[$28>>2] = 0;
  HEAP32[$aRoot>>2] = 0;
  HEAP32[$nRoot>>2] = 0;
  HEAP32[$bAppendable>>2] = 0;
  $29 = HEAP32[$pSelect>>2]|0;
  $30 = $4;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_i64Add(($32|0),($35|0),1,0)|0);
  $37 = tempRet0;
  (_sqlite3_bind_int64($29,1,$36,$37)|0);
  $38 = HEAP32[$pSelect>>2]|0;
  $39 = HEAP32[$5>>2]|0;
  (_sqlite3_bind_int($38,2,$39)|0);
  $40 = HEAP32[$pSelect>>2]|0;
  $41 = (_sqlite3_step($40)|0);
  $42 = ($41|0)==(100);
  $43 = HEAP32[$pSelect>>2]|0;
  if (!($42)) {
   $116 = (_sqlite3_reset($43)|0);
   HEAP32[$2>>2] = $116;
   $393 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($393|0);
  }
  $44 = (_sqlite3_column_int64($43,1)|0);
  $45 = tempRet0;
  $46 = $iStart;
  $47 = $46;
  HEAP32[$47>>2] = $44;
  $48 = (($46) + 4)|0;
  $49 = $48;
  HEAP32[$49>>2] = $45;
  $50 = HEAP32[$pSelect>>2]|0;
  $51 = (_sqlite3_column_int64($50,2)|0);
  $52 = tempRet0;
  $53 = $iLeafEnd;
  $54 = $53;
  HEAP32[$54>>2] = $51;
  $55 = (($53) + 4)|0;
  $56 = $55;
  HEAP32[$56>>2] = $52;
  $57 = HEAP32[$pSelect>>2]|0;
  $58 = HEAP32[$8>>2]|0;
  $59 = ((($58)) + 40|0);
  _fts3ReadEndBlockField($57,3,$iEnd,$59);
  $60 = HEAP32[$8>>2]|0;
  $61 = ((($60)) + 40|0);
  $62 = $61;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)<(0);
  if ($68) {
   $69 = HEAP32[$8>>2]|0;
   $70 = ((($69)) + 40|0);
   $71 = $70;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($71) + 4)|0;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (___muldi3(($73|0),($76|0),-1,-1)|0);
   $78 = tempRet0;
   $79 = HEAP32[$8>>2]|0;
   $80 = ((($79)) + 40|0);
   $81 = $80;
   $82 = $81;
   HEAP32[$82>>2] = $77;
   $83 = (($81) + 4)|0;
   $84 = $83;
   HEAP32[$84>>2] = $78;
  }
  $85 = HEAP32[$8>>2]|0;
  $86 = ((($85)) + 40|0);
  $87 = $86;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  $93 = ($89|0)==(0);
  $94 = ($92|0)==(0);
  $95 = $93 & $94;
  $96 = $95&1;
  $97 = $96&255;
  $98 = HEAP32[$8>>2]|0;
  $99 = ((($98)) + 48|0);
  HEAP8[$99>>0] = $97;
  $100 = HEAP32[$pSelect>>2]|0;
  $101 = (_sqlite3_column_bytes($100,4)|0);
  HEAP32[$nRoot>>2] = $101;
  $102 = HEAP32[$pSelect>>2]|0;
  $103 = (_sqlite3_column_blob($102,4)|0);
  HEAP32[$aRoot>>2] = $103;
  $104 = HEAP32[$3>>2]|0;
  $105 = $iEnd;
  $106 = $105;
  $107 = HEAP32[$106>>2]|0;
  $108 = (($105) + 4)|0;
  $109 = $108;
  $110 = HEAP32[$109>>2]|0;
  $111 = (_fts3IsAppendable($104,$107,$110,$bAppendable)|0);
  HEAP32[$rc>>2] = $111;
  $112 = HEAP32[$rc>>2]|0;
  $113 = ($112|0)==(0);
  $114 = HEAP32[$bAppendable>>2]|0;
  $115 = ($114|0)!=(0);
  $or$cond = $113 & $115;
  if ($or$cond) {
   HEAP32[$aLeaf>>2] = 0;
   HEAP32[$nLeaf>>2] = 0;
   $117 = HEAP32[$3>>2]|0;
   $118 = $iLeafEnd;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   $124 = (_sqlite3Fts3ReadBlock($117,$120,$123,$aLeaf,$nLeaf,0)|0);
   HEAP32[$rc>>2] = $124;
   $125 = HEAP32[$rc>>2]|0;
   $126 = ($125|0)==(0);
   if ($126) {
    $127 = HEAP32[$aLeaf>>2]|0;
    $128 = HEAP32[$nLeaf>>2]|0;
    $129 = (_nodeReaderInit($reader,$127,$128)|0);
    HEAP32[$rc>>2] = $129;
    while(1) {
     $130 = HEAP32[$rc>>2]|0;
     $131 = ($130|0)==(0);
     $132 = HEAP32[$reader>>2]|0;
     $133 = ($132|0)!=(0|0);
     $134 = $131 ? $133 : 0;
     if (!($134)) {
      break;
     }
     $135 = (_nodeReaderNext($reader)|0);
     HEAP32[$rc>>2] = $135;
    }
    $136 = HEAP32[$6>>2]|0;
    $137 = HEAP32[$7>>2]|0;
    $138 = ((($reader)) + 24|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = ((($reader)) + 24|0);
    $141 = ((($140)) + 4|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = (_fts3TermCmp($136,$137,$139,$142)|0);
    $144 = ($143|0)<=(0);
    if ($144) {
     HEAP32[$bAppendable>>2] = 0;
    }
    _nodeReaderRelease($reader);
   }
   $145 = HEAP32[$aLeaf>>2]|0;
   _sqlite3_free($145);
  }
  $146 = HEAP32[$rc>>2]|0;
  $147 = ($146|0)==(0);
  $148 = HEAP32[$bAppendable>>2]|0;
  $149 = ($148|0)!=(0);
  $or$cond3 = $147 & $149;
  L23: do {
   if ($or$cond3) {
    $150 = HEAP32[$aRoot>>2]|0;
    $151 = HEAP8[$150>>0]|0;
    $152 = $151 << 24 >> 24;
    HEAP32[$nHeight>>2] = $152;
    $153 = $iEnd;
    $154 = $153;
    $155 = HEAP32[$154>>2]|0;
    $156 = (($153) + 4)|0;
    $157 = $156;
    $158 = HEAP32[$157>>2]|0;
    $159 = $iStart;
    $160 = $159;
    $161 = HEAP32[$160>>2]|0;
    $162 = (($159) + 4)|0;
    $163 = $162;
    $164 = HEAP32[$163>>2]|0;
    $165 = (_i64Subtract(($155|0),($158|0),($161|0),($164|0))|0);
    $166 = tempRet0;
    $167 = (_i64Add(($165|0),($166|0),1,0)|0);
    $168 = tempRet0;
    $169 = (($167|0) / 16)&-1;
    $170 = HEAP32[$8>>2]|0;
    HEAP32[$170>>2] = $169;
    $171 = $iStart;
    $172 = $171;
    $173 = HEAP32[$172>>2]|0;
    $174 = (($171) + 4)|0;
    $175 = $174;
    $176 = HEAP32[$175>>2]|0;
    $177 = HEAP32[$8>>2]|0;
    $178 = ((($177)) + 24|0);
    $179 = $178;
    $180 = $179;
    HEAP32[$180>>2] = $173;
    $181 = (($179) + 4)|0;
    $182 = $181;
    HEAP32[$182>>2] = $176;
    $183 = $iEnd;
    $184 = $183;
    $185 = HEAP32[$184>>2]|0;
    $186 = (($183) + 4)|0;
    $187 = $186;
    $188 = HEAP32[$187>>2]|0;
    $189 = HEAP32[$8>>2]|0;
    $190 = ((($189)) + 32|0);
    $191 = $190;
    $192 = $191;
    HEAP32[$192>>2] = $185;
    $193 = (($191) + 4)|0;
    $194 = $193;
    HEAP32[$194>>2] = $188;
    $195 = $4;
    $196 = $195;
    $197 = HEAP32[$196>>2]|0;
    $198 = (($195) + 4)|0;
    $199 = $198;
    $200 = HEAP32[$199>>2]|0;
    $201 = HEAP32[$8>>2]|0;
    $202 = ((($201)) + 8|0);
    $203 = $202;
    $204 = $203;
    HEAP32[$204>>2] = $197;
    $205 = (($203) + 4)|0;
    $206 = $205;
    HEAP32[$206>>2] = $200;
    $207 = HEAP32[$5>>2]|0;
    $208 = HEAP32[$8>>2]|0;
    $209 = ((($208)) + 16|0);
    HEAP32[$209>>2] = $207;
    $210 = HEAP32[$nHeight>>2]|0;
    $211 = (($210) + 1)|0;
    HEAP32[$i>>2] = $211;
    while(1) {
     $212 = HEAP32[$i>>2]|0;
     $213 = ($212|0)<(16);
     if (!($213)) {
      break;
     }
     $214 = HEAP32[$8>>2]|0;
     $215 = ((($214)) + 24|0);
     $216 = $215;
     $217 = $216;
     $218 = HEAP32[$217>>2]|0;
     $219 = (($216) + 4)|0;
     $220 = $219;
     $221 = HEAP32[$220>>2]|0;
     $222 = HEAP32[$i>>2]|0;
     $223 = HEAP32[$8>>2]|0;
     $224 = HEAP32[$223>>2]|0;
     $225 = Math_imul($222, $224)|0;
     $226 = ($225|0)<(0);
     $227 = $226 << 31 >> 31;
     $228 = (_i64Add(($218|0),($221|0),($225|0),($227|0))|0);
     $229 = tempRet0;
     $230 = HEAP32[$i>>2]|0;
     $231 = HEAP32[$8>>2]|0;
     $232 = ((($231)) + 56|0);
     $233 = (($232) + ($230<<5)|0);
     $234 = $233;
     $235 = $234;
     HEAP32[$235>>2] = $228;
     $236 = (($234) + 4)|0;
     $237 = $236;
     HEAP32[$237>>2] = $229;
     $238 = HEAP32[$i>>2]|0;
     $239 = (($238) + 1)|0;
     HEAP32[$i>>2] = $239;
    }
    $240 = HEAP32[$nHeight>>2]|0;
    $241 = HEAP32[$8>>2]|0;
    $242 = ((($241)) + 56|0);
    $243 = (($242) + ($240<<5)|0);
    HEAP32[$pNode>>2] = $243;
    $244 = HEAP32[$8>>2]|0;
    $245 = ((($244)) + 24|0);
    $246 = $245;
    $247 = $246;
    $248 = HEAP32[$247>>2]|0;
    $249 = (($246) + 4)|0;
    $250 = $249;
    $251 = HEAP32[$250>>2]|0;
    $252 = HEAP32[$8>>2]|0;
    $253 = HEAP32[$252>>2]|0;
    $254 = HEAP32[$nHeight>>2]|0;
    $255 = Math_imul($253, $254)|0;
    $256 = ($255|0)<(0);
    $257 = $256 << 31 >> 31;
    $258 = (_i64Add(($248|0),($251|0),($255|0),($257|0))|0);
    $259 = tempRet0;
    $260 = HEAP32[$pNode>>2]|0;
    $261 = $260;
    $262 = $261;
    HEAP32[$262>>2] = $258;
    $263 = (($261) + 4)|0;
    $264 = $263;
    HEAP32[$264>>2] = $259;
    $265 = HEAP32[$pNode>>2]|0;
    $266 = ((($265)) + 20|0);
    $267 = HEAP32[$nRoot>>2]|0;
    $268 = HEAP32[$3>>2]|0;
    $269 = ((($268)) + 224|0);
    $270 = HEAP32[$269>>2]|0;
    $271 = ($267|0)>($270|0);
    if ($271) {
     $272 = HEAP32[$nRoot>>2]|0;
     $276 = $272;
    } else {
     $273 = HEAP32[$3>>2]|0;
     $274 = ((($273)) + 224|0);
     $275 = HEAP32[$274>>2]|0;
     $276 = $275;
    }
    _blobGrowBuffer($266,$276,$rc);
    $277 = HEAP32[$rc>>2]|0;
    $278 = ($277|0)==(0);
    if ($278) {
     $279 = HEAP32[$pNode>>2]|0;
     $280 = ((($279)) + 20|0);
     $281 = HEAP32[$280>>2]|0;
     $282 = HEAP32[$aRoot>>2]|0;
     $283 = HEAP32[$nRoot>>2]|0;
     _memcpy(($281|0),($282|0),($283|0))|0;
     $284 = HEAP32[$nRoot>>2]|0;
     $285 = HEAP32[$pNode>>2]|0;
     $286 = ((($285)) + 20|0);
     $287 = ((($286)) + 4|0);
     HEAP32[$287>>2] = $284;
    }
    $288 = HEAP32[$nHeight>>2]|0;
    HEAP32[$i>>2] = $288;
    while(1) {
     $289 = HEAP32[$i>>2]|0;
     $290 = ($289|0)>=(0);
     $291 = HEAP32[$rc>>2]|0;
     $292 = ($291|0)==(0);
     $293 = $290 ? $292 : 0;
     if (!($293)) {
      break L23;
     }
     $294 = HEAP32[$i>>2]|0;
     $295 = HEAP32[$8>>2]|0;
     $296 = ((($295)) + 56|0);
     $297 = (($296) + ($294<<5)|0);
     HEAP32[$pNode>>2] = $297;
     $298 = HEAP32[$pNode>>2]|0;
     $299 = ((($298)) + 20|0);
     $300 = HEAP32[$299>>2]|0;
     $301 = HEAP32[$pNode>>2]|0;
     $302 = ((($301)) + 20|0);
     $303 = ((($302)) + 4|0);
     $304 = HEAP32[$303>>2]|0;
     $305 = (_nodeReaderInit($reader1,$300,$304)|0);
     HEAP32[$rc>>2] = $305;
     while(1) {
      $306 = HEAP32[$reader1>>2]|0;
      $307 = ($306|0)!=(0|0);
      $308 = HEAP32[$rc>>2]|0;
      $309 = ($308|0)==(0);
      $310 = $307 ? $309 : 0;
      if (!($310)) {
       break;
      }
      $311 = (_nodeReaderNext($reader1)|0);
      HEAP32[$rc>>2] = $311;
     }
     $312 = HEAP32[$pNode>>2]|0;
     $313 = ((($312)) + 8|0);
     $314 = ((($reader1)) + 24|0);
     $315 = ((($314)) + 4|0);
     $316 = HEAP32[$315>>2]|0;
     _blobGrowBuffer($313,$316,$rc);
     $317 = HEAP32[$rc>>2]|0;
     $318 = ($317|0)==(0);
     if ($318) {
      $319 = HEAP32[$pNode>>2]|0;
      $320 = ((($319)) + 8|0);
      $321 = HEAP32[$320>>2]|0;
      $322 = ((($reader1)) + 24|0);
      $323 = HEAP32[$322>>2]|0;
      $324 = ((($reader1)) + 24|0);
      $325 = ((($324)) + 4|0);
      $326 = HEAP32[$325>>2]|0;
      _memcpy(($321|0),($323|0),($326|0))|0;
      $327 = ((($reader1)) + 24|0);
      $328 = ((($327)) + 4|0);
      $329 = HEAP32[$328>>2]|0;
      $330 = HEAP32[$pNode>>2]|0;
      $331 = ((($330)) + 8|0);
      $332 = ((($331)) + 4|0);
      HEAP32[$332>>2] = $329;
      $333 = HEAP32[$i>>2]|0;
      $334 = ($333|0)>(0);
      if ($334) {
       HEAP32[$aBlock>>2] = 0;
       HEAP32[$nBlock>>2] = 0;
       $335 = HEAP32[$i>>2]|0;
       $336 = (($335) - 1)|0;
       $337 = HEAP32[$8>>2]|0;
       $338 = ((($337)) + 56|0);
       $339 = (($338) + ($336<<5)|0);
       HEAP32[$pNode>>2] = $339;
       $340 = ((($reader1)) + 16|0);
       $341 = $340;
       $342 = $341;
       $343 = HEAP32[$342>>2]|0;
       $344 = (($341) + 4)|0;
       $345 = $344;
       $346 = HEAP32[$345>>2]|0;
       $347 = HEAP32[$pNode>>2]|0;
       $348 = $347;
       $349 = $348;
       HEAP32[$349>>2] = $343;
       $350 = (($348) + 4)|0;
       $351 = $350;
       HEAP32[$351>>2] = $346;
       $352 = HEAP32[$3>>2]|0;
       $353 = ((($reader1)) + 16|0);
       $354 = $353;
       $355 = $354;
       $356 = HEAP32[$355>>2]|0;
       $357 = (($354) + 4)|0;
       $358 = $357;
       $359 = HEAP32[$358>>2]|0;
       $360 = (_sqlite3Fts3ReadBlock($352,$356,$359,$aBlock,$nBlock,0)|0);
       HEAP32[$rc>>2] = $360;
       $361 = HEAP32[$pNode>>2]|0;
       $362 = ((($361)) + 20|0);
       $363 = HEAP32[$nBlock>>2]|0;
       $364 = HEAP32[$3>>2]|0;
       $365 = ((($364)) + 224|0);
       $366 = HEAP32[$365>>2]|0;
       $367 = ($363|0)>($366|0);
       if ($367) {
        $368 = HEAP32[$nBlock>>2]|0;
        $372 = $368;
       } else {
        $369 = HEAP32[$3>>2]|0;
        $370 = ((($369)) + 224|0);
        $371 = HEAP32[$370>>2]|0;
        $372 = $371;
       }
       _blobGrowBuffer($362,$372,$rc);
       $373 = HEAP32[$rc>>2]|0;
       $374 = ($373|0)==(0);
       if ($374) {
        $375 = HEAP32[$pNode>>2]|0;
        $376 = ((($375)) + 20|0);
        $377 = HEAP32[$376>>2]|0;
        $378 = HEAP32[$aBlock>>2]|0;
        $379 = HEAP32[$nBlock>>2]|0;
        _memcpy(($377|0),($378|0),($379|0))|0;
        $380 = HEAP32[$nBlock>>2]|0;
        $381 = HEAP32[$pNode>>2]|0;
        $382 = ((($381)) + 20|0);
        $383 = ((($382)) + 4|0);
        HEAP32[$383>>2] = $380;
       }
       $384 = HEAP32[$aBlock>>2]|0;
       _sqlite3_free($384);
      }
     }
     _nodeReaderRelease($reader1);
     $385 = HEAP32[$i>>2]|0;
     $386 = (($385) + -1)|0;
     HEAP32[$i>>2] = $386;
    }
   }
  } while(0);
  $387 = HEAP32[$pSelect>>2]|0;
  $388 = (_sqlite3_reset($387)|0);
  HEAP32[$rc2>>2] = $388;
  $389 = HEAP32[$rc>>2]|0;
  $390 = ($389|0)==(0);
  if ($390) {
   $391 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $391;
  }
 }
 $392 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $392;
 $393 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($393|0);
}
function _fts3IncrmergeWriter($p,$0,$1,$iIdx,$pCsr,$pWriter) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $pCsr = $pCsr|0;
 $pWriter = $pWriter|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $nLeafEst = 0, $pFirstBlock = 0, $pLeafEst = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $rc = sp + 24|0;
 $i = sp + 20|0;
 $nLeafEst = sp + 16|0;
 $pLeafEst = sp + 12|0;
 $pFirstBlock = sp + 8|0;
 HEAP32[$3>>2] = $p;
 $8 = $4;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$5>>2] = $iIdx;
 HEAP32[$6>>2] = $pCsr;
 HEAP32[$7>>2] = $pWriter;
 HEAP32[$nLeafEst>>2] = 0;
 HEAP32[$pLeafEst>>2] = 0;
 HEAP32[$pFirstBlock>>2] = 0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (_fts3SqlStmt($12,29,$pLeafEst,0)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(0);
 if ($15) {
  $16 = HEAP32[$pLeafEst>>2]|0;
  $17 = $4;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($17) + 4)|0;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  (_sqlite3_bind_int64($16,1,$19,$22)|0);
  $23 = HEAP32[$pLeafEst>>2]|0;
  $24 = HEAP32[$6>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  (_sqlite3_bind_int64($23,2,$26,$28)|0);
  $29 = HEAP32[$pLeafEst>>2]|0;
  $30 = (_sqlite3_step($29)|0);
  $31 = (100)==($30|0);
  if ($31) {
   $32 = HEAP32[$pLeafEst>>2]|0;
   $33 = (_sqlite3_column_int($32,0)|0);
   HEAP32[$nLeafEst>>2] = $33;
  }
  $34 = HEAP32[$pLeafEst>>2]|0;
  $35 = (_sqlite3_reset($34)|0);
  HEAP32[$rc>>2] = $35;
 }
 $36 = HEAP32[$rc>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $38;
  $152 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $39 = HEAP32[$3>>2]|0;
 $40 = (_fts3SqlStmt($39,10,$pFirstBlock,0)|0);
 HEAP32[$rc>>2] = $40;
 $41 = HEAP32[$rc>>2]|0;
 $42 = ($41|0)==(0);
 if ($42) {
  $43 = HEAP32[$pFirstBlock>>2]|0;
  $44 = (_sqlite3_step($43)|0);
  $45 = (100)==($44|0);
  if ($45) {
   $46 = HEAP32[$pFirstBlock>>2]|0;
   $47 = (_sqlite3_column_int64($46,0)|0);
   $48 = tempRet0;
   $49 = HEAP32[$7>>2]|0;
   $50 = ((($49)) + 24|0);
   $51 = $50;
   $52 = $51;
   HEAP32[$52>>2] = $47;
   $53 = (($51) + 4)|0;
   $54 = $53;
   HEAP32[$54>>2] = $48;
   $55 = HEAP32[$7>>2]|0;
   $56 = ((($55)) + 24|0);
   $57 = $56;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (($57) + 4)|0;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = (_i64Subtract(($59|0),($62|0),1,0)|0);
   $64 = tempRet0;
   $65 = HEAP32[$7>>2]|0;
   $66 = ((($65)) + 32|0);
   $67 = $66;
   $68 = $67;
   HEAP32[$68>>2] = $63;
   $69 = (($67) + 4)|0;
   $70 = $69;
   HEAP32[$70>>2] = $64;
   $71 = HEAP32[$nLeafEst>>2]|0;
   $72 = $71<<4;
   $73 = ($72|0)<(0);
   $74 = $73 << 31 >> 31;
   $75 = HEAP32[$7>>2]|0;
   $76 = ((($75)) + 32|0);
   $77 = $76;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($77) + 4)|0;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (_i64Add(($79|0),($82|0),($72|0),($74|0))|0);
   $84 = tempRet0;
   $85 = $76;
   $86 = $85;
   HEAP32[$86>>2] = $83;
   $87 = (($85) + 4)|0;
   $88 = $87;
   HEAP32[$88>>2] = $84;
  }
  $89 = HEAP32[$pFirstBlock>>2]|0;
  $90 = (_sqlite3_reset($89)|0);
  HEAP32[$rc>>2] = $90;
 }
 $91 = HEAP32[$rc>>2]|0;
 $92 = ($91|0)!=(0);
 if ($92) {
  $93 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $93;
  $152 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $94 = HEAP32[$3>>2]|0;
 $95 = HEAP32[$7>>2]|0;
 $96 = ((($95)) + 32|0);
 $97 = $96;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (($97) + 4)|0;
 $101 = $100;
 $102 = HEAP32[$101>>2]|0;
 $103 = (_fts3WriteSegment($94,$99,$102,0,0)|0);
 HEAP32[$rc>>2] = $103;
 $104 = HEAP32[$rc>>2]|0;
 $105 = ($104|0)!=(0);
 if ($105) {
  $106 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $106;
  $152 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $107 = $4;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($107) + 4)|0;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = HEAP32[$7>>2]|0;
 $114 = ((($113)) + 8|0);
 $115 = $114;
 $116 = $115;
 HEAP32[$116>>2] = $109;
 $117 = (($115) + 4)|0;
 $118 = $117;
 HEAP32[$118>>2] = $112;
 $119 = HEAP32[$nLeafEst>>2]|0;
 $120 = HEAP32[$7>>2]|0;
 HEAP32[$120>>2] = $119;
 $121 = HEAP32[$5>>2]|0;
 $122 = HEAP32[$7>>2]|0;
 $123 = ((($122)) + 16|0);
 HEAP32[$123>>2] = $121;
 HEAP32[$i>>2] = 0;
 while(1) {
  $124 = HEAP32[$i>>2]|0;
  $125 = ($124|0)<(16);
  if (!($125)) {
   break;
  }
  $126 = HEAP32[$7>>2]|0;
  $127 = ((($126)) + 24|0);
  $128 = $127;
  $129 = $128;
  $130 = HEAP32[$129>>2]|0;
  $131 = (($128) + 4)|0;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$i>>2]|0;
  $135 = HEAP32[$7>>2]|0;
  $136 = HEAP32[$135>>2]|0;
  $137 = Math_imul($134, $136)|0;
  $138 = ($137|0)<(0);
  $139 = $138 << 31 >> 31;
  $140 = (_i64Add(($130|0),($133|0),($137|0),($139|0))|0);
  $141 = tempRet0;
  $142 = HEAP32[$i>>2]|0;
  $143 = HEAP32[$7>>2]|0;
  $144 = ((($143)) + 56|0);
  $145 = (($144) + ($142<<5)|0);
  $146 = $145;
  $147 = $146;
  HEAP32[$147>>2] = $140;
  $148 = (($146) + 4)|0;
  $149 = $148;
  HEAP32[$149>>2] = $141;
  $150 = HEAP32[$i>>2]|0;
  $151 = (($150) + 1)|0;
  HEAP32[$i>>2] = $151;
 }
 HEAP32[$2>>2] = 0;
 $152 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($152|0);
}
function _fts3IncrmergeAppend($p,$pWriter,$pCsr) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aDoclist = 0, $nDoclist = 0, $nPrefix = 0;
 var $nSpace = 0, $nSuffix = 0, $nTerm = 0, $pLeaf = 0, $rc = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $zTerm = sp + 32|0;
 $nTerm = sp + 28|0;
 $aDoclist = sp + 24|0;
 $nDoclist = sp + 20|0;
 $rc = sp + 16|0;
 $nSpace = sp + 12|0;
 $nPrefix = sp + 8|0;
 $nSuffix = sp + 4|0;
 $pLeaf = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pWriter;
 HEAP32[$2>>2] = $pCsr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$zTerm>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nTerm>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 48|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aDoclist>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$nDoclist>>2] = $14;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 56|0);
 HEAP32[$pLeaf>>2] = $16;
 $17 = HEAP32[$pLeaf>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$pLeaf>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$zTerm>>2]|0;
 $25 = HEAP32[$nTerm>>2]|0;
 $26 = (_fts3PrefixCompress($19,$23,$24,$25)|0);
 HEAP32[$nPrefix>>2] = $26;
 $27 = HEAP32[$nTerm>>2]|0;
 $28 = HEAP32[$nPrefix>>2]|0;
 $29 = (($27) - ($28))|0;
 HEAP32[$nSuffix>>2] = $29;
 $30 = HEAP32[$nPrefix>>2]|0;
 $31 = ($30|0)<(0);
 $32 = $31 << 31 >> 31;
 $33 = (_sqlite3Fts3VarintLen($30,$32)|0);
 HEAP32[$nSpace>>2] = $33;
 $34 = HEAP32[$nSuffix>>2]|0;
 $35 = ($34|0)<(0);
 $36 = $35 << 31 >> 31;
 $37 = (_sqlite3Fts3VarintLen($34,$36)|0);
 $38 = HEAP32[$nSuffix>>2]|0;
 $39 = (($37) + ($38))|0;
 $40 = HEAP32[$nSpace>>2]|0;
 $41 = (($40) + ($39))|0;
 HEAP32[$nSpace>>2] = $41;
 $42 = HEAP32[$nDoclist>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = (_sqlite3Fts3VarintLen($42,$44)|0);
 $46 = HEAP32[$nDoclist>>2]|0;
 $47 = (($45) + ($46))|0;
 $48 = HEAP32[$nSpace>>2]|0;
 $49 = (($48) + ($47))|0;
 HEAP32[$nSpace>>2] = $49;
 $50 = HEAP32[$pLeaf>>2]|0;
 $51 = ((($50)) + 20|0);
 $52 = ((($51)) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = ($53|0)>(0);
 if ($54) {
  $55 = HEAP32[$pLeaf>>2]|0;
  $56 = ((($55)) + 20|0);
  $57 = ((($56)) + 4|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$nSpace>>2]|0;
  $60 = (($58) + ($59))|0;
  $61 = HEAP32[$0>>2]|0;
  $62 = ((($61)) + 224|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($60|0)>($63|0);
  if ($64) {
   $65 = HEAP32[$0>>2]|0;
   $66 = HEAP32[$pLeaf>>2]|0;
   $67 = $66;
   $68 = $67;
   $69 = HEAP32[$68>>2]|0;
   $70 = (($67) + 4)|0;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$pLeaf>>2]|0;
   $74 = ((($73)) + 20|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$pLeaf>>2]|0;
   $77 = ((($76)) + 20|0);
   $78 = ((($77)) + 4|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (_fts3WriteSegment($65,$69,$72,$75,$79)|0);
   HEAP32[$rc>>2] = $80;
   $81 = HEAP32[$1>>2]|0;
   $82 = ((($81)) + 4|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = (($83) + 1)|0;
   HEAP32[$82>>2] = $84;
   $85 = HEAP32[$rc>>2]|0;
   $86 = ($85|0)==(0);
   if ($86) {
    $87 = HEAP32[$0>>2]|0;
    $88 = HEAP32[$1>>2]|0;
    $89 = HEAP32[$zTerm>>2]|0;
    $90 = HEAP32[$nPrefix>>2]|0;
    $91 = (($90) + 1)|0;
    $92 = (_fts3IncrmergePush($87,$88,$89,$91)|0);
    HEAP32[$rc>>2] = $92;
   }
   $93 = HEAP32[$pLeaf>>2]|0;
   $94 = $93;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = (_i64Add(($96|0),($99|0),1,0)|0);
   $101 = tempRet0;
   $102 = $93;
   $103 = $102;
   HEAP32[$103>>2] = $100;
   $104 = (($102) + 4)|0;
   $105 = $104;
   HEAP32[$105>>2] = $101;
   $106 = HEAP32[$pLeaf>>2]|0;
   $107 = ((($106)) + 8|0);
   $108 = ((($107)) + 4|0);
   HEAP32[$108>>2] = 0;
   $109 = HEAP32[$pLeaf>>2]|0;
   $110 = ((($109)) + 20|0);
   $111 = ((($110)) + 4|0);
   HEAP32[$111>>2] = 0;
   $112 = HEAP32[$nTerm>>2]|0;
   HEAP32[$nSuffix>>2] = $112;
   HEAP32[$nSpace>>2] = 1;
   $113 = HEAP32[$nSuffix>>2]|0;
   $114 = ($113|0)<(0);
   $115 = $114 << 31 >> 31;
   $116 = (_sqlite3Fts3VarintLen($113,$115)|0);
   $117 = HEAP32[$nSuffix>>2]|0;
   $118 = (($116) + ($117))|0;
   $119 = HEAP32[$nSpace>>2]|0;
   $120 = (($119) + ($118))|0;
   HEAP32[$nSpace>>2] = $120;
   $121 = HEAP32[$nDoclist>>2]|0;
   $122 = ($121|0)<(0);
   $123 = $122 << 31 >> 31;
   $124 = (_sqlite3Fts3VarintLen($121,$123)|0);
   $125 = HEAP32[$nDoclist>>2]|0;
   $126 = (($124) + ($125))|0;
   $127 = HEAP32[$nSpace>>2]|0;
   $128 = (($127) + ($126))|0;
   HEAP32[$nSpace>>2] = $128;
  }
 }
 $129 = HEAP32[$nSpace>>2]|0;
 $130 = ($129|0)<(0);
 $131 = $130 << 31 >> 31;
 $132 = HEAP32[$1>>2]|0;
 $133 = ((($132)) + 40|0);
 $134 = $133;
 $135 = $134;
 $136 = HEAP32[$135>>2]|0;
 $137 = (($134) + 4)|0;
 $138 = $137;
 $139 = HEAP32[$138>>2]|0;
 $140 = (_i64Add(($136|0),($139|0),($129|0),($131|0))|0);
 $141 = tempRet0;
 $142 = $133;
 $143 = $142;
 HEAP32[$143>>2] = $140;
 $144 = (($142) + 4)|0;
 $145 = $144;
 HEAP32[$145>>2] = $141;
 $146 = HEAP32[$pLeaf>>2]|0;
 $147 = ((($146)) + 20|0);
 $148 = HEAP32[$pLeaf>>2]|0;
 $149 = ((($148)) + 20|0);
 $150 = ((($149)) + 4|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = HEAP32[$nSpace>>2]|0;
 $153 = (($151) + ($152))|0;
 _blobGrowBuffer($147,$153,$rc);
 $154 = HEAP32[$rc>>2]|0;
 $155 = ($154|0)==(0);
 if (!($155)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $156 = HEAP32[$pLeaf>>2]|0;
 $157 = ((($156)) + 20|0);
 $158 = ((($157)) + 4|0);
 $159 = HEAP32[$158>>2]|0;
 $160 = ($159|0)==(0);
 if ($160) {
  $161 = HEAP32[$pLeaf>>2]|0;
  $162 = ((($161)) + 20|0);
  $163 = ((($162)) + 4|0);
  HEAP32[$163>>2] = 1;
  $164 = HEAP32[$pLeaf>>2]|0;
  $165 = ((($164)) + 20|0);
  $166 = HEAP32[$165>>2]|0;
  HEAP8[$166>>0] = 0;
 }
 $167 = HEAP32[$pLeaf>>2]|0;
 $168 = ((($167)) + 20|0);
 $169 = HEAP32[$pLeaf>>2]|0;
 $170 = ((($169)) + 8|0);
 $171 = HEAP32[$zTerm>>2]|0;
 $172 = HEAP32[$nTerm>>2]|0;
 $173 = HEAP32[$aDoclist>>2]|0;
 $174 = HEAP32[$nDoclist>>2]|0;
 $175 = (_fts3AppendToNode($168,$170,$171,$172,$173,$174)|0);
 HEAP32[$rc>>2] = $175;
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _fts3IncrmergeChomp($p,$0,$1,$pCsr,$pnRem) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pCsr = $pCsr|0;
 $pnRem = $pnRem|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $i = 0, $j = 0, $nRem = 0, $nTerm = 0, $pSeg = 0, $rc = 0, $zTerm = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $i = sp + 32|0;
 $nRem = sp + 28|0;
 $rc = sp + 24|0;
 $pSeg = sp + 20|0;
 $j = sp + 16|0;
 $zTerm = sp + 12|0;
 $nTerm = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $pCsr;
 HEAP32[$5>>2] = $pnRem;
 HEAP32[$nRem>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $10 = HEAP32[$4>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) - 1)|0;
 HEAP32[$i>>2] = $13;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = ($14|0)>=(0);
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)==(0);
  $18 = $15 ? $17 : 0;
  if (!($18)) {
   break;
  }
  HEAP32[$pSeg>>2] = 0;
  HEAP32[$j>>2] = 0;
  while(1) {
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)<($22|0);
   if (!($23)) {
    break;
   }
   $24 = HEAP32[$j>>2]|0;
   $25 = HEAP32[$4>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($24<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   HEAP32[$pSeg>>2] = $28;
   $29 = HEAP32[$pSeg>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$i>>2]|0;
   $32 = ($30|0)==($31|0);
   if ($32) {
    break;
   }
   $33 = HEAP32[$j>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$j>>2] = $34;
  }
  $35 = HEAP32[$pSeg>>2]|0;
  $36 = ((($35)) + 40|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)==(0|0);
  if ($38) {
   $39 = HEAP32[$2>>2]|0;
   $40 = HEAP32[$pSeg>>2]|0;
   $41 = (_fts3DeleteSegment($39,$40)|0);
   HEAP32[$rc>>2] = $41;
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)==(0);
   if ($43) {
    $44 = HEAP32[$2>>2]|0;
    $45 = $3;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($45) + 4)|0;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$pSeg>>2]|0;
    $52 = HEAP32[$51>>2]|0;
    $53 = (_fts3RemoveSegdirEntry($44,$47,$50,$52)|0);
    HEAP32[$rc>>2] = $53;
   }
   $54 = HEAP32[$5>>2]|0;
   HEAP32[$54>>2] = 0;
  } else {
   $55 = HEAP32[$pSeg>>2]|0;
   $56 = ((($55)) + 64|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$zTerm>>2] = $57;
   $58 = HEAP32[$pSeg>>2]|0;
   $59 = ((($58)) + 60|0);
   $60 = HEAP32[$59>>2]|0;
   HEAP32[$nTerm>>2] = $60;
   $61 = HEAP32[$2>>2]|0;
   $62 = $3;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = HEAP32[$pSeg>>2]|0;
   $69 = HEAP32[$68>>2]|0;
   $70 = HEAP32[$zTerm>>2]|0;
   $71 = HEAP32[$nTerm>>2]|0;
   $72 = (_fts3TruncateSegment($61,$64,$67,$69,$70,$71)|0);
   HEAP32[$rc>>2] = $72;
   $73 = HEAP32[$nRem>>2]|0;
   $74 = (($73) + 1)|0;
   HEAP32[$nRem>>2] = $74;
  }
  $75 = HEAP32[$i>>2]|0;
  $76 = (($75) + -1)|0;
  HEAP32[$i>>2] = $76;
 }
 $77 = HEAP32[$rc>>2]|0;
 $78 = ($77|0)==(0);
 if (!($78)) {
  $92 = HEAP32[$nRem>>2]|0;
  $93 = HEAP32[$5>>2]|0;
  HEAP32[$93>>2] = $92;
  $94 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($94|0);
 }
 $79 = HEAP32[$nRem>>2]|0;
 $80 = HEAP32[$4>>2]|0;
 $81 = ((($80)) + 4|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = ($79|0)!=($82|0);
 if (!($83)) {
  $92 = HEAP32[$nRem>>2]|0;
  $93 = HEAP32[$5>>2]|0;
  HEAP32[$93>>2] = $92;
  $94 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($94|0);
 }
 $84 = HEAP32[$2>>2]|0;
 $85 = $3;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = (_fts3RepackSegdirLevel($84,$87,$90)|0);
 HEAP32[$rc>>2] = $91;
 $92 = HEAP32[$nRem>>2]|0;
 $93 = HEAP32[$5>>2]|0;
 HEAP32[$93>>2] = $92;
 $94 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($94|0);
}
function _fts3IncrmergeHintPush($pHint,$0,$1,$nInput,$pRc) {
 $pHint = $pHint|0;
 $0 = $0|0;
 $1 = $1|0;
 $nInput = $nInput|0;
 $pRc = $pRc|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $pHint;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nInput;
 HEAP32[$5>>2] = $pRc;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 20)|0;
 $15 = HEAP32[$5>>2]|0;
 _blobGrowBuffer($10,$14,$15);
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($21)|0);
 $25 = $3;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3Fts3PutVarint($24,$27,$30)|0);
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31))|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$2>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + ($38)|0);
 $42 = HEAP32[$4>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = (_sqlite3Fts3PutVarint($41,$42,$44)|0);
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + ($45))|0;
 HEAP32[$47>>2] = $49;
 STACKTOP = sp;return;
}
function _fts3IncrmergeRelease($p,$pWriter,$pRc) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iRoot = 0, $or$cond = 0, $pBlock = 0, $pNode = 0, $pNode1 = 0, $pRoot = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $i = sp + 24|0;
 $iRoot = sp + 20|0;
 $pRoot = sp + 16|0;
 $rc = sp + 12|0;
 $pNode = sp + 8|0;
 $pBlock = sp + 4|0;
 $pNode1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pWriter;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$rc>>2] = $4;
 HEAP32[$iRoot>>2] = 15;
 while(1) {
  $5 = HEAP32[$iRoot>>2]|0;
  $6 = ($5|0)>=(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$iRoot>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 56|0);
  $10 = (($9) + ($7<<5)|0);
  HEAP32[$pNode>>2] = $10;
  $11 = HEAP32[$pNode>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)>(0);
  if ($15) {
   break;
  }
  $16 = HEAP32[$pNode>>2]|0;
  $17 = ((($16)) + 20|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3_free($18);
  $19 = HEAP32[$pNode>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3_free($21);
  $22 = HEAP32[$iRoot>>2]|0;
  $23 = (($22) + -1)|0;
  HEAP32[$iRoot>>2] = $23;
 }
 $24 = HEAP32[$iRoot>>2]|0;
 $25 = ($24|0)<(0);
 if ($25) {
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$iRoot>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 56|0);
  $30 = ((($29)) + 32|0);
  $31 = ((($30)) + 20|0);
  HEAP32[$pBlock>>2] = $31;
  $32 = HEAP32[$pBlock>>2]|0;
  _blobGrowBuffer($32,11,$rc);
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)==(0);
  if ($34) {
   $35 = HEAP32[$pBlock>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   HEAP8[$36>>0] = 1;
   $37 = HEAP32[$pBlock>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = ((($38)) + 1|0);
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 56|0);
   $42 = $41;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_sqlite3Fts3PutVarint($39,$44,$47)|0);
   $49 = (1 + ($48))|0;
   $50 = HEAP32[$pBlock>>2]|0;
   $51 = ((($50)) + 4|0);
   HEAP32[$51>>2] = $49;
  }
  HEAP32[$iRoot>>2] = 1;
 }
 $52 = HEAP32[$iRoot>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = ((($53)) + 56|0);
 $55 = (($54) + ($52<<5)|0);
 HEAP32[$pRoot>>2] = $55;
 HEAP32[$i>>2] = 0;
 while(1) {
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$iRoot>>2]|0;
  $58 = ($56|0)<($57|0);
  if (!($58)) {
   break;
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 56|0);
  $62 = (($61) + ($59<<5)|0);
  HEAP32[$pNode1>>2] = $62;
  $63 = HEAP32[$pNode1>>2]|0;
  $64 = ((($63)) + 20|0);
  $65 = ((($64)) + 4|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = ($66|0)>(0);
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)==(0);
  $or$cond = $67 & $69;
  if ($or$cond) {
   $70 = HEAP32[$0>>2]|0;
   $71 = HEAP32[$pNode1>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = HEAP32[$pNode1>>2]|0;
   $79 = ((($78)) + 20|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = HEAP32[$pNode1>>2]|0;
   $82 = ((($81)) + 20|0);
   $83 = ((($82)) + 4|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (_fts3WriteSegment($70,$74,$77,$80,$84)|0);
   HEAP32[$rc>>2] = $85;
  }
  $86 = HEAP32[$pNode1>>2]|0;
  $87 = ((($86)) + 20|0);
  $88 = HEAP32[$87>>2]|0;
  _sqlite3_free($88);
  $89 = HEAP32[$pNode1>>2]|0;
  $90 = ((($89)) + 8|0);
  $91 = HEAP32[$90>>2]|0;
  _sqlite3_free($91);
  $92 = HEAP32[$i>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$i>>2] = $93;
 }
 $94 = HEAP32[$rc>>2]|0;
 $95 = ($94|0)==(0);
 if ($95) {
  $96 = HEAP32[$0>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  $98 = ((($97)) + 8|0);
  $99 = $98;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (($99) + 4)|0;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (_i64Add(($101|0),($104|0),1,0)|0);
  $106 = tempRet0;
  $107 = HEAP32[$1>>2]|0;
  $108 = ((($107)) + 16|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = HEAP32[$1>>2]|0;
  $111 = ((($110)) + 24|0);
  $112 = $111;
  $113 = $112;
  $114 = HEAP32[$113>>2]|0;
  $115 = (($112) + 4)|0;
  $116 = $115;
  $117 = HEAP32[$116>>2]|0;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 56|0);
  $120 = $119;
  $121 = $120;
  $122 = HEAP32[$121>>2]|0;
  $123 = (($120) + 4)|0;
  $124 = $123;
  $125 = HEAP32[$124>>2]|0;
  $126 = HEAP32[$1>>2]|0;
  $127 = ((($126)) + 32|0);
  $128 = $127;
  $129 = $128;
  $130 = HEAP32[$129>>2]|0;
  $131 = (($128) + 4)|0;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$1>>2]|0;
  $135 = ((($134)) + 48|0);
  $136 = HEAP8[$135>>0]|0;
  $137 = $136&255;
  $138 = ($137|0)==(0);
  if ($138) {
   $139 = HEAP32[$1>>2]|0;
   $140 = ((($139)) + 40|0);
   $141 = $140;
   $142 = $141;
   $143 = HEAP32[$142>>2]|0;
   $144 = (($141) + 4)|0;
   $145 = $144;
   $146 = HEAP32[$145>>2]|0;
   $154 = $143;$155 = $146;
  } else {
   $154 = 0;$155 = 0;
  }
  $147 = HEAP32[$pRoot>>2]|0;
  $148 = ((($147)) + 20|0);
  $149 = HEAP32[$148>>2]|0;
  $150 = HEAP32[$pRoot>>2]|0;
  $151 = ((($150)) + 20|0);
  $152 = ((($151)) + 4|0);
  $153 = HEAP32[$152>>2]|0;
  $156 = (_fts3WriteSegdir($96,$105,$106,$109,$114,$117,$122,$125,$130,$133,$154,$155,$149,$153)|0);
  HEAP32[$rc>>2] = $156;
 }
 $157 = HEAP32[$pRoot>>2]|0;
 $158 = ((($157)) + 20|0);
 $159 = HEAP32[$158>>2]|0;
 _sqlite3_free($159);
 $160 = HEAP32[$pRoot>>2]|0;
 $161 = ((($160)) + 8|0);
 $162 = HEAP32[$161>>2]|0;
 _sqlite3_free($162);
 $163 = HEAP32[$rc>>2]|0;
 $164 = HEAP32[$2>>2]|0;
 HEAP32[$164>>2] = $163;
 STACKTOP = sp;return;
}
function _fts3IncrmergeHintStore($p,$pHint) {
 $p = $p|0;
 $pHint = $pHint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pReplace = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pReplace = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pHint;
 HEAP32[$pReplace>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,23,$pReplace,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $6 = HEAP32[$pReplace>>2]|0;
 (_sqlite3_bind_int($6,1,1)|0);
 $7 = HEAP32[$pReplace>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 (_sqlite3_bind_blob($7,2,$9,$12,0)|0);
 $13 = HEAP32[$pReplace>>2]|0;
 (_sqlite3_step($13)|0);
 $14 = HEAP32[$pReplace>>2]|0;
 $15 = (_sqlite3_reset($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _blobGrowBuffer($pBlob,$nMin,$pRc) {
 $pBlob = $pBlob|0;
 $nMin = $nMin|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $nAlloc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nAlloc = sp + 4|0;
 $a = sp;
 HEAP32[$0>>2] = $pBlob;
 HEAP32[$1>>2] = $nMin;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>($9|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$nAlloc>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$nAlloc>>2]|0;
 $15 = (_sqlite3_realloc($13,$14)|0);
 HEAP32[$a>>2] = $15;
 $16 = HEAP32[$a>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$nAlloc>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 8|0);
  HEAP32[$20>>2] = $18;
  $21 = HEAP32[$a>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  HEAP32[$22>>2] = $21;
  STACKTOP = sp;return;
 } else {
  $23 = HEAP32[$2>>2]|0;
  HEAP32[$23>>2] = 7;
  STACKTOP = sp;return;
 }
}
function _fts3IsAppendable($p,$0,$1,$pbRes) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pbRes = $pbRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bRes = 0, $pCheck = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp;
 $4 = sp + 20|0;
 $bRes = sp + 16|0;
 $pCheck = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pbRes;
 HEAP32[$bRes>>2] = 0;
 HEAP32[$pCheck>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,34,$pCheck,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $25 = HEAP32[$bRes>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  HEAP32[$26>>2] = $25;
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $13 = HEAP32[$pCheck>>2]|0;
 $14 = $3;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 (_sqlite3_bind_int64($13,1,$16,$19)|0);
 $20 = HEAP32[$pCheck>>2]|0;
 $21 = (_sqlite3_step($20)|0);
 $22 = (100)==($21|0);
 if ($22) {
  HEAP32[$bRes>>2] = 1;
 }
 $23 = HEAP32[$pCheck>>2]|0;
 $24 = (_sqlite3_reset($23)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$bRes>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _nodeReaderInit($p,$aNode,$nNode) {
 $p = $p|0;
 $aNode = $aNode|0;
 $nNode = $nNode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $aNode;
 HEAP32[$2>>2] = $nNode;
 $3 = HEAP32[$0>>2]|0;
 dest=$3; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 $13 = HEAP32[$0>>2]|0;
 if ($12) {
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 1|0);
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = (_sqlite3Fts3GetVarint($15,$17)|0);
  $19 = (1 + ($18))|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 8|0);
  HEAP32[$21>>2] = $19;
  $23 = HEAP32[$0>>2]|0;
  $24 = (_nodeReaderNext($23)|0);
  STACKTOP = sp;return ($24|0);
 } else {
  $22 = ((($13)) + 8|0);
  HEAP32[$22>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  $24 = (_nodeReaderNext($23)|0);
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _nodeReaderNext($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bFirst = 0, $nPrefix = 0, $nSuffix = 0;
 var $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $bFirst = sp + 12|0;
 $nPrefix = sp + 8|0;
 $nSuffix = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 $6 = $5&1;
 HEAP32[$bFirst>>2] = $6;
 HEAP32[$nPrefix>>2] = 0;
 HEAP32[$nSuffix>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)!=(0);
 $16 = ($14|0)!=(0);
 $17 = $15 | $16;
 $18 = HEAP32[$bFirst>>2]|0;
 $19 = ($18|0)==(0);
 $or$cond = $17 & $19;
 if ($or$cond) {
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = $21;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (_i64Add(($24|0),($27|0),1,0)|0);
  $29 = tempRet0;
  $30 = $21;
  $31 = $30;
  HEAP32[$31>>2] = $28;
  $32 = (($30) + 4)|0;
  $33 = $32;
  HEAP32[$33>>2] = $29;
 }
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = ($36|0)>=($39|0);
 if ($40) {
  $41 = HEAP32[$0>>2]|0;
  HEAP32[$41>>2] = 0;
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $42 = HEAP32[$bFirst>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($46)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 & 128;
  $53 = ($52|0)!=(0);
  $54 = HEAP32[$0>>2]|0;
  $55 = ((($54)) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$0>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($56)|0);
  if ($53) {
   $60 = (_sqlite3Fts3GetVarint32($59,$nPrefix)|0);
   $67 = $60;
  } else {
   $61 = HEAP8[$59>>0]|0;
   $62 = $61&255;
   HEAP32[$nPrefix>>2] = $62;
   $67 = 1;
  }
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 8|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + ($67))|0;
  HEAP32[$64>>2] = $66;
 }
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 8|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$0>>2]|0;
 $72 = HEAP32[$71>>2]|0;
 $73 = (($72) + ($70)|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 $76 = $75 & 128;
 $77 = ($76|0)!=(0);
 $78 = HEAP32[$0>>2]|0;
 $79 = ((($78)) + 8|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = HEAP32[$0>>2]|0;
 $82 = HEAP32[$81>>2]|0;
 $83 = (($82) + ($80)|0);
 if ($77) {
  $84 = (_sqlite3Fts3GetVarint32($83,$nSuffix)|0);
  $91 = $84;
 } else {
  $85 = HEAP8[$83>>0]|0;
  $86 = $85&255;
  HEAP32[$nSuffix>>2] = $86;
  $91 = 1;
 }
 $87 = HEAP32[$0>>2]|0;
 $88 = ((($87)) + 8|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + ($91))|0;
 HEAP32[$88>>2] = $90;
 $92 = HEAP32[$0>>2]|0;
 $93 = ((($92)) + 24|0);
 $94 = HEAP32[$nPrefix>>2]|0;
 $95 = HEAP32[$nSuffix>>2]|0;
 $96 = (($94) + ($95))|0;
 _blobGrowBuffer($93,$96,$rc);
 $97 = HEAP32[$rc>>2]|0;
 $98 = ($97|0)==(0);
 if (!($98)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $99 = HEAP32[$nPrefix>>2]|0;
 $100 = HEAP32[$0>>2]|0;
 $101 = ((($100)) + 24|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) + ($99)|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = ((($104)) + 8|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = HEAP32[$0>>2]|0;
 $108 = HEAP32[$107>>2]|0;
 $109 = (($108) + ($106)|0);
 $110 = HEAP32[$nSuffix>>2]|0;
 _memcpy(($103|0),($109|0),($110|0))|0;
 $111 = HEAP32[$nPrefix>>2]|0;
 $112 = HEAP32[$nSuffix>>2]|0;
 $113 = (($111) + ($112))|0;
 $114 = HEAP32[$0>>2]|0;
 $115 = ((($114)) + 24|0);
 $116 = ((($115)) + 4|0);
 HEAP32[$116>>2] = $113;
 $117 = HEAP32[$nSuffix>>2]|0;
 $118 = HEAP32[$0>>2]|0;
 $119 = ((($118)) + 8|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = (($120) + ($117))|0;
 HEAP32[$119>>2] = $121;
 $122 = HEAP32[$0>>2]|0;
 $123 = ((($122)) + 16|0);
 $124 = $123;
 $125 = $124;
 $126 = HEAP32[$125>>2]|0;
 $127 = (($124) + 4)|0;
 $128 = $127;
 $129 = HEAP32[$128>>2]|0;
 $130 = ($126|0)==(0);
 $131 = ($129|0)==(0);
 $132 = $130 & $131;
 if (!($132)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $133 = HEAP32[$0>>2]|0;
 $134 = ((($133)) + 8|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = HEAP32[$0>>2]|0;
 $137 = HEAP32[$136>>2]|0;
 $138 = (($137) + ($135)|0);
 $139 = HEAP8[$138>>0]|0;
 $140 = $139&255;
 $141 = $140 & 128;
 $142 = ($141|0)!=(0);
 $143 = HEAP32[$0>>2]|0;
 $144 = ((($143)) + 8|0);
 $145 = HEAP32[$144>>2]|0;
 $146 = HEAP32[$0>>2]|0;
 $147 = HEAP32[$146>>2]|0;
 $148 = (($147) + ($145)|0);
 if ($142) {
  $149 = HEAP32[$0>>2]|0;
  $150 = ((($149)) + 40|0);
  $151 = (_sqlite3Fts3GetVarint32($148,$150)|0);
  $160 = $151;
 } else {
  $152 = HEAP8[$148>>0]|0;
  $153 = $152&255;
  $154 = HEAP32[$0>>2]|0;
  $155 = ((($154)) + 40|0);
  HEAP32[$155>>2] = $153;
  $160 = 1;
 }
 $156 = HEAP32[$0>>2]|0;
 $157 = ((($156)) + 8|0);
 $158 = HEAP32[$157>>2]|0;
 $159 = (($158) + ($160))|0;
 HEAP32[$157>>2] = $159;
 $161 = HEAP32[$0>>2]|0;
 $162 = ((($161)) + 8|0);
 $163 = HEAP32[$162>>2]|0;
 $164 = HEAP32[$0>>2]|0;
 $165 = HEAP32[$164>>2]|0;
 $166 = (($165) + ($163)|0);
 $167 = HEAP32[$0>>2]|0;
 $168 = ((($167)) + 36|0);
 HEAP32[$168>>2] = $166;
 $169 = HEAP32[$0>>2]|0;
 $170 = ((($169)) + 40|0);
 $171 = HEAP32[$170>>2]|0;
 $172 = HEAP32[$0>>2]|0;
 $173 = ((($172)) + 8|0);
 $174 = HEAP32[$173>>2]|0;
 $175 = (($174) + ($171))|0;
 HEAP32[$173>>2] = $175;
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _fts3TermCmp($zLhs,$nLhs,$zRhs,$nRhs) {
 $zLhs = $zLhs|0;
 $nLhs = $nLhs|0;
 $zRhs = $zRhs|0;
 $nRhs = $nRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $nCmp = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $nCmp = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $zLhs;
 HEAP32[$1>>2] = $nLhs;
 HEAP32[$2>>2] = $zRhs;
 HEAP32[$3>>2] = $nRhs;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($4|0)<($5|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = $6 ? $7 : $8;
 HEAP32[$nCmp>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$nCmp>>2]|0;
 $13 = (_memcmp($10,$11,$12)|0);
 HEAP32[$res>>2] = $13;
 $14 = HEAP32[$res>>2]|0;
 $15 = ($14|0)==(0);
 if (!($15)) {
  $19 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($16) - ($17))|0;
 HEAP32[$res>>2] = $18;
 $19 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _nodeReaderRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 STACKTOP = sp;return;
}
function _fts3IncrmergePush($p,$pWriter,$zTerm,$nTerm) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iLayer = 0, $iNextPtr = 0, $iPtr = 0, $nPrefix = 0, $nSpace = 0, $nSuffix = 0;
 var $or$cond = 0, $pBlk = 0, $pNode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $iPtr = sp + 8|0;
 $iLayer = sp + 40|0;
 $iNextPtr = sp;
 $pNode = sp + 36|0;
 $rc = sp + 32|0;
 $nPrefix = sp + 28|0;
 $nSuffix = sp + 24|0;
 $nSpace = sp + 20|0;
 $pBlk = sp + 16|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pWriter;
 HEAP32[$3>>2] = $zTerm;
 HEAP32[$4>>2] = $nTerm;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $iPtr;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 HEAP32[$iLayer>>2] = 1;
 while(1) {
  $17 = HEAP32[$iLayer>>2]|0;
  $18 = ($17|0)<(16);
  if (!($18)) {
   label = 16;
   break;
  }
  $19 = $iNextPtr;
  $20 = $19;
  HEAP32[$20>>2] = 0;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$iLayer>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ((($24)) + 56|0);
  $26 = (($25) + ($23<<5)|0);
  HEAP32[$pNode>>2] = $26;
  HEAP32[$rc>>2] = 0;
  $27 = HEAP32[$pNode>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pNode>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = ((($31)) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  $36 = (_fts3PrefixCompress($29,$33,$34,$35)|0);
  HEAP32[$nPrefix>>2] = $36;
  $37 = HEAP32[$4>>2]|0;
  $38 = HEAP32[$nPrefix>>2]|0;
  $39 = (($37) - ($38))|0;
  HEAP32[$nSuffix>>2] = $39;
  $40 = HEAP32[$nPrefix>>2]|0;
  $41 = ($40|0)<(0);
  $42 = $41 << 31 >> 31;
  $43 = (_sqlite3Fts3VarintLen($40,$42)|0);
  HEAP32[$nSpace>>2] = $43;
  $44 = HEAP32[$nSuffix>>2]|0;
  $45 = ($44|0)<(0);
  $46 = $45 << 31 >> 31;
  $47 = (_sqlite3Fts3VarintLen($44,$46)|0);
  $48 = HEAP32[$nSuffix>>2]|0;
  $49 = (($47) + ($48))|0;
  $50 = HEAP32[$nSpace>>2]|0;
  $51 = (($50) + ($49))|0;
  HEAP32[$nSpace>>2] = $51;
  $52 = HEAP32[$pNode>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = ((($53)) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)==(0);
  if ($56) {
   label = 5;
  } else {
   $57 = HEAP32[$pNode>>2]|0;
   $58 = ((($57)) + 20|0);
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$nSpace>>2]|0;
   $62 = (($60) + ($61))|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 224|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($62|0)<=($65|0);
   if ($66) {
    label = 5;
   } else {
    $164 = HEAP32[$1>>2]|0;
    $165 = HEAP32[$pNode>>2]|0;
    $166 = $165;
    $167 = $166;
    $168 = HEAP32[$167>>2]|0;
    $169 = (($166) + 4)|0;
    $170 = $169;
    $171 = HEAP32[$170>>2]|0;
    $172 = HEAP32[$pNode>>2]|0;
    $173 = ((($172)) + 20|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = HEAP32[$pNode>>2]|0;
    $176 = ((($175)) + 20|0);
    $177 = ((($176)) + 4|0);
    $178 = HEAP32[$177>>2]|0;
    $179 = (_fts3WriteSegment($164,$168,$171,$174,$178)|0);
    HEAP32[$rc>>2] = $179;
    $180 = HEAP32[$iLayer>>2]|0;
    $181 = $180&255;
    $182 = HEAP32[$pNode>>2]|0;
    $183 = ((($182)) + 20|0);
    $184 = HEAP32[$183>>2]|0;
    HEAP8[$184>>0] = $181;
    $185 = HEAP32[$pNode>>2]|0;
    $186 = ((($185)) + 20|0);
    $187 = HEAP32[$186>>2]|0;
    $188 = ((($187)) + 1|0);
    $189 = $iPtr;
    $190 = $189;
    $191 = HEAP32[$190>>2]|0;
    $192 = (($189) + 4)|0;
    $193 = $192;
    $194 = HEAP32[$193>>2]|0;
    $195 = (_i64Add(($191|0),($194|0),1,0)|0);
    $196 = tempRet0;
    $197 = (_sqlite3Fts3PutVarint($188,$195,$196)|0);
    $198 = (1 + ($197))|0;
    $199 = HEAP32[$pNode>>2]|0;
    $200 = ((($199)) + 20|0);
    $201 = ((($200)) + 4|0);
    HEAP32[$201>>2] = $198;
    $202 = HEAP32[$pNode>>2]|0;
    $203 = $202;
    $204 = $203;
    $205 = HEAP32[$204>>2]|0;
    $206 = (($203) + 4)|0;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = $iNextPtr;
    $210 = $209;
    HEAP32[$210>>2] = $205;
    $211 = (($209) + 4)|0;
    $212 = $211;
    HEAP32[$212>>2] = $208;
    $213 = HEAP32[$pNode>>2]|0;
    $214 = $213;
    $215 = $214;
    $216 = HEAP32[$215>>2]|0;
    $217 = (($214) + 4)|0;
    $218 = $217;
    $219 = HEAP32[$218>>2]|0;
    $220 = (_i64Add(($216|0),($219|0),1,0)|0);
    $221 = tempRet0;
    $222 = $213;
    $223 = $222;
    HEAP32[$223>>2] = $220;
    $224 = (($222) + 4)|0;
    $225 = $224;
    HEAP32[$225>>2] = $221;
    $226 = HEAP32[$pNode>>2]|0;
    $227 = ((($226)) + 8|0);
    $228 = ((($227)) + 4|0);
    HEAP32[$228>>2] = 0;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $67 = HEAP32[$pNode>>2]|0;
   $68 = ((($67)) + 20|0);
   HEAP32[$pBlk>>2] = $68;
   $69 = HEAP32[$pBlk>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)==(0);
   if ($72) {
    $73 = HEAP32[$pBlk>>2]|0;
    $74 = HEAP32[$1>>2]|0;
    $75 = ((($74)) + 224|0);
    $76 = HEAP32[$75>>2]|0;
    _blobGrowBuffer($73,$76,$rc);
    $77 = HEAP32[$rc>>2]|0;
    $78 = ($77|0)==(0);
    if ($78) {
     $79 = HEAP32[$iLayer>>2]|0;
     $80 = $79&255;
     $81 = HEAP32[$pBlk>>2]|0;
     $82 = HEAP32[$81>>2]|0;
     HEAP8[$82>>0] = $80;
     $83 = HEAP32[$pBlk>>2]|0;
     $84 = HEAP32[$83>>2]|0;
     $85 = ((($84)) + 1|0);
     $86 = $iPtr;
     $87 = $86;
     $88 = HEAP32[$87>>2]|0;
     $89 = (($86) + 4)|0;
     $90 = $89;
     $91 = HEAP32[$90>>2]|0;
     $92 = (_sqlite3Fts3PutVarint($85,$88,$91)|0);
     $93 = (1 + ($92))|0;
     $94 = HEAP32[$pBlk>>2]|0;
     $95 = ((($94)) + 4|0);
     HEAP32[$95>>2] = $93;
    }
   }
   $96 = HEAP32[$pBlk>>2]|0;
   $97 = HEAP32[$pBlk>>2]|0;
   $98 = ((($97)) + 4|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$nSpace>>2]|0;
   $101 = (($99) + ($100))|0;
   _blobGrowBuffer($96,$101,$rc);
   $102 = HEAP32[$pNode>>2]|0;
   $103 = ((($102)) + 8|0);
   $104 = HEAP32[$4>>2]|0;
   _blobGrowBuffer($103,$104,$rc);
   $105 = HEAP32[$rc>>2]|0;
   $106 = ($105|0)==(0);
   if ($106) {
    $107 = HEAP32[$pNode>>2]|0;
    $108 = ((($107)) + 8|0);
    $109 = ((($108)) + 4|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = ($110|0)!=(0);
    if ($111) {
     $112 = HEAP32[$pBlk>>2]|0;
     $113 = ((($112)) + 4|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = HEAP32[$pBlk>>2]|0;
     $116 = HEAP32[$115>>2]|0;
     $117 = (($116) + ($114)|0);
     $118 = HEAP32[$nPrefix>>2]|0;
     $119 = ($118|0)<(0);
     $120 = $119 << 31 >> 31;
     $121 = (_sqlite3Fts3PutVarint($117,$118,$120)|0);
     $122 = HEAP32[$pBlk>>2]|0;
     $123 = ((($122)) + 4|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = (($124) + ($121))|0;
     HEAP32[$123>>2] = $125;
    }
    $126 = HEAP32[$pBlk>>2]|0;
    $127 = ((($126)) + 4|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$pBlk>>2]|0;
    $130 = HEAP32[$129>>2]|0;
    $131 = (($130) + ($128)|0);
    $132 = HEAP32[$nSuffix>>2]|0;
    $133 = ($132|0)<(0);
    $134 = $133 << 31 >> 31;
    $135 = (_sqlite3Fts3PutVarint($131,$132,$134)|0);
    $136 = HEAP32[$pBlk>>2]|0;
    $137 = ((($136)) + 4|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = (($138) + ($135))|0;
    HEAP32[$137>>2] = $139;
    $140 = HEAP32[$pBlk>>2]|0;
    $141 = ((($140)) + 4|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = HEAP32[$pBlk>>2]|0;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($144) + ($142)|0);
    $146 = HEAP32[$nPrefix>>2]|0;
    $147 = HEAP32[$3>>2]|0;
    $148 = (($147) + ($146)|0);
    $149 = HEAP32[$nSuffix>>2]|0;
    _memcpy(($145|0),($148|0),($149|0))|0;
    $150 = HEAP32[$nSuffix>>2]|0;
    $151 = HEAP32[$pBlk>>2]|0;
    $152 = ((($151)) + 4|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = (($153) + ($150))|0;
    HEAP32[$152>>2] = $154;
    $155 = HEAP32[$pNode>>2]|0;
    $156 = ((($155)) + 8|0);
    $157 = HEAP32[$156>>2]|0;
    $158 = HEAP32[$3>>2]|0;
    $159 = HEAP32[$4>>2]|0;
    _memcpy(($157|0),($158|0),($159|0))|0;
    $160 = HEAP32[$4>>2]|0;
    $161 = HEAP32[$pNode>>2]|0;
    $162 = ((($161)) + 8|0);
    $163 = ((($162)) + 4|0);
    HEAP32[$163>>2] = $160;
   }
  }
  $229 = HEAP32[$rc>>2]|0;
  $230 = ($229|0)!=(0);
  $231 = $iNextPtr;
  $232 = $231;
  $233 = HEAP32[$232>>2]|0;
  $234 = (($231) + 4)|0;
  $235 = $234;
  $236 = HEAP32[$235>>2]|0;
  $237 = ($233|0)==(0);
  $238 = ($236|0)==(0);
  $239 = $237 & $238;
  $or$cond = $230 | $239;
  if ($or$cond) {
   label = 14;
   break;
  }
  $241 = $iNextPtr;
  $242 = $241;
  $243 = HEAP32[$242>>2]|0;
  $244 = (($241) + 4)|0;
  $245 = $244;
  $246 = HEAP32[$245>>2]|0;
  $247 = $iPtr;
  $248 = $247;
  HEAP32[$248>>2] = $243;
  $249 = (($247) + 4)|0;
  $250 = $249;
  HEAP32[$250>>2] = $246;
  $251 = HEAP32[$iLayer>>2]|0;
  $252 = (($251) + 1)|0;
  HEAP32[$iLayer>>2] = $252;
 }
 if ((label|0) == 14) {
  $240 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $240;
  $253 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($253|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $253 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($253|0);
 }
 return (0)|0;
}
function _fts3AppendToNode($pNode,$pPrev,$zTerm,$nTerm,$aDoclist,$nDoclist) {
 $pNode = $pNode|0;
 $pPrev = $pPrev|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bFirst = 0, $nPrefix = 0, $nSuffix = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $rc = sp + 12|0;
 $bFirst = sp + 8|0;
 $nPrefix = sp + 4|0;
 $nSuffix = sp;
 HEAP32[$1>>2] = $pNode;
 HEAP32[$2>>2] = $pPrev;
 HEAP32[$3>>2] = $zTerm;
 HEAP32[$4>>2] = $nTerm;
 HEAP32[$5>>2] = $aDoclist;
 HEAP32[$6>>2] = $nDoclist;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$bFirst>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 _blobGrowBuffer($12,$13,$rc);
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = HEAP32[$4>>2]|0;
 $24 = (_fts3PrefixCompress($18,$21,$22,$23)|0);
 HEAP32[$nPrefix>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = HEAP32[$nPrefix>>2]|0;
 $27 = (($25) - ($26))|0;
 HEAP32[$nSuffix>>2] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP32[$4>>2]|0;
 _memcpy(($29|0),($30|0),($31|0))|0;
 $32 = HEAP32[$4>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 4|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$bFirst>>2]|0;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($39)|0);
  $43 = HEAP32[$nPrefix>>2]|0;
  $44 = ($43|0)<(0);
  $45 = $44 << 31 >> 31;
  $46 = (_sqlite3Fts3PutVarint($42,$43,$45)|0);
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 4|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($46))|0;
  HEAP32[$48>>2] = $50;
 }
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$1>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + ($53)|0);
 $57 = HEAP32[$nSuffix>>2]|0;
 $58 = ($57|0)<(0);
 $59 = $58 << 31 >> 31;
 $60 = (_sqlite3Fts3PutVarint($56,$57,$59)|0);
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 4|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = (($63) + ($60))|0;
 HEAP32[$62>>2] = $64;
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 4|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69) + ($67)|0);
 $71 = HEAP32[$nPrefix>>2]|0;
 $72 = HEAP32[$3>>2]|0;
 $73 = (($72) + ($71)|0);
 $74 = HEAP32[$nSuffix>>2]|0;
 _memcpy(($70|0),($73|0),($74|0))|0;
 $75 = HEAP32[$nSuffix>>2]|0;
 $76 = HEAP32[$1>>2]|0;
 $77 = ((($76)) + 4|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = (($78) + ($75))|0;
 HEAP32[$77>>2] = $79;
 $80 = HEAP32[$5>>2]|0;
 $81 = ($80|0)!=(0|0);
 if ($81) {
  $82 = HEAP32[$1>>2]|0;
  $83 = ((($82)) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + ($84)|0);
  $88 = HEAP32[$6>>2]|0;
  $89 = ($88|0)<(0);
  $90 = $89 << 31 >> 31;
  $91 = (_sqlite3Fts3PutVarint($87,$88,$90)|0);
  $92 = HEAP32[$1>>2]|0;
  $93 = ((($92)) + 4|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91))|0;
  HEAP32[$93>>2] = $95;
  $96 = HEAP32[$1>>2]|0;
  $97 = ((($96)) + 4|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = HEAP32[$1>>2]|0;
  $100 = HEAP32[$99>>2]|0;
  $101 = (($100) + ($98)|0);
  $102 = HEAP32[$5>>2]|0;
  $103 = HEAP32[$6>>2]|0;
  _memcpy(($101|0),($102|0),($103|0))|0;
  $104 = HEAP32[$6>>2]|0;
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 4|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($107) + ($104))|0;
  HEAP32[$106>>2] = $108;
 }
 HEAP32[$0>>2] = 0;
 $109 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($109|0);
}
function _fts3RemoveSegdirEntry($p,$0,$1,$iIdx) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDelete = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $pDelete = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $iIdx;
 HEAP32[$pDelete>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,30,$pDelete,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $25 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $13 = HEAP32[$pDelete>>2]|0;
 $14 = $3;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 (_sqlite3_bind_int64($13,1,$16,$19)|0);
 $20 = HEAP32[$pDelete>>2]|0;
 $21 = HEAP32[$4>>2]|0;
 (_sqlite3_bind_int($20,2,$21)|0);
 $22 = HEAP32[$pDelete>>2]|0;
 (_sqlite3_step($22)|0);
 $23 = HEAP32[$pDelete>>2]|0;
 $24 = (_sqlite3_reset($23)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _fts3TruncateSegment($p,$0,$1,$iIdx,$zTerm,$nTerm) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBlock = 0, $aRoot = 0, $block = 0, $iBlock = 0, $iNewStart = 0;
 var $iOldStart = 0, $nBlock = 0, $nRoot = 0, $or$cond = 0, $pChomp = 0, $pDel = 0, $pFetch = 0, $rc = 0, $rc2 = 0, $root = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $2 = sp + 104|0;
 $3 = sp + 24|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $6 = sp + 92|0;
 $rc = sp + 88|0;
 $root = sp + 76|0;
 $block = sp + 64|0;
 $iBlock = sp + 16|0;
 $iNewStart = sp + 8|0;
 $iOldStart = sp;
 $pFetch = sp + 60|0;
 $rc2 = sp + 56|0;
 $aRoot = sp + 52|0;
 $nRoot = sp + 48|0;
 $aBlock = sp + 44|0;
 $nBlock = sp + 40|0;
 $pDel = sp + 36|0;
 $pChomp = sp + 32|0;
 HEAP32[$2>>2] = $p;
 $7 = $3;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$4>>2] = $iIdx;
 HEAP32[$5>>2] = $zTerm;
 HEAP32[$6>>2] = $nTerm;
 HEAP32[$rc>>2] = 0;
 ;HEAP32[$root>>2]=0|0;HEAP32[$root+4>>2]=0|0;HEAP32[$root+8>>2]=0|0;
 ;HEAP32[$block>>2]=0|0;HEAP32[$block+4>>2]=0|0;HEAP32[$block+8>>2]=0|0;
 $11 = $iBlock;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = $iNewStart;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = $iOldStart;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 HEAP32[$pFetch>>2] = 0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (_fts3SqlStmt($23,32,$pFetch,0)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)==(0);
 if ($26) {
  $27 = HEAP32[$pFetch>>2]|0;
  $28 = $3;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($28) + 4)|0;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  (_sqlite3_bind_int64($27,1,$30,$33)|0);
  $34 = HEAP32[$pFetch>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  (_sqlite3_bind_int($34,2,$35)|0);
  $36 = HEAP32[$pFetch>>2]|0;
  $37 = (_sqlite3_step($36)|0);
  $38 = (100)==($37|0);
  if ($38) {
   $39 = HEAP32[$pFetch>>2]|0;
   $40 = (_sqlite3_column_blob($39,4)|0);
   HEAP32[$aRoot>>2] = $40;
   $41 = HEAP32[$pFetch>>2]|0;
   $42 = (_sqlite3_column_bytes($41,4)|0);
   HEAP32[$nRoot>>2] = $42;
   $43 = HEAP32[$pFetch>>2]|0;
   $44 = (_sqlite3_column_int64($43,1)|0);
   $45 = tempRet0;
   $46 = $iOldStart;
   $47 = $46;
   HEAP32[$47>>2] = $44;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = HEAP32[$aRoot>>2]|0;
   $51 = HEAP32[$nRoot>>2]|0;
   $52 = HEAP32[$5>>2]|0;
   $53 = HEAP32[$6>>2]|0;
   $54 = (_fts3TruncateNode($50,$51,$root,$52,$53,$iBlock)|0);
   HEAP32[$rc>>2] = $54;
  }
  $55 = HEAP32[$pFetch>>2]|0;
  $56 = (_sqlite3_reset($55)|0);
  HEAP32[$rc2>>2] = $56;
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  if ($58) {
   $59 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $59;
  }
 }
 while(1) {
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(0);
  $62 = $iBlock;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = ($64|0)!=(0);
  $69 = ($67|0)!=(0);
  $70 = $68 | $69;
  $71 = $61 ? $70 : 0;
  if (!($71)) {
   break;
  }
  HEAP32[$aBlock>>2] = 0;
  HEAP32[$nBlock>>2] = 0;
  $72 = $iBlock;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = $iNewStart;
  $79 = $78;
  HEAP32[$79>>2] = $74;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = HEAP32[$2>>2]|0;
  $83 = $iBlock;
  $84 = $83;
  $85 = HEAP32[$84>>2]|0;
  $86 = (($83) + 4)|0;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (_sqlite3Fts3ReadBlock($82,$85,$88,$aBlock,$nBlock,0)|0);
  HEAP32[$rc>>2] = $89;
  $90 = HEAP32[$rc>>2]|0;
  $91 = ($90|0)==(0);
  if ($91) {
   $92 = HEAP32[$aBlock>>2]|0;
   $93 = HEAP32[$nBlock>>2]|0;
   $94 = HEAP32[$5>>2]|0;
   $95 = HEAP32[$6>>2]|0;
   $96 = (_fts3TruncateNode($92,$93,$block,$94,$95,$iBlock)|0);
   HEAP32[$rc>>2] = $96;
  }
  $97 = HEAP32[$rc>>2]|0;
  $98 = ($97|0)==(0);
  if ($98) {
   $99 = HEAP32[$2>>2]|0;
   $100 = $iNewStart;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($100) + 4)|0;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = HEAP32[$block>>2]|0;
   $107 = ((($block)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (_fts3WriteSegment($99,$102,$105,$106,$108)|0);
   HEAP32[$rc>>2] = $109;
  }
  $110 = HEAP32[$aBlock>>2]|0;
  _sqlite3_free($110);
 }
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 $113 = $iNewStart;
 $114 = $113;
 $115 = HEAP32[$114>>2]|0;
 $116 = (($113) + 4)|0;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = ($115|0)!=(0);
 $120 = ($118|0)!=(0);
 $121 = $119 | $120;
 $or$cond = $112 & $121;
 if ($or$cond) {
  HEAP32[$pDel>>2] = 0;
  $122 = HEAP32[$2>>2]|0;
  $123 = (_fts3SqlStmt($122,17,$pDel,0)|0);
  HEAP32[$rc>>2] = $123;
  $124 = HEAP32[$rc>>2]|0;
  $125 = ($124|0)==(0);
  if ($125) {
   $126 = HEAP32[$pDel>>2]|0;
   $127 = $iOldStart;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   (_sqlite3_bind_int64($126,1,$129,$132)|0);
   $133 = HEAP32[$pDel>>2]|0;
   $134 = $iNewStart;
   $135 = $134;
   $136 = HEAP32[$135>>2]|0;
   $137 = (($134) + 4)|0;
   $138 = $137;
   $139 = HEAP32[$138>>2]|0;
   $140 = (_i64Subtract(($136|0),($139|0),1,0)|0);
   $141 = tempRet0;
   (_sqlite3_bind_int64($133,2,$140,$141)|0);
   $142 = HEAP32[$pDel>>2]|0;
   (_sqlite3_step($142)|0);
   $143 = HEAP32[$pDel>>2]|0;
   $144 = (_sqlite3_reset($143)|0);
   HEAP32[$rc>>2] = $144;
  }
 }
 $145 = HEAP32[$rc>>2]|0;
 $146 = ($145|0)==(0);
 if (!($146)) {
  $174 = HEAP32[$root>>2]|0;
  _sqlite3_free($174);
  $175 = HEAP32[$block>>2]|0;
  _sqlite3_free($175);
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 HEAP32[$pChomp>>2] = 0;
 $147 = HEAP32[$2>>2]|0;
 $148 = (_fts3SqlStmt($147,33,$pChomp,0)|0);
 HEAP32[$rc>>2] = $148;
 $149 = HEAP32[$rc>>2]|0;
 $150 = ($149|0)==(0);
 if (!($150)) {
  $174 = HEAP32[$root>>2]|0;
  _sqlite3_free($174);
  $175 = HEAP32[$block>>2]|0;
  _sqlite3_free($175);
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $151 = HEAP32[$pChomp>>2]|0;
 $152 = $iNewStart;
 $153 = $152;
 $154 = HEAP32[$153>>2]|0;
 $155 = (($152) + 4)|0;
 $156 = $155;
 $157 = HEAP32[$156>>2]|0;
 (_sqlite3_bind_int64($151,1,$154,$157)|0);
 $158 = HEAP32[$pChomp>>2]|0;
 $159 = HEAP32[$root>>2]|0;
 $160 = ((($root)) + 4|0);
 $161 = HEAP32[$160>>2]|0;
 (_sqlite3_bind_blob($158,2,$159,$161,0)|0);
 $162 = HEAP32[$pChomp>>2]|0;
 $163 = $3;
 $164 = $163;
 $165 = HEAP32[$164>>2]|0;
 $166 = (($163) + 4)|0;
 $167 = $166;
 $168 = HEAP32[$167>>2]|0;
 (_sqlite3_bind_int64($162,3,$165,$168)|0);
 $169 = HEAP32[$pChomp>>2]|0;
 $170 = HEAP32[$4>>2]|0;
 (_sqlite3_bind_int($169,4,$170)|0);
 $171 = HEAP32[$pChomp>>2]|0;
 (_sqlite3_step($171)|0);
 $172 = HEAP32[$pChomp>>2]|0;
 $173 = (_sqlite3_reset($172)|0);
 HEAP32[$rc>>2] = $173;
 $174 = HEAP32[$root>>2]|0;
 _sqlite3_free($174);
 $175 = HEAP32[$block>>2]|0;
 _sqlite3_free($175);
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _fts3RepackSegdirLevel($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $aIdx = 0, $aNew = 0, $i = 0, $nAlloc = 0, $nIdx = 0, $pSelect = 0, $pUpdate = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp;
 $rc = sp + 40|0;
 $aIdx = sp + 36|0;
 $nIdx = sp + 32|0;
 $nAlloc = sp + 28|0;
 $i = sp + 24|0;
 $pSelect = sp + 20|0;
 $pUpdate = sp + 16|0;
 $rc2 = sp + 12|0;
 $aNew = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP32[$aIdx>>2] = 0;
 HEAP32[$nIdx>>2] = 0;
 HEAP32[$nAlloc>>2] = 0;
 HEAP32[$pSelect>>2] = 0;
 HEAP32[$pUpdate>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_fts3SqlStmt($8,35,$pSelect,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = HEAP32[$pSelect>>2]|0;
  $13 = $3;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  (_sqlite3_bind_int64($12,1,$15,$18)|0);
  while(1) {
   $19 = HEAP32[$pSelect>>2]|0;
   $20 = (_sqlite3_step($19)|0);
   $21 = (100)==($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$nIdx>>2]|0;
   $23 = HEAP32[$nAlloc>>2]|0;
   $24 = ($22|0)>=($23|0);
   if ($24) {
    $25 = HEAP32[$nAlloc>>2]|0;
    $26 = (($25) + 16)|0;
    HEAP32[$nAlloc>>2] = $26;
    $27 = HEAP32[$aIdx>>2]|0;
    $28 = HEAP32[$nAlloc>>2]|0;
    $29 = $28<<2;
    $30 = (_sqlite3_realloc($27,$29)|0);
    HEAP32[$aNew>>2] = $30;
    $31 = HEAP32[$aNew>>2]|0;
    $32 = ($31|0)!=(0|0);
    if (!($32)) {
     label = 6;
     break;
    }
    $33 = HEAP32[$aNew>>2]|0;
    HEAP32[$aIdx>>2] = $33;
   }
   $34 = HEAP32[$pSelect>>2]|0;
   $35 = (_sqlite3_column_int($34,0)|0);
   $36 = HEAP32[$nIdx>>2]|0;
   $37 = (($36) + 1)|0;
   HEAP32[$nIdx>>2] = $37;
   $38 = HEAP32[$aIdx>>2]|0;
   $39 = (($38) + ($36<<2)|0);
   HEAP32[$39>>2] = $35;
  }
  if ((label|0) == 6) {
   HEAP32[$rc>>2] = 7;
  }
  $40 = HEAP32[$pSelect>>2]|0;
  $41 = (_sqlite3_reset($40)|0);
  HEAP32[$rc2>>2] = $41;
  $42 = HEAP32[$rc>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $44;
  }
 }
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  $47 = HEAP32[$2>>2]|0;
  $48 = (_fts3SqlStmt($47,31,$pUpdate,0)|0);
  HEAP32[$rc>>2] = $48;
 }
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$pUpdate>>2]|0;
  $52 = $3;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  (_sqlite3_bind_int64($51,2,$54,$57)|0);
 }
 $58 = HEAP32[$2>>2]|0;
 $59 = ((($58)) + 232|0);
 HEAP8[$59>>0] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(0);
  if (!($61)) {
   label = 21;
   break;
  }
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$nIdx>>2]|0;
  $64 = ($62|0)<($63|0);
  if (!($64)) {
   label = 21;
   break;
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$aIdx>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$i>>2]|0;
  $70 = ($68|0)!=($69|0);
  if ($70) {
   $71 = HEAP32[$pUpdate>>2]|0;
   $72 = HEAP32[$i>>2]|0;
   $73 = HEAP32[$aIdx>>2]|0;
   $74 = (($73) + ($72<<2)|0);
   $75 = HEAP32[$74>>2]|0;
   (_sqlite3_bind_int($71,3,$75)|0);
   $76 = HEAP32[$pUpdate>>2]|0;
   $77 = HEAP32[$i>>2]|0;
   (_sqlite3_bind_int($76,1,$77)|0);
   $78 = HEAP32[$pUpdate>>2]|0;
   (_sqlite3_step($78)|0);
   $79 = HEAP32[$pUpdate>>2]|0;
   $80 = (_sqlite3_reset($79)|0);
   HEAP32[$rc>>2] = $80;
  }
  $81 = HEAP32[$i>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$i>>2] = $82;
 }
 if ((label|0) == 21) {
  $83 = HEAP32[$2>>2]|0;
  $84 = ((($83)) + 232|0);
  HEAP8[$84>>0] = 0;
  $85 = HEAP32[$aIdx>>2]|0;
  _sqlite3_free($85);
  $86 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 return (0)|0;
}
function _fts3TruncateNode($aNode,$nNode,$pNew,$zTerm,$nTerm,$piBlock) {
 $aNode = $aNode|0;
 $nNode = $nNode|0;
 $pNew = $pNew|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $piBlock = $piBlock|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bLeaf = 0, $or$cond = 0, $prev = 0, $rc = 0, $reader = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $reader = sp;
 $prev = sp + 60|0;
 $rc = sp + 56|0;
 $bLeaf = sp + 52|0;
 $res = sp + 48|0;
 HEAP32[$1>>2] = $aNode;
 HEAP32[$2>>2] = $nNode;
 HEAP32[$3>>2] = $pNew;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $piBlock;
 ;HEAP32[$prev>>2]=0|0;HEAP32[$prev+4>>2]=0|0;HEAP32[$prev+8>>2]=0|0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$bLeaf>>2] = $11;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 _blobGrowBuffer($12,$13,$rc);
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $111 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($111|0);
 }
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 4|0);
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = (_nodeReaderInit($reader,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 while(1) {
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)==(0);
  $24 = HEAP32[$reader>>2]|0;
  $25 = ($24|0)!=(0|0);
  $26 = $23 ? $25 : 0;
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$3>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = ((($reader)) + 24|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ((($reader)) + 24|0);
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$4>>2]|0;
   $37 = HEAP32[$5>>2]|0;
   $38 = (_fts3TermCmp($32,$35,$36,$37)|0);
   HEAP32[$res>>2] = $38;
   $39 = HEAP32[$res>>2]|0;
   $40 = ($39|0)<(0);
   if (!($40)) {
    $41 = HEAP32[$bLeaf>>2]|0;
    $42 = ($41|0)==(0);
    $43 = HEAP32[$res>>2]|0;
    $44 = ($43|0)==(0);
    $or$cond = $42 & $44;
    if (!($or$cond)) {
     $45 = HEAP32[$3>>2]|0;
     $46 = HEAP32[$1>>2]|0;
     $47 = HEAP8[$46>>0]|0;
     $48 = $47 << 24 >> 24;
     $49 = ((($reader)) + 16|0);
     $50 = $49;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (($50) + 4)|0;
     $54 = $53;
     $55 = HEAP32[$54>>2]|0;
     _fts3StartNode($45,$48,$52,$55);
     $56 = ((($reader)) + 16|0);
     $57 = $56;
     $58 = $57;
     $59 = HEAP32[$58>>2]|0;
     $60 = (($57) + 4)|0;
     $61 = $60;
     $62 = HEAP32[$61>>2]|0;
     $63 = HEAP32[$6>>2]|0;
     $64 = $63;
     $65 = $64;
     HEAP32[$65>>2] = $59;
     $66 = (($64) + 4)|0;
     $67 = $66;
     HEAP32[$67>>2] = $62;
     label = 9;
    }
   }
  } else {
   label = 9;
  }
  if ((label|0) == 9) {
   label = 0;
   $68 = HEAP32[$3>>2]|0;
   $69 = ((($reader)) + 24|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ((($reader)) + 24|0);
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = ((($reader)) + 36|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ((($reader)) + 40|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (_fts3AppendToNode($68,$prev,$70,$73,$75,$77)|0);
   HEAP32[$rc>>2] = $78;
   $79 = HEAP32[$rc>>2]|0;
   $80 = ($79|0)!=(0);
   if ($80) {
    break;
   }
  }
  $81 = (_nodeReaderNext($reader)|0);
  HEAP32[$rc>>2] = $81;
 }
 $82 = HEAP32[$3>>2]|0;
 $83 = ((($82)) + 4|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = ($84|0)==(0);
 if ($85) {
  $86 = HEAP32[$3>>2]|0;
  $87 = HEAP32[$1>>2]|0;
  $88 = HEAP8[$87>>0]|0;
  $89 = $88 << 24 >> 24;
  $90 = ((($reader)) + 16|0);
  $91 = $90;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (($91) + 4)|0;
  $95 = $94;
  $96 = HEAP32[$95>>2]|0;
  _fts3StartNode($86,$89,$93,$96);
  $97 = ((($reader)) + 16|0);
  $98 = $97;
  $99 = $98;
  $100 = HEAP32[$99>>2]|0;
  $101 = (($98) + 4)|0;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = HEAP32[$6>>2]|0;
  $105 = $104;
  $106 = $105;
  HEAP32[$106>>2] = $100;
  $107 = (($105) + 4)|0;
  $108 = $107;
  HEAP32[$108>>2] = $103;
 }
 _nodeReaderRelease($reader);
 $109 = HEAP32[$prev>>2]|0;
 _sqlite3_free($109);
 $110 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $110;
 $111 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($111|0);
}
function _fts3StartNode($pNode,$iHeight,$0,$1) {
 $pNode = $pNode|0;
 $iHeight = $iHeight|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$2>>2] = $pNode;
 HEAP32[$3>>2] = $iHeight;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9&255;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP8[$12>>0] = $10;
 $13 = $4;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($15|0)!=(0);
 $20 = ($18|0)!=(0);
 $21 = $19 | $20;
 $22 = HEAP32[$2>>2]|0;
 if ($21) {
  $23 = HEAP32[$22>>2]|0;
  $24 = ((($23)) + 1|0);
  $25 = $4;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3Fts3PutVarint($24,$27,$30)|0);
  $32 = (1 + ($31))|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = ((($33)) + 4|0);
  HEAP32[$34>>2] = $32;
  STACKTOP = sp;return;
 } else {
  $35 = ((($22)) + 4|0);
  HEAP32[$35>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _fts3DeleteTerms($pRC,$p,$pRowid,$aSz,$pbFound) {
 $pRC = $pRC|0;
 $p = $p|0;
 $pRowid = $pRowid|0;
 $aSz = $aSz|0;
 $pbFound = $pbFound|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $i = 0, $iCol = 0, $iDocid = 0, $iLangid = 0, $pSelect = 0, $rc = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $rc = sp + 28|0;
 $pSelect = sp + 24|0;
 $i = sp + 20|0;
 $iLangid = sp + 16|0;
 $iDocid = sp;
 $iCol = sp + 12|0;
 $zText = sp + 8|0;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pRowid;
 HEAP32[$3>>2] = $aSz;
 HEAP32[$4>>2] = $pbFound;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (_fts3SqlStmt($8,7,$pSelect,$2)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 $12 = HEAP32[$pSelect>>2]|0;
 if ($11) {
  $13 = (_sqlite3_step($12)|0);
  $14 = (100)==($13|0);
  do {
   if ($14) {
    $15 = HEAP32[$1>>2]|0;
    $16 = HEAP32[$pSelect>>2]|0;
    $17 = (_langidFromSelect($15,$16)|0);
    HEAP32[$iLangid>>2] = $17;
    $18 = HEAP32[$pSelect>>2]|0;
    $19 = (_sqlite3_column_int64($18,0)|0);
    $20 = tempRet0;
    $21 = $iDocid;
    $22 = $21;
    HEAP32[$22>>2] = $19;
    $23 = (($21) + 4)|0;
    $24 = $23;
    HEAP32[$24>>2] = $20;
    $25 = HEAP32[$1>>2]|0;
    $26 = HEAP32[$iLangid>>2]|0;
    $27 = $iDocid;
    $28 = $27;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($27) + 4)|0;
    $31 = $30;
    $32 = HEAP32[$31>>2]|0;
    $33 = (_fts3PendingTermsDocid($25,1,$26,$29,$32)|0);
    HEAP32[$rc>>2] = $33;
    HEAP32[$i>>2] = 1;
    while(1) {
     $34 = HEAP32[$rc>>2]|0;
     $35 = ($34|0)==(0);
     if (!($35)) {
      break;
     }
     $36 = HEAP32[$i>>2]|0;
     $37 = HEAP32[$1>>2]|0;
     $38 = ((($37)) + 24|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = ($36|0)<=($39|0);
     if (!($40)) {
      break;
     }
     $41 = HEAP32[$i>>2]|0;
     $42 = (($41) - 1)|0;
     HEAP32[$iCol>>2] = $42;
     $43 = HEAP32[$iCol>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = ((($44)) + 32|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = (($46) + ($43)|0);
     $48 = HEAP8[$47>>0]|0;
     $49 = $48&255;
     $50 = ($49|0)==(0);
     if ($50) {
      $51 = HEAP32[$pSelect>>2]|0;
      $52 = HEAP32[$i>>2]|0;
      $53 = (_sqlite3_column_text($51,$52)|0);
      HEAP32[$zText>>2] = $53;
      $54 = HEAP32[$1>>2]|0;
      $55 = HEAP32[$iLangid>>2]|0;
      $56 = HEAP32[$zText>>2]|0;
      $57 = HEAP32[$iCol>>2]|0;
      $58 = HEAP32[$3>>2]|0;
      $59 = (($58) + ($57<<2)|0);
      $60 = (_fts3PendingTermsAdd($54,$55,$56,-1,$59)|0);
      HEAP32[$rc>>2] = $60;
      $61 = HEAP32[$pSelect>>2]|0;
      $62 = HEAP32[$i>>2]|0;
      $63 = (_sqlite3_column_bytes($61,$62)|0);
      $64 = HEAP32[$1>>2]|0;
      $65 = ((($64)) + 24|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$3>>2]|0;
      $68 = (($67) + ($66<<2)|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = (($69) + ($63))|0;
      HEAP32[$68>>2] = $70;
     }
     $71 = HEAP32[$i>>2]|0;
     $72 = (($71) + 1)|0;
     HEAP32[$i>>2] = $72;
    }
    $73 = HEAP32[$rc>>2]|0;
    $74 = ($73|0)!=(0);
    if (!($74)) {
     $78 = HEAP32[$4>>2]|0;
     HEAP32[$78>>2] = 1;
     break;
    }
    $75 = HEAP32[$pSelect>>2]|0;
    (_sqlite3_reset($75)|0);
    $76 = HEAP32[$rc>>2]|0;
    $77 = HEAP32[$0>>2]|0;
    HEAP32[$77>>2] = $76;
    STACKTOP = sp;return;
   }
  } while(0);
  $79 = HEAP32[$pSelect>>2]|0;
  $80 = (_sqlite3_reset($79)|0);
  HEAP32[$rc>>2] = $80;
 } else {
  (_sqlite3_reset($12)|0);
 }
 $81 = HEAP32[$rc>>2]|0;
 $82 = HEAP32[$0>>2]|0;
 HEAP32[$82>>2] = $81;
 STACKTOP = sp;return;
}
function _fts3IsEmpty($p,$pRowid,$pisEmpty) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 $pisEmpty = $pisEmpty|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pRowid;
 HEAP32[$2>>2] = $pisEmpty;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  HEAP32[$7>>2] = 0;
  HEAP32[$rc>>2] = 0;
  $20 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (_fts3SqlStmt($8,1,$pStmt,$1)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $20 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $12 = HEAP32[$pStmt>>2]|0;
 $13 = (_sqlite3_step($12)|0);
 $14 = (100)==($13|0);
 if ($14) {
  $15 = HEAP32[$pStmt>>2]|0;
  $16 = (_sqlite3_column_int($15,0)|0);
  $17 = HEAP32[$2>>2]|0;
  HEAP32[$17>>2] = $16;
 }
 $18 = HEAP32[$pStmt>>2]|0;
 $19 = (_sqlite3_reset($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3Fts3PendingTermsFlush($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$rc>>2]|0;
  $2 = ($1|0)==(0);
  if ($2) {
   $3 = HEAP32[$i>>2]|0;
   $4 = HEAP32[$0>>2]|0;
   $5 = ((($4)) + 248|0);
   $6 = HEAP32[$5>>2]|0;
   $7 = ($3|0)<($6|0);
   $59 = $7;
  } else {
   $59 = 0;
  }
  $8 = HEAP32[$0>>2]|0;
  if (!($59)) {
   break;
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 272|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$i>>2]|0;
  $13 = (_fts3SegmentMerge($8,$11,$12,-1)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  $15 = ($14|0)==(101);
  $$ = $15 ? 0 : $13;
  HEAP32[$rc>>2] = $$;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$i>>2] = $17;
 }
 _sqlite3Fts3PendingTermsClear($8);
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 229|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 48|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(255);
 if (!($28)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 52|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31>>>0)>(0);
 if (!($32)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 HEAP32[$pStmt>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 $34 = (_fts3SqlStmt($33,22,$pStmt,0)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if (!($36)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $37 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_bind_int($37,1,2)|0);
 $38 = HEAP32[$pStmt>>2]|0;
 $39 = (_sqlite3_step($38)|0);
 HEAP32[$rc>>2] = $39;
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(100);
 if ($41) {
  $42 = HEAP32[$pStmt>>2]|0;
  $43 = (_sqlite3_column_int($42,0)|0);
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 48|0);
  HEAP32[$45>>2] = $43;
  $46 = HEAP32[$0>>2]|0;
  $47 = ((($46)) + 48|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)==(1);
  if ($49) {
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 48|0);
   HEAP32[$51>>2] = 8;
  }
 } else {
  $52 = HEAP32[$rc>>2]|0;
  $53 = ($52|0)==(101);
  if ($53) {
   $54 = HEAP32[$0>>2]|0;
   $55 = ((($54)) + 48|0);
   HEAP32[$55>>2] = 0;
  }
 }
 $56 = HEAP32[$pStmt>>2]|0;
 $57 = (_sqlite3_reset($56)|0);
 HEAP32[$rc>>2] = $57;
 $58 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _fts3EncodeIntArray($N,$a,$zBuf,$pNBuf) {
 $N = $N|0;
 $a = $a|0;
 $zBuf = $zBuf|0;
 $pNBuf = $pNBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $pNBuf;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)<($5|0);
  $7 = HEAP32[$j>>2]|0;
  if (!($6)) {
   break;
  }
  $8 = HEAP32[$2>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + ($10<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3Fts3PutVarint($9,$13,0)|0);
  $15 = HEAP32[$j>>2]|0;
  $16 = (($15) + ($14))|0;
  HEAP32[$j>>2] = $16;
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
 }
 $19 = HEAP32[$3>>2]|0;
 HEAP32[$19>>2] = $7;
 STACKTOP = sp;return;
}
function _fts3DecodeIntArray($N,$a,$zBuf,$nBuf) {
 $N = $N|0;
 $a = $a|0;
 $zBuf = $zBuf|0;
 $nBuf = $nBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $x = sp;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $nBuf;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$j>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = (_sqlite3Fts3GetVarint($9,$x)|0);
  $11 = HEAP32[$j>>2]|0;
  $12 = (($11) + ($10))|0;
  HEAP32[$j>>2] = $12;
  $13 = $x;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + ($19<<2)|0);
  HEAP32[$21>>2] = $15;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 STACKTOP = sp;return;
}
function _fts3SetHasStat($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bHasStat = 0, $pStmt = 0, $rc = 0;
 var $vararg_buffer = 0, $vararg_ptr1 = 0, $zFmt = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $rc = sp + 24|0;
 $zFmt = sp + 20|0;
 $zSql = sp + 16|0;
 $pStmt = sp + 12|0;
 $bHasStat = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 229|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 if (!($5)) {
  $36 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 HEAP32[$zFmt>>2] = 44530;
 $6 = HEAP32[$zFmt>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $12;
 $13 = (_sqlite3_mprintf($6,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $13;
 $14 = HEAP32[$zSql>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  HEAP32[$rc>>2] = 7;
  $36 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 HEAP32[$pStmt>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$zSql>>2]|0;
 $20 = (_sqlite3_prepare_v2($18,$19,-1,$pStmt,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = HEAP32[$pStmt>>2]|0;
  $24 = (_sqlite3_step($23)|0);
  $25 = ($24|0)==(100);
  $26 = $25&1;
  HEAP32[$bHasStat>>2] = $26;
  $27 = HEAP32[$pStmt>>2]|0;
  $28 = (_sqlite3_finalize($27)|0);
  HEAP32[$rc>>2] = $28;
  $29 = HEAP32[$rc>>2]|0;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = HEAP32[$bHasStat>>2]|0;
   $32 = $31&255;
   $33 = HEAP32[$0>>2]|0;
   $34 = ((($33)) + 229|0);
   HEAP8[$34>>0] = $32;
  }
 }
 $35 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($35);
 $36 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3Fts3MaxLevel($p,$pnMax) {
 $p = $p|0;
 $pnMax = $pnMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mxLevel = 0, $pStmt = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $mxLevel = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnMax;
 HEAP32[$mxLevel>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,36,$pStmt,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $13 = HEAP32[$mxLevel>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$14>>2] = $13;
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 $6 = HEAP32[$pStmt>>2]|0;
 $7 = (_sqlite3_step($6)|0);
 $8 = (100)==($7|0);
 if ($8) {
  $9 = HEAP32[$pStmt>>2]|0;
  $10 = (_sqlite3_column_int($9,0)|0);
  HEAP32[$mxLevel>>2] = $10;
 }
 $11 = HEAP32[$pStmt>>2]|0;
 $12 = (_sqlite3_reset($11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$mxLevel>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _fts3SnippetFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iCol = 0, $nToken = 0, $or$cond = 0, $or$cond3 = 0, $pCsr = 0, $zEllipsis = 0, $zEnd = 0, $zStart = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pCsr = sp + 20|0;
 $zStart = sp + 16|0;
 $zEnd = sp + 12|0;
 $zEllipsis = sp + 8|0;
 $iCol = sp + 4|0;
 $nToken = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$zStart>>2] = 44586;
 HEAP32[$zEnd>>2] = 44590;
 HEAP32[$zEllipsis>>2] = 44595;
 HEAP32[$iCol>>2] = -1;
 HEAP32[$nToken>>2] = 15;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(6);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  _sqlite3_result_error($5,44606,-1);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_fts3FunctionArg($5,39950,$7,$pCsr)|0);
 $9 = ($8|0)!=(0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 switch ($10|0) {
 case 6:  {
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_int($13)|0);
  HEAP32[$nToken>>2] = $14;
  label = 6;
  break;
 }
 case 5:  {
  label = 6;
  break;
 }
 case 4:  {
  label = 7;
  break;
 }
 case 3:  {
  label = 8;
  break;
 }
 case 2:  {
  label = 9;
  break;
 }
 default: {
 }
 }
 if ((label|0) == 6) {
  $15 = HEAP32[$2>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3_value_int($17)|0);
  HEAP32[$iCol>>2] = $18;
  label = 7;
 }
 if ((label|0) == 7) {
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3_value_text($21)|0);
  HEAP32[$zEllipsis>>2] = $22;
  label = 8;
 }
 if ((label|0) == 8) {
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3_value_text($25)|0);
  HEAP32[$zEnd>>2] = $26;
  label = 9;
 }
 if ((label|0) == 9) {
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (_sqlite3_value_text($29)|0);
  HEAP32[$zStart>>2] = $30;
 }
 $31 = HEAP32[$zEllipsis>>2]|0;
 $32 = ($31|0)!=(0|0);
 $33 = HEAP32[$zEnd>>2]|0;
 $34 = ($33|0)!=(0|0);
 $or$cond = $32 & $34;
 $35 = HEAP32[$zStart>>2]|0;
 $36 = ($35|0)!=(0|0);
 $or$cond3 = $or$cond & $36;
 if (!($or$cond3)) {
  $37 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($37);
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$nToken>>2]|0;
 $39 = ($38|0)==(0);
 $40 = HEAP32[$0>>2]|0;
 if ($39) {
  _sqlite3_result_text($40,17532,-1,0);
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pCsr>>2]|0;
 $42 = (_fts3CursorSeek($40,$41)|0);
 $43 = (0)==($42|0);
 if (!($43)) {
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$pCsr>>2]|0;
 $46 = HEAP32[$zStart>>2]|0;
 $47 = HEAP32[$zEnd>>2]|0;
 $48 = HEAP32[$zEllipsis>>2]|0;
 $49 = HEAP32[$iCol>>2]|0;
 $50 = HEAP32[$nToken>>2]|0;
 _sqlite3Fts3Snippet($44,$45,$46,$47,$48,$49,$50);
 STACKTOP = sp;return;
}
function _fts3OffsetsFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_fts3FunctionArg($3,39958,$5,$pCsr)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$pCsr>>2]|0;
 $10 = (_fts3CursorSeek($8,$9)|0);
 $11 = (0)==($10|0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$pCsr>>2]|0;
 _sqlite3Fts3Offsets($12,$13);
 STACKTOP = sp;return;
}
function _fts3OptimizeFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pCursor = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $p = sp + 4|0;
 $pCursor = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_fts3FunctionArg($3,39976,$5,$pCursor)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pCursor>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = (_sqlite3Fts3Optimize($10)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 switch ($12|0) {
 case 0:  {
  $13 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($13,44696,-1,0);
  STACKTOP = sp;return;
  break;
 }
 case 101:  {
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($14,44712,-1,0);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($15,$16);
  STACKTOP = sp;return;
 }
 }
}
function _fts3MatchinfoFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $zArg = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pCsr = sp + 4|0;
 $zArg = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_fts3FunctionArg($3,39966,$5,$pCsr)|0);
 $7 = (0)==($6|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 HEAP32[$zArg>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)>(1);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_sqlite3_value_text($12)|0);
  HEAP32[$zArg>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$pCsr>>2]|0;
 $16 = HEAP32[$zArg>>2]|0;
 _sqlite3Fts3Matchinfo($14,$15,$16);
 STACKTOP = sp;return;
}
function _fts3FunctionArg($pContext,$zFunc,$pVal,$ppCsr) {
 $pContext = $pContext|0;
 $zFunc = $zFunc|0;
 $pVal = $pVal|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pRet = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pRet = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $zFunc;
 HEAP32[$3>>2] = $pVal;
 HEAP32[$4>>2] = $ppCsr;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_sqlite3_value_type($5)|0);
 $7 = ($6|0)!=(4);
 if (!($7)) {
  $8 = HEAP32[$3>>2]|0;
  $9 = (_sqlite3_value_bytes($8)|0);
  $10 = ($9|0)!=(4);
  if (!($10)) {
   $16 = HEAP32[$3>>2]|0;
   $17 = (_sqlite3_value_blob($16)|0);
   ;HEAP8[$pRet>>0]=HEAP8[$17>>0]|0;HEAP8[$pRet+1>>0]=HEAP8[$17+1>>0]|0;HEAP8[$pRet+2>>0]=HEAP8[$17+2>>0]|0;HEAP8[$pRet+3>>0]=HEAP8[$17+3>>0]|0;
   $18 = HEAP32[$pRet>>2]|0;
   $19 = HEAP32[$4>>2]|0;
   HEAP32[$19>>2] = $18;
   HEAP32[$0>>2] = 0;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 $12 = (_sqlite3_mprintf(44654,$vararg_buffer)|0);
 HEAP32[$zErr>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$zErr>>2]|0;
 _sqlite3_result_error($13,$14,-1);
 $15 = HEAP32[$zErr>>2]|0;
 _sqlite3_free($15);
 HEAP32[$0>>2] = 1;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3Fts3Snippet($pCtx,$pCsr,$zStart,$zEnd,$zEllipsis,$iCol,$nToken) {
 $pCtx = $pCtx|0;
 $pCsr = $pCsr|0;
 $zStart = $zStart|0;
 $zEnd = $zEnd|0;
 $zEllipsis = $zEllipsis|0;
 $iCol = $iCol|0;
 $nToken = $nToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aSnippet = 0, $i = 0, $iBestScore = 0, $iRead = 0;
 var $iS = 0, $iSnip = 0, $mCovered = 0, $mSeen = 0, $nFToken = 0, $nSnippet = 0, $or$cond = 0, $pFragment = 0, $pTab = 0, $rc = 0, $res = 0, $sF = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 212|0;
 $1 = sp + 208|0;
 $2 = sp + 204|0;
 $3 = sp + 200|0;
 $4 = sp + 196|0;
 $5 = sp + 192|0;
 $6 = sp + 188|0;
 $pTab = sp + 184|0;
 $rc = sp + 180|0;
 $i = sp + 176|0;
 $res = sp + 164|0;
 $nSnippet = sp + 160|0;
 $aSnippet = sp + 40|0;
 $nFToken = sp + 156|0;
 $iSnip = sp + 152|0;
 $mCovered = sp + 32|0;
 $mSeen = sp + 24|0;
 $iBestScore = sp + 148|0;
 $iRead = sp + 144|0;
 $pFragment = sp + 140|0;
 $sF = sp;
 $iS = sp + 136|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $zStart;
 HEAP32[$3>>2] = $zEnd;
 HEAP32[$4>>2] = $zEllipsis;
 HEAP32[$5>>2] = $iCol;
 HEAP32[$6>>2] = $nToken;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTab>>2] = $8;
 HEAP32[$rc>>2] = 0;
 ;HEAP32[$res>>2]=0|0;HEAP32[$res+4>>2]=0|0;HEAP32[$res+8>>2]=0|0;
 HEAP32[$nSnippet>>2] = 0;
 HEAP32[$nFToken>>2] = -1;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $13 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($13,17532,0,0);
  STACKTOP = sp;return;
 }
 HEAP32[$nSnippet>>2] = 1;
 L5: while(1) {
  $14 = $mCovered;
  $15 = $14;
  HEAP32[$15>>2] = 0;
  $16 = (($14) + 4)|0;
  $17 = $16;
  HEAP32[$17>>2] = 0;
  $18 = $mSeen;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$6>>2]|0;
  $23 = ($22|0)>=(0);
  $24 = HEAP32[$6>>2]|0;
  if ($23) {
   $25 = HEAP32[$nSnippet>>2]|0;
   $26 = (($24) + ($25))|0;
   $27 = (($26) - 1)|0;
   $28 = HEAP32[$nSnippet>>2]|0;
   $29 = (($27|0) / ($28|0))&-1;
   HEAP32[$nFToken>>2] = $29;
  } else {
   $30 = Math_imul(-1, $24)|0;
   HEAP32[$nFToken>>2] = $30;
  }
  HEAP32[$iSnip>>2] = 0;
  while(1) {
   $31 = HEAP32[$iSnip>>2]|0;
   $32 = HEAP32[$nSnippet>>2]|0;
   $33 = ($31|0)<($32|0);
   if (!($33)) {
    break;
   }
   HEAP32[$iBestScore>>2] = -1;
   $34 = HEAP32[$iSnip>>2]|0;
   $35 = (($aSnippet) + (($34*24)|0)|0);
   HEAP32[$pFragment>>2] = $35;
   $36 = HEAP32[$pFragment>>2]|0;
   ;HEAP32[$36>>2]=0|0;HEAP32[$36+4>>2]=0|0;HEAP32[$36+8>>2]=0|0;HEAP32[$36+12>>2]=0|0;HEAP32[$36+16>>2]=0|0;HEAP32[$36+20>>2]=0|0;
   HEAP32[$iRead>>2] = 0;
   while(1) {
    $37 = HEAP32[$iRead>>2]|0;
    $38 = HEAP32[$pTab>>2]|0;
    $39 = ((($38)) + 24|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = ($37|0)<($40|0);
    if (!($41)) {
     break;
    }
    ;HEAP32[$sF>>2]=0|0;HEAP32[$sF+4>>2]=0|0;HEAP32[$sF+8>>2]=0|0;HEAP32[$sF+12>>2]=0|0;HEAP32[$sF+16>>2]=0|0;HEAP32[$sF+20>>2]=0|0;
    HEAP32[$iS>>2] = 0;
    $42 = HEAP32[$5>>2]|0;
    $43 = ($42|0)>=(0);
    if ($43) {
     $44 = HEAP32[$iRead>>2]|0;
     $45 = HEAP32[$5>>2]|0;
     $46 = ($44|0)!=($45|0);
     if (!($46)) {
      label = 13;
     }
    } else {
     label = 13;
    }
    if ((label|0) == 13) {
     label = 0;
     $47 = HEAP32[$nFToken>>2]|0;
     $48 = HEAP32[$1>>2]|0;
     $49 = HEAP32[$iRead>>2]|0;
     $50 = $mCovered;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (($50) + 4)|0;
     $54 = $53;
     $55 = HEAP32[$54>>2]|0;
     $56 = (_fts3BestSnippet($47,$48,$49,$52,$55,$mSeen,$sF,$iS)|0);
     HEAP32[$rc>>2] = $56;
     $57 = HEAP32[$rc>>2]|0;
     $58 = ($57|0)!=(0);
     if ($58) {
      break L5;
     }
     $59 = HEAP32[$iS>>2]|0;
     $60 = HEAP32[$iBestScore>>2]|0;
     $61 = ($59|0)>($60|0);
     if ($61) {
      $62 = HEAP32[$pFragment>>2]|0;
      ;HEAP32[$62>>2]=HEAP32[$sF>>2]|0;HEAP32[$62+4>>2]=HEAP32[$sF+4>>2]|0;HEAP32[$62+8>>2]=HEAP32[$sF+8>>2]|0;HEAP32[$62+12>>2]=HEAP32[$sF+12>>2]|0;HEAP32[$62+16>>2]=HEAP32[$sF+16>>2]|0;HEAP32[$62+20>>2]=HEAP32[$sF+20>>2]|0;
      $63 = HEAP32[$iS>>2]|0;
      HEAP32[$iBestScore>>2] = $63;
     }
    }
    $64 = HEAP32[$iRead>>2]|0;
    $65 = (($64) + 1)|0;
    HEAP32[$iRead>>2] = $65;
   }
   $66 = HEAP32[$pFragment>>2]|0;
   $67 = ((($66)) + 8|0);
   $68 = $67;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = (($68) + 4)|0;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = $mCovered;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($74) + 4)|0;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = $76 | $70;
   $81 = $79 | $73;
   $82 = $mCovered;
   $83 = $82;
   HEAP32[$83>>2] = $80;
   $84 = (($82) + 4)|0;
   $85 = $84;
   HEAP32[$85>>2] = $81;
   $86 = HEAP32[$iSnip>>2]|0;
   $87 = (($86) + 1)|0;
   HEAP32[$iSnip>>2] = $87;
  }
  $88 = $mSeen;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = $mCovered;
  $95 = $94;
  $96 = HEAP32[$95>>2]|0;
  $97 = (($94) + 4)|0;
  $98 = $97;
  $99 = HEAP32[$98>>2]|0;
  $100 = ($90|0)==($96|0);
  $101 = ($93|0)==($99|0);
  $102 = $100 & $101;
  $103 = HEAP32[$nSnippet>>2]|0;
  $104 = ($103|0)==(4);
  $or$cond = $102 | $104;
  if ($or$cond) {
   label = 20;
   break;
  }
  $105 = HEAP32[$nSnippet>>2]|0;
  $106 = (($105) + 1)|0;
  HEAP32[$nSnippet>>2] = $106;
 }
 L27: do {
  if ((label|0) == 20) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $107 = HEAP32[$i>>2]|0;
    $108 = HEAP32[$nSnippet>>2]|0;
    $109 = ($107|0)<($108|0);
    $110 = HEAP32[$rc>>2]|0;
    $111 = ($110|0)==(0);
    $112 = $109 ? $111 : 0;
    if (!($112)) {
     break L27;
    }
    $113 = HEAP32[$1>>2]|0;
    $114 = HEAP32[$i>>2]|0;
    $115 = (($aSnippet) + (($114*24)|0)|0);
    $116 = HEAP32[$i>>2]|0;
    $117 = HEAP32[$i>>2]|0;
    $118 = HEAP32[$nSnippet>>2]|0;
    $119 = (($118) - 1)|0;
    $120 = ($117|0)==($119|0);
    $121 = $120&1;
    $122 = HEAP32[$nFToken>>2]|0;
    $123 = HEAP32[$2>>2]|0;
    $124 = HEAP32[$3>>2]|0;
    $125 = HEAP32[$4>>2]|0;
    $126 = (_fts3SnippetText($113,$115,$116,$121,$122,$123,$124,$125,$res)|0);
    HEAP32[$rc>>2] = $126;
    $127 = HEAP32[$i>>2]|0;
    $128 = (($127) + 1)|0;
    HEAP32[$i>>2] = $128;
   }
  }
 } while(0);
 $129 = HEAP32[$pTab>>2]|0;
 _sqlite3Fts3SegmentsClose($129);
 $130 = HEAP32[$rc>>2]|0;
 $131 = ($130|0)!=(0);
 $132 = HEAP32[$0>>2]|0;
 if ($131) {
  $133 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($132,$133);
  $134 = HEAP32[$res>>2]|0;
  _sqlite3_free($134);
  STACKTOP = sp;return;
 } else {
  $135 = HEAP32[$res>>2]|0;
  _sqlite3_result_text($132,$135,-1,142);
  STACKTOP = sp;return;
 }
}
function _fts3BestSnippet($nSnippet,$pCsr,$iCol,$0,$1,$pmSeen,$pFragment,$piScore) {
 $nSnippet = $nSnippet|0;
 $pCsr = $pCsr|0;
 $iCol = $iCol|0;
 $0 = $0|0;
 $1 = $1|0;
 $pmSeen = $pmSeen|0;
 $pFragment = $pFragment|0;
 $piScore = $piScore|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iBestScore = 0;
 var $iPos = 0, $iScore = 0, $mCover = 0, $mHighlite = 0, $nByte = 0, $nList = 0, $rc = 0, $sIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 16|0;
 $7 = sp + 88|0;
 $8 = sp + 84|0;
 $9 = sp + 80|0;
 $rc = sp + 76|0;
 $nList = sp + 72|0;
 $sIter = sp + 48|0;
 $nByte = sp + 40|0;
 $iBestScore = sp + 36|0;
 $i = sp + 32|0;
 $iPos = sp + 28|0;
 $iScore = sp + 24|0;
 $mCover = sp + 8|0;
 $mHighlite = sp;
 HEAP32[$3>>2] = $nSnippet;
 HEAP32[$4>>2] = $pCsr;
 HEAP32[$5>>2] = $iCol;
 $10 = $6;
 $11 = $10;
 HEAP32[$11>>2] = $0;
 $12 = (($10) + 4)|0;
 $13 = $12;
 HEAP32[$13>>2] = $1;
 HEAP32[$7>>2] = $pmSeen;
 HEAP32[$8>>2] = $pFragment;
 HEAP32[$9>>2] = $piScore;
 HEAP32[$iBestScore>>2] = -1;
 ;HEAP32[$sIter>>2]=0|0;HEAP32[$sIter+4>>2]=0|0;HEAP32[$sIter+8>>2]=0|0;HEAP32[$sIter+12>>2]=0|0;HEAP32[$sIter+16>>2]=0|0;HEAP32[$sIter+20>>2]=0|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = (_fts3ExprLoadDoclists($14,$nList,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $18;
  $119 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $19 = HEAP32[$nList>>2]|0;
 $20 = ($19*24)|0;
 HEAP32[$nByte>>2] = $20;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (_sqlite3_malloc($21)|0);
 $23 = ((($sIter)) + 16|0);
 HEAP32[$23>>2] = $22;
 $24 = ((($sIter)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  HEAP32[$2>>2] = 7;
  $119 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $27 = ((($sIter)) + 16|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$nByte>>2]|0;
 _memset(($28|0),0,($29|0))|0;
 $30 = HEAP32[$4>>2]|0;
 HEAP32[$sIter>>2] = $30;
 $31 = HEAP32[$5>>2]|0;
 $32 = ((($sIter)) + 4|0);
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$3>>2]|0;
 $34 = ((($sIter)) + 8|0);
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$nList>>2]|0;
 $36 = ((($sIter)) + 12|0);
 HEAP32[$36>>2] = $35;
 $37 = ((($sIter)) + 20|0);
 HEAP32[$37>>2] = -1;
 $38 = HEAP32[$4>>2]|0;
 $39 = ((($38)) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (_fts3ExprIterate($40,156,$sIter)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $44 = HEAP32[$i>>2]|0;
   $45 = HEAP32[$nList>>2]|0;
   $46 = ($44|0)<($45|0);
   if (!($46)) {
    break;
   }
   $47 = HEAP32[$i>>2]|0;
   $48 = ((($sIter)) + 16|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + (($47*24)|0)|0);
   $51 = ((($50)) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)!=(0|0);
   if ($53) {
    $54 = HEAP32[$i>>2]|0;
    $55 = (_bitshift64Shl(1,0,($54|0))|0);
    $56 = tempRet0;
    $57 = HEAP32[$7>>2]|0;
    $58 = $57;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = $60 | $55;
    $65 = $63 | $56;
    $66 = $57;
    $67 = $66;
    HEAP32[$67>>2] = $64;
    $68 = (($66) + 4)|0;
    $69 = $68;
    HEAP32[$69>>2] = $65;
   }
   $70 = HEAP32[$i>>2]|0;
   $71 = (($70) + 1)|0;
   HEAP32[$i>>2] = $71;
  }
  $72 = HEAP32[$5>>2]|0;
  $73 = HEAP32[$8>>2]|0;
  HEAP32[$73>>2] = $72;
  while(1) {
   $74 = (_fts3SnippetNextCandidate($sIter)|0);
   $75 = ($74|0)!=(0);
   $76 = $75 ^ 1;
   if (!($76)) {
    break;
   }
   $77 = $6;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($77) + 4)|0;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   _fts3SnippetDetails($sIter,$79,$82,$iPos,$iScore,$mCover,$mHighlite);
   $83 = HEAP32[$iScore>>2]|0;
   $84 = HEAP32[$iBestScore>>2]|0;
   $85 = ($83|0)>($84|0);
   if (!($85)) {
    continue;
   }
   $86 = HEAP32[$iPos>>2]|0;
   $87 = HEAP32[$8>>2]|0;
   $88 = ((($87)) + 4|0);
   HEAP32[$88>>2] = $86;
   $89 = $mHighlite;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = (($89) + 4)|0;
   $93 = $92;
   $94 = HEAP32[$93>>2]|0;
   $95 = HEAP32[$8>>2]|0;
   $96 = ((($95)) + 16|0);
   $97 = $96;
   $98 = $97;
   HEAP32[$98>>2] = $91;
   $99 = (($97) + 4)|0;
   $100 = $99;
   HEAP32[$100>>2] = $94;
   $101 = $mCover;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = (($101) + 4)|0;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = HEAP32[$8>>2]|0;
   $108 = ((($107)) + 8|0);
   $109 = $108;
   $110 = $109;
   HEAP32[$110>>2] = $103;
   $111 = (($109) + 4)|0;
   $112 = $111;
   HEAP32[$112>>2] = $106;
   $113 = HEAP32[$iScore>>2]|0;
   HEAP32[$iBestScore>>2] = $113;
  }
  $114 = HEAP32[$iBestScore>>2]|0;
  $115 = HEAP32[$9>>2]|0;
  HEAP32[$115>>2] = $114;
 }
 $116 = ((($sIter)) + 16|0);
 $117 = HEAP32[$116>>2]|0;
 _sqlite3_free($117);
 $118 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $118;
 $119 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($119|0);
}
function _fts3SnippetText($pCsr,$pFragment,$iFragment,$isLast,$nSnippet,$zOpen,$zClose,$zEllipsis,$pOut) {
 $pCsr = $pCsr|0;
 $pFragment = $pFragment|0;
 $iFragment = $iFragment|0;
 $isLast = $isLast|0;
 $nSnippet = $nSnippet|0;
 $zOpen = $zOpen|0;
 $zClose = $zClose|0;
 $zEllipsis = $zEllipsis|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $DUMMY1 = 0, $ZDUMMY = 0, $hlmask = 0, $iBegin = 0, $iCol = 0, $iCurrent = 0, $iEnd = 0, $iFin = 0, $iPos = 0, $isHighlight = 0, $isShiftDone = 0, $n = 0, $nDoc = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pC = 0;
 var $pMod = 0, $pTab = 0, $rc = 0, $zDoc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 88|0;
 $7 = sp + 84|0;
 $8 = sp + 80|0;
 $9 = sp + 76|0;
 $pTab = sp + 72|0;
 $rc = sp + 68|0;
 $zDoc = sp + 64|0;
 $nDoc = sp + 60|0;
 $iCurrent = sp + 56|0;
 $iEnd = sp + 52|0;
 $isShiftDone = sp + 48|0;
 $iPos = sp + 44|0;
 $hlmask = sp;
 $iCol = sp + 40|0;
 $pMod = sp + 36|0;
 $pC = sp + 32|0;
 $ZDUMMY = sp + 28|0;
 $DUMMY1 = sp + 24|0;
 $iBegin = sp + 20|0;
 $iFin = sp + 16|0;
 $isHighlight = sp + 12|0;
 $n = sp + 8|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pFragment;
 HEAP32[$3>>2] = $iFragment;
 HEAP32[$4>>2] = $isLast;
 HEAP32[$5>>2] = $nSnippet;
 HEAP32[$6>>2] = $zOpen;
 HEAP32[$7>>2] = $zClose;
 HEAP32[$8>>2] = $zEllipsis;
 HEAP32[$9>>2] = $pOut;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pTab>>2] = $11;
 HEAP32[$iCurrent>>2] = 0;
 HEAP32[$iEnd>>2] = 0;
 HEAP32[$isShiftDone>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$iPos>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = $hlmask;
 $24 = $23;
 HEAP32[$24>>2] = $19;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 1)|0;
 HEAP32[$iCol>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$iCol>>2]|0;
 $34 = (_sqlite3_column_text($32,$33)|0);
 HEAP32[$zDoc>>2] = $34;
 $35 = HEAP32[$zDoc>>2]|0;
 $36 = ($35|0)==(0|0);
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$iCol>>2]|0;
 if ($36) {
  $41 = (_sqlite3_column_type($39,$40)|0);
  $42 = ($41|0)!=(5);
  if ($42) {
   HEAP32[$0>>2] = 7;
   $182 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($182|0);
  } else {
   HEAP32[$0>>2] = 0;
   $182 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($182|0);
  }
 }
 $43 = (_sqlite3_column_bytes($39,$40)|0);
 HEAP32[$nDoc>>2] = $43;
 $44 = HEAP32[$pTab>>2]|0;
 $45 = ((($44)) + 36|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pMod>>2] = $47;
 $48 = HEAP32[$pTab>>2]|0;
 $49 = ((($48)) + 36|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$zDoc>>2]|0;
 $55 = HEAP32[$nDoc>>2]|0;
 $56 = (_sqlite3Fts3OpenTokenizer($50,$53,$54,$55,$pC)|0);
 HEAP32[$rc>>2] = $56;
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $59;
  $182 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($182|0);
 }
 while(1) {
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(0);
  if (!($61)) {
   break;
  }
  HEAP32[$DUMMY1>>2] = -1;
  HEAP32[$iBegin>>2] = 0;
  HEAP32[$iFin>>2] = 0;
  HEAP32[$isHighlight>>2] = 0;
  $62 = HEAP32[$pMod>>2]|0;
  $63 = ((($62)) + 20|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$pC>>2]|0;
  $66 = (FUNCTION_TABLE_iiiiiii[$64 & 255]($65,$ZDUMMY,$DUMMY1,$iBegin,$iFin,$iCurrent)|0);
  HEAP32[$rc>>2] = $66;
  $67 = HEAP32[$rc>>2]|0;
  $68 = ($67|0)!=(0);
  if ($68) {
   label = 9;
   break;
  }
  $76 = HEAP32[$iCurrent>>2]|0;
  $77 = HEAP32[$iPos>>2]|0;
  $78 = ($76|0)<($77|0);
  if ($78) {
   continue;
  }
  $79 = HEAP32[$isShiftDone>>2]|0;
  $80 = ($79|0)!=(0);
  if (!($80)) {
   $81 = HEAP32[$nDoc>>2]|0;
   $82 = HEAP32[$iBegin>>2]|0;
   $83 = (($81) - ($82))|0;
   HEAP32[$n>>2] = $83;
   $84 = HEAP32[$pTab>>2]|0;
   $85 = HEAP32[$1>>2]|0;
   $86 = ((($85)) + 16|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$5>>2]|0;
   $89 = HEAP32[$iBegin>>2]|0;
   $90 = HEAP32[$zDoc>>2]|0;
   $91 = (($90) + ($89)|0);
   $92 = HEAP32[$n>>2]|0;
   $93 = (_fts3SnippetShift($84,$87,$88,$91,$92,$iPos,$hlmask)|0);
   HEAP32[$rc>>2] = $93;
   HEAP32[$isShiftDone>>2] = 1;
   $94 = HEAP32[$rc>>2]|0;
   $95 = ($94|0)==(0);
   do {
    if ($95) {
     $96 = HEAP32[$iPos>>2]|0;
     $97 = ($96|0)>(0);
     $98 = HEAP32[$3>>2]|0;
     $99 = ($98|0)>(0);
     $or$cond = $97 | $99;
     if ($or$cond) {
      $100 = HEAP32[$9>>2]|0;
      $101 = HEAP32[$8>>2]|0;
      $102 = (_fts3StringAppend($100,$101,-1)|0);
      HEAP32[$rc>>2] = $102;
      break;
     }
     $103 = HEAP32[$iBegin>>2]|0;
     $104 = ($103|0)!=(0);
     if ($104) {
      $105 = HEAP32[$9>>2]|0;
      $106 = HEAP32[$zDoc>>2]|0;
      $107 = HEAP32[$iBegin>>2]|0;
      $108 = (_fts3StringAppend($105,$106,$107)|0);
      HEAP32[$rc>>2] = $108;
     }
    }
   } while(0);
   $109 = HEAP32[$rc>>2]|0;
   $110 = ($109|0)!=(0);
   if ($110) {
    continue;
   }
   $111 = HEAP32[$iCurrent>>2]|0;
   $112 = HEAP32[$iPos>>2]|0;
   $113 = ($111|0)<($112|0);
   if ($113) {
    continue;
   }
  }
  $114 = HEAP32[$iCurrent>>2]|0;
  $115 = HEAP32[$iPos>>2]|0;
  $116 = HEAP32[$5>>2]|0;
  $117 = (($115) + ($116))|0;
  $118 = ($114|0)>=($117|0);
  if ($118) {
   label = 21;
   break;
  }
  $124 = $hlmask;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (($124) + 4)|0;
  $128 = $127;
  $129 = HEAP32[$128>>2]|0;
  $130 = HEAP32[$iCurrent>>2]|0;
  $131 = HEAP32[$iPos>>2]|0;
  $132 = (($130) - ($131))|0;
  $133 = (_bitshift64Shl(1,0,($132|0))|0);
  $134 = tempRet0;
  $135 = $126 & $133;
  $136 = $129 & $134;
  $137 = ($135|0)!=(0);
  $138 = ($136|0)!=(0);
  $139 = $137 | $138;
  $140 = $139&1;
  HEAP32[$isHighlight>>2] = $140;
  $141 = HEAP32[$iCurrent>>2]|0;
  $142 = HEAP32[$iPos>>2]|0;
  $143 = ($141|0)>($142|0);
  if ($143) {
   $144 = HEAP32[$9>>2]|0;
   $145 = HEAP32[$iEnd>>2]|0;
   $146 = HEAP32[$zDoc>>2]|0;
   $147 = (($146) + ($145)|0);
   $148 = HEAP32[$iBegin>>2]|0;
   $149 = HEAP32[$iEnd>>2]|0;
   $150 = (($148) - ($149))|0;
   $151 = (_fts3StringAppend($144,$147,$150)|0);
   HEAP32[$rc>>2] = $151;
  }
  $152 = HEAP32[$rc>>2]|0;
  $153 = ($152|0)==(0);
  $154 = HEAP32[$isHighlight>>2]|0;
  $155 = ($154|0)!=(0);
  $or$cond3 = $153 & $155;
  if ($or$cond3) {
   $156 = HEAP32[$9>>2]|0;
   $157 = HEAP32[$6>>2]|0;
   $158 = (_fts3StringAppend($156,$157,-1)|0);
   HEAP32[$rc>>2] = $158;
  }
  $159 = HEAP32[$rc>>2]|0;
  $160 = ($159|0)==(0);
  if ($160) {
   $161 = HEAP32[$9>>2]|0;
   $162 = HEAP32[$iBegin>>2]|0;
   $163 = HEAP32[$zDoc>>2]|0;
   $164 = (($163) + ($162)|0);
   $165 = HEAP32[$iFin>>2]|0;
   $166 = HEAP32[$iBegin>>2]|0;
   $167 = (($165) - ($166))|0;
   $168 = (_fts3StringAppend($161,$164,$167)|0);
   HEAP32[$rc>>2] = $168;
  }
  $169 = HEAP32[$rc>>2]|0;
  $170 = ($169|0)==(0);
  $171 = HEAP32[$isHighlight>>2]|0;
  $172 = ($171|0)!=(0);
  $or$cond5 = $170 & $172;
  if ($or$cond5) {
   $173 = HEAP32[$9>>2]|0;
   $174 = HEAP32[$7>>2]|0;
   $175 = (_fts3StringAppend($173,$174,-1)|0);
   HEAP32[$rc>>2] = $175;
  }
  $176 = HEAP32[$iFin>>2]|0;
  HEAP32[$iEnd>>2] = $176;
 }
 if ((label|0) == 9) {
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(101);
  if ($70) {
   $71 = HEAP32[$9>>2]|0;
   $72 = HEAP32[$iEnd>>2]|0;
   $73 = HEAP32[$zDoc>>2]|0;
   $74 = (($73) + ($72)|0);
   $75 = (_fts3StringAppend($71,$74,-1)|0);
   HEAP32[$rc>>2] = $75;
  }
 }
 else if ((label|0) == 21) {
  $119 = HEAP32[$4>>2]|0;
  $120 = ($119|0)!=(0);
  if ($120) {
   $121 = HEAP32[$9>>2]|0;
   $122 = HEAP32[$8>>2]|0;
   $123 = (_fts3StringAppend($121,$122,-1)|0);
   HEAP32[$rc>>2] = $123;
  }
 }
 $177 = HEAP32[$pMod>>2]|0;
 $178 = ((($177)) + 16|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = HEAP32[$pC>>2]|0;
 (FUNCTION_TABLE_ii[$179 & 255]($180)|0);
 $181 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $181;
 $182 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($182|0);
}
function _fts3ExprLoadDoclists($pCsr,$pnPhrase,$pnToken) {
 $pCsr = $pCsr|0;
 $pnPhrase = $pnPhrase|0;
 $pnToken = $pnToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var $sCtx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $sCtx = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pnPhrase;
 HEAP32[$2>>2] = $pnToken;
 ;HEAP32[$sCtx>>2]=0|0;HEAP32[$sCtx+4>>2]=0|0;HEAP32[$sCtx+8>>2]=0|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$sCtx>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_fts3ExprIterate($6,157,$sCtx)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = ((($sCtx)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  HEAP32[$12>>2] = $11;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $15 = ((($sCtx)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3ExprIterate($pExpr,$x,$pCtx) {
 $pExpr = $pExpr|0;
 $x = $x|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $iPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $iPhrase = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $x;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$iPhrase>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_fts3ExprIterate2($3,$iPhrase,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _fts3SnippetFindPositions($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iFirst = 0, $p = 0, $pCsr = 0, $pPhrase = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $p = sp + 16|0;
 $pPhrase = sp + 12|0;
 $pCsr = sp + 8|0;
 $rc = sp + 4|0;
 $iFirst = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + (($4*24)|0)|0);
 HEAP32[$pPhrase>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pPhrase>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$p>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3Fts3EvalPhrasePoslist($16,$17,$20,$pCsr)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$pCsr>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $39 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 HEAP32[$iFirst>>2] = 0;
 $24 = HEAP32[$pCsr>>2]|0;
 $25 = HEAP32[$pPhrase>>2]|0;
 $26 = ((($25)) + 4|0);
 HEAP32[$26>>2] = $24;
 _fts3GetDeltaPosition($pCsr,$iFirst);
 $27 = HEAP32[$pCsr>>2]|0;
 $28 = HEAP32[$pPhrase>>2]|0;
 $29 = ((($28)) + 12|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$pCsr>>2]|0;
 $31 = HEAP32[$pPhrase>>2]|0;
 $32 = ((($31)) + 20|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$iFirst>>2]|0;
 $34 = HEAP32[$pPhrase>>2]|0;
 $35 = ((($34)) + 8|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$iFirst>>2]|0;
 $37 = HEAP32[$pPhrase>>2]|0;
 $38 = ((($37)) + 16|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _fts3SnippetNextCandidate($pIter) {
 $pIter = $pIter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $i = 0, $iEnd = 0, $iStart = 0, $pPhrase = 0, $pPhrase1 = 0, $pPhrase2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $i = sp + 20|0;
 $pPhrase = sp + 16|0;
 $iStart = sp + 12|0;
 $iEnd = sp + 8|0;
 $pPhrase1 = sp + 4|0;
 $pPhrase2 = sp;
 HEAP32[$1>>2] = $pIter;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 L1: do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 20|0);
   HEAP32[$7>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $8 = HEAP32[$i>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = ((($9)) + 12|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($8|0)<($11|0);
    if (!($12)) {
     break L1;
    }
    $13 = HEAP32[$i>>2]|0;
    $14 = HEAP32[$1>>2]|0;
    $15 = ((($14)) + 16|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + (($13*24)|0)|0);
    HEAP32[$pPhrase>>2] = $17;
    $18 = HEAP32[$pPhrase>>2]|0;
    $19 = ((($18)) + 12|0);
    $20 = HEAP32[$pPhrase>>2]|0;
    $21 = ((($20)) + 8|0);
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    _fts3SnippetAdvance($19,$21,$24);
    $25 = HEAP32[$i>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[$i>>2] = $26;
   }
  } else {
   HEAP32[$iEnd>>2] = 2147483647;
   HEAP32[$i>>2] = 0;
   while(1) {
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 12|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($27|0)<($30|0);
    if (!($31)) {
     break;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = (($35) + (($32*24)|0)|0);
    HEAP32[$pPhrase1>>2] = $36;
    $37 = HEAP32[$pPhrase1>>2]|0;
    $38 = ((($37)) + 12|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)!=(0|0);
    if ($40) {
     $41 = HEAP32[$pPhrase1>>2]|0;
     $42 = ((($41)) + 8|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = HEAP32[$iEnd>>2]|0;
     $45 = ($43|0)<($44|0);
     if ($45) {
      $46 = HEAP32[$pPhrase1>>2]|0;
      $47 = ((($46)) + 8|0);
      $48 = HEAP32[$47>>2]|0;
      HEAP32[$iEnd>>2] = $48;
     }
    }
    $49 = HEAP32[$i>>2]|0;
    $50 = (($49) + 1)|0;
    HEAP32[$i>>2] = $50;
   }
   $51 = HEAP32[$iEnd>>2]|0;
   $52 = ($51|0)==(2147483647);
   if ($52) {
    HEAP32[$0>>2] = 1;
    $84 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($84|0);
   }
   $53 = HEAP32[$iEnd>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($53) - ($56))|0;
   $58 = (($57) + 1)|0;
   HEAP32[$iStart>>2] = $58;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 20|0);
   HEAP32[$60>>2] = $58;
   HEAP32[$i>>2] = 0;
   while(1) {
    $61 = HEAP32[$i>>2]|0;
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 12|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ($61|0)<($64|0);
    if (!($65)) {
     break L1;
    }
    $66 = HEAP32[$i>>2]|0;
    $67 = HEAP32[$1>>2]|0;
    $68 = ((($67)) + 16|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + (($66*24)|0)|0);
    HEAP32[$pPhrase2>>2] = $70;
    $71 = HEAP32[$pPhrase2>>2]|0;
    $72 = ((($71)) + 12|0);
    $73 = HEAP32[$pPhrase2>>2]|0;
    $74 = ((($73)) + 8|0);
    $75 = HEAP32[$iEnd>>2]|0;
    $76 = (($75) + 1)|0;
    _fts3SnippetAdvance($72,$74,$76);
    $77 = HEAP32[$pPhrase2>>2]|0;
    $78 = ((($77)) + 20|0);
    $79 = HEAP32[$pPhrase2>>2]|0;
    $80 = ((($79)) + 16|0);
    $81 = HEAP32[$iStart>>2]|0;
    _fts3SnippetAdvance($78,$80,$81);
    $82 = HEAP32[$i>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[$i>>2] = $83;
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $84 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($84|0);
}
function _fts3SnippetDetails($pIter,$0,$1,$piToken,$piScore,$pmCover,$pmHighlight) {
 $pIter = $pIter|0;
 $0 = $0|0;
 $1 = $1|0;
 $piToken = $piToken|0;
 $piScore = $piScore|0;
 $pmCover = $pmCover|0;
 $pmHighlight = $pmHighlight|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCsr = 0, $iScore = 0, $iStart = 0, $j = 0, $mCover = 0, $mHighlight = 0, $mPhrase = 0, $mPos = 0, $pCsr = 0;
 var $pPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $2 = sp + 84|0;
 $3 = sp + 32|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $7 = sp + 68|0;
 $iStart = sp + 64|0;
 $iScore = sp + 60|0;
 $i = sp + 56|0;
 $mCover = sp + 24|0;
 $mHighlight = sp + 16|0;
 $pPhrase = sp + 52|0;
 $pCsr = sp + 48|0;
 $iCsr = sp + 44|0;
 $j = sp + 40|0;
 $mPhrase = sp + 8|0;
 $mPos = sp;
 HEAP32[$2>>2] = $pIter;
 $8 = $3;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$4>>2] = $piToken;
 HEAP32[$5>>2] = $piScore;
 HEAP32[$6>>2] = $pmCover;
 HEAP32[$7>>2] = $pmHighlight;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$iStart>>2] = $14;
 HEAP32[$iScore>>2] = 0;
 $15 = $mCover;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = $mHighlight;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + (($28*24)|0)|0);
  HEAP32[$pPhrase>>2] = $32;
  $33 = HEAP32[$pPhrase>>2]|0;
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0|0);
  L4: do {
   if ($36) {
    $37 = HEAP32[$pPhrase>>2]|0;
    $38 = ((($37)) + 20|0);
    $39 = HEAP32[$38>>2]|0;
    HEAP32[$pCsr>>2] = $39;
    $40 = HEAP32[$pPhrase>>2]|0;
    $41 = ((($40)) + 16|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$iCsr>>2] = $42;
    while(1) {
     $43 = HEAP32[$iCsr>>2]|0;
     $44 = HEAP32[$iStart>>2]|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = ((($45)) + 8|0);
     $47 = HEAP32[$46>>2]|0;
     $48 = (($44) + ($47))|0;
     $49 = ($43|0)<($48|0);
     if (!($49)) {
      break L4;
     }
     $50 = HEAP32[$i>>2]|0;
     $51 = (_bitshift64Shl(1,0,($50|0))|0);
     $52 = tempRet0;
     $53 = $mPhrase;
     $54 = $53;
     HEAP32[$54>>2] = $51;
     $55 = (($53) + 4)|0;
     $56 = $55;
     HEAP32[$56>>2] = $52;
     $57 = HEAP32[$iCsr>>2]|0;
     $58 = HEAP32[$iStart>>2]|0;
     $59 = (($57) - ($58))|0;
     $60 = (_bitshift64Shl(1,0,($59|0))|0);
     $61 = tempRet0;
     $62 = $mPos;
     $63 = $62;
     HEAP32[$63>>2] = $60;
     $64 = (($62) + 4)|0;
     $65 = $64;
     HEAP32[$65>>2] = $61;
     $66 = $mCover;
     $67 = $66;
     $68 = HEAP32[$67>>2]|0;
     $69 = (($66) + 4)|0;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = $3;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = (($72) + 4)|0;
     $76 = $75;
     $77 = HEAP32[$76>>2]|0;
     $78 = $68 | $74;
     $79 = $71 | $77;
     $80 = $mPhrase;
     $81 = $80;
     $82 = HEAP32[$81>>2]|0;
     $83 = (($80) + 4)|0;
     $84 = $83;
     $85 = HEAP32[$84>>2]|0;
     $86 = $78 & $82;
     $87 = $79 & $85;
     $88 = ($86|0)!=(0);
     $89 = ($87|0)!=(0);
     $90 = $88 | $89;
     $91 = HEAP32[$iScore>>2]|0;
     if ($90) {
      $92 = (($91) + 1)|0;
      HEAP32[$iScore>>2] = $92;
     } else {
      $93 = (($91) + 1000)|0;
      HEAP32[$iScore>>2] = $93;
     }
     $94 = $mPhrase;
     $95 = $94;
     $96 = HEAP32[$95>>2]|0;
     $97 = (($94) + 4)|0;
     $98 = $97;
     $99 = HEAP32[$98>>2]|0;
     $100 = $mCover;
     $101 = $100;
     $102 = HEAP32[$101>>2]|0;
     $103 = (($100) + 4)|0;
     $104 = $103;
     $105 = HEAP32[$104>>2]|0;
     $106 = $102 | $96;
     $107 = $105 | $99;
     $108 = $mCover;
     $109 = $108;
     HEAP32[$109>>2] = $106;
     $110 = (($108) + 4)|0;
     $111 = $110;
     HEAP32[$111>>2] = $107;
     HEAP32[$j>>2] = 0;
     while(1) {
      $112 = HEAP32[$j>>2]|0;
      $113 = HEAP32[$pPhrase>>2]|0;
      $114 = HEAP32[$113>>2]|0;
      $115 = ($112|0)<($114|0);
      if (!($115)) {
       break;
      }
      $116 = $mPos;
      $117 = $116;
      $118 = HEAP32[$117>>2]|0;
      $119 = (($116) + 4)|0;
      $120 = $119;
      $121 = HEAP32[$120>>2]|0;
      $122 = HEAP32[$j>>2]|0;
      $123 = (_bitshift64Lshr(($118|0),($121|0),($122|0))|0);
      $124 = tempRet0;
      $125 = $mHighlight;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = (($125) + 4)|0;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = $127 | $123;
      $132 = $130 | $124;
      $133 = $mHighlight;
      $134 = $133;
      HEAP32[$134>>2] = $131;
      $135 = (($133) + 4)|0;
      $136 = $135;
      HEAP32[$136>>2] = $132;
      $137 = HEAP32[$j>>2]|0;
      $138 = (($137) + 1)|0;
      HEAP32[$j>>2] = $138;
     }
     $139 = HEAP32[$pCsr>>2]|0;
     $140 = HEAP8[$139>>0]|0;
     $141 = $140 << 24 >> 24;
     $142 = $141 & 254;
     $143 = (0)==($142|0);
     if ($143) {
      break L4;
     }
     _fts3GetDeltaPosition($pCsr,$iCsr);
    }
   }
  } while(0);
  $144 = HEAP32[$i>>2]|0;
  $145 = (($144) + 1)|0;
  HEAP32[$i>>2] = $145;
 }
 $146 = HEAP32[$iStart>>2]|0;
 $147 = HEAP32[$4>>2]|0;
 HEAP32[$147>>2] = $146;
 $148 = HEAP32[$iScore>>2]|0;
 $149 = HEAP32[$5>>2]|0;
 HEAP32[$149>>2] = $148;
 $150 = $mCover;
 $151 = $150;
 $152 = HEAP32[$151>>2]|0;
 $153 = (($150) + 4)|0;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = HEAP32[$6>>2]|0;
 $157 = $156;
 $158 = $157;
 HEAP32[$158>>2] = $152;
 $159 = (($157) + 4)|0;
 $160 = $159;
 HEAP32[$160>>2] = $155;
 $161 = $mHighlight;
 $162 = $161;
 $163 = HEAP32[$162>>2]|0;
 $164 = (($161) + 4)|0;
 $165 = $164;
 $166 = HEAP32[$165>>2]|0;
 $167 = HEAP32[$7>>2]|0;
 $168 = $167;
 $169 = $168;
 HEAP32[$169>>2] = $163;
 $170 = (($168) + 4)|0;
 $171 = $170;
 HEAP32[$171>>2] = $166;
 STACKTOP = sp;return;
}
function _fts3ExprLoadDoclistsCb($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pPhrase = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pPhrase = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPhrase>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$pPhrase>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13))|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3ExprIterate2($pExpr,$piPhrase,$x,$pCtx) {
 $pExpr = $pExpr|0;
 $piPhrase = $piPhrase|0;
 $x = $x|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $or$cond = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $eType = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $piPhrase;
 HEAP32[$2>>2] = $x;
 HEAP32[$3>>2] = $pCtx;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$eType>>2] = $5;
 $6 = HEAP32[$eType>>2]|0;
 $7 = ($6|0)!=(5);
 if (!($7)) {
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = (FUNCTION_TABLE_iiii[$26 & 255]($27,$29,$30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$32>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_fts3ExprIterate2($10,$11,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 $17 = HEAP32[$eType>>2]|0;
 $18 = ($17|0)!=(2);
 $or$cond = $16 & $18;
 if (!($or$cond)) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$3>>2]|0;
 $25 = (_fts3ExprIterate2($21,$22,$23,$24)|0);
 HEAP32[$rc>>2] = $25;
 $35 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3Fts3EvalPhrasePoslist($pCsr,$pExpr,$iCol,$ppOut) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $iCol = $iCol|0;
 $ppOut = $ppOut|0;
 var $$old = 0, $$old4 = 0, $$old5 = 0, $$old6 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0;
 var $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0;
 var $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0;
 var $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0;
 var $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0;
 var $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0;
 var $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0;
 var $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0;
 var $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0;
 var $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0;
 var $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0;
 var $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0;
 var $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0;
 var $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0;
 var $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDescDoclist = 0, $bEof = 0, $bEofSave = 0, $bMatch = 0, $bOr = 0, $bTreeEof = 0, $dummy = 0, $iDocid = 0, $iThis = 0, $or$cond = 0, $or$cond7 = 0, $p = 0;
 var $pIter = 0, $pNear = 0, $pPh = 0, $pPhrase = 0, $pTab = 0, $pTest = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $pPhrase = sp + 60|0;
 $pTab = sp + 56|0;
 $pIter = sp + 52|0;
 $iThis = sp + 48|0;
 $iDocid = sp;
 $rc = sp + 44|0;
 $bDescDoclist = sp + 40|0;
 $bOr = sp + 36|0;
 $bTreeEof = sp + 85|0;
 $p = sp + 32|0;
 $pNear = sp + 28|0;
 $bMatch = sp + 24|0;
 $bEofSave = sp + 20|0;
 $bEof = sp + 84|0;
 $pTest = sp + 16|0;
 $pPh = sp + 12|0;
 $dummy = sp + 8|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $ppOut;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pPhrase>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTab>>2] = $9;
 $10 = HEAP32[$4>>2]|0;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pPhrase>>2]|0;
 $12 = ((($11)) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pTab>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($13|0)<($16|0);
 if ($17) {
  $18 = HEAP32[$pPhrase>>2]|0;
  $19 = ((($18)) + 68|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = ($20|0)!=($21|0);
  if ($22) {
   HEAP32[$0>>2] = 0;
   $363 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($363|0);
  }
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 24|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = $iDocid;
 $32 = $31;
 HEAP32[$32>>2] = $27;
 $33 = (($31) + 4)|0;
 $34 = $33;
 HEAP32[$34>>2] = $30;
 $35 = HEAP32[$pPhrase>>2]|0;
 $36 = ((($35)) + 28|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$pIter>>2] = $37;
 $38 = $iDocid;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = $45;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) + 4)|0;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($40|0)!=($48|0);
 $53 = ($43|0)!=($51|0);
 $54 = $52 | $53;
 if ($54) {
  label = 6;
 } else {
  $55 = HEAP32[$2>>2]|0;
  $56 = ((($55)) + 32|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = $57&255;
  $59 = ($58|0)!=(0);
  if ($59) {
   label = 6;
  }
 }
 do {
  if ((label|0) == 6) {
   HEAP32[$rc>>2] = 0;
   $60 = HEAP32[$pTab>>2]|0;
   $61 = ((($60)) + 231|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   HEAP32[$bDescDoclist>>2] = $63;
   HEAP32[$bOr>>2] = 0;
   HEAP8[$bTreeEof>>0] = 0;
   $64 = HEAP32[$2>>2]|0;
   HEAP32[$pNear>>2] = $64;
   $65 = HEAP32[$2>>2]|0;
   $66 = ((($65)) + 8|0);
   $67 = HEAP32[$66>>2]|0;
   HEAP32[$p>>2] = $67;
   while(1) {
    $68 = HEAP32[$p>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     break;
    }
    $70 = HEAP32[$p>>2]|0;
    $71 = HEAP32[$70>>2]|0;
    $72 = ($71|0)==(4);
    if ($72) {
     HEAP32[$bOr>>2] = 1;
    }
    $73 = HEAP32[$p>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = ($74|0)==(1);
    if ($75) {
     $76 = HEAP32[$p>>2]|0;
     HEAP32[$pNear>>2] = $76;
    }
    $77 = HEAP32[$p>>2]|0;
    $78 = ((($77)) + 32|0);
    $79 = HEAP8[$78>>0]|0;
    $80 = ($79<<24>>24)!=(0);
    if ($80) {
     HEAP8[$bTreeEof>>0] = 1;
    }
    $81 = HEAP32[$p>>2]|0;
    $82 = ((($81)) + 8|0);
    $83 = HEAP32[$82>>2]|0;
    HEAP32[$p>>2] = $83;
   }
   $84 = HEAP32[$bOr>>2]|0;
   $85 = ($84|0)==(0);
   if ($85) {
    HEAP32[$0>>2] = 0;
    $363 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($363|0);
   }
   $86 = HEAP32[$pPhrase>>2]|0;
   $87 = ((($86)) + 40|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)!=(0);
   L27: do {
    if ($89) {
     $90 = HEAP32[$pNear>>2]|0;
     $91 = ((($90)) + 32|0);
     $92 = HEAP8[$91>>0]|0;
     $93 = $92&255;
     HEAP32[$bEofSave>>2] = $93;
     $94 = HEAP32[$1>>2]|0;
     $95 = HEAP32[$pNear>>2]|0;
     _fts3EvalRestart($94,$95,$rc);
     L29: while(1) {
      $$old = HEAP32[$rc>>2]|0;
      $$old4 = ($$old|0)==(0);
      if (!($$old4)) {
       break L27;
      }
      while(1) {
       $96 = HEAP32[$pNear>>2]|0;
       $97 = ((($96)) + 32|0);
       $98 = HEAP8[$97>>0]|0;
       $99 = ($98<<24>>24)!=(0);
       $100 = $99 ^ 1;
       if (!($100)) {
        break L27;
       }
       $101 = HEAP32[$1>>2]|0;
       $102 = HEAP32[$pNear>>2]|0;
       _fts3EvalNextRow($101,$102,$rc);
       $103 = HEAP32[$bEofSave>>2]|0;
       $104 = ($103|0)==(0);
       if (!($104)) {
        continue L29;
       }
       $105 = HEAP32[$pNear>>2]|0;
       $106 = ((($105)) + 24|0);
       $107 = $106;
       $108 = $107;
       $109 = HEAP32[$108>>2]|0;
       $110 = (($107) + 4)|0;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = $iDocid;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (($113) + 4)|0;
       $117 = $116;
       $118 = HEAP32[$117>>2]|0;
       $119 = ($109|0)!=($115|0);
       $120 = ($112|0)!=($118|0);
       $121 = $119 | $120;
       $122 = HEAP32[$rc>>2]|0;
       $123 = ($122|0)==(0);
       $or$cond = $121 & $123;
       if (!($or$cond)) {
        break L27;
       }
      }
     }
    }
   } while(0);
   $124 = HEAP8[$bTreeEof>>0]|0;
   $125 = ($124<<24>>24)!=(0);
   $126 = HEAP32[$rc>>2]|0;
   $127 = ($126|0)==(0);
   $or$cond7 = $125 & $127;
   L36: do {
    if ($or$cond7) {
     while(1) {
      $128 = HEAP32[$pNear>>2]|0;
      $129 = ((($128)) + 32|0);
      $130 = HEAP8[$129>>0]|0;
      $131 = ($130<<24>>24)!=(0);
      $132 = $131 ^ 1;
      if (!($132)) {
       break L36;
      }
      $133 = HEAP32[$1>>2]|0;
      $134 = HEAP32[$pNear>>2]|0;
      _fts3EvalNextRow($133,$134,$rc);
      $$old5 = HEAP32[$rc>>2]|0;
      $$old6 = ($$old5|0)==(0);
      if (!($$old6)) {
       break;
      }
     }
    }
   } while(0);
   $135 = HEAP32[$rc>>2]|0;
   $136 = ($135|0)!=(0);
   if ($136) {
    $137 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $137;
    $363 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($363|0);
   }
   HEAP32[$bMatch>>2] = 1;
   $138 = HEAP32[$pNear>>2]|0;
   HEAP32[$p>>2] = $138;
   while(1) {
    $139 = HEAP32[$p>>2]|0;
    $140 = ($139|0)!=(0|0);
    if (!($140)) {
     break;
    }
    HEAP8[$bEof>>0] = 0;
    $141 = HEAP32[$p>>2]|0;
    HEAP32[$pTest>>2] = $141;
    $142 = HEAP32[$pTest>>2]|0;
    $143 = HEAP32[$142>>2]|0;
    $144 = ($143|0)==(1);
    if ($144) {
     $145 = HEAP32[$pTest>>2]|0;
     $146 = ((($145)) + 16|0);
     $147 = HEAP32[$146>>2]|0;
     HEAP32[$pTest>>2] = $147;
    }
    $148 = HEAP32[$pTest>>2]|0;
    $149 = ((($148)) + 20|0);
    $150 = HEAP32[$149>>2]|0;
    HEAP32[$pPh>>2] = $150;
    $151 = HEAP32[$pPh>>2]|0;
    $152 = ((($151)) + 48|0);
    $153 = HEAP32[$152>>2]|0;
    HEAP32[$pIter>>2] = $153;
    $154 = HEAP32[$pPh>>2]|0;
    $155 = ((($154)) + 56|0);
    $156 = $155;
    $157 = $156;
    $158 = HEAP32[$157>>2]|0;
    $159 = (($156) + 4)|0;
    $160 = $159;
    $161 = HEAP32[$160>>2]|0;
    $162 = $iDocid;
    $163 = $162;
    HEAP32[$163>>2] = $158;
    $164 = (($162) + 4)|0;
    $165 = $164;
    HEAP32[$165>>2] = $161;
    $166 = HEAP32[$1>>2]|0;
    $167 = ((($166)) + 52|0);
    $168 = HEAP8[$167>>0]|0;
    $169 = $168&255;
    $170 = HEAP32[$bDescDoclist>>2]|0;
    $171 = ($169|0)==($170|0);
    $172 = HEAP32[$pPh>>2]|0;
    $173 = ((($172)) + 4|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = ($174|0)!=(0);
    L51: do {
     if ($171) {
      if ($175) {
       $176 = HEAP32[$pIter>>2]|0;
       $177 = HEAP32[$pPh>>2]|0;
       $178 = HEAP32[$177>>2]|0;
       $179 = HEAP32[$pPh>>2]|0;
       $180 = ((($179)) + 4|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = (($178) + ($181)|0);
       $183 = ($176>>>0)>=($182>>>0);
       $185 = $183;
      } else {
       $185 = 1;
      }
      $184 = $185&1;
      $186 = $184&255;
      HEAP8[$bEof>>0] = $186;
      while(1) {
       $187 = HEAP32[$pIter>>2]|0;
       $188 = ($187|0)==(0|0);
       if (!($188)) {
        $189 = HEAP32[$bDescDoclist>>2]|0;
        $190 = ($189|0)!=(0);
        $191 = $190 ? -1 : 1;
        $192 = ($191|0)<(0);
        $193 = $192 << 31 >> 31;
        $194 = $iDocid;
        $195 = $194;
        $196 = HEAP32[$195>>2]|0;
        $197 = (($194) + 4)|0;
        $198 = $197;
        $199 = HEAP32[$198>>2]|0;
        $200 = HEAP32[$1>>2]|0;
        $201 = ((($200)) + 32|0);
        $202 = $201;
        $203 = $202;
        $204 = HEAP32[$203>>2]|0;
        $205 = (($202) + 4)|0;
        $206 = $205;
        $207 = HEAP32[$206>>2]|0;
        $208 = (_i64Subtract(($196|0),($199|0),($204|0),($207|0))|0);
        $209 = tempRet0;
        (___muldi3(($191|0),($193|0),($208|0),($209|0))|0);
        $210 = tempRet0;
        $211 = ($210|0)<(0);
        if (!($211)) {
         break L51;
        }
       }
       $212 = HEAP8[$bEof>>0]|0;
       $213 = $212&255;
       $214 = ($213|0)==(0);
       if (!($214)) {
        break L51;
       }
       $215 = HEAP32[$bDescDoclist>>2]|0;
       $216 = HEAP32[$pPh>>2]|0;
       $217 = HEAP32[$216>>2]|0;
       $218 = HEAP32[$pPh>>2]|0;
       $219 = ((($218)) + 4|0);
       $220 = HEAP32[$219>>2]|0;
       _sqlite3Fts3DoclistNext($215,$217,$220,$pIter,$iDocid,$bEof);
      }
     } else {
      if ($175) {
       $221 = HEAP32[$pIter>>2]|0;
       $222 = ($221|0)!=(0|0);
       if ($222) {
        $223 = HEAP32[$pIter>>2]|0;
        $224 = HEAP32[$pPh>>2]|0;
        $225 = HEAP32[$224>>2]|0;
        $226 = ($223>>>0)<=($225>>>0);
        $228 = $226;
       } else {
        $228 = 0;
       }
      } else {
       $228 = 1;
      }
      $227 = $228&1;
      $229 = $227&255;
      HEAP8[$bEof>>0] = $229;
      while(1) {
       $230 = HEAP32[$pIter>>2]|0;
       $231 = ($230|0)==(0|0);
       if (!($231)) {
        $232 = HEAP32[$bDescDoclist>>2]|0;
        $233 = ($232|0)!=(0);
        $234 = $233 ? -1 : 1;
        $235 = ($234|0)<(0);
        $236 = $235 << 31 >> 31;
        $237 = $iDocid;
        $238 = $237;
        $239 = HEAP32[$238>>2]|0;
        $240 = (($237) + 4)|0;
        $241 = $240;
        $242 = HEAP32[$241>>2]|0;
        $243 = HEAP32[$1>>2]|0;
        $244 = ((($243)) + 32|0);
        $245 = $244;
        $246 = $245;
        $247 = HEAP32[$246>>2]|0;
        $248 = (($245) + 4)|0;
        $249 = $248;
        $250 = HEAP32[$249>>2]|0;
        $251 = (_i64Subtract(($239|0),($242|0),($247|0),($250|0))|0);
        $252 = tempRet0;
        $253 = (___muldi3(($234|0),($236|0),($251|0),($252|0))|0);
        $254 = tempRet0;
        $255 = ($254|0)>(0);
        $256 = ($253>>>0)>(0);
        $257 = ($254|0)==(0);
        $258 = $257 & $256;
        $259 = $255 | $258;
        if (!($259)) {
         break L51;
        }
       }
       $260 = HEAP8[$bEof>>0]|0;
       $261 = $260&255;
       $262 = ($261|0)==(0);
       if (!($262)) {
        break L51;
       }
       $263 = HEAP32[$bDescDoclist>>2]|0;
       $264 = HEAP32[$pPh>>2]|0;
       $265 = HEAP32[$264>>2]|0;
       $266 = HEAP32[$pPh>>2]|0;
       $267 = ((($266)) + 4|0);
       $268 = HEAP32[$267>>2]|0;
       _sqlite3Fts3DoclistPrev($263,$265,$268,$pIter,$iDocid,$dummy,$bEof);
      }
     }
    } while(0);
    $269 = HEAP32[$pIter>>2]|0;
    $270 = HEAP32[$pPh>>2]|0;
    $271 = ((($270)) + 48|0);
    HEAP32[$271>>2] = $269;
    $272 = $iDocid;
    $273 = $272;
    $274 = HEAP32[$273>>2]|0;
    $275 = (($272) + 4)|0;
    $276 = $275;
    $277 = HEAP32[$276>>2]|0;
    $278 = HEAP32[$pPh>>2]|0;
    $279 = ((($278)) + 56|0);
    $280 = $279;
    $281 = $280;
    HEAP32[$281>>2] = $274;
    $282 = (($280) + 4)|0;
    $283 = $282;
    HEAP32[$283>>2] = $277;
    $284 = HEAP8[$bEof>>0]|0;
    $285 = $284&255;
    $286 = ($285|0)!=(0);
    if ($286) {
     label = 50;
    } else {
     $287 = $iDocid;
     $288 = $287;
     $289 = HEAP32[$288>>2]|0;
     $290 = (($287) + 4)|0;
     $291 = $290;
     $292 = HEAP32[$291>>2]|0;
     $293 = HEAP32[$1>>2]|0;
     $294 = ((($293)) + 32|0);
     $295 = $294;
     $296 = $295;
     $297 = HEAP32[$296>>2]|0;
     $298 = (($295) + 4)|0;
     $299 = $298;
     $300 = HEAP32[$299>>2]|0;
     $301 = ($289|0)!=($297|0);
     $302 = ($292|0)!=($300|0);
     $303 = $301 | $302;
     if ($303) {
      label = 50;
     }
    }
    if ((label|0) == 50) {
     label = 0;
     HEAP32[$bMatch>>2] = 0;
    }
    $304 = HEAP32[$p>>2]|0;
    $305 = ((($304)) + 12|0);
    $306 = HEAP32[$305>>2]|0;
    HEAP32[$p>>2] = $306;
   }
   $307 = HEAP32[$bMatch>>2]|0;
   $308 = ($307|0)!=(0);
   if ($308) {
    $309 = HEAP32[$pPhrase>>2]|0;
    $310 = ((($309)) + 48|0);
    $311 = HEAP32[$310>>2]|0;
    HEAP32[$pIter>>2] = $311;
    break;
   } else {
    HEAP32[$pIter>>2] = 0;
    break;
   }
  }
 } while(0);
 $312 = HEAP32[$pIter>>2]|0;
 $313 = ($312|0)==(0|0);
 if ($313) {
  HEAP32[$0>>2] = 0;
  $363 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($363|0);
 }
 $314 = HEAP32[$pIter>>2]|0;
 $315 = HEAP8[$314>>0]|0;
 $316 = $315 << 24 >> 24;
 $317 = ($316|0)==(1);
 if ($317) {
  $318 = HEAP32[$pIter>>2]|0;
  $319 = ((($318)) + 1|0);
  HEAP32[$pIter>>2] = $319;
  $320 = HEAP32[$pIter>>2]|0;
  $321 = HEAP8[$320>>0]|0;
  $322 = $321&255;
  $323 = $322 & 128;
  $324 = ($323|0)!=(0);
  $325 = HEAP32[$pIter>>2]|0;
  if ($324) {
   $326 = (_sqlite3Fts3GetVarint32($325,$iThis)|0);
   $331 = $326;
  } else {
   $327 = HEAP8[$325>>0]|0;
   $328 = $327&255;
   HEAP32[$iThis>>2] = $328;
   $331 = 1;
  }
  $329 = HEAP32[$pIter>>2]|0;
  $330 = (($329) + ($331)|0);
  HEAP32[$pIter>>2] = $330;
 } else {
  HEAP32[$iThis>>2] = 0;
 }
 while(1) {
  $332 = HEAP32[$iThis>>2]|0;
  $333 = HEAP32[$3>>2]|0;
  $334 = ($332|0)<($333|0);
  if (!($334)) {
   break;
  }
  _fts3ColumnlistCopy(0,$pIter);
  $335 = HEAP32[$pIter>>2]|0;
  $336 = HEAP8[$335>>0]|0;
  $337 = $336 << 24 >> 24;
  $338 = ($337|0)==(0);
  if ($338) {
   label = 65;
   break;
  }
  $339 = HEAP32[$pIter>>2]|0;
  $340 = ((($339)) + 1|0);
  HEAP32[$pIter>>2] = $340;
  $341 = HEAP32[$pIter>>2]|0;
  $342 = HEAP8[$341>>0]|0;
  $343 = $342&255;
  $344 = $343 & 128;
  $345 = ($344|0)!=(0);
  $346 = HEAP32[$pIter>>2]|0;
  if ($345) {
   $347 = (_sqlite3Fts3GetVarint32($346,$iThis)|0);
   $352 = $347;
  } else {
   $348 = HEAP8[$346>>0]|0;
   $349 = $348&255;
   HEAP32[$iThis>>2] = $349;
   $352 = 1;
  }
  $350 = HEAP32[$pIter>>2]|0;
  $351 = (($350) + ($352)|0);
  HEAP32[$pIter>>2] = $351;
 }
 if ((label|0) == 65) {
  HEAP32[$0>>2] = 0;
  $363 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($363|0);
 }
 $353 = HEAP32[$pIter>>2]|0;
 $354 = HEAP8[$353>>0]|0;
 $355 = $354 << 24 >> 24;
 $356 = ($355|0)==(0);
 if ($356) {
  HEAP32[$pIter>>2] = 0;
 }
 $357 = HEAP32[$3>>2]|0;
 $358 = HEAP32[$iThis>>2]|0;
 $359 = ($357|0)==($358|0);
 $360 = HEAP32[$pIter>>2]|0;
 $361 = $359 ? $360 : 0;
 $362 = HEAP32[$4>>2]|0;
 HEAP32[$362>>2] = $361;
 HEAP32[$0>>2] = 0;
 $363 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($363|0);
}
function _fts3GetDeltaPosition($pp,$piPos) {
 $pp = $pp|0;
 $piPos = $piPos|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $piPos;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 128;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 if ($7) {
  $10 = (_sqlite3Fts3GetVarint32($9,$iVal)|0);
  $16 = $10;
 } else {
  $11 = HEAP8[$9>>0]|0;
  $12 = $11&255;
  HEAP32[$iVal>>2] = $12;
  $16 = 1;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($16)|0);
 HEAP32[$13>>2] = $15;
 $17 = HEAP32[$iVal>>2]|0;
 $18 = (($17) - 2)|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18))|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _fts3EvalRestart($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pPhrase = 0, $pToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pPhrase = sp + 8|0;
 $i = sp + 4|0;
 $pToken = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pPhrase>>2] = $10;
 $11 = HEAP32[$pPhrase>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$pPhrase>>2]|0;
  _fts3EvalInvalidatePoslist($13);
  $14 = HEAP32[$pPhrase>>2]|0;
  $15 = ((($14)) + 40|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $18 = HEAP32[$i>>2]|0;
    $19 = HEAP32[$pPhrase>>2]|0;
    $20 = ((($19)) + 64|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($18|0)<($21|0);
    if (!($22)) {
     break;
    }
    $23 = HEAP32[$i>>2]|0;
    $24 = HEAP32[$pPhrase>>2]|0;
    $25 = ((($24)) + 72|0);
    $26 = (($25) + (($23*24)|0)|0);
    HEAP32[$pToken>>2] = $26;
    $27 = HEAP32[$pToken>>2]|0;
    $28 = ((($27)) + 20|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($29|0)!=(0|0);
    if ($30) {
     $31 = HEAP32[$pToken>>2]|0;
     $32 = ((($31)) + 20|0);
     $33 = HEAP32[$32>>2]|0;
     (_sqlite3Fts3MsrIncrRestart($33)|0);
    }
    $34 = HEAP32[$i>>2]|0;
    $35 = (($34) + 1)|0;
    HEAP32[$i>>2] = $35;
   }
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$pPhrase>>2]|0;
   $38 = (_fts3EvalPhraseStart($36,0,$37)|0);
   $39 = HEAP32[$2>>2]|0;
   HEAP32[$39>>2] = $38;
  }
  $40 = HEAP32[$pPhrase>>2]|0;
  $41 = ((($40)) + 8|0);
  HEAP32[$41>>2] = 0;
  $42 = HEAP32[$pPhrase>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = $43;
  $45 = $44;
  HEAP32[$45>>2] = 0;
  $46 = (($44) + 4)|0;
  $47 = $46;
  HEAP32[$47>>2] = 0;
  $48 = HEAP32[$pPhrase>>2]|0;
  $49 = ((($48)) + 48|0);
  HEAP32[$49>>2] = 0;
 }
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = $51;
 $53 = $52;
 HEAP32[$53>>2] = 0;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = 0;
 $56 = HEAP32[$1>>2]|0;
 $57 = ((($56)) + 32|0);
 HEAP8[$57>>0] = 0;
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 33|0);
 HEAP8[$59>>0] = 0;
 $60 = HEAP32[$0>>2]|0;
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 12|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = HEAP32[$2>>2]|0;
 _fts3EvalRestart($60,$63,$64);
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 16|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = HEAP32[$2>>2]|0;
 _fts3EvalRestart($65,$68,$69);
 STACKTOP = sp;return;
}
function _sqlite3Fts3DoclistNext($bDescIdx,$aDoclist,$nDoclist,$ppIter,$piDocid,$pbEof) {
 $bDescIdx = $bDescIdx|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 $ppIter = $ppIter|0;
 $piDocid = $piDocid|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $iVar = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $p = sp + 8|0;
 $iVar = sp;
 HEAP32[$0>>2] = $bDescIdx;
 HEAP32[$1>>2] = $aDoclist;
 HEAP32[$2>>2] = $nDoclist;
 HEAP32[$3>>2] = $ppIter;
 HEAP32[$4>>2] = $piDocid;
 HEAP32[$5>>2] = $pbEof;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  $13 = (_sqlite3Fts3GetVarint($11,$12)|0);
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + ($13)|0);
  HEAP32[$p>>2] = $15;
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  STACKTOP = sp;return;
 }
 _fts3PoslistCopy(0,$p);
 while(1) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + ($17)|0);
  $20 = ($16>>>0)<($19>>>0);
  if ($20) {
   $21 = HEAP32[$p>>2]|0;
   $22 = HEAP8[$21>>0]|0;
   $23 = $22 << 24 >> 24;
   $24 = ($23|0)==(0);
   $64 = $24;
  } else {
   $64 = 0;
  }
  $25 = HEAP32[$p>>2]|0;
  if (!($64)) {
   break;
  }
  $26 = ((($25)) + 1|0);
  HEAP32[$p>>2] = $26;
 }
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + ($27)|0);
 $30 = ($25>>>0)>=($29>>>0);
 if ($30) {
  $31 = HEAP32[$5>>2]|0;
  HEAP8[$31>>0] = 1;
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  STACKTOP = sp;return;
 } else {
  $32 = HEAP32[$p>>2]|0;
  $33 = (_sqlite3Fts3GetVarint($32,$iVar)|0);
  $34 = HEAP32[$p>>2]|0;
  $35 = (($34) + ($33)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$0>>2]|0;
  $37 = ($36|0)!=(0);
  $38 = $37 ? -1 : 1;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = $iVar;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (___muldi3(($38|0),($40|0),($43|0),($46|0))|0);
  $48 = tempRet0;
  $49 = HEAP32[$4>>2]|0;
  $50 = $49;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (_i64Add(($52|0),($55|0),($47|0),($48|0))|0);
  $57 = tempRet0;
  $58 = $49;
  $59 = $58;
  HEAP32[$59>>2] = $56;
  $60 = (($58) + 4)|0;
  $61 = $60;
  HEAP32[$61>>2] = $57;
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  STACKTOP = sp;return;
 }
}
function _sqlite3Fts3MsrIncrRestart($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 HEAP32[$4>>2] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 80|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($16<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ((($20)) + 84|0);
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($22<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 88|0);
  $28 = $27;
  $29 = $28;
  HEAP32[$29>>2] = 0;
  $30 = (($28) + 4)|0;
  $31 = $30;
  HEAP32[$31>>2] = 0;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 STACKTOP = sp;return 0;
}
function _fts3SnippetAdvance($ppIter,$piIter,$iNext) {
 $ppIter = $ppIter|0;
 $piIter = $piIter|0;
 $iNext = $iNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iIter = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pIter = sp + 4|0;
 $iIter = sp;
 HEAP32[$0>>2] = $ppIter;
 HEAP32[$1>>2] = $piIter;
 HEAP32[$2>>2] = $iNext;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIter>>2] = $4;
 $5 = HEAP32[$pIter>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$iIter>>2] = $8;
 while(1) {
  $9 = HEAP32[$iIter>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = $14 & 254;
  $16 = (0)==($15|0);
  if ($16) {
   label = 5;
   break;
  }
  _fts3GetDeltaPosition($pIter,$iIter);
 }
 if ((label|0) == 5) {
  HEAP32[$iIter>>2] = -1;
  HEAP32[$pIter>>2] = 0;
 }
 $17 = HEAP32[$iIter>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$pIter>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 HEAP32[$20>>2] = $19;
 STACKTOP = sp;return;
}
function _fts3StringAppend($pStr,$zAppend,$nAppend) {
 $pStr = $pStr|0;
 $zAppend = $zAppend|0;
 $nAppend = $nAppend|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nAlloc = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $nAlloc = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $pStr;
 HEAP32[$2>>2] = $zAppend;
 HEAP32[$3>>2] = $nAppend;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_strlen($6)|0);
  HEAP32[$3>>2] = $7;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = (($12) + 1)|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($13|0)>=($16|0);
 do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$3>>2]|0;
   $22 = (($20) + ($21))|0;
   $23 = (($22) + 100)|0;
   HEAP32[$nAlloc>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$nAlloc>>2]|0;
   $27 = (_sqlite3_realloc($25,$26)|0);
   HEAP32[$zNew>>2] = $27;
   $28 = HEAP32[$zNew>>2]|0;
   $29 = ($28|0)!=(0|0);
   if ($29) {
    $30 = HEAP32[$zNew>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    HEAP32[$31>>2] = $30;
    $32 = HEAP32[$nAlloc>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 8|0);
    HEAP32[$34>>2] = $32;
    break;
   }
   HEAP32[$0>>2] = 7;
   $54 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($54|0);
  }
 } while(0);
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + ($37)|0);
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 _memcpy(($40|0),($41|0),($42|0))|0;
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 4|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + ($43))|0;
 HEAP32[$45>>2] = $47;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$1>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + ($50)|0);
 HEAP8[$53>>0] = 0;
 HEAP32[$0>>2] = 0;
 $54 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _fts3SnippetShift($pTab,$iLangid,$nSnippet,$zDoc,$nDoc,$piPos,$pHlmask) {
 $pTab = $pTab|0;
 $iLangid = $iLangid|0;
 $nSnippet = $nSnippet|0;
 $zDoc = $zDoc|0;
 $nDoc = $nDoc|0;
 $piPos = $piPos|0;
 $pHlmask = $pHlmask|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $DUMMY1 = 0, $DUMMY2 = 0, $DUMMY3 = 0, $ZDUMMY = 0, $hlmask = 0, $iCurrent = 0, $nDesired = 0;
 var $nLeft = 0, $nRight = 0, $nShift = 0, $or$cond = 0, $pC = 0, $pMod = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $hlmask = sp;
 $nLeft = sp + 52|0;
 $nRight = sp + 48|0;
 $nDesired = sp + 44|0;
 $nShift = sp + 40|0;
 $iCurrent = sp + 36|0;
 $rc = sp + 32|0;
 $pMod = sp + 28|0;
 $pC = sp + 24|0;
 $ZDUMMY = sp + 20|0;
 $DUMMY1 = sp + 16|0;
 $DUMMY2 = sp + 12|0;
 $DUMMY3 = sp + 8|0;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $nSnippet;
 HEAP32[$4>>2] = $zDoc;
 HEAP32[$5>>2] = $nDoc;
 HEAP32[$6>>2] = $piPos;
 HEAP32[$7>>2] = $pHlmask;
 $8 = HEAP32[$7>>2]|0;
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $hlmask;
 $16 = $15;
 HEAP32[$16>>2] = $11;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 $19 = $hlmask;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($21|0)!=(0);
 $26 = ($24|0)!=(0);
 $27 = $25 | $26;
 do {
  if ($27) {
   HEAP32[$nLeft>>2] = 0;
   while(1) {
    $28 = $hlmask;
    $29 = $28;
    $30 = HEAP32[$29>>2]|0;
    $31 = (($28) + 4)|0;
    $32 = $31;
    $33 = HEAP32[$32>>2]|0;
    $34 = HEAP32[$nLeft>>2]|0;
    $35 = (_bitshift64Shl(1,0,($34|0))|0);
    $36 = tempRet0;
    $37 = $30 & $35;
    $38 = $33 & $36;
    $39 = ($37|0)!=(0);
    $40 = ($38|0)!=(0);
    $41 = $39 | $40;
    $42 = $41 ^ 1;
    if (!($42)) {
     break;
    }
    $43 = HEAP32[$nLeft>>2]|0;
    $44 = (($43) + 1)|0;
    HEAP32[$nLeft>>2] = $44;
   }
   HEAP32[$nRight>>2] = 0;
   while(1) {
    $45 = $hlmask;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($45) + 4)|0;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$3>>2]|0;
    $52 = (($51) - 1)|0;
    $53 = HEAP32[$nRight>>2]|0;
    $54 = (($52) - ($53))|0;
    $55 = (_bitshift64Shl(1,0,($54|0))|0);
    $56 = tempRet0;
    $57 = $47 & $55;
    $58 = $50 & $56;
    $59 = ($57|0)!=(0);
    $60 = ($58|0)!=(0);
    $61 = $59 | $60;
    $62 = $61 ^ 1;
    if (!($62)) {
     break;
    }
    $63 = HEAP32[$nRight>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[$nRight>>2] = $64;
   }
   $65 = HEAP32[$nLeft>>2]|0;
   $66 = HEAP32[$nRight>>2]|0;
   $67 = (($65) - ($66))|0;
   $68 = (($67|0) / 2)&-1;
   HEAP32[$nDesired>>2] = $68;
   $69 = HEAP32[$nDesired>>2]|0;
   $70 = ($69|0)>(0);
   if ($70) {
    HEAP32[$iCurrent>>2] = 0;
    $71 = HEAP32[$1>>2]|0;
    $72 = ((($71)) + 36|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    HEAP32[$pMod>>2] = $74;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 36|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$2>>2]|0;
    $79 = HEAP32[$4>>2]|0;
    $80 = HEAP32[$5>>2]|0;
    $81 = (_sqlite3Fts3OpenTokenizer($77,$78,$79,$80,$pC)|0);
    HEAP32[$rc>>2] = $81;
    $82 = HEAP32[$rc>>2]|0;
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $84;
     $132 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($132|0);
    }
    while(1) {
     $85 = HEAP32[$rc>>2]|0;
     $86 = ($85|0)==(0);
     if (!($86)) {
      break;
     }
     $87 = HEAP32[$iCurrent>>2]|0;
     $88 = HEAP32[$3>>2]|0;
     $89 = HEAP32[$nDesired>>2]|0;
     $90 = (($88) + ($89))|0;
     $91 = ($87|0)<($90|0);
     if (!($91)) {
      break;
     }
     HEAP32[$DUMMY1>>2] = 0;
     HEAP32[$DUMMY2>>2] = 0;
     HEAP32[$DUMMY3>>2] = 0;
     $92 = HEAP32[$pMod>>2]|0;
     $93 = ((($92)) + 20|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = HEAP32[$pC>>2]|0;
     $96 = (FUNCTION_TABLE_iiiiiii[$94 & 255]($95,$ZDUMMY,$DUMMY1,$DUMMY2,$DUMMY3,$iCurrent)|0);
     HEAP32[$rc>>2] = $96;
    }
    $97 = HEAP32[$pMod>>2]|0;
    $98 = ((($97)) + 16|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = HEAP32[$pC>>2]|0;
    (FUNCTION_TABLE_ii[$99 & 255]($100)|0);
    $101 = HEAP32[$rc>>2]|0;
    $102 = ($101|0)!=(0);
    $103 = HEAP32[$rc>>2]|0;
    $104 = ($103|0)!=(101);
    $or$cond = $102 & $104;
    $105 = HEAP32[$rc>>2]|0;
    if ($or$cond) {
     HEAP32[$0>>2] = $105;
     $132 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($132|0);
    } else {
     $106 = ($105|0)==(101);
     $107 = $106&1;
     $108 = HEAP32[$iCurrent>>2]|0;
     $109 = (($107) + ($108))|0;
     $110 = HEAP32[$3>>2]|0;
     $111 = (($109) - ($110))|0;
     HEAP32[$nShift>>2] = $111;
     $112 = HEAP32[$nShift>>2]|0;
     $113 = ($112|0)>(0);
     if (!($113)) {
      break;
     }
     $114 = HEAP32[$nShift>>2]|0;
     $115 = HEAP32[$6>>2]|0;
     $116 = HEAP32[$115>>2]|0;
     $117 = (($116) + ($114))|0;
     HEAP32[$115>>2] = $117;
     $118 = $hlmask;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = (($118) + 4)|0;
     $122 = $121;
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nShift>>2]|0;
     $125 = (_bitshift64Lshr(($120|0),($123|0),($124|0))|0);
     $126 = tempRet0;
     $127 = HEAP32[$7>>2]|0;
     $128 = $127;
     $129 = $128;
     HEAP32[$129>>2] = $125;
     $130 = (($128) + 4)|0;
     $131 = $130;
     HEAP32[$131>>2] = $126;
     break;
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $132 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _sqlite3Fts3Offsets($pCtx,$pCsr) {
 $pCtx = $pCtx|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $NDUMMY = 0, $ZDUMMY = 0, $aBuffer = 0, $i = 0, $iCol = 0, $iCurrent = 0, $iEnd = 0, $iMinPos = 0, $iStart = 0, $nDoc = 0, $nToken = 0, $pC = 0, $pMod = 0, $pT = 0, $pTab = 0, $pTerm = 0, $rc = 0, $res = 0, $sCtx = 0;
 var $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zDoc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer = sp + 32|0;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $pTab = sp + 124|0;
 $pMod = sp + 120|0;
 $rc = sp + 116|0;
 $nToken = sp + 112|0;
 $iCol = sp + 108|0;
 $res = sp + 96|0;
 $sCtx = sp;
 $pC = sp + 92|0;
 $ZDUMMY = sp + 88|0;
 $NDUMMY = sp + 84|0;
 $iStart = sp + 80|0;
 $iEnd = sp + 76|0;
 $iCurrent = sp + 72|0;
 $zDoc = sp + 68|0;
 $nDoc = sp + 64|0;
 $i = sp + 60|0;
 $iMinPos = sp + 56|0;
 $pTerm = sp + 52|0;
 $pT = sp + 48|0;
 $aBuffer = sp + 136|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pCsr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTab>>2] = $3;
 $4 = HEAP32[$pTab>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pMod>>2] = $7;
 ;HEAP32[$res>>2]=0|0;HEAP32[$res+4>>2]=0|0;HEAP32[$res+8>>2]=0|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($12,17532,0,0);
  STACKTOP = sp;return;
 }
 ;HEAP32[$sCtx>>2]=0|0;HEAP32[$sCtx+4>>2]=0|0;HEAP32[$sCtx+8>>2]=0|0;HEAP32[$sCtx+12>>2]=0|0;HEAP32[$sCtx+16>>2]=0|0;HEAP32[$sCtx+20>>2]=0|0;HEAP32[$sCtx+24>>2]=0|0;HEAP32[$sCtx+28>>2]=0|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_fts3ExprLoadDoclists($13,0,$nToken)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)!=(0);
 L5: do {
  if (!($16)) {
   $17 = HEAP32[$nToken>>2]|0;
   $18 = ($17*12)|0;
   $19 = (_sqlite3_malloc($18)|0);
   $20 = ((($sCtx)) + 24|0);
   HEAP32[$20>>2] = $19;
   $21 = ((($sCtx)) + 24|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (0|0)==($22|0);
   if ($23) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 32|0);
   $26 = $25;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = ((($sCtx)) + 16|0);
   $33 = $32;
   $34 = $33;
   HEAP32[$34>>2] = $28;
   $35 = (($33) + 4)|0;
   $36 = $35;
   HEAP32[$36>>2] = $31;
   $37 = HEAP32[$1>>2]|0;
   HEAP32[$sCtx>>2] = $37;
   HEAP32[$iCol>>2] = 0;
   while(1) {
    $38 = HEAP32[$iCol>>2]|0;
    $39 = HEAP32[$pTab>>2]|0;
    $40 = ((($39)) + 24|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = ($38|0)<($41|0);
    if (!($42)) {
     break L5;
    }
    HEAP32[$NDUMMY>>2] = 0;
    HEAP32[$iStart>>2] = 0;
    HEAP32[$iEnd>>2] = 0;
    HEAP32[$iCurrent>>2] = 0;
    $43 = HEAP32[$iCol>>2]|0;
    $44 = ((($sCtx)) + 4|0);
    HEAP32[$44>>2] = $43;
    $45 = ((($sCtx)) + 8|0);
    HEAP32[$45>>2] = 0;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 12|0);
    $48 = HEAP32[$47>>2]|0;
    (_fts3ExprIterate($48,158,$sCtx)|0);
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 8|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$iCol>>2]|0;
    $53 = (($52) + 1)|0;
    $54 = (_sqlite3_column_text($51,$53)|0);
    HEAP32[$zDoc>>2] = $54;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 8|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = HEAP32[$iCol>>2]|0;
    $59 = (($58) + 1)|0;
    $60 = (_sqlite3_column_bytes($57,$59)|0);
    HEAP32[$nDoc>>2] = $60;
    $61 = HEAP32[$zDoc>>2]|0;
    $62 = ($61|0)==(0|0);
    if ($62) {
     $63 = HEAP32[$1>>2]|0;
     $64 = ((($63)) + 8|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$iCol>>2]|0;
     $67 = (($66) + 1)|0;
     $68 = (_sqlite3_column_type($65,$67)|0);
     $69 = ($68|0)==(5);
     if (!($69)) {
      break;
     }
    } else {
     $70 = HEAP32[$pTab>>2]|0;
     $71 = ((($70)) + 36|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = HEAP32[$1>>2]|0;
     $74 = ((($73)) + 16|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = HEAP32[$zDoc>>2]|0;
     $77 = HEAP32[$nDoc>>2]|0;
     $78 = (_sqlite3Fts3OpenTokenizer($72,$75,$76,$77,$pC)|0);
     HEAP32[$rc>>2] = $78;
     $79 = HEAP32[$rc>>2]|0;
     $80 = ($79|0)!=(0);
     if ($80) {
      break L5;
     }
     $81 = HEAP32[$pMod>>2]|0;
     $82 = ((($81)) + 20|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = HEAP32[$pC>>2]|0;
     $85 = (FUNCTION_TABLE_iiiiiii[$83 & 255]($84,$ZDUMMY,$NDUMMY,$iStart,$iEnd,$iCurrent)|0);
     HEAP32[$rc>>2] = $85;
     while(1) {
      $86 = HEAP32[$rc>>2]|0;
      $87 = ($86|0)==(0);
      if (!($87)) {
       break;
      }
      HEAP32[$iMinPos>>2] = 2147483647;
      HEAP32[$pTerm>>2] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $88 = HEAP32[$i>>2]|0;
       $89 = HEAP32[$nToken>>2]|0;
       $90 = ($88|0)<($89|0);
       if (!($90)) {
        break;
       }
       $91 = HEAP32[$i>>2]|0;
       $92 = ((($sCtx)) + 24|0);
       $93 = HEAP32[$92>>2]|0;
       $94 = (($93) + (($91*12)|0)|0);
       HEAP32[$pT>>2] = $94;
       $95 = HEAP32[$pT>>2]|0;
       $96 = HEAP32[$95>>2]|0;
       $97 = ($96|0)!=(0|0);
       if ($97) {
        $98 = HEAP32[$pT>>2]|0;
        $99 = ((($98)) + 4|0);
        $100 = HEAP32[$99>>2]|0;
        $101 = HEAP32[$pT>>2]|0;
        $102 = ((($101)) + 8|0);
        $103 = HEAP32[$102>>2]|0;
        $104 = (($100) - ($103))|0;
        $105 = HEAP32[$iMinPos>>2]|0;
        $106 = ($104|0)<($105|0);
        if ($106) {
         $107 = HEAP32[$pT>>2]|0;
         $108 = ((($107)) + 4|0);
         $109 = HEAP32[$108>>2]|0;
         $110 = HEAP32[$pT>>2]|0;
         $111 = ((($110)) + 8|0);
         $112 = HEAP32[$111>>2]|0;
         $113 = (($109) - ($112))|0;
         HEAP32[$iMinPos>>2] = $113;
         $114 = HEAP32[$pT>>2]|0;
         HEAP32[$pTerm>>2] = $114;
        }
       }
       $115 = HEAP32[$i>>2]|0;
       $116 = (($115) + 1)|0;
       HEAP32[$i>>2] = $116;
      }
      $117 = HEAP32[$pTerm>>2]|0;
      $118 = ($117|0)!=(0|0);
      if (!($118)) {
       HEAP32[$rc>>2] = 101;
       continue;
      }
      $119 = HEAP32[$pTerm>>2]|0;
      $120 = HEAP32[$119>>2]|0;
      $121 = HEAP8[$120>>0]|0;
      $122 = $121 << 24 >> 24;
      $123 = 254 & $122;
      $124 = (0)==($123|0);
      $125 = HEAP32[$pTerm>>2]|0;
      if ($124) {
       HEAP32[$125>>2] = 0;
      } else {
       $126 = HEAP32[$pTerm>>2]|0;
       $127 = ((($126)) + 4|0);
       _fts3GetDeltaPosition($125,$127);
      }
      while(1) {
       $128 = HEAP32[$rc>>2]|0;
       $129 = ($128|0)==(0);
       if (!($129)) {
        break;
       }
       $130 = HEAP32[$iCurrent>>2]|0;
       $131 = HEAP32[$iMinPos>>2]|0;
       $132 = ($130|0)<($131|0);
       if (!($132)) {
        break;
       }
       $133 = HEAP32[$pMod>>2]|0;
       $134 = ((($133)) + 20|0);
       $135 = HEAP32[$134>>2]|0;
       $136 = HEAP32[$pC>>2]|0;
       $137 = (FUNCTION_TABLE_iiiiiii[$135 & 255]($136,$ZDUMMY,$NDUMMY,$iStart,$iEnd,$iCurrent)|0);
       HEAP32[$rc>>2] = $137;
      }
      $138 = HEAP32[$rc>>2]|0;
      $139 = ($138|0)==(0);
      if ($139) {
       $140 = HEAP32[$iCol>>2]|0;
       $141 = HEAP32[$pTerm>>2]|0;
       $142 = ((($sCtx)) + 24|0);
       $143 = HEAP32[$142>>2]|0;
       $144 = $141;
       $145 = $143;
       $146 = (($144) - ($145))|0;
       $147 = (($146|0) / 12)&-1;
       $148 = HEAP32[$iStart>>2]|0;
       $149 = HEAP32[$iEnd>>2]|0;
       $150 = HEAP32[$iStart>>2]|0;
       $151 = (($149) - ($150))|0;
       HEAP32[$vararg_buffer>>2] = $140;
       $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
       HEAP32[$vararg_ptr1>>2] = $147;
       $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
       HEAP32[$vararg_ptr2>>2] = $148;
       $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
       HEAP32[$vararg_ptr3>>2] = $151;
       (_sqlite3_snprintf(64,$aBuffer,44683,$vararg_buffer)|0);
       $152 = (_fts3StringAppend($res,$aBuffer,-1)|0);
       HEAP32[$rc>>2] = $152;
       continue;
      }
      $153 = HEAP32[$rc>>2]|0;
      $154 = ($153|0)==(101);
      if (!($154)) {
       continue;
      }
      $155 = HEAP32[$pTab>>2]|0;
      $156 = ((($155)) + 40|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = ($157|0)==(0|0);
      if (!($158)) {
       continue;
      }
      HEAP32[$rc>>2] = 267;
     }
     $159 = HEAP32[$rc>>2]|0;
     $160 = ($159|0)==(101);
     if ($160) {
      HEAP32[$rc>>2] = 0;
     }
     $161 = HEAP32[$pMod>>2]|0;
     $162 = ((($161)) + 16|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = HEAP32[$pC>>2]|0;
     (FUNCTION_TABLE_ii[$163 & 255]($164)|0);
     $165 = HEAP32[$rc>>2]|0;
     $166 = ($165|0)!=(0);
     if ($166) {
      break L5;
     }
    }
    $167 = HEAP32[$iCol>>2]|0;
    $168 = (($167) + 1)|0;
    HEAP32[$iCol>>2] = $168;
   }
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $169 = ((($sCtx)) + 24|0);
 $170 = HEAP32[$169>>2]|0;
 _sqlite3_free($170);
 $171 = HEAP32[$pTab>>2]|0;
 _sqlite3Fts3SegmentsClose($171);
 $172 = HEAP32[$rc>>2]|0;
 $173 = ($172|0)!=(0);
 $174 = HEAP32[$0>>2]|0;
 if ($173) {
  $175 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($174,$175);
  $176 = HEAP32[$res>>2]|0;
  _sqlite3_free($176);
  STACKTOP = sp;return;
 } else {
  $177 = HEAP32[$res>>2]|0;
  $178 = ((($res)) + 4|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = (($179) - 1)|0;
  _sqlite3_result_text($174,$177,$180,142);
  STACKTOP = sp;return;
 }
}
function _fts3ExprTermOffsetInit($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iPos = 0, $iTerm = 0, $nTerm = 0, $p = 0, $pList = 0, $pT = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $p = sp + 24|0;
 $nTerm = sp + 20|0;
 $iTerm = sp + 16|0;
 $pList = sp + 12|0;
 $iPos = sp + 8|0;
 $rc = sp + 4|0;
 $pT = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 HEAP32[$iPos>>2] = 0;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3Fts3EvalPhrasePoslist($5,$6,$9,$pList)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 64|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$nTerm>>2] = $15;
 $16 = HEAP32[$pList>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  _fts3GetDeltaPosition($pList,$iPos);
 }
 HEAP32[$iTerm>>2] = 0;
 while(1) {
  $18 = HEAP32[$iTerm>>2]|0;
  $19 = HEAP32[$nTerm>>2]|0;
  $20 = ($18|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$p>>2]|0;
  $22 = ((($21)) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$22>>2] = $24;
  $25 = HEAP32[$p>>2]|0;
  $26 = ((($25)) + 24|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($23*12)|0)|0);
  HEAP32[$pT>>2] = $28;
  $29 = HEAP32[$nTerm>>2]|0;
  $30 = HEAP32[$iTerm>>2]|0;
  $31 = (($29) - ($30))|0;
  $32 = (($31) - 1)|0;
  $33 = HEAP32[$pT>>2]|0;
  $34 = ((($33)) + 8|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$pList>>2]|0;
  $36 = HEAP32[$pT>>2]|0;
  HEAP32[$36>>2] = $35;
  $37 = HEAP32[$iPos>>2]|0;
  $38 = HEAP32[$pT>>2]|0;
  $39 = ((($38)) + 4|0);
  HEAP32[$39>>2] = $37;
  $40 = HEAP32[$iTerm>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$iTerm>>2] = $41;
 }
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3Fts3Optimize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3_exec($3,44734,0,0,0)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = (_fts3DoOptimize($7,1)|0);
   HEAP32[$rc>>2] = $8;
   $9 = HEAP32[$rc>>2]|0;
   $10 = ($9|0)==(0);
   $11 = HEAP32[$rc>>2]|0;
   $12 = ($11|0)==(101);
   $or$cond = $10 | $12;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   if (!($or$cond)) {
    (_sqlite3_exec($15,44762,0,0,0)|0);
    $20 = HEAP32[$0>>2]|0;
    $21 = ((($20)) + 12|0);
    $22 = HEAP32[$21>>2]|0;
    (_sqlite3_exec($22,44749,0,0,0)|0);
    break;
   }
   $16 = (_sqlite3_exec($15,44749,0,0,0)|0);
   HEAP32[$rc2>>2] = $16;
   $17 = HEAP32[$rc2>>2]|0;
   $18 = ($17|0)!=(0);
   if ($18) {
    $19 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $19;
   }
  }
 } while(0);
 $23 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegmentsClose($23);
 $24 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3Fts3Matchinfo($pContext,$pCsr,$zArg) {
 $pContext = $pContext|0;
 $pCsr = $pCsr|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, $zFormat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTab = sp + 4|0;
 $zFormat = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  HEAP32[$zFormat>>2] = $7;
 } else {
  HEAP32[$zFormat>>2] = 44779;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 $12 = HEAP32[$0>>2]|0;
 if ($11) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$zFormat>>2]|0;
  _fts3GetMatchinfo($12,$13,$14);
  $15 = HEAP32[$pTab>>2]|0;
  _sqlite3Fts3SegmentsClose($15);
  STACKTOP = sp;return;
 } else {
  _sqlite3_result_blob($12,17532,0,0);
  STACKTOP = sp;return;
 }
}
function _fts3GetMatchinfo($pCtx,$pCsr,$zArg) {
 $pCtx = $pCtx|0;
 $pCsr = $pCsr|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOut = 0;
 var $bGlobal = 0, $i = 0, $n = 0, $nMatchinfo = 0, $pTab = 0, $rc = 0, $sInfo = 0, $xDestroyOut = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $sInfo = sp;
 $pTab = sp + 64|0;
 $rc = sp + 60|0;
 $bGlobal = sp + 56|0;
 $aOut = sp + 52|0;
 $xDestroyOut = sp + 48|0;
 $nMatchinfo = sp + 44|0;
 $i = sp + 40|0;
 $zErr = sp + 36|0;
 $n = sp + 32|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bGlobal>>2] = 0;
 HEAP32[$aOut>>2] = 0;
 HEAP32[$xDestroyOut>>2] = 0;
 ;HEAP32[$sInfo>>2]=0|0;HEAP32[$sInfo+4>>2]=0|0;HEAP32[$sInfo+8>>2]=0|0;HEAP32[$sInfo+12>>2]=0|0;HEAP32[$sInfo+16>>2]=0|0;HEAP32[$sInfo+20>>2]=0|0;HEAP32[$sInfo+24>>2]=0|0;HEAP32[$sInfo+28>>2]=0|0;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$sInfo>>2] = $5;
 $6 = HEAP32[$pTab>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($sInfo)) + 4|0);
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 92|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 92|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = (_strcmp($18,$19)|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 92|0);
   $24 = HEAP32[$23>>2]|0;
   _sqlite3Fts3MIBufferFree($24);
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 92|0);
   HEAP32[$26>>2] = 0;
  }
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 92|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)==(0|0);
 if ($30) {
  HEAP32[$nMatchinfo>>2] = 0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_fts3ExprPhraseCount($33)|0);
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 20|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 20|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ((($sInfo)) + 8|0);
  HEAP32[$40>>2] = $39;
  HEAP32[$i>>2] = 0;
  while(1) {
   $41 = HEAP32[$i>>2]|0;
   $42 = HEAP32[$2>>2]|0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = ($44<<24>>24)!=(0);
   if (!($45)) {
    break;
   }
   HEAP32[$zErr>>2] = 0;
   $46 = HEAP32[$pTab>>2]|0;
   $47 = HEAP32[$i>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = (($48) + ($47)|0);
   $50 = HEAP8[$49>>0]|0;
   $51 = (_fts3MatchinfoCheck($46,$50,$zErr)|0);
   $52 = ($51|0)!=(0);
   if ($52) {
    label = 8;
    break;
   }
   $56 = HEAP32[$i>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = (($57) + ($56)|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = (_fts3MatchinfoSize($sInfo,$59)|0);
   $61 = HEAP32[$nMatchinfo>>2]|0;
   $62 = (($61) + ($60))|0;
   HEAP32[$nMatchinfo>>2] = $62;
   $63 = HEAP32[$i>>2]|0;
   $64 = (($63) + 1)|0;
   HEAP32[$i>>2] = $64;
  }
  if ((label|0) == 8) {
   $53 = HEAP32[$0>>2]|0;
   $54 = HEAP32[$zErr>>2]|0;
   _sqlite3_result_error($53,$54,-1);
   $55 = HEAP32[$zErr>>2]|0;
   _sqlite3_free($55);
   STACKTOP = sp;return;
  }
  $65 = HEAP32[$nMatchinfo>>2]|0;
  $66 = HEAP32[$2>>2]|0;
  $67 = (_fts3MIBufferNew($65,$66)|0);
  $68 = HEAP32[$1>>2]|0;
  $69 = ((($68)) + 92|0);
  HEAP32[$69>>2] = $67;
  $70 = HEAP32[$1>>2]|0;
  $71 = ((($70)) + 92|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)!=(0|0);
  if (!($73)) {
   HEAP32[$rc>>2] = 7;
  }
  $74 = HEAP32[$1>>2]|0;
  $75 = ((($74)) + 88|0);
  HEAP32[$75>>2] = 1;
  HEAP32[$bGlobal>>2] = 1;
 }
 $76 = HEAP32[$rc>>2]|0;
 $77 = ($76|0)==(0);
 if ($77) {
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 92|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = (_fts3MIBufferAlloc($80,$aOut)|0);
  HEAP32[$xDestroyOut>>2] = $81;
  $82 = HEAP32[$xDestroyOut>>2]|0;
  $83 = ($82|0)==(0|0);
  if ($83) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $84 = HEAP32[$rc>>2]|0;
 $85 = ($84|0)==(0);
 if ($85) {
  $86 = HEAP32[$aOut>>2]|0;
  $87 = ((($sInfo)) + 28|0);
  HEAP32[$87>>2] = $86;
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 20|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = ((($sInfo)) + 8|0);
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP32[$bGlobal>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = (_fts3MatchinfoValues($92,$93,$sInfo,$94)|0);
  HEAP32[$rc>>2] = $95;
  $96 = HEAP32[$bGlobal>>2]|0;
  $97 = ($96|0)!=(0);
  if ($97) {
   $98 = HEAP32[$1>>2]|0;
   $99 = ((($98)) + 92|0);
   $100 = HEAP32[$99>>2]|0;
   _fts3MIBufferSetGlobal($100);
  }
 }
 $101 = HEAP32[$rc>>2]|0;
 $102 = ($101|0)!=(0);
 if (!($102)) {
  $109 = HEAP32[$1>>2]|0;
  $110 = ((($109)) + 92|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ((($111)) + 4|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = $113<<2;
  HEAP32[$n>>2] = $114;
  $115 = HEAP32[$0>>2]|0;
  $116 = HEAP32[$aOut>>2]|0;
  $117 = HEAP32[$n>>2]|0;
  $118 = HEAP32[$xDestroyOut>>2]|0;
  _sqlite3_result_blob($115,$116,$117,$118);
  STACKTOP = sp;return;
 }
 $103 = HEAP32[$0>>2]|0;
 $104 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($103,$104);
 $105 = HEAP32[$xDestroyOut>>2]|0;
 $106 = ($105|0)!=(0|0);
 if (!($106)) {
  STACKTOP = sp;return;
 }
 $107 = HEAP32[$xDestroyOut>>2]|0;
 $108 = HEAP32[$aOut>>2]|0;
 FUNCTION_TABLE_vi[$107 & 255]($108);
 STACKTOP = sp;return;
}
function _fts3ExprPhraseCount($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $nPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nPhrase = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$nPhrase>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 (_fts3ExprIterate($1,159,$nPhrase)|0);
 $2 = HEAP32[$nPhrase>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _fts3MatchinfoCheck($pTab,$cArg,$pzErr) {
 $pTab = $pTab|0;
 $cArg = $cArg|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 16|0;
 $3 = sp + 4|0;
 HEAP32[$1>>2] = $pTab;
 HEAP8[$2>>0] = $cArg;
 HEAP32[$3>>2] = $pzErr;
 $4 = HEAP8[$2>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(112);
 do {
  if (!($6)) {
   $7 = HEAP8[$2>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)==(99);
   if (!($9)) {
    $10 = HEAP8[$2>>0]|0;
    $11 = $10 << 24 >> 24;
    $12 = ($11|0)==(110);
    if ($12) {
     $13 = HEAP32[$1>>2]|0;
     $14 = ((($13)) + 228|0);
     $15 = HEAP8[$14>>0]|0;
     $16 = $15&255;
     $17 = ($16|0)!=(0);
     if ($17) {
      break;
     }
    }
    $18 = HEAP8[$2>>0]|0;
    $19 = $18 << 24 >> 24;
    $20 = ($19|0)==(97);
    if ($20) {
     $21 = HEAP32[$1>>2]|0;
     $22 = ((($21)) + 228|0);
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)!=(0);
     if ($25) {
      break;
     }
    }
    $26 = HEAP8[$2>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)==(108);
    if ($28) {
     $29 = HEAP32[$1>>2]|0;
     $30 = ((($29)) + 230|0);
     $31 = HEAP8[$30>>0]|0;
     $32 = $31&255;
     $33 = ($32|0)!=(0);
     if ($33) {
      break;
     }
    }
    $34 = HEAP8[$2>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(115);
    if (!($36)) {
     $37 = HEAP8[$2>>0]|0;
     $38 = $37 << 24 >> 24;
     $39 = ($38|0)==(120);
     if (!($39)) {
      $40 = HEAP8[$2>>0]|0;
      $41 = $40 << 24 >> 24;
      $42 = ($41|0)==(121);
      if (!($42)) {
       $43 = HEAP8[$2>>0]|0;
       $44 = $43 << 24 >> 24;
       $45 = ($44|0)==(98);
       if (!($45)) {
        $46 = HEAP32[$3>>2]|0;
        $47 = HEAP8[$2>>0]|0;
        $48 = $47 << 24 >> 24;
        HEAP32[$vararg_buffer>>2] = $48;
        _sqlite3Fts3ErrMsg($46,44783,$vararg_buffer);
        HEAP32[$0>>2] = 1;
        $49 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($49|0);
       }
      }
     }
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _fts3MatchinfoSize($pInfo,$cArg) {
 $pInfo = $pInfo|0;
 $cArg = $cArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $nVal = sp;
 HEAP32[$0>>2] = $pInfo;
 HEAP8[$1>>0] = $cArg;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2 << 24 >> 24;
 switch ($3|0) {
 case 99: case 112: case 110:  {
  HEAP32[$nVal>>2] = 1;
  break;
 }
 case 115: case 108: case 97:  {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$nVal>>2] = $6;
  break;
 }
 case 121:  {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = Math_imul($9, $12)|0;
  HEAP32[$nVal>>2] = $13;
  break;
 }
 case 98:  {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 31)|0;
  $21 = (($20|0) / 32)&-1;
  $22 = Math_imul($16, $21)|0;
  HEAP32[$nVal>>2] = $22;
  break;
 }
 default: {
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = Math_imul($25, $28)|0;
  $30 = ($29*3)|0;
  HEAP32[$nVal>>2] = $30;
 }
 }
 $31 = HEAP32[$nVal>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _fts3MIBufferNew($nElem,$zMatchinfo) {
 $nElem = $nElem|0;
 $zMatchinfo = $zMatchinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $nStr = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRet = sp + 8|0;
 $nByte = sp + 4|0;
 $nStr = sp;
 HEAP32[$0>>2] = $nElem;
 HEAP32[$1>>2] = $zMatchinfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = $2<<1;
 $4 = (($3) + 1)|0;
 $5 = $4<<2;
 $6 = (($5) + 20)|0;
 HEAP32[$nByte>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_strlen($7)|0);
 HEAP32[$nStr>>2] = $8;
 $9 = HEAP32[$nByte>>2]|0;
 $10 = HEAP32[$nStr>>2]|0;
 $11 = (($9) + ($10))|0;
 $12 = (($11) + 1)|0;
 $13 = (_sqlite3_malloc($12)|0);
 HEAP32[$pRet>>2] = $13;
 $14 = HEAP32[$pRet>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $54 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $16 = HEAP32[$pRet>>2]|0;
 $17 = HEAP32[$nByte>>2]|0;
 _memset(($16|0),0,($17|0))|0;
 $18 = HEAP32[$pRet>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$pRet>>2]|0;
 $22 = $20;
 $23 = $21;
 $24 = (($22) - ($23))|0;
 $25 = HEAP32[$pRet>>2]|0;
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$pRet>>2]|0;
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 1)|0;
 $32 = $31<<2;
 $33 = (($29) + ($32))|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (1 + ($34))|0;
 $36 = HEAP32[$pRet>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = (($37) + ($35<<2)|0);
 HEAP32[$38>>2] = $33;
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$pRet>>2]|0;
 $41 = ((($40)) + 4|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pRet>>2]|0;
 $43 = HEAP32[$nByte>>2]|0;
 $44 = (($42) + ($43)|0);
 $45 = HEAP32[$pRet>>2]|0;
 $46 = ((($45)) + 12|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$pRet>>2]|0;
 $48 = ((($47)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$nStr>>2]|0;
 $52 = (($51) + 1)|0;
 _memcpy(($49|0),($50|0),($52|0))|0;
 $53 = HEAP32[$pRet>>2]|0;
 HEAP8[$53>>0] = 1;
 $54 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _fts3MIBufferAlloc($p,$paOut) {
 $p = $p|0;
 $paOut = $paOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aOut = 0, $xRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $xRet = sp + 4|0;
 $aOut = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $paOut;
 HEAP32[$xRet>>2] = 0;
 HEAP32[$aOut>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 1|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$0>>2]|0;
 do {
  if ($6) {
   $8 = ((($7)) + 1|0);
   HEAP8[$8>>0] = 1;
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 16|0);
   $11 = ((($10)) + 4|0);
   HEAP32[$aOut>>2] = $11;
   HEAP32[$xRet>>2] = 147;
  } else {
   $12 = ((($7)) + 2|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)==(0);
   $16 = HEAP32[$0>>2]|0;
   if ($15) {
    $17 = ((($16)) + 2|0);
    HEAP8[$17>>0] = 1;
    $18 = HEAP32[$0>>2]|0;
    $19 = ((($18)) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + 2)|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 16|0);
    $24 = (($23) + ($21<<2)|0);
    HEAP32[$aOut>>2] = $24;
    HEAP32[$xRet>>2] = 147;
    break;
   }
   $25 = ((($16)) + 4|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $26<<2;
   $28 = (_sqlite3_malloc($27)|0);
   HEAP32[$aOut>>2] = $28;
   $29 = HEAP32[$aOut>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    HEAP32[$xRet>>2] = 142;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$aOut>>2]|0;
     $36 = HEAP32[$0>>2]|0;
     $37 = ((($36)) + 16|0);
     $38 = ((($37)) + 4|0);
     $39 = HEAP32[$0>>2]|0;
     $40 = ((($39)) + 4|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = $41<<2;
     _memcpy(($35|0),($38|0),($42|0))|0;
    }
   }
  }
 } while(0);
 $43 = HEAP32[$aOut>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 HEAP32[$44>>2] = $43;
 $45 = HEAP32[$xRet>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _fts3MatchinfoValues($pCsr,$bGlobal,$pInfo,$zArg) {
 $pCsr = $pCsr|0;
 $bGlobal = $bGlobal|0;
 $pInfo = $pInfo|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $a3 = 0, $i = 0, $iCol = 0, $iCol2 = 0, $iVal = 0, $nDoc = 0, $nDoc1 = 0, $nToken = 0, $nToken4 = 0, $nZero = 0, $pExpr = 0, $pSelect = 0, $pSelectDocsize = 0, $pTab = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $rc = sp + 76|0;
 $i = sp + 72|0;
 $pTab = sp + 68|0;
 $pSelect = sp + 64|0;
 $nDoc = sp + 24|0;
 $nDoc1 = sp + 16|0;
 $a = sp + 60|0;
 $iCol = sp + 56|0;
 $iVal = sp + 52|0;
 $nToken = sp + 8|0;
 $pSelectDocsize = sp + 48|0;
 $iCol2 = sp + 44|0;
 $a3 = sp + 40|0;
 $nToken4 = sp;
 $nZero = sp + 36|0;
 $pExpr = sp + 32|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $bGlobal;
 HEAP32[$2>>2] = $pInfo;
 HEAP32[$3>>2] = $zArg;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 HEAP32[$pSelect>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$rc>>2]|0;
  $7 = ($6|0)==(0);
  if (!($7)) {
   label = 31;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  $10 = (($9) + ($8)|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   label = 31;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + ($14)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 24|0);
  HEAP8[$19>>0] = $17;
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = (($21) + ($20)|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23 << 24 >> 24;
  L5: do {
   switch ($24|0) {
   case 112:  {
    $25 = HEAP32[$1>>2]|0;
    $26 = ($25|0)!=(0);
    if ($26) {
     $27 = HEAP32[$2>>2]|0;
     $28 = ((($27)) + 8|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$2>>2]|0;
     $31 = ((($30)) + 28|0);
     $32 = HEAP32[$31>>2]|0;
     HEAP32[$32>>2] = $29;
    }
    break;
   }
   case 99:  {
    $33 = HEAP32[$1>>2]|0;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$2>>2]|0;
     $36 = ((($35)) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = HEAP32[$2>>2]|0;
     $39 = ((($38)) + 28|0);
     $40 = HEAP32[$39>>2]|0;
     HEAP32[$40>>2] = $37;
    }
    break;
   }
   case 110:  {
    $41 = HEAP32[$1>>2]|0;
    $42 = ($41|0)!=(0);
    if ($42) {
     $43 = $nDoc;
     $44 = $43;
     HEAP32[$44>>2] = 0;
     $45 = (($43) + 4)|0;
     $46 = $45;
     HEAP32[$46>>2] = 0;
     $47 = HEAP32[$pTab>>2]|0;
     $48 = (_fts3MatchinfoSelectDoctotal($47,$pSelect,$nDoc,0)|0);
     HEAP32[$rc>>2] = $48;
     $49 = $nDoc;
     $50 = $49;
     $51 = HEAP32[$50>>2]|0;
     $52 = (($49) + 4)|0;
     $53 = $52;
     $54 = HEAP32[$53>>2]|0;
     $55 = HEAP32[$2>>2]|0;
     $56 = ((($55)) + 28|0);
     $57 = HEAP32[$56>>2]|0;
     HEAP32[$57>>2] = $51;
    }
    break;
   }
   case 97:  {
    $58 = HEAP32[$1>>2]|0;
    $59 = ($58|0)!=(0);
    if ($59) {
     $60 = HEAP32[$pTab>>2]|0;
     $61 = (_fts3MatchinfoSelectDoctotal($60,$pSelect,$nDoc1,$a)|0);
     HEAP32[$rc>>2] = $61;
     $62 = HEAP32[$rc>>2]|0;
     $63 = ($62|0)==(0);
     if ($63) {
      HEAP32[$iCol>>2] = 0;
      while(1) {
       $64 = HEAP32[$iCol>>2]|0;
       $65 = HEAP32[$2>>2]|0;
       $66 = ((($65)) + 4|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = ($64|0)<($67|0);
       if (!($68)) {
        break L5;
       }
       $69 = HEAP32[$a>>2]|0;
       $70 = (_sqlite3Fts3GetVarint($69,$nToken)|0);
       $71 = HEAP32[$a>>2]|0;
       $72 = (($71) + ($70)|0);
       HEAP32[$a>>2] = $72;
       $73 = $nToken;
       $74 = $73;
       $75 = HEAP32[$74>>2]|0;
       $76 = (($73) + 4)|0;
       $77 = $76;
       $78 = HEAP32[$77>>2]|0;
       $79 = $nDoc1;
       $80 = $79;
       $81 = HEAP32[$80>>2]|0;
       $82 = (($79) + 4)|0;
       $83 = $82;
       $84 = HEAP32[$83>>2]|0;
       $85 = (___divdi3(($81|0),($84|0),2,0)|0);
       $86 = tempRet0;
       $87 = (_i64Add(($75|0),0,($85|0),($86|0))|0);
       $88 = tempRet0;
       $89 = $nDoc1;
       $90 = $89;
       $91 = HEAP32[$90>>2]|0;
       $92 = (($89) + 4)|0;
       $93 = $92;
       $94 = HEAP32[$93>>2]|0;
       $95 = (___divdi3(($87|0),($88|0),($91|0),($94|0))|0);
       $96 = tempRet0;
       HEAP32[$iVal>>2] = $95;
       $97 = HEAP32[$iVal>>2]|0;
       $98 = HEAP32[$iCol>>2]|0;
       $99 = HEAP32[$2>>2]|0;
       $100 = ((($99)) + 28|0);
       $101 = HEAP32[$100>>2]|0;
       $102 = (($101) + ($98<<2)|0);
       HEAP32[$102>>2] = $97;
       $103 = HEAP32[$iCol>>2]|0;
       $104 = (($103) + 1)|0;
       HEAP32[$iCol>>2] = $104;
      }
     }
    }
    break;
   }
   case 108:  {
    HEAP32[$pSelectDocsize>>2] = 0;
    $105 = HEAP32[$pTab>>2]|0;
    $106 = HEAP32[$0>>2]|0;
    $107 = ((($106)) + 32|0);
    $108 = $107;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (($108) + 4)|0;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = (_sqlite3Fts3SelectDocsize($105,$110,$113,$pSelectDocsize)|0);
    HEAP32[$rc>>2] = $114;
    $115 = HEAP32[$rc>>2]|0;
    $116 = ($115|0)==(0);
    L27: do {
     if ($116) {
      $117 = HEAP32[$pSelectDocsize>>2]|0;
      $118 = (_sqlite3_column_blob($117,0)|0);
      HEAP32[$a3>>2] = $118;
      HEAP32[$iCol2>>2] = 0;
      while(1) {
       $119 = HEAP32[$iCol2>>2]|0;
       $120 = HEAP32[$2>>2]|0;
       $121 = ((($120)) + 4|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = ($119|0)<($122|0);
       if (!($123)) {
        break L27;
       }
       $124 = HEAP32[$a3>>2]|0;
       $125 = (_sqlite3Fts3GetVarint($124,$nToken4)|0);
       $126 = HEAP32[$a3>>2]|0;
       $127 = (($126) + ($125)|0);
       HEAP32[$a3>>2] = $127;
       $128 = $nToken4;
       $129 = $128;
       $130 = HEAP32[$129>>2]|0;
       $131 = (($128) + 4)|0;
       $132 = $131;
       $133 = HEAP32[$132>>2]|0;
       $134 = HEAP32[$iCol2>>2]|0;
       $135 = HEAP32[$2>>2]|0;
       $136 = ((($135)) + 28|0);
       $137 = HEAP32[$136>>2]|0;
       $138 = (($137) + ($134<<2)|0);
       HEAP32[$138>>2] = $130;
       $139 = HEAP32[$iCol2>>2]|0;
       $140 = (($139) + 1)|0;
       HEAP32[$iCol2>>2] = $140;
      }
     }
    } while(0);
    $141 = HEAP32[$pSelectDocsize>>2]|0;
    (_sqlite3_reset($141)|0);
    break;
   }
   case 115:  {
    $142 = HEAP32[$0>>2]|0;
    $143 = (_fts3ExprLoadDoclists($142,0,0)|0);
    HEAP32[$rc>>2] = $143;
    $144 = HEAP32[$rc>>2]|0;
    $145 = ($144|0)==(0);
    if ($145) {
     $146 = HEAP32[$0>>2]|0;
     $147 = HEAP32[$2>>2]|0;
     $148 = (_fts3MatchinfoLcs($146,$147)|0);
     HEAP32[$rc>>2] = $148;
    }
    break;
   }
   case 121: case 98:  {
    $149 = HEAP32[$2>>2]|0;
    $150 = HEAP32[$i>>2]|0;
    $151 = HEAP32[$3>>2]|0;
    $152 = (($151) + ($150)|0);
    $153 = HEAP8[$152>>0]|0;
    $154 = (_fts3MatchinfoSize($149,$153)|0);
    $155 = $154<<2;
    HEAP32[$nZero>>2] = $155;
    $156 = HEAP32[$2>>2]|0;
    $157 = ((($156)) + 28|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = HEAP32[$nZero>>2]|0;
    _memset(($158|0),0,($159|0))|0;
    $160 = HEAP32[$0>>2]|0;
    $161 = ((($160)) + 12|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = HEAP32[$2>>2]|0;
    _fts3ExprLHitGather($162,$163);
    break;
   }
   default: {
    $164 = HEAP32[$0>>2]|0;
    $165 = ((($164)) + 12|0);
    $166 = HEAP32[$165>>2]|0;
    HEAP32[$pExpr>>2] = $166;
    $167 = HEAP32[$0>>2]|0;
    $168 = (_fts3ExprLoadDoclists($167,0,0)|0);
    HEAP32[$rc>>2] = $168;
    $169 = HEAP32[$rc>>2]|0;
    $170 = ($169|0)!=(0);
    if (!($170)) {
     $171 = HEAP32[$1>>2]|0;
     $172 = ($171|0)!=(0);
     if ($172) {
      $173 = HEAP32[$0>>2]|0;
      $174 = ((($173)) + 24|0);
      $175 = HEAP32[$174>>2]|0;
      $176 = ($175|0)!=(0|0);
      if ($176) {
       $177 = HEAP32[$pTab>>2]|0;
       $178 = HEAP32[$2>>2]|0;
       $179 = ((($178)) + 16|0);
       $180 = (_fts3MatchinfoSelectDoctotal($177,$pSelect,$179,0)|0);
       HEAP32[$rc>>2] = $180;
       $181 = HEAP32[$rc>>2]|0;
       $182 = ($181|0)!=(0);
       if ($182) {
        break L5;
       }
      }
      $183 = HEAP32[$pExpr>>2]|0;
      $184 = HEAP32[$2>>2]|0;
      $185 = (_fts3ExprIterate($183,160,$184)|0);
      HEAP32[$rc>>2] = $185;
      $186 = HEAP32[$0>>2]|0;
      (_sqlite3Fts3EvalTestDeferred($186,$rc)|0);
      $187 = HEAP32[$rc>>2]|0;
      $188 = ($187|0)!=(0);
      if ($188) {
       break L5;
      }
     }
     $189 = HEAP32[$pExpr>>2]|0;
     $190 = HEAP32[$2>>2]|0;
     (_fts3ExprIterate($189,161,$190)|0);
    }
   }
   }
  } while(0);
  $191 = HEAP32[$2>>2]|0;
  $192 = HEAP32[$i>>2]|0;
  $193 = HEAP32[$3>>2]|0;
  $194 = (($193) + ($192)|0);
  $195 = HEAP8[$194>>0]|0;
  $196 = (_fts3MatchinfoSize($191,$195)|0);
  $197 = HEAP32[$2>>2]|0;
  $198 = ((($197)) + 28|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + ($196<<2)|0);
  HEAP32[$198>>2] = $200;
  $201 = HEAP32[$i>>2]|0;
  $202 = (($201) + 1)|0;
  HEAP32[$i>>2] = $202;
 }
 if ((label|0) == 31) {
  $203 = HEAP32[$pSelect>>2]|0;
  (_sqlite3_reset($203)|0);
  $204 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($204|0);
 }
 return (0)|0;
}
function _fts3MIBufferSetGlobal($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 HEAP32[$2>>2] = 1;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (2 + ($5))|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = (($8) + ($6<<2)|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15<<2;
 _memcpy(($9|0),($12|0),($16|0))|0;
 STACKTOP = sp;return;
}
function _fts3ExprPhraseCountCb($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 36|0);
 HEAP32[$8>>2] = $6;
 STACKTOP = sp;return 0;
}
function _fts3MIBufferFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pBuf = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + -4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (0 - ($4))|0;
 $6 = (($1) + ($5)|0);
 HEAP32[$pBuf>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pBuf>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = ((($9)) + 4|0);
 $11 = ($7|0)==($10|0);
 $12 = HEAP32[$pBuf>>2]|0;
 if ($11) {
  $13 = ((($12)) + 1|0);
  HEAP8[$13>>0] = 0;
 } else {
  $14 = ((($12)) + 2|0);
  HEAP8[$14>>0] = 0;
 }
 $15 = HEAP32[$pBuf>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$pBuf>>2]|0;
 $20 = ((($19)) + 1|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = ($22|0)==(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$pBuf>>2]|0;
 $25 = ((($24)) + 2|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)==(0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$pBuf>>2]|0;
 _sqlite3_free($29);
 STACKTOP = sp;return;
}
function _fts3MatchinfoSelectDoctotal($pTab,$ppStmt,$pnDoc,$paLen) {
 $pTab = $pTab|0;
 $ppStmt = $ppStmt|0;
 $pnDoc = $pnDoc|0;
 $paLen = $paLen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $nDoc = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pStmt = sp + 16|0;
 $a = sp + 12|0;
 $nDoc = sp;
 $rc = sp + 8|0;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $ppStmt;
 HEAP32[$3>>2] = $pnDoc;
 HEAP32[$4>>2] = $paLen;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3Fts3SelectDoctotal($8,$9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $13;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pStmt>>2] = $15;
 $16 = HEAP32[$pStmt>>2]|0;
 $17 = (_sqlite3_column_blob($16,0)|0);
 HEAP32[$a>>2] = $17;
 $18 = HEAP32[$a>>2]|0;
 $19 = (_sqlite3Fts3GetVarint($18,$nDoc)|0);
 $20 = HEAP32[$a>>2]|0;
 $21 = (($20) + ($19)|0);
 HEAP32[$a>>2] = $21;
 $22 = $nDoc;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($24|0)==(0);
 $29 = ($27|0)==(0);
 $30 = $28 & $29;
 if ($30) {
  HEAP32[$0>>2] = 267;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $31 = $nDoc;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$3>>2]|0;
 $38 = $37;
 $39 = $38;
 HEAP32[$39>>2] = $33;
 $40 = (($38) + 4)|0;
 $41 = $40;
 HEAP32[$41>>2] = 0;
 $42 = HEAP32[$4>>2]|0;
 $43 = ($42|0)!=(0|0);
 if ($43) {
  $44 = HEAP32[$a>>2]|0;
  $45 = HEAP32[$4>>2]|0;
  HEAP32[$45>>2] = $44;
 }
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3Fts3SelectDocsize($pTab,$0,$1,$ppStmt) {
 $pTab = $pTab|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppStmt = $ppStmt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $pTab;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $ppStmt;
 $9 = HEAP32[$2>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = (_fts3SelectDocsize($9,$12,$15,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _fts3MatchinfoLcs($pCsr,$pInfo) {
 $pCsr = $pCsr|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aIter = 0, $i = 0, $iCol = 0, $nLcs = 0;
 var $nLive = 0, $nThisLcs = 0, $nToken = 0, $pAdv = 0, $pIt = 0, $pIter = 0, $pIter1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $aIter = sp + 44|0;
 $i = sp + 40|0;
 $iCol = sp + 36|0;
 $nToken = sp + 32|0;
 $pIter = sp + 28|0;
 $nLcs = sp + 24|0;
 $nLive = sp + 20|0;
 $rc = sp + 16|0;
 $pIt = sp + 12|0;
 $pAdv = sp + 8|0;
 $nThisLcs = sp + 4|0;
 $pIter1 = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pInfo;
 HEAP32[$nToken>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<4;
 $7 = (_sqlite3_malloc($6)|0);
 HEAP32[$aIter>>2] = $7;
 $8 = HEAP32[$aIter>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  HEAP32[$0>>2] = 7;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 $10 = HEAP32[$aIter>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13<<4;
 _memset(($10|0),0,($14|0))|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$aIter>>2]|0;
 (_fts3ExprIterate($17,162,$18)|0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($19|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$aIter>>2]|0;
  $26 = (($25) + ($24<<4)|0);
  HEAP32[$pIter>>2] = $26;
  $27 = HEAP32[$pIter>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 20|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ((($30)) + 64|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$nToken>>2]|0;
  $34 = (($33) - ($32))|0;
  HEAP32[$nToken>>2] = $34;
  $35 = HEAP32[$nToken>>2]|0;
  $36 = HEAP32[$pIter>>2]|0;
  $37 = ((($36)) + 4|0);
  HEAP32[$37>>2] = $35;
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 HEAP32[$iCol>>2] = 0;
 L9: while(1) {
  $40 = HEAP32[$iCol>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 4|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   label = 33;
   break;
  }
  HEAP32[$nLcs>>2] = 0;
  HEAP32[$nLive>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $45 = HEAP32[$i>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ($45|0)<($48|0);
   if (!($49)) {
    break;
   }
   $50 = HEAP32[$i>>2]|0;
   $51 = HEAP32[$aIter>>2]|0;
   $52 = (($51) + ($50<<4)|0);
   HEAP32[$pIt>>2] = $52;
   $53 = HEAP32[$1>>2]|0;
   $54 = HEAP32[$pIt>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$iCol>>2]|0;
   $57 = HEAP32[$pIt>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = (_sqlite3Fts3EvalPhrasePoslist($53,$55,$56,$58)|0);
   HEAP32[$rc>>2] = $59;
   $60 = HEAP32[$rc>>2]|0;
   $61 = ($60|0)!=(0);
   if ($61) {
    label = 11;
    break L9;
   }
   $63 = HEAP32[$pIt>>2]|0;
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($65|0)!=(0|0);
   if ($66) {
    $67 = HEAP32[$pIt>>2]|0;
    $68 = ((($67)) + 4|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = HEAP32[$pIt>>2]|0;
    $71 = ((($70)) + 12|0);
    HEAP32[$71>>2] = $69;
    $72 = HEAP32[$i>>2]|0;
    $73 = HEAP32[$aIter>>2]|0;
    $74 = (($73) + ($72<<4)|0);
    (_fts3LcsIteratorAdvance($74)|0);
    $75 = HEAP32[$nLive>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$nLive>>2] = $76;
   }
   $77 = HEAP32[$i>>2]|0;
   $78 = (($77) + 1)|0;
   HEAP32[$i>>2] = $78;
  }
  while(1) {
   $79 = HEAP32[$nLive>>2]|0;
   $80 = ($79|0)>(0);
   if (!($80)) {
    break;
   }
   HEAP32[$pAdv>>2] = 0;
   HEAP32[$nThisLcs>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $81 = HEAP32[$i>>2]|0;
    $82 = HEAP32[$2>>2]|0;
    $83 = ((($82)) + 8|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = ($81|0)<($84|0);
    if (!($85)) {
     break;
    }
    $86 = HEAP32[$i>>2]|0;
    $87 = HEAP32[$aIter>>2]|0;
    $88 = (($87) + ($86<<4)|0);
    HEAP32[$pIter1>>2] = $88;
    $89 = HEAP32[$pIter1>>2]|0;
    $90 = ((($89)) + 8|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    if ($92) {
     HEAP32[$nThisLcs>>2] = 0;
    } else {
     $93 = HEAP32[$pAdv>>2]|0;
     $94 = ($93|0)==(0|0);
     if ($94) {
      label = 22;
     } else {
      $95 = HEAP32[$pIter1>>2]|0;
      $96 = ((($95)) + 12|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = HEAP32[$pAdv>>2]|0;
      $99 = ((($98)) + 12|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = ($97|0)<($100|0);
      if ($101) {
       label = 22;
      }
     }
     if ((label|0) == 22) {
      label = 0;
      $102 = HEAP32[$pIter1>>2]|0;
      HEAP32[$pAdv>>2] = $102;
     }
     $103 = HEAP32[$nThisLcs>>2]|0;
     $104 = ($103|0)==(0);
     if ($104) {
      label = 25;
     } else {
      $105 = HEAP32[$pIter1>>2]|0;
      $106 = ((($105)) + 12|0);
      $107 = HEAP32[$106>>2]|0;
      $108 = HEAP32[$pIter1>>2]|0;
      $109 = ((($108)) + -16|0);
      $110 = ((($109)) + 12|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = ($107|0)==($111|0);
      if ($112) {
       label = 25;
      } else {
       HEAP32[$nThisLcs>>2] = 1;
      }
     }
     if ((label|0) == 25) {
      label = 0;
      $113 = HEAP32[$nThisLcs>>2]|0;
      $114 = (($113) + 1)|0;
      HEAP32[$nThisLcs>>2] = $114;
     }
     $115 = HEAP32[$nThisLcs>>2]|0;
     $116 = HEAP32[$nLcs>>2]|0;
     $117 = ($115|0)>($116|0);
     if ($117) {
      $118 = HEAP32[$nThisLcs>>2]|0;
      HEAP32[$nLcs>>2] = $118;
     }
    }
    $119 = HEAP32[$i>>2]|0;
    $120 = (($119) + 1)|0;
    HEAP32[$i>>2] = $120;
   }
   $121 = HEAP32[$pAdv>>2]|0;
   $122 = (_fts3LcsIteratorAdvance($121)|0);
   $123 = ($122|0)!=(0);
   if (!($123)) {
    continue;
   }
   $124 = HEAP32[$nLive>>2]|0;
   $125 = (($124) + -1)|0;
   HEAP32[$nLive>>2] = $125;
  }
  $126 = HEAP32[$nLcs>>2]|0;
  $127 = HEAP32[$iCol>>2]|0;
  $128 = HEAP32[$2>>2]|0;
  $129 = ((($128)) + 28|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = (($130) + ($127<<2)|0);
  HEAP32[$131>>2] = $126;
  $132 = HEAP32[$iCol>>2]|0;
  $133 = (($132) + 1)|0;
  HEAP32[$iCol>>2] = $133;
 }
 if ((label|0) == 11) {
  $62 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $62;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 else if ((label|0) == 33) {
  $134 = HEAP32[$aIter>>2]|0;
  _sqlite3_free($134);
  HEAP32[$0>>2] = 0;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 return (0)|0;
}
function _fts3ExprLHitGather($pExpr,$p) {
 $pExpr = $pExpr|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 32|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 32|0);
 $18 = $17;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($11|0)==($20|0);
 $25 = ($14|0)==($23|0);
 $26 = $24 & $25;
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 $31 = HEAP32[$0>>2]|0;
 if ($30) {
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  _fts3ExprLHitGather($33,$34);
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  _fts3ExprLHitGather($37,$38);
  STACKTOP = sp;return;
 } else {
  $39 = HEAP32[$1>>2]|0;
  _fts3ExprLHits($31,$39);
  STACKTOP = sp;return;
 }
}
function _fts3ExprGlobalHitsCb($pExpr,$iPhrase,$pCtx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7*3)|0;
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = Math_imul($8, $11)|0;
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 28|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<2)|0);
 $17 = (_sqlite3Fts3EvalPhraseStats($5,$6,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _fts3ExprLocalHitsCb($pExpr,$iPhrase,$pCtx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iStart = 0, $p = 0, $pCsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $p = sp + 12|0;
 $iStart = sp + 8|0;
 $i = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = Math_imul($4, $7)|0;
 $9 = ($8*3)|0;
 HEAP32[$iStart>>2] = $9;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)==(0);
  $17 = $14 ? $16 : 0;
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$p>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$i>>2]|0;
  $22 = (_sqlite3Fts3EvalPhrasePoslist($19,$20,$21,$pCsr)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$pCsr>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = (_fts3ColumnlistCount($pCsr)|0);
   $26 = HEAP32[$iStart>>2]|0;
   $27 = HEAP32[$i>>2]|0;
   $28 = ($27*3)|0;
   $29 = (($26) + ($28))|0;
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 28|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + ($29<<2)|0);
   HEAP32[$33>>2] = $25;
  } else {
   $34 = HEAP32[$iStart>>2]|0;
   $35 = HEAP32[$i>>2]|0;
   $36 = ($35*3)|0;
   $37 = (($34) + ($36))|0;
   $38 = HEAP32[$p>>2]|0;
   $39 = ((($38)) + 28|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + ($37<<2)|0);
   HEAP32[$41>>2] = 0;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$i>>2] = $43;
 }
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _fts3SelectDocsize($pTab,$0,$1,$ppStmt) {
 $pTab = $pTab|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppStmt = $ppStmt|0;
 var $$ = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $pTab;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $ppStmt;
 HEAP32[$pStmt>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,21,$pStmt,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$pStmt>>2]|0;
   $14 = $3;
   $15 = $14;
   $16 = HEAP32[$15>>2]|0;
   $17 = (($14) + 4)|0;
   $18 = $17;
   $19 = HEAP32[$18>>2]|0;
   (_sqlite3_bind_int64($13,1,$16,$19)|0);
   $20 = HEAP32[$pStmt>>2]|0;
   $21 = (_sqlite3_step($20)|0);
   HEAP32[$rc>>2] = $21;
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)!=(100);
   if (!($23)) {
    $24 = HEAP32[$pStmt>>2]|0;
    $25 = (_sqlite3_column_type($24,0)|0);
    $26 = ($25|0)!=(4);
    if (!($26)) {
     HEAP32[$rc>>2] = 0;
     break;
    }
   }
   $27 = HEAP32[$pStmt>>2]|0;
   $28 = (_sqlite3_reset($27)|0);
   HEAP32[$rc>>2] = $28;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(0);
   $$ = $30 ? 267 : $28;
   HEAP32[$rc>>2] = $$;
   HEAP32[$pStmt>>2] = 0;
  }
 } while(0);
 $31 = HEAP32[$pStmt>>2]|0;
 $32 = HEAP32[$4>>2]|0;
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _fts3MatchinfoLcsCb($pExpr,$iPhrase,$pCtx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aIter = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$aIter>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$aIter>>2]|0;
 $7 = (($6) + ($5<<4)|0);
 HEAP32[$7>>2] = $4;
 STACKTOP = sp;return 0;
}
function _fts3LcsIteratorAdvance($pIter) {
 $pIter = $pIter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iRead = 0, $or$cond = 0, $pRead = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pRead = sp + 12|0;
 $iRead = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $pIter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pRead>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$pRead>>2]|0;
 $5 = (_sqlite3Fts3GetVarint($4,$iRead)|0);
 $6 = HEAP32[$pRead>>2]|0;
 $7 = (($6) + ($5)|0);
 HEAP32[$pRead>>2] = $7;
 $8 = $iRead;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)==(0);
 $15 = ($13|0)==(0);
 $16 = $14 & $15;
 $17 = $iRead;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($19|0)==(1);
 $24 = ($22|0)==(0);
 $25 = $23 & $24;
 $or$cond = $16 | $25;
 if ($or$cond) {
  HEAP32[$pRead>>2] = 0;
  HEAP32[$rc>>2] = 1;
  $38 = HEAP32[$pRead>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $26 = $iRead;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (_i64Subtract(($28|0),($31|0),2,0)|0);
  $33 = tempRet0;
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + ($32))|0;
  HEAP32[$35>>2] = $37;
  $38 = HEAP32[$pRead>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _fts3ExprLHits($pExpr,$p) {
 $pExpr = $pExpr|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $iCol = 0, $iStart = 0, $nHit = 0, $pIter = 0, $pPhrase = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTab = sp + 20|0;
 $iStart = sp + 16|0;
 $pPhrase = sp + 12|0;
 $pIter = sp + 8|0;
 $iCol = sp + 4|0;
 $nHit = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pPhrase>>2] = $7;
 $8 = HEAP32[$pPhrase>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pIter>>2] = $10;
 HEAP32[$iCol>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13 << 24 >> 24;
 $15 = ($14|0)==(121);
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 36|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 if ($15) {
  $22 = Math_imul($18, $21)|0;
  HEAP32[$iStart>>2] = $22;
 } else {
  $23 = (($21) + 31)|0;
  $24 = (($23|0) / 32)&-1;
  $25 = Math_imul($18, $24)|0;
  HEAP32[$iStart>>2] = $25;
 }
 while(1) {
  $26 = (_fts3ColumnlistCount($pIter)|0);
  HEAP32[$nHit>>2] = $26;
  $27 = HEAP32[$pPhrase>>2]|0;
  $28 = ((($27)) + 68|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pTab>>2]|0;
  $31 = ((($30)) + 24|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($29|0)>=($32|0);
  if ($33) {
   label = 6;
  } else {
   $34 = HEAP32[$pPhrase>>2]|0;
   $35 = ((($34)) + 68|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$iCol>>2]|0;
   $38 = ($36|0)==($37|0);
   if ($38) {
    label = 6;
   }
  }
  do {
   if ((label|0) == 6) {
    label = 0;
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 24|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41 << 24 >> 24;
    $43 = ($42|0)==(121);
    $44 = HEAP32[$nHit>>2]|0;
    if ($43) {
     $45 = HEAP32[$iStart>>2]|0;
     $46 = HEAP32[$iCol>>2]|0;
     $47 = (($45) + ($46))|0;
     $48 = HEAP32[$1>>2]|0;
     $49 = ((($48)) + 28|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (($50) + ($47<<2)|0);
     HEAP32[$51>>2] = $44;
     break;
    }
    $52 = ($44|0)!=(0);
    if ($52) {
     $53 = HEAP32[$iCol>>2]|0;
     $54 = $53 & 31;
     $55 = 1 << $54;
     $56 = HEAP32[$iStart>>2]|0;
     $57 = HEAP32[$iCol>>2]|0;
     $58 = (($57) + 1)|0;
     $59 = (($58|0) / 32)&-1;
     $60 = (($56) + ($59))|0;
     $61 = HEAP32[$1>>2]|0;
     $62 = ((($61)) + 28|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = (($63) + ($60<<2)|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = $65 | $55;
     HEAP32[$64>>2] = $66;
    }
   }
  } while(0);
  $67 = HEAP32[$pIter>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 << 24 >> 24;
  $70 = ($69|0)!=(1);
  if ($70) {
   break;
  }
  $71 = HEAP32[$pIter>>2]|0;
  $72 = ((($71)) + 1|0);
  HEAP32[$pIter>>2] = $72;
  $73 = HEAP32[$pIter>>2]|0;
  $74 = HEAP8[$73>>0]|0;
  $75 = $74&255;
  $76 = $75 & 128;
  $77 = ($76|0)!=(0);
  $78 = HEAP32[$pIter>>2]|0;
  if ($77) {
   $79 = (_sqlite3Fts3GetVarint32($78,$iCol)|0);
   $84 = $79;
  } else {
   $80 = HEAP8[$78>>0]|0;
   $81 = $80&255;
   HEAP32[$iCol>>2] = $81;
   $84 = 1;
  }
  $82 = HEAP32[$pIter>>2]|0;
  $83 = (($82) + ($84)|0);
  HEAP32[$pIter>>2] = $83;
 }
 STACKTOP = sp;return;
}
function _fts3ColumnlistCount($ppCollist) {
 $ppCollist = $ppCollist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $c = 0, $nEntry = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pEnd = sp + 4|0;
 $c = sp + 12|0;
 $nEntry = sp;
 HEAP32[$0>>2] = $ppCollist;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pEnd>>2] = $2;
 HEAP8[$c>>0] = 0;
 HEAP32[$nEntry>>2] = 0;
 while(1) {
  $3 = HEAP32[$pEnd>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = HEAP8[$c>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = $5 | $7;
  $9 = 254 & $8;
  $10 = ($9|0)!=(0);
  $11 = HEAP32[$pEnd>>2]|0;
  if (!($10)) {
   break;
  }
  $12 = ((($11)) + 1|0);
  HEAP32[$pEnd>>2] = $12;
  $13 = HEAP8[$11>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = $14 & 128;
  $16 = $15&255;
  HEAP8[$c>>0] = $16;
  $17 = HEAP8[$c>>0]|0;
  $18 = ($17<<24>>24)!=(0);
  if ($18) {
   continue;
  }
  $19 = HEAP32[$nEntry>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$nEntry>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 HEAP32[$21>>2] = $11;
 $22 = HEAP32[$nEntry>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3Fts3EvalPhraseStats($pCsr,$pExpr,$aiOut) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $aiOut = $aiOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $iCol = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pTab = sp + 8|0;
 $rc = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $aiOut;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 34|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(1);
  if ($14) {
   HEAP32[$iCol>>2] = 0;
   while(1) {
    $15 = HEAP32[$iCol>>2]|0;
    $16 = HEAP32[$pTab>>2]|0;
    $17 = ((($16)) + 24|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($15|0)<($18|0);
    if (!($19)) {
     break;
    }
    $20 = HEAP32[$0>>2]|0;
    $21 = ((($20)) + 64|0);
    $22 = $21;
    $23 = $22;
    $24 = HEAP32[$23>>2]|0;
    $25 = (($22) + 4)|0;
    $26 = $25;
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$iCol>>2]|0;
    $29 = ($28*3)|0;
    $30 = (($29) + 1)|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = (($31) + ($30<<2)|0);
    HEAP32[$32>>2] = $24;
    $33 = HEAP32[$0>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = $34;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($35) + 4)|0;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$iCol>>2]|0;
    $42 = ($41*3)|0;
    $43 = (($42) + 2)|0;
    $44 = HEAP32[$2>>2]|0;
    $45 = (($44) + ($43<<2)|0);
    HEAP32[$45>>2] = $37;
    $46 = HEAP32[$iCol>>2]|0;
    $47 = (($46) + 1)|0;
    HEAP32[$iCol>>2] = $47;
   }
   $86 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($86|0);
  }
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$1>>2]|0;
 $50 = (_fts3EvalGatherStats($48,$49)|0);
 HEAP32[$rc>>2] = $50;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if (!($52)) {
  $86 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 HEAP32[$iCol>>2] = 0;
 while(1) {
  $53 = HEAP32[$iCol>>2]|0;
  $54 = HEAP32[$pTab>>2]|0;
  $55 = ((($54)) + 24|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($53|0)<($56|0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$iCol>>2]|0;
  $59 = ($58*3)|0;
  $60 = (($59) + 1)|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = ((($61)) + 40|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + ($60<<2)|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$iCol>>2]|0;
  $67 = ($66*3)|0;
  $68 = (($67) + 1)|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = (($69) + ($68<<2)|0);
  HEAP32[$70>>2] = $65;
  $71 = HEAP32[$iCol>>2]|0;
  $72 = ($71*3)|0;
  $73 = (($72) + 2)|0;
  $74 = HEAP32[$1>>2]|0;
  $75 = ((($74)) + 40|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76) + ($73<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = HEAP32[$iCol>>2]|0;
  $80 = ($79*3)|0;
  $81 = (($80) + 2)|0;
  $82 = HEAP32[$2>>2]|0;
  $83 = (($82) + ($81<<2)|0);
  HEAP32[$83>>2] = $78;
  $84 = HEAP32[$iCol>>2]|0;
  $85 = (($84) + 1)|0;
  HEAP32[$iCol>>2] = $85;
 }
 $86 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _fts3EvalGatherStats($pCsr,$pExpr) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bEof = 0, $iDocid = 0, $iPrevId = 0, $p = 0, $pE = 0, $pRoot = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $pTab = sp + 28|0;
 $pRoot = sp + 24|0;
 $p = sp + 20|0;
 $iPrevId = sp + 8|0;
 $iDocid = sp;
 $bEof = sp + 48|0;
 $pE = sp + 16|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = HEAP32[$7>>2]|0;
   HEAP32[$pTab>>2] = $8;
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 32|0);
   $11 = $10;
   $12 = $11;
   $13 = HEAP32[$12>>2]|0;
   $14 = (($11) + 4)|0;
   $15 = $14;
   $16 = HEAP32[$15>>2]|0;
   $17 = $iPrevId;
   $18 = $17;
   HEAP32[$18>>2] = $13;
   $19 = (($17) + 4)|0;
   $20 = $19;
   HEAP32[$20>>2] = $16;
   $21 = HEAP32[$2>>2]|0;
   HEAP32[$pRoot>>2] = $21;
   while(1) {
    $22 = HEAP32[$pRoot>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($24|0)!=(0|0);
    if ($25) {
     $26 = HEAP32[$pRoot>>2]|0;
     $27 = ((($26)) + 8|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = HEAP32[$28>>2]|0;
     $30 = ($29|0)==(1);
     $188 = $30;
    } else {
     $188 = 0;
    }
    $31 = HEAP32[$pRoot>>2]|0;
    if (!($188)) {
     break;
    }
    $32 = ((($31)) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$pRoot>>2] = $33;
   }
   $34 = ((($31)) + 24|0);
   $35 = $34;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = $iDocid;
   $42 = $41;
   HEAP32[$42>>2] = $37;
   $43 = (($41) + 4)|0;
   $44 = $43;
   HEAP32[$44>>2] = $40;
   $45 = HEAP32[$pRoot>>2]|0;
   $46 = ((($45)) + 32|0);
   $47 = HEAP8[$46>>0]|0;
   HEAP8[$bEof>>0] = $47;
   $48 = HEAP32[$pRoot>>2]|0;
   HEAP32[$p>>2] = $48;
   while(1) {
    $49 = HEAP32[$p>>2]|0;
    $50 = ($49|0)!=(0|0);
    if (!($50)) {
     break;
    }
    $51 = HEAP32[$p>>2]|0;
    $52 = HEAP32[$51>>2]|0;
    $53 = ($52|0)==(5);
    $54 = HEAP32[$p>>2]|0;
    if ($53) {
     $57 = $54;
    } else {
     $55 = ((($54)) + 16|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = $56;
    }
    HEAP32[$pE>>2] = $57;
    $58 = HEAP32[$pTab>>2]|0;
    $59 = ((($58)) + 24|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = ($60*3)|0;
    $62 = $61<<2;
    $63 = (_sqlite3_malloc($62)|0);
    $64 = HEAP32[$pE>>2]|0;
    $65 = ((($64)) + 40|0);
    HEAP32[$65>>2] = $63;
    $66 = HEAP32[$pE>>2]|0;
    $67 = ((($66)) + 40|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     label = 12;
     break;
    }
    $70 = HEAP32[$pE>>2]|0;
    $71 = ((($70)) + 40|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = HEAP32[$pTab>>2]|0;
    $74 = ((($73)) + 24|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ($75*3)|0;
    $77 = $76<<2;
    _memset(($72|0),0,($77|0))|0;
    $78 = HEAP32[$p>>2]|0;
    $79 = ((($78)) + 12|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$p>>2] = $80;
   }
   if ((label|0) == 12) {
    HEAP32[$0>>2] = 7;
    $187 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($187|0);
   }
   $81 = HEAP32[$1>>2]|0;
   $82 = HEAP32[$pRoot>>2]|0;
   _fts3EvalRestart($81,$82,$rc);
   while(1) {
    $83 = HEAP32[$1>>2]|0;
    $84 = ((($83)) + 6|0);
    $85 = HEAP8[$84>>0]|0;
    $86 = $85&255;
    $87 = ($86|0)==(0);
    $88 = HEAP32[$rc>>2]|0;
    $89 = ($88|0)==(0);
    $90 = $87 ? $89 : 0;
    if (!($90)) {
     break;
    }
    while(1) {
     $91 = HEAP32[$1>>2]|0;
     $92 = ((($91)) + 7|0);
     $93 = HEAP8[$92>>0]|0;
     $94 = $93&255;
     $95 = ($94|0)==(0);
     if ($95) {
      $96 = HEAP32[$1>>2]|0;
      $97 = ((($96)) + 8|0);
      $98 = HEAP32[$97>>2]|0;
      (_sqlite3_reset($98)|0);
     }
     $99 = HEAP32[$1>>2]|0;
     $100 = HEAP32[$pRoot>>2]|0;
     _fts3EvalNextRow($99,$100,$rc);
     $101 = HEAP32[$pRoot>>2]|0;
     $102 = ((($101)) + 32|0);
     $103 = HEAP8[$102>>0]|0;
     $104 = HEAP32[$1>>2]|0;
     $105 = ((($104)) + 6|0);
     HEAP8[$105>>0] = $103;
     $106 = HEAP32[$1>>2]|0;
     $107 = ((($106)) + 7|0);
     HEAP8[$107>>0] = 1;
     $108 = HEAP32[$1>>2]|0;
     $109 = ((($108)) + 88|0);
     HEAP32[$109>>2] = 1;
     $110 = HEAP32[$pRoot>>2]|0;
     $111 = ((($110)) + 24|0);
     $112 = $111;
     $113 = $112;
     $114 = HEAP32[$113>>2]|0;
     $115 = (($112) + 4)|0;
     $116 = $115;
     $117 = HEAP32[$116>>2]|0;
     $118 = HEAP32[$1>>2]|0;
     $119 = ((($118)) + 32|0);
     $120 = $119;
     $121 = $120;
     HEAP32[$121>>2] = $114;
     $122 = (($120) + 4)|0;
     $123 = $122;
     HEAP32[$123>>2] = $117;
     $124 = HEAP32[$1>>2]|0;
     $125 = ((($124)) + 6|0);
     $126 = HEAP8[$125>>0]|0;
     $127 = $126&255;
     $128 = ($127|0)==(0);
     if (!($128)) {
      break;
     }
     $129 = HEAP32[$pRoot>>2]|0;
     $130 = HEAP32[$129>>2]|0;
     $131 = ($130|0)==(1);
     if (!($131)) {
      break;
     }
     $132 = HEAP32[$1>>2]|0;
     $133 = (_sqlite3Fts3EvalTestDeferred($132,$rc)|0);
     $134 = ($133|0)!=(0);
     if (!($134)) {
      break;
     }
    }
    $135 = HEAP32[$rc>>2]|0;
    $136 = ($135|0)==(0);
    if (!($136)) {
     continue;
    }
    $137 = HEAP32[$1>>2]|0;
    $138 = ((($137)) + 6|0);
    $139 = HEAP8[$138>>0]|0;
    $140 = $139&255;
    $141 = ($140|0)==(0);
    if (!($141)) {
     continue;
    }
    $142 = HEAP32[$pRoot>>2]|0;
    _fts3EvalUpdateCounts($142);
   }
   $143 = HEAP32[$1>>2]|0;
   $144 = ((($143)) + 6|0);
   HEAP8[$144>>0] = 0;
   $145 = $iPrevId;
   $146 = $145;
   $147 = HEAP32[$146>>2]|0;
   $148 = (($145) + 4)|0;
   $149 = $148;
   $150 = HEAP32[$149>>2]|0;
   $151 = HEAP32[$1>>2]|0;
   $152 = ((($151)) + 32|0);
   $153 = $152;
   $154 = $153;
   HEAP32[$154>>2] = $147;
   $155 = (($153) + 4)|0;
   $156 = $155;
   HEAP32[$156>>2] = $150;
   $157 = HEAP8[$bEof>>0]|0;
   $158 = ($157<<24>>24)!=(0);
   if ($158) {
    $159 = HEAP8[$bEof>>0]|0;
    $160 = HEAP32[$pRoot>>2]|0;
    $161 = ((($160)) + 32|0);
    HEAP8[$161>>0] = $159;
    break;
   }
   $162 = HEAP32[$1>>2]|0;
   $163 = HEAP32[$pRoot>>2]|0;
   _fts3EvalRestart($162,$163,$rc);
   while(1) {
    $164 = HEAP32[$1>>2]|0;
    $165 = HEAP32[$pRoot>>2]|0;
    _fts3EvalNextRow($164,$165,$rc);
    $166 = HEAP32[$pRoot>>2]|0;
    $167 = ((($166)) + 24|0);
    $168 = $167;
    $169 = $168;
    $170 = HEAP32[$169>>2]|0;
    $171 = (($168) + 4)|0;
    $172 = $171;
    $173 = HEAP32[$172>>2]|0;
    $174 = $iDocid;
    $175 = $174;
    $176 = HEAP32[$175>>2]|0;
    $177 = (($174) + 4)|0;
    $178 = $177;
    $179 = HEAP32[$178>>2]|0;
    $180 = ($170|0)!=($176|0);
    $181 = ($173|0)!=($179|0);
    $182 = $180 | $181;
    $183 = HEAP32[$rc>>2]|0;
    $184 = ($183|0)==(0);
    $185 = $182 ? $184 : 0;
    if (!($185)) {
     break;
    }
   }
  }
 } while(0);
 $186 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $186;
 $187 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($187|0);
}
function _fts3EvalUpdateCounts($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $c = 0, $iCnt = 0, $iCol = 0, $p = 0, $pPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pPhrase = sp + 12|0;
 $iCol = sp + 8|0;
 $p = sp + 4|0;
 $c = sp + 20|0;
 $iCnt = sp;
 HEAP32[$0>>2] = $pExpr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPhrase>>2] = $5;
 $6 = HEAP32[$pPhrase>>2]|0;
 $7 = ($6|0)!=(0|0);
 L4: do {
  if ($7) {
   $8 = HEAP32[$pPhrase>>2]|0;
   $9 = ((($8)) + 28|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    HEAP32[$iCol>>2] = 0;
    $12 = HEAP32[$pPhrase>>2]|0;
    $13 = ((($12)) + 28|0);
    $14 = HEAP32[$13>>2]|0;
    HEAP32[$p>>2] = $14;
    while(1) {
     HEAP8[$c>>0] = 0;
     HEAP32[$iCnt>>2] = 0;
     while(1) {
      $15 = HEAP32[$p>>2]|0;
      $16 = HEAP8[$15>>0]|0;
      $17 = $16 << 24 >> 24;
      $18 = HEAP8[$c>>0]|0;
      $19 = $18&255;
      $20 = $17 | $19;
      $21 = 254 & $20;
      $22 = ($21|0)!=(0);
      if (!($22)) {
       break;
      }
      $23 = HEAP8[$c>>0]|0;
      $24 = $23&255;
      $25 = $24 & 128;
      $26 = ($25|0)==(0);
      if ($26) {
       $27 = HEAP32[$iCnt>>2]|0;
       $28 = (($27) + 1)|0;
       HEAP32[$iCnt>>2] = $28;
      }
      $29 = HEAP32[$p>>2]|0;
      $30 = ((($29)) + 1|0);
      HEAP32[$p>>2] = $30;
      $31 = HEAP8[$29>>0]|0;
      $32 = $31 << 24 >> 24;
      $33 = $32 & 128;
      $34 = $33&255;
      HEAP8[$c>>0] = $34;
     }
     $35 = HEAP32[$iCnt>>2]|0;
     $36 = HEAP32[$iCol>>2]|0;
     $37 = ($36*3)|0;
     $38 = (($37) + 1)|0;
     $39 = HEAP32[$0>>2]|0;
     $40 = ((($39)) + 40|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($38<<2)|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = (($43) + ($35))|0;
     HEAP32[$42>>2] = $44;
     $45 = HEAP32[$iCnt>>2]|0;
     $46 = ($45|0)>(0);
     $47 = $46&1;
     $48 = HEAP32[$iCol>>2]|0;
     $49 = ($48*3)|0;
     $50 = (($49) + 2)|0;
     $51 = HEAP32[$0>>2]|0;
     $52 = ((($51)) + 40|0);
     $53 = HEAP32[$52>>2]|0;
     $54 = (($53) + ($50<<2)|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (($55) + ($47))|0;
     HEAP32[$54>>2] = $56;
     $57 = HEAP32[$p>>2]|0;
     $58 = HEAP8[$57>>0]|0;
     $59 = $58 << 24 >> 24;
     $60 = ($59|0)==(0);
     if ($60) {
      break L4;
     }
     $61 = HEAP32[$p>>2]|0;
     $62 = ((($61)) + 1|0);
     HEAP32[$p>>2] = $62;
     $63 = HEAP32[$p>>2]|0;
     $64 = HEAP8[$63>>0]|0;
     $65 = $64&255;
     $66 = $65 & 128;
     $67 = ($66|0)!=(0);
     $68 = HEAP32[$p>>2]|0;
     if ($67) {
      $69 = (_sqlite3Fts3GetVarint32($68,$iCol)|0);
      $74 = $69;
     } else {
      $70 = HEAP8[$68>>0]|0;
      $71 = $70&255;
      HEAP32[$iCol>>2] = $71;
      $74 = 1;
     }
     $72 = HEAP32[$p>>2]|0;
     $73 = (($72) + ($74)|0);
     HEAP32[$p>>2] = $73;
    }
   }
  }
 } while(0);
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 12|0);
 $77 = HEAP32[$76>>2]|0;
 _fts3EvalUpdateCounts($77);
 $78 = HEAP32[$0>>2]|0;
 $79 = ((($78)) + 16|0);
 $80 = HEAP32[$79>>2]|0;
 _fts3EvalUpdateCounts($80);
 STACKTOP = sp;return;
}
function _fts3tokConnectMethod($db,$pHash,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pHash = $pHash|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $azArg = 0, $azDequote = 0, $nDequote = 0, $pMod = 0, $pTab = 0, $pTok = 0, $rc = 0, $zModule = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $pTab = sp + 28|0;
 $pMod = sp + 24|0;
 $pTok = sp + 20|0;
 $rc = sp + 16|0;
 $azDequote = sp + 12|0;
 $nDequote = sp + 8|0;
 $zModule = sp + 4|0;
 $azArg = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pHash;
 HEAP32[$3>>2] = $argc;
 HEAP32[$4>>2] = $argv;
 HEAP32[$5>>2] = $ppVtab;
 HEAP32[$6>>2] = $pzErr;
 HEAP32[$pTab>>2] = 0;
 HEAP32[$pMod>>2] = 0;
 HEAP32[$pTok>>2] = 0;
 HEAP32[$azDequote>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3_declare_vtab($7,45090)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $11;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) - 3)|0;
 HEAP32[$nDequote>>2] = $13;
 $14 = HEAP32[$nDequote>>2]|0;
 $15 = HEAP32[$4>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = (_fts3tokDequoteArray($14,$16,$azDequote)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$nDequote>>2]|0;
  $21 = ($20|0)<(1);
  if ($21) {
   HEAP32[$zModule>>2] = 39911;
  } else {
   $22 = HEAP32[$azDequote>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$zModule>>2] = $23;
  }
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$zModule>>2]|0;
  $26 = HEAP32[$6>>2]|0;
  $27 = (_fts3tokQueryTokenizer($24,$25,$pMod,$26)|0);
  HEAP32[$rc>>2] = $27;
 }
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)==(0);
 if ($29) {
  $30 = HEAP32[$azDequote>>2]|0;
  $31 = ((($30)) + 4|0);
  HEAP32[$azArg>>2] = $31;
  $32 = HEAP32[$pMod>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$nDequote>>2]|0;
  $36 = ($35|0)>(1);
  $37 = HEAP32[$nDequote>>2]|0;
  $38 = (($37) - 1)|0;
  $39 = $36 ? $38 : 0;
  $40 = HEAP32[$azArg>>2]|0;
  $41 = (FUNCTION_TABLE_iiii[$34 & 255]($39,$40,$pTok)|0);
  HEAP32[$rc>>2] = $41;
 }
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = (_sqlite3_malloc(20)|0);
  HEAP32[$pTab>>2] = $44;
  $45 = HEAP32[$pTab>>2]|0;
  $46 = ($45|0)==(0|0);
  if ($46) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if ($48) {
  $49 = HEAP32[$pTab>>2]|0;
  ;HEAP32[$49>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;
  $50 = HEAP32[$pMod>>2]|0;
  $51 = HEAP32[$pTab>>2]|0;
  $52 = ((($51)) + 12|0);
  HEAP32[$52>>2] = $50;
  $53 = HEAP32[$pTok>>2]|0;
  $54 = HEAP32[$pTab>>2]|0;
  $55 = ((($54)) + 16|0);
  HEAP32[$55>>2] = $53;
  $56 = HEAP32[$pTab>>2]|0;
  $57 = HEAP32[$5>>2]|0;
  HEAP32[$57>>2] = $56;
 } else {
  $58 = HEAP32[$pTok>>2]|0;
  $59 = ($58|0)!=(0|0);
  if ($59) {
   $60 = HEAP32[$pMod>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$pTok>>2]|0;
   (FUNCTION_TABLE_ii[$62 & 255]($63)|0);
  }
 }
 $64 = HEAP32[$azDequote>>2]|0;
 _sqlite3_free($64);
 $65 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $65;
 $66 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($66|0);
}
function _fts3tokBestIndexMethod($pVTab,$pInfo) {
 $pVTab = $pVTab|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $pInfo;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   label = 8;
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*12)|0)|0);
  $12 = ((($11)) + 5|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$i>>2]|0;
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + (($16*12)|0)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0);
   if ($22) {
    $23 = HEAP32[$i>>2]|0;
    $24 = HEAP32[$2>>2]|0;
    $25 = ((($24)) + 4|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = (($26) + (($23*12)|0)|0);
    $28 = ((($27)) + 4|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(2);
    if ($31) {
     label = 6;
     break;
    }
   }
  }
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$i>>2] = $48;
 }
 if ((label|0) == 6) {
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 20|0);
  HEAP32[$33>>2] = 1;
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<3)|0);
  HEAP32[$38>>2] = 1;
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = ((($40)) + 16|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + ($39<<3)|0);
  $44 = ((($43)) + 4|0);
  HEAP8[$44>>0] = 1;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 40|0);
  HEAPF64[$46>>3] = 1.0;
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 8) {
  $49 = HEAP32[$2>>2]|0;
  $50 = ((($49)) + 20|0);
  HEAP32[$50>>2] = 0;
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 return (0)|0;
}
function _fts3tokDisconnectMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pTab>>2] = $1;
 $2 = HEAP32[$pTab>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 (FUNCTION_TABLE_ii[$6 & 255]($9)|0);
 $10 = HEAP32[$pTab>>2]|0;
 _sqlite3_free($10);
 STACKTOP = sp;return 0;
}
function _fts3tokOpenMethod($pVTab,$ppCsr) {
 $pVTab = $pVTab|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $ppCsr;
 $3 = (_sqlite3_malloc(36)|0);
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$pCsr>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 7;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  $6 = HEAP32[$pCsr>>2]|0;
  dest=$6; stop=dest+36|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $7 = HEAP32[$pCsr>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  HEAP32[$8>>2] = $7;
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _fts3tokCloseMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 _fts3tokResetCursor($2);
 $3 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($3);
 STACKTOP = sp;return 0;
}
function _fts3tokFilterMethod($pCursor,$idxNum,$idxStr,$nVal,$apVal) {
 $pCursor = $pCursor|0;
 $idxNum = $idxNum|0;
 $idxStr = $idxStr|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $pCsr = 0, $pTab = 0, $rc = 0, $zByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $rc = sp + 16|0;
 $pCsr = sp + 12|0;
 $pTab = sp + 8|0;
 $zByte = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $idxNum;
 HEAP32[$3>>2] = $idxStr;
 HEAP32[$4>>2] = $nVal;
 HEAP32[$5>>2] = $apVal;
 HEAP32[$rc>>2] = 1;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTab>>2] = $8;
 $9 = HEAP32[$pCsr>>2]|0;
 _fts3tokResetCursor($9);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(1);
 do {
  if ($11) {
   $12 = HEAP32[$5>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = (_sqlite3_value_text($13)|0);
   HEAP32[$zByte>>2] = $14;
   $15 = HEAP32[$5>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3_value_bytes($16)|0);
   HEAP32[$nByte>>2] = $17;
   $18 = HEAP32[$nByte>>2]|0;
   $19 = (($18) + 1)|0;
   $20 = (_sqlite3_malloc($19)|0);
   $21 = HEAP32[$pCsr>>2]|0;
   $22 = ((($21)) + 4|0);
   HEAP32[$22>>2] = $20;
   $23 = HEAP32[$pCsr>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)==(0|0);
   if ($26) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $27 = HEAP32[$pCsr>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$zByte>>2]|0;
   $31 = HEAP32[$nByte>>2]|0;
   _memcpy(($29|0),($30|0),($31|0))|0;
   $32 = HEAP32[$nByte>>2]|0;
   $33 = HEAP32[$pCsr>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32)|0);
   HEAP8[$36>>0] = 0;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 12|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pTab>>2]|0;
   $43 = ((($42)) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pCsr>>2]|0;
   $46 = ((($45)) + 4|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$nByte>>2]|0;
   $49 = HEAP32[$pCsr>>2]|0;
   $50 = ((($49)) + 8|0);
   $51 = (FUNCTION_TABLE_iiiii[$41 & 255]($44,$47,$48,$50)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP32[$pTab>>2]|0;
    $55 = ((($54)) + 16|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$pCsr>>2]|0;
    $58 = ((($57)) + 8|0);
    $59 = HEAP32[$58>>2]|0;
    HEAP32[$59>>2] = $56;
   }
  }
 } while(0);
 $60 = HEAP32[$rc>>2]|0;
 $61 = ($60|0)!=(0);
 if ($61) {
  $62 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $62;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 } else {
  $63 = HEAP32[$1>>2]|0;
  $64 = (_fts3tokNextMethod($63)|0);
  HEAP32[$0>>2] = $64;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _fts3tokNextMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pCsr = sp + 8|0;
 $pTab = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTab>>2] = $3;
 $4 = HEAP32[$pCsr>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$pTab>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pCsr>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$pCsr>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$pCsr>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$pCsr>>2]|0;
 $21 = ((($20)) + 24|0);
 $22 = HEAP32[$pCsr>>2]|0;
 $23 = ((($22)) + 28|0);
 $24 = HEAP32[$pCsr>>2]|0;
 $25 = ((($24)) + 32|0);
 $26 = (FUNCTION_TABLE_iiiiiii[$12 & 255]($15,$17,$19,$21,$23,$25)|0);
 HEAP32[$rc>>2] = $26;
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)!=(0);
 if (!($28)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $29 = HEAP32[$pCsr>>2]|0;
 _fts3tokResetCursor($29);
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(101);
 if (!($31)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 HEAP32[$rc>>2] = 0;
 $32 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _fts3tokEofMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _fts3tokColumnMethod($pCursor,$pCtx,$iCol) {
 $pCursor = $pCursor|0;
 $pCtx = $pCtx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pCtx;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 0:  {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$pCsr>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  _sqlite3_result_text($5,$8,-1,(-1));
  STACKTOP = sp;return 0;
  break;
 }
 case 1:  {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$pCsr>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pCsr>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3_result_text($9,$12,$15,(-1));
  STACKTOP = sp;return 0;
  break;
 }
 case 2:  {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pCsr>>2]|0;
  $18 = ((($17)) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3_result_int($16,$19);
  STACKTOP = sp;return 0;
  break;
 }
 case 3:  {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$pCsr>>2]|0;
  $22 = ((($21)) + 28|0);
  $23 = HEAP32[$22>>2]|0;
  _sqlite3_result_int($20,$23);
  STACKTOP = sp;return 0;
  break;
 }
 default: {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$pCsr>>2]|0;
  $26 = ((($25)) + 32|0);
  $27 = HEAP32[$26>>2]|0;
  _sqlite3_result_int($24,$27);
  STACKTOP = sp;return 0;
 }
 }
 return (0)|0;
}
function _fts3tokRowidMethod($pCursor,$pRowid) {
 $pCursor = $pCursor|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pRowid;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$pCsr>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = HEAP32[$1>>2]|0;
 $9 = $8;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $7;
 STACKTOP = sp;return 0;
}
function _fts3tokDequoteArray($argc,$argv,$pazDequote) {
 $argc = $argc|0;
 $argv = $argv|0;
 $pazDequote = $pazDequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $azDequote = 0, $i = 0, $n = 0;
 var $nByte = 0, $pSpace = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rc = sp + 20|0;
 $i = sp + 16|0;
 $nByte = sp + 12|0;
 $azDequote = sp + 8|0;
 $pSpace = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $argc;
 HEAP32[$1>>2] = $argv;
 HEAP32[$2>>2] = $pazDequote;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  HEAP32[$5>>2] = 0;
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 HEAP32[$nByte>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + ($9<<2)|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_strlen($12)|0);
  $14 = (($13) + 1)|0;
  $15 = HEAP32[$nByte>>2]|0;
  $16 = (($15) + ($14))|0;
  HEAP32[$nByte>>2] = $16;
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = $19<<2;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (($20) + ($21))|0;
 $23 = (_sqlite3_malloc($22)|0);
 HEAP32[$azDequote>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$azDequote>>2]|0;
 $26 = ($25|0)==(0|0);
 if ($26) {
  HEAP32[$rc>>2] = 7;
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$azDequote>>2]|0;
 $29 = (($28) + ($27<<2)|0);
 HEAP32[$pSpace>>2] = $29;
 HEAP32[$i>>2] = 0;
 while(1) {
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$0>>2]|0;
  $32 = ($30|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + ($33<<2)|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (_strlen($36)|0);
  HEAP32[$n>>2] = $37;
  $38 = HEAP32[$pSpace>>2]|0;
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$azDequote>>2]|0;
  $41 = (($40) + ($39<<2)|0);
  HEAP32[$41>>2] = $38;
  $42 = HEAP32[$pSpace>>2]|0;
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + ($43<<2)|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$n>>2]|0;
  $48 = (($47) + 1)|0;
  _memcpy(($42|0),($46|0),($48|0))|0;
  $49 = HEAP32[$pSpace>>2]|0;
  _sqlite3Fts3Dequote($49);
  $50 = HEAP32[$n>>2]|0;
  $51 = (($50) + 1)|0;
  $52 = HEAP32[$pSpace>>2]|0;
  $53 = (($52) + ($51)|0);
  HEAP32[$pSpace>>2] = $53;
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 $56 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _fts3tokQueryTokenizer($pHash,$zName,$pp,$pzErr) {
 $pHash = $pHash|0;
 $zName = $zName|0;
 $pp = $pp|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 $nName = sp + 4|0;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $pp;
 HEAP32[$4>>2] = $pzErr;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_strlen($5)|0);
 HEAP32[$nName>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$nName>>2]|0;
 $10 = (($9) + 1)|0;
 $11 = (_sqlite3Fts3HashFind($7,$8,$10)|0);
 HEAP32[$p>>2] = $11;
 $12 = HEAP32[$p>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $14 = HEAP32[$4>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $15;
  _sqlite3Fts3ErrMsg($14,43015,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _fts3tokResetCursor($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$pTab>>2] = $6;
  $7 = HEAP32[$pTab>>2]|0;
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  (FUNCTION_TABLE_ii[$11 & 255]($14)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 8|0);
  HEAP32[$16>>2] = 0;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 _sqlite3_free($19);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 16|0);
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 20|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 24|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 28|0);
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 32|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 12|0);
 HEAP32[$33>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3InitOne($db,$iDb,$pzErrMsg) {
 $db = $db|0;
 $iDb = $iDb|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$ = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azArg = 0, $encoding = 0, $i = 0, $initData = 0, $meta = 0, $openedTransaction = 0, $or$cond = 0, $pDb = 0, $rc = 0, $size = 0, $vararg_buffer = 0, $vararg_ptr3 = 0, $xAuth = 0, $zMasterName = 0;
 var $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $rc = sp + 92|0;
 $i = sp + 88|0;
 $size = sp + 84|0;
 $pDb = sp + 80|0;
 $azArg = sp + 64|0;
 $meta = sp + 40|0;
 $initData = sp + 24|0;
 $zMasterName = sp + 20|0;
 $openedTransaction = sp + 16|0;
 $encoding = sp + 112|0;
 $zSql = sp + 12|0;
 $xAuth = sp + 8|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pzErrMsg;
 HEAP32[$openedTransaction>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(1);
 $6 = $5 ? 21260 : 21279;
 HEAP32[$zMasterName>>2] = $6;
 HEAP32[$azArg>>2] = $6;
 $7 = ((($azArg)) + 4|0);
 HEAP32[$7>>2] = 33811;
 $8 = ((($azArg)) + 8|0);
 HEAP32[$8>>2] = 45238;
 $9 = ((($azArg)) + 12|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$initData>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($initData)) + 8|0);
 HEAP32[$12>>2] = $11;
 $13 = ((($initData)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($initData)) + 4|0);
 HEAP32[$15>>2] = $14;
 (_sqlite3InitCallback($initData,3,$azArg,0)|0);
 $16 = ((($initData)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = ((($initData)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$rc>>2] = $20;
 } else {
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  HEAP32[$pDb>>2] = $25;
  $26 = HEAP32[$pDb>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   $30 = HEAP32[$2>>2]|0;
   $31 = ($30|0)==(1);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 16|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = ((($34)) + 16|0);
    $36 = ((($35)) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ((($37)) + 78|0);
    $39 = HEAP16[$38>>1]|0;
    $40 = $39&65535;
    $41 = $40 | 1;
    $42 = $41&65535;
    HEAP16[$38>>1] = $42;
   }
   HEAP32[$0>>2] = 0;
   $243 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($243|0);
  }
  $43 = HEAP32[$pDb>>2]|0;
  $44 = ((($43)) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  _sqlite3BtreeEnter($45);
  $46 = HEAP32[$pDb>>2]|0;
  $47 = ((($46)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (_sqlite3BtreeIsInReadTrans($48)|0);
  $50 = ($49|0)!=(0);
  do {
   if ($50) {
    label = 11;
   } else {
    $51 = HEAP32[$pDb>>2]|0;
    $52 = ((($51)) + 4|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (_sqlite3BtreeBeginTrans($53,0)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)!=(0);
    if ($56) {
     $57 = HEAP32[$3>>2]|0;
     $58 = HEAP32[$1>>2]|0;
     $59 = HEAP32[$rc>>2]|0;
     $60 = (_sqlite3ErrStr($59)|0);
     _sqlite3SetString($57,$58,$60);
     break;
    } else {
     HEAP32[$openedTransaction>>2] = 1;
     label = 11;
     break;
    }
   }
  } while(0);
  L16: do {
   if ((label|0) == 11) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $61 = HEAP32[$i>>2]|0;
     $62 = ($61|0)<(5);
     if (!($62)) {
      break;
     }
     $63 = HEAP32[$pDb>>2]|0;
     $64 = ((($63)) + 4|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$i>>2]|0;
     $67 = (($66) + 1)|0;
     $68 = HEAP32[$i>>2]|0;
     $69 = (($meta) + ($68<<2)|0);
     _sqlite3BtreeGetMeta($65,$67,$69);
     $70 = HEAP32[$i>>2]|0;
     $71 = (($70) + 1)|0;
     HEAP32[$i>>2] = $71;
    }
    $72 = HEAP32[$meta>>2]|0;
    $73 = HEAP32[$pDb>>2]|0;
    $74 = ((($73)) + 12|0);
    $75 = HEAP32[$74>>2]|0;
    HEAP32[$75>>2] = $72;
    $76 = ((($meta)) + 16|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = ($77|0)!=(0);
    $79 = HEAP32[$2>>2]|0;
    do {
     if ($78) {
      $80 = ($79|0)==(0);
      $81 = ((($meta)) + 16|0);
      $82 = HEAP32[$81>>2]|0;
      if ($80) {
       $83 = $82&255;
       $84 = $83&255;
       $85 = $84 & 3;
       $86 = $85&255;
       HEAP8[$encoding>>0] = $86;
       $87 = HEAP8[$encoding>>0]|0;
       $88 = $87&255;
       $89 = ($88|0)==(0);
       $$ = $89 ? 1 : $86;
       HEAP8[$encoding>>0] = $$;
       $90 = HEAP8[$encoding>>0]|0;
       $91 = HEAP32[$1>>2]|0;
       $92 = ((($91)) + 62|0);
       HEAP8[$92>>0] = $90;
       break;
      }
      $93 = HEAP32[$1>>2]|0;
      $94 = ((($93)) + 62|0);
      $95 = HEAP8[$94>>0]|0;
      $96 = $95&255;
      $97 = ($82|0)!=($96|0);
      if ($97) {
       $98 = HEAP32[$3>>2]|0;
       $99 = HEAP32[$1>>2]|0;
       _sqlite3SetString($98,$99,36297);
       HEAP32[$rc>>2] = 1;
       break L16;
      }
     } else {
      $100 = HEAP32[$1>>2]|0;
      $101 = ((($100)) + 16|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) + ($79<<4)|0);
      $104 = ((($103)) + 12|0);
      $105 = HEAP32[$104>>2]|0;
      $106 = ((($105)) + 78|0);
      $107 = HEAP16[$106>>1]|0;
      $108 = $107&65535;
      $109 = $108 | 4;
      $110 = $109&65535;
      HEAP16[$106>>1] = $110;
     }
    } while(0);
    $111 = HEAP32[$1>>2]|0;
    $112 = ((($111)) + 62|0);
    $113 = HEAP8[$112>>0]|0;
    $114 = HEAP32[$pDb>>2]|0;
    $115 = ((($114)) + 12|0);
    $116 = HEAP32[$115>>2]|0;
    $117 = ((($116)) + 77|0);
    HEAP8[$117>>0] = $113;
    $118 = HEAP32[$pDb>>2]|0;
    $119 = ((($118)) + 12|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = ((($120)) + 80|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = ($122|0)==(0);
    if ($123) {
     $124 = ((($meta)) + 8|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = (_sqlite3AbsInt32($125)|0);
     HEAP32[$size>>2] = $126;
     $127 = HEAP32[$size>>2]|0;
     $128 = ($127|0)==(0);
     $$1 = $128 ? 2000 : $126;
     HEAP32[$size>>2] = $$1;
     $129 = HEAP32[$size>>2]|0;
     $130 = HEAP32[$pDb>>2]|0;
     $131 = ((($130)) + 12|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = ((($132)) + 80|0);
     HEAP32[$133>>2] = $129;
     $134 = HEAP32[$pDb>>2]|0;
     $135 = ((($134)) + 4|0);
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$pDb>>2]|0;
     $138 = ((($137)) + 12|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ((($139)) + 80|0);
     $141 = HEAP32[$140>>2]|0;
     (_sqlite3BtreeSetCacheSize($136,$141)|0);
    }
    $142 = ((($meta)) + 4|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = $143&255;
    $145 = HEAP32[$pDb>>2]|0;
    $146 = ((($145)) + 12|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = ((($147)) + 76|0);
    HEAP8[$148>>0] = $144;
    $149 = HEAP32[$pDb>>2]|0;
    $150 = ((($149)) + 12|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = ((($151)) + 76|0);
    $153 = HEAP8[$152>>0]|0;
    $154 = $153&255;
    $155 = ($154|0)==(0);
    if ($155) {
     $156 = HEAP32[$pDb>>2]|0;
     $157 = ((($156)) + 12|0);
     $158 = HEAP32[$157>>2]|0;
     $159 = ((($158)) + 76|0);
     HEAP8[$159>>0] = 1;
    }
    $160 = HEAP32[$pDb>>2]|0;
    $161 = ((($160)) + 12|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = ((($162)) + 76|0);
    $164 = HEAP8[$163>>0]|0;
    $165 = $164&255;
    $166 = ($165|0)>(4);
    if ($166) {
     $167 = HEAP32[$3>>2]|0;
     $168 = HEAP32[$1>>2]|0;
     _sqlite3SetString($167,$168,45314);
     HEAP32[$rc>>2] = 1;
     break;
    }
    $169 = HEAP32[$2>>2]|0;
    $170 = ($169|0)==(0);
    if ($170) {
     $171 = ((($meta)) + 4|0);
     $172 = HEAP32[$171>>2]|0;
     $173 = ($172|0)>=(4);
     if ($173) {
      $174 = HEAP32[$1>>2]|0;
      $175 = ((($174)) + 24|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = $176 & -32769;
      HEAP32[$175>>2] = $177;
     }
    }
    $178 = HEAP32[$1>>2]|0;
    $179 = HEAP32[$2>>2]|0;
    $180 = HEAP32[$1>>2]|0;
    $181 = ((($180)) + 16|0);
    $182 = HEAP32[$181>>2]|0;
    $183 = (($182) + ($179<<4)|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = HEAP32[$zMasterName>>2]|0;
    HEAP32[$vararg_buffer>>2] = $184;
    $vararg_ptr3 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr3>>2] = $185;
    $186 = (_sqlite3MPrintf($178,45338,$vararg_buffer)|0);
    HEAP32[$zSql>>2] = $186;
    $187 = HEAP32[$1>>2]|0;
    $188 = ((($187)) + 288|0);
    $189 = HEAP32[$188>>2]|0;
    HEAP32[$xAuth>>2] = $189;
    $190 = HEAP32[$1>>2]|0;
    $191 = ((($190)) + 288|0);
    HEAP32[$191>>2] = 0;
    $192 = HEAP32[$1>>2]|0;
    $193 = HEAP32[$zSql>>2]|0;
    $194 = (_sqlite3_exec($192,$193,140,$initData,0)|0);
    HEAP32[$rc>>2] = $194;
    $195 = HEAP32[$xAuth>>2]|0;
    $196 = HEAP32[$1>>2]|0;
    $197 = ((($196)) + 288|0);
    HEAP32[$197>>2] = $195;
    $198 = HEAP32[$rc>>2]|0;
    $199 = ($198|0)==(0);
    if ($199) {
     $200 = ((($initData)) + 12|0);
     $201 = HEAP32[$200>>2]|0;
     HEAP32[$rc>>2] = $201;
    }
    $202 = HEAP32[$1>>2]|0;
    $203 = HEAP32[$zSql>>2]|0;
    _sqlite3DbFree($202,$203);
    $204 = HEAP32[$rc>>2]|0;
    $205 = ($204|0)==(0);
    if ($205) {
     $206 = HEAP32[$1>>2]|0;
     $207 = HEAP32[$2>>2]|0;
     (_sqlite3AnalysisLoad($206,$207)|0);
    }
    $208 = HEAP32[$1>>2]|0;
    $209 = ((($208)) + 65|0);
    $210 = HEAP8[$209>>0]|0;
    $211 = ($210<<24>>24)!=(0);
    if ($211) {
     HEAP32[$rc>>2] = 7;
     $212 = HEAP32[$1>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($212);
    }
    $213 = HEAP32[$rc>>2]|0;
    $214 = ($213|0)==(0);
    if (!($214)) {
     $215 = HEAP32[$1>>2]|0;
     $216 = ((($215)) + 24|0);
     $217 = HEAP32[$216>>2]|0;
     $218 = $217 & 65536;
     $219 = ($218|0)!=(0);
     if (!($219)) {
      break;
     }
    }
    $220 = HEAP32[$2>>2]|0;
    $221 = HEAP32[$1>>2]|0;
    $222 = ((($221)) + 16|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = (($223) + ($220<<4)|0);
    $225 = ((($224)) + 12|0);
    $226 = HEAP32[$225>>2]|0;
    $227 = ((($226)) + 78|0);
    $228 = HEAP16[$227>>1]|0;
    $229 = $228&65535;
    $230 = $229 | 1;
    $231 = $230&65535;
    HEAP16[$227>>1] = $231;
    HEAP32[$rc>>2] = 0;
   }
  } while(0);
  $232 = HEAP32[$openedTransaction>>2]|0;
  $233 = ($232|0)!=(0);
  if ($233) {
   $234 = HEAP32[$pDb>>2]|0;
   $235 = ((($234)) + 4|0);
   $236 = HEAP32[$235>>2]|0;
   (_sqlite3BtreeCommit($236)|0);
  }
 }
 $237 = HEAP32[$rc>>2]|0;
 $238 = ($237|0)==(7);
 $239 = HEAP32[$rc>>2]|0;
 $240 = ($239|0)==(3082);
 $or$cond = $238 | $240;
 if ($or$cond) {
  $241 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($241);
 }
 $242 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $242;
 $243 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($243|0);
}
function _sqlite3WalFile($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function ___errno_location() {
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[9360>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  $$0 = 9408;
 } else {
  $2 = (_pthread_self()|0);
  $3 = ((($2)) + 60|0);
  $4 = HEAP32[$3>>2]|0;
  $$0 = $4;
 }
 return ($$0|0);
}
function _strerror($e) {
 $e = $e|0;
 var $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$03 = 0, $i$03$lcssa = 0, $i$12 = 0, $s$0$lcssa = 0, $s$01 = 0, $s$1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $i$03 = 0;
 while(1) {
  $1 = (45405 + ($i$03)|0);
  $2 = HEAP8[$1>>0]|0;
  $3 = $2&255;
  $4 = ($3|0)==($e|0);
  if ($4) {
   $i$03$lcssa = $i$03;
   label = 2;
   break;
  }
  $5 = (($i$03) + 1)|0;
  $6 = ($5|0)==(87);
  if ($6) {
   $i$12 = 87;$s$01 = 45493;
   label = 5;
   break;
  } else {
   $i$03 = $5;
  }
 }
 if ((label|0) == 2) {
  $0 = ($i$03$lcssa|0)==(0);
  if ($0) {
   $s$0$lcssa = 45493;
  } else {
   $i$12 = $i$03$lcssa;$s$01 = 45493;
   label = 5;
  }
 }
 if ((label|0) == 5) {
  while(1) {
   label = 0;
   $s$1 = $s$01;
   while(1) {
    $7 = HEAP8[$s$1>>0]|0;
    $8 = ($7<<24>>24)==(0);
    $9 = ((($s$1)) + 1|0);
    if ($8) {
     $$lcssa = $9;
     break;
    } else {
     $s$1 = $9;
    }
   }
   $10 = (($i$12) + -1)|0;
   $11 = ($10|0)==(0);
   if ($11) {
    $s$0$lcssa = $$lcssa;
    break;
   } else {
    $i$12 = $10;$s$01 = $$lcssa;
    label = 5;
   }
  }
 }
 return ($s$0$lcssa|0);
}
function _fcntl($fd,$cmd,$varargs) {
 $fd = $fd|0;
 $cmd = $cmd|0;
 $varargs = $varargs|0;
 var $$ = 0, $$0 = 0, $$off = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $ex = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $switch = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer18 = 0, $vararg_buffer23 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer31 = 0, $vararg_buffer36 = 0, $vararg_buffer41 = 0;
 var $vararg_buffer46 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr27 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, $vararg_ptr44 = 0, $vararg_ptr45 = 0, $vararg_ptr49 = 0, $vararg_ptr50 = 0;
 var $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer46 = sp + 152|0;
 $vararg_buffer41 = sp + 136|0;
 $vararg_buffer36 = sp + 120|0;
 $vararg_buffer31 = sp + 104|0;
 $vararg_buffer28 = sp + 96|0;
 $vararg_buffer23 = sp + 80|0;
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $ap = sp + 176|0;
 $ex = sp + 168|0;
 HEAP32[$ap>>2] = $varargs;
 $arglist_current = HEAP32[$ap>>2]|0;
 $0 = $arglist_current;
 $1 = ((0) + 4|0);
 $expanded2 = $1;
 $expanded = (($expanded2) - 1)|0;
 $2 = (($0) + ($expanded))|0;
 $3 = ((0) + 4|0);
 $expanded6 = $3;
 $expanded5 = (($expanded6) - 1)|0;
 $expanded4 = $expanded5 ^ -1;
 $4 = $2 & $expanded4;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $arglist_next = ((($5)) + 4|0);
 HEAP32[$ap>>2] = $arglist_next;
 $7 = ($cmd|0)==(4);
 $8 = $6 | 32768;
 $$ = $7 ? $8 : $6;
 L1: do {
  switch ($cmd|0) {
  case 14:  {
   $9 = $$;
   HEAP32[$vararg_buffer>>2] = $fd;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 14;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $9;
   $10 = (___syscall221(221,($vararg_buffer|0))|0);
   $11 = (___syscall_ret($10)|0);
   $$0 = $11;
   break;
  }
  case 9:  {
   HEAP32[$vararg_buffer3>>2] = $fd;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = 16;
   $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
   HEAP32[$vararg_ptr7>>2] = $ex;
   $12 = (___syscall221(221,($vararg_buffer3|0))|0);
   switch ($12|0) {
   case -22:  {
    $13 = $$;
    HEAP32[$vararg_buffer8>>2] = $fd;
    $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
    HEAP32[$vararg_ptr11>>2] = 9;
    $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
    HEAP32[$vararg_ptr12>>2] = $13;
    $14 = (___syscall221(221,($vararg_buffer8|0))|0);
    $$0 = $14;
    break L1;
    break;
   }
   case 0:  {
    $16 = HEAP32[$ex>>2]|0;
    $17 = ($16|0)==(2);
    $18 = ((($ex)) + 4|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = (0 - ($19))|0;
    $21 = $17 ? $20 : $19;
    $$0 = $21;
    break L1;
    break;
   }
   default: {
    $15 = (___syscall_ret($12)|0);
    $$0 = $15;
    break L1;
   }
   }
   break;
  }
  case 1030:  {
   HEAP32[$vararg_buffer13>>2] = $fd;
   $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
   HEAP32[$vararg_ptr16>>2] = 1030;
   $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
   HEAP32[$vararg_ptr17>>2] = $$;
   $22 = (___syscall221(221,($vararg_buffer13|0))|0);
   $23 = ($22|0)==(-22);
   if (!($23)) {
    $24 = ($22|0)>(-1);
    if ($24) {
     HEAP32[$vararg_buffer18>>2] = $22;
     $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
     HEAP32[$vararg_ptr21>>2] = 2;
     $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
     HEAP32[$vararg_ptr22>>2] = 1;
     (___syscall221(221,($vararg_buffer18|0))|0);
    }
    $25 = (___syscall_ret($22)|0);
    $$0 = $25;
    break L1;
   }
   HEAP32[$vararg_buffer23>>2] = $fd;
   $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
   HEAP32[$vararg_ptr26>>2] = 1030;
   $vararg_ptr27 = ((($vararg_buffer23)) + 8|0);
   HEAP32[$vararg_ptr27>>2] = 0;
   $26 = (___syscall221(221,($vararg_buffer23|0))|0);
   $27 = ($26|0)==(-22);
   if ($27) {
    HEAP32[$vararg_buffer31>>2] = $fd;
    $vararg_ptr34 = ((($vararg_buffer31)) + 4|0);
    HEAP32[$vararg_ptr34>>2] = 0;
    $vararg_ptr35 = ((($vararg_buffer31)) + 8|0);
    HEAP32[$vararg_ptr35>>2] = $$;
    $30 = (___syscall221(221,($vararg_buffer31|0))|0);
    $31 = ($30|0)>(-1);
    if ($31) {
     HEAP32[$vararg_buffer36>>2] = $30;
     $vararg_ptr39 = ((($vararg_buffer36)) + 4|0);
     HEAP32[$vararg_ptr39>>2] = 2;
     $vararg_ptr40 = ((($vararg_buffer36)) + 8|0);
     HEAP32[$vararg_ptr40>>2] = 1;
     (___syscall221(221,($vararg_buffer36|0))|0);
    }
    $32 = (___syscall_ret($30)|0);
    $$0 = $32;
    break L1;
   } else {
    $28 = ($26|0)>(-1);
    if ($28) {
     HEAP32[$vararg_buffer28>>2] = $26;
     (___syscall6(6,($vararg_buffer28|0))|0);
    }
    $29 = (___syscall_ret(-22)|0);
    $$0 = $29;
    break L1;
   }
   break;
  }
  default: {
   $$off = (($cmd) + -12)|0;
   $switch = ($$off>>>0)<(5);
   if ($switch) {
    $33 = $$;
    HEAP32[$vararg_buffer41>>2] = $fd;
    $vararg_ptr44 = ((($vararg_buffer41)) + 4|0);
    HEAP32[$vararg_ptr44>>2] = $cmd;
    $vararg_ptr45 = ((($vararg_buffer41)) + 8|0);
    HEAP32[$vararg_ptr45>>2] = $33;
    $34 = (___syscall221(221,($vararg_buffer41|0))|0);
    $35 = (___syscall_ret($34)|0);
    $$0 = $35;
    break L1;
   } else {
    HEAP32[$vararg_buffer46>>2] = $fd;
    $vararg_ptr49 = ((($vararg_buffer46)) + 4|0);
    HEAP32[$vararg_ptr49>>2] = $cmd;
    $vararg_ptr50 = ((($vararg_buffer46)) + 8|0);
    HEAP32[$vararg_ptr50>>2] = $$;
    $36 = (___syscall221(221,($vararg_buffer46|0))|0);
    $37 = (___syscall_ret($36)|0);
    $$0 = $37;
    break L1;
   }
  }
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function _open($filename,$flags,$varargs) {
 $filename = $filename|0;
 $flags = $flags|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $ap = sp + 16|0;
 HEAP32[$ap>>2] = $varargs;
 $arglist_current = HEAP32[$ap>>2]|0;
 $0 = $arglist_current;
 $1 = ((0) + 4|0);
 $expanded2 = $1;
 $expanded = (($expanded2) - 1)|0;
 $2 = (($0) + ($expanded))|0;
 $3 = ((0) + 4|0);
 $expanded6 = $3;
 $expanded5 = (($expanded6) - 1)|0;
 $expanded4 = $expanded5 ^ -1;
 $4 = $2 & $expanded4;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $arglist_next = ((($5)) + 4|0);
 HEAP32[$ap>>2] = $arglist_next;
 $7 = $flags | 32768;
 HEAP32[$vararg_buffer>>2] = $filename;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $7;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $6;
 $8 = (___syscall5(5,($vararg_buffer|0))|0);
 $9 = (___syscall_ret($8)|0);
 STACKTOP = sp;return ($9|0);
}
function ___procfdname($buf,$fd) {
 $buf = $buf|0;
 $fd = $fd|0;
 var $$01 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i$0 = 0, $i$13 = 0, $i$22 = 0, $j$04 = 0, $scevgep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $i$0 = 0;
 while(1) {
  $0 = (47297 + ($i$0)|0);
  $1 = HEAP8[$0>>0]|0;
  $2 = (($buf) + ($i$0)|0);
  HEAP8[$2>>0] = $1;
  $3 = ($i$0|0)==(14);
  $4 = (($i$0) + 1)|0;
  if ($3) {
   break;
  } else {
   $i$0 = $4;
  }
 }
 $5 = ($fd|0)==(0);
 if ($5) {
  $scevgep = ((($buf)) + 14|0);
  HEAP8[$scevgep>>0] = 48;
  $6 = ((($buf)) + 15|0);
  HEAP8[$6>>0] = 0;
 } else {
  $i$13 = 14;$j$04 = $fd;
  while(1) {
   $7 = (($j$04>>>0) / 10)&-1;
   $8 = (($i$13) + 1)|0;
   $9 = ($j$04>>>0)<(10);
   if ($9) {
    $$lcssa = $8;
    break;
   } else {
    $i$13 = $8;$j$04 = $7;
   }
  }
  $10 = (($buf) + ($$lcssa)|0);
  HEAP8[$10>>0] = 0;
  $$01 = $fd;$i$22 = $$lcssa;
  while(1) {
   $11 = (($$01>>>0) % 10)&-1;
   $12 = $11 | 48;
   $13 = $12&255;
   $14 = (($i$22) + -1)|0;
   $15 = (($buf) + ($14)|0);
   HEAP8[$15>>0] = $13;
   $16 = (($$01>>>0) / 10)&-1;
   $17 = ($$01>>>0)<(10);
   if ($17) {
    break;
   } else {
    $$01 = $16;$i$22 = $14;
   }
  }
 }
 return;
}
function ___syscall_ret($r) {
 $r = $r|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($r>>>0)>(4294963200);
 if ($0) {
  $1 = (0 - ($r))|0;
  $2 = (___errno_location()|0);
  HEAP32[$2>>2] = $1;
  $$0 = -1;
 } else {
  $$0 = $r;
 }
 return ($$0|0);
}
function ___mmap($start,$len,$prot,$flags,$fd,$off) {
 $start = $start|0;
 $len = $len|0;
 $prot = $prot|0;
 $flags = $flags|0;
 $fd = $fd|0;
 $off = $off|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0;
 var $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = ($off|0)<(0);
 $1 = $0 << 31 >> 31;
 $2 = $off & 4095;
 $3 = $1 & -4096;
 $4 = ($2|0)==(0);
 $5 = ($3|0)==(0);
 $6 = $4 & $5;
 do {
  if ($6) {
   $8 = ($len>>>0)>(2147483646);
   if ($8) {
    $9 = (___errno_location()|0);
    HEAP32[$9>>2] = 12;
    $$0 = (-1);
    break;
   }
   $10 = $flags & 16;
   $11 = ($10|0)!=(0);
   if ($11) {
    _dummy1(-1);
   }
   $12 = $off >> 12;
   HEAP32[$vararg_buffer>>2] = $start;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $len;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $prot;
   $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
   HEAP32[$vararg_ptr3>>2] = $flags;
   $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
   HEAP32[$vararg_ptr4>>2] = $fd;
   $vararg_ptr5 = ((($vararg_buffer)) + 20|0);
   HEAP32[$vararg_ptr5>>2] = $12;
   $13 = (___syscall192(192,($vararg_buffer|0))|0);
   $14 = (___syscall_ret($13)|0);
   $15 = $14;
   if ($11) {
    _dummy0();
    $$0 = $15;
   } else {
    $$0 = $15;
   }
  } else {
   $7 = (___errno_location()|0);
   HEAP32[$7>>2] = 22;
   $$0 = (-1);
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function ___munmap($start,$len) {
 $start = $start|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 _dummy1(-1);
 HEAP32[$vararg_buffer>>2] = $start;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $len;
 $0 = (___syscall91(91,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 _dummy0();
 STACKTOP = sp;return ($1|0);
}
function _fchmod($fd,$mode) {
 $fd = $fd|0;
 $mode = $mode|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $mode;
 $0 = (___syscall94(94,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer2>>2] = $fd;
  $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
  HEAP32[$vararg_ptr5>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer2|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer6>>2] = $buf;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $mode;
   $5 = (___syscall15(15,($vararg_buffer6|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _fstat($fd,$st) {
 $fd = $fd|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $st;
 $0 = (___syscall197(197,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer2>>2] = $fd;
  $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
  HEAP32[$vararg_ptr5>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer2|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer6>>2] = $buf;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $st;
   $5 = (___syscall195(195,($vararg_buffer6|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _lstat($path,$buf) {
 $path = $path|0;
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $0 = (___syscall196(196,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _mkdir($path,$mode) {
 $path = $path|0;
 $mode = $mode|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $mode;
 $0 = (___syscall39(39,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _stat($path,$buf) {
 $path = $path|0;
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $0 = (___syscall195(195,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function ___lockfile($f) {
 $f = $f|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 0;
}
function ___unlockfile($f) {
 $f = $f|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function ___stdio_close($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = ((($f)) + 60|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 $2 = (___syscall6(6,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 STACKTOP = sp;return ($3|0);
}
function ___stdio_seek($f,$off,$whence) {
 $f = $f|0;
 $off = $off|0;
 $whence = $whence|0;
 var $$pre = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $ret = sp + 20|0;
 $0 = ((($f)) + 60|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $off;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $ret;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $whence;
 $2 = (___syscall140(140,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 $4 = ($3|0)<(0);
 if ($4) {
  HEAP32[$ret>>2] = -1;
  $5 = -1;
 } else {
  $$pre = HEAP32[$ret>>2]|0;
  $5 = $$pre;
 }
 STACKTOP = sp;return ($5|0);
}
function ___stdio_write($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $$0 = 0, $$phi$trans$insert = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt$0 = 0, $cnt$1 = 0, $iov$0 = 0, $iov$0$lcssa11 = 0, $iov$1 = 0, $iovcnt$0 = 0;
 var $iovcnt$0$lcssa12 = 0, $iovcnt$1 = 0, $iovs = 0, $rem$0 = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $iovs = sp + 32|0;
 $0 = ((($f)) + 28|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$iovs>>2] = $1;
 $2 = ((($iovs)) + 4|0);
 $3 = ((($f)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4;
 $6 = (($5) - ($1))|0;
 HEAP32[$2>>2] = $6;
 $7 = ((($iovs)) + 8|0);
 HEAP32[$7>>2] = $buf;
 $8 = ((($iovs)) + 12|0);
 HEAP32[$8>>2] = $len;
 $9 = (($6) + ($len))|0;
 $10 = ((($f)) + 60|0);
 $11 = ((($f)) + 44|0);
 $iov$0 = $iovs;$iovcnt$0 = 2;$rem$0 = $9;
 while(1) {
  $12 = HEAP32[9360>>2]|0;
  $13 = ($12|0)==(0|0);
  if ($13) {
   $17 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $17;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = $iov$0;
   $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
   HEAP32[$vararg_ptr7>>2] = $iovcnt$0;
   $18 = (___syscall146(146,($vararg_buffer3|0))|0);
   $19 = (___syscall_ret($18)|0);
   $cnt$0 = $19;
  } else {
   _pthread_cleanup_push((148|0),($f|0));
   $14 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer>>2] = $14;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $iov$0;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $iovcnt$0;
   $15 = (___syscall146(146,($vararg_buffer|0))|0);
   $16 = (___syscall_ret($15)|0);
   _pthread_cleanup_pop(0);
   $cnt$0 = $16;
  }
  $20 = ($rem$0|0)==($cnt$0|0);
  if ($20) {
   label = 6;
   break;
  }
  $27 = ($cnt$0|0)<(0);
  if ($27) {
   $iov$0$lcssa11 = $iov$0;$iovcnt$0$lcssa12 = $iovcnt$0;
   label = 8;
   break;
  }
  $35 = (($rem$0) - ($cnt$0))|0;
  $36 = ((($iov$0)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($cnt$0>>>0)>($37>>>0);
  if ($38) {
   $39 = HEAP32[$11>>2]|0;
   HEAP32[$0>>2] = $39;
   HEAP32[$3>>2] = $39;
   $40 = (($cnt$0) - ($37))|0;
   $41 = ((($iov$0)) + 8|0);
   $42 = (($iovcnt$0) + -1)|0;
   $$phi$trans$insert = ((($iov$0)) + 12|0);
   $$pre = HEAP32[$$phi$trans$insert>>2]|0;
   $50 = $$pre;$cnt$1 = $40;$iov$1 = $41;$iovcnt$1 = $42;
  } else {
   $43 = ($iovcnt$0|0)==(2);
   if ($43) {
    $44 = HEAP32[$0>>2]|0;
    $45 = (($44) + ($cnt$0)|0);
    HEAP32[$0>>2] = $45;
    $50 = $37;$cnt$1 = $cnt$0;$iov$1 = $iov$0;$iovcnt$1 = 2;
   } else {
    $50 = $37;$cnt$1 = $cnt$0;$iov$1 = $iov$0;$iovcnt$1 = $iovcnt$0;
   }
  }
  $46 = HEAP32[$iov$1>>2]|0;
  $47 = (($46) + ($cnt$1)|0);
  HEAP32[$iov$1>>2] = $47;
  $48 = ((($iov$1)) + 4|0);
  $49 = (($50) - ($cnt$1))|0;
  HEAP32[$48>>2] = $49;
  $iov$0 = $iov$1;$iovcnt$0 = $iovcnt$1;$rem$0 = $35;
 }
 if ((label|0) == 6) {
  $21 = HEAP32[$11>>2]|0;
  $22 = ((($f)) + 48|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + ($23)|0);
  $25 = ((($f)) + 16|0);
  HEAP32[$25>>2] = $24;
  $26 = $21;
  HEAP32[$0>>2] = $26;
  HEAP32[$3>>2] = $26;
  $$0 = $len;
 }
 else if ((label|0) == 8) {
  $28 = ((($f)) + 16|0);
  HEAP32[$28>>2] = 0;
  HEAP32[$0>>2] = 0;
  HEAP32[$3>>2] = 0;
  $29 = HEAP32[$f>>2]|0;
  $30 = $29 | 32;
  HEAP32[$f>>2] = $30;
  $31 = ($iovcnt$0$lcssa12|0)==(2);
  if ($31) {
   $$0 = 0;
  } else {
   $32 = ((($iov$0$lcssa11)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($len) - ($33))|0;
   $$0 = $34;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___stdout_write($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tio = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $tio = sp + 12|0;
 $0 = ((($f)) + 36|0);
 HEAP32[$0>>2] = 163;
 $1 = HEAP32[$f>>2]|0;
 $2 = $1 & 64;
 $3 = ($2|0)==(0);
 if ($3) {
  $4 = ((($f)) + 60|0);
  $5 = HEAP32[$4>>2]|0;
  HEAP32[$vararg_buffer>>2] = $5;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 21505;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $tio;
  $6 = (___syscall54(54,($vararg_buffer|0))|0);
  $7 = ($6|0)==(0);
  if (!($7)) {
   $8 = ((($f)) + 75|0);
   HEAP8[$8>>0] = -1;
  }
 }
 $9 = (___stdio_write($f,$buf,$len)|0);
 STACKTOP = sp;return ($9|0);
}
function _fflush($f) {
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$012 = 0, $$014 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $phitmp = 0, $r$0$lcssa = 0, $r$03 = 0, $r$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($f|0)==(0|0);
 do {
  if ($0) {
   $7 = HEAP32[9404>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $27 = 0;
   } else {
    $9 = HEAP32[9404>>2]|0;
    $10 = (_fflush($9)|0);
    $27 = $10;
   }
   ___lock(((9388)|0));
   $$012 = HEAP32[(9384)>>2]|0;
   $11 = ($$012|0)==(0|0);
   if ($11) {
    $r$0$lcssa = $27;
   } else {
    $$014 = $$012;$r$03 = $27;
    while(1) {
     $12 = ((($$014)) + 76|0);
     $13 = HEAP32[$12>>2]|0;
     $14 = ($13|0)>(-1);
     if ($14) {
      $15 = (___lockfile($$014)|0);
      $24 = $15;
     } else {
      $24 = 0;
     }
     $16 = ((($$014)) + 20|0);
     $17 = HEAP32[$16>>2]|0;
     $18 = ((($$014)) + 28|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = ($17>>>0)>($19>>>0);
     if ($20) {
      $21 = (___fflush_unlocked($$014)|0);
      $22 = $21 | $r$03;
      $r$1 = $22;
     } else {
      $r$1 = $r$03;
     }
     $23 = ($24|0)==(0);
     if (!($23)) {
      ___unlockfile($$014);
     }
     $25 = ((($$014)) + 56|0);
     $$01 = HEAP32[$25>>2]|0;
     $26 = ($$01|0)==(0|0);
     if ($26) {
      $r$0$lcssa = $r$1;
      break;
     } else {
      $$014 = $$01;$r$03 = $r$1;
     }
    }
   }
   ___unlock(((9388)|0));
   $$0 = $r$0$lcssa;
  } else {
   $1 = ((($f)) + 76|0);
   $2 = HEAP32[$1>>2]|0;
   $3 = ($2|0)>(-1);
   if (!($3)) {
    $4 = (___fflush_unlocked($f)|0);
    $$0 = $4;
    break;
   }
   $5 = (___lockfile($f)|0);
   $phitmp = ($5|0)==(0);
   $6 = (___fflush_unlocked($f)|0);
   if ($phitmp) {
    $$0 = $6;
   } else {
    ___unlockfile($f);
    $$0 = $6;
   }
  }
 } while(0);
 return ($$0|0);
}
function _qsort($base,$nel,$width,$cmp) {
 $base = $base|0;
 $nel = $nel|0;
 $width = $width|0;
 $cmp = $cmp|0;
 var $$0$i = 0, $$0$i30 = 0, $$02$i$i = 0, $$02$i3$i = 0, $$lcssa = 0, $$lcssa57 = 0, $$phi$trans$insert$i = 0, $$pre = 0, $$pre$i = 0, $$pre$i11 = 0, $$pre$i20 = 0, $$pre$i5 = 0, $$pre$i8 = 0, $$pre1$i = 0, $$pre1$i12 = 0, $$pre1$i27$pre = 0, $$pre1$i6 = 0, $$pre1$i9 = 0, $$sum = 0, $$sum2 = 0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $8$phi = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $head$0$lcssa = 0, $head$036 = 0;
 var $head$1$be = 0, $head$153 = 0, $i$0 = 0, $lp = 0, $nTrailingZeros$03$i$i = 0, $nTrailingZeros$03$i2$i = 0, $nTrailingZeros$03$i2$i$lcssa = 0, $or$cond = 0, $or$cond48 = 0, $or$cond4852 = 0, $or$cond51 = 0, $p = 0, $pshift$0$lcssa = 0, $pshift$037 = 0, $pshift$1 = 0, $pshift$2$be = 0, $pshift$254 = 0, $sum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $lp = sp + 8|0;
 $p = sp;
 $0 = Math_imul($width, $nel)|0;
 $1 = $p;
 $2 = $1;
 HEAP32[$2>>2] = 1;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = ($0|0)==(0);
 if (!($5)) {
  $$sum = (($0) - ($width))|0;
  $6 = ((($lp)) + 4|0);
  HEAP32[$6>>2] = $width;
  HEAP32[$lp>>2] = $width;
  $10 = $width;$8 = $width;$i$0 = 2;
  while(1) {
   $7 = (($8) + ($width))|0;
   $9 = (($7) + ($10))|0;
   $11 = (($lp) + ($i$0<<2)|0);
   HEAP32[$11>>2] = $9;
   $12 = ($9>>>0)<($0>>>0);
   $13 = (($i$0) + 1)|0;
   if ($12) {
    $8$phi = $10;$10 = $9;$i$0 = $13;$8 = $8$phi;
   } else {
    break;
   }
  }
  $14 = (0 - ($width))|0;
  $15 = (($base) + ($$sum)|0);
  $16 = ($$sum|0)>(0);
  $$phi$trans$insert$i = ((($p)) + 4|0);
  if ($16) {
   $17 = $15;
   $19 = 1;$head$036 = $base;$pshift$037 = 1;
   while(1) {
    $18 = $19 & 3;
    $20 = ($18|0)==(3);
    do {
     if ($20) {
      _sift($head$036,$width,$cmp,$pshift$037,$lp);
      $$pre$i = HEAP32[$p>>2]|0;
      $$pre1$i = HEAP32[$$phi$trans$insert$i>>2]|0;
      $21 = $$pre$i >>> 2;
      $22 = $$pre1$i << 30;
      $23 = $22 | $21;
      HEAP32[$p>>2] = $23;
      $24 = $$pre1$i >>> 2;
      HEAP32[$$phi$trans$insert$i>>2] = $24;
      $25 = (($pshift$037) + 2)|0;
      $48 = $23;$pshift$1 = $25;
     } else {
      $26 = (($pshift$037) + -1)|0;
      $27 = (($lp) + ($26<<2)|0);
      $28 = HEAP32[$27>>2]|0;
      $29 = $head$036;
      $30 = (($17) - ($29))|0;
      $31 = ($28>>>0)<($30>>>0);
      if ($31) {
       _sift($head$036,$width,$cmp,$pshift$037,$lp);
      } else {
       _trinkle($head$036,$width,$cmp,$p,$pshift$037,0,$lp);
      }
      $32 = ($pshift$037|0)==(1);
      if ($32) {
       $$pre$i5 = HEAP32[$$phi$trans$insert$i>>2]|0;
       $$pre1$i6 = HEAP32[$p>>2]|0;
       $33 = $$pre$i5 << 1;
       $34 = $$pre1$i6 >>> 31;
       $35 = $34 | $33;
       HEAP32[$$phi$trans$insert$i>>2] = $35;
       $36 = $$pre1$i6 << 1;
       HEAP32[$p>>2] = $36;
       $48 = $36;$pshift$1 = 0;
       break;
      }
      $37 = ($26>>>0)>(31);
      if ($37) {
       $38 = (($pshift$037) + -33)|0;
       $39 = HEAP32[$p>>2]|0;
       HEAP32[$$phi$trans$insert$i>>2] = $39;
       HEAP32[$p>>2] = 0;
       $$0$i = $38;$41 = $39;$44 = 0;
      } else {
       $$pre$i11 = HEAP32[$$phi$trans$insert$i>>2]|0;
       $$pre1$i12 = HEAP32[$p>>2]|0;
       $$0$i = $26;$41 = $$pre$i11;$44 = $$pre1$i12;
      }
      $40 = $41 << $$0$i;
      $42 = (32 - ($$0$i))|0;
      $43 = $44 >>> $42;
      $45 = $43 | $40;
      HEAP32[$$phi$trans$insert$i>>2] = $45;
      $46 = $44 << $$0$i;
      HEAP32[$p>>2] = $46;
      $48 = $46;$pshift$1 = 1;
     }
    } while(0);
    $47 = $48 | 1;
    HEAP32[$p>>2] = $47;
    $49 = (($head$036) + ($width)|0);
    $50 = ($49>>>0)<($15>>>0);
    if ($50) {
     $19 = $47;$head$036 = $49;$pshift$037 = $pshift$1;
    } else {
     $head$0$lcssa = $49;$pshift$0$lcssa = $pshift$1;
     break;
    }
   }
  } else {
   $head$0$lcssa = $base;$pshift$0$lcssa = 1;
  }
  _trinkle($head$0$lcssa,$width,$cmp,$p,$pshift$0$lcssa,0,$lp);
  $51 = ((($p)) + 4|0);
  $52 = ($pshift$0$lcssa|0)==(1);
  $53 = HEAP32[$p>>2]|0;
  $54 = ($53|0)==(1);
  $or$cond51 = $52 & $54;
  $55 = HEAP32[$51>>2]|0;
  $56 = ($55|0)==(0);
  $or$cond4852 = $or$cond51 & $56;
  if (!($or$cond4852)) {
   $59 = $53;$head$153 = $head$0$lcssa;$pshift$254 = $pshift$0$lcssa;
   while(1) {
    $57 = ($pshift$254|0)<(2);
    if ($57) {
     $58 = (($59) + -1)|0;
     $60 = ($58|0)==(0);
     do {
      if ($60) {
       $81 = 32;
       label = 30;
      } else {
       $61 = $58 & 1;
       $62 = ($61|0)==(0);
       if ($62) {
        $$02$i$i = $58;$nTrailingZeros$03$i$i = 0;
        while(1) {
         $63 = (($nTrailingZeros$03$i$i) + 1)|0;
         $64 = $$02$i$i >>> 1;
         $65 = $64 & 1;
         $66 = ($65|0)==(0);
         if ($66) {
          $$02$i$i = $64;$nTrailingZeros$03$i$i = $63;
         } else {
          $$lcssa = $63;
          break;
         }
        }
        $67 = ($$lcssa|0)==(0);
        if ($67) {
         label = 24;
        } else {
         $79 = $$lcssa;
        }
       } else {
        label = 24;
       }
       if ((label|0) == 24) {
        label = 0;
        $68 = HEAP32[$$phi$trans$insert$i>>2]|0;
        $69 = ($68|0)==(0);
        if ($69) {
         $81 = 64;
         label = 30;
         break;
        }
        $70 = $68 & 1;
        $71 = ($70|0)==(0);
        if ($71) {
         $$02$i3$i = $68;$nTrailingZeros$03$i2$i = 0;
        } else {
         $$0$i30 = 0;$84 = $59;$87 = $68;$91 = 0;
         break;
        }
        while(1) {
         $72 = (($nTrailingZeros$03$i2$i) + 1)|0;
         $73 = $$02$i3$i >>> 1;
         $74 = $73 & 1;
         $75 = ($74|0)==(0);
         if ($75) {
          $$02$i3$i = $73;$nTrailingZeros$03$i2$i = $72;
         } else {
          $$lcssa57 = $72;$nTrailingZeros$03$i2$i$lcssa = $nTrailingZeros$03$i2$i;
          break;
         }
        }
        $76 = (($nTrailingZeros$03$i2$i$lcssa) + 33)|0;
        $77 = ($$lcssa57|0)==(0);
        if ($77) {
         $$0$i30 = 0;$84 = $59;$87 = $68;$91 = 0;
         break;
        } else {
         $79 = $76;
        }
       }
       $78 = ($79>>>0)>(31);
       if ($78) {
        $81 = $79;
        label = 30;
       } else {
        $$pre1$i27$pre = HEAP32[$$phi$trans$insert$i>>2]|0;
        $$0$i30 = $79;$84 = $59;$87 = $$pre1$i27$pre;$91 = $79;
       }
      }
     } while(0);
     if ((label|0) == 30) {
      label = 0;
      $80 = (($81) + -32)|0;
      $82 = HEAP32[$$phi$trans$insert$i>>2]|0;
      HEAP32[$p>>2] = $82;
      HEAP32[$$phi$trans$insert$i>>2] = 0;
      $$0$i30 = $80;$84 = $82;$87 = 0;$91 = $81;
     }
     $83 = $84 >>> $$0$i30;
     $85 = (32 - ($$0$i30))|0;
     $86 = $87 << $85;
     $88 = $86 | $83;
     HEAP32[$p>>2] = $88;
     $89 = $87 >>> $$0$i30;
     HEAP32[$$phi$trans$insert$i>>2] = $89;
     $90 = (($91) + ($pshift$254))|0;
     $$pre = (($head$153) + ($14)|0);
     $head$1$be = $$pre;$pshift$2$be = $90;
    } else {
     $$pre$i20 = HEAP32[$$phi$trans$insert$i>>2]|0;
     $92 = $$pre$i20 << 2;
     $93 = $59 >>> 30;
     $94 = $93 | $92;
     $95 = (($pshift$254) + -2)|0;
     $96 = $59 << 1;
     $97 = $96 & 2147483646;
     $98 = $93 << 31;
     $99 = $97 | $98;
     $100 = $99 ^ 3;
     HEAP32[$p>>2] = $100;
     $101 = $94 >>> 1;
     HEAP32[$$phi$trans$insert$i>>2] = $101;
     $102 = (($lp) + ($95<<2)|0);
     $103 = HEAP32[$102>>2]|0;
     $sum = (($103) + ($width))|0;
     $$sum2 = (0 - ($sum))|0;
     $104 = (($head$153) + ($$sum2)|0);
     $105 = (($pshift$254) + -1)|0;
     _trinkle($104,$width,$cmp,$p,$105,1,$lp);
     $$pre$i8 = HEAP32[$$phi$trans$insert$i>>2]|0;
     $$pre1$i9 = HEAP32[$p>>2]|0;
     $106 = $$pre$i8 << 1;
     $107 = $$pre1$i9 >>> 31;
     $108 = $107 | $106;
     HEAP32[$$phi$trans$insert$i>>2] = $108;
     $109 = $$pre1$i9 << 1;
     $110 = $109 | 1;
     HEAP32[$p>>2] = $110;
     $111 = (($head$153) + ($14)|0);
     _trinkle($111,$width,$cmp,$p,$95,1,$lp);
     $head$1$be = $111;$pshift$2$be = $95;
    }
    $112 = ($pshift$2$be|0)==(1);
    $113 = HEAP32[$p>>2]|0;
    $114 = ($113|0)==(1);
    $or$cond = $112 & $114;
    $115 = HEAP32[$51>>2]|0;
    $116 = ($115|0)==(0);
    $or$cond48 = $or$cond & $116;
    if ($or$cond48) {
     break;
    } else {
     $59 = $113;$head$153 = $head$1$be;$pshift$254 = $pshift$2$be;
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _memcmp($vl,$vr,$n) {
 $vl = $vl|0;
 $vr = $vr|0;
 $n = $n|0;
 var $$03 = 0, $$lcssa = 0, $$lcssa19 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$04 = 0, $r$05 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 L1: do {
  if ($0) {
   $11 = 0;
  } else {
   $$03 = $n;$l$04 = $vl;$r$05 = $vr;
   while(1) {
    $1 = HEAP8[$l$04>>0]|0;
    $2 = HEAP8[$r$05>>0]|0;
    $3 = ($1<<24>>24)==($2<<24>>24);
    if (!($3)) {
     $$lcssa = $1;$$lcssa19 = $2;
     break;
    }
    $4 = (($$03) + -1)|0;
    $5 = ((($l$04)) + 1|0);
    $6 = ((($r$05)) + 1|0);
    $7 = ($4|0)==(0);
    if ($7) {
     $11 = 0;
     break L1;
    } else {
     $$03 = $4;$l$04 = $5;$r$05 = $6;
    }
   }
   $8 = $$lcssa&255;
   $9 = $$lcssa19&255;
   $10 = (($8) - ($9))|0;
   $11 = $10;
  }
 } while(0);
 return ($11|0);
}
function _strcmp($l,$r) {
 $l = $l|0;
 $r = $r|0;
 var $$014 = 0, $$05 = 0, $$lcssa = 0, $$lcssa2 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = HEAP8[$l>>0]|0;
 $1 = HEAP8[$r>>0]|0;
 $2 = ($0<<24>>24)!=($1<<24>>24);
 $3 = ($0<<24>>24)==(0);
 $or$cond3 = $3 | $2;
 if ($or$cond3) {
  $$lcssa = $0;$$lcssa2 = $1;
 } else {
  $$014 = $l;$$05 = $r;
  while(1) {
   $4 = ((($$014)) + 1|0);
   $5 = ((($$05)) + 1|0);
   $6 = HEAP8[$4>>0]|0;
   $7 = HEAP8[$5>>0]|0;
   $8 = ($6<<24>>24)!=($7<<24>>24);
   $9 = ($6<<24>>24)==(0);
   $or$cond = $9 | $8;
   if ($or$cond) {
    $$lcssa = $6;$$lcssa2 = $7;
    break;
   } else {
    $$014 = $4;$$05 = $5;
   }
  }
 }
 $10 = $$lcssa&255;
 $11 = $$lcssa2&255;
 $12 = (($10) - ($11))|0;
 return ($12|0);
}
function ___strdup($s) {
 $s = $s|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strlen($s)|0);
 $1 = (($0) + 1)|0;
 $2 = (_malloc($1)|0);
 $3 = ($2|0)==(0|0);
 if ($3) {
  $$0 = 0;
 } else {
  _memcpy(($2|0),($s|0),($1|0))|0;
  $$0 = $2;
 }
 return ($$0|0);
}
function _strlen($s) {
 $s = $s|0;
 var $$0 = 0, $$01$lcssa = 0, $$014 = 0, $$1$lcssa = 0, $$lcssa20 = 0, $$pn = 0, $$pn15 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w$0 = 0, $w$0$lcssa = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $s;
 $1 = $0 & 3;
 $2 = ($1|0)==(0);
 L1: do {
  if ($2) {
   $$01$lcssa = $s;
   label = 4;
  } else {
   $$014 = $s;$21 = $0;
   while(1) {
    $3 = HEAP8[$$014>>0]|0;
    $4 = ($3<<24>>24)==(0);
    if ($4) {
     $$pn = $21;
     break L1;
    }
    $5 = ((($$014)) + 1|0);
    $6 = $5;
    $7 = $6 & 3;
    $8 = ($7|0)==(0);
    if ($8) {
     $$01$lcssa = $5;
     label = 4;
     break;
    } else {
     $$014 = $5;$21 = $6;
    }
   }
  }
 } while(0);
 if ((label|0) == 4) {
  $w$0 = $$01$lcssa;
  while(1) {
   $9 = HEAP32[$w$0>>2]|0;
   $10 = (($9) + -16843009)|0;
   $11 = $9 & -2139062144;
   $12 = $11 ^ -2139062144;
   $13 = $12 & $10;
   $14 = ($13|0)==(0);
   $15 = ((($w$0)) + 4|0);
   if ($14) {
    $w$0 = $15;
   } else {
    $$lcssa20 = $9;$w$0$lcssa = $w$0;
    break;
   }
  }
  $16 = $$lcssa20&255;
  $17 = ($16<<24>>24)==(0);
  if ($17) {
   $$1$lcssa = $w$0$lcssa;
  } else {
   $$pn15 = $w$0$lcssa;
   while(1) {
    $18 = ((($$pn15)) + 1|0);
    $$pre = HEAP8[$18>>0]|0;
    $19 = ($$pre<<24>>24)==(0);
    if ($19) {
     $$1$lcssa = $18;
     break;
    } else {
     $$pn15 = $18;
    }
   }
  }
  $20 = $$1$lcssa;
  $$pn = $20;
 }
 $$0 = (($$pn) - ($0))|0;
 return ($$0|0);
}
function _strncmp($_l,$_r,$n) {
 $_l = $_l|0;
 $_r = $_r|0;
 $n = $n|0;
 var $$03 = 0, $$08 = 0, $$08$in = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $l$06 = 0, $or$cond = 0, $or$cond4 = 0, $r$0$lcssa = 0, $r$07 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 if ($0) {
  $$03 = 0;
 } else {
  $1 = HEAP8[$_l>>0]|0;
  $2 = ($1<<24>>24)==(0);
  L3: do {
   if ($2) {
    $13 = 0;$r$0$lcssa = $_r;
   } else {
    $$08$in = $n;$7 = $1;$l$06 = $_l;$r$07 = $_r;
    while(1) {
     $$08 = (($$08$in) + -1)|0;
     $3 = HEAP8[$r$07>>0]|0;
     $4 = ($3<<24>>24)!=(0);
     $5 = ($$08|0)!=(0);
     $or$cond = $5 & $4;
     $6 = ($7<<24>>24)==($3<<24>>24);
     $or$cond4 = $6 & $or$cond;
     if (!($or$cond4)) {
      $13 = $7;$r$0$lcssa = $r$07;
      break L3;
     }
     $8 = ((($l$06)) + 1|0);
     $9 = ((($r$07)) + 1|0);
     $10 = HEAP8[$8>>0]|0;
     $11 = ($10<<24>>24)==(0);
     if ($11) {
      $13 = 0;$r$0$lcssa = $9;
      break;
     } else {
      $$08$in = $$08;$7 = $10;$l$06 = $8;$r$07 = $9;
     }
    }
   }
  } while(0);
  $12 = $13&255;
  $14 = HEAP8[$r$0$lcssa>>0]|0;
  $15 = $14&255;
  $16 = (($12) - ($15))|0;
  $$03 = $16;
 }
 return ($$03|0);
}
function _access($filename,$amode) {
 $filename = $filename|0;
 $amode = $amode|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $filename;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $amode;
 $0 = (___syscall33(33,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _close($fd) {
 $fd = $fd|0;
 var $$ = 0, $0 = 0, $1 = 0, $2 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $0 = (___syscall6(6,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-4);
 $$ = $1 ? -115 : $0;
 $2 = (___syscall_ret($$)|0);
 STACKTOP = sp;return ($2|0);
}
function _fchown($fd,$uid,$gid) {
 $fd = $fd|0;
 $uid = $uid|0;
 $gid = $gid|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $buf = sp + 36|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $uid;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $gid;
 $0 = (___syscall207(207,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer3>>2] = $fd;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer3|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer7>>2] = $buf;
   $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
   HEAP32[$vararg_ptr10>>2] = $uid;
   $vararg_ptr11 = ((($vararg_buffer7)) + 8|0);
   HEAP32[$vararg_ptr11>>2] = $gid;
   $5 = (___syscall212(212,($vararg_buffer7|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _fsync($fd) {
 $fd = $fd|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $0 = (___syscall118(118,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _ftruncate($fd,$length) {
 $fd = $fd|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = ($length|0)<(0);
 $1 = $0 << 31 >> 31;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $length;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $1;
 $2 = (___syscall194(194,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _getcwd($buf,$size) {
 $buf = $buf|0;
 $size = $size|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $tmp = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 4112|0;
 $vararg_buffer = sp;
 $tmp = sp + 8|0;
 $0 = ($buf|0)==(0|0);
 if ($0) {
  $$0 = 4096;$$01 = $tmp;
  label = 4;
 } else {
  $1 = ($size|0)==(0);
  if ($1) {
   $2 = (___errno_location()|0);
   HEAP32[$2>>2] = 22;
   $$02 = 0;
  } else {
   $$0 = $size;$$01 = $buf;
   label = 4;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$vararg_buffer>>2] = $$01;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $$0;
  $3 = (___syscall183(183,($vararg_buffer|0))|0);
  $4 = (___syscall_ret($3)|0);
  $5 = ($4|0)<(0);
  if ($5) {
   $$02 = 0;
  } else {
   $6 = ($$01|0)==($tmp|0);
   if ($6) {
    $7 = (___strdup($tmp)|0);
    $$02 = $7;
   } else {
    $$02 = $$01;
   }
  }
 }
 STACKTOP = sp;return ($$02|0);
}
function _geteuid() {
 var $0 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = (___syscall201(201,($vararg_buffer|0))|0);
 STACKTOP = sp;return ($0|0);
}
function _getpid() {
 var $0 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = (___syscall20(20,($vararg_buffer|0))|0);
 STACKTOP = sp;return ($0|0);
}
function _lseek($fd,$offset,$whence) {
 $fd = $fd|0;
 $offset = $offset|0;
 $whence = $whence|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $result = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $result = sp + 20|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $offset;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $result;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $whence;
 $0 = (___syscall140(140,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 $2 = ($1|0)!=(0);
 $3 = HEAP32[$result>>2]|0;
 $4 = $2 ? -1 : $3;
 STACKTOP = sp;return ($4|0);
}
function _read($fd,$buf,$count) {
 $fd = $fd|0;
 $buf = $buf|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $count;
 $0 = (___syscall3(3,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _readlink($path,$buf,$bufsize) {
 $path = $path|0;
 $buf = $buf|0;
 $bufsize = $bufsize|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $bufsize;
 $0 = (___syscall85(85,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _rmdir($path) {
 $path = $path|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $0 = (___syscall40(40,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _sleep($seconds) {
 $seconds = $seconds|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $tv = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $tv = sp;
 HEAP32[$tv>>2] = $seconds;
 $0 = ((($tv)) + 4|0);
 HEAP32[$0>>2] = 0;
 $1 = (_nanosleep(($tv|0),($tv|0))|0);
 $2 = ($1|0)==(0);
 $3 = HEAP32[$tv>>2]|0;
 $$0 = $2 ? 0 : $3;
 STACKTOP = sp;return ($$0|0);
}
function _unlink($path) {
 $path = $path|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $0 = (___syscall10(10,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _write($fd,$buf,$count) {
 $fd = $fd|0;
 $buf = $buf|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $count;
 $0 = (___syscall4(4,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _cleanup392($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($p)) + 68|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  ___unlockfile($p);
 }
 return;
}
function ___fflush_unlocked($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 20|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($f)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($1>>>0)>($3>>>0);
 if ($4) {
  $5 = ((($f)) + 36|0);
  $6 = HEAP32[$5>>2]|0;
  (FUNCTION_TABLE_iiii[$6 & 255]($f,0,0)|0);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)==(0|0);
  if ($8) {
   $$0 = -1;
  } else {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $9 = ((($f)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($f)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($10>>>0)<($12>>>0);
  if ($13) {
   $14 = ((($f)) + 40|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $10;
   $17 = $12;
   $18 = (($16) - ($17))|0;
   (FUNCTION_TABLE_iiii[$15 & 255]($f,$18,1)|0);
  }
  $19 = ((($f)) + 16|0);
  HEAP32[$19>>2] = 0;
  HEAP32[$2>>2] = 0;
  HEAP32[$0>>2] = 0;
  HEAP32[$11>>2] = 0;
  HEAP32[$9>>2] = 0;
  $$0 = 0;
 }
 return ($$0|0);
}
function _sift($head,$width,$cmp,$pshift,$lp) {
 $head = $head|0;
 $width = $width|0;
 $cmp = $cmp|0;
 $pshift = $pshift|0;
 $lp = $lp|0;
 var $$0$be = 0, $$01$be = 0, $$012 = 0, $$03 = 0, $$pre = 0, $$sum = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ar = 0, $i$0$lcssa = 0, $i$04 = 0, $sum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $ar = sp;
 HEAP32[$ar>>2] = $head;
 $0 = ($pshift|0)>(1);
 L1: do {
  if ($0) {
   $1 = (0 - ($width))|0;
   $$012 = $pshift;$$03 = $head;$7 = $head;$i$04 = 1;
   while(1) {
    $2 = (($$03) + ($1)|0);
    $3 = (($$012) + -2)|0;
    $4 = (($lp) + ($3<<2)|0);
    $5 = HEAP32[$4>>2]|0;
    $sum = (($5) + ($width))|0;
    $$sum = (0 - ($sum))|0;
    $6 = (($$03) + ($$sum)|0);
    $8 = (FUNCTION_TABLE_iii[$cmp & 255]($7,$6)|0);
    $9 = ($8|0)>(-1);
    if ($9) {
     $10 = (FUNCTION_TABLE_iii[$cmp & 255]($7,$2)|0);
     $11 = ($10|0)>(-1);
     if ($11) {
      $i$0$lcssa = $i$04;
      break L1;
     }
    }
    $12 = (FUNCTION_TABLE_iii[$cmp & 255]($6,$2)|0);
    $13 = ($12|0)>(-1);
    $14 = (($i$04) + 1)|0;
    $15 = (($ar) + ($i$04<<2)|0);
    if ($13) {
     HEAP32[$15>>2] = $6;
     $16 = (($$012) + -1)|0;
     $$0$be = $6;$$01$be = $16;
    } else {
     HEAP32[$15>>2] = $2;
     $$0$be = $2;$$01$be = $3;
    }
    $17 = ($$01$be|0)>(1);
    if (!($17)) {
     $i$0$lcssa = $14;
     break L1;
    }
    $$pre = HEAP32[$ar>>2]|0;
    $$012 = $$01$be;$$03 = $$0$be;$7 = $$pre;$i$04 = $14;
   }
  } else {
   $i$0$lcssa = 1;
  }
 } while(0);
 _cycle($width,$ar,$i$0$lcssa);
 STACKTOP = sp;return;
}
function _trinkle($head,$width,$cmp,$pp,$pshift,$trusty,$lp) {
 $head = $head|0;
 $width = $width|0;
 $cmp = $cmp|0;
 $pp = $pp|0;
 $pshift = $pshift|0;
 $trusty = $trusty|0;
 $lp = $lp|0;
 var $$0$i = 0, $$0$lcssa = 0, $$0$lcssa49 = 0, $$01162 = 0, $$01162$phi = 0, $$02$i$i = 0, $$02$i3$i = 0, $$02$lcssa = 0, $$02$lcssa51 = 0, $$02964 = 0, $$03$lcssa = 0, $$03865 = 0, $$lcssa = 0, $$lcssa75 = 0, $$pre = 0, $$sum = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $ar = 0, $i$0$lcssa = 0, $i$0$lcssa50 = 0, $i$01063 = 0, $nTrailingZeros$03$i$i = 0, $nTrailingZeros$03$i2$i = 0, $nTrailingZeros$03$i2$i$lcssa = 0, $or$cond = 0, $phitmp = 0, $sum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $ar = sp;
 $0 = HEAP32[$pp>>2]|0;
 $1 = ((($pp)) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$ar>>2] = $head;
 $3 = (0 - ($width))|0;
 $4 = ($0|0)!=(1);
 $5 = ($2|0)!=(0);
 $6 = $5 | $4;
 L1: do {
  if ($6) {
   $7 = (($lp) + ($pshift<<2)|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (0 - ($8))|0;
   $10 = (($head) + ($9)|0);
   $11 = (FUNCTION_TABLE_iii[$cmp & 255]($10,$head)|0);
   $12 = ($11|0)<(1);
   if ($12) {
    $$0$lcssa = $head;$$02$lcssa = $pshift;$$03$lcssa = $trusty;$i$0$lcssa = 1;
    label = 19;
   } else {
    $phitmp = ($trusty|0)==(0);
    $$01162 = $head;$$02964 = $pshift;$$03865 = $phitmp;$18 = $10;$27 = $0;$37 = $2;$i$01063 = 1;
    while(1) {
     $13 = ($$02964|0)>(1);
     $or$cond = $$03865 & $13;
     if ($or$cond) {
      $14 = (($$01162) + ($3)|0);
      $15 = (($$02964) + -2)|0;
      $16 = (($lp) + ($15<<2)|0);
      $17 = HEAP32[$16>>2]|0;
      $19 = (FUNCTION_TABLE_iii[$cmp & 255]($14,$18)|0);
      $20 = ($19|0)>(-1);
      if ($20) {
       $$0$lcssa49 = $$01162;$$02$lcssa51 = $$02964;$i$0$lcssa50 = $i$01063;
       label = 20;
       break L1;
      }
      $sum = (($17) + ($width))|0;
      $$sum = (0 - ($sum))|0;
      $21 = (($$01162) + ($$sum)|0);
      $22 = (FUNCTION_TABLE_iii[$cmp & 255]($21,$18)|0);
      $23 = ($22|0)>(-1);
      if ($23) {
       $$0$lcssa49 = $$01162;$$02$lcssa51 = $$02964;$i$0$lcssa50 = $i$01063;
       label = 20;
       break L1;
      }
     }
     $24 = (($i$01063) + 1)|0;
     $25 = (($ar) + ($i$01063<<2)|0);
     HEAP32[$25>>2] = $18;
     $26 = (($27) + -1)|0;
     $28 = ($26|0)==(0);
     do {
      if ($28) {
       $49 = 32;
       label = 16;
      } else {
       $29 = $26 & 1;
       $30 = ($29|0)==(0);
       if ($30) {
        $$02$i$i = $26;$nTrailingZeros$03$i$i = 0;
        while(1) {
         $31 = (($nTrailingZeros$03$i$i) + 1)|0;
         $32 = $$02$i$i >>> 1;
         $33 = $32 & 1;
         $34 = ($33|0)==(0);
         if ($34) {
          $$02$i$i = $32;$nTrailingZeros$03$i$i = $31;
         } else {
          $$lcssa = $31;
          break;
         }
        }
        $35 = ($$lcssa|0)==(0);
        if ($35) {
         label = 11;
        } else {
         $47 = $$lcssa;
        }
       } else {
        label = 11;
       }
       if ((label|0) == 11) {
        label = 0;
        $36 = ($37|0)==(0);
        if ($36) {
         $49 = 64;
         label = 16;
         break;
        }
        $38 = $37 & 1;
        $39 = ($38|0)==(0);
        if ($39) {
         $$02$i3$i = $37;$nTrailingZeros$03$i2$i = 0;
        } else {
         $$0$i = 0;$51 = $27;$54 = $37;$58 = 0;
         break;
        }
        while(1) {
         $40 = (($nTrailingZeros$03$i2$i) + 1)|0;
         $41 = $$02$i3$i >>> 1;
         $42 = $41 & 1;
         $43 = ($42|0)==(0);
         if ($43) {
          $$02$i3$i = $41;$nTrailingZeros$03$i2$i = $40;
         } else {
          $$lcssa75 = $40;$nTrailingZeros$03$i2$i$lcssa = $nTrailingZeros$03$i2$i;
          break;
         }
        }
        $44 = (($nTrailingZeros$03$i2$i$lcssa) + 33)|0;
        $45 = ($$lcssa75|0)==(0);
        if ($45) {
         $$0$i = 0;$51 = $27;$54 = $37;$58 = 0;
         break;
        } else {
         $47 = $44;
        }
       }
       $46 = ($47>>>0)>(31);
       if ($46) {
        $49 = $47;
        label = 16;
       } else {
        $$0$i = $47;$51 = $27;$54 = $37;$58 = $47;
       }
      }
     } while(0);
     if ((label|0) == 16) {
      label = 0;
      $48 = (($49) + -32)|0;
      $$0$i = $48;$51 = $37;$54 = 0;$58 = $49;
     }
     $50 = $51 >>> $$0$i;
     $52 = (32 - ($$0$i))|0;
     $53 = $54 << $52;
     $55 = $53 | $50;
     $56 = $54 >>> $$0$i;
     $57 = (($58) + ($$02964))|0;
     $59 = ($55|0)!=(1);
     $60 = ($56|0)!=(0);
     $61 = $60 | $59;
     if (!($61)) {
      $$0$lcssa49 = $18;$$02$lcssa51 = $57;$i$0$lcssa50 = $24;
      label = 20;
      break L1;
     }
     $$pre = HEAP32[$ar>>2]|0;
     $62 = (($lp) + ($57<<2)|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = (0 - ($63))|0;
     $65 = (($18) + ($64)|0);
     $66 = (FUNCTION_TABLE_iii[$cmp & 255]($65,$$pre)|0);
     $67 = ($66|0)<(1);
     if ($67) {
      $$0$lcssa = $18;$$02$lcssa = $57;$$03$lcssa = 0;$i$0$lcssa = $24;
      label = 19;
      break;
     } else {
      $$01162$phi = $18;$$02964 = $57;$$03865 = 1;$18 = $65;$27 = $55;$37 = $56;$i$01063 = $24;$$01162 = $$01162$phi;
     }
    }
   }
  } else {
   $$0$lcssa = $head;$$02$lcssa = $pshift;$$03$lcssa = $trusty;$i$0$lcssa = 1;
   label = 19;
  }
 } while(0);
 if ((label|0) == 19) {
  $68 = ($$03$lcssa|0)==(0);
  if ($68) {
   $$0$lcssa49 = $$0$lcssa;$$02$lcssa51 = $$02$lcssa;$i$0$lcssa50 = $i$0$lcssa;
   label = 20;
  }
 }
 if ((label|0) == 20) {
  _cycle($width,$ar,$i$0$lcssa50);
  _sift($$0$lcssa49,$width,$cmp,$$02$lcssa51,$lp);
 }
 STACKTOP = sp;return;
}
function _dummy1($x) {
 $x = $x|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _dummy0() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _cycle($width,$ar,$n) {
 $width = $width|0;
 $ar = $ar|0;
 $n = $n|0;
 var $$02 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $exitcond = 0, $i$01 = 0;
 var $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $tmp = sp;
 $0 = ($n|0)<(2);
 L1: do {
  if (!($0)) {
   $1 = (($ar) + ($n<<2)|0);
   HEAP32[$1>>2] = $tmp;
   $2 = ($width|0)==(0);
   if (!($2)) {
    $$02 = $width;$6 = $tmp;
    while(1) {
     $3 = ($$02>>>0)>(256);
     $4 = $3 ? 256 : $$02;
     $5 = HEAP32[$ar>>2]|0;
     _memcpy(($6|0),($5|0),($4|0))|0;
     $i$01 = 0;
     while(1) {
      $7 = (($ar) + ($i$01<<2)|0);
      $8 = HEAP32[$7>>2]|0;
      $9 = (($i$01) + 1)|0;
      $10 = (($ar) + ($9<<2)|0);
      $11 = HEAP32[$10>>2]|0;
      _memcpy(($8|0),($11|0),($4|0))|0;
      $12 = HEAP32[$7>>2]|0;
      $13 = (($12) + ($4)|0);
      HEAP32[$7>>2] = $13;
      $exitcond = ($9|0)==($n|0);
      if ($exitcond) {
       break;
      } else {
       $i$01 = $9;
      }
     }
     $14 = ($$02|0)==($4|0);
     if ($14) {
      break L1;
     }
     $15 = (($$02) - ($4))|0;
     $$pre = HEAP32[$1>>2]|0;
     $$02 = $15;$6 = $$pre;
    }
   }
  }
 } while(0);
 STACKTOP = sp;return;
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$3$i = 0, $$lcssa = 0, $$lcssa211 = 0, $$lcssa215 = 0, $$lcssa216 = 0, $$lcssa217 = 0, $$lcssa219 = 0, $$lcssa222 = 0, $$lcssa224 = 0, $$lcssa226 = 0, $$lcssa228 = 0, $$lcssa230 = 0, $$lcssa232 = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i22$i = 0, $$pre$i25 = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i23$iZ2D = 0;
 var $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre105 = 0, $$pre106 = 0, $$pre14$i$i = 0, $$pre43$i = 0, $$pre56$i$i = 0, $$pre57$i$i = 0, $$pre8$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0, $$sum$i13$i = 0, $$sum$i14$i = 0, $$sum$i17$i = 0, $$sum$i19$i = 0;
 var $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i15$i = 0, $$sum1$i20$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum11$i = 0, $$sum11$i$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0;
 var $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum121$i = 0, $$sum122$i = 0, $$sum123$i = 0, $$sum124$i = 0, $$sum125$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0;
 var $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i16$i = 0, $$sum2$i18$i = 0, $$sum2$i21$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0, $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i27 = 0;
 var $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0, $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0;
 var $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0;
 var $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0;
 var $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0;
 var $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0;
 var $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0;
 var $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0;
 var $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0;
 var $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0;
 var $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0;
 var $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0;
 var $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0;
 var $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0;
 var $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0;
 var $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0;
 var $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0;
 var $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0;
 var $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0;
 var $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0;
 var $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0;
 var $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0;
 var $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0;
 var $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0;
 var $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0;
 var $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0;
 var $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0;
 var $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0;
 var $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0;
 var $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0;
 var $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0;
 var $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0;
 var $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0;
 var $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0;
 var $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0;
 var $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0;
 var $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0;
 var $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0;
 var $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0;
 var $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0;
 var $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0;
 var $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0;
 var $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0;
 var $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0;
 var $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0;
 var $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0;
 var $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0;
 var $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0;
 var $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0;
 var $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0;
 var $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0;
 var $F5$0$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$029$i = 0, $K2$07$i$i = 0, $K8$051$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i$i$lcssa = 0, $R$0$i$lcssa = 0, $R$0$i18 = 0, $R$0$i18$lcssa = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i$i$lcssa = 0, $RP$0$i$lcssa = 0;
 var $RP$0$i17 = 0, $RP$0$i17$lcssa = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i25$i = 0, $T$028$i = 0, $T$028$i$lcssa = 0, $T$050$i$i = 0, $T$050$i$i$lcssa = 0, $T$06$i$i = 0, $T$06$i$i$lcssa = 0, $br$0$ph$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0;
 var $not$$i = 0, $not$$i$i = 0, $not$$i26$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i30 = 0, $or$cond1$i = 0, $or$cond19$i = 0, $or$cond2$i = 0, $or$cond3$i = 0, $or$cond5$i = 0, $or$cond57$i = 0, $or$cond6$i = 0, $or$cond8$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i$lcssa = 0, $rsize$0$i15 = 0, $rsize$1$i = 0;
 var $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$331$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$084$i = 0, $sp$084$i$lcssa = 0, $sp$183$i = 0, $sp$183$i$lcssa = 0, $ssize$0$$i = 0, $ssize$0$i = 0, $ssize$1$ph$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0;
 var $t$2$v$3$i = 0, $t$230$i = 0, $tbase$255$i = 0, $tsize$0$ph$i = 0, $tsize$0323944$i = 0, $tsize$1$i = 0, $tsize$254$i = 0, $v$0$i = 0, $v$0$i$lcssa = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0, $v$3$ph$i = 0, $v$332$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   $2 = (($bytes) + 11)|0;
   $3 = $2 & -8;
   $4 = $1 ? 16 : $3;
   $5 = $4 >>> 3;
   $6 = HEAP32[9524>>2]|0;
   $7 = $6 >>> $5;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($5))|0;
    $13 = $12 << 1;
    $14 = (9564 + ($13<<2)|0);
    $$sum10 = (($13) + 2)|0;
    $15 = (9564 + ($$sum10<<2)|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[9524>>2] = $22;
     } else {
      $23 = HEAP32[(9540)>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = ((($18)) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = ((($16)) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    return ($mem$0|0);
   }
   $34 = HEAP32[(9532)>>2]|0;
   $35 = ($4>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $5;
     $38 = 2 << $5;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = (9564 + ($65<<2)|0);
     $$sum4 = (($65) + 2)|0;
     $67 = (9564 + ($$sum4<<2)|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = ((($68)) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[9524>>2] = $74;
       $89 = $34;
      } else {
       $75 = HEAP32[(9540)>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = ((($70)) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        $$pre = HEAP32[(9532)>>2]|0;
        $89 = $$pre;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($4))|0;
     $82 = $4 | 3;
     $83 = ((($68)) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($4)|0);
     $85 = $81 | 1;
     $$sum56 = $4 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $88 = ($89|0)==(0);
     if (!($88)) {
      $90 = HEAP32[(9544)>>2]|0;
      $91 = $89 >>> 3;
      $92 = $91 << 1;
      $93 = (9564 + ($92<<2)|0);
      $94 = HEAP32[9524>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[9524>>2] = $98;
       $$pre105 = (($92) + 2)|0;
       $$pre106 = (9564 + ($$pre105<<2)|0);
       $$pre$phiZ2D = $$pre106;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = (9564 + ($$sum9<<2)|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[(9540)>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = ((($F4$0)) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = ((($90)) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = ((($90)) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[(9532)>>2] = $81;
     HEAP32[(9544)>>2] = $84;
     $mem$0 = $69;
     return ($mem$0|0);
    }
    $106 = HEAP32[(9528)>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $4;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = (9828 + ($130<<2)|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = ((($132)) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($4))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = ((($t$0$i)) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = ((($t$0$i)) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        $rsize$0$i$lcssa = $rsize$0$i;$v$0$i$lcssa = $v$0$i;
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = ((($144)) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($4))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[(9540)>>2]|0;
     $150 = ($v$0$i$lcssa>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i$lcssa) + ($4)|0);
     $152 = ($v$0$i$lcssa>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = ((($v$0$i$lcssa)) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = ((($v$0$i$lcssa)) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i$lcssa|0);
     do {
      if ($157) {
       $167 = ((($v$0$i$lcssa)) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = ((($v$0$i$lcssa)) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = ((($R$0$i)) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = ((($R$0$i)) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         $R$0$i$lcssa = $R$0$i;$RP$0$i$lcssa = $RP$0$i;
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i$lcssa>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i$lcssa>>2] = 0;
        $R$1$i = $R$0$i$lcssa;
        break;
       }
      } else {
       $158 = ((($v$0$i$lcssa)) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = ((($159)) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i$lcssa|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = ((($156)) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i$lcssa|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = ((($v$0$i$lcssa)) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = (9828 + ($182<<2)|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i$lcssa|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[(9528)>>2]|0;
         $189 = $188 & $187;
         HEAP32[(9528)>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[(9540)>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = ((($154)) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i$lcssa|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = ((($154)) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[(9540)>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = ((($R$1$i)) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = ((($v$0$i$lcssa)) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = ($201>>>0)<($197>>>0);
         if ($203) {
          _abort();
          // unreachable;
         } else {
          $204 = ((($R$1$i)) + 16|0);
          HEAP32[$204>>2] = $201;
          $205 = ((($201)) + 24|0);
          HEAP32[$205>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $206 = ((($v$0$i$lcssa)) + 20|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = ($207|0)==(0|0);
       if (!($208)) {
        $209 = HEAP32[(9540)>>2]|0;
        $210 = ($207>>>0)<($209>>>0);
        if ($210) {
         _abort();
         // unreachable;
        } else {
         $211 = ((($R$1$i)) + 20|0);
         HEAP32[$211>>2] = $207;
         $212 = ((($207)) + 24|0);
         HEAP32[$212>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $213 = ($rsize$0$i$lcssa>>>0)<(16);
     if ($213) {
      $214 = (($rsize$0$i$lcssa) + ($4))|0;
      $215 = $214 | 3;
      $216 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$216>>2] = $215;
      $$sum4$i = (($214) + 4)|0;
      $217 = (($v$0$i$lcssa) + ($$sum4$i)|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $218 | 1;
      HEAP32[$217>>2] = $219;
     } else {
      $220 = $4 | 3;
      $221 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$221>>2] = $220;
      $222 = $rsize$0$i$lcssa | 1;
      $$sum$i35 = $4 | 4;
      $223 = (($v$0$i$lcssa) + ($$sum$i35)|0);
      HEAP32[$223>>2] = $222;
      $$sum1$i = (($rsize$0$i$lcssa) + ($4))|0;
      $224 = (($v$0$i$lcssa) + ($$sum1$i)|0);
      HEAP32[$224>>2] = $rsize$0$i$lcssa;
      $225 = HEAP32[(9532)>>2]|0;
      $226 = ($225|0)==(0);
      if (!($226)) {
       $227 = HEAP32[(9544)>>2]|0;
       $228 = $225 >>> 3;
       $229 = $228 << 1;
       $230 = (9564 + ($229<<2)|0);
       $231 = HEAP32[9524>>2]|0;
       $232 = 1 << $228;
       $233 = $231 & $232;
       $234 = ($233|0)==(0);
       if ($234) {
        $235 = $231 | $232;
        HEAP32[9524>>2] = $235;
        $$pre$i = (($229) + 2)|0;
        $$pre8$i = (9564 + ($$pre$i<<2)|0);
        $$pre$phi$iZ2D = $$pre8$i;$F1$0$i = $230;
       } else {
        $$sum3$i = (($229) + 2)|0;
        $236 = (9564 + ($$sum3$i<<2)|0);
        $237 = HEAP32[$236>>2]|0;
        $238 = HEAP32[(9540)>>2]|0;
        $239 = ($237>>>0)<($238>>>0);
        if ($239) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $236;$F1$0$i = $237;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $227;
       $240 = ((($F1$0$i)) + 12|0);
       HEAP32[$240>>2] = $227;
       $241 = ((($227)) + 8|0);
       HEAP32[$241>>2] = $F1$0$i;
       $242 = ((($227)) + 12|0);
       HEAP32[$242>>2] = $230;
      }
      HEAP32[(9532)>>2] = $rsize$0$i$lcssa;
      HEAP32[(9544)>>2] = $151;
     }
     $243 = ((($v$0$i$lcssa)) + 8|0);
     $mem$0 = $243;
     return ($mem$0|0);
    }
   } else {
    $nb$0 = $4;
   }
  } else {
   $244 = ($bytes>>>0)>(4294967231);
   if ($244) {
    $nb$0 = -1;
   } else {
    $245 = (($bytes) + 11)|0;
    $246 = $245 & -8;
    $247 = HEAP32[(9528)>>2]|0;
    $248 = ($247|0)==(0);
    if ($248) {
     $nb$0 = $246;
    } else {
     $249 = (0 - ($246))|0;
     $250 = $245 >>> 8;
     $251 = ($250|0)==(0);
     if ($251) {
      $idx$0$i = 0;
     } else {
      $252 = ($246>>>0)>(16777215);
      if ($252) {
       $idx$0$i = 31;
      } else {
       $253 = (($250) + 1048320)|0;
       $254 = $253 >>> 16;
       $255 = $254 & 8;
       $256 = $250 << $255;
       $257 = (($256) + 520192)|0;
       $258 = $257 >>> 16;
       $259 = $258 & 4;
       $260 = $259 | $255;
       $261 = $256 << $259;
       $262 = (($261) + 245760)|0;
       $263 = $262 >>> 16;
       $264 = $263 & 2;
       $265 = $260 | $264;
       $266 = (14 - ($265))|0;
       $267 = $261 << $264;
       $268 = $267 >>> 15;
       $269 = (($266) + ($268))|0;
       $270 = $269 << 1;
       $271 = (($269) + 7)|0;
       $272 = $246 >>> $271;
       $273 = $272 & 1;
       $274 = $273 | $270;
       $idx$0$i = $274;
      }
     }
     $275 = (9828 + ($idx$0$i<<2)|0);
     $276 = HEAP32[$275>>2]|0;
     $277 = ($276|0)==(0|0);
     L123: do {
      if ($277) {
       $rsize$2$i = $249;$t$1$i = 0;$v$2$i = 0;
       label = 86;
      } else {
       $278 = ($idx$0$i|0)==(31);
       $279 = $idx$0$i >>> 1;
       $280 = (25 - ($279))|0;
       $281 = $278 ? 0 : $280;
       $282 = $246 << $281;
       $rsize$0$i15 = $249;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $276;$v$0$i16 = 0;
       while(1) {
        $283 = ((($t$0$i14)) + 4|0);
        $284 = HEAP32[$283>>2]|0;
        $285 = $284 & -8;
        $286 = (($285) - ($246))|0;
        $287 = ($286>>>0)<($rsize$0$i15>>>0);
        if ($287) {
         $288 = ($285|0)==($246|0);
         if ($288) {
          $rsize$331$i = $286;$t$230$i = $t$0$i14;$v$332$i = $t$0$i14;
          label = 90;
          break L123;
         } else {
          $rsize$1$i = $286;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $289 = ((($t$0$i14)) + 20|0);
        $290 = HEAP32[$289>>2]|0;
        $291 = $sizebits$0$i >>> 31;
        $292 = (((($t$0$i14)) + 16|0) + ($291<<2)|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = ($290|0)==(0|0);
        $295 = ($290|0)==($293|0);
        $or$cond19$i = $294 | $295;
        $rst$1$i = $or$cond19$i ? $rst$0$i : $290;
        $296 = ($293|0)==(0|0);
        $297 = $sizebits$0$i << 1;
        if ($296) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         label = 86;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $297;$t$0$i14 = $293;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     if ((label|0) == 86) {
      $298 = ($t$1$i|0)==(0|0);
      $299 = ($v$2$i|0)==(0|0);
      $or$cond$i = $298 & $299;
      if ($or$cond$i) {
       $300 = 2 << $idx$0$i;
       $301 = (0 - ($300))|0;
       $302 = $300 | $301;
       $303 = $247 & $302;
       $304 = ($303|0)==(0);
       if ($304) {
        $nb$0 = $246;
        break;
       }
       $305 = (0 - ($303))|0;
       $306 = $303 & $305;
       $307 = (($306) + -1)|0;
       $308 = $307 >>> 12;
       $309 = $308 & 16;
       $310 = $307 >>> $309;
       $311 = $310 >>> 5;
       $312 = $311 & 8;
       $313 = $312 | $309;
       $314 = $310 >>> $312;
       $315 = $314 >>> 2;
       $316 = $315 & 4;
       $317 = $313 | $316;
       $318 = $314 >>> $316;
       $319 = $318 >>> 1;
       $320 = $319 & 2;
       $321 = $317 | $320;
       $322 = $318 >>> $320;
       $323 = $322 >>> 1;
       $324 = $323 & 1;
       $325 = $321 | $324;
       $326 = $322 >>> $324;
       $327 = (($325) + ($326))|0;
       $328 = (9828 + ($327<<2)|0);
       $329 = HEAP32[$328>>2]|0;
       $t$2$ph$i = $329;$v$3$ph$i = 0;
      } else {
       $t$2$ph$i = $t$1$i;$v$3$ph$i = $v$2$i;
      }
      $330 = ($t$2$ph$i|0)==(0|0);
      if ($330) {
       $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$3$ph$i;
      } else {
       $rsize$331$i = $rsize$2$i;$t$230$i = $t$2$ph$i;$v$332$i = $v$3$ph$i;
       label = 90;
      }
     }
     if ((label|0) == 90) {
      while(1) {
       label = 0;
       $331 = ((($t$230$i)) + 4|0);
       $332 = HEAP32[$331>>2]|0;
       $333 = $332 & -8;
       $334 = (($333) - ($246))|0;
       $335 = ($334>>>0)<($rsize$331$i>>>0);
       $$rsize$3$i = $335 ? $334 : $rsize$331$i;
       $t$2$v$3$i = $335 ? $t$230$i : $v$332$i;
       $336 = ((($t$230$i)) + 16|0);
       $337 = HEAP32[$336>>2]|0;
       $338 = ($337|0)==(0|0);
       if (!($338)) {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $337;$v$332$i = $t$2$v$3$i;
        label = 90;
        continue;
       }
       $339 = ((($t$230$i)) + 20|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = ($340|0)==(0|0);
       if ($341) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $340;$v$332$i = $t$2$v$3$i;
        label = 90;
       }
      }
     }
     $342 = ($v$3$lcssa$i|0)==(0|0);
     if ($342) {
      $nb$0 = $246;
     } else {
      $343 = HEAP32[(9532)>>2]|0;
      $344 = (($343) - ($246))|0;
      $345 = ($rsize$3$lcssa$i>>>0)<($344>>>0);
      if ($345) {
       $346 = HEAP32[(9540)>>2]|0;
       $347 = ($v$3$lcssa$i>>>0)<($346>>>0);
       if ($347) {
        _abort();
        // unreachable;
       }
       $348 = (($v$3$lcssa$i) + ($246)|0);
       $349 = ($v$3$lcssa$i>>>0)<($348>>>0);
       if (!($349)) {
        _abort();
        // unreachable;
       }
       $350 = ((($v$3$lcssa$i)) + 24|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = ((($v$3$lcssa$i)) + 12|0);
       $353 = HEAP32[$352>>2]|0;
       $354 = ($353|0)==($v$3$lcssa$i|0);
       do {
        if ($354) {
         $364 = ((($v$3$lcssa$i)) + 20|0);
         $365 = HEAP32[$364>>2]|0;
         $366 = ($365|0)==(0|0);
         if ($366) {
          $367 = ((($v$3$lcssa$i)) + 16|0);
          $368 = HEAP32[$367>>2]|0;
          $369 = ($368|0)==(0|0);
          if ($369) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $368;$RP$0$i17 = $367;
          }
         } else {
          $R$0$i18 = $365;$RP$0$i17 = $364;
         }
         while(1) {
          $370 = ((($R$0$i18)) + 20|0);
          $371 = HEAP32[$370>>2]|0;
          $372 = ($371|0)==(0|0);
          if (!($372)) {
           $R$0$i18 = $371;$RP$0$i17 = $370;
           continue;
          }
          $373 = ((($R$0$i18)) + 16|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = ($374|0)==(0|0);
          if ($375) {
           $R$0$i18$lcssa = $R$0$i18;$RP$0$i17$lcssa = $RP$0$i17;
           break;
          } else {
           $R$0$i18 = $374;$RP$0$i17 = $373;
          }
         }
         $376 = ($RP$0$i17$lcssa>>>0)<($346>>>0);
         if ($376) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17$lcssa>>2] = 0;
          $R$1$i20 = $R$0$i18$lcssa;
          break;
         }
        } else {
         $355 = ((($v$3$lcssa$i)) + 8|0);
         $356 = HEAP32[$355>>2]|0;
         $357 = ($356>>>0)<($346>>>0);
         if ($357) {
          _abort();
          // unreachable;
         }
         $358 = ((($356)) + 12|0);
         $359 = HEAP32[$358>>2]|0;
         $360 = ($359|0)==($v$3$lcssa$i|0);
         if (!($360)) {
          _abort();
          // unreachable;
         }
         $361 = ((($353)) + 8|0);
         $362 = HEAP32[$361>>2]|0;
         $363 = ($362|0)==($v$3$lcssa$i|0);
         if ($363) {
          HEAP32[$358>>2] = $353;
          HEAP32[$361>>2] = $356;
          $R$1$i20 = $353;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $377 = ($351|0)==(0|0);
       do {
        if (!($377)) {
         $378 = ((($v$3$lcssa$i)) + 28|0);
         $379 = HEAP32[$378>>2]|0;
         $380 = (9828 + ($379<<2)|0);
         $381 = HEAP32[$380>>2]|0;
         $382 = ($v$3$lcssa$i|0)==($381|0);
         if ($382) {
          HEAP32[$380>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $383 = 1 << $379;
           $384 = $383 ^ -1;
           $385 = HEAP32[(9528)>>2]|0;
           $386 = $385 & $384;
           HEAP32[(9528)>>2] = $386;
           break;
          }
         } else {
          $387 = HEAP32[(9540)>>2]|0;
          $388 = ($351>>>0)<($387>>>0);
          if ($388) {
           _abort();
           // unreachable;
          }
          $389 = ((($351)) + 16|0);
          $390 = HEAP32[$389>>2]|0;
          $391 = ($390|0)==($v$3$lcssa$i|0);
          if ($391) {
           HEAP32[$389>>2] = $R$1$i20;
          } else {
           $392 = ((($351)) + 20|0);
           HEAP32[$392>>2] = $R$1$i20;
          }
          $393 = ($R$1$i20|0)==(0|0);
          if ($393) {
           break;
          }
         }
         $394 = HEAP32[(9540)>>2]|0;
         $395 = ($R$1$i20>>>0)<($394>>>0);
         if ($395) {
          _abort();
          // unreachable;
         }
         $396 = ((($R$1$i20)) + 24|0);
         HEAP32[$396>>2] = $351;
         $397 = ((($v$3$lcssa$i)) + 16|0);
         $398 = HEAP32[$397>>2]|0;
         $399 = ($398|0)==(0|0);
         do {
          if (!($399)) {
           $400 = ($398>>>0)<($394>>>0);
           if ($400) {
            _abort();
            // unreachable;
           } else {
            $401 = ((($R$1$i20)) + 16|0);
            HEAP32[$401>>2] = $398;
            $402 = ((($398)) + 24|0);
            HEAP32[$402>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $403 = ((($v$3$lcssa$i)) + 20|0);
         $404 = HEAP32[$403>>2]|0;
         $405 = ($404|0)==(0|0);
         if (!($405)) {
          $406 = HEAP32[(9540)>>2]|0;
          $407 = ($404>>>0)<($406>>>0);
          if ($407) {
           _abort();
           // unreachable;
          } else {
           $408 = ((($R$1$i20)) + 20|0);
           HEAP32[$408>>2] = $404;
           $409 = ((($404)) + 24|0);
           HEAP32[$409>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $410 = ($rsize$3$lcssa$i>>>0)<(16);
       L199: do {
        if ($410) {
         $411 = (($rsize$3$lcssa$i) + ($246))|0;
         $412 = $411 | 3;
         $413 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$413>>2] = $412;
         $$sum18$i = (($411) + 4)|0;
         $414 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $415 = HEAP32[$414>>2]|0;
         $416 = $415 | 1;
         HEAP32[$414>>2] = $416;
        } else {
         $417 = $246 | 3;
         $418 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$418>>2] = $417;
         $419 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $246 | 4;
         $420 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$420>>2] = $419;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($246))|0;
         $421 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$421>>2] = $rsize$3$lcssa$i;
         $422 = $rsize$3$lcssa$i >>> 3;
         $423 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($423) {
          $424 = $422 << 1;
          $425 = (9564 + ($424<<2)|0);
          $426 = HEAP32[9524>>2]|0;
          $427 = 1 << $422;
          $428 = $426 & $427;
          $429 = ($428|0)==(0);
          if ($429) {
           $430 = $426 | $427;
           HEAP32[9524>>2] = $430;
           $$pre$i25 = (($424) + 2)|0;
           $$pre43$i = (9564 + ($$pre$i25<<2)|0);
           $$pre$phi$i26Z2D = $$pre43$i;$F5$0$i = $425;
          } else {
           $$sum17$i = (($424) + 2)|0;
           $431 = (9564 + ($$sum17$i<<2)|0);
           $432 = HEAP32[$431>>2]|0;
           $433 = HEAP32[(9540)>>2]|0;
           $434 = ($432>>>0)<($433>>>0);
           if ($434) {
            _abort();
            // unreachable;
           } else {
            $$pre$phi$i26Z2D = $431;$F5$0$i = $432;
           }
          }
          HEAP32[$$pre$phi$i26Z2D>>2] = $348;
          $435 = ((($F5$0$i)) + 12|0);
          HEAP32[$435>>2] = $348;
          $$sum15$i = (($246) + 8)|0;
          $436 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$436>>2] = $F5$0$i;
          $$sum16$i = (($246) + 12)|0;
          $437 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$437>>2] = $425;
          break;
         }
         $438 = $rsize$3$lcssa$i >>> 8;
         $439 = ($438|0)==(0);
         if ($439) {
          $I7$0$i = 0;
         } else {
          $440 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($440) {
           $I7$0$i = 31;
          } else {
           $441 = (($438) + 1048320)|0;
           $442 = $441 >>> 16;
           $443 = $442 & 8;
           $444 = $438 << $443;
           $445 = (($444) + 520192)|0;
           $446 = $445 >>> 16;
           $447 = $446 & 4;
           $448 = $447 | $443;
           $449 = $444 << $447;
           $450 = (($449) + 245760)|0;
           $451 = $450 >>> 16;
           $452 = $451 & 2;
           $453 = $448 | $452;
           $454 = (14 - ($453))|0;
           $455 = $449 << $452;
           $456 = $455 >>> 15;
           $457 = (($454) + ($456))|0;
           $458 = $457 << 1;
           $459 = (($457) + 7)|0;
           $460 = $rsize$3$lcssa$i >>> $459;
           $461 = $460 & 1;
           $462 = $461 | $458;
           $I7$0$i = $462;
          }
         }
         $463 = (9828 + ($I7$0$i<<2)|0);
         $$sum2$i = (($246) + 28)|0;
         $464 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$464>>2] = $I7$0$i;
         $$sum3$i27 = (($246) + 16)|0;
         $465 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($246) + 20)|0;
         $466 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$466>>2] = 0;
         HEAP32[$465>>2] = 0;
         $467 = HEAP32[(9528)>>2]|0;
         $468 = 1 << $I7$0$i;
         $469 = $467 & $468;
         $470 = ($469|0)==(0);
         if ($470) {
          $471 = $467 | $468;
          HEAP32[(9528)>>2] = $471;
          HEAP32[$463>>2] = $348;
          $$sum5$i = (($246) + 24)|0;
          $472 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$472>>2] = $463;
          $$sum6$i = (($246) + 12)|0;
          $473 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$473>>2] = $348;
          $$sum7$i = (($246) + 8)|0;
          $474 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$474>>2] = $348;
          break;
         }
         $475 = HEAP32[$463>>2]|0;
         $476 = ((($475)) + 4|0);
         $477 = HEAP32[$476>>2]|0;
         $478 = $477 & -8;
         $479 = ($478|0)==($rsize$3$lcssa$i|0);
         L217: do {
          if ($479) {
           $T$0$lcssa$i = $475;
          } else {
           $480 = ($I7$0$i|0)==(31);
           $481 = $I7$0$i >>> 1;
           $482 = (25 - ($481))|0;
           $483 = $480 ? 0 : $482;
           $484 = $rsize$3$lcssa$i << $483;
           $K12$029$i = $484;$T$028$i = $475;
           while(1) {
            $491 = $K12$029$i >>> 31;
            $492 = (((($T$028$i)) + 16|0) + ($491<<2)|0);
            $487 = HEAP32[$492>>2]|0;
            $493 = ($487|0)==(0|0);
            if ($493) {
             $$lcssa232 = $492;$T$028$i$lcssa = $T$028$i;
             break;
            }
            $485 = $K12$029$i << 1;
            $486 = ((($487)) + 4|0);
            $488 = HEAP32[$486>>2]|0;
            $489 = $488 & -8;
            $490 = ($489|0)==($rsize$3$lcssa$i|0);
            if ($490) {
             $T$0$lcssa$i = $487;
             break L217;
            } else {
             $K12$029$i = $485;$T$028$i = $487;
            }
           }
           $494 = HEAP32[(9540)>>2]|0;
           $495 = ($$lcssa232>>>0)<($494>>>0);
           if ($495) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$$lcssa232>>2] = $348;
            $$sum11$i = (($246) + 24)|0;
            $496 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$496>>2] = $T$028$i$lcssa;
            $$sum12$i = (($246) + 12)|0;
            $497 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$497>>2] = $348;
            $$sum13$i = (($246) + 8)|0;
            $498 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$498>>2] = $348;
            break L199;
           }
          }
         } while(0);
         $499 = ((($T$0$lcssa$i)) + 8|0);
         $500 = HEAP32[$499>>2]|0;
         $501 = HEAP32[(9540)>>2]|0;
         $502 = ($500>>>0)>=($501>>>0);
         $not$$i = ($T$0$lcssa$i>>>0)>=($501>>>0);
         $503 = $502 & $not$$i;
         if ($503) {
          $504 = ((($500)) + 12|0);
          HEAP32[$504>>2] = $348;
          HEAP32[$499>>2] = $348;
          $$sum8$i = (($246) + 8)|0;
          $505 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$505>>2] = $500;
          $$sum9$i = (($246) + 12)|0;
          $506 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$506>>2] = $T$0$lcssa$i;
          $$sum10$i = (($246) + 24)|0;
          $507 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$507>>2] = 0;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $508 = ((($v$3$lcssa$i)) + 8|0);
       $mem$0 = $508;
       return ($mem$0|0);
      } else {
       $nb$0 = $246;
      }
     }
    }
   }
  }
 } while(0);
 $509 = HEAP32[(9532)>>2]|0;
 $510 = ($509>>>0)<($nb$0>>>0);
 if (!($510)) {
  $511 = (($509) - ($nb$0))|0;
  $512 = HEAP32[(9544)>>2]|0;
  $513 = ($511>>>0)>(15);
  if ($513) {
   $514 = (($512) + ($nb$0)|0);
   HEAP32[(9544)>>2] = $514;
   HEAP32[(9532)>>2] = $511;
   $515 = $511 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $516 = (($512) + ($$sum2)|0);
   HEAP32[$516>>2] = $515;
   $517 = (($512) + ($509)|0);
   HEAP32[$517>>2] = $511;
   $518 = $nb$0 | 3;
   $519 = ((($512)) + 4|0);
   HEAP32[$519>>2] = $518;
  } else {
   HEAP32[(9532)>>2] = 0;
   HEAP32[(9544)>>2] = 0;
   $520 = $509 | 3;
   $521 = ((($512)) + 4|0);
   HEAP32[$521>>2] = $520;
   $$sum1 = (($509) + 4)|0;
   $522 = (($512) + ($$sum1)|0);
   $523 = HEAP32[$522>>2]|0;
   $524 = $523 | 1;
   HEAP32[$522>>2] = $524;
  }
  $525 = ((($512)) + 8|0);
  $mem$0 = $525;
  return ($mem$0|0);
 }
 $526 = HEAP32[(9536)>>2]|0;
 $527 = ($526>>>0)>($nb$0>>>0);
 if ($527) {
  $528 = (($526) - ($nb$0))|0;
  HEAP32[(9536)>>2] = $528;
  $529 = HEAP32[(9548)>>2]|0;
  $530 = (($529) + ($nb$0)|0);
  HEAP32[(9548)>>2] = $530;
  $531 = $528 | 1;
  $$sum = (($nb$0) + 4)|0;
  $532 = (($529) + ($$sum)|0);
  HEAP32[$532>>2] = $531;
  $533 = $nb$0 | 3;
  $534 = ((($529)) + 4|0);
  HEAP32[$534>>2] = $533;
  $535 = ((($529)) + 8|0);
  $mem$0 = $535;
  return ($mem$0|0);
 }
 $536 = HEAP32[9996>>2]|0;
 $537 = ($536|0)==(0);
 do {
  if ($537) {
   $538 = (_sysconf(30)|0);
   $539 = (($538) + -1)|0;
   $540 = $539 & $538;
   $541 = ($540|0)==(0);
   if ($541) {
    HEAP32[(10004)>>2] = $538;
    HEAP32[(10000)>>2] = $538;
    HEAP32[(10008)>>2] = -1;
    HEAP32[(10012)>>2] = -1;
    HEAP32[(10016)>>2] = 0;
    HEAP32[(9968)>>2] = 0;
    $542 = (_time((0|0))|0);
    $543 = $542 & -16;
    $544 = $543 ^ 1431655768;
    HEAP32[9996>>2] = $544;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $545 = (($nb$0) + 48)|0;
 $546 = HEAP32[(10004)>>2]|0;
 $547 = (($nb$0) + 47)|0;
 $548 = (($546) + ($547))|0;
 $549 = (0 - ($546))|0;
 $550 = $548 & $549;
 $551 = ($550>>>0)>($nb$0>>>0);
 if (!($551)) {
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $552 = HEAP32[(9964)>>2]|0;
 $553 = ($552|0)==(0);
 if (!($553)) {
  $554 = HEAP32[(9956)>>2]|0;
  $555 = (($554) + ($550))|0;
  $556 = ($555>>>0)<=($554>>>0);
  $557 = ($555>>>0)>($552>>>0);
  $or$cond1$i = $556 | $557;
  if ($or$cond1$i) {
   $mem$0 = 0;
   return ($mem$0|0);
  }
 }
 $558 = HEAP32[(9968)>>2]|0;
 $559 = $558 & 4;
 $560 = ($559|0)==(0);
 L258: do {
  if ($560) {
   $561 = HEAP32[(9548)>>2]|0;
   $562 = ($561|0)==(0|0);
   L260: do {
    if ($562) {
     label = 174;
    } else {
     $sp$0$i$i = (9972);
     while(1) {
      $563 = HEAP32[$sp$0$i$i>>2]|0;
      $564 = ($563>>>0)>($561>>>0);
      if (!($564)) {
       $565 = ((($sp$0$i$i)) + 4|0);
       $566 = HEAP32[$565>>2]|0;
       $567 = (($563) + ($566)|0);
       $568 = ($567>>>0)>($561>>>0);
       if ($568) {
        $$lcssa228 = $sp$0$i$i;$$lcssa230 = $565;
        break;
       }
      }
      $569 = ((($sp$0$i$i)) + 8|0);
      $570 = HEAP32[$569>>2]|0;
      $571 = ($570|0)==(0|0);
      if ($571) {
       label = 174;
       break L260;
      } else {
       $sp$0$i$i = $570;
      }
     }
     $594 = HEAP32[(9536)>>2]|0;
     $595 = (($548) - ($594))|0;
     $596 = $595 & $549;
     $597 = ($596>>>0)<(2147483647);
     if ($597) {
      $598 = (_sbrk(($596|0))|0);
      $599 = HEAP32[$$lcssa228>>2]|0;
      $600 = HEAP32[$$lcssa230>>2]|0;
      $601 = (($599) + ($600)|0);
      $602 = ($598|0)==($601|0);
      $$3$i = $602 ? $596 : 0;
      if ($602) {
       $603 = ($598|0)==((-1)|0);
       if ($603) {
        $tsize$0323944$i = $$3$i;
       } else {
        $tbase$255$i = $598;$tsize$254$i = $$3$i;
        label = 194;
        break L258;
       }
      } else {
       $br$0$ph$i = $598;$ssize$1$ph$i = $596;$tsize$0$ph$i = $$3$i;
       label = 184;
      }
     } else {
      $tsize$0323944$i = 0;
     }
    }
   } while(0);
   do {
    if ((label|0) == 174) {
     $572 = (_sbrk(0)|0);
     $573 = ($572|0)==((-1)|0);
     if ($573) {
      $tsize$0323944$i = 0;
     } else {
      $574 = $572;
      $575 = HEAP32[(10000)>>2]|0;
      $576 = (($575) + -1)|0;
      $577 = $576 & $574;
      $578 = ($577|0)==(0);
      if ($578) {
       $ssize$0$i = $550;
      } else {
       $579 = (($576) + ($574))|0;
       $580 = (0 - ($575))|0;
       $581 = $579 & $580;
       $582 = (($550) - ($574))|0;
       $583 = (($582) + ($581))|0;
       $ssize$0$i = $583;
      }
      $584 = HEAP32[(9956)>>2]|0;
      $585 = (($584) + ($ssize$0$i))|0;
      $586 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $587 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i30 = $586 & $587;
      if ($or$cond$i30) {
       $588 = HEAP32[(9964)>>2]|0;
       $589 = ($588|0)==(0);
       if (!($589)) {
        $590 = ($585>>>0)<=($584>>>0);
        $591 = ($585>>>0)>($588>>>0);
        $or$cond2$i = $590 | $591;
        if ($or$cond2$i) {
         $tsize$0323944$i = 0;
         break;
        }
       }
       $592 = (_sbrk(($ssize$0$i|0))|0);
       $593 = ($592|0)==($572|0);
       $ssize$0$$i = $593 ? $ssize$0$i : 0;
       if ($593) {
        $tbase$255$i = $572;$tsize$254$i = $ssize$0$$i;
        label = 194;
        break L258;
       } else {
        $br$0$ph$i = $592;$ssize$1$ph$i = $ssize$0$i;$tsize$0$ph$i = $ssize$0$$i;
        label = 184;
       }
      } else {
       $tsize$0323944$i = 0;
      }
     }
    }
   } while(0);
   L280: do {
    if ((label|0) == 184) {
     $604 = (0 - ($ssize$1$ph$i))|0;
     $605 = ($br$0$ph$i|0)!=((-1)|0);
     $606 = ($ssize$1$ph$i>>>0)<(2147483647);
     $or$cond5$i = $606 & $605;
     $607 = ($545>>>0)>($ssize$1$ph$i>>>0);
     $or$cond6$i = $607 & $or$cond5$i;
     do {
      if ($or$cond6$i) {
       $608 = HEAP32[(10004)>>2]|0;
       $609 = (($547) - ($ssize$1$ph$i))|0;
       $610 = (($609) + ($608))|0;
       $611 = (0 - ($608))|0;
       $612 = $610 & $611;
       $613 = ($612>>>0)<(2147483647);
       if ($613) {
        $614 = (_sbrk(($612|0))|0);
        $615 = ($614|0)==((-1)|0);
        if ($615) {
         (_sbrk(($604|0))|0);
         $tsize$0323944$i = $tsize$0$ph$i;
         break L280;
        } else {
         $616 = (($612) + ($ssize$1$ph$i))|0;
         $ssize$2$i = $616;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$ph$i;
       }
      } else {
       $ssize$2$i = $ssize$1$ph$i;
      }
     } while(0);
     $617 = ($br$0$ph$i|0)==((-1)|0);
     if ($617) {
      $tsize$0323944$i = $tsize$0$ph$i;
     } else {
      $tbase$255$i = $br$0$ph$i;$tsize$254$i = $ssize$2$i;
      label = 194;
      break L258;
     }
    }
   } while(0);
   $618 = HEAP32[(9968)>>2]|0;
   $619 = $618 | 4;
   HEAP32[(9968)>>2] = $619;
   $tsize$1$i = $tsize$0323944$i;
   label = 191;
  } else {
   $tsize$1$i = 0;
   label = 191;
  }
 } while(0);
 if ((label|0) == 191) {
  $620 = ($550>>>0)<(2147483647);
  if ($620) {
   $621 = (_sbrk(($550|0))|0);
   $622 = (_sbrk(0)|0);
   $623 = ($621|0)!=((-1)|0);
   $624 = ($622|0)!=((-1)|0);
   $or$cond3$i = $623 & $624;
   $625 = ($621>>>0)<($622>>>0);
   $or$cond8$i = $625 & $or$cond3$i;
   if ($or$cond8$i) {
    $626 = $622;
    $627 = $621;
    $628 = (($626) - ($627))|0;
    $629 = (($nb$0) + 40)|0;
    $630 = ($628>>>0)>($629>>>0);
    $$tsize$1$i = $630 ? $628 : $tsize$1$i;
    if ($630) {
     $tbase$255$i = $621;$tsize$254$i = $$tsize$1$i;
     label = 194;
    }
   }
  }
 }
 if ((label|0) == 194) {
  $631 = HEAP32[(9956)>>2]|0;
  $632 = (($631) + ($tsize$254$i))|0;
  HEAP32[(9956)>>2] = $632;
  $633 = HEAP32[(9960)>>2]|0;
  $634 = ($632>>>0)>($633>>>0);
  if ($634) {
   HEAP32[(9960)>>2] = $632;
  }
  $635 = HEAP32[(9548)>>2]|0;
  $636 = ($635|0)==(0|0);
  L299: do {
   if ($636) {
    $637 = HEAP32[(9540)>>2]|0;
    $638 = ($637|0)==(0|0);
    $639 = ($tbase$255$i>>>0)<($637>>>0);
    $or$cond9$i = $638 | $639;
    if ($or$cond9$i) {
     HEAP32[(9540)>>2] = $tbase$255$i;
    }
    HEAP32[(9972)>>2] = $tbase$255$i;
    HEAP32[(9976)>>2] = $tsize$254$i;
    HEAP32[(9984)>>2] = 0;
    $640 = HEAP32[9996>>2]|0;
    HEAP32[(9560)>>2] = $640;
    HEAP32[(9556)>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $641 = $i$02$i$i << 1;
     $642 = (9564 + ($641<<2)|0);
     $$sum$i$i = (($641) + 3)|0;
     $643 = (9564 + ($$sum$i$i<<2)|0);
     HEAP32[$643>>2] = $642;
     $$sum1$i$i = (($641) + 2)|0;
     $644 = (9564 + ($$sum1$i$i<<2)|0);
     HEAP32[$644>>2] = $642;
     $645 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($645|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $645;
     }
    }
    $646 = (($tsize$254$i) + -40)|0;
    $647 = ((($tbase$255$i)) + 8|0);
    $648 = $647;
    $649 = $648 & 7;
    $650 = ($649|0)==(0);
    $651 = (0 - ($648))|0;
    $652 = $651 & 7;
    $653 = $650 ? 0 : $652;
    $654 = (($tbase$255$i) + ($653)|0);
    $655 = (($646) - ($653))|0;
    HEAP32[(9548)>>2] = $654;
    HEAP32[(9536)>>2] = $655;
    $656 = $655 | 1;
    $$sum$i13$i = (($653) + 4)|0;
    $657 = (($tbase$255$i) + ($$sum$i13$i)|0);
    HEAP32[$657>>2] = $656;
    $$sum2$i$i = (($tsize$254$i) + -36)|0;
    $658 = (($tbase$255$i) + ($$sum2$i$i)|0);
    HEAP32[$658>>2] = 40;
    $659 = HEAP32[(10012)>>2]|0;
    HEAP32[(9552)>>2] = $659;
   } else {
    $sp$084$i = (9972);
    while(1) {
     $660 = HEAP32[$sp$084$i>>2]|0;
     $661 = ((($sp$084$i)) + 4|0);
     $662 = HEAP32[$661>>2]|0;
     $663 = (($660) + ($662)|0);
     $664 = ($tbase$255$i|0)==($663|0);
     if ($664) {
      $$lcssa222 = $660;$$lcssa224 = $661;$$lcssa226 = $662;$sp$084$i$lcssa = $sp$084$i;
      label = 204;
      break;
     }
     $665 = ((($sp$084$i)) + 8|0);
     $666 = HEAP32[$665>>2]|0;
     $667 = ($666|0)==(0|0);
     if ($667) {
      break;
     } else {
      $sp$084$i = $666;
     }
    }
    if ((label|0) == 204) {
     $668 = ((($sp$084$i$lcssa)) + 12|0);
     $669 = HEAP32[$668>>2]|0;
     $670 = $669 & 8;
     $671 = ($670|0)==(0);
     if ($671) {
      $672 = ($635>>>0)>=($$lcssa222>>>0);
      $673 = ($635>>>0)<($tbase$255$i>>>0);
      $or$cond57$i = $673 & $672;
      if ($or$cond57$i) {
       $674 = (($$lcssa226) + ($tsize$254$i))|0;
       HEAP32[$$lcssa224>>2] = $674;
       $675 = HEAP32[(9536)>>2]|0;
       $676 = (($675) + ($tsize$254$i))|0;
       $677 = ((($635)) + 8|0);
       $678 = $677;
       $679 = $678 & 7;
       $680 = ($679|0)==(0);
       $681 = (0 - ($678))|0;
       $682 = $681 & 7;
       $683 = $680 ? 0 : $682;
       $684 = (($635) + ($683)|0);
       $685 = (($676) - ($683))|0;
       HEAP32[(9548)>>2] = $684;
       HEAP32[(9536)>>2] = $685;
       $686 = $685 | 1;
       $$sum$i17$i = (($683) + 4)|0;
       $687 = (($635) + ($$sum$i17$i)|0);
       HEAP32[$687>>2] = $686;
       $$sum2$i18$i = (($676) + 4)|0;
       $688 = (($635) + ($$sum2$i18$i)|0);
       HEAP32[$688>>2] = 40;
       $689 = HEAP32[(10012)>>2]|0;
       HEAP32[(9552)>>2] = $689;
       break;
      }
     }
    }
    $690 = HEAP32[(9540)>>2]|0;
    $691 = ($tbase$255$i>>>0)<($690>>>0);
    if ($691) {
     HEAP32[(9540)>>2] = $tbase$255$i;
     $755 = $tbase$255$i;
    } else {
     $755 = $690;
    }
    $692 = (($tbase$255$i) + ($tsize$254$i)|0);
    $sp$183$i = (9972);
    while(1) {
     $693 = HEAP32[$sp$183$i>>2]|0;
     $694 = ($693|0)==($692|0);
     if ($694) {
      $$lcssa219 = $sp$183$i;$sp$183$i$lcssa = $sp$183$i;
      label = 212;
      break;
     }
     $695 = ((($sp$183$i)) + 8|0);
     $696 = HEAP32[$695>>2]|0;
     $697 = ($696|0)==(0|0);
     if ($697) {
      $sp$0$i$i$i = (9972);
      break;
     } else {
      $sp$183$i = $696;
     }
    }
    if ((label|0) == 212) {
     $698 = ((($sp$183$i$lcssa)) + 12|0);
     $699 = HEAP32[$698>>2]|0;
     $700 = $699 & 8;
     $701 = ($700|0)==(0);
     if ($701) {
      HEAP32[$$lcssa219>>2] = $tbase$255$i;
      $702 = ((($sp$183$i$lcssa)) + 4|0);
      $703 = HEAP32[$702>>2]|0;
      $704 = (($703) + ($tsize$254$i))|0;
      HEAP32[$702>>2] = $704;
      $705 = ((($tbase$255$i)) + 8|0);
      $706 = $705;
      $707 = $706 & 7;
      $708 = ($707|0)==(0);
      $709 = (0 - ($706))|0;
      $710 = $709 & 7;
      $711 = $708 ? 0 : $710;
      $712 = (($tbase$255$i) + ($711)|0);
      $$sum112$i = (($tsize$254$i) + 8)|0;
      $713 = (($tbase$255$i) + ($$sum112$i)|0);
      $714 = $713;
      $715 = $714 & 7;
      $716 = ($715|0)==(0);
      $717 = (0 - ($714))|0;
      $718 = $717 & 7;
      $719 = $716 ? 0 : $718;
      $$sum113$i = (($719) + ($tsize$254$i))|0;
      $720 = (($tbase$255$i) + ($$sum113$i)|0);
      $721 = $720;
      $722 = $712;
      $723 = (($721) - ($722))|0;
      $$sum$i19$i = (($711) + ($nb$0))|0;
      $724 = (($tbase$255$i) + ($$sum$i19$i)|0);
      $725 = (($723) - ($nb$0))|0;
      $726 = $nb$0 | 3;
      $$sum1$i20$i = (($711) + 4)|0;
      $727 = (($tbase$255$i) + ($$sum1$i20$i)|0);
      HEAP32[$727>>2] = $726;
      $728 = ($720|0)==($635|0);
      L324: do {
       if ($728) {
        $729 = HEAP32[(9536)>>2]|0;
        $730 = (($729) + ($725))|0;
        HEAP32[(9536)>>2] = $730;
        HEAP32[(9548)>>2] = $724;
        $731 = $730 | 1;
        $$sum42$i$i = (($$sum$i19$i) + 4)|0;
        $732 = (($tbase$255$i) + ($$sum42$i$i)|0);
        HEAP32[$732>>2] = $731;
       } else {
        $733 = HEAP32[(9544)>>2]|0;
        $734 = ($720|0)==($733|0);
        if ($734) {
         $735 = HEAP32[(9532)>>2]|0;
         $736 = (($735) + ($725))|0;
         HEAP32[(9532)>>2] = $736;
         HEAP32[(9544)>>2] = $724;
         $737 = $736 | 1;
         $$sum40$i$i = (($$sum$i19$i) + 4)|0;
         $738 = (($tbase$255$i) + ($$sum40$i$i)|0);
         HEAP32[$738>>2] = $737;
         $$sum41$i$i = (($736) + ($$sum$i19$i))|0;
         $739 = (($tbase$255$i) + ($$sum41$i$i)|0);
         HEAP32[$739>>2] = $736;
         break;
        }
        $$sum2$i21$i = (($tsize$254$i) + 4)|0;
        $$sum114$i = (($$sum2$i21$i) + ($719))|0;
        $740 = (($tbase$255$i) + ($$sum114$i)|0);
        $741 = HEAP32[$740>>2]|0;
        $742 = $741 & 3;
        $743 = ($742|0)==(1);
        if ($743) {
         $744 = $741 & -8;
         $745 = $741 >>> 3;
         $746 = ($741>>>0)<(256);
         L332: do {
          if ($746) {
           $$sum3738$i$i = $719 | 8;
           $$sum124$i = (($$sum3738$i$i) + ($tsize$254$i))|0;
           $747 = (($tbase$255$i) + ($$sum124$i)|0);
           $748 = HEAP32[$747>>2]|0;
           $$sum39$i$i = (($tsize$254$i) + 12)|0;
           $$sum125$i = (($$sum39$i$i) + ($719))|0;
           $749 = (($tbase$255$i) + ($$sum125$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $751 = $745 << 1;
           $752 = (9564 + ($751<<2)|0);
           $753 = ($748|0)==($752|0);
           do {
            if (!($753)) {
             $754 = ($748>>>0)<($755>>>0);
             if ($754) {
              _abort();
              // unreachable;
             }
             $756 = ((($748)) + 12|0);
             $757 = HEAP32[$756>>2]|0;
             $758 = ($757|0)==($720|0);
             if ($758) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $759 = ($750|0)==($748|0);
           if ($759) {
            $760 = 1 << $745;
            $761 = $760 ^ -1;
            $762 = HEAP32[9524>>2]|0;
            $763 = $762 & $761;
            HEAP32[9524>>2] = $763;
            break;
           }
           $764 = ($750|0)==($752|0);
           do {
            if ($764) {
             $$pre57$i$i = ((($750)) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $765 = ($750>>>0)<($755>>>0);
             if ($765) {
              _abort();
              // unreachable;
             }
             $766 = ((($750)) + 8|0);
             $767 = HEAP32[$766>>2]|0;
             $768 = ($767|0)==($720|0);
             if ($768) {
              $$pre$phi58$i$iZ2D = $766;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $769 = ((($748)) + 12|0);
           HEAP32[$769>>2] = $750;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $748;
          } else {
           $$sum34$i$i = $719 | 24;
           $$sum115$i = (($$sum34$i$i) + ($tsize$254$i))|0;
           $770 = (($tbase$255$i) + ($$sum115$i)|0);
           $771 = HEAP32[$770>>2]|0;
           $$sum5$i$i = (($tsize$254$i) + 12)|0;
           $$sum116$i = (($$sum5$i$i) + ($719))|0;
           $772 = (($tbase$255$i) + ($$sum116$i)|0);
           $773 = HEAP32[$772>>2]|0;
           $774 = ($773|0)==($720|0);
           do {
            if ($774) {
             $$sum67$i$i = $719 | 16;
             $$sum122$i = (($$sum2$i21$i) + ($$sum67$i$i))|0;
             $784 = (($tbase$255$i) + ($$sum122$i)|0);
             $785 = HEAP32[$784>>2]|0;
             $786 = ($785|0)==(0|0);
             if ($786) {
              $$sum123$i = (($$sum67$i$i) + ($tsize$254$i))|0;
              $787 = (($tbase$255$i) + ($$sum123$i)|0);
              $788 = HEAP32[$787>>2]|0;
              $789 = ($788|0)==(0|0);
              if ($789) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $788;$RP$0$i$i = $787;
              }
             } else {
              $R$0$i$i = $785;$RP$0$i$i = $784;
             }
             while(1) {
              $790 = ((($R$0$i$i)) + 20|0);
              $791 = HEAP32[$790>>2]|0;
              $792 = ($791|0)==(0|0);
              if (!($792)) {
               $R$0$i$i = $791;$RP$0$i$i = $790;
               continue;
              }
              $793 = ((($R$0$i$i)) + 16|0);
              $794 = HEAP32[$793>>2]|0;
              $795 = ($794|0)==(0|0);
              if ($795) {
               $R$0$i$i$lcssa = $R$0$i$i;$RP$0$i$i$lcssa = $RP$0$i$i;
               break;
              } else {
               $R$0$i$i = $794;$RP$0$i$i = $793;
              }
             }
             $796 = ($RP$0$i$i$lcssa>>>0)<($755>>>0);
             if ($796) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i$lcssa>>2] = 0;
              $R$1$i$i = $R$0$i$i$lcssa;
              break;
             }
            } else {
             $$sum3536$i$i = $719 | 8;
             $$sum117$i = (($$sum3536$i$i) + ($tsize$254$i))|0;
             $775 = (($tbase$255$i) + ($$sum117$i)|0);
             $776 = HEAP32[$775>>2]|0;
             $777 = ($776>>>0)<($755>>>0);
             if ($777) {
              _abort();
              // unreachable;
             }
             $778 = ((($776)) + 12|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = ($779|0)==($720|0);
             if (!($780)) {
              _abort();
              // unreachable;
             }
             $781 = ((($773)) + 8|0);
             $782 = HEAP32[$781>>2]|0;
             $783 = ($782|0)==($720|0);
             if ($783) {
              HEAP32[$778>>2] = $773;
              HEAP32[$781>>2] = $776;
              $R$1$i$i = $773;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $797 = ($771|0)==(0|0);
           if ($797) {
            break;
           }
           $$sum30$i$i = (($tsize$254$i) + 28)|0;
           $$sum118$i = (($$sum30$i$i) + ($719))|0;
           $798 = (($tbase$255$i) + ($$sum118$i)|0);
           $799 = HEAP32[$798>>2]|0;
           $800 = (9828 + ($799<<2)|0);
           $801 = HEAP32[$800>>2]|0;
           $802 = ($720|0)==($801|0);
           do {
            if ($802) {
             HEAP32[$800>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $803 = 1 << $799;
             $804 = $803 ^ -1;
             $805 = HEAP32[(9528)>>2]|0;
             $806 = $805 & $804;
             HEAP32[(9528)>>2] = $806;
             break L332;
            } else {
             $807 = HEAP32[(9540)>>2]|0;
             $808 = ($771>>>0)<($807>>>0);
             if ($808) {
              _abort();
              // unreachable;
             }
             $809 = ((($771)) + 16|0);
             $810 = HEAP32[$809>>2]|0;
             $811 = ($810|0)==($720|0);
             if ($811) {
              HEAP32[$809>>2] = $R$1$i$i;
             } else {
              $812 = ((($771)) + 20|0);
              HEAP32[$812>>2] = $R$1$i$i;
             }
             $813 = ($R$1$i$i|0)==(0|0);
             if ($813) {
              break L332;
             }
            }
           } while(0);
           $814 = HEAP32[(9540)>>2]|0;
           $815 = ($R$1$i$i>>>0)<($814>>>0);
           if ($815) {
            _abort();
            // unreachable;
           }
           $816 = ((($R$1$i$i)) + 24|0);
           HEAP32[$816>>2] = $771;
           $$sum3132$i$i = $719 | 16;
           $$sum119$i = (($$sum3132$i$i) + ($tsize$254$i))|0;
           $817 = (($tbase$255$i) + ($$sum119$i)|0);
           $818 = HEAP32[$817>>2]|0;
           $819 = ($818|0)==(0|0);
           do {
            if (!($819)) {
             $820 = ($818>>>0)<($814>>>0);
             if ($820) {
              _abort();
              // unreachable;
             } else {
              $821 = ((($R$1$i$i)) + 16|0);
              HEAP32[$821>>2] = $818;
              $822 = ((($818)) + 24|0);
              HEAP32[$822>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum120$i = (($$sum2$i21$i) + ($$sum3132$i$i))|0;
           $823 = (($tbase$255$i) + ($$sum120$i)|0);
           $824 = HEAP32[$823>>2]|0;
           $825 = ($824|0)==(0|0);
           if ($825) {
            break;
           }
           $826 = HEAP32[(9540)>>2]|0;
           $827 = ($824>>>0)<($826>>>0);
           if ($827) {
            _abort();
            // unreachable;
           } else {
            $828 = ((($R$1$i$i)) + 20|0);
            HEAP32[$828>>2] = $824;
            $829 = ((($824)) + 24|0);
            HEAP32[$829>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $744 | $719;
         $$sum121$i = (($$sum9$i$i) + ($tsize$254$i))|0;
         $830 = (($tbase$255$i) + ($$sum121$i)|0);
         $831 = (($744) + ($725))|0;
         $oldfirst$0$i$i = $830;$qsize$0$i$i = $831;
        } else {
         $oldfirst$0$i$i = $720;$qsize$0$i$i = $725;
        }
        $832 = ((($oldfirst$0$i$i)) + 4|0);
        $833 = HEAP32[$832>>2]|0;
        $834 = $833 & -2;
        HEAP32[$832>>2] = $834;
        $835 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i19$i) + 4)|0;
        $836 = (($tbase$255$i) + ($$sum10$i$i)|0);
        HEAP32[$836>>2] = $835;
        $$sum11$i$i = (($qsize$0$i$i) + ($$sum$i19$i))|0;
        $837 = (($tbase$255$i) + ($$sum11$i$i)|0);
        HEAP32[$837>>2] = $qsize$0$i$i;
        $838 = $qsize$0$i$i >>> 3;
        $839 = ($qsize$0$i$i>>>0)<(256);
        if ($839) {
         $840 = $838 << 1;
         $841 = (9564 + ($840<<2)|0);
         $842 = HEAP32[9524>>2]|0;
         $843 = 1 << $838;
         $844 = $842 & $843;
         $845 = ($844|0)==(0);
         do {
          if ($845) {
           $846 = $842 | $843;
           HEAP32[9524>>2] = $846;
           $$pre$i22$i = (($840) + 2)|0;
           $$pre56$i$i = (9564 + ($$pre$i22$i<<2)|0);
           $$pre$phi$i23$iZ2D = $$pre56$i$i;$F4$0$i$i = $841;
          } else {
           $$sum29$i$i = (($840) + 2)|0;
           $847 = (9564 + ($$sum29$i$i<<2)|0);
           $848 = HEAP32[$847>>2]|0;
           $849 = HEAP32[(9540)>>2]|0;
           $850 = ($848>>>0)<($849>>>0);
           if (!($850)) {
            $$pre$phi$i23$iZ2D = $847;$F4$0$i$i = $848;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i23$iZ2D>>2] = $724;
         $851 = ((($F4$0$i$i)) + 12|0);
         HEAP32[$851>>2] = $724;
         $$sum27$i$i = (($$sum$i19$i) + 8)|0;
         $852 = (($tbase$255$i) + ($$sum27$i$i)|0);
         HEAP32[$852>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i19$i) + 12)|0;
         $853 = (($tbase$255$i) + ($$sum28$i$i)|0);
         HEAP32[$853>>2] = $841;
         break;
        }
        $854 = $qsize$0$i$i >>> 8;
        $855 = ($854|0)==(0);
        do {
         if ($855) {
          $I7$0$i$i = 0;
         } else {
          $856 = ($qsize$0$i$i>>>0)>(16777215);
          if ($856) {
           $I7$0$i$i = 31;
           break;
          }
          $857 = (($854) + 1048320)|0;
          $858 = $857 >>> 16;
          $859 = $858 & 8;
          $860 = $854 << $859;
          $861 = (($860) + 520192)|0;
          $862 = $861 >>> 16;
          $863 = $862 & 4;
          $864 = $863 | $859;
          $865 = $860 << $863;
          $866 = (($865) + 245760)|0;
          $867 = $866 >>> 16;
          $868 = $867 & 2;
          $869 = $864 | $868;
          $870 = (14 - ($869))|0;
          $871 = $865 << $868;
          $872 = $871 >>> 15;
          $873 = (($870) + ($872))|0;
          $874 = $873 << 1;
          $875 = (($873) + 7)|0;
          $876 = $qsize$0$i$i >>> $875;
          $877 = $876 & 1;
          $878 = $877 | $874;
          $I7$0$i$i = $878;
         }
        } while(0);
        $879 = (9828 + ($I7$0$i$i<<2)|0);
        $$sum12$i$i = (($$sum$i19$i) + 28)|0;
        $880 = (($tbase$255$i) + ($$sum12$i$i)|0);
        HEAP32[$880>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i19$i) + 16)|0;
        $881 = (($tbase$255$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i19$i) + 20)|0;
        $882 = (($tbase$255$i) + ($$sum14$i$i)|0);
        HEAP32[$882>>2] = 0;
        HEAP32[$881>>2] = 0;
        $883 = HEAP32[(9528)>>2]|0;
        $884 = 1 << $I7$0$i$i;
        $885 = $883 & $884;
        $886 = ($885|0)==(0);
        if ($886) {
         $887 = $883 | $884;
         HEAP32[(9528)>>2] = $887;
         HEAP32[$879>>2] = $724;
         $$sum15$i$i = (($$sum$i19$i) + 24)|0;
         $888 = (($tbase$255$i) + ($$sum15$i$i)|0);
         HEAP32[$888>>2] = $879;
         $$sum16$i$i = (($$sum$i19$i) + 12)|0;
         $889 = (($tbase$255$i) + ($$sum16$i$i)|0);
         HEAP32[$889>>2] = $724;
         $$sum17$i$i = (($$sum$i19$i) + 8)|0;
         $890 = (($tbase$255$i) + ($$sum17$i$i)|0);
         HEAP32[$890>>2] = $724;
         break;
        }
        $891 = HEAP32[$879>>2]|0;
        $892 = ((($891)) + 4|0);
        $893 = HEAP32[$892>>2]|0;
        $894 = $893 & -8;
        $895 = ($894|0)==($qsize$0$i$i|0);
        L418: do {
         if ($895) {
          $T$0$lcssa$i25$i = $891;
         } else {
          $896 = ($I7$0$i$i|0)==(31);
          $897 = $I7$0$i$i >>> 1;
          $898 = (25 - ($897))|0;
          $899 = $896 ? 0 : $898;
          $900 = $qsize$0$i$i << $899;
          $K8$051$i$i = $900;$T$050$i$i = $891;
          while(1) {
           $907 = $K8$051$i$i >>> 31;
           $908 = (((($T$050$i$i)) + 16|0) + ($907<<2)|0);
           $903 = HEAP32[$908>>2]|0;
           $909 = ($903|0)==(0|0);
           if ($909) {
            $$lcssa = $908;$T$050$i$i$lcssa = $T$050$i$i;
            break;
           }
           $901 = $K8$051$i$i << 1;
           $902 = ((($903)) + 4|0);
           $904 = HEAP32[$902>>2]|0;
           $905 = $904 & -8;
           $906 = ($905|0)==($qsize$0$i$i|0);
           if ($906) {
            $T$0$lcssa$i25$i = $903;
            break L418;
           } else {
            $K8$051$i$i = $901;$T$050$i$i = $903;
           }
          }
          $910 = HEAP32[(9540)>>2]|0;
          $911 = ($$lcssa>>>0)<($910>>>0);
          if ($911) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$$lcssa>>2] = $724;
           $$sum23$i$i = (($$sum$i19$i) + 24)|0;
           $912 = (($tbase$255$i) + ($$sum23$i$i)|0);
           HEAP32[$912>>2] = $T$050$i$i$lcssa;
           $$sum24$i$i = (($$sum$i19$i) + 12)|0;
           $913 = (($tbase$255$i) + ($$sum24$i$i)|0);
           HEAP32[$913>>2] = $724;
           $$sum25$i$i = (($$sum$i19$i) + 8)|0;
           $914 = (($tbase$255$i) + ($$sum25$i$i)|0);
           HEAP32[$914>>2] = $724;
           break L324;
          }
         }
        } while(0);
        $915 = ((($T$0$lcssa$i25$i)) + 8|0);
        $916 = HEAP32[$915>>2]|0;
        $917 = HEAP32[(9540)>>2]|0;
        $918 = ($916>>>0)>=($917>>>0);
        $not$$i26$i = ($T$0$lcssa$i25$i>>>0)>=($917>>>0);
        $919 = $918 & $not$$i26$i;
        if ($919) {
         $920 = ((($916)) + 12|0);
         HEAP32[$920>>2] = $724;
         HEAP32[$915>>2] = $724;
         $$sum20$i$i = (($$sum$i19$i) + 8)|0;
         $921 = (($tbase$255$i) + ($$sum20$i$i)|0);
         HEAP32[$921>>2] = $916;
         $$sum21$i$i = (($$sum$i19$i) + 12)|0;
         $922 = (($tbase$255$i) + ($$sum21$i$i)|0);
         HEAP32[$922>>2] = $T$0$lcssa$i25$i;
         $$sum22$i$i = (($$sum$i19$i) + 24)|0;
         $923 = (($tbase$255$i) + ($$sum22$i$i)|0);
         HEAP32[$923>>2] = 0;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $$sum1819$i$i = $711 | 8;
      $924 = (($tbase$255$i) + ($$sum1819$i$i)|0);
      $mem$0 = $924;
      return ($mem$0|0);
     } else {
      $sp$0$i$i$i = (9972);
     }
    }
    while(1) {
     $925 = HEAP32[$sp$0$i$i$i>>2]|0;
     $926 = ($925>>>0)>($635>>>0);
     if (!($926)) {
      $927 = ((($sp$0$i$i$i)) + 4|0);
      $928 = HEAP32[$927>>2]|0;
      $929 = (($925) + ($928)|0);
      $930 = ($929>>>0)>($635>>>0);
      if ($930) {
       $$lcssa215 = $925;$$lcssa216 = $928;$$lcssa217 = $929;
       break;
      }
     }
     $931 = ((($sp$0$i$i$i)) + 8|0);
     $932 = HEAP32[$931>>2]|0;
     $sp$0$i$i$i = $932;
    }
    $$sum$i14$i = (($$lcssa216) + -47)|0;
    $$sum1$i15$i = (($$lcssa216) + -39)|0;
    $933 = (($$lcssa215) + ($$sum1$i15$i)|0);
    $934 = $933;
    $935 = $934 & 7;
    $936 = ($935|0)==(0);
    $937 = (0 - ($934))|0;
    $938 = $937 & 7;
    $939 = $936 ? 0 : $938;
    $$sum2$i16$i = (($$sum$i14$i) + ($939))|0;
    $940 = (($$lcssa215) + ($$sum2$i16$i)|0);
    $941 = ((($635)) + 16|0);
    $942 = ($940>>>0)<($941>>>0);
    $943 = $942 ? $635 : $940;
    $944 = ((($943)) + 8|0);
    $945 = (($tsize$254$i) + -40)|0;
    $946 = ((($tbase$255$i)) + 8|0);
    $947 = $946;
    $948 = $947 & 7;
    $949 = ($948|0)==(0);
    $950 = (0 - ($947))|0;
    $951 = $950 & 7;
    $952 = $949 ? 0 : $951;
    $953 = (($tbase$255$i) + ($952)|0);
    $954 = (($945) - ($952))|0;
    HEAP32[(9548)>>2] = $953;
    HEAP32[(9536)>>2] = $954;
    $955 = $954 | 1;
    $$sum$i$i$i = (($952) + 4)|0;
    $956 = (($tbase$255$i) + ($$sum$i$i$i)|0);
    HEAP32[$956>>2] = $955;
    $$sum2$i$i$i = (($tsize$254$i) + -36)|0;
    $957 = (($tbase$255$i) + ($$sum2$i$i$i)|0);
    HEAP32[$957>>2] = 40;
    $958 = HEAP32[(10012)>>2]|0;
    HEAP32[(9552)>>2] = $958;
    $959 = ((($943)) + 4|0);
    HEAP32[$959>>2] = 27;
    ;HEAP32[$944>>2]=HEAP32[(9972)>>2]|0;HEAP32[$944+4>>2]=HEAP32[(9972)+4>>2]|0;HEAP32[$944+8>>2]=HEAP32[(9972)+8>>2]|0;HEAP32[$944+12>>2]=HEAP32[(9972)+12>>2]|0;
    HEAP32[(9972)>>2] = $tbase$255$i;
    HEAP32[(9976)>>2] = $tsize$254$i;
    HEAP32[(9984)>>2] = 0;
    HEAP32[(9980)>>2] = $944;
    $960 = ((($943)) + 28|0);
    HEAP32[$960>>2] = 7;
    $961 = ((($943)) + 32|0);
    $962 = ($961>>>0)<($$lcssa217>>>0);
    if ($962) {
     $964 = $960;
     while(1) {
      $963 = ((($964)) + 4|0);
      HEAP32[$963>>2] = 7;
      $965 = ((($964)) + 8|0);
      $966 = ($965>>>0)<($$lcssa217>>>0);
      if ($966) {
       $964 = $963;
      } else {
       break;
      }
     }
    }
    $967 = ($943|0)==($635|0);
    if (!($967)) {
     $968 = $943;
     $969 = $635;
     $970 = (($968) - ($969))|0;
     $971 = HEAP32[$959>>2]|0;
     $972 = $971 & -2;
     HEAP32[$959>>2] = $972;
     $973 = $970 | 1;
     $974 = ((($635)) + 4|0);
     HEAP32[$974>>2] = $973;
     HEAP32[$943>>2] = $970;
     $975 = $970 >>> 3;
     $976 = ($970>>>0)<(256);
     if ($976) {
      $977 = $975 << 1;
      $978 = (9564 + ($977<<2)|0);
      $979 = HEAP32[9524>>2]|0;
      $980 = 1 << $975;
      $981 = $979 & $980;
      $982 = ($981|0)==(0);
      if ($982) {
       $983 = $979 | $980;
       HEAP32[9524>>2] = $983;
       $$pre$i$i = (($977) + 2)|0;
       $$pre14$i$i = (9564 + ($$pre$i$i<<2)|0);
       $$pre$phi$i$iZ2D = $$pre14$i$i;$F$0$i$i = $978;
      } else {
       $$sum4$i$i = (($977) + 2)|0;
       $984 = (9564 + ($$sum4$i$i<<2)|0);
       $985 = HEAP32[$984>>2]|0;
       $986 = HEAP32[(9540)>>2]|0;
       $987 = ($985>>>0)<($986>>>0);
       if ($987) {
        _abort();
        // unreachable;
       } else {
        $$pre$phi$i$iZ2D = $984;$F$0$i$i = $985;
       }
      }
      HEAP32[$$pre$phi$i$iZ2D>>2] = $635;
      $988 = ((($F$0$i$i)) + 12|0);
      HEAP32[$988>>2] = $635;
      $989 = ((($635)) + 8|0);
      HEAP32[$989>>2] = $F$0$i$i;
      $990 = ((($635)) + 12|0);
      HEAP32[$990>>2] = $978;
      break;
     }
     $991 = $970 >>> 8;
     $992 = ($991|0)==(0);
     if ($992) {
      $I1$0$i$i = 0;
     } else {
      $993 = ($970>>>0)>(16777215);
      if ($993) {
       $I1$0$i$i = 31;
      } else {
       $994 = (($991) + 1048320)|0;
       $995 = $994 >>> 16;
       $996 = $995 & 8;
       $997 = $991 << $996;
       $998 = (($997) + 520192)|0;
       $999 = $998 >>> 16;
       $1000 = $999 & 4;
       $1001 = $1000 | $996;
       $1002 = $997 << $1000;
       $1003 = (($1002) + 245760)|0;
       $1004 = $1003 >>> 16;
       $1005 = $1004 & 2;
       $1006 = $1001 | $1005;
       $1007 = (14 - ($1006))|0;
       $1008 = $1002 << $1005;
       $1009 = $1008 >>> 15;
       $1010 = (($1007) + ($1009))|0;
       $1011 = $1010 << 1;
       $1012 = (($1010) + 7)|0;
       $1013 = $970 >>> $1012;
       $1014 = $1013 & 1;
       $1015 = $1014 | $1011;
       $I1$0$i$i = $1015;
      }
     }
     $1016 = (9828 + ($I1$0$i$i<<2)|0);
     $1017 = ((($635)) + 28|0);
     HEAP32[$1017>>2] = $I1$0$i$i;
     $1018 = ((($635)) + 20|0);
     HEAP32[$1018>>2] = 0;
     HEAP32[$941>>2] = 0;
     $1019 = HEAP32[(9528)>>2]|0;
     $1020 = 1 << $I1$0$i$i;
     $1021 = $1019 & $1020;
     $1022 = ($1021|0)==(0);
     if ($1022) {
      $1023 = $1019 | $1020;
      HEAP32[(9528)>>2] = $1023;
      HEAP32[$1016>>2] = $635;
      $1024 = ((($635)) + 24|0);
      HEAP32[$1024>>2] = $1016;
      $1025 = ((($635)) + 12|0);
      HEAP32[$1025>>2] = $635;
      $1026 = ((($635)) + 8|0);
      HEAP32[$1026>>2] = $635;
      break;
     }
     $1027 = HEAP32[$1016>>2]|0;
     $1028 = ((($1027)) + 4|0);
     $1029 = HEAP32[$1028>>2]|0;
     $1030 = $1029 & -8;
     $1031 = ($1030|0)==($970|0);
     L459: do {
      if ($1031) {
       $T$0$lcssa$i$i = $1027;
      } else {
       $1032 = ($I1$0$i$i|0)==(31);
       $1033 = $I1$0$i$i >>> 1;
       $1034 = (25 - ($1033))|0;
       $1035 = $1032 ? 0 : $1034;
       $1036 = $970 << $1035;
       $K2$07$i$i = $1036;$T$06$i$i = $1027;
       while(1) {
        $1043 = $K2$07$i$i >>> 31;
        $1044 = (((($T$06$i$i)) + 16|0) + ($1043<<2)|0);
        $1039 = HEAP32[$1044>>2]|0;
        $1045 = ($1039|0)==(0|0);
        if ($1045) {
         $$lcssa211 = $1044;$T$06$i$i$lcssa = $T$06$i$i;
         break;
        }
        $1037 = $K2$07$i$i << 1;
        $1038 = ((($1039)) + 4|0);
        $1040 = HEAP32[$1038>>2]|0;
        $1041 = $1040 & -8;
        $1042 = ($1041|0)==($970|0);
        if ($1042) {
         $T$0$lcssa$i$i = $1039;
         break L459;
        } else {
         $K2$07$i$i = $1037;$T$06$i$i = $1039;
        }
       }
       $1046 = HEAP32[(9540)>>2]|0;
       $1047 = ($$lcssa211>>>0)<($1046>>>0);
       if ($1047) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$$lcssa211>>2] = $635;
        $1048 = ((($635)) + 24|0);
        HEAP32[$1048>>2] = $T$06$i$i$lcssa;
        $1049 = ((($635)) + 12|0);
        HEAP32[$1049>>2] = $635;
        $1050 = ((($635)) + 8|0);
        HEAP32[$1050>>2] = $635;
        break L299;
       }
      }
     } while(0);
     $1051 = ((($T$0$lcssa$i$i)) + 8|0);
     $1052 = HEAP32[$1051>>2]|0;
     $1053 = HEAP32[(9540)>>2]|0;
     $1054 = ($1052>>>0)>=($1053>>>0);
     $not$$i$i = ($T$0$lcssa$i$i>>>0)>=($1053>>>0);
     $1055 = $1054 & $not$$i$i;
     if ($1055) {
      $1056 = ((($1052)) + 12|0);
      HEAP32[$1056>>2] = $635;
      HEAP32[$1051>>2] = $635;
      $1057 = ((($635)) + 8|0);
      HEAP32[$1057>>2] = $1052;
      $1058 = ((($635)) + 12|0);
      HEAP32[$1058>>2] = $T$0$lcssa$i$i;
      $1059 = ((($635)) + 24|0);
      HEAP32[$1059>>2] = 0;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   }
  } while(0);
  $1060 = HEAP32[(9536)>>2]|0;
  $1061 = ($1060>>>0)>($nb$0>>>0);
  if ($1061) {
   $1062 = (($1060) - ($nb$0))|0;
   HEAP32[(9536)>>2] = $1062;
   $1063 = HEAP32[(9548)>>2]|0;
   $1064 = (($1063) + ($nb$0)|0);
   HEAP32[(9548)>>2] = $1064;
   $1065 = $1062 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1066 = (($1063) + ($$sum$i32)|0);
   HEAP32[$1066>>2] = $1065;
   $1067 = $nb$0 | 3;
   $1068 = ((($1063)) + 4|0);
   HEAP32[$1068>>2] = $1067;
   $1069 = ((($1063)) + 8|0);
   $mem$0 = $1069;
   return ($mem$0|0);
  }
 }
 $1070 = (___errno_location()|0);
 HEAP32[$1070>>2] = 12;
 $mem$0 = 0;
 return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$lcssa = 0, $$pre = 0, $$pre$phi59Z2D = 0, $$pre$phi61Z2D = 0, $$pre$phiZ2D = 0, $$pre57 = 0, $$pre58 = 0, $$pre60 = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum1718 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0;
 var $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0;
 var $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0;
 var $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0;
 var $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0;
 var $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0;
 var $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0;
 var $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0;
 var $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0;
 var $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0;
 var $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0;
 var $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0;
 var $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0;
 var $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0;
 var $321 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $K19$052 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0;
 var $R7$0 = 0, $R7$0$lcssa = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $RP9$0 = 0, $RP9$0$lcssa = 0, $T$0$lcssa = 0, $T$051 = 0, $T$051$lcssa = 0, $cond = 0, $cond47 = 0, $not$ = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  return;
 }
 $1 = ((($mem)) + -8|0);
 $2 = HEAP32[(9540)>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = ((($mem)) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[(9544)>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $103 = (($mem) + ($$sum3)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $104 & 3;
    $106 = ($105|0)==(3);
    if (!($106)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[(9532)>>2] = $15;
    $107 = $104 & -2;
    HEAP32[$103>>2] = $107;
    $108 = $15 | 1;
    $$sum20 = (($$sum2) + 4)|0;
    $109 = (($mem) + ($$sum20)|0);
    HEAP32[$109>>2] = $108;
    HEAP32[$9>>2] = $15;
    return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum30 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum30)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum31 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum31)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = (9564 + ($25<<2)|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = ((($22)) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[9524>>2]|0;
     $36 = $35 & $34;
     HEAP32[9524>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre60 = ((($24)) + 8|0);
     $$pre$phi61Z2D = $$pre60;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = ((($24)) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi61Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = ((($22)) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi61Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum22 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum22)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum23 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum23)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum25 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum25)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum24 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum24)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = ((($R$0)) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = ((($R$0)) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0$lcssa>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum29 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum29)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = ((($49)) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = ((($46)) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum26 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum26)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (9828 + ($72<<2)|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[(9528)>>2]|0;
      $79 = $78 & $77;
      HEAP32[(9528)>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[(9540)>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = ((($44)) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = ((($44)) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[(9540)>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = ((($R$1)) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum27 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum27)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = ($91>>>0)<($87>>>0);
      if ($93) {
       _abort();
       // unreachable;
      } else {
       $94 = ((($R$1)) + 16|0);
       HEAP32[$94>>2] = $91;
       $95 = ((($91)) + 24|0);
       HEAP32[$95>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum2) + 20)|0;
    $96 = (($mem) + ($$sum28)|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ($97|0)==(0|0);
    if ($98) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $99 = HEAP32[(9540)>>2]|0;
     $100 = ($97>>>0)<($99>>>0);
     if ($100) {
      _abort();
      // unreachable;
     } else {
      $101 = ((($R$1)) + 20|0);
      HEAP32[$101>>2] = $97;
      $102 = ((($97)) + 24|0);
      HEAP32[$102>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $110 = ($p$0>>>0)<($9>>>0);
 if (!($110)) {
  _abort();
  // unreachable;
 }
 $$sum19 = (($8) + -4)|0;
 $111 = (($mem) + ($$sum19)|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $112 & 1;
 $114 = ($113|0)==(0);
 if ($114) {
  _abort();
  // unreachable;
 }
 $115 = $112 & 2;
 $116 = ($115|0)==(0);
 if ($116) {
  $117 = HEAP32[(9548)>>2]|0;
  $118 = ($9|0)==($117|0);
  if ($118) {
   $119 = HEAP32[(9536)>>2]|0;
   $120 = (($119) + ($psize$0))|0;
   HEAP32[(9536)>>2] = $120;
   HEAP32[(9548)>>2] = $p$0;
   $121 = $120 | 1;
   $122 = ((($p$0)) + 4|0);
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[(9544)>>2]|0;
   $124 = ($p$0|0)==($123|0);
   if (!($124)) {
    return;
   }
   HEAP32[(9544)>>2] = 0;
   HEAP32[(9532)>>2] = 0;
   return;
  }
  $125 = HEAP32[(9544)>>2]|0;
  $126 = ($9|0)==($125|0);
  if ($126) {
   $127 = HEAP32[(9532)>>2]|0;
   $128 = (($127) + ($psize$0))|0;
   HEAP32[(9532)>>2] = $128;
   HEAP32[(9544)>>2] = $p$0;
   $129 = $128 | 1;
   $130 = ((($p$0)) + 4|0);
   HEAP32[$130>>2] = $129;
   $131 = (($p$0) + ($128)|0);
   HEAP32[$131>>2] = $128;
   return;
  }
  $132 = $112 & -8;
  $133 = (($132) + ($psize$0))|0;
  $134 = $112 >>> 3;
  $135 = ($112>>>0)<(256);
  do {
   if ($135) {
    $136 = (($mem) + ($8)|0);
    $137 = HEAP32[$136>>2]|0;
    $$sum1718 = $8 | 4;
    $138 = (($mem) + ($$sum1718)|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = $134 << 1;
    $141 = (9564 + ($140<<2)|0);
    $142 = ($137|0)==($141|0);
    if (!($142)) {
     $143 = HEAP32[(9540)>>2]|0;
     $144 = ($137>>>0)<($143>>>0);
     if ($144) {
      _abort();
      // unreachable;
     }
     $145 = ((($137)) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = ($146|0)==($9|0);
     if (!($147)) {
      _abort();
      // unreachable;
     }
    }
    $148 = ($139|0)==($137|0);
    if ($148) {
     $149 = 1 << $134;
     $150 = $149 ^ -1;
     $151 = HEAP32[9524>>2]|0;
     $152 = $151 & $150;
     HEAP32[9524>>2] = $152;
     break;
    }
    $153 = ($139|0)==($141|0);
    if ($153) {
     $$pre58 = ((($139)) + 8|0);
     $$pre$phi59Z2D = $$pre58;
    } else {
     $154 = HEAP32[(9540)>>2]|0;
     $155 = ($139>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     }
     $156 = ((($139)) + 8|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($157|0)==($9|0);
     if ($158) {
      $$pre$phi59Z2D = $156;
     } else {
      _abort();
      // unreachable;
     }
    }
    $159 = ((($137)) + 12|0);
    HEAP32[$159>>2] = $139;
    HEAP32[$$pre$phi59Z2D>>2] = $137;
   } else {
    $$sum5 = (($8) + 16)|0;
    $160 = (($mem) + ($$sum5)|0);
    $161 = HEAP32[$160>>2]|0;
    $$sum67 = $8 | 4;
    $162 = (($mem) + ($$sum67)|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = ($163|0)==($9|0);
    do {
     if ($164) {
      $$sum9 = (($8) + 12)|0;
      $175 = (($mem) + ($$sum9)|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = ($176|0)==(0|0);
      if ($177) {
       $$sum8 = (($8) + 8)|0;
       $178 = (($mem) + ($$sum8)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      } else {
       $R7$0 = $176;$RP9$0 = $175;
      }
      while(1) {
       $181 = ((($R7$0)) + 20|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ($182|0)==(0|0);
       if (!($183)) {
        $R7$0 = $182;$RP9$0 = $181;
        continue;
       }
       $184 = ((($R7$0)) + 16|0);
       $185 = HEAP32[$184>>2]|0;
       $186 = ($185|0)==(0|0);
       if ($186) {
        $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
        break;
       } else {
        $R7$0 = $185;$RP9$0 = $184;
       }
      }
      $187 = HEAP32[(9540)>>2]|0;
      $188 = ($RP9$0$lcssa>>>0)<($187>>>0);
      if ($188) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0$lcssa>>2] = 0;
       $R7$1 = $R7$0$lcssa;
       break;
      }
     } else {
      $165 = (($mem) + ($8)|0);
      $166 = HEAP32[$165>>2]|0;
      $167 = HEAP32[(9540)>>2]|0;
      $168 = ($166>>>0)<($167>>>0);
      if ($168) {
       _abort();
       // unreachable;
      }
      $169 = ((($166)) + 12|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==($9|0);
      if (!($171)) {
       _abort();
       // unreachable;
      }
      $172 = ((($163)) + 8|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = ($173|0)==($9|0);
      if ($174) {
       HEAP32[$169>>2] = $163;
       HEAP32[$172>>2] = $166;
       $R7$1 = $163;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $189 = ($161|0)==(0|0);
    if (!($189)) {
     $$sum12 = (($8) + 20)|0;
     $190 = (($mem) + ($$sum12)|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (9828 + ($191<<2)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($9|0)==($193|0);
     if ($194) {
      HEAP32[$192>>2] = $R7$1;
      $cond47 = ($R7$1|0)==(0|0);
      if ($cond47) {
       $195 = 1 << $191;
       $196 = $195 ^ -1;
       $197 = HEAP32[(9528)>>2]|0;
       $198 = $197 & $196;
       HEAP32[(9528)>>2] = $198;
       break;
      }
     } else {
      $199 = HEAP32[(9540)>>2]|0;
      $200 = ($161>>>0)<($199>>>0);
      if ($200) {
       _abort();
       // unreachable;
      }
      $201 = ((($161)) + 16|0);
      $202 = HEAP32[$201>>2]|0;
      $203 = ($202|0)==($9|0);
      if ($203) {
       HEAP32[$201>>2] = $R7$1;
      } else {
       $204 = ((($161)) + 20|0);
       HEAP32[$204>>2] = $R7$1;
      }
      $205 = ($R7$1|0)==(0|0);
      if ($205) {
       break;
      }
     }
     $206 = HEAP32[(9540)>>2]|0;
     $207 = ($R7$1>>>0)<($206>>>0);
     if ($207) {
      _abort();
      // unreachable;
     }
     $208 = ((($R7$1)) + 24|0);
     HEAP32[$208>>2] = $161;
     $$sum13 = (($8) + 8)|0;
     $209 = (($mem) + ($$sum13)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     do {
      if (!($211)) {
       $212 = ($210>>>0)<($206>>>0);
       if ($212) {
        _abort();
        // unreachable;
       } else {
        $213 = ((($R7$1)) + 16|0);
        HEAP32[$213>>2] = $210;
        $214 = ((($210)) + 24|0);
        HEAP32[$214>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum14 = (($8) + 12)|0;
     $215 = (($mem) + ($$sum14)|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = ($216|0)==(0|0);
     if (!($217)) {
      $218 = HEAP32[(9540)>>2]|0;
      $219 = ($216>>>0)<($218>>>0);
      if ($219) {
       _abort();
       // unreachable;
      } else {
       $220 = ((($R7$1)) + 20|0);
       HEAP32[$220>>2] = $216;
       $221 = ((($216)) + 24|0);
       HEAP32[$221>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $222 = $133 | 1;
  $223 = ((($p$0)) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($p$0) + ($133)|0);
  HEAP32[$224>>2] = $133;
  $225 = HEAP32[(9544)>>2]|0;
  $226 = ($p$0|0)==($225|0);
  if ($226) {
   HEAP32[(9532)>>2] = $133;
   return;
  } else {
   $psize$1 = $133;
  }
 } else {
  $227 = $112 & -2;
  HEAP32[$111>>2] = $227;
  $228 = $psize$0 | 1;
  $229 = ((($p$0)) + 4|0);
  HEAP32[$229>>2] = $228;
  $230 = (($p$0) + ($psize$0)|0);
  HEAP32[$230>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $231 = $psize$1 >>> 3;
 $232 = ($psize$1>>>0)<(256);
 if ($232) {
  $233 = $231 << 1;
  $234 = (9564 + ($233<<2)|0);
  $235 = HEAP32[9524>>2]|0;
  $236 = 1 << $231;
  $237 = $235 & $236;
  $238 = ($237|0)==(0);
  if ($238) {
   $239 = $235 | $236;
   HEAP32[9524>>2] = $239;
   $$pre = (($233) + 2)|0;
   $$pre57 = (9564 + ($$pre<<2)|0);
   $$pre$phiZ2D = $$pre57;$F16$0 = $234;
  } else {
   $$sum11 = (($233) + 2)|0;
   $240 = (9564 + ($$sum11<<2)|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[(9540)>>2]|0;
   $243 = ($241>>>0)<($242>>>0);
   if ($243) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $240;$F16$0 = $241;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $244 = ((($F16$0)) + 12|0);
  HEAP32[$244>>2] = $p$0;
  $245 = ((($p$0)) + 8|0);
  HEAP32[$245>>2] = $F16$0;
  $246 = ((($p$0)) + 12|0);
  HEAP32[$246>>2] = $234;
  return;
 }
 $247 = $psize$1 >>> 8;
 $248 = ($247|0)==(0);
 if ($248) {
  $I18$0 = 0;
 } else {
  $249 = ($psize$1>>>0)>(16777215);
  if ($249) {
   $I18$0 = 31;
  } else {
   $250 = (($247) + 1048320)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 8;
   $253 = $247 << $252;
   $254 = (($253) + 520192)|0;
   $255 = $254 >>> 16;
   $256 = $255 & 4;
   $257 = $256 | $252;
   $258 = $253 << $256;
   $259 = (($258) + 245760)|0;
   $260 = $259 >>> 16;
   $261 = $260 & 2;
   $262 = $257 | $261;
   $263 = (14 - ($262))|0;
   $264 = $258 << $261;
   $265 = $264 >>> 15;
   $266 = (($263) + ($265))|0;
   $267 = $266 << 1;
   $268 = (($266) + 7)|0;
   $269 = $psize$1 >>> $268;
   $270 = $269 & 1;
   $271 = $270 | $267;
   $I18$0 = $271;
  }
 }
 $272 = (9828 + ($I18$0<<2)|0);
 $273 = ((($p$0)) + 28|0);
 HEAP32[$273>>2] = $I18$0;
 $274 = ((($p$0)) + 16|0);
 $275 = ((($p$0)) + 20|0);
 HEAP32[$275>>2] = 0;
 HEAP32[$274>>2] = 0;
 $276 = HEAP32[(9528)>>2]|0;
 $277 = 1 << $I18$0;
 $278 = $276 & $277;
 $279 = ($278|0)==(0);
 L199: do {
  if ($279) {
   $280 = $276 | $277;
   HEAP32[(9528)>>2] = $280;
   HEAP32[$272>>2] = $p$0;
   $281 = ((($p$0)) + 24|0);
   HEAP32[$281>>2] = $272;
   $282 = ((($p$0)) + 12|0);
   HEAP32[$282>>2] = $p$0;
   $283 = ((($p$0)) + 8|0);
   HEAP32[$283>>2] = $p$0;
  } else {
   $284 = HEAP32[$272>>2]|0;
   $285 = ((($284)) + 4|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = $286 & -8;
   $288 = ($287|0)==($psize$1|0);
   L202: do {
    if ($288) {
     $T$0$lcssa = $284;
    } else {
     $289 = ($I18$0|0)==(31);
     $290 = $I18$0 >>> 1;
     $291 = (25 - ($290))|0;
     $292 = $289 ? 0 : $291;
     $293 = $psize$1 << $292;
     $K19$052 = $293;$T$051 = $284;
     while(1) {
      $300 = $K19$052 >>> 31;
      $301 = (((($T$051)) + 16|0) + ($300<<2)|0);
      $296 = HEAP32[$301>>2]|0;
      $302 = ($296|0)==(0|0);
      if ($302) {
       $$lcssa = $301;$T$051$lcssa = $T$051;
       break;
      }
      $294 = $K19$052 << 1;
      $295 = ((($296)) + 4|0);
      $297 = HEAP32[$295>>2]|0;
      $298 = $297 & -8;
      $299 = ($298|0)==($psize$1|0);
      if ($299) {
       $T$0$lcssa = $296;
       break L202;
      } else {
       $K19$052 = $294;$T$051 = $296;
      }
     }
     $303 = HEAP32[(9540)>>2]|0;
     $304 = ($$lcssa>>>0)<($303>>>0);
     if ($304) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$$lcssa>>2] = $p$0;
      $305 = ((($p$0)) + 24|0);
      HEAP32[$305>>2] = $T$051$lcssa;
      $306 = ((($p$0)) + 12|0);
      HEAP32[$306>>2] = $p$0;
      $307 = ((($p$0)) + 8|0);
      HEAP32[$307>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $308 = ((($T$0$lcssa)) + 8|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[(9540)>>2]|0;
   $311 = ($309>>>0)>=($310>>>0);
   $not$ = ($T$0$lcssa>>>0)>=($310>>>0);
   $312 = $311 & $not$;
   if ($312) {
    $313 = ((($309)) + 12|0);
    HEAP32[$313>>2] = $p$0;
    HEAP32[$308>>2] = $p$0;
    $314 = ((($p$0)) + 8|0);
    HEAP32[$314>>2] = $309;
    $315 = ((($p$0)) + 12|0);
    HEAP32[$315>>2] = $T$0$lcssa;
    $316 = ((($p$0)) + 24|0);
    HEAP32[$316>>2] = 0;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $317 = HEAP32[(9556)>>2]|0;
 $318 = (($317) + -1)|0;
 HEAP32[(9556)>>2] = $318;
 $319 = ($318|0)==(0);
 if ($319) {
  $sp$0$in$i = (9980);
 } else {
  return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $320 = ($sp$0$i|0)==(0|0);
  $321 = ((($sp$0$i)) + 8|0);
  if ($320) {
   break;
  } else {
   $sp$0$in$i = $321;
  }
 }
 HEAP32[(9556)>>2] = -1;
 return;
}
function _realloc($oldmem,$bytes) {
 $oldmem = $oldmem|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $mem$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($oldmem|0)==(0|0);
 if ($0) {
  $1 = (_malloc($bytes)|0);
  $mem$0 = $1;
  return ($mem$0|0);
 }
 $2 = ($bytes>>>0)>(4294967231);
 if ($2) {
  $3 = (___errno_location()|0);
  HEAP32[$3>>2] = 12;
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $4 = ($bytes>>>0)<(11);
 $5 = (($bytes) + 11)|0;
 $6 = $5 & -8;
 $7 = $4 ? 16 : $6;
 $8 = ((($oldmem)) + -8|0);
 $9 = (_try_realloc_chunk($8,$7)|0);
 $10 = ($9|0)==(0|0);
 if (!($10)) {
  $11 = ((($9)) + 8|0);
  $mem$0 = $11;
  return ($mem$0|0);
 }
 $12 = (_malloc($bytes)|0);
 $13 = ($12|0)==(0|0);
 if ($13) {
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $14 = ((($oldmem)) + -4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 & -8;
 $17 = $15 & 3;
 $18 = ($17|0)==(0);
 $19 = $18 ? 8 : 4;
 $20 = (($16) - ($19))|0;
 $21 = ($20>>>0)<($bytes>>>0);
 $22 = $21 ? $20 : $bytes;
 _memcpy(($12|0),($oldmem|0),($22|0))|0;
 _free($oldmem);
 $mem$0 = $12;
 return ($mem$0|0);
}
function _try_realloc_chunk($p,$nb) {
 $p = $p|0;
 $nb = $nb|0;
 var $$pre = 0, $$pre$phiZ2D = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum2728 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum78 = 0;
 var $$sum910 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $cond = 0, $newp$0 = 0, $notlhs = 0;
 var $notrhs = 0, $or$cond$not = 0, $or$cond30 = 0, $storemerge = 0, $storemerge21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($p)) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 & -8;
 $3 = (($p) + ($2)|0);
 $4 = HEAP32[(9540)>>2]|0;
 $5 = $1 & 3;
 $notlhs = ($p>>>0)>=($4>>>0);
 $notrhs = ($5|0)!=(1);
 $or$cond$not = $notrhs & $notlhs;
 $6 = ($p>>>0)<($3>>>0);
 $or$cond30 = $or$cond$not & $6;
 if (!($or$cond30)) {
  _abort();
  // unreachable;
 }
 $$sum2728 = $2 | 4;
 $7 = (($p) + ($$sum2728)|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)==(0);
 if ($10) {
  _abort();
  // unreachable;
 }
 $11 = ($5|0)==(0);
 if ($11) {
  $12 = ($nb>>>0)<(256);
  if ($12) {
   $newp$0 = 0;
   return ($newp$0|0);
  }
  $13 = (($nb) + 4)|0;
  $14 = ($2>>>0)<($13>>>0);
  if (!($14)) {
   $15 = (($2) - ($nb))|0;
   $16 = HEAP32[(10004)>>2]|0;
   $17 = $16 << 1;
   $18 = ($15>>>0)>($17>>>0);
   if (!($18)) {
    $newp$0 = $p;
    return ($newp$0|0);
   }
  }
  $newp$0 = 0;
  return ($newp$0|0);
 }
 $19 = ($2>>>0)<($nb>>>0);
 if (!($19)) {
  $20 = (($2) - ($nb))|0;
  $21 = ($20>>>0)>(15);
  if (!($21)) {
   $newp$0 = $p;
   return ($newp$0|0);
  }
  $22 = (($p) + ($nb)|0);
  $23 = $1 & 1;
  $24 = $23 | $nb;
  $25 = $24 | 2;
  HEAP32[$0>>2] = $25;
  $$sum23 = (($nb) + 4)|0;
  $26 = (($p) + ($$sum23)|0);
  $27 = $20 | 3;
  HEAP32[$26>>2] = $27;
  $28 = HEAP32[$7>>2]|0;
  $29 = $28 | 1;
  HEAP32[$7>>2] = $29;
  _dispose_chunk($22,$20);
  $newp$0 = $p;
  return ($newp$0|0);
 }
 $30 = HEAP32[(9548)>>2]|0;
 $31 = ($3|0)==($30|0);
 if ($31) {
  $32 = HEAP32[(9536)>>2]|0;
  $33 = (($32) + ($2))|0;
  $34 = ($33>>>0)>($nb>>>0);
  if (!($34)) {
   $newp$0 = 0;
   return ($newp$0|0);
  }
  $35 = (($33) - ($nb))|0;
  $36 = (($p) + ($nb)|0);
  $37 = $1 & 1;
  $38 = $37 | $nb;
  $39 = $38 | 2;
  HEAP32[$0>>2] = $39;
  $$sum22 = (($nb) + 4)|0;
  $40 = (($p) + ($$sum22)|0);
  $41 = $35 | 1;
  HEAP32[$40>>2] = $41;
  HEAP32[(9548)>>2] = $36;
  HEAP32[(9536)>>2] = $35;
  $newp$0 = $p;
  return ($newp$0|0);
 }
 $42 = HEAP32[(9544)>>2]|0;
 $43 = ($3|0)==($42|0);
 if ($43) {
  $44 = HEAP32[(9532)>>2]|0;
  $45 = (($44) + ($2))|0;
  $46 = ($45>>>0)<($nb>>>0);
  if ($46) {
   $newp$0 = 0;
   return ($newp$0|0);
  }
  $47 = (($45) - ($nb))|0;
  $48 = ($47>>>0)>(15);
  if ($48) {
   $49 = (($p) + ($nb)|0);
   $50 = (($p) + ($45)|0);
   $51 = $1 & 1;
   $52 = $51 | $nb;
   $53 = $52 | 2;
   HEAP32[$0>>2] = $53;
   $$sum19 = (($nb) + 4)|0;
   $54 = (($p) + ($$sum19)|0);
   $55 = $47 | 1;
   HEAP32[$54>>2] = $55;
   HEAP32[$50>>2] = $47;
   $$sum20 = (($45) + 4)|0;
   $56 = (($p) + ($$sum20)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = $57 & -2;
   HEAP32[$56>>2] = $58;
   $storemerge = $49;$storemerge21 = $47;
  } else {
   $59 = $1 & 1;
   $60 = $59 | $45;
   $61 = $60 | 2;
   HEAP32[$0>>2] = $61;
   $$sum17 = (($45) + 4)|0;
   $62 = (($p) + ($$sum17)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $63 | 1;
   HEAP32[$62>>2] = $64;
   $storemerge = 0;$storemerge21 = 0;
  }
  HEAP32[(9532)>>2] = $storemerge21;
  HEAP32[(9544)>>2] = $storemerge;
  $newp$0 = $p;
  return ($newp$0|0);
 }
 $65 = $8 & 2;
 $66 = ($65|0)==(0);
 if (!($66)) {
  $newp$0 = 0;
  return ($newp$0|0);
 }
 $67 = $8 & -8;
 $68 = (($67) + ($2))|0;
 $69 = ($68>>>0)<($nb>>>0);
 if ($69) {
  $newp$0 = 0;
  return ($newp$0|0);
 }
 $70 = (($68) - ($nb))|0;
 $71 = $8 >>> 3;
 $72 = ($8>>>0)<(256);
 do {
  if ($72) {
   $$sum15 = (($2) + 8)|0;
   $73 = (($p) + ($$sum15)|0);
   $74 = HEAP32[$73>>2]|0;
   $$sum16 = (($2) + 12)|0;
   $75 = (($p) + ($$sum16)|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = $71 << 1;
   $78 = (9564 + ($77<<2)|0);
   $79 = ($74|0)==($78|0);
   if (!($79)) {
    $80 = ($74>>>0)<($4>>>0);
    if ($80) {
     _abort();
     // unreachable;
    }
    $81 = ((($74)) + 12|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = ($82|0)==($3|0);
    if (!($83)) {
     _abort();
     // unreachable;
    }
   }
   $84 = ($76|0)==($74|0);
   if ($84) {
    $85 = 1 << $71;
    $86 = $85 ^ -1;
    $87 = HEAP32[9524>>2]|0;
    $88 = $87 & $86;
    HEAP32[9524>>2] = $88;
    break;
   }
   $89 = ($76|0)==($78|0);
   if ($89) {
    $$pre = ((($76)) + 8|0);
    $$pre$phiZ2D = $$pre;
   } else {
    $90 = ($76>>>0)<($4>>>0);
    if ($90) {
     _abort();
     // unreachable;
    }
    $91 = ((($76)) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)==($3|0);
    if ($93) {
     $$pre$phiZ2D = $91;
    } else {
     _abort();
     // unreachable;
    }
   }
   $94 = ((($74)) + 12|0);
   HEAP32[$94>>2] = $76;
   HEAP32[$$pre$phiZ2D>>2] = $74;
  } else {
   $$sum = (($2) + 24)|0;
   $95 = (($p) + ($$sum)|0);
   $96 = HEAP32[$95>>2]|0;
   $$sum2 = (($2) + 12)|0;
   $97 = (($p) + ($$sum2)|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($98|0)==($3|0);
   do {
    if ($99) {
     $$sum4 = (($2) + 20)|0;
     $109 = (($p) + ($$sum4)|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ($110|0)==(0|0);
     if ($111) {
      $$sum3 = (($2) + 16)|0;
      $112 = (($p) + ($$sum3)|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = ($113|0)==(0|0);
      if ($114) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $113;$RP$0 = $112;
      }
     } else {
      $R$0 = $110;$RP$0 = $109;
     }
     while(1) {
      $115 = ((($R$0)) + 20|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($116|0)==(0|0);
      if (!($117)) {
       $R$0 = $116;$RP$0 = $115;
       continue;
      }
      $118 = ((($R$0)) + 16|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = ($119|0)==(0|0);
      if ($120) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $119;$RP$0 = $118;
      }
     }
     $121 = ($RP$0$lcssa>>>0)<($4>>>0);
     if ($121) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum14 = (($2) + 8)|0;
     $100 = (($p) + ($$sum14)|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = ($101>>>0)<($4>>>0);
     if ($102) {
      _abort();
      // unreachable;
     }
     $103 = ((($101)) + 12|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = ($104|0)==($3|0);
     if (!($105)) {
      _abort();
      // unreachable;
     }
     $106 = ((($98)) + 8|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = ($107|0)==($3|0);
     if ($108) {
      HEAP32[$103>>2] = $98;
      HEAP32[$106>>2] = $101;
      $R$1 = $98;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $122 = ($96|0)==(0|0);
   if (!($122)) {
    $$sum11 = (($2) + 28)|0;
    $123 = (($p) + ($$sum11)|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = (9828 + ($124<<2)|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ($3|0)==($126|0);
    if ($127) {
     HEAP32[$125>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $128 = 1 << $124;
      $129 = $128 ^ -1;
      $130 = HEAP32[(9528)>>2]|0;
      $131 = $130 & $129;
      HEAP32[(9528)>>2] = $131;
      break;
     }
    } else {
     $132 = HEAP32[(9540)>>2]|0;
     $133 = ($96>>>0)<($132>>>0);
     if ($133) {
      _abort();
      // unreachable;
     }
     $134 = ((($96)) + 16|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = ($135|0)==($3|0);
     if ($136) {
      HEAP32[$134>>2] = $R$1;
     } else {
      $137 = ((($96)) + 20|0);
      HEAP32[$137>>2] = $R$1;
     }
     $138 = ($R$1|0)==(0|0);
     if ($138) {
      break;
     }
    }
    $139 = HEAP32[(9540)>>2]|0;
    $140 = ($R$1>>>0)<($139>>>0);
    if ($140) {
     _abort();
     // unreachable;
    }
    $141 = ((($R$1)) + 24|0);
    HEAP32[$141>>2] = $96;
    $$sum12 = (($2) + 16)|0;
    $142 = (($p) + ($$sum12)|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = ($143|0)==(0|0);
    do {
     if (!($144)) {
      $145 = ($143>>>0)<($139>>>0);
      if ($145) {
       _abort();
       // unreachable;
      } else {
       $146 = ((($R$1)) + 16|0);
       HEAP32[$146>>2] = $143;
       $147 = ((($143)) + 24|0);
       HEAP32[$147>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum13 = (($2) + 20)|0;
    $148 = (($p) + ($$sum13)|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = ($149|0)==(0|0);
    if (!($150)) {
     $151 = HEAP32[(9540)>>2]|0;
     $152 = ($149>>>0)<($151>>>0);
     if ($152) {
      _abort();
      // unreachable;
     } else {
      $153 = ((($R$1)) + 20|0);
      HEAP32[$153>>2] = $149;
      $154 = ((($149)) + 24|0);
      HEAP32[$154>>2] = $R$1;
      break;
     }
    }
   }
  }
 } while(0);
 $155 = ($70>>>0)<(16);
 if ($155) {
  $156 = $1 & 1;
  $157 = $68 | $156;
  $158 = $157 | 2;
  HEAP32[$0>>2] = $158;
  $$sum910 = $68 | 4;
  $159 = (($p) + ($$sum910)|0);
  $160 = HEAP32[$159>>2]|0;
  $161 = $160 | 1;
  HEAP32[$159>>2] = $161;
  $newp$0 = $p;
  return ($newp$0|0);
 } else {
  $162 = (($p) + ($nb)|0);
  $163 = $1 & 1;
  $164 = $163 | $nb;
  $165 = $164 | 2;
  HEAP32[$0>>2] = $165;
  $$sum5 = (($nb) + 4)|0;
  $166 = (($p) + ($$sum5)|0);
  $167 = $70 | 3;
  HEAP32[$166>>2] = $167;
  $$sum78 = $68 | 4;
  $168 = (($p) + ($$sum78)|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = $169 | 1;
  HEAP32[$168>>2] = $170;
  _dispose_chunk($162,$70);
  $newp$0 = $p;
  return ($newp$0|0);
 }
 return (0)|0;
}
function _dispose_chunk($p,$psize) {
 $p = $p|0;
 $psize = $psize|0;
 var $$0 = 0, $$02 = 0, $$1 = 0, $$lcssa = 0, $$pre = 0, $$pre$phi50Z2D = 0, $$pre$phi52Z2D = 0, $$pre$phiZ2D = 0, $$pre48 = 0, $$pre49 = 0, $$pre51 = 0, $$sum = 0, $$sum1 = 0, $$sum10 = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum16 = 0, $$sum17 = 0;
 var $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum21 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0, $$sum25 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum7 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0;
 var $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0;
 var $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0;
 var $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0;
 var $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0;
 var $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0;
 var $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0;
 var $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0;
 var $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0;
 var $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0;
 var $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0;
 var $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0;
 var $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I19$0 = 0, $K20$043 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0, $R7$0 = 0, $R7$0$lcssa = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $RP9$0 = 0, $RP9$0$lcssa = 0, $T$0$lcssa = 0, $T$042 = 0, $T$042$lcssa = 0, $cond = 0;
 var $cond39 = 0, $not$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + ($psize)|0);
 $1 = ((($p)) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = HEAP32[$p>>2]|0;
   $6 = $2 & 3;
   $7 = ($6|0)==(0);
   if ($7) {
    return;
   }
   $8 = (0 - ($5))|0;
   $9 = (($p) + ($8)|0);
   $10 = (($5) + ($psize))|0;
   $11 = HEAP32[(9540)>>2]|0;
   $12 = ($9>>>0)<($11>>>0);
   if ($12) {
    _abort();
    // unreachable;
   }
   $13 = HEAP32[(9544)>>2]|0;
   $14 = ($9|0)==($13|0);
   if ($14) {
    $$sum = (($psize) + 4)|0;
    $99 = (($p) + ($$sum)|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $100 & 3;
    $102 = ($101|0)==(3);
    if (!($102)) {
     $$0 = $9;$$02 = $10;
     break;
    }
    HEAP32[(9532)>>2] = $10;
    $103 = $100 & -2;
    HEAP32[$99>>2] = $103;
    $104 = $10 | 1;
    $$sum14 = (4 - ($5))|0;
    $105 = (($p) + ($$sum14)|0);
    HEAP32[$105>>2] = $104;
    HEAP32[$0>>2] = $10;
    return;
   }
   $15 = $5 >>> 3;
   $16 = ($5>>>0)<(256);
   if ($16) {
    $$sum24 = (8 - ($5))|0;
    $17 = (($p) + ($$sum24)|0);
    $18 = HEAP32[$17>>2]|0;
    $$sum25 = (12 - ($5))|0;
    $19 = (($p) + ($$sum25)|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $15 << 1;
    $22 = (9564 + ($21<<2)|0);
    $23 = ($18|0)==($22|0);
    if (!($23)) {
     $24 = ($18>>>0)<($11>>>0);
     if ($24) {
      _abort();
      // unreachable;
     }
     $25 = ((($18)) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==($9|0);
     if (!($27)) {
      _abort();
      // unreachable;
     }
    }
    $28 = ($20|0)==($18|0);
    if ($28) {
     $29 = 1 << $15;
     $30 = $29 ^ -1;
     $31 = HEAP32[9524>>2]|0;
     $32 = $31 & $30;
     HEAP32[9524>>2] = $32;
     $$0 = $9;$$02 = $10;
     break;
    }
    $33 = ($20|0)==($22|0);
    if ($33) {
     $$pre51 = ((($20)) + 8|0);
     $$pre$phi52Z2D = $$pre51;
    } else {
     $34 = ($20>>>0)<($11>>>0);
     if ($34) {
      _abort();
      // unreachable;
     }
     $35 = ((($20)) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)==($9|0);
     if ($37) {
      $$pre$phi52Z2D = $35;
     } else {
      _abort();
      // unreachable;
     }
    }
    $38 = ((($18)) + 12|0);
    HEAP32[$38>>2] = $20;
    HEAP32[$$pre$phi52Z2D>>2] = $18;
    $$0 = $9;$$02 = $10;
    break;
   }
   $$sum16 = (24 - ($5))|0;
   $39 = (($p) + ($$sum16)|0);
   $40 = HEAP32[$39>>2]|0;
   $$sum17 = (12 - ($5))|0;
   $41 = (($p) + ($$sum17)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==($9|0);
   do {
    if ($43) {
     $$sum18 = (16 - ($5))|0;
     $$sum19 = (($$sum18) + 4)|0;
     $53 = (($p) + ($$sum19)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)==(0|0);
     if ($55) {
      $56 = (($p) + ($$sum18)|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==(0|0);
      if ($58) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $57;$RP$0 = $56;
      }
     } else {
      $R$0 = $54;$RP$0 = $53;
     }
     while(1) {
      $59 = ((($R$0)) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==(0|0);
      if (!($61)) {
       $R$0 = $60;$RP$0 = $59;
       continue;
      }
      $62 = ((($R$0)) + 16|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $63;$RP$0 = $62;
      }
     }
     $65 = ($RP$0$lcssa>>>0)<($11>>>0);
     if ($65) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum23 = (8 - ($5))|0;
     $44 = (($p) + ($$sum23)|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($45>>>0)<($11>>>0);
     if ($46) {
      _abort();
      // unreachable;
     }
     $47 = ((($45)) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)==($9|0);
     if (!($49)) {
      _abort();
      // unreachable;
     }
     $50 = ((($42)) + 8|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)==($9|0);
     if ($52) {
      HEAP32[$47>>2] = $42;
      HEAP32[$50>>2] = $45;
      $R$1 = $42;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $66 = ($40|0)==(0|0);
   if ($66) {
    $$0 = $9;$$02 = $10;
   } else {
    $$sum20 = (28 - ($5))|0;
    $67 = (($p) + ($$sum20)|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (9828 + ($68<<2)|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($9|0)==($70|0);
    if ($71) {
     HEAP32[$69>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $72 = 1 << $68;
      $73 = $72 ^ -1;
      $74 = HEAP32[(9528)>>2]|0;
      $75 = $74 & $73;
      HEAP32[(9528)>>2] = $75;
      $$0 = $9;$$02 = $10;
      break;
     }
    } else {
     $76 = HEAP32[(9540)>>2]|0;
     $77 = ($40>>>0)<($76>>>0);
     if ($77) {
      _abort();
      // unreachable;
     }
     $78 = ((($40)) + 16|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)==($9|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
     } else {
      $81 = ((($40)) + 20|0);
      HEAP32[$81>>2] = $R$1;
     }
     $82 = ($R$1|0)==(0|0);
     if ($82) {
      $$0 = $9;$$02 = $10;
      break;
     }
    }
    $83 = HEAP32[(9540)>>2]|0;
    $84 = ($R$1>>>0)<($83>>>0);
    if ($84) {
     _abort();
     // unreachable;
    }
    $85 = ((($R$1)) + 24|0);
    HEAP32[$85>>2] = $40;
    $$sum21 = (16 - ($5))|0;
    $86 = (($p) + ($$sum21)|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($87|0)==(0|0);
    do {
     if (!($88)) {
      $89 = ($87>>>0)<($83>>>0);
      if ($89) {
       _abort();
       // unreachable;
      } else {
       $90 = ((($R$1)) + 16|0);
       HEAP32[$90>>2] = $87;
       $91 = ((($87)) + 24|0);
       HEAP32[$91>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum22 = (($$sum21) + 4)|0;
    $92 = (($p) + ($$sum22)|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = ($93|0)==(0|0);
    if ($94) {
     $$0 = $9;$$02 = $10;
    } else {
     $95 = HEAP32[(9540)>>2]|0;
     $96 = ($93>>>0)<($95>>>0);
     if ($96) {
      _abort();
      // unreachable;
     } else {
      $97 = ((($R$1)) + 20|0);
      HEAP32[$97>>2] = $93;
      $98 = ((($93)) + 24|0);
      HEAP32[$98>>2] = $R$1;
      $$0 = $9;$$02 = $10;
      break;
     }
    }
   }
  } else {
   $$0 = $p;$$02 = $psize;
  }
 } while(0);
 $106 = HEAP32[(9540)>>2]|0;
 $107 = ($0>>>0)<($106>>>0);
 if ($107) {
  _abort();
  // unreachable;
 }
 $$sum1 = (($psize) + 4)|0;
 $108 = (($p) + ($$sum1)|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = $109 & 2;
 $111 = ($110|0)==(0);
 if ($111) {
  $112 = HEAP32[(9548)>>2]|0;
  $113 = ($0|0)==($112|0);
  if ($113) {
   $114 = HEAP32[(9536)>>2]|0;
   $115 = (($114) + ($$02))|0;
   HEAP32[(9536)>>2] = $115;
   HEAP32[(9548)>>2] = $$0;
   $116 = $115 | 1;
   $117 = ((($$0)) + 4|0);
   HEAP32[$117>>2] = $116;
   $118 = HEAP32[(9544)>>2]|0;
   $119 = ($$0|0)==($118|0);
   if (!($119)) {
    return;
   }
   HEAP32[(9544)>>2] = 0;
   HEAP32[(9532)>>2] = 0;
   return;
  }
  $120 = HEAP32[(9544)>>2]|0;
  $121 = ($0|0)==($120|0);
  if ($121) {
   $122 = HEAP32[(9532)>>2]|0;
   $123 = (($122) + ($$02))|0;
   HEAP32[(9532)>>2] = $123;
   HEAP32[(9544)>>2] = $$0;
   $124 = $123 | 1;
   $125 = ((($$0)) + 4|0);
   HEAP32[$125>>2] = $124;
   $126 = (($$0) + ($123)|0);
   HEAP32[$126>>2] = $123;
   return;
  }
  $127 = $109 & -8;
  $128 = (($127) + ($$02))|0;
  $129 = $109 >>> 3;
  $130 = ($109>>>0)<(256);
  do {
   if ($130) {
    $$sum12 = (($psize) + 8)|0;
    $131 = (($p) + ($$sum12)|0);
    $132 = HEAP32[$131>>2]|0;
    $$sum13 = (($psize) + 12)|0;
    $133 = (($p) + ($$sum13)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $129 << 1;
    $136 = (9564 + ($135<<2)|0);
    $137 = ($132|0)==($136|0);
    if (!($137)) {
     $138 = ($132>>>0)<($106>>>0);
     if ($138) {
      _abort();
      // unreachable;
     }
     $139 = ((($132)) + 12|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = ($140|0)==($0|0);
     if (!($141)) {
      _abort();
      // unreachable;
     }
    }
    $142 = ($134|0)==($132|0);
    if ($142) {
     $143 = 1 << $129;
     $144 = $143 ^ -1;
     $145 = HEAP32[9524>>2]|0;
     $146 = $145 & $144;
     HEAP32[9524>>2] = $146;
     break;
    }
    $147 = ($134|0)==($136|0);
    if ($147) {
     $$pre49 = ((($134)) + 8|0);
     $$pre$phi50Z2D = $$pre49;
    } else {
     $148 = ($134>>>0)<($106>>>0);
     if ($148) {
      _abort();
      // unreachable;
     }
     $149 = ((($134)) + 8|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = ($150|0)==($0|0);
     if ($151) {
      $$pre$phi50Z2D = $149;
     } else {
      _abort();
      // unreachable;
     }
    }
    $152 = ((($132)) + 12|0);
    HEAP32[$152>>2] = $134;
    HEAP32[$$pre$phi50Z2D>>2] = $132;
   } else {
    $$sum2 = (($psize) + 24)|0;
    $153 = (($p) + ($$sum2)|0);
    $154 = HEAP32[$153>>2]|0;
    $$sum3 = (($psize) + 12)|0;
    $155 = (($p) + ($$sum3)|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = ($156|0)==($0|0);
    do {
     if ($157) {
      $$sum5 = (($psize) + 20)|0;
      $167 = (($p) + ($$sum5)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = ($168|0)==(0|0);
      if ($169) {
       $$sum4 = (($psize) + 16)|0;
       $170 = (($p) + ($$sum4)|0);
       $171 = HEAP32[$170>>2]|0;
       $172 = ($171|0)==(0|0);
       if ($172) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $171;$RP9$0 = $170;
       }
      } else {
       $R7$0 = $168;$RP9$0 = $167;
      }
      while(1) {
       $173 = ((($R7$0)) + 20|0);
       $174 = HEAP32[$173>>2]|0;
       $175 = ($174|0)==(0|0);
       if (!($175)) {
        $R7$0 = $174;$RP9$0 = $173;
        continue;
       }
       $176 = ((($R7$0)) + 16|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = ($177|0)==(0|0);
       if ($178) {
        $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
        break;
       } else {
        $R7$0 = $177;$RP9$0 = $176;
       }
      }
      $179 = ($RP9$0$lcssa>>>0)<($106>>>0);
      if ($179) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0$lcssa>>2] = 0;
       $R7$1 = $R7$0$lcssa;
       break;
      }
     } else {
      $$sum11 = (($psize) + 8)|0;
      $158 = (($p) + ($$sum11)|0);
      $159 = HEAP32[$158>>2]|0;
      $160 = ($159>>>0)<($106>>>0);
      if ($160) {
       _abort();
       // unreachable;
      }
      $161 = ((($159)) + 12|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ($162|0)==($0|0);
      if (!($163)) {
       _abort();
       // unreachable;
      }
      $164 = ((($156)) + 8|0);
      $165 = HEAP32[$164>>2]|0;
      $166 = ($165|0)==($0|0);
      if ($166) {
       HEAP32[$161>>2] = $156;
       HEAP32[$164>>2] = $159;
       $R7$1 = $156;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $180 = ($154|0)==(0|0);
    if (!($180)) {
     $$sum8 = (($psize) + 28)|0;
     $181 = (($p) + ($$sum8)|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = (9828 + ($182<<2)|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = ($0|0)==($184|0);
     if ($185) {
      HEAP32[$183>>2] = $R7$1;
      $cond39 = ($R7$1|0)==(0|0);
      if ($cond39) {
       $186 = 1 << $182;
       $187 = $186 ^ -1;
       $188 = HEAP32[(9528)>>2]|0;
       $189 = $188 & $187;
       HEAP32[(9528)>>2] = $189;
       break;
      }
     } else {
      $190 = HEAP32[(9540)>>2]|0;
      $191 = ($154>>>0)<($190>>>0);
      if ($191) {
       _abort();
       // unreachable;
      }
      $192 = ((($154)) + 16|0);
      $193 = HEAP32[$192>>2]|0;
      $194 = ($193|0)==($0|0);
      if ($194) {
       HEAP32[$192>>2] = $R7$1;
      } else {
       $195 = ((($154)) + 20|0);
       HEAP32[$195>>2] = $R7$1;
      }
      $196 = ($R7$1|0)==(0|0);
      if ($196) {
       break;
      }
     }
     $197 = HEAP32[(9540)>>2]|0;
     $198 = ($R7$1>>>0)<($197>>>0);
     if ($198) {
      _abort();
      // unreachable;
     }
     $199 = ((($R7$1)) + 24|0);
     HEAP32[$199>>2] = $154;
     $$sum9 = (($psize) + 16)|0;
     $200 = (($p) + ($$sum9)|0);
     $201 = HEAP32[$200>>2]|0;
     $202 = ($201|0)==(0|0);
     do {
      if (!($202)) {
       $203 = ($201>>>0)<($197>>>0);
       if ($203) {
        _abort();
        // unreachable;
       } else {
        $204 = ((($R7$1)) + 16|0);
        HEAP32[$204>>2] = $201;
        $205 = ((($201)) + 24|0);
        HEAP32[$205>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum10 = (($psize) + 20)|0;
     $206 = (($p) + ($$sum10)|0);
     $207 = HEAP32[$206>>2]|0;
     $208 = ($207|0)==(0|0);
     if (!($208)) {
      $209 = HEAP32[(9540)>>2]|0;
      $210 = ($207>>>0)<($209>>>0);
      if ($210) {
       _abort();
       // unreachable;
      } else {
       $211 = ((($R7$1)) + 20|0);
       HEAP32[$211>>2] = $207;
       $212 = ((($207)) + 24|0);
       HEAP32[$212>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $213 = $128 | 1;
  $214 = ((($$0)) + 4|0);
  HEAP32[$214>>2] = $213;
  $215 = (($$0) + ($128)|0);
  HEAP32[$215>>2] = $128;
  $216 = HEAP32[(9544)>>2]|0;
  $217 = ($$0|0)==($216|0);
  if ($217) {
   HEAP32[(9532)>>2] = $128;
   return;
  } else {
   $$1 = $128;
  }
 } else {
  $218 = $109 & -2;
  HEAP32[$108>>2] = $218;
  $219 = $$02 | 1;
  $220 = ((($$0)) + 4|0);
  HEAP32[$220>>2] = $219;
  $221 = (($$0) + ($$02)|0);
  HEAP32[$221>>2] = $$02;
  $$1 = $$02;
 }
 $222 = $$1 >>> 3;
 $223 = ($$1>>>0)<(256);
 if ($223) {
  $224 = $222 << 1;
  $225 = (9564 + ($224<<2)|0);
  $226 = HEAP32[9524>>2]|0;
  $227 = 1 << $222;
  $228 = $226 & $227;
  $229 = ($228|0)==(0);
  if ($229) {
   $230 = $226 | $227;
   HEAP32[9524>>2] = $230;
   $$pre = (($224) + 2)|0;
   $$pre48 = (9564 + ($$pre<<2)|0);
   $$pre$phiZ2D = $$pre48;$F16$0 = $225;
  } else {
   $$sum7 = (($224) + 2)|0;
   $231 = (9564 + ($$sum7<<2)|0);
   $232 = HEAP32[$231>>2]|0;
   $233 = HEAP32[(9540)>>2]|0;
   $234 = ($232>>>0)<($233>>>0);
   if ($234) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $231;$F16$0 = $232;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $$0;
  $235 = ((($F16$0)) + 12|0);
  HEAP32[$235>>2] = $$0;
  $236 = ((($$0)) + 8|0);
  HEAP32[$236>>2] = $F16$0;
  $237 = ((($$0)) + 12|0);
  HEAP32[$237>>2] = $225;
  return;
 }
 $238 = $$1 >>> 8;
 $239 = ($238|0)==(0);
 if ($239) {
  $I19$0 = 0;
 } else {
  $240 = ($$1>>>0)>(16777215);
  if ($240) {
   $I19$0 = 31;
  } else {
   $241 = (($238) + 1048320)|0;
   $242 = $241 >>> 16;
   $243 = $242 & 8;
   $244 = $238 << $243;
   $245 = (($244) + 520192)|0;
   $246 = $245 >>> 16;
   $247 = $246 & 4;
   $248 = $247 | $243;
   $249 = $244 << $247;
   $250 = (($249) + 245760)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 2;
   $253 = $248 | $252;
   $254 = (14 - ($253))|0;
   $255 = $249 << $252;
   $256 = $255 >>> 15;
   $257 = (($254) + ($256))|0;
   $258 = $257 << 1;
   $259 = (($257) + 7)|0;
   $260 = $$1 >>> $259;
   $261 = $260 & 1;
   $262 = $261 | $258;
   $I19$0 = $262;
  }
 }
 $263 = (9828 + ($I19$0<<2)|0);
 $264 = ((($$0)) + 28|0);
 HEAP32[$264>>2] = $I19$0;
 $265 = ((($$0)) + 16|0);
 $266 = ((($$0)) + 20|0);
 HEAP32[$266>>2] = 0;
 HEAP32[$265>>2] = 0;
 $267 = HEAP32[(9528)>>2]|0;
 $268 = 1 << $I19$0;
 $269 = $267 & $268;
 $270 = ($269|0)==(0);
 if ($270) {
  $271 = $267 | $268;
  HEAP32[(9528)>>2] = $271;
  HEAP32[$263>>2] = $$0;
  $272 = ((($$0)) + 24|0);
  HEAP32[$272>>2] = $263;
  $273 = ((($$0)) + 12|0);
  HEAP32[$273>>2] = $$0;
  $274 = ((($$0)) + 8|0);
  HEAP32[$274>>2] = $$0;
  return;
 }
 $275 = HEAP32[$263>>2]|0;
 $276 = ((($275)) + 4|0);
 $277 = HEAP32[$276>>2]|0;
 $278 = $277 & -8;
 $279 = ($278|0)==($$1|0);
 L191: do {
  if ($279) {
   $T$0$lcssa = $275;
  } else {
   $280 = ($I19$0|0)==(31);
   $281 = $I19$0 >>> 1;
   $282 = (25 - ($281))|0;
   $283 = $280 ? 0 : $282;
   $284 = $$1 << $283;
   $K20$043 = $284;$T$042 = $275;
   while(1) {
    $291 = $K20$043 >>> 31;
    $292 = (((($T$042)) + 16|0) + ($291<<2)|0);
    $287 = HEAP32[$292>>2]|0;
    $293 = ($287|0)==(0|0);
    if ($293) {
     $$lcssa = $292;$T$042$lcssa = $T$042;
     break;
    }
    $285 = $K20$043 << 1;
    $286 = ((($287)) + 4|0);
    $288 = HEAP32[$286>>2]|0;
    $289 = $288 & -8;
    $290 = ($289|0)==($$1|0);
    if ($290) {
     $T$0$lcssa = $287;
     break L191;
    } else {
     $K20$043 = $285;$T$042 = $287;
    }
   }
   $294 = HEAP32[(9540)>>2]|0;
   $295 = ($$lcssa>>>0)<($294>>>0);
   if ($295) {
    _abort();
    // unreachable;
   }
   HEAP32[$$lcssa>>2] = $$0;
   $296 = ((($$0)) + 24|0);
   HEAP32[$296>>2] = $T$042$lcssa;
   $297 = ((($$0)) + 12|0);
   HEAP32[$297>>2] = $$0;
   $298 = ((($$0)) + 8|0);
   HEAP32[$298>>2] = $$0;
   return;
  }
 } while(0);
 $299 = ((($T$0$lcssa)) + 8|0);
 $300 = HEAP32[$299>>2]|0;
 $301 = HEAP32[(9540)>>2]|0;
 $302 = ($300>>>0)>=($301>>>0);
 $not$ = ($T$0$lcssa>>>0)>=($301>>>0);
 $303 = $302 & $not$;
 if (!($303)) {
  _abort();
  // unreachable;
 }
 $304 = ((($300)) + 12|0);
 HEAP32[$304>>2] = $$0;
 HEAP32[$299>>2] = $$0;
 $305 = ((($$0)) + 8|0);
 HEAP32[$305>>2] = $300;
 $306 = ((($$0)) + 12|0);
 HEAP32[$306>>2] = $T$0$lcssa;
 $307 = ((($$0)) + 24|0);
 HEAP32[$307>>2] = 0;
 return;
}
function runPostSets() {
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _memcpy(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _memmove(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if (((src|0) < (dest|0)) & ((dest|0) < ((src + num)|0))) {
      // Unlikely case: Copy backwards in a safe manner
      ret = dest;
      src = (src + num)|0;
      dest = (dest + num)|0;
      while ((num|0) > 0) {
        dest = (dest - 1)|0;
        src = (src - 1)|0;
        num = (num - 1)|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      }
      dest = ret;
    } else {
      _memcpy(dest, src, num) | 0;
    }
    return dest | 0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return $10$0 | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 16 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return $1$0 | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 16 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (Math_clz32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0 | 0, $d_sroa_0_0_insert_insert99$1 | 0, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



  
function dynCall_iiii(index,a1,a2,a3) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0;
  return FUNCTION_TABLE_iiii[index&255](a1|0,a2|0,a3|0)|0;
}


function jsCall_iiii_0(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(0,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_1(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(1,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_2(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(2,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_3(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(3,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_4(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(4,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_5(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(5,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_6(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(6,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_7(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(7,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_8(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(8,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_9(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(9,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_10(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(10,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_11(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(11,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_12(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(12,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_13(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(13,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_14(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(14,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_15(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(15,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_16(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(16,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_17(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(17,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_18(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(18,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_19(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(19,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_20(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(20,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_21(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(21,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_22(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(22,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_23(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(23,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_24(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(24,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_25(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(25,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_26(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(26,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_27(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(27,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_28(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(28,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_29(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(29,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_30(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(30,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_31(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(31,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_32(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(32,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_33(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(33,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_34(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(34,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_35(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(35,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_36(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(36,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_37(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(37,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_38(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(38,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_39(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(39,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_40(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(40,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_41(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(41,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_42(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(42,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_43(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(43,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_44(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(44,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_45(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(45,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_46(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(46,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_47(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(47,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_48(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(48,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_49(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(49,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_50(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(50,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_51(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(51,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_52(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(52,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_53(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(53,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_54(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(54,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_55(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(55,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_56(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(56,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_57(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(57,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_58(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(58,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_59(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(59,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_60(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(60,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_61(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(61,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_62(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(62,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_63(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(63,a1|0,a2|0,a3|0)|0;
}



function dynCall_i(index) {
  index = index|0;
  
  return FUNCTION_TABLE_i[index&255]()|0;
}


function jsCall_i_0() {
  
  return jsCall_i(0)|0;
}



function jsCall_i_1() {
  
  return jsCall_i(1)|0;
}



function jsCall_i_2() {
  
  return jsCall_i(2)|0;
}



function jsCall_i_3() {
  
  return jsCall_i(3)|0;
}



function jsCall_i_4() {
  
  return jsCall_i(4)|0;
}



function jsCall_i_5() {
  
  return jsCall_i(5)|0;
}



function jsCall_i_6() {
  
  return jsCall_i(6)|0;
}



function jsCall_i_7() {
  
  return jsCall_i(7)|0;
}



function jsCall_i_8() {
  
  return jsCall_i(8)|0;
}



function jsCall_i_9() {
  
  return jsCall_i(9)|0;
}



function jsCall_i_10() {
  
  return jsCall_i(10)|0;
}



function jsCall_i_11() {
  
  return jsCall_i(11)|0;
}



function jsCall_i_12() {
  
  return jsCall_i(12)|0;
}



function jsCall_i_13() {
  
  return jsCall_i(13)|0;
}



function jsCall_i_14() {
  
  return jsCall_i(14)|0;
}



function jsCall_i_15() {
  
  return jsCall_i(15)|0;
}



function jsCall_i_16() {
  
  return jsCall_i(16)|0;
}



function jsCall_i_17() {
  
  return jsCall_i(17)|0;
}



function jsCall_i_18() {
  
  return jsCall_i(18)|0;
}



function jsCall_i_19() {
  
  return jsCall_i(19)|0;
}



function jsCall_i_20() {
  
  return jsCall_i(20)|0;
}



function jsCall_i_21() {
  
  return jsCall_i(21)|0;
}



function jsCall_i_22() {
  
  return jsCall_i(22)|0;
}



function jsCall_i_23() {
  
  return jsCall_i(23)|0;
}



function jsCall_i_24() {
  
  return jsCall_i(24)|0;
}



function jsCall_i_25() {
  
  return jsCall_i(25)|0;
}



function jsCall_i_26() {
  
  return jsCall_i(26)|0;
}



function jsCall_i_27() {
  
  return jsCall_i(27)|0;
}



function jsCall_i_28() {
  
  return jsCall_i(28)|0;
}



function jsCall_i_29() {
  
  return jsCall_i(29)|0;
}



function jsCall_i_30() {
  
  return jsCall_i(30)|0;
}



function jsCall_i_31() {
  
  return jsCall_i(31)|0;
}



function jsCall_i_32() {
  
  return jsCall_i(32)|0;
}



function jsCall_i_33() {
  
  return jsCall_i(33)|0;
}



function jsCall_i_34() {
  
  return jsCall_i(34)|0;
}



function jsCall_i_35() {
  
  return jsCall_i(35)|0;
}



function jsCall_i_36() {
  
  return jsCall_i(36)|0;
}



function jsCall_i_37() {
  
  return jsCall_i(37)|0;
}



function jsCall_i_38() {
  
  return jsCall_i(38)|0;
}



function jsCall_i_39() {
  
  return jsCall_i(39)|0;
}



function jsCall_i_40() {
  
  return jsCall_i(40)|0;
}



function jsCall_i_41() {
  
  return jsCall_i(41)|0;
}



function jsCall_i_42() {
  
  return jsCall_i(42)|0;
}



function jsCall_i_43() {
  
  return jsCall_i(43)|0;
}



function jsCall_i_44() {
  
  return jsCall_i(44)|0;
}



function jsCall_i_45() {
  
  return jsCall_i(45)|0;
}



function jsCall_i_46() {
  
  return jsCall_i(46)|0;
}



function jsCall_i_47() {
  
  return jsCall_i(47)|0;
}



function jsCall_i_48() {
  
  return jsCall_i(48)|0;
}



function jsCall_i_49() {
  
  return jsCall_i(49)|0;
}



function jsCall_i_50() {
  
  return jsCall_i(50)|0;
}



function jsCall_i_51() {
  
  return jsCall_i(51)|0;
}



function jsCall_i_52() {
  
  return jsCall_i(52)|0;
}



function jsCall_i_53() {
  
  return jsCall_i(53)|0;
}



function jsCall_i_54() {
  
  return jsCall_i(54)|0;
}



function jsCall_i_55() {
  
  return jsCall_i(55)|0;
}



function jsCall_i_56() {
  
  return jsCall_i(56)|0;
}



function jsCall_i_57() {
  
  return jsCall_i(57)|0;
}



function jsCall_i_58() {
  
  return jsCall_i(58)|0;
}



function jsCall_i_59() {
  
  return jsCall_i(59)|0;
}



function jsCall_i_60() {
  
  return jsCall_i(60)|0;
}



function jsCall_i_61() {
  
  return jsCall_i(61)|0;
}



function jsCall_i_62() {
  
  return jsCall_i(62)|0;
}



function jsCall_i_63() {
  
  return jsCall_i(63)|0;
}



function dynCall_vi(index,a1) {
  index = index|0;
  a1=a1|0;
  FUNCTION_TABLE_vi[index&255](a1|0);
}


function jsCall_vi_0(a1) {
  a1=a1|0;
  jsCall_vi(0,a1|0);
}



function jsCall_vi_1(a1) {
  a1=a1|0;
  jsCall_vi(1,a1|0);
}



function jsCall_vi_2(a1) {
  a1=a1|0;
  jsCall_vi(2,a1|0);
}



function jsCall_vi_3(a1) {
  a1=a1|0;
  jsCall_vi(3,a1|0);
}



function jsCall_vi_4(a1) {
  a1=a1|0;
  jsCall_vi(4,a1|0);
}



function jsCall_vi_5(a1) {
  a1=a1|0;
  jsCall_vi(5,a1|0);
}



function jsCall_vi_6(a1) {
  a1=a1|0;
  jsCall_vi(6,a1|0);
}



function jsCall_vi_7(a1) {
  a1=a1|0;
  jsCall_vi(7,a1|0);
}



function jsCall_vi_8(a1) {
  a1=a1|0;
  jsCall_vi(8,a1|0);
}



function jsCall_vi_9(a1) {
  a1=a1|0;
  jsCall_vi(9,a1|0);
}



function jsCall_vi_10(a1) {
  a1=a1|0;
  jsCall_vi(10,a1|0);
}



function jsCall_vi_11(a1) {
  a1=a1|0;
  jsCall_vi(11,a1|0);
}



function jsCall_vi_12(a1) {
  a1=a1|0;
  jsCall_vi(12,a1|0);
}



function jsCall_vi_13(a1) {
  a1=a1|0;
  jsCall_vi(13,a1|0);
}



function jsCall_vi_14(a1) {
  a1=a1|0;
  jsCall_vi(14,a1|0);
}



function jsCall_vi_15(a1) {
  a1=a1|0;
  jsCall_vi(15,a1|0);
}



function jsCall_vi_16(a1) {
  a1=a1|0;
  jsCall_vi(16,a1|0);
}



function jsCall_vi_17(a1) {
  a1=a1|0;
  jsCall_vi(17,a1|0);
}



function jsCall_vi_18(a1) {
  a1=a1|0;
  jsCall_vi(18,a1|0);
}



function jsCall_vi_19(a1) {
  a1=a1|0;
  jsCall_vi(19,a1|0);
}



function jsCall_vi_20(a1) {
  a1=a1|0;
  jsCall_vi(20,a1|0);
}



function jsCall_vi_21(a1) {
  a1=a1|0;
  jsCall_vi(21,a1|0);
}



function jsCall_vi_22(a1) {
  a1=a1|0;
  jsCall_vi(22,a1|0);
}



function jsCall_vi_23(a1) {
  a1=a1|0;
  jsCall_vi(23,a1|0);
}



function jsCall_vi_24(a1) {
  a1=a1|0;
  jsCall_vi(24,a1|0);
}



function jsCall_vi_25(a1) {
  a1=a1|0;
  jsCall_vi(25,a1|0);
}



function jsCall_vi_26(a1) {
  a1=a1|0;
  jsCall_vi(26,a1|0);
}



function jsCall_vi_27(a1) {
  a1=a1|0;
  jsCall_vi(27,a1|0);
}



function jsCall_vi_28(a1) {
  a1=a1|0;
  jsCall_vi(28,a1|0);
}



function jsCall_vi_29(a1) {
  a1=a1|0;
  jsCall_vi(29,a1|0);
}



function jsCall_vi_30(a1) {
  a1=a1|0;
  jsCall_vi(30,a1|0);
}



function jsCall_vi_31(a1) {
  a1=a1|0;
  jsCall_vi(31,a1|0);
}



function jsCall_vi_32(a1) {
  a1=a1|0;
  jsCall_vi(32,a1|0);
}



function jsCall_vi_33(a1) {
  a1=a1|0;
  jsCall_vi(33,a1|0);
}



function jsCall_vi_34(a1) {
  a1=a1|0;
  jsCall_vi(34,a1|0);
}



function jsCall_vi_35(a1) {
  a1=a1|0;
  jsCall_vi(35,a1|0);
}



function jsCall_vi_36(a1) {
  a1=a1|0;
  jsCall_vi(36,a1|0);
}



function jsCall_vi_37(a1) {
  a1=a1|0;
  jsCall_vi(37,a1|0);
}



function jsCall_vi_38(a1) {
  a1=a1|0;
  jsCall_vi(38,a1|0);
}



function jsCall_vi_39(a1) {
  a1=a1|0;
  jsCall_vi(39,a1|0);
}



function jsCall_vi_40(a1) {
  a1=a1|0;
  jsCall_vi(40,a1|0);
}



function jsCall_vi_41(a1) {
  a1=a1|0;
  jsCall_vi(41,a1|0);
}



function jsCall_vi_42(a1) {
  a1=a1|0;
  jsCall_vi(42,a1|0);
}



function jsCall_vi_43(a1) {
  a1=a1|0;
  jsCall_vi(43,a1|0);
}



function jsCall_vi_44(a1) {
  a1=a1|0;
  jsCall_vi(44,a1|0);
}



function jsCall_vi_45(a1) {
  a1=a1|0;
  jsCall_vi(45,a1|0);
}



function jsCall_vi_46(a1) {
  a1=a1|0;
  jsCall_vi(46,a1|0);
}



function jsCall_vi_47(a1) {
  a1=a1|0;
  jsCall_vi(47,a1|0);
}



function jsCall_vi_48(a1) {
  a1=a1|0;
  jsCall_vi(48,a1|0);
}



function jsCall_vi_49(a1) {
  a1=a1|0;
  jsCall_vi(49,a1|0);
}



function jsCall_vi_50(a1) {
  a1=a1|0;
  jsCall_vi(50,a1|0);
}



function jsCall_vi_51(a1) {
  a1=a1|0;
  jsCall_vi(51,a1|0);
}



function jsCall_vi_52(a1) {
  a1=a1|0;
  jsCall_vi(52,a1|0);
}



function jsCall_vi_53(a1) {
  a1=a1|0;
  jsCall_vi(53,a1|0);
}



function jsCall_vi_54(a1) {
  a1=a1|0;
  jsCall_vi(54,a1|0);
}



function jsCall_vi_55(a1) {
  a1=a1|0;
  jsCall_vi(55,a1|0);
}



function jsCall_vi_56(a1) {
  a1=a1|0;
  jsCall_vi(56,a1|0);
}



function jsCall_vi_57(a1) {
  a1=a1|0;
  jsCall_vi(57,a1|0);
}



function jsCall_vi_58(a1) {
  a1=a1|0;
  jsCall_vi(58,a1|0);
}



function jsCall_vi_59(a1) {
  a1=a1|0;
  jsCall_vi(59,a1|0);
}



function jsCall_vi_60(a1) {
  a1=a1|0;
  jsCall_vi(60,a1|0);
}



function jsCall_vi_61(a1) {
  a1=a1|0;
  jsCall_vi(61,a1|0);
}



function jsCall_vi_62(a1) {
  a1=a1|0;
  jsCall_vi(62,a1|0);
}



function jsCall_vi_63(a1) {
  a1=a1|0;
  jsCall_vi(63,a1|0);
}



function dynCall_vii(index,a1,a2) {
  index = index|0;
  a1=a1|0; a2=a2|0;
  FUNCTION_TABLE_vii[index&255](a1|0,a2|0);
}


function jsCall_vii_0(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(0,a1|0,a2|0);
}



function jsCall_vii_1(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(1,a1|0,a2|0);
}



function jsCall_vii_2(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(2,a1|0,a2|0);
}



function jsCall_vii_3(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(3,a1|0,a2|0);
}



function jsCall_vii_4(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(4,a1|0,a2|0);
}



function jsCall_vii_5(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(5,a1|0,a2|0);
}



function jsCall_vii_6(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(6,a1|0,a2|0);
}



function jsCall_vii_7(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(7,a1|0,a2|0);
}



function jsCall_vii_8(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(8,a1|0,a2|0);
}



function jsCall_vii_9(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(9,a1|0,a2|0);
}



function jsCall_vii_10(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(10,a1|0,a2|0);
}



function jsCall_vii_11(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(11,a1|0,a2|0);
}



function jsCall_vii_12(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(12,a1|0,a2|0);
}



function jsCall_vii_13(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(13,a1|0,a2|0);
}



function jsCall_vii_14(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(14,a1|0,a2|0);
}



function jsCall_vii_15(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(15,a1|0,a2|0);
}



function jsCall_vii_16(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(16,a1|0,a2|0);
}



function jsCall_vii_17(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(17,a1|0,a2|0);
}



function jsCall_vii_18(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(18,a1|0,a2|0);
}



function jsCall_vii_19(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(19,a1|0,a2|0);
}



function jsCall_vii_20(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(20,a1|0,a2|0);
}



function jsCall_vii_21(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(21,a1|0,a2|0);
}



function jsCall_vii_22(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(22,a1|0,a2|0);
}



function jsCall_vii_23(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(23,a1|0,a2|0);
}



function jsCall_vii_24(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(24,a1|0,a2|0);
}



function jsCall_vii_25(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(25,a1|0,a2|0);
}



function jsCall_vii_26(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(26,a1|0,a2|0);
}



function jsCall_vii_27(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(27,a1|0,a2|0);
}



function jsCall_vii_28(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(28,a1|0,a2|0);
}



function jsCall_vii_29(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(29,a1|0,a2|0);
}



function jsCall_vii_30(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(30,a1|0,a2|0);
}



function jsCall_vii_31(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(31,a1|0,a2|0);
}



function jsCall_vii_32(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(32,a1|0,a2|0);
}



function jsCall_vii_33(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(33,a1|0,a2|0);
}



function jsCall_vii_34(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(34,a1|0,a2|0);
}



function jsCall_vii_35(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(35,a1|0,a2|0);
}



function jsCall_vii_36(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(36,a1|0,a2|0);
}



function jsCall_vii_37(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(37,a1|0,a2|0);
}



function jsCall_vii_38(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(38,a1|0,a2|0);
}



function jsCall_vii_39(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(39,a1|0,a2|0);
}



function jsCall_vii_40(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(40,a1|0,a2|0);
}



function jsCall_vii_41(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(41,a1|0,a2|0);
}



function jsCall_vii_42(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(42,a1|0,a2|0);
}



function jsCall_vii_43(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(43,a1|0,a2|0);
}



function jsCall_vii_44(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(44,a1|0,a2|0);
}



function jsCall_vii_45(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(45,a1|0,a2|0);
}



function jsCall_vii_46(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(46,a1|0,a2|0);
}



function jsCall_vii_47(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(47,a1|0,a2|0);
}



function jsCall_vii_48(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(48,a1|0,a2|0);
}



function jsCall_vii_49(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(49,a1|0,a2|0);
}



function jsCall_vii_50(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(50,a1|0,a2|0);
}



function jsCall_vii_51(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(51,a1|0,a2|0);
}



function jsCall_vii_52(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(52,a1|0,a2|0);
}



function jsCall_vii_53(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(53,a1|0,a2|0);
}



function jsCall_vii_54(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(54,a1|0,a2|0);
}



function jsCall_vii_55(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(55,a1|0,a2|0);
}



function jsCall_vii_56(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(56,a1|0,a2|0);
}



function jsCall_vii_57(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(57,a1|0,a2|0);
}



function jsCall_vii_58(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(58,a1|0,a2|0);
}



function jsCall_vii_59(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(59,a1|0,a2|0);
}



function jsCall_vii_60(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(60,a1|0,a2|0);
}



function jsCall_vii_61(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(61,a1|0,a2|0);
}



function jsCall_vii_62(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(62,a1|0,a2|0);
}



function jsCall_vii_63(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(63,a1|0,a2|0);
}



function dynCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return FUNCTION_TABLE_iiiiiii[index&255](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}


function jsCall_iiiiiii_0(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(0,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_1(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(1,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_2(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(2,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_3(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(3,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_4(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(4,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_5(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(5,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_6(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(6,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_7(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(7,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_8(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(8,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_9(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(9,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_10(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(10,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_11(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(11,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_12(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(12,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_13(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(13,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_14(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(14,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_15(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(15,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_16(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(16,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_17(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(17,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_18(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(18,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_19(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(19,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_20(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(20,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_21(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(21,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_22(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(22,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_23(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(23,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_24(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(24,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_25(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(25,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_26(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(26,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_27(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(27,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_28(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(28,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_29(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(29,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_30(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(30,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_31(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(31,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_32(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(32,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_33(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(33,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_34(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(34,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_35(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(35,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_36(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(36,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_37(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(37,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_38(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(38,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_39(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(39,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_40(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(40,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_41(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(41,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_42(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(42,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_43(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(43,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_44(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(44,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_45(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(45,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_46(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(46,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_47(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(47,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_48(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(48,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_49(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(49,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_50(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(50,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_51(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(51,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_52(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(52,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_53(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(53,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_54(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(54,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_55(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(55,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_56(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(56,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_57(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(57,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_58(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(58,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_59(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(59,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_60(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(60,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_61(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(61,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_62(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(62,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_63(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(63,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function dynCall_ii(index,a1) {
  index = index|0;
  a1=a1|0;
  return FUNCTION_TABLE_ii[index&255](a1|0)|0;
}


function jsCall_ii_0(a1) {
  a1=a1|0;
  return jsCall_ii(0,a1|0)|0;
}



function jsCall_ii_1(a1) {
  a1=a1|0;
  return jsCall_ii(1,a1|0)|0;
}



function jsCall_ii_2(a1) {
  a1=a1|0;
  return jsCall_ii(2,a1|0)|0;
}



function jsCall_ii_3(a1) {
  a1=a1|0;
  return jsCall_ii(3,a1|0)|0;
}



function jsCall_ii_4(a1) {
  a1=a1|0;
  return jsCall_ii(4,a1|0)|0;
}



function jsCall_ii_5(a1) {
  a1=a1|0;
  return jsCall_ii(5,a1|0)|0;
}



function jsCall_ii_6(a1) {
  a1=a1|0;
  return jsCall_ii(6,a1|0)|0;
}



function jsCall_ii_7(a1) {
  a1=a1|0;
  return jsCall_ii(7,a1|0)|0;
}



function jsCall_ii_8(a1) {
  a1=a1|0;
  return jsCall_ii(8,a1|0)|0;
}



function jsCall_ii_9(a1) {
  a1=a1|0;
  return jsCall_ii(9,a1|0)|0;
}



function jsCall_ii_10(a1) {
  a1=a1|0;
  return jsCall_ii(10,a1|0)|0;
}



function jsCall_ii_11(a1) {
  a1=a1|0;
  return jsCall_ii(11,a1|0)|0;
}



function jsCall_ii_12(a1) {
  a1=a1|0;
  return jsCall_ii(12,a1|0)|0;
}



function jsCall_ii_13(a1) {
  a1=a1|0;
  return jsCall_ii(13,a1|0)|0;
}



function jsCall_ii_14(a1) {
  a1=a1|0;
  return jsCall_ii(14,a1|0)|0;
}



function jsCall_ii_15(a1) {
  a1=a1|0;
  return jsCall_ii(15,a1|0)|0;
}



function jsCall_ii_16(a1) {
  a1=a1|0;
  return jsCall_ii(16,a1|0)|0;
}



function jsCall_ii_17(a1) {
  a1=a1|0;
  return jsCall_ii(17,a1|0)|0;
}



function jsCall_ii_18(a1) {
  a1=a1|0;
  return jsCall_ii(18,a1|0)|0;
}



function jsCall_ii_19(a1) {
  a1=a1|0;
  return jsCall_ii(19,a1|0)|0;
}



function jsCall_ii_20(a1) {
  a1=a1|0;
  return jsCall_ii(20,a1|0)|0;
}



function jsCall_ii_21(a1) {
  a1=a1|0;
  return jsCall_ii(21,a1|0)|0;
}



function jsCall_ii_22(a1) {
  a1=a1|0;
  return jsCall_ii(22,a1|0)|0;
}



function jsCall_ii_23(a1) {
  a1=a1|0;
  return jsCall_ii(23,a1|0)|0;
}



function jsCall_ii_24(a1) {
  a1=a1|0;
  return jsCall_ii(24,a1|0)|0;
}



function jsCall_ii_25(a1) {
  a1=a1|0;
  return jsCall_ii(25,a1|0)|0;
}



function jsCall_ii_26(a1) {
  a1=a1|0;
  return jsCall_ii(26,a1|0)|0;
}



function jsCall_ii_27(a1) {
  a1=a1|0;
  return jsCall_ii(27,a1|0)|0;
}



function jsCall_ii_28(a1) {
  a1=a1|0;
  return jsCall_ii(28,a1|0)|0;
}



function jsCall_ii_29(a1) {
  a1=a1|0;
  return jsCall_ii(29,a1|0)|0;
}



function jsCall_ii_30(a1) {
  a1=a1|0;
  return jsCall_ii(30,a1|0)|0;
}



function jsCall_ii_31(a1) {
  a1=a1|0;
  return jsCall_ii(31,a1|0)|0;
}



function jsCall_ii_32(a1) {
  a1=a1|0;
  return jsCall_ii(32,a1|0)|0;
}



function jsCall_ii_33(a1) {
  a1=a1|0;
  return jsCall_ii(33,a1|0)|0;
}



function jsCall_ii_34(a1) {
  a1=a1|0;
  return jsCall_ii(34,a1|0)|0;
}



function jsCall_ii_35(a1) {
  a1=a1|0;
  return jsCall_ii(35,a1|0)|0;
}



function jsCall_ii_36(a1) {
  a1=a1|0;
  return jsCall_ii(36,a1|0)|0;
}



function jsCall_ii_37(a1) {
  a1=a1|0;
  return jsCall_ii(37,a1|0)|0;
}



function jsCall_ii_38(a1) {
  a1=a1|0;
  return jsCall_ii(38,a1|0)|0;
}



function jsCall_ii_39(a1) {
  a1=a1|0;
  return jsCall_ii(39,a1|0)|0;
}



function jsCall_ii_40(a1) {
  a1=a1|0;
  return jsCall_ii(40,a1|0)|0;
}



function jsCall_ii_41(a1) {
  a1=a1|0;
  return jsCall_ii(41,a1|0)|0;
}



function jsCall_ii_42(a1) {
  a1=a1|0;
  return jsCall_ii(42,a1|0)|0;
}



function jsCall_ii_43(a1) {
  a1=a1|0;
  return jsCall_ii(43,a1|0)|0;
}



function jsCall_ii_44(a1) {
  a1=a1|0;
  return jsCall_ii(44,a1|0)|0;
}



function jsCall_ii_45(a1) {
  a1=a1|0;
  return jsCall_ii(45,a1|0)|0;
}



function jsCall_ii_46(a1) {
  a1=a1|0;
  return jsCall_ii(46,a1|0)|0;
}



function jsCall_ii_47(a1) {
  a1=a1|0;
  return jsCall_ii(47,a1|0)|0;
}



function jsCall_ii_48(a1) {
  a1=a1|0;
  return jsCall_ii(48,a1|0)|0;
}



function jsCall_ii_49(a1) {
  a1=a1|0;
  return jsCall_ii(49,a1|0)|0;
}



function jsCall_ii_50(a1) {
  a1=a1|0;
  return jsCall_ii(50,a1|0)|0;
}



function jsCall_ii_51(a1) {
  a1=a1|0;
  return jsCall_ii(51,a1|0)|0;
}



function jsCall_ii_52(a1) {
  a1=a1|0;
  return jsCall_ii(52,a1|0)|0;
}



function jsCall_ii_53(a1) {
  a1=a1|0;
  return jsCall_ii(53,a1|0)|0;
}



function jsCall_ii_54(a1) {
  a1=a1|0;
  return jsCall_ii(54,a1|0)|0;
}



function jsCall_ii_55(a1) {
  a1=a1|0;
  return jsCall_ii(55,a1|0)|0;
}



function jsCall_ii_56(a1) {
  a1=a1|0;
  return jsCall_ii(56,a1|0)|0;
}



function jsCall_ii_57(a1) {
  a1=a1|0;
  return jsCall_ii(57,a1|0)|0;
}



function jsCall_ii_58(a1) {
  a1=a1|0;
  return jsCall_ii(58,a1|0)|0;
}



function jsCall_ii_59(a1) {
  a1=a1|0;
  return jsCall_ii(59,a1|0)|0;
}



function jsCall_ii_60(a1) {
  a1=a1|0;
  return jsCall_ii(60,a1|0)|0;
}



function jsCall_ii_61(a1) {
  a1=a1|0;
  return jsCall_ii(61,a1|0)|0;
}



function jsCall_ii_62(a1) {
  a1=a1|0;
  return jsCall_ii(62,a1|0)|0;
}



function jsCall_ii_63(a1) {
  a1=a1|0;
  return jsCall_ii(63,a1|0)|0;
}



function dynCall_viii(index,a1,a2,a3) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0;
  FUNCTION_TABLE_viii[index&255](a1|0,a2|0,a3|0);
}


function jsCall_viii_0(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(0,a1|0,a2|0,a3|0);
}



function jsCall_viii_1(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(1,a1|0,a2|0,a3|0);
}



function jsCall_viii_2(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(2,a1|0,a2|0,a3|0);
}



function jsCall_viii_3(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(3,a1|0,a2|0,a3|0);
}



function jsCall_viii_4(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(4,a1|0,a2|0,a3|0);
}



function jsCall_viii_5(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(5,a1|0,a2|0,a3|0);
}



function jsCall_viii_6(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(6,a1|0,a2|0,a3|0);
}



function jsCall_viii_7(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(7,a1|0,a2|0,a3|0);
}



function jsCall_viii_8(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(8,a1|0,a2|0,a3|0);
}



function jsCall_viii_9(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(9,a1|0,a2|0,a3|0);
}



function jsCall_viii_10(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(10,a1|0,a2|0,a3|0);
}



function jsCall_viii_11(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(11,a1|0,a2|0,a3|0);
}



function jsCall_viii_12(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(12,a1|0,a2|0,a3|0);
}



function jsCall_viii_13(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(13,a1|0,a2|0,a3|0);
}



function jsCall_viii_14(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(14,a1|0,a2|0,a3|0);
}



function jsCall_viii_15(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(15,a1|0,a2|0,a3|0);
}



function jsCall_viii_16(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(16,a1|0,a2|0,a3|0);
}



function jsCall_viii_17(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(17,a1|0,a2|0,a3|0);
}



function jsCall_viii_18(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(18,a1|0,a2|0,a3|0);
}



function jsCall_viii_19(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(19,a1|0,a2|0,a3|0);
}



function jsCall_viii_20(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(20,a1|0,a2|0,a3|0);
}



function jsCall_viii_21(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(21,a1|0,a2|0,a3|0);
}



function jsCall_viii_22(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(22,a1|0,a2|0,a3|0);
}



function jsCall_viii_23(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(23,a1|0,a2|0,a3|0);
}



function jsCall_viii_24(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(24,a1|0,a2|0,a3|0);
}



function jsCall_viii_25(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(25,a1|0,a2|0,a3|0);
}



function jsCall_viii_26(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(26,a1|0,a2|0,a3|0);
}



function jsCall_viii_27(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(27,a1|0,a2|0,a3|0);
}



function jsCall_viii_28(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(28,a1|0,a2|0,a3|0);
}



function jsCall_viii_29(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(29,a1|0,a2|0,a3|0);
}



function jsCall_viii_30(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(30,a1|0,a2|0,a3|0);
}



function jsCall_viii_31(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(31,a1|0,a2|0,a3|0);
}



function jsCall_viii_32(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(32,a1|0,a2|0,a3|0);
}



function jsCall_viii_33(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(33,a1|0,a2|0,a3|0);
}



function jsCall_viii_34(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(34,a1|0,a2|0,a3|0);
}



function jsCall_viii_35(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(35,a1|0,a2|0,a3|0);
}



function jsCall_viii_36(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(36,a1|0,a2|0,a3|0);
}



function jsCall_viii_37(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(37,a1|0,a2|0,a3|0);
}



function jsCall_viii_38(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(38,a1|0,a2|0,a3|0);
}



function jsCall_viii_39(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(39,a1|0,a2|0,a3|0);
}



function jsCall_viii_40(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(40,a1|0,a2|0,a3|0);
}



function jsCall_viii_41(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(41,a1|0,a2|0,a3|0);
}



function jsCall_viii_42(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(42,a1|0,a2|0,a3|0);
}



function jsCall_viii_43(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(43,a1|0,a2|0,a3|0);
}



function jsCall_viii_44(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(44,a1|0,a2|0,a3|0);
}



function jsCall_viii_45(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(45,a1|0,a2|0,a3|0);
}



function jsCall_viii_46(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(46,a1|0,a2|0,a3|0);
}



function jsCall_viii_47(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(47,a1|0,a2|0,a3|0);
}



function jsCall_viii_48(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(48,a1|0,a2|0,a3|0);
}



function jsCall_viii_49(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(49,a1|0,a2|0,a3|0);
}



function jsCall_viii_50(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(50,a1|0,a2|0,a3|0);
}



function jsCall_viii_51(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(51,a1|0,a2|0,a3|0);
}



function jsCall_viii_52(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(52,a1|0,a2|0,a3|0);
}



function jsCall_viii_53(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(53,a1|0,a2|0,a3|0);
}



function jsCall_viii_54(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(54,a1|0,a2|0,a3|0);
}



function jsCall_viii_55(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(55,a1|0,a2|0,a3|0);
}



function jsCall_viii_56(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(56,a1|0,a2|0,a3|0);
}



function jsCall_viii_57(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(57,a1|0,a2|0,a3|0);
}



function jsCall_viii_58(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(58,a1|0,a2|0,a3|0);
}



function jsCall_viii_59(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(59,a1|0,a2|0,a3|0);
}



function jsCall_viii_60(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(60,a1|0,a2|0,a3|0);
}



function jsCall_viii_61(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(61,a1|0,a2|0,a3|0);
}



function jsCall_viii_62(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(62,a1|0,a2|0,a3|0);
}



function jsCall_viii_63(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(63,a1|0,a2|0,a3|0);
}



function dynCall_v(index) {
  index = index|0;
  
  FUNCTION_TABLE_v[index&255]();
}


function jsCall_v_0() {
  
  jsCall_v(0);
}



function jsCall_v_1() {
  
  jsCall_v(1);
}



function jsCall_v_2() {
  
  jsCall_v(2);
}



function jsCall_v_3() {
  
  jsCall_v(3);
}



function jsCall_v_4() {
  
  jsCall_v(4);
}



function jsCall_v_5() {
  
  jsCall_v(5);
}



function jsCall_v_6() {
  
  jsCall_v(6);
}



function jsCall_v_7() {
  
  jsCall_v(7);
}



function jsCall_v_8() {
  
  jsCall_v(8);
}



function jsCall_v_9() {
  
  jsCall_v(9);
}



function jsCall_v_10() {
  
  jsCall_v(10);
}



function jsCall_v_11() {
  
  jsCall_v(11);
}



function jsCall_v_12() {
  
  jsCall_v(12);
}



function jsCall_v_13() {
  
  jsCall_v(13);
}



function jsCall_v_14() {
  
  jsCall_v(14);
}



function jsCall_v_15() {
  
  jsCall_v(15);
}



function jsCall_v_16() {
  
  jsCall_v(16);
}



function jsCall_v_17() {
  
  jsCall_v(17);
}



function jsCall_v_18() {
  
  jsCall_v(18);
}



function jsCall_v_19() {
  
  jsCall_v(19);
}



function jsCall_v_20() {
  
  jsCall_v(20);
}



function jsCall_v_21() {
  
  jsCall_v(21);
}



function jsCall_v_22() {
  
  jsCall_v(22);
}



function jsCall_v_23() {
  
  jsCall_v(23);
}



function jsCall_v_24() {
  
  jsCall_v(24);
}



function jsCall_v_25() {
  
  jsCall_v(25);
}



function jsCall_v_26() {
  
  jsCall_v(26);
}



function jsCall_v_27() {
  
  jsCall_v(27);
}



function jsCall_v_28() {
  
  jsCall_v(28);
}



function jsCall_v_29() {
  
  jsCall_v(29);
}



function jsCall_v_30() {
  
  jsCall_v(30);
}



function jsCall_v_31() {
  
  jsCall_v(31);
}



function jsCall_v_32() {
  
  jsCall_v(32);
}



function jsCall_v_33() {
  
  jsCall_v(33);
}



function jsCall_v_34() {
  
  jsCall_v(34);
}



function jsCall_v_35() {
  
  jsCall_v(35);
}



function jsCall_v_36() {
  
  jsCall_v(36);
}



function jsCall_v_37() {
  
  jsCall_v(37);
}



function jsCall_v_38() {
  
  jsCall_v(38);
}



function jsCall_v_39() {
  
  jsCall_v(39);
}



function jsCall_v_40() {
  
  jsCall_v(40);
}



function jsCall_v_41() {
  
  jsCall_v(41);
}



function jsCall_v_42() {
  
  jsCall_v(42);
}



function jsCall_v_43() {
  
  jsCall_v(43);
}



function jsCall_v_44() {
  
  jsCall_v(44);
}



function jsCall_v_45() {
  
  jsCall_v(45);
}



function jsCall_v_46() {
  
  jsCall_v(46);
}



function jsCall_v_47() {
  
  jsCall_v(47);
}



function jsCall_v_48() {
  
  jsCall_v(48);
}



function jsCall_v_49() {
  
  jsCall_v(49);
}



function jsCall_v_50() {
  
  jsCall_v(50);
}



function jsCall_v_51() {
  
  jsCall_v(51);
}



function jsCall_v_52() {
  
  jsCall_v(52);
}



function jsCall_v_53() {
  
  jsCall_v(53);
}



function jsCall_v_54() {
  
  jsCall_v(54);
}



function jsCall_v_55() {
  
  jsCall_v(55);
}



function jsCall_v_56() {
  
  jsCall_v(56);
}



function jsCall_v_57() {
  
  jsCall_v(57);
}



function jsCall_v_58() {
  
  jsCall_v(58);
}



function jsCall_v_59() {
  
  jsCall_v(59);
}



function jsCall_v_60() {
  
  jsCall_v(60);
}



function jsCall_v_61() {
  
  jsCall_v(61);
}



function jsCall_v_62() {
  
  jsCall_v(62);
}



function jsCall_v_63() {
  
  jsCall_v(63);
}



function dynCall_iiiii(index,a1,a2,a3,a4) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return FUNCTION_TABLE_iiiii[index&255](a1|0,a2|0,a3|0,a4|0)|0;
}


function jsCall_iiiii_0(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(0,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_1(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(1,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_2(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(2,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_3(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(3,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_4(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(4,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_5(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(5,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_6(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(6,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_7(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(7,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_8(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(8,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_9(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(9,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_10(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(10,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_11(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(11,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_12(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(12,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_13(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(13,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_14(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(14,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_15(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(15,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_16(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(16,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_17(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(17,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_18(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(18,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_19(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(19,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_20(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(20,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_21(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(21,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_22(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(22,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_23(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(23,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_24(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(24,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_25(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(25,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_26(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(26,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_27(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(27,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_28(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(28,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_29(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(29,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_30(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(30,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_31(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(31,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_32(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(32,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_33(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(33,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_34(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(34,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_35(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(35,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_36(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(36,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_37(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(37,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_38(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(38,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_39(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(39,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_40(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(40,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_41(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(41,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_42(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(42,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_43(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(43,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_44(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(44,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_45(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(45,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_46(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(46,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_47(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(47,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_48(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(48,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_49(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(49,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_50(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(50,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_51(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(51,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_52(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(52,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_53(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(53,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_54(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(54,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_55(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(55,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_56(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(56,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_57(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(57,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_58(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(58,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_59(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(59,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_60(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(60,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_61(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(61,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_62(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(62,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_63(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(63,a1|0,a2|0,a3|0,a4|0)|0;
}



function dynCall_viiiiii(index,a1,a2,a3,a4,a5,a6) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  FUNCTION_TABLE_viiiiii[index&255](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}


function jsCall_viiiiii_0(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(0,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_1(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(1,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_2(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(2,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_3(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(3,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_4(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(4,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_5(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(5,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_6(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(6,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_7(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(7,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_8(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(8,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_9(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(9,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_10(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(10,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_11(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(11,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_12(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(12,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_13(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(13,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_14(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(14,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_15(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(15,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_16(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(16,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_17(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(17,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_18(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(18,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_19(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(19,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_20(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(20,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_21(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(21,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_22(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(22,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_23(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(23,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_24(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(24,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_25(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(25,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_26(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(26,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_27(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(27,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_28(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(28,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_29(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(29,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_30(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(30,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_31(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(31,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_32(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(32,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_33(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(33,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_34(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(34,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_35(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(35,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_36(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(36,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_37(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(37,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_38(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(38,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_39(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(39,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_40(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(40,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_41(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(41,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_42(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(42,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_43(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(43,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_44(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(44,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_45(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(45,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_46(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(46,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_47(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(47,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_48(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(48,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_49(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(49,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_50(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(50,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_51(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(51,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_52(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(52,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_53(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(53,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_54(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(54,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_55(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(55,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_56(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(56,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_57(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(57,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_58(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(58,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_59(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(59,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_60(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(60,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_61(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(61,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_62(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(62,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_63(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(63,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function dynCall_iii(index,a1,a2) {
  index = index|0;
  a1=a1|0; a2=a2|0;
  return FUNCTION_TABLE_iii[index&255](a1|0,a2|0)|0;
}


function jsCall_iii_0(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(0,a1|0,a2|0)|0;
}



function jsCall_iii_1(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(1,a1|0,a2|0)|0;
}



function jsCall_iii_2(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(2,a1|0,a2|0)|0;
}



function jsCall_iii_3(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(3,a1|0,a2|0)|0;
}



function jsCall_iii_4(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(4,a1|0,a2|0)|0;
}



function jsCall_iii_5(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(5,a1|0,a2|0)|0;
}



function jsCall_iii_6(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(6,a1|0,a2|0)|0;
}



function jsCall_iii_7(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(7,a1|0,a2|0)|0;
}



function jsCall_iii_8(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(8,a1|0,a2|0)|0;
}



function jsCall_iii_9(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(9,a1|0,a2|0)|0;
}



function jsCall_iii_10(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(10,a1|0,a2|0)|0;
}



function jsCall_iii_11(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(11,a1|0,a2|0)|0;
}



function jsCall_iii_12(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(12,a1|0,a2|0)|0;
}



function jsCall_iii_13(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(13,a1|0,a2|0)|0;
}



function jsCall_iii_14(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(14,a1|0,a2|0)|0;
}



function jsCall_iii_15(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(15,a1|0,a2|0)|0;
}



function jsCall_iii_16(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(16,a1|0,a2|0)|0;
}



function jsCall_iii_17(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(17,a1|0,a2|0)|0;
}



function jsCall_iii_18(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(18,a1|0,a2|0)|0;
}



function jsCall_iii_19(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(19,a1|0,a2|0)|0;
}



function jsCall_iii_20(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(20,a1|0,a2|0)|0;
}



function jsCall_iii_21(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(21,a1|0,a2|0)|0;
}



function jsCall_iii_22(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(22,a1|0,a2|0)|0;
}



function jsCall_iii_23(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(23,a1|0,a2|0)|0;
}



function jsCall_iii_24(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(24,a1|0,a2|0)|0;
}



function jsCall_iii_25(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(25,a1|0,a2|0)|0;
}



function jsCall_iii_26(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(26,a1|0,a2|0)|0;
}



function jsCall_iii_27(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(27,a1|0,a2|0)|0;
}



function jsCall_iii_28(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(28,a1|0,a2|0)|0;
}



function jsCall_iii_29(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(29,a1|0,a2|0)|0;
}



function jsCall_iii_30(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(30,a1|0,a2|0)|0;
}



function jsCall_iii_31(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(31,a1|0,a2|0)|0;
}



function jsCall_iii_32(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(32,a1|0,a2|0)|0;
}



function jsCall_iii_33(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(33,a1|0,a2|0)|0;
}



function jsCall_iii_34(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(34,a1|0,a2|0)|0;
}



function jsCall_iii_35(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(35,a1|0,a2|0)|0;
}



function jsCall_iii_36(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(36,a1|0,a2|0)|0;
}



function jsCall_iii_37(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(37,a1|0,a2|0)|0;
}



function jsCall_iii_38(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(38,a1|0,a2|0)|0;
}



function jsCall_iii_39(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(39,a1|0,a2|0)|0;
}



function jsCall_iii_40(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(40,a1|0,a2|0)|0;
}



function jsCall_iii_41(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(41,a1|0,a2|0)|0;
}



function jsCall_iii_42(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(42,a1|0,a2|0)|0;
}



function jsCall_iii_43(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(43,a1|0,a2|0)|0;
}



function jsCall_iii_44(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(44,a1|0,a2|0)|0;
}



function jsCall_iii_45(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(45,a1|0,a2|0)|0;
}



function jsCall_iii_46(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(46,a1|0,a2|0)|0;
}



function jsCall_iii_47(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(47,a1|0,a2|0)|0;
}



function jsCall_iii_48(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(48,a1|0,a2|0)|0;
}



function jsCall_iii_49(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(49,a1|0,a2|0)|0;
}



function jsCall_iii_50(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(50,a1|0,a2|0)|0;
}



function jsCall_iii_51(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(51,a1|0,a2|0)|0;
}



function jsCall_iii_52(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(52,a1|0,a2|0)|0;
}



function jsCall_iii_53(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(53,a1|0,a2|0)|0;
}



function jsCall_iii_54(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(54,a1|0,a2|0)|0;
}



function jsCall_iii_55(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(55,a1|0,a2|0)|0;
}



function jsCall_iii_56(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(56,a1|0,a2|0)|0;
}



function jsCall_iii_57(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(57,a1|0,a2|0)|0;
}



function jsCall_iii_58(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(58,a1|0,a2|0)|0;
}



function jsCall_iii_59(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(59,a1|0,a2|0)|0;
}



function jsCall_iii_60(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(60,a1|0,a2|0)|0;
}



function jsCall_iii_61(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(61,a1|0,a2|0)|0;
}



function jsCall_iii_62(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(62,a1|0,a2|0)|0;
}



function jsCall_iii_63(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(63,a1|0,a2|0)|0;
}



function dynCall_iiiiii(index,a1,a2,a3,a4,a5) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return FUNCTION_TABLE_iiiiii[index&255](a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}


function jsCall_iiiiii_0(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(0,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_1(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(1,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_2(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(2,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_3(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(3,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_4(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(4,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_5(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(5,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_6(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(6,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_7(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(7,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_8(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(8,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_9(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(9,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_10(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(10,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_11(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(11,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_12(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(12,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_13(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(13,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_14(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(14,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_15(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(15,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_16(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(16,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_17(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(17,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_18(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(18,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_19(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(19,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_20(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(20,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_21(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(21,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_22(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(22,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_23(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(23,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_24(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(24,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_25(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(25,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_26(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(26,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_27(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(27,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_28(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(28,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_29(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(29,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_30(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(30,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_31(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(31,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_32(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(32,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_33(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(33,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_34(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(34,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_35(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(35,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_36(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(36,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_37(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(37,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_38(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(38,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_39(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(39,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_40(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(40,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_41(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(41,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_42(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(42,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_43(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(43,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_44(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(44,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_45(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(45,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_46(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(46,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_47(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(47,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_48(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(48,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_49(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(49,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_50(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(50,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_51(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(51,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_52(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(52,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_53(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(53,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_54(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(54,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_55(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(55,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_56(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(56,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_57(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(57,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_58(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(58,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_59(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(59,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_60(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(60,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_61(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(61,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_62(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(62,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_63(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(63,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function dynCall_viiii(index,a1,a2,a3,a4) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  FUNCTION_TABLE_viiii[index&255](a1|0,a2|0,a3|0,a4|0);
}


function jsCall_viiii_0(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(0,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_1(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(1,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_2(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(2,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_3(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(3,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_4(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(4,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_5(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(5,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_6(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(6,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_7(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(7,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_8(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(8,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_9(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(9,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_10(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(10,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_11(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(11,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_12(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(12,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_13(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(13,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_14(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(14,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_15(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(15,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_16(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(16,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_17(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(17,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_18(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(18,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_19(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(19,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_20(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(20,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_21(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(21,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_22(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(22,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_23(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(23,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_24(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(24,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_25(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(25,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_26(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(26,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_27(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(27,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_28(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(28,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_29(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(29,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_30(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(30,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_31(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(31,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_32(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(32,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_33(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(33,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_34(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(34,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_35(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(35,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_36(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(36,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_37(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(37,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_38(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(38,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_39(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(39,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_40(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(40,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_41(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(41,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_42(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(42,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_43(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(43,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_44(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(44,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_45(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(45,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_46(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(46,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_47(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(47,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_48(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(48,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_49(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(49,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_50(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(50,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_51(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(51,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_52(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(52,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_53(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(53,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_54(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(54,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_55(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(55,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_56(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(56,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_57(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(57,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_58(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(58,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_59(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(59,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_60(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(60,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_61(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(61,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_62(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(62,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_63(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(63,a1|0,a2|0,a3|0,a4|0);
}


function b0(p0,p1,p2) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0; abort(0);return 0;
}
function b1() {
 ; abort(1);return 0;
}
function b2(p0) {
 p0 = p0|0; abort(2);
}
function b3(p0,p1) {
 p0 = p0|0;p1 = p1|0; abort(3);
}
function b4(p0,p1,p2,p3,p4,p5) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; abort(4);return 0;
}
function b5(p0) {
 p0 = p0|0; abort(5);return 0;
}
function b6(p0,p1,p2) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0; abort(6);
}
function b7() {
 ; abort(7);
}
function b8(p0,p1,p2,p3) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; abort(8);return 0;
}
function b9(p0,p1,p2,p3,p4,p5) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; abort(9);
}
function b10(p0,p1) {
 p0 = p0|0;p1 = p1|0; abort(10);return 0;
}
function b11(p0,p1,p2,p3,p4) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; abort(11);return 0;
}
function b12(p0,p1,p2,p3) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; abort(12);
}

// EMSCRIPTEN_END_FUNCS
var FUNCTION_TABLE_iiii = [b0,b0,jsCall_iiii_0,b0,jsCall_iiii_1,b0,jsCall_iiii_2,b0,jsCall_iiii_3,b0,jsCall_iiii_4,b0,jsCall_iiii_5,b0,jsCall_iiii_6,b0,jsCall_iiii_7,b0,jsCall_iiii_8,b0,jsCall_iiii_9,b0,jsCall_iiii_10,b0,jsCall_iiii_11,b0,jsCall_iiii_12,b0,jsCall_iiii_13
,b0,jsCall_iiii_14,b0,jsCall_iiii_15,b0,jsCall_iiii_16,b0,jsCall_iiii_17,b0,jsCall_iiii_18,b0,jsCall_iiii_19,b0,jsCall_iiii_20,b0,jsCall_iiii_21,b0,jsCall_iiii_22,b0,jsCall_iiii_23,b0,jsCall_iiii_24,b0,jsCall_iiii_25,b0,jsCall_iiii_26,b0,jsCall_iiii_27,b0,jsCall_iiii_28
,b0,jsCall_iiii_29,b0,jsCall_iiii_30,b0,jsCall_iiii_31,b0,jsCall_iiii_32,b0,jsCall_iiii_33,b0,jsCall_iiii_34,b0,jsCall_iiii_35,b0,jsCall_iiii_36,b0,jsCall_iiii_37,b0,jsCall_iiii_38,b0,jsCall_iiii_39,b0,jsCall_iiii_40,b0,jsCall_iiii_41,b0,jsCall_iiii_42,b0,jsCall_iiii_43
,b0,jsCall_iiii_44,b0,jsCall_iiii_45,b0,jsCall_iiii_46,b0,jsCall_iiii_47,b0,jsCall_iiii_48,b0,jsCall_iiii_49,b0,jsCall_iiii_50,b0,jsCall_iiii_51,b0,jsCall_iiii_52,b0,jsCall_iiii_53,b0,jsCall_iiii_54,b0,jsCall_iiii_55,b0,jsCall_iiii_56,b0,jsCall_iiii_57,b0,jsCall_iiii_58
,b0,jsCall_iiii_59,b0,jsCall_iiii_60,b0,jsCall_iiii_61,b0,jsCall_iiii_62,b0,jsCall_iiii_63,b0,_unixDelete,_unixRandomness,_unixGetLastError,_unixSetSystemCall,_unixTruncate,_unixFileControl,_posixOpen,_fcntl,_read,_write,_fchown,_readlink,_memjrnlTruncate,_pcache1Create,_pcache1Fetch,_fts3ColumnMethod,_unicodeCreate,_fts3auxColumnMethod,_simpleCreate
,_porterCreate,_fts3tokColumnMethod,___stdout_write,___stdio_seek,_vdbeRecordCompareInt,_vdbeRecordCompareString,_sqlite3VdbeRecordCompare,_fts3SnippetFindPositions,_fts3ExprLoadDoclistsCb,_fts3ExprTermOffsetInit,_fts3ExprPhraseCountCb,_fts3ExprGlobalHitsCb,_fts3ExprLocalHitsCb,_fts3MatchinfoLcsCb,___stdio_write,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0];
var FUNCTION_TABLE_i = [b1,b1,jsCall_i_0,b1,jsCall_i_1,b1,jsCall_i_2,b1,jsCall_i_3,b1,jsCall_i_4,b1,jsCall_i_5,b1,jsCall_i_6,b1,jsCall_i_7,b1,jsCall_i_8,b1,jsCall_i_9,b1,jsCall_i_10,b1,jsCall_i_11,b1,jsCall_i_12,b1,jsCall_i_13
,b1,jsCall_i_14,b1,jsCall_i_15,b1,jsCall_i_16,b1,jsCall_i_17,b1,jsCall_i_18,b1,jsCall_i_19,b1,jsCall_i_20,b1,jsCall_i_21,b1,jsCall_i_22,b1,jsCall_i_23,b1,jsCall_i_24,b1,jsCall_i_25,b1,jsCall_i_26,b1,jsCall_i_27,b1,jsCall_i_28
,b1,jsCall_i_29,b1,jsCall_i_30,b1,jsCall_i_31,b1,jsCall_i_32,b1,jsCall_i_33,b1,jsCall_i_34,b1,jsCall_i_35,b1,jsCall_i_36,b1,jsCall_i_37,b1,jsCall_i_38,b1,jsCall_i_39,b1,jsCall_i_40,b1,jsCall_i_41,b1,jsCall_i_42,b1,jsCall_i_43
,b1,jsCall_i_44,b1,jsCall_i_45,b1,jsCall_i_46,b1,jsCall_i_47,b1,jsCall_i_48,b1,jsCall_i_49,b1,jsCall_i_50,b1,jsCall_i_51,b1,jsCall_i_52,b1,jsCall_i_53,b1,jsCall_i_54,b1,jsCall_i_55,b1,jsCall_i_56,b1,jsCall_i_57,b1,jsCall_i_58
,b1,jsCall_i_59,b1,jsCall_i_60,b1,jsCall_i_61,b1,jsCall_i_62,b1,jsCall_i_63,b1,_geteuid,_unixGetpagesize,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1];
var FUNCTION_TABLE_vi = [b2,b2,jsCall_vi_0,b2,jsCall_vi_1,b2,jsCall_vi_2,b2,jsCall_vi_3,b2,jsCall_vi_4,b2,jsCall_vi_5,b2,jsCall_vi_6,b2,jsCall_vi_7,b2,jsCall_vi_8,b2,jsCall_vi_9,b2,jsCall_vi_10,b2,jsCall_vi_11,b2,jsCall_vi_12,b2,jsCall_vi_13
,b2,jsCall_vi_14,b2,jsCall_vi_15,b2,jsCall_vi_16,b2,jsCall_vi_17,b2,jsCall_vi_18,b2,jsCall_vi_19,b2,jsCall_vi_20,b2,jsCall_vi_21,b2,jsCall_vi_22,b2,jsCall_vi_23,b2,jsCall_vi_24,b2,jsCall_vi_25,b2,jsCall_vi_26,b2,jsCall_vi_27,b2,jsCall_vi_28
,b2,jsCall_vi_29,b2,jsCall_vi_30,b2,jsCall_vi_31,b2,jsCall_vi_32,b2,jsCall_vi_33,b2,jsCall_vi_34,b2,jsCall_vi_35,b2,jsCall_vi_36,b2,jsCall_vi_37,b2,jsCall_vi_38,b2,jsCall_vi_39,b2,jsCall_vi_40,b2,jsCall_vi_41,b2,jsCall_vi_42,b2,jsCall_vi_43
,b2,jsCall_vi_44,b2,jsCall_vi_45,b2,jsCall_vi_46,b2,jsCall_vi_47,b2,jsCall_vi_48,b2,jsCall_vi_49,b2,jsCall_vi_50,b2,jsCall_vi_51,b2,jsCall_vi_52,b2,jsCall_vi_53,b2,jsCall_vi_54,b2,jsCall_vi_55,b2,jsCall_vi_56,b2,jsCall_vi_57,b2,jsCall_vi_58
,b2,jsCall_vi_59,b2,jsCall_vi_60,b2,jsCall_vi_61,b2,jsCall_vi_62,b2,jsCall_vi_63,b2,_unixShmBarrier,_minMaxFinalize,_sumFinalize,_totalFinalize,_avgFinalize,_countFinalize,_groupConcatFinalize,_sqlite3MemFree,_sqlite3MemShutdown,_pcache1Shutdown,_pcache1Destroy,_pcache1Shrink,_sqlite3_free,_pageReinit,_sqlite3SchemaClear,_stat4Destructor,_hashDestroy,_fts3MIBufferFree,_cleanup392
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2];
var FUNCTION_TABLE_vii = [b3,b3,jsCall_vii_0,b3,jsCall_vii_1,b3,jsCall_vii_2,b3,jsCall_vii_3,b3,jsCall_vii_4,b3,jsCall_vii_5,b3,jsCall_vii_6,b3,jsCall_vii_7,b3,jsCall_vii_8,b3,jsCall_vii_9,b3,jsCall_vii_10,b3,jsCall_vii_11,b3,jsCall_vii_12,b3,jsCall_vii_13
,b3,jsCall_vii_14,b3,jsCall_vii_15,b3,jsCall_vii_16,b3,jsCall_vii_17,b3,jsCall_vii_18,b3,jsCall_vii_19,b3,jsCall_vii_20,b3,jsCall_vii_21,b3,jsCall_vii_22,b3,jsCall_vii_23,b3,jsCall_vii_24,b3,jsCall_vii_25,b3,jsCall_vii_26,b3,jsCall_vii_27,b3,jsCall_vii_28
,b3,jsCall_vii_29,b3,jsCall_vii_30,b3,jsCall_vii_31,b3,jsCall_vii_32,b3,jsCall_vii_33,b3,jsCall_vii_34,b3,jsCall_vii_35,b3,jsCall_vii_36,b3,jsCall_vii_37,b3,jsCall_vii_38,b3,jsCall_vii_39,b3,jsCall_vii_40,b3,jsCall_vii_41,b3,jsCall_vii_42,b3,jsCall_vii_43
,b3,jsCall_vii_44,b3,jsCall_vii_45,b3,jsCall_vii_46,b3,jsCall_vii_47,b3,jsCall_vii_48,b3,jsCall_vii_49,b3,jsCall_vii_50,b3,jsCall_vii_51,b3,jsCall_vii_52,b3,jsCall_vii_53,b3,jsCall_vii_54,b3,jsCall_vii_55,b3,jsCall_vii_56,b3,jsCall_vii_57,b3,jsCall_vii_58
,b3,jsCall_vii_59,b3,jsCall_vii_60,b3,jsCall_vii_61,b3,jsCall_vii_62,b3,jsCall_vii_63,b3,_pcache1Cachesize,_pcache1Truncate,_selectPopWith,_selectAddSubqueryTypeInfo,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3];
var FUNCTION_TABLE_iiiiiii = [b4,b4,jsCall_iiiiiii_0,b4,jsCall_iiiiiii_1,b4,jsCall_iiiiiii_2,b4,jsCall_iiiiiii_3,b4,jsCall_iiiiiii_4,b4,jsCall_iiiiiii_5,b4,jsCall_iiiiiii_6,b4,jsCall_iiiiiii_7,b4,jsCall_iiiiiii_8,b4,jsCall_iiiiiii_9,b4,jsCall_iiiiiii_10,b4,jsCall_iiiiiii_11,b4,jsCall_iiiiiii_12,b4,jsCall_iiiiiii_13
,b4,jsCall_iiiiiii_14,b4,jsCall_iiiiiii_15,b4,jsCall_iiiiiii_16,b4,jsCall_iiiiiii_17,b4,jsCall_iiiiiii_18,b4,jsCall_iiiiiii_19,b4,jsCall_iiiiiii_20,b4,jsCall_iiiiiii_21,b4,jsCall_iiiiiii_22,b4,jsCall_iiiiiii_23,b4,jsCall_iiiiiii_24,b4,jsCall_iiiiiii_25,b4,jsCall_iiiiiii_26,b4,jsCall_iiiiiii_27,b4,jsCall_iiiiiii_28
,b4,jsCall_iiiiiii_29,b4,jsCall_iiiiiii_30,b4,jsCall_iiiiiii_31,b4,jsCall_iiiiiii_32,b4,jsCall_iiiiiii_33,b4,jsCall_iiiiiii_34,b4,jsCall_iiiiiii_35,b4,jsCall_iiiiiii_36,b4,jsCall_iiiiiii_37,b4,jsCall_iiiiiii_38,b4,jsCall_iiiiiii_39,b4,jsCall_iiiiiii_40,b4,jsCall_iiiiiii_41,b4,jsCall_iiiiiii_42,b4,jsCall_iiiiiii_43
,b4,jsCall_iiiiiii_44,b4,jsCall_iiiiiii_45,b4,jsCall_iiiiiii_46,b4,jsCall_iiiiiii_47,b4,jsCall_iiiiiii_48,b4,jsCall_iiiiiii_49,b4,jsCall_iiiiiii_50,b4,jsCall_iiiiiii_51,b4,jsCall_iiiiiii_52,b4,jsCall_iiiiiii_53,b4,jsCall_iiiiiii_54,b4,jsCall_iiiiiii_55,b4,jsCall_iiiiiii_56,b4,jsCall_iiiiiii_57,b4,jsCall_iiiiiii_58
,b4,jsCall_iiiiiii_59,b4,jsCall_iiiiiii_60,b4,jsCall_iiiiiii_61,b4,jsCall_iiiiiii_62,b4,jsCall_iiiiiii_63,b4,___mmap,_fts3CreateMethod,_fts3ConnectMethod,_unicodeNext,_fts3auxConnectMethod,_simpleNext,_porterNext,_fts3tokConnectMethod,_vdbeSorterCompareInt,_vdbeSorterCompareText,_vdbeSorterCompare,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4];
var FUNCTION_TABLE_ii = [b5,b5,jsCall_ii_0,b5,jsCall_ii_1,b5,jsCall_ii_2,b5,jsCall_ii_3,b5,jsCall_ii_4,b5,jsCall_ii_5,b5,jsCall_ii_6,b5,jsCall_ii_7,b5,jsCall_ii_8,b5,jsCall_ii_9,b5,jsCall_ii_10,b5,jsCall_ii_11,b5,jsCall_ii_12,b5,jsCall_ii_13
,b5,jsCall_ii_14,b5,jsCall_ii_15,b5,jsCall_ii_16,b5,jsCall_ii_17,b5,jsCall_ii_18,b5,jsCall_ii_19,b5,jsCall_ii_20,b5,jsCall_ii_21,b5,jsCall_ii_22,b5,jsCall_ii_23,b5,jsCall_ii_24,b5,jsCall_ii_25,b5,jsCall_ii_26,b5,jsCall_ii_27,b5,jsCall_ii_28
,b5,jsCall_ii_29,b5,jsCall_ii_30,b5,jsCall_ii_31,b5,jsCall_ii_32,b5,jsCall_ii_33,b5,jsCall_ii_34,b5,jsCall_ii_35,b5,jsCall_ii_36,b5,jsCall_ii_37,b5,jsCall_ii_38,b5,jsCall_ii_39,b5,jsCall_ii_40,b5,jsCall_ii_41,b5,jsCall_ii_42,b5,jsCall_ii_43
,b5,jsCall_ii_44,b5,jsCall_ii_45,b5,jsCall_ii_46,b5,jsCall_ii_47,b5,jsCall_ii_48,b5,jsCall_ii_49,b5,jsCall_ii_50,b5,jsCall_ii_51,b5,jsCall_ii_52,b5,jsCall_ii_53,b5,jsCall_ii_54,b5,jsCall_ii_55,b5,jsCall_ii_56,b5,jsCall_ii_57,b5,jsCall_ii_58
,b5,jsCall_ii_59,b5,jsCall_ii_60,b5,jsCall_ii_61,b5,jsCall_ii_62,b5,jsCall_ii_63,b5,_unixClose,_unixSectorSize,_unixDeviceCharacteristics,_close,_unlink,_rmdir,_nolockClose,_dotlockClose,_memjrnlClose,_sqlite3MemMalloc,_sqlite3MemSize,_sqlite3MemRoundup,_sqlite3MemInit,_pcache1Init,_pcache1Pagecount,_fts3DisconnectMethod,_fts3DestroyMethod,_fts3CloseMethod,_fts3NextMethod
,_fts3EofMethod,_fts3BeginMethod,_fts3SyncMethod,_fts3CommitMethod,_fts3RollbackMethod,_unicodeDestroy,_unicodeClose,_fts3auxDisconnectMethod,_fts3auxCloseMethod,_fts3auxNextMethod,_fts3auxEofMethod,_simpleDestroy,_simpleClose,_porterDestroy,_porterClose,_fts3tokDisconnectMethod,_fts3tokCloseMethod,_fts3tokNextMethod,_fts3tokEofMethod,___stdio_close,_sqlite3MallocSize,_sqlite3_value_text,_btreeInvokeBusyHandler,_m_gt_0,_hasVowel,_m_gt_1,b5,b5,b5,b5
,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5];
var FUNCTION_TABLE_viii = [b6,b6,jsCall_viii_0,b6,jsCall_viii_1,b6,jsCall_viii_2,b6,jsCall_viii_3,b6,jsCall_viii_4,b6,jsCall_viii_5,b6,jsCall_viii_6,b6,jsCall_viii_7,b6,jsCall_viii_8,b6,jsCall_viii_9,b6,jsCall_viii_10,b6,jsCall_viii_11,b6,jsCall_viii_12,b6,jsCall_viii_13
,b6,jsCall_viii_14,b6,jsCall_viii_15,b6,jsCall_viii_16,b6,jsCall_viii_17,b6,jsCall_viii_18,b6,jsCall_viii_19,b6,jsCall_viii_20,b6,jsCall_viii_21,b6,jsCall_viii_22,b6,jsCall_viii_23,b6,jsCall_viii_24,b6,jsCall_viii_25,b6,jsCall_viii_26,b6,jsCall_viii_27,b6,jsCall_viii_28
,b6,jsCall_viii_29,b6,jsCall_viii_30,b6,jsCall_viii_31,b6,jsCall_viii_32,b6,jsCall_viii_33,b6,jsCall_viii_34,b6,jsCall_viii_35,b6,jsCall_viii_36,b6,jsCall_viii_37,b6,jsCall_viii_38,b6,jsCall_viii_39,b6,jsCall_viii_40,b6,jsCall_viii_41,b6,jsCall_viii_42,b6,jsCall_viii_43
,b6,jsCall_viii_44,b6,jsCall_viii_45,b6,jsCall_viii_46,b6,jsCall_viii_47,b6,jsCall_viii_48,b6,jsCall_viii_49,b6,jsCall_viii_50,b6,jsCall_viii_51,b6,jsCall_viii_52,b6,jsCall_viii_53,b6,jsCall_viii_54,b6,jsCall_viii_55,b6,jsCall_viii_56,b6,jsCall_viii_57,b6,jsCall_viii_58
,b6,jsCall_viii_59,b6,jsCall_viii_60,b6,jsCall_viii_61,b6,jsCall_viii_62,b6,jsCall_viii_63,b6,_attachFunc,_detachFunc,_statInit,_statPush,_statGet,_trimFunc,_minmaxFunc,_minmaxStep,_typeofFunc,_lengthFunc,_instrFunc,_substrFunc,_printfFunc,_unicodeFunc,_charFunc,_absFunc,_roundFunc,_upperFunc,_lowerFunc
,_versionFunc,_hexFunc,_randomFunc,_randomBlob,_nullifFunc,_sourceidFunc,_errlogFunc,_compileoptionusedFunc,_compileoptiongetFunc,_quoteFunc,_last_insert_rowid,_changes,_total_changes,_replaceFunc,_zeroblobFunc,_sumStep,_countStep,_groupConcatStep,_likeFunc,_juliandayFunc,_dateFunc,_timeFunc,_datetimeFunc,_strftimeFunc,_ctimeFunc,_ctimestampFunc,_cdateFunc,_renameTableFunc,_renameTriggerFunc,_renameParentFunc
,_pcache1Unpin,_fts3SnippetFunc,_fts3OffsetsFunc,_fts3OptimizeFunc,_fts3MatchinfoFunc,_sqlite3InvalidFunction,_btreeParseCellPtr,_btreeParseCellPtrNoPayload,_btreeParseCellPtrIndex,_scalarFunc,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6];
var FUNCTION_TABLE_v = [b7,b7,jsCall_v_0,b7,jsCall_v_1,b7,jsCall_v_2,b7,jsCall_v_3,b7,jsCall_v_4,b7,jsCall_v_5,b7,jsCall_v_6,b7,jsCall_v_7,b7,jsCall_v_8,b7,jsCall_v_9,b7,jsCall_v_10,b7,jsCall_v_11,b7,jsCall_v_12,b7,jsCall_v_13
,b7,jsCall_v_14,b7,jsCall_v_15,b7,jsCall_v_16,b7,jsCall_v_17,b7,jsCall_v_18,b7,jsCall_v_19,b7,jsCall_v_20,b7,jsCall_v_21,b7,jsCall_v_22,b7,jsCall_v_23,b7,jsCall_v_24,b7,jsCall_v_25,b7,jsCall_v_26,b7,jsCall_v_27,b7,jsCall_v_28
,b7,jsCall_v_29,b7,jsCall_v_30,b7,jsCall_v_31,b7,jsCall_v_32,b7,jsCall_v_33,b7,jsCall_v_34,b7,jsCall_v_35,b7,jsCall_v_36,b7,jsCall_v_37,b7,jsCall_v_38,b7,jsCall_v_39,b7,jsCall_v_40,b7,jsCall_v_41,b7,jsCall_v_42,b7,jsCall_v_43
,b7,jsCall_v_44,b7,jsCall_v_45,b7,jsCall_v_46,b7,jsCall_v_47,b7,jsCall_v_48,b7,jsCall_v_49,b7,jsCall_v_50,b7,jsCall_v_51,b7,jsCall_v_52,b7,jsCall_v_53,b7,jsCall_v_54,b7,jsCall_v_55,b7,jsCall_v_56,b7,jsCall_v_57,b7,jsCall_v_58
,b7,jsCall_v_59,b7,jsCall_v_60,b7,jsCall_v_61,b7,jsCall_v_62,b7,jsCall_v_63,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7];
var FUNCTION_TABLE_iiiii = [b8,b8,jsCall_iiiii_0,b8,jsCall_iiiii_1,b8,jsCall_iiiii_2,b8,jsCall_iiiii_3,b8,jsCall_iiiii_4,b8,jsCall_iiiii_5,b8,jsCall_iiiii_6,b8,jsCall_iiiii_7,b8,jsCall_iiiii_8,b8,jsCall_iiiii_9,b8,jsCall_iiiii_10,b8,jsCall_iiiii_11,b8,jsCall_iiiii_12,b8,jsCall_iiiii_13
,b8,jsCall_iiiii_14,b8,jsCall_iiiii_15,b8,jsCall_iiiii_16,b8,jsCall_iiiii_17,b8,jsCall_iiiii_18,b8,jsCall_iiiii_19,b8,jsCall_iiiii_20,b8,jsCall_iiiii_21,b8,jsCall_iiiii_22,b8,jsCall_iiiii_23,b8,jsCall_iiiii_24,b8,jsCall_iiiii_25,b8,jsCall_iiiii_26,b8,jsCall_iiiii_27,b8,jsCall_iiiii_28
,b8,jsCall_iiiii_29,b8,jsCall_iiiii_30,b8,jsCall_iiiii_31,b8,jsCall_iiiii_32,b8,jsCall_iiiii_33,b8,jsCall_iiiii_34,b8,jsCall_iiiii_35,b8,jsCall_iiiii_36,b8,jsCall_iiiii_37,b8,jsCall_iiiii_38,b8,jsCall_iiiii_39,b8,jsCall_iiiii_40,b8,jsCall_iiiii_41,b8,jsCall_iiiii_42,b8,jsCall_iiiii_43
,b8,jsCall_iiiii_44,b8,jsCall_iiiii_45,b8,jsCall_iiiii_46,b8,jsCall_iiiii_47,b8,jsCall_iiiii_48,b8,jsCall_iiiii_49,b8,jsCall_iiiii_50,b8,jsCall_iiiii_51,b8,jsCall_iiiii_52,b8,jsCall_iiiii_53,b8,jsCall_iiiii_54,b8,jsCall_iiiii_55,b8,jsCall_iiiii_56,b8,jsCall_iiiii_57,b8,jsCall_iiiii_58
,b8,jsCall_iiiii_59,b8,jsCall_iiiii_60,b8,jsCall_iiiii_61,b8,jsCall_iiiii_62,b8,jsCall_iiiii_63,b8,_unixAccess,_unixFullPathname,_unixShmLock,_unixUnfetch,_fts3UpdateMethod,_unicodeOpen,_simpleOpen,_porterOpen,_sqlite3BtreeData,_sqlite3WalDefaultHook,_sqlite3InitCallback,_analysisLoader,_fts3StrCompare,_fts3BinCompare,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8];
var FUNCTION_TABLE_viiiiii = [b9,b9,jsCall_viiiiii_0,b9,jsCall_viiiiii_1,b9,jsCall_viiiiii_2,b9,jsCall_viiiiii_3,b9,jsCall_viiiiii_4,b9,jsCall_viiiiii_5,b9,jsCall_viiiiii_6,b9,jsCall_viiiiii_7,b9,jsCall_viiiiii_8,b9,jsCall_viiiiii_9,b9,jsCall_viiiiii_10,b9,jsCall_viiiiii_11,b9,jsCall_viiiiii_12,b9,jsCall_viiiiii_13
,b9,jsCall_viiiiii_14,b9,jsCall_viiiiii_15,b9,jsCall_viiiiii_16,b9,jsCall_viiiiii_17,b9,jsCall_viiiiii_18,b9,jsCall_viiiiii_19,b9,jsCall_viiiiii_20,b9,jsCall_viiiiii_21,b9,jsCall_viiiiii_22,b9,jsCall_viiiiii_23,b9,jsCall_viiiiii_24,b9,jsCall_viiiiii_25,b9,jsCall_viiiiii_26,b9,jsCall_viiiiii_27,b9,jsCall_viiiiii_28
,b9,jsCall_viiiiii_29,b9,jsCall_viiiiii_30,b9,jsCall_viiiiii_31,b9,jsCall_viiiiii_32,b9,jsCall_viiiiii_33,b9,jsCall_viiiiii_34,b9,jsCall_viiiiii_35,b9,jsCall_viiiiii_36,b9,jsCall_viiiiii_37,b9,jsCall_viiiiii_38,b9,jsCall_viiiiii_39,b9,jsCall_viiiiii_40,b9,jsCall_viiiiii_41,b9,jsCall_viiiiii_42,b9,jsCall_viiiiii_43
,b9,jsCall_viiiiii_44,b9,jsCall_viiiiii_45,b9,jsCall_viiiiii_46,b9,jsCall_viiiiii_47,b9,jsCall_viiiiii_48,b9,jsCall_viiiiii_49,b9,jsCall_viiiiii_50,b9,jsCall_viiiiii_51,b9,jsCall_viiiiii_52,b9,jsCall_viiiiii_53,b9,jsCall_viiiiii_54,b9,jsCall_viiiiii_55,b9,jsCall_viiiiii_56,b9,jsCall_viiiiii_57,b9,jsCall_viiiiii_58
,b9,jsCall_viiiiii_59,b9,jsCall_viiiiii_60,b9,jsCall_viiiiii_61,b9,jsCall_viiiiii_62,b9,jsCall_viiiiii_63,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9];
var FUNCTION_TABLE_iii = [b10,b10,jsCall_iii_0,b10,jsCall_iii_1,b10,jsCall_iii_2,b10,jsCall_iii_3,b10,jsCall_iii_4,b10,jsCall_iii_5,b10,jsCall_iii_6,b10,jsCall_iii_7,b10,jsCall_iii_8,b10,jsCall_iii_9,b10,jsCall_iii_10,b10,jsCall_iii_11,b10,jsCall_iii_12,b10,jsCall_iii_13
,b10,jsCall_iii_14,b10,jsCall_iii_15,b10,jsCall_iii_16,b10,jsCall_iii_17,b10,jsCall_iii_18,b10,jsCall_iii_19,b10,jsCall_iii_20,b10,jsCall_iii_21,b10,jsCall_iii_22,b10,jsCall_iii_23,b10,jsCall_iii_24,b10,jsCall_iii_25,b10,jsCall_iii_26,b10,jsCall_iii_27,b10,jsCall_iii_28
,b10,jsCall_iii_29,b10,jsCall_iii_30,b10,jsCall_iii_31,b10,jsCall_iii_32,b10,jsCall_iii_33,b10,jsCall_iii_34,b10,jsCall_iii_35,b10,jsCall_iii_36,b10,jsCall_iii_37,b10,jsCall_iii_38,b10,jsCall_iii_39,b10,jsCall_iii_40,b10,jsCall_iii_41,b10,jsCall_iii_42,b10,jsCall_iii_43
,b10,jsCall_iii_44,b10,jsCall_iii_45,b10,jsCall_iii_46,b10,jsCall_iii_47,b10,jsCall_iii_48,b10,jsCall_iii_49,b10,jsCall_iii_50,b10,jsCall_iii_51,b10,jsCall_iii_52,b10,jsCall_iii_53,b10,jsCall_iii_54,b10,jsCall_iii_55,b10,jsCall_iii_56,b10,jsCall_iii_57,b10,jsCall_iii_58
,b10,jsCall_iii_59,b10,jsCall_iii_60,b10,jsCall_iii_61,b10,jsCall_iii_62,b10,jsCall_iii_63,b10,_unixSleep,_unixCurrentTime,_unixCurrentTimeInt64,_unixGetSystemCall,_unixNextSystemCall,_posixIoFinderImpl,_nolockIoFinderImpl,_dotlockIoFinderImpl,_unixSync,_unixFileSize,_unixLock,_unixUnlock,_unixCheckReservedLock,_unixShmUnmap,_access,_getcwd,_stat,_fstat,_ftruncate
,_fchmod,_openDirectory,_mkdir,___munmap,_lstat,_nolockLock,_nolockUnlock,_nolockCheckReservedLock,_dotlockLock,_dotlockUnlock,_dotlockCheckReservedLock,_memjrnlSync,_memjrnlFileSize,_sqlite3MemRealloc,_fts3BestIndexMethod,_fts3OpenMethod,_fts3RowidMethod,_fts3RenameMethod,_fts3SavepointMethod,_fts3ReleaseMethod,_fts3RollbackToMethod,_fts3auxBestIndexMethod,_fts3auxOpenMethod,_fts3auxRowidMethod,_fts3tokBestIndexMethod,_fts3tokOpenMethod,_fts3tokRowidMethod,_sqlite3Malloc,_sqliteDefaultBusyCallback,_pagerStress
,_cellSizePtr,_cellSizePtrNoPayload,_pagerUndoCallback,_sqlite3BtreeNext,_sqlite3BtreePrevious,_exprNodeIsConstant,_selectNodeIsConstant,_resolveExprStep,_resolveSelectStep,_exprSrcCount,_incrAggDepth,_sqlite3ExprWalkNoop,_convertCompoundSelectToSubquery,_selectExpander,_analyzeAggregate,_analyzeAggregatesInSelect,_checkConstraintExprNode,_fts3CompareElemByTerm,_fts3StrHash,_fts3BinHash,_fts3SegReaderCmp,_fts3SegReaderDoclistCmpRev,_fts3SegReaderDoclistCmp,b10,b10,b10,b10,b10,b10,b10
,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10];
var FUNCTION_TABLE_iiiiii = [b11,b11,jsCall_iiiiii_0,b11,jsCall_iiiiii_1,b11,jsCall_iiiiii_2,b11,jsCall_iiiiii_3,b11,jsCall_iiiiii_4,b11,jsCall_iiiiii_5,b11,jsCall_iiiiii_6,b11,jsCall_iiiiii_7,b11,jsCall_iiiiii_8,b11,jsCall_iiiiii_9,b11,jsCall_iiiiii_10,b11,jsCall_iiiiii_11,b11,jsCall_iiiiii_12,b11,jsCall_iiiiii_13
,b11,jsCall_iiiiii_14,b11,jsCall_iiiiii_15,b11,jsCall_iiiiii_16,b11,jsCall_iiiiii_17,b11,jsCall_iiiiii_18,b11,jsCall_iiiiii_19,b11,jsCall_iiiiii_20,b11,jsCall_iiiiii_21,b11,jsCall_iiiiii_22,b11,jsCall_iiiiii_23,b11,jsCall_iiiiii_24,b11,jsCall_iiiiii_25,b11,jsCall_iiiiii_26,b11,jsCall_iiiiii_27,b11,jsCall_iiiiii_28
,b11,jsCall_iiiiii_29,b11,jsCall_iiiiii_30,b11,jsCall_iiiiii_31,b11,jsCall_iiiiii_32,b11,jsCall_iiiiii_33,b11,jsCall_iiiiii_34,b11,jsCall_iiiiii_35,b11,jsCall_iiiiii_36,b11,jsCall_iiiiii_37,b11,jsCall_iiiiii_38,b11,jsCall_iiiiii_39,b11,jsCall_iiiiii_40,b11,jsCall_iiiiii_41,b11,jsCall_iiiiii_42,b11,jsCall_iiiiii_43
,b11,jsCall_iiiiii_44,b11,jsCall_iiiiii_45,b11,jsCall_iiiiii_46,b11,jsCall_iiiiii_47,b11,jsCall_iiiiii_48,b11,jsCall_iiiiii_49,b11,jsCall_iiiiii_50,b11,jsCall_iiiiii_51,b11,jsCall_iiiiii_52,b11,jsCall_iiiiii_53,b11,jsCall_iiiiii_54,b11,jsCall_iiiiii_55,b11,jsCall_iiiiii_56,b11,jsCall_iiiiii_57,b11,jsCall_iiiiii_58
,b11,jsCall_iiiiii_59,b11,jsCall_iiiiii_60,b11,jsCall_iiiiii_61,b11,jsCall_iiiiii_62,b11,jsCall_iiiiii_63,b11,_unixOpen,_unixRead,_unixWrite,_unixShmMap,_unixFetch,_memjrnlRead,_memjrnlWrite,_fts3FilterMethod,_fts3FindFunctionMethod,_fts3auxFilterMethod,_fts3tokFilterMethod,_binCollFunc,_nocaseCollatingFunc,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11];
var FUNCTION_TABLE_viiii = [b12,b12,jsCall_viiii_0,b12,jsCall_viiii_1,b12,jsCall_viiii_2,b12,jsCall_viiii_3,b12,jsCall_viiii_4,b12,jsCall_viiii_5,b12,jsCall_viiii_6,b12,jsCall_viiii_7,b12,jsCall_viiii_8,b12,jsCall_viiii_9,b12,jsCall_viiii_10,b12,jsCall_viiii_11,b12,jsCall_viiii_12,b12,jsCall_viiii_13
,b12,jsCall_viiii_14,b12,jsCall_viiii_15,b12,jsCall_viiii_16,b12,jsCall_viiii_17,b12,jsCall_viiii_18,b12,jsCall_viiii_19,b12,jsCall_viiii_20,b12,jsCall_viiii_21,b12,jsCall_viiii_22,b12,jsCall_viiii_23,b12,jsCall_viiii_24,b12,jsCall_viiii_25,b12,jsCall_viiii_26,b12,jsCall_viiii_27,b12,jsCall_viiii_28
,b12,jsCall_viiii_29,b12,jsCall_viiii_30,b12,jsCall_viiii_31,b12,jsCall_viiii_32,b12,jsCall_viiii_33,b12,jsCall_viiii_34,b12,jsCall_viiii_35,b12,jsCall_viiii_36,b12,jsCall_viiii_37,b12,jsCall_viiii_38,b12,jsCall_viiii_39,b12,jsCall_viiii_40,b12,jsCall_viiii_41,b12,jsCall_viiii_42,b12,jsCall_viiii_43
,b12,jsCall_viiii_44,b12,jsCall_viiii_45,b12,jsCall_viiii_46,b12,jsCall_viiii_47,b12,jsCall_viiii_48,b12,jsCall_viiii_49,b12,jsCall_viiii_50,b12,jsCall_viiii_51,b12,jsCall_viiii_52,b12,jsCall_viiii_53,b12,jsCall_viiii_54,b12,jsCall_viiii_55,b12,jsCall_viiii_56,b12,jsCall_viiii_57,b12,jsCall_viiii_58
,b12,jsCall_viiii_59,b12,jsCall_viiii_60,b12,jsCall_viiii_61,b12,jsCall_viiii_62,b12,jsCall_viiii_63,b12,_pcache1Rekey,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12];

  return { _sqlite3_value_blob: _sqlite3_value_blob, _sqlite3_column_name: _sqlite3_column_name, _sqlite3_reset: _sqlite3_reset, _sqlite3_column_type: _sqlite3_column_type, _sqlite3_exec: _sqlite3_exec, _sqlite3_result_null: _sqlite3_result_null, _sqlite3_step: _sqlite3_step, _bitshift64Lshr: _bitshift64Lshr, _sqlite3_prepare_v2: _sqlite3_prepare_v2, _sqlite3_close_v2: _sqlite3_close_v2, _sqlite3_open: _sqlite3_open, _bitshift64Shl: _bitshift64Shl, _sqlite3_result_text: _sqlite3_result_text, _fflush: _fflush, _sqlite3_column_bytes: _sqlite3_column_bytes, _sqlite3_bind_int: _sqlite3_bind_int, _sqlite3_bind_blob: _sqlite3_bind_blob, _memset: _memset, _sqlite3_value_double: _sqlite3_value_double, _memcpy: _memcpy, _sqlite3_result_double: _sqlite3_result_double, _sqlite3_value_text: _sqlite3_value_text, _sqlite3_create_function_v2: _sqlite3_create_function_v2, _sqlite3_column_blob: _sqlite3_column_blob, _sqlite3_bind_parameter_index: _sqlite3_bind_parameter_index, _sqlite3_value_type: _sqlite3_value_type, _i64Subtract: _i64Subtract, _sqlite3_column_text: _sqlite3_column_text, _i64Add: _i64Add, _sqlite3_value_bytes: _sqlite3_value_bytes, _sqlite3_finalize: _sqlite3_finalize, _sqlite3_column_double: _sqlite3_column_double, _sqlite3_free: _sqlite3_free, _sqlite3_value_int: _sqlite3_value_int, _sqlite3_data_count: _sqlite3_data_count, _sqlite3_bind_text: _sqlite3_bind_text, _sqlite3_bind_double: _sqlite3_bind_double, ___errno_location: ___errno_location, _free: _free, _memmove: _memmove, _sqlite3_errmsg: _sqlite3_errmsg, _sqlite3_clear_bindings: _sqlite3_clear_bindings, _malloc: _malloc, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, establishStackSpace: establishStackSpace, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii, dynCall_i: dynCall_i, dynCall_vi: dynCall_vi, dynCall_vii: dynCall_vii, dynCall_iiiiiii: dynCall_iiiiiii, dynCall_ii: dynCall_ii, dynCall_viii: dynCall_viii, dynCall_v: dynCall_v, dynCall_iiiii: dynCall_iiiii, dynCall_viiiiii: dynCall_viiiiii, dynCall_iii: dynCall_iii, dynCall_iiiiii: dynCall_iiiiii, dynCall_viiii: dynCall_viiii };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var _sqlite3_value_blob = Module["_sqlite3_value_blob"] = asm["_sqlite3_value_blob"];
var _sqlite3_column_name = Module["_sqlite3_column_name"] = asm["_sqlite3_column_name"];
var _sqlite3_reset = Module["_sqlite3_reset"] = asm["_sqlite3_reset"];
var _sqlite3_column_type = Module["_sqlite3_column_type"] = asm["_sqlite3_column_type"];
var _sqlite3_exec = Module["_sqlite3_exec"] = asm["_sqlite3_exec"];
var _sqlite3_result_null = Module["_sqlite3_result_null"] = asm["_sqlite3_result_null"];
var _sqlite3_step = Module["_sqlite3_step"] = asm["_sqlite3_step"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _sqlite3_prepare_v2 = Module["_sqlite3_prepare_v2"] = asm["_sqlite3_prepare_v2"];
var _sqlite3_close_v2 = Module["_sqlite3_close_v2"] = asm["_sqlite3_close_v2"];
var _sqlite3_open = Module["_sqlite3_open"] = asm["_sqlite3_open"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var _sqlite3_result_text = Module["_sqlite3_result_text"] = asm["_sqlite3_result_text"];
var _fflush = Module["_fflush"] = asm["_fflush"];
var _sqlite3_column_bytes = Module["_sqlite3_column_bytes"] = asm["_sqlite3_column_bytes"];
var _sqlite3_bind_int = Module["_sqlite3_bind_int"] = asm["_sqlite3_bind_int"];
var _sqlite3_bind_blob = Module["_sqlite3_bind_blob"] = asm["_sqlite3_bind_blob"];
var _memset = Module["_memset"] = asm["_memset"];
var _sqlite3_value_double = Module["_sqlite3_value_double"] = asm["_sqlite3_value_double"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _sqlite3_result_double = Module["_sqlite3_result_double"] = asm["_sqlite3_result_double"];
var _sqlite3_value_text = Module["_sqlite3_value_text"] = asm["_sqlite3_value_text"];
var _sqlite3_create_function_v2 = Module["_sqlite3_create_function_v2"] = asm["_sqlite3_create_function_v2"];
var _sqlite3_column_blob = Module["_sqlite3_column_blob"] = asm["_sqlite3_column_blob"];
var _sqlite3_bind_parameter_index = Module["_sqlite3_bind_parameter_index"] = asm["_sqlite3_bind_parameter_index"];
var _sqlite3_value_type = Module["_sqlite3_value_type"] = asm["_sqlite3_value_type"];
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _sqlite3_column_text = Module["_sqlite3_column_text"] = asm["_sqlite3_column_text"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _sqlite3_value_bytes = Module["_sqlite3_value_bytes"] = asm["_sqlite3_value_bytes"];
var _sqlite3_finalize = Module["_sqlite3_finalize"] = asm["_sqlite3_finalize"];
var _sqlite3_column_double = Module["_sqlite3_column_double"] = asm["_sqlite3_column_double"];
var _sqlite3_free = Module["_sqlite3_free"] = asm["_sqlite3_free"];
var _sqlite3_value_int = Module["_sqlite3_value_int"] = asm["_sqlite3_value_int"];
var _sqlite3_data_count = Module["_sqlite3_data_count"] = asm["_sqlite3_data_count"];
var _sqlite3_bind_text = Module["_sqlite3_bind_text"] = asm["_sqlite3_bind_text"];
var _sqlite3_bind_double = Module["_sqlite3_bind_double"] = asm["_sqlite3_bind_double"];
var ___errno_location = Module["___errno_location"] = asm["___errno_location"];
var _free = Module["_free"] = asm["_free"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var _memmove = Module["_memmove"] = asm["_memmove"];
var _sqlite3_errmsg = Module["_sqlite3_errmsg"] = asm["_sqlite3_errmsg"];
var _sqlite3_clear_bindings = Module["_sqlite3_clear_bindings"] = asm["_sqlite3_clear_bindings"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_i = Module["dynCall_i"] = asm["dynCall_i"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_vii = Module["dynCall_vii"] = asm["dynCall_vii"];
var dynCall_iiiiiii = Module["dynCall_iiiiiii"] = asm["dynCall_iiiiiii"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_viii = Module["dynCall_viii"] = asm["dynCall_viii"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_viiiiii = Module["dynCall_viiiiii"] = asm["dynCall_viiiiii"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];
var dynCall_iiiiii = Module["dynCall_iiiiii"] = asm["dynCall_iiiiii"];
var dynCall_viiii = Module["dynCall_viiii"] = asm["dynCall_viiii"];
;

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.establishStackSpace = asm['establishStackSpace'];

Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];



// === Auto-generated postamble setup entry stuff ===


function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun']) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);


  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    exit(ret, /* implicit = */ true);
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    if (ABORT) return; 

    ensureInitRuntime();

    preMain();


    if (Module['onRuntimeInitialized']) Module['onRuntimeInitialized']();

    if (Module['_main'] && shouldRunNow) Module['callMain'](args);

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status, implicit) {
  if (implicit && Module['noExitRuntime']) {
    return;
  }

  if (Module['noExitRuntime']) {
  } else {

    ABORT = true;
    EXITSTATUS = status;
    STACKTOP = initialStackTop;

    exitRuntime();

    if (Module['onExit']) Module['onExit'](status);
  }

  if (ENVIRONMENT_IS_NODE) {
    // Work around a node.js bug where stdout buffer is not flushed at process exit:
    // Instead of process.exit() directly, wait for stdout flush event.
    // See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
    // Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
    process['stdout']['once']('drain', function () {
      process['exit'](status);
    });
    console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
    // Work around another node bug where sometimes 'drain' is never fired - make another effort
    // to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
    setTimeout(function() {
      process['exit'](status);
    }, 500);
  } else
  if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
    quit(status);
  }
  // if we reach here, we must throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

var abortDecorators = [];

function abort(what) {
  if (what !== undefined) {
    Module.print(what);
    Module.printErr(what);
    what = JSON.stringify(what)
  } else {
    what = '';
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '\nIf this abort() is unexpected, build with -s ASSERTIONS=1 which can give more information.';

  var output = 'abort(' + what + ') at ' + stackTrace() + extra;
  if (abortDecorators) {
    abortDecorators.forEach(function(decorator) {
      output = decorator(output, what);
    });
  }
  throw output;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



// Generated by CoffeeScript 1.9.2
var Database, NULL, Runtime, SQLite, Statement, apiTemp, i, sqlite3_bind_blob, sqlite3_bind_double, sqlite3_bind_int, sqlite3_bind_parameter_index, sqlite3_bind_text, sqlite3_clear_bindings, sqlite3_close_v2, sqlite3_column_blob, sqlite3_column_bytes, sqlite3_column_double, sqlite3_column_name, sqlite3_column_text, sqlite3_column_type, sqlite3_create_function_v2, sqlite3_data_count, sqlite3_errmsg, sqlite3_exec, sqlite3_finalize, sqlite3_free, sqlite3_open, sqlite3_prepare_v2, sqlite3_prepare_v2_sqlptr, sqlite3_reset, sqlite3_result_double, sqlite3_result_null, sqlite3_result_text, sqlite3_step, sqlite3_value_blob, sqlite3_value_bytes, sqlite3_value_double, sqlite3_value_int, sqlite3_value_text, sqlite3_value_type;

Runtime = Module['Runtime'];

apiTemp = Runtime.stackAlloc(4);

SQLite = {};


/* Represents an prepared statement.

Prepared statements allow you to have a template sql string,
that you can execute multiple times with different parameters.

You can't instantiate this class directly, you have to use a [Database](Database.html)
object in order to create a statement.

**Warning**: When you close a database (using db.close()), all its statements are
closed too and become unusable.

@see Database.html#prepare-dynamic
@see https://en.wikipedia.org/wiki/Prepared_statement
 */

Statement = (function() {
  function Statement(stmt1, db) {
    this.stmt = stmt1;
    this.db = db;
    this.pos = 1;
    this.allocatedmem = [];
  }


  /* Bind values to the parameters, after having reseted the statement
  
  SQL statements can have parameters, named *'?', '?NNN', ':VVV', '@VVV', '$VVV'*,
  where NNN is a number and VVV a string.
  This function binds these parameters to the given values.
  
  *Warning*: ':', '@', and '$' are included in the parameters names
  
  ## Binding values to named parameters
  @example Bind values to named parameters
      var stmt = db.prepare("UPDATE test SET a=@newval WHERE id BETWEEN $mini AND $maxi");
      stmt.bind({$mini:10, $maxi:20, '@newval':5});
  - Create a statement that contains parameters like '$VVV', ':VVV', '@VVV'
  - Call Statement.bind with an object as parameter
  
  ## Binding values to parameters
  @example Bind values to anonymous parameters
      var stmt = db.prepare("UPDATE test SET a=? WHERE id BETWEEN ? AND ?");
      stmt.bind([5, 10, 20]);
   - Create a statement that contains parameters like '?', '?NNN'
   - Call Statement.bind with an array as parameter
  
  ## Value types
  Javascript type | SQLite type
  --- | ---
  number | REAL, INTEGER
  boolean | INTEGER
  string | TEXT
  Array, Uint8Array | BLOB
  null | NULL
  @see http://www.sqlite.org/datatype3.html
  
  @see http://www.sqlite.org/lang_expr.html#varparam
  @param values [Array,Object] The values to bind
  @return [Boolean] true if it worked
  @throw [String] SQLite Error
   */

  Statement.prototype['bind'] = function(values) {
    if (!this.stmt) {
      throw "Statement closed";
    }
    this['reset']();
    if (Array.isArray(values)) {
      return this.bindFromArray(values);
    } else {
      return this.bindFromObject(values);
    }
  };


  /* Execute the statement, fetching the the next line of result,
  that can be retrieved with [Statement.get()](#get-dynamic) .
  
  @return [Boolean] true if a row of result available
  @throw [String] SQLite Error
   */

  Statement.prototype['step'] = function() {
    var ret;
    if (!this.stmt) {
      throw "Statement closed";
    }
    this.pos = 1;
    switch (ret = sqlite3_step(this.stmt)) {
      case SQLite.ROW:
        return true;
      case SQLite.DONE:
        return false;
      default:
        return this.db.handleError(ret);
    }
  };

  Statement.prototype.getNumber = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_column_double(this.stmt, pos);
  };

  Statement.prototype.getString = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_column_text(this.stmt, pos);
  };

  Statement.prototype.getBlob = function(pos) {
    var i, k, ptr, ref, result, size;
    if (pos == null) {
      pos = this.pos++;
    }
    size = sqlite3_column_bytes(this.stmt, pos);
    ptr = sqlite3_column_blob(this.stmt, pos);
    result = new Uint8Array(size);
    for (i = k = 0, ref = size; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
      result[i] = HEAP8[ptr + i];
    }
    return result;
  };


  /* Get one row of results of a statement.
  If the first parameter is not provided, step must have been called before get.
  @param [Array,Object] Optional: If set, the values will be bound to the statement, and it will be executed
  @return [Array<String,Number,Uint8Array,null>] One row of result
  
  @example Print all the rows of the table test to the console
  
      var stmt = db.prepare("SELECT * FROM test");
      while (stmt.step()) console.log(stmt.get());
   */

  Statement.prototype['get'] = function(params) {
    var field, k, ref, results1;
    if (params != null) {
      this['bind'](params) && this['step']();
    }
    results1 = [];
    for (field = k = 0, ref = sqlite3_data_count(this.stmt); 0 <= ref ? k < ref : k > ref; field = 0 <= ref ? ++k : --k) {
      switch (sqlite3_column_type(this.stmt, field)) {
        case SQLite.INTEGER:
        case SQLite.FLOAT:
          results1.push(this.getNumber(field));
          break;
        case SQLite.TEXT:
          results1.push(this.getString(field));
          break;
        case SQLite.BLOB:
          results1.push(this.getBlob(field));
          break;
        default:
          results1.push(null);
      }
    }
    return results1;
  };


  /* Get the list of column names of a row of result of a statement.
  @return [Array<String>] The names of the columns
  @example
  
      var stmt = db.prepare("SELECT 5 AS nbr, x'616200' AS data, NULL AS nothing;");
      stmt.step(); // Execute the statement
      console.log(stmt.getColumnNames()); // Will print ['nbr','data','nothing']
   */

  Statement.prototype['getColumnNames'] = function() {
    var i, k, ref, results1;
    results1 = [];
    for (i = k = 0, ref = sqlite3_data_count(this.stmt); 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
      results1.push(sqlite3_column_name(this.stmt, i));
    }
    return results1;
  };


  /* Get one row of result as a javascript object, associating column names with
  their value in the current row.
  @param [Array,Object] Optional: If set, the values will be bound to the statement, and it will be executed
  @return [Object] The row of result
  @see [Statement.get](#get-dynamic)
  
  @example
  
      var stmt = db.prepare("SELECT 5 AS nbr, x'616200' AS data, NULL AS nothing;");
      stmt.step(); // Execute the statement
      console.log(stmt.getAsObject()); // Will print {nbr:5, data: Uint8Array([1,2,3]), nothing:null}
   */

  Statement.prototype['getAsObject'] = function(params) {
    var i, k, len, name, names, rowObject, values;
    values = this['get'](params);
    names = this['getColumnNames']();
    rowObject = {};
    for (i = k = 0, len = names.length; k < len; i = ++k) {
      name = names[i];
      rowObject[name] = values[i];
    }
    return rowObject;
  };


  /* Shorthand for bind + step + reset
  Bind the values, execute the statement, ignoring the rows it returns, and resets it
  @param [Array,Object] Value to bind to the statement
   */

  Statement.prototype['run'] = function(values) {
    if (values != null) {
      this['bind'](values);
    }
    this['step']();
    return this['reset']();
  };

  Statement.prototype.bindString = function(string, pos) {
    var bytes, strptr;
    if (pos == null) {
      pos = this.pos++;
    }
    bytes = intArrayFromString(string);
    this.allocatedmem.push(strptr = allocate(bytes, 'i8', ALLOC_NORMAL));
    this.db.handleError(sqlite3_bind_text(this.stmt, pos, strptr, bytes.length - 1, 0));
    return true;
  };

  Statement.prototype.bindBlob = function(array, pos) {
    var blobptr;
    if (pos == null) {
      pos = this.pos++;
    }
    this.allocatedmem.push(blobptr = allocate(array, 'i8', ALLOC_NORMAL));
    this.db.handleError(sqlite3_bind_blob(this.stmt, pos, blobptr, array.length, 0));
    return true;
  };

  Statement.prototype.bindNumber = function(num, pos) {
    var bindfunc;
    if (pos == null) {
      pos = this.pos++;
    }
    bindfunc = num === (num | 0) ? sqlite3_bind_int : sqlite3_bind_double;
    this.db.handleError(bindfunc(this.stmt, pos, num));
    return true;
  };

  Statement.prototype.bindNull = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_bind_blob(this.stmt, pos, 0, 0, 0) === SQLite.OK;
  };

  Statement.prototype.bindValue = function(val, pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    switch (typeof val) {
      case "string":
        return this.bindString(val, pos);
      case "number":
      case "boolean":
        return this.bindNumber(val + 0, pos);
      case "object":
        if (val === null) {
          return this.bindNull(pos);
        } else if (val.length != null) {
          return this.bindBlob(val, pos);
        } else {
          throw "Wrong API use : tried to bind a value of an unknown type (" + val + ").";
        }
    }
  };


  /* Bind names and values of an object to the named parameters of the statement
  @param [Object]
  @private
  @nodoc
   */

  Statement.prototype.bindFromObject = function(valuesObj) {
    var name, num, value;
    for (name in valuesObj) {
      value = valuesObj[name];
      num = sqlite3_bind_parameter_index(this.stmt, name);
      if (num !== 0) {
        this.bindValue(value, num);
      }
    }
    return true;
  };


  /* Bind values to numbered parameters
  @param [Array]
  @private
  @nodoc
   */

  Statement.prototype.bindFromArray = function(values) {
    var k, len, num, value;
    for (num = k = 0, len = values.length; k < len; num = ++k) {
      value = values[num];
      this.bindValue(value, num + 1);
    }
    return true;
  };


  /* Reset a statement, so that it's parameters can be bound to new values
  It also clears all previous bindings, freeing the memory used by bound parameters.
   */

  Statement.prototype['reset'] = function() {
    this.freemem();
    return sqlite3_clear_bindings(this.stmt) === SQLite.OK && sqlite3_reset(this.stmt) === SQLite.OK;
  };


  /* Free the memory allocated during parameter binding
   */

  Statement.prototype.freemem = function() {
    var mem;
    while (mem = this.allocatedmem.pop()) {
      _free(mem);
    }
    return null;
  };


  /* Free the memory used by the statement
  @return [Boolean] true in case of success
   */

  Statement.prototype['free'] = function() {
    var res;
    this.freemem();
    res = sqlite3_finalize(this.stmt) === SQLite.OK;
    delete this.db.statements[this.stmt];
    this.stmt = NULL;
    return res;
  };

  return Statement;

})();

Database = (function() {
  function Database(data) {
    this.filename = 'dbfile_' + (0xffffffff * Math.random() >>> 0);
    if (data != null) {
      FS.createDataFile('/', this.filename, data, true, true);
    }
    this.handleError(sqlite3_open(this.filename, apiTemp));
    this.db = getValue(apiTemp, 'i32');
    this.statements = {};
  }


  /* Execute an SQL query, ignoring the rows it returns.
  
  @param sql [String] a string containing some SQL text to execute
  @param params [Array] (*optional*) When the SQL statement contains placeholders, you can pass them in here. They will be bound to the statement before it is executed.
  
  If you use the params argument, you **cannot** provide an sql string that contains several
  queries (separated by ';')
  
  @example Insert values in a table
      db.run("INSERT INTO test VALUES (:age, :name)", {':age':18, ':name':'John'});
  
  @return [Database] The database object (usefull for method chaining)
   */

  Database.prototype['run'] = function(sql, params) {
    var stmt;
    if (!this.db) {
      throw "Database closed";
    }
    if (params) {
      stmt = this['prepare'](sql, params);
      stmt['step']();
      stmt['free']();
    } else {
      this.handleError(sqlite3_exec(this.db, sql, 0, 0, apiTemp));
    }
    return this;
  };


  /* Execute an SQL query, and returns the result.
  
  This is a wrapper against Database.prepare, Statement.step, Statement.get,
  and Statement.free.
  
  The result is an array of result elements. There are as many result elements
  as the number of statements in your sql string (statements are separated by a semicolon)
  
  Each result element is an object with two properties:
      'columns' : the name of the columns of the result (as returned by Statement.getColumnNames())
      'values' : an array of rows. Each row is itself an array of values
  
  ## Example use
  We have the following table, named *test* :
  
  | id | age |  name  |
  |:--:|:---:|:------:|
  | 1  |  1  | Ling   |
  | 2  |  18 | Paul   |
  | 3  |  3  | Markus |
  
  
  We query it like that:
  ```javascript
  var db = new SQL.Database();
  var res = db.exec("SELECT id FROM test; SELECT age,name FROM test;");
  ```
  
  `res` is now :
  ```javascript
      [
          {columns: ['id'], values:[[1],[2],[3]]},
          {columns: ['age','name'], values:[[1,'Ling'],[18,'Paul'],[3,'Markus']]}
      ]
  ```
  
  @param sql [String] a string containing some SQL text to execute
  @return [Array<QueryResults>] An array of results.
   */

  Database.prototype['exec'] = function(sql) {
    var curresult, nextSqlPtr, pStmt, pzTail, results, stack, stmt;
    if (!this.db) {
      throw "Database closed";
    }
    stack = Runtime.stackSave();
    nextSqlPtr = Runtime.stackAlloc(sql.length << 2 + 1);
    writeStringToMemory(sql, nextSqlPtr);
    pzTail = Runtime.stackAlloc(4);
    results = [];
    while (getValue(nextSqlPtr, 'i8') !== NULL) {
      setValue(apiTemp, 0, 'i32');
      setValue(pzTail, 0, 'i32');
      this.handleError(sqlite3_prepare_v2_sqlptr(this.db, nextSqlPtr, -1, apiTemp, pzTail));
      pStmt = getValue(apiTemp, 'i32');
      nextSqlPtr = getValue(pzTail, 'i32');
      if (pStmt === NULL) {
        continue;
      }
      stmt = new Statement(pStmt, this);
      curresult = null;
      while (stmt['step']()) {
        if (curresult === null) {
          curresult = {
            'columns': stmt['getColumnNames'](),
            'values': []
          };
          results.push(curresult);
        }
        curresult['values'].push(stmt['get']());
      }
      stmt['free']();
    }
    Runtime.stackRestore(stack);
    return results;
  };


  /* Execute an sql statement, and call a callback for each row of result.
  
  **Currently** this method is synchronous, it will not return until the callback has
  been called on every row of the result. But this might change.
  
  @param sql [String] A string of SQL text. Can contain placeholders that will be
  bound to the parameters given as the second argument
  @param params [Array<String,Number,null,Uint8Array>] (*optional*) Parameters to bind
  to the query
  @param callback [Function(Object)] A function that will be called on each row of result
  @param done [Function] A function that will be called when all rows have been retrieved
  
  @return [Database] The database object. Usefull for method chaining
  
  @example Read values from a table
      db.each("SELECT name,age FROM users WHERE age >= $majority",
                      {$majority:18},
                      function(row){console.log(row.name + " is a grown-up.")}
                  );
   */

  Database.prototype['each'] = function(sql, params, callback, done) {
    var stmt;
    if (typeof params === 'function') {
      done = callback;
      callback = params;
      params = void 0;
    }
    stmt = this['prepare'](sql, params);
    while (stmt['step']()) {
      callback(stmt['getAsObject']());
    }
    stmt['free']();
    if (typeof done === 'function') {
      return done();
    }
  };


  /* Prepare an SQL statement
  @param sql [String] a string of SQL, that can contain placeholders ('?', ':VVV', ':AAA', '@AAA')
  @param params [Array] (*optional*) values to bind to placeholders
  @return [Statement] the resulting statement
  @throw [String] SQLite error
   */

  Database.prototype['prepare'] = function(sql, params) {
    var pStmt, stmt;
    setValue(apiTemp, 0, 'i32');
    this.handleError(sqlite3_prepare_v2(this.db, sql, -1, apiTemp, NULL));
    pStmt = getValue(apiTemp, 'i32');
    if (pStmt === NULL) {
      throw 'Nothing to prepare';
    }
    stmt = new Statement(pStmt, this);
    if (params != null) {
      stmt.bind(params);
    }
    this.statements[pStmt] = stmt;
    return stmt;
  };


  /* Exports the contents of the database to a binary array
  @return [Uint8Array] An array of bytes of the SQLite3 database file
   */

  Database.prototype['export'] = function() {
    var _, binaryDb, ref, stmt;
    ref = this.statements;
    for (_ in ref) {
      stmt = ref[_];
      stmt['free']();
    }
    this.handleError(sqlite3_close_v2(this.db));
    binaryDb = FS.readFile(this.filename, {
      encoding: 'binary'
    });
    this.handleError(sqlite3_open(this.filename, apiTemp));
    this.db = getValue(apiTemp, 'i32');
    return binaryDb;
  };


  /* Close the database, and all associated prepared statements.
  
  The memory associated to the database and all associated statements
  will be freed.
  
  **Warning**: A statement belonging to a database that has been closed cannot
  be used anymore.
  
  Databases **must** be closed, when you're finished with them, or the
  memory consumption will grow forever
   */

  Database.prototype['close'] = function() {
    var _, ref, stmt;
    ref = this.statements;
    for (_ in ref) {
      stmt = ref[_];
      stmt['free']();
    }
    this.handleError(sqlite3_close_v2(this.db));
    FS.unlink('/' + this.filename);
    return this.db = null;
  };


  /* Analyze a result code, return null if no error occured, and throw
  an error with a descriptive message otherwise
  @nodoc
   */

  Database.prototype.handleError = function(returnCode) {
    var errmsg;
    if (returnCode === SQLite.OK) {
      return null;
    } else {
      errmsg = sqlite3_errmsg(this.db);
      throw new Error(errmsg);
    }
  };


  /* Register a custom function with SQLite
  @example Register a simple function
      db.create_function("addOne", function(x) {return x+1;})
      db.exec("SELECT addOne(1)") # 2
  
  @param name [String] the name of the function as referenced in SQL statements.
  @param func [Function] the actual function to be executed.
   */

  Database.prototype['create_function'] = function(name, func) {
    var func_ptr, wrapped_func;
    wrapped_func = function(cx, argc, argv) {
      var arg, args, data_func, i, k, ref, result, value_ptr, value_type;
      args = [];
      for (i = k = 0, ref = argc; 0 <= ref ? k <= ref : k >= ref; i = 0 <= ref ? ++k : --k) {
        value_ptr = getValue(argv + (4 * i), 'i32');
        value_type = sqlite3_value_type(value_ptr);
        data_func = (function() {
          switch (false) {
            case value_type !== 1:
              return sqlite3_value_int;
            case value_type !== 2:
              return sqlite3_value_double;
            case value_type !== 3:
              return sqlite3_value_text;
            case value_type !== 4:
              return function(ptr) {
                var blob_arg, blob_ptr, j, l, ref1, size;
                size = sqlite3_value_bytes(ptr);
                blob_ptr = sqlite3_value_blob(ptr);
                blob_arg = new Uint8Array(size);
                for (j = l = 0, ref1 = size; 0 <= ref1 ? l < ref1 : l > ref1; j = 0 <= ref1 ? ++l : --l) {
                  blob_arg[j] = HEAP8[blob_ptr + j];
                }
                return blob_arg;
              };
            default:
              return function(ptr) {
                return null;
              };
          }
        })();
        arg = data_func(value_ptr);
        args.push(arg);
      }
      result = func.apply(null, args);
      if (!result) {
        return sqlite3_result_null(cx);
      } else {
        switch (typeof result) {
          case 'number':
            return sqlite3_result_double(cx, result);
          case 'string':
            return sqlite3_result_text(cx, result, -1, -1);
        }
      }
    };
    func_ptr = Runtime.addFunction(wrapped_func);
    this.handleError(sqlite3_create_function_v2(this.db, name, func.length, SQLite.UTF8, 0, func_ptr, 0, 0, 0));
    return this;
  };

  return Database;

})();

sqlite3_open = Module['cwrap']('sqlite3_open', 'number', ['string', 'number']);

sqlite3_close_v2 = Module['cwrap']('sqlite3_close_v2', 'number', ['number']);

sqlite3_exec = Module['cwrap']('sqlite3_exec', 'number', ['number', 'string', 'number', 'number', 'number']);

sqlite3_free = Module['cwrap']('sqlite3_free', '', ['number']);

sqlite3_prepare_v2 = Module['cwrap']('sqlite3_prepare_v2', 'number', ['number', 'string', 'number', 'number', 'number']);

sqlite3_prepare_v2_sqlptr = Module['cwrap']('sqlite3_prepare_v2', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_text = Module['cwrap']('sqlite3_bind_text', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_blob = Module['cwrap']('sqlite3_bind_blob', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_double = Module['cwrap']('sqlite3_bind_double', 'number', ['number', 'number', 'number']);

sqlite3_bind_int = Module['cwrap']('sqlite3_bind_int', 'number', ['number', 'number', 'number']);

sqlite3_bind_parameter_index = Module['cwrap']('sqlite3_bind_parameter_index', 'number', ['number', 'string']);

sqlite3_step = Module['cwrap']('sqlite3_step', 'number', ['number']);

sqlite3_errmsg = Module['cwrap']('sqlite3_errmsg', 'string', ['number']);

sqlite3_data_count = Module['cwrap']('sqlite3_data_count', 'number', ['number']);

sqlite3_column_double = Module['cwrap']('sqlite3_column_double', 'number', ['number', 'number']);

sqlite3_column_text = Module['cwrap']('sqlite3_column_text', 'string', ['number', 'number']);

sqlite3_column_blob = Module['cwrap']('sqlite3_column_blob', 'number', ['number', 'number']);

sqlite3_column_bytes = Module['cwrap']('sqlite3_column_bytes', 'number', ['number', 'number']);

sqlite3_column_type = Module['cwrap']('sqlite3_column_type', 'number', ['number', 'number']);

sqlite3_column_name = Module['cwrap']('sqlite3_column_name', 'string', ['number', 'number']);

sqlite3_reset = Module['cwrap']('sqlite3_reset', 'number', ['number']);

sqlite3_clear_bindings = Module['cwrap']('sqlite3_clear_bindings', 'number', ['number']);

sqlite3_finalize = Module['cwrap']('sqlite3_finalize', 'number', ['number']);

sqlite3_create_function_v2 = Module['cwrap']('sqlite3_create_function_v2', 'number', ['number', 'string', 'number', 'number', 'number', 'number', 'number', 'number', 'number']);

sqlite3_value_type = Module['cwrap']('sqlite3_value_type', 'number', ['number']);

sqlite3_value_bytes = Module['cwrap']('sqlite3_value_bytes', 'number', ['number']);

sqlite3_value_text = Module['cwrap']('sqlite3_value_text', 'string', ['number']);

sqlite3_value_int = Module['cwrap']('sqlite3_value_int', 'number', ['number']);

sqlite3_value_blob = Module['cwrap']('sqlite3_value_blob', 'number', ['number']);

sqlite3_value_double = Module['cwrap']('sqlite3_value_double', 'number', ['number']);

sqlite3_result_double = Module['cwrap']('sqlite3_result_double', '', ['number', 'number']);

sqlite3_result_null = Module['cwrap']('sqlite3_result_null', '', ['number']);

sqlite3_result_text = Module['cwrap']('sqlite3_result_text', '', ['number', 'string', 'number', 'number']);

this['SQL'] = {
  'Database': Database
};

for (i in this['SQL']) {
  Module[i] = this['SQL'][i];
}

NULL = 0;

SQLite.OK = 0;

SQLite.ERROR = 1;

SQLite.INTERNAL = 2;

SQLite.PERM = 3;

SQLite.ABORT = 4;

SQLite.BUSY = 5;

SQLite.LOCKED = 6;

SQLite.NOMEM = 7;

SQLite.READONLY = 8;

SQLite.INTERRUPT = 9;

SQLite.IOERR = 10;

SQLite.CORRUPT = 11;

SQLite.NOTFOUND = 12;

SQLite.FULL = 13;

SQLite.CANTOPEN = 14;

SQLite.PROTOCOL = 15;

SQLite.EMPTY = 16;

SQLite.SCHEMA = 17;

SQLite.TOOBIG = 18;

SQLite.CONSTRAINT = 19;

SQLite.MISMATCH = 20;

SQLite.MISUSE = 21;

SQLite.NOLFS = 22;

SQLite.AUTH = 23;

SQLite.FORMAT = 24;

SQLite.RANGE = 25;

SQLite.NOTADB = 26;

SQLite.NOTICE = 27;

SQLite.WARNING = 28;

SQLite.ROW = 100;

SQLite.DONE = 101;

SQLite.INTEGER = 1;

SQLite.FLOAT = 2;

SQLite.TEXT = 3;

SQLite.BLOB = 4;

SQLite.NULL = 5;

SQLite.UTF8 = 1;

return this['SQL'];
})();
if (typeof module !== 'undefined') module.exports = SQL;
if (typeof define === 'function') define(SQL);
